/*!
*  filename: ej.web.all.js
*  version : 12.3
*  Copyright Syncfusion Inc. 2001 - 2014. All rights reserved.
*  Use of this code is subject to the terms of our license.
*  A copy of the current license can be obtained at any time by e-mailing
*  licensing@syncfusion.com. Any infringement will be prosecuted under
*  applicable laws. 
*/

window.ej = window.Syncfusion = window.Syncfusion || {};

(function ($, ej, undefined) {
    'use strict';

    ej.consts = {
        NamespaceJoin: '-'
    };
    ej.TextAlign = {
        Center: 'center',
        Justify: 'justify',
        Left: 'left',
        Right: 'right'
    };
    ej.Orientation = { Horizontal: "horizontal", Vertical: "vertical" };

    if (!Object.prototype.hasOwnProperty) {
        Object.prototype.hasOwnProperty = function (obj, prop) {
            return obj[prop] !== undefined;
        };
    }

    String.format = function () {
        var source = arguments[0];
        for (var i = 0; i < arguments.length - 1; i++)
            source = source.replace(new RegExp("\\{" + i + "\\}", "gm"), arguments[i + 1]);

        source = source.replace(/\{[0-9]\}/g, "");
        return source;
    };

    // Function to create new class
    ej.defineClass = function (className, constructor, proto, replace) {
        /// <summary>Creates the javascript class with given namespace & class name & constructor etc</summary>
        /// <param name="className" type="String">class name prefixed with namespace</param>
        /// <param name="constructor" type="Function">constructor function</param>
        /// <param name="proto" type="Object">prototype for the class</param>
        /// <param name="replace" type="Boolean">[Optional]Replace existing class if exists</param>
        /// <returns type="Function">returns the class function</returns>
        if (!className || !proto) return undefined;

        var parts = className.split(".");

        // Object creation
        var obj = window, i = 0;
        for (; i < parts.length - 1; i++) {

            if (ej.isNullOrUndefined(obj[parts[i]]))
                obj[parts[i]] = {};

            obj = obj[parts[i]];
        }

        if (replace || ej.isNullOrUndefined(obj[parts[i]])) {

            //constructor
            constructor = typeof constructor === "function" ? constructor : function () {
            };

            obj[parts[i]] = constructor;

            // prototype
            obj[parts[i]].prototype = proto;
        }

        return obj[parts[i]];
    };

    ej.util = {
        getNameSpace: function (className) {
            /// <summary>Internal function, this will create namespace for plugins using class name</summary>
            /// <param name="className" type="String"></param>
            /// <returns type="String"></returns>
            var splits = className.toLowerCase().split(".");
            splits[0] === "ej" && (splits[0] = "e");

            return splits.join(ej.consts.NamespaceJoin);
        },

        getObject: function (nameSpace, from) {
            if (!from) return undefined;

            var value = from, splits = nameSpace.split('.');

            for (var i = 0; i < splits.length; i++) {

                if (ej.util.isNullOrUndefined(value)) break;

                value = value[splits[i]];
            }

            return value;
        },

        createObject: function (nameSpace, value, initIn) {
            var splits = nameSpace.split('.'), start = initIn || window, from = start, i;

            for (i = 0; i < splits.length; i++) {

                if (i + 1 == splits.length)
                    from[splits[i]] = ej.isNullOrUndefined(value) ? {} : value;
                else if (ej.isNullOrUndefined(from[splits[i]]))
                    from[splits[i]] = {};

                from = from[splits[i]];
            }

            return start;
        },

        isNullOrUndefined: function (value) {
            /// <summary>Util to check null or undefined</summary>
            /// <param name="value" type="Object"></param>
            /// <returns type="Boolean"></returns>
            return value === undefined || value === null;
        },
		print:function(element){
		    var $div=ej.buildTag('div')
		    var elementClone=element.clone();
			$div.append(elementClone);
			var printWin = window.open('','pint',"'height=452,width=1024,tabbar=no");
			printWin.document.write('<!DOCTYPE html>');
			var links = $('head').find('link');
			if (ej.browserInfo().name === "msie") {
			    var a = ""
			    links.each(function (index, obj) {
			        $(obj).attr('href', obj.href);
			        a += obj.outerHTML;
			    });
			    printWin.document.write('<html><head></head><body>' + a + $div[0].innerHTML + '</body></html>');
			}
			else {
			    var a = ""
			    printWin.document.write('<html><head>')
			    links.each(function (index, obj) {
			        $(obj).attr('href', obj.href);
			        a += obj.outerHTML;
			    });
			    printWin.document.writeln(a + '</head><body>')
			    printWin.document.writeln($div[0].innerHTML + '</body></html>')
			}
			printWin.focus();
			setTimeout(function () {
			    if (!ej.isNullOrUndefined(printWin.window)) {
			        printWin.print();
			        printWin.location.reload();
			       setTimeout(function(){printWin.close()},1000);
			    }
			}, 1000);
		},
        buildTag: function (tag, innerHtml, styles, attrs) {
            /// <summary>Helper to build jQuery element</summary>
            /// <param name="tag" type="String">tagName#id.cssClass</param>
            /// <param name="innerHtml" type="String"></param>
            /// <param name="styles" type="Object">A set of key/value pairs that configure styles</param>
            /// <param name="attrs" type="Object">A set of key/value pairs that configure attributes</param>
            /// <returns type="jQuery"></returns>
            var tagName = /^[a-z]*[0-9a-z]+/ig.exec(tag)[0];

            var id = /#([a-z]+[-_0-9a-z]+)/ig.exec(tag);
            id = id ? id[id.length - 1] : undefined;

            var className = /\.([a-z]+[-_0-9a-z ]+)/ig.exec(tag);
            className = className ? className[className.length - 1] : undefined;

            return $(document.createElement(tagName))
                .attr(id ? { "id": id } : {})
                .addClass(className || "")
                .css(styles || {})
                .attr(attrs || {})
                .html(innerHtml || "");
        },
        _preventDefaultException: function (el, exceptions) {
        if (el) {
            for (var i in exceptions) {
                if (exceptions[i].test(el[i])) {
                    return true;
                }
            }
        }

        return false;
        },

        //Gets the maximum z-index in the document
        getMaxZindex: function () {
            var maxZ = 1;
            maxZ = Math.max.apply(null, $.map($('body *'), function (e, n) {
                if ($(e).css('position') == 'absolute' || $(e).css('position') == 'fixed')
                    return parseInt($(e).css('z-index')) || 1;
            })
            );
            if (maxZ == undefined || maxZ == null)
                maxZ = 1;
            return maxZ;
        },

        //To prevent default actions for the element
        blockDefaultActions: function (e) {
            e.cancelBubble = true;
            e.returnValue = false;
            if (e.preventDefault) e.preventDefault();
            if (e.stopPropagation) e.stopPropagation();
        },

        //To get dimensions of the element when its hidden
        getDimension: function (element, method) {
            var value;
            var $hidden = $(element).parents().andSelf().filter(':hidden');
            if ($hidden) {
                var prop = { visibility: 'hidden', display: 'block' };
                var tmp = [];
                $hidden.each(function () {
                    var temp = {}, name;
                    for (name in prop) {
                        temp[name] = this.style[name];
                        this.style[name] = prop[name];
                    }
                    tmp.push(temp);
                });
                value = /(outer)/g.test(method) ?
                $(element)[method](true) :
               $(element)[method]();

                $hidden.each(function (i) {
                    var temp = tmp[i], name;
                    for (name in prop) {
                        this.style[name] = temp[name];
                    }
                });
            }
            return value;
        },
        //Get triggers when transition End
        transitionEndEvent: function () {
            var transitionEnd = {
                '': 'transitionend',
                'webkit': 'webkitTransitionEnd',
                'Moz': 'transitionend',
                'O': 'otransitionend',
                'ms': 'MSTransitionEnd'
            };

            return transitionEnd[ej.userAgent()];
        },
        //To return the start event to bind for element
        startEvent: function () {
            return (ej.isTouchDevice() || $.support.hasPointer) ? "touchstart" : "mousedown";
        },
        //To return end event to bind for element
        endEvent: function () {
            return (ej.isTouchDevice() || $.support.hasPointer) ? "touchend" : "mouseup"
        },
        //To return move event to bind for element
        moveEvent: function () {
            return (ej.isTouchDevice() || $.support.hasPointer) ? ($.support.hasPointer && !ej.isMobile()) ? "ejtouchmove" : "touchmove" : "mousemove";
        },
        //To return cancel event to bind for element
        cancelEvent: function () {
            return (ej.isTouchDevice() || $.support.hasPointer) ? "touchcancel" : "mousecancel";
        },
        //To return tap event to bind for element
        tapEvent: function () {
            return (ej.isTouchDevice() || $.support.hasPointer) ? "tap" : "click";
        },
        //To return tap hold event to bind for element
        tapHoldEvent: function () {
            return (ej.isTouchDevice() || $.support.hasPointer) ? "taphold" : "click";
        },
        //To check whether its Device
        isDevice: function () {
            if (ej.getBooleanVal($('head'), 'data-ej-forceset', false))
                return ej.getBooleanVal($('head'), 'data-ej-device', this._device());
            else
                return this._device();
        },
        //To check whether its windows web view
        isWindowsWebView: function () {
            return location.href.indexOf("x-wmapp") != -1;
        },
        _device: function () {
            return (/mobile|tablet|android|kindle/i.test(navigator.userAgent.toLowerCase()));
        },
        //To check whether its Touch Device
        isTouchDevice: function () {
            return ('ontouchstart' in window && this.isDevice());
        },
        //To get the outerHTML string for object
        getClearString: function(string){
            return $.trim(string.replace(/\s+/g," ").replace(/(\r\n|\n|\r)/gm,""));
        },
        //Get the attribute value with boolean type of element
        getBooleanVal: function (ele, val, option) {
            /// <summary>Util to get the property from data attributes</summary>
            /// <param name="ele" type="Object"></param>
            /// <param name="val" type="String"></param>
            /// <param name="option" type="GenericType"></param>
            /// <returns type="GenericType"></returns>
            var value = $(ele).attr(val);
            if (value != null)
                return value.toLowerCase() == "true";
            else
                return option;
        },
        //Gets the Skew class based on the element current position
        _getSkewClass: function (item, pageX, pageY) {
            var itemwidth = item.width();
            var itemheight = item.height();
            var leftOffset = item.offset().left;
            var rightOffset = item.offset().left + itemwidth;
            var topOffset = item.offset().top;
            var bottomOffset = item.offset().top + itemheight;
            var widthoffset = itemwidth * 0.3;
            var heightoffset = itemheight * 0.3;
            if (pageX < leftOffset + widthoffset && pageY < topOffset + heightoffset)
                return "e-m-skew-topleft";
            if (pageX > rightOffset - widthoffset && pageY < topOffset + heightoffset)
                return "e-m-skew-topright";
            if (pageX > rightOffset - widthoffset && pageY > bottomOffset - heightoffset)
                return "e-m-skew-bottomright";
            if (pageX < leftOffset + widthoffset && pageY > bottomOffset - heightoffset)
                return "e-m-skew-bottomleft";
            if (pageX > leftOffset + widthoffset && pageY < topOffset + heightoffset && pageX < rightOffset - widthoffset)
                return "e-m-skew-top";
            if (pageX < leftOffset + widthoffset)
                return "e-m-skew-left";
            if (pageX > rightOffset - widthoffset)
                return "e-m-skew-right";
            if (pageY > bottomOffset - heightoffset)
                return "e-m-skew-bottom";
            return "e-m-skew-center";
        },
        //Removes the added Skew class on the element
        _removeSkewClass: function (element) {
            $(element).removeClass("e-m-skew-top e-m-skew-bottom e-m-skew-left e-m-skew-right e-m-skew-topleft e-m-skew-topright e-m-skew-bottomleft e-m-skew-bottomright e-m-skew-center e-skew-top e-skew-bottom e-skew-left e-skew-right e-skew-topleft e-skew-topright e-skew-bottomleft e-skew-bottomright e-skew-center");
        },
        _touchStartPoints: function (evt, object) {
            var point = evt.touches ? evt.touches[0] : evt; 
            object._distX = 0;
            object._distY = 0;
            object._moved = false;
            object._pointX = point.pageX;
            object._pointY = point.pageY;
        },
        _isTouchMoved: function (evt, object) {
            var point = evt.touches ? evt.touches[0] : evt,
              deltaX = point.pageX - object._pointX,
              deltaY = point.pageY - object._pointY,
              timestamp = Date.now(),
              newX, newY,
              absDistX, absDistY;
            object._pointX = point.pageX;
            object._pointY = point.pageY;
            object._distX += deltaX;
            object._distY += deltaY;
            absDistX = Math.abs(object._distX);
            absDistY = Math.abs(object._distY); 
            return !(absDistX < 5 && absDistY < 5);
        },
        //To bind events for element
        listenEvents: function (selectors, eventTypes, handlers, remove, disableMouse) {
            for (var i = 0; i < selectors.length; i++) {
                ej.listenTouchEvent(selectors[i], eventTypes[i], handlers[i], remove, disableMouse);
            }
        },
        //To bind touch events for element
        listenTouchEvent: function (selector, eventType, handler, remove, disableMouse) {
            var event = remove ? "removeEventListener" : "addEventListener";
            var jqueryEvent = remove ? "off" : "on";
            var elements = $(selector);
            for (var i = 0; i < elements.length; i++) {
                var element = elements[i];
                switch (eventType) {
                    case "touchstart":
                        ej._bindEvent(element, event, eventType, handler, "mousedown", "MSPointerDown", "pointerdown", disableMouse);
                        break;
                    case "touchmove":
                        ej._bindEvent(element, event, eventType, handler, "mousemove", "MSPointerMove", "pointermove", disableMouse);
                        break;
                    case "touchend":
                        ej._bindEvent(element, event, eventType, handler, "mouseup", "MSPointerUp", "pointerup", disableMouse);
                        break;
                    case "touchcancel":
                        ej._bindEvent(element, event, eventType, handler, "mousecancel", "MSPointerCancel", "pointercancel", disableMouse);
                        break;
                    case "tap": case "taphold": case "ejtouchmove":
                        $(element)[jqueryEvent](eventType, handler);
                        break;
                    default:
                        element[event](eventType, handler, true);
                        break;
                }
            }
        },
        //To bind events for element
        _bindEvent: function (element, event, eventType, handler, mouseEvent, pointerEvent, ie11pointerEvent, disableMouse) {
            if ($.support.hasPointer)
                element[event](window.navigator.pointerEnabled ? ie11pointerEvent : pointerEvent, handler, true);
            else
                element[event](eventType, handler, true);
        },
        _browser: function () {
            return (/webkit/i).test(navigator.appVersion) ? 'webkit' : (/firefox/i).test(navigator.userAgent) ? 'Moz' : (/trident/i).test(navigator.userAgent) ? 'ms' : 'opera' in window ? 'O' : '';
        },
		styles: document.createElement('div').style,
         /**		
        * To get the userAgent Name     
        * @example             
        * &lt;script&gt;
        *       ej.userAgent();//return user agent name
        * &lt;/script&gt         
        * @memberof AppView
        * @instance
        */
        userAgent: function () {
            var agents = 't,webkitT,MozT,msT,OT'.split(','),
            t,
            i = 0,
            l = agents.length;

            for (; i < l; i++) {
                t = agents[i] + 'ransform';
                if (t in ej.styles) {
                    return agents[i].substr(0, agents[i].length - 1);
                }
            }

            return false;
        },
        addPrefix: function (style) {
            if (ej.userAgent() === '') return style;

            style = style.charAt(0).toUpperCase() + style.substr(1);
            return ej.userAgent() + style;
        },
        //To Prevent Default Exception

        //To destroy the mobile widgets
        destroyWidgets: function (element) {
            var dataEl = $(element).find("[data-role *= ejm]");
            dataEl.each(function (index, element) {
                var $element = $(element);
                var plugin = $element.data("ejWidgets");
                if (plugin)
                    $element[plugin]("destroy");
            });
        },
        //Get the attribute value of element
        getAttrVal: function (ele, val, option) {
            /// <summary>Util to get the property from data attributes</summary>
            /// <param name="ele" type="Object"></param>
            /// <param name="val" type="String"></param>
            /// <param name="option" type="GenericType"></param>
            /// <returns type="GenericType"></returns>
            var value = $(ele).attr(val);
            if (value != null)
                return value;
            else
                return option;
        },
    };

    $.extend(ej, ej.util);

    // base class for all ej widgets. It will automatically inhertied
    ej.widgetBase = {
        droppables: { 'default': [] },
        resizables: { 'default': [] },

        _renderEjTemplate: function (selector, data) {
            return ej.template.render(this, selector, data);
        },

        destroy: function () {

            if (this._trigger("destroy"))
                return;

            if (this.model.enablePersistence)
                this.persistState();

            try {
                this._destroy();
            } catch (e) { }

            var arr = this.element.data("ejWidgets") || [];
            for (var i = 0; i < arr.length; i++) {
                if (arr[i] == this.pluginName) {
                    arr.splice(i, 1);
                }
            }
            if (!arr.length)
                this.element.removeData("ejWidgets");

            while (this._events) {
                var item = this._events.pop(), args = [];

                if (!item)
                    break;

                for (var i = 0; i < item[1].length; i++)
                    if (!$.isPlainObject(item[1][i]))
                        args.push(item[1][i]);

                $.fn.off.apply(item[0], args);
            }

            this._events = null;

            this.element
                .removeClass(ej.util.getNameSpace(this.sfType))
                .removeClass("e-js")
                .removeData(this.pluginName);

            this.element = null;
            this.model = null;
        },

        _on: function (element) {
            if (!this._events)
                this._events = [];
            var args = [].splice.call(arguments, 1, arguments.length - 1);

            var handler = {}, i = args.length;
            while (handler && typeof handler !== "function") {
                handler = args[--i];
            }

            args[i] = ej.proxy(args[i], this);

            this._events.push([element, args, handler, args[i]]);

            $.fn.on.apply(element, args);

            return this;
        },

        _off: function (element, eventName, selector, handlerObject) {
            var e = this._events, temp;
            if (!e || !e.length)
                return this;
            if (typeof selector == "function") {
                temp = handlerObject;
                handlerObject = selector;
                selector = temp;
            }
            for (var i = 0; i < e.length; i++) {
                var arg = e[i],
                r = arg[0].length && (!handlerObject || arg[2] === handlerObject) && arg[1][0] === eventName && (!selector || arg[1][1] === selector) && $.inArray(element[0], arg[0]) > -1;
                if (r) {
                    $.fn.off.apply(element, handlerObject ? [eventName, selector, arg[3]] : [eventName, selector]);
                    e.splice(i, 1);
                    break;
                }
            }

            return this;
        },

        // Client side events wire-up / trigger helper.
        _trigger: function (eventName, eventProp) {
            var fn = this.model[eventName], returnValue, args, clientProp = {};
            $.extend(clientProp, eventProp)

            if (fn) {
                if (typeof fn === "string") {
                    fn = ej.util.getObject(fn, window);
                }

                if ($.isFunction(fn)) {

                    args = ej.event(eventName, this.model, eventProp);

                    returnValue = fn.call(this, args);

                    // sending changes back - deep copy option should not be enabled for this $.extend 
                    if (eventProp) $.extend(eventProp, args);

                    if (args.cancel || !ej.isNullOrUndefined(returnValue))
                        return returnValue === false || args.cancel;
                }
            }

            var isPropDefined = Boolean(eventProp);
            eventProp = eventProp || {};
            eventProp.originalEventType = eventName;
            eventProp.type = this.pluginName + eventName;

            args = $.Event(eventProp.type, ej.event(eventProp.type, this.model, eventProp));

            this.element.trigger(args);

            // sending changes back - deep copy option should not be enabled for this $.extend 
            if (isPropDefined) $.extend(eventProp, args);

            if (ej.isOnWebForms && args.cancel == false && this.model.serverEvents && this.model.serverEvents.length)
                ej.raiseWebFormsServerEvents(eventName, eventProp, clientProp);

            return args.cancel;
        },

        setModel: function (options, forceSet) {
            // check for whether to apply values are not. if _setModel function is defined in child,
            //  this will call that function and validate it using return value

            if (this._trigger("modelChange", { "changes": options }))
                return;


            for (var prop in options) {
                if (!forceSet) {
                    if (this.model[prop] === options[prop]) {
                        delete options[prop];
                        continue;
                    }
                    if ($.isPlainObject(options[prop])) {
                        iterateAndRemoveProps(this.model[prop], options[prop]);
                        if ($.isEmptyObject(options[prop]))
                            continue;
                    }
                }

                if (this.dataTypes) {
                    var returnValue = this._isValidModelValue(prop, this.dataTypes, options);
                    if (returnValue !== true)
                        throw "setModel - Invalid input for property :" + prop + " - " + returnValue;
                }
                if (this.model.notifyOnEachPropertyChanges && this.model[prop] !== options[prop]) {
                    var arg = {
                        oldValue: this.model[prop],
                        newValue: options[prop]
                    };

                    options[prop] = this._trigger(prop + "Change", arg) ? this.model[prop] : arg.newValue;
                }
            }
            if ($.isEmptyObject(options))
                return;

            if (this._setFirst) {
                var ds = options.dataSource;
                if (ds) delete options.dataSource

                $.extend(true, this.model, options);
                if (ds) {
                    this.model.dataSource = (ds instanceof Array) ? ds.slice() : ds;
                    options["dataSource"] = this.model.dataSource;
                }
                !this._setModel || this._setModel(options);

            } else if (!this._setModel || this._setModel(options) !== false) {
                $.extend(true, this.model, options);
            }
            if ("enablePersistence" in options) {
                if (options.enablePersistence === true) {
                    this._persistHandler = ej.proxy(this.persistState, this);
                    $(window).on("unload", this._persistHandler);
                } else {
                    $(window).off("unload", this._persistHandler);
                }
            }
        },

        option: function (prop, value, forceSet) {
            if (!prop)
                return this.model;

            if ($.isPlainObject(prop))
                return this.setModel(prop);

            if (typeof prop === "string") {
                prop = prop.replace(/^model\./, "");
                var oldValue = ej.getObject(prop, this.model);

                if (value === undefined)
                    return oldValue;

                if (forceSet && value === ej.extensions.modelGUID) {
                    return this._setModel(ej.createObject(prop, ej.getObject(prop, this.model), {}));
                }

                if (forceSet || ej.getObject(prop, this.model) !== value)
                    return this.setModel(ej.createObject(prop, value, {}), forceSet);
            }
            return undefined;
        },

        _isValidModelValue: function (prop, types, options) {
            var value = types[prop], option = options[prop], returnValue;

            if (!value)
                return true;

            if (typeof value === "string") {
                if (value == "enum") {
                    options[prop] = option ? option.toString().toLowerCase() : option;
                    value = "string";
                }

                if (value === "array") {
                    if (option instanceof Array)
                        return true;
                }
                else if (value === "data") {
                    return true;
                }
                else if (typeof option === value)
                    return true;

                return "Expected type - " + value;
            }

            if (option instanceof Array) {
                for (var i = 0; i < option.length; i++) {
                    returnValue = this._isValidModelValue(prop, types, option[i]);
                    if (returnValue !== true) {
                        return " [" + i + "] - " + returnValue;
                    }
                }
                return true;
            }

            for (var innerProp in option) {
                returnValue = this._isValidModelValue(innerProp, value, option);
                if (returnValue !== true)
                    return innerProp + " : " + returnValue;
            }

            return true;
        },

        persistState: function () {
            var model = copyObject({}, this.model);

            if (this._ignoreOnPersist) {
                for (var i = 0; i < this._ignoreOnPersist; i++) {
                    delete model[this._ignoreOnPersist[i]];
                }
            }

            if (this._persistState) {
                model.customPersists = {};
                this._persistState(model.customPersists);
            }

            if (window.localStorage)
                window.localStorage.setItem(this.pluginName + this._id, JSON.stringify(model));
            else if (document.cookie)
                ej.cookie.set(this.pluginName + this._id, model);
        },

        restoreState: function (silent) {
            var value = null;
            if (window.localStorage)
                value = window.localStorage.getItem(this.pluginName + this._id);
            else if (document.cookie)
                value = ej.cookie.get(this.pluginName + this._id);

            if (value) {
                var model = JSON.parse(value);

                if (this._restoreState) {
                    this._restoreState(model.customPersists);
                    delete model.customPersists;
                }

                this.model = $.extend(true, this.model, model);
            }

            if (!silent && value && this._setModel)
                this._setModel(this.model);
        }
    };

    var iterateAndRemoveProps = function (source, target) {
        for (var prop in source) {
            if (source[prop] === target[prop])
                delete target[prop];
            if ($.isPlainObject(target[prop]) && $.isPlainObject(source[prop]))
                iterateAndRemoveProps(source[prop], target[prop]);
        }
    }

    ej.widget = function (pluginName, className, proto) {
        /// <summary>Widget helper for developers, this set have predefined function to jQuery plug-ins</summary>
        /// <param name="pluginName" type="String">the plugin name that will be added in jquery.fn</param>
        /// <param name="className" type="String">the class name for your plugin, this will help create default cssClas</param>
        /// <param name="proto" type="Object">prototype for of the plug-in</param>

        if (typeof pluginName === "object") {
            proto = className;
            for (var prop in pluginName) {
                var name = pluginName[prop];

                if (name instanceof Array) {
                    proto._rootCSS = name[1];
                    name = name[0];
                }

                ej.widget(prop, name, proto);

                if (pluginName[prop] instanceof Array)
                    proto._rootCSS = "";
            }

            return;
        }

        var nameSpace = proto._rootCSS || ej.getNameSpace(className);

        proto = ej.defineClass(className, function (element, options) {

            this.sfType = className;
            this.pluginName = pluginName;
            this.instance = pInstance;

            if (ej.isNullOrUndefined(this._setFirst))
                this._setFirst = true;

            this["ob.values"] = {};

            $.extend(this, ej.widgetBase);

            if (this.dataTypes) {
                for (var property in options) {
                    var returnValue = this._isValidModelValue(property, this.dataTypes, options);
                    if (returnValue !== true)
                        throw "setModel - Invalid input for property :" + property + " - " + returnValue;
                }
            }

            var arr = (element.data("ejWidgets") || []);
            arr.push(pluginName);
            element.data("ejWidgets", arr);

            for (var i = 0; ej.widget.observables && this.observables && i < this.observables.length; i++) {
                var t = ej.getObject(this.observables[i], options);
                if (t) ej.createObject(this.observables[i], ej.widget.observables.register(t, this.observables[i], this, element), options);
            }

            this.element = element.jquery ? element : $(element);
            this.model = copyObject(true, {}, proto.prototype.defaults, options);
            this.model.keyConfigs = copyObject(this.keyConfigs);
			
			this.element.addClass(nameSpace + " e-js").data(pluginName, this);

            this._id = element[0].id;

            if (!this.element.attr("tabIndex"))
                this.element.attr("tabIndex", "");

            if (this.model.enablePersistence) {
                this._persistHandler = ej.proxy(this.persistState, this);
                $(window).on("unload", this._persistHandler);
                this.restoreState(true);
            }

            this._init();

            if (typeof this.model.keyConfigs === "object" && !(this.model.keyConfigs instanceof Array)) {
                var requiresEvt = false;
                if (this.model.keyConfigs.focus)
                    this.element.attr("accesskey", this.model.keyConfigs.focus);

                for (var keyProps in this.model.keyConfigs) {
                    if (keyProps !== "focus") {
                        requiresEvt = true;
                        break;
                    }
                }

                if (requiresEvt && this._keyPressed) {
                    var el = element, evt = "keydown";

                    if (this.keySettings) {
                        el = this.keySettings.getElement ? this.keySettings.getElement() || el : el;
                        evt = this.keySettings.event || evt;
                    }

                    this._on(el, evt, function (e) {
                        if (!this.model.keyConfigs) return;

                        var action = keyFn.getActionFromCode(this.model.keyConfigs, e.which, e.ctrlKey, e.shiftKey, e.altKey);
                        var arg = {
                            code: e.which,
                            ctrl: e.ctrlKey,
                            alt: e.altKey,
                            shift: e.shiftKey
                        };
                        if (!action) return;

                        if (this._keyPressed(action, e.target, arg, e) === false)
                            e.preventDefault();
                    });
                }
            }

            this._trigger("create");
        }, proto);

        $.fn[pluginName] = function (options) {

            for (var i = 0; i < this.length; i++) {

                var $this = $(this[i]),
                    pluginObj = $this.data(pluginName),
                    isAlreadyExists = pluginObj && $this.hasClass(nameSpace),
                    obj = null;

                // ----- plug-in creation/init
                if (!isAlreadyExists) {
                    if (!options || typeof options === "object") {
                        new proto($this, options);
                    }
                    else {
                        throwError(pluginName + ": methods/properties can be accessed only after plugin creation");
                    }
                    continue;
                }

                if (!options) continue;

                // --- Function/property set/access
                if ($.isPlainObject(options)) {
                    // setModel using JSON object
                    pluginObj.setModel(options);
                }

                    // function/property name starts with "_" is private so ignore it.
                else if (options.indexOf('_') !== 0
                    && !ej.isNullOrUndefined(obj = ej.getObject(options, pluginObj))
                    || options.indexOf("model.") === 0) {

                    if (!obj || !$.isFunction(obj)) {

                        // if property is accessed, then break the jquery chain
                        if (arguments.length == 1)
                            return obj;

                        //setModel using string input
                        pluginObj.option(options, arguments[1]);

                        continue;
                    }

                    var value = obj.apply(pluginObj, [].splice.call(arguments, 1, arguments.length - 1));

                    // If function call returns any value, then break the jquery chain
                    if (value !== undefined)
                        return value;

                } else {
                    throwError(className + ": function/property - " + options + " does not exist");
                }
            }

            // maintaining jquery chain
            return this;
        };

        ej.widget.register(pluginName, className, proto.prototype);
    };

    $.extend(ej.widget, (function () {
        var _widgets = {},

        register = function (pluginName, className, prototype) {
            if (!ej.isNullOrUndefined(_widgets[pluginName]))
                throwError("ej.widget : The widget named " + pluginName + " is trying to register twice.");

            _widgets[pluginName] = { name: pluginName, className: className, proto: prototype };
        }

        return {
            register: register,
            registeredWidgets: _widgets
        };

    })());

    ej.widget.destroyAll = function (elements) {
        if (!elements || !elements.length) return;

        for (var i = 0; i < elements.length; i++) {
            var data = elements.eq(i).data(), wds = data["ejWidgets"];
            if (wds && wds.length) {
                for (var j = 0; j < wds.length; j++) {
                    if (data[wds[j]] && data[wds[j]].destroy)
                        data[wds[j]].destroy();
                }
            }
        }
    };

    ej.cookie = {
        get: function (name) {
            var value = RegExp(name + "=([^;]+)").exec(document.cookie);

            if (value && value.length > 1)
                return value[1];

            return undefined;
        },
        set: function (name, value, expiryDate) {
            if (typeof value === "object")
                value = JSON.stringify(value);

            value = escape(value) + ((expiryDate == null) ? "" : "; expires=" + expiryDate.toUTCString());
            document.cookie = name + "=" + value;
        }
    };

    var keyFn = {
        getActionFromCode: function (keyConfigs, keyCode, isCtrl, isShift, isAlt) {
            isCtrl = isCtrl || false;
            isShift = isShift || false;
            isAlt = isAlt || false;

            for (var keys in keyConfigs) {
                if (keys === "focus") continue;

                var key = keyFn.getKeyObject(keyConfigs[keys]);
                if (keyCode === key.code && isCtrl == key.isCtrl && isShift == key.isShift && isAlt == key.isAlt)
                    return keys;
            }
            return null;
        },
        getKeyObject: function (key) {
            var res = {
                isCtrl: false,
                isShift: false,
                isAlt: false
            };
            key = key.split("+");
            for (var i = 0; i < key.length; i++) {
                if (key[i] === "ctrl")
                    res.isCtrl = true;
                else if (key[i] === "shift")
                    res.isShift = true;
                else if (key[i] === "alt")
                    res.isAlt = true;
                else res.code = parseInt(key[i], 10);
            }
            return res;
        }
    };

    ej.browserInfo = function () {
        var browser = {}, clientInfo = [],
        browserClients = {
            webkit: /(chrome)[ \/]([\w.]+)/i, safari: /(webkit)[ \/]([\w.]+)/i, msie: /(msie) ([\w.]+)/i,
            opera: /(opera)(?:.*version|)[ \/]([\w.]+)/i, mozilla: /(mozilla)(?:.*? rv:([\w.]+)|)/i
        };
        for (var client in browserClients) {
            if (browserClients.hasOwnProperty(client)) {
                clientInfo = navigator.userAgent.match(browserClients[client]);
                if (clientInfo) {
                    browser.name = clientInfo[1].toLowerCase();
                    browser.version = clientInfo[2];
                    if (!!navigator.userAgent.match(/Trident\/7\./)) {
                        browser.name = "msie";
                    }
                    break;
                }
            }
        }
        browser.isMSPointerEnabled = (browser.name == 'msie') && browser.version > 9 && window.navigator.msPointerEnabled;
        return browser;
    };

    ej.eventType = {
        mouseDown: "mousedown touchstart",
        mouseMove: "mousemove touchmove",
        mouseUp: "mouseup touchend",
        mouseLeave: "mouseleave touchcancel",
        click: "click touchend"
    };

    ej.event = function (type, data, eventProp) {

        var e = $.extend(eventProp || {},
            {
                "type": type,
                "model": data,
                "cancel": false
            });

        return e;
    };

    ej.proxy = function (fn, context, arg) {
        if (!fn || typeof fn !== "function")
            return null;

        if ('bind' in fn && context)
            return arg ? fn.bind(context, arg) : fn.bind(context);

        return function () {
            var args = arg ? [arg] : []; args.push.apply(args, arguments);
            return fn.apply(context || this, args);
        };
    };

    ej.hasStyle = function (prop) {
        var style = document.documentElement.style;

        if (prop in style) return true;

        var prefixs = ['ms', 'Moz', 'Webkit', 'O', 'Khtml'];

        prop = prop[0].toUpperCase() + prop.slice(1);

        for (var i = 0; i < prefixs.length; i++) {
            if (prefixs[i] + prop in style)
                return true;
        }

        return false;
    };

    Array.prototype.indexOf = Array.prototype.indexOf || function (searchElement) {
        var len = this.length;

        if (len === 0) return -1;

        for (var i = 0; i < len; i++) {
            if (i in this && this[i] === searchElement)
                return i;
        }
        return -1;
    };

    String.prototype.startsWith = String.prototype.startsWith || function (key) {
        return this.slice(0, key.length) === key;
    };
    var copyObject = ej.copyObject = function (isFirstDefault, target) {
        var start = 2, current, source;
        if (typeof isFirstDefault !== "boolean") {
            start = 1;
        }
        var objects = [].slice.call(arguments, start);
        if (start === 1) {
            target = isFirstDefault;
            isFirstDefault = undefined;
        }

        for (var i = 0; i < objects.length; i++) {
            for (var prop in objects[i]) {
                current = target[prop], source = objects[i][prop];

                if (source === undefined || current === source || objects[i] === source || target === source)
                    continue;
                if (source instanceof Array) {
                    if (i === 0 && isFirstDefault)
                        target[prop] = source.slice();
                    else
                        target[prop] = source.slice();
                } else if (ej.isPlainObject(source)) {
                    target[prop] = current || {};
                    if (isFirstDefault)
                        copyObject(isFirstDefault, target[prop], source);
                    else
                        copyObject(target[prop], source);
                } else
                    target[prop] = source;
            }
        }
        return target;
    };
    var pInstance = function () {
        return this;
    }

    ej.template = {
        render: function (self, selector, data) {
            if (selector.slice(0, 1) !== "#")
                selector = ["<div>", selector, "</div>"].join("");
            return $(selector).render(data);
        }
    }

    ej.template.defaults = ej.template.render;

    ej.isPlainObject = function (obj) {
        if (!obj) return false;

        if (typeof obj !== "object" || obj.nodeType || jQuery.isWindow(obj)) {
            return false;
        }

        try {
            if (obj.constructor &&
                !obj.constructor.prototype.hasOwnProperty("isPrototypeOf")) {
                return false;
            }
        } catch (e) {
            return false;
        }

        var key, ownLast = ej.support.isOwnLast;
        for (key in obj) {
            if (ownLast) break;
        }

        return key === undefined || obj.hasOwnProperty(key);
    };
    ej.util.valueFunction = function (prop) {
        return function (value) {
            var val = ej.getObject(prop, this.model);

            if (value === undefined)
                return typeof val === "function" ? val() : val;

            if (typeof val === "function") {
                this["ob.values"][prop] = value;
                val.call(this, value);
            }
            else
                ej.createObject(prop, value, this.model);
        }
    };
    ej.util.getVal = function (val) {
        if (typeof val === "function")
            return val();
        return val;
    };
    ej.support = {
        isOwnLast: function () {
            var fn = function () { this.a = 1; };
            fn.prototype.b = 1;

            for (var p in new fn()) {
                return p === "b";
            }
        }(),
        outerHTML: function () {
            return document.createElement("div").outerHTML !== undefined;
        }()
    };

    var throwError = ej.throwError = function (er) {
        try {
            throw new Error(er);
        } catch (e) {
            throw e.message + "\n" + e.stack;
        }
    };
   
    

    ej.extensions = {};
    ej.extensions.modelGUID = "{0B1051BA-1CCB-42C2-A3B5-635389B92A50}";
})(window.jQuery, window.Syncfusion);
$.fn.addAttrs = function (json) {
        var $this = $(this);
        $.each(json, function (i, attr) {
            if (attr && attr.specified) {
                $this.attr(attr.name, attr.value);
            }
        });

    };
    $.fn.removeAttrs = function (regex) {
        return this.each(function () {
            var $this = $(this),
                names = [],
                attrs = $(this.attributes).clone();
            $.each(attrs, function (i, attr) {
                if (attr && attr.specified && regex.test(attr.name)) {
                    $this.removeAttr(attr.name);
                }
            });
        });
    };
    $.fn.removeEmptyAttrs = function () {
        return this.each(function () {
            var $this = $(this),
                names = [],
                attrs = $(this.attributes).clone();
            $.each(attrs, function (i, attr) {
                if (attr && attr.specified && attr.value === "") {
                    $this.removeAttr(attr.name);
                }
            });
        });
    };;
window.ej = window.Syncfusion = window.Syncfusion || {};

(function ($, ej, doc, undefined) {
    'use strict';

    ej.DataManager = function (dataSource, query, adaptor) {
          if (!(this instanceof ej.DataManager))
            return new ej.DataManager(dataSource, query, adaptor);

        if (!dataSource)
            throwError("DataManger : a dataSource is required to create Manager");
        adaptor = adaptor || dataSource.adaptor

        if (typeof (adaptor) === "string") 
            adaptor = new ej[adaptor]();
        var data = [], self = this;

        if (dataSource instanceof Array) {
            // JSON array
            data = {
                json: dataSource,
                offline: true
            };

        } else if (typeof dataSource === "object") {
            if ($.isPlainObject(dataSource)) {
                if (dataSource.table)
                    dataSource.json = this._getJsonFromElement(dataSource.table, dataSource.headerOption);
                data = {
                    url: dataSource.url,
                    insertUrl: dataSource.insertUrl,
                    removeUrl: dataSource.removeUrl,
                    updateUrl: dataSource.updateUrl,
                    crudUrl: dataSource.crudUrl,
                    batchUrl: dataSource.batchUrl,
                    json: dataSource.json,
                    headers: dataSource.headers,
                    accept: dataSource.accept,
                    data: dataSource.data,
                    timeTillExpiration: dataSource.timeTillExpiration,
                    cachingPageSize: dataSource.cachingPageSize,
                    enableCaching: dataSource.enableCaching,
                    requestType: dataSource.requestType,
                    key: dataSource.key,
                    crossDomain: dataSource.crossDomain,
                    jsonp: dataSource.jsonp,
                    dataType: dataSource.dataType,
                    offline: dataSource.offline !== undefined ? dataSource.offline : dataSource.adaptor == "remoteSaveAdaptor" ? false : dataSource.url ? false : true,
                    requiresFormat: dataSource.requiresFormat
                };
            } else if (dataSource.jquery || isHtmlElement(dataSource)) {
                data = {
                    json: this._getJsonFromElement(dataSource),
                    offline: true,
                    table: dataSource
                };
            }
        } else if (typeof dataSource === "string") {
            data = {
                url: dataSource,
                offline: false,
                dataType: "json"
            };
        }

        if (data.requiresFormat === undefined && !ej.support.cors)
            data.requiresFormat = isNull(data.crossDomain) ? true : data.crossDomain;
        if (data.dataType === undefined)
            data.dataType = "json";

        this.dataSource = data;
        this.defaultQuery = query;

        if (data.url && data.offline && !data.json) {
            this.isDataAvailable = false;
            this.adaptor = adaptor || new ej.ODataAdaptor();
            this.dataSource.offline = false;
            this.ready = this.executeQuery(query || ej.Query()).done(function (e) {
                self.dataSource.offline = true;
                self.isDataAvailable = true;
                data.json = e.result;
                self.adaptor = new ej.JsonAdaptor();
            });
        }
        else
            this.adaptor = data.offline ? new ej.JsonAdaptor() : new ej.ODataAdaptor();
        if (data.enableCaching)
            adaptor = new ej.CacheAdaptor(this.adaptor, data.timeTillExpiration, data.cachingPageSize);
        if (!data.jsonp && this.adaptor instanceof ej.ODataAdaptor)
            data.jsonp = "callback";
        this.adaptor = adaptor || this.adaptor;
        return this;
    };

    ej.DataManager.prototype = {
        setDefaultQuery: function (query) {
            this.defaultQuery = query;
        },

        executeQuery: function (query, done, fail, always) {
            if (typeof query === "function") {
                always = fail;
                fail = done;
                done = query;
                query = null;
            }

            if (!query)
                query = this.defaultQuery;

            if (!(query instanceof ej.Query))
                throwError("DataManager - executeQuery() : A query is required to execute");

            var deffered = $.Deferred();

            deffered.then(done, fail, always);
            var args = { query: query };

            if (!this.dataSource.offline && this.dataSource.url != undefined) {
				 var result = this.adaptor.processQuery(this, query);
                if (!ej.isNullOrUndefined(result.url))
                    this._makeRequest(result, deffered, args, query);
                else {
                    nextTick(function () {
                        args = this._getDeferedArgs(query, result, args);
                        deffered.resolveWith(this, [args]);;
                    }, this);
                }
            } else {
                nextTick(function () {
                    var res = this.executeLocal(query);
                    args = this._getDeferedArgs(query, res, args);
                    deffered.resolveWith(this, [args]);
                }, this);
            }

            return deffered.promise();
        },
        _getDeferedArgs: function (query, result, args) {
            if (query._requiresCount) {
                args.result = result.result;
                args.count = result.count;
            } else
                args.result = result;
            args.getTableModel = getTableModel(query._fromTable, args.result, this);
            args.getKnockoutModel = getKnockoutModel(args.result);
            return args;
        },
        executeLocal: function (query) {
            if (!this.defaultQuery && !(query instanceof ej.Query))
                throwError("DataManager - executeLocal() : A query is required to execute");

            if (!this.dataSource.json)
                throwError("DataManager - executeLocal() : Json data is required to execute");

            var ret, key;
            query = query || this.defaultQuery;

            var result = this.adaptor.processQuery(this, query);

            if (query._subQuery) {
                var from = query._subQuery._fromTable, lookup = query._subQuery._lookup,
                    res = query._requiresCount ? result.result : result;

                if (lookup && lookup instanceof Array) {
                    buildHierarchy(query._subQuery._fKey, from, res, lookup);
                }

                for (var j = 0; j < res.length; j++) {
                    if (res[j][from] instanceof Array) {
                        res[j] = $.extend({}, res[j]);
                        res[j][from] = this.adaptor.processResponse(query._subQuery.using(ej.DataManager(res[j][from].slice(0))).executeLocal(), this, query);
                    }
                }
            }

            return this.adaptor.processResponse(result, this, query);
        },

        _makeRequest: function (url, deffered, args, query) {
            var isSelector = !!query._subQuerySelector;

            var fnFail = $proxy(function (e) {
                args.error = e;
                deffered.rejectWith(this, [args]);
            }, this);

            var process = $proxy(function (data, count, xhr, request, actual) {
                if (isSelector) return;

                args.xhr = xhr;
                args.count = parseInt(count, 10);
                args.result = data;
                args.request = request;
                args.getTableModel = getTableModel(query._fromTable, data, this);
                args.getKnockoutModel = getKnockoutModel(data);
                args.actual = actual;
                deffered.resolveWith(this, [args]);

            }, this);

            var fnQueryChild = $proxy(function (data, selector) {
                var subDeffer = $.Deferred(),
                    childArgs = { parent: args };

                query._subQuery._isChild = true;

                var subUrl = this.adaptor.processQuery(this, query._subQuery, data ? this.adaptor.processResponse(data) : selector);

                var childReq = this._makeRequest(subUrl, subDeffer, childArgs, query._subQuery);

                subDeffer.then(function (subData) {
                    if (data) {
                        buildHierarchy(query._subQuery._fKey, query._subQuery._fromTable, data, subData);
                        process(data);
                    }
                }, fnFail);

                return childReq;
            }, this);

            var fnSuccess = proxy(function (data, status, xhr, request) {
                data = p.parseJson(data);
                var result = this.adaptor.processResponse(data, this, query, xhr, request), count = 0;
                if (query._requiresCount) {
                    count = result.count;
                    result = result.result;
                }

                if (!query._subQuery) {
                    process(result, count, xhr, request, data);
                    return;
                }

                if (!isSelector)
                    fnQueryChild(result);

            }, this);

            var req = $.extend({
                type: "GET",
                dataType: this.dataSource.dataType,
                crossDomain: this.dataSource.crossDomain,
                jsonp: this.dataSource.jsonp,
                cache: true,
                beforeSend: $proxy(this._beforeSend, this),
                processData: false,
                success: fnSuccess,
                error: fnFail
            }, url);

            req = $.ajax(req);

            if (isSelector) {
                var res = query._subQuerySelector.call(this, { query: query._subQuery, parent: query });

                if (res && res.length) {
                    req = $.when(req, fnQueryChild(null, res));

                    req.then(proxy(function (pData, cData, requests) {
                        var pResult = this.adaptor.processResponse(pData[0], this, query, pData[2], requests[0]), count = 0;
                        if (query._requiresCount) {
                            count = pResult.count;
                            pResult = pResult.result;
                        }
                        var cResult = this.adaptor.processResponse(cData[0], this, query._subQuery, cData[2], requests[1]), count = 0;
                        if (query._subQuery._requiresCount) {
                            count = cResult.count;
                            cResult = cResult.result;
                        }

                        buildHierarchy(query._subQuery._fKey, query._subQuery._fromTable, pResult, cResult);
                        isSelector = false;
                        process(pResult, count, pData[2]);

                    }, this), fnFail);
                } else {
                    isSelector = false;
                }
            }

            return req;
        },

        _beforeSend: function (request, settings) {
            this.adaptor.beforeSend(this, request, settings);

            var headers = this.dataSource.headers, props;
            for (var i = 0; headers && i < headers.length; i++) {
                props = []
                for (var prop in headers[i]) {
                    props.push(prop)
                    request.setRequestHeader(prop, headers[i][prop]);
                }
            }
        },

        saveChanges: function (changes, key, tableName) {
            var args = {
                url: tableName,
                key: key || this.dataSource.key
            };

            var req = this.adaptor.batchRequest(this, changes, args);

            if (this.dataSource.offline) {
                return req;
            }

            var deff = $.Deferred();
            $.ajax($.extend({
                beforeSend: $proxy(this._beforeSend, this),
                success: proxy(function (data, status, xhr, request) {
                    deff.resolveWith(this, this.adaptor.processResponse(data, this, null, xhr, request, changes));
                }, this),
                fail: function (e) {
                    deff.rejectWith(this, e);
                }
            }, req));

            return deff;
        },

        insert: function (data, tableName) {
            var res = this.adaptor.insert(this, data, tableName);
            if (this.dataSource.offline) {
                return res;
            }

            var deffer = $.Deferred();

            $.ajax($.extend({
                type: "POST",
                contentType: "application/json; charset=utf-8",
                dataType: "json",
                processData: false,
                beforeSend: $proxy(this._beforeSend, this),
                success: proxy(function (record, status, xhr, request) {
                    record = this.adaptor.processResponse(p.parseJson(record), this, null, xhr, request);
                    deffer.resolveWith(this, [{ record: record, dataManager: this }]);
                }, this),
                error: function (e) {
                    deffer.rejectWith(this, [{ error: e, dataManager: this }]);
                }
            }, res));

            return deffer.promise();
        },

        remove: function (keyField, value, tableName) {
            if (typeof value === "object")
                value = value[keyField];

            var res = this.adaptor.remove(this, keyField, value, tableName);

            if (this.dataSource.offline)
                return res;

            return $.ajax($.extend({
                type: "POST",
                contentType: "application/json; charset=utf-8",
                beforeSend: $proxy(this._beforeSend, this),
                dataType: "json"
            }, res));
        },

        update: function (keyField, value, tableName) {
            var res = this.adaptor.update(this, keyField, value, tableName);

            if (this.dataSource.offline) {
                return res;
            }

            return $.ajax($.extend({
                contentType: "application/json; charset=utf-8",
                dataType: "json",
                beforeSend: $proxy(this._beforeSend, this)
            }, res));
        },

        _getJsonFromElement: function (ds) {
            if (typeof (ds) == "string")
                ds = $($(ds).html());

            ds = ds.jquery ? ds[0] : ds;

            var tagName = ds.tagName.toLowerCase();

            if (tagName !== "table")
                throwError("ej.DataManager : Unsupported htmlElement : " + tagName);

            return ej.parseTable(ds);
        }
    };

    var buildHierarchy = function (fKey, from, source, lookup) {
        var i, grp = {}, t;
        if (lookup.result) lookup = lookup.result;

        if (lookup.GROUPGUID)
            throwError("ej.DataManager: Do not have support Grouping in hierarchy");

        for (i = 0; i < lookup.length; i++) {
            t = grp[lookup[i][fKey]] || (grp[lookup[i][fKey]] = []);

            t.push(lookup[i]);
        }

        for (i = 0; i < source.length; i++) {
            source[i][from] = grp[source[i][fKey]];
        }
    };

    var oData = {
        accept: "application/json;odata=light;q=1,application/json;odata=verbose;q=0.5",
        multipartAccept: "multipart/mixed",
        batch: "$batch",
        changeSet: "--changeset_",
        batchPre: "batch_",
        contentId: "Content-Id: ",
        batchContent: "Content-Type: multipart/mixed; boundary=",
        changeSetContent: "Content-Type: application/http\nContent-Transfer-Encoding: binary ",
        batchChangeSetContentType: "Content-Type: application/json; charset=utf-8 "
    };
    var p = {
        parseJson: function (jsonText) {
            var type = typeof jsonText;
            if (type === "string") {
                jsonText = JSON.parse(jsonText, p.jsonReviver);
            } else if (jsonText instanceof Array) {
                p.iterateAndReviveArray(jsonText);
            } else if (type === "object")
                p.iterateAndReviveJson(jsonText);
            return jsonText;
        },
        iterateAndReviveArray: function (array) {
            for (var i = 0; i < array.length; i++) {
                if (typeof array[i] === "object")
                    p.iterateAndReviveJson(array[i]);
                else
                    array[i] = p.parseJson(array[i]);
            }
        },
        iterateAndReviveJson: function (json) {
            var value;

            for (var prop in json) {
                if (prop.startsWith("__"))
                    continue;

                value = json[prop];
                if (typeof value === "object") {
                    if (value instanceof Array)
                        p.iterateAndReviveArray(value);
                    else
                        p.iterateAndReviveJson(value);
                } else
                    json[prop] = p.jsonReviver(prop, value);
            }
        },
        jsonReviver: function (field, value) {
            var s = value;
            if (typeof value === "string") {
                var ms = /^\/Date\(([+-]?[0-9]+)([+-][0-9]{4})?\)\/$/.exec(value);
                if (ms)
                    return new Date(parseInt(ms[1]));
                else if (/^(\d{4}\-\d\d\-\d\d([tT][\d:\.]*)?)([zZ]|([+\-])(\d\d):?(\d\d))?$/.test(value)) {
                    value = new Date(value);
                    if (value == "Invalid Date") {
                        var a = s.split(/[^0-9]/);
                        value = new Date(a[0], a[1] - 1, a[2], a[3], a[4], a[5]);
                    }
                }
            }

            return value;
        },
        isJson: function (jsonData) {
            if(typeof jsonData[0]== "string")
                return jsonData;
            return ej.parseJSON(jsonData);
        }
    };

    ej.isJSON = p.isJson;
    ej.parseJSON = p.parseJson;

    ej.Query = function (from) {
        if (!(this instanceof ej.Query))
            return new ej.Query(from);

        this.queries = [];
        this._key = "";
        this._fKey = "";

        if (typeof from === "string")
            this._fromTable = from || "";
        else if (from && from instanceof Array)
            this._lookup = from;

        this._expands = [];
        this._sortedColumns = [];
        this._groupedColumns = [];
        this._subQuery = null;
        this._isChild = false;
        this._params = [];
        return this;
    };

    ej.Query.prototype = {
        key: function (field) {
            if (typeof field === "string")
                this._key = field;

            return this;
        },

        using: function (dataManager) {
            if (dataManager instanceof ej.DataManager) {
                this.dataManagar = dataManager;
                return this;
            }

            return throwError("Query - using() : 'using' function should be called with parameter of instance ej.DataManager");
        },

        execute: function (dataManager, done, fail, always) {
            dataManager = dataManager || this.dataManagar;

            if (dataManager && dataManager instanceof ej.DataManager)
                return dataManager.executeQuery(this, done, fail, always);

            return throwError("Query - execute() : dataManager needs to be is set using 'using' function or should be passed as argument");
        },

        executeLocal: function (dataManager) {
            // this does not support for URL binding

            dataManager = dataManager || this.dataManagar;

            if (dataManager && dataManager instanceof ej.DataManager)
                return dataManager.executeLocal(this);

            return throwError("Query - executeLocal() : dataManager needs to be is set using 'using' function or should be passed as argument");
        },

        clone: function () {
            var cl = new ej.Query();
            cl.queries = this.queries.slice(0);
            cl._key = this._key;
            cl._isChild = this._isChild;
            cl.dataManagar = this.dataManager;
            cl._fromTable = this._fromTable;
            cl._params = this._params.slice(0);
            cl._expands = this._expands.slice(0);
            cl._sortedColumns = this._sortedColumns.slice(0);
            cl._groupedColumns = this._groupedColumns.slice(0);
            cl._subQuerySelector = this._subQuerySelector;
            cl._subQuery = this._subQuery;
            cl._fKey = this._fKey;
            cl._requiresCount = this._requiresCount;
            return cl;
        },

        from: function (tableName) {
            if (typeof tableName === "string")
                this._fromTable = tableName;

            return this;
        },

        addParams: function (key, value) {
            if (typeof value !== "function" && !ej.isPlainObject(value))
                this._params.push({ key: key, value: value });
            else if (typeof value === "function")
                this._params.push({ key: key, fn: value });

            return this;
        },

        expand: function (tables) {
            if (typeof tables === "string")
                this._expands = [].slice.call(arguments, 0);
            else
                this._expands = tables.slice(0);

            return this;
        },

        where: function (fieldName, operator, value, ignoreCase) {
            operator = (operator || ej.FilterOperators.equal).toLowerCase();
            var predicate = null;

            if (typeof fieldName === "string")
                predicate = new ej.Predicate(fieldName, operator, value, ignoreCase);
            else if (fieldName instanceof ej.Predicate)
                predicate = fieldName;
            else
                throwError("Query - where : Invalid arguments");

            this.queries.push({
                fn: "onWhere",
                e: predicate
            });
            return this;
        },

        search: function (searchKey, fieldNames, operator, ignoreCase) {
            if (!fieldNames || typeof fieldNames === "boolean") {
                fieldNames = [];
                ignoreCase = fieldNames;
            } else if (typeof fieldNames === "string")
                fieldNames = [fieldNames];

            if (typeof operator === "boolean") {
                ignoreCase = operator;
                operator = null;
            }
            operator = operator || ej.FilterOperators.contains;
            if (operator.length < 3)
                operator = ej.data.operatorSymbols[operator];

            var comparer = ej.data.fnOperators[operator] || ej.data.fnOperators.processSymbols(operator);

            this.queries.push({
                fn: "onSearch",
                e: {
                    fieldNames: fieldNames,
                    operator: operator,
                    searchKey: searchKey,
                    ignoreCase: ignoreCase,
                    comparer: comparer
                }
            });
            return this;
        },

        sortBy: function (fieldName, comparer, isFromGroup) {
            var order = ej.sortOrder.Ascending, sorts, t;

            if (typeof fieldName === "string" && fieldName.toLowerCase().endsWith(" desc")) {
                fieldName = fieldName.replace(/ desc$/i, '');
                comparer = ej.sortOrder.Descending;
            }

            if (typeof comparer === "boolean")
                comparer = !comparer ? ej.sortOrder.Ascending : ej.sortOrder.Descending;
            else if (typeof comparer === "function")
                order = "custom";

            if (!comparer || typeof comparer === "string") {
                order = comparer ? comparer.toLowerCase() : ej.sortOrder.Ascending;
                comparer = ej.pvt.fnSort(comparer);
            }
            if (isFromGroup) {
                sorts = filterQueries(this.queries, "onSortBy");

                for (var i = 0; i < sorts.length; i++) {
                    t = sorts[i].e.fieldName;
                    if (typeof t === "string") {
                        if (t === fieldName) return;
                    } else if (t instanceof Array) {
                        for (var j = 0; j < t.length; j++)
                            if (t[j] === fieldName || fieldName.toLowerCase() === t[j] + " desc")
                                return;
                    }
                }
            }

            this.queries.push({
                fn: "onSortBy",
                e: {
                    fieldName: fieldName,
                    comparer: comparer,
                    direction: order
                }
            });

            return this;
        },

        sortByDesc: function (fieldName) {
            return this.sortBy(fieldName, ej.sortOrder.Descending);
        },

        group: function (fieldName) {
            this.sortBy(fieldName, null, true);

            this.queries.push({
                fn: "onGroup",
                e: {
                    fieldName: fieldName
                }
            });
            return this;
        },

        page: function (pageIndex, pageSize) {
            this.queries.push({
                fn: "onPage",
                e: {
                    pageIndex: pageIndex,
                    pageSize: pageSize
                }
            });
            return this;
        },

        range: function (start, end) {
            if (typeof start !== "number" || typeof end !== "number")
                throwError("Query() - range : Arguments type should be a number");

            this.queries.push({
                fn: "onRange",
                e: {
                    start: start,
                    end: end
                }
            });
            return this;
        },

        take: function (nos) {
            if (typeof nos !== "number")
                throwError("Query() - Take : Argument type should be a number");

            this.queries.push({
                fn: "onTake",
                e: {
                    nos: nos
                }
            });
            return this;
        },

        skip: function (nos) {
            if (typeof nos !== "number")
                throwError("Query() - Skip : Argument type should be a number");

            this.queries.push({
                fn: "onSkip",
                e: { nos: nos }
            });
            return this;
        },

        select: function (fieldNames) {
            if (typeof fieldNames === "string")
                fieldNames = [].slice.call(arguments, 0);

            if (!(fieldNames instanceof Array)) {
                throwError("Query() - Select : Argument type should be String or Array");
            }

            this.queries.push({
                fn: "onSelect",
                e: { fieldNames: fieldNames }
            });
            return this;
        },

        hierarchy: function (query, selectorFn) {
            if (!query || !(query instanceof ej.Query))
                throwError("Query() - hierarchy : query must be instance of ej.Query");

            if (typeof selectorFn === "function")
                this._subQuerySelector = selectorFn;

            this._subQuery = query;
            return this;
        },

        foreignKey: function (key) {
            if (typeof key === "string")
                this._fKey = key;

            return this;
        },

        requiresCount: function () {
            this._requiresCount = true;

            return this;
        }
    };

    ej.Adaptor = function (ds) {
        this.dataSource = ds;
        this.pvt = {};
		this.init.apply(this, [].slice.call(arguments, 1));
    };

    ej.Adaptor.prototype = {
        options: {
            from: "table",
            requestType: "json",
            sortBy: "sorted",
            select: "select",
            skip: "skip",
            group: "group",
            take: "take",
            search: "search",
            count: "requiresCounts",
            where: "where"
        },
        init: function () {
        },
        extend: function (overrides) {
            var fn = function (ds) {
                this.dataSource = ds;

                if (this.options)
                    this.options = $.extend({}, this.options);
				this.init.apply(this, [].slice.call(arguments, 0));

                this.pvt = {};
            };
            fn.prototype = new this.type();
            fn.prototype.type = fn;

            var base = fn.prototype.base = {};
            for (var p in overrides) {
                if (fn.prototype[p])
                    base[p] = fn.prototype[p];
            }
            $.extend(true, fn.prototype, overrides);
            return fn;
        },
        processQuery: function (dm, query) {
            // this needs to be overridden
        },
        processResponse: function (data, ds, query, xhr) {
            return data;
        },
        convertToQueryString: function (req, query, dm) {
            return $.param(req);
        },
        type: ej.Adaptor
    };

    ej.UrlAdaptor = new ej.Adaptor().extend({
        processQuery: function (dm, query, hierarchyFilters) {
            var sorted = filterQueries(query.queries, "onSortBy"),
                grouped = filterQueries(query.queries, "onGroup"),
                filters = filterQueries(query.queries, "onWhere"),
                searchs = filterQueries(query.queries, "onSearch"),
                singles = filterQueryLists(query.queries, ["onSelect", "onPage", "onSkip", "onTake", "onRange"]),
                params = query._params,
                url = dm.dataSource.url, tmp, skip, take = null,
                op = this.options;

            var r = {
                sorted: [],
                grouped: [],
                filters: [],
                searches: []
            };

            // calc Paging & Range
            if (singles["onPage"]) {
                tmp = singles["onPage"];
                skip = getValue(tmp.pageIndex, query);
                take = getValue(tmp.pageSize, query);
				skip = (skip - 1) * take;
            } else if (singles["onRange"]) {
                tmp = singles["onRange"];
                skip = tmp.start;
                take = tmp.end - tmp.start;
            }

            // Sorting
            for (var i = 0; i < sorted.length; i++) {
                tmp = getValue(sorted[i].e.fieldName, query);

                r.sorted.push(callAdaptorFunc(this, "onEachSort", { name: tmp, direction: sorted[i].e.direction }, query));
            }

            // hierarchy
            if (hierarchyFilters) {
                tmp = this.getFiltersFrom(hierarchyFilters, query);
                if (tmp)
                    r.filters.push(callAdaptorFunc(this, "onEachWhere", tmp.toJSON(), query));
            }

            // Filters
            for (var i = 0; i < filters.length; i++) {
                r.filters.push(callAdaptorFunc(this, "onEachWhere", filters[i].e.toJSON(), query));

                for (var prop in r.filters[i]) {
                    if (isNull(r[prop]))
                        delete r[prop];
                }
            }

            // Searches
            for (var i = 0; i < searchs.length; i++) {
                tmp = searchs[i].e;
                r.searches.push(callAdaptorFunc(this, "onEachSearch", {
                    fields: tmp.fieldNames,
                    operator: tmp.operator,
                    key: tmp.searchKey,
                    ignoreCase: tmp.ignoreCase
                }, query));
            }

            // Grouping
            for (var i = 0; i < grouped.length; i++) {
                r.grouped.push(getValue(grouped[i].e.fieldName, query));
            }

            var req = {};
            req[op.from] = query._fromTable;
            if (op.expand) req[op.expand] = query._expands;
            req[op.select] = singles["onSelect"] ? callAdaptorFunc(this, "onSelect", getValue(singles["onSelect"].fieldNames, query), query) : "";
            req[op.count] = query._requiresCount ? callAdaptorFunc(this, "onCount", query._requiresCount, query) : "";
            req[op.search] = r.searches.length ? callAdaptorFunc(this, "onSearch", r.searches, query) : "";
            req[op.skip] = singles["onSkip"] ? callAdaptorFunc(this, "onSkip", getValue(singles["onSkip"].nos, query), query) : "";
            req[op.take] = singles["onTake"] ? callAdaptorFunc(this, "onTake", getValue(singles["onTake"].nos, query), query) : "";
            req[op.where] = r.filters.length || r.searches.length ? callAdaptorFunc(this, "onWhere", r.filters, query) : "";
            req[op.sortBy] = r.sorted.length ? callAdaptorFunc(this, "onSortBy", r.sorted, query) : "";
            req[op.group] = r.grouped.length ? callAdaptorFunc(this, "onGroup", r.grouped, query) : "";

            // Params
            for (var i = 0; tmp = params[i]; i++) {
                if (req[tmp.key]) throwError("ej.Query: Custom Param is conflicting other request arguments");

                req[tmp.key] = tmp.value;
                if (tmp.fn)
                    req[tmp.key] = tmp.fn.call(query, tmp.key, query, dm);
            }

            // cleanup
            for (var prop in req) {
                if (isNull(req[prop]) || req[prop] === "" || req[prop].length === 0)
                    delete req[prop];
            }

            if (!(op.skip in req && op.take in req) && take !== null) {
                req[op.skip] = callAdaptorFunc(this, "onSkip", skip, query);
                req[op.take] = callAdaptorFunc(this, "onTake", take, query);
            }
            var p = this.pvt;
            this.pvt = {};

            if (this.options.requestType === "json") {
                return {
                    data: JSON.stringify(req),
                    url: url,
                    ejPvtData: p,
                    type: "POST",
                    contentType: "application/json; charset=utf-8"
                }
            }
            tmp = this.convertToQueryString(req, query, dm);
            return {
                type: "GET",
                url: tmp.length ? url.replace(/\/*$/, "/" + tmp) : url,
                ejPvtData: p
            };
        },
        convertToQueryString: function (req, query, dm) {
            if (dm.dataSource.url && dm.dataSource.url.indexOf("?") !== -1)
                return $.param(req);
            return "?" + $.param(req);
        },
        processResponse: function (data, ds, query, xhr, request, changes) {
            var pvt = request.ejPvtData || {};
            var d = JSON.parse(request.data)
            if (d && d.action === "batch" && data.added) {
                changes.added = data.added;
                return changes;
            }
            if (data.d)
                data = data.d;

            if (pvt.groups && pvt.groups.length) {
                var groups = pvt.groups, args = {};
                if (data["count"]) args.count = data.count;
                if (data["result"]) data = data.result;
                for (var i = 0; i < groups.length; i++)
                    data = ej.group(data, groups[i]);
                if (args.count)
                    args.result = data;
                else
                    args = data;
                return args;
            }
            return data;
        },
        onGroup: function (e) {
            this.pvt.groups = e;
        },
        batchRequest: function (dm, changes, e) {
            return {
                type: "POST",
                url: dm.dataSource.batchUrl || dm.dataSource.crudUrl || dm.dataSource.url,
                contentType: "application/json; charset=utf-8",
                dataType: "json",
                data: JSON.stringify({
                    changed: changes.changed,
                    added: changes.added,
                    deleted: changes.deleted,
                    action: "batch",
                    table: e.url,
                    key: e.key
                })
            };
        },
        beforeSend: function (dm, request) {
        },
        insert: function (dm, data, tableName) {
            return {
                url: dm.dataSource.insertUrl || dm.dataSource.crudUrl || dm.dataSource.url,
                data: JSON.stringify({
                    value: data,
                    table: tableName,
                    action: "insert"
                })
            };
        },
        remove: function (dm, keyField, value, tableName) {
            return {
                type: "POST",
                url: dm.dataSource.removeUrl || dm.dataSource.crudUrl || dm.dataSource.url,
                data: JSON.stringify({
                    key: value,
                    keyColumn: keyField,
                    table: tableName,
                    action: "remove"
                })
            };
        },
        update: function (dm, keyField, value, tableName) {
            return {
                type: "POST",
                url: dm.dataSource.updateUrl || dm.dataSource.crudUrl || dm.dataSource.url,
                data: JSON.stringify({
                    value: value,
                    action: "update",
                    keyColumn: keyField,
                    key: value[keyField],
                    table: tableName
                })
            };
        },
        getFiltersFrom: function (data, query) {
            if (!(data instanceof Array) || !data.length)
                throwError("ej.SubQuery: Array of key values required");
            var key = query._fKey, value, prop = key, predicats = [],
                isValues = typeof data[0] !== "object";

            if (typeof data[0] !== "object") prop = null;

            for (var i = 0; i < data.length; i++) {
                value = !isValues ? ej.pvt.getObject(prop, data[i]) : data[i];
                predicats.push(new ej.Predicate(key, "==", value));
            }

            return ej.Predicate.or(predicats);
        }
    });
    ej.CacheAdaptor = new ej.UrlAdaptor().extend({
        init: function (adaptor, timeStamp, pageSize) {
            if (!ej.isNullOrUndefined(adaptor)) {
                this.cacheAdaptor = adaptor;
            }
            this.pageSize = pageSize;
            this.guidId = ej.getGuid("cacheAdaptor");
            var obj = { keys: [], results: [] };
            if (window.localStorage)
                window.localStorage.setItem(this.guidId, JSON.stringify(obj));
            var guid = this.guidId
            if (!ej.isNullOrUndefined(timeStamp)) {
                setInterval(function () {
                    var data = ej.parseJSON(window.localStorage.getItem(guid));
                    var forDel = [];
                    for (var i = 0; i < data.results.length; i++) {
                        data.results[i].timeStamp = new Date() - new Date(data.results[i].timeStamp)
                        if (new Date() - new Date(data.results[i].timeStamp) > timeStamp)
                            forDel.push(i);
                    }
                    var d = forDel;
                    for (var i = 0; i < forDel.length; i++) {
                        data.results.splice(forDel[i], 1);
                        data.keys.splice(forDel[i], 1);
                    }
                    window.localStorage.removeItem(guid)
                    window.localStorage.setItem(guid, JSON.stringify(data))
                }, timeStamp)
            }
        },
        generateKey: function (url, query) {
            var sorted = filterQueries(query.queries, "onSortBy"),
                grouped = filterQueries(query.queries, "onGroup"),
                filters = filterQueries(query.queries, "onWhere"),
                searchs = filterQueries(query.queries, "onSearch"),
				pageQuery = filterQueries(query.queries, "onPage"),
                singles = filterQueryLists(query.queries, ["onSelect", "onPage", "onSkip", "onTake", "onRange"]),
                params = query._params;
            var key = url;
            key += singles["onPage"].pageIndex;
            sorted.forEach(function (obj) {
                key += obj.e.direction + obj.e.fieldName;
            });
            grouped.forEach(function (obj) {
                key += obj.e.fieldName
            });
            searchs.forEach(function (obj) {
                key += obj.e.searchKey
            });
            for (var filter = 0; filter < filters.length; filter++) {
                var currentFilter = filters[filter];
                if (currentFilter.e.isComplex) {
                    var newQuery = query.clone()
                    newQuery.queries = [];
                    for (var i = 0; i < currentFilter.e.predicates.length; i++) {
                        newQuery.queries.push({ fn: "onWhere", e: currentFilter.e.predicates[i], filter: query.queries.filter })
                    }
                    key += currentFilter.e.condition + this.generateKey(url, newQuery)
                }
                else
                    key += currentFilter.e.field + currentFilter.e.operator + currentFilter.e.value
            }
            return key;
        },
        processQuery: function (dm, query, hierarchyFilters) {
            var key = this.generateKey(dm.dataSource.url, query);
            var cachedItems;
            if (window.localStorage)
                cachedItems = ej.parseJSON(window.localStorage.getItem(this.guidId));
            var data = cachedItems ? cachedItems.results[cachedItems.keys.indexOf(key)] : null;
            if (data != null) {
                return data;
            }
            return this.cacheAdaptor.processQuery.apply(this.cacheAdaptor, [].slice.call(arguments, 0))
        },
        processResponse: function (data, ds, query, xhr, request, changes) {
            var data = this.cacheAdaptor.processResponse.apply(this, [].slice.call(arguments, 0));
            var key = this.generateKey(ds.dataSource.url, query)
            var obj = {};
            if (window.localStorage)
                obj = ej.parseJSON(window.localStorage.getItem(this.guidId));
            obj.results[obj.keys.push(key) - 1] = { keys: key, result: data.result, timeStamp: new Date(), count: data.count }
            while (obj.results.length > this.pageSize) {
                obj.results.splice(0, 1)
                obj.keys.splice(0, 1)
            }
            window.localStorage.setItem(this.guidId, JSON.stringify(obj));
            return data;
        }
    });
    var filterQueries = function (queries, name) {
        return queries.filter(function (q) {
            return q.fn === name;
        }) || [];
    };
    var filterQueryLists = function (queries, singles) {
        var filtered = queries.filter(function (q) {
            return singles.indexOf(q.fn) !== -1;
        }), res = {};
        for (var i = 0; i < filtered.length; i++) {
            if (!res[filtered[i].fn])
                res[filtered[i].fn] = filtered[i].e;
        }
        return res;
    };
    var callAdaptorFunc = function (obj, fnName, param, param1) {
        if (obj[fnName]) {
            var res = obj[fnName](param, param1);
            if (!isNull(res)) param = res;
        }
        return param;
    };

    ej.ODataAdaptor = new ej.UrlAdaptor().extend({
        options: {
            requestType: "get",
            accept: "application/json;odata=light;q=1,application/json;odata=verbose;q=0.5",
            multipartAccept: "multipart/mixed",
            sortBy: "$orderby",
            select: "$select",
            skip: "$skip",
            take: "$top",
            count: "$inlinecount",
            where: "$filter",
            expand: "$expand",
            batch: "$batch",
            changeSet: "--changeset_",
            batchPre: "batch_",
            contentId: "Content-Id: ",
            batchContent: "Content-Type: multipart/mixed; boundary=",
            changeSetContent: "Content-Type: application/http\nContent-Transfer-Encoding: binary ",
            batchChangeSetContentType: "Content-Type: application/json; charset=utf-8 "
        },
        onEachWhere: function (filter, requiresCast) {
            return filter.isComplex ? this.onComplexPredicate(filter, requiresCast) : this.onPredicate(filter, requiresCast);
        },
        onPredicate: function (pred, query, requiresCast) {
            var returnValue = "",
                operator = pred.operator,
                val = pred.value,
                type = typeof val,
                field = this._p(pred.field);

            if (val instanceof Date)
                val = "datetime'" + dateSerialize(val) + "'";

            if (type === "string") {
                val = "'" + val + "'";

                if (requiresCast) {
                    field = "cast(" + field + ", 'Edm.String')";
                }

                if (pred.ignoreCase) {
                    field = "tolower(" + field + ")";
                    val = val.toLowerCase();
                }
            }

            operator = ej.data.odBiOperator[pred.operator];
            if (operator) {
                returnValue += field;
                returnValue += operator;

                return returnValue + val;
            }

            operator = ej.data.odUniOperator[pred.operator];
            if (!operator || type !== "string") return "";

            if (operator === "substringof") {
                var t = val;
                val = field;
                field = t;
            }

            returnValue += operator + "(";
            returnValue += field + ",";
            returnValue += val + ")";

            return returnValue;
        },
        onComplexPredicate: function (pred, requiresCast) {
            var res = [];
            for (var i = 0; i < pred.predicates.length; i++) {
                res.push("(" + this.onEachWhere(pred.predicates[i], requiresCast) + ")");
            }
            return res.join(" " + pred.condition + " ");
        },
        onWhere: function (filters) {
            if (this.pvt.searches)
                filters.push(this.onEachWhere(this.pvt.searches, null, true));

            return filters.join(" and ");
        },
        onEachSearch: function (e) {
            if (e.fields.length === 0)
                throwError("Query() - Search : oData search requires list of field names to search");

            var filter = this.pvt.searches || [];
            for (var i = 0; i < e.fields.length; i++) {
                filter.push(new ej.Predicate(e.fields[i], e.operator, e.key, e.ignoreCase));
            }
            this.pvt.searches = filter;
        },
        onSearch: function (e) {
            this.pvt.searches = ej.Predicate.or(this.pvt.searches);
            return "";
        },
        onEachSort: function (e) {
            var res = [];
            if (e.name instanceof Array) {
                for (var i = 0; i < e.name.length; i++)
                    res.push(this._p(e.name[i]));
            } else
                res.push(this._p(e.name) + (e.direction === "descending" ? " desc" : ""));

            return res.join(",");
        },
        onSortBy: function (e) {
            return e.reverse().join(",");
        },
        onGroup: function (e) {
            this.pvt.groups = e;
            return "";
        },
        onSelect: function (e) {
            for (var i = 0; i < e.length; i++)
                e[i] = this._p(e[i]);

            return e.join(',');
        },
        onCount: function (e) {
            return e === true ? "allpages" : "";
        },
        beforeSend: function (dm, request, settings) {
            if (settings.url.endsWith(this.options.batch) && settings.type.toLowerCase() === "post") {
                request.setRequestHeader("Accept", oData.multipartAccept);
                request.setRequestHeader("DataServiceVersion", "2.0");
                request.overrideMimeType("text/plain; charset=x-user-defined");
            }

            if (!dm.dataSource.crossDomain) {
                request.setRequestHeader("DataServiceVersion", "2.0");
                request.setRequestHeader("MaxDataServiceVersion", "2.0");
            }
        },
        processResponse: function (data, ds, query, xhr, request, changes) {
            var pvt = request && request.ejPvtData;
            if (request && this.options.batch && request.url.endsWith(this.options.batch) && request.type.toLowerCase() === "post") {
                var guid = xhr.getResponseHeader("Content-Type"), cIdx, jsonObj;
                guid = guid.substring(guid.indexOf("=batchresponse") + 1);
                data = data.split(guid);
                if (data.length < 2) return;

                data = data[1];
                var exVal = /(?:\bContent-Type.+boundary=)(changesetresponse.+)/i.exec(data);
                data.replace(exVal[0], "");

                var changeGuid = exVal[1];
                data = data.split(changeGuid);

                for (var i = data.length; i > -1; i--) {
                    if (!/\bContent-ID:/i.test(data[i]) || !/\bHTTP.+201/.test(data[i]))
                        continue;

                    cIdx = parseInt(/\bContent-ID: (\d+)/i.exec(data[i])[1]);

                    if (changes.added[cIdx]) {
                        jsonObj = p.parseJson(/^\{.+\}/m.exec(data[i])[0]);
                        $.extend(changes.added[cIdx], this.processResponse(jsonObj));
                    }
                }
                return changes;
            }
            var version = xhr && xhr.getResponseHeader("DataServiceVersion"), count = null;
            version = (version && parseInt(version, 10)) || 2;

            if (query && query._requiresCount) {
                if (data.__count || data['odata.count']) count = data.__count || data['odata.count'];
                if (data.d) data = data.d;
                if (data.__count || data['odata.count']) count = data.__count || data['odata.count'];
            }

            if (version === 3 && data.value) data = data.value;
            if (data.d) data = data.d;
            if (version < 3 && data.results) data = data.results;

            if (pvt && pvt.groups && pvt.groups.length) {
                var groups = pvt.groups;
                for (var i = 0; i < groups.length; i++)
                    data = ej.group(data, groups[i]);
            }
            return isNull(count) ? data : { result: data, count: count };
        },
        convertToQueryString: function (req, query, dm) {
            var res = [], tableName = req.table || "";
            delete req.table;

            if (dm.dataSource.requiresFormat)
                req["$format"] = "json";

            for (var prop in req)
                res.push(prop + "=" + req[prop]);

            res = res.join("&");

            if (dm.dataSource.url && dm.dataSource.url.indexOf("?") !== -1 && !tableName)
                return res;

            return res.length ? tableName + "?" + res : tableName || "";
        },
        insert: function (dm, data, tableName) {
            return {
                url: dm.dataSource.url.replace(/\/*$/, tableName ? '/' + tableName : ''),
                data: JSON.stringify(data)
            }
        },
        remove: function (dm, keyField, value, tableName) {
            return {
                type: "DELETE",
                url: dm.dataSource.url.replace(/\/*$/, tableName ? '/' + tableName : '') + '(' + value + ')'
            };
        },
        update: function (dm, keyField, value, tableName) {
            return {
                type: "PUT",
                url: dm.dataSource.url.replace(/\/*$/, tableName ? '/' + tableName : '') + '(' + value[keyField] + ')',
                data: JSON.stringify(value),
                accept: this.options.accept
            };
        },
        batchRequest: function (dm, changes, e) {
            var initialGuid = e.guid = ej.getGuid(oData.batchPre);
            var url = dm.dataSource.url.replace(/\/*$/, '/' + this.options.batch);
            var args = {
                url: e.url,
                key: e.key,
                cid: 1,
                cSet: ej.getGuid(oData.changeSet)
            };
            var req = "--" + initialGuid + "\n";

            req += "Content-Type: multipart/mixed; boundary=" + args.cSet.replace("--", "") + "\n";

            req += this.generateInsertRequest(changes.added, args);
            req += this.generateUpdateRequest(changes.changed, args);
            req += this.generateDeleteRequest(changes.deleted, args);

            req += args.cSet + "--\n";
            req += "--" + initialGuid + "--";

            return {
                type: "POST",
                url: url,
                contentType: "multipart/mixed; charset=UTF-8;boundary=" + initialGuid,
                data: req
            };
        },
        generateDeleteRequest: function (arr, e) {
            if (!arr) return "";
            var req = "";

            for (var i = 0; i < arr.length; i++) {
                req += "\n" + e.cSet + "\n";
                req += oData.changeSetContent + "\n\n";
                req += "DELETE ";
                req += e.url + "(" + arr[i][e.key] + ") HTTP/1.1\n";
                req += "Accept: " + oData.accept + "\n";
                req += oData.batchChangeSetContentType + "\n";
            }

            return req + "\n";
        },
        generateInsertRequest: function (arr, e) {
            if (!arr) return "";
            var req = "";

            for (var i = 0; i < arr.length; i++) {
                req += "\n" + e.cSet + "\n";
                req += oData.changeSetContent + "\n\n";
                req += "POST ";
                req += e.url + " HTTP/1.1\n";
                req += "Accept: " + oData.accept + "\n";
                req += "Content-Id: " + i + "\n";
                req += oData.batchChangeSetContentType + "\n\n";

                req += JSON.stringify(arr[i]) + "\n";
            }

            return req;
        },
        generateUpdateRequest: function (arr, e) {
            if (!arr) return "";
            var req = "";

            for (var i = 0; i < arr.length; i++) {
                req += "\n" + e.cSet + "\n";
                req += oData.changeSetContent + "\n\n";
                req += "PUT ";
                req += e.url + "(" + arr[i][e.key] + ")" + " HTTP/1.1\n";
                req += "Accept: " + oData.accept + "\n";
                req += oData.batchChangeSetContentType + "\n\n";

                req += JSON.stringify(arr[i]) + "\n\n";
            }

            return req;
        },
        _p: function (prop) {
            return prop.replace(/\./g, "/");
        }
    });

    ej.JsonAdaptor = new ej.Adaptor().extend({
        processQuery: function (ds, query) {
            var result = ds.dataSource.json.slice(0), count = result.length, cntFlg = true, ret, key;

            for (var i = 0; i < query.queries.length; i++) {
                key = query.queries[i];
                ret = this[key.fn].call(this, result, key.e, query);
                result = ret !== undefined ? ret : result;

                if (key.fn === "onPage" || key.fn === "onSkip" || key.fn === "onTake" || key.fn === "onRange") cntFlg = false;

                if (cntFlg) count = result.length;
            }

            if (query._requiresCount) {
                result = {
                    result: result,
                    count: count
                };
            }

            return result;
        },
        batchRequest: function (dm, changes, e) {
            for (var i = 0; i < changes.added.length; i++)
                this.insert(dm, changes.added[i]);
            for (var i = 0; i < changes.changed.length; i++)
                this.update(dm, e.key, changes.changed[i]);
            for (var i = 0; i < changes.deleted.length; i++)
                this.remove(dm, e.key, changes.deleted[i]);
            return changes;
        },
        onWhere: function (ds, e) {
            if (!ds) return ds;

            return ds.filter(function (obj) {
                return e.validate(obj);
            });
        },
        onSearch: function (ds, e) {
            if (!ds || !ds.length) return ds;

            if (e.fieldNames.length === 0) {
                ej.pvt.getFieldList(ds[0], e.fieldNames);
            }

            return ds.filter(function (obj) {
                for (var j = 0; j < e.fieldNames.length; j++) {
                    if (e.comparer.call(obj, ej.pvt.getObject(e.fieldNames[j], obj), e.searchKey, e.ignoreCase))
                        return true;
                }
                return false;
            });
        },
        onSortBy: function (ds, e, query) {
            if (!ds) return ds;
            var fnCompare, field = getValue(e.fieldName, query);
            if (!field)
                return ds.sort(e.comparer);

            if (field instanceof Array) {
                field = field.slice(0);

                for (var i = field.length - 1; i >= 0; i--) {
                    if (!field[i]) continue;

                    fnCompare = e.comparer;

                    if (field[i].endsWith(" desc")) {
                        fnCompare = ej.pvt.fnSort(ej.sortOrder.Descending);
                        field[i] = field[i].replace(" desc", "");
                    }

                    ds = stableSort(ds, field[i], fnCompare);
                }
                return ds;
            }
            return stableSort(ds, field, e.comparer);
        },
        onGroup: function (ds, e, query) {
            if (!ds) return ds;

            return ej.group(ds, getValue(e.fieldName, query));
        },
        onPage: function (ds, e, query) {
            var size = getValue(e.pageSize, query),
                start = (getValue(e.pageIndex, query) - 1) * size, end = start + size;

            if (!ds) return ds;

            return ds.slice(start, end);
        },
        onRange: function (ds, e) {
            if (!ds) return ds;
            return ds.slice(getValue(e.start), getValue(e.end));
        },
        onTake: function (ds, e) {
            if (!ds) return ds;

            return ds.slice(0, getValue(e.nos));
        },
        onSkip: function (ds, e) {
            if (!ds) return ds;
            return ds.slice(getValue(e.nos));
        },
        onSelect: function (ds, e) {
            if (!ds) return ds;
            return ej.select(ds, getValue(e.fieldNames));
        },
        insert: function (dm, data) {
            return dm.dataSource.json.push(data);
        },
        remove: function (dm, keyField, value, tableName) {
            var ds = dm.dataSource.json, i;
            if (typeof value === "object")
                value = value[keyField];
            for (i = 0; i < ds.length; i++) {
                if (ds[i][keyField] === value) break;
            }

            return i !== ds.length ? ds.splice(i, 1) : null;
        },
        update: function (dm, keyField, value, tableName) {
            var ds = dm.dataSource.json, i, key = value[keyField];

            for (i = 0; i < ds.length; i++) {
                if (ds[i][keyField] === key) break;
            }

            return i < ds.length ? $.extend(ds[i], value) : null;
        }
    });
    ej.remoteSaveAdaptor = new ej.JsonAdaptor().extend({
        beforeSend: ej.UrlAdaptor.prototype.beforeSend,
        insert: ej.UrlAdaptor.prototype.insert,
        update: ej.UrlAdaptor.prototype.update,
        remove: ej.UrlAdaptor.prototype.remove,
        batchRequest: function (dm, changes, e) {
            for (var i = 0; i < changes.added.length; i++)
                ej.JsonAdaptor.prototype.insert(dm, changes.added[i]);
            for (var i = 0; i < changes.changed.length; i++)
                ej.JsonAdaptor.prototype.update(dm, e.key, changes.changed[i]);
            for (var i = 0; i < changes.deleted.length; i++)
                ej.JsonAdaptor.prototype.remove(dm, e.key, changes.deleted[i]);
            return {
                type: "POST",
                url: dm.dataSource.batchUrl || dm.dataSource.crudUrl || dm.dataSource.url,
                contentType: "application/json; charset=utf-8",
                dataType: "json",
                data: JSON.stringify({
                    changed: changes.changed,
                    added: changes.added,
                    deleted: changes.deleted,
                    action: "batch",
                    table: e.url,
                    key: e.key
                })
            };
        }
    });
    ej.WebApiAdaptor = new ej.ODataAdaptor().extend({
        insert: function (dm, data, tableName) {
            return {
                type: "POST",
                url: dm.dataSource.url,
                data: JSON.stringify(data)
            };
        },
        remove: function (dm, keyField, value, tableName) {
            return {
                type: "DELETE",
                url: dm.dataSource.url + "/" + value,
                data: JSON.stringify(value)
            };
        },
        update: function (dm, keyField, value, tableName) {
            return {
                type: "PUT",
                url: dm.dataSource.url,
                data: JSON.stringify(value)
            };
        },
        processResponse: function (data, ds, query, xhr, request, changes) {

            var pvt = request && request.ejPvtData;
            if (request && request.type.toLowerCase() != "post") {
                var version = xhr && xhr.getResponseHeader("DataServiceVersion"), count = null;
                version = (version && parseInt(version, 10)) || 2;

                if (query && query._requiresCount) {
                    if (data.Count) count = data.Count;
                }

                if (version < 3 && data.Items) data = data.Items;

                if (pvt && pvt.groups && pvt.groups.length) {
                    var groups = pvt.groups;
                    for (var i = 0; i < groups.length; i++)
                        data = ej.group(data, groups[i]);
                }
                return isNull(count) ? data : { result: data, count: count };
            }
        }
    });
    var getValue = function (value, inst) {
        if (typeof value === "function")
            return value.call(inst || {});
        return value;
    }

    ej.TableModel = function (name, jsonArray, dataManager, modelComputed) {
        if (!instance(this, ej.TableModel))
            return new ej.TableModel(jsonArray);

        if (!instance(jsonArray, Array))
            throwError("ej.TableModel - Json Array is required");

        var rows = [], model, dirtyFn = $proxy(setDirty, this);

        for (var i = 0; i < jsonArray.length; i++) {
            model = new ej.Model(jsonArray[i], this);
            model.state = "unchanged";
            model.on("stateChange", dirtyFn);
            if (modelComputed)
                model.computes(modelComputed);
            rows.push(model);
        }

        this.name = name || "table1";

        this.rows = ej.NotifierArray(rows);
        this._deleted = [];

        this._events = $({});

        this.dataManager = dataManager;

        this._isDirty = false;

        return this;
    };

    ej.TableModel.prototype = {
        on: function (eventName, handler) {
            this._events.on(eventName, handler);
        },

        off: function (eventName, handler) {
            this._events.off(eventName, handler);
        },

        setDataManager: function (dataManager) {
            this.dataManagar = dataManager;
        },

        saveChanges: function () {
            if (!this.dataManager || !instance(this.dataManager, ej.DataManager))
                throwError("ej.TableModel - saveChanges : Set the dataManager using setDataManager function");

            if (!this.isDirty())
                return;

            var promise = this.dataManager.saveChanges(this.getChanges(), this.key, this.name);

            promise.done($proxy(function (changes) {
                var rows = this.toArray();
                for (var i = 0; i < rows.length; i++) {
                    if (rows.state === "added") {
                        rows.set(this.key, changes.added.filter(function (e) {
                            return e[this.key] === rows.get(this.key);
                        })[0][this.key]);
                    }
                    rows[i].markCommit();
                }

                this._events.triggerHandler({ type: "save", table: this });

            }, this));

            promise.fail($proxy(function (e) {
                this.rejectChanges();
                this._events.triggerHandler({ type: "reject", table: this, error: e });
            }, this));

            this._isDirty = false;
        },

        rejectChanges: function () {
            var rows = this.toArray();
            for (var i = 0; i < rows.length; i++)
                rows[i].revert(true);

            this._isDirty = false;
            this._events.triggerHandler({ type: "reject", table: this });
        },

        insert: function (json) {
            var model = new ej.Model(json);
            model._isDirty = this._isDirty = true;

            this.rows.push(model);

            this._events.triggerHandler({ type: "insert", model: model, table: this });
        },

        update: function (value) {
            if (!this.key)
                throwError("TableModel - update : Primary key should be assigned to TableModel.key");

            var row = value, model, key = this.key, keyValue = row[key];

            model = this.rows.array.filter(function (obj) {
                return obj.get(key) === keyValue;
            });

            model = model[0];

            for (var col in row) {
                model.set(col, row[col]);
            }

            this._isDirty = true;

            this._events.triggerHandler({ type: "update", model: model, table: this });
        },

        remove: function (key) {
            if (!this.key)
                throwError("TableModel - update : Primary key should be assigned to TableModel.key");

            var field = this.key;

            var index = -1, model;

            if (key && typeof key === "object") {
                key = key[field] !== undefined ? key[field] : key.get(field);
            }

            for (var i = 0; i < this.rows.length() ; i++) {
                if (this.rows.array[i].get(field) === key) {
                    index = i;
                    break;
                }
            }

            if (index > -1) {
                model = this.rows.removeAt(index);
                model.markDelete();

                this._deleted.push({ model: model, position: index });

                this._isDirty = true;
                this._events.triggerHandler({ type: "remove", model: model, table: this });
            }
        },

        isDirty: function () {
            return this._isDirty;
        },

        getChanges: function () {

            var changes = {
                added: [],
                changed: []
            };
            var rows = this.toArray();
            for (var i = 0; i < rows.length; i++) {
                if (changes[rows[i].state])
                    changes[rows[i].state].push(rows[i].json);
            }

            changes.deleted = ej.select(this._deleted, ["model"]);

            return changes;
        },

        toArray: function () {
            return this.rows.toArray();
        },

        setDirty: function (dirty, model) {
            if (this._isDirty === !!dirty) return;

            this._isDirty = !!dirty;

            this._events.triggerHandler({ type: "dirty", table: this, model: model });
        },
        get: function (index) {
            return this.rows.array[index];
        },
        length: function () {
            return this.rows.array.length;
        },

        bindTo: function (element) {
            var marker = tDiv, template = $(element.html()), rows = this.toArray(), cur;
            if ($.inArray(element.prop("tagName").toLowerCase(), ["table", "tbody"]))
                marker = tTR;

            marker.insertBefore(element);
            element.detach().empty();

            for (var i = 0; i < rows.length; i++) {
                cur = template.clone();
                rows[i].bindTo(cur);
                element.append(cur);
            }

            element.insertAfter(marker);
            marker.remove();
        }
    };

    var tDiv = doc ? $(document.createElement("div")) : {},
        tTR = doc ? $(document.createElement("tr")) : {};

    ej.Model = function (json, table, name) {
        if (typeof table === "string") {
            name = table;
            table = null;
        }
        this.$id = getUid("m");

        this.json = json;
        this.table = table instanceof ej.TableModel ? table : null;
        this.name = name || (this.table && this.table.name);
        this.dataManager = (table instanceof ej.DataManager) ? table : table.dataManagar;
        this.actual = {};
        this._events = $({});
        this.isDirty = false;
        this.state = "added";
        this._props = [];
        this._computeEls = {};
        this._fields = {};
        this._attrEls = {};
        this._updates = {};
        this.computed = {};
    };

    ej.Model.prototype = {
        computes: function (value) {
            $.extend(this.computed, value);
        },
        on: function (eventName, handler) {
            this._events.on(eventName, handler);
        },
        off: function (eventName, handler) {
            this._events.off(eventName, handler);
        },
        set: function (field, value) {
            var obj = this.json, actual = field, prev;
            field = field.split('.');

            for (var i = 0; i < field.length - 1; i++) {
                field = field[0];
                obj = obj[field[0]];
            }

            this.isDirty = true;
            this.changeState("changed", { from: "set" });

            prev = obj[field];

            if (this.actual[field] === undefined && !(field in this.actual))
                this.actual[field] = value; // Complex property ?

            obj[field] = value;

            this._updateValues(field, value);
            this._events.triggerHandler({ type: actual, current: value, previous: prev, model: this });
        },
        get: function (field) {
            return ej.pvt.getObject(field, this.json);
        },
        revert: function (suspendEvent) {
            for (var prop in this.actual) {
                this.json[prop] = this.actual[prop];
            }

            this.isDirty = false;

            if (suspendEvent)
                this.state = "unchanged";
            else
                this.changeState("unchanged", { from: "revert" });
        },
        save: function (dm, key) {
            dm = dm || this.dataManagar;
            key = key || dm.dataSource.key;
            if (!dm) throwError("ej.Model - DataManager is required to commit the changes");
            if (this.state === "added") {
                return dm.insert(this.json, this.name).done(ej.proxy(function (e) {
                    $.extend(this.json, e.record);
                }, this));
            }
            else if (this.state === "changed") {
                return dm.update(key, this.json, this.name);
            }
            else if (this.state === "deleted") {
                return dm.remove(key, this.json, this.name);
            }
        },
        markCommit: function () {
            this.isDirty = false;
            this.changeState("unchanged", { from: "commit" });
        },
        markDelete: function () {
            this.changeState("deleted", { from: "delete" });
        },
        changeState: function (state, args) {
            if (this.state === state) return;

            if (this.state === "added") {
                if (state === "deleted")
                    state = "unchanged";
                else return;
            }

            var prev = state;
            args = args || {};

            this.state = state;
            this._events.triggerHandler($.extend({ type: "stateChange", current: state, previous: prev, model: this }, args));
        },
        properties: function () {
            if (this._props.length)
                return this._props;

            for (var pr in this.json) {
                this._props.push(pr);
                this._updates[pr] = { read: [], input: [] };
            }

            return this._props;
        },
        bindTo: function (element) {
            var props = this.properties(), el = $(element), ctl, field,
                elements = el.find("[ej-observe], [ej-computed], [ej-prop]"), len = elements.length;

            el.data("ejModel", this);
            var unbindData = { fields: [], props: [], computes: [] };
            for (var i = 0; i < len; i++) {
                ctl = elements.eq(i);

                field = ctl.attr("ej-prop")
                if (field) {
                    this._processAttrib(field, ctl, unbindData);
                }
                field = ctl.attr("ej-observe");
                if (field && this._props.indexOf(field) !== -1) {
                    this._processField(ctl, field, unbindData);
                    continue;
                }

                field = ctl.attr("ej-computed");
                if (field) {
                    this._processComputed(field, ctl, unbindData);
                    continue;
                }
            }
            el.data("ejModelBinding" + this.$id, unbindData);
        },
        unbind: function (element) {
            var props = this.properties(), tmp, data = {
                props: this._attrEls,
                computes: this._computeEls
            }, isCustom = false;

            if (element) {
                data = $(element).removeData("ejModel").data("ejModelBinding" + this.$id) || data;
                isCustom = true;
            }

            for (var p in this.computed) {
                tmp = data.computes[p], p = this.computed[p];
                if (tmp && p.deps) {
                    this.off(p.deps.join(' '), tmp.handle);
                    if (isCustom)
                        delete this._computeEls[p];
                }
            }
            if (!isCustom)
                this._computeEls = {};

            for (var p in data.props) {
                tmp = data.props[p];
                if (tmp) {
                    this.off(tmp.deps.join(' '), tmp.handle);
                    delete data.props[p];
                    if (isCustom)
                        delete this._attrEls[p];
                }
            }
            if (!isCustom)
                this._attrEls = {};

            if (data.fields && data.fields.length) {
                var len = data.fields.length, ctl, idx, ty;
                for (var i = 0; i < len; i++) {
                    ctl = data.fields[i];
                    $(ctl).off("change", null, this._changeHandler);

                    ty = this.formElements.indexOf(ctl.tagName.toLowerCase()) !== -1 ? "input" : "read";
                    idx = this._updates[ty].indexOf(ctl);
                    if (idx !== -1)
                        this._updates[ty].splice(idx, 1);
                }
            }
        },
        _processComputed: function (value, element, data) {
            if (!value) return;

            var val, deps = [], safeVal = safeStr(value),
            type = this.formElements.indexOf(element[0].tagName.toLowerCase()) !== -1 ? "val" : "html";

            if (!this.computed[value] || !this.computed[safeVal]) {
                this.computed[safeVal] = {
                    value: new Function("var e = this; return " + value),
                    deps: this._generateDeps(value)
                }
                value = safeVal;
            }

            val = this.computed[value];
            if (!val.get) {
                val.get = function () {
                    val.value.call(this.json);
                }
            }

            deps = val.deps;
            val = val.value;

            this._updateDeps(deps);
            this._updateElement(element, type, val);

            val = { el: element, handle: $proxy(this._computeHandle, this, { value: value, type: type }) };
            this._computeEls[value] = val;
            data.computes[value] = val;

            this.on(deps.join(' '), val.handle);
        },
        _computeHandle: function (e) {
            var el = this._computeEls[e.value];
            if (el && this.computed[e.value])
                this._updateElement(el.el, e.type, this.computed[e.value].value);
        },
        _updateElement: function (el, type, val) {
            el[type](val.call($.extend({}, this.json, this.computed)));
        },
        _updateDeps: function (deps) {
            for (var i = 0; i < deps.length; i++) {
                if (!(deps[i] in this.json) && deps[i] in this.computed)
                    ej.merge(deps, this.computed[deps[i]].deps);
            }
        },
        _generateDeps: function (value) {
            var splits = value.replace(/(^e\.)|( e\.)/g, '#%^*##ej.#').split("#%^*#"),
                field, deps = [];

            for (var i = 0; i < splits.length; i++) {
                if (splits[i].startsWith("#ej.#")) {
                    field = splits[i].replace("#ej.#", "").split(' ')[0];
                    if (field && this._props.indexOf(field) !== -1)
                        deps.push(field);
                }
            }

            return deps;
        },
        _processAttrib: function (value, el, data) {
            var prop, val, res = {};
            value = value.replace(/^ +| +$/g, "").split(";");
            for (var i = 0; i < value.length; i++) {
                value[i] = value[i].split(":");
                if (value[i].length < 2) continue;

                prop = value[i][0].replace(/^ +| +$/g, "").replace(/^'|^"|'$|"$/g, "");
                res[prop] = value[i][1].replace(/^ +| +$/g, "").replace(/^'|^"|'$|"$/g, "");
            }
            value = res;
            var deps = [];
            for (var prop in value)
                deps.push(value[prop]);

            this._updateDeps(deps);
            this._updateProps(el, value);

            res = getUid("emak");
            val = { el: el, handle: $proxy(this._attrHandle, this, res), value: value, deps: deps };
            el.prop("ejmodelattrkey", res);

            data.props[res] = val;
            this._attrEls[res] = val;

            this.on(deps.join(' '), val.handle);
        },
        _attrHandle: function (res) {
            var el = this._attrEls[res];
            if (el)
                this._updateProps(el.el, el.value);
        },
        _updateProps: function (element, value) {
            var json = this.json, t, c = this.computed;
            for (var prop in value) {
                t = value[prop];
                if (t in json)
                    t = json[t];
                else if (t in this.computed) {
                    t = this.computed[t];
                    if (t) {
                        t = t.value.call($.extend({}, this.json, this.computed));
                    }
                }

                if (!isNull(t)) {
                    element.prop(prop, t);
                }
            }
        },
        _updateValues: function (prop, value) {
            var arr = this._updates[prop];

            if (!arr || (!arr.read && !arr.input)) return;

            this._ensureItems(arr.read, "html", value);
            this._ensureItems(arr.input, "val", value);
        },
        _ensureItems: function (a, type, value) {
            if (!a) return;

            for (var i = a.length - 1; i > -1; i--) {
                if (!a[i].offsetParent) {
                    a.splice(i, 1);
                    continue;
                }
                $(a[i])[type](value);
            }
        },
        _changeHandler: function (e) {
            e.data.self.set(e.data.prop, $(this).val());
        },
        _processField: function (ctl, field, data) {
            var e = { self: this, prop: field }, val = this.get(field);

            data.fields.push(ctl[0]);

            if (this.formElements.indexOf(ctl[0].tagName.toLowerCase()) === -1) {
                ctl.html(val);
                return this._updates[field].read.push(ctl[0]);
            }

            ctl.val(val)
                    .off("change", null, this._changeHandler)
                    .on("change", null, e, this._changeHandler);

            return this._updates[field].input.push(ctl[0]);
        },
        formElements: ["input", "select", "textarea"]
    };

    var safeReg = /[^\w]+/g;
    var safeStr = function (value) {
        return value.replace(safeReg, "_");
    };
    var setDirty = function (e) {
        this.setDirty(true, e.model);
    };

    ej.Predicate = function (field, operator, value, ignoreCase) {
        if (!(this instanceof ej.Predicate))
            return new ej.Predicate(field, operator, value, ignoreCase);

        if (typeof field === "string") {
            this.field = field;
            this.operator = operator;
            this.value = value;
            this.ignoreCase = ignoreCase;
            this.isComplex = false;

            this._comparer = ej.data.fnOperators.processOperator(this.operator);

        } else if (field instanceof ej.Predicate && value instanceof ej.Predicate || value instanceof Array) {
            this.isComplex = true;
            this.condition = operator.toLowerCase();
            this.predicates = [field];
            if (value instanceof Array)
                [].push.apply(this.predicates, value);
            else
                this.predicates.push(value);
        }
        return this;
    };

    ej.Predicate.and = function () {
        return pvtPredicate._combinePredicates([].slice.call(arguments, 0), "and");
    };

    ej.Predicate.or = function () {
        return pvtPredicate._combinePredicates([].slice.call(arguments, 0), "or");
    };

    // Private fn
    var pvtPredicate = {
        _combinePredicates: function (predicates, operator) {
            if (!predicates.length) return undefined;
            if (predicates.length === 1) {
                if (!instance(predicates[0], Array))
                    return predicates[0];
                predicates = predicates[0];
            }
            return new ej.Predicate(predicates[0], operator, predicates.slice(1));;
        },

        _combine: function (pred, field, operator, value, condition, ignoreCase) {
            if (field instanceof ej.Predicate)
                return ej.Predicate[condition](pred, field);

            if (typeof field === "string")
                return ej.Predicate[condition](pred, new ej.Predicate(field, operator, value, ignoreCase));

            return throwError("Predicate - " + condition + " : invalid arguments");
        }
    };

    ej.Predicate.prototype = {
        and: function (field, operator, value, ignoreCase) {
            return pvtPredicate._combine(this, field, operator, value, "and", ignoreCase);
        },
        or: function (field, operator, value, ignoreCase) {
            return pvtPredicate._combine(this, field, operator, value, "or", ignoreCase);
        },
        validate: function (record) {
            var value, p = this.predicates, isAnd, ret;

            if (!this.isComplex) {
                return this._comparer.call(this, ej.pvt.getObject(this.field, record), this.value, this.ignoreCase);
            }

            isAnd = this.condition === "and";

            for (var i = 0; i < p.length; i++) {
                ret = p[i].validate(record);
                if (isAnd) {
                    if (!ret) return false;
                } else {
                    if (ret) return true;
                }
            }

            return isAnd;
        },
        toJSON: function () {
            var predicates, p;
            if (this.isComplex) {
                predicates = [], p = this.predicates;
                for (var i = 0; i < p.length; i++)
                    predicates.push(p[i].toJSON());
            }
            return {
                isComplex: this.isComplex,
                field: this.field,
                operator: this.operator,
                value: this.value,
                ignoreCase: this.ignoreCase,
                condition: this.condition,
                predicates: predicates
            }
        }
    };

    ej.dataUtil = {
        swap: function (array, x, y) {
            if (x == y) return;

            var tmp = array[x];
            array[x] = array[y];
            array[y] = tmp;
        },

        mergeSort: function (jsonArray, fieldName, comparer) {
            if (!comparer || typeof comparer === "string")
                comparer = ej.pvt.fnSort(comparer, true);

            if (typeof fieldName === "function") {
                comparer = fieldName;
                fieldName = null;
            }

            return ej.pvt.mergeSort(jsonArray, fieldName, comparer);
        },

        max: function (jsonArray, fieldName, comparer) {
            if (typeof fieldName === "function") {
                comparer = fieldName;
                fieldName = null;
            }

            return ej.pvt.getItemFromComparer(jsonArray, fieldName, comparer || ej.pvt.fnDescending);
        },

        min: function (jsonArray, fieldName, comparer) {
            if (typeof fieldName === "function") {
                comparer = fieldName;
                fieldName = null;
            }

            return ej.pvt.getItemFromComparer(jsonArray, fieldName, comparer || ej.pvt.fnAscending);
        },

        distinct: function (json, fieldName, requiresCompleteRecord) {
            var result = [], val, tmp = {};
            for (var i = 0; i < json.length; i++) {
                val = getVal(json, fieldName, i);
                if (!(val in tmp)) {
                    result.push(!requiresCompleteRecord ? val : json[i]);
                    tmp[val] = 1;
                }
            }
            return result;
        },

        sum: function (json, fieldName) {
            var result = 0, val, castRequired = typeof getVal(json, fieldName, 0) !== "number";

            for (var i = 0; i < json.length; i++) {
                val = getVal(json, fieldName, i);
                if (val !== undefined){
                   if(castRequired && +val)
                       val = +val;
                   result += val;
                }
            }
            return result;
        },

        avg: function (json, fieldName) {
            return ej.sum(json, fieldName) / json.length;
        },

        select: function (jsonArray, fields) {
            var newData = [];

            for (var i = 0; i < jsonArray.length; i++) {
                newData.push(ej.pvt.extractFields(jsonArray[i], fields));
            }

            return newData;
        },

        group: function (jsonArray, field, /* internal */ level) {
            level = level || 1;

            if (jsonArray.GROUPGUID == ej.pvt.consts.GROUPGUID) {
                for (var j = 0; j < jsonArray.length; j++) {
                    jsonArray[j].items = ej.group(jsonArray[j].items, field, jsonArray.level + 1);
                    jsonArray[j].count = jsonArray[j].items.length;
                }

                jsonArray.childLevels += 1;
                return jsonArray;
            }

            var grouped = {}, groupedArray = [];

            groupedArray.GROUPGUID = ej.pvt.consts.GROUPGUID;
            groupedArray.level = level;
            groupedArray.childLevels = 0;
            groupedArray.records = jsonArray;

            for (var i = 0; i < jsonArray.length; i++) {
                var val = getVal(jsonArray, field, i);

                if (!grouped[val]) {
                    grouped[val] = {
                        key: val,
                        count: 0,
                        items: [],
                        field: field
                    };
                    groupedArray.push(grouped[val]);
                }

                grouped[val].count += 1;
                grouped[val].items.push(jsonArray[i]);
            }

            return groupedArray;
        },

        parseTable: function (table, headerOption, headerRowIndex) {
            var tr = table.rows, headerRow, headerTds = [], data = [], i;

            if (!tr.length) return [];

            headerRowIndex = headerRowIndex || 0;

            switch ((headerOption || "").toLowerCase()) {
                case ej.headerOption.tHead:
                    headerRow = table.tHead.rows[headerRowIndex];
                    break;
                case ej.headerOption.row:
                default:
                    headerRow = table.rows[headerRowIndex];
                    break;
            }

            var hTd = headerRow.cells;

            for (i = 0; i < hTd.length; i++)
                headerTds.push($.trim(hTd[i].innerHTML));

            for (i = headerRowIndex + 1; i < tr.length; i++) {
                var json = {}, td = tr[i].cells;
                for (var j = 0; j < td.length; j++) {
                    json[headerTds[j]] = td[j].innerHTML;
                }
                data.push(json);
            }
            return data;
        }
    };

    ej.headerOption = {
        tHead: "thead",
        row: "row"
    };

    ej.pvt = {
        filterQueries: filterQueries,
        mergeSort: function (jsonArray, fieldName, comparer) {
            if (jsonArray.length <= 1)
                return jsonArray;

            // else list size is > 1, so split the list into two sublists
            var middle = parseInt(jsonArray.length / 2, 10);

            var left = jsonArray.slice(0, middle),
                right = jsonArray.slice(middle);

            left = ej.pvt.mergeSort(left, fieldName, comparer);
            right = ej.pvt.mergeSort(right, fieldName, comparer);

            return ej.pvt.merge(left, right, fieldName, comparer);
        },

        getItemFromComparer: function (array, field, comparer) {
            var keyVal = getVal(array, field, 0), current, key = array[0];
            for (var i = 1; i < array.length; i++) {
                current = getVal(array, field, i);
                if (comparer(keyVal, current) > 0) {
                    keyVal = current;
                    key = array[i];
                }
            }
            return key;
        },

        quickSelect: function (array, fieldName, left, right, k, comparer) {
            if (left == right)
                return array[left];

            var pivotNewIndex = ej.pvt.partition(array, fieldName, left, right, comparer);

            var pivotDist = pivotNewIndex - left + 1;

            if (pivotDist == k)
                return array[pivotNewIndex];

            else if (k < pivotDist)
                return ej.pvt.quickSelect(array, fieldName, left, pivotNewIndex - 1, k, comparer);
            else
                return ej.pvt.quickSelect(array, fieldName, pivotNewIndex + 1, right, k - pivotDist, comparer);
        },

        extractFields: function (obj, fields) {
            var newObj = {};

            if (fields.length == 1)
                return ej.pvt.getObject(fields[0], obj);

            for (var i = 0; i < fields.length; i++) {
                newObj[fields[i].replace('.', ej.pvt.consts.complexPropertyMerge)] = ej.pvt.getObject(fields[i], obj);
            }

            return newObj;
        },

        partition: function (array, field, left, right, comparer) {

            var pivotIndex = parseInt((left + right) / 2, 10),
                pivot = getVal(array, field, pivotIndex);

            ej.swap(array, pivotIndex, right);

            pivotIndex = left;

            for (var i = left; i < right; i++) {
                if (comparer(getVal(array, field, i), pivot)) {
                    ej.swap(array, i, pivotIndex);
                    pivotIndex++;
                }
            }

            ej.swap(array, pivotIndex, right);

            return pivotIndex;
        },

        fnSort: function (order) {
            order = order ? order.toLowerCase() : ej.sortOrder.Ascending;

            if (order == ej.sortOrder.Ascending)
                return ej.pvt.fnAscending;

            return ej.pvt.fnDescending;
        },

        fnGetComparer: function (field, fn) {
            return function (x, y) {
                return fn(ej.pvt.getObject(field, x), ej.pvt.getObject(field, y));
            }
        },

        fnAscending: function (x, y) {
            if (x && x.localeCompare)
                return x.localeCompare(y);
            if (x === undefined) {
                if (y === undefined)
                    return 0;
                return 1;
            }
            if (y === undefined)
                return -1;
            return x - y;
        },

        fnDescending: function (x, y) {
            if (x && x.localeCompare)
                return x.localeCompare(y) * -1;
            if (x === undefined) {
                if (y === undefined)
                    return 0;
                return -1;
            }
            if (y === undefined)
                return 1;
            return y - x;
        },

        merge: function (left, right, fieldName, comparer) {
            var result = [], current;

            while (left.length > 0 || right.length > 0) {
                if (left.length > 0 && right.length > 0) {
                    if (comparer)
                        current = comparer(getVal(left, fieldName, 0), getVal(right, fieldName, 0)) < 1 ? left : right;
                    else
                        current = left[0][fieldName] < left[0][fieldName] ? left : right;
                } else {
                    current = left.length > 0 ? left : right;
                }

                result.push(current.shift());
            }

            return result;
        },

        getObject: function (nameSpace, from) {
            if (!from) return undefined;
            if (!nameSpace) return from;

            if (nameSpace.indexOf('.') === -1) return from[nameSpace];

            var value = from, splits = nameSpace.split('.');

            for (var i = 0; i < splits.length; i++) {

                if (value == null) break;

                value = value[splits[i]];
            }

            return value;
        },

        createObject: function (nameSpace, value, initIn) {
            var splits = nameSpace.split('.'), start = initIn || window, from = start, i;

            for (i = 0; i < splits.length; i++) {

                if (i + 1 == splits.length)
                    from[splits[i]] = value === undefined ? {} : value;
                else if (from[splits[i]] == null)
                    from[splits[i]] = {};

                from = from[splits[i]];
            }

            return start;
        },

        getFieldList: function (obj, fields, prefix) {
            if (prefix === undefined)
                prefix = "";

            if (fields === undefined || fields === null)
                return ej.pvt.getFieldList(obj, [], prefix);

            for (var prop in obj) {
                if (typeof obj[prop] === "object" && !(obj[prop] instanceof Array))
                    ej.pvt.getFieldList(obj[prop], fields, prefix + prop + ".");
                else
                    fields.push(prefix + prop);
            }

            return fields;
        }
    };

    ej.FilterOperators = {
        lessThan: "lessthan",
        greaterThan: "greaterthan",
        lessThanOrEqual: "lessthanorequal",
        greaterThanOrEqual: "greaterthanorequal",
        equal: "equal",
        contains: "contains",
        startsWith: "startswith",
        endsWith: "endswith",
        notEqual: "notequal"
    };

    ej.data = {};

    ej.data.operatorSymbols = {
        "<": "lessthan",
        ">": "greaterthan",
        "<=": "lessthanorequal",
        ">=": "greaterthanorequal",
        "==": "equal",
        "!=": "notequal",
        "*=": "contains",
        "$=": "endswith",
        "^=": "startswith"
    };

    ej.data.odBiOperator = {
        "<": " lt ",
        ">": " gt ",
        "<=": " le ",
        ">=": " ge ",
        "==": " eq ",
        "!=": " ne ",
        "lessthan": " lt ",
        "lessthanorequal": " le ",
        "greaterthan": " gt ",
        "greaterthanorequal": " ge ",
        "equal": " eq ",
        "notequal": " ne "
    };

    ej.data.odUniOperator = {
        "$=": "endswith",
        "^=": "startswith",
        "*=": "substringof",
        "endswith": "endswith",
        "startswith": "startswith",
        "contains": "substringof"
    };

    ej.data.fnOperators = {
        equal: function (actual, expected, ignoreCase) {
            if (ignoreCase)
                return toLowerCase(actual) == toLowerCase(expected);

            return actual == expected;
        },
        notequal: function (actual, expected, ignoreCase) {
            return !ej.data.fnOperators.equal(actual, expected, ignoreCase);
        },
        lessthan: function (actual, expected, ignoreCase) {
            if (ignoreCase)
                return toLowerCase(actual) < toLowerCase(expected);

            return actual < expected;
        },
        greaterthan: function (actual, expected, ignoreCase) {
            if (ignoreCase)
                return toLowerCase(actual) > toLowerCase(expected);

            return actual > expected;
        },
        lessthanorequal: function (actual, expected, ignoreCase) {
            if (ignoreCase)
                return toLowerCase(actual) <= toLowerCase(expected);

            return actual <= expected;
        },
        greaterthanorequal: function (actual, expected, ignoreCase) {
            if (ignoreCase)
                return toLowerCase(actual) >= toLowerCase(expected);

            return actual >= expected;
        },
        contains: function (actual, expected, ignoreCase) {
            if (ignoreCase)
                return !isNull(actual) && !isNull(expected) && toLowerCase(actual).indexOf(toLowerCase(expected)) != -1;

            return !isNull(actual) && !isNull(expected) && actual.toString().indexOf(expected) != -1;
        },
        notnull: function (actual) {
            return actual !== null;
        },
        isnull: function (actual) {
            return actual === null;
        },
        startswith: function (actual, expected, ignoreCase) {
            if (ignoreCase)
                return actual && expected && toLowerCase(actual).startsWith(toLowerCase(expected));

            return actual && expected && actual.startsWith(expected);
        },
        endswith: function (actual, expected, ignoreCase) {
            if (ignoreCase)
                return actual && expected && toLowerCase(actual).endsWith(toLowerCase(expected));

            return actual && expected && actual.endsWith(expected);
        },

        processSymbols: function (operator) {
            var fnName = ej.data.operatorSymbols[operator];
            if (fnName) {
                var fn = ej.data.fnOperators[fnName];
                if (fn) return fn;
            }

            return throwError("Query - Process Operator : Invalid operator");
        },

        processOperator: function (operator) {
            var fn = ej.data.fnOperators[operator];
            if (fn) return fn;
            return ej.data.fnOperators.processSymbols(operator);
        }
    };

    ej.NotifierArray = function (array) {
        if (!instance(this, ej.NotifierArray))
            return new ej.NotifierArray(array);

        this.array = array;

        this._events = $({});
        this._isDirty = false;

        return this;
    };

    ej.NotifierArray.prototype = {
        on: function (eventName, handler) {
            this._events.on(eventName, handler);
        },
        off: function (eventName, handler) {
            this._events.off(eventName, handler);
        },
        push: function (item) {
            var ret;

            if (instance(item, Array))
                ret = [].push.apply(this.array, item);
            else
                ret = this.array.push(item);

            this._raise("add", { item: item, index: this.length() - 1 });

            return ret;
        },
        pop: function () {
            var ret = this.array.pop();

            this._raise("remove", { item: ret, index: this.length() - 1 });

            return ret;
        },
        addAt: function (index, item) {
            this.array.splice(index, 0, item);

            this._raise("add", { item: item, index: index });

            return item;
        },
        removeAt: function (index) {
            var ret = this.array.splice(index, 1)[0];

            this._raise("remove", { item: ret, index: index });

            return ret;
        },
        remove: function (item) {
            var index = this.array.indexOf(item);

            if (index > -1) {
                this.array.splice(index, 1);
                this._raise("remove", { item: item, index: index });
            }

            return index;
        },
        length: function () {
            return this.array.length;
        },
        _raise: function (e, args) {
            this._events.triggerHandler($.extend({ type: e }, args));
            this._events.triggerHandler({ type: "all", name: e, args: args });
        },
        toArray: function () {
            return this.array;
        }
    };

    $.extend(ej, ej.dataUtil);

    // For IE8
    Array.prototype.forEach = Array.prototype.forEach || function (fn, scope) {
        for (var i = 0, len = this.length; i < len; ++i) {
            fn.call(scope, this[i], i, this);
        }
    };

    Array.prototype.indexOf = Array.prototype.indexOf || function (searchElement) {
        var len = this.length;

        if (len === 0) return -1;

        for (var i = 0; i < len; i++) {
            if (i in this && this[i] === searchElement)
                return i;
        }
        return -1;
    };

    Array.prototype.filter = Array.prototype.filter || function (fn) {
        if (typeof fn != "function")
            throw new TypeError();

        var res = [];
        var thisp = arguments[1] || this;
        for (var i = 0; i < this.length; i++) {
            var val = this[i]; // in case fun mutates this
            if (fn.call(thisp, val, i, this))
                res.push(val);
        }

        return res;
    };

    String.prototype.endsWith = String.prototype.endsWith || function (key) {
        return this.slice(-key.length) === key;
    };

    String.prototype.startsWith = String.prototype.startsWith || function (key) {
        return this.slice(0, key.length) === key;
    };

    if (!ej.support) ej.support = {};
    ej.support.stableSort = function () {
        var res = [0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16].sort(function () { return 0; });
		for(var i = 0; i < 17; i++){
		    if(i !== res[i]) return false;
		}
        return true;
    }();
    ej.support.cors = $.support.cors;

    if (!$.support.cors && window.XDomainRequest) {
        var httpRegEx = /^https?:\/\//i;
        var getOrPostRegEx = /^get|post$/i;
        var sameSchemeRegEx = new RegExp('^' + location.protocol, 'i');
        var xmlRegEx = /\/xml/i;

        // ajaxTransport exists in jQuery 1.5+
        $.ajaxTransport('text html xml json', function (options, userOptions, jqXHR) {
            // XDomainRequests must be: asynchronous, GET or POST methods, HTTP or HTTPS protocol, and same scheme as calling page
            if (options.crossDomain && options.async && getOrPostRegEx.test(options.type) && httpRegEx.test(userOptions.url) && sameSchemeRegEx.test(userOptions.url)) {
                var xdr = null;
                var userType = (userOptions.dataType || '').toLowerCase();
                return {
                    send: function (headers, complete) {
                        xdr = new XDomainRequest();
                        if (/^\d+$/.test(userOptions.timeout)) {
                            xdr.timeout = userOptions.timeout;
                        }
                        xdr.ontimeout = function () {
                            complete(500, 'timeout');
                        };
                        xdr.onload = function () {
                            var allResponseHeaders = 'Content-Length: ' + xdr.responseText.length + '\r\nContent-Type: ' + xdr.contentType;
                            var status = {
                                code: 200,
                                message: 'success'
                            };
                            var responses = {
                                text: xdr.responseText
                            };

                            try {
                                if (userType === 'json') {
                                    try {
                                        responses.json = JSON.parse(xdr.responseText);
                                    } catch (e) {
                                        status.code = 500;
                                        status.message = 'parseerror';
                                        //throw 'Invalid JSON: ' + xdr.responseText;
                                    }
                                } else if ((userType === 'xml') || ((userType !== 'text') && xmlRegEx.test(xdr.contentType))) {
                                    var doc = new ActiveXObject('Microsoft.XMLDOM');
                                    doc.async = false;
                                    try {
                                        doc.loadXML(xdr.responseText);
                                    } catch (e) {
                                        doc = undefined;
                                    }
                                    if (!doc || !doc.documentElement || doc.getElementsByTagName('parsererror').length) {
                                        status.code = 500;
                                        status.message = 'parseerror';
                                        throw 'Invalid XML: ' + xdr.responseText;
                                    }
                                    responses.xml = doc;
                                }
                            } catch (parseMessage) {
                                throw parseMessage;
                            } finally {
                                complete(status.code, status.message, responses, allResponseHeaders);
                            }
                        };
                        xdr.onerror = function () {
                            complete(500, 'error', {
                                text: xdr.responseText
                            });
                        };
						if(navigator.userAgent.indexOf("MSIE 9.0") != -1)
							xdr.onprogress = function() {};
                        xdr.open(options.type, options.url);
                        //xdr.send(userOptions.data);
                        xdr.send();
                    },
                    abort: function () {
                        if (xdr) {
                            xdr.abort();
                        }
                    }
                };
            }
        });
    }

    $.support.cors = true;

    ej.sortOrder = {
        Ascending: "ascending",
        Descending: "descending"
    };

    // privates
    ej.pvt.consts = {
        GROUPGUID: "{271bbba0-1ee7}",
        complexPropertyMerge: "_"
    };

    // private utils
    var nextTick = function (fn, context) {
        if (context) fn = $proxy(fn, context);
        (window.setImmediate || window.setTimeout)(fn, 0);
    };

    ej.support.enableLocalizedSort = false;

    var stableSort = function (ds, field, comparer) {
        if (ej.support.stableSort) {
            if(!ej.support.enableLocalizedSort && typeof ej.pvt.getObject(field, ds[0] || {}) == "string" 
                && (comparer === ej.pvt.fnAscending || comparer === ej.pvt.fnDescending))
                return fastSort(ds, field, comparer === ej.pvt.fnDescending);
            return ds.sort(ej.pvt.fnGetComparer(field, comparer));
        }
        return ej.mergeSort(ds, field, comparer);
    };

    var fastSort = function(ds, field, isDesc){
        var old = Object.prototype.toString;
        Object.prototype.toString = (field.indexOf('.') === -1) ? function(){
            return this[field];
        }:function(){
            return ej.pvt.getObject(field, this);
        };
        ds = ds.sort();
        Object.prototype.toString = old;
        if(isDesc)
            ds.reverse();
    }

    var toLowerCase = function (val) {
        return val ? val.toLowerCase ? val.toLowerCase() : val.toString() : (val === 0 || val === false) ? val.toString() : "";
    };

    var getVal = function (array, field, index) {
        return field ? ej.pvt.getObject(field, array[index]) : array[index];
    };

    var isHtmlElement = function (e) {
        return typeof HTMLElement === "object" ? e instanceof HTMLElement :
            e && e.nodeType === 1 && typeof e === "object" && typeof e.nodeName === "string";
    };

    var instance = function (obj, element) {
        return obj instanceof element;
    };

    var getTableModel = function (name, result, dm, computed) {
        return function (tName) {
            if (typeof tName === "object") {
                computed = tName;
                tName = null;
            }
            return new ej.TableModel(tName || name, result, dm, computed);
        };
    };

    var getKnockoutModel = function (result) {
        return function (computedObservables, ko) {
            ko = ko || window.ko;

            if (!ko) throwError("Knockout is undefined");

            var model, koModels = [], prop, ob;
            for (var i = 0; i < result.length; i++) {
                model = {};
                for (prop in result[i]) {
                    if (!prop.startsWith("_"))
                        model[prop] = ko.observable(result[i][prop]);
                }
                for (prop in computedObservables) {
                    ob = computedObservables[prop];

                    if ($.isPlainObject(ob)) {
                        if (!ob.owner) ob.owner = model;
                        ob = ko.computed(ob);
                    } else
                        ob = ko.computed(ob, model);

                    model[prop] = ob;
                }
                koModels.push(model);
            }

            return ko.observableArray(koModels);
        };
    };

    var uidIndex = 0;
    var getUid = function (prefix) {
        uidIndex += 1;
        return prefix + uidIndex;
    };

    ej.getGuid = function (prefix) {
        var hexs = '0123456789abcdef', rand;
        return (prefix || "") + '00000000-0000-4000-0000-000000000000'.replace(/0/g, function (val, i) {
            rand = Math.random() * 16 | 0;
            return hexs[i === 19 ? rand & 0x3 | 0x8 : rand];
        });
    };

    var proxy = function (fn, context) {
        return function () {
            var args = [].slice.call(arguments, 0);
            args.push(this);

            return fn.apply(context || this, args);
        };
    };

    var $proxy = function (fn, context, arg) {
        if ('bind' in fn)
            return arg ? fn.bind(context, arg) : fn.bind(context);

        return function () {
            var args = arg ? [arg] : []; args.push.apply(args, arguments);
            fn.apply(context || this, args);
        };
    };

    var dateSerialize = function (date) {
        date = JSON.parse(JSON.stringify({ val: date })).val;
        return date.slice(0);
    };

    ej.merge = function (first, second) {
        if (!first || !second) return;

        Array.prototype.push.apply(first, second);
    };

    var isNull = function (val) {
        return val === undefined || val === null;
    };

    var throwError = function (er) {
        try {
            throw new Error(er);
        } catch (e) {
            throw e.message + "\n" + e.stack;
        }
    };

})(window.jQuery, window.Syncfusion, window.document);;

(function ($, undefined) {

    var $document = $(document);
    // add new event shortcuts
    $.each(("touchstart touchmove touchend " +
		"tap doubletap taphold " +
		"swipe swipeleft swiperight " + "pinch pinchin pinchout pinchstop " +
		"scrollstart scrollstop").split(" "), function (i, name) {

		    $.fn[name] = function (fn) {
		        return fn ? this.bind(name, fn) : this.trigger(name);
		    };

		    // jQuery < 1.8
		    if ($.attrFn) {
		        $.attrFn[name] = true;
		    }
		});

    var isPointer = browserInfo().isMSPointerEnabled,
    isIE11Pointer = browserInfo().pointerEnabled,
    supportTouch = 'ontouchstart' in window,
	scrollEvent = "scroll",
	touchStartEvent = isPointer ? (isIE11Pointer ? "pointerdown" : "MSPointerDown") : (supportTouch ? "touchstart" : "mousedown"),
	touchStopEvent = isPointer ? (isIE11Pointer ? "pointerup" : "MSPointerUp") : (supportTouch ? "touchend" : "mouseup"),
	touchMoveEvent = isPointer ? (isIE11Pointer ? "pointermove" : "MSPointerMove") : (supportTouch ? "touchmove" : "mousemove"),
    touchCancelEvent = isPointer ? (isIE11Pointer ? "pointercancel" : "MSPointerCancel") : (supportTouch ? "touchcancel" : "mouseleave"),
    mouseStartEvent = isPointer ? touchStartEvent : "touchstart mousedown",
    mouseStopEvent = isPointer ? touchStopEvent : "touchend mouseup",
    mouseMoveEvent = isPointer ? touchMoveEvent : "touchmove mousemove";

    function browserInfo() {
        var browser = {}, clientInfo = [],
        browserClients = {
            webkit: /(chrome)[ \/]([\w.]+)/i, safari: /(webkit)[ \/]([\w.]+)/i, msie: /(msie) ([\w.]+)/i,
            opera: /(opera)(?:.*version|)[ \/]([\w.]+)/i, mozilla: /(mozilla)(?:.*? rv:([\w.]+)|)/i
        };
        for (var client in browserClients) {
            if (browserClients.hasOwnProperty(client)) {
                clientInfo = navigator.userAgent.match(browserClients[client]);
                if (clientInfo) {
                    browser.name = clientInfo[1].toLowerCase();
                    browser.version = clientInfo[2];
                    if (!!navigator.userAgent.match(/Trident\/7\./)) {
                        browser.name = "msie";
                    }
                    break;
                }
            }
        }
        browser.isMSPointerEnabled = (browser.name == 'msie') && browser.version > 9 && window.navigator.msPointerEnabled;
        browser.pointerEnabled = window.navigator.pointerEnabled;
        return browser;
    }
    function initiateCustomEvent(obj, eventType, e) {
        var originalType = e.type;
        e.type = eventType;
        $.event.dispatch.call(obj, e);
        e.type = originalType;
    }
    function copyObject(e, origEvent) {
        if (origEvent) {
            for (prop in origEvent) {
                if (!(prop in e)) {
                    e[prop] = origEvent[prop];
                }
            }
        }
    }
    $.event.special.ejtouchmove = {
        setup: function () {
            var thisObj = this,
				$this = $(thisObj);
            $this.bind(touchStartEvent, startMoveHandler);
            $document.bind(touchStopEvent, clearTouchMoveHandlers);
            function clearTouchMoveHandlers() {
               // $this.unbind(touchMoveEvent, moveHandler)
            }
            var coords = {};
            function startMoveHandler(e) {
                if (!(e.which && e.which !== 1)) {
                    var origTarget = e.target,
					origEvent = e.originalEvent;
                    if (isPointer)
                        coords = { x: origEvent.x, y: origEvent.y };
                    $this.bind(touchMoveEvent, moveHandler);
                }
            }
            function moveHandler(e) {
                if (!(e.which && e.which !== 1)) {
                    var origTarget = e.target,
					origEvent = e.originalEvent;
                    copyObject(e, e.originalEvent);
                    if (!isPointer || !coords || (Math.abs(coords.x - origEvent.x) > 10 || Math.abs(coords.y - origEvent.y) > 10 && isPointer))
                        initiateCustomEvent(thisObj, "ejtouchmove", e);
                }
            }
        }
    };
    function touchObj(e) {
        return e.originalEvent.touches ?
					e.originalEvent.touches[0] : isPointer ? e.originalEvent : e;
    }
    // handles swipeup and swipedown
    $.event.special.swipeupdown = {
        setup: function () {
            var thisObj = this, $this = $(thisObj);
            checkMsieTouch($this);
            $this.bind(touchStartEvent, function (e) {
                var _startevent = e;
                var data = touchObj(e),
                            startPoint = {
                                time: (new Date).getTime(),
                                coords: [data.pageX, data.pageY],
                                origin: $(e.target)
                            },
                            stopPoint;
                function moveHandler(e) {
                    e.preventDefault();
                    e.stopPropagation();
                    if (!startPoint) return;
                    var data = touchObj(e);
                    stopPoint = {
                        time: (new Date).getTime(),
                        coords: [data.pageX, data.pageY]
                    };
                    if (Math.abs(startPoint.coords[1] - stopPoint.coords[1]) > 10) e.preventDefault();
                }

                $this
                            .bind(touchMoveEvent, moveHandler)
                            .one(touchStopEvent, function (e) {
                                e.preventDefault();
                                $this.unbind(touchMoveEvent, moveHandler);
                                if (startPoint && stopPoint) {
                                    if (stopPoint.time - startPoint.time < 1000 &&
                                    Math.abs(startPoint.coords[1] - stopPoint.coords[1]) > 30 &&
                                    Math.abs(startPoint.coords[0] - stopPoint.coords[0]) < 75) {
                                        var _addDetails = { time: stopPoint.time - startPoint.time, _isSwipe: true, _isDelta: true, stopPoint: stopPoint };
                                        var _options = _getOptions(e, _addDetails, _startevent);

                                        startPoint.origin
                                        .trigger($.extend(true, { type: "swipeupdown" }, _options))
                                        .trigger($.extend(true, { type: startPoint.coords[1] > stopPoint.coords[1] ? "swipeup" : "swipedown" }, _options));
                                    }
                                }
                                startPoint = stopPoint = undefined;
                            });
            });
        }
    };
    $.event.special.scrollstart = {
        isEnabled: true,
        setup: function () {
            var thisObj = this, $this = $(thisObj), scrolling, timer;
            function trigger(e, scrollState) {
                scrolling = scrollState;
                initiateCustomEvent(thisObj, scrolling ? "scrollstart" : "scrollstop", e);
            }
            $this.bind(scrollEvent, function (e) {
                if (!$.event.special.scrollstart.isEnabled) return;
                if (!scrolling) trigger(e, true);
                clearTimeout(timer);
                timer = setTimeout(function () {
                    trigger(e, false);
                }, 250);
            });
        }
    };

    // also handles doubletap, taphold
    $.event.special.tap = {
        doubleTapThreshold: 500,
        tapholdThreshold: 750,
        canDoubleTap: function (d) {
            return ((getTimeSpan() - d.doubleTapStartTime) <= $.event.special.tap.doubleTapThreshold);
        },
        setup: function () {
            var thisObj = this, $this = $(thisObj), d = $this.data();
            checkMsieTouch($this);
            d.isDoubleTapWait = false;
            d.stopProcess = false;
            d.preTouchend = null;
            d.preTouchstart = null;

            $this.bind(mouseStartEvent, function (event) {
                d = $this.data();
                d.startTime = getTimeSpan();
                if (!d.isDoubleTapWait) d.doubleTapStartTime = d.startTime;
                if (event.type == "touchstart") d.preTouchstart = d.startTime;
                // checked mousedown event arrives within 300'ms after the touchend completes
                if (event.type == "mousedown" && (d.startTime - d.preTouchend < 300 || d.startTime - d.preTouchstart < 30))
                    d.stopProcess = true;
                else d.stopProcess = false;

                var origTarget = event.currentTarget,
					origEvent = event.originalEvent,
					timer;

                function clearTapHandlers() {
                    clearTimeout(timer);

                    $this.unbind(mouseStopEvent, clickHandler);
                    $this.unbind(touchCancelEvent, clearTapHandlers);
                    $document.unbind(mouseMoveEvent, touchMoveAction);
                }
                function touchMoveAction(e) {
                    var coor1 = e, coor2 = event;
                    if (isPointer) coor1 = e.originalEvent, coor2 = event.originalEvent;
                    else if (e.type == "touchmove" && event.type == "touchstart")
                        coor1 = e.originalEvent.changedTouches[0],
                        coor2 = event.originalEvent.changedTouches[0];
                    if (!(coor1.pageX == coor2.pageX && coor1.pageY == coor2.pageY))
                        clearTapHandlers();
                }
                function clickHandler(e) {
                    if (e.type == "touchend") d.preTouchend = getTimeSpan();
                    clearTapHandlers();

                    // ONLY trigger a 'tap' event if the startPoint target is
                    // the same as the stopPoint target.
                    if (origTarget === e.currentTarget) {
                        initiateCustomEvent(thisObj, "tap", $.extend(_getBaseOptions(e), {
                            time: getTimeSpan() - d.startTime
                        }));

                        if (d.isDoubleTapWait && $.event.special.tap.canDoubleTap(d)) {
                            d.isDoubleTapWait = false;

                            initiateCustomEvent(thisObj, "doubletap", $.extend(_getBaseOptions(e), {
                                time: getTimeSpan() - d.doubleTapStartTime
                            }));
                        }
                        else {
                            if (d.isDoubleTapWait) {
                                d.isDoubleTapWait = false;
                                d.doubleTapStartTime = d.startTime;
                            }
                            if ($.event.special.tap.canDoubleTap(d)) {
                                d.isDoubleTapWait = true;
                            }
                        }
                    }
                }

                if (!(event.which && event.which !== 1) && !d.stopProcess) {

                    $this.bind(mouseStopEvent, clickHandler);
                    $this.bind(touchCancelEvent, clearTapHandlers);
                    $document.bind(mouseMoveEvent, touchMoveAction);

                    timer = setTimeout(function () {
                        if (d.isDoubleTapWait) d.isDoubleTapWait = false;
                        initiateCustomEvent(thisObj, "taphold", $.extend(_getBaseOptions(event), {
                            options: origEvent,
                            time: getTimeSpan() - d.startTime
                        }));

                    }, $.event.special.tap.tapholdThreshold);
                }
                else if (d.stopProcess) d.stopProcess = false;
            });
        }
    };    
    $.event.special.swipe = {
        scrollSupression: 10,
        duration: 1000,
        horizontalDistance: 30,
        verticalDistance: 75,
        pointers: window.navigator.msPointerEnabled,
        startPoint: function (e) {
            var data = touchObj(e);
            return {
                time: (new Date()).getTime(),
                coords: [data.pageX, data.pageY],
                origin: $(e.target)
            };
        },
        stopPoint: function (e) {
            var data = touchObj(e);
            return {
                time: (new Date()).getTime(),
                coords: [data.pageX, data.pageY]
            };
        },
        handleSwipe: function (startPoint, stopPoint, e, _startevent) {

            if (stopPoint.time - startPoint.time < $.event.special.swipe.duration &&
				Math.abs(startPoint.coords[0] - stopPoint.coords[0]) > $.event.special.swipe.horizontalDistance &&
				Math.abs(startPoint.coords[1] - stopPoint.coords[1]) < $.event.special.swipe.verticalDistance) {

                var _addDetails = { time: stopPoint.time - startPoint.time, _isSwipe: true, _isDelta: true, stopPoint: stopPoint };
                var _options = _getOptions(e, _addDetails, _startevent);

                startPoint.origin.trigger($.extend(true, { type: "swipe" }, _options))
					.trigger($.extend(true,
                    { type: startPoint.coords[0] > stopPoint.coords[0] ? "swipeleft" : "swiperight" }, _options));
            }
        },

        setup: function () {
            var thisObj = this, $this = $(thisObj);
            checkMsieTouch($this);

            $this.bind(touchStartEvent, function (e) {
                var startPoint = $.event.special.swipe.startPoint(e),
					stopPoint;
                var _startevent = e;
                $(e.target).data('_dataTouchStart', { event: e, _now: new Date().getTime() });

                function moveHandler(e) {
                    if (!startPoint) return;
                    stopPoint = $.event.special.swipe.stopPoint(e);
                    if (Math.abs(startPoint.coords[0] - stopPoint.coords[0]) > $.event.special.swipe.scrollSupression) e.preventDefault();
                }

                $this.bind(touchMoveEvent, moveHandler)
					.one(touchStopEvent, function (e) {
					    $this.unbind(touchMoveEvent, moveHandler);
					    if (startPoint && stopPoint) {
					        $.event.special.swipe.handleSwipe(startPoint, stopPoint, e, _startevent);
					    }
					    startPoint = stopPoint = undefined;
					});
            });
        }
    };
    // also handles pinchin, pinchout
    $.event.special.pinch = {
        distance: function (e) {
            if (e.originalEvent.touches.length < 2) return null;
            return $.event.special.pinch._getdistance(e.originalEvent.touches[0], e.originalEvent.touches[1]);
        },
        _getdistance: function (coor1, coor2) {
            return Math.sqrt((coor1.pageX - coor2.pageX) * (coor1.pageX - coor2.pageX) +
                (coor1.pageY - coor2.pageY) * (coor1.pageY - coor2.pageY));
        },
        setup: function () {
            var thisObj = this, $this = $(thisObj);
            checkMsieTouch($this);
            $this.bind(touchStartEvent, function (e) {
                var _startevent = e;
                if (e.originalEvent.touches && e.originalEvent.touches.length >= 2) {
                    var startPoint = $.event.special.pinch.distance(e), stopPoint, minDistance = 5;

                    var _pinchDistance = startPoint, moveEvent;
                    var _options = _getOptions(e, { _isPinch: true, _pinchDistance: _pinchDistance }, _startevent);

                    $(e.target).trigger($.extend(true, { type: "pinch" }, _options));

                    function moveHandler(e) {
                        moveEvent = e;
                        stopPoint = $.event.special.pinch.distance(e) || null;
                        if (startPoint && stopPoint && Math.abs(startPoint - stopPoint) > minDistance) {

                            $(e.target).trigger($.extend(true,
                                { type: startPoint > stopPoint ? "pinchin" : "pinchout" }, _getOptions(e,
                                { _isPinch: true, _pinchDistance: _pinchDistance }, _startevent)));

                            startPoint = stopPoint;
                        }
                    }

                    $this.bind(touchMoveEvent, moveHandler)
                        .one(touchStopEvent, function () {
                            $this.unbind(touchMoveEvent, moveHandler);
                            $(e.target).trigger($.extend(true, { type: "pinchstop" }, _getOptions(moveEvent,
                                { _isPinch: true, _pinchDistance: stopPoint }, _startevent)));
                            startPoint = stopPoint = undefined;
                        });
                }
            });
        }
    };
    //  handles touchdrag, touchdrag
    $.event.special.touchdrag = {

        setup: function () {
            var thisObj = this, $this = $(thisObj);
            checkMsieTouch($this);

            $this.bind(touchStartEvent, function (e) {

                var startPoint = touchObj(e),
					stopPoint;
                var _startevent = e;
                $(e.target).data('_dataTouchStart', { event: e, _now: new Date().getTime() });

                function moveHandler(e) {
                    if (!startPoint) {
                        return;
                    }
                    stopPoint = touchObj(e);

                    if ($.event.special.pinch._getdistance(startPoint, stopPoint) > 5)
                        $(e.target).trigger($.extend(true, { type: "touchdrag" },
                            _getOptions(e, { _isdrag: true, stopPoint: stopPoint, _isDelta: true }, _startevent)));
                }

                $this.bind(touchMoveEvent, moveHandler)
					.one(touchStopEvent, function (e) {
					    $this.unbind(touchMoveEvent, moveHandler);
					    startPoint = stopPoint = undefined;
					});
            });
        }
    };
    function _getBaseOptions(e) {
        var _pointer = isPointer ? e.originalEvent.pointerType : (!e.originalEvent.touches ? "mouse" : "touch")
        var pointerType = (isPointer && isIE11Pointer == undefined) ? (_pointer == 4 ? "mouse" : "touch") : _pointer;
        e["pointerType"] = pointerType;
        if (e.type != "mousedown" && e.type != "mouseup") {
            copyObject(e, e.originalEvent);
        }
        return e;
    }
    function _getOptions(e, _details, _startevent) {
        var _distance, _time, _scale, _iDelta = {}, _startXY, _endXY;
        if (_startevent) {
            var coor;
            if (!_startevent.originalEvent.touches) coor = [_startevent.originalEvent, e.originalEvent];
            else coor = [_startevent.originalEvent.touches[0], e.originalEvent.changedTouches[0]];

            if (_details._isSwipe || _details._isdrag) {
                _distance = $.event.special.pinch._getdistance(coor[0], coor[1]);
                _time = _details.time;
                _endXY = { pageX: _details.stopPoint.pageX, pageY: _details.stopPoint.pageY };
            }
            else if (_details._isPinch) {
                _distance = $.event.special.pinch.distance(e);
                _time = e.timeStamp - _startevent.timeStamp;
                _scale = _details._pinchDistance;
            }
            if (_details._isDelta) {
                _iDelta._dTime = e.timeStamp - _startevent.timeStamp;
                _iDelta._x = coor[1].pageX - coor[0].pageX;
                _iDelta._y = coor[1].pageY - coor[0].pageY;
            }
        }

        return {
            options: e,
            delta: {
                time: _iDelta._dTime || null,
                X: _iDelta._x || null,
                Y: _iDelta._y || null
            },
            distance: _distance,
            scale: _details._isPinch ? _scale : null,
            time: _time,
            velocity: {
                XY: _distance / _iDelta._dTime || null,
                X: _iDelta._x / _iDelta._dTime || null,
                Y: _iDelta._y / _iDelta._dTime || null
            },
            currentPosition: { pageX: _endXY ? _endXY.pageX : null, pageY: _endXY ? _endXY.pageY : null }
        };
    }

    function getTimeSpan() {
        var now = new Date();
        return now.getTime();
    }
    function checkMsieTouch($this) {
        if (isPointer)
            $this.css("-ms-touch-action", "pinch-zoom").css("touch-action", "pinch-zoom");
    }

    $.each({
        scrollstop: "scrollstart",
        doubletap: "tap",
        taphold: "tap",
        swipeleft: "swipe",
        swiperight: "swipe",
        swipedown: "swipeupdown",
        swipeup: "swipeupdown",
        pinchin: "pinch",
        pinchout: "pinch",
        pinchstop: "pinch"
    }, function (event, sourceEvent) {

        $.event.special[event] = {
            setup: function () {
                $(this).bind(sourceEvent, $.noop);
            }
        };
    });

})(jQuery);;
/**
* @fileOverview Plugin to drag the html elements
* @copyright Copyright Syncfusion Inc. 2001 - 2013. All rights reserved.
*  Use of this code is subject to the terms of our license.
*  A copy of the current license can be obtained at any time by e-mailing
*  licensing@syncfusion.com. Any infringement will be prosecuted under
*  applicable laws. 
* @version 12.1 
* @author <a href="mailto:licensing@syncfusion.com">Syncfusion Inc</a>
*/
(function ($, ej, undefined) {
    /**
    * @namespace ej
	* @class ejDraggable
	* @requires jQuery
	* @requires ej.core
	* @classdesc Custom Design for dragging HTML elements.
	* @example 
	* &lt;div  id="dragable" &gt;&lt;/ div &gt; <br> 
	* &lt;script&gt;
	* // Create Dragable
    * $('#dragable').ejDraggable(); 	
	* &lt;/script&gt;
	*/
    ej.widget("ejDraggable", "ej.Draggable", {
        // widget element will be automatically set in this
        element: null,

        // user defined model will be automatically set in this
        model: null,
        validTags: ["div", "span", "a"],

        // default model
        defaults: {
            /**		
			* Used to group sets of draggable and droppable items, in addition to droppable's accept option. A draggable with the same scope value as a droppable will be accepted by the droppable.
			* @default 'default'
			* @type {string}
			* @example 
			* //To set scope API value during initialization  
			* 	$("#dragable").ejDraggable({ scope: 'default' });				* 
			* @example 
			* //Get or set the scope API, after initialization:<br>
			*	//Gets the scope value  
			*	$("#dragable").ejDraggable('option', 'scope');<br>			
			* @memberof ejDraggable
			* @instance
			*/
            scope: 'default', /*Used to group sets of draggable and droppable items, in addition to droppable's accept option. A draggable with the same scope value as a droppable will be accepted by the droppable.*/
            /**		
			* If specified, restricts drag start click to the specified element(s).
			* @default null
			* @type {string}
			* @example 
			* //To set handle API value during initialization  
			* 	$("#dragable").ejDraggable({ handle: null });				* 
			* @example 
			* //Get or set the handle API, after initialization:<br>
			*	//Gets the handle value  
			*	$("#dragable").ejDraggable('option', 'handle');<br>			
			* @memberof ejDraggable
			* @instance
			*/
            handle: null,  /*If specified, restricts drag start click to the specified element(s).*/
            /**		
			* If Drag area is specified.
			* @default false
			* @type {boolean}
			* @example 
			* //To set dragArea API value during initialization  
			* 	$("#dragable").ejDraggable({ dragArea: true });				* 
			* @example 
			* //Get or set the dragArea API, after initialization:<br>
			*	//Gets the dragArea value  
			*	$("#dragable").ejDraggable('option', 'dragArea');<br>			
			* @memberof ejDraggable
			* @instance
			*/
            dragArea: false,
            /**		
            * If clone is specified.
            * @default false
            * @type {boolean}
            * @example 
            * //To set clone API value during initialization  
            * 	$("#dragable").ejDraggable({ clone: true });				* 
            * @example 
            * //Get or set the clone API, after initialization:<br>
            *	//Gets the clone value  
            *	$("#dragable").ejDraggable('option', 'clone');<br>			
            * @memberof ejDraggable
            * @instance
            */
            clone: false,
            /**		
            * Distance in pixels after mousedown the mouse must move before dragging should start. This option can be used to prevent unwanted drags when clicking on an element.
            * @default 1
            * @type {number}
            * @example 
            * //To set distance API value during initialization  
            * 	$("#dragable").ejDraggable({ distance: 1 });				* 
            * @example 
            * //Get or set the distance API, after initialization:<br>
            *	//Gets the distance value  
            *	$("#dragable").ejDraggable('option', 'distance');<br>			
            * @memberof ejDraggable
            * @instance
            */
            distance: 1, /* Distance in pixels after mousedown the mouse must move before dragging should start. This option can be used to prevent unwanted drags when clicking on an element. */
            /**		
            *Sets the offset of the dragging helper relative to the mouse cursor.
            * @default { top: -1, left: -2 }
            * @type {object}
            * @example 
            * //To set cursorAt API value during initialization  
            * 	$("#dragable").ejDraggable({ cursorAt:  { top: 1, left: -2 } });				* 
            * @example 
            * //Get or set the cursorAt API, after initialization:<br>
            *	//Gets the cursorAt value  
            *	$("#dragable").ejDraggable('option', 'cursorAt');<br>			
            * @memberof ejDraggable
            * @instance
            */
            cursorAt: { top: -1, left: -2 }, /* Sets the offset of the dragging helper relative to the mouse cursor.  */
            /**     
			* Supply a callback function to handle the drag start event as an init option.
			* @event
			* @name ejDraggable#dragStart 	
			* @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
			* @param {object}  argument.model returns the autocomplete model
			* @param {string}  argument.type returns the name of the event
			* @example 
			* //dragStart event for Draggable
            * $("#dragable").ejDraggable({ 
            *   	dragStart: function(args) {}
            * });      
			* @memberof ejDraggable
			* @instance
			*/
            dragStart: null, /* Supply a callback function to handle the drag start event as an init option. */
            /**     
			* This event is triggered when the mouse is moved during the dragging.
			* @event
			* @name ejDraggable#drag 	
			* @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
			* @param {object}  argument.model returns the autocomplete model
			* @param {string}  argument.type returns the name of the event
			* @example 
			* //drag event for Draggable
            * $("#dragable").ejDraggable({ 
            *   	drag: function(args) {}
            * });      
			* @memberof ejDraggable
			* @instance
			*/
            drag: null, /* This event is triggered when the mouse is moved during the dragging. */
            /**     
            * This event is triggered when the mouse is moved during the dragging.
            * @event
            * @name ejDraggable#dragStop 	
            * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
            * @param {object}  argument.model returns the autocomplete model
            * @param {string}  argument.type returns the name of the event
            * @example 
            * //dragStop event for Draggable
            * $("#dragable").ejDraggable({ 
            *   	dragStop: function(args) {}
            * });      
            * @memberof ejDraggable
            * @instance
            */
            dragStop: null, /* This event is triggered when dragging stops. */
            /**     
            * This event is triggered when dragging events are destroyed.
            * @event
            * @name ejDraggable#destroy 	
            * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
            * @param {object}  argument.model returns the autocomplete model
            * @param {string}  argument.type returns the name of the event
            * @example 
            * //destroy event for Draggable
            * $("#dragable").ejDraggable({ 
            *   	destroy: function(args) {}
            * });      
            * @memberof ejDraggable
            * @instance
            */
            destroy: null, /* This event is triggered when dragging events are destroyed. */
            /**     
            * This event is triggered when dragged.
            * @event
            * @name ejDraggable#helper 	
            * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
            * @param {object}  argument.model returns the autocomplete model
            * @param {string}  argument.type returns the name of the event
            * @example 
            * //helper event for Draggable
            * $("#dragable").ejDraggable({ 
            *   	helper: function () {
                return $('<div class="e-drag-helper" />').html("draggable").appendTo(document.body);}
            * });      
            * @memberof ejDraggable
            * @instance
            */
            helper: function () {
                return $('<div class="e-drag-helper" />').html("draggable").appendTo(document.body);
            }
        },

        // constructor function
        _init: function () {
            this.handler = function () { },
			this.resizables = {},
			helpers = {};
            this._wireEvents();
        },

        _setModel: function (options) {

        },
        /**
        * destroy in the dragable.
        * @return jQuery
        * @example 
        * &lt; div  id="dragable" &gt; &lt;/div &gt; <br> 
        * &lt;script&gt;
        * // Create dragableObj
        * var dragableObj  = $("#dragable").data("ejDraggable");
        * dragableObj.destroy(); 
        * &lt;/script&gt;
        *@memberof ejDraggable
        * @instance
        */
        // all events bound using this._on will be unbind automatically
        _destroy: function () {
            $(document)
                .unbind(ej.eventType.mouseUp, this._destroyHandler)
                .unbind(ej.eventType.mouseUp, this._dragStopHandler)
                .unbind(ej.eventType.mouseMove, this._dragStartHandler)
                .unbind(ej.eventType.mouseMove, this._dragHandler)
                .unbind('selectstart', false);

            // this._raise(e, this.destroy);

            ej.widgetBase.droppables[this.scope] = null;
            //helpers[this.handle] = null;
        },

        _initialize: function (e) {
            var ori = e;
            e.preventDefault();
            e = this._getCoordinate(e);
            this.target = $(ori.currentTarget);
            this._initPosition = { x: e.pageX, y: e.pageY };
            
            $(document).bind(ej.eventType.mouseMove, this._dragStartHandler).bind(ej.eventType.mouseUp, this._destroyHandler);
            if (!this.model.clone) {
                var _offset = this.element.offset();
                this._relXposition = e.pageX - _offset.left;
                this._relYposition = e.pageY - _offset.top;
            }
            $(document.documentElement).trigger(ej.eventType.mouseDown, ori); // The next statement will prevent 'mousedown', so manually trigger it.
           //return false;
        },
        _setDragArea: function () {
            var _dragElement = $(this.model.dragArea)[0]; if (!_dragElement) return;
            var over = ($(_dragElement).css("overflow") != 'hidden');

            this.dragArea = [
                (parseInt($(_dragElement).css("borderLeftWidth"), 10) || 0) + (parseInt($(_dragElement).css("paddingLeft"), 10) || 0),
                (parseInt($(_dragElement).css("borderTopWidth"), 10) || 0) + (parseInt($(_dragElement).css("paddingTop"), 10) || 0),
                (over ? Math.max(_dragElement.scrollWidth, _dragElement.offsetWidth) : _dragElement.offsetWidth) - (parseInt($(_dragElement).css("borderLeftWidth"), 10) || 0) - (parseInt($(_dragElement).css("paddingRight"), 10) || 0) - this.helper.outerWidth() - this.margins.left - this.margins.right,
                (over ? Math.max(_dragElement.scrollHeight, _dragElement.offsetHeight) : _dragElement.offsetHeight) - (parseInt($(_dragElement).css("borderTopWidth"), 10) || 0) - (parseInt($(_dragElement).css("paddingBottom"), 10) || 0) - this.helper.outerHeight() - this.margins.top - this.margins.bottom
            ];
            this.relativeArea = $(this.model.dragArea);
        },
        _dragStart: function (e) {
            var ori = e;
            e = this._getCoordinate(e);
            
            this.margins = {
                left: (parseInt(this.element.css("marginLeft"), 10) || 0),
                top: (parseInt(this.element.css("marginTop"), 10) || 0),
                right: (parseInt(this.element.css("marginRight"), 10) || 0),
                bottom: (parseInt(this.element.css("marginBottom"), 10) || 0)
            };
            this.offset = this.element.offset();
            this.offset = {
                top: this.offset.top - this.margins.top,
                left: this.offset.left - this.margins.left
            };
            this.position = this._getMousePosition(ori);
            var x = this._initPosition.x - e.pageX, y = this._initPosition.y - e.pageY;
            var distance = Math.sqrt((x * x) + (y * y));

            if (distance >= this.model.distance) {
                var dragTargetElmnt = this.model.handle = this.helper = this.model.helper({ sender: ori, element: this.target });
                if (this.model.dragArea)
                    this._setDragArea();

                //if (!this.model.clone)
                //    this.position.left = this.position.left - (ori.target.offsetWidth / 2);
                //this.position.top = this.position.top - (ori.target.offsetHeight - ori.offsetY);

                if (this.model.dragStart) {
                    var currTarget = null;
                    if (ori.type == 'touchmove') {
                        var coor = ori.originalEvent.changedTouches[0];
                        currTarget = document.elementFromPoint(coor.pageX, coor.pageY);
                    }
                    else currTarget = ori.originalEvent.target || ori.target;
                    if (this._trigger("dragStart", { event: ori, element: this.element, target: currTarget })) {
                        this._destroy();
                        return false;
                    }
                }

                $(document).unbind(ej.eventType.mouseMove, this._dragStartHandler).unbind(ej.eventType.mouseUp, this._destroyHandler)
                    .bind(ej.eventType.mouseMove, this._dragHandler).bind(ej.eventType.mouseUp, this._dragStopHandler).bind("selectstart", false);
                ej.widgetBase.droppables[this.model.scope] = {
                    draggable: this.element,
                    helper: dragTargetElmnt.css({ position: 'absolute', left: this.position.left, top: this.position.top }),
                    destroy: this._destroyHandler
                }
            }
        },

        _drag: function (e) {
            this.position = this._getMousePosition(e);
            if (this.position.top < 0)
                this.position.top = 0;
            if ($(document).height() < this.position.top)
                this.position.top = $(document).height();
            if ($(document).width() < this.position.left)
                this.position.left = $(document).width();
            var helperElement = ej.widgetBase.droppables[this.model.scope].helper;
            if (this.model.drag) {
                var currTarget = null;
                if (e.type == 'touchmove') {
                    var coor = e.originalEvent.changedTouches[0];
                    currTarget = document.elementFromPoint(coor.pageX, coor.pageY);
                }
                else currTarget = e.originalEvent.target || e.target;
                this._trigger("drag", { event: e, element: this.target, target: currTarget });// Raise the dragging event
            }
            ej.widgetBase.droppables[this.model.scope].helper.css({ left: this.position.left, top: this.position.top });
        },

        _dragStop: function (e) {
            if (e.type == 'mouseup' || e.type == 'touchend') 
                this._destroy(e);
            if (this.model.dragStop) {
                var currTarget = null;
                if (e.type == 'touchend') {
                    var coor = e.originalEvent.changedTouches[0];
                    currTarget = document.elementFromPoint(coor.pageX, coor.pageY);
                }
                else currTarget = e.originalEvent.target || e.target;
                this._trigger("dragStop", { event: e, element: this.target, target: currTarget });// Raise the dragstop event
            }
        },

        _wireEvents: function () {
            this._on(this.element, ej.eventType.mouseDown, this._initialize);
            this._dragStartHandler = $.proxy(this._dragStart, this);
            this._destroyHandler = $.proxy(this._destroy, this);
            this._dragStopHandler = $.proxy(this._dragStop, this);
            this._dragHandler = $.proxy(this._drag, this);
        },
        _getMousePosition: function (event) {
            event = this._getCoordinate(event);
            var pageX = this.model.clone ? event.pageX : event.pageX - this._relXposition;
            var pageY = this.model.clone ? event.pageY : event.pageY - this._relYposition;
            if (this.dragArea) {
                if (this.relativeArea) {
                    var co = this.relativeArea.offset();
                    _area = [this.dragArea[0] + co.left, this.dragArea[1] + co.top, this.dragArea[2] + co.left, this.dragArea[3] + co.top];
                }
                else {
                    _area = this.dragArea;
                }
                if (event.pageX - this.margins.left < _area[0]) pageX = _area[0] + this.margins.left;
                if (event.pageY - this.margins.top < _area[1]) pageY = _area[1] + this.margins.top;
                if (event.pageX - this.margins.left > _area[2]) pageX = _area[2] + this.margins.left;
                if (event.pageY - this.margins.top > _area[3]) pageY = _area[3] + this.margins.top;
            }
            return { left: pageX - this.margins.left - this.model.cursorAt.left, top: pageY - this.margins.top - this.model.cursorAt.top };
        },
        _getCoordinate: function (evt) {
            var coor = evt;
            if (evt.type == "touchmove" || evt.type == "touchstart" || evt.type == "touchend")
                coor = evt.originalEvent.changedTouches[0];
            return coor;
        }
    });

})(jQuery, Syncfusion);

/**
* @fileOverview Plugin to drop the html elements
* @copyright Copyright Syncfusion Inc. 2001 - 2013. All rights reserved.
*  Use of this code is subject to the terms of our license.
*  A copy of the current license can be obtained at any time by e-mailing
*  licensing@syncfusion.com. Any infringement will be prosecuted under
*  applicable laws. 
* @version 12.1 
* @author <a href="mailto:licensing@syncfusion.com">Syncfusion Inc</a>
*/
(function ($, ej, undefined) {
    /**
   * @namespace ej
   * @class ejDroppable
   * @requires jQuery
   * @requires ej.core
   * @classdesc Custom Design for droping HTML elements.
   * @example 
   * &lt;div  id="dropable" &gt;&lt;/ div &gt; <br> 
   * &lt;script&gt;
   * // Create Dragable
   * $('#dropable').ejDroppable(); 	
   * &lt;/script&gt;
   */
    // Example plugin creation code
    // sfSample is the plugin name 
    // "ej.Sample" is "namespace.className" will hold functions and properties

    ej.widget("ejDroppable", "ej.Droppable", {
        // widget element will be automatically set in this
        element: null,

        // user defined model will be automatically set in this
        model: null,
        validTags: ["div", "span", "a"],

        // default model
        defaults: {
            /**		
			* Used to accept the droppable items.
			* @default null
			* @type {object}
			* @example 
			* //To set scope API value during initialization  
			* 	$("#droppable").ejDroppable({ accept: null });				* 
			* @example 
			* //Get or set the accept API, after initialization:<br>
			*	//Gets the accept value  
			*	$("#droppable").ejDroppable('option', 'accept');<br>			
			* @memberof ejDroppable
			* @instance
			*/
            accept: null,
            /**		
			* Used to group sets of droppable items, in addition to droppable's accept option. A draggable with the same scope value as a droppable will be accepted by the droppable.
			* @default 'default'
			* @type {string}
			* @example 
			* //To set scope API value during initialization  
			* 	$("#droppable").ejDroppable({ scope: 'default' });				* 
			* @example 
			* //Get or set the scope API, after initialization:<br>
			*	//Gets the scope value  
			*	$("#droppable").ejDroppable('option', 'scope');<br>			
			* @memberof ejDroppable
			* @instance
			*/
            scope: 'default',
            /**     
			* This event is triggered when the mouse up is moved during the dragging.
			* @event
			* @name ejDroppable#drop 	
			* @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
			* @param {object}  argument.model returns the autocomplete model
			* @param {string}  argument.type returns the name of the event
			* @example 
			* //drop event for Droppable
            * $("#droppable").ejDroppable({ 
            *   	drop: function(args) {}
            * });      
			* @memberof ejDroppable
			* @instance
			*/
            drop: null,
            /**     
            * This event is triggered when the mouse  is moved over.
            * @event
            * @name ejDroppable#over 	
            * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
            * @param {object}  argument.model returns the autocomplete model
            * @param {string}  argument.type returns the name of the event
            * @example 
            * //drop event for Droppable
            * $("#droppable").ejDroppable({ 
            *   	over: function(args) {}
            * });      
            * @memberof ejDroppable
            * @instance
            */
            over: null,
            /**     
            * This event is triggered when the mouse  is moved out.
            * @event
            * @name ejDroppable#out 	
            * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
            * @param {object}  argument.model returns the autocomplete model
            * @param {string}  argument.type returns the name of the event
            * @example 
            * //drop event for Droppable
            * $("#droppable").ejDroppable({ 
            *   	out: function(args) {}
            * });      
            * @memberof ejDroppable
            * @instance
            */
            out: null
        },

        // constructor function
        _init: function () {
            if (this.model.accept) {
                $(this.element).delegate(this.accept, 'mouseenter', $.proxy(this._over, this))
							.delegate(this.accept, 'mouseup', $.proxy(this._drop, this))
							.delegate(this.accept, 'mouseleave', $.proxy(this._out, this));
            }
            else {
                $(this.element).bind('mouseup', $.proxy(this._drop, this));
                $(this.element).bind("mouseenter", $.proxy(this._over, this));
                $(this.element).bind("mouseleave", $.proxy(this._out, this));
            }
            this._on($(document), 'touchend', this._drop);
        },

        _setModel: function (options) {

        },
        /**
        * destroy in the Droppable.
        * @return jQuery
        * @example 
        * &lt; div  id="droppable" &gt; &lt;/div &gt; <br> 
        * &lt;script&gt;
        * // Create droppabaleObj
        * var droppabaleObj  = $("#droppable").data("ejDroppable");
        * droppabaleObj.destroy(); 
        * &lt;/script&gt;
        *@memberof ejDroppable
        * @instance
        */
        // all events bound using this._on will be unbind automatically
        _destroy: function () {

        },

        _over: function (e) {
            this._trigger("mouseover", e);  
        },
        _out: function (e) {
            this._trigger("mouseout", e);  
        },
        _drop: function (e) {
            var drag = ej.widgetBase.droppables[this.model.scope];
            var isDragged = !ej.isNullOrUndefined(drag.helper) && drag.helper.is(":visible");
            var area = this._isDropArea(e);
            if (drag && !ej.isNullOrUndefined(this.model.drop) && isDragged && area.canDrop) {
                this.model.drop($.extend(e, { dropTarget: area.target }, true), drag);
            }
        },
        _isDropArea: function (e) {
            // check for touch devices only
            var area = { canDrop: true, target: $(e.target) };
            if (e.type == "touchend") {
                var coor = e.originalEvent.changedTouches[0], _target;
                _target = document.elementFromPoint(coor.pageX, coor.pageY);
                area.canDrop = false;
                var _parents = $(_target).parents();

                for (var i = 0; i < this.element.length; i++) {
                    if ($(_target).is($(this.element[i]))) area = { canDrop: true, target: $(_target) };
                    else for (var j = 0; j < _parents.length; j++) {
                        if ($(this.element[i]).is($(_parents[j]))) {
                            area = { canDrop: true, target: $(_target) };
                            break;
                        }
                    }
                    if (area.canDrop) break;
                }
            }
            return area;
        }
    });

})(jQuery, Syncfusion);

/**
* @fileOverview Plugin to resize the Html elements
* @copyright Copyright Syncfusion Inc. 2001 - 2013. All rights reserved.
*  Use of this code is subject to the terms of our license.
*  A copy of the current license can be obtained at any time by e-mailing
*  licensing@syncfusion.com. Any infringement will be prosecuted under
*  applicable laws. 
* @version 12.1 
* @author <a href="mailto:licensing@syncfusion.com">Syncfusion Inc</a>
*/
(function ($, ej, undefined) {
    /**
    * @namespace ej
    * @class ejResizable
    * @requires jQuery
    * @requires ej.core
    * @classdesc Custom Design for resizing HTML elements.
    * @example 
    * &lt;div  id="resizing" &gt;&lt;/ div &gt; <br> 
    * &lt;script&gt;
    * // Create resizing
    * $('#resizing').ejResizable(); 	
    * &lt;/script&gt;
    */
    ej.widget("ejResizable", "ej.resizable", {
        // widget element will be automatically set in this
        element: null,

        // user defined model will be automatically set in this
        model: null,
        validTags: ["div", "span", "a"],

        // default model
        defaults: {
            /**		
           * Used to group sets of resizeable items.
           * @default 'default'
           * @type {string}
           * @example 
           * //To set scope API value during initialization  
           * 	$("#resizing").ejResizable({ scope: 'default' });				* 
           * @example 
           * //Get or set the scope API, after initialization:<br>
           *	//Gets the scope value  
           *	$("#resizing").ejResizable('option', 'scope');<br>			
           * @memberof ejResizable
           * @instance
           */
            scope: 'default',
            /**		
			* If specified, restricts resize start click to the specified element(s).
			* @default null
			* @type {string}
			* @example 
			* //To set handle API value during initialization  
			* 	$("#resizing").ejResizable({ handle: null });				* 
			* @example 
			* //Get or set the handle API, after initialization:<br>
			*	//Gets the handle value  
			*	$("#resizing").ejResizable('option', 'handle');<br>			
			* @memberof ejResizable
			* @instance
			*/
            handle: null,
            /**		
            * Distance in pixels after mousedown the mouse must move before resizing should start. This option can be used to prevent unwanted drags when clicking on an element.
            * @default 1
            * @type {number}
            * @example 
            * //To set distance API value during initialization  
            * 	$("#resizing").ejResizable({ distance: 1 });				* 
            * @example 
            * //Get or set the distance API, after initialization:<br>
            *	//Gets the distance value  
            *	$("#resizing").ejResizable('option', 'distance');<br>			
            * @memberof ejResizable
            * @instance
            */
            distance: 1,
            /**		
            *Sets the max height for resizing
            * @default null
            * @type {number}
            * @example 
            * //To set maxHeight API value during initialization  
            * 	$("#resizing").ejResizable({ maxHeight: null });				* 
            * @example 
            * //Get or set the maxHeight API, after initialization:<br>
            *	//Gets the maxHeight value  
            *	$("#resizing").ejResizable('option', 'maxHeight');<br>			
            * @memberof ejResizable
            * @instance
            */
            maxHeight: null,
            /**		
            *Sets the max width for resizing
            * @default null
            * @type {number}
            * @example 
            * //To set maxWidth API value during initialization  
            * 	$("#resizing").ejResizable({ maxWidth: null });				* 
            * @example 
            * //Get or set the maxWidth API, after initialization:<br>
            *	//Gets the maxWidth value  
            *	$("#resizing").ejResizable('option', 'maxWidth');<br>			
            * @memberof ejResizable
            * @instance
            */
            maxWidth: null,
            /**		
            *Sets the min Height for resizing
            * @default 10
            * @type {number}
            * @example 
            * //To set minHeight API value during initialization  
            * 	$("#resizing").ejResizable({ minHeight: null });				* 
            * @example 
            * //Get or set the minHeight API, after initialization:<br>
            *	//Gets the minHeight value  
            *	$("#resizing").ejResizable('option', 'minHeight');<br>			
            * @memberof ejResizable
            * @instance
            */
            minHeight: 10,
            /**		
            *Sets the min Width for resizing
            * @default 10
            * @type {number}
            * @example 
            * //To set minWidth API value during initialization  
            * 	$("#resizing").ejResizable({ minWidth: null });				* 
            * @example 
            * //Get or set the minWidth API, after initialization:<br>
            *	//Gets the minWidth value  
            *	$("#resizing").ejResizable('option', 'minWidth');<br>			
            * @memberof ejResizable
            * @instance
            */
            minWidth: 10,
            /**		
            *Sets the offset of the resizing helper relative to the mouse cursor.
            * @default { top: -1, left: -2 }
            * @type {object}
            * @example 
            * //To set cursorAt API value during initialization  
            * 	$("#resizing").ejResizable({ cursorAt:  { top: 1, left: -2 } });				* 
            * @example 
            * //Get or set the cursorAt API, after initialization:<br>
            *	//Gets the cursorAt value  
            *	$("#resizing").ejResizable('option', 'cursorAt');<br>			
            * @memberof ejResizable
            * @instance
            */
            cursorAt: { top: 1, left: 1 },
            /**     
            * This event is triggered when the resizing is start.
            * @event
            * @name ejResizable#resizeStart 	
            * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
            * @param {object}  argument.model returns the autocomplete model
            * @param {string}  argument.type returns the name of the event
            * @example 
            * //resizeStart event for Resizable
            * $("#resizing").ejResizable({ 
            *   	resizeStart: function(args) {}
            * });      
            * @memberof ejResizable
            * @instance
            */
            resizeStart: null,
            /**     
            * This event is triggered when the resizing in progress.
            * @event
            * @name ejResizable#resize 	
            * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
            * @param {object}  argument.model returns the autocomplete model
            * @param {string}  argument.type returns the name of the event
            * @example 
            * //resizeStart event for Resizable
            * $("#resizing").ejResizable({ 
            *   	resize: function(args) {}
            * });      
            * @memberof ejResizable
            * @instance
            */
            resize: null,
            /**     
            * This event is triggered when the resizing is stop.
            * @event
            * @name ejResizable#resizeStop 	
            * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
            * @param {object}  argument.model returns the autocomplete model
            * @param {string}  argument.type returns the name of the event
            * @example 
            * //resizeStop event for Resizable
            * $("#resizing").ejResizable({ 
            *   	resizeStop: function(args) {}
            * });      
            * @memberof ejResizable
            * @instance
            */
            resizeStop: null,
            /**     
           * This event is triggered when the widget destroys.
           * @event
           * @name ejResizable#resizeStop 	
           * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
           * @param {object}  argument.model returns the autocomplete model
           * @param {string}  argument.type returns the name of the event
           * @example 
           * //destroy event for Resizable
           * $("#resizing").ejResizable({ 
           *   	destroy: function(args) {}
           * });      
           * @memberof ejResizable
           * @instance
           */
            destroy: null,
            /**     
           * This event is triggered when resized.
           * @event
           * @name ejResizable#helper 	
           * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
           * @param {object}  argument.model returns the autocomplete model
           * @param {string}  argument.type returns the name of the event
           * @example 
           * //helper event for Resizable
           * $("#resizing").ejResizable({ 
           *   	helper: function () {
               return $('<div class="e-resize-helper" />').html("resizable").appendTo(document.body);}
           * });      
           * @memberof ejResizable
           * @instance
           */
            helper: function () {
                return $('<div class="e-resize-helper" />').html("resizable").appendTo(document.body);
            }
        },
        // constructor function
        _init: function () {
            this.target = this.element;
            if (this.handle != null) {
                $(this.target).delegate(this.handle, ej.eventType.mouseDown, $.proxy(this._mousedown, this))
                .delegate(this.handle, 'resizestart', this._blockDefaultActions);
            }
            else {
                //$(this.target).bind("mousedown", $.proxy(this._mousedown, this));
                $(this.target).bind("mouseover", $.proxy(this._mouseover, this));
                // .bind("resizestart", $.proxy(this, $.Handler(this._init, this)));
            }

            this._resizeStartHandler = $.proxy(this._resizeStart, this);
            this._destroyHandler = $.proxy(this._destroy, this);
            this._resizeStopHandler = $.proxy(this._resizeStop, this);
            this._resizeHandler = $.proxy(this._resize, this);
        },
        _mouseover: function (e) {
            if ($(e.target).hasClass("e-resizable")) {
                $(e.target).css({ cursor: "se-resize" });
                $(this.target).bind(ej.eventType.mouseDown, $.proxy(this._mousedown, this));
            }
            else {
                $(this.target).unbind(ej.eventType.mouseDown);
                $(this.target).css({ cursor: "" });
            }
        },
        _blockDefaultActions: function (e) {
            e.cancelBubble = true;
            e.returnValue = false;
            if (e.preventDefault) e.preventDefault();
            if (e.stopPropagation) e.stopPropagation();
        },
        _setModel: function (options) {

        },
        _mousedown: function (e) {
            var ori = e;
            e = this._getCoordinate(e);
            this.target = $(ori.currentTarget);
            this._initPosition = { x: e.pageX, y: e.pageY };

            $(document).bind(ej.eventType.mouseMove, this._resizeStartHandler).bind(ej.eventType.mouseUp, this._destroyHandler);

            $(document.documentElement).trigger(ej.eventType.mouseDown, ori); // The next statement will prevent 'mousedown', so manually trigger it.
            return false;
        },

        _resizeStart: function (e) {
            if ($(e.target).hasClass("e-resizable")) {
                e = this._getCoordinate(e);
                var x = this._initPosition.x - e.pageX, y = this._initPosition.y - e.pageY;
                var distance = Math.sqrt((x * x) + (y * y));
                if (distance >= this.model.distance) {
                    if (this.model.resizeStart != null)
                        this.model.resizeStart({ element: this.target });  // Raise the resize start event
                    var resizeTargetElmnt = this.model.helper({ element: this.target });
                    var pos = this.getElementPosition(resizeTargetElmnt);
                    $(document).unbind(ej.eventType.mouseMove, this._resizeStartHandler).unbind(ej.eventType.mouseUp, this._destroyHandler)
                        .bind(ej.eventType.mouseMove, this._resizeHandler).bind(ej.eventType.mouseUp, this._resizeStopHandler).bind("selectstart", false);
                    ej.widgetBase.resizables[this.scope] = {
                        resizable: this.target,
                        helper: resizeTargetElmnt.css({ width: (e.pageX - pos.left) + this.model.cursorAt.left, height: (e.pageY - pos.top) + this.model.cursorAt.top }),
                        destroy: this._destroyHandler
                    }
                }
            }
        },

        _resize: function (e) {
            e = this._getCoordinate(e);
            if (this.model.resize != null)
                this.model.resize({ element: this.target });  // Raise the drag start event
            var pos = this.getElementPosition(ej.widgetBase.resizables[this.scope].helper);
            var _width = (e.pageX - pos.left) + this.model.cursorAt.left;
            var _height = (e.pageY - pos.top) + this.model.cursorAt.top;
            if (_width < this.model.minWidth)
                _width = this.model.minWidth;
            if (_height < this.model.minHeight)
                _height = this.model.minHeight;
            if (this.model.maxHeight != null && _height > this.model.maxHeight)
                _height = this.model.maxHeight;
            if (this.model.maxWidth != null && _width > this.model.maxWidth)
                _width = this.model.maxWidth;
            ej.widgetBase.resizables[this.scope].helper.css({ width: _width, height: _height });
        },

        _resizeStop: function (e) {
            if (this.model.resizeStop != null)
                this.model.resizeStop({ element: this.target });  // Raise the resize stop event
            if (e.type == 'mouseup' || e.type == 'touchend')
                this._destroy(e);
        },
        /**
        * destroy in the Resizable.
        * @return jQuery
        * @example 
        * &lt; div  id="resizing" &gt; &lt;/div &gt; <br> 
        * &lt;script&gt;
        * // Create resizingObj
        * var resizingObj  = $("#resizing").data("ejResizable");
        * resizingObj.destroy(); 
        * &lt;/script&gt;
        *@memberof ejResizable
        * @instance
        */
        _destroy: function (e) {
            $(document)
                .unbind(ej.eventType.mouseUp, this._destroyHandler)
                .unbind(ej.eventType.mouseUp, this._resizeStopHandler)
                .unbind(ej.eventType.mouseMove, this._resizeStartHandler)
                .unbind(ej.eventType.mouseMove, this._resizeHandler)
                .unbind('selectstart', false);

            // this._raise(e, this.destroy);

            ej.widgetBase.resizables[this.scope] = null;
            //helpers[this.handle] = null;
        },

        getElementPosition: function (elemnt) {
            if (elemnt != null && elemnt.length > 0)
                return {
                    left: elemnt[0].offsetLeft,
                    top: elemnt[0].offsetTop
                };
            else
                return null;
        },
        _getCoordinate: function (evt) {
            var coor = evt;
            if (evt.type == "touchmove" || evt.type == "touchstart" || evt.type == "touchend")
                coor = evt.originalEvent.changedTouches[0];
            return coor;
        }
    });

})(jQuery, Syncfusion);;
/**
* @fileOverview Plugin to style the Html Button elements
* @copyright Copyright Syncfusion Inc. 2001 - 2013. All rights reserved.
*  Use of this code is subject to the terms of our license.
*  A copy of the current license can be obtained at any time by e-mailing
*  licensing@syncfusion.com. Any infringement will be prosecuted under
*  applicable laws. 
* @version 12.1 
* @author <a href="mailto:licensing@syncfusion.com">Syncfusion Inc</a>
*/

(function ($, ej, window, undefined) {
    'use strict';
    /**
* @namespace ej
* @class ejScroller
* @requires jQuery
* @requires ej.core.js
* @requires ej.scroller.js
* @requires ej.draggable.js
* @classdesc Custom Design for Html Scroller control.
* @example 
* &lt;div id="scrollcontent" style="width:900px;" &gt;
		*&lt;p&gt;Model–view–controller (MVC) is a software architecture pattern which separates the
        * representation of information from the user's interaction with it.
        * The model consists of application data, business rules, logic, and functions. A view can be any
        * output representation of data, such as a chart or a diagram. Multiple views of the same data 
        * are possible, such as a bar chart for management and a tabular view for accountants. 
        *The controller mediates input, converting it to commands for the model or view.The central 
        *ideas behind MVC are code reusability and n addition to dividing the application into three 
        *kinds of components, the MVC design defines the interactions between them.</p>
        *&lt;ul&gt;
         *&lt;li&gt;
         *&lt;b&gt;A controller*&lt;/b&gt;  can send commands to its associated view to change the view's presentation of the model (e.g., by scrolling through a document). 
         It can also send commands to the model to update the model's state (e.g., editing a document).
        *&lt;/li&gt;
        *&lt;/ul&gt;     
*&lt;/div&gt; <br> 
* &lt;script&gt;
* // Create Scroller
* $('#scrollcontent').ejScroller(); 	
* &lt;/script&gt;
*/
    ej.widget("ejScroller", "ej.Scroller", {
        defaults: {
            /**		
* Specifies the height of Scroll panel and scrollbars.
* @default 250
* @type {number}
* @example 
* &lt;div id="scrollcontent" style="width:900px;" &gt;
*&lt;p&gt;Model–view–controller (MVC) is a software architecture pattern which separates the
        * representation of information from the user's interaction with it.
        * The model consists of application data, business rules, logic, and functions. A view can be any
        * output representation of data, such as a chart or a diagram. Multiple views of the same data 
        * are possible, such as a bar chart for management and a tabular view for accountants. 
        *The controller mediates input, converting it to commands for the model or view.The central 
        *ideas behind MVC are code reusability and n addition to dividing the application into three 
        *kinds of components, the MVC design defines the interactions between them.</p>
        *&lt;ul&gt;
         *&lt;li&gt;
        *&lt;b&gt;A controller*&lt;/b&gt;can send commands to its associated view to change the view's presentation of the model (e.g., by scrolling through a document). 
        *It can also send commands to the model to update the model's state (e.g., editing a document).
        *&lt;/li&gt;
        *&lt;/ul&gt; 
*&lt;/div&gt; <br> 
* &lt;script&gt;
* //To set height API value during initialization  
* 	$("#scrollcontent").ejScroller({height: 200 });	
* &lt;/script&gt;
 * @memberof ejScroller
* @instance
*/
            height: 250,
            autoHide: false,
            /**		
* Specifies the width of Scroll panel and scrollbars.
* @default 0
* @type {number}
* @example 
* &lt;div id="scrollcontent" style="width:900px;" &gt;
*&lt;p&gt;Model–view–controller (MVC) is a software architecture pattern which separates the
        * representation of information from the user's interaction with it.
        * The model consists of application data, business rules, logic, and functions. A view can be any
        * output representation of data, such as a chart or a diagram. Multiple views of the same data 
        * are possible, such as a bar chart for management and a tabular view for accountants. 
        *The controller mediates input, converting it to commands for the model or view.The central 
        *ideas behind MVC are code reusability and n addition to dividing the application into three 
        *kinds of components, the MVC design defines the interactions between them.</p>
        *&lt;ul&gt;
         *&lt;li&gt;
        *&lt;b&gt;A controller*&lt;/b&gt;can send commands to its associated view to change the view's presentation of the model (e.g., by scrolling through a document). 
        *It can also send commands to the model to update the model's state (e.g., editing a document).
        *&lt;/li&gt;
        *&lt;/ul&gt; 
*&lt;/div&gt; <br> 
* &lt;script&gt;
* //To set width API value during initialization
* 	$("#scrollcontent").ejScroller({width: 500 });	
* &lt;/script&gt;
* @memberof ejScroller
* @instance
*/
            width: 0,
            /**		
* While press on the arrow key the scrollbar position added to the given pixel value.
* @default 57
* @type {number}
* @example 
* &lt;div id="scrollcontent" style="width:900px;" &gt;
*&lt;p&gt;Model–view–controller (MVC) is a software architecture pattern which separates the
        * representation of information from the user's interaction with it.
        * The model consists of application data, business rules, logic, and functions. A view can be any
        * output representation of data, such as a chart or a diagram. Multiple views of the same data 
        * are possible, such as a bar chart for management and a tabular view for accountants. 
        *The controller mediates input, converting it to commands for the model or view.The central 
        *ideas behind MVC are code reusability and n addition to dividing the application into three 
        *kinds of components, the MVC design defines the interactions between them.</p>
        *&lt;ul&gt;
         *&lt;li&gt;
        *&lt;b&gt;A controller*&lt;/b&gt;can send commands to its associated view to change the view's presentation of the model (e.g., by scrolling through a document). 
        *It can also send commands to the model to update the model's state (e.g., editing a document).
        *&lt;/li&gt;
        *&lt;/ul&gt; 
*&lt;/div&gt; <br> 
* &lt;script&gt;
* //To set scrollOneStepBy API value during initialization
* 	$("#scrollcontent").ejScroller({scrollOneStepBy: 40 });	
* &lt;/script&gt;
* @memberof ejScroller
* @instance
*/
            scrollOneStepBy: 57,
            /**		
* Specifies the button height for vertical scrollbar; for horizontal scrollbar specifies the width of the button in the scrollbar.
* @default 18
* @type {number}
* @example 
* &lt;div id="scrollcontent" style="width:900px;" &gt;
*&lt;p&gt;Model–view–controller (MVC) is a software architecture pattern which separates the
        * representation of information from the user's interaction with it.
        * The model consists of application data, business rules, logic, and functions. A view can be any
        * output representation of data, such as a chart or a diagram. Multiple views of the same data 
        * are possible, such as a bar chart for management and a tabular view for accountants. 
        *The controller mediates input, converting it to commands for the model or view.The central 
        *ideas behind MVC are code reusability and n addition to dividing the application into three 
        *kinds of components, the MVC design defines the interactions between them.</p>
        *&lt;ul&gt;
         *&lt;li&gt;
        *&lt;b&gt;A controller*&lt;/b&gt;can send commands to its associated view to change the view's presentation of the model (e.g., by scrolling through a document). 
        *It can also send commands to the model to update the model's state (e.g., editing a document).
        *&lt;/li&gt;
        *&lt;/ul&gt; 
*&lt;/div&gt; <br> 
* &lt;script&gt;
* //To set Button Size of Scroller during initialization
* 	$("#scrollcontent").ejScroller({buttonSize: 20 });	
* &lt;/script&gt;
* @memberof ejScroller
* @instance
*/
            buttonSize: 18,
            /**		
* The Scroller content and scrollbars move left with given value. 
* @default 0
* @type {number}
* @example 
* &lt;div id="scrollcontent" style="width:900px;" &gt;
*&lt;p&gt;Model–view–controller (MVC) is a software architecture pattern which separates the
        * representation of information from the user's interaction with it.
        * The model consists of application data, business rules, logic, and functions. A view can be any
        * output representation of data, such as a chart or a diagram. Multiple views of the same data 
        * are possible, such as a bar chart for management and a tabular view for accountants. 
        *The controller mediates input, converting it to commands for the model or view.The central 
        *ideas behind MVC are code reusability and n addition to dividing the application into three 
        *kinds of components, the MVC design defines the interactions between them.</p>
        *&lt;ul&gt;
         *&lt;li&gt;
        *&lt;b&gt;A controller*&lt;/b&gt;can send commands to its associated view to change the view's presentation of the model (e.g., by scrolling through a document). 
        *It can also send commands to the model to update the model's state (e.g., editing a document).
        *&lt;/li&gt;
        *&lt;/ul&gt; 
*&lt;/div&gt; <br> 
* &lt;script&gt;
* //To set Scroll left API during initialization
* 	$("#scrollcontent").ejScroller({scrollLeft: 40 });	
* &lt;/script&gt;
* @memberof ejScroller
* @instance
*/
            scrollLeft: 0,
            /**		
* The Scroller content and scrollbars move to top position with specified value. 
* @default 0
* @type {number}
* @example 
* &lt;div id="scrollcontent" style="width:900px;" &gt;
*&lt;p&gt;Model–view–controller (MVC) is a software architecture pattern which separates the
        * representation of information from the user's interaction with it.
        * The model consists of application data, business rules, logic, and functions. A view can be any
        * output representation of data, such as a chart or a diagram. Multiple views of the same data 
        * are possible, such as a bar chart for management and a tabular view for accountants. 
        *The controller mediates input, converting it to commands for the model or view.The central 
        *ideas behind MVC are code reusability and n addition to dividing the application into three 
        *kinds of components, the MVC design defines the interactions between them.</p>
        *&lt;ul&gt;
         *&lt;li&gt;
        *&lt;b&gt;A controller*&lt;/b&gt;can send commands to its associated view to change the view's presentation of the model (e.g., by scrolling through a document). 
        *It can also send commands to the model to update the model's state (e.g., editing a document).
        *&lt;/li&gt;
        *&lt;/ul&gt; 
*&lt;/div&gt; <br> 
* &lt;script&gt;
* //To set Scroll top API during initialization
* 	$("#scrollcontent").ejScroller({scrollTop: 40 });	
* &lt;/script&gt;
* @memberof ejScroller
* @instance
*/
            scrollTop: 0,
            /**		
* Indicates the target area to which scroller have to appear. 
* @default null
* @type {string}
* @example 
* &lt;div id="scrollcontent" style="width:900px;" &gt;
*&lt;p&gt;Model–view–controller (MVC) is a software architecture pattern which separates the
        * representation of information from the user's interaction with it.
        * The model consists of application data, business rules, logic, and functions. A view can be any
        * output representation of data, such as a chart or a diagram. Multiple views of the same data 
        * are possible, such as a bar chart for management and a tabular view for accountants. 
        *The controller mediates input, converting it to commands for the model or view.The central 
        *ideas behind MVC are code reusability and n addition to dividing the application into three 
        *kinds of components, the MVC design defines the interactions between them.</p>
        *&lt;ul&gt;
         *&lt;li&gt;
        *&lt;b&gt;A controller*&lt;/b&gt;can send commands to its associated view to change the view's presentation of the model (e.g., by scrolling through a document). 
        *It can also send commands to the model to update the model's state (e.g., editing a document).
        *&lt;/li&gt;
        *&lt;/ul&gt; 
*&lt;/div&gt; <br> 
* &lt;script&gt;
* //To set Scroller for the specified target panel during initialization
* 	$("#scrollcontent").ejScroller({targetPane: "contentarea" });	
* &lt;/script&gt;
* @memberof ejScroller
* @instance
*/
            targetPane:null,
            /**		
* Set the Scrollbar height and width for this API; if vertical scrollbar,apply the width else apply the height.
* @default 18
* @type {number}
* @example 
* &lt;div id="scrollcontent" style="width:900px;" &gt;
*&lt;p&gt;Model–view–controller (MVC) is a software architecture pattern which separates the
        * representation of information from the user's interaction with it.
        * The model consists of application data, business rules, logic, and functions. A view can be any
        * output representation of data, such as a chart or a diagram. Multiple views of the same data 
        * are possible, such as a bar chart for management and a tabular view for accountants. 
        *The controller mediates input, converting it to commands for the model or view.The central 
        *ideas behind MVC are code reusability and n addition to dividing the application into three 
        *kinds of components, the MVC design defines the interactions between them.</p>
        *&lt;ul&gt;
         *&lt;li&gt;
        *&lt;b&gt;A controller*&lt;/b&gt;can send commands to its associated view to change the view's presentation of the model (e.g., by scrolling through a document). 
        *It can also send commands to the model to update the model's state (e.g., editing a document).
        *&lt;/li&gt;
        *&lt;/ul&gt; 
*&lt;/div&gt; <br> 
* &lt;script&gt;
* //Enable scrollerSize on initialization 
* 	//To set scroller Size API value 
* 	$("#scrollcontent").ejScroller({scrollerSize: 20 });	
* &lt;/script&gt;
* @memberof ejScroller
* @instance
*/
            scrollerSize: 18,
            /**		
* Save current model value to browser cookies for state maintanence. While refresh the page Rating control values are retained.  
* @default false
* @type {boolean}
* @example 
* &lt;div id="scrollcontent" style="width:900px;" &gt;
*&lt;p&gt;Model–view–controller (MVC) is a software architecture pattern which separates the
        * representation of information from the user's interaction with it.
        * The model consists of application data, business rules, logic, and functions. A view can be any
        * output representation of data, such as a chart or a diagram. Multiple views of the same data 
        * are possible, such as a bar chart for management and a tabular view for accountants. 
        *The controller mediates input, converting it to commands for the model or view.The central 
        *ideas behind MVC are code reusability and n addition to dividing the application into three 
        *kinds of components, the MVC design defines the interactions between them.</p>
        *&lt;ul&gt;
         *&lt;li&gt;
        *&lt;b&gt;A controller*&lt;/b&gt;can send commands to its associated view to change the view's presentation of the model (e.g., by scrolling through a document). 
        *It can also send commands to the model to update the model's state (e.g., editing a document).
        *&lt;/li&gt;
        *&lt;/ul&gt; 
*&lt;/div&gt; <br> 
* &lt;script&gt;
* //Enable enablePersistence API on initialization
* 	$("#scrollcontent").ejScroller({enablePersistence: true });
* &lt;/script&gt;	
* @memberof ejScroller
* @instance
*/
            enablePersistence: false,
            /**		
* Indicates the Right to Left direction to scroller  
* @default undefined
* @type {boolean}
* @example 
* &lt;div id="scrollcontent" style="width:900px;" &gt;
*&lt;p&gt;Model–view–controller (MVC) is a software architecture pattern which separates the
        * representation of information from the user's interaction with it.
        * The model consists of application data, business rules, logic, and functions. A view can be any
        * output representation of data, such as a chart or a diagram. Multiple views of the same data 
        * are possible, such as a bar chart for management and a tabular view for accountants. 
        *The controller mediates input, converting it to commands for the model or view.The central 
        *ideas behind MVC are code reusability and n addition to dividing the application into three 
        *kinds of components, the MVC design defines the interactions between them.</p>
        *&lt;ul&gt;
         *&lt;li&gt;
        *&lt;b&gt;A controller*&lt;/b&gt;can send commands to its associated view to change the view's presentation of the model (e.g., by scrolling through a document). 
        *It can also send commands to the model to update the model's state (e.g., editing a document).
        *&lt;/li&gt;
        *&lt;/ul&gt; 
*&lt;/div&gt; <br> 
* &lt;script&gt;
* //Enable enableRTL API on initialization
* 	$("#scrollcontent").ejScroller({enableRTL: true });	
* &lt;/script&gt;	
* @memberof ejScroller
* @instance
*/
            enableRTL: undefined,
            /**		
* Enables or Disbale the touch Scroll  
* @default true
* @type {boolean}
* @example 

* &lt;div id="scrollcontent" style="width:900px;" &gt;
*&lt;p&gt;Model–view–controller (MVC) is a software architecture pattern which separates the
        * representation of information from the user's interaction with it.
        * The model consists of application data, business rules, logic, and functions. A view can be any
        * output representation of data, such as a chart or a diagram. Multiple views of the same data 
        * are possible, such as a bar chart for management and a tabular view for accountants. 
        *The controller mediates input, converting it to commands for the model or view.The central 
        *ideas behind MVC are code reusability and n addition to dividing the application into three 
        *kinds of components, the MVC design defines the interactions between them.</p>
        *&lt;ul&gt;
         *&lt;li&gt;
        *&lt;b&gt;A controller*&lt;/b&gt;can send commands to its associated view to change the view's presentation of the model (e.g., by scrolling through a document). 
        *It can also send commands to the model to update the model's state (e.g., editing a document).
        *&lt;/li&gt;
        *&lt;/ul&gt; 
*&lt;/div&gt; <br> 
* &lt;script&gt;
* //Disable Touch Scroll API on initialization
* 	$("#scrollcontent").ejScroller({enableTouchScroll: false });	
* &lt;/script&gt;	
* @memberof ejScroller
* @instance
*/
            enableTouchScroll: true,
/**     
* Fires when Scroller control is created.
* @event
* @name ejScroller#create	
* @param {Object} argument Event parameters from scroller     
* @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
* @param {object}  argument.model returns the scroller model
* @param {string}  argument.type returns the name of the event.
* @example 
* &lt;div id="scrollcontent" style="width:900px;" &gt;
*&lt;p&gt;Model–view–controller (MVC) is a software architecture pattern which separates the
        * representation of information from the user's interaction with it.
        * The model consists of application data, business rules, logic, and functions. A view can be any
        * output representation of data, such as a chart or a diagram. Multiple views of the same data 
        * are possible, such as a bar chart for management and a tabular view for accountants. 
        *The controller mediates input, converting it to commands for the model or view.The central 
        *ideas behind MVC are code reusability and n addition to dividing the application into three 
        *kinds of components, the MVC design defines the interactions between them.</p>
        *&lt;ul&gt;
         *&lt;li&gt;
        *&lt;b&gt;A controller*&lt;/b&gt;can send commands to its associated view to change the view's presentation of the model (e.g., by scrolling through a document). 
        *It can also send commands to the model to update the model's state (e.g., editing a document).
        *&lt;/li&gt;
        *&lt;/ul&gt; 
*&lt;/div&gt; <br> 
* &lt;script&gt;
* //create event for scroller
* $("#scrollcontent").ejScroller({
*    create: function (args) {}
* });   
* &lt;/script&gt;	  
* @memberof ejScroller
* @instance
*/
            create: null,
            /**     
* Fires when Scroller control is destroyed.
* @event
* @name ejScroller#destroy	
* @param {Object} argument Event parameters from scroller     
* @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
* @param {object}  argument.model returns the scroller model
* @param {string}  argument.type returns the name of the event.
* @example 
* &lt;div id="scrollcontent" style="width:900px;" &gt;
*&lt;p&gt;Model–view–controller (MVC) is a software architecture pattern which separates the
        * representation of information from the user's interaction with it.
        * The model consists of application data, business rules, logic, and functions. A view can be any
        * output representation of data, such as a chart or a diagram. Multiple views of the same data 
        * are possible, such as a bar chart for management and a tabular view for accountants. 
        *The controller mediates input, converting it to commands for the model or view.The central 
        *ideas behind MVC are code reusability and n addition to dividing the application into three 
        *kinds of components, the MVC design defines the interactions between them.</p>
        *&lt;ul&gt;
         *&lt;li&gt;
        *&lt;b&gt;A controller*&lt;/b&gt;can send commands to its associated view to change the view's presentation of the model (e.g., by scrolling through a document). 
        *It can also send commands to the model to update the model's state (e.g., editing a document).
        *&lt;/li&gt;
        *&lt;/ul&gt; 
*&lt;/div&gt; <br> 
* &lt;script&gt;
* //destroy event for scroller
* $("#scrollcontent").ejScroller({
*    destroy: function (args) {}
* });  
* &lt;/script&gt;	    
* @memberof ejScroller
* @instance
*/
            destroy: null
        },
        validTags: ["div"],
        type: "transclude",
        /**
* Specify the data types for default properties 
* @private
*/
        dataTypes: {
            buttonSize: "number",
            scrollOneStepBy: "number"
        },
        observables: ["scrollTop", "scrollLeft"],
        scrollTop: ej.util.valueFunction("scrollTop"),
        scrollLeft: ej.util.valueFunction("scrollLeft"),
        /**
* Specifies the keys used for keyboard navigation. 
* @private
*/
        keyConfigs: {
            up: "38",
            down: "40",
            left: "37",
            right: "39",
            pageUp: "33",
            pageDown: "34",
            pageLeft: "ctrl+37",
            pageRight: "ctrl+39"
        },
        /**
* Specify the content area for which the scrollerbar is specified.
* @private
*/
        content: function () {
            if (!this._content || !this._content.length || !this._content[0].offsetParent)
                this._content = this.element.children("div").first().addClass("e-content");

            return this._content;
        },
        _setFirst: true,
        /**
* Create the scroller widget
* @private
*/
        _init: function () {
            this.element.addClass("e-widget");
            this.content();
            this._ensureScrollers();
            if (this.model.enableRTL === undefined) {
                this.model.enableRTL = this.element.css("direction") === "rtl";
            }
            if (this.model.enableRTL) {
                this.element.addClass("e-rtl");
            }
            if (this.scrollLeft())
                this.content().scrollLeft(this.scrollLeft());
            if (this.scrollTop())
                this.content().scrollTop(this.scrollTop());

            this._on(this.content(), "scroll", this._scroll);
            this.model.targetPane != null && this._on(this.content().find(this.model.targetPane), "scroll", this._scroll);
            if (this.model.autoHide) {
                this.element.addClass("e-autohide");
                this._on(this.element, "mouseenter mouseleave", this._scrollerHover);
            }
        },
        /**
* Ensures the scrollbar height and width.
* @private
*/
        _ensureScrollers: function () {
            var jqVersion = $.fn.jquery, height, width;
            if (this.model.height) {
                this.element.height(this.model.height);
            }
            if (this.model.width) {
                this.element.width(this.model.width);
            }

            this._off(this.content(), "mousedown touchstart");
			if(this.content().length>0){
            if (this.isVScroll()) {
                if (!this._v) {
                    var d1 = this._createScroller("Height", "Y", "Top", "e-v", this.isHScroll());

                    if (this.model.enableTouchScroll)
                        this._on(this.content(), "mousedown touchstart", { d: d1 }, this._mouseDownOnContent);
                }
            } else {
                this._v = null;
                this.element.find(".e-vscroll").remove();
            }
            if (this.isHScroll()) {
                if (!this._h) {
                    var d2 = this._createScroller("Width", "X", "Left", "e-h", this.isVScroll());

                    if (this.model.enableTouchScroll)
                        this._on(this.content(), "mousedown touchstart", { d: d2 }, this._mouseDownOnContent);
                }
            } else {
                this.element.find(".e-hscroll").remove();
                this._h = null;
            }

            if (!this._v && !this._h)
                this.content().css({ width: "auto", height: "auto" });

            if (this.element.find(".e-hscroll").length > 0)
                !this.element.hasClass("e-hscrollbar") && this.element.addClass("e-hscrollbar");
            else {
                if (this.element.hasClass("e-hscrollbar")) {
                    this.element.removeClass("e-hscrollbar");
                    if (this._v) {
                        this.content().outerHeight(this.content().outerHeight() - 1);
                    }
                }
            }

            if (this.isHScroll() || this.isVScroll()) {
                this.content().addClass("e-content");
                jqVersion === "1.7.1" || jqVersion === "1.7.2" ? (height = "height", width = "width") : (height = "outerHeight", width = "outerWidth");
                this.content()[height](this.element.height() - (this.isHScroll() && !this.model.autoHide ? this.model.scrollerSize : 0));
                this.content()[width](this.element.width() - (this.isVScroll() && !this.model.autoHide ? this.model.scrollerSize : 0));
            } else
                this.content().removeClass("e-content");
			}
        },
        /**
* Returns horizontal scrollbar is shown or not.		
* @return Boolean value
* @example 
* &lt;div id="scrollcontent" style="width:900px;" &gt;
*&lt;p&gt;Model–view–controller (MVC) is a software architecture pattern which separates the
        * representation of information from the user's interaction with it.
        * The model consists of application data, business rules, logic, and functions. A view can be any
        * output representation of data, such as a chart or a diagram. Multiple views of the same data 
        * are possible, such as a bar chart for management and a tabular view for accountants. 
        *The controller mediates input, converting it to commands for the model or view.The central 
        *ideas behind MVC are code reusability and n addition to dividing the application into three 
        *kinds of components, the MVC design defines the interactions between them.</p>
        *&lt;ul&gt;
         *&lt;li&gt;
        *&lt;b&gt;A controller*&lt;/b&gt;can send commands to its associated view to change the view's presentation of the model (e.g., by scrolling through a document). 
        *It can also send commands to the model to update the model's state (e.g., editing a document).
        *&lt;/li&gt;
        *&lt;/ul&gt; 
*&lt;/div&gt; <br> 
* &lt;script&gt;

* // To check horizontal scrollbar is rendered or not
* var scrollerObj  = $("#scrollcontent").data("ejScroller");
* scrollerObj.isHScroll(); // Returns horizontal scrollbar is shown or not.
* &lt;/script&gt;
* @example 
* &lt;div id="scrollcontent" style="width:900px;" &gt;
*&lt;p&gt;Model–view–controller (MVC) is a software architecture pattern which separates the
        * representation of information from the user's interaction with it.
        * The model consists of application data, business rules, logic, and functions. A view can be any
        * output representation of data, such as a chart or a diagram. Multiple views of the same data 
        * are possible, such as a bar chart for management and a tabular view for accountants. 
        *The controller mediates input, converting it to commands for the model or view.The central 
        *ideas behind MVC are code reusability and n addition to dividing the application into three 
        *kinds of components, the MVC design defines the interactions between them.</p>
        *&lt;ul&gt;
         *&lt;li&gt;
        *&lt;b&gt;A controller*&lt;/b&gt;can send commands to its associated view to change the view's presentation of the model (e.g., by scrolling through a document). 
        *It can also send commands to the model to update the model's state (e.g., editing a document).
        *&lt;/li&gt;
        *&lt;/ul&gt; 
*&lt;/div&gt; <br> 
* &lt;script&gt;
* // To check horizontal scrollbar is rendered or not
* $("#scrollcontent").ejScroller("isHScroll");	
* &lt;/script&gt;
* @memberof ejScroller
* @instance
*/
        isHScroll: function () {
            if (this.model.width > 0) {
                var $paneObject = this.content().find(this.model.targetPane);
                if (this.model.targetPane != null && $paneObject.length)
                    return ($paneObject[0].scrollWidth + $paneObject.siblings().width()) > this.model.width;
                else
                    return this.content()[0].scrollWidth > this.model.width;
            }
            return false;
        },
        /**
* Returns vertical scrollbar is shown or not.		
* @return Boolean value
* @example 
* &lt;div id="scrollcontent" style="width:900px;" &gt;
*&lt;p&gt;Model–view–controller (MVC) is a software architecture pattern which separates the
        * representation of information from the user's interaction with it.
        * The model consists of application data, business rules, logic, and functions. A view can be any
        * output representation of data, such as a chart or a diagram. Multiple views of the same data 
        * are possible, such as a bar chart for management and a tabular view for accountants. 
        *The controller mediates input, converting it to commands for the model or view.The central 
        *ideas behind MVC are code reusability and n addition to dividing the application into three 
        *kinds of components, the MVC design defines the interactions between them.</p>
        *&lt;ul&gt;
         *&lt;li&gt;
        *&lt;b&gt;A controller*&lt;/b&gt;can send commands to its associated view to change the view's presentation of the model (e.g., by scrolling through a document). 
        *It can also send commands to the model to update the model's state (e.g., editing a document).
        *&lt;/li&gt;
        *&lt;/ul&gt; 
*&lt;/div&gt; <br> 
* &lt;script&gt;
* $('#scrollcontent').ejScroller(); 	
* // To check vertical scrollbar is rendered or not
* var scrollerObj  = $("#scrollcontent").data("ejScroller");
* scrollerObj.isVScroll(); // Returns vertical scrollbar is shown or not.
* &lt;/script&gt;
* @example 
* &lt;div id="scrollcontent" style="width:900px;"&gt;
*&lt;p&gt;Model–view–controller (MVC) is a software architecture pattern which separates the
        * representation of information from the user's interaction with it.
        * The model consists of application data, business rules, logic, and functions. A view can be any
        * output representation of data, such as a chart or a diagram. Multiple views of the same data 
        * are possible, such as a bar chart for management and a tabular view for accountants. 
        *The controller mediates input, converting it to commands for the model or view.The central 
        *ideas behind MVC are code reusability and n addition to dividing the application into three 
        *kinds of components, the MVC design defines the interactions between them.</p>
        *&lt;ul&gt;
         *&lt;li&gt;
        *&lt;b&gt;A controller*&lt;/b&gt;can send commands to its associated view to change the view's presentation of the model (e.g., by scrolling through a document). 
        *It can also send commands to the model to update the model's state (e.g., editing a document).
        *&lt;/li&gt;
        *&lt;/ul&gt; 
*&lt;/div&gt; <br> 
* &lt;script&gt;
* $('#scrollcontent').ejScroller(); 	
* // To check vertical scrollbar is rendered or not
* $("#scrollcontent").ejScrollbar("isVScroll");	
* &lt;/script&gt;
* @memberof ejScroller
* @instance
*/
        isVScroll: function () {
            return this.model.height > 0 && this.content()[0].scrollHeight > this.model.height;
        },
        /**
* To configure the properties at runtime using SetModel		
* @private
*/
        _setModel: function (option) {
            for (var prop in option) {
                if (prop === "enableRTL") {
                    if (option[prop])
                        this.element.addClass("e-rtl");
                    else
                        this.element.removeClass("e-rtl");
                } else if (prop === "scrollLeft") {
                    this.content().scrollLeft(this.scrollLeft());
                } else if (prop === "scrollTop") {
                    this.content().scrollTop(this.scrollTop());
                } else if (prop === "touchScroll") {
                    if (!this.model.enableTouchScroll)
                        this._off(this.content(), "mousedown touchstart");
                    else {
                        if(this._v)
                            this._on(this.content(), "mousedown touchstart", { d: this._v }, this._mouseDownOnContent);
                        if(this._h)
                            this._on(this.content(), "mousedown touchstart", { d: this._h }, this._mouseDownOnContent);
                    }
                } else {
                    this.scrollTop(0);
                    this.scrollLeft(0);
                    this.refresh();
                    break;
                }
            }
        },
        /**
* Creates the scroller in on demand time
* @private
*/
        _createScroller: function (dimension, xy, position, css, isOtherScroll) {
            var temp = css == "e-v" ? "v" : "h", height;
            var d = this["_" + temp] = {};
            var jqVersion = $.fn.jquery;
            d.dimension = dimension;
            d.xy = xy;
            d.position = position;
            d.css = css;
            d.isOtherScroll = isOtherScroll;
            d.uDimension = dimension;

            var dim = dimension === "Width" ? "Height" : "Width";
            jqVersion === "1.7.1" || jqVersion === "1.7.2" ? height = dim.toLowerCase() : height = "outer" + dim;
            this.content()[height](this.element[dim.toLowerCase()]() - (this.isVScroll() && !this.model.autoHide ? this.model.scrollerSize : 0));
            this._calculateLayout(d);
            this._createLayout(d);
            var buttons = this[d.main].find(".e-button");

            this._off(buttons, "mousedown")
                ._on(buttons, "mousedown", { d: d, step: 1 }, this._spaceMouseDown);
            this._off(css === "e-v" ? this.element : this[d.scroll], "mousewheel DOMMouseScroll")
                ._on(css === "e-v" ? this.element : this[d.scroll], "mousewheel DOMMouseScroll", { d: d }, this._mouseWheel);
            this._off(this[d.scroll], "mousedown")
                ._on(this[d.scroll], "mousedown", { d: d }, this._spaceMouseDown);
            this._off(this[d.handler], "mousedown touchstart")
                ._on(this[d.handler], "mousedown touchstart", { d: d }, this._mouseDown);
            
            return d;
        },
        /**
* Creates the layout for the scrollbars to be rendered.	
* @private
*/
        _createLayout: function (d) {
            var divString = "<div class='" + d.css + "{0}' style='" + d.dimension + ":{1}px'>{2}</div>";
            var jqVersion = $.fn.jquery;
            var lit = {}, height;
            lit[d.dimension] = d.modelDim;

                var el = ej.buildTag(
                    "div." + d.css + "scroll",
                        String.format(divString, "up e-icon e-button", d.buttonSize) +
                        String.format(divString, "handlespace", d.handleSpace,
                            String.format(divString, "handle", d.handle)) +
                        String.format(divString, "down e-icon e-button", d.buttonSize),
                    lit
                );

            if (d.css == "e-v" && $(this.element.find(">.e-content")).length > 0) $(this.element.find(">.e-content")).after(el);
            else this.element.append(el);
			if(!this.isVScroll())this.element['height'](this.content()['height']()+this.model.scrollerSize);
            jqVersion === "1.7.1" || jqVersion === "1.7.2" ? height = d.uDimension.toLowerCase() : height = "outer" + d.uDimension; 
            this[d.handler] = this.element.find("." + d.handler)[height](d.handle);
            this[d.scroll] = this[d.handler].parent();
            this[d.main] = this[d.scroll].parent();
            this[d.main].find(".e-button")["outer" + d.uDimension](d.buttonSize);
            if (this.model.autoHide)
                this[d.main].hide();
        },
        /**
* Calculates the scrollbar layout based on the values specified for scrollbr to appear
* @private
*/
        _calculateLayout: function (d) {
            d.enableRTL = this.model.enableRTL === undefined ? this.element.css("direction") === "rtl" : this.model.enableRTL;
            d.enableRTL = d.css === "e-h" ? d.enableRTL : false;

            d.scrollDim = "scroll" + d.dimension;
            d.lPosition = d.position.toLowerCase();
            d.clientXy = "page" + d.xy;
            d.scrollVal = "scroll" + d.position;
            d.scrollOneStepBy = this.model.scrollOneStepBy;
            d.modelDim = this.model[(d.dimension = d.dimension.toLowerCase())] - (d.isOtherScroll ? this.model.scrollerSize : 0);
            d.handler = d.css + "handle";
            d.buttonSize = this.model.buttonSize;
            d.main = d.css + "scroll";
            d.scroll = d.css + "ScrollSpace";
            d.handleSpace = d.modelDim - 2 * d.buttonSize;

            var $pane = this.content().find(this.model.targetPane);
            if (this.model.targetPane != null && $pane.length && d.dimension != "height")
                d.sScrollDim = $pane[0][d.scrollDim] + $pane.parent().width() - $pane.width();
            else
                d.sScrollDim = this.content()[0][d.scrollDim];
            d.scrollable = d.sScrollDim - d.modelDim;
            d.handle = Math.floor(d.modelDim / d.sScrollDim * d.handleSpace);
            if (d.handle < 20) d.handle = 20;
            d.onePx = d.scrollable / (d.handleSpace - d.handle);
            d.fromScroller = false;
            d.up = true;
            d.vInterval = undefined;
            d.scrollable *= d.enableRTL ? -1 : 1;
        },
        /**
* Updates scrollbar layout based on ondemand values
* @private
*/
        _updateLayout: function (d) {
            this.element.find("." + d.css + "scroll").css(d.dimension, d.modelDim + "px")
                .find(".e-button").css(d.dimension, this.model.buttonSize).end()
                .find("." + d.css + "handlespace").css(d.dimension, d.handleSpace + "px")
                .find("." + d.css + "handle").css(d.dimension, d.handle + "px");
        },
        /**
* User refreshes the Scroller control at any time.		
* @return jQuery
* @example 
* &lt;div id="scrollcontent" style="width:900px;" &gt;
*&lt;p&gt;Model–view–controller (MVC) is a software architecture pattern which separates the
        * representation of information from the user's interaction with it.
        * The model consists of application data, business rules, logic, and functions. A view can be any
        * output representation of data, such as a chart or a diagram. Multiple views of the same data 
        * are possible, such as a bar chart for management and a tabular view for accountants. 
        *The controller mediates input, converting it to commands for the model or view.The central 
        *ideas behind MVC are code reusability and n addition to dividing the application into three 
        *kinds of components, the MVC design defines the interactions between them.</p>
        *&lt;ul&gt;
         *&lt;li&gt;
        *&lt;b&gt;A controller*&lt;/b&gt;can send commands to its associated view to change the view's presentation of the model (e.g., by scrolling through a document). 
        *It can also send commands to the model to update the model's state (e.g., editing a document).
        *&lt;/li&gt;
        *&lt;/ul&gt; 
*&lt;/div&gt; <br> 
* &lt;script&gt;
* $('#scrollcontent').ejScroller(); 	
* // To refresh the Scroller control at any time.
* var scrollerObj  = $("#scrollcontent").data("ejScroller");
* scrollerObj.refresh(); // refreshes the Scroller control at any time
* &lt;/script&gt;
* @example 
* &lt;div id="scrollcontent" style="width:900px;" &gt;
*&lt;p&gt;Model–view–controller (MVC) is a software architecture pattern which separates the
        * representation of information from the user's interaction with it.
        * The model consists of application data, business rules, logic, and functions. A view can be any
        * output representation of data, such as a chart or a diagram. Multiple views of the same data 
        * are possible, such as a bar chart for management and a tabular view for accountants. 
        *The controller mediates input, converting it to commands for the model or view.The central 
        *ideas behind MVC are code reusability and n addition to dividing the application into three 
        *kinds of components, the MVC design defines the interactions between them.</p>
        *&lt;ul&gt;
         *&lt;li&gt;
        *&lt;b&gt;A controller*&lt;/b&gt;can send commands to its associated view to change the view's presentation of the model (e.g., by scrolling through a document). 
        *It can also send commands to the model to update the model's state (e.g., editing a document).
        *&lt;/li&gt;
        *&lt;/ul&gt; 
*&lt;/div&gt; <br> 
* &lt;script&gt;
* $('#scrollcontent').ejScroller(); 	
* // Refresh the scroller control
* $("#scrollcontent").ejScroller("refresh");	
* &lt;/script&gt;
* @memberof ejScroller
* @instance
*/
        refresh: function () {
            this._ensureScrollers();

            if (this.scrollLeft())
                this.content().scrollLeft(this.scrollLeft());
            if ((this.scrollTop() && this._v == null) || (this._v != null && !this._v.skipChange))
                this.content().scrollTop(this.scrollTop());

            if (this._v) {
                this._v.dimension = "Height";
                this._v.isOtherScroll = this.isVScroll() && this.isHScroll() ? true: false;
                this.isVScroll() && !this._calculateLayout(this._v) && this._updateLayout(this._v);
            }
            if (this._h) {
                this._h.dimension = "Width";
                this._h.isOtherScroll = this.isVScroll() && this.isHScroll() ? true : false;
                this.isHScroll() && !this._calculateLayout(this._h) && this._updateLayout(this._h);
            }
            this.model.targetPane != null && this._on(this.content().find(this.model.targetPane), "scroll", this._scroll);
        },
        /**
* Section for handling scrollbar based on keypressed
* @private
*/
        _keyPressed: function (action, target) {
            if (["input", "select", "textarea"].indexOf(target.tagName.toLowerCase()) !== -1)
                return true;

            var d, iChar;

            if (["up", "down", "pageUp", "pageDown"].indexOf(action) !== -1) {
                d = this._v;
                iChar = "o";
            } else if (["left", "right", "pageLeft", "pageRight"].indexOf(action) !== -1) {
                d = this._h;
                iChar = "i";
            } else return true;
            if (!d) return true;

            return !this._changeTop(d, (action.indexOf(iChar) < 0 ? -1 : 1) * (action[0] !== "p" ? 1 : 3) * d.scrollOneStepBy, "key");
        },
        /**
* Scroller moves to given pixel in Y (top) position.		
* @return jQuery
* @example 
* &lt;div id="scrollcontent" style="width:900px;" &gt;
*&lt;p&gt;Model–view–controller (MVC) is a software architecture pattern which separates the
        * representation of information from the user's interaction with it.
        * The model consists of application data, business rules, logic, and functions. A view can be any
        * output representation of data, such as a chart or a diagram. Multiple views of the same data 
        * are possible, such as a bar chart for management and a tabular view for accountants. 
        *The controller mediates input, converting it to commands for the model or view.The central 
        *ideas behind MVC are code reusability and n addition to dividing the application into three 
        *kinds of components, the MVC design defines the interactions between them.</p>
        *&lt;ul&gt;
         *&lt;li&gt;
        *&lt;b&gt;A controller*&lt;/b&gt;can send commands to its associated view to change the view's presentation of the model (e.g., by scrolling through a document). 
        *It can also send commands to the model to update the model's state (e.g., editing a document).
        *&lt;/li&gt;
        *&lt;/ul&gt; 
*&lt;/div&gt; <br> 
* &lt;script&gt;
* $('#scrollcontent').ejScroller(); 	
* // Moves scroller to given pixel in Y (top) position.
* var scrollerObj  = $("#scrollcontent").data("ejScroller");
* scrollerObj.scrollY("scrollY", 25, true); // call scrollY method.
* &lt;/script&gt;
* @example 
* &lt;div id="scrollcontent" style="width:900px;" &gt;
*&lt;p&gt;Model–view–controller (MVC) is a software architecture pattern which separates the
        * representation of information from the user's interaction with it.
        * The model consists of application data, business rules, logic, and functions. A view can be any
        * output representation of data, such as a chart or a diagram. Multiple views of the same data 
        * are possible, such as a bar chart for management and a tabular view for accountants. 
        *The controller mediates input, converting it to commands for the model or view.The central 
        *ideas behind MVC are code reusability and n addition to dividing the application into three 
        *kinds of components, the MVC design defines the interactions between them.</p>
        *&lt;ul&gt;
         *&lt;li&gt;
        *&lt;b&gt;A controller*&lt;/b&gt;can send commands to its associated view to change the view's presentation of the model (e.g., by scrolling through a document). 
        *It can also send commands to the model to update the model's state (e.g., editing a document).
        *&lt;/li&gt;
        *&lt;/ul&gt; 
*&lt;/div&gt; <br> 
* &lt;script&gt;
* $('#scrollcontent').ejScroller(); 	
* //Moves scroller to given pixel in Y (top) position.
* $("#scrollcontent").ejScroller("scrollY", 25, true);	
* &lt;/script&gt;
* @memberof ejScroller
* @instance
*/
        scrollY: function (pixel, noAnimation, source) {
            if (noAnimation) {
                if (this._trigger("scroll", { source: source || "custom", scrollData: this._v, scrollTop: pixel }))
                    return;
                this.scrollTop(pixel);
                this.content().scrollTop(pixel);
                return;
            }
            this.content().stop().animate({
                scrollTop: pixel
            }, 100, 'linear');
        },
        /**
* Scroller moves to given pixel in X (left) position.		
* @return jQuery
* @example 
* &lt;div id="scrollcontent" style="width:900px;" &gt;
*&lt;p&gt;Model–view–controller (MVC) is a software architecture pattern which separates the
        * representation of information from the user's interaction with it.
        * The model consists of application data, business rules, logic, and functions. A view can be any
        * output representation of data, such as a chart or a diagram. Multiple views of the same data 
        * are possible, such as a bar chart for management and a tabular view for accountants. 
        *The controller mediates input, converting it to commands for the model or view.The central 
        *ideas behind MVC are code reusability and n addition to dividing the application into three 
        *kinds of components, the MVC design defines the interactions between them.</p>
        *&lt;ul&gt;
         *&lt;li&gt;
        *&lt;b&gt;A controller*&lt;/b&gt;can send commands to its associated view to change the view's presentation of the model (e.g., by scrolling through a document). 
        *It can also send commands to the model to update the model's state (e.g., editing a document).
        *&lt;/li&gt;
        *&lt;/ul&gt; 
		*&lt;/div&gt; <br> 
		* &lt;script&gt;
		* $('#scrollcontent').ejScroller(); 	
		* // Moves scroller to given pixel in X (left) position.
		* var scrollerObj  = $("#scrollcontent").data("ejScroller");
		* scrollerObj.scrollX("scrollX", 25, true); // call scrollX method.
		* &lt;/script&gt;
		* @example 
		* &lt;div id="scrollcontent" style="width:900px;" &gt;
		*&lt;p&gt;Model–view–controller (MVC) is a software architecture pattern which separates the
        * representation of information from the user's interaction with it.
        * The model consists of application data, business rules, logic, and functions. A view can be any
        * output representation of data, such as a chart or a diagram. Multiple views of the same data 
        * are possible, such as a bar chart for management and a tabular view for accountants. 
        *The controller mediates input, converting it to commands for the model or view.The central 
        *ideas behind MVC are code reusability and n addition to dividing the application into three 
        *kinds of components, the MVC design defines the interactions between them.</p>
        *&lt;ul&gt;
         *&lt;li&gt;
        *&lt;b&gt;A controller*&lt;/b&gt;can send commands to its associated view to change the view's presentation of the model (e.g., by scrolling through a document). 
        *It can also send commands to the model to update the model's state (e.g., editing a document).
        *&lt;/li&gt;
        *&lt;/ul&gt; 
		*&lt;/div&gt; <br> 
		* &lt;script&gt;
		* $('#scrollcontent').ejScroller(); 	
		* //Moves scroller to given pixel in X (left) position.
		* $("#scrollcontent").ejScroller("scrollX", 25, true);	
		* &lt;/script&gt;
		* @memberof ejScroller
		* @instance
		*/
        scrollX: function (pixel, noAnimation, source) {
            if (noAnimation) {
                if (this._trigger("scroll", { source: source || "custom", scrollData: this._h, scrollLeft: pixel }))
                    return;
                this.scrollLeft(pixel);
                if (this.model.targetPane != null)
                    this.content().find(this.model.targetPane).scrollLeft(pixel);
                else
                    this.content().scrollLeft(pixel);
                return;
            }
            this.content().stop().animate({
                scrollLeft: pixel
            }, 100, 'linear');
        },
        /**
* Updates top value based on scroller movement
* @private
*/
        _changeTop: function (d, step, source) {
            var start = this.model.targetPane != null && d.dimension != "height" ? this.content().find(this.model.targetPane)[d.scrollVal]() : this.content()[d.scrollVal](), t;

            if (d.dimension == "height" && start == 0)
                start = this.scrollTop() != 0 ? this.scrollTop() : 0;
            t = start + step;
            if (!d.enableRTL ? t > d.scrollable : t < d.scrollable) t = d.scrollable;
            if (!d.enableRTL ? t < 0 : t > 0) t = 0;

            if (t !== start) {
                this["scroll" + d.xy](t, true, source);
            }

            return t !== start;
        },
        /**
* To handle scroll based on movement via mouse wheel
* @private
*/
        _mouseWheel: function (e) {
            if (!e.data) return;

            var delta = 0, data = e.data.d, ori = e;
            e = e.originalEvent;
            if (e.wheelDelta) {
                delta = -e.wheelDelta / 120;
                if (window.opera) {
                    if (parseFloat(window.opera.version, 10) < 10)
                        delta = -delta;
                }
            } else if (e.detail) delta = e.detail / 3;

            if (!delta) return;

            if (this._changeTop(data, delta * data.scrollOneStepBy, "wheel")) {
                e.preventDefault ? e.preventDefault() : ori.preventDefault();
                if (data.css === "e-h") {
                    ori.stopImmediatePropagation();
                    ori.stopPropagation();
                }
            }
        },
        _scrollerHover: function (e) {
            if (e.type == "mouseenter" && !this.content().siblings().is(":visible")) 
                this.content().siblings().css("display","block");
            else if(e.type == "mouseleave") 
                this.content().siblings().hide();
        },
        /**
* To handle mouse up movement
* @private
*/
        _mouseUp: function (e) {
            if (!e.data) return;

            var d = e.data.d;
            e.type === "mouseup" && this[d.handler].removeClass("e-active");
            if (e.type === "mouseup" || e.type === "touchend" || (!e.toElement && !e.relatedTarget)) {
                this._off($(document), "mousemove touchmove");
                $(document).off("mouseout mouseup touchend", ej.proxy(this._mouseUp, this));
                d.fromScroller = false;
                this[d.scroll].off("mousemove");
                this[d.handler].off("mousemove").css("transition", "");

                if (e.data.source === "thumb" && !ej.isNullOrUndefined(this.model)) {
                    $.when(this.content()).done(ej.proxy(function () {
                        this._trigger("thumbEnd", { originalEvent: e, scrollData: d });
                    }, this));
                }
            }
            d.up = true;
        },
        /**
* To handle mouse down movement on content
* @private
*/
        _mouseDownOnContent: function (down) {
            var d = down.data.d;
			if(this._trigger("thumbStart", { originalEvent: down, scrollData: d }))
				return;
            
            if( down.which ==3 && down.button == 2 ) return;
            d.fromScroller = true;
            this[d.handler].css("transition", "none");

            var prevY = null, skip = 1, min = 5, direction;

            this._on($(document), "mousemove touchmove", (function (move) {
                move.preventDefault();
                var pageXY = move.type == "mousemove" ? move[d.clientXy] : move.originalEvent.changedTouches[0][d.clientXy];

                if (prevY && pageXY !== prevY) {
                    var diff = pageXY - prevY, sTop = this.model[d.scrollVal] - (diff * d.onePx / min);
                    
                    if (skip == 1 && Math.abs(diff) > min) {
                        direction = d.position;
                        skip = 0;
                    }
                    if (skip == 0) prevY = pageXY;

                    if (sTop >= 0 && sTop <= d.scrollable && direction === d.position) {
                        this["scroll" + d.xy](sTop, true, "thumb");
                        this[d.handler].css(d.position.toLowerCase(), sTop / d.onePx + "px");
                        this.content().css("cursor", "pointer");
                        this._trigger("thumbMove", { originalEvent: move, scrollData: d });
                    }
                }
                if (prevY == null) prevY = pageXY;
            }));

            $(document).one("mouseup touchend", { d: d, source: "thumb" }, ej.proxy(this._mouseUp, this));
            $(document).mouseout({ d: d, source: "thumb" }, ej.proxy(this._mouseUp, this));
        },
        /**
* To handle mouse down movement
* @private
*/
        _mouseDown: function (down) {
            //if (down.which !== 1) return;
            var d = down.data.d;

            down.preventDefault();
            d.fromScroller = true;
            this[d.handler].css("transition", "none");

            var prevY, top = parseInt(this[d.handler].css(d.lPosition)) || 0;

            this._on($(document), "mousemove touchmove", (function (move) {
                move.preventDefault();
                var skip = 1;
                var pageXY = move.type == "mousemove" ? move[d.clientXy] : move.originalEvent.changedTouches[0][d.clientXy];
                if (prevY && pageXY !== prevY) {
                    top += (pageXY - prevY);

                    if (d.enableRTL ? top > 0 : top < 0) skip = top = 0;

                    if ((top * (d.enableRTL ? -1 : 1)) + d.handle >= d.handleSpace)
                        skip = top = (d.handleSpace - d.handle) * (d.enableRTL ? -1 : 1);

                    this["scroll" + d.xy](Math.ceil(top * d.onePx), true, "thumb");
                    this[d.handler].css(d.position.toLowerCase(), top + "px");
                    this.content().css("cursor", "pointer");
                    this._trigger("thumbMove", { originalEvent: move, scrollData: d });
                }

                if (skip === 1)
                    prevY = pageXY;
            }));

            this._trigger("thumbStart", { originalEvent: down, scrollData: d });

            $(document).one("mouseup touchend", { d: d, source: "thumb" }, ej.proxy(this._mouseUp, this));
            $(document).mouseout({ d: d, source: "thumb" }, ej.proxy(this._mouseUp, this));
        },
        /**
* To handle mouse down based on the interval 
* @private
*/
        _spaceMouseDown: function (e) {
            if (!e.data) return;

            var d = e.data.d;
            
            if (e.which !== 1 || e.target === this[d.handler][0]) return;

            var step = (e.data.step || 3) * d.scrollOneStepBy, hTop = e.data.top || this[d.handler].offset()[d.lPosition];

            e[d.clientXy] = e[d.clientXy] || 0;

            if (e[d.clientXy] < hTop) step *= -1;

            this._changeTop(d, step, step === 3 ? "track" : "button");

            if (e.data.step !== 1) {
                this[d.scroll].mousemove(function () {
                    d.up = true;
                });
            }

            d.up = false;
            d.vInterval = setInterval(ej.proxy(function () {

                if (step < 0 ? hTop + (step / d.onePx) < e[d.clientXy] : hTop + d.handle + (step / d.onePx) > e[d.clientXy])
                    d.up = true;

                if (d.up) {
                    clearInterval(d.vInterval);
                    return;
                }

                this._changeTop(d, step, step === 3 ? "track" : "button");

                hTop = e.data.top || this[d.handler].offset()[d.lPosition];
            }, this), 150);

            $(document).one("mouseup", { d: d }, ej.proxy(this._mouseUp, this));
            $(document).mouseout({ d: d }, ej.proxy(this._mouseUp, this));
        },
        /**
* To handle scroll movement
* @private
*/
        _scroll: function (e) {
            var dS = [this._v, this._h];

            for (var i = 0; i < 2; i++) {
                var d = dS[i];
                if (!d || d.skipChange) continue;

                if (this.model.targetPane != null && i == 1)
                    d.sTop = this.content().find(this.model.targetPane)[0][d.scrollVal];
                else
                    d.sTop = e.target[d.scrollVal];
                this[d.scrollVal](d.sTop);
                if (d.fromScroller) return;
                this[d.handler].css(d.lPosition, (d.sTop / d.onePx) + "px");
            }
        },
        /**
* Changes the vertical handler position	
* @private
*/
        _changevHandlerPosition: function (top) {
            top = this._v != null && top >= this._v.scrollable ? this._v.scrollable : top;
            if (this._v != null && top >= 0 && top <= this._v.scrollable)
                this[this._v.handler].css(this._v.lPosition, (top / this._v.onePx) + "px");
        },
        /**
* Changes the horizontal handler position	
* @private
*/
        _changehHandlerPosition: function (left) {
            left = this._h != null && left >= this._h.scrollable ? this._h.scrollable : left;
            if (this._h != null && left >= 0 && left <= this._h.scrollable)
                this[this._h.handler].css(this._h.lPosition, (left / this._h.onePx) + "px");
        },
        /**
		* destroy the Scroller control
		* all events bound using this._on will be unbind automatically and bring the control to pre-init state.
		* @alias destroy
		* @return jQuery
		* @example 
		* &lt;div id="scrollcontent" style="width:900px;" &gt;
		*&lt;p&gt;Model–view–controller (MVC) is a software architecture pattern which separates the
        * representation of information from the user's interaction with it.
        * The model consists of application data, business rules, logic, and functions. A view can be any
        * output representation of data, such as a chart or a diagram. Multiple views of the same data 
        * are possible, such as a bar chart for management and a tabular view for accountants. 
        *The controller mediates input, converting it to commands for the model or view.The central 
        *ideas behind MVC are code reusability and n addition to dividing the application into three 
        *kinds of components, the MVC design defines the interactions between them.</p>
        *&lt;ul&gt;&lt;li&gt;
        *&lt;b&gt;A controller&lt;/b&gt;can send commands to its associated view to change the view's presentation of the model (e.g., by scrolling through a document). 
        *It can also send commands to the model to update the model's state (e.g., editing a document).
        *&lt;/li&gt;
		*&lt;/ul&gt; 
		*&lt;/div&gt; <br> 
		* &lt;script&gt;
		* $('#scrollcontent').ejScroller(); 	
		* // Destroy Scroller
		* var scrollerObj  = $("#scrollcontent").data("ejScroller");
		* scrollerObj.destroy(); // destroy the scroller
		* &lt;/script&gt;
		* @example 
		* &lt;div id="scrollcontent" style="width:900px;" &gt;
		*&lt;p&gt;Model–view–controller (MVC) is a software architecture pattern which separates the
        * representation of information from the user's interaction with it.
        * The model consists of application data, business rules, logic, and functions. A view can be any
        * output representation of data, such as a chart or a diagram. Multiple views of the same data 
        * are possible, such as a bar chart for management and a tabular view for accountants. 
        *The controller mediates input, converting it to commands for the model or view.The central 
        *ideas behind MVC are code reusability and n addition to dividing the application into three 
        *kinds of components, the MVC design defines the interactions between them.</p>
        *&lt;ul&gt;*&lt;li&gt;
        *&lt;b&gt;A controller*&lt;/b&gt;can send commands to its associated view to change the view's presentation of the model (e.g., by scrolling through a document). 
        *It can also send commands to the model to update the model's state (e.g., editing a document).
        *&lt;/li&gt;
        *&lt;/ul&gt; 
		*&lt;/div&gt; <br> 
		* &lt;script&gt;
		* $('#scrollcontent').ejScroller(); 	
		* // destroy the scroller
		* $("#scrollcontent").ejScroller("destroy");	
		* &lt;/script&gt;
		* @memberof ejScroller
		* @instance
		*/
        _destroy: function () {
            this.element.css({ "width": "", "height": "" }).find(".e-vscroll,.e-hscroll").remove();
            this.content().removeClass("e-content").css({ "width": "", "height": "" });
        }
    });
})(jQuery, Syncfusion, window);;
(function ($, ej, window, undefined) {
    'use strict';
    ej.widget("ejScrollbar", "ej.Scrollbar", {
        defaults: {
            orientation: "horizontal",
            viewportSize: 0,
            height: 18,
            width: 18,
            smallChange: 57,
            largeChange: 57,
            value: 0,
            maximum: 0,
            minimum: 0,
            buttonSize: 18
        },
        validTags: ["div"],
        type: "transclude",
        dataTypes: {
            buttonSize: "number",
            smallChange: "number",
            largeChange: "number",
        },
        observables: ["value"],
        value: ej.util.valueFunction("value"),
        content: function () {
            if (!this._content || !this._content.length) {
                if (this.model.orientation === "horizontal") {
                    this._content = this.element.find(".e-hhandle");
                }
                else {
                    this._content = this.element.find(".e-vhandle");
                }
            }
            return this._content;
        },
        _init: function () {
            this.element.addClass("e-widget");
            this._ensureScrollers();
            this.content();
            this._setInitialValues();

        },

        _setInitialValues: function () {
            var xy = "X";
            if (this.model.orientation === ej.Scrollbar.Orientation.Horizontal) {
                this.element.addClass("e-hscrollbar");
            }
            else {
                this.element.addClass("e-vscrollbar");
                xy = "Y";
            }
            if (this.value() !== 0 || this.model.minimum !== 0) {
                if (this.value() < this.model.minimum)
                    this.value(this.model.minimum);
                this["scroll" + xy](this.value(), "none");
            }
        },

        _ensureScrollers: function () {
            var jqVersion = $.fn.jquery, height, width;
            if (this.model.height) {
                this.element.height(this.model.height);
            }
            if (this.model.width) {
                this.element.width(this.model.width);
            }
            var d2;
            if (!this._scrollData) {
                if (this.model.orientation === "vertical") {
                    this._scrollData = this._createScroller("Height", "Y", "Top", "e-v");
                }
                else {
                    this._scrollData = this._createScroller("Width", "X", "Left", "e-h");
                }
            }
        },

        _setModel: function (option) {
            for (var prop in option) {
                if (prop === "value") {
                    if (this.value()) {
                        if (this.model.orientation === "horizontal") {
                            this.scrollX(this.value(), "none");
                        }
                        else
                            this.scrollY(this.value(), "none");
                    }
                } else {
                    this.refresh();
                    break;
                }
            }
        },

        _createScroller: function (dimension, xy, position, css) {
            var height;
            var d = {};
            var jqVersion = $.fn.jquery;
            d.dimension = dimension;
            d.xy = xy;
            d.position = position;
            d.css = css;
            d.uDimension = dimension;

            this._calculateLayout(d);
            this._createLayout(d);
            var buttons = this[d.main].find(".e-button");

            this._off(buttons, "mousedown")
                ._on(buttons, "mousedown", { d: d, step: 1 }, this._spaceMouseDown);
            this._off(this[d.scroll], "mousedown")
                ._on(this[d.scroll], "mousedown", { d: d }, this._spaceMouseDown);
            this._off(this[d.handler], "mousedown touchstart")
                ._on(this[d.handler], "mousedown touchstart", { d: d }, this._mouseDown);

            return d;
        },
        _createLayout: function (d) {
            var divString = "<div class='" + d.css + "{0}' style='" + d.dimension + ":{1}px'>{2}</div>";
            var jqVersion = $.fn.jquery;
            var lit = {}, height;
            lit[d.dimension] = d.modelDim;

            var el = ej.buildTag(
                "div." + d.css + "scroll",
                    String.format(divString, "up e-icon e-button", d.buttonSize) +
                    String.format(divString, "handlespace", d.handleSpace,
                        String.format(divString, "handle", d.handle)) +
                    String.format(divString, "down e-icon e-button", d.buttonSize),
                lit
            );

            this.element.append(el);
            jqVersion === "1.7.1" || jqVersion === "1.7.2" ? height = d.uDimension.toLowerCase() : height = "outer" + d.uDimension;
            this[d.handler] = this.element.find("." + d.handler);
			this[d.handler].css("transition", "none");
            this[d.scroll] = this[d.handler].parent();
            this[d.main] = this[d.scroll].parent();
            this[d.main].find(".e-button")["outer" + d.uDimension](d.buttonSize);
        },
        _calculateLayout: function (d) {
            d.scrollDim = "scroll" + d.dimension;
            d.lPosition = d.position.toLowerCase();
            d.clientXy = "page" + d.xy;
            d.scrollVal = "scroll" + d.position;
            d.scrollOneStepBy = this.model.smallChange;
            d.modelDim = this.model[(d.dimension = d.dimension.toLowerCase())];
            d.handler = d.css + "handle";
            d.buttonSize = this.model.buttonSize;
            d.main = d.css + "scroll";
            d.scroll = d.css + "ScrollSpace";
            d.handleSpace = d.modelDim - 2 * d.buttonSize;
            d.scrollable = (this.model.maximum - this.model.minimum);
            var trackLength = this.model.height;
            if (this.model.orientation === "horizontal")
                trackLength = this.model.width;
            d.handle = (this.model.viewportSize / ((this.model.maximum - this.model.minimum) + this.model.viewportSize)) * (trackLength - 2 * this.model.buttonSize);
            if (d.handle < 20) d.handle = 20;
            d.onePx = d.scrollable / (d.handleSpace - d.handle);
            d.fromScroller = false;
            d.up = true;
            d.vInterval = undefined;
        },
        _updateLayout: function (d) {
            this.element.find("." + d.css + "scroll").css(d.dimension, d.modelDim + "px")
                .find(".e-button").css(d.dimension, this.model.buttonSize).end()
                .find("." + d.css + "handlespace").css(d.dimension, d.handleSpace + "px")
                .find("." + d.css + "handle").css(d.dimension, d.handle + "px");
        },
        refresh: function () {
            this._ensureScrollers();
            if (this.value()) {
                if (this.model.orientation === "horizontal") {
                    this.scrollX(this.value(), "none");
                }
                else
                    this.scrollY(this.value(), "none");
            }
            if (this._scrollData) {
                this._calculateLayout(this._scrollData);
                this._updateLayout(this._scrollData);
            }
        },

        scrollY: function (pixel, source, triggerEvent) {
            var dS = this._scrollData;
            if (!triggerEvent)
                if (this._trigger("scroll", { source: source || "custom", scrollData: this._scrollData, scrollTop: pixel }))
                    return;
            this.value(pixel);
            if (this.content().length > 0)
                this.content()[0].style.top = (pixel - this.model.minimum) / this._scrollData.onePx + "px";
            return;
        },

        scrollX: function (pixel, source, triggerEvent) {
            var dS = this._scrollData;
            if (!triggerEvent)
                if (this._trigger("scroll", { source: source || "custom", scrollData: this._scrollData, scrollLeft: pixel }))
                    return;
            this.value(pixel);
            if (this.content().length > 0)
                this.content()[0].style.left = (pixel - this.model.minimum) / this._scrollData.onePx + "px";
            return;
        },

        _changeTop: function (d, step, source) {
            var start, t;
            if (d.dimension === "height")
                start = this.value();
            else
                start = this.value();
            t = start + step;

            if (step > 0) {
                if (t > this.model.maximum)
                    t = this.model.maximum;
            }
            else {
                if (t < this.model.minimum)
                    t = this.model.minimum;
            }
            if (t !== start) {
                this["scroll" + d.xy](t, source);
            }
            return t !== start;
        },

        _mouseUp: function (e) {
            if (!e.data) return;

            var d = e.data.d;
            e.type === "mouseup" && this[d.handler].removeClass("e-active");
            if (e.type === "mouseup" || e.type === "touchend" || (!e.toElement && !e.relatedTarget)) {
                this._off($(document), "mousemove touchmove");
                $(document).off("mouseout mouseup touchend", ej.proxy(this._mouseUp, this));
                d.fromScroller = false;
                this[d.scroll].off("mousemove");
                this[d.handler].off("mousemove").css("transition", "");

                if (e.data.source === "thumb" && !ej.isNullOrUndefined(this.model)) {
                    $.when(this.content()).done(ej.proxy(function () {
                        this._trigger("thumbEnd", { originalEvent: e, scrollData: d });
                    }, this));
                }
            }
            d.up = true;
        },


        _mouseDown: function (down) {
            //if (down.which !== 1) return;
            var d = down.data.d;

            d.fromScroller = true;
            this[d.handler].css("transition", "none");

            var prevY, top = parseInt(this[d.handler].css(d.lPosition)) || 0;

            this._on($(document), "mousemove touchmove", (function (move) {
                move.preventDefault();
                var skip = 1;
                var pageXY = move.type == "mousemove" ? move[d.clientXy] : move.originalEvent.changedTouches[0][d.clientXy];
                if (prevY && pageXY !== prevY) {
                    top = (pageXY - prevY);
                    var value = top * d.onePx;
                    this._changeTop(d, value, "thumb");
                    this._trigger("thumbMove", { originalEvent: move, scrollData: d });
                }

                if (skip === 1)
                    prevY = pageXY;
            }));

            this._trigger("thumbStart", { originalEvent: down, scrollData: d });

            $(document).one("mouseup touchend", { d: d, source: "thumb" }, ej.proxy(this._mouseUp, this));
            $(document).mouseout({ d: d, source: "thumb" }, ej.proxy(this._mouseUp, this));
        },

        _spaceMouseDown: function (e) {
            if (!e.data) return;

            var d = e.data.d;

            if (e.which !== 1 || e.target === this[d.handler][0]) return;

            var step = e.data.step ? this.model.smallChange : this.model.largeChange, hTop = e.data.top || this[d.handler].offset()[d.lPosition];

            e[d.clientXy] = e[d.clientXy] || 0;

            if (e[d.clientXy] < hTop) step *= -1;

            this._changeTop(d, step, step === 3 ? "track" : "button");

            if (e.data.step !== 1) {
                this[d.scroll].mousemove(function () {
                    d.up = true;
                });
            }

            d.up = false;
            d.vInterval = setInterval(ej.proxy(function () {

                if (step < 0 ? hTop + (step / d.onePx) < e[d.clientXy] : hTop + d.handle + (step / d.onePx) > e[d.clientXy])
                    d.up = true;

                if (d.up) {
                    clearInterval(d.vInterval);
                    return;
                }

                this._changeTop(d, step, step === 3 ? "track" : "button");

                hTop = e.data.top || this[d.handler].offset()[d.lPosition];
            }, this), 150);

            $(document).one("mouseup", { d: d }, ej.proxy(this._mouseUp, this));
            $(document).mouseout({ d: d }, ej.proxy(this._mouseUp, this));
        },

        remove: function () {
            if (this.model.orientation === ej.Scrollbar.Orientation.Horizontal)
                this.element.find(".e-hscroll").remove();
            if (this.model.orientation === ej.Scrollbar.Orientation.Vertical)
                this.element.find(".e-vscroll").remove();
            this._scrollData = null;
            this._content = null;
        },

        _destroy: function () {
            this.element.remove();
        },
    });

    ej.Scrollbar.Orientation = {
        Horizontal: "horizontal",
        Vertical: "vertical"
    };
})(jQuery, Syncfusion, window);;;
/**
* @fileOverview Plugin to style the Html AutoComplete elements
* @copyright Copyright Syncfusion Inc. 2001 - 2013. All rights reserved.
*  Use of this code is subject to the terms of our license.
*  A copy of the current license can be obtained at any time by e-mailing
*  licensing@syncfusion.com. Any infringement will be prosecuted under
*  applicable laws. 
* @version 12.1 
* @author <a href="mailto:licensing@syncfusion.com">Syncfusion Inc</a>
*/

(function ($, ej, undefined) {

    ej.widget("ejNavigationDrawerBase", "ej.NavigationDrawerBase", {

        defaults: {
            //Web
            /**		
            * Sets the Direction for the control. See {@link Direction}
            * @default left
            * @member direction {object}
            * @type {enum}
            * @example 
            * //Set the direction property in unobtrusive way.
            * &lt;div id="home" class="navsubpage"&gt;
            * &lt;div align="center" class="content"&gt;
            * &lt;h2 class="title"&gt;
            * Home&lt;/h2&gt;
            * &lt;p&gt;
            * Founded by industry experts in 2001,Syncfusion, Inc. provides the broadest range of enterprise-class software components and tools for the Microsoft .NET platform.
            * &lt;/p&gt;
            * &lt;/div&gt;
            * &lt;/div&gt;
            * &lt;style&gt;
            * .list {
            *   border-bottom: 1px solid;
            *   line-height: 50px;
            *   text-align: center;
            *   width:200px;
            *   }
            * &lt;/style&gt;
            * &lt;div data-role="ejnavigationdrawer" id="navpane" data-ej-direction="left" &gt;
            * &lt;div class="list"&gt; Home &lt;/div&gt;
            * &lt;div class="list"&gt; Communities &lt;/div&gt;
            * &lt;/div&gt;
            * @example
            * // Set Navigation Drawer direction on initialization. 
            * //To set direction API value 
            * &lt;div id="home" class="navsubpage"&gt;
            * &lt;div align="center" class="content"&gt;
            * &lt;h2 class="title"&gt;
            * Home&lt;/h2&gt;
            * &lt;p&gt;
            * Founded by industry experts in 2001,Syncfusion, Inc. provides the broadest range of enterprise-class software components and tools for the Microsoft .NET platform.
            * &lt;/p&gt;
            * &lt;/div&gt;
            * &lt;/div&gt;
            * &lt;style&gt;
            * .list {
            *   border-bottom: 1px solid;
            *   line-height: 50px;
            *   text-align: center;
            *   width:200px;
            *   }
            * &lt;/style&gt;
            * &lt;div &gt;
            * &lt;div class="list"&gt; Home &lt;/div&gt;
            * &lt;div class="list"&gt; Communities &lt;/div&gt;
            * &lt;/div&gt;
            * &lt;script&gt;
            * $(function () {
            * $("#navpane").ejNavigationDrawer("direction","left");	
            * });
            * &lt;/script&gt;
            * @example 
            * &lt;script&gt;
            * //Get or set the Navigation Drawer direction, after initialization:
            * $(function () {
            * // Gets the direction API value.		
            * $("#navpane").ejNavigationDrawer("option", "direction");	
            * // Sets the direction API	
            * $("#navpane").ejNavigationDrawer ("option", "direction", "left");  
            * });
            * &lt;/script&gt;  
            * @memberof ejNavigationDrawer
            * @instance
            */
            //Mobile
            /**		
            * Sets the Direction for the control. See {@link Direction}
            * @default left
            * @member direction {object}
            * @type {enum}
            * @example 
            * //Set the direction property in unobtrusive way.
            * &lt;div id="home" class="navsubpage"&gt;
            * &lt;div align="center" class="content"&gt;
            * &lt;h2 class="title"&gt;
            * Home&lt;/h2&gt;
            * &lt;p&gt;
            * Founded by industry experts in 2001,Syncfusion, Inc. provides the broadest range of enterprise-class software components and tools for the Microsoft .NET platform.
            * &lt;/p&gt;
            * &lt;/div&gt;
            * &lt;/div&gt;
            * &lt;style&gt;
            * .list {
            *   border-bottom: 1px solid;
            *   line-height: 50px;
            *   text-align: center;
            *   width:200px;
            *   }
            * &lt;/style&gt;
            * &lt;div data-role="ejmnavigationdrawer" id="navpane" data-ej-direction="left" &gt;
            * &lt;div class="list"&gt; Home &lt;/div&gt;
            * &lt;div class="list"&gt; Communities &lt;/div&gt;
            * &lt;/div&gt;
            * @example
            * // Set Navigation Drawer direction on initialization. 
            * //To set direction API value 
            * &lt;div id="home" class="navsubpage"&gt;
            * &lt;div align="center" class="content"&gt;
            * &lt;h2 class="title"&gt;
            * Home&lt;/h2&gt;
            * &lt;p&gt;
            * Founded by industry experts in 2001,Syncfusion, Inc. provides the broadest range of enterprise-class software components and tools for the Microsoft .NET platform.
            * &lt;/p&gt;
            * &lt;/div&gt;
            * &lt;/div&gt;
            * &lt;style&gt;
            * .list {
            *   border-bottom: 1px solid;
            *   line-height: 50px;
            *   text-align: center;
            *   width:200px;
            *   }
            * &lt;/style&gt;
            * &lt;div &gt;
            * &lt;div class="list"&gt; Home &lt;/div&gt;
            * &lt;div class="list"&gt; Communities &lt;/div&gt;
            * &lt;/div&gt;
            * &lt;script&gt;
            * $(function () {
            * $("#navpane").ejmNavigationDrawer("direction","left");	
            * });
            * &lt;/script&gt;
            * @example 
            * &lt;script&gt;
            * //Get or set the Navigation Drawer direction, after initialization:
            * $(function () {
            * // Gets the direction API value.		
            * $("#navpane").ejNavigationDrawer("option", "direction");	
            * // Sets the direction API	
            * $("#navpane").ejNavigationDrawer ("option", "direction", "left");  
            * });
            * &lt;/script&gt;  
            * @memberof ejmNavigationDrawer
            * @instance
            */
            direction: "left",
            //Web
            /**		
            * Specifies the targetId for navigation drawer
            * @default ""
            * @member targetid {object}
            * @type {string}
            * @example 
            * //Set the direction property in unobtrusive way.
            * &lt;div id="home" class="navsubpage"&gt;
            * &lt;div align="center" class="content"&gt;
            * &lt;h2 class="title"&gt;
            * Home&lt;/h2&gt;
            * &lt;p&gt;
            * Founded by industry experts in 2001,Syncfusion, Inc. provides the broadest range of enterprise-class software components and tools for the Microsoft .NET platform.
            * &lt;/p&gt;
            * &lt;/div&gt;
            * &lt;input id="target" type="button" text="target"/&gt;
            * &lt;/div&gt;
            * &lt;style&gt;
            * .list {
            *   border-bottom: 1px solid;
            *   line-height: 50px;
            *   text-align: center;
            *   width:200px;
            *   }
            * &lt;/style&gt;
            * &lt;div data-role="ejnavigationdrawer" id="navpane" data-ej-targetid="target" &gt;
            * &lt;div class="list"&gt; Home &lt;/div&gt;
            * &lt;div class="list"&gt; Communities &lt;/div&gt;
            * &lt;/div&gt;
            * @example
            * // Set Navigation Drawer direction on initialization. 
            * //To set direction API value 
            * &lt;div id="home" class="navsubpage"&gt;
            * &lt;div align="center" class="content"&gt;
            * &lt;h2 class="title"&gt;
            * Home&lt;/h2&gt;
            * &lt;p&gt;
            * Founded by industry experts in 2001,Syncfusion, Inc. provides the broadest range of enterprise-class software components and tools for the Microsoft .NET platform.
            * &lt;/p&gt;
            * &lt;/div&gt;            
            * &lt;input id="target" type="button" text="target"/&gt;
            * &lt;/div&gt;
            * &lt;style&gt;
            * .list {
            *   border-bottom: 1px solid;
            *   line-height: 50px;
            *   text-align: center;
            *   width:200px;
            *   }
            * &lt;/style&gt;
            * &lt;div &gt;
            * &lt;div class="list"&gt; Home &lt;/div&gt;
            * &lt;div class="list"&gt; Communities &lt;/div&gt;
            * &lt;/div&gt;
            * &lt;script&gt;
            * $(function () {
            * $("#navpane").ejNavigationDrawer("targetId","target");	
            * });
            * &lt;/script&gt;
            * @example 
            * &lt;script&gt;
            * //Get or set the Navigation Drawer targetId, after initialization:
            * $(function () {
            * // Gets the targetId API value.		
            * $("#navpane").ejNavigationDrawer("option", "targetId");	
            * // Sets the targetId API	
            * $("#navpane").ejNavigationDrawer ("option", "targetId", "left");  
            * });
            * &lt;/script&gt;  
            * @memberof ejNavigationDrawer
            * @instance
            */
            //Mobile
            /**		
            * Specifies the targetId for navigation drawer
            * @default ""
            * @member targetid {object}
            * @type {string}
            * @example 
            * //Set the direction property in unobtrusive way.
            * &lt;div id="home" class="navsubpage"&gt;
            * &lt;div align="center" class="content"&gt;
            * &lt;h2 class="title"&gt;
            * Home&lt;/h2&gt;
            * &lt;p&gt;
            * Founded by industry experts in 2001,Syncfusion, Inc. provides the broadest range of enterprise-class software components and tools for the Microsoft .NET platform.
            * &lt;/p&gt;
            * &lt;/div&gt;
            * &lt; input id="target" data-role="button" type="button" /&gt;
            * &lt;/div&gt;
            * &lt;style&gt;
            * .list {
            *   border-bottom: 1px solid;
            *   line-height: 50px;
            *   text-align: center;
            *   width:200px;
            *   }
            * &lt;/style&gt;
            * &lt;div data-role="ejmnavigationdrawer" id="navpane" data-ej-targetid="target" &gt;
            * &lt;div class="list"&gt; Home &lt;/div&gt;
            * &lt;div class="list"&gt; Communities &lt;/div&gt;
            * &lt;/div&gt;
            * @example
            * // Set Navigation Drawer direction on initialization. 
            * //To set direction API value 
            * &lt;div id="home" class="navsubpage"&gt;
            * &lt;div align="center" class="content"&gt;
            * &lt;h2 class="title"&gt;
            * Home&lt;/h2&gt;
            * &lt;p&gt;
            * Founded by industry experts in 2001,Syncfusion, Inc. provides the broadest range of enterprise-class software components and tools for the Microsoft .NET platform.
            * &lt;/p&gt;
            * &lt;/div&gt;
            * &lt; input id="target" data-role="button" type="button" /&gt;
            * &lt;/div&gt;
            * &lt;style&gt;
            * .list {
            *   border-bottom: 1px solid;
            *   line-height: 50px;
            *   text-align: center;
            *   width:200px;
            *   }
            * &lt;/style&gt;
            * &lt;div &gt;
            * &lt;div class="list"&gt; Home &lt;/div&gt;
            * &lt;div class="list"&gt; Communities &lt;/div&gt;
            * &lt;/div&gt;
            * &lt;script&gt;
            * $(function () {
            * $("#navpane").ejmNavigationDrawer("targetId","left");	
            * });
            * &lt;/script&gt;
            * @example 
            * //Get or set the Navigation Drawer targetId, after initialization:
            * &lt;script&gt;
            * // Gets the TargetId API value.		
            *  $("#navpane").ejmNavigationDrawer ("option", "targetId");		
            * // Sets the TargetId API
            * $("#navpane").ejmNavigationDrawer ("option", "targetId", "sample");   
            * &lt;/script&gt;
            * @memberof ejmNavigationDrawer
            * @instance
            */
            targetId: null,
            //Web
            /**		
            * Specifies position whether it is in fixed or relative to the page. See {@link Position}
            * @default normal
            * @member position {object}
            * @type {enum}
            * @example 
            * //Set the position property in unobtrusive way.
            * &lt;div id="home" class="navsubpage"&gt;
            * &lt;div align="center" class="content"&gt;
            * &lt;h2 class="title"&gt;
            * Home&lt;/h2&gt;
            * &lt;p&gt;
            * Founded by industry experts in 2001,Syncfusion, Inc. provides the broadest range of enterprise-class software components and tools for the Microsoft .NET platform.
            * &lt;/p&gt;
            * &lt;/div&gt;
            * &lt;/div&gt;
            * &lt;style&gt;
            * .list {
            *   border-bottom: 1px solid;
            *   line-height: 50px;
            *   text-align: center;
            *   width:200px;
            *   }
            * &lt;/style&gt;
            * &lt;div data-role="ejnavigationdrawer" id="navpane" data-ej-position="normal" &gt;
            * &lt;div class="list"&gt; Home &lt;/div&gt;
            * &lt;div class="list"&gt; Communities &lt;/div&gt;
            * &lt;/div&gt;
            * @example
            * // Set Navigation Drawer position on initialization. 
            * //To set position API value 
            * &lt;div id="home" class="navsubpage"&gt;
            * &lt;div align="center" class="content"&gt;
            * &lt;h2 class="title"&gt;
            * Home&lt;/h2&gt;
            * &lt;p&gt;
            * Founded by industry experts in 2001,Syncfusion, Inc. provides the broadest range of enterprise-class software components and tools for the Microsoft .NET platform.
            * &lt;/p&gt;
            * &lt;/div&gt;
            * &lt;/div&gt;
            * &lt;style&gt;
            * .list {
            *   border-bottom: 1px solid;
            *   line-height: 50px;
            *   text-align: center;
            *   width:200px;
            *   }
            * &lt;/style&gt;
            * &lt;div &gt;
            * &lt;div class="list"&gt; Home &lt;/div&gt;
            * &lt;div class="list"&gt; Communities &lt;/div&gt;
            * &lt;/div&gt;
            * &lt;script&gt;
            * $(function () {
            * $("#navpane").ejNavigationDrawer("position","normal");	
            * });
            * &lt;/script&gt;
            * @example 
            * &lt;script&gt;
            * //Get or set the Navigation Drawer position, after initialization:
            * $(function () {
            * // Gets the position API value.		
            * $("#navpane").ejNavigationDrawer("option", "position");	
            * // Sets the position API	
            * $("#navpane").ejNavigationDrawer ("option", "position", "normal");  
            * });
            * &lt;/script&gt;  
            * @memberof ejNavigationDrawer
            * @instance
            */
            //Mobile
            /**		
            * Specifies position whether it is in fixed or relative to the page. See {@link Position}
            * @default normal
            * @member position {object}
            * @type {enum}
            * @example 
            * //Set the position property in unobtrusive way.
            * &lt;div id="home" class="navsubpage"&gt;
            * &lt;div align="center" class="content"&gt;
            * &lt;h2 class="title"&gt;
            * Home&lt;/h2&gt;
            * &lt;p&gt;
            * Founded by industry experts in 2001,Syncfusion, Inc. provides the broadest range of enterprise-class software components and tools for the Microsoft .NET platform.
            * &lt;/p&gt;
            * &lt;/div&gt;
            * &lt;/div&gt;
            * &lt;style&gt;
            * .list {
            *   border-bottom: 1px solid;
            *   line-height: 50px;
            *   text-align: center;
            *   width:200px;
            *   }
            * &lt;/style&gt;
            * &lt;div data-role="ejmnavigationdrawer" id="navpane" data-ej-position="normal" &gt;
            * &lt;div class="list"&gt; Home &lt;/div&gt;
            * &lt;div class="list"&gt; Communities &lt;/div&gt;
            * &lt;/div&gt;
            * @example
            * // Set Navigation Drawer position on initialization. 
            * //To set position API value 
            * &lt;div id="home" class="navsubpage"&gt;
            * &lt;div align="center" class="content"&gt;
            * &lt;h2 class="title"&gt;
            * Home&lt;/h2&gt;
            * &lt;p&gt;
            * Founded by industry experts in 2001,Syncfusion, Inc. provides the broadest range of enterprise-class software components and tools for the Microsoft .NET platform.
            * &lt;/p&gt;
            * &lt;/div&gt;
            * &lt;/div&gt;
            * &lt;style&gt;
            * .list {
            *   border-bottom: 1px solid;
            *   line-height: 50px;
            *   text-align: center;
            *   width:200px;
            *   }
            * &lt;/style&gt;
            * &lt;div &gt;
            * &lt;div class="list"&gt; Home &lt;/div&gt;
            * &lt;div class="list"&gt; Communities &lt;/div&gt;
            * &lt;/div&gt;
            * &lt;script&gt;
            * $(function () {
            * $("#navpane").ejmNavigationDrawer("position","normal");	
            * });
            * &lt;/script&gt;
            * @example 
            * //Get or set the Navigation Drawer position, after initialization:
            * &lt;script&gt;
            * // Gets the position API value.		
            *  $("#navpane").ejmNavigationDrawer ("option", "position");		
            * // Sets the position API
            * $("#navpane").ejmNavigationDrawer ("option", "position", "normal");   
            * &lt;/script&gt;
            * @memberof ejmNavigationDrawer
            * @instance
            */
            position: "normal",
            //Web
            /**		
            * Sets the listview to be enabled or not
            * @default false
            * @member enablelistview {object}
            * @type {boolean}
            * @example 
            * //Set the listview property in unobtrusive way.
            * &lt;div id="home" class="navsubpage"&gt;
            * &lt;div align="center" class="content"&gt;
            * &lt;h2 class="title"&gt;
            * Home&lt;/h2&gt;
            * &lt;p&gt;
            * Founded by industry experts in 2001,Syncfusion, Inc. provides the broadest range of enterprise-class software components and tools for the Microsoft .NET platform.
            * &lt;/p&gt;
            * &lt;/div&gt;
            * &lt;/div&gt;
            * &lt;style&gt;
            * .list {
            *   border-bottom: 1px solid;
            *   line-height: 50px;
            *   text-align: center;
            *   width:200px;
            *   }
            * &lt;/style&gt;
            * &lt;div data-role="ejnavigationdrawer" id="navpane" data-ej-enablelistview="false" &gt;
            * &lt;div class="list"&gt; Home &lt;/div&gt;
            * &lt;div class="list"&gt; Communities &lt;/div&gt;
            * &lt;/div&gt;
            * @example
            * // Set Navigation Drawer listview on initialization. 
            * //To set listview API value 
            * &lt;div id="home" class="navsubpage"&gt;
            * &lt;div align="center" class="content"&gt;
            * &lt;h2 class="title"&gt;
            * Home&lt;/h2&gt;
            * &lt;p&gt;
            * Founded by industry experts in 2001,Syncfusion, Inc. provides the broadest range of enterprise-class software components and tools for the Microsoft .NET platform.
            * &lt;/p&gt;
            * &lt;/div&gt;
            * &lt;/div&gt;
            * &lt;style&gt;
            * .list {
            *   border-bottom: 1px solid;
            *   line-height: 50px;
            *   text-align: center;
            *   width:200px;
            *   }
            * &lt;/style&gt;
            * &lt;div &gt;
            * &lt;div class="list"&gt; Home &lt;/div&gt;
            * &lt;div class="list"&gt; Communities &lt;/div&gt;
            * &lt;/div&gt;
            * &lt;script&gt;
            * $(function () {
            * $("#navpane").ejNavigationDrawer("enableListView","false");	
            * });
            * &lt;/script&gt;
            * @example 
            * &lt;script&gt;
            * //Get or set the Navigation Drawer listview, after initialization:
            * $(function () {
            * // Gets the listview API value.		
            * $("#navpane").ejNavigationDrawer("option", "enableListView");	
            * // Sets the listview API	
            * $("#navpane").ejNavigationDrawer ("option", "enableListView", "false");  
            * });
            * &lt;/script&gt;  
            * @memberof ejNavigationDrawer
            * @instance
            */
            //Mobile
            /**		
            * Sets the listview to be enabled or not
            * @default false
            * @member enablelistview {object}
            * @type {boolean}
            * @example 
            * //Set the listview property in unobtrusive way.
            * &lt;div id="home" class="navsubpage"&gt;
            * &lt;div align="center" class="content"&gt;
            * &lt;h2 class="title"&gt;
            * Home&lt;/h2&gt;
            * &lt;p&gt;
            * Founded by industry experts in 2001,Syncfusion, Inc. provides the broadest range of enterprise-class software components and tools for the Microsoft .NET platform.
            * &lt;/p&gt;
            * &lt;/div&gt;
            * &lt;/div&gt;
            * &lt;style&gt;
            * .list {
            *   border-bottom: 1px solid;
            *   line-height: 50px;
            *   text-align: center;
            *   width:200px;
            *   }
            * &lt;/style&gt;
            * &lt;div data-role="ejmnavigationdrawer" id="navpane" data-ej-enablelistview="false" &gt;
            * &lt;div class="list"&gt; Home &lt;/div&gt;
            * &lt;div class="list"&gt; Communities &lt;/div&gt;
            * &lt;/div&gt;
            * @example
            * // Set Navigation Drawer listview on initialization. 
            * //To set listview API value 
            * &lt;div id="home" class="navsubpage"&gt;
            * &lt;div align="center" class="content"&gt;
            * &lt;h2 class="title"&gt;
            * Home&lt;/h2&gt;
            * &lt;p&gt;
            * Founded by industry experts in 2001,Syncfusion, Inc. provides the broadest range of enterprise-class software components and tools for the Microsoft .NET platform.
            * &lt;/p&gt;
            * &lt;/div&gt;
            * &lt;/div&gt;
            * &lt;style&gt;
            * .list {
            *   border-bottom: 1px solid;
            *   line-height: 50px;
            *   text-align: center;
            *   width:200px;
            *   }
            * &lt;/style&gt;
            * &lt;div &gt;
            * &lt;div class="list"&gt; Home &lt;/div&gt;
            * &lt;div class="list"&gt; Communities &lt;/div&gt;
            * &lt;/div&gt;
            * &lt;script&gt;
            * $(function () {
            * $("#navpane").ejmNavigationDrawer("enableListView","false");	
            * });
            * &lt;/script&gt;
            * @example 
            * //Get or set the Navigation Drawer listview, after initialization:
            * &lt;script&gt;
            * // Gets the listview API value.		
            *  $("#navpane").ejmNavigationDrawer ("option", "enableListView");		
            * // Sets the listview API
            * $("#navpane").ejmNavigationDrawer ("option", "enableListView", false);   
            * &lt;/script&gt;
            * @memberof ejmNavigationDrawer
            * @instance
            */
            enableListView: false,
            //Web
            /**		
            * Sets all the properties of listview to render in navigation drawer
            * @member listviewsettings {object}
            * @example
            * &lt;div id="home" class="navsubpage"&gt;
            * &lt;div align="center" class="content"&gt;
            * &lt;h2 class="title"&gt;
            * Home&lt;/h2&gt;
            * &lt;p&gt;
            * Founded by industry experts in 2001,Syncfusion, Inc. provides the broadest range of enterprise-class software components and tools for the Microsoft .NET platform.
            * &lt;/p&gt;
            * &lt;/div&gt;
            * &lt;/div&gt;
            * &lt;style&gt;
            * .list {
            *   border-bottom: 1px solid;
            *   line-height: 50px;
            *   text-align: center;
            *   width:200px;
            *   }
            * &lt;/style&gt;
            * &lt;div data-role="ejnavigationdrawer" id="navpane" data-ej-listviewsettings-width="200" &gt;
            * &lt;div class="list"&gt; Home &lt;/div&gt;
            * &lt;div class="list"&gt; Communities &lt;/div&gt;
            * &lt;/div&gt;
            * @example
            * // Set Navigation Drawer listview on initialization. 
            * //To set listview API value 
            * &lt;div id="home" class="navsubpage"&gt;
            * &lt;div align="center" class="content"&gt;
            * &lt;h2 class="title"&gt;
            * Home&lt;/h2&gt;
            * &lt;p&gt;
            * Founded by industry experts in 2001,Syncfusion, Inc. provides the broadest range of enterprise-class software components and tools for the Microsoft .NET platform.
            * &lt;/p&gt;
            * &lt;/div&gt;
            * &lt;/div&gt;
            * &lt;style&gt;
            * .list {
            *   border-bottom: 1px solid;
            *   line-height: 50px;
            *   text-align: center;
            *   width:200px;
            *   }
            * &lt;/style&gt;
            * &lt;div &gt;
            * &lt;div class="list"&gt; Home &lt;/div&gt;
            * &lt;div class="list"&gt; Communities &lt;/div&gt;
            * &lt;/div&gt;
            * &lt;script&gt;
            * $(function () {
            * $("#navpane").ejNavigationDrawer({model.listViewSettings{width:200});	
            * });
            * &lt;/script&gt;
            * @example 
            * &lt;script&gt;
            * //Get or set the Navigation Drawer listViewSettings, after initialization:
            * $(function () {
            * // Gets the listViewSettings API value.		
            * $("#navpane").ejNavigationDrawer("option", "listViewSettings.width");	
            * // Sets the listViewSettings API	
            * $("#navpane").ejNavigationDrawer ("option", "listViewSettings.width", "200");  
            * });
            * &lt;/script&gt;  
            * @memberof ejNavigationDrawer
            * @instance
            */
            //Mobile
            /**		
            * Sets all the properties of listview to render in navigation drawer
            * @member listviewsettings {object}
            * @example 
            * //Set the listView Settings property in unobtrusive way.
            * &lt;div id="home" class="navsubpage"&gt;
            * &lt;div align="center" class="content"&gt;
            * &lt;h2 class="title"&gt;
            * Home&lt;/h2&gt;
            * &lt;p&gt;
            * Founded by industry experts in 2001,Syncfusion, Inc. provides the broadest range of enterprise-class software components and tools for the Microsoft .NET platform.
            * &lt;/p&gt;
            * &lt;/div&gt;
            * &lt;/div&gt;
            * &lt;style&gt;
            * .list {
            *   border-bottom: 1px solid;
            *   line-height: 50px;
            *   text-align: center;
            *   width:200px;
            *   }
            * &lt;/style&gt;
            * &lt;div data-role="ejmnavigationdrawer" id="navpane" data-ej-listviewsettings-width="200" &gt;
            * &lt;div class="list"&gt; Home &lt;/div&gt;
            * &lt;div class="list"&gt; Communities &lt;/div&gt;
            * &lt;/div&gt;
            * @example
            * // Set Navigation Drawer listview settings on initialization. 
            * //To set listview settings API value 
            * &lt;div id="home" class="navsubpage"&gt;
            * &lt;div align="center" class="content"&gt;
            * &lt;h2 class="title"&gt;
            * Home&lt;/h2&gt;
            * &lt;p&gt;
            * Founded by industry experts in 2001,Syncfusion, Inc. provides the broadest range of enterprise-class software components and tools for the Microsoft .NET platform.
            * &lt;/p&gt;
            * &lt;/div&gt;
            * &lt;/div&gt;
            * &lt;style&gt;
            * .list {
            *   border-bottom: 1px solid;
            *   line-height: 50px;
            *   text-align: center;
            *   width:200px;
            *   }
            * &lt;/style&gt;
            * &lt;div &gt;
            * &lt;div class="list"&gt; Home &lt;/div&gt;
            * &lt;div class="list"&gt; Communities &lt;/div&gt;
            * &lt;/div&gt;
            * &lt;script&gt;
            * $(function () {
            * $("#navpane").ejmNavigationDrawer({model.listViewSettings{width:200});	
            * });
            * &lt;/script&gt;
            * @example 
            * //Get or set the Navigation Drawer listViewSettings, after initialization:
            * &lt;script&gt;
            * // Gets the listViewSettings API value.		
            *  $("#navpane").ejmNavigationDrawer ("option", "listViewSettings.width");	
            * // Sets the listViewSettings API
            * $("#navpane").ejmNavigationDrawer ("option", "listViewSettings.width", "false");   
            * &lt;/script&gt;
            * @memberof ejmNavigationDrawer
            * @instance
            */
            listViewSettings: {},
            //Web
            /**		
            * Sets the rendering type of the control.  See {@link Type}
            * @default overlay            
            * @member type {object}
            * @type {enum}
            * @example
            * &lt;div id="home" class="navsubpage"&gt;
            * &lt;div align="center" class="content"&gt;
            * &lt;h2 class="title"&gt;
            * Home&lt;/h2&gt;
            * &lt;p&gt;
            * Founded by industry experts in 2001,Syncfusion, Inc. provides the broadest range of enterprise-class software components and tools for the Microsoft .NET platform.
            * &lt;/p&gt;
            * &lt;/div&gt;
            * &lt;/div&gt;
            * &lt;style&gt;
            * .list {
            *   border-bottom: 1px solid;
            *   line-height: 50px;
            *   text-align: center;
            *   width:200px;
            *   }
            * &lt;/style&gt;
            * &lt;div data-role="ejnavigationdrawer" id="navpane" data-ej-type="overlay" &gt;
            * &lt;div class="list"&gt; Home &lt;/div&gt;
            * &lt;div class="list"&gt; Communities &lt;/div&gt;
            * &lt;/div&gt;
            * @example
            * // Set Navigation Drawer type on initialization. 
            * //To set type API value 
            * &lt;div id="home" class="navsubpage"&gt;
            * &lt;div align="center" class="content"&gt;
            * &lt;h2 class="title"&gt;
            * Home&lt;/h2&gt;
            * &lt;p&gt;
            * Founded by industry experts in 2001,Syncfusion, Inc. provides the broadest range of enterprise-class software components and tools for the Microsoft .NET platform.
            * &lt;/p&gt;
            * &lt;/div&gt;
            * &lt;/div&gt;
            * &lt;style&gt;
            * .list {
            *   border-bottom: 1px solid;
            *   line-height: 50px;
            *   text-align: center;
            *   width:200px;
            *   }
            * &lt;/style&gt;
            * &lt;div &gt;
            * &lt;div class="list"&gt; Home &lt;/div&gt;
            * &lt;div class="list"&gt; Communities &lt;/div&gt;
            * &lt;/div&gt;
            * &lt;script&gt;
            * $(function () {
            * $("#navpane").ejNavigationDrawer("type","overlay");	
            * });
            * &lt;/script&gt;
            * @example 
            * &lt;script&gt;
            * //Get or set the Navigation Drawer type, after initialization:
            * $(function () {
            * // Gets the type API value.		
            * $("#navpane").ejNavigationDrawer("option", "type");	
            * // Sets the type API	
            * $("#navpane").ejNavigationDrawer ("option", "type", "overlay");  
            * });
            * &lt;/script&gt;  
            * @memberof ejNavigationDrawer
            * @instance
            */
            //Mobile
            /**		
            * Sets the rendering type of the control.  See {@link Type}
            * @default overlay      
            * @member type {object}
            * @type {enum}
            * @example 
            * //Set the type property in unobtrusive way.
            * &lt;div id="home" class="navsubpage"&gt;
            * &lt;div align="center" class="content"&gt;
            * &lt;h2 class="title"&gt;
            * Home&lt;/h2&gt;
            * &lt;p&gt;
            * Founded by industry experts in 2001,Syncfusion, Inc. provides the broadest range of enterprise-class software components and tools for the Microsoft .NET platform.
            * &lt;/p&gt;
            * &lt;/div&gt;
            * &lt;/div&gt;
            * &lt;style&gt;
            * .list {
            *   border-bottom: 1px solid;
            *   line-height: 50px;
            *   text-align: center;
            *   width:200px;
            *   }
            * &lt;/style&gt;
            * &lt;div data-role="ejmnavigationdrawer" id="navpane" data-ej-type="overlay" &gt;
            * &lt;div class="list"&gt; Home &lt;/div&gt;
            * &lt;div class="list"&gt; Communities &lt;/div&gt;
            * &lt;/div&gt;
            * @example
            * // Set Navigation Drawer type on initialization. 
            * //To set type API value 
            * &lt;div id="home" class="navsubpage"&gt;
            * &lt;div align="center" class="content"&gt;
            * &lt;h2 class="title"&gt;
            * Home&lt;/h2&gt;
            * &lt;p&gt;
            * Founded by industry experts in 2001,Syncfusion, Inc. provides the broadest range of enterprise-class software components and tools for the Microsoft .NET platform.
            * &lt;/p&gt;
            * &lt;/div&gt;
            * &lt;/div&gt;
            * &lt;style&gt;
            * .list {
            *   border-bottom: 1px solid;
            *   line-height: 50px;
            *   text-align: center;
            *   width:200px;
            *   }
            * &lt;/style&gt;
            * &lt;div &gt;
            * &lt;div class="list"&gt; Home &lt;/div&gt;
            * &lt;div class="list"&gt; Communities &lt;/div&gt;
            * &lt;/div&gt;
            * &lt;script&gt;
            * $(function () {
            * $("#navpane").ejmNavigationDrawer("type","overlay");	
            * });
            * &lt;/script&gt;
            * @example 
            * //Get or set the Navigation Drawer type, after initialization:
            * &lt;script&gt;
            * // Gets the type API value.		
            *  $("#navpane").ejmNavigationDrawer ("option", "type");		
            * // Sets the type API
            * $("#navpane").ejmNavigationDrawer ("option", "type", "overlay");   
            * &lt;/script&gt;
            * @memberof ejmNavigationDrawer
            * @instance
            */
            type: "overlay",
            /**		
            * Specifies the width of the control
            * @default auto      
            * @member width {object}
            * @type {int}
            * @example 
            * //Set the width property in unobtrusive way.
            * &lt;div id="home" class="navsubpage"&gt;
            * &lt;div align="center" class="content"&gt;
            * &lt;h2 class="title"&gt;
            * Home&lt;/h2&gt;
            * &lt;p&gt;
            * Founded by industry experts in 2001,Syncfusion, Inc. provides the broadest range of enterprise-class software components and tools for the Microsoft .NET platform.
            * &lt;/p&gt;
            * &lt;/div&gt;
            * &lt;/div&gt;
            * &lt;style&gt;
            * .list {
            *   border-bottom: 1px solid;
            *   line-height: 50px;
            *   text-align: center;
            *   width:200px;
            *   }
            * &lt;/style&gt;
            * &lt;div data-role="ejnavigationdrawer" id="navpane" data-ej-width="200" &gt;
            * &lt;div class="list"&gt; Home &lt;/div&gt;
            * &lt;div class="list"&gt; Communities &lt;/div&gt;
            * &lt;/div&gt;
            * @example
            * // Set Navigation Drawer width on initialization. 
            * //To set width API value 
            * &lt;div id="home" class="navsubpage"&gt;
            * &lt;div align="center" class="content"&gt;
            * &lt;h2 class="title"&gt;
            * Home&lt;/h2&gt;
            * &lt;p&gt;
            * Founded by industry experts in 2001,Syncfusion, Inc. provides the broadest range of enterprise-class software components and tools for the Microsoft .NET platform.
            * &lt;/p&gt;
            * &lt;/div&gt;
            * &lt;/div&gt;
            * &lt;style&gt;
            * .list {
            *   border-bottom: 1px solid;
            *   line-height: 50px;
            *   text-align: center;
            *   width:200px;
            *   }
            * &lt;/style&gt;
            * &lt;div &gt;
            * &lt;div class="list"&gt; Home &lt;/div&gt;
            * &lt;div class="list"&gt; Communities &lt;/div&gt;
            * &lt;/div&gt;
            * &lt;script&gt;
            * $(function () {
            * $("#navpane").ejNavigationDrawer("width","200");	
            * });
            * &lt;/script&gt;
            * @example 
            * &lt;script&gt;
            * //Get or set the Navigation Drawer width, after initialization:
            * $(function () {
            * // Gets the width API value.		
            * $("#navpane").ejNavigationDrawer("option", "width");	
            * // Sets the width API	
            * $("#navpane").ejNavigationDrawer ("option", "width", "overlay");  
            * });
            * &lt;/script&gt;  
            * @memberof ejNavigationDrawer
            * @instance
            */
            //Mobile
            /**		
            * Specifies the width of the control
            * @default auto 
            * @member width {object}
            * @type {int}
            * @example 
            * //Set the width property in unobtrusive way.
            * &lt;div id="home" class="navsubpage"&gt;
            * &lt;div align="center" class="content"&gt;
            * &lt;h2 class="title"&gt;
            * Home&lt;/h2&gt;
            * &lt;p&gt;
            * Founded by industry experts in 2001,Syncfusion, Inc. provides the broadest range of enterprise-class software components and tools for the Microsoft .NET platform.
            * &lt;/p&gt;
            * &lt;/div&gt;
            * &lt;/div&gt;
            * &lt;style&gt;
            * .list {
            *   border-bottom: 1px solid;
            *   line-height: 50px;
            *   text-align: center;
            *   width:200px;
            *   }
            * &lt;/style&gt;
            * &lt;div data-role="ejmnavigationdrawer" id="navpane" data-ej-width="200" &gt;
            * &lt;div class="list"&gt; Home &lt;/div&gt;
            * &lt;div class="list"&gt; Communities &lt;/div&gt;
            * &lt;/div&gt;
            * @example
            * // Set Navigation Drawer width on initialization. 
            * //To set width API value 
            * &lt;div id="home" class="navsubpage"&gt;
            * &lt;div align="center" class="content"&gt;
            * &lt;h2 class="title"&gt;
            * Home&lt;/h2&gt;
            * &lt;p&gt;
            * Founded by industry experts in 2001,Syncfusion, Inc. provides the broadest range of enterprise-class software components and tools for the Microsoft .NET platform.
            * &lt;/p&gt;
            * &lt;/div&gt;
            * &lt;/div&gt;
            * &lt;style&gt;
            * .list {
            *   border-bottom: 1px solid;
            *   line-height: 50px;
            *   text-align: center;
            *   width:200px;
            *   }
            * &lt;/style&gt;
            * &lt;div &gt;
            * &lt;div class="list"&gt; Home &lt;/div&gt;
            * &lt;div class="list"&gt; Communities &lt;/div&gt;
            * &lt;/div&gt;
            * &lt;script&gt;
            * $(function () {
            * $("#navpane").ejmNavigationDrawer("width","200");	
            * });
            * @example 
            * //Get or set the Navigation Drawer width, after initialization:
            * &lt;script&gt;
            * // Gets the type API value.		
            *  $("#navpane").ejmNavigationDrawer ("option", "width");		
            * // Sets the type API
            * $("#navpane").ejmNavigationDrawer ("option", "width", "auto");   
            * &lt;/script&gt;
            * @memberof ejmNavigationDrawer
            * @instance
            */
            width: "auto",
            //Web
            /**    
            * Event triggers when the Swipe happens.
            * @event
            * @name ejNavigationDrawer#swipe 	
            * @param {Object} argument Event parameters from Navigation Drawer     
            * @param {boolean} argument.cancel if the event should be canceled; otherwise, false.
            * @param {Object} argument.model returns the Navigation Drawer model
            * @param {string} argument.type returns the name of the event
            * @param {object} argument.item returns the item of element
            * @param {String} argument.itemName returns the name of item
            * @example 
            * &lt;div id="home" class="navsubpage"&gt;
            * &lt;div align="center" class="content"&gt;
            * &lt;h2 class="title"&gt;
            * Home&lt;/h2&gt;
            * &lt;p&gt;
            * Founded by industry experts in 2001,Syncfusion, Inc. provides the broadest range of enterprise-class software components and tools for the Microsoft .NET platform.
            * &lt;/p&gt;
            * &lt;/div&gt;
            * &lt;/div&gt;
            * &lt;style&gt;
            * .list {
            *   border-bottom: 1px solid;
            *   line-height: 50px;
            *   text-align: center;
            *   width:200px;
            *   }
            * &lt;/style&gt;
            * &lt;div data-role="ejnavigationdrawer" id="navpane" data-ej-swipe="onSwipe" &gt;
            * &lt;div class="list"&gt; Home &lt;/div&gt;
            * &lt;div class="list"&gt; Communities &lt;/div&gt;
            * &lt;/div&gt;
            * &lt;script &gt;
            * function onSwipe(args)
            * { //handle the event
            * }
            * &lt;/script &gt;
            * @example 
            * //Swipe event for Navigation pane
            * &lt;div id="home" class="navsubpage"&gt;
            * &lt;div align="center" class="content"&gt;
            * &lt;h2 class="title"&gt;
            * Home&lt;/h2&gt;
            * &lt;p&gt;
            * Founded by industry experts in 2001,Syncfusion, Inc. provides the broadest range of enterprise-class software components and tools for the Microsoft .NET platform.
            * &lt;/p&gt;
            * &lt;/div&gt;
            * &lt;/div&gt;
            * &lt;style&gt;
            * .list {
            *   border-bottom: 1px solid;
            *   line-height: 50px;
            *   text-align: center;
            *   width:200px;
            *   }
            * &lt;/style&gt;
            * &lt;div id="navpane" &gt;
            * &lt;div class="list"&gt; Home &lt;/div&gt;
            * &lt;div class="list"&gt; Communities &lt;/div&gt;
            * &lt;/div&gt;
            * &lt;script&gt;
            * $("#navpane").ejNavigationDrawer({
            *   swipe: function (args) { //handle the event
            * }
            * });   
            * &lt;/script&gt;
            * @memberof ejNavigationDrawer
            * @instance
            */
            //Mobile
            /**    
            * Event triggers when the Swipe happens.
            * @event
            * @name ejmNavigationDrawer#swipe 	
            * @param {Object} argument Event parameters from Navigation Drawer     
            * @param {boolean} argument.cancel if the event should be canceled; otherwise, false.
            * @param {Object} argument.model returns the Navigation Drawer model
            * @param {string} argument.type returns the name of the event
            * @param {object} argument.item returns the item of element
            * @param {String} argument.itemName returns the name of item
            * @example 
            * &lt;div id="home" class="navsubpage"&gt;
            * &lt;div align="center" class="content"&gt;
            * &lt;h2 class="title"&gt;
            * Home&lt;/h2&gt;
            * &lt;p&gt;
            * Founded by industry experts in 2001,Syncfusion, Inc. provides the broadest range of enterprise-class software components and tools for the Microsoft .NET platform.
            * &lt;/p&gt;
            * &lt;/div&gt;
            * &lt;/div&gt;
            * &lt;style&gt;
            * .list {
            *   border-bottom: 1px solid;
            *   line-height: 50px;
            *   text-align: center;
            *   width:200px;
            *   }
            * &lt;/style&gt;
            * &lt;div data-role="ejmnavigationdrawer" id="navpane" data-ej-swipe="onSwipe" &gt;
            * &lt;div class="list"&gt; Home &lt;/div&gt;
            * &lt;div class="list"&gt; Communities &lt;/div&gt;
            * &lt;/div&gt;
            * &lt;script &gt;
            * function onSwipe(args)
            * { //handle the event
            * }
            * &lt;/script &gt;
            * @example 
            * //Swipe event for Navigation pane
            * &lt;div id="home" class="navsubpage"&gt;
            * &lt;div align="center" class="content"&gt;
            * &lt;h2 class="title"&gt;
            * Home&lt;/h2&gt;
            * &lt;p&gt;
            * Founded by industry experts in 2001,Syncfusion, Inc. provides the broadest range of enterprise-class software components and tools for the Microsoft .NET platform.
            * &lt;/p&gt;
            * &lt;/div&gt;
            * &lt;/div&gt;
            * &lt;style&gt;
            * .list {
            *   border-bottom: 1px solid;
            *   line-height: 50px;
            *   text-align: center;
            *   width:200px;
            *   }
            * &lt;/style&gt;
            * &lt;div id="navpane" &gt;
            * &lt;div class="list"&gt; Home &lt;/div&gt;
            * &lt;div class="list"&gt; Communities &lt;/div&gt;
            * &lt;/div&gt;
            * &lt;script&gt;
            * $("#navpane").ejmNavigationDrawer({
            *   swipe: function (args) { //handle the event
            * }
            * });   
            * &lt;/script&gt;
            * @memberof ejmNavigationDrawer
            * @instance
            */
            swipe: null,
            //Web
            /**    
            * Event triggers when the control open.
            * @event
            * @name ejNavigationDrawer#open 	
            * @param {Object} argument Event parameters from Navigation Drawer     
            * @param {boolean} argument.cancel if the event should be canceled; otherwise, false.
            * @param {Object} argument.model returns the Navigation Drawer model
            * @param {string} argument.type returns the name of the event
            * @param {object} argument.item returns the item of element
            * @param {String} argument.itemName returns the name of item
            * @example 
            * &lt;div id="home" class="navsubpage"&gt;
            * &lt;div align="center" class="content"&gt;
            * &lt;h2 class="title"&gt;
            * Home&lt;/h2&gt;
            * &lt;p&gt;
            * Founded by industry experts in 2001,Syncfusion, Inc. provides the broadest range of enterprise-class software components and tools for the Microsoft .NET platform.
            * &lt;/p&gt;
            * &lt;/div&gt;
            * &lt;/div&gt;
            * &lt;style&gt;
            * .list {
            *   border-bottom: 1px solid;
            *   line-height: 50px;
            *   text-align: center;
            *   width:200px;
            *   }
            * &lt;/style&gt;
            * &lt;div data-role="ejnavigationdrawer" id="navpane" data-ej-open="onOpen" &gt;
            * &lt;div class="list"&gt; Home &lt;/div&gt;
            * &lt;div class="list"&gt; Communities &lt;/div&gt;
            * &lt;/div&gt;
            * &lt;script &gt;
            * function onOpen(args)
            * { //handle the event
            * }
            * &lt;/script &gt;
            * @example 
            * //Open event for Navigation pane
            * &lt;div id="home" class="navsubpage"&gt;
            * &lt;div align="center" class="content"&gt;
            * &lt;h2 class="title"&gt;
            * Home&lt;/h2&gt;
            * &lt;p&gt;
            * Founded by industry experts in 2001,Syncfusion, Inc. provides the broadest range of enterprise-class software components and tools for the Microsoft .NET platform.
            * &lt;/p&gt;
            * &lt;/div&gt;
            * &lt;/div&gt;
            * &lt;style&gt;
            * .list {
            *   border-bottom: 1px solid;
            *   line-height: 50px;
            *   text-align: center;
            *   width:200px;
            *   }
            * &lt;/style&gt;
            * &lt;div id="navpane" &gt;
            * &lt;div class="list"&gt; Home &lt;/div&gt;
            * &lt;div class="list"&gt; Communities &lt;/div&gt;
            * &lt;/div&gt;
            * &lt;script&gt;
            * $("#navpane").ejNavigationDrawer({
            *   open: function (args) { //handle the event
            * }
            * });   
            * &lt;/script&gt;
            * @memberof ejNavigationDrawer
            * @instance
            */
            //Mobile
            /**    
            * Event triggers when the control open.
            * @event
            * @name ejmNavigationDrawer#open 	
            * @param {Object} argument Event parameters from Navigation Drawer     
            * @param {boolean} argument.cancel if the event should be canceled; otherwise, false.
            * @param {Object} argument.model returns the Navigation Drawer model
            * @param {string} argument.type returns the name of the event
            * @param {object} argument.item returns the item of element
            * @param {String} argument.itemName returns the name of item
            * @example 
            * &lt;div id="home" class="navsubpage"&gt;
            * &lt;div align="center" class="content"&gt;
            * &lt;h2 class="title"&gt;
            * Home&lt;/h2&gt;
            * &lt;p&gt;
            * Founded by industry experts in 2001,Syncfusion, Inc. provides the broadest range of enterprise-class software components and tools for the Microsoft .NET platform.
            * &lt;/p&gt;
            * &lt;/div&gt;
            * &lt;/div&gt;
            * &lt;style&gt;
            * .list {
            *   border-bottom: 1px solid;
            *   line-height: 50px;
            *   text-align: center;
            *   width:200px;
            *   }
            * &lt;/style&gt;
            * &lt;div data-role="ejmnavigationdrawer" id="navpane" data-ej-open="onOpen" &gt;
            * &lt;div class="list"&gt; Home &lt;/div&gt;
            * &lt;div class="list"&gt; Communities &lt;/div&gt;
            * &lt;/div&gt;
            * &lt;script &gt;
            * function onOpen(args)
            * { //handle the event
            * }
            * &lt;/script &gt;
            * @example 
            * //Open event for Navigation pane
            * &lt;div id="home" class="navsubpage"&gt;
            * &lt;div align="center" class="content"&gt;
            * &lt;h2 class="title"&gt;
            * Home&lt;/h2&gt;
            * &lt;p&gt;
            * Founded by industry experts in 2001,Syncfusion, Inc. provides the broadest range of enterprise-class software components and tools for the Microsoft .NET platform.
            * &lt;/p&gt;
            * &lt;/div&gt;
            * &lt;/div&gt;
            * &lt;style&gt;
            * .list {
            *   border-bottom: 1px solid;
            *   line-height: 50px;
            *   text-align: center;
            *   width:200px;
            *   }
            * &lt;/style&gt;
            * &lt;div id="navpane" &gt;
            * &lt;div class="list"&gt; Home &lt;/div&gt;
            * &lt;div class="list"&gt; Communities &lt;/div&gt;
            * &lt;/div&gt;
            * &lt;script&gt;
            * $("#navpane").ejNavigationDrawer({
            *   open: function (args) { //handle the event
            * }
            * });   
            * &lt;/script&gt;
            * @memberof ejmNavigationDrawer
            * @instance
            */
            open: null,
            //Web
            /**    
            * Event triggers before the control gets closed.
            * @event
            * @name ejNavigationDrawer#beforeclose 	
            * @param {Object} argument Event parameters from Navigation Drawer     
            * @param {boolean} argument.cancel if the event should be canceled; otherwise, false.
            * @param {Object} argument.model returns the Navigation Drawer model
            * @param {string} argument.type returns the name of the event
            * @param {object} argument.item returns the item of element
            * @param {String} argument.itemName returns the name of item
            * @example 
            * &lt;div id="home" class="navsubpage"&gt;
            * &lt;div align="center" class="content"&gt;
            * &lt;h2 class="title"&gt;
            * Home&lt;/h2&gt;
            * &lt;p&gt;
            * Founded by industry experts in 2001,Syncfusion, Inc. provides the broadest range of enterprise-class software components and tools for the Microsoft .NET platform.
            * &lt;/p&gt;
            * &lt;/div&gt;
            * &lt;/div&gt;
            * &lt;style&gt;
            * .list {
            *   border-bottom: 1px solid;
            *   line-height: 50px;
            *   text-align: center;
            *   width:200px;
            *   }
            * &lt;/style&gt;
            * &lt;div data-role="ejnavigationdrawer" id="navpane" data-ej-beforeclose="onBeforeClose" &gt;
            * &lt;div class="list"&gt; Home &lt;/div&gt;
            * &lt;div class="list"&gt; Communities &lt;/div&gt;
            * &lt;/div&gt;
            * &lt;script &gt;
            * function onBeforeClose(args)
            * { //handle the event
            * }
            * &lt;/script &gt;
            * @example 
            * //BeforeClose event for Navigation pane
            * &lt;div id="home" class="navsubpage"&gt;
            * &lt;div align="center" class="content"&gt;
            * &lt;h2 class="title"&gt;
            * Home&lt;/h2&gt;
            * &lt;p&gt;
            * Founded by industry experts in 2001,Syncfusion, Inc. provides the broadest range of enterprise-class software components and tools for the Microsoft .NET platform.
            * &lt;/p&gt;
            * &lt;/div&gt;
            * &lt;/div&gt;
            * &lt;style&gt;
            * .list {
            *   border-bottom: 1px solid;
            *   line-height: 50px;
            *   text-align: center;
            *   width:200px;
            *   }
            * &lt;/style&gt;
            * &lt;div id="navpane" &gt;
            * &lt;div class="list"&gt; Home &lt;/div&gt;
            * &lt;div class="list"&gt; Communities &lt;/div&gt;
            * &lt;/div&gt;
            * &lt;script&gt;
            * $("#navpane").ejNavigationDrawer({
            *   beforeClose: function (args) { //handle the event
            * }
            * });   
            * &lt;/script&gt;
            * @memberof ejNavigationDrawer
            * @instance
            */
            //Mobile
            /**    
            * Event triggers before the control gets closed.
            * @event
            * @name ejmNavigationDrawer#beforeclose 	
            * @param {Object} argument Event parameters from Navigation Drawer     
            * @param {boolean} argument.cancel if the event should be canceled; otherwise, false.
            * @param {Object} argument.model returns the Navigation Drawer model
            * @param {string} argument.type returns the name of the event
            * @param {object} argument.item returns the item of element
            * @param {String} argument.itemName returns the name of item
            * @example  
            * &lt;div id="home" class="navsubpage"&gt;
            * &lt;div align="center" class="content"&gt;
            * &lt;h2 class="title"&gt;
            * Home&lt;/h2&gt;
            * &lt;p&gt;
            * Founded by industry experts in 2001,Syncfusion, Inc. provides the broadest range of enterprise-class software components and tools for the Microsoft .NET platform.
            * &lt;/p&gt;
            * &lt;/div&gt;
            * &lt;/div&gt;
            * &lt;style&gt;
            * .list {
            *   border-bottom: 1px solid;
            *   line-height: 50px;
            *   text-align: center;
            *   width:200px;
            *   }
            * &lt;/style&gt;
            * &lt;div data-role="ejmnavigationdrawer" id="navpane" data-ej-beforeclose="onBeforeClose" &gt;
            * &lt;div class="list"&gt; Home &lt;/div&gt;
            * &lt;div class="list"&gt; Communities &lt;/div&gt;
            * &lt;/div&gt;
            * &lt;script &gt;
            * function onBeforeClose(args)
            * { //handle the event
            * }
            * &lt;/script &gt;
            * @example 
            * //BeforeClose event for Navigation pane
            * &lt;div id="home" class="navsubpage"&gt;
            * &lt;div align="center" class="content"&gt;
            * &lt;h2 class="title"&gt;
            * Home&lt;/h2&gt;
            * &lt;p&gt;
            * Founded by industry experts in 2001,Syncfusion, Inc. provides the broadest range of enterprise-class software components and tools for the Microsoft .NET platform.
            * &lt;/p&gt;
            * &lt;/div&gt;
            * &lt;/div&gt;
            * &lt;style&gt;
            * .list {
            *   border-bottom: 1px solid;
            *   line-height: 50px;
            *   text-align: center;
            *   width:200px;
            *   }
            * &lt;/style&gt;
            * &lt;div id="navpane" &gt;
            * &lt;div class="list"&gt; Home &lt;/div&gt;
            * &lt;div class="list"&gt; Communities &lt;/div&gt;
            * &lt;/div&gt;
            * &lt;script&gt;
            * $("#navpane").ejmNavigationDrawer({
            *   beforeClose: function (args) { //handle the event
            * }
            * });   
            * &lt;/script&gt;
            * @memberof ejmNavigationDrawer
            * @instance
            */
            beforeClose: null
        },
        //datatype of model variables
        dataTypes: {
            direction: "enum",
            type: "enum"
        },
        //renders the control
        _renderControl: function () {
            this._elementOverlay = ej.buildTag("div#" + this._id + "_Overlay", {}, {}, { class: this._rootCSS + " " + this._prefixClass + "nb-layout " + this._prefixClass + "nb-overlay" });
            this._elementShadow = ej.buildTag("div#" + this._id + "_shadow", {}, {}, { class: this._rootCSS + " " + this._prefixClass + "nb-shadow " + "" + this._prefixClass + "nb-type-" + (this.model.type == "slide" ? "slide" : "overlay") });
            this._elementOverlay.css("z-index", ej.getMaxZindex() + 1);
            this.element.css("z-index", ej.getMaxZindex() + 2);
            this.element.addClass(this._prefixClass + "user-select");
            this._nbHome.append(this._elementOverlay);
            this._elementOverlay.hide();
            this.element.hide();
            this.element.addClass(this._prefixClass + 'nb-layout ' + (this.model.direction.toLowerCase() == "left" ? this._prefixClass + 'nb-left' : this._prefixClass + 'nb-right'));
            this.element.addClass(this._prefixClass + 'nb-type-' + (this.model.type == "slide" ? "slide" : "overlay"));
            this.element.prepend(this._elementShadow);
            this._setWidth();
            this._setLayout();
        },
        /**
        * To configure the transform property for Navigation drawer control
        * @private
        */
        _transform: function (dist, time, element) {
            var translate = "-" + this._browser + "-transform",
                translateVal = "translate3d(" + dist + "px,0px,0px) scale(1)";
            var trans = "-" + this._browser + "-transition-property",
            transVal = "transform",
            transDur = "-" + this._browser + "-transition-duration",
            transDurVal = time + "ms";
            if (element) {
                this.element.addClass("" + this._prefixClass + "nb-animate");
                this.element.css(translate, translateVal).css(trans, transVal).css(transDur, transDurVal);
            }
            else {
                this._nbHome.addClass(this._rootCSS + " " + this._prefixClass + "nb-animate");
                this._nbHome.css(translate, translateVal).css(trans, transVal).css(transDur, transDurVal)
            }
        },
        /**
        * To configure the show property for Navigation drawer control
        * @private
        */
        _show: function () {
            this.element.show();
            this.element.addClass("" + this._prefixClass + "nb-opened");
            this.model.direction == "left" ? this._transform($(this.element).width(), 400, this._isTransitionElement()) : this._transform(-($(this.element).width()), 400, this._isTransitionElement());
            this._elementOverlay.show();
            this._data = { element: this.element };
            this.model.enableListbox ? $.extend(this._data, { listbox: this._lb }) : undefined;
            this._trigger("open", this._data);
        },
        /**
        * To configure the hide property for Navigation drawer control
        * @private
        */
        _hide: function () {
            ej.listenTouchEvent((this._isTransitionElement() ? this.element : this._nbHome), ej.transitionEndEvent(), this._transitionCloseHandler, false);
            this.element.removeClass("" + this._prefixClass + "nb-opened");
            this._transform(0, 400, this._isTransitionElement());
            this._elementOverlay.hide();
        },


        _isOpened: function () {
            return this.element.hasClass("" + this._prefixClass + "nb-opened");
        },

        _isOtherOpened: function () {
            return $("." + this._prefixClass + "nb-opened").length;
        },

        _isTransitionElement: function () {
            return this.model.type == "overlay" ? true : false;
        },
        /**
        * To configure the width property for Navigation drawer control
        * @private
        */
        _setWidth: function () {
            $(this._nbHome).width() < 500 ? this.element.css("maxwidth", "80%") : this.element.css("maxwidth", "40%");
            this.element.height("100%");
        },

        _setLayout: function () {
            this.element.show();
            if (this.model.direction.toLowerCase() == "left") {
                this.element.offset({ left: this._nbHome.offset().left != 0 ? -((this.element.width() - this._nbHome.offset().left)) : -(this.element.width()) });
            }
            else {
                this.element.offset({ left: this._nbHome.offset().left != 0 ? ($(this._nbHome).width() + this._nbHome.offset().left) : $(this._nbHome).width() });
            }
            this.element.hide();
        },
        /**
        * To configure the destroy property for Navigation drawer control
        * @private
        */
        _destroy: function () {
            this._wireEvents(true);
            this._elementOverlay.remove();
            this._elementShadow.remove();
            if (this._elementWrapper) this.element.unwrap();
            this.element.removeAttr("class style");
            this.element.offset({ left: 0 });
            this._destroyEJMPlugin();
        },
        /**
        * To configure the refresh property for Navigation drawer control
        * @private
        */
        _refresh: function () {
            this._destroy();
            this._init();
        },

        /**
        * Creates Delegates for the functions 
        * @private
        */
        _createDelegate: function () {
            //Swipe Events Handler
            this._swipeStartHandler = $.proxy(this._swipeStart, this);
            this._swipeEndHandler = $.proxy(this._swipeEnd, this);
            this._swipeMoveHandler = $.proxy(this._swipeMove, this);

            //Touch Event Handler
            this._touchMoveHandler = $.proxy(this._touchMove, this);
            this._touchEndHandler = $.proxy(this._touchEnd, this);

            //Transition Event
            this._transitionCloseHandler = $.proxy(this._transitionCloseEnd, this);

            //Target Touch Event Handler
            this._targetTouchStartHandler = $.proxy(this._targetTouchStart, this);
            this._targetTouchMoveHandler = $.proxy(this._targetTouchMove, this);
            this._targetTouchEndHandler = $.proxy(this._targetTouchEnd, this);

            // Size Event Handler
            this._onOrientationChangeHandler = $.proxy(this._onOrientationChangeEvent, this);
            this._onResizeEventHandler = $.proxy(this._onResizeEvent, this);
        },

        _wireEvents: function (remove) {
            //Event Binding
            if (this.model.targetId) {
                if (ej.getCurrentPage)
                    var ele = ej.getCurrentPage();
                else
                    var ele = $("body")
                ej.listenEvents([ele.find("#" + this.model.targetId)], [ej.endEvent()], [this._targetTouchEndHandler], remove);
            }
            else {
                ej.listenEvents([this._nbHome, this._nbHome], [ej.startEvent(), ej.endEvent()], [this._swipeStartHandler, this._swipeEndHandler], remove);
            }
            //Ovelay Binding
            ej.listenTouchEvent(this._elementOverlay, ej.endEvent(), this._targetTouchEndHandler, remove);

            //Resize Events
            if (ej.isTouchDevice())
                ej.listenEvents([window, window], ["orientationchange", "resize"], [this._onOrientationChangeHandler, this._onResizeEventHandler], remove);
            else
                ej.listenTouchEvent(window, "resize", this._onResizeEventHandler, remove);
        },

        _transitionCloseEnd: function () {
            this.element.hide();
            ej.listenTouchEvent((this._isTransitionElement() ? this.element : this._nbHome), ej.transitionEndEvent(), this._transitionCloseHandler, true);

        },
        /**
        * Configure touchend on Navigation Drawer  
        * @private
        */
        _targetTouchEnd: function (e) {
            e.preventDefault();
            if (ej.isTouchDevice())
                e = e.changedTouches[0];
            if (this._isOpened()) {
                if ((Math.abs(this._relativeX) < 50 || Math.abs(this._relativeX) < -50) && e.clientX != this._startSwipeOffsetActualX) {
                    this._show();
                }
                else {
                    this._data = { element: this.element };
                    this.model.enableListbox ? $.extend(this._data, { listbox: this._lb }) : undefined;
                    this._trigger("beforeclose", this._data);
                    this._hide();
                }
            } else {
                if (ej.isIOS7) {
                    if (!ej.isIOS7() || e.target.id == this.model.targetId) if (!this._isOtherOpened()) this._show();
                }
                else
                    if (e.target.id == this.model.targetId) if (!this._isOtherOpened()) this._show();
            }
        },
        /**
        * Swipe start on Navigation drawer 
        * @private
        */
        _swipeStart: function (e) {
            if (ej.isTouchDevice())
                e = e.changedTouches[0];
            this._startSwipeOffsetX = e.clientX - this._nbHome.offset().left;
            this._startSwipeOffsetActualX = e.clientX;
            this._containerX = this._isTransitionElement() ? this.element.offset().left : this._nbHome.offset().left;
            ej.listenTouchEvent(this._nbHome, ej.moveEvent(), this._swipeMoveHandler, false);
        },
        /**
        * Swipe Move on Navigation drawer  
        * @private
        */
        _swipeMove: function (e) {
            if (ej.isTouchDevice())
                e = e.changedTouches[0];
            var allowOpDirection = this._isOpened() ? (this._swipeDirection != this.model.direction.toLowerCase() ? true : false) : false;
            if (this._startSwipeOffsetX && this._startSwipeOffsetX > (e.clientX - this._nbHome.offset().left)) { this._swipeDirection = "right"; }
            else { this._swipeDirection = "left"; }
            if (!allowOpDirection) {
                if (this._swipeDirection == this.model.direction.toLowerCase() && !this._isOtherOpened()) {
                    ej.listenEvents([this._nbHome, this._nbHome, this._nbHome], [ej.moveEvent(), ej.endEvent()], [this._touchMoveHandler, this._touchEndHandler], false);
                }
                else {
                    ej.listenEvents([this._nbHome, this._nbHome, this._nbHome], [ej.moveEvent(), ej.endEvent()], [this._touchMoveHandler, this._touchEndHandler], true);
                }
            }
            else {
                if (this._startSwipeOffsetX && this._isOpened()) {
                    this._relativeX = e.clientX - this._startSwipeOffsetX;
                    this._relativeY = e.clientY - this._startSwipeOffsetY;
                    if ($(e.target).hasClass("" + this._prefixClass + "nb-overlay")) {
                        if (($.isFunction(ej.isIOS7) ? ej.isIOS7() : false)) {
                            this._hide();
                            ej.listenTouchEvent(this._nbHome, ej.moveEvent(), this._swipeMoveHandler, true);
                        }
                        else
                            if (this.model.direction == "left" ? -(this._relativeX) < this.element.width() : this._relativeX < this.element.width())
                                this._transform((this.model.direction == "left" ? (this.element.width() + this._relativeX) - this._nbHome.offset().left : -((this.element.width() - this._relativeX) - this._nbHome.offset().left)), 0, this._isTransitionElement());
                    }

                }
            }
        },
        /**
        * Swipe end on Navigation drawer  
        * @private
        */
        _swipeEnd: function (e) {
            if (ej.isTouchDevice())
                e = e.changedTouches[0];
            ej.listenTouchEvent(this._nbHome, ej.moveEvent(), this._swipeMoveHandler, true);
        },
        /**
        * Touch move on Navigation drawer  
        * @private
        */
        _touchMove: function (e) {
            if (ej.isTouchDevice())
                e = e.changedTouches[0];
            ej.listenTouchEvent(this._nbHome, ej.moveEvent(), this._swipeMoveHandler, true);
            if (this._isTransitionElement()) { this._containerX = 0; }
            if (this._startSwipeOffsetX && !this._isOpened()) {
                this._relativeX = e.clientX - this._startSwipeOffsetX;
                this._relativeY = e.clientY - this._startSwipeOffsetY;
                if (this.model.direction == "left" ? this._startSwipeOffsetX < 50 : this._startSwipeOffsetX > this._nbHome.width() - 50 && !($.isFunction(ej.isIOS7) ? ej.isIOS7() : false)) {
                    this.element.show();
                    if (this.model.direction == "left" ? this._relativeX - this._nbHome.offset().left < this.element.width() : this._relativeX - this._nbHome.offset().left > -(this.element.width())) {
                        !this._containerX || this.model.targetHome ? this._transform((this._containerX + this._relativeX) - this._nbHome.offset().left, 0, this._isTransitionElement()) : undefined;
                    }
                }
                else {
                    if (($.isFunction(ej.isIOS7) ? ej.isIOS7() : false)) {
                        this._show();
                    }
                }
            }
        },
        /**
        * Touch end on Navigation drawer  
        * @private
        */
        _touchEnd: function (e) {
            if (ej.isTouchDevice())
                e = e.changedTouches[0];
            ej.listenTouchEvent(this._nbHome, ej.moveEvent(), this._touchMoveHandler, true);
            ej.listenTouchEvent((this._isTransitionElement() ? this.element : this._nbHome), ej.transitionEndEvent(), this._transitionCloseHandler, true);
            if (!this._isOpened() && !($.isFunction(ej.isIOS7) ? ej.isIOS7() : false)) {
                if (!this._containerX) {
                    this._relativeX = e.clientX - this._startSwipeOffsetX;
                    this._relativeY = e.clientY - this._startSwipeOffsetY;
                    if (this.model.direction == "left" ? this._startSwipeOffsetX < 50 : this._startSwipeOffsetX > this._nbHome.width() - 50) {
                        if (this._relativeX > 30 || this._relativeX < -30)
                            this._show();
                        else
                            this._hide();
                    }
                }
                else {
                    this._hide();
                }
            }
            this._data = { targetElement: this._nbHome, element: this.element, direction: this._swipeDirection };
            this.model.enableListbox ? $.extend(this._data, { listbox: this._lb }) : undefined;
            this._trigger("swipe", this._data);
            ej.listenTouchEvent(this._nbHome, ej.endEvent(), this._touchEndHandler, true);
            ej.listenTouchEvent(this._nbHome, "mouseleave", this._touchEndHandler, true);
        },
        _onOrientationChangeEvent: function (e) {
            var state;
            if (this._isOpened()) { state = "opned"; } else { state = "clsed" }
            this.element.show();
            if (window.orientation == 0 || window.orientation == 180 || window.orientation == 90) { window.scrollTo(0, 0); }
            this._transform(0, 0, this._isTransitionElement());
            this.element.hide();
            this.element.removeClass("" + this._prefixClass + "nb-opened");
            this._setWidth();
            this._setLayout();
            if (state == "opned") { this._show(); }
        },

        _onResizeEvent: function (e) {
            var state;
            if (this._isOpened()) { state = "opned"; } else { state = "clsed" }
            this.element.show();
            this._transform(0, 0, this._isTransitionElement());
            this.element.hide();
            this.element.removeClass("" + this._prefixClass + "nb-opened");
            this._setWidth();
            this._setLayout();
            if (state == "opned") { this._show(); }
        },
        /*---------------Public Methods---------------*/
        //Web
        /**
        * To open the navigation drawer control
        * @method open
        * @return jQuery
        * @example 
        * &lt;div id="home" class="navsubpage"&gt;
        * &lt;div align="center" class="content"&gt;
        * &lt;h2 class="title"&gt;
        * Home&lt;/h2&gt;
        * &lt;p&gt;
        * Founded by industry experts in 2001,Syncfusion, Inc. provides the broadest range of enterprise-class software components and tools for the Microsoft .NET platform.
        * &lt;/p&gt;
        * &lt;/div&gt;
        * &lt;/div&gt;
        * &lt;style&gt;
        * .list {
        *   border-bottom: 1px solid;
        *   line-height: 50px;
        *   text-align: center;
        *   width:200px;
        *   }
        * &lt;/style&gt;
        * &lt;div data-role="ejnavigationdrawer" id="navpane" &gt;
        * &lt;div class="list"&gt; Home &lt;/div&gt;
        * &lt;div class="list"&gt; Communities &lt;/div&gt;
        * &lt;/div&gt;
        * &lt;script &gt;
        * $(function()
        * { 
        * var lsm = $("#navpane").data("ejNavigationDrawer");
        * lsm.open();
        * });
        * &lt;/script &gt;
        * @memberof ejNavigationDrawer
        * @instance
        */
        //Mobile
        /**
        * To open the navigation drawer control
        * @return jQuery
        * @method open
        * @example 
        * &lt;div id="home" class="navsubpage"&gt;
        * &lt;div align="center" class="content"&gt;
        * &lt;h2 class="title"&gt;
        * Home&lt;/h2&gt;
        * &lt;p&gt;
        * Founded by industry experts in 2001,Syncfusion, Inc. provides the broadest range of enterprise-class software components and tools for the Microsoft .NET platform.
        * &lt;/p&gt;
        * &lt;/div&gt;
        * &lt;/div&gt;
        * &lt;style&gt;
        * .list {
        *   border-bottom: 1px solid;
        *   line-height: 50px;
        *   text-align: center;
        *   width:200px;
        *   }
        * &lt;/style&gt;
        * &lt;div data-role="ejmnavigationdrawer" id="navpane" &gt;
        * &lt;div class="list"&gt; Home &lt;/div&gt;
        * &lt;div class="list"&gt; Communities &lt;/div&gt;
        * &lt;/div&gt;
        * &lt;script &gt;
        * $(function()
        * { 
        * var lsm = $("#navpane").data("ejmNavigationDrawer");
        * lsm.open();
        * });
        * &lt;/script &gt;
        * @memberof ejmNavigationDrawer
        * @instance
        */
        open: function (e) {
            this._show();
        },
        //Web
        /**
        * To close the navigation drawer control
        * @method close
        * @return jQuery
        * @example 
        * &lt;div id="home" class="navsubpage"&gt;
        * &lt;div align="center" class="content"&gt;
        * &lt;h2 class="title"&gt;
        * Home&lt;/h2&gt;
        * &lt;p&gt;
        * Founded by industry experts in 2001,Syncfusion, Inc. provides the broadest range of enterprise-class software components and tools for the Microsoft .NET platform.
        * &lt;/p&gt;
        * &lt;/div&gt;
        * &lt;/div&gt;
        * &lt;style&gt;
        * .list {
        *   border-bottom: 1px solid;
        *   line-height: 50px;
        *   text-align: center;
        *   width:200px;
        *   }
        * &lt;/style&gt;
        * &lt;div data-role="ejnavigationdrawer" id="navpane" &gt;
        * &lt;div class="list"&gt; Home &lt;/div&gt;
        * &lt;div class="list"&gt; Communities &lt;/div&gt;
        * &lt;/div&gt;
        * &lt;script &gt;
        * $(function()
        * { 
        * var lsm = $("#navpane").data("ejNavigationDrawer");
        * lsm.close();
        * });
        * &lt;/script &gt;
        * @memberof ejNavigationDrawer
        * @instance
        */
        //Mobile
        /**
        * To close the navigation drawer control
        * @method close
        * @return jQuery
        * @example 
        * &lt;div id="home" class="navsubpage"&gt;
        * &lt;div align="center" class="content"&gt;
        * &lt;h2 class="title"&gt;
        * Home&lt;/h2&gt;
        * &lt;p&gt;
        * Founded by industry experts in 2001,Syncfusion, Inc. provides the broadest range of enterprise-class software components and tools for the Microsoft .NET platform.
        * &lt;/p&gt;
        * &lt;/div&gt;
        * &lt;/div&gt;
        * &lt;style&gt;
        * .list {
        *   border-bottom: 1px solid;
        *   line-height: 50px;
        *   text-align: center;
        *   width:200px;
        *   }
        * &lt;/style&gt;
        * &lt;div data-role="ejmnavigationdrawer" id="navpane" &gt;
        * &lt;div class="list"&gt; Home &lt;/div&gt;
        * &lt;div class="list"&gt; Communities &lt;/div&gt;
        * &lt;/div&gt;
        * &lt;script &gt;
        * $(function()
        * { 
        * var lsm = $("#navpane").data("ejmNavigationDrawer");
        * lsm.close();
        * });
        * &lt;/script &gt;
        * @memberof ejmNavigationDrawer
        * @instance
        */
        close: function (e) {
            this._data = { element: this.element };
            this.model.enableListbox ? $.extend(this._data, { listbox: this._lb }) : undefined;
            this._trigger("beforeClose", this._data);
            this._hide();
        },
        //Web
        /**
        * To Toggle the navigation drawer control
        * @method toggle
        * @return jQuery
        * @example 
        * &lt;div id="home" class="navsubpage"&gt;
        * &lt;div align="center" class="content"&gt;
        * &lt;h2 class="title"&gt;
        * Home&lt;/h2&gt;
        * &lt;p&gt;
        * Founded by industry experts in 2001,Syncfusion, Inc. provides the broadest range of enterprise-class software components and tools for the Microsoft .NET platform.
        * &lt;/p&gt;
        * &lt;/div&gt;
        * &lt;/div&gt;
        * &lt;style&gt;
        * .list {
        *   border-bottom: 1px solid;
        *   line-height: 50px;
        *   text-align: center;
        *   width:200px;
        *   }
        * &lt;/style&gt;
        * &lt;div data-role="ejnavigationdrawer" id="navpane" &gt;
        * &lt;div class="list"&gt; Home &lt;/div&gt;
        * &lt;div class="list"&gt; Communities &lt;/div&gt;
        * &lt;/div&gt;
        * &lt;script &gt;
        * $(function()
        * { 
        * var lsm = $("#navpane").data("ejNavigationDrawer");
        * lsm.toggle();
        * });
        * &lt;/script &gt;
        * @memberof ejNavigationDrawer
        * @instance
        */
        //Mobile
        /**
        * To Toggle the navigation drawer control
        * @method toggle
        * @return jQuery
        * @example 
        * &lt;div id="home" class="navsubpage"&gt;
        * &lt;div align="center" class="content"&gt;
        * &lt;h2 class="title"&gt;
        * Home&lt;/h2&gt;
        * &lt;p&gt;
        * Founded by industry experts in 2001,Syncfusion, Inc. provides the broadest range of enterprise-class software components and tools for the Microsoft .NET platform.
        * &lt;/p&gt;
        * &lt;/div&gt;
        * &lt;/div&gt;
        * &lt;style&gt;
        * .list {
        *   border-bottom: 1px solid;
        *   line-height: 50px;
        *   text-align: center;
        *   width:200px;
        *   }
        * &lt;/style&gt;
        * &lt;div data-role="ejmnavigationdrawer" id="navpane" &gt;
        * &lt;div class="list"&gt; Home &lt;/div&gt;
        * &lt;div class="list"&gt; Communities &lt;/div&gt;
        * &lt;/div&gt;
        * &lt;script &gt;
        * $(function()
        * { 
        * var lsm = $("#navpane").data("ejmNavigationDrawer");
        * lsm.toggle();
        * });
        * &lt;/script &gt;
        * @memberof ejmNavigationDrawer
        * @instance
        */
        toggle: function (e) {
            this._isOpened() ? this.close() : this.open();
        }
        /*---------------Public Methods End---------------*/
    });

})(jQuery, Syncfusion);;
/**
* @fileOverview Plugin to style the Html ListView elements
* @copyright Copyright Syncfusion Inc. 2001 - 2013. All rights reserved.
*  Use of this code is subject to the terms of our license.
*  A copy of the current license can be obtained at any time by e-mailing
*  licensing@syncfusion.com. Any infringement will be prosecuted under
*  applicable laws. 
* @version 12.1 
* @author <a href="mailto:licensing@syncfusion.com">Syncfusion Inc</a>
*/

(function ($, ej, undefined) {



    ej.widget("ejListViewBase", "ej.ListViewBase", {


        defaults: {
            // Web
            /**		
            * Specifies the height.
            * @default undefined
            * @member height {object}
            * @type {number}
            * @example 
            * //Set the height property in unobtrusive way.
            * &lt;div id="lb" data-role="ejlistview" data-ej-height="300" data-ej-allowScrolling=true &gt;
            *	 &lt;ul&gt;
            *		&lt;li data-ej-text="Artwork"&gt;&lt;/li&gt;
            * 		&lt;li data-ej-text="Abstract"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="2 Acrylic Mediums"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Creative Acrylic"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Modern Painting"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Canvas Art"&gt;&lt;/li&gt;
            * 		&lt;li data-ej-text="Black white"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Children"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Preschool Crafts"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="School-age Crafts"&gt;&lt;/li&gt;
            * 	&lt;/ul&gt;
            * &lt;/div&gt;
            * @example 
            * //Set the height property in obtrusive way.
            * &lt;div id="lb" data-ej-allowScrolling=true &gt;
            *	 &lt;ul&gt;
            *		&lt;li data-ej-text="Artwork"&gt;&lt;/li&gt;
            * 		&lt;li data-ej-text="Abstract"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="2 Acrylic Mediums"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Creative Acrylic"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Modern Painting"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Canvas Art"&gt;&lt;/li&gt;
            * 		&lt;li data-ej-text="Black white"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Children"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Preschool Crafts"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="School-age Crafts"&gt;&lt;/li&gt;
            * 	&lt;/ul&gt;
            * &lt;/div&gt;
            * &lt;script&gt;
            * // Set height on initialization. 
            * //To set height API value 
            * $("#lb").ejListView ({ height: 300 });
            * &lt;/script&gt;
            * @example 
            * &lt;script&gt;
            * //Get or set  height, after initialization:<br>
            * // Get the height API value.		
            *  $("#lb").ejListView ("option", "height");			
            * // Set the height API
            * $("#lb").ejListView ("option", "height", 300);
            * &lt;/script&gt;
            * @memberof ejListView
            * @instance
            */
            //Mobile
            /**		
            * Specifies the height of the ListView.
            * @default undefined
            * @member height {object}
            * @type {number}
            * @example 
            * //Set the height property in Unobtrusive way.
            * &lt;div id="lb" data-role="ejmlistview" data-ej-height="300"&gt;
            *	 &lt;ul&gt;
            *		&lt;li data-ej-text="Artwork"&gt;&lt;/li&gt;
            * 		&lt;li data-ej-text="Abstract"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="2 Acrylic Mediums"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Creative Acrylic"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Modern Painting"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Canvas Art"&gt;&lt;/li&gt;
            * 		&lt;li data-ej-text="Black white"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Children"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Preschool Crafts"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="School-age Crafts"&gt;&lt;/li&gt;
            * 	&lt;/ul&gt;
            * &lt;/div&gt;
            * @example 
            * // Set ListView height on initialization. 
            * //To set height API value 
            * $("#lb").ejmListView ({ height: 300 });
            * //Get or set the ListView height, after initialization:<br>
            * // Gets the height API value.		
            *  $("#lb").ejmListView ("option", "height");			
            * // Sets the height API
            * $("#lb").ejmListView ("option", "height", 300);
            * @memberof ejmListView
            * @instance
            */
            height: null,
            /**		
            * Specifies the width.
            * @default undefined
            * @member width {object}
            * @type {number}
            * @example 
            * //Set the width property in unobtrusive way.
            * &lt;div id="lb" data-role="ejlistview" data-ej-width="200"&gt;
            *	 &lt;ul&gt;
            *		&lt;li data-ej-text="Artwork"&gt;&lt;/li&gt;
            * 		&lt;li data-ej-text="Abstract"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="2 Acrylic Mediums"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Creative Acrylic"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Modern Painting"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Canvas Art"&gt;&lt;/li&gt;
            * 		&lt;li data-ej-text="Black white"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Children"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Preschool Crafts"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="School-age Crafts"&gt;&lt;/li&gt;
            * 	&lt;/ul&gt;
            * &lt;/div&gt;
            * @example 
            * //Set the width property in obtrusive way.
            * &lt;div id="lb"&gt;
            *	 &lt;ul&gt;
            *		&lt;li data-ej-text="Artwork"&gt;&lt;/li&gt;
            * 		&lt;li data-ej-text="Abstract"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="2 Acrylic Mediums"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Creative Acrylic"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Modern Painting"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Canvas Art"&gt;&lt;/li&gt;
            * 		&lt;li data-ej-text="Black white"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Children"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Preschool Crafts"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="School-age Crafts"&gt;&lt;/li&gt;
            * 	&lt;/ul&gt;
            * &lt;/div&gt;
            * &lt;script&gt;
            * // Set width on initialization. 
            * //To set width API value 
            * $("#lb").ejListView ({ width: 200 });
            * &lt;/script&gt;
            * @example 
            * &lt;script&gt;
            * //Get or set  width, after initialization:<br>
            * // Get the width API value.		
            *  $("#lb").ejListView ("option", "width");			
            * // Set the width API
            * $("#lb").ejListView ("option", "width", 200);
            * &lt;/script&gt;
            * @memberof ejListView
            * @instance
            */
            //Mobile
            /**		
            * Specifies the width of the ListView.
            * @default undefined
            * @member width {object}
            * @type {number}
            * @example 
            * //Set the width property in Unobtrusive way.
            * &lt;div id="lb" data-role="ejmlistview" data-ej-width="200"&gt;
            *	 &lt;ul&gt;
            *		&lt;li data-ej-text="Artwork"&gt;&lt;/li&gt;
            * 		&lt;li data-ej-text="Abstract"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="2 Acrylic Mediums"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Creative Acrylic"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Modern Painting"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Canvas Art"&gt;&lt;/li&gt;
            * 		&lt;li data-ej-text="Black white"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Children"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Preschool Crafts"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="School-age Crafts"&gt;&lt;/li&gt;
            * 	&lt;/ul&gt;
            * &lt;/div&gt;
            * @example 
            * // Set ListView width on initialization. 
            * //To set width API value 
            * $("#lb").ejmListView ({ width: 200 });
            * //Get or set the ListView width, after initialization:<br>
            * // Gets the width API value.		
            *  $("#lb").ejmListView ("option", "width");			
            * // Sets the width API
            * $("#lb").ejmListView ("option", "width", 200);
            * @memberof ejmListView
            * @instance
            */
            width: null,
            /**		
            * Specifies the index of item which need to be in selected state initially while loading.
            * @default 0
            * @member selectedItemIndex {object}
            * @type {number}
            * @example 
            * //Set the selectedItemIndex property in unobtrusive way.
            * &lt;div id="lb" data-role="ejlistview" data-ej-selectedItemIndex="2"&gt;
            *	 &lt;ul&gt;
            *		&lt;li data-ej-text="Artwork"&gt;&lt;/li&gt;
            * 		&lt;li data-ej-text="Abstract"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="2 Acrylic Mediums"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Creative Acrylic"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Modern Painting"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Canvas Art"&gt;&lt;/li&gt;
            * 		&lt;li data-ej-text="Black white"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Children"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Preschool Crafts"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="School-age Crafts"&gt;&lt;/li&gt;
            * 	&lt;/ul&gt;
            * &lt;/div&gt;
            * @example 
            * //Set the selectedItemIndex property in obtrusive way.
            * &lt;div id="lb"&gt;
            *	 &lt;ul&gt;
            *		&lt;li data-ej-text="Artwork"&gt;&lt;/li&gt;
            * 		&lt;li data-ej-text="Abstract"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="2 Acrylic Mediums"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Creative Acrylic"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Modern Painting"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Canvas Art"&gt;&lt;/li&gt;
            * 		&lt;li data-ej-text="Black white"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Children"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Preschool Crafts"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="School-age Crafts"&gt;&lt;/li&gt;
            * 	&lt;/ul&gt;
            * &lt;/div&gt;
            * &lt;script&gt;
            * // Set selectedItemIndex on initialization. 
            * //To set selectedItemIndex API value 
            * $("#lb").ejListView ({ selectedItemIndex: 2 });
            * &lt;/script&gt;
            * @example 
            * &lt;script&gt;
            * //Get or set  selectedItemIndex, after initialization:<br>
            * // Get the selectedItemIndex API value.		
            *  $("#lb").ejListView ("option", "selectedItemIndex");			
            * // Set the selectedItemIndex API
            * $("#lb").ejListView ("option", "selectedItemIndex", 2);
            * &lt;/script&gt;
            * @memberof ejListView
            * @instance
            */
            //Mobile
            /**		
            * Specifies the selectedItemIndex of the ListView.
            * @default 0
            * @member selectedItemIndex {object}
            * @type {number}
            * @example 
            * //Set the selectedItemIndex property in Unobtrusive way.
            * &lt;div id="lb" data-role="ejmlistview" data-ej-selectedItemIndex="2"&gt;
            *	 &lt;ul&gt;
            *		&lt;li data-ej-text="Artwork"&gt;&lt;/li&gt;
            * 		&lt;li data-ej-text="Abstract"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="2 Acrylic Mediums"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Creative Acrylic"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Modern Painting"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Canvas Art"&gt;&lt;/li&gt;
            * 		&lt;li data-ej-text="Black white"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Children"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Preschool Crafts"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="School-age Crafts"&gt;&lt;/li&gt;
            * 	&lt;/ul&gt;
            * &lt;/div&gt;
            * @example 
            * // Set ListView selectedItemIndex on initialization. 
            * //To set selectedItemIndex API value 
            * $("#lb").ejmListView ({ selectedItemIndex: 2 });
            * //Get or set the ListView selectedItemIndex, after initialization:<br>
            * // Gets the selectedItemIndex API value.		
            *  $("#lb").ejmListView ("option", "selectedItemIndex");			
            * // Sets the selectedItemIndex API
            * $("#lb").ejmListView ("option", "selectedItemIndex", 2);
            * @memberof ejmListView
            * @instance
            */
            selectedItemIndex: -1,
            /**		
            * Specifies whether to group the list item.
            * @default false
            * @member enableGroupList {object}
            * @type {boolean}
            * @example 
            * //Set the enableGroupList property in unobtrusive way.
            * &lt;div id="lb" data-role="ejlistview" data-ej-enableGroupList="true"&gt;
            * 	&lt;ul data-ej-grouplisttitle="Network"&gt;
            *		&lt;li data-ej-text="Airplane Mode"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Wi-Fi"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Notifications"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Location Services"&gt;&lt;/li&gt;
            *	&lt;/ul&gt;
            * 	&lt;ul data-ej-grouplisttitle="Apps"&gt;
            *		&lt;li data-ej-text="Sound"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Brightness"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Wallpaper"&gt;&lt;/li&gt;
            *	&lt;/ul&gt;
            * 	&lt;ul data-ej-grouplisttitle="Settings"&gt;
            *		&lt;li data-ej-text="General"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Brightness"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Wallpaper"&gt;&lt;/li&gt;
            *	&lt;/ul&gt;
            * &lt;/div&gt;
            * @example 
            * //Set the enableGroupList property in obtrusive way.
            * &lt;div id="lb"&gt;
            * 	&lt;ul data-ej-grouplisttitle="Network"&gt;
            *		&lt;li data-ej-text="Airplane Mode"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Wi-Fi"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Notifications"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Location Services"&gt;&lt;/li&gt;
            *	&lt;/ul&gt;
            * 	&lt;ul data-ej-grouplisttitle="Apps"&gt;
            *		&lt;li data-ej-text="Sound"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Brightness"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Wallpaper"&gt;&lt;/li&gt;
            *	&lt;/ul&gt;
            * 	&lt;ul data-ej-grouplisttitle="Settings"&gt;
            *		&lt;li data-ej-text="General"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Brightness"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Wallpaper"&gt;&lt;/li&gt;
            *	&lt;/ul&gt;
            * &lt;/div&gt;
            * &lt;script&gt;
            * // Set enableGroupList on initialization. 
            * //To set enableGroupList API value 
            * $("#lb").ejListView ({ enableGroupList: true });
            * &lt;/script&gt;
            * @example 
            * &lt;script&gt;
            * //Get or set  enableGroupList, after initialization:<br>
            * // Get the enableGroupList API value.		
            *  $("#lb").ejListView ("option", "enableGroupList");			
            * // Set the enableGroupList API
            * $("#lb").ejListView ("option", "enableGroupList", true);
            * &lt;/script&gt;
            * @memberof ejListView
            * @instance
            */
            //Mobile
            /**		
            * Specifies whether to group the list item.
            * @default false
            * @member enableGroupList {object}
            * @type {boolean}
            * @example 
            * //Set the enableGroupList property in unobtrusive way.
            * &lt;div id="lb" data-role="ejmlistview" data-ej-enableGroupList="true"&gt;
            * 	&lt;ul data-ej-grouplisttitle="Network"&gt;
            *		&lt;li data-ej-text="Airplane Mode"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Wi-Fi"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Notifications"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Location Services"&gt;&lt;/li&gt;
            *	&lt;/ul&gt;
            * 	&lt;ul data-ej-grouplisttitle="Apps"&gt;
            *		&lt;li data-ej-text="Sound"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Brightness"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Wallpaper"&gt;&lt;/li&gt;
            *	&lt;/ul&gt;
            * 	&lt;ul data-ej-grouplisttitle="Settings"&gt;
            *		&lt;li data-ej-text="General"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Brightness"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Wallpaper"&gt;&lt;/li&gt;
            *	&lt;/ul&gt;
            * &lt;/div&gt;
            * @example 
            * //Set the enableGroupList property in obtrusive way.
            * &lt;div id="lb"&gt;
            * 	&lt;ul data-ej-grouplisttitle="Network"&gt;
            *		&lt;li data-ej-text="Airplane Mode"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Wi-Fi"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Notifications"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Location Services"&gt;&lt;/li&gt;
            *	&lt;/ul&gt;
            * 	&lt;ul data-ej-grouplisttitle="Apps"&gt;
            *		&lt;li data-ej-text="Sound"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Brightness"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Wallpaper"&gt;&lt;/li&gt;
            *	&lt;/ul&gt;
            * 	&lt;ul data-ej-grouplisttitle="Settings"&gt;
            *		&lt;li data-ej-text="General"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Brightness"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Wallpaper"&gt;&lt;/li&gt;
            *	&lt;/ul&gt;
            * &lt;/div&gt;
            * &lt;script&gt;
            * // Set enableGroupList on initialization. 
            * //To set enableGroupList API value 
            * $("#lb").ejmListView ({ enableGroupList: true });
            * &lt;/script&gt;
            * @example 
            * &lt;script&gt;
            * //Get or set  enableGroupList, after initialization:<br>
            * // Get the enableGroupList API value.		
            *  $("#lb").ejmListView ("option", "enableGroupList");			
            * // Set the enableGroupList API
            * $("#lb").ejmListView ("option", "enableGroupList", true);
            * &lt;/script&gt;
            * @memberof ejmListView
            * @instance
            */
            enableGroupList: false,
            /**		
            * Specifies whether to load ajax content while selecting item.
            * @default false
            * @member enableAjax {object}
            * @type {boolean}
            * @example 
            * //Set the enableAjax property in unobtrusive way.
            * &lt;div id="lb" data-role="ejlistview"&gt;
            * 	&lt;ul&gt;
            *		&lt;li data-ej-text="Man of Steel" data-ej-primarykey="1" data-ej-href="load1.html" data-ej-enableajax="true"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="World War Z" data-ej-primarykey="2" data-ej-href="load2.html" data-ej-enableajax="true"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Monsters University" data-ej-primarykey="3" data-ej-href="load3.html" data-ej-enableajax="true"&gt;&lt;/li&gt;
            *	&lt;/ul&gt;
            * &lt;/div&gt;
            * @example  
            * //Set the enableAjax property in obtrusive way.
            * &lt;div id="lb"&gt;
            * 	&lt;ul&gt;
            *		&lt;li data-ej-text="Man of Steel" data-ej-primarykey="1" data-ej-href="load1.html" data-ej-enableajax="true"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="World War Z" data-ej-primarykey="2" data-ej-href="load2.html" data-ej-enableajax="true"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Monsters University" data-ej-primarykey="3" data-ej-href="load3.html" data-ej-enableajax="true"&gt;&lt;/li&gt;
            *	&lt;/ul&gt;
            * &lt;/div&gt;
            * &lt;script&gt;
            * // Set enableAjax on initialization. 
            * //To set enableAjax API value
            * $("#lb").ejListView ({ enableAjax: true });
            * &lt;/script&gt;
            * @example 
            * &lt;script&gt;
            * //Get or set  enableAjax, after initialization:<br>
            * // Get the enableAjax API value.		
            * $("#lb").ejListView ("option", "enableAjax");			
            * // Set the enableAjax API
            * $("#lb").ejListView ("option", "enableAjax", true);
            * &lt;/script&gt;
            * @memberof ejListView
            * @instance
            */
            //Mobile
            /**		
            * Specifies whether to load ajax content while selecting item.
            * @default false
            * @member enableAjax {object}
            * @type {boolean}
            * @example 
            * //Set the enableAjax property in unobtrusive way.
            * &lt;div id="lb" data-role="ejmlistview"&gt;
            * 	&lt;ul&gt;
            *		&lt;li data-ej-text="Man of Steel" data-ej-primarykey="1" data-ej-href="load1.html" data-ej-enableajax="true"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="World War Z" data-ej-primarykey="2" data-ej-href="load2.html" data-ej-enableajax="true"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Monsters University" data-ej-primarykey="3" data-ej-href="load3.html" data-ej-enableajax="true"&gt;&lt;/li&gt;
            *	&lt;/ul&gt;
            * &lt;/div&gt;
            * @example  
            * //Set the enableAjax property in obtrusive way.
            * &lt;div id="lb"&gt;
            * 	&lt;ul&gt;
            *		&lt;li data-ej-text="Man of Steel" data-ej-primarykey="1" data-ej-href="load1.html" data-ej-enableajax="true"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="World War Z" data-ej-primarykey="2" data-ej-href="load2.html" data-ej-enableajax="true"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Monsters University" data-ej-primarykey="3" data-ej-href="load3.html" data-ej-enableajax="true"&gt;&lt;/li&gt;
            *	&lt;/ul&gt;
            * &lt;/div&gt;
            * &lt;script&gt;
            * // Set enableAjax on initialization. 
            * //To set enableAjax API value
            * $("#lb").ejmListView ({ enableAjax: true });
            * &lt;/script&gt;
            * @example 
            * &lt;script&gt;
            * //Get or set  enableAjax, after initialization:<br>
            * // Get the enableAjax API value.		
            * $("#lb").ejmListView ("option", "enableAjax");			
            * // Set the enableAjax API
            * $("#lb").ejmListView ("option", "enableAjax", true);
            * &lt;/script&gt;
            * @memberof ejmListView
            * @instance
            */
            enableAjax: false,
            /**		
            * Specifies whether to enable caching the content.
            * @default false
            * @member enableCache {object}
            * @type {boolean}
            * @example 
            * //Set the enableCache property in unobtrusive way.
            * &lt;div id="lb" data-role="ejlistview" data-ej-enableCache="true"&gt;
            * 	&lt;ul&gt;
            *		&lt;li data-ej-text="Man of Steel" data-ej-primarykey="1" data-ej-href="load1.html" data-ej-enableajax="true" data-ej-enableCache="true"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="World War Z" data-ej-primarykey="2" data-ej-href="load2.html" data-ej-enableajax="true" data-ej-enableCache="true"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Monsters University" data-ej-primarykey="3" data-ej-href="load3.html" data-ej-enableajax="true" data-ej-enableCache="true"&gt;&lt;/li&gt;
            *	&lt;/ul&gt;
            * &lt;/div&gt;
            * @example 
            * //Set the enableCache property in obtrusive way.
            * &lt;div id="lb" data-role="ejlistview" data-ej-enableCache="true"&gt;
            * 	&lt;ul&gt;
            *		&lt;li data-ej-text="Man of Steel" data-ej-primarykey="1" data-ej-href="load1.html" data-ej-enableajax="true" data-ej-enableCache="true"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="World War Z" data-ej-primarykey="2" data-ej-href="load2.html" data-ej-enableajax="true" data-ej-enableCache="true"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Monsters University" data-ej-primarykey="3" data-ej-href="load3.html" data-ej-enableajax="true" data-ej-enableCache="true"&gt;&lt;/li&gt;
            *	&lt;/ul&gt;
            * &lt;/div&gt;
            * &lt;script&gt;
            * // Set enableCache on initialization. 
            * //To set enableCache API value 
            * $("#lb").ejListView ({ enableCache: true });
            * &lt;/script&gt;
            * @example 
            * &lt;script&gt;
            * //Get or set  enableCache, after initialization:<br>
            * // Get the enableCache API value.		
            * $("#lb").ejListView ("option", "enableCache");			
            * // Set the enableCache API
            * $("#lb").ejListView ("option", "enableCache", true);
            * &lt;/script&gt;
            * @memberof ejListView
            * @instance
            */
            //Mobile
            /**		
            * Specifies whether to enable caching the content.
            * @default false
            * @member enableCache {object}
            * @type {boolean}
            * @example 
            * //Set the enableCache property in unobtrusive way.
            * &lt;div id="lb" data-role="ejmlistview" data-ej-enableCache="true"&gt;
            * 	&lt;ul&gt;
            *		&lt;li data-ej-text="Man of Steel" data-ej-primarykey="1" data-ej-href="load1.html" data-ej-enableajax="true" data-ej-enableCache="true"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="World War Z" data-ej-primarykey="2" data-ej-href="load2.html" data-ej-enableajax="true" data-ej-enableCache="true"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Monsters University" data-ej-primarykey="3" data-ej-href="load3.html" data-ej-enableajax="true" data-ej-enableCache="true"&gt;&lt;/li&gt;
            *	&lt;/ul&gt;
            * &lt;/div&gt;
            * @example 
            * //Set the enableCache property in obtrusive way.
            * &lt;div id="lb" data-role="ejmlistview" data-ej-enableCache="true"&gt;
            * 	&lt;ul&gt;
            *		&lt;li data-ej-text="Man of Steel" data-ej-primarykey="1" data-ej-href="load1.html" data-ej-enableajax="true" data-ej-enableCache="true"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="World War Z" data-ej-primarykey="2" data-ej-href="load2.html" data-ej-enableajax="true" data-ej-enableCache="true"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Monsters University" data-ej-primarykey="3" data-ej-href="load3.html" data-ej-enableajax="true" data-ej-enableCache="true"&gt;&lt;/li&gt;
            *	&lt;/ul&gt;
            * &lt;/div&gt;
            * &lt;script&gt;
            * // Set enableCache on initialization. 
            * //To set enableCache API value 
            * $("#lb").ejmListView ({ enableCache: true });
            * &lt;/script&gt;
            * @example 
            * &lt;script&gt;
            * //Get or set  enableCache, after initialization:<br>
            * // Get the enableCache API value.		
            * $("#lb").ejmListView ("option", "enableCache");			
            * // Set the enableCache API
            * $("#lb").ejmListView ("option", "enableCache", true);
            * &lt;/script&gt;
            * @memberof ejmListView
            * @instance
            */
            enableCache: false,
            /**		
            * Specifies to maintain the current model value to browser cookies for state maintenance. While refresh the page, the model value will get apply to the control from browser cookies.
            * @default false
            * @member enablePersistence {object}
            * @type {boolean}
            * @example 
            * //Set the enablePersistence property in unobtrusive way.
            * &lt;div id="lb" data-role="ejlistview" data-ej-enablepersistence=true &gt;
            *	 &lt;ul&gt;
            *		&lt;li data-ej-text="Artwork"&gt;&lt;/li&gt;
            * 		&lt;li data-ej-text="Abstract"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="2 Acrylic Mediums"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Creative Acrylic"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Modern Painting"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Canvas Art"&gt;&lt;/li&gt;
            * 		&lt;li data-ej-text="Black white"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Children"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Preschool Crafts"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="School-age Crafts"&gt;&lt;/li&gt;
            * 	&lt;/ul&gt;
            * &lt;/div&gt;
            * @example 
            * //Set the enablePersistence property in obtrusive way.
            * &lt;div id="lb"&gt;
            *	 &lt;ul&gt;
            *		&lt;li data-ej-text="Artwork"&gt;&lt;/li&gt;
            * 		&lt;li data-ej-text="Abstract"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="2 Acrylic Mediums"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Creative Acrylic"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Modern Painting"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Canvas Art"&gt;&lt;/li&gt;
            * 		&lt;li data-ej-text="Black white"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Children"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Preschool Crafts"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="School-age Crafts"&gt;&lt;/li&gt;
            * 	&lt;/ul&gt;
            * &lt;/div&gt;
            * &lt;script&gt;
            * // Set enablePersistence on initialization. 
            * //To set enablePersistence API value 
            * $("#lb").ejListView ({ enablePersistence: true });
            * &lt;/script&gt;
            * @example 
            * &lt;script&gt;
            * //Get or set  enablePersistence, after initialization:<br>
            * // Get the enablePersistence API value.		
            *  $("#lb").ejListView ("option", "enablePersistence");			
            * // Set the enablePersistence API
            * $("#lb").ejListView ("option", "enablePersistence", true);
            * &lt;/script&gt;
            * @memberof ejListView
            * @instance
            */
            //Mobile
            /**		
            * Specifies to maintain the current model value to browser cookies for state maintenance. While refresh the page, the model value will get apply to the control from browser cookies.
            * @default false
            * @member enablePersistence {object}
            * @type {boolean}
            * @example 
            * //Set the enablePersistence property in unobtrusive way.
            * &lt;div id="lb" data-role="ejmlistview" data-ej-enablepersistence=true &gt;
            *	 &lt;ul&gt;
            *		&lt;li data-ej-text="Artwork"&gt;&lt;/li&gt;
            * 		&lt;li data-ej-text="Abstract"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="2 Acrylic Mediums"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Creative Acrylic"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Modern Painting"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Canvas Art"&gt;&lt;/li&gt;
            * 		&lt;li data-ej-text="Black white"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Children"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Preschool Crafts"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="School-age Crafts"&gt;&lt;/li&gt;
            * 	&lt;/ul&gt;
            * &lt;/div&gt;
            * @example 
            * //Set the enablePersistence property in obtrusive way.
            * &lt;div id="lb"&gt;
            *	 &lt;ul&gt;
            *		&lt;li data-ej-text="Artwork"&gt;&lt;/li&gt;
            * 		&lt;li data-ej-text="Abstract"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="2 Acrylic Mediums"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Creative Acrylic"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Modern Painting"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Canvas Art"&gt;&lt;/li&gt;
            * 		&lt;li data-ej-text="Black white"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Children"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Preschool Crafts"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="School-age Crafts"&gt;&lt;/li&gt;
            * 	&lt;/ul&gt;
            * &lt;/div&gt;
            * &lt;script&gt;
            * // Set enablePersistence on initialization. 
            * //To set enablePersistence API value 
            * $("#lb").ejmListView ({ enablePersistence: true });
            * &lt;/script&gt;
            * @example 
            * &lt;script&gt;
            * //Get or set  enablePersistence, after initialization:<br>
            * // Get the enablePersistence API value.		
            *  $("#lb").ejmListView ("option", "enablePersistence");			
            * // Set the enablePersistence API
            * $("#lb").ejmListView ("option", "enablePersistence", true);
            * &lt;/script&gt;
            * @memberof ejmListView
            * @instance
            */
            enablePersistence: false,
            //Events
            /**    
            * Event triggers before the items loaded.
            * @event
            * @name ejListView#load	
            * @param {Object} argument Event parameters from listview.
            * @param {boolean} argument.cancel returns true if the event should be cancelled; otherwise, false.
            * @param {string} argument.type returns the name of the event.
            * @param {Object} argument.model returns the model value of the control.
            * @example 
            * //Set the load property in unobtrusive way.
            * &lt;div id="lb" data-role="ejlistview" data-ej-load="load"&gt;
            * 	&lt;ul&gt;
            *		&lt;li data-ej-text="Artwork"&gt;&lt;/li&gt;
            * 		&lt;li data-ej-text="Abstract"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="2 Acrylic Mediums"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Creative Acrylic"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Modern Painting"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Canvas Art"&gt;&lt;/li&gt;
            * 		&lt;li data-ej-text="Black white"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Children"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Preschool Crafts"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="School-age Crafts"&gt;&lt;/li&gt;
            *	&lt;/ul&gt;
            * &lt;/div&gt;
            * @example 
            * //load event for ListView
            * &lt;div id="lb" data-role="ejlistview"&gt;
            * 	&lt;ul&gt;
            *		&lt;li data-ej-text="Artwork"&gt;&lt;/li&gt;
            * 		&lt;li data-ej-text="Abstract"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="2 Acrylic Mediums"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Creative Acrylic"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Modern Painting"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Canvas Art"&gt;&lt;/li&gt;
            * 		&lt;li data-ej-text="Black white"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Children"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Preschool Crafts"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="School-age Crafts"&gt;&lt;/li&gt;
            *	&lt;/ul&gt;
            * &lt;/div&gt;
            * &lt;script&gt;
            * $(document).ready(function(){
            * $("#lb").ejListView({
            *   	load: function (args) { //handle the event 
            * }
            *	});         
            * });
            * &lt;/script&gt;
            * @memberof ejListView
            * @instance
            */
            //Mobile
            /**    
            * Event triggers before the items loaded.
            * @event
            * @name ejmListView#load	
            * @param {Object} argument Event parameters from listview.
            * @param {boolean} argument.cancel returns true if the event should be cancelled; otherwise, false.
            * @param {string} argument.type returns the name of the event.
            * @param {Object} argument.model returns the model value of the control.
            * @example 
            * //Set the load property in unobtrusive way.
            * &lt;div id="lb" data-role="ejmlistview" data-ej-load="load"&gt;
            * 	&lt;ul&gt;
            *		&lt;li data-ej-text="Artwork"&gt;&lt;/li&gt;
            * 		&lt;li data-ej-text="Abstract"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="2 Acrylic Mediums"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Creative Acrylic"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Modern Painting"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Canvas Art"&gt;&lt;/li&gt;
            * 		&lt;li data-ej-text="Black white"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Children"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Preschool Crafts"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="School-age Crafts"&gt;&lt;/li&gt;
            *	&lt;/ul&gt;
            * &lt;/div&gt;
            * @example 
            * //load event for ListView
            * &lt;div id="lb" data-role="ejmlistview"&gt;
            * 	&lt;ul&gt;
            *		&lt;li data-ej-text="Artwork"&gt;&lt;/li&gt;
            * 		&lt;li data-ej-text="Abstract"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="2 Acrylic Mediums"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Creative Acrylic"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Modern Painting"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Canvas Art"&gt;&lt;/li&gt;
            * 		&lt;li data-ej-text="Black white"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Children"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Preschool Crafts"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="School-age Crafts"&gt;&lt;/li&gt;
            *	&lt;/ul&gt;
            * &lt;/div&gt;
            * &lt;script&gt;
            * $(document).ready(function(){
            * $("#lb").ejmListView({
            *   	load: function (args) { //handle the event 
            * }
            *	});         
            * });
            * &lt;/script&gt;
            * @memberof ejmListView
            * @instance
            */
            load: null,
            /**    
            * Event triggers after the items loaded.
            * @event
            * @name ejListView#loadComplete	
            * @param {Object} argument Event parameters from listview.
            * @param {boolean} argument.cancel returns true if the event should be cancelled; otherwise, false.
            * @param {string} argument.type returns the name of the event.
            * @param {Object} argument.model returns the model value of the control.
            * @example 
            * //Set the loadComplete property in unobtrusive way.
            * &lt;div id="lb" data-role="ejlistview" data-ej-loadComplete="loadComplete"&gt;
            * 	&lt;ul&gt;
            *		&lt;li data-ej-text="Artwork"&gt;&lt;/li&gt;
            * 		&lt;li data-ej-text="Abstract"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="2 Acrylic Mediums"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Creative Acrylic"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Modern Painting"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Canvas Art"&gt;&lt;/li&gt;
            * 		&lt;li data-ej-text="Black white"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Children"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Preschool Crafts"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="School-age Crafts"&gt;&lt;/li&gt;
            *	&lt;/ul&gt;
            * &lt;/div&gt;
            * @example 
            * //loadComplete event for ListView
            * &lt;div id="lb" data-role="ejlistview"&gt;
            * 	&lt;ul&gt;
            *		&lt;li data-ej-text="Artwork"&gt;&lt;/li&gt;
            * 		&lt;li data-ej-text="Abstract"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="2 Acrylic Mediums"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Creative Acrylic"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Modern Painting"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Canvas Art"&gt;&lt;/li&gt;
            * 		&lt;li data-ej-text="Black white"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Children"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Preschool Crafts"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="School-age Crafts"&gt;&lt;/li&gt;
            *	&lt;/ul&gt;
            * &lt;/div&gt;
            * &lt;script&gt;
            * $(document).ready(function(){
            * $("#lb").ejListView({
            *   	loadComplete: function (args) { //handle the event 
            * }
            *	});         
            * });
            * &lt;/script&gt;
            * @memberof ejListView
            * @instance
            */
            //Mobile
            /**    
            * Event triggers after the items loaded.
            * @event
            * @name ejmListView#loadComplete	
            * @param {Object} argument Event parameters from listview.
            * @param {boolean} argument.cancel returns true if the event should be cancelled; otherwise, false.
            * @param {string} argument.type returns the name of the event.
            * @param {Object} argument.model returns the model value of the control.
            * @example 
            * //Set the loadComplete property in unobtrusive way.
            * &lt;div id="lb" data-role="ejmlistview" data-ej-loadComplete="loadComplete"&gt;
            * 	&lt;ul&gt;
            *		&lt;li data-ej-text="Artwork"&gt;&lt;/li&gt;
            * 		&lt;li data-ej-text="Abstract"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="2 Acrylic Mediums"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Creative Acrylic"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Modern Painting"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Canvas Art"&gt;&lt;/li&gt;
            * 		&lt;li data-ej-text="Black white"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Children"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Preschool Crafts"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="School-age Crafts"&gt;&lt;/li&gt;
            *	&lt;/ul&gt;
            * &lt;/div&gt;
            * @example 
            * //loadComplete event for ListView
            * &lt;div id="lb" data-role="ejmlistview"&gt;
            * 	&lt;ul&gt;
            *		&lt;li data-ej-text="Artwork"&gt;&lt;/li&gt;
            * 		&lt;li data-ej-text="Abstract"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="2 Acrylic Mediums"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Creative Acrylic"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Modern Painting"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Canvas Art"&gt;&lt;/li&gt;
            * 		&lt;li data-ej-text="Black white"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Children"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Preschool Crafts"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="School-age Crafts"&gt;&lt;/li&gt;
            *	&lt;/ul&gt;
            * &lt;/div&gt;
            * &lt;script&gt;
            * $(document).ready(function(){
            * $("#lb").ejmListView({
            *   	loadComplete: function (args) { //handle the event 
            * }
            *	});         
            * });
            * &lt;/script&gt;
            * @memberof ejmListView
            * @instance
            */
            loadComplete: null,
            /**    
            * Event triggers before the ajax request happens.
            * @event
            * @name ejListView#ajaxBeforeLoad	
            * @param {Object} argument Event parameters from listview.
            * @param {boolean} argument.cancel returns true if the event should be cancelled; otherwise, false.
            * @param {string} argument.type returns the name of the event.
            * @param {Object} argument.model returns the model value of the control.
            * @param {Object} argument.ajaxData returns the ajax settings.
            * @example 
            * //Set the ajaxBeforeLoad property in unobtrusive way.
            * &lt;div id="lb" data-role="ejlistview" data-ej-ajaxBeforeLoad="ajaxBeforeLoad"&gt;
            * 	&lt;ul&gt;
            *		&lt;li data-ej-text="Man of Steel" data-ej-primarykey="1" data-ej-href="load1.html" data-ej-enableajax="true"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="World War Z" data-ej-primarykey="2" data-ej-href="load2.html" data-ej-enableajax="true"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Monsters University" data-ej-primarykey="3" data-ej-href="load3.html" data-ej-enableajax="true"&gt;&lt;/li&gt;
            *	&lt;/ul&gt;
            * &lt;/div&gt;
            * @example 
            * //ajaxBeforeLoad event for ListView
            * &lt;div id="lb" data-role="ejlistview"&gt;
            * 	&lt;ul&gt;
            *		&lt;li data-ej-text="Man of Steel" data-ej-primarykey="1" data-ej-href="load1.html" data-ej-enableajax="true"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="World War Z" data-ej-primarykey="2" data-ej-href="load2.html" data-ej-enableajax="true"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Monsters University" data-ej-primarykey="3" data-ej-href="load3.html" data-ej-enableajax="true"&gt;&lt;/li&gt;
            *	&lt;/ul&gt;
            * &lt;/div&gt;
            * &lt;script&gt;
            * $(document).ready(function(){
            * $("#lb").ejListView({
            *   	ajaxBeforeLoad: function (args) { //handle the event 
            * }
            *	});           
            * });
            * &lt;/script&gt;
            * @memberof ejListView
            * @instance
            */
            //Mobile
            /**    
            * Event triggers before the ajax request happens.
            * @event
            * @name ejmListView#ajaxBeforeLoad	
            * @param {Object} argument Event parameters from listview.
            * @param {boolean} argument.cancel returns true if the event should be cancelled; otherwise, false.
            * @param {string} argument.type returns the name of the event.
            * @param {Object} argument.model returns the model value of the control.
            * @param {Object} argument.ajaxData returns the ajax settings.
            * @example 
            * //Set the ajaxBeforeLoad property in unobtrusive way.
            * &lt;div id="lb" data-role="ejmlistview" data-ej-ajaxBeforeLoad="ajaxBeforeLoad"&gt;
            * 	&lt;ul&gt;
            *		&lt;li data-ej-text="Man of Steel" data-ej-primarykey="1" data-ej-href="load1.html" data-ej-enableajax="true"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="World War Z" data-ej-primarykey="2" data-ej-href="load2.html" data-ej-enableajax="true"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Monsters University" data-ej-primarykey="3" data-ej-href="load3.html" data-ej-enableajax="true"&gt;&lt;/li&gt;
            *	&lt;/ul&gt;
            * &lt;/div&gt;
            * @example 
            * //ajaxBeforeLoad event for ListView
            * &lt;div id="lb" data-role="ejmlistview"&gt;
            * 	&lt;ul&gt;
            *		&lt;li data-ej-text="Man of Steel" data-ej-primarykey="1" data-ej-href="load1.html" data-ej-enableajax="true"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="World War Z" data-ej-primarykey="2" data-ej-href="load2.html" data-ej-enableajax="true"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Monsters University" data-ej-primarykey="3" data-ej-href="load3.html" data-ej-enableajax="true"&gt;&lt;/li&gt;
            *	&lt;/ul&gt;
            * &lt;/div&gt;
            * &lt;script&gt;
            * $(document).ready(function(){
            * $("#lb").ejmListView({
            *   	ajaxBeforeLoad: function (args) { //handle the event 
            * }
            *	});           
            * });
            * &lt;/script&gt;
            * @memberof ejmListView
            * @instance
            */
            ajaxBeforeLoad: null,
            /**    
            * Event triggers after the ajax content loaded successfully.
            * @event
            * @name ejListView#ajaxSuccess
            * @param {Object} argument Event parameters from listview.
            * @param {boolean} argument.cancel returns true if the event should be cancelled; otherwise, false.
            * @param {string} argument.type returns the name of the event.
            * @param {Object} argument.model returns the model value of the control.
            * @param {string}  argument.content returns the ajax current content.
            * @param {object}  argument.item returns the curent list item.
            * @param {string}  argument.text returns the current item text.
            * @param {number}  argument.index returns the current item index.
            * @param {string}  argument.url returns the current url of the ajax post.
            * @example 
            * //Set the ajaxSuccess property in unobtrusive way.
            * &lt;div id="lb" data-role="ejlistview" data-ej-ajaxSuccess="ajaxSuccess"&gt;
            * 	&lt;ul&gt;
            *		&lt;li data-ej-text="Man of Steel" data-ej-primarykey="1" data-ej-href="load1.html" data-ej-enableajax="true"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="World War Z" data-ej-primarykey="2" data-ej-href="load2.html" data-ej-enableajax="true"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Monsters University" data-ej-primarykey="3" data-ej-href="load3.html" data-ej-enableajax="true"&gt;&lt;/li&gt;
            *	&lt;/ul&gt;
            * &lt;/div&gt;
            * @example 
            * //ajaxSuccess event for ListView
            * &lt;div id="lb" data-role="ejlistview"&gt;
            * 	&lt;ul&gt;
            *		&lt;li data-ej-text="Man of Steel" data-ej-primarykey="1" data-ej-href="load1.html" data-ej-enableajax="true"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="World War Z" data-ej-primarykey="2" data-ej-href="load2.html" data-ej-enableajax="true"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Monsters University" data-ej-primarykey="3" data-ej-href="load3.html" data-ej-enableajax="true"&gt;&lt;/li&gt;
            *	&lt;/ul&gt;
            * &lt;/div&gt;
            * &lt;script&gt;
            * $(document).ready(function(){
            * $("#lb").ejListView({
            *   	ajaxSuccess: function (args) { //handle the event 
            * }
            *	});         
            * });
            * &lt;/script&gt;
            * @memberof ejListView
            * @instance
            */
            //Mobile
            /**    
           * Event triggers after the ajax content loaded successfully.
           * @event
           * @name ejmListView#ajaxSuccess
           * @param {Object} argument Event parameters from listview.
           * @param {boolean} argument.cancel returns true if the event should be cancelled; otherwise, false.
           * @param {string} argument.type returns the name of the event.
           * @param {Object} argument.model returns the model value of the control.
           * @param {string}  argument.content returns the ajax current content.
           * @param {object}  argument.item returns the curent list item.
           * @param {string}  argument.text returns the current item text.
           * @param {number}  argument.index returns the current item index.
           * @param {string}  argument.url returns the current url of the ajax post.
           * @example 
           * //Set the ajaxSuccess property in unobtrusive way.
           * &lt;div id="lb" data-role="ejmlistview" data-ej-ajaxSuccess="ajaxSuccess"&gt;
           * 	&lt;ul&gt;
           *		&lt;li data-ej-text="Man of Steel" data-ej-primarykey="1" data-ej-href="load1.html" data-ej-enableajax="true"&gt;&lt;/li&gt;
           *		&lt;li data-ej-text="World War Z" data-ej-primarykey="2" data-ej-href="load2.html" data-ej-enableajax="true"&gt;&lt;/li&gt;
           *		&lt;li data-ej-text="Monsters University" data-ej-primarykey="3" data-ej-href="load3.html" data-ej-enableajax="true"&gt;&lt;/li&gt;
           *	&lt;/ul&gt;
           * &lt;/div&gt;
           * @example 
           * //ajaxSuccess event for ListView
           * &lt;div id="lb" data-role="ejmlistview"&gt;
           * 	&lt;ul&gt;
           *		&lt;li data-ej-text="Man of Steel" data-ej-primarykey="1" data-ej-href="load1.html" data-ej-enableajax="true"&gt;&lt;/li&gt;
           *		&lt;li data-ej-text="World War Z" data-ej-primarykey="2" data-ej-href="load2.html" data-ej-enableajax="true"&gt;&lt;/li&gt;
           *		&lt;li data-ej-text="Monsters University" data-ej-primarykey="3" data-ej-href="load3.html" data-ej-enableajax="true"&gt;&lt;/li&gt;
           *	&lt;/ul&gt;
           * &lt;/div&gt;
           * &lt;script&gt;
           * $(document).ready(function(){
           * $("#lb").ejmListView({
           *   	ajaxSuccess: function (args) { //handle the event 
           * }
           *	});         
           * });
           * &lt;/script&gt;
           * @memberof ejmListView
           * @instance
           */
            ajaxSuccess: null,
            /**    
            * Event triggers when the ajax request failed.
            * @event
            * @name ejListView#ajaxError	
            * @param {Object} argument Event parameters from listview.
            * @param {boolean} argument.cancel returns true if the event should be cancelled; otherwise, false.
            * @param {string} argument.type returns the name of the event.
            * @param {Object} argument.model returns the model value of the control.
            * @param {Object} argument.errorThrown returns the error thrown in the ajax post.
            * @param {Object} argument.textStatus returns the status.
            * @param {object} argument.item returns the curent list item.
            * @param {string} argument.text returns the current item text.
            * @param {number} argument.index returns the current item index.
            * @example 
            * //Set the ajaxError property in unobtrusive way.
            * &lt;div id="lb" data-role="ejlistview" data-ej-ajaxError="ajaxError"&gt;
            * 	&lt;ul&gt;
            *		&lt;li data-ej-text="Man of Steel" data-ej-primarykey="1" data-ej-href="load1.html" data-ej-enableajax="true"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="World War Z" data-ej-primarykey="2" data-ej-href="load2.html" data-ej-enableajax="true"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Monsters University" data-ej-primarykey="3" data-ej-href="load3.html" data-ej-enableajax="true"&gt;&lt;/li&gt;
            *	&lt;/ul&gt;
            * &lt;/div&gt;
            * @example 
            * //ajaxError event for ListView
            * &lt;div id="lb" data-role="ejlistview"&gt;
            * 	&lt;ul&gt;
            *		&lt;li data-ej-text="Man of Steel" data-ej-primarykey="1" data-ej-href="load1.html" data-ej-enableajax="true"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="World War Z" data-ej-primarykey="2" data-ej-href="load2.html" data-ej-enableajax="true"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Monsters University" data-ej-primarykey="3" data-ej-href="load3.html" data-ej-enableajax="true"&gt;&lt;/li&gt;
            *	&lt;/ul&gt;
            * &lt;/div&gt;
            * &lt;script&gt;
            * $(document).ready(function(){
            * $("#lb").ejListView({
            *   	ajaxError: function (args) { //handle the event 
            * }
            *	});         
            * });
            * &lt;/script&gt;
            * @memberof ejListView
            * @instance
            */
            //Mobile
            /**    
            * Event triggers when the ajax request failed.
            * @event
            * @name ejmListView#ajaxError	
            * @param {Object} argument Event parameters from listview.
            * @param {boolean} argument.cancel returns true if the event should be cancelled; otherwise, false.
            * @param {string} argument.type returns the name of the event.
            * @param {Object} argument.model returns the model value of the control.
            * @param {Object} argument.errorThrown returns the error thrown in the ajax post.
            * @param {Object} argument.textStatus returns the status.
            * @param {object} argument.item returns the curent list item.
            * @param {string} argument.text returns the current item text.
            * @param {number} argument.index returns the current item index.
            * @example 
            * //Set the ajaxError property in unobtrusive way.
            * &lt;div id="lb" data-role="ejmlistview" data-ej-ajaxError="ajaxError"&gt;
            * 	&lt;ul&gt;
            *		&lt;li data-ej-text="Man of Steel" data-ej-primarykey="1" data-ej-href="load1.html" data-ej-enableajax="true"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="World War Z" data-ej-primarykey="2" data-ej-href="load2.html" data-ej-enableajax="true"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Monsters University" data-ej-primarykey="3" data-ej-href="load3.html" data-ej-enableajax="true"&gt;&lt;/li&gt;
            *	&lt;/ul&gt;
            * &lt;/div&gt;
            * @example 
            * //ajaxError event for ListView
            * &lt;div id="lb" data-role="ejmlistview"&gt;
            * 	&lt;ul&gt;
            *		&lt;li data-ej-text="Man of Steel" data-ej-primarykey="1" data-ej-href="load1.html" data-ej-enableajax="true"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="World War Z" data-ej-primarykey="2" data-ej-href="load2.html" data-ej-enableajax="true"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Monsters University" data-ej-primarykey="3" data-ej-href="load3.html" data-ej-enableajax="true"&gt;&lt;/li&gt;
            *	&lt;/ul&gt;
            * &lt;/div&gt;
            * &lt;script&gt;
            * $(document).ready(function(){
            * $("#lb").ejmListView({
            *   	ajaxError: function (args) { //handle the event 
            * }
            *	});         
            * });
            * &lt;/script&gt;
            * @memberof ejmListView
            * @instance
            */
            ajaxError: null,
            /**    
            * Event triggers after the ajax content loaded completely.
            * @event
            * @name ejListView#ajaxComplete	
            * @param {Object} argument Event parameters from listview.
            * @param {boolean} argument.cancel returns true if the event should be cancelled; otherwise, false.
            * @param {string} argument.type returns the name of the event.
            * @param {Object} argument.model returns the model value of the control.
            * @example 
            * //Set the ajaxComplete property in unobtrusive way.
            * &lt;div id="lb" data-role="ejlistview" data-ej-ajaxComplete="ajaxComplete"&gt;
            * 	&lt;ul&gt;
            *		&lt;li data-ej-text="Man of Steel" data-ej-primarykey="1" data-ej-href="load1.html" data-ej-enableajax="true"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="World War Z" data-ej-primarykey="2" data-ej-href="load2.html" data-ej-enableajax="true"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Monsters University" data-ej-primarykey="3" data-ej-href="load3.html" data-ej-enableajax="true"&gt;&lt;/li&gt;
            *	&lt;/ul&gt;
            * &lt;/div&gt;
            * @example 
            * //ajaxComplete event for ListView
            * &lt;div id="lb" data-role="ejlistview" &gt;
            * 	&lt;ul&gt;
            *		&lt;li data-ej-text="Man of Steel" data-ej-primarykey="1" data-ej-href="load1.html" data-ej-enableajax="true"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="World War Z" data-ej-primarykey="2" data-ej-href="load2.html" data-ej-enableajax="true"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Monsters University" data-ej-primarykey="3" data-ej-href="load3.html" data-ej-enableajax="true"&gt;&lt;/li&gt;
            *	&lt;/ul&gt;
            * &lt;/div&gt;
            * &lt;script&gt;
            * $(document).ready(function(){
            * $("#lb").ejListView({
            *   	ajaxComplete: function (args) { //handle the event 
            * }
            *	});         
            * });
            * &lt;/script&gt;
            * @memberof ejListView
            * @instance
            */
            //Mobile
            /**    
            * Event triggers after the ajax content loaded completely.
            * @event
            * @name ejmListView#ajaxComplete	
            * @param {Object} argument Event parameters from listview.
            * @param {boolean} argument.cancel returns true if the event should be cancelled; otherwise, false.
            * @param {string} argument.type returns the name of the event.
            * @param {Object} argument.model returns the model value of the control.
            * @example 
            * //Set the ajaxComplete property in unobtrusive way.
            * &lt;div id="lb" data-role="ejmlistview" data-ej-ajaxComplete="ajaxComplete"&gt;
            * 	&lt;ul&gt;
            *		&lt;li data-ej-text="Man of Steel" data-ej-primarykey="1" data-ej-href="load1.html" data-ej-enableajax="true"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="World War Z" data-ej-primarykey="2" data-ej-href="load2.html" data-ej-enableajax="true"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Monsters University" data-ej-primarykey="3" data-ej-href="load3.html" data-ej-enableajax="true"&gt;&lt;/li&gt;
            *	&lt;/ul&gt;
            * &lt;/div&gt;
            * @example 
            * //ajaxComplete event for ListView
            * &lt;div id="lb" data-role="ejmlistview" &gt;
            * 	&lt;ul&gt;
            *		&lt;li data-ej-text="Man of Steel" data-ej-primarykey="1" data-ej-href="load1.html" data-ej-enableajax="true"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="World War Z" data-ej-primarykey="2" data-ej-href="load2.html" data-ej-enableajax="true"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Monsters University" data-ej-primarykey="3" data-ej-href="load3.html" data-ej-enableajax="true"&gt;&lt;/li&gt;
            *	&lt;/ul&gt;
            * &lt;/div&gt;
            * &lt;script&gt;
            * $(document).ready(function(){
            * $("#lb").ejmListView({
            *   	ajaxComplete: function (args) { //handle the event 
            * }
            *	});         
            * });
            * &lt;/script&gt;
            * @memberof ejmListView
            * @instance
            */
            ajaxComplete: null,
            /**		
            * Specifies the settings option for ajax request.
            * @type {JSONObject}  
            * @example 
            * //Set ajaxSettings in unobtrusive way
            * &lt;div id="lb" data-role="ejlistview" data-ej-ajaxsettings-type="GET" data-ej-ajaxsettings-cache=false data-ej-ajaxsettings-async=true
            * data-ej-ajaxsettings-datatype="html" data-ej-ajaxsettings-contenttype="html" &gt;
            * 	&lt;ul&gt;
            *		&lt;li data-ej-text="Man of Steel" data-ej-primarykey="1" data-ej-href="load1.html" data-ej-enableajax="true"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="World War Z" data-ej-primarykey="2" data-ej-href="load2.html" data-ej-enableajax="true"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Monsters University" data-ej-primarykey="3" data-ej-href="load3.html" data-ej-enableajax="true"&gt;&lt;/li&gt;
            *	&lt;/ul&gt;
            * &lt;/div&gt;          
            * @example 
            * //Set ajaxSettings in obtrusive way
            * &lt;div id="lb" data-role="ejlistview" &gt;
            * 	&lt;ul&gt;
            *		&lt;li data-ej-text="Man of Steel" data-ej-primarykey="1" data-ej-href="load1.html" data-ej-enableajax="true"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="World War Z" data-ej-primarykey="2" data-ej-href="load2.html" data-ej-enableajax="true"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Monsters University" data-ej-primarykey="3" data-ej-href="load3.html" data-ej-enableajax="true"&gt;&lt;/li&gt;
            *	&lt;/ul&gt;
            * &lt;/div&gt;
            * // Set Tab ajaxSettings on initialization. 
            * //To set ajaxSettings API value 
            * &lt;script&gt;
            * $("#lb").ejListView({ ajaxSettings: { type: 'GET',
            *   cache: false,
            *   async: true,
            *   dataType: "html",
            *   contentType: "html"
            *   } });		
            * &lt;/script&gt;
            * @example 
            * &lt;script&gt;
            * //Get or set the ListView ajaxSettings, after initialization:<br>
            * // Get the ajaxSettings API value.
            * $("#lb").ejListView ("option", "ajaxSettings");			
            * // Set the ajaxSettings API
            * $("#lb").ejListView ("option", "ajaxSettings", { type: 'GET',cache: false,async: true,dataType: "html",contentType: "html",url: "",data: {}});        
            * &lt;/script&gt;    
            * @memberof ejListView
            * @instance				
            */
            //Mobile
            /**		
            * Specifies the settings option for ajax request.
            * @type {JSONObject}  
            * @example 
            * //Set ajaxSettings in unobtrusive way
            * &lt;div id="lb" data-role="ejmlistview" data-ej-ajaxsettings-type="GET" data-ej-ajaxsettings-cache=false data-ej-ajaxsettings-async=true
            * data-ej-ajaxsettings-datatype="html" data-ej-ajaxsettings-contenttype="html" &gt;
            * 	&lt;ul&gt;
            *		&lt;li data-ej-text="Man of Steel" data-ej-primarykey="1" data-ej-href="load1.html" data-ej-enableajax="true"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="World War Z" data-ej-primarykey="2" data-ej-href="load2.html" data-ej-enableajax="true"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Monsters University" data-ej-primarykey="3" data-ej-href="load3.html" data-ej-enableajax="true"&gt;&lt;/li&gt;
            *	&lt;/ul&gt;
            * &lt;/div&gt;          
            * @example 
            * //Set ajaxSettings in obtrusive way
            * &lt;div id="lb" data-role="ejmlistview" &gt;
            * 	&lt;ul&gt;
            *		&lt;li data-ej-text="Man of Steel" data-ej-primarykey="1" data-ej-href="load1.html" data-ej-enableajax="true"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="World War Z" data-ej-primarykey="2" data-ej-href="load2.html" data-ej-enableajax="true"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Monsters University" data-ej-primarykey="3" data-ej-href="load3.html" data-ej-enableajax="true"&gt;&lt;/li&gt;
            *	&lt;/ul&gt;
            * &lt;/div&gt;
            * // Set Tab ajaxSettings on initialization. 
            * //To set ajaxSettings API value 
            * &lt;script&gt;
            * $("#lb").ejmListView({ ajaxSettings: { type: 'GET',
            *   cache: false,
            *   async: true,
            *   dataType: "html",
            *   contentType: "html"
            *   } });		
            * &lt;/script&gt;
            * @example 
            * &lt;script&gt;
            * //Get or set the ListView ajaxSettings, after initialization:<br>
            * // Get the ajaxSettings API value.
            * $("#lb").ejmListView ("option", "ajaxSettings");			
            * // Set the ajaxSettings API
            * $("#lb").ejmListView ("option", "ajaxSettings", { type: 'GET',cache: false,async: true,dataType: "html",contentType: "html",url: "",data: {}});        
            * &lt;/script&gt;    
            * @memberof ejmListView
            * @instance				
            */
            ajaxSettings: {
                type: 'GET',
                cache: false,
                async: true,
                dataType: "html",
                contentType: "html",
                url: "",
                data: {}
            },
            /**		
            * Specifies whether need to render the control with the template contents.
            * @default false
            * @member renderTemplate {object}
            * @type {boolean}
            * @example 
            * //Set the renderTemplate property in unobtrusive way.
            * &lt;div id="lb" data-role="ejlistview"&gt;
            * 	&lt;ul&gt;
            *		&lt;li data-ej-rendertemplate=true data-ej-templateid="target1"&gt;&lt;/li&gt;
            * 		&lt;li data-ej-rendertemplate=true data-ej-templateid="target2"&gt;&lt;/li&gt;
            *		&lt;li data-ej-rendertemplate=true data-ej-templateid="target3"&gt;&lt;/li&gt;
            *	&lt;/ul&gt;
            * &lt;/div&gt;
            * &lt;div id="target1"&gt;
            *	&lt;div&gt Template1 &lt;/div&gt;
            * &lt;/div&gt;
            * &lt;div id="target2"&gt;
            *	&lt;div&gt Template2 &lt;/div&gt;
            * &lt;/div&gt;
            * &lt;div id="target3"&gt;
            *	&lt;div&gt Template3 &lt;/div&gt;
            * &lt;/div&gt;            
            * @memberof ejListView
            * @instance
            */
            //Mobile
            /**		
            * Specifies whether need to render the control with the template contents.
            * @default false
            * @member renderTemplate {object}
            * @type {boolean}
            * @example 
            * //Set the renderTemplate property in unobtrusive way.
            * &lt;div id="lb" data-role="ejmlistview"&gt;
            * 	&lt;ul&gt;
            *		&lt;li data-ej-rendertemplate=true data-ej-templateid="target1"&gt;&lt;/li&gt;
            * 		&lt;li data-ej-rendertemplate=true data-ej-templateid="target2"&gt;&lt;/li&gt;
            *		&lt;li data-ej-rendertemplate=true data-ej-templateid="target3"&gt;&lt;/li&gt;
            *	&lt;/ul&gt;
            * &lt;/div&gt;
            * &lt;div id="target1"&gt;
            *	&lt;div&gt Template1 &lt;/div&gt;
            * &lt;/div&gt;
            * &lt;div id="target2"&gt;
            *	&lt;div&gt Template2 &lt;/div&gt;
            * &lt;/div&gt;
            * &lt;div id="target3"&gt;
            *	&lt;div&gt Template3 &lt;/div&gt;
            * &lt;/div&gt;            
            * @memberof ejmListView
            * @instance
            */
            renderTemplate: false,
            /**		
            * Specifies ID of the element contains template contents.
            * @default false
            * @member templateId {object}
            * @type {boolean}
            * @example 
            * //Set the templateId property in unobtrusive way.
            * &lt;div id="lb" data-role="ejlistview"&gt;
            * 	&lt;ul&gt;
            *		&lt;li data-ej-rendertemplate=true data-ej-templateid="target1"&gt;&lt;/li&gt;
            * 		&lt;li data-ej-rendertemplate=true data-ej-templateid="target2"&gt;&lt;/li&gt;
            *		&lt;li data-ej-rendertemplate=true data-ej-templateid="target3"&gt;&lt;/li&gt;
            *	&lt;/ul&gt;
            * &lt;/div&gt;
            * &lt;div id="target1"&gt;
            *	&lt;div&gt Template1 &lt;/div&gt;
            * &lt;/div&gt;
            * &lt;div id="target2"&gt;
            *	&lt;div&gt Template2 &lt;/div&gt;
            * &lt;/div&gt;
            * &lt;div id="target3"&gt;
            *	&lt;div&gt Template3 &lt;/div&gt;
            * &lt;/div&gt;
            * @memberof ejListView
            * @instance
            */
            //Mobile
            /**		
            * Specifies ID of the element contains template contents.
            * @default false
            * @member templateId {object}
            * @type {boolean}
            * @example 
            * //Set the templateId property in unobtrusive way.
            * &lt;div id="lb" data-role="ejmlistview"&gt;
            * 	&lt;ul&gt;
            *		&lt;li data-ej-rendertemplate=true data-ej-templateid="target1"&gt;&lt;/li&gt;
            * 		&lt;li data-ej-rendertemplate=true data-ej-templateid="target2"&gt;&lt;/li&gt;
            *		&lt;li data-ej-rendertemplate=true data-ej-templateid="target3"&gt;&lt;/li&gt;
            *	&lt;/ul&gt;
            * &lt;/div&gt;
            * &lt;div id="target1"&gt;
            *	&lt;div&gt Template1 &lt;/div&gt;
            * &lt;/div&gt;
            * &lt;div id="target2"&gt;
            *	&lt;div&gt Template2 &lt;/div&gt;
            * &lt;/div&gt;
            * &lt;div id="target3"&gt;
            *	&lt;div&gt Template3 &lt;/div&gt;
            * &lt;/div&gt;
            * @memberof ejmListView
            * @instance
            */
            templateId: null,
            /**		
            * Specifies whether to retain the selection of the item.
            * @default false
            * @member persistSelection {object}
            * @type {boolean}
            * @example 
            * //Set the persistSelection property in unobtrusive way.
            * &lt;div id="lb" data-role="ejlistview" data-ej-persistSelection="true"&gt;
            * 	&lt;ul&gt;
            *		&lt;li data-ej-text="Artwork"&gt;&lt;/li&gt;
            * 		&lt;li data-ej-text="Abstract"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="2 Acrylic Mediums"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Creative Acrylic"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Modern Painting"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Canvas Art"&gt;&lt;/li&gt;
            * 		&lt;li data-ej-text="Black white"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Children"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Preschool Crafts"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="School-age Crafts"&gt;&lt;/li&gt;
            *	&lt;/ul&gt;
            * &lt;/div&gt;
            * @example 
            * //Set the persistSelection property in obtrusive way.
            * &lt;div id="lb"&gt;
            * 	&lt;ul&gt;
            *		&lt;li data-ej-text="Artwork"&gt;&lt;/li&gt;
            * 		&lt;li data-ej-text="Abstract"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="2 Acrylic Mediums"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Creative Acrylic"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Modern Painting"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Canvas Art"&gt;&lt;/li&gt;
            * 		&lt;li data-ej-text="Black white"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Children"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Preschool Crafts"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="School-age Crafts"&gt;&lt;/li&gt;
            *	&lt;/ul&gt;
            * &lt;/div&gt;
            * &lt;script&gt;
            * // Set persistSelection on initialization. 
            * //To set persistSelection API value 
            * $("#lb").ejListView ({ persistSelection: true });
            * &lt;/script&gt;
            * @example 
            * &lt;script&gt;
            * //Get or set the ListView persistSelection, after initialization:<br>
            * // Get the persistSelection API value.		
            * $("#lb").ejListView ("option", "persistSelection");			
            * // Set the persistSelection API
            * $("#lb").ejListView ("option", "persistSelection", true);
            * &lt;/script&gt;
            * @memberof ejListView
            * @instance
            */
            //Mobile
            /**		
            * Specifies whether to retain the selection of the item.
            * @default false
            * @member persistSelection {object}
            * @type {boolean}
            * @example 
            * //Set the persistSelection property in unobtrusive way.
            * &lt;div id="lb" data-role="ejmlistview" data-ej-persistSelection="true"&gt;
            * 	&lt;ul&gt;
            *		&lt;li data-ej-text="Artwork"&gt;&lt;/li&gt;
            * 		&lt;li data-ej-text="Abstract"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="2 Acrylic Mediums"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Creative Acrylic"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Modern Painting"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Canvas Art"&gt;&lt;/li&gt;
            * 		&lt;li data-ej-text="Black white"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Children"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Preschool Crafts"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="School-age Crafts"&gt;&lt;/li&gt;
            *	&lt;/ul&gt;
            * &lt;/div&gt;
            * @example 
            * //Set the persistSelection property in obtrusive way.
            * &lt;div id="lb"&gt;
            * 	&lt;ul&gt;
            *		&lt;li data-ej-text="Artwork"&gt;&lt;/li&gt;
            * 		&lt;li data-ej-text="Abstract"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="2 Acrylic Mediums"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Creative Acrylic"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Modern Painting"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Canvas Art"&gt;&lt;/li&gt;
            * 		&lt;li data-ej-text="Black white"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Children"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Preschool Crafts"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="School-age Crafts"&gt;&lt;/li&gt;
            *	&lt;/ul&gt;
            * &lt;/div&gt;
            * &lt;script&gt;
            * // Set persistSelection on initialization. 
            * //To set persistSelection API value 
            * $("#lb").ejmListView ({ persistSelection: true });
            * &lt;/script&gt;
            * @example 
            * &lt;script&gt;
            * //Get or set the ListView persistSelection, after initialization:<br>
            * // Get the persistSelection API value.		
            * $("#lb").ejmListView ("option", "persistSelection");			
            * // Set the persistSelection API
            * $("#lb").ejmListView ("option", "persistSelection", true);
            * &lt;/script&gt;
            * @memberof ejmListView
            * @instance
            */
            persistSelection: false,
            /**		
            * Specifies whether to prevent the selection of the item.
            * @default false
            * @member preventSelection {object}
            * @type {boolean}
            * @example 
            * //Set the preventSelection property in unobtrusive way.
            * &lt;div id="lb" data-role="ejlistview" data-ej-preventSelection="true"&gt;
            * 	&lt;ul&gt;
            *		&lt;li data-ej-text="Artwork"&gt;&lt;/li&gt;
            * 		&lt;li data-ej-text="Abstract"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="2 Acrylic Mediums"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Creative Acrylic"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Modern Painting"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Canvas Art"&gt;&lt;/li&gt;
            * 		&lt;li data-ej-text="Black white"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Children"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Preschool Crafts"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="School-age Crafts"&gt;&lt;/li&gt;
            *	&lt;/ul&gt;
            * &lt;/div&gt;
            * @example 
            * //Set the preventSelection property in obtrusive way.
            * &lt;div id="lb"&gt;
            * 	&lt;ul&gt;
            *		&lt;li data-ej-text="Artwork"&gt;&lt;/li&gt;
            * 		&lt;li data-ej-text="Abstract"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="2 Acrylic Mediums"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Creative Acrylic"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Modern Painting"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Canvas Art"&gt;&lt;/li&gt;
            * 		&lt;li data-ej-text="Black white"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Children"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Preschool Crafts"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="School-age Crafts"&gt;&lt;/li&gt;
            *	&lt;/ul&gt;
            * &lt;/div&gt;
            * &lt;script&gt;
            * // Set preventSelection on initialization. 
            * //To set preventSelection API value 
            * $("#lb").ejListView ({ preventSelection: true });
            * &lt;/script&gt;
            * @example 
            * &lt;script&gt;
            * //Get or set the ListView preventSelection, after initialization:<br>
            * // Get the preventSelection API value.		
            * $("#lb").ejListView ("option", "preventSelection");			
            * // Set the preventSelection API
            * $("#lb").ejListView ("option", "preventSelection", true);
            * &lt;/script&gt;
            * @memberof ejListView
            * @instance
            */
            //Mobile
            /**		
            * Specifies whether to prevent the selection of the item.
            * @default false
            * @member preventSelection {object}
            * @type {boolean}
            * @example 
            * //Set the preventSelection property in unobtrusive way.
            * &lt;div id="lb" data-role="ejmlistview" data-ej-preventSelection="true"&gt;
            * 	&lt;ul&gt;
            *		&lt;li data-ej-text="Artwork"&gt;&lt;/li&gt;
            * 		&lt;li data-ej-text="Abstract"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="2 Acrylic Mediums"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Creative Acrylic"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Modern Painting"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Canvas Art"&gt;&lt;/li&gt;
            * 		&lt;li data-ej-text="Black white"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Children"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Preschool Crafts"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="School-age Crafts"&gt;&lt;/li&gt;
            *	&lt;/ul&gt;
            * &lt;/div&gt;
            * @example 
            * //Set the preventSelection property in obtrusive way.
            * &lt;div id="lb"&gt;
            * 	&lt;ul&gt;
            *		&lt;li data-ej-text="Artwork"&gt;&lt;/li&gt;
            * 		&lt;li data-ej-text="Abstract"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="2 Acrylic Mediums"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Creative Acrylic"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Modern Painting"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Canvas Art"&gt;&lt;/li&gt;
            * 		&lt;li data-ej-text="Black white"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Children"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Preschool Crafts"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="School-age Crafts"&gt;&lt;/li&gt;
            *	&lt;/ul&gt;
            * &lt;/div&gt;
            * &lt;script&gt;
            * // Set preventSelection on initialization. 
            * //To set preventSelection API value 
            * $("#lb").ejmListView ({ preventSelection: true });
            * &lt;/script&gt;
            * @example 
            * &lt;script&gt;
            * //Get or set the ListView preventSelection, after initialization:<br>
            * // Get the preventSelection API value.		
            * $("#lb").ejmListView ("option", "preventSelection");			
            * // Set the preventSelection API
            * $("#lb").ejmListView ("option", "preventSelection", true);
            * &lt;/script&gt;
            * @memberof ejmListView
            * @instance
            */
            preventSelection: false,
            /**		
            * Specifies whether to allow the databinding support for the control.
            * @default false
            * @member dataBinding {object}
            * @type {boolean}
            * @example 
            * //Set the dataBinding property in unobtrusive way.
            * &lt;div id="lb" data-role="ejlistview" data-ej-dataBinding="true" data-ej-datasource="window.dbitem" &gt;
            * &lt;/div&gt;           
            * &lt;script&gt;
            * window.dbitem =
            * [   { "text": "Hot Singles"},
            *     { "text": "Rising Artists"},
            *     { "text": "Live Music" },
            *     { "text": "Best of 2013 So Far"},
            *     { "text": "100 Albums - $5 Each"},
            *     { "text": "Hip-Hop and R&B Sale"},
            *     { "text": "CD Deals"}];
            * &lt;/script&gt;                
            * @example 
            * //Set the dataBinding property in obtrusive way.
            * &lt;div id="lb" &gt;
            * &lt;/div&gt;           
            * &lt;script&gt;
            * $(function(){
            * $("#lb").ejListView({dataBinding:true, dataSource:window.dbitem});
            * });
            * window.dbitem =
            * [   { "text": "Hot Singles"},
            *     { "text": "Rising Artists"},
            *     { "text": "Live Music" },
            *     { "text": "Best of 2013 So Far"},
            *     { "text": "100 Albums - $5 Each"},
            *     { "text": "Hip-Hop and R&B Sale"},
            *     { "text": "CD Deals"}];
            * &lt;/script&gt;  
            * @example      
            * &lt;script&gt;
            * //Get or set  dataBinding, after initialization:<br>
            * // Get the dataBinding API value.		
            * $("#lb").ejListView ("option", "dataBinding");			
            * // Set the dataBinding API
            * $("#lb").ejListView ("option", "dataBinding", true);  
            * &lt;/script&gt;                                  
            * @memberof ejListView
            * @instance
            */
            //Mobile
            /**		
            * Specifies whether to allow the databinding support for the control.
            * @default false
            * @member dataBinding {object}
            * @type {boolean}
            * @example 
            * //Set the dataBinding property in unobtrusive way.
            * &lt;div id="lb" data-role="ejmlistview" data-ej-dataBinding="true" data-ej-datasource="window.dbitem" &gt;
            * &lt;/div&gt;           
            * &lt;script&gt;
            * window.dbitem =
            * [   { "text": "Hot Singles"},
            *     { "text": "Rising Artists"},
            *     { "text": "Live Music" },
            *     { "text": "Best of 2013 So Far"},
            *     { "text": "100 Albums - $5 Each"},
            *     { "text": "Hip-Hop and R&B Sale"},
            *     { "text": "CD Deals"}];
            * &lt;/script&gt;                
            * @example 
            * //Set the dataBinding property in obtrusive way.
            * &lt;div id="lb" &gt;
            * &lt;/div&gt;           
            * &lt;script&gt;
            * $(function(){
            * $("#lb").ejmListView({dataBinding:true, dataSource:window.dbitem});
            * });
            * window.dbitem =
            * [   { "text": "Hot Singles"},
            *     { "text": "Rising Artists"},
            *     { "text": "Live Music" },
            *     { "text": "Best of 2013 So Far"},
            *     { "text": "100 Albums - $5 Each"},
            *     { "text": "Hip-Hop and R&B Sale"},
            *     { "text": "CD Deals"}];
            * &lt;/script&gt;  
            * @example      
            * &lt;script&gt;
            * //Get or set  dataBinding, after initialization:<br>
            * // Get the dataBinding API value.		
            * $("#lb").ejmListView ("option", "dataBinding");			
            * // Set the dataBinding API
            * $("#lb").ejmListView ("option", "dataBinding", true);  
            * &lt;/script&gt;                                  
            * @memberof ejmListView
            * @instance
            */
            dataBinding: false,
            /**		
            * Specifies the datasource when databinding feature is enabled.
            * @default []
            * @member dataSource {object}
            * @type {JSONArray}
            * @example 
            * //Set the dataSource property in unobtrusive way.
            * &lt;div id="lb" data-role="ejlistview" data-ej-dataBinding="true" data-ej-datasource="window.dbitem"&gt;
            * &lt;/div&gt;           
            * &lt;script&gt;
            * window.dbitem =
            * [   { "text": "Hot Singles"},
            *     { "text": "Rising Artists"},
            *     { "text": "Live Music" },
            *     { "text": "Best of 2013 So Far"},
            *     { "text": "100 Albums - $5 Each"},
            *     { "text": "Hip-Hop and R&B Sale"},
            *     { "text": "CD Deals"}];
            * &lt;/script&gt; 
            * @example 
            * //Set the dataSource property in obtrusive way.
            * &lt;div id="lb" &gt;
            * &lt;/div&gt;           
            * &lt;script&gt;
            * $(function(){
            * $("#lb").ejListView({dataBinding:true, dataSource:window.dbitem});
            * });
            * window.dbitem =
            * [   { "text": "Hot Singles"},
            *     { "text": "Rising Artists"},
            *     { "text": "Live Music" },
            *     { "text": "Best of 2013 So Far"},
            *     { "text": "100 Albums - $5 Each"},
            *     { "text": "Hip-Hop and R&B Sale"},
            *     { "text": "CD Deals"}];
            * &lt;/script&gt;  
            * @example       
            * &lt;script&gt;
            * //Get or set  dataSource, after initialization:<br>
            * // Get the dataSource API value.		
            * $("#lb").ejListView ("option", "dataSource");			
            * // Set the dataSource API
            * $("#lb").ejListView ("option", "dataSource", true);   
            * &lt;/script&gt;                     
            * @memberof ejListView
            * @instance
            */
            //Mobile
            /**		
            * Specifies the datasource when databinding feature is enabled.
            * @default []
            * @member dataSource {object}
            * @type {JSONArray}
            * @example 
            * //Set the dataSource property in unobtrusive way.
            * &lt;div id="lb" data-role="ejmlistview" data-ej-dataBinding="true" data-ej-datasource="window.dbitem"&gt;
            * &lt;/div&gt;           
            * &lt;script&gt;
            * window.dbitem =
            * [   { "text": "Hot Singles"},
            *     { "text": "Rising Artists"},
            *     { "text": "Live Music" },
            *     { "text": "Best of 2013 So Far"},
            *     { "text": "100 Albums - $5 Each"},
            *     { "text": "Hip-Hop and R&B Sale"},
            *     { "text": "CD Deals"}];
            * &lt;/script&gt; 
            * @example 
            * //Set the dataSource property in obtrusive way.
            * &lt;div id="lb" &gt;
            * &lt;/div&gt;           
            * &lt;script&gt;
            * $(function(){
            * $("#lb").ejmListView({dataBinding:true, dataSource:window.dbitem});
            * });
            * window.dbitem =
            * [   { "text": "Hot Singles"},
            *     { "text": "Rising Artists"},
            *     { "text": "Live Music" },
            *     { "text": "Best of 2013 So Far"},
            *     { "text": "100 Albums - $5 Each"},
            *     { "text": "Hip-Hop and R&B Sale"},
            *     { "text": "CD Deals"}];
            * &lt;/script&gt;  
            * @example       
            * &lt;script&gt;
            * //Get or set  dataSource, after initialization:<br>
            * // Get the dataSource API value.		
            * $("#lb").ejmListView ("option", "dataSource");			
            * // Set the dataSource API
            * $("#lb").ejmListView ("option", "dataSource", true);   
            * &lt;/script&gt;                     
            * @memberof ejmListView
            * @instance
            */
            dataSource: [],
            /**		
            * Specifies the query to execute with the datasource when databinding feature is enabled.
            * @default null
            * @member query {object}
            * @type {ej.Query}
            * @example 
            * //Set the query property in unobtrusive way.
            * &lt;div id="lb" data-role="ejlistview" data-ej-dataBinding="true" data-ej-fieldSettings="window.dbitem" data-ej-datasource="window.datasource" 
            * data-ej-query="ej.Query().from('Orders').select('ShipCity').take(5)"&gt;
            * &lt;/div&gt;            
            * &lt;script&gt;            
            *		// DataManager creation
            *		window.datasource = ej.DataManager({
            *   	url: "http://mvc.syncfusion.com/Services/Northwnd.svc/"
            *		});
            *		window.dbitem = { "text": "ShipCity" };            
            * &lt;/script&gt;
            * @example 
            * //Set the query property in obtrusive way.
            * &lt;div id="lb" &gt;
            * &lt;/div&gt;            
            * &lt;script&gt;            
            *		// DataManager creation
            *		window.datasource = ej.DataManager({
            *   	url: "http://mvc.syncfusion.com/Services/Northwnd.svc/"
            *		});
            *		window.dbitem = { "text": "ShipCity" };   
            * $(function(){
            * $("#lb").ejListView({dataBinding:true,fieldSettings:"window.dbitem",dataSource:"window.datasource",query:"ej.Query().from('Orders').select('ShipCity').take(5)"});
            * });         
            * &lt;/script&gt;
            * @example        
            * &lt;script&gt;  
            * //Get or set  query, after initialization:<br>
            * // Get the query API value.		
            * $("#lb").ejListView ("option", "query");			
            * // Set the query API
            * $("#lb").ejListView ("option", "query", true);  
            * &lt;/script&gt;
            * @memberof ejListView
            * @instance
            */
            //Mobile
            /**		
            * Specifies the query to execute with the datasource when databinding feature is enabled.
            * @default null
            * @member query {object}
            * @type {ej.Query}
            * @example 
            * //Set the query property in unobtrusive way.
            * &lt;div id="lb" data-role="ejmlistview" data-ej-dataBinding="true" data-ej-fieldSettings="window.dbitem" data-ej-datasource="window.datasource" 
            * data-ej-query="ej.Query().from('Orders').select('ShipCity').take(5)"&gt;
            * &lt;/div&gt;            
            * &lt;script&gt;            
            *		// DataManager creation
            *		window.datasource = ej.DataManager({
            *   	url: "http://mvc.syncfusion.com/Services/Northwnd.svc/"
            *		});
            *		window.dbitem = { "text": "ShipCity" };            
            * &lt;/script&gt;
            * @example 
            * //Set the query property in obtrusive way.
            * &lt;div id="lb" &gt;
            * &lt;/div&gt;            
            * &lt;script&gt;            
            *		// DataManager creation
            *		window.datasource = ej.DataManager({
            *   	url: "http://mvc.syncfusion.com/Services/Northwnd.svc/"
            *		});
            *		window.dbitem = { "text": "ShipCity" };   
            * $(function(){
            * $("#lb").ejmListView({dataBinding:true,fieldSettings:"window.dbitem",dataSource:"window.datasource",query:"ej.Query().from('Orders').select('ShipCity').take(5)"});
            * });         
            * &lt;/script&gt;
            * @example        
            * &lt;script&gt;  
            * //Get or set  query, after initialization:<br>
            * // Get the query API value.		
            * $("#lb").ejmListView ("option", "query");			
            * // Set the query API
            * $("#lb").ejmListView ("option", "query", true);  
            * &lt;/script&gt;
            * @memberof ejmListView
            * @instance
            */
            query: null,
            /**		
            * Specifies whether to show the header.
            * @default true
            * @member showHeader {object}
            * @type {boolean}
            * @example 
            * //Set the showHeader property in unobtrusive way.
            * &lt;div id="lb" data-role="ejlistview" data-ej-showHeader="true"&gt;
            *	 &lt;ul&gt;
            *		&lt;li data-ej-text="Artwork"&gt;&lt;/li&gt;
            * 		&lt;li data-ej-text="Abstract"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="2 Acrylic Mediums"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Creative Acrylic"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Modern Painting"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Canvas Art"&gt;&lt;/li&gt;
            * 		&lt;li data-ej-text="Black white"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Children"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Preschool Crafts"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="School-age Crafts"&gt;&lt;/li&gt;
            * 	&lt;/ul&gt;
            * &lt;/div&gt;
            * @example 
            * //Set the showHeader property in obtrusive way.
            * &lt;div id="lb"&gt;
            *	 &lt;ul&gt;
            *		&lt;li data-ej-text="Artwork"&gt;&lt;/li&gt;
            * 		&lt;li data-ej-text="Abstract"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="2 Acrylic Mediums"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Creative Acrylic"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Modern Painting"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Canvas Art"&gt;&lt;/li&gt;
            * 		&lt;li data-ej-text="Black white"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Children"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Preschool Crafts"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="School-age Crafts"&gt;&lt;/li&gt;
            * 	&lt;/ul&gt;
            * &lt;/div&gt;
            * &lt;script&gt;
            * // Set showHeader on initialization. 
            * //To set showHeader API value 
            * $("#lb").ejListView ({ showHeader: true });
            * &lt;/script&gt;
            * @example 
            * &lt;script&gt;
            * //Get or set  showHeader, after initialization:<br>
            * // Get the showHeader API value.		
            *  $("#lb").ejListView ("option", "showHeader");			
            * // Set the showHeader API
            * $("#lb").ejListView ("option", "showHeader", true);  
            * &lt;/script&gt;          
            * @memberof ejListView
            * @instance
            */
            //Mobile
            /**		
            * Specifies whether to show the header.
            * @default false
            * @member showHeader {object}
            * @type {boolean}
            * @example 
            * //Set the showHeader property in unobtrusive way.
            * &lt;div id="lb" data-role="ejmlistview" data-ej-showHeader="true"&gt;
            *	 &lt;ul&gt;
            *		&lt;li data-ej-text="Artwork"&gt;&lt;/li&gt;
            * 		&lt;li data-ej-text="Abstract"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="2 Acrylic Mediums"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Creative Acrylic"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Modern Painting"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Canvas Art"&gt;&lt;/li&gt;
            * 		&lt;li data-ej-text="Black white"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Children"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Preschool Crafts"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="School-age Crafts"&gt;&lt;/li&gt;
            * 	&lt;/ul&gt;
            * &lt;/div&gt;
            * @example 
            * //Set the showHeader property in obtrusive way.
            * &lt;div id="lb"&gt;
            *	 &lt;ul&gt;
            *		&lt;li data-ej-text="Artwork"&gt;&lt;/li&gt;
            * 		&lt;li data-ej-text="Abstract"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="2 Acrylic Mediums"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Creative Acrylic"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Modern Painting"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Canvas Art"&gt;&lt;/li&gt;
            * 		&lt;li data-ej-text="Black white"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Children"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Preschool Crafts"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="School-age Crafts"&gt;&lt;/li&gt;
            * 	&lt;/ul&gt;
            * &lt;/div&gt;
            * &lt;script&gt;
            * // Set showHeader on initialization. 
            * //To set showHeader API value 
            * $("#lb").ejmListView ({ showHeader: true });
            * &lt;/script&gt;
            * @example 
            * &lt;script&gt;
            * //Get or set  showHeader, after initialization:<br>
            * // Get the showHeader API value.		
            *  $("#lb").ejmListView ("option", "showHeader");			
            * // Set the showHeader API
            * $("#lb").ejmListView ("option", "showHeader", true);  
            * &lt;/script&gt;          
            * @memberof ejmListView
            * @instance
            */
            showHeader: false,
            /**		
            * Specifies the title of the header.
            * @default Title
            * @member headerTitle {object}
            * @type {string}
            * @example 
            * //Set the headerTitle property in unobtrusive way.
            * &lt;div id="lb" data-role="ejlistview" data-ej-headerTitle="Title"&gt;
            *	 &lt;ul&gt;
            *		&lt;li data-ej-text="Artwork"&gt;&lt;/li&gt;
            * 		&lt;li data-ej-text="Abstract"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="2 Acrylic Mediums"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Creative Acrylic"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Modern Painting"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Canvas Art"&gt;&lt;/li&gt;
            * 		&lt;li data-ej-text="Black white"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Children"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Preschool Crafts"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="School-age Crafts"&gt;&lt;/li&gt;
            * 	&lt;/ul&gt;
            * &lt;/div&gt;
            * @example 
            * //Set the headerTitle property in obtrusive way.
            * &lt;div id="lb" &gt;
            *	 &lt;ul&gt;
            *		&lt;li data-ej-text="Artwork"&gt;&lt;/li&gt;
            * 		&lt;li data-ej-text="Abstract"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="2 Acrylic Mediums"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Creative Acrylic"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Modern Painting"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Canvas Art"&gt;&lt;/li&gt;
            * 		&lt;li data-ej-text="Black white"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Children"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Preschool Crafts"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="School-age Crafts"&gt;&lt;/li&gt;
            * 	&lt;/ul&gt;
            * &lt;/div&gt;
            * &lt;script&gt;
            * // Set headerTitle on initialization. 
            * //To set headerTitle API value 
            * $("#lb").ejListView ({ headerTitle: "Title" });
            * &lt;/script&gt;
            * @example 
            * &lt;script&gt;
            * //Get or set  headerTitle, after initialization:<br>
            * // Get the headerTitle API value.		
            *  $("#lb").ejListView ("option", "headerTitle");			
            * // Set the headerTitle API
            * $("#lb").ejListView ("option", "headerTitle", "Title");
            * &lt;/script&gt;
            * @memberof ejListView
            * @instance
            */
            //Mobile
            /**		
            * Specifies the title of the header.
            * @default Title
            * @member headerTitle {object}
            * @type {string}
            * @example 
            * //Set the headerTitle property in unobtrusive way.
            * &lt;div id="lb" data-role="ejmlistview" data-ej-headerTitle="Title"&gt;
            *	 &lt;ul&gt;
            *		&lt;li data-ej-text="Artwork"&gt;&lt;/li&gt;
            * 		&lt;li data-ej-text="Abstract"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="2 Acrylic Mediums"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Creative Acrylic"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Modern Painting"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Canvas Art"&gt;&lt;/li&gt;
            * 		&lt;li data-ej-text="Black white"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Children"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Preschool Crafts"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="School-age Crafts"&gt;&lt;/li&gt;
            * 	&lt;/ul&gt;
            * &lt;/div&gt;
            * @example 
            * //Set the headerTitle property in obtrusive way.
            * &lt;div id="lb" &gt;
            *	 &lt;ul&gt;
            *		&lt;li data-ej-text="Artwork"&gt;&lt;/li&gt;
            * 		&lt;li data-ej-text="Abstract"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="2 Acrylic Mediums"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Creative Acrylic"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Modern Painting"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Canvas Art"&gt;&lt;/li&gt;
            * 		&lt;li data-ej-text="Black white"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Children"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Preschool Crafts"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="School-age Crafts"&gt;&lt;/li&gt;
            * 	&lt;/ul&gt;
            * &lt;/div&gt;
            * &lt;script&gt;
            * // Set headerTitle on initialization. 
            * //To set headerTitle API value 
            * $("#lb").ejmListView ({ headerTitle: "Title" });
            * &lt;/script&gt;
            * @example 
            * &lt;script&gt;
            * //Get or set  headerTitle, after initialization:<br>
            * // Get the headerTitle API value.		
            *  $("#lb").ejmListView ("option", "headerTitle");			
            * // Set the headerTitle API
            * $("#lb").ejmListView ("option", "headerTitle", "Title");
            * &lt;/script&gt;
            * @memberof ejmListView
            * @instance
            */
            headerTitle: "Title",
            /**		
            * Specifies the text of the back button in the header.
            * @default null
            * @member headerBackButtonText {object}
            * @type {string}
            * @example 
            * //Set the headerBackButtonText property in unobtrusive way.
            * &lt;div id="lb" data-role="ejlistview" data-ej-showHeaderBackButton="true" data-ej-headerBackButtonText="Back"&gt;
            *	 &lt;ul&gt;
            *		&lt;li data-ej-text="Artwork"&gt;&lt;/li&gt;
            * 		&lt;li data-ej-text="Abstract"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="2 Acrylic Mediums"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Creative Acrylic"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Modern Painting"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Canvas Art"&gt;&lt;/li&gt;
            * 		&lt;li data-ej-text="Black white"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Children"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Preschool Crafts"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="School-age Crafts"&gt;&lt;/li&gt;
            * 	&lt;/ul&gt;
            * &lt;/div&gt;
            * @example 
            * //Set the headerBackButtonText property in obtrusive way.
            * &lt;div id="lb" data-role="ejmlistview" data-ej-showHeaderBackButton="true"&gt;
            *	 &lt;ul&gt;
            *		&lt;li data-ej-text="Artwork"&gt;&lt;/li&gt;
            * 		&lt;li data-ej-text="Abstract"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="2 Acrylic Mediums"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Creative Acrylic"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Modern Painting"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Canvas Art"&gt;&lt;/li&gt;
            * 		&lt;li data-ej-text="Black white"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Children"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Preschool Crafts"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="School-age Crafts"&gt;&lt;/li&gt;
            * 	&lt;/ul&gt;
            * &lt;/div&gt;
            * &lt;script&gt;
            * // Set headerBackButtonText on initialization. 
            * //To set headerBackButtonText API value 
            * $("#lb").ejListView ({ headerBackButtonText: "Back" });
            * &lt;/script&gt;
            * @example 
            * &lt;script&gt;
            * //Get or set  headerBackButtonText, after initialization:<br>
            * // Get the headerBackButtonText API value.		
            *  $("#lb").ejListView ("option", "headerBackButtonText");			
            * // Set the headerBackButtonText API
            * $("#lb").ejListView ("option", "headerBackButtonText", "Back");
            * &lt;/script&gt;
            * @memberof ejListView
            * @instance
            */
            //Mobile
            /**		
            * Specifies the text of the back button in the header.
            * @default undefined
            * @member headerBackButtonText {object}
            * @type {string}
            * @example 
            * //Set the headerBackButtonText property in unobtrusive way.
            * &lt;div id="lb" data-role="ejmlistview" data-ej-showHeaderBackButton="true" data-ej-headerBackButtonText="Back"&gt;
            *	 &lt;ul&gt;
            *		&lt;li data-ej-text="Artwork"&gt;&lt;/li&gt;
            * 		&lt;li data-ej-text="Abstract"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="2 Acrylic Mediums"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Creative Acrylic"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Modern Painting"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Canvas Art"&gt;&lt;/li&gt;
            * 		&lt;li data-ej-text="Black white"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Children"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Preschool Crafts"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="School-age Crafts"&gt;&lt;/li&gt;
            * 	&lt;/ul&gt;
            * &lt;/div&gt;
            * @example 
            * //Set the headerBackButtonText property in obtrusive way.
            * &lt;div id="lb" data-role="ejmlistview" data-ej-showHeaderBackButton="true"&gt;
            *	 &lt;ul&gt;
            *		&lt;li data-ej-text="Artwork"&gt;&lt;/li&gt;
            * 		&lt;li data-ej-text="Abstract"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="2 Acrylic Mediums"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Creative Acrylic"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Modern Painting"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Canvas Art"&gt;&lt;/li&gt;
            * 		&lt;li data-ej-text="Black white"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Children"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Preschool Crafts"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="School-age Crafts"&gt;&lt;/li&gt;
            * 	&lt;/ul&gt;
            * &lt;/div&gt;
            * &lt;script&gt;
            * // Set headerBackButtonText on initialization. 
            * //To set headerBackButtonText API value 
            * $("#lb").ejmListView ({ headerBackButtonText: "Back" });
            * &lt;/script&gt;
            * @example 
            * &lt;script&gt;
            * //Get or set  headerBackButtonText, after initialization:<br>
            * // Get the headerBackButtonText API value.		
            *  $("#lb").ejmListView ("option", "headerBackButtonText");			
            * // Set the headerBackButtonText API
            * $("#lb").ejmListView ("option", "headerBackButtonText", "Back");
            * &lt;/script&gt;
            * @memberof ejmListView
            * @instance
            */
            headerBackButtonText: null,
            /**		
            * Specifies the field settings to map the datasource when databinding feature is enabled.
            * @type {JSONArray}
            * @member fieldSettings {object}
            * @example 
            * //Set the fieldSettings property in unobtrusive way.
            * &lt;div id="lb" data-role="ejlistview" data-ej-dataBinding="true" data-ej-fieldSettings="window.musicFields" data-ej-datasource="window.dbitem" &gt;
            * &lt;/div&gt;           
            * &lt;script&gt;
            * window.dbitem =
            * [{ "Texts": "Discover Music", "PrimaryKeys": "1" },
            *     { "Texts": "Hot Singles", "ParentPrimaryKeyss": "1" },
            *     { "Texts": "Rising Artists", "PrimaryKeyss": null, "ParentPrimaryKeyss": "1" },
            *     { "Texts": "Live Music", "ParentPrimaryKeyss": "1" },
            *     { "Texts": "Best of 2013 So Far", "ParentPrimaryKeyss": "1" },
            *{ "Texts": "Sales and Events", "PrimaryKeys": "2" },
            *     { "Texts": "100 Albums - $5 Each", "ParentPrimaryKeyss": "2" },
            *     { "Texts": "Hip-Hop and R&B Sale", "ParentPrimaryKeyss": "2" },
            *     { "Texts": "CD Deals", "ParentPrimaryKeyss": "2" }];
            * window.musicFields = {
            * "href": "Hrefs",
            * "text": "Texts",
            * "primaryKey": "PrimaryKeys",
            * "parentPrimaryKey": "ParentPrimaryKeyss"
            * };
            * &lt;/script&gt;
            * @example 
            * //Set the fieldSettings property in obtrusive way.
            * &lt;div id="lb" &gt;
            * &lt;/div&gt;           
            * &lt;script&gt;
            * window.dbitem =
            * [{ "Texts": "Discover Music", "PrimaryKeys": "1" },
            *     { "Texts": "Hot Singles", "ParentPrimaryKeyss": "1" },
            *     { "Texts": "Rising Artists", "PrimaryKeyss": null, "ParentPrimaryKeyss": "1" },
            *     { "Texts": "Live Music", "ParentPrimaryKeyss": "1" },
            *     { "Texts": "Best of 2013 So Far", "ParentPrimaryKeyss": "1" },
            *{ "Texts": "Sales and Events", "PrimaryKeys": "2" },
            *     { "Texts": "100 Albums - $5 Each", "ParentPrimaryKeyss": "2" },
            *     { "Texts": "Hip-Hop and R&B Sale", "ParentPrimaryKeyss": "2" },
            *     { "Texts": "CD Deals", "ParentPrimaryKeyss": "2" }];
            * window.musicFields = {
            * "href": "Hrefs",
            * "text": "Texts",
            * "primaryKey": "PrimaryKeys",
            * "parentPrimaryKey": "ParentPrimaryKeyss"
            * };
            * $(function(){
            * $("#lb").ejListView({dataBinding:true,fieldSettings:"window.musicFields",dataSource:"window.dbitem"});
            * });
            * &lt;/script&gt;
            * @example         
            * &lt;script&gt;
            * //Get or set  fieldSettings, after initialization:<br>
            * // Get the fieldSettings API value.		
            * $("#lb").ejListView ("option", "fieldSettings");			
            * // Set the fieldSettings API
            * $("#lb").ejListView ("option", "fieldSettings", true); 
            * &lt;/script&gt; 
            * @memberof ejListView
            * @instance
            */
            //Mobile
            /**		
            * Specifies the field settings to map the datasource when databinding feature is enabled.
            * @member fieldSettings {object}
            * @type {JSONArray}
            * @example 
            * //Set the fieldSettings property in unobtrusive way.
            * &lt;div id="lb" data-role="ejmlistview" data-ej-dataBinding="true" data-ej-fieldSettings="window.musicFields" data-ej-datasource="window.dbitem" &gt;
            * &lt;/div&gt;           
            * &lt;script&gt;
            * window.dbitem =
            * [{ "Texts": "Discover Music", "PrimaryKeys": "1" },
            *     { "Texts": "Hot Singles", "ParentPrimaryKeyss": "1" },
            *     { "Texts": "Rising Artists", "PrimaryKeyss": null, "ParentPrimaryKeyss": "1" },
            *     { "Texts": "Live Music", "ParentPrimaryKeyss": "1" },
            *     { "Texts": "Best of 2013 So Far", "ParentPrimaryKeyss": "1" },
            *{ "Texts": "Sales and Events", "PrimaryKeys": "2" },
            *     { "Texts": "100 Albums - $5 Each", "ParentPrimaryKeyss": "2" },
            *     { "Texts": "Hip-Hop and R&B Sale", "ParentPrimaryKeyss": "2" },
            *     { "Texts": "CD Deals", "ParentPrimaryKeyss": "2" }];
            * window.musicFields = {
            * "href": "Hrefs",
            * "text": "Texts",
            * "primaryKey": "PrimaryKeys",
            * "parentPrimaryKey": "ParentPrimaryKeyss"
            * };
            * &lt;/script&gt;
            * @example 
            * //Set the fieldSettings property in obtrusive way.
            * &lt;div id="lb" &gt;
            * &lt;/div&gt;           
            * &lt;script&gt;
            * window.dbitem =
            * [{ "Texts": "Discover Music", "PrimaryKeys": "1" },
            *     { "Texts": "Hot Singles", "ParentPrimaryKeyss": "1" },
            *     { "Texts": "Rising Artists", "PrimaryKeyss": null, "ParentPrimaryKeyss": "1" },
            *     { "Texts": "Live Music", "ParentPrimaryKeyss": "1" },
            *     { "Texts": "Best of 2013 So Far", "ParentPrimaryKeyss": "1" },
            *{ "Texts": "Sales and Events", "PrimaryKeys": "2" },
            *     { "Texts": "100 Albums - $5 Each", "ParentPrimaryKeyss": "2" },
            *     { "Texts": "Hip-Hop and R&B Sale", "ParentPrimaryKeyss": "2" },
            *     { "Texts": "CD Deals", "ParentPrimaryKeyss": "2" }];
            * window.musicFields = {
            * "href": "Hrefs",
            * "text": "Texts",
            * "primaryKey": "PrimaryKeys",
            * "parentPrimaryKey": "ParentPrimaryKeyss"
            * };
            * $(function(){
            * $("#lb").ejmListView({dataBinding:true,fieldSettings:"window.musicFields",dataSource:"window.dbitem"});
            * });
            * &lt;/script&gt;
            * @example         
            * &lt;script&gt;
            * //Get or set  fieldSettings, after initialization:<br>
            * // Get the fieldSettings API value.		
            * $("#lb").ejmListView ("option", "fieldSettings");			
            * // Set the fieldSettings API
            * $("#lb").ejmListView ("option", "fieldSettings", true); 
            * &lt;/script&gt; 
            * @memberof ejmListView
            * @instance
            */
            fieldSettings: {
                "navigateUrl": "navigateUrl",
                "href": "href",
                "enableAjax": "enableAjax",
                "preventSelection": "preventSelection",
                "persistSelection": "persistSelection",
                "text": "text",
                "enableCheckMark": "enableCheckMark",
                "checked": "checked",
                "primaryKey": "primaryKey",
                "parentPrimaryKey": "parentPrimaryKey",
                "imageClass": "imageClass",
                "imageUrl": "imageUrl",
                "childHeaderTitle": "childHeaderTitle",
                "childId": "childId",
                "childHeaderBackButtonText": "childHeaderBackButtonText",
                "renderTemplate": "renderTemplate",
                "templateId": "templateId",
                "touchStart": "touchStart",
                "touchEnd": "touchEnd",
                "attributes": "attributes"
            },
            /**		
            * Specifies whether to enable the filtering feature to filter the item.
            * @default false
            * @member enableFiltering {object}
            * @type {boolean}
            * @example 
            * //Set the enableFiltering property in unobtrusive way.
            * &lt;div id="lb" data-role="ejlistview" data-ej-enableFiltering="true"&gt;
            * 	&lt;ul&gt;
            *		&lt;li data-ej-text="Artwork"&gt;&lt;/li&gt;
            * 		&lt;li data-ej-text="Abstract"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="2 Acrylic Mediums"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Creative Acrylic"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Modern Painting"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Canvas Art"&gt;&lt;/li&gt;
            * 		&lt;li data-ej-text="Black white"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Children"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Preschool Crafts"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="School-age Crafts"&gt;&lt;/li&gt;
            *	&lt;/ul&gt;
            * &lt;/div&gt;
            * @example 
            * //Set the enableFiltering property in obtrusive way.
            * &lt;div id="lb"&gt;
            * 	&lt;ul&gt;
            *		&lt;li data-ej-text="Artwork"&gt;&lt;/li&gt;
            * 		&lt;li data-ej-text="Abstract"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="2 Acrylic Mediums"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Creative Acrylic"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Modern Painting"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Canvas Art"&gt;&lt;/li&gt;
            * 		&lt;li data-ej-text="Black white"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Children"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Preschool Crafts"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="School-age Crafts"&gt;&lt;/li&gt;
            *	&lt;/ul&gt;
            * &lt;/div&gt;
            * &lt;script&gt;
            * // Set enableFiltering on initialization. 
            * //To set enableFiltering API value 
            * $("#lb").ejListView ({ enableFiltering: true });
            * &lt;/script&gt;
            * @example 
            * &lt;script&gt;
            * //Get or set the ListView enableFiltering, after initialization:<br>
            * // Get the enableFiltering API value.		
            * $("#lb").ejListView ("option", "enableFiltering");			
            * // Set the enableFiltering API
            * $("#lb").ejListView ("option", "enableFiltering", true);    
            * &lt;/script&gt;        
            * @memberof ejListView
            * @instance
            */
            //Mobile
            /**		
            * Specifies whether to enable the filtering feature to filter the item.
            * @default false
            * @member enableFiltering {object}
            * @type {boolean}
            * @example 
            * //Set the enableFiltering property in unobtrusive way.
            * &lt;div id="lb" data-role="ejmlistview" data-ej-enableFiltering="true"&gt;
            * 	&lt;ul&gt;
            *		&lt;li data-ej-text="Artwork"&gt;&lt;/li&gt;
            * 		&lt;li data-ej-text="Abstract"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="2 Acrylic Mediums"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Creative Acrylic"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Modern Painting"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Canvas Art"&gt;&lt;/li&gt;
            * 		&lt;li data-ej-text="Black white"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Children"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Preschool Crafts"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="School-age Crafts"&gt;&lt;/li&gt;
            *	&lt;/ul&gt;
            * &lt;/div&gt;
            * @example 
            * //Set the enableFiltering property in obtrusive way.
            * &lt;div id="lb"&gt;
            * 	&lt;ul&gt;
            *		&lt;li data-ej-text="Artwork"&gt;&lt;/li&gt;
            * 		&lt;li data-ej-text="Abstract"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="2 Acrylic Mediums"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Creative Acrylic"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Modern Painting"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Canvas Art"&gt;&lt;/li&gt;
            * 		&lt;li data-ej-text="Black white"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Children"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Preschool Crafts"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="School-age Crafts"&gt;&lt;/li&gt;
            *	&lt;/ul&gt;
            * &lt;/div&gt;
            * &lt;script&gt;
            * // Set enableFiltering on initialization. 
            * //To set enableFiltering API value 
            * $("#lb").ejmListView ({ enableFiltering: true });
            * &lt;/script&gt;
            * @example 
            * &lt;script&gt;
            * //Get or set the ListView enableFiltering, after initialization:<br>
            * // Get the enableFiltering API value.		
            * $("#lb").ejmListView ("option", "enableFiltering");			
            * // Set the enableFiltering API
            * $("#lb").ejmListView ("option", "enableFiltering", true);    
            * &lt;/script&gt;        
            * @memberof ejmListView
            * @instance
            */
            enableFiltering: false,
            /**		
            * Specifies whether to enable check mark for the item.
            * @default false
            * @member enableCheckMark {object}
            * @type {boolean}
            * @example 
            * //Set the enableCheckMark property in unobtrusive way.
            * &lt;div id="lb" data-role="ejlistview" data-ej-enableCheckMark="true"&gt;
            * 	&lt;ul&gt;
            *		&lt;li data-ej-text="Artwork"&gt;&lt;/li&gt;
            * 		&lt;li data-ej-text="Abstract"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="2 Acrylic Mediums"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Creative Acrylic"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Modern Painting"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Canvas Art"&gt;&lt;/li&gt;
            * 		&lt;li data-ej-text="Black white"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Children"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Preschool Crafts"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="School-age Crafts"&gt;&lt;/li&gt;
            *	&lt;/ul&gt;
            * &lt;/div&gt;
            * @example 
            * //Set the enableCheckMark property in obtrusive way.
            * &lt;div id="lb"&gt;
            * 	&lt;ul&gt;
            *		&lt;li data-ej-text="Artwork"&gt;&lt;/li&gt;
            * 		&lt;li data-ej-text="Abstract"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="2 Acrylic Mediums"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Creative Acrylic"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Modern Painting"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Canvas Art"&gt;&lt;/li&gt;
            * 		&lt;li data-ej-text="Black white"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Children"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Preschool Crafts"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="School-age Crafts"&gt;&lt;/li&gt;
            *	&lt;/ul&gt;
            * &lt;/div&gt;
            * &lt;script&gt;
            * // Set enableCheckMark on initialization. 
            * //To set enableCheckMark API value 
            * $("#lb").ejListView ({ enableCheckMark: true });
            * &lt;/script&gt;
            * @example 
            * &lt;script&gt;
            * //Get or set the ListView enableCheckMark, after initialization:<br>
            * // Get the enableCheckMark API value.		
            * $("#lb").ejListView ("option", "enableCheckMark");			
            * // Set the enableCheckMark API
            * $("#lb").ejListView ("option", "enableCheckMark", true);    
            * &lt;/script&gt;        
            * @memberof ejListView
            * @instance
            */
            /**		
            * Specifies whether to enable check mark for the item.
            * @default false
            * @member enableCheckMark {object}
            * @type {boolean}
            * @example 
            * //Set the enableCheckMark property in unobtrusive way.
            * &lt;div id="lb" data-role="ejmlistview" data-ej-enableCheckMark="true"&gt;
            * 	&lt;ul&gt;
            *		&lt;li data-ej-text="Artwork"&gt;&lt;/li&gt;
            * 		&lt;li data-ej-text="Abstract"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="2 Acrylic Mediums"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Creative Acrylic"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Modern Painting"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Canvas Art"&gt;&lt;/li&gt;
            * 		&lt;li data-ej-text="Black white"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Children"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Preschool Crafts"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="School-age Crafts"&gt;&lt;/li&gt;
            *	&lt;/ul&gt;
            * &lt;/div&gt;
            * @example 
            * //Set the enableCheckMark property in obtrusive way.
            * &lt;div id="lb"&gt;
            * 	&lt;ul&gt;
            *		&lt;li data-ej-text="Artwork"&gt;&lt;/li&gt;
            * 		&lt;li data-ej-text="Abstract"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="2 Acrylic Mediums"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Creative Acrylic"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Modern Painting"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Canvas Art"&gt;&lt;/li&gt;
            * 		&lt;li data-ej-text="Black white"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Children"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Preschool Crafts"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="School-age Crafts"&gt;&lt;/li&gt;
            *	&lt;/ul&gt;
            * &lt;/div&gt;
            * &lt;script&gt;
            * // Set enableCheckMark on initialization. 
            * //To set enableCheckMark API value 
            * $("#lb").ejmListView ({ enableCheckMark: true });
            * &lt;/script&gt;
            * @example 
            * &lt;script&gt;
            * //Get or set the ListView enableCheckMark, after initialization:<br>
            * // Get the enableCheckMark API value.		
            * $("#lb").ejmListView ("option", "enableCheckMark");			
            * // Set the enableCheckMark API
            * $("#lb").ejmListView ("option", "enableCheckMark", true);    
            * &lt;/script&gt;        
            * @memberof ejmListView
            * @instance
            */
            enableCheckMark: false


        },

        observables: ["selectedItemIndex", "dataSource"],
        selectedItemIndex: ej.util.valueFunction("selectedItemIndex"),
        dataSource: ej.util.valueFunction("dataSource"),

        //Before loading the list
        /**
        * Section for ListView control before load
        * @private
        */
        _load: function () {
            this._orgEle = this.element.clone();
            this._index = 0;
            this._items = [];
            this._indexVal = 0;
            this.model.id = this.element[0].id;
            if (this.model.load)
                this._trigger("load");
            this.model.fieldSettings = eval(this.model.fieldSettings);
            if (this.model.fieldSettings && this.model.query != null) {
                var proxy = this;
                this.dataSource().executeQuery(eval(this.model.query)).done(function (e) {
                    proxy.model.dataSource = e.result;
                    proxy._renderControl();
                });
            }
            else
                this._renderControl();
        },
        //To render the list control
        /**
        * Render Section for ListView control	
        * @private
        */
        _renderControl: function () {
            this.element.addClass(this._prefixClass + "parentlv ");
            this._lbEle = ej.buildTag("div." + this._rootCSS);
            this._lbEle.addClass("subpage");
            if (ej.isMobile)
                this._lbEle.addClass("e-m-" + this.model.renderMode + " e-m-" + this.model.theme).addClass(!ej.isMobile() ? "e-m-tablet" : "e-m-mobile").addClass(this.model.ios7.inline ? "e-m-inline" : "");
            this._lContainer = ej.buildTag("div." + this._prefixClass + "list-container#" + this.model.id + "_container");
            var ul = this.element.find(">ul");
            if (this.model.showHeader) {
                var hdr = this._renderHeader(this.model.id, this.model.showHeaderBackButton, this.model.headerTitle, this.model.headerBackButtonText);
                this._lbEle.prepend(hdr);
            }
            if (ej.isMobile)
                if (this.model.allowScrolling)
                    this._renderScrollPanel(this._lbEle, this.model.id + "_scroll", this.model.id + "_container");
            if (this.model.dataBinding && !this.model.renderTemplate)
                this.element.empty();
            if (this.model.renderTemplate) {
                if (this.model.templateId) {
                    if (ej.getCurrentPage().find('#' + this.model.templateId).length) {
                        this._template = ej.getCurrentPage().find('#' + this.model.templateId);
                        ej.destroyWidgets(this._template);
                        this._template.remove();
                        this._template = this._template[0].nodeName && this._template[0].nodeName.toLowerCase() == "script" ? ej.getClearString(this._template[0].innerHTML) : this._template[0].outerHTML;
                    }
                }
                else {
                    ej.destroyWidgets(this.element);
                    this._template = this.element.html();
                    this.element.empty();
                }
            }
            if (this.model.renderTemplate && !this.model.dataBinding && !this.model.enableGroupList) {
                this.element.append(this._lbEle.append(this._lContainer));
                if (this._template) {
                    this._lContainer.append(this._template);
                    if (ej.widget.init)
                        ej.widget.init(this._lContainer);
                }
            }
            else if (!this.model.renderTemplate || this.model.renderTemplate && this.model.dataBinding || this.model.renderTemplate && this.model.enableGroupList) {
                if (this.model.enableGroupList) {
                    this._lContainer.addClass(this._prefixClass + "grouped");
                    var innerDiv = ej.buildTag("div." + this._prefixClass + "grouplist");
                    ul = this.element.children();
                    if (ul.length) {
                        var length = this.model.dataBinding ? this.dataSource().length : ul.length;
                        var title;
                        for (var i = 0; i < length; i++) {
                            if (this.model.dataBinding) {
                                this._items = this.dataSource()[i].items;
                                if (!this.model.renderTemplate) {
                                    ulItem = ej.buildTag('ul');
                                    this._lbEle.append(ulItem);
                                }
                                else
                                    ulItem = ul[i];
                            }
                            else {
                                this._items = [];
                                ulItem = ul[i];
                            }
                            title = this.model.dataBinding ? this.dataSource()[i].key : ej.getAttrVal($(ulItem), 'data-ej-grouplisttitle', "GroupList" + (i + 1));
                            var groupdiv = ej.buildTag("div." + this._prefixClass + "groupdiv", ej.buildTag("div." + this._prefixClass + "grouptitle", title));
                            innerDiv.append(groupdiv.append(this._renderListItems($(ulItem).addClass(this._prefixClass + "grouped"))));
                        }
                    }
                    else if (this._template) {
                        ej.destroyWidgets(this._template);
                        innerDiv[0].innerHTML = this._template;
                    }

                    this._lContainer.append(innerDiv);
                    if (ej.widget.init)
                        ej.widget.init(innerDiv);
                }
                else {
                    if (this.model.dataBinding) {
                        var ul = ej.buildTag('ul');
                        this._items = this.dataSource();
                    }
                    if (ul.length) {
                        ej.destroyWidgets(ul);
                        this._lContainer.append(this._renderListItems(ul));
                    }
                    else if (this._template) {
                        ej.destroyWidgets(this._template);
                        this._lContainer[0].innerHTML = this._template;
                    }
                }
                this.element.prepend(this._lbEle);
                if (this.model.enableFiltering)
                    this._createFilterWrapper(this._lbEle);
                this._lbEle.append(this._lContainer);
                if (ej.widget.init)
                    ej.widget.init(this._lContainer);
                var eLi = this.element.find('li.' + this._prefixClass + 'list');
                eLi.removeEmptyAttrs();
                eLi.find('.' + this._prefixClass + 'list-anchor').removeEmptyAttrs();
                if (ej.widget.init)
                    ej.widget.init(this._lbEle);
                var ulItems = this.element.find('ul');
                ulItems.find('li:first').addClass(this._prefixClass + "firstitem");
                ulItems.find('li:last').addClass(this._prefixClass + "lastitem");
                this._liEl = this.element.find("li");
                if (this.selectedItemIndex() >= 0 && !this.model.preventSelection && this.model.persistSelection) {
                    if (!ej.getBooleanVal(ulItems[this.selectedItemIndex()], 'data-preventselection', this.model.preventSelection) && ej.getBooleanVal(ulItems[this.selectedItemIndex()], 'data-persistselection', this.model.persistSelection)) {
                        this._currentItem = $(this._liEl[this.selectedItemIndex()]);
                        this._prevItem = this._currentItem;
                        this._currentItem.removeClass(this._prefixClass + "state-default").addClass(this._prefixClass + "state-active");
                    }
                }
                $(this.element.find('.' + this._prefixClass + 'lv-check').parent()).addClass(this._prefixClass + 'lv-checkdiv');
                $(this.element.find('.' + this._prefixClass + 'lv-check').closest('li.' + this._prefixClass + 'list')).addClass(this._prefixClass + 'list-check');
                $(this.element.find('.' + this._prefixClass + 'lv-input').closest('.' + this._prefixClass + 'lv-filter')).addClass(this._prefixClass + 'lv-inputdiv');
                this._wireEvents();
            }
            this._setHeightWidth();
            if (this.model.loadComplete)
                this._trigger("loadComplete");
        },
        //To render filtering box wrapper
        /**
        * Creating Filter Wrapper for ListView control	
        * @private
        */
        _createFilterWrapper: function (element) {
            var fDiv = this._createFilterDiv();
            var fAnchor = ej.buildTag("a." + this._prefixClass + "lv-anchor", ej.buildTag('span.' + this._prefixClass + 'input-btn'), {}, { "Title": "Clear text", "data-role": "none" });
            $(fDiv).append(fAnchor);
            element.append(fDiv);
        },
        //To render list items
        /**
        * Section for Render ListItems for ListView control	
        * @private
        */
        _renderListItems: function (ul) {
            var proxy = this;
            ul.addClass(this._prefixClass + "list-hdr " + this._prefixClass + "clearall");
            this._liItems = ul.find(">li");
            if (this._liItems.length && !this.model.dataBinding || this.model.dataBinding) {
                this._renderParentChildTemplate();
                if (!this.model.dataBinding) {
                    this._liItems.each(function (index, element) {
                        proxy._generateData(element, null, ej.getAttrVal(element, 'data-ej-primarykey', null));
                    });
                }
                this._items = eval(this._items);
                var dataMgr = ej.DataManager(this._items);
                var parentItem = dataMgr.executeLocal(ej.Query().from(this._items).where(ej.Predicate(this.model.fieldSettings['parentPrimaryKey'], ej.FilterOperators.equal, null)).group(this.model.fieldSettings['parentPrimaryKey']));
                var grouped = dataMgr.executeLocal(ej.Query().from(this._items).where(ej.Predicate(this.model.fieldSettings['parentPrimaryKey'], ej.FilterOperators.notEqual, null)).group(this.model.fieldSettings['parentPrimaryKey']));
                if (parentItem[0]) {
                    ul.empty().html($(this.jsRender).render(parentItem[0].items));
                    if (grouped.length) {
                        this.element.append($(this.jsChildRender).render(grouped));
                        $.each(grouped, function (index, element) {
                            if (ej.widget.init)
                                ej.widget.init(proxy.element.find('#child' + element.key));
                            var ul = proxy.element.find('#child' + element.key).find('ul');
                            var li = ul.find('li');
                            k = 0;
                            for (var j = 0; j < li.length; j++) {
                                if (grouped[k].items[j] && grouped[k].items[j].attributes)
                                    $(li[j]).addAttrs(grouped[k].items[j].attributes);
                            }
                            k++;
                        });
                    }
                    var ulItems = ul.find('>li');
                    for (var j = 0; j < ulItems.length; j++) {
                        if (parentItem[0].items[j] && parentItem[0].items[j].attributes)
                            $(ulItems[j]).addAttrs(parentItem[0].items[j].attributes);
                    }
                }
            }
            if (this._initEJCheckBox)
                this._initEJCheckBox(ul);
            return ul;
        },


        //To render parent child template
        /**
        * Section for Render Parent Child Template for ListView control	
        * @private
        */
        _renderParentChildTemplate: function () {
            var list = this._renderLists();
            $.views.helpers({ _checkAjaxUrls: this._checkAjaxUrl, _checkImgUrls: this._checkImgUrl, _checkIsChecked: this._checkIsCheck, Object: this, ej: ej });
            //Template for parent item
            this.jsRender = ej.buildTag("script#" + this.model.id + "_Template", "", {}, { "type": "text/x-jsrender" });
            if (ej.browserInfo().name == "msie" && ej.browserInfo().version < 10)
                this.jsRender[0].innerText = list[0].outerHTML.replace(/&gt;/g, '>');
            else
                this.jsRender[0].innerHTML = list[0].outerHTML.replace(/&gt;/g, '>');
            //Template for child item
            this.jsChildRender = ej.buildTag("script#" + this.model.id + "_ChildTemplate", "", {}, { "type": "text/x-jsrender" });
            var ul = ej.buildTag("ul." + this._prefixClass + "childcontainer " + this._prefixClass + "list-hdr " + this._prefixClass + "clearall");
            list.addClass(this._prefixClass + "childli");
            ul[0].innerHTML = "{{for items}}" + list[0].outerHTML + "{{/for}}";
            var div = ej.buildTag("div." + this._rootCSS + " subpage " + this._prefixClass + "childitem", "", {}, { "id": "{{if key}}child{{>key}}{{else " + this.model.fieldSettings['childId'] + "}}{{>" + this.model.fieldSettings['childId'] + "}}{{else}}{{/if}}", "style": "display:none" });
            if (ej.isMobile)
                div.addClass(this._prefixClass + this.model.renderMode + this._prefixClass + this.model.theme + !ej.isMobile() ? " e-m-tablet" : " e-m-mobile");
            var innerdiv = ej.buildTag("div." + this._prefixClass + "list-container", "", {}, { "id": "{{if key}}child{{>key}}{{else " + this.model.fieldSettings['childId'] + "}}{{>" + this.model.fieldSettings['childId'] + "}}{{else}}{{/if}}_container" });
            if (this.model.showHeader)
                div.append(this._renderHeader("{{if key}}child{{>key}}{{else " + this.model.fieldSettings['childId'] + "}}{{>" + this.model.fieldSettings['childId'] + "}}{{else}}{{/if}}", true, "Title"));
            if (this.model.enableFiltering)
                this._createFilterWrapper(div);
            div.append(innerdiv.append(ul));
            if (ej.isMobile)
                if (this.model.allowScrolling)
                    this._renderScrollPanel(div, "{{if key}}child{{>key}}{{else " + this.model.fieldSettings['childId'] + "}}{{>" + this.model.fieldSettings['childId'] + "}}{{else}}{{/if}}_scroll", "{{if key}}child{{>key}}{{else " + this.model.fieldSettings['childId'] + "}}{{>" + this.model.fieldSettings['childId'] + "}}{{else}}{{/if}}_container");
            if (ej.browserInfo().name == "msie" && ej.browserInfo().version < 10)
                this.jsChildRender[0].innerText = div[0].outerHTML.replace(/&gt;/g, '>');
            else
                this.jsChildRender[0].innerHTML = div[0].outerHTML.replace(/&gt;/g, '>');
        },
        //To render the list items
        /**
        * Section for Render Lists for ListView control	
        * @private
        */
        _renderLists: function () {
            var list = ej.buildTag("li", "", {}, {
                "class": this._prefixClass + "user-select " + this._prefixClass + "list " + this._prefixClass + "state-default{{if " + this.model.fieldSettings['primaryKey'] + " || " + this.model.fieldSettings['enableAjax'] + " || " + this.model.enableAjax + "}} " + this._prefixClass + "arrow{{/if}}{{if " + this.model.fieldSettings['imageClass'] + "}} " + this._prefixClass + "margin{{else}}{{if " + this.model.fieldSettings['imageUrl'] + "}} " + this._prefixClass + "margin{{/if}}{{/if}}",
                "data-childid": "{{if " + this.model.fieldSettings['primaryKey'] + "}}child{{>" + this.model.fieldSettings['primaryKey'] + "}}{{else " + this.model.fieldSettings['childId'] + "}}{{>" + this.model.fieldSettings['childId'] + "}}{{else}}{{/if}}",
                "data-childtitle": "{{>" + this.model.fieldSettings['childHeaderTitle'] + "}}",
                "data-childbackbutton": "{{>" + this.model.fieldSettings['childHeaderBackButtonText'] + "}}",
                "data-preventSelection": "{{>" + this.model.fieldSettings['preventSelection'] + "}}",
                "data-persistSelection": "{{>" + this.model.fieldSettings['persistSelection'] + "}}",
                "data-navigateUrl": "{{>" + this.model.fieldSettings['navigateUrl'] + "}}",
                "data-loadajax": "{{>" + this.model.fieldSettings['enableAjax'] + "}}",
                "data-href": '{{>' + this.model.fieldSettings['href'] + '}}{{:~_checkAjaxUrls()}}',
                "data-check": '{{>' + this.model.fieldSettings['enableCheckMark'] + '}}',
                "data-templateid": "{{>" + this.model.fieldSettings['renderTemplate'] + "}}",
                "data-touchstart": "{{>" + this.model.fieldSettings['touchStart'] + "}}",
                "data-touchend": "{{>" + this.model.fieldSettings['touchEnd'] + "}}"
            });
            if (this.model.renderTemplate) {
                if (this.model.dataBinding && this._template)
                    list[0].innerHTML = this._template;
            }
            else {
                var linkTag = ej.buildTag("a", "", {}, { "class": this._prefixClass + "list-anchor " + this._prefixClass + "remove-shadow{{if " + this.model.fieldSettings['imageClass'] + "}} " + this._prefixClass + "margin{{else}}{{if " + this.model.fieldSettings['imageUrl'] + "}} " + this._prefixClass + "margin{{/if}}{{/if}}{{if " + this.model.fieldSettings['primaryKey'] + " || " + this.model.fieldSettings['enableAjax'] + " || " + this.model.enableAjax + "}} " + this._prefixClass + "fontimage{{/if}}", "href": "{{>" + this.model.fieldSettings['navigateUrl'] + "}}" });
                var span = ej.buildTag("span", "{{>" + this.model.fieldSettings['text'] + "}}", {}, { "class": this._prefixClass + "list-text " + this._prefixClass + "rel " + this._prefixClass + "user-select{{if " + this.model.fieldSettings['imageClass'] + "}} " + this._prefixClass + "text{{else}}{{if " + this.model.fieldSettings['imageUrl'] + "}} " + this._prefixClass + "text{{/if}}{{/if}}" });
                var imgClass = ej.buildTag("div", "", {}, { 'class': this._prefixClass + "list-img " + this._prefixClass + "rel " + this._prefixClass + "user-select {{>" + this.model.fieldSettings['imageClass'] + "}}" });
                var check = this._createCheckBox();
                if (this.model.renderMode == "windows" && ej.isMobile())
                    linkTag[0].innerHTML = "{{if !" + this.model.fieldSettings['primaryKey'] + "}}{{if " + this.model.fieldSettings['enableAjax'] + " == undefined || " + this.model.fieldSettings['enableAjax'] + ".toString() == 'false'}}{{if !" + this.model.fieldSettings['navigateUrl'] + "}}{{if " + this.model.fieldSettings['enableCheckMark'] + " !== undefined}}{{if " + this.model.fieldSettings['enableCheckMark'] + ".toString() == 'false' ? " + this.model.fieldSettings['enableCheckMark'] + " : " + this.model.enableCheckMark + "}}" + check[0].outerHTML + "{{/if}}{{else}}{{if " + this.model.enableCheckMark + "}}" + check[0].outerHTML + "{{/if}}{{/if}}{{/if}}{{/if}}{{/if}}" + "{{if " + this.model.fieldSettings['imageClass'] + "}}" + imgClass[0].outerHTML + "{{else}}{{if " + this.model.fieldSettings['imageUrl'] + "}}{{:~_checkImgUrls()}}{{/if}}{{/if}}" + span[0].outerHTML;
                else
                    linkTag[0].innerHTML = "{{if " + this.model.fieldSettings['imageClass'] + "}}" + imgClass[0].outerHTML + "{{else}}{{if " + this.model.fieldSettings['imageUrl'] + "}}{{:~_checkImgUrls()}}{{/if}}{{/if}}" + span[0].outerHTML + "{{if !" + this.model.fieldSettings['primaryKey'] + "}}{{if " + this.model.fieldSettings['enableAjax'] + " == undefined || " + this.model.fieldSettings['enableAjax'] + ".toString() == 'false'}}{{if !" + this.model.fieldSettings['navigateUrl'] + "}}{{if " + this.model.fieldSettings['enableCheckMark'] + " !== undefined}}{{if " + this.model.fieldSettings['enableCheckMark'] + ".toString() == 'false' ? " + this.model.fieldSettings['enableCheckMark'] + " : " + this.model.enableCheckMark + "}}" + check[0].outerHTML + "{{/if}}{{else}}{{if " + this.model.enableCheckMark + "}}" + check[0].outerHTML + "{{/if}}{{/if}}{{/if}}{{/if}}{{/if}}";

                list[0].innerHTML = "{{if " + this.model.fieldSettings['renderTemplate'] + " == undefined || " + this.model.fieldSettings['renderTemplate'] + ".toString() == 'false'}}" + linkTag[0].outerHTML + "{{else}}{{:templateId}}{{/if}}";
            }
            if (this.model.renderMode == "ios7") {
                var innerDiv = ej.buildTag("div." + this._prefixClass + "list-div");
                innerDiv[0].innerHTML = list[0].innerHTML;
                list.empty().append(innerDiv);
            }
            return list;
        },
        //To render child wrapper
        /**
        * Section for Render Child Items for ListView control	
        * @private
        */
        _renderChild: function (childId) {
            this._currentItem.attr('data-childid', childId);
            if (!this._currentItem.attr('data-childtitle'))
                this._currentItem.attr('data-childtitle', this._currentItem.text());
            this._createListDiv(childId);
            this._div.hide();
            if (ej.isMobile)
                this._div.addClass(!ej.isMobile() ? this._prefixClass + "m-tablet" : this._prefixClass + "m-mobile");
            this._container = ej.buildTag("div." + this._prefixClass + "list-container#" + childId + "_container");
            this.element.append(this._div);
            if (this.model.allowScrolling)
                this._renderScrollPanel(this._div, childId + "_scroll", childId + "_container");
        },
        //To check the url while jsrender
        /**
        * Section for check Ajax Url for ListView control	
        * @private
        */
        _checkAjaxUrl: function () {
            var href = this.data.href;
            var listObj = this.getRsc("helpers", "Object");
            var ej = this.getRsc("helpers", "ej");
            if (href && href.indexOf("#") != -1 && href != "#") {
                if (!listObj._storedContent[this.index]) {
                    var ele = ej.getCurrentPage().find(href).remove();
                    listObj._storedContent[this.index] = ele[0].nodeName && ele[0].nodeName.toLowerCase() == "script" ? ej.getClearString(ele[0].innerHTML) : ele[0].outerHTML;
                }
                else if (listObj._storedContent[this.index] && ej.getCurrentPage().find(href).length) {
                    var ele = ej.getCurrentPage().find(href).remove();
                    ele = ele[0].nodeName && ele[0].nodeName.toLowerCase() == "script" ? ej.getClearString(ele[0].innerHTML) : ele[0].outerHTML;
                    listObj._storedContent = ej._pushValue(listObj._storedContent, ele, this.index);
                }
            }
            else {
                if (listObj._storedContent[this.index])
                    listObj._storedContent = ej._pushValue(listObj._storedContent, "", this.index);
            }
        },
        /**
        * Section for check Image URL for ListView control	
        * @private
        */
        _checkImgUrl: function (item) {
            var obj = this.getRsc("helpers", "Object");
            var img = this.data[obj.model.fieldSettings.imageUrl];
            if (!$.support.pushstate)
                img = typeof App == "object" ? App.route.makeUrlAbsolute(img, true) : img;
            return "<img src = " + img + " class='" + obj._prefixClass + "list-img " + obj._prefixClass + "rel " + obj._prefixClass + "user-select'/>";
        },
        /**
        * Section for check isChecked for ListView control	
        * @private
        */
        _checkIsCheck: function (item) {
            return this.data[this.getRsc("helpers", "Object").model.fieldSettings.checked] ? true : false;
        },
        //To generate data to pass in jsrender
        /**
        * Section for Generate Data for ListView control	
        * @private
        */
        _generateData: function (element, parentKey, primaryKey) {
            var childId = null, text = ej.getAttrVal(element, 'data-ej-text', $(element).text());
            var headerTitle = null, $ele = $(element), backButton = ej.getAttrVal(element, 'data-ej-childbuttontext', '');
            var renderTemp = ej.getBooleanVal(element, 'data-ej-rendertemplate', false);
            if (renderTemp) {
                if (ej.getCurrentPage().find('#' + ej.getAttrVal(element, 'data-ej-templateid')).length) {
                    var ele = ej.getCurrentPage().find('#' + ej.getAttrVal(element, 'data-ej-templateid')).remove();
                    this._storedTemplate[this._indexVal] = ele[0].nodeName && ele[0].nodeName.toLowerCase() == "script" ? ej.getClearString(ele[0].innerHTML) : ele[0].outerHTML;
                }
                else if (!this._storedTemplate[this._indexVal])
                    this._storedTemplate[this._indexVal] = element.innerHTML;
                else if (this._storedTemplate[this._indexVal] && ej.getCurrentPage().find('#' + ej.getAttrVal(element, 'data-ej-templateid')).length) {
                    var ele = ej.getCurrentPage().find('#' + ej.getAttrVal(element, 'data-ej-templateid')).remove();
                    ele = ele[0].nodeName && ele[0].nodeName.toLowerCase() == "script" ? ej.getClearString(ele[0].innerHTML) : ele[0].outerHTML;
                    this._storedTemplate = ej._pushValue(this._storedTemplate, ele, this._indexVal);
                }
                this._indexVal++;
            }
            if ($ele.find(">ul").length) {
                var proxy = this;
                headerTitle = ej.getAttrVal(element, 'data-ej-childtitle', text);
                var li = $ele.find(">ul").find('>li');
                li.each(function (index, element) {
                    proxy._generateData(element, ej.getAttrVal(element, 'data-ej-parentkey', primaryKey), ej.getAttrVal(element, 'data-ej-primarykey', null));
                });
            }
            this._items[this._index] = {
                "navigateUrl": ej.getAttrVal(element, 'data-ej-navigateUrl', ""),
                "href": ej.getAttrVal(element, 'data-ej-href', ""),
                "enableAjax": ej.getBooleanVal(element, 'data-ej-enableAjax', this.model.enableAjax),
                "preventSelection": ej.getBooleanVal(element, 'data-ej-preventSelection', this.model.preventSelection),
                "persistSelection": ej.getBooleanVal(element, 'data-ej-persistSelection', this.model.persistSelection),
                "text": text,
                "enableCheckMark": ej.getBooleanVal(element, 'data-ej-enableCheckMark', this.model.enableCheckMark),
                "checked": ej.getBooleanVal(element, 'data-ej-checked', false),
                "primaryKey": primaryKey,
                "parentPrimaryKey": parentKey,
                "imageClass": ej.getAttrVal(element, 'data-ej-imageclass', ""),
                "imageUrl": ej.getAttrVal(element, 'data-ej-imageurl', ""),
                "childHeaderTitle": headerTitle,
                "childId": ej.getAttrVal(element, 'data-ej-childid', "page_" + parseInt(Math.random().toFixed(3) * 1000)),
                "childHeaderBackButtonText": backButton,
                "renderTemplate": renderTemp,
                "templateId": this._storedTemplate[this._indexVal - 1],
                "touchStart": ej.getAttrVal(element, 'data-ej-touchstart', null),
                "touchEnd": ej.getAttrVal(element, 'data-ej-touchend', null),
                "attributes": $ele.removeAttrs(/^data-ej-/)[0].attributes
            };
            this._index += 1;
        },
        //Touch Start event handler for list item
        /**
        * To handle Touch Start Handler Event for list Items.
        * @private
        */
        _onTouchStartHandler: function (evt) {
            if (!ej.isDevice() && !ej._preventDefaultException(evt.target, this._preventDefaultException))
                evt.preventDefault && evt.preventDefault();
            if (ej.isWindows && ej.isWindows())
                ej._touchStartPoints(evt, this);
            this._currentItem = $(evt.currentTarget);
            this._scroll = false;
            if (!ej.getBooleanVal(this._currentItem, 'data-preventSelection', this.model.preventSelection))
                this._addSelection();
            if (this.model.renderMode == "windows" && !this.model.windows.preventSkew)
                this._currentItem.addClass(ej.isMobile() ? ej._getSkewClass($(this._currentItem), evt.pageX, evt.pageY) : this._prefixClass + "m-skew-center");
            this._triggerStartEvent(this._returnData());
            ej.listenEvents([this._liEl, this._liEl],
                           [ej.endEvent(), ej.moveEvent(), ej.cancelEvent()],
                           [this._touchEndDelegate, this._touchMoveDelegate, this._touchMoveDelegate]);
            $(window).bind(ej.endEvent(), this._docClickDelegate);
        },
        //Touch Move event handler for list item to prevent selection while scrolling
        /**
        * To handle Touch move handler for list items to prevent selection while scrolling.
        * @private
        */
        _onTouchMoveHandler: function (evt) {
            if (!ej.isWindows || (ej.isWindows && !ej.isWindows()) || (ej.isWindows && ej.isWindows() && ej._isTouchMoved(evt, this))) {
                this._scroll = true;
                if (!ej.getBooleanVal(this._currentItem, 'data-persistSelection', this.model.persistSelection))
                    this._removeSelection();
                else if (this._prevItem && this._prevItem[0] != this._currentItem[0])
                    this._removeSelection();
                if (this.model.renderMode == "windows" && !this.model.windows.preventSkew)
                    ej._removeSkewClass(this._currentItem);
            }
        },
        //Touch End event handler for list item
        /**
        * To Handle Touch End Handler for List Item.
        * @private
        */
        _onTouchEndHandler: function (evt) {
            this._isFromAjax = false;
            if (!ej.getBooleanVal(this._currentItem, 'data-persistSelection', this.model.persistSelection))
                this._removeSelection();
            if (this.model.renderMode == "windows" && !this.model.windows.preventSkew)
                ej._removeSkewClass(this._currentItem);
            if (this._scroll) {
                this._setCurrent();
                this._unbindEvents(evt);
                return false;
            }
            else if (!this._scroll) {
                if (this._currentItem.find('.' + this._prefixClass + 'lv-check').length) {
                    var lbCheck = this._currentItem.find('.' + this._prefixClass + 'lv-check');
                    this._toggleCheckboxValue(lbCheck);
                }
                if (ej.mobile)
                    var backButton = this._currentItem.attr('data-childbackbutton') == undefined ? this._currentItem.closest('.' + this._rootCSS + '').find('.' + this._prefixClass + 'header').length ? this._currentItem.closest('.' + this._rootCSS + '').find('.' + this._prefixClass + 'header').ejmHeader('getTitle') : "Back" : ej.getAttrVal(this._currentItem, 'data-childbackbutton');
                else
                    var backButton = this._currentItem.attr('data-childbackbutton') == undefined ? this._currentItem.closest('.' + this._rootCSS + '').find('.' + this._prefixClass + 'header').length ? this._currentItem.closest('.' + this._rootCSS + '').find('.' + this._prefixClass + 'header .' + this._prefixClass + 'htitle').text() : "Back" : ej.getAttrVal(this._currentItem, 'data-childbackbutton');
                var urlVal = this._currentItem.attr('data-href');
                if (!this._currentItem.attr('data-navigateUrl')) {
                    var page = this.element.find("[data-ajaxurl='" + this._convertToRelativeUrl(urlVal) + "']");
                    if (ej.getBooleanVal(this._currentItem, 'data-loadajax', this.model.enableAjax) && (!this.model.enableCache || page.length == 0)) {
                        if (page.length)
                            page.remove();
                        this.loadAjaxContent(urlVal, backButton);
                        this._unbindEvents(evt);
                        return;
                    }
                    else if (urlVal && urlVal.indexOf("#") != -1) {
                        if (!$('#' + urlVal.replace('#', '')).length) {
                            this._renderChild(ej.getAttrVal(this._currentItem, 'data-childid', "page_" + parseInt(Math.random().toFixed(3) * 1000)));
                            var content = ej.buildTag("div." + this._prefixClass + "content", this._storedContent[this._currentItem.index()]);
                            this._div.append(this._container.append(content));
                            if (ej.widget.init)
                                ej.widget.init(this._div);
                        }
                    }
                    this._updateContent(this._currentItem, backButton);
                    this._touchEndEventHandler();
                }
                else
                    this._touchEndEventHandler();
                this._prevItem = this._currentItem;
            }
            else if (this.model.renderMode == "windows" && !ej.isMobile())
                this._touchEndEventHandler();
            this._unbindEvents(evt);
        },
        /**
        * Section for Set Current List Item.	
        * @private
        */
        _setCurrent: function (e) {
            if (this._prevItem && ej.getBooleanVal(this._currentItem, 'data-persistSelection', this.model.persistSelection)) {
                this._currentItem = this._prevItem;
                this._currentItem.removeClass(this._prefixClass + "state-default").addClass(this._prefixClass + "state-active");
            }
        },
        /**
        * Section for to UnBind the Events.
        * @private
        */
        _unbindEvents: function (e) {
            if (e && e.target.nodeName && e.target.nodeName.toLowerCase() != "a") {
                //   ej.blockDefaultActions(e);
                this._scroll = false;
            }
            ej.listenEvents([this._liEl, this._liEl],
                           [ej.endEvent(), ej.moveEvent(), ej.cancelEvent()],
                           [this._touchEndDelegate, this._touchMoveDelegate, this._touchMoveDelegate], true);
        },
        //To make the list as selected
        /**
        * Section for to make the list as selected.
        * @private
        */
        _addSelection: function () {
            if (!this._scroll) {
                this._currentItem.closest('.' + this._prefixClass + 'list-container').find('.' + this._prefixClass + 'state-active').removeClass(this._prefixClass + "state-active").addClass(this._prefixClass + "state-default");
                this._currentItem.removeClass(this._prefixClass + "state-default").addClass(this._prefixClass + "state-active");
            }
        },
        //To make the list as unselected
        /**
        * Section for to remove the selected list.
        * @private
        */
        _removeSelection: function () {
            var proxy = this;
            proxy._currentItem.removeClass(this._prefixClass + "state-active").addClass(this._prefixClass + "state-default");
        },

        /**
        * Section for to set Height Width for ListView Control.
        * @private
        */
        _setHeightWidth: function () {
            if (this.model.autoAdjustHeight)
                this.element.height(window.innerHeight);
            else if (this.model.height)
                this.element.height(this.model.height);
            else {
                var hgt = this.element[0].scrollHeight ? this.element[0].scrollHeight : ej.getDimension(this._lbEle.find('.' + this._prefixClass + 'list-container'), 'outerHeight');
                if (this.model.showHeader)
                    hgt = hgt + ej.getDimension(this._lbEle.find('.' + this._prefixClass + 'header'), 'outerHeight');
                if (this.model.enableFiltering)
                    hgt = hgt + ej.getDimension(this._lbEle.find('.' + this._prefixClass + 'lv-inputdiv'), 'outerHeight');
                this.element.height(hgt);
            }
            if (this.model.width)
                this.element.width(this.model.width);
        },
        //Touch end event handler
        /**
        * To handle Touch End event handler.
        * @private
        */
        _touchEndEventHandler: function () {
            this._triggerEndEvent(this._returnData());
        },

        //Doc Click
        /**
        * To handle Doc Click.
        * @private
        */
        _docClick: function (evt) {
            if (this._scroll)
                this._setCurrent();
            $(window).unbind(ej.endEvent(), this._docClickDelegate);
            ej.listenEvents([this._liEl, this._liEl],
                           [ej.endEvent(), ej.moveEvent()],
                           [this._touchEndDelegate, this._touchMoveDelegate], true);
        },
        //Pop State Navigation
        /**
        * To handle pop state navigation.
        * @private
        */
        _popStateNavigation: function (evt, data) {
            if (data.pageUrl && this.model.enableFiltering)
                this._initializeFiltering($("div[data-url='" + data.pageUrl.replace('#', '') + "']"));
        },
        //Anchor tag click handler
        /**
        * To handle anchor click handler.
        * @private
        */
        _anchorClickHandler: function (e) {
            if (this._scroll) {
                ej.blockDefaultActions(e);
                return false;
            }
        },
        /**
        * Section for to resize handler.
        * @private
        */
        _onResize: function () {
            var proxy = this;
            setTimeout(function () {
                proxy._setHeightWidth();
            }, ej.isAndroid() ? 200 : 0);
        },
        _onScrollStop: function (e) {
            ej.blockDefaultActions(e);
        },
        //To create delegates for event handling
        /**
        * Section for to create delegates for Event handling.
        * @private
        */
        _createDelegates: function () {
            this._anchorClickDelegate = $.proxy(this._anchorClickHandler, this);
            this._keyup = $.proxy(this._onKeyUp, this);
            this._touchStartDelegate = $.proxy(this._onTouchStartHandler, this);
            this._touchEndDelegate = $.proxy(this._onTouchEndHandler, this);
            this._touchMoveDelegate = $.proxy(this._onTouchMoveHandler, this);
            this._resizeDelegate = $.proxy(this._onResize, this);
            this._popStateDelegate = $.proxy(this._popStateNavigation, this);
            this._docClickDelegate = $.proxy(this._docClick, this);
        },
        //To wire events for the list
        /**
        * Section for to wire events for the list.
        * @private
        */
        _wireEvents: function (remove) {
            this._createDelegates();
            var eventType = remove ? "unbind" : "bind";
            if (this.model.autoAdjustHeight) {
                var evt = !ej.isDevice() && !ej.isTouchDevice() ? "resize" : "orientationchange";
                ej.listenEvents([window], [evt], [this._resizeDelegate], remove);
            }
            ej.listenEvents([this._liEl.find('a'), this.element.find('.' + this._prefixClass + 'lv-input'), this._liEl, this._liEl],
                ["click", "keyup", ej.startEvent(), ej.cancelEvent()],
                [this._anchorClickDelegate, this._keyup, this._touchStartDelegate, this._touchMoveDelegate], remove);
            this._lContainer.on("scrollstop", $.proxy(this._onScrollStop, this));
            $('body')[eventType]('viewpopstate', this._popStateDelegate);
            if (this.model.enableFiltering)
                this._initializeFiltering(this._lbEle);
        },
        //To initialize filtering
        /**
        * Section for to initialize filtering.
        * @private
        */
        _initializeFiltering: function (element) {
            this._searchItems = $(element).find("." + this._prefixClass + "list");
            this._emptyFilterTextValue(element);
            element.find('.' + this._prefixClass + 'list[style*="display: none"]').show();
            var selector = "a";
            this._elementText = [];
            for (var i = 0; i < this._searchItems.length; i++) {
                if ($(this._searchItems[i]).find(selector))
                    this._elementText.push($.trim($(this._searchItems[i]).find(selector).html().replace(new RegExp('<[^<]+\>', 'g'), "").toLowerCase()));
                else
                    this._elementText.push("");
            }
        },
        //To perform filtering in keyup event
        /**
        * Section for to perform Key Up filtering.
        * @private
        */
        _onKeyUp: function (evt) {
            for (var i = 0; i < this._searchItems.length; i++) {
                if (this._elementText[i].indexOf(evt.target.value) == -1)
                    $(this._searchItems[i]).css("display", "none");
                else
                    $(this._searchItems[i]).css("display", "");
            }
        },
        //To set the model values
        /**
        * To configure the properties at runtime using SetModel		
        * @private
        */
        _setModel: function (options) {
            var refresh = false;
            for (var prop in options) {
                var setModel = "_set" + prop.charAt(0).toUpperCase() + prop.slice(1);
                if (this[setModel])
                    this[setModel](options[prop]);
                else
                    refresh = true;
            }
            if (refresh)
                this._refresh();
        },
        //To change the Theme property
        /**
        * Section for to change the theme property.
        * @private
        */
        _setTheme: function (value) {
            if (value) {
                this.model.theme = value;
                this._lbEle.removeClass("e-m-dark e-m-light e-default").addClass("e-m-" + this.model.theme);
                if (this.model.enableFiltering)
                    this.element.find('.' + this._prefixClass + 'text-input').ejmTextBox('model.theme', this.model.theme);
                if (this.model.showHeader)
                    this._lbEle.find('#' + this.model.id + "_header").ejmHeader('model.theme', this.model.theme);
                if (this.element.find('.' + this._prefixClass + 'childitem').length) {
                    $(this.element.find('.' + this._prefixClass + 'childitem')).removeClass("e-m-dark e-m-light e-default").addClass("e-m-" + this.model.theme);
                    var header = this.element.find('.' + this._prefixClass + 'childitem .' + this._prefixClass + 'header');
                    var proxy = this;
                    header.each(function (index, element) {
                        $(element).ejmHeader('model.theme', proxy.model.theme);
                    });
                }
            }
        },
        //To update the data source
        /**
        * Section for to update the data source.
        * @private
        */
        _setDataSource: function (dataSource, fieldSettings) {
            if (this.model.dataBinding && dataSource) {
                this.dataSource(dataSource);
                if (fieldSettings)
                    this.model.fieldSettings = fieldSettings;
                this._refresh();
            }
        },
        //To get the element
        /**
        * Section for to get the element.	
        * @private
        */
        _getElement: function (childId) {
            return childId ? this.element.find('#' + childId) : this._lbEle;
        },
        //To check the element is disabled
        /**
        * Section for to check the element is disabled.	
        * @private
        */
        _isEnable: function (item) {
            return item.hasClass(this._prefixClass + 'state-disabled') ? false : true;
        },
        //To refresh the control
        _refresh: function () {
            this._destroy();
            this.element.addClass(this._rootCSS);
            this._load();
        },
        //To clear the element
        _clearElement: function () {
            this.element.removeAttr("class");
            this.element.empty().html(this._orgEle.html());
        },
        // all events bound using this._on will be unbind automatically
        _destroy: function () {
            this._wireEvents(true);
            this._clearElement();
        },

        /*-------------------------------Public Methods--------------------*/
        //To load Ajax content while clicking list item
        /**
        * To load the ajax content while selecting the item.
        * @method selectItem
        * @return jQuery
        * @example 
        * &lt;div id="lb" data-role="ejlistview"&gt;
        * 	&lt;ul&gt;
        *		&lt;li data-ej-text="Man of Steel" data-ej-primarykey="1" data-ej-href="load1.html" data-ej-enableAjax="true"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="World War Z" data-ej-primarykey="2" data-ej-href="load2.html" data-ej-enableAjax="true"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="Monsters University" data-ej-primarykey="3" data-ej-href="load3.html" data-ej-enableAjax="true"&gt;&lt;/li&gt;
        *	&lt;/ul&gt;
        * &lt;/div&gt;
        * &lt;script&gt;
        * // Call enableAjax method.
        * $(document).ready(function(){
        * $("#lb").ejListView("loadAjaxContent","load1.html");
        * });
        * &lt;/script&gt;
        * @memberof ejListView
        * @instance
        */
        //Mobile
        /**
        * To load the ajax content while selecting the item.
        * @method selectItem
        * @return jQuery
        * @example 
        * &lt;div id="lb" data-role="ejmlistview"&gt;
        * 	&lt;ul&gt;
        *		&lt;li data-ej-text="Man of Steel" data-ej-primarykey="1" data-ej-href="load1.html" data-ej-enableAjax="true"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="World War Z" data-ej-primarykey="2" data-ej-href="load2.html" data-ej-enableAjax="true"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="Monsters University" data-ej-primarykey="3" data-ej-href="load3.html" data-ej-enableAjax="true"&gt;&lt;/li&gt;
        *	&lt;/ul&gt;
        * &lt;/div&gt;
        * &lt;script&gt;
        * // Call enableAjax method.
        * $(document).ready(function(){
        * $("#lb").ejmListView("loadAjaxContent","load1.html");
        * });
        * &lt;/script&gt;
        * @memberof ejmListView
        * @instance
        */
        loadAjaxContent: function (urlVal, backButton) {
            var proxy = this;
            this._isFromAjax = true;
            this._renderChild(ej.getAttrVal(this._currentItem, 'data-childid', "page_" + parseInt(Math.random().toFixed(3) * 1000)));
            if (!$.support.pushstate || ej.isWindowsWebView())
                urlVal = App.route.makeUrlAbsolute(urlVal);
            var loadData = { content: proxy._div, item: proxy._currentItem, index: $(proxy._currentItem).index(), text: $(proxy._currentItem).text(), url: proxy.model.ajaxSettings.url ? proxy.model.ajaxSettings.url : urlVal };
            if (this.model.ajaxBeforeLoad)
                this._trigger("ajaxBeforeLoad", loadData);
            var ajaxSettings = {
                cache: proxy.model.ajaxSettings.cache,
                async: proxy.model.ajaxSettings.async,
                type: proxy.model.ajaxSettings.type,
                contentType: proxy.model.ajaxSettings.contentType,
                url: ej.isWindowsWebView() ? urlVal : proxy.model.ajaxSettings.url ? proxy.model.ajaxSettings.url : urlVal,
                dataType: proxy.model.ajaxSettings.dataType,
                data: proxy.model.ajaxSettings.data,
                "successHandler": function (data) { //Ajax post success event handler
                    var content = ej.buildTag("div." + this._prefixClass + "content", (/<\/?body[^>]*>/gmi).test(data) ? data.split(/<\/?body[^>]*>/gmi)[1] : data || "");
                    proxy._div.append(proxy._container.append(content));
                    proxy._div.attr("data-ajaxurl", App.route.convertToRelativeUrl(urlVal));
                    proxy._updateContent(proxy._currentItem, backButton);
                    var successData = { content: proxy._div, item: proxy._currentItem, index: $(proxy._currentItem).index(), text: $(proxy._currentItem).text(), url: proxy.model.ajaxSettings.url ? proxy.model.ajaxSettings.url : urlVal };
                    if (proxy.model.ajaxSuccess)
                        proxy._trigger("ajaxSuccess", successData);
                },
                "errorHandler": function (xhr, textStatus, errorThrown) {
                    var errorData = { "xhr": xhr, "textStatus": textStatus, "errorThrown": errorThrown, item: proxy._currentItem, index: $(proxy._currentItem).index(), text: $(proxy._currentItem).text(), url: proxy.model.ajaxSettings.url ? proxy.model.ajaxSettings.url : urlVal };
                    if (proxy.model.ajaxError)
                        proxy._trigger("ajaxError", errorData);
                },
                "completeHandler": function (data) {
                    var completeData = { content: proxy._div, item: proxy._currentItem, index: $(proxy._currentItem).index(), text: $(proxy._currentItem).text(), url: proxy.model.ajaxSettings.url ? proxy.model.ajaxSettings.url : urlVal };
                    if (proxy.model.ajaxComplete)
                        proxy._trigger("ajaxComplete", completeData);
                }
            };
            ej.sendAjaxRequest(ajaxSettings);
        },
        //To select the list item
        /**
        * To select item in the given index.
        * @method selectItem
        * @return jQuery
        * @example 
        * &lt;div id="lb" data-role="ejlistview"&gt;
        * 	&lt;ul&gt;
        *		&lt;li data-ej-text="Artwork"&gt;&lt;/li&gt;
        * 		&lt;li data-ej-text="Abstract"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="2 Acrylic Mediums"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="Creative Acrylic"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="Modern Painting"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="Canvas Art"&gt;&lt;/li&gt;
        * 		&lt;li data-ej-text="Black white"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="Children"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="Preschool Crafts"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="School-age Crafts"&gt;&lt;/li&gt;
        *	&lt;/ul&gt;
        * &lt;/div&gt;
        * &lt;script&gt;
        * // Call selectItem method.
        * $(document).ready(function(){
        * $("#lb").ejListView("selectItem",2);
        * });
        * &lt;/script&gt;
        * @memberof ejListView
        * @instance
        */
        //Mobile
        /**
        * To select item in the given index.
        * @method selectItem
        * @return jQuery
        * @example 
        * &lt;div id="lb" data-role="ejmlistview"&gt;
        * 	&lt;ul&gt;
        *		&lt;li data-ej-text="Artwork"&gt;&lt;/li&gt;
        * 		&lt;li data-ej-text="Abstract"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="2 Acrylic Mediums"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="Creative Acrylic"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="Modern Painting"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="Canvas Art"&gt;&lt;/li&gt;
        * 		&lt;li data-ej-text="Black white"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="Children"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="Preschool Crafts"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="School-age Crafts"&gt;&lt;/li&gt;
        *	&lt;/ul&gt;
        * &lt;/div&gt;
        * &lt;script&gt;
        * // Call selectItem method.
        * $(document).ready(function(){
        * $("#lb").ejmListView("selectItem",2);
        * });
        * &lt;/script&gt;
        * @memberof ejmListView
        * @instance
        */
        selectItem: function (index, childId) {
            if (index >= 0 && this._isEnable($(this._getElement(childId).find('li.' + this._prefixClass + 'list')[index]))) {
                this._currentItem = $(this._getElement(childId).find('li.' + this._prefixClass + 'list')[index]);
                this._prevItem = this._currentItem;
                this._onTouchEndHandler();
            }
        },
        //To make the List item to be active
        /**
        * To make the item in the given index to be active state.
        * @method setActive
        * @return jQuery
        * @example 
        * &lt;div id="lb" data-role="ejlistview"&gt;
        * 	&lt;ul&gt;
        *		&lt;li data-ej-text="Artwork"&gt;&lt;/li&gt;
        * 		&lt;li data-ej-text="Abstract"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="2 Acrylic Mediums"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="Creative Acrylic"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="Modern Painting"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="Canvas Art"&gt;&lt;/li&gt;
        * 		&lt;li data-ej-text="Black white"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="Children"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="Preschool Crafts"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="School-age Crafts"&gt;&lt;/li&gt;
        *	&lt;/ul&gt;
        * &lt;/div&gt;
        * &lt;script&gt;
        * // Call setActive method.
        * $(document).ready(function(){
        * $("#lb").ejListView("setActive",2);
        * });
        * &lt;/script&gt;
        * @memberof ejListView
        * @instance
        */
        //Mobile
        /**
        * To make the item in the given index to be active state.
        * @method setActive
        * @return jQuery
        * @example 
        * &lt;div id="lb" data-role="ejmlistview"&gt;
        * 	&lt;ul&gt;
        *		&lt;li data-ej-text="Artwork"&gt;&lt;/li&gt;
        * 		&lt;li data-ej-text="Abstract"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="2 Acrylic Mediums"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="Creative Acrylic"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="Modern Painting"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="Canvas Art"&gt;&lt;/li&gt;
        * 		&lt;li data-ej-text="Black white"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="Children"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="Preschool Crafts"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="School-age Crafts"&gt;&lt;/li&gt;
        *	&lt;/ul&gt;
        * &lt;/div&gt;
        * &lt;script&gt;
        * // Call setActive method.
        * $(document).ready(function(){
        * $("#lb").ejmListView("setActive",2);
        * });
        * &lt;/script&gt;
        * @memberof ejmListView
        * @instance
        */
        setActive: function (index, childId) {
            if (index >= 0) {
                element = this._getElement(childId);
                if (this._isEnable($(element.find('li.' + this._prefixClass + 'list')[index])) && ej.getBooleanVal($(element.find('li.' + this._prefixClass + 'list')[index]), 'data-persistSelection', this.model.persistSelection)) {
                    element.find('li.' + this._prefixClass + 'list.' + this._prefixClass + 'state-active').removeClass(this._prefixClass + 'state-active').addClass(this._prefixClass + 'state-default');
                    this._currentItem = $(element.find('li.' + this._prefixClass + 'list')[index]);
                    this._prevItem = this._currentItem;
                    this._currentItem.removeClass(this._prefixClass + 'state-default').addClass(this._prefixClass + 'state-active');
                }
            }
        },
        //To un set the List item to be active
        /**
        * To make the item in the given index to be default state.
        * @method deActive
        * @return jQuery
        * @example 
        * &lt;div id="lb" data-role="ejlistview"&gt;
        * 	&lt;ul&gt;
        *		&lt;li data-ej-text="Artwork"&gt;&lt;/li&gt;
        * 		&lt;li data-ej-text="Abstract"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="2 Acrylic Mediums"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="Creative Acrylic"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="Modern Painting"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="Canvas Art"&gt;&lt;/li&gt;
        * 		&lt;li data-ej-text="Black white"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="Children"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="Preschool Crafts"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="School-age Crafts"&gt;&lt;/li&gt;
        *	&lt;/ul&gt;
        * &lt;/div&gt;
        * &lt;script&gt;
        * // Call deActive method.
        * $(document).ready(function(){
        * $("#lb").ejListView("deActive",2);
        * });
        * &lt;/script&gt;
        * @memberof ejListView
        * @instance
        */
        //Mobile
        /**
        * To make the item in the given index to be default state.
        * @method deActive
        * @return jQuery
        * @example 
        * &lt;div id="lb" data-role="ejmlistview"&gt;
        * 	&lt;ul&gt;
        *		&lt;li data-ej-text="Artwork"&gt;&lt;/li&gt;
        * 		&lt;li data-ej-text="Abstract"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="2 Acrylic Mediums"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="Creative Acrylic"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="Modern Painting"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="Canvas Art"&gt;&lt;/li&gt;
        * 		&lt;li data-ej-text="Black white"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="Children"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="Preschool Crafts"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="School-age Crafts"&gt;&lt;/li&gt;
        *	&lt;/ul&gt;
        * &lt;/div&gt;
        * &lt;script&gt;
        * // Call deActive method.
        * $(document).ready(function(){
        * $("#lb").ejmListView("deActive",2);
        * });
        * &lt;/script&gt;
        * @memberof ejmListView
        * @instance
        */
        deActive: function (index, childId) {
            if (index >= 0 && this._isEnable($(this._getElement(childId).find('li.' + this._prefixClass + 'list')[index])))
                $(this._getElement(childId).find('li.' + this._prefixClass + 'list')[index]).removeClass(this._prefixClass + 'state-active').addClass(this._prefixClass + 'state-default');
        },
        //To enable the List item
        /**
        * To enable item in the given index.
        * @method enableItem
        * @return jQuery
        * @example 
        * &lt;div id="lb" data-role="ejlistview"&gt;
        * 	&lt;ul&gt;
        *		&lt;li data-ej-text="Artwork"&gt;&lt;/li&gt;
        * 		&lt;li data-ej-text="Abstract"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="2 Acrylic Mediums"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="Creative Acrylic"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="Modern Painting"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="Canvas Art"&gt;&lt;/li&gt;
        * 		&lt;li data-ej-text="Black white"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="Children"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="Preschool Crafts"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="School-age Crafts"&gt;&lt;/li&gt;
        *	&lt;/ul&gt;
        * &lt;/div&gt;
        * &lt;script&gt;
        * // Call enableItem method.
        * $(document).ready(function(){
        * $("#lb").ejListView("enableItem",2);
        * });
        * &lt;/script&gt;
        * @memberof ejListView
        * @instance
        */
        //Mobile
        /**
        * To enable item in the given index.
        * @method enableItem
        * @return jQuery
        * @example 
        * &lt;div id="lb" data-role="ejmlistview"&gt;
        * 	&lt;ul&gt;
        *		&lt;li data-ej-text="Artwork"&gt;&lt;/li&gt;
        * 		&lt;li data-ej-text="Abstract"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="2 Acrylic Mediums"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="Creative Acrylic"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="Modern Painting"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="Canvas Art"&gt;&lt;/li&gt;
        * 		&lt;li data-ej-text="Black white"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="Children"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="Preschool Crafts"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="School-age Crafts"&gt;&lt;/li&gt;
        *	&lt;/ul&gt;
        * &lt;/div&gt;
        * &lt;script&gt;
        * // Call enableItem method.
        * $(document).ready(function(){
        * $("#lb").ejmListView("enableItem",2);
        * });
        * &lt;/script&gt;
        * @memberof ejmListView
        * @instance
        */
        enableItem: function (index, childId) {
            if (index >= 0)
                $(this._getElement(childId).find('li.' + this._prefixClass + 'list')[index]).removeClass(this._prefixClass + 'state-disabled');
        },
        //To disable the List item
        /**
        * To disable item in the given index.
        * @method disableItem
        * @return jQuery
        * @example 
        * &lt;div id="lb" data-role="ejlistview"&gt;
        * 	&lt;ul&gt;
        *		&lt;li data-ej-text="Artwork"&gt;&lt;/li&gt;
        * 		&lt;li data-ej-text="Abstract"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="2 Acrylic Mediums"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="Creative Acrylic"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="Modern Painting"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="Canvas Art"&gt;&lt;/li&gt;
        * 		&lt;li data-ej-text="Black white"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="Children"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="Preschool Crafts"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="School-age Crafts"&gt;&lt;/li&gt;
        *	&lt;/ul&gt;
        * &lt;/div&gt;
        * &lt;script&gt;
        * // Call disableItem method.
        * $(document).ready(function(){
        * $("#lb").ejListView("disableItem",2);
        * });
        * &lt;/script&gt;
        * @memberof ejListView
        * @instance
        */
        //Mobile
        /**
        * To disable item in the given index.
        * @method disableItem
        * @return jQuery
        * @example 
        * &lt;div id="lb" data-role="ejmlistview"&gt;
        * 	&lt;ul&gt;
        *		&lt;li data-ej-text="Artwork"&gt;&lt;/li&gt;
        * 		&lt;li data-ej-text="Abstract"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="2 Acrylic Mediums"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="Creative Acrylic"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="Modern Painting"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="Canvas Art"&gt;&lt;/li&gt;
        * 		&lt;li data-ej-text="Black white"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="Children"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="Preschool Crafts"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="School-age Crafts"&gt;&lt;/li&gt;
        *	&lt;/ul&gt;
        * &lt;/div&gt;
        * &lt;script&gt;
        * // Call disableItem method.
        * $(document).ready(function(){
        * $("#lb").ejmListView("disableItem",2);
        * });
        * &lt;/script&gt;
        * @memberof ejmListView
        * @instance
        */
        disableItem: function (index, childId) {
            if (index >= 0)
                $(this._getElement(childId).find('li.' + this._prefixClass + 'list')[index]).addClass(this._prefixClass + 'state-disabled');
        },
        //To remove check mark to the list
        /**
        * To remove the check mark either for specific item in the given index or for all items.
        * @method removeCheckMark
        * @return jQuery
        * @example 
        * &lt;div id="lb" data-role="ejmlistview" data-ej-enableCheckMark="true"&gt;
        * 	&lt;ul&gt;
        *		&lt;li data-ej-text="Artwork"&gt;&lt;/li&gt;
        * 		&lt;li data-ej-text="Abstract"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="2 Acrylic Mediums"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="Creative Acrylic"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="Modern Painting"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="Canvas Art"&gt;&lt;/li&gt;
        * 		&lt;li data-ej-text="Black white"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="Children"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="Preschool Crafts"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="School-age Crafts"&gt;&lt;/li&gt;
        *	&lt;/ul&gt;
        * &lt;/div&gt;
        * &lt;script&gt;
        * // Call removeCheckMark method.
        * $(document).ready(function(){
        * $("#lb").ejmListView("removeCheckMark",2);
        * });
        * &lt;/script&gt;
        * @memberof ejmListView
        * @instance
        */
        //Mobile
        //To remove check mark to the list
        /**
        * To remove the check mark either for specific item in the given index or for all items.
        * @method removeCheckMark
        * @return jQuery
        * @example 
        * &lt;div id="lb" data-role="ejmlistview" data-ej-enableCheckMark="true"&gt;
        * 	&lt;ul&gt;
        *		&lt;li data-ej-text="Artwork"&gt;&lt;/li&gt;
        * 		&lt;li data-ej-text="Abstract"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="2 Acrylic Mediums"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="Creative Acrylic"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="Modern Painting"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="Canvas Art"&gt;&lt;/li&gt;
        * 		&lt;li data-ej-text="Black white"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="Children"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="Preschool Crafts"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="School-age Crafts"&gt;&lt;/li&gt;
        *	&lt;/ul&gt;
        * &lt;/div&gt;
        * &lt;script&gt;
        * // Call removeCheckMark method.
        * $(document).ready(function(){
        * $("#lb").ejmListView("removeCheckMark",2);
        * });
        * &lt;/script&gt;
        * @memberof ejmListView
        * @instance
        */
        removeCheckMark: function (index, childId) {
            element = this._getElement(childId);
            if (index >= 0 && this._isEnable($(element.find('li.' + this._prefixClass + 'list')[index])))
                $(element.find('li.' + this._prefixClass + 'list')[index]).find('.' + this._prefixClass + 'lv-checkdiv').remove();
            else
                element.find('.' + this._prefixClass + 'lv-checkdiv').remove();
        },
        //To check selected item in the list
        /**
        * To check item in the given index.
        * @method checkItem
        * @return jQuery
        * @example 
        * &lt;div id="lb" data-role="ejlistview" data-ej-enableCheckMark="true"&gt;
        * 	&lt;ul&gt;
        *		&lt;li data-ej-text="Artwork"&gt;&lt;/li&gt;
        * 		&lt;li data-ej-text="Abstract"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="2 Acrylic Mediums"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="Creative Acrylic"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="Modern Painting"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="Canvas Art"&gt;&lt;/li&gt;
        * 		&lt;li data-ej-text="Black white"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="Children"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="Preschool Crafts"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="School-age Crafts"&gt;&lt;/li&gt;
        *	&lt;/ul&gt;
        * &lt;/div&gt;
        * &lt;script&gt;
        * // Call checkItem method.
        * $(document).ready(function(){
        * $("#lb").ejListView("checkItem",2);
        * });
        * &lt;/script&gt;
        * @memberof ejListView
        * @instance
        */
        //Mobile
        /**
        * To check item in the given index.
        * @method checkItem
        * @return jQuery
        * @example 
        * &lt;div id="lb" data-role="ejmlistview" data-ej-enableCheckMark="true"&gt;
        * 	&lt;ul&gt;
        *		&lt;li data-ej-text="Artwork"&gt;&lt;/li&gt;
        * 		&lt;li data-ej-text="Abstract"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="2 Acrylic Mediums"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="Creative Acrylic"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="Modern Painting"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="Canvas Art"&gt;&lt;/li&gt;
        * 		&lt;li data-ej-text="Black white"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="Children"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="Preschool Crafts"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="School-age Crafts"&gt;&lt;/li&gt;
        *	&lt;/ul&gt;
        * &lt;/div&gt;
        * &lt;script&gt;
        * // Call checkItem method.
        * $(document).ready(function(){
        * $("#lb").ejmListView("checkItem",2);
        * });
        * &lt;/script&gt;
        * @memberof ejmListView
        * @instance
        */
        checkItem: function (index, childId) {
            if (index >= 0 && this._isEnable($(this._getElement(childId).find('li.' + this._prefixClass + 'list')[index])))
                this._setCheckboxValue($(this._getElement(childId).find('.' + this._prefixClass + 'lv-check')[index]), true);
        },
        //To uncheck selected item in the list
        /**
        * To uncheck item in the given index.
        * @method unCheckItem
        * @return jQuery
        * @example 
        * &lt;div id="lb" data-role="ejlistview"&gt;
        * 	&lt;ul&gt;
        *		&lt;li data-ej-text="Artwork"&gt;&lt;/li&gt;
        * 		&lt;li data-ej-text="Abstract"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="2 Acrylic Mediums"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="Creative Acrylic"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="Modern Painting"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="Canvas Art"&gt;&lt;/li&gt;
        * 		&lt;li data-ej-text="Black white"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="Children"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="Preschool Crafts"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="School-age Crafts"&gt;&lt;/li&gt;
        *	&lt;/ul&gt;
        * &lt;/div&gt;
        * &lt;script&gt;
        * // Call unCheckItem method.
        * $(document).ready(function(){
        * $("#lb").ejListView("unCheckItem",2);
        * });
        * &lt;/script&gt;
        * @memberof ejListView
        * @instance
        */
        //Mobile
        /**
        * To uncheck item in the given index.
        * @method unCheckItem
        * @return jQuery
        * @example 
        * &lt;div id="lb" data-role="ejmlistview"&gt;
        * 	&lt;ul&gt;
        *		&lt;li data-ej-text="Artwork"&gt;&lt;/li&gt;
        * 		&lt;li data-ej-text="Abstract"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="2 Acrylic Mediums"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="Creative Acrylic"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="Modern Painting"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="Canvas Art"&gt;&lt;/li&gt;
        * 		&lt;li data-ej-text="Black white"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="Children"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="Preschool Crafts"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="School-age Crafts"&gt;&lt;/li&gt;
        *	&lt;/ul&gt;
        * &lt;/div&gt;
        * &lt;script&gt;
        * // Call unCheckItem method.
        * $(document).ready(function(){
        * $("#lb").ejmListView("unCheckItem",2);
        * });
        * &lt;/script&gt;
        * @memberof ejmListView
        * @instance
        */
        unCheckItem: function (index, childId) {
            if (index >= 0 && this._isEnable($(this._getElement(childId).find('li.' + this._prefixClass + 'list')[index])))
                this._setCheckboxValue($(this._getElement(childId).find('.' + this._prefixClass + 'lv-check')[index]), false);
        },
        //To check all the items in the list
        /**
        * To check all the items.
        * @method checkAllItem
        * @return jQuery
        * @example 
        * &lt;div id="lb" data-role="ejlistview" data-ej-enableCheckMark="true"&gt;
        * 	&lt;ul&gt;
        *		&lt;li data-ej-text="Artwork"&gt;&lt;/li&gt;
        * 		&lt;li data-ej-text="Abstract"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="2 Acrylic Mediums"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="Creative Acrylic"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="Modern Painting"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="Canvas Art"&gt;&lt;/li&gt;
        * 		&lt;li data-ej-text="Black white"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="Children"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="Preschool Crafts"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="School-age Crafts"&gt;&lt;/li&gt;
        *	&lt;/ul&gt;
        * &lt;/div&gt;
        * &lt;script&gt;
        * // Call checkAllItem method.
        * $(document).ready(function(){
        * $("#lb").ejListView("checkAllItem");
        * });
        * &lt;/script&gt;
        * @memberof ejListView
        * @instance
        */
        //Mobile
        /**
        * To check all the items.
        * @method checkAllItem
        * @return jQuery
        * @example 
        * &lt;div id="lb" data-role="ejmlistview" data-ej-enableCheckMark="true"&gt;
        * 	&lt;ul&gt;
        *		&lt;li data-ej-text="Artwork"&gt;&lt;/li&gt;
        * 		&lt;li data-ej-text="Abstract"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="2 Acrylic Mediums"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="Creative Acrylic"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="Modern Painting"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="Canvas Art"&gt;&lt;/li&gt;
        * 		&lt;li data-ej-text="Black white"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="Children"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="Preschool Crafts"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="School-age Crafts"&gt;&lt;/li&gt;
        *	&lt;/ul&gt;
        * &lt;/div&gt;
        * &lt;script&gt;
        * // Call checkAllItem method.
        * $(document).ready(function(){
        * $("#lb").ejmListView("checkAllItem");
        * });
        * &lt;/script&gt;
        * @memberof ejmListView
        * @instance
        */
        checkAllItem: function (childId) {
            var proxy = this;
            this._getElement(childId).find('.' + this._prefixClass + 'lv-check').each(function (index, check) {
                if (proxy._isEnable($(proxy._getElement(childId).find('li.' + proxy._prefixClass + 'list')[index])))
                    proxy._setCheckboxValue($(check), true);
            });
        },
        //To uncheck all the items in the list
        /**
        * To uncheck all the items.
        * @method unCheckAllItem
        * @return jQuery
        * @example 
        * &lt;div id="lb" data-role="ejlistview" data-ej-enableCheckMark="true"&gt;
        * 	&lt;ul&gt;
        *		&lt;li data-ej-text="Artwork"&gt;&lt;/li&gt;
        * 		&lt;li data-ej-text="Abstract"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="2 Acrylic Mediums"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="Creative Acrylic"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="Modern Painting"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="Canvas Art"&gt;&lt;/li&gt;
        * 		&lt;li data-ej-text="Black white"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="Children"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="Preschool Crafts"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="School-age Crafts"&gt;&lt;/li&gt;
        *	&lt;/ul&gt;
        * &lt;/div&gt;
        * &lt;script&gt;
        * // Call unCheckAllItem method.
        * $(document).ready(function(){
        * $("#lb").ejListView("unCheckAllItem");
        * });
        * &lt;/script&gt;
        * @memberof ejListView
        * @instance
        */
        //Mobile
        /**
        * To uncheck all the items.
        * @method unCheckAllItem
        * @return jQuery
        * @example 
        * &lt;div id="lb" data-role="ejmlistview" data-ej-enableCheckMark="true"&gt;
        * 	&lt;ul&gt;
        *		&lt;li data-ej-text="Artwork"&gt;&lt;/li&gt;
        * 		&lt;li data-ej-text="Abstract"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="2 Acrylic Mediums"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="Creative Acrylic"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="Modern Painting"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="Canvas Art"&gt;&lt;/li&gt;
        * 		&lt;li data-ej-text="Black white"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="Children"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="Preschool Crafts"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="School-age Crafts"&gt;&lt;/li&gt;
        *	&lt;/ul&gt;
        * &lt;/div&gt;
        * &lt;script&gt;
        * // Call unCheckAllItem method.
        * $(document).ready(function(){
        * $("#lb").ejmListView("unCheckAllItem");
        * });
        * &lt;/script&gt;
        * @memberof ejmListView
        * @instance
        */
        unCheckAllItem: function (childId) {
            var proxy = this;
            this._getElement(childId).find('.' + this._prefixClass + 'lv-check').each(function (index, check) {
                if (proxy._isEnable($(proxy._getElement(childId).find('li.' + proxy._prefixClass + 'list')[index])))
                    proxy._setCheckboxValue($(check), false);
            });
        },
        //To get active list item
        /**
        * To get the active item.
        * @method getActiveItem
        * @return jQuery
        * @example 
        * &lt;div id="lb" data-role="ejlistview"&gt;
        * 	&lt;ul&gt;
        *		&lt;li data-ej-text="Artwork"&gt;&lt;/li&gt;
        * 		&lt;li data-ej-text="Abstract"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="2 Acrylic Mediums"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="Creative Acrylic"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="Modern Painting"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="Canvas Art"&gt;&lt;/li&gt;
        * 		&lt;li data-ej-text="Black white"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="Children"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="Preschool Crafts"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="School-age Crafts"&gt;&lt;/li&gt;
        *	&lt;/ul&gt;
        * &lt;/div&gt;
        * &lt;script&gt;
        * // Call getActiveItem method.
        * $(document).ready(function(){
        * $("#lb").ejListView("getActiveItem");
        * });
        * &lt;/script&gt;
        * @memberof ejListView
        * @instance
        */
        //Mobile
        /**
        * To get the active item.
        * @method getActiveItem
        * @return jQuery
        * @example 
        * &lt;div id="lb" data-role="ejmlistview"&gt;
        * 	&lt;ul&gt;
        *		&lt;li data-ej-text="Artwork"&gt;&lt;/li&gt;
        * 		&lt;li data-ej-text="Abstract"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="2 Acrylic Mediums"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="Creative Acrylic"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="Modern Painting"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="Canvas Art"&gt;&lt;/li&gt;
        * 		&lt;li data-ej-text="Black white"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="Children"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="Preschool Crafts"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="School-age Crafts"&gt;&lt;/li&gt;
        *	&lt;/ul&gt;
        * &lt;/div&gt;
        * &lt;script&gt;
        * // Call getActiveItem method.
        * $(document).ready(function(){
        * $("#lb").ejmListView("getActiveItem");
        * });
        * &lt;/script&gt;
        * @memberof ejmListView
        * @instance
        */
        getActiveItem: function (childId) {
            return this._getElement(childId).find('li.' + this._prefixClass + 'list.' + this._prefixClass + 'state-active');
        },
        /**
        * To get the text of the active item.
        * @method getActiveItemText
        * @return jQuery
        * @example 
        * &lt;div id="lb" data-role="ejlistview"&gt;
        * 	&lt;ul&gt;
        *		&lt;li data-ej-text="Artwork"&gt;&lt;/li&gt;
        * 		&lt;li data-ej-text="Abstract"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="2 Acrylic Mediums"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="Creative Acrylic"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="Modern Painting"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="Canvas Art"&gt;&lt;/li&gt;
        * 		&lt;li data-ej-text="Black white"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="Children"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="Preschool Crafts"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="School-age Crafts"&gt;&lt;/li&gt;
        *	&lt;/ul&gt;
        * &lt;/div&gt;
        * &lt;script&gt;
        * // Call getActiveItemText method.
        * $(document).ready(function(){
        * $("#lb").ejListView("getActiveItemText");
        * });
        * &lt;/script&gt;
        * @memberof ejListView
        * @instance
        */
        //Mobile
        /**
        * To get the text of the active item.
        * @method getActiveItemText
        * @return jQuery
        * @example 
        * &lt;div id="lb" data-role="ejmlistview"&gt;
        * 	&lt;ul&gt;
        *		&lt;li data-ej-text="Artwork"&gt;&lt;/li&gt;
        * 		&lt;li data-ej-text="Abstract"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="2 Acrylic Mediums"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="Creative Acrylic"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="Modern Painting"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="Canvas Art"&gt;&lt;/li&gt;
        * 		&lt;li data-ej-text="Black white"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="Children"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="Preschool Crafts"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="School-age Crafts"&gt;&lt;/li&gt;
        *	&lt;/ul&gt;
        * &lt;/div&gt;
        * &lt;script&gt;
        * // Call getActiveItemText method.
        * $(document).ready(function(){
        * $("#lb").ejmListView("getActiveItemText");
        * });
        * &lt;/script&gt;
        * @memberof ejmListView
        * @instance
        */
        getActiveItemText: function (childId) {
            return this._getElement(childId).find('li.' + this._prefixClass + 'list.' + this._prefixClass + 'state-active').text();
        },
        //To get text of the list item
        /**
        * To get the text of the item in the given index.
        * @method getItemText
        * @return jQuery
        * @example 
        * &lt;div id="lb" data-role="ejlistview"&gt;
        * 	&lt;ul&gt;
        *		&lt;li data-ej-text="Artwork"&gt;&lt;/li&gt;
        * 		&lt;li data-ej-text="Abstract"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="2 Acrylic Mediums"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="Creative Acrylic"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="Modern Painting"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="Canvas Art"&gt;&lt;/li&gt;
        * 		&lt;li data-ej-text="Black white"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="Children"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="Preschool Crafts"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="School-age Crafts"&gt;&lt;/li&gt;
        *	&lt;/ul&gt;
        * &lt;/div&gt;
        * &lt;script&gt;
        * // Call getItemText method.
        * $(document).ready(function(){
        * $("#lb").ejListView("getItemText",2);
        * });
        * &lt;/script&gt;
        * @memberof ejListView
        * @instance
        */
        //Mobile
        /**
        * To get the text of the item in the given index.
        * @method getItemText
        * @return jQuery
        * @example 
        * &lt;div id="lb" data-role="ejmlistview"&gt;
        * 	&lt;ul&gt;
        *		&lt;li data-ej-text="Artwork"&gt;&lt;/li&gt;
        * 		&lt;li data-ej-text="Abstract"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="2 Acrylic Mediums"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="Creative Acrylic"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="Modern Painting"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="Canvas Art"&gt;&lt;/li&gt;
        * 		&lt;li data-ej-text="Black white"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="Children"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="Preschool Crafts"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="School-age Crafts"&gt;&lt;/li&gt;
        *	&lt;/ul&gt;
        * &lt;/div&gt;
        * &lt;script&gt;
        * // Call getItemText method.
        * $(document).ready(function(){
        * $("#lb").ejmListView("getItemText",2);
        * });
        * &lt;/script&gt;
        * @memberof ejmListView
        * @instance
        */
        getItemText: function (index, childId) {
            if (index >= 0)
                return $(this._getElement(childId).find('li.' + this._prefixClass + 'list')[index]).text();
        },
        //To get all checked list items
        /**
        * To get all the checked items.
        * @method getCheckedItems
        * @return jQuery
        * @example 
        * &lt;div id="lb" data-role="ejlistview" data-ej-enableCheckMark="true"&gt;
        * 	&lt;ul&gt;
        *		&lt;li data-ej-text="Artwork"&gt;&lt;/li&gt;
        * 		&lt;li data-ej-text="Abstract"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="2 Acrylic Mediums"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="Creative Acrylic"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="Modern Painting"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="Canvas Art"&gt;&lt;/li&gt;
        * 		&lt;li data-ej-text="Black white"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="Children"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="Preschool Crafts"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="School-age Crafts"&gt;&lt;/li&gt;
        *	&lt;/ul&gt;
        * &lt;/div&gt;
        * &lt;script&gt;
        * // Call getCheckedItems method.
        * $(document).ready(function(){
        * $("#lb").ejListView("getCheckedItems");
        * });
        * &lt;/script&gt;
        * @memberof ejListView
        * @instance
        */
        /**
        * To get all the checked items.
        * @method getCheckedItems
        * @return jQuery
        * @example 
        * &lt;div id="lb" data-role="ejmlistview" data-ej-enableCheckMark="true"&gt;
        * 	&lt;ul&gt;
        *		&lt;li data-ej-text="Artwork"&gt;&lt;/li&gt;
        * 		&lt;li data-ej-text="Abstract"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="2 Acrylic Mediums"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="Creative Acrylic"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="Modern Painting"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="Canvas Art"&gt;&lt;/li&gt;
        * 		&lt;li data-ej-text="Black white"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="Children"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="Preschool Crafts"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="School-age Crafts"&gt;&lt;/li&gt;
        *	&lt;/ul&gt;
        * &lt;/div&gt;
        * &lt;script&gt;
        * // Call getCheckedItems method.
        * $(document).ready(function(){
        * $("#lb").ejmListView("getCheckedItems");
        * });
        * &lt;/script&gt;
        * @memberof ejmListView
        * @instance
        */
        getCheckedItems: function (childId) {
            return this._getElement(childId).find('input.' + this._prefixClass + 'lv-check[checked="checked"]').closest('li.' + this._prefixClass + 'list');
        },
        //To get all checked list item's text
        /**
        * To get the text of all the checked items.
        * @method getCheckedItemsText
        * @return jQuery
        * @example 
        * &lt;div id="lb" data-role="ejlistview" data-ej-enableCheckMark="true"&gt;
        * 	&lt;ul&gt;
        *		&lt;li data-ej-text="Artwork"&gt;&lt;/li&gt;
        * 		&lt;li data-ej-text="Abstract"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="2 Acrylic Mediums"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="Creative Acrylic"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="Modern Painting"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="Canvas Art"&gt;&lt;/li&gt;
        * 		&lt;li data-ej-text="Black white"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="Children"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="Preschool Crafts"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="School-age Crafts"&gt;&lt;/li&gt;
        *	&lt;/ul&gt;
        * &lt;/div&gt;
        * &lt;script&gt;
        * // Call getCheckedItemsText method.
        * $(document).ready(function(){
        * $("#lb").ejListView("getCheckedItemsText");
        * });
        * &lt;/script&gt;
        * @memberof ejListView
        * @instance
        */
        //Mobile
        /**
        * To get the text of all the checked items.
        * @method getCheckedItemsText
        * @return jQuery
        * @example 
        * &lt;div id="lb" data-role="ejmlistview" data-ej-enableCheckMark="true"&gt;
        * 	&lt;ul&gt;
        *		&lt;li data-ej-text="Artwork"&gt;&lt;/li&gt;
        * 		&lt;li data-ej-text="Abstract"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="2 Acrylic Mediums"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="Creative Acrylic"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="Modern Painting"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="Canvas Art"&gt;&lt;/li&gt;
        * 		&lt;li data-ej-text="Black white"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="Children"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="Preschool Crafts"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="School-age Crafts"&gt;&lt;/li&gt;
        *	&lt;/ul&gt;
        * &lt;/div&gt;
        * &lt;script&gt;
        * // Call getCheckedItemsText method.
        * $(document).ready(function(){
        * $("#lb").ejmListView("getCheckedItemsText");
        * });
        * &lt;/script&gt;
        * @memberof ejmListView
        * @instance
        */
        getCheckedItemsText: function (childId) {
            return $(this._getElement(childId).find('input.' + this._prefixClass + 'lv-check[checked="checked"]').closest('li.' + this._prefixClass + 'list')).map(function () { return $(this).text(); }).get();
        },
        //To check whether the list item has child or not
        /**
        * To check whether the item in the given index has child item.
        * @method hasChild
        * @return jQuery
        * @example 
        * &lt;div id="lb" data-role="ejlistview"&gt;
        * 	&lt;ul&gt;
        *		&lt;li data-ej-text="Artwork"&gt;&lt;/li&gt;
        * 		&lt;li data-ej-text="Abstract"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="2 Acrylic Mediums"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="Creative Acrylic"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="Modern Painting"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="Canvas Art"&gt;&lt;/li&gt;
        * 		&lt;li data-ej-text="Black white"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="Children"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="Preschool Crafts"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="School-age Crafts"&gt;&lt;/li&gt;
        *	&lt;/ul&gt;
        * &lt;/div&gt;
        * &lt;script&gt;
        * // Call hasChild method.
        * $(document).ready(function(){
        * $("#lb").ejListView("hasChild",2);
        * });
        * &lt;/script&gt;
        * @memberof ejListView
        * @instance
        */
        //Mobile
        /**
        * To check whether the item in the given index has child item.
        * @method hasChild
        * @return jQuery
        * @example 
        * &lt;div id="lb" data-role="ejmlistview"&gt;
        * 	&lt;ul&gt;
        *		&lt;li data-ej-text="Artwork"&gt;&lt;/li&gt;
        * 		&lt;li data-ej-text="Abstract"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="2 Acrylic Mediums"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="Creative Acrylic"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="Modern Painting"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="Canvas Art"&gt;&lt;/li&gt;
        * 		&lt;li data-ej-text="Black white"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="Children"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="Preschool Crafts"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="School-age Crafts"&gt;&lt;/li&gt;
        *	&lt;/ul&gt;
        * &lt;/div&gt;
        * &lt;script&gt;
        * // Call hasChild method.
        * $(document).ready(function(){
        * $("#lb").ejmListView("hasChild",2);
        * });
        * &lt;/script&gt;
        * @memberof ejmListView
        * @instance
        */
        hasChild: function (index, childId) {
            if (index >= 0)
                return this.element.find('#' + $(this._getElement(childId).find('li.' + this._prefixClass + 'list')[index]).attr('data-childid')).length ? true : false;
        },
        //To check whether the list item is checked or not
        /**
        * To check whether item in the given index is checked.
        * @method isChecked
        * @return jQuery
        * @example 
        * &lt;div id="lb" data-role="ejlistview" data-ej-enableCheckMark="true"&gt;
        * 	&lt;ul&gt;
        *		&lt;li data-ej-text="Artwork"&gt;&lt;/li&gt;
        * 		&lt;li data-ej-text="Abstract"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="2 Acrylic Mediums"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="Creative Acrylic"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="Modern Painting"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="Canvas Art"&gt;&lt;/li&gt;
        * 		&lt;li data-ej-text="Black white"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="Children"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="Preschool Crafts"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="School-age Crafts"&gt;&lt;/li&gt;
        *	&lt;/ul&gt;
        * &lt;/div&gt;
        * &lt;script&gt;
        * // Call isChecked method.
        * $(document).ready(function(){
        * $("#lb").ejListView("isChecked",2);
        * });
        * &lt;/script&gt;
        * @memberof ejListView
        * @instance
        */
        //Mobile
        /**
        * To check whether item in the given index is checked.
        * @method isChecked
        * @return jQuery
        * @example 
        * &lt;div id="lb" data-role="ejmlistview" data-ej-enableCheckMark="true"&gt;
        * 	&lt;ul&gt;
        *		&lt;li data-ej-text="Artwork"&gt;&lt;/li&gt;
        * 		&lt;li data-ej-text="Abstract"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="2 Acrylic Mediums"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="Creative Acrylic"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="Modern Painting"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="Canvas Art"&gt;&lt;/li&gt;
        * 		&lt;li data-ej-text="Black white"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="Children"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="Preschool Crafts"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="School-age Crafts"&gt;&lt;/li&gt;
        *	&lt;/ul&gt;
        * &lt;/div&gt;
        * &lt;script&gt;
        * // Call isChecked method.
        * $(document).ready(function(){
        * $("#lb").ejmListView("isChecked",2);
        * });
        * &lt;/script&gt;
        * @memberof ejmListView
        * @instance
        */
        isChecked: function (index, childId) {
            if (index >= 0)
                return $(this._getElement(childId).find('li.' + this._prefixClass + 'list input.' + this._prefixClass + 'lv-check')[index]).attr('checked') == "checked" ? true : false;
        },
        //To show the list item
        /**
        * To show item in the given index.
        * @method showItem
        * @return jQuery
        * @example 
        * &lt;div id="lb" data-role="ejlistview"&gt;
        * 	&lt;ul&gt;
        *		&lt;li data-ej-text="Artwork"&gt;&lt;/li&gt;
        * 		&lt;li data-ej-text="Abstract"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="2 Acrylic Mediums"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="Creative Acrylic"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="Modern Painting"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="Canvas Art"&gt;&lt;/li&gt;
        * 		&lt;li data-ej-text="Black white"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="Children"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="Preschool Crafts"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="School-age Crafts"&gt;&lt;/li&gt;
        *	&lt;/ul&gt;
        * &lt;/div&gt;
        * &lt;script&gt;
        * // Call showItem method.
        * $(document).ready(function(){
        * $("#lb").ejListView("showItem",2);
        * });
        * &lt;/script&gt;
        * @memberof ejListView
        * @instance
        */
        //Mobile
        /**
        * To show item in the given index.
        * @method showItem
        * @return jQuery
        * @example 
        * &lt;div id="lb" data-role="ejmlistview"&gt;
        * 	&lt;ul&gt;
        *		&lt;li data-ej-text="Artwork"&gt;&lt;/li&gt;
        * 		&lt;li data-ej-text="Abstract"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="2 Acrylic Mediums"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="Creative Acrylic"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="Modern Painting"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="Canvas Art"&gt;&lt;/li&gt;
        * 		&lt;li data-ej-text="Black white"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="Children"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="Preschool Crafts"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="School-age Crafts"&gt;&lt;/li&gt;
        *	&lt;/ul&gt;
        * &lt;/div&gt;
        * &lt;script&gt;
        * // Call showItem method.
        * $(document).ready(function(){
        * $("#lb").ejmListView("showItem",2);
        * });
        * &lt;/script&gt;
        * @memberof ejmListView
        * @instance
        */
        showItem: function (index, childId) {
            if (index >= 0)
                $(this._getElement(childId).find('li.' + this._prefixClass + 'list')[index]).css('visibility', '');
        },
        //To hide the list item
        /**
        * To hide item in the given index.
        * @method hideItem
        * @return jQuery
        * @example 
        * &lt;div id="lb" data-role="ejlistview"&gt;
        * 	&lt;ul&gt;
        *		&lt;li data-ej-text="Artwork"&gt;&lt;/li&gt;
        * 		&lt;li data-ej-text="Abstract"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="2 Acrylic Mediums"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="Creative Acrylic"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="Modern Painting"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="Canvas Art"&gt;&lt;/li&gt;
        * 		&lt;li data-ej-text="Black white"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="Children"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="Preschool Crafts"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="School-age Crafts"&gt;&lt;/li&gt;
        *	&lt;/ul&gt;
        * &lt;/div&gt;
        * &lt;script&gt;
        * // Call hideItem method.
        * $(document).ready(function(){
        * $("#lb").ejListView("hideItem",2);
        * });
        * &lt;/script&gt;
        * @memberof ejListView
        * @instance
        */
        //Mobile
        /**
        * To hide item in the given index.
        * @method hideItem
        * @return jQuery
        * @example 
        * &lt;div id="lb" data-role="ejmlistview"&gt;
        * 	&lt;ul&gt;
        *		&lt;li data-ej-text="Artwork"&gt;&lt;/li&gt;
        * 		&lt;li data-ej-text="Abstract"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="2 Acrylic Mediums"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="Creative Acrylic"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="Modern Painting"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="Canvas Art"&gt;&lt;/li&gt;
        * 		&lt;li data-ej-text="Black white"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="Children"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="Preschool Crafts"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="School-age Crafts"&gt;&lt;/li&gt;
        *	&lt;/ul&gt;
        * &lt;/div&gt;
        * &lt;script&gt;
        * // Call hideItem method.
        * $(document).ready(function(){
        * $("#lb").ejmListView("hideItem",2);
        * });
        * &lt;/script&gt;
        * @memberof ejmListView
        * @instance
        */
        hideItem: function (index, childId) {
            if (index >= 0)
                $(this._getElement(childId).find('li.' + this._prefixClass + 'list')[index]).css('visibility', 'hidden');
        },
        //To show the list
        /**
        * To show the list.
        * @method show
        * @return jQuery
        * @example 
        * &lt;div id="lb" data-role="ejlistview"&gt;
        * 	&lt;ul&gt;
        *		&lt;li data-ej-text="Artwork"&gt;&lt;/li&gt;
        * 		&lt;li data-ej-text="Abstract"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="2 Acrylic Mediums"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="Creative Acrylic"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="Modern Painting"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="Canvas Art"&gt;&lt;/li&gt;
        * 		&lt;li data-ej-text="Black white"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="Children"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="Preschool Crafts"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="School-age Crafts"&gt;&lt;/li&gt;
        *	&lt;/ul&gt;
        * &lt;/div&gt;
        * &lt;script&gt;
        * // Call show method.
        * $(document).ready(function(){
        * $("#lb").ejListView("show");
        * });
        * &lt;/script&gt;
        * @memberof ejListView
        * @instance
        */
        //Mobile
        /**
        * To show the list.
        * @method show
        * @return jQuery
        * @example 
        * &lt;div id="lb" data-role="ejmlistview"&gt;
        * 	&lt;ul&gt;
        *		&lt;li data-ej-text="Artwork"&gt;&lt;/li&gt;
        * 		&lt;li data-ej-text="Abstract"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="2 Acrylic Mediums"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="Creative Acrylic"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="Modern Painting"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="Canvas Art"&gt;&lt;/li&gt;
        * 		&lt;li data-ej-text="Black white"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="Children"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="Preschool Crafts"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="School-age Crafts"&gt;&lt;/li&gt;
        *	&lt;/ul&gt;
        * &lt;/div&gt;
        * &lt;script&gt;
        * // Call show method.
        * $(document).ready(function(){
        * $("#lb").ejmListView("show");
        * });
        * &lt;/script&gt;
        * @memberof ejmListView
        * @instance
        */
        show: function (childId) {
            this._getElement(childId).css('visibility', '');
        },
        //To hide the list
        /**
        * To hide the list.
        * @method hide
        * @return jQuery
        * @example 
        * &lt;div id="lb" data-role="ejlistview"&gt;
        * 	&lt;ul&gt;
        *		&lt;li data-ej-text="Artwork"&gt;&lt;/li&gt;
        * 		&lt;li data-ej-text="Abstract"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="2 Acrylic Mediums"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="Creative Acrylic"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="Modern Painting"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="Canvas Art"&gt;&lt;/li&gt;
        * 		&lt;li data-ej-text="Black white"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="Children"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="Preschool Crafts"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="School-age Crafts"&gt;&lt;/li&gt;
        *	&lt;/ul&gt;
        * &lt;/div&gt;
        * &lt;script&gt;
        * // Call hide method.
        * $(document).ready(function(){
        * $("#lb").ejListView("hide");
        * });
        * &lt;/script&gt;
        * @memberof ejListView
        * @instance
        */
        //Mobile
        /**
        * To hide the list.
        * @method hide
        * @return jQuery
        * @example 
        * &lt;div id="lb" data-role="ejmlistview"&gt;
        * 	&lt;ul&gt;
        *		&lt;li data-ej-text="Artwork"&gt;&lt;/li&gt;
        * 		&lt;li data-ej-text="Abstract"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="2 Acrylic Mediums"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="Creative Acrylic"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="Modern Painting"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="Canvas Art"&gt;&lt;/li&gt;
        * 		&lt;li data-ej-text="Black white"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="Children"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="Preschool Crafts"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="School-age Crafts"&gt;&lt;/li&gt;
        *	&lt;/ul&gt;
        * &lt;/div&gt;
        * &lt;script&gt;
        * // Call hide method.
        * $(document).ready(function(){
        * $("#lb").ejmListView("hide");
        * });
        * &lt;/script&gt;
        * @memberof ejmListView
        * @instance
        */

        hide: function (childId) {
            this._getElement(childId).css('visibility', 'hidden');
        },
        //To add list item
        /**
        * To add item in the given index.
        * @method addItem
        * @return jQuery
        * @example 
        * &lt;div id="lb" data-role="ejlistview"&gt;
        * 	&lt;ul&gt;
        *		&lt;li data-ej-text="Artwork"&gt;&lt;/li&gt;
        * 		&lt;li data-ej-text="Abstract"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="2 Acrylic Mediums"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="Creative Acrylic"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="Modern Painting"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="Canvas Art"&gt;&lt;/li&gt;
        * 		&lt;li data-ej-text="Black white"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="Children"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="Preschool Crafts"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="School-age Crafts"&gt;&lt;/li&gt;
        *	&lt;/ul&gt;
        * &lt;/div&gt;
        * &lt;script&gt;
        * // Call addItem method.
        * $(document).ready(function(){
        * $("#lb").ejListView("addItem",$("&ltli data-ej-text='Comic / Cartoon'&gt&lt/li&gt"),2);
        * });
        * &lt;/script&gt;
        * @memberof ejListView
        * @instance
        */
        //Mobile
        /**
        * To add item in the given index.
        * @method addItem
        * @return jQuery
        * @example 
        * &lt;div id="lb" data-role="ejmlistview"&gt;
        * 	&lt;ul&gt;
        *		&lt;li data-ej-text="Artwork"&gt;&lt;/li&gt;
        * 		&lt;li data-ej-text="Abstract"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="2 Acrylic Mediums"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="Creative Acrylic"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="Modern Painting"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="Canvas Art"&gt;&lt;/li&gt;
        * 		&lt;li data-ej-text="Black white"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="Children"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="Preschool Crafts"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="School-age Crafts"&gt;&lt;/li&gt;
        *	&lt;/ul&gt;
        * &lt;/div&gt;
        * &lt;script&gt;
        * // Call addItem method.
        * $(document).ready(function(){
        * $("#lb").ejmListView("addItem",$("&ltli data-ej-text='Comic / Cartoon'&gt&lt/li&gt"),2);
        * });
        * &lt;/script&gt;
        * @memberof ejmListView
        * @instance
        */
        addItem: function (list, index) {
            if (index >= 0)
                list.insertBefore($(this._orgEle.children().children()[index]));
            else
                this._orgEle.children().append(list);
            this._refresh();
        },
        //To remove list item
        /**
        * To remove item in the given index.
        * @method removeItem
        * @return jQuery
        * @example 
        * &lt;div id="lb" data-role="ejlistview"&gt;
        * 	&lt;ul&gt;
        *		&lt;li data-ej-text="Artwork"&gt;&lt;/li&gt;
        * 		&lt;li data-ej-text="Abstract"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="2 Acrylic Mediums"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="Creative Acrylic"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="Modern Painting"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="Canvas Art"&gt;&lt;/li&gt;
        * 		&lt;li data-ej-text="Black white"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="Children"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="Preschool Crafts"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="School-age Crafts"&gt;&lt;/li&gt;
        *	&lt;/ul&gt;
        * &lt;/div&gt;
        * &lt;script&gt;
        * // Call removeItem method.
        * $(document).ready(function(){
        * $("#lb").ejListView("removeItem",3);
        * });
        * &lt;/script&gt;
        * @memberof ejListView
        * @instance
        */
        //Mobile
        /**
        * To remove item in the given index.
        * @method removeItem
        * @return jQuery
        * @example 
        * &lt;div id="lb" data-role="ejmlistview"&gt;
        * 	&lt;ul&gt;
        *		&lt;li data-ej-text="Artwork"&gt;&lt;/li&gt;
        * 		&lt;li data-ej-text="Abstract"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="2 Acrylic Mediums"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="Creative Acrylic"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="Modern Painting"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="Canvas Art"&gt;&lt;/li&gt;
        * 		&lt;li data-ej-text="Black white"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="Children"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="Preschool Crafts"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="School-age Crafts"&gt;&lt;/li&gt;
        *	&lt;/ul&gt;
        * &lt;/div&gt;
        * &lt;script&gt;
        * // Call removeItem method.
        * $(document).ready(function(){
        * $("#lb").ejmListView("removeItem",3);
        * });
        * &lt;/script&gt;
        * @memberof ejmListView
        * @instance
        */
        removeItem: function (index, childId) {
            if (index >= 0) {
                element = this._getElement(childId);
                var child = $(element.find('li.' + this._prefixClass + 'list')[index]).attr('data-childid');
                if (this.element.find($('#' + child).length))
                    this.element.find($('#' + child)).remove();
                $(element.find('li.' + this._prefixClass + 'list')[index]).remove();
            }
        },
        //To clear the control before updating with new datasource
        /**
        * To clear all the list item in the control before updating with new datasource.
        * @method clear
        * @return jQuery
        * @example 
        * &lt;div id="lb" data-role="ejlistview" data-ej-dataBinding="true" data-ej-datasource="window.dbitem1" &gt;
        * &lt;/div&gt;  
        * &lt;input id="button" type="button" data-role="ejbutton" data-ej-text="Clear" data-ej-mouseUP="mouseUP" /&gt;<br>         
        * &lt;script&gt;
        * window.dbitem1 =
        * [   { "text": "Hot Singles"},
        *     { "text": "Rising Artists"},
        *     { "text": "Live Music" },
        *     { "text": "Best of 2013 So Far"},
        *     { "text": "100 Albums - $5 Each"},
        *     { "text": "Hip-Hop and R&B Sale"},
        *     { "text": "CD Deals"}];
        * window.dbitem2 =
        * [   { "text": "Music"},
        *     { "text": "Videos"},
        *     { "text": "Games" },
        *     { "text": "Chat"},
        *     { "text": "Others"}];
        * function mouseUP(){
        *     $('#lb').ejListView("clear");
        *     $("#lb").ejListView({dataSource:"window.dbitem2"});
        * }
        * &lt;/script&gt;  
        * @memberof ejListView
        * @instance
        */
        //Mobile
        /**
        * To clear all the list item in the control before updating with new datasource.
        * @method clear
        * @return jQuery
        * @example 
        * &lt;div id="lb" data-role="ejmlistview" data-ej-dataBinding="true" data-ej-datasource="window.dbitem1" &gt;
        * &lt;/div&gt;  
        * &lt;input id="button" type="button" data-role="ejmbutton" data-ej-text="Clear" data-ej-touchEnd="touchEnd" /&gt;<br>         
        * &lt;script&gt;
        * window.dbitem1 =
        * [   { "text": "Hot Singles"},
        *     { "text": "Rising Artists"},
        *     { "text": "Live Music" },
        *     { "text": "Best of 2013 So Far"},
        *     { "text": "100 Albums - $5 Each"},
        *     { "text": "Hip-Hop and R&B Sale"},
        *     { "text": "CD Deals"}];
        * window.dbitem2 =
        * [   { "text": "Music"},
        *     { "text": "Videos"},
        *     { "text": "Games" },
        *     { "text": "Chat"},
        *     { "text": "Others"}];
        * function touchEnd(){
        *     $('#lb').ejmListView("clear");
        *     $("#lb").ejmListView({dataSource:"window.dbitem2"});
        * }
        * &lt;/script&gt;  
        * @memberof ejmListView
        * @instance
        */
        clear: function () {
            this.element.empty();
        },
        //To return the list item count
        /**
        * To get the total item count.
        * @method getItemsCount
        * @example 
        * &lt;div id="lb" data-role="ejlistview"&gt;
        * 	&lt;ul&gt;
        *		&lt;li data-ej-text="Artwork"&gt;&lt;/li&gt;
        * 		&lt;li data-ej-text="Abstract"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="2 Acrylic Mediums"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="Creative Acrylic"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="Modern Painting"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="Canvas Art"&gt;&lt;/li&gt;
        * 		&lt;li data-ej-text="Black white"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="Children"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="Preschool Crafts"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="School-age Crafts"&gt;&lt;/li&gt;
        *	&lt;/ul&gt;
        * &lt;/div&gt;
        * &lt;script&gt;
        * // Call getItemsCount method.
        * $(document).ready(function(){
        * $("#lb").ejListView("getItemsCount");
        * });
        * &lt;/script&gt;
        * @memberof ejListView
        * @instance
        */
        //Mobile
        /**
        * To get the total item count.
        * @method getItemsCount
        * @example 
        * &lt;div id="lb" data-role="ejmlistview"&gt;
        * 	&lt;ul&gt;
        *		&lt;li data-ej-text="Artwork"&gt;&lt;/li&gt;
        * 		&lt;li data-ej-text="Abstract"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="2 Acrylic Mediums"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="Creative Acrylic"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="Modern Painting"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="Canvas Art"&gt;&lt;/li&gt;
        * 		&lt;li data-ej-text="Black white"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="Children"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="Preschool Crafts"&gt;&lt;/li&gt;
        *		&lt;li data-ej-text="School-age Crafts"&gt;&lt;/li&gt;
        *	&lt;/ul&gt;
        * &lt;/div&gt;
        * &lt;script&gt;
        * // Call getItemsCount method.
        * $(document).ready(function(){
        * $("#lb").ejmListView("getItemsCount");
        * });
        * &lt;/script&gt;
        * @memberof ejmListView
        * @instance
        */
        getItemsCount: function (childId) {
            return this._getElement(childId).find('li.' + this._prefixClass + 'list').length;
        }
    });
})(jQuery, Syncfusion);;
/**
* @fileOverview Plugin to style the Waiting pop elements
* @copyright Copyright Syncfusion Inc. 2001 - 2013. All rights reserved.
*  Use of this code is subject to the terms of our license.
*  A copy of the current license can be obtained at any time by e-mailing
*  licensing@syncfusion.com. Any infringement will be prosecuted under
*  applicable laws. 
* @version 12.1 
* @author <a href="mailto:licensing@syncfusion.com">Syncfusion Inc</a>
*/(function ($, ej, undefined) {
 /**
    * @namespace ej
	* @class ejWaitingPopup
	* @requires jQuery
    * @requires jquery.easing.1.3.js
	* @requires ej.core.js
	* @requires ej.waitingpopup.js
	* @classdesc Custom Design for Html Button control.
	* @example 
	*  &lt;div id="target"&gt;&lt;/div&gt;
	* &lt;script&gt;
	* // Simple waiting popup creation.
    * $("#target").ejWaitingPopup({ showOnInit: true });
	* &lt;/script&gt;
    	        * &lt;style&gt;
              #target {
            height: 200px;
            width: 600px;
            margin: 0 auto;
        }

       #target_WaitingPopup .e-image {
            display: block;
            height: 70px;
        }
	        * &lt;/style&gt;
	*/
    // ejWaitingPopup is the plugin name 
    // "ej.WaitingPopup" is "namespace.className" will hold functions and properties

    ej.widget("ejWaitingPopup", "ej.WaitingPopup", {
        // widget element will be automatically set in this
        element: null,

        // user defined model will be automatically set in this
        model: null,
        validTags: ["div", "span"],
        _setFirst: false,
        _rootCSS: "e-waitingpopup",

        // default model
        defaults: {
			/**
			* Enables the visibility of the WaitingPopup control
			* @default false
			* @type {Boolean}
			* @example 
	        *  &lt;div id="target"&gt;&lt;/div&gt;
	        * &lt;script&gt;
			* //To set showOnInit API value during initialization  
			* 	$("#target").ejWaitingPopup({ showOnInit: true});
	        * &lt;/script&gt;
            	        * &lt;style&gt;
              #target {
            height: 200px;
            width: 600px;
            margin: 0 auto;
        }

       #target_WaitingPopup .e-image {
            display: block;
            height: 70px;
        }
	        * &lt;/style&gt;
			 * @memberof ejWaitingPopup
			* @instance
			*/
            showOnInit: false,
			/**
			* Enables or disables the default loading icon.
			* @default true
			* @type {Boolean}
			* @example 
	        *  &lt;div id="target"&gt;&lt;/div&gt;
	        * &lt;script&gt;
			* //To set showImage API value during initialization  
			* 	$("#target").ejWaitingPopup({ showOnInit: true, showImage: false});
	        * &lt;/script&gt;
            	        * &lt;style&gt;
              #target {
            height: 200px;
            width: 600px;
            margin: 0 auto;
        }

       #target_WaitingPopup .e-image {
            display: block;
            height: 70px;
        }
	        * &lt;/style&gt;
			 * @memberof ejWaitingPopup
			* @instance
			*/
            showImage: true,
			/**
			* Sets the root class for the WaitingPopup control theme
			* @default null
			* @type {String}
			* @example 
	        *  &lt;div id="target"&gt;&lt;/div&gt;
	        * &lt;script&gt;
			* //To Initialize the WaitingPopup with the cssClass  value specified. 
			* 	$("#target").ejWaitingPopup({showOnInit: true, cssClass : 'gradient-lime '});
	        * &lt;/script&gt;
            	        * &lt;style&gt;
              #target {
            height: 200px;
            width: 600px;
            margin: 0 auto;
        }

       #target_WaitingPopup .e-image {
            display: block;
            height: 70px;
        }
	        * &lt;/style&gt;
			 * @memberof ejWaitingPopup
			* @instance
			*/
            cssClass : "",
			/**
			* Sets the custom text in the pop-up panel to notify the waiting process
			* @default null
			* @type {String}
			* @example 
	        *  &lt;div id="target"&gt;&lt;/div&gt;
	        * &lt;script&gt;
			* //To Initialize the WaitingPopup with the text value specified
			* 	$("#target").ejWaitingPopup({showOnInit: true, text: 'waiting…' });
	        * &lt;/script&gt;
	        * &lt;style&gt;
              #target {
            height: 200px;
            width: 600px;
            margin: 0 auto;
        }

       #target_WaitingPopup .e-image {
            display: block;
            height: 70px;
        }
	        * &lt;/style&gt;
			 * @memberof ejWaitingPopup
			* @instance
			*/	
            text: null,
			/**
			* Loads HTML content inside the popup panel instead of the default icon
			* @default null
			* @type {object}
			* @example 
			*&lt;div id="content"&gt;
			*   &lt;div class="block"&gt;
            *   &lt;div class="logo"&gt;
            *&lt;/div&gt;
			*    &lt;div class="text"&gt;
			*     &lt;p&gt; Content is loading ... &lt;/p&gt;
			*      &lt;/div&gt;
            &lt;/div&gt;
            *&lt;div class="loader"&gt;
            *&lt;/div&gt;
			*&lt;/div&gt;
	        * &lt;script&gt;
			* //To Initialize the WaitingPopup control with the template value specified.
			* 	$("#target").ejWaitingPopup({ showOnInit: true,template: $('#content') });
	        * &lt;/script&gt;
	        * &lt;style&gt;
         
          .block {
            height: 76px;
        }

        .logo {
            background-image: url("themes/images/waitingpopup/js_logo.png");
            float: left;
            height: 100%;
            width: 77px;
            margin-right: 15px;
        }

        .txt {
            float: left;
            font-size: 17px;
            height: 100%;
            text-align: left;
        }

            .txt p {
                margin: 0;
            }

        .loader {
            background: url("themes/images/waitingpopup/load_light.gif") no-repeat scroll -5px 18px transparent;
            height: 40px;
            width: 100%;
        }

        .darktheme .loader {
            background-image: url("themes/images/waitingpopup/load_dark.gif");
        }

        #content {
            cursor: default;
            height: 112px;
            width: 275px;
        }

            
	        * &lt;/style&gt;
			 * @memberof ejWaitingPopup
			* @instance
			*/	
            template: null,
             /**    
			 * Fires after Create waitingpopup successfully
			 * @event
			 * @name ejWaitingPopup#create		
			 * @param {boolean}  argument.cancel returns the cancel option value
			 * @param {object}  argument.model returns the waitingpopup model
			 * @param {string}  argument.type returns the name of the event			 
			 * @example 
	        *  &lt;div id="target"&gt;&lt;/div&gt;
	        * &lt;script&gt;
			* //To Initialize the WaitingPopup with the text value specified with create event
			* 	$("#target").ejWaitingPopup({showOnInit: true, text: 'waiting…',create: function (args) {} });
	        * &lt;/script&gt;
	        * &lt;style&gt;
              #target {
            height: 200px;
            width: 600px;
            margin: 0 auto;
        }

       #target_WaitingPopup .e-image {
            display: block;
            height: 70px;
        }
	        * &lt;/style&gt;
			 * @memberof ejWaitingPopup
			* @instance
			*/
            create: null,
            /**    
           * Fires after Destroy waitingpopup successfully
           * @event
           * @name ejWaitingPopup#destroy		
           * @param {boolean}  argument.cancel returns the cancel option value
           * @param {object}  argument.model returns the waitingpopup model
           * @param {string}  argument.type returns the name of the event			 
           * @example 
          *  &lt;div id="target"&gt;&lt;/div&gt;
          * &lt;script&gt;
          * //To Initialize the WaitingPopup with the text value specified with destroy event
          * 	$("#target").ejWaitingPopup({showOnInit: true, text: 'waiting…',destroy: function (args) {} });
          * &lt;/script&gt;
          * &lt;style&gt;
            #target {
          height: 200px;
          width: 600px;
          margin: 0 auto;
      }

     #target_WaitingPopup .e-image {
          display: block;
          height: 70px;
      }
          * &lt;/style&gt;
           * @memberof ejWaitingPopup
          * @instance
          */
            destroy: null
        },
		/**
         * Specify the data types for default properties 
		 * @private
         */
        dataTypes: {
            showOnInit: "boolean",
            showImage: "boolean",
            cssClass : "string"
        },
		 /**
        * To show the waiting popup
		* @return jQuery
		* @example 
		*  &lt;div id="target"&gt;&lt;/div&gt;
		* &lt;script&gt;
        * $("#target").ejWaitingPopup({showOnInit: true});
		* // Initialize the WaitingPopup object.
		*   var popupObj = $("#target").data("ejWaitingPopup");
		* // Calls the show method of WaitingPopup to display.
	    * popupObj.show();
	    * &lt;/script&gt;
        	        * &lt;style&gt;
              #target {
            height: 200px;
            width: 600px;
            margin: 0 auto;
        }

       #target_WaitingPopup .e-image {
            display: block;
            height: 70px;
        }
	        * &lt;/style&gt;
		* @example 
		*  &lt;div id="target"&gt;&lt;/div&gt;
		* &lt;script&gt;
        * $("#target").ejWaitingPopup({showOnInit: true});
		* // Display WaitingPopup using the show method.
		* $("#target").ejWaitingPopup("show");
		* &lt;/script&gt;
        	        * &lt;style&gt;
              #target {
            height: 200px;
            width: 600px;
            margin: 0 auto;
        }

       #target_WaitingPopup .e-image {
            display: block;
            height: 70px;
        }
	        * &lt;/style&gt;
		*@memberof ejWaitingPopup
		* @instance
        */
        show: function () {
            this._refreshPanel();
            this.maindiv.css("display", "block");
            this.model.showOnInit = true;
        },
		 /**
        * To hide the waiting popup
		* @return jQuery
		* @example 
		*  &lt;div id="target"&gt;&lt;/div&gt;
		* &lt;script&gt;
        * $("#target").ejWaitingPopup({showOnInit: true});
		* // Initialize the WaitingPopup object.
		*   var popupObj = $("#target").data("ejWaitingPopup");
		* // Calls the hide method of WaitingPopup not to display.
	    * popupObj.hide();
	    * &lt;/script&gt;
        	        * &lt;style&gt;
              #target {
            height: 200px;
            width: 600px;
            margin: 0 auto;
        }

       #target_WaitingPopup .e-image {
            display: block;
            height: 70px;
        }
	        * &lt;/style&gt;
		* @example 
		*  &lt;div id="target"&gt;&lt;/div&gt;
		* &lt;script&gt;
        * $("#target").ejWaitingPopup({showOnInit: true});
		* // hide WaitingPopup using the hide method.
		* $("#target").ejWaitingPopup('hide');
		* &lt;/script&gt;
        	        * &lt;style&gt;
              #target {
            height: 200px;
            width: 600px;
            margin: 0 auto;
        }

       #target_WaitingPopup .e-image {
            display: block;
            height: 70px;
        }
	        * &lt;/style&gt;
		*@memberof ejWaitingPopup
		* @instance
        */
        hide: function () {
            this.maindiv.css("display", "none");
            this.model.showOnInit = false;
        },
		/**
        * Refreshes the WaitingPopup control by resetting the pop-up panel position and content position
		* @return jQuery
		* @example 
		*  &lt;div id="target"&gt;&lt;/div&gt;
		* &lt;script&gt;
        * $("#target").ejWaitingPopup({showOnInit: true});
		* // Initialize the WaitingPopup object.
		*   var popupObj = $("#target").data("ejWaitingPopup");
	    * popupObj.refresh();
	    * &lt;/script&gt;
        	        * &lt;style&gt;
              #target {
            height: 200px;
            width: 600px;
            margin: 0 auto;
        }

       #target_WaitingPopup .e-image {
            display: block;
            height: 70px;
        }
	        * &lt;/style&gt;
		* @example 
		*  &lt;div id="target"&gt;&lt;/div&gt;
		* &lt;script&gt;
        * $("#target").ejWaitingPopup({showOnInit: true});
		* // Refresh the WaitingPopup using refresh method.
		* $("#target").ejWaitingPopup('refresh');
		* &lt;/script&gt;
        	        * &lt;style&gt;
              #target {
            height: 200px;
            width: 600px;
            margin: 0 auto;
        }

       #target_WaitingPopup .e-image {
            display: block;
            height: 70px;
        }
	        * &lt;/style&gt;
		*@memberof ejWaitingPopup
		* @instance
        */
        refresh: function () {
            this._refreshPanel();
        },
		 /**
         * To configure text for the waiting popup.		
		 * @private
         */	 
        _setText: function (text) {
            if (text) {
                if (this.popupText) this.popupText.html(text);
                else {
                    this._generateTextTag(text);
                    this._setContentPosition();
                }
            }
            else if (this.popupText) {
                this.popupText.remove();
                this.popupText = null;
            }
        },
		 /**
         * To configure showImage for the waiting popup.		
		 * @private
         */	
        _showImage: function (boolean) {
            if (boolean) {
                this.popupImage = ej.buildTag("span.e-image");
                if (this.popupText) this.popupImage.insertBefore(this.popupText);
                else this.maindiv.append(this.popupImage);
            }
            else if (this.popupImage) {
                this.popupImage.remove();
                this.popupImage = null;
            }
        },
		/**
         * To configure setTemplate for the waiting popup.		
		 * @private
         */	
        _setTemplate: function () {
            var template = this.model.template;
            if (typeof template === "string") template = $(template);
            if (typeof template === "object" && typeof template.css === "function") 
                this.templateObj = template;
            else 
                this.templateObj = ej.buildTag("div", "", { "text-align": "center" }).append(template);

            this.templateObj.css({ "visibility": "visible", "display": "block" });
            this.maindiv.append(this.templateObj);
        },
		/**
         * To configure setTheme for the waiting popup.		
		 * @private
         */	
        _setTheme: function (skin) {
            this.maindiv.removeClass(this.model.cssClass ).addClass(skin);
        },

        // constructor function
		/**
         * Create the WaitingPopup widget
		 * @private
         */	
        _init: function () {
            this._initialize();
            this._render();
        },
		 /**
         * To configure the properties at runtime using SetModel		
		 * @private
         */	 
        _setModel: function (options) {
            var option;
            for (option in options) {
                switch (option) {
                    case "text": this._setText(options[option]); break;
                    case "cssClass ": this._setTheme(options[option]); break;
                    case "showOnInit": this._setVisibility(options[option]); break;
                    case "showImage": this._showImage(options[option]); this._setContentPosition(); break;
                    case "template":
                        this.maindiv.empty();
                        if (options[option]) {
                            this.model.template = options[option];
                            this._setTemplate();
                        }
                        else {
                            this.model.template = options[option] = null;
                            this._showImage(this.model.showImage);
                            if (this.model.text) this._generateTextTag(this.model.text);
                        }
                        this._setContentPosition();
                        break;
                }
            }
        },

        _destroy: function () {
            this.maindiv.remove();
        },
		 /**
         * To initialize the waiting popup		
		 * @private
         */	 
        _initialize: function () {
            this.maindiv = null;
            this.popupText = null;
            this.popupImage = null;
            this.templateObj = null;
        },

		 /**
         * Render Section For waiting popup		
		 * @private
         */	
        _render: function () {
			var oldWrapper = $("#" + this.element.context.id + "_WaitingPopup").get(0);
            if (oldWrapper)
                $(oldWrapper).remove();
            this.maindiv = ej.buildTag("div.e-waitpopup-pane e-widget " + this.model.cssClass  + "#" + this.element[0].id + "_WaitingPopup");
            if (this.model.template) {
                this._setTemplate();
            }
            else {
                this._showImage(this.model.showImage);
                if (this.model.text) {
                    this._generateTextTag(this.model.text);
                }
            }
            $('body').append(this.maindiv);
            this._setVisibility(this.model.showOnInit);
        },
		/**
         * To configure refreshPanel for the waiting popup	
		 * @private
         */	
        _refreshPanel: function () {
            this.maindiv.width(this.element.outerWidth());
            this.maindiv.height(this.element.outerHeight());
            this._setPanelPosition();
            this._setContentPosition();
        },
		/**
         * To configure setPanelPosition for the waiting popup	
		 * @private
         */	
        _setPanelPosition: function() {
            var location = this.element.offset();
            this.maindiv.css({
                "left": Math.ceil(location.left) + "px",
                "top": Math.ceil(location.top) + "px",
                "z-index": this._maxZindex() + 1
            });
        },
		/**
         * To configure setContentPosition for the waiting popup	
		 * @private
         */	
        _setContentPosition: function () {
            if (this.model.template == null) {
                var textHeight = 0, imgHeight = 0, targetHeight, top = null;
                targetHeight = this.element.outerHeight();
                if (this.popupText) textHeight = this.popupText.outerHeight();
                if (this.popupImage) imgHeight = this.popupImage.outerHeight();

                if (this.popupImage) {
                    top = Math.ceil((targetHeight - (imgHeight + textHeight)) / 2);
                    this.popupImage.css("top", top + "px");
                }
                if (this.popupText) {
                    if (!top) top = Math.ceil((targetHeight - textHeight) / 2);
                    this.popupText.css("top", top + "px");
                }
            }
            else {
                this.templateObj.css({
                    "position": "relative",
                    "left": Math.ceil((this.element.outerWidth() - this.templateObj.outerWidth()) / 2),
                    "top": Math.ceil((this.element.outerHeight() - this.templateObj.outerHeight()) / 2)
                });
            }
        },
		/**
         * To configure generateTextTag for the waiting popup	
		 * @private
         */	
        _generateTextTag: function (text) {
            this.popupText = ej.buildTag("div.e-text", text);
            this.maindiv.append(this.popupText);
        },
		/**
         * To configure setVisibility for the waiting popup	
		 * @private
         */
        _setVisibility: function (showOnInit) {
            if (showOnInit) this.show();
            else this.hide();
        },
		/**
         * To configure maxZindex for the waiting popup	
		 * @private
         */
        _maxZindex: function () {
            var elements = document.getElementsByTagName("div"), maxIndex = 0, i, zindex;
            for (i = 0; i < elements.length - 1; i++) {
                zindex = parseInt(elements[i].style.zIndex, 10);  // 10 - radix value
                if (zindex > maxIndex) {
                    maxIndex = zindex;
                }
            }
            return maxIndex;
        }
    });
})(jQuery, Syncfusion);;
/*!
*  filename: ej.button.js
*  version : 12.1
*  Copyright Syncfusion Inc. 2001 - 2013. All rights reserved.
*  Use of this code is subject to the terms of our license.
*  A copy of the current license can be obtained at any time by e-mailing
*  licensing@syncfusion.com. Any infringement will be prosecuted under
*  applicable laws. 
*/

/**
* @fileOverview Plugin to style the Html Button elements
* @copyright Copyright Syncfusion Inc. 2001 - 2013. All rights reserved.
*  Use of this code is subject to the terms of our license.
*  A copy of the current license can be obtained at any time by e-mailing
*  licensing@syncfusion.com. Any infringement will be prosecuted under
*  applicable laws. 
* @version 12.1 
* @author <a href="mailto:licensing@syncfusion.com">Syncfusion Inc</a>
*/
 
(function ($, ej, undefined) {
    /**
    * @namespace ej
	* @classdesc Custom Design for Html Button control.
	* @class ejButton
	* @param {object} options - settings for button
	* @requires jQuery
	* @requires ej.core.js
    * @requires ej.button.js
	* @example 
	* &lt;button id="button1"&gt;Button&lt;/button&gt; <br> 
	* &lt;script&gt;
	* // Create Button
    * $('#button1').ejButton(); 	
	* &lt;/script&gt;
	*/

    ej.widget("ejButton", "ej.Button", {
        // widget element will be automatically set in this
        element: null,
        // user defined model will be automatically set in this
        model: null,
        validTags: ["button", "input"],
        _setFirst: false,
        //Root Css Class
        _rootCSS: "e-button",
		
        defaults: {  
			/**		
			* Specifies the size of the Button.	See {@link ButtonSize}
			* @default ej.ButtonSize.Normal
			* @type {enum}
			* @example 
			* &lt;button id="button1"&gt;Button&lt;/button&gt; <br> 
			* &lt;script&gt;
			* //To set size API value during initialization  
			* 	$("#button1").ejButton({  size: ej.ButtonSize.Mini});	
			* &lt;/script&gt;
			* @memberof ejButton
			* @instance
			*/		
            size: "normal",
            /**		
			* Specifies the Type of the Button.	See {@link ButtonType}
			* @default ej.ButtonType.Submit
			* @type {enum | string}
			* @example 
			* &lt;button id="button1"&gt;Button&lt;/button&gt; <br> 
			* &lt;script&gt;
			* //To set type API value during initialization  
			* 	$("#button1").ejButton({type: ej.ButtonType.Submit});	
			* &lt;/script&gt;
			* @memberof ejButton
			* @instance
			*/
			type:"submit",
			/**		
			* Specifies the height of the Button.
			* @default 28
			* @type {number}
			* @example 
			* &lt;button id="button1"&gt;Button&lt;/button&gt; <br> 
			* &lt;script&gt;
			* //To set height API value during initialization  
			* 	$("#button1").ejButton({ height:"30px" });	
            * &lt;/script&gt;			
			* @memberof ejButton
			* @instance
			*/
            height: "",
			/**		
			* Specifies the width of the Button.
			* @default 100
			* @type {number}
			* @example 
			* &lt;button id="button1"&gt;Button&lt;/button&gt; <br> 
			* &lt;script&gt;
			* //To set width API value during initialization  
			* 	$("#button1").ejButton({width: "150px"});	
			* &lt;/script&gt;
			* @memberof ejButton
			* @instance
			*/
            width: "",
			/**		
			* Specifies the button control state.
			* @default true
			* @type {boolean}
			* @example 
			* &lt;button id="button1"&gt;Button&lt;/button&gt; <br> 
			* &lt;script&gt;
			* // Enable Button on initialization. 
			* 	$("#button1").ejButton({ enabled:true });	
		    * &lt;/script&gt;
			* @memberof ejButton
			* @instance
			*/
            enabled: true,
			/**		
			* Specifies the text content for Button.
			* @default null
			* @type {string}
			* @example 
			* &lt;button id="button1"&gt;Button&lt;/button&gt; <br> 
			* &lt;script&gt;
			* // Set the button text on initialization. 			
			* 	$("#button1").ejButton({  text: "Hello Word" });
            * &lt;/script&gt;			
			* @memberof ejButton
			* @instance
			*/
            text: null,
			/**		
			* Specifies the contentType of the Button. See {@link ContentType}
			* @default ej.ContentType.TextOnly
			* @type {enum}
			* @example 
			* &lt;button id="button1"&gt;Button&lt;/button&gt; <br> 
			* &lt;script&gt;
			* // Set the button contentType on initialization. 			
			* 	$("#button1").ejButton({  contentType : ej.ContentType.ImageOnly,prefixIcon: "e-uiLight e-icon e-handup" });			
			* &lt;/script&gt;
            * @memberof ejButton
			* @instance
			*/
            contentType: "textonly",
			/**		
			*  Specifies the image position of the Button. This image position is applicable only with the textandimage contentType property. The images can be positioned in both imageLeft and imageRight options. See {@link ImagePosition}
			* @default ej.ImagePosition.ImageLeft
			* @type {enum}
			* @example 
			* &lt;button id="button1"&gt;Button&lt;/button&gt; <br> 
			* &lt;script&gt;
			* // Set the image position for button during initialization. 			
			* 	$("#button1").ejButton(
            * {
            *    contentType: ej.ContentType.TextAndImage,
            *    imagePosition: ej.ImagePosition.ImageRight,
			*	prefixIcon: "e-uiLight e-icon e-handup" //Specifies the primary icon for Button
            * });	
            * &lt;/script&gt;			
			 * @memberof ejButton
			* @instance
			*/
            imagePosition: "imageleft",
			/**		
			* Specify the rounded corner to button
			* @default false
			* @type {boolean}
			* @example 
			* &lt;button id="button1"&gt;Button&lt;/button&gt; <br> 
			* &lt;script&gt;
			* // Set the showRoundedCorner during initialization. 			
			* 	$("#button1").ejButton({  showRoundedCorner : true });			
            * &lt;/script&gt;			
			 * @memberof ejButton
			* @instance
			*/
            showRoundedCorner: false,
			/**		
			* Specify the cssClass to button to achieve custom theme.
			* @default emptyString
			* @type {string}
			* @example 
			* &lt;button id="button1"&gt;Button&lt;/button&gt; <br> 
			* &lt;script&gt;
			* // Set the cssClass during initialization. 			
			* 	$("#button1").ejButton({  cssClass : "gradient-lime" });			
			* &lt;/script&gt;
			* @memberof ejButton
			* @instance
			*/
            cssClass: "",
			/**		
			* Specifies the primary icon for Button. This is applicable for the content type’s imageOnly, textandimage, imagetextimage and imageboth.
			* @default null
			* @type {string}
			* @example 
			* &lt;button id="button1"&gt;Button&lt;/button&gt; <br> 
			* &lt;script&gt;
			* // Set the primary icon during initialization. 			
			* 	$("#button1").ejButton(
            *  {
            *     contentType: "imageonly",
            *     prefixIcon: "e-uiLight e-icon e-handup"
            *  });		
			* &lt;/script&gt;
			* @memberof ejButton
			* @instance
			*/
            prefixIcon: null,
			/**		
			* Specifies the secondary icon for Button. This is applicable for the content type’s imagetextimage and imageboth.
			* @default null
			* @type {string}
			* @example 
			* &lt;button id="button1"&gt;Button&lt;/button&gt; <br> 
			* &lt;script&gt;
			* // Set the secondary icon during initialization. 			
			* 	$("#button1").ejButton(
            *  {
            *       contentType: "imageonly",
			*		prefixIcon: "e-uiLight e-icon e-handup",
            *	    suffixIcon: "e-uiLight e-icon e-padlockclosed"
            *  });		
			* &lt;/script&gt;
			* @memberof ejButton
			* @instance
			*/
            suffixIcon: null,
			/**		
			* Specify the Right to Left direction to button
			* @default false
			* @type {boolean}
			* @example 
			* &lt;button id="button1"&gt;Button&lt;/button&gt; <br> 
			* &lt;script&gt;
			* // Set the enableRTL during initialization. 			
			* 	$("#button1").ejButton({ contentType: ej.ContentType.TextAndImage,
            *    imagePosition: ej.ImagePosition.ImageLeft,
            *    prefixIcon: "e-uiLight e-login", enableRTL : true });	
            * &lt;/script&gt;					
			* @memberof ejButton
			* @instance
			*/
            enableRTL: false,
			/**		
			* Convert the button as repeat button. It raises the 'Click' event repeattedly from the it is pressed until it is released.
			* @default false
			* @type {boolean}
			* @example 
			* &lt;button id="button1"&gt;Button&lt;/button&gt; <br> 
			* &lt;script&gt;
			* // Create repeat button during initialization. 			
			* 	$("#button1").ejButton({  repeatButton : true });
            * &lt;/script&gt;			
			* @memberof ejButton
			* @instance
			*/
            repeatButton: false,
			/**		
			* Specified the timeInterval between two 'click' events while button in repeat button mode.
			* @default "150"
			* @type {string}
			* @example 
			* &lt;button id="button1"&gt;Button&lt;/button&gt; <br> 
			* &lt;script&gt;
			* // Set interval during initialization. 			
			* 	$("#button1").ejButton({  
			*			repeatButton : true,
			* 			timeInterval : "100" });
			* &lt;/script&gt;
			 * @memberof ejButton
			* @instance
			*/
            timeInterval:"150",
			 /**     
			 * Fires after Button control is created.If the user want to perform any operation after the button control creation then the user can make use of this create event. 
			 * @event
			 * @name ejButton#create		
			 * @param {Object} argument Event parameters from button     
			 * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
			 * @param {object}  argument.model returns the button model
			 * @param {string}  argument.type returns the name of the event			
			 * @example 
			 * &lt;button id="button1"&gt;Button&lt;/button&gt; <br> 
			 * &lt;script&gt;
			 * //create event for button
             * $("#button1").ejButton({
             *    create: function (args) {
			 * // Write a code block to perform operation after creating the button.
			 * }
             * });
			 * &lt;/script&gt;
			 * @memberof ejButton
			 * @instance
			 */		
            create: null,
			 /**     
			 * Fires when Button control is clicked successfully.Consider the scenario to perform any validation,modification of content or any other operations click on button,we can make use of this click event to achieve the scenario.  
			 * @event
			 * @name ejButton#click 	
			 * @param {Object} argument Event parameters from button     
			 * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
			 * @param {object}  argument.model returns the button model
			 * @param {string}  argument.type returns the name of the event
			 * @param {boolean}  argument.status return the button state
			 * @example 
			 * &lt;button id="button1"&gt;Button&lt;/button&gt; <br> 
			 * &lt;script&gt;
			 * //click event for button
             * $("#button1").ejButton({
             *    click: function (args) {
			 * // Write a code block to perform operation while click on button.
			 * }
             * });
             * &lt;/script&gt;			 
			 * @memberof ejButton
			 * @instance
			 */
            click: null,
			 /**    
			 * Fires when the button is destroyed successfully.If the user want to perform any operation after the destroy button control then the user can make use of this destroy event.  
			 * @event
			 * @name ejButton#destroy 	
			 * @param {Object} argument Event parameters from button     
			 * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
			 * @param {object}  argument.model returns the button model
			 * @param {string}  argument.type returns the name of the event			 
			 * @example 
			 * &lt;button id="button1"&gt;Button&lt;/button&gt; <br> 
			 * &lt;script&gt;
			 * //destroy event for button
             * $("#button1").ejButton({
             *    destroy: function (args) {
			 * // Write a code block to perform operation after destroy the button.
			 * }
             * });
			 * &lt;/script&gt;
			 * @memberof ejButton
			 * @instance
			 */
            destroy: null
        },

          /**
         * Specify the data types for default properties 
		 * @private
         */
        dataTypes: {
            size: "enum",            
            enabled: "boolean",
			type:"enum",
            showRoundedCorner: "boolean",
            text: "string",
            contentType: "enum",
            imagePosition: "enum",
            prefixIcon: "string",
            suffixIcon: "string",
            cssClass: "string",
            repeatButton: "boolean",
            enableRTL: "boolean",
            timeInterval:"string"
        },
        /**
        * To disable the button  
		* @return jQuery
		* @example 
		* &lt;button id="button1"&gt;Button&lt;/button&gt; <br> 
		* &lt;script&gt;
		* // Create Button
		* $("#button1").ejButton();
		* var btnObj = $("#button1").data("ejButton");
		* btnObj.disable(); // disable the button
		* &lt;/script&gt;
		* @example 
		* &lt;button id="button1"&gt;Button&lt;/button&gt; <br> 
		* &lt;script&gt;
		* // disable the button
		* $("#button1").ejButton();
		* $("#button1").ejButton("disable");	
		* &lt;/script&gt;
		*@memberof ejButton
		* @instance
        */
        disable: function () {
            /// <summary>This function is  used to Disable the Button Object</summary>
            this.element.addClass("e-disable").attr("aria-disabled",true);
            this.model.enabled = false;
        },
		/**
        * To enable the button 		
		* @return jQuery
		* @example 
		* &lt;button id="button1"&gt;Button&lt;/button&gt; <br> 
		* &lt;script&gt;
		* // Create Button
		* $("#button1").ejButton();
		* var btnObj = $("#button1").data("ejButton");
		* btnObj.enable(); // enable the button
		* &lt;/script&gt;
		* @example 
		* &lt;button id="button1"&gt;Button&lt;/button&gt; <br> 
		* &lt;script&gt;
		* // enable the button
		* $("#button1").ejButton();
		* $("#button1").ejButton("enable");	
		* &lt;/script&gt;
		* @memberof ejButton
		* @instance
         */
        enable: function () {
            /// <summary>This function is  used to Enable the Button Object</summary>
            this.element.removeClass("e-disable").attr("aria-disabled",false);
            this.model.enabled = true;			
        },
        /**
         * Create the butto widget
		 * @private
         */		 
        _init: function () {
            this._initialize();
            this._render();
            this._controlStatus(this.model.enabled);
            this._wireEvents(this.model.repeatButton);
        },

		 /**
        * destroy the button widget
		* all events bound using this._on will be unbind automatically and bring the control to pre-init state.
		* @alias destroy
		* @return jQuery
		* @example 
		* &lt;button id="button1"&gt;Button&lt;/button&gt; <br> 
		* &lt;script&gt;
		* // Create Button
		* $("#button1").ejButton();
		* var btnObj = $("#button1").data("ejButton");
		* btnObj.destroy(); // destroy the button
		* &lt;/script&gt;
		* @example 
		* &lt;button id="button1"&gt;Button&lt;/button&gt; <br> 
		* &lt;script&gt;
		* // enable the button
		* $("#button1").ejButton();
		* $("#button1").ejButton("destroy");	
		* &lt;/script&gt;
		* @memberof ejButton
		* @instance
         */	       
        _destroy: function () {
            /// <summary>This function is  used to destroy the Button Object</summary>
            this.element.removeClass(this.model.cssClass + " e-btn e-select e-disable e-corner-all e-widget");
        },
		
		 /**
         * To configure the properties at runtime using SetModel		
		 * @private
         */	 
        _setModel: function (options) {
            if (!(this.model.enabled) && ej.isNullOrUndefined(options["enabled"])) return false;
            var option;
            for (option in options) {
                switch (option) {
                    case "size":
                        this._setSize(options[option]);
                        break;
                    case "height":
                        this._setHeight(options[option]);
                        break;
                    case "width":
                        this._setWidth(options[option]);
                        break;
                    case "contentType":
                        this._setContentType(options[option]);
                        break;
                    case "imagePosition":
                        this._setImagePosition(options[option]);
                        break;
                    case "text":
                        this._setText(options[option]);
                        break;
                    case "prefixIcon":
                        this._setMajorIcon(options[option]);
                        break;
                    case "suffixIcon":
                        this._setMinorIcon(options[option]);
                        break;
                    case "enabled":
                        this._controlStatus(options[option]);
                        break;
                    case "showRoundedCorner":
                        this._roundedCorner(options[option]);
                        break;
                    case "cssClass":
                        this._setSkin(options[option]);
                        break;
                    case "enableRTL":
                        this._setRTL(options[option]);
                        break;
                    case "timeInterval":
                        this.model.timeInterval = options[option];
                        break;
                }
            }
        },
		
		 /**
         * To configure size of the button		
		 * @private
         */	 
        _setSize: function (val) {
            this.element.removeClass('e-btn-mini e-btn-medium e-btn-small e-btn-large e-btn-normal');
            this.element.addClass("e-btn-" + val);
        },
		_setType: function (val) {
            this.element.attr({"type":val});
		},
		 /**
         * To configure the height		
		 * @private
         */	 
        _setHeight: function (val) {
            /// <summary>This will set Text property of button </summary>
            this.element.css('height', val);
        },
		 /**
         * To configure width.		
		 * @private
         */	 
        _setWidth: function (val) {
            /// <summary>This will set Text property of button </summary>
            this.element.css('width', val);
        },
		 /**
         * To configure the button text		
		 * @private
         */	 
        _setText: function (val) {
            /// <summary>This will set Text property of button </summary>
            if(this.buttonType == "inputButton"){
                this.element.val(val);
            } else {
                if (this.model.contentType == ej.ContentType.TextOnly) {
                    this.element.text(val);
                } else {
                    this.textspan.text(val);
                }
            }
            this.model.text = val;
        },
		 /**
         * To configure button's prefix image		
		 * @private
         */	 
        _setMajorIcon: function (val) {
            /// <summary>This will set Major Icon property of button </summary>
            this.majorimgtag.removeClass(this.model.prefixIcon);
            this.majorimgtag.addClass(val);
            this.model.prefixIcon = val;
        },
		 /**
         * To configure button's suffix image		
		 * @private
         */	 
        _setMinorIcon: function (val) {
            /// <summary>This will set Minor Icon property of button </summary>
            this.minorimgtag.removeClass(this.model.suffixIcon);
            this.minorimgtag.addClass(val);
            this.model.suffixIcon = val;
        },
		 /**
         * To configure button contents type		
		 * @private
         */	 
        _setContentType: function (val) {
            if (val != this.model.contentType) {
                this.element.empty();
                this.model.contentType = val;
                this._renderButtonNormal();
            }
        },
		 /**
         * To configure images position inside the button		
		 * @private
         */	 
        _setImagePosition: function (val) {
            if ((this.model.contentType == ej.ContentType.TextAndImage) && (val != this.model.imagePosition)) {
                this.element.empty();
                this.model.imagePosition = val;
                this._renderButtonNormal();
            }
        },
		 /**
         * To enable or disable the Right to Left behaviour 		
		 * @private
         */	 
        _setRTL: function (val) {
            if (val) {
                this.element.addClass("e-rtl");
            } else {
                this.element.removeClass("e-rtl");
            }
        },
		 /**
         * To enable or disable the button state		
		 * @private
         */	 
        _controlStatus: function (value) {
            //disable status
            if (!value) {
                this.disable();
            } else {
                this.enable();
            }
        },
		 /**
         * To configure the custom theme for button using cssClass property		
		 * @private
         */	 
        _setSkin: function (skin) {
            if (this.model.cssClass != skin) {
                this.element.removeClass(this.model.cssClass);
                this.element.addClass(skin);
            }
        },
		 /**
         * To initialize the button		
		 * @private
         */	 
        _initialize: function () {
            /// <summary>This function is  used to Initialize the Button Object</summary>
			if (this.element.is("input")) 
			{
			this.buttonType="inputButton";
			}
			else if ((this.element.is("a")) || (this.element.is("button"))) {
                this.buttonType = "tagButton";
            }
			else {
                this.element.removeClass("e-button");
            }
			if(this.element.attr("type"))
			{		
			this.model.type=this.element.attr("type");
			}
			else
			this._setType(this.model.type);
			
			 
			this._timeout = null;
        },

		 /**
         * Render Section For DifferentTypes		
		 * @private
         */	         
        _render: function () {
            /// <summary>This function is  used to Render the Button Object</summary>
            this._setSize(this.model.size);
            this._setHeight(this.model.height);
            this._setWidth(this.model.width);
            this._setRTL(this.model.enableRTL);
            this.element.addClass(this.model.cssClass + " e-btn e-select e-widget").attr("role", "button");
            if (this.buttonType == "inputButton") {
                this.element.addClass("e-txt");
                if ((this.model.text != null) && (this.model.text != "")) {
                    this.element.val(this.model.text);
                } else {
                    this.model.text = this.element.val();
                }
            } else { this._renderButtonNormal(); }
            this._roundedCorner(this.model.showRoundedCorner);
            if (this.model.text)
                this.element.attr("aria-describedby", this.model.text);
        },
		 /**
         * Render Section For Normal button		
		 * @private
         */	
        _renderButtonNormal: function () {
            if ((this.model.text == null) || (this.model.text == "")) {
                this.model.text = this.element.text();
            }
            this.element.empty();
            /*Image and Text*/
            this.textspan = ej.buildTag('span.e-btntxt', this.model.text);
            if (this.model.contentType.indexOf("image") > -1) {
                this.majorimgtag = ej.buildTag('span.e-icon ' + this.model.prefixIcon);
                this.minorimgtag = ej.buildTag('span.e-icon ' + this.model.suffixIcon);
                this.imgtxtwrap = ej.buildTag('div');
            }
            /*Rendering Option*/
            if (this.model.contentType == ej.ContentType.TextAndImage) {
				switch (this.model.imagePosition) {
                    case ej.ImagePosition.ImageRight:
                    	this.imgtxtwrap.append(this.textspan, this.majorimgtag);
						break;
					case ej.ImagePosition.ImageLeft:
                    	this.imgtxtwrap.append(this.majorimgtag, this.textspan);
						break;
					case ej.ImagePosition.ImageBottom:	
						this.majorimgtag.attr("style","display:inherit");
                    	this.imgtxtwrap.append(this.textspan, this.majorimgtag);
						break;
                	case ej.ImagePosition.ImageTop:
						this.majorimgtag.attr("style","display:inherit");
                    	this.imgtxtwrap.append(this.majorimgtag, this.textspan);
						break;
                }
				this.element.append(this.imgtxtwrap);
            } else if (this.model.contentType == ej.ContentType.ImageTextImage) {
                this.imgtxtwrap.append(this.majorimgtag, this.textspan, this.minorimgtag);
                this.element.append(this.imgtxtwrap);
            } else if (this.model.contentType == ej.ContentType.ImageBoth) {
                this.imgtxtwrap.append(this.majorimgtag, this.minorimgtag);
                this.element.append(this.imgtxtwrap);
            } else if (this.model.contentType == ej.ContentType.ImageOnly) {
                this.imgtxtwrap.append(this.majorimgtag);
                this.element.append(this.imgtxtwrap);
            } else {
                this.element.addClass("e-txt");
                this.element.text(this.model.text);
            }
        },
		 /**
         * To enable or disable the rounded corner behaviour 		
		 * @private
         */	
        _roundedCorner: function (value) {
            value == true ? this.element.addClass('e-corner-all') : this.element.removeClass('e-corner-all');
        },
         /**
         * Wiring the events to button control		
		 * @private
         */			 
        _wireEvents: function (val) {
            /// <summary>This function is  used to Apply Events in the Button Object</summary>
            if (val) {
                this._on(this.element, "mousedown", this._btnRepatMouseClickEvent);
                this._on($(document), 'mouseup', this._mouseUpClick);
                this._on(this.element, "keyup", this._btnRepatKeyUpEvent);
                this._on($(document), "keypress", this._btnRepatKeyDownEvent);
                
            }
                this._on(this.element, "click", this._btnMouseClickEvent);
        },
		 /**
         * Section For handle the click event
		 * @private
         */	
        _btnMouseClickEvent: function (e) {
            var self = this;
            if (!self.element.hasClass("e-disable")) {
                var args = { status: self.model.enabled };
                self._trigger("click", args);
            }
        },
		/**
         * Section For handle repeat button click action.
		 * @private
         */	
        _btnRepatMouseClickEvent: function (e) {
            var self = this;
            if (!self.element.hasClass("e-disable")) {
                var args = { status: self.model.enabled };
                if((e.button == 0) ||(e.which == 1)){

                    self._timeout = setInterval(function () { self._trigger("click", { target: e.currentTarget, status: self.model.enabled } ); }, this.model.timeInterval);
                }
            }
        },
		/**
         * Section to track the mouse up event		
		 * @private
         */	
        _mouseUpClick: function (event) {
            clearTimeout(this._timeout);
        },
		/**
         * Section to watch the repeat key down action.		
		 * @private
         */
        _btnRepatKeyDownEvent: function (e) {
            var self = this;
            if (!self.element.hasClass("e-disable")) {
                var args = { status: self.model.enabled };
                if ((e.keyCode ==32 ) || (e.keyCode ==13 )) {
                    self._trigger("click", args);
                }
            }
        },
		/**
         * Section to watch the repeat key up action.		
		 * @private
         */
        _btnRepatKeyUpEvent: function (e) {
           if ((e.keyCode == 32) || (e.keyCode == 13)) {
                clearTimeout(this._timeout);
            }
        },
    });
	
	/**
	 * Enum for button content mode.	 
	 * @enum {string}
	 * @global 
	 */
    ej.ContentType = { 
			/**  Supports only for text content only */
			TextOnly: "textonly", 
			/** Supports only for image content only */
			ImageOnly: "imageonly", 
			/** Supports image for both ends of the button */
			ImageBoth: "imageboth", 
			/** Supports image with the text content */
			TextAndImage: "textandimage", 
			/** Supports image with both ends of the text */
			ImageTextImage: "imagetextimage" 
			};
	
	/**
	 * Enum for Button positioning	 
	 * @enum {string}
	 * @global 
	 */
    ej.ImagePosition = { 
			/**  support for aligning text in left and image in right. */
			ImageRight: "imageright", 
			/**  support for aligning text in right and image in left. */
			ImageLeft: "imageleft",
			/**  support for aligning text in bottom and image in top. */
			ImageTop: "imagetop", 
			/**  support for aligning text in top and image in bottom. */
			ImageBottom: "imagebottom"
			};
	/**
	 * Enum for various button sizes	 
	 * @enum {string}
	 * @global 
	 */
    ej.ButtonSize = { 
			/**  Creates button with inbuilt default size height, width specified */
			Normal : "normal",
			/**  Creates button with inbuilt mini size height, width specified */
			Mini: "mini", 
			/**  Creates button with inbuilt small size height, width specified */
			Small: "small",
			/**  Creates button with inbuilt medium size height, width specified */
			Medium:"medium", 
			/**  Creates button with inbuilt large size height, width specified */
			Large: "large"
    };
    /**
	 * Enum for various button types	 
	 * @enum {string}
	 * @global 
	 */
	ej.ButtonType = { 
				/**  Creates button with inbuilt button type specified */
			Button : "button",
			/**  Creates button with inbuilt reset type specified */
			Reset: "reset", 
			/**  Creates button with inbuilt submit type specified */
			Submit: "submit"};
})(jQuery, Syncfusion);
;
/**
* @fileOverview Calculate engine to perform the calculatin
* @copyright Copyright Syncfusion Inc. 2001 - 2013. All rights reserved.
*  Use of this code is subject to the terms of our license.
*  A copy of the current license can be obtained at any time by e-mailing
*  licensing@syncfusion.com. Any infringement will be prosecuted under
*  applicable laws. 
* @version 12.3 
* @author <a href="mailto:licensing@syncfusion.com">Syncfusion Inc</a>
*/
(function ($, ej, undefined) {
    
    
	/**
    * @namespace ej
	* @class ejOlapGrid
    * @requires jquery-1.10.2.min.js
    * @requires jquery.easing.1.3.min.js
    * @requires ej.core.js
    * @requires ej.calculate.js
	* @classdesc Custom engine to perfom calculation like excel sheet
	* @example 
    * &lt;script&gt;
	* // Create calcEngine with grid data
    * var calcObj = new CalcEngine($("#Grid").data("ejGrid"));
	* &lt;/script&gt;
	*/
    CalcEngine = function (parentObject) {
        this.parentObject = parentObject;
        this._useFormulaValues = false;
        this._multiTick = false;
        this._isErrorString = false;
        this._ignoreBracet = false;
        this._libraryFunctions = new HashTable();
        this._customlibraryFunctions = new HashTable();
        this._refreshedCells = new HashTable();
        this._treatStringsAsZero = true;
        this._supportLogicalOperators = false;
        this._isRangeOperand = false;
        this._uniqueStringMarker = String.fromCharCode(127);
        this._isParseArgumentSeparator = false;
        this._isParseDecimalSeparatorChanged = false;
        this._validPrecedingChars = " (+-*/^&<>=";
        this._validFunctionNameChars = "_";
        this.bMARKER = String.fromCharCode(146);
        this.bMARKER2 = this.bMARKER + this.bMARKER;
        this.tic = "\"";
        this._parseDecimalSeparator = '.';
        this.parseDateTimeSeparator = '/';
        this._parseArgumentSeparator = ',';
        this._rightBracket = String.fromCharCode(131);
        this._leftBracket = String.fromCharCode(130);
        this.braceLeft = "{";
        this.braceRight = "}";
        this._braceRightNLeft = ")(";
        this.computedValueLevel = 0;
        this._circCheckList = [];
        this._maximumRecursiveCalls = 100;
        this._sortedSheetNames = null;
        this.trueValueStr = "TRUE";
        this.falseValueStr = "FALSE";
        this.useDatesInCalcs = false;
        this.sheetToken = '!';
        this.namedRanges = null;
        this._namedRangesNonScoped = null;
        this.char_add = '+';
        this.char_and = 'i';
        this.char_ANDop = String.fromCharCode(140);
        this.char_divide = '/';
        this.char_ELSEop = String.fromCharCode(144);
        this.char_EM = 'r';
        this.char_EP = 'x';
        this.char_equal = '=';
        this.char_greater = '>';
        this.char_greatereq = 'h';
        this.char_IFop = String.fromCharCode(142);
        this.char_less = '<';
        this.char_lesseq = 'f';
        this.char_multiply = '*';
        this.char_noequal = 'p';
        this.char_NOTop = String.fromCharCode(145);
        this.char_or = 'w';
        this.char_ORop = String.fromCharCode(139);
        this.char_subtract = '-';
        this.char_THENop = String.fromCharCode(143);
        this.char_XORop = String.fromCharCode(141);
        this.chartic = "'"[0];
        this._string_and = "&";
        this._string_E = "E";
        this._string_EM = "E-";
        this._string_empty = "";
        this._string_EP = "E+";
        this._string_fixedreference = "$";
        this._string_greatereq = ">=";
        this._string_lesseq = "<=";
        this._string_noequal = "<>";
        this._string_or = "^";
        this.token_add = 'a';
        this.token_and = 'c';
        this.token_ANDop = String.fromCharCode(133);
        this.token_divide = 'd';
        this.token_ELSEop = String.fromCharCode(137);
        this.token_EM = 'v';
        this.token_EP = 't';
        this.token_equal = 'e';
        this.token_greater = 'g';
        this.token_greatereq = 'j';
        this.token_IFop = String.fromCharCode(135);
        this.token_less = 'l';
        this.token_lesseq = 'k';
        this.token_multiply = 'm';
        this.token_noequal = 'o';
        this.token_NOTop = String.fromCharCode(138);
        this.token_or = String.fromCharCode(126);
        this.token_ORop = String.fromCharCode(132);
        this.token_subtract = 's';
        this.token_THENop = String.fromCharCode(136);
        this.token_XORop = String.fromCharCode(134);
        this.tokens = [
            this.token_add, this.token_subtract, this.token_multiply, this.token_divide, this.token_less,
            this.token_greater, this.token_equal, this.token_lesseq, this.token_greatereq, this.token_noequal, this.token_and, this.token_or];
        this._dateTime1900 = new Date(1900, 1, 1, 0, 0, 0);
        this._preserveLeadingZeros = false;
        this._ignoreCellValue = false;
        this._errorStrings = null;
        this._cell = "";
        this._iterationMaxCount = 0;
        this._supportRangeOperands = false;
        this._ensureIFCallDuringShortCircuit = false;
        this._allowShortCircuitIFs = false;
        this._processUpperCaseFormula = "";
        this._processUpperCaseIvalue = 0;
        this._processUpperCaseSheet = "";
        this._markerChar = '`';
        this._rowMaxCount = -1;
        this._columnMaxCount = -1;
        this._isInteriorFunction = false;
        this._tempSheetPlaceHolder = String.fromCharCode(133);
        this.sheetFamilyID = 0;
        this._supportsSheetRanges = true;
        this._markers = "()+-*/=><.,!";
        this._formulaInfoTable = null;
        this._dependentFormulaCells = null;
        this._dependentCells = new HashTable();
        this._calculatingSuspended = false;
        this._inAPull = false;
        this._useDatesInCalcs = false;
        this._excelLikeComputations = false;
        this._rethrowLibraryComputationExceptions = false;
        this.formulaErrorStrings = [
            "binary operators cannot start an expression",
            "cannot parse",
            "bad library",
            "invalid char in front of",
            "number contains 2 decimal points",
            "expression cannot end with an operator",
            "invalid characters following an operator",
            "invalid character in number",
            "mismatched parentheses",
            "unknown formula name",
            "requires a single argument",
            "requires 3 arguments",
            "invalid Math argument",
            "requires 2 arguments",
            "#NAME?",
            "too complex",
            "circular reference: ",
            "missing formula",
            "improper formula",
            "invalid expression",
            "cell empty",
            "bad formula",
            "empty expression",
            "",
            "mismatched string quotes",
            "wrong number of arguments",
            "invalid arguments",
            "iterations do not converge",
            "Control named '{0}' is already registered",
            "Calculation overflow",
            "Missing sheet"
        ];
        this._parseDateTimeSeparator = '/';
        this._millisecondsOfaDay = 24 * 60 * 60 * 1000;
        this.treat1900AsLeapYear = false;
        this._oaDate = new Date(1899, 11, 30);
        this._saveStringsText = "";
        this._processedCells = [];
        this.ignoreValueChanged = false;
        this._breakedFormulaCells = [];
        this._tempBreakedFormulaCells = [];
        this._lockDependencies = false;
        this._useDependencies = false;
        this._inHandleIterations = false;
        this._inRecalculateRange = false;
        this._useNoAmpersandQuotes = false;
        this._calcID = 0;
        this._operators_cannot_start_an_expression = 0;
        this._reservedWord_AND = 1;
        this._reservedWord_XOR = 2;
        this._reservedWord_IF = 3;
        this._number_contains_2_decimal_points = 4;
        this._reservedWord_ELSE = 5;
        this._reservedWord_NOT = 6;
        this._invalid_char_in_number = 7;
        this._mismatched_parentheses = 8;
        this._unknown_formula_name = 9;
        this._requires_a_single_argument = 10;
        this._requires_3_args = 11;
        this._invalid_Math_argument = 12;
        this._requires_2_args = 13;
        this._bad_index = 14;
        this._too_complex = 15;
        this._circular_reference_ = 16;
        this._missing_formula = 17;
        this._improper_formula = 18;
        this._invalid_expression = 19;
        this._cell_empty = 20;
        this._bad_formula = 21;
        this._empty_expression = 22;
        this._virtual_mode_required = 23;
        this._mismatched_tics = 24;
        this._wrong_number_arguments = 25;
        this._invalid_arguments = 26;
        this._iterations_dont_converge = 27;
        this._calculation_overflow = 29;
        this._already_registered = 28;
        this._missing_sheet = 30;
        this._alwaysComputeDuringRefresh = true;
        this._libraryComputationException = null;
        this._dependencyLevel = 0;
        this._isDisposed = undefined;
        this._forceRefreshCall = false;
        this.grid = this.parentObject;
        this._enableLookupTableCaching = 0 /* None */;
        this._lookupTables = new HashTable();

        this._addFunction = function (name, func) {
            name = name.toUpperCase();
            if (this._libraryFunctions.getItem(name) == undefined) {
                this._libraryFunctions.add(name, func);
                return true;
            }
            return false;
        };
        this._addToFormulaDependentCells = function (s) {
            var cell1 = this.cell;
            var family = CalcEngine.getSheetFamilyItem(this.grid);
            if (family.sheetNameToParentObject != null && cell1.indexOf(this.sheetToken) == -1) {
                var token = family.parentObjectToToken.getItem(this.grid);
                cell1 = token + cell1;
            }

            if (!this.getDependentFormulaCells().containsKey(cell1)) {
                this.getDependentFormulaCells().add(cell1, new HashTable());
                this.getDependentFormulaCells().getItem(cell1).add(s, s);
            } else if (!(this.getDependentFormulaCells().getItem(cell1)).containsKey(s)) {
                this.getDependentFormulaCells().getItem(cell1).add(s, s);
            }
        };
        this._arrayRemove = function (array, value) {
            var index = null;

            while ((index = array.indexOf(value)) !== -1)
                array.splice(index, 1);

            return array;
        };
        this._canGetRowIndex = function (s) {
            var i = 0;
            if (i < s.length && s[i] == this.sheetToken) {
                i++;
                while (i < s.length && s[i] != this.sheetToken) {
                    i++;
                }
                i++;
            }

            while (i < s.length && this._isLetter(s[i])) {
                i++;
            }

            if (i < s.length) {
                if (this._isDigit(s[i]))
                    return true;
                else
                    return false;
            }
            return false;
        };
        this._checkAddNonScopedNamedRange = function (key) {
            var loc = -1;
            if ((loc = key.indexOf('!')) > -1) {
                var key1 = key.substring(loc + 1);
                if (!this._namedRangesNonScoped.containsKey(key1)) {
                    this._namedRangesNonScoped.add(key1, this.namedRanges.getItem(key));
                }
            }
        };
        this._checkHasCharBeforeNumber = function (tempFormula) {
            var check = false;
            for (var x = tempFormula.length - 1; x > 0; x--) {
                if (this._isLetter(tempFormula[x])) {
                    check = true;
                    break;
                }
            }
            return check;
        };
        this._checkIfScopedRange = function (text, scopedRange) {
            scopedRange = "";
            var b = "NaN";
            var id = this.getSheetID(this.grid);
            var sheet = CalcEngine.getSheetFamilyItem(this.grid);

            if (text[0] == this.sheetToken.toString()) {
                var i = text.indexOf(this.sheetToken, 1);
                if (i > 1 && !isNaN(parseInt(text.substring(1, i - 1)))) {
                    //id reset to proper value
                    text = text.substring(i + 1);
                }
            }

            var token = '!' + id.toString() + '!';

            if (sheet.sheetNameToToken == null)
                return b;
            var sheetNameKeys = sheet.sheetNameToToken.keys();
            for (var name in sheetNameKeys) {
                if (sheet.sheetNameToToken.getItem(sheetNameKeys[name]).toString() == token) {
                    var s = (sheetNameKeys[name] + '!' + text).toUpperCase();
                    if (this.getNamedRanges().containsKey(s)) {
                        scopedRange = (this.getNamedRanges().getItem(s)).toUpperCase();
                        b = scopedRange;
                    }
                }
            }
            return b;
        };
        this._checkUnderTolerance = function (d, oldValue) {
            if (Math.abs(oldValue) > this._absoluteZero) {
                return Math.abs((d - oldValue) / oldValue) < this._iterationMaxTolerance;
            } else {
                return Math.abs(d - oldValue) < this._iterationMaxTolerance;
            }
        };
        this._combineStack = function (formula, i, _tempStack) {
            var combineString = "";
            if (_tempStack.length == 2) {
                var j = 3;
                var checkString = formula.substring(i);
                while (j > 1) {
                    var popStringValue = _tempStack.pop().toString();
                    combineString = (popStringValue == (this.tic + this.tic)) ? combineString + this.tic : popStringValue + combineString;
                    checkString = (j == 3) ? popStringValue : checkString;
                    j--;
                }
                if (_tempStack[0] == (this.tic + this.tic))
                    _tempStack.pop();
                combineString = (formula.length == i && this._isTextEmpty(checkString.split(this.tic).join(""))) ? combineString + checkString : combineString;

                _tempStack.push(combineString);
            }
            return _tempStack;
        };
        this._computeMath = function (args, func) {
            var ans = 0;
            var s1;
            var isDegree = false;

            ////Only work with first arg.
            if (args.length > 0) {
                args = this.splitArgsPreservingQuotedCommas(args)[0];
            }
            var indexArray = [this.getParseArgumentSeparator(), ':'];

            ////parsed formula
            if (args.length > 0 && ((!this._isLetter(args[0]) && args[0] != this.sheetToken && args[0] != this.bMARKER) || (args[0] == 'u' && this._isDigit(args[1]))) && this._indexOfAny(args, indexArray) == -1) {
                ////Swap out unary minus.
                args = (args.split('u').join('-')).split("n").join("");

                var d1 = this._parseDouble(args);

                if (!isNaN(d1)) {
                    ans = func(d1);
                } else {
                    return this.getErrorStrings()[1].toString();
                }
            } else if (args.length > 0 && (args[0] == this.bMARKER || args[0] == 'u' || args[0] == 'n' || this._indexOfAny(args, this.tokens) > -1)) {
                ////parsed formula
                args = args.split('{').join('(');
                args = args.split('}').join(')');
                var rightparam = "";
                var decimalPI = (Math.PI).toString();
                try  {
                    if (args.indexOf(decimalPI) > -1) {
                        s1 = this._substring(args, 1, args.indexOf(decimalPI) - 2);
                        s1 = (this._isTextEmpty(s1)) ? "1" : s1;
                        rightparam = (this._indexOfAny(args, this.tokens) > -1) ? args.split(decimalPI.toString())[1] : "1";
                        isDegree = true;
                    } else
                        s1 = this.computedValue(args);
                } catch (ex) {
                    if (this._rethrowLibraryComputationExceptions && this.getLibraryComputationException() != null) {
                        throw this.getLibraryComputationException();
                    }

                    return ex;
                }
                if (isDegree) {
                    var leftparam = this.computedValue(s1.toString());
                    if (rightparam.indexOf("180") > -1) {
                        d1 = this._parseDouble(leftparam);
                        if (d1)
                            ans = func(d1 * (Math.PI / 180));
                        else {
                            return this.getErrorStrings()[1].toString();
                        }
                    } else {
                        var temp, d2;
                        temp = this.computedValue(leftparam + rightparam);
                        d2 = this._parseDouble(temp);
                        if (!isNaN(d2))
                            ans = func(d2 * (Math.PI));
                        else {
                            return this.getErrorStrings()[1].toString();
                        }
                    }
                } else {
                    d1 = this._parseDouble(s1);
                    if (!isNaN(d1))
                        ans = func(d1);
                    else {
                        return this.getErrorStrings()[1].toString();
                    }
                }
            } else {
                ////Swap out unary minus.
                args = args.split('u').join('-');
                var cells = this.getCellsFromArgs(args);
                for (var s in cells) {
                    try  {
                        s1 = this.getValueFromArg(cells[s]);
                    } catch (ex) {
                        if (this._rethrowLibraryComputationExceptions && this.getLibraryComputationException() != null) {
                            throw this.getLibraryComputationException();
                        }

                        return ex;
                    }

                    if (s1.length > 0) {
                        d1 = this._parseDouble(s1);
                        if (!isNaN(d1)) {
                            ans = func(d1);
                        } else
                            return this.getErrorStrings()[1].toString();
                    }

                    break;
                }
            }
            if (this.computeIsErr(ans.toString()) == this.trueValueStr) {
                return this.getErrorStrings()[4].toString();
            }
            return ans.toString();
        };
        this._computeInteriorFunctions = function (formula) {
            try  {
                if (this._isTextEmpty(formula)) {
                    return formula;
                }

                this.computeFunctionLevel++;

                ////int q = formula.LastIndexOf('q');
                var q = this._findLastqNotInBrackets(formula);
                while (q > 0) {
                    var last = formula.substring(q).indexOf(this._rightBracket);
                    if (last == -1) {
                        throw this.formulaErrorStrings[this._bad_formula];
                    }
                    this._isInteriorFunction = true;
                    var s = this._substring(formula, q, last + 1);
                    s = this.computedValue(s);
                    if (!(s == "") && s[0] == this.tic[0] && s[s.length - 1] == this.tic[0]) {
                        var newS = this._substring(s, 1, s.length - 2);
                        if (newS.indexOf(this.tic) != -1) {
                            this._multiTick = true;
                            newS = newS.split(this.tic).join("|");
                        }
                        s = this.tic + newS + this.tic;
                    }
                    s = this._markupResultToIncludeInFormula(s);
                    if (!this._isInteriorFunction)
                        formula = formula.substring(0, q) + s.split(this.tic).join("") + formula.substring(q + last + 1);
                    else
                        formula = formula.substring(0, q) + s + formula.substring(q + last + 1);
                    this._isInteriorFunction = false;
                    q = this._findLastqNotInBrackets(formula);
                }
            } catch (ex) {
                return ex;
            } finally {
                this.computeFunctionLevel--;
            }

            return formula;
        };
        this._findAndCheckPrecedingChar = function (copy, op, loc) {
            var loc1 = copy.indexOf(op, loc);
            if (loc1 > 0) {
                while (loc1 > -1 && this.getValidPrecedingChars().indexOf(copy[loc1 - 1]) == -1) {
                    loc1 = copy.indexOf(op, loc1 + 1);
                }
            }
            this._findAndCheckPrecedingCharCopy = copy;
            return loc1;
        };
        this._findLastNonQB = function (text) {
            var ret = -1;
            if (text.indexOf(this.bMARKER) > -1) {
                var bracketLevel = 0;
                for (var i = text.length - 1; i >= 0; --i) {
                    if (text[i] == this._rightBracket) {
                        bracketLevel--;
                    } else if (text[i] == this._leftBracket) {
                        bracketLevel++;
                    } else if (text[i] == this.bMARKER && bracketLevel == 0) {
                        ret = i;
                        break;
                    }
                }
            }

            return ret;
        };
        this._findLastqNotInBrackets = function (s) {
            var found = -1;
            var lastBracket = false;
            var i = s.length - 1;
            while (i > -1) {
                if (s[i] == 'q' && lastBracket) {
                    found = i;
                    break;
                }

                if (s[i] == this._leftBracket) {
                    lastBracket = true;
                } else if (s[i] == this._rightBracket) {
                    lastBracket = false;
                }

                i--;
            }

            return found;
        };
        this._findNextSeparator = function (formula, location) {
            var qCount = 0;
            var found = false;
            while (!found && location < formula.length) {
                if (formula[location] == 'q') {
                    qCount++;
                } else if (formula[location] == this._rightBracket) {
                    qCount--;
                } else if (qCount == 0 && formula[location] == this.getParseArgumentSeparator()) {
                    found = true;
                    location--;
                }

                location++;
            }

            if (found)
                return location.toString();
            else
                return "NaN";
        };
        this._findNonQB = function (text) {
            var ret = -1;
            if (text.indexOf(this.bMARKER) > -1) {
                var bracketLevel = 0;
                for (var i = 0; i < text.length; ++i) {
                    if (text[i] == this._rightBracket) {
                        bracketLevel--;
                    } else if (text[i] == this._leftBracket) {
                        bracketLevel++;
                    } else if (text[i] == this.bMARKER && bracketLevel == 0) {
                        ret = i;
                        break;
                    }
                }
            }

            return ret;
        };
        this._findRightBracket = function (formula, location) {
            var qCount = 0;
            var found = false;
            while (!found && location < formula.length) {
                if (formula[location] == 'q') {
                    qCount++;
                } else if (qCount == 0 && formula[location] == this._rightBracket) {
                    found = true;
                    location--;
                } else if (formula[location] == this._rightBracket) {
                    qCount--;
                }

                location++;
            }

            if (found)
                return location.toString();
            else
                return "NaN";
        };
        this._fromOADate = function (doubleNumber) {
            var result = new Date();
            result.setTime((doubleNumber * this._millisecondsOfaDay) + Date.parse(this._oaDate));
            return result;
        };
        this._getDateFromSerialDate = function (days) {
            if (this.treat1900AsLeapYear && days > 59) {
                days -= 1;
            }

            return new Date(this._oaDate.setDate(this._oaDate.getDate() + days));
        };
        this._getDoubleArray = function (range) {
            var d;
            var s1 = "";
            var x = [];

            range = this.adjustRangeArg(range);
            var ranges = this.splitArgsPreservingQuotedCommas(range);
            for (var r in ranges) {
                ////cell range
                if (ranges[r].indexOf(':') > -1) {
                    var cells = this.getCellsFromArgs(ranges[r]);
                    for (var s in cells) {
                        s1 = this.getValueFromArg(cells[s]);

                        if (s1.length > 0) {
                            d = this._parseDouble(s1);
                            if (!isNaN(d)) {
                                x.push(Number(d));
                            } else {
                                x.push(0.0);
                            }
                        }
                    }
                } else {
                    s1 = this.getValueFromArg(ranges[r]);
                    d = this._parseDouble(s1);
                    if (!isNaN(Number(d))) {
                        x.push(d);
                    } else {
                        x.push(0.0);
                    }
                }
            }
            return x;
        };
        this._getDoubleArrayA = function (range) {
            var d;
            var s1 = "";
            var x = [];

            range = this.adjustRangeArg(range);
            var ranges = this.splitArgsPreservingQuotedCommas(range);
            for (var r in ranges) {
                ////cell range
                if (ranges[r].indexOf(':') > -1) {
                    var cells = this.getCellsFromArgs(ranges[r]);
                    for (var s in cells) {
                        s1 = this.getValueFromArg(cells[s]);

                        if (s1.length > 0) {
                            if (s1 == this.trueValueStr) {
                                d = 0.0;
                            } else {
                                d = this._parseDouble(s1);
                                if (isNaN(Number(d))) {
                                    d == 0.0;
                                }
                            }
                            x.push(d);
                        }
                    }
                } else {
                    s1 = this.getValueFromArg(ranges[r]);
                    if (s1 == this.trueValueStr) {
                        d = 0.0;
                    } else {
                        d = this._parseDouble(Number(s1));
                        if (isNaN(d)) {
                            d == 0.0;
                        }
                    }
                    x.push(d);
                }
            }
            return x;
        };
        this._getFormulaArrayBounds = function (currentCell, arrayHeight, arrayWidth) {
            var currentColIndex = this.colIndex(this.cell);
            var currentRowIndex = this.rowIndex(this.cell);

            var formulaText = "";
            var formula = this.getFormulaInfoTable().containsKey(this.cell) ? this.getFormulaInfoTable().getItem(this.cell) : null;
            if (formula != null)
                formulaText = formula.getFormulaText();
            else if (this.parentObject.getValueRowCol == undefined)
                formulaText = this.getValueRowCol(this.getSheetID(this.grid) + 1, currentRowIndex, currentColIndex).toString();
            else
                formulaText = this.parentObject.getValueRowCol(this.getSheetID(this.grid)+1, currentRowIndex, currentColIndex).toString();
            var startRowIndex = 1, startColIndex = 1;
            for (var x = 1; x <= arrayHeight; x++) {
                if ((currentColIndex - x) > 0) {
                    var tempCell = this._getSheetTokenFromReference(this.cell) + RangeInfo.getAlphaLabel(currentColIndex - x) + currentRowIndex.toString();
                    var tempFormulaText = "";
                    var formula1 = this.getFormulaInfoTable().containsKey(tempCell) ? this.getFormulaInfoTable().getItem(tempCell) : null;
                    if (formula1 != null)
                        tempFormulaText = formula1.getFormulaText();
                    else if (this.parentObject.getValueRowCol == undefined)
                        tempFormulaText = this.getValueRowCol(this.getSheetID(this.grid) + 1, currentRowIndex, currentColIndex - x).toString();
                    else
                        tempFormulaText = this.parentObject.getValueRowCol(this.getSheetID(this.grid) + 1, currentRowIndex, currentColIndex - x).toString();
                    startColIndex = currentColIndex - x;
                    if (formulaText != null && tempFormulaText != formulaText) {
                        startColIndex++;
                        break;
                    }
                }
            }
            for (var y = 1; y <= arrayWidth; y++) {
                if ((currentRowIndex - y) > 0) {
                    var tempCell = this._getSheetTokenFromReference(this.cell) + RangeInfo.getAlphaLabel(startColIndex) + (currentRowIndex - y).toString();
                    var tempFormulaText = "";
                    var formula1 = this.getFormulaInfoTable().containsKey(tempCell) ? this.getFormulaInfoTable().getItem(tempCell) : null;
                    if (formula1 != null)
                        tempFormulaText = formula1.getFormulaText();
                    else if (this.parentObject.getValueRowCol == undefined)
                        tempFormulaText = this.getValueRowCol(this.getSheetID(this.grid) + 1, currentRowIndex - y, startColIndex).toString();
                    else
                        tempFormulaText = this.parentObject.getValueRowCol(this.getSheetID(this.grid) + 1, currentRowIndex - y, startColIndex).toString();
                    startRowIndex = currentRowIndex - y;
                    if (formulaText != null && tempFormulaText != formulaText) {
                        startRowIndex++;
                        break;
                    }
                }
            }

            this._getFormulaArrayBoundsfirstRowIndex = startRowIndex;
            this._getFormulaArrayBoundsfirstColIndex = startColIndex;
            this._getFormulaArrayBoundslastRowIndex = startRowIndex + arrayHeight;
            this._getFormulaArrayBoundslastColIndex = startColIndex + arrayWidth;
            return true;
        };
        this._getSerialDateFromDate = function (y, m, d) {
            var days = 0;
            if (y < 1900) {
                y += 1900;
            }

           
            var isValidMonth = false;
            while (!isValidMonth) {
                while (m > 12) {
                    y++;
                    m -= 12;
                }
                isValidMonth = true;
                var x = new Date(y, m, 1, -1).getDate();
                while (d > x) {
                    x = new Date(y, m, 1, -1).getDate();
                    m++;
                    d -= x;
                    isValidMonth = false;
                }
                if (d < 1) {
                    m--;
                    x = new Date(y, m, 1, -1).getDate();
                    d = x - d;
                }
            }
            var dateTime = Date.parse(y.toString() + this.getParseDateTimeSeparator() + m.toString() + this.getParseDateTimeSeparator() + d.toString());
            if (!isNaN(dateTime)) {
                days = this._toOADate(new Date(dateTime));
            }
            if (this.treat1900AsLeapYear && days > 59) {
                days += 1;
            }
            return days;
        };
        this._getSerialDateTimeFromDate = function (dt) {
            var d = this._toOADate(dt);

            if (this.treat1900AsLeapYear && d > 59) {
                d += 1;
            }

            return d;
        };
        this._getSheetTokenFromReference = function (r) {
            var sheet = "";
            if (r.length > 2 && r[0] == this.sheetToken) {
                var i = 1;
                sheet = this.sheetToken;
                while (i < r.length && r[i] != this.sheetToken) {
                    sheet += r[i];
                    i++;
                }

                sheet += this.sheetToken;
            }

            return sheet;
        };
        this._getValueComputeFormulaIfNecessary = function (row, col, grd) {
            var alreadyComputed = false;

            var formula = this.getFormulaInfoTable().getItem(this.cell);
            var o;
            if (this.parentObject.getValueRowCol == undefined)
                o = this.getValueRowCol(this.getSheetID(grd) + 1, row, col);
            else
                o = this.parentObject.getValueRowCol(this.getSheetID(grd) + 1, row, col);

            var val = (o != null && o != "") ? o.toString() : "";

            if (val[val.length - 1] == ("}") && val[0] == ("{")) {
                val = this._substring(val, 1, val.length - 2);
            }

            if ((val == "" && (formula == null || formula == undefined)) || (val != "" && val[0] != this.getFormulaCharacter() && val[val.length - 1] != "%")) {
                if (formula != null && val == formula.getFormulaValue())
                    return formula.getFormulaValue();
                else
                    return val;
            }

            if (val.length > 0 && val[0] == this.getFormulaCharacter()) {
                if (formula != null) {
                    ////will be set later..
                    //// val = formula.FormulaValue;
                    ////val = ComputeFormula(formula.ParsedFormula);
                } else {
                    formula = new FormulaInfo();
                    formula.setFormulaText(o.toString());

                    if (!this.getDependentFormulaCells().containsKey(this.cell)) {
                        this.getDependentFormulaCells().add(this.cell, new HashTable());
                    }

                    var compute = true;
                    try  {
                        formula.setParsedFormula(this.parseFormula(val));
                    } catch (ex) {
                        if (this._inAPull) {
                            val = ex;
                            formula = null;
                        } else {
                            formula.setFormulaValue(ex);
                        }
                        compute = false;
                    }

                    if (compute) {
                        formula.setFormulaValue(this.computeFormula(formula.getParsedFormula()));
                        alreadyComputed = true;
                    }
                    if (formula != null) {
                        formula.calcID = this._calcID;
                        if (!this.getFormulaInfoTable().containsKey(this.cell))
                            this.getFormulaInfoTable().add(this.cell, formula);
                        val = formula.getFormulaValue() != null ? formula.getFormulaValue() : "";
                    }
                }
            }

            if (formula != null) {
                if (this.getUseFormulaValues() || this.getCalculatingSuspended() && (!this._inAPull || alreadyComputed)) {
                    val = formula.getFormulaValue() != null ? formula.getFormulaValue() : "";
                } else if (!alreadyComputed) {
                    if (this._calcID == formula.calcID) {
                        val = formula.getFormulaValue();
                    } else {
                        val = this.computeFormula(formula.getParsedFormula());
                        formula.setFormulaValue(val);
                        formula.calcID = this._calcID;
                    }
                }
                if (this.getTreatStringsAsZero() && val == "") {
                    return "0";
                }
            }

            if (val == "" || val == undefined) {
                val = "";
            }

            var d = this._parseDouble(val.substring(0, val.length - 1));

            if (val[val.length - 1] == ("%") && !isNaN(d)) {
                val = (Number(d) / 100).toString();
            }
            return val;
        };
        this._handleEmbeddedEs = function (text) {
            var j = 0;
            while (j > -1 && ((j = text.indexOf(this._string_EP, j)) > -1)) {
                if (this._notInBlock(text, j)) {
                    var left = j;
                    while (left > 0 && (this._isDigit(text[left - 1]) || text[left - 1] == this.getParseDecimalSeparator())) {
                        left--;
                    }

                    if (left != j && (left == 0 || !this._isUpperChar(text[left - 1]))) {
                        var right = j + this._string_EP.length;
                        while (right < text.length && this._isDigit(text[right])) {
                            right++;
                        }

                        if (right != j + this._string_EP.length) {
                            text = text.substring(0, j) + this.char_EP + this._substring(text, j + this._string_EP.length);
                        }
                    }
                }

                j += 1;
            }

            j = 0;
            while (j > -1 && ((j = text.indexOf(this._string_EM, j)) > -1)) {
                if (this._notInBlock(text, j)) {
                    var left = j;
                    while (left > 0 && (this._isDigit(text[left - 1]) || text[left - 1] == this.getParseDecimalSeparator())) {
                        left--;
                    }

                    if (left != j && (left == 0 || !this._isUpperChar(text[left - 1]))) {
                        var right = j + this._string_EM.length;
                        while (right < text.length && this._isDigit(text[right])) {
                            right++;
                        }

                        if (right != j + this._string_EM.length) {
                            text = text.substring(0, j) + this.char_EM + this._substring(text, j + this._string_EM.length);
                        }
                    }
                }

                j += 1;
            }

            j = 0;

            while (j > -1 && ((j = text.indexOf(this._string_E, j)) > -1) && text[0] != this.bMARKER) {
                if (this._notInBlock(text, j)) {
                    var left = j;
                    while (left > 0 && (this._isDigit(text[left - 1]) || text[left - 1] == this.getParseDecimalSeparator())) {
                        left--;
                    }

                    if (left != j && (left == 0 || !this._isUpperChar(text[left - 1]))) {
                        var right = j + this._string_E.length;
                        while (right < text.length && this._isDigit(text[right])) {
                            right++;
                        }

                        if (right != j + this._string_E.length && (left == -1 || !this._isUpperChar(text[left]))) {
                            text = text.substring(0, j) + this.char_EP + this._substring(text, j + this._string_E.length);
                        }
                    }
                }

                j += 1;
            }

            return text;
        };
        this._handleIterations = function (formula) {
            this._inHandleIterations = true;
            var oldValue = 0;
            var d = 0;
            var count = 1;
            var first = true;
            while (count < this._iterationMaxCount - 1 && (first || !this._checkUnderTolerance(d, oldValue))) {
                first = false;
                if (!this.getIterationValues().containsKey(this.cell)) {
                    this.getIterationValues().add(this.cell, "0");
                }

                this.getIterationValues().add(this.cell, (formula.getFormulaValue() == "" ? "0" : formula.getFormulaValue()));

                formula.setFormulaValue(this.computeFormula(formula.getParsedFormula()));
                oldValue = d;
                d = this._parseDouble(formula.getFormulaValue());
                if (isNaN(d))
                    d = 0;
                count++;
            }

            ////save the last iteration
            this.getIterationValues.getItem(this.cell, (formula.getFormulaValue() == "" ? "0" : formula.getFormulaValue()));

            this._inHandleIterations = false;
        };
        this._indexOfAny = function (text, operators) {
            for (var i = 0; i < text.length; i++) {
                if (operators.indexOf(text[i]) > -1) {
                    return i;
                }
            }
            return -1;
        };
        this._initLibraryFunctions = function () {
            this._libraryFunctions = new HashTable();

            ////Hand-coded formulas.
            this._addFunction("SUM", "computeSum");
            this._addFunction("EncodeURL", "computeEncodeURL");

            
            //TEXT FORMULA
            this._addFunction("CHAR", "computeChar");
            this._addFunction("CODE", "computeCode");
            this._addFunction("UNICODE", "computeUniCode");
            this._addFunction("UNICHAR", "computeUniChar");
            this._addFunction("UPPER", "computeUpper");
            this._addFunction("LOWER", "computeLower");
            this._addFunction("LEN", "computeLen");
            this._addFunction("MID", "computeMid");
            this._addFunction("LEFT", "computeLeft");
            this._addFunction("CLEAN", "computeClean");
            this._addFunction("REPT", "computeRept");
            this._addFunction("RIGHT", "computeRight");
            this._addFunction("REPLACE", "computeReplace");
            this._addFunction("EXACT", "computeExact");
            this._addFunction("FIND", "computeFind");
            this._addFunction("TRIM", "computeTrim");
            this._addFunction("SEARCH", "computeSearch");
            this._addFunction("SUBSTITUTE", "computeSubstitute");
            this._addFunction("PROPER", "computeProper");
            this._addFunction("T", "computeT");
            this._addFunction("NUMBERVALUE", "computeNumberValue");
            this._addFunction("CONCATENATE", "computeConcatenate");
            this._addFunction("VALUE", "computeValue");
            this._addFunction("DOLLAR", "computeDollar");
            this._addFunction("FIXED", "computeFixed");

            //Engineering formulas
            this._addFunction("BIN2DEC", "computeBin2Dec");
            this._addFunction("BIN2OCT", "computeBin2Oct");
            this._addFunction("BIN2HEX", "computeBin2Hex");
            this._addFunction("DEC2BIN", "computeDec2Bin");
            this._addFunction("DEC2OCT", "computeDec2Oct");
            this._addFunction("HEX2BIN", "computeHex2Bin");
            this._addFunction("HEX2OCT", "computeHex2Oct");

            // DATE and TIME
            this._addFunction("DATE", "computeDate");
            this._addFunction("DATEVALUE", "computeDatevalue");
            this._addFunction("DAY", "computeDay");
            this._addFunction("DAYS", "computeDays");
            this._addFunction("DAYS360", "computeDays360");
            this._addFunction("EDATE", "computeEDate");
            this._addFunction("EOMONTH", "computeEOMonth");
            this._addFunction("HOUR", "computeHour");
            this._addFunction("ISOWEEKNUM", "computeISOWeeknum");
            this._addFunction("MINUTE", "computeMinute");
            this._addFunction("MONTH", "computeMonth");
            this._addFunction("NETWORKDAYS", "computeNetworkDays");
            this._addFunction("NETWORKDAYSINTL", "computeNetworkDaysintl");
            this._addFunction("NOW", "computeNow");
            this._addFunction("SECOND", "computeSecond");
            this._addFunction("TIME", "computeTime");
            this._addFunction("TIMEVALUE", "computeTimevalue");
            this._addFunction("TODAY", "computeToday");
            this._addFunction("WEEKDAY", "computeWeekday");
            this._addFunction("WEEKNUM", "computeWeeknum");
            this._addFunction("WORKDAY", "computeWorkDay");
            this._addFunction("WORKDAYINTL", "computeWorkDayintl");
            this._addFunction("YEAR", "computeYear");

            //look and ref
            this._addFunction("ADDRESS", "computeAddress");
            this._addFunction("AREAS", "computeAreas");
            this._addFunction("CHOOSE", "computeChoose");
            this._addFunction("COLUMN", "computeColumn");
            this._addFunction("COLUMNS", "computeColumns");
            this._addFunction("FORMULATEXT", "computeFormulaText");
            this._addFunction("HYPERLINK", "computeHyperlink");
            this._addFunction("HLOOKUP", "computeHLookUp");
            this._addFunction("INDEX", "computeIndex");
            this._addFunction("INDIRECT", "computeIndirect");
            this._addFunction("LOOKUP", "computeLookUp");
            this._addFunction("OFFSET", "computeOffSet");
            this._addFunction("TRANSPOSE", "computeTranspose");



            // Statics
            this._addFunction("LOGNORM.INV", "computeLognormOinv");
            this._addFunction("NORM.INV", "computeNormOinv");
            this._addFunction("NORM.DIST", "computeNormOdist");
            this._addFunction("NORM.S.DIST", "computeNormOsODist");
            this._addFunction("NORM.S.INV", "computeNormOsOInv");
            this._addFunction("PERMUT", "computePermut");
            this._addFunction("PERMUTATIONA", "computePermutationA");
            this._addFunction("STANDARDIZE", "computeStandardize");
            this._addFunction("BINOM.DIST", "computeBinomOdist");
            this._addFunction("CHISQ.INV.RT", "computeChisqOinvOrt");
            this._addFunction("CHISQ.INV", "computeChisqOinv");
            this._addFunction("CHISQ.DIST.RT", "computeChisqOdistOrt");
            this._addFunction("F.DIST", "computeFOdist");
            this._addFunction("GAMMALN", "computeGammaln");
            this._addFunction("CONFIDENCE.NORM", "computeConfidenceOnorm");
            this._addFunction("EXPON.DIST", "computeExponOdist");
            this._addFunction("FISHER", "computeFisher");
            this._addFunction("FISHERINV", "computeFisherInv");
            this._addFunction("GAMMALN.PRECISE", "computeGammalnOPrecise");
            this._addFunction("GAMMA.INV", "computeGammaOinv");
            this._addFunction("AVERAGE", "computeAverage");
            this._addFunction("AVERAGEA", "computeAverageA");
            this._addFunction("POISSON.DIST", "computePoissonODist");
            this._addFunction("WEIBULL.DIST", "computeWeiBullODist");
            this._addFunction("F.INV.RT", "computeFOinvOrt");
            this._addFunction("T.DIST", "computeTOdist");
            this._addFunction("MAX", "computeMax");
            this._addFunction("MAXA", "computeMaxa");
            this._addFunction("MEDIAN", "computeMedian");
            this._addFunction("MIN", "computeMin");
            this._addFunction("MINA", "computeMina");
            this._addFunction("PERCENTRANK.INC", "computePercentrankInc");
            this._addFunction("PERCENTILE", "computePercentile");
            this._addFunction("RANK.EQ", "computeRankOEq");
            this._addFunction("COUNT", "computeCount");
            this._addFunction("COUNTA", "computeCounta");
            this._addFunction("DEVSQ", "computeDevsq");
            this._addFunction("F.DIST.RT", "computeFOdistORt");
            this._addFunction("FORECAST", "computeForecast");
            this._addFunction("GEOMEAN", "computeGeomean");
            this._addFunction("HARMEAN", "computeHarmean");
            this._addFunction("INTERCEPT", "computeIntercept");
            this._addFunction("LARGE", "computeLarge");
            this._addFunction("SMALL", "computeSmall");
            this._addFunction("LOGNORM.DIST", "computeLognormOdist");          
            this._addFunction("AVEDEV", "computeAvedev");
            this._addFunction("COUNTBLANK", "computeCountblank"); 
            this._addFunction("STDEV.P", "computeStdevOp");
            this._addFunction("STDEV.S", "computeStdevOS");
            this._addFunction("STDEVA", "computeStdeva");
            this._addFunction("STDEVPA", "computeStdevpa");
            this._addFunction("T.INV", "computeTOinv");
            this._addFunction("VAR.P", "computeVarp");
            this._addFunction("VARA", "computeVara");
            this._addFunction("VARPA", "computeVarpa");
            this._addFunction("CORREL","computeCorrel");
            this._addFunction("NEGBINOM.DIST", "computeNegbinomODist");
            this._addFunction("PERCENTILE.EXC", "computePercentileExc");
            this._addFunction("PERCENTILE.INC", "computePercentileOInc");
            this._addFunction("TRIMMEAN", "computeTrimmean");
            this._addFunction("RSQ", "computeRsq");
            //60 statics function
            

            // math and Trig
            this._addFunction("ABS", "computeAbs");
            this._addFunction("ACOS", "computeAcos");
            this._addFunction("ACOSH", "computeAcosh");
            this._addFunction("ACOT", "computeAcot");
            this._addFunction("ACOTH", "computeAcoth");
            this._addFunction("ARABIC", "computeArabic");
            this._addFunction("ASIN", "computeAsin");
            this._addFunction("ATAN", "computeAtan");
            this._addFunction("ATAN2", "computeAtan2");
            this._addFunction("CEILINGMATH", "computeCeilingMath");
            this._addFunction("CEILING", "computeCeiling");
            this._addFunction("COMBIN", "computeCombin");
            this._addFunction("COMBINA", "computeCombinA");
            this._addFunction("COS", "computeCos");
            this._addFunction("COSH", "computeCosh");
            this._addFunction("COT", "computeCot");
            this._addFunction("CSC", "computeCsc");
            this._addFunction("CSCH", "computeCsch");
            this._addFunction("DECIMAL", "computeDecimal");
            this._addFunction("DEGREES", "computeDegrees");
            this._addFunction("ISTEXT", "computeIsText");
            this._addFunction("EXP", "computeExp");
            this._addFunction("EVEN", "computeEven");
            this._addFunction("FACT", "computeFact");
            this._addFunction("FACTDOUBLE", "computeFactdouble");
            this._addFunction("FLOOR", "computeFloor");
            this._addFunction("INT", "computeInt");
            this._addFunction("LN", "computeLn");
            this._addFunction("LOG", "computeLog");
            this._addFunction("PI", "computePI");
            this._addFunction("PRODUCT", "computeProduct");
            this._addFunction("SEC", "computeSecant");
            this._addFunction("SERIESSUM", "computeSeriessum");
            this._addFunction("SIN", "computeSin");
            this._addFunction("SINH", "computeSinh");
            this._addFunction("SQRT", "computeSqrt");
            this._addFunction("SUBTOTAL", "computeSubTotal");
            this._addFunction("SUMIF", "computeSumif");
            this._addFunction("TRUNC", "computeTrunc");
            this._addFunction("TAN", "computeTan");
            this._addFunction("LOG10", "computeLogTen");

            //Logic Function

            this._addFunction("AND", "computeAnd");
            this._addFunction("FALSE", "computeFalse");
            this._addFunction("IF", "computeIf");
            this._addFunction("IFERROR", "computeIfError");
            this._addFunction("IFNA", "computeIfNA");
            this._addFunction("NOT", "computeNot");
            this._addFunction("OR", "computeOr");
            this._addFunction("TRUE", "computeTrue");
            this._addFunction("XOR", "computeXor");

            // Information Function

            this._addFunction("CELL", "computeCell");
            this._addFunction("ERROR.TYPE", "computeErrorType");
            this._addFunction("INFO", "computeInfo");
            this._addFunction("ISBLANK", "computeIsBlank");
            this._addFunction("ISERR", "computeIsErr");
            this._addFunction("ISERROR", "computeIsError");
            this._addFunction("ISEVEN", "computeIsEven");
            this._addFunction("ISFORMULA", "computeIsFormula");
            this._addFunction("ISLOGICAL", "computeIsLogical");
            this._addFunction("ISNA", "computeIsNA");
            this._addFunction("ISNONTEXT", "computeIsNonText");
            this._addFunction("ISNUMBER", "computeIsNumber");
            this._addFunction("ISODD", "computeIsOdd");
            this._addFunction("ISREF", "computeIsRef");
            this._addFunction("ISTEXT", "computeIsText");
            this._addFunction("N", "computeN");
            this._addFunction("NA", "computeNA");
            this._addFunction("SHEET", "computeSheet");
            this._addFunction("SHEETS", "computeSheets");
            this._addFunction("TYPE", "computeType");
            this._addFunction("ROW", "computeRow");
            this._addFunction("ROWS", "computeRows");

        };



        this._comb = function (k, n) {
            var top = 1;
            for (var i = k + 1; i <= n; ++i) {
                top = top * i;
            }

            var bottom = 1;
            for (var i = 2; i <= (n - k) ; ++i) {
                bottom = bottom * i;
            }

            return top / bottom;
        };
        this._finv = function (p, df1, df2) {
            var mult = Math.exp(this._gammaln((df1 + df2) / 2) - this._gammaln(df1 / 2) - this._gammaln(df2 / 2) + (df1 / 2) * Math.log(df1 / df2));

            var guess = mult;

            var lastIncrement = guess / 2;
            var gi = 0;
            var eps = 1e-7;
            var k = 100;
            var tries = 3;
            var its = 0;
            while (k == 100 && tries > 0) {
                tries--;
                guess = guess / 2;
                lastIncrement = guess / 2;

                for (k = 0; k < 100; ++k) {
                    its++;
                    gi = 1 - mult * this._fdist(guess, df1, df2);

                    ////Console.WriteLine("{0}: guess {1}  computed {2}  target {3}", its, guess, gi, p);
                    if (Math.abs((gi - p) / p) < eps) {
                        break;
                    }

                    ////greater since looking for 1 - xxxx
                    if (gi > p) {
                        guess = guess + lastIncrement;
                    } else {
                        lastIncrement = lastIncrement / 2;
                        if (guess - lastIncrement < 0) {
                            lastIncrement = guess / 2;
                        }

                        guess = guess - lastIncrement;
                    }
                }
            }

            if (k == 100) {
                guess = -1;
            }

            return guess;
        };
        this._tProbabilityDensity = function (x, k) {
            var a = this._gammaFunction(0.5 * k + 0.5);
            var b = Math.pow(1.0 + (x * x) / k, -0.5 * k - 0.5);
            var c = Math.sqrt(k * Math.PI) * this._gammaFunction(0.5 * k);
            return a * b / c;
        };
        this._tCumulativeDensity = function (x, k) {
            if (isNaN(x))
                return NaN;
            if (x == 0)
                return 0.5;
            if (x > 0) {
                var x2 = k / (x * x + k);
                return 1.0 - 0.5 * this._rIBetaFunction(x2, 0.5 * k, 0.5);
            }
            return 1.0 - this._tCumulativeDensity(-x, k);
        };
        this._sign = function (p) {
            if (p === 0 || isNaN(p))
                return p;
            return p > 0 ? 1 : -1;
        };
        this._var = function (x) {
            var sumx = 0;
            var n = x.length;
            for (var i = 0; i < n; ++i) {
                sumx += x[i];
            }

            sumx = sumx / n;

            var sumx2 = 0;
            var d;
            for (var i = 0; i < n; ++i) {
                d = x[i] - sumx;
                sumx2 += d * d;
            }

            return sumx2 / (n - 1);
        };
        this._tCumulativeDistributionInverse = function (p, k) {
            var maxIterations = 10;
            var minAccuracy = 0.0001;

            if (p < 0 || p > 1.0)
                throw "Probability must be between 0 and 1";
            if (p == 0)
                return Number.NEGATIVE_INFINITY;
            if (p == 1)
                return Number.POSITIVE_INFINITY;
            if (p == 0.5)
                return 0.0;

            switch (k) {
                case 1:
                    return Math.tan(Math.PI * (p - 0.5));
                case 2:
                    var alpha = 4 * p * (1 - p);
                    return 2 * (p - 0.5) * Math.sqrt(2.0 / alpha);
                case 4:
                    var alpha = 4 * p * (1 - p);
                    var sqrtAlpha = Math.sqrt(alpha);
                    var q = Math.cos((1.0 / 3.0) * Math.acos(sqrtAlpha)) / sqrtAlpha;
                    return this._sign(p - 0.5) * 2 * Math.sqrt(q - 1);
                default:
                    var x;
                    if (k > 6)
                        x = this._standardNormalCumulativeDistributionFunctionInverse(p);
                    else
                        x = this._tCumulativeDistributionInverse(p, 4);

                    for (var i = 0; i < maxIterations; i++) {
                        var p0 = this._tCumulativeDensity(x, k);
                        var absAccuracy = Math.abs(p - p0);
                        if (absAccuracy < minAccuracy)
                            return x;
                        var slope = this._tProbabilityDensity(x, k);
                        x = x + (p - p0) / slope;
                    }
                    throw "Solution did not converge";
            }
        };
        this._fdist = function (x, df1, df2) {
            var nPanels = 32;
            var leftside = 0;
            var rightside = x;

            var h = (rightside - leftside) / nPanels;
            var mult = h / 3;

            var sum1 = this._fdensity(leftside, df1, df2) + this._fdensity(rightside, df1, df2);
            var sum4 = 0;
            for (var i = 1; i < nPanels; i += 2) {
                sum4 = sum4 + 4 * this._fdensity(leftside + i * h, df1, df2);
            }

            var sum2 = 0;
            for (var i = 2; i < nPanels; i += 2) {
                sum2 = sum2 + 2 * this._fdensity(leftside + i * h, df1, df2);
            }

            var gd = mult * (sum1 + sum2 + sum4);
            var oldgd = gd;
            var eps = 1e-7;

            var k = 0;

            for (k = 0; k < 10; ++k) {
                nPanels = nPanels * 2;

                sum2 += sum4 / 2;
                sum4 = 0;
                h = (rightside - leftside) / nPanels;
                for (var i = 0; i < nPanels; ++i) {
                    if (i % 2 == 1) {
                        var d = this._fdensity(leftside + h * i, df1, df2);
                        sum4 = sum4 + d;
                    }
                }

                sum4 = 4 * sum4;
                mult = h / 3;
                gd = mult * (sum1 + sum2 + sum4);

                if (Math.abs((gd - oldgd) / oldgd) < eps) {
                    break;
                }

                oldgd = gd;
            }

            return gd;
        };
        this._fdensity = function (x, df1, df2) {
            return Math.pow(x, (df1 - 2) / 2) / Math.pow(1 + df1 * x / df2, (df1 + df2) / 2);
        };
        this._normaldist = function (x, u, s) {
            var nPanels = 32;
            var leftside;
            var rightside;
            if (x > u) {
                leftside = u - (x - u);
                rightside = x;
            } else {
                leftside = x;
                rightside = u + (u - x);
            }

            var h = (rightside - leftside) / nPanels;
            var mult = h / 3;

            var sum1 = this._normaldensity(leftside, u, s) + this._normaldensity(rightside, u, s);
            var sum4 = 0;
            for (var i = 1; i < nPanels; i += 2) {
                sum4 = sum4 + 4 * this._normaldensity(leftside + i * h, u, s);
            }

            var sum2 = 0;
            for (var i = 2; i < nPanels; i += 2) {
                sum2 = sum2 + 2 * this._normaldensity(leftside + i * h, u, s);
            }

            var gd = mult * (sum1 + sum2 + sum4);
            var oldgd = gd;
            var eps = 1e-7;

            var k = 0;

            for (k = 0; k < 10; ++k) {
                nPanels *= 2;

                sum2 = sum2 + sum4 / 2;
                sum4 = 0;
                h = h = (rightside - leftside) / nPanels;
                for (var i = 0; i < nPanels; ++i) {
                    if (i % 2 == 1) {
                        var d = this._normaldensity(leftside + h * i, u, s);
                        sum4 += d;
                    }
                }

                sum4 = 4 * sum4;
                mult = h / 3;
                gd = mult * (sum1 + sum2 + sum4);

                if (Math.abs((gd - oldgd) / oldgd) < eps) {
                    break;
                }

                oldgd = gd;
            }

            if (x > u) {
                ////Add the left tails.
                gd = gd + (1 - gd) / 2;
            } else {
                ////Return only the left tail.
                gd = (1 - gd) / 2;
            }

            ////Console.WriteLine(k.toString() + "  " + gd.toString());
            return gd;
        };
        this._normaldensity = function (x, u, s) {
            return 1 / (Math.sqrt(2 * Math.PI) * s) * Math.exp(-(x - u) * (x - u) / (2 * s * s));
        };
        this._standardNormalCumulativeDistribution = function (x) {
            //Approimation based on Abramowitz & Stegun (1964)
            if (x < 0)
                return 1.0 - this._standardNormalCumulativeDistribution(-x);
            var b0 = 0.2316419;
            var b1 = 0.319381530;
            var b2 = -0.356563782;
            var b3 = 1.781477937;
            var b4 = -1.821255978;
            var b5 = 1.330274429;
            var pdf = this._standardNormalProbabilityDensityFunction(x);
            var a = 1.0 / (1.0 + b0 * x);
            return 1.0 - pdf * (b1 * a + b2 * Math.pow(a, 2) + b3 * Math.pow(a, 3) + b4 * Math.pow(a, 4) + b5 * Math.pow(a, 5));
        };
        this._normalinv = function (p, u, s) {
            var guess = u;
            if (p < .05) {
                guess = u - 2 * s;
            } else if (p < .5) {
                guess = u;
            } else if (p < .95) {
                guess = u + 2 * s;
            } else {
                guess = u + 5 * s;
            }

            var lastIncrement = guess / 2;
            var gi = 0;
            var eps = 1e-7;
            var k = 100;
            var tries = 3;
            var its = 0;
            while (k == 100 && tries > 0) {
                tries--;
                guess = guess / 2;
                lastIncrement = guess / 2;

                for (k = 0; k < 100; ++k) {
                    its++;
                    gi = this._normaldist(guess, u, s);
                    if (Math.abs((gi - p) / p) < eps) {
                        break;
                    }

                    if (gi < p) {
                        guess = guess + lastIncrement;
                    } else {
                        lastIncrement = lastIncrement / 2;
                        if (guess - lastIncrement < 0) {
                            lastIncrement = guess / 2;
                        }

                        guess = guess - lastIncrement;
                    }
                }
            }

            if (k == 100) {
                guess = -1;
            }

            return guess;
        };
        this._normalCumulativeDistributionFunctionInverse = function (p, mean, standardDeviation) {
            var x = this._standardNormalCumulativeDistributionInverse(p);
            return standardDeviation * x + mean;
        };
        this._standardNormalCumulativeDistributionInverse = function (p) {
            if (p < 0 || p > 1.0)
                throw "Probability must be between 0 and 1";
            if (p == 0)
                return Number.NEGATIVE_INFINITY;
            if (p == 1)
                return Number.POSITIVE_INFINITY;
            if (p == 0.5)
                return 0.0;

            var a1 = -3.969683028665376e+01;
            var a2 = 2.209460984245205e+02;
            var a3 = -2.759285104469687e+02;
            var a4 = 1.383577518672690e+02;
            var a5 = -3.066479806614716e+01;
            var a6 = 2.506628277459239e+00;

            var b1 = -5.447609879822406e+01;
            var b2 = 1.615858368580409e+02;
            var b3 = -1.556989798598866e+02;
            var b4 = 6.680131188771972e+01;
            var b5 = -1.328068155288572e+01;

            var c1 = -7.784894002430293e-03;
            var c2 = -3.223964580411365e-01;
            var c3 = -2.400758277161838e+00;
            var c4 = -2.549732539343734e+00;
            var c5 = 4.374664141464968e+00;
            var c6 = 2.938163982698783e+00;

            var d1 = 7.784695709041462e-03;
            var d2 = 3.224671290700398e-01;
            var d3 = 2.445134137142996e+00;
            var d4 = 3.754408661907416e+00;

            var pLow = 0.02425;
            var pHigh = 1 - pLow;

            var q, x;
            if (0 < p && p < pLow) {
                q = Math.sqrt(-2 * Math.log(p));
                x = (((((c1 * q + c2) * q + c3) * q + c4) * q + c5) * q + c6) / ((((d1 * q + d2) * q + d3) * q + d4) * q + 1);
                return x;
            }

            if (pLow <= p && p <= pHigh) {
                q = p - 0.5;
                var r = q * q;
                x = (((((a1 * r + a2) * r + a3) * r + a4) * r + a5) * r + a6) * q / (((((b1 * r + b2) * r + b3) * r + b4) * r + b5) * r + 1);
                return x;
            }

            //if(p_high < p && p < 1)
            q = Math.sqrt(-2 * Math.log(1 - p));
            x = -(((((c1 * q + c2) * q + c3) * q + c4) * q + c5) * q + c6) / ((((d1 * q + d2) * q + d3) * q + d4) * q + 1);
            return x;
        };
        this._normalProbabilityDensity = function (x, mean, standardDeviation) {
            var sqrtTwoPiInv = 0.398942280401433;
            var z = (x - mean) / standardDeviation;
            return sqrtTwoPiInv * Math.exp(-0.5 * z * z) / standardDeviation;
        };
        this._standardNormalProbabilityDensityFunction = function (x) {
            var SqrtTwoPiInv = 0.398942280401433;
            return SqrtTwoPiInv * Math.exp(-0.5 * x * x);
        };
        this._standardNormalCumulativeDistributionFunction = function (x) {
            if (x < 0)
                return 1.0 - this._standardNormalCumulativeDistributionFunction(-x);
            var b0 = 0.2316419;
            var b1 = 0.319381530;
            var b2 = -0.356563782;
            var b3 = 1.781477937;
            var b4 = -1.821255978;
            var b5 = 1.330274429;
            var pdf = this._standardNormalProbabilityDensityFunction(x);
            var a = 1.0 / (1.0 + b0 * x);
            return 1.0 - pdf * (b1 * a + b2 * Math.pow(a, 2) + b3 * Math.pow(a, 3) + b4 * Math.pow(a, 4) + b5 * Math.pow(a, 5));
        };
        this._standardNormalCumulativeDistributionFunctionInverse = function (p) {
            if (p < 0 || p > 1.0)
                throw "Probability must be between 0 and 1";
            if (p == 0)
                return Number.NEGATIVE_INFINITY;
            if (p == 1)
                return Number.POSITIVE_INFINITY;
            if (p == 0.5)
                return 0.0;

            var a1 = -3.969683028665376e+01;
            var a2 = 2.209460984245205e+02;
            var a3 = -2.759285104469687e+02;
            var a4 = 1.383577518672690e+02;
            var a5 = -3.066479806614716e+01;
            var a6 = 2.506628277459239e+00;

            var b1 = -5.447609879822406e+01;
            var b2 = 1.615858368580409e+02;
            var b3 = -1.556989798598866e+02;
            var b4 = 6.680131188771972e+01;
            var b5 = -1.328068155288572e+01;

            var c1 = -7.784894002430293e-03;
            var c2 = -3.223964580411365e-01;
            var c3 = -2.400758277161838e+00;
            var c4 = -2.549732539343734e+00;
            var c5 = 4.374664141464968e+00;
            var c6 = 2.938163982698783e+00;

            var d1 = 7.784695709041462e-03;
            var d2 = 3.224671290700398e-01;
            var d3 = 2.445134137142996e+00;
            var d4 = 3.754408661907416e+00;

            var pLow = 0.02425;
            var pHigh = 1 - pLow;

            var q, x;
            if (0 < p && p < pLow) {
                q = Math.sqrt(-2 * Math.log(p));
                x = (((((c1 * q + c2) * q + c3) * q + c4) * q + c5) * q + c6) / ((((d1 * q + d2) * q + d3) * q + d4) * q + 1);
                return x;
            }

            if (pLow <= p && p <= pHigh) {
                q = p - 0.5;
                var r = q * q;
                x = (((((a1 * r + a2) * r + a3) * r + a4) * r + a5) * r + a6) * q / (((((b1 * r + b2) * r + b3) * r + b4) * r + b5) * r + 1);
                return x;
            }

            //if(p_high < p && p < 1)
            q = Math.sqrt(-2 * Math.log(1 - p));
            x = -(((((c1 * q + c2) * q + c3) * q + c4) * q + c5) * q + c6) / ((((d1 * q + d2) * q + d3) * q + d4) * q + 1);
            return x;
        };
        this._chiSquaredProbabilityDensityFunction = function (x, k) {
            var g = this._gammaFunction(0.5 * k);
            var a = 1.0 / (Math.pow(2, 0.5 * k) * g);
            return a * Math.pow(x, 0.5 * k - 1.0) * Math.exp(-0.5 * x);
        };
        this._binomdist = function (trials, successes, p) {
            var pPow = 1;
            var pm1 = 1 - p;

            var p1Pow = Math.pow(pm1, trials);

            var dist = 0;
            var cbn = pPow * Math.pow(pm1, trials);
            if (cbn == 0) {
                return NaN;
            }

            for (var i = 0; i <= successes; ++i) {
                dist += cbn;
                cbn = cbn * p / pm1 * (trials - i) / (i + 1);
                if (isFinite(cbn) || !isNaN(cbn)) {
                    dist = NaN;
                    break;
                }
            }

            return dist;
        };
        this._gammadensity = function (a, b, x) {
            //// excel version     dist = 1 / (Math.Pow(b, a) * gammaln(a)) * Math.Pow(x, a - 1) * Math.Exp(-x / b);
            //// http://www.itl.nist.gov/div898/handbook/eda/section3/eda366b.htm
            return Math.pow(x / b, a - 1) * Math.exp(-x / b) / (b * Math.exp(this._gammaln(a)));
        };
        
        this.minValue = -1.79769e+308;
        this.maxValue = 2147483647;
        this._charTable = [this._string_empty, "A", "B", "C", "D", "E", "F", "G", "H", "I", "J", "K", "L", "M", "N", "O", "P", "Q", "R", "S", "T", "U", "V", "W", "X", "Y", "Z"];
        this._stdevdotP = function (range) {
            var n = range.length;
            var mean = 0;
            var sigma = 0;
            var count = 0;
            for (var i = 0; i < n; ++i)
                mean = mean + range[i];
            mean = mean / n;
            for (var i = 0; i < n; i++) {
                sigma = sigma + Math.pow(range[i] - mean, 2);
                count++;
            }

            return Math.sqrt(sigma / count);
        };
        this._determinant = function (a, k) {
            var s = 1, det = 0;
            var matlength = parseInt(k.toString());
            var b = [matlength, matlength];
            var i, j, m, n, c;
            if (k == 1) {
                return (a[0, 0]);
            } else {
                det = 0;
                for (c = 0; c < k; c++) {
                    m = 0;
                    n = 0;
                    for (i = 0; i < k; i++) {
                        for (j = 0; j < k; j++) {
                            b[i, j] = 0;
                            if (i != 0 && j != c) {
                                b[m, n] = a[i, j];
                                if (n < (k - 2))
                                    n++;
                                else {
                                    n = 0;
                                    m++;
                                }
                            }
                        }
                    }
                    det = det + s * (a[0, c] * this._determinant(b, k - 1));
                    s = -1 * s;
                }
            }

            return (det);
        };
        this._factorialTable = [
            1, 1, 2, 6, 24, 120, 720,
            5040, 40320, 362880, 3628800,
            39916800, 479001600
        ];
        this._mean = function (array) {
            return this._mean(array, 1.0);
        };
        this._negbinomdensity = function (failures, successes, p) {
            return this._comb(successes - 1, failures + successes - 1) * Math.pow(p, successes) * Math.pow(1 - p, failures);
        };
        this._sd = function (x, xbar) {
            var n = x.length;
            xbar = 0;
            for (var i = 0; i < n; ++i) {
                xbar += x[i];
            }

            xbar = xbar / n;
            var sumx2 = 0;
            var d = 0;

            for (var i = 0; i < n; ++i) {
                d = x[i] - xbar;
                sumx2 += d * d;
            }
            if (n == 1)
                return 0;
            return Math.sqrt(sumx2 / (n - 1));
        };
        this._betaCumulativeDist = function (x, a, b) {
            if (x < 0 || x > 1)
                throw "x must be between 0 and 1";
            return this._rIBetaFunction(x, a, b);
        };
        this._betaProbabilityDens = function (x, a, b) {
            if (x < 0 || x > 1)
                throw "x must be between 0 and 1";

            var B = this._betaFunction(a, b);
            var d = Math.pow(x, a - 1) * Math.pow(1 - x, b - 1) / B;
            return d;
        };
        this._critbinom = function (nTrials, p, alpha) {
            var checkval = nTrials;
            var half = nTrials;
            var dist = 1;
            var dist1 = 1;
            do {
                half = half / 2 + 1;
                if (dist >= alpha) {
                    dist1 = this._binomdist(nTrials, checkval - 1, p);
                    if (!isNaN(dist1)) {
                        return this.maxValue;
                    }

                    if (dist1 < alpha && dist1 > 0) {
                        break;
                    }

                    checkval = checkval - half;
                } else {
                    dist1 = this._binomdist(nTrials, checkval + 1, p);
                    if (dist1 >= alpha) {
                        checkval = checkval + 1;
                        break;
                    }

                    checkval = checkval + half;
                }

                dist = this._binomdist(nTrials, checkval, p);
            } while (checkval < nTrials && checkval > 0);

            return checkval;
        };
        this._newnormalinv = function (p) {
            var region1 = [
                -3.969683028665376e+01, 2.209460984245205e+02,
                -2.759285104469687e+02, 1.383577518672690e+02,
                -3.066479806614716e+01, 2.506628277459239e+00];

            var region2 = [
                -5.447609879822406e+01, 1.615858368580409e+02,
                -1.556989798598866e+02, 6.680131188771972e+01,
                -1.328068155288572e+01];

            var region3 = [
                -7.784894002430293e-03, -3.223964580411365e-01,
                -2.400758277161838e+00, -2.549732539343734e+00,
                4.374664141464968e+00, 2.938163982698783e+00];

            var region4 = [
                7.784695709041462e-03, 3.224671290700398e-01,
                2.445134137142996e+00, 3.754408661907416e+00];

            var pLow = 0.02425;
            var pHigh = 1 - pLow;
            var q = 0.0;
            var r = 0.0;

            // For lower region:
            if (p < pLow) {
                q = Math.sqrt(-2 * Math.log(p));
                return (((((region3[0] * q + region3[1]) * q + region3[2]) * q + region3[3]) * q + region3[4]) * q + region3[5]) / ((((region4[0] * q + region4[1]) * q + region4[2]) * q + region4[3]) * q + 1);
            }

            // For upper region:
            if (pHigh < p) {
                q = Math.sqrt(-2 * Math.log(1 - p));
                return -(((((region3[0] * q + region3[1]) * q + region3[2]) * q + region3[3]) * q + region3[4]) * q + region3[5]) / ((((region4[0] * q + region4[1]) * q + region4[2]) * q + region4[3]) * q + 1);
            }

            // For central region:
            q = p - 0.5;
            r = q * q;
            return (((((region1[0] * r + region1[1]) * r + region1[2]) * r + region1[3]) * r + region1[4]) * r + region1[5]) * q / (((((region2[0] * r + region2[1]) * r + region2[2]) * r + region2[3]) * r + region2[4]) * r + 1);
        };
        this._chiinv = function (p, v) {
            var guess = p;

            var lastIncrement = Number(guess / 2);
            var gi = 0;
            var eps = 1e-7;
            var k = 100;
            var tries = 3;
            while (k == 100 && tries > 0) {
                tries--;
                guess = (guess / 2);
                lastIncrement = guess / 2;

                for (k = 0; k < 100; ++k) {
                    gi = 1 - this._chidist(guess, v);
                    if (Math.abs((gi - p) / p) < eps) {
                        break;
                    }

                    if (gi > p) {
                        guess = guess + lastIncrement;
                    } else {
                        lastIncrement = lastIncrement / 2;
                        if (guess - lastIncrement < 0) {
                            lastIncrement = guess / 2;
                        }

                        guess = guess - lastIncrement;
                    }
                }
            }

            if (k == 100) {
                guess = -1;
            }

            return guess;
        };
        this._chidist = function (x, v) {
            var gauss_w = [
                0.09654008851,
                0.09563872008,
                0.09384439908,
                0.09117387870,
                0.08765209300,
                0.08331192423,
                0.07819389579,
                0.07234579411,
                0.06582222278,
                0.05868409348,
                0.05099805926,
                0.04283589802,
                0.03427386291,
                0.02539206531,
                0.01627439473,
                0.00701861001
            ];

            var gauss_x = [
                0.04830766569,
                0.14447196158,
                0.23928736225,
                0.33186860228,
                0.42135127613,
                0.50689990893,
                0.58771575724,
                0.66304426693,
                0.73218211874,
                0.79448379597,
                0.84936761373,
                0.89632115577,
                0.93490607594,
                0.96476225559,
                0.98561151155,
                0.99726386185
            ];

            var gauss_n = 16;
            var ex = v / 2;
            var mult = 1 / (Math.pow(2, ex) * Math.exp(this._gammaln(ex)));
            ex = ex - 1;

            var a = 0;
            var b = x;

            var midPt = (a + b) / 2;
            var radius = (b - a) / 2;

            var sum = 0;

            var offset;
            for (var i = 0; i < gauss_n; ++i) {
                offset = gauss_x[i] * radius;
                sum = Number(sum) + Number(gauss_w[i] * (Math.pow(midPt + offset, ex) * Math.exp(-(midPt + offset) / 2) + Math.pow(midPt - offset, ex) * Math.exp(-(midPt - offset) / 2)));
            }

            sum = Number(mult) * Number(sum) * Number(radius);
            return sum;
        };
        this._gammaln = function (x) {
            var gammaAs = [0.918938533204673, 0.000595238095238, 0.000793650793651, 0.002777777777778, 0.083333333333333];
            var y = x;
            var f = 0;
            if (y < 7) {
                f = y;
                y++;
                while (y < 7) {
                    f = f * y;
                    if (y < 7) {
                        y++;
                    }
                }

                f = -Math.log(f);
            }

            var z = this._parseDouble(1) / (y * y);
            return f + (y - this._parseDouble(0.5)) * Math.log(y) - y + gammaAs[0] + (((-gammaAs[1] * z + gammaAs[2]) * z - gammaAs[3]) * z + gammaAs[4]) / y;
        };
        this._pearson = function (x, y, n) {
            var sumx = 0;
            var sumy = 0;
            for (var i = 0; i < n; ++i) {
                sumx += x[i];
                sumy += y[i];
            }

            sumx = sumx / n;
            sumy = sumy / n;

            var sumxy = 0;
            var sumx2 = 0;
            var sumy2 = 0;
            var d1, d;
            for (var i = 0; i < n; ++i) {
                d = x[i] - sumx;
                d1 = y[i] - sumy;
                sumxy += d * d1;
                sumx2 += d * d;
                sumy2 += d1 * d1;
            }

            return sumxy / Math.sqrt(sumx2 * sumy2);
        };
        this._fProbabilityDensity = function (x, k1, k2) {
            if (k1 <= 0 || k2 <= 0)
                throw "k1 and k2 must be greater than 0.";
            if (x == 0)
                return 0.0;
            var a1 = Math.pow(k1 * x, 0.5 * k1);
            var a2 = Math.pow(k2, 0.5 * k2);
            var a3 = Math.pow(k1 * x + k2, 0.5 * (k1 + k2));
            var a = a1 * a2 / a3;
            var b = this._betaFunction(0.5 * k1, 0.5 * k2);
            var c = x * b;
            return a / c;
        };
        this._fCumulativeDensity = function (x, k1, k2) {
            if (k1 <= 0 || k2 <= 0)
                throw "k1 and k2 must be greater than 0.";
            if (x < 0)
                throw "x must be greater than 0.";

            if (x == 0)
                return 0.0;
            var x2 = (k1 * x) / (k1 * x + k2);
            var p = this._rIBetaFunction(x2, 0.5 * k1, 0.5 * k2);
            return Math.min(1, p);
        };
        this._rIBetaFunction = function (x, a, b) {
            if (x <= 0.0)
                return 0.0;
            if (x >= 1.0)
                return 1.0;

            if (a % 1.0 == 0 && b % 1.0 == 0 && a + b > 0)
                return this._rIBetaFunction1(x, a, b);
            if (b % 1.0 == 0 && a + b < 172)
                return this._rIBetaFunction2(x, a, b);
            if (a % 1.0 == 0 && a + b < 172)
                return 1.0 - this._rIBetaFunction2(1.0 - x, b, a);
            if (a == 0.5 && b == 0.5)
                return (2.0 / Math.PI) * Math.atan(Math.sqrt(x / (1.0 - x)));
            if (a == 0.5 && b % 0.5 == 0) {
                return 1.0 - this._rIBetaFunction(1.0 - x, b, 0.5);
            }
            if (a % 0.5 == 0 && b == 0.5) {
                if (a < 45) {
                    //CASE B (partial)
                    var s = 0;
                    var n = Math.round(a - 0.5);
                    var logGammaHalf = this._parseDouble(this.computeGammaln("0.5".toString()));
                    for (var i = 0; i < n; i++) {
                        s = s + Math.exp(this._gammaln(i + 1) - this._gammaln(i + 1.5) - logGammaHalf) * Math.pow(x, i);
                    }
                    return this._rIBetaFunction(x, 0.5, 0.5) - Math.sqrt(x * (1 - x)) * s;
                } else {
                    //CASE C (partial)
                    var M = this._betaFunction(a, 0.5);
                    var lambda = Math.sqrt(1.0 - Math.pow(M * Math.sqrt((a - 1.0) / Math.PI) * this._epsilon, 1.0 / (a - 1.0)));

                    //Gaussian weights
                    var weights = [
                        0.066671344308688, 0.14945134915058, 0.21908636251598, 0.26926671931000, 0.29552422471475,
                        0.29552422471475, 0.26926671931000, 0.21908636251598, 0.14945134915058, 0.066671344308688];

                    var mgas = [
                        0.013046735791414, 0.067468316655507, 0.16029521585049, 0.28330230293538, 0.42556283050918,
                        0.57443716949081, 0.71669769706462, 0.83970478414951, 0.93253168334449, 0.98695326420859];

                    var sqrtOneMinusX = Math.sqrt(1 - x);
                    var s = 0;
                    for (var i = 0; i < 10; i++) {
                        s = s + weights[i] * Math.pow(1.0 - Math.pow((lambda - sqrtOneMinusX) * mgas[i] + sqrtOneMinusX, 2), a - 1.0);
                    }
                    var l = Math.exp(this._gammaln(a + 0.5) - this._gammaln(a) - this._gammaln(0.5));
                    return (lambda - sqrtOneMinusX) * l * s;
                }
            }
            if (a % 0.5 == 0 && b % 0.5 == 0) {
                //CASE B and C
                var s = 0;
                var n = Math.round(b - 0.5);
                var logGammaA = this._gammaln(a);
                var xToTheA = Math.pow(x, a);
                for (var i = 0; i < n; i++) {
                    s = s + Math.exp(this._gammaln(a + i + 0.5) - logGammaA - this._gammaln(i + 1.5)) * Math.pow(1 - x, i) * xToTheA;
                }
                var d = this._rIBetaFunction(x, a, 0.5) + Math.sqrt(1 - x) * s;
                return Math.max(0.0, Math.min(1.0, d));
            }
            if (x > 0.5)
                return 1.0 - this._rIBetaFunction(1.0 - x, b, a);

            var ribf = this._iBetaFunction(x, a, b) / this._betaFunction(a, b);
            return Math.max(0.0, Math.min(1.0, ribf));
        };
        this._rIBetaFunction1 = function (x, a, b) {
            var s = 0;
            var c = a + b - 1;
            if (c < 21) {
                for (var i = a; i < a + b; i++) {
                    s = s + Math.pow(x, i) * Math.pow(1.0 - x, c - i) / (this._factorial(i) * this._factorial(c - i));
                }
                s = s * this._factorial(c);
            } else {
                for (var i = a; i < a + b; i++) {
                    s = s + Math.pow(x, i) * Math.pow(1.0 - x, c - i) * this._combinations(c, i);
                }
            }

            return Math.max(0.0, Math.min(1.0, s));
        };
        this._rIBetaFunction2 = function (x, a, b) {
            if (a + b > 172)
                throw "Cannot currently compute RegularizedIncompleteBetaFunction for a + b > 172";

            var s = 0;
            for (var i = 1; i < b + 1; i++) {
                s = s + Math.pow(1 - x, i - 1) * Math.exp(this._gammaln(a + i - 1) - this._gammaln(i));
            }
            s = s * (Math.pow(x, a) * Math.exp(-this._gammaln(a)));
            return s;
        };
        this._iBetaFunction = function (x, a, b) {
            if (x == 0)
                return 0;
            if (x == 1)
                return this._betaFunction(a, b);

            if (x <= 0.9) {
                return this._pIBetaFunction(0, x, a, b);
            }
            if (x <= 0.99) {
                var s1 = this._pIBetaFunction(0, 0.9, a, b);
                var s2 = this._pIBetaFunction(0.90, x, a, b);
                return s1 + s2;
            } else {
                var s1 = this._pIBetaFunction(0, 0.9, a, b);
                var s2 = this._pIBetaFunction(0.9, 0.99, a, b);
                var s3 = this._pIBetaFunction(0.99, x, a, b);
                return s1 + s2 + s3;
            }
        };
        this._pIBetaFunction = function (xL, xU, a, b) {
            if (xU == xL)
                return 0.0;
            if (xU < xL)
                return NaN;

            var nSteps = 1000;
            if (a < 1)
                nSteps = 80000;

            var dt = (xU - xL) / nSteps;
            var t = xL + 0.5 * dt;
            var sum = 0;
            for (var i = 0; i < nSteps; i++) {
                sum = sum + Math.pow(t, a - 1) * Math.pow(1.0 - t, b - 1) * dt;
                t = t + dt;
            }
            return sum;
        };
        this._betaFunction = function (a, b) {
            if (a + b > 143) {
                if (b > 20)
                    return 2.5066282746310002 * Math.pow(a, a - 0.5) * Math.pow(b, b - 0.5) / Math.pow(a + b, a + b - 0.5);
                return this._gammaFunction(b) * Math.pow(a, -b);
            }
            return this._gammaFunction(a) * this._gammaFunction(b) / this._gammaFunction(a + b);
        };
        this._gammaFunction = function (z) {
            //approximation based on Stirling's formula
            //based on approximation from NIST Handbook of Mathematical Function. 2010. Cambridge University Press.
            if (z > 143)
                throw "Cannot currently compute gamma function for z > 143";

            //need z<21 or z<11 becaus Factorial(x) will not work for x>20
            if (z > 0 && z < 21 && z % 1.0 == 0)
                return this._factorial(Math.round(z - 1));
            if (z > 0 && z < 11 && z % 0.5 == 0) {
                var n = z;
                var sqrtPi = 1.77245385090552;
                return sqrtPi * this._factorial(2 * n) / (Math.pow(4, n) * this._factorial(n));
            }

            var sqrtTwoPi = 2.5066282746310002;
            var d = 1.0 + 1.0 / (12 * z) + 1.0 / (288 * z * z) - 139.0 / (51840 * Math.pow(z, 3)) - 571.0 / (2488320 * Math.pow(z, 4)) + 163879.0 / (209018880 * Math.pow(z, 5)) + 5246819.0 / (75246796800 * Math.pow(z, 6));
            var g = Math.pow(z, z - 0.5) * Math.exp(-z) * sqrtTwoPi * d;
            return g;
        };
        this._epsilon = 4.94066e-324;
        this._factorial = function (n) {
            if (n < 0)
                throw "Factorial not defined for negative n";
            if (n > 20)
                throw "Answer will exceed max long";
            var fact = 1;
            for (var i = n; i > 0; i--)
                fact = fact * i;
            return fact;
        };
        this._combinations = function (n, k) {
            var logCombin = this._logCombin(n, k);
            var combin = Math.exp(logCombin);
            return Math.round(combin);
        };
        this._logCombin = function (n, k) {
            return this._logFactorial(n) - this._logFactorial(k) - this._logFactorial(n - k);
        };
        this._logFactorial = function (x) {
            var lf = 0;
            for (var i = 2; i <= x; i++)
                lf = lf + Math.log(i);
            return lf;
        };
        this._isCellReference = function (args) {
            if (args == "") {
                return false;
            }
            args = this.setTokensForSheets(args);
            var sheetToken1 = this._sheetToken(args);
            var containsBoth = false;
            if (sheetToken1 != "") {
                args = args.split(this.sheetToken).join("");
            }

            var isAlpha = false, isNum = false;
            if (args.indexOf(':') != args.lastIndexOf(':')) {
                return false;
            }
            var charArray = (args.split('').join(this.getParseArgumentSeparator())).split(this.getParseArgumentSeparator());
            for (var c in charArray) {
                if (this._isLetter(charArray[c])) {
                    isAlpha = true;
                } else if (this._isDigit(charArray[c])) {
                    if (charArray[c] === 0)
                        return false;
                    isNum = true;
                } else if (charArray[c] == ':') {
                    if (isAlpha && isNum) {
                        containsBoth = true;
                    }
                    isAlpha = false;
                    isNum = false;
                } else
                    return false;
            }
            if (args.indexOf(":") > -1 && args.indexOf(this.tic) == -1) {
                if (containsBoth && isAlpha && isNum)
                    return true;
                else if (((isAlpha && !isNum) || (!isAlpha && isNum)) && !containsBoth) {
                    return true;
                } else
                    return false;
            }
            if (isAlpha && isNum && args.indexOf(this.tic) == -1) {
                return true;
            }

            return false;
        };
        this._isDate = function (dateString) {
            var date = new Date(Date.parse(dateString));
            if (date >= this._dateTime1900) {
                return date;
            } else
                return "NaN";
        };
        this._isDigit = function (text) {
            var charCode = text.charCodeAt(0);

            if ((charCode > 47) && (charCode < 58)) {
                return true;
            }
            return false;
        };
        this._isHLookupCachingEnabled = function () {
            return ((this._enableLookupTableCaching & 3 /* Both */) != 0) || ((this._enableLookupTableCaching & 2 /* HLOOKUP */) != 0);
        };
        this._isLetter = function (c) {
            var cc = c.charCodeAt(0);
            if ((cc >= 0x41 && cc <= 0x5A) || (cc >= 0x61 && cc <= 0x7A)) {
                return true;
            }
            return false;
        };
        this._isLetterOrDigit = function (c) {
            var cc = c.charCodeAt(0);
            if ((cc >= 0x30 && cc <= 0x39) || (cc >= 0x41 && cc <= 0x5A) || (cc >= 0x61 && cc <= 0x7A)) {
                return true;
            }
            return false;
        };
        this._isLookupCachingEnabled = function () {
            return ((this._enableLookupTableCaching & 3 /* Both */) != 0) || ((this._enableLookupTableCaching & 1 /* VLOOKUP */) != 0) || ((this._enableLookupTableCaching & 2 /* HLOOKUP */) != 0);
        };
        this._isOptimizedMatchesEnabled = function () {
            return (this._enableLookupTableCaching & 4 /* OptimizeForMatches */) != 0;
        };
        this._isRange = function (range) {
            ////checks if range holds a stand-alone range. Used to id when user inputs
            ////a formula that is just a range. Part of the support for using ranges as
            ////operands of binary operators. This method allows a range to be used with
            ////the formula assignment operator, =
            var isRange = false;
            var i = range.indexOf(':');
            if (i > 1 && i < range.length - 2) {
                ////check left side
                var j = i - 1;

                if (this._isDigit(range[j])) {
                    //// left side must end in digit
                    var needToCheckRightSide = false;
                    j--;
                    while (j > 0 && this._isDigit(range[j])) {
                        j--;
                    }

                    if (this._isLetter(range[j])) {
                        ////letters must come now
                        j--;
                        while (j >= 0 && this._isLetter(range[j])) {
                            j--;
                        }

                        if (j > -1 && range[j] == this._string_fixedreference[0]) {
                            j--;
                        }

                        if (j < 0) {
                            needToCheckRightSide = true; ////might have a range
                        } else {
                            if (range[j] == this.sheetToken) {
                                if (j-- > 1 && range[j] == this.tic[0]) {
                                    needToCheckRightSide = range.substring(0, j - 1).lastIndexOf(this.tic[0]) == 0;
                                } else if (j > 0 && this._isDigit(range[j])) {
                                    needToCheckRightSide = range.substring(0, j).lastIndexOf(this.sheetToken) == 0;
                                }
                            }
                        }
                    }

                    if (needToCheckRightSide) {
                        ////check right side
                        j = i + 1;

                        ////handle possible sheetnames
                        if (j < range.length - 6 && range[j] == this.tic[0]) {
                            j = range.indexOf(this.sheetToken, j + 1);
                            if (j < range.length - 2) {
                                j++;
                            }
                        }

                        if (j < range.length - 2 && range[j] == this._string_fixedreference[0]) {
                            j++;
                        }

                        if (this._isLetter(range[j])) {
                            j++;
                            while (j < range.length - 1 && this._isLetter(range[j])) {
                                j++;
                            }

                            if (this._isDigit(range[j])) {
                                j++;
                                while (j < range.length && this._isDigit(range[j])) {
                                    j++;
                                }

                                isRange = j == range.length;
                            }
                        }
                    }
                }
            }

            return isRange;
        };
        this._isUpperChar = function (text) {
            var charCode = text.charCodeAt(0);
            return ((charCode > 64) && (charCode < 91));
        };
        this._iisVLookupCachingEnabled = function () {
            return ((this._enableLookupTableCaching & 3 /* Both */) != 0) || ((this._enableLookupTableCaching & 1 /* VLOOKUP */) != 0);
        };
        this._mark = function (copy, text, op, token, checkPrecedingChar) {
            var loc = 0;
            if (checkPrecedingChar) {
                loc = this._findAndCheckPrecedingChar(copy, op, loc);
                copy = this._findAndCheckPrecedingCharCopy;
            } else
                loc = copy.indexOf(op);
            var len = op.length;
            while (loc > -1) {
                copy = copy.substring(0, loc) + token + copy.substring(loc + len);
                text = text.substring(0, loc) + token + text.substring(loc + len);
                if (checkPrecedingChar) {
                    loc = this._findAndCheckPrecedingChar(copy, op, loc);
                    copy = this._findAndCheckPrecedingCharCopy;
                } else
                    loc = copy.indexOf(op, loc);
            }
            this._markCopy = "";
            return text;
        };
        this._markColonsInQuotes = function (args) {
            var inQuotes = false;
            for (var i = 0; i < args.length; ++i) {
                if (args[i] == this.tic[0]) {
                    inQuotes = !inQuotes;
                } else if (args[i] == ':' && inQuotes) {
                    args = args.split(':').join(this._markerChar);
                }
            }
            return args;
        };
        this._markIF = function (copy, text) {
            var locIF = copy.indexOf(this.getReservedWordOperators()[this._reservedWord_IF]);
            var locTHEN = copy.indexOf(this.getReservedWordOperators()[this._reservedWord_THEN]);
            var locELSE = copy.indexOf(this.getReservedWordOperators()[this._reservedWord_ELSE]);

            if (locELSE > -1 && locTHEN > -1) {
                var ifStart = this.getReservedWordOperators()[this._reservedWord_IF].length;
                var iflength = locTHEN - ifStart;
                var thenStart = locTHEN + this.getReservedWordOperators()[this._reservedWord_THEN].length;
                var thenlength = locELSE - thenStart;
                var elseStart = locELSE + this.getReservedWordOperators()[this._reservedWord_ELSE].length;
                var elselength = text.length - elseStart;
                text = "IF((" + this._substring(text, ifStart, iflength) + ")" + this.getParseArgumentSeparator() + "(" + this._substring(text, thenStart, thenlength) + ")" + this.getParseArgumentSeparator() + "(" + this._substring(text, elseStart, elselength) + "))";

                copy = text;
            } else if (locTHEN > -1) {
                var ifStart = 0;
                var iflength = locTHEN;
                var thenStart = locTHEN + this.getReservedWordOperators()[this._reservedWord_THEN].length;
                var thenlength = text.length - thenStart + 1;
                text = "IF((" + this._substring(text, ifStart, iflength) + ")" + this.getParseArgumentSeparator() + "(" + this._substring(text, thenStart, thenlength) + "))";

                copy = text;
            }

            this._markIFCopy = text;
            return text;
        };
        this._markLibraryFormulas = function (formula) {
            var rightParens = formula.indexOf(')');
            if (rightParens == -1) {
                formula = this._markNamedRanges(formula);
            } else {
                while (rightParens > -1) {
                    var parenCount = 0;
                    var leftParens = rightParens - 1;
                    while (leftParens > -1 && (formula[leftParens] != '(' || parenCount != 0)) {
                        if (formula[leftParens] == ')') {
                            parenCount++;
                        } else if (formula[leftParens] == ')') {
                            parenCount--;
                        }

                        leftParens--;
                    }

                    if (leftParens == -1) {
                        throw this.formulaErrorStrings[this._mismatched_parentheses];
                    }

                    var i = leftParens - 1;
                    while (i > -1 && (this._isLetterOrDigit(formula[i]) || this._validFunctionNameChars.indexOf(formula[i]) > -1 || formula[i] == this.getParseDecimalSeparator())) {
                        i--;
                    }

                    var len = leftParens - i - 1;
                    if (len > 0 && this.getLibraryFunctions().getItem(this._substring(formula, i + 1, len)) != undefined) {
                        if (this._substring(formula, i + 1, len) == "AREAS")
                            this._ignoreBracet = true;
                        else
                            this._ignoreBracet = false;
                        var s = this._substring(formula, leftParens, rightParens - leftParens + 1);
                        s = this._markNamedRanges(s);
                        formula = formula.substring(0, i + 1) + 'q' + this._substring(formula, i + 1, len) + (s.split('(').join(this._leftBracket)).split(')').join(this._rightBracket) + formula.substring(rightParens + 1);
                    } else if (len > 0) {
                        if (this.unknownFunction != null) {
                            var family = CalcEngine.getSheetFamilyItem(this.grid);
                            var grd = this.grid;
                            var sheet = this._sheetToken(this.cell);
                            var s = this.cell;
                            if (sheet.length > 0) {
                                grd = family.tokenToParentObject.getItem(sheet);
                                s = s.substring(s.lastIndexOf(this.sheetToken) + 1);
                            }

                            var sheetNamekeys = family.sheetNameToParentObject.keys();
                            for (var key in sheetNamekeys) {
                                if (family.sheetNameToParentObject.getItem(sheetNamekeys[key]) == grd) {
                                    s = sheetNamekeys[key] + this.sheetToken + s;
                                    break;
                                }
                            }

                            var args = new UnknownFunctionEventArgs();
                            args.setMissingFunctionName(this._substring(formula, i + 1, len));
                            args.setCellLocation(s);

                            this.unknownFunction(this, args);
                            if (formula.substring(i + 1, len) == "AREAS")
                                this._ignoreBracet = true;
                            else
                                this._ignoreBracet = false;
                        }
                        if (this.getRethrowLibraryComputationExceptions())
                            throw this.formulaErrorStrings[this._unknown_formula_name] + " " + this._substring(formula, i + 1, len);
                        return this.getErrorStrings()[5].toString();

                    } else {
                        var s = "";
                        if (leftParens > 0) {
                            s = formula.substring(0, leftParens);
                        }

                        s = s + '{' + this._substring(formula, leftParens + 1, rightParens - leftParens - 1) + '}';
                        if (rightParens < formula.length) {
                            s = s + formula.substring(rightParens + 1);
                        }

                        s = this._markNamedRanges(s);
                        formula = s;
                    }

                    rightParens = formula.indexOf(')');
                }
            }

            formula = (formula.split('{').join('(')).split('}').join(')');
            return formula;
        };
        this._markNamedRanges = function (argList) {
            ////Console.WriteLine("MarkNamedRanges in#{0}#", argList);
            var rightParens = argList.indexOf(')');
            var markers = [')', this.getParseArgumentSeparator(), '}', '+', '-', '*', '/', '<', '>', '=', '&'];

            //  int i = (argList.length > 0 && argList[0] == '(')
            var i = (argList.length > 0 && (argList[0] == '(' || argList[0] == '{')) ? 1 : 0;
            if (argList.indexOf("#N/A") > -1) {
                argList = argList.split("#N/A").join("#N~A");
            }
            if (argList.indexOf("#DIV/0!") > -1) {
                argList = argList.split("#DIV/0!").join("#DIV~0!");
            }
            var end = this._indexOfAny(argList.substring(i), markers);

            while (end > -1 && end + i < argList.length) {
                var scopedRange = "";
                var s = this.getNamedRanges().getItem(this._substring(argList, i, end));
                if (s != undefined && !(typeof s === 'string'))
                    s = s.getItem(this._substring(argList, i, end));
                if (s == null) {
                    scopedRange = this._checkIfScopedRange(this._substring(argList, i, end));
                    if (!isNaN(scopedRange)) {
                        s = scopedRange;
                    } else if (this._substring(argList, i, end).startsWith(this.sheetToken.toString())) {
                        var iii = this._substring(argList, i, end).indexOf(this.sheetToken, 1);
                        if (iii > 1) {
                            s = this.getNamedRanges().getItem(this._substring(argList.substring(i), iii + 1, end - iii - 1));
                        }
                    }
                }
                if (s != null) {
                    s = s.toUpperCase();
                    s = this.setTokensForSheets(s);
                    s = this._markLibraryFormulas(s); ////recursive call
                }

                if (s != null) {
                    argList = argList.substring(0, i) + s + argList.substring(i + end);
                    i += s.length + 1;
                } else {
                    i += end + 1;
                    while (i < argList.length && !this._isUpperChar(argList[i]) && argList[i] != this.sheetToken) {
                        i++;
                    }
                }

                end = this._indexOfAny(argList.substring(i), markers);
                while (end == 0 && i < argList.length - 1) {
                    i++;
                    end = this._indexOfAny(argList.substring(i), markers);
                }

                if (end == -1 && i < argList.length) {
                    ////check last part of arglist
                    s = this.getNamedRanges().length > 0 ? this.getNamedRanges().getItem(argList.substring(i)) : s;
                    if (s == null) {
                        scopedRange = this._checkIfScopedRange(argList.substring(i));
                        if (!isNaN(scopedRange))
                            s = scopedRange;
                    }
                    if (s != null) {
                        s = s.toUpperCase();
                        s = this.setTokensForSheets(s);
                        s = this._markLibraryFormulas(s);
                        if (s != null) {
                            argList = argList.substring(0, i) + s; //// +argList.Substring(i + end);
                            i += s.toString().length + 1;
                        }
                    }

                    end = (i < argList.length) ? this._indexOfAny(argList.substring(i), markers) : -1;
                }
            }
            if (argList.indexOf("#N~A") > -1) {
                argList = argList.split("#N~A").join("#N/A");
            }

            if (argList.indexOf("#DIV~0!") > -1) {
                argList = argList.split("#DIV~0!").join("#DIV/0!");
            }
            return argList;
        };
        this._markReserveWords = function (text) {
            var copy = text.toLowerCase();

            if (copy.trim()[0] == (this.getReservedWordOperators()[this._reservedWord_IF])) {
                text = this._markIF(copy, text);
                copy = this._markIFCopy;
            }

            text = this._mark(copy, text, this.getReservedWordOperators()[this._reservedWord_NOT], this.char_NOTop, true);
            copy = this._markCopy;
            text = this._mark(copy, text, this.getReservedWordOperators()[this._reservedWord_OR], this.char_ORop, false);
            copy = this._markCopy;
            text = this._mark(copy, text, this.getReservedWordOperators()[this._reservedWord_AND], this.char_ANDop, false);
            copy = this._markCopy;
            text = this._mark(copy, text, this.getReservedWordOperators()[this._reservedWord_XOR], this.char_XORop, false);
            copy = this._markCopy;
            return text;
        };
        this._markupResultToIncludeInFormula = function (s) {
            var d3;
            if (s.length > 0 && s[0] == '-' && !isNaN(this._parseDouble(s))) {
                s = "nu" + s.substring(1);
            } else if (s.length > 0 && (s[0] == this.tic[0] || s[0] == this.bMARKER || s[0] == '#')) {
                ////Pass on the string...
            } else if (s[0] == this.trueValueStr || s[0] == this.falseValueStr) {
                ////Pass on the bool...
            } else {
                ////if(double.TryParse(s, NumberStyles.Any, null, out d))
                if (!isNaN(this._parseDouble(s))) {
                    // s = d.toString(); ////defect 8721
                    s = s.split(this.getParseArgumentSeparator()).join(String.fromCharCode(32));

                    s = 'n' + s;
                } else {
                    if (!this._isRange(s)) {
                        s = this.tic + s + this.tic;
                    }
                }
            }
            return s;
        };
        this._matchCompare = function (o1, o2) {
            var s1 = o1.toString();
            var s2 = o2.toString();
            var d1 = this._parseDouble(s1), d2 = this._parseDouble(s2);
            if (!isNaN(d1) && !isNaN(d2)) {
                if (d1 == d2)
                    return 0;
                if (d1 > d2)
                    return 1;
                else
                    return -1;
            } else {
                if (s1 == s2)
                    return 0;
                if (s1 > s2)
                    return 1;
                else
                    return -1;
            }
        };
        this._matchingRightBracket = function (formula) {
            var ret = -1;
            var loc = 1;
            var bracketLevel = 0;
            var token = this._sheetToken(formula);
            while (ret == -1 && loc < formula.length) {
                if (formula[loc] == this._rightBracket) {
                    if (bracketLevel == 0) {
                        ret = loc;
                    } else {
                        bracketLevel--;
                        if (bracketLevel == 0 && loc == formula.length - 1) {
                            ret = loc;
                        }
                    }
                } else if (formula[loc] == 'q') {
                    var val = loc + 1;
                    if (val < formula.length) {
                        var libFunc = this._processUpperCase(formula, val, token);
                        formula = this._processUpperCaseFormula;
                        val = this._processUpperCaseIvalue;
                        token = this._processUpperCaseSheet;
                        if (libFunc != "" && this.getLibraryFunctions().containsKey(libFunc))
                            bracketLevel++;
                    }
                }

                loc++;
            }

            return ret;
        };
        this._notInBlock = function (text, position) {
            var i = text.indexOf(this.bMARKER);
            var inBlock = false;
            while (i > -1 && i < position) {
                inBlock = !inBlock;
                i = text.indexOf(this.bMARKER, i + 1);
            }

            return !inBlock;
        };
        this._parse = function (formulaText) {
            var text = formulaText;
            if (this._isTextEmpty(text)) {
                return text;
            }
            text = this.getFormulaText(text);

            if (this.getSupportLogicalOperators()) {
                text = this._markReserveWords(text);
            }

            if (this.getFormulaCharacter() != String.fromCharCode(0) && this.getFormulaCharacter() == text[0]) {
                text = text.substring(1);
            }

            ////Special check for single namedrange.
            if (this.getNamedRanges().length > 0) {
                if (this.getNamedRanges().getItem(text.toUpperCase()) != undefined) {
                    text = this.getNamedRanges().getItem(text.toUpperCase()).toUpperCase();
                } else {
                    var scopedRange = "";
                    scopedRange = this._checkIfScopedRange(text.toUpperCase());
                    if (!isNaN(scopedRange)) {
                        text = scopedRange.toUpperCase();
                    }
                }
            }

            var formulaStrings = this._saveStrings(text);
            text = this._saveStringsText;
            text = text.split(this.braceLeft).join(this.tic);
            text = text.split(this.braceRight).join(this.tic);

            text = text.split("-+").join("-");
            var i = 0;
            if (text[text.length - 1] != this.bMARKER || this._indexOfAny(text, this.tokens) != (text.length - 2))
                text = text.toUpperCase();

            if (text.indexOf(this.sheetToken) > -1) {
                //replace sheet references with tokens.
                var family = CalcEngine.getSheetFamilyItem(this.grid);
                if (family.sheetNameToParentObject != null && family.sheetNameToParentObject.length > 0) {
                    try  {
                        text = this.setTokensForSheets(text);
                    } catch (ex) {
                        if (this._rethrowExceptions) {
                            throw ex;
                        } else {
                            return ex;
                        }
                    }
                }
            }

            if (this._isRangeOperand) {
                this._isRangeOperand = false;
                return this._getCellFrom(this._parseSimple(text));
            }

            ////next 3 lines moved from above - //keep the tic defect#541
            text = text.split(" ").join("");
            text = text.split("=>").join(">=");
            text = text.split("=<").join("<=");

            try  {
                text = this._markLibraryFormulas(text);
            } catch (ex) {
                if (this._rethrowExceptions) {
                    throw ex;
                } else {
                    return ex;
                }
            }

            ////Look for inner matching and parse pieces without parens with ParseSimple.
            if (!this._ignoreBracet) {
                while ((i = text.indexOf(')')) > -1) {
                    var k = text.substring(0, i).lastIndexOf('(');
                    if (k == -1) {
                        throw this.formulaErrorStrings[this._mismatched_parentheses];
                    }

                    if (k == i - 1) {
                        throw this.formulaErrorStrings[this._empty_expression];
                    }
                    var s = "";
                    if (this._ignoreBracet) {
                        s = this._substring(text, k, i - k + 1);
                    } else
                        s = this._substring(text,k + 1, i - k - 1);
                    text = text.substring(0, k) + this._parseSimple(s) + text.substring(i + 1);
                }
            }

            ////All parens should have been removed.
            if (!this._ignoreBracet && text.indexOf('(') > -1) {
                throw this.formulaErrorStrings[this._mismatched_parentheses];
            }

            var retValue = this._parseSimple(text);

            if (formulaStrings != null && formulaStrings.length > 0) {
                retValue = this._setStrings(retValue, formulaStrings);
            }
            return retValue;
        };
        this._parseDouble = function (value) {
            return !isNaN(Globalize.parseInt(value)) ? Number(value.replace(/[^0-9\.]-+/g,"")) * 1 : NaN;
        };
        this._parseSimple = function (formulaText) {
            var text = formulaText;

            ////strip leading plus
            if (text.length > 0 && text[0] == '+') {
                text = text.substring(1);
            }

            if (text == "#N/A" || text == "#N~A") {
                return "#N/A";
            } else if (text.indexOf("#N/A") > -1) {
                text = text.split("#N/A").join("#N~A");
            }
            if (text == "#DIV/0!" || text == "#DIV~0!") {
                return "#DIV/0!";
            } else if (text.indexOf("#DIV/0!") > -1) {
                text = text.split("#DIV/0!").join("#DIV~0!");
            }

            text = this._handleEmbeddedEs(text);

            text = text.split(this._string_lesseq).join(this.char_lesseq);
            text = text.split(this._string_greatereq).join(this.char_greatereq);
            text = text.split(this._string_noequal).join(this.char_noequal);
            text = text.split(this._string_fixedreference).join(this._string_empty);
            text = text.split(this._string_or).join(this.char_or);
            text = text.split(this._string_and).join(this.char_and);

            if (text == "") {
                return text;
            }

            var needToContinue = true;

            ////doing things sequentially imposes computation hierarchy for 6 levels
            //// 0. E+ E- //handles exponential notation like 1.2e-1 or 1.2e+1 to 1.2e1
            //// 1. ^ exponentiation
            //// 2. * /
            //// 3. + -
            //// 4. < > = <= >= <>
            //// 5. & concatenation
            //// in each level parsing done left to right
            var expTokenArray = [this.token_EP, this.token_EM];
            var orTokenArray = [this.token_or];
            var mulTokenArray = [this.token_multiply, this.token_divide];
            var addTokenArray = [this.token_add, this.token_subtract];
            var compareTokenArray = [this.token_less, this.token_greater, this.token_equal, this.token_lesseq, this.token_greatereq, this.token_noequal];
            var notTokenArray = [this.token_NOTop];
            var logicTokenArray = [this.token_ORop, this.token_ANDop, this.token_XORop];
            var andTokenArray = [this.token_and];

            var expCharArray = [this.char_EP, this.char_EM];
            var orCharArray = [this.char_or];
            var mulCharArray = [this.char_multiply, this.char_divide];
            var addCharArray = [this.char_add, this.char_subtract];
            var compareCharArray = [this.char_less, this.char_greater, this.char_equal, this.char_lesseq, this.char_greatereq, this.char_noequal];
            var notCharArray = [this.char_NOTop];
            var logicCharArray = [this.char_ORop, this.char_ANDop, this.char_XORop];
            var andCharArray = [this.char_and];

            text = this._parseSimpleOperators(text, expTokenArray, expCharArray);

            text = this._parseSimpleOperators(text, orTokenArray, orCharArray);

            if (needToContinue) {
                text = this._parseSimpleOperators(text, mulTokenArray, mulCharArray);
            }

            if (needToContinue) {
                text = this._parseSimpleOperators(text, addTokenArray, addCharArray);
            }

            if (needToContinue) {
                text = this._parseSimpleOperators(text, compareTokenArray, compareCharArray);
            }

            if (needToContinue) {
                text = this._parseSimpleOperators(text, notTokenArray, notCharArray);
            }

            if (needToContinue) {
                text = this._parseSimpleOperators(text, logicTokenArray, logicCharArray);
            }

            if (needToContinue) {
                text = this._parseSimpleOperators(text, andTokenArray, andCharArray);
            }

            if (text.indexOf("#N~A") > -1) {
                text = text.split("#N~A").join("#N/A");
            }

            if (text.indexOf("#DIV~0!") > -1) {
                text = text.split("#DIV~0!").join("#DIV/0!");
            }
            return text;
        };
        this._parseSimpleOperators = function (formulaText, markers, operators) {
            var text = formulaText;
            var i;

            ////op is string containing this.char_xxxx's for each operator in this level
            var op = "";
            for (var c = 0; c < operators.length; c++) {
                op = op + operators[c];
            }

            //text = sb.replace(",-", ",u").replace(this._leftBracket + "-", this._leftBracket + "u").replace("=-", "=u").replace(">-", ">u").replace("<-", "<u").replace("/-", "/u").replace("*-", "*u").replace("+-", "+u").replace("--", "-u").replace("w-", "wu").toString();
            text = text.split("---").join("-").split("--").join("+").split(this.getParseArgumentSeparator() + "-").join(this.getParseArgumentSeparator() + "u").split(this._leftBracket + "-").join(this._leftBracket + "u").split("=-").join("=u");
            text = text.split(">-").join(">u").split("<-").join("<u").split("/-").join("/u").split("*-").join("*u").split("+-").join("-").split("--").join("-u").split("w-").join("wu").split(this.tic + "-").join(this.tic + "u").toString();

            ////Get rid of leading pluses.
            text = text.split(",+").join(",").split(this._leftBracket + "+").join(this._leftBracket).split("=+").join("=").split(">+").join(">").split("<+").join("<").split("/+").join("/").split("*+").join("*").split("++").join("+").toString();

            if (text.length > 0 && text[0] == '-') {
                ////Leading unary minus.
                text = "0" + text;
            } else if (text.length > 0 && text[0] == '+') {
                ////Leading plus.
                text = text.substring(1);
            }

            if (this._indexOfAny(text, operators) > -1) {
                while ((i = this._indexOfAny(text, operators)) > -1) {
                    var left = "";
                    var right = "";

                    var leftIndex = 0;
                    var rightIndex = 0;
                    var isNotOperator = this._supportLogicalOperators && text[i] == this.char_NOTop;
                    var j = 0;

                    if (!isNotOperator) {
                        if (i < 1 && text[i] != '-') {
                            throw this.formulaErrorStrings[this._operators_cannot_start_an_expression];
                        }

                        j = i - 1;

                        if (i == 0 && text[i] == '-') {
                            ////Unary minus - block and continue.
                            text = this.bMARKER + "nu" + text.substring(1) + this.bMARKER;
                            continue;
                        } else if (text[j] == this.tic[0]) {
                            ////string
                            var k = text.substring(0, j - 1).lastIndexOf(this.tic);
                            if (k < 0) {
                                throw this.formulaErrorStrings[this._cannot_parse];
                            }

                            left = this._substring(text, k, j - k + 1); ////Keep the tics.
                            leftIndex = k;
                        } else if (text[j] == this.bMARKER) {
                            var k = this._findLastNonQB(text.substring(0, j - 1));
                            if (k < 0) {
                                throw this.formulaErrorStrings[this._cannot_parse];
                            }
                            left = this._substring(text, k + 1, j - k - 1);
                            leftIndex = k + 1;
                        } else if (text[j] == this._rightBracket) {
                            ////Library member.
                            var bracketCount = 0;
                            var k = j - 1;
                            while (k > 0 && (text[k] != 'q' || bracketCount != 0)) {
                                if (text[k] == 'q') {
                                    bracketCount--;
                                } else if (text[k] == this._rightBracket) {
                                    bracketCount++;
                                }

                                k--;
                            }

                            if (k < 0) {
                                throw this.formulaErrorStrings[this._cannot_parse];
                            }

                            left = this._substring(text, k, j - k + 1);
                            leftIndex = k;
                        } else if (!this._isDigit(text[j]) && text[j] != '%') {
                            while (j >= 0 && (this._isUpperChar(text[j]) || text[j] == '_' || text[j] == '.')) {
                                j--;
                            }

                            left = this._substring(text, j + 1, i - j - 1); ////'n' for number
                            leftIndex = j + 1;
                            var scopedRange = "";

                            if (this.getNamedRanges().containsKey(left) > -1) {
                                left = this._parse(this.getNamedRanges()[left]);
                            } else if (!isNaN(scopedRange = this._checkIfScopedRange(left, scopedRange))) {
                                left = this._parse(scopedRange);
                            } else if (left == this.trueValueStr) {
                                left = 'n' + this.trueValueStr;
                            } else if (left == this.falseValueStr) {
                                left = 'n' + this.falseValueStr;
                            } else {
                                if (this.getRethrowLibraryComputationExceptions())
                                    throw this.formulaErrorStrings[this._invalid_char_in_front_of]  + " " + text[i];
                                return this.getErrorStrings()[5].toString();
                            }
                        } else {
                            var period = false;
                            var percent = false;

                            while (j > -1 && (this._isDigit(text[j]) || (!period && text[j] == this.getParseDecimalSeparator()) || (!percent && text[j] == '%') || text[j] == 'u')) {
                                if (text[j] == this.getParseDecimalSeparator()) {
                                    period = true;
                                } else if (text[j] == '%') {
                                    percent = true;
                                }
                                j = j - 1;
                            }

                            ////Add error check for 2%.
                            if (j > -1 && period && text[j] == this.getParseDecimalSeparator()) {
                                throw this.formulaErrorStrings[this._number_contains_2_decimal_points];
                            }

                            j = j + 1;

                            if (j == 0 || (j > 0 && !this._isUpperChar(text[j - 1]))) {
                                left = 'n' + this._substring(text, j, i - j); ////'n' for number
                                leftIndex = j;
                            } else {
                                ////We have a cell reference.
                                j = j - 1;
                                while (j > -1 && this._isUpperChar(text[j])) {
                                    j = j - 1;
                                }

                                ////include any unary minus as part of the left piece
                                if (j > -1 && text[j] == 'u') {
                                    j = j - 1;
                                }
                                if (j > -1 && text[j] == this.sheetToken) {
                                    j = j - 1;
                                    while (j > -1 && text[j] != this.sheetToken) {
                                        j = j - 1;
                                    }

                                    if (j > -1 && text[j] == this.sheetToken) {
                                        j = j - 1;
                                    }
                                }

                                if (j > -1 && text[j] == ':') {
                                    //// handle range operands
                                    j = j - 1;
                                    while (j > -1 && this._isDigit(text[j])) {
                                        j = j - 1;
                                    }

                                    while (j > -1 && this._isUpperChar(text[j])) {
                                        j = j - 1;
                                    }

                                    if (j > -1 && text[j] == this.sheetToken) {
                                        j--;
                                        while (j > -1 && text[j] != this.sheetToken) {
                                            j--;
                                        }

                                        if (j > -1 && text[j] == this.sheetToken) {
                                            j--;
                                        }
                                    }

                                    j = j + 1;
                                    left = this._substring(text, j, i - j);
                                    left = this._getCellFrom(left);
                                } else {
                                    //// handle normal cell reference
                                    j = j + 1;
                                    left = this._substring(text, j, i - j);
                                }
                                this.updateDependencies(left);
                                leftIndex = j;
                            }
                        }
                    } else {
                        leftIndex = i;
                    }
                    if (i == text.length - 1) {
                        throw this.formulaErrorStrings[this._expression_cannot_end_with_an_operator];
                    } else {
                        j = i + 1;

                        var isU = text[j] == 'u';
                        if (isU) {
                            j++; ////ship for now, but add back later
                        }

                        if (text[j] == this.tic[0]) {
                            ////string
                            var k = text.substring(j + 1).indexOf(this.tic);
                            if (k < 0) {
                                throw this.formulaErrorStrings[this._cannot_parse];
                            }

                            right = this._substring(text, j, k + 2);
                            rightIndex = k + j + 2;
                        } else if (text[j] == this.bMARKER) {
                            ////Block of already parsed code.
                            var k = this._findNonQB(text.substring(j + 1));
                            if (k < 0) {
                                throw this.formulaErrorStrings[this._cannot_parse];
                            }

                            right = this._substring(text, j + 1, k);

                            if (isU) {
                                right = right + "nu1m"; ////multiply quantity by -1...
                            }

                            rightIndex = k + j + 2;
                        } else if (text[j] == 'q') {
                            ////library
                            var bracketCount = 0;
                            var k = j + 1;
                            while (k < text.length && (text[k] != this._rightBracket || bracketCount != 0)) {
                                if (text[k] == this._rightBracket) {
                                    bracketCount++;
                                } else if (text[k] == 'q') {
                                    bracketCount--;
                                }

                                k++;
                            }

                            if (k == text.length) {
                                throw this.formulaErrorStrings[this._cannot_parse];
                            }

                            right = this._substring(text, j, k - j + 1);

                            if (isU) {
                                right = 'u' + right;
                            }

                            rightIndex = k + 1;
                        } else if (this._isDigit(text[j]) || text[j] == this.getParseDecimalSeparator()) {
                            var period = text[j] == this.getParseDecimalSeparator();
                            j = j + 1;
                            while (j < text.length && (this._isDigit(text[j]) || (!period && text[j] == this.getParseDecimalSeparator()))) {
                                if (text[j] == this.getParseDecimalSeparator()) {
                                    period = true;
                                }

                                j = j + 1;
                            }

                            if (j < text.length && text[j] == '%') {
                                j += 1;
                            }

                            if (period && j < text.length && text[j] == this.getParseDecimalSeparator()) {
                                throw this.formulaErrorStrings[this._number_contains_2_decimal_points];
                            }
                            right = 'n' + this._substring(text, i + 1, j - i - 1);
                            rightIndex = j;
                        } else if (this._isUpperChar(text[j]) || text[j] == this.sheetToken || text[j] == 'u') {
                            if (text[j] == this.sheetToken) {
                                j++;
                                while (j < text.length && text[j] != this.sheetToken) {
                                    j++;
                                }
                            }
                            j = j + 1;
                            var j0 = 0;
                            while (j < text.length && (this._isUpperChar(text[j]) || text[j] == '_' || text[j] == '.')) {
                                j++;
                                j0++;
                            }

                            var noCellReference = (j == text.length) || !this._isDigit(text[j]);

                            if (j0 > 4) {
                                while (j < text.length && (this._isUpperChar(text[j]) || this._isDigit(text[j]))) {
                                    j++;
                                }

                                noCellReference = true;
                            }

                            while (j < text.length && this._isDigit(text[j])) {
                                j = j + 1;
                            }

                            if (j < text.length && text[j] == ':') {
                                //// handle range operands
                                j = j + 1;
                                if (j < text.length && text[j] == this.sheetToken) {
                                    j++;
                                    while (j < text.length && text[j] != this.sheetToken) {
                                        j = j + 1;
                                    }

                                    if (j < text.length && text[j] == this.sheetToken) {
                                        j++;
                                    }
                                }

                                while (j < text.length && this._isUpperChar(text[j])) {
                                    j = j + 1;
                                }

                                while (j < text.length && this._isDigit(text[j])) {
                                    j = j + 1;
                                }

                                j = j - 1;

                                right = this._substring(text, i + 1, j - i);
                                right = this._getCellFrom(right);
                            } else {
                                //// handle normal cell reference
                                j = j - 1;
                                right = this._substring(text, i + 1, j - i);
                                isU = text[j] == 'u';
                                if (isU) {
                                    right = 'u' + right;
                                }
                            }
                            if (!noCellReference) {
                                this.updateDependencies(right);
                            } else {
                                var scopedRange = "";
                                if (this.getNamedRanges().getItem(right) > -1) {
                                    right = this._parse(this.getNamedRanges()[right]);
                                } else if (!isNaN(scopedRange = this._checkIfScopedRange(right, scopedRange))) {
                                    right = this._parse(scopedRange);
                                } else if (right == this.trueValueStr) {
                                    right = 'n' + this.trueValueStr;
                                } else if (right == this.falseValueStr) {
                                    right = 'n' + this.falseValueStr;
                                } else {
                                    if(this.getRethrowLibraryComputationExceptions())
                                        throw this.formulaErrorStrings[this._invalid_characters_following_an_operator];
                                    return this.getErrorStrings()[5].toString();
                                }
                            }

                            rightIndex = j + 1;
                        } else {
                            throw this.formulaErrorStrings[this._invalid_characters_following_an_operator];
                        }
                    }
                    var p = op.indexOf(text[i]);
                    var s = this.bMARKER + this._zapBlocks(left) + this._zapBlocks(right) + markers[p] + this.bMARKER;
                    if (leftIndex > 0) {
                        s = text.substring(0, leftIndex) + s;
                    }

                    if (rightIndex < text.length) {
                        s = s + text.substring(rightIndex);
                    }

                    s = s.split(this.bMARKER2).join(this.bMARKER.toString());

                    //// s = s.replace(tic+tic, tic); removed forum post http://64.78.18.34/Support/Forums/message.aspx?MessageID=22483
                    text = s;
                }
            } else {
                ////Process left argument.
                var j = text.length - 1;
                if (text[j] == this.bMARKER) {
                    var k = this._findLastNonQB(text.substring(0, j - 1));
                    if (k < 0) {
                        throw this.formulaErrorStrings[this._cannot_parse];
                    }
                } else if (text[j] == this._rightBracket) {
                    ////library member
                    var bracketCount = 0;
                    var k = j - 1;
                    while (k > 0 && (text[k] != 'q' || bracketCount != 0)) {
                        if (text[k] == 'q') {
                            bracketCount--;
                        } else if (text[k] == this._rightBracket) {
                            bracketCount++;
                        }

                        k--;
                    }

                    if (k < 0) {
                        throw this.formulaErrorStrings[this._bad_library];
                    }
                } else if (!this._isDigit(text[j])) {
                    // return "ERROR";
                } else {
                    var period = false;
                    var percent = false;

                    while (j > -1 && (this._isDigit(text[j]) || (!period && text[j] == this.getParseDecimalSeparator()) || (!percent && text[j] == '%'))) {
                        if (text[j] == this.getParseDecimalSeparator()) {
                            period = true;
                        } else if (text[j] == '%') {
                            percent = true;
                        }

                        j = j - 1;
                    }

                    if (j > -1 && period && text[j] == this.getParseDecimalSeparator()) {
                        throw this.formulaErrorStrings[this._number_contains_2_decimal_points];
                    }
                }
                if (text.length > 0 && (this._isUpperChar(text[0]) || text[0] == this.sheetToken)) {
                    ////Check if cell reference.
                    var ok = true;
                    var checkLetter = true;
                    var oneTokenFound = false;
                    for (var k = 0; k < text.length; ++k) {
                        if (text[k] == this.sheetToken) {
                            if (k > 0 && !oneTokenFound) {
                                throw this.formulaErrorStrings[this._missing_sheet];
                                ////break;
                            }

                            oneTokenFound = true;
                            k++;
                            while (k < text.length && this._isDigit(text[k])) {
                                k++;
                            }

                            if (k == text.length || text[k] != this.sheetToken) {
                                ok = false;
                                break;
                            }
                        } else {
                            ////if(!checkLetter && char.IsUpper(text, k))
                            if (!checkLetter && this._isLetter(text, k)) {
                                ok = false;
                                break;
                            }

                            if (this._isLetterOrDigit(text[k]) || text[k] == this.sheetToken) {
                                checkLetter = this._isUpperChar(text[k]);
                            } else {
                                ok = false;
                                break;
                            }
                        }
                    }

                    if (ok) {
                        this.updateDependencies(text); ////cb
                        //needToContinue = false;
                    }
                }
            }
            return text;
        };
        this._pop = function (_stack) {
            var o = _stack.pop();

            var s;
            if (o != null) {
                if (o.toString() == this.tic + this.tic) {
                    return NaN;
                } else
                    s = o.toString().split(this.tic).join("");
                if (this.getErrorStrings().indexOf(s) > -1) {
                    this._isErrorString = true;
                    return this.getErrorStrings.indexOf(s);
                }

                ////moved from ComputedValue above as result of DT26064
                if (s[0] == "#" || s == "") {
                    return 0;
                }

                if (s == this.trueValueStr) {
                    return 1;
                } else if (s == this.falseValueStr) {
                    return 0;
                }

                var d = this._parseDouble(s);
                if (!isNaN(d)) {
                    return d;
                } else if (this.getUseDatesInCalculations()) {
                    var dt = this._isDate(s);
                    if (!isNaN(dt))
                        return this._getSerialDateTimeFromDate(dt);
                    else
                        return 0;
                }
            }
            if (s == ""  && this.getTreatStringsAsZero()) {
                return 0;
            }
            else if (o != null && o.toString().length > 0) {
                return Number.NaN;
            }
            return 0;
        };
        this._popString = function (_stack) {
            var o = _stack.pop();

            if (o == null) {
                o = "";
            } else if (this.getUseDatesInCalculations()) {
                var dt = this._isDate(o.split(this.tic).join(""));
                if (!isNaN(dt))
                    return this._getSerialDateTimeFromDate(dt);
            }

            return this.removeTics(o.toString());
        };
        this._populateNamedRangesNonScoped = function () {
            if (this._namedRangesNonScoped == null) {
                this._namedRangesNonScoped = new HashTable();
            }
            this._namedRangesNonScoped.length = 0;

            for (var key in this.namedRanges.keys()) {
                this._checkAddNonScopedNamedRange(this.namedRanges.keys()[key]);
            }
        };
        this._processUpperCase = function (formula, i, sheet) {
            var s = "";
            while (i < formula.length && this._isUpperChar(formula[i])) {
                s = s + formula[i];
                i = i + 1;
            }

            while (i < formula.length && this._isDigit(formula[i])) {
                s = s + formula[i];
                i = i + 1;
            }

            if (this._supportRangeOperands && i < formula.length && formula[i] == ':') {
                s = s + formula[i];
                i = i + 1;
                if (i < formula.length && formula[i] == this.sheetToken) {
                    s = s + formula[i];
                    i = i + 1;

                    while (i < formula.length && formula[i] != this.sheetToken) {
                        s = s + formula[i];
                        i = i + 1;
                    }
                }

                while (i < formula.length && this._isUpperChar(formula[i])) {
                    s = s + formula[i];
                    i = i + 1;
                }

                while (i < formula.length && this._isDigit(formula[i])) {
                    s = s + formula[i];
                    i = i + 1;
                }
                s = sheet + this._getCellFrom(s);
                sheet = "";
            } else {
                s = sheet + s;
                sheet = "";
            }
            this._processUpperCaseFormula = formula;
            this._processUpperCaseIvalue = i;
            this._processUpperCaseSheet = sheet;
            return s;
        };
        this._saveStrings = function (text) {
            var strings = null;
            var TICs2 = this.tic + this.tic;
            var id = 0;
            var i = -1;
            if ((i = text.indexOf(this.tic)) > -1) {
                while (i > -1 && i < text.length) {
                    if (strings == null) {
                        strings = new HashTable();
                    }

                    var j = (i + 1) < text.length ? text.indexOf(this.tic, i + 1) : -1;
                    if (j > -1) {
                        var key = this.tic + this._uniqueStringMarker + id.toString() + this.tic;
                        if (j < text.length - 2 && text[j + 1] == this.tic[0]) {
                            j = text.indexOf(this.tic, j + 2);
                            if (j == -1) {
                                throw this.formulaErrorStrings[this._mismatched_tics];
                            }
                        }

                        var s = this._substring(text, i, j - i + 1);
                        strings.add(key, s);
                        s = s.split(TICs2).join(this.tic);
                        id++;
                        text = text.substring(0, i) + key + text.substring(j + 1);
                        i = i + key.length;
                        if (i < text.length) {
                            i = text.indexOf(this.tic, i);
                        }
                    } else if (j == -1 && text.indexOf(this.sheetToken) > -1 && i < text.indexOf(this.sheetToken, i)) {
                        var sheetName = "";
                        for (var k = (text.indexOf(this.sheetToken, i)) - 1; k > -1; k--) {
                            if (this.getValidPrecedingChars().indexOf(text[k].toString()) == -1)
                                sheetName = text[k] + sheetName;
                            else
                                break;
                        }
                        if (!(sheetName[0] == "'" && sheetName[sheetName.length - 1] == "'")) {
                            throw this.formulaErrorStrings[this._mismatched_tics];
                        }
                        if (sheetName[0] == "'")
                            sheetName = sheetName.substring(1);
                        if (sheetName[sheetName.length - 1] == "'")
                            sheetName = sheetName.substring(0, sheetName.length - 1);
                        if (this.getSortedSheetNames().indexOf(sheetName.toUpperCase()) == -1) {
                            throw this.formulaErrorStrings[this._missing_sheet];
                        }
                        if ((i + 1) < text.length) {
                            i = text.indexOf(this.tic, i + 1);
                        }
                    } else {
                        throw this.formulaErrorStrings[this._mismatched_tics];
                    }
                }
            }
            this._saveStringsText = text;
            return strings;
        };
        this._setStrings = function (retValue, strings) {
            var keys = strings.keys();
            for (var s in keys) {
                retValue = retValue.split(keys[s]).join(strings.getItem(keys[s]));
            }
            return retValue;
        };
        this._sheetToken = function (s) {
            var i = 0;
            var s1 = "";
            if (i < s.length && s[i] == this.sheetToken) {
                i++;
                while (i < s.length && s[i] != this.sheetToken) {
                    i++;
                }

                s1 = s.substring(0, i + 1);
            }

            if (i < s.length) {
                return s1;
            }

            throw this.formulaErrorStrings[this._bad_index];
        };
        this._splitArguments = function (args, argSeperator) {
            return args.split(argSeperator);
        };
        this._stripTics0 = function (s) {
            if (s.length > 1 && s[0] == this.tic[0] && s[s.length - 1] == this.tic[0]) {
                s = this._substring(s, 1, s.length - 2);
            }

            return s;
        };
        this._substring = function (text, startIndex, length) {
            return text.substring(startIndex, length + startIndex);
        };
        this._isTextEmpty = function (s) {
            return s == null || s == "";
        };
        this._toOADate = function (dateTime) {
            var result = (dateTime.getTime() - Date.parse(this._oaDate)) / this._millisecondsOfaDay;
            return Math.round(result);
        };
        this._zapBlocks = function (text) {
            if (text.indexOf(this.bMARKER) > -1) {
                var bracketLevel = 0;
                for (var i = text.length - 1; i > 0; --i) {
                    if (text[i] == this._rightBracket) {
                        bracketLevel--;
                    } else if (text[i] == this._leftBracket) {
                        bracketLevel++;
                    } else if (text[i] == this.bMARKER && bracketLevel == 0) {
                        text = text.substring(0, i - 1) + text.substring(i + 1);
                    }
                }
            }

            return text;
        };

        /**		
        * Add the custom formuls with function in CalcEngine library 
        * @return boolean
        * @param {string} FormulaName pass the formula name 
        * @param {string} FunctionName pass the custom function name to call 
        * @example 
        * &lt;div id="Grid"&gt;&lt;/div&gt; 
        * &lt;script&gt;
        * // Create Grid
        * $('#Grid').ejGrid({
        *     dataSource: window.gridData
        * });         
        * var calcObj = new CalcEngine($("#Grid").data("ejGrid"));
        * var sheetID = calcObj.createSheetFamilyID();
        * calcObj.registerGridAsSheet("sheet1", $("#Grid").data("ejGrid"), sheetID);
        * calcObj.addCustomFunction("ADD", "customAdd");
        *    customAdd = function (argsList) {
        *    var splitArgs = argsList.split(calcObj.getParseArgumentSeparator())
        *       var result = 0;
        *       for (var i in splitArgs) {
        *           var s1 = calcObj.getValueFromArg(splitArgs[i]);
        *           result = Number(result) + Number(s1);
        *       }
        *       return result;
        *    }
        * &lt;/script&gt;
        */
        this.addCustomFunction = function (name, func) {
            name = name.toUpperCase();
            this._addFunction(name, func);
            if (this._customlibraryFunctions[name] == undefined) {
                this._customlibraryFunctions[name] = func;
                return true;
            }
            return false;
        };

        /**		
        * Adds a named range to the NamedRanges collection 
        * @return boolean
        * @param {string} Name pass the namedRange's name 
        * @param {string} cellRange pass the cell range of NamedRange
        * @example
        * &lt;div id="Grid"&gt;&lt;/div&gt; 
        * &lt;script&gt;
        * // Create Grid
        * $('#Grid').ejGrid({
        *     dataSource: window.gridData
        * });         
        * var calcObj = new CalcEngine($("#Grid").data("ejGrid"));
        * var sheetID = calcObj.createSheetFamilyID();
        * calcObj.registerGridAsSheet("sheet1", $("#Grid").data("ejGrid"), sheetID);
        * calcObj.addNamedRange("FIRSTCELL","A1");
        * &lt;/script&gt;
        */
        this.addNamedRange = function (name, range) {
            name = name.toUpperCase();
            if (this.getNamedRanges().getItem(name) == null) {
                this.getNamedRanges().add(name, range);
                this._checkAddNonScopedNamedRange(name);
                return true;
            }
            return false;
        };
        /**		
        * Accepts a possible parsed formula and returns the calculated value without quotes.
        * @return adjusted range
        * @param {string} Name pass the cell range to adjust its range
        * @example
        * var calcObj = new CalcEngine($("#Grid").data("ejGrid"));
        * var sheetID = calcObj.createSheetFamilyID();
        * calcObj.registerGridAsSheet("sheet1", $("#Grid").data("ejGrid"), sheetID);
        * calcObj.addNamedRange("FIRSTCELL","A1");
        * &lt;/script&gt;
        */
        this.adjustRangeArg = function (range) {
            if (range.length > 1 && range[0] == this.bMARKER && range[range.length - 1] == this.bMARKER && this._substring(range, 1, range.length - 2).indexOf(this.bMARKER) == -1) {
                range = this.computedValue(range);
            }

            if (range.length > 1 && range[0] == this.tic[0] && range[range.length - 1] == this.tic[0]) {
                range = this._substring(range, 1, range.length - 2);
            }
            return range;
        };
        /**		
        * When a formula cell changes, call this method to clear it from its dependent cells.
        * @param {string} Cell pass the changed cell address
        * @example
        * var calcObj = new CalcEngine($("#Grid").data("ejGrid"));
        * calcObj.clearFormulaDependentCells("A1");
        * &lt;/script&gt;
        */
        this.clearFormulaDependentCells = function (cell) {
            var ht = this.getDependentFormulaCells().getItem(cell);
            if (ht != null) {
                var keys = ht.keys();
                for (var o in keys) {
                    var s = keys[o];
                    var al = this.getDependentCells().getItem(s);
                    this._arrayRemove(al, cell);
                    if (al.length == 0) {
                        this.getDependentCells().remove(s);
                    }
                }

                this.getDependentFormulaCells().remove(cell);
            }
        };
        /**		
        *  Call this method to clear whether an exception was raised during the computation of a library function.
        * @example
        * var calcObj = new CalcEngine($("#Grid").data("ejGrid"));
        * calcObj.clearLibraryComputationException();
        * &lt;/script&gt;
        */
        this.clearLibraryComputationException = function () {
            this._libraryComputationException = null;
        };
        /**		
        * Get the column index from a cell reference passed in.
        * @param {string} Cell pass the cell address
        * @example
        * var calcObj = new CalcEngine($("#Grid").data("ejGrid"));
        * calcObj.colIndex("A1");
        * &lt;/script&gt;
        */
        this.colIndex = function (s) {
            var i = 0;
            var k = 0;
            s = s.toUpperCase();

            if (i < s.length && s[i] == this.sheetToken) {
                i++;
                while (i < s.length && s[i] != this.sheetToken) {
                    i++;
                }

                i++;
            }

            while (i < s.length && this._isLetter(s[i])) {
                var charCode = s[i].charCodeAt(0);
                k = k * 26 + charCode - 64;
                i++;
            }

            if (k == 0) {
                return -1;
            }

            return k;
        };
        /**		
        * Evaluates a parsed formula.
        * @return computed value of formula
        * @param {string} Formula pass the parsed formula
        * @example
        * var calcObj = new CalcEngine($("#Grid").data("ejGrid"));
        * calcObj.computedValue("n10n2a");
        * &lt;/script&gt;
        */
        this.computedValue = function (formula) {
            if (this._isTextEmpty(formula)) {
                return formula;
            }
            try  {
                this.computedValueLevel++;
                if (this.computedValueLevel > this._maximumRecursiveCalls) {
                    this.computedValueLevel = 0;
                    throw this.formulaErrorStrings[this._too_complex];
                }
                var _stack = [];

                var i = 0;
                _stack.length = 0;
                var sheet = "";
                if (this.getAllowShortCircuitIFs() && this.getEnsureIFCallDuringShortCircuit()) {
                    var loc = -1;
                    do {
                        ////code peels all only the outer most IF functions  and relies on a custom IF to handle the proper alternative
                        if (i < formula.length && (i = formula.indexOf(this.iFMarker, i)) > -1) {
                            loc = this._matchingRightBracket(formula.substring(i));
                            if (loc > -1) {
                                var func = this.getLibraryFunctions().getItem("IF");
                                var result = "";
                                try  {
                                    result = this[func](formula.substring(i + this.iFMarker.length, loc - this.iFMarker.length));
                                } catch (ex) {
                                    if (this.getRethrowLibraryComputationExceptions()) {
                                        this._libraryComputationException = ex;
                                        throw ex;
                                    }
                                }

                                //MarkupResultToIncludeInFormula(ref result);
                                var rightPiece = "";
                                if (i + loc + 1 < formula.length)
                                    rightPiece = formula.substring(i + loc + 1);
                                formula = formula.substring(0, i) + result + rightPiece;
                            }
                        }
                    } while(formula.indexOf(this.iFMarker) > -1 && loc > -1);
                }
                i = 0;
                while (i < formula.length) {
                    if (formula[i] == this.bMARKER) {
                        i = i + 1;
                        continue;
                    }

                    var uFound = formula[i] == 'u';
                    if (uFound) {
                        i++;
                        if (i >= formula.length) {
                            continue;
                        }

                        if (formula[i] == this.bMARKER) {
                            i++;
                        }

                        if (i >= formula.length) {
                            continue;
                        }
                    }

                    ////defect 8729
                    ////************
                    //// The % is the last thing in the cell contents here
                    if (formula[i] == '%' && _stack.length > 0) {
                        var o = _stack[0];
                        var d = this._parseDouble(o);
                        if (!isNaN(d)) {
                            _stack.pop(); ////remove it
                            _stack.push(Number(d) / 100); ////push it back
                        }
                        i = i + 1;
                        continue;
                    }
                    if (formula[i] == this.sheetToken) {
                        sheet = formula[i].toString();
                        i++;
                        while (i < formula.length && formula[i] != this.sheetToken) {
                            sheet += formula[i];
                            i++;
                        }

                        if (i < formula.length) {
                            sheet += formula[i];
                            i++;
                        } else {
                            continue;
                        }
                    }
                    if ((formula.substring(i)).indexOf(this.trueValueStr) == 0) {
                        _stack.push(this.trueValueStr);
                        i += this.trueValueStr.length;
                    } else if (formula.substring(i).indexOf(this.falseValueStr) == 0) {
                        _stack.push(this.falseValueStr);
                        i += this.falseValueStr.length;
                    } else if (formula[i] == this.tic[0] || formula[i] == '|') {
                        if (_stack.length == 3)
                            _stack = this._combineStack(formula, i - 2, _stack);

                        var s = formula[i].toString();
                        i++;
                        while (i < formula.length && formula[i] != this.tic[0]) {
                            s = s + formula[i];
                            i = i + 1;
                        }
                        if (this._multiTick) {
                            s = s.split("|").join(this.tic);
                        }
                        _stack.push(s + this.tic);
                        i += 1;
                    } else if (this._isUpperChar(formula[i])) {
                        ////cell loc
                        var s = this._processUpperCase(formula, i, sheet);
                        formula = this._processUpperCaseFormula;
                        i = this._processUpperCaseIvalue;
                        sheet = this._processUpperCaseSheet;
                        if (uFound) {
                            s = this.getValueFromParentObjectCell(s);
                            var d3;
                            if (!isNaN(this._parseDouble(s))) {
                                d3 = -d3;
                                _stack.push(d3.toString());
                            } else {
                                _stack.push(s);
                            }
                        } else {
                            if (this.getUseFormulaValues() && this.computedValueLevel > 50) {
                                if (this._breakedFormulaCells.indexOf(s) == -1 && this.getUseFormulaValues()) {
                                    var family = CalcEngine.getSheetFamilyItem(this.grid);
                                    var token = this.sheetToken + this.getSheetID(this.grid).toString() + this.sheetToken;
                                    s = token + s;
                                    this._breakedFormulaCells.add(s);
                                    this._tempBreakedFormulaCells.add(s);
                                    break;
                                }
                                _stack.push("FALSE");
                            } else
                                _stack.push(this.getValueFromParentObjectCell(s));
                        }
                    } else if (formula[i] == 'q') {
                        ////library
                        formula = this._computeInteriorFunctions(formula);

                        var ii = formula.substring(i + 1).indexOf(this._leftBracket);
                        if (ii > 0) {
                            var bracketCount = 0;
                            var bracketFound = false;
                            var start = ii + i + 2;
                            var k = start;
                            while (k < formula.length && (formula[k] != this._rightBracket || bracketCount > 0)) {
                                if (formula[k] == this._leftBracket) {
                                    bracketCount++;
                                    bracketFound = true;
                                } else if (formula[k] == this._leftBracket) {
                                    bracketCount--;
                                }

                                k++;
                            }

                            if (bracketFound) {
                                var s = this._substring(formula, start, k - start - 2);
                                var s1 = "";
                                var splits = this.splitArgsPreservingQuotedCommas(s);
                                for (var t in splits) {
                                    if (s1.length > 0) {
                                        s1 += ",";
                                    }

                                    var j = this._findLastqNotInBrackets(t);
                                    if (j > 0) {
                                        s1 += splits[t].substring(0, j) + this.computedValue(splits[t].substring(j));
                                    } else {
                                        s1 += this.computedValue(splits[t]);
                                    }
                                }

                                formula = formula.substring(0, start) + s1 + formula.substring(k - 2);
                            }

                            var name = this._substring(formula, i + 1, ii);
                            if (name == "AVG" && this._excelLikeComputations) {
                                throw this.formulaErrorStrings[this._bad_index];
                            }
                            if (this.getLibraryFunctions().getItem(name) != undefined) {
                                var j = formula.substring(i + ii + 1).indexOf(this._rightBracket);
                                var args = this._substring(formula, i + ii + 2, j - 1);
                                try  {
                                    var functionName = this.getLibraryFunctions().getItem(name);
                                    var result;
                                    if (this.getCustomLibraryFunctions().getItem(name) != undefined) {
                                        result = eval(functionName)(args);
                                    } else
                                        result = this[functionName](args);
                                    _stack.push(result);
                                } catch (ex) {
                                    if (this.getRethrowLibraryComputationExceptions()) {
                                        this._libraryComputationException = ex;
                                        throw ex;
                                    }

                                    return ex;
                                }

                                i += j + ii + 2;
                            } else {
                                if (this.getRethrowLibraryComputationExceptions()) {
                                    throw this.formulaErrorStrings[this._missing_formula];
                                    return this.getErrorStrings()[5].toString();
                                }

                            }
                        } else if (formula[0] == this.bMARKER) {
                            ////Restart the processing with the formula without library finctions.
                            i = 0;
                            _stack.length == 0;
                            continue;
                        } else {
                            throw this.formulaErrorStrings[this._improper_formula];
                        }
                    } else if (this._isDigit(formula[i]) || formula[i] == 'u') {
                        var s = "";
                        if (formula[i] == 'u' || uFound) {
                            s = "-";
                            if (!uFound) {
                                i++;
                            } else {
                                uFound = false;
                            }
                        }

                        if (i < formula.length && this._isUpperChar(formula[i])) {
                            s = s + this.getValueFromParentObjectCell(this._processUpperCase(formula, i, sheet));
                            formula = this._processUpperCaseFormula;
                            i = this._processUpperCaseIvalue;
                            sheet = this._processUpperCaseSheet;
                        } else {
                            while (i < formula.length && (this._isDigit(formula[i]) || formula[i] == this.getParseDecimalSeparator())) {
                                s = s + formula[i];
                                i = i + 1;
                            }
                        }

                        _stack.push(s);
                    } else {
                        switch (formula[i]) {
                            case '#': {
                                var errIndex = 0;
                                if (formula.indexOf('!') == -1) {
                                    errIndex = formula.indexOf('?') + 1;
                                } else
                                    errIndex = formula.indexOf('!') + 1;
                                _stack.push(this._substring(formula, i, errIndex - i));
                                i = errIndex;
                                break;
                            }

                            case 'n':
                                {
                                    i = i + 1;

                                    var s = "";
                                    if (formula.substring(i).indexOf("Infinity") == 0) {
                                        s = "Infinity";
                                        i += s.length;
                                    } else if (formula.substring(i).indexOf("uInfinity") == 0) {
                                        s = "Infinity";
                                        i += s.length + 1;
                                    } else if (formula.substring(i).indexOf(this.trueValueStr) == 0) {
                                        s = this.trueValueStr;
                                        i += s.length;
                                    } else if (formula.substring(i).indexOf(this.falseValueStr) == 0) {
                                        s = this.falseValueStr;
                                        i += s.length;
                                    } else if (i <= formula.length - 3 && formula.substring(i, 3) == "NaN") {
                                        i += 3;
                                        s = "0";
                                    } else {
                                        if (formula[i] == 'u' || uFound) {
                                            s = "-";
                                            if (!uFound) {
                                                i = i + 1;
                                            } else {
                                                uFound = false;
                                            }
                                        }

                                        while (i < formula.length && (this._isDigit(formula[i]) || formula[i] == this.getParseDecimalSeparator())) {
                                            s = s + formula[i];
                                            i = i + 1;
                                        }
                                        if (i < formula.length && formula[i] == '%') {
                                            i = i + 1;

                                            ////defect 8729
                                            ////**************
                                            if (s == "") {
                                                if (_stack.length > 0) {
                                                    var o = _stack[0];
                                                    var d = this._parseDouble(o.toString());
                                                    if (!isNaN(d)) {
                                                        _stack.pop(); ////remove it
                                                        s = (d / 100).toString(); ////push it back
                                                    }
                                                }
                                            } else {
                                                ////**************
                                                s = (Number(s) / 100).toString();
                                            }
                                        } else if (i < formula.length - 2 && formula[i] == 'E' && (formula[i + 1] == '+' || formula[i + 1] == '-')) {
                                            s += this._substring(formula, i, 4);
                                            i += 4;
                                        }
                                    }
                                    _stack.push(s);
                                }
                                break;
                            case this.token_add:
                                {
                                    var d = this._pop(_stack);
                                    var x = parseInt(d.toString());
                                    if (!isNaN(x) && this._isErrorString) {
                                        this._isErrorString = false;
                                        return this.getErrorStrings()[x].toString();
                                    }
                                    var d1 = this._pop(_stack);
                                    x = parseInt(d1.toString());
                                    if (!isNaN(x) && this._isErrorString) {
                                        this._isErrorString = false;
                                        return this.getErrorStrings()[x].toString();
                                    }
                                    if (isNaN(d) || isNaN(d1)) {
                                        _stack.push("#VALUE!");
                                    } else
                                        _stack.push((Number(d1) + Number(d)).toString());
                                    i = i + 1;
                                }

                                break;
                            case this.token_subtract:
                                {
                                    var d = this._pop(_stack);
                                    var x = parseInt(d.toString());
                                    if (!isNaN(x) && this._isErrorString) {
                                        this._isErrorString = false;
                                        return this.getErrorStrings()[x].toString();
                                    }
                                    var d1 = this._pop(_stack);
                                    x = parseInt(d1.toString());
                                    if (!isNaN(x) && this._isErrorString) {
                                        this._isErrorString = false;
                                        return this.getErrorStrings()[x].toString();
                                    }
                                    if (isNaN(d) || isNaN(d1)) {
                                        _stack.push("#VALUE!");
                                    } else {
                                        _stack.push((d1 - d).toString());
                                    }
                                    i = i + 1;
                                }

                                break;
                            case this.token_multiply:
                                {
                                    var d = this._pop(_stack);
                                    var x = parseInt(d.toString());
                                    if (!isNaN(x) && this._isErrorString) {
                                        this._isErrorString = false;
                                        return this.getErrorStrings()[x].toString();
                                    }
                                    var d1 = this._pop(_stack);
                                    x = parseInt(d1.toString());
                                    if (!isNaN(x) && this._isErrorString) {
                                        this._isErrorString = false;
                                        return this.getErrorStrings()[x].toString();
                                    }
                                    if (isNaN(d) || isNaN(d1)) {
                                        _stack.push("#VALUE!");
                                    } else {
                                        _stack.push((d1 * d).toString());
                                    }
                                    i = i + 1;
                                }

                                break;
                            case this.token_divide:
                                {
                                    var d = this._pop(_stack);
                                    var x = parseInt(d.toString());
                                    if (!isNaN(x) && this._isErrorString) {
                                        this._isErrorString = false;
                                        return this.getErrorStrings()[x].toString();
                                    }
                                    var d1 = this._pop(_stack);
                                    x = parseInt(d1.toString());
                                    if (!isNaN(x) && this._isErrorString) {
                                        this._isErrorString = false;
                                        return this.getErrorStrings()[x].toString();
                                    }
                                    if (isNaN(d) || isNaN(d1)) {
                                        _stack.push("#VALUE!");
                                    } else if (d == 0) {
                                        _stack.push(this.getErrorStrings()[3]);
                                    } else {
                                        _stack.push((d1 / d).toString());
                                    }
                                    i = i + 1;
                                }

                                break;
                            case this.token_EP:
                                {
                                    var d = this._pop(_stack);
                                    var d1 = this._pop(_stack);
                                    _stack.push((d1 * Math.pow(10, d)).toString());
                                    i = i + 1;
                                }
                                break;
                            case this.token_EM:
                                {
                                    var d = this._pop(_stack);
                                    var d1 = this._pop(_stack);
                                    _stack.push((Number(d1) * Math.pow(10, -Number(d))).toString());
                                    i = i + 1;
                                }
                                break;
                            case this.token_less:
                                {
                                    var s1 = this._popString(_stack);
                                    var s2 = this._popString(_stack);
                                    if (s1 == "" && this.getTreatStringsAsZero()) {
                                        s1 = "0";
                                    }
                                    if (s2 == "" && this.getTreatStringsAsZero()) {
                                        s2 = "0";
                                    }
                                    var d = this._parseDouble(s1);
                                    var d1 = this._parseDouble(s2);
                                    var val;
                                    if (!isNaN(d) && !isNaN(d1)) {
                                        val = (d1 < d) ? this.trueValueStr : this.falseValueStr;
                                    } else {
                                        if (s1 == "#VALUE!" || s2 == "#VALUE!")
                                            val = "#VALUE!";
                                        else
                                            val = (s2.toUpperCase().split(this.tic).join("").localeCompare(s1.toUpperCase().split(this.tic).join("")) < 0) ? this.trueValueStr : this.falseValueStr;
                                    }

                                    _stack.push(val);
                                    i = i + 1;
                                }
                                break;
                            case this.token_greater:
                                {
                                    var s1 = this._popString(_stack);
                                    var s2 = this._popString(_stack);
                                    if (s1 == "") {
                                        if (this.this.getTreatStringsAsZero())
                                            s1 = "0";
                                        else
                                            s1 = s2 + 1;
                                    }

                                    if (s2 == "") {
                                        if (this.this.getTreatStringsAsZero())
                                            s2 = "0";
                                        else
                                            s2 = s1 + 1;
                                    }
                                    var d = this._parseDouble(s1);
                                    var d1 = this._parseDouble(s2);
                                    var val;
                                    if (!isNaN(d) && !isNaN(d1)) {
                                        val = (Number(d1) > Number(d)) ? this.trueValueStr : this.falseValueStr;
                                    } else {
                                        if (s1 == "#VALUE!" || s2 == "#VALUE!")
                                            val = "#VALUE!";
                                        else
                                            val = (s2.toUpperCase().split(this.tic).join("").localeCompare(s1.toUpperCase().split(this.tic).join("")) > 0) ? this.trueValueStr : this.falseValueStr;
                                    }

                                    _stack.push(val);
                                    i = i + 1;
                                }
                                break;
                            case this.token_equal:
                                {
                                    var s1 = this._popString(_stack);
                                    var s2 = this._popString(_stack);
                                    if (s1 == "" && this.getTreatStringsAsZero()) {
                                        s1 = "0";
                                    }
                                    if (s2 == "" && this.getTreatStringsAsZero()) {
                                        s2 = "0";
                                    }
                                    var d = this._parseDouble(s1);
                                    var d1 = this._parseDouble(s2);
                                    var val;
                                    if (!isNaN(d) && !isNaN(d1)) {
                                        val = (Number(d1) == Number(d)) ? this.trueValueStr : this.falseValueStr;
                                    } else {
                                        if (s1 == "#VALUE!" || s2 == "#VALUE!")
                                            val = "#VALUE!";
                                        else
                                            val = (s1.toUpperCase().split(this.tic).join("") == s2.toUpperCase().split(this.tic).join("")) ? this.trueValueStr : this.falseValueStr;
                                    }
                                    _stack.push(val);
                                    i = i + 1;
                                }
                                break;
                            case this.token_lesseq:
                                {
                                    var s1 = this._popString(_stack);
                                    var s2 = this._popString(_stack);
                                    if (s1 == "" && this.getTreatStringsAsZero()) {
                                        s1 = "0";
                                    }
                                    if (s2 == "" && this.getTreatStringsAsZero()) {
                                        s2 = "0";
                                    }
                                    var d = this._parseDouble(s1);
                                    var d1 = this._parseDouble(s2);
                                    var val;
                                    if (!isNaN(d) && !isNaN(d1)) {
                                        val = (Number(d1) <= Number(d)) ? this.trueValueStr : this.falseValueStr;
                                    } else {
                                        if (s1 == "#VALUE!" || s2 == "#VALUE!")
                                            val = "#VALUE!";
                                        else
                                            val = (s1.toUpperCase().split(this.tic).join("").localeCompare(s2.toUpperCase().split(this.tic).join("")) <= 0) ? this.trueValueStr : this.falseValueStr;
                                    }

                                    _stack.push(val);
                                    i = i + 1;
                                }
                                break;
                            case this.token_greatereq:
                                {
                                    var s1 = this._popString(_stack);
                                    var s2 = this._popString(_stack);
                                    if (s1 == "" && this.getTreatStringsAsZero()) {
                                        s1 = "0";
                                    }
                                    if (s2 == "" && this.getTreatStringsAsZero()) {
                                        s2 = "0";
                                    }
                                    var d = this._parseDouble(s1);
                                    var d1 = this._parseDouble(s2);
                                    var val;
                                    if (!isNaN(d) && !isNaN(d1)) {
                                        val = (Number(d1) >= Number(d)) ? this.trueValueStr : this.falseValueStr;
                                    } else {
                                        if (s1 == "#VALUE!" || s2 == "#VALUE!")
                                            val = "#VALUE!";
                                        else
                                            val = (s1.toUpperCase().split(this.tic).join("").localeCompare(s2.toUpperCase().split(this.tic).join("")) >= 0) ? this.trueValueStr : this.falseValueStr;
                                    }
                                    _stack.push(val);
                                    i = i + 1;
                                }
                                break;
                            case this.token_noequal:
                                {
                                    var s1 = this._popString(_stack);
                                    var s2 = this._popString(_stack);
                                    if (s1 == "" && this.getTreatStringsAsZero()) {
                                        s1 = "0";
                                    }
                                    if (s2 == "" && this.getTreatStringsAsZero()) {
                                        s2 = "0";
                                    }
                                    var d = this._parseDouble(s1);
                                    var d1 = this._parseDouble(s2);
                                    var val;
                                    if (!isNaN(d) && !isNaN(d1)) {
                                        val = (Number(d1) != Number(d)) ? this.trueValueStr : this.falseValueStr;
                                    } else {
                                        if (s1 == "#VALUE!" || s2 == "#VALUE!")
                                            val = "#VALUE!";
                                        else
                                            val = (s1.toUpperCase().split(this.tic).join("") != s2.toUpperCase().split(this.tic).join("")) ? this.trueValueStr : this.falseValueStr;
                                    }
                                    _stack.push(val);
                                    i = i + 1;
                                }
                                break;
                            case this.token_and:
                                {
                                    var s1 = this._popString(_stack);
                                    var s2 = "";
                                    if (_stack.length > 0)
                                        s2 = this._popString(_stack);
                                    if (this.getUseNoAmpersandQuotes()) {
                                        _stack.push(s2 + s1);
                                    } else {
                                        _stack.push(this.tic + s2 + s1 + this.tic);
                                    }
                                    i = i + 1;
                                }
                                break;
                            case this.token_or:
                                {
                                    var d = this._pop(_stack);
                                    var d1 = this._pop(_stack);
                                    _stack.push(Math.pow(d1, d).toString());
                                    i = i + 1;
                                }
                                break;
                            case this.token_ORop:
                                {
                                    var s1 = this._popString(_stack);
                                    var s2 = this._popString(_stack);
                                    var d = this._parseDouble(s1);
                                    var d1 = this._parseDouble(s2);
                                    var val;
                                    if (s1 == "") {
                                        s1 = this.falseValueStr;
                                    } else if (!isNaN(d)) {
                                        s1 = (Number(s1) != 0) ? this.trueValueStr : this.falseValueStr;
                                    }

                                    if (s2 == "") {
                                        s2 = this.falseValueStr;
                                    } else if (!isNaN(d1)) {
                                        s2 = (Number(s2) != 0) ? this.trueValueStr : this.falseValueStr;
                                    }

                                    val = (s1.toUpperCase().split(this.tic).join("") == this.trueValueStr || s2.toUpperCase().split(this.tic).join("") == this.trueValueStr) ? this.trueValueStr : this.falseValueStr;

                                    _stack.push(val);
                                    i = i + 1;
                                }
                                break;
                            case this.token_ANDop:
                                {
                                    var s1 = this._popString(_stack);
                                    var s2 = this._popString(_stack);
                                    var d = this._parseDouble(s1);
                                    var d1 = this._parseDouble(s2);
                                    var val;
                                    if (s1 == "") {
                                        s1 = this.falseValueStr;
                                    } else if (!isNaN(d)) {
                                        s1 = (Number(s1) != 0) ? this.trueValueStr : this.falseValueStr;
                                    }

                                    if (s2 == "") {
                                        s2 = this.falseValueStr;
                                    } else if (!isNaN(d1)) {
                                        s2 = (Number(s2) != 0) ? this.trueValueStr : this.falseValueStr;
                                    }

                                    val = (s1.toUpperCase().split(this.tic).join("") == this.trueValueStr && s2.toUpperCase().split(this.tic).join("") == this.trueValueStr) ? this.trueValueStr : this.falseValueStr;

                                    _stack.push(val);
                                    i = i + 1;
                                }
                                break;
                            case this.token_XORop:
                                {
                                    var s1 = this._popString(_stack);
                                    var s2 = this._popString(_stack);
                                    var d = this._parseDouble(s1);
                                    var d1 = this._parseDouble(s2);
                                    var val;
                                    if (s1 == "") {
                                        s1 = this.falseValueStr;
                                    } else if (!isNaN(d)) {
                                        s1 = (Number(s1) != 0) ? this.trueValueStr : this.falseValueStr;
                                    }

                                    if (s2 == "") {
                                        s2 = this.falseValueStr;
                                    } else if (!isNaN(d1)) {
                                        s2 = (Number(s2) != 0) ? this.trueValueStr : this.falseValueStr;
                                    }

                                    val = ((s1.toUpperCase().split(this.tic).join("") == this.trueValueStr && s2.toUpperCase().split(this.tic).join("") != this.trueValueStr) || (s2.toUpperCase().split(this.tic).join("") == this.trueValueStr && s1.toUpperCase().split(this.tic).join("") != this.trueValueStr)) ? this.trueValueStr : this.falseValueStr;

                                    _stack.push(val);
                                    i = i + 1;
                                }
                                break;
                            case this.token_NOTop:
                                {
                                    var s1 = this._popString(_stack);
                                    var d = this._parseDouble(s1);
                                    var val;
                                    if (s1 == "") {
                                        s1 = this.falseValueStr;
                                    } else if (!isNaN(d)) {
                                        s1 = (Number(s1) != 0) ? this.trueValueStr : this.falseValueStr;
                                    }

                                    val = (s1.toUpperCase().split(this.tic).join("") == this.falseValueStr) ? this.trueValueStr : this.falseValueStr;

                                    _stack.push(val);
                                    i = i + 1;
                                }
                                break;
                            default: {
                                this.computedValueLevel = 0;
                                throw this.formulaErrorStrings[this._invalid_expression];
                            }
                        }
                    }
                }
                if (this._checkDanglingStack && _stack.length > 1 && formula.length > 1 && (formula.indexOf(this.bmarker.toString()) != 0 || formula.indexOf(this.bmarker) != (formula.length - 1) || formula.indexOf(this.bmarker2) > -1)) {
                    this.computedValueLevel = 0;
                    return this.formulaErrorStrings[this._improper_formula];
                }

                if (_stack.length == 0) {
                    return "";
                } else {
                    var s = "";
                    var d;
                    var cc = _stack.length;
                    do {
                        s = _stack.pop().toString() + s;
                        return s;
                        cc--;
                    } while(cc > 0 && !(s.indexOf(this.FALSEVALUESTR) > -1 || s.indexOf(this.tRUEVALUESTR) > 1));

                    ////This check moved below in Pop as result of DT26064
                    ////if(_textIsEmpty(s))
                    ////    s = "0"; //Empty is zero in calculations.
                    return s;
                }
            } catch (ex) {
                this.computedValueLevel = 0;
                if (ex.indexOf(this.formulaErrorStrings[this._circular_reference_]) > -1 || (this._rethrowLibraryComputationExceptions && this.getLibraryComputationException() != null)) {
                    if (this._rethrowLibraryComputationExceptions && this.getLibraryComputationException() != null) {
                        throw this.getLibraryComputationException();
                    }

                    throw ex;
                }

                if (ex.indexOf(this.formulaErrorStrings[this._cell_empty]) > -1) {
                    return "";
                } else {
                    if (this.getRethrowLibraryComputationExceptions()) {
                        this._libraryComputationException = ex;
                        throw ex;
                    }
                    return ex;
                }
            } finally {
                this.computedValueLevel--;
                if (this.computedValueLevel < 0) {
                    this.computedValueLevel = 0;
                }
            }
        };
        this.computeFormula = function (parsedFormula) {
            if (this.getThrowCircularException()) {
                if (this._iterationMaxCount > 0) {
                    if (this.computedValueLevel == 0) {
                        this._circCheckList.length = 0;
                        this._circCheckList.push(this.cell);
                    } else if (this._circCheckList.indexOf(this.cell) > -1) {
                        if (!this.getIterationValues().containsKey(this.cell)) {
                            this.getIterationValues().add(this.cell, "0");
                        }

                        return this.getIterationValues().getItem(this.cell).toString();
                    }
                } else {
                    if (this.computedValueLevel == 0) {
                        this._circCheckList.length = 0;
                    } else if (this._circCheckList.indexOf(this.cell) > -1) {
                        this.computedValueLevel = 0;
                        this._circCheckList.length = 0;
                        throw this.formulaErrorStrings[this._circular_reference_];
                    }

                    this._circCheckList.push(this.cell);
                }
            }

            var s = this.computedValue(parsedFormula);

            if (this.getUseNoAmpersandQuotes() && s.length > 1 && s[0] == this.tic[0] && s[s.length - 1] == this.tic[0]) {
                s = this._substring(s, 1, s.length - 2);
            }
            return s;
        };
        
        this.dispose = function () {
            if (!this._isDisposed) {
                var i = 0;
                if (this.grid != null) {
                    i = this.getSheetID(this.grid);
                }
                CalcEngine._tokenCount--;
                if (CalcEngine._sheetFamiliesList != null && CalcEngine._sheetFamiliesList.length > 0) {
                    CalcEngine._sheetFamiliesList[i] = null;
                    CalcEngine._sheetFamiliesList.remove(i);
                }
                if (CalcEngine.modelToSheetID != null && CalcEngine.modelToSheetID.length > 0) {
                    CalcEngine.modelToSheetID[i] = null;
                    CalcEngine._sheetFamiliesList.remove(i);
                }
                if (this._formulaInfoTable != null) {
                    this._formulaInfoTable.clear();
                    this._formulaInfoTable = null;
                }
                if (this._circCheckList != null) {
                    this._circCheckList.length = 0;
                    this._circCheckList = null;
                }
                if (this._dependentCells != null) {
                    this._dependentCells.clear();
                    this._dependentCells = null;
                }
                if (this._dependentFormulaCells != null) {
                    this._dependentFormulaCells.clear();
                    this._dependentFormulaCells = null;
                }

                if (this.getiterationValues() != null) {
                    this.getiterationValues().clear();
                    this.setiterationValues(null);
                }
                if (this._libraryFunctions != null) {
                    this._libraryFunctions.length = 0;
                    this._libraryFunctions = null;
                }
                if (this._lookupTables != null) {
                    this._lookupTables.clear();
                    this._lookupTables = null;
                }
                this.grid = null;
                if (this.getLibraryFunctions() != null) {
                    this.getLibraryFunctions().length = 0;
                    this.setLibraryFunctions(null);
                }
                this._isDisposed = true;
            }
        };
        this.getCalcID = function () {
            return this.calcID;
        };
        this.getCellsFromArgs = function (args) {
            args = this._markColonsInQuotes(args);

            var row1;
            var col1;

            var i = args.indexOf(':');
            if (i == -1) {
                args = args.split(this._markerChar).join(':');
                i = args.indexOf(this.getParseArgumentSeparator());
                if (i == -1) {
                    row1 = this.rowIndex(args); ////maybe throw exception
                    col1 = this.colIndex(args); ////maybe throw exception
                    var argsArray = [];
                    argsArray.push(args);
                    return argsArray;
                } else {
                    return this.splitArgsPreservingQuotedCommas(args);
                }
            }

            var sheet = "";
            var argsRet = args;
            var j = args.indexOf(this.sheetToken);
            if (j > -1) {
                var j1 = args.indexOf(this.sheetToken, j + 1);
                if (j1 > -1) {
                    sheet = this._substring(args, j, j1 - j + 1);
                    args = args.split(sheet).join("");
                    i = args.indexOf(':');
                }
            }

            ////check if column only range
            if (i > 0 && this._isLetter(args[i - 1])) {
                var count = (this._rowMaxCount > 0) ? this._rowMaxCount : 50;
                args = args.substring(0, i) + "1:" + args.substring(i + 1) + count.toString();
                i = args.indexOf(':');
            } else if (i > 0 && this._isDigit(args[i - 1])) {
                var k1 = i - 2;
                while (k1 >= 0 && this._isDigit(args[k1])) {
                    k1--;
                }

                if (k1 == -1 || !this._isLetter(args[k1])) {
                    var count = (this._columnMaxCount > 0) ? this._columnMaxCount : 50;
                    args = "A" + args.substring(0, i) + ":" + RangeInfo.getAlphaLabel(count) + args.substring(i + 1);
                    i = args.indexOf(':');
                }
            }

            // Check Whether rowindex can get from the arguments
            var isDigit = this._canGetRowIndex(args.substring(0, i));
            if (!isDigit) {
                // if unable to get index return the arguments
                this._ignoreCellValue = true;
                args = argsRet;
                return this.splitArgsPreservingQuotedCommas(args);
            }
            row1 = this.rowIndex(args.substring(0, i));
            var row2 = this.rowIndex(this._substring(args, i + 1, args.length - i - 1));
            var errorStr;
            if ((!(!(row1 == -1) || (row2 == -1)) == (((row1 == -1) || !(row2 == -1))))) {
                throw this.getErrorStrings()[5].toString();
                // errorStr[0] = this.getErrorStrings()[5].toString();
                // return errorStr;
            }

            col1 = this.colIndex(args.substring(0, i));

            var col2 = this.colIndex(this._substring(args, i + 1, args.length - i - 1));

            if (row1 > row2) {
                i = row2;
                row2 = row1;
                row1 = i;
            }

            if (col1 > col2) {
                i = col2;
                col2 = col1;
                col1 = i;
            }

            var numCells = (row2 - row1 + 1) * (col2 - col1 + 1);
            var cells = [];
            var k = 0;
            for (i = row1; i <= row2; ++i) {
                for (j = col1; j <= col2; ++j) {
                    try  {
                        cells[k++] = sheet + RangeInfo.getAlphaLabel(j) + i.toString();
                    } catch (ex) {
                        continue;
                    }
                }
            }
            return cells;
        };
        this.getFormulaRowCol = function (grd, row, col) {
            var family = CalcEngine.getSheetFamilyItem(grd);
            var s = RangeInfo.getAlphaLabel(col) + row.toString();
            if (family.sheetNameToParentObject != null) {
                var token = family.parentObjectToToken.getItem(grd);
                s = token + s;
            }

            if (this.getFormulaInfoTable().containsKey(s)) {
                var formula = this.getFormulaInfoTable.getItem(s);
                var value = formula.getFormulaText();
                this.getFormulaText(value);
                return value;
            }

            return "";
        };
        this.getFormulaText = function (s) {
            var e = new FormulaParsing(s);
            s = e.getText();
            return s;
        };
        this.getSheetID = function (grd) {
            var family = CalcEngine.getSheetFamilyItem(grd);
            if (family.sheetNameToParentObject != null && family.sheetNameToParentObject.length > 0) {
                var token = family.parentObjectToToken.getItem(grd);
                token = token.split("!").join("");
                var d = this._parseDouble(token);
                if (!isNaN(d)) {
                    return d;
                }
            }

            return -1;
        };
        this.getStringArray = function (s) {
            //ArrayList argList = new ArrayList();
            var argList = [];
            var argStart = 0;
            var inQuote = false;
            for (var argEnd = 0; argEnd < s.length; argEnd++) {
                var ch = s[argEnd];
                if (ch == this.tic[0]) {
                    inQuote = !inQuote;
                } else if (!inQuote && ch == this.getParseArgumentSeparator()) {
                    argList.push(this._substring(s, argStart, argEnd - argStart));
                    argStart = argEnd + 1;
                }
            }

            argList.push(s.substring(argStart));
            return argList;
        };
        this.getValueFromArg = function (arg) {
            var d;
            var tempDate = this._dateTime1900;

            ////handle special characters - probably need to revise...
            ////if(arg.IndexOf("%") > -1)
            ////	Console.WriteLine("special");
            ////arg = arg//.replace(TRUEVALUESTR, TRUEVALUE.toString())
            ////		//.replace(FALSEVALUESTR, FALSEVALUE.toString())
            ////		.replace("%", "");
            if (this._isTextEmpty(arg)) {
                return arg;
            } else if (arg[0] == this.tic[0]) {
                ////string
                var doubleValue;
                tempDate = this._isDate(arg.split(this.tic).join(""));
                if (this.getExcelLikeComputations() && this.getUseDatesInCalculations() && isNaN(this._parseDouble(arg.split(this.tic).join(""))) && !isNaN(tempDate.getDate()) && this._dateTime1900 <= tempDate) {
                    return this._toOADate(tempDate).toString();
                }
                return arg;
            } else if (arg[0] == this.bMARKER || arg[0] == 'q') {
                ////parsed formula
                arg = arg.split('{').join('(');
                arg = arg.split('}').join(')');
                return this.computedValue(arg);
            } else if (arg.length > 1 && arg.substring(0, 2) == "ub") {
                ////parsed formula
                arg = arg.split('{').join('(');
                arg = arg.split('}').join(')');
                var ggg = arg.substring(1);
                ggg = this.computedValue(ggg);
                var d2 = this._parseDouble(ggg);
                if (!isNaN(d2)) {
                    d2 = -d2;
                    return d2.toString();
                }
                return this.computedValue(arg);
            } else {
                if (arg.indexOf("unu") == 0) {
                    arg = "n" + arg.substring(3);
                } else if (arg.indexOf("un") == 0) {
                    arg = "-" + arg.substring(2);
                }

                arg = arg.split('u').join('-');
                if (!this._isUpperChar(arg[0]) && (this._isDigit(arg[0]) || arg[0] == this.getParseDecimalSeparator() || arg[0] == '-' || arg[0] == 'n')) {
                    if (arg[0] == 'n') {
                        arg = arg.substring(1);
                    }

                    d = this._parseDouble(arg);
                    if (!isNaN(d)) {
                        return this._preserveLeadingZeros ? arg : d.toString();
                    } else if (arg.indexOf(this.trueValueStr) == 0 || arg.indexOf(this.falseValueStr) == 0) {
                        return arg;
                    }
                }
            }
            if (this._ignoreCellValue && !(arg.indexOf(this.trueValueStr) == 0 || arg.indexOf(this.falseValueStr) == 0)) {
                this._ignoreCellValue = false;
                return arg;
            }

            var symbolArray = ['+', '-', '/', '*', ')', ')', '{'];

            ////Not a number.
            if ((this._indexOfAny(arg, symbolArray) == -1 && this._isUpperChar(arg[0])) || arg[0] == this.sheetToken) {
                ////Check for circular reference.
                if (arg == this.cell) {
                    var ht = this.getDependentCells().getItem(arg);
                    if (ht != null && ht.indexOf(arg) > -1) {
                        this._arrayRemove(ht, arg);
                    }
                    if (!this.getDependentFormulaCells().containsKey(this.cell))
                        this.clearFormulaDependentCells(this.cell);
                    throw this.formulaErrorStrings[this._circular_reference_] + arg;
                }

                var s1 = this.getValueFromParentObjectCell(arg);
                if (arg != this.trueValueStr && arg != this.falseValueStr) {
                    d = this._parseDouble(s1.split(this.tic).join(""));
                    if (!this._preserveLeadingZeros && s1.length > 0 && !isNaN(d)) {
                        s1 = d.toString();
                    }

                    this.updateDependencies(arg);
                }
                if (s1 == this.tic + this.tic) {
                    return "NaN";
                }
                var doubleValue;
                tempDate = this._isDate(s1);
                if (this.getExcelLikeComputations() && this.getUseDatesInCalculations() && isNaN(this._parseDouble(s1)) && !isNaN(tempDate.getDate()) && this._dateTime1900 <= tempDate) {
                    s1 = this._toOADate(tempDate);
                }
                return s1;
            }

            ////Must be a formula, so try to parse it and compute.
            arg = arg.split('{').join('(');
            arg = arg.split('}').join(')');
            arg = this._parse(arg);

            d = this._parseDouble(arg.substring(0, arg.length - 1));
            if (arg.indexOf("%") == (arg.length - 1) && !isNaN(d)) {
                arg = (Number(d) / 100).toString();
            }
            if (this.getErrorStrings().indexOf(arg) > -1) {
                return arg;
            }

            return this.computedValue(arg);
        };
        this.getValueFromParentObjectCell = function (cell1) {
            if (cell1 == this.trueValueStr || cell1 == this.falseValueStr) {
                return cell1;
            }

            var i = cell1.lastIndexOf(this.sheetToken);
            var row = 0, col = 0;
            var grd = this.grid;
            var family = CalcEngine.getSheetFamilyItem(this.grid);
            if (i > -1 && family.tokenToParentObject != null) {
                this.grid = family.tokenToParentObject.getItem(cell1.substring(0, i + 1));
                row = this.rowIndex(cell1);
                col = this.colIndex(cell1);
            } else if (i == -1) {
                var j = 0;
                while (j < cell1.length && this._isLetter(cell1[j])) {
                    j++;
                }
                if (j == cell1.length) {
                    cell1 = cell1.toLowerCase();
                    return cell1;
                } else {
                    row = this.rowIndex(cell1);
                    col = this.colIndex(cell1);
                    if (this.isSheetMember() && family.parentObjectToToken != null) {
                        cell1 = family.parentObjectToToken.getItem(this.grid) + cell1;
                    }
                }
            }

            var saveCell = (this.cell == "" || this.cell == null) ? "" : this.cell;
            this.cell = cell1;

            if (this._iterationMaxCount > 0) {
                if (this._circCheckList.indexOf(this.cell) > -1 && this.computedValueLevel > 0) {
                    this.grid = grd;
                    this.cell = saveCell;

                    if (!this.getIterationValues().containsKey(this.cell)) {
                        this.getIterationValues().add(this.cell, "0");
                    }

                    return this.getIterationValues().getItem(this.cell).toString();
                } else if (this.getIterationValues().containsKey(this.cell) && this.computedValueLevel > 0) {
                    return this.getIterationValues.getItem(this.cell).toString();
                }
            }

            var val = this._getValueComputeFormulaIfNecessary(row, col, this.grid);

            this.grid = grd;
            this.cell = saveCell;
            return val;
        };
        this.getValueFromParentObject = function (grd, row, col) {
            var family = CalcEngine.getSheetFamilyItem(grd);
            var cell1 = (family.parentObjectToToken == null || family.parentObjectToToken.length == 0) ? "" : family.parentObjectToToken.getItem(grd).toString();
            cell1 = cell1 + RangeInfo.getAlphaLabel(col) + row.toString();

            var saveGrid = this.grid;
            var saveCell = this.cell;
            this.cell = cell1;
            this.grid = grd;

            var val = this._getValueComputeFormulaIfNecessary(row, col, grd);
            var tempDate = new Date(Date.parse(val));
            var doubleValue = this._parseDouble(val);
            if (this.getExcelLikeComputations() && this.getUseDatesInCalculations() && isNaN(doubleValue) && !isNaN(tempDate.getDate()) && this._dateTime1900 <= tempDate) {
                val = this._toOADate(tempDate).toString();
            }
            this.grid = saveGrid;
            this.cell = saveCell;
            return val;
        };
        this.getValueFromParentObjectRowCol = function (row, col) {
            return this._getValueComputeFormulaIfNecessary(row, col, this.grid);
        }

        this.handleIteration = function (s, formula) {
            if (this.getFormulaInfoTable().containsKey(s)) {
                this.getFormulaInfoTable.add(s, formula);
            } else {
                this.getFormulaInfoTable.add(s, formula);
            }
            if (this._iterationMaxCount > 0) {
                if (s == this.cell) {
                    this._handleIterations(formula);
                }
            }
            return formula.getFormulaValue();
        };
        this.handleSheetRanges = function (text, family) {
            var sheetLoc = text.indexOf(this.sheetToken);
            var start = 0;
            while (sheetLoc > 0) {
                var colonLoc = this._substring(text, start, sheetLoc).lastIndexOf(':');
                if (colonLoc > -1) {
                    var rightSide = this._substring(text, start + colonLoc + 1, sheetLoc - colonLoc - 1).toUpperCase().split("'").join("");
                    if (family.sheetNameToToken.containsKey(rightSide.trim())) {
                        var leftStart = start + colonLoc - 1;
                        if (this._sortedSheetNames.contains(rightSide)) {
                            leftStart = leftStart - rightSide.length - 1;
                        } else {
                            while (leftStart > 0 && this._markers.indexOf(text[leftStart]) == -1) {
                                leftStart--;
                            }
                        }

                        var leftSide = this._substring(text, leftStart + 1, colonLoc - leftStart + start - 1).toUpperCase().split("'").join("");

                        if (family.sheetNameToToken.containsKey(leftSide.trim())) {
                            var rightEnd = sheetLoc + start + 1;
                            if (this._sortedSheetNames.contains(leftSide.trim())) {
                                rightEnd = text.length - 1;
                            } else {
                                while (rightEnd < text.length && this._markers.indexOf(text[rightEnd]) == -1) {
                                    rightEnd++;
                                }
                            }

                            var otherSide = this._substring(text, start + sheetLoc + 1, rightEnd - start - sheetLoc - 1);
                            var left = family.sheetNameToToken.getItem(leftSide.trim()).toString();
                            var right = family.sheetNameToToken.getItem(rightSide.trim()).toString();

                            ////make sure they are ordered properly
                            if (right < left) {
                                var s = left;
                                left = right;
                                right = s;
                            }

                            var replacement = "";

                            for (var name in this._sortedSheetNames) {
                                var s = family.sheetNameToToken.getItem(this._sortedSheetNames[name]).toString();
                                var sint = parseInt(s.split("!").join(""));
                                var leftint = parseInt(left.split("!").join(""));
                                var rightint = parseInt(right.split("!").join(""));
                                if (sint > leftint && sint < rightint) {
                                    if (replacement.length > 0) {
                                        replacement += this.getParseArgumentSeparator();
                                    }

                                    replacement += name + String.fromCharCode(131) + otherSide;
                                }
                            }

                            text = text.substring(0, leftStart + 1) + replacement + text.substring(rightEnd);
                            start = text.length - rightEnd;
                        } else {
                            start = sheetLoc + start;
                        }
                    } else {
                        start = sheetLoc + start;
                    }
                } else {
                    start = sheetLoc + start;
                }

                sheetLoc = text.substring(start + 1).indexOf(this.sheetToken) + 1;
            }

            text = text.split(String.fromCharCode(131)).join(this.sheetToken);
            return text;
        };
        this.isSheetMember = function () {
            var family = CalcEngine.getSheetFamilyItem(this.grid);
            return (family == null || family == undefined) ? false : family.isSheetMember;
        };
        this.parseAndComputeFormula = function (formulaString) {
            var formula = formulaString;
            if (formula.length > 0 && formula[0] == this.getFormulaCharacter()) {
                formula = formula.substring(1);
            }

            if (formula.length > 0 && formula[0] == '+') {
                formula = formula.substring(1);
            }
            if (formula.length > 1 && formula[0] == ',') {
                if (formula[1] >= '0' && formula[1] <= '9')
                    formula = "0" + ',' + formula.substring(1);
            }
            var tempFormula = formula.split(" ").join("");

            for (var i = tempFormula.indexOf("("); i != -1 && i < tempFormula.length; i++) {
                if (i > 0) {
                    if (this._isDigit(tempFormula[i - 1])) {
                        if (!this._checkHasCharBeforeNumber(tempFormula.substring(0, i)))
                            throw (this.formulaErrorStrings[this._bad_formula] + " " + tempFormula.substring(0, i + 1));
                    }
                    var next = tempFormula.substring(i + 1).indexOf('(');
                    if (next == -1) {
                        break;
                    } else {
                        i += next;
                    }
                }
            }
            for (var i = tempFormula.indexOf(")"); i != -1 && i < tempFormula.length - 1; i++) {
                if (this._isDigit(tempFormula[i + 1])) {
                    throw (this.formulaErrorStrings[this._bad_formula] + " " + tempFormula.substring(0, i + 2));
                }
                var next = tempFormula.substring(i + 1).indexOf(')');
                if (next == -1) {
                    break;
                } else {
                    i += next;
                }
            }
            this._multiTick = false;

            var s = this._parse(formula);
            return this.computedValue(s);
        };
        this.parseFormula = function (formula) {
            if (formula.length > 0 && formula[0] == this.getFormulaCharacter()) {
                formula = formula.substring(1);
            }

            if (formula.length > 0 && formula[0] == '+') {
                formula = formula.substring(1);
            }

            this._isRangeOperand = this._supportRangeOperands && this._isRange(formula);
            if (this.getCheckDanglingStack() && (formula.split(" ").join("")).indexOf(this._braceRightNLeft) > -1) {
                this.computedValueLevel = 0;
                return this.formulaErrorStrings[this._improper_formula];
            }
            return this._parse(formula.trim());
        };
        this.getUpdatedValueCell = function (cellRef) {
            var isUseFormulaValueChanged = false;
            this._inAPull = true;
            var grd = this.grid;
            var saveCell = this.cell;
            var family = CalcEngine.getSheetFamilyItem(this.grid);
            var s = cellRef.toUpperCase();
            var i;
            if ((i = s.indexOf(this.sheetToken)) == -1 && this.cell != "" && this.cell != undefined) {
                i = this.cell.indexOf(this.sheetToken, 1);
                if (i > -1 && family.tokenToParentObject != null) {
                    s = this.cell.substring(0, i + 1) + s;
                    this.grid = family.tokenToParentObject.getItem(this.cell.substring(0, i + 1));
                }
            } else {
                if (i > 0 && family.sheetNameToToken != null && family.tokenToParentObject != null) {
                    ////need sheet id
                    var token = family.sheetNameToToken.getItem(s.substring(0, i));
                    s = token + s.substring(i + 1);
                    this.grid = family.tokenToParentObject.getItem(token);
                    this.cell = s;
                }
            }

            this.updateCalcID();
            var txt;
            if (!this.getDependentFormulaCells().containsKey(s) && !this.getFormulaInfoTable().containsKey(s)) {
                txt = this.getValueFromParentObjectCell(s);
                while (this._breakedFormulaCells.length == 1) {
                    this.updateCalcID();
                    txt = this.getValueFromParentObjectCell(this._breakedFormulaCells[0].toString());
                    this._breakedFormulaCells.removeAt(0);
                    this.setUseFormulaValues(true);
                }
                if (this.getUseFormulaValues()) {
                    this.isUseFormulaValueChanged = true;
                    this.setUseFormulaValue(false);
                }
                if (this._tempBreakedFormulaCells.length > 0) {
                    for (var p = 1; p <= this._tempBreakedFormulaCells.length; p++) {
                        txt = this.getValueFromParentObjectCell(this._tempBreakedFormulaCells[this._tempBreakedFormulaCells.length - p].toString());
                    }
                    this._tempBreakedFormulaCells.length = 0;
                    txt = this.getValueFromParentObjectCell(s);
                }
                this.setUseFormulaValues(this.isUseFormulaValueChanged);
                var saveIVC = this.ignoreValueChanged;
                this.ignoreValueChanged = true;
                var row = this.rowIndex(s);

                var col = this.colIndex(s);
                if (this.getPreserveFormula()) {
                    var token = this._sheetToken(cellRef);
                    if (token == "") {
                        token = "!" + this.getSheetID(this.grid) + "!";
                        var formula = this.getFormulaInfoTable().getItem(token + s);
                        if (formula != null && formula != undefined) {
                            if (this.parentObject.setValueRowCol == undefined)
                                this.setValueRowCol(this.getSheetID(this.grid) + 1, formula.getFormulaText(), row, col);
                            else
                                this.parentObject.setValueRowCol(this.getSheetID(this.grid) + 1, formula.getFormulaText(), row, col);
                        }
                    } else if (this.parentObject.setValueRowCol == undefined)
                        this.setValueRowCol(this.getSheetID(this.grid) + 1, txt, row, col);
                    else
                        this.parentObject.setValueRowCol(this.getSheetID(this.grid) + 1, txt, row, col);
                } else if (this.parentObject.setValueRowCol == undefined)
                    this.setValueRowCol(this.getSheetID(this.grid) + 1, txt, row, col);
                else
                    this.parentObject.setValueRowCol(this.getSheetID(this.grid) + 1, txt, row, col);

                this.ignoreValueChanged = saveIVC;
            } else {
                this._processedCells.clear();
                this.updateDependenciesAndCell(s);
                this._processedCells.clear();
                txt = this.getValueFromParentObject(s);
            }

            this.grid = grd;
            this.cell = saveCell;
            this._inAPull = false;

            return txt;
        };
        this.getUpdatedValueRowCol = function (targetSheetID, row, col) {
            var saveGrid = this.grid;
            var family = CalcEngine.getSheetFamilyItem(this.grid);
            var token = this.sheetToken + targetSheetID.toString() + this.sheetToken;
            this.grid = family.tokenToParentObject.getItem(token);
            this._inAPull = true;
            var s = token + RangeInfo.getAlphaLabel(col) + row.toString();

            this.updateCalcID();

            if (!this.getDependentFormulaCells().containsKey(s) && !this.formulaInfoTable.containsKey(s)) {
                var txt = this.getValueFromParentObjectCell(s);
                var saveIVC = this.ignoreValueChanged;
                this.ignoreValueChanged = true;
                if (this.parentObject.setValueRowCol == undefined)
                    this.setValueRowCol(this.getSheetID(this.grid) + 1, txt, row, col);
                else
                    this.parentObject.setValueRowCol(this.getSheetID(this.grid) + 1, txt, row, col);
                this.ignoreValueChanged = saveIVC;
            } else {
                this._processedCells.clear();
                this.updateDependenciesAndCell(s);
                this._processedCells.clear();
            }

            this.grid = saveGrid;
            this._inAPull = false;
        };
        this.setTokensForSheets = function (text) {
            var family = CalcEngine.getSheetFamilyItem(this.grid);

            if (this.getSupportsSheetRanges()) {
                text = this.handleSheetRanges(text, family);
            }

            var sortedSheetNamesCollection = this.getSortedSheetNames();

            ////family.SheetNameToToken.Keys)
            if (sortedSheetNamesCollection != null) {
                for (var name in sortedSheetNamesCollection) {
                    var token = family.sheetNameToToken.getItem(sortedSheetNamesCollection[name]);
                    token = token.split(this.sheetToken).join(this._tempSheetPlaceHolder);

                    ////keep the tic defect#541 - TODO: Optimize this process - cache tic names for instance
                    var s = "'" + sortedSheetNamesCollection[name].toUpperCase() + "'" + this.sheetToken;
                    if (text.indexOf(s) == -1) {
                        s = sortedSheetNamesCollection[name].toUpperCase() + this.sheetToken;
                    }

                    text = text.split(s).join(token);

                    //do it again without the tics
                    s = sortedSheetNamesCollection[name].toUpperCase() + this.sheetToken;

                    text = text.split(s).join(token);
                }
            }

            text = text.split(this._tempSheetPlaceHolder).join(this.sheetToken);
            return text;
        };
        this.recalculateRange = function (range, data) {
            this._inRecalculateRange = true;
            for (var row = range.getTop(); row <= range.getBottom(); ++row) {
                for (var col = range.getLeft() ; col <= range.getRight() ; ++col) {
                    if (data.getValueRowCol == undefined)
                        data.setValueRowCol(this.getSheetID(data) + 1, data.getValueRowCol(this.getSheetID(data) + 1, row, col), row, col);
                    else
                        data.setValueRowCol(this.getSheetID(data) + 1, data.getValueRowCol(this.getSheetID(data) + 1, row, col), row, col);
                }
            }

            this._inRecalculateRange = false;
        };
        this.refresh = function (s) {
            ////Don't refresh any cells.
            if (this.getCalculatingSuspended()) {
                return;
            }

            if (this._dependencyLevel == 0) {
                this._refreshedCells.length = 0;
            }

            if (this.getDependentCells().containsKey(s) && this.getDependentCells().getItem(s) != null) {
                this._dependencyLevel++;
                try  {
                    var family = CalcEngine.getSheetFamilyItem(this.grid);

                    var ht = this.getDependentCells().getItem(s);
                    var save1 = this._lockDependencies;
                    this._lockDependencies = true;
                    for (var o in ht) {
                        var s1 = ht[o];
                        if (s1 != null) {
                            var grd = this.grid;
                            var sheet = this._sheetToken(s1);
                            if (sheet.length > 0) {
                                this.grid = family.tokenToParentObject.getItem(sheet);
                            }
                            try  {
                                var row = this.rowIndex(s1);
                                var col = this.colIndex(s1);

                                var info = this.getFormulaInfoTable().getItem(s1);
                                if (info != null) {
                                    var save = this.cell;
                                    this.cell = s1;
                                    if (this.getAlwaysComputeDuringRefresh() || info.calcID != this._calcID || info.getFormulaValue() == "") {
                                        info.setFormulaValue(this.computeFormula(info.getParsedFormula()));
                                    }

                                    info.calcID = this._calcID;
                                    this.cell = save;
                                    var saveIVC = this.ignoreValueChanged;
                                    this.ignoreValueChanged = true;
                                    if (this.parentObject.setValueRowCol == undefined)
                                        this.setValueRowCol(this.getSheetID(this.grid) + 1, info.getFormulaValue(), row, col);
                                    else
                                        this.parentObject.setValueRowCol(this.getSheetID(this.grid) + 1, info.getFormulaValue(), row, col);
                                    this.ignoreValueChanged = saveIVC;
                                    if (!this._refreshedCells.containsKey(s1)) {
                                        this._refreshedCells.add(s1, 0); ////from below ...a
                                        this.refresh(s1); ////recursive call
                                    }
                                }
                            } catch (ex) {
                                continue;
                            }
                            this.grid = grd;
                        }
                    }

                    this._lockDependencies = save1;
                } finally {
                    if (!this._refreshedCells.containsKey(s)) {
                        this._refreshedCells.add(s, 0);
                    }

                    this._dependencyLevel--;
                    if (this._dependencyLevel == 0) {
                        this._refreshedCells.length = 0;
                    }
                }
            }
        };
        this.refreshRange = function (range) {
            for (var r = range.getTop(); r <= range.getBottom(); r++) {
                for (var c =range.getLeft(); c <= range.getRight(); c++) {
                    var s = RangeInfo.getAlphaLabel(c) + r.toString();
                    this._dependencyLevel = 0;
                    var family = CalcEngine.getSheetFamilyItem(this.grid);
                    var token = this.sheetToken + this.getSheetID(this.grid).toString() + this.sheetToken;
                    if (family.tokenToParentObject != null && family.tokenToParentObject.contains(token))
                        this.grid = family.tokenToParentObject.getItem(token);

                    s = token + s;
                    this.refresh(s);
                }
            }
        };
        this.registerGridAsSheet = function (refName, model, sheetFamilyID) {
            ////refName = refName.replace(" ", ""); //keep the spaces defect#541
            if (CalcEngine.modelToSheetID != null) {
                // this.wireParentObject();
            }


            if (CalcEngine.modelToSheetID == null) {
                CalcEngine.modelToSheetID = new HashTable();
            }

            if (CalcEngine.modelToSheetID.getItem(model) == null || CalcEngine.modelToSheetID.getItem(model) == undefined) {
                CalcEngine.modelToSheetID.add(model, sheetFamilyID);
            }

            var family = CalcEngine.getSheetFamilyItem(model);
            family.isSheetMember = true;

            var refName1 = refName.toUpperCase();
            if (family.sheetNameToParentObject == null) {
                family.sheetNameToParentObject = new HashTable();
            }

            if (family.tokenToParentObject == null) {
                family.tokenToParentObject = new HashTable();
            }

            if (family.sheetNameToToken == null) {
                family.sheetNameToToken = new HashTable();
            }

            if (family.parentObjectToToken == null) {
                family.parentObjectToToken = new HashTable();
            }

            if (family.sheetNameToParentObject.getItem(refName1) != undefined) {
                var token = family.sheetNameToToken.getItem(refName1);
                family.tokenToParentObject.add(token, model);
                family.parentObjectToToken.add(model, token);
            } else {
                var token = this.sheetToken + CalcEngine._tokenCount.toString() + this.sheetToken;
                CalcEngine._tokenCount++;
                family.tokenToParentObject.add(token, model);
                family.sheetNameToToken.add(refName1, token);
                family.sheetNameToParentObject.add(refName1, model);
                family.parentObjectToToken.add(model, token);
                this.sortedSheetNames = null;
            }
            return refName;
        };

        this.removeFunction = function (name) {
            if (this._customlibraryFunctions.getItem(name) != undefined) {
                this._customlibraryFunctions.remove(name);
            }
            if (this._libraryFunctions.getItem(name) != null) {
                this._libraryFunctions.remove(name);
                return true;
            }
            return false;
        };
        this.removeNamedRange = function (name) {
            name = name.toUpperCase();
            if (this.getNamedRanges().getItem(name) != null) {
                this.getNamedRanges().remove(name);
                this._populateNamedRangesNonScoped();
                return true;
            }
            return false;
        };
        this.rowIndex = function (s) {
            var i = 0;
            var result;

            var isLetter = false;
            if (i < s.length && s[i] == this.sheetToken) {
                i++;
                while (i < s.length && s[i] != this.sheetToken) {
                    i++;
                }

                i++;
            }

            while (i < s.length && this._isLetter(s[i])) {
                isLetter = true;
                i++;
            }
            result = parseInt(s.substring(i));
            if (i < s.length && !isNaN(result)) {
                return result;
            }

            if (isLetter) {
                return -1;
            }
            throw this.formulaErrorStrings[this._bad_index];
        };
        this.splitArgsPreservingQuotedCommas = function (args) {
            if (args.indexOf(this.tic) == -1) {
                return args.split(this._parseArgumentSeparator);
            }

            var formulaStrings = this._saveStrings(args);
            args = this._saveStringsText;
            var results = args.split(this._parseArgumentSeparator);
            var pieces = [];
            for (var s in results) {
                var s1 = results[s];
                s1 = this._setStrings(s1, formulaStrings); ////replace tokens with original strings
                pieces.push(s1);
            }
            return pieces;
        };
        this.removeTics = function (s) {
            if (s.length > 1 && s[0] == this.tic[0] && s[s.length - 1] == this.tic[0]) {
                if (this._substring(s, 1, s.length - 2).indexOf(this.tic) == -1) {
                    s = this._substring(s, 1, s.length - 2);
                } else if (this._multiTick) {
                    s = this._substring(s, 1, s.length - 2);
                }
            }
            return s;
        };
        this.updateCalcID = function () {
            this._calcID++;
            if (this._calcID == Number.MAX_VALUE) {
                this._calcID = Number.MIN_VALUE + 1;
            }
        };
        this.updateDependencies = function (s) {
            ////TraceUtil.TraceCurrentMethodInfoIf(Switches.FormulaCell.TraceVerbose, this);
            if (this._lockDependencies || !this.getUseDependencies()) {
                return;
            }
            var family = CalcEngine.getSheetFamilyItem(this.grid);
            var cell1 = this.cell;
            if (family.sheetNameToParentObject != null && cell1.indexOf(this.sheetToken) == -1) {
                var token = family.parentObjectToToken.getItem(this.grid);
                cell1 = token + cell1;
            }
            if (family.sheetNameToParentObject != null && s.indexOf(this.sheetToken) == -1) {
                var token = family.parentObjectToToken.getItem(this.grid);
                s = token + s;
            }

            if (!this.getDependentCells().containsKey(s)) {
                var ht1 = [];
                this.getDependentCells().add(s, ht1);

                ht1.push(cell1);

                if (this.getDependentFormulaCells().containsKey(s)) {
                    var ht = this.getDependentFormulaCells()[s];
                    if (ht != null) {
                        var keys = ht.keys();
                        for (var o in keys) {
                            var s1 = keys[o];
                            var ht2 = this.getDependentCells().getItem(s1);
                            if (ht2 == null) {
                                ht2 = [];
                                this.getDependentCells().add(s1, ht2);
                            }

                            if (ht2.indexOf(cell1) == -1) {
                                ht2.push(cell1);
                            }
                        }
                    }
                }

                ////	else
                this._addToFormulaDependentCells(s);

                if (this.getDependentCells().containsKey(cell1)) {
                    var htCells = this.getDependentCells().getItem(cell1);
                    if (htCells != null && htCells != ht1) {
                        for (var o in htCells) {
                            var s1 = htCells[o];
                            ht1.push(s1);
                        }
                    }
                }
            } else {
                var ht1 = this.getDependentCells().getItem(s);

                if (ht1.indexOf(cell1) == -1) {
                    ht1.push(cell1);
                }

                this._addToFormulaDependentCells(s);

                ////maybe remove this block ..... not needed if other dependencies working OK
                if (this.getDependentFormulaCells().containsKey(s)) {
                    var ht = this.getDependentFormulaCells().getItem(s);
                    if (ht != null) {
                        var keys = ht.keys();
                        for (var o in keys) {
                            var s1 = keys[o];
                            var ht2 = this.getDependentCells().getItem(s1);
                            if (ht2 == null) {
                                ht2 = [];
                                this.getDependentCells().add(s1, ht2);
                            }
                            if (ht2.indexOf(cell1) == -1) {
                                ht2.push(cell1);
                            }
                        }
                    }
                }
            }
        };
        this.updateDependenciesAndCell = function (cell1) {
            var grd = this.grid;
            var family = CalcEngine.getSheetFamilyItem(this.grid);

            var sheet = this._sheetToken(cell1);
            if (sheet.length > 0) {
                this.grid = family.tokenToParentObject.getItem(sheet);
            }

            if (this.formulaInfoTable.containsKey(cell1)) {
                var formula = this.formulaInfoTable[cell1];
                if (formula.calcID != this._calcID) {
                    var saveCell = this.cell;
                    this.cell = cell1;

                    if (this._iterationMaxCount > 0 && this._circCheckList.indexOf(this.cell) > -1) {
                        this._handleIterations(formula);
                    } else {
                        formula.setFormulaValue(this.computeFormula(formula.getParsedFormula()));
                    }

                    if (this.getDependentCells().containsKey(cell1)) {
                        var ht = this.getDependentCells().getItem(cell1);
                        for (var s in ht) {
                            var f = this.FormulaInfoTable.getItem(ht[s]);
                            if (f != null && f != undefined) {
                                f.calcID = Number.MIN_VALUE; ////mark as dirty
                            }
                        }
                    }

                    formula.calcID = this._calcID;
                    this.cell = saveCell;
                }

                var saveIVC = this.ignoreValueChanged;
                this.ignoreValueChanged = true;

                var row = this.rowIndex(cell1);
                var col = this.colIndex(cell1);
                if (this.getPreserveFormula()) {
                    if (this.parentObject.setValueRowCol == undefined)
                        this.setValueRowCol(this.getSheetID(this.grid) + 1, formula.getFormulaText(), row, col);
                    else
                        this.parentObject.setValueRowCol(this.getSheetID(this.grid) + 1, formula.getFormulaText(), row, col);
                }
                this.ignoreValueChanged = saveIVC;

                this.grid = grd;

                if (this._processedCells.contains(cell1)) {
                    return;
                } else {
                    this._processedCells.add(cell1);
                }
                if (this.parentObject.setValueRowCol == undefined)
                    this.setValueRowCol(this.getSheetID(this.grid) + 1, formula.getFormulaValue(), row, col);
                else
                    this.parentObject.setValueRowCol(this.getSheetID(this.grid) + 1, formula.getFormulaValue(), row, col);
                if (this.getDependentFormulaCells().containsKey(cell1)) {
                    var hthash = this.getDependentFormulaCells().getItem(cell1);
                    var keys = hthash.keys();
                    for (var c in keys) {
                        this.updateDependenciesAndCell(keys[c]); ////recursive call
                    }
                }
            }
        };
        this.valueChanged = function (grid, e) {
            if (this.ignoreValueChanged) {
                return;
            }

            var grd = grid;
            this.grid = grid;

            var computedValueChanged = true;
            var family = CalcEngine.getSheetFamilyItem(grd);
            var s = RangeInfo.getAlphaLabel(e.getColIndex()) + e.getRowIndex().toString();
            if (family.sheetNameToParentObject != null && family.sheetNameToParentObject.length > 0) {
                var token = family.parentObjectToToken.getItem(grd);
                s = token + s;
            }

            if (e.getValue().length > 0 && e.getValue()[0] == this.getFormulaCharacter()) {
                this.cell = s;
                var formula;
                var compute = true;
                if (this.getFormulaInfoTable().containsKey(s)) {
                    formula = this.getFormulaInfoTable().getItem(s);
                    if (e.getValue() != formula.getFormulaText() || formula.getParsedFormula() == null) {
                        formula.setFormulaText(e.getValue());
                        if (this.getDependentFormulaCells().containsKey(this.cell))
                            this.clearFormulaDependentCells(this.cell);
                        try  {
                            formula.setParsedFormula(this.parseFormula(e.getValue()));
                        } catch (ex) {
                            formula.setFormulaValue(ex);
                            compute = false;
                        }
                    }

                    if (compute) {
                        var s1 = this.computeFormula(formula.getParsedFormula());

                        computedValueChanged = (s1 != formula.getFormulaValue()) || this.getForceRefreshCall();

                        formula.setFormulaValue(s1);
                    }

                    formula.calcID = this._calcID;
                } else {
                    formula = new FormulaInfo();
                    formula.setFormulaText(e.getValue());
                    if (!this.getDependentFormulaCells().containsKey(s)) {
                        this.getDependentFormulaCells().add(s, new HashTable());
                    }

                    try  {
                        formula.setParsedFormula(this.parseFormula(e.getValue()));
                    } catch (ex) {
                        formula.setFormulaValue(ex);
                        compute = false;
                    }

                    if (compute) {
                        formula.setFormulaValue(this.computeFormula(formula.getParsedFormula()));
                    }

                    formula.calcID = this._calcID;

                    if (this.getFormulaInfoTable().containsKey(s)) {
                        this.getFormulaInfoTable().add(s, formula);
                    } else {
                        this.getFormulaInfoTable().add(s, formula);
                    }
                }

                if (this._iterationMaxCount > 0 && compute && !this._inHandleIterations) {
                    if (s == this.cell) {
                        this._handleIterations(formula);
                    }
                }

                var saveIVC = this.ignoreValueChanged;
                this.ignoreValueChanged = true;
                if (this.parentObject.setValueRowCol == undefined)
                    this.setValueRowCol(this.getSheetID(grd) + 1, formula.getFormulaValue(), e.getRowIndex(), e.getColIndex());
                else
                    this.parentObject.setValueRowCol(this.getSheetID(grd) + 1, formula.getFormulaValue(), e.getRowIndex(), e.getColIndex());
                if (formula != null && formula.getFormulaValue() == "" && e.getValue().startsWith(this.getFormulaCharacter())) {
                    if (this.parentObject.setValueRowCol == undefined)
                        this.setValueRowCol(this.getSheetID(this.grid) + 1, "0", e.getRowIndex(), e.getColIndex());
                    else
                        this.parentObject.setValueRowCol(this.getSheetID(this.grid) + 1, "0", e.getRowIndex(), e.getColIndex());
                }
                this.ignoreValueChanged = saveIVC;
            } else if (!this._inRecalculateRange && this.getFormulaInfoTable().containsKey(s)) {
                this.getFormulaInfoTable().remove(s);
                if (this.getDependentFormulaCells().containsKey(s))
                    this.clearFormulaDependentCells(s);
            }

            if (computedValueChanged && this.getDependentCells().containsKey(s)) {
                this._dependencyLevel = 0;
                this.refresh(s);
            }
        };
        this.getActiveCell = function () {
            return this.cell;
        };
        this.getAllowShortCircuitIFs = function () {
            return this._allowShortCircuitIFs;
        };
        this.setAllowShortCircuitIFs = function (value) {
            this._allowShortCircuitIFs = value;
        };
        this.getAlwaysComputeDuringRefresh = function () {
            return this._alwaysComputeDuringRefresh;
        };
        this.setAlwaysComputeDuringRefresh = function (value) {
            this._alwaysComputeDuringRefresh = value;
        };
        this.getCalculatingSuspended = function () {
            return this._calculatingSuspended;
        };
        this.setCalculatingSuspended = function (value) {
            this._calculatingSuspended = value;
        };
        this.getCheckDanglingStack = function () {
            return this._checkDanglingStack;
        };
        this.setCheckDanglingStack = function (value) {
            this._checkDanglingStack = value;
        };
        this.getCustomLibraryFunctions = function () {
            return this._customlibraryFunctions;
        };
        this.getDependentCells = function () {
            if (this.isSheetMember()) {
                var family = CalcEngine.getSheetFamilyItem(this.grid);
                if (family.sheetDependentCells == null) {
                    family.sheetDependentCells = new HashTable();
                }

                return family.sheetDependentCells;
            } else {
                if (this._dependentCells == null) {
                    this._dependentCells = new HashTable();
                }

                return this._dependentCells;
            }
        };
        this.getDependentFormulaCells = function () {
            if (this.isSheetMember()) {
                var family = CalcEngine.getSheetFamilyItem(this.grid);
                if (family.sheetDependentFormulaCells == null) {
                    family.sheetDependentFormulaCells = new HashTable();
                }

                return family.sheetDependentFormulaCells;
            } else {
                if (this._dependentFormulaCells == null) {
                    this._dependentFormulaCells = new HashTable();
                }

                return this._dependentFormulaCells;
            }
        };
        this.getEnableLookupTableCaching = function () {
            return this._enableLookupTableCaching;
        };
        this.setEnableLookupTableCaching = function (value) {
            this._enableLookupTableCaching = value;
        };
        this.getEnsureIFCallDuringShortCircuit = function () {
            return this._ensureIFCallDuringShortCircuit;
        };
        this.setEnsureIFCallDuringShortCircuit = function (value) {
            this._ensureIFCallDuringShortCircuit = value;
        };
        this.getErrorStrings = function () {
            if (this._errorStrings == null) {
                this._errorStrings = ["#N/A", "#VALUE!", "#REF!", "#DIV/0!", "#NUM!", "#NAME?", "#NULL!"];
            }
            return this._errorStrings;
        };
        this.setErrorStrings = function (value) {
            this._errorStrings = value;
        };
        this.getExcelLikeComputations = function () {
            return this._excelLikeComputations;
        };
        this.setExcelLikeComputations = function (value) {
            this._excelLikeComputations = value;
        };
        this.getForceRefreshCall = function () {
            return this._forceRefreshCall;
        };
        this.setForceRefreshCall = function (value) {
            this._forceRefreshCall = value;
        };
        this.getFormulaCharacter = function () {
            if (CalcEngine._formulaChar == '\0') {
                CalcEngine._formulaChar = '=';
            }
            return CalcEngine._formulaChar;
        };
        this.setFormulaCharacter = function (value) {
            CalcEngine._formulaChar = value;
        };
        this.getFormulaInfoTable = function () {
            if (this.isSheetMember()) {
                var family = CalcEngine.getSheetFamilyItem(this.grid);
                if (family.sheetFormulaInfoTable == null) {
                    family.sheetFormulaInfoTable = new HashTable();
                }

                return family.sheetFormulaInfoTable;
            } else {
                if (this._formulaInfoTable == null) {
                    this._formulaInfoTable = new HashTable();
                }

                return this._formulaInfoTable;
            }
        };
        this.getPreserveLeadingZeros = function () {
            return this._preserveLeadingZeros;
        };
        this.setPreserveLeadingZeros = function (value) {
            this._preserveLeadingZeros = value;
        };
        this.getIterationMaxCount = function () {
            return this._iterationMaxCount;
        };
        this.setIterationMaxCount = function (value) {
            this._iterationMaxCount = value;
            if (this._iterationMaxCount > 0) {
                this.setThrowCircularException(true);
            }
        };
        this.getIterationMaxTolerance = function () {
            return this._iterationMaxTolerance;
        };
        this.setIterationMaxTolerance = function (value) {
            this._iterationMaxTolerance = value;
        };
        this.getIterationValues = function () {
            if (this._iterationValues == null) {
                this._iterationValues = new HashTable();
            }
            return this._iterationValues;
        };
        this.getLibraryComputationException = function () {
            return this._libraryComputationException;
        };
        this.getLibraryFunctions = function () {
            return this._libraryFunctions;
        };
        this.getLockDependencies = function () {
            return this._lockDependencies;
        };
        this.setLockDependencies = function (value) {
            this._lockDependencies = value;
        };
        this.getMaximumRecursiveCalls = function () {
            return this._maximumRecursiveCalls;
        };
        this.setMaximumRecursiveCalls = function (value) {
            this._maximumRecursiveCalls = value;
        };
        this.getNamedRanges = function () {
            if (this.namedRanges == null) {
                this.namedRanges = new HashTable();
                this._namedRangesNonScoped = new HashTable();
            }
            return this.namedRanges;
        };
        this.setNamedRanges = function (value) {
            this.namedRanges = value;
            this._populateNamedRangesNonScoped();
        };
        this.getParseArgumentSeparator = function () {
            if (this._parseArgumentSeparator == '\0') {
                this._parseArgumentSeparator = ',';
            }
            var seperator = ',';
            if (!this._isParseArgumentSeparator && seperator != this._parseArgumentSeparator) {
                this._parseArgumentSeparator = seperator;
            }
            return this._parseArgumentSeparator;
        };
        this.setParseArgumentSeparator = function (value) {
            this._parseArgumentSeparator = value;
            this._isParseArgumentSeparator = true;
        };
        this.getParseDateTimeSeparator = function () {
            return this._parseDateTimeSeparator;
        };
        this.setParseDateTimeSeparator = function (value) {
            this._parseDateTimeSeparator = value;
        };
        this.getParseDecimalSeparator = function () {
            ////DT 26058 incident
            ////handle problems with threading by dynamically assigning the static values
            if (this._parseDecimalSeparator == '\0') {
                this._parseDecimalSeparator = '.';
            }

            var seperator = '.';
            if (!this._isParseDecimalSeparatorChanged && seperator != this._parseDecimalSeparator) {
                this._parseDecimalSeparator = seperator;
            }

            return this._parseDecimalSeparator;
        };
        this.setParseDecimalSeparator = function (value) {
            this._parseDecimalSeparator = value;
            this._parseDecimalSeparator = true;
        };
        this.getPreserveFormula = function () {
            return this._preserveFormula;
        };
        this.setPreserveFormula = function (value) {
            this._preserveFormula = value;
        };
        this.getReservedWordOperators = function () {
            if (this._reservedWordOperators == null) {
                this._reservedWordOperators = [" or ", " and ", " xor ", "if ", " then ", " else ", "not "];
            }

            return this._reservedWordOperators;
        };
        this.setReservedWordOperators = function (value) {
            this._reservedWordOperators = value;
        };
        this.getRethrowLibraryComputationExceptions = function () {
            return this._rethrowLibraryComputationExceptions;
        };
        this.setRethrowLibraryComputationExceptions = function (value) {
            this._rethrowLibraryComputationExceptions = value;
        };
        this.getRethrowParseExceptions = function () {
            return this._rethrowExceptions;
        };
        this.setRethrowParseExceptions = function (value) {
            this._rethrowExceptions = value;
        };
        this.getRowMaxCount = function () {
            return this._rowMaxCount;
        };
        this.setRowMaxCount = function (value) {
            this._rowMaxCount = value;
        };
        this.getSortedSheetNames = function () {
            if (this._sortedSheetNames == null) {
                var family = CalcEngine.getSheetFamilyItem(this.grid);
                if (family != null && family.sheetNameToToken != null) {
                    this._sortedSheetNames = family.sheetNameToToken.keys();
                    this._sortedSheetNames.sort(); //(new LenComparer());
                }
            }

            return this._sortedSheetNames;
        };
        this.getSupportLogicalOperators = function () {
            return this._supportLogicalOperators;
        };
        this.setSupportLogicalOperators = function (value) {
            this._supportLogicalOperators = value;
        };
        this.getSupportRangeOperands = function () {
            return this._supportRangeOperands;
        };
        this.setSupportRangeOperands = function (value) {
            this._supportRangeOperands = value;
        };
        this.getSupportsSheetRanges = function () {
            return this._supportsSheetRanges;
        };
        this.setSupportsSheetRanges = function (value) {
            this._supportsSheetRanges = value;
        };
        this.getThrowCircularException = function () {
            return this._throwCircularException;
        };
        this.setThrowCircularException = function (value) {
            this._throwCircularException = value;
        };
        this.getTreatStringsAsZero = function () {
            return this._treatStringsAsZero;
        };
        this.setTreatStringsAsZero = function (value) {
            this._treatStringsAsZero = value;
        };
        this.getUseDatesInCalculations = function () {
            return this._useDatesInCalcs;
        };
        this.setUseDatesInCalculations = function (value) {
            this._useDatesInCalcs = value;
        };
        this.getUseDependencies = function () {
            return this._useDependencies;
        };
        this.setUseDependencies = function (value) {
            this._useDependencies = value;
        };
        this.getUseFormulaValues = function () {
            return this._useFormulaValues;
        };
        this.setUseFormulaValues = function (value) {
            this._useFormulaValues = value;
        };
        this.getUseNoAmpersandQuotes = function () {
            return this._useNoAmpersandQuotes;
        };
        this.setUseNoAmpersandQuotes = function (value) {
            this._useNoAmpersandQuotes = value;
        };
        this.getValidPrecedingChars = function () {
            return this._validPrecedingChar + this.getParseArgumentSeparator();
        };
        this.setValidPrecedingChars = function (value) {
            this._validPrecedingChars = value;
        };
        this.getWeekEndType = function () {
            return ["", "6,0", "0,1", "1,2", "2,3", "3,4", "4,5", "5,6", "", "", "", "0", "1", "2", "3", "4", "5", "6"];
        };
        this.computeSum = function (range) {
            var sum = 0, s1, d, adjustRange;
            var ranges = this.splitArgsPreservingQuotedCommas(range);
            for (var key in ranges) {
                adjustRange = ranges[key];
                if (adjustRange.indexOf(':') > -1) {
                    var cellCollection = this.getCellsFromArgs(adjustRange);
                    for (var s in cellCollection) {
                        try  {
                            s1 = this.getValueFromArg(cellCollection[s]).split(this.tic).join("");
                        } catch (ex) {
                            if (this.getRethrowLibraryComputationExceptions() && this.getLibraryComputationException() != null) {
                                throw this.getLibraryComputationException();
                            }

                            return ex;
                        }

                        if (s1 != "") {
                            d = (s1 == this.trueValueStr) ? true : (s1 == this.falseValueStr) ? false : this._parseDouble(s1);
                            if (this.getErrorStrings().indexOf(s1) > -1) {
                                return s1;
                            }
                            if (isNaN(d)) {
                                s1 = "";
                            } else {
                                sum = sum + d;
                            }
                        }
                    }
                } else {
                    try  {
                        s1 = this.getValueFromArg(adjustRange).split(this.tic).join("");
                    } catch (ex) {
                        if (this.getRethrowLibraryComputationExceptions() && this.getLibraryComputationException() != null) {
                            throw this.getLibraryComputationException();
                        }

                        return ex;
                    }

                    if (s1.length > 0) {
                        d = (s1 == this.trueValueStr) ? true : (s1 == this.falseValueStr) ? false : this._parseDouble(s1);
                        if (this.getErrorStrings().indexOf(s1) > -1) {
                            return s1;
                        }
                        if (!isNaN(d)) {
                            sum = sum + d;
                        }
                    }
                }
            }
            return sum.toString();
        };

        this.computeDate = function (argList) {
            var args = this.splitArgsPreservingQuotedCommas(argList);
            var argCount = args.length;
            if (argCount != 3) {
                return this.formulaErrorStrings[this._wrong_number_arguments];
            }

            for (var i = 0; i < argCount; ++i) {
                args[i] = this.getValueFromArg(args[i]);
            }

            var year = this._parseDouble(args[0]);
            var month = this._parseDouble(args[1]);
            var day = this._parseDouble(args[2]);
            var days = 0;

            if (!isNaN(year) && !isNaN(month) && !isNaN(day)) {
                if (year < 0) {
                    if (this.getRethrowLibraryComputationExceptions())
                        throw this.getErrorStrings()[4].toString();
                    return this.getErrorStrings()[4].toString();
                }
                while (month > 12) {
                    month -= 12;
                    year++;
                }

                days = this._getSerialDateFromDate(year, month, day);
            }
            if (this._excelLikeComputations) {
                var date = this._fromOADate(days);
                if (date.toString() != "Invalid Date") {
                    return date.toLocaleDateString();
                }
            }
            return days.toString();
        };
        this.computeDatevalue = function (argList) {
            var args = this.splitArgsPreservingQuotedCommas(argList);
            var argCount = args.length;
            if (argCount != 1) {
                return this.formulaErrorStrings[this._wrong_number_arguments];
            }
            var dt = new Date(Date.parse(args[0]));
            var dtNum = this._parseDouble(args[0]);
            try  {
                if (!isNaN(dtNum) || dt.toString() == "Invalid Date") {
                    if (dtNum > 0) {
                        dt = this._fromOADate(dtNum);
                    } else {
                        var ranges = argList.split(this.getParseDateTimeSeparator());
                        var argCount = ranges.length;
                        var arg = "";
                        for (var i = 0; i < argCount; ++i) {
                            ranges[i] = this.computedValue(ranges[i]);
                            if (i < argCount - 1)
                                arg += ranges[i] + this.getParseDateTimeSeparator();
                            else
                                arg += ranges[i];
                        }
                        arg = arg.split(this.tic).join(this._string_empty);
                        dt = new Date(Date.parse(arg));
                    }
                }
            } catch (ex) {
                return this.formulaErrorStrings[this.invalid_arguments];
            }
            if (args[0].indexOf("2001") == -1 && dt.getFullYear() == 2001) {
                dt.setFullYear((new Date(Date.now())).getFullYear());
            }
            var days = this._toOADate(dt);
            if (isNaN(days)) {
                if (this.getRethrowLibraryComputationExceptions())
                    throw this.formulaErrorStrings[this._invalid_Math_argument];
                return this.getErrorStrings()[1].toString();
            }
            if (this.treat1900AsLeapYear && days > 59) {
                days += 1;
            }

            return Math.round(days).toString();
        };
        this.computeDay = function (argList) {
            var day = 1;
            var s = this.getValueFromArg(argList);
            var serialdate = this._parseDouble(s);
            if (!isNaN(serialdate)) {
                if (serialdate < 1) {
                    if (this.getRethrowLibraryComputationExceptions())
                        throw this.formulaErrorStrings[this._invalid_Math_argument];
                    return this.getErrorStrings()[4].toString();
                } else {
                    date = this._getDateFromSerialDate(serialdate);
                    day = date.getDate();
                }
            } else {
                var date = new Date(Date.parse(s));
                if (date.toString() != "Invalid Date") {
                    day = date.getDate();
                } else {
                    if (this.getRethrowLibraryComputationExceptions())
                        throw this.formulaErrorStrings[this._invalid_Math_argument];
                    return this.getErrorStrings()[1].toString();
                }
            }
            return day.toString();
        };
        this.computeDays = function (argList) {
            var argument = argList;
            var args = this.splitArgsPreservingQuotedCommas(argList);
            if (args.length != 2) {
                if (this.getRethrowLibraryComputationExceptions())
                    throw this.getFormulaErrorStrings()[this._wrong_number_arguments];
                return this.getFormulaErrorStrings()[this._wrong_number_arguments];
            }

            var startDateValue = this.getValueFromArg(args[1]);
            var endDateValue = this.getValueFromArg(args[0]);

            var startDateTime, endDateTime;

            var startDateSerial = this._parseDouble(startDateValue);
            var endDateSerial = this._parseDouble(endDateValue);

            if (!isNaN(startDateSerial)) {
                startDateTime = this._getDateFromSerialDate(startDateSerial);
            } else
                startDateTime = new Date(Date.parse(startDateValue));

            if (!isNaN(endDateSerial)) {
                endDateTime = this._getDateFromSerialDate(endDateSerial);
            } else
                endDateTime = new Date(Date.parse(endDateValue));

            var timeDiff = endDateTime.getTime() - startDateTime.getTime();

            var diffDays = Math.ceil(timeDiff / (1000 * 3600 * 24));
            return diffDays.toString();
        };
        this.computeDays360 = function (argList) {
            var args = this.splitArgsPreservingQuotedCommas(argList);
            var argCount = args.length;
            if (argCount != 2 && argCount != 3) {
                return this.formulaErrorStrings[this._wrong_number_arguments];
            }

            var method = false;
            var days = 0;
            var dt1 = new Date(Date.parse(this.getValueFromArg(args[0])));
            var dt2 = new Date(Date.parse(this.getValueFromArg(args[1])));
            var serialdate1 = this._parseDouble(this.getValueFromArg(args[0]));
            var serialdate2 = this._parseDouble(this.getValueFromArg(args[1]));

            if ((!isNaN(serialdate1) || !isNaN(this._isDate(dt1))) && (!isNaN(serialdate2) || !isNaN(this._isDate(dt2)))) {
                dt1 = (serialdate1 > 0) ? this._getDateFromSerialDate(serialdate1) : dt1;
                dt2 = (serialdate2 > 0) ? this._getDateFromSerialDate(serialdate2) : dt2;
                var flipSign = false;
                var mathodStr = this.falseValueStr;
                if (argCount == 3)
                    mathodStr = this.getValueFromArg(args[2]);
                method = (mathodStr == this.trueValueStr) ? true : false;
                if (dt1.getDate() == 31) {
                    dt1 = new Date(dt1.setDate(dt1.getDate() - 1));
                }

                if (dt2.getDate() == 31 && !method && dt1.getDate() < 30) {
                    dt2 = new Date(dt2.setDate(dt2.getDate() + 1));
                } else if (dt2.getDate() == 31) {
                    dt2 = new Date(dt2.setDate(dt2.getDate() - 1));
                }

                if (dt2 < dt1) {
                    flipSign = true;
                    var t = dt1;
                    dt1 = dt2;
                    dt2 = t;
                }

                days = dt2.getDate() - dt1.getDate();
                days += 30 * (dt2.getMonth() - dt1.getMonth());
                days += 360 * (dt2.getFullYear() - dt1.getFullYear());
                if (flipSign) {
                    days = -days;
                }
            } else {
                if (this.getRethrowLibraryComputationExceptions())
                    throw this.formulaErrorStrings[this._invalid_Math_argument];
                return this.getErrorStrings()[1].toString();
            }
            return days.toString();
        };
        this.computeEDate = function (argList) {
            var argument = argList;
            var args = this.splitArgsPreservingQuotedCommas(argList);
            var argCount = args.length;
            if (argCount != 2) {
                if (this.getRethrowLibraryComputationExceptions())
                    throw this.formulaErrorStrings[this._wrong_number_arguments];
                return this.formulaErrorStrings[this._wrong_number_arguments];
            }
            var startDate = this.getValueFromArg(args[0]).split(this.tic).join("");
            var StartDateValue = parseInt(startDate);
            var startDateTime = (isNaN(StartDateValue)) ? new Date(Date.parse(startDate)) : this._fromOADate(StartDateValue);
            var monthCount = this.getValueFromArg(args[1]).split(this.tic).join("");
            var month = parseInt(monthCount);
            if (startDateTime.toString() == "Invalid Date" && isNaN(month)) {
                try {
                    var today = new Date(Date.now());
                    var first = this._parseDouble(startDate);
                    var second = parseInt(monthCount);
                    if (second < 0 && second < first) {
                        if (this.getRethrowLibraryComputationExceptions())
                            throw this.formulaErrorStrings[this._bad_index];
                        return this.getErrorStrings()[4].toString();
                    }
                    if (first != 0) {
                        var dt = this.dateTime1900;
                        dt = this._fromOADate(first);
                        startDateTime = new Date(dt.setMonth(dt.getMonth() + second));
                    } else {
                        startDateTime = new Date(today.setMonth(today.getMonth() + second));
                        var days = this.computeDays(today.toLocaleDateString() + this.getParseArgumentSeparator() + startDateTime.toLocaleDateString());
                        return days;
                    }
                }
                catch (ex) {
                    if (this.getRethrowLibraryComputationExceptions())
                        throw this.formulaErrorStrings[this._bad_formula] + ex;
                    return this.getErrorStrings()[1].toString();
                }
            }
            startDateTime = new Date(startDateTime.setMonth(startDateTime.getMonth() + month));
            if (this.getExcelLikeComputations()) {
                return startDateTime.toLocaleDateString();
            } else {
                return parseInt(this._toOADate(startDateTime)).toString();
            }
        };
        this.computeEOMonth = function (argList) {
            var argument = argList;
            var args = this.splitArgsPreservingQuotedCommas(argList);
            var argCount = args.length;
            if (argCount != 2) {
                if (this.getRethrowLibraryComputationExceptions())
                    throw this.formulaErrorStrings[this._wrong_number_arguments];
                return this.formulaErrorStrings[this._wrong_number_arguments];
            }
            var startDate = this.getValueFromArg(args[0]).split(this.tic).join("");
            var startDateTime = new Date(startDate);
            var month = this._parseDouble(this.getValueFromArg(args[1]).split(this.TIC).join(""));
            if(startDateTime.toString() == "Invalid Date")
            {
                if (this.getRethrowLibraryComputationExceptions())
                    throw this.formulaErrorStrings[this._bad_index];
                return this.getErrorStrings()[1].toString();
            }
            if (isNaN(month)) {
                try  {
                    var today = new Date(Date.now());
                    var first = parseInt(startDate);
                    month = this._parseDouble(this.getValueFromArg(args[1]).split(this.tic).join(""));
                    if (month < 0 && first <= 0 && month < first) {
                        if (this.getRethrowLibraryComputationExceptions())
                            throw this.formulaErrorStrings[this._bad_index];
                        return this.getErrorStrings()[4].toString();
                    }
                } catch (ex) {
                    if (this.getRethrowLibraryComputationExceptions())
                        throw this.formulaErrorStrings[this._bad_index];
                    return this.getErrorStrings()[1].toString();
                }
            }
            var dateValue = parseInt(startDate);
            if (!isNaN(dateValue) && startDateTime.getFullYear() > 9999) {
                startDateTime = this._fromOADate(dateValue);
            }
            startDateTime = new Date(startDateTime.setMonth(startDateTime.getMonth() + month));
            var totalDays = (new Date(startDateTime.getFullYear(), startDateTime.getMonth() + 1, 1, -1)).getDate();
            startDateTime = new Date(startDateTime.getFullYear(), startDateTime.getMonth(), totalDays);

            if (this.getExcelLikeComputations())
                return startDateTime.toLocaleDateString();
            else
                return this._getSerialDateTimeFromDate(startDateTime).toString();
        };
        this.computeHour = function (argList) {
            var time;
            var dt = new Date(Date.now());
            argList = this.getValueFromArg(argList);
            argList = argList.split(this.tic).join("");
            dt = new Date(Date.parse(argList));
            var hourValue = parseInt(argList);
            if (hourValue < 0)
            {
                if (this.getRethrowLibraryComputationExceptions())
                    throw this.formulaErrorStrings[this._bad_index];
                return this.getErrorStrings()[4].toString();
            }
            if (dt.toString() == "Invalid Date") {
                var argVal = (new Date(Date.now())).toLocaleDateString() + " " + argList;
                dt = new Date(Date.parse(argVal));
            }
            if (dt.toString() == "Invalid Date") {
                dt = this._fromOADate(argList);
            }
            if (dt.toString() == "Invalid Date") {
                if (this.getRethrowLibraryComputationExceptions())
                    throw this.formulaErrorStrings[this._bad_index];
                return this.getErrorStrings()[1].toString();
            }
            return dt.getHours().toString();
        };
        this.computeISOWeeknum = function (argList) {
            var args = this.splitArgsPreservingQuotedCommas(argList);
            var argCount = args.length;
            if (argCount != 1) {
                if (this.getRethrowLibraryComputationExceptions())
                    throw this.formulaErrorStrings[this._wrong_number_arguments];
                return this.formulaErrorStrings[this._wrong_number_arguments];
            }
            var value = this.getValueFromArg(args[0]);
            if (value[0] == (this.tic) && this._isTextEmpty(value.split(this.tic).join(""))) {
                if (this.getRethrowLibraryComputationExceptions())
                    throw this.formulaErrorStrings[this._bad_formula];
                return this.getErrorStrings()[1].toString();
            }
            if (this._isTextEmpty(value)) {
                argList = this.computeDatevalue(new Date(1990, 12, 28).toLocaleDateString());
            }

            if (argList.indexOf(":") > -1 || !this._isCellReference(this.getValueFromArg(args[0])) && this.getValueFromArg(args[0]).indexOf(":") > -1) {
                argList = "0";
            }
            var weekDate = this.getValueFromArg(args[0]).split(this.tic).join("");
            var weekDateTime = new Date(Date.parse(weekDate)), firstDate = this.dateTime1900;
            ;
            if (weekDateTime.toString() != "Invalid Date") {
                try  {
                    var weeknumber = parseInt(weekDate);
                    weekDateTime = this.fromOADate(weeknumber);
                    weekDate = weekDateTime.toLocaleDateString();
                } catch (ex) {
                    if (this.getRethrowLibraryComputationExceptions())
                        throw "Bad Cell reference";
                    return this.getErrorStrings()[1].toString();
                }
            }
            var isoarg = argList + this.getParseArgumentSeparator() + "21";
            var weekno = parseInt(this.computeWeeknum(isoarg)) - 1;
            return weekno.toString();
        };
        this.computeMinute = function (argList) {
            var time;
            var dt = new Date(Date.now());
            argList = this.getValueFromArg(argList);
            argList = argList.split(this.tic).join("");
            dt = new Date(Date.parse(argList));
            if (argList < 0) {
                if (this.getRethrowLibraryComputationExceptions())
                    throw this.formulaErrorStrings[this._bad_index];
                return this.getErrorStrings()[4].toString();
            }
            if (dt.toString() == "Invalid Date") {
                var argVal = (new Date(Date.now())).toLocaleDateString() + " " + argList;
                dt = new Date(Date.parse(argVal));
            }
            if (dt.toString() == "Invalid Date") {
                dt = this._fromOADate(argList);
            }
            if (dt.toString() == "Invalid Date") {
                if (this.getRethrowLibraryComputationExceptions())
                    throw this.formulaErrorStrings[this._bad_index];
                return this.getErrorStrings()[1].toString();
            }
            return dt.getMinutes().toString();
        };
        this.computeMonth = function (argList) {
            var month = 1;
            var s = this.getValueFromArg(argList);
            var date = new Date(Date.parse(s.split(this.tic).join("")));

            var dateValue = parseInt(s);
            if (!isNaN(dateValue) && date.getFullYear() > 9999) {
                date = this._fromOADate(dateValue);
            }
            if (s < 0) {
                if (this.getRethrowLibraryComputationExceptions())
                    throw this.formulaErrorStrings[this._bad_index];
                return this.getErrorStrings()[4].toString();
            }
            if (date.toString() == "Invalid Date") {
                argList = (new Date(Date.now())).toLocaleDateString() + " " + argList;
                date = new Date(Date.parse(argList));
            }
            if (date.toString() == "Invalid Date") {
                if (this.getRethrowLibraryComputationExceptions())
                    throw this.formulaErrorStrings[this._bad_index];
                return this.getErrorStrings()[1].toString();
            }
            if (date.toString() != "Invalid Date") {
                month = date.getMonth() + 1;
            } else {
                var serialdate = this._parseDouble(this.getValueFromArg(argList));
                if (!isNaN(serialdate)) {
                    ////serialdate = serialdate - 1 - ((Treat1900AsLeapYear && serialdate > 59) ?  1 : 0);
                    if (serialdate < 1) {
                        month = 1;
                    } else {
                        date = this._getDateFromSerialDate(serialdate); ////dateTime1900.AddDays(serialdate);
                        month = date.getMonth() +1;
                    }
                } else {
                    return this.formulaErrorStrings[this._invalid_arguments];
                }
            }
            return month.toString();
        };
        this.computeNetworkDays = function (argList) {
            var args = this.splitArgsPreservingQuotedCommas(argList);
            var argCount = args.length;
            var adjustRange, s1;
            var date;
            var holidays = [];

            if (argCount != 2 && argCount != 3) {
                return this.formulaErrorStrings[this._wrong_number_arguments];
            }

            var startDate = this.getValueFromArg(args[0]).split(this.tic).join("");
            var endDate = this.getValueFromArg(args[1]).split(this.tic).join("");
            var startDateTime = new Date(Date.parse(startDate));
            var endDateTime = new Date(Date.parse(endDate));
            var dval = this._parseDouble(startDate);
            if (!isNaN(dval) && startDateTime.getFullYear() > 9999) {
                startDateTime = this._fromOADate(dval);
            }
            if (startDateTime.toString() == "Invalid Date") {
                startDateTime = this._fromOADate(dval);
            }
            if (startDateTime.toString() == "Invalid Date") {
                if (this.getRethrowLibraryComputationExceptions())
                    throw this.getErrorStrings()[1].toString();
                return this.getErrorStrings()[1].toString();
            }
            var dval = this._parseDouble(endDate);
            if (!isNaN(dval) && endDateTime.getFullYear() > 9999) {
                endDateTime = this._fromOADate(dval);
            }
            if (endDateTime.toString() == "Invalid Date") {
                endDateTime = this._fromOADate(dval);
            }
            if (endDateTime.toString() == "Invalid Date") {
                if (this.getRethrowLibraryComputationExceptions())
                    throw this.getErrorStrings()[1].toString();
                return this.getErrorStrings()[1].toString();
            }
            if (argCount == 3) {
                adjustRange = args[2];
                ////is a cellrange
                if (adjustRange.indexOf(':') > -1) {
                    var cells = this.getCellsFromArgs(adjustRange);
                    for (var s in cells) {
                        try  {
                            s1 = this.getValueFromArg(s).split(this.tic).join("");
                        } catch (ex) {
                            if (this._rethrowLibraryComputationExceptions && this.getLibraryComputationException() != null) {
                                throw this.getLibraryComputationException();
                            }

                            return ex;
                        }

                        if (s1 != "") {
                            date = new Date(Date.parse(s1));
                            if (date.toString() != "Invalid Date" && (holidays.indexOf(date) == -1) && date > startDateTime && date < endDateTime && date.getDay() != 0 && date.getDay() != 6) {
                                holidays.push(date);
                            }
                        }
                    }
                } else {
                    var dateArray = this.splitArgsPreservingQuotedCommas(adjustRange.split(this.tic).join(""));
                    for (var i = 0; i < dateArray.length; i++) {
                        try {
                            s1 = this.getValueFromArg(this.tic + dateArray[i] + this.tic).split(this.tic).join("");
                        } catch (ex) {
                            if (this._rethrowLibraryComputationExceptions && this.getLibraryComputationException() != null) {
                                throw this.getLibraryComputationException();
                            }

                            return ex;
                        }

                        if (s1.length > 0) {
                            date = new Date(Date.parse(s1));
                            if (date.toString() != "Invalid Date" && (holidays.indexOf(date) == -1) && date > startDateTime && date < endDateTime && date.getDay() != 0 && date.getDay() != 6) {
                                holidays.push(date);
                            }
                        }
                    }
                }
            }

            var numberOfWorkingDays = 0;

            var noofDays = parseInt(this.computeDatevalue(endDate)) - parseInt(this.computeDatevalue(startDate));

            var startWeekDay = startDateTime.getDay();
            var endWeekDay = endDateTime.getDay();
            var excesStartDays = startWeekDay - 1;
            var excesEndDays = 7 - endWeekDay;
            noofDays = noofDays + excesStartDays + excesEndDays;

            var noofholidays = parseInt((noofDays / 7) * 2);

            numberOfWorkingDays = noofDays - excesStartDays - excesEndDays - noofholidays + 2 - holidays.length;
            if (endWeekDay == 6 || startWeekDay == 0)
                numberOfWorkingDays--;
            return numberOfWorkingDays.toString();
        };
        this.computeNetworkDaysintl = function (argList) {
            var args = this.splitArgsPreservingQuotedCommas(argList);
            var argCount = args.length;
            var adjustRange, s1;
            var date;
            var holidays = [];

            if (argCount != 2 && argCount != 3) {
                return this.formulaErrorStrings[this._wrong_number_arguments];
            }

            var startDate = this.getValueFromArg(args[0]).split(this.tic).join("");
            var endDate = this.getValueFromArg(args[1]).spli(this.tic).join("");
            var startDateTime = new Date(Date.parse(startDate));
            var endDateTime = new Date(Date.parse(endDate));

            if (startDateTime.toString() == "Invalid Date") {
                var dval = this._parseDouble(startDate);
                if (!isNaN(dval)) {
                    startDateTime = this.fromOADate(dval);
                } else {
                    if (this.getRethrowLibraryComputationExceptions())
                        throw this.getErrorStrings()[1].toString();
                    return this.getErrorStrings()[1].toString();
                }
            }
            if (endDateTime.toString() == "Invalid Date") {
                var dval = this._parseDouble(endDate);
                if (!isNaN(dval)) {
                    endDateTime = this._fromOADate(dval);
                } else {
                    if (this.getRethrowLibraryComputationExceptions())
                        throw this.getErrorStrings()[1].toString();
                    return this.getErrorStrings()[1].toString();
                }
            }
            if (argCount == 4) {
                adjustRange = args[3];
                adjustRange = this.adjustRangeArg(adjustRange);

                ////is a cellrange
                if (adjustRange.indexOf(':') > -1 || adjustRange.indexOf(',') > -1) {
                    var cells = this.getCellsFromArgs(adjustRange);
                    for (var s in cells) {
                        try  {
                            s1 = this.getValueFromArg(s).split(this.tic).join("");
                        } catch (ex) {
                            if (this._rethrowLibraryComputationExceptions && this.getLibraryComputationException() != null) {
                                throw this.getLibraryComputationException();
                            }

                            return ex;
                        }

                        if (s1 != "") {
                            date = new Date(Date.parse(s1));
                            if (date.toString() != "Invalid Date" && (holidays.indexOf(date) == -1) && date > startDateTime && date > endDateTime && date.getDay() != 0 && date.getDay() != 6) {
                                holidays.push(date);
                            }
                        }
                    }
                } else {
                    try  {
                        s1 = this.getValueFromArg(adjustRange).split(this.tic).join("");
                    } catch (ex) {
                        if (this._rethrowLibraryComputationExceptions && this.getLibraryComputationException() != null) {
                            throw this.getLibraryComputationException();
                        }

                        return ex;
                    }

                    if (s1.length > 0) {
                        date = new Date(Date.parse(s1));
                        if (date.toString() != "Invalid Date" && (holidays.indexOf(date) == -1) && date > startDateTime && date > endDateTime && date.getDay() != 0 && date.getDay() != 6) {
                            holidays.push(date);
                        }
                    }
                }
            }

            var weekend = parseInt(args[2]);
            if (!(argCount > 2 && !isNaN(weekend))) {
                weekend = 1;
            } else {
                if (this.getRethrowLibraryComputationExceptions())
                    throw this.formulaErrorStrings[this._bad_formula];
                return this.getErrorStrings()[4].toString();
            }
            var weekendDays = this._splitArguments(this.getWeekEndType()[weekend].toString(), ',');
            var noofDays = parseInt(this.computeDatevalue(endDate)) - parseInt(this.computeDatevalue(startDate)) + 1;
            var netWorkDays = noofDays;
            var j = 1;
            var tempStartDate = startDateTime;
            while (j < noofDays) {
                var dayofweek = tempStartDate.getDay();
                if (weekendDays.indexOf(dayofweek.toString()) >= 0) {
                    netWorkDays--;
                } else if (holidays.indexOf(tempStartDate) >= 0) {
                    netWorkDays--;
                }
                tempStartDate = new Date(tempStartDate.setDate(tempStartDate.getDate() + 1));
                j++;
            }
            return netWorkDays.toString();
        };
        this.computeNow = function (argList) {
            var args = this.splitArgsPreservingQuotedCommas(argList);
            var argCount = args.length;
            if (argCount != 1) {
                return this.formulaErrorStrings[this._wrong_number_arguments];
            }
            return new Date(Date.now()).toString();
        };
        this.computeSecond = function (argList) {
            var args = this.splitArgsPreservingQuotedCommas(argList);
            var argCount = args.length;
            if (argCount != 1) {
                return this.formulaErrorStrings[this._wrong_number_arguments];
            }
            var time;
            var dt = new Date(Date.now());
            argList = this.getValueFromArg(argList);
            argList = argList.split(this.tic).join("");
            dt = new Date(Date.parse(argList));
            if (argList < 0) {
                if (this.getRethrowLibraryComputationExceptions())
                    throw this.formulaErrorStrings[this._bad_index];
                return this.getErrorStrings()[4].toString();
            }
            if (dt.toString() == "Invalid Date") {
                var argVal = (new Date(Date.now())).toLocaleDateString() + " " + argList;
                dt = new Date(Date.parse(argVal));
            }
            if (dt.toString() == "Invalid Date") {
                dt = this._fromOADate(argList);
            }
            if (dt.toString() == "Invalid Date") {
                if (this.getRethrowLibraryComputationExceptions())
                    throw this.formulaErrorStrings[this._bad_index];
                return this.getErrorStrings()[1].toString();
            }
            return dt.getSeconds().toString();
        };
        this.computeTime = function (argList) {
            var args = this.splitArgsPreservingQuotedCommas(argList);
            var argCount = args.length;
            if (argCount != 3) {
                return this.formulaErrorStrings[this._wrong_number_arguments];
            }

            var hour = this._parseDouble(this.getValueFromArg(args[0]));
            var minute = this._parseDouble(this.getValueFromArg(args[1]));
            var second = this._parseDouble(this.getValueFromArg(args[2]));
            var time = 0;

            if (!isNaN(hour) && !isNaN(minute) && !isNaN(second)) {
                time = (3600 * hour + 60 * minute + second) / 86400;
                var dateString = (new Date(Date.now())).toLocaleDateString() + " " + time;
                dt = new Date(Date.parse(dateString));
                if (time < 0) {
                    if (this.getRethrowLibraryComputationExceptions())
                        throw this.formulaErrorStrings[this._bad_index];
                    return this.getErrorStrings()[4].toString();
                }
                return time.toString();
            }
            else {
                if (this.getRethrowLibraryComputationExceptions())
                    throw this.formulaErrorStrings[this._bad_index];
                return this.getErrorStrings()[1].toString();
            }
        };
        this.computeTimevalue = function (argList) {
            var args = this.splitArgsPreservingQuotedCommas(argList);
            var argCount = args.length;
            if (argCount != 1) {
                return this.formulaErrorStrings[this._wrong_number_arguments];
            }
            var argValue = this.getValueFromArg(args[0]).split(this.tic).join("");

            argValue = argValue.split(this.tic).join("");
            if (argValue.indexOf(":") > -1) {
                var date = new Date();
                var argValue = date.toLocaleDateString() + " " + argValue;
            }
            var dateTime = new Date(Date.parse(argValue));
            if (dateTime.toString() == "Invalid Date") {
                argList = (new Date(Date.now())).toLocaleDateString() + " " + argList;
                dateTime = new Date(Date.parse(argList));
            }
            if (dateTime.toString() == "Invalid Date") {
                if (this.getRethrowLibraryComputationExceptions())
                    throw this.formulaErrorStrings[this._bad_index];
                return this.getErrorStrings()[1].toString();
            }
            var time = (3600 * dateTime.getHours() + 60 * dateTime.getMinutes() + dateTime.getSeconds()) / 86400;
            return time.toString();
        };
        this.computeToday = function (argList) {
            var dt = new Date(Date.now());

            if (this.getExcelLikeComputations()) {
                return dt.toLocaleDateString();
            } else
                return this._toOADate(dt).toString();
        };
        this.computeWeekday = function (argList) {
            var args = this.splitArgsPreservingQuotedCommas(argList);
            var argCount = args.length;
            if (argCount != 1 && argCount != 2) {
                return this.formulaErrorStrings[this._wrong_number_arguments];
            }

            var dateString = this.getValueFromArg(args[0]);

            var dt = new Date(Date.parse(dateString.split(this.tic).join("")));
            var dateValue = parseInt(dateString);
            if (!isNaN(dateValue) && dt.getFullYear() > 9999) {
                dt = this._fromOADate(dateValue);
            }
            args[1] = (argCount == 2) ? args[1] : "1";
            if (dt.toString() == "Invalid Date")
                dt = this._fromOADate(dateValue);
            var return_type = parseInt(this.getValueFromArg(args[1]));
            var day = dt.getDay();

            if (return_type == 1) {
                day += 1;
            } else {
                if (day == 0) {
                    day = 7;
                }

                if (return_type == 3) {
                    day -= 1;
                }
            }
            return day.toString();
        };
        this.computeWeeknum = function (argList) {
            var args = this.splitArgsPreservingQuotedCommas(argList);
            var argCount = args.length;
            if (argCount > 3) {
                return this.formulaErrorStrings[this._wrong_number_arguments];
            }
            var weekDate = this.getValueFromArg(args[0]).split(this.tic).join("");

            var weekDateTime = new Date(Date.parse(weekDate));

            if (weekDateTime.toString() == "Invalid Date")
                ;
            {
                if (this.getRethrowLibraryComputationExceptions())
                    throw this.formulaErrorStrings[this._bad_formula];
                return this.getErrorStrings()[4].toString();
            }
            var weekStart = ["", "0", "1", "", "", "", "", "", "", "", "", "1", "2", "3", "4", "5", "6", "0", "", "", "", "1"];

            var weektype = 1;
            weektype = parseInt(this.getValueFromArg(args[1]).split(this.tic).join(""));

            if (!isNaN(weektype)) {
            }
            var weehStartValue = parseInt(weekStart[weektype]);
            var firstDate = new Date(weekDateTime.getFullYear(), 1, 1);
            var firstDay = firstDate.getDay();
            var offset = (firstDay < weehStartValue) ? weehStartValue - firstDay + 1 : firstDay - weehStartValue;
            var dt = new Date(this._toOADate(firstDate) - offset);
            var noofDays = this._toOADate(weekDateTime) - this._toOADate(dt);
            var dval = Math.floor(noofDays / 7) + 1 + weehStartValue;
            return dval.toString();
        };
        this.computeWorkDay = function (argList) {
            var args = this.splitArgsPreservingQuotedCommas(argList);
            var argCount = args.length;

            if (argCount != 2 && argCount > 3) {
                if (this.getRethrowLibraryComputationExceptions())
                    throw this.formulaErrorStrings[this._wrong_number_arguments];
                return this.formulaErrorStrings[this._wrong_number_arguments];
            }
            var argString = "";
            var dateValue = 0;
            if (argCount == 3) {
                argString = args[0] + this.getParseArgumentSeparator() + args[1] + this.getParseArgumentSeparator() + "1" + this.getParseArgumentSeparator() + args[2];
            } else
                argString = args[0] + this.getParseArgumentSeparator() + args[1] + this.getParseArgumentSeparator() + "1";
            var value = this.computeWorkDayintl(argString);

            var dateTime = new Date(Date.parse(value));
            if (dateTime.toString() == "Invalid Date") {
                if (this.getRethrowLibraryComputationExceptions())
                    throw this.formulaErrorStrings[this.bad_formula];
                return this.getErrorStrings()[1].toString();
            }

            return dateTime.toLocaleDateString();
        };
        this.computeWorkDayintl = function (argList) {
            var args = this.splitArgsPreservingQuotedCommas(argList);
            var argCount = args.length;

            if (argCount > 4 || argCount < 2) {
                if (this.getRethrowLibraryComputationExceptions())
                    throw this.formulaErrorStrings[this._wrong_number_arguments];
                return this.formulaErrorStrings[this._wrong_number_arguments];
            }
            var adjustRange, s1;
            var date;
            var holidays;

            var startDate = this.getValueFromArg(args[0]).split(this.tic).join("");
            var startDateTime = new Date(Date.parse(startDate));

            var temp = this.getValueFromArg(args[1]).split(this.tic).join("");
            var dateval;
            if (isNaN(temp)) {
                if (this.getRethrowLibraryComputationExceptions())
                    throw this.formulaErrorStrings[this.bad_formula];
                return this.getErrorStrings()[1].toString();
            } else {
                dateval = this._toOADate(startDateTime) + temp;
            }

            if (startDate.split(this.tic).join("") == "") {
                if (this.getRethrowLibraryComputationExceptions())
                    throw this.formulaErrorStrings[this.bad_formula];
                return this.getErrorStrings()[0].toString();
            } else if (startDateTime.toString() == "Invalid Date") {
                if (this.getRethrowLibraryComputationExceptions())
                    throw this.formulaErrorStrings[this.bad_formula];
                return this.getErrorStrings()[1].toString();
            }
            var totalDays = parseInt(temp.toString());
            if (argCount == 4) {
                adjustRange = args[3];
                adjustRange = this.adjustRangeArg(adjustRange);

                ////is a cellrange
                if (adjustRange.indexOf(':') > -1 || adjustRange.indexOf(',') > -1) {
                    var cells = this.getCellsFromArgs(adjustRange);
                    for (var s in cells) {
                        try  {
                            s1 = this.getValueFromArg(cells[s]).split(this.tic).join("");
                        } catch (ex) {
                            if (this.getRethrowLibraryComputationExceptions())
                                throw this.formulaErrorStrings[this.bad_formula];
                            return this.getErrorStrings()[4].toString();
                        }

                        if (s1 != "") {
                            var dt = new Date(Date.parse(s1));
                            if (dt.toString() != "Invalid Date" && holidays.indexOf(date) == -1 && date > startDateTime) {
                                holidays.push(date);
                            }
                        }
                    }
                } else {
                    try  {
                        s1 = this.getValueFromArg(adjustRange).split(this.tic).join("");
                    } catch (Exception) {
                        if (this.getRethrowLibraryComputationExceptions())
                            throw this.formulaErrorStrings[this.bad_formula];
                        return this.getErrorStrings()[4].toString();
                    }

                    if (s1 != "") {
                        var dt = new Date(Date.parse(s1));
                        if (dt.toString() != "Invalid Date" && holidays.indexOf(date) == -1 && date > startDateTime) {
                            holidays.push(date);
                        }
                    }
                }
            }
            var weekend = parseInt(args[2]);
            if (!(argCount > 2 && !isNaN(weekend))) {
                weekend = 1;
            }
            if (weekend == 0) {
                if (this.getRethrowLibraryComputationExceptions())
                    throw this.formulaErrorStrings[this.bad_formula];
                return this.getErrorStrings()[4].toString();
            }
            var weekendDays = [this._splitArguments(this.weekEndType[weekend].toString()), ","];
            var j = 0;
            var tempStartDate = startDateTime;
            if (totalDays < 0) {
                while (j > totalDays) {
                    tempStartDate = new Date(tempStartDate.setDate(tempStartDate.getDate() - 1));
                    var dayofweek = tempStartDate.getDay();
                    if (weekendDays.indexOf(dayofweek.toString()) < 0 && holidays.indexOf(tempStartDate) < 0) {
                        j--;
                    }
                }
            } else {
                while (j < totalDays) {
                    tempStartDate = new Date(tempStartDate.setDate(tempStartDate.getDate() + 1));
                    var dayofweek = tempStartDate.getDay();
                    if (weekendDays.indexOf(dayofweek.toString()) < 0 && holidays.indexOf(tempStartDate) < 0) {
                        j--;
                    }
                }
            }
            return this._toOADate(tempStartDate).toString();
        };
        this.computeYear = function (argList) {
            var args = this.splitArgsPreservingQuotedCommas(argList);
            var argCount = args.length;

            if (argCount != 1) {
                if (this.getRethrowLibraryComputationExceptions())
                    throw this.formulaErrorStrings[this._wrong_number_arguments];
                return this.formulaErrorStrings[this._wrong_number_arguments];
            }
            var startDate = this.getValueFromArg(args[0]).split(this.tic).join("");
            var startDateTime = new Date(Date.parse(startDate));
            var dateValue = parseInt(startDate);
            if (!isNaN(dateValue) && startDateTime.getFullYear() > 9999) {
                startDateTime = this._fromOADate(dateValue);
            }
            if (dateValue == 0) {
                return "1900";
            }
            if (dateValue < 0) {
                if (this.getRethrowLibraryComputationExceptions())
                    throw this.formulaErrorStrings[this.bad_formula];
                return this.getErrorStrings()[4].toString();
            }
            if (startDate.split(this.tic).join("") == "") {
                if (this.getRethrowLibraryComputationExceptions())
                    throw this.formulaErrorStrings[this.bad_formula];
                return this.getErrorStrings()[0].toString();
            } else if (startDateTime.toString() == "Invalid Date") {
                if (this.getRethrowLibraryComputationExceptions())
                    throw this.formulaErrorStrings[this.bad_formula];
                return this.getErrorStrings()[1].toString();
            }

            return startDateTime.getFullYear().toString();
        };


        this.computeChar = function (arg) {
            var ranges = this.splitArgsPreservingQuotedCommas(arg);
            var argCount = ranges.length;
            if (argCount != 1) {
                if (this.getRethrowLibraryComputationExceptions() && this.getLibraryComputationException() != null)
                    throw this.getLibraryComputationException();
                return this.formulaErrorStrings[this._wrong_number_arguments];
            }
            var s = this.getValueFromArg(ranges[0]).split(this.tic).join("");
            var i = this._parseDouble(s);
            if (!isNaN(i) && i > 0 && i < 256) {
                return String.fromCharCode(s);
            }
            if (this.getErrorStrings().indexOf(s) > -1) {
                if (this.getRethrowLibraryComputationExceptions() && this.getLibraryComputationException() != null)
                    throw this.getLibraryComputationException();
                return s;
            }
            return (s[0] == this.tic) ? this.getErrorStrings()[1].toString() : this.getErrorStrings()[5].toString();
        };
        this.computeCode = function (argList) {
            var args = this.splitArgsPreservingQuotedCommas(argList);
            var argCount = args.length;
            if (argCount != 1) {
                if (this.getRethrowLibraryComputationExceptions() && this.getLibraryComputationException() != null)
                    throw this.getLibraryComputationException();
                return this.formulaErrorStrings[this._wrong_number_arguments];
            }
            var text = this.getValueFromArg(args[0]).split(this.tic).join("");
            var num =  parseInt(text);
            if (!isNaN(num))
            {
                return text.charCodeAt(0).toString();
            }
            if (text == null || text == "") {
                if (this.getRethrowLibraryComputationExceptions() && this.getLibraryComputationException() != null)
                    throw this.getLibraryComputationException();
                return (args[0].length > 4) ? this.getErrorStrings()[5].toString() : this.getErrorStrings()[1].toString();
            }
            if (!this._isCellReference(args[0]) && args[0].indexOf(this.tic) == -1) {
                if (this.getRethrowLibraryComputationExceptions() && this.getLibraryComputationException() != null)
                    throw this.getLibraryComputationException();
                return this.getErrorStrings()[5].toString();
            }
            var code = text;

            return code.charCodeAt(0).toString();
        };
        this.computeUniCode = function (argList) {
            var args = this.splitArgsPreservingQuotedCommas(argList);
            var argCount = args.length;
            if (argCount != 1) {
                if (this.getRethrowLibraryComputationExceptions() && this.getLibraryComputationException() != null)
                    throw this.getLibraryComputationException();
                return this.formulaErrorStrings[this._wrong_number_arguments];
            }
            var text = this.getValueFromArg(args[0]);
            if (!this._isCellReference(args[0]) && args[0].indexOf(this.tic) == -1) {
                if (this.getRethrowLibraryComputationExceptions() && this.getLibraryComputationException() != null)
                    throw this.getLibraryComputationException();
                return this.getErrorStrings()[5].toString();
            }
            var str = this._parseDouble(text);
            if (text == "invalid expression" || text == null || text == "" || !isNaN(str)) {
                if (this.getRethrowLibraryComputationExceptions() && this.getLibraryComputationException() != null)
                    throw this.getLibraryComputationException();
                return this.getErrorStrings()[1].toString();
            }
            var unicodeString = String.fromCharCode(text);
            return unicodeString.toString();
        };
        this.computeUpper = function (args) {

            if (!this._isCellReference(args) && args[0].indexOf(this.tic) == -1)
            {
                if (this.getRethrowLibraryComputationExceptions() && this.getLibraryComputationException() != null)
                    throw this.getLibraryComputationException();
                return this.getErrorStrings()[5].toString();
            }
            return this._stripTics0(this.getValueFromArg(args)).toUpperCase();
        };
        this.computeLower = function (args) {

            if (!this._isCellReference(args) && args[0].indexOf(this.tic) == -1) {
                if (this.getRethrowLibraryComputationExceptions() && this.getLibraryComputationException() != null)
                    throw this.getLibraryComputationException();
                return this.getErrorStrings()[5].toString();
            }
            return this._stripTics0(this.getValueFromArg(args)).toLowerCase();
        };
        this.computeLen = function (range) {

            var args = this.splitArgsPreservingQuotedCommas(range);
            var argCount = args.length;
            if (argCount != 1) {
                return this.formulaErrorStrings[this._wrong_number_arguments];
            }            
            var s1 = this._stripTics0(this.getValueFromArg(args[0]));
            var hasTics = s1[0] == this.tic && s1[s1.length - 1] == this.tic;
            return (hasTics ? s1.length - 2 : s1.length).toString();
        };
        this.computeMid = function (range) {
            var args = this.splitArgsPreservingQuotedCommas(range);
            var argCount = args.length;
            if (argCount != 3) {
                return this.formulaErrorStrings[this._wrong_number_arguments];
            }
            var s1 = args[0];
            s1 = this.getValueFromArg(s1);
            if (!this._isCellReference(args[0]) && args[0][0] != this.tic)
            {
                if (this.getRethrowLibraryComputationExceptions() && this.getLibraryComputationException() != null)
                    throw this.getLibraryComputationException();
                return this.getErrorStrings()[5].toString();
            }
            var hasTics = s1[0] == this.tic && s1[s1.length - 1] == this.tic;
            var s2 = this.getValueFromArg(args[1]);
            var len = this._parseDouble(args[2]);
            s2 = this.getValueFromArg(args[1]);
            var start = Number(s2) + Number(hasTics ? 0 : -1);

            if (args[1].indexOf("#VALUE!") > -1) {
                return "#VALUE!";
            }
            if (Number(start) > s1.length) {
                return "";
            }
            if (start + len > s1.length) {
                s1 = s1.substring(start);
            }
            else
                s1 = this._substring(s1, start, len);
            if (hasTics && s1[0] != this.tic) {
                s1 = this.tic + s1;
            }

            if (hasTics && s1) {
                s1 = s1 + this.tic;
            }

            return this._stripTics0(s1);
        };
        this.computeLeft = function (range) {
            var args = this.splitArgsPreservingQuotedCommas(range);
            var argCount = args.length;
            if (argCount != 1 && argCount != 2) {
                return this.formulaErrorStrings[this._wrong_number_arguments];
            }

            var s1 = this._stripTics0(this.getValueFromArg(args[0]));
            if (!this._isCellReference(args[0]) && args[0].indexOf(this.tic) == -1) {
                if (this.getRethrowLibraryComputationExceptions() && this.getLibraryComputationException() != null)
                    throw this.getLibraryComputationException();
                return this.getErrorStrings()[5].toString();
            }
            var hasTics = s1[0] == this.tic && s1[s1.length - 1] == this.tic;
            var s2 = (argCount == 2) ? args[1] : "1";
            s2 = this.computedValue(s2);
            var len = Number(s2) + Number(hasTics ? 1 : 0);
            len = (s1.length >= len) ? len : s1.length;
            len = (s1.length >= len) ? len : s1.length;
            if (len < 1) {
                return this.getErrorStrings()[1].toString();
            }
            s1 = s1.substring(0, len);
            if (hasTics && s1[s1.length - 1] != this.tic) {
                s1 = s1 + this.tic;
            }
            if (this.getUseNoAmpersandQuotes() && s1.length > 1 && s1[0] == this.tic[0] && s1[s1.length - 1] == this.tic[0]) {
                s1 = this._substring(s1,1, s1.length - 2);
            }
            return s1;
        };
        this.computeRight = function (range) {
            var args = this.splitArgsPreservingQuotedCommas(range);
            var argCount = args.length;
            if (argCount != 1 && argCount != 2) {
                return this.formulaErrorStrings[this._wrong_number_arguments];
            }           
            s1 = this._stripTics0(this.getValueFromArg(args[0]));
            if (!this._isCellReference(args[0]) && args[0].indexOf(this.tic) == -1) {
                if (this.getRethrowLibraryComputationExceptions() && this.getLibraryComputationException() != null)
                    throw this.getLibraryComputationException();
                return this.getErrorStrings()[5].toString();
            }
            var hasTics = s1[0] == this.tic && s1[s1.length - 1] == this.tic;
            var s2 = (argCount == 2) ? this.getValueFromArg(args[1]) : "1";
            var len = parseInt(s2);
            if (isNaN(len) || len <0 || s2.indexOf("#VALUE!") > -1) {
                return "#VALUE!";
            }
            if (len == 0)
                return "";
            var len = Number(s2) + Number(hasTics ? 1 : 0);
            var start = (s1.length >= len) ? s1.length - len : 0;
            s1 = s1.substring(start);
            if (hasTics && s1[0] != this.tic) {
                s1 = this.tic + s1;
            }

            return s1;
        };
        this.computeReplace = function (argList) {
            var args = this.splitArgsPreservingQuotedCommas(argList);
            var argCount = args.length;
            if (argCount != 4) {
                return this.formulaErrorStrings[this._wrong_number_arguments];
            }
            var firstText = this._stripTics0(this.getValueFromArg(args[0]));
            var newText = this._stripTics0(this.getValueFromArg(args[3]));
            var resultText = "";
            var startIndex = this._parseDouble(this.getValueFromArg(args[1]));
            var noOfChar = this._parseDouble(this.getValueFromArg(args[2]));
            if ((!this._isCellReference(args[0]) && args[0][0] != this.tic) || (!this._isCellReference(args[3]) && args[3][0] != this.tic)) {
                if (this.getRethrowLibraryComputationExceptions() && this.getLibraryComputationException() != null)
                    throw this.getLibraryComputationException();
                return this.getErrorStrings()[5].toString();
            }
            if (startIndex == 0 && noOfChar == 0)
            {
                if (this.getRethrowLibraryComputationExceptions() && this.getLibraryComputationException() != null)
                    throw this.getLibraryComputationException();
                else
                    return this.getErrorStrings()[1].toString();
            }
            if (!isNaN(startIndex) && !isNaN(noOfChar)) {
                try {
                    var replaceText = "";
                    if (firstText == null || firstText == "") {
                        return newText;
                    }
                    if (firstText.length <= startIndex && startIndex > noOfChar) {
                        resultText = firstText + newText;
                    } else if (firstText.length <= startIndex) {
                        replaceText = firstText;
                        resultText = firstText.split(replaceText).join(newText);
                    } else if (noOfChar == 0) {
                        resultText = newText + firstText;
                    }else {
                        replaceText = this._substring(firstText, startIndex - 1, noOfChar);
                        resultText = firstText.split(replaceText).join(newText);
                    }
                } catch (ex) {
                    if (this.getRethrowLibraryComputationExceptions() && this.getLibraryComputationException() != null)
                        throw this.getLibraryComputationException();
                    else
                        return this.getErrorStrings()[1].toString();
                }
            } else {
                if (this.getRethrowLibraryComputationExceptions() && this.getLibraryComputationException() != null)
                    throw this.getLibraryComputationException();
                else
                    return this.getErrorStrings()[1].toString();
            }

            return resultText;
        };
        this.computeExact = function (range) {
            //this.AdjustRangeArg(ref range);
            var args = this.splitArgsPreservingQuotedCommas(range);
            if (args.length != 2) {
                return this.formulaErrorStrings[this._requires_2_args];
            }

            var s1, s2, d;

            s1 = this.getValueFromArg(args[0]);
            s2 = this.getValueFromArg(args[1]);
            var str1 = this._parseDouble(s1);
            var str2 = this._parseDouble(s2);

            if ((!this._isCellReference(args[0]) && args[0].indexOf(this.tic) == -1) || (!this._isCellReference(args[1]) && args[1].indexOf(this.tic) == -1)) {
                if (this.getRethrowLibraryComputationExceptions() && this.getLibraryComputationException() != null)
                    throw this.getLibraryComputationException();
                return this.getErrorStrings()[5].toString();
            }
            if (!isNaN(str1)) {
                s1 = str1.toString();
            }

            if (!isNaN(str2)) {
                s2 = str2.toString();
            }

            return (s1 == s2) ? "TRUE" : "FALSE";
        };
        this.computeFind = function (arg) {
            var args = this.splitArgsPreservingQuotedCommas(arg);
            if (args.length != 2 && args.length != 3) {
                return this.formulaErrorStrings[this._wrong_number_arguments];
            }
            if ((!this._isCellReference(args[0]) && args[0].indexOf(this.tic) == -1) || (!this._isCellReference(args[1]) && args[1].indexOf(this.tic) == -1) || (!this._isCellReference(args[0]) && args[0].indexOf(this.tic) == -1)) {
                if (this.getRethrowLibraryComputationExceptions() && this.getLibraryComputationException() != null)
                    throw this.getLibraryComputationException();
                return this.getErrorStrings()[5].toString();
            }
            ////strip off outside tics (if any) on the evaluated arguments
            var lookFor = this._stripTics0(this.getValueFromArg(args[0]));
            var lookIn = this._stripTics0(this.getValueFromArg(args[1]));
            var start = 1;

            if (args.length == 3) {
                var s3 = this.getValueFromArg(args[2]);
                start = this._parseDouble(s3);
                if (isNaN(start)) {
                    start = 1;
                }
            }

            if (start <= 0 || start > lookIn.length) {
                return "#VALUE!";
            }

            var loc = lookIn.indexOf(lookFor, start - 1);
            if (loc < 0) {
                return "#VALUE!";
            }

            return (Number(loc) + Number(1)).toString();
        };
        this.computeSearch = function (argList) {
            var args = this.splitArgsPreservingQuotedCommas(argList);
            var argCount = args.length;
            var index;
            var startNum = 1;
            if (argCount != 2 && argCount != 3) {
                return this.formulaErrorStrings[this._wrong_number_arguments];
            }
            var firstStr = this._stripTics0(this.getValueFromArg(args[0])).toUpperCase();
            var secondStr = this._stripTics0(this.getValueFromArg(args[1])).toUpperCase();
            if ((!this._isCellReference(args[0]) && args[0][0] != this.tic) || (!this._isCellReference(args[1]) && args[1][0] != this.tic)) {
                if (this.getRethrowLibraryComputationExceptions() && this.getLibraryComputationException() != null)
                    throw this.getLibraryComputationException();
                return this.getErrorStrings()[5].toString();
            }

            var qusfirstStr = firstStr;
            if (firstStr.indexOf("*") >-1)
            {
                firstStr = firstStr.split("*").join("")[0].toString();
            }
            else if (firstStr.indexOf("?") >-1)
            {
                var subStr = qusfirstStr.substring(0, qusfirstStr.indexOf("?"));
                var strIndex = Number(secondStr.indexOf(subStr));
                while (secondStr.indexOf(qusfirstStr) == -1)
                {
                    strIndex = Number(secondStr.substring(strIndex).indexOf(subStr)) + Number(strIndex) + 1;
                    qusfirstStr = firstStr;
                    while (qusfirstStr.indexOf("?") != -1)
                    {
                        if (secondStr.length <= strIndex)
                            return this.getErrorStrings()[1].toString();
                        qusfirstStr = qusfirstStr.substring(0, qusfirstStr.indexOf("?")) + secondStr[strIndex] + firstStr.substring(qusfirstStr.indexOf("?") + 1);
                        strIndex++;
                    }
                }
                firstStr = qusfirstStr;
            }
            index = Number(secondStr.indexOf(firstStr)) + 1;
            if (argCount == 3) {

                var startNum = parseInt(this.getValueFromArg(args[2]));
                if (startNum == 1 && firstStr[0] == secondStr[0])
                {
                    return startNum;
                }
                if (!isNaN(startNum)) {
                    try {
                        index = Number(secondStr.indexOf(firstStr, startNum)) + 1;
                    } catch (ex) {
                        return this.getErrorStrings()[1].toString();
                    }
                }
            }
            if (index == 0) {
                return this.getErrorStrings()[1].toString();
            }

            return index.toString();
        };
        this.computeSubstitute = function (range) {
            var args = this.splitArgsPreservingQuotedCommas(range);
            if (args.length != 3 && args.length != 4) {
                return this.formulaErrorStrings[this._wrong_number_arguments];
            }

            ////strip off outside tics (if any) on the evaluated arguments
            var s1 = this._stripTics0(this.getValueFromArg(args[0]));
            var s2 = this._stripTics0(this.getValueFromArg(args[1]));
            var s3 = this._stripTics0(this.getValueFromArg(args[2]));
            if ((!this._isCellReference(args[0]) && args[0][0] != this.tic) || (!this._isCellReference(args[1]) && args[1][0] != this.tic)
                || (!this._isCellReference(args[2]) && args[2][0] != this.tic)) {
                if (this.getRethrowLibraryComputationExceptions() && this.getLibraryComputationException() != null)
                    throw this.getLibraryComputationException();
                return this.getErrorStrings()[5].toString();
            }
            if (args.length == 3) {
                if(s2=="" || s2==null)
                {
                    return s1;
                }
                s1 = s1.split(s2).join(s3);
            } else {
                var s4 = this.getValueFromArg(args[3]);
                if (!this._isCellReference(args[3]) && args[3][0] != this.tic) {
                    if (this.getRethrowLibraryComputationExceptions() && this.getLibraryComputationException() != null)
                        throw this.getLibraryComputationException();
                    return this.getErrorStrings()[5].toString();
                }
                var d = parseInt(s4)

                if (!isNaN(d)) {
                    var count = d;
                    var loc = -1;
                    while (count > 0 && (loc = s1.indexOf(s2, Number(loc) + 1)) > -1) {
                        count--;
                    }
                    if (count == 0) {
                        s1 = s1.substring(0, loc) + s3 + s1.substring(Number(loc) + Number(s2.length));
                    }
                }
            }
            return  s1;
        };
        this.computeUniChar = function (argList) {
            var args = this.splitArgsPreservingQuotedCommas(argList);
            var argCount = args.length;
            if (argCount != 1) {
                if (this.getRethrowLibraryComputationExceptions() && this.getLibraryComputationException() != null)
                    throw this.getLibraryComputationException();
                return this.formulaErrorStrings[this._wrong_number_arguments];
            }
            var text = this.getValueFromArg(args[0]);
            var code = this._parseDouble(this._stripTics0(text));
            if (isNaN(code) || code <=0) {
                if (this.getRethrowLibraryComputationExceptions() && this.getLibraryComputationException() != null)
                    throw this.getLibraryComputationException();
                return (text[0] == this.tic) ? this.getErrorStrings()[5].toString() : this.getErrorStrings()[1].toString();
            }
            var unicharString = String.fromCharCode(text);
            return unicharString.toString();
        };
        this.computeClean = function (arg) {
            var args = this.splitArgsPreservingQuotedCommas(arg);
            var result;
            if (args.length != 1) {
                return this.formulaErrorStrings[this._wrong_number_arguments].toString();
            }
            var cleanedArg = this.getValueFromArg(args[0]);
            var isLogic = true;
            if (!this._isCellReference(args[0]) && args[0][0] != this.tic) {
                if (this.getRethrowLibraryComputationExceptions() && this.getLibraryComputationException() != null)
                    throw this.getLibraryComputationException();
                return this.getErrorStrings()[5].toString();
            }
            var boolFlag = (cleanedArg == "TRUE" || cleanedArg == "FALSE") ? true : false;
            if (this.getErrorStrings().indexOf(cleanedArg) > -1)
            {
                return cleanedArg;
            }
            if (cleanedArg.indexOf(this.tic) == -1 && !this._isCellReference(arg) && isNaN(this._parseDouble(cleanedArg)) && !boolFlag) {
                return this.getErrorStrings()[5].toString();
            }
            for (var i = 0; i <= 31; i++) {
                var nonPrintChar = this.computeChar(i.toString());
                if (cleanedArg.indexOf(nonPrintChar) > -1)
                    cleanedArg = cleanedArg.replace(nonPrintChar, "");
            }
            if (cleanedArg[0] == this.tic && cleanedArg[cleanedArg.length - 1] == this.tic) {
                cleanedArg = this._substring(cleanedArg, 1, cleanedArg.length - 1);
            }
            return cleanedArg;
        };
        this.computeTrim = function (args) {
            var s = this._stripTics0(this.getValueFromArg(args));
            var arr = [this.tic, ''];
            s = s.split(this.tic).join("");
            var len = 0;
            if (!this._isCellReference(args) && args[0] != this.tic) {
                if (this.getRethrowLibraryComputationExceptions() && this.getLibraryComputationException() != null)
                    throw this.getLibraryComputationException();
                return this.getErrorStrings()[5].toString();
            }
            while (s.length != len) {
                len = s.length;
                s = s.split("  ").join(" ");
            }

            return s;
        };
        this.computeRept = function (argList) {
            var args = this.splitArgsPreservingQuotedCommas(argList);
            var argCount = args.length;
            if (argCount != 2) {
                if (this.getRethrowLibraryComputationExceptions() && this.getLibraryComputationException() != null)
                    throw this.getLibraryComputationException();
                return this.formulaErrorStrings[this._wrong_number_arguments];
            }
            var text = this._stripTics0(this.getValueFromArg(args[0]));
            if (!this._isCellReference(args[0]) && args[0][0] != this.tic) {
                if (this.getRethrowLibraryComputationExceptions() && this.getLibraryComputationException() != null)
                    throw this.getLibraryComputationException();
                return this.getErrorStrings()[5].toString();
            }
            var repeatTime = 1;
            var value = parseInt(this.getValueFromArg(args[1].split(this.tic).join("")));
            if (value == null || value == "") {
                return "";
            }
            if (value < 0 || isNaN(value)) {
                if (this.getRethrowLibraryComputationExceptions())
                    throw this.formulaErrorStrings[this._bad_index];
                return this.getErrorStrings()[1].toString();
            }
            var reptString = "";
            for (var i = 0; i < value ; i++) {
                if (reptString.length > 32767) {
                    if (this.getRethrowLibraryComputationExceptions())
                        throw this.formulaErrorStrings[this._bad_index];
                    return this.getErrorStrings()[1].toString();
                }
                reptString = reptString.concat(text);
            }

            return reptString;
        };
        this.computeProper = function (argList) {
            var args = this.splitArgsPreservingQuotedCommas(argList);
            var argCount = args.length;
            if (argCount != 1) {
                if (this.getRethrowLibraryComputationExceptions() && this.getLibraryComputationException() != null)
                    throw this.getLibraryComputationException();
                return this.formulaErrorStrings[this._wrong_number_arguments];
            }
            var textValue = this.getValueFromArg(args[0]);
            if (!this._isCellReference(args[0]) && args[0][0] != this.tic) {
                if (this.getRethrowLibraryComputationExceptions() && this.getLibraryComputationException() != null)
                    throw this.getLibraryComputationException();
                return this.getErrorStrings()[5].toString();
            }
            var text = this._stripTics0(textValue);

            var quoteIndex = text.indexOf("'");

            if (quoteIndex >= 0 && quoteIndex < text.length - 1) {
                text = text.replace(/[']/, " '");
            }
            if (/[^a - zA - Z]/.test(text)) {

                text = text.replace(/\w\S*/g, function (text) {
                    var i = 0;
                    var charCode = (text[i].toString().toUpperCase()).charCodeAt(0);
                    while (!(charCode > 64 && charCode < 91))
                    {
                        i++;
                        if (text[i] != undefined)
                            charCode = (text[i].toUpperCase()).charCodeAt(0);
                        else
                        {
                            i--;
                            charCode = 0;
                        }
                    }
                    return text.substr(0, i) + text.charAt(i).toUpperCase() + text.substr(i + 1).toLowerCase();
                });
            }
            else if (/[a - zA - Z0 - 9] + $/.test(text)) {
                text = text.replace(/[^a-zA-Z0-9_\\]/, "");
            }

            return text;

            if (text == null || text == "") {
                return "";
            }
            text = text[0].toUpperCase() + text.slice(1).split(" '").join("'");
            return text.toString();
        };
        this.computeT = function (args) {
            var elements = 0;
            var arg = this.splitArgsPreservingQuotedCommas(args);
            if (arg.length > 0) {
                for (var r in arg) {
                    elements++;
                    if (r.indexOf(':') > -1) {
                        --elements;
                        for (var s1 in this.getCellsFromArgs(r)) {
                            elements++;
                        }
                    }
                }
            }
            if (elements > 1) {
                return this.formulaErrorStrings[this._wrong_number_arguments];
            }
            var s = this.getValueFromArg(args);
            var d;
            var pattern = /[a-zA-Z0-9!#$%&'()*+,/:;<=>?@\^_`{|}~-]/;
            var flag = (args == "true" || args == "false") ? true : false;
            if (!isNaN(this._parseDouble(s)) || flag || !isNaN(this._isDate(s))) {
                return " ";
            } else if (/[a-zA-Z0-9!#$%&'()*+,/:;<=>?@\^_`{|}~-]/.test(s)) {
                s = this._stripTics0(s);
                return s;
            } else {
                return " ";
            }
        };

        this.computeNumberValue = function (argList) {
            var args = this.splitArgsPreservingQuotedCommas(argList);
            var argCount = args.length;
            var decimalseparator = this._parseDecimalSeparator;
            var groupseparator = this._parseArgumentSeparator;
            if (argCount > 3) {
                if (this.getRethrowLibraryComputationExceptions() && this.getLibraryComputationException() != null)
                    throw this.getLibraryComputationException();
                return this.formulaErrorStrings[this._wrong_number_arguments];
            }
            var text = this._stripTics0(this.getValueFromArg(args[0]));
            if (argCount > 1) {
                try {
                    var deci = (args.length > 1 && args[2] != null) ? this._stripTics0(this.getValueFromArg(args[1])) : groupseparator;
                    var gro = (args.length > 1 && args[2]!=null) ? this._stripTics0(this.getValueFromArg(args[2])) : decimalseparator;
                    if ((deci == "invalid expression" || gro == "invalid expression") || ((deci == null || deci == "") && (gro == null || gro == ""))) {
                        if (this.getRethrowLibraryComputationExceptions() && this.getLibraryComputationException() != null)
                            throw this.getLibraryComputationException();
                        return this.getErrorStrings()[1].toString();
                    } else {
                        decimalseparator = this._stripTics0(deci)[0];
                        groupseparator = this._stripTics0(gro)[0];
                    }
                } catch (ex) {
                    if (this.getRethrowLibraryComputationExceptions() && this.getLibraryComputationException() != null)
                        throw this.getLibraryComputationException();
                    return this.getErrorStrings()[1].toString();
                }
            }
            if (text == "" || text == null) {
                return "0";
            }
            var indexDecimal = text.indexOf(decimalseparator);
            var indexGroup = text.indexOf(groupseparator);
            if ((indexGroup >= indexDecimal) && (indexDecimal != -1 || indexGroup != -1) && args.length>2) {
                if (this.getRethrowLibraryComputationExceptions() && this.getLibraryComputationException() != null)
                    throw this.getLibraryComputationException();
                return this.getErrorStrings()[1].toString();
            }
            text = text.replace(this.tic, "");
            text = text.replace(" ", "");
            text = text.replace(groupseparator.toString(), "");
            text = text.replace(decimalseparator, this._parseDecimalSeparator);
            var compText = this.computeValue(text);
            if (isNaN(compText) || (text[text.length - 1] == "$")) {
                if (this.getRethrowLibraryComputationExceptions() && this.getLibraryComputationException() != null)
                    throw this.getLibraryComputationException();
                return this.getErrorStrings()[1].toString();
            }
            return compText;
        };
        this.computeConcatenate = function (range) {
            var text = "";
            var ar = this.splitArgsPreservingQuotedCommas(range);
            if ((!this._isCellReference(ar[0]) && ar[0].indexOf(this.tic) == -1) || (!this._isCellReference(ar[1]) && ar[1].indexOf(this.tic) == -1)) {
                if (this.getRethrowLibraryComputationExceptions() && this.getLibraryComputationException() != null)
                    throw this.getLibraryComputationException();
                return this.getErrorStrings()[5].toString();
            }
            for (var r in ar) {
                var toAppend = ar[r];
                text = text.concat(this._stripTics0(this.getValueFromArg(ar[r])));
            }
            if (text.indexOf("#N/A") > -1) {
                text = "#N/A";
            }
            return text;
        };
        this.computeValue = function (range) {
            var val = "";
            try {
                var args = this.splitArgsPreservingQuotedCommas(range);
                var argCount = args.length;
                if (argCount == 1) {
                    var s1 = args[0];
                    var ticContains = false;
                    var quotation = false;
                    if (s1.indexOf(this.tic) > -1)
                        ticContains = true;
                    s1 = this._stripTics0(this.getValueFromArg(s1));
                    if ((s1 != null || s1 != "") && !this._isCellReference(s1) && ticContains && isNaN(this._parseDouble(s1.split(",").join("").split("$").join("")))) {
                        s1 = s1.split(this.tic).join("");
                        quotation = true;
                    }
                    if (!this._isCellReference(args[0]) && args[0].indexOf(this.tic) == -1 && isNaN(this._parseDouble(s1))) {
                        if (this.getRethrowLibraryComputationExceptions() && this.getLibraryComputationException() != null)
                            throw this.getLibraryComputationException();
                        return this.getErrorStrings()[5].toString();
                    }
                    if ((s1 == null || s1 == "") && ticContains) {
                        if (this.getRethrowLibraryComputationExceptions() && this.getLibraryComputationException() != null)
                            throw this.getLibraryComputationException();
                        return this.getErrorStrings()[1].toString();
                    } else if ((s1 == null || s1 == "") && !ticContains) {
                        if (this.getRethrowLibraryComputationExceptions() && this.getLibraryComputationException() != null)
                            throw this.getLibraryComputationException();
                        return this.formulaErrorStrings[this._invalid_arguments];
                    }
                    if (s1.indexOf(":") > -1)
                    {
                        val = this.computeTimevalue(s1);
                        return val.toString();
                    }
                    if (s1[0] == "$") {
                        s1 = s1.split("$").join("");
                    }
                    if (s1.indexOf(",") > -1) {
                        s1 = s1.split(",").join("");
                    }
                    if (s1[0] == "%" || s1[s1.length - 1] == "%") {
                        s1 = s1.split("%").join("");
                        var per = this._parseDouble(s1);
                        if (!isNaN(per)) {
                            s1 = (per / 100).toString();
                        }
                    }
                    var dt = new Date(Date.parse(s1));
                    var dous = this._parseDouble(s1);
                    if (!isNaN(dous)) {
                        val = dous.toString();
                    } else if (dt.toString() != "Invalid Date") {
                        val = this._toOADate(dt);
                    } else {
                        if (this.getRethrowLibraryComputationExceptions() && this.getLibraryComputationException() != null)
                            throw this.getLibraryComputationException();
                        return this.getErrorStrings()[1].toString();
                    }
                } else {
                    if (this.getRethrowLibraryComputationExceptions() && this.getLibraryComputationException() != null)
                        throw this.getLibraryComputationException();
                    return this.formulaErrorStrings[this._wrong_number_arguments];
                }
            } catch (ex) {
                if (this.getRethrowLibraryComputationExceptions() && this.getLibraryComputationException() != null)
                    throw this.getLibraryComputationException();
                return this.getErrorStrings()[1].toString();
            }

            return val.toString();
        };
        this.computeDollar = function (args) {
            var argsArray = this.splitArgsPreservingQuotedCommas(args);
            var s1 = argsArray[0];
            var s2 = "2";
            if (argsArray.length == 2) {
                s2 = argsArray[1];
            }
            if (argsArray.length > 2) {
                if (this.getRethrowLibraryComputationExceptions() && this.getLibraryComputationException() != null)
                    throw this.getLibraryComputationException();
                return this.formulaErrorStrings[this._wrong_number_arguments];
            }           
            s1 = this.getValueFromArg(s1);
            s2 = this.getValueFromArg(s2);
            var number = this._parseDouble(s1);
            var emptyCell = false;
            if ((!this._isCellReference(argsArray[0]) && argsArray[0].indexOf(this.tic) == -1) || (!this._isCellReference(argsArray[1]) && argsArray[1].indexOf(this.tic) == -1 && isNaN(this._parseDouble(s2)))) {
                if (this.getRethrowLibraryComputationExceptions() && this.getLibraryComputationException() != null)
                    throw this.getLibraryComputationException();
                return this.getErrorStrings()[5].toString();
            }
            if ((s1 == null || s1 == "") && this._isCellReference(argsArray[0]) && (s1[0] != this.tic && s1[s1.length - 1] != this.tic))
            {
                number=0;
                emptyCell=true;
            }            
            if (isNaN(number) && !emptyCell) {
                if (this.getRethrowLibraryComputationExceptions() && this.getLibraryComputationException() != null)
                    throw this.getLibraryComputationException();
                return this.getErrorStrings()[1].toString();
            }
            var decimals = this._parseDouble(s2);
            var round = 0.0;
            if (isNaN(decimals)) {
                decimals = (s2 == "") ? 0 : 2;
            }
            if (decimals > 0) {
                return Globalize.format(number, "c" + decimals);
            }
            else {
                var mult = Math.pow(10, -decimals);
                round = Math.round(number / mult) * mult;
                return Globalize.format(round, "c0");
            }
        };
        this.computeFixed = function (args) {
            var argsArray = this.splitArgsPreservingQuotedCommas(args);
            var s1 = argsArray[0];
            var s2 = "2";
            var s3 = "FALSE";

            var argCount = argsArray.length;
            if (argCount > 1) {
                s2 = argsArray[1];
            }

            if (argCount > 2) {
                s3 = argsArray[2];
            }

            s1 = this.getValueFromArg(s1);
            s2 = this.getValueFromArg(s2);
            s3 = this.getValueFromArg(s3);

            var numbers, decimals;
            numbers = this._parseDouble(s1);
            if (isNaN(numbers)) {
                if (this.getRethrowLibraryComputationExceptions() && this.getLibraryComputationException() != null)
                    throw this.getLibraryComputationException();
                return this.getErrorStrings()[1].toString();
            }
            decimals = this._parseDouble(s2);
            if (isNaN(decimals)) {
                decimals = (s2 == "") ? 0 : 2;
            }
            var no_commas_flag;
            var no_commas;
            no_commas_flag = this._parseDouble(s3);
            if (!isNaN(no_commas_flag)) {
                if (no_commas_flag == 0) {
                    no_commas = false;
                } else {
                    no_commas = true;
                }
            } else {
                if (s3.toUpperCase() == this.falseValueStr) {
                    no_commas = false;
                } else {
                    if (s3.toUpperCase() == this.trueValueStr) {
                        no_commas = true;
                    } else {
                        return "#NAME?";
                    }
                }
            }
            if (no_commas)
            {
                Globalize.culture().numberFormat[","] = "";
            }
            if (decimals > 0) {
                return Globalize.format(numbers, "n" + decimals)
            } else {
                var mult = Math.pow(10, -decimals);
                numbers = Math.round(numbers / mult) * mult;
                return Globalize.format(numbers, "n0");
            }
        };
        this.computeBin2Dec = function (argList) {
            var ranges = this.splitArgsPreservingQuotedCommas(argList);
            if (ranges.length > 1) {
                if (this.getRethrowLibraryComputationExceptions() && this.getLibraryComputationException() != null)
                    throw this.getLibraryComputationException();
                else
                    return this.formulaErrorStrings[this._wrong_number_arguments];
            }
            var value = this.getValueFromArg(ranges[0]).replace(this.tic, "");
            if (!/^[01]{1,10}$/.test(value))
                return this.getErrorStrings()[4].toString();
            try {
                if (value == "")
                    value = "0";

                // Convert binary number to decimal
                var result = parseInt(value, 2);

                // Handle negative numbers
                if (value.length == 10 && value.substring(0, 1) == "1") {
                    return (parseInt(value.substring(1), 2) - 512).toString();
                } else {
                    return result.toString();
                }
            } catch (Exception) {
                if (this.getRethrowLibraryComputationExceptions() && this.getLibraryComputationException() != null)
                    throw this.getLibraryComputationException();
                return this.getErrorStrings()[1].toString();
            }
        };
        this.computeBin2Oct = function (argList) {
            var charCount = 0;
            var ranges = this.splitArgsPreservingQuotedCommas(argList);
            var number = this.getValueFromArg(ranges[0]).replace(this.tic, "");
            var result = "";
            // Return error if number is not binary or contains more than 10 characters (10 digits)
            if (!/^[01]{1,10}$/.test(number))
                return this.getErrorStrings()[4].toString();

            if (ranges.length > 2) {
                if (this.getRethrowLibraryComputationExceptions() && this.getLibraryComputationException() != null)
                    throw this.getLibraryComputationException();
                else
                    return this.formulaErrorStrings[this._wrong_number_arguments];
            } else if (ranges.length > 1) {
                places = parseInt(this.getValueFromArg(ranges[1]).replace(this.tic, ""));
            }

            //// Ignore places and return a 10-character octal number if number is negative
            //if (value.length == 10 && value.substring(0, 1) == "1") {
            //    var i = 10;
            //    var s = "";
            //    for (var j = 0; j < 3; j++) {
            //        i = i - 3;
            //        var rmv = value.substring(i, 3);
            //        var valResult = this._parseDouble(parseInt(rmv, 2).toString(8));
            //        var sum = 0;
            //        for (var n = valResult; n > 0; sum += n % 10, n /= 10)
            //            ;
            //        s = sum.toString() + s;
            //    }
            //    return "7777777" + s;
            //}

            //try {
            //    if (value.length > 10) {
            //        if (this.getRethrowLibraryComputationExceptions() && this.getLibraryComputationException() != null)
            //            throw this.getLibraryComputationException();
            //        else
            //            return this.getErrorStrings()[4].toString();
            //    } else {
            //        result = parseInt(value, 2).toString(8);
            //        if (ranges.length > 1) {
            //            if (charCount >= result.length && charCount <= 10) {
            //                result = this._padLeft('0', charCount, result);
            //            } else {
            //                if (this.getRethrowLibraryComputationExceptions() && this.getLibraryComputationException() != null)
            //                    throw this.getLibraryComputationException();
            //                else
            //                    result = this.getErrorStrings()[4].toString();
            //            }
            //        }
            //    }
            //} catch (Exception) {
            //    if (this.getRethrowLibraryComputationExceptions() && this.getLibraryComputationException() != null)
            //        throw this.getLibraryComputationException();
            //            result = this.getErrorStrings()[5].toString();
            //    }

            // Ignore places and return a 10-character octal number if number is negative
            if (number < 0) {
                return (1073741824 + number).toString(8);
            }

            // Convert decimal number to octal
            var result = parseInt(number, 10).toString(8);

            // Return octal number using the minimum number of characters necessary if places is undefined
            if (typeof places === 'undefined') {
                return result;
            } else {
                // Return error if places is nonnumeric
                if (isNaN(places)) return '#VALUE!';

                // Return error if places is negative
                if (places < 0) return '#NUM!';

                // Truncate places in case it is not an integer
                places = Math.floor(places);

                // Pad return value with leading 0s (zeros) if necessary (using Underscore.string)
                return (places >= result.length) ? _s.repeat('0', places - result.length) + result : '#NUM!';
            }
            //return result;
        };
        this.computeBin2Hex = function (argList) {
            var charCount = 0;
            var ranges = this.splitArgsPreservingQuotedCommas(argList);
            var value = this.getValueFromArg(ranges[0]).replace(this.tic, "");
            var result = "";

            if (ranges.length > 2) {
                if (this.getRethrowLibraryComputationExceptions() && this.getLibraryComputationException() != null)
                    throw this.getLibraryComputationException();
                else
                    return this.formulaErrorStrings[this._wrong_number_arguments];
            } else if (ranges.length > 1) {
                charCount = this._parseDouble(ranges[1]);
            }

            if (value == "1111111111") {
                return value.replace('1', 'F');
            }

            // Ignore places and return a 10-character octal number if number is negative
            if (value.length == 10 && value.substring(0, 1) == "1") {
                return (1073741312 + parseInt(value, 2).toString(8)).toString();
            }

            try {
                if (value.length > 10) {
                    if (this.getRethrowLibraryComputationExceptions() && this.getLibraryComputationException() != null)
                        throw this.getLibraryComputationException();
                    else
                        return this.getErrorStrings()[4].toString();
                } else {
                    result = parseInt(value, 2).toString(16);
                    if (ranges.length > 1) {
                        if (charCount >= result.length && charCount <= 10) {
                            result = this._padLeft('0', charCount, result);
                        } else {
                            if (this.getRethrowLibraryComputationExceptions() && this.getLibraryComputationException() != null)
                                throw this.getLibraryComputationException();
                            else
                                result = this.getErrorStrings()[4].toString();
                        }
                    }
                }
            } catch (Exception) {
                if (this.getRethrowLibraryComputationExceptions() && this.getLibraryComputationException() != null)
                    throw this.getLibraryComputationException();
                else {
                    if (this.computeIsText(value) == this.trueValueStr)
                        result = this.getErrorStrings()[5].toString();
                    else if (value == "")
                        return "0";
                    else
                        return this.getErrorStrings()[4].toString();
                }
            }

            return result.toUpperCase();
        };
        this.computeDec2Bin = function (argList) {
            var charCount = 0;
            var ranges = this.splitArgsPreservingQuotedCommas(argList);
            var value = this.getValueFromArg(ranges[0]).replace(this.tic, "");
            var range1 = (ranges.length > 1) ? this.getValueFromArg(ranges[1]).replace(this.tic, "") : "10";
            var result = "";
            if (argList == null || argList == "" || ranges.length > 2 || ranges.length < 1) {
                if (this.getRethrowLibraryComputationExceptions() && this.getLibraryComputationException() != null)
                    throw this.getLibraryComputationException();
                return this.formulaErrorStrings[this._wrong_number_arguments];
            }

            // double d1, d2;
            if (isNaN(this._parseDouble(value)) && isNaN(this._parseDouble(range1))) {
                if (((value != null || value != "") && (this.computeIsText(value) == this.trueValueStr)) || ((range1 != null || range1 != "") && (this.computeIsText(range1) == this.trueValueStr))) {
                    if (this.getRethrowLibraryComputationExceptions() && this.getLibraryComputationException() != null)
                        throw this.getLibraryComputationException();
                    return this.getErrorStrings()[1].toString();
                }
                if (((value != null && value != "") && (this.computeIsText(value) == this.falseValueStr)) || ((value != null && value != "") && (this.computeIsText(range1) == this.falseValueStr))) {
                    if (this.getRethrowLibraryComputationExceptions() && this.getLibraryComputationException() != null)
                        throw this.getLibraryComputationException();
                    return this.getErrorStrings()[1].toString();
                }
                if ((value == null && value == "") || (range1 == null || range1 == "")) {
                    if (this.getRethrowLibraryComputationExceptions() && this.getLibraryComputationException() != null)
                        throw this.getLibraryComputationException();
                    return this.getErrorStrings()[1].toString();
                }
            }
            try {
                if (ranges.length > 1) {
                    var value1 = this.getValueFromArg(ranges[1]).replace(this.tic, "");
                    charCount = this._parseDouble(value1);
                }

                if ((this._parseDouble(value) < -512 || this._parseDouble(value) > 511)) {
                    if (this.getRethrowLibraryComputationExceptions() && this.getLibraryComputationException() != null)
                        throw this.getLibraryComputationException();
                    else
                        return this.getErrorStrings()[4].toString();
                } else {
                    result = parseInt(value, 10).toString(2); //BaseToBase(value, 10, 2);
                    if (!isNaN(this._parseDouble(value)) && this._parseDouble(value) < 0) {
                        result = result.substring(result.length - 10);
                    }
                    if (ranges.length > 1) {
                        if (charCount > 10) {
                            if (this.getRethrowLibraryComputationExceptions() && this.getLibraryComputationException() != null)
                                throw this.getLibraryComputationException();
                            return this.getErrorStrings()[4].toString();
                        }
                        if (charCount <= 10)
                            result = this._padLeft('0', charCount, result);
                    }
                }
            } catch (Exception) {
                if (this.getRethrowLibraryComputationExceptions() && this.getLibraryComputationException() != null)
                    throw this.getLibraryComputationException();
            }
            return result;
        };
       
        this.computeDec2Oct = function (argList) {
            var charCount = 0;
            var ranges = this.splitArgsPreservingQuotedCommas(argList);
            var value = this.getValueFromArg(ranges[0]).replace(this.tic, "");
            var range1 = (ranges.length > 1) ? this.getValueFromArg(ranges[1]).replace(this.tic, "") : "10";
            var result = "";
            if (argList == null || argList == "" || ranges.length > 2 || ranges.length < 1) {
                if (this.getRethrowLibraryComputationExceptions() && this.getLibraryComputationException() != null)
                    throw this.getLibraryComputationException();
                return this.formulaErrorStrings[this._wrong_number_arguments];
            }
            if (isNaN(this._parseDouble(value)) && isNaN(this._parseDouble(range1))) {
                if (((value != null || value != "") && (this.computeIsText(value) == this.trueValueStr)) || ((range1 != null || range1 != "") && (this.computeIsText(range1) == this.trueValueStr))) {
                    if (this.getRethrowLibraryComputationExceptions() && this.getLibraryComputationException() != null)
                        throw this.getLibraryComputationException();
                    return this.getErrorStrings()[1].toString();
                }

                if (this.getErrorStrings().indexOf(value) > -1)
                    return value;
                if (this.getErrorStrings().indexOf(range1) > -1)
                    return range1;

                if (((value != null && value != "") && (this.computeIsText(value) == this.falseValueStr)) || ((value != null && value != "") && (this.computeIsText(range1) == this.falseValueStr))) {
                    if (this.getRethrowLibraryComputationExceptions() && this.getLibraryComputationException() != null)
                        throw this.getLibraryComputationException();
                    return this.getErrorStrings()[1].toString();
                }
                if ((value == null && value == "") || (range1 == null || range1 == "")) {
                    if (this.getRethrowLibraryComputationExceptions() && this.getLibraryComputationException() != null)
                        throw this.getLibraryComputationException();
                    return this.getErrorStrings()[1].toString();
                }
            }
            try {
                if (ranges.length > 1) {
                    var value1 = this.getValueFromArg(ranges[1]).replace(this.tic, "");
                    charCount = this._parseDouble(value1);
                }

                if ((this._parseDouble(value) < -536870912 || this._parseDouble(value) > 536870911)) {
                    if (this.getRethrowLibraryComputationExceptions() && this.getLibraryComputationException() != null)
                        throw this.getLibraryComputationException();
                    else
                        return this.getErrorStrings()[4].toString();
                } else {
                    result = parseInt(value, 10).toString(8);
                    if (!isNaN(this._parseDouble(value)) && this._parseDouble(value) < 0) {
                        result = result.substring(result.length - 10);
                    }
                    if (ranges.length > 1) {
                        if (charCount > 10) {
                            if (this.getRethrowLibraryComputationExceptions() && this.getLibraryComputationException() != null)
                                throw this.getLibraryComputationException();
                            return this.getErrorStrings()[4].toString();
                        }
                        if (charCount <= 10)
                            result = this._padLeft('0', charCount, result);
                    }
                }
            } catch (Exception) {
                if (this.getRethrowLibraryComputationExceptions() && this.getLibraryComputationException() != null)
                    throw this.getLibraryComputationException();
            }
            return result;
        };
        this.computeHex2Bin = function (argList) {
            var charCount = 0;
            var ranges = this.splitArgsPreservingQuotedCommas(argList);
            if (argList == this._parseArgumentSeparator.toString()) {
                if (this.getRethrowLibraryComputationExceptions() && this.getLibraryComputationException() != null)
                    throw this.getLibraryComputationException();
                else
                    return this.getErrorStrings()[0].toString();
            }
            var value = this.getValueFromArg(ranges[0]);
            var chars = (ranges.length == 2) ? this.getValueFromArg(ranges[1]) : "0";

            if (value != "") {
                if (value[0] != this.tic && !this._isCellReference(value) && isNaN(this._parseDouble(value))) {
                    if (this.getRethrowLibraryComputationExceptions() && this.getLibraryComputationException() != null)
                        throw this.getLibraryComputationException();
                    else
                        return this.getErrorStrings()[5].toString();
                }
            } else if (this._isCellReference(ranges[0]) && value == "") {
                return "0";
            }
            if ((value[0] == this.tic) && (value.split(this.tic).join("") == null || value.split(this.tic).join("") == "")) {
                return "0";
            }
            if (chars[0] == this.tic) {
                if (this.getRethrowLibraryComputationExceptions() && this.getLibraryComputationException() != null)
                    throw this.getLibraryComputationException();
                else
                    return this.getErrorStrings()[1].toString();
            } else if (isNaN(this._parseDouble(chars.split(this.tic).join("")))) {
                if (this.getRethrowLibraryComputationExceptions() && this.getLibraryComputationException() != null)
                    throw this.getLibraryComputationException();
                else
                    return this.getErrorStrings()[5].toString();
            }
            charCount = this._parseDouble(chars);
            if (charCount < 0 || value[0] == "-") {
                if (this.getRethrowLibraryComputationExceptions() && this.getLibraryComputationException() != null)
                    throw this.getLibraryComputationException();
                else
                    return this.getErrorStrings()[4].toString();
            }
            var result = "";
            value = value.split(this.tic).join("");
            try {
                for (var i = 0; i < value.length; i++) {
                    // Convert to decimal
                    var output = this._padLeft('0', 4, parseInt(value.charAt(i), 16).toString(2));
                    result += output;
                }
                if (value.split(this.tic).join("") == "FFFFFFFFFF") {
                    return this._parseDouble(value.split('F').join('1'));
                }

                if (result.length > 10 || charCount > 10 || (charCount < result.length && charCount != 0)) {
                    if (this.getRethrowLibraryComputationExceptions() && this.getLibraryComputationException() != null)
                        throw this.getLibraryComputationException();
                    else
                        return this.getErrorStrings()[4].toString();
                }

                if (charCount >= result.length)
                    result = this._padLeft('0', charCount, result);
                else if (charCount != 0) {
                    if (this.getRethrowLibraryComputationExceptions() && this.getLibraryComputationException() != null)
                        throw this.getLibraryComputationException();
                    else
                        return this.getErrorStrings()[4].toString();
                }

                if (!isNaN(this._parseDouble(result)) && (this._parseDouble(result) > 536870911.0)) {
                    if (this.getRethrowLibraryComputationExceptions() && this.getLibraryComputationException() != null)
                        throw this.getLibraryComputationException();
                    else
                        return this.getErrorStrings()[4].toString();
                }
            } catch (ex) {
                if (this.getRethrowLibraryComputationExceptions() && this.getLibraryComputationException() != null)
                    throw this.getLibraryComputationException();
                else
                    return this.getErrorStrings()[4].toString();
            }

            return result;
        };
        
        this.computeHex2Oct = function (argList) {
            var charCount = 0;
            var ranges = this.splitArgsPreservingQuotedCommas(argList);
            if (argList == this._parseArgumentSeparator.toString()) {
                if (this.getRethrowLibraryComputationExceptions() && this.getLibraryComputationException() != null)
                    throw this.getLibraryComputationException();
                else
                    return this.getErrorStrings()[0].toString();
            }
            var value = this.getValueFromArg(ranges[0]);
            ranges[0] = ranges[0].split(this.TIC).join("");
            var chars = (ranges.length == 2) ? this.getValueFromArg(ranges[1]) : "0";
            if ((value[0] == this.tic) && (value.split(this.tic).join("") == null || value.split(this.tic).join("") == "")) {
                return "0";
            } else if (value.indexOf(this._parseArgumentSeparator.toString()) > -1) {
                var len = value.indexOf(this._parseArgumentSeparator);
                value = value.split(this.tic).join("").substring(0, len - 1);
            }
            var result = "";
            if (isNaN(this._parseDouble(chars.split(this.tic).join("")))) {
                if (this.getRethrowLibraryComputationExceptions() && this.getLibraryComputationException() != null)
                    throw this.getLibraryComputationException();
                else
                    return this.getErrorStrings()[5].toString();
            }
            charCount = this._parseDouble(chars);
            if (charCount < 0 || value[0] == "-") {
                if (this.getRethrowLibraryComputationExceptions() && this.getLibraryComputationException() != null)
                    throw this.getLibraryComputationException();
                else
                    return this.getErrorStrings()[4].toString();
            }
            if (ranges.length > 2) {
                if (this.getRethrowLibraryComputationExceptions() && this.getLibraryComputationException() != null)
                    throw this.getLibraryComputationException();
                else
                    return this.getErrorStrings()[4].toString();
            }

            if (ranges.length > 1)
                charCount = this._parseDouble(ranges[1]);
            try {
                value = value.split(this.tic).join("");
                result = parseInt(value, 16).toString(8);
                if (!isNaN(this._parseDouble(result)) && result.length > 10) {
                    result = result.substring(result.length - 10);
                }
                if (ranges.length > 1) {
                    if (charCount >= result.length && charCount <= 10)
                        result = this._padLeft('0', charCount, result);
                    else {
                        if (this.getRethrowLibraryComputationExceptions() && this.getLibraryComputationException() != null)
                            throw this.getLibraryComputationException();
                        else
                            return this.getErrorStrings()[4].toString();
                    }
                    if (!isNaN(this._parseDouble(result)) && (this._parseDouble(result) > 536870911.0)) {
                        if (this.getRethrowLibraryComputationExceptions() && this.getLibraryComputationException() != null)
                            throw this.getLibraryComputationException();
                        else
                            return this.getErrorStrings()[4].toString();
                    }
                }
            } catch (Exception) {
                if (this.getRethrowLibraryComputationExceptions() && this.getLibraryComputationException() != null)
                    throw this.getLibraryComputationException();
                else {
                    if (this.computeIsText(value) == this.trueValueStr)
                        return this.getErrorStrings()[4].toString();
                    else if (value == "")
                        return "0";
                    else
                        return this.getErrorStrings()[4].toString();
                }
            }
            return result;
        };
        this._padLeft = function (paddingChar, length, stringpad) {
            var strlen = stringpad.length;
            if ((stringpad.length < length) && (paddingChar.toString().length > 0)) {
                for (var i = 0; i < (length - strlen) ; i++)
                    stringpad = paddingChar.toString().charAt(0).concat(stringpad);
            }
            return stringpad;
        };


        this.computeAddress = function (argList) {
            var args = this.splitArgsPreservingQuotedCommas(argList);
            var argCount = args.length;
            var ans = this._string_empty;
            var arg0 = this.getValueFromArg(args[0].split(this.tic).join(this._string_empty));
            var arg1 = this.getValueFromArg(args[1].split(this.tic).join(this._string_empty));
            var row = this._parseDouble(arg0);
            var col = this._parseDouble(arg1);
            if (row < 1 || col < 1) {
                if (this._rethrowLibraryComputationExceptions)
                    throw this.getErrorStrings()[1].toString();
                return this.getErrorStrings()[1].toString();
            }
            var absNum = (argCount > 2) ? this.getValueFromArg(args[2].split(this.tic).join(this._string_empty)) : "1";
            var temp = col;
            var rem = temp % 26;
            temp = parseInt(temp / 26);
            ans = ans + this._charTable[rem];
            ans = ans + this._charTable[parseInt(temp)];
            var arr = (ans.split('').join(',')).split(this.getParseArgumentSeparator());
            arr.reverse();
            var ans1 = arr;
            ans = '$' + ans1 + '$' + arg0;
            if (argCount > 2) {
                switch (absNum) {
                    case "":
                        break;
                    case "1":
                        break;
                    case "2":
                        ans = ans1 + '$' + arg0;
                        break;
                    case "3":
                        ans = '$' + ans1 + arg0;
                        break;
                    case "4":
                        ans = ans1 + arg0;
                        break;
                }
            }
            if (argCount > 3 && (this.getValueFromArg(args[3].split(this.tic).join(this._string_empty)) == "FALSE" || this.getValueFromArg(args[2].split(this.tic).join(this._string_empty)) == "0")) {
                switch (absNum) {
                    case "":
                        ans = "R" + row + "C" + col;
                        break;
                    case "1":
                        ans = "R" + row + "C" + col;
                        break;
                    case "2":
                        ans = "R" + row + "C[" + col + "]";
                        break;
                    case "3":
                        ans = "R[" + row + "]C" + col;
                        break;
                    case "4":
                        ans = "R[" + row + "]C[" + col + "]";
                        break;
                }
            }
            if ((argCount > 4 && this.getValueFromArg(args[3].split(this.tic).join(this._string_empty)) == this._string_empty)) {
                args[4] = this.getValueFromArg(args[4].split(this.tic).join(this._string_empty));
                ans = args[4] + "!" + ans;
            }
            if (argCount > 4) {

                args[4] = getValueFromArg(args[4]);
                ans = args[4] + "!" + ans;
                ans = (args[4].indexOf(" ") > -1) ? ans.split(this.tic).join("'") : ans.split(this.tic).join("");
            }
            return ans;
        };
        this.computeAreas = function (arg) {
            if (arg.indexOf(" ") > 0) {
                arg = arg.substring(0, arg.indexOf(" "));
            }
            if (!this._isCellReference(arg) && !this.getNamedRanges().containsKey(arg) && !arg.indexOf(")")) {
                if (this._rethrowLibraryComputationExceptions)
                    throw this.formulaErrorStrings[this._invalid_arguments];
                return this.formulaErrorStrings[this._invalid_arguments];
            }
            var args1;
            args1 = this._splitArguments(arg, ')');
            if (args1.length > 2) {
                if (this._rethrowLibraryComputationExceptions)
                    throw this.formulaErrorStrings[this._wrong_number_arguments];
                return this.getErrorStrings()[1].toString();
            }
            var argument = args1[0].split("(").join(this._string_empty);
            var args;
            args = this.splitArgsPreservingQuotedCommas(argument);
            var argCount = args.length;
            for (var i = 0; i < args.length; i++) {
                if (!this._isCellReference(args[i])) {
                    if (this._rethrowLibraryComputationExceptions)
                        throw this.formulaErrorStrings[this._invalid_arguments];
                    return this.getErrorStrings()[1].toString();
                }
            }
            return args.length.toString();
        };
        this.computeChoose = function (arg) {
            var args;
            args = this.splitArgsPreservingQuotedCommas(arg);
            if (args.length < 2) {
                return this.formulaErrorStrings[this._wrong_number_arguments];
            }
            var s3 = this.getValueFromArg(args[0]).split(this.tic).join("");
            var indexArgs = this.splitArgsPreservingQuotedCommas(s3);
            s3 = indexArgs[0];
            var loc = parseInt(s3);
            if (isNaN(loc) || loc > args.length - 1 || loc < 1) {
                return "#VALUE!";
            }
            if (this._isInteriorFunction)
            {
                this._isInteriorFunction = !this._isInteriorFunction;
                return args[loc];
            }
            var value = this.getValueFromArg(args[loc]);
            if (value == "")
            {
                if (this.getRethrowLibraryComputationExceptions())
                    throw this.formulaErrorStrings[this._invalid_Math_argument];
                return this.getErrorStrings()[5].toString();
            }
            return value;
        };
        this.computeColumn = function (arg) {
            var rowInd, colInd;
            if (arg == null || arg == this._string_empty) {
                colInd = this.colIndex(this.cell).toString();
                rowInd = this.rowIndex(this.cell).toString();
            }

            colInd =  this.colIndex(arg).toString();
            rowInd = this.rowIndex(arg).toString();
            if (rowInd <= 0)
            {
                if (this.getRethrowLibraryComputationExceptions())
                    throw this.formulaErrorStrings[this._invalid_Math_argument];
                return this.getErrorStrings()[5].toString();
            }
            return colInd;
        };
        this.computeColumns = function (arg) {
            var args;
            args = this.splitArgsPreservingQuotedCommas(arg);
            if (args.length != 1) {
                if (this._rethrowLibraryComputationExceptions)
                    throw this.formulaErrorStrings[this._wrong_number_arguments];
                return this.formulaErrorStrings[this._wrong_number_arguments];
            }
            var val = -1;
            if (this._parseDouble(args[0], val)) {
                return "1";
            } else if ((args[0].indexOf(this.tic) && !args[0].indexOf(";") && !args[0].indexOf(this.getParseArgumentSeparator().toString())) || (!this._isCellReference(args[0]) && !this.getNamedRanges().containsValue(args[0]) && !arg.indexOf(";") && !arg.indexOf(this.getParseArgumentSeparator().toString()))) {
                if (this._rethrowLibraryComputationExceptions)
                    throw this.getErrorStrings()[1].toString();
                return this.getErrorStrings()[1].toString();
            }

            var firstCol, LastCol, totalCols = 1;
            if (arg.indexOf(':') > -1) {
                var cells;
                cells = this.getCellsFromArgs(arg);
                firstCol = this.colIndex(cells[0].toString());
                LastCol = this.colIndex(cells[cells.length - 1].toString());
                totalCols = LastCol - firstCol + 1;
                totalCols = (totalCols > 0) ? totalCols : 1;
            } else if (arg.indexOf(';') > -1) {
                var arrayArg;
                arrayArg = this._splitArguments(arg.split(this.tic).join(this._string_empty), ';');
                for (var i = 1; i < arrayArg.length; i++) {
                    if (this.splitArgsPreservingQuotedCommas(arrayArg[i - 1]).length != this.splitArgsPreservingQuotedCommas(arrayArg[1]).length) {
                        if (this._rethrowLibraryComputationExceptions)
                            throw this.formulaErrorStrings[this._wrong_number_arguments];
                        return this.formulaErrorStrings[this._wrong_number_arguments];
                    }
                    totalCols = this.splitArgsPreservingQuotedCommas(arrayArg[0]).length;
                }
            } else {
                var arrayArg;
                arrayArg = this.splitArgsPreservingQuotedCommas(args[0].split(this.tic).join(this._string_empty));
                totalCols = arrayArg.length;
            }
            return totalCols.toString();
        };
        this.computeFormulaText = function (argList) {
            var args = this.splitArgsPreservingQuotedCommas(argList);
            var argCount = args.length;
            var value = this._string_empty;
            if (argCount != 1) {
                if (this._rethrowLibraryComputationExceptions)
                    throw this.formulaErrorStrings[this._wrong_number_arguments];
                return this.formulaErrorStrings[this._wrong_number_arguments];
            }
            value = this.getValueFromArg(args[0]);
            if (this.getNamedRanges().containsValue(value)) {
                if (this._rethrowLibraryComputationExceptions)
                    throw this.formulaErrorStrings[this._bad_index];
                return this.getErrorStrings()[1].toString();
            }
            var family = CalcEngine.getSheetFamilyItem(this.grid);
            if (args[0].indexOf(this.sheetToken) == -1 && this.isSheetMember() && family.parentObjectToToken != null) {
                args[0] = family.parentObjectToToken.getItem(this.grid) + args[0];
            }
            if (this.getFormulaInfoTable().containsKey(args[0])) {
                var formula = this.getFormulaInfoTable().getItem(args[0]);
                value = formula.getFormulaText();
                this.getFormulaText(value);
            } else {
                if (this._rethrowLibraryComputationExceptions) {
                    throw this.formulaErrorStrings[this._bad_formula];
                }
                return (this.rowIndex(args[0]) <= 0) ? this.getErrorStrings()[5].toString() : this.getErrorStrings()[0].toString();
            }

            return value;
        };
        this.computeHyperlink = function (argList) {
            var args = this.splitArgsPreservingQuotedCommas(argList);
            var argCount = args.length;
            if (argCount > 2) {
                if (this._rethrowLibraryComputationExceptions)
                    throw this.formulaErrorStrings[this._wrong_number_arguments];
                return this.formulaErrorStrings[this._wrong_number_arguments];
            }
            var firstStr = this._stripTics0(this.getValueFromArg(args[0]));
            if (argCount == 2) {
                var displayText = this._stripTics0(this.getValueFromArg(args[1]));

                if (displayText == null) {
                    return "0";
                }
                if (displayText == this._string_empty) {
                    return this._string_empty;
                }
                return displayText;
            }
            return firstStr;
        };
        this.computeAbs = function (argList) {
            var args = this.splitArgsPreservingQuotedCommas(argList);
            var argCount = args.length;
            if (argCount != 1) {
                if (this._rethrowLibraryComputationExceptions)
                    throw this.formulaErrorStrings[this._wrong_number_arguments];
                return this.formulaErrorStrings[this._wrong_number_arguments];
            }
            var argValue = this.getValueFromArg(args[0]);
            argValue = (argValue.split(this.tic).join("") == "TRUE") ? "1" : (argValue.split(this.tic).join("") == "FALSE") ? "0" : argValue;
            var dVal = this._parseDouble(argValue.split(this.tic).join(""));
            if (isNaN(dVal)) {
                if (this._rethrowLibraryComputationExceptions)
                    throw this.formulaErrorStrings[this._bad_formula];
                return (argValue[0] == this.tic || this._isCellReference(args[0])) ? this.getErrorStrings()[1].toString() : this.getErrorStrings()[5].toString();
            }
            return Math.abs(dVal).toString();
        };
        this.computeAcos = function (argList) {
            var args = this.splitArgsPreservingQuotedCommas(argList);
            var argCount = args.length;
            if (argCount != 1) {
                if (this._rethrowLibraryComputationExceptions)
                    throw this.formulaErrorStrings[this._wrong_number_arguments];
                return this.formulaErrorStrings[this._wrong_number_arguments];
            }
            var argValue = this.getValueFromArg(args[0]);
            var dVal = this._parseDouble(argValue);
            if (isNaN(dVal)) {
                if (this._rethrowLibraryComputationExceptions)
                    throw this.formulaErrorStrings[this._bad_formula];
                return (argValue[0] == this.tic || this._isCellReference(args[0])) ? this.getErrorStrings()[1].toString() : this.getErrorStrings()[5].toString();
            }
            if (dVal > 1 || dVal < -1) {
                if (this._rethrowLibraryComputationExceptions)
                    throw this.formulaErrorStrings[this._bad_formula];
                return this.getErrorStrings()[4].toString();
            }
            return Math.acos(dVal).toString();
        };
        this.computeAcosh = function (argList) {
            var args = this.splitArgsPreservingQuotedCommas(argList);
            var argCount = args.length;
            if (argCount != 1) {
                if (this._rethrowLibraryComputationExceptions)
                    throw this.formulaErrorStrings[this._wrong_number_arguments];
                return this.formulaErrorStrings[this._wrong_number_arguments];
            }
            var argValue = this.getValueFromArg(args[0]);
            var dVal = this._parseDouble(argValue);
            if (isNaN(dVal)) {
                if (this._rethrowLibraryComputationExceptions)
                    throw this.formulaErrorStrings[this._bad_formula];
                return (argValue[0] == this.tic || this._isCellReference(args[0])) ? this.getErrorStrings()[1].toString() : this.getErrorStrings()[5].toString();
            }
            z = Math.log(dVal + Math.sqrt(dVal * dVal - 1));
            if (dVal <= 0 || z == Infinity) {
                if (this._rethrowLibraryComputationExceptions)
                    throw this.formulaErrorStrings[this._bad_formula];
                return this.getErrorStrings()[4].toString();
            }
            return z.toString();
        };
        this.computeAcot = function (argList) {
            var args = this.splitArgsPreservingQuotedCommas(argList);
            var argCount = args.length;
            if (argCount != 1) {
                if (this._rethrowLibraryComputationExceptions)
                    throw this.formulaErrorStrings[this._wrong_number_arguments];
                return this.formulaErrorStrings[this._wrong_number_arguments];
            }
            var argValue = this.getValueFromArg(args[0]);
            var dVal = this._parseDouble(argValue);
            if (isNaN(dVal)) {
                if (this._rethrowLibraryComputationExceptions)
                    throw this.formulaErrorStrings[this._bad_formula];
                return (argValue[0] == this.tic || this._isCellReference(args[0])) ? this.getErrorStrings()[1].toString() : this.getErrorStrings()[5].toString();
            }
            if (dVal < 1 && dVal > -1) {
                if (this._rethrowLibraryComputationExceptions)
                    throw this.formulaErrorStrings[this._bad_formula];
                return this.getErrorStrings()[4].toString();
            }
            var d1 = this._parseDouble(argList), d2 = 0;

            if (!isNaN(d1)) {
                d2 = Math.PI / 2 - Math.atan(d1);
            }
            return d2.toString();
        };
        this.computeAcoth = function (argList) {
            var args = this.splitArgsPreservingQuotedCommas(argList);
            var argCount = args.length;
            if (argCount != 1) {
                if (this._rethrowLibraryComputationExceptions)
                    throw this.formulaErrorStrings[this._wrong_number_arguments];
                return this.formulaErrorStrings[this._wrong_number_arguments];
            }
            var argValue = this.getValueFromArg(args[0]);
            var dVal = this._parseDouble(argValue);
            if (isNaN(dVal)) {
                if (this._rethrowLibraryComputationExceptions)
                    throw this.formulaErrorStrings[this._bad_formula];
                return (argValue[0] == this.tic || this._isCellReference(args[0])) ? this.getErrorStrings()[1].toString() : this.getErrorStrings()[5].toString();
            }
            if (dVal < 1 && dVal > -1) {
                if (this._rethrowLibraryComputationExceptions)
                    throw this.formulaErrorStrings[this._bad_formula];
                return this.getErrorStrings()[4].toString();
            }
            var acothVal = (Math.log((dVal + 1) / (dVal - 1)) / 2).toString();

            return acothVal;
        };
        this.computeArabic = function (arg) {
            var args = this.splitArgsPreservingQuotedCommas(arg);
            if (args.length != 1) {
                if (this._rethrowLibraryComputationExceptions)
                    throw this.formulaErrorStrings[this._wrong_number_arguments];
                return this.formulaErrorStrings[this._wrong_number_arguments];
            }
            var text = this.getValueFromArg(args[0]);
            var dvalue = parseInt(text.split(this.tic).join("").toUpperCase());
            if (!isNaN(dvalue) || (!this._isCellReference(args[0]) && args[0][0] != this.tic)) {
                if (this._rethrowLibraryComputationExceptions)
                    throw this.formulaErrorStrings[this._bad_formula];
                return (!isNaN(dvalue)) ? this.getErrorStrings()[1].toString() : this.getErrorStrings()[5].toString();
            }
            text = text.split(this.tic).join("").toUpperCase()
            var i = 0;
            var ch = null;
            var result = 0;
            var new_value = 0;
            var old_value = 0;
            old_value = 1000;
            var isNegative = false;
            for (i = 1; i <= text.length; i++) {
                // See what the next character is worth.
                ch = this._substring(text, i - 1, 1);
                switch (ch) {
                    case "-":
                        isNegative = true;
                        break;
                    case "I":
                        new_value = 1;
                        break;
                    case "V":
                        new_value = 5;
                        break;
                    case "X":
                        new_value = 10;
                        break;
                    case "L":
                        new_value = 50;
                        break;
                    case "C":
                        new_value = 100;
                        break;
                    case "D":
                        new_value = 500;
                        break;
                    case "M":
                        new_value = 1000;
                        break;
                    default: {
                        if (this._rethrowLibraryComputationExceptions)
                            throw this.formulaErrorStrings[this._bad_formula];
                        return this.getErrorStrings()[1].toString();
                    }
                }

                // See if this character is bigger
                // than the previous one.
                if (new_value > old_value) {
                    // The new value > the previous one.
                    // Add this value to the result
                    // and subtract the previous one twice.
                    result = result + new_value - 2 * old_value;
                } else {
                    // The new value <= the previous one.
                    // Add it to the result.
                    result = result + new_value;
                }
                old_value = new_value;
            }
            if (isNegative)
                result = -result;
            return result.toString();
        };
        this.computeAsin = function (argList) {
            var args = this.splitArgsPreservingQuotedCommas(argList);
            var argCount = args.length;
            if (argCount != 1) {
                if (this._rethrowLibraryComputationExceptions)
                    throw this.formulaErrorStrings[this._wrong_number_arguments];
                return this.formulaErrorStrings[this._wrong_number_arguments];
            }
            var argValue = this.getValueFromArg(args[0]);
            var dVal = this._parseDouble(argValue);
            if (isNaN(dVal)) {
                if (this._rethrowLibraryComputationExceptions)
                    throw this.formulaErrorStrings[this._bad_formula];
                return (argValue[0] == this.tic || this._isCellReference(args[0])) ? this.getErrorStrings()[1].toString() : this.getErrorStrings()[5].toString();
            }
            if (dVal > 1 || dVal < -1) {
                if (this._rethrowLibraryComputationExceptions)
                    throw this.formulaErrorStrings[this._bad_formula];
                return this.getErrorStrings()[4].toString();
            }
            var d1 = this._parseDouble(argList), d2 = 0;

            if (!isNaN(d1)) {
                d2 = Math.asin(d1);
            }
            return d2.toString();
        };
        this.computeAtan = function (args) {
            return this._computeMath(args, Math.atan).toString();
        };
        this.computeAtan2 = function (argList) {
            var args = this.splitArgsPreservingQuotedCommas(argList);
            var argCount = args.length;
            if (argCount != 2) {
                return this.formulaErrorStrings[this.requires_2_args];
            }

            var x;
            var y = 0;
            var atan2 = 0;
            var param1, param2;
            var flag1, flag2;
            param1 = this.getValueFromArg(args[0]);
            param2 = this.getValueFromArg(args[1]);

            flag1 = (param1 == "true") ? true : false;
            flag2 = (param2 == "true") ? true : false;

            if (flag1)
                param1 = (flag1).toString();
            else if (param1 == (this._string_empty))
                param1 = "0";
            if (flag2)
                param2 = flag2.toString();
            else if (param2 == (this._string_empty))
                param2 = "0";
            if (this.getErrorStrings().indexOf(param1) > -1) {
                return param1;
            }
            if (this.getErrorStrings().indexOf(param2) > -1) {
                return param2;
            }
            x = this._parseDouble(param1);
            y = this._parseDouble(param2);
            if (isNaN(x) || isNaN(y)) {
                if (this._rethrowLibraryComputationExceptions)
                    throw this.getErrorStrings()[1].toString();
                else
                    return this.getErrorStrings()[1].toString();
            } else if (y == 0 && x == 0) {
                if (this._rethrowLibraryComputationExceptions)
                    throw this.getErrorStrings()[3].toString();
                else
                    return this.getErrorStrings()[3].toString();
            }
            atan2 = Math.atan2(y, x);
            return atan2.toString();
        };
        this.computeCeilingMath = function (args) {
            var range = this.splitArgsPreservingQuotedCommas(args);
            var argCount = range.length;
            if (this._isTextEmpty(args) || argCount > 3 || argCount < 1) {
                if (this._rethrowLibraryComputationExceptions)
                    throw this.formulaErrorStrings[this._wrong_number_arguments];
                return this.formulaErrorStrings[this._wrong_number_arguments];
            }
            var range0 = this.getValueFromArg(range[0]);
            var range1 = (argCount > 1 && range[1].length != 0) ? this.getValueFromArg(range[1]) : "1";
            var range2 = (argCount == 3 && range[2].length != 0) ? this.getValueFromArg(range[2]) : "1";

            var d1, d2 = -1, d3 = -1, result;
            var numberresult = false, signresult = false, moderesult = false;
            numberresult = (range0.split(this.tic).join(this._string_empty) == "true") ? true : false;
            signresult = (range1.split(this.tic).join(this._string_empty) == "true") ? true : false;
            moderesult = (range2.split(this.tic).join(this._string_empty) == "true") ? true : false;
            range0 = numberresult ? (numberresult).toString() : range0;
            range1 = signresult ? (signresult).toString() : range1;
            range2 = moderesult ? (moderesult).toString() : range2;

            if (this.getErrorStrings().indexOf(range0) != -1)
                return range0;
            if (this.getErrorStrings().indexOf(range1) != -1)
                return range1;
            if (this.getErrorStrings().indexOf(range2) != -1)
                return range2;

            d1 = this._parseDouble(range0); //.split(this.tic).join(this._string_empty);
            d2 = this._parseDouble(range1); //.split(this.tic).join(this._string_empty);
            d3 = this._parseDouble(range2); //.split(this.tic).join(this._string_empty);
            if (isNaN(d1) && !isNaN(d2) && !isNaN(d3)) {
                if ((d1 == 0 && !this._isTextEmpty(range0) && (this.computeIsText(range0) == this.falseValueStr)) || (d2 == 0 && !this._isTextEmpty(range1) && (this.computeIsText(range1) == this.falseValueStr) || (d3 == 0 && !this._isTextEmpty(range2) && (this.computeIsText(range2) == this.falseValueStr)))) {
                    if (this._rethrowLibraryComputationExceptions)
                        throw this.formulaErrorStrings[this._bad_index];
                    return this.getErrorStrings()[5].toString();
                }
                if ((this._isTextEmpty(range0.split(this.tic).join(this._string_empty)) && range0[0] == (this.tic)) || (this._isTextEmpty(range1.split(this.tic).join(this._string_empty)) && range1[1] == (this.tic)) || (this._isTextEmpty(range2.split(this.tic).join(this._string_empty)) && range2[2] == (this.tic))) {
                    if (this._rethrowLibraryComputationExceptions)
                        throw this.formulaErrorStrings[this._invalid_arguments];
                    return this.getErrorStrings()[1].toString();
                }
                if (this._isTextEmpty(range0.split(this.tic).join(this._string_empty)) || this._isTextEmpty(range1.split(this.tic).join(this._string_empty)) || this._isTextEmpty(range2.split(this.tic).join(this._string_empty))) {
                    return "0";
                }
                if (this._rethrowLibraryComputationExceptions)
                    throw this.formulaErrorStrings[this._invalid_arguments];
                return this.getErrorStrings()[1].toString();
            }

            if (d1 < 0 && d3 < 0) {
                d2 = -1;
            }
            result = this._parseDouble(this.computeCeiling(d1.toString() + this._parseArgumentSeparator + d2.toString()));

            return result.toString();
        };
        this.computeCeiling = function (args) {
            var range = this.splitArgsPreservingQuotedCommas(args);
            var argCount = range.length;
            if (argCount != 2) {
                return this.formulaErrorStrings[this._wrong_number_arguments];
            }

            var range0 = (argCount > 1) ? this.getValueFromArg(range[0]) : "1";
            var range1 = (argCount > 1 && range[1].length != 0) ? this.getValueFromArg(range[1]) : "1";

            var numberresult = false, signresult = false;

            numberresult = (range0.split(this.tic).join(this._string_empty) == "true") ? true : false;
            signresult = (range1.split(this.tic).join(this._string_empty) == "true") ? true : false;
            range0 = numberresult ? numberresult.toString() : range0;
            range1 = signresult ? signresult.toString() : range1;

            if (this.getErrorStrings().indexOf(range0) != -1)
                return range0;
            if (this.getErrorStrings().indexOf(range1) != -1)
                return range1;
            var d1, d2 = -1, d3 = -1;
            d1 = this._parseDouble(range0);
            d2 = this._parseDouble(range1);
            if (isNaN(d1) && !isNaN(d2)) {
                if ((d1 == 0 && !this._isTextEmpty(range0) && (this.computeIsText(range0) == this.falseValueStr)) || (d2 == 0 && !this._isTextEmpty(range1) && (this.computeIsText(range1) == this.falseValueStr))) {
                    if (this._rethrowLibraryComputationExceptions)
                        throw this.formulaErrorStrings[this._bad_index];
                    return this.getErrorStrings()[5].toString();
                }
                if ((this._isTextEmpty(range0.split(this.tic).join(this._string_empty)) && range0[0] == (this.tic)) || (this._isTextEmpty(range1.split(this.tic).join(this._string_empty)) && range1[0] == (this.TIC))) {
                    if (this._rethrowLibraryComputationExceptions)
                        throw this.formulaErrorStrings[this.invalid_arguments];
                    return this.getErrorStrings()[1].toString();
                }
                if (this._isTextEmpty(range0.split(this.tic).join(this._string_empty)) || this._isTextEmpty(range1.split(this.tic).join(this._string_empty))) {
                    return "0";
                }
                if (this._rethrowLibraryComputationExceptions)
                    throw this.formulaErrorStrings[this._invalid_arguments];
                return this.getErrorStrings()[1].toString();
            }
            d1 = this._parseDouble(range0);
            d2 = this._parseDouble(range1);
            if (!isNaN(d1) && !isNaN(d2)) {
                if (d1 > 0 && d2 < 0)
                {
                    if (this._rethrowLibraryComputationExceptions)
                        throw this.formulaErrorStrings[this._invalid_arguments];
                    return this.getErrorStrings()[4].toString();
                }
                if (d1 == 0)
                    return d1.toString();
                d3 = Math.floor(d1 / d2) * d2;
                if (d2 > 0) {
                    while (d3 < d1) {
                        d3 += d2;
                    }
                } else {
                    while (d3 > d1) {
                        d3 += d2;
                    }
                }
            }
            return d3.toString();
        };
        this.computeDecimal2 = function (argList) {
            var args = this.splitArgsPreservingQuotedCommas(argList);
            var argCount = args.length;
            if (argCount > 3) {
                if (this._rethrowLibraryComputationExceptions)
                    throw this.formulaErrorStrings[this._wrong_number_arguments];
                return this.formulaErrorStrings[this._wrong_number_arguments];
            }
            var Base64 = "0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz+/";
            if (!this.getValueFromArg(args[0]).indexOf(this.tic) && !this._isCellReference(args[0])) {
                if (this._rethrowLibraryComputationExceptions)
                    throw this.formulaErrorStrings[this._invalid_Math_argument];
                return this.getErrorStrings()[1].toString();
            }
            var strValue;
            var radix = 0;
            try {
                strValue = this.getValueFromArg(args[0]).split(this.tic).join("");
                radix = parseInt(this.getValueFromArg(args[1]).split(this.tic).join(""));
            } catch (ex) {
                if (this._rethrowLibraryComputationExceptions)
                    throw this.formulaErrorStrings[this._invalid_Math_argument] + ex;
                return this.getErrorStrings()[1].toString();
            }
            var result;
            if (radix < 2 || radix > Base64.length) {
                if (this._rethrowLibraryComputationExceptions)
                    throw "Base requested outside range, it should be from 2 - 16";
                return this.getErrorStrings()[4].toString();
            }
            strValue = strValue.trim();
            if (radix <= 36)
                strValue = strValue.toUpperCase();
            var pos = 0;
            result = 0;

            while (pos < strValue.length && !this._isWhiteSpace(strValue[pos])) {
                var digit = strValue.substring(pos, 1);
                var i = Base64.indexOf(digit);
                if (i >= 0 && i < radix) {
                    result *= radix;
                    result += i;
                    pos++;
                } else {
                    if (this._rethrowLibraryComputationExceptions)
                        throw "Base requested outside range, it should be from 2 - 16";
                    return this.getErrorStrings()[4].toString();
                }
            }
            return result.toString();
        };
        this.computeCombin = function (argList) {
            var args = this.splitArgsPreservingQuotedCommas(argList);
            var argCount = args.length;
            if (argCount != 2) {
                if (this._rethrowLibraryComputationExceptions)
                    throw this.formulaErrorStrings[this._this.requires_2_args];
                else
                    return this.formulaErrorStrings[this.requires_2_args];
            }

            ////N things taken k at the time.
            var nd;
            var kd;
            var combin = 0;
            if (args[0].split(this.tic).join("") == (this._string_empty) || args[1].split(this.tic).join("") == (this._string_empty)) {
                if (this._rethrowLibraryComputationExceptions)
                    throw this.getErrorStrings()[1].toString();
                else
                    return this.getErrorStrings()[1].toString();
            }
            nd = this._parseDouble(this.getValueFromArg(args[0]).split(this.tic).join(""));
            kd = this._parseDouble(this.getValueFromArg(args[1]).split(this.tic).join(""));
            if (!isNaN(nd) && !isNaN(kd)) {
                var k = parseInt((kd + 0.1).toString());
                var n = parseInt((nd + 0.1).toString());
                if (n < k || n < 1) {
                    if (this._rethrowLibraryComputationExceptions)
                        throw this.getErrorStrings()[1].toString();
                    else
                        return this.getErrorStrings()[4].toString();
                }
                combin = this._comb(k, n);
            } else {
                if (!this._rethrowLibraryComputationExceptions)
                    throw this.getErrorStrings()[1].toString();
                else
                    return this.formulaErrorStrings[this._invalid_arguments];
            }
            return combin.toString();
        };
        this.computeCombinA = function (argList) {
            var args = this.splitArgsPreservingQuotedCommas(argList);
            var argCount = args.length;
            if (argList.indexOf("u") > -1) {
                if (this._rethrowLibraryComputationExceptions) {
                    throw this.getErrorStrings()[4].toString();
                }
                return this.getErrorStrings()[4].toString();
            }
            if (argCount != 2) {
                if (this._rethrowLibraryComputationExceptions) {
                    throw this.formulaErrorStrings[this._wrong_number_arguments];
                }
                return this.formulaErrorStrings[this._wrong_number_arguments];
            }
            if (argList.length > 15) {
                if (this._rethrowLibraryComputationExceptions) {
                    throw this.getErrorStrings()[4].toString();
                }
                return this.getErrorStrings()[4].toString();
            }
            var number1 = parseInt(this.getValueFromArg(args[0]).split(this.tic).join(""));
            var choosedNumber = parseInt(this.getValueFromArg(args[1]).split(this.tic).join(""));
            var result;
            if (number1 == 0 && choosedNumber == 0) {
                result = "1";
            } else {
                var number2 = number1 + choosedNumber - 1;
                var pass = number2 + "," + (number1 - 1);
                result = this.computeCombin(pass);
            }
            return result;
        };
        this.computeCos = function (args) {
            return this._computeMath(args, Math.cos).toString();
        };
       
        this.computeCosh = function (args) {
            var arg = this._splitArguments(args, this.getParseArgumentSeparator().toString());
            var argCount = arg.length;
            if (argCount > 1) {
                return this.formulaErrorStrings[this._wrong_number_arguments];
            }
            var argVal = this.getValueFromArg(arg[0]);
            var value = this._parseDouble(argVal.split(this.tic).join(""));
            if (isNaN(value)) {
                if (this._rethrowLibraryComputationExceptions)
                    throw this.formulaErrorStrings[this._bad_formula];
                return (argVal[0] == this.tic || this._isCellReference(arg[0])) ? this.getErrorStrings()[1].toString() : this.getErrorStrings()[5].toString();
            }
            var y = Math.exp(value);
            return (y + 1 / y) / 2;
        };
        this.computeCot = function (argList) {
            var args = this.splitArgsPreservingQuotedCommas(argList);
            var argCount = args.length;
            var argCount = args.length;
            if (argCount != 1) {
                if (this._rethrowLibraryComputationExceptions)
                    throw this.formulaErrorStrings[this._wrong_number_arguments];
                return this.formulaErrorStrings[this._wrong_number_arguments];
            }
            if (argList[0] == (this.tic) && argList[length - 1] == (this.tic))
                argList = argList.split(this.tic).join("");
            if (argList == this._string_empty) {
                if (this._rethrowLibraryComputationExceptions)
                    throw this.getErrorStrings()[4].toString();
                return this.getErrorStrings()[4].toString();
            }
            var isNumber = this.computeIsNumber(argList);
            if (isNumber == this.trueValueStr) {
                var tempList = argList;
                tempList = tempList.indexOf("u") ? tempList.split("u").join(this._string_empty) : tempList;
                var number = this._parseDouble(tempList);
                if (number >= 134217728) {
                    if (this._rethrowLibraryComputationExceptions)
                        throw this.getErrorStrings()[4].toString();
                    return this.getErrorStrings()[4].toString();
                }
            } else {
                if (this._rethrowLibraryComputationExceptions)
                    throw this.getErrorStrings()[4].toString();
                return this.getErrorStrings()[5].toString();
            }
            if (argList == "0") {
                if (this._rethrowLibraryComputationExceptions)
                    throw this.getErrorStrings()[3].toString();
                return this.getErrorStrings()[3].toString();
            }
            var cotVal = this._computeMath(argList, Math.tan);
            if (cotVal != "#NUM!" || cotVal != "#VALUE!") {
                cotVal = (1 / parseFloat(cotVal)).toString();
            }
            return cotVal;
        };
        this.computeCsc = function (argList) {
            var args = this.splitArgsPreservingQuotedCommas(argList);
            var argCount = args.length;
            if (argCount != 1) {
                if (this._rethrowLibraryComputationExceptions)
                    throw this.formulaErrorStrings[this._wrong_number_arguments];
                return this.formulaErrorStrings[this._wrong_number_arguments];
            }
            var argValue = this.getValueFromArg(args[0]);
            var dVal = this._parseDouble(argValue);
            if (isNaN(dVal)) {
                if (this._rethrowLibraryComputationExceptions)
                    throw this.formulaErrorStrings[this._bad_formula];
                return (argValue[0] == this.tic || this._isCellReference(args[0])) ? this.getErrorStrings()[1].toString() : this.getErrorStrings()[5].toString();
            }
            if (dVal > 1 || dVal < -1) {
                if (this._rethrowLibraryComputationExceptions)
                    throw this.formulaErrorStrings[this._bad_formula];
                return this.getErrorStrings()[4].toString();
            }
            var cscVal = (1 / parseFloat(Math.sin(dVal))).toString();
            return cscVal;
        };
        
        this.computeCsch = function (argList) {
            var args = this.splitArgsPreservingQuotedCommas(argList);
            var argCount = args.length;
            if (argCount != 1) {
                if (this._rethrowLibraryComputationExceptions)
                    throw this.formulaErrorStrings[this._wrong_number_arguments];
                return this.formulaErrorStrings[this._wrong_number_arguments];
            }
            var argValue = this.getValueFromArg(args[0]);
            var dVal = this._parseDouble(argValue);
            var isNumber = this.computeIsNumber(argList);
            if (isNaN(dVal)) {
                if (this._rethrowLibraryComputationExceptions)
                    throw this.formulaErrorStrings[this._bad_formula];
                return (argValue[0] == this.tic || this._isCellReference(args[0])) ? this.getErrorStrings()[1].toString() : this.getErrorStrings()[5].toString();
            }
            if (dVal == 0) {
                if (this._rethrowLibraryComputationExceptions)
                    throw this.formulaErrorStrings[this._bad_formula];
                return this.getErrorStrings()[3].toString();
            }
            if (dVal > 709) {
                return "0";
            }
            var cschVal = (2 / (Math.exp(dVal) - Math.exp(-dVal))).toString();

            return cschVal;
        };
        this.computeDecimal = function (argList) {
            var args = this.splitArgsPreservingQuotedCommas(argList);
            var argCount = args.length;
            if (argCount > 3) {
                if (this._rethrowLibraryComputationExceptions)
                    throw this.formulaErrorStrings[this._wrong_number_arguments];
                return this.formulaErrorStrings[this._wrong_number_arguments];
            }
            var Base64 = "0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz+/";
            if (this.getValueFromArg(args[0]).indexOf(this.tic) != -1 && !this._isCellReference(args[0])) {
                if (this._rethrowLibraryComputationExceptions)
                    throw this.formulaErrorStrings[this._invalid_Math_argument];
                return this.getErrorStrings()[1].toString();
            }
            var strValue;
            var radix = 0;
            try {
                strValue = this.getValueFromArg(args[0]).split(this.tic).join("");
                radix = parseInt(this.getValueFromArg(args[1]).split(this.tic).join(""));
            } catch (ex) {
                if (this._rethrowLibraryComputationExceptions)
                    throw this.formulaErrorStrings[this._invalid_Math_argument] + ex;
                return this.getErrorStrings()[1].toString();
            }
            var result;
            if (radix < 2 || radix > Base64.length) {
                if (this._rethrowLibraryComputationExceptions)
                    throw "Base requested outside range, it should be from 2 - 16";
                return this.getErrorStrings()[4].toString();
            }
            strValue = strValue.trim();
            if (radix <= 36)
                strValue = strValue.toUpperCase();
            var pos = 0;
            result = 0;

            while (pos < strValue.length && !this._isWhiteSpace(strValue[pos])) {
                var digit = strValue.substring(pos, 1);
                var i = Base64.indexOf(digit);
                if (i >= 0 && i < radix) {
                    result *= radix;
                    result += i;
                    pos++;
                } else {
                    if (this._rethrowLibraryComputationExceptions)
                        throw "Base requested outside range, it should be from 2 - 16";
                    return this.getErrorStrings()[4].toString();
                }
            }

            return result.toString();
        };
        this.computeDegrees = function (argList) {
            var args = this.splitArgsPreservingQuotedCommas(argList);
            var argCount = args.length;
            if (argCount != 1) {
                if (this._rethrowLibraryComputationExceptions)
                    throw this.formulaErrorStrings[this._wrong_number_arguments];
                return this.formulaErrorStrings[this._wrong_number_arguments];
            }
            var argValue = this.getValueFromArg(args[0]);
            var radians = this._parseDouble(argValue);
            if (isNaN(radians)) {
                if (this._rethrowLibraryComputationExceptions)
                    throw this.formulaErrorStrings[this._bad_formula];
                return (argValue[0] == this.tic || this._isCellReference(args[0])) ? this.getErrorStrings()[1].toString() : this.getErrorStrings()[5].toString();
            }
            var degrees = 0;
            degrees = 180 * radians / Math.PI;
            return degrees.toString();
        };
        
        this.computeExp = function (argList) {
            var args = this.splitArgsPreservingQuotedCommas(argList);
            var argCount = args.length;
            if (argCount > 1) {
                return this.formulaErrorStrings[this.invalid_arguments];
            }
            var argVal = this.getValueFromArg(args[0]);
            if (argVal == "")
                argList = "0";
            var x = this._parseDouble(argVal);
            if (x > 709) {
                if (this._rethrowLibraryComputationExceptions)
                    throw this.formulaErrorStrings[this._bad_formula];
                return this.getErrorStrings()[4].toString();
            }
            return this._computeMath(argList, Math.exp).toString();
        };

        this.computeEven = function (argList) {
            var even = 0;
            var args = this.splitArgsPreservingQuotedCommas(argList);
            var argCount = args.length;
            if (argCount > 1) {
                return this.formulaErrorStrings[this.invalid_arguments];
            }
            var argVal = this.getValueFromArg(args[0]);
            var number = this._parseDouble(argVal);
            if (!isNaN(number)) {
                var sgn = number > 0 ? 1 : (number === 0 || isNaN(number)) ? x : -1;
                number = Math.abs(number);
                number = Math.ceil(number);
                if ((number % 2) == 1) {
                    even = sgn * (number + 1);
                }
                else {
                    even = sgn * number;
                }
            }
            else {
                if (this._rethrowLibraryComputationExceptions)
                    throw this.formulaErrorStrings[this._bad_formula];
                return (argVal[0] == this.tic || this._isCellReference(args[0])) ? this.getErrorStrings()[1].toString() : this.getErrorStrings()[5].toString();
            }

            return even.toString();
        }

        this.computeFact = function (args) {
            var fact = 0;
            var numList = this.splitArgsPreservingQuotedCommas(args);
            if (numList.length > 1) {
                return this.formulaErrorStrings[this.invalid_arguments];
            }
            number = parseInt(this.getValueFromArg(args));
            if (isNaN(number)) {
                return this.getErrorStrings()[1].toString();
            } else if (number < 0) {
                return this.getErrorStrings()[4].toString();
            } else {
                var x = number;
                if (x > 12) {
                    fact = this._factorialTable[12];
                    for (var i = 13; i <= x; i++) {
                        fact *= i;
                    }
                } else {
                    fact = this._factorialTable[x];
                }
            }
            return fact.toString();
        };
        this.computeFactdouble = function (args) {
            var number;
            var fact = 1;
            var numList = this.splitArgsPreservingQuotedCommas(args);
            if (numList.length > 1) {
                return this.formulaErrorStrings[this.invalid_arguments];
            }
            number = parseInt(this.getValueFromArg(args));
            if (isNaN(number)) {
                return this.getErrorStrings()[1].toString();
            } else if (number < 0) {
                return this.getErrorStrings()[4].toString();
            } else {
                var x = number;
                var n = x;
                if (x > 3) {
                    while (n > 0) {
                        fact = fact * n;
                        n = n - 2;
                    }
                } else if (x == 0)
                    fact = 1;
                else
                    fact = x;
            }
            return fact.toString();
        };
        this.computeFloor = function (args) {
            var range = this.splitArgsPreservingQuotedCommas(args);
            var argCount = range.length;
            if (argCount != 2) {
                return this.formulaErrorStrings[this._wrong_number_arguments];
            }

            var d1 = this._parseDouble(this.getValueFromArg(range[0]));
            var d2 = this._parseDouble(this.getValueFromArg(range[1])), d3 = 0;
            if (!isNaN(d1) && !isNaN(d2)) {
                if (d1 == 0)
                    return d3.toString();
                if (d1 * d2 <= 0 && !(d1 < 0)) {
                    return this.formulaErrorStrings[this._invalid_arguments];
                }

                d3 = Math.ceil(d1 / d2) * d2;
                if (d2 > 0) {
                    while (d3 > d1) {
                        d3 -= d2;
                    }
                } else {
                    while (d3 < d1) {
                        d3 -= d2;
                    }
                }
                if (d2 == 0.1) {
                    d3 = d1;
                }
            }
            return d3.toString();
        };
        this.computeInt = function (range) {
            var args = this.splitArgsPreservingQuotedCommas(range);
            var argCount = args.length;
            if (argCount > 1) {
                return this.formulaErrorStrings[this.invalid_arguments];
            }
            var argVal = this.getValueFromArg(args[0]);
            var x = parseInt(argVal);
            if (x <= 0) {
                if (this._rethrowLibraryComputationExceptions)
                    throw this.formulaErrorStrings[this._bad_formula];
                return this.getErrorStrings()[4].toString();
            }
            if (!isNaN(x)) {
                return x;
            }
            else {
                if (this._rethrowLibraryComputationExceptions)
                    throw this.formulaErrorStrings[this._bad_formula];
                return (argVal[0] == this.tic || this._isCellReference(args[0])) ? this.getErrorStrings()[1].toString() : this.getErrorStrings()[5].toString();
            }
        };
        this.computeLn = function (argList) {
            var args = this.splitArgsPreservingQuotedCommas(argList);
            var argCount = args.length;
            if (argCount > 1) {
                return this.formulaErrorStrings[this.invalid_arguments];
            }
            var argVal = this.getValueFromArg(args[0]);
            var x = this._parseDouble(argVal);
            if (x <= 0) {
                if (this._rethrowLibraryComputationExceptions)
                    throw this.formulaErrorStrings[this._bad_formula];
                return this.getErrorStrings()[4].toString();
            }
            if (!isNaN(x)) {
                return (Math.log(x)).toString();
            }
            else {
                if (this._rethrowLibraryComputationExceptions)
                    throw this.formulaErrorStrings[this._bad_formula];
                return (argVal[0] == this.tic || this._isCellReference(args[0])) ? this.getErrorStrings()[1].toString() : this.getErrorStrings()[5].toString();
            }
        };
        this.computeLog = function (argList) {
            var args = this.splitArgsPreservingQuotedCommas(argList);
            var argCount = args.length;
            if (argCount > 2) {
                return this.formulaErrorStrings[this.invalid_arguments];
            }
            var argVal = this.getValueFromArg(args[0]);
            var x = this._parseDouble(argVal);
            var baseVal = (argCount == 2) ? this.getValueFromArg(args[1]) : "10";
            var b =  this._parseDouble(baseVal);
            if (x <= 0 || b <= 0) {
                if (this._rethrowLibraryComputationExceptions)
                    throw this.formulaErrorStrings[this._bad_formula];
                return this.getErrorStrings()[4].toString();
            }
            if (!isNaN(x) && !isNaN(b)) {
                return ((Math.log(x) / Math.LN10) / (Math.log(b) / Math.LN10)).toString();
            }
            else {
                if (this._rethrowLibraryComputationExceptions)
                    throw this.formulaErrorStrings[this._bad_formula];
                return (argVal[0] == this.tic || baseVal[0] == this.tic) ? this.getErrorStrings()[1].toString() : this.getErrorStrings()[5].toString();
            }
        };
        this.computeLogTen = function (args) {
            var arg = this._splitArguments(args, this.getParseArgumentSeparator().toString());
            var argCount = arg.length;
            if (argCount > 1) {
                return this.formulaErrorStrings[this._wrong_number_arguments];
            }
            var argVal = this.getValueFromArg(arg[0]);
            var value = this._parseDouble(argVal.split(this.tic).join(""));
            if (isNaN(value)) {
                if (this._rethrowLibraryComputationExceptions)
                    throw this.formulaErrorStrings[this._bad_formula];
                return (argVal[0] == this.tic || this._isCellReference(arg[0])) ? this.getErrorStrings()[1].toString() : this.getErrorStrings()[5].toString();
            }
            if (value <= 0) {
                if (this._rethrowLibraryComputationExceptions)
                    throw this.formulaErrorStrings[this._bad_formula];
                return this.getErrorStrings()[4].toString();
            }
            return (this._computeMath(args, Math.log)/Math.LN10).toString();
        };
        
        this.computePI = function (args) {
            return Math.PI.toString();
        };
        this.computeProduct = function (range) {
            var prod = 1;
            var d;
            var s1;
            var nohits = true;
            this.adjustRangeArg(range);
            var ranges = this.splitArgsPreservingQuotedCommas(range);
            for (var r in ranges) {
                var adjustRange = ranges[r];

                ////is a cellrange
                if (ranges[r].indexOf(':') > -1) {
                    var cellCollection = this.getCellsFromArgs(adjustRange);
                    for (var s in cellCollection) {
                        try {
                            s1 = this.getValueFromArg(cellCollection[s]);
                        } catch (ex) {
                            return ex;
                        }

                        if (s1.length > 0) {
                            d = this._parseDouble(s1);
                            if (!isNaN(d)) {
                                prod = prod * d;
                                nohits = false;
                            } else if (this.getErrorStrings().indexOf(s1) > -1) {
                                return s1;
                            }
                            else {
                                if (this._rethrowLibraryComputationExceptions)
                                    throw this.formulaErrorStrings[this._bad_formula];
                                return this.getErrorStrings()[1].toString();
                            }
                        }
                    }
                } else {
                    try {
                        s1 = this.getValueFromArg(ranges[r]);
                    } catch (ex) {
                        return ex;
                    }

                    if (s1.length > 0) {
                        d = this._parseDouble(s1);
                        if (!isNaN(d)) {
                            prod = prod * d;
                            nohits = false;
                        } else if (this.getErrorStrings().indexOf(s1) > -1) {
                            return s1;
                        }
                        else {
                            if (this._rethrowLibraryComputationExceptions)
                                throw this.formulaErrorStrings[this._bad_formula];
                            return (s1 == this.tic || this._isCellReference(ranges[r])) ? this.getErrorStrings()[1].toString() : this.getErrorStrings()[5].toString();
                        }
                    }
                }
            }
            return nohits ? "0" : prod.toString();
        };
        this.computeSecant = function (argList) {
            var args = this.splitArgsPreservingQuotedCommas(argList);
            argList = this.getValueFromArg(argList);
            var argCount = args.length;
            if (argCount != 1) {
                if (this._rethrowLibraryComputationExceptions)
                    throw this.formulaErrorStrings[this._wrong_number_arguments];
                return this.formulaErrorStrings[this._wrong_number_arguments];
            }
            if (argList[0] == (this.tic) && argList[length - 1] == (this.tic))
                argList = argList.split(this.tic).join("");
            if (argList == this._string_empty) {
                if (this._rethrowLibraryComputationExceptions)
                    throw this.getErrorStrings()[4].toString();
                return this.getErrorStrings()[4].toString();
            }
            var isNumber = this.computeIsNumber(argList);
            if (isNumber == this.trueValueStr) {
                var tempList = argList;
                tempList = tempList.indexOf("u") ? tempList.split("u").join("") : tempList;
                var number = parseFloat(tempList);
                if (number >= 134217728) {
                    if (this._rethrowLibraryComputationExceptions)
                        throw this.getErrorStrings()[4].toString();
                    return this.getErrorStrings()[4].toString();
                }
            } else {
                if (this._rethrowLibraryComputationExceptions)
                    throw this.getErrorStrings()[4].toString();
                return this.getErrorStrings()[5].toString();
            }
            var secVal = this._computeMath(argList, Math.cos);
            if (secVal != "#NUM!" || secVal != "#VALUE!")
                secVal = (1 / parseFloat(secVal)).toString();
            return secVal;
        };
        this.computeSeriessum = function (arg) {
            var args = this.splitArgsPreservingQuotedCommas(arg);
            if (args.length != 4) {
                if (this._rethrowLibraryComputationExceptions)
                    throw this.formulaErrorStrings[this._wrong_number_arguments];
                return this.formulaErrorStrings[this._wrong_number_arguments];
            }
            var xValue = 0, nValue = 0, mValue = 0, coeff = 0, result = 0;
            var arrayArg;
            if (args[3].indexOf(';') > -1) {
                arrayArg = this._splitArguments(args[3].split(this.tic).join(""), ';');
            } else {
                arrayArg = this.getCellsFromArgs(args[3].split(this.tic).join(""));
            }
            if (args[0] == "" || args[1] == "" || args[2] == "") {
                if (this._rethrowLibraryComputationExceptions)
                    throw this.formulaErrorStrings[this._bad_formula];
                return this.getErrorStrings()[0].toString();
            }
            xValue = this._parseDouble(this.getValueFromArg(args[0]));
            nValue = this._parseDouble(this.getValueFromArg(args[1]));
            mValue = this._parseDouble(this.getValueFromArg(args[2]));
            if (isNaN(xValue) || isNaN(nValue) || isNaN(mValue)) {
                if (this._rethrowLibraryComputationExceptions)
                    throw this.formulaErrorStrings[this._bad_formula];
                return (argVal[0] == this.tic || this._isCellReference(arg[0])) ? this.getErrorStrings()[1].toString() : this.getErrorStrings()[5].toString();
            }
            arrayArg = this.getCellsFromArgs(args[3].split(this.tic).join(""));
            for (var i = 0; i < arrayArg.length; i++) {
                coeff = this._parseDouble(this.getValueFromArg(arrayArg[i]));
                if (!isNaN(coeff)) {
                    result = result + (coeff * Math.pow(xValue, (nValue + (i * mValue))));
                } else {
                    if (this._rethrowLibraryComputationExceptions)
                        throw this.formulaErrorStrings[this._bad_formula];
                    return this.getErrorStrings()[1].toString();
                }
            }
            if (isNaN(result)) {
                if (this._rethrowLibraryComputationExceptions)
                    throw this.formulaErrorStrings[this._bad_formula];
                return this.getErrorStrings()[4].toString();
            }
            return result.toString();
        };
        this.computeSin = function (args) {
            return this._computeMath(args, Math.sin).toString();
        };
        this.computeSinh = function (args) {
            var arg = this._splitArguments(args, this.getParseArgumentSeparator().toString());
            var argCount = arg.length;
            if (argCount > 1) {
                return this.formulaErrorStrings[this._wrong_number_arguments];
            }
            var argVal = this.getValueFromArg(arg[0]);
            var value = this._parseDouble(argVal.split(this.tic).join(""));
            if (isNaN(value)) {
                if (this._rethrowLibraryComputationExceptions)
                    throw this.formulaErrorStrings[this._bad_formula];
                return (argVal[0] == this.tic || this._isCellReference(arg[0])) ? this.getErrorStrings()[1].toString() : this.getErrorStrings()[5].toString();
            }
            if (value < 0) {
                if (this._rethrowLibraryComputationExceptions)
                    throw this.formulaErrorStrings[this._bad_formula];
                return this.getErrorStrings()[4].toString();
            }
            var y = Math.exp(value);
            return (y - 1 / y) / 2;
        };
        this.computeSqrt = function (args) {
            var arg = this._splitArguments(args, this.getParseArgumentSeparator().toString());
            var argCount = arg.length;
            if (argCount > 1) {
                return this.formulaErrorStrings[this._wrong_number_arguments];
            }
            var argVal = this.getValueFromArg(arg[0]);
            var value = parseInt(argVal.split(this.tic).join(""));
            if (isNaN(value)) {
                if (this._rethrowLibraryComputationExceptions)
                    throw this.formulaErrorStrings[this._bad_formula];
                return (argVal[0] == this.tic) ? this.getErrorStrings()[1].toString() : this.getErrorStrings()[5].toString();
            }
            if (value < 0) {
                if (this._rethrowLibraryComputationExceptions)
                    throw this.formulaErrorStrings[this._bad_formula];
                return this.getErrorStrings()[4].toString();
            }
            return this._computeMath(args, Math.sqrt).toString();
        };
        this.computeSubTotal = function (args) {
            var ignoreSubtotal = false;
            var cellReference = this._string_empty;
            var val;
            var index = 0;
            var arg = this._splitArguments(args, this.getParseArgumentSeparator().toString());
            var argCount = arg.length;
            if (argCount < 2) {
                return this.formulaErrorStrings[this._wrong_number_arguments];
            }
            if (this._isCellReference(arg[0])) {
                cellReference = this.getValueFromArg(arg[0]);
            } else {
                cellReference = arg[0].toString();
            }
            val = this._parseDouble(cellReference);
            if (!isNaN(val)) {
                index = parseInt(val.toString());
                if (index < 1 || (index > 11 && index < 101) || index > 111) {
                    return this.getErrorStrings()[1].toString();
                }
            }
            else
            {
                if (this._rethrowLibraryComputationExceptions)
                    throw this.formulaErrorStrings[this._bad_formula];
                return this.getErrorStrings()[5].toString();
            }
            if (ignoreSubtotal)
                return "0";

            cellReference = args.substring((arg[0].toString()).length + 1);
            ignoreSubtotal = true;
            var value = this._string_empty;
            switch (index) {
                case 1:
                case 101:
                    value = this.computeAverage(cellReference);
                    break;
                case 2:
                case 102:
                    value = this.computeCount(cellReference);
                    break;
                case 3:
                case 103:
                    value = this.computeCounta(cellReference);
                    break;
                case 4:
                case 104:
                    value = this.computeMax(cellReference);
                    break;
                case 5:
                case 105:
                    value = this.ComputeMin(cellReference);
                    break;
                case 6:
                case 106:
                    value = this.computeProduct(cellReference);
                    break;
                case 7:
                case 107:
                    value = this.computeStdev(cellReference);
                    break;
                case 8:
                case 108:
                    value = this.computeStdevp(cellReference);
                    break;
                case 9:
                case 109:
                    value = this.computeSum(cellReference);
                    break;
                case 10:
                case 110:
                    value = this.computeVar(cellReference);
                    break;
                case 11:
                case 111:
                    value = this.ComputeVarp(cellReference);
                    break;
                default:
                    value = this.getErrorStrings()[1].toString();
                    break;
            }
            ignoreSubtotal = false;
            return value;
        };
        this.computeSumif = function (argList) {
            var args = this.splitArgsPreservingQuotedCommas(argList);
            var argCount = args.length;
            if (argCount != 2 && argCount != 3) {
                return this.formulaErrorStrings[this._wrong_number_arguments];
            }

            var criteriaRange = args[0];
            var criteria = args[1];
            if (criteria.length < 1) {
                return "0";
            }

            var op = this.token_equal;
            if (criteria[0] != this.tic[0] && "=><".indexOf(criteria[0]) == -1) {
                ////cell reference
                criteria = this.getValueFromArg(criteria);
            }

            var offset = (criteria.length > 0 && criteria[0] == this.tic[0]) ? 1 : 0;
            var compare = this.minValue;
            if (criteria.substring(offset)[0] == (">=")) {
                criteria = this._substring(criteria, offset + 2, criteria.length - 2 - 2 * offset);
                op = this.token_greatereq;
            } else if (criteria.substring(offset)[0] == ("<=")) {
                criteria = this._substring(criteria, offset + 2, criteria.length - 2 - 2 * offset);
                op = this.token_lesseq;
            } else if (criteria.substring(offset)[0] == ("<")) {
                criteria = this._substring(criteria, offset + 1, criteria.length - 1 - 2 * offset);
                op = this.token_less;
            } else if (criteria.substring(offset)[0] == (">")) {
                criteria = this._substring(criteria, offset + 1, criteria.length - 1 - 2 * offset);
                op = this.token_greater;
            } else if (criteria.substring(offset)[0] == ("=")) {
                criteria = this._substring(criteria, offset + 1, criteria.length - 1 - 2 * offset);
            }

            criteria = criteria.split(this.tic).join("");
            var numer = this._parseDouble(criteria, compare);

            var sumRange = (argCount == 2) ? criteriaRange : args[2];

            var s1 = this.getCellsFromArgs(criteriaRange);
            var s2 = this.getCellsFromArgs(sumRange);

            var count = s1.length;

            if (count > s2.length) {
                var i = sumRange.indexOf(':');
                if (i > -1) {
                    var startRow = this.rowIndex(this._substring(sumRange, 0, i));
                    var startCol = this.colIndex(this._substring(sumRange, 0, i));
                    var row = this.rowIndex(this._substring(sumRange, i + 1));
                    var col = this.CclIndex(this._substring(sumRange, i + 1));
                    if (startRow != row) {
                        row += count - s2.length;
                    } else if (startCol != col) {
                        col += count - s2.length;
                    }

                    sumRange = this._substring(sumRange, 0, i + 1) + RangeInfo.getAlphaLabel(col) + row.toString();
                    s2 = this.getCellsFromArgs(sumRange);
                }
            }

            var sum = 0;

            var d;
            var s;
            switch (op) {
                case this.token_equal:
                    {
                        for (var index = 0; index < count; ++index) {
                            s = this.getValueFromArg(s1[index]); //// +criteria;

                            if (s.split(this.tic).join("") == criteria) {
                                s = s2[index];
                                d = this._parseDouble((s.split(this.tic).join("")).split("$").join(""));
                                s = this.getValueFromArg(s);
                                if (!isNaN(d)) {
                                    sum += d;
                                }
                            }
                        }
                    }

                    break;
                case this.token_greatereq:
                    {
                        for (var index = 0; index < count; ++index) {
                            s = this.getValueFromArg(s1[index]); //// +criteria;
                            d = this._parseDouble((s.split(this.tic).join("")).split("$").join(""));
                            if (!isNaN(d)) {
                                if (d >= compare) {
                                    s = s2[index];
                                    s = this.getValueFromArg(s);
                                    d = this._parseDouble(s);
                                    if (!isNaN(d)) {
                                        sum += d;
                                    }
                                }
                            }
                        }
                    }

                    break;
                case this.token_greater:
                    {
                        for (var index = 0; index < count; ++index) {
                            s = this.getValueFromArg(s1[index]); //// +criteria;
                            d = this._parseDouble((s.split(this.tic).join("")).split("$").join(""));
                            if (!isNaN(d)) {
                                if (d > compare) {
                                    s = s2[index];
                                    s = this.getValueFromArg(s);
                                    d = this._parseDouble(s);
                                    if (!isNaN(d)) {
                                        sum += d;
                                    }
                                }
                            }
                        }
                    }

                    break;
                case this.token_less:
                    {
                        for (var index = 0; index < count; ++index) {
                            s = this.getValueFromArg(s1[index]); //// +criteria;
                            d = this._parseDouble((s.split(this.tic).join("")).split("$").join(""));
                            if (!isNaN(d)) {
                                if (d < compare) {
                                    s = s2[index];
                                    s = this.getValueFromArg(s);
                                    d = this._parseDouble(s);
                                    if (!isNaN(d)) {
                                        sum += d;
                                    }
                                }
                            }
                        }
                    }

                    break;
                case this.token_lesseq:
                    {
                        for (var index = 0; index < count; ++index) {
                            s = this.getValueFromArg(s1[index]); //// +criteria;
                            d = this._parseDouble((s.split(this.tic).join("")).split("$").join(""));
                            if (!isNaN(d)) {
                                if (d <= compare) {
                                    s = s2[index];
                                    s = this.getValueFromArg(s);
                                    d = this._parseDouble(s);
                                    if (!isNaN(d)) {
                                        sum += d;
                                    }
                                }
                            }
                        }
                    }

                    break;
                default:
                    break;
            }
            return sum.toString();
        };
        this.computeTan = function (args) {
            return this._computeMath(args, Math.tan).toString();
        };
        this.computeTrunc = function (range) {
            var args = this.splitArgsPreservingQuotedCommas(range);
            var argCount = args.length;
            var digits = 0;

            if (argCount == 2) {
                ////ignore return value...
                digits = this._parseDouble(this.getValueFromArg(args[1]));
            }
            var argVal = this.getValueFromArg(args[0]);
            var value = this._parseDouble(argVal.split(this.tic).join(""));
            if (isNaN(value)) {
                if (this._rethrowLibraryComputationExceptions)
                    throw this.formulaErrorStrings[this._bad_formula];
                return (argVal[0] == this.tic || this._isCellReference(args[0])) ? this.getErrorStrings()[1].toString() : this.getErrorStrings()[5].toString();
            }
            var normalizer = Math.pow(10, digits);
            var d1 = (value < 0) ? -1 : 1;
            return (d1 * Math.floor(normalizer * Math.abs(value)) / normalizer).toString();
        };
        this.computeLognormOdist = function (argList) {
            var args = this.splitArgsPreservingQuotedCommas(argList);
            var argCount = args.length;
            if (argCount != 4) {
                return this.formulaErrorStrings[this._wrong_number_arguments];
            }

            var x;
            var u;
            var s;
            var dist = 0;

            for (var i = 0; i < argCount; ++i) {
                args[i] = this.getValueFromArg(args[i]).split(this.trueValueStr).join(this._string_empty);
            }
            x = this._parseDouble(args[0]);
            u = this._parseDouble(args[1]);
            s = this._parseDouble(args[2]);
            if (!isNaN(x) && !isNaN(u) && !isNaN(s)) {
                if (x <= 0 || s <= 0) {
                    if (this._rethrowLibraryComputationExceptions) {
                        throw "#Num! Passsed value is incorrect";
                    }
                    return this.getErrorStrings()[4].toString();
                }
                if (args[3] == this.falseValueStr) {
                    var mean = 0;
                    var std = 0;

                    // mean = Math.Exp(u+Math.Pow(u,2))
                    dist = (Math.exp(-(Math.pow((Math.log(x) - u), 2) / (2 * Math.pow(s, 2))))); //this.normaldist(Math.Log(x), u, s);
                    dist = dist / x * u * Math.sqrt(2 * Math.PI);
                } else {
                    dist = this._standardNormalCumulativeDistribution((Math.log(x) - u) / s);
                }
            } else {
                if (this._rethrowLibraryComputationExceptions) {
                    throw "#Value! Passsed value is non - numeric";
                }
                return this.getErrorStrings()[1].toString();
            }
            return dist.toString();
        };
        this.computeLognormOinv = function (argList) {
            var args = this.splitArgsPreservingQuotedCommas(argList);
            var argCount = args.length;
            if (argCount != 3) {
                return this.formulaErrorStrings[this._wrong_number_arguments];
            }

            var p;
            var u;
            var s;
            var dist = 0;

            for (var i = 0; i < argCount; ++i) {
                args[i] = this.getValueFromArg(args[i]);
            }
            p = this._parseDouble(args[0]);
            u = this._parseDouble(args[1]);
            s = this._parseDouble(args[2]);
            if (!isNaN(p) && !isNaN(u) && !isNaN(s)) {
                dist = Math.exp(this._normalinv(p, u, s));
            }
            return dist.toString();
        };
        this.computeNormOinv = function (argList) {
            var args = this.splitArgsPreservingQuotedCommas(argList);
            var argCount = args.length;
            if (argCount != 3) {
                if (this._rethrowLibraryComputationExceptions)
                    throw this.formulaErrorStrings[this._wrong_number_arguments];
                return this.formulaErrorStrings[this._wrong_number_arguments];
            }

            var p;
            var u;
            var s;
            var invdist = 0;

            for (var i = 0; i < argCount; ++i)
                args[i] = this.getValueFromArg(args[i]).split(this.tic).join(this._string_empty);

            p = this._parseDouble(args[0]);
            u = this._parseDouble(args[1]);
            s = this._parseDouble(args[2]);
            if (!isNaN(p) && !isNaN(u) && !isNaN(s)) {
                if (p >= 0 && p <= 0 || s <= 0) {
                    if (this._rethrowLibraryComputationExceptions) {
                        throw this.formulaErrorStrings[this._wrong_number_arguments] + this.getErrorStrings()[4].toString();
                    }
                    return this.getErrorStrings()[4].toString();
                }
                invdist = this._normalCumulativeDistributionFunctionInverse(p, u, s); //this.normalinv(p, u, s);
            }

            if (invdist <= 0) {
                return this.formulaErrorStrings[this._iterations_dont_converge];
            }
            return invdist.toString();
        };
        this.computeNormOdist = function (argList) {
            var args = this.splitArgsPreservingQuotedCommas(argList);
            var argCount = args.length;
            if (argCount != 4) {
                if (this._rethrowLibraryComputationExceptions) {
                    throw this.formulaErrorStrings[this._wrong_number_arguments];
                }
                return this.formulaErrorStrings[this._wrong_number_arguments];
            }

            var x;
            var u;
            var s;
            var cum = 0;
            var dist = 0;

            for (var i = 0; i < argCount; ++i) {
                args[i] = this.getValueFromArg(args[i]);
            }
            x = this._parseDouble(args[0]);
            u = this._parseDouble(args[1]);
            s = this._parseDouble(args[2]);
            if (!isNaN(x) && !isNaN(u) && !isNaN(s)) {
                if (argCount != 3) {
                    cum = (args[3] == this.trueValueStr) ? 1 : 0;
                    var check = 0;
                    check = this._parseDouble(args[3]);
                    if (!isNaN(check))
                        cum = 1;
                }

                if (cum == 0) {
                    dist = this._normalProbabilityDensity(x, u, s);
                } else {
                    dist = this._normaldist(x, u, s);
                }
            }
            return dist.toString();
        };
        this.computeNormOsODist = function (argList) {
            var args = this.splitArgsPreservingQuotedCommas(argList);
            var argCount = args.length;
            if (argCount != 1 && argCount != 2) {
                if (this._rethrowLibraryComputationExceptions) {
                    throw this.formulaErrorStrings[this._wrong_number_arguments];
                }
                return this.formulaErrorStrings[this._wrong_number_arguments];
            }

            var x;
            var cum = 0;
            var dist = 0;

            for (var i = 0; i < argCount; ++i) {
                args[i] = this.getValueFromArg(args[i]).split(this.tic).join(this._string_empty);
            }
            x = this._parseDouble(args[0]);
            if (!isNaN(x)) {
                if (argCount == 2) {
                    cum = (args[1] == this.trueValueStr) ? 1 : 0;
                    var check = 0;
                    check = this._parseDouble(args[1]);
                    if (!isNaN(check))
                        cum = 1;
                }

                if (cum == 0) {
                    dist = this._standardNormalProbabilityDensity(x);
                } else {
                    dist = this._standardNormalCumulativeDistributionFunction(x);
                }
            } else {
                if (this._rethrowLibraryComputationExceptions) {
                    throw "#VALUE! Passed argument value is non numeric";
                }
                return this.getErrorStrings()[1].toString();
            }
            return dist.toString();
        };
        this.computeNormOsOInv = function (argList) {
            var args = this.splitArgsPreservingQuotedCommas(argList);
            var argCount = args.length;
            if (argCount != 1) {
                if (this._rethrowLibraryComputationExceptions) {
                    throw this.formulaErrorStrings[this._wrong_number_arguments];
                }
                return this.formulaErrorStrings[this._wrong_number_arguments];
            }

            var x;
            var dist = 0;

            for (var i = 0; i < argCount; ++i) {
                args[i] = this.getValueFromArg(args[i]).split(this.tic).join(this._string_empty);
            }
            x = this._parseDouble(args[0]);
            if (!isNaN(x)) {
                if (x <= 0 || x >= 1) {
                    if (this._rethrowLibraryComputationExceptions) {
                        throw "#NUM! Passed argument value doesnot match with in range level";
                    }
                    return this.getErrorStrings()[4].toString();
                }
                dist = this._standardNormalCumulativeDistributionFunctionInverse(x);
            } else {
                if (this._rethrowLibraryComputationExceptions) {
                    throw "#VALUE! Passed argument value is non numeric";
                }
                return this.getErrorStrings()[1].toString();
            }
            return dist.toString();
        };
        this.computePermut = function (argList) {
            var args = this.splitArgsPreservingQuotedCommas(argList);
            var argCount = args.length;
            if (argCount != 2) {
                return this.formulaErrorStrings[this.requires_2_args];
            }

            ////N things taken k at the time.
            var nd;
            var kd;
            var combin = 0;
            nd = this._parseDouble(this.getValueFromArg(args[0]));
            kd = this._parseDouble(this.getValueFromArg(args[1]));
            if (!isNaN(nd) && !isNaN(kd)) {
                var k = (kd + 0.1);
                var n = (nd + 0.1);

                var top = 1;
                for (var i = (n - k) + 1; i <= n; ++i) {
                    top = top * i;
                }

                combin = top;
            } else {
                return this.formulaErrorStrings[this._invalid_arguments];
            }
            return combin.toString();
        };
        this.computePermutationA = function (argList) {
            var args = this.splitArgsPreservingQuotedCommas(argList);
            var argCount = args.length;
            if (argCount != 2) {
                return this.formulaErrorStrings[this.requires_2_args];
            }

            var nd;
            var kd;
            var combin = 0;
            nd = this._parseDouble(args[0]);
            kd = this._parseDouble(args[1]);
            if (!isNaN(nd) && !isNaN(kd)) {
                var k = (kd + 0.1);
                var n = (nd + 0.1);
            } else {
                return this.formulaErrorStrings[this._invalid_arguments];
            }
            return Math.pow(nd, kd).toString();
        };
        this.computeStandardize = function (argList) {
            var args = this.splitArgsPreservingQuotedCommas(argList);
            var argCount = args.length;
            if (argCount != 3) {
                return this.formulaErrorStrings[this._wrong_number_arguments];
            }

            var x;
            var u;
            var sd = 0;
            var dist = 0;

            for (var i = 0; i < argCount; ++i) {
                args[i] = this.getValueFromArg(args[i]);
            }
            x = this._parseDouble(args[0]);
            u = this._parseDouble(args[1]);
            sd = this._parseDouble(args[2]);
            if (!isNaN(x) && !isNaN(u) && !isNaN(sd)) {
                dist = (x - u) / sd;
            }
            return dist.toString();
        };
        this.computeBinomOdist = function (argList) {
            var args = this.splitArgsPreservingQuotedCommas(argList);
            var argCount = args.length;
            if (argCount != 4) {
                return this.formulaErrorStrings[this._wrong_number_arguments];
            }

            var successes;
            var trials;
            var p;
            var cum = 0;
            var dist = 0;

            for (var i = 0; i < argCount; ++i) {
                args[i] = this.getValueFromArg(args[i]).split(this.tic).join(this._string_empty);
            }
            successes = this._parseDouble(args[0]);
            trials = this._parseDouble(args[1]);
            p = this._parseDouble(args[2]);
            if (!isNaN(successes) && !isNaN(trials) && !isNaN(p)) {
                if (successes < 0 || successes > trials || p < 0 || p > 1) {
                    if (this._rethrowLibraryComputationExceptions) {
                        throw "#NAME! Passed argument value is not equal to minimum par value";
                    }
                    return this.getErrorStrings()[4].toString();
                }
                cum = (args[3] == this.trueValueStr) ? 1 : 0;
                if (cum == 0) {
                    dist = this._comb(successes, trials) * Math.pow(p, successes) * Math.pow(1 - p, trials - successes);
                } else {
                    dist = this._binomdist(trials, successes, p);
                }
            } else {
                if (this._rethrowLibraryComputationExceptions) {
                    throw "#VALUE! Passed value is nonnumeric";
                }
                return this.getErrorStrings()[1].toString();
            }
            return dist.toString();
        };
        this.computeChisqOinvOrt = function (argList) {
            var args = this.splitArgsPreservingQuotedCommas(argList);
            var argCount = args.length;
            if (argCount != 2) {
                return this.formulaErrorStrings[this._wrong_number_arguments];
            }

            var p;
            var v;
            var dist = 0;

            for (var i = 0; i < argCount; ++i) {
                args[i] = this.getValueFromArg(args[i]);
            }
            p = this._parseDouble(args[0]);
            v = this._parseDouble(args[1]);
            if (!isNaN(p) && !isNaN(v)) {
                dist = this._chiinv(p, v);
            }
            return dist.toString();
        };
        this.computeChisqOdistOrt = function (argList) {
            var args = this.splitArgsPreservingQuotedCommas(argList);
            var argCount = args.length;
            if (argCount != 2) {
                return this.formulaErrorStrings[this._wrong_number_arguments];
            }

            var x;
            var v;
            var dist = 0;

            for (var i = 0; i < argCount; ++i) {
                args[i] = this.getValueFromArg(args[i]);
            }
            x = this._parseDouble(args[0]);
            v = this._parseDouble(args[1]);
            if (!isNaN(x) && !isNaN(v)) {
                if (v == 1 && this._excelLikeComputations) {
                    //continue;
                } else if (v < 1) {
                    if (this._excelLikeComputations)
                        return "#NUM!";
                    return this.formulaErrorStrings[this._invalid_arguments];
                }

                dist = 1 - this._chidist(x, v);
                for (var i = 0; i < dist.toString().length; i++) {
                }
                var ss = Math.round(dist);
            }
            return dist.toString();
        };
        this.computeFOdist = function (argList) {
            var args = this.splitArgsPreservingQuotedCommas(argList);
            var argCount = args.length;
            if (argCount != 4) {
                return this.formulaErrorStrings[this._wrong_number_arguments];
            }

            var x;
            var df1 = 0;
            var df2 = 0;
            var dist = 0;

            for (var i = 0; i < argCount; ++i) {
                args[i] = this.getValueFromArg(args[i]).split(this.tic).join(this._string_empty);
            }
            x = this._parseDouble(args[0]);
            df1 = this._parseDouble(args[1]);
            df2 = this._parseDouble(args[2]);
            if (!isNaN(x) && !isNaN(df1) && !isNaN(df2)) {
                if (df1 < 1 || df2 < 1 || (x[0] == ("-"))) {
                    if (this._rethrowLibraryComputationExceptions) {
                        throw "#NUM! Passed argument value is incorrect";
                    }
                    return this.getErrorStrings()[4].toString();
                }

                if (args[3] == this.trueValueStr) {
                    dist = this._fCumulativeDensity(x, df1, df2);
                } else if (args[3] == this.falseValueStr) {
                    dist = this._fProbabilityDensity(x, df1, df2);
                }
            } else {
                if (this._rethrowLibraryComputationExceptions) {
                    throw "#VALUE! Passed argument value is incorrect";
                }
                return this.getErrorStrings()[1].toString();
            }
            return dist.toString();
        };
        this.computeGammaln = function (argList) {
            var x = 0;
            x = this._parseDouble(argList);
            if (!isNaN(x) && x > 0) {
                x = this._gammaln(x);
            } else {
                return this.formulaErrorStrings[this._invalid_arguments];
            }
            return x.toString();
        };
        this.computeConfidenceOnorm = function (argList) {
            var args = this.splitArgsPreservingQuotedCommas(argList);
            var argCount = args.length;
            if (argCount != 3) {
                if (this._rethrowLibraryComputationExceptions) {
                    throw "Wrong number of arguments";
                }
                return this.formulaErrorStrings[this._wrong_number_arguments];
            }

            var alpha;
            var s;
            var sz;
            var val = 0;

            for (var i = 0; i < argCount; ++i) {
                args[i] = this.getValueFromArg(args[i]).split(this.tic).join(this._string_empty);
            }

            alpha = this._parseDouble(args[0]);
            s = this._parseDouble(args[1]);
            sz = this._parseDouble(args[2]);
            if (!isNaN(alpha) && !isNaN(s) && !isNaN(sz)) {
                if (alpha <= 0 || alpha >= 1 || s <= 0 || sz < 1) {
                    if (this._rethrowLibraryComputationExceptions) {
                        throw "Passed argument value is different from minimum par";
                    }
                    return this.getErrorStrings()[4].toString();
                }

                //// 1 - alpha  would be from -inf to 1-alpha. adding alpha + 2
                //// takes away the left tail making the integral from alpha-1 to 1-alpha.
                if (this._excelLikeComputations)
                    val = this._normalinv(1 - alpha + alpha / 2, 0, 1);
                else
                    val = this._newnormalinv(1 - alpha + alpha / 2);
                val = val * s / Math.sqrt(sz);
            }

            return val.toString();
        };
        this.computeExponOdist = function (argList) {
            var args = this.splitArgsPreservingQuotedCommas(argList);
            var argCount = args.length;
            if (argCount != 3) {
                return this.formulaErrorStrings[this._wrong_number_arguments];
            }

            var x;
            var lambda;
            var cum = 0;
            var dist = 0;

            for (var i = 0; i < argCount; ++i) {
                args[i] = this.getValueFromArg(args[i]);
            }
            x = this._parseDouble(args[0]);
            lambda = this._parseDouble(args[1]);
            if (!isNaN(x) && !isNaN(lambda)) {
                if (x < 0 || lambda <= 0) {
                    if (this._rethrowLibraryComputationExceptions)
                        throw "Passed argument value is below or equal to 0";
                    return this.getErrorStrings()[4].toString();
                }
                cum = (args[2] == this.trueValueStr) ? 1 : 0;
                var check = this._parseDouble(args[2]);
                if (!isNaN(check))
                    cum = 0;
                if (cum == 0) {
                    dist = lambda * Math.exp(-lambda * x);
                } else {
                    dist = 1 - Math.exp(-lambda * x);
                }
            } else {
                if (this._rethrowLibraryComputationExceptions)
                    throw "Passed argument value is non-numerical";
                return this.getErrorStrings()[1].toString();
            }
            return dist.toString();
        };
        this.computeFisher = function (argList) {
            var args = this.splitArgsPreservingQuotedCommas(argList);
            var argCount = args.length;
            if (argCount != 1) {
                return this.formulaErrorStrings[this._wrong_number_arguments];
            }

            var x;
            var z = 0;

            var s = this.getValueFromArg(args[0]);

            x = this._parseDouble(s);
            if (!isNaN(x) && (x > -1 && x < 1)) {
                z = 0.5 * Math.log((1 + x) / (1 - x));
            }
            return z.toString();
        };
        this.computeFisherInv = function (argList) {
            var args = this.splitArgsPreservingQuotedCommas(argList);
            var argCount = args.length;
            if (argCount != 1) {
                return this.formulaErrorStrings[this._wrong_number_arguments];
            }

            var y;
            var x = 0;

            var s = this.getValueFromArg(args[0]);

            y = this._parseDouble(s);
            if (!isNaN(y)) {
                var d = Math.exp(2 * y);
                x = (d - 1) / (d + 1);
            }
            return x.toString();
        };
        this.computeGammalnOPrecise = function (argList) {
            var args = this.splitArgsPreservingQuotedCommas(argList);
            var argCount = args.length;
            if (args[0] == this.tic + this.tic) {
                if (this._rethrowLibraryComputationExceptions)
                    throw "Passed Argument is empty";
                return this.getErrorStrings()[1].toString();
            }

            if (argCount != 1) {
                if (this._rethrowLibraryComputationExceptions)
                    throw this.formulaErrorStrings[this._wrong_number_arguments];
                return this.formulaErrorStrings[this._wrong_number_arguments];
            }

            var x = 0;
            x = this._parseDouble(args);
            if (!isNaN(x) && x > 0)
                x = this._gammaln(x);
            else {
                if (x <= 0) {
                    if (this._rethrowLibraryComputationExceptions)
                        throw "Passed Argument value is less than or equal to minimum value 0";
                    return this.getErrorStrings()[4].toString();
                }
                if (this._rethrowLibraryComputationExceptions)
                    throw "Passed Argument is non numerical";
                return this.getErrorStrings()[1].toString();
            }
            return x.toString();
        };
        
        this.computeLarge = function (range) {
            var args = this.splitArgsPreservingQuotedCommas(range);
            var argCount = args.length;
            if (argCount != 2) {
                return this.formulaErrorStrings[this._wrong_number_arguments];
            }

            var x0;
            var s1 = this.getValueFromArg(args[1]);
            x0 = this._parseDouble(s1);
            if (isNaN(x0)) {
                return this.formulaErrorStrings[this._invalid_arguments];
            }

            var k = x0;
            var dd = this._getDoubleArray(args[0]);
            var n = dd.length;
            if (k < 1 || k > n) {
                return this.formulaErrorStrings[this._invalid_arguments];
            }

            dd.sort(function (a, b) {
                if (isNaN(a) || isNaN(b)) {
                    if (a > b) return 1;
                    else return -1;
                }
                return a - b;
            });
            return dd[n - k].toString();
        };
        this.computeSmall = function (range) {
            var args = this.splitArgsPreservingQuotedCommas(range);
            var argCount = args.length;
            if (argCount != 2) {
                return this.formulaErrorStrings[this._wrong_number_arguments];
            }

            var s1 = this.getValueFromArg(args[1]);
            var x0 = this._parseDouble(s1);
            if (isNaN(x0)) {
                return this.formulaErrorStrings[this._invalid_arguments];
            }

            var k = x0;
            var dd = this._getDoubleArray(args[0]);
            if (k < 1 || k > dd.length) {
                return this.formulaErrorStrings[this._invalid_arguments];
            }

            dd.sort();
            return dd[k - 1].toString();
        };
        this.computeCounta = function (range) {
            var count = 0;
            var s1;
            var ranges = this.splitArgsPreservingQuotedCommas(range);
            var argsRange;

            if (this._isTextEmpty(range)) {
                if (this._rethrowLibraryComputationExceptions)
                    throw this.formulaErrorStrings[this.invalid_arguments];
                else
                    return this.formulaErrorStrings[this.invalid_arguments];
            }
            for (var r in ranges) {
                argsRange = ranges[r];

                ////is a cellrange
                if (argsRange.indexOf(':') > -1) {
                    var cellCollection = this.getCellsFromArgs(argsRange);
                    for (var s in cellCollection) {
                        try {
                            s1 = this.getValueFromArg(cellCollection[s]).split(this.tic).join(this._string_empty);
                        } catch (ex) {
                            if (this._rethrowLibraryComputationExceptions && this.getLibraryComputationException() != null) {
                                throw this.getLibraryComputationException();
                            }

                            return ex;
                        }

                        if (s1.length > 0) {
                            count++;
                        }
                    }
                } else if (argsRange == "") {
                    count++;
                } else if (this._isLetter(argsRange[0])) {
                    s1 = this.getValueFromArg(argsRange);
                    if (s1.length > 0) {
                        count++;
                    }
                } else {
                    if (argsRange.indexOf(this.getParseArgumentSeparator()) > -1) {
                        var array = this.splitArgsPreservingQuotedCommas(argsRange.split(this.tic).join(""));
                        for (var str in array) {
                            if (str[0] != this.tic && this._isCellReference(array[str])) {
                                if (this._rethrowLibraryComputationExceptions)
                                    throw this.formulaErrorStrings[this.invalid_arguments];
                                else
                                    return this.formulaErrorStrings[this.invalid_arguments];
                            }
                            if (str.length > 0 || str == "") {
                                count++;
                            }
                        }
                    }
                    else if (r.length > 0) {
                        count++;
                    }
                }
            }
            return count.toString();
        };
        this.computeAverage = function (range) {
            var sum = 0, count = 0, d, s1, adjustRange = [];
            var ranges = this.splitArgsPreservingQuotedCommas(range);
            if (ranges.length < 1) {
                if (this._rethrowLibraryComputationExceptions)
                    throw this.formulaErrorStrings[this.invalid_arguments];
                else
                    return this.formulaErrorStrings[this.invalid_arguments];
            }
            for (var r in ranges) {
                adjustRange = ranges[r];
                if (adjustRange.indexOf(':') > -1) {
                    var cellCollection = this.getCellsFromArgs(adjustRange);
                    for (var s in cellCollection) {
                        try {
                            s1 = this.getValueFromArg(cellCollection[s]).split(this.tic).join("");
                        } catch (ex) {
                            if (this._rethrowLibraryComputationExceptions)
                                throw this.getErrorStrings()[4].toString();
                            else
                                return this.getErrorStrings()[4].toString();
                        }

                        if (s1.length > 0) {
                            d = this._parseDouble(s1);
                            if (!isNaN(d)) {
                                sum = Number(sum) + Number(d);
                                count++;
                            } else if (this.getErrorStrings().indexOf(s1) == -1) {
                                return s1;
                            } else {
                                if (this._rethrowLibraryComputationExceptions)
                                    throw this.getErrorStrings()[5].toString();
                                else
                                    return this.getErrorStrings()[5].toString();
                            }
                        }
                    }
                } else {
                    try {
                        s1 = this.getValueFromArg(adjustRange).split(this.tic).join(this._string_empty);
                    } catch (ex) {
                        if (this._rethrowLibraryComputationExceptions)
                            throw this.getErrorStrings()[4].toString();
                        else
                            return this.getErrorStrings()[4].toString();
                    }

                    if (s1.length > 0) {
                        d = this._parseDouble(s1);
                        if (!isNaN(d)) {
                            sum = Number(sum) + Number(d);
                            count++;
                        } else if (this.getErrorStrings().indexOf(s1)) {
                            return s1;
                        } else if (s1[0] == (this.tic)) {
                            if (this._rethrowLibraryComputationExceptions)
                                throw this.getErrorStrings()[1].toString();
                            else
                                return this.getErrorStrings()[1].toString();
                        } else {
                            if (this._rethrowLibraryComputationExceptions)
                                throw this.getErrorStrings()[5].toString();
                            else
                                return this.getErrorStrings()[5].toString();
                        }
                    }
                }
            }

            if (count > 0) {
                sum = Number(sum) / Number(count);
            }
            return sum.toString();
        };
        this.computeAverageA = function (range) {
            var ranges = this.splitArgsPreservingQuotedCommas(range);
            var newargs = this._string_empty;
            var d = 0;
            for (var i = 0; i < ranges.length; i++) {
                if (ranges[i].indexOf(this.getParseArgumentSeparator().toString()) > -1) {
                    var temp = this.splitArgsPreservingQuotedCommas(ranges[i].split(this.tic).join(this._string_empty));
                    var j = 0;
                    for (var item in temp) {
                        var adjustRange = temp[item];
                        newargs = newargs + adjustRange + this.getParseArgumentSeparator().toString();
                        ranges[j] = adjustRange;
                    }
                } else if (this._isCellReference(ranges[i]))
                    newargs += ranges[i] + this.getParseArgumentSeparator().toString();
                else if (!this._parseDouble(ranges[i], d))
                    newargs = newargs + "0" + this.getParseArgumentSeparator().toString();
                else
                    newargs = newargs + ranges[i] + this.getParseArgumentSeparator().toString();
            }

            // if (newargs.endsWith(this.getParseArgumentSeparator().toString()))
            if (newargs[newargs.length - 1] == (","))
                newargs = newargs.substring(0, newargs.length - 1);
            var sum = this.computeAverage(newargs);
            return sum;
        };
        this.computeMax = function (range) {
            var max = this.minValue;
            var d;
            var s1;
            var argslist;
            this.adjustRangeArg(range);
            var ranges = this.splitArgsPreservingQuotedCommas(range);
            for (var r in ranges) {
                argslist = ranges[r];

                ////cell range
                if (argslist.indexOf(':') > -1) {
                    var cellCollection = this.getCellsFromArgs(argslist);
                    for (var s in cellCollection) {
                        try {
                            s1 = this.getValueFromArg(cellCollection[s]).split(this.tic).join(this._string_empty);
                        } catch (ex) {
                            if (this._rethrowLibraryComputationExceptions && this.getLibraryComputationException() != null) {
                                throw this.getLibraryComputationException();
                            }

                            return ex;
                        }

                        if (s1.length > 0) {
                            d = this._parseDouble(s1);
                            if (!isNaN(d)) {
                                max = Math.max(max, d);
                            } else if (this.getErrorStrings().indexOf(s1)) {
                                return s1;
                            }
                        }
                    }
                } else {
                    try {
                        s1 = (argslist == this._string_empty) ? "0" : this.getValueFromArg(argslist);
                    } catch (ex) {
                        if (this._rethrowLibraryComputationExceptions && this.getLibraryComputationException() != null) {
                            throw this.getLibraryComputationException();
                        }

                        return ex;
                    }

                    if (s1.length > 0) {
                        d = this._parseDouble(s1);
                        if (!isNaN(d)) {
                            max = Math.max(max, d);
                        } else if (this.getErrorStrings().indexOf(s1)) {
                            return s1;
                        }
                    }
                }
            }
            if (max != this.minValue) {
                return max.toString();
            }

            return this._string_empty;
        };
        this.computeMaxa = function (range) {
            var max = this.minValue;
            var d;
            var s1;

            this.adjustRangeArg(range);
            var ranges = this.splitArgsPreservingQuotedCommas(range);
            for (var r in ranges) {
                ////cell range
                if (ranges[r].indexOf(':') > -1) {
                    var cell = this.getCellsFromArgs(ranges[r]);
                    for (var s in cell) {
                        try {
                            s1 = this.getValueFromArg(cell[s]);
                        } catch (ex) {
                            if (this._rethrowLibraryComputationExceptions && this.getLibraryComputationException() != null) {
                                throw this.getLibraryComputationException();
                                return ex;
                            }
                        }

                        ////ignore if empty
                        if (s1.length > 0) {
                            d = 0;
                            if (s1.toUpperCase() == (this.trueValueStr)) {
                                d = 1;
                            } else if (this.getErrorStrings().indexOf(s1) > -1) {
                                return s1;
                            } else {
                                d = this._parseDouble(s1);
                            }

                            max = Math.max(max, d);
                        }
                    }
                } else {
                    try {
                        s1 = this.getValueFromArg(ranges[r]);
                    } catch (ex) {
                        if (this._rethrowLibraryComputationExceptions && this.getLibraryComputationException() != null) {
                            throw this.getLibraryComputationException();
                            return ex;
                        }
                    }

                    if (s1.length > 0) {
                        d = 0;
                        if (s1.toUpperCase() == (this.trueValueStr)) {
                            d = 1;
                        } else if (this.getErrorStrings().indexOf(s1) == -1) {
                            return s1;
                        } else {
                            d = this._parseDouble(s1);
                        }

                        max = Math.max(max, d);
                    }
                }
            }
            if (max != this.minValue) {
                return max.toString();
            }

            return this._string_empty;
        };
        this.computeMedian = function (range) {
            var dd = this._getDoubleArray(range);
            dd.sort(function (a, b) {
                if (isNaN(a) || isNaN(b)) {
                    if (a > b) return 1;
                    else return -1;
                }
                return a - b;
            });
            var n = parseInt((dd.length / 2).toString());
            var s1 = "";
            if (dd.length % 2 == 1) {
                s1 = dd[n].toString();
            } else {
                s1 = ((dd[n] + dd[n - 1]) / 2).toString();
            }
            return s1;
        };
        this.computeMin = function (range) {
            var min = this.maxValue;
            var d;
            var s1;

            this.adjustRangeArg(range);
            var ranges = this.splitArgsPreservingQuotedCommas(range);
            var rang = range.split(';');
            if (rang.length > 1) {
                for (var r in rang) {
                    var valRange = this.splitArgsPreservingQuotedCommas(rang[r]);
                    for (var s in valRange) {
                        var tempval = this.getValueFromArg(valRange[s]);
                        if (tempval.length > 0) {
                            d = this._parseDouble(tempval);
                            if (!isNaN(d)) {
                                min = Math.min(min, d);
                            } else if (this.getErrorStrings().indexOf(tempval) == -1) {
                                return tempval;
                            }
                        }
                    }
                }
                if (min != this.maxValue) {
                    return min.toString();
                }
            }
            for (var r in ranges) {
                ////cell range
                if (ranges[r].indexOf(':') > -1) {
                    var cell = this.getCellsFromArgs(ranges[r]);
                    for (var s in cell) {
                        try {
                            s1 = this.getValueFromArg(cell[s]);
                        } catch (ex) {
                            if (this._rethrowLibraryComputationExceptions && this.getLibraryComputationException() != null) {
                                throw this.getLibraryComputationException();
                            }

                            return ex;
                        }

                        if (s1.length > 0) {
                            d = this._parseDouble(s1);
                            if (!isNaN(d)) {
                                min = Math.min(min, d);
                            } else if (this.getErrorStrings().indexOf(s1) == -1) {
                                return s1;
                            }
                        }
                    }
                } else {
                    try {
                        s1 = (ranges[r] == this._string_empty) ? "0" : this.getValueFromArg(ranges[r]);
                    } catch (ex) {
                        if (this._rethrowLibraryComputationExceptions && this.getLibraryComputationException() != null) {
                            throw this.getLibraryComputationException();
                        }

                        return ex;
                    }

                    if (s1.length > 0) {
                        d = this._parseDouble(s1);
                        if (!isNaN(d)) {
                            min = Math.min(min, d);
                        } else if (this.getErrorStrings().indexOf(s1) == -1) {
                            return s1;
                        }
                    }
                }
            }
            if (min != this.maxValue) {
                return min.toString();
            }

            return this._string_empty;
        };
        this.computeMina = function (range) {
            var min = this.maxValue;
            var d;
            var s1;

            this.adjustRangeArg(range);
            var ranges = this.splitArgsPreservingQuotedCommas(range);
            var rang = range.split(';');
            if (rang.length > 1) {
                for (var r in rang) {
                    var valRange = this.splitArgsPreservingQuotedCommas(rang[r]);
                    for (var s in valRange) {
                        var tempval = this.getValueFromArg(valRange[s]);
                        if (tempval.length > 0) {
                            d = 0;
                            if (tempval.toUpperCase() == (this.trueValueStr)) {
                                d = 1;
                            } else if (this.getErrorStrings().indexOf(tempval) == -1) {
                                return tempval;
                            } else {
                                d = this._parseDouble(tempval);
                            }

                            min = Math.min(min, d);
                        }
                    }
                }
                if (min != this.maxValue) {
                    return min.toString();
                }
            }
            for (var r in ranges) {
                ////cell range
                if (ranges[r].indexOf(':') > -1) {
                    var cell = this.getCellsFromArgs(ranges[r]);
                    for (var s in cell) {
                        try {
                            s1 = this.getValueFromArg(cell[s]);
                        } catch (ex) {
                            if (this._rethrowLibraryComputationExceptions && this.getLibraryComputationException() != null) {
                                throw this.getLibraryComputationException();
                            }

                            return ex;
                        }

                        if (s1.length > 0) {
                            d = 0;
                            if (s1.toUpperCase() == (this.trueValueStr)) {
                                d = 1;
                            } else if (this.getErrorStrings().indexOf(s1) == -1) {
                                return s1;
                            } else {
                                d = this._parseDouble(s1);
                            }

                            min = Math.min(min, d);
                        }
                    }
                } else {
                    try {
                        s1 = this.getValueFromArg(ranges[r]);
                    } catch (ex) {
                        if (this._rethrowLibraryComputationExceptions && this.getLibraryComputationException() != null) {
                            throw this.getLibraryComputationException();
                        }

                        return ex;
                    }

                    if (s1.length > 0) {
                        d = 0;
                        if (s1.toUpperCase() == (this.trueValueStr)) {
                            d = 1;
                        } else if (this.getErrorStrings().indexOf(s1) == -1) {
                            return s1;
                        } else {
                            d = this._parseDouble(s1);
                        }

                        min = Math.min(min, d);
                    }
                }
            }
            if (min != this.maxValue) {
                return min.toString();
            }

            return this._string_empty;
        };
       
        this.computePercentrankInc = function (range) {
            var args = this.splitArgsPreservingQuotedCommas(range);
            var argCount = args.length;
            if (argCount != 2 && argCount != 3) {
                if (this._rethrowLibraryComputationExceptions) {
                    throw this.formulaErrorStrings[this._wrong_number_arguments];
                }
                return this.formulaErrorStrings[this._wrong_number_arguments];
            }

            var x;
            var signif = 3;
            var s1 = this.getValueFromArg(args[1]);
            x = this._parseDouble(s1);
            if (isNaN(x)) {
                if (this._rethrowLibraryComputationExceptions) {
                    throw this.formulaErrorStrings[this._invalid_arguments];
                }
                return this.formulaErrorStrings[this._invalid_arguments];
            } else if (x == 0) {
            }

            if (argCount == 3) {
                s1 = this.getValueFromArg(args[2]);
                signif = this._parseDouble(s1);
                if (isNaN(signif) && signif < 1) {
                    if (this._rethrowLibraryComputationExceptions) {
                        throw this.formulaErrorStrings[this._invalid_arguments];
                    }
                    return this.formulaErrorStrings[this._invalid_arguments];
                }
            }
            var dd = this._getDoubleArray(args[0]);
            var n = dd.length;

            dd.sort();

            var d = 1;
            for (var i = 0; i < n; ++i) {
                if (dd[i] == x) {
                    var k = 0;
                    while (k + i < n && dd[k + i] == x) {
                        k++;
                    }

                    d = ((i - 1)) / (i + n - i - k - 1);

                    if (i > 0 && dd[i - 1] < x) {
                        var di = (i) / (n - 1);
                        d = di + (d - di) * (1 - (x - dd[i - 1]) / (dd[i] - dd[i - 1]));
                    }

                    break;
                }
            }
            var fmt = "0." + ('#', parseInt(signif.toString()));
            return d.toString();
        };
        
        this.computeRankOEq = function (range) {
            var args = this.splitArgsPreservingQuotedCommas(range);
            var argCount = args.length;
            if (argCount != 2 && argCount != 3) {
                return this.formulaErrorStrings[this._wrong_number_arguments];
            }

            var rank = 0;
            var x;
            var s1 = this.getValueFromArg(args[0]);
            if (args[0] == this._string_empty || args[1] == this._string_empty || !(this._isCellReference(args[1])))
                return this.getErrorStrings()[4].toString();
            x = this._parseDouble(s1);
            if (!isNaN(x)) {
                var order = 0;
                if (argCount == 3) {
                    s1 = this.getValueFromArg(args[2]);
                    if (s1 == this._string_empty)
                        s1 = "0";
                    else if (this._parseDouble(s1) > 1)
                        s1 = "1";
                    order = this._parseDouble(s1);
                }

                var r = args[1];

                var d = 0;
                var eq = false;
                if (r.indexOf(":") == -1)
                    return this.getErrorStrings()[4].toString();
                if (r.indexOf(':') > -1) {
                    for (var s in this.getCellsFromArgs(r)) {
                        try {
                            s1 = this.getValueFromArg(s);
                        } catch (ex) {
                            if (this._rethrowLibraryComputationExceptions && this.getLibraryComputationException() != null) {
                                throw this.getLibraryComputationException();
                            }

                            return ex;
                        }
                        d = this._parseDouble(s1);
                        if (!isNaN(d)) {
                            if (order == 1) {
                                if (d < x) {
                                    rank += 1;
                                } else if (d == x) {
                                    eq = true;
                                }
                            } else {
                                if (d > x) {
                                    rank += 1;
                                } else if (d == x) {
                                    eq = true;
                                }
                            }
                        }
                    }

                    if (eq) {
                        rank += 1;
                    }
                }
            }
            return rank.toString();
        };
        this.computePercentile = function (range) {
            var args = this.splitArgsPreservingQuotedCommas(range);
            var argCount = args.length;
            if (argCount != 2) {
                return this.formulaErrorStrings[this._wrong_number_arguments];
            }

            var k;
            var s1 = this.getValueFromArg(args[1]);
            k = this._parseDouble(s1);
            if (isNaN(k) && (k < 0 || k > 1)) {
                return this.formulaErrorStrings[this._invalid_arguments];
            }

            var dd = this._getDoubleArray(args[0]);
            var n = dd.length;

            dd.sort();

            var h = 1 / (n - 1);
            var d = dd[n - 1];
            for (var i = 0; i < n - 1; ++i) {
                if ((i + 1) * h > k) {
                    k = (k - i * h) / h;
                    d = dd[i] + k * (dd[i + 1] - dd[i]);
                    break;
                }
            }
            return d.toString();
        };
        this.computePoissonODist = function (argList) {
            var args = this.splitArgsPreservingQuotedCommas(argList);
            var argCount = args.length;
            if (argCount != 3) {
                return this.formulaErrorStrings[this._wrong_number_arguments];
            }

            var x;
            var u;
            var cum = 0;
            var dist = 0;

            for (var i = 0; i < argCount; ++i) {
                args[i] = this.getValueFromArg(args[i]);
            }
            x = this._parseDouble(args[0]);
            u = this._parseDouble(args[1]);
            if (!isNaN(x) && !isNaN(u)) {
                cum = (args[2] == this.trueValueStr) ? 1 : 0;
                var n = x;
                if (cum == 0) {
                    var prod = 1;
                    for (var i = 2; i <= n; ++i) {
                        prod *= i;
                    }
                    dist = Math.exp(-u) * Math.pow(u, n) / prod;
                } else {
                    var prod = 1;
                    dist = 0;
                    var ui = 1;
                    for (var i = 0; i <= n; ++i) {
                        dist += ui / prod;
                        prod *= i + 1;
                        ui *= u;
                    }

                    dist = Math.exp(-u) * dist;
                }
            }
            return dist.toString();
        };
        this.computeWeiBullODist = function (argList) {
            var args = this.splitArgsPreservingQuotedCommas(argList);
            var argCount = args.length;

            if (argCount != 4) {
                if (this._rethrowLibraryComputationExceptions)
                    throw this.formulaErrorStrings[this._wrong_number_arguments];
                return this.formulaErrorStrings[this._wrong_number_arguments];
            }

            var x;
            var alpha;
            var beta;
            var cumulative = 0;
            var cumBool = true;

            for (var i = 0; i < argCount; ++i)
                args[i] = this.getValueFromArg(args[i]).split(this.tic).join(this._string_empty);
            x = this._parseDouble(args[0]);
            alpha = this._parseDouble(args[1]);
            beta = this._parseDouble(args[2]);
            if (!isNaN(x) && !isNaN(alpha) && !isNaN(beta)) {
                if (x < 0 || alpha <= 0 || beta <= 0) {
                    if (this._rethrowLibraryComputationExceptions)
                        throw "Passed argument value is below 0";
                    return this.getErrorStrings()[4].toString();
                }
                if (alpha.toString().length >= 5 && beta.toString().length >= 3) {
                    if (this._rethrowLibraryComputationExceptions)
                        throw "Passed argument length exceeded the minimum length";
                    return this.getErrorStrings()[4].toString();
                }
                if (argCount != 3) {
                    cumulative = (args[3] == this.trueValueStr) ? 1 : 0;
                    var check = 0;
                    check = this._parseDouble(args[3]);
                    if (!isNaN(check))
                        cumulative = 1;
                }

                cumBool = (cumulative == 1) ? true : false;
                cumulative = ((cumBool) ? 1 - Math.exp(-Math.pow(x / beta, alpha)) : Math.pow(x, alpha - 1) * Math.exp(-Math.pow(x / beta, alpha)) * alpha / Math.pow(beta, alpha));
            } else {
                if (this._rethrowLibraryComputationExceptions)
                    throw "Passed argument value is non-numerical";
                return this.getErrorStrings()[1].toString();
            }
            return cumulative.toString();
        };
        this.computeFOinvOrt = function (argList) {
            var args = this.splitArgsPreservingQuotedCommas(argList);
            var argCount = args.length;
            if (argCount != 3) {
                if (this._rethrowLibraryComputationExceptions) {
                    throw this.formulaErrorStrings[this._wrong_number_arguments];
                }
                return this.formulaErrorStrings[this._wrong_number_arguments];
            }

            var p;
            var df1;
            var df2;
            var invdist = 0;

            for (var i = 0; i < argCount; ++i) {
                args[i] = this.getValueFromArg(args[i]);
            }
            p = this._parseDouble(args[0]);
            df1 = this._parseDouble(args[1]);
            df2 = this._parseDouble(args[2]);

            if (!isNaN(p) && (p > 0 && p < 1) && !isNaN(df1) && !isNaN(df2)) {
                invdist = this._finv(p, df1, df2);
            }

            if (invdist <= 0) {
                if (this._rethrowLibraryComputationExceptions) {
                    throw this.formulaErrorStrings[this._iterations_dont_converge];
                }
                return this.formulaErrorStrings[this._iterations_dont_converge];
            }
            return invdist.toString();
        };
        this.computeTOdist = function (argList) {
            var args = this.splitArgsPreservingQuotedCommas(argList);
            var argCount = args.length;
            if (argCount != 3) {
                return this.formulaErrorStrings[this._wrong_number_arguments];
            }

            var x;
            var df1 = 0;
            var dist = 0;

            for (var i = 0; i < argCount; ++i) {
                args[i] = this.getValueFromArg(args[i]).split(this.tic).join(this._string_empty);
            }
            x = this._parseDouble(args[0]);
            df1 = this._parseDouble(args[1]);
            if (!isNaN(x) && !isNaN(df1)) {
                if (df1 < 1 || (x[0] == ("-"))) {
                    if (this._rethrowLibraryComputationExceptions) {
                        throw "#NUM! Passed argument value is incorrect";
                    }
                    return this.getErrorStrings()[4].toString();
                }

                if (args[2] == this.trueValueStr) {
                    dist = this._tCumulativeDensity(x, df1);
                    //dist = StudentsTCumulativeDistributionFunctionInverse(x, (int)u);
                } else if (args[2] == this.falseValueStr) {
                    dist = this._tProbabilityDensity(x, df1);
                }
            } else {
                if (this._rethrowLibraryComputationExceptions) {
                    throw "#VALUE! Passed argument value is incorrect";
                }
                return this.getErrorStrings()[1].toString();
            }

            return dist.toString();
        };
        this.computeAvedev = function (range) {
            var sum = 0, s1, d, ranges;
            var x = [];
            ranges = this.splitArgsPreservingQuotedCommas(range);
            var adjustRange;
            for (var r in ranges) {
                adjustRange = ranges[r];

                ////is a cellrange
                if (adjustRange.indexOf(':') > -1) {
                    var cellCollection = this.getCellsFromArgs(adjustRange);
                    for (var s in cellCollection) {
                        try {
                            s1 = this.getValueFromArg(cellCollection[s]).split(this.tic).join(this._string_empty);
                        } catch (ex) {
                            if (this._rethrowLibraryComputationExceptions && this.getLibraryComputationException() != null) {
                                throw this.getLibraryComputationException();
                            }
                            return ex;
                        }

                        if (s1.length > 0) {
                            d = this._parseDouble(s1);
                            if (!isNaN(d)) {
                                sum = sum + d;
                                x.push(d);
                            } else if ((this.getErrorStrings().indexOf(s1)) > 0) {
                                return s1;
                            }
                        }
                    }
                } else {
                    try {
                        s1 = this.getValueFromArg(adjustRange).split(this.tic).join(this._string_empty);
                    } catch (ex) {
                        if (this._rethrowLibraryComputationExceptions && this.getLibraryComputationException() != null) {
                            throw this.getLibraryComputationException();
                        }

                        return ex;
                    }
                    if (s1.length > 0) {
                        d = this._parseDouble(s1);
                        if (!isNaN(d)) {
                            sum = sum + d;
                            x.push(d);
                        } else if ((this.getErrorStrings().indexOf(s1) == -1)) {
                            return s1;
                        }
                    }
                }
            }

            if (x.length > 0) {
                var ave = sum / x.length;
                sum = 0;
                for (var i = 0; i < x.length; ++i) {
                    sum = sum + Math.abs((x[i]) - ave);
                }

                sum = sum / x.length;
            }
            return sum.toString();
        };
        this.computeTOinv = function (argList) {
            var args = this.splitArgsPreservingQuotedCommas(argList);
            var argCount = args.length;
            if (argCount != 2) {
                return this.formulaErrorStrings[this._wrong_number_arguments];
            }

            var x;
            var df1 = 0;
            var dist = 0;

            for (var i = 0; i < argCount; ++i) {
                args[i] = this.getValueFromArg(args[i]).split(this.tic).join(this._string_empty);
            }
            x = this._parseDouble(args[0]);
            df1 = this._parseDouble(args[1]);
            if (!isNaN(x) && !isNaN(df1)) {
                if (df1 < 1 || x < 0 || (x[0] == ("-"))) {
                    if (this._rethrowLibraryComputationExceptions) {
                        throw "#NUM! Passed argument value is incorrect";
                    }
                    return this.getErrorStrings()[4].toString();
                }
                dist = this._tCumulativeDistributionInverse(x, df1);
            } else {
                if (this._rethrowLibraryComputationExceptions) {
                    throw "#VALUE! Passed argument value is incorrect";
                }
                return this.getErrorStrings()[1].toString();
            }
            return dist.toString();
        };
       
        this.computeChisqOinv = function (argList) {
            var args = this.splitArgsPreservingQuotedCommas(argList);
            var argCount = args.length;
            if (argCount != 2) {
                return this.formulaErrorStrings[this._wrong_number_arguments];
            }

            var p;
            var v;
            var dist = 0;

            for (var i = 0; i < argCount; ++i) {
                args[i] = this.getValueFromArg(args[i]);
            }

            p = this._parseDouble(args[0]);
            v = this._parseDouble(args[1]);
            if (!isNaN(p) && !isNaN(v)) {
                dist = this._chiinv(p, v);
                return dist.toString();
            }
        };
        this.computeCount = function (range) {
            var count = 0;
            var s1;
            var d;
            var dt = new Date;

            var ranges = this.splitArgsPreservingQuotedCommas(range);
            for (var r in ranges) {
                ////is a cellrange
                if (ranges[r].indexOf(':') > -1) {
                    var cell = this.getCellsFromArgs(ranges[r].split(this.tic).join(""));
                    for (var s in cell) {
                        try {
                            s1 = this.getValueFromArg(cell[s]);
                        } catch (Exception) {
                            if (this._rethrowLibraryComputationExceptions)
                                throw this.getLibraryComputationException();
                            return this.getErrorStrings()[4].toString();
                        }

                        if (s1.length > 0) {
                            if (s1 == (this.formulaErrorStrings[19])) {
                                if (this.geLibraryComputationException() != null)
                                    throw this.getLibraryComputationException();
                                return this.formulaErrorStrings[19];
                            }
                            d = this._parseDouble(s1);
                            dt = new Date(Date.parse(s1));
                            if ((!isNaN(d) || dt.toString() != "Invalid Date") && this.getErrorStrings().indexOf(s1) == -1) {
                                count++;
                            }
                        }
                    }
                } else {
                    try {
                        if (ranges[r] == this._string_empty && !(ranges[r][0] == (this.tic)))
                            count++;
                        s1 = this.getValueFromArg(r.split(this.tic).join(""));
                    } catch (Exception) {
                        if (this.getLibraryComputationException() != null)
                            throw this.getLibraryComputationException();
                        throw this.getErrorStrings()[4].toString();
                    }

                    if (s1.length > 0) {
                        if (s1[0] == (this.formulaErrorStrings[19])) {
                            if (this.getLibraryComputationException() != null)
                                throw this.getLibraryComputationException();
                            return this.formulaErrorStrings[19];
                        }
                        d = this._parseDouble(s1);
                        dt = new Date(Date.parse(s1));
                        if ((!isNaN(d) || dt.toString() != "Invalid Date") && this.getErrorStrings().indexOf(s1) == -1) {
                            count++;
                        }
                    }
                }
            }
            return count.toString();
        };
        this.computeFOdistORt = function (argList) {
            var args = this.splitArgsPreservingQuotedCommas(argList);
            var argCount = args.length;
            if (argCount != 3) {
                return this.formulaErrorStrings[this._wrong_number_arguments];
            }

            var x;
            var df1 = 0;
            var df2 = 0;
            var dist = 0;

            for (var i = 0; i < argCount; ++i) {
                args[i] = this.getValueFromArg(args[i]).split(this.tic).join("");
            }
            x = this._parseDouble(args[0]);
            df1 = this._parseDouble(args[1]);
            df2 = this._parseDouble(args[2]);
            if (!isNaN(x) && !isNaN(df1) && !isNaN(df2)) {
                var mult = Math.exp(this._gammaln((df1 + df2) / 2) - this._gammaln(df1 / 2) - this._gammaln(df2 / 2) + (df1 / 2) * Math.log(df1 / df2));

                dist = 1 - mult * this._fdist(x, parseInt(df1.toString()), parseInt(df2.toString()));
            } else {
                if (this._rethrowLibraryComputationExceptions)
                    throw this.formulaErrorStrings[this.invalid_Math_argument];
                return this.this._formulaErrorStrings[this._invalid_Math_argument];
            }
            return dist.toString();
        };
        this.computeGeomean = function (range) {
            var sum = 1;
            var s1;
            var d;
            var count = 0;

            this.adjustRangeArg(range);
            var ranges = this.splitArgsPreservingQuotedCommas(range);
            for (var r in ranges) {
                ////is a cellrange
                if (ranges[r].indexOf(':') > -1) {
                    var cell = this.getCellsFromArgs(ranges[r]);
                    for (var s in cell) {
                        try {
                            s1 = this.getValueFromArg(cell[s]);
                        } catch (ex) {
                            if (this._rethrowLibraryComputationExceptions && this.getLibraryComputationException() != null) {
                                throw this.getLibraryComputationException();
                            }

                            return ex;
                        }

                        if (s1.length > 0) {
                            d = this._parseDouble(s1);
                            if (!isNaN(d)) {
                                count++;
                                sum = sum * d;
                            } else if (this.getErrorStrings().indexOf(s1) == -1) {
                                return s1;
                            }
                        }
                    }
                } else {
                    try {
                        s1 = this.getValueFromArg(ranges[r]);
                    } catch (ex) {
                        if (this._rethrowLibraryComputationExceptions && this.getLibraryComputationException() != null) {
                            throw this.getLibraryComputationException();
                        }

                        return ex;
                    }

                    if (s1.length > 0) {
                        d = this._parseDouble(s1);
                        if (!isNaN(d)) {
                            count++;
                            sum = sum * d;
                        } else if (this.getErrorStrings().indexOf(s1) == -1) {
                            return s1;
                        }
                    }
                }
            }

            if (count > 0) {
                sum = Math.pow(sum, 1 / count);
            }
            return sum.toString();
        };
        this.computeHarmean = function (range) {
            var sum = 0;
            var s1;
            var d;
            var count = 0;

            this.adjustRangeArg(range);
            var ranges = this.splitArgsPreservingQuotedCommas(range);
            for (var r in ranges) {
                ////is a cellrange
                if (ranges[r].indexOf(':') > -1) {
                    var cell = this.getCellsFromArgs(ranges[r]);
                    for (var s in cell) {
                        try {
                            s1 = this.getValueFromArg(cell[s]);
                        } catch (ex) {
                            if (this._rethrowLibraryComputationExceptions && this.getLibraryComputationException() != null) {
                                throw this.getLibraryComputationException();
                            }

                            return ex;
                        }

                        if (s1.length > 0) {
                            d = this._parseDouble(s1);
                            if (!isNaN(d) && d != 0) {
                                count++;
                                sum = sum + 1 / d;
                            } else if (this.getErrorStrings().indexOf(s1) == -1) {
                                return s1;
                            }
                        }
                    }
                } else {
                    try {
                        s1 = this.getValueFromArg(ranges[r]);
                    } catch (ex) {
                        if (this._rethrowLibraryComputationExceptions && this.getLibraryComputationException() != null) {
                            throw this.getLibraryComputationException();
                        }

                        return ex;
                    }

                    if (s1.length > 0) {
                        d = this._parseDouble(s1);
                        if (!isNaN(d) && d != 0) {
                            count++;
                            sum = sum + 1 / d;
                        } else if (this.getErrorStrings().indexOf(s1)) {
                            return s1;
                        }
                    }
                }
            }

            if (count > 0) {
                sum = (count) / sum;
            }
            return sum.toString();
        };
        this.computeIntercept = function (range) {
            var args = this.splitArgsPreservingQuotedCommas(range);
            var argCount = args.length;
            if (argCount != 2) {
                return this.formulaErrorStrings[this._wrong_number_arguments];
            }

            var y = this._getDoubleArray(args[0]);
            var x = this._getDoubleArray(args[1]);
            var n = x.length;

            if (n <= 0 || n != y.length) {
                return this.formulaErrorStrings[this._wrong_number_arguments];
            }

            var sumx = 0;
            var sumy = 0;
            for (var i = 0; i < n; ++i) {
                sumx += x[i];
                sumy += y[i];
            }

            sumx = sumx / n;
            sumy = sumy / n;

            var sumxy = 0;
            var sumx2 = 0;
            var d;
            for (var i = 0; i < n; ++i) {
                d = x[i] - sumx;
                sumxy += d * (y[i] - sumy);
                sumx2 += d * d;
            }
            return (sumy - sumxy / sumx2 * sumx).toString();
        };
        this.computeCountblank = function (range) {
            var count = 0;
            var s1;

            this.adjustRangeArg(range);
            var ranges = this.splitArgsPreservingQuotedCommas(range);
            var arglist;
            if (this._isTextEmpty(range)) {
                if (this._rethrowLibraryComputationExceptions)
                    throw this.formulaErrorStrings[this.invalid_arguments];
                else
                    return this.formulaErrorStrings[this.invalid_arguments];
            }
            for (var r in ranges) {
                arglist = ranges[r];

                ////is a cellrange
                if (arglist.indexOf(':') > -1) {
                    var cellCollection = this.getCellsFromArgs(arglist);
                    for (var s in cellCollection) {
                        try {
                            s1 = this.getValueFromArg(cellCollection[s]);
                        } catch (Exception) {
                            if (this._rethrowLibraryComputationExceptions && this.getLibraryComputationException() != null) {
                                throw this.getLibraryComputationException();
                            }

                            return this.getErrorStrings()[4].toString();
                        }

                        if (s1 == this._string_empty) {
                            count++;
                        }
                    }
                } else {
                    try {
                        s1 = this.getValueFromArg(ranges);
                    } catch (Exception) {
                        if (this._rethrowLibraryComputationExceptions && this.getLibraryComputationException() != null) {
                            throw this.getLibraryComputationException();
                        } else
                            return this.getErrorStrings()[4].toString();
                    }
                    if (!this._isCellReference(arglist)) {
                        var temp = this._parseDouble(s1);

                        if (!isNaN(temp) || this.getErrorStrings.indexOf(s1)) {
                            if (this._rethrowLibraryComputationExceptions)
                                throw this.formulaErrorStrings[this.invalid_arguments].toString();
                            else
                                return this.formulaErrorStrings[this.invalid_arguments].toString();
                        }
                        if (!(s1.split(this.tic).join(this._string_empty) == (this._string_empty))) {
                            if (this._rethrowLibraryComputationExceptions)
                                throw this.getErrorStrings()[5].toString();
                            else
                                return this.getErrorStrings()[5].toString();
                        } else {
                            if (this._rethrowLibraryComputationExceptions)
                                throw this.formulaErrorStrings[this.invalid_arguments].toString();
                            else
                                return this.formulaErrorStrings[this.invalid_arguments].toString();
                        }
                    }
                    if (s1 == this._string_empty) {
                        count++;
                    }
                }
            }
            return count.toString();
        };
        this.computeDevsq = function (range) {
            var d;
            var ranges = this.splitArgsPreservingQuotedCommas(range);
            var x = this._getDoubleArray(range);
            var n = x.length;
            if (ranges.length != 1 || range == "") {
                return this.formulaErrorStrings[this._wrong_number_arguments];
            }
            if (x.length <= 0)
            {
                if (this.getLibraryComputationException() != null)
                    throw this.formulaErrorStrings[this._bad_formula];
                throw this.getErrorStrings()[4].toString();
            }
            var sumx = 0;
            for (var i = 0; i < n; ++i) {
                sumx = sumx + x[i];
            }

            sumx = sumx / n;
            var sumx2 = 0;

            for (var i = 0; i < n; ++i) {
                d = x[i] - sumx;
                sumx2 = sumx2 + d * d;
            }
            return sumx2.toString();
        };
        this.computeForecast = function (range) {
            var args = this.splitArgsPreservingQuotedCommas(range);
            var argCount = args.length;
            if (argCount != 3) {
                return this.formulaErrorStrings[this._wrong_number_arguments];
            }

            var x0, d;
            var s1 = this.getValueFromArg(args[0]);
            x0 = this._parseDouble(s1);
            if (isNaN(x0)) {
                return this.formulaErrorStrings[this._invalid_arguments];
            }
            var y, x;
            y = this._getDoubleArray(args[1]);
            x = this._getDoubleArray(args[2]);
            var n = x.length;

            if (n <= 0 || n != y.length) {
                return this.formulaErrorStrings[this._wrong_number_arguments];
            }

            var sumx = 0;
            var sumy = 0;
            for (var i = 0; i < n; ++i) {
                sumx = sumx + x[i];
                sumy = sumy + y[i];
            }

            sumx = sumx / n;
            sumy = sumy / n;

            var sumxy = 0;
            var sumx2 = 0;
            for (var i = 0; i < n; ++i) {
                d = x[i] - sumx;
                sumxy = sumxy + d * (y[i] - sumy);
                sumx2 = sumx2 + d * d;
            }

            var b = sumxy / sumx2;
            var a = sumy - b * sumx;
            return (a + b * x0).toString();
        };
        
        this.computeStdevOp = function (range) {
            var dd = this._getDoubleArrayA(range);
            var n = dd.length;

            if (n < 2) {
                if (this._rethrowLibraryComputationExceptions) {
                    throw this.formulaErrorStrings[this._invalid_arguments];
                }
                return this.formulaErrorStrings[this.invalid_arguments];
            }
            return this._stdevdotP(dd).toString();
        };
        this.computeStdevOS = function (range) {
            var dd = this._getDoubleArrayA(range);
            var n = dd.length;

            if (n < 2) {
                if (this._rethrowLibraryComputationExceptions) {
                    throw this.formulaErrorStrings[this._invalid_arguments];
                }
                return this.formulaErrorStrings[this.invalid_arguments];
            }
            var xbar = 0;
            return this._sd(dd, xbar).toString();
        };
        this.computeStdeva = function (range) {
            var dd = this._getDoubleArrayA(range);
            var n = dd.length;
            if (n < 2) {
                return this.formulaErrorStrings[this._invalid_arguments];
            }

            var xbar = 0;
            return this._sd(dd, xbar).toString();
        };
        this.computeStdevpa = function (range) {
            var dd = this._getDoubleArrayA(range);
            var n = dd.length;
            if (n < 2) {
                return this.formulaErrorStrings[this._invalid_arguments];
            }

            var xbar = 0;
            var sdev = this._sd(dd, xbar);
            return (sdev * Math.sqrt(n - 1) / Math.sqrt(n)).toString();
        };
        this.computeVarp = function (range) {
            var dd = this._getDoubleArray(range);
            var n = dd.length;
            return ((n - 1) * this._var(dd) / n).toString();
        };
        this.computeVara = function (range) {
            var dd = this._getDoubleArrayA(range);
            return this._var(dd).toString();
        };
        this.computeVarpa = function (range) {
            var dd = this._getDoubleArrayA(range);
            var n = dd.length;
            return ((n - 1) * this._var(dd) / n).toString();
        };
        this.computeCorrel = function (range) {
            var args = this.splitArgsPreservingQuotedCommas(range);
            var argCount = args.length;
            if (argCount != 2) {
                return this.formulaErrorStrings[this._wrong_number_arguments];
            }
            var y, x;
            y = this._getDoubleArray(args[0]);
            x = this._getDoubleArray(args[1]);
            var n = x.length;
            if (n <= 0 || n != y.length) {
                return this.formulaErrorStrings[this._wrong_number_arguments];
            }

            var sumx = 0;
            var sumy = 0;
            for (var i = 0; i < n; ++i) {
                sumx = sumx + x[i];
                sumy = sumy + y[i];
            }

            sumx = sumx / n;
            sumy = sumy / n;

            var sumxy = 0;
            var sumxb2 = 0;
            var sumyb2 = 0;
            var xb = 0;
            var yb = 0;

            for (var i = 0; i < n; ++i) {
                xb = x[i] - sumx;
                yb = y[i] - sumy;
                sumxy = sumxy + xb * yb;
                sumxb2 = sumxb2 + xb * xb;
                sumyb2 = sumyb2 + yb * yb;
            }

            //  var v = (sumxy / Math.Sqrt(sumxb2 * sumyb2)).toString();
            //        return v;
            return (sumxy / Math.sqrt(sumxb2 * sumyb2)).toString();
        };

        this.computeNegbinomODist = function (argList) {
            var args = this.splitArgsPreservingQuotedCommas(argList);
            var argCount = args.length;
            if (argCount != 4) {
                return this.formulaErrorStrings[this._wrong_number_arguments];
            }

            var successes;
            var failures;
            var p;
            var dist = 0;

            for (var i = 0; i < argCount; ++i) {
                args[i] = this.getValueFromArg(args[i]);
            }
            failures = this._parseDouble(args[0]);
            successes = this._parseDouble(args[1]);
            p = this._parseDouble(args[2]);
            if (!isNaN(failures) && !isNaN(successes) && !isNaN(p)) {
                dist = this._negbinomdensity(parseInt(failures.toString()), parseInt(successes.toString()), p);
            }
            return dist.toString();
        };

        this.computePercentileExc = function (range) {
            var args = this.splitArgsPreservingQuotedCommas(range);
            var argCount = args.length;
            if (argCount != 2) {
                if (this._rethrowLibraryComputationExceptions) {
                    throw this.formulaErrorStrings[this._wrong_number_arguments];
                }
                return this.formulaErrorStrings[this._wrong_number_arguments];
            }

            var k;
            var s1 = this.getValueFromArg(args[1]);
            k = this._parseDouble(s1);
            if (isNaN(k) && (k < 0 || k > 1)) {
                return this.formulaErrorStrings[this._invalid_arguments];
            }
            var ss = this._parseDouble(s1);
            if (ss == 0 || ss <= 0 || ss >= 1) {
                if (this._rethrowLibraryComputationExceptions) {
                    throw this.getErrorStrings()[4].toString();
                }
                return this.getErrorStrings()[4].toString();
            }
            var dd = this._getDoubleArray(args[0]);
            var n = dd.length;

            dd.sort();

            var num = dd.length;
            if (k <= 1 / (num + 1) && k >= num / (num + 1)) {
                if (this._rethrowLibraryComputationExceptions) {
                    throw this.getErrorStrings()[4].toString();
                }
                return this.getErrorStrings()[4].toString();
            }
            var l = k * (num + 1);
            return l.toString();
        };

        this.computePercentileOInc = function (range) {
            var args = this.splitArgsPreservingQuotedCommas(range);
            var argCount = args.length;
            if (argCount != 2) {
                if (this._rethrowLibraryComputationExceptions) {
                    throw this.formulaErrorStrings[this._wrong_number_arguments];
                }
                return this.formulaErrorStrings[this._wrong_number_arguments];
            }

            var k;
            var s1 = this.getValueFromArg(args[1]);
            k = this._parseDouble(s1);
            if (isNaN(k) && (k < 0 || k > 1)) {
                return this.formulaErrorStrings[this._invalid_arguments];
            }
            var ss = this._parseDouble(s1);
            if (ss == 0 || ss <= 0 || ss >= 1) {
                if (this._rethrowLibraryComputationExceptions) {
                    throw this.getErrorStrings()[4].toString();
                }
                return this.getErrorStrings()[4].toString();
            }
            var dd = this._getDoubleArray(args[0]);
            var n = dd.length;

            dd.sort();

            var h = 1 / (n - 1);
            var d = dd[n - 1];
            for (var i = 0; i < n - 1; ++i) {
                if ((i + 1) * h > k) {
                    k = (k - i * h) / h;
                    d = dd[i] + k * (dd[i + 1] - dd[i]);
                    break;
                }
            }
            return d.toString();
        };

        this.computeTrimmean = function (range) {
            var args = this.splitArgsPreservingQuotedCommas(range);
            var argCount = args.length;
            if (argCount != 2) {
                return this.formulaErrorStrings[this._wrong_number_arguments];
            }

            var percent;
            var s1 = this.getValueFromArg(args[1]);
            percent = this._parseDouble(s1);
            if (isNaN(percent)) {
                return this.formulaErrorStrings[this._invalid_arguments];
            }

            var dd = this._getDoubleArray(args[0]);
            var n = dd.length;
            var k = parseInt((percent * n).toString());
            k = parseInt(k / 2);
            if (k < 1 || 2 * k >= n) {
                return this.formulaErrorStrings[this._invalid_arguments];
            }
            dd.sort(function (a, b) {
                if (isNaN(a) || isNaN(b)) {
                    if (a > b) return 1;
                    else return -1;
                }
                return a - b;
            });
            var sum = 0;
            n = n - k;
            for (var i = k; i < n; ++i) {
                sum += dd[i];
            }
            return (sum / (n - k)).toString();
        };

        this.computeRsq = function (range) {
            try
            {
                var s = this.computePearson(range);
                var d = 0;
                d = this._parseDouble(s);
                if (!isNaN(d)) {
                    d = d * d;
                } else if (this.getErrorStrings().indexOf(s) == -1) {
                    return s;
                }
                return d.toString();
            }
            catch(ex)
            {
                if (this.getRethrowLibraryComputationExceptions())
                    throw this.formulaErrorStrings[this._invalid_Math_argument];
                return this.getErrorStrings()[5].toString();
            }
        };

        this.computeHLookUp = function (range) {
            var cachingEnabled = this._isHLookupCachingEnabled();
            var optimizedMatchesEnabled = this._isOptimizedMatchesEnabled();
            if (cachingEnabled && this._lookupTables == null)
                this.lookupTables = new HashTable();

            var s = this.splitArgsPreservingQuotedCommas(range);
            var lookUp = this.getValueFromArg(s[0]);
            lookUp = lookUp.split(this.tic).join("").toUpperCase();
            var r = s[1].split("\"").join("");
            var o1 = this.getValueFromArg(s[2]).split(this.tic).join("");
            var d = this._parseDouble(o1);
            if (isNaN(d)) {
                return "#N/A";
            }

            ////int row = int.Parse(o1);
            var row = parseInt(o1);

            var match = true;
            if (s.length == 4) {
                match = this.getValueFromArg(s[3]) == this.trueValueStr;
            }

            d = this._parseDouble(lookUp);
            var typeIsNumber = match ? !isNaN(d) : false;

            var i = r.indexOf(":");

            ////single cell
            if (i == -1) {
                r = r + ":" + r;
                i = r.indexOf(":");
            }

            var k = r.substring(0, i).lastIndexOf(this.sheetToken);
            var grd = this.grid;
            var family = CalcEngine.getSheetFamilyItem(this.grid);

            if (k > -1) {
                this.grid = family.tokenToParentObject.getItem(r.substring(0, k + 1));
            }

            var row1 = this.rowIndex(r.substring(0, i));
            var row2 = this.rowIndex(r.substring(i + 1));
            if (!(row1 != -1 || row2 == -1) == (row1 == -1 || row2 != -1)) {
                return this.getErrorStrings()[5].toString();
            }

            var col1 = this.colIndex(r.substring(0, i));
            var col2 = this.colIndex(r.substring(i + 1));

            var newTable = false;
            var key = "";
            var lookUpTable = null;
            var matchedTable = null;
            if (cachingEnabled) {
                key = row1.toString() + "_" + col1.toString() + "_" + col2.toString() + "_" + this.grid.GetHashCode();

                if (!this._lookupTables.containsKey(key)) {
                    if (optimizedMatchesEnabled) {
                        var _lookUp = new LookUps();
                        _lookUp.setMatchLookUpList(new HashTable());
                        this._lookupTables.add(key, _lookUp);
                    } else {
                        var _lookUp = new LookUps();
                        this._lookupTables.add(key, _lookUp);
                    }
                    newTable = true;
                }
                lookUpTable = this._lookupTables.getItem(key).getLinearLookUpList();
                if (optimizedMatchesEnabled) {
                    matchedTable = this._lookupTables.getItem(key).getMatchLookUpList();
                }
            }

            var val = "";
            var lastCol = col1;
            var s1 = "";
            var d1 = 0;
            var doLastColMark = true;
            var exactMatch = false;
            if (cachingEnabled && optimizedMatchesEnabled && matchedTable.containsKey(lookUp)) {
                lastCol = matchedTable.getItem(lookUp);
                s1 = lookUp;
            } else {
                for (var col = col1; col <= col2; ++col) {
                    if (!cachingEnabled || col - col1 >= lookUpTable.length || newTable) {
                        s1 = this.getValueFromParentObject(this.grid, row1, col).toString().toUpperCase().split("\"").join("");
                        if (cachingEnabled) {
                            lookUpTable.push(s1);
                            if (optimizedMatchesEnabled) {
                                matchedTable.add(s1, col);
                            }
                        }
                    } else {
                        s1 = lookUpTable[col - col1];
                    }
                    d1 = (typeIsNumber) ? this._parseDouble(s1) : d1;
                    if (s1 == lookUp || (match && (typeIsNumber ? (!isNaN(d1) && d1 > d) : (s1 > lookUp)))) {
                        if (s1 == lookUp) {
                            lastCol = col;
                            match = true;
                            exactMatch = true;
                        }

                        if (!newTable)
                            break;
                        else
                            doLastColMark = false;
                    }
                    if (doLastColMark)
                        lastCol = col;
                    match = newTable = true;
                }
            }

            if (match || s1 == lookUp) {
                if (!exactMatch && !typeIsNumber) {
                    this.grid = grd;
                    return "#N/A";
                }
                if ((row + row1 - 1) > row2)
                    return "#REF!";
                val = this.getValueFromParentObject(this.grid, row + row1 - 1, lastCol).toString();
                if (val.length > 0 && val[0] == this.getFormulaCharacter()) {
                    val = this.parseFormula(val);
                }

                d = this._parseDouble(val);
                if (val.length > 0 && val[0] != this.tic[0] && isNaN(d)) {
                    val = this.tic + val + this.tic;
                }
            } else {
                val = "#N/A";
            }

            ////val = "\"#N/A\"";
            this.grid = grd;

            return val;
        };
        this.computeIndex = function (arg) {
            var args = this.splitArgsPreservingQuotedCommas(arg);
            var argCount = args.length;

            if (argCount < 2) {
                if (this.getRethrowLibraryComputationExceptions())
                    throw this.formulaErrorStrings[this._wrong_number_arguments];
                return this.formulaErrorStrings[this._wrong_number_arguments];
            }
            var r = args[0];
            r = r.split(this.tic).join("");
            var i = r.indexOf(":");

            ////single cell
            if (i == -1) {
                if (this._isCellReference(r)) {
                    r = r + ":" + r;
                } else {
                    if (r.indexOf(";") > -1)
                    {
                        var rowNumber = parseInt(this.getValueFromArg(args[1]));
                        var arr = this._splitArguments(r, ';');
                        var colInd = (argCount == 3) ? parseInt(this.getValueFromArg(args[2])) : 1;
                        try
                        {
                            rowNumber = (rowNumber == 0) ? 1 : rowNumber;
                            colInd = (colInd == 0) ? 1 : colInd;
                            var colArray = this.splitArgsPreservingQuotedCommas(arr[rowNumber -1]);
                            return colArray[colInd -1];
                        }
                        catch(ex)
                        {
                            if (this.getRethrowLibraryComputationExceptions())
                                throw this.formulaErrorStrings[this._invalid_Math_argument];
                            return this.getErrorStrings()[2].toString();
                        }
                    }
                    var indexArray = this.splitArgsPreservingQuotedCommas(r);
                    if (!isNaN(rowNumber) && rowNumber > 0) {
                        if (indexArray.length >= rowNumber) {
                            return (parseInt(rowNumber.toString()) == 0) ? indexArray[0] : indexArray[parseInt(rowNumber.toString()) - 1];
                        } else {
                            if (this.getRethrowLibraryComputationExceptions())
                                throw this.formulaErrorStrings[this._invalid_arguments];
                            return "#REF";
                        }
                    } else {
                        if (this.getRethrowLibraryComputationExceptions())
                            throw this.formulaErrorStrings[this._invalid_arguments];
                        return this.getErrorStrings()[1].toString();
                    }
                }
            }
            i = r.indexOf(":");
            if (arg.indexOf("#N/A") > -1 || arg.indexOf("#N~A") > -1) {
                return "#N/A";
            }
            if (arg.indexOf("#DIV/0!") > -1 || arg.indexOf("#DIV~0!") > -1) {
                return "#DIV/0!";
            }
            var sheet = this._getSheetTokenFromReference(r);
            args[1] = (argCount == 1 || args[1] == "") ? "1" : args[1];
            args[2] = (argCount <= 2 || args[2] == "") ? "1" : args[2];
            var d = parseInt(this.getValueFromArg(args[1]));
            var row = !isNaN(d) ? d: -1;
            d = parseInt(this.getValueFromArg(args[2]));
            var col = !isNaN(d) ? d : -1;
            if (row == -1 || col == -1) {
                return "#REF";
            }

            var top = this.rowIndex(r.substring(0, i));
            var bot = this.rowIndex(r.substring(i + 1));
            if (!(top != -1 || bot == -1) == (top == -1 || bot != -1)) {
                return this.getErrorStrings()[5].toString();
            }

            var left = this.colIndex(r.substring(0, i));
            var right = this.colIndex(r.substring(i + 1));

            if (row > bot - top + 1 || col > right - left + 1) {
                return "#REF";
            }

            row = this.rowIndex(r.substring(0, i)) + row - 1;

            col = this.colIndex(r.substring(0, i)) + col - 1;

            var argsVal = sheet + RangeInfo.getAlphaLabel(col) + row;
            var result = this.getValueFromArg(argsVal);
            if (this._isTextEmpty(result))
                return "0";
            return result;
        };
        this.computeIndirect = function (args) {
            var arg = this.splitArgsPreservingQuotedCommas(args);
            var argCount = arg.length;

            if (argCount > 2 || argCount == 0) {
                if (this.getRethrowLibraryComputationExceptions())
                    throw this.formulaErrorStrings[this._wrong_number_arguments];
                return this.formulaErrorStrings[this._wrong_number_arguments];
            }

            arg[0] = arg[0].toUpperCase();
            arg[0] = this.setTokensForSheets(arg[0]);
            var sheetToken1 = this._sheetToken(arg[0].split(this.tic).join(""));

            ////Remove SheetToken
            if (sheetToken1 != "") {
                arg[0] = arg[0].split(sheetToken1).join("");
            }

            if (arg.length == 2 && arg[1] == this.falseValueStr) {
                var hasTIC = arg[0][0] == this.tic && arg[0][args[0].length - 1] == this.TIC;
                var rcCell = (arg[0].toUpperCase()).split(this.tic).join("");
                var cells = rcCell.split(':');

                var rc = (cells[0].split("R").join("C")).split("C");
                if (cells.length > 2) {
                    return this.getErrorStrings()[2].toString();
                }

                arg[0] = RangeInfo.getAlphaLabel(parseInt(rc[1])) + rc[0];

                if (cells.length == 2) {
                    var st = this._sheetToken(cells[1]);
                    if (st != null || st != "") {
                        cells[1] = cells[1].split(st).join("");
                    }

                    rc = (cells[0].split("R").join("C")).split("C");
                    arg[0] += ":" + RangeInfo.getAlphaLabel(parseInt(rc[1])) + rc[0];
                }

                if (hasTIC) {
                    arg[0] = this.TIC + arg[0] + this.TIC;
                }
            }

            var cellReference = "";

            ////String
            if (arg[0][0] == this.tic) {
                cellReference = sheetToken1 + arg[0].split(this.tic).join("");
            } else {
                if (this._isCellReference(arg[0])) {
                    var scopedRange = "";
                    cellReference = this.getValueFromArg(sheetToken1 + arg[0]);
                    if (cellReference == null || cellReference == "") {
                        return "0";
                    }
                    if (this._isInteriorFunction) {
                        this._isInteriorFunction = !this._isInteriorFunction;
                        return arg[0].split(this._string_fixedreference).join("");
                    }

                    scopedRange = this._checkIfScopedRange(cellReference.toUpperCase());
                    if (this.getNamedRanges().containsKey(cellReference.toUpperCase())) {
                        cellReference = this.getNamedRanges().getItem(cellReference.toUpperCase());
                        cellReference = cellReference.toUpperCase();
                        cellReference = cellReference.split(this._string_fixedreference).join("");
                        cellReference = this.setTokensForSheets(cellReference);
                        if (cellReference[0] != this.sheetToken) {
                            cellReference = sheetToken1 + cellReference;
                        }
                    } else if (scopedRange != "NaN") {
                        cellReference = scopedRange;
                        cellReference = cellReference.toUpperCase();
                        cellReference = cellReference.split(this._string_fixedreference).join("");
                        cellReference = this.setTokensForSheets(cellReference);
                        if (cellReference[0] != this.sheetToken) {
                            cellReference = sheetToken1 + cellReference;
                        }
                    } else if (this._isCellReference(cellReference)) {
                        return this.getValueFromArg(cellReference);
                    } else if (cellReference.indexOf("Invalid" == 0))
                        return this.getErrorStrings()[2].toString();
                    else {
                        return cellReference;
                    }
                } else {
                    cellReference = arg[0];
                }
            }

            if (!this._isCellReference(cellReference.split(this._string_fixedreference).join(""))) {
                var scopedRange = "";
                scopedRange = this._checkIfScopedRange(cellReference.toUpperCase());
                if (this.getNamedRanges().containsKey(cellReference.toUpperCase())) {
                    cellReference = this.getNamedRanges().getItem(cellReference.toUpperCase());
                    if (cellReference[0] != this.sheetToken) {
                        cellReference = sheetToken1 + cellReference;
                    }
                } else if (scopedRange != "NaN") {
                    cellReference = scopedRange;
                    if (cellReference[0] != this.sheetToken) {
                        cellReference = sheetToken1 + cellReference;
                    }
                } else {
                    return this.getErrorStrings()[2].toString();
                }
            }

            if (cellReference.indexOf(":") > -1 && this._isInteriorFunction) {
                this._isInteriorFunction = !this._isInteriorFunction;

                return cellReference.split(this._string_fixedreference).join("");
            }
            return this.getValueFromArg(cellReference);
        };
        this.computeLookUp = function (range) {
            var cachingEnabled = this._isHLookupCachingEnabled();
            var optimizedMatchesEnabled = this._isOptimizedMatchesEnabled();
            if (cachingEnabled && this._lookupTables == null)
                this.lookupTables = new HashTable();

            var s = this.splitArgsPreservingQuotedCommas(range);
            var lookUp = this.getValueFromArg(s[0]);
            lookUp = lookUp.split(this.tic).join("").toUpperCase();

            var finalResult = "";
            var r = s[1].split("\"").join("");
            s[2] = (s.length = 3) ? s[2].split(this.tic).join("") : s[2];
            var tempLookUp = this.getValueFromArg(s[0]);
            var cellValue;
            var resultValue;

            var o1 = this.getValueFromArg(s[2]).split(this.tic).join("");
            var d = this._parseDouble(o1);
            if (isNaN(d)) {
                return "#N/A";
            }
            if (s.length == 2) {
                var secondOne = s[1];
                var stringArg = this.splitArgsPreservingQuotedCommas(s[1]);
                if (s[1].indexOf(";") > -1) {
                    var sepArg = this._splitArguments(s[1], ';');
                    if (sepArg.length == 1) {
                        var string1 = this.splitArgsPreservingQuotedCommas(sepArg[0].toUpperCase());
                        for (var cell in string1) {
                            cellValue.push(this.getValueFromArg(string1[cell]).toUpperCase());
                            resultValue.push(this.getValueFromArg(string1[cell]).toUpperCase());
                        }
                    } else if (sepArg.length == 2) {
                        var strin1 = this.splitArgsPreservingQuotedCommas(sepArg[0].toUpperCase());
                        var strin2 = this.splitArgsPreservingQuotedCommas(sepArg[1].toUpperCase());
                        for (var cell in strin1) {
                            cellValue.push(this.getValueFromArg(strin1[cell]).toUpperCase());
                        }
                        for (var cell in strin2) {
                            resultValue.push(this.getValueFromArg(strin2[cell]).toUpperCase());
                        }
                    } else
                        for (var y = 0; y < sepArg.length; y++) {
                            var strin1 = this.splitArgsPreservingQuotedCommas(sepArg[y].toUpperCase());
                            cellValue.push(this.getValueFromArg(strin1[0]).toUpperCase());
                            resultValue.push(this.getValueFromArg(strin1[1]).toUpperCase());
                        }
                } else {
                    if (s[1].indexOf(':') > -1) {
                        var cells = this.getCellsFromArgs(s[1]);
                        for (var cell1 in cells) {
                            cellValue.push(this.getValueFromArg(cells[cell1]).toUpperCase());
                            resultValue.push(this.getValueFromArg(cells[cell1]).toUpperCase());
                        }
                    } else {
                        var string1 = this.splitArgsPreservingQuotedCommas(s[1].toUpperCase());
                        for (var cell in string1) {
                            cellValue.push(this.getValueFromArg(string1[cell]).toUpperCase());
                            resultValue.push(this.getValueFromArg(string1[cell]).toUpperCase());
                        }
                    }
                }
            } else if (s.length == 3) {
                if (r.indexOf(':') > -1) {
                    var cells = this.getCellsFromArgs(r);

                    for (var cell1 in cells) {
                        try  {
                            cellValue.push(this.getValueFromArg(cells[cell1]).toUpperCase());
                        } catch (ex) {
                            return this.getErrorStrings()[0].toString();
                        }
                    }
                } else {
                    var s2 = this.splitArgsPreservingQuotedCommas(s[1]);
                    for (var cell1 in s2) {
                        try  {
                            cellValue.push(this.getValueFromArg(s2[cell1]).toUpperCase());
                        } catch (ex) {
                            return this.getErrorStrings()[0].toString();
                        }
                    }
                }
                var result = s[2];
                if (result.indexOf(':') > -1) {
                    var cells = this.getCellsFromArgs(result);

                    for (var cell1 in cells) {
                        try  {
                            resultValue.push(this.getValueFromArg(cells[cell1]).toUpperCase());
                        } catch (ex) {
                            return this.getErrorStrings()[0].toString();
                        }
                    }
                } else {
                    var s2 = this.splitArgsPreservingQuotedCommas(s[1]);
                    for (var cell1 in s2) {
                        try  {
                            resultValue.push(this.getValueFromArg(s2[cell1]).toUpperCase());
                        } catch (ex) {
                            return this.getErrorStrings()[0].toString();
                        }
                    }
                }
            }
            if (cellValue.indexOf(lookUp) == -1) {
                var temp = cellValue[1];
                var valCount = 0;
                for (var x = 1; x < cellValue.length; x++) {
                    if ((cellValue[x] > temp && lookUp > cellValue[x] && temp > lookUp) || lookUp > cellValue[x]) {
                        temp = (lookUp > cellValue[x]) ? cellValue[x - 1] : cellValue[x];
                        valCount++;
                    } else if (valCount != 0)
                        break;
                }
                if (valCount == 0) {
                    return this.getErrorStrings()[0].toString();
                }
                lookUp = temp;
            }
            var no = 0;
            var index = 0;
            if (s.length == 3 && s[2].indexOf(':') > -1) {
                for (var y = 0; y < cellValue.length; y++) {
                    if (cellValue[y] == lookUp) {
                        index = cellValue.indexOf(lookUp);
                        no++;
                    } else if (no != 0)
                        break;
                }

                finalResult = resultValue[index + no - 1];
            } else {
                for (var y = 0; y < cellValue.length; y++) {
                    if (cellValue[y] == lookUp) {
                        index = cellValue.indexOf(lookUp);
                        no++;
                    } else if (no != 0)
                        break;
                }
                finalResult = resultValue[index + no - 1];
            }
            return finalResult;
        };
        this.computeOffSet = function (arg) {
            var args = this.splitArgsPreservingQuotedCommas(arg);
            var argCount = args.length;

            if (argCount < 3 || argCount > 5) {
                if (this.getRethrowLibraryComputationExceptions())
                    throw this.formulaErrorStrings[this._wrong_number_arguments];
                return this.formulaErrorStrings[this._wrong_number_arguments];
            }
            var r = args[0];
            var result;
            var d = this._parseDouble(this.getValueFromArg(args[1]));
            var rows = (!isNaN(d)) ? parseInt(d.toString()) : -1;
            d = this._parseDouble(this.getValueFromArg(args[2]));

            var cols = (!isNaN(d)) ? parseInt(d.toString()) : -1;
            var hightNum = (argCount >= 4) ? args[3] : "-1";
            d = parseInt(this.getValueFromArg(hightNum));
            var height = (!isNaN(d) ? (d) : 1);
            var widthNum = (argCount == 5) ? args[4] : "-1";
            d = parseInt(this.getValueFromArg(widthNum));
            var width = (!isNaN(d) ? (d) : 1);

            var i = r.indexOf(":");
            var singleCell = i == -1;

            ////single cell
            if (singleCell) {
                r = r + ":" + r;
                i = r.indexOf(":");
            }

            singleCell = (singleCell && width <= 1 && height <= 1); ////only treat as single cell if no hieght or width given...
            if (width == -1)
                singleCell = true;
            var sheet = this._getSheetTokenFromReference(r);

            var row1 = this.rowIndex(r.substring(0, i)) + rows;
            var row2 = this.rowIndex(r.substring(i + 1)) + rows;
            if (!(this.rowIndex(r.substring(0, i)) != -1 || this.rowIndex(r.substring(i + 1)) != -1) == ((this.rowIndex(r.substring(0, i)) == -1 || this.rowIndex(r.substring(i + 1)) != -1))) {
                return this.getErrorStrings()[5].toString();
            }

            var col1 = this.colIndex(r.substring(0, i)) + cols;
            var col2 = this.colIndex(r.substring(i + 1)) + cols;

            if (row1 <= 0 || col1 <= 0) {
                return this.getErrorStrings()[2].toString();
            }
            if ((height > 0 && width < 0) || (height < 1 && width > 1)) {
                row1 = this.rowIndex(this.cell);
            }
            if (col2 == (parseInt(this.computeColumn(r)) - 1))
                singleCell = true;
            var cellArg = sheet + RangeInfo.getAlphaLabel(col1) + row1;
            result = singleCell ? this.computedValue(cellArg) : cellArg + ":" + RangeInfo.getAlphaLabel(col1 + width - 1) + (row1 + height - 1);
            return result;
        };
        this.computeTranspose = function (arg) {
            if (!this._isCellReference(arg) && !this.getNamedRanges().containsKey(arg) && arg.indexOf(";") == -1) {
                var arrayArg = this._splitArguments(arg.split(this.tic).join(""), ';');
                if (arrayArg.length == 1) {
                    return this.getValueFromArg(this.splitArgsPreservingQuotedCommas(arrayArg[0])[0]);
                } else {
                    if (this.getRethrowLibraryComputationExceptions())
                        throw this.formulaErrorStrings[this._invalid_arguments];
                    return this.formulaErrorStrings[this._invalid_arguments];
                }
            }
            var args = this.splitArgsPreservingQuotedCommas(arg);
            var argCount = args.length;

            if (argCount != 1) {
                if (this.getRethrowLibraryComputationExceptions())
                    throw this.formulaErrorStrings[this._wrong_number_arguments];
                return this.formulaErrorStrings[this._wrong_number_arguments];
            }
            var firstRow, LastRow, firstCol, LastCol;
            var result = "";
            if (arg.indexOf(';') > -1) {
                var arrayArg = this._splitArguments(arg.split(this.tic).join(""), ';');
                for (var i = 1; i < arrayArg.length; i++) {
                    if (this.splitArgsPreservingQuotedCommas(arrayArg[i - 1]).length != this.splitArgsPreservingQuotedCommas(arrayArg[1]).length) {
                        if (this.getRethrowLibraryComputationExceptions())
                            throw this.formulaErrorStrings[this._wrong_number_arguments];
                        return this.formulaErrorStrings[this._wrong_number_arguments];
                    }
                }
                result = this.splitArgsPreservingQuotedCommas(arrayArg[0])[0];
            } else {
                var cells = this.getCellsFromArgs(arg);
                firstRow = this.rowIndex(cells[0].toString());
                LastRow = this.rowIndex(cells[cells.length - 1].toString());
                firstCol = this.colIndex(cells[0].toString());
                LastCol = this.colIndex(cells[cells.length - 1].toString());
                var currentCell = this.cell;
                var currentColIndex = this.colIndex(currentCell);
                var currentRowIndex = this.rowIndex(currentCell);
                var arrayHeight = LastRow - firstRow + 1;
                var arrayWidth = LastCol - firstCol + 1;
                var startRow, startCol, endRow, endCol;

                this._getFormulaArrayBounds(this.cell, arrayHeight, arrayWidth);

                startRow = this._getFormulaArrayBoundsfirstRowIndex;
                startCol = this._getFormulaArrayBoundsfirstColIndex;
                endRow = this._getFormulaArrayBoundslastRowIndex;
                endCol = this._getFormulaArrayBoundslastColIndex;

                var x = currentRowIndex - startRow;
                var y = currentColIndex - startCol;
                var i1 = arg.indexOf(":");

                ////single cell
                if (i1 == -1) {
                    arg = arg + ":" + arg;
                    i1 = arg.indexOf(":");
                }

                var k = arg.substring(0, i1).lastIndexOf(this.sheetToken);
                var grd = this.grid;
                var family = CalcEngine.getSheetFamilyItem(this.grid);

                if (k > -1) {
                    this.grid = family.tokenToParentObject[arg.substring(0, k + 1)];
                }
                result = this._getValueComputeFormulaIfNecessary(firstRow + y, firstCol + x, this.grid);
                this.grid = grd;
            }
            if (result == null || result == "")
                result = "0";
            return result;
        };
        this.computeEncodeURL = function (argList) {
            var args = this.splitArgsPreservingQuotedCommas(argList);
            var argCount = args.length;

            if (argCount != 1) {
                if (this.getRethrowLibraryComputationExceptions())
                    throw this.formulaErrorStrings[this._wrong_number_arguments];
                return this.formulaErrorStrings[this._wrong_number_arguments];
            }
            if (!this._isCellReference(args[0]) && !this.getNamedRanges().containsKey(args[0]) && args[0].indexOf(";") == -1) {
                if (this.getRethrowLibraryComputationExceptions())
                    throw this.formulaErrorStrings[this._invalid_arguments];
                return this.getErrorStrings()[5];
            }
            var text = this._stripTics0(this.getValueFromArg(args[0]));
            return encodeURIComponent(text);
        };
        

        this.computeAnd = function (range) {
            var sum = true, s1, d;
            if (range == null || range == "") {
                if (this.getRethrowLibraryComputationExceptions())
                    throw this.formulaErrorStrings[this._bad_formula];
                else
                    return this.formulaErrorStrings[this._bad_formula];
            }
            var ranges = this.splitArgsPreservingQuotedCommas(range);
            for (var r in ranges) {
                if (ranges[r] == (this.tic)) {
                    if (this.getRethrowLibraryComputationExceptions())
                        throw this.getErrorStrings()[1].toString();
                    else
                        return this.getErrorStrings()[1].toString();
                }
                if (ranges[r].indexOf(':') > -1) {
                    for (var s in this.getCellsFromArgs(ranges[r])) {
                        if (this.getErrorStrings().indexOf(s) > -1) {
                            return s;
                        } else if (s.startsWith(this.tic)) {
                            if (this.getRethrowLibraryComputationExceptions)
                                throw new this.getErrorStrings()[5].toString();
                            else
                                return this.getErrorStrings()[5].toString();
                        }
                        try {
                            s1 = this.getValueFromArg(s);
                            if (this.getErrorStrings().indexOf(s1) > -1) {
                                return s1;
                            }
                        } catch (ex) {
                            if (this.rethrowLibraryComputationExceptions && this.getLibraryComputationException != null) {
                                throw this.getErrorStrings()[4].toString();
                            }
                            return this.getErrorStrings()[4].toString();
                        }

                        sum = (sum & (s1 == "" ? true : ((s1.split(this.tic).join("").toUpperCase() == this.trueValueStr) || (d = this._parseDouble(s1)) && d != 0)));
                    }
                } else {
                    try {
                        s1 = this.getValueFromArg(ranges[r]);
                        var tempdate = Date.parse(s1.split(this.tic).join(""));
                        d = this._parseDouble(s1.split(this.tic).join(""))
                        if (!isNaN(tempdate)) {
                            return this.trueValueStr;
                        } else if (isNaN(d) && !(s1 == this._string_empty) && !(s1.split(this.tic).join("").toUpperCase() == this.trueValueStr || s1.split(this.tic).join("").toUpperCase() == this.falseValueStr)) {
                            if (this.getRethrowLibraryComputationExceptions())
                                throw this.getErrorStrings()[1].toString();
                            else
                                return (this._isCellReference(ranges[r]) || s1[0] == this.tic) ? this.getErrorStrings()[1].toString() : this.getErrorStrings()[5].toString();
                        }
                    } catch (Exception) {
                        if (this.getRethrowLibraryComputationExceptions() && this.getLibraryComputationException() != null) {
                            throw this.getErrorStrings()[4].toString();
                        }

                        return this.getErrorStrings()[4].toString();
                    }
                    var dp = this._parseDouble(s1);
                    sum = (sum & ((s1.split(this.tic).join("").toUpperCase() == this.trueValueStr) || (!isNaN(dp) && d != 0)));
                    // sum &= (s1 == this.trueValueStr) || (d = this._parseDouble(s1)) && d != 0;
                }
            }
            return sum ? this.trueValueStr : this.falseValueStr;
        };
        this.computeFalse = function (empty) {
            return this.falseValueStr;
        };
        this.computeIf = function (args) {
            var s1 = this._string_empty;
            var array1 = [this.getParseArgumentSeparator(), ':'];

            ////parsed formula
            if (args.length > 0 && this._indexOfAny(args, array1) == -1) {
                return this.getFormulaErrorStrings();
            } else {
                var s = this.getCellsFromArgs(args);

                if (s.length >= 2) {
                    try {
                        s1 = this.getValueFromArg(s[0]);
                        var d;
                        if (s1 == true || (d = this._parseDouble(s1)) && d != 0) {
                            s1 = this.getValueFromArg(s[1]);
                        } else if (s1 == this.falseValueStr || (s1 == "") || (d = this._parseDouble(s1)) && d == 0) {
                            s1 = s.length == 3 ? this.getValueFromArg(s[2]) : false;
                        } else {
                            //  if (this.tic().indexOf(s[0]) == -1)
                            if (s.indexOf(this.tic) > -1) {
                                var first = s[0].indexOf(this.tic) + 1;
                                var last = s[0].lastIndexOf(this.tic) - 1;
                                var st = s[0].substring(first, last - first + 1);
                                for (var err in this.getFormulaErrorStrings) {
                                    if (st === (err))
                                        return err;
                                }
                            }
                            s1 = "NaN";
                        }
                    } catch (ex) {
                        if (this.rethrowLibraryComputationExceptions && this.getLibraryComputationException != null) {
                            throw this.getLibraryComputationException;
                        }

                        return ex;
                    }
                } else {
                    return this.getFormulaErrorStrings();
                }
            }
            return s1;
        };
        this.computeIfError = function (args) {
            var argsArray = args.split(this.getParseArgumentSeparator());
            var range = argsArray[0];

            //    var arrgsArray = [this.getParseArgumentSeparator, ];
            if (argsArray.length != 2) {
                if (this.getRethrowLibraryComputationExceptions())
                    throw this.formulaErrorStrings[this._wrong_number_arguments];
                return this.formulaErrorStrings[this._wrong_number_arguments];
            }
            if (range[0] == this.tic)
                return this.getValueFromArg(argsArray[0]);
            try {
                if (range[0] == this.bMARKER) {
                    range = range.replace(this.bMARKER, ' ');
                    range = range.trim();
                    if (range[0] == ("NAN") || range[0] == ("-NAN") || range[0] == ("INFINITY") || range[0] == ("-INFINITY") || range[0] == ("#") || range[0] == ("n#"))
                        return this.getValueFromArg(argsArray[1]);
                }
                range = argsArray[0];

                range = this.getValueFromArg(range).toUpperCase().split(this.tic).join(this._string_empty);
            } catch (ex) {
                range = range.toUpperCase();
            }
            if (range[0] == ("NAN") || range[0] == ("-NAN") || range[0] == ("INFINITY") || range[0] == ("-INFINITY") || range[0] == ("#") || range[0] == ("n#"))
                return this.getValueFromArg(argsArray[1]);
            else
                return this.getValueFromArg(argsArray[0]);
        };
        this.computeIfNA = function (range) {
            var args = this.splitArgsPreservingQuotedCommas(range);
            var argCount = args.length, arg1 = this._string_empty, index = -1, result;
            if (argCount != 2) {
                return this.getFormulaErrorStrings();
            }

            if (this.computeIsNA(args[0]) == this.trueValueStr) {
                if ((this.tic.indexOf(args[1]) == -1) && !this._isCellReference(args[1]) && !(result = this._parseDouble(args[0])) && !this.namedRanges.containsKey(args[1])) {
                    return this.getErrorStrings()[5].toString();
                    ;
                }
                arg1 = this.getValueFromArg(args[1]);
            } else {
                if (this._isRange(args[0])) {
                    arg1 = this.getErrorStrings()[1].toString();
                } else {
                    if ((this.tic.indexOf(args[0]) == -1) && !this._isCellReference(args[0]) && !(result = this._parseDouble(args[0])) && !this.NamedRanges.ContainsKey(args[0])) {
                        return this.getErrorStrings()[5].toString();
                    }
                    arg1 = this.getValueFromArg(args[0]);
                }
            }
            if (arg1[0] == this.tic && arg1[length - 1] == this.tic) {
                arg1 = this.substring(arg1, 1, arg1.length - 2);
            }
            return arg1;
        };
        this.computeNot = function (args) {
            var s = args;
            var d1;
            var array1 = [this.getParseArgumentSeparator, ':'];

            //var array1 = [this.getParseArgumentSeparator()];
            ////parsed formula
            if ((args.length > 0) && this._indexOfAny(args, array1) > -1) {
                return this.getFormulaErrorStrings();
            } else {
                try {
                    s = this.getValueFromArg(s);
                    if (s == this.trueValueStr) {
                        s = this.falseValueStr;
                    } else if (s == this.falseValueStr) {
                        s = this.trueValueStr;
                    } else if (d1 = this._parseDouble(s)) {
                        ////Flip the value.
                        if (Math.abs(d1) > 1e-10) {
                            s = this.falseValueStr;
                        } else {
                            s = this.trueValueStr;
                        }
                    }
                } catch (ex) {
                    if (this.rethrowLibraryComputationExceptions && this.getLibraryComputationException != null) {
                        throw this.getLibraryComputationException;
                    }

                    return ex;
                }
            }

            return s;
        };
        this.computeOr = function (args) {
            var sum = this.falseValueStr;
            var s1;
            var d;

            // args = this.adjustRangeArg(args);
            var ranges = this.splitArgsPreservingQuotedCommas(args);
            for (var r in ranges) {
                ////is a cellrange
                if (ranges[r].indexOf(':') > -1 && this._isCellReference(r)) {
                    for (var s in this.getCellsFromArgs(ranges[r])) {
                        try {
                            s1 = this.getValueFromArg(s);
                        } catch (ex) {
                            if (this.rethrowLibraryComputationExceptions && this.getLibraryComputationException != null) {
                                throw this.getLibraryComputationException;
                            }

                            return ex;
                        }

                        sum = (s1 == this.trueValueStr) || (d == this._parseDouble(s1) && d != 0);
                    }
                } else {
                    try {
                        s1 = this.getValueFromArg(ranges[r]);
                    } catch (ex) {
                        if (this.rethrowLibraryComputationExceptions && this.getLibraryComputationException != null) {
                            throw this.getLibraryComputationException;
                        }

                        return ex;
                    }

                    sum = (s1 == this.trueValueStr) || (d == this._parseDouble(s1) && d != 0);
                }
            }
            return sum ? this.trueValueStr : this.falseValueStr;
        };
        this.computeTrue = function (empty) {
            return this.trueValueStr;
        };
        this.computeXor = function (arg) {
            var sum = false, s1, d;

            // range = this.adjustRangeArg;
            var ranges = this.splitArgsPreservingQuotedCommas(arg);
            for (var r in ranges) {
                ////is a cellrange
                if (ranges[r].indexOf(':') > -1 && this._isCellReference(r)) {
                    for (var s in this.getCellsFromArgs(ranges[r])) {
                        try {
                            s1 = this.getValueFromArg(s);
                        } catch (ex) {
                            if (this.rethrowLibraryComputationExceptions && this.getLibraryComputationException != null) {
                                throw this.getLibraryComputationException;
                            }

                            return ex;
                        }

                        sum = (s1 == this.trueValueStr) || (d == this._parseDouble(s1) && (d != 0));
                    }
                } else {
                    try {
                        s1 = this.getValueFromArg(ranges[r]);
                    } catch (ex) {
                        if (this.rethrowLibraryComputationExceptions && this.getLibraryComputationException != null) {
                            throw this.getLibraryComputationException;
                        }

                        return ex;
                    }

                    sum = (s1 == this.trueValueStr) || (d == this._parseDouble(s1)) && (d != 0);
                }
            }
            return sum ? this.trueValueStr : this.falseValueStr;
        };

        this.computeCell = function (arg) {
            var args = this.splitArgsPreservingQuotedCommas(arg);
            if (args.length > 2) {
                if (this.getRethrowLibraryComputationExceptions())
                    throw this.formulaErrorStrings[this._wrong_number_arguments];
                return this.formulaErrorStrings[this._wrong_number_arguments];
            }
            var reference = "";
            if (args.length == 2) {
                if (!this._isCellReference(args[1]) && !this.namedRanges.containsKey(args[1])) {
                    if (this.getRethrowLibraryComputationExceptions())
                        throw this.formulaErrorStrings[this._bad_formula];
                    return this.getErrorStrings()[5].toString();
                }
                reference = args[1];
            } else {
                reference = this.cell;
            }
            var type = args[0].split(this.tic).join(this._string_empty).toLowerCase(), result = "";
            switch (type) {
                case "address":
                    result = this.rowIndex(reference).toString() + this.getParseArgumentSeparator() + this.colIndex(reference).toString();
                    result = this.computeAddress(result);
                    break;
                case "col":
                    result = this.computeColumn(reference);
                    break;
                case "color":
                    break;
                case "contents":
                    result = this.getValueFromArg(reference);
                    break;
                case "filename":
                    result = "Not Supported";
                    break;
                case "format":
                    break;
                case "parentheses":
                    break;
                case "prefix":
                    break;
                case "protect":
                    break;
                case "row":
                    result = this.computeRow(reference);
                    break;
                case "type":
                    if (this.computeIsBlank(reference) == this.trueValueStr)
                        result = "b";
                    else if (this.computeIsText(reference) == this.trueValueStr)
                        result = "l";
                    else
                        result = "v";
                    break;
                case "width":
                    break;
                default:
                    break;
            }
            return result.toString();
        };
        this.computeErrorType = function (args) {
            var cellReference = this._string_empty;
            var arg = this._splitArguments(args, this.getParseArgumentSeparator());
            var argCount = arg.length;
            if (argCount != 1) {
                if (this.getRethrowLibraryComputationExceptions())
                    throw this.formulaErrorStrings[this._wrong_number_arguments];
                return this.formulaErrorStrings[this._wrong_number_arguments];
            }
            if (this._isCellReference(args)) {
                cellReference = this.getValueFromArg(args);
            } else {
                cellReference = args;
            }
            if (cellReference.length > 1 && cellReference[0] == this.tic[0] && cellReference[cellReference.length - 1] == this.tic[0]) {
                return "#N/A";
            }
            if (cellReference == "#NULL!")
                return "1";
            else if (cellReference == "#DIV/0!")
                return "2";
            else if (cellReference == "#VALUE!")
                return "3";
            else if (cellReference == "#REF!")
                return "4";
            else if (cellReference == "#NAME?")
                return "5";
            else if (cellReference == "#NUM!")
                return "6";
            else if (cellReference == "#N/A")
                return "7";
            else if (cellReference == "#GETTING_DATA")
                return "8";
            return "#N/A";
        };
        this.computeInfo = function (argList) {
            var args = this.splitArgsPreservingQuotedCommas(argList);
            var argCount = args.length;
            if (argCount != 1 || argList=="") {
                if (this.getRethrowLibraryComputationExceptions())
                    throw this.formulaErrorStrings[this._wrong_number_arguments];
                return this.formulaErrorStrings[this._wrong_number_arguments];
            }
            var firstStr = this._stripTics0(this.getValueFromArg(args[0])).toLowerCase();
            var result = this._string_empty;
            switch (firstStr) {
                case "directory":
                    result = this.System.Environment.CurrentDirectory;
                    break;
                case "numfile":
                    if (this.getSortedSheetNames() != null && CalcEngine._sheetFamiliesList.length == 0)
                        result = this.getSortedSheetNames().length.toString();
                    else
                        result = "1";
                    break;
                case "origin":
                    var cell = this.rowIndex(this.cell).toString() + this.getParseArgumentSeparator() + this.colIndex(this.cell).toString();
                    result = "$A: " + this.computeAddress(cell);
                    break;
                case "osversion":
                    result = this.System.Environment.OSVersion.toString();
                    break;
                case "recalc":
                    if (this._alwaysComputeDuringRefresh)
                        result = "Automatic";
                    else
                        result = "Manual";
                    break;
                case "release":
                    result = this.System.Reflection.Assembly.GetExecutingAssembly().GetName().Version.toString();
                    break;
                case "system":
                    switch (this.Environment.OSVersion.Platform) {
                        case this.PlatformID.Unix:
                            result = "unix";
                            break;
                        case this.PlatformID.MacOSX:
                            result = "mac";
                            break;
                        default:
                            result = "pcdos";
                            break;
                    }
                    break;
                default:
                    if (this.getRethrowLibraryComputationExceptions())
                        throw this.formulaErrorStrings[this._bad_formula];
                    return this.getErrorStrings()[1].toString();
            }
            return result;
        };
        this.computeIsBlank = function (args) {

            var arg = this.splitArgsPreservingQuotedCommas(args);
            var argCount = arg.length;
            if (argCount != 1 || args == "") {
                if (this.getRethrowLibraryComputationExceptions())
                    throw this.formulaErrorStrings[this._wrong_number_arguments];
                return this.formulaErrorStrings[this._wrong_number_arguments];
            }
            if (this.getValueFromArg(args) == "") {
                return this.trueValueStr;
            }
            return this.falseValueStr;
        };
        this.computeIsErr = function (range) {
            var arg = this.splitArgsPreservingQuotedCommas(range);
            var argCount = arg.length;
            if (argCount != 1 || range == "") {
                if (this.getRethrowLibraryComputationExceptions())
                    throw this.formulaErrorStrings[this._wrong_number_arguments];
                return this.formulaErrorStrings[this._wrong_number_arguments];
            }
            range = this.getValueFromArg(range).toUpperCase().split(this.tic).join(this._string_empty);
            if ((range[0] == ("NAN") || range[0] == ("-NAN") || range[0] == ("INFINITY") || range[0] == ("-INFINITY") || range[0] == ("#") || range[0] == ("n#")) && !(range[0] == ("#N/A"))) {
                return this.trueValueStr;
            } else {
                return this.falseValueStr;
            }
        };

        this.computeIsError = function (range) {

            var arg = this.splitArgsPreservingQuotedCommas(range);
            var argCount = arg.length;
            if (argCount != 1 || range == "") {
                if (this.getRethrowLibraryComputationExceptions())
                    throw this.formulaErrorStrings[this._wrong_number_arguments];
                return this.formulaErrorStrings[this._wrong_number_arguments];
            }
            try {
                range = this.getValueFromArg(range).toUpperCase().split(this.tic).join(this._string_empty);
            }
            catch (ex) {
                return this.trueValueStr;
            }
            if (range.indexOf("NAN") == 0 || range.indexOf("-NAN") == 0 || range.indexOf("INFINITY") == 0 || range.indexOf("-INFINITY") == 1 || range.indexOf("#") == 0 || range.indexOf("n#") == 0) {
                return this.trueValueStr;
            } else {
                return this.falseValueStr;
            }
        };

        this.computeIsEven = function (args) {
            var cellReference = this._string_empty;
            var val = 0.0;
            var arg = this.splitArgsPreservingQuotedCommas(args);
            var argCount = arg.length;
            if (argCount != 1 || args == "") {
                if (this.getRethrowLibraryComputationExceptions())
                    throw this.formulaErrorStrings[this._wrong_number_arguments];
                return this.formulaErrorStrings[this._wrong_number_arguments];
            }
            var argVal = this.getValueFromArg(arg[0]);
            var value = parseInt(argVal.split(this.tic).join(""));
            if (isNaN(value)) {
                if (this._rethrowLibraryComputationExceptions)
                    throw this.formulaErrorStrings[this._bad_formula];
                return (argVal[0] == this.tic || this._isCellReference(arg[0])) ? this.getErrorStrings()[1].toString() : this.getErrorStrings()[5].toString();
            }
            var dt = new Date(Date.parse(argVal));
            if (isNaN(value) && dt.toString() != "invalid Date") {
                value = this._toOADate(dt);
            }
            var result = value;
            if (result % 2 == 0) {
                return this.trueValueStr;
            }
            return this.falseValueStr;
        };
        this.computeIsFormula = function (argList) {
            var args = this.splitArgsPreservingQuotedCommas(argList);
            var argCount = args.length;
            var value = this._string_empty;
            if (argCount != 1) {
                if (this.getRethrowLibraryComputationExceptions())
                    throw this.formulaErrorStrings[this._wrong_number_arguments];
                return this.formulaErrorStrings[this._wrong_number_arguments];
            }
            value = this.getValueFromArg(args[0]);
            if (this.namedRanges.containsValue(value)) {
                if (this.getRethrowLibraryComputationExceptions())
                    throw this.formulaErrorStrings[this._bad_formula];
                return this.getErrorStrings()[1].toString();
            }
            var intVal = parseInt(value.split(this.tic).join(""));
            if (!isNaN(intVal) || !this._isCellReference(args[0])) {
                if (this.getRethrowLibraryComputationExceptions())
                    throw this.formulaErrorStrings[this._bad_formula];
                return this.getErrorStrings()[5].toString();
            }
            var family = CalcEngine.getSheetFamilyItem(this.grid);
            if (this.isSheetMember() && family.parentObjectToToken != null) {
                args[0] = family.parentObjectToToken.getItem(this.grid) + args[0];
            }
            if (this.getFormulaInfoTable().containsKey(args[0])) {
                return this.trueValueStr;
            } else {
                return this.falseValueStr;
            }
        };
        this.computeIsLogical = function (args) {
            if (args == this.falseValueStr || args == this.trueValueStr) {
                return this.trueValueStr;
            }
            return this.falseValueStr;
        };
        this.computeIsNA = function (args) {
            if (this.getErrorStrings().indexOf(args.toUpperCase()) != -1) {
                if (args.toUpperCase() == ("#N/A")) {
                    return this.trueValueStr;
                } else {
                    return this.falseValueStr;
                }
            }
            try {
                args = this.getValueFromArg(args).toUpperCase();
            } catch (ex) {
                return this.falseValueStr;
            }
            if (args[0] == ("#N/A")) {
                return this.trueValueStr;
            }
            return this.falseValueStr;
        };
        this.computeIsNonText = function (args) {
            if (this.computeIsText(args) == this.trueValueStr) {
                return this.falseValueStr;
            } else {
                return this.trueValueStr;
            }
        };
        this.computeIsNumber = function (range) {
            var d;
            range = this.getValueFromArg(range);
            if (!isNaN(this._parseDouble(range))) {
                return this.trueValueStr;
            } else {
                return this.falseValueStr;
            }
        };
        this.computeIsRef = function (args) {
            var arg = this.splitArgsPreservingQuotedCommas(args);
            if (arg.length != 1) {
                if (this.getRethrowLibraryComputationExceptions())
                    throw this.formulaErrorStrings[this._wrong_number_arguments];
                return this.formulaErrorStrings[this._wrong_number_arguments];
            }
            return ((this.namedRanges.containsKey(args) || this._isCellReference(args)) && args[0] != (this.tic) && args[length - 1] != (this.tic)).toString().toUpperCase();
        };
        this.computeIsOdd = function (args) {
            var cellReference = this._string_empty;
            var val = 0.0;
            var arg = this._splitArguments(args, this.getParseArgumentSeparator());
            var argCount = arg.length;
            var array1 = [this.getParseArgumentSeparator(), ':'];
            if (argCount != 1) {
                if (this.getRethrowLibraryComputationExceptions())
                    throw this.formulaErrorStrings[this._wrong_number_arguments];
                return this.formulaErrorStrings[this._wrong_number_arguments];
            }
            if (args != this._string_empty) {
                if (this._indexOfAny(args, array1) != -1) {
                    return "#VALUE!";
                }
                cellReference = this.getValueFromArg(arg[0]);
                val = parseInt(cellReference.split(this.tic).join(this._string_empty))
                if (isNaN(val)) {
                    if (this._rethrowLibraryComputationExceptions)
                        throw this.formulaErrorStrings[this._bad_formula];
                    return (cellReference[0] == this.tic || this._isCellReference(arg[0])) ? this.getErrorStrings()[1].toString() : this.getErrorStrings()[5].toString();
                }
                if (val % 2 != 0) {
                    return this.trueValueStr;
                }
                return this.falseValueStr;
            }
            return this.trueValueStr;
        };
        this.computeIsText = function (args) {
            var isCell = this._isCellReference(args);
            var s = (isCell) ? this.getValueFromArg(args) : args;
            if ((isCell || s[0] == this.tic) && s.length > 0 && isNaN(this._parseDouble(s))) {
                return this.trueValueStr;
            } else {
                return this.falseValueStr;
            }
        };
        this.computeN = function (args) {
            var cellReference = this._string_empty;
            var val = 0.0;
            var date;
            var arg = this._splitArguments(args, this.getParseArgumentSeparator());
            var argCount = arg.length;
            if (argCount != 1) {
                if (this.getRethrowLibraryComputationExceptions())
                    throw this.formulaErrorStrings[this._wrong_number_arguments];
                return this.formulaErrorStrings[this._wrong_number_arguments];
            }
            cellReference = this.getValueFromArg(args);
            val = this._parseDouble(cellReference);
            if (cellReference == this.trueValueStr) {
                val = 1;
            }
            else if (cellReference == this.falseValueStr) {
                val = 0;
            }
            else if (this.getErrorStrings().indexOf(cellReference) > -1 || this.formulaErrorStrings.indexOf(cellReference) > -1) {
                return cellReference;
            }
            else if (!isNaN(val)) {
                return val.toString();
            }
            else {
                return "0";
            }

            return val.toString();
        };
        this.computeNA = function () {
            return "#N/A";
        };
        this.computeSheet = function (argList) {
            var args = this.splitArgsPreservingQuotedCommas(argList);
            var argCount = args.length;
            var value = this._string_empty;
            if (argCount > 1) {
                if (this.getRethrowLibraryComputationExceptions())
                    throw this.formulaErrorStrings[this._wrong_number_arguments];
                return this.formulaErrorStrings[this._wrong_number_arguments];
            }
            var cellReference = args[0].toUpperCase();
            cellReference = (cellReference == null || cellReference == this._string_empty) ? this.cell : cellReference;
            if (cellReference == null || cellReference == this._string_empty) {
                return "1";
            }
            if (this.getErrorStrings().indexOf(cellReference)>-1) {
                return cellReference;
            }
            if (!this._isCellReference(cellReference) && !this.namedRanges.containsKey(cellReference) && this.getSortedSheetNames().indexOf(cellReference) == -1) {
                if (this.getRethrowLibraryComputationExceptions())
                    throw this.formulaErrorStrings[this._bad_formula];
                return this.getErrorStrings()[0].toString();
            }
            var family = this.getSheetFamilyItem(this.grid);
            var sheet1 = this._getSheetTokenFromReference(cellReference);
            var sheetNumber = 0;
            if ((sheet1 == null || sheet1 == this._string_empty) && this.getSortedSheetNames().indexOf(cellReference) == -1 && !cellReference.indexOf(this.sheetToken.toString())) {
                sheetNumber = this.getSheetID(this.grid) + 1;
            } else if (sheet1.length > 0) {
                sheetNumber = parseInt(sheet1.Replace(this.sheetToken.split(this.tic).join(this._string_empty))) + 1;
            } else {
                try {
                    for (var griddata in family.tokenToParentObject) {
                        if (family.sheetNameToParentObject[cellReference].Equals(griddata.Value)) {
                            var gg = griddata.Value;
                            sheetNumber = this.getSheetID(gg) + 1;
                            break;
                        }
                    }
                } catch (ex) {
                    if (this.getRethrowLibraryComputationExceptions())
                        throw this.formulaErrorStrings[this._bad_formula];
                    return this.getErrorStrings()[2].toString();
                }
            }
            return sheetNumber.toString();
        };
        this.computeSheets = function (argList) {
            var args = this.splitArgsPreservingQuotedCommas(argList);
            var argCount = args.length;
            var value = this._string_empty;
            if (argCount > 1) {
                if (this.getRethrowLibraryComputationExceptions())
                    throw this.formulaErrorStrings[this._wrong_number_arguments];
                return this.formulaErrorStrings[this._wrong_number_arguments];
            }
            var sheetNumber = 0;
            var cellReference = args[0].toUpperCase();
            if (this.getErrorStrings().indexOf(cellReference) > -1) {
                return cellReference;
            }
            if (cellReference == null || cellReference == this._string_empty) {
                return this.getSortedSheetNames().length.toString();
            }
            else if (cellReference.split(this.tic).join(this._string_empty) == "") {
                if (this.getRethrowLibraryComputationExceptions())
                    throw this.formulaErrorStrings[this._bad_formula];
                return this.getErrorStrings()[5].toString();
            } else if (!this._isCellReference(cellReference) && !this.namedRanges.containsKey(cellReference) && this.getSortedSheetNames().indexOf(cellReference) == -1) {
                if (this.getRethrowLibraryComputationExceptions())
                    throw this.formulaErrorStrings[this._bad_formula];
                return this.getErrorStrings()[5].toString();
            }
            try {
                var sheetsCount = this._splitArguments(cellReference, '!');
                sheetNumber = (sheetsCount.length - 1) / 2;
                if (sheetNumber == 0) {
                    if (this.getRethrowLibraryComputationExceptions())
                        throw this.formulaErrorStrings[this._bad_formula];
                    return this.getErrorStrings()[5].toString();
                }
            } catch (ex) {
                if (this.getRethrowLibraryComputationExceptions())
                    throw this.getErrorStrings()[2].toString();
                return this.getErrorStrings()[2].toString();
            }
            return sheetNumber.toString();
        };
        this.computeType = function (argList) {
            var args = this.splitArgsPreservingQuotedCommas(argList);
            var argCount = args.length;
            var result = 0;
            if (argCount != 1) {
                if (this.getRethrowLibraryComputationExceptions())
                    throw this.formulaErrorStrings[this._wrong_number_arguments];
                return this.formulaErrorStrings[this._wrong_number_arguments];
            }
            var text = this.getValueFromArg(args[0]);
            if (text == null || text == this._string_empty) {
                if (this.getRethrowLibraryComputationExceptions())
                    throw this.getErrorStrings()[1].toString();
                return this.getErrorStrings()[1].toString();
            } else if (argList.indexOf(this.getParseArgumentSeparator()) > -1) {
                result = 64;
            } else if (this.computeIsNumber(text) == this.trueValueStr) {
                result = 1;
            } else if (this.computeIsLogical(text) == this.trueValueStr) {
                result = 4;
            } else if (this.getErrorStrings().indexOf(text) > -1) {
                result = 16;
            } else if (this.computeIsText(text) == this.trueValueStr) {
                result = 2;
            }
            return result.toString();
        };
        
        this.computeRow = function (arg) {
            var args = this.splitArgsPreservingQuotedCommas(arg);
            var argCount = args.length;
            if (argCount != 1) {
                if (this.getRethrowLibraryComputationExceptions())
                    throw this.formulaErrorStrings[this._wrong_number_arguments];
                return this.formulaErrorStrings[this._wrong_number_arguments];
            }
            if (args[0] == this._string_empty) {
                return this.rowIndex(this.cell).toString();
            }
            if (!this._isCellReference(args[0]) && !this.namedRanges.containsKey(args[0])) {
                if (this.getRethrowLibraryComputationExceptions())
                    throw this.getErrorStrings()[4].toString();
                return this.getErrorStrings()[4].toString();
            }
            if (arg.indexOf(':') > -1) {
                arg = this.getCellsFromArgs(arg)[0]; //considers first cell reference from the range of cells.
            }
            return this.rowIndex(arg).toString();
        };
        this.computeRows = function (arg) {
            var args = this.splitArgsPreservingQuotedCommas(arg);
            if (args.length != 1) {
                if (this.getRethrowLibraryComputationExceptions())
                    throw this.formulaErrorStrings[this._wrong_number_arguments];
                return this.formulaErrorStrings[this._wrong_number_arguments];
            }
            if (!this._isCellReference(args[0]) && arg[0].indexOf(';') == -1) {
                if (this.getRethrowLibraryComputationExceptions())
                    throw this.getErrorStrings()[4].toString();
                return this.getErrorStrings()[4].toString();
            }
            var firstRow, LastRow, totalRows = 1;
            if (args[0].indexOf(':') > -1) {
                var cells = this.getCellsFromArgs(arg);
                firstRow = this.rowIndex(cells[0].toString());
                LastRow = this.rowIndex(cells[cells.length - 1].toString());
                totalRows = LastRow - firstRow + 1;
                totalRows = (totalRows > 0) ? totalRows : 1;
            } else if (args[0].indexOf(';') > -1) {
                var arrayArg = this._splitArguments(arg.split(this.tic).join(this._string_empty), ';');
                for (var i = 1; i < arrayArg.length; i++) {
                    if (this.splitArgsPreservingQuotedCommas(arrayArg[i - 1]).length != this.splitArgsPreservingQuotedCommas(arrayArg[1]).length) {
                        if (this.getRethrowLibraryComputationExceptions())
                            throw this.getErrorStrings()[4].toString();
                        return this.getErrorStrings()[4].toString();
                    }
                    totalRows = arrayArg.length;
                }
            }
            return totalRows.toString();
        };


        //this.grid.wireParentObject();
        this._initLibraryFunctions();
        //var _rangeInfo = new RangeInfo();
    }
    CalcEngine.CalcEngine = function () {
    };
    CalcEngine._formulaChar = '=';

    CalcEngine.sheetFamilyID = 0;
    CalcEngine._tokenCount = 0;
    CalcEngine.modelToSheetID = null;
    CalcEngine._sheetFamiliesList = null;

    CalcEngine._defaultFamilyItem = null;

    CalcEngine.createSheetFamilyID = function () {
        if (this.sheetFamilyID == Number.MAX_VALUE) {
            this.sheetFamilyID = Number.MIN_VALUE;
        }
        return this.sheetFamilyID++;
    };

    CalcEngine.getSheetFamilyItem = function (model) {
        if (this.sheetFamilyID == 0) {
            if (CalcEngine._defaultFamilyItem == null) {
                CalcEngine._defaultFamilyItem = new GridSheetFamilyItem();
            }

            return CalcEngine._defaultFamilyItem;
        }

        if (CalcEngine._sheetFamiliesList == null) {
            CalcEngine._sheetFamiliesList = new HashTable();
        }

        var i = CalcEngine.modelToSheetID.getItem(model);

        if (!CalcEngine._sheetFamiliesList.containsKey(i)) {
            CalcEngine._sheetFamiliesList.add(i, new GridSheetFamilyItem());
        }

        return CalcEngine._sheetFamiliesList.getItem(i);
    };

    CalcEngine.resetSheetFamilyID = function () {
        this.sheetFamilyID = 0;

        if (CalcEngine.modelToSheetID != null) {
            CalcEngine.modelToSheetID.length = 0;
            CalcEngine.modelToSheetID = null;
        }

        if (CalcEngine._sheetFamiliesList != null) {
            CalcEngine._sheetFamiliesList.clear();
            CalcEngine._sheetFamiliesList = null;

            //Fix for defect #12445.
            this.resetSheetIDs();
        }
    };

    CalcEngine.resetSheetIDs = function () {
        this._tokenCount = 0;
    };

    CalcEngine.unregisterGridAsSheet = function (refName, model) {
        var family = CalcEngine.getSheetFamilyItem(model);
        var refName1 = refName.toUpperCase();
        if (family.sheetNameToParentObject != null && family.sheetNameToParentObject.containsKey(refName1)) {
            family.sheetNameToParentObject.remove(refName1);
            var token = family.sheetNameToToken.getItem(refName1);
            family.sheetNameToToken.remove(refName1);
            family.tokenToParentObject.remove(token);
            family.parentObjectToToken.remove(model);
        }
    };
    return CalcEngine;
})(jQuery, Syncfusion);

var LookupCachingMode;
(function (LookupCachingMode) {
    LookupCachingMode[LookupCachingMode["None"] = 0] = "None";
    LookupCachingMode[LookupCachingMode["VLOOKUP"] = 1] = "VLOOKUP";
    LookupCachingMode[LookupCachingMode["HLOOKUP"] = 2] = "HLOOKUP";
    LookupCachingMode[LookupCachingMode["Both"] = 3] = "Both";
    LookupCachingMode[LookupCachingMode["OptimizeForMatches"] = 4] = "OptimizeForMatches";
})(LookupCachingMode || (LookupCachingMode = {}));
;

var HashTable = (function () {
    function HashTable() {
        this.length = 0;
        this.items = [];
        this.add = function (key, value) {
            this.previous = undefined;
            if (this.containsKey(key)) {
                this.previous = this.items[key];
            } else {
                this.length++;
            }
            this.items[key] = value;
            return this.previous;
        };
        this.clear = function () {
            this.items = {};
            this.length = 0;
        };
        this.contains = function (key) {
            return this.items.hasOwnProperty(key);
        };
        this.containsKey = function (key) {
            return this.items.hasOwnProperty(key);
        };
        this.containsValue = function (key) {
            return (this.items.hasOwnProperty(key) && this.items[key] != undefined) ? true : false;
        };
        this.getItem = function (key) {
            if (this.containsKey(key))
            {

                return this.items[key]
            }
            else
            {
                return  undefined;
            }
        };
        this.keys = function () {
            var keys = [];
            for (var k in this.items) {
                if (this.containsKey(k)) {
                    keys.push(k);
                }
            }
            return keys;
        };
        this.remove = function (key) {
            if (this.containsKey(key)) {
                this.previous = this.items[key];
                this.length--;
                delete this.items[key];
                return this.previous;
            } else {
                return undefined;
            }
        };
        this.values = function () {
            var values = [];
            for (var k in this.items) {
                if (this.containsKey(k)) {
                    values.push(this.items[k]);
                }
            }
            return values;
        };
        this.each = function (fn) {
            for (var k in this.items) {
                if (this.containsKey(k)) {
                    fn(k, this.items[k]);
                }
            }
        };
        var previous = undefined;
    }
    return HashTable;
})();

var RangeInfo = (function () {
    function RangeInfo(top, left, bottom,right) {
        this.top = top;
        this.bottom = bottom;
        this.left = left;
        this.right = right;
        this.getBottom = function () {
            return this.bottom;
        };
        this.setBottom = function (value) {
            this.bottom = value;
        };
        this.getTop = function () {
            return this.top;
        };
        this.setTop = function (value) {
            this.top = value;
        };
        this.getLeft = function () {
            return this.left;
        };
        this.setLeft = function (value) {
            this.left = value;
        };
        this.getRight = function () {
            return this.right;
        };
        this.setRight = function (value) {
            this.right = value;
        };
    }
    RangeInfo.cells = function (top, left, bottom, right) {
        return new RangeInfo(top, left, bottom,right);
    };

    RangeInfo.getAlphaLabel = function (col) {
        var cols = [];
        var n = 0;
        var charText = "A";
        while (col > 0 && n < 9) {
            col--;
            var aCharNo = charText.charCodeAt(0);
            cols[n] = String.fromCharCode(col % 26 + aCharNo);
            col = parseInt((col / 26).toString());
            n++;
        }

        var chs = [];
        for (var i = 0; i < n; i++) {
            chs[n - i - 1] = cols[i];
        }

        return chs.join("");
    };
    return RangeInfo;
})();

var GridSheetFamilyItem = (function () {
    function GridSheetFamilyItem() {
        this.isSheetMember = false;
        this.parentObjectToToken = null;
        this.sheetDependentCells = null;
        this.sheetDependentFormulaCells = null;
        this.sheetFormulaInfoTable = null;
        /// <summary>
        /// Holds mapping from sheet name to parent object.
        /// </summary>
        this.sheetNameToParentObject = null;
        /// <summary>
        /// Holds mapping from parent object to sheet name.
        /// </summary>
        this.sheetNameToToken = null;
        /// <summary>
        /// Holds mapping from sheet token to parent object.
        /// </summary>
        this.tokenToParentObject = null;
    }
    return GridSheetFamilyItem;
})();

var FormulaInfo = (function () {
    function FormulaInfo() {
        this.calcID = Number.MIN_VALUE + 1;
        /// <summary>
        /// A property that gets/sets the text of the initial formula.
        /// </summary>
        this.getFormulaText = function () {
            return this._formulaText;
        };
        this.setFormulaText = function (value) {
            this._formulaText = value;
        };
        /// <summary>
        /// A property that gets/sets the last computed value of this formula.
        /// </summary>
        this.getFormulaValue = function () {
            return this._formulaValue;
        };
        this.setFormulaValue = function (value) {
            this._formulaValue = value;
        };
        /// <summary>
        /// A property that gets/sets the formula as a parsed string.
        /// </summary>
        this.getParsedFormula = function () {
            return this._parsedFormula;
        };
        this.setParsedFormula = function (value) {
            this._parsedFormula = value;
        };
        var _formulaText;
        var _formulaValue;
        var _parsedFormula;
        var _calcID = Number.MIN_VALUE + 1;
    }
    return FormulaInfo;
})();

var ValueChangedArgs = (function () {
    function ValueChangedArgs(row, col, value) {
        this.row = row;
        this.col = col;
        this.value = value;
        this.getRowIndex = function () {
            return this.row;
        };
        this.setRowIndex = function (value) {
            this.row = value;
        };
        this.getColIndex = function () {
            return this.col;
        };
        this.setColIndex = function (value) {
            this.col = value;
        };
        this.getValue = function () {
            return this.value;
        };
        this.setValue = function (value) {
            this.value = value;
        };
    }
    return ValueChangedArgs;
})();

var FormulaParsing = (function () {
    function FormulaParsing(_text) {
        this._text = _text;
        this.getText = function () {
            return this._text;
        };
        this.setText = function (value) {
            this._text = value;
        };
    }
    return FormulaParsing;
})();

var UnknownFunctionEventArgs = (function () {
    function UnknownFunctionEventArgs() {
        /// <summary>
        /// Gets the name of the unknown function.
        /// </summary>
        this.getMissingFunctionName = function () {
            return this.m_missingFunctionName;
        };
        this.setMissingFunctionName = function (value) {
            this.m_missingFunctionName = value;
        };
        this.getCellLocation = function () {
            return this.m_cellLocation;
        };
        this.setCellLocation = function (value) {
            this.m_cellLocation = value;
        };
    }
    return UnknownFunctionEventArgs;
})();

var LookUps = (function () {
    function LookUps() {
        this.getLinearLookUpList = function () {
            return this._linearLookUpList;
        };
        this.setLinearLookUpList = function (value) {
            this._linearLookUpList = value;
        };
        this.getMatchLookUpList = function () {
            return this._matchLookUpList;
        };
        this.setMatchLookUpList = function (value) {
            this._matchLookUpList = value;
        };
    }
    return LookUps;
})();;
/*!
*  filename: ej.checkbox.js
*  version : 12.1
*  Copyright Syncfusion Inc. 2001 - 2013. All rights reserved.
*  Use of this code is subject to the terms of our license.
*  A copy of the current license can be obtained at any time by e-mailing
*  licensing@syncfusion.com. Any infringement will be prosecuted under
*  applicable laws. 
*/

/**
* @fileOverview Plugin to style the Html CheckBox elements
* @copyright Copyright Syncfusion Inc. 2001 - 2013. All rights reserved.
*  Use of this code is subject to the terms of our license.
*  A copy of the current license can be obtained at any time by e-mailing
*  licensing@syncfusion.com. Any infringement will be prosecuted under
*  applicable laws. 
* @version 12.1 
* @author <a href="mailto:licensing@syncfusion.com">Syncfusion Inc</a>
*/

(function ($, ej, undefined) {
    /**
   * @namespace ej
   * @class ejCheckBox
   * @requires jQuery
   * @requires ej.core.js
   * @requires ej.checkbox.js
   * @classdesc Custom Design for Html CheckBox Control.
   * @example 
   
   *&lt;input type="checkbox" id="chkbox"/&gt;
   *&lt;label for="chkbox"&gt;Experienced&lt;/label&gt;
   * &lt;script&gt; 
   * // Create Checkbox  
   * $("#chkbox").ejCheckBox(); 
   * &lt;/script&gt;
   */

    ej.widget("ejCheckBox", "ej.CheckBox", {
        _rootCSS: "e-checkbox",
        // widget element will be automatically set in this
        element: null,

        // user defined model will be automatically set in this
        model: null,
        validTags: ["input"],
        _setFirst: false,

        // default model
        defaults: {
            /**
			* Specifies the id atribute of the Checkbox.
			* @default null
			* @type {String}
			* @example 
			*&lt;input type="checkbox" id="chkbox"/&gt;
   *&lt;label for="chkbox"&gt;Experienced&lt;/label&gt;
   * &lt;script&gt; 
			* //To set id API value during initialization  
			* $("#chkbox").ejCheckBox({  id: "sync" });
			* &lt;/script&gt;
			* @memberof ejCheckBox
			* @instance
			*/
            id: null,
            /**
			* Specifies the name attribute of the Checkbox.
			* @default null
			* @type {String}
			* @example 
			*&lt;input type="checkbox" id="chkbox"/&gt;
   *&lt;label for="chkbox"&gt;Experienced&lt;/label&gt;
   * &lt;script&gt; 
			* //To set name API value during initialization  
			* $("#chkbox").ejCheckBox({  name: "sync" });
			* &lt;/script&gt;
			* @memberof ejCheckBox
			* @instance
			*/
            name: null,
            /**
			* Specifies the value attribute of the Checkbox.
			* @default null
			* @type {String}
			* @example 
			*&lt;input type="checkbox" id="chkbox"/&gt;
   *&lt;label for="chkbox"&gt;Experienced&lt;/label&gt;
   * &lt;script&gt; 
			* //To set value API value during initialization  
			* $("#chkbox").ejCheckBox({ value: "Hello World"});
			* &lt;/script&gt;
			* @memberof ejCheckBox
			* @instance
			*/
            value: null,
            /**
			* Specifies the check attribute of the Checkbox.
			* @default false
			* @type {Boolean}
			* @example 
			*&lt;input type="checkbox" id="chkbox"/&gt;
   *&lt;label for="chkbox"&gt;Experienced&lt;/label&gt;
   * &lt;script&gt; 
			* //To set check API value during initialization  
			* $("#chkbox").ejCheckBox({ checked:  true });
			* &lt;/script&gt;
			* @memberof ejCheckBox
			* @instance
			*/
            checked: false,
            /**
			* Specifies the checkbox control state.
			* @default true
			* @type {boolean}
			* @example 
			*&lt;input type="checkbox" id="chkbox"/&gt;
   *&lt;label for="chkbox"&gt;Experienced&lt;/label&gt;
   * &lt;script&gt; 
			* //To Enable checkbox on initialization. 
			* //To set width API value 
			* $("#chkbox").ejCheckBox ({ enabled: true });
			* &lt;/script&gt;
			* @memberof ejCheckBox
			* @instance
			*/
            enabled: true,
            /**
			* Specifies the enable or disable Tri-State for checkbox control.
			* @default false
			* @type {boolean}
			* @example 
			*&lt;input type="checkbox" id="chkbox"/&gt;
   *&lt;label for="chkbox"&gt;Experienced&lt;/label&gt;
   * &lt;script&gt; 
			* // Specifies to enable or disable Tri-State option checkbox while initialization. 
			* //To set enableTriState API value 
			* $("#chkbox").ejCheckBox({  enableTriState: true });
			* &lt;/script&gt;
			* @memberof ejCheckBox
			* @instance
			*/
            enableTriState: false,
            /**
			* Specify the rounded corner to checkbox
			* @default false
			* @type {boolean}
			* @example 
			*&lt;input type="checkbox" id="chkbox"/&gt;
   *&lt;label for="chkbox"&gt;Experienced&lt;/label&gt;
   * &lt;script&gt; 
			* //To Set the rounded corner during initialization.
			* $("#chkbox").ejCheckBox({ showRoundedCorner: true });
			* &lt;/script&gt;
			* @memberof ejCheckBox
			* @instance
			*/
            showRoundedCorner: false,
            /**
			* Specifies the persist property for Checkbox while initialization. The persist API save current model value to browser cookies for state maintains. While refreshing the Checkbox control page the model value apply from browser cookies. 
			* @default false
			* @type {boolean}
			* @example 
			*&lt;input type="checkbox" id="chkbox"/&gt;
   *&lt;label for="chkbox"&gt;Experienced&lt;/label&gt;
   * &lt;script&gt; 
			* //To set persist API value 
			* $("#chkbox").ejCheckBox({ enablePersistence : false });
			* &lt;/script&gt;
			* @memberof ejCheckBox
			* @instance
			*/
            enablePersistence: false,
            /**
			* Specify the CSS class to CheckBox to achieve custom theme.
			* @default ""
			* @type {string}
			* @example
			*&lt;input type="checkbox" id="chkbox"/&gt;
   *&lt;label for="chkbox"&gt;Experienced&lt;/label&gt;
   * &lt;script&gt;  
			* // Set the root class for Checkbox control theme. This cssClass API helps to use custom skinning option for Checkbox control. By defining the root class using this API, we need to include this root class in CSS. 			
			* $("#chkbox").ejCheckBox({cssClass: "gradient-lime"}); 
			* &lt;/script&gt;
			* @memberof ejCheckBox
			* @instance
			*/
            cssClass: "",
            /**
			* Specifies the text content for CheckBox.
			* @default ""
			* @type {string}
			* @example 
			*&lt;input type="checkbox" id="chkbox"/&gt;
   
   * &lt;script&gt; 
			* // To set text API value 
			* $("#chkbox").ejCheckBox({ text: "Hello World"});
			* &lt;/script&gt;
			* @memberof ejCheckBox
			* @instance
			*/
            text: "",
            /**
			* Specify the Right to Left direction to CheckBox
			* @default false
			* @type {boolean}
			* @example 
			*&lt;input type="checkbox" id="chkbox"/&gt;
   *&lt;label for="chkbox"&gt;Experienced&lt;/label&gt;
   * &lt;script&gt; 
			* // Set the RTL during initialization.
			* $("#chkbox").ejCheckBox({  enableRTL : true });
			* &lt;/script&gt;
			* @memberof ejCheckBox
			* @instance
			*/
            enableRTL: false,
            /**
			* Specify the idprefix value to be added before the current id of the checkbox.
			* @default "ej"
			* @type {String}
			* @example 
			*&lt;input type="checkbox" id="chkbox"/&gt;
   *&lt;label for="chkbox"&gt;Experienced&lt;/label&gt;
   * &lt;script&gt; 
			* // To set  idPrefix  API value
			* $("#chkbox").ejCheckBox ({  idPrefix : "ej" });
			* &lt;/script&gt;
			* @memberof ejCheckBox
			* @instance
			*/
            idPrefix: "ej",
            /**
			* Specifies the size of the CheckBox.See {@link CheckboxSize}
			* @default "mini"
			* @type {enum}
			* @example 
			*&lt;input type="checkbox" id="chkbox"/&gt;
   *&lt;label for="chkbox"&gt;Experienced&lt;/label&gt;
   * &lt;script&gt; 
			* //To set size API value during initialization
			* $("#chkbox").ejCheckBox({  size: "medium"});
			* &lt;/script&gt;
			* @memberof ejCheckBox
			* @instance
			*/
            size: "small",
            /**
           * Specifies the State of ChecKBox.See {@link CheckState}
           * @default null
           * @type {enum}
           * @example 
           *&lt;input type="checkbox" id="chkbox"/&gt;
           *&lt;label for="chkbox"&gt;Experienced&lt;/label&gt;
           * &lt;script&gt; 
           * //To set CheckState API value during initialization
           * $("#chkbox").ejCheckBox({ enableTriState: true , checkState:"indeterminate"});
           * &lt;/script&gt;
           * @memberof ejCheckBox
           * @instance
           */
            checkState: null,
            /**
            * Fires before the Checkbox is going to changed its state successfully
            * @event
            * @name ejCheckBox#beforeChange
            * @param {Object} argument Event parameters from CheckBox     
            * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
            * @param {boolean} argument.model returns the Checkbox model
            * @param {boolean} argument.type returns the name of the event
            * @param {boolean} argument.data.element returns the current element
            * @param {boolean} argument.data.isChecked  returns the status of the element
            * @example 
            *&lt;input type="checkbox" id="chkbox"/&gt;
   *&lt;label for="chkbox"&gt;Experienced&lt;/label&gt;
   * &lt;script&gt;  
            * //To create beforeChange event for checkbox
            * $("#chkbox").ejCheckBox({
            * beforeChange: function (args) {}
            * });
            * &lt;/script&gt;
            * @memberof ejCheckBox
            * @instance
            */
            beforeChange: null,
            /**
            * Fires when the Checkbox state  is changed successfully
            * @event
            * @name ejCheckBox#change
            * @param {Object} argument Event parameters from CheckBox
            * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
            * @param {boolean} argument.model returns the Checkbox model
            * @param {boolean} argument.type returns the name of the event
            * @param {boolean} argument.data.element returns the current element
            * @param {boolean} argument.data.isChecked  returns the status of the element
            * @example
            *&lt;input type="checkbox" id="chkbox"/&gt;
   *&lt;label for="chkbox"&gt;Experienced&lt;/label&gt;
   * &lt;script&gt; 
            * // change event for checkbox
            * $("#chkbox").ejCheckBox({
            * change: function (args) {}
            * });
            * &lt;/script&gt;
            * @memberof ejCheckBox
            * @instance
            */
            change: null,
            /**
			 * Fires when the Checkbox state  is created successfully
			 * @event
			 * @name ejCheckBox#create
			 * @param {Object} argument Event parameters from CheckBox
			 * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
             * @param {boolean} argument.model returns the Checkbox model
             * @param {boolean} argument.type returns the name of the event             
			 * @example 
			 *&lt;input type="checkbox" id="chkbox"/&gt;
   *&lt;label for="chkbox"&gt;Experienced&lt;/label&gt;
   * &lt;script&gt; 
			 * //To create event for checkbox
             * $("#chkbox").ejCheckBox({
             *   create: function (args) {}
             * });    
             * &lt;/script&gt;
			 * @memberof ejCheckBox
			 * @instance
			 */
            create: null,
            /**
			 * Fires when the Checkbox state  is destroyed successfully
			 * @event
			 * @name ejCheckBox#destroy
			 * @param {Object} argument Event parameters from CheckBox
			 * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
             * @param {boolean} argument.model returns the Checkbox model
             * @param {boolean} argument.type returns the name of the event             
			 * @example 
			 *&lt;input type="checkbox" id="chkbox"/&gt;
   *&lt;label for="chkbox"&gt;Experienced&lt;/label&gt;
   * &lt;script&gt; 
			 * //To create destroy event for checkbox
             * $("#chkbox").ejCheckBox({
             * destroy: function (args) {}
             * });
             * &lt;/script&gt;
			 * @memberof ejCheckBox
			 * @instance
			 */
            destroy: null

        },
        /**
        * Specify the data types for default properties 
        * @private
        */
        dataTypes: {
            id: "string",
            name: "string",
            checked: "boolean",
            enablePersistence: "boolean",
            enableTriState: "boolean",
            size: "enum",
            checkState: "enum",
            enabled: "boolean",
            idPrefix: "string"
        },
        /**
        * Create the CheckBox widget
        * @private
        */

        // constructor function
        _init: function () {
            this._setValue();
            this._renderControl();
            if (this.model.enabled)
                this._wireEvents();
            this._setEnabled(this.model.enabled);
        },
        /**
         * To configure the properties at runtime using SetModel		
		 * @private
         */

        _setModel: function (options) {
            if (!(this.model.enabled) && ej.isNullOrUndefined(options["enabled"])) return false;
            for (var prop in options) {
                switch (prop) {
                    case "checked":
                        this._isChecked = options[prop];
                        if (this._isChecked)
                            this._checked();
                        else
                            this._unChecked();
                        break;
                    case "enableTriState":
                        if (options[prop]) {
                            this._setIndeterminate(this._indeterminateState);
                        }
                        break;
                    case "checkState":
                        if (this.model.enableTriState) {
                            this.model.checkState = options[prop];
                            this._changeState(this.model.checkState);
                        }
                        break;
                    case "cssClass": this._changeSkin(options[prop]); break;
                    case "enableRTL":
                        if (options[prop])
                            $("#" + this.model.idPrefix + this.model.id + "_wrapper").addClass("e-rtl");
                        else
                            $("#" + this.model.idPrefix + this.model.id + "_wrapper").removeClass("e-rtl");
                        break;
                    case "text": this._setText(options[prop]); break;
                    case "id": this._setIdAttr(options[prop]); break;
                    case "name": this.chkbx.attr('name', options[prop]); break;
                    case "value": this.chkbx.attr('value', options[prop]); break;
                    case "size": this._setSize(options[prop]); break;
                    case "showRoundedCorner": this._setRoundedCorner(options[prop]); break;
                    case "enabled": this._setEnabled(options[prop]); break;
                }
            }
        },
        /**
        * destroy the CheckBox widget
		* all events bound using this._on will be unbind automatically and bring the control to pre-init state.
		* @alias destroy
		* @return jQuery
		* @example 
		*&lt;input type="checkbox" id="chkbox"/&gt;
   *&lt;label for="chkbox"&gt;Experienced&lt;/label&gt;
   * &lt;script&gt; 
        * $("#chkbox").ejCheckBox();
		* // Create Checkbox instance
		* var chkObj = $("#chkbox").data("ejCheckBox");
		* chkObj.destroy();// Destroy the CheckBox control
		* &lt;/script&gt;
		* @example 
		*&lt;input type="checkbox" id="chkbox"/&gt;
   *&lt;label for="chkbox"&gt;Experienced&lt;/label&gt;
   * &lt;script&gt; 
        * $("#chkbox").ejCheckBox();
		* //To destroy the CheckBox control
		* $("#chkbox").ejCheckBox("destroy");
		* &lt;/script&gt;
		* @memberof ejCheckBox
		* @instance
         */

        // all events bound using this._on will be unbind automatically
        _destroy: function () {
            this.chkbx.removeClass("e-checkbox e-input");
            this.chkbx.insertBefore(this.element);
            this.element.remove();
            this.element = this.chkbx;
        },
        /**
         * To configure the custom theme for checkbox using cssClass property		
		 * @private
         */
        _changeSkin: function (skin) {
            if (this.model.cssClass != skin) {
                this.element.removeClass(this.model.cssClass).addClass(skin);
                $("#" + this.model.idPrefix + this.model.id + "_wrapper").removeClass(this.model.cssClass).addClass(skin);
            }
        },
        /**
         * To configure the Value		
		 * @private
         */
        _setValue: function () {
            this._indeterminateState = false;
            !ej.isNullOrUndefined(this.element.attr("id")) && (this.model.id = this.element.attr('id'));
            !ej.isNullOrUndefined(this.element.attr('name')) && (this.model.name = this.element.attr('name'));
            !ej.isNullOrUndefined(this.element.attr('value')) && (this.model.value = this.element.attr('value'));
            ej.isNullOrUndefined(this.model.name) && (this.model.name = this.model.id);
            this._isChecked = this.element.attr('checked');            
            this._isChecked = this._isChecked != undefined ? this._isChecked : this.model.checked;
        },
        /**
         * To configure the Size of the CheckBox		
		 * @private
         */
        _setSize: function (val) {
            if (val == ej.CheckboxSize.Medium)
                this.innerdiv.removeClass('e-chkbox-small').addClass('e-chkbox-medium');
            else
                this.innerdiv.removeClass('e-chkbox-medium').addClass('e-chkbox-small');
        },
        /**
         * To configure the showRoundedCorner of the CheckBox		
		 * @private
         */
        _setRoundedCorner: function (val) {
            if (val)
                this.span.addClass("e-corner-all");
            else
                this.span.removeClass("e-corner-all");
        },
        /**
         * To configure the Enable status of the CheckBox		
		 * @private
         */
        _setEnabled: function (val) {
            if (val) {
                this.enable();
            } else {
                this.disable();
            }
        },
        _setCheckBoxState: function () {
            if (this.model.enableTriState) {
                if (this.model.checkState == "indeterminate")
                    this._indeterminateState = true
                else if (this.model.checkState == "check") {
                    this.model.checked = true;
                    this._isChecked = this.model.checked == null ? this._isChecked : this.model.checked;
                }
                else if (this.model.checkState == "uncheck") {
                    this.model.checked = false;
                    this._isChecked = this.model.checked == null ? this._isChecked : this.model.checked;
                }
            }
        },
        /**
         * Render Section For CheckBox control		
		 * @private
         */
        _renderControl: function () {
            this._setCheckBoxState();
            this.maindiv = ej.buildTag("span.e-chkbox-wrap e-widget " + this.model.cssClass + "#" + this.model.idPrefix + this.model.id, "", {}, { "role": "checkbox", tabindex: 0 });
            this.innerdiv = $('<div></div>');
            this._setSize(this.model.size);
            this.span = $('<span></span>');
            this.spanImg = $('<span class="e-chk-image"></span>').attr("role", "presentation");
            this.element.addClass("e-input");
            this.model.name = ej.isNullOrUndefined(this.model.name) ? this.model.id : this.model.name;
            this.element.attr({ "id": this.model.id, "name": this.model.name, "value": this.model.value });
            //this.span.addClass("e-innerspan");
            this._setRoundedCorner(this.model.showRoundedCorner);
            if (this._isChecked) {
                this.spanImg.addClass("e-chk-activeicon");
                this.span.addClass("e-chk-act");
                this.maindiv.attr({ "aria-checked": true });
            }
            else {
                this.span.addClass("e-chk-inact");
                this.maindiv.attr({ "aria-checked": false });
            }
            this.span.append(this.spanImg);
            this.innerdiv.append(this.span);
            this.maindiv.insertBefore(this.element);
            this.maindiv.append(this.element);
            this.maindiv.append(this.innerdiv);
            this._setTextWrapper(this.model.text);
            this.chkbx = this.element;
            this.element = this.maindiv;
            if (this.model.enableTriState == true && this._indeterminateState == true)
                this._setIndeterminate(this._indeterminateState);
        },
        _changeState: function (state) {
            if (state == "indeterminate") {
                this.spanImg.removeClass("e-chk-activeicon").addClass("e-chk-indetericon");
                this.span.removeClass("e-chk-act").removeClass("e-chk-inact").addClass("e-chk-indeter");
                this.maindiv.attr({ "aria-checked": "mixed" });
                this.element.find('input').prop('enableTriState', true);
                this.model.checked = null;
            }
            else if (state == "check") {
                this.spanImg.removeClass("e-chk-indetericon").addClass("e-chk-activeicon");
                this.span.removeClass("e-chk-act").removeClass("e-chk-inact").removeClass("e-chk-indeter").addClass("e-chk-act");
                this.maindiv.attr({ "aria-checked": true });
            }
            else if (state == "uncheck") {
                this.spanImg.removeClass("e-chk-activeicon").removeClass("e-chk-indetericon");
                this.span.removeClass("e-chk-act").removeClass("e-chk-indeter").addClass("e-chk-inact");
                this.maindiv.attr({ "aria-checked": false });
            }
        },
        /**
        * To configure checkbox tri-state type		
        * @private
        */
        _setIndeterminate: function (indeter) {
            if (indeter) {
                this.spanImg.removeClass("e-chk-activeicon").addClass("e-chk-indetericon");
                this.span.removeClass("e-chk-act").removeClass("e-chk-inact").addClass("e-chk-indeter");
                this.maindiv.attr({ "aria-checked": "mixed" });
                this.element.find('input').prop('enableTriState', true);
                this.model.checked = null;
            }
            else {
                this.element.find(".e-chk-indeter").removeClass("e-chk-indeter");
                this.element.find(".e-chk-image").removeClass("e-chk-indetericon");
                this.element.find('input').removeAttr('enableTriState');
                this.element.find('input').prop('enableTriState', false);
                if (this.model.checked)
                    this._checked();
                else
                    this._unChecked();
            }
        },
        /**
         * To configure checkbox textwrapper		
		 * @private
         */
        _setTextWrapper: function (val) {
            if (val != "") {
                //this.textWrapDiv = ej.buildTag("div.e-chkbox-wrap " + this.model.cssClass + "#" + this.model.idPrefix + this.model.id + "_wrapper");
                //this.maindiv.wrap(this.textWrapDiv);
                this.txtSpan = ej.buildTag("div.e-text", val);
                this.maindiv.append(this.txtSpan);
                if (this.model.enableRTL)
                    $("#" + this.model.idPrefix + this.model.id + "_wrapper").addClass("e-rtl");
            }
        },
        /**
         * To configure checkbox text value
		 * @private
         */
        _setText: function (val) {
            if ((this.model.text == "") && (val != "")) {
                this._setTextWrapper(val);
            } else {
                this.txtSpan.html(val);
            }
        },
        /**
         * To configure checkbox ID attribute value
		 * @private
         */
        _setIdAttr: function (val) {
            $("#" + this.model.idPrefix + this.model.id + "_wrapper").attr('id', this.model.idPrefix + val + "_wrapper");
            this.chkbx.attr('id', val);
        },
        /**
        * Wiring the events to checkbox control
        * @private
        */
        _wireEvents: function () {
            this._on(this.element, "click", this._checkedHandler);
            this._on(this.element, "focus", this._focusIn);
            this._on(this.element, "focusout", this._focusOut);
        },
        /**
         * Unwiring the events to checkbox control
		 * @private
         */

        _unWireEvents: function () {
            this._off(this.element, "click");
            this._off(this.element, "focus");
            this._off(this.element, "focusout");
        },
        _focusIn: function (evt) {
            $(this.element).addClass("e-focus");
            $(this.element).bind("keypress", $.proxy(this._checkUnCheck, this));
        },
        _focusOut: function (evt) {
            $(this.element).removeClass("e-focus");
            $(this.element).unbind("keypress", $.proxy(this._checkUnCheck, this));
        },
        _checkUnCheck: function (evt) {
            //Space bar to check and uncheck
            if (evt.keyCode == 32) {
                evt.preventDefault();
                this._checkedHandler();
            }
        },
        _checkedHandler: function (evt) {
            var data = { isChecked: this._isChecked, event: evt };
            if (true == this._trigger("beforeChange", data)) {
                return false;
            }
            if (this.element.find("span:first").hasClass("e-chk-inact")) {
                this._checked();
                this._isChecked = this.model.checked = true;
                if (this.model.enableTriState) {
                    this._indeterminateState = true;
                }
            }
            else if (this.element.find("span:first").hasClass("e-chk-act")) {
                if ((this.model.enableTriState == true) && (this._indeterminateState == true)) {
                    this._setIndeterminate(true);
                    this._isChecked = this.model.checked = true;
                } else {
                    this._unChecked();
                    this._isChecked = this.model.checked = false;
                }


            }
            else if (this.element.find("span:first").hasClass("e-chk-indeter")) {
                this._isChecked = this.model.checked = false;
                this._setIndeterminate(false);
                this._indeterminateState = false;
            }
            var data = { isChecked: this._isChecked, event: evt };
            this._trigger("change", data);
            return true;
        },
        /**
         * Section For handling the Check event handler
		 * @private
         */

        _checked: function () {
            this.element.find("span:first").removeClass("e-chk-inact").addClass("e-chk-act");
            this.element.find(".e-chk-image").removeClass("e-chk-indetericon").addClass("e-chk-activeicon");
            this.maindiv.attr({ "aria-checked": true });
            this.element.find('input[type=checkbox]').prop('checked', true);
            this.model.checkState = "check";
        },
        /**
         * Section For handling the Uncheck event handler
		 * @private
         */

        _unChecked: function () {
            this.element.find("span:first").removeClass("e-chk-act").addClass("e-chk-inact");
            this.maindiv.attr({ "aria-checked": false });
            this.element.find(".e-chk-image").removeClass("e-chk-activeicon").removeClass("e-chk-indetericon");
            this.element.find('input[type=checkbox]').removeAttr('checked');
            this.model.checkState = "uncheck";
        },
        /**
        * To disable the checkbox
		* @return jQuery
		* @example 
		*&lt;input type="checkbox" id="chkbox"/&gt;
   *&lt;label for="chkbox"&gt;Experienced&lt;/label&gt;
   * &lt;script&gt; 
        * $("#chkbox").ejCheckBox();
		* // Create Checkbox instance 
		* var chkObj = $("#chkbox").data("ejCheckBox");
		* chkObj.disable(); //disables the CheckBox
		* &lt;/script&gt;
		* @example 
		*&lt;input type="checkbox" id="chkbox"/&gt;
   *&lt;label for="chkbox"&gt;Experienced&lt;/label&gt;
   * &lt;script&gt; 
        * $("#chkbox").ejCheckBox();
		* //To disable the CheckBox
		* $("#chkbox").ejCheckBox("disable");
		* &lt;/script&gt;
		* @memberof ejCheckBox
		* @instance
        */

        disable: function () {
            if (!this.element.hasClass("e-disable")) {
                this.element.addClass("e-disable").attr("aria-disabled", true);
                this._unWireEvents();
                this.model.enabled = false;
            }
        },
        /**
        * To enable the checkbox
		* @return jQuery
		* @example 
		*&lt;input type="checkbox" id="chkbox"/&gt;
   *&lt;label for="chkbox"&gt;Experienced&lt;/label&gt;
   * &lt;script&gt; 
        * $("#chkbox").ejCheckBox();
		* // Create Checkbox instance 
		* var chkObj = $("#chkbox").data("ejCheckBox");
		* chkObj.enable(); // enables the CheckBox
		* &lt;/script&gt;
		* @example 
		*&lt;input type="checkbox" id="chkbox"/&gt;
   *&lt;label for="chkbox"&gt;Experienced&lt;/label&gt;
   * &lt;script&gt; 
        * $("#chkbox").ejCheckBox();
		* //To enable the CheckBox
		* $("#chkbox").ejCheckBox("enable");
		* &lt;/script&gt;
		* @memberof ejCheckBox
		* @instance
        */
        enable: function () {
            if (this.element.hasClass("e-disable")) {
                this.element.removeClass("e-disable").attr("aria-disabled", false);
                this._wireEvents();
                this.model.enabled = true;
            }
        },
        /**
        * To Check the status of checkbox
		* @return jQuery
		* @example 
		*&lt;input type="checkbox" id="chkbox"/&gt;
   *&lt;label for="chkbox"&gt;Experienced&lt;/label&gt;
   * &lt;script&gt; 
        * $("#chkbox").ejCheckBox();
		* // Create Checkbox  instance
		* var chkObj = $("#chkbox").data("ejCheckBox");
		* chkObj.isChecked(); // check the status of checkbox
		* &lt;/script&gt;
		* @example 
		*&lt;input type="checkbox" id="chkbox"/&gt;
   *&lt;label for="chkbox"&gt;Experienced&lt;/label&gt;
   * &lt;script&gt; 
        * $("#chkbox").ejCheckBox();
		* //To check the status of checkbox
		* $("#chkbox").ejCheckBox("isChecked");
		* &lt;/script&gt;
		* @memberof ejCheckBox
		* @instance
        */
        isChecked: function () {
            if ((this._isChecked != null) && (this._isChecked != undefined))
                return this._isChecked;
        }
    });
    /**
	 * Enum for various checkbox sizes	 
	 * @enum {string}
	 * @global 
	 */
    ej.CheckboxSize = {
        /**  Creates checkbox with inbuilt small size height, width specified */
        Small: "small",
        /**  Creates checkbox with inbuilt medium size height, width specified */
        Medium: "medium"
    };
    /**
    * Enum for various checkbox states	 
    * @enum {string}
    * @global 
    */
    ej.CheckState = {
        /**  Specifies the Check attribute of the Checkbox */
        Check: "check",
        /**  Specifies the Uncheck attribute of the Checkbox */
        Uncheck: "uncheck",
        /**  Specifies the Indeterminate state of the Checkbox */
        Indeterminate: "indeterminate"
    };
})(jQuery, Syncfusion);;;
/*!
*  filename: ej.radiobutton.js
*  version : 12.1
*  Copyright Syncfusion Inc. 2001 - 2013. All rights reserved.
*  Use of this code is subject to the terms of our license.
*  A copy of the current license can be obtained at any time by e-mailing
*  licensing@syncfusion.com. Any infringement will be prosecuted under
*  applicable laws. 
*/

/**
* @fileOverview Plugin to style the Html Radiobutton elements
* @copyright Copyright Syncfusion Inc. 2001 - 2013. All rights reserved.
*  Use of this code is subject to the terms of our license.
*  A copy of the current license can be obtained at any time by e-mailing
*  licensing@syncfusion.com. Any infringement will be prosecuted under
*  applicable laws. 
* @version 12.1 
* @author <a href="mailto:licensing@syncfusion.com">Syncfusion Inc</a>
*/
(function ($, ej, undefined) {
    /**
    * @namespace ej
	* @class ejRadioButton
	* @requires jQuery
   * @requires ej.core.js
   * @requires ej.radiobutton.js
	* @classdesc Custom Design for Html RadioButton Control. 
	* @example
	*&lt;input type="radio" id="radiobtn"/&gt;
	*&lt;label for="radiobtn"&gt;Experienced&lt;/label&gt;
    *&lt;br/&gt; &lt;input type="radio" name="radiobutn" id="radiobtn1"/&gt;
	*&lt;label for="radiobtn1"&gt; Fresher &lt;/label&gt;
	*&lt;script&gt;
	*$("#radiobtn").ejRadioButton({checked:true});
    *$("#radiobtn1").ejRadioButton();
	*&lt;/script&gt;
	*/
    ej.widget("ejRadioButton", "ej.RadioButton", {
        _rootCSS: "e-radiobtn",
        // widget element will be automatically set in this
        element: null,

        // user defined model will be automatically set in this
        model: null,
        validTags: ["input"],
        _setFirst: false,

        // default model
        defaults: {
            /**		
			* Specifies the id attribute for the Radio Button while initialization.
			* @default null
			* @type {String}
			* @example 
			*&lt;input type="radio" name="radiobtn" id="radiobtn"/&gt;
			*&lt;label for="radiobtn"&gt;Experienced&lt;/label&gt;
             *&lt;br/&gt; &lt;input type="radio" name="radiobtn" id="radiobtn1"/&gt;
	         *&lt;label for="radiobtn1"&gt;Fresher &lt;/label&gt;
			*&lt;script&gt;
			* //To set id API value during initialization  
			* 	$("#radiobtn").ejRadioButton({  id: "sync" });
            * 	$("#radiobtn1").ejRadioButton({  id: "sync1" });
			 * &lt;/script&gt;
			 * @memberof ejRadioButton
			* @instance
			*/
            id: null,
            /**		
			* Specifies the name attribute for the Radio Button while initialization.
			* @default null
			* @type {String}
			 * @memberof ejRadioButton
			* @instance
			*/
            name: null,
            /**		
			* Specifies the value atribute of the Radio Button.
			* @default null
			* @type {String}
			* @example 
			*&lt;input type="radio" name="radiobutn" id="radiobtn"/&gt;
			*&lt;label for="radiobtn"&gt;Experienced&lt;/label&gt;
             *&lt;br/&gt; &lt;input type="radio" name="radiobutn" id="radiobtn1"/&gt;
	         *&lt;label for="radiobtn1"&gt;Fresher &lt;/label&gt;
	        *&lt;script&gt;
			* //To set value API value during initialization  
			* 	$("#radiobtn").ejRadioButton({ value: "Experienced"});
            * 	$("#radiobtn1").ejRadioButton({ value: "Fresher"});
			* &lt;/script&gt;		
			* @memberof ejRadioButton
			* @instance
			*/
            value: null,
            /**		
			* Specifies the check atribute of the Radio Button.
			* @default false
			* @type {Boolean}
			* @example 
			*&lt;input type="radio" name="radiobutn" id="radiobtn"/&gt;
			*&lt;label for="radiobtn"&gt;Experienced&lt;/label&gt;
             *&lt;br/&gt; &lt;input type="radio" name="radiobutn" id="radiobtn1"/&gt;
	         *&lt;label for="radiobtn1"&gt;Fresher &lt;/label&gt;
	*&lt;script&gt;
			* //To set check API value during initialization  
			* 	$("#radiobtn").ejRadioButton({ checked:  true });	
            * 	$("#radiobtn1").ejRadioButton({ checked:  true });
			  * &lt;/script&gt;
			 * @memberof ejRadioButton
			* @instance
			*/
            checked: false,
            /**		
			* Specify the CSS class to RadioButton to achieve custom theme.
			* @default ""
			* @type {string}
			* @example 
			*&lt;input type="radio" name="radiobutn" id="radiobtn"/&gt;
			*&lt;label for="radiobtn"&gt;Experienced&lt;/label&gt;
             *&lt;br/&gt; &lt;input type="radio" name="radiobutn" id="radiobtn1"/&gt;
	         *&lt;label for="radiobtn1"&gt;Fresher &lt;/label&gt;
	*&lt;script&gt;
			* // Set the root class for RadioButton control theme. This cssClass API helps to use custom skinning option for RadioButton control. By defining the root class using this API, we need to include this root class in CSS. 			
			* 	$("#radiobtn").ejRadioButton({cssClass: "gradient-lime"});
            * 	$("#radiobtn1").ejRadioButton({cssClass: "gradient-lime"});
			* &lt;/script&gt;
			* @memberof ejRadioButton
			* @instance
			*/
            cssClass: "",
            /**		
			* Specifies the text content for RadioButton.
			* @default ""
			* @type {string}
			* @memberof ejRadioButton
			* @instance
			*/
            text: "",
            /**		
			* Specify the Right to Left direction to RadioButton
			* @default false
			* @type {boolean}
			* @example 
			*&lt;input type="radio" name="radiobutn" id="radiobtn"/&gt;
			*&lt;label for="radiobtn"&gt;Experienced&lt;/label&gt;
             *&lt;br/&gt; &lt;input type="radio" name="radiobutn" id="radiobtn1"/&gt;
	         *&lt;label for="radiobtn1"&gt;Fresher &lt;/label&gt;
	*&lt;script&gt;
			* // Set the enableRTL during initialization. 			
			* 	$("#radiobtn").ejRadioButton({  enableRTL:true });	
            * 	$("#radiobtn1").ejRadioButton({  enableRTL:true });	
			* &lt;/script&gt;
			* @memberof ejRadioButton
			* @instance
			*/
            enableRTL: false,
            /**		
			* Specifies the enablePersistence property for RadioButton while initialization. The enablePersistence API save current model value to browser cookies for state maintains. While refreshing the radiobutton control page the model value apply from browser cookies. 
			* @default false
			* @type {boolean}
			* @example 
			*&lt;input type="radio" name="radiobutn" id="radiobtn"/&gt;
			*&lt;label for="radiobtn"&gt;Experienced&lt;/label&gt;
             *&lt;br/&gt; &lt;input type="radio" name="radiobutn" id="radiobtn1"/&gt;
	         *&lt;label for="radiobtn1"&gt;Fresher &lt;/label&gt;
	*&lt;script&gt;
			* // To set enablePersistence API value 
			* $("#radiobtn").ejRadioButton({ enablePersistence: false });	
            * $("#radiobtn1").ejRadioButton({ enablePersistence: false });
			* &lt;/script&gt;
			 * @memberof ejRadioButton
			* @instance
			*/
            enablePersistence: false,
            /**		
			* Specify the idprefix value to be added before the current id of the RadioButton.
			* @default "ej"
			* @type {String}
			* @example 
			*&lt;input type="radio" name="radiobutn" id="radiobtn"/&gt;
			*&lt;label for="radiobtn"&gt;Experienced&lt;/label&gt;
             *&lt;br/&gt; &lt;input type="radio" name="radiobutn" id="radiobtn1"/&gt;
	         *&lt;label for="radiobtn1"&gt;Fresher &lt;/label&gt;
	*&lt;script&gt;
			* // To set  idPrefix  API value
			* $("#radiobtn").ejRadioButton ({  idPrefix : "ej" }); 
            * $("#radiobtn1").ejRadioButton ({  idPrefix : "ej" }); 
			* &lt;/script&gt;
			* @memberof ejRadioButton
			* @instance
			*/
            idPrefix: "ej",
            /**		
			* Specifies the size of the RadioButton.	See {@link RadioButtonSize}
			* @default "small"
			* @type {enum}
			* @example 
			*&lt;input type="radio" name="radiobutn" id="radiobtn"/&gt;
			*&lt;label for="radiobtn"&gt;Experienced&lt;/label&gt;
             *&lt;br/&gt; &lt;input type="radio" name="radiobutn" id="radiobtn1"/&gt;
	         *&lt;label for="radiobtn1"&gt;Fresher &lt;/label&gt;
	        *&lt;script&gt;
			* //To set size API value during initialization  
			* 	$("#radiobtn").ejRadioButton({  size: "medium"});	
            * 	$("#radiobtn1").ejRadioButton({  size: "medium"});
			 * &lt;/script&gt;
			* @memberof ejRadioButton
			* @instance
			*/
            size: "small",
            /**		
			* Specifies the RadioButton control state.
			* @default true
			* @type {boolean}
			* @example 
			*&lt;input type="radio" name="radiobutn" id="radiobtn"/&gt;
			*&lt;label for="radiobtn"&gt;Experienced&lt;/label&gt;
             *&lt;br/&gt; &lt;input type="radio" name="radiobutn" id="radiobtn1"/&gt;
	         *&lt;label for="radiobtn1"&gt;Fresher &lt;/label&gt;
	        *&lt;script&gt;
			* // Enable RadioButton on initialization. 
			* 	//To set width API value 
			* 	 $("#radiobtn").ejRadioButton ({ enabled: true });	
            * 	 $("#radiobtn1").ejRadioButton ({ enabled: true });	
			* &lt;/script&gt;
			 * @memberof ejRadioButton
			* @instance
			*/
            enabled: true,
            /**    
			 * Fires before the RadioButton is going to changed its state successfully
			 * @event
			 * @name ejRadioButton#beforeChange 	
			 * @param {Object} argument Event parameters from RadioButton     
			 * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
             * @param {boolean} argument.model returns the RadioButton model
             * @param {boolean} argument.type returns the name of the event
             * @param {boolean} argument.data.element returns the current element
             * @param {boolean} argument.data.isChecked  returns the status of the element
			 * @example 
			 *&lt;input type="radio" name="radiobutn" id="radiobtn"/&gt;
			*&lt;label for="radiobtn"&gt;Experienced&lt;/label&gt;   
	         *&lt;script&gt;
			 * //beforeChange event for RadioButton
             * $("#radiobtn").ejRadioButton({
             *   beforeChange:function (args){ }
             *});  
             * &lt;/script&gt;         
			 * @memberof ejRadioButton
			 * @instance
			 */
            beforeChange: null,
            /**    
			 * Fires when the RadioButton state  is changed successfully
			 * @event
			 * @name ejRadioButton#change 	
			 * @param {Object} argument Event parameters from RadioButton     
			 * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
             * @param {boolean} argument.model returns the RadioButton model
             * @param {boolean} argument.type returns the name of the event
             * @param {boolean} argument.data.element returns the current element
             * @param {boolean} argument.data.isChecked  returns the status of the element
			 * @example 
			 *&lt;input type="radio" name="radiobutn" id="radiobtn"/&gt;
			*&lt;label for="radiobtn"&gt;Experienced&lt;/label&gt;
	        *&lt;script&gt;
			 * //change event for RadioButton
             * $("#radiobtn").ejRadioButton({
             *   change: function (args){}
             *}); 
             * &lt;/script&gt;          
			 * @memberof ejRadioButton
			 * @instance
			 */
            change: null,
            /**    
            * Fires when the RadioButton created successfully
            * @event
            * @name ejRadioButton#create 	
            * @param {Object} argument Event parameters from RadioButton     
            * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
            * @param {boolean} argument.model returns the RadioButton model
            * @param {boolean} argument.type returns the name of the event
            * @example 
            * &lt;input type="radio" name="radiobutn" id="radiobtn"/&gt;
            * &lt;label for="radiobtn"&gt;Experienced&lt;/label&gt;
            * &lt;script&gt;
            * // //create event for RadioButton
            * $("#radiobtn").ejRadioButton({
            *   create:function(args){}
            *}); 
            * &lt;/script&gt;          
            * @memberof ejRadioButton
            * @instance
            */
            create: null,
            /**    
            * Fires when the RadioButton destroyed successfully
            * @event
            * @name ejRadioButton#destroy 	
            * @param {Object} argument Event parameters from RadioButton     
            * @param {boolean} argument.cancel if the event should be canceled; otherwise, false.
            * @param {boolean} argument.model returns the RadioButton model
            * @param {boolean} argument.type returns the name of the event           
            * @example 
            *&lt;input type="radio" name="radiobutn" id="radiobtn"/&gt;
           *&lt;label for="radiobtn"&gt;Experienced&lt;/label&gt;
           *&lt;script&gt;
            * // //destroy event for RadioButton
            * $("#radiobtn").ejRadioButton({
            *   destroy:function(args){}
            *}); 
            * &lt;/script&gt;          
            * @memberof ejRadioButton
            * @instance
            */
            destroy: null
        },
        /**
         * Specify the data types for default properties 
		 * @private
         */

        dataTypes: {
            id: "string",
            name: "string",
            checked: "boolean",
            enablePersistence: "boolean",
            size: "enum",
            enabled: "boolean",
            idPrefix: "string"
        },
        /**
		 * Create the RadioButton widget
		 * @private
		 */
        // constructor function
        _init: function () {
            this._setValue();
            this._renderControl();
            this._setEnabled(this.model.enabled);
            this._wireEvents();
            if (this.isChecked)
                this._checkedHandler();
        },
        /**
         * To configure the properties at runtime using SetModel		
		 * @private
         */

        _setModel: function (options) {
            if (!(this.model.enabled) && ej.isNullOrUndefined(options["enabled"])) return false;
            for (var prop in options) {
                switch (prop) {
                    case "cssClass": this._changeSkin(options[prop]); break;
                    case "enableRTL":
                        if (options[prop])
                            this.textWrapDiv.addClass("e-rtl");
                        else
                            this.textWrapDiv.removeClass("e-rtl");
                        break;
                    case "text": this._setText(options[prop]); break;
                    case "size": this._setSize(options[prop]); break;
                    case "checked": this.model.checked = options[prop]; this._checkedChange(this.model.checked); break;
                    case "enabled": this._setEnabled(options[prop]); break;
                    case "id": this._setIdAttr(options[prop]); break;
                    case "name": this.radbtn.attr('name', options[prop]); break;
                    case "value": this.radbtn.attr('value', options[prop]); break;
                }
            }
        },
        /**
        * destroy the RadioButton widget
		* all events bound using this._on will be unbind automatically and bring the control to pre-init state.
		* @alias destroy
		* @return jQuery
		* @example 
		*&lt;input type="radio" name="radiobutn" id="radiobtn"/&gt;
	    *&lt;label for="radiobtn"&gt;Experienced&lt;/label&gt;
	   *&lt;script&gt; 
		* // Create RadioButton
		* $("#radiobtn").ejRadioButton();
		* var chkObj = $("#radiobtn").data("ejRadioButton");
		* chkObj.destroy();
		* &lt;/script&gt;
		* @example 
		*&lt;input type="radio" name="radiobutn" id="radiobtn"/&gt;
		*&lt;label for="radiobtn"&gt;Experienced&lt;/label&gt;
	    *&lt;script&gt;
		* // enable the RadioButton
		* $("#radiobtn").ejRadioButton();
		* $("#radiobtn").ejRadioButton("destroy");	
		* &lt;/script&gt;
		* @memberof ejRadioButton
		* @instance
         */

        // all events bound using this._on will be unbind automatically _checkedChange
        _destroy: function () {
            this.radbtn.removeClass("e-radiobtn e-input");
            this.radbtn.insertBefore(this.element);
            this.element.remove();
            this.element = this.radbtn;
        },
        /**
         * To configure the custom theme for RadioButton using cssClass property		
		 * @private
         */
        //Skin Change at run time
        _changeSkin: function (skin) {
            if (this.model.cssClass != skin) {
                this.element.removeClass(this.model.cssClass).addClass(skin);
                this.textWrapDiv.removeClass(this.model.cssClass).addClass(skin);
            }
        },
        /**
         * To configure the Value		
		 * @private
         */
        _setValue: function () {
		
		     if(!ej.isNullOrUndefined(this.element.attr("id")))
		         this.model.id=this.element.attr("id");
		     if(!ej.isNullOrUndefined(this.element.attr("name")))
		         this.model.name=this.element.attr("name");
		     if(!ej.isNullOrUndefined(this.element.attr("value")))
		         this.model.value=this.element.attr("value");
		     this.element.attr({ "id": this.model.id,"name": this.model.name,"value": this.model.value});
             this.isChecked = this.element.attr('checked');
             this.isChecked = this.isChecked != undefined ? this.isChecked : this.model.checked;
        },
        /**
       * To configure Radio button ID attribute value
       * @private
       */
        _setIdAttr: function (val) {
            $("#" + this.model.idPrefix + this.model.id + "_wrapper").attr('id', this.model.idPrefix + val + "_wrapper");
            $("#" + this.model.idPrefix + this.model.id).attr('id', this.model.idPrefix + val);
            this.radbtn.attr('id', val);
        },
        /**
         * To configure the Size of the RadioButton		
		 * @private
         */
        _setSize: function (val) {
            if (val == ej.RadioButtonSize.Medium)
                this.span.removeClass('e-radsmaller').addClass('e-radmedium');
            else
                this.span.removeClass('e-radmedium').addClass('e-radsmaller');
        },
        /**
         * To configure the Enable status of the RadioButton		
		 * @private
         */
        _setEnabled: function (val) {
            if (val) {
                this.enable();
            } else {
                this.disable();
            }
        },
        /**
         * Render Section For RadioButton control		
		 * @private
         */
        _renderControl: function () {
            this.div = $('<div class="e-radiobtn-wrap e-widget" ></div>');
            this.div.attr({'id':this.model.idPrefix + this.model.id,"role":"radio","tabindex":-1,"aria-checked":false});
            this.span = $('<span></span>');
            this.span.addClass("e-spanicon");
            this._setSize(this.model.size);
            this.spanImg = $('<span class="e-rad-icon e-rad-select"></span>', "", {}, {"role":"presentation"});
            this.element.addClass("e-input");
            this.element.attr("id", this.model.id);
            this.div.addClass(this.model.cssClass);
            this.span.append(this.spanImg);
            this.div.insertBefore(this.element);
            this.div.append(this.element);
            this.div.append(this.span);
            this._setTextWrapper(this.model.text);
            this.radbtn = this.element;
            this.element = this.div;
            if (this.isChecked) {
                this.element.find(".e-input").attr('checked', true);
            }
        },
        /**
         * To configure RadioButton textwrapper		
		 * @private
         */
        _setTextWrapper: function (val) {
            if (val != "") {
                this.textWrapDiv = ej.buildTag("div.e-radiobtn-wrap " + this.model.cssClass + "#" + this.model.idPrefix + this.model.id + "_wrapper");
                this.div.wrapAll(this.textWrapDiv);
                this.txtSpan = ej.buildTag("div.e-text", this.model.text);
                this.textWrapDiv = $("#" + this.model.idPrefix + this.model.id + "_wrapper");
                this.textWrapDiv.append(this.txtSpan);
                if (this.model.enableRTL)
                    this.textWrapDiv.addClass("e-rtl");
            }
        },
        /**
         * To configure RadioButton text value		
		 * @private
         */
        _setText: function (val) {
            if ((this.model.text == "") && (val != "")) {
                this._setTextWrapper(val);
            } else {
                this.txtSpan.html(val);
            }
        },
        /**
        * Wiring the events to RadioButton control		
        * @private
        */
        _wireEvents: function () {
            this._on(this.element, "click", this._checkedHandler);
            this._on(this.element, "focus", this._focusIn);
            this._on(this.element, "focusout", this._focusOut);
        },
        _focusIn: function (evt) {
            $(this.element).addClass("e-focus");
        },
        _focusOut: function (evt) {
            $(this.element).removeClass("e-focus");
        },
        _checkedHandler: function (evt) {
            if (!this.element.hasClass('e-disable')) {
                this.isChecked = this.element.find('input.e-radiobtn:radio').attr('checked') == 'checked' ? true : false;
                this._changeEvent();
            }
        },
        /**
         * Section For handle the change in check for radiobutton
		 * @private
         */
        _checkedChange: function (val) {
            this.isChecked = val;
            if ((this.isChecked) && (this.model.enabled))
                this._changeEvent();
        },
        /**
         * Section For handle the change event for radiobutton
		 * @private
         */
        _changeEvent: function () {
            var data = { isChecked: this.isChecked };
            if (true == this._trigger("beforeChange", data)) {
                return false;
            }
            if (!$(this.element).find(".e-rad-icon").hasClass("e-rad-active")) {
                var curname = this.element.find(".e-input").attr('name'),
                input = $('input.e-radiobtn[name=' + curname + ']:radio'),
                proxy = this,
                currElement = this.element.find('.e-input'),
                currObj = $(currElement).data("ejRadioButton");
                if (data.isChecked) {
                    this.spanImg.addClass("e-rad-active").removeClass('e-rad-select');
                    this.div.attr({ "tabindex": 0, "aria-checked": true });
                }
                $.each(input, function (i, obj) {
                    $(obj).parent().parent().find(".e-rad-icon").removeClass("e-rad-active").addClass("e-rad-select");
                    $(obj).parent().parent().find('.e-radiobtn-wrap').attr({ "tabindex": -1, "aria-checked": false });
                    var prevObj = $(obj).data("ejRadioButton");
                    if (prevObj != null) {
                        prevObj.model.checked = false;
                    }
                });
                if (currObj != null)
                    currObj.model.checked = true;
                this.element.find(".e-rad-icon").addClass("e-rad-active").removeClass("e-rad-select");
                this.div.attr({ "tabindex": 0, "aria-checked": true });
                this.element.find(".e-input").click();
                this.isChecked = true;
            }
            var data = { isChecked: this.isChecked };
            this._trigger("change", data)
        },
        /**
        * To disable the RadioButton
		* @return jQuery
		* @example 
		*&lt;input type="radio" name="radiobutn" id="radiobtn"/&gt;
			*&lt;label for="radiobtn"&gt;Experienced&lt;/label&gt;           
	*&lt;script&gt; 
		* // Create RadioButton
		* $("#radiobtn").ejRadioButton();
		* var chkObj = $("#radiobtn").data("ejRadioButton");
		* chkObj.disable();
		* &lt;/script&gt;
		* @example 
		*&lt;input type="radio" name="radiobutn" id="radiobtn"/&gt;
		*&lt;label for="radiobtn"&gt;Experienced&lt;/label&gt;
	    *&lt;script&gt; 
		* // disable the RadioButton
		* $("#radiobtn").ejRadioButton();
		* $("#radiobtn").ejRadioButton("disable");	
		* &lt;/script&gt;
		* @memberof ejRadioButton
		* @instance
        */
        disable: function () {
            if (!this.element.hasClass("e-disable"))
                this.element.addClass("e-disable");
            this.div.attr("aria-disabled", true);
            this.model.enabled = false;
        },
        /**
        * To enable the RadioButton
		* @return jQuery
		* @example 
		*&lt;input type="radio" name="radiobutn" id="radiobtn"/&gt;
		*&lt;label for="radiobtn"&gt;Experienced&lt;/label&gt;
	    *&lt;script&gt; 
		* // Create RadioButton
		* $("#radiobtn").ejRadioButton();
		* var chkObj = $("#radiobtn").data("ejRadioButton");
		* chkObj.enable();
		* &lt;/script&gt;
		* @example 
		*&lt;input type="radio" name="radiobutn" id="radiobtn"/&gt;
		*&lt;label for="radiobtn"&gt;Experienced&lt;/label&gt;
	    *&lt;script&gt; 
		* // enable the RadioButton
		* $("#radiobtn").ejRadioButton();
		* $("#radiobtn").ejRadioButton("enable");	
		* &lt;/script&gt;
		* @memberof ejRadioButton
		* @instance
        */
        enable: function () {
            if (this.element.hasClass("e-disable"))
                this.element.removeClass("e-disable");
            this.div.attr("aria-disabled", false);
            this.model.enabled = true;
        }
    });
    /**
	 * Enum for various radio button sizes	 
	 * @enum {string}
	 * @global 
	 */
    ej.RadioButtonSize = {
        /**  Creates radio button with inbuilt small size height, width specified */
        Small: "small",
        /**  Creates radio button with inbuilt medium size height, width specified */
        Medium: "medium"
    };
})(jQuery, Syncfusion);;
/// <reference path="../../samples/web/autocomplete/angularsupport.html" />
/**
* @fileOverview Plugin to style the Html input elements
* @copyright Copyright Syncfusion Inc. 2001 - 2013. All rights reserved.
*  Use of this code is subject to the terms of our license.
*  A copy of the current license can be obtained at any time by e-mailing
*  licensing@syncfusion.com. Any infringement will be prosecuted under
*  applicable laws. 
* @version 12.1 
* @author <a href="mailto:licensing@syncfusion.com">Syncfusion Inc</a>
*/

(function ($, ej, undefined) {
    /**
    * @namespace ej
	* @classdesc Custom Design for Html Textbox control.
	* @class ejAutocomplete
	* @requires jQuery
    * @requires jquery.easing.1.3.js
	* @requires ej.core.js
    * @requires ej.data.js
	* @requires ej.autocomplete.js
	* @requires ej.scroller.js
	* @example 
	* &lt;input type="text" id="autocomplete" /&gt; <br> 
	* &lt;script&gt;
	* // Create AutoComplete
    * $('#autocomplete').ejAutocomplete({ dataSource: window.carList,value:"Austin-Healey" }); 	
	* &lt;/script&gt;
	*/

    // ejAutocomplete is the plugin name 
    // "ej.Autocomplete" is "namespace.className" will hold functions and properties

    ej.widget("ejAutocomplete", "ej.Autocomplete", {
        // widget element will be automatically set in this
        element: null,

        // user defined model will be automatically set in this
        model: null,
        validTags: ["input"],
        _setFirst: false,
        //Root Css Class
        _rootCSS: "e-autocomplete",
        type: "editor",

        // default model
        defaults: {
            /**		
			* Specifies the Data Source of the AutoComplete.	
			* @default null
			* @type {data}
			* @example 
			* &lt;input type="text" id="autocomplete" /&gt; <br> 
			* &lt;script&gt;
			* //To set dataSource API value during initialization  
			* 	$("#autocomplete").ejAutocomplete({ dataSource: window.carList,value:"Austin-Healey"});				
			* &lt;/script&gt;
			* @memberof ejAutocomplete
			* @instance
			*/
            dataSource: null,
            /**		
             * Specifies the query to retrieve the data from online server.	
             * @default null
             * @type {object}
             * @example
			 * &lt;input type="text" id="autocomplete" /&gt; <br> 
			 * &lt;script&gt;			 
             * //To set query API value during initialization  
             * var dataManger = ej.DataManager({       url: "http://mvc.syncfusion.com/Services/Northwnd.svc/"});
             * var queryString = ej.Query().from("Suppliers").select("ContactName");
             * 	$("#autocomplete").ejAutocomplete({ dataSource: dataManger, query: queryString, fields: { text: "ContactName" }});
            * &lt;/script&gt;
             * @memberof ejAutocomplete
             * @instance
             */
            query: null,
            /**		
             * Specifies mapping fields for the data items of the Autocomplete textbox.	
             * @default null
             * @type {object}
             * @example 
			 * &lt;input type="text" id="autocomplete" /&gt; <br> 
			 * &lt;script&gt;	
             * //To set fields API value during initialization         
             * 	$("#autocomplete").ejAutocomplete({ dataSource:window.countriesField,fields: { text: "name", key: "key" }});
             * &lt;/script&gt;
             * @memberof ejAutocomplete
             * @instance
             */
            fields: /** @lends ejAutocomplete# */{ 
			     /**		
                 * Defines the tag value or display text..
				 * @alias ejAutocomplete#fields->text
				 * @type String
                 */
				text: null, 
				  /**		
                 * Defines the key for the items to differentiate two items with same.
				 * @alias ejAutocomplete#fields->key
				 * @type String
                 */
				key: null,
                 /**		
                 * Used to categorize the items. It is used when the grouping is enabled..
				 * @alias ejAutocomplete#fields->category
				 * @type String
                 */				
				category: null, 
				 /**		
                 * Defines the html attributes such as id, class, styles for the item..
				 * @alias ejAutocomplete#fields->htmlAttributes
				 * @type Object
                 */
				htmlAttributes: null 
				},
            /**		
			* Specifies the template for Autocomplete.
			* @default null
			* @type {string}
			* @example 
			* &lt;input type="text" id="autocomplete" /&gt; <br> 
			* &lt;script&gt;
			* // To set template API value during initialization. 	
			* $("#autocomplete").ejAutocomplete({ dataSource: window.countries,template:"&lt;div class='flag ${sprite}'&gt; &lt;/div&gt;"+"&lt;div class='txt'&gt; ${text} &lt;/div&gt;"});
			* &lt;/script&gt;
			* @memberof ejAutocomplete
			* @instance
			*/
            template: null,
            /**		
			* Groups the search result based on the category value.
			* @default false
			* @type {boolean}
			* @example 
			* &lt;input type="text" id="autocomplete" /&gt; <br> 
			* &lt;script&gt;
			* // Initialize the Autocomplete with the grouping value specified.
			* $("#autocomplete").ejAutocomplete({ dataSource: window.vehicle,	allowGrouping: true});
			* &lt;/script&gt;
			* @memberof ejAutocomplete
			* @instance
			*/
            allowGrouping: false,
            /**		
            * Prevents the duplicate names presents in the search result.
            * @default false
            * @type {boolean}
            * @example 
			* &lt;input type="text" id="autocomplete" /&gt; <br> 
			* &lt;script&gt;
            * // Initialize the Autocomplete with the enableDistinct value specified.
            * $("#autocomplete").ejAutocomplete({dataSource: window.carList,enableDistinct: true });
            * &lt;/script&gt;
            * @memberof ejAutocomplete
            * @instance
            */
            enableDistinct: false,
            /**		
            * Sorts the lists value in ascending order if set to true.
            * @default true
            * @type {boolean}
            * @example 
			* &lt;input type="text" id="autocomplete" /&gt; <br> 
			* &lt;script&gt;
            * // Initialize the Autocomplete with the allowSorting value specified.
            * $("#autocomplete").ejAutocomplete({dataSource: window.carList,allowSorting: false });
            * &lt;/script&gt;
            * @memberof ejAutocomplete
            * @instance
            */
            allowSorting: true,
            /**		
            * Sort order specifies whether the suggestion list values has to display in ascending or descending order. 	See {@link SortOrder}
            * @default ej.SortOrder.Ascending
            * @type {enum}
            * @example 
			* &lt;input type="text" id="autocomplete" /&gt; <br> 
			 * &lt;script&gt;
            * //Initialize the Autocomplete with the sortOrder value specified
            * 	$("#autocomplete").ejAutocomplete({dataSource: window.carList,sortOrder:"descending" });					
            * &lt;/script&gt;
            * @memberof ejAutocomplete
            * @instance
            */
            sortOrder: "ascending",
            /**		
            * Allows to select multiple values from the suggestion list. Multiple values can be selected through either of the following options,Delimiter-Multiple values separated using comma.Visual mode- Each values are displayed in separate box with close button. 	See {@link MultiSelectMode}
            * @default ej.MultiSelectMode.None
            * @type {enum}
            * @example 
			* &lt;input type="text" id="autocomplete" /&gt; <br> 
			 * &lt;script&gt;
            * //Initialize the Autocomplete with the multiSelectMode value specified
            * 	$("#autocomplete").ejAutocomplete({dataSource: window.carList,multiSelectMode: ej.MultiSelectMode.Delimiter  });					 
            * &lt;/script&gt;
            * @memberof ejAutocomplete
            * @instance
            */
            multiSelectMode: "none",
            /**		
            * Sets the separator to allow multiple word searches. While typing the texts in the text box, if we enter the delimiter value, the texts after the delimiter are considered as a separate word or query. The delimiter string should have a single character and must be a symbol. Mostly the delimiter symbol is used as (comma ,) or (semi-colon ;) or any other special character.
            * @default ';'
            * @type {string}
            * @example 
			* &lt;input type="text" id="autocomplete" /&gt; <br> 
			 * &lt;script&gt;
            * //Initialize the Autocomplete with the delimiterChar value specified
            * 	$("#autocomplete").ejAutocomplete({dataSource: window.carList,multiSelectMode: ej.MultiSelectMode.Delimiter,delimiterChar: ';' });
            * &lt;/script&gt;
            * @memberof ejAutocomplete
            * @instance
            */
            delimiterChar: ',',
            /**		
            * Specifies new values can be added to the autocomplete input other than the values in the suggestion list.
            * @default false
            * @type {boolean}
            * @example 
			* &lt;input type="text" id="autocomplete" /&gt; <br> 
			 * &lt;script&gt;
            * //Initialize the Autocomplete with the allowAddNew value specified.
            * $("#autocomplete").ejAutocomplete({dataSource: window.carList,allowAddNew: true });
            * &lt;/script&gt;
            * @memberof ejAutocomplete
            * @instance
            */
            allowAddNew: false,
            /**		
            * Allows new text to be added to in the dropdown list when there are no suggestions. This property can only be used in the Visual mode only.
            * @default "Add New"
            * @type {string}
            * @example 
			* &lt;input type="text" id="autocomplete" /&gt; <br> 
			 * &lt;script&gt;
            * //Initialize the Autocomplete with the addNewText value specified
            * 	$("#autocomplete").ejAutocomplete({dataSource: window.carList,allowAddNew: true,addNewText: "Add New Car" });
            * &lt;/script&gt;
            * @memberof ejAutocomplete
            * @instance
            */
            addNewText: "Add New",
            /**		
            * Autocomplete textbox to be displayed with rounded corner style.
            * @default true
            * @type {boolean}
            * @example 
			* &lt;input type="text" id="autocomplete" /&gt; <br> 
			 * &lt;script&gt;
            * // Initialize the Autocomplete with the showRoundedCorner value specified.
            * $("#autocomplete").ejAutocomplete({dataSource: window.carList,showRoundedCorner: true });
            * &lt;/script&gt;
            * @memberof ejAutocomplete
            * @instance
            */
            showRoundedCorner: false,
           /**		
           * Indicates that the autocomplete textbox values can only be read.
           * @default true
           * @type {boolean}
           * @example 
		   * &lt;input type="text" id="autocomplete" /&gt; <br> 
			 * &lt;script&gt;
           * // Initialize the Autocomplete with the readOnly value specified.
           * $("#autocomplete").ejAutocomplete({dataSource: window.carList,readOnly: true });
           * &lt;/script&gt;
           * @memberof ejAutocomplete
           * @instance
           */
            readOnly: false,
            /**		
            * Sets the root class for Autocomplete theme. This cssClass API helps to use custom skinning option for Autocomplete control. By defining the root class using this API, we need to include this root class in CSS.
            * @default "gradient-lime"
            * @type {string}
            * @example 
			* &lt;input type="text" id="autocomplete" /&gt; <br> 
			 * &lt;script&gt;
            * //Initialize the Autocomplete with the cssClass value specified
            * 	$("#autocomplete").ejAutocomplete({ dataSource: window.carList,cssClass: 'gradient-lime'});
            * &lt;/script&gt;
            * @memberof ejAutocomplete
            * @instance
            */
            cssClass: "",
            /**		
            * Sets the watermarkText text. When the textbox is empty the watermarkText text is visible like a shaded text. 
            * @default "Enter text"
            * @type {string}
            * @example 
			* &lt;input type="text" id="autocomplete" /&gt; <br> 
			 * &lt;script&gt;
            * //Initialize the Autocomplete with the watermarkText value specified
            * 	$("#autocomplete").ejAutocomplete({ dataSource: window.carList,watermarkText: 'Enter the car name' });
            * &lt;/script&gt;
            * @memberof ejAutocomplete
            * @instance
            */
            watermarkText: null,
            /**		
            * Defines the default value to be display in the autocomplete textbox.
            * @default Null
            * @type {string}
            * @example 
			* &lt;input type="text" id="autocomplete" /&gt; <br> 
			 * &lt;script&gt;
            * //Initialize the Autocomplete value property with the  value specified
            * 	$("#autocomplete").ejAutocomplete({ dataSource: window.carList,value:"Elantra" });
            * &lt;/script&gt;
            * @memberof ejAutocomplete
            * @instance
            */
            value: "",
            /**		
            * Sets the search filter type. There are several filter types are available such as ‘startswith’, ‘contains’, ‘endswith’, ‘lessthan’, ‘lessthanorequal’, ‘greaterthan’, ‘greaterthanorequal’, ‘equal’, ‘notequal’. 	See {@link filterType}
            * @default ej.filterType.StartsWith
            * @type {enum}
            * @example 
			* &lt;input type="text" id="autocomplete" /&gt; <br> 
			 * &lt;script&gt;
            * //Initialize the Autocomplete with the filterType value specified
            * 	$("#autocomplete").ejAutocomplete({dataSource: window.carList,filterType: 'contains'  });					 
            * &lt;/script&gt;
            * @memberof ejAutocomplete
            * @instance
            */
            filterType: "startswith",
            /**		
            * Sets the case sensitivity of the search operation..
            * @default false
            * @type {boolean}
            * @example 
			* &lt;input type="text" id="autocomplete" /&gt; <br> 
			 * &lt;script&gt;
            * // Initialize the Autocomplete with the caseSensitiveSearch value specified.
            * $("#autocomplete").ejAutocomplete({ dataSource: window.carList,caseSensitiveSearch: true });
            * &lt;/script&gt;
            * @memberof ejAutocomplete
            * @instance
            */
            caseSensitiveSearch: false,
            /**		
            * Enables the loading icon to intimate the searching operation. The loading icon is visible when there is a time delay to perform the search.
            * @default true
            * @type {boolean}
            * @example 
			* &lt;input type="text" id="autocomplete" /&gt; <br> 
			 * &lt;script&gt;
            * // Initialize the Autocomplete with the showLoadingIcon value specified.
            * $("#autocomplete").ejAutocomplete({ dataSource: window.carList,showLoadingIcon: false });
            * &lt;/script&gt;
            * @memberof ejAutocomplete
            * @instance
            */
            showLoadingIcon: true,
            itemsCount: 0,
            minCharacter: 1,
            /**		
          * The delaySuggestionTimeout used to set the milliseconds time between a keypress and when the widget displays the suggestion popup.
          * @default 200
          * @type {number}
          * @example 
          * &lt;input type="text" id="autocomplete" /&gt; <br> 
            * &lt;script&gt;
          * // Initialize the Autocomplete with delaySuggestionTimeout in milliseconds value specified.
          * $("#autocomplete").ejAutocomplete({ dataSource: window.carList,delaySuggestionTimeout : 500 });
          * &lt;/script&gt;
          * @memberof ejAutocomplete
          * @instance
          */
            delaySuggestionTimeout:200,
            /**		
           * Enables the showPopup button. When the Showpopup button clicks, it displays the full list from the dataSource.
           * @default false
           * @type {boolean}
           * @example 
		   * &lt;input type="text" id="autocomplete" /&gt; <br> 
			 * &lt;script&gt;
           * // Initialize the Autocomplete with the showPopupButton  value specified.
           * $("#autocomplete").ejAutocomplete({ dataSource: window.carList,showPopupButton : true });
           * &lt;/script&gt;
           * @memberof ejAutocomplete
           * @instance
           */
            showPopupButton: false,
            /**		
            * Enables the highlight search option. When the highlightSearch option set to true, the corresponding string entered in the textbox is highlighted in the suggestion list.
            * @default false
            * @type {boolean}
            * @example 
			* &lt;input type="text" id="autocomplete" /&gt; <br> 
			 * &lt;script&gt;
            * // Initialize the Autocomplete with the highlightSearch  value specified.
            * $("#autocomplete").ejAutocomplete({dataSource: window.carList, highlightSearch : true });
            * &lt;/script&gt;
            * @memberof ejAutocomplete
            * @instance
            */
            highlightSearch: false,
            /**		
            * Automatically fills the first item from the suggestion list in an AutoComplete text box. The autoFill property is only applicable for “startswith” filterType type.
            * @default false
            * @type {boolean}
            * @example 
			* &lt;input type="text" id="autocomplete" /&gt; <br> 
			* &lt;script&gt;
            * // Initialize the Autocomplete with the enableAutoFill  value specified.
            * $("#autocomplete").ejAutocomplete({ dataSource: window.carList,enableAutoFill : true });
            * &lt;/script&gt;
            * @memberof ejAutocomplete
            * @instance
            */
            enableAutoFill: false,
            /**		
            * Sets the Autocomplete textbox direction as right to left alignment.
            * @default false
            * @type {boolean}
            * @example 
			* &lt;input type="text" id="autocomplete" /&gt; <br> 
			 * &lt;script&gt;
            * // Initialize the Autocomplete with the enableRTL    value specified.
            * $("#autocomplete").ejAutocomplete({ dataSource: window.carList,enableRTL   : true });
            * &lt;/script&gt;
            * @memberof ejAutocomplete
            * @instance
            */
            enableRTL  : false,
            /**		
            * When this property sets to false, it disables the Autocomplete control.
            * @default true
            * @type {boolean}
            * @example 
			* &lt;input type="text" id="autocomplete" /&gt; <br> 
			 * &lt;script&gt;
            * // Initialize the Autocomplete with the enabled  value specified.
            * $("#autocomplete").ejAutocomplete({ dataSource: window.carList,enabled : false });
           * &lt;/script&gt;
            * @memberof ejAutocomplete
            * @instance
            */
            enabled: true,
            /**		
            * Defines the height of the Autocomplete textbox.
            * @default Null
            * @type {string | number}
            * @example 
			* &lt;input type="text" id="autocomplete" /&gt; <br> 
			 * &lt;script&gt;
            * //Initialize the Autocomplete height property with the  value specified
            * 	$("#autocomplete").ejAutocomplete({dataSource: window.carList, height: 30 });
           * &lt;/script&gt;
            * @memberof ejAutocomplete
            * @instance
            */
            height: "",
            /**		
            * Defines the width of the Autocomplete textbox.
            * @default Null
            * @type {string | number}
            * @example 
			* &lt;input type="text" id="autocomplete" /&gt; <br> 
			 * &lt;script&gt;
            * //Initialize the Autocomplete width property with the width value specified
            * 	$("#autocomplete").ejAutocomplete({ dataSource: window.carList,width: 200 });
           * &lt;/script&gt;
            * @memberof ejAutocomplete
            * @instance
            */
            width: "",
            /**		
            * Sets the emptyResultText message text. When there is no suggestions are available at the time this 	message will be shown.
            * @default "No suggestions"
            * @type {string | number}
            * @example 
			* &lt;input type="text" id="autocomplete" /&gt; <br> 
			 * &lt;script&gt;
            * //Initialize the Autocomplete emptyResultText property with the  value specified
            * 	$("#autocomplete").ejAutocomplete({ dataSource: window.carList,emptyResultText: 'No Results Found' });
            * &lt;/script&gt;
            * @memberof ejAutocomplete
            * @instance
            */
            emptyResultText: "No suggestions",
            /**		
            * Sets whether the noResults message will be shown or not.
            * @default true
            * @type {boolean}
            * @example 
			* &lt;input type="text" id="autocomplete" /&gt; <br> 
			 * &lt;script&gt;
            * // Initialize the Autocomplete with the showEmptyResultText value specified.
            * $("#autocomplete").ejAutocomplete({ dataSource: window.carList, showEmptyResultText : false });
            * &lt;/script&gt;
            * @memberof ejAutocomplete
            * @instance
            */
            showEmptyResultText : true,
            /**		
            * Save current model value to browser cookies for state maintains. While refresh the Autocomplete control page retains the model value apply from browser cookies.  
            * @default false
            * @type {boolean}
            * @example 
			* &lt;input type="text" id="autocomplete" /&gt; <br> 
			* &lt;script&gt;
            * // Initialize the Autocomplete with the enablePersistence   value specified.
            * $("#autocomplete").ejAutocomplete({ dataSource: window.carList,enablePersistence  : true });
            * &lt;/script&gt;
            * @memberof ejAutocomplete
            * @instance
            */
            enablePersistence : false,
            /**		
            * Defines the popupHeight of the suggestion box.
            * @default "152px"
            * @type {string | number}
            * @example 
			* &lt;input type="text" id="autocomplete" /&gt; <br> 
			* &lt;script&gt;
            * //Initialize the Autocomplete popupHeight property with the  value specified
            * 	$("#autocomplete").ejAutocomplete({ dataSource: window.carList,popupHeight: '152px' });
            * &lt;/script&gt;
            * @memberof ejAutocomplete
            * @instance
            */
            popupHeight: "152px",
            /**		
            * Defines the popupWidth of the suggestion box.
            * @default "auto"
            * @type {string | number}
            * @example 
			* &lt;input type="text" id="autocomplete" /&gt; <br> 
			* &lt;script&gt;
            * //Initialize the Autocomplete popupWidth property with the  value specified
            * 	$("#autocomplete").ejAutocomplete({ dataSource: window.carList,popupWidth: '152px' });
            * &lt;/script&gt;
            * @memberof ejAutocomplete
            * @instance
            */
            popupWidth: "auto",
            /**     
			* Fires when focusIn successfully.
			* @event
			* @name ejAutocomplete#focusIn 	
			* @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
			* @param {object}  argument.model returns the autocomplete model
			* @param {string}  argument.type returns the name of the event
			* @param {string}  argument.value returns the value 
			* @example 
			* &lt;input type="text" id="autocomplete" /&gt; <br> 
			* &lt;script&gt;
			* $("#autocomplete").ejAutocomplete({ dataSource: window.carList});
			* //focusIn event for Autocomplete
            * $("#autocomplete").ejAutocomplete({ 
            *   	focusIn: function(args) {}
            * });      
			* &lt;/script&gt;			
			* @memberof ejAutocomplete
			* @instance
			*/
            focusIn: null,
            /**     
            * Fires when focusOut successfully.
            * @event
            * @name ejAutocomplete#focusOut 	
            * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
            * @param {object}  argument.model returns the autocomplete model
            * @param {string}  argument.type returns the name of the event
			* @param {string}  argument.value returns the value 
            * @example 
			* &lt;input type="text" id="autocomplete" /&gt; <br> 
			* &lt;script&gt;
			* $("#autocomplete").ejAutocomplete({ dataSource: window.carList});
            * //focusOut event for Autocomplete
            * $("#autocomplete").ejAutocomplete({ 
            *   	focusOut: function(args) {}
            * });  
            * &lt;/script&gt;						
            * @memberof ejAutocomplete
            * @instance
            */
            focusOut: null,
            /**     
            * Fires when change successfully.
            * @event
            * @name ejAutocomplete#change 	
            * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
            * @param {object}  argument.model returns the autocomplete model
            * @param {string}  argument.type returns the name of the event
			* @param {string}  argument.value returns the selected value
            * @example 
			* &lt;input type="text" id="autocomplete" /&gt; <br> 
			* &lt;script&gt;
			* $("#autocomplete").ejAutocomplete({ dataSource: window.carList});
            * //change event for Autocomplete
            * $("#autocomplete").ejAutocomplete({ 
            *   	change: function(args) {}
            * });  
            * &lt;/script&gt;			
            * @memberof ejAutocomplete
            * @instance
            */
            change: null,
            /**     
            * Fires when select successfully.
            * @event
            * @name ejAutocomplete#select 	
            * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
            * @param {object}  argument.model returns the autocomplete model
            * @param {string}  argument.type returns the name of the event
			* @param {string}  argument.text returns the name of the event
			* @param {string}  argument.value returns the selected value
			* @param {string}  argument.type returns the selected text
			* @param {object}  argument.key returns the selected value key
            * @example 
			* &lt;input type="text" id="autocomplete" /&gt; <br> 
			* &lt;script&gt;
			* $("#autocomplete").ejAutocomplete({ dataSource: window.carList});
            * //select event for Autocomplete
            * $("#autocomplete").ejAutocomplete({ 
            *   	select: function(args) {}
            * }); 
            * &lt;/script&gt;			
            * @memberof ejAutocomplete
            * @instance
            */
            select: null,
			  /**     
          * Fires after Autocomplete control is created.
          * @event
          * @name ejAutocomplete#create 	
          * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
          * @param {object}  argument.model returns the Autocomplete model
          * @param {string}  argument.type returns the name of the event		  
          * @example 
		  * &lt;input type="text" id="autocomplete" /&gt; <br> 
			* &lt;script&gt;
			* $("#autocomplete").ejAutocomplete({ dataSource: window.carList});
            * //create event for Autocomplete
            * $("#autocomplete").ejAutocomplete({ 
            *   	create: function(args) {}
            * }); 
            * &lt;/script&gt;			 
          * @memberof ejAutocomplete
          * @instance
          */
            create: null,
		  /**     
          * Fires when the Autocomplete is destroyed successfully
          * @event
          * @name ejAutocomplete#destroy 	
          * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
          * @param {object}  argument.model returns the Autocomplete model
          * @param {string}  argument.type returns the name of the event
          * @example 
		  * &lt;input type="text" id="autocomplete" /&gt; <br> 
			* &lt;script&gt;
			* $("#autocomplete").ejAutocomplete({ dataSource: window.carList});
            * //destroy event for Autocomplete
            * $("#autocomplete").ejAutocomplete({ 
            *   	destroy: function(args) {}
            * }); 
            * &lt;/script&gt;			 
          * @memberof ejAutocomplete
          * @instance
          */
            destroy: null
        },
        /**
         * Specify the data types for default properties 
		 * @private
         */
        dataTypes: {
            cssClass: "string",
            filterType: "enum",
            caseSensitiveSearch: "boolean",
            showLoadingIcon: "boolean",
            template: "string",
            allowGrouping: "boolean",
            enableDistinct: "boolean",
            allowSorting: "boolean",
            sortOrder: "enum",
            allowAddNew: "boolean",
            addNewText: "string",
            showRoundedCorner: "boolean",
            readOnly: "boolean",
            itemsCount: "number",
            minCharacter: "number",
            showPopupButton: "boolean",
            highlightSearch: "boolean",
            enableAutoFill: "boolean",
            enableRTL  : "boolean",
            multiSelectMode: "enum",
            delimiterChar: "string",
            emptyResultText: "string",
            showEmptyResultText : "boolean",
            enabled: "boolean",
            enablePersistence : "boolean",
            dataSource: "data",
            query: "data",
            fields: "data"
        },
        observables: ["value"],
        /**
       * To enable the autocomplete  
       * @return jQuery
       * @example 
       * &lt;input type="text" id="autocomplete" /&gt; <br> 
       * &lt;script&gt;
	   * $('#autocomplete').ejAutocomplete({dataSource: window.carList,value:"Aston Martin"}); 	
       * // Create autocomplete
       * var autocompleteObj  = $("#autocomplete").data("ejAutocomplete");
       * autocompleteObj.enable(); // enable the autocomplete
       * &lt;/script&gt;
       * @example 
       * &lt;input type="text" id="autocomplete" /&gt; <br> 
	   * &lt;script&gt;
	   * $('#autocomplete').ejAutocomplete({dataSource: window.carList,value:"Aston Martin"}); 	
	   * // Create autocomplete
       * $('#autocomplete').ejAutocomplete("enable"); 	
	   * &lt;/script&gt;
       *@memberof ejAutocomplete
       * @instance
       */
        enable: function () {
            if (!this.model.enabled) {
                this.model.enabled = true;
                this.target.disabled = false;
                this.element.removeClass("e-disable").attr({ "aria-disabled": false });
                if (this.model.showPopupButton) this.dropdownbutton.removeClass("e-disable").attr({ "aria-disabled": false });
                if (this.model.multiSelectMode == "visualmode") this._ulBox.removeClass("e-disable").attr({ "aria-disabled": false });
            }
        },
        /**
      * To disable the autocomplete  
      * @return jQuery
      * @example 
      * &lt;input type="text" id="autocomplete" /&gt; <br> 
      * &lt;script&gt;
	  * $('#autocomplete').ejAutocomplete({dataSource: window.carList,value:"Aston Martin"}); 
      * // Create autocomplete
      * var autocompleteObj  = $("#autocomplete").data("ejAutocomplete");
      * autocompleteObj.disable(); // disable the autocomplete
      * &lt;/script&gt;
      * @example 
      * &lt;input type="text" id="autocomplete" /&gt; <br> 
      * &lt;script&gt;
	  * $('#autocomplete').ejAutocomplete({dataSource: window.carList,value:"Aston Martin"}); 
      * // Create autocomplete
      * $('#autocomplete').ejAutocomplete("disable"); 	
      * &lt;/script&gt;
      *@memberof ejAutocomplete
      * @instance
      */
        disable: function () {
            if (this.model.enabled) {
                this._hideResult();
                this.element.attr("aria-expanded", false);
                this.model.enabled = false;
                this.target.disabled = true;
                this.element.addClass("e-disable").attr({ "aria-disabled": true });
                if (this.model.showPopupButton) this.dropdownbutton.addClass("e-disable").attr({ "aria-disabled": true });
                if (this.model.multiSelectMode == "visualmode") this._ulBox.addClass("e-disable").attr({ "aria-disabled": true });
            }
        },
        /**
      * Clears the text in the Autocomplete textbox.
      * @return jQuery
      * @example 
      * &lt;input type="text" id="autocomplete" /&gt; <br> 
      * &lt;script&gt;
	  * $('#autocomplete').ejAutocomplete({dataSource: window.carList,value:"Aston Martin"}); 
      * // Create autocomplete
      * var autocompleteObj  = $("#autocomplete").data("ejAutocomplete");
      * autocompleteObj.clearText(); // clear the autocomplete text
      * &lt;/script&gt;
      * @example 
      * &lt;input type="text" id="autocomplete" /&gt; <br> 
      * &lt;script&gt;
	  * $('#autocomplete').ejAutocomplete({dataSource: window.carList,value:"Aston Martin"}); 
      * // Create autocomplete
      * $('#autocomplete').ejAutocomplete("clearText"); 	
      * &lt;/script&gt;
      *@memberof ejAutocomplete
      * @instance
      */
        clearText: function () {
            if (this.model.enabled) {
                if (this.model.multiSelectMode == "visualmode")
                    this._deleteBox(this._ulBox.children("li"));
                this.element.val("");
                this._valueChange();
                if (this._isFocused) this.element.blur();
                else this._targetBlur();
            }
        },
        /**
      * Returns the current value selected in the Autocomplete textbox.
      * @return jQuery
      * @example 
      * &lt;input type="text" id="autocomplete" /&gt; <br> 
      * &lt;script&gt;
	  * $('#autocomplete').ejAutocomplete({dataSource: window.carList,value:"Aston Martin"}); 
      * // Create autocomplete
      * var autocompleteObj  = $("#autocomplete").data("ejAutocomplete");
      * autocompleteObj.getValue(); // getValue of the autocomplete text
      * &lt;/script&gt;
      * @example 
      * &lt;input type="text" id="autocomplete" /&gt; <br> 
      * &lt;script&gt;
	  * $('#autocomplete').ejAutocomplete({dataSource: window.carList,value:"Aston Martin"}); 
      * // Create autocomplete
      * $('#autocomplete').ejAutocomplete("getValue"); 	
      * &lt;/script&gt;
      *@memberof ejAutocomplete
      * @instance
      */
        getValue: function () {
            if (this.model.multiSelectMode == "visualmode")
                return this._hiddenInput.val();
            if (this.element.hasClass("e-watermark"))
                return "";
            return this.target.value;
        },
        /**
     * Returns the values selected in the Autocomplete textbox.
     * @return jQuery
     * @example 
     * &lt;input type="text" id="autocomplete" /&gt; <br> 
     * &lt;script&gt;
	 * $('#autocomplete').ejAutocomplete({dataSource: window.carList,value:"Aston Martin"}); 
     * // Create autocomplete
     * var autocompleteObj  = $("#autocomplete").data("ejAutocomplete");
     * autocompleteObj.getSelectedItems(); // getSelectedItems the autocomplete text
     * &lt;/script&gt;
     * @example 
     * &lt;input type="text" id="autocomplete" /&gt; <br> 
     * &lt;script&gt;
	 * $('#autocomplete').ejAutocomplete({dataSource: window.carList,value:"Aston Martin"}); 
     * // Create autocomplete
     * $('#autocomplete').ejAutocomplete("getSelectedItems"); 	
     * &lt;/script&gt;
     *@memberof ejAutocomplete
     * @instance
     */
        getSelectedItems: function () {
            return this._selectedItems;
        },
        _setValue: function (value) {
            if (this.element.hasClass("e-watermark"))
                this.element.removeClass("e-watermark")
            if (typeof value === "object" || (typeof value === "number" && isNaN(value))) value = "";
            value = $.trim(value);
            this.element.val("");
            if (value) {
                if (this.model.multiSelectMode == "visualmode") {
                    this._selectedItems = [];
                    this._deleteBox(this._ulBox.children("li"));
                    this._hiddenInput.val(value);
                    var values = value.split(this.model.delimiterChar);

                    if (ej.DataManager && this.model.dataSource instanceof ej.DataManager) {
                        this._createBoxForObjectType(values);
                    }
                    else {
                        this.suggestionListItems = this.model.dataSource;
                        if (typeof this.suggestionListItems[0] != "object") {
                            for (var i = 0; i < values.length; i++) {
                                if (values[i]) {
                                    this._ulBox.append(this._createBox(values[i]));
                                    this._selectedItems.push(values[i]);
                                }
                            }
                        }
                        else this._createBoxForObjectType(values);
                    }
                }
                else {
                    this.element.val(value);
                    this._hiddenInput.val(value);
                    this._updateSelectedItemArray();
                }
            }
            else
                this._hiddenInput.val("");
            this.model.value = this.model.multiSelectMode == "visualmode" ? this._hiddenInput.val() : this.element.val();
            this._preVal = this.element.val();
            !this._isWatermark && this._setWatermarkTxt();
            return value;
        },
        _createBoxForObjectType: function (values) {
            var dataQuery, promise, mapper, strData, proxy = this;
            mapper = this.model.fields, mapFld = { _key: null, _text: null, _attr: null };
            mapFld._text = (mapper && mapper.text) ? mapper["text"] : "text";
            for (var data = 0; data < values.length; data++) {
                var _val = $.trim(values[data]);
                if (ej.DataManager && this.model.dataSource instanceof ej.DataManager) {
                    dataQuery = this._getQuery().where(mapFld._text, "equal", _val, false);
                    promise = (this.model.dataSource).executeQuery(dataQuery);
                    promise.done(function (e) {
                        var res = e.result;
                        strData = res instanceof Array && res.length ? res[0] : _val;
                        proxy._selectedItems.push(strData);
                    }).fail(function (e) {
                        proxy._selectedItems.push(_val);
                    });
                }
                else {
                    dataQuery = ej.Query().where(mapFld._text, "equal", _val, false);
                    promise = ej.DataManager(this.suggestionListItems).executeLocal(dataQuery);
                    strData = promise instanceof Array && promise.length ? promise[0] : _val;
                    this._selectedItems.push(strData);
                }
                this._ulBox.append(this._createBox(_val));
            }
        },

        /**
  * Set the values to the Autocomplete textbox by input key value.
  * @return jQuery
  * @example 
  * &lt;input type="text" id="autocomplete" /&gt; <br> 
  * &lt;script&gt;
   * $('#autocomplete').ejAutocomplete({dataSource: window.vehicle});   
  * // Create autocomplete
  * var autocompleteObj  = $("#autocomplete").data("ejAutocomplete");
  * autocompleteObj.selectValueByKey("F"); // set key value corresponding text to the autocomplete textbox
  * &lt;/script&gt;
  * @example 
  * &lt;input type="text" id="autocomplete" /&gt; <br> 
  * &lt;script&gt;
  * // Create autocomplete
  * $('#autocomplete').ejAutocomplete({dataSource: window.vehicle});
  * $('#autocomplete').ejAutocomplete("selectValueByKey","F");  
  * &lt;/script&gt;
  *@memberof ejAutocomplete
  * @instance
  */
        selectValueByKey: function (key) {
            this._setOperation(key, "key");
        },
        /**
    * Set the values to the Autocomplete textbox by input text value.
    * @return jQuery
    * @example 
    * &lt;input type="text" id="autocomplete" /&gt; <br> 
    * &lt;script&gt;
	 * $('#autocomplete').ejAutocomplete({dataSource: window.vehicle}); 	
    * // Create autocomplete
    * var autocompleteObj  = $("#autocomplete").data("ejAutocomplete");
    * autocompleteObj.selectValueByText("BMW 7"); // set text value to the autocomplete textbox
    * &lt;/script&gt;
    * @example 
    * &lt;input type="text" id="autocomplete" /&gt; <br> 
    * &lt;script&gt;
    * // Create autocomplete
    * $('#autocomplete').ejAutocomplete({dataSource: window.vehicle}); 
    * $('#autocomplete').ejAutocomplete("selectValueByText","BMW 7"); 	
    * &lt;/script&gt;
    *@memberof ejAutocomplete
    * @instance
    */
        selectValueByText: function (text) {
            this._setOperation(text, "text");
        },

        //Based on source value and mapper value to generate query to find the required result set
        _setOperation: function (source, value) {
            if (!this.model.enabled) return false;
            var bindTo = "", promise, dataQuery, proxy = this, list = this.model.dataSource;
            if (ej.isNullOrUndefined(list)) return false;
            if (typeof list[0] == "object" || list instanceof ej.DataManager) {
                var mapper = this.model.fields;
                bindTo = (mapper && mapper[value]) ? mapper[value] : value;
            } else if (value == "key")
                return false;
            if (ej.DataManager && list instanceof ej.DataManager) {
                dataQuery = this._getQuery().where(bindTo, "equal", source, !this.model.caseSensitiveSearch);
                promise = (list).executeQuery(dataQuery);
                promise.done(function (e) {
                    proxy._setText(e.result[0]);
                });
            }
            else {
                if ((!list || !list.length || list.length < 1)) return false;
                dataQuery = ej.Query().where(bindTo, "equal", source, !this.model.caseSensitiveSearch);
                promise = ej.DataManager(list).executeLocal(dataQuery);
                this._setText(promise[0]);
            }
        },
        //Obtained data set check with its type to set the value to textbox 
        _setText: function (data) {
            if (!data) return false;
            var currentValue, mapper = this.model.fields;
            if ((typeof this.model.dataSource[0] == "object") || (typeof data == "object"))
                currentValue = (mapper && mapper["text"]) ? data[mapper["text"]] : data["text"];
            else currentValue = data;
            if (currentValue) {
                if (this.model.multiSelectMode == "visualmode" && this._removeDuplicates(data)) return false;
                this._valueToTextBox(currentValue, data, true);
                this.model.value = this.model.multiSelectMode == "visualmode" ? this._hiddenInput.val() : this.element.val();
            }
        },
        /*To set value to textbox in visualmode or delimiter mode 
        currentValue- specify the value to be displayed
        date- specify the current value object
        flag-set true ,when value set by using public methods,false when select value from popuplist
        */
        _valueToTextBox: function (currentValue, data, flag) {
            var delimiterIndex;
            if (this.model.multiSelectMode == "visualmode") {
                if (this._addNewTemplate) currentValue = currentValue.substr(0, currentValue.length - this._addNewTemplate.length);
                delimiterIndex = this._hiddenInput.val() ? this.model.delimiterChar : "";
                this._hiddenInput.val(this._hiddenInput.val() + delimiterIndex + currentValue);
                this.element.val("");
                this._ulBox.append(this._createBox(currentValue));
            }
            else if (this.model.multiSelectMode == "delimiter") {
                delimiterIndex = (this.target.value).lastIndexOf(this.model.delimiterChar)
                if (flag)
                    this.element.val(this.element.val() == "" ? (currentValue) : (this.element.val() + this.model.delimiterChar + currentValue));
                else
                    this.element.val(this._queryString.substr(0, delimiterIndex + 1) + currentValue + this.model.delimiterChar + " ");
            }
            else this.element.val(currentValue);
            this._selectedItems.push(data);
            this._moveCaretToEnd(this.element[0]);
        },
        // while we set the value using public methods,it prevents the duplicate entry in visual mode
        _removeDuplicates: function (currentValue) {
            if (this._selectedItems.length == 0) return false;
            if (this._selectedItems.indexOf(currentValue) != -1) return true;
        },
        /**
    * search values in the Autocomplete textbox.
    * @return jQuery
    * @example 
    * &lt;input type="text" id="autocomplete" /&gt; <br> 
    * &lt;script&gt;
	* $('#autocomplete').ejAutocomplete({dataSource: window.carList,value:"Aston Martin"});
    * // Create autocomplete
    * var autocompleteObj  = $("#autocomplete").data("ejAutocomplete");
    * autocompleteObj.search(); // search the autocomplete text
    * &lt;/script&gt;
    * @example 
    * &lt;input type="text" id="autocomplete" /&gt; <br> 
    * &lt;script&gt;
	* $('#autocomplete').ejAutocomplete({dataSource: window.carList,value:"Aston Martin"});
    * // Create autocomplete
    * $('#autocomplete').ejAutocomplete("search"); 	
    * &lt;/script&gt;
    *@memberof ejAutocomplete
    * @instance
    */
        search: function () {
            if (this.model.enabled && this._checkDelimiter()) {
                this._hideResult();
                this.element.attr("aria-expanded", false);
                this._autoFill = false;
                this._queryString = $.trim(this._queryString);
                if (this._queryString.length > 0) this._OnTextEnter();
            }
        },

        hide: function () {
            this._hideResult();
            this.element.attr("aria-expanded", false);
        },

        _changeWatermark: function (text) {
            if (this._isWatermark) this.element.attr("placeholder", text);
            else if (this.element.hasClass("e-watermark")) this.element.val(text);
        },
        _changeSkin: function (skin) {
            this.wrapper.removeClass(this.model.cssClass).addClass(skin);
            this.suggestionList.removeClass(this.model.cssClass).addClass(skin);
        },
        _setDropdown: function (boolean) {
            this.model.showPopupButton = boolean;
            if (boolean) this._renderDropdown();
            else this._destroyDropdown();
        },
        _changeHeight: function (height) {
            this.wrapper.height(height);
        },
        _changeWidth: function (width) {
            this.wrapper.width(width);
            this._setListWidth();
        },

        // constructor function
        _init: function () {
            if (!this.element.is("input") || (this.element.attr('type') && this.element.attr('type') != "text")) return false;
            this._initialize();
            this._render();
            this._wireEvents();
        },
        /**
        * To configure the properties at runtime using SetModel		
        * @private
        */
        _setModel: function (options) {
            if (!this.model.enabled && ej.isNullOrUndefined(options["enabled"])) return false;
            var option;
            for (option in options) {
                switch (option) {
                    case "dataSource": this.model.dataSource = null; break;
                    case "watermarkText": this._changeWatermark(options[option]); break;
                    case "delaySuggestionTimeout": this.model.delaySuggestionTimeout= parseInt(options[option]); break;
                    case "value":
                        options[option] = this._setValue(options[option]);
                        this._valueChange();
                        break;
                    case "showPopupButton": this._setDropdown(options[option]); break;
                    case "enableRTL": this._RightToLeft(options[option]); break;
                    case "showRoundedCorner": this._setRoundedCorner(options[option]); break;
                    case "readOnly": this.model.readOnly = options[option]; this._checkReadOnly(); break;
                    case "delimiterChar": options[option] = this._validateDelimiter(options[option]); break;
                    case "multiSelectMode":
                        this.model.multiSelectMode = options[option];
                        if (options[option] == "visualmode") this._renderBoxModel();
                        else if (this.element.hasClass("e-visual-mode")) this._destroyBoxModel();
                        this._setValue(this.model.value);
                        break;
                    case "enabled": this._disabled(!options[option]); break;
                    case "height": this._changeHeight(options[option]); break;
                    case "width": this._changeWidth(options[option]); break;
                    case "cssClass": this._changeSkin(options[option]); break;
                    case "itemsCount ": if (options[option] <= 0 || isNaN(options[option])) options[option] = 0; break;
                    case "popupHeight": this.suggestionList.css({ "max-height": options[option] }); break;
                    case "popupWidth": this.model.popupWidth = options[option]; this._setListWidth(); break;
                }
            }
        },
        /**
 * destroy in the Autocomplete textbox.
 * @alias destroy
 * @return jQuery
 * @example 
 * &lt;input type="text" id="autocomplete" /&gt; <br> 
 * &lt;script&gt;
 * $('#autocomplete').ejAutocomplete({dataSource: window.carList,value:"Aston Martin"});
 * // Create autocomplete
 * var autocompleteObj  = $("#autocomplete").data("ejAutocomplete");
 * autocompleteObj.destroy(); // destroy the autocomplete 
 * &lt;/script&gt;
 * @example 
 * &lt;input type="text" id="autocomplete" /&gt; <br> 
 * &lt;script&gt;
 * $('#autocomplete').ejAutocomplete({dataSource: window.carList,value:"Aston Martin"});
 * // Create autocomplete
 * $('#autocomplete').ejAutocomplete("destroy"); 	
 * &lt;/script&gt;
 *@memberof ejAutocomplete
 * @instance
 */
        _destroy: function () {
            this.element.insertAfter(this.wrapper);
            this.wrapper.remove();
            this.element.removeClass("e-input e-watermark").val("");
            if (this._isWatermark) this.element.removeAttr("placeholder");
            this.suggestionList.remove();
        },

        _initialize: function () {
            this.element.attr("role", "combobox").attr("aria-label", "Autocomplete textbox").attr("aria-expanded", false).attr("tabindex", 0).attr("aria-autocomplete", "list").attr("value", this.model.value);
            this.target = this.element[0];
            this.dropdownbutton = null;
            this.showSuggestionBox = false;
            this.noresult = true;
            this._queryString = null;
            this.suggLen = 0;
            this._selectedItems = [];

            this._activeItem = 0;
            this.ctrlKeyPressed = false;
            this._isFocused = false;
            this._isOpened = false;
            this._isWatermark = this._checkWatermarkSupport();
        },

        _render: function () {
            this._renderWrapper();
            this._setDimentions();
            this._renderDropdown();
            this._checkProperties();
            this._setWatermark();
            this._renderSuggestionList();
            this._RightToLeft(this.model.enableRTL);
            this._setRoundedCorner(this.model.showRoundedCorner);
        },

        _renderWrapper: function () {
            this.element.addClass("e-input").attr("autocomplete", "off");
            this.wrapper = ej.buildTag("span.e-atc e-widget " + this.model.cssClass + "#" + this.target.id + "_wrapper").insertAfter(this.element);
            this.container = ej.buildTag("span.e-in-wrap e-box").append(this.element);
            this.wrapper.append(this.container);
            this._hiddenInput = ej.buildTag("input#" + this.target.id + "_hidden", "", {}, { type: "hidden" }).insertBefore(this.element);
        },

        _renderDropdown: function () {
            if (this.model.showPopupButton) {
                var span = ej.buildTag("span.e-icon e-search");
                this.dropdownbutton = ej.buildTag("span.e-select#" + this.target.id + "_dropdown").append(span);
                this.container.append(this.dropdownbutton).addClass("e-padding");
                this.dropdownbutton.bind("mousedown", $.proxy(this._OnDropdownClick, this));
            }
        },

        _setDimentions: function () {
            if (this.model.height)
                this.wrapper.height(this.model.height);
            if (this.model.width)
                this.wrapper.width(this.model.width);
        },

        _renderBoxModel: function () {
            this._ulBox = ej.buildTag("ul.e-ul e-boxes");
            this.element.val("").removeAttr("name").width(1).addClass("e-visual-mode");
            this.container.prepend(this._hiddenInput, this._ulBox);
            this.wrapper.height("auto");

            this._on(this.container, "mousedown", function (e) {
                if (!this.model.enabled) return false;
                var $target = $(e.target);
                if (!$target.is(this.element)) {
                    e.preventDefault();
                    if (!this._isFocused) this.element.focus();
                    if ($target.hasClass("e-options")) {
                        if (!e.ctrlKey && $target.siblings().hasClass("e-active")) this._removeActive();
                        if ($target.hasClass("e-active")) $target.removeClass("e-active");
                        else $target.addClass("e-active");
                    }
                    else this._moveCaretToEnd(this.element[0]);
                }
                if (!e.ctrlKey && ($target.hasClass("e-boxes") || $target.hasClass("e-input"))) this._removeActive();
            });
        },
        _destroyBoxModel: function () {
            this.container.prepend(this.element);
            this.element.attr({ name: this._hiddenInput.attr("name") }).removeAttr("style").removeClass("e-visual-mode");
            this.wrapper.height(this.model.height);
            this._hiddenInput.remove();
            this._ulBox.remove();
            this._off(this.container, "mousedown");
        },
        _deleteLastBox: function () {
            var items = this._ulBox.children();
            var item = items.last(), flag = item.hasClass("e-active");
            this._removeActive();
            flag ? this._deleteBox(item) : item.addClass("e-active");
        },
        _deleteBox: function (items) {
            for (var i = 0; i < items.length; i++) {
                var boxes = this._ulBox.children();
                var index = boxes.index(items[i]);
                this._selectedItems.splice(index, 1);

                var deli = this.model.delimiterChar;
                var values = this._hiddenInput.val().split(deli);
                values.splice(index, 1);
                this._hiddenInput.val(values.join(deli));
                $(items[i]).remove();
            }
            if (this.showSuggestionBox) this._refreshPopup();
            this._valueChange();
        },
        _removeActive: function () {
            this._ulBox.children("li.e-active").removeClass("e-active");
        },
        _adjustWidth: function () {
            var tempSpan = ej.buildTag("span", this.element.val()), wid, minWidth;
            this.container.append(tempSpan);
            minWidth = 30;  //  some additional width for textbox in visualmode
            wid = tempSpan.width() + minWidth;
            if (this.element.width() != wid)
                this.element.width(wid);
            tempSpan.remove();
        },

        _checkProperties: function () {
            this._checkReadOnly();
            this.model.delimiterChar = this._validateDelimiter(this.model.delimiterChar);
            if (!this.model.enabled) {
                this.model.enabled = true;
                this._disabled(true);
            }
            if (this.model.multiSelectMode == "visualmode") this._renderBoxModel();
            this._checkNameAttr();
            this.model.value = this._setValue(this.model.value);
        },

        _checkNameAttr: function () {
            if (this.element.attr("name"))
                this._hiddenInput.attr("name", this.element.attr("name"));
            else {
                this._hiddenInput.attr("name", this.element[0].id);
				this.element.attr("name", this.element[0].id);
			}
        },

        _disabled: function (boolean) {
            if (boolean) this.disable();
            else this.enable();
        },

        _destroyDropdown: function () {
            this.dropdownbutton.unbind("mousedown", $.proxy(this._OnDropdownClick, this));
            this.dropdownbutton.remove();
            this.dropdownbutton = null;
            this.container.removeClass("e-padding");
        },

        _validateDelimiter: function (deli) {
            if (deli.length == 1) {
                var RegEx = /^[a-zA-Z0-9]+$/;
                if (!RegEx.test(deli)) return deli;
            }
            return ",";
        },

        _checkWatermarkSupport: function () {
            return 'placeholder' in document.createElement('input');
        },
        _setWatermark: function () {
            if (this._isWatermark) this.element.attr("placeholder", this.model.watermarkText);
        },
        _setWatermarkTxt: function () {
            if (this.model.watermarkText != null && $.trim(this.element.val()) == "") {
                this.element.addClass("e-watermark");
                this.element.val(this.model.watermarkText);
            }
        },

        _renderSuggestionList: function () {
			var oldWrapper = $("#" + this.element.context.id + "_suggestion").get(0);
            if (oldWrapper)
                $(oldWrapper).remove();
            this.suggestionList = ej.buildTag("div.e-atc-popup e-popup e-widget e-box " + this.model.cssClass + "#" + this.target.id + "_suggestion", "", { "display": "none" }).attr("role", "listbox");
            this.element.attr("aria-owns", this.target.id + "_suggestion");
            this.popup = this.suggestionList;
            var scrollerDiv = ej.buildTag("div");
            this.ul = ej.buildTag("ul.e-ul").attr("role", "listbox");
            scrollerDiv.append(this.ul);
            this.suggestionList.append(scrollerDiv);
            $('body').append(this.suggestionList);
            this._setListWidth();
            this._setListHeight();
            this.suggestionList.ejScroller({ height: this.suggestionList.height(), width: 0, scrollerSize: 20 });
            this.scrollerObj = this.suggestionList.ejScroller("instance");
        },

        _checkEmptyList: function () {
            if (this.model.multiSelectMode == "visualmode") this._removeRepeated();
            if (this.suggestionListItems.length == 0) {
                this.suggestionListItems.push(this.model.emptyResultText);
                this.noresult = true;
            }
            else this.noresult = false;
        },
        _showSuggestionList: function () {
            this._checkEmptyList();
            this._addNewTemplate = null;
            if (this.noresult && this.model.multiSelectMode == "visualmode" && this.model.allowAddNew && this.element.val() != "") {
                this.noresult = false;
                this.suggestionListItems.pop();
                this._addNewTemplate = "   (" + this.model.addNewText + ")";
                this.suggestionListItems.push(this.element.val() + this._addNewTemplate);
                this._checkEmptyList();
            }

            if (!this.noresult || this.model.showEmptyResultText )
                this._generateSuggestionList();
        },
        /**
* generate suggestion list using the given object
* @private
*/
        _generateSuggestionList: function () {
            var list = this.suggestionListItems, i, suggList = [];
            this.ul.empty();

            if (typeof list[0] != "object") {
			
                if (this.model.enableDistinct) list = ej.dataUtil.distinct(list, "", true);
                for (i = 0; i < list.length; i++) {
                    var _txt = (this.model.highlightSearch && !this.noresult) ? this._highlightSuggestion(list[i]) : list[i];
                    this.ul.append(ej.buildTag("li", _txt).attr("role","option"));
                }
                this._currList = list;
                this._mapper = { txt: null, key: null };
            }
            else {
                var mapper = this.model.fields, mapFld = { _key: null, _text: null, _attr: null };
                mapFld._key = (mapper && mapper.key) ? mapper["key"] : "key";
                mapFld._text = (mapper && mapper.text) ? mapper["text"] : "text";
                mapFld._attr = (mapper && mapper.htmlAttributes) ? mapper["htmlAttributes"] : "htmlAttributes";
                this._mapper = { txt: mapFld._text, key: mapFld._key }, this._currList = [];
                if (this.model.enableDistinct) list = ej.dataUtil.distinct(list, mapFld._text, true);

                if (this.model.allowGrouping) {
                    var mapCateg = (mapper && mapper.category) ? mapper["category"] : "category", groupedList, _query;
                    _query = ej.Query().group(mapCateg);
                    this._addSortingQuery(_query, "key");
                    groupedList = ej.DataManager(list).executeLocal(_query);
                    this._swapUnCategorized(groupedList);
                    for (i = 0; i < groupedList.length; i++) {
                        if (groupedList[i].key)
                            this.ul.append(ej.buildTag("li.e-category", groupedList[i].key).attr("role", "option"));
                        this._generateLi(groupedList[i].items, mapFld);
                    }
                }
                else this._generateLi(list, mapFld);
            }
            if (this._getLiTags().length > 0) this._showResult();
        },
        /**
* swaping uncategorized values in the sorting order
* @private
*/
        _swapUnCategorized: function (list) {
            $(list).each(function (i, obj) {
                if (!obj.key) {
                    for (var j = i; j > 0; j--) {
                        list[j] = list[j - 1];
                    }
                    list[j] = obj;
                    return false;
                }
            });
        },
        /**
* generate li tagd with the given data source
* @private
*/
        _generateLi: function (list, mapFld) {
            for (var j = 0; j < list.length; j++) {
                var _text = this._getField(list[j], mapFld._text);
                if (!ej.isNullOrUndefined(_text)) {
                    if (this.model.highlightSearch) _text = this._highlightSuggestion(_text);
                    if (this.model.template) _text = this._getTemplatedString(list[j], mapFld._text, _text);

                    var li = $(document.createElement("li")).append(_text);
                    this._setAttributes(this._getField(list[j], mapFld._attr), li);
                    this.ul.append(li);
                    this._currList = $.merge(this._currList, [list[j]]);
                }
            }
        },
        /**
* generate li tagd for the box model
* @private
*/
        _getLiTags: function () {
            return this.ul.children("li:not('.e-category')");
        },
        /**
* refresh popup using templated string
* @private
*/
        _getTemplatedString: function (list, searchLabl, searchTxt) {
            var str = this.model.template, start = str.indexOf("${"), end = str.indexOf("}");
            while (start != -1 && end != -1) {
                var content = str.substring(start, end + 1);
                var field = content.replace("${", "").replace("}", "");
                var replace = this._getField(list, field);
                // if highlightSearch is enabled, it replaces the highlighted search text
                if (searchLabl == field) replace = searchTxt;
                if (!replace) replace = "";
                str = str.replace(content, replace);
                start = str.indexOf("${"), end = str.indexOf("}");
            }
            return str;
        },
        /**
* get the field name for the given object
* @private
*/
        _getField: function (obj, fieldName) {
            return ej.pvt.getObject(fieldName, obj);
        },
        /**
* Set the key attribute for the input element
* @private
*/
        _setAttributes: function (data, element) {
            if (data) {
                for (var key in data)
                    element.attr(key, data[key]);
            }
        },
        /**
* Set width for the popuplist
* @private
*/
        _setListWidth: function () {
            var width = this.model.popupWidth;
            if (width && width != "auto") this.suggestionList.css({ "width": width });
            else this.suggestionList.css({ "width": this.wrapper.width() });
        },
        /**
* Set height for the popuplist
* @private
*/
        _setListHeight: function () {
            this.suggestionList.css({ "max-height": this.model.popupHeight });
        },
        /**
* Refresh the popup with the given width
* @private
*/
        _refreshPopup: function () {
            if (this.model.popupWidth == "auto" && this.wrapper.outerWidth() != this.suggestionList.outerWidth()) {
                this.suggestionList.css({ "width": this.wrapper.width() });
                this._refreshScroller();
            }
            this._setListPosition();
        },
        /**
* showing the popup with the result
* @private
*/
        _showResult: function () {
            this._refreshPopup();
            this._refreshScroller();
            if (this._isOpened) 
                $(document).bind("mousedown", $.proxy(this._OnDocumentClick, this));
            else {
                this.suggestionList.css("display", "none");
                var tis = this;
                clearTimeout(this._typing);
                this._typing = setTimeout(function () {
                    tis.suggestionList.slideDown(200, "easeOutQuad", function () {
                        $(document).bind("mousedown", $.proxy(tis._OnDocumentClick, tis));
                    });
                },this.model.delaySuggestionTimeout);
            }

            this._isOpened = true;
            this.showSuggestionBox = true;
            var _suggestionListItems = this._getLiTags();
            this._listSize = _suggestionListItems.size();

            _suggestionListItems.bind("mouseenter", $.proxy(this._OnMouseEnter, this));
            _suggestionListItems.bind("mouseleave", $.proxy(this._OnMouseLeave, this));
            _suggestionListItems.bind("click", $.proxy(this._OnMouseClick, this));
            $(window).bind("resize", $.proxy(this._OnWindowResize, this));
        },
        /**
* Refresh the scroller popup functions
* @private
*/
        _hideResult: function () {
            if (this.showSuggestionBox) {
                this.showSuggestionBox = false;
                this._activeItem = 0;
                if (this._isOpened) this.suggestionList.css("display", "none");
                else this.suggestionList.slideUp(100, "easeOutQuad");
                $(document).unbind("mousedown", $.proxy(this._OnDocumentClick, this));
                $(window).unbind("resize", $.proxy(this._OnWindowResize, this));
            }
        },
        /**
* Refresh the scroller popup functions
* @private
*/
        _refreshScroller: function () {
            this.suggestionList.css("height", "auto");
            this.suggestionList.find(".e-content, .e-vscroll").removeAttr("style");
            this.suggestionList.find(".e-vscroll div").removeAttr("style");
			this.suggestionList.find(".e-content, .e-vhandle").removeAttr("style");
            this.suggestionList.find(".e-vhandle div").removeAttr("style");

            this.suggestionList.css("display", "block");

             if (this.suggestionList.height() > (parseInt(this.model.popupHeight.replace("px", "")) - 4)) {
                this.scrollerObj.model.height = this.suggestionList.height();
                this.scrollerObj.refresh();
                this.scrollerObj.option("scrollTop", 0);
                this.suggestionList.addClass("e-scroller")
            }
            else
                this.suggestionList.removeClass("e-scroller")
            this.suggestionList.css("height", "auto");
        },
        /**
* set list position while opening the popup
* @private
*/
        _setListPosition: function () {
            var elementObj = this.wrapper;
            var pos = this._getOffset(elementObj),
            left = pos.left,
            totalHeight = elementObj.outerHeight(),
            border = (totalHeight - elementObj.height()) / 2,
            maxZ = this._getZindexPartial();
            if (this.model.enableRTL  ) left -= this.suggestionList.outerWidth() - elementObj.outerWidth();

            this.suggestionList.css({
                "left": left + "px",
                "top": pos.top + totalHeight - border + 3 + "px",
                "z-index": maxZ
            });
        },
        _getOffset: function (ele) {
            var pos = ele.offset();
            if ($("body").css("position") != "static") {
                var bodyPos = $("body").offset();
                pos.left -= bodyPos.left;
                pos.top -= bodyPos.top;
            }
            return pos;
        },

        /**
* Z-index calculation for body element
* @private
*/
        _getZindexPartial: function () {
            var parents = this.element.parents(), bodyEle;
            bodyEle = $('body').children(), index = bodyEle.index(this.popup);
            bodyEle.splice(index, 1);
            $(bodyEle).each(function (i, ele) { parents.push(ele); });

            var maxZ = Math.max.apply(maxZ, $.map(parents, function (e, n) {
                if ($(e).css('position') != 'static') return parseInt($(e).css('z-index')) || 1;
            }));
            if (!maxZ || maxZ < 10000) maxZ = 10000;
            else maxZ += 1;
            return maxZ;
        },
        /**
* define target focus for the value.
* @private
*/
        _targetFocus: function () {
            var minWidth = 30;  //  minimum width for textbox in visualmode
            if (this.model.multiSelectMode == "visualmode") this.element.width(minWidth);
            if (this.element.hasClass("e-watermark")) {
                this.element.removeClass("e-watermark");
                this.element.val("");
            }
            this.wrapper.addClass("e-focus");
            this._isFocused = true;
            /* Raise focusIn Event*/
           
            this._trigger("focusIn", { value:this.model.value });
        },
        /**
* define visual mode and other modes with value
* @private
*/
        _targetBlur: function () {
            this._isFocused = false;
            this.wrapper.removeClass("e-focus");
            !this._isWatermark && this._setWatermarkTxt();
            this._removeSelection();

            if (this.model.multiSelectMode == "visualmode") {
                this._removeActive();
                this.element.val("").width(1);
                this._preVal = "";
            }
            else {
                if (this.model.multiSelectMode == "delimiter") {
                    var val, deli, last;
                    val = $.trim(this.element.val());
                    deli = this.model.delimiterChar;
                    last = val.substr(val.length - deli.length, val.length);
                    if (last == deli) {
                        this.element.val(val.substr(0, val.length - deli.length));
                        this._valueChange();
                    }
                }
                this._updateSelectedItemArray();
            }

            /* Raise focusOut Event*/
            this.model.multiSelectMode != "visualmode" && this._hiddenInput.val(this.element.val());
            this._trigger("focusOut", { value: this.model.value });
        },
        /**
* remove selection from the autocomplete textbox.
* @private
*/
        _removeSelection: function () {
            if (this.model.enableAutoFill) {
                this.element.attr("aria-autocomplete", "both");
                var selection = this._getCaretSelection();
                if (selection.end - selection.start != 0 && selection.end - selection.start != this.element.val().length) 
                    this.target.value = this.target.value.substr(0, selection.start);
            }
        },
        /**
* remove list hover after hover item is changed.
* @private
*/
        _removeListHover: function () {
            this._getLiTags().removeClass("e-hover");
        },
        /**
* add hover class for the hovered  list item.
* @private
*/
        _addListHover: function () {
            var activeItem = $(this._getLiTags()[this._activeItem - 1]);
            activeItem.addClass("e-hover");
            this.scrollerObj.setModel({ "scrollTop": this._calcScrollTop() });
            activeItem.focus();
        },
        /**
* Caluculate scroll top for  the scroller.
* @private
*/
        _calcScrollTop: function () {
            var ulH = this.ul.outerHeight(), li = this.ul.find("li"), liH = 0, index, top, i;
            index = this.ul.find("li.e-hover").index();
            for (i = 0; i < index; i++) { liH += li.eq(i).outerHeight(); }
            top = liH - ((this.suggestionList.outerHeight() - li.eq(index).outerHeight()) / 2);
            return top;
        },
        /**
* Retrives the text from the active item.
* @private
*/
        _getActiveText: function () {
            if (this._mapper.txt) return this._getField(this._currList[this._activeItem - 1], this._mapper.txt);
            else return this._currList[this._activeItem - 1];
        },
        /**
* Retrives the unique key of the curent selected item.
* @private
*/
        _getUniqueKey: function () {
            var key = null;
            if (this._mapper.key) key = this._getField(this._currList[this._activeItem - 1], this._mapper.key);
            ej.isNullOrUndefined(key) && (key = null);
            return key;
        },
        /**
* set the textbox value for selecting the function
* @private
*/
        _setTextBoxValue: function () {
            if (this._activeItem && !this.noresult) {
                var currentValue, selection, val, text;
                currentValue = this._getActiveText();
                selection = this._getCaretSelection();
                this.suggLen = selection.start;
                val = this.target.value.substr(0, this.suggLen);

                if (val && val.toLowerCase() == currentValue.substr(0, this.suggLen).toLowerCase()) {
                    if (this.model.multiSelectMode == "delimiter") {
                        var _deliIndex = (this.target.value).lastIndexOf(this.model.delimiterChar);
                        var query = $.trim(val.substr(_deliIndex + 1, this.suggLen));
                        text = val + currentValue.substr(query.length, currentValue.length);
                    }
                    else {
                        text = val + currentValue.substr(val.length, currentValue.length);
                    }
                    this.element.val(text);
                    this._autofilSelection();
                    if (this.model.multiSelectMode == "visualmode")
                        this._adjustWidth();
                }
                else this._removeSelection();
            }
        },
        /**
* entering textbox value with the value selected.
* @private
*/
        _enterTextBoxValue: function () {
			this.element.removeClass("e-watermark");
            if (this._activeItem && !this.noresult && !this.model.readOnly) {
                var currentValue = this._getActiveText(), currItem = this._currList[this._activeItem - 1];
                this._valueToTextBox(currentValue, currItem, false);
                var _value = this.model.multiSelectMode == "visualmode" ? this._hiddenInput.val() : this.element.val();
               
                    this._trigger("select", { value: _value, text: currentValue, key: this._getUniqueKey() });
                this._valueChange();
            }
        },
        /**
* creating box functionality for the box model.
* @private
*/
        _createBox: function (value) {
            var span = ej.buildTag("span.e-icon e-close");
            var li = ej.buildTag("li.e-options", value).append(span);

            this._on(span, "click", function (e) {
                if (!this.model.enabled) return false;
                this._deleteBox($(e.target).parent());
            });
            return li;
        },
        /**
* add loading class while data in search.
* @private
*/
        _addLoadingClass: function () {
            if (this.model.showLoadingIcon)
                this.element.addClass("e-load");
        },
        /**
* Remove loading class after data is loaded.
* @private
*/
        _removeLoadingClass: function () {
            this.element.removeClass("e-load");
        },
        /**
* define functionalities for the highlighted search.
* @private
*/

        _highlightSuggestion: function (suggestion) {
            if ($.trim(this._queryString) != "") {
                var caseSensitive, tempQueryString, RegEx, mch, split, query;
                caseSensitive = this.model.caseSensitiveSearch ? "g" : "gi";
                query = $.trim(this._queryString);
                tempQueryString = this.model.filterType == "startswith" ? "^" + query : query;
                RegEx = new RegExp(tempQueryString, caseSensitive);

                if (RegEx.test(suggestion)) {
                    mch = suggestion.match(RegEx);
                    split = suggestion.split(RegEx);
                    suggestion = "";
                    $(split).each(function (i, val) {
                        if (mch[i])
                            suggestion += val + "<span class='e-hilight-txt'>" + mch[i] + "</span>";
                        else
                            suggestion += val;
                    });
                }
            }
            return suggestion;
        },
        /**
* Applying rtl option for the autocomplete widget.
* @private
*/
        _RightToLeft: function (value) {
            if (value) {
                this.wrapper.addClass("e-rtl");
                this.suggestionList.addClass("e-rtl");
            }
            else {
                this.wrapper.removeClass("e-rtl");
                this.suggestionList.removeClass("e-rtl");
            }
        },
        /**
* Set rounded corner for the autocomplete widget.
* @private
*/
        _setRoundedCorner: function (value) {
            if (value) {
                this.container.addClass("e-corner-all");
                this.suggestionList.addClass("e-corner-all");
            }
            else {
                this.container.removeClass("e-corner-all");
                this.suggestionList.removeClass("e-corner-all");
            }
        },
        /**
*Checking readonly functionality of autocomplete widget.
* @private
*/
        _checkReadOnly: function () {
            if (this.model.readOnly) {
                this.element.attr({ "readonly": "readonly", "aria-readonly": true });
                this._off(this.element, "keydown", this._OnKeyDown);
                this._off(this.element, "keyup", this._OnKeyUp);
            }
            else {
                this.element.removeAttr("readonly").removeAttr("aria-readonly");
                this._on(this.element, "keydown", this._OnKeyDown);
                this._on(this.element, "keyup", this._OnKeyUp);
            }
        },
        /**
* Defining key down functionality for autocomplete widget.
* @private
*/
        _OnKeyDown: function (e) {
            if (this.model.filterType != "startswith")
                this.model.enableAutoFill = false;

            switch (e.keyCode) {
                case 37:    // Left arrow
                case 35:    // End Key
                case 36:    // Home Key
                    this._removeSelection();
                case 39:    //Right arrow
                    break;
                case 38:    //Up arrow
                    e.preventDefault();
                    if (this.showSuggestionBox && this.suggestionList) {
                        this._removeListHover();
                        if (this._activeItem > 1)
                            this._activeItem -= 1;
                        else
                            this._activeItem = this._listSize;
                        this._addListHover();
                        if (this.model.enableAutoFill) {
                            this.element.attr("aria-autocomplete", "both");
                            this._queryString = this.target.value;
                            this._setTextBoxValue();
                        }
                    }
                    break;
                case 40:    //Down arrow
                    e.preventDefault();
                    if (this.showSuggestionBox && this.suggestionList) {
                        this._removeListHover();
                        if (this._activeItem < this._listSize)
                            this._activeItem += 1;
                        else
                            this._activeItem = 1;
                        this._addListHover();
                        if (this.model.enableAutoFill) {
                            this.element.attr("aria-autocomplete", "both");
                            this._queryString = this.target.value;
                            this._setTextBoxValue();
                        }
                    }
                    break;
                case 8:    // Backspace key
                    if ($.trim(this.element.val()) == "") this._isOpened = false;
                    if (this.model.multiSelectMode == "visualmode" && this.element.val() == "") {
                        this._deleteLastBox();
                    }
                    break;
                case 17:    // Ctrl key
                    this.ctrlKeyPressed = true;
                    break;
                case 9:    // Tab key
                    if (this.showSuggestionBox) {
                        e.preventDefault();
                        this._queryString = this.element.val();
                        this._enterTextBoxValue();
                        this._hideResult();
                        this.element.attr("aria-expanded", false);
                    }
                    break;
                case 27:    // Esc key
                    this._isOpened = false;
                    this._hideResult();
                    this.element.attr("aria-expanded", false);
                    break;
            }
            if (this.model.multiSelectMode == "visualmode") {
                if (e.keyCode != 17 && e.keyCode != 8 && e.keyCode != 46) this._removeActive();
                this._adjustWidth();
            }
        },
        /**
 * Defining key up  for autocomplete widget.
 * @private
 */
        _OnKeyUp: function (e) {
            this._keyDownComplete();
            if (this.ctrlKeyPressed) {
                if (e.keyCode == 17)
                    this.ctrlKeyPressed = false;
                return false;
            }

            if ($.trim(this.element.val()) == "" && e.keyCode == 38 && e.keyCode == 40) {
                this._hideResult();
                this.element.attr("aria-expanded", false);
                return false;
            }
            if (!this._checkDelimiter()) return false;
            this._queryString = $.trim(this._queryString);

            switch (e.keyCode) {
                //Restricts the keyup event for other functional keys          

                case 38: // Up Key
                case 40: // Down Key
                case 37: // Left Key
                case 39: // Right Key
                case 20: // CapsLk
                case 16: // Shift Key
                case 17: // Ctrl Key
                case 18: // Alt Key
                case 35: // End Key
                case 36: // Home Key
                case 144: // Num Lock
                case 9:    // Tab key
                case 27: break;  // Esc Key

                case 13:    // Enter Key
                    e.preventDefault();
                    this._queryString = this.element.val();
                    this._enterTextBoxValue();
                    if (this.model.enableAutoFill) {
                        this.element.attr("aria-autocomplete", "both");
                        this.suggLen = this.element.val().length;
                        this._autofilSelection();
                    }
                    this._isOpened = false;
                    this._hideResult();
                    this.element.attr("aria-expanded", false);
                    break;

                case 46:    // Delete Key
                    if (this.model.multiSelectMode == "visualmode" && this.element.val() == "") {
                        this._deleteBox(this._ulBox.children("li.e-active"));
                        break;
                    }
                case 8:     // Backspace Key
                    if (this._queryString.length >= this.model.minCharacter) {
                        this._autoFill = false;
                        this._OnTextEnter();
                    }
                    else {
                        this.noresult = true;
                        this._hideResult();
                        this.element.attr("aria-expanded", false);
                        if ($.trim(this.element.val()) == "") this._isOpened = false;
                    }
                    break;

                default:
                    if (this._queryString.length >= this.model.minCharacter) {
                        this._autoFill = true;
                        this._OnTextEnter();
                    }
                    else this.noresult = true;
                    break;
            }
        },
        /**
* Getting filtered list from the data source.
* @private
*/
        _getFilteredList: function (list) {
            if (!list || !list.length || list.length < 1) this.suggestionListItems = [];
            else {
                var tempQuery = ej.Query();
                this._addQuery(tempQuery, typeof list[0] == "object");
                this.suggestionListItems = ej.DataManager(list).executeLocal(tempQuery);
            }
            this._doneRemaining();
        },
        /**
* Performing searching operation using query.
* @private
*/
        _performSearch: function () {
            var source = this.model.dataSource;
            if (ej.DataManager && source instanceof ej.DataManager) {
                if (!source.dataSource.offline) {
                    window.clearTimeout(this.timer);
                    var proxy = this;
                    // a time delay to avoid the continuous request
                    this.timer = window.setTimeout(function () {
                        proxy._fetchRemoteDat(source);
                    }, 700);
                }
                else this._getFilteredList(source.dataSource.json);
            }
            else this._getFilteredList(source);
        },
        /**
* Fetching data from the remote location using query.
* @private
*/
        _fetchRemoteDat: function (source) {
            var proxy = this, queryPromise, tempQuery = this._getQuery();
            this._addQuery(tempQuery, true);
            queryPromise = source.executeQuery(tempQuery);
            queryPromise.fail(function (e) {
                proxy.suggestionListItems = null;
                proxy._removeLoadingClass();
            }).done(function (e) {
                proxy.suggestionListItems = e.result;
                proxy._doneRemaining();
            });
        },
        /**
 * Add sorting operation for the query.
 * @private
 */
        _addSortingQuery: function (query, key) {
            if (this.model.allowSorting) {
                var order = (this.model.sortOrder == "descending") ? true : false;
                query.sortBy(key, order);
            }
        },
        /**
 * Add value to the query for by retriving value from maaper value field
 * @private
 */
        _addQuery: function (_query, checkMapper) {
            var bindTo = "";
            if (checkMapper) {
                var mapper = this.model.fields;
                bindTo = (mapper && mapper.text) ? mapper["text"] : "text";
            }
            if (this._queryString) _query.where(bindTo, this.model.filterType, this._queryString, !this.model.caseSensitiveSearch);
            this._addSortingQuery(_query, bindTo);
            if (this.model.itemsCount > 0) _query.take(this.model.itemsCount);
        },
        /**
 * creates a query for the given value to process the data spurce
 * @private
 */
        _getQuery: function () {
            if (ej.isNullOrUndefined(this.model.query)) {
                var column = [], queryManager = ej.Query(), mapper = this.model.fields;
                for (var col in mapper) {
                    if (col !== "tableName" && mapper[col])
                        column.push(mapper[col]);
                }
                if (column.length > 0)
                    queryManager.select(column);
                if (!this.model.dataSource.dataSource.url.match(mapper.tableName + "$"))
                    !ej.isNullOrUndefined(mapper.tableName) && queryManager.from(mapper.tableName);
            }
            else queryManager = this.model.query.clone();
            return queryManager;
        },
        /**
 * While typing the text in text box the functionalities has to be done are mentioned here
 * @private
 */
        _OnTextEnter: function () {
            this._addLoadingClass();
            this._hideResult();
            this.element.attr("aria-expanded", false);
            this._performSearch();
        },
        /**
     * Fill the text box with the complete word in the auto fill option
     * @private
     */
        _doneRemaining: function () {
            this._showSuggestionList();
            this.element.attr("aria-expanded", true).attr("aria-haspopup", true);
            if (this.model.enableAutoFill && this._autoFill && !this.noresult) {
                this.element.attr("aria-autocomplete", "both");
                this._activeItem = 1;
                this._queryString = this.target.value;
                this._setTextBoxValue();
            }
            this._removeLoadingClass();
        },
        /**
      * Remove repeated values in the suggestion list
      * @private
      */
        _removeRepeated: function () {
            var results = this.suggestionListItems;
            if (!results || results.length == 0 || this._selectedItems.length == 0)
                return false;
            for (var i = 0; i < this._selectedItems.length; i++) {
                var index = results.indexOf(this._selectedItems[i]);
                if (index != -1) this.suggestionListItems.splice(index, 1);
            }
        },
        /**
      * Check deleimiter while typing in textbox widget
      * @private
      */
        _checkDelimiter: function () {
            this._queryString = !this.element.hasClass("e-watermark") ? this.element.val() : "";
            var _deliIndex = (this.model.multiSelectMode != "delimiter") ? -1 : this._queryString.lastIndexOf(this.model.delimiterChar);

            /*Check Delimiter Conditions and current text*/
            if (_deliIndex == -1) {
                return true;
            }
            else if (_deliIndex + 1 == this._queryString.length) {
                this._hideResult();
                this.element.attr("aria-expanded", false);
                return false;
            }
            else {
                this._queryString = this._queryString.substr(_deliIndex + 1, this._queryString.length);
                return true;
            }
        },
        /**
      * autofill selection in autocomplete is processed with below functionalities
      * @private
      */
        _autofilSelection: function () {
            var element = this.element[0], totLen = this.element.val().length;

            if (element.setSelectionRange)
                element.setSelectionRange(this.suggLen, totLen);
            else if (element.createTextRange) {
                // For lower version browsers (IE8, IE7 ...)
                element = element.createTextRange();
                element.collapse(true);
                element.moveEnd('character', totLen);
                element.moveStart('character', this.suggLen);
                element.select();
            }
        },
        /**
      * dropdown button click is processed with the following action
      * @private
      */
        _OnDropdownClick: function (e) {
            if (this.model.enabled && !this.model.readOnly) {
                e.preventDefault();
                this._addLoadingClass();
                if (this.showSuggestionBox) {
                    this._isOpened = false;
                    this._hideResult();
                    this.element.attr("aria-expanded", false);
                    this._removeLoadingClass();
                }
                else this._showFullList();
            }
        },
        /**
      * Shows the autocomplete popup with full list
      * @private
      */
        _showFullList: function () {
            if (!this._isFocused)
                this.element.focus();
            this._queryString = null;
            this._autoFill = false;
            this._performSearch();
        },
        /**
      * Mouse enter in popup is detected and processed here
      * @private
      */
        _OnMouseEnter: function (e) {
            var targetEle = e.target;
            if (e.target.tagName != "LI") targetEle = $(e.target).parents("li");
            this._getLiTags().removeClass("e-hover");
            $(targetEle).addClass("e-hover");

            this._queryString = this.element.val();
            this._activeItem = this._getLiTags().index($(targetEle)) + 1;
        },
        /**
      * Mouse leave in popup is detected and processed here
      * @private
      */
        _OnMouseLeave: function (e) {
            this._getLiTags().removeClass("e-hover");
            if (this.model.highlightSearch)
                this._getLiTags().find(".e-hilight-txt").removeClass("e-hover");
        },
        /**
      * Mouse click of popup is detected and processed here	
      * @private
      */
        _OnMouseClick: function (e) {
            if (!this.noresult) {
                this._enterTextBoxValue();
                this._isOpened = false;
                this._hideResult();
                this.element.attr("aria-expanded", false);
            }
        },
        /**
      * document click detection and binding and prodcessing
      * @private
      */
        _OnDocumentClick: function (e) {
            if (!$(e.target).is(this.suggestionList) && !$(e.target).parents(".e-atc-popup").is(this.suggestionList) &&
                !$(e.target).is(this.element) && !$(e.target).parents(".e-atc").is(this.wrapper)) {
                this._isOpened = false;
                this._hideResult();
                this.element.attr("aria-expanded", false);
            }
            else if ($(e.target).is(this.suggestionList) || $(e.target).parents(".e-atc-popup").is(this.suggestionList))
                e.preventDefault();
        },
        /**
      * window resize events for popup control		
      * @private
      */
        _OnWindowResize: function (e) {
            this._refreshPopup();
        },
        /**
      * value change fuction in checkbox events	
      * @private
      */
        _valueChange: function () {
            var currValue = this.model.multiSelectMode == "visualmode" ? this._hiddenInput.val() : this.element.val();
            if (this.model.value != currValue) {
                this.model.value = currValue;
                this.model.multiSelectMode != "visualmode" && this._hiddenInput.val(this.element.val());
                    this._trigger("change", { value: currValue });
            }
        },
        /**
      * Update selected array of selection from datasource 
      * @private
      */
        _updateSelectedItemArray: function (e) {
            var value = this.getValue();
            this._selectedItems = [];
            if (this.model.multiSelectMode == "delimiter") {
                var values = value.split(this.model.delimiterChar);
                for (var i = 0; i < values.length; i++)
                    if (values[i]) this._selectedItems.push(values[i]);
            }
            else if (this.model.multiSelectMode == "none" && value)
                this._selectedItems.push(value);
        },
        /**
       * KeyDown complete of selection event		
       * @private
       */
        _keyDownComplete: function () {
            var currValue = this.element.val();
            if (this._preVal != currValue) {
                this._preVal = currValue;
                if (this.model.multiSelectMode == "visualmode") this._adjustWidth();
                this._valueChange();
            }
        },
        /**
       * character Move to End of Autocomple control		
       * @private
       */
        _moveCaretToEnd: function (el) {
            if (typeof el.selectionStart == "number") {
                el.selectionStart = el.selectionEnd = el.value.length;
            } else if (typeof el.createTextRange != "undefined") {
                var range = el.createTextRange();
                range.collapse(false);
                range.select();
            }
        },
        /**
       * character selection of Autocomple control		
       * @private
       */
        _getCaretSelection: function () {
            var input = this.element[0], start = 0, end = 0;

            if (!isNaN(input.selectionStart)) {
                start = input.selectionStart;
                end = input.selectionEnd;
                return { start: Math.abs(start), end: Math.abs(end) };
            }
            // For lower version browsers (IE8, IE7 ...)
            var bookmark = document.selection.createRange().getBookmark();
            var selection = input.createTextRange();
            selection.moveToBookmark(bookmark);

            var before = input.createTextRange();
            before.collapse(true);
            before.setEndPoint("EndToStart", selection);
            var beforeLength = before.text.length, selLength = selection.text.length;
            return { start: beforeLength, end: beforeLength + selLength };
        },

        //-------------------- Event Wire-up -------------------------//
        /**
         * Wiring the events to Autocomple control		
		 * @private
         */
        _wireEvents: function () {
            this._on(this.element, "focus", this._targetFocus);
            this._on(this.element, "blur", this._targetBlur);
        }
    });
    /**
	 * Enum for Autocomplete Filter mode.	 
	 * @enum {string}
	 * @global 
	 */
    ej.filterType = {
        /**  Supports to search text with startswith  */
        StartsWith: "startswith",
        /**  Supports to search text with contains */
        Contains: "contains",
        /**  Supports to search text with endswith */
        EndsWith: "endswith",
        /**  Supports only for number lessthan only */
        LessThan: "lessthan",
        /**  Supports only for number greaterthan only */
        GreaterThan: "greaterthan",
        /**  Supports only for number lessthanorequal only */
        LessThanOrEqual: "lessthanorequal",
        /**  Supports only for number greaterthanorequal only */
        GreaterThanOrEqual: "greaterthanorequal",
        /**  Supports only for number equal only */
        Equal: "equal",
        /** Supports only for number notequal only */
        NotEqual: "notequal"
    };
    /**
	 * Enum for Autocomplete sortOrder mode.	 
	 * @enum {string}
	 * @global 
	 */
    ej.SortOrder = {
        /** Supports to sorts with ascending only */
        Ascending: "ascending",
        /** Supports to sorts with descending only */
        Descending: "descending"
    };
    /**
	 * Enum for Autocomplete selection mode.	 
	 * @enum {string}
	 * @global 
	 */
    ej.MultiSelectMode = {
        /** Supports to selection mode with none only */
        None: "none",
        /** Supports to selection mode with delimitter only */
        Delimiter: "delimiter",
        /** Supports to selection mode with visualmode only */
        VisualMode: "visualmode"
    }
})(jQuery, Syncfusion);;
/**
* @fileOverview Plugin provides support to display calendar within your web page and allows to pick the date.
* @copyright Copyright Syncfusion Inc. 2001 - 2014. All rights reserved.
*  Use of this code is subject to the terms of our license.
*  A copy of the current license can be obtained at any time by e-mailing
*  licensing@syncfusion.com. Any infringement will be prosecuted under
*  applicable laws. 
* @version 12.1 
* @author <a href="mailto:licensing@syncfusion.com">Syncfusion Inc</a>
*/

(function ($, ej, undefined) {
    /**
    * @namespace ej
	* @classdesc Date selection with the input field.
	* @class ejDatePicker
	* @param {object} options - settings for Date Picker.
	* @requires jQuery
    * @requires jquery.easing.1.3.js
    * @requires jquery.globalize.js
    * @requires globalize.cultures.min.js
	* @requires ej.core.js
	* @requires ej.datepicker.js
	* @example 
	* &lt;input type="text" id="datepicker" /&gt;
	* &lt;script&gt;
	* // Create DatePicker
    * $("#datepicker").ejDatePicker();
	* &lt;/script&gt;
	*/

    ej.widget("ejDatePicker", "ej.DatePicker", /** @lends ejDatePicker# */ {
        // widget element will be automatically set in this
        element: null,
        _rootCss: "e-datepicker",
        // user defined model will be automatically set in this
        model: null,
        validTags: ["input", "div", "span"],
        _setFirst: false,
		_cancelValue: false,
        type: "editor",

        // default model
        defaults:/** @lends ejDatePicker# */{
            /**		
			* Specifies the header format of days in short, longer or min types. See {@link Header}
			* @default ej.DatePicker.Header.ShowHeaderMin
			* @type {String | Enum}
			* @example 
			* &lt;input type="text" id="datepicker" /&gt;
	        * &lt;script&gt;
			* //To set dayHeaderFormat API value during initialization  
			* 	$("#datepicker").ejDatePicker({  dayHeaderFormat: ej.DatePicker.Header.ShowHeaderShort });
			* &lt;/script&gt;
			* @memberof ejDatePicker
			* @instance
			*/
            dayHeaderFormat: "showheadermin",

            /**		
			* Shows the date icon button at right side of textbox and shows DatePicker popup on clicking it.
			* @default true
			* @type {Boolean}
			* @example 
			* &lt;input type="text" id="datepicker" /&gt;
	        * &lt;script&gt;
			* //To set showPopupButton API during initialization  
			* 	$("#datepicker").ejDatePicker({  showPopupButton: false });
			* &lt;/script&gt;
			* @memberof ejDatePicker
			* @instance
			*/
            showPopupButton: true,
			enableAnimation: true,
            /**		
			* It allows to show footer in DatePicker calendar to select today date.
			* @default true
			* @type {Boolean}
			* @example 
			* &lt;input type="text" id="datepicker" /&gt;
	        * &lt;script&gt;
			* //To set showFooter API during initialization  
			* 	$("#datepicker").ejDatePicker({  showFooter: false });
			* &lt;/script&gt;
			 * @memberof ejDatePicker
			* @instance
			*/
            showFooter: true,

            /**		
			* Allows to embed the DatePicker in the page. Also associate DatePicker with div element instead of input.
			* @default false
			* @type {Boolean}
			* @example 
			* &lt;input type="text" id="datepicker" /&gt;
	        * &lt;script&gt;
			* //To set displayInline API during initialization  
			* 	$("#datepicker").ejDatePicker({  displayInline: true });
			* &lt;/script&gt;
			 * @memberof ejDatePicker
			* @instance
			*/
            displayInline: false,

            /**		
			* Specifies the date format to be displayed in the input textbox of Datepicker. The selected Datepicker value will be displayed in specified date format.
			* @default "MM/dd/yyyy"
			* @type {String}
			* @example 
			* &lt;input type="text" id="datepicker" /&gt;
	        * &lt;script&gt;
			* //To set dateFormat API during initialization  
			* 	$("#datepicker").ejDatePicker({  dateFormat: "dd/MM/yyyy" });
			* &lt;/script&gt;
			 * @memberof ejDatePicker
			* @instance
			*/
            dateFormat: '',

            /**		
			* Specifies the water mark text to be display in input text.
			* @default "Select date"
			* @type {String}
			* @example 
			* &lt;input type="text" id="datepicker" /&gt;
	        * &lt;script&gt;
			* //To set watermarkText during initialization  
			* 	$("#datepicker").ejDatePicker({  watermarkText: "Enter date" });
			* &lt;/script&gt;
			 * @memberof ejDatePicker
			* @instance
			*/
            watermarkText: "Select date",

            /**		
			* Allow to display default date value in input textbox.
			* @default true
			* @type {Boolean}
			* @example 
			* &lt;input type="text" id="datepicker" /&gt;
	        * &lt;script&gt;
			* //To set displayDefaultDate API during initialization  
			* 	$("#datepicker").ejDatePicker({  displayDefaultDate: true });
			* &lt;/script&gt;
			* @memberof ejDatePicker
			* @instance
			*/
            displayDefaultDate: true,

            /**		
			* Specifies the selected date value.
			* @default null
			* @type {String | DateObject}
			* @example 
			* &lt;input type="text" id="datepicker" /&gt;
	        * &lt;script&gt;
			* //To set the datepicker value during initialization  
			* 	$("#datepicker").ejDatePicker({  value: new Date("5/5/2014") });
			* &lt;/script&gt;
			 * @memberof ejDatePicker
			* @instance
			*/
            value: null,

           /**		
           * Specifies the minimum date range to be displayed in DatePicker calendar.
           * @default new Date(1900, 00, 01)
           * @type {String | DateObject}
           * @example 
		   * &lt;input type="text" id="datepicker" /&gt;
	       * &lt;script&gt;
           * //To set minDate value during initialization  
           * 	$("#datepicker").ejDatePicker({  minDate: new Date("5/1/2013") });
           * &lt;/script&gt;
            * @memberof ejDatePicker
           * @instance
           */
            minDate: new Date(1900, 01, 01),

            /**		
           * Specifies the maximum date range to be displayed in DatePicker calendar.
           * @default new Date(2099, 11, 31)
           * @type {String | DateObject}
           * @example 
		   * &lt;input type="text" id="datepicker" /&gt;
	       * &lt;script&gt;
           * //To set maxDate value during initialization  
           * 	$("#datepicker").ejDatePicker({  maxDate : new Date("5/30/2015") });
           * &lt;/script&gt;
            * @memberof ejDatePicker
           * @instance
           */
            maxDate: new Date(2099, 12, 31),

            /**		
			* Specifies the start level view in DatePicker calendar. See {@link Level}
			* @default ej.DatePicker.Level.Month
			* @type {String | Enum}
			* @example 
			* &lt;input type="text" id="datepicker" /&gt;
	        * &lt;script&gt;
			* //To set startLevel API during initialization  
			* 	$("#datepicker").ejDatePicker({  startLevel: ej.DatePicker.Level.Year });
			* &lt;/script&gt;
			* @memberof ejDatePicker
			* @instance
			*/
            startLevel: "month",

            /**		
			* Specifies the start level view in DatePicker calendar. See {@link Level}
			* @default ""
			* @type {String | Enum}
			* @example 
			* &lt;input type="text" id="datepicker" /&gt;
	        * &lt;script&gt;
			* //To set depthLevel API during initialization  
			* 	$("#datepicker").ejDatePicker({  depthLevel: ej.DatePicker.Level.Year });
			* &lt;/script&gt;
			 * @memberof ejDatePicker
			* @instance
			*/
            depthLevel: "",

            /**		
			* Set the root class for DatePicker theme. This cssClass API helps to use custom skinning option for DatePicker control. 
			* @default ""
			* @type {String}
			* @example 
			* &lt;input type="text" id="datepicker" /&gt;
	        * &lt;script&gt;
			* //To set cssClass API during initialization  
			* 	$("#datepicker").ejDatePicker({  cssClass: "gradient-lime" });
			* &lt;/script&gt;
			 * @memberof ejDatePicker
			* @instance
			*/
            cssClass: "",

            /**		
			* Specifies the start day of the week in DatePicker calendar.
			* @default 1
			* @type {Number}
			* @example 
			* &lt;input type="text" id="datepicker" /&gt;
	        * &lt;script&gt;
			* //To set startDay API during initialization  
			* 	$("#datepicker").ejDatePicker({  startDay: 2 });
			* &lt;/script&gt;
			 * @memberof ejDatePicker
			* @instance
			*/
            startDay: 0,

            /**		
			* Specifies the number of months to navigate at one click in next and previous button.
			* @default 1
			* @type {Number}
			* @example 
			* &lt;input type="text" id="datepicker" /&gt;
	        * &lt;script&gt;
			* //To set stepMonths API during initialization  
			* 	$("#datepicker").ejDatePicker({  stepMonths: 2 });
			* &lt;/script&gt;
			 * @memberof ejDatePicker
			* @instance
			*/
            stepMonths: 1,

            /**		
			* Culture the language of DatePicker calendar.
			* @default "en-US"
			* @type {String}
			* @example 
		    * &lt;input type="text" id="datepicker" /&gt;
	        * &lt;script&gt;
			* //To set locale API during initialization  
			* 	$("#datepicker").ejDatePicker({  locale: "en-US" });
			* &lt;/script&gt;
			 * @memberof ejDatePicker
			* @instance
			*/
            locale: "en-US",

            /**		
			* It allows to show days in other months of DatePicker calendar.
			* @default true
			* @type {Boolean}
			* @example 
			* &lt;input type="text" id="datepicker" /&gt;
	        * &lt;script&gt;
			* //To set showOtherMonths API during initialization  
			* 	$("#datepicker").ejDatePicker({  showOtherMonths: false });
			* &lt;/script&gt; 
			 * @memberof ejDatePicker
			* @instance
			*/
            showOtherMonths: true,

            /**		
			* When enableStrictMode true it allows the value outside of the range also, otherwise it internally changed to the correct value.
			* @default false
			* @type {Boolean}
			* @example 
			* &lt;input type="text" id="datepicker" /&gt;
	        * &lt;script&gt;
			* //To set enableStrictMode API during initialization  
			* 	$("#datepicker").ejDatePicker({  enableStrictMode: true });
		    * &lt;/script&gt; 
			 * @memberof ejDatePicker
			* @instance
			*/
            enableStrictMode: false,

            /**		
			* Enables or disables the state maintenance of DatePicker.
			* @default false
			* @type {Boolean}
			* @example 
			* &lt;input type="text" id="datepicker" /&gt;
	        * &lt;script&gt;
			* //To set enablePersistence API during initialization  
			* 	$("#datepicker").ejDatePicker({  enablePersistence: true });
			* &lt;/script&gt; 
			 * @memberof ejDatePicker
			* @instance
			*/
            enablePersistence: false,

            /**		
			* Enables or disables the datepicker control.
			* @default true
			* @type {Boolean}
			* @example 
			* &lt;input type="text" id="datepicker" /&gt;
	        * &lt;script&gt;
			* //To set enabled API during initialization  
			* 	$("#datepicker").ejDatePicker({  enabled: false });
			* &lt;/script&gt;
			 * @memberof ejDatePicker
			* @instance
			*/
            enabled: true,

            /**		
			* Specifies the width of the datepicker input text.
			* @default "160px"
			* @type {String}
			* @example 
			* &lt;input type="text" id="datepicker" /&gt;
	        * &lt;script&gt;
			* //To set width API during initialization  
			* 	$("#datepicker").ejDatePicker({  width: 200 });
			* &lt;/script&gt; 
			 * @memberof ejDatePicker
			* @instance
			*/
            width: "",

            /**		
			* Specifies the height of the datepicker input text.
			* @default "28px"
			* @type {String}
			* @example 
			* &lt;input type="text" id="datepicker" /&gt;
	        * &lt;script&gt;
			* //To set height API during initialization  
			* 	$("#datepicker").ejDatePicker({  height: 35 });
			* &lt;/script&gt; 
			 * @memberof ejDatePicker
			* @instance
			*/
            height: "",

            /**		
			* Display Right to Left direction of DatePicker calendar and input box.
			* @default false
			* @type {Boolean}
			* @example 
			* &lt;input type="text" id="datepicker" /&gt;
	        * &lt;script&gt;
			* //To set enableRTL API during initialization  
			* 	$("#datepicker").ejDatePicker({  enableRTL : true });
			* &lt;/script&gt;
			 * @memberof ejDatePicker
			* @instance
			*/
            enableRTL: false,

            /**		
			* DatePicker input will be displayed in rounded corner style, when this property is set to true.
			* @default false
			* @type {Boolean}
			* @example 
			* &lt;input type="text" id="datepicker" /&gt;
	        * &lt;script&gt;
			* //To set showRoundedCorner API during initialization  
			* 	$("#datepicker").ejDatePicker({  showRoundedCorner : true });
			* &lt;/script&gt;
			 * @memberof ejDatePicker
			* @instance
			*/
            showRoundedCorner: false,

            /**		
			* Specifies the header format to be displayed in the pop up of Datepicker.
			* @default "MMMM yyyy"
			* @type {String}
			* @example 
			* &lt;input type="text" id="datepicker" /&gt;
	        * &lt;script&gt;
			* //To set headerFormat API during initialization  
			* 	$("#datepicker").ejDatePicker({  headerFormat : "MMMM yy" });
			* &lt;/script&gt; 
			 * @memberof ejDatePicker
			* @instance
			*/
            headerFormat: 'MMMM yyyy',

            /**		
			* Set the text name for the today button in the datepicker popup. 
			* @default "Today"
			* @type {String}
			* @example 
			* &lt;input type="text" id="datepicker" /&gt;
	        * &lt;script&gt;
			* //To set buttonText API during initialization  
			* 	$("#datepicker").ejDatePicker({  buttonText : "Now" });
			* &lt;/script&gt;
			* @memberof ejDatePicker
			* @instance
			*/
            buttonText: 'Today',

            /**		
			* Indicates that the datepicker value can only be read. 
			* @default false
			* @type {Boolean}
			* @example 
			* &lt;input type="text" id="datepicker" /&gt;
	        * &lt;script&gt;
			* //To set readOnly API during initialization  
			* 	$("#datepicker").ejDatePicker({  readOnly : true });
			* &lt;/script&gt; 
			* @memberof ejDatePicker
			* @instance
			*/
            readOnly: false,
            /**		
           * Specify the special dates in datepicker
           * @default null
           * @type {object}
           * @example 
           * &lt;input type="text" id="datepicker" /&gt;
           * &lt;script&gt;
           * //To set specialDates API value during initialization             
		   * // declaration
           *$("#datepicker").ejDatePicker({specialDates:window.spldays});
           * &lt;/script&gt;
           * @memberof ejDatePicker
           * @instance
           */
            specialDates: null,
            /**		
        * Specify the fields mapping in datepicker
        * @default null
        * @type {object}
        * @example 
        * &lt;input type="text" id="datepicker" /&gt;
        * &lt;script&gt;
        * //To set fields API value during initialization         
        * // declaration
        * $("#datepicker").ejDatePicker({	
        * specialDates: window.spldays, fields: {date:"date",tooltip:"tooltip",icon:"icon"}});
        * &lt;/script&gt;
        * @memberof ejDatePicker
        * @instance
        */
            fields: {
                date: "date",
                tooltip: "tooltip",
                icon: "icon",
            },
            /**		
             * DatePicker Tooltip will be displayed, when this property is set to true.
             * @default true
             * @type {Boolean}
             * @example 
             * &lt;input type="text" id="datepicker" /&gt;
             * &lt;script&gt;
             * //To set tooltip API during initialization  
             * 	$("#datepicker").ejDatePicker({  showTooltip : false });
             * &lt;/script&gt;
              * @memberof ejDatePicker
             * @instance
             */
            showTooltip: true,
           /**		
           * HighlightSection used to highlight current month, week, workdays. See {@link HighlightSection}
           * @default "none"
           * @type {String | Enum}
           * @example 
           * &lt;input type="text" id="datepicker" /&gt;
           * &lt;script&gt;
           * //To set highlightSection API during initialization  
           * 	$("#datepicker").ejDatePicker({  highlightSection: "week" });
           * &lt;/script&gt;
           * @memberof ejDatePicker
           * @instance
           */
            highlightSection: "none",
            /**		
            * Week end will be displayed in bold, when this property is set to true.
            * @default false
            * @type {Boolean}
            * @example 
            * &lt;input type="text" id="datepicker" /&gt;
            * &lt;script&gt;
            * //To set highlightWeekend API during initialization  
            * 	$("#datepicker").ejDatePicker({  highlightWeekend : true });
            * &lt;/script&gt;
             * @memberof ejDatePicker
            * @instance
            */
            highlightWeekend: false,

            /**     
            * Fires when each date is created in the DatePicker popup.
            * @event
            * @name ejDatePicker#beforeDateCreate 	
            * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
            * @param {object}  argument.model returns the datepicker model
            * @param {string}  argument.type returns the name of the event
            * @param {object}  argument.date returns the current created date object
            * @param {object}  argument.element returns the current DOM object of the date from the Calendar
            * @example 
			* &lt;input type="text" id="datepicker" /&gt;
	        * &lt;script&gt;
            * //beforeDateCreate event for datepicker
            * $("#datepicker").ejDatePicker({
            *    beforeDateCreate: function (args) {}
            * });  
		    * &lt;/script&gt;				
            * @memberof ejDatePicker
            * @instance
            */
            beforeDateCreate: null,

            /**     
            * Fires when DatePicker popup opened.
            * @event
            * @name ejDatePicker#open 	
            * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
            * @param {object}  argument.model returns the datepicker model
            * @param {string}  argument.type returns the name of the eventclose"
            * @param {string}  argument.value returns the current date value
            * @param {string}  argument.prevDate returns the previously selected value
            * @example 
			* &lt;input type="text" id="datepicker" /&gt;
	        * &lt;script&gt;
            * //open event for datepicker
            * $("#datepicker").ejDatePicker({
            *    open: function (args) {}
            * });  
		    * &lt;/script&gt;				
            * @memberof ejDatePicker
            * @instance
            */
            open: null,

            /**     
            * Fires when DatePicker popup closed.
            * @event
            * @name ejDatePicker#close 	
            * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
            * @param {object}  argument.model returns the datepicker model
            * @param {string}  argument.type returns the name of the event
            * @param {string}  argument.value returns the current date value
            * @param {string}  argument.prevDate returns the previously selected value
            * @example 
			* &lt;input type="text" id="datepicker" /&gt;
	        * &lt;script&gt;
            * //close event for datepicker
            * $("#datepicker").ejDatePicker({
            *    close: function (args) {}
            * });   
		    * &lt;/script&gt;				
            * @memberof ejDatePicker
            * @instance
            */
            close: null,

            /**     
            * Fires when a date is selected from the datepicker popup.
            * @event
            * @name ejDatePicker#select 	
            * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
            * @param {object}  argument.model returns the datepicker model
            * @param {string}  argument.type returns the name of the event
            * @param {string}  argument.value returns the current date value
            * @param {string}  argument.prevDate returns the previously selected value
            * @example 
			* &lt;input type="text" id="datepicker" /&gt;
	        * &lt;script&gt;
            * //select event for datepicker
            * $("#datepicker").ejDatePicker({
            *    select: function (args) {}
            * }); 
		    * &lt;/script&gt;				
            * @memberof ejDatePicker
            * @instance
            */
            select: null,

            /**     
            * Fires when the datepicker input value is changed.
            * @event
            * @name ejDatePicker#change 	
            * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
            * @param {object}  argument.model returns the datepicker model
            * @param {string}  argument.type returns the name of the event
            * @param {string}  argument.value returns the datepicker input value
            * @param {string}  argument.prevDate returns the previously selected value
            * @example 
			* &lt;input type="text" id="datepicker" /&gt;
	        * &lt;script&gt;
            * //change event for datepicker
            * $("#datepicker").ejDatePicker({
            *    change: function (args) {}
            * });  
		    * &lt;/script&gt;				
            * @memberof ejDatePicker
            * @instance
            */
            change: null,

            /**     
            * Fires when datePicker input gets focus.
            * @event
            * @name ejDatePicker#focusIn 	
            * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
            * @param {object}  argument.model returns the datepicker model
            * @param {string}  argument.type returns the name of the event
            * @example 
			* &lt;input type="text" id="datepicker" /&gt;
	        * &lt;script&gt;
            * //focusIn event for datepicker
            * $("#datepicker").ejDatePicker({
            *    focusIn: function (args) {}
            * }); 
		    * &lt;/script&gt;				
            * @memberof ejDatePicker
            * @instance
            */
            focusIn: null,

            /**     
            * Fires when datePicker input losses the focus.
            * @event
            * @name ejDatePicker#focusOut 	
            * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
            * @param {object}  argument.model returns the datepicker model
            * @param {string}  argument.type returns the name of the event
            * @example 
			* &lt;input type="text" id="datepicker" /&gt;
	        * &lt;script&gt;
            * //focusOut event for datepicker
            * $("#datepicker").ejDatePicker({
            *    focusOut: function (args) {}
            * });
		    * &lt;/script&gt;				
            * @memberof ejDatePicker
            * @instance
            */

            focusOut: null,           

			/**     
          * Fires when create DatePicker successfully.
          * @event
          * @name ejDatePicker#create 	
          * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
          * @param {object}  argument.model returns the DatePicker model
          * @param {string}  argument.type returns the name of the event
          * @example 
			* &lt;input type="text" id="datepicker" /&gt;
			* &lt;script&gt;
			* //create event for datepicker
            * $("#datepicker").ejDatePicker({
            *    create: function (args) {}
            * });
		  * &lt;/script&gt;				
          * @memberof ejDatePicker
          * @instance
          */
            create: null,
			/**     
          * Fires when the DatePicker is destroyed successfully.
          * @event
          * @name ejDatePicker#destroy 	
          * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
          * @param {object}  argument.model returns the DatePicker model
          * @param {string}  argument.type returns the name of the event
          * @example 
			* &lt;input type="text" id="datepicker" /&gt;
			* &lt;script&gt;
			* //destroy event for datepicker
            * $("#datepicker").ejDatePicker({
            *    destroy: function (args) {}
            * }); 
		  * &lt;/script&gt;			
          * @memberof ejDatePicker
          * @instance
          */
            destroy: null


        },

        /**
         * Specify the data types for default properties 
		 * @private
         */
        dataTypes: {
            startDay: "number",
            stepMonths: "number",
            showOtherMonths: "boolean",
            enableStrictMode: "boolean",
            showRoundedCorner: "boolean",
            enableRTL: "boolean",
            displayDefaultDate: "boolean",
            displayInline: "boolean",
            showPopupButton: "boolean",
            locale: "string",
            readOnly: "boolean",
            cssClass: "string",
            dateFormat: "string",
            watermarkText: "string",
            headerFormat: "string",
            buttonText: "string",
            specialDates: "data",
            showTooltip: "boolean",
            highlightSection: "enum",
            highlightWeekend: "boolean",
			enableAnimation: "boolean",          

        },

        _setModel: function (jsondata) {
            if (!(this.model.enabled) && ej.isNullOrUndefined(jsondata["enabled"])) return false;
            var callRefresh = false, start = false, validate = false;
            for (var key in jsondata) {
                switch (key) {
                    case "dayHeaderFormat":
                        this.model.dayHeaderFormat = jsondata[key];
                        callRefresh = true;
                        break;
                    case "showPopupButton":
                        this._renderDateIcon(jsondata[key], true);
                        break;
                    case "displayInline":
                        this._setDisplayInline(jsondata[key]);
                        break;
                    case "value":
                        this._setDateValue(jsondata[key]);
                        jsondata[key] = this.model.value;
                        validate = true;
                        callRefresh = true;
                        break;
                    case "specialDates":
                        this.model.specialDates = jsondata[key];
                        this._createSpecialDateObject();
                        callRefresh = true;
                        break;
                    case "fields":
                        this.model.fields = jsondata[key];
                        this._mapField = this._getMapper();
                        callRefresh = true;
                        break;
                    case "showTooltip":
                        this.model.showTooltip = jsondata[key];
                        callRefresh = true;
                        break;
                    case "highlightWeekend":
                        this.model.highlightWeekend = jsondata[key];
                        callRefresh = true;
                        break;
                    case "highlightSection":
                        this.model.highlightSection = jsondata[key];                        
                        callRefresh = true;
                        break;
                    case "dateFormat":
                        this.model.dateFormat = jsondata[key];
                        this._setDateValue(this.model.value);
                        break;
                    case "minDate":
                        this._setMinDate(jsondata[key]);
                        jsondata[key] = this.model.minDate;
                        validate = true;
                        callRefresh = true;
                        break;
                    case "maxDate":
                        this._setMaxDate(jsondata[key]);
                        jsondata[key] = this.model.maxDate;
                        validate = true;
                        callRefresh = true;
                        break;
                    case "locale":
                        this.model.locale = jsondata[key];
                        this.model.dateFormat = '';
                        this._setCulture(jsondata[key]);
                        this._setDateValue(this.model.value);
                        callRefresh = true;
                        break;
                    case "showOtherMonths":
                        this.model.showOtherMonths = jsondata[key];
                        this._otherMonthsVisibility();
                        break;
                    case "enableStrictMode":
                        this.model.enableStrictMode = jsondata[key];
                        validate = true;
                        callRefresh = true;
                        break;
                    case "readOnly":
                        this.model.readOnly = jsondata[key];
                        this._disbleMaualInput();
                        break;
                    case "width":
                        this._setWidth(jsondata[key]);
                        break;
                    case "height":
                        this._setHeight(jsondata[key]);
                        break;
                    case "cssClass":
                        this._setSkin(jsondata[key]);
                        break;
                    case "enableRTL":
                        this._setRTL(jsondata[key]);
                        break;
                    case "showRoundedCorner":
                        this._setRoundedCorner(jsondata[key]);
                        break;
                    case "enabled":
                        if (!jsondata[key]) this.disable();
                        else this.enable();
                        break;
                    case "buttonText":
                        this._setFooterText(jsondata[key]);
                        break;
                    case "showFooter":
                        this._enableFooter(jsondata[key]);
                        break;
                    case "watermarkText":
                        this.model.watermarkText = jsondata[key]; this._setWaterMark();
                        break;
                    case "startDay":
                        if (parseInt(jsondata[key]) < 0 || parseInt(jsondata[key]) > 6) jsondata[key] = 0;
                        this.model.startDay = jsondata[key];
                        callRefresh = true;
                        break;
                    case "startLevel":
                        this.model.startLevel = jsondata[key]; start = true;
                        callRefresh = true;
                        break;
                    case "headerFormat":
                        this.model.headerFormat = jsondata[key];
                        callRefresh = true;
                        break;
                    case "depthLevel":
                        this.model.depthLevel = jsondata[key]; start = true;
                        callRefresh = true;
                        break;
					case "enableAnimation":
							this.model.enableAnimation=options[key];
                        break;
                
                }
            }
            if (validate) {
                this._validateMinMaxDate();
                jsondata["value"] = this.model.value;
                jsondata["maxDate"] = this.model.maxDate;
                jsondata["minDate"] = this.model.minDate;
            }
            this._setWaterMark();

            if (callRefresh && this.isValidState )
                this._refreshDatepicker();            
            if (start) this._startLevel(this.model.startLevel);
            this._triggerChangeEvent();
            this._checkErrorClass();
        },
        observables: ["value"],
        // all events bound using this._on will be unbind automatically
        _destroy: function () {
            if (this._isOpen)
                this.hide();
            this.sfCalendar.remove();
            if (this.wrapper) {
                this.element.insertAfter(this.wrapper);
                this.wrapper.remove();
            }
            this.element.removeClass('e-datepicker e-input');
        },
        // constructor function
        _init: function () {
            this._setValues();
            this._createDatePicker();
        },
        //------------------------------Private Methods---------------------------------------------
        _setValues: function () {
            this.Date = new Date();
            this._id = this.element[0].id;
            this.isValidState = true;
            this._setCulture(this.model.locale);
            this._setMinDate(this.model.minDate);
            this._setMaxDate(this.model.maxDate);
            this._calendarDate = this._zeroTime(new Date());
            if (this.model.startDay < 0 || this.model.startDay > 6) this.model.startDay = 0;
            this.Date.firstDayOfWeek = this.model.startDay;
            this.Date.fullYearStart = '20';
            this._showHeader = true,
            this._validateMinMaxDate();
            this._dateValue = new Date(this._calendarDate.toString());
            this._isIE7 = this._checkIE7();
            // this variable is set to true in DateTimePicker control
            this._getInternalEvents = false;
            this._flag = true;
            this._ejHLWeekEnd = false;
            this._isOpen = false;
            this._prevDate = null;
            this._isFocused = false;
        },

        _createDatePicker: function () {
            this._createWrapper();
            this._wireEvents();
            if (this.model.displayInline)
                this.show();
            if (this.model.enableRTL) this._setRTL(true);
            if (this.model.showRoundedCorner) this._setRoundedCorner(true);
        },
        _checkNameAttr: function () {
            if (this.element.attr("name"))
                this._hiddenInput.attr("name", this.element.attr("name"));
            else {
                this._hiddenInput.attr("name", this.element[0].id);
				this.element.attr("name", this.element[0].id);
			}
        },
        _createWrapper: function () {
            this._mapField = this._getMapper();
            if (this.model.specialDates) {
                this._createSpecialDateObject();
            }
            if (this._isInputBox()) {
                this.element.addClass("e-input").attr({ 'aria-atomic': 'true', 'aria-live': 'assertive', 'tabindex': '0' }); //
                this.wrapper = ej.buildTag("span.e-datewidget e-widget " + this.model.cssClass);
                this.innerWrapper = ej.buildTag("span.e-in-wrap e-box");
                this.wrapper.append(this.innerWrapper).insertBefore(this.element);
                this.innerWrapper.append(this.element);
                this.innerWrapper.addClass('e-padding');
                this.dateIcon = ej.buildTag("span.e-select#" + this._id + "-img", "", null)
                    .append(ej.buildTag("span.e-icon e-date", "", {}, { 'aria-label': 'Select' })).insertAfter(this.element);
            }
            this._hiddenInput = ej.buildTag("input#" + this._id + "_hidden", "", {}, { type: "hidden" }).insertBefore(this.element);
            this._checkNameAttr();
            this._setHeight(this.model.height);
            if (this.model.width != "") this._setWidth(this.model.width);
			var oldWrapper = $("#e-" + this.element.context.id).get(0);
            if (oldWrapper)
                $(oldWrapper).remove();
            this.sfCalendar = ej.buildTag('div.e-datepicker e-popup e-widget ' + this.model.cssClass + ' e-calendar ' + (this.model.specialDates ? (this.model.specialDates[0][this._mapField._icon] ? 'e-icons ' : '') : ''), "", {}, { id: 'e-' + this._id }).attr({ 'aria-hidden': 'true' })
                .insertBefore(this.element);
            this.popup = this.sfCalendar;
            this._resizeCalender();
            if (this.model.displayDefaultDate) this._setDateValue(this.model.value);
            this._preValue = this._textboxVal();
            this._setWaterMark();
            this._createCalender();
            this._setDisplayInline(this.model.displayInline);
            if (this.model.readOnly) this._disbleMaualInput();
            if (!this.model.enabled) this.disable();
            this._layoutChanged();
            this._checkErrorClass();
        },
        _isInputBox: function () {
            return (this.element.is("input") && (this.element.is("input[type=text]") || !this.element.attr('type')));
        },

        _renderDateIcon: function (bool, reRender) {
            if (reRender && this.model.showPopupButton == bool) return;
            if (!bool && this.dateIcon) {
                this.dateIcon.css('display', 'none');
                this.innerWrapper.removeClass('e-padding');
                this._bindInputEvent();
            }
            else {
                if (this.innerWrapper) {
                    this.innerWrapper.addClass('e-padding');
                    this.dateIcon.css('display', 'block');
                }
                if (!this.model.displayInline)
                    this._bindDateButton();
            }
            this.model.showPopupButton = bool;
        },

        _resizeCalender: function () {
            if (this.model.dayHeaderFormat == "showheadershort")
                this.sfCalendar.removeClass("e-headerlong");
            else if (this.model.dayHeaderFormat == "showheaderlong") {
                this.sfCalendar.addClass("e-headerlong");
            }
        },

        _setWidth: function (value) {
            if (this.wrapper) this.wrapper.width(value);
            else this.element.width(value);
        },
        _setHeight: function (value) {
            if (value) {
                if (this.wrapper) this.wrapper.height(value);
                else this.element.height(value);
            }
            if (this._isIE7) this.element.height(this.innerWrapper.height());
        },
        _setRTL: function (isRTL) {
            if (isRTL) {
                if (this.model.showPopupButton && this.wrapper)
                    this.wrapper.addClass("e-rtl");
                else this.element.addClass("e-rtl");
                this.sfCalendar.addClass("e-rtl");
            }
            else {
                if (this.model.showPopupButton && this.wrapper)
                    this.wrapper.removeClass("e-rtl");
                else this.element.removeClass("e-rtl");
                this.sfCalendar.removeClass("e-rtl");
            }
        },
        _setRoundedCorner: function (bool) {
            if (bool) {
                if (this.innerWrapper) this.innerWrapper.addClass("e-corner-all");
                this.sfCalendar.addClass("e-corner-all");
            }
            else {
                if (this.innerWrapper) this.innerWrapper.removeClass("e-corner-all");
                this.sfCalendar.removeClass("e-corner-all");
            }
        },

        _refreshDatepicker: function () {
            this._setDateValue(this._textboxVal());
            $(".e-text", this.sfCalendar).text(this._asString(this._calendarDate, this.model.headerFormat));
            this._resizeCalender();
            this._dateValue = new Date(this._calendarDate.toString());
            this._hoverDate = this._calendarDate.getDate() - 1;
            this._renderCalendar(this, this._dateValue);
            this._setFooterText(this.model.buttonText);
            this._enableFooter(this.model.showFooter);
            this._layoutChanged();
        },
        _setFooterText: function (footerText) {
            $('.e-footer-text', this.sfCalendar).html(footerText);
        },
        _setSkin: function (skin) {
            if (this.wrapper) {
                this.wrapper.removeClass(this.model.cssClass);
                this.wrapper.addClass(skin);
            }
            else {
                this.element.removeClass(this.model.cssClass);
                this.element.addClass(skin);
            }
            this.sfCalendar.removeClass(this.model.cssClass);
            this.sfCalendar.addClass(skin);
        },
        _setDisplayInline: function (isDisplayInline) {
            this.model.displayInline = isDisplayInline;
            if (isDisplayInline && this._isInputBox()) {
                this.sfCalendar.insertAfter(this.wrapper);
                this._setDatePickerPosition();
            }
            else if (isDisplayInline) {
                this.element.append(this.sfCalendar);
            }
            else {
                this.sfCalendar.css('display', 'none');
                $('body').append(this.sfCalendar);
                this._isOpen = false;
            }
            if (isDisplayInline) {
                this.show();
                this._off($(this.dateIcon, this.wrapper), "click", $.proxy(this._showDatePopUp, this));
                this.element.unbind("click", $.proxy(this._showDatePopUp, this));
            }
            else this._renderDateIcon(this.model.showPopupButton, false);
        },
        /*Disabling Manual input */
        _disbleMaualInput: function () {
            if (this.model.readOnly) {
                $(this.element).attr("readonly", "readonly");
                if (!this.model.displayInline) this.hide();
            }
            else {
                $(this.element).removeAttr("readonly");
            }
        },
        _checkDateObject: function (date) {
            if (!date || (typeof JSON === "object" && JSON.stringify(date) === "{}")) return date = "";
            else if (!(date instanceof Date)) {
                var val = this._parseDate(date);
                date = (val != null) ? val : ((this._prevDate == null) ? ((this._textboxVal() != date && this._textboxVal() == "") ? new Date(date) :null) : null);
            }
            if (!isNaN(Date.parse(date))) return this._dateValue = this._calendarDate = this._zeroTime(date);
            return null;
        },
        _checkInstanceType: function (date) {
            date = this._stringToObject(date);
            if (!date) return null;
            else if (!(date instanceof Date)) {
                date = this._parseDate(date);
            }
            if (!isNaN(Date.parse(date))) return this._zeroTime(date);
            return null;
        },
        _stringToObject: function (value) {
            if (typeof value === "string") {
                var val = Globalize.parseDate(value, this.model.dateFormat);
                value = (val != null) ? val : new Date(value);
            }
            return value;
        },
        _validateMinMaxDate: function () {
            var dateChange = false, valueExceed = false;
            if (this.model.maxDate < this.model.minDate) this.model.minDate = this.model.maxDate;
            if (!this.model.enableStrictMode) {
                if (this.model.value) {
                    if (this.model.value < this.model.minDate) {
                        this._calendarDate = this.model.value = this.model.minDate;
                        dateChange = true;
                    }
                    else if (this.model.value > this.model.maxDate) {
                        this._calendarDate = this.model.value = this.model.maxDate;
                        dateChange = true;
                    }
                }
                else {
                    this.element.val("");
                    if (this._calendarDate < this.model.minDate) this._calendarDate = this.model.minDate;
                    else if (this._calendarDate > this.model.maxDate) this._calendarDate = this.model.maxDate;
                }
                this.isValidState = true;
            }
            else {
                if (this.model.value) {
                    if (this.model.value < this.model.minDate) {
                        this._calendarDate = this.model.minDate;
                        this.isValidState = false;
                        valueExceed = true;
                    }
                    else if (this.model.value > this.model.maxDate) {
                        this._calendarDate = this.model.maxDate;
                        this.isValidState = false;
                        valueExceed = true;
                    }
                    else this.isValidState = true;
                }
                else {
                    if (this._calendarDate < this.model.minDate) this._calendarDate = this.model.minDate;
                    else if (this._calendarDate > this.model.maxDate) this._calendarDate = this.model.maxDate;
                }
            }
            if (dateChange) this.element.val(this._formatter(this.model.value, this.model.dateFormat));
            if (valueExceed && this._getInternalEvents) this._trigger("outOfRange");
        },
        _setCulture: function (culture) {
            this.culture = Globalize.culture(culture);
            if (this.culture) {
                this.Date.dayNames = this.culture.calendar.days.names;
                this.Date.dayNamesMin = this.culture.calendar.days.namesShort;
                this.Date.abbrDayNames = this.culture.calendar.days.namesAbbr;
                this.Date.monthNames = this.culture.calendar.months.names;
                this.Date.abbrMonthNames = this.culture.calendar.months.namesAbbr;
                this.Date.format = this.culture.calendar.patterns.d;
                if (this.model.dateFormat == '') this.model.dateFormat = this.culture.calendar.patterns.d;
            } else {
                this.Date.dayNames = ['Sunday', 'Monday', 'Tuesday', 'Wednesday', 'Thursday', 'Friday', 'Saturday'];
                this.Date.dayNamesMin = ["Su", "Mo", "Tu", "We", "Th", "Fr", "Sa"];
                this.Date.abbrDayNames = ['Sun', 'Mon', 'Tue', 'Wed', 'Thu', 'Fri', 'Sat'];
                this.Date.monthNames = ['January', 'February', 'March', 'April', 'May', 'June', 'July', 'August', 'September', 'October', 'November', 'December'];
                this.Date.abbrMonthNames = ['Jan', 'Feb', 'Mar', 'Apr', 'May', 'Jun', 'Jul', 'Aug', 'Sep', 'Oct', 'Nov', 'Dec'];
                this.Date.format = 'MM/dd/yyyy';
                if (this.model.dateFormat == '') this.model.dateFormat = 'MM/dd/yyyy';
            }
            this._separator = this._getSeparator();
        },
        /* Set Water Mark Text */
        _setWaterMark: function () {
            if (this.element != null && this.element.hasClass("e-input") && this._textboxVal() == "" && this.model.watermarkText) {
                this.isValidState = true;
                this._checkErrorClass();
                this.element.addClass("e-watermark").val(this.model.watermarkText);
                this.model.value = null;
                return true;
            }
        },
        _setDatePickerPosition: function () {
            var pos = this._getOffset(this.wrapper);
            var left = pos.left;
            var totalHeight = this.wrapper.outerHeight();
            var maxZ = this._getZindexPartial();
            if (this.model.enableRTL) left += this.wrapper.width() - this.sfCalendar.width();

            this.sfCalendar.css({
                "left": left + "px",
                'position': 'absolute',
                "top": (pos.top + totalHeight + 3) + "px",
                "z-index": maxZ
            });
        },
        _getOffset: function (ele) {
            var pos = ele.offset();
            if ($("body").css("position") != "static") {
                var bodyPos = $("body").offset();
                pos.left -= bodyPos.left;
                pos.top -= bodyPos.top;
            }
            return pos;
        },
        _getZindexPartial: function () {
            var parents = this.element.parents(), bodyEle;
            bodyEle = $('body').children(), index = bodyEle.index(this.popup);
            bodyEle.splice(index, 1);
            $(bodyEle).each(function (i, ele) { parents.push(ele); });

            var maxZ = Math.max.apply(maxZ, $.map(parents, function (e, n) {
                if ($(e).css('position') != 'static') return parseInt($(e).css('z-index')) || 1;
            }));
            if (!maxZ || maxZ < 10000) maxZ = 10000;
            else maxZ += 1;
            return maxZ;
        },

        _setMinDate: function (d) {
            this.model.minDate = this._checkInstanceType(d);
            if (!this.model.minDate) {
                this.model.minDate = (new Date('11/31/1899'));
            }
        },

        _setMaxDate: function (d) {
            this.model.maxDate = this._checkInstanceType(d);
            if (!this.model.maxDate) {
                this.model.maxDate = (new Date('12/31/2099')); // using the JS Date.parse function which expects mm/dd/yyyy
            }
        },
        _setDateValue: function (date) {
            this._removeWatermark();
            var newDate = this._checkDateObject(date);
            if (newDate != null) {
                this.isValidState = true;
                if (newDate == "") {
                    this.element.val("");
                    this.model.value = "";
                } else {
                    this.model.value = newDate;
                    this._validateMinMaxDate();
                    this._preTxtValue = this.element.val(this._formatter(this.model.value, this.model.dateFormat));
                }
            }
            else {
                (typeof date === "string" && this.model.enableStrictMode) ? this.element.val(date) : this.element.val("");
                this.model.value = "";
                this.isValidState = (this._textboxVal() == "") ? true : false;
            }
            this._hiddenInput.val(this._textboxVal());
        },
        _updateInputVal: function () {
            var val = this._validateValue();
            if (val != null) {
                if (this.sfCalendar.find('.e-datepicker-days').is(':visible')) {
                    this._refreshDatepicker();
                }
                else if (this.model.displayDefaultDate) {
                    this._setDateValue(val);
                }
            }
        },
        _validateInputVal: function () {
            var val = this._validateValue();
            if (val != null) {
                if (val <= this.model.maxDate && val >= this.model.minDate)
                    this.isValidState = true;
                else {
                    this.model.value = "";
                    this.isValidState = false;
                }
            }
        },

        _validateValue: function () {
            return this._parseDate(this._textboxVal());
        },
        _getSeparator: function () {
            if (this.culture) {
                formats = this.culture.calendar.patterns.d;
            }
            else formats = this.model.dateFormat;
            var regex = new RegExp("^[a-zA-Z0-9]+$");
            for (var i = 0; i < formats.length; i++) {
                if (!regex.test(formats[i])) return formats[i];
            }
        },
        _checkIE7: function () {
            if (navigator.appName == 'Microsoft Internet Explorer') {
                var re = new RegExp("MSIE ([0-9]{1,}[\.0-9]{0,})"), version = -1;
                if (re.exec(navigator.userAgent) != null)
                    version = parseFloat(RegExp.$1);
                if (version >= 7 && version < 8) return true;
            }
            return false;
        },
        _isValidDate: function (dateObj) {
            return dateObj && typeof dateObj.getTime === "function" && isFinite(dateObj.getTime());
        },
        /*----------------------------------------------region date methods------------------------------------------------------*/

        //Date formatter - Convert date object to specific date format
        _formatter: function (date, format) {
            return Globalize.format(date, format, this.model.locale);
        },
        _parseDate: function (date) {
            return Globalize.parseDate(date, this.model.dateFormat);
        },
        // Checks if the year is a leap year.
        isLeapYear: function (year) {
            return (year % 4 == 0 && year % 100 != 0) || year % 400 == 0;
        },
        //Sets the time component of this Date to zero for cleaner, easier comparison of dates where time is not relevant.
        _zeroTime: function (date) {
			var newDate = typeof date === "string" ? this._parseDate(date) : new Date(date);
            newDate.setMilliseconds(0);
            newDate.setSeconds(0);
            newDate.setMinutes(0);
            newDate.setHours(0);
            return newDate;
        },
        _zeroPad: function (number) {
            // Pad a number with a zero, to make it 2 digits
            return ((number < 10) ? "0" : "") + String(number);
        },
        //Convert date object to specific format
        _formatDate: function (date) {
            return this._formatter(date, this.Date.format);
        },
        //Gets the number of days in the month.
        _getDaysInMonth: function (date) {
            return [31, (this.isLeapYear(date) ? 29 : 28), 31, 30, 31, 30, 31, 31, 30, 31, 30, 31][date.getMonth()];
        },
        //Add a number of days to the date object.
        _addDays: function (d, number) {
            d.setDate(d.getDate() + number);
            return d;
        },
        //Add a number of years to the date object.
        _addYears: function (d, number) {
            d.setFullYear(d.getFullYear() + number);
            return d;
        },
        //Add a number of months to the date object.
        _addMonths: function (d, number) {
            var tempDatedateMonth = d.getDate();
            d.setMonth(d.getMonth() + number);
            if (tempDatedateMonth > d.getDate())
                this._addDays(d, -d.getDate());
            return d;
        },
        //Checks if the day is a weekend day (Sat or Sun).
        _isWeekend: function (date) {
            return date.getDay() == 0 || date.getDay() == 6;
        },

        _isSpecialDates: function (dates) {
            if (this.model.specialDates) {
                for (var i = 0; i < this.model.specialDates.length; i++) {
                    if (this.model.specialDates[i]) {
                        if (dates.getDate() == this.model.specialDates[i][this._mapField._date].getDate() && dates.getMonth() == this.model.specialDates[i][this._mapField._date].getMonth() && dates.getFullYear() == this.model.specialDates[i][this._mapField._date].getFullYear()) {
                            this._getIndex = i;
                            return true;
                        }
                    }
                }
            }
            return false;
        },
        _getMapper: function () {
            var mapper = this.model.fields, mapFld = { _date: null, _tooltip: null, _icon: null };
            mapFld._date = (mapper && mapper.date) ? mapper["date"] : "date";
            mapFld._tooltip = (mapper && mapper.tooltip) ? mapper["tooltip"] : "tooltip";
            mapFld._icon = (mapper && mapper.icon) ? mapper["icon"] : "icon";
            return mapFld;
        },
        _createSpecialDateObject: function () {
            
            for (var i = 0; i < this.model.specialDates.length; i++) {
                if (!(this.model.specialDates[i][this._mapField._date] instanceof Date))
                    this.model.specialDates[i][this._mapField._date] = this._parseDate(this.model.specialDates[i][this._mapField._date]);
            }
            
        },

        // Returns a string representation of the date object according to Date.Format.
        _asString: function (date, format) {
            var r = format || this.Date.format;
            return r
                .split('yyyy').join(date.getFullYear())
                .split('yy').join((date.getFullYear() + '').substring(2))
                .split('MMMM').join(this._getMonthName(false, date))
                .split('MMM').join(this._getMonthName(true, date))
                .split('MM').join(this._zeroPad(date.getMonth() + 1))
                .split('dd').join(this._zeroPad(date.getDate()))
                .split('hh').join(this._zeroPad(date.getHours()))
                .split('min').join(this._zeroPad(date.getMinutes()))
                .split('ss').join(this._zeroPad(date.getSeconds()));
        },
        //Gets the name of the month.
        _getMonthName: function (abbreviated, date) {
            return abbreviated ? this.Date.abbrMonthNames[date.getMonth()] : this.Date.monthNames[date.getMonth()];
        },
        _currentDay: function () {
            var today = new Date();
            var dd = today.getDate();
            var mm = today.getMonth() + 1; //January is 0!
            var yyyy = today.getFullYear();
            return new Date(mm + '/' + dd + '/' + yyyy);
        },
        /*---------------------------------------------region Basic Functionality---------------------------------------*/

        _displayNewMonth: function (m, y) {
            this._setDisplayedMonth(this.displayedMonth + m, this.displayedYear + y, true);
            return false;
        },

        _setDisplayedMonth: function (m, y, rerender) {
            if (this.model.minDate == undefined || this.model.maxDate == undefined) {
                return;
            }
            var s = new Date(this.model.minDate.getTime());
            s.setDate(1);
            var e = new Date(this.model.maxDate.getTime());
            e.setDate(1);

            var t;
            if ((!m && !y) || (isNaN(m) && isNaN(y))) {
                // no month or year passed - default to current month
                t = this._zeroTime(new Date());
                t.setDate(1);
            } else if (isNaN(m)) {
                // just year passed in - presume we want the displayedMonth
                t = new Date(y, this.displayedMonth, 1);
            } else if (isNaN(y)) {
                // just month passed in - presume we want the displayedYear
                t = new Date(this.displayedYear, m, 1);
            } else {
                // year and month passed in - that's the date we want!
                t = new Date(y, m, 1);
            }
            // check if the desired date is within the range of our defined minDate and maxDate
            if (t.getTime() < s.getTime()) {
                t = s;
            } else if (t.getTime() > e.getTime()) {
                t = e;
            }
            var oldMonth = this.displayedMonth;
            var oldYear = this.displayedYear;
            this.displayedMonth = t.getMonth();
            this.displayedYear = t.getFullYear();
            var tempDate = t;
            if (rerender && (this.displayedMonth != oldMonth || this.displayedYear != oldYear)) {
                this._renderCalendar(this, tempDate);
                this._dateValue = tempDate;
                this._trigger("monthChanged", [this.displayedMonth, this.displayedYear]);
            }
        },
        _clearSelected: function () {
            this.numSelected = 0;
            if (this.model.highlightSection == "week") {
                $('td.e-active', this.sfCalendar).removeClass('e-active').attr('aria-selected', false).parent().removeClass('e-selected-week');
            }
            else if (this.model.highlightSection == "month") {
                $('td.e-active', this.sfCalendar).removeClass('e-active').attr('aria-selected', false).parent().parent().removeClass('e-selected-month');
            }
            else if (this.model.highlightSection == "workdays") {
                $('td.e-active', this.sfCalendar).removeClass('e-active').attr('aria-selected', false).parent().removeClass('e-work-week');
            }
            else
                $('td.e-active', this.sfCalendar).removeClass('e-active').attr('aria-selected', false);

        },
        _addSelected: function () {
            if (this.model.highlightSection == "week") {
                $('td.e-active', this.sfCalendar).parent().addClass('e-selected-week');
            }
            else if (this.model.highlightSection == "month") {
                $('td.e-active, this.sfCalendar').parent().parent().addClass('e-selected-month');
            }
            else if (this.model.highlightSection == "workdays") {
                $('td.e-active', this.sfCalendar).parent().addClass('e-work-week');
            }
        },

        _hideOtherMonths: function (sfCalendar) {
            $('td.other-month', sfCalendar).css("visibility", "hidden");
        },
        _showOtherMonths: function (sfCalendar) {
            $('td.other-month', sfCalendar).css({ 'visibility': 'visible' });
        },
		_otherMonthsVisibility: function () {
            if (this.model.showOtherMonths)
				this._showOtherMonths(this.sfCalendar);
            else
                this._hideOtherMonths(this.sfCalendar);
        },


        /*---------------------Region Rendering simple calender----------------------------------------------------------*/

        _createCalender: function () {
            ej.buildTag("div.e-header")
                    .append(ej.buildTag("span.e-prev").append(ej.buildTag('a.e-icon e-left-arrow').attr({ 'role': 'button' })))
                    .append(ej.buildTag("span.e-text").text(this._asString(this._calendarDate, this.model.headerFormat)).attr({ 'aria-atomic': 'true', 'aria-live': 'assertive', 'role': 'heading' }))
                    .append(ej.buildTag("span.e-next").append(ej.buildTag('a.e-icon e-right-arrow').attr({ 'role': 'button' })))
                    .appendTo(this.sfCalendar);
            this._enableHeader(this._showHeader);
            var table = ej.buildTag("table.e-dp-viewdays", "", {}, { 'cellspacing': 2 }).data("e-table", "data").attr({ 'role': 'grid', 'aria-activedescendant': '' });
            this.sfCalendar.append(table);

            this._renderCalendar(this);
            this._startLevel(this.model.startLevel);
            ej.buildTag("div.e-footer")
                .append(ej.buildTag("span.e-footer-icon"))
                .append(ej.buildTag("span.e-footer-text"))
                .appendTo(this.sfCalendar);
            $('.e-footer-text', this.sfCalendar).html(this.model.buttonText);
            this._enableFooter(this.model.showFooter);
        },
        _enableHeader: function (show) {
            if (show) $(".e-header", this.sfCalendar).show();
            else $(".e-header", this.sfCalendar).hide();
        },
        _enableFooter: function (show) {
            if (show) $('.e-footer', this.sfCalendar).show();
            else $('.e-footer', this.sfCalendar).hide();
        },

        //Date picker navigation arrows enable/disable operations
        _checkArrows: function (min, max) {
            this._preArrowCondition(min, this.model.minDate.getFullYear());
            this._nextArrowCondition(max, this.model.maxDate.getFullYear());
        },
        _checkDateArrows: function () {
            this._preArrowCondition(this._tempMinDate, this.model.minDate);
            this._nextArrowCondition(this._tempMaxDate, this.model.maxDate);
        },
        _preArrowCondition: function (val1, val2) {
            if (val1 <= val2) this.sfCalendar.find(".e-left-arrow").addClass("e-disable").attr({ "aria-disabled": true });
            else this.sfCalendar.find(".e-left-arrow").removeClass("e-disable").attr({ "aria-disabled": false });
        },
        _nextArrowCondition: function (val1, val2) {
            if (val1 >= val2) this.sfCalendar.find(".e-right-arrow").addClass("e-disable").attr({ "aria-disabled": true });
            else this.sfCalendar.find(".e-right-arrow").removeClass("e-disable").attr({ "aria-disabled": false });
        },

        _previousNextHandler: function (event) {
            if (this.model.readOnly || !this.model.enabled) return false;
            event.preventDefault();
            var element = ($(event.target).is('a')) ? $(event.target.parentNode) : $(event.target);
            var progress = element.hasClass('e-prev') ? true : false;
            this._processNextPrevDate(progress);
        },
        _processNextPrevDate: function (progress) {
            if (progress && this.sfCalendar.find(".e-left-arrow").hasClass("e-disable")) return false;
            else if (!progress && this.sfCalendar.find(".e-right-arrow").hasClass("e-disable")) return false;

            var currentTable = $("table", this.sfCalendar), temp;
            var tClassName = currentTable.get(0).className;
            switch (tClassName) {
                case 'e-dp-viewdays':
                    var step = this.model.stepMonths;
                    if (progress) {
                        if (this._dateValue <= this.model.minDate) {
                            this._flag = false;
                            return false;
                        }
                    } else {
                        if (this._dateValue >= this.model.maxDate) {
                            this._flag = false;
                            return false;
                        }
                    }
                    this._flag = true;
                    this._addMonths(this._dateValue, (progress ? -step : step));
                    if (this._clickedDate)
                        this._calendarDate = this._clickedDate;
                    this._renderCalendar(this, this._dateValue);
                    $('.e-text', this.sfCalendar).text(this._asString(this._dateValue, this.model.headerFormat));
                    this._addFocus('day', this._hoverDate);
                    break;
                case 'e-dp-viewmonths':
                    if (progress) {
                        if (this._dateValue.getFullYear() <= this.model.minDate.getFullYear()) {
                            this._flag = false;
                            return false;
                        }
                    } else {
                        if (this._dateValue.getFullYear() >= this.model.maxDate.getFullYear()) {
                            this._flag = false;
                            return false;
                        }
                    }
                    this._flag = true;
                    this._addYears(this._dateValue, (progress ? -1 : 1));
                    this._renderCalendar(this, this._dateValue);
                    temp = this._dateValue.getFullYear();
                    $('.e-text', this.sfCalendar).text(temp);
                    $('tbody,tr.e-week-header', currentTable).not('.e-datepicker-months').hide();
                    $($(currentTable).find('.e-datepicker-months')).show();
                    this._addFocus('month', this._hoverMonth);
                    this._checkArrows(temp, temp);
                    break;
                case 'e-dp-viewyears':
                    if (progress) {
                        if (parseInt($('td.e-year-first:first').text()) <= this.model.minDate.getFullYear()) {
                            this._flag = false;
                            return false;
                        }
                    } else {
                        if (parseInt($('td.e-year-last:first').prev().text()) >= this.model.maxDate.getFullYear()) {
                            this._flag = false;
                            return false;
                        }
                    }
                    this._flag = true;
                    this._dateValue.setFullYear(this._dateValue.getFullYear() + (progress ? -10 : 10));
                    this._renderCalendar(this, this._dateValue);
                    var setYear = parseInt(this._dateValue.getFullYear()) - ((parseInt(this._dateValue.getFullYear()) % 10) + 1);
                    $('.e-text').text((setYear + 1) + ' - ' + (setYear + 10));
                    $('tbody,tr.e-week-header', currentTable).not('.e-datepicker-years').hide();
                    $($(currentTable).find('.e-datepicker-years')).show();
                    this._addFocus('year', this._hoverYear);
                    this._checkArrows(setYear + 1, setYear + 10);
                    break;
                case 'e-dp-viewallyears':
                    var headYears;
                    if (progress) {
                        headYears = parseFloat($('td.e-allyear-first', currentTable.get(0)).text().split('-')[1]);
                        if (headYears <= this.model.minDate.getFullYear()) {
                            this._flag = false;
                            return false;
                        } else {
                            this._flag = true;
                        }

                    } else {
                        headYears = parseFloat($('td.e-allyear-last', currentTable.get(0)).prev().text().split('-')[1]);
                        if (headYears >= this.model.maxDate.getFullYear()) {
                            this._flag = false;
                            return false;
                        } else
                            this._flag = true;
                    }
                    this._dateValue.setFullYear(headYears + (progress ? -10 : 10));
                    this._renderCalendar(this, this._dateValue);
                    var setYear = parseInt(this._dateValue.getFullYear()) - ((parseInt(this._dateValue.getFullYear()) % 100) + 1);
                    temp = parseFloat($('td.e-allyear-last', currentTable.get(0)).prev().text().split('-')[1]);
                    $('.e-text', this.sfCalendar).text((setYear + 1) + ' - ' + temp);
                    $('tbody,tr.e-week-header', currentTable).not('.e-datepicker-allyears').hide();
                    $($(currentTable).find('.e-datepicker-allyears')).show();
                    this._addFocus('allyear', this._hoverAllYear);
                    this._checkArrows(setYear + 1, temp);
                    break;
            }
            this._layoutChanged();
        },
        _addFocus: function (selection, index) {
            var cls = 'e-current-' + selection;
            if (selection == 'day') cls = 'current-month';
            var items = this.sfCalendar.find('tbody tr td.' + cls);
            var cell = items[index];
            if (!cell) cell = items.last();
            this.sfCalendar.find('table td').removeClass("e-state-hover");
            $(cell).addClass("e-state-hover");
            return $(cell);
        },
        _setFocusByName: function (name, value) {
            var allValues = this.sfCalendar.find('tbody tr td.e-current-' + name), index, cell;
            $(allValues).each(function (i, ele) {
                if (parseInt(ele.innerHTML) == parseInt(value)) {
                    index = i;
                    return;
                }
            });
            cell = allValues[index];
            if (!cell) cell = allValues.last();
            this.sfCalendar.find('table td').removeClass("e-state-hover");
            $(cell).addClass("e-state-hover");
            return index;
        },
        _getHeaderTxt: function () {
            return this.sfCalendar.find(".e-text").text();
        },
        _forwardNavHandler: function (event) {
            if (this.model.readOnly || !this.model.enabled) return false;
            if (event) event.preventDefault();

            var currentTable = $("table", this.sfCalendar);
            var tclassName = $("table", this.sfCalendar).get(0).className, proxy = this, headerTxt;
            switch (tclassName) {
                case 'e-dp-viewdays':
                    this._hoverMonth = this._getDateObj(currentTable.find(".e-state-hover")).getMonth() ||
                                this._getDateObj(currentTable.find(".e-active")).getMonth() || 0;
                    this._startLevel("year");
                    this._addFocus('month', this._hoverMonth);
                    break;
                case 'e-dp-viewmonths':
                    headerTxt = this._getHeaderTxt();
                    this._startLevel("decade");
                    this._hoverYear = this._setFocusByName('year', headerTxt);
                    break;
                case 'e-dp-viewyears':
                    headerTxt = this._getHeaderTxt();
                    this._startLevel("century");
                    this._hoverAllYear = this._setFocusByName('allyear', headerTxt);
                    break;
            }
            this._layoutChanged();
        },
        _backwardNavHandler: function (event) {

            if (this.model.readOnly || !this.model.enabled) return false;
            var element;
            if (event.type) {
                event.preventDefault();
                element = $(event.currentTarget);
            }
            else element = event;
            var cTable = $("table", this.sfCalendar), temp;
            var tclassName = $("table", this.sfCalendar).get(0).className, proxy = this;
            switch (tclassName) {
                case 'e-dp-viewmonths':
                    cTable.removeClass("e-dp-viewmonths").addClass("e-dp-viewdays");
                    this._dateValue = new Date(this._dateValue.getFullYear(), parseInt($(element).attr('index')), 1);
                    this._renderCalendar(this, this._dateValue);
                    $('tbody', cTable).not('.e-datepicker-days,.e-week-header').hide();
                    $($(cTable).find('.e-datepicker-days,.e-week-header')).fadeIn("fast", function () {
                        proxy._addFocus('day', proxy._hoverDate || 0);
                    });
                    $('.e-text', this.sfCalendar).text(this._asString(this._dateValue, this.model.headerFormat));
                    break;
                case 'e-dp-viewyears':
                    cTable.removeClass("e-dp-viewyears").addClass("e-dp-viewmonths");
                    this._dateValue.setFullYear(parseInt(element.text()));
                    this._renderCalendar(this, this._dateValue);
                    $('tbody,tr.e-week-header', cTable).not('.e-datepicker-months').hide();
                    $($(cTable).find('.e-datepicker-months')).fadeIn("fast", function () {
                        proxy._addFocus('month', proxy._hoverMonth || 0);
                    });
                    temp = element.text();
                    $('.e-text', this.sfCalendar).text(temp);
                    this._checkArrows(temp, temp);
                    break;
                case 'e-dp-viewallyears':
                    var headYears = element.text().split('-');
                    cTable.removeClass("e-dp-viewallyears").addClass("e-dp-viewyears");
                    if (headYears[0] < this.model.minDate.getFullYear()) headYears[0] = this.model.minDate.getFullYear().toString();
                    else if (headYears[0] > this.model.maxDate.getFullYear()) headYears[0] = this.model.maxDate.getFullYear().toString();
                    this._renderCalendar(this, (new Date(headYears[0], 0, 1)));
                    $('tbody,tr.e-week-header', cTable).not('.e-datepicker-years').hide();
                    $($(cTable).find('.e-datepicker-years')).fadeIn("fast", function () {
                        proxy._addFocus('year', proxy._hoverYear || 0);
                    });
                    $('.e-text', this.sfCalendar).text(headYears[0] + '-' + headYears[1]);
                    this._checkArrows(headYears[0], headYears[1]);
                    break;
                default:
                    this._clearSelected();
                    this.sfCalendar.find('table td').removeClass("e-state-hover");
                    element.not('td.other-month').addClass('e-active e-state-hover').attr('aria-selected', true);
                    this._addSelected();
                    //this.sfCalendar.find('table').attr({ 'aria-activedescendant': element[0].id, 'aria-describedby': element[0].id });                  
                    this._hoverDate = this._getDateObj(element).getDate() - 1;
                    this._dateValue = new Date(element.attr('date'));
                    this._clickedDate = new Date(element.attr('date'));
                    break;
            }
            this._layoutChanged();
        },

        _startLevel: function (start) {
            var cTable = $("table", this.sfCalendar);
            var headerText = $(".e-text", this.sfCalendar), s, e;
            switch (start) {
                case "decade":
                    cTable.removeClass("e-dp-viewmonths e-dp-viewdays").addClass("e-dp-viewyears");
                    $('tbody,tr.e-week-header', cTable).not('.e-datepicker-years').hide();
                    $($(cTable).find('.e-datepicker-years')).show();
                    var setYear = parseInt(this._dateValue.getFullYear()) - ((parseInt(this._dateValue.getFullYear()) % 10) + 1);
                    s = setYear + 1;
                    e = setYear + 10;
                    headerText.text(s + ' - ' + e);
                    this._checkArrows(s, e);
                    break;
                case "century":
                    this._renderCalendar(this, this._dateValue);
                    cTable.removeClass("e-dp-viewyears e-dp-viewdays e-dp-viewmonths").addClass("e-dp-viewallyears");
                    $('tbody,tr.e-week-header', cTable).not('.e-datepicker-allyears').hide();
                    $($(cTable).find('.e-datepicker-allyears')).show();
                    s = parseFloat($('td.e-allyear-first', cTable.get(0)).text().split('-')[1]) + 1;
                    e = parseFloat($('td.e-allyear-last', cTable.get(0)).prev().text().split('-')[1]);
                    var headYears = s + '-' + e;
                    headerText.text(headYears);
                    this._checkArrows(s, e);
                    break;
                case "year":
                    cTable.removeClass("e-dp-viewdays").addClass("e-dp-viewmonths");
                    $('tbody,tr.e-week-header', cTable).hide();
                    $($(cTable).find('.e-datepicker-months')).show();
                    s = this._dateValue.getFullYear();
                    headerText.text(s);
                    this._checkArrows(s, s);
                    break;
            }
        },
        _depthLevel: function (depth) {
            var calendarTable = this.sfCalendar;
            switch (depth) {
                case "year":
                    $(calendarTable.find('.e-current-year,.e-current-allyear')).bind("click", $.proxy(this._backwardNavHandler, this));
                    this._on($('.e-current-month', this.sfCalendar), "click", $.proxy(this._onDepthSelectHandler, this));
                    break;
                case "decade":
                    $(calendarTable.find('.e-current-allyear')).bind("click", $.proxy(this._backwardNavHandler, this));
                    $('.e-current-year', this.sfCalendar).bind("click", $.proxy(this._onDepthSelectHandler, this));
                    break;
                case "century":
                    $(calendarTable.find('.e-current-allyear')).bind("click", $.proxy(this._onDepthSelectHandler, this));
                    break;
            }
        },
        _onDepthSelectHandler: function (e) {
            if (this.model.readOnly || !this.model.enabled) return false;
            if ($(e.target).hasClass("e-current-month"))
                this._dateValue = new Date(this._dateValue.setMonth(parseInt(e.target.attributes["index"].value)));
            else if ($(e.target).hasClass("e-current-year"))
                this._dateValue = new Date(this._dateValue.setFullYear(parseInt(e.target.innerHTML)));
            else if ($(e.target).hasClass("e-current-allyear"))
                this._dateValue = new Date(this._dateValue.setFullYear(parseInt(e.target.innerHTML)));
            this._onSetCancelDateHandler(e);
        },
        //Generates datepicker months view
        _datepickerMonths: function (tbody, calendarTable, currentDate) {
            var dc = function (a) {
                return document.createElement(a);
            };
            var month = 0;
            for (var i = 0; i < 3; i++) {
                var row = $(dc('tr'));
                for (var j = 0; j < 4; j++) {
                    var td = $(dc('td'))
                        .addClass('e-current-month e-state-default')
                        .attr('index', month)
                        .html(this.Date.abbrMonthNames[month++]);
                    if (currentDate.getFullYear() < this.model.minDate.getFullYear() || currentDate.getFullYear() > this.model.maxDate.getFullYear()) {
                        td.addClass('e-hidedate e-disable');
                        td.removeClass('e-current-month');
                    }
                    else if ((currentDate.getFullYear() <= this.model.minDate.getFullYear() && month < this.model.minDate.getMonth() + 1) ||
                        (currentDate.getFullYear() >= this.model.maxDate.getFullYear() && month > this.model.maxDate.getMonth() + 1)) {
                        td.addClass('e-hidedate e-disable');
                        td.removeClass('e-current-month');
                    }
                    row.append(td);
                }
                tbody.append(row);
            }
            calendarTable.append(tbody);
            var s = currentDate.getFullYear();
            this._checkArrows(s, s);
        },
        //Generates datepicker years view
        _datepickerYears: function (tbody, calendarTable, currentYear) {
            var dc = function (a) {
                return document.createElement(a);
            };
            var Year = parseInt(currentYear) - ((parseInt(currentYear) % 10) + 1);
            var years = [];
            for (var j = 0; j < 12; j++) {
                years.push(Year + j);
            }
            var year = 0;
            for (var i = 0; i < 3; i++) {
                var row = $(dc('tr'));
                for (var j = 0; j < 4; j++) {
                    var td = $(dc('td'));
                    if (year == 0) {
                        if (parseInt(years[year]) <= this.model.minDate.getFullYear())
                            td.css("visibility", "hidden");
                        else
                            td.css("visibility", "visible");
                        td.addClass('e-year-first e-current-year ');
                    } else if (year == 11) {
                        if (parseInt(years[year]) >= this.model.maxDate.getFullYear()) {
                            td.css("visibility", "hidden");
                        }
                        else {
                            td.css("visibility", "visible");
                        }
                        td.addClass('e-year-last e-current-year ');
                    } else
                        td.addClass('e-current-year e-state-default');
                    if (years[year] < this.model.minDate.getFullYear() || years[year] > this.model.maxDate.getFullYear()) {
                        td.addClass('e-hidedate e-disable');
                        td.removeClass('e-current-year');
                    }
                    td.html(years[year++]);
                    row.append(td);
                }
                tbody.append(row);
            }
            calendarTable.append(tbody);
            this._checkArrows(years[0], years[years.length]);
        },
        //Generates datepicker all-years view
        _datepickerAllYears: function (tbody, calendarTable, currentYear) {
            var Year = parseInt(currentYear) - ((parseInt(currentYear) % 100) + 10);
            var headYear = Year;
            var years = [];

            for (var j = 0; j < 12; j++) {
                years.push(parseInt(Year) + "-\n" + parseInt(Year + 9));
                Year = Year + 10;
            }
            var year = 0;
            for (var i = 0; i < 3; i++) {
                var row = $(document.createElement('tr'));
                for (var j = 0; j < 4; j++) {
                    var td = $(document.createElement('td'));
                    if (year == 0) {
                        if (parseInt(years[year].split('-\n')[1]) <= this.model.minDate.getFullYear())
                            td.css("visibility", "hidden");
                        else
                            td.css("visibility", "visible");
                        td.addClass('e-allyear-first e-current-allyear ');
                    } else if (year == 11) {
                        if (parseInt(years[year].split('-\n')[0] - 1) >= this.model.maxDate.getFullYear()) {
                            td.css("visibility", "hidden");
                        } else {
                            td.css("visibility", "visible");
                        }
                        td.addClass('e-allyear-last e-current-allyear ');
                    } else
                        td.addClass('e-current-allyear e-state-default');
                    if (parseInt(years[year].split('-\n')[1]) < this.model.minDate.getFullYear() || parseInt(years[year].split('-\n')[0]) > this.model.maxDate.getFullYear()) {
                        td.addClass('e-hidedate e-disable');
                        td.removeClass('e-current-allyear');
                    }
                    td.html(years[year++]);
                    row.append(td);
                }
                tbody.append(row);
            }
            calendarTable.append(tbody);
        },
        _renderHeader: function (dpObject) {
            var thead = $(document.createElement('thead'));
            if (dpObject.model.dayHeaderFormat != "showheadernone") {
                var headRow = ej.buildTag("tr.e-week-header").attr('role', 'columnheader');
                for (var i = this.Date.firstDayOfWeek; i < this.Date.firstDayOfWeek + 7; i++) {
                    var weekday = i % 7;
                    var day = this.Date.dayNames[weekday];
                    var headerday;
                    if (dpObject.model.dayHeaderFormat == "showheadershort")
                        headerday = day.substr(0, 3);
                    else if (dpObject.model.dayHeaderFormat == "showheaderlong") headerday = day;
                    else headerday = day.substr(0, 2);
                    var th = ej.buildTag("th", "", {}, { 'scope': 'col', 'abbr': day, 'date': day, 'title': this._formatter(day, "dddd"), 'class': (weekday == 0 || weekday == 6 ? 'e-week-end' : 'e-week-day') })
                            .html(headerday);
                    headRow.append(th);
                }
            };
            return thead.append(headRow);
        },
        // Render a calendar table into any matched elements.
        _renderCalendar: function (dpObject, date) {
            var proxy = this, today;
            dpObject = $.extend({}, ej.DatePicker.prototype.defaults, dpObject);
            this.Date.firstDayOfWeek = this.model.startDay;
            if (date) today = date;
            else if (this._calendarDate) today = this._calendarDate;
            else today = proxy._zeroTime(new Date());
            var calendarTable = $('table', this.sfCalendar);
            calendarTable.empty();
            // header render section
            calendarTable.append(this._renderHeader(dpObject));
            //all years render section
            var tbody = ej.buildTag('tbody.e-datepicker-allyears', "", { 'display': 'none' }, {});
            this._datepickerAllYears(tbody, calendarTable, today.getFullYear());
            //all years render section
            tbody = ej.buildTag("tbody.e-datepicker-years", "", { 'display': 'none' }, {});
            this._datepickerYears(tbody, calendarTable, today.getFullYear());
            var month = dpObject.model.month == undefined ? today.getMonth() : dpObject.model.month;
            var year = dpObject.model.year || today.getFullYear();
            var currentDate = (new Date(year, month, 1, 0, 0, 0));
            var firstDayOffset = this.Date.firstDayOfWeek - currentDate.getDay() + 1;
            if (firstDayOffset > 1) firstDayOffset -= 7;
            var weeksToDraw = Math.ceil(((-1 * firstDayOffset + 1) + this._getDaysInMonth(currentDate)) / 7);
            this._addDays(currentDate, (firstDayOffset - 1));
            var newdate = proxy._zeroTime(new Date());
            var selected = this._calendarDate;
            tbody = ej.buildTag('tbody.e-datepicker-months', "", { 'display': 'none' }, {});
            // month rennder section
            this._datepickerMonths(tbody, calendarTable, today);
            //days render section
            tbody = ej.buildTag('tbody.e-datepicker-days', "", { 'display': 'none' }, {});
            var w = 0, _first = true, _last = true;
            while (w++ < weeksToDraw) {
                var r = jQuery(document.createElement('tr'));
                for (var i = 0; i < 7; i++) {
                    var thisMonth = currentDate.getMonth() == month;
                    var checkSpecialDate = this._isSpecialDates(currentDate);
                    var index = this._getIndex;
                    var d = $(document.createElement('td')).
                        html(checkSpecialDate ? '<span></span><span>' + currentDate.getDate() + '</span>' : currentDate.getDate() + '')
                        .attr({
                            //'id':this._formatter(currentDate, "ddd MMM dd yyyy").replace(/ /g,''),
                            'date': currentDate.toDateString(),
                            'title': (this.model.showTooltip ? (checkSpecialDate && this.model.specialDates[index][this._mapField._tooltip] ? this.model.specialDates[index][this._mapField._tooltip] : this._formatter(currentDate, "ddd MMM dd yyyy")) : ''),
                            'aria-selected': false,
                            'role': 'gridcell'

                        })
                        .addClass((thisMonth ? 'current-month e-state-default ' : 'other-month e-state-default ') +
                            (this._isWeekend(currentDate) ? (this._ejHLWeekEnd ? 'e-dp-weekend e-week-end ' : (this.model.highlightWeekend ? 'e-week-end ' : '')) : 'e-week-day ') +
                            (checkSpecialDate ? (this.model.specialDates[index][this._mapField._icon] ? 'flag ' + this.model.specialDates[index][this._mapField._icon] + ' ' : 'e-special-day ') : '') +
                            (thisMonth && currentDate.getTime() == newdate.getTime() ? 'today ' : ''));
                    d.find('span:first-of-type ').addClass('e-image ');

                    if (selected.getTime() == currentDate.getTime() && thisMonth) {
                        d.addClass('e-active').attr({ 'aria-selected': true });
                        if (this.model.highlightSection == "week") {
                            r.addClass('e-selected-week');
                        }
                        if (this.model.highlightSection == "month") {
                            tbody.addClass('e-selected-month');
                        }
                        if (this.model.highlightSection == "workdays") {
                            r.addClass('e-work-week');
                        }
                        //this.sfCalendar.find('table').attr({ 'aria-activedescendant': d[0].id, 'aria-describedby': d[0].id });
                        if (!this._hoverDate) {
                            d.addClass('e-state-hover');
                            this._hoverDate = currentDate.getDate() - 1;
                        }
                    }
                    var cond = true;
                    if (currentDate < this.model.minDate || currentDate > this.model.maxDate) {
                        d.addClass('e-hidedate e-disable');
                        d.removeClass('current-month other-month');
                        cond = _last = false;
                    }
                    if (thisMonth) {
                        if (cond && _first) {
                            this._tempMinDate = currentDate;
                            _first = false; _last = true;
                        }
                        if (_last) this._tempMaxDate = currentDate;
                    }
                    this._trigger("beforeDateCreate", { date: currentDate, element: d });
                    r.append(d);
                    currentDate = new Date(currentDate.getFullYear(), currentDate.getMonth(), currentDate.getDate() + 1, 0, 0, 0);
                }
                tbody.append(r);
            }
            calendarTable.append(tbody);
            $(tbody).fadeIn("fast");
            if (this.model.startLevel === this.model.depthLevel)
                this._depthLevel(this.model.depthLevel);
            else if (this.model.depthLevel != "month" && this.model.depthLevel != "") {
                if (this.model.startLevel == "century")
                    this._depthLevel(this.model.depthLevel);
                else if (this.model.startLevel == "decade" && this.model.depthLevel != "century")
                    this._depthLevel(this.model.depthLevel);
                else if (this.model.startLevel == "year" && this.model.depthLevel != "decade" && this.model.depthLevel != "century")
                    this._depthLevel(this.model.depthLevel);
                else {
                    this._on(calendarTable.find('.current-month,.other-month,.e-current-month,.e-current-year,.e-current-allyear'), "click", $.proxy(this._backwardNavHandler, this));
                    this._on(calendarTable.find('.current-month , .other-month'), "click", $.proxy(this._onSetCancelDateHandler, this));
                }
            }
            else {
                this._on(calendarTable.find('.current-month,.other-month,.e-current-month,.e-current-year,.e-current-allyear'), "click", $.proxy(this._backwardNavHandler, this));
                this._on(calendarTable.find('.current-month , .other-month'), "click", $.proxy(this._onSetCancelDateHandler, this));
            }

            this._otherMonthsVisibility();
            this._checkDateArrows();
        },

        /*---------------------Keyboard Support----------------------------------------------------------*/

        _keyboardNavigation: function (e) {
            if ((this._isOpen) && (e.keyCode == 37 || e.keyCode == 38 || e.keyCode == 39 || e.keyCode == 40 || e.keyCode == 13 || e.keyCode == 36 || e.keyCode == 35)) {
                e.preventDefault && e.preventDefault();
                var t = { row: null, col: null };

                t.col = this.sfCalendar.find('tbody tr td.e-state-hover').index();
                t.row = this.sfCalendar.find('tbody tr td.e-state-hover').parent().index();

                t.col = (t.col != -1) ? t.col + 1 : this.sfCalendar.find('tbody tr td.e-active').index() + 1;
                t.row = (t.row != -1) ? t.row + 1 : this.sfCalendar.find('tbody tr td.e-active').parent().index() + 1;

                var tableClass = this.sfCalendar.find('table')[0].className, next, rowLength = 3, colLength = 4;
                switch (tableClass) {
                    case "e-dp-viewallyears":
                        next = this._changeRowCol(t, e.keyCode, rowLength, colLength, "yearall", e.ctrlKey);
                        if (!e.ctrlKey) this._hoverAllYear = this.sfCalendar.find('tbody.e-datepicker-allyears tr td').index(next);
                        break;
                    case "e-dp-viewyears":
                        next = this._changeRowCol(t, e.keyCode, rowLength, colLength, "year", e.ctrlKey);
                        if (!e.ctrlKey) this._hoverYear = this.sfCalendar.find('tbody.e-datepicker-years tr td').index(next);
                        break;
                    case "e-dp-viewmonths":
                        next = this._changeRowCol(t, e.keyCode, rowLength, colLength, "month", e.ctrlKey);
                        if (!e.ctrlKey) this._hoverMonth = this.sfCalendar.find('tbody.e-datepicker-months tr td').index(next);
                        break;
                    case "e-dp-viewdays":
                        rowLength = this.sfCalendar.find('tbody.e-datepicker-days tr').length, colLength = 7;
                        next = this._changeRowCol(t, e.keyCode, rowLength, colLength, "day", e.ctrlKey);
                        if (!e.ctrlKey) this._hoverDate = this._getDateObj(next).getDate() - 1;
                        break;
                }
                if (!e.ctrlKey) {
                    this.sfCalendar.find('table td').removeClass("e-state-hover");
                    next.addClass("e-state-hover");
                }
            }
            else if (!this.model.displayInline && (e.keyCode == 27 || e.keyCode == 9)) { this.hide(); e.stopPropagation(); }
            else if (e.altKey && e.keyCode == 40) { this.show(); return false; }
        },
        _changeRowCol: function (t, key, rows, cols, target, ctrlKey) {
            var eleClass, cls = { parent: null, child: null };
            switch (target) {
                case "day": eleClass = "tbody.e-datepicker-days tr td.current-month";
                    cls.parent = ".e-datepicker-days", cls.child = ".current-month";
                    break;
                case "month": eleClass = "tbody.e-datepicker-months tr td.e-current-month";
                    cls.parent = ".e-datepicker-months", cls.child = ".e-current-month";
                    break;
                case "year": eleClass = "tbody.e-datepicker-years tr td.e-current-year";
                    cls.parent = ".e-datepicker-years", cls.child = ".e-current-year";
                    break;
                case "yearall": eleClass = "tbody.e-datepicker-allyears tr td.e-current-allyear";
                    cls.parent = ".e-datepicker-allyears", cls.child = ".e-current-allyear";
                    break;
            }
            if (t.row <= 0 && t.col <= 0)
                return this.sfCalendar.find(eleClass + ':first');
            var cell, proxy = this;
            switch (key) {
                case 36:         // Home Key
                    return this.sfCalendar.find(eleClass + ':first');
                    break;
                case 35:         // End Key
                    return this.sfCalendar.find(eleClass + ':last');
                    break;
                case 38:         // Up Key
                    if (ctrlKey) {
                        this._forwardNavHandler();
                    }
                    else if (t.row > 1) {
                        t.row -= 1;
                    }
                    else {
                        this._processNextPrevDate(true);
                        cell = this.sfCalendar.find(eleClass + ':nth-child(' + t.col + '):last');
                        return cell;
                    }
                    cell = this._getCell(t, cls);
                    if (cell.length <= 0) {
                        this._processNextPrevDate(true);
                        cell = this.sfCalendar.find(eleClass + ':nth-child(' + t.col + '):last');
                    }
                    return cell;
                    break;
                case 37:        // Left Key
                    if (ctrlKey) {
                        this._processNextPrevDate(true);
                        return this.sfCalendar.find('tbody tr td.e-state-hover');
                    }
                    else if (t.col > 1)
                        t.col -= 1;
                    else if (t.row > 1) {
                        t = { row: t.row - 1, col: cols }
                    }
                    else {
                        this._processNextPrevDate(true);
                        cell = this.sfCalendar.find(eleClass + ':last');
                        return cell;
                    }
                    cell = this._getCell(t, cls);
                    if (cell.length <= 0) {
                        this._processNextPrevDate(true);
                        cell = this.sfCalendar.find(eleClass + ':last');
                    }
                    return cell;
                    break;
                case 39:         // Right Key
                    if (ctrlKey) {
                        this._processNextPrevDate(false);
                        return this.sfCalendar.find('tbody tr td.e-state-hover');
                    }
                    else if (t.col < cols)
                        t.col += 1;
                    else if (t.row < rows) {
                        t = { row: t.row + 1, col: 1 }
                    }
                    else {
                        this._processNextPrevDate(false);
                        cell = this.sfCalendar.find(eleClass + ':first');
                        return cell;
                    }
                    cell = this._getCell(t, cls);
                    if (cell.length <= 0) {
                        this._processNextPrevDate(false);
                        cell = this.sfCalendar.find(eleClass + ':first');
                    }
                    return cell;
                    break;
                case 40:        // Down Key
                    if (!ctrlKey) {
                        if (t.row < rows) {
                            t.row += 1;
                        }
                        else {
                            this._processNextPrevDate(false);
                            cell = this.sfCalendar.find(eleClass + ':nth-child(' + t.col + '):first');
                            return cell;
                        }
                        cell = this._getCell(t, cls);
                        if (cell.length <= 0) {
                            this._processNextPrevDate(false);
                            cell = this.sfCalendar.find(eleClass + ':nth-child(' + t.col + '):first');
                        }
                        return cell;
                        break;
                    }
                case 13:    // Enter Key
                    var tclassName = $("table", this.sfCalendar).get(0).className;
                    var ele = this._getCell(t, cls);
                    this._backwardNavHandler(ele);
                    if (tclassName == "e-dp-viewdays")
                        this._onSetCancelDateHandler({ type: null, target: ele });
                    break;
            }
            return this._getCell(t, cls);
        },
        _getCell: function (t, cls) {
            return this.sfCalendar.find('tbody' + cls.parent + ' tr:nth-child(' + t.row + ') td' + cls.child + ':nth-child(' + t.col + ')');
        },
        _getDateObj: function (element) {
            return new Date(element.attr("date"));
        },
        _touchCalendar: function (e) {
            var tableClass = this.sfCalendar.find('table')[0].className;
            switch (e.type) {
                case "pinchin":
                    if (tableClass != "e-dp-viewdays")
                        this._keyboardNavigation({ keyCode: 13 });
                    break;
                case "pinchout":
                    if (tableClass != "e-dp-viewallyears")
                        this._forwardNavHandler();
                    break;
                case "swipeleft":
                    this._processNextPrevDate(false);
                    break;
                case "swiperight":
                    this._processNextPrevDate(true);
                    break;
            }
        },

        /*---------------------client side methods----------------------------------------------------------*/

        /**
        * Opens the datepicker popup  
		* @return jQuery
		* @example 
		* &lt;input type="text" id="datepicker" /&gt;
		* &lt;script&gt;
		* $("#datepicker").ejDatePicker();
		* // Create DatePicker instance
		* var dateObj = $("#datepicker").data("ejDatePicker");
		* dateObj.show(); // shows the datepicker popup
		* &lt;/script&gt;
		* @example 
	    * &lt;input type="text" id="datepicker" /&gt;
	    * &lt;script&gt;
		* $("#datepicker").ejDatePicker();
	    * // shows the datepicker popup
        * $("#datepicker").ejDatePicker("show");
	    * &lt;/script&gt;
		*@memberof ejDatePicker
		* @instance
        */
        show: function () {
            if (this._isOpen) return false;
            var proxy = this;
            if (!this.model.enabled) return;
            if (!this.model.displayInline) this._setDatePickerPosition();
            this.sfCalendar.attr({ 'aria-hidden': 'false' })
            this.sfCalendar.slideDown(this.model.enableAnimation?200:0, "easeOutQuad", function () {
                proxy._isOpen = true;
                if (!proxy.model.displayInline)
                    $(document).bind("mousedown", $.proxy(proxy._onDocumentClick, proxy));
            });
            if (this._textboxVal() !== "") this._updateInputVal();

            this._trigger("open", { prevDate: this._prevDate, value: this._formatter(this.model.value, this.model.dateFormat) });

        },

        /**
        * Hides the datepicker popup, if in opended state.
		* @return jQuery
		* @example 
		* &lt;input type="text" id="datepicker" /&gt;
		* &lt;script&gt;
		* $("#datepicker").ejDatePicker();
		* // Create DatePicker instance
		* var dateObj = $("#datepicker").data("ejDatePicker");
		* dateObj.hide(); // hides the datepicker popup
		* &lt;/script&gt;
		* @example 
	    * &lt;input type="text" id="datepicker" /&gt;
	    * &lt;script&gt;
		* $("#datepicker").ejDatePicker();
	    * // hides the datepicker popup
        * $("#datepicker").ejDatePicker("hide");
	    * &lt;/script&gt;
		*@memberof ejDatePicker
		* @instance
        */
        hide: function () {
            if (!this._isOpen) return false;
            var proxy = this;
            this.sfCalendar.attr({ 'aria-hidden': 'true' })
            this.sfCalendar.slideUp(this.model.enableAnimation?100:0, "easeOutQuad", function () {
                proxy._isOpen = false;
                $(document).unbind("mousedown", $.proxy(proxy._onDocumentClick, proxy));
                proxy._setWaterMark();
            });
            if (this._textboxVal() != "" && !this.element.hasClass("e-watermark")) this._validateInputVal();

            this._trigger("close", { prevDate: this._prevDate, value: this._formatter(this.model.value, this.model.dateFormat) });
        },

        /**
        * Enables the datepicker control
		* @return jQuery
		* @example 
		* &lt;input type="text" id="datepicker" /&gt;
		* &lt;script&gt;
		* $("#datepicker").ejDatePicker();
		* // Create DatePicker instance
		* var dateObj = $("#datepicker").data("ejDatePicker");
		* dateObj.enable(); // enables the datepicker
		* &lt;/script&gt;
		* @example 
	    * &lt;input type="text" id="datepicker" /&gt;
	    * &lt;script&gt;
		* $("#datepicker").ejDatePicker();
	    * // enables the datepicker
        * $("#datepicker").ejDatePicker("enable");
	    * &lt;/script&gt;
		*@memberof ejDatePicker
		* @instance
        */
        enable: function () {
            this.model.enabled = true;
            this.element.removeClass('e-disable').attr({ "aria-disabled": false });
            if (this.dateIcon) this.dateIcon.removeClass('e-disable').attr({ "aria-disabled": false });
            this.element.removeAttr("disabled");
            this.sfCalendar.removeClass('e-disable').attr({ "aria-disabled": false });
        },

        /**
        * Disables the datepicker control
		* @return jQuery
		* @example 
		* &lt;input type="text" id="datepicker" /&gt;
		* &lt;script&gt;
		* $("#datepicker").ejDatePicker();
		* // Create DatePicker instance
		* var dateObj = $("#datepicker").data("ejDatePicker");
		* dateObj.disable(); // disables the datepicker
		* &lt;/script&gt;
		* @example 
	    * &lt;input type="text" id="datepicker" /&gt;
	    * &lt;script&gt;
		* $("#datepicker").ejDatePicker();
	    * // disables the datepicker
        * $("#datepicker").ejDatePicker("disable");
	    * &lt;/script&gt;
		*@memberof ejDatePicker
		* @instance
        */
        disable: function () {
            this.model.enabled = false;
            this.element.addClass('e-disable').attr({ "aria-disabled": true });
            if (this.dateIcon) this.dateIcon.addClass('e-disable').attr({ "aria-disabled": true });
            this.element.attr("disabled", "disabled");
            this.sfCalendar.addClass('e-disable').attr({ "aria-disabled": true });
            if (this._isOpen) {
                if (this.element.is(':input')) this.element.blur();
                if (!this.model.displayInline) this.hide();
            }
        },

        /**
        * Returns the current date value in the datepicker control
		* @return Date value
		* @example 
		* &lt;input type="text" id="datepicker" /&gt;
		* &lt;script&gt;
		* $("#datepicker").ejDatePicker();
		* // Create DatePicker instance
		* var dateObj = $("#datepicker").data("ejDatePicker");
		* dateObj.getValue(); // returns the date value
		* &lt;/script&gt;
		* @example 
	    * &lt;input type="text" id="datepicker" /&gt;
	    * &lt;script&gt;
	    * // returns the date value
		* $("#datepicker").ejDatePicker();
        * $("#datepicker").ejDatePicker("getValue");
	    * &lt;/script&gt;
		*@memberof ejDatePicker
		* @instance
        */
        getValue: function () { return this._formatter(this.model.value, this.model.dateFormat); },

        /*----------------------------------------region - Event Binding and Handlers---------------------------------------- */

        _wireEvents: function () {
            this._on($('.e-text', this.sfCalendar), "click", $.proxy(this._forwardNavHandler, this));
            this._on($('.e-next', this.sfCalendar), "click", $.proxy(this._previousNextHandler, this));
            this._on($('.e-prev', this.sfCalendar), "click", $.proxy(this._previousNextHandler, this));

            if (this.element.is(":input")) {
                this._on(this.element, "blur", this._onFocusOut);
                this._on(this.element, "focus", this._onFocusIn);
                this._on(this.element, "keyup", this._keyUp);
            }

            if (this.model.showFooter) {
                this._on($('.e-footer', this.sfCalendar), "click", this._setCurrDate);
            }

            this._on(this.sfCalendar, "pinchin pinchout swipeleft swiperight", $.proxy(this._touchCalendar, this));

        },
        _bindDateButton: function () {
            this._on($(this.dateIcon, this.wrapper), "mousedown", $.proxy(this._showDatePopUp, this));
            this.element.unbind("mousedown", $.proxy(this._showDatePopUp, this));
        },
        _bindInputEvent: function () {
            this._off($(this.dateIcon, this.wrapper), "mousedown", $.proxy(this._showDatePopUp, this));
            if (this.element.is(":input") && !this.model.displayInline) {
                this.element.bind("mousedown", $.proxy(this._showDatePopUp, this));
            }
        },

        _onFocusOut: function (e) {
            var calenderDate = this._formatter(this._calendarDate, this.model.dateFormat);
            var currDate = this._formatter(this._currentDay(), this.model.dateFormat);
            this._isFocused = false;
            var val = this._validateValue();
            if (val != null && !this.model.enableStrictMode) {
                this._validateMinMaxDate();
                var _currentVal = this._textboxVal();
                if (this._preValue != _currentVal) {
                    this._triggerChangeEvent(e);
                }
            }
            else if (val == null && !this.model.enableStrictMode) {
                if (this._preTxtValue == null || this.element.val() == "")
                    this.element.addClass("e-watermark").val(this.model.watermarkText);
                else
                    this.element.val(calenderDate);
                this._triggerChangeEvent(e);
            }
            else
                this.isValidState = false;
            this.wrapper.removeClass("e-focus");
            if ((!this._isOpen || this.model.displayInline) && !this._setWaterMark() && this._preValue != this._textboxVal()) this._updateInputVal();
            if (this._textboxVal() != "" && (!this._isOpen || this.model.displayInline)) this._validateInputVal();
            this.element.unbind("keydown", $.proxy(this._keyboardNavigation, this));

            this._trigger("focusOut");
            this._checkErrorClass();
        },
        _onFocusIn: function (e) {
            e.preventDefault();
            this._isFocused = true;
            this.wrapper.removeClass('e-error').attr('aria-invalid', "false");
            this.isValidState = true;
            this.wrapper.addClass("e-focus");
            if (this.model.readOnly)
                return;
            this._removeWatermark();
            this.element.bind("keydown", $.proxy(this._keyboardNavigation, this));

            this._trigger("focusIn");
        },
        _removeWatermark: function () {
            if (this.element.val() == this.model.watermarkText) {
                this.element.val("");
                this.element.removeClass('e-watermark');
            }
        },
        _keyUp: function (e) {
            var _currentVal = this._textboxVal();
            this._prevDate = this._formatter(this.model.value, this.model.dateFormat);
            if (this._preValue != _currentVal && this.model.enableStrictMode)
                this._triggerChangeEvent();
        },
        _textboxVal: function (e) {
            return this.element.hasClass("e-watermark") ? "" : this.element.val();
        },
        _showhidePopup: function () {
            if (!this.model.enabled) return false;
            if (this._isOpen) {
                if (!this._isFocused && this.element.is(':input')) this.element.focus();
                if (!this._cancelValue) this.hide();
            }
            else {                              // Open
                if (!this._isFocused && this.element.is(':input')) this.element.focus();
                this.show();
            }
        },

        _triggerChangeEvent: function (e) {
            var _currentVal = this._textboxVal();
            this.model.value = this._parseDate(_currentVal);
            var data = { prevDate: this._prevDate, value: _currentVal };
            if (this._preValue != _currentVal) {
                this._preValue = _currentVal;
                this._trigger("change", data);
            }
            this._hiddenInput.val(this._textboxVal());
        },

        _triggerSelectEvent: function (e) {
            var val = this._textboxVal();
            if (this._parseDate(val)) {
                var data = { prevDate: this._prevDate, value: val, isSpecialDay: this._isSpecialDates(this.model.value) };
                if (this._prevDate != val) {
                    if (this._parseDate(data.value) && (this.model.value >= this.model.minDate && this.model.value <= this.model.maxDate)) {
                        this._cancelValue = this._trigger("select", data);
                    }
                }
            }
        },

        _onDocumentClick: function (e) {
            if (this.model) {
                if (!$(e.target).is(this.popup) && !$(e.target).parents(".e-popup").is(this.popup) &&
                    !$(e.target).is(this.wrapper) && !$(e.target).parents(".e-datewidget").is(this.wrapper)) {
                        this._showhidePopup();
                }
                else if ($(e.target).is(this.popup) || $(e.target).parents(".e-popup").is(this.popup)) {
                    e.preventDefault();
                }
            }
        },
        _showDatePopUp: function (e) {
            if (this.model.readOnly) return;
            e.preventDefault();
            if (!this.model.enabled && this.model.displayInline) return false;
            this._showhidePopup();
        },
        _layoutChanged: function (e) {
            // this event internally used to observe the layout change in "DateTimePicker" control
            if (this._getInternalEvents) this._trigger("layoutChange");
        },
        _setCurrDate: function (e) {
            if (this.model.readOnly || !this.model.enabled) return false;
            if (e) e.preventDefault();
            var proxy = this;
            this._prevDate = this._formatter(this.model.value, this.model.dateFormat);
            this._dateValue = this._zeroTime(new Date());
            this.model.value = this._calendarDate = this._dateValue;
            this._setDateValue(this.model.value);
            this._triggerSelectEvent(e);
            this._triggerChangeEvent(e);
            this._refreshDatepicker();
            this._changeDayClass();
            this._startLevel(this.model.startLevel);
            this._onSetCancelDateHandler(e);
            this._layoutChanged();
        },
        _changeDayClass: function () {
            var className = this.popup.children("table")[0].className;
            if (className != "e-dp-viewdays") {
                this.popup.children("table").removeClass(className).addClass("e-dp-viewdays");
            }
        },
        //handler for setting dates to input field
        _onSetCancelDateHandler: function (e) {
            if (this.model.readOnly || !this.model.enabled) return false;
            if (e && $(e.target).hasClass("e-disable")) return false;
            if (e && e.type) e.preventDefault();
            this.model.value = this._parseDate(this._textboxVal());
            //sets prev date before value gets change to current selected date
            this._prevDate = this._formatter(this.model.value, this.model.dateFormat);
            this._setDateValue(this._dateValue);
            this._triggerSelectEvent(e);
            this._triggerChangeEvent(e);
            if (this.element.is(':input') && !this.model.displayInline) {
                this._showhidePopup();
            }
            if (e && $(e.target).hasClass("other-month"))
                this._refreshDatepicker();
        },
        _closeCalendar: function (ele) {
            if (!ele || ele == this.element) {
                this.sfCalendar.empty().remove();
            }
        },
        //Error class for input value validation
        _checkErrorClass: function () {
            if (this.wrapper) {
                if (this.isValidState) this.wrapper.removeClass("e-error").attr('aria-invalid', "false");
                else this.wrapper.addClass("e-error").attr('aria-invalid', "true");
            }
        }
    });

    /**
	 * Enum for DatePicker dayHeaderFormat	 
	 * @enum {String}
	 * @global 
	 */
    ej.DatePicker.Header = {
        /**  Shows the day header format in short like Sun, Mon, Tue … */
        ShowHeaderNone: "showheadernone",
        /**  Shows the day header format in min like Su, Mo, Tu … */
        ShowHeaderShort: "showheadershort",
        /**  Shows the day header format in long like Sunday, Monday, Tuesday … */
        ShowHeaderMin: "showheadermin",
        /**  Removes the day header */
        ShowHeaderLong: "showheaderlong"
    };
    /**
    * Enum for DatePicker highlightSection	 
    * @enum {String}
    * @global 
    */
    ej.DatePicker.HighlightSection = {
        /**  Highlight the Current Month. */
        Month: "month",
        /**  Highlight the Current Week. */
        Week: "week",
        /**  Highlight the Current WorkDays. */
        WorkDays: "workdays",
        /** Don't Highlight Anything. */
        None: "none"
    };

    /**
	 * Enum for DatePicker startLevel and depthLevel	 
	 * @enum {String}
	 * @global 
	 */
    ej.DatePicker.Level = {
        /**  Starts from month level view. */
        Month: "month",
        /**  Starts from year level view. */
        Year: "year",
        /**  Starts from year decade level view. */
        Decade: "decade",
        /**  Starts from century level view.  */
        Century: "century"
    };
   

})(jQuery, Syncfusion);;
/**
* @fileOverview Plugin to craete a Timepicker with the Html input element
* @copyright Copyright Syncfusion Inc. 2001 - 2014. All rights reserved.
*  Use of this code is subject to the terms of our license.
*  A copy of the current license can be obtained at any time by e-mailing
*  licensing@syncfusion.com. Any infringement will be prosecuted under
*  applicable laws. 
* @version 12.1 
* @author <a href="mailto:licensing@syncfusion.com">Syncfusion Inc</a>
*/

(function ($, ej, undefined) {
    /**
    * @namespace ej
	* @classdesc Time selection with the input field.
	* @class ejTimePicker
	* @requires jQuery
    * @requires jquery.easing.1.3.js
    * @requires jquery.globalize.js
    * @requires globalize.cultures.min.js
	* @requires ej.core.js
	* @requires ej.timepicker.js
	* @requires ej.scroller.js
	
	* @example 
	* &lt;input type="text" id="timepicker" /&gt;<br/>
	* &lt;script&gt;
	* // Create TimePicker
    * $("#timepicker").ejTimePicker();
	* &lt;/script&gt;
	*/

    ej.widget("ejTimePicker", "ej.TimePicker", {
        // widget element will be automatically set in this
        element: null,
        // user defined model will be automatically set in this
        model: null,
        validTags: ["input"],
        _rootCSS: "e-timepicker",
        _setFirst: false,
        type: "editor",

        // default model
        defaults: {
            /**		
			* Specify the CSS class to timepicker to achieve custom theme.
			* @default ""
			* @type {String}
			* @example 
			* &lt;input type="text" id="timepicker" /&gt;<br/>
			* &lt;script&gt;
			* // Set the CSS class during initialization. 			
			* 	$("#timepicker").ejTimePicker({  cssClass : "gradient-lime" });
			* &lt;/script&gt; <br>
			 * @memberof ejTimePicker
			* @instance
			*/
            cssClass: "",

            /**		
			* Defines the time format displayed in the TimePicker.
			* @default "h:mm tt"
			* @type {String}
			* @example 
			* &lt;input type="text" id="timepicker" /&gt;<br/>
			* &lt;script&gt;
			* // Set the timeFormat during initialization. 			
			* 	$("#timepicker").ejTimePicker({  timeFormat : "h:mm:ss tt" });
			* &lt;/script&gt; <br>
			 * @memberof ejTimePicker
			* @instance
			*/
            timeFormat: "",

            /**		
			* Sets a specified time value on the TimePicker.
			* @default null
			* @type {String | DateObject}
			* @example 
			* &lt;input type="text" id="timepicker" /&gt;<br/>
			* &lt;script&gt;
			* // Set the time value during initialization. 			
			* 	$("#timepicker").ejTimePicker({  value : "5:10 PM" });
			* &lt;/script&gt; <br>
			 * @memberof ejTimePicker
			* @instance
			*/
            value: null,
			enableAnimation: true,
            /**		
			* Defines the localization locale for TimePicker.
			* @default "en-US"
			* @type {String}
			* @example 
			* &lt;input type="text" id="timepicker" /&gt;<br/>
			* &lt;script&gt;
			* // Set the locale value during initialization. 			
			* 	$("#timepicker").ejTimePicker({  locale : "en-US" });
			* &lt;/script&gt; <br>
			 * @memberof ejTimePicker
			* @instance
			*/
            locale: "en-US",

            /**		
			* Indicates that the timepicker value can only be read.
			* @default true
			* @type {Boolean}
			* @example 
			* &lt;input type="text" id="timepicker" /&gt;<br/>
			* &lt;script&gt;
			* // Set the readOnly value during initialization. 			
			* 	$("#timepicker").ejTimePicker({  readOnly : false });
			* &lt;/script&gt; <br>
			 * @memberof ejTimePicker
			* @instance
			*/
            readOnly: false,

            /**		
			* Shows or hides the arrow button from the TimePicker textbox.
			* @default true
			* @type {Boolean}
			* @example 
			* &lt;input type="text" id="timepicker" /&gt;<br/>
			* &lt;script&gt;
			* // Set the showPopupButton property during initialization. 			
			* 	$("#timepicker").ejTimePicker({  showPopupButton : false });
			* &lt;/script&gt; <br>
			 * @memberof ejTimePicker
			* @instance
			*/
            showPopupButton: true,
			
			/**		
			* When enableStrictMode true it allows the value outside of the range also, otherwise it internally changed to the correct value.
			* @default false
			* @type {Boolean}
			* @example 
			* &lt;input type="text" id="timepicker" /&gt;
	        * &lt;script&gt;
			* //To set enableStrictMode API during initialization  
			* 	$("#timepicker").ejTimePicker({  enableStrictMode: true });
		    * &lt;/script&gt; 
			 * @memberof ejTimePicker
			* @instance
			*/
            enableStrictMode: false,

            /**		
			* Sets the time interval between the two adjacent time values in the popup.
			* @default 30
			* @type {Number}
			* @example 
			* &lt;input type="text" id="timepicker" /&gt;<br/>
			* &lt;script&gt;
			* // Set the interval value during initialization. 			
			* 	$("#timepicker").ejTimePicker({  interval : 60 });
			* &lt;/script&gt; <br>
			 * @memberof ejTimePicker
			* @instance
			*/
            interval: 30,

            /**		
			* Sets the step value for increment an hour value through arrow keys or mouse scroll.
			* @default 1
			* @type {Number}
			* @example 
			* &lt;input type="text" id="timepicker" /&gt;<br/>
			* &lt;script&gt;
			* // Set the hourInterval value during initialization. 			
			* 	$("#timepicker").ejTimePicker({  hourInterval : 2 });
			* &lt;/script&gt; <br>
			 * @memberof ejTimePicker
			* @instance
			*/
            hourInterval: 1,

            /**		
			* Sets the step value for increment the minute value through arrow keys or mouse scroll.
			* @default 1
			* @type {Number}
			* @example 
			* &lt;input type="text" id="timepicker" /&gt;<br/>
			* &lt;script&gt;
			* // Set the minute interval value during initialization. 			
			* 	$("#timepicker").ejTimePicker({  minutesInterval : 5 });
			* &lt;/script&gt; <br>
			 * @memberof ejTimePicker
			* @instance
			*/
            minutesInterval: 1,

            /**		
			* Sets the step value for increment the seconds value through arrow keys or mouse scroll.
			* @default 1
			* @type {Number}
			* @example 
			* &lt;input type="text" id="timepicker" /&gt;<br/>
			* &lt;script&gt;
			* // Set the seconds interval value during initialization. 			
			* 	$("#timepicker").ejTimePicker({ timeFormat : "h:mm:ss tt",secondsInterval : 5 });
			* &lt;/script&gt; <br>
			 * @memberof ejTimePicker
			* @instance
			*/
            secondsInterval: 1,

            /**		
			* Defines the height of the TimePicker textbox.
			* @default ""
			* @type {String | Number}
			* @example 
			* &lt;input type="text" id="timepicker" /&gt;<br/>
			* &lt;script&gt;
			* // Set the height value during initialization. 			
			* 	$("#timepicker").ejTimePicker({  height : "35" });
			* &lt;/script&gt; <br>
			 * @memberof ejTimePicker
			* @instance
			*/
            height: "",

            /**		
			* Defines the width of the TimePicker textbox.
			* @default ""
			* @type {String | Number}
			* @example 
			* &lt;input type="text" id="timepicker" /&gt;<br/>
			* &lt;script&gt;
			* // Set the width value during initialization. 			
			* 	$("#timepicker").ejTimePicker({  width : "120" });
			* &lt;/script&gt; <br>
			 * @memberof ejTimePicker
			* @instance
			*/
            width: "",

            /**		
			* Sets the minimum time value to the TimePicker.
			* @default "12:00:00 AM"
			* @type {String}
			* @example 
			* &lt;input type="text" id="timepicker" /&gt;<br/>
			* &lt;script&gt;
			* // Set the minTime value during initialization. 			
			* 	$("#timepicker").ejTimePicker({  minTime : "8:00 AM" });
			* &lt;/script&gt; <br>
			 * @memberof ejTimePicker
			* @instance
			*/
            minTime: "12:00:00 AM",

            /**		
			* Sets the maximum time value to the TimePicker.
			* @default "11:59:59 PM"
			* @type {String}
			* @example 
			* &lt;input type="text" id="timepicker" /&gt;<br/>
			* &lt;script&gt;
			* // Set the maxTime value during initialization. 			
			* 	$("#timepicker").ejTimePicker({  maxTime : "5:00 PM" });
			* &lt;/script&gt; <br>
			 * @memberof ejTimePicker
			* @instance
			*/
            maxTime: "11:59:59 PM",

            /**		
			* Changes the sharped edges into rounded corner for the TimePicker textbox and popup.
			* @default false
			* @type {Boolean}
			* @example 
			* &lt;input type="text" id="timepicker" /&gt;<br/>
			* &lt;script&gt;
			* // Set the showRoundedCorner value during initialization. 			
			* 	$("#timepicker").ejTimePicker({  showRoundedCorner : true });
			* &lt;/script&gt; <br>
			 * @memberof ejTimePicker
			* @instance
			*/
            showRoundedCorner: false,

            /**		
			* Sets the TimePicker direction as right to left alignment.
			* @default false
			* @type {Boolean}
			* @example 
			* &lt;input type="text" id="timepicker" /&gt;<br/>
			* &lt;script&gt;
			* // Set the enableRTL value during initialization. 			
			* 	$("#timepicker").ejTimePicker({  enableRTL : true });
			* &lt;/script&gt; <br>
			 * @memberof ejTimePicker
			* @instance
			*/
            enableRTL: false,

            /**		
			* Defines the height of the TimePicker popup.
			* @default "191px"
			* @type {String | Number}
			* @example 
			* &lt;input type="text" id="timepicker" /&gt;<br/>
			* &lt;script&gt;
			* // Set the popupHeight value during initialization. 			
			* 	$("#timepicker").ejTimePicker({  popupHeight : "250px" });
			* &lt;/script&gt; <br>
			 * @memberof ejTimePicker
			* @instance
			*/
            popupHeight: "191px",

            /**		
			* Defines the width of the TimePicker popup.
			* @default "auto"
			* @type {String | Number}
			* @example 
			* &lt;input type="text" id="timepicker" /&gt;<br/>
			* &lt;script&gt;
			* // Set the popupWidth value during initialization. 			
			* 	$("#timepicker").ejTimePicker({  popupWidth : "150px" });
			* &lt;/script&gt; <br>
			 * @memberof ejTimePicker
			* @instance
			*/
            popupWidth: "auto",

            /**		
			* When this property is set to false, it disables the timepicker control.
			* @default true
			* @type {Boolean}
			* @example 
			* &lt;input type="text" id="timepicker" /&gt;<br/>
			* &lt;script&gt;
			* // Set the enabled value during initialization. 			
			* 	$("#timepicker").ejTimePicker({  enabled : false });
			* &lt;/script&gt; <br>
			 * @memberof ejTimePicker
			* @instance
			*/
            enabled: true,

            /**		
			* Enables or disables the state maintenance of TimePicker.
			* @default false
			* @type {Boolean}
			* @example 
			* &lt;input type="text" id="timepicker" /&gt;<br/>
			* &lt;script&gt;
			* // Set the enablePersistence value during initialization. 			
			* 	$("#timepicker").ejTimePicker({  enablePersistence : true });
			* &lt;/script&gt; <br>
			* @memberof ejTimePicker
			* @instance
			*/
            enablePersistence: false,

            /**     
			 * Fires when the timepicker control gets focus.
			 * @event
			 * @name ejTimePicker#focusIn 	
			 * @param {Object} argument Event parameters from timepicker     
			 * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
			 * @param {object}  argument.model returns the timepicker model
			 * @param {string}  argument.type returns the name of the event
			 * @param {string}  argument.value returns the current time value
			 * @example 
			 * &lt;input type="text" id="timepicker" /&gt;<br/>
			 * &lt;script&gt;
			 * //focusIn event for timepicker
             * $("#timepicker").ejTimePicker({
             *    focusIn: function (args) {}
             * });      
			 * &lt;/script&gt; <br>
			 * @memberof ejTimePicker
			 * @instance
			 */
            focusIn: null,

            /**     
			 * Fires when the timepicker control get lost focus.
			 * @event
			 * @name ejTimePicker#focusOut 	
			 * @param {Object} argument Event parameters from timepicker     
			 * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
			 * @param {object}  argument.model returns the timepicker model
			 * @param {string}  argument.type returns the name of the event
			 * @param {string}  argument.value returns the current time value
			 * @example 
			 * &lt;input type="text" id="timepicker" /&gt;<br/>
			 * &lt;script&gt;
			 * //focusOut event for timepicker
             * $("#timepicker").ejTimePicker({
             *    focusOut: function (args) {}
             * }); 
			 * &lt;/script&gt; <br>			 
			 * @memberof ejTimePicker
			 * @instance
			 */
            focusOut: null,
			
		    /**     
            * Fires when the time value changed in the TimePicker.
            * @event
            * @name ejTimePicker#beforeChange 	
            * @param {Object} argument Event parameters from timepicker     
            * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
            * @param {object}  argument.model returns the timepicker model
            * @param {string}  argument.type returns the name of the event
            * @param {string}  argument.value returns the modified time value
            * @example 
			* &lt;input type="text" id="timepicker" /&gt;<br/>
			* &lt;script&gt;
            * //change event for timepicker
            * $("#timepicker").ejTimePicker({
            *    beforeChange: function (args) {}
            * });  
			* &lt;/script&gt; <br>			
            * @memberof ejTimePicker
            * @instance
            */
            beforeChange: null,

            /**     
            * Fires when the time value changed in the TimePicker.
            * @event
            * @name ejTimePicker#change 	
            * @param {Object} argument Event parameters from timepicker     
            * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
            * @param {object}  argument.model returns the timepicker model
            * @param {string}  argument.type returns the name of the event
            * @param {string}  argument.value returns the modified time value
            * @example 
			* &lt;input type="text" id="timepicker" /&gt;<br/>
			* &lt;script&gt;
            * //change event for timepicker
            * $("#timepicker").ejTimePicker({
            *    change: function (args) {}
            * });  
			* &lt;/script&gt; <br>			
            * @memberof ejTimePicker
            * @instance
            */
            change: null,

            /**     
            * Fires when the value is selected from the timepicker dropdownlist.
            * @event
            * @name ejTimePicker#select 	
            * @param {Object} argument Event parameters from timepicker     
            * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
            * @param {object}  argument.model returns the timepicker model
            * @param {string}  argument.type returns the name of the event
            * @param {string}  argument.value returns the selected time value
            * @example 
			* &lt;input type="text" id="timepicker" /&gt;<br/>
			* &lt;script&gt;
            * //select event for timepicker
            * $("#timepicker").ejTimePicker({
            *    select: function (args) {}
            * });   
			* &lt;/script&gt; <br>			
            * @memberof ejTimePicker
            * @instance
            */
            select: null,
			/**     
          * Fires when create TimePicker successfully.
          * @event
          * @name ejTimePicker#create 	
          * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
          * @param {object}  argument.model returns the TimePicker model
          * @param {string}  argument.type returns the name of the event
          * @example 
			* &lt;input type="text" id="timepicker" /&gt;<br/>
			* &lt;script&gt;
            * //create event for timepicker
            * $("#timepicker").ejTimePicker({
            *    create: function (args) {}
            * });   
			* &lt;/script&gt; <br>					
          * @memberof ejTimePicker
          * @instance
          */
            create: null,
			/**     
          * Fires when the TimePicker is destroyed successfully.
          * @event
          * @name ejTimePicker#destroy 	
          * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
          * @param {object}  argument.model returns the TimePicker model
          * @param {string}  argument.type returns the name of the event
          * @example 
			* &lt;input type="text" id="timepicker" /&gt;<br/>
			* &lt;script&gt;
            * //destroy event for timepicker
            * $("#timepicker").ejTimePicker({
            *    destroy: function (args) {}
            * });   
			* &lt;/script&gt; <br>					
          * @memberof ejTimePicker
          * @instance
          */
            destroy: null,
            /**     
          * Fires when the TimePicker popup before opened .
          * @event
          * @name ejTimePicker#beforeOpen 	
          * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
          * @param {object}  argument.model returns the TimePicker model
          * @param {string}  argument.type returns the name of the event
          * @param {string}  argument.model returns the previous value
          * @param {string}  argument.value returns the time value
          * @example 
			* &lt;input type="text" id="timepicker" /&gt;<br/>
			* &lt;script&gt;
            * //beforeOpen event for timepicker
            * $("#timepicker").ejTimePicker({
            *    beforeOpen: function (args) {}
            * });   
			* &lt;/script&gt; <br>					
          * @memberof ejTimePicker
          * @instance
          */
            beforeOpen: null,
            /**     
          * Fires when the TimePicker popup opened.
          * @event
          * @name ejTimePicker#open 	
          * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
          * @param {object}  argument.model returns the TimePicker model
          * @param {string}  argument.type returns the name of the event
          * @param {string}  argument.model returns the previous value
          * @param {string}  argument.value returns the time value
          * @example 
			* &lt;input type="text" id="timepicker" /&gt;<br/>
			* &lt;script&gt;
            * //open event for timepicker
            * $("#timepicker").ejTimePicker({
            *    open: function (args) {}
            * });   
			* &lt;/script&gt; <br>					
          * @memberof ejTimePicker
          * @instance
          */
            open: null,
            /**     
          * Fires when the TimePicker popup closed.
          * @event
          * @name ejTimePicker#close 	
          * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
          * @param {object}  argument.model returns the TimePicker model
          * @param {string}  argument.type returns the name of the event
          * @param {string}  argument.model returns the previous value
          * @param {string}  argument.value returns the time value
          * @example 
			* &lt;input type="text" id="timepicker" /&gt;<br/>
			* &lt;script&gt;
            * //close event for timepicker
            * $("#timepicker").ejTimePicker({
            *    close: function (args) {}
            * });   
			* &lt;/script&gt; <br>					
          * @memberof ejTimePicker
          * @instance
          */
            close:null
        },

        /**
         * Specify the data types for default properties 
		 * @private
         */
        dataTypes: {
            timeFormat: "string",
            minTime: "string",
            maxTime: "string",
            readOnly: "boolean",
            interval: "number",
            showPopupButton: "boolean",
            locale: "string",
            hourInterval: "number",
            minutesInterval: "number",
            secondsInterval: "number",
            enabled: "boolean",
            enablePersistence: "boolean",
			enableAnimation: "boolean",
			enableStrictMode: "boolean",
        },

        observables: ["value"],
        // public functions

        /**
        * To enable the timepicker  
		* @return jQuery
		* @example 
		* &lt;input type="text" id="timepicker" /&gt;<br/>
		* &lt;script&gt;
		* $("#timepicker").ejTimePicker();
		* // Create TimePicker instance
		* var timeObj = $("#timepicker").data("ejTimePicker");
		* timeObj.enable(); // enables the timepicker
		* &lt;/script&gt;
		* @example 
	    * &lt;input type="text" id="timepicker" /&gt;<br/>
	    * &lt;script&gt;
		* $("#timepicker").ejTimePicker();
	    * // enables the timepicker
        * $("#timepicker").ejTimePicker("enable");
	    * &lt;/script&gt;
		*@memberof ejTimePicker
		* @instance
        */
        enable: function () {
            if (!this.model.enabled) {
                this.element[0].disabled = false;
                this.model.enabled = true;
                this.element.removeClass("e-disable").attr("aria-disabled", false);
                if (this.model.showPopupButton) {
                    this.timeIcon.removeClass("e-disable").attr("aria-disabled", false);
                    this.popupList.removeClass("e-disable").attr("aria-disabled", false);
                }
            }
        },

        /**
        * To disable the timepicker  
		* @return jQuery
		* @example 
		* &lt;input type="text" id="timepicker" /&gt;<br/>
		* &lt;script&gt;
		* $("#timepicker").ejTimePicker();
		* // Create TimePicker instance
		* var timeObj = $("#timepicker").data("ejTimePicker");
		* timeObj.disable(); // disable the timepicker
		* &lt;/script&gt;
		* @example 
	    * &lt;input type="text" id="timepicker" /&gt;<br/>
	    * &lt;script&gt;
		* $("#timepicker").ejTimePicker();
	    * // disable the timepicker
        * $("#timepicker").ejTimePicker("disable");
	    * &lt;/script&gt;
		*@memberof ejTimePicker
		* @instance
        */
        disable: function () {
            if (this.model.enabled) {
                this.element[0].disabled = true;
                this.model.enabled = false;
                this.element.addClass("e-disable").attr("aria-disabled", true);
                if (this.model.showPopupButton) {
                    this.timeIcon.addClass("e-disable").attr("aria-disabled", true);
                    this.popupList.addClass("e-disable").attr("aria-disabled", true);
                }
                this._hideResult();
            }
        },

        /**
        * returns the current time value
		* @return jQuery
		* @example 
		* &lt;input type="text" id="timepicker" /&gt;<br/>
		* &lt;script&gt;
		* $("#timepicker").ejTimePicker();
		* // Create TimePicker instance
		* var timeObj = $("#timepicker").data("ejTimePicker");
		* timeObj.getValue(); // returns the timepicker value
		* &lt;/script&gt;
		* @example 
	    * &lt;input type="text" id="timepicker" /&gt;<br/>
	    * &lt;script&gt;
		* $("#timepicker").ejTimePicker();
	    * // to get the time value
        * $("#timepicker").ejTimePicker("getValue");
	    * &lt;/script&gt;
		*@memberof ejTimePicker
		* @instance
        */
        getValue: function () {
            return this.element.val();
        },

        /**
        * updates the current system time to timepicker
		* @return jQuery
		* @example 
		* &lt;input type="text" id="timepicker" /&gt;<br/>
		* &lt;script&gt;
		* $("#timepicker").ejTimePicker();
		* // Create TimePicker instance
		* var timeObj = $("#timepicker").data("ejTimePicker");
		* timeObj.setCurrentTime(); // updates the current system
		* &lt;/script&gt;
		* @example 
	    * &lt;input type="text" id="timepicker" /&gt;<br/>
	    * &lt;script&gt;
		* $("#timepicker").ejTimePicker();
	    * // updates the current system
        * $("#timepicker").ejTimePicker("setCurrentTime");
	    * &lt;/script&gt;
		*@memberof ejTimePicker
		* @instance
        */
        setCurrentTime: function () {
            if (this.model.enabled && !this.model.readOnly) this._setMask();
        },

        _timeFormat: function (format) {
            var validatedformat = this._validateTimeFormat(format);
            if (validatedformat) {
                this.model.timeFormat = validatedformat;
                this.model.value = this._localizeTime(this.getValue());
                this.element.val(this.model.value);
            }
            return validatedformat;
        },

        _getTimeFormat: function () {
            this.model.timeFormat = Globalize.culture(this.model.locale).calendar.patterns.t || "h:mm tt";
            this.seperator = this._getSeperator();
        },

        _changeSkin: function (skin) {
            this.wrapper.removeClass(this.model.cssClass).addClass(skin);
            if (this.popupList) this.popupList.removeClass(this.model.cssClass).addClass(skin);
        },

        _localize: function (culture) {
            var currentTime = this._createObject(this.element.val());
            this.model.locale = culture;
            this._getTimeFormat()

            this.model.value = this._localizeTime(currentTime);
            this.element.val(this.model.value);
            this._getAmPm();
        },

        // constructor function
        _init: function () {
            if (!this.element.is("input") || (this.element.attr('type') && this.element.attr('type') != "text")) return false;
            this._initialize();
            this._render();
            this._wireEvents();
        },

        _setModel: function (options) {
            if (!this.model.enabled && ej.isNullOrUndefined(options["enabled"])) return false;

            var change = false, prev = this.model.value;
            if (!ej.isNullOrUndefined(options["minTime"]) || !ej.isNullOrUndefined(options["maxTime"])) {
                if (!ej.isNullOrUndefined(options["minTime"]) && $.trim(options["minTime"]) && this._isValid(options["minTime"])) {
						this.model.minTime = options["minTime"];
						this._validateTimes();
					}
                if (!ej.isNullOrUndefined(options["maxTime"]) && $.trim(options["maxTime"]) && this._isValid(options["maxTime"])) {
						this.model.maxTime = options["maxTime"];
						this._validateTimes();
					}

                this._validateMinMax();
                if (!ej.isNullOrUndefined(options["minTime"])) options["minTime"] = this.model.minTime;
                if (!ej.isNullOrUndefined(options["maxTime"])) options["maxTime"] = this.model.maxTime;

                if (this.model.showPopupButton) this._reRenderDropdown();
                if (options["value"] == undefined) this._enableMask();
                change = true;
            }

            var option;
            for (option in options) {
                switch (option) {
                    case "timeFormat":
                        var newFormat = this._timeFormat(options[option]);
                        options[option] = this.model.timeFormat;
                        if (newFormat) {
                            this.seperator = this._getSeperator();
                            if (this.model.showPopupButton) this._reRenderDropdown();
                        }
                        break;
                    case "locale":
                        this._localize(options[option]);
                        if (this.model.showPopupButton) this._reRenderDropdown();
                        break;
                    case "interval":
                        this.model.interval = options[option];
                        if (this.model.showPopupButton) this._reRenderDropdown();
                        break;
                    case "showPopupButton": this._showButton(options[option]); break;
                    case "cssClass": this._changeSkin(options[option]); break;
                    case "showRoundedCorner": this._setRoundedCorner(options[option]); break;
                    case "enableRTL": this._setRtl(options[option]); break;
                    case "height":
                        this._setHeight(options[option]); break;
                    case "width":
                        this.wrapper.width(options[option]);
                        this._setListWidth();
                        break;
                    case "value":
						if (!this.model.enableStrictMode){
							if (this._isValid(options[option])) {
								this.model.value = options[option];
								this._enableMask();
							}
						}
						else {
							this.model.value = options[option];
							this._enableMask();
						}
                        options[option] = this.model.value;
                        change = true;
                        break;
					case "enableStrictMode":
                        this.model.enableStrictMode = jsondata[key];
                        break;
					case "enableAnimation":{
							this.model.enableAnimation=options[key];
							break;
					}
                    case "popupHeight": this.model.popupHeight = options[option]; this._setListHeight(); break;
                    case "popupWidth": this.model.popupWidth = options[option]; this._setListWidth(); break;
                    case "enabled": if (options[option]) this.enable(); else this.disable(); break;
                }
            }
            if (change) this._raiseChangeEvent(prev);
            this._checkErrorClass();
        },

        // all events bound using this._on will be unbind automatically
        _destroy: function () {
            this.element.insertAfter(this.wrapper);
            this.wrapper.remove();
            this.element.removeClass("e-input").removeAttr("ondragstart draggable").val("");
            if (this.popupList) this.popupList.remove();
        },

        _initialize: function () {
            this.target = this.element[0];
            this.timeIcon = null;
            this.popupList = null;
            this.focused = false;
            this.start = 0;
            this.end = 0;
            this.min = null;
            this.max = null;
            this.incomplete = false;
            this.downPosition = 0;
            this._getAmPm();
            this.showDropdown = false;
            this._activeItem = 0;
            this.isValidState = true;
            this._manualFocus = false;
            this._isIE7 = this._checkIE7();
            // _getInternalEvents is used when TimePicker used as a subcontrol of DateTimePicker 
            this._getInternalEvents = false;
            if (!this.model.timeFormat) this._getTimeFormat();
            else this.seperator = this._getSeperator();
        },

        _render: function () {
            this._renderWrapper();
            this._setDimentions();
            this._renderTimeIcon();
            this._validateTimes();
            this._renderDropdown();
            this._checkProperties();
            this._enableMask();
            this._checkErrorClass();
            this.element.attr({ 'aria-atomic': 'true', 'aria-live': 'assertive', "aria-readonly": this.model.readOnly, "value":this.model.value });
			if (this.model.showPopupButton || !ej.isNullOrUndefined(this.popupList))
            this.ul.find("li").attr({ 'tabindex': -1, 'aria-selected': false });
        },

        _renderWrapper: function () {
            this.element.addClass("e-input").attr("tabindex", "0");
            this.wrapper = ej.buildTag("span.e-timewidget e-widget " + this.model.cssClass + "#" + this.target.id + "_timewidget").insertAfter(this.element);
            this.container = ej.buildTag("span.e-in-wrap e-box").append(this.element);
            this.wrapper.append(this.container);
        },

        _renderTimeIcon: function () {
            if (this.model.showPopupButton) {
                this.timeIcon = ej.buildTag("span.e-select").attr({ 'role': 'button', 'aria-label': 'select' });
                var icon = ej.buildTag("span.e-icon e-time").attr('role', 'presentation');
                this.timeIcon.append(icon);
                this.container.append(this.timeIcon).addClass("e-padding");
                this._on(this.timeIcon, "mousedown", this._timeIconClick);
            }
        },

        _renderDropdown: function () {
			var oldWrapper = $("#" + this.element.context.id + "_popup").get(0);
            if (oldWrapper)
                $(oldWrapper).remove();
            if (!this.model.showPopupButton || this.popupList) return false;
            this.popupList = ej.buildTag("div.e-time-popup e-popup e-widget e-box " + this.model.cssClass + "#" + this.target.id + "_popup", "", {}, { 'tabindex': 0, 'aria-activedescendant': '' });
            this.popup = this.popupList;
            this.ul = ej.buildTag("ul.e-ul");
            var scrollDiv = ej.buildTag("div").append(this.ul);
            $('body').append(this.popupList.append(scrollDiv));
            this._renderLiTags();
            this._setListHeight();
            this._setListWidth();
            this.popupList.ejScroller({ height: this.popupList.height(), width: 0, scrollerSize: 20 });
            this.scrollerObj = this.popupList.ejScroller("instance");
            this.popupList.css("display", "none");
            this._listSize = this.ul.find("li").size();
        },
        _renderLiTags: function () {
            if (this.model.interval < 1) return false;
            var start, end, timeVal, interval = this.model.interval * 60000
            start = (this.model.minTime) ? this._createObject(this.model.minTime) : this._createObject("12:00:00 AM");
            end = (this.model.maxTime) ? this._createObject(this.model.maxTime) : this._createObject("11:59:59 PM");

            while (this._compareTime(end, start, true)) {
                timeVal = this._localizeTime(start);
                this.ul.append(ej.buildTag("li", timeVal));
                start = this._createObject(start).getTime() + interval;
            }

            var liTags = this.ul.find("li");
            this._on(liTags, "mouseenter", $.proxy(this._OnMouseEnter, this));
            this._on(liTags, "mouseleave", $.proxy(this._OnMouseLeave, this));
            this._on(liTags, "click", $.proxy(this._OnMouseClick, this));
        },
        _reRenderDropdown: function () {
            this.ul.empty();
            this._renderLiTags();
            this._refreshScroller();
            this._changeActiveEle();
        },
        _refreshScroller: function () {
            var flag = this.popupList.css("display") == "none" ? true : false;
            this.popupList.css("height", "auto");
            this.popupList.find(".e-content, .e-vscroll").removeAttr("style");
            this.popupList.find(".e-vscroll div").removeAttr("style");

            if (flag) this.popupList.css("display", "block");  // For get the height of the popup
            this.scrollerObj.model.height = this.popupList.height();
            this.scrollerObj.model.scrollTop = 0;
            this.scrollerObj.refresh();
            if (flag) this.popupList.css("display", "none");
        },

        _setListWidth: function () {
            if (this.popupList) {
                var width = this.model.popupWidth;
                if (width && width != "auto") this.popupList.css({ "width": width });
                else this.popupList.css({ "width": this.wrapper.width() });
            }
        },
        _setListHeight: function () {
            if (this.popupList) this.popupList.css({ "max-height": this.model.popupHeight || "191px" });
            if (this.scrollerObj) {
                this._refreshScroller();
                this._updateScrollTop();
            }
        },
        _updateScrollTop: function () {
            this.scrollerObj.setModel({ "scrollTop": this._calcScrollTop() });
        },
        _refreshPopup: function () {
            if (this.model.popupWidth == "auto") this.popupList.css({ "width": this.wrapper.width() });
            this._setListPosition();
            this._refreshScroller();
        },
        _setListPosition: function () {
            var pos = this._getOffset(this.wrapper),
            left = pos.left,
            totalHeight = this.wrapper.outerHeight(),
            border = (totalHeight - this.wrapper.height()) / 2,
            maxZ = this._getZindexPartial();
            if (this.model.enableRTL) left -= this.popupList.outerWidth() - this.wrapper.outerWidth();

            this.popupList.css({
                "left": left + "px",
                "top": pos.top + totalHeight - border + 3 + "px",
                "z-index": maxZ
            });
        },
        _getOffset: function (ele) {
            var pos = ele.offset();
            if ($("body").css("position") != "static") {
                var bodyPos = $("body").offset();
                pos.left -= bodyPos.left;
                pos.top -= bodyPos.top;
            }
            return pos;
        },
        _getZindexPartial: function () {
            var parents = this.element.parents(), bodyEle;
            bodyEle = $('body').children(), index = bodyEle.index(this.popup);
            bodyEle.splice(index, 1);
            $(bodyEle).each(function (i, ele) { parents.push(ele); });

            var maxZ = Math.max.apply(maxZ, $.map(parents, function (e, n) {
                if ($(e).css('position') != 'static') return parseInt($(e).css('z-index')) || 1;
            }));
            if (maxZ == undefined || maxZ == null || maxZ < 10000) maxZ = 10000;
            else maxZ += 1;
            return maxZ;
        },

        _enableMask: function () {
            if (this.model.value) {
                var flag = false;
                if ((this.model.minTime && this._compareTime(this.model.minTime, this.model.value)) ||
                    this.model.maxTime && this._compareTime(this.model.value, this.model.maxTime))
                    this.isValidState = false;
                else this.isValidState = true;

                this._setTime(this.model.value);
                if (this._getInternalEvents && !this.isValidState) this._trigger("outOfRange");
            }
            else this._setMask();
            this._changeActiveEle();
            this._preVal = this.element.val();
        },
        _setTime: function (time) {
            var modifiedTime = this._localizeTime(time);
            this.element.val(modifiedTime);
            this.model.value = modifiedTime;
        },
        _setMask: function () {
            this.model.value = new Date();
            this._enableMask();
        },

        _validateTimes: function () {
            var validatedformat = this._validateTimeFormat(this.model.timeFormat);
            if (validatedformat) this.model.timeFormat = validatedformat;
            else this.model.timeFormat = "h:mm tt";

            if (!this._isValid(this.model.minTime)) this.model.minTime = "12:00:00 AM";
            if (!this._isValid(this.model.maxTime)) this.model.maxTime = "11:59:59 PM";
			if (!this._isValid(this.model.value)) this.model.value = null;
			if(!this._checkMinMax(this.model.value) && !this.model.enableStrictMode)
				this.model.value = this.model.minTime;
            this._validateMinMax();
        },
        _validateMinMax: function () {
            if (this.model.minTime && this.model.maxTime && this._compareTime(this.model.minTime, this.model.maxTime)) {
                this.model.minTime = this.model.maxTime;
            }
        },
        _checkProperties: function () {
            if (!this.model.enabled) {
                this.model.enabled = true;
                this.disable();
            }
            this._addProperty();
            this._checkAttributes();
        },
        _addProperty: function () {
            this._setRtl(this.model.enableRTL);
            this._setRoundedCorner(this.model.showRoundedCorner);
        },
        _setRtl: function (boolean) {
            if (boolean) {
                this.wrapper.addClass("e-rtl");
                if (this.popupList) this.popupList.addClass("e-rtl");
            }
            else {
                this.wrapper.removeClass("e-rtl");
                if (this.popupList) this.popupList.removeClass("e-rtl");
            }
        },
        _setRoundedCorner: function (boolean) {
            if (boolean) {
                this.container.addClass("e-corner-all");
                if (this.popupList) this.popupList.addClass("e-corner-all");
            }
            else {
                this.container.removeClass("e-corner-all");
                if (this.popupList) this.popupList.removeClass("e-corner-all");
            }
        },
        _showButton: function (show) {
            this.model.showPopupButton = show;
            if (show) {
                this.container.addClass("e-padding");
                this._renderTimeIcon();
                this._renderDropdown();
                this._addProperty();
            }
            else {
                this.container.removeClass("e-padding");
                this.timeIcon.remove();
                this.popupList.remove();
                this.timeIcon = this.popupList = null;
            }
        },
        _checkAttributes: function () {
            if (!this.element.attr("name"))
                this.element.attr({ "name": this.element[0].id });
            if ('ondragstart' in document.createElement('input'))
                this.element.attr({ "ondragstart": "return false" });
            if ('draggable' in document.createElement('input'))
                this.element.attr({ "draggable": "false" });
        },

        _getAmPm: function () {
            this.ttAM = $.trim(this._localizeMeridian("00:00"));
            this.ttPM = $.trim(this._localizeMeridian("23:00"));
        },

        _setDimentions: function () {
            this._setHeight(this.model.height);
            if (this.model.width) this.wrapper.width(this.model.width);
        },
        _setHeight: function (height) {
            if (height) this.wrapper.height(height);
            if (this._isIE7) this.element.height(this.container.height());
        },

        _validateTimeFormat: function (timeFormat) {
            var parts = timeFormat.split(" "), format = "";
            if (parts.length == 1 || parts.length == 2) {
                $(parts).each(function (i, part) {
                    format += $.trim(part) + " ";
                });
                return $.trim(format);
            }
            else return null;
        },

        _getSeperator: function () {
            var p = this._getElePlace(), formats = this.model.timeFormat.split(" ")[p.time];
            var regex = new RegExp("^[a-zA-Z0-9]+$");

            for (var i = 0; i < formats.length; i++) {
                if (!regex.test(formats.charAt(i))) return formats.charAt(i);
            }
        },

        _checkInComplete: function () {
            var pos = this._getCaretSelection(), cursor = this._getStartEnd(pos);
            var replace = "00", selected = this._getSelectedValue(cursor), category = this._getCategory(cursor);
            if (pos.end - pos.start == this.element.val().length) this._checkAll();

            if (category && category != "tt") {
                this._findCategoryPosition(category);
                if (selected == "__") {
                    if (category == "h" || category == "hh") replace = "12";
                    this._changeToDefault(replace);
                }
                else if (category.length != 1 && selected.length == 1) {
                    selected = this._changeWhole(selected);
                    this.element.val(this._replaceAt(this.target.value, this.start, this.end, selected));
                }
            }
        },
        _checkAll: function () {
            var i, p = this._getElePlace(), categories = this.model.timeFormat.split(" ")[p.time].split(this.seperator);
            for (i = 0; i < categories.length; i++) {
                this._findCategoryPosition(categories[i]);
                var selected = this._getSelectedValue({ start: this.start, end: this.end });

                if (categories[i].length != 1 && selected.length == 1) {
                    selected = this._changeWhole(selected);
                    this.element.val(this._replaceAt(this.element.val(), this.start, this.end, selected));
                }
            }
        },

        _changeToDefault: function (replace) {
            this.incomplete = true;
            var preVal = this.element[0].value
            this.element[0].value = this._replaceAt(this.target.value, this.start, this.end, replace);
            var timeValue = this._checkExceedRange(this.target.value);
            if (!!timeValue) {
                this._setTime(this.model[timeValue]);
            }
            this._setSelection(this.start, this.end);
            this._raiseChangeEvent(preVal);
        },

        _setSelection: function (start, end) {
            var element = this.element[0];

            if (element.setSelectionRange)
                element.setSelectionRange(start, end);
            else if (element.createTextRange) {
                // For lower version browsers (IE8, IE7 ...)
                element = element.createTextRange();
                element.collapse(true);
                element.moveEnd('character', end);
                element.moveStart('character', start);
                element.select();
            }
        },

        _getSelectedValue: function (cursor) {
            return this.target.value.substring(cursor.start, cursor.end);
        },

        _getMinMax: function (currPart, keydown) {
            if (currPart == "hh" || currPart == "h") {
                this.min = 1; this.max = 11;
                if (keydown) this.max = 12;
            }
            else if (currPart == "HH" || currPart == "H") {
                this.min = 0; this.max = 23;
            }
            else if (currPart == "mm" || currPart == "m" || currPart == "ss" || currPart == "s") {
                this.min = 0; this.max = 59;
            }
        },

        _focusElement: function () {
            this._manualFocus = true;
            this.element.focus();
        },
        _targetFocus: function (e) {
            e.preventDefault();
            this.focused = true;
            this.element.bind('mousewheel DOMMouseScroll', $.proxy(this._mouseWheel, this));
            this.wrapper.addClass("e-focus").removeClass("e-error").attr('aria-invalid', "false");
            if (!this._manualFocus) {
                this._findCategoryPosition(this._getLeast(false));
                this._setSelection(this.start, this.end);
            }
            this._manualFocus = false;
			this._prevTimeVal = this.element.val();
            this._raiseEvent("focusIn");
        },
        _targetBlur: function () {
            this.focused = false;
            this.element.unbind('mousewheel DOMMouseScroll', $.proxy(this._mouseWheel, this));
            this.wrapper.removeClass("e-focus");
            this._checkInComplete();
            if (!this._checkMinMax(this.target.value)){
				if(!this.model.enableStrictMode){
					this.element.val(this._prevTimeVal);
					this.model.value = this._prevTimeVal;
					this.isValidState = true;
				}
				else
					this.isValidState = false;
			}
            else this.isValidState = true;
            this._checkErrorClass();
            this._raiseEvent("focusOut");
        },
        _checkErrorClass: function () {
            if (this.isValidState) this.wrapper.removeClass("e-error").attr('aria-invalid', "false");
            else this.wrapper.addClass("e-error").attr('aria-invalid', "true");
        },

        _getCaretSelection: function () {
            var input = this.element[0], start = 0, end = 0;

            if (!isNaN(input.selectionStart)) {
                start = input.selectionStart;
                end = input.selectionEnd;
                return { start: Math.abs(start), end: Math.abs(end) };
            }
            // For lower version browsers (IE8, IE7 ...)
            var bookmark = document.selection.createRange().getBookmark();
            var selection = input.createTextRange();
            selection.moveToBookmark(bookmark);

            var before = input.createTextRange();
            before.collapse(true);
            before.setEndPoint("EndToStart", selection);
            var beforeLength = before.text.length, selLength = selection.text.length;
            return { start: beforeLength, end: beforeLength + selLength };
        },

        _mouseDownOnInput: function (e) {
            if (!this.focused) this._focusElement();
            this.downPosition = this._getCaretSelection();
            this._checkInComplete();
            $(document).bind("mouseup", $.proxy(this._mouseUpOnInput, this));
        },

        _mouseUpOnInput: function (e) {
            e.preventDefault();
            $(document).unbind("mouseup", $.proxy(this._mouseUpOnInput, this));
            var pos = this._getCaretSelection();

            if (this.incomplete) {
                this.incomplete = false;
                pos = this.downPosition;
            }
            var cursor = this._getStartEnd(pos);
            this._setSelection(cursor.start, cursor.end);
        },

        _getCategoryPosition: function (category) {
            var s = 0, e = 0, parts = this.target.value.split(" "), p = this._getElePlace(), sep = this.seperator, valid = false;
            var fParts = this.model.timeFormat.split(" ")[p.time].split(sep);
            var tParts = parts[p.time].split(sep);
            if (fParts.length > tParts.length) return { start: s, end: e, isValid: valid };

            if (category == "tt") {
                if (parts[p.tt] == this.ttAM || parts[p.tt] == this.ttPM) {
                    if (p.tt == 0) s = 0;
                    else s = parts[p.time].length + 1;
                    e = s + parts[p.tt].length;
                    valid = true;
                }
            }
            else {
                if (p.time == 0) s = 0;
                else s = parts[p.tt].length + 1;

                var index = fParts.indexOf(category);
                if (index != -1) {
                    for (var i = 0; i < fParts.length; i++) {
                        e = tParts[i].length + 1;
                        if (i == index) break;
                        else s += e;
                    }
                    e += s - 1;
                    valid = true;
                }
            }
            return { start: s, end: e, isValid: valid };
        },
        _getCategory: function (cursor) {
            var parts = this.model.timeFormat.split(" "), sep = this.seperator;

            var p = this._getElePlace();

            if (cursor.isTT) return parts[p.tt];
            else return parts[p.time].split(sep)[cursor.index];
        },

        _getStartEnd: function (pos) {
            var sep = this.seperator;
            var value = this.element.val(), parts = value.split(" "), s = 0, e = 0, place = tt = null, i, j;

            for (j = 0; j < parts.length; j++) {
                if (parts[j] != this.ttAM && parts[j] != this.ttPM) {
                    var time = parts[j].split(sep), tempS = s, tempE = s + time[0].length;
                    for (i = 0; i < time.length; i++) {
                        e = time[i].length + s;
                        if (pos.start <= e) {
                            place = i;
                            tt = false;
                            j = parts.length;
                            break;
                        }
                        else s += time[i].length + 1;
                    }
                }
                else {
                    if (pos.start <= s + parts[j].length) {
                        e = parts[j].length + s;
                        place = 0;
                        tt = true;
                        j = parts.length;
                        break;
                    }
                    else s += parts[j].length + 1;
                }
            }
            if (place == null) s = tempS, e = tempE, place = 0, tt = false;

            return { start: s, end: e, index: place, isTT: tt };
        },

        _modifyValue: function (isIncrement) {
            this._checkInComplete();
            var pos = this._getCaretSelection(), cursor;
            if (pos.start == pos.end) {
                var cate = this._getLeast(true);
                var position = this._getCategoryPosition(cate);
                cursor = this._getStartEnd(position);
            }
            else cursor = this._getStartEnd(pos);
            this.start = cursor.start; this.end = cursor.end;

            this._changeValue(cursor, isIncrement);
        },

        _keyUpOnInput: function (e) {
            e.preventDefault();
            if (this._preVal != this.element.val()) {
                this._preVal = this.element.val();
                this._raiseChangeEvent();
            }
        },

        _getNextCategory: function (cate, direction) {
            var categories = [], sep = this.seperator;
            var fParts = this.model.timeFormat.split(" ");
            $(fParts).each(function (i, part) {
                if (part == "tt") categories.push(part);
                else {
                    var inner = part.split(sep);
                    categories = inner.concat(categories);
                }
            });
            var index = categories.indexOf(cate), ix;
            if (index != -1) {
                if (direction) {
                    if (index == 0) ix = categories.length - 1;
                    else ix = index - 1;
                }
                else {
                    if (index == categories.length - 1) ix = 0;
                    else ix = index + 1;
                }
                return categories[ix];
            }
            return cate;
        },
        _getElePlace: function () {
            var fParts = this.model.timeFormat.split(" "), time, tt;
            if (fParts[0] == "tt") time = 1, tt = 0;
            else time = 0, tt = 1;
            return { time: time, tt: tt };
        },
        _movePosition: function (pos, direction) {
            var cursor = this._getStartEnd(pos);
            var currCate = this._getCategory(cursor);
            if (!currCate) currCate = this._getLeast(direction);
            var next = this._getNextCategory(currCate, direction);
            var cursor = this._getCategoryPosition(next);

            if (cursor.isValid) {
                this._setSelection(cursor.start, cursor.end);
            }
        },

        _keyDownOnInput: function (e) {
            if (this.model.readOnly && !this._readOnlyKeys(e)) return false;

            var pos, cursor, category, key = e.keyCode;
            // _getInternalEvents is set to true when TimePicker used inside DateTimePicker control
            // in DateTimePicker control it allows Up, Down, Home, End, Tab keys only
            if (this._getInternalEvents && key != 38 && key != 40 && key != 36 && key != 35 && key != 9) return false;
            // Up, Down, Esc
            if (this.showDropdown && key != 38 && key != 40 && key != 27 && !this._readOnlyKeys(e)) return false;
            else if (this.showDropdown && (key == 37 || key == 39)) e.keyCode = (key == 37) ? 38 : 40;

            pos = this._getCaretSelection();
            cursor = this._getStartEnd(pos);
            category = this._getCategory(cursor);

            switch (e.keyCode) {
                case 38:        // Up Key
                    e.preventDefault();
                    if (!this.showDropdown)
                        this._modifyValue(true);
                    else {
                        if (this._activeItem > 1) this._activeItem -= 1;
                        //else this._activeItem = this._listSize;         // For cycle rotation
                        this._addListHover();
                        this._selectTimeItem(this._getActiveItem());
                    }
                    break;
                case 40:        // Down Key
                    e.preventDefault();
                    if (e.altKey && this.model.showPopupButton)
                        this._showhidePopup();
                    else if (!this.showDropdown)
                        this._modifyValue(false);
                    else {
                        if (this._activeItem < this._listSize) this._activeItem += 1;
                        //else this._activeItem = 1;        // For cycle rotation
                        this._addListHover();
                        this._selectTimeItem(this._getActiveItem());
                    }
                    break;
                case 37:        // Left Key
                    e.preventDefault();
                    this._checkInComplete();
                    if (pos.start == pos.end) this._setSelection(pos.start - 1, pos.start - 1);
                    else this._movePosition(pos, true);
                    break;
                case 39:         // Right Key
                    e.preventDefault();
                    this._checkInComplete();
                    if (pos.start == pos.end) this._setSelection(pos.start + 1, pos.start + 1);
                    else this._movePosition(pos, false);
                    break;

                case 36:         // Home Key
                    e.preventDefault();
                    if (!this.showDropdown) {
                        var homecate = this._firstlastVal(true);
                        var hPos = this._getCategoryPosition(homecate);
                        if (hPos.isValid) this._setSelection(hPos.start, hPos.end);
                    }
                    else {
                        this._activeItem = 1;
                        this._addListHover();
                        this._selectTimeItem(this._getActiveItem());
                    }
                    break;
                case 35:         // End Key
                    e.preventDefault();
                    if (!this.showDropdown) {
                        var endcate = this._firstlastVal(false);
                        var ePos = this._getCategoryPosition(endcate);
                        if (ePos.isValid) this._setSelection(ePos.start, ePos.end);
                    }
                    else {
                        this._activeItem = this._listSize;
                        this._addListHover();
                        this._selectTimeItem(this._getActiveItem());
                    }
                    break;
                case 9:     // Tab Key
                    if (this._getInternalEvents) break;
                    this._hideResult();
                    var flag = null;
                    if (e.shiftKey && pos.start > 0) flag = true;
                    else if (!e.shiftKey && pos.end < this.element.val().length) flag = false;
                    if (flag != null) {
                        e.preventDefault();
                        this._checkInComplete();
                        this._movePosition(pos, flag);
                    }
                    break;
                case 27:    // Esc Key
                case 13:    // Enter Key
                    this._hideResult();
                    break;
                case 8:          // Backspace Key
                case 46:         // Delete Key
                    e.preventDefault();
                    if (category && category != "tt") {
                        this._findCategoryPosition(category);
                        var _doBackspace = (key == 8 && pos.start != this.start), _doDelete = (key == 46 && pos.end != this.end), len;
                        len = this.end - this.start;

                        if ((pos.start != pos.end || len == 1) && (_doBackspace || _doDelete || pos.start != pos.end)) {
                            var s1 = this.start, s2 = this.end, te;
                            this.element[0].value = this._replaceAt(this.target.value, s1, s2, "__");
                            te = (s2 - s1 != 2) ? s2 + 1 : s2;
                            this._setSelection(s1, te);
                        }
                        else {
                            if (_doBackspace) {
                                this.element[0].value = this._replaceAt(this.target.value, pos.start - 1, pos.start, "");
                                this._setSelection(pos.start - 1, pos.start - 1);
                            }
                            else if (_doDelete) {
                                this.element[0].value = this._replaceAt(this.target.value, pos.end, pos.end + 1, "");
                                this._setSelection(pos.end, pos.end);
                            }
                        }
                    }
                    break;
            }

            var currSelection = this._getSelectedValue(cursor);
            var unicode = e.keyCode ? e.keyCode : e.charCode, actualkey;

            if (e.keyCode > 47 && e.keyCode < 58)
                actualkey = String.fromCharCode(unicode);
            else if (e.keyCode > 95 && e.keyCode < 106)
                actualkey = String.fromCharCode(unicode - 48);

            if (category == "tt" && ((!e.shiftKey && !e.ctrlKey && !e.altKey) && (e.keyCode > 64 && e.keyCode < 91) || (e.keyCode > 47 && e.keyCode < 58) || (e.keyCode > 95 && e.keyCode < 106))) {
                e.preventDefault();
                var ttPos = this._getCategoryPosition(category);
                this.start = ttPos.start;
                this.end = ttPos.end;
                this._changeAmPm(currSelection);
                this._raiseChangeEvent();
            }

            if ((!e.shiftKey && !e.ctrlKey && !e.altKey) && (e.keyCode > 47 && e.keyCode < 58) || (e.keyCode > 95 && e.keyCode < 106)) {
                if (category != "tt") {
                    this._getMinMax(category, true);
                    if (pos.start == pos.end) {
                        this._findCategoryPosition(category);
                        var newVal;
                        if (pos.start == this.start) newVal = actualkey + currSelection;
                        else newVal = currSelection + actualkey;
                        if (newVal.length > 2 || !(Number(newVal) >= this.min && this.max >= Number(newVal)))
                            e.preventDefault();
                    }
                    else if (!(Number(actualkey) >= this.min && this.max >= Number(actualkey)))
                        e.preventDefault();
                }
            }
            else if (!this._allowKeyCodes(e))
				!this.model.enableStrictMode ? e.preventDefault() : e.stopPropagation();
        },

        _allowKeyCodes: function (e) {
            // ctrl + A, C, Z, Y,  Tab, F5, Enter
            if ((e.ctrlKey && (e.keyCode == 65 || e.keyCode == 67 || e.keyCode == 90 || e.keyCode == 89))
                || e.keyCode == 9 || e.keyCode == 116 || e.keyCode == 13)
                return true;
            return false;
        },
        _readOnlyKeys: function (e) {
            // Left, Right, Home, End
            if (e.keyCode == 35 || e.keyCode == 36 || e.keyCode == 37 || e.keyCode == 39 || this._allowKeyCodes(e))
                return true;
            return false;
        },

        _firstlastVal: function (initial) {
            var parts = this.model.timeFormat.split(" "), sep = this.seperator;
            if (initial) {
                if (parts[0] != "tt") return parts[0].split(sep)[0];
                return "tt";
            }
            else {
                if (parts[0] != "tt") return "tt";
                else if (parts[1]) {
                    var lastItem = parts[1].split(sep);
                    return lastItem.length ? lastItem[lastItem.length - 1] : "tt";
                }
                return "tt";
            }
        },

        _mouseWheel: function (event) {
            event.preventDefault();
            if (this.model.readOnly) return false;

            var delta, rawEvent = event.originalEvent;
            if (rawEvent.wheelDelta) {
                // IE and Opera use wheelDelta, which is a multiple of 120 (possible values -120, 0, 120).
                delta = rawEvent.wheelDelta / 120;
                // In Opera, value is negated.
                //if (Sys.Browser.agent === Sys.Browser.Opera) delta = -delta;
            }
            else if (rawEvent.detail) {
                // Firefox uses detail property, which is a multiple of 3.
                delta = -rawEvent.detail / 3;
            }
            if (delta > 0)
                this._modifyValue(true);
            else if (delta < 0)
                this._modifyValue(false);
        },

        _addListHover: function () {
            this._addSelected();
            this._updateScrollTop();
        },
        _addSelected: function () {
            this.ul.find("li").removeClass("e-active");
            var activeItem = this._getActiveItem();
            activeItem.addClass("e-active");
        },
        _getActiveItem: function () {
            return $(this.ul.find("li")[this._activeItem - 1]);
        },

        _timeIconClick: function (event) {
            event.preventDefault();
            if (!this.model.enabled || this.model.readOnly || this.ul.find("li").length < 1) return false;
            this._showhidePopup();
            var len = this.element.val().length;
            this._setSelection(len, len);
        },
        _showhidePopup: function () {
            if (this._getInternalEvents) return false;
            if (!this.showDropdown)
                this._showResult();
            else
                this._hideResult();
        },
        _showResult: function () {
            this._raiseEvent("beforeOpen");
            this._refreshPopup();
            if (!this.focused) this._focusElement();
            this._changeActiveEle();
            var proxy = this, sTop = this._vissibleAndCalculateTop();
            this.popupList.slideDown(this.model.enableAnimation?200:0, "easeOutQuad", function () {
                $(document).bind("mousedown", $.proxy(proxy._OnDocumentClick, proxy));
            });
            this.scrollerObj.setModel({ "scrollTop": sTop });
            this.showDropdown = true;
            this._listSize = this.ul.find("li").size();
            $(window).bind("resize", $.proxy(this._OnWindowResize, this));
            this._raiseEvent("open");
        },
        _hideResult: function () {
            if (this.showDropdown && !this._getInternalEvents) {
                this.showDropdown = false;
                this.popupList.slideUp(this.model.enableAnimation?100:0, "easeOutQuad");
                $(document).unbind("mousedown", $.proxy(this._OnDocumentClick, this));
                $(window).unbind("resize", $.proxy(this._OnWindowResize, this));
                this._raiseEvent("close");
            }
        },

        _vissibleAndCalculateTop: function () {
            this.popupList.css({ "display": "block" });  // For get the height of the popup
            var scrollTop = this._calcScrollTop();
            this.popupList.css({ "display": "none" });
            return scrollTop;
        },
        _calcScrollTop: function () {
            var ulH = this.ul.outerHeight(), liH = this.ul.find("li").outerHeight(), index, top;
            index = this.ul.find("li.e-active").index();
            top = (liH * index) - ((this.popupList.outerHeight() - liH) / 2);
            return top;
        },
        _changeActiveEle: function () {
		    if (!this.model.showPopupButton || !this.popupList) return false;
            var elements = this.ul.find("li");
            var currTime = this.element.val(), firstTime = elements.first().html(), index;
            index = (this._parse(currTime) - this._parse(firstTime)) / (this.model.interval * 60000);
            index = Math.round(index);
            this._activeItem = (index == elements.length) ? index : index + 1;
            if (this._activeItem < 0 || this._activeItem > elements.length || isNaN(this._activeItem)) this._activeItem = 1;
            this._addListHover();
        },

        _OnDocumentClick: function (e) {
            if (!$(e.target).is(this.popupList) && !$(e.target).parents(".e-time-popup").is(this.popupList) &&
                !$(e.target).is(this.wrapper) && !$(e.target).parents(".e-timewidget").is(this.wrapper)) {
                this._hideResult();
            }
            else if ($(e.target).is(this.popupList) || $(e.target).parents(".e-time-popup").is(this.popupList))
                e.preventDefault();
        },
        _OnWindowResize: function (e) {
            this._refreshPopup();
        },

        _OnMouseEnter: function (e) {
            var targetEle = e.target;
            this.ul.find("li").removeClass("e-hover");
            $(targetEle).addClass("e-hover");
        },
        _OnMouseLeave: function (e) {
            this.ul.find("li").removeClass("e-hover");
        },
        _OnMouseClick: function (e) {
            e.preventDefault();
            if (this.model.enabled && !this.model.readOnly) {
                this._activeItem = $(e.target).index() + 1;
                this.ul.find("li").attr({ 'tabindex': -1, 'aria-selected': false });
                $(e.target).attr({ 'aria-selected': true, 'tabindex': 0 });
                this._addSelected();
                this._selectTimeItem($(e.target));
            }
            this._showhidePopup();
        },
        _selectTimeItem: function (ele) {
            this._beforeChange(ele);
            var flag = this._raiseChangeEvent();
            if (flag)
                this._trigger("select", { value: this.model.value,prevTime:this._previousValue });
        },

        _findCategoryPosition: function (category) {
            if (category == "least") category = this._getLeast(true);
            var pos = this._getCategoryPosition(category);
            this.start = pos.start;
            this.end = pos.end;
        },

        _getLeast: function (lower) {
            var formats = this.model.timeFormat.split(" "), sep = this.seperator, res = null;
            $(formats).each(function (i, e) {
                if (e != "tt") {
                    var times = e.split(sep);
                    if (lower) res = times[times.length - 1];
                    else res = times[0];
                }
            });
            return res;
        },

        _changeValue: function (cursor, isIncrement) {
            var preVal = this.target.value, currValue, category = this._getCategory(cursor);
            if (!category) return false;
            this._setSelection(this.start, this.end);
            currValue = this.target.value.substring(this.start, this.end);
            if (this._checkMinMax(this.target.value)) {
                if (currValue != this.ttAM && currValue != this.ttPM) {
                    currValue = this._changeCurrentValue(currValue, category, isIncrement);
                    if (category.length != 1) currValue = this._changeWhole(currValue);
                    this._findCategoryPosition(category);
                    this.model.value = this._replaceAt(this.target.value, this.start, this.end, currValue);
                    this.element.val(this.model.value);
                    this.end = this.start + currValue.toString().length;
                    this._setSelection(this.start, this.end);
                }
                else this._changeAmPm(currValue);
            }
            else {
                var timeValue = this._checkExceedRange(this.target.value);
                this._setTime(this.model[timeValue]);
                this._findCategoryPosition(category);
                this._setSelection(this.start, this.end);
            }
            // Min Max validation
            if (!this._checkMinMax(this.target.value)) {
                this.element.val(preVal);
                this._findCategoryPosition(category);
                this._setSelection(this.start, this.end);
            }
            else this._raiseChangeEvent(preVal);
        },

        _checkMinMax: function (value) {
            var res = this._checkExceedRange(value);
            return !res;
        },
        _checkExceedRange: function (value) {
            if (this.model.minTime && !this._compareTime(value, this.model.minTime, true)) return "minTime";
            if (this.model.maxTime && !this._compareTime(this.model.maxTime, value, true)) return "maxTime";
            return null;
        },

        _changeWhole: function (currValue) {
            return currValue > 9 ? "" + currValue : "0" + currValue;
        },
        _changeAmPm: function (ampm) {
            ampm = ampm == this.ttAM ? this.ttPM : this.ttAM;
            this.element.val(this._replaceAt(this.target.value, this.start, this.end, ampm));
            this._setSelection(this.start, this.end);
        },
        _changeMinute: function (isIncrement) {
            var formats = ["mm", "m"];
            var currFormat = this._getExactFormat(formats);
            if (currFormat) {
                this._findCategoryPosition(currFormat);
                var minute = Number(this.target.value.substring(this.start, this.end));
                this._getMinMax(currFormat);
                if (isIncrement) {
                    if (minute == this.max) {
                        minute = this.min;
                        this._changeHour();
                    }
                    else minute += 1;
                }
                else {
                    if (minute == this.min) {
                        minute = this.max;
                        this._changeHour();
                    }
                    else minute -= 1;
                }
                this._findCategoryPosition(currFormat);
                if (currFormat.length != 1) minute = this._changeWhole(minute);
                this.element.val(this._replaceAt(this.target.value, this.start, this.end, minute));
            }
        },
        _changeHour: function (isIncrement) {
            var formats = ["hh", "h", "HH", "H"];
            var currFormat = this._getExactFormat(formats);
            if (currFormat) {
                this._findCategoryPosition(currFormat);
                var hour = Number(this.target.value.substring(this.start, this.end));
                this._getMinMax(currFormat);
                if (isIncrement) {
                    if (hour == this.max) {
                        hour += 1;
                        this._changeMeridian();
                    }
                    else if (hour > this.max) hour = this.min;
                    else hour += 1;
                }
                else {
                    if (hour == this.min) hour = this.max + 1;
                    else if (hour > this.max) {
                        hour = this.max;
                        this._changeMeridian();
                    }
                    else hour -= 1;
                }
                this._findCategoryPosition(currFormat);
                if (currFormat.length != 1) hour = this._changeWhole(hour);
                this.element.val(this._replaceAt(this.target.value, this.start, this.end, hour));
            }
        },
        _getExactFormat: function (cate) {
            var tFormat = this.model.timeFormat;
            for (var i = 0; i < cate.length; i++) {
                if (tFormat.indexOf(cate[i]) != -1) return cate[i];
            }
            return null;
        },
        _changeMeridian: function () {
            var start = this.model.timeFormat.indexOf("tt");
            if (start != -1) {
                this._findCategoryPosition("tt");
                var meridian = this.target.value.substring(this.start, this.end);
                meridian = (meridian == this.ttAM) ? this.ttPM : this.ttAM;
                this.element.val(this._replaceAt(this.target.value, this.start, this.end, meridian));
            }
        },
        _changeCurrentValue: function (current, category, isIncrement) {
            current = Number(current);
            var c = category, step = 1, change = true;
            this._getMinMax(c);

            if (c == "hh" || c == "h" || c == "HH" || c == "H") step = this.model.hourInterval;
            else if (c == "mm" || c == "m") step = this.model.minutesInterval;
            else if (c == "ss" || c == "s") step = this.model.secondsInterval;
            if (step <= 0) return current;

            if (isIncrement) {
                if ((c == "hh" || c == "h") && current > this.max) current = this.min - 1 + step;
                else if (current < this.max) current += step;
                else {
                    change = false;
                    if (c != "hh" && c != "h") current = this.min - 1 + step;
                    else current += step;
                    this._changeAdjacent(c, isIncrement);
                }
                if ((c == "hh" || c == "h") && current == this.max + 1)
                    change && this._changeAdjacent(c, isIncrement);
                else if (current > this.max + 1) {
                    current = current - (this.max + 1);
                    change && this._changeAdjacent(c, isIncrement);
                }
                if ((c != "hh" && c != "h") && current == this.max + 1) {
                    current = this.min;
                    change && this._changeAdjacent(c, isIncrement);
                }
            }
            else {
                if ((c != "hh" && c != "h") && current > this.min) current -= step;
                else if ((c == "hh" || c == "h") && current > this.min && current <= this.max) current -= step;
                else if ((c == "hh" || c == "h") && current == this.min) current = this.max + 2 - step;
                else {
                    change = false;
                    current = this.max + 1 - step;
                    this._changeAdjacent(c, isIncrement);
                }
                if (current < this.min) {
                    current = current + (this.max + 1);
                    change && this._changeAdjacent(c, isIncrement);
                }
            }
            return current;
        },
        _changeAdjacent: function (c, isIncrement) {
            if (c == "ss" || c == "s") this._changeMinute(isIncrement);
            else if (c == "mm" || c == "m") this._changeHour(isIncrement);
            else if (c == "hh" || c == "h" || c == "HH" || c == "H") this._changeMeridian();
        },

        _valueChange: function (e) {
            this._raiseChangeEvent();
        },

		_beforeChange:function (ele)
		{
		 if(!this._raiseEvent("beforeChange"))
		 {
		   this.element.val(ele.text());
		 }
		 return true;
		},

        _raiseChangeEvent: function (prev) {
            prev = (prev == undefined) ? this.model.value : prev;
            this._previousValue = prev;
            if (prev == this.target.value) return false;
            if (this._checkMinMax(this.target.value)) this.isValidState = true;
            else this.isValidState = false;
            this.model.value = this.target.value;
            this._raiseEvent("change", { prevTime: this._previousValue });
            return true;
        },
        _raiseEvent: function (name) {           
            return(this._trigger(name, { value: this.model.value, prevTime: this._previousValue }));
        },
        _checkIE7: function () {
            if (navigator.appName == 'Microsoft Internet Explorer') {
                var re = new RegExp("MSIE ([0-9]{1,}[\.0-9]{0,})"), version = -1;
                if (re.exec(navigator.userAgent) != null)
                    version = parseFloat(RegExp.$1);
                if (version >= 7 && version < 8) return true;
            }
            return false;
        },
        _replaceAt: function (mainString, from, to, replace) {
            return mainString.substring(0, from) + replace + mainString.substring(to);
        },
        _localizeTime: function (value) {
            return $.trim(Globalize.format(this._createObject(value), this.model.timeFormat, this.model.locale));
        },
        _localizeMeridian: function (value) {
            return $.trim(Globalize.format(this._createObject(value), "tt", this.model.locale));
        },
        _compareTime: function (time1, time2, orEqual) {
            orEqual = (!orEqual) ? false : true;
            if (orEqual) return this._parse(time1) >= this._parse(time2);
            else return this._parse(time1) > this._parse(time2);
        },
        _isValid: function (time) {
            time = this._createObject(time);
            return time && typeof time.getTime === "function" && isFinite(time.getTime());
        },
        _parse: function (time) {
            return Date.parse(this._createObject(time));
        },
        _setEmptyDate: function (date) {
            var newDate = new Date(date);
            newDate.setDate(1);
            newDate.setMonth(0);
            newDate.setFullYear(2000);
            return newDate;
        },
        _createObject: function (value) {
            if (typeof value === "string" || value == null) {
                var obj = Globalize.parseDate("1/1/2000 " + value, "d/M/yyyy " + this.model.timeFormat);
                if (!obj) obj = new Date("1/1/2000 " + value);
                return obj;
            }
            else if (typeof value === "number")
                return new Date(value);
            else if (value instanceof Date)
                return this._setEmptyDate(value);
        },

        //-------------------- Event Wire-up -------------------------//
        _wireEvents: function () {
            this._on(this.element, "focus", this._targetFocus);
            this._on(this.element, "blur", this._targetBlur);
            this._on(this.element, "mousedown", this._mouseDownOnInput);
            this._on(this.element, "keydown", this._keyDownOnInput);
            this._on(this.element, "keyup", this._keyUpOnInput);
        }
    });
})(jQuery, Syncfusion);;
/**
* @fileOverview Plugin to select the date and time values.
* @copyright Copyright Syncfusion Inc. 2001 - 2014. All rights reserved.
*  Use of this code is subject to the terms of our license.
*  A copy of the current license can be obtained at any time by e-mailing
*  licensing@syncfusion.com. Any infringement will be prosecuted under
*  applicable laws. 
* @version 12.1 
* @author <a href="mailto:licensing@syncfusion.com">Syncfusion Inc</a>
*/

(function ($, ej, undefined) {
    /**
    * @namespace ej
	* @classdesc Date and Time selection with the input field.
	* @class ejDateTimePicker
	* @param {object} options - settings for Date Picker.
	* @requires jQuery
    * @requires jquery.easing.1.3.js
    * @requires jquery.globalize.js
    * @requires globalize.cultures.min.js
	* @requires ej.core.js
	* @requires ej.datetimepicker.js
	* @requires ej.datepicker.js
	* @requires ej.timepicker.js
	* @requires ej.scroller.js
	* @example 
	* &lt;input type="text" id="datetime" /&gt;
	* &lt;script&gt;
	* // Create DateTimePicker
    * $("#datetime").ejDateTimePicker();
	* &lt;/script&gt;
	*/

    ej.widget("ejDateTimePicker", "ej.DateTimePicker", {
        // widget element will be automatically set in this
        element: null,

        // user defined model will be automatically set in this
        model: null,
        validTags: ["input"],
        _setFirst: false,
        _rootCSS: "e-datetimepicker",
        type: "editor",

        // default model
        defaults: {
            // Common properties
            /**		
			* Set the root class for DateTimePicker theme. This cssClass API helps to use custom skinning option for DateTimePicker control. 
			* @default ""
			* @type {string}
			* @example
			* &lt;input type="text" id="datetime" /&gt;
			* &lt;script&gt;			
			* //To set cssClass API during initialization  
			* 	$("#datetime").ejDateTimePicker({  cssClass: "gradient-lime" });
			* &lt;/script&gt;
			 * @memberof ejDateTimePicker
			* @instance
			*/
            cssClass: "",

            /**		
			* Defines the localization culture for DateTimePicker.
			* @default "en-US"
			* @type {string}
			* @example 
			* &lt;input type="text" id="datetime" /&gt;
			* &lt;script&gt;
			* //To set locale API during initialization  
			* 	$("#datetime").ejDateTimePicker({  locale: "en-US" });
			* &lt;/script&gt;
			 * @memberof ejDateTimePicker
			* @instance
			*/
            locale: "en-US",

            /**		
			* Indicates that the DateTimePicker value can only be read and can’t change.
			* @default false
			* @type {Boolean}
			* @example 
			* &lt;input type="text" id="datetime" /&gt;
			* &lt;script&gt;
			* //To set readOnly API during initialization  
			* 	$("#datetime").ejDateTimePicker({  readOnly: true });
			* &lt;/script&gt;
			 * @memberof ejDateTimePicker
			* @instance
			*/
            readOnly: false,

            /**		
			* Changes the sharped edges into rounded corner for the DateTimePicker textbox and popup.
			* @default false
			* @type {Boolean}
			* @example 
			* &lt;input type="text" id="datetime" /&gt;
			* &lt;script&gt;
			* //To set showRoundedCorner API during initialization  
			* 	$("#datetime").ejDateTimePicker({  showRoundedCorner: true });
			* &lt;/script&gt; 
			 * @memberof ejDateTimePicker
			* @instance
			*/
            showRoundedCorner: false,

            /**		
			* Sets the DateTimePicker direction as right to left alignment.
			* @default false
			* @type {Boolean}
			* @example 
			* &lt;input type="text" id="datetime" /&gt;
			* &lt;script&gt;
			* //To set enableRTL API during initialization  
			* 	$("#datetime").ejDateTimePicker({  enableRTL: true });
			* &lt;/script&gt; 
			 * @memberof ejDateTimePicker
			* @instance
			*/
            enableRTL: false,

            /**		
			* When this property is set to false, it disables the DateTimePicker control.
			* @default false
			* @type {Boolean}
			* @example 
			* &lt;input type="text" id="datetime" /&gt;
			* &lt;script&gt;
			* //To set enabled API during initialization  
			* 	$("#datetime").ejDateTimePicker({  enabled: true });
			* &lt;/script&gt;
			 * @memberof ejDateTimePicker
			* @instance
			*/
            enabled: true,

            /**		
			* Sets the DateTime value to the control.
			* @default ""
			* @type {String | DateObject}
			* @example 
			* &lt;input type="text" id="datetime" /&gt;
			* &lt;script&gt;
			* //To set value API during initialization  
			* 	$("#datetime").ejDateTimePicker({  value:"6/2/2014 6:00 AM" });
			* &lt;/script&gt; 
			 * @memberof ejDateTimePicker
			* @instance
			*/
            value: "",

            /**		
			* Sets the minimum value to the DateTimePicker. Behind the minimum value an error class is added to the wrapper element.
			* @default new Date("1/1/1900 12:00:00 AM")
			* @type {String | DateObject}
			* @example 
			* &lt;input type="text" id="datetime" /&gt;
			* &lt;script&gt;
			* //To set minDateTime API during initialization  
			* 	$("#datetime").ejDateTimePicker({  minDateTime: new Date("5/5/2010 12:00:00 AM") });
			* &lt;/script&gt;
			* @memberof ejDateTimePicker
			* @instance
			*/
            minDateTime: new Date("1/1/1900 12:00:00 AM"),

            /**		
			* Sets the maximum value to the DateTimePicker. Beyond the maximum value an error class is added to the wrapper element.
			* @default new Date("12/31/2099 11:59:59 PM")
			* @type {String | DateObject}
			* @example 
			* &lt;input type="text" id="datetime" /&gt;
			* &lt;script&gt;
			* //To set maxDateTime API during initialization  
			* 	$("#datetime").ejDateTimePicker({  maxDateTime: new Date("12/10/2050 8:00:00 PM") });
			* &lt;/script&gt;
			 * @memberof ejDateTimePicker
			* @instance
			*/
            maxDateTime: new Date("12/31/2099 11:59:59 PM"),

            /**		
			* Defines the height of the DateTimePicker textbox.
			* @default 30
			* @type {String | Number}
			* @example 
			* &lt;input type="text" id="datetime" /&gt;
			* &lt;script&gt;
			* //To set height API during initialization  
			* 	$("#datetime").ejDateTimePicker({  height: 40 });
			* &lt;/script&gt; 
			 * @memberof ejDateTimePicker
			* @instance
			*/
            height: "",

            /**		
			* Defines the width of the DateTimePicker textbox.
			* @default 143
			* @type {String | Number}
			* @example 
			* &lt;input type="text" id="datetime" /&gt;
			* &lt;script&gt;
			* //To set width API during initialization  
			* 	$("#datetime").ejDateTimePicker({  width: 210 });
			* &lt;/script&gt; 
			 * @memberof ejDateTimePicker
			* @instance
			*/
            width: "",

            /**		
			* Defines the datetime format displayed in the DateTimePicker. The value should be a combination of date format and time format. 
			* @default "M/d/yyyy h:mm tt"
			* @type { String }
			* @example 
			* &lt;input type="text" id="datetime" /&gt;
			* &lt;script&gt;
			* //To set dateTimeFormat API during initialization  
			* 	$("#datetime").ejDateTimePicker({  dateTimeFormat: "d/M/yyyy tt h:mm" });
			* &lt;/script&gt; 
			 * @memberof ejDateTimePicker
			* @instance
			*/
            dateTimeFormat: "",

            /**		
			* Shows or hides the arrow button from the DateTimePicker textbox. 
              When the button disabled, the DateTimePicker popup opens while focus in the textbox and hides while focusout from the textbox.
			* @default true
			* @type { Boolean }
			* @example 
			* &lt;input type="text" id="datetime" /&gt;
			* &lt;script&gt;
			* //To set showPopupButton API during initialization  
			* 	$("#datetime").ejDateTimePicker({  showPopupButton: false });
			* &lt;/script&gt;
			 * @memberof ejDateTimePicker
			* @instance
			*/
            showPopupButton: true,
			
			/**		
			* When enableStrictMode true it allows the value outside of the range also, otherwise it internally changed to the correct value.
			* @default false
			* @type {Boolean}
			* @example 
			* &lt;input type="text" id="datetime" /&gt;
	        * &lt;script&gt;
			* //To set enableStrictMode API during initialization  
			* 	$("#datetime").ejDateTimePicker({  enableStrictMode: true });
		    * &lt;/script&gt; 
			 * @memberof ejDateTimePicker
			* @instance
			*/
            enableStrictMode: false,

            /**		
			* Displays the custom text for the buttons inside the DateTimePicker popup.
              when the culture value changed, we can change the buttons text based on the culture. 
			* @default { today: "Today", now: "Now", done: "Done", timeTitle: "Time" }
			* @type { JSONObject }
			* @example 
			* &lt;input type="text" id="datetime" /&gt;
			* &lt;script&gt;
			* //To set buttonText API during initialization  
			* 	$("#datetime").ejDateTimePicker({  buttonText: { done: "做过" } });
			* &lt;/script&gt;
			 * @memberof ejDateTimePicker
			* @instance
			*/
            buttonText: /** @lends ejDateTimePicker# */ {
                /**		
                 * Sets the text for the Today button inside the datetime popup.
				 * @alias ejDateTimePicker#buttonText->today
				 * @type String
				 * @example 
				 * &lt;input type="text" id="datetime" /&gt;
				 * &lt;script&gt;
				 * //To set buttonText API during initialization  
			     * 	$("#datetime").ejDateTimePicker({ buttonText: { today: "Today" }});
			     * &lt;/script&gt;
			     * @memberof ejDateTimePicker#buttonText
			     * @instance
                 */
                today: "Today",

                /**		
                 * Sets the text for the Now button inside the datetime popup.
				 * @alias ejDateTimePicker#buttonText->now
				 * @type String
				 * @example 
				 * &lt;input type="text" id="datetime" /&gt;
				 * &lt;script&gt;
				 * //To set buttonText API during initialization  
			     * 	$("#datetime").ejDateTimePicker({ buttonText: { now: "Now" }});
			     * &lt;/script&gt;
			     * @memberof ejDateTimePicker#buttonText
			     * @instance
                 
                 */
                now: "Now",

                /**		
                 * Sets the text for the Done button inside the datetime popup.
				 * @alias ejDateTimePicker#buttonText->done
				 * @type String
				 * @example 
				 * &lt;input type="text" id="datetime" /&gt;
				 * &lt;script&gt;
				 * //To set buttonText API during initialization  
			     * 	$("#datetime").ejDateTimePicker({ buttonText: { done: "Done" }});
			     * &lt;/script&gt;
			     * @memberof ejDateTimePicker#buttonText
			     * @instance
                 */
                done: "Done",

                /**		
                 * Sets the header text for the Time dropdown.
				 * @alias ejDateTimePicker#buttonText->timeTitle
				 * @type String
				 * @example 
				 * &lt;input type="text" id="datetime" /&gt;
				 * &lt;script&gt;
				 * //To set buttonText API during initialization  
			     * 	$("#datetime").ejDateTimePicker({ buttonText: { timeTitle: "Time" }});
			     * &lt;/script&gt;
			     * @memberof ejDateTimePicker#buttonText
			     * @instance
                 */
                timeTitle: "Time"
            },

            /**		
			* Enables or disables the state maintenance of DateTimePicker. 
			* @default false
			* @type { Boolean }
			* @example 
			* &lt;input type="text" id="datetime" /&gt;
			* &lt;script&gt;
			* //To set enablePersistence API during initialization  
			* 	$("#datetime").ejDateTimePicker({  enablePersistence: true });
			* &lt;/script&gt;
			 * @memberof ejDateTimePicker
			* @instance
			*/
            enablePersistence: false,

            /**		
			* Sets the time interval between the two adjacent time values in the time popup. 
			* @default 30
			* @type { Number }
			* @example 
			* &lt;input type="text" id="datetime" /&gt;
			* &lt;script&gt;
			* //To set interval API during initialization  
			* 	$("#datetime").ejDateTimePicker({  interval: 60 });
			* &lt;/script&gt;
			 * @memberof ejDateTimePicker
			* @instance
			*/
            interval: 30,

            /**		
			* Defines the time format displayed in the time dropdown inside the DateTimePicker popup. 
			* @default "h:mm tt"
			* @type { String }
			* @example 
			* &lt;input type="text" id="datetime" /&gt;
			* &lt;script&gt;
			* //To set timeDisplayFormat API during initialization  
			* 	$("#datetime").ejDateTimePicker({  timeDisplayFormat: "HH:mm" });
			* &lt;/script&gt; 
			 * @memberof ejDateTimePicker
			* @instance
			*/
            timeDisplayFormat: "",

            /**		
			* Defines the width of the time dropdown inside the DateTimePicker popup.
			* @default 100
			* @type { String | Number }
			* @example 
			* &lt;input type="text" id="datetime" /&gt;
			* &lt;script&gt;
			* //To set timePopupWidth API during initialization  
			* 	$("#datetime").ejDateTimePicker({  timePopupWidth: 150 });
			* &lt;/script&gt; 
			 * @memberof ejDateTimePicker
			* @instance
			*/
            timePopupWidth: 100,

            /**		
			* Specifies the header format of the datepicker inside the DateTimePicker popup. See {@link DatePicker.Level}
			* @default ej.DatePicker.Header.ShowHeaderMin
			* @type { String | Enum }
			* @example 
			* &lt;input type="text" id="datetime" /&gt;
			* &lt;script&gt;
			* //To set dayHeaderFormat API during initialization  
			* 	$("#datetime").ejDateTimePicker({  dayHeaderFormat: "showheadershort" });
			* &lt;/script&gt;
			 * @memberof ejDateTimePicker
			* @instance
			*/
            dayHeaderFormat: "showheadershort",

            /**		
			* Specifies the start level view in datepicker inside the DateTimePicker popup. See {@link DatePicker.Level}
			* @default ej.DatePicker.Level.Month or "month"
			* @type { String | Enum }
			* @example 
			* &lt;input type="text" id="datetime" /&gt;
			* &lt;script&gt;
			* //To set startLevel API during initialization  
			* 	$("#datetime").ejDateTimePicker({  startLevel:ej.DatePicker.Level.Year });
		    * &lt;/script&gt; 
			 * @memberof ejDateTimePicker
			* @instance
			*/
            startLevel: "month",

            /**		
			* Specifies the drill down level in datepicker inside the DateTimePicker popup.		See {@link ej.DatePicker.Level} 
			* @default ""
			* @type { enum }
			* @example 
			* &lt;input type="text" id="datetime" /&gt;
			* &lt;script&gt;
			* //To set depthLevel API during initialization  
			* 	$("#datetime").ejDateTimePicker({  depthLevel: "decade" });
			* &lt;/script&gt;
			 * @memberof ejDateTimePicker
			* @instance
			*/
            depthLevel: "",

            /**		
			* Specifies the start day of the week in datepicker inside the DateTimePicker popup. 
			* @default 1
			* @type { Number }
			* @example 
			* &lt;input type="text" id="datetime" /&gt;
			* &lt;script&gt;
			* //To set startDay API during initialization  
			* 	$("#datetime").ejDateTimePicker({  startDay: 2 });
			* &lt;/script&gt;
			 * @memberof ejDateTimePicker
			* @instance
			*/
            startDay: 0,

            /**		
			* Specifies the number of months to navigate at one click of next and previous button in datepicker inside the DateTimePicker popup. 
			* @default 1
			* @type { Number }
			* @example 
			* &lt;input type="text" id="datetime" /&gt;
			* &lt;script&gt;
			* //To set stepMonths API during initialization  
			* 	$("#datetime").ejDateTimePicker({  stepMonths: 2 });
			* &lt;/script&gt;
			 * @memberof ejDateTimePicker
			* @instance
			*/
            stepMonths: 1,

            /**		
			* It allows showing days in other months of DatePicker calendar inside the DateTimePicker popup.
			* @default true
			* @type { Boolean }
			* @example 
			* &lt;input type="text" id="datetime" /&gt;
			* &lt;script&gt;
			* //To set showOtherMonths API during initialization  
			* 	$("#datetime").ejDateTimePicker({  showOtherMonths: false });
			* &lt;/script&gt; 
			 * @memberof ejDateTimePicker
			* @instance
			*/
            showOtherMonths: true,
			enableAnimation: true,
            /**		
			* Specifies the header format to be displayed in the DatePicker calendar inside the DateTimePicker popup. 
			* @default "MMMM yyyy"
			* @type { String }
			* @example 
			* &lt;input type="text" id="datetime" /&gt;
			* &lt;script&gt;
			* //To set headerFormat API during initialization  
			* 	$("#datetime").ejDateTimePicker({  headerFormat: "MM - yyyy" });
			* &lt;/script&gt;
			 * @memberof ejDateTimePicker
			* @instance
			*/
            headerFormat: 'MMMM yyyy',

            /**     
            * Fires when DateTimePicker popup opens.
            * @event
            * @name ejDateTimePicker#open 	
            * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
            * @param {object}  argument.model returns the timepicker model
            * @param {string}  argument.type returns the name of the event
			* @param {string}  argument.value returns the modified datetime value
			* @param {string}  argument.prevDateTime returns the previously selected date time value
            * @example 
			* &lt;input type="text" id="datetime" /&gt;
			* &lt;script&gt;
            * //open event for datetimepicker
            * $("#datetime").ejDateTimePicker({
            *    open: function (args) {}
            * });      
			* &lt;/script&gt;
            * @memberof ejDateTimePicker
            * @instance
            */
            open: null,

            /**     
            * Fires when DateTimePicker popup closes.
            * @event
            * @name ejDateTimePicker#close 	
            * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
            * @param {object}  argument.model returns the timepicker model
            * @param {string}  argument.type returns the name of the event
			* @param {string}  argument.value returns the modified datetime value
			* @param {string}  argument.prevDateTime returns the previously selected date time value
            * @example 
			* &lt;input type="text" id="datetime" /&gt;
			* &lt;script&gt;
            * //close event for datetimepicker
            * $("#datetime").ejDateTimePicker({
            *    close: function (args) {}
            * }); 
			* &lt;/script&gt;			
            * @memberof ejDateTimePicker
            * @instance
            */
            close: null,
            /**     
            * Fires when the datetime value changed in the DateTimePicker textbox.
            * @event
            * @name ejDateTimePicker#change 	
            * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
            * @param {object}  argument.model returns the timepicker model
            * @param {string}  argument.type returns the name of the event
            * @param {boolean}  argument.isValidState returns the current value is valid or not
            * @param {string}  argument.value returns the modified datetime value			
			* @param {string}  argument.prevDateTime returns the previously selected date time value
            * @example 
			* &lt;input type="text" id="datetime" /&gt;
			* &lt;script&gt;
            * //change event for datetimepicker
            * $("#datetime").ejDateTimePicker({
            *    change: function (args) {}
            * }); 
			* &lt;/script&gt;			
            * @memberof ejDateTimePicker
            * @instance
            */
            change: null,
			/**     
          * Fires after DateTimePicker control is created.
          * @event
          * @name ejDateTimePicker#create 	
          * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
          * @param {object}  argument.model returns the DateTimePicker model
          * @param {string}  argument.type returns the name of the event
          * @example 
			* &lt;input type="text" id="datetime" /&gt;
			* &lt;script&gt;
            * //create event for datetimepicker
            * $("#datetime").ejDateTimePicker({
            *    create: function (args) {}
            * }); 
			* &lt;/script&gt;				
          * @memberof ejDateTimePicker
          * @instance
          */
            create: null,
			/**     
          * Fires when the DateTimePicker is destroyed successfully
          * @event
          * @name ejDateTimePicker#destroy 	
          * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
          * @param {object}  argument.model returns the DateTimePicker model
          * @param {string}  argument.type returns the name of the event
          * @example 
			* &lt;input type="text" id="datetime" /&gt;
			* &lt;script&gt;
            * //destroy event for datetimepicker
            * $("#datetime").ejDateTimePicker({
            *    destroy: function (args) {}
            * }); 
			* &lt;/script&gt;				
          * @memberof ejDateTimePicker
          * @instance
          */
            destroy: null,
            /**     
           * Fires when the focus-in happens in the DateTimePicker textbox.
           * @event
           * @name ejDateTimePicker#focusIn
           * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
           * @param {object}  argument.model returns the timepicker model
           * @param {string}  argument.type returns the name of the event           
           * @param {string}  argument.value returns the datetime value, which is in text box
           * @example 
           * &lt;input type="text" id="datetime" /&gt;
           * &lt;script&gt;
           * //focusIn event for datetimepicker
           * $("#datetime").ejDateTimePicker({
           *    focusIn: function (args) {}
           * }); 
           * &lt;/script&gt;			
           * @memberof ejDateTimePicker
           * @instance
           */
            focusIn: null,
            /**     
         * Fires when the focus-out happens in the DateTimePicker textbox.
         * @event
         * @name ejDateTimePicker#focusOut
         * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
         * @param {object}  argument.model returns the timepicker model
         * @param {string}  argument.type returns the name of the event           
         * @param {string}  argument.value returns the datetime value, which is in text box
         * @example 
         * &lt;input type="text" id="datetime" /&gt;
         * &lt;script&gt;
         * //focusOut event for datetimepicker
         * $("#datetime").ejDateTimePicker({
         *    focusOut: function (args) {}
         * }); 
         * &lt;/script&gt;			
         * @memberof ejDateTimePicker
         * @instance
         */
            focusOut: null

        },

        /**
         * Specify the data types for default properties 
		 * @private
         */
        dataTypes: {
            cssClass: "string",
            locale: "string",
            readOnly: "boolean",
            showRoundedCorner: "boolean",
            enableRTL: "boolean",
            enabled: "boolean",
			enableAnimation: "boolean",
            dateTimeFormat: "string",
            showPopupButton: "boolean",
            buttonText: "data",
            enablePersistence: "boolean",
			enableStrictMode: "boolean",
            interval: "number",
            timeDisplayFormat: "string",

            dayHeaderFormat: "string",
            startLevel: "string",
            depthLevel: "string",
            startDay: "number",
            stepMonths: "number",
            showOtherMonths: "boolean",
            headerFormat: "string"
        },

        _setModel: function (options) {
            if (!this.model.enabled && ej.isNullOrUndefined(options["enabled"])) return false;
            var option, validate = false;
            for (option in options) {
                switch (option) {
                    case "cssClass": this._changeSkin(options[option]); break;
                    case "locale": this._localize(options[option]); break;
                    case "readOnly": this._readOnly(options[option]); break;
                    case "showRoundedCorner": this._setRoundedCorner(options[option]); break;
                    case "enableRTL": this._setRtl(options[option]); break;
                    case "enabled": this._enabled(options[option]); break;

                    case "value":
                        options[option] = this._setValue(options[option]);
                        validate = true; break;
					case "enableAnimation":{
							this.model.enableAnimation=options[option];
							break;
					}
					case "enableStrictMode":
                        this.model.enableStrictMode = options[option];
                        validate = true;
                        break;
                    case "minDateTime":
                    case "maxDateTime":
                        var temp = this._stringToObject(options[option]);
                        if (this._isValidDate(temp)) {
                            this.datePicker.option(option + "Date", temp);
                            options[option] = temp;
                        }
                        else options[option] = this.model[option];
                        validate = true; break;
                    case "height": this.wrapper.height(options[option]); break;
                    case "width": this.wrapper.width(options[option]); break;                    
                    case "dateTimeFormat":
                        this.model.dateTimeFormat = options[option];
                        if (this.isValidState) this._setValue(this.model.value);
                        break;
                    case "showPopupButton": this._showButton(options[option]); break;
                    case "buttonText": this._buttonText(options[option]); break;

                    case "interval": this.timePicker.option("interval", options[option]); break;
                    case "timeDisplayFormat": this.timePicker.option("timeFormat", options[option]); break;
                    case "timePopupWidth": this.timePicker.option("popupWidth", options[option]); break;

                    case "dayHeaderFormat": this.datePicker.option("dayHeaderFormat", options[option]); break;
                    case "startLevel": this.datePicker.option("startLevel", options[option]); break;
                    case "depthLevel": this.datePicker.option("depthLevel", options[option]); break;
                    case "startDay": this.datePicker.option("startDay", options[option]); break;
                    case "stepMonths": this.datePicker.option("stepMonths", options[option]); break;
                    case "showOtherMonths": this.datePicker.option("showOtherMonths", options[option]); break;
                    case "headerFormat": this.datePicker.option("headerFormat", options[option]); break;
                }
            }
            if (validate) this._validateMinMax();
            this._valueChange();
            this._checkErrorClass();
        },
        observables: ["value"],
        // all events bound using this._on will be unbind automatically
        _destroy: function () {
            this.element.insertAfter(this.wrapper);
            this.element.removeClass("e-input").val("");
            this.wrapper.remove();
            this.popup.remove();
        },

        // constructor function
        _init: function () {
            if (!this.element.is("input") || (this.element.attr('type') && this.element.attr('type') != "text")) return false;
            this._initialize();
            this._render();
            this._wireEvents();
        },

        _initialize: function () {
            this.popup = null;
            this.isPopupOpen = false;
            this.isValidState = true;
            if (!this.model.dateTimeFormat || !this.model.timeDisplayFormat) this._getDateTimeFormat();
            if (typeof this.model.value == "string" && typeof this.model.value != "") {
                var val = this._stringToObject(this.model.value);
                if (val) this.model.value = val;
            }
            var min = this.model.minDateTime = this._stringToObject(this.model.minDateTime);
            if (!min || !this._isValidDate(min)) this.model.minDateTime = this.defaults.minDateTime;
            var max = this.model.maxDateTime = this._stringToObject(this.model.maxDateTime);
            if (!max || !this._isValidDate(max)) this.model.maxDateTime = this.defaults.maxDateTime;
        },

        _render: function () {
            this._renderWrapper();
            this._renderIcon();
            this._setDimentions();
            this._renderDropdown();
            this._checkProperties();
        },

        _renderWrapper: function () {
            this.element.addClass("e-input").attr({ 'aria-atomic': 'true', 'aria-live': 'assertive', "tabindex": "0", "value": this.model.value });
            this.wrapper = ej.buildTag("span.e-datetime-wrap e-widget " + this.model.cssClass + "#" + this.element[0].id + "_wrapper").insertAfter(this.element);
            this.container = ej.buildTag("span.e-in-wrap e-box").append(this.element);
            this.wrapper.append(this.container);
        },
        _renderIcon: function () {
            if (!this.model.showPopupButton) return false;
            this.datetimeIcon = ej.buildTag("span.e-select");
            var icon = ej.buildTag("span.e-icon e-datetime", "", {}, {"aria-label":"select"});
            this.datetimeIcon.append(icon);
            this.container.append(this.datetimeIcon).addClass("e-padding");
            this._on(this.datetimeIcon, "click", this._iconClick);
            this._on(this.datetimeIcon, "mousedown", function (e) { e.preventDefault(); });
        },
        _setDimentions: function () {
            if (this.model.height) this.wrapper.height(this.model.height);
            if (this.model.width) this.wrapper.width(this.model.width);
        },

        _renderDropdown: function () {
			var oldWrapper = $("#" + this.element.context.id + "_popup").get(0);
            if (oldWrapper)
                $(oldWrapper).remove();
            this.popup = ej.buildTag("div.e-datetime-popup e-popup e-widget e-box " + this.model.cssClass + "#" + this.element[0].id + "_popup").css("visibility", "hidden");
            $('body').append(this.popup);
            this._renderControls();
            var _timeTitle, _dateContainer, _timeContainer, popupContainer, _today, _now, _done, buttonContainer;

            _timeTitle = ej.buildTag("div.e-header", this.model.buttonText.timeTitle);
            _dateContainer = ej.buildTag("div.e-datecontainer").append(this.datePicker.popup);
            _timeContainer = ej.buildTag("div.e-timecontainer").append(_timeTitle, this.timePicker.popup);
            popupContainer = ej.buildTag("div.e-popup-container").append(_dateContainer, _timeContainer);

            _today = ej.buildTag("div.e-dt-button e-dt-today", this.model.buttonText.today);
            _now = ej.buildTag("div.e-dt-button e-dt-now", this.model.buttonText.now);
            _done = ej.buildTag("div.e-dt-button e-dt-done", this.model.buttonText.done);
            buttonContainer = ej.buildTag("div.e-button-container").append(_today, _now, _done);

            this.popup.append(popupContainer, buttonContainer);
            this._updateTimeHeight();

            this._bindOperations();
            this._updateValues();
            this.popup.css({ "visibility": "visible", "display": "none" });

            this._on(_today, "click", this._todayClick);
            this._on(_now, "click", this._nowClick);
            this._on(_done, "click", this._doneClick);
        },
        _renderControls: function () {
            this._renderDateControl();
            this._renderTimeControl();

            var tempContainer = ej.buildTag("span").append(this.datePicker.wrapper, this.timePicker.wrapper);
            tempContainer.find("span").css("display", "none");
            this.popup.append(tempContainer);
        },
        _renderDateControl: function () {
            var dateInput = ej.buildTag("input#" + this.element[0].id + "_date", "", {}, { "type": "text" });
            this.popup.append(dateInput);

            dateInput.ejDatePicker({
                // static values //
                height: "0px", width: "0px",
                displayInline: true,
                showDateIcon: false,
                showFooter: false,
                enableStrictMode: true,

                // default values //
                minDate: this._stringToObject(this.model.minDateTime),
                maxDate: this._stringToObject(this.model.maxDateTime),
                //value: this.model.value,
                dayHeaderFormat: this.model.dayHeaderFormat,
                startLevel: this.model.startLevel,
                depthLevel: this.model.depthLevel,
                startDay: this.model.startDay,
                stepMonths: this.model.stepMonths,
                showOtherMonths: this.model.showOtherMonths,
                headerFormat: this.model.headerFormat,
                buttonText: this.model.buttonText.today,

                // common values //
                enabled: this.model.enabled,
                enableRTL: this.model.enableRTL,
                showRoundedCorner: this.model.showRoundedCorner,
                readOnly: this.model.readOnly,
                cssClass: this.model.cssClass,
                locale: this.model.locale
            });
            this.datePicker = dateInput.data("ejDatePicker");
            this.datePicker._getInternalEvents = true;
            this.datePicker.popup.css({ "position": "static", "display": "block" });
        },
        _renderTimeControl: function () {
            var timeInput = ej.buildTag("input#" + this.element[0].id + "_time", "", {}, { "type": "text" });
            this.popup.append(timeInput);

            timeInput.ejTimePicker({
                // static values //
                height: "0px", width: "0px",

                // default values //
                interval: this.model.interval,
                //value: this.model.value,
                timeFormat: this.model.timeDisplayFormat,
                popupWidth: this.model.timePopupWidth,

                // common values //
                enabled: this.model.enabled,
                enableRTL: this.model.enableRTL,
                showRoundedCorner: this.model.showRoundedCorner,
                readOnly: this.model.readOnly,
                cssClass: this.model.cssClass,
                locale: this.model.locale
            });
            this.timePicker = timeInput.data("ejTimePicker");
            this.timePicker._getInternalEvents = true;
            this.timePicker.showDropdown = true;
            this.timePicker.popup.css({ "position": "static", "display": "block" });
        },
        _updateTimeHeight: function () {
            var height = this.datePicker.popup.outerHeight() - this.popup.find(".e-header").outerHeight() - 2;
            this.timePicker.option("popupHeight", height);
        },

        _bindOperations: function () {
            var proxy = this;
            this.datePicker.option("layoutChange", function () { proxy._updateTimeHeight(); });
            this.datePicker.option("outOfRange", function () { proxy.isValidState = false; });
            this.timePicker.option("outOfRange", function () { proxy.isValidState = false; });
            this.datePicker.option("change", function (a) { proxy._refreshTimes(a); });
            this.datePicker.option("select", function (e) { proxy._updateInput(e); });
            this.timePicker.option("select", function () { proxy._updateInput(); });
        },
        _updateInput: function ( e) {
            var date = this._getDate() || new Date(), time = this._getTime() || new Date();
			this.model.value = new Date(date.getFullYear(), date.getMonth(), date.getDate(),
                time.getHours(), time.getMinutes(), time.getSeconds());
            this._preVal = this._objectToString(this.model.value);
			this._updateDateTime();
            this._raiseChangeEvent();
            if(e)
            e.cancel = true;
        },
        _updateDateTime: function () {
            this.isValidState = true;
            var datetime = this._objectToString(this.model.value);
            this.element.val(datetime);
        },
        _refreshTimes: function (args) {
            var value = this._getDateObj(args.value, this.datePicker.model.dateFormat);
            if (!value) return false;
            this.isValidState = true;

            if (this._compare(value, this._setEmptyTime(this.model.minDateTime))) {
                var mintime = this._getFormat(this.model.minDateTime, "HH:mm:ss");
                var preTime = this._getTime();
                this.timePicker.option("minTime", mintime);
                if (!this._compare(preTime, this._getTime())) {
                    this.timePicker.model.value = this._getFormat(preTime, this.timePicker.model.timeFormat);
                    this.isValidState = false;
                }
            }
            else this.timePicker.option("minTime", "12:00:00 AM");

            if (this._compare(value, this._setEmptyTime(this.model.maxDateTime))) {
                var maxtime = this._getFormat(this.model.maxDateTime, "HH:mm:ss");
                var preTime = this._getTime();
                this.timePicker.option("maxTime", maxtime);
                if (!this._compare(preTime, this._getTime())) {
                    this.timePicker.model.value = this._getFormat(preTime, this.timePicker.model.timeFormat);
                    this.isValidState = false;
                }
            }
            else this.timePicker.option("maxTime", "11:59:59 PM");
        },

        _updateValues: function () {
            this._setValue(this.model.value);

            this.datePicker.option("value", this.model.value);
            this.timePicker.option("value", this.model.value);
            this._validateMinMax();
            this._preVal = this.element.val();
            this._checkErrorClass();
        },
        _setValue: function (value) {
            if (value == null || (typeof JSON === "object" && JSON.stringify(value) === "{}")) {
                this.element.val("");
                this.model.value = null;
                this.isValidState = true;
            }
            else if (typeof value === "string") {
                this.element.val(value);
                this._updateModel();
				this._validateMinMax();
				this._checkStrictMode();
            }
            else if (value instanceof Date && this._isValidDate(value)) {
                this.model.value = value;
                this._updateDateTime();
				this._validateMinMax();
				this._checkStrictMode();
            }

            this._checkErrorClass();
            return this.model.value;
        },
        _validateValue: function (value) {
            var dateObj = this._stringToObject(value);
            if (!dateObj) {
                this.model.value = value;
                this.isValidState = false;
            }
            else {
                this.isValidState = true;
                this.model.value = dateObj;
            }
        },
        _validateMinMax: function () {
            var value, min, max;
            value = (this.model.value) ? this._stringToObject(this.model.value) : null;
            min = (this.model.minDateTime) ? this._stringToObject(this.model.minDateTime) : this.defaults.minDateTime;
            max = (this.model.maxDateTime) ? this._stringToObject(this.model.maxDateTime) : this.defaults.maxDateTime;
            if (!value || !min || !max) return false;

            if (min > max) this.model.minDateTime = this.model.maxDateTime;
            if (value < min || value > max) this.isValidState = false;
            else this.isValidState = true;
        },

        _checkProperties: function () {
            this.model.readOnly && this._readOnly(true);
            this.model.showRoundedCorner && this._setRoundedCorner(true);
            this.model.enableRTL && this._setRtl(true);
            this.model.enabled && this._enabled(true);
            if (!this.element.attr("name")) this.element.attr({ "name": this.element[0].id });
			this._checkStrictMode();
            this._checkErrorClass();
        },
        
		_checkStrictMode: function () {
			if(!this.model.enableStrictMode){
				if(!this.isValidState){
					if(this.model.value < this.model.minDateTime){
						this.element.val(this._objectToString(this.model.minDateTime));
						this.model.value = this.model.minDateTime;
						this.isValidState = true;
					}
					else if(this.model.value > this.model.maxDateTime){
						this.element.val(this._objectToString(this.model.maxDateTime));
						this.model.value = this.model.maxDateTime;
						this.isValidState = true;
					}
					else {
						this.model.value = "";
						this.element.val("");
						this.isValidState = true;
					}
				}
			}
		},
		
        _targetFocus: function (e) {
            e.preventDefault();
            this.isFocused = true;
            this.wrapper.addClass("e-focus");
            this.wrapper.removeClass("e-error");
			this._prevDateTimeVal = this.element.val();
            if (!this.model.showPopupButton && !this.model.readOnly) this._showResult();
            this._trigger("focusIn", { value: this.model.value });
        },
        _targetBlur: function () {
            this.isFocused = false;
            this.wrapper.removeClass("e-focus");
            if (!this.model.showPopupButton) this._hideResult();
			if(!this.model.enableStrictMode){
				if(!this.isValidState){
					this.element.val(this._prevDateTimeVal);
					this._preVal = this._prevDateTimeVal;
					this.model.value = this._stringToObject(this._prevDateTimeVal);
					this.isValidState = true;
				}
				else
					this._prevDateTimeVal = this.element.val();
			}
			this._checkErrorClass();
            this._trigger("focusOut", { value: this.model.value });
        },

        _keyDownOnInput: function (e) {
            switch (e.keyCode) {
                case 40:        // Down Key
                    if (e.altKey) this._showhidePopup();
                    break;
                case 37:        // Left Key
                case 39:         // Right Key
                    if (e.altKey && this.isPopupOpen) {
                        e.preventDefault();
                        this._addPrevNextFocus(e.keyCode == 37);
                    }
                    break;
                case 27:        // Esc Key
                    e.preventDefault();
                case 9:         // Tab Key
                    this._hideResult();
                    break;
            }
        },
        _addFocus: function (target) {
            if (!target.hasClass("e-focus")) {
                this._removeFocus();
                target.addClass("e-focus");
                if (target.hasClass("e-datecontainer"))
                    $(document).bind("keydown", $.proxy(this.datePicker._keyboardNavigation, this.datePicker));
                else if (target.hasClass("e-timecontainer"))
                    $(document).bind("keydown", $.proxy(this.timePicker._keyDownOnInput, this.timePicker));
                else if (target.hasClass("e-dt-button"))
                    $(document).bind("keydown", $.proxy(this._buttonClick, this));
            }
        },
        _removeFocus: function () {
            var target = this._getFocusedElement();
            if (target.length > 0) {
                target.removeClass("e-focus");
                if (target.hasClass("e-datecontainer"))
                    $(document).unbind("keydown", $.proxy(this.datePicker._keyboardNavigation, this.datePicker));
                else if (target.hasClass("e-timecontainer"))
                    $(document).unbind("keydown", $.proxy(this.timePicker._keyDownOnInput, this.timePicker));
                else if (target.hasClass("e-dt-button"))
                    $(document).unbind("keydown", $.proxy(this._buttonClick, this));
            }
        },
        _addPrevNextFocus: function (flag) {
            // flag true means previous focus, false means next focus
            var target = this._getFocusedElement(), next;
            if (target.length > 0) {
                if (target.hasClass("e-datecontainer"))
                    next = flag ? this.popup.find(".e-dt-done") : this.popup.find(".e-timecontainer");
                else if (target.hasClass("e-timecontainer"))
                    next = flag ? this.popup.find(".e-datecontainer") : this.popup.find(".e-dt-today");
                else if (target.hasClass("e-dt-today"))
                    next = flag ? this.popup.find(".e-timecontainer") : this.popup.find(".e-dt-now");
                else if (target.hasClass("e-dt-now"))
                    next = flag ? this.popup.find(".e-dt-today") : this.popup.find(".e-dt-done");
                else if (target.hasClass("e-dt-done"))
                    next = flag ? this.popup.find(".e-dt-now") : this.popup.find(".e-datecontainer");
            }
            else next = flag ? this.popup.find(".e-dt-done") : this.popup.find(".e-datecontainer");
            this._addFocus(next);
        },
        _getFocusedElement: function () {
            return this.popup.children("div").children("div.e-focus");
        },

        _valueChange: function () {
            if (this._preVal != this.element.val()) {
                this._preVal = this.element.val();
                this._updateModel();
                this._validateMinMax();
                this._raiseChangeEvent();
            }
        },
        _updateModel: function () {
            var value = this.element.val();
            if (value == "") {
                this.model.value = null;
                this.isValidState = true;
            }
            else {
                var dateObj = this._stringToObject(value);
                if (dateObj) {
                    this.model.value = dateObj;
                    this.isValidState = true;
                    this._refreshPopup();
                }
                else {
                    this.model.value = value;
                    this.isValidState = false;
                }
            }
        },
        _refreshPopup: function () {
            if (this.isValidState && this.isPopupOpen) {
                var date = this._setEmptyTime(this.model.value), time = this._setEmptyDate(this.model.value);
                var getDate = this._getDate(), getTime = this._getTime();
                if (!getDate || !this._compare(getDate, date)) this.datePicker.option("value", date);
                if (!getTime || !this._compare(getTime, time)) this.timePicker.option("value", time);
            }
        },

        _buttonClick: function (e) {
            if (e.keyCode == 13) {
                var target = this._getFocusedElement();
                if (target.hasClass("e-dt-today"))
                    this._todayClick();
                else if (target.hasClass("e-dt-now"))
                    this._nowClick();
                else if (target.hasClass("e-dt-done"))
                    this._doneClick();
            }
        },
        _todayClick: function () {
            if (!this.model.enabled || this.model.readOnly) return false;
            if (!this.datePicker.popup.find(".today").hasClass("e-active") ||
                !this.datePicker.popup.children("table").hasClass("e-dp-viewdays")||
                this.element.val() == "" || !this.isValidState)
            {
                this.datePicker._setCurrDate();
                this._updateInput();
            }
        },
        _nowClick: function () {
            if (!this.model.enabled || this.model.readOnly) return false;
            this.timePicker.setCurrentTime();
            this._updateInput();
        },
        _doneClick: function () {
            this._hideResult();
        },

        _iconClick: function (e) {
            e.preventDefault();
            if (!this.isFocused) this.element.focus();
            this._showhidePopup();
        },
        _showhidePopup: function () {
            if (this.model.readOnly) return false;
            if (!this.isPopupOpen)
                this._showResult();
            else
                this._hideResult();
        },
        _showResult: function () {
            if (this.isPopupOpen || !this.model.enabled) return false;
            if (this._raiseEvent("open")) return false;
            this.isPopupOpen = true;
            this._setListPosition();
            var proxy = this;
            this.popup.slideDown(this.model.enableAnimation?200:0, "easeOutQuad", function () {
                proxy._on($(document), "mousedown", proxy._OnDocumentClick);
            });
            this._updateModel();
            this._updateTimeHeight();
            this._validateMinMax();
            this._on($(window), "resize", this._OnWindowResize);
        },
        _hideResult: function () {
            if (!this.isPopupOpen) return false;
            if (this._raiseEvent("close")) return false;
            this.isPopupOpen = false;
            this._removeFocus();
            this.popup.slideUp(this.model.enableAnimation?100:0, "easeOutQuad");
            this._off($(document), "mousedown", this._OnDocumentClick);
            this._off($(window), "resize", this._OnWindowResize);
        },
        _setListPosition: function () {
            var pos = this._getOffset(this.wrapper),
            left = pos.left,
            top = pos.top + this.wrapper.outerHeight() + 3 + "px",
            maxZ = this._getZindexPartial();
            if (this.model.enableRTL) left -= this.popup.outerWidth() - this.wrapper.outerWidth();

            this.popup.css({ "left": left + "px", "top": top, "z-index": maxZ });
        },
        _getOffset: function (ele) {
            var pos = ele.offset();
            if ($("body").css("position") != "static") {
                var bodyPos = $("body").offset();
                pos.left -= bodyPos.left;
                pos.top -= bodyPos.top;
            }
            return pos;
        },
       
        _OnDocumentClick: function (e) {
            if (!$(e.target).is(this.popup) && !$(e.target).parents(".e-datetime-popup").is(this.popup) &&
                !$(e.target).is(this.wrapper) && !$(e.target).parents(".e-datetime-wrap").is(this.wrapper)) {
                this._hideResult();
            }
            else if ($(e.target).is(this.popup) || $(e.target).parents(".e-datetime-popup").is(this.popup)) {
                e.preventDefault();
                if ($(e.target).parents(".e-datecontainer").length > 0) this._addFocus($(e.target).parents(".e-datecontainer"));
                else if ($(e.target).parents(".e-timecontainer").length > 0) this._addFocus($(e.target).parents(".e-timecontainer"));
                else if ($(e.target).hasClass("e-dt-button")) this._addFocus($(e.target));
                else this._removeFocus();
            }
        },
        _OnWindowResize: function (e) {
            this._setListPosition();
        },

        _raiseChangeEvent: function () {
            if (this.element != null && this.model.change)
                this._trigger("change", {
                    prevDateTime: this._preVal,
                    value: this.element.val(),
                    isValidState: this.isValidState
                });
        },
        _raiseEvent: function (name) {
            if (this.element != null && this.model[name]) return this._trigger(name, { prevDateTime: this._preVal, value:this.element.val() });
            return false;
        },
        _getDateTimeFormat: function () {
            var pattern = Globalize.culture(this.model.locale).calendar.patterns;

            if (!this.model.dateTimeFormat) this.model.dateTimeFormat = pattern.d + " " + pattern.t;
            if (!this.model.timeDisplayFormat) this.model.timeDisplayFormat = pattern.t;
        },
        _getZindexPartial: function () {
            var parents = this.element.parents(), bodyEle;
            bodyEle = $('body').children(), index = bodyEle.index(this.popup);
            bodyEle.splice(index, 1);
            $(bodyEle).each(function (i, ele) { parents.push(ele); });

            var maxZ = Math.max.apply(maxZ, $.map(parents, function (e, n) {
                if ($(e).css('position') != 'static') return parseInt($(e).css('z-index')) || 1;
            }));
            if (maxZ == undefined || maxZ == null || maxZ < 10000) maxZ = 10000;
            else maxZ += 1;
            return maxZ;
        },
        _checkErrorClass: function () {
            if (this.isValidState) this.wrapper.removeClass("e-error");
            else this.wrapper.addClass("e-error");
        },
        _getDate: function () {
            return this.datePicker.model.value;
        },
        _getTime: function () {
            return this._getDateObj(this.timePicker.model.value, this.timePicker.model.timeFormat);
        },
        _setEmptyTime: function (date) {
            var newDate = new Date(date);
            newDate.setMilliseconds(0);
            newDate.setSeconds(0);
            newDate.setMinutes(0);
            newDate.setHours(0);
            return newDate;
        },
        _setEmptyDate: function (date) {
            var newDate = new Date(date);
            newDate.setDate(1);
            newDate.setMonth(0);
            newDate.setFullYear(2000);
            return newDate;
        },
        _objectToString: function (obj) {
            return this._getFormat(obj, this.model.dateTimeFormat);
        },
        _stringToObject: function (value) {
            return this._getDateObj(value, this.model.dateTimeFormat);
        },
        _getFormat: function (value, format) {
            if (value instanceof Date)
                return Globalize.format(value, format, this.model.locale);
            else return value;
        },
        _getDateObj: function (value, format) {
            if (typeof value === "string")
            {
                var temp = Globalize.parseDate(value, format);
                if(temp != null)
                	return temp;
                else
                {
                	var validDateObj = new Date(value);
                	if(this._isValidDate(validDateObj))
	                	return validDateObj; 
	                else return null;
                }
            }
            else return value;
        },
        _compare: function (obj1, obj2) {
            return obj1 && obj2 && obj1.getTime() == obj2.getTime();
        },
        _isValidDate: function (dateObj) {
            return dateObj && typeof dateObj.getTime === "function" && isFinite(dateObj.getTime());
        },

        //-------------------- Private Methods for setModel -------------------------//
        _change: function (property, value) {
            this.datePicker.option(property, value);
            this.timePicker.option(property, value);
        },
        _changeSkin: function (skin) {
            this.wrapper.removeClass(this.model.cssClass).addClass(skin);
            this.popup.removeClass(this.model.cssClass).addClass(skin);

            this._change("cssClass", skin);
        },
        _localize: function (culture) {
            this.model.locale = culture;
            this.model.dateTimeFormat = this.model.timeDisplayFormat = "";
            this._getDateTimeFormat();
            if (this.isValidState || (this.model.value instanceof Date && this._isValidDate(this.model.value)))
                this.element.val(this._objectToString(this.model.value));
            this._preVal = this.element.val();
            this._change("locale", culture);
        },
        _readOnly: function (boolean) {
            if (boolean) this.element.attr("readonly", "readonly");
            else this.element.removeAttr("readonly");

            this._change("readOnly", boolean);
        },
        _setRoundedCorner: function (boolean) {
            if (boolean) {
                this.container.addClass("e-corner-all");
                this.popup.addClass("e-corner-all");
                this.popup.find(".e-dt-button").addClass("e-corner-all");
            }
            else {
                this.container.removeClass("e-corner-all");
                this.popup.removeClass("e-corner-all");
                this.popup.find(".e-dt-button").removeClass("e-corner-all");
            }
            this.datePicker.option("showRoundedCorner", boolean);
        },
        _setRtl: function (boolean) {
            if (boolean) {
                this.wrapper.addClass("e-rtl");
                this.popup.addClass("e-rtl");
            }
            else {
                this.wrapper.removeClass("e-rtl");
                this.popup.removeClass("e-rtl");
            }

            this._change("enableRTL", boolean);
        },
        _enabled: function (boolean) {
            if (boolean) this.enable();
            else {
                this.model.enabled = true; // for initial rendering
                this.disable();
            }
        },
        _showButton: function (show) {
            this.model.showPopupButton = show;
            if (show) {
                this.container.addClass("e-padding");
                this._renderIcon();
            }
            else {
                this.container.removeClass("e-padding");
                this.datetimeIcon.remove();
                this.datetimeIcon = null;
            }
        },
        _buttonText: function (data) {
            $.extend(this.model.buttonText, data);
            this.popup.find(".e-dt-today").html(this.model.buttonText.today);
            this.popup.find(".e-dt-now").html(this.model.buttonText.now);
            this.popup.find(".e-dt-done").html(this.model.buttonText.done);
            this.popup.find(".e-timecontainer").find(".e-header").html(this.model.buttonText.timeTitle);
        },

        //-------------------- Public Methods -------------------------//

        /**
        * Enables the DateTimePicker control.
		* @return jQuery
		* @example 
		* &lt;input type="text" id="datetime" /&gt;
		* &lt;script&gt;
		* $("#datetime").ejDateTimePicker();
		* // Create DateTimePicker instance
		* var datetimeObj = $("#datetime").data("ejDateTimePicker");
		* datetimeObj.enable(); // enables the datetimepicker
		* &lt;/script&gt;
		* @example 
	    * &lt;input type="text" id="datetime" /&gt;
	    * &lt;script&gt;
		* $("#datetime").ejDateTimePicker();
	    * // enables the datetimepicker
        * $("#datetime").ejDateTimePicker("enable");
	    * &lt;/script&gt;
		*@memberof ejDateTimePicker
		* @instance
        */
        enable: function () {
            if (!this.model.enabled) {
                this.element[0].disabled = false;
                this.model.enabled = true;
                this.element.removeClass("e-disable").attr("aria-disabled", false);
                if (this.datetimeIcon) this.datetimeIcon.removeClass("e-disable").attr("aria-disabled", false);
                this.popup.children("div").removeClass("e-disable").attr("aria-disabled", false);
                this._change("enabled", true);
            }
        },

        /**
        * Disables the DateTimePicker control.
		* @return jQuery
		* @example 
		* &lt;input type="text" id="datetime" /&gt;
		* &lt;script&gt;
		* $("#datetime").ejDateTimePicker();
		* // Create DateTimePicker instance
		* var datetimeObj = $("#datetime").data("ejDateTimePicker");
		* datetimeObj.disable(); // disables the datetimepicker
		* &lt;/script&gt;
		* @example 
	    * &lt;input type="text" id="datetime" /&gt;
	    * &lt;script&gt;
		* $("#datetime").ejDateTimePicker();
	    * // disables the datetimepicker
        * $("#datetime").ejDateTimePicker("disable");
	    * &lt;/script&gt;
		*@memberof ejDateTimePicker
		* @instance
        */
        disable: function () {
            if (this.model.enabled) {
                this.element[0].disabled = true;
                this.model.enabled = false;
                this.element.addClass("e-disable").attr("aria-disabled", true);
                if (this.datetimeIcon) this.datetimeIcon.addClass("e-disable").attr("aria-disabled", true);
                this.popup.children("div").addClass("e-disable").attr("aria-disabled", true);
                this._hideResult();

                this._change("enabled", false);
                this.datePicker.popup.removeClass("e-disable").attr("aria-disabled", false);
                this.timePicker.popup.removeClass("e-disable").attr("aria-disabled", false);
            }
        },

        /**
        * Returns the current datetime value in the DateTimePicker.
		* @return datetime value
		* @example 
		* &lt;input type="text" id="datetime" /&gt;
		* &lt;script&gt;
		* $("#datetime").ejDateTimePicker();
		* // Create DateTimePicker instance
		* var datetimeObj = $("#datetime").data("ejDateTimePicker");
		* datetimeObj.getValue(); // returns the datetime value
		* &lt;/script&gt;
		* @example 
	    * &lt;input type="text" id="datetime" /&gt;
	    * &lt;script&gt;
		* $("#datetime").ejDateTimePicker();
	    * // returns the datetime value
        * $("#datetime").ejDateTimePicker("getValue");
	    * &lt;/script&gt;
		*@memberof ejDateTimePicker
		* @instance
        */
        getValue: function () {
            return this._objectToString(this.model.value);
        },

        /**
        * Updates the current system date value and time value to the DateTimePicker.
		* @return jQuery
		* @example 
		* &lt;input type="text" id="datetime" /&gt;
		* &lt;script&gt;
		* $("#datetime").ejDateTimePicker();
		* // Create DateTimePicker instance
		* var datetimeObj = $("#datetime").data("ejDateTimePicker");
		* datetimeObj.setCurrentDateTime(); // updates the current datetime value
		* &lt;/script&gt;
		* @example 
	    * &lt;input type="text" id="datetime" /&gt;
	    * &lt;script&gt;
		* $("#datetime").ejDateTimePicker();
	    * // updates the current datetime value
        * $("#datetime").ejDateTimePicker("setCurrentDateTime");
	    * &lt;/script&gt;
		*@memberof ejDateTimePicker
		* @instance
        */
        setCurrentDateTime: function () {
            if (this.model.enabled && !this.model.readOnly)
                this._setValue(new Date());
        },

        /**
        * Shows or opens the DateTimePicker popup.
		* @return jQuery
		* @example 
		* &lt;input type="text" id="datetime" /&gt;
		* &lt;script&gt;
		* $("#datetime").ejDateTimePicker();
		* // Create DateTimePicker instance
		* var datetimeObj = $("#datetime").data("ejDateTimePicker");
		* datetimeObj.show(); // opens the datetime popup
		* &lt;/script&gt;
		* @example 
	    * &lt;input type="text" id="datetime" /&gt;
	    * &lt;script&gt;
		* $("#datetime").ejDateTimePicker();
	    * // opens the datetime popup
        * $("#datetime").ejDateTimePicker("show");
	    * &lt;/script&gt;
		*@memberof ejDateTimePicker
		* @instance
        */
        show: function () {
            this._showResult();
        },

        /**
        * Hides or closes the DateTimePicker popup.
		* @return jQuery
		* @example 
		* &lt;input type="text" id="datetime" /&gt;
		* &lt;script&gt;
		* $("#datetime").ejDateTimePicker();
		* // Create DateTimePicker instance
		* var datetimeObj = $("#datetime").data("ejDateTimePicker");
		* datetimeObj.hide(); // hides the datetime popup
		* &lt;/script&gt;
		* @example 
	    * &lt;input type="text" id="datetime" /&gt;
	    * &lt;script&gt;
		* $("#datetime").ejDateTimePicker();
	    * // hides the datetime popup
        * $("#datetime").ejDateTimePicker("hide");
	    * &lt;/script&gt;
		*@memberof ejDateTimePicker
		* @instance
        */
        hide: function () {
            this._hideResult();
        },

        //-------------------- Event Wire-up -------------------------//
        _wireEvents: function () {
            this._on(this.element, "focus", this._targetFocus);
            this._on(this.element, "blur", this._targetBlur);
            this._on(this.element, "keydown", this._keyDownOnInput);
            this._on(this.element, "keyup", this._valueChange);
        }
    });
})(jQuery, Syncfusion);;
/**
* @fileOverview Plugin to style the Dialog control
* @copyright Copyright Syncfusion Inc. 2001 - 2013. All rights reserved.
*  Use of this code is subject to the terms of our license.
*  A copy of the current license can be obtained at any time by e-mailing
*  licensing@syncfusion.com. Any infringement will be prosecuted under
*  applicable laws. 
* @version 12.1 
* @author <a href="mailto:licensing@syncfusion.com">Syncfusion Inc</a>
*/
(function ($, ej, undefined) {
    /**
       * @namespace ej
       * @class ejDialog
       * @param {object} options - settings for Dialog.
       * @requires jQuery
       * @requires jquery.easing.1.3.js
       * @requires ej.core.js
       * @requires ej.dialog.js
       * @requires ej.scroller.js
       * @requires ej.draggable.js
       * @classdesc Custom Design for Html Dialog control.
       *@example
       * &lt;div id="dialog" title="WPF"&gt;
       *Developed by Microsoft, the Windows Presentation Foundation (or WPF) is a computer-software graphical subsystem for rendering user interfaces in Windows-based applications. WPF, previously known as "Avalon", was initially released as part of .NET Framework 3.0. Rather than relying on the older GDI subsystem, WPF uses DirectX. WPF attempts to provide a consistent programming model for building applications and separates the user interface from business logic. It resembles similar XML-oriented object models, such as those implemented in XUL and SVG.
       * &lt;/div&gt;
       * &lt;script&gt;
       * // Create Dialog control
       *  $("#dialog").ejDialog({position: { X: 300, Y: 10 }}); 	
       * &lt;/script&gt;
       */
    ej.widget("ejDialog", "ej.Dialog", {
        _rootCSS: "e-dialog",
        // widget element will be automatically set in this
        element: null,

        // user defined model will be automatically set in this
        model: null,
        validTags: ["div", "span"],
        _setFirst: false,

        // default model
        defaults: {
            /// <summary>This Contains default property of Sycnfusion Dialog </summary>
            
			/**
			* Enables the Dialog window to open automatically.
			* @default true
			* @type {Boolean}
			* @example 
			* &lt;div id="dialog" title="WPF"&gt;
	*Developed by Microsoft, the Windows Presentation Foundation (or WPF) is a computer-software graphical subsystem for rendering user interfaces in Windows-based applications. WPF, previously known as "Avalon", was initially released as part of .NET Framework 3.0. Rather than relying on the older GDI subsystem, WPF uses DirectX. WPF attempts to provide a consistent programming model for building applications and separates the user interface from business logic. It resembles similar XML-oriented object models, such as those implemented in XUL and SVG.
	* &lt;/div&gt;
	* &lt;script&gt;
			* // Enables the Dialog window to open automatically during initialization  
			* 	 $("#dialog").ejDialog({  position: { X: 500, Y: 26 },showOnInit:true});				
			* &lt;/script&gt;
			 * @memberof ejDialog
			* @instance
			*/	
           
			showOnInit: true,
			/**		
			* Allows the Dialog window to be closed by pressing the Esc key.
			* @default true
			* @type {Boolean}
			* @example 
			* &lt;div id="dialog" title="WPF"&gt;
	*Developed by Microsoft, the Windows Presentation Foundation (or WPF) is a computer-software graphical subsystem for rendering user interfaces in Windows-based applications. WPF, previously known as "Avalon", was initially released as part of .NET Framework 3.0. Rather than relying on the older GDI subsystem, WPF uses DirectX. WPF attempts to provide a consistent programming model for building applications and separates the user interface from business logic. It resembles similar XML-oriented object models, such as those implemented in XUL and SVG.
	* &lt;/div&gt;
	* &lt;script&gt;
			* // Enable closeOnEscape during initialization  
			* 	 $("#dialog").ejDialog({position: { X: 300, Y: 10 }, closeOnEscape: false});				 
			* &lt;/script&gt;
			 * @memberof ejDialog
			* @instance
			*/	
            closeOnEscape: true,
			/**		
			* Customizes the close button tooltip text.
			* @default close
			* @type {String}
			* @example 
			* &lt;div id="dialog" title="WPF"&gt;
	*Developed by Microsoft, the Windows Presentation Foundation (or WPF) is a computer-software graphical subsystem for rendering user interfaces in Windows-based applications. WPF, previously known as "Avalon", was initially released as part of .NET Framework 3.0. Rather than relying on the older GDI subsystem, WPF uses DirectX. WPF attempts to provide a consistent programming model for building applications and separates the user interface from business logic. It resembles similar XML-oriented object models, such as those implemented in XUL and SVG.
	* &lt;/div&gt;
	* &lt;script&gt;
			* //To set closeText API value during initialization  
			* 	$("#dialog").ejDialog({position: { X: 300, Y: 10 },closeIconTooltip: "hide" });				
			* &lt;/script&gt;
			 * @memberof ejDialog
			* @instance
			*/	
           closeIconTooltip: "close",
			enableAnimation: true,
				/**		
			* To enable the dialog window to dragged within the page.
			* @default true
			* @type {Boolean}
			* @example 
			* &lt;div id="dialog" title="WPF"&gt;
	*Developed by Microsoft, the Windows Presentation Foundation (or WPF) is a computer-software graphical subsystem for rendering user interfaces in Windows-based applications. WPF, previously known as "Avalon", was initially released as part of .NET Framework 3.0. Rather than relying on the older GDI subsystem, WPF uses DirectX. WPF attempts to provide a consistent programming model for building applications and separates the user interface from business logic. It resembles similar XML-oriented object models, such as those implemented in XUL and SVG.
	* &lt;/div&gt;
	* &lt;script&gt;
			* //Enable draggable during initialization  
			* 	$("#dialog").ejDialog({position: { X: 300, Y: 10 }, allowDraggable: false});					
			* &lt;/script&gt;
			 * @memberof ejDialog
			* @instance
			*/
            allowDraggable: true,
					/**		
			* Set the height for the dialog control.
			* @default auto
			* @type {String | integer}
			* @example 
			* &lt;div id="dialog" title="WPF"&gt;
	*Developed by Microsoft, the Windows Presentation Foundation (or WPF) is a computer-software graphical subsystem for rendering user interfaces in Windows-based applications. WPF, previously known as "Avalon", was initially released as part of .NET Framework 3.0. Rather than relying on the older GDI subsystem, WPF uses DirectX. WPF attempts to provide a consistent programming model for building applications and separates the user interface from business logic. It resembles similar XML-oriented object models, such as those implemented in XUL and SVG.
	* &lt;/div&gt;
	* &lt;script&gt;
			* //Set Height during initialization  
			* 	$("#dialog").ejDialog({position: { X: 300, Y: 10 }, height: 400 });				
			* &lt;/script&gt;
			 * @memberof ejDialog
			* @instance
			*/
            height: "auto",
			/**		
			* Set the minimum height for the dialog control.
			* @default 150
			* @type {Integer}
			* @example 
			* &lt;div id="dialog" title="WPF"&gt;
	*Developed by Microsoft, the Windows Presentation Foundation (or WPF) is a computer-software graphical subsystem for rendering user interfaces in Windows-based applications. WPF, previously known as "Avalon", was initially released as part of .NET Framework 3.0. Rather than relying on the older GDI subsystem, WPF uses DirectX. WPF attempts to provide a consistent programming model for building applications and separates the user interface from business logic. It resembles similar XML-oriented object models, such as those implemented in XUL and SVG.
	* &lt;/div&gt;
	* &lt;script&gt;
			* //Set minimum Height during initialization  
			* 	$("#dialog").ejDialog({position: { X: 300, Y: 10 }, minHeight: 400 });				
			* &lt;/script&gt;
			 * @memberof ejDialog
			* @instance
			*/
            minHeight: 0,
			/**		
			* Set the minimum width for the dialog control.
			* @default 150
			* @type {Integer}
			* @example 
			* &lt;div id="dialog" title="WPF"&gt;
	*Developed by Microsoft, the Windows Presentation Foundation (or WPF) is a computer-software graphical subsystem for rendering user interfaces in Windows-based applications. WPF, previously known as "Avalon", was initially released as part of .NET Framework 3.0. Rather than relying on the older GDI subsystem, WPF uses DirectX. WPF attempts to provide a consistent programming model for building applications and separates the user interface from business logic. It resembles similar XML-oriented object models, such as those implemented in XUL and SVG.
	* &lt;/div&gt;
	* &lt;script&gt;
			* //Set minimum Width during initialization  
			* 	 $("#dialog").ejDialog({position: { X: 300, Y: 10 }, minWidth: 400 });			
			* &lt;/script&gt;
			 * @memberof ejDialog
			* @instance
			*/
            minWidth: 0,
			/**		
			* Set the maximum height for the dialog control.
			* @default null
			* @type {Integer}
			* @example 
			* &lt;div id="dialog" title="WPF"&gt;
	*Developed by Microsoft, the Windows Presentation Foundation (or WPF) is a computer-software graphical subsystem for rendering user interfaces in Windows-based applications. WPF, previously known as "Avalon", was initially released as part of .NET Framework 3.0. Rather than relying on the older GDI subsystem, WPF uses DirectX. WPF attempts to provide a consistent programming model for building applications and separates the user interface from business logic. It resembles similar XML-oriented object models, such as those implemented in XUL and SVG.
	* &lt;/div&gt;
	* &lt;script&gt;
			* //Set maximum height during initialization  
			* 	  $("#dialog").ejDialog({position: { X: 300, Y: 10 }, maxHeight: 600 });		
			* &lt;/script&gt;
			 * @memberof ejDialog
			* @instance
			*/
            maxHeight: null,
			/**		
			* Set the maximum width for the dialog control.
			* @default 150
			* @type {Integer}
			* @example 
			* &lt;div id="dialog" title="WPF"&gt;
	*Developed by Microsoft, the Windows Presentation Foundation (or WPF) is a computer-software graphical subsystem for rendering user interfaces in Windows-based applications. WPF, previously known as "Avalon", was initially released as part of .NET Framework 3.0. Rather than relying on the older GDI subsystem, WPF uses DirectX. WPF attempts to provide a consistent programming model for building applications and separates the user interface from business logic. It resembles similar XML-oriented object models, such as those implemented in XUL and SVG.
	* &lt;/div&gt;
	* &lt;script&gt;
			* //Set maximum width during initialization  
			* 	   $("#dialog").ejDialog({ position: { X: 300, Y: 10 },maxWidth: 600 });
			* &lt;/script&gt;
			 * @memberof ejDialog
			* @instance
			*/
            maxWidth: null,
			/**		
			* Allows the dialog window to be used as a modal form.
			* @default false
			* @type {Boolean}
			* @example 
			* &lt;div id="dialog" title="WPF"&gt;
	*Developed by Microsoft, the Windows Presentation Foundation (or WPF) is a computer-software graphical subsystem for rendering user interfaces in Windows-based applications. WPF, previously known as "Avalon", was initially released as part of .NET Framework 3.0. Rather than relying on the older GDI subsystem, WPF uses DirectX. WPF attempts to provide a consistent programming model for building applications and separates the user interface from business logic. It resembles similar XML-oriented object models, such as those implemented in XUL and SVG.
	* &lt;/div&gt;
	* &lt;script&gt;
			* //Enable modal during initialization  
			* 	    $("#dialog").ejDialog({position: { X: 300, Y: 10 }, enableModal: true});
			* &lt;/script&gt;
			 * @memberof ejDialog
			* @instance
			*/
            enableModal: false,
			/**		
			* Displays the dialog window at the given X and Y position.
			* X: Dialog set the left position value.
			* Y: Dialog set the top position value.
			* @default Null
			* @type {JSONobject}
			* @example 
			* &lt;div id="dialog" title="WPF"&gt;
	*Developed by Microsoft, the Windows Presentation Foundation (or WPF) is a computer-software graphical subsystem for rendering user interfaces in Windows-based applications. WPF, previously known as "Avalon", was initially released as part of .NET Framework 3.0. Rather than relying on the older GDI subsystem, WPF uses DirectX. WPF attempts to provide a consistent programming model for building applications and separates the user interface from business logic. It resembles similar XML-oriented object models, such as those implemented in XUL and SVG.
	* &lt;/div&gt;
	* &lt;script&gt;
			* //Enable position during initialization  
			* 	    $("#dialog").ejDialog({position: { X: 300, Y: 10 }});
			* &lt;/script&gt;
			 * @memberof ejDialog
			* @instance
			*/
            position: { X: "", Y: ""},
			/**		
			* Allows the dialog to be resized above the minimum height and minimum width values.
			* @default true
			* @type {Boolean}
			* @example 
			* &lt;div id="dialog" title="WPF"&gt;
	*Developed by Microsoft, the Windows Presentation Foundation (or WPF) is a computer-software graphical subsystem for rendering user interfaces in Windows-based applications. WPF, previously known as "Avalon", was initially released as part of .NET Framework 3.0. Rather than relying on the older GDI subsystem, WPF uses DirectX. WPF attempts to provide a consistent programming model for building applications and separates the user interface from business logic. It resembles similar XML-oriented object models, such as those implemented in XUL and SVG.
	* &lt;/div&gt;
	* &lt;script&gt;
			* //Allows the dialog to be resized during initialization  
			* 	    $("#dialog").ejDialog({ position: { X: 300, Y: 10 },enableResize: false});
			* &lt;/script&gt;
			 * @memberof ejDialog
			* @instance
			*/
            enableResize: true,
			/**		
			* Shows or hides the dialog header element.
			* @default true
			* @type {Boolean}
			* @example 
			* &lt;div id="dialog" title="WPF"&gt;
	*Developed by Microsoft, the Windows Presentation Foundation (or WPF) is a computer-software graphical subsystem for rendering user interfaces in Windows-based applications. WPF, previously known as "Avalon", was initially released as part of .NET Framework 3.0. Rather than relying on the older GDI subsystem, WPF uses DirectX. WPF attempts to provide a consistent programming model for building applications and separates the user interface from business logic. It resembles similar XML-oriented object models, such as those implemented in XUL and SVG.
	* &lt;/div&gt;
	* &lt;script&gt;
			* // Enable showHeader on initialization.  
			* 	   $("#dialog").ejDialog({ position: { X: 300, Y: 10 },showHeader: false});
			* &lt;/script&gt;
			* @memberof ejDialog
			* @instance
			*/
            showHeader: true,
			/**		
			* To load the dialog content at run time, such as ajax, images, and iframe.
			* @default null
			* @type {String}
			* @example
			* &lt;div id="dialog" title="WPF"&gt;
	*Developed by Microsoft, the Windows Presentation Foundation (or WPF) is a computer-software graphical subsystem for rendering user interfaces in Windows-based applications. WPF, previously known as "Avalon", was initially released as part of .NET Framework 3.0. Rather than relying on the older GDI subsystem, WPF uses DirectX. WPF attempts to provide a consistent programming model for building applications and separates the user interface from business logic. It resembles similar XML-oriented object models, such as those implemented in XUL and SVG.
	* &lt;/div&gt;
	* &lt;script&gt; 
	*  $("#dialog").ejDialog({position: { X: 300, Y: 10 }}); 
			* //Enable content on initialization.  
			* 	   $("#dialog").ejDialog({ position: { X: 300, Y: 10 },contentType: "ajax" });
			* &lt;/script&gt;
			* @memberof ejDialog
			* @instance
			*/
            contentType: null, //'ajax', 'iframe','image'
			/**		
			* Loads AJAX, image, and iframe content from a given location (URL).
			* @default null
			* @type {String}
			* @example 
			* &lt;div id="dialog" title="WPF"&gt;	
	* &lt;/div&gt;
	* &lt;script&gt;
			* //loadUrl on initialization.  
			* 	   $("#dialog").ejDialog({ position: { X: 300, Y: 10 },contentType: "ajax",contentUrl: "http://js.syncfusion.com/demos/web/dialog/ajaxcontent.html" });
			* &lt;/script&gt;
			* @memberof ejDialog
			* @instance
			*/
            contentUrl: null,
			
            ajaxSettings: { type: 'GET', cache: false, url: "", data: {}, dataType: "html", contentType: "html", async: true },
 /**		
			*Provides title text for the Dialog control.
			* @default ""
			* @type {String}
			* @example 
			* &lt;div id="dialog" title="WPF"&gt;
	*Developed by Microsoft, the Windows Presentation Foundation (or WPF) is a computer-software graphical subsystem for rendering user interfaces in Windows-based applications. WPF, previously known as "Avalon", was initially released as part of .NET Framework 3.0. Rather than relying on the older GDI subsystem, WPF uses DirectX. WPF attempts to provide a consistent programming model for building applications and separates the user interface from business logic. It resembles similar XML-oriented object models, such as those implemented in XUL and SVG.
	* &lt;/div&gt;
	* &lt;script&gt;
			* //Title on initialization.  
			* 	    $("#dialog").ejDialog({position: { X: 300, Y: 10 }, title: "Custom title" });
			* &lt;/script&gt;
			* @memberof ejDialog
			* @instance
			*/
            title: "",
			/**		
			*Sets the width of the Dialog control.
			* @default ""
			* @type {String}
			* @example 
			* &lt;div id="dialog" title="WPF"&gt;
	*Developed by Microsoft, the Windows Presentation Foundation (or WPF) is a computer-software graphical subsystem for rendering user interfaces in Windows-based applications. WPF, previously known as "Avalon", was initially released as part of .NET Framework 3.0. Rather than relying on the older GDI subsystem, WPF uses DirectX. WPF attempts to provide a consistent programming model for building applications and separates the user interface from business logic. It resembles similar XML-oriented object models, such as those implemented in XUL and SVG.
	* &lt;/div&gt;
	* &lt;script&gt;
			* //Width on initialization.  
			* 	 $("#dialog").ejDialog({ position: { X: 300, Y: 10 },width: 500 });
			* &lt;/script&gt;
			* @memberof ejDialog
			* @instance
			*/
            width: 400,
			/**		
			*Sets the z-index value of the control for the Dialog control.
			* @default 1000
			* @type {Integer}
			* @example 
			* &lt;div id="dialog" title="WPF"&gt;
	*Developed by Microsoft, the Windows Presentation Foundation (or WPF) is a computer-software graphical subsystem for rendering user interfaces in Windows-based applications. WPF, previously known as "Avalon", was initially released as part of .NET Framework 3.0. Rather than relying on the older GDI subsystem, WPF uses DirectX. WPF attempts to provide a consistent programming model for building applications and separates the user interface from business logic. It resembles similar XML-oriented object models, such as those implemented in XUL and SVG.
	* &lt;/div&gt;
	* &lt;script&gt;
			* //zIndex on initialization.  
			* 	 $("#dialog").ejDialog({position: { X: 300, Y: 10 }, zIndex: 500 });
			* &lt;/script&gt;
			* @memberof ejDialog
			* @instance
			*/
            zIndex: 1000,
			/**		
			*Set the root class for the Dialog control theme.
			* @default ""
			* @type {String}
			* @example 
			* &lt;div id="dialog" title="WPF"&gt;
	*Developed by Microsoft, the Windows Presentation Foundation (or WPF) is a computer-software graphical subsystem for rendering user interfaces in Windows-based applications. WPF, previously known as "Avalon", was initially released as part of .NET Framework 3.0. Rather than relying on the older GDI subsystem, WPF uses DirectX. WPF attempts to provide a consistent programming model for building applications and separates the user interface from business logic. It resembles similar XML-oriented object models, such as those implemented in XUL and SVG.
	* &lt;/div&gt;
	* &lt;script&gt;
			* //cssClass on initialization.  
			* 	$("#dialog").ejDialog({position: { X: 300, Y: 10 }, cssClass: "gradient-lime" });
			* &lt;/script&gt;
			* @memberof ejDialog
			* @instance
			*/
            cssClass: "",
			/**		
			*Displays dialog content from right to left when enabled
			* @default false
			* @type {Boolean}
			* @example 
			* &lt;div id="dialog" title="WPF"&gt;
	*Developed by Microsoft, the Windows Presentation Foundation (or WPF) is a computer-software graphical subsystem for rendering user interfaces in Windows-based applications. WPF, previously known as "Avalon", was initially released as part of .NET Framework 3.0. Rather than relying on the older GDI subsystem, WPF uses DirectX. WPF attempts to provide a consistent programming model for building applications and separates the user interface from business logic. It resembles similar XML-oriented object models, such as those implemented in XUL and SVG.
	* &lt;/div&gt;
	* &lt;script&gt;
			* //Enable rtl on initialization.  
			* 	 $("#dialog").ejDialog({position: { X: 300, Y: 10 }, enableRTL: true});
			* &lt;/script&gt;
			* @memberof ejDialog
			* @instance
			*/
            enableRTL: false,
			/**		
			*Enables users to navigate right, left, up, and down in the control with keyboard action
			* @default true
			* @type {Boolean}
			* @example 
			* &lt;div id="dialog" title="WPF"&gt;
	*Developed by Microsoft, the Windows Presentation Foundation (or WPF) is a computer-software graphical subsystem for rendering user interfaces in Windows-based applications. WPF, previously known as "Avalon", was initially released as part of .NET Framework 3.0. Rather than relying on the older GDI subsystem, WPF uses DirectX. WPF attempts to provide a consistent programming model for building applications and separates the user interface from business logic. It resembles similar XML-oriented object models, such as those implemented in XUL and SVG.
	* &lt;/div&gt;
	* &lt;script&gt;
			* //Enable  allowKeyboardNavigation on initialization.  
			* 	 $("#dialog").ejDialog({ position: { X: 300, Y: 10 },allowKeyboardNavigation: true});
			* &lt;/script&gt;
			* @memberof ejDialog
			* @instance
			*/
            allowKeyboardNavigation: true,
			/**		
			*Gives dialog window rounded corners.
			* @default false
			* @type {Boolean}
			* @example 
			* &lt;div id="dialog" title="WPF"&gt;
	*Developed by Microsoft, the Windows Presentation Foundation (or WPF) is a computer-software graphical subsystem for rendering user interfaces in Windows-based applications. WPF, previously known as "Avalon", was initially released as part of .NET Framework 3.0. Rather than relying on the older GDI subsystem, WPF uses DirectX. WPF attempts to provide a consistent programming model for building applications and separates the user interface from business logic. It resembles similar XML-oriented object models, such as those implemented in XUL and SVG.
	* &lt;/div&gt;
	* &lt;script&gt;
			* //Enable roundedCorner on initialization.  
			* 	 $("#dialog").ejDialog({position: { X: 300, Y: 10 }, showRoundedCorner: true});
			* &lt;/script&gt;
			* @memberof ejDialog
			* @instance
			*/
            showRoundedCorner : false,
			/**		
			*Gives dialog window rounded corners.
			* @default [“close”]
			* @type {StringArray}
			* @example 
			* &lt;div id="dialog" title="WPF"&gt;
	*Developed by Microsoft, the Windows Presentation Foundation (or WPF) is a computer-software graphical subsystem for rendering user interfaces in Windows-based applications. WPF, previously known as "Avalon", was initially released as part of .NET Framework 3.0. Rather than relying on the older GDI subsystem, WPF uses DirectX. WPF attempts to provide a consistent programming model for building applications and separates the user interface from business logic. It resembles similar XML-oriented object models, such as those implemented in XUL and SVG.
	* &lt;/div&gt;
	* &lt;script&gt;
			* //iconAction on initialization.  
			* 	 $("#dialog").ejDialog({ position: { X: 300, Y: 10 },actionButtons: ["close","collapsible","pin"]});
			* &lt;/script&gt;
			* @memberof ejDialog
			* @instance
			*/
            actionButtons: ["close"], // ["close","collapsible","maximize","minimize","custom","pin"]
 /**		
			*Custom icons for custom actions can be provided by using a CSS class.
			* @default null
			* @type {String}
			* @example 
			* &lt;div id="dialog" title="WPF"&gt;
	*Developed by Microsoft, the Windows Presentation Foundation (or WPF) is a computer-software graphical subsystem for rendering user interfaces in Windows-based applications. WPF, previously known as "Avalon", was initially released as part of .NET Framework 3.0. Rather than relying on the older GDI subsystem, WPF uses DirectX. WPF attempts to provide a consistent programming model for building applications and separates the user interface from business logic. It resembles similar XML-oriented object models, such as those implemented in XUL and SVG.
	* &lt;/div&gt;
	* &lt;script&gt;
	*  $("#dialog").ejDialog({position: { X: 300, Y: 10 }}); 
			* //customIconCss on initialization.  
			* 	 ("#dialog").ejDialog({ position: { X: 300, Y: 10 }, faviconCSS  : "custom-icon" });
			* &lt;/script&gt;
			* @memberof ejDialog
			* @instance
			*/
            faviconCSS  : null,
			   /**		
			*Places the dialog in the given container.
			* @default null
			* @type {String}
			* @example 
			* &lt;div id="dialog" title="WPF"&gt;
	*Developed by Microsoft, the Windows Presentation Foundation (or WPF) is a computer-software graphical subsystem for rendering user interfaces in Windows-based applications. WPF, previously known as "Avalon", was initially released as part of .NET Framework 3.0. Rather than relying on the older GDI subsystem, WPF uses DirectX. WPF attempts to provide a consistent programming model for building applications and separates the user interface from business logic. It resembles similar XML-oriented object models, such as those implemented in XUL and SVG.
	* &lt;/div&gt;
	* &lt;script&gt;
	*  $("#dialog").ejDialog({position: { X: 300, Y: 10 }}); 
			* // Enable contentContainer on initialization.  
			* 	$("#dialog").ejDialog({ position: { X: 300, Y: 10 },content: "#samplearea" });
			* &lt;/script&gt;
			* @memberof ejDialog
			* @instance
			*/
            content: null,
			/**		
			*Saves the current model value to browser cookies for maintaining state.
			* @default False
			* @type {Boolean}
			* @example 
			* &lt;div id="dialog" title="WPF"&gt;
	*Developed by Microsoft, the Windows Presentation Foundation (or WPF) is a computer-software graphical subsystem for rendering user interfaces in Windows-based applications. WPF, previously known as "Avalon", was initially released as part of .NET Framework 3.0. Rather than relying on the older GDI subsystem, WPF uses DirectX. WPF attempts to provide a consistent programming model for building applications and separates the user interface from business logic. It resembles similar XML-oriented object models, such as those implemented in XUL and SVG.
	* &lt;/div&gt;
	* &lt;script&gt;
			* //Enable persist on initialization.  
			* $("#dialog").ejDialog({position: { X: 300, Y: 10 }, enablePersistence: true});
			* &lt;/script&gt;
			* @memberof ejDialog
			* @instance
			*/
            enablePersistence: false,
			/**		
			*Enables or disables the Dialog control.
			* @default true
			* @type {Boolean}
			* @example 
			* &lt;div id="dialog" title="WPF"&gt;
	*Developed by Microsoft, the Windows Presentation Foundation (or WPF) is a computer-software graphical subsystem for rendering user interfaces in Windows-based applications. WPF, previously known as "Avalon", was initially released as part of .NET Framework 3.0. Rather than relying on the older GDI subsystem, WPF uses DirectX. WPF attempts to provide a consistent programming model for building applications and separates the user interface from business logic. It resembles similar XML-oriented object models, such as those implemented in XUL and SVG.
	* &lt;/div&gt;
	* &lt;script&gt;
			* //Set enabled on initialization.  
			*  $("#dialog").ejDialog({position: { X: 300, Y: 10 }, enabled: true });
			* &lt;/script&gt;
			* @memberof ejDialog
			* @instance
			*/
            enabled: true,
			/**		
			*Enables  the window resizing.
			* @default true
			* @type {Boolean}
			* @example 
			* &lt;div id="dialog" title="WPF"&gt;
	*Developed by Microsoft, the Windows Presentation Foundation (or WPF) is a computer-software graphical subsystem for rendering user interfaces in Windows-based applications. WPF, previously known as "Avalon", was initially released as part of .NET Framework 3.0. Rather than relying on the older GDI subsystem, WPF uses DirectX. WPF attempts to provide a consistent programming model for building applications and separates the user interface from business logic. It resembles similar XML-oriented object models, such as those implemented in XUL and SVG.
	* &lt;/div&gt;
	* &lt;script&gt;
			* //Enable windowResizing on initialization.  
			*  $("#dialog").ejDialog({position: { X: 300, Y: 10 }, enableAutoResize: true });
			* &lt;/script&gt;
			* @memberof ejDialog
			* @instance
			*/
            enableAutoResize: false,
            //Events
			 /**     
			 * Fires before dialog control is closed.
			 * @event
			 * @name ejDialog#beforeClose		
			 * @param {Object} argument.event returns the close icon click event args    
			 * @param {boolean}  argument.cancel returns the cancel option value
			 * @param {object}  argument.model returns the dialog model
			 * @param {string}  argument.type returns the name of the event
			 * @example 
			 * &lt;div id="dialog" title="WPF"&gt;
	*Developed by Microsoft, the Windows Presentation Foundation (or WPF) is a computer-software graphical subsystem for rendering user interfaces in Windows-based applications. WPF, previously known as "Avalon", was initially released as part of .NET Framework 3.0. Rather than relying on the older GDI subsystem, WPF uses DirectX. WPF attempts to provide a consistent programming model for building applications and separates the user interface from business logic. It resembles similar XML-oriented object models, such as those implemented in XUL and SVG.
	* &lt;/div&gt;
	* &lt;script&gt;
			 * //beforeClose event for dialog
             *  $("#dialog").ejDialog({
             *	position: { X: 300, Y: 10 },
			 *  beforeClose: function (args) {}
             * });
             * &lt;/script&gt;
			 * @memberof ejDialog
			 * @instance
			 */	
            beforeClose: null,
			/**     
			 * Fires  after dialog control is closed.
			 * @event
			 * @name ejDialog#Close		
			 * @param {Object} argument.event returns the close icon click event args    
			 * @param {boolean}  argument.cancel returns the cancel option value
			 * @param {object}  argument.model returns the dialog model
			 * @param {string}  argument.type returns the name of the event
			 * @example 
			 * &lt;div id="dialog" title="WPF"&gt;
	*Developed by Microsoft, the Windows Presentation Foundation (or WPF) is a computer-software graphical subsystem for rendering user interfaces in Windows-based applications. WPF, previously known as "Avalon", was initially released as part of .NET Framework 3.0. Rather than relying on the older GDI subsystem, WPF uses DirectX. WPF attempts to provide a consistent programming model for building applications and separates the user interface from business logic. It resembles similar XML-oriented object models, such as those implemented in XUL and SVG.
	* &lt;/div&gt;
	* &lt;script&gt;
			 * //beforeClose event for dialog
             *  $("#dialog").ejDialog({
             *	position: { X: 300, Y: 10 },
			 *  Close: function (args) {}
             * });
             * &lt;/script&gt;
			 * @memberof ejDialog
			 * @instance
			 */	
            close: null,
			/**     
			 * Fires  before dialog control is opened.
			 * @event
			 * @name ejDialog# beforeOpen		
			 * @param {boolean}  argument.cancel returns the cancel option value
			 * @param {object}  argument.model returns the dialog model
			 * @param {string}  argument.type returns the name of the event
			 * @example 
			 * &lt;div id="dialog" title="WPF"&gt;
	*Developed by Microsoft, the Windows Presentation Foundation (or WPF) is a computer-software graphical subsystem for rendering user interfaces in Windows-based applications. WPF, previously known as "Avalon", was initially released as part of .NET Framework 3.0. Rather than relying on the older GDI subsystem, WPF uses DirectX. WPF attempts to provide a consistent programming model for building applications and separates the user interface from business logic. It resembles similar XML-oriented object models, such as those implemented in XUL and SVG.
	* &lt;/div&gt;
	* &lt;script&gt;
			 * // beforeOpen event for dialog
             *  $("#dialog").ejDialog({
             *	position: { X: 300, Y: 10 },
			 *   beforeOpen: function (args) {}
             * });
             * &lt;/script&gt;
			 * @memberof ejDialog
			 * @instance
			 */	
            beforeOpen: null,
				/**     
			 * Fires  after dialog control is opened.
			 * @event
			 * @name ejDialog#open		
			 * @param {boolean}  argument.cancel returns the cancel option value
			 * @param {object}  argument.model returns the dialog model
			 * @param {string}  argument.type returns the name of the event
			 * @example 
			 * &lt;div id="dialog" title="WPF"&gt;
	*Developed by Microsoft, the Windows Presentation Foundation (or WPF) is a computer-software graphical subsystem for rendering user interfaces in Windows-based applications. WPF, previously known as "Avalon", was initially released as part of .NET Framework 3.0. Rather than relying on the older GDI subsystem, WPF uses DirectX. WPF attempts to provide a consistent programming model for building applications and separates the user interface from business logic. It resembles similar XML-oriented object models, such as those implemented in XUL and SVG.
	* &lt;/div&gt;
	* &lt;script&gt;
			 * //  open event for dialog
             *  $("#dialog").ejDialog({
             *	position: { X: 300, Y: 10 },
			 *    open: function (args) {}
             * });
             * &lt;/script&gt;
			 * @memberof ejDialog
			 * @instance
			 */	
            open: null,
			 /**     
			 * Fires  when dialog control is dragged.
			 * @event
			 * @name ejDialog#drag		
			 * @param {boolean}  argument.cancel returns the cancel option value
			 * @param {object}  argument.model returns the dialog model
			 * @param {string}  argument.type returns the name of the event
			 * @param {Object} argument.event  returns the mouse move event args
			 * @example 
			 * &lt;div id="dialog" title="WPF"&gt;
	*Developed by Microsoft, the Windows Presentation Foundation (or WPF) is a computer-software graphical subsystem for rendering user interfaces in Windows-based applications. WPF, previously known as "Avalon", was initially released as part of .NET Framework 3.0. Rather than relying on the older GDI subsystem, WPF uses DirectX. WPF attempts to provide a consistent programming model for building applications and separates the user interface from business logic. It resembles similar XML-oriented object models, such as those implemented in XUL and SVG.
	* &lt;/div&gt;
	* &lt;script&gt;
			 * //   drag event for dialog
             *  $("#dialog").ejDialog({
             *	position: { X: 300, Y: 10 },
			 *     drag: function (args) {}
             * });
             * &lt;/script&gt;
			 * @memberof ejDialog
			 * @instance
			 */	
            drag: null,
			/**     
			 * Fires when the dialog is starting to be dragged
			 * @event
			 * @name ejDialog#dragStart		
			 * @param {boolean}  argument.cancel returns the cancel option value
			 * @param {object}  argument.model returns the dialog model
			 * @param {string}  argument.type returns the name of the event
			 * @param {Object} argument.event  returns the mouse down event args
			 * @example 
			 * &lt;div id="dialog" title="WPF"&gt;
	*Developed by Microsoft, the Windows Presentation Foundation (or WPF) is a computer-software graphical subsystem for rendering user interfaces in Windows-based applications. WPF, previously known as "Avalon", was initially released as part of .NET Framework 3.0. Rather than relying on the older GDI subsystem, WPF uses DirectX. WPF attempts to provide a consistent programming model for building applications and separates the user interface from business logic. It resembles similar XML-oriented object models, such as those implemented in XUL and SVG.
	* &lt;/div&gt;
	* &lt;script&gt;
			 * //    dragStart event for dialog
             *  $("#dialog").ejDialog({
             *	position: { X: 300, Y: 10 },
			 *      dragStart: function (args) {}
             * });
             * &lt;/script&gt;
			 * @memberof ejDialog
			 * @instance
			 */	
            dragStart: null,
				/**     
			 * Fires after the dialog dragging stops
			 * @event
			 * @name ejDialog#dragStop		
			 * @param {boolean}  argument.cancel returns the cancel option value
			 * @param {object}  argument.model returns the dialog model
			 * @param {string}  argument.type returns the name of the event
			 * @param {Object} argument.event  returns the mouse down event args
			 * @example 
			 * &lt;div id="dialog" title="WPF"&gt;
	*Developed by Microsoft, the Windows Presentation Foundation (or WPF) is a computer-software graphical subsystem for rendering user interfaces in Windows-based applications. WPF, previously known as "Avalon", was initially released as part of .NET Framework 3.0. Rather than relying on the older GDI subsystem, WPF uses DirectX. WPF attempts to provide a consistent programming model for building applications and separates the user interface from business logic. It resembles similar XML-oriented object models, such as those implemented in XUL and SVG.
	* &lt;/div&gt;
	* &lt;script&gt;
			 * //     dragStop event for dialog
             *  $("#dialog").ejDialog({
             *	position: { X: 300, Y: 10 },
			 *       dragStop: function (args) {}
             * });
             * &lt;/script&gt;
			 * @memberof ejDialog
			 * @instance
			 */	
            dragStop: null,
	         /**    
			 * Fires on dialog resize action
			 * @event
			 * @name ejDialog#resize		
			 * @param {boolean}  argument.cancel returns the cancel option value
			 * @param {object}  argument.model returns the dialog model
			 * @param {string}  argument.type returns the name of the event
			 * @param {Object} argument.event  returns the mouse move event args
			 * @example 
			 * &lt;div id="dialog" title="WPF"&gt;
	*Developed by Microsoft, the Windows Presentation Foundation (or WPF) is a computer-software graphical subsystem for rendering user interfaces in Windows-based applications. WPF, previously known as "Avalon", was initially released as part of .NET Framework 3.0. Rather than relying on the older GDI subsystem, WPF uses DirectX. WPF attempts to provide a consistent programming model for building applications and separates the user interface from business logic. It resembles similar XML-oriented object models, such as those implemented in XUL and SVG.
	* &lt;/div&gt;
	* &lt;script&gt;
			 * //      resize event for dialog
             *  $("#dialog").ejDialog({
             *	position: { X: 300, Y: 10 },
			 *        resize: function (args) {}
             * });
             * &lt;/script&gt;
			 * @memberof ejDialog
			 * @instance
			 */	
            resize: null,
			 /**    
			 * Fires on dialog resize start action
			 * @event
			 * @name ejDialog#resizeStart		
			 * @param {boolean}  argument.cancel returns the cancel option value
			 * @param {object}  argument.model returns the dialog model
			 * @param {string}  argument.type returns the name of the event
			 * @param {Object} argument.event  returns the mouse down event args
			 * @example 
			 * &lt;div id="dialog" title="WPF"&gt;
	*Developed by Microsoft, the Windows Presentation Foundation (or WPF) is a computer-software graphical subsystem for rendering user interfaces in Windows-based applications. WPF, previously known as "Avalon", was initially released as part of .NET Framework 3.0. Rather than relying on the older GDI subsystem, WPF uses DirectX. WPF attempts to provide a consistent programming model for building applications and separates the user interface from business logic. It resembles similar XML-oriented object models, such as those implemented in XUL and SVG.
	* &lt;/div&gt;
	* &lt;script&gt;
			 * //       resizeStart event for dialog
             *  $("#dialog").ejDialog({
             *	position: { X: 300, Y: 10 },
			 *         resizeStart: function (args) {}
             * });
             * &lt;/script&gt;
			 * @memberof ejDialog
			 * @instance
			 */	
            resizeStart: null,
			 /**    
			 * Fires on dialog resize stop action
			 * @event
			 * @name ejDialog#resizeStop		
			 * @param {boolean}  argument.cancel returns the cancel option value
			 * @param {object}  argument.model returns the dialog model
			 * @param {string}  argument.type returns the name of the event
			 * @param {Object} argument.event  returns the mouse leave event args
			 * @example 
			 * &lt;div id="dialog" title="WPF"&gt;
	*Developed by Microsoft, the Windows Presentation Foundation (or WPF) is a computer-software graphical subsystem for rendering user interfaces in Windows-based applications. WPF, previously known as "Avalon", was initially released as part of .NET Framework 3.0. Rather than relying on the older GDI subsystem, WPF uses DirectX. WPF attempts to provide a consistent programming model for building applications and separates the user interface from business logic. It resembles similar XML-oriented object models, such as those implemented in XUL and SVG.
	* &lt;/div&gt;
	* &lt;script&gt;
			 * //        resizeStop event for dialog
             *  $("#dialog").ejDialog({
             *	position: { X: 300, Y: 10 },
			 *          resizeStop: function (args) {}
             * });
             * &lt;/script&gt;
			 * @memberof ejDialog
			 * @instance
			 */	
            resizeStop: null,
				 /**    
			 * Fires on AJAX content, iframe, or image load actions
			 * @event
			 * @name ejDialog#contentLoad		
			 * @param {boolean}  argument.cancel returns the cancel option value
			 * @param {object}  argument.model returns the dialog model
			 * @param {string}  argument.type returns the name of the event
			 * @param {string}  argument.url  returns the content location
			 * @param {Object} argument.contentType  returns the content type
			 * @example 
			 * &lt;div id="dialog" title="WPF"&gt;
	*Developed by Microsoft, the Windows Presentation Foundation (or WPF) is a computer-software graphical subsystem for rendering user interfaces in Windows-based applications. WPF, previously known as "Avalon", was initially released as part of .NET Framework 3.0. Rather than relying on the older GDI subsystem, WPF uses DirectX. WPF attempts to provide a consistent programming model for building applications and separates the user interface from business logic. It resembles similar XML-oriented object models, such as those implemented in XUL and SVG.
	* &lt;/div&gt;
	* &lt;script&gt;
			 * //         load event for dialog
             *  $("#dialog").ejDialog({
             *	position: { X: 300, Y: 10 },
			 *           contentLoad: function (args) {}
             * });
             * &lt;/script&gt;
			 * @memberof ejDialog
			 * @instance
			 */	
            contentLoad: null,
			/**    
			 * Fires when AJAX content is loaded successfully
			 * @event
			 * @name ejDialog#ajaxSuccess		
			 * @param {boolean}  argument.cancel returns the cancel option value
			 * @param {object}  argument.model returns the dialog model
			 * @param {string}  argument.type returns the name of the event
			 * @param {string}  argument.url  returns the content location
			 * @param {string} argument.data  returns the content text
			 * @example 
			 * &lt;div id="dialog" title="WPF"&gt;
	*Developed by Microsoft, the Windows Presentation Foundation (or WPF) is a computer-software graphical subsystem for rendering user interfaces in Windows-based applications. WPF, previously known as "Avalon", was initially released as part of .NET Framework 3.0. Rather than relying on the older GDI subsystem, WPF uses DirectX. WPF attempts to provide a consistent programming model for building applications and separates the user interface from business logic. It resembles similar XML-oriented object models, such as those implemented in XUL and SVG.
	* &lt;/div&gt;
	* &lt;script&gt;
			 * //          ajaxSuccess event for dialog
             *  $("#dialog").ejDialog({
             *	position: { X: 300, Y: 10 },
			 *            ajaxSuccess: function (args) {}
             * });
             * &lt;/script&gt;
			 * @memberof ejDialog
			 * @instance
			 */	
            ajaxSuccess: null,
			/**    
			 * Fires after an AJAX content loading error
			 * @event
			 * @name ejDialog#ajaxError		
			 * @param {boolean}  argument.cancel returns the cancel option value
			 * @param {object}  argument.model returns the dialog model
			 * @param {string}  argument.type returns the name of the event
			 * @param {string}  argument.url  returns the content location
			 * @param {string} argument.data  returns the content text
			 * @example 
			 * &lt;div id="dialog" title="WPF"&gt;
	*Developed by Microsoft, the Windows Presentation Foundation (or WPF) is a computer-software graphical subsystem for rendering user interfaces in Windows-based applications. WPF, previously known as "Avalon", was initially released as part of .NET Framework 3.0. Rather than relying on the older GDI subsystem, WPF uses DirectX. WPF attempts to provide a consistent programming model for building applications and separates the user interface from business logic. It resembles similar XML-oriented object models, such as those implemented in XUL and SVG.
	* &lt;/div&gt;
	* &lt;script&gt;
			 * //           ajaxError event for dialog
             *  $("#dialog").ejDialog({
             *	position: { X: 300, Y: 10 },
			 *             ajaxError: function (args) {}
             * });
             * &lt;/script&gt;
			 * @memberof ejDialog
			 * @instance
			 */	
            ajaxError: null,
            /**    
			 * Fires after Create dialog successfully
			 * @event
			 * @name ejDialog#create		
			 * @param {boolean}  argument.cancel returns the cancel option value
			 * @param {object}  argument.model returns the dialog model
			 * @param {string}  argument.type returns the name of the event			 
			 * @example 
			 * &lt;div id="dialog" title="WPF"&gt;
	*Developed by Microsoft, the Windows Presentation Foundation (or WPF) is a computer-software graphical subsystem for rendering user interfaces in Windows-based applications. WPF, previously known as "Avalon", was initially released as part of .NET Framework 3.0. Rather than relying on the older GDI subsystem, WPF uses DirectX. WPF attempts to provide a consistent programming model for building applications and separates the user interface from business logic. It resembles similar XML-oriented object models, such as those implemented in XUL and SVG.
	* &lt;/div&gt;
	* &lt;script&gt;
			 * //           Create event for dialog
             *  $("#dialog").ejDialog({
             *	position: { X: 300, Y: 10 },
			 *             create: function (args) {}
             * });
             * &lt;/script&gt;
			 * @memberof ejDialog
			 * @instance
			 */	
            create: null,
            /**    
			 * Fires after Destroy event successfully
			 * @event
			 * @name ejDialog#destroy		
			 * @param {boolean}  argument.cancel returns the cancel option value
			 * @param {object}  argument.model returns the dialog model
			 * @param {string}  argument.type returns the name of the event			
			 * @example 
			 * &lt;div id="dialog" title="WPF"&gt;
	*Developed by Microsoft, the Windows Presentation Foundation (or WPF) is a computer-software graphical subsystem for rendering user interfaces in Windows-based applications. WPF, previously known as "Avalon", was initially released as part of .NET Framework 3.0. Rather than relying on the older GDI subsystem, WPF uses DirectX. WPF attempts to provide a consistent programming model for building applications and separates the user interface from business logic. It resembles similar XML-oriented object models, such as those implemented in XUL and SVG.
	* &lt;/div&gt;
	* &lt;script&gt;
			 * //           destroy event for dialog
             *  $("#dialog").ejDialog({
            
			 *             destroy: function (args) {}
             * });
             * &lt;/script&gt;
			 * @memberof ejDialog
			 * @instance
			 */	
            destroy: null

        },

        dataTypes: {
            showOnInit: "boolean",
            closeOnEscape: "boolean",
			enableAnimation: "boolean",
            closeIconTooltip: "string",
            allowDraggable: "boolean",
            enableModal: "boolean",
            enableResize: "boolean",
            enableAutoResize: "boolean",
            showHeader: "boolean",
            title: "string",
            zIndex: "number",
            cssClass: "string",
            contentUrl: "string",
            contentType: "string",
            ajaxSettings: "data",
            actionButtons: "array"
        },
		/**
         * To configure the properties at runtime using SetModel		
		 * @private
         */	 
        _setModel: function (options) {
            if (!this.model.enabled && ej.isNullOrUndefined(options["enabled"])) return false;
            for (var key in options) {
                switch (key) {
                    case "closeIconTooltip": this._ejDialog.find("div.e-dialog-icon").attr("title", options[key]); break;
                    case "title": this._ejDialog.find("span.e-title").html(options[key]); break;
                    case "width": this.model.width = options[key]; this._setSize(); this._setContainerSize(); this._resetScroller(); break;
                    case "height": this.model.height = options[key]; this._setSize(); this._setContainerSize(); this._resetScroller(); break;
                    case "position": this.model.position = options[key]; this._dialogPosition(); break;
                    case "cssClass": this._changeSkin(options[key]); break;
                    case "showRoundedCorner ": this._roundedCorner(options[key]); break;
                    case "contentType": { this._appendContent(options[key]); break; }
                    case "enabled": { this.model.enabled = options[key]; this._enabledAction(options[key]); break; }
                    case "contentUrl": { this.model.contentUrl = options[key]; this._appendContent(this.model.contentType); break; }
                    case "content": { this._ejDialog.appendTo($(options[key])); break; }
                    case "minHeight": { this._ejDialog.css("minHeight", options[key]); break; }
                    case "minWidth": { this._ejDialog.css("minWidth", options[key]); break; }
					case "maxHeight": { this._ejDialog.css("maxHeight", options[key]); break; }
                    case "maxWidth": { this._ejDialog.css("maxWidth", options[key]); break; }
                    case "zIndex": { this._ejDialog.css('z-index', options[key]); break; }
                    case "faviconCSS  ": { this._dialogCustom.find("span").removeClass().addClass(options[key]); break; }
                    case "enableAutoResize": {
                        this.model.enableAutoResize = options[key];
                        this._ejDialog.addClass("e-dialog-resize");
                        this._wireResizing();
                        break;
                    }
					case "enableAnimation":{
							this.model.enableAnimation=options[key];
                        break;
                    }
					
                    case "allowDraggable": {
                        this.model.allowDraggable = options[key];
                        if (options[key])
                            this._enableDrag();
                        else {
                            this._dialogTitlebar.removeClass("e-draggable");
                        }
                        break;
                    }
                    case "enableResize": {
                        this.model.enableResize = options[key];
                        if (options[key])
                            this._enableResize();
                        else {
                            this._ejDialog.removeClass("e-resizable");
                            this._ejDialog.find(".e-resize-handle").remove();
                        }
                        break;
                    }
                    case "showHeader": {
                        this.model.showHeader = options[key];
                        if (options[key]) {
                            this._renderTitleBar();
                            this._iconsRender(this.model.actionButtons);
                            this._enableDrag();
                        }
                        else
                            this._ejDialog.find(".e-titlebar").remove();
                        break;
                    }
                    case "enableRTL":
                        {
                            if (options[key])
                                this._ejDialog.addClass("e-rtl");
                            else
                                this._ejDialog.removeClass("e-rtl");
                            break;
                        }
                    case "actionButtons":
                        {
                            if (!ej.isNullOrUndefined(this._dialogTitlebar)) {
                                this._removeAllIcons();
                                this._iconsRender(options[key]);
                            }
                        }
                    case "enableModal": this._enableModal(options[key]); break;
                }
            }
        },

         // all events bound using this._on will be unbind automatically
		/**
        * destroy the dialog widget
		* all events bound using this._on will be unbind automatically and bring the control to pre-init state.
		* @alias destroy
		* @return jQuery
		* @example 
		* &lt;div id="dialog" title="WPF"&gt;
		*Developed by Microsoft, the Windows Presentation Foundation (or WPF) is a computer-software graphical subsystem for rendering user interfaces in Windows-based applications. WPF, previously known as "Avalon", was initially released as part of .NET Framework 3.0. Rather than relying on the older GDI subsystem, WPF uses DirectX. WPF attempts to provide a consistent programming model for building applications and separates the user interface from business logic. It resembles similar XML-oriented object models, such as those implemented in XUL and SVG.
		* &lt;/div&gt;
		* &lt;script&gt;
		 *  $("#dialog").ejDialog(); 
		* // Create Dialog control
		*  var dialogObj = $("#dialog").data("ejDialog");	//Initialize the Dialog object.
		*  dialogObj.destroy(); //Calls the destroy method of Dialog to destroy
		* &lt;/script&gt;
		* @example 
		* &lt;div id="dialog" title="WPF"&gt;
		*Developed by Microsoft, the Windows Presentation Foundation (or WPF) is a computer-software graphical subsystem for rendering user interfaces in Windows-based applications. WPF, previously known as "Avalon", was initially released as part of .NET Framework 3.0. Rather than relying on the older GDI subsystem, WPF uses DirectX. WPF attempts to provide a consistent programming model for building applications and separates the user interface from business logic. It resembles similar XML-oriented object models, such as those implemented in XUL and SVG.
		* &lt;/div&gt;
		* &lt;script&gt;
		*  //call destroy method
		*  $("#dialog").ejDialog("destroy"); 
		* &lt;/script&gt;
		* @memberof ejDialog
		* @instance
         */
        _destroy: function () {
            if (this._overLay) this._overLay.remove();
            this._cloneElement.appendTo(this._ejDialog.parent());
            this._ejDialog.remove();
            this._cloneElement.removeClass("e-dialog");
            this.element = this._cloneElement;
            this._isOpen = false;
        },

        keyConfigs: [37, 38, 39, 40], //keyConfigs for up,down,left and right keys

        // constructor function
		/**
         * Create the dialog widget
		 * @private
         */	
        _init: function () {
            this._widthPercent = null;
            this._heightPercent = null;
            this._renderControl();
            this._wireEvents();
        },
		/**
         * Render Section For Dialog	
		 * @private
         */	
        // render dialog control
        _renderControl: function () {
            //Clone element     
            this._cloneElement = this.element.clone();
            this.element.attr("tabindex", 0).attr("role", "dialog").attr("aria-labelledby", this.element.prop("id") + "_title");
            this._ejDialog = ej.buildTag("div.e-dialog e-widget e-box " + this.model.cssClass + " e-dialog-wrap e-shadow#" + this.element.prop("id") + "_wrapper", "", { display: "none", zIndex: this.model.zIndex }, { tabidex: 0 });
            this.model.enableAutoResize ? this._ejDialog.addClass("e-dialog-resize") : "";
            this.wrapper = this._ejDialog;
            if (!ej.isNullOrUndefined(this.model.content))
                this._ejDialog.appendTo($(this.model.content));
            else
                this._ejDialog.appendTo(document.body);
            if (this.model.enableRTL)
                this._ejDialog.addClass("e-rtl");
            //Render the Title Bar
            if (this.model.showHeader)
                this._renderTitleBar();
            //Icons Rendering
            if (this.model.showHeader)
                this._iconsRender(this.model.actionButtons);
            //Append Content
            this._appendContent(this.model.contentType);
            this._enabledAction(this.model.enabled);
            this._enableDrag();
            this._setSize();
            this._sizeInPercent();
            if (this.model.showOnInit && this.model.contentType != "ajax")
                this.open();
            if (this.model.contentType != "ajax") {
                this._setContainerSize();
                this._renderScroller();
            }
            this._roundedCorner(this.model.showRoundedCorner);
            this.scroller = this.contentDiv.data("ejScroller");
         },
		 /**
         * To set Container Size for the Dialog control		
		 * @private
         */	
        _setContainerSize: function () {
            if (this.model.height != "auto") {
                this.contentDiv.height(this._ejDialog.height() - $(this._ejDialog.find("div.e-titlebar")).outerHeight());
                this.element.outerHeight(this._ejDialog.height() - $(this._ejDialog.find("div.e-titlebar")).outerHeight());
            }
        },
		 /**
         * To enable drag for the Dialog control		
		 * @private
         */	
        //Enable Drag
        _enableDrag: function () {
            if (this.model.allowDraggable && this.model.showHeader) {
                this._dialogTitlebar.addClass("e-draggable");
                this._dragDialog();
            }
        },
		/**
         * To add resize icon for the Dialog control		
		 * @private
         */	
        //Add resize icon
        _enableResize: function () {
            if (this.model.enableResize && this.model.enabled) {
                this._ejDialog.addClass("e-resizable");
                var resizeDiv = ej.buildTag("div.e-icon e-resize-handle");
                resizeDiv.appendTo(this._ejDialog);
                this._resizeDialog();
            }
        },
		/**
         * To change skin for the Dialog control		
		 * @private
         */	
        //Skin Change at run time
        _changeSkin: function (skin) {
            if (this.model.cssClass != skin) {
                this._ejDialog.removeClass(this.model.cssClass).addClass(skin);
            }
        },
        _enableModal: function (value) {
            if (value) this._isOpen && this._createOverlay();
            else if (this._overLay) this._overLay.remove();
        },
		/**
         * To enabledAction  for the Dialog control		
		 * @private
         */	
        _enabledAction: function (flag) {
            if (flag) {
                this._ejDialog.removeClass("e-disable");
                this._enableResize();
            }
            else {
                this._ejDialog.addClass("e-disable");
                this._ejDialog.removeClass("e-resizable");
                this._ejDialog.find(".e-resize-handle").remove();
            }
        },
		/**
         * To render title bar  for the Dialog control		
		 * @private
         */
        _renderTitleBar: function () {
            this._elementTitle = this.element.attr("title");
            if (typeof this._elementTitle !== "string") {
                this._elementTitle = "";
            }
            this.model.title = this.model.title || this._elementTitle;
            this._dialogTitlebar = ej.buildTag("div#" + this.element.prop("id") + "_title.e-titlebar e-header").prependTo(this._ejDialog);
            if (this.model.title) {
                //Add Title 
                ej.buildTag("span.e-title", this.model.title).prependTo(this._dialogTitlebar);
            }
        },
		/**
         * To render icons  for the Dialog control		
		 * @private
         */
        _iconsRender: function (iconArray) {
            for (var icon = 0; icon < iconArray.length; icon++) {
                switch (iconArray[icon]) {
                    case "close": {
                        this._closeIcon();
                        break;
                    }
                    case "collapsible": {
                        this._collapsibleIcon();
                        break;
                    }
                    case "maximize": {
                        this._maximizeIcon();
                        break;
                    }
                    case "minimize": {
                        this._minimizeIcon();
                        break;
                    }
                    case "pin": {
                        this._pinIcon();
                        break;
                    }
                    case "custom": {
                        this._customIcon();
                        break;
                    }
                }
            }
        },
		/**
         * To remove all the icons for the Dialog control		
		 * @private
         */
        _removeAllIcons: function () {
            this._dialogTitlebar.find("div.e-dialog-icon").remove();
        },
		/**
         * To append content  for the Dialog control		
		 * @private
         */
        _appendContent: function (contentType) {
            this.contentDiv = ej.isNullOrUndefined(this.contentDiv) ? ej.buildTag("div.e-dialog-scroller") : this.contentDiv;
            this.element.removeAttr("title").addClass("e-widget-content");
            if (!ej.isNullOrUndefined(this.model.contentUrl) && !ej.isNullOrUndefined(contentType)) {
                if (contentType == "ajax") {
                    this.model.ajaxSettings.url = this.model.contentUrl;
                    this._sendAjaxOptions(this.element, this.model.ajaxSettings.url);
                }
                else if (contentType == "iframe") {
                    if (this.element.children('.e-iframe').length > 0) {
                        var iframe = this.element.find('iframe.e-iframe');
                        iframe.attr('src', this.model.contentUrl);
                    }
                    else {
                        var iframe = ej.buildTag("iframe.e-iframe", "", { width: "100%" }, { scrolling: "no", frameborder: 0, src: this.model.contentUrl });
                        this.element.append(iframe).show().appendTo(this.contentDiv);
                    }
                    this._trigger("contentLoad", { contentType: contentType, url: this.model.contentUrl });
                }
                else if (contentType == "image") {
                    var img = ej.buildTag("img.e-images", "", "", { src: this.model.contentUrl });
                    this.element.append(img).show().appendTo(this.contentDiv);
                    this._trigger("contentLoad", { contentType: contentType, url: this.model.contentUrl });
                }
            }
            else {
                this.element.show()
                .appendTo(this.contentDiv);
            }
            this.contentDiv.appendTo(this._ejDialog);
        },
		/**
         * To set showRoundedCorner   for the Dialog control		
		 * @private
         */
        _roundedCorner: function (value) {
            if (value) {
                this._ejDialog.addClass('e-corner-all');
                this._dialogTitlebar.addClass('e-corner-top');
                this._dialogClose.addClass('e-corner-all');
            }
            else if (this._ejDialog.hasClass('e-corner-all')) {
                this._ejDialog.removeClass('e-corner-all');
                this._dialogTitlebar.removeClass('e-corner-top');
                this._dialogClose.removeClass('e-corner-all');
            }
        },
		/**
         * To reRender scroller for the Dialog control		
		 * @private
         */
        _reRenderScroller: function () {
            if( this.scroller!=undefined)
            this.scroller.refresh();
        },
		/**
         * To Render scroller  for the Dialog control		
		 * @private
         */
        _renderScroller: function () {
            var scrollerWidth = this.model.width;
            if ((this.model.maxHeight !== null && this.model.maxHeight < this.element.outerHeight()) || !(this.model.height === "auto" || this.model.height === "100%")) {
                if (!(this.model.width === "auto" || this.model.width === "100%"))
                    scrollerWidth = this.element.outerWidth();
				var scrHeight = (this.model.maxHeight !== null) ? this.model.maxHeight - $("#"+this.element.prop("id") + "_title").outerHeight() : this.element.outerHeight();
                this.contentDiv.ejScroller({ width: scrollerWidth, height: scrHeight, enableRTL: this.model.enableRTL });
                this.scroller = this.contentDiv.data("ejScroller");
                this.contentDiv.width("100%");
            }
        },
        //Gets the maximum z-index in the document
         _dialogMaxZindex: function () {
             var parents = this.element.parents(), bodyEle;
            bodyEle = $('body').children(), index = bodyEle.index(this.popup);
            bodyEle.splice(index, 1);
            $(bodyEle).each(function (i, ele) { parents.push(ele); });

            var maxZ = Math.max.apply(maxZ, $.map(parents, function (e, n) {
                if ($(e).css('position') != 'static') return parseInt($(e).css('z-index')) || 1;
            }));
            if (!maxZ || maxZ < 10000) maxZ = 10000;
            else maxZ += 1;
            return maxZ;
        },
        //Set the Zindex for dialog
		/**
         * To set the maximum z-index  for the Dialog control		
		 * @private
         */
        _setZindex: function () {
            var zindex = this._dialogMaxZindex();
            if (this.model.zIndex <= zindex) {
                this._ejDialog.css({ zIndex: zindex + 1 });
            }
        },
        //Create the modal overlay div
		/**
         * To Create the modal overlay div for the Dialog control		
		 * @private
         */
        _createOverlay: function () {
            var zindex = this._ejDialog.css('zIndex');//this._dialogMaxZindex() < this.model.zIndex ? this.model.zIndex : this._dialogMaxZindex();
            this._overLay = ej.buildTag("div#" + this.element.attr("id") + "_overLay.e-overlay", "", { zIndex: zindex - 1 });
            this._overLay.appendTo(document.body);
        },

        //handling the ajax request
		/**
         * To handle ajax request for the Dialog control		
		 * @private
         */
        _sendAjaxOptions: function (content, link) {
            //load waiting popup
            content.addClass("e-load");
            var proxy = this;
            var curTitle = this.model.title;
            var hrefLink = link;
            var ajaxOptions = {

                "success": function (data) {
                    try {
                        proxy._ajaxSuccessHandler(data, content, link, curTitle);
                    } catch (e) {

                    }
                }, "error": function (e) {
                    try {
                        proxy._ajaxErrorHandler(e.statusText, content, link, curTitle);
                    } catch (e) {

                    }
                }, "complete": function () {
                    try {
                        proxy._setContainerSize();
                        proxy._renderScroller();
                    } catch (e) {

                    }
                }
            };
            $.extend(true, ajaxOptions, this.model.ajaxSettings);
            this._sendAjaxRequest(ajaxOptions);
        },
		/**
         * To send ajax request for the Dialog control		
		 * @private
         */
        _sendAjaxRequest: function (ajaxOptions) {
            $.ajax({
                type: ajaxOptions.type,
                cache: ajaxOptions.cache,
                url: ajaxOptions.url,
                dataType: ajaxOptions.dataType,
                data: ajaxOptions.data,
                contentType: ajaxOptions.contentType,
                async: ajaxOptions.async,
                success: ajaxOptions.success,
                error: ajaxOptions.error,
                beforeSend: ajaxOptions.beforeSend,
                complete: ajaxOptions.complete
            });
        },
/**
         * Section For handle the ajaxSuccess event
		 * @private
         */	
        _ajaxSuccessHandler: function (data, content, link, curTitle) {
            content.removeClass("e-load");
            content.html(data).addClass("e-dialog-loaded").appendTo(this._ejDialog);
            content.appendTo(this.contentDiv);
            this._dialogPosition();
            if (this.model.showOnInit)
            this.open();
            this._trigger("ajaxSuccess", { data: data, url: link });
        },
		/**
         * Section For handle the ajaxSuccess event
		 * @private
         */	
        _ajaxErrorHandler: function (data, content, link, curTitle) {
            content.addClass("e-dialog-loaded");
            content.appendTo(this.contentDiv);
            this._dialogPosition();
            this.open();
            this._trigger("ajaxError", { data: data, url: link });
        },
		/**
         * To set close icon for the dialog control
		 * @private
         */	
        _closeIcon: function () {
            this._dialogClose = ej.util.buildTag("div.e-dialog-icon");
            var span = ej.util.buildTag("span.e-icon e-close", "", {}, { role: "presentation" });
            span.appendTo(this._dialogClose);
            this._dialogClose.appendTo(this._dialogTitlebar).attr("title", this.model.closeIconTooltip);
            this._on(this._dialogClose, "touchstart click", this._closeClick);
        },
		/**
         * To set collapsible icon for the dialog control.
		 * @private
         */	
        _collapsibleIcon: function () {
            this._dialogCollapsible = ej.util.buildTag("div.e-dialog-icon")
            var span = ej.util.buildTag("span.e-icon e-collapse-arrow", "", {}, { role: "presentation" });
            span.appendTo(this._dialogCollapsible);
            this._dialogCollapsible.appendTo(this._dialogTitlebar);
            this._on(this._dialogCollapsible, "mousedown", this._collapsibleClick);
        },
		/**
         * To set maximize icon for the dialog control.
		 * @private
         */	
        _maximizeIcon: function () {
            this._dialogMaximize = ej.util.buildTag("div.e-dialog-icon")
            var span = ej.util.buildTag("span.e-icon e-maximize", "", {}, { role: "presentation" });
            span.appendTo(this._dialogMaximize);
            this._dialogMaximize.appendTo(this._dialogTitlebar);
            this._on(this._dialogMaximize, "mousedown", this._maximizeClick);
        },
		/**
         * To set minimize icon for the dialog control.
		 * @private
         */	
        _minimizeIcon: function () {
            this._dialogMinimize = ej.util.buildTag("div.e-dialog-icon")
            var span = ej.util.buildTag("span.e-icon e-minimize", "", {}, { role: "presentation" });
            span.appendTo(this._dialogMinimize);
            this._dialogMinimize.appendTo(this._dialogTitlebar);
            this._on(this._dialogMinimize, "mousedown", this._minimizeClick);
        },
		/**
         * To set pin icon for the dialog control.
		 * @private
         */	
        _pinIcon: function () {
            this._dialogPin = ej.util.buildTag("div.e-dialog-icon")
            var span = ej.util.buildTag("span.e-icon e-unpin", "", {}, { role: "presentation" });
            span.appendTo(this._dialogPin);
            this._dialogPin.appendTo(this._dialogTitlebar);
            this._on(this._dialogPin, "mousedown", this._pinClick);
        },
		/**
         * To set custom icon for the dialog control.
		 * @private
         */	
        _customIcon: function () {
            this._dialogCustom = ej.util.buildTag("div.e-dialog-icon","", {},{ style: "float:left" });
            var span = ej.util.buildTag("span.e-dialog-custom", "", {}, { role: "presentation" });
            span.appendTo(this._dialogCustom);
            this._dialogCustom.appendTo(this._dialogTitlebar);
            if (!ej.isNullOrUndefined(this.model.faviconCSS  ))
                span.addClass(this.model.faviconCSS  );
        },
		/**
         * To set size for the dialog control.
		 * @private
         */	
        _setSize: function () {
            var mdl = this.model;
            this._ejDialog.css({ width: mdl.width, minWidth: mdl.minWidth, maxWidth: mdl.maxWidth });
            this._ejDialog.css({ height: mdl.height, minHeight: mdl.minHeight, maxHeight: mdl.maxHeight });
        },
		/**
         * To reset Scroller for the dialog control.
		 * @private
         */	
        _resetScroller: function () {
            var scrHeight = parseInt(this.model.height) - $(this._ejDialog.find("div.e-titlebar")).outerHeight();
            this.contentDiv.ejScroller({ "width": this.model.width, "height": scrHeight });
            this.element.width("auto");
            this.scroller = this.contentDiv.data("ejScroller");
            this.scroller.refresh();
            this._padding = parseInt($(this.element).css("padding-top")) + parseInt($(this.element).css("padding-bottom"));
            if (!ej.isNullOrUndefined(this.contentDiv.height()) && this.contentDiv.height() > 0)
            this.element.css({ "height": this.contentDiv.height() - this._padding });
        },
		/**
         * To drag dialog control.
		 * @private
         */	
        _dragDialog: function () {
            var proxy = this;
            var pos = this._ejDialog.parents(".e-dialog-scroller");
            this._dialogTitlebar.ejDraggable({
                handle: ".e-titlebar",
                cursorAt: !pos.offset() ? { top: 0, left: 0 } : pos.offset(),
                dragStart: function (event) {
                    proxy._clickHandler();
                    if (proxy.dialogPin || !proxy.model.allowDraggable || !proxy.model.enabled)
                        event.cancel = true;
                    if (proxy._trigger("dragStart", { event: event })) {
                        event.cancel = true;
                        return false;
                    }
                },
                drag: function (event) {
                    proxy._trigger("drag", { event: event });
                },
                dragStop: function (event) {
                    proxy._ejDialog.focus();
                    if (proxy.element.find(".e-draggable.e-titlebar")) {
                        dragobject = $("#" + proxy.element.find(".e-draggable.e-titlebar").attr("id")).data("ejDraggable");
                        if (dragobject)
                            dragobject.option("cursorAt",  proxy.element.offset());
                    }
                    proxy._trigger("dragStop", { event: event });
                },
                helper: function (event) {
                    return $(proxy._ejDialog).addClass("dragClone");
                }
            });
        },
		/**
         * To resize dialog control.
		 * @private
         */	
        _resizeDialog: function () {
            var proxy = this, started = false;
            this._ejDialog.find("div.e-resize-handle").ejResizable(
                {
                    minHeight: parseInt(proxy.model.minHeight),
                    minWidth: parseInt(proxy.model.minWidth),
                    maxHeight: parseInt(proxy.model.maxHeight),
                    maxWidth: parseInt(proxy.model.maxWidth),
                    handle: "e-widget-content",
                    resizeStart: function (event) {
                        !started && proxy._trigger("resizeStart", { event: event });
                        started = true;
                    },
                    resize: function (event) {
                        var reElement = $(event.element).parents("div.e-dialog-wrap");
                        proxy.model.height = $(reElement).height();
                        proxy.model.width = $(reElement).width();
                        proxy._setSize();
                        proxy._resetScroller();
                        proxy._trigger("resize", { event: event });
                    },
                    resizeStop: function (event) {
                        proxy._ejDialog.focus();
                        proxy._sizeInPercent();
                        started && proxy._trigger("resizeStop", { event: event });
                        started = false;
                    },
                    helper: function (event) {
                        var reElement = $(event.element).parents("div.e-dialog-wrap");
                        proxy.model.height = $(reElement).height();
                        proxy.model.width = $(reElement).width();
                        proxy._setSize();
                        proxy._resetScroller();
                        return $(proxy._ejDialog);
                    }
                });
        },
		/**
         * To set position for the dialog control.
		 * @private
         */	
        _dialogPosition: function () {
            if (this._ejDialog.parents("form").length > 0 && ej.isNullOrUndefined(this.model.content)) {
                this._ejDialog.appendTo(this._ejDialog.parents("form"));
            }
            if (this.model.position.X != "" || this.model.position.Y != "") {
                this._ejDialog.css("position", "absolute");
                this._ejDialog.css("left", this.model.position.X + "px");
                this._ejDialog.css("top", this.model.position.Y + "px");
            }
            else {
                var x, y;
                if (!ej.isNullOrUndefined(this.model.content)) {
                    var location = $(this.model.content).position();
                    x = ($(this.model.content).outerWidth() > this._ejDialog.width()) ? (($(this.model.content).outerWidth() - this._ejDialog.width()) / 2) + Math.ceil(location.left) : Math.ceil(location.left);
                    y = ($(this.model.content).outerHeight() > this._ejDialog.height()) ? (($(this.model.content).outerHeight() - this._ejDialog.height()) / 2) + Math.ceil(location.top) : Math.ceil(location.top);
                }
                else {
                    x = ($(window).outerWidth() > this._ejDialog.width()) ? ($(window).outerWidth() - this._ejDialog.outerWidth()) / 2 : 0;
                    y = ($(window).outerHeight() > this._ejDialog.height()) ? ($(window).outerHeight() - this._ejDialog.outerHeight()) / 2 : 0;
                }
                this._ejDialog.css({ top: y, left: x });
                this._ejDialog.css("position", "absolute");
            }         
        },
		/**
         * Section handles when close icon is clicked
		 * @private
         */	
        _closeClick: function (event) {
            event.stopPropagation();
            this.close(event);
        },
			/**
         * Section handles when collapsible icon is clicked
		 * @private
         */	
        _collapsibleClick: function (e) {
            if ($(e.target).hasClass("e-collapse-arrow")) {
                this._dialogCollapsible.find("span.e-collapse-arrow").removeClass("e-collapse-arrow").addClass("e-expand-arrow");
                this._ejDialog.find("div.e-resize-handle").hide();
                this._ejDialog.find(".e-widget-content").parent().slideUp("fast");
                this._ejDialog.removeClass("e-shadow");
                this._ejDialog.css("minHeight", "0");
                this._ejDialog.height("auto");
            }
            else if ($(e.target).hasClass("e-expand-arrow")) {
                this._dialogCollapsible.find("span.e-expand-arrow").removeClass("e-expand-arrow").addClass("e-collapse-arrow");
                this._ejDialog.addClass("e-shadow");
                this._ejDialog.find(".e-widget-content").parent().slideDown("fast");
                this._ejDialog.find("div.e-resize-handle").show();
                this._ejDialog.height(this.model.height);
            }
        },
		/**
         * Section handles when maximize icon is clicked
		 * @private
         */	
        _maximizeClick: function (e) {
            var _target = $(e.target);
            if (_target.hasClass("e-maximize")) {
                this._ejDialog.css("top", "0px").css("left", "0px").css("position", "fixed");
                this._ejDialog.css({ width: "100%", height: "100%" });
                this.element.css({ width: "100%", height: "100%" });
                this.contentDiv.css({ width: "100%", height: "100%" });
                _target.removeClass("e-maximize").addClass("e-restore");
                this._maximize = true;
                this.dialogPin = true;
            }
            else if (_target.hasClass("e-restore")) {
                this.element.height("");
                this.element.width("");
                this._restoreDialog();
                _target.removeClass("e-restore").addClass("e-maximize");
            }
        },
		/**
         * Section handles when minimize icon is clicked
		 * @private
         */	
        _minimizeClick: function (e) {
            var _target = $(e.target);
            if (_target.hasClass("e-minimize")) {
                if (this._maximize)
                    this._setSize();
                var top = ($(window).height() - this._ejDialog.height()) + this.element.height() + 14;
                this._ejDialog.css("top", "").css("bottom", "0").css("left", "0").css("position", "fixed");
                this._ejDialog.css("minHeight", "0");
                this._ejDialog.css("height", (this._ejDialog.height() - this.contentDiv.height()));
                this.contentDiv.hide();
                _target.parent().hide();
                this._dialogTitlebar.find(".e-maximize").removeClass("e-maximize").addClass("e-restore");
                           
            }
        },
		/**
         * Section handles when pinicon is clicked
		 * @private
         */	
        _pinClick: function (e) {
            var _target = $(e.target);
            if (_target.hasClass("e-unpin")) {
                this.dialogPin = true;
                _target.removeClass("e-unpin").addClass("e-pin");
            }
            else if (_target.hasClass("e-pin")) {
                this.dialogPin = false;
                _target.removeClass("e-pin").addClass("e-unpin");
            }
        },
		/**
         * to restore dialog control
		 * @private
         */	
        _restoreDialog: function () {
            this.dialogPin = false;
            this.contentDiv.show();
            this._ejDialog.css("position", "absolute").css("bottom", "");
            this._setSize();
            this._dialogPosition();
            this._reRenderScroller();
            this._dialogTitlebar.find(".e-minimize").parent().show();
        },
		/**
         * Section handles  click event
		 * @private
         */	
        _clickHandler: function (e) {
            var zindex = this._dialogMaxZindex();
            if (parseInt(this._ejDialog.css("zIndex")) < zindex) {
                this._ejDialog.css({ zIndex: zindex + 1 });
            }
        },
		/**
         * Section handles keydown event
		 * @private
         */	
        _docKeyDown: function (e) {
            if (this.model.enabled) {
                if (e.keyCode) code = e.keyCode; // ie and mozilla/gecko
                else if (e.which) code = e.which; // ns4 and opera
                else code = e.charCode;
                if (this.model.closeOnEscape && code === 27) {
                    this.close(e);
                    e.preventDefault();
                }
            }
        },
		/**
         * Section handles keydown event
		 * @private
         */	
        _keyDown: function (e) {
            if (e.keyCode) code = e.keyCode; // ie and mozilla/gecko
            else if (e.which) code = e.which; // ns4 and opera
            else code = e.charCode;
            if (this.model.allowKeyboardNavigation && this.model.enabled && $(e.target).hasClass("e-dialog")) {
                if ($.inArray(code, this.keyConfigs) > -1 && this.model.allowDraggable && !this.dialogPin) {
                    this._keyPressed(code, e.ctrlKey);
                    e.preventDefault();
                }
            }
        },
		/**
         * Section handles keypress event for the dialog control
		 * @private
         */	
        _keyPressed: function (code, ctrlKey) {
            switch (code) {
                case 40:    //Down arrow
                    ctrlKey ? this._resetWidthHeight(this.model.width, this._ejDialog.height() + 3) : this._ejDialog.css("top", this._ejDialog.position().top + 3 + "px");
                    break;
                case 39:    //Right arrow
                    ctrlKey ? this._resetWidthHeight(this._ejDialog.width() + 3, this._ejDialog.height()) : this._ejDialog.css("left", this._ejDialog.position().left + 3 + "px");
                    break;
                case 38:    //Up arrow
                    ctrlKey ? this._resetWidthHeight(this._ejDialog.width() - 3, this._ejDialog.height()) : this._ejDialog.css("top", this._ejDialog.position().top - 3 + "px");
                    break;
                case 37:    //Left arrow
                    ctrlKey ? this._ejDialog.width(this._ejDialog.width() - 3) : this._ejDialog.css("left", this._ejDialog.position().left - 3 + "px");
                    break;
            }
        },
		/**
         * to set for the dialog control in percent
		 * @private
         */	
        _sizeInPercent: function () {
            if (!this._enableWindowResize()) return false;
            var parentObj = this._getParentObj();
            this._widthPercent = this._convertPixelToPercentage(parentObj.outerWidth(), this._ejDialog.outerWidth());
            this._heightPercent = this._convertPixelToPercentage(parentObj.outerHeight(), this._ejDialog.outerHeight());
            if (this._widthPercent > 100) {
                this._widthPercent = 100;
                this._ejDialog.outerWidth(parentObj.outerWidth());
                this.model.width = this._ejDialog.width();
            }
            if (this._heightPercent > 100) {
                this._heightPercent = 100;
                this._ejDialog.outerHeight(parentObj.outerHeight());
                this.model.height = this._ejDialog.height();
            }
        },
		/**
         * to get the parent of the dialog control
		 * @private
         */	
        _getParentObj: function () {
            if (!ej.isNullOrUndefined(this.model.content))
                return $(this.model.content);
            else
                return (this._ejDialog.parent()[0].nodeName == "BODY" ? $(window) : "");
        },
		/**
         * to convert the percentage value to pixels
		 * @private
         */	
        _convertPercentageToPixel: function (parent, child) {
            return Math.round((child * parent) / 100);  //round       
        },
		/**
         * to convert the pixel value to percentage
		 * @private
         */		
        _convertPixelToPercentage: function (parent, child) {
            return Math.round((child / parent) * 100); //round
        },
        //_setPosition: function () {
        //    if ((this.model.allowWindowResizing) || ((isNaN(this.model.width)&&(this.model.width.indexOf("%") > 0))||(isNaN(this.model.height)&&(this.model.height.indexOf("%") > 0)))) this._dialogPosition();
        //},
		/**
         * section handles when window is resized
		 * @private
         */
        _reSizeHandler: function () {
            var parentObj;
            this._dialogPosition();
            parentObj = this._getParentObj();
            this._ejDialog.outerWidth(this._convertPercentageToPixel(parentObj.outerWidth(), this._widthPercent));
            this._ejDialog.outerHeight(this._convertPercentageToPixel(parentObj.outerHeight(), this._heightPercent));
            this.contentDiv.width(this._ejDialog.width());
            this.element.outerWidth( this.contentDiv.width());
			this.contentDiv.height(this._ejDialog.height() - $(this._ejDialog.find("div.e-titlebar")).outerHeight());
			this.element.outerHeight(this._ejDialog.height() - $(this._ejDialog.find("div.e-titlebar")).outerHeight());
			this.scroller = this.contentDiv.ejScroller({ width: this._ejDialog.width(), height: this.element.outerHeight(), rtl: this.model.rtl });
            this.scroller = this.contentDiv.data("ejScroller");
            this.scroller.refresh();
            this._padding = parseInt($(this.element).css("padding-top")) + parseInt($(this.element).css("padding-bottom"));
            if (!ej.isNullOrUndefined(this.contentDiv.height()) && this.contentDiv.height() > 0)
            this.element.css({ "height": this.contentDiv.height() - this._padding });
            this._dialogPosition();
        },
		/**
         * to enable window resize
		 * @private
         */
        _enableWindowResize:function()
        {
            if (this.model.enableAutoResize || ((isNaN(this.model.width) && (this.model.width.indexOf("%") > 0)) && (isNaN(this.model.height) && (this.model.height.indexOf("%") > 0)&&(this.model.height!="auto"))))
                return true;
            else 
                return false;
        },
        _wireResizing: function () {         
            this._enableWindowResize() ? $(window).bind('resize', $.proxy(this._reSizeHandler, this)) : $(window).unbind('resize', $.proxy(this._reSizeHandler, this))
        },
        
        //-------------------- Event Wire-up -------------------------//
		/**
         * Wiring the events to dialog control		
		 * @private
         */	
        _wireEvents: function () {
            //tabs Click evnts
            this._on($(document), "keydown", this._docKeyDown);
            this._on(this._ejDialog, "keydown", this._keyDown);
            this._wireResizing();
        },

        // ------------------Client side methods-------------------------//
		/**
         * To open the dialog 
		 * @return jQuery
		 * @example 
		 * &lt;div id="dialog" title="WPF"&gt;
	     * Developed by Microsoft, the Windows Presentation Foundation (or WPF) is a computer-software graphical subsystem for rendering user interfaces in Windows-based applications. WPF, previously known as "Avalon", was initially released as part of .NET Framework 3.0. Rather than relying on the older GDI subsystem, WPF uses DirectX. WPF attempts to provide a consistent programming model for building applications and separates the user interface from business logic. It resembles similar XML-oriented object models, such as those implemented in XUL and SVG.
	     * &lt;/div&gt;
	     * &lt;script&gt;
	* // Create Dialog control
    *  $("#dialog").ejDialog({position: { X: 300, Y: 10 }}); 
         * //initialize the dialog object
         *	var dialogObj = $("#dialog").data("ejDialog");
         *	//To open the dialog 
         *	dialogObj.open();
         * &lt;/script&gt;
         * @example 
         &lt;div id="dialog" title="WPF"&gt;
	     * Developed by Microsoft, the Windows Presentation Foundation (or WPF) is a computer-software graphical subsystem for rendering user interfaces in Windows-based applications. WPF, previously known as "Avalon", was initially released as part of .NET Framework 3.0. Rather than relying on the older GDI subsystem, WPF uses DirectX. WPF attempts to provide a consistent programming model for building applications and separates the user interface from business logic. It resembles similar XML-oriented object models, such as those implemented in XUL and SVG.
	     * &lt;/div&gt;
	     * &lt;script&gt;
	* // Create Dialog control
   *  $("#dialog").ejDialog({position: { X: 300, Y: 10 }}); 
         * //To open the dialog 
         * $("#dialog").ejDialog("open");
         * &lt;/script&gt;
		 * @memberof ejDialog
		 * @instance
        */  
        open: function () {
            if (!this.model.enabled) return false;
            if (this._isOpen) {
                return true;
            }
            if (true == this._trigger("beforeOpen")) {
                return false;
            }
            this.element.css("display", "block");
            this._setZindex();
            this._dialogPosition();
            var proxy = this;
            this._ejDialog.fadeIn(this.model.enableAnimation?400:0, "easeOutQuad", function () {
                // Animation complete
                proxy._trigger("open");
            });
            if (this.model.enableModal == true)
                this._createOverlay();
            this._ejDialog.eq(0).focus();
            this._isOpen = true;
            if (this.element.find(".e-draggable.e-titlebar")) {
                dragobject = $("#" + this.element.find(".e-draggable.e-titlebar").attr("id")).data("ejDraggable");
                if (dragobject)
                    dragobject.option("cursorAt", this.element.offset());
            }
            return this;
        },
		/**
         * To close the dialog
		 * @return jQuery
		 * @example 
		 * &lt;div id="dialog" title="WPF"&gt;
	     * Developed by Microsoft, the Windows Presentation Foundation (or WPF) is a computer-software graphical subsystem for rendering user interfaces in Windows-based applications. WPF, previously known as "Avalon", was initially released as part of .NET Framework 3.0. Rather than relying on the older GDI subsystem, WPF uses DirectX. WPF attempts to provide a consistent programming model for building applications and separates the user interface from business logic. It resembles similar XML-oriented object models, such as those implemented in XUL and SVG.
	     * &lt;/div&gt;
	     * &lt;script&gt; 
	     *  $("#dialog").ejDialog({position: { X: 300, Y: 10 }});
         * //initialize the dialog object
         *	var dialogObj = $("#dialog").data("ejDialog");
         *	//To close the dialog 
         *	dialogObj.close();
          * &lt;/script&gt; 
         * @example 
         * &lt;div id="dialog" title="WPF"&gt;
	     * Developed by Microsoft, the Windows Presentation Foundation (or WPF) is a computer-software graphical subsystem for rendering user interfaces in Windows-based applications. WPF, previously known as "Avalon", was initially released as part of .NET Framework 3.0. Rather than relying on the older GDI subsystem, WPF uses DirectX. WPF attempts to provide a consistent programming model for building applications and separates the user interface from business logic. It resembles similar XML-oriented object models, such as those implemented in XUL and SVG.
	     * &lt;/div&gt;
	     * &lt;script&gt; 
	    *  $("#dialog").ejDialog({position: { X: 300, Y: 10 }}); 
         * //To close the dialog 
         * $("#dialog").ejDialog("close");
         * &lt;/script&gt; 
		 * @memberof ejDialog
		 * @instance
        */   
        close: function (event) {
            if (!this._isOpen || !this.model.enabled) {
                return true;
            }
            if (true == this._trigger("beforeClose", { event: event })) {
                return false;
            }
            this._isOpen = false;
            this.element.css("display", "none");
            var proxy = this;
            this._ejDialog.fadeOut(this.model.enableAnimation?300:0, "easeOutQuad", function () {
                // Animation complete
                proxy._trigger("close", { event: event });
            });
            if (this.model.enableModal) {
                if (this._overLay) this._overLay.remove();
            }
            return this;
        },
		/**
         * To get dialog control is opened or not
		 * @return boolean
		 * @example 
		 * &lt;div id="dialog" title="WPF"&gt;
	     * Developed by Microsoft, the Windows Presentation Foundation (or WPF) is a computer-software graphical subsystem for rendering user interfaces in Windows-based applications. WPF, previously known as "Avalon", was initially released as part of .NET Framework 3.0. Rather than relying on the older GDI subsystem, WPF uses DirectX. WPF attempts to provide a consistent programming model for building applications and separates the user interface from business logic. It resembles similar XML-oriented object models, such as those implemented in XUL and SVG.
	     * &lt;/div&gt;
	      * &lt;script&gt;
	      *  $("#dialog").ejDialog({position: { X: 300, Y: 10 }}); 
         * //initialize the dialog object
         *	var dialogObj = $("#dialog").data("ejDialog");
         * //To get the dialog status
         *	dialogObj.isOpened();
          * &lt;/script&gt;
         * @example 
         * &lt;div id="dialog" title="WPF"&gt;
	     * Developed by Microsoft, the Windows Presentation Foundation (or WPF) is a computer-software graphical subsystem for rendering user interfaces in Windows-based applications. WPF, previously known as "Avalon", was initially released as part of .NET Framework 3.0. Rather than relying on the older GDI subsystem, WPF uses DirectX. WPF attempts to provide a consistent programming model for building applications and separates the user interface from business logic. It resembles similar XML-oriented object models, such as those implemented in XUL and SVG.
	     * &lt;/div&gt;
          * &lt;script&gt;
          *  $("#dialog").ejDialog({position: { X: 300, Y: 10 }}); 
         * //To get the dialog status
         * $("#dialog").ejDialog("isOpened");
          * &lt;/script&gt;
		 * @memberof ejDialog
		 * @instance
        */   
        isOpened: function () {
            return this._isOpen;
        }
    });

})(jQuery, Syncfusion);;
/**
* @fileOverview Plugin to style the Html input elements
* @copyright Copyright Syncfusion Inc. 2001 - 2013. All rights reserved.
*  Use of this code is subject to the terms of our license.
*  A copy of the current license can be obtained at any time by e-mailing
*  licensing@syncfusion.com. Any infringement will be prosecuted under
*  applicable laws. 
* @version 12.1 
* @author <a href="mailto:licensing@syncfusion.com">Syncfusion Inc</a>
*/

(function ($, ej, undefined) {
    /**
   * @namespace ej
   * @classdesc The DropDownList control provides a list options to make user to choose an item from the list. It is capable of including other html elements such as images, textboxes, check box, radio buttons and so on.
   * @class ejDropDownList
   * @requires jQuery
   * @requires jquery.easing.1.3.js
   * @requires ej.core.js
   * @requires ej.data.js
   * @requires ej.dropdownlist.js
   * @requires ej.checkbox.js
   * @requires ej.scroller.js
   * @example 
   * &lt;input type="text" id="drpdwn" /&gt; <br> 
   *  &lt;div id="carsList"&gt;
   *    &lt;ul&gt;
   *       &lt;li&gt;Audi A4&lt;/li&gt;
   *       &lt;li&gt;Audi A5&lt;/li&gt;
   *       &lt;li&gt;Audi A6&lt;/li&gt;
   *       &lt;li&gt;Audi A7&lt;/li&gt;
   *       &lt;li&gt;Audi A8&lt;/li&gt;
   *    &lt;/ul&gt;
   *  &lt;/div&gt;
   * &lt;script&gt;
   * // Create DropDownList
   * $('#drpdwn').ejDropDownList({targetID: "carsList"}); 	
   * &lt;/script&gt; 
   *@example
    * // Another way to render DropDownList control.
	*   &lt;select name="selectIndex" id="drpdwn"&gt;
	*        &lt;option value="Art"&gt;Art&lt;/option&gt;
	*        &lt;option value="Architecture"&gt;Architecture&lt;/option&gt;
	*        &lt;option value="Biographies"&gt;Biographies&lt;/option&gt;
	*        &lt;option value="Business"&gt;Business&lt;/option&gt;
	*        &lt;option value="ComputerIT"&gt;ComputerIT&lt;/option&gt;
	*        &lt;option value="Comics"&gt;Comics&lt;/option&gt;
	*        &lt;option value="Cookery"&gt;Cookery&lt;/option&gt;
	*        &lt;option value="Environment"&gt;Environment&lt;/option&gt;
	*        &lt;option value="Fiction"&gt;Fiction&lt;/option&gt;
	*        &lt;option value="Health"&gt;Health&lt;/option&gt;
	*        &lt;option value="Humanities"&gt;Humanities&lt;/option&gt;
	*        &lt;option value="Language"&gt;Language&lt;/option&gt;
	*    &lt;/select&gt;
	* &lt;script&gt;
    * // Create DropDownList
    * $('#drpdwn').ejDropDownList(); 	
    * &lt;/script&gt; 
    */
    // ejDropdownlist is the plugin name 
    // "ej.Dropdownlist" is "namespace.className" will hold functions and properties

    ej.widget("ejDropDownList", "ej.DropDownList", {
        // widget element will be automatically set in this
        element: null,
        // user defined model will be automatically set in this
        model: null,
        validTags: ["select", "input"],
        _setFirst: false,
        //Root Css Class
        _rootCSS: "e-dropdownlist",

        // default model
        defaults: {
            /**		
            * Sets the root class for DropDownList theme. This cssClass API helps to use custom skinning option for DropDownList control. By defining the root class using this API, we need to include this root class in CSS.
            * @default "gradient-lime"
            * @type {string}
            * @example 
			* &lt;input type="text" id="drpdwn" /&gt; <br> 
			*  &lt;div id="carsList"&gt;
			*    &lt;ul&gt;
			*       &lt;li&gt;Audi A4&lt;/li&gt;
			*       &lt;li&gt;Audi A5&lt;/li&gt;
			*       &lt;li&gt;Audi A6&lt;/li&gt;
			*       &lt;li&gt;Audi A7&lt;/li&gt;
			*       &lt;li&gt;Audi A8&lt;/li&gt;
			*    &lt;/ul&gt;
			*  &lt;/div&gt;<br/>
			* &lt;script&gt;
            * //Initialize the DropDownList with the cssClass value specified
            * 	$("#drpdwn").ejDropDownList({ targetID: "carsList",cssClass: 'gradient-lime'});
            * &lt;/script&gt;
            * @memberof ejDropDownList
            * @instance
            */
            cssClass: "",
            /**
            * Defines the default value to be display in the DropDownList textbox.
            * @default Null
            * @type {string}
            * @example 
			* &lt;input type="text" id="drpdwn" /&gt; <br> 
			*  &lt;div id="carsList"&gt;
			*    &lt;ul&gt;
			*       &lt;li&gt;Audi A4&lt;/li&gt;
			*       &lt;li&gt;Audi A5&lt;/li&gt;
			*       &lt;li&gt;Audi A6&lt;/li&gt;
			*       &lt;li&gt;Audi A7&lt;/li&gt;
			*       &lt;li&gt;Audi A8&lt;/li&gt;
			*    &lt;/ul&gt;
			*  &lt;/div&gt;<br/> 
			* &lt;script&gt;
            * //Initialize the DropDownList value property with the  value specified
            * 	$("#drpdwn").ejDropDownList({ targetID: "carsList",value:"Audi A7" });
             * &lt;/script&gt;
            * @memberof ejDropDownList
            * @instance
            */
            value: "",
           
            text: "",
           
            itemValue: "",
            /**		
            * Specifies the itemsCount   for DropDownList.
            * @default 0
            * @type {number}
            * @example 
			* &lt;input type="text" id="drpdwn" /&gt; <br> 
			*  &lt;div id="carsList"&gt;
			*    &lt;ul&gt;
			*       &lt;li&gt;Audi A4&lt;/li&gt;
			*       &lt;li&gt;Audi A5&lt;/li&gt;
			*       &lt;li&gt;Audi A6&lt;/li&gt;
			*       &lt;li&gt;Audi A7&lt;/li&gt;
			*       &lt;li&gt;Audi A8&lt;/li&gt;
			*    &lt;/ul&gt;
			*  &lt;/div&gt;<br/> 
			* &lt;script&gt;		
            * // To set itemsCount   API value during initialization  . 	
            * 	$("#drpdwn").ejDropDownList({  targetID: "carsList",itemsCount  : 2 });
            * &lt;/script&gt;
            * @memberof ejDropDownList
            * @instance
            */
            itemsCount: 0,
            /**		
            * Specifies the data source of the DropDownList. The data source contains the list of data for generating the DropDownList items.	
            * @default null
            * @type {data}
            * @example 
			* &lt;input type="text" id="drpdwn" /&gt; <br> 
			* &lt;script&gt;		
            * //To set dataSource API value during initialization  
            * 	$("#drpdwn").ejDropDownList({ dataSource: window.countries });			 
            * &lt;/script&gt;
            * @memberof ejDropDownList
            * @instance
            */
            dataSource: null,
            /**		
            * Specifies the query to retrieve the data from online server.	
            * @default null
            * @type {object}
            * @example 
			* &lt;input type="text" id="drpdwn" /&gt; <br> 
			* &lt;script&gt;		
            * //To set query API value during initialization  
            * var dataManger = ej.DataManager({       url: "http://mvc.syncfusion.com/Services/Northwnd.svc/"});
            * var queryString = ej.Query().from("Suppliers").select("ContactName");
            * 	$("#drpdwn").ejDropDownList({ dataSource: dataManger, query: queryString, fields: { text: "ContactName" }});
            * &lt;/script&gt;
            * @memberof ejDropDownList
            * @instance
            */
            query: null,
            /**		
            * Specifies mapping fields for the data items of the DropDownList textbox.	
            * @default null
            * @type {object}
            * @example 
			* &lt;input type="text" id="drpdwn" /&gt; <br> 
			* &lt;script&gt;		
            * //To set fields API value during initialization  
            * 	$("#drpdwn").ejDropDownList({ dataSource: window.countriesField,   fields: { text: "name", value: "key" }});
             * &lt;/script&gt;
            * @memberof ejDropDownList
            * @instance
            */
            fields:/** @lends ejDropDownList# */ { 
				/**		
                 * Defines id for the tag.
				 * @alias ejDropDownList#fields->id
				 * @type String
                 */
			id: null, 
				/**		
                 * Defines the text content for the tag.
				 * @alias ejDropDownList#fields->text
				 * @type String
                 */
			text: null, 
				/**		
                 * Defines the tag value or display text..
				 * @alias ejDropDownList#fields->value
				 * @type String
                 */
			value: null, 
				/**		
                 * Defines the imageURL for the image location. 
				 * @alias ejDropDownList#fields->imageUrl
				 * @type String
                 */
			imageUrl: null,
                 /**		
                 * Defines the image attributes such as height, width, styles and so on.
				 * @alias ejDropDownList#fields->imageAttributes
				 * @type String
                 */			
			imageAttributes: null, 
				/**		
                 * Defines the sprite css for the image tag.
				 * @alias ejDropDownList#fields->spriteCssClass
				 * @type String
                 */
			spriteCssClass: null, 
				/**		
                 * Defines the html attributes such as id, class, styles for the item.
				 * @alias ejDropDownList#fields->htmlAttributes
				 * @type Object
                 */
			htmlAttributes: null, 
				/**		
                 * Defines the tag value to be selected initially
				 * @alias ejDropDownList#fields->selected
				 * @type Boolean
                 */
			selected: null, 
				/**		
                 * Defines the table name for tag value or display text while render with remote data.
				 * @alias ejDropDownList#fields->tableName
				 * @type String
                 */
			tableName: null
			},
            /**		
            * Sets the watermark text. When the textbox is empty the watermark text is visible like a shaded text. 
            * @default "Enter text"
            * @type {string}
            * @example 
			* &lt;input type="text" id="drpdwn" /&gt; <br> 
			*  &lt;div id="carsList"&gt;
			*    &lt;ul&gt;
			*       &lt;li&gt;Audi A4&lt;/li&gt;
			*       &lt;li&gt;Audi A5&lt;/li&gt;
			*       &lt;li&gt;Audi A6&lt;/li&gt;
			*       &lt;li&gt;Audi A7&lt;/li&gt;
			*       &lt;li&gt;Audi A8&lt;/li&gt;
			*    &lt;/ul&gt;
			*  &lt;/div&gt;<br/> 
			* &lt;script&gt;		
            * //Initialize the DropDownList with the watermarkText value specified
            * 	$("#drpdwn").ejDropDownList({ targetID: "carsList",watermarkText: 'Enter text' });
            * &lt;/script&gt;
            * @memberof ejDropDownList
            * @instance
            */
            watermarkText: null,
            /**		
            * Defines the height of the DropDownList textbox.
            * @default Null
            * @type {string | number}
            * @example 
			* &lt;input type="text" id="drpdwn" /&gt; <br> 
			*  &lt;div id="carsList"&gt;
			*    &lt;ul&gt;
			*       &lt;li&gt;Audi A4&lt;/li&gt;
			*       &lt;li&gt;Audi A5&lt;/li&gt;
			*       &lt;li&gt;Audi A6&lt;/li&gt;
			*       &lt;li&gt;Audi A7&lt;/li&gt;
			*       &lt;li&gt;Audi A8&lt;/li&gt;
			*    &lt;/ul&gt;
			*  &lt;/div&gt;<br/> 
			* &lt;script&gt;		
            * //Initialize the DropDownList height property with the  value specified
            * 	$("#drpdwn").ejDropDownList({ targetID: "carsList",height: 30 });
          * &lt;/script&gt;
            * @memberof ejDropDownList
            * @instance
            */
            height: "",
            /**		
            * Defines the width of the DropDownList textbox.
            * @default Null
            * @type {string | number}
            * @example 
			* &lt;input type="text" id="drpdwn" /&gt; <br> 
			*  &lt;div id="carsList"&gt;
			*    &lt;ul&gt;
			*       &lt;li&gt;Audi A4&lt;/li&gt;
			*       &lt;li&gt;Audi A5&lt;/li&gt;
			*       &lt;li&gt;Audi A6&lt;/li&gt;
			*       &lt;li&gt;Audi A7&lt;/li&gt;
			*       &lt;li&gt;Audi A8&lt;/li&gt;
			*    &lt;/ul&gt;
			*  &lt;/div&gt;<br/> 
			* &lt;script&gt;		
            * //Initialize the DropDownList width property with the width value specified
            * 	$("#drpdwn").ejDropDownList({ targetID: "carsList",width: 250 });
           * &lt;/script&gt;
            * @memberof ejDropDownList
            * @instance
            */
            width: "",
            /**		
            * Defines the popupHeight of the suggestion box.
            * @default "152px"
            * @type {string | number}
            * @example 
			* &lt;input type="text" id="drpdwn" /&gt; <br> 
			*  &lt;div id="carsList"&gt;
			*    &lt;ul&gt;
			*       &lt;li&gt;Audi A4&lt;/li&gt;
			*       &lt;li&gt;Audi A5&lt;/li&gt;
			*       &lt;li&gt;Audi A6&lt;/li&gt;
			*       &lt;li&gt;Audi A7&lt;/li&gt;
			*       &lt;li&gt;Audi A8&lt;/li&gt;
			*    &lt;/ul&gt;
			*  &lt;/div&gt;<br/> 
			* &lt;script&gt;		
            * //Initialize the DropDownList popupHeight property with the  value specified
            * 	$("#drpdwn").ejDropDownList({ targetID: "carsList",popupHeight: '152px' });
             * &lt;/script&gt;
            * @memberof ejDropDownList
            * @instance
            */
            popupHeight: "152px",
            /**		
            * Defines the popupWidth of the suggestion box.
            * @default "auto"
            * @type {string | number}
            * @example 
			* &lt;input type="text" id="drpdwn" /&gt; <br> 
			*  &lt;div id="carsList"&gt;
			*    &lt;ul&gt;
			*       &lt;li&gt;Audi A4&lt;/li&gt;
			*       &lt;li&gt;Audi A5&lt;/li&gt;
			*       &lt;li&gt;Audi A6&lt;/li&gt;
			*       &lt;li&gt;Audi A7&lt;/li&gt;
			*       &lt;li&gt;Audi A8&lt;/li&gt;
			*    &lt;/ul&gt;
			*  &lt;/div&gt;<br/> 
			* &lt;script&gt;		
            * //Initialize the DropDownList popupWidth property with the  value specified
            * 	$("#drpdwn").ejDropDownList({ targetID: "carsList",popupWidth: '152px' });
            * &lt;/script&gt;
            * @memberof ejDropDownList
            * @instance
            */
            popupWidth: "auto",
            /**		
            * Specifies the targetID for DropDownList.
            * @default null
            * @type {string}
            * @example 
			* &lt;input type="text" id="drpdwn" /&gt; <br> 
			*  &lt;div id="carsList"&gt;
			*    &lt;ul&gt;
			*       &lt;li&gt;Audi A4&lt;/li&gt;
			*       &lt;li&gt;Audi A5&lt;/li&gt;
			*       &lt;li&gt;Audi A6&lt;/li&gt;
			*       &lt;li&gt;Audi A7&lt;/li&gt;
			*       &lt;li&gt;Audi A8&lt;/li&gt;
			*    &lt;/ul&gt;
			*  &lt;/div&gt;<br/> 
			* &lt;script&gt;		
            * // To set targetID API value during initialization  . 	
            * 	$("#drpdwn").ejDropDownList({ targetID: "carsList" });
           * &lt;/script&gt;
            * @memberof ejDropDownList
            * @instance
            */
            targetID: null,
            /**		
            * Specifies the template for DropDownList.
            * @default null
            * @type {string}
            * @example <br/> 
			* &lt;input type="text" id="drpdwn" /&gt; <br> 
			* &lt;script&gt;		
            * // To set template API value during initialization  . 	
            * $("#drpdwn").ejDropDownList({ dataSource: window.drpdwnempList, template: '&lt;img class="eimg" src="styles/images/Employee/${eimg}.png" alt="employee" height="50px" width="50px"/&gt;' +
            *   '&lt;div class="ename"&gt; ${text} &lt;/div&gt;&lt;div class="desig"&gt; ${desig} &lt;/div&gt;&lt;div class="cont"&gt; ${country} &lt;/div&gt;',width: "200px"});
            * &lt;/script&gt;
            * @memberof ejDropDownList
            * @instance
            */
            template: null,
            /**		
            * Specifies the selectedItemIndex   for DropDownList.
            * @default null
            * @type {number}
            * @example 
			* &lt;input type="text" id="drpdwn" /&gt; <br> 
			*  &lt;div id="carsList"&gt;
			*    &lt;ul&gt;
			*       &lt;li&gt;Audi A4&lt;/li&gt;
			*       &lt;li&gt;Audi A5&lt;/li&gt;
			*       &lt;li&gt;Audi A6&lt;/li&gt;
			*       &lt;li&gt;Audi A7&lt;/li&gt;
			*       &lt;li&gt;Audi A8&lt;/li&gt;
			*    &lt;/ul&gt;
			*  &lt;/div&gt;<br/> 
			* &lt;script&gt;		
            * // To set selectedItemIndex   API value during initialization  . 	
            * 	$("#drpdwn").ejDropDownList({  targetID: "carsList",selectedItemIndex  : 2 });
            * &lt;/script&gt;
            * @memberof ejDropDownList
            * @instance
            */
            selectedItemIndex: null,
            /**		
            * Specifies the selectedItems  for DropDownList.
			* @remarks
            * If We would use the selectedItems option,we will follow this:
            *  1.To enable the checkbox option as true{showCheckbox:true} See {@link ejDropDownList#showCheckbox}
            * @default []
            * @type {integerarray}
            * @example 
			* &lt;input type="text" id="drpdwn" /&gt; <br> 
			*  &lt;div id="carsList"&gt;
			*    &lt;ul&gt;
			*       &lt;li&gt;Audi A4&lt;/li&gt;
			*       &lt;li&gt;Audi A5&lt;/li&gt;
			*       &lt;li&gt;Audi A6&lt;/li&gt;
			*       &lt;li&gt;Audi A7&lt;/li&gt;
			*       &lt;li&gt;Audi A8&lt;/li&gt;
			*    &lt;/ul&gt;
			*  &lt;/div&gt;<br/> 
			* &lt;script&gt;		
            * // To set selectedItems   API value during initialization  . 	
            * 	$("#drpdwn").ejDropDownList({  targetID: "carsList",showCheckbox: true, selectedItems  : [1,2] });
            * &lt;/script&gt;
            * @memberof ejDropDownList
            * @instance
            */
            selectedItems: [],
            /**		
            * cascadeTois used in cascading Dropdown list scenario, to map the child Dropdown list widget in the parent Dropdown list widget. By selecting an option in the parent dropdown, the child DropDownList has to load the corresponding value regarding the parent Dropdown. 
            * @default null
            * @type {string}
            * @example 
			* &lt;div style="float: left;"&gt;
			*       &lt;span class="txt"&gt;Select Group&lt;/span&gt;
			*       &lt;input id="groupsList" type="text" /&gt;
			* &lt;/div&gt;
			*
			* &lt;div style="float: right;"&gt;
			*        &lt;span class="txt"&gt;Select Country&lt;/span&gt;
			*        &lt;input id="countryList" type="text"/&gt;
			* &lt;/div&gt;
			* &lt;script&gt;
			*  var groups = [
			*          { parentId: 'a', text: "Group A" },
			*          { parentId: 'b', text: "Group B" },
			*          { parentId: 'c', text: "Group C" },
			*          { parentId: 'd', text: "Group D" },
			*          { parentId: 'e', text: "Group E" }]
			*            //first level child
			*            var countries = [{ value: 11, parentId: 'a', text: "Algeria", sprite: "flag-dz" },
			*           { value: 12, parentId: 'a', text: "Armenia", sprite: "flag-am" },
			*           { value: 13, parentId: 'a', text: "Bangladesh", sprite: "flag-bd" },
			*           { value: 14, parentId: 'a', text: "Cuba", sprite: "flag-cu" },
			*           { value: 15, parentId: 'b', text: "Denmark", sprite: "flag-dk" },
			*           { value: 16, parentId: 'b', text: "Egypt", sprite: "flag-eg" },
			*           { value: 17, parentId: 'c', text: "Finland", sprite: "flag-fi" },
			*           { value: 18, parentId: 'c', text: "India", sprite: "flag-in" },
			*           { value: 19, parentId: 'c', text: "Malaysia", sprite: "flag-my" },
			*           { value: 20, parentId: 'd', text: "New Zealand", sprite: "flag-nz" },
			*           { value: 21, parentId: 'd', text: "Norway", sprite: "flag-no" },
			*           { value: 22, parentId: 'd', text: "Poland", sprite: "flag-pl" },
			*           { value: 23, parentId: 'e', text: "Romania", sprite: "flag-ro" },
			*           { value: 24, parentId: 'e', text: "Singapore", sprite: "flag-sg" },
			*           { value: 25, parentId: 'e', text: "Thailand", sprite: "flag-th" },
			*           { value: 26, parentId: 'e', text: "Ukraine", sprite: "flag-ua" }]
			* // To set cascadeTo API value during initialization  . 
			*            $('#groupsList').ejDropDownList({
			*                dataSource: groups,
			*                fields: { value: "parentId" },
			*                cascadeTo: 'countryList'
			*            });
			*            $('#countryList').ejDropDownList({
			*                dataSource: countries,
			*                enabled:false
			*            });
            * &lt;/script&gt;
            * @memberof ejDropDownList
            * @instance
            */
            cascadeTo: null,
            /**		
            * DropDownList textbox to be displayed with rounded corner style.
            * @default true
            * @type {boolean}
            * @example 
			* &lt;input type="text" id="drpdwn" /&gt; <br> 
			*  &lt;div id="carsList"&gt;
			*    &lt;ul&gt;
			*       &lt;li&gt;Audi A4&lt;/li&gt;
			*       &lt;li&gt;Audi A5&lt;/li&gt;
			*       &lt;li&gt;Audi A6&lt;/li&gt;
			*       &lt;li&gt;Audi A7&lt;/li&gt;
			*       &lt;li&gt;Audi A8&lt;/li&gt;
			*    &lt;/ul&gt;
			*  &lt;/div&gt;<br/> 
			* &lt;script&gt;		
            * // Initialize the DropDownList with the showRoundedCorner value specified.
            * $("#drpdwn").ejDropDownList({targetID: "carsList",showRoundedCorner: true });
            * &lt;/script&gt;
            * @memberof ejDropDownList
            * @instance
            */
            showRoundedCorner: false,
            /**		
            * DropDownList textbox to be displayed with Popup shown.
            * @default true
            * @type {boolean}
            * @example 
			* &lt;input type="text" id="drpdwn" /&gt; <br> 
			*  &lt;div id="carsList"&gt;
			*    &lt;ul&gt;
			*       &lt;li&gt;Audi A4&lt;/li&gt;
			*       &lt;li&gt;Audi A5&lt;/li&gt;
			*       &lt;li&gt;Audi A6&lt;/li&gt;			
			*    &lt;/ul&gt;
			*  &lt;/div&gt;<br/> 
			* &lt;script&gt;		
            * // Initialize the DropDownList with the showPopupOnLoad value specified.
            * $("#drpdwn").ejDropDownList({targetID: "carsList",showPopupOnLoad: true });
           * &lt;/script&gt;
            * @memberof ejDropDownList
            * @instance
            */
            showPopupOnLoad: false,
            /**		
            * Sets the DropDownList textbox direction as right to left alignment.
            * @default false
            * @type {boolean}
            * @example 
			* &lt;input type="text" id="drpdwn" /&gt; <br> 
			*  &lt;div id="carsList"&gt;
			*    &lt;ul&gt;
			*       &lt;li&gt;Audi A4&lt;/li&gt;
			*       &lt;li&gt;Audi A5&lt;/li&gt;
			*       &lt;li&gt;Audi A6&lt;/li&gt;
			*       &lt;li&gt;Audi A7&lt;/li&gt;
			*       &lt;li&gt;Audi A8&lt;/li&gt;
			*    &lt;/ul&gt;
			*  &lt;/div&gt;<br/> 
			* &lt;script&gt;		
            * // Initialize the DropDownList with the enableRTL  value specified.
            * $("#drpdwn").ejDropDownList({ targetID: "carsList",enableRTL : true });
            * &lt;/script&gt;
            * @memberof ejDropDownList
            * @instance
            */
            enableRTL: false,
            /**		
            * When this property sets to false, it disables the DropDownList control.
            * @default true
            * @type {boolean}
            * @example 
			* &lt;input type="text" id="drpdwn" /&gt; <br> 
			*  &lt;div id="carsList"&gt;
			*    &lt;ul&gt;
			*       &lt;li&gt;Audi A4&lt;/li&gt;
			*       &lt;li&gt;Audi A5&lt;/li&gt;
			*       &lt;li&gt;Audi A6&lt;/li&gt;
			*       &lt;li&gt;Audi A7&lt;/li&gt;
			*       &lt;li&gt;Audi A8&lt;/li&gt;
			*    &lt;/ul&gt;
			*  &lt;/div&gt;<br/> 
			* &lt;script&gt;		
            * // Initialize the DropDownList with the enabled  value specified.
            * $("#drpdwn").ejDropDownList({ targetID: "carsList",enabled : false });
            * &lt;/script&gt;
            * @memberof ejDropDownList
            * @instance
            */
            enabled: true,
            /**		
            * Sets the case sensitivity of the search operation..
            * @default false
            * @type {boolean}
            * @example 
			* &lt;input type="text" id="drpdwn" /&gt; <br> 
			*  &lt;div id="carsList"&gt;
			*    &lt;ul&gt;
			*       &lt;li&gt;Audi A4&lt;/li&gt;
			*       &lt;li&gt;Audi A5&lt;/li&gt;
			*       &lt;li&gt;Audi A6&lt;/li&gt;
			*       &lt;li&gt;Audi A7&lt;/li&gt;
			*       &lt;li&gt;Audi A8&lt;/li&gt;
			*    &lt;/ul&gt;
			*  &lt;/div&gt;<br/> 
			* &lt;script&gt;		
            * // Initialize the DropDownList with the caseSensitiveSearch value specified.
            * $("#drpdwn").ejDropDownList({ targetID: "carsList",caseSensitiveSearch: true });
            * &lt;/script&gt;
            * @memberof ejDropDownList
            * @instance
            */
            caseSensitiveSearch: false,
            /**		
            * Specifies the multi selection option in DropDownList with the help of checkbox control. For this we have to enable it by showCheckbox option true.
            * @default false
            * @type {boolean}
            * @example 
			* &lt;input type="text" id="drpdwn" /&gt; <br> 
			*  &lt;div id="carsList"&gt;
			*    &lt;ul&gt;
			*       &lt;li&gt;Audi A4&lt;/li&gt;
			*       &lt;li&gt;Audi A5&lt;/li&gt;
			*       &lt;li&gt;Audi A6&lt;/li&gt;
			*       &lt;li&gt;Audi A7&lt;/li&gt;
			*       &lt;li&gt;Audi A8&lt;/li&gt;
			*    &lt;/ul&gt;
			*  &lt;/div&gt;<br/> 
			* &lt;script&gt;		
            * // Initialize the DropDownList with the showCheckbox value specified.
            * $("#drpdwn").ejDropDownList({ targetID: "carsList",showCheckbox: true });
            * &lt;/script&gt;
            * @memberof ejDropDownList
            * @instance
            */
            showCheckbox: false,
            /**		
            * Specifies to select all the items of DropDownList can be done with the help of this checkAll property, it supports only when the showCheckbox property true.
			* @remarks
			* If We would use the checkAll option,we will follow this:
            *  1.To enable the checkbox option as true{showCheckbox:true}See {@link ejDropDownList#showCheckbox}
            * @default false
            * @type {boolean}
            * @example 
			* &lt;input type="text" id="drpdwn" /&gt; <br> 
			*  &lt;div id="carsList"&gt;
			*    &lt;ul&gt;
			*       &lt;li&gt;Audi A4&lt;/li&gt;
			*       &lt;li&gt;Audi A5&lt;/li&gt;
			*       &lt;li&gt;Audi A6&lt;/li&gt;
			*       &lt;li&gt;Audi A7&lt;/li&gt;
			*       &lt;li&gt;Audi A8&lt;/li&gt;
			*    &lt;/ul&gt;
			*  &lt;/div&gt;<br/> 
			* &lt;script&gt;		
            * // Initialize the DropDownList with the checkAll value specified.
            * $("#drpdwn").ejDropDownList({ targetID: "carsList",showCheckbox: true, checkAll: true });
           * &lt;/script&gt;
            * @memberof ejDropDownList
            * @instance
            */
            checkAll: false,
            /**		
            * Specifies to unselect all the items of DropDownList can be done with the help of this checkAll property, it supports only when the showCheckbox property true.
			* @remarks
			* If We would use the uncheckAll option,we will follow this:
            *  1.To enable the checkbox option as true{showCheckbox:true}See {@link ejDropDownList#showCheckbox}
            * @default false
            * @type {boolean}
            * @example 
			* &lt;input type="text" id="drpdwn" /&gt; <br> 
			*  &lt;div id="carsList"&gt;
			*    &lt;ul&gt;
			*       &lt;li&gt;Audi A4&lt;/li&gt;
			*       &lt;li&gt;Audi A5&lt;/li&gt;
			*       &lt;li&gt;Audi A6&lt;/li&gt;
			*       &lt;li&gt;Audi A7&lt;/li&gt;
			*       &lt;li&gt;Audi A8&lt;/li&gt;
			*    &lt;/ul&gt;
			*  &lt;/div&gt;<br/> 
			* &lt;script&gt;		
            * // Initialize the DropDownList with the uncheckAll value specified.
            * $("#drpdwn").ejDropDownList({ targetID: "carsList",showCheckbox: true, uncheckAll: true });
           * &lt;/script&gt;
            * @memberof ejDropDownList
            * @instance
            */
            uncheckAll: false,
            /**		
            * Save current model value to browser cookies for state maintains. While refresh the DropDownList control page retains the model value apply from browser cookies.  
            * @default false
            * @type {boolean}
            * @example 
			* &lt;input type="text" id="drpdwn" /&gt; <br> 
			*  &lt;div id="carsList"&gt;
			*    &lt;ul&gt;
			*       &lt;li&gt;Audi A4&lt;/li&gt;
			*       &lt;li&gt;Audi A5&lt;/li&gt;
			*       &lt;li&gt;Audi A6&lt;/li&gt;
			*       &lt;li&gt;Audi A7&lt;/li&gt;
			*       &lt;li&gt;Audi A8&lt;/li&gt;
			*    &lt;/ul&gt;
			*  &lt;/div&gt;<br/> 
			* &lt;script&gt;		
            * // Initialize the DropDownList with the enablePersistence  value specified.
            * $("#drpdwn").ejDropDownList({ targetID: "carsList",enablePersistence : false });
            * &lt;/script&gt;
            * @memberof ejDropDownList
            * @instance
            */
            enablePersistence: false,
            /**		
            * Specifies to perform incremental search for selection of items from DropDownList can be done with the help of this property, this will helpful in selecting the item using the typed character.
            * @default true
            * @type {boolean}
            * @example 
			* &lt;input type="text" id="drpdwn" /&gt; <br> 
			*  &lt;div id="carsList"&gt;
			*    &lt;ul&gt;
			*       &lt;li&gt;Audi A4&lt;/li&gt;
			*       &lt;li&gt;Audi A5&lt;/li&gt;
			*       &lt;li&gt;Audi A6&lt;/li&gt;
			*       &lt;li&gt;Audi A7&lt;/li&gt;
			*       &lt;li&gt;Audi A8&lt;/li&gt;
			*    &lt;/ul&gt;
			*  &lt;/div&gt;<br/> 
			* &lt;script&gt;		
            * // Initialize the enableIncrementalSearch with the value specified.
            * $("#drpdwn").ejDropDownList({targetID: "carsList",enableIncrementalSearch: true });
            * &lt;/script&gt;
            * @memberof ejDropDownList
            * @instance
            */
            enableIncrementalSearch: false,
            /**		
            * Indicates that the DropDownList textbox values can only be read.
            * @default true
            * @type {boolean}
            * @example 
			* &lt;input type="text" id="drpdwn" /&gt; <br> 
			*  &lt;div id="carsList"&gt;
			*    &lt;ul&gt;
			*       &lt;li&gt;Audi A4&lt;/li&gt;
			*       &lt;li&gt;Audi A5&lt;/li&gt;
			*       &lt;li&gt;Audi A6&lt;/li&gt;
			*       &lt;li&gt;Audi A7&lt;/li&gt;
			*       &lt;li&gt;Audi A8&lt;/li&gt;
			*    &lt;/ul&gt;
			*  &lt;/div&gt;<br/> 
			* &lt;script&gt;		
            * // Initialize the DropDownList with the readOnly value specified.
            * $("#drpdwn").ejDropDownList({targetID: "carsList",readOnly: true });
           * &lt;/script&gt;
            * @memberof ejDropDownList
            * @instance
            */
            readOnly: false,
            /**		
            * Indicates that the multiSelectMode values can only be read.
			* @remarks
			* If We would use the uncheckAll option,we will follow this:
            *  1.To enable the checkbox option as true{showCheckbox:true}See {@link ejDropDownList#showCheckbox}
            * @default true
            * @type {boolean}
            * @example 
			* &lt;input type="text" id="drpdwn" /&gt; <br> 
			*  &lt;div id="carsList"&gt;
			*    &lt;ul&gt;
			*       &lt;li&gt;Audi A4&lt;/li&gt;
			*       &lt;li&gt;Audi A5&lt;/li&gt;
			*       &lt;li&gt;Audi A6&lt;/li&gt;
			*       &lt;li&gt;Audi A7&lt;/li&gt;
			*       &lt;li&gt;Audi A8&lt;/li&gt;
			*    &lt;/ul&gt;
			*  &lt;/div&gt;<br/>
			* &lt;script&gt;
            * // Initialize the allowMultiSelection with the value specified.
            * $("#drpdwn").ejDropDownList({targetID: "carsList",showCheckbox: true,allowMultiSelection: true });
            * &lt;/script&gt;
            * @memberof ejDropDownList
            * @instance
            */
            allowMultiSelection: false,

            /**     
           * Fires when create successfully.
           * @event
           * @name ejDropDownList#create 	
           * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
           * @param {object}  argument.model returns the DropDownList model
           * @param {string}  argument.type returns the name of the event
           * @example 
		   * &lt;input type="text" id="drpdwn" /&gt; <br> 
			*  &lt;div id="carsList"&gt;
			*    &lt;ul&gt;
			*       &lt;li&gt;Audi A4&lt;/li&gt;
			*       &lt;li&gt;Audi A5&lt;/li&gt;
			*       &lt;li&gt;Audi A6&lt;/li&gt;
			*       &lt;li&gt;Audi A7&lt;/li&gt;
			*       &lt;li&gt;Audi A8&lt;/li&gt;
			*    &lt;/ul&gt;
			*  &lt;/div&gt;<br/> 
			* &lt;script&gt;		
           * //change event for DropDownList
           * $("#drpdwn").ejDropDownList({ 
		   *    targetID: "carsList",
           *   	create: function(args) {}
           * });
          * &lt;/script&gt;		   
           * @memberof ejDropDownList
           * @instance
           */
            create: null,
            /**     
          * Fires when popupHide successfully.
          * @event
          * @name ejDropDownList#popupHide 	
          * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
          * @param {object}  argument.model returns the DropDownList model
          * @param {string}  argument.type returns the name of the event
          * @param {string}  argument.text returns the selected text
          * @param {string}  argument.value returns the selected value
          * @example 
		  * &lt;input type="text" id="drpdwn" /&gt; <br> 
			*  &lt;div id="carsList"&gt;
			*    &lt;ul&gt;
			*  &lt;/div&gt;<br/> 
			* &lt;script&gt;		
          * //popupHide event for DropDownList
          * $("#drpdwn").ejDropDownList({ 
		  *		targetID: "carsList",
          *   	popupHide: function(args) {}
          * });
          * &lt;/script&gt;		  
          * @memberof ejDropDownList
          * @instance
          */
            popupHide: null,
            /**     
         * Fires when popupShown successfully.
         * @event
         * @name ejDropDownList#popupShown 	
         * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
         * @param {object}  argument.model returns the DropDownList model
         * @param {string}  argument.type returns the name of the event
         * @param {string}  argument.text returns the selected text
          * @param {string}  argument.value returns the selected value
         * @example 
		 * &lt;input type="text" id="drpdwn" /&gt; <br> 
			*  &lt;div id="carsList"&gt;
			*    &lt;ul&gt;
			*       &lt;li&gt;Audi A4&lt;/li&gt;
			*       &lt;li&gt;Audi A5&lt;/li&gt;
			*       &lt;li&gt;Audi A6&lt;/li&gt;
			*       &lt;li&gt;Audi A7&lt;/li&gt;
			*       &lt;li&gt;Audi A8&lt;/li&gt;
			*    &lt;/ul&gt;
			*  &lt;/div&gt;<br/> 
			* &lt;script&gt;		
         * //popupShown event for DropDownList
         * $("#drpdwn").ejDropDownList({ 
		 *		targetID: "carsList",
         *   	popupShown: function(args) {}
         * });
          * &lt;/script&gt;		 
         * @memberof ejDropDownList
         * @instance
         */
            popupShown: null,
            /**     
         * Fires beore the popup going to display.
         * @event
         * @name ejDropDownList#beforePopupShown 	
         * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
         * @param {object}  argument.model returns the DropDownList model
         * @param {string}  argument.type returns the name of the event
         * @param {string}  argument.text returns the selected text
          * @param {string}  argument.value returns the selected value
         * @example 
		 * &lt;input type="text" id="drpdwn" /&gt; <br> 
			*  &lt;div id="carsList"&gt;
			*    &lt;ul&gt;
			*       &lt;li&gt;Audi A4&lt;/li&gt;
			*       &lt;li&gt;Audi A5&lt;/li&gt;
			*       &lt;li&gt;Audi A6&lt;/li&gt;
			*       &lt;li&gt;Audi A7&lt;/li&gt;
			*       &lt;li&gt;Audi A8&lt;/li&gt;
			*    &lt;/ul&gt;
			*  &lt;/div&gt;<br/> 
			* &lt;script&gt;		
         * //beforePopupShown event for DropDownList
         * $("#drpdwn").ejDropDownList({ 
		 *		targetID: "carsList",
         *   	beforePopupShown: function(args) {}
         * });
          * &lt;/script&gt;		 
         * @memberof ejDropDownList
         * @instance
         */
            beforePopupShown: null,
            /**
            * Fires beore the popup going to hide.
         * @event
         * @name ejDropDownList#beforePopupHide 	
         * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
         * @param {object}  argument.model returns the DropDownList model
         * @param {string}  argument.type returns the name of the event
         * @param {string}  argument.text returns the selected text
          * @param {string}  argument.value returns the selected value
         * @example 
		 * &lt;input type="text" id="drpdwn" /&gt; <br> 
			*  &lt;div id="carsList"&gt;
			*    &lt;ul&gt;
			*       &lt;li&gt;Audi A4&lt;/li&gt;
			*       &lt;li&gt;Audi A5&lt;/li&gt;
			*       &lt;li&gt;Audi A6&lt;/li&gt;
			*       &lt;li&gt;Audi A7&lt;/li&gt;
			*       &lt;li&gt;Audi A8&lt;/li&gt;
			*    &lt;/ul&gt;
			*  &lt;/div&gt;<br/> 
			* &lt;script&gt;		
         * //beforePopupHide event for DropDownList
         * $("#drpdwn").ejDropDownList({ 
		 *		targetID: "carsList",
         *   	beforePopupHide: function(args) {}
         * });
          * &lt;/script&gt;		 
         * @memberof ejDropDownList
         * @instance
         */
            beforePopupHide: null,
            /**     
           * Fires when change successfully.
           * @event
           * @name ejDropDownList#change 	
           * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
           * @param {object}  argument.model returns the DropDownList model
           * @param {string}  argument.type returns the name of the event
           * @example 
		   * &lt;input type="text" id="drpdwn" /&gt; <br> 
			*  &lt;div id="carsList"&gt;
			*    &lt;ul&gt;
			*       &lt;li&gt;Audi A4&lt;/li&gt;
			*       &lt;li&gt;Audi A5&lt;/li&gt;
			*       &lt;li&gt;Audi A6&lt;/li&gt;
			*       &lt;li&gt;Audi A7&lt;/li&gt;
			*       &lt;li&gt;Audi A8&lt;/li&gt;
			*    &lt;/ul&gt;
			*  &lt;/div&gt;<br/> 
			* &lt;script&gt;		
           * //change event for DropDownList
           * $("#drpdwn").ejDropDownList({ 
		   *		targetID: "carsList",
           *   	change: function(args) {}
           * }); 
          * &lt;/script&gt;		   
           * @memberof ejDropDownList
           * @instance
           */
            change: null,
            /**     
          * Fires when select successfully.
          * @event
          * @name ejDropDownList#select 	
          * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
          * @param {object}  argument.model returns the DropDownList model
          * @param {string}  argument.type returns the name of the event
          * @example 
		  * &lt;input type="text" id="drpdwn" /&gt; <br> 
			*  &lt;div id="carsList"&gt;
			*    &lt;ul&gt;
			*       &lt;li&gt;Audi A4&lt;/li&gt;
			*       &lt;li&gt;Audi A5&lt;/li&gt;
			*       &lt;li&gt;Audi A6&lt;/li&gt;
			*       &lt;li&gt;Audi A7&lt;/li&gt;
			*       &lt;li&gt;Audi A8&lt;/li&gt;
			*    &lt;/ul&gt;
			*  &lt;/div&gt;<br/> 
			* &lt;script&gt;		
          * //select event for DropDownList
          * $("#drpdwn").ejDropDownList({ 
		  *		targetID: "carsList",
          *   	select: function(args) {}
          * });      
		  * &lt;/script&gt;
          * @memberof ejDropDownList
          * @instance
          */
            select: null,
            /**     
          * Fires when checkChange successfully.
          * @event
          * @name ejDropDownList#checkChange 	
          * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
          * @param {object}  argument.model returns the DropDownList model
          * @param {string}  argument.type returns the name of the event
          * @example 
		  * &lt;input type="text" id="drpdwn" /&gt; <br> 
			*  &lt;div id="carsList"&gt;
			*    &lt;ul&gt;
			*       &lt;li&gt;Audi A4&lt;/li&gt;
			*       &lt;li&gt;Audi A5&lt;/li&gt;
			*       &lt;li&gt;Audi A6&lt;/li&gt;
			*       &lt;li&gt;Audi A7&lt;/li&gt;
			*       &lt;li&gt;Audi A8&lt;/li&gt;
			*    &lt;/ul&gt;
			*  &lt;/div&gt;<br/> 
			* &lt;script&gt;		
          * //checkChange event for DropDownList
          * $("#drpdwn").ejDropDownList({ 
		  *		targetID: "carsList",
          *   	checkChange: function(args) {}
          * }); 
          * &lt;/script&gt;		  
          * @memberof ejDropDownList
          * @instance
          */
            checkChange: null,
            /**     
          * Fires when destroy successfully.
          * @event
          * @name ejDropDownList#destroy 	
          * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
          * @param {object}  argument.model returns the DropDownList model
          * @param {string}  argument.type returns the name of the event
          * @example 
		  * &lt;input type="text" id="drpdwn" /&gt; <br> 
			*  &lt;div id="carsList"&gt;
			*    &lt;ul&gt;
			*       &lt;li&gt;Audi A4&lt;/li&gt;
			*       &lt;li&gt;Audi A5&lt;/li&gt;
			*       &lt;li&gt;Audi A6&lt;/li&gt;
			*       &lt;li&gt;Audi A7&lt;/li&gt;
			*       &lt;li&gt;Audi A8&lt;/li&gt;
			*    &lt;/ul&gt;
			*  &lt;/div&gt;<br/> 
			* &lt;script&gt;		
          * //destroy event for DropDownList
          * $("#drpdwn").ejDropDownList({ 
		  *		targetID: "carsList",
          *   	destroy: function(args) {}
          * });
          * &lt;/script&gt;		  
          * @memberof ejDropDownList
          * @instance
          */
            destroy: null

        },
        /**
         * Specify the data types for default properties 
		 * @private
         */
        dataTypes: {
            cssClass: "string",
            itemsCount: "number",
            watermarkText: "string",
            popupHeight: "string",
            popupWidth: "string",
            template: "string",
            selectedItemIndex: "number",
            cascadeTo: "string",
            showRoundedCorner: "boolean",
            showPopupOnLoad: "boolean",
            enableRTL: "boolean",
            enablePersistence: "boolean",
            enabled: "boolean",
            readOnly: "boolean",
            allowMultiSelection: "boolean",
            dataSource: "data",
            query: "data",
            fields: "data",
            selectedItems: "array",
			enableAnimation: "boolean"
        },

        observables: ["value"],
        value: ej.util.valueFunction("value"),
        /**
      * To enable the DropDownList  
      * @return jQuery
      * @example 
       * &lt;input type="text" id="drpdwn" /&gt; <br> 
			*  &lt;div id="carsList"&gt;
			*    &lt;ul&gt;
			*       &lt;li&gt;Audi A4&lt;/li&gt;
			*       &lt;li&gt;Audi A5&lt;/li&gt;
			*       &lt;li&gt;Audi A6&lt;/li&gt;
			*       &lt;li&gt;Audi A7&lt;/li&gt;
			*       &lt;li&gt;Audi A8&lt;/li&gt;
			*    &lt;/ul&gt;
			*  &lt;/div&gt;<br/> 
			* &lt;script&gt;
      * // Create DropDownList
	  * $('#drpdwn').ejDropDownList({targetID: "carsList",value:"Audi A5"});
      * var DropDownListObj  = $("#drpdwn").data("ejDropDownList");
      * DropDownListObj.enable(); // enable the DropDownList
      * &lt;/script&gt;
      * @example 
      * &lt;input type="text" id="drpdwn" /&gt; <br> 
			*  &lt;div id="carsList"&gt;
			*    &lt;ul&gt;
			*       &lt;li&gt;Audi A4&lt;/li&gt;
			*       &lt;li&gt;Audi A5&lt;/li&gt;
			*       &lt;li&gt;Audi A6&lt;/li&gt;
			*       &lt;li&gt;Audi A7&lt;/li&gt;
			*       &lt;li&gt;Audi A8&lt;/li&gt;
			*    &lt;/ul&gt;
			*  &lt;/div&gt;<br/> 
	  * &lt;script&gt;
      * // Create DropDownList
	  * $('#drpdwn').ejDropDownList({targetID: "carsList",value:"Audi A5"});
      * $('#drpdwn').ejDropDownList("enable"); 	
      * &lt;/script&gt;
      *@memberof ejDropDownList
      * @instance
      */
        //public methods
        enable: function () {
            if (this.element.hasClass("e-disable")) {
                this.target.disabled = false;
                this.model.enabled = true;
                this.element.removeClass('e-disable');
                this.dropdownbutton.removeClass('e-disable');
                this.element.bind("mousedown", $.proxy(this._OnDropdownClick, this));
                this.dropdownbutton.bind("mousedown", $.proxy(this._OnDropdownClick, this));
                if (this.model.allowMultiSelection) this._ulBox.removeClass("e-disable");
            }
        },
        /**
     * To disable the DropDownList  
     * @return jQuery
     * @example 
     * &lt;input type="text" id="drpdwn" /&gt; <br> 
			*  &lt;div id="carsList"&gt;
			*    &lt;ul&gt;
			*       &lt;li&gt;Audi A4&lt;/li&gt;
			*       &lt;li&gt;Audi A5&lt;/li&gt;
			*       &lt;li&gt;Audi A6&lt;/li&gt;
			*       &lt;li&gt;Audi A7&lt;/li&gt;
			*       &lt;li&gt;Audi A8&lt;/li&gt;
			*    &lt;/ul&gt;
			*  &lt;/div&gt;<br/> 
	  * &lt;script&gt;
     * // Create DropDownList
	 * $('#drpdwn').ejDropDownList({targetID: "carsList",value:"Audi A5"});
     * var DropDownListObj  = $("#drpdwn").data("ejDropDownList");
     * DropDownListObj.disable(); // disable the DropDownList
     * &lt;/script&gt;
     * @example 
    * &lt;input type="text" id="drpdwn" /&gt; <br> 
			*  &lt;div id="carsList"&gt;
			*    &lt;ul&gt;
			*       &lt;li&gt;Audi A4&lt;/li&gt;
			*       &lt;li&gt;Audi A5&lt;/li&gt;
			*       &lt;li&gt;Audi A6&lt;/li&gt;
			*       &lt;li&gt;Audi A7&lt;/li&gt;
			*       &lt;li&gt;Audi A8&lt;/li&gt;
			*    &lt;/ul&gt;
			*  &lt;/div&gt;<br/> 
	  * &lt;script&gt;
     * // Create DropDownList
	 * $('#drpdwn').ejDropDownList({targetID: "carsList",value:"Audi A5"});
     * $('#drpdwn').ejDropDownList("disable"); 	
     * &lt;/script&gt;
     *@memberof ejDropDownList
     * @instance
     */
        disable: function () {
            if (!this.element.hasClass("e-disable")) {
                this.target.disabled = true;
                this.model.enabled = false;
                this.element.addClass('e-disable');
                this.dropdownbutton.addClass('e-disable');
                if (this.model.allowMultiSelection) this._ulBox.addClass("e-disable");
                this.element.unbind("mousedown", $.proxy(this._OnDropdownClick, this));
                this.dropdownbutton.unbind("mousedown", $.proxy(this._OnDropdownClick, this));
            }
        },
        /**
      * Returns the current value selected in the DropDownList textbox.
      * @return jQuery
      * @example 
      * &lt;input type="text" id="drpdwn" /&gt; <br> 
			*  &lt;div id="carsList"&gt;
			*    &lt;ul&gt;
			*       &lt;li&gt;Audi A4&lt;/li&gt;
			*       &lt;li&gt;Audi A5&lt;/li&gt;
			*       &lt;li&gt;Audi A6&lt;/li&gt;
			*       &lt;li&gt;Audi A7&lt;/li&gt;
			*       &lt;li&gt;Audi A8&lt;/li&gt;
			*    &lt;/ul&gt;
			*  &lt;/div&gt;<br/> 
	  * &lt;script&gt;
      * // Create DropDownList
	 * $('#drpdwn').ejDropDownList({targetID: "carsList",value:"Audi A5"});
      * var DropDownListObj  = $("#drpdwn").data("ejDropDownList");
      * DropDownListObj.getValue(); // getValue of the DropDownList text
      * &lt;/script&gt;
      * @example 
      * &lt;input type="text" id="drpdwn" /&gt; <br> 
			*  &lt;div id="carsList"&gt;
			*    &lt;ul&gt;
			*       &lt;li&gt;Audi A4&lt;/li&gt;
			*       &lt;li&gt;Audi A5&lt;/li&gt;
			*       &lt;li&gt;Audi A6&lt;/li&gt;
			*       &lt;li&gt;Audi A7&lt;/li&gt;
			*       &lt;li&gt;Audi A8&lt;/li&gt;
			*    &lt;/ul&gt;
			*  &lt;/div&gt;<br/> 
	  * &lt;script&gt;
      * // Create DropDownList
	 * $('#drpdwn').ejDropDownList({targetID: "carsList",value:"Audi A5"});
      * $('#drpdwn').ejDropDownList("getValue"); 	
      * &lt;/script&gt;
      *@memberof ejDropDownList
      * @instance
      */
        getValue: function () {
            if (this.element.hasClass("e-watermark"))
                return "";
            return this.target.value;
        },
        _setValue: function (value) {
            if (!this.model.enabled) return false;
            if (this.element.hasClass("e-watermark"))
                this.element.removeClass("e-watermark")
            this.element.val(value);
            this.value(value);
            this._hiddenInput.val(value);
            var args = { text: this.element[0].value, selectedText: this.currentValue, itemId: this.selectedIndexValue, value: this.model.value, isChecked: this.checkedStatus };
            this._trigger("change", args);
        },
        _setText: function (text) {
            this.model.text = text;
        },
        _setItemValue: function (itemValue) {
            this.model.itemValue = itemValue;
        },
        _changeWatermark: function (text) {
            if (!this.model.enabled) return false;
            if (this.element.hasClass("e-watermark"))
                this.element.val(text);
        },
        /**
     * popup list hide in the DropDownList textbox.
     * @return jQuery
     * @example 
     * &lt;input type="text" id="drpdwn" /&gt; <br> 
			*  &lt;div id="carsList"&gt;
			*    &lt;ul&gt;
			*       &lt;li&gt;Audi A4&lt;/li&gt;
			*       &lt;li&gt;Audi A5&lt;/li&gt;
			*       &lt;li&gt;Audi A6&lt;/li&gt;
			*       &lt;li&gt;Audi A7&lt;/li&gt;
			*       &lt;li&gt;Audi A8&lt;/li&gt;
			*    &lt;/ul&gt;
			*  &lt;/div&gt;<br/> 
	  * &lt;script&gt;
     * // Create DropDownList
	* $('#drpdwn').ejDropDownList({targetID: "carsList",value:"Audi A5"});
     * var DropDownListObj  = $("#drpdwn").data("ejDropDownList");
     * DropDownListObj.hidePopup(); // hidePopup of the DropDownList 
     * &lt;/script&gt;
     * @example 
    * &lt;input type="text" id="drpdwn" /&gt; <br> 
			*  &lt;div id="carsList"&gt;
			*    &lt;ul&gt;
			*       &lt;li&gt;Audi A4&lt;/li&gt;
			*       &lt;li&gt;Audi A5&lt;/li&gt;
			*       &lt;li&gt;Audi A6&lt;/li&gt;
			*       &lt;li&gt;Audi A7&lt;/li&gt;
			*       &lt;li&gt;Audi A8&lt;/li&gt;
			*    &lt;/ul&gt;
			*  &lt;/div&gt;<br/> 
	  * &lt;script&gt;
     * // Create DropDownList
	* $('#drpdwn').ejDropDownList({targetID: "carsList",value:"Audi A5"});
     * $('#drpdwn').ejDropDownList("hidePopup"); 	
     * &lt;/script&gt;
     *@memberof ejDropDownList
     * @instance
     */
        hidePopup: function () {
            if (!this.model.enabled) return false;
            if (this.ultag.find('li').length > 0)
                this._hideResult();
        },
        /**
   * popup list show in the DropDownList textbox.
   * @return jQuery
   * @example 
  * &lt;input type="text" id="drpdwn" /&gt; <br> 
			*  &lt;div id="carsList"&gt;
			*    &lt;ul&gt;
			*       &lt;li&gt;Audi A4&lt;/li&gt;
			*       &lt;li&gt;Audi A5&lt;/li&gt;
			*       &lt;li&gt;Audi A6&lt;/li&gt;
			*       &lt;li&gt;Audi A7&lt;/li&gt;
			*       &lt;li&gt;Audi A8&lt;/li&gt;
			*    &lt;/ul&gt;
			*  &lt;/div&gt;<br/> 
	  * &lt;script&gt;
   * // Create DropDownList
   * $('#drpdwn').ejDropDownList({targetID: "carsList",value:"Audi A5"});
   * var DropDownListObj  = $("#drpdwn").data("ejDropDownList");
   * DropDownListObj.showPopup(); // 
   * &lt;/script&gt;
   * @example 
  * &lt;input type="text" id="drpdwn" /&gt; <br> 
			*  &lt;div id="carsList"&gt;
			*    &lt;ul&gt;
			*       &lt;li&gt;Audi A4&lt;/li&gt;
			*       &lt;li&gt;Audi A5&lt;/li&gt;
			*       &lt;li&gt;Audi A6&lt;/li&gt;
			*       &lt;li&gt;Audi A7&lt;/li&gt;
			*       &lt;li&gt;Audi A8&lt;/li&gt;
			*    &lt;/ul&gt;
			*  &lt;/div&gt;<br/> 
	  * &lt;script&gt;
   * // Create DropDownList
   * $('#drpdwn').ejDropDownList({targetID: "carsList",value:"Audi A5"});
   * $('#drpdwn').ejDropDownList("showPopup"); 	
   * &lt;/script&gt;
   *@memberof ejDropDownList
   * @instance
   */
        showPopup: function () {
            if (!this.model.enabled) return false;
            if (this.ultag.find('li').length > 0)
                this._showResult();
        },
        /**
     * Clears the text in the DropDownList textbox.
     * @return jQuery
     * @example 
     * &lt;input type="text" id="drpdwn" /&gt; <br> 
			*  &lt;div id="carsList"&gt;
			*    &lt;ul&gt;
			*       &lt;li&gt;Audi A4&lt;/li&gt;
			*       &lt;li&gt;Audi A5&lt;/li&gt;
			*       &lt;li&gt;Audi A6&lt;/li&gt;
			*       &lt;li&gt;Audi A7&lt;/li&gt;
			*       &lt;li&gt;Audi A8&lt;/li&gt;
			*    &lt;/ul&gt;
			*  &lt;/div&gt;<br/> 
	  * &lt;script&gt;
     * // Create DropDownList
	 * $('#drpdwn').ejDropDownList({targetID: "carsList",value:"Audi A5"});
     * var DropDownListObj  = $("#drpdwn").data("ejDropDownList");
     * DropDownListObj.clearText(); // clear the DropDownList text
     * &lt;/script&gt;
     * @example 
    * &lt;input type="text" id="drpdwn" /&gt; <br> 
			*  &lt;div id="carsList"&gt;
			*    &lt;ul&gt;
			*       &lt;li&gt;Audi A4&lt;/li&gt;
			*       &lt;li&gt;Audi A5&lt;/li&gt;
			*       &lt;li&gt;Audi A6&lt;/li&gt;
			*       &lt;li&gt;Audi A7&lt;/li&gt;
			*       &lt;li&gt;Audi A8&lt;/li&gt;
			*    &lt;/ul&gt;
			*  &lt;/div&gt;<br/> 
	  * &lt;script&gt;
     * // Create DropDownList
     * $('#drpdwn').ejDropDownList({targetID: "carsList",value:"Audi A5"});
     * $('#drpdwn').ejDropDownList("clearText"); 	
     * &lt;/script&gt;
     *@memberof ejDropDownList
     * @instance
     */
        clearText: function () {
            if (this.model.enabled) {
                this.element.val("");
                this.value("");
                this.model.text = "";
                if (this.model.watermarkText != null) {
                    if ($.trim(this.element.val()) === '') {
                        this.element.val(this.model.watermarkText);
                        this.element.addClass("e-watermark");
                    }
                }
            }
        },
      
        _selectItemByIndex: function (index) {
            if (!this.model.enabled) return false;
            if (index != null) {
                this._activeItem = index;
                if (this._activeItem < 0) {
                    this._activeItem = 0;
                }
                else if (this._activeItem == this._listSize - 1) {
                    this._activeItem = this._listSize - 1;
                }
                if (this.model.template) {
                    this._templateValue();
                } else {
                    this._enterTextBoxValue();
                }
            }
        },
		
   /**
   * This method is used to unselect a list item in DropDownList using given index field.
   * @example 
   *   &lt;select name="selectIndex" id="drpdwn"&gt;
	*        &lt;option value="Art"&gt;Art&lt;/option&gt;
	*        &lt;option value="Architecture"&gt;Architecture&lt;/option&gt;
	*        &lt;option value="Biographies"&gt;Biographies&lt;/option&gt;
	*        &lt;option value="Business"&gt;Business&lt;/option&gt;
	*        &lt;option value="ComputerIT"&gt;Computer IT&lt;/option&gt;
	*        &lt;option value="Comics"&gt;Comics&lt;/option&gt;
	*        &lt;option value="Cookery"&gt;Cookery&lt;/option&gt;
	*        &lt;option value="Environment"&gt;Environment&lt;/option&gt;
	*        &lt;option value="Fiction"&gt;Fiction&lt;/option&gt;
	*        &lt;option value="Health"&gt;Health&lt;/option&gt;
	*        &lt;option value="Humanities"&gt;Humanities&lt;/option&gt;
	*        &lt;option value="Language"&gt;Language&lt;/option&gt;
	*    &lt;/select&gt;
	  * &lt;script&gt;
   * // Create DropDownList
   *$('#drpdwn').ejDropDownList();
   *var DropDownListObj  = $("#drpdwn").data("ejDropDownList");
   *DropDownListObj.setSelectedValue("Art"); // setSelectedValue for the DropDownList text
   *DropDownListObj.unselectItemByIndex(0); // unselectItemByIndex for the DropDownList text
   * &lt;/script&gt;
   * @example 
    *&lt;select name="selectIndex" id="drpdwn"&gt;
	*        &lt;option value="Art"&gt;Art&lt;/option&gt;
	*        &lt;option value="Architecture"&gt;Architecture&lt;/option&gt;
	*        &lt;option value="Biographies"&gt;Biographies&lt;/option&gt;
	*        &lt;option value="Business"&gt;Business&lt;/option&gt;
	*        &lt;option value="ComputerIT"&gt;Computer IT&lt;/option&gt;
	*        &lt;option value="Comics"&gt;Comics&lt;/option&gt;
	*        &lt;option value="Cookery"&gt;Cookery&lt;/option&gt;
	*        &lt;option value="Environment"&gt;Environment&lt;/option&gt;
	*        &lt;option value="Fiction"&gt;Fiction&lt;/option&gt;
	*        &lt;option value="Health"&gt;Health&lt;/option&gt;
	*        &lt;option value="Humanities"&gt;Humanities&lt;/option&gt;
	*        &lt;option value="Language"&gt;Language&lt;/option&gt;
	*    &lt;/select&gt;
	  * &lt;script&gt;
      * // Create DropDownList
    * $('#drpdwn').ejDropDownList();
    * $('#drpdwn').ejDropDownList("setSelectedValue","Art");  // setSelectedValue for the DropDownList text	
    *$('#drpdwn').ejDropDownList("unselectItemByIndex",0); // unselectItemByIndex for the DropDownList text      
   * &lt;/script&gt;
   *@memberof ejDropDownList
   * @instance
   */
        unselectItemByIndex: function (index) {
            if (!this.model.enabled) return false;
            if (this.selectedIndexValue == index) {
                this._activeItem = index;
                this._removeTextBoxValue();
            }
        },
        /**
* This method is used to select a list item in DropDownList using given value field.
* @example 
*   &lt;select name="selectIndex" id="drpdwn"&gt;
	*        &lt;option value="Art"&gt;Art&lt;/option&gt;
	*        &lt;option value="Architecture"&gt;Architecture&lt;/option&gt;
	*        &lt;option value="Biographies"&gt;Biographies&lt;/option&gt;
	*        &lt;option value="Business"&gt;Business&lt;/option&gt;
	*        &lt;option value="ComputerIT"&gt;Computer IT&lt;/option&gt;
	*        &lt;option value="Comics"&gt;Comics&lt;/option&gt;
	*        &lt;option value="Cookery"&gt;Cookery&lt;/option&gt;
	*        &lt;option value="Environment"&gt;Environment&lt;/option&gt;
	*        &lt;option value="Fiction"&gt;Fiction&lt;/option&gt;
	*        &lt;option value="Health"&gt;Health&lt;/option&gt;
	*        &lt;option value="Humanities"&gt;Humanities&lt;/option&gt;
	*        &lt;option value="Language"&gt;Language&lt;/option&gt;
	*    &lt;/select&gt;
	* &lt;script&gt;
* // Create DropDownList
 * $('#drpdwn').ejDropDownList();
* var DropDownListObj  = $("#drpdwn").data("ejDropDownList");
* DropDownListObj.setSelectedValue("ComputerIT"); // setSelectedValue for the DropDownList text
* &lt;/script&gt;
* @example 
*   &lt;select name="selectIndex" id="drpdwn"&gt;
	*        &lt;option value="Art"&gt;Art&lt;/option&gt;
	*        &lt;option value="Architecture"&gt;Architecture&lt;/option&gt;
	*        &lt;option value="Biographies"&gt;Biographies&lt;/option&gt;
	*        &lt;option value="Business"&gt;Business&lt;/option&gt;
	*        &lt;option value="ComputerIT"&gt;Computer IT&lt;/option&gt;
	*        &lt;option value="Comics"&gt;Comics&lt;/option&gt;
	*        &lt;option value="Cookery"&gt;Cookery&lt;/option&gt;
	*        &lt;option value="Environment"&gt;Environment&lt;/option&gt;
	*        &lt;option value="Fiction"&gt;Fiction&lt;/option&gt;
	*        &lt;option value="Health"&gt;Health&lt;/option&gt;
	*        &lt;option value="Humanities"&gt;Humanities&lt;/option&gt;
	*        &lt;option value="Language"&gt;Language&lt;/option&gt;
	*    &lt;/select&gt;
	* &lt;script&gt;
* // Create DropDownList
* $('#drpdwn').ejDropDownList();
* $('#drpdwn').ejDropDownList("setSelectedValue","ComputerIT"); 	
* &lt;/script&gt;
*@memberof ejDropDownList
* @instance
*/
        setSelectedValue: function (idvalue) {
            if (!this.model.enabled) return false;
            var i, hidelement;
            this.listitems = this.popupList.find("ol,ul").children("li");
            for (i = 0; i < this.listitems.length; i++) {
                if (this.listitems[i].getAttribute("value") != null) {
                    this._selectedValue = this.listitems[i].getAttribute("value");
                    if (this._selectedValue == idvalue) {
                        this._activeItem = i;
                        this._enterTextBoxValue();
                        return false;
                    }
                }
            }
        },
		
		        /**
* This method is used to unselect a list item in DropDownList using given value field.
* @example 
*   &lt;select name="selectIndex" id="drpdwn"&gt;
	*        &lt;option value="Art"&gt;Art&lt;/option&gt;
	*        &lt;option value="Architecture"&gt;Architecture&lt;/option&gt;
	*        &lt;option value="Biographies"&gt;Biographies&lt;/option&gt;
	*        &lt;option value="Business"&gt;Business&lt;/option&gt;
	*        &lt;option value="ComputerIT"&gt;Computer IT&lt;/option&gt;
	*        &lt;option value="Comics"&gt;Comics&lt;/option&gt;
	*        &lt;option value="Cookery"&gt;Cookery&lt;/option&gt;
	*        &lt;option value="Environment"&gt;Environment&lt;/option&gt;
	*        &lt;option value="Fiction"&gt;Fiction&lt;/option&gt;
	*        &lt;option value="Health"&gt;Health&lt;/option&gt;
	*        &lt;option value="Humanities"&gt;Humanities&lt;/option&gt;
	*        &lt;option value="Language"&gt;Language&lt;/option&gt;
	*    &lt;/select&gt;
	* &lt;script&gt;<br/> 
* // Create DropDownList
* $('#drpdwn').ejDropDownList();
* var DropDownListObj  = $("#drpdwn").data("ejDropDownList");
*DropDownListObj.setSelectedValue("ComputerIT"); // setSelectedValue for the DropDownList text
* DropDownListObj.unselectItemByValue("ComputerIT"); // unselectItemByValue for the DropDownList text
* &lt;/script&gt;
* @example 
*   &lt;select name="selectIndex" id="drpdwn"&gt;
	*        &lt;option value="Art"&gt;Art&lt;/option&gt;
	*        &lt;option value="Architecture"&gt;Architecture&lt;/option&gt;
	*        &lt;option value="Biographies"&gt;Biographies&lt;/option&gt;
	*        &lt;option value="Business"&gt;Business&lt;/option&gt;
	*        &lt;option value="ComputerIT"&gt;Computer IT&lt;/option&gt;
	*        &lt;option value="Comics"&gt;Comics&lt;/option&gt;
	*        &lt;option value="Cookery"&gt;Cookery&lt;/option&gt;
	*        &lt;option value="Environment"&gt;Environment&lt;/option&gt;
	*        &lt;option value="Fiction"&gt;Fiction&lt;/option&gt;
	*        &lt;option value="Health"&gt;Health&lt;/option&gt;
	*        &lt;option value="Humanities"&gt;Humanities&lt;/option&gt;
	*        &lt;option value="Language"&gt;Language&lt;/option&gt;
	*    &lt;/select&gt;
	* &lt;script&gt;<br/> 
* // Create DropDownList
* $('#drpdwn').ejDropDownList();
* $('#drpdwn').ejDropDownList("setSelectedValue","ComputerIT");  // setSelectedValue for the DropDownList text	
* $('#drpdwn').ejDropDownList("unselectItemByValue","ComputerIT"); 	
* &lt;/script&gt;
*@memberof ejDropDownList
* @instance
*/
		
        unselectItemByValue: function (idvalue) {
            if (!this.model.enabled) return false;
            var i, hidelement;
            this.listitems = this.popupList.find("ol,ul").children("li");
            for (i = 0; i < this.listitems.length; i++) {
                if (this.listitems[i].getAttribute("value") != null) {
                    this._selectedValue = this.listitems[i].getAttribute("value");
                    if (this._selectedValue == idvalue) {
                        this._activeItem = i;
                        this._removeTextBoxValue();
                        return false;
                    }
                }
            }
        },
        /**
* This method is used to select a list item in DropDownList using given text field.
* @example 
*   &lt;select name="selectIndex" id="drpdwn"&gt;
	*        &lt;option value="Art"&gt;Art&lt;/option&gt;
	*        &lt;option value="Architecture"&gt;Architecture&lt;/option&gt;
	*        &lt;option value="Biographies"&gt;Biographies&lt;/option&gt;
	*        &lt;option value="Business"&gt;Business&lt;/option&gt;
	*        &lt;option value="ComputerIT"&gt;Computer IT&lt;/option&gt;
	*        &lt;option value="Comics"&gt;Comics&lt;/option&gt;
	*        &lt;option value="Cookery"&gt;Cookery&lt;/option&gt;
	*        &lt;option value="Environment"&gt;Environment&lt;/option&gt;
	*        &lt;option value="Fiction"&gt;Fiction&lt;/option&gt;
	*        &lt;option value="Health"&gt;Health&lt;/option&gt;
	*        &lt;option value="Humanities"&gt;Humanities&lt;/option&gt;
	*        &lt;option value="Language"&gt;Language&lt;/option&gt;
	*    &lt;/select&gt;
	* &lt;script&gt;<br/>
* // Create DropDownList
* $('#drpdwn').ejDropDownList();
* var DropDownListObj  = $("#drpdwn").data("ejDropDownList");
* DropDownListObj.setSelectedText("Computer IT"); // setSelectedText for the DropDownList text
* &lt;/script&gt;
* @example 
*   &lt;select name="selectIndex" id="drpdwn"&gt;
	*        &lt;option value="Art"&gt;Art&lt;/option&gt;
	*        &lt;option value="Architecture"&gt;Architecture&lt;/option&gt;
	*        &lt;option value="Biographies"&gt;Biographies&lt;/option&gt;
	*        &lt;option value="Business"&gt;Business&lt;/option&gt;
	*        &lt;option value="ComputerIT"&gt;Computer IT&lt;/option&gt;
	*        &lt;option value="Comics"&gt;Comics&lt;/option&gt;
	*        &lt;option value="Cookery"&gt;Cookery&lt;/option&gt;
	*        &lt;option value="Environment"&gt;Environment&lt;/option&gt;
	*        &lt;option value="Fiction"&gt;Fiction&lt;/option&gt;
	*        &lt;option value="Health"&gt;Health&lt;/option&gt;
	*        &lt;option value="Humanities"&gt;Humanities&lt;/option&gt;
	*        &lt;option value="Language"&gt;Language&lt;/option&gt;
	*    &lt;/select&gt;
	* &lt;script&gt;<br/>
* // Create DropDownList
* $('#drpdwn').ejDropDownList();
* $('#drpdwn').ejDropDownList("setSelectedText","Computer IT"); 	
* &lt;/script&gt;
*@memberof ejDropDownList
* @instance
*/
        setSelectedText: function (idvalue) {
            if (!this.model.enabled) return false;
            var i, hidelement;
            this.listitems = this.popupList.find("ol,ul").children("li");
            for (i = 0; i < this.listitems.length; i++) {
                this.selectedTextValue = $(this.listitems[i]).text();
                if (this.selectedTextValue == idvalue) {
                    this._activeItem = i;
                    this._enterTextBoxValue();
                    return false;
                }
            }
        },

		   /**
* This method is used to unselect a list item in DropDownList using given text field.
* @example 
*   &lt;select name="selectIndex" id="drpdwn"&gt;
	*        &lt;option value="Art"&gt;Art&lt;/option&gt;
	*        &lt;option value="Architecture"&gt;Architecture&lt;/option&gt;
	*        &lt;option value="Biographies"&gt;Biographies&lt;/option&gt;
	*        &lt;option value="Business"&gt;Business&lt;/option&gt;
	*        &lt;option value="ComputerIT"&gt;Computer IT&lt;/option&gt;
	*        &lt;option value="Comics"&gt;Comics&lt;/option&gt;
	*        &lt;option value="Cookery"&gt;Cookery&lt;/option&gt;
	*        &lt;option value="Environment"&gt;Environment&lt;/option&gt;
	*        &lt;option value="Fiction"&gt;Fiction&lt;/option&gt;
	*        &lt;option value="Health"&gt;Health&lt;/option&gt;
	*        &lt;option value="Humanities"&gt;Humanities&lt;/option&gt;
	*        &lt;option value="Language"&gt;Language&lt;/option&gt;
	*    &lt;/select&gt;
	* &lt;script&gt;<br/>
* // Create DropDownList
* $('#drpdwn').ejDropDownList();
* var DropDownListObj  = $("#drpdwn").data("ejDropDownList");
*DropDownListObj.setSelectedValue("ComputerIT"); // setSelectedValue for the DropDownList text
* DropDownListObj.unselectItemByText("Computer IT"); // unselectItemByText for the DropDownList text
* &lt;/script&gt;
* @example 
*   &lt;select name="selectIndex" id="drpdwn"&gt;
	*        &lt;option value="Art"&gt;Art&lt;/option&gt;
	*        &lt;option value="Architecture"&gt;Architecture&lt;/option&gt;
	*        &lt;option value="Biographies"&gt;Biographies&lt;/option&gt;
	*        &lt;option value="Business"&gt;Business&lt;/option&gt;
	*        &lt;option value="ComputerIT"&gt;Computer IT&lt;/option&gt;
	*        &lt;option value="Comics"&gt;Comics&lt;/option&gt;
	*        &lt;option value="Cookery"&gt;Cookery&lt;/option&gt;
	*        &lt;option value="Environment"&gt;Environment&lt;/option&gt;
	*        &lt;option value="Fiction"&gt;Fiction&lt;/option&gt;
	*        &lt;option value="Health"&gt;Health&lt;/option&gt;
	*        &lt;option value="Humanities"&gt;Humanities&lt;/option&gt;
	*        &lt;option value="Language"&gt;Language&lt;/option&gt;
	*    &lt;/select&gt;
	* &lt;script&gt;<br/>
* // Create DropDownList
* $('#drpdwn').ejDropDownList();
* $('#drpdwn').ejDropDownList("setSelectedValue","ComputerIT");  // setSelectedValue for the DropDownList text	
* $('#drpdwn').ejDropDownList("unselectItemByText","Computer IT"); 	
* &lt;/script&gt;
*@memberof ejDropDownList
* @instance
*/
        unselectItemByText: function (idvalue) {
            if (!this.model.enabled) return false;
            var i, hidelement;
            this.listitems = this.popupList.find("ol,ul").children("li");
            for (i = 0; i < this.listitems.length; i++) {
                this.unselectedTextValue = $(this.listitems[i]).text();
                if (this.unselectedTextValue == idvalue) {
                    this._activeItem = i;
                    this._removeTextBoxValue();
                    return false;
                }
            }
        },

		   /**
* This method is used to get the selected items in DropDownList.
* @return string
* @example 
* &lt;input type="text" id="drpdwn" /&gt; <br> 
			*  &lt;div id="carsList"&gt;
			*    &lt;ul&gt;
			*       &lt;li&gt;Audi A4&lt;/li&gt;
			*       &lt;li&gt;Audi A5&lt;/li&gt;
			*       &lt;li&gt;Audi A6&lt;/li&gt;
			*       &lt;li&gt;Audi A7&lt;/li&gt;
			*       &lt;li&gt;Audi A8&lt;/li&gt;
			*    &lt;/ul&gt;
			*  &lt;/div&gt;<br/> 
	  * &lt;script&gt;
* // Create DropDownList
* $('#drpdwn').ejDropDownList({targetID: "carsList",value:"Audi A8"});
* var DropDownListObj  = $("#drpdwn").data("ejDropDownList");
* DropDownListObj.getSelectedItem(); 
* &lt;/script&gt;
* @example 
* &lt;input type="text" id="drpdwn" /&gt; <br> 
			*  &lt;div id="carsList"&gt;
			*    &lt;ul&gt;
			*       &lt;li&gt;Audi A4&lt;/li&gt;
			*       &lt;li&gt;Audi A5&lt;/li&gt;
			*       &lt;li&gt;Audi A6&lt;/li&gt;
			*       &lt;li&gt;Audi A7&lt;/li&gt;
			*       &lt;li&gt;Audi A8&lt;/li&gt;
			*    &lt;/ul&gt;
			*  &lt;/div&gt;<br/> 
	  * &lt;script&gt;
* // Create DropDownList
* $('#drpdwn').ejDropDownList({targetID: "carsList",value:"Audi A8"});
* $('#drpdwn').ejDropDownList("getSelectedItem"); 	
* &lt;/script&gt;
*@memberof ejDropDownList
* @instance
*/
        getSelectedItem: function () {
            return this.selectedTextValue;
        },
		
      
		 /**
* This method is used to get the selected items value in DropDownList.
* @return string
* @example 
*   &lt;select name="selectIndex" id="drpdwn"&gt;
	*        &lt;option value="Art"&gt;Art&lt;/option&gt;
	*        &lt;option value="Architecture"&gt;Architecture&lt;/option&gt;
	*        &lt;option value="Biographies"&gt;Biographies&lt;/option&gt;
	*        &lt;option value="Business"&gt;Business&lt;/option&gt;
	*        &lt;option value="ComputerIT"&gt;Computer IT&lt;/option&gt;
	*        &lt;option value="Comics"&gt;Comics&lt;/option&gt;
	*        &lt;option value="Cookery"&gt;Cookery&lt;/option&gt;
	*        &lt;option value="Environment"&gt;Environment&lt;/option&gt;
	*        &lt;option value="Fiction"&gt;Fiction&lt;/option&gt;
	*        &lt;option value="Health"&gt;Health&lt;/option&gt;
	*        &lt;option value="Humanities"&gt;Humanities&lt;/option&gt;
	*        &lt;option value="Language"&gt;Language&lt;/option&gt;
	*    &lt;/select&gt;
	  * &lt;script&gt;
* // Create DropDownList
* $('#drpdwn').ejDropDownList({value:"Computer IT"});
* var DropDownListObj  = $("#drpdwn").data("ejDropDownList");
* DropDownListObj.getSelectedValue(); 
* &lt;/script&gt;
* @example 
*   &lt;select name="selectIndex" id="drpdwn"&gt;
	*        &lt;option value="Art"&gt;Art&lt;/option&gt;
	*        &lt;option value="Architecture"&gt;Architecture&lt;/option&gt;
	*        &lt;option value="Biographies"&gt;Biographies&lt;/option&gt;
	*        &lt;option value="Business"&gt;Business&lt;/option&gt;
	*        &lt;option value="ComputerIT"&gt;Computer IT&lt;/option&gt;
	*        &lt;option value="Comics"&gt;Comics&lt;/option&gt;
	*        &lt;option value="Cookery"&gt;Cookery&lt;/option&gt;
	*        &lt;option value="Environment"&gt;Environment&lt;/option&gt;
	*        &lt;option value="Fiction"&gt;Fiction&lt;/option&gt;
	*        &lt;option value="Health"&gt;Health&lt;/option&gt;
	*        &lt;option value="Humanities"&gt;Humanities&lt;/option&gt;
	*        &lt;option value="Language"&gt;Language&lt;/option&gt;
	*    &lt;/select&gt; 
	  * &lt;script&gt;
* // Create DropDownList
* $('#drpdwn').ejDropDownList({value:"Computer IT"});
* $('#drpdwn').ejDropDownList("getSelectedValue");
* &lt;/script&gt;
*@memberof ejDropDownList
* @instance
*/
        getSelectedValue: function () {
            if (this.element[0].value != "")
                return this._selectedValue;
            else return "";
        },
        getSelectedItemsID: function () {
            return this._selectedItemsID;
        },
        _removeText: function (currentValue) {
            var res, index, removeHidden, hiddenIndex, checkSplit, checkCurrent;
			res = this.element[0].value;
			removeHidden = this._hiddenInput[0].value;
			checkSplit=res.indexOf(currentValue+",");
			checkCurrent=res.indexOf(currentValue);
			if(checkSplit >= 0){
				res = res.replace(currentValue+",", "");
				removeHidden = removeHidden.replace(this._hiddenValue+",", "");
			}
			else if(res.length == checkCurrent+currentValue.length && checkCurrent == 0){
				res = res.replace(currentValue, "");
				removeHidden = removeHidden.replace(this._hiddenValue, "");
			}
			else{
				res = res.replace(","+currentValue, "");
				removeHidden = removeHidden.replace(","+this._hiddenValue, "");
			}
			this.element[0].value = res;
			this._hiddenInput[0].value = removeHidden;
        },
        _addText: function (currentValue) {
            if ((this.model.watermarkText) && (this.element.hasClass("e-watermark"))) {
                this.element.val(this.element[0].value.replace(this.model.watermarkText, ""));
                this._hiddenInput.val("");
            }
            this._delimiterIndex = (this.element[0].value).lastIndexOf(",");
            this._hiddenDelimiterIndex = (this._hiddenInput[0].value).lastIndexOf(",");
            if (((this._delimiterIndex == -1) || (this._delimiterIndex != this.element[0].value.length - 1)) && (this.element[0].value != "")) {
                this.element.val(this.element[0].value + ",");
                this._hiddenInput.val(this._hiddenInput[0].value + ",");
                this._delimiterIndex = (this.element[0].value).lastIndexOf(",");
                this._hiddenDelimiterIndex = (this._hiddenInput[0].value).lastIndexOf(",");
            }
            if (!this._checkContains(currentValue)) {
                this.element.val(this.element[0].value.substr(0, this._delimiterIndex + 1) + currentValue);
                this._hiddenInput.val(this._hiddenInput[0].value.substr(0, this._hiddenDelimiterIndex + 1) + this._hiddenValue);
            }
        },
        _checkContains: function (Value) {
            var chkValue = Value;
            var values = this.element[0].value.split(",");
            this.contains = false;
            for (var i = 0; i < values.length; i++) {
                if (values[i] == chkValue)
                    this.contains = true;
            }
            return this.contains;
        },
        _init: function () {
            if ((this.element.is("input") && (this.element.is("input[type=text]") || !this.element.attr('type'))) || this.element.is("select")) {
				this._id=this.element[0].id;
                this._initialize();
                this._render();
                this._enabled(this.model.enabled);
                this._wireEvents();
                this._initValue = false;
            }
        },
        _setInitialPopup: function (value) {
            value == false ? this._hideResult() : this._showResult();
        },
        _changeSkin: function (skin) {
            if (this.model.cssClass != skin) {
                this.wrapper.removeClass(this.model.cssClass).addClass(skin);
                this.popupList.removeClass(this.model.cssClass).addClass(skin);
            }
        },

        _setRTL: function (boolean) {
            if (this.model.enableRTL != boolean) {
                this.model.enableRTL = boolean;
                this._RightToLeft();
                this._dropbtnRTL();
            }
        },
        //height width
        _changeHeight: function (height) {
            this.wrapper.height(height);
            this._setListHeight();
        },
        _changeWidth: function (width) {
            this.wrapper.width(width);
            this._setListWidth();
        },
        /**
       * To configure the properties at runtime using SetModel		
       * @private
       */
        _setModel: function (options) {
            if (!(this.model.enabled) && ej.isNullOrUndefined(options["enabled"])) return false;
            var option;
            for (option in options) {
                switch (option) {
                    case "dataSource": this._checkModelDataBinding(options[option]); break;
                    case "query": this._queryCheck(options[option]); break;
                    case "fields": this.model.fields = options[option]; this._checkModelDataBinding(this.model.dataSource); break;
                    case "template": this.model.template = options[option]; this._checkModelDataBinding(this.model.dataSource); break;
                    case "value": this._setValue(ej.util.getVal(options[option])); break;
                    case "text": this._setText(options[option]); break;
                    case "itemValue": this._setItemValue(options[option]); break;
                    case "enableRTL": this._setRTL(options[option]); break;
                    case "enabled": this._enabled(options[option]); break;
                    case "height": this._changeHeight(options[option]); break;
                    case "width": this._changeWidth(options[option]); break;
                    case "popupHeight": this.model.popupHeight = options[option]; this._setListHeight(options[option]); break;
                    case "popupWidth": this.model.popupWidth = options[option]; this._setListWidth(); break;
                    case "cssClass": this._changeSkin(options[option]); break;
                    case "showCheckbox": this._checkboxHideShow(options[option]); break;
                    case "checkAll": this._setCheckAll(options[option]); break;
                    case "uncheckAll": this._setUncheckAll(options[option]); break;
                    case "watermarkText": this._changeWatermark(options[option]); break;
                    case "showRoundedCorner": this._roundedCorner(options[option]); this.model.showRoundedCorner = options[option]; break;
                    case "showPopupOnLoad": this._setInitialPopup(options[option]); break;
                    case "targetID": this.model.targetID = options[option]; this._showFullList(); break;
                    case "selectedItemIndex": this._selectItemByIndex(options[option]); break;
                    case "selectedItems": this._selectCheckedItem(options[option]); this.model.selectedItems = options[option]; break;
                    case "allowMultiSelection": this.model.allowMultiSelection = options[option];
                        options[option] ? this._renderBoxModel() : this._destroyBoxModel();
                        break;
					case "enableAnimation":{
							this.model.enableAnimation=options[key];
							break;
					}
                }
            }
        },
        /**
 * destroy in the DropDownList textbox.
 * @alias destroy
 * @return jQuery
 * @example 
 * &lt;input type="text" id="drpdwn" /&gt; <br> 
			*  &lt;div id="carsList"&gt;
			*    &lt;ul&gt;
			*       &lt;li&gt;Audi A4&lt;/li&gt;
			*       &lt;li&gt;Audi A5&lt;/li&gt;
			*       &lt;li&gt;Audi A6&lt;/li&gt;
			*       &lt;li&gt;Audi A7&lt;/li&gt;
			*       &lt;li&gt;Audi A8&lt;/li&gt;
			*    &lt;/ul&gt;
			*  &lt;/div&gt;<br/> 
	  * &lt;script&gt;
 * // Create DropDownList
 * $('#drpdwn').ejDropDownList({targetID: "carsList",value:"Audi A5"});
 * var DropDownListObj  = $("#drpdwn").data("ejDropDownList");
 * DropDownListObj.destroy(); // hide the DropDownList text
 * &lt;/script&gt;
 * @example 
* &lt;input type="text" id="drpdwn" /&gt; <br> 
			*  &lt;div id="carsList"&gt;
			*    &lt;ul&gt;
			*       &lt;li&gt;Audi A4&lt;/li&gt;
			*       &lt;li&gt;Audi A5&lt;/li&gt;
			*       &lt;li&gt;Audi A6&lt;/li&gt;
			*       &lt;li&gt;Audi A7&lt;/li&gt;
			*       &lt;li&gt;Audi A8&lt;/li&gt;
			*    &lt;/ul&gt;
			*  &lt;/div&gt;<br/> 
	  * &lt;script&gt;
 * // Create DropDownList
 * $('#drpdwn').ejDropDownList({targetID: "carsList",value:"Audi A5"});
 * $('#drpdwn').ejDropDownList("destroy"); 	
 * &lt;/script&gt;
 *@memberof ejDropDownList
 * @instance
 */
        _destroy: function () {
            this._destroyDropdown();
            this.element.removeClass("e-input " + this.model.cssClass + " e-watermark");
            this.element.attr('accesskey', this.wrapper.attr('accesskey'));
            this.element[0].value = "";
            if (this.model.dataSource != null)
                this.popupPanelWrapper.remove();
            else {
                this.docbdy.append(this.ultag.html());
                this.popupPanelWrapper.remove();
            }
        },
        /* cookie */

        _finalize: function () {
            if (this.model.selectedItemIndex != null) {
                this._selectItemByIndex(this.model.selectedItemIndex);
            } else if ((this.model.showCheckbox == true) && (this.model.selectedItems.length > 0)) {
                this._selectCheckedItem(this.model.selectedItems);
            }
        },

        /**/
        _initialize: function () {
            if (this.element.is("select")) {
                this.selectelement = true;
                this._renderSelectToDropdown();
            }
            this._selectedItemsID = [];
            this.target = this.element[0];
            this.showPopupBox = false;
            this._queryString = null;
            this.suggLen = 0;
            this._itemId = null;
            this.checkedStatus = false;
            this._incqueryString = "";
            this._activeItem = null;
            this.ddWidth = 0;
            this._initValue = true;
        },
        _renderSelectToDropdown: function () {
            var i, optionLength;
            this.inputElement = ej.buildTag("input.e-dropdownlist#" + this._id + "_input", "", {}, { "type": "text" });
            this.inputElement.insertAfter(this.element);
            this.optionDiv = ej.buildTag("div#" + this._id + "_list");
            this.optionDiv.insertAfter(this.inputElement);
            this.optionUl = ej.buildTag("ul");
            this.optionDiv.append(this.optionUl);
            this.selectOptions = this.element;
            this.selectOptions.attr('id', this._id);
            this.selectOptionItems = this.element.children("option");
            optionLength = this.selectOptionItems.length;
            this.optionDummyUl = $();

            for (i = 0; i < optionLength; i++) {
                if ((this.selectOptionItems[i].text != null) && ((this.selectOptionItems[i].text != ""))) {
                    this.optionLi = ej.buildTag("li", this.selectOptionItems[i].text, {}, { value: this.selectOptionItems[i].value });
                    this.optionDummyUl.push(this.optionLi[0]);
                }
            }
            this.optionUl.append(this.optionDummyUl);

            this.element.css('display', 'none');
            this.element = this.inputElement;
        },


        _render: function () {
            this._renderDropdown();
            this._setWatermark();
            this._renderPopupPanelWrapper();
            this._initDataSource(this.model.dataSource);
            this._showFullList();
            this._finalize();
        },

        _queryCheck: function (value) {
            this.model.query = value;
            this._checkModelDataBinding(this.model.dataSource);
        },

        _checkModelDataBinding: function (source) {
            this.model.dataSource = source;
            if (source != null) {
                if (ej.DataManager && source instanceof ej.DataManager) {
                    this._initDataSource(source);
                } else {
                    this._showFullList();
                }
            }
            else {
                this.ultag.empty();
            }
        },
        _initDataSource: function (source) {
            var proxy = this;
            if (ej.DataManager && source instanceof ej.DataManager) {
                proxy.popupListItems = proxy.model.dataSource;
                proxy._addLoadingClass();
                var queryPromise = source.executeQuery(this._getQuery());
                queryPromise.done(function (e) {
                    proxy.popupListItems = e.result;
                    proxy._removeLoadingClass();
                    proxy._showFullList();

                }).fail(function (e) {
                    proxy.model.dataSource = null;
                    proxy._addLoadingClass();
                });
            }
        },
        _getQuery: function () {
            if (ej.isNullOrUndefined(this.model.query)) {
                var column = [], queryManager = ej.Query(), mapper = this.model.fields;
                for (var col in mapper) {
                    if (col !== "tableName" && mapper[col])
                        column.push(mapper[col]);
                }
                if (column.length > 0)
                    queryManager.select(column);
                if (!this.model.dataSource.dataSource.url.match(mapper.tableName + "$"))
                    !ej.isNullOrUndefined(mapper.tableName) && queryManager.from(mapper.tableName);
            }
            else queryManager = this.model.query;
            return queryManager;
        },

        _addLoadingClass: function () {
            this.dropdownbutton.addClass("e-load");
            this.drpbtnspan.removeClass("e-icon e-down-arrow");
            this.model.readOnly = true;
        },
        _removeLoadingClass: function () {
            this.dropdownbutton.removeClass("e-load");
            this.drpbtnspan.addClass("e-icon e-down-arrow");
            this.model.readOnly = false;
        },
        //render drop button text box
        _renderDropdown: function () {
            this.element.addClass("e-input " + this.model.cssClass);
            if (this.value())
                this.element.val(this.value());
            this.wrapper = ej.buildTag("span.e-ddl e-widget " + this.model.cssClass + "#" + this._id + "_wrapper", "", {}, { "tabindex": "0", "accesskey": this.element.attr("accesskey") });//
            this.container = ej.buildTag("span.e-in-wrap e-box " + this.model.cssClass + "#" + this._id + "_container");
            this.element.removeAttr('accesskey');
            this.element.attr({ "readonly": true, "tabindex": -1, "role": "combobox", "aria-expanded": false, "aria-autocomplete": "list", "aria-haspopup": true, "aria-owns": this._id + "_popup", "value": this.value() });
            this.drpbtnspan = ej.buildTag("span.e-icon e-down-arrow", "", {}, { "aria-label": "select" });
            this.dropdownbutton = ej.buildTag("span.e-select#" + this._id + "_dropdown", "", {}, { "role": "button" }).append(this.drpbtnspan);
            this.container.insertAfter(this.element);
            this.container.append(this.element);
            this.container.append(this.dropdownbutton);
            this.wrapper.insertBefore(this.container);
            this.wrapper.append(this.container);
            if (this.selectelement) {
                this.selectOptions.insertBefore(this.element);
            }
            this._hiddenInput = ej.buildTag("input#" + this._id + "_hidden", "", {}, { type: "hidden" }).insertBefore(this.element);
            this._hiddenInput.attr("value", this.value());
            this._checkNameAttr();
            this._setDimentions();
            this._RightToLeft();
            this.ddWidth = (this.dropdownbutton.outerWidth() > 0) ? this.dropdownbutton.outerWidth() : 24;
            this.element.bind("mousedown", $.proxy(this._OnDropdownClick, this));
            this.dropdownbutton.bind("mousedown", $.proxy(this._OnDropdownClick, this));
            this._roundedCorner(this.model.showRoundedCorner);
            this._renderBoxModel();
        },
        /*name attribute for input box*/
        _checkNameAttr: function () {
			if (this.element.attr("name"))
                this._hiddenInput.attr("name", this.element.attr("name"));
            else {
                this._hiddenInput.attr("name", this._id);
				this.element.attr("name", this._id);
			}
        },
        _renderBoxModel: function () {
            if (!this.model.allowMultiSelection) return false;
            this.element.css('display', 'none');
            this._ulBox = ej.buildTag("ul.e-ul e-boxes");
            this.container.prepend(this._ulBox);
            this._ulBox.css('min-height', '30px');
            this.wrapper.css({ 'height': 'auto' });

            this._on(this.container, "mousedown", function (e) {
                if (!this.model.enabled) return false;
                var $target = $(e.target);
                if ($target.hasClass("e-options")) {
                    if (!e.ctrlKey && $target.siblings().hasClass("e-active")) this._removeActive();
                    if ($target.hasClass("e-active")) $target.removeClass("e-active");
                    else $target.addClass("e-active");
                }
                if (!e.ctrlKey && ($target.hasClass("e-boxes"))) this._removeActive();
            });
        },
        //render popup pannel wrapper
        _renderPopupPanelWrapper: function () {
			var oldWrapper = $("#" + this.element.context.id + "_popup_wrapper").get(0);
            if (oldWrapper)
                $(oldWrapper).remove();
            this.popupPanelWrapper = ej.buildTag("div#" + this._id + "_popup_wrapper");
            $('body').append(this.popupPanelWrapper);
            //render popup list 
            this.popupList = ej.buildTag("div.e-ddl-popup e-box e-popup e-widget " + this.model.cssClass + "#" + this._id + "_popup", "", { "visibility": "hidden" }, { "tabIndex": 0 });
            this.popup = this.popupList;
            this.popupScroller = ej.buildTag("div"); this.ultag = ej.buildTag("ul.e-ul", "", {}, { "role": "listbox" });
            this.popupScroller.append(this.ultag);
            this.popupList.append(this.popupScroller);
            this.popupPanelWrapper.append(this.popupList);
        },
        //render popup list items
        _renderPopupList: function () {
            var list = this.popupListItems, i, ulempty, ulno, litag, _id, _txt, mapper = this.model.fields, predecessor;
            this.ultag.empty();
            this.dummyUl = $();
            if (this.model.enableRTL)
                this.popupList.addClass("e-rtl");
            if (this.model.dataSource == null || this.model.dataSource.length < 1) {
                predecessor = this.element.parents().last();
                if (this.model.targetID)
                    this.docbdy = predecessor.find("#" + this.model.targetID);
                else if (this.optionDiv)
                    this.docbdy = this.optionDiv;
                else return false;
                this.itemsContainer = this.docbdy.children("ol,ul");
                this.itemsContainer.children("ol,ul").remove();
                this.items = this.itemsContainer.children('li');
                this.items.children("img").addClass("e-align");
                this.items.children("div").addClass("e-align");
                this.itemsContainer.children("span").addClass("e-ghead");
                //This will append the list with the popup wrapper
                this.ultag.append(this.itemsContainer.children());
                this.itemsContainer.empty();
                this.docbdy.css({ 'display': 'none' }); this.itemsContainer.css({ 'display': 'none' });
            }
            else if (this.model.dataSource != null && typeof list[0] != "object") {
                if (list.length > 0) {
                    for (i = 0; i < list.length; i++) {
                        litag = ej.buildTag("li", list[i]);
                        this.dummyUl.push(litag[0]);
                    }
                    this.ultag.append(this.dummyUl);
                }
            }
            else {
                this.mapFld = { _id: null, _imageUrl: null, _imageAttributes: null, _spriteCSS: null, _text: null, _value: null, _htmlAttributes: null, _selected: null };
                this.mapFld._id = (mapper && mapper.id) ? mapper["id"] : "id";
                this.mapFld._imageUrl = (mapper && mapper.imageUrl) ? mapper["imageUrl"] : "imageUrl";
                this.mapFld._imageAttributes = (mapper && mapper.imageAttributes) ? mapper["imageAttributes"] : "imageAttributes";
                this.mapFld._spriteCSS = (mapper && mapper.spriteCssClass) ? mapper["spriteCssClass"] : "spriteCssClass";
                this.mapFld._text = (mapper && mapper.text) ? mapper["text"] : "text";
                this.mapFld._value = (mapper && mapper.value) ? mapper["value"] : "value";
                this.mapFld._htmlAttributes = (mapper && mapper.htmlAttributes) ? mapper["htmlAttributes"] : "htmlAttributes";
                this.mapFld._selected = (mapper && mapper.selected) ? mapper["selected"] : "selected";

                if (list.length > 0) {
                    if (this.model.template != null) {
                        for (i = 0; i < list.length; i++) {
                            litag = ej.buildTag("li", list[i]);
                            if (this.model.template) litag.append(this._getTemplatedString(list[i]));
                            this.dummyUl.push(litag[0]);
                        }
                        this.ultag.append(this.dummyUl);
                    } else {
                        for (i = 0; i < list.length; i++) {
                            var _did = this._getField(list[i], this.mapFld._id);
                            var _dimageUrl = this._getField(list[i], this.mapFld._imageUrl);
                            var _dimageAttributes = this._getField(list[i], this.mapFld._imageAttributes);
                            var _dspriteCss = this._getField(list[i], this.mapFld._spriteCSS);
                            var _dtext = this._getField(list[i], this.mapFld._text);
                            var _dvalue = this._getField(list[i], this.mapFld._value);
                            var _dhtmlAttributes = this._getField(list[i], this.mapFld._htmlAttributes);
                            var _dselected = this._getField(list[i], this.mapFld._selected);
                            if ((_dvalue) && (_dvalue != ""))
                                litag = ej.buildTag('li', "", {}, { value: _dvalue });
                            else
                                litag = ej.buildTag('li');
                            if (_did)
                                litag.attr('id', _did);
                            if ((_dimageUrl) && (_dimageUrl != "")) {
                                //Creating the image tag
                                imgtag = ej.buildTag('img.e-align', '', {}, { 'src': _dimageUrl, 'alt': _dtext });
                                if ((_dimageAttributes) && (_dimageAttributes != "")) 
                                    imgtag.attr(_dimageAttributes);
                                litag.append(imgtag);
                            }
                            if ((_dspriteCss) && (_dspriteCss != "")) {
                                //Creating the Sprite image tag
                                divtag = ej.buildTag('div.e-align ' + _dspriteCss + ' sprite-image');
                                litag.append(divtag);
                            }
                            if ((_dtext) && (_dtext != "")) {
                                //Creating text Content inside the  tag
                                litag.append(_dtext);
                            }
                            if ((_dhtmlAttributes) && (_dhtmlAttributes != ""))
                                litag.attr(_dhtmlAttributes);
                            if (_dselected) {
                                litag.addClass("chkselect");
                            }
                            //
                            this.dummyUl.push(litag[0]);
                        }
                    }
                    //ko binding
                    this.ultag.append(this.dummyUl);
                }

            }
            this.ultag.children('li').attr({ "role": "option" });
            this._setListWidth();
            this._setListHeight();
            this._setListPosition();
            this.popupList.css("height", ""); this.popupScroller.css({ "height": "", "width": "" });
            this.popupList.ejScroller({ height: this.popupList.height(), width: 0, scrollerSize: 20 });
            this.scrollerObj = this.popupList.ejScroller("instance");
            this.popupList.css({ 'display': 'none', 'visibility': 'visible' });
            this._checkboxHideShow(this.model.showCheckbox);
            this._setCheckAll(this.model.checkAll);
            this._setUncheckAll(this.model.uncheckAll);
            this.model.showPopupOnLoad && this._showResult();
        },
        //get object
        _getField: function (obj, fieldName) {
            return ej.pvt.getObject(fieldName, obj);
        },
        //template
        _getTemplatedString: function (list) {

            var str = this.model.template, start = str.indexOf("${"), end = str.indexOf("}");
            while (start != -1 && end != -1) {
                var content = str.substring(start, end + 1);
                var field = content.replace("${", "").replace("}", "");
                str = str.replace(content, this._getField(list, field));
                start = str.indexOf("${"), end = str.indexOf("}");
            }
            return str;
        },
        //water mark
        _setWatermark: function () {
            if ((this.model.watermarkText != null) && (this.element.val() == "")) {
                this.element.addClass("e-watermark");
                this.element.val(this.model.watermarkText);
            }
        },
        //checkbox properties
        _checkboxHideShow: function (value) {
            if (value) {
                this._createCheckbox();
            } else {
                this._removeCheckbox();
            }
            this.model.showCheckbox = value;
        },
        _createCheckbox: function () {
            var i, _extchk, chklist, me = this;
            this.listitems = this.popupList.find("ol,ul").children("li");
            chklist = this.listitems.find('input[type=checkbox]');
            if (chklist.length == 0) {
                for (i = 0; i < this.listitems.length; i++) {
                    $checkbox = ej.buildTag("input.listcheckbox e-align#popuplist" + i + "_" + this._id, "", {}, { type: "checkbox", name: "list" + i });
                    $(this.listitems[i]).prepend($checkbox);
                }
                this.popupList.find(".listcheckbox").ejCheckBox({ cssClass: this.model.cssClass, change: $.proxy(this._OnClickCheckList, this) });
                this.element.val("");
                this._setWatermark();
                for (i = 0; i < this.listitems.length; i++) {
                    if ($(this.listitems[i]).hasClass('chkselect')) {
                        $(this.listitems[i]).find(".listcheckbox").ejCheckBox({ "checked": true });
                        this._activeItem = i;
                        this.checkedStatus = true;
                        if (!this._initValue)
                            this._trigger('checkChange', { isChecked: this.checkedStatus, data: this.model });
                        this._enterTextBoxValue();
                        $(this.listitems[i]).removeClass('chkselect');
                    }
                }
            }
        },

        _removeCheckbox: function () {
            var i, checkbox;
            this.listitems = this.popupList.find("ol,ul").children("li");
            checkbox = this.listitems.find('.listcheckbox');
            if (checkbox.length > 0) {
                this.listitems.find('.listcheckbox').ejCheckBox('destroy');
                this.listitems.find('input[type=checkbox]').remove();
            }
        },
        _setCheckAll: function (value) {
            //Check all the nodes on CheckAll = true
            if ((this.model.showCheckbox) && (value))
                this.checkAll();
        },
        _setUncheckAll: function (value) {
            //UnCheck all the nodes on UnCheckAll = true
            if ((this.model.showCheckbox) && (value))
                this.unCheckAll();
        },
        /**
      * This method is used to set all the  items to checked.
      * @return jQuery
      * @example 
      * &lt;input type="text" id="drpdwn" /&gt; <br> 
			*  &lt;div id="carsList"&gt;
			*    &lt;ul&gt;
			*       &lt;li&gt;Audi A4&lt;/li&gt;
			*       &lt;li&gt;Audi A5&lt;/li&gt;
			*       &lt;li&gt;Audi A6&lt;/li&gt;
			*       &lt;li&gt;Audi A7&lt;/li&gt;
			*       &lt;li&gt;Audi A8&lt;/li&gt;
			*    &lt;/ul&gt;
			*  &lt;/div&gt;<br/> 
	  * &lt;script&gt;
      * // Create DropDownList
	  * $('#drpdwn').ejDropDownList({targetID: "carsList",value:"Audi A5",showCheckbox:true});
      * var DropDownListObj  = $("#drpdwn").data("ejDropDownList");
      * DropDownListObj.checkAll(); // checkAll values the DropDownList
      * &lt;/script&gt;
      * @example 
      * &lt;input type="text" id="drpdwn" /&gt; <br> 
			*  &lt;div id="carsList"&gt;
			*    &lt;ul&gt;
			*       &lt;li&gt;Audi A4&lt;/li&gt;
			*       &lt;li&gt;Audi A5&lt;/li&gt;
			*       &lt;li&gt;Audi A6&lt;/li&gt;
			*       &lt;li&gt;Audi A7&lt;/li&gt;
			*       &lt;li&gt;Audi A8&lt;/li&gt;
			*    &lt;/ul&gt;
			*  &lt;/div&gt;<br/> 
	  * &lt;script&gt;
      * // Create DropDownList
	  * $('#drpdwn').ejDropDownList({targetID: "carsList",value:"Audi A5",showCheckbox:true});
      * $('#drpdwn').ejDropDownList("checkAll"); 	
      * &lt;/script&gt;
      *@memberof ejDropDownList
      * @instance
      */
        checkAll: function () {
            if (!this.model.enabled) return false;
            var _nodes = this.model.selectedItems;
            for (i = 0; i < this.listitems.length; i++) {
                if ($(this.listitems[i].firstChild).find('.listcheckbox').ejCheckBox('isChecked') == false) {
                    $(this.listitems[i].firstChild).find('.listcheckbox').ejCheckBox('option', 'checked', true);
                    if ((this.model.watermarkText) && (this.element.hasClass("e-watermark"))) {
                        this.element.val(this.element[0].value.replace(this.model.watermarkText, ""));
                        this._hiddenInput.val("");
                    }
                    this.currentValue = $.trim($(this.listitems[i]).text());
                    this._itemID = $(this.listitems[i]).attr("id");
                    if (!ej.isNullOrUndefined(this._itemID) && this._itemID != "")
                        this._selectedItemsID.push(this._itemID);
                    if (this.listitems[i].getAttribute("value")) {
                        this._hiddenValue = this.listitems[i].getAttribute("value");
                    }
                    else {
                        this._hiddenValue = this.currentValue;
                    }
                    this._delimiterIndex = (this.element[0].value).lastIndexOf(",");
                    this._hiddenDelimiterIndex = (this._hiddenInput[0].value).lastIndexOf(",");
                    if ((this._delimiterIndex >= -1) && (this.element[0].value != "") && (this._delimiterIndex != this.element[0].value.length-1)) {
                        this.element.val(this.element[0].value + ",");
                        this._hiddenInput.val(this._hiddenInput[0].value + ",");
                        this._delimiterIndex = (this.element[0].value).lastIndexOf(",");
                        this._hiddenDelimiterIndex = (this._hiddenInput[0].value).lastIndexOf(",");
						this.element.val(this.element[0].value.substr(0, this._delimiterIndex + 1) + this.currentValue);
                    	this._hiddenInput.val(this._hiddenInput[0].value.substr(0, this._hiddenDelimiterIndex + 1) + this._hiddenValue);
                    }
                    else{
                    	this.element.val(this.element[0].value.substr(0, this._delimiterIndex + 1) + this.currentValue + ",");
                    	this._hiddenInput.val(this._hiddenInput[0].value.substr(0, this._hiddenDelimiterIndex + 1) + this._hiddenValue + ",");
					}
                    //
                    var _nodes = this.model.selectedItems;
                    if ($.inArray(i, _nodes) == -1) {
                        this.model.selectedItems.push(i);
                    }
                    //
                    this.checkedStatus = true;
                    if (this.listitems[i].getAttribute("value") != null)
                        this._selectedValue = this.listitems[i].getAttribute("value");
                    else
                        this._selectedValue = "";
                    if (!this._initValue)
                        this._trigger('checkChange', { isChecked: this.checkedStatus, data: this.model });
                    args = { text: this.element[0].value, selectedText: this.currentValue, itemId: i, value: this._selectedValue, isChecked: this.checkedStatus };
                    if (!this._initValue)
                        this._trigger("change", args);
                    this._cascadeAction();
                }
            }
            this.value(this.element.val());
            this.model.text = this.currentValue;
            this.model.itemValue = this._selectedValue;
            /*Watermark*/
            if (this.model.watermarkText != null) {
                if (($.trim(this.element.val()) === '') || ($.trim(this.element.val()) === this.model.watermarkText)) {
                    this.element.val(this.model.watermarkText);
                    this.element.addClass("e-watermark");
                } else {
                    this.element.removeClass("e-watermark");
                }
            }
        },
        _selectCheckedItem: function (chkitems) {
            if (chkitems.length > 0) {
                for (i = 0; i < chkitems.length; i++) {
                    if ($(this.listitems[chkitems[i]].firstChild).find('.listcheckbox').ejCheckBox('isChecked') == false) {
                        $(this.listitems[chkitems[i]].firstChild).find('.listcheckbox').ejCheckBox('option', 'checked', true);
                        this.checkedStatus = true;
                        this.currentValue = $.trim($(this.listitems[chkitems[i]]).text());
                        if (this.model.allowMultiSelection)
                            this._ulBox.append(this._createBox(this.currentValue));
                        if (this.listitems[chkitems[i]].getAttribute("value")) {
                            this._hiddenValue = this.listitems[chkitems[i]].getAttribute("value");
                        }
                        else {
                            this._hiddenValue = this.currentValue;
                        }
                        this._addText(this.currentValue);
                        if (this.listitems[chkitems[i]].getAttribute("value") != null)
                            this._selectedValue = this.listitems[chkitems[i]].getAttribute("value");
                        else
                            this._selectedValue = "";
                        if (!this._initValue)
                            this._trigger('checkChange', { isChecked: this.checkedStatus, data: this.model });
                        args = { text: this.element[0].value, selectedText: this.currentValue, itemId: this.selectedIndexValue, value: this._selectedValue, isChecked: this.checkedStatus };
                        if (!this._initValue)
                            this._trigger("change", args);
                        this._cascadeAction();
                    }
                }
                /*Watermark*/
                if (this.model.watermarkText != null) {
                    if (($.trim(this.element.val()) === '') || ($.trim(this.element.val()) === this.model.watermarkText)) {
                        this.element.val(this.model.watermarkText);
                        this.element.addClass("e-watermark");
                    } else {
                        this.element.removeClass("e-watermark");
                    }
                }
            }
        },
        /**
      * This method is used to set all the  items to uncheck.
      * @return jQuery
      * @example 
      * &lt;input type="text" id="drpdwn" /&gt; <br> 
			*  &lt;div id="carsList"&gt;
			*    &lt;ul&gt;
			*       &lt;li&gt;Audi A4&lt;/li&gt;
			*       &lt;li&gt;Audi A5&lt;/li&gt;
			*       &lt;li&gt;Audi A6&lt;/li&gt;
			*       &lt;li&gt;Audi A7&lt;/li&gt;
			*       &lt;li&gt;Audi A8&lt;/li&gt;
			*    &lt;/ul&gt;
			*  &lt;/div&gt;<br/> 
	  * &lt;script&gt;
      * // Create DropDownList
	  * $('#drpdwn').ejDropDownList({targetID: "carsList",value:"Audi A5",showCheckbox:true});
      * var DropDownListObj  = $("#drpdwn").data("ejDropDownList");
      * DropDownListObj.unCheckAll(); // UncheckAll values the DropDownList
      * &lt;/script&gt;
      * @example 
     * &lt;input type="text" id="drpdwn" /&gt; <br> 
			*  &lt;div id="carsList"&gt;
			*    &lt;ul&gt;
			*       &lt;li&gt;Audi A4&lt;/li&gt;
			*       &lt;li&gt;Audi A5&lt;/li&gt;
			*       &lt;li&gt;Audi A6&lt;/li&gt;
			*       &lt;li&gt;Audi A7&lt;/li&gt;
			*       &lt;li&gt;Audi A8&lt;/li&gt;
			*    &lt;/ul&gt;
			*  &lt;/div&gt;<br/> 
	  * &lt;script&gt;
      * // Create DropDownList
	  * $('#drpdwn').ejDropDownList({targetID: "carsList",value:"Audi A5",showCheckbox:true});
      * $('#drpdwn').ejDropDownList("unCheckAll"); 	
      * &lt;/script&gt;
      *@memberof ejDropDownList
      * @instance
      */
        unCheckAll: function () {
            if (!this.model.enabled) return false;
            for (i = 0; i < this.listitems.length; i++) {
                if ($(this.listitems[i].firstChild).find('.listcheckbox').ejCheckBox('isChecked')) {
					$(this.listitems[i]).removeClass('e-active');
                    $(this.listitems[i].firstChild).find('.listcheckbox').ejCheckBox('option', 'checked', false);
                    this.currentValue = $.trim($(this.listitems[i]).text());
                    if (this.listitems[i].getAttribute("value")) {
                        this._hiddenValue = this.listitems[i].getAttribute("value");
                    }
                    else {
                        this._hiddenValue = this.currentValue;
                    }
                    this._txtpos = (this.element[0].value).lastIndexOf(this.currentValue);
                    this._delimiterIndex = (this.element[0].value).lastIndexOf(",");
                    if ((this._txtpos == 0) && (this._delimiterIndex <= -1)) {
                        this.element.val(this.element[0].value.replace(this.currentValue, ""));
                        this._hiddenInput.val(this._hiddenInput[0].value.replace(this._hiddenValue, ""));
                    } else if (((this._txtpos == 0) && (this._delimiterIndex > -1)) || ((this._txtpos > -1) && (this._delimiterIndex > this._txtpos))) {
                        if(this.element[0].value == this.currentValue || this.element[0].value.length == this._txtpos+this.currentValue.length){
							this.element.val(this.element[0].value.replace(this.currentValue, ""));
                        	this._hiddenInput.val(this._hiddenInput[0].value.replace(this._hiddenValue, ""));
						}
						else{
                       		this.element.val(this.element[0].value.replace(this.currentValue + ",", ""));
                        	this._hiddenInput.val(this._hiddenInput[0].value.replace(this._hiddenValue + ",", ""));
						}
                    } else if ((this._txtpos > -1) && (this._delimiterIndex < this._txtpos)) {
                        this.element.val(this.element[0].value.replace("," + this.currentValue, ""));
                        this._hiddenInput.val(this._hiddenInput[0].value.replace("," + this._hiddenValue, ""));
                    } else if ((this._txtpos > -1) && (this._delimiterIndex < this._txtpos)) {
                        this.element.val(this.element[0].value.replace(this.currentValue, ""));
                        this._hiddenInput.val(this._hiddenInput[0].value.replace(this._hiddenValue, ""));
                    }
                }
                this.checkedStatus = false;
                var _nodes = this.model.selectedItems;
                if ($.inArray(i, _nodes) > -1) {
                    this.model.selectedItems.splice($.inArray(i, _nodes), 1);
                }
                if (this.listitems[i].getAttribute("value") != null)
                    this._selectedValue = this.listitems[i].getAttribute("value");
                else
                    this._selectedValue = "";
                this._itemID = $(this.listitems[i]).attr("id");
                if (!ej.isNullOrUndefined(this._itemID) && this._itemID != "")
                    this._removeSelectedItemsID();
                if (!this._initValue)
                    this._trigger('checkChange', { isChecked: this.checkedStatus, data: this.model });
                args = { text: this.element[0].value, selectedText: this.currentValue, itemId: i, value: this._selectedValue, isChecked: this.checkedStatus };
                if (!this._initValue)
                    this._trigger("change", args);
                this._cascadeAction();
            }
            this.value(this.element.val());
            this.model.text = this.currentValue;
            this.model.itemValue = this._selectedValue;
            /*watermarkText*/
            if (this.model.watermarkText != null) {
                if (($.trim(this.element.val()) === '') || ($.trim(this.element.val()) === this.model.watermarkText)) {
                    this.element.val(this.model.watermarkText);
                    this.element.addClass("e-watermark");
                } else {
                    this.element.removeClass("e-watermark");
                }
            }
        },
        _removeSelectedItemsID: function () {
            var itemToRemove;
            itemToRemove = this._selectedItemsID.indexOf(this._itemID);
            this._selectedItemsID.splice(itemToRemove, 1);
            this._itemID = "";
        },
        //List Properties
        _refreshScroller: function () {
            this.popupList.css("height", "auto");
            this.popupList.find(".e-content, .e-vhandle").removeAttr("style");
            this.popupList.find(".e-vhandle div").removeAttr("style");

            this.popupList.css({ "display": "block" });  // For get the height of the popup
            this.scrollerObj.model.height = this.popupList.outerHeight();
            this.scrollerObj.refresh();
            this.scrollerObj.option("scrollTop", 0);
            this.popupList.css({ "height": "auto" });
        },
        _setListWidth: function () {
            var width = this.model.popupWidth;
            if (width != "auto") this.popupList.css({ "width": width });
            else this.popupList.css({ "min-width": this.wrapper.width() });
        },

        _setListHeight: function () {
            if (this.model.itemsCount > 0) { this.popupList.css({ "max-height": this.model.itemsCount * 30 }); }
            else this.popupList.css({ "max-height": this.model.popupHeight });
			
        },
        _refreshPopup: function () {
            if (this.model.popupWidth == "auto") this.popupList.css({ "min-width": this.wrapper.width() });
            this._setListPosition();
            this._refreshScroller();
        },
        _setListPosition: function () {
            var elementObj = this.wrapper;
            var pos = this._getOffset(elementObj),
            left = pos.left,
            totalHeight = elementObj.outerHeight(),
            border = (totalHeight - elementObj.height()) / 2,
            maxZ = this._getZindexPartial();
            if (this.model.enableRTL) left -= this.popupList.outerWidth() - elementObj.outerWidth();
            this.popupList.css({
                "left": left + "px",
                "top": pos.top + totalHeight - border + 3 + "px",
                "z-index": maxZ
            });
        },
        _getOffset: function (ele) {
            var pos = ele.offset();
            if ($("body").css("position") != "static") {
                var bodyPos = $("body").offset();
                pos.left -= bodyPos.left;
                pos.top -= bodyPos.top;
            }
            return pos;
        },
        _getZindexPartial: function () {
            var parents = this.element.parents(), bodyEle;
            bodyEle = $('body').children(), index = bodyEle.index(this.popup);
            bodyEle.splice(index, 1);
            $(bodyEle).each(function (i, ele) { parents.push(ele); });

            var maxZ = Math.max.apply(maxZ, $.map(parents, function (e, n) {
                if ($(e).css('position') != 'static')
                    return parseInt($(e).css('z-index')) || 1;
            }));
            if (!maxZ || maxZ < 10000) maxZ = 10000;
            else maxZ += 1;
            return maxZ;
        },
        //Popup status
        _showResult: function () {
            var proxy = this;
            if (this._trigger("beforePopupShown", { text: this.element[0].value, value: this._selectedValue })) return;
            this._refreshPopup();
            this.popupList.css("display", "none");
            this.popupList.slideDown(this.model.enableAnimation ?200:1, "easeOutQuad", function () {
                proxy.popupList.focus();
                $(document).bind("mousedown", $.proxy(proxy._OnDocumentClick, proxy));
            });
            this.showPopupBox = true;
            this.element.attr({ "aria-expanded": true }); var _popupListItems = this.ultag.children("li");
            this._listSize = _popupListItems.size();

            _popupListItems.bind("mouseenter", $.proxy(this._OnMouseEnter, this));
            _popupListItems.bind("mouseleave", $.proxy(this._OnMouseLeave, this));
            _popupListItems.bind("click", $.proxy(this._OnMouseClick, this));
            $(window).bind("resize", $.proxy(this._OnWindowResize, this));
            
            this._trigger("popupShown", { text:this.element[0].value, value:this._selectedValue });
        },

        _OnWindowResize: function (e) {
            this._refreshPopup();
            this.popupList.css("display", "block");
        },
        _hideResult: function () {
            if (this.model && this.showPopupBox) {
                if (this._trigger("beforePopupHide", { text: this.element[0].value, value: this._selectedValue })) return;
                this.showPopupBox = false;
                var proxy = this;
				this.popupList.css("display", "none");
                this.popupList.slideUp(this.model.enableAnimation?100:0, "easeOutQuad", function () {
                    proxy.wrapper.focus();
                    $(document).unbind("mousedown", $.proxy(proxy._OnDocumentClick, proxy));
                });
				if (this.element != null)
                this.element.attr({ "aria-expanded": false }); var _popupListItems = this.ultag.children("li");
                $(window).unbind("resize", $.proxy(this._OnWindowResize, this));
                _popupListItems.unbind("mouseenter", $.proxy(this._OnMouseEnter, this));
                _popupListItems.unbind("mouseleave", $.proxy(this._OnMouseLeave, this));
                _popupListItems.unbind("click", $.proxy(this._OnMouseClick, this));
                if (this.element != null)
                    this._trigger("popupHide", {  text:this.element[0].value, value:this._selectedValue });
            }
        },
        // selecting text box value
        _enterTextBoxValue: function () {
            var args;
            this.removeID = false;
            this._chooseSelectionType();
            args = { text: this.currentValue, selectedText: this.currentValue, itemId: this.selectedIndexValue, value: this._selectedValue, isChecked: this.checkedStatus };
            if (!this._initValue) this._trigger("select", args);
            if (this.element[0].value != this.currentValue) {
                if (this.model.showCheckbox) {
                    if (this.model.allowMultiSelection) {
                        if (this.activeItem.find('.listcheckbox').ejCheckBox('isChecked') == false) {
                            this.activeItem.find('.listcheckbox').ejCheckBox('option', 'checked', true)
                            this.checkedStatus = true;
                        }
                        this._ulBox.append(this._createBox(this.currentValue));
                        if (this.showPopupBox)
                            this._setListPosition();
                    }
					else{
						if (this.activeItem.find('.listcheckbox').ejCheckBox('isChecked') == false) {
                            this.activeItem.find('.listcheckbox').ejCheckBox('option', 'checked', true)
                            this.checkedStatus = true;
                        }
					}
                    this._maintainHiddenValue();
                    this._addText(this.currentValue);
                    var _nodes = this.model.selectedItems;
                    if ($.inArray(this.selectedIndexValue, _nodes) == -1) {
                        this.model.selectedItems.push(this.selectedIndexValue);
                    }
                } else {
                    this.ultag.children("li").removeClass('e-hover').removeClass('e-active');
                    this.activeItem.addClass('e-active');
                    this.element.val(this.currentValue);
                    this._maintainHiddenValue();
                    this._hiddenInput.val(this._hiddenValue);
                    this.model.selectedItemIndex = this.selectedIndexValue;
                }

                args = { text: this.element[0].value, selectedText: this.currentValue, itemId: this.selectedIndexValue, value: this._selectedValue, isChecked: this.checkedStatus };

                if (!this._initValue)
                    this._trigger("change", args);
                this._cascadeAction();
                this.value(this.element.val());
                this.model.itemValue = this._selectedValue;
                this.model.text = this.currentValue;
                /*Watermark*/
                if (this.model.watermarkText != null) {
                    if (($.trim(this.element.val()) === '') || ($.trim(this.element.val()) === this.model.watermarkText)) {
                        this.element.val(this.model.watermarkText);
                        this.element.addClass("e-watermark");
                    } else {
                        this.element.removeClass("e-watermark");
                    }
                }
            }
        },
        _chooseSelectionType: function () {
            this.activeItem = this._getActiveItem();
            this.selectedIndexValue = this._activeItem;
            if (this.model.dataSource != null && typeof this.model.dataSource[0] == "object") {
                this.currentValue = $.trim(this._getField(this.popupListItems[this._activeItem], this.mapFld._text));
                this.hidelement = this._getField(this.popupListItems[this._activeItem], this.mapFld._value);
                this._selectedValue = this.hidelement;
                this._itemID = this._getField(this.popupListItems[this._activeItem], this.mapFld._id);
            } else {
                this.currentValue = $.trim(this.activeItem.text());
				if (this.activeItem[0].getAttribute("value") != null)
                    this._selectedValue = this.activeItem[0].getAttribute("value");
                else {
					if(this.currentValue!=null) {
						this.activeItem[0].setAttribute("value",this.currentValue);
						this._selectedValue=this.currentValue;
						}
					else
                       this._selectedValue = "";
                   }
                this._itemID = $(this.activeItem).attr("id");
            }
            if (!ej.isNullOrUndefined(this._itemID) && this._itemID != "") {
                if (!this.model.showCheckbox) {
                    this._selectedItemsID = [];
                    !this.removeID && this._selectedItemsID.push(this._itemID);
                }
                else
                    !this.removeID ? this._selectedItemsID.push(this._itemID) : this._removeSelectedItemsID();
            }
            this.selectedTextValue = this.currentValue;
        },
        _maintainHiddenValue:function(){
            if (this.activeItem[0].getAttribute("value")) {
                this._hiddenValue = this.activeItem[0].getAttribute("value");
            }
            else {
                this._hiddenValue = this.currentValue;
            }
        },
        _removeTextBoxValue: function () {
            this.removeID = true;
            this._chooseSelectionType();
            if (this.model.showCheckbox) {
                this._maintainHiddenValue();
                this._removeText(this.currentValue);
                var _nodes = this.model.selectedItems;
                if ($.inArray(this.selectedIndexValue, _nodes) > -1) {
                    this.model.selectedItems.splice($.inArray(this.selectedIndexValue, _nodes), 1);
                }
                if (this.model.allowMultiSelection) {
                    this._deleteBoxCheck(this.currentValue);
                    if (this.showPopupBox)
                        this._setListPosition();
                }
				else{
					if (this.activeItem.find('.listcheckbox').ejCheckBox('isChecked') == true) {
						this.activeItem.find('.listcheckbox').ejCheckBox('option', 'checked', false)
						this.checkedStatus = false;
					}
				}
            } else {
                this.ultag.children("li").removeClass('e-hover').removeClass('e-active');
                this._txtpos = (this.element[0].value).lastIndexOf(this.currentValue);
                if (this._txtpos > -1)
                    this.element.val(this.element[0].value.replace(this.currentValue, ""));
            }
            args = { text: this.currentValue, selectedText: this.currentValue, itemId: this.selectedIndexValue, value: this._selectedValue, isChecked: this.checkedStatus };
            if (!this._initValue)
                this._trigger("select", args);
            args = { text: this.element[0].value, selectedText: this.currentValue, itemId: this.selectedIndexValue, value: this._selectedValue, isChecked: this.checkedStatus };
            if (!this._initValue)
                this._trigger("change", args);
            this.value(this.element.val());
            this.model.itemValue = this._selectedValue;
            this.model.text = this.currentValue;
            /*Watermark*/
            if (this.model.watermarkText != null) {
                if (($.trim(this.element.val()) === '') || ($.trim(this.element.val()) === this.model.watermarkText)) {
                    this.element.val(this.model.watermarkText);
                    this.element.addClass("e-watermark");
                } else {
                    this.element.removeClass("e-watermark");
                }
            }
        },
        /* box */

        _createBox: function (value) {
            if (!this._checkContains(value) && this.checkedStatus) {
                var span = ej.buildTag("span.e-icon e-close");
                var li = ej.buildTag("li.e-options", value).append(span);
                this._on(span, "click", function (e) {
                    if (!this.model.enabled) return false;
                    this._deleteBox($(e.target).parent());
                });
                return li;
            }
        },
        _deleteBoxCheck: function (val) {
            var items = this._ulBox.children('li');
            for (var i = 0; i < items.length; i++) {
                if ($(items[i]).text() == val) {
                    $(items[i]).remove();
                }
            }
        },
        _deleteLastBox: function () {
            var items = this._ulBox.children("li:not(.e-search-box)");
            var item = items.last();
            if (item.hasClass("e-active")) this._deleteBox(item);
            else {
                this._removeActive();
                item.addClass("e-active");
            }
        },
        _deleteBox: function (items) {
            for (var i = 0; i < items.length; i++) {
                var cobj = $(items[i]);
                deltext = cobj.text();
                var listItems = this.ultag.children('li');
                for (var j = 0; j < listItems.length; j++) {
                    if ($(listItems[j]).text() == deltext) {
                        this._activeItem = j;
                        this._uncheckboxModel(this._activeItem);
                    }
                }
                cobj.remove();
            }
        },
        _removeActive: function () {
            this._ulBox.children("li").removeClass("e-active");
        },
        _adjustWidth: function () {
            var tempSpan = ej.buildTag("span", this.element.val());
            this.container.append(tempSpan);
            this.element.width(tempSpan.width() + 30);
            tempSpan.remove();
        },
        _uncheckboxModel: function (val) {
            var listItems = this.ultag.children('li');
            if ($(listItems[val].firstChild).find('.listcheckbox').ejCheckBox('isChecked')) {
                $(listItems[val].firstChild).find('.listcheckbox').ejCheckBox('option', 'checked', false);
                this._removeTextBoxValue();
                if (!this._initValue)
                    this._trigger('checkChange', { isChecked: this.checkedStatus, data: this.model });
            }
        },
        _destroyBoxModel: function () {
            this.container.prepend(this.element);
            this.element.css('display', 'block');
            this.wrapper.height(this.model.height);
            this._ulBox.remove();
            this._off(this.container, "mousedown");
        },
        //
        _removeListHover: function () {
            this.ultag.children("li").removeClass("e-hover");
        },

        _addListHover: function () {
            var activeItem = this._getActiveItem();
            activeItem.addClass("e-hover");
            this.scrollerObj.setModel({ "scrollTop": this._calcScrollTop() });
            activeItem.focus();
        },
        _calcScrollTop: function () {
            var ulH = this.ultag.outerHeight(), li = this.ultag.find("li"), liH = 0, index, top, i;
            index = this.ultag.find("li.e-hover").index();
            for (i = 0; i < index; i++) { liH += li.eq(i).outerHeight(); }
            top = liH - ((this.popupList.outerHeight() - li.eq(index).outerHeight()) / 2);
            return top;
        },
        _getActiveItem: function () {
            return $(this.ultag.children("li")[this._activeItem]);
        },
        _setDimentions: function () {
            if (this.model.height)
                this.wrapper.height(this.model.height);
            if (this.model.width)
                this.wrapper.width(this.model.width);
        },


        _roundedCorner: function (val) {
            if (val) {
                this.container.addClass("e-corner-all");
            }
            else {
                this.container.removeClass("e-corner-all");
            }

        },

        _enabled: function (boolean) {
            if (boolean) this.enable();
            else this.disable();
        },

        _destroyDropdown: function () {
            this.element.insertAfter(this.wrapper);
            this.element.width(this.element.width() + this.dropdownbutton.outerWidth());
            this.wrapper.remove();
            this.container.remove();
            this.element.unbind("mousedown", $.proxy(this._OnDropdownClick, this));//need to write for removing the list 
            this.dropdownbutton.unbind("mousedown", $.proxy(this._OnDropdownClick, this));//need to write for removing the list 
        },


        _RightToLeft: function () {
            if (this.model.enableRTL) {
                this.wrapper.addClass("e-rtl");
            }
            else {
                this.wrapper.removeClass("e-rtl");
            }

        },
        _dropbtnRTL: function () {
            if (this.model.enableRTL) {
                this.popupList.addClass("e-rtl");
            }
            else {
                this.popupList.removeClass("e-rtl");
            }
        },
        _OnDropdownClick: function (e) {
            e.preventDefault();
            if (this.model.readOnly) return false;
            if (this.ultag.find('li').length > 0) {
                if (this.showPopupBox) {
                    this._hideResult();
                }
                else {
                    this._showResult();
                    this.wrapper.focus();
                }
            }
        },

        _showFullList: function () {
            if (this.model.dataSource != null) {
                if (!(ej.DataManager && this.model.dataSource instanceof ej.DataManager)) {
                    this.popupListItems = this.model.dataSource;
                }
            }
            this._renderPopupList();
        },

        // cascade options
        _cascadeAction: function () {
            if (this.model.cascadeTo) {
                this._currentValue = this._getField(this.popupListItems[this._activeItem], this.mapFld._value);
                this.selectDropObj = $('#' + this.model.cascadeTo).ejDropDownList('instance');
                if (ej.isNullOrUndefined(this._dSource))
                    this._dSource = this.selectDropObj.model.dataSource;
                if (ej.DataManager && this._dSource instanceof ej.DataManager) {
                    this._performOdataInit();
                } else {
                    this._performJsonDataInit();
                }
            }
        },
        _performOdataInit: function () {
            var proxy = this, queryPromise, tempQuery;
            if (ej.isNullOrUndefined(this._dQuery)) {
                this._dQuery = this.selectDropObj.model.query.clone();
            }
            tempQuery = this._dQuery.clone();
            proxy.selectDropObj._addLoadingClass();
            queryPromise = this._dSource.executeQuery(tempQuery.where(this.mapFld._value, "equal", this._currentValue));
            queryPromise.fail(function (e) {
                proxy._changedSource = null;
                proxy.selectDropObj.setModel({ dataSource: proxy._changedSource, enabled: false });
            }).done(function (e) {
                proxy._changedSource = e.result;
                proxy.selectDropObj.setModel({ dataSource: proxy._changedSource, enabled: true });
                proxy.selectDropObj._removeLoadingClass();
            });
        },
        _performJsonDataInit: function () {
            this._changedSource = ej.DataManager(this._dSource).executeLocal(ej.Query().where(this.mapFld._value, "==", this._currentValue));
            this.selectDropObj.setModel({ dataSource: this._changedSource, enabled: true, value:"" });
        },
        //
        _OnMouseEnter: function (e) {
            var targetEle;
            this.ultag.children("li").removeClass("e-hover");
            if ($(e.target).is("li")) { $(e.target).addClass("e-hover"); }
            else if (e.target.tagName != "li") {
                targetEle = $(e.target).parents("li");
                $(targetEle).addClass("e-hover");
            }
            var activeItem = 0;
            this.ultag.children("li").each(function (index) {
                if ($(this).hasClass("e-hover")) {
                    activeItem = index;
                    return false;
                }
            });
            this._activeItem = activeItem;
        },
        _OnMouseLeave: function (e) {
            this.ultag.children("li").removeClass("e-hover");
        },
        _OnMouseClick: function (e) {
            if (!this.model.showCheckbox) {
                if (this.model.template) {
                    this._templateValue();
                } else {
                    this._enterTextBoxValue();
                }
                this._hideResult();
            } else {
                if (($(e.currentTarget).is("li")) && ($(e.target).is("li"))) {
                    if ($(e.currentTarget.firstChild).find('.listcheckbox').ejCheckBox('isChecked')) {
                        $(e.currentTarget.firstChild).find('.listcheckbox').ejCheckBox('option', 'checked', false);
                        this.checkedStatus = false;
                        this._removeTextBoxValue();
                    } else {
                        $(e.currentTarget.firstChild).find('.listcheckbox').ejCheckBox('option', 'checked', true);
                        this.checkedStatus = true;
                        this._enterTextBoxValue();
                    }
                }
                if (!this._initValue)
                    this._trigger('checkChange', { isChecked: this.checkedStatus, data: this.model });
            }
        },
        _OnClickCheckList: function (e) {
            if (e.isChecked) {
                this.checkedStatus = true;
                this._enterTextBoxValue();
            } else {
                this.checkedStatus = false;
                this._removeTextBoxValue();
            }
            if (!this._initValue)
                this._trigger('checkChange', { isChecked: this.checkedStatus, data: this.model });

        },
        _OnListSelect: function (e) {
            e.preventDefault();
            activeItem = this._getActiveItem();
            if ($(activeItem).find('.listcheckbox').ejCheckBox('isChecked')) {
                $(activeItem).find('.listcheckbox').ejCheckBox('option', 'checked', false);
                this.checkedStatus = false;
                this._removeTextBoxValue();
            } else {
                $(activeItem).find('.listcheckbox').ejCheckBox('option', 'checked', true);
                this.checkedStatus = true;
                this._enterTextBoxValue();
            }

            if (!this._initValue)
                this._trigger('checkChange', { isChecked: this.checkedStatus, data: this.model });
        },
        _OnDocumentClick: function (e) {
            if (!$(e.target).is(this.popupList) && !$(e.target).parents(".e-ddl-popup").is(this.popupList) &&
                !$(e.target).is(this.element) && !$(e.target).parents(".e-ddl").is(this.wrapper)) {
                this._hideResult();
            }
            else if ($(e.target).is(this.popupList) || $(e.target).parents(".e-ddl-popup").is(this.popupList))
                e.preventDefault();
        },

        //keyboard events
        _OnKeyDown: function (e) {
            if ((!this.model.enabled) || (this.model.readOnly)) return false;
            this._itemId = null;
            var _popupListItems = this.ultag.children("li");
            this._listSize = this.ultag.children("li").size();
			var liH, popupH, activeitem,flag;
            popupH = this.popupList.height();
            liH = this.ultag.children("li").outerHeight();
            activeitem = Math.round(popupH / liH);
			
            switch (e.keyCode) {
                case 38://Up arrow
                    if (e.altKey) {
                        if (this.ultag.find('li').length > 0)
                            this._hideResult();
					break;
                    }
                case 33:// pageup key
                    var flag = ((e.keyCode == 38) ? 1 : activeitem);
                    if (this.showPopupBox) {
                        this._removeListHover();
                        this.ultag.children("li").removeClass('e-active');
                        if ((this._activeItem <= 0) || (this._activeItem == null) || (this._activeItem > this._listSize - 1))
                            this._activeItem = this._listSize - 1;
                        else
                            this._activeItem -= flag;
                        this._addListHover();
                        if ((!this.model.showCheckbox) && (this.model.template == null))
                            this._enterTextBoxValue();
                    } else {
                        this.ultag.children("li").removeClass('e-active');
                        if ((this._activeItem <= 0) || (this._activeItem == null) || (this._activeItem > this._listSize - 1))
                            this._activeItem = this._listSize - 1;
                        else
                            this._activeItem -= flag;
                        if ((!this.model.showCheckbox) && (this.model.template == null))
                            this._enterTextBoxValue();
                    }
                    e.preventDefault();
                    return false;
					break;
                case 40: //Down arrow
                    if (e.altKey) {
                        if (this.ultag.find('li').length > 0)
                            this._showResult();
					break;
                    }
                case 34://pagedown key  
                    var flag = ((e.keyCode == 40) ? 1 : activeitem);
                    if (this.showPopupBox) {
                        this._removeListHover();
                        this.ultag.children("li").removeClass('e-active');
                        if ((this._activeItem >= this._listSize - 1) || (this._activeItem == null) || (this._activeItem < 0))
                            this._activeItem = 0;
                        else
                            this._activeItem += flag;
                        this._addListHover();
                        if ((!this.model.showCheckbox) && (this.model.template == null))
                            this._enterTextBoxValue();
                    } else {
                        this.ultag.children("li").removeClass('e-active');
                        if ((this._activeItem >= this._listSize - 1) || (this._activeItem == null) || (this._activeItem < 0))
                            this._activeItem = 0;
                        else
                            this._activeItem += flag;
                        if ((!this.model.showCheckbox) && (this.model.template == null))
                            this._enterTextBoxValue();
                    }
                    e.preventDefault();
                    return false;  
                    break;
                case 37:// Left Key equals up
                    if (this.showPopupBox) {
                        this._removeListHover();
                        this.ultag.children("li").removeClass('e-active');
                        if ((this._activeItem <= 0) || (this._activeItem == null) || (this._activeItem > this._listSize - 1))
                            this._activeItem = this._listSize - 1;
                        else
                            this._activeItem -= 1;
                        this._addListHover();
                        if ((!this.model.showCheckbox) && (this.model.template == null))
                            this._enterTextBoxValue();
                    } else {
                        this.ultag.children("li").removeClass('e-active');
                        if ((this._activeItem <= 0) || (this._activeItem == null) || (this._activeItem > this._listSize - 1))
                            this._activeItem = this._listSize - 1;
                        else
                            this._activeItem -= 1;
                        if ((!this.model.showCheckbox) && (this.model.template == null))
                            this._enterTextBoxValue();
                    }
                    e.preventDefault();
                    return false;
                    break;  // Left Key
                case 39:// Right Key equals down
                    if (this.showPopupBox) {
                        this._removeListHover();
                        this.ultag.children("li").removeClass('e-active');
                        if ((this._activeItem >= this._listSize - 1) || (this._activeItem == null) || (this._activeItem < 0))
                            this._activeItem = 0;
                        else
                            this._activeItem += 1;
                        this._addListHover();
                        if ((!this.model.showCheckbox) && (this.model.template == null))
                            this._enterTextBoxValue();
                    } else {
                        this.ultag.children("li").removeClass('e-active');
                        if ((this._activeItem >= this._listSize - 1) || (this._activeItem == null) || (this._activeItem < 0))
                            this._activeItem = 0;
                        else
                            this._activeItem += 1;
                        if ((!this.model.showCheckbox) && (this.model.template == null))
                            this._enterTextBoxValue();
                    }
                    e.preventDefault();
                    return false;
                    break;  // Right Key
                case 8:    // Backspace key
                    if ((this.model.allowMultiSelection == true) && (this.model.showCheckbox)) {
                        this._deleteLastBox();
                    }
                    e.preventDefault();
                    return false;
                    break;
                case 9:    // Tab key
                    this._hideResult();
                    break;
                case 35:
                    if (this.showPopupBox) {
                        this._removeListHover();
                        this.ultag.children("li").removeClass('e-active');
                        this._activeItem = this._listSize - 1;
                        this._addListHover();
                        if ((!this.model.showCheckbox) && (this.model.template == null))
                            this._enterTextBoxValue();
                    }
                    else {
                        this.ultag.children("li").removeClass('e-active');
                        this._activeItem = this._listSize - 1;
                        if ((!this.model.showCheckbox) && (this.model.template == null))
                            this._enterTextBoxValue();
                    }
                    e.preventDefault();
                    return false;
                    break;  // End Key
                case 36:
                    if (this.showPopupBox) {
                        this._removeListHover();
                        this.ultag.children("li").removeClass('e-active');
                        this._activeItem = 0;
                        this._addListHover();
                        if ((!this.model.showCheckbox) && (this.model.template == null))
                            this._enterTextBoxValue();
                    }
                    else {
                        this.ultag.children("li").removeClass('e-active');
                        this._activeItem = 0;
                        if ((!this.model.showCheckbox) && (this.model.template == null))
                            this._enterTextBoxValue();
                    }
                    e.preventDefault();
                    return false;
                    break;  // Home Key
                case 17:    // Ctrl key
                    break;
                case 18:    // Alt key
                    break;
                case 27:    // Esc key
                    this._hideResult();
					e.stopPropagation();
                    break;
                case 32:    // space Key
                    e.preventDefault();
                    return false;
                    break;
            }
        },

        _OnKeyUp: function (e) {
            if ((!this.model.enabled) || (this.model.readOnly)) return false;
            e.preventDefault();
            var target = e.target;
            if ($.trim(this.element.val()) == "" && e.keyCode == 38 && e.keyCode == 40) {
                this._hideResult();
                return false;
            }

            switch (e.keyCode) {
                //Restricts the keyup event for other functional keys
                case 38: break;  // Up Key
                case 40: break;  // Down Key
                case 37: break;  // Left Key
                case 39: break;  // Right Key

                case 20: break;  // CapsLk
                case 16: break;  // Shift Key
                case 17: break;  // Ctrl Key
                case 18: break;  // Alt Key
                case 35: break;  // End Key
                case 36: break;  // Home Key
                case 144: break;  // Num Lock
                case 27: break;  // Esc Key
                case 9: break;  // Tab Key

                case 13:    // Enter Key
                    e.preventDefault();
                    if (this.model.showCheckbox) {
                        this._OnListSelect(e);
                    } else if (this.model.template) {
                        this._templateValue();
                        this._hideResult();
                    }
                    else {
                        this._enterTextBoxValue();
                        this._hideResult();
                    }

                    return false;
                    break;
                case 32:    // space Key
                    e.preventDefault();
                    if (this.model.showCheckbox) {
                        this._OnListSelect(e);
                    } else if (this.model.template) {
                        this._templateValue();
                        this._hideResult();
                    }
                    else {
                        this._enterTextBoxValue();
                        this._hideResult();
                    }
                    return false;
                    break;
                case 8:     // Backspace Key
                    // this._OnTextEnter("backspace");
                    this._hideResult();
                    break;
                case 46:    // Delete Key
                    if ((this.model.allowMultiSelection == true) && (this.model.showCheckbox)) {
                        this._deleteBox(this._ulBox.children("li.e-active"));
                        break;
                    }

                //default:
                //    if  (this.model.enableIncrementalSearch) {
                //        this._OnTextEnter(e.keyCode);
                //    }
                //    break;
            }
        },
        _OnKeyPress: function (e) {
            if (this.model.enableIncrementalSearch) {
                this._OnTextEnter(e.charCode);
            }
        },
        //
        //template type
        _templateValue: function () {
            var activeitem = $(this.ultag.children("li")[this._activeItem]);
            currentText = this._getField(this.popupListItems[this._activeItem], this.mapFld._text);
            currentValue = this._getField(this.popupListItems[this._activeItem], this.mapFld._value);
            if (this.ultag.children("li")[this._activeItem].getAttribute("value")) 
                this._hiddenValue = currentValue;
            else 
                this._hiddenValue = currentText;
            args = { text: currentText, selectedText: currentText, itemId: this._activeItem, value: currentValue };
            if (!this._initValue)
                this._trigger("select", args);
            if (this._activeItem != this._aselectedItem) {
                this.ultag.children("li").removeClass('e-hover').removeClass('e-active');
                activeitem.addClass('e-active');
                this._aselectedItem = this._activeItem;
                this.element.val(currentText);
                this._hiddenInput.val(this._hiddenValue);
                args = { text: this.element[0].value, selectedText: currentText, itemId: this._activeItem, value: currentValue };
                if (!this._initValue)
                    this._trigger("change", args);
                this.value(this.element.val());
                this.model.itemValue = currentValue;
                this.model.text = currentText;
            }
            /*Watermark*/
            if (this.model.watermarkText != null) {
                if (($.trim(this.element.val()) === '') || ($.trim(this.element.val()) === this.model.watermarkText)) {
                    this.element.val(this.model.watermarkText);
                    this.element.addClass("e-watermark");
                } else {
                    this.element.removeClass("e-watermark");
                }
            }
        },
        //
        //On Text type
        _OnTextEnter: function (from) {
            // this._hideResult();
            _proxy = this;
            this._incqueryString += String.fromCharCode(from);
            if (this._incqueryString.length > 0) {
                setTimeout(function () { _proxy._incqueryString = ""; }, 1000);
            }
            var list = this.listitems, i, strlen,
            caseSence = this.model.caseSensitiveSearch, mapper = this.model.fields,
            tempSuggestion = [],
            str, queryStr = this._incqueryString,
            querylength = this._incqueryString.length, searchflag = false;

            if (!caseSence) queryStr = queryStr.toLowerCase();

            for (i = 0; i < list.length; i++) {
                str = list[i].textContent.trim();
                str = caseSence ? str : str.toLowerCase();
                if (str.substr(0, querylength) == queryStr) {
                    if (this.showPopupBox) {
                        this._removeListHover();
                        this._activeItem = i;
                        if(!this.model.showCheckbox)
                        this._enterTextBoxValue();
                        this._addListHover();
                    } else {
                        this._activeItem = i;
                        if (!this.model.showCheckbox)
                        this._enterTextBoxValue();
                    }
                    searchflag = true;
                } else if ((i == list.length - 1) && (searchflag == false)) {
                    if (this.showPopupBox) {
                        this._removeListHover();
                        this._activeItem = i;
                        if (!this.model.showCheckbox)
                        this._enterTextBoxValue();
                        this._addListHover();
                    } else {
                        this._activeItem = i;
                        if (!this.model.showCheckbox)
                        this._enterTextBoxValue();
                    }
                    searchflag = true;
                }
                if (searchflag) break;
            }

        },

        //foucus in and out
        _targetFocus: function () {
            if (this.model.enabled) {
                if (this.element.hasClass("e-watermark")) {
                    this.element.removeClass("e-watermark");
                    this.element.val(null);
                }
                this.wrapper.addClass("e-focus");
                this._isFocused = true;
                /* Raise focusIn Event*/
               
                    this._trigger("focusIn");
            }
        },

        _targetBlur: function () {
            if (this.model.enabled) {
                this._isFocused = false;
                this.wrapper.removeClass("e-focus");
                if (this.model.watermarkText != null) {
                    if ($.trim(this.element.val()) === '') {
                        this.element.val(this.model.watermarkText);
                        this.element.addClass("e-watermark");
                    }
                }
                /* Raise focusOut Event*/
                
                    this._trigger("focusOut");
            }
        },

        //-------------------- Event Wire-up -------------------------//
        _wireEvents: function () {
            this._on(this.wrapper, "focus", this._targetFocus);
            this._on(this.wrapper, "blur", this._targetBlur);
            // this._on(this.element, "input", this._valueChange);
            //this._on(this.element, "keydown", this._OnKeyDown);
            //this._on(this.element, "keyup", this._OnKeyUp);


            this._on(this.wrapper, "keydown", this._OnKeyDown);
            this._on(this.popupList, "keydown", this._OnKeyDown);
            this._on(this.popupList, "keyup", this._OnKeyUp);
            this._on(this.wrapper, "keyup", this._OnKeyUp);
            this._on(this.popupList, "keypress", this._OnKeyPress);
            this._on(this.wrapper, "keypress", this._OnKeyPress);

        }
    });
})(jQuery, Syncfusion);;
/**
* @fileOverview Plugin to style the Html ListView elements
* @copyright Copyright Syncfusion Inc. 2001 - 2013. All rights reserved.
*  Use of this code is subject to the terms of our license.
*  A copy of the current license can be obtained at any time by e-mailing
*  licensing@syncfusion.com. Any infringement will be prosecuted under
*  applicable laws. 
* @version 12.1 
* @author <a href="mailto:licensing@syncfusion.com">Syncfusion Inc</a>
*/

(function ($, ej, undefined) {
    //ListView is the plugin name 
    //"ListView" is "namespace.className" will hold functions and properties
    /**
    * @namespace ej
    * @class ejListView
    * @requires jQuery
    * @requires ej.core
    * @requires ej.unobtrusive
    * @requires ej.data
    * @requires ej.touch
    * @requires ej.checkbox
    * @requires ej.listviewbase
    * @classdesc Custom Design for Html ListView control.
    * @example 
    * //Set listview in unobtrusive way.
    * &lt;div id="lb" data-role="ejlistview"&gt;
    *	 &lt;ul&gt;
    *		&lt;li data-ej-text="Artwork"&gt;&lt;/li&gt;
    * 		&lt;li data-ej-text="Abstract"&gt;&lt;/li&gt;
    *		&lt;li data-ej-text="2 Acrylic Mediums"&gt;&lt;/li&gt;
    *		&lt;li data-ej-text="Creative Acrylic"&gt;&lt;/li&gt;
    *		&lt;li data-ej-text="Modern Painting"&gt;&lt;/li&gt;
    *		&lt;li data-ej-text="Canvas Art"&gt;&lt;/li&gt;
    * 		&lt;li data-ej-text="Black white"&gt;&lt;/li&gt;
    *		&lt;li data-ej-text="Children"&gt;&lt;/li&gt;
    *		&lt;li data-ej-text="Preschool Crafts"&gt;&lt;/li&gt;
    *		&lt;li data-ej-text="School-age Crafts"&gt;&lt;/li&gt;
    * 	&lt;/ul&gt;
    * &lt;/div&gt;
    * @example 
    * //Set listview in obtrusive way.
    * &lt;div id="lb"&gt;
    *	 &lt;ul&gt;
    *		&lt;li data-ej-text="Artwork"&gt;&lt;/li&gt;
    * 		&lt;li data-ej-text="Abstract"&gt;&lt;/li&gt;
    *		&lt;li data-ej-text="2 Acrylic Mediums"&gt;&lt;/li&gt;
    *		&lt;li data-ej-text="Creative Acrylic"&gt;&lt;/li&gt;
    *		&lt;li data-ej-text="Modern Painting"&gt;&lt;/li&gt;
    *		&lt;li data-ej-text="Canvas Art"&gt;&lt;/li&gt;
    * 		&lt;li data-ej-text="Black white"&gt;&lt;/li&gt;
    *		&lt;li data-ej-text="Children"&gt;&lt;/li&gt;
    *		&lt;li data-ej-text="Preschool Crafts"&gt;&lt;/li&gt;
    *		&lt;li data-ej-text="School-age Crafts"&gt;&lt;/li&gt;
    * 	&lt;/ul&gt;
    * &lt;/div&gt;
    * &lt;script&gt; 
    * // Create ListView
    * $("#lb").ejListView(); 
    * &lt;/script&gt;
    */

    ej.widget("ejListView", "ej.ListView", {
        _rootCSS: "e-lv",
        //Default model values
        _prefixClass: "e-",
        defaults: {

            /**    
            * Event triggers when mouse down happens on the item.
            * @event
            * @name ejListView#mouseDown
            * @param {Object} argument Event parameters from listview.
            * @param {boolean} argument.cancel returns true if the event should be cancelled; otherwise, false.
            * @param {string} argument.type returns the name of the event.
            * @param {Object} argument.model returns the model value of the control.
            * @param {boolean} argument.hasChild If the child element exist return true; otherwise, false.
            * @param {string} argument.item  returns the current list item.
            * @param {string} argument.text returns the current text of item.
            * @param {number} argument.index returns the current Index of the item.
            * @param {boolean} argument.isChecked If checked return true; otherwise, false.
            * @param {number} argument.checkedItems returns the list of checked items.
            * @param {string} argument.checkedItemsText returns the current checked item text.
            * @example 
            * //Set the mouseDown property in unobtrusive way.
            * &lt;div id="lb" data-role="ejlistview" data-ej-mouseDown="mouseDown"&gt;
            * 	&lt;ul&gt;
            *		&lt;li data-ej-text="Artwork"&gt;&lt;/li&gt;
            * 		&lt;li data-ej-text="Abstract"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="2 Acrylic Mediums"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Creative Acrylic"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Modern Painting"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Canvas Art"&gt;&lt;/li&gt;
            * 		&lt;li data-ej-text="Black white"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Children"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Preschool Crafts"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="School-age Crafts"&gt;&lt;/li&gt;
            *	&lt;/ul&gt;
            * &lt;/div&gt;
            * @example 
            * //mouseDown event for ListView
            * &lt;div id="lb" data-role="ejlistview"&gt;
            * 	&lt;ul&gt;
            *		&lt;li data-ej-text="Artwork"&gt;&lt;/li&gt;
            * 		&lt;li data-ej-text="Abstract"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="2 Acrylic Mediums"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Creative Acrylic"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Modern Painting"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Canvas Art"&gt;&lt;/li&gt;
            * 		&lt;li data-ej-text="Black white"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Children"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Preschool Crafts"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="School-age Crafts"&gt;&lt;/li&gt;
            *	&lt;/ul&gt;
            * &lt;/div&gt;
            * &lt;script&gt;
            * $(document).ready(function(){
            * $("#lb").ejListView({
            *   	mouseDown: function (args) { //handle the event 
            * }
            *	});         
            * });
            * &lt;/script&gt;
            * @memberof ejListView
            * @instance
            */
            mouseDown: null,
            /**    
            * Event triggers when mouse up happens on the item.
            * @event
            * @name ejListView#mouseUP	
            * @param {Object} argument Event parameters from listview.
            * @param {boolean} argument.cancel returns true if the event should be cancelled; otherwise, false.
            * @param {string} argument.type returns the name of the event.
            * @param {Object} argument.model returns the model value of the control.
            * @param {boolean} argument.hasChild If the child element exist return true; otherwise, false.
            * @param {string} argument.item  returns the current list item.
            * @param {string} argument.text returns the current text of item.
            * @param {number} argument.index returns the current Index of the item.
            * @param {boolean} argument.isChecked If checked return true; otherwise, false.
            * @param {number} argument.checkedItems returns the list of checked items.
            * @param {string} argument.checkedItemsText returns the current checked item text.
            * @example 
            * //Set the mouseUP property in unobtrusive way.
            * &lt;div id="lb" data-role="ejlistview" data-ej-mouseUP="mouseUP"&gt;
            * 	&lt;ul&gt;
            *		&lt;li data-ej-text="Artwork"&gt;&lt;/li&gt;
            * 		&lt;li data-ej-text="Abstract"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="2 Acrylic Mediums"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Creative Acrylic"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Modern Painting"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Canvas Art"&gt;&lt;/li&gt;
            * 		&lt;li data-ej-text="Black white"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Children"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Preschool Crafts"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="School-age Crafts"&gt;&lt;/li&gt;
            *	&lt;/ul&gt;
            * &lt;/div&gt;
            * @example 
            * //mouseUP event for ListView
            * &lt;div id="lb" data-role="ejlistview"&gt;
            * 	&lt;ul&gt;
            *		&lt;li data-ej-text="Artwork"&gt;&lt;/li&gt;
            * 		&lt;li data-ej-text="Abstract"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="2 Acrylic Mediums"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Creative Acrylic"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Modern Painting"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Canvas Art"&gt;&lt;/li&gt;
            * 		&lt;li data-ej-text="Black white"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Children"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="Preschool Crafts"&gt;&lt;/li&gt;
            *		&lt;li data-ej-text="School-age Crafts"&gt;&lt;/li&gt;
            *	&lt;/ul&gt;
            * &lt;/div&gt;
            * &lt;script&gt;
            * $(document).ready(function(){
            * $("#lb").ejListView({
            *   	mouseUP: function (args) { //handle the event 
            * }
            *	});    
            * });
            * &lt;/script&gt;
            * @memberof ejListView
            * @instance
            */
            mouseUp: null,

        },

        //Datatypes of model variables
        /**
        * Specify the data types for default properties 
        * @private
        */
        dataTypes: {
            dataSource: "data",
            fieldSettings: "data",
            renderMode: "enum",
            theme: "enum",
            enablePersistence: "boolean"
        },
        observables: ["selectedItemIndex", "dataSource"],
        selectedItemIndex: ej.util.valueFunction("selectedItemIndex"),
        dataSource: ej.util.valueFunction("dataSource"),
        //Constructor function
        /**
        * Create the ListView widget
        * @private
        */
        _init: function () {
            this._preventDefaultException = { tagName: /^(INPUT|TEXTAREA|BUTTON|SELECT)$/ };
            this._storedContent = [];
            this._storedTemplate = [];
            this._touchStart = this.model.mouseDown;
            this._touchEnd = this.model.mouseUp;
            this._load();
        },
        _updateContent: function (currentItem, backButton) {
            var proxy = this;
            var $curIte = $(currentItem), childid = $curIte.attr('data-childid'),
                id = this.element.find('#' + childid);
            if (id.length) {
                var hdr = this.element.find('#' + childid + '_header');
                if (this.model.enableFiltering)
                    this._initializeFiltering($(id));
                var title = $curIte.attr('data-childtitle') == undefined ? currentItem.text() : $curIte.attr('data-childtitle');
                if (hdr.hasClass('e-header')) {
                    this._setHeaderVal(childid, title, backButton);
                }
                id.attr("data-hdr-title", title).attr("data-hdr-bckbtn", backButton);
                $curIte.closest('.subpage').hide();
                this._initEJCheckBox(id);
                id.show();
                $(this.element.children()[0]).removeClass("e-slideright");
                id.addClass("e-slideleft");
            }
        },

        //To render header
        /**
        * Section for Render Header for Listbox control	
        * @private
        */
        _renderHeader: function (id, showbutton, title, backtext) {
            var header = ej.buildTag("div", "", {}, { "id": id + "_header", "class": "e-header" });
            if (showbutton) {
                header.append("<span class='e-hicon e-icon e-rarrowleft-2x'></span>");
                header.append("<div class='e-btn-text'>" + (backtext ? backtext : "Back") + "</div>");
            }
            else
                header.append("<div class='e-htitle'>" + title + "</div>");
            return header;
        },
        _setHeaderVal: function (id, title, backtext) {
            this._onBackButtonDelegate = $.proxy(this._onBackButtonClick, this);
            ej.listenTouchEvent($("#" + id + "_header"), ej.endEvent(), this._onBackButtonDelegate, false);
            $("#" + id + "_header").find(".e-btn-text").text(this.model.headerBackButtonText?this.model.headerBackButtonText:"Back");
            $("#" + id + "_header").find(".e-htitle").text(title);
        },
        _onBackButtonClick: function (e) {
            this.element.find(".e-slideleft").removeClass("e-slideleft");
            this.element.children(":visible").hide();
            $(this.element.children()[0]).show();
            $(this.element.children()[0]).addClass("e-slideright");
        },
        //Return the data in event triggering to the user
        /**
        * Section for to return data in event triggering to the user.
        * @private
        */
        _returnData: function () {
            var checkedItem = this._currentItem.closest('ul.e-list-hdr').find('.e-chkbox-wrap[aria-checked="true"]').closest("li");
            return {
                hasChild: this.element.find('#' + this._currentItem.attr('data-childid')).length ? true : false,
                item: this._currentItem,
                text: this._currentItem.text(),
                index: this._currentItem.index(),
                isChecked: this._currentItem.find("input.e-lv-check").closest('.e-chkbox-wrap').attr('aria-checked') == "true" ? true : false,
                checkedItems: checkedItem.length ? checkedItem : null,
                checkedItemsText: $(checkedItem).map(function () { return $(this).text(); }).get()
            };
        },
        _initEJCheckBox: function (ul) {
            $(ul.find('.e-lv-check')).ejCheckBox();
            ul.find('.e-lv-checkdiv').removeClass("e-lv-checkdiv")
            ul.find('.e-lv-check').parent().addClass("e-lv-checkdiv")
        },
        _triggerStartEvent: function (data) {
            this.model.mouseDown = ej.getAttrVal(this._currentItem, 'data-mousedown', this._touchStart);
            if (this.model.mouseDown)
                this._trigger("mouseDown", data);
        },
        _triggerEndEvent: function (data) {
            this.model.mouseUp = ej.getAttrVal(this._currentItem, 'data-mouseup', this._touchEnd);
            this.selectedItemIndex(this._currentItem.index());
            if (this.model.mouseUp)
                this._trigger("mouseUp", data);
        },
        _createFilterDiv: function () {
            return ej.buildTag('div.e-lv-filter', ej.buildTag('input.e-lv-input', "", {}, { 'type': 'text', 'placeholder': 'search' }));
        },
        _emptyFilterTextValue: function (element) {
            element.find('.e-lv-input').val("");
        },
        _createListDiv: function () {
            this._div = ej.buildTag("div#" + childId + "." + this._rootCSS + " subpage e-childitem e-ajaxchild", this.model.showHeader ? this._renderHeader(childId, true, this._currentItem.text()) : "");
        },
        _createCheckBox: function () {
            return ej.buildTag("input.e-lv-check", "", {}, {});
        },
        _toggleCheckboxValue: function (lbCheck) {
            lbCheck.ejCheckBox({ checked: $(lbCheck.closest('.e-chkbox-wrap')).attr('aria-checked') == "true" ? false : true });
        },
        _setCheckboxValue: function (element, val) {
            element.ejCheckBox({ checked: val });
        },
        _convertToRelativeUrl: function (url) {
            return url;
        }
    });
    $.extend(true, ej.ListView.prototype, ej.ListViewBase.prototype);
})(jQuery, Syncfusion);;
/*!
*  filename: ej.navigationdrawer.js
*  version : 12.1
*  Copyright Syncfusion Inc. 2001 - 2013. All rights reserved.
*  Use of this code is subject to the terms of our license.
*  A copy of the current license can be obtained at any time by e-mailing
*  licensing@syncfusion.com. Any infringement will be prosecuted under
*  applicable laws. 
*/

/**
* @fileOverview Plugin to style the Html Navigation Drawer elements
* @copyright Copyright Syncfusion Inc. 2001 - 2013. All rights reserved.
*  Use of this code is subject to the terms of our license.
*  A copy of the current license can be obtained at any time by e-mailing
*  licensing@syncfusion.com. Any infringement will be prosecuted under
*  applicable laws. 
* @version 12.1 
* @author <a href="mailto:licensing@syncfusion.com">Syncfusion Inc</a>
*/
(function ($, ej, undefined) {
    /**
    * @namespace ej
    * @class ejNavigationDrawer
    * @requires jQuery
    * @requires ej.core.js
    * @requires ej.navigationdrawer.js
    * @requires ej.listview.js
    * @classdesc Custom Design for Html Navigation Drawer control.
    * @example 
    * //create Navigation Drawer in Obtrusive way
    * &lt;div id="home" class="navsubpage"&gt;
    * &lt;div align="center" class="content"&gt;
    * &lt;h2 class="title"&gt;
    * Home&lt;/h2&gt;
    * &lt;p&gt;
    * Founded by industry experts in 2001,Syncfusion, Inc. provides the broadest range of enterprise-class software components and tools for the Microsoft .NET platform.
    * &lt;/p&gt;
    * &lt;/div&gt;
    * &lt;/div&gt;
    * &lt;style&gt;
    * .list {
    *   border-bottom: 1px solid;
    *   line-height: 50px;
    *   text-align: center;
    *   width:200px;
    *   }
    * &lt;/style&gt;
    * &lt;div &gt;
    * &lt;div class="list"&gt; Home &lt;/div&gt;
    * &lt;div class="list"&gt; Communities &lt;/div&gt;
    * &lt;/div&gt;
    * &lt;script&gt;
    * $(function () {
    * $("#navpane").ejNavigationDrawer();	
    * });
    * &lt;/script&gt;
    **/
    ej.widget("ejNavigationDrawer", "ej.NavigationDrawer", {

        _setFirst: false,
        validTags: ["div"],
        _rootCSS: "e-nb",
        _prefixClass: "e-",
        defaults: {

    },

    dataTypes: {

},

_init: function () {
    this._load();
    this._renderControl();
    this._renderEJMControls();
    this._createDelegate();
    this._wireEvents();
},

_load: function () {
    this._browser = this._browser();
    if (this.model.position == "normal") {
        this._elementWrapper = ej.buildTag("div#" + this._id + "_WrapContainer", {}, {}, { class: this._rootCSS + " e-nb-container" });
        this.element.wrapAll(this._elementWrapper);
        this._nbHome = $("#" + this._id + "_WrapContainer").parent();
    }
    else {
        this._nbHome = $("body");
        this.element.appendTo(this._nbHome);
    }
},

_renderEJMControls: function () {
    if (this.model.enableListView) {
        this._lb = ej.buildTag("div#" + this._id + "_listbox", {}, {}, { class: "e-nb-listbox" });
        this.element.find("ul").wrapAll(this._lb);
        this._lb = $("#" + this._id + "_listbox");
        this._lb.ejListView(this.model.listViewSettings);
        this._lb.ejListView({ loadComplete: $.proxy(this._setLayout, this) });
    }

},

_setModel: function (options) {
    var refresh = false;
    for (var prop in options) {
        switch (prop) {
            case "width":
                this._setWidth();
                this._setLayout();
                break;
            case "direction":
                this._setLayout();
                break;
            case "type":
                this._transform(0, 0, true);
                this._transform(0, 0, false);
                break;
            default:
                refresh = true;
        }
        if (refresh) { this._refresh(); }
    }
},

_destroyEJMPlugin: function () {
    this._lb.ejListView("destroy");
    if (this._lb) this._lb.unwrap();
},

_browser: function () {
    return (/webkit/i).test(navigator.appVersion) ? 'webkit' : (/firefox/i).test(navigator.userAgent) ? 'Moz' : (/trident/i).test(navigator.userAgent) ? 'ms' : 'opera' in window ? 'O' : '';
}

});
$.extend(true, ej.NavigationDrawer.prototype, ej.NavigationDrawerBase.prototype);
$.extend(true, ej.NavigationDrawer.prototype.defaults.listViewSettings, ej.ListView.prototype.defaults);

})(jQuery, Syncfusion);
;
/**
* @fileOverview Plugin to style the Html List elements
* @copyright Copyright Syncfusion Inc. 2001 - 2013. All rights reserved.
*  Use of this code is subject to the terms of our license.
*  A copy of the current license can be obtained at any time by e-mailing
*  licensing@syncfusion.com. Any infringement will be prosecuted under
*  applicable laws. 
* @version 12.1 
* @author <a href="mailto:licensing@syncfusion.com">Syncfusion Inc</a>
*/


(function ($, ej, undefined) {
    /**
   * @namespace ej
   * @classdesc The Listbox control provides a list options to make user to choose an item from the list. It is capable of including other html elements such as images, textboxes, check box, radio buttons and so on.
   * @class ejListBox
   * @requires jQuery
   * @requires jquery.easing.1.3.js
   * @requires ej.core.js
   * @requires ej.data.js
   * @requires ej.listbox.js
   * @requires ej.checkbox.js
   * @requires ej.scroller.js
   * @example 
  
   * 
   *    &lt;ul id="carsList"&gt;
   *       &lt;li&gt;Audi A4&lt;/li&gt;
   *       &lt;li&gt;Audi A5&lt;/li&gt;
   *       &lt;li&gt;Audi A6&lt;/li&gt;
   *       &lt;li&gt;Audi A7&lt;/li&gt;
   *       &lt;li&gt;Audi A8&lt;/li&gt;
   *	   &lt;li&gt;BMW 501&lt;/li&gt;
   *       &lt;li&gt;BMW 502&lt;/li&gt;
   *       &lt;li&gt;BMW 503&lt;/li&gt;
   *       &lt;li&gt;BMW 507&lt;/li&gt;
   *       &lt;li&gt;BMW 3200&lt;/li&gt;
   *    &lt;/ul&gt;
   *
   * &lt;script&gt;
   * // Create ListBox
   * $('#carsList').ejListBox(); 	
   * &lt;/script&gt; 
   *@example
   * // Another way to render ListBox control, using its targetID.
   *	&lt;ul id="listboxsample"&gt;
   *    &lt;/ul&gt;
   *	   &lt;li&gt;BMW 501&lt;/li&gt;
   *       &lt;li&gt;BMW 502&lt;/li&gt;
   
   *    &lt;ul id="carsList"&gt;
   *       &lt;li&gt;Audi A4&lt;/li&gt;
   *       &lt;li&gt;Audi A5&lt;/li&gt;
   *       &lt;li&gt;Audi A6&lt;/li&gt;
   *       &lt;li&gt;Audi A7&lt;/li&gt;
   *       &lt;li&gt;Audi A8&lt;/li&gt;
   *       &lt;li&gt;BMW 503&lt;/li&gt;
   *       &lt;li&gt;BMW 507&lt;/li&gt;
   *       &lt;li&gt;BMW 3200&lt;/li&gt;
   *    &lt;/ul&gt;
   * &lt;script&gt;
    * // Create ListBox
    * $('#listboxsample').ejListBox({targetID: "carlist"}); 	
    * &lt;/script&gt; 
    */
    // ejlistBox is the plugin name 
    // "ej.ListBox" is "namespace.className" will hold functions and properties

	
    ej.widget("ejListBox", "ej.ListBox", {
        // widget element will be automatically set in this
        element: null,
        // user defined model will be automatically set in this
        model: null,
     	 validTags: ["ul"],
        _setFirst: false,
		//Root Css Class
         _rootCSS: "e-listbox",
        // default model
        defaults: {
		     /**		
            * Specifies the itemsCount   for ListBox.
            * @default 0
            * @type {number}
            * @example 
		 
			*    &lt;ul id="carsList"&gt;
			*       &lt;li&gt;Audi A4&lt;/li&gt;
			*       &lt;li&gt;Audi A5&lt;/li&gt;
			*       &lt;li&gt;Audi A6&lt;/li&gt;
			*       &lt;li&gt;Audi A7&lt;/li&gt;
			*       &lt;li&gt;Audi A8&lt;/li&gt;
			*	    &lt;li&gt;BMW 501&lt;/li&gt;
			*       &lt;li&gt;BMW 502&lt;/li&gt;
			*       &lt;li&gt;BMW 503&lt;/li&gt;
			*       &lt;li&gt;BMW 507&lt;/li&gt;
			*       &lt;li&gt;BMW 3200&lt;/li&gt;
			*    &lt;/ul&gt;
  
			* &lt;script&gt;
			* // To set itemsCount   API value during initialization  . 	
			* $('#carsList').ejListBox({itemsCount  : 2 }); 	
			* &lt;/script&gt; 
			
            * @memberof ejListBox
            * @instance
            */
            itemsCount: 0,
			 /**		
            * Specifies the data source of the ListBox. The data source contains the list of data for generating the List items.	
            * @default null
            * @type {data}
            * @example 
			*  &lt;ul id="countrylist"&gt;  &lt;/ul&gt;
			* &lt;script&gt;		
            * //To set dataSource API value during initialization  
            * 	$("#countrylist").ejListBox({ dataSource: window.countries });			 
            * &lt;/script&gt;
            * @memberof ejListBox
            * @instance
            */
            dataSource: null,
			  /**		
            * Specifies the query to retrieve the data from online server.	
            * @default null
            * @type {object}
            * @example 
			*  &lt;ul id="customerlist"&gt;  &lt;/ul&gt;
			* &lt;script&gt;		
            * //To set query API value during initialization  
            * var dataManger = ej.DataManager({       url: "http://mvc.syncfusion.com/Services/Northwnd.svc/"});
            * var queryString = ej.Query().from("Suppliers").select("Customers");
            * 	$("#customerlist").ejListBox({ dataSource: dataManger, query: queryString, fields: { text: "CustomerID" }});
            * &lt;/script&gt;
            * @memberof ejListBox
            * @instance
            */
            query: null,
			 /**		
            * Specifies mapping fields for the data items of the ListBox.	
            * @default null
            * @type {object}
            * @example 
			*  &lt;ul id="countrylist"&gt;  &lt;/ul&gt;
			* &lt;script&gt;		
            * //To set fields API value during initialization  
            * 	$("#countrylist").ejListBox({ dataSource: window.countriesField,   fields: { text: "name", value: "key" }});
             * &lt;/script&gt;
            * @memberof ejListBox
            * @instance
            */
            fields: { 
				/**		
                 * Defines id for the tag.
				 * @alias ejListBox#fields->id
				 * @type String
                 */
			id: null, 
				/**		
                 * Defines the text content for the tag.
				 * @alias ejListBox#fields->text
				 * @type String
                 */
			text: null,  
			/**		
                 * Defines the imageURL for the image location. 
				 * @alias ejListBox#fields->imageUrl
				 * @type String
                 */
			imageUrl: null,
			 /**		
                 * Defines the image attributes such as height, width, styles and so on.
				 * @alias ejListBox#fields->imageAttributes
				 * @type String
                 */	
			imageAttributes: null, 
			/**		
                 * Defines the sprite css for the image tag.
				 * @alias ejListBox#fields->spriteCssClass
				 * @type String
                 */
			spriteCssClass: null, 
			/**		
                 * Defines the html attributes such as id, class, styles for the item.
				 * @alias ejListBox#fields->htmlAttributes
				 * @type Object
                 */
			htmlAttributes: null, 
				/**		
                 * Defines the tooltip text to be displayed for the data list item.
				 * @alias ejListBox#fields->toolTipText
				 * @type Object
                 */
			toolTipText:null,
				/**		
                 * Defines the tag value to be selected initially
				 * @alias ejListBox#fields->selected
				 * @type Boolean
                 */
			selected: null,
			/**		
                 * Defines the category for data item.
				 * @alias ejListBox#fields->category
				 * @type String
                 */
			category: null,
			/**		
                 * Defines the table name for tag value or display text while render with remote data.
				 * @alias ejListBox#fields->tableName
				 * @type String
                 */
			tableName: null
			},
                  /**		
            * Defines the height of the ListBox.
            * @default Null
            * @type {string | number}
            * @example 
			   *    &lt;ul id="carsList"&gt;
   *       &lt;li&gt;Audi A4&lt;/li&gt;
   *       &lt;li&gt;Audi A5&lt;/li&gt;
   *       &lt;li&gt;Audi A6&lt;/li&gt;
   *       &lt;li&gt;Audi A7&lt;/li&gt;
   *       &lt;li&gt;Audi A8&lt;/li&gt;
   *	   &lt;li&gt;BMW 501&lt;/li&gt;
   *       &lt;li&gt;BMW 502&lt;/li&gt;
   *       &lt;li&gt;BMW 503&lt;/li&gt;
   *       &lt;li&gt;BMW 507&lt;/li&gt;
   *       &lt;li&gt;BMW 3200&lt;/li&gt;
   *    &lt;/ul&gt;
   *
   * &lt;script&gt;
     * //Initialize the ListBox height property with the  value specified
   * $('#carsList').ejListBox({ height: "300"}); 	
   * &lt;/script&gt; 
		  
            * @memberof ejListBox
            * @instance
            */      
                 
            height: "272",  
            /**		
            * Defines the width of the ListBox.
            * @default Null
            * @type {string | number}
            * @example 
			   *    &lt;ul id="carsList"&gt;
   *       &lt;li&gt;Audi A4&lt;/li&gt;
   *       &lt;li&gt;Audi A5&lt;/li&gt;
   *       &lt;li&gt;Audi A6&lt;/li&gt;
   *       &lt;li&gt;Audi A7&lt;/li&gt;
   *       &lt;li&gt;Audi A8&lt;/li&gt;
   *	   &lt;li&gt;BMW 501&lt;/li&gt;
   *       &lt;li&gt;BMW 502&lt;/li&gt;
   *       &lt;li&gt;BMW 503&lt;/li&gt;
   *       &lt;li&gt;BMW 507&lt;/li&gt;
   *       &lt;li&gt;BMW 3200&lt;/li&gt;
   *    &lt;/ul&gt;
   *
   * &lt;script&gt;
     * //Initialize the ListBox height property with the  value specified
   * $('#carsList').ejListBox({ width: "220"}); 	
   * &lt;/script&gt; 
		  
            * @memberof ejListBox
            * @instance
            */   			
            width: "200",   
 /**		
            * Specifies the template for ListBox.
            * @default null
            * @type {string}
            * @example <br/> 
			* &lt;input type="text" id="drpdwn" /&gt; <br> 
			* &lt;script&gt;		
            * // To set template API value during initialization  . 	
            * $("#drpdwn").ejListBox({ dataSource: window.drpdwnempList, template: '&lt;img class="eimg" src="styles/images/Employee/${eimg}.png" alt="employee" height="50px" width="50px"/&gt;' +
            *   '&lt;div class="ename"&gt; ${text} &lt;/div&gt;&lt;div class="desig"&gt; ${desig} &lt;/div&gt;&lt;div class="cont"&gt; ${country} &lt;/div&gt;'});
            * &lt;/script&gt;
            * @memberof ejListBox
            * @instance
            */			
            template: null,  
			text:"",
			            /**		
            * Specifies the selectedItemIndex   for ListBox.
            * @default null
            * @type {number}
            * @example 
		  *    &lt;ul id="carsList"&gt;
   *       &lt;li&gt;Audi A4&lt;/li&gt;
   *       &lt;li&gt;Audi A5&lt;/li&gt;
   *       &lt;li&gt;Audi A6&lt;/li&gt;
   *       &lt;li&gt;Audi A7&lt;/li&gt;
   *       &lt;li&gt;Audi A8&lt;/li&gt;
   *	   &lt;li&gt;BMW 501&lt;/li&gt;
   *       &lt;li&gt;BMW 502&lt;/li&gt;
   *       &lt;li&gt;BMW 503&lt;/li&gt;
   *       &lt;li&gt;BMW 507&lt;/li&gt;
   *       &lt;li&gt;BMW 3200&lt;/li&gt;
   *    &lt;/ul&gt;
   *
   * &lt;script&gt;
 * // To set selectedItemIndex   API value during initialization  . 	
   * $('#carsList').ejListBox({selectedItemIndex  : 2}); 	
   * &lt;/script&gt; 
            * @memberof ejListBox
            * @instance
            */
            selectedItemIndex:null,
			            /**		
            * Specifies the selectedItems  for ListBox.
	
            * @default []
            * @type {integerarray}
            * @example 
		 *    &lt;ul id="carsList"&gt;
   *       &lt;li&gt;Audi A4&lt;/li&gt;
   *       &lt;li&gt;Audi A5&lt;/li&gt;
   *       &lt;li&gt;Audi A6&lt;/li&gt;
   *       &lt;li&gt;Audi A7&lt;/li&gt;
   *       &lt;li&gt;Audi A8&lt;/li&gt;
   *	   &lt;li&gt;BMW 501&lt;/li&gt;
   *       &lt;li&gt;BMW 502&lt;/li&gt;
   *       &lt;li&gt;BMW 503&lt;/li&gt;
   *       &lt;li&gt;BMW 507&lt;/li&gt;
   *       &lt;li&gt;BMW 3200&lt;/li&gt;
   *    &lt;/ul&gt;
   *
   * &lt;script&gt;
 * // To set selectedItemIndex   API value during initialization  . 	
   * $('#carsList').ejListBox({ selectedItems  : [1,2] }); 	
   * &lt;/script&gt; 
            * @memberof ejListBox
            * @instance
            */
            selectedItems: [],
			            /**		
            * Specifies the selectedItems  for ListBox.
	
            * @default []
            * @type {integerarray}
            * @example 
		 *    &lt;ul id="carsList"&gt;
   *       &lt;li&gt;Audi A4&lt;/li&gt;
   *       &lt;li&gt;Audi A5&lt;/li&gt;
   *       &lt;li&gt;Audi A6&lt;/li&gt;
   *       &lt;li&gt;Audi A7&lt;/li&gt;
   *       &lt;li&gt;Audi A8&lt;/li&gt;
   *	   &lt;li&gt;BMW 501&lt;/li&gt;
   *       &lt;li&gt;BMW 502&lt;/li&gt;
   *       &lt;li&gt;BMW 503&lt;/li&gt;
   *       &lt;li&gt;BMW 507&lt;/li&gt;
   *       &lt;li&gt;BMW 3200&lt;/li&gt;
   *    &lt;/ul&gt;
   *
   * &lt;script&gt;
 * // To set selectedItemIndex   API value during initialization  . 	
   * $('#carsList').ejListBox({ showCheckbox:true,checkedItems  : [1,2] }); 	
   * &lt;/script&gt; 
            * @memberof ejListBox
            * @instance
            */
            checkedItems:[],
            checkedItemlist: [],
            selectedItemlist: [],
			 /**		
            * cascadeTo is used in cascading ListBox scenario, to map the child ListBox list widget in the parent ListBox list widget. By selecting an option in the parent ListBox, the child ListBox has to load the corresponding value regarding the parent ListBox. 
            * @default null
            * @type {string}
            * @example 
			* &lt;div style="float: left;"&gt;
			*       &lt;span class="txt"&gt;Select Group&lt;/span&gt;
			*   &lt;ul id="groupsList" /&gt; &lt;/ul&gt;
			* &lt;/div&gt;
			*
			* &lt;div style="float: right;"&gt;
			*        &lt;span class="txt"&gt;Select Country&lt;/span&gt;
			*        &lt;ul id="countryList" /&gt; &lt;/ul&gt;
			* &lt;/div&gt;
			* &lt;script&gt;
			*  var groups = [
			*          { parentId: 'a', text: "Group A" },
			*          { parentId: 'b', text: "Group B" },
			*          { parentId: 'c', text: "Group C" },
			*          { parentId: 'd', text: "Group D" },
			*          { parentId: 'e', text: "Group E" }]
			*            //first level child
			*            var countries = [{ value: 11, parentId: 'a', text: "Algeria", sprite: "flag-dz" },
			*           { value: 12, parentId: 'a', text: "Armenia", sprite: "flag-am" },
			*           { value: 13, parentId: 'a', text: "Bangladesh", sprite: "flag-bd" },
			*           { value: 14, parentId: 'a', text: "Cuba", sprite: "flag-cu" },
			*           { value: 15, parentId: 'b', text: "Denmark", sprite: "flag-dk" },
			*           { value: 16, parentId: 'b', text: "Egypt", sprite: "flag-eg" },
			*           { value: 17, parentId: 'c', text: "Finland", sprite: "flag-fi" },
			*           { value: 18, parentId: 'c', text: "India", sprite: "flag-in" },
			*           { value: 19, parentId: 'c', text: "Malaysia", sprite: "flag-my" },
			*           { value: 20, parentId: 'd', text: "New Zealand", sprite: "flag-nz" },
			*           { value: 21, parentId: 'd', text: "Norway", sprite: "flag-no" },
			*           { value: 22, parentId: 'd', text: "Poland", sprite: "flag-pl" },
			*           { value: 23, parentId: 'e', text: "Romania", sprite: "flag-ro" },
			*           { value: 24, parentId: 'e', text: "Singapore", sprite: "flag-sg" },
			*           { value: 25, parentId: 'e', text: "Thailand", sprite: "flag-th" },
			*           { value: 26, parentId: 'e', text: "Ukraine", sprite: "flag-ua" }]
			* // To set cascadeTo API value during initialization  . 
			*            $('#groupsList').ejListBox({
			*                dataSource: groups,
			*                fields: { value: "parentId" },
			*                cascadeTo: 'countryList'
			*            });
			*            $('#countryList').ejListBox({
			*                dataSource: countries,
			*                enabled:false
			*            });
            * &lt;/script&gt;
            * @memberof ejListBox
            * @instance
            */
            cascadeTo: null,
			
			value: "",
			  /**		
            * Sets the root class for ListBox theme. This cssClass API helps to use custom skinning option for ListBox control. By defining the root class using this API, we need to include this root class in CSS.
            * @default "gradient-lime"
            * @type {string}
            * @example 
			* &lt;input type="text" id="drpdwn" /&gt; <br> 
			*  &lt;div id="carsList"&gt;
			*    &lt;ul&gt;
			*       &lt;li&gt;Audi A4&lt;/li&gt;
			*       &lt;li&gt;Audi A5&lt;/li&gt;
			*       &lt;li&gt;Audi A6&lt;/li&gt;
			*       &lt;li&gt;Audi A7&lt;/li&gt;
			*       &lt;li&gt;Audi A8&lt;/li&gt;
			*    &lt;/ul&gt;
			*  &lt;/div&gt;<br/>
			* &lt;script&gt;
            * //Initialize the ListBox with the cssClass value specified
            * 	$("#drpdwn").ejListBox({ targetID: "carsList",cssClass: 'gradient-lime'});
            * &lt;/script&gt;
            * @memberof ejListBox
            * @instance
            */
			cssClass:"",
			     /**		
            * ListBox textbox to be displayed with rounded corner style.
            * @default true
            * @type {boolean}
            * @example 
			   *    &lt;ul id="carsList"&gt;
   *       &lt;li&gt;Audi A4&lt;/li&gt;
   *       &lt;li&gt;Audi A5&lt;/li&gt;
   *       &lt;li&gt;Audi A6&lt;/li&gt;
   *       &lt;li&gt;Audi A7&lt;/li&gt;
   *       &lt;li&gt;Audi A8&lt;/li&gt;
   *	   &lt;li&gt;BMW 501&lt;/li&gt;
   *       &lt;li&gt;BMW 502&lt;/li&gt;
   *       &lt;li&gt;BMW 503&lt;/li&gt;
   *       &lt;li&gt;BMW 507&lt;/li&gt;
   *       &lt;li&gt;BMW 3200&lt;/li&gt;
   *    &lt;/ul&gt;
   *
   * &lt;script&gt;
* // Initialize the ListBox with the showRoundedCorner value specified.
   * $('#carsList').ejListBox({ showRoundedCorner: true }); 	
   * &lt;/script&gt; 
            * @memberof ejListBox
            * @instance
            */
            showRoundedCorner: false,
			     /**		
            * Sets the ListBox textbox direction as right to left alignment.
            * @default false
            * @type {boolean}
            * @example 
			  *    &lt;ul id="carsList"&gt;
   *       &lt;li&gt;Audi A4&lt;/li&gt;
   *       &lt;li&gt;Audi A5&lt;/li&gt;
   *       &lt;li&gt;Audi A6&lt;/li&gt;
   *       &lt;li&gt;Audi A7&lt;/li&gt;
   *       &lt;li&gt;Audi A8&lt;/li&gt;
   *	   &lt;li&gt;BMW 501&lt;/li&gt;
   *       &lt;li&gt;BMW 502&lt;/li&gt;
   *       &lt;li&gt;BMW 503&lt;/li&gt;
   *       &lt;li&gt;BMW 507&lt;/li&gt;
   *       &lt;li&gt;BMW 3200&lt;/li&gt;
   *    &lt;/ul&gt;
   *
   * &lt;script&gt;
 * // Initialize the ListBox with the enableRTL  value specified.
   * $('#carsList').ejListBox({enableRTL : true  }); 	
   * &lt;/script&gt; 
            * @memberof ejListBox
            * @instance
            */
            enableRTL: false,
				     /**		
            * Sets to enable tooltip text for ListBox.
            * @default false
            * @type {boolean}
            * @example 
			  *    &lt;ul id="carsList"&gt;
   *       &lt;li&gt;Audi A4&lt;/li&gt;
   *       &lt;li&gt;Audi A5&lt;/li&gt;
   *       &lt;li&gt;Audi A6&lt;/li&gt;
   *       &lt;li&gt;Audi A7&lt;/li&gt;
   *       &lt;li&gt;Audi A8&lt;/li&gt;
   *	   &lt;li&gt;BMW 501&lt;/li&gt;
   *       &lt;li&gt;BMW 502&lt;/li&gt;
   *       &lt;li&gt;BMW 503&lt;/li&gt;
   *       &lt;li&gt;BMW 507&lt;/li&gt;
   *       &lt;li&gt;BMW 3200&lt;/li&gt;
   *    &lt;/ul&gt;
   *
   * &lt;script&gt;
 * // Initialize the ListBox with the enableRTL  value specified.
   * $('#carsList').ejListBox({enableTooltip : true  }); 	
   * &lt;/script&gt; 
            * @memberof ejListBox
            * @instance
            */
			enableTooltip: false,
			      /**		
            * When this property sets to false, it disables the ListBox control.
            * @default true
            * @type {boolean}
            * @example 
		 * 
   *    &lt;ul id="carsList"&gt;
   *       &lt;li&gt;Audi A4&lt;/li&gt;
   *       &lt;li&gt;Audi A5&lt;/li&gt;
   *       &lt;li&gt;Audi A6&lt;/li&gt;
   *       &lt;li&gt;Audi A7&lt;/li&gt;
   *       &lt;li&gt;Audi A8&lt;/li&gt;
   *	   &lt;li&gt;BMW 501&lt;/li&gt;
   *       &lt;li&gt;BMW 502&lt;/li&gt;
   *       &lt;li&gt;BMW 503&lt;/li&gt;
   *       &lt;li&gt;BMW 507&lt;/li&gt;
   *       &lt;li&gt;BMW 3200&lt;/li&gt;
   *    &lt;/ul&gt;
   *
   * &lt;script&gt;
 * // Initialize the ListBox with the enabled  value specified.
   * $('#carsList').ejListBox({enabled : false  }); 	
   * &lt;/script&gt; 
            * @memberof ejListBox
            * @instance
            */
            enabled: true,
			    /**		
            * Specifies the multi selection option in ListBox with the help of checkbox control. For this we have to enable it by showCheckbox option true.
            * @default false
            * @type {boolean}
            * @example 
			   *    &lt;ul id="carsList"&gt;
   *       &lt;li&gt;Audi A4&lt;/li&gt;
   *       &lt;li&gt;Audi A5&lt;/li&gt;
   *       &lt;li&gt;Audi A6&lt;/li&gt;
   *       &lt;li&gt;Audi A7&lt;/li&gt;
   *       &lt;li&gt;Audi A8&lt;/li&gt;
   *	   &lt;li&gt;BMW 501&lt;/li&gt;
   *       &lt;li&gt;BMW 502&lt;/li&gt;
   *       &lt;li&gt;BMW 503&lt;/li&gt;
   *       &lt;li&gt;BMW 507&lt;/li&gt;
   *       &lt;li&gt;BMW 3200&lt;/li&gt;
   *    &lt;/ul&gt;
   *
   * &lt;script&gt;
 * // Initialize the ListBox with the showCheckbox value specified.
   * $('#carsList').ejListBox({showCheckbox: true }); 	
   * &lt;/script&gt; 
            * @memberof ejListBox
            * @instance
            */
            showCheckbox: false,
					    /**		
            * Specifies the grouping option in ListBox. 
            * @default false
            * @type {boolean}
            * @example 
			   *    &lt;ul id="carsList"&gt;
   *       &lt;li&gt;Audi A4&lt;/li&gt;
   *       &lt;li&gt;Audi A5&lt;/li&gt;
   *       &lt;li&gt;Audi A6&lt;/li&gt;
   *       &lt;li&gt;Audi A7&lt;/li&gt;
   *       &lt;li&gt;Audi A8&lt;/li&gt;
   *	   &lt;li&gt;BMW 501&lt;/li&gt;
   *       &lt;li&gt;BMW 502&lt;/li&gt;
   *       &lt;li&gt;BMW 503&lt;/li&gt;
   *       &lt;li&gt;BMW 507&lt;/li&gt;
   *       &lt;li&gt;BMW 3200&lt;/li&gt;
   *    &lt;/ul&gt;
   *
   * &lt;script&gt;
 * // Initialize the ListBox with the allowGrouping value specified.
   * $('#carsList').ejListBox({allowGrouping: true }); 	
   * &lt;/script&gt; 
            * @memberof ejListBox
            * @instance
            */
            allowGrouping:false,
			     /**		
            * Specifies to select all the items of ListBox can be done with the help of this checkAll property, it supports only when the showCheckbox property true.
			* @remarks
			* If We would use the checkAll option,we will follow this:
            *  1.To enable the checkbox option as true{showCheckbox:true}See {@link ejListBox#showCheckbox}
            * @default false
            * @type {boolean}
            * @example 
			  *    &lt;ul id="carsList"&gt;
   *       &lt;li&gt;Audi A4&lt;/li&gt;
   *       &lt;li&gt;Audi A5&lt;/li&gt;
   *       &lt;li&gt;Audi A6&lt;/li&gt;
   *       &lt;li&gt;Audi A7&lt;/li&gt;
   *       &lt;li&gt;Audi A8&lt;/li&gt;
   *	   &lt;li&gt;BMW 501&lt;/li&gt;
   *       &lt;li&gt;BMW 502&lt;/li&gt;
   *       &lt;li&gt;BMW 503&lt;/li&gt;
   *       &lt;li&gt;BMW 507&lt;/li&gt;
   *       &lt;li&gt;BMW 3200&lt;/li&gt;
   *    &lt;/ul&gt;
   *
   * &lt;script&gt;
// Initialize the ListBox with the checkAll value specified.
   * $('#carsList').ejListBox({showCheckbox: true, checkAll: true  }); 	
   * &lt;/script&gt; 
            * @memberof ejListBox
            * @instance
            */
            checkAll: false,
			  /**		
            * Specifies to unselect all the items of ListBox can be done with the help of this checkAll property, it supports only when the showCheckbox property true.
			* @remarks
			* If We would use the uncheckAll option,we will follow this:
            *  1.To enable the checkbox option as true{showCheckbox:true}See {@link ejListBox#showCheckbox}
            * @default false
            * @type {boolean}
            * @example 
			*    &lt;ul id="carsList"&gt;
   *       &lt;li&gt;Audi A4&lt;/li&gt;
   *       &lt;li&gt;Audi A5&lt;/li&gt;
   *       &lt;li&gt;Audi A6&lt;/li&gt;
   *       &lt;li&gt;Audi A7&lt;/li&gt;
   *       &lt;li&gt;Audi A8&lt;/li&gt;
   *	   &lt;li&gt;BMW 501&lt;/li&gt;
   *       &lt;li&gt;BMW 502&lt;/li&gt;
   *       &lt;li&gt;BMW 503&lt;/li&gt;
   *       &lt;li&gt;BMW 507&lt;/li&gt;
   *       &lt;li&gt;BMW 3200&lt;/li&gt;
   *    &lt;/ul&gt;
   *
   * &lt;script&gt;
   * // Initialize the ListBox with the uncheckAll value specified.
   * $('#carsList').ejListBox({showCheckbox: true,  uncheckAll: true  }); 	
   * &lt;/script&gt; 

            * @memberof ejListBox
            * @instance
            */
            uncheckAll: false,
				 /**		
            * Specifies to enable Load data items onDemand for the ListBox.	
             * @default false
            * @type {boolean}
            * @example 
			*  &lt;ul id="countrylist"&gt;  &lt;/ul&gt;
			* &lt;script&gt;		
            * //To set fields API value during initialization  
            * 	$("#countrylist").ejListBox({ dataSource: window.countriesField, enableloadOnDemand:true,  fields: { text: "name", value: "key" }});
             * &lt;/script&gt;
            * @memberof ejListBox
            * @instance
            */
		
				
            enableloadOnDemand:false,
				 /**		
            * Specifies to enable virtual scrolling behavior along with Load data items onDemand for the ListBox.	
             * @default false
            * @type {boolean}
            * @example 
			*  &lt;ul id="countrylist"&gt;  &lt;/ul&gt;
			* &lt;script&gt;		
            * //To set fields API value during initialization  
            * 	$("#countrylist").ejListBox({ dataSource: window.countriesField, enableloadOnDemand:true,enablevirtualScrolling:true,  fields: { text: "name", value: "key" }});
             * &lt;/script&gt;
            * @memberof ejListBox
            * @instance
            */
			enablevirtualScrolling:false,
			/**		
            * Save current model value to browser cookies for state maintains. While refresh the ListBox control page retains the model value apply from browser cookies.  
            * @default false
            * @type {boolean}
            * @example 
			  *    &lt;ul id="carsList"&gt;
   *       &lt;li&gt;Audi A4&lt;/li&gt;
   *       &lt;li&gt;Audi A5&lt;/li&gt;
   *       &lt;li&gt;Audi A6&lt;/li&gt;
   *       &lt;li&gt;Audi A7&lt;/li&gt;
   *       &lt;li&gt;Audi A8&lt;/li&gt;
   *	   &lt;li&gt;BMW 501&lt;/li&gt;
   *       &lt;li&gt;BMW 502&lt;/li&gt;
   *       &lt;li&gt;BMW 503&lt;/li&gt;
   *       &lt;li&gt;BMW 507&lt;/li&gt;
   *       &lt;li&gt;BMW 3200&lt;/li&gt;
   *    &lt;/ul&gt;
   *
   * &lt;script&gt;
* // Initialize the ListBox with the enablePersistence  value specified.
   * $('#carsList').ejListBox({enablePersistence : false}); 	
   * &lt;/script&gt;
            * @memberof ejListBox
            * @instance
            */
            enablePersistence: false,
 /**		
            * To allow multiple selection of list items
			* @remarks
			* If We would use the uncheckAll option,we will follow this:
            *  1.To enable the checkbox option as true{showCheckbox:true}See {@link ejListBox#showCheckbox}
            * @default true
            * @type {boolean}
            * @example 
		  *    &lt;ul id="carsList"&gt;
   *       &lt;li&gt;Audi A4&lt;/li&gt;
   *       &lt;li&gt;Audi A5&lt;/li&gt;
   *       &lt;li&gt;Audi A6&lt;/li&gt;
   *       &lt;li&gt;Audi A7&lt;/li&gt;
   *       &lt;li&gt;Audi A8&lt;/li&gt;
   *	   &lt;li&gt;BMW 501&lt;/li&gt;
   *       &lt;li&gt;BMW 502&lt;/li&gt;
   *       &lt;li&gt;BMW 503&lt;/li&gt;
   *       &lt;li&gt;BMW 507&lt;/li&gt;
   *       &lt;li&gt;BMW 3200&lt;/li&gt;
   *    &lt;/ul&gt;
   *
   * &lt;script&gt;
 * // Initialize the allowMultiSelection with the value specified.
   * $('#carsList').ejListBox({allowMultiSelection: true}); 	
   * &lt;/script&gt; 
            * @memberof ejListBox
            * @instance
            */			
            allowMultiSelection: false,
			/**		
            * To enable the drag and drop nodes. 
			* @remarks
	
            * @default true
            * @type {boolean}
            * @example 
		   *    &lt;ul id="carsList"&gt;
   *       &lt;li&gt;Audi A4&lt;/li&gt;
   *       &lt;li&gt;Audi A5&lt;/li&gt;
   *       &lt;li&gt;Audi A6&lt;/li&gt;
   *       &lt;li&gt;Audi A7&lt;/li&gt;
   *       &lt;li&gt;Audi A8&lt;/li&gt;
   *	   &lt;li&gt;BMW 501&lt;/li&gt;
   *       &lt;li&gt;BMW 502&lt;/li&gt;
   *       &lt;li&gt;BMW 503&lt;/li&gt;
   *       &lt;li&gt;BMW 507&lt;/li&gt;
   *       &lt;li&gt;BMW 3200&lt;/li&gt;
   *    &lt;/ul&gt;
   *
   * &lt;script&gt;
 * // Initialize the with the allowDragAndDrop value specified.
   * $('#carsList').ejListBox({allowDragAndDrop: true}); 	
   * &lt;/script&gt; 
            * @memberof ejListBox
            * @instance
            */
				
            allowDragAndDrop: false,
			   /**     
           * Fires when create successfully.
           * @event
           * @name ejListBox#create 	
           * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
           * @param {object}  argument.model returns the ListBox model
           * @param {string}  argument.type returns the name of the event
           * @example 
		    *    &lt;ul id="carsList"&gt;
   *       &lt;li&gt;Audi A4&lt;/li&gt;
   *       &lt;li&gt;Audi A5&lt;/li&gt;
   *       &lt;li&gt;Audi A6&lt;/li&gt;
   *       &lt;li&gt;Audi A7&lt;/li&gt;
   *       &lt;li&gt;Audi A8&lt;/li&gt;
   *	   &lt;li&gt;BMW 501&lt;/li&gt;
   *       &lt;li&gt;BMW 502&lt;/li&gt;
   *       &lt;li&gt;BMW 503&lt;/li&gt;
   *       &lt;li&gt;BMW 507&lt;/li&gt;
   *       &lt;li&gt;BMW 3200&lt;/li&gt;
   *    &lt;/ul&gt;
   *
   * &lt;script&gt;
 * //create event for ListBox
   * $('#carsList').ejListBox({	create: function(args) {}}); 	
   * &lt;/script&gt; 	   
           * @memberof ejListBox
           * @instance
           */
            create: null,  
			   /**     
           * Fires when selected list item Index Changed successfully.
           * @event
           * @name ejListBox#create 	
           * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
           * @param {object}  argument.model returns the ListBox model
           * @param {string}  argument.type returns the name of the event
           * @example 
		    *    &lt;ul id="carsList"&gt;
   *       &lt;li&gt;Audi A4&lt;/li&gt;
   *       &lt;li&gt;Audi A5&lt;/li&gt;
   *       &lt;li&gt;Audi A6&lt;/li&gt;
   *       &lt;li&gt;Audi A7&lt;/li&gt;
   *       &lt;li&gt;Audi A8&lt;/li&gt;
   *	   &lt;li&gt;BMW 501&lt;/li&gt;
   *       &lt;li&gt;BMW 502&lt;/li&gt;
   *       &lt;li&gt;BMW 503&lt;/li&gt;
   *       &lt;li&gt;BMW 507&lt;/li&gt;
   *       &lt;li&gt;BMW 3200&lt;/li&gt;
   *    &lt;/ul&gt;
   *
   * &lt;script&gt;
 * //create event for ListBox
   * $('#carsList').ejListBox({	create: function(args) {}}); 	
   * &lt;/script&gt; 	   
           * @memberof ejListBox
           * @instance
           */
            selectIndexChanged: null,
			   /**     
           * Fires when selected list item gets selected successfully.
           * @event
           * @name ejListBox#selected 	

           * @example 
		    *    &lt;ul id="carsList"&gt;
   *       &lt;li&gt;Audi A4&lt;/li&gt;
   *       &lt;li&gt;Audi A5&lt;/li&gt;
   *       &lt;li&gt;Audi A6&lt;/li&gt;
   *       &lt;li&gt;Audi A7&lt;/li&gt;
   *       &lt;li&gt;Audi A8&lt;/li&gt;
   *	   &lt;li&gt;BMW 501&lt;/li&gt;
   *       &lt;li&gt;BMW 502&lt;/li&gt;
   *       &lt;li&gt;BMW 503&lt;/li&gt;
   *       &lt;li&gt;BMW 507&lt;/li&gt;
   *       &lt;li&gt;BMW 3200&lt;/li&gt;
   *    &lt;/ul&gt;
   *
   * &lt;script&gt;
 * //selected event for ListBox
   * $('#carsList').ejListBox({	selected: function(args) {}}); 	
   * &lt;/script&gt; 	   
           * @memberof ejListBox
           * @instance
           */
            selected: null,
			   /**     
           * Fires when  list item started to drag successfully.
           * @event
           * @name ejListBox#itemDragStart 	

           * @example 
		    *    &lt;ul id="carsList"&gt;
   *       &lt;li&gt;Audi A4&lt;/li&gt;
   *       &lt;li&gt;Audi A5&lt;/li&gt;
   *       &lt;li&gt;Audi A6&lt;/li&gt;
   *       &lt;li&gt;Audi A7&lt;/li&gt;
   *       &lt;li&gt;Audi A8&lt;/li&gt;
   *	   &lt;li&gt;BMW 501&lt;/li&gt;
   *       &lt;li&gt;BMW 502&lt;/li&gt;
   *       &lt;li&gt;BMW 503&lt;/li&gt;
   *       &lt;li&gt;BMW 507&lt;/li&gt;
   *       &lt;li&gt;BMW 3200&lt;/li&gt;
   *    &lt;/ul&gt;
   *
   * &lt;script&gt;
 * //itemDragStart event for ListBox
   * $('#carsList').ejListBox({	create: function(args) {}}); 	
   * &lt;/script&gt; 	   
           * @memberof ejListBox
           * @instance
           */
            itemDragStart: null,
			 /**     
           * Fires when  list item is being dragged successfully.
           * @event
           * @name ejListBox#itemDrag 	

           * @example 
		    *    &lt;ul id="carsList"&gt;
   *       &lt;li&gt;Audi A4&lt;/li&gt;
   *       &lt;li&gt;Audi A5&lt;/li&gt;
   *       &lt;li&gt;Audi A6&lt;/li&gt;
   *       &lt;li&gt;Audi A7&lt;/li&gt;
   *       &lt;li&gt;Audi A8&lt;/li&gt;
   *	   &lt;li&gt;BMW 501&lt;/li&gt;
   *       &lt;li&gt;BMW 502&lt;/li&gt;
   *       &lt;li&gt;BMW 503&lt;/li&gt;
   *       &lt;li&gt;BMW 507&lt;/li&gt;
   *       &lt;li&gt;BMW 3200&lt;/li&gt;
   *    &lt;/ul&gt;
   *
   * &lt;script&gt;
 * //itemDrag event for ListBox
   * $('#carsList').ejListBox({	itemDrag: function(args) {}}); 	
   * &lt;/script&gt; 	   
           * @memberof ejListBox
           * @instance
           */
            itemDrag: null,
			 /**     
           * Fires when  list item stopped dragging successfully.
           * @event
           * @name ejListBox#itemDragStop 	

           * @example 
		    *    &lt;ul id="carsList"&gt;
   *       &lt;li&gt;Audi A4&lt;/li&gt;
   *       &lt;li&gt;Audi A5&lt;/li&gt;
   *       &lt;li&gt;Audi A6&lt;/li&gt;
   *       &lt;li&gt;Audi A7&lt;/li&gt;
   *       &lt;li&gt;Audi A8&lt;/li&gt;
   *	   &lt;li&gt;BMW 501&lt;/li&gt;
   *       &lt;li&gt;BMW 502&lt;/li&gt;
   *       &lt;li&gt;BMW 503&lt;/li&gt;
   *       &lt;li&gt;BMW 507&lt;/li&gt;
   *       &lt;li&gt;BMW 3200&lt;/li&gt;
   *    &lt;/ul&gt;
   *
   * &lt;script&gt;
 * //itemDragStop event for ListBox
   * $('#carsList').ejListBox({	itemDragStop: function(args) {}}); 	
   * &lt;/script&gt; 	   
           * @memberof ejListBox
           * @instance
           */
            itemDragStop: null,
			 /**     
           * Fires when  list item stopped dragging successfully.
           * @event
           * @name ejListBox#itemDropped 	

           * @example 
		    *    &lt;ul id="carsList"&gt;
   *       &lt;li&gt;Audi A4&lt;/li&gt;
   *       &lt;li&gt;Audi A5&lt;/li&gt;
   *       &lt;li&gt;Audi A6&lt;/li&gt;
   *       &lt;li&gt;Audi A7&lt;/li&gt;
   *       &lt;li&gt;Audi A8&lt;/li&gt;
   *	   &lt;li&gt;BMW 501&lt;/li&gt;
   *       &lt;li&gt;BMW 502&lt;/li&gt;
   *       &lt;li&gt;BMW 503&lt;/li&gt;
   *       &lt;li&gt;BMW 507&lt;/li&gt;
   *       &lt;li&gt;BMW 3200&lt;/li&gt;
   *    &lt;/ul&gt;
   *
   * &lt;script&gt;
 * //itemDropped event for ListBox
   * $('#carsList').ejListBox({	itemDropped: function(args) {}}); 	
   * &lt;/script&gt; 	   
           * @memberof ejListBox
           * @instance
           */
            itemDropped:null,
			/**     
           * Fires when check state of an list item changed successfully.
           * @event
           * @name ejListBox#checkChange 	

           * @example 
		    *    &lt;ul id="carsList"&gt;
   *       &lt;li&gt;Audi A4&lt;/li&gt;
   *       &lt;li&gt;Audi A5&lt;/li&gt;
   *       &lt;li&gt;Audi A6&lt;/li&gt;
   *       &lt;li&gt;Audi A7&lt;/li&gt;
   *       &lt;li&gt;Audi A8&lt;/li&gt;
   *	   &lt;li&gt;BMW 501&lt;/li&gt;
   *       &lt;li&gt;BMW 502&lt;/li&gt;
   *       &lt;li&gt;BMW 503&lt;/li&gt;
   *       &lt;li&gt;BMW 507&lt;/li&gt;
   *       &lt;li&gt;BMW 3200&lt;/li&gt;
   *    &lt;/ul&gt;
   *
   * &lt;script&gt;
 * //checkChange event for ListBox
   * $('#carsList').ejListBox({	showCheckbox:true,checkChange: function(args) {}}); 	
   * &lt;/script&gt; 	   
           * @memberof ejListBox
           * @instance
           */
            checkChange: null,
			/**     
          * Fires when destroy successfully.
          * @event
          * @name ejListBox#destroy 	

          * @example 
		   *    &lt;ul id="carsList"&gt;
   *       &lt;li&gt;Audi A4&lt;/li&gt;
   *       &lt;li&gt;Audi A5&lt;/li&gt;
   *       &lt;li&gt;Audi A6&lt;/li&gt;
   *       &lt;li&gt;Audi A7&lt;/li&gt;
   *       &lt;li&gt;Audi A8&lt;/li&gt;
   *	   &lt;li&gt;BMW 501&lt;/li&gt;
   *       &lt;li&gt;BMW 502&lt;/li&gt;
   *       &lt;li&gt;BMW 503&lt;/li&gt;
   *       &lt;li&gt;BMW 507&lt;/li&gt;
   *       &lt;li&gt;BMW 3200&lt;/li&gt;
   *    &lt;/ul&gt;
   *
   * &lt;script&gt;
 * //create event for ListBox
   * $('#carsList').ejListBox({	destroy: function(args) {}}); 	
   * &lt;/script&gt; 	  
          * @memberof ejListBox
          * @instance
          */
            destroy: null
        },
        /**
         * Specify the data types for default properties 
		 * @private
         */
        dataTypes: {
            cssClass: "string",
            itemsCount: "number",
            allowDragAndDrop: "boolean",
            height: "string",
            width: "string",
            template: "string",
            selectedItemIndex: "number",
            cascadeTo: "string",
            showRoundedCorner: "boolean",
            enableRTL: "boolean",
			enableTooltip:"boolean",
            enablePersistence: "boolean",
            enabled: "boolean",
            allowMultiSelection: "boolean",
            dataSource: "data",
            selectedItems: "data",
            checkedItems: "data",
            checkedItemlist: "data",
            selectedItemlist: "data"
        },

        observables: ["value"],
        value: ej.util.valueFunction("value"),
       
        //public methods
		
		  /**
      * To enable the ListBox  
      * @return jQuery
      * @example 
         *    &lt;ul id="carsList"&gt;
   *       &lt;li&gt;Audi A4&lt;/li&gt;
   *       &lt;li&gt;Audi A5&lt;/li&gt;
   *       &lt;li&gt;Audi A6&lt;/li&gt;
   *       &lt;li&gt;Audi A7&lt;/li&gt;
   *       &lt;li&gt;Audi A8&lt;/li&gt;
   *	   &lt;li&gt;BMW 501&lt;/li&gt;
   *       &lt;li&gt;BMW 502&lt;/li&gt;
   *       &lt;li&gt;BMW 503&lt;/li&gt;
   *       &lt;li&gt;BMW 507&lt;/li&gt;
   *       &lt;li&gt;BMW 3200&lt;/li&gt;
   *    &lt;/ul&gt;
			* &lt;script&gt;
      * // Create ListBox
	   * $('#carsList').ejListBox(); 	
      * var ListBoxObj  = $("#drpdwn").data("ejListBox");
      * ListBoxObj.enable(); // enable the ListBox
      * &lt;/script&gt;
      * @example 
           *    &lt;ul id="carsList"&gt;
   *       &lt;li&gt;Audi A4&lt;/li&gt;
   *       &lt;li&gt;Audi A5&lt;/li&gt;
   *       &lt;li&gt;Audi A6&lt;/li&gt;
   *       &lt;li&gt;Audi A7&lt;/li&gt;
   *       &lt;li&gt;Audi A8&lt;/li&gt;
   *	   &lt;li&gt;BMW 501&lt;/li&gt;
   *       &lt;li&gt;BMW 502&lt;/li&gt;
   *       &lt;li&gt;BMW 503&lt;/li&gt;
   *       &lt;li&gt;BMW 507&lt;/li&gt;
   *       &lt;li&gt;BMW 3200&lt;/li&gt;
   *    &lt;/ul&gt;
			* &lt;script&gt;
      * // Create ListBox
	  * $('#drpdwn').ejListBox();
      * $('#drpdwn').ejListBox("enable"); 	
      * &lt;/script&gt;
      *@memberof ejListBox
      * @instance
      */
        enable: function () {
            if (this.listContainer.hasClass("e-disable")) {
                this.target.disabled = false;
                this.model.enabled = true;
                this.listContainer.removeClass('e-disable');
                if (this.model.allowMultiSelection) this.listContainer.removeClass("e-disable");
            }
        },
          /**
      * To disable the ListBox  
      * @return jQuery
      * @example 
         *    &lt;ul id="carsList"&gt;
   *       &lt;li&gt;Audi A4&lt;/li&gt;
   *       &lt;li&gt;Audi A5&lt;/li&gt;
   *       &lt;li&gt;Audi A6&lt;/li&gt;
   *       &lt;li&gt;Audi A7&lt;/li&gt;
   *       &lt;li&gt;Audi A8&lt;/li&gt;
   *	   &lt;li&gt;BMW 501&lt;/li&gt;
   *       &lt;li&gt;BMW 502&lt;/li&gt;
   *       &lt;li&gt;BMW 503&lt;/li&gt;
   *       &lt;li&gt;BMW 507&lt;/li&gt;
   *       &lt;li&gt;BMW 3200&lt;/li&gt;
   *    &lt;/ul&gt;
			* &lt;script&gt;
      * // Create ListBox
	   * $('#carsList').ejListBox(); 	
      * var ListBoxObj  = $("#drpdwn").data("ejListBox");
      * ListBoxObj.disable(); // disable the ListBox
      * &lt;/script&gt;
      * @example 
           *    &lt;ul id="carsList"&gt;
   *       &lt;li&gt;Audi A4&lt;/li&gt;
   *       &lt;li&gt;Audi A5&lt;/li&gt;
   *       &lt;li&gt;Audi A6&lt;/li&gt;
   *       &lt;li&gt;Audi A7&lt;/li&gt;
   *       &lt;li&gt;Audi A8&lt;/li&gt;
   *	   &lt;li&gt;BMW 501&lt;/li&gt;
   *       &lt;li&gt;BMW 502&lt;/li&gt;
   *       &lt;li&gt;BMW 503&lt;/li&gt;
   *       &lt;li&gt;BMW 507&lt;/li&gt;
   *       &lt;li&gt;BMW 3200&lt;/li&gt;
   *    &lt;/ul&gt;
			* &lt;script&gt;
      * // Create ListBox
	  * $('#drpdwn').ejListBox();
      * $('#drpdwn').ejListBox("disable"); 	
      * &lt;/script&gt;
      *@memberof ejListBox
      * @instance
      */
        disable: function () {
            if (!this.listContainer.hasClass("e-disable")) {
                this.target.disabled = true;
                this.model.enabled = false;
                this.listContainer.addClass('e-disable');
                if (this.model.allowMultiSelection) this.listContainer.addClass("e-disable");
            }
        },
   
        /**
     * To select an Item using its index in the ListBox  
     * @return jQuery
     * @example 
           *    &lt;ul id="carsList"&gt;
   *       &lt;li&gt;Audi A4&lt;/li&gt;
   *       &lt;li&gt;Audi A5&lt;/li&gt;
   *       &lt;li&gt;Audi A6&lt;/li&gt;
   *       &lt;li&gt;Audi A7&lt;/li&gt;
   *       &lt;li&gt;Audi A8&lt;/li&gt;
   *	   &lt;li&gt;BMW 501&lt;/li&gt;
   *       &lt;li&gt;BMW 502&lt;/li&gt;
   *       &lt;li&gt;BMW 503&lt;/li&gt;
   *       &lt;li&gt;BMW 507&lt;/li&gt;
   *       &lt;li&gt;BMW 3200&lt;/li&gt;
   *    &lt;/ul&gt;
			* &lt;script&gt;
     * // Create ListBox
	   * $('#carsList').ejListBox(); 	
     * var ListBoxObj  = $("#drpdwn").data("ejListBox");
     * ListBoxObj.selectItemByIndex(2); //selects the item in the ListBox
     * &lt;/script&gt;
     * @example 
       *    &lt;ul id="carsList"&gt;
   *       &lt;li&gt;Audi A4&lt;/li&gt;
   *       &lt;li&gt;Audi A5&lt;/li&gt;
   *       &lt;li&gt;Audi A6&lt;/li&gt;
   *       &lt;li&gt;Audi A7&lt;/li&gt;
   *       &lt;li&gt;Audi A8&lt;/li&gt;
   *	   &lt;li&gt;BMW 501&lt;/li&gt;
   *       &lt;li&gt;BMW 502&lt;/li&gt;
   *       &lt;li&gt;BMW 503&lt;/li&gt;
   *       &lt;li&gt;BMW 507&lt;/li&gt;
   *       &lt;li&gt;BMW 3200&lt;/li&gt;
   *    &lt;/ul&gt;
			* &lt;script&gt;
     * // Create ListBox
   * $('#carsList').ejListBox(); 	
     * $('#drpdwn').ejListBox("selectItemByIndex" ,"2"); 	
     * &lt;/script&gt;
     *@memberof ejListBox
     * @instance
     */
        selectItemByIndex: function (index) {
		index=parseInt(index)-1;
            if (!this.model.enabled) return false;
            if (index != null) {
                this._activeItem = index;
                if (this._activeItem <= 0) {
                    this._activeItem = 0;
                }
                else if (this._activeItem == this._listSize - 1) {
                    this._activeItem = this._listSize - 1;
                }
                    var activeitem = $(this.element.children("li")[this._activeItem]);
                    if (this._activeItem != this._aselectedItem) {
                        this.element.children("li").removeClass('e-hover');
                        activeitem.addClass('e-select');
                } 
            }
        },
		   /**
     * To select all the list items in the ListBox  
     * @return jQuery
     * @example 
           *    &lt;ul id="carsList"&gt;
   *       &lt;li&gt;Audi A4&lt;/li&gt;
   *       &lt;li&gt;Audi A5&lt;/li&gt;
   *       &lt;li&gt;Audi A6&lt;/li&gt;
   *       &lt;li&gt;Audi A7&lt;/li&gt;
   *       &lt;li&gt;Audi A8&lt;/li&gt;
   *	   &lt;li&gt;BMW 501&lt;/li&gt;
   *       &lt;li&gt;BMW 502&lt;/li&gt;
   *       &lt;li&gt;BMW 503&lt;/li&gt;
   *       &lt;li&gt;BMW 507&lt;/li&gt;
   *       &lt;li&gt;BMW 3200&lt;/li&gt;
   *    &lt;/ul&gt;
			* &lt;script&gt;
     * // Create ListBox
	   * $('#carsList').ejListBox(); 	
     * var ListBoxObj  = $("#drpdwn").data("ejListBox");
     * ListBoxObj.selectAll(); // selects all the list items in the ListBox
     * &lt;/script&gt;
     * @example 
       *    &lt;ul id="carsList"&gt;
   *       &lt;li&gt;Audi A4&lt;/li&gt;
   *       &lt;li&gt;Audi A5&lt;/li&gt;
   *       &lt;li&gt;Audi A6&lt;/li&gt;
   *       &lt;li&gt;Audi A7&lt;/li&gt;
   *       &lt;li&gt;Audi A8&lt;/li&gt;
   *	   &lt;li&gt;BMW 501&lt;/li&gt;
   *       &lt;li&gt;BMW 502&lt;/li&gt;
   *       &lt;li&gt;BMW 503&lt;/li&gt;
   *       &lt;li&gt;BMW 507&lt;/li&gt;
   *       &lt;li&gt;BMW 3200&lt;/li&gt;
   *    &lt;/ul&gt;
			* &lt;script&gt;
     * // Create ListBox
   * $('#carsList').ejListBox(); 	
     * $('#drpdwn').ejListBox("selectAll"); 	
     * &lt;/script&gt;
     *@memberof ejListBox
     * @instance
     */
		selectAll: function(){
		this.element.children("li").addClass("e-hover");
		this.element.children("li").addClass("e-select");
		},
			   /**
     * To unselect all the list items in the ListBox  
     * @return jQuery
     * @example 
           *    &lt;ul id="carsList"&gt;
   *       &lt;li&gt;Audi A4&lt;/li&gt;
   *       &lt;li&gt;Audi A5&lt;/li&gt;
   *       &lt;li&gt;Audi A6&lt;/li&gt;
   *       &lt;li&gt;Audi A7&lt;/li&gt;
   *       &lt;li&gt;Audi A8&lt;/li&gt;
   *	   &lt;li&gt;BMW 501&lt;/li&gt;
   *       &lt;li&gt;BMW 502&lt;/li&gt;
   *       &lt;li&gt;BMW 503&lt;/li&gt;
   *       &lt;li&gt;BMW 507&lt;/li&gt;
   *       &lt;li&gt;BMW 3200&lt;/li&gt;
   *    &lt;/ul&gt;
			* &lt;script&gt;
     * // Create ListBox
	   * $('#carsList').ejListBox(); 	
     * var ListBoxObj  = $("#drpdwn").data("ejListBox");
     * ListBoxObj.unSelectAll(); // disable the ListBox
     * &lt;/script&gt;
     * @example 
       *    &lt;ul id="carsList"&gt;
   *       &lt;li&gt;Audi A4&lt;/li&gt;
   *       &lt;li&gt;Audi A5&lt;/li&gt;
   *       &lt;li&gt;Audi A6&lt;/li&gt;
   *       &lt;li&gt;Audi A7&lt;/li&gt;
   *       &lt;li&gt;Audi A8&lt;/li&gt;
   *	   &lt;li&gt;BMW 501&lt;/li&gt;
   *       &lt;li&gt;BMW 502&lt;/li&gt;
   *       &lt;li&gt;BMW 503&lt;/li&gt;
   *       &lt;li&gt;BMW 507&lt;/li&gt;
   *       &lt;li&gt;BMW 3200&lt;/li&gt;
   *    &lt;/ul&gt;
			* &lt;script&gt;
     * // Create ListBox
   * $('#carsList').ejListBox(); 	
     * $('#drpdwn').ejListBox("unSelectAll"); 	
     * &lt;/script&gt;
     *@memberof ejListBox
     * @instance
     */
		unSelectAll: function(){
		this._removeListHover();
		},
			   /**
     * To select set of list items in the ListBox  
     * @return jQuery
     * @example 
           *    &lt;ul id="carsList"&gt;
   *       &lt;li&gt;Audi A4&lt;/li&gt;
   *       &lt;li&gt;Audi A5&lt;/li&gt;
   *       &lt;li&gt;Audi A6&lt;/li&gt;
   *       &lt;li&gt;Audi A7&lt;/li&gt;
   *       &lt;li&gt;Audi A8&lt;/li&gt;
   *	   &lt;li&gt;BMW 501&lt;/li&gt;
   *       &lt;li&gt;BMW 502&lt;/li&gt;
   *       &lt;li&gt;BMW 503&lt;/li&gt;
   *       &lt;li&gt;BMW 507&lt;/li&gt;
   *       &lt;li&gt;BMW 3200&lt;/li&gt;
   *    &lt;/ul&gt;
			* &lt;script&gt;
     * // Create ListBox
	   * $('#carsList').ejListBox(); 	
     * var ListBoxObj  = $("#drpdwn").data("ejListBox");
     * ListBoxObj.selectItemsByIndex(2,3); // selects the specified items the ListBox
     * &lt;/script&gt;
     * @example 
       *    &lt;ul id="carsList"&gt;
   *       &lt;li&gt;Audi A4&lt;/li&gt;
   *       &lt;li&gt;Audi A5&lt;/li&gt;
   *       &lt;li&gt;Audi A6&lt;/li&gt;
   *       &lt;li&gt;Audi A7&lt;/li&gt;
   *       &lt;li&gt;Audi A8&lt;/li&gt;
   *	   &lt;li&gt;BMW 501&lt;/li&gt;
   *       &lt;li&gt;BMW 502&lt;/li&gt;
   *       &lt;li&gt;BMW 503&lt;/li&gt;
   *       &lt;li&gt;BMW 507&lt;/li&gt;
   *       &lt;li&gt;BMW 3200&lt;/li&gt;
   *    &lt;/ul&gt;
			* &lt;script&gt;
     * // Create ListBox
   * $('#carsList').ejListBox(); 	
     * $('#drpdwn').ejListBox("selectItemsByIndex","2,3"); 	
     * &lt;/script&gt;
     *@memberof ejListBox
     * @instance
     */
	    selectItemsByIndex: function (value) {
            if (!this.model.enabled) return false;
			var selectitems = value.split(',');
			for(var i=0;i<selectitems.length; i++){
            if (selectitems[i] != null) {
				 var index=parseInt(selectitems[i])-1;
                this._activeItem = index;
                if (this._activeItem < 0) {
                    this._activeItem = 0;
                }
                else if (this._activeItem == this._listSize - 1) {
                    this._activeItem = this._listSize - 1;
                }
                    var activeitem = $(this.element.children("li")[this._activeItem]);
                    if (this._activeItem != this._aselectedItem) {
                        this.element.children("li").removeClass('e-hover');
                        activeitem.addClass('e-select');
                } 
            }
			}
        },
			   /**
     * To unselect set of list items in the ListBox  
     * @return jQuery
     * @example 
           *    &lt;ul id="carsList"&gt;
   *       &lt;li&gt;Audi A4&lt;/li&gt;
   *       &lt;li&gt;Audi A5&lt;/li&gt;
   *       &lt;li&gt;Audi A6&lt;/li&gt;
   *       &lt;li&gt;Audi A7&lt;/li&gt;
   *       &lt;li&gt;Audi A8&lt;/li&gt;
   *	   &lt;li&gt;BMW 501&lt;/li&gt;
   *       &lt;li&gt;BMW 502&lt;/li&gt;
   *       &lt;li&gt;BMW 503&lt;/li&gt;
   *       &lt;li&gt;BMW 507&lt;/li&gt;
   *       &lt;li&gt;BMW 3200&lt;/li&gt;
   *    &lt;/ul&gt;
			* &lt;script&gt;
     * // Create ListBox
	   * $('#carsList').ejListBox(); 	
     * var ListBoxObj  = $("#drpdwn").data("ejListBox");
     * ListBoxObj.unselectItemsByIndex(2,3); // .unselectItems specified the ListBox
     * &lt;/script&gt;
     * @example 
       *    &lt;ul id="carsList"&gt;
   *       &lt;li&gt;Audi A4&lt;/li&gt;
   *       &lt;li&gt;Audi A5&lt;/li&gt;
   *       &lt;li&gt;Audi A6&lt;/li&gt;
   *       &lt;li&gt;Audi A7&lt;/li&gt;
   *       &lt;li&gt;Audi A8&lt;/li&gt;
   *	   &lt;li&gt;BMW 501&lt;/li&gt;
   *       &lt;li&gt;BMW 502&lt;/li&gt;
   *       &lt;li&gt;BMW 503&lt;/li&gt;
   *       &lt;li&gt;BMW 507&lt;/li&gt;
   *       &lt;li&gt;BMW 3200&lt;/li&gt;
   *    &lt;/ul&gt;
			* &lt;script&gt;
     * // Create ListBox
   * $('#carsList').ejListBox(); 	
     * $('#drpdwn').ejListBox("unselectItemsByIndex","2,3"); 	
     * &lt;/script&gt;
     *@memberof ejListBox
     * @instance
     */
		unselectItemsByIndex: function (value) {
            if (!this.model.enabled) return false;
			var selectitems = value.split(',');
			for(var i=0;i<selectitems.length; i++){
            if (selectitems[i] != null) {
			 var index=parseInt(selectitems[i])-1;
				var activeitem = $(this.element.children("li")[index]);
                this._activeItem = index;
				activeitem.removeClass('e-active');
				activeitem.removeClass('e-select');
            }
			}
        },
			   /**
     * To unselect a list item in the ListBox  
     * @return jQuery
     * @example 
           *    &lt;ul id="carsList"&gt;
   *       &lt;li&gt;Audi A4&lt;/li&gt;
   *       &lt;li&gt;Audi A5&lt;/li&gt;
   *       &lt;li&gt;Audi A6&lt;/li&gt;
   *       &lt;li&gt;Audi A7&lt;/li&gt;
   *       &lt;li&gt;Audi A8&lt;/li&gt;
   *	   &lt;li&gt;BMW 501&lt;/li&gt;
   *       &lt;li&gt;BMW 502&lt;/li&gt;
   *       &lt;li&gt;BMW 503&lt;/li&gt;
   *       &lt;li&gt;BMW 507&lt;/li&gt;
   *       &lt;li&gt;BMW 3200&lt;/li&gt;
   *    &lt;/ul&gt;
			* &lt;script&gt;
     * // Create ListBox
	   * $('#carsList').ejListBox(); 	
     * var ListBoxObj  = $("#drpdwn").data("ejListBox");
     * ListBoxObj.unselectItemByIndex(2); // unselects the list item specified in the index  in the ListBox
     * &lt;/script&gt;
     * @example 
       *    &lt;ul id="carsList"&gt;
   *       &lt;li&gt;Audi A4&lt;/li&gt;
   *       &lt;li&gt;Audi A5&lt;/li&gt;
   *       &lt;li&gt;Audi A6&lt;/li&gt;
   *       &lt;li&gt;Audi A7&lt;/li&gt;
   *       &lt;li&gt;Audi A8&lt;/li&gt;
   *	   &lt;li&gt;BMW 501&lt;/li&gt;
   *       &lt;li&gt;BMW 502&lt;/li&gt;
   *       &lt;li&gt;BMW 503&lt;/li&gt;
   *       &lt;li&gt;BMW 507&lt;/li&gt;
   *       &lt;li&gt;BMW 3200&lt;/li&gt;
   *    &lt;/ul&gt;
			* &lt;script&gt;
     * // Create ListBox
   * $('#carsList').ejListBox(); 	
     * $('#drpdwn').ejListBox("unselectItemByIndex","2"); 	
     * &lt;/script&gt;
     *@memberof ejListBox
     * @instance
     */
        unselectItemByIndex: function (index) {
		index=parseInt(index)-1;
            if (!this.model.enabled) return false;
			var unselectitem = $(this.element.children("li")[index]);
            if(unselectitem.hasClass('e-select')){
				unselectitem.removeClass('e-active');
				unselectitem.removeClass('e-select');
            }
        },
		   /**
     * To get the selected list item in the ListBox  
     * @return jQuery
     * @example 
           *    &lt;ul id="carsList"&gt;
   *       &lt;li&gt;Audi A4&lt;/li&gt;
   *       &lt;li&gt;Audi A5&lt;/li&gt;
   *       &lt;li&gt;Audi A6&lt;/li&gt;
   *       &lt;li&gt;Audi A7&lt;/li&gt;
   *       &lt;li&gt;Audi A8&lt;/li&gt;
   *	   &lt;li&gt;BMW 501&lt;/li&gt;
   *       &lt;li&gt;BMW 502&lt;/li&gt;
   *       &lt;li&gt;BMW 503&lt;/li&gt;
   *       &lt;li&gt;BMW 507&lt;/li&gt;
   *       &lt;li&gt;BMW 3200&lt;/li&gt;
   *    &lt;/ul&gt;
			* &lt;script&gt;
     * // Create ListBox
	   * $('#carsList').ejListBox(); 	
     * var ListBoxObj  = $("#drpdwn").data("ejListBox");
     * ListBoxObj.getSelectedItem(); // disable the ListBox
     * &lt;/script&gt;
     * @example 
       *    &lt;ul id="carsList"&gt;
   *       &lt;li&gt;Audi A4&lt;/li&gt;
   *       &lt;li&gt;Audi A5&lt;/li&gt;
   *       &lt;li&gt;Audi A6&lt;/li&gt;
   *       &lt;li&gt;Audi A7&lt;/li&gt;
   *       &lt;li&gt;Audi A8&lt;/li&gt;
   *	   &lt;li&gt;BMW 501&lt;/li&gt;
   *       &lt;li&gt;BMW 502&lt;/li&gt;
   *       &lt;li&gt;BMW 503&lt;/li&gt;
   *       &lt;li&gt;BMW 507&lt;/li&gt;
   *       &lt;li&gt;BMW 3200&lt;/li&gt;
   *    &lt;/ul&gt;
			* &lt;script&gt;
     * // Create ListBox
   * $('#carsList').ejListBox(); 	
     * $('#drpdwn').ejListBox("getSelectedItem"); 	
     * &lt;/script&gt;
     *@memberof ejListBox
     * @instance
     */
        getSelectedItem: function () {
            return this.model._aselectedItem;
        },
			   /**
     * To get the list of selected list items in the ListBox  
     * @return jQuery
     * @example 
           *    &lt;ul id="carsList"&gt;
   *       &lt;li&gt;Audi A4&lt;/li&gt;
   *       &lt;li&gt;Audi A5&lt;/li&gt;
   *       &lt;li&gt;Audi A6&lt;/li&gt;
   *       &lt;li&gt;Audi A7&lt;/li&gt;
   *       &lt;li&gt;Audi A8&lt;/li&gt;
   *	   &lt;li&gt;BMW 501&lt;/li&gt;
   *       &lt;li&gt;BMW 502&lt;/li&gt;
   *       &lt;li&gt;BMW 503&lt;/li&gt;
   *       &lt;li&gt;BMW 507&lt;/li&gt;
   *       &lt;li&gt;BMW 3200&lt;/li&gt;
   *    &lt;/ul&gt;
			* &lt;script&gt;
     * // Create ListBox
	   * $('#carsList').ejListBox(); 	
     * var ListBoxObj  = $("#drpdwn").data("ejListBox");
     * ListBoxObj.getSelectedItems(); // disable the ListBox
     * &lt;/script&gt;
     * @example 
       *    &lt;ul id="carsList"&gt;
   *       &lt;li&gt;Audi A4&lt;/li&gt;
   *       &lt;li&gt;Audi A5&lt;/li&gt;
   *       &lt;li&gt;Audi A6&lt;/li&gt;
   *       &lt;li&gt;Audi A7&lt;/li&gt;
   *       &lt;li&gt;Audi A8&lt;/li&gt;
   *	   &lt;li&gt;BMW 501&lt;/li&gt;
   *       &lt;li&gt;BMW 502&lt;/li&gt;
   *       &lt;li&gt;BMW 503&lt;/li&gt;
   *       &lt;li&gt;BMW 507&lt;/li&gt;
   *       &lt;li&gt;BMW 3200&lt;/li&gt;
   *    &lt;/ul&gt;
			* &lt;script&gt;
     * // Create ListBox
   * $('#carsList').ejListBox(); 	
     * $('#drpdwn').ejListBox("getSelectedItems"); 	
     * &lt;/script&gt;
     *@memberof ejListBox
     * @instance
     */
        getSelectedItems: function () {
            return this.model.selectedItems;
        },
			   /**
     * To get the list of checked list items in the ListBox  
     * @return jQuery
     * @example 
           *    &lt;ul id="carsList"&gt;
   *       &lt;li&gt;Audi A4&lt;/li&gt;
   *       &lt;li&gt;Audi A5&lt;/li&gt;
   *       &lt;li&gt;Audi A6&lt;/li&gt;
   *       &lt;li&gt;Audi A7&lt;/li&gt;
   *       &lt;li&gt;Audi A8&lt;/li&gt;
   *	   &lt;li&gt;BMW 501&lt;/li&gt;
   *       &lt;li&gt;BMW 502&lt;/li&gt;
   *       &lt;li&gt;BMW 503&lt;/li&gt;
   *       &lt;li&gt;BMW 507&lt;/li&gt;
   *       &lt;li&gt;BMW 3200&lt;/li&gt;
   *    &lt;/ul&gt;
			* &lt;script&gt;
     * // Create ListBox
	   * $('#carsList').ejListBox(); 	
     * var ListBoxObj  = $("#drpdwn").data("ejListBox");
     * ListBoxObj.getCheckedItems(); // disable the ListBox
     * &lt;/script&gt;
     * @example 
       *    &lt;ul id="carsList"&gt;
   *       &lt;li&gt;Audi A4&lt;/li&gt;
   *       &lt;li&gt;Audi A5&lt;/li&gt;
   *       &lt;li&gt;Audi A6&lt;/li&gt;
   *       &lt;li&gt;Audi A7&lt;/li&gt;
   *       &lt;li&gt;Audi A8&lt;/li&gt;
   *	   &lt;li&gt;BMW 501&lt;/li&gt;
   *       &lt;li&gt;BMW 502&lt;/li&gt;
   *       &lt;li&gt;BMW 503&lt;/li&gt;
   *       &lt;li&gt;BMW 507&lt;/li&gt;
   *       &lt;li&gt;BMW 3200&lt;/li&gt;
   *    &lt;/ul&gt;
			* &lt;script&gt;
     * // Create ListBox
   * $('#carsList').ejListBox(); 	
     * $('#drpdwn').ejListBox("getCheckedItems"); 	
     * &lt;/script&gt;
     *@memberof ejListBox
     * @instance
     */
        getCheckedItems: function () {
            return this.model.checkedItems;
        },
				   /**
     * To remove an list item in the ListBox  
     * @return jQuery
     * @example 
           *    &lt;ul id="carsList"&gt;
   *       &lt;li&gt;Audi A4&lt;/li&gt;
   *       &lt;li&gt;Audi A5&lt;/li&gt;
   *       &lt;li&gt;Audi A6&lt;/li&gt;
   *       &lt;li&gt;Audi A7&lt;/li&gt;
   *       &lt;li&gt;Audi A8&lt;/li&gt;
   *	   &lt;li&gt;BMW 501&lt;/li&gt;
   *       &lt;li&gt;BMW 502&lt;/li&gt;
   *       &lt;li&gt;BMW 503&lt;/li&gt;
   *       &lt;li&gt;BMW 507&lt;/li&gt;
   *       &lt;li&gt;BMW 3200&lt;/li&gt;
   *    &lt;/ul&gt;
			* &lt;script&gt;
     * // Create ListBox
	   * $('#carsList').ejListBox(); 	
     * var ListBoxObj  = $("#drpdwn").data("ejListBox");
     * ListBoxObj.removeItem(); // removes the selected item from the ListBox
     * &lt;/script&gt;
     * @example 
       *    &lt;ul id="carsList"&gt;
   *       &lt;li&gt;Audi A4&lt;/li&gt;
   *       &lt;li&gt;Audi A5&lt;/li&gt;
   *       &lt;li&gt;Audi A6&lt;/li&gt;
   *       &lt;li&gt;Audi A7&lt;/li&gt;
   *       &lt;li&gt;Audi A8&lt;/li&gt;
   *	   &lt;li&gt;BMW 501&lt;/li&gt;
   *       &lt;li&gt;BMW 502&lt;/li&gt;
   *       &lt;li&gt;BMW 503&lt;/li&gt;
   *       &lt;li&gt;BMW 507&lt;/li&gt;
   *       &lt;li&gt;BMW 3200&lt;/li&gt;
   *    &lt;/ul&gt;
			* &lt;script&gt;
     * // Create ListBox
   * $('#carsList').ejListBox(); 	
     * $('#drpdwn').ejListBox("removeItem"); 	
     * &lt;/script&gt;
     *@memberof ejListBox
     * @instance
     */
		 removeItem: function () {
			this._getItem(this._selectedItemindex()).remove();
            this._refreshItems();
        },
				   /**
     * To disable an list item in the ListBox  
     * @return jQuery
     * @example 
           *    &lt;ul id="carsList"&gt;
   *       &lt;li&gt;Audi A4&lt;/li&gt;
   *       &lt;li&gt;Audi A5&lt;/li&gt;
   *       &lt;li&gt;Audi A6&lt;/li&gt;
   *       &lt;li&gt;Audi A7&lt;/li&gt;
   *       &lt;li&gt;Audi A8&lt;/li&gt;
   *	   &lt;li&gt;BMW 501&lt;/li&gt;
   *       &lt;li&gt;BMW 502&lt;/li&gt;
   *       &lt;li&gt;BMW 503&lt;/li&gt;
   *       &lt;li&gt;BMW 507&lt;/li&gt;
   *       &lt;li&gt;BMW 3200&lt;/li&gt;
   *    &lt;/ul&gt;
			* &lt;script&gt;
     * // Create ListBox
	   * $('#carsList').ejListBox(); 	
     * var ListBoxObj  = $("#drpdwn").data("ejListBox");
     * ListBoxObj.disableItem(); // disable selected item in the ListBox
     * &lt;/script&gt;
     * @example 
       *    &lt;ul id="carsList"&gt;
   *       &lt;li&gt;Audi A4&lt;/li&gt;
   *       &lt;li&gt;Audi A5&lt;/li&gt;
   *       &lt;li&gt;Audi A6&lt;/li&gt;
   *       &lt;li&gt;Audi A7&lt;/li&gt;
   *       &lt;li&gt;Audi A8&lt;/li&gt;
   *	   &lt;li&gt;BMW 501&lt;/li&gt;
   *       &lt;li&gt;BMW 502&lt;/li&gt;
   *       &lt;li&gt;BMW 503&lt;/li&gt;
   *       &lt;li&gt;BMW 507&lt;/li&gt;
   *       &lt;li&gt;BMW 3200&lt;/li&gt;
   *    &lt;/ul&gt;
			* &lt;script&gt;
     * // Create ListBox
   * $('#carsList').ejListBox(); 	
     * $('#drpdwn').ejListBox("disableItem"); 	
     * &lt;/script&gt;
     *@memberof ejListBox
     * @instance
     */
        disableItem: function () {
            this._getItem(this._selectedItemindex()).addClass("e-disable");
        },
		   /**
     * To move the selected list item one step Up in the ListBox  
     * @return jQuery
     * @example 
           *    &lt;ul id="carsList"&gt;
   *       &lt;li&gt;Audi A4&lt;/li&gt;
   *       &lt;li&gt;Audi A5&lt;/li&gt;
   *       &lt;li&gt;Audi A6&lt;/li&gt;
   *       &lt;li&gt;Audi A7&lt;/li&gt;
   *       &lt;li&gt;Audi A8&lt;/li&gt;
   *	   &lt;li&gt;BMW 501&lt;/li&gt;
   *       &lt;li&gt;BMW 502&lt;/li&gt;
   *       &lt;li&gt;BMW 503&lt;/li&gt;
   *       &lt;li&gt;BMW 507&lt;/li&gt;
   *       &lt;li&gt;BMW 3200&lt;/li&gt;
   *    &lt;/ul&gt;
			* &lt;script&gt;
     * // Create ListBox
	   * $('#carsList').ejListBox(); 	
     * var ListBoxObj  = $("#drpdwn").data("ejListBox");
     * ListBoxObj.moveUp(); // moves up selected item in the ListBox
     * &lt;/script&gt;
     * @example 
       *    &lt;ul id="carsList"&gt;
   *       &lt;li&gt;Audi A4&lt;/li&gt;
   *       &lt;li&gt;Audi A5&lt;/li&gt;
   *       &lt;li&gt;Audi A6&lt;/li&gt;
   *       &lt;li&gt;Audi A7&lt;/li&gt;
   *       &lt;li&gt;Audi A8&lt;/li&gt;
   *	   &lt;li&gt;BMW 501&lt;/li&gt;
   *       &lt;li&gt;BMW 502&lt;/li&gt;
   *       &lt;li&gt;BMW 503&lt;/li&gt;
   *       &lt;li&gt;BMW 507&lt;/li&gt;
   *       &lt;li&gt;BMW 3200&lt;/li&gt;
   *    &lt;/ul&gt;
			* &lt;script&gt;
     * // Create ListBox
   * $('#carsList').ejListBox(); 	
     * $('#drpdwn').ejListBox("moveUp"); 	
     * &lt;/script&gt;
     *@memberof ejListBox
     * @instance
     */
        moveUp: function () {
            var PreviousItem = this._getItem(this._selectedItemindex()).prev();
            var selectedItem = this._getItem(this._selectedItemindex());
            var previndex = this._selectedItemindex()-1;
            var selectedindex = this._selectedItemindex();
            PreviousItem.insertAfter(selectedItem);
			this._refreshContainer();
			this._trigger('indexchange', {  data: this.model });
        },
		/**
     * To move the selected list item one step Down in the ListBox  
     * @return jQuery
     * @example 
           *    &lt;ul id="carsList"&gt;
   *       &lt;li&gt;Audi A4&lt;/li&gt;
   *       &lt;li&gt;Audi A5&lt;/li&gt;
   *       &lt;li&gt;Audi A6&lt;/li&gt;
   *       &lt;li&gt;Audi A7&lt;/li&gt;
   *       &lt;li&gt;Audi A8&lt;/li&gt;
   *	   &lt;li&gt;BMW 501&lt;/li&gt;
   *       &lt;li&gt;BMW 502&lt;/li&gt;
   *       &lt;li&gt;BMW 503&lt;/li&gt;
   *       &lt;li&gt;BMW 507&lt;/li&gt;
   *       &lt;li&gt;BMW 3200&lt;/li&gt;
   *    &lt;/ul&gt;
			* &lt;script&gt;
     * // Create ListBox
	   * $('#carsList').ejListBox(); 	
     * var ListBoxObj  = $("#drpdwn").data("ejListBox");
     * ListBoxObj.moveDown(); //moves down the selected item in the ListBox
     * &lt;/script&gt;
     * @example 
       *    &lt;ul id="carsList"&gt;
   *       &lt;li&gt;Audi A4&lt;/li&gt;
   *       &lt;li&gt;Audi A5&lt;/li&gt;
   *       &lt;li&gt;Audi A6&lt;/li&gt;
   *       &lt;li&gt;Audi A7&lt;/li&gt;
   *       &lt;li&gt;Audi A8&lt;/li&gt;
   *	   &lt;li&gt;BMW 501&lt;/li&gt;
   *       &lt;li&gt;BMW 502&lt;/li&gt;
   *       &lt;li&gt;BMW 503&lt;/li&gt;
   *       &lt;li&gt;BMW 507&lt;/li&gt;
   *       &lt;li&gt;BMW 3200&lt;/li&gt;
   *    &lt;/ul&gt;
			* &lt;script&gt;
     * // Create ListBox
   * $('#carsList').ejListBox(); 	
     * $('#drpdwn').ejListBox("moveDown"); 	
     * &lt;/script&gt;
     *@memberof ejListBox
     * @instance
     */
        moveDown: function () {
            var nextItem = this._getItem(this._selectedItemindex()).next();
            var selectedItem = this._getItem(this._selectedItemindex());
            var nextindex = this._selectedItemindex() + 1;
            var selectedindex = this._selectedItemindex();
            nextItem.insertBefore(selectedItem);
            this._refreshContainer();
			this._trigger('indexchange', {  data: this.model });
        },
		/**
     * To check all the list items in the ListBox  
     * @return jQuery
     * @example 
           *    &lt;ul id="carsList"&gt;
   *       &lt;li&gt;Audi A4&lt;/li&gt;
   *       &lt;li&gt;Audi A5&lt;/li&gt;
   *       &lt;li&gt;Audi A6&lt;/li&gt;
   *       &lt;li&gt;Audi A7&lt;/li&gt;
   *       &lt;li&gt;Audi A8&lt;/li&gt;
   *	   &lt;li&gt;BMW 501&lt;/li&gt;
   *       &lt;li&gt;BMW 502&lt;/li&gt;
   *       &lt;li&gt;BMW 503&lt;/li&gt;
   *       &lt;li&gt;BMW 507&lt;/li&gt;
   *       &lt;li&gt;BMW 3200&lt;/li&gt;
   *    &lt;/ul&gt;
			* &lt;script&gt;
     * // Create ListBox
	   * $('#carsList').ejListBox({showCheckbox:true}); 	
     * var ListBoxObj  = $("#drpdwn").data("ejListBox");
     * ListBoxObj.checkAll(); // checks all the list items in the ListBox
     * &lt;/script&gt;
     * @example 
       *    &lt;ul id="carsList"&gt;
   *       &lt;li&gt;Audi A4&lt;/li&gt;
   *       &lt;li&gt;Audi A5&lt;/li&gt;
   *       &lt;li&gt;Audi A6&lt;/li&gt;
   *       &lt;li&gt;Audi A7&lt;/li&gt;
   *       &lt;li&gt;Audi A8&lt;/li&gt;
   *	   &lt;li&gt;BMW 501&lt;/li&gt;
   *       &lt;li&gt;BMW 502&lt;/li&gt;
   *       &lt;li&gt;BMW 503&lt;/li&gt;
   *       &lt;li&gt;BMW 507&lt;/li&gt;
   *       &lt;li&gt;BMW 3200&lt;/li&gt;
   *    &lt;/ul&gt;
			* &lt;script&gt;
     * // Create ListBox
   * $('#carsList').ejListBox({showCheckbox:true}); 	
     * $('#drpdwn').ejListBox("checkAll"); 	
     * &lt;/script&gt;
     *@memberof ejListBox
     * @instance
     */
		checkAll: function () {
            if (!this.model.enabled) return false;
            for (i = 0; i < this.listitems.length; i++) {
			 if (!($(this.listitems[i].firstChild).find('.listcheckbox').ejCheckBox('isChecked'))) {
                    $(this.listitems[i].firstChild).find('.listcheckbox').ejCheckBox('option', 'checked', true);
                    this.model.checkedItems.push(this.listitems[i]);
				   }
                }
        },
		/**
     * To check items in the checkitemslist in the ListBox  
     * @return jQuery
     * @example 
           *    &lt;ul id="carsList"&gt;
   *       &lt;li&gt;Audi A4&lt;/li&gt;
   *       &lt;li&gt;Audi A5&lt;/li&gt;
   *       &lt;li&gt;Audi A6&lt;/li&gt;
   *       &lt;li&gt;Audi A7&lt;/li&gt;
   *       &lt;li&gt;Audi A8&lt;/li&gt;
   *	   &lt;li&gt;BMW 501&lt;/li&gt;
   *       &lt;li&gt;BMW 502&lt;/li&gt;
   *       &lt;li&gt;BMW 503&lt;/li&gt;
   *       &lt;li&gt;BMW 507&lt;/li&gt;
   *       &lt;li&gt;BMW 3200&lt;/li&gt;
   *    &lt;/ul&gt;
			* &lt;script&gt;
     * // Create ListBox
	   * $('#carsList').ejListBox({showCheckbox:true}); 	
     * var ListBoxObj  = $("#drpdwn").data("ejListBox");
     * ListBoxObj.checkitems(); // checks all the list items in the ListBox
     * &lt;/script&gt;
     * @example 
       *    &lt;ul id="carsList"&gt;
   *       &lt;li&gt;Audi A4&lt;/li&gt;
   *       &lt;li&gt;Audi A5&lt;/li&gt;
   *       &lt;li&gt;Audi A6&lt;/li&gt;
   *       &lt;li&gt;Audi A7&lt;/li&gt;
   *       &lt;li&gt;Audi A8&lt;/li&gt;
   *	   &lt;li&gt;BMW 501&lt;/li&gt;
   *       &lt;li&gt;BMW 502&lt;/li&gt;
   *       &lt;li&gt;BMW 503&lt;/li&gt;
   *       &lt;li&gt;BMW 507&lt;/li&gt;
   *       &lt;li&gt;BMW 3200&lt;/li&gt;
   *    &lt;/ul&gt;
			* &lt;script&gt;
     * // Create ListBox
   * $('#carsList').ejListBox({showCheckbox:true}); 	
     * $('#drpdwn').ejListBox("checkitems"); 	
     * &lt;/script&gt;
     *@memberof ejListBox
     * @instance
     */
		checkitems: function () {
		    if (!this.model.enabled) return false;
		    if (this.model.showCheckbox) {
		         for (i = 0; i < this.model.checkedItemlist.length; i++) {
			var item =this.model.checkedItemlist[i];
			 if (!($(this.listitems[item].firstChild).find('.listcheckbox').ejCheckBox('isChecked'))) {
                    $(this.listitems[item].firstChild).find('.listcheckbox').ejCheckBox('option', 'checked', true);
                    this.model.checkedItems.push(this.listitems[item]);
				   }
                }
		    }
		    else {
		        if(this.model.allowMultiSelection){
		        for (i = 0; i < this.model.selectedItemlist.length; i++) {
		            var item = this.model.selectedItemlist[i];
		            if (!($(this.listitem[item]).hasClass("e-select"))) {
		                $(this.listitem[item]).addClass("e-select")
		                this.model.selectedItems.push($(this.listitem[item]));
		            }
		            else {
		                $(this.listitem[item]).removeClass("e-select")
		                this.model.selectedItems.pop($(this.listitem[item]));
		            }
		        }
		    }
		        else {
		            this.unSelectAll();
		         }
		    }
		    //this._refreshContainer();
        },
	/**
     * To uncheck all the list items in the ListBox  
     * @return jQuery
     * @example 
           *    &lt;ul id="carsList"&gt;
   *       &lt;li&gt;Audi A4&lt;/li&gt;
   *       &lt;li&gt;Audi A5&lt;/li&gt;
   *       &lt;li&gt;Audi A6&lt;/li&gt;
   *       &lt;li&gt;Audi A7&lt;/li&gt;
   *       &lt;li&gt;Audi A8&lt;/li&gt;
   *	   &lt;li&gt;BMW 501&lt;/li&gt;
   *       &lt;li&gt;BMW 502&lt;/li&gt;
   *       &lt;li&gt;BMW 503&lt;/li&gt;
   *       &lt;li&gt;BMW 507&lt;/li&gt;
   *       &lt;li&gt;BMW 3200&lt;/li&gt;
   *    &lt;/ul&gt;
			* &lt;script&gt;
     * // Create ListBox
	   * $('#carsList').ejListBox({showCheckbox:true}); 	
     * var ListBoxObj  = $("#drpdwn").data("ejListBox");
     * ListBoxObj.unCheckAll(); // checks all the list items in the ListBox
     * &lt;/script&gt;
     * @example 
       *    &lt;ul id="carsList"&gt;
   *       &lt;li&gt;Audi A4&lt;/li&gt;
   *       &lt;li&gt;Audi A5&lt;/li&gt;
   *       &lt;li&gt;Audi A6&lt;/li&gt;
   *       &lt;li&gt;Audi A7&lt;/li&gt;
   *       &lt;li&gt;Audi A8&lt;/li&gt;
   *	   &lt;li&gt;BMW 501&lt;/li&gt;
   *       &lt;li&gt;BMW 502&lt;/li&gt;
   *       &lt;li&gt;BMW 503&lt;/li&gt;
   *       &lt;li&gt;BMW 507&lt;/li&gt;
   *       &lt;li&gt;BMW 3200&lt;/li&gt;
   *    &lt;/ul&gt;
			* &lt;script&gt;
     * // Create ListBox
   * $('#carsList').ejListBox({showCheckbox:true}); 	
     * $('#drpdwn').ejListBox("unCheckAll"); 	
     * &lt;/script&gt;
     *@memberof ejListBox
     * @instance
     */
        unCheckAll: function () {
            if (!this.model.enabled) return false;
            for (i = 0; i < this.listitems.length; i++) {
                if ($(this.listitems[i].firstChild).find('.listcheckbox').ejCheckBox('isChecked')) {
                    $(this.listitems[i].firstChild).find('.listcheckbox').ejCheckBox('option', 'checked', false);
                }
                this.model.checkedItems.pop(this.listitems[i]);
            }
        },
		   /**
     * To add an list item in the ListBox  
     * @return jQuery
     * @example 
           *    &lt;ul id="carsList"&gt;
   *       &lt;li&gt;Audi A4&lt;/li&gt;
   *       &lt;li&gt;Audi A5&lt;/li&gt;
   *       &lt;li&gt;Audi A6&lt;/li&gt;
   *       &lt;li&gt;Audi A7&lt;/li&gt;
   *       &lt;li&gt;Audi A8&lt;/li&gt;
   *	   &lt;li&gt;BMW 501&lt;/li&gt;
   *       &lt;li&gt;BMW 502&lt;/li&gt;
   *       &lt;li&gt;BMW 503&lt;/li&gt;
   *       &lt;li&gt;BMW 507&lt;/li&gt;
   *       &lt;li&gt;BMW 3200&lt;/li&gt;
   *    &lt;/ul&gt;
			* &lt;script&gt;
     * // Create ListBox
	   * $('#carsList').ejListBox(); 	
     * var ListBoxObj  = $("#drpdwn").data("ejListBox");
     * ListBoxObj.addItem("Java"); // add an new item in the ListBox
     * &lt;/script&gt;
     * @example 
       *    &lt;ul id="carsList"&gt;
   *       &lt;li&gt;Audi A4&lt;/li&gt;
   *       &lt;li&gt;Audi A5&lt;/li&gt;
   *       &lt;li&gt;Audi A6&lt;/li&gt;
   *       &lt;li&gt;Audi A7&lt;/li&gt;
   *       &lt;li&gt;Audi A8&lt;/li&gt;
   *	   &lt;li&gt;BMW 501&lt;/li&gt;
   *       &lt;li&gt;BMW 502&lt;/li&gt;
   *       &lt;li&gt;BMW 503&lt;/li&gt;
   *       &lt;li&gt;BMW 507&lt;/li&gt;
   *       &lt;li&gt;BMW 3200&lt;/li&gt;
   *    &lt;/ul&gt;
			* &lt;script&gt;
     * // Create ListBox
   * $('#carsList').ejListBox(); 	
     * $('#drpdwn').ejListBox("addItem","Java"); 	
     * &lt;/script&gt;
     *@memberof ejListBox
     * @instance
     */
		addItem:function(val){
            this.listitem=  this.listContainer.find('ul').append('<li role="option">'+val+'</li>');
            this._refreshItems();
        },
        _checkContains: function (Value) {
            var chkValue = Value;
            var values = this.element[0].value.split(",");
            this.contains = false;
            for (var i = 0; i < values.length; i++) {
                if (values[i] == chkValue)
                    this.contains = true;
            }
            return this.contains;
        },
        _init: function () {
            	this._id=this.element[0].id;
                this._initialize();
                this._render();
                this._enabled(this.model.enabled);
                this._wireEvents();
                this._initValue = false;
        },
       
        _changeSkin: function (skin) {
            if (this.model.cssClass != skin) {
                this.listContainer.removeClass(this.model.cssClass).addClass(skin);
            }
        },

        _setRTL: function (boolean) {
            this.listContainer.addClass("e-rtl");
        },
     	_setText: function (text) {
		var len=this.listitems.length;
		for(i=0;i<len;i++){
		if(this.element.children("li")[i].textContent == text){
		this.unSelectAll();
		this.selectItemByIndex(i+1);
		}
		}
        },
        /**
       * To configure the properties at runtime using SetModel		
       * @private
       */
        _setModel: function (options) {
            if (!(this.model.enabled) && ej.isNullOrUndefined(options["enabled"])) return false;
            var option;
            for (option in options) {
                switch (option) {

                    case "dataSource": this._checkModelDataBinding(options[option]); break;
                    case "query": this._queryCheck(options[option]); break;
                    case "fields": this.model.fields = options[option]; this._checkModelDataBinding(this.model.dataSource); break;
                    case "template": this.model.template = options[option]; this._checkModelDataBinding(this.model.dataSource); break;
                    case "value": this._setText(ej.util.getVal(options[option])); break;
                    case "enableRTL": this._setRTL(options[option]); break;
                    case "enabled": this._enabled(options[option]); break;
                    case "height": this.model.Height = options[option]; this._setListHeight(options[option]); break;
                    case "width": this.model.Width = options[option]; this._setListWidth(); break;
                    case "cssClass": this._changeSkin(options[option]); break;
                    case "showCheckbox": this.checkboxHideShow(options[option]); break;
                    case "checkAll": this._setCheckAll(options[option]); break;
                    case "uncheckAll": this._setUncheckAll(options[option]); break;
                    case "showRoundedCorner": this._roundedCorner(options[option]); this.model.showRoundedCorner = options[option]; break;
                    case "selectedItemIndex": this.selectItemByIndex(options[option]); break;
                    case "selectedItems": this._selectCheckedItem(options[option]); this.model.selectedItems = options[option]; break;
                    case "checkedItems": this._selectCheckedItem(options[option]); this.model.checkedItems = options[option]; break;
                    case "checkedItemlist": this.model.checkedItemlist = options[option]; break;
                    case "selectedItemlist": this.model.selectedItemlist = options[option]; break;
                    case "allowMultiSelection": this.model.allowMultiSelection = options[option];
                        options[option] ? this._renderBoxModel() : this._destroyBoxModel();
                        break;
                    case "allowDragAndDrop":
                        {
                            if (options[key])
                                this._enableDargDrop();
                            break;
                        }
		
                }
            }
        },
        
        _destroy: function () {
            this.element[0].value = "";
			var predecessor =$(this.element.parents()[2]);
             this.element.removeClass("e-listbox e-js e-ul");
            if (this.model.dataSource != null){
			
			this.element.children().remove();
			predecessor.append(this.element.parents().first().html());
                this.listContainer.remove();
				}
            else {
			predecessor.append(this.element.parents().first().html());
                this.listContainer.remove();
            }
        },
        /* cookie */

        _finalize: function () {
            if (this.model.selectedItemIndex != null) {
                this.selectItemByIndex(this.model.selectedItemIndex);
            } else if ((this.model.showCheckbox == true) && (this.model.selectedItems.length > 0)) {
                this._selectCheckedItem(this.model.selectedItems);
            }
        },

        /**/
        _initialize: function () {
            this.target = this.element[0];
            this._queryString = null;
            this._itemId = null;
            this.checkedStatus = false;
            this._incqueryString = "";
            this._activeItem = null;
            this._initValue = true;
        },
      


        _render: function () {
            this._renderContainer();
            this._initDataSource(this.model.dataSource);
            this._showFullList();
            this._addDragableClass();
            this._finalize();
        },

        _queryCheck: function (value) {
            this.model.query = value;
            this._checkModelDataBinding(this.model.dataSource);
        },

        _checkModelDataBinding: function (source) {
            this.model.dataSource = source;
            if (source != null) {
                if (ej.DataManager && source instanceof ej.DataManager) {
                    this._initDataSource(source);
                } else {
                    this._showFullList();
                }
            }
            else {
                this.element.empty();
            }
        },
        _initDataSource: function (source) {
            var proxy = this;
            if (ej.DataManager && source instanceof ej.DataManager) {
                proxy.listitems = proxy.model.dataSource;
                proxy._addLoadingClass();
                var queryPromise = source.executeQuery(this._getQuery());
                queryPromise.done(function (e) {
                    proxy.listitems = e.result;
                    proxy._removeLoadingClass();
                    proxy._showFullList();

                }).fail(function (e) {
                    proxy.model.dataSource = null;
                    proxy._addLoadingClass();
                });
            }
        },


     
        _getQuery: function () {
            if (ej.isNullOrUndefined(this.model.query)) {
                var column = [], queryManager = ej.Query(), mapper = this.model.fields;
                for (var col in mapper) {
                    if (col !== "tableName")
                        column.push(mapper[col]);
                }
                if (column.length > 0)
                    queryManager.select(column);
                if (!this.model.dataSource.dataSource.url.match(mapper.tableName + "$"))
                    !ej.isNullOrUndefined(mapper.tableName) && queryManager.from(mapper.tableName);
            }
            else queryManager = this.model.query;
            return queryManager;
        },
        _addDragableClass: function () {
            if (this.model.allowDragAndDrop) {
			this.element.css("cursor","pointer");
			this.listContainer.addClass("e-droppable");
			this.listBoxScroller.addClass("e-droppable");
                this.element.children("li").each(function (index) {
                    ($(this).addClass("e-draggable e-droppable"));
                });
            }
        },
     
        _enableDargDrop: function () {
            if (this.model.allowDragAndDrop) {
                this._drag();
            }
        },
        _addLoadingClass: function () {
            this.listContainer.addClass("e-load");
            //this.drpbtnspan.removeClass("e-icon e-down-arrow");
            //this.model.readOnly = true;
        },

        _removeLoadingClass: function () {
            this.listContainer.removeClass("e-load");
        },

        _checkNameAttr: function () {
			if (this.element.attr("name"))
                this._hiddenInput.attr("name", this.element.attr("name"));
            else {
                this._hiddenInput.attr("name", this._id);
				this.element.attr("name", this._id);
			}
        },
     
        //render listBox pannel wrapper
        _renderContainer: function () {
            //render listBox 
			 this.listContainer = ej.buildTag("div.e-ddl-popup e-box e-popup e-widget " + this.model.cssClass + "#" + this._id + "_container", "", { "visibility": "hidden" }, { "tabIndex": 0 });
            this.listBoxScroller = ej.buildTag("div"); this.ultag = ej.buildTag("ul.e-ul", "", {}, { "role": "listbox" });
			this.element= this.element.addClass("e-ul");
			this.listContainer.append(this.listBoxScroller);
			this.listContainer.insertAfter(this.element);
			this.listBoxScroller.append(this.element);
			 this.element.attr('unselectable', 'on')
             this.element.css('user-select', 'none')
             this.element.on('selectstart', false);
        },
        //render listContainer
        _renderlistContainer: function () {
            var list = this.listitems, i, ulempty, ulno, litag, _id, _txt, mapper = this.model.fields, predecessor;
			this.lastScrollTop = 0;
            this.dummyUl = $();
            if (this.model.enableRTL)
                this.listContainer.addClass("e-rtl");
            if (this.model.dataSource == null || this.model.dataSource.length < 1) {
                predecessor = this.element.parents().last();
				 if (this.model.targetID)
				   this.docbdy = predecessor.find("#" + this.model.targetID);
                else
				this.docbdy = predecessor.find("#" + this._id);
				this.itemsContainer = this.docbdy;
                this.itemsContainer.children("ol,ul").remove();
                this.items = this.itemsContainer.children('li');
                this.items.children("img").addClass("e-align");
                this.items.children("div").addClass("e-align");
                //this.itemsContainer.children("span").addClass("e-ghead");
                //This will append the list with the listContainerwrapper
                this.element.append(this.itemsContainer.children());
            }
            else if (this.model.dataSource != null && typeof list[0] != "object") {
                if (list.length > 0) {
                    for (i = 0; i < list.length; i++) {
                        litag = ej.buildTag("li", list[i]);
                        this.dummyUl.push(litag[0]);
                    }
                    this.element.append(this.dummyUl);
                }
            }
            else {
                this.mapFld = { _id: null, _imageUrl: null, _imageAttributes: null,_toolTipText:null, _spriteCSS: null, _text: null, _value: null, _htmlAttributes: null, _selected: null };
                this.mapFld._id = (mapper && mapper.id) ? mapper["id"] : "id";
                this.mapFld._imageUrl = (mapper && mapper.imageUrl) ? mapper["imageUrl"] : "imageUrl";
				this.mapFld._toolTipText = (mapper && mapper.toolTipText) ? mapper["toolTipText"] : "toolTipText";
                this.mapFld._imageAttributes = (mapper && mapper.imageAttributes) ? mapper["imageAttributes"] : "imageAttributes";
                this.mapFld._spriteCSS = (mapper && mapper.spriteCssClass) ? mapper["spriteCssClass"] : "spriteCssClass";
                this.mapFld._text = (mapper && mapper.text) ? mapper["text"] : "text";
                this.mapFld._value = (mapper && mapper.value) ? mapper["value"] : "value";
                this.mapFld._htmlAttributes = (mapper && mapper.htmlAttributes) ? mapper["htmlAttributes"] : "htmlAttributes";
                this.mapFld._selected = (mapper && mapper.selected) ? mapper["selected"] : "selected";
                var mapCateg = (mapper && mapper.category) ? mapper["category"] : "category", groupedList, _query;
                _query = ej.Query();
                 if (this.model.enableloadOnDemand) {
				if(!this.model.enablevirtualScrolling){ 
                    for (var i = 0; i < this.listitems.length; i++) {
                        var litag = ej.buildTag('li');
                        litag.css("height", "20px");
                        this.dummyUl.push(litag[0]);
                    }
                    this.element.append(this.dummyUl);
					}
					this.realUllength=0;
                    _query = _query.range(0, 11);
                    this.displayeditem = 2;
                    groupedList = ej.DataManager(list).executeLocal(_query);
                    this._loadlist(groupedList);
                }
                else {
                groupedList = ej.DataManager(list).executeLocal(_query);
                  if (groupedList.length > 0) {
                    if (this.model.template != null) {  //Template binding
                        for (i = 0; i < list.length; i++) {
                           litag = ej.buildTag("li", list[i]);
                           if (this.model.template) litag.append(this._getTemplatedString(list[i]));
                           this.dummyUl.push(litag[0]);
                       }
                       this.element.append(this.dummyUl);
                    } else {
                        _query = ej.Query().group(mapCateg); // Grouping with Category
                        groupedList = ej.DataManager(list).executeLocal(_query);
                        if (this.model.allowGrouping) {
                            for (i = 0; i < groupedList.length; i++) {
                                var spantag = ej.buildTag('span');
                                spantag.append(groupedList[i].key);
                                spantag.addClass("e-ghead");
                                this.dummyUl.push(spantag[0]);
                                this._loadlist(groupedList[i].items);
                            }
                        }
                        else {
						this.realUllength =0;
                            this._loadlist(groupedList[0].items);
							}
						} 
				  }
				}

            }
	
			
            this.element.children('li').attr({ "role": "option" }).css("list-style","none");
            this._setListWidth();
            this._setListHeight();
            this._setListPosition();
            this.listContainer.css("height", ""); this.listBoxScroller.css({ "height": "", "width": "" });
            this.listContainer.ejScroller({ height: this.listContainer.height(), width: 0, scrollerSize: 20 });
            this.scrollerObj = this.listContainer.ejScroller("instance");
			this._setListHeight();
            this.listContainer.css({ 'display': 'none', 'visibility': 'visible' });
            this.checkboxHideShow(this.model.showCheckbox);
            this._setCheckAll(this.model.checkAll);
            this._setUncheckAll(this.model.uncheckAll);
			this.checkitems();
            this._showResult();
        },
        _loadlist: function (sublist) {
                for (var j = 0; j < sublist.length; j++) {
                    var _did = this._getField(sublist[j], this.mapFld._id);
                    var _dimageUrl = this._getField(sublist[j], this.mapFld._imageUrl);
                    var _dimageAttributes = this._getField(sublist[j], this.mapFld._imageAttributes);
                    var _dspriteCss = this._getField(sublist[j], this.mapFld._spriteCSS);
                    var _dtext = this._getField(sublist[j], this.mapFld._text);
                    var _dvalue = this._getField(sublist[j], this.mapFld._value);
                    var _dhtmlAttributes = this._getField(sublist[j], this.mapFld._htmlAttributes);
                    var _dselected = this._getField(sublist[j], this.mapFld._selected);
					var _dtoolTipText = this._getField(sublist[j], this.mapFld._toolTipText);
					var k =this.realUllength;                   
					 if ((_dvalue) && (_dvalue != ""))
                        litag = ej.buildTag('li', "", {}, { value: _dvalue });
                    else
                        litag = ej.buildTag('li');
                    if (_did)
                        litag.attr('id', _did);
                    if ((_dimageUrl) && (_dimageUrl != "")) {
                        //Creating the image tag
                        imgtag = ej.buildTag('img.e-align', '', {}, { 'src': _dimageUrl, 'alt': _dtext });
                        if ((_dimageAttributes) && (_dimageAttributes != ""))
                            imgtag.attr(_dimageAttributes);
                        litag.append(imgtag);
                    }
                    if ((_dspriteCss) && (_dspriteCss != "")) {
                        //Creating the Sprite image tag
                        divtag = ej.buildTag('div.e-align ' + _dspriteCss + ' sprite-image');
                        litag.append(divtag);
                    }
                    if ((_dtext) && (_dtext != "")) {
                        //Creating text Content inside the  tag
                        litag.append(_dtext);
                    }
                    if ((_dhtmlAttributes) && (_dhtmlAttributes != ""))
                        litag.attr(_dhtmlAttributes);
					if ((_dtoolTipText) && (_dtoolTipText != "")  && this.model.enableTooltip)
                        litag.attr('title',_dtoolTipText);
                    if (_dselected) {
                        litag.addClass("chkselect");
                    }
					if(this.element.children()[k] !=null && this.model.enableloadOnDemand && $(this.element.children()[k]).text()== "" ){
					$(this.element.children()[k]).replaceWith(litag[0]);
					}
					else
					this.dummyUl.push(litag[0]);
					this.realUllength +=1;
                }
		   if(!this.model.enableloadOnDemand)
				 this.element.children().remove();
           if(this.element.children()[k] ==null && (!this.model.enableloadOnDemand ||this.model.enablevirtualScrolling))
           this.element.append(this.dummyUl);
              
        },
        //get object
        _getField: function (obj, fieldName) {
            return ej.pvt.getObject(fieldName, obj);
        },
        //template
        _getTemplatedString: function (list) {
            var str = this.model.template, start = str.indexOf("${"), end = str.indexOf("}");
            while (start != -1 && end != -1) {
                var content = str.substring(start, end + 1);
                var field = content.replace("${", "").replace("}", "");
                str = str.replace(content, this._getField(list, field));
                start = str.indexOf("${"), end = str.indexOf("}");
            }
            return str;
        },
   
        //checkbox properties
        checkboxHideShow: function (value) {
            if (value) {
                this._createCheckbox();
            } else {
                this._removeCheckbox();
            }
            this.model.showCheckbox = value;
        },
        _createCheckbox: function () {
            var i, _extchk, chklist, me = this;
            this.listitems = this.listContainer.find("ol,ul").children("li");
            chklist = this.listitems.find('input[type=checkbox]');
            if (chklist.length == 0) {
                for (i = 0; i < this.listitems.length; i++) {
                    $checkbox = ej.buildTag("input.listcheckbox e-align#popuplist" + i + "_" + this._id, "", {}, { type: "checkbox", name: "list" + i });
                    $(this.listitems[i]).prepend($checkbox);
                }
                this.listContainer.find(".listcheckbox").ejCheckBox({ cssClass: this.model.cssClass, change: $.proxy(this._OnClickCheckList, this) });
                for (i = 0; i < this.listitems.length; i++) {
                    if ($(this.listitems[i]).hasClass('chkselect')) {
                        $(this.listitems[i]).find(".listcheckbox").ejCheckBox({ "checked": true });
                        this._activeItem = i;
                        this.checkedStatus = true;
                        if (!this._initValue)
                            this._trigger('checkChange', { isChecked: this.checkedStatus, data: this.model });
                           $(this.listitems[i]).removeClass('chkselect');
                    }
                }
            }
        },

        _removeCheckbox: function () {
            var i, checkbox;
            this.listitem = this.listContainer.find("ol,ul").children("li");
            checkbox = this.listitem.find('.listcheckbox');
            if (checkbox.length > 0) {
                this.listitem.find('.listcheckbox').ejCheckBox('destroy');
                this.listitem.find('input[type=checkbox]').remove();
            }
        },
        _setCheckAll: function (value) {
            //Check all the nodes on CheckAll = true
            if ((this.model.showCheckbox) && (value))
                this.checkAll();
        },
        _setUncheckAll: function (value) {
            //UnCheck all the nodes on UnCheckAll = true
            if ((this.model.showCheckbox) && (value))
                this.unCheckAll();
        },
 
        _selectCheckedItem: function (chkitems) {
            if (chkitems.length > 0) {
                for (i = 0; i < chkitems.length; i++) {
              this.model.selectedItems.push(chkitems[i]);
                }
            }
        },
        
     
        //List Properties
        _refreshScroller: function () {
		if(this.model.enablevirtualScrolling){
		  this.listContainer.find(".e-content, .e-vhandle").removeAttr("style");
            this.listContainer.find(".e-vhandle div").removeAttr("style");
            this.listContainer.css({ "display": "block" });  // For get the height of the popup
            this.scrollerObj.model.height = this.listContainer.height();
            this.scrollerObj.refresh();
            this.scrollerObj.option("scrollTop", 0);
		}
		else{
            this.listContainer.find(".e-vhandle div").removeAttr("style");
			var listboxcontent = this.listBoxScroller.height();
            this.listContainer.css({ "display": "block" });
				// For get the height of the listBox
            this.scrollerObj.model.height = this.listContainer.height();
            this.scrollerObj.refresh();
			this.listBoxScroller.css("height","");	
		 this.listBoxScroller.css("height","100%");		
		 //this.element.css("height","");	
		 
			}
        },
         _setListWidth: function () {
            var width = this.model.width;
            this.listContainer.css({ "width": width });
        },

        _setListHeight: function () {
		 var height = this.model.height;
		 this.listContainer.css({ "height": height });
        },
        _refreshContainer: function () {           
            this.listContainer.css({ "width": this.model.width });
            this._setListPosition();
			this._setListHeight();
			this._roundedCorner();
            this._refreshScroller();
        },
        _setListPosition: function () {
            this.listContainer.css({
                "position":"relative"
            });
        },

        //dragEventHandler()
        _drag: function () {
            var proxy = this;
            var pre = false;
            var _clonedElement = null;
            var dragContainment = null;
            this._listitem = this.element.parent();
            $(this._listitem).find("li").not(".e-js").ejDraggable({
                dragArea: dragContainment,
                clone: true,
                dragStart: function (args) {
				var targetid = this.element.parent()[0].id; // to get id of the dragged control
            var draggedobj = $("#" + targetid).data("ejListBox");
			draggedobj ._refreshItems();
                    var data = { target: args.target, event: args.event };
                    if (proxy._onDragStarts(data)) {
                        args.cancel = true;
                        _clonedElement && _clonedElement.remove();
                        return false;
                    }
                },
                drag: function (args) {
                    var target = args.target;
                    var data = { target: target, event: args.event };
                    if (proxy._onDrag(data))
                        return false;
                    if ($(target).hasClass('e-droppable') || $(target).parent().hasClass('e-droppable')) {
                        document.body.style.cursor = '';
                        $(target).addClass("allowDrop");
                    }
                },
                dragStop: function (args) {
                    if (!args.element.dropped) {
                        _clonedElement && _clonedElement.remove();
                        document.body.style.cursor = '';
                    }
                    var target = args.target;
                    var targetObj = proxy;
                    var position = pre ? "Before" : "After";
                    var data = { target: target, position: position, event: args.event };
                    if (proxy._onDragStop(data))
                        return false;
                    $(args.element).removeClass("e-active");
                 
                        if (target.nodeName == 'UL')
                            target = $(target).children()[0];
                        if (target.nodeName != 'LI')
                            target = $(target).parent('.e-droppable')[0] || $(target).parent();
                        if (target.nodeName == 'LI' && $(target).hasClass('e-droppable')) 
                               proxy._dropItem(target, args.element, pre, args.event);
						else if ($(target).hasClass('e-droppable')) 
                               proxy._dropItemContainer(target, args.element, pre, args.event);
           
                    $(".allowDrop").removeClass("allowDrop");
                    var data = { target: target, event: args.event, dropedElement: args.element };
                    if (proxy._onDropped(data))
                        return false;
                    document.body.style.cursor = '';
				
                },
                helper: function (event, ui) {
                    _clonedElement = $(event.sender.target).clone().addClass("dragClone dragClonelist");
                    return _clonedElement.appendTo($("body"));
                }
            });
        },

        //Drop list Item
        _dropItem: function (target, element, pre, event) {
            var li = event.ctrlKey ? $(element).parent().parent().clone() : $(element);
            pre ? $(li).insertBefore(target) : $(li).insertAfter(target);
            this._refreshItems();
            var targetid = $(target).parents('div')[1].id.replace('_container', ''); // to get id of the dropped control
            var droppedobj = $("#" + targetid).data("ejListBox");
			droppedobj._refreshItems();
        },
		
		_dropItemContainer: function (target, element, pre, event) { // when dropped into listcontainer
            var li = event.ctrlKey ? $(element).parent().parent().clone() : $(element);
			$(li).insertAfter($($(target).find('li')).last());
			$(target).find('ul').append($(li))
            this._refreshItems();
            var targetid = $(target)[0].id.replace('_container', ''); // to get id of the dropped control
            var droppedobj = $("#" + targetid).data("ejListBox");
			droppedobj._refreshItems();
        },
        //listContainerstatus
        _showResult: function () {
            var proxy = this;
            this._refreshContainer();
            this.element.attr({ "aria-expanded": true }); var _ListItemsContainer = this.element.children("li");
            this._listSize = _ListItemsContainer.size();
			_ListItemsContainer.unbind("mouseenter", $.proxy(this._OnMouseEnter, this));
            _ListItemsContainer.unbind("mouseleave", $.proxy(this._OnMouseLeave, this));
            _ListItemsContainer.unbind("click", $.proxy(this._OnMouseClick, this));
				this._off($(this.listContainer), "DOMMouseScroll", this._onScroll);
				this._on($(this.listContainer), "DOMMouseScroll", this._onScroll);
			
            _ListItemsContainer.bind("mouseenter", $.proxy(this._OnMouseEnter, this));
            _ListItemsContainer.bind("mouseleave", $.proxy(this._OnMouseLeave, this));
            _ListItemsContainer.bind("click", $.proxy(this._OnMouseClick, this));
			 $(window).bind("resize", $.proxy(this._OnWindowResize, this));
			 	 
        },

        _OnWindowResize: function (e) {
            this._refreshContainer();
            this.listContainer.css("display", "block");
        },
        _removeActive: function () {
            this._ulBox.children("li").removeClass("e-active");
        },
      
        _removeListHover: function () {
		this.model.selectedItems =[];
            this.element.children("li").removeClass("e-hover");
			 this.element.children("li").removeClass("e-select");
        },

        _addListHover: function () {
            var activeItem = this._getItem(this._selectedItem);
            activeItem.addClass("e-select");
			activeItem.addClass("e-hover");
            this.scrollerObj.setModel({ "scrollTop": this._calcScrollTop() });
            activeItem.focus();
			this._OnListSelect(this.prevselectedItem, this._selectedItem);
        },
 
        _calcScrollTop: function () {
            var ulH = this.element.outerHeight(), li = this.element.find("li"), liH = 0, index, top, i;
            index = this.element.find("li.e-select").index();
			if(this.model.enablevirtualScrolling)
			index =  this.displayeditem;
            for (i = 0; i < index; i++) { liH += li.eq(i).outerHeight(); }
            top = liH - ((this.listContainer.outerHeight() - li.eq(index).outerHeight()) / 2);
            return top;
        },
     
        _refreshItems: function () {
            this.listBoxScroller.append(this.element);
            this.listContainer.append(this.listBoxScroller);
            this._refreshContainer();
            this._showResult();
        },
       
        _selectedItemindex: function () {
            var selectItem;
            this.element.children("li").each(function (index) {
              if ($(this).hasClass("e-select")) {
                    selectItem = index;
                    return false;
                }
            });
            this._selectedItem = selectItem;
            return selectItem;
        },
        _getActiveItem: function () {		
		   return $(this.element.children("li")[this._activeItem]);          
        },
        _getItem: function (val) {
            return $(this.element.children("li")[val]);
        },
        _roundedCorner: function (val) {
            if (this.model.showRoundedCorner) {
               this.listContainer.addClass("e-corner-all");
            }
        },

        _enabled: function (boolean) {
            if (boolean) this.enable();
            else this.disable();
        },

        _showFullList: function () {
            if (this.model.dataSource != null) {
                if (!(ej.DataManager && this.model.dataSource instanceof ej.DataManager)) {
                    this.listitems = this.model.dataSource;
                }
            }
            this._renderlistContainer();
        },

        // cascade options
        _cascadeAction: function () {
            if (this.model.cascadeTo) {
                this._currentValue = this._getField(this.listitems[this._activeItem], this.mapFld._value);
                this.selectDropObj = $('#' + this.model.cascadeTo).ejListBox('instance');
                if (ej.isNullOrUndefined(this._dSource))
                    this._dSource = this.selectDropObj.model.dataSource;
                    this._performJsonDataInit();
              
            }
        },
        _performJsonDataInit: function () {
            this._changedSource = ej.DataManager(this._dSource).executeLocal(ej.Query().where(this.mapFld._value, "==", this._currentValue));
            this.selectDropObj.setModel({ dataSource: this._changedSource, enabled: true, value:"" });
        },
        //Events
        _OnMouseEnter: function (e) {
		if(this.model.enabled){
            var targetEle;
            this.element.children("li").removeClass("e-hover");
            if ($(e.target).is("li")) { $(e.target).addClass("e-hover"); }
            else if (e.target.tagName != "li") {
                targetEle = $(e.target).parents("li");
                $(targetEle).addClass("e-hover");
            }
            var activeItem = 0, selectItem=0;
            this.element.children("li").each(function (index) {
                if ($(this).hasClass("e-hover")) {
                    activeItem = index;
                    return false;
                }
            });
            this._activeItem = activeItem;
			}
        },
    
        _OnMouseLeave: function (e) {
            this.element.children("li").removeClass("e-hover");
        },
        _OnMouseClick: function (e) {
		if(this.model.enabled){
            if (!this.model.showCheckbox) {
                this.prevselectedItem = this._selectedItemindex();
                var activeitem = $(this.element.children("li")[this._activeItem]);
                if (!this.model.allowMultiSelection || (!e.ctrlKey && !e.shiftKey)) {
                    this._removeListHover();
                }
              
                this.element.children("li").removeClass('e-hover');
                if (!activeitem.hasClass('e-select')) {
                    activeitem.addClass('e-select');
					this.model.selectedItems.push(activeitem);
					this.model.selectedItemlist.push(this._activeItem);
                    if(e.shiftKey &&(this.model.allowMultiSelection)){var initial,last;
                        if( this.prevselectedItem<this._activeItem){
                            initial=this.prevselectedItem;
                            last=this._activeItem
                        }
                        else{
                            initial=this._activeItem
                            last=this.prevselectedItem;
                        }
                        for(var i=initial;i<=last;i++){
                            activeitem = $(this.element.children("li")[i])
                            if (!activeitem.hasClass('e-select')) 
                                activeitem.addClass('e-select');
                            this.model.selectedItems.push(activeitem);
							this.model.selectedItemlist.push(this._activeItem);
                        }
                    }
                }
                else {
                    activeitem.removeClass('e-select');
                    this.model.selectedItems.pop(activeitem);
					this.model.selectedItemlist.pop(this._activeItem);
                }

                this._selectedItem = this._selectedItemindex();
                this._cascadeAction();
				var selecteditem =$(this.element.children("li")[this._selectedItem]);
			if($(selecteditem).text() != ""){
			this.element.val($(selecteditem).text());
			this.value($(selecteditem).text());
			this.element.attr({ "value": "this.element.val()" });
			}
				this.model.selectedText=activeitem.text();
                var data = { item: this.model};
				 this.model.selectedItemIndex = this._selectedItem;
                this._trigger("selected", data);
               
              } else {
                if (($(e.currentTarget).is("li")) && ($(e.target).is("li"))) {
                    if ($(e.currentTarget.firstChild).find('.listcheckbox').ejCheckBox('isChecked')) {
                        $(e.currentTarget.firstChild).find('.listcheckbox').ejCheckBox('option', 'checked', false);
                        this.model.checkedItems.pop(e.currentTarget);
						this.model.checkedItemlist.pop(this._selectedItem);
                    
                    } else {
                        $(e.currentTarget.firstChild).find('.listcheckbox').ejCheckBox('option', 'checked', true);
                        this.model.checkedItems.push(e.currentTarget);
						this.model.checkedItemlist.push(this._activeItem);
                    }
                }
				else if($(e.currentTarget).is("li")){
				if ($(e.currentTarget.firstChild).find('.listcheckbox').ejCheckBox('isChecked')) {
				    	this.model.checkedItems.push(e.currentTarget);
						this.model.checkedItemlist.push(this._activeItem);
						}
				else{
					this.model.checkedItems.pop(e.currentTarget);
					this.model.checkedItemlist.pop(this._activeItem);
					}
				}
				else{
				return false;
				}
                if (!this.element.hasClass("e-disable")) {
                    var args = { status: this.model.enabled };
                    this._trigger("checkChange", args);
                }
                 }
            this._OnListSelect(this.prevselectedItem,this._activeItem);
			}
	
        },
        _OnClickCheckList: function (e) {
            if (e.isChecked) {
                this.checkedStatus = true;
               
            } else {
                this.checkedStatus = false;
       
            }
            if (!this._initValue)
                this._trigger('checkChange', { isChecked: this.checkedStatus, data: this.model });

        },
        _OnListSelect: function (previtem, selecteditem) {
            if (previtem != selecteditem)
                this._trigger('selectIndexChanged', { data: this.model });
        },
        //keyboard events
    
        _OnKeyDown: function (e) {
		if(this.model.enabled){
            if ((!this.model.enabled)) return false;
            this._itemId = null;
            var _ListItemsContainer = this.element.children("li");
            this._listSize = this.element.children("li").size();

            switch (e.keyCode) {
                case 38:    //Up arrow 
                    this.prevselectedItem = this._selectedItemindex();
                    if ((!e.ctrlKey && !e.shiftKey) || !this.model.allowMultiSelection) {
                            this._removeListHover();
                    } 
                            this.element.children("li").removeClass('e-active');
                            if ((this._selectedItem <= 0) || (this._selectedItem == null) || (this._selectedItem > this._listSize - 1))
                                this._selectedItem = this._listSize - 1;
                            else
                                this._selectedItem -= 1;
                            this._addListHover();
                            e.preventDefault();
                            return false;
                   
                    break;

                case 40:    //Down arrow
				
                    this.prevselectedItem = this._selectedItemindex();
					 var selecteditem =0; 
					this.element.children("li").each(function (index) {
					if ($(this).hasClass("e-select"))
                     selecteditem = index;
					});
                    if ((!e.ctrlKey && !e.shiftKey) || !this.model.allowMultiSelection) {
                            this._removeListHover();
                    }                   
					
					this.element.children("li").removeClass('e-active');
					this._selectedItem=selecteditem;
                            if ((this._selectedItem >= this._listSize - 1) || (this._selectedItem == null) || (this._selectedItem < 0))
                                this._selectedItem = 0;
                            else
                                this._selectedItem += 1;
                            this._addListHover();
							e.preventDefault();
							 if (this.model.enableloadOnDemand){
							this._onScroll();
							this.element.focus();
							}
							return false;
                                   break;
                case 37:// Left Key equals up
                    this.prevselectedItem = this._selectedItemindex();
                    if ((!e.ctrlKey && !e.shiftKey) || !this.model.allowMultiSelection) {
                        this._removeListHover();
                    } 
                        this.element.children("li").removeClass('e-active');
                        if ((this._selectedItem <= 0) || (this._selectedItem == null) || (this._selectedItem > this._listSize - 1))
                            this._selectedItem = this._listSize - 1;
                        else
                            this._selectedItem -= 1;
                        this._addListHover();
                 
                    e.preventDefault();
                    return false;
                    break;  // Left Key

                case 39:// Right Key equals down
                    this.prevselectedItem = this._selectedItemindex();
					 var selecteditem =0; 
					this.element.children("li").each(function (index) {
					if ($(this).hasClass("e-select"))
                     selecteditem = index;
					});
					
                    if ((!e.ctrlKey && !e.shiftKey) || !this.model.allowMultiSelection) {
                        this._removeListHover();
                    }
					this.element.children("li").removeClass('e-active');
					this._selectedItem=selecteditem;
                        this.element.children("li").removeClass('e-active');
                        if ((this._selectedItem >= this._listSize - 1) || (this._selectedItem == null) || (this._selectedItem < 0))
                            this._selectedItem = 0;
                        else
                            this._selectedItem += 1;
                        this._addListHover();
                    e.preventDefault();
					 if (this.model.enableloadOnDemand){
							this._onScroll();
							this.element.focus();
							}
                    return false;
                    break;  // Right Key

                case 8:    // Backspace key
                    if ((this.model.allowMultiSelection == true) && (this.model.showCheckbox)) {
                        this._deleteLastBox();
                    }
                    e.preventDefault();
                    return false;
                    break;
                case 9:    // Tab key
                    break;
                case 33:
                    break;//up key
                case 34:
                    break;//down key

                case 35:
                    this.prevselectedItem = this._selectedItemindex();
                    this._removeListHover();
                        this.element.children("li").removeClass('e-active');
                        this._selectedItem = this._listSize - 1;
                        this._addListHover();
                   
                    e.preventDefault();
                    return false;
                    break;  // End Key

                case 36:
                    this.prevselectedItem = this._selectedItemindex();
                    this._removeListHover();
                        this.element.children("li").removeClass('e-active');
                        this._selectedItem = 0;
                        this._addListHover();
                    e.preventDefault();
                    return false;
                    break;  // Home Key
                case 18:    // Alt key
                    break;
                case 32:    // space Key
                    e.preventDefault();
                    return false;
                    break;
            }
            }
        },
		_onmousedown:function (){
		this._on($(window), "mouseup", this._onScroll);
		},
        //scroll event
        _onScroll: function () {
		if(this.model.enableloadOnDemand && !this.model.enablevirtualScrolling){
		if(this.scrollerObj.scrollTop() >0){
            var _query = ej.Query();
			var listitemheight = $(this.element.find('li')[0]).outerHeight();
			var end =(this.scrollerObj.scrollTop()/listitemheight) +10;
		    var start = (this.scrollerObj.scrollTop()/listitemheight) ;
			this.realUllength=Math.floor((this.scrollerObj.scrollTop())/listitemheight);
            _query.range(start, end);
                var groupedList = ej.DataManager(this.listitems).executeLocal(_query);
				
				this._loadlist(groupedList);
                this._refreshItems();
			}
				}
			else if(this.model.enablevirtualScrolling)	{
			if(this.scrollerObj.model.height + this.scrollerObj.scrollTop() == this.scrollerObj._v.sScrollDim){
			   var st = this.scrollerObj.scrollTop();
			if(st > this.lastScrollTop){
				    var _query = ej.Query();
            var end = this.element.children().length + 10;
            var start = this.element.children().length;
            _query.range(start, end);
                var groupedList = ej.DataManager(this.listitems).executeLocal(_query);
				this.realUllength=start;
                this._loadlist(groupedList);
                if (arguments[0].originalEvent.detail < 0) {
                    this.displayeditem -= 5;
                }
                else{
                    this.displayeditem += 5;
                }
                this._refreshItems();
                this.listContainer.css("height", "auto");
                this.listContainer.css({ "display": "block" });  // For get the height of the popup
                this.scrollerObj.model.height = this.listContainer.height();
                this.scrollerObj.refresh();
                this.scrollerObj.option("scrollTop", this._calcScrollTop());
                this.listContainer.css({ "height": "auto" });
			
			}
			 this.lastScrollTop = this.scrollerObj.scrollTop();
			 }
			}
			this._off($(window), "mouseup", this._onScroll);
        },
       
        //trigger client side events
      
        _onDragStarts: function (data) {
            return this._trigger("itemDragStart", data);
        },
        _onDrag: function (data) {
            return this._trigger("itemDrag", data);
        },
        _onDragStop: function (data) {
            return this._trigger("itemDragStop", data);
        },
        _onDropped: function (data) {
            return this._trigger("itemDropped", data);
        },


        //-------------------- Event Wire-up -------------------------//
        _wireEvents: function () {
            this._on(this.listContainer, "keydown", this._OnKeyDown);
            this._on(this.listContainer, "DOMMouseScroll", this._onScroll);
			this._on($(".e-vup"), "mousedown", this._onScroll);
			this._on($(".e-vhandlespace"), "mousedown", this._onmousedown);
			this._on($(".e-vhandle"), "mouseup", this._onScroll);
			this._on($(".e-vdown"), "mousedown", this._onScroll);
			this._on(this.listContainer, "mousewheel", this._onScroll);
            this._enableDargDrop();
        }
    });
})(jQuery, Syncfusion);;
/**
* @fileOverview Plugin to style the Html Button elements
* @copyright Copyright Syncfusion Inc. 2001 - 2013. All rights reserved.
*  Use of this code is subject to the terms of our license.
*  A copy of the current license can be obtained at any time by e-mailing
*  licensing@syncfusion.com. Any infringement will be prosecuted under
*  applicable laws. 
* @version 12.1 
* @author <a href="mailto:licensing@syncfusion.com">Syncfusion Inc</a>
*/
(function ($, ej, undefined) {
	/**
    * @namespace ej
	* @classdesc Custom Design for Html Textbox Control.
	* @class ejTextBoxes
	* @requires jQuery
	* @requires ej.core.js
    * @requires jquery.globalize.js
    * @requires globalize.cultures.min.js
	* @requires ej.editor.js
	* @example 
	* &lt;input id="numeric" type="text" /&gt; <br> 
	* &lt;input id="currency" type="text" /&gt; <br> 
	* &lt;input id="percentage" type="text" /&gt; <br> 
	* &lt;script&gt;<br>
	* // Create Textbox Editors <br>
    * $('#numeric').ejNumericTextbox({value:10}); <br>	
	* $('#currency').ejCurrencyTextbox({value:1000}); <br>
	* $('#percentage').ejPercentageTextbox({value:100}); <br>
	* &lt;/script&gt;
	*/

    ej.widget(
    {
        "ejNumericTextbox": ["ej.NumericTextbox", "e-numerictextbox"],
        "ejPercentageTextbox": ["ej.PercentageTextbox", "e-percentagetextbox"],
        "ejCurrencyTextbox": ["ej.CurrencyTextbox", "e-currencytextbox"]
    },
    {
        // widget element will be automatically set in this
        element: null,
        // user defined model will be automatically set in this
        model: null,
        validTags: ["input"],
        _setFirst: false,
        type: "editor",

        // default model
        defaults: {
		/**		
			* Specifies the width of the editor.
			* @default 143pixel
			* @type {String}
			* @example 
			* &lt;input id="numeric" type="text" /&gt; <br> 
			* &lt;input id="currency" type="text" /&gt; <br> 
			* &lt;input id="percentage" type="text" /&gt; <br> 
			* &lt;script&gt;<br>
			* //To set width API value during initialization  
			* 	$("#numeric").ejNumericTextbox({ width: "143px", value:5 });	
			* 	$("#currency").ejCurrencyTextbox({ width: "143px", value:55 });
			* 	$("#percentage").ejPercentageTextbox({ width: "143px", value:555 });			
			* &lt;/script&gt;
			* @memberof ejTextBoxes
			* @instance
			*/
            width: "",
			
			/**		
			* Specifies the height of the editor.
			* @default 30pixel
			* @type {String}
			* @example 
			* &lt;input id="numeric" type="text" /&gt; <br> 
			* &lt;input id="currency" type="text" /&gt; <br> 
			* &lt;input id="percentage" type="text" /&gt; <br> 
			* &lt;script&gt;<br>
			* //To set height API value during initialization  
			* 	$("#numeric").ejNumericTextbox({ height: "30px", value:5  });	
			* 	$("#currency").ejCurrencyTextbox({ height: "30px", value:55  });
			* 	$("#percentage").ejPercentageTextbox({ height: "30px", value:555  });			 
			* &lt;/script&gt;
			* @memberof ejTextBoxes
			* @instance
			*/
			
            height: "",
			
			/**		
			* Specifies the value of the editor.
			* @default 0
			* @type {number}
			* @example 
			* &lt;input id="numeric" type="text" /&gt; <br> 
			* &lt;input id="currency" type="text" /&gt; <br> 
			* &lt;input id="percentage" type="text" /&gt; <br> 
			* &lt;script&gt;<br>
			* //To set value API value during initialization  
			* 	$("#numeric").ejNumericTextbox({ value: 10 });	
			* 	$("#currency").ejCurrencyTextbox({ value: 10 });
			* 	$("#percentage").ejPercentageTextbox({ value: 10 });			 
			* &lt;/script&gt;
			* @memberof ejTextBoxes
			* @instance
			*/
			
            value: null,
						
			/**		
			* Specifies the name of the editor.
			* @default Sets id as name if it is null.
			* @type {string}
			* @example 
			* &lt;input id="numeric" type="text" /&gt; <br> 
			* &lt;input id="currency" type="text" /&gt; <br> 
			* &lt;input id="percentage" type="text" /&gt; <br> 
			* &lt;script&gt;<br>
			* //To set name API value during initialization  
			* 	$("#numeric").ejNumericTextbox({ name: "numeric", value:5  });	
			* 	$("#currency").ejCurrencyTextbox({ name: "currency", value:55  });
			* 	$("#percentage").ejPercentageTextbox({ name: "percentage", value:500  });			 
			* &lt;/script&gt;
			* @memberof ejTextBoxes
			* @instance
			*/
			
            name: null,
			
			/**		
			* Specifies the minValue of the editor.
			* @default 0 for CurrencyTextbox
			* @default null for NumericTextbox
			* @default null for PercentageTextbox
			* @type {number}
			* @example 
			* &lt;input id="numeric" type="text" /&gt; <br> 
			* &lt;input id="currency" type="text" /&gt; <br> 
			* &lt;input id="percentage" type="text" /&gt; <br> 
			* &lt;script&gt;<br>
			* //To set minValue API value during initialization  
			* 	$("#numeric").ejNumericTextbox({ minValue: 50, value:55  });	
			* 	$("#currency").ejCurrencyTextbox({ minValue: 50, value:5  });
			* 	$("#percentage").ejPercentageTextbox({ minValue: 50, value:555  });			 
			* &lt;/script&gt;
			* @memberof ejTextBoxes
			* @instance
			*/
			
            minValue: -(Number.MAX_VALUE),
			
			/**		
			* Specifies the maxValue of the editor.
			* @default null for CurrencyTextbox
			* @default null for NumericTextbox
			* @default 100 for PercentageTextbox
			* @type {number}
			* @example 
			* &lt;input id="numeric" type="text" /&gt; <br> 
			* &lt;input id="currency" type="text" /&gt; <br> 
			* &lt;input id="percentage" type="text" /&gt; <br> 
			* &lt;script&gt;<br>
			* //To set maxValue API value during initialization  
			* 	$("#numeric").ejNumericTextbox({ maxValue: 100, value:500  });	
			* 	$("#currency").ejCurrencyTextbox({ maxValue: 100, value:550  });
			* 	$("#percentage").ejPercentageTextbox({ maxValue: 100, value:50  });			 
			* &lt;/script&gt;
			* @memberof ejTextBoxes
			* @instance
			*/
			
            maxValue: Number.MAX_VALUE,
			
			/**		
			* Specifies the incrementStep of the editor.
			* @default 1		
			* @type {number}
			* @example 
			* &lt;input id="numeric" type="text" /&gt; <br> 
			* &lt;input id="currency" type="text" /&gt; <br> 
			* &lt;input id="percentage" type="text" /&gt; <br> 
			* &lt;script&gt;<br>
			* //To set incrementStep API value during initialization  
			* 	$("#numeric").ejNumericTextbox({ incrementStep: 2, value:5  });	
			* 	$("#currency").ejCurrencyTextbox({ incrementStep: 2 , value:55 });
			* 	$("#percentage").ejPercentageTextbox({ incrementStep: 2, value:50  });			 
			* &lt;/script&gt;
			* @memberof ejTextBoxes
			* @instance
			*/
			
            incrementStep: 1,
			
			/**		
			* Specifies the decimalPlaces of the editor.
			* @default 0		
			* @type {number}
			* @example 
			* &lt;input id="numeric" type="text" /&gt; <br> 
			* &lt;input id="currency" type="text" /&gt; <br> 
			* &lt;input id="percentage" type="text" /&gt; <br> 
			* &lt;script&gt;<br>
			* //To set decimalPlaces API value during initialization  
			* 	$("#numeric").ejNumericTextbox({ decimalPlaces: 2, value:5  });	
			* 	$("#currency").ejCurrencyTextbox({ decimalPlaces: 2 , value:5 });
			* 	$("#percentage").ejPercentageTextbox({ decimalPlaces: 2, value:5  });			 
			* &lt;/script&gt;
			* @memberof ejTextBoxes
			* @instance
			*/
			
            decimalPlaces: 0,
			
			/**		
			* Specify the CSS class to editor to achieve custom theme.
			* @default ""
			* @type {string}
			* @example 
			* &lt;input id="numeric" type="text" /&gt; <br> 
			* &lt;input id="currency" type="text" /&gt; <br> 
			* &lt;input id="percentage" type="text" /&gt; <br> 
			* &lt;script&gt;<br>
			* //To set cssClass API value during initialization  
			* 	$("#numeric").ejNumericTextbox({ cssClass: "gradient-lime" , value:5 });	
			* 	$("#currency").ejCurrencyTextbox({ cssClass: "gradient-lime", value:100  });
			* 	$("#percentage").ejPercentageTextbox({ cssClass: "gradient-lime", value:505  });			 
			* &lt;/script&gt;
			* @memberof ejTextBoxes
			* @instance
			*/
					
            cssClass: "",
			
			/**		
			* Specify the enablePersistence to editor to save current model value to browser cookies for state maintains
			* @default false
			* @type {boolean}
			* @example 
			* &lt;input id="numeric" type="text" /&gt; <br> 
			* &lt;input id="currency" type="text" /&gt; <br> 
			* &lt;input id="percentage" type="text" /&gt; <br> 
			* &lt;script&gt;<br>
			* //To set enablePersistence API value during initialization  
			* 	$("#numeric").ejNumericTextbox({ enablePersistence: true, value:5  });	
			* 	$("#currency").ejCurrencyTextbox({ enablePersistence: true, value:5  });
			* 	$("#percentage").ejPercentageTextbox({ enablePersistence: true, value:5  });			 
			* &lt;/script&gt;
			* @memberof ejTextBoxes
			* @instance
			*/
			
            enablePersistence: false,
			
			/**		
			* Specify the showSpinButton to editor 
			* @default true
			* @type {boolean}
			* @example 
			* &lt;input id="numeric" type="text" /&gt; <br> 
			* &lt;input id="currency" type="text" /&gt; <br> 
			* &lt;input id="percentage" type="text" /&gt; <br> 
			* &lt;script&gt;<br>
			* //To set showSpinButton API value during initialization  
			* 	$("#numeric").ejNumericTextbox({ showSpinButton: false, value:5  });	
			* 	$("#currency").ejCurrencyTextbox({ showSpinButton: false, value:55  });
			* 	$("#percentage").ejPercentageTextbox({ showSpinButton: false, value:580  });			 
			* &lt;/script&gt;
			* @memberof ejTextBoxes
			* @instance
			*/
			
            showSpinButton: true,
			
			/**		
			* Specify the Localization to editor 
			* @default en-US
			* @type {string}
			* @example 
			* &lt;input id="numeric" type="text" /&gt; <br> 
			* &lt;input id="currency" type="text" /&gt; <br> 
			* &lt;input id="percentage" type="text" /&gt; <br> 
			* &lt;script&gt;<br>
			* //To set culture API value during initialization  
			* 	$("#numeric").ejNumericTextbox({ locale: "en-US", value:5  });	
			* 	$("#currency").ejCurrencyTextbox({ locale: "en-US", value:5000  });
			* 	$("#percentage").ejPercentageTextbox({ locale: "en-US", value:455  });			 
			* &lt;/script&gt;
			* @memberof ejTextBoxes
			* @instance
			*/
			
            locale: "en-US",
			
			/**		
			* Specify the strictMode to editor 
			* @default false
			* @type {booelan}
			* @example 
			* &lt;input id="numeric" type="text" /&gt; <br> 
			* &lt;input id="currency" type="text" /&gt; <br> 
			* &lt;input id="percentage" type="text" /&gt; <br> 
			* &lt;script&gt;<br>
			* //To set enableStrictMode API value during initialization  
			* 	$("#numeric").ejNumericTextbox({ enableStrictMode: true, value:5  });	
			* 	$("#currency").ejCurrencyTextbox({ enableStrictMode: true, value:55  });
			* 	$("#percentage").ejPercentageTextbox({ enableStrictMode: true, value:555  });			
			* &lt;/script&gt;
			* @memberof ejTextBoxes
			* @instance
			*/
			
            enableStrictMode: false,
			
			/**		
			* Specify the rounded Corner to editor 
			* @default false
			* @type {boolean}
			* @example 
			* &lt;input id="numeric" type="text" /&gt; <br> 
			* &lt;input id="currency" type="text" /&gt; <br> 
			* &lt;input id="percentage" type="text" /&gt; <br> 
			* &lt;script&gt;<br>
			* //To set showRoundedCorner API value during initialization  
			* 	$("#numeric").ejNumericTextbox({ showRoundedCorner: true, value:5  });	
			* 	$("#currency").ejCurrencyTextbox({ showRoundedCorner: true , value:5 });
			* 	$("#percentage").ejPercentageTextbox({ showRoundedCorner: true, value:5 });			 
			* &lt;/script&gt;
			* @memberof ejTextBoxes
			* @instance
			*/
			
            showRoundedCorner: false,
			
			/**		
			* Specify the readOnly to editor 
			* @default false
			* @type {boolean}
			* @example 
			* &lt;input id="numeric" type="text" /&gt; <br> 
			* &lt;input id="currency" type="text" /&gt; <br> 
			* &lt;input id="percentage" type="text" /&gt; <br> 
			* &lt;script&gt;<br>
			* //To set readOnly API value during initialization  
			* 	$("#numeric").ejNumericTextbox({ readOnly: true , value:5 });	
			* 	$("#currency").ejCurrencyTextbox({ readOnly: true , value:5 });
			* 	$("#percentage").ejPercentageTextbox({ readOnly: true , value:5 });			 
			* &lt;/script&gt;
			* @memberof ejTextBoxes
			* @instance
			*/
			
            readOnly: false,
			
			/**		
			* Specify the editor control state.
			* @default true
			* @type {boolean}
			* @example 
			* &lt;input id="numeric" type="text" /&gt; <br> 
			* &lt;input id="currency" type="text" /&gt; <br> 
			* &lt;input id="percentage" type="text" /&gt; <br> 
			* &lt;script&gt;<br>
			* //To set enabled API value during initialization  
			* 	$("#numeric").ejNumericTextbox({ enabled: true, value:1200  });	
			* 	$("#currency").ejCurrencyTextbox({ enabled: true , value:50 });
			* 	$("#percentage").ejPercentageTextbox({ enabled: true, value:100  });			 
			* &lt;/script&gt;
			* @memberof ejTextBoxes
			* @instance
			*/
			
            enabled: true,
			
			/**		
			* Specify the Right to Left Direction to editor.
			* @default false
			* @type {boolean}
			* @example 
			* &lt;input id="numeric" type="text" /&gt; <br> 
			* &lt;input id="currency" type="text" /&gt; <br> 
			* &lt;input id="percentage" type="text" /&gt; <br> 
			* &lt;script&gt;<br>
			* //To set enableRTL API value during initialization  
			* 	$("#numeric").ejNumericTextbox({ enableRTL: true, value:5  });	
			* 	$("#currency").ejCurrencyTextbox({ enableRTL: true , value:45 });
			* 	$("#percentage").ejPercentageTextbox({ enableRTL: true, value:567  });			 
			* &lt;/script&gt;
			* @memberof ejTextBoxes
			* @instance
			*/
			
            enableRTL: false,
			
			/**		
			* Specify the watermark text to editor.
			* @default ""
			* @type {string}
			* @example 
			* &lt;input id="numeric" type="text" /&gt; <br> 
			* &lt;input id="currency" type="text" /&gt; <br> 
			* &lt;input id="percentage" type="text" /&gt; <br> 
			* &lt;script&gt;<br>
			* //To set enableRTL API value during initialization  
			* 	$("#numeric").ejNumericTextbox({ watermarkText: "Enter the value" });	
			* 	$("#currency").ejCurrencyTextbox({ watermarkText: "Enter the currency value" });
			* 	$("#percentage").ejPercentageTextbox({ watermarkText: "Enter the percentage" });			 
			* &lt;/script&gt;
			* @memberof ejTextBoxes
			* @instance
			*/
			
            watermarkText: "",
			
			 /**     
			 * Fires after editor control is value is changed.
			 * @event
			 * @name ejTextBoxes#change 
			 * @param {Object} argument Event parameters from editors.    			 
			 * @param {boolean}  argument.cancel if the event should be cancelled; otherwise, false.
			 * @param {object}  argument.model returns the corresponding editor model.
			 * @param {string}  argument.type returns the name of the event.	
			 * @param {number}  argument.value returns the corresponding editor control value.			 
			 * @example 
			 * &lt;input id="numeric" type="text" /&gt; <br> 
			 * &lt;input id="currency" type="text" /&gt; <br> 
			 * &lt;input id="percentage" type="text" /&gt; <br> 
			 * &lt;script&gt;<br>
			 * //change event for editors
             * $("#numeric").ejNumericTextbox({
			 *	  value:10,	
             *    change: function (args) {}
             * });
			 * $("#currency").ejCurrencyTextbox({
			 *	  value:100,	
             *    change: function (args) {}
             * });
			 * $("#percentage").ejPercentageTextbox({
			 *	  value:1000,	
             *    change: function (args) {}
             * });
			 * &lt;/script&gt;
			 * @memberof ejTextBoxes
			 * @instance
			 */		
			
            change: null,			
			
			 /**     
			 * Fires after editor control is focused.
			 * @event
			 * @name ejTextBoxes#focusIn 
			 * @param {Object} argument Event parameters from editors.    			 
			 * @param {boolean}  argument.cancel if the event should be cancelled; otherwise, false.
			 * @param {object}  argument.model returns the corresponding editor model.
			 * @param {string}  argument.type returns the name of the event.
             * @param {number}  argument.value returns the corresponding editor control value.
			 * @example 
			 * &lt;input id="numeric" type="text" /&gt; <br> 
			* &lt;input id="currency" type="text" /&gt; <br> 
			* &lt;input id="percentage" type="text" /&gt; <br> 
			* &lt;script&gt;<br>
			 * //focusIn event for editors
             * $("#numeric").ejNumericTextbox({
			 *	  value:20,	
             *    focusIn: function (args) {}
             * });
			 * $("#currency").ejCurrencyTextbox({
			 *	  value:200,	
             *    focusIn: function (args) {}
             * });
			 * $("#percentage").ejPercentageTextbox({
			 *	  value:2000,	
             *    focusIn: function (args) {}
             * });
			 * &lt;/script&gt;
			 * @memberof ejTextBoxes
			 * @instance
			 */		
			
            focusIn: null,
			
			/**     
			 * Fires after editor control is loss the focus.
			 * @event
			 * @name ejTextBoxes#focusOut 
			 * @param {Object} argument Event parameters from editors.    			 
			 * @param {boolean}  argument.cancel if the event should be cancelled; otherwise, false.
			 * @param {object}  argument.model returns the corresponding editor model.
			 * @param {string}  argument.type returns the name of the event.	
             * @param {number}  argument.value returns the corresponding editor control value.
			 * @example 
			 * &lt;input id="numeric" type="text" /&gt; <br> 
			* &lt;input id="currency" type="text" /&gt; <br> 
			* &lt;input id="percentage" type="text" /&gt; <br> 
			* &lt;script&gt;<br>
			 * //focusOut event for editors
             * $("#numeric").ejNumericTextbox({
			 *	  value:50,	
             *    focusOut: function (args) {}
             * });
			 * $("#currency").ejCurrencyTextbox({
			 *	  value:505,	
             *    focusOut: function (args) {}
             * });
			 * $("#percentage").ejPercentageTextbox({
			 *	  value:1500,	
             *    focusOut: function (args) {}
             * });
			 * &lt;/script&gt;

			 * @memberof ejTextBoxes
			 * @instance
			 */					
            focusOut: null,
			  /**     
          * Fires after editor control is created.
          * @event
          * @name ejTextBoxes#create 	
          * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
          * @param {object}  argument.model returns the editor model
          * @param {string}  argument.type returns the name of the event
          * @example 
		  * &lt;input id="numeric" type="text" /&gt; <br> 
			* &lt;input id="currency" type="text" /&gt; <br> 
			* &lt;input id="percentage" type="text" /&gt; <br> 
			* &lt;script&gt;<br>
			 * //create event for editors
             * $("#numeric").ejNumericTextbox({
			 *	  value:50,	
             *    create: function (args) {}
             * });
			 * $("#currency").ejCurrencyTextbox({
			 *	  value:505,	
             *    create: function (args) {}
             * });
			 * $("#percentage").ejPercentageTextbox({
			 *	  value:1500,	
             *    create: function (args) {}
             * });
			 * &lt;/script&gt;			 
          * @memberof ejTextBoxes
          * @instance
          */
            create: null,
			 /**     
          * Fires when the editor is destroyed successfully.
          * @event
          * @name ejTextBoxes#destroy 	
          * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
          * @param {object}  argument.model returns the editor model
          * @param {string}  argument.type returns the name of the event
          * @example 
		  * &lt;input id="numeric" type="text" /&gt; <br> 
			* &lt;input id="currency" type="text" /&gt; <br> 
			* &lt;input id="percentage" type="text" /&gt; <br> 
			* &lt;script&gt;<br>
			 * //destroy event for editors
             * $("#numeric").ejNumericTextbox({
			 *	  value:50,	
             *    destroy: function (args) {}
             * });
			 * $("#currency").ejCurrencyTextbox({
			 *	  value:505,	
             *    destroy: function (args) {}
             * });
			 * $("#percentage").ejPercentageTextbox({
			 *	  value:1500,	
             *    destroy: function (args) {}
             * });
			 * &lt;/script&gt;			 
          * @memberof ejTextBoxes
          * @instance
          */
            destroy: null
        },
		/**
         * Specify the data types for default properties 
		 * @private
         */
        dataTypes: {
            minValue: "number",
            maxValue: "number",
            incrementStep: "number",
            decimalPlaces: "number",
            showSpinButton: "boolean",
            enableStrictMode: "boolean",
            showRoundedCorner: "boolean",
            enableRTL: "boolean",
            locale: "string",
            watermarkText: "string",
            cssClass: "string",
            readOnly: "boolean",
            enabled: "boolean"
        },
        observables: ["value"],
		 /**
         * To configure the properties at runtime using SetModel		
		 * @private
         */
        _setModel: function (jsondata) {
            var validate = false;
            if (!this.model.enabled && !jsondata["enabled"]) return;
            for (var key in jsondata) {
                switch (key) {
                    case "value":
                        if (ej.isNullOrUndefined(jsondata["minValue"]) && ej.isNullOrUndefined(jsondata["maxValue"])) {
                            this._setValue(jsondata[key]);
                            jsondata[key] = this.model.value;
                        }
                        else {
                            this.model.value = this._checkNumValue(jsondata[key]);
                            this._localizedFormat();
                            this._raiseChangeEvent();
                            validate = true;
                        }
                        break;
                    case "enableRTL": this._enableRTL(jsondata[key]); break;
                    case "width": this._setWidth(jsondata[key]); break;
                    case "height": this._setHeight(jsondata[key]); break;
                    case "minValue":
                        if (isNaN(jsondata[key])) return;
                        this.model.minValue = parseFloat(jsondata[key].toFixed(this.model.decimalPlaces));
                        validate = true;
                        break;
                    case "maxValue":
                        if (isNaN(jsondata[key])) return;
                        this.model.maxValue = parseFloat(jsondata[key].toFixed(this.model.decimalPlaces));
                        validate = true;
                        break;
                    case "incrementStep":
                        if (isNaN(jsondata[key])) return;
                        this.model.incrementStep = parseFloat(jsondata[key].toFixed(this.model.decimalPlaces));
                        break;
                    case "showSpinButton": this._showSpin(jsondata[key]); break;
                    case "showRoundedCorner": this._roundedCorner(jsondata[key]); break;
                    case "locale": this._setLocalize(jsondata[key]); break;
                    case "decimalPlaces": this._setDecimal(jsondata[key]); break;
                    case "cssClass": this._setSkin(jsondata[key]); break;
                    case "readOnly": this._setReadOnly(jsondata[key]); break;
                    case "enabled": if (jsondata[key]) this.enable(); else this.disable(); break;
                    case "watermarkText":
                        this.model.watermarkText = jsondata[key]; this._setWaterMark(); break;
                }
            }
            if (validate) {
                this._validateMinMaxValue(true);
                jsondata["value"] = this.model.value;
                jsondata["maxValue"] = this.model.maxValue;
                jsondata["minValue"] = this.model.minValue;
                if (this.model.minValue != -(Number.MAX_VALUE)) this._startValue = this.model.minValue;
                this.element.attr({ 'aria-valuemin': this.model.minValue, 'aria-valuemax': this.model.maxValue, 'aria-valuenow': this.model.value });
            }
            this._checkErrorClass();
        },
		 /**
        * destroy the editor widgets
		* all events bound using this._on will be unbind automatically and bring the control to pre-init state.
		* @alias destroy
		* @return jQuery
		* @example 
		* &lt;input id="numeric" type="text" /&gt; <br> 
		* &lt;input id="currency" type="text" /&gt; <br> 
		* &lt;input id="percentage" type="text" /&gt; <br> 
		* &lt;script&gt;
		* $("#numeric").ejNumericTextbox({value:5});
		* $("#currency").ejCurrencyTextbox({value:55});
		* $("#percentage").ejPercentageTextbox({value:555});
		* // Create Editors
		* var numObj = $("#numeric").data("ejNumericTextbox");
		* var curObj = $("#currency").data("ejCurrencyTextbox");
		* var perObj = $("#percentage").data("ejPercentageTextbox");
		* numObj.destroy(); // destroy the numericTextbox
		* curObj.destroy(); // destroy the currencyTextbox
		* perObj.destroy(); // destroy the percentagTextbox
		* &lt;/script&gt;
		* @example 
		* &lt;input id="numeric" type="text" /&gt; <br> 
		* &lt;input id="currency" type="text" /&gt; <br> 
		* &lt;input id="percentage" type="text" /&gt; <br> 
		* &lt;script&gt;
		* $("#numeric").ejNumericTextbox({value:5});
		* $("#currency").ejCurrencyTextbox({value:55});
		* $("#percentage").ejPercentageTextbox({value:555});
		* // enable the editors
		* $("#numeric").ejNumericTextbox("destroy");
		* $("#currency").ejCurrencyTextbox("destroy");
		* $("#percentage").ejPercentageTextbox("destroy");		
		* &lt;/script&gt;
		* @memberof ejTextBoxes
		* @instance
         */	       
        // all events bound using this._on will be unbind automatically
        _destroy: function () {
            if (this.wrapper) {
                this.element.insertBefore(this.wrapper);
                this.wrapper.remove();
            }
            if (this.model.name) this.element.removeAttr("name");
            this.element.val("").removeClass('e-input e-watermark').empty();
        },
		
		 /**
         * Initialize the editors widget
		 * @private
         */		
        // constructor function
        _init: function () {
            if (this.element.is("input") && (this.element.is("input[type=text]") || !this.element.attr('type'))) {
                this._renderControl();
                this._setValues();
                this._wireEvents();
                this._initObjects();
            }
            else {
                this._destroy();
                return false;
            }
        },

        // -----------------------------------private function----------------------------------//
		
		/**
         * To configure the editors value		
		 * @private
         */	 		
        _setValues: function () {
            this._id = this.element[0].id;
            this._textBox = this.element[0];
            this._error = false;
            this._timeout = null;
            this.isValidState = true;
            this._allowkeyboard = true;
            this._validateOnType = false;
            this._focused = false;
            this._startValue = 0;
            if (this.sfType === "ej.CurrencyTextbox" && this.model.minValue == -(Number.MAX_VALUE))
                this.model.minValue = 0;
            this.model.minValue = parseFloat(this.model.minValue.toFixed(this.model.decimalPlaces));
            this.model.maxValue = parseFloat(this.model.maxValue.toFixed(this.model.decimalPlaces));
            if (this.model.minValue != -(Number.MAX_VALUE)) this._startValue = this.model.minValue;
            this._localizedFormat();
            this._validateMinMaxValue(true);
            this.model.watermarkText = this._checkWaterMark(this.model.watermarkText);
            this._percentSymbol = Globalize.culture(this.model.locale).numberFormat.percent.symbol;
            this._currencySymbol = Globalize.culture(this.model.locale).numberFormat.currency.symbol;
        },
		
		 /**
         * Render Section For DifferentTypes		
		 * @private
         */	   
        _renderControl: function () {
            var editorwidget = $(document.createElement('span')).addClass('e-widget');
            var editorwidgetInner = $(document.createElement('span')).addClass('e-in-wrap e-box');
            if (this.sfType === "ej.NumericTextbox")
                editorwidget.addClass('e-numeric');
            else if (this.sfType === "ej.PercentageTextbox")
                editorwidget.addClass('e-percent');
            else if (this.sfType === "ej.CurrencyTextbox")
                editorwidget.addClass('e-currency');
            editorwidget.append(editorwidgetInner).insertAfter(this.element);
            editorwidgetInner.append(this.element);
            this._hiddenInput = ej.buildTag("input#" + this._id + "_hidden", "", {}, { type: "hidden" }).insertBefore(this.element);
            this.model.name = this.element.attr("name") != null ? this.element.attr("name") : (this.model.name != null ? this.model.name : this.element[0].id);
            if(this.element.attr("name") == null)
                this.element.attr({'name': this.model.name});
            this.element.addClass('e-input');
			this.model.value = ej.isNullOrUndefined(this.model.value) ? this._checkNumValue(this.element[0].value) : this._checkNumValue(this.model.value);
            this._hiddenInput.attr({ 'name': this.model.name, 'value': this.model.value }).addClass('e-input');
            this.element.attr({ 'role': 'spinbutton', 'aria-valuemin': this.model.minValue, 'aria-valuemax': this.model.maxValue, 'aria-valuenow': this.model.value, 'tabindex': '0', 'aria-live': 'assertive', "value": this.model.value });
            var spinbutton = $('<span class="e-select"><span class="e-spin e-spin-up " role="button" "aria-label"="Increase Value" /><span class="e-spin e-spin-down" role="button" "aria-label"="Decrease Value" /></span>');
            var spinimg = ej.buildTag('span.e-icon e-arrow').attr('role','presentation');
            editorwidgetInner.append(spinbutton);
            spinbutton.find('.e-spin').append(spinimg);
            this.innerWrap = this.element.parent();
            this.wrapper = this.innerWrap.parent();
            this.spin = this.wrapper.find('.e-select');
            this.spinUp = this.wrapper.find('.e-spin-up');
            this.spinDown = this.wrapper.find('.e-spin-down');
            this._setWidth(this.model.width);
            this._setHeight(this.model.height);
            if (this.model.cssClass != "") this._setSkin(this.model.cssClass);
            this._showSpin(this.model.showSpinButton);
            if (this.model.showRoundedCorner) this._roundedCorner(this.model.showRoundedCorner);
            if (this.model.enableRTL) this._enableRTL(this.model.enableRTL);
            if (this.model.readOnly) this._setReadOnly(this.model.readOnly);
            if (!this.model.enabled) this.disable();
            this.wrapper = editorwidget;
        },
		
		/**
         * To initialize the editors		
		 * @private
         */	 
        _initObjects: function () {
            this._preVal = this._numberValue();
            if (this._textBox.value == "" && this.model.watermarkText != "")
                this._setWaterMark();
            if (this.sfType === "ej.PercentageTextbox" && this._textBox.value != "")
                this._appendPercentSymbol(this._textBox.value);
            else if (this.sfType === "ej.CurrencyTextbox" && this._textBox.value != "")
                this._appendCurrencySymbol(this._textBox.value);
            if (this.model.value == "") {
                this.isValidState = true;
                this._hiddenInput.val("");
            }
            else if ((this.model.value < this.model.minValue) || (this.model.value > this.model.maxValue))
                this.isValidState = false;
            this._checkErrorClass();
        },
		
		 /**
         * To enable or disable the show spin button in editors.
		 * @private
         */	
        _showSpin: function (value) {
            if (!value) {
                if (this.spin) {
                    this.spin.hide();
                    this.innerWrap.removeClass('e-padding');
                }
                this._unbindSpinEvents();
            }
            else {
                if (this.spin) {
                    this.spin.show();
                    this.innerWrap.addClass('e-padding');
                }
                this._bindSpinEvents();
            }
        },
		
		 /**
         * To enable or disable the rounded corner behaviour 		
		 * @private
         */	
        _roundedCorner: function (value) {
            if (value && !this.innerWrap.hasClass('e-corner-all'))
                this.innerWrap.addClass('e-corner-all');
            else if (this.innerWrap.hasClass('e-corner-all'))
                this.innerWrap.removeClass('e-corner-all');
        },
		
		 /**
         * To enable or disable the Right to Left Direction behaviour 		
		 * @private
         */	
        _enableRTL: function (enableRTL) {
            if (enableRTL) {
                if (this.spin) this.wrapper.addClass("e-rtl");
                else this.element.addClass("e-rtl");
            }
            else {
                if (this.spin) this.wrapper.removeClass("e-rtl");
                else this.element.removeClass("e-rtl");
            }
        },
		
		 /**
         * To configure editors width		
		 * @private
         */	
        _setWidth: function (value) {
            this.wrapper.width(value);
        },
		
		 /**
         * To configure editors height		
		 * @private
         */	
        _setHeight: function (value) {
            this.wrapper.height(value);
        },
		
		 /**
         * To configure the custom theme for editors using cssClass property		
		 * @private
         */	 
        _setSkin: function (skin) {
            this.wrapper.removeClass(this.model.cssClass);
            this.wrapper.addClass(skin);
        },
		
		 /**
         * To configure value to editors		
		 * @private
         */	 
        _setValue: function (value) {
            this.model.value = this._checkNumValue(value);
            this._validateMinMaxValue(false);
            if ((this.model.minValue <= this.model.value) && (this.model.value <= this.model.maxValue)) {
                this.isValidState=true;
            } else
                this.isValidState=false;
            this._checkErrorClass();
            this._localizedFormat();
            this._raiseChangeEvent();
        },
		
		 /**
         * To configure localization to editors		
		 * @private
         */	 
        _setLocalize: function (val) {
            this.model.locale = val;
            this._percentSymbol = Globalize.culture(this.model.locale).numberFormat.percent.symbol;
            this._currencySymbol = Globalize.culture(this.model.locale).numberFormat.currency.symbol;
            this._localizedFormat();
        },
		
		 /**
         * To configure decimal property to allow decimal values to editors		
		 * @private
         */	 
        _setDecimal: function (val) {
            this.model.decimalPlaces = val;
            this._localizedFormat();
        },
		
		/**
         * To check whether value within min max range.		
		 * @private
         */	
        _validateMinMaxValue: function (fromMinMax) {
            var valChange = false;
            if (this.model.minValue > this.model.maxValue) this.model.minValue = this.model.maxValue;
            if (this.model.value != "" && this.model.minValue > this.model.value) {
                if (this.model.enableStrictMode != true) {
                    this.isValidState = true;
                    this._startValue = this.model.value = this.model.minValue;
                }
                else this.isValidState = false;
                valChange = true;
            }
            else if (this.model.value != "" && this.model.maxValue < this.model.value) {
                if (this.model.enableStrictMode != true) {
                    this.isValidState = true;
                    this.model.value = this.model.maxValue;
                }
                else this.isValidState = false;
                valChange = true;
            }
            else this.isValidState = true;
            if (this.model.minValue == this.model.maxValue) this._startValue = this.model.minValue;
            if ((valChange && fromMinMax)) {
                this._hiddenInput.val(this.model.value);
                this._localizedFormat();
                this._raiseChangeEvent();
            }
        },

		/**
         * To configure localization format to editors		
		 * @private
         */	
        _localizedFormat: function () {
            this._wipeWaterMark();
            this._textBox.value = Globalize.format((this.model.value), "n" + this.model.decimalPlaces, this.model.locale);
            if (!this._focused && this._textBox.value != "") {
                if (this.sfType === "ej.PercentageTextbox") this._appendPercentSymbol(this._textBox.value);
                else if (this.sfType === "ej.CurrencyTextbox") this._appendCurrencySymbol(this._textBox.value);
            }
        },
		
		/**
         * To check whether value is proper format in editors		
		 * @private
         */	
        _checkNumValue: function (value) {
            if (typeof value == "string" && !isNaN(Globalize.parseFloat(value, this.model.locale))) {
                value = Globalize.parseFloat(value, this.model.locale);
                return parseFloat(value);
            }
            else if ((typeof value == "number") && !isNaN(value))
                return value;
            else return "";
        },
		
		/**
         * To check whether waterMark in editors		
		 * @private
         */	
        _checkWaterMark: function (val) {
            if (!val)
                return "";
            else return val;
        },
		
		 /**
         * To enable or disable the readOnly behaviour 		
		 * @private
         */	
        _setReadOnly: function (bool) {
            if (bool) this.element.attr("readonly", true);
            else this.element.removeAttr("readonly");
        },
		
		/**
         * To configure the editors waterMark text		
		 * @private
         */	
        /* Set Water Mark Text */
        _setWaterMark: function () {
            this.element.addClass("e-watermark");
            this._textBox.value = this.model.watermarkText;
        },
		
		/**
         * To remove the editors waterMark text		
		 * @private
         */	
        /* Clear Water Mark Text */
        _wipeWaterMark: function () {
            this.element.removeClass("e-watermark");
        },
		
		/**
         * To configure the editors value selection range. 		
		 * @private
         */	
        _setSelectionRange: function (selectionStart, selectionEnd) {
            var input = this._textBox;
            try {
                if (input.setSelectionRange) {
                    input.setSelectionRange(selectionStart, selectionEnd);
                    input.focus();
                }
                else if (input.createTextRange) {
                    var range = input.createTextRange();
                    _setselction(range);
                }
            }
            catch (e) {
                var control = this;
                window.setTimeout(function () {
                    document.body.focus();
                    control._textBox.select();
                    var range = document.selection.createRange();
                    _setselction(range);
                }, 1);
            }
            function _setselction(range) {
                range.collapse(true);
                range.moveEnd('character', selectionEnd);
                range.moveStart('character', selectionStart);
                range.select();
            }
        },
		
		/**
         * To obtain the editors value in selection range. 		
		 * @private
         */	
        _getSelection: function (value) {
            var oSel = null;
            if (document.selection) {
                oSel = document.selection.createRange();
                return (oSel.text == "" ? oSel.text : this._removeFormats(oSel.text));
            }
            else {
                if (value == null)
                    return this.model.value;
                else {
                    oSel = this._removeFormats(value.substring(this._textBox.selectionStart, this._textBox.selectionEnd));
                    return oSel;
                }
            }
        },
		
		/**
         * To calculate selection range position in editors value. 		
		 * @private
         */	
        _caretPosition: function () {
            var oField = this._textBox;
            // Initialize
            var iCaretPos = 0;
            // IE Support
            if (document.selection) {
                // Set focus on the element
                oField.focus();
                // To get cursor position, get empty selection range
                var oSel = document.selection.createRange();
                // Move selection start to 0 position
                oSel.moveStart('character', -oField.value.length);
                // The caret position is selection length
                iCaretPos = oSel.text.length;
            }
            // Firefox support
            else if (oField.selectionStart || oField.selectionStart == '0')
                iCaretPos = oField.selectionEnd;
            // Return results
            return (iCaretPos);
        },
		
		/**
         * To configure the percentage symbol in percentageTextbox value. 		
		 * @private
         */	
        _appendPercentSymbol: function (value) {
            if (this._percentSymbol) value = value.replace(this._percentSymbol, "");
            if (value.indexOf(Globalize.culture(this.model.locale).numberFormat.percent.symbol) < 0)
                this._textBox.value = value + Globalize.culture(this.model.locale).numberFormat.percent.symbol;
            this._percentSymbol = Globalize.culture(this.model.locale).numberFormat.percent.symbol;

        },
		
		/**
         * To configure the currency symbol in currencyTextbox value. 		
		 * @private
         */	
        _appendCurrencySymbol: function (value) {
            if (this._currencySymbol) value = value.replace(this._currencySymbol, "");
            if (value.indexOf(Globalize.culture(this.model.locale).numberFormat.currency.symbol) < 0)
                if (Number(value) < 0)
                    this._textBox.value = "(" + Globalize.culture(this.model.locale).numberFormat.currency.symbol + Math.abs(value) + ")";
                else
                    this._textBox.value = Globalize.culture(this.model.locale).numberFormat.currency.symbol + value;
            this._currencySymbol = Globalize.culture(this.model.locale).numberFormat.currency.symbol;

        },
		
		/**
         * To remove the localization format in editors. 		
		 * @private
         */	
        _removeFormats: function (val) {
            var grpSep = Globalize.culture(this.model.locale).numberFormat[","];
            if (val != null) {
                if (grpSep == ".")
                    return (val.toString().replace(/["."]/g, ""));
                else if (val.toString().match(new RegExp(grpSep, "g")))
                    return (val.toString().replace(new RegExp(grpSep, "g"), ""));
                else
                    return val;
            }
            else
                return null;
        },
		
		/**
         * To check whether error class added or not editors. 		
		 * @private
         */	
        _checkErrorClass: function (obj) {
            if (this.isValidState) this.wrapper.removeClass("e-error");
            else this.wrapper.addClass("e-error");
        },
        // -----------------------------------public function----------------------------------//
		/**
        * To enable the corresponding editors 		
		* @return jQuery
		* @example 
		* &lt;input id="numeric" type="text" /&gt; <br> 
		* &lt;input id="currency" type="text" /&gt; <br> 
		* &lt;input id="percentage" type="text" /&gt; <br> 
		* &lt;script&gt;
		* $("#numeric").ejNumericTextbox({value:10});
		* $("#currency").ejCurrencyTextbox({value:100});
		* $("#percentage").ejPercentageTextbox({value:1000});
		* // Create Editors
		* var numObj = $("#numeric").data("ejNumericTextbox");
		* var curObj = $("#currency").data("ejCurrencyTextbox");
		* var perObj = $("#percentage").data("ejPercentageTextbox");
		* numObj.enable(); // enable the numericTextbox
		* curObj.enable(); // enable the currencyTextbox
		* perObj.enable(); // enable the percentagTextbox
		* &lt;/script&gt;
		 @example 
		* &lt;input id="numeric" type="text" /&gt; <br> 
		* &lt;input id="currency" type="text" /&gt; <br> 
		* &lt;input id="percentage" type="text" /&gt; <br> 
		* &lt;script&gt;
		* $("#numeric").ejNumericTextbox({value:10});
		* $("#currency").ejCurrencyTextbox({value:100});
		* $("#percentage").ejPercentageTextbox({value:1000});
		* // enable the editors
		* $("#numeric").ejNumericTextbox("enable");
		* $("#currency").ejCurrencyTextbox("enable");
		* $("#percentage").ejPercentageTextbox("enable");		
		* &lt;/script&gt;
		* @memberof ejTextBoxes
		* @instance
         */	       		 
        enable: function () {
            this.model.enabled = true;
            this._textBox.disabled = false;
            this.element.removeAttr("disabled");
            this.element.removeClass('e-disable').attr({ "aria-disabled": false });
            this.wrapper.find(".e-select").removeClass('e-disable').attr({ "aria-disabled": false });
        },
		
		/**
        * To disable the corresponding editors 		
		* @return jQuery
		* @example 
		* &lt;input id="numeric" type="text" /&gt; <br> 
		* &lt;input id="currency" type="text" /&gt; <br> 
		* &lt;input id="percentage" type="text" /&gt; <br> 
		* &lt;script&gt;
		* $("#numeric").ejNumericTextbox({value:20});
		* $("#currency").ejCurrencyTextbox({value:400});
		* $("#percentage").ejPercentageTextbox({value:2000});
		* // Create Editors	
		* var numObj = $("#numeric").data("ejNumericTextbox");
		* var curObj = $("#currency").data("ejCurrencyTextbox");
		* var perObj = $("#percentage").data("ejPercentageTextbox");
		* numObj.disable(); // enable the numericTextbox
		* curObj.disable(); // enable the currencyTextbox
		* perObj.disable(); // enable the percentagTextbox
		* &lt;/script&gt;
		 @example 
		* &lt;input id="numeric" type="text" /&gt; <br> 
		* &lt;input id="currency" type="text" /&gt; <br> 
		* &lt;input id="percentage" type="text" /&gt; <br> 
		* &lt;script&gt;
		* $("#numeric").ejNumericTextbox({value:20});
		* $("#currency").ejCurrencyTextbox({value:400});
		* $("#percentage").ejPercentageTextbox({value:2000});
		* // enable the editors
		* $("#numeric").ejNumericTextbox("disable");
		* $("#currency").ejCurrencyTextbox("disable");
		* $("#percentage").ejPercentageTextbox("disable");		
		* &lt;/script&gt;
		* @memberof ejTextBoxes
		* @instance
         */	      
        disable: function () {
            this.model.enabled = false;
            this._textBox.disabled = true;
            this.element.attr("disabled", "disabled");
            this.element.addClass('e-disable').attr({ "aria-disabled": true });
            this.wrapper.find(".e-select").addClass('e-disable').attr({ "aria-disabled": true });
        },
		
		/**
        * To get value from corresponding editors 		
		* @return jQuery
		* @example 
		* &lt;input id="numeric" type="text" /&gt; <br> 
		* &lt;input id="currency" type="text" /&gt; <br> 
		* &lt;input id="percentage" type="text" /&gt; <br> 
		* &lt;script&gt;
		* $("#numeric").ejNumericTextbox({value:20});
		* $("#currency").ejCurrencyTextbox({value:500});
		* $("#percentage").ejPercentageTextbox({value:1000});
		* // Create Editors
		* var numObj = $("#numeric").data("ejNumericTextbox");
		* var curObj = $("#currency").data("ejCurrencyTextbox");
		* var perObj = $("#percentage").data("ejPercentageTextbox");
		* numObj.getValue(); // get value from numericTextbox
		* curObj.getValue(); // get value from currencyTextbox
		* perObj.getValue(); // get value from percentagTextbox
		* &lt;/script&gt;
		 @example 
		* &lt;input id="numeric" type="text" /&gt; <br> 
		* &lt;input id="currency" type="text" /&gt; <br> 
		* &lt;input id="percentage" type="text" /&gt; <br> 
		* &lt;script&gt;
		* $("#numeric").ejNumericTextbox({value:20});
		* $("#currency").ejCurrencyTextbox({value:500});
		* $("#percentage").ejPercentageTextbox({value:1000});
		* // get value from editors
		* $("#numeric").ejNumericTextbox("getValue");
		* $("#currency").ejCurrencyTextbox("getValue");
		* $("#percentage").ejPercentageTextbox("getValue");		
		* &lt;/script&gt;
		* @memberof ejTextBoxes
		* @instance
         */	      
        getValue: function () {
            return this.model.value == null ? "" : this.model.value;
        },
        // -----------------------------------wire and unwire events and handlers function----------------------------------//
		
		/**
         * Wiring the events to editors control		
		 * @private
         */	
        _wireEvents: function () {
            this._on(this.element, 'focus', this._focusIn);
            this._on(this.element, 'blur', this._focusOut);
            this._on(this.element, 'keydown', this._keyDown);
            this._on(this.element, "keyup", this._keyUp);
            this._on(this.element, 'mousewheel', this._mouseWheel);
            this._on(this.element, 'DOMMouseScroll', this._mouseWheel);
        },
		/**
         * Wiring the events to editors spinbutton		
		 * @private
         */	
        _bindSpinEvents: function () {
            this._on(this.spinUp, "mousedown", this._spinUpClick);
            this._on(this.spinDown, "mousedown", this._spinDownClick);
            this._on(this.spinUp, "mouseup", this._spinUpClick);
            this._on(this.spinDown, "mouseup", this._spinDownClick);
        },
		/**
         * unWiring the events from editors spinbutton		
		 * @private
         */	
        _unbindSpinEvents: function () {
            this._off(this.spinUp, "mousedown", this._spinUpClick);
            this._off(this.spinDown, "mousedown", this._spinDownClick);
            this._off(this.spinUp, "mouseup", this._spinUpClick);
            this._off(this.spinDown, "mouseup", this._spinDownClick);
        },
		 /**
         * Section For handle the spin button over mouse up click event
		 * @private
         */	
        _spinUpClick: function (event) {
            var self = this;
            event.preventDefault();
            clearTimeout(this._timeout);
            if (!this.model.enabled || this.model.readOnly) return;
            this._on(this.spinUp, 'mouseleave', this._mouseUpClick);
            this.spinUp.addClass("e-active");
            var self = this;
            if (event.type == "mouseup") {
                this._incrementValue(event);
                this.spinUp.removeClass("e-active");
                this._off($(document), 'mouseup', this._mouseUpClick);
            }
            else if (event.type == "mousedown") {
                if (!this._focused) this.element[0].focus();
                this._timeout = setInterval(function () { self._incrementValue(event); }, 150);
                this._on($(document), 'mouseup', this._mouseUpClick);
            }
        },
		 /**
         * Section For handle the spin button over mouse down click event
		 * @private
         */	
        _spinDownClick: function (event) {
            var self = this;
            event.preventDefault();
            clearTimeout(this._timeout);
            if (!this.model.enabled || this.model.readOnly) return;
            this._on(this.spinDown, 'mouseleave', this._mouseUpClick);
            this.spinDown.addClass("e-active");
            if (event.type == "mouseup") {
                this._decrementValue(event);
                this.spinDown.removeClass("e-active");
                this._off($(document), 'mouseup', this._mouseUpClick);
            }
            else if (event.type == "mousedown") {
                if (!this._focused) this.element[0].focus();
                this._timeout = setInterval(function () { self._decrementValue(event); }, 150);
                this._on($(document), 'mouseup', this._mouseUpClick);
            }
        },
		/**
         * Section For handle the mouse up click event
		 * @private
         */	
        _mouseUpClick: function (event) {
            event.stopPropagation();
            clearTimeout(this._timeout);
            this._off(this.spinUp, 'mouseleave', this._mouseUpClick);
            this._off(this.spinDown, 'mouseleave', this._mouseUpClick);
            this.spinDown.removeClass("e-active");
            this.spinUp.removeClass("e-active");
        },
		/**
         * Section For handle the mouse wheel event
		 * @private
         */	
        _mouseWheel: function (event) {
            event.preventDefault();
            if (!this._focused) this.element[0].focus();
			if (!this.model.enabled || this.model.readOnly) return;
            var delta;
            var rawEvent = event.originalEvent;
            if (rawEvent.wheelDelta) {
                // IE and Opera use wheelDelta, which is a multiple of 120 (possible values -120, 0, 120).
                delta = rawEvent.wheelDelta / 120;
                // In Opera, value is negated.
                //if (Sys.Browser.agent === Sys.Browser.Opera) delta = -delta;
            }
            else if (rawEvent.detail) {
                // Firefox uses detail property, which is a multiple of 3.
                delta = -rawEvent.detail / 3;
            }
            if (delta > 0)
                this._incrementValue(event);
            else if (delta < 0)
                this._decrementValue(event);
            this._cancelEvent(event);
        },
		
		/**
         * Section For handle number value from editors
		 * @private
         */	
        _numberValue: function () {
            var value = this._textBox.value;
            if (this.sfType === "ej.PercentageTextbox") {
                value = value.split(this._percentSymbol)[0];
            }
            else if (this.sfType === "ej.CurrencyTextbox") {
                var temp = value.split(this._currencySymbol)[1];
                value = temp ? temp.replace(")", "") : value;
            }
            return value;
        },
		/**
         * Section For handle the focus in event
		 * @private
         */	
        _focusIn: function (event) {
            this._focused = true;
            this.wrapper.addClass("e-focus");
            // this.element.addClass("e-focus");
            if (this.model.readOnly)
                return;
            this.wrapper.removeClass('e-error');
            if (this._textBox.value == this.model.watermarkText) {
                this._textBox.value = "";
                if (this.element.hasClass("e-watermark")) this._wipeWaterMark();
            }
            if (!this._error) {
                if (this._textBox.value != "")
                    this._textBox.value = this._numberValue();
                this._setSelectionRange(0, this._textBox.value.length);
            }
            this._trigger("focusIn",{value: this.model.value });
        },
		/**
         * Section For handle the focus out event
		 * @private
         */	
        _focusOut: function (event) {
            this._focused = false;
            this.wrapper.removeClass("e-focus");
            if (!this._error) {
                if (this._textBox.value != "" && this._textBox.value != this.model.watermarkText) {
                    if (parseFloat(this._textBox.value) < this.model.minValue) {
                        if (this.model.enableStrictMode == false)
                            this._textBox.value = this.model.minValue.toFixed(this.model.decimalPlaces);
                        else
                            this.isValidState = false
                    }
                    else if (parseFloat(this._textBox.value) > this.model.maxValue) {
                        if (this.model.enableStrictMode == false)
                            this._textBox.value = this.model.maxValue.toFixed(this.model.decimalPlaces);
                        else
                            this.isValidState = false;
                    }
                    if (this.sfType === "ej.NumericTextbox")
                        this.model.value = Globalize.parseFloat(this._textBox.value, this.model.locale);
                    else if (this.sfType === "ej.PercentageTextbox" || this.sfType === "ej.CurrencyTextbox") {
                        var editorvalue = this._textBox.value, value = editorvalue;
                        if (editorvalue.indexOf(Globalize.culture(this.model.locale).numberFormat.percent.symbol) > -1)
                            value = editorvalue.substring(0, editorvalue.length - 1);
                        else if (editorvalue.indexOf(Globalize.culture(this.model.locale).numberFormat.currency.symbol) > -1)
                            value = editorvalue.substring(1);
                        this.model.value = Globalize.parseFloat(value, this.model.locale);
                    }
                    if (isNaN(this.model.value) && !this.model.value) {
                        this.model.value = "";
                        this.isValidState = false;
                    }
                    this._localizedFormat();
                    this._checkErrorClass();
                }
                else {
                    if (this._textBox.value == "" && this.model.watermarkText == "")
                        this._textBox.value = "";
                    else if (this._textBox.value == "" && this.model.watermarkText != "")
                        this._setWaterMark();
                    if (this._textBox.value != "") {
                        if (this._textBox.value == this.model.watermarkText)
                            this.model.value = "";
                        else
                            this._setValue(this._textBox.value);
                    }
                }
                if (this.model.value == "" || this.model.value >= this.model.minValue && this.model.value <= this.model.maxValue) this.isValidState = true;
                else if (this.model.enableStrictMode) this.isValidState = false;
                this._keyUp();
                this._trigger("focusOut", { value: this.model.value });
            }
            this._checkErrorClass();
        },
		/**
         * Section For handle the cancel event
		 * @private
         */	
        _cancelEvent: function (e) {
            e.cancelBubble = true;
            e.returnValue = false;
            e.stopPropagation();
            e.preventDefault();
            return false;
        },
		/**
         * Section For handle the increment value in editors.
		 * @private
         */	
        /* Increment the Value */
        _incrementValue: function (e) {
            this.isValidState = true;
            if (this._textBox.value != this.model.watermarkText || this._textBox.value == "") {
                if (this._textBox.value == "")
                    this._textBox.value = this._startValue;
                if (this.sfType === "ej.PercentageTextbox" || this.sfType === "ej.CurrencyTextbox") {
                    var editorvalue = this._textBox.value, value = editorvalue;
                    if (editorvalue.indexOf(Globalize.culture(this.model.locale).numberFormat.percent.symbol) > -1)
                        value = editorvalue.substring(0, editorvalue.length - 1);
                    else if (editorvalue.indexOf(Globalize.culture(this.model.locale).numberFormat.currency.symbol) > -1)
                        value = editorvalue.substring(1);
                    this.model.value = Globalize.parseFloat(value, this.model.locale);
                }
                else if (this.sfType == "ej.NumericTextbox")
                    this.model.value = Globalize.parseFloat(this._textBox.value, this.model.locale);

                if (isNaN(this.model.value) && !this.model.value) {
                    this.model.value = this._startValue;
                }
                if (this.model.enableStrictMode == true) {
                    if (this.model.value > this.model.minValue && this.model.value > this.model.maxValue) {
                        this.model.value = this.model.maxValue;
                        this._setValue(this.model.value);
                        this._wipeWaterMark();
                    }
                    else if (this.model.value < this.model.minValue && this.model.value < this.model.maxValue) {
                        this.model.value = this.model.minValue;
                        this._setValue(this.model.value);
                        this._wipeWaterMark();
                    }
                    else
                        if (this.model.value >= this.model.minValue && this.model.value < this.model.maxValue) {
                            this.model.value = this.model.value + this.model.incrementStep;
                            if (this.model.value >= this.model.minValue && this.model.value <= this.model.maxValue) {
                                this._setValue(this.model.value);
                                this._wipeWaterMark();
                            }
                        };
                }
                else
                    if (this.model.value >= this.model.minValue && this.model.value < this.model.maxValue) {
                        this.model.value = this.model.value + this.model.incrementStep;
                        if (this.model.value >= this.model.minValue && this.model.value <= this.model.maxValue) {
                            this._setValue(this.model.value);
                            this._wipeWaterMark();
                        }
                    }
            }
            this._checkErrorClass();
        },
		/**
         * Section For handle the decrement value in editors.
		 * @private
         */	
        /* Decrement the Value */
        _decrementValue: function (e) {
            this.isValidState = true;
            var itemValue = this.model.value;
            if (this._textBox.value != this.model.watermarkText || this._textBox.value == "") {
                if (this._textBox.value == "")
                    this._textBox.value = this._startValue;
                else
                    if (this._textBox.value.indexOf(" ") >= 0)
                        this._textBox.value = this._textBox.value.replace(" ", "");
                if (this.sfType === "ej.PercentageTextbox" || this.sfType === "ej.CurrencyTextbox") {
                    var editorvalue = this._textBox.value, value = editorvalue;
                    if (editorvalue.indexOf(Globalize.culture(this.model.locale).numberFormat.percent.symbol) > -1)
                        value = editorvalue.substring(0, editorvalue.length - 1);
                    else if (editorvalue.indexOf(Globalize.culture(this.model.locale).numberFormat.currency.symbol) > -1)
                        value = editorvalue.substring(1);
                    this.model.value = Globalize.parseFloat(value, this.model.locale);
                }
                else if (this.sfType == "ej.NumericTextbox")
                    this.model.value = Globalize.parseFloat(this._textBox.value, this.model.locale);

                if (isNaN(this.model.value) && !this.model.value) {
                    this.model.value = this._startValue;
                }
                if (this.model.enableStrictMode == true) {
                    if (this.model.value >= this.model.minValue && this.model.value > this.model.maxValue) {
                        this.model.value = this.model.maxValue;
                        this._setValue(this.model.value);
                        this._wipeWaterMark();
                    }
                    else if (this.model.value < this.model.minValue && this.model.value <= this.model.maxValue) {
                        this.model.value = this.model.minValue;
                        this._setValue(this.model.value);
                        this._wipeWaterMark();
                    }
                    else if (this.model.value > this.model.minValue && this.model.value <= this.model.maxValue) {
                        this.model.value = this.model.value - this.model.incrementStep;
                        if (this.model.value >= this.model.minValue && this.model.value <= this.model.maxValue) {
                            this._setValue(this.model.value);
                            this._wipeWaterMark();
                        }
                    }
                }
                else
                    if (this.model.value > this.model.minValue && this.model.value <= this.model.maxValue) {
                        this.model.value = this.model.value - this.model.incrementStep;
                        if (this.model.value >= this.model.minValue && this.model.value <= this.model.maxValue) {
                            this._setValue(this.model.value);
                            this._wipeWaterMark();
                        }
                    }
            }
            this._checkErrorClass();
        },
		/**
         * Section For handle the validate decimal value
		 * @private
         */	
        _validateDecimal: function (e) {
            if (window.navigator.appName == "Microsoft Internet Explorer") {
                if (e.originalEvent.char == Globalize.culture(this.model.locale).numberFormat["."])
                    return true;
            }
            else {
                var char = String.fromCharCode(e.keyCode);
                if (e.keyCode == 188) char = ",";
                else if (e.keyCode == 190 || e.keyCode == 110) char = ".";
                if (Globalize.culture(this.model.locale).numberFormat["."].charCodeAt(0) == char.charCodeAt(0))
                    return true;
            }
            return false;
        },
		/**
         * Section For handle the keycodes
		 * @private
         */	
        _allowKeyCodes: function (e) {
            var keys = new Array(38, 40, 35, 36, 109, 189, 46, 8, 127, 37, 39, 190, 9, 13, 16, 17, 18, 20, 110, 173, 86, 88, 67);
            for (var i = 0; i < keys.length; i++) {
                if (e.keyCode == keys[i] || (this._validateDecimal(e) && this.model.decimalPlaces != 0))
                    return true;
            }
            return false;
        },
		/**
         * Section For raise the event.
		 * @private
         */	
        _raiseChangeEvent: function () {
            var currVal = this._numberValue();
            if (!ej.isNullOrUndefined(this._preVal) && (this._preVal.toString() != currVal)) {
                this._preVal = currVal;
                this.model.value = this._checkNumValue(this._preVal);
                this._hiddenInput.val(this.model.value);
                this.element.attr('aria-valuenow', this.model.value);
                this._trigger("change", { value: this.model.value });
            }
        },
		/**
         * Section For handle the key up event
		 * @private
         */	
        _keyUp: function () {
            this._raiseChangeEvent();
        },
		/**
         * Section For handle the key down event
		 * @private
         */	
        _keyDown: function (e) {
            if (this.model.readOnly) return;
            var cursor, cancelEvent, beforeCursor, afterCursor, sel;
            if (e.keyCode >= 48 && e.keyCode <= 57 || e.keyCode >= 96 && e.keyCode <= 105 || this._allowKeyCodes(e)) {
                if (e.shiftKey && (e.keyCode == 35 || e.keyCode == 36 || e.keyCode == 37 || e.keyCode == 39 || e.keyCode == 46 || e.keyCode == 127) || (e.ctrlKey && (e.keyCode == 86 || e.keyCode == 118 || e.keyCode == 67 || e.keyCode == 88)))
                    return true;
                if (((e.shiftKey == true || e.ctrlKey == true) && (e.keyCode != 9 && e.keyCode != 17 && e.keyCode != 86 && e.keyCode != 67)) || (e.keyCode == 67 || e.keyCode == 86 || e.keyCode == 88)) {//Prevent Shift + Tab event
                    this._keypressFlag = false;
                    this._cancelEvent(e);
                    return false;
                }

                if (e.keyCode >= 48 && e.keyCode <= 57 || e.keyCode >= 96 && e.keyCode <= 105 || e.keyCode == 110) {//Numbers from 0 to 9 in keyboard and numpad (110 for decimal)
                    this._wipeWaterMark();
                    cursor = this._caretPosition();
                    beforeCursor = this._textBox.value.toString().substring(0, cursor);
                    afterCursor = this._textBox.value.toString().substring(cursor);
                    sel = this._getSelection(this._textBox.value);
                    // For Numpad issue
                    if (e.keyCode >= 96 && e.keyCode <= 105)
                        e.keyCode -= 48;
                    if (sel.length == 0)
                    // if (beforeCursor.length != 0 && String.fromCharCode(e.keyCode) == 0)                           
                        this.model.value = Globalize.parseFloat((beforeCursor + String.fromCharCode(e.keyCode) + afterCursor), this.model.locale);
                    else
                        this.model.value = Globalize.parseFloat((beforeCursor.substring(0, beforeCursor.length - sel.length) + String.fromCharCode(e.keyCode) + afterCursor), this.model.locale);
                    if (this._validateOnType) {
                        var decSep = Globalize.culture(this.model.locale).numberFormat["."];
                        var temp = this.model.value.toString();
                        var tempIndex = temp.indexOf(decSep);
                        var value = this.model.value;
                        var replaceminVal = this.model.minValue.toFixed(this.model.decimalPlaces);
                        var replacemaxVal = this.model.maxValue.toFixed(this.model.decimalPlaces);
                        var exp = false;
                        if (replaceminVal.toString().indexOf("e") > 0 || replacemaxVal.toString().indexOf("e") > 0)
                            exp = true;
                        if (!exp) {
                            if (decSep != ".") {
                                temp = this._textBox.value.toString();
                                if (this.model.minValue.toString().match(new RegExp(".", "g")))
                                    replaceminVal = this.model.minValue.toString().replace(/["."]/g, decSep);
                                if (this.model.maxValue.toString().match(new RegExp(".", "g")))
                                    replacemaxVal = this.model.maxValue.toString().replace(/["."]/g, decSep);
                            }
                            if (replaceminVal.toString().indexOf(decSep) > 0)
                                minVal = Number(replaceminVal.toString().substring(0, replaceminVal.toString().indexOf(decSep)));
                            else
                                minVal = this.model.minValue;
                            if (replacemaxVal.toString().indexOf(decSep) > 0)
                                maxVal = Number(replacemaxVal.toString().toString().substring(0, replacemaxVal.toString().indexOf(decSep)));
                            else
                                maxVal = this.model.maxValue;
                            if (tempIndex > 0) {
                                value = Number(temp.substring(temp.indexOf(decSep) + 1, temp.toString().length));
                                if (replaceminVal.toString().indexOf(decSep) > 0)
                                    minVal = Number(replaceminVal.toString().substring(replaceminVal.toString().indexOf(decSep) + 1, replaceminVal.toString().length));
                                else
                                    minVal = 0;
                                if (replacemaxVal.toString().indexOf(decSep) > 0)
                                    maxVal = Number(replacemaxVal.toString().substring(replacemaxVal.toString().indexOf(decSep) + 1, replacemaxVal.toString().length));
                                else
                                    maxVal = 0;
                                var valb4Dec = Number(temp.substring(0, temp.indexOf(decSep)));
                                var minValb4Dec = Number(replaceminVal.toString().substring(0, replaceminVal.toString().indexOf(decSep)));
                                var maxValb4Dec = Number(replacemaxVal.toString().substring(0, replacemaxVal.toString().indexOf(decSep)));
                                if (!this._validateValue(value, minVal, maxVal, true, valb4Dec, minValb4Dec, maxValb4Dec))
                                    cancelEvent = true;
                                else
                                    cancelEvent = false;
                            }
                            else {
                                if (!this._validateValue(value, minVal, maxVal, false))
                                    cancelEvent = true;
                                else
                                    cancelEvent = false;
                            }
                        }
                        else if (this.model.decimalPlaces != 0) {
                            var afterdeci = (this._textBox.value).split(".")[1];
                            if (temp.indexOf(decSep) > 0 || (Number(temp) < Number(replaceminVal) || Number(temp) > Number(replacemaxVal))) {
                                if ((temp.substring(temp.indexOf(decSep) + 1, temp.length)).length > this.model.decimalPlaces)
                                    cancelEvent = true;
                                else if (sel.length == 0 && cursor > (this._textBox.value.indexOf(decSep)) && afterdeci && afterdeci.length >= this.model.decimalPlaces) cancelEvent = true;
                                else cancelEvent = false;
                            }
                            else if (sel.length == 0 && cursor > (this._textBox.value.indexOf(decSep)) && afterdeci && afterdeci.length >= this.model.decimalPlaces) cancelEvent = true;
                            else
                                cancelEvent = false;
                        }
                        else
                            cancelEvent = false;
                        if (cancelEvent) {
                            if (sel.length == 0)
                                this.model.value = parseFloat(beforeCursor + afterCursor);
                            else
                                this.model.value = parseFloat(beforeCursor.substring(0, beforeCursor.length - sel.length) + afterCursor);
                            this._keypressFlag = false;
                            this._cancelEvent(e);
                            return false;
                        }
                    }
                    this._raiseChangeEvent();
                }
                if (e.keyCode == 38 && this._allowkeyboard)//Up Arrow  
                {
                    this._incrementValue(e);
                    this._cancelEvent(e);
                }

                if (e.keyCode == 40 && this._allowkeyboard)//Down Arrow 
                {
                    this._decrementValue(e);
                    this._cancelEvent(e);

                }
                if (e.keyCode == 8) {//BackSpace Key
                    cursor = this._caretPosition();
                    beforeCursor = this._textBox.value.substring(0, cursor);
                    afterCursor = this._textBox.value.substring(cursor);
                    sel = this._getSelection(this._textBox.value);
                    if (sel.length == 0)
                        this.model.value = beforeCursor.substring(0, beforeCursor.length - 1) + afterCursor;
                    else
                        this.model.value = beforeCursor.substring(0, beforeCursor.length - sel.length) + afterCursor;

                    this._raiseChangeEvent();
                }

                if (e.keyCode == 46 || e.keyCode == 127) {//Delete key
                    cursor = this._caretPosition();
                    beforeCursor = this._textBox.value.substring(0, cursor);
                    afterCursor = this._textBox.value.substring(cursor);
                    sel = this._getSelection(this._textBox.value);
                    if (sel.length == 0)
                        this.model.value = beforeCursor + afterCursor.substring(1, afterCursor.length);
                    else
                        this.model.value = beforeCursor.substring(0, beforeCursor.length - sel.length) + afterCursor.substring(0, afterCursor.length);
                    this._raiseChangeEvent();
                }
                if (this._validateDecimal(e) && this.model.decimalPlaces != 0 && e.keyCode != 46)//'.' char key press
                {
                    var decChar = Globalize.culture(this.model.locale).numberFormat["."];
                    var minVal, maxVal;
                    var dotSplit = this._textBox.value.split(decChar);
                    if (dotSplit[1] == undefined) {
                        var curPosition = this._caretPosition();
                        var strBeforeCursor = dotSplit[0].substring(0, curPosition);
                        var strAfterCursor = dotSplit[0].substring(curPosition);
                        if (this.model.minValue.toString().indexOf("e") > 0 || this.model.maxValue.toString().indexOf("e") > 0)
                            exp = true;
                        if (this._validateOnType && !exp) {
                            if (this.model.minValue.toString().match(new RegExp(".", "g")))
                                minVal = this.model.minValue.toString().replace(/["."]/g, decChar);
                            if (this.model.maxValue.toString().match(new RegExp(".", "g")))
                                maxVal = this.model.maxValue.toString().replace(/["."]/g, decChar);
                            if (minVal.indexOf(decChar) > 0)
                                minVal = Number(minVal.substring(0, minVal.indexOf(decChar)));
                            else
                                minVal = Number(minVal);
                            if (maxVal.indexOf(decChar) > 0)
                                maxVal = Number(maxVal.substring(0, maxVal.indexOf(decChar)));
                            else
                                maxVal = Number(maxVal);
                            if (this._validateValue(this.model.value, minVal, maxVal, "DecimalKeyPressed"))
                                this.model.value = strBeforeCursor + decChar + strAfterCursor;
                            else {
                                this._keypressFlag = false;
                                this._cancelEvent(e);
                                return false;
                            }
                        }
                        else
                            this.model.value = strBeforeCursor + decChar + strAfterCursor;
                        this._setSelectionRange(this.model.value.length, this.model.value.length);
                        this._textBox.value = this.model.value;
                    } else {
                        this.model.value = this._textBox.value;
                    }
                    this._cancelEvent(e);
                } else if (e.keyCode == 190 || e.keyCode == 110) {
                    this._cancelEvent(e);
                }
                if (e.keyCode == 109 || e.keyCode == 189 || e.keyCode == 173)//'-' char key press.
                {
                    if ((this._caretPosition() != 0 && this._getSelection(this._removeFormats(this._textBox.value)) != this.model.value) || this.model.minValue >= 0 || (this._textBox.value.toString().match(new RegExp("-", "g")) && this._getSelection(this._textBox.value) == ""))
                        this._cancelEvent(e);

                    else if (this._getSelection() == this.model.value)
                        this.model.value = null;
                }
            }
            //For watermark issue while press ctrl+z
            else if (e.keyCode != 27 && !e.ctrlKey || (e.ctrlKey && e.keyCode == 90 && $.trim(this._textBox.value) == "")) {
                this._keypressFlag = false;
                this._cancelEvent(e);
            }
        }
    });

})(jQuery, Syncfusion);;
/**
* @fileOverview Plugin to style the Html Button elements
* @copyright Copyright Syncfusion Inc. 2001 - 2013. All rights reserved.
*  Use of this code is subject to the terms of our license.
*  A copy of the current license can be obtained at any time by e-mailing
*  licensing@syncfusion.com. Any infringement will be prosecuted under
*  applicable laws. 
* @version 12.1 
* @author <a href="mailto:licensing@syncfusion.com">Syncfusion Inc</a>
*/
(function ($, ej, undefined) {
	/**
    * @namespace ej
	* @classdesc Custom Design for Html ejMaskEdit Textbox control.
	* @class ejMaskEdit
	* @requires jQuery
	* @requires ej.core.js
	* @requires jquery.globalize.js
    * @requires globalize.cultures.min.js
	* @requires ej.maskedit.js
	* @example 	
	* &lt;input id="mask" type="text" /&gt; <br> 
	* &lt;script&gt; <br>
	* // Create Slider <br>
    * $('#mask').ejMaskEdit(); <br>	
	* &lt;/script&gt; <br>
	*/    
    ej.widget("ejMaskEdit", "ej.MaskEdit", {
        // widget element will be automatically set in this
        element: null,

        // user defined model will be automatically set in this
        model: null,
        validTags: ["input"],
        _setFirst: false,
        type: "editor",

        // default model
        defaults: {
			/**		
			* Specifies the maskFormat string for the maskedit textbox control.	
			* @default null
			* @type {string}
			* @example 
			* &lt;input id="mask" type="text" /&gt; <br> 
	        * &lt;script&gt; 
			* //To set maskFormat API value during initialization  
			* 	$("#mask").ejMaskEdit({ maskFormat: "99-9999"});					 
			* &lt;/script&gt; 
			* @memberof ejMaskEdit
			* @instance
			*/		
            maskFormat: "",
			/**		
			* Specifies the value string for the maskedit textbox control.	
			* @default null
			* @type {string}
			* @example
			* &lt;input id="mask" type="text" /&gt; <br> 
			* &lt;script&gt;			
			* //To set value API  during initialization  
			* 	$("#mask").ejMaskEdit({ maskFormat: "99-9999",value: "459978"});					 
			* &lt;/script&gt; 
			* @memberof ejMaskEdit
			* @instance
			*/		
            value: "",
			/**		
			* Specifies the watermark text string for the maskedit textbox control.	
			* @default null
			* @type {string}
			* @example 
			* &lt;input id="mask" type="text" /&gt; <br> 
			* &lt;script&gt; 
			* //To set watermarkText API value during initialization  
			* 	$("#mask").ejMaskEdit({ watermarkText: "Enter value"});					
		    * &lt;/script&gt;
			* @memberof ejMaskEdit
			* @instance
			*/		
            watermarkText: "",
			/**		
			* Specifies the height for the maskedit textbox control.	
			* @default 28pixel
			* @type {string}
			* @example 
			* &lt;input id="mask" type="text" /&gt; <br> 
			* &lt;script&gt; 
			* //To set height API value during initialization  
			* 	$("#mask").ejMaskEdit({ maskFormat: "99-9999",height: "28px"});					 
			* &lt;/script&gt;
			* @memberof ejMaskEdit
			* @instance
			*/		
            height: "",
			/**		
			* Specifies the width for the maskedit textbox control.	
			* @default 143pixel
			* @type {string}
			* @example 
			* &lt;input id="mask" type="text" /&gt; <br> 
			* &lt;script&gt;
			* //To set width API value during initialization  
			* 	$("#mask").ejMaskEdit({ maskFormat: "99-9999",width: 143});					 
			* &lt;/script&gt;
			* @memberof ejMaskEdit
			* @instance
			*/	
            width: "",
			/**		
			* Specifies the showError until correct value entered in the maskedit textbox control.	
			* @default false
			* @type {boolean}
			* @example 
			* &lt;input id="mask" type="text" /&gt; <br> 
			* &lt;script&gt;
			* //To set showError API value during initialization  
			* 	$("#mask").ejMaskEdit({maskFormat: "99-9999", showError: true});					 
		    * &lt;/script&gt; 
			* @memberof ejMaskEdit
			* @instance
			*/	
            showError: false,
			/**		
			* Specify the cssClass to maskedit textbox control to achieve custom theme.
			* @default null
			* @type {string}
			* @example 
			* &lt;input id="mask" type="text" /&gt; <br> 
			* &lt;script&gt;
			* //To set cssClass API value during initialization  
			* 	$("#mask").ejMaskEdit({maskFormat: "99-9999",cssClass: "gradient-lime"});					 
			* &lt;/script&gt;
			* @memberof ejMaskEdit
			* @instance
			*/	
            cssClass: "",
			/**		
			* Specify the custom character allowed to entered in maskedit textbox control.
			* @default null
			* @type {string}
			* @example 
			* &lt;input id="mask" type="text" /&gt; <br> 
			* &lt;script&gt; 
			* //To set customCharacter API value during initialization  
			* 	$("#mask").ejMaskEdit({ maskFormat: "99-9999",customCharacter: "gradient-lime"});					
			* &lt;/script&gt; 
			* @memberof ejMaskEdit
			* @instance
			*/	
            customCharacter: null,
			/**		
			* Specify the inputMode for maskedit textbox control. See {@link InputMode}
			* @default ej.InputMode.Text 
			* @type {enum}
			* @example 
			* &lt;input id="mask" type="text" /&gt; <br> 
			* &lt;script&gt;
			* //To set inputMode API value during initialization  
			* 	$("#mask").ejMaskEdit({maskFormat: "99-9999",inputMode: ej.InputMode.Password });					 
			* &lt;/script&gt; 
			* @memberof ejMaskEdit
			* @instance
			*/	
            inputMode: "text",
			/**		
			* Specify the readOnly for maskedit textbox control.
			* @default false 
			* @type {boolean}
			* @example 
			* &lt;input id="mask" type="text" /&gt; <br> 
			* &lt;script&gt; 
			* //To set readOnly API value during initialization  
			* 	$("#mask").ejMaskEdit({ maskFormat: "99-9999",value:"456789",readOnly: true });					 
			* &lt;/script&gt;
			* @memberof ejMaskEdit
			* @instance
			*/
            readOnly: false,
			/**		
			* Specify the text alignment for maskedit textbox control.
			* @default "left" 
			* @type {enum}
			* @example 
			* &lt;input id="mask" type="text" /&gt; <br> 
			* &lt;script&gt;
			* //To set textAlign API value during initialization  
			* 	$("#mask").ejMaskEdit({maskFormat: "99-9999",textAlign: "left" });					 
			* &lt;/script&gt;
			* @memberof ejMaskEdit
			* @instance
			*/	
            textAlign: ej.TextAlign.Left,			
			/**		
			* Specify the hidePromptOnLeave for maskedit textbox control to hide the mask on focus out.
			* @default false 
			* @type {boolean}
			* @example 
			* &lt;input id="mask" type="text" /&gt; <br> 
			* &lt;script&gt;
			* //To set hidePromptOnLeave API value during initialization  
			* 	$("#mask").ejMaskEdit({ maskFormat: "99-9999",hidePromptOnLeave: true });					 
			* &lt;/script&gt;
			* @memberof ejMaskEdit
			* @instance
			*/	
            hidePromptOnLeave: false,
			/**		
			* Specify the rounded corner for maskedit textbox control.
			* @default false 
			* @type {boolean}
			* @example 
			* &lt;input id="mask" type="text" /&gt; <br> 
			* &lt;script&gt;
			* //To set showRoundedCorner API value during initialization  
			* 	$("#mask").ejMaskEdit({ maskFormat: "99-9999",showRoundedCorner:true});					 
			* &lt;/script&gt;
			* @memberof ejMaskEdit
			* @instance
			*/	            
            showRoundedCorner: false,
			/**		
			* Specify the enablePersistence to mask edit textbox control to save current model value to browser cookies for state maintains.
			* @default false 
			* @type {boolean}
			* @example 
			* &lt;input id="mask" type="text" /&gt; <br> 
			* &lt;script&gt;
			* //To set enablePersistence API value during initialization  
			* 	$("#mask").ejMaskEdit({ enablePersistence: "left" });					 
			* &lt;/script&gt;
			* @memberof ejMaskEdit
			* @instance
			*/	  
            enablePersistence: false,
			/**		
			* Specify the state of the maskedit textbox control.
			* @default true 
			* @type {boolean}
			* @example 
			* &lt;input id="mask" type="text" /&gt; <br> 
			* &lt;script&gt;
			* //To set enabled API value during initialization  
			* 	$("#mask").ejMaskEdit({ maskFormat: "99-9999",enabled: true });					 
			* &lt;/script&gt;
			* @memberof ejMaskEdit
			* @instance
			*/	  
            enabled: true,
			/**     
			 * Fires when keydown in mask edit textbox control.
			 * @event
			 * @name ejMaskEdit#keydown 	
			 * @param {Object} argument Event parameters from maskedit textbox control     
			 * @param {boolean}  argument.cancel if the event should be cancelled; otherwise, false.			 
			 * @param {object}  argument.model returns the maskedit model	
			 * @param {string}  argument.type returns the name of the event				 
			 * @param {number}  argument.value returns the maskedit value
			 * @param {string}  argument.unmaskedValue  returns unstripped value in maskedit textbox control.
			 * @example 
			 * &lt;input id="mask" type="text" /&gt; <br> 
			 * &lt;script&gt;
			 * //keydown event for mask edit textbox control
             * $("#mask").ejMaskEdit({
			 *    maskFormat: "99-9999",
             *    keydown: function (args) {}
             * }); 
			 * &lt;/script&gt;			 
			 * @memberof ejMaskEdit
			 * @instance
			 */
			keydown: null,
			/**     
			 * Fires when keyup in mask edit textbox control.
			 * @event
			 * @name ejMaskEdit#keyup 	
			 * @param {Object} argument Event parameters from maskedit textbox control     
			 * @param {boolean}  argument.cancel if the event should be cancelled; otherwise, false.			 
			 * @param {object}  argument.model returns the maskedit model	
			 * @param {string}  argument.type returns the name of the event				 
			 * @param {number}  argument.value returns the maskedit value
			 * @param {string}  argument.unmaskedValue  returns unstripped value in maskedit textbox control.
			 * @example
			 * &lt;input id="mask" type="text" /&gt; <br> 
			 * &lt;script&gt;		 
			 * //keyup event for mask edit textbox control
             * $("#mask").ejMaskEdit({
			 *    maskFormat: "99-99-9999",
             *    keyup: function (args) {}
             * });      
			 * &lt;/script&gt;
			 * @memberof ejMaskEdit
			 * @instance
			 */
            keyup: null,
			/**     
			 * Fires when keypress in mask edit textbox control.
			 * @event
			 * @name ejMaskEdit#keyPress 	
			 * @param {Object} argument Event parameters from maskedit textbox control     
			 * @param {boolean}  argument.cancel if the event should be cancelled; otherwise, false.			 
			 * @param {object}  argument.model returns the maskedit model	
			 * @param {string}  argument.type returns the name of the event				 
			 * @param {number}  argument.value returns the maskedit value
			 * @param {string}  argument.unmaskedValue  returns unstripped value in maskedit textbox control.
			 * @example 
			 * &lt;input id="mask" type="text" /&gt; <br> 
			 * &lt;script&gt;
			 * //keyPress event for mask edit textbox control
             * $("#mask").ejMaskEdit({
			 *    maskFormat: "99-99-9999",
             *    keyPress: function (args) {}
             * });      
			 * &lt;/script&gt;
			 * @memberof ejMaskEdit
			 * @instance
			 */
            keyPress: null,
			/**     
			 * Fires when value changed in mask edit textbox control.
			 * @event
			 * @name ejMaskEdit#change 	
			 * @param {Object} argument Event parameters from maskedit textbox control     
			 * @param {boolean}  argument.cancel if the event should be cancelled; otherwise, false.			 
			 * @param {object}  argument.model returns the maskedit model	
			 * @param {string}  argument.type returns the name of the event				 
			 * @param {number}  argument.value returns the maskedit value
			 * @param {string}  argument.unmaskedValue  returns unstripped value in maskedit textbox control.
			 * @example 
			 * &lt;input id="mask" type="text" /&gt; <br> 
			 * &lt;script&gt;
			 * //change event for mask edit textbox control
             * $("#mask").ejMaskEdit({
			 *    maskFormat: "99999 - 9999",
             *    change: function (args) {}
             * });    
			 * &lt;/script&gt;			 
			 * @memberof ejMaskEdit
			 * @instance
			 */
            change: null,
			/**     
			 * Fires when mouse over in mask edit textbox control.
			 * @event
			 * @name ejMaskEdit#mouseover 	
			 * @param {Object} argument Event parameters from maskedit textbox control     
			 * @param {boolean}  argument.cancel if the event should be cancelled; otherwise, false.			 
			 * @param {object}  argument.model returns the maskedit model	
			 * @param {string}  argument.type returns the name of the event				 
			 * @param {number}  argument.value returns the maskedit value
			 * @param {string}  argument.unmaskedValue  returns unstripped value in maskedit textbox control.
			 * @example 
			 * &lt;input id="mask" type="text" /&gt; <br> 
			 * &lt;script&gt; 
			 * //mouseover event for toggle button
             * $("#mask").ejMaskEdit({
			 *    maskFormat: "$99999",
             *    mouseover: function (args) {}
             * });   
			 * &lt;/script&gt;			 
			 * @memberof ejMaskEdit
			 * @instance
			 */
            mouseover: null,
			/**     
			 * Fires when mouse out in mask edit textbox control.
			 * @event
			 * @name ejMaskEdit#mouseout 	
			 * @param {Object} argument Event parameters from maskedit textbox control     
			 * @param {boolean}  argument.cancel if the event should be cancelled; otherwise, false.			 
			 * @param {object}  argument.model returns the maskedit model	
			 * @param {string}  argument.type returns the name of the event				 
			 * @param {number}  argument.value returns the maskedit value
			 * @param {string}  argument.unmaskedValue  returns unstripped value in maskedit textbox control.
			 * @example 
			 * &lt;input id="mask" type="text" /&gt; <br> 
			 * &lt;script&gt; 
			 * //mouseout event for toggle button
             * $("#mask").ejMaskEdit({
			 *    maskFormat: "***-**-****",
             *    mouseout: function (args) {}
             * });    
			 * &lt;/script&gt;			 
			 * @memberof ejMaskEdit
			 * @instance
			 */
            mouseout: null,
			/**     
			 * Fires when focused in mask edit textbox control.
			 * @event
			 * @name ejMaskEdit#focusIn 	
			 * @param {Object} argument Event parameters from maskedit textbox control     
			 * @param {boolean}  argument.cancel if the event should be cancelled; otherwise, false.			 
			 * @param {object}  argument.model returns the maskedit model	
			 * @param {string}  argument.type returns the name of the event				 
			 * @param {number}  argument.value returns the maskedit value
			 * @param {string}  argument.unmaskedValue  returns unstripped value in maskedit textbox control.
			 * @example 
			 * &lt;input id="mask" type="text" /&gt; <br> 
			 * &lt;script&gt;
			 * //focusIn event for mask edit textbox control
             * $("#mask").ejMaskEdit({
			 *    maskFormat: "aa-99-99-a",
             *    focusIn: function (args) {}
             * });
			 * &lt;/script&gt;			 
			 * @memberof ejMaskEdit
			 * @instance
			 */
            focusIn: null,
			/**     
			 * Fires when focused out in mask edit textbox control.
			 * @event
			 * @name ejMaskEdit#focusOut 	
			 * @param {Object} argument Event parameters from maskedit textbox control     
			 * @param {boolean}  argument.cancel if the event should be cancelled; otherwise, false.			 
			 * @param {object}  argument.model returns the maskedit model	
			 * @param {string}  argument.type returns the name of the event				 
			 * @param {number}  argument.value returns the maskedit value
			 * @param {string}  argument.unmaskedValue  returns unstripped value in maskedit textbox control.
			 * @example 
			 * &lt;input id="mask" type="text" /&gt; <br> 
			 * &lt;script&gt;
			 * //focusOut event for mask edit textbox control
             * $("#mask").ejMaskEdit({
			 *    maskFormat: "(999)999-9999",
             *    focusOut: function (args) {}
             * }); 
			 * &lt;/script&gt;			 
			 * @memberof ejMaskEdit
			 * @instance
			 */
            focusOut: null,
			  /**     
          * Fires after MaskEdit control is created.
          * @event
          * @name ejMaskEdit#create 	
          * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
          * @param {object}  argument.model returns the MaskEdit model
          * @param {string}  argument.type returns the name of the event
          * @example 
		  * &lt;input id="mask" type="text" /&gt; <br> 
		  * &lt;script&gt;
			 * //create event for mask edit textbox control
             * $("#mask").ejMaskEdit({
			 *    maskFormat: "(999)999-9999",
             *    create: function (args) {}
             * }); 
		  * &lt;/script&gt;		 
          * @memberof ejMaskEdit
          * @instance
          */
            create: null,
			 /**     
          * Fires when the MaskEdit is destroyed successfully.
          * @event
          * @name ejMaskEdit#destroy 	
          * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
          * @param {object}  argument.model returns the MaskEdit model
          * @param {string}  argument.type returns the name of the event
          * @example 
		  * &lt;input id="mask" type="text" /&gt; <br> 
		  * &lt;script&gt;
			 * //destroy event for mask edit textbox control
             * $("#mask").ejMaskEdit({
			 *    maskFormat: "(999)999-9999",
             *    destroy: function (args) {}
             * }); 
		  * &lt;/script&gt;		 
          * @memberof ejMaskEdit
          * @instance
          */
            destroy: null
            
        },
		/**
         * Specify the data types for default properties 
		 * @private
         */
        dataTypes: {
            maskFormat: "string",
            showError: "boolean",
            enabled: "boolean",
            customCharacter: "string",
            cssClass: "string",
            watermarkText: "string",
            showRoundedCorner: "boolean",
            inputMode: "enum",
            textAlign: "enum",
            hidePromptOnLeave: "boolean",
            readOnly: "boolean"
        },
		/**
         * To configure the properties at runtime using SetModel		
		 * @private
         */
        _setModel: function (jsondata) {
            if (!(this.model.enabled) && ej.isNullOrUndefined(jsondata["enabled"])) return false;
            for (var key in jsondata) {
                switch (key) {
                    case "value": this._setValue(jsondata[key]); jsondata[key] = this.get_StrippedValue(); break;
                    case "width": this._setWidth(jsondata[key]); break;
                    case "height": this._setHeight(jsondata[key]); break;
                    case "watermarkText": this.model.watermarkText = jsondata[key]; this._setWaterMark(); break;
                    case "showRoundedCorner": this._roundedCorner(jsondata[key]); break;
                    case "textAlign": this._setTextAlign(jsondata[key]); break;
                    case "inputMode": this._setInputMode(jsondata[key]); break;
                    case "maskFormat": this._setMask(jsondata[key]); break;
                    case "cssClass": this._setSkin(jsondata[key]); break;
                    case "readOnly": this._setReadOnly(jsondata[key]); break;
                    case "enabled": this.model.enabled = jsondata[key]; this._controlStatus(jsondata[key]); break;
                }
            }
        },
        observables: ["value"],
        // all events bound using this._on will be unbind automatically
		/**
         * To configure destroy of the maskedit textbox control.		
		 * @private
         */	
        _destroy: function () {
            if (this.model.name) this.element.removeAttr("name");
            this.element.insertAfter(this.wrapper);
            this.wrapper.remove();
            if (this.model.textAlign) this.element.css("text-align", "");
            this.element.val("").removeClass(' e-mask e-input e-corner-all e-disable e-watermark').empty();
        },
		/**
         * To initialize the maskedit textbox control.		
		 * @private
         */
        // constructor function
        _init: function () {
            if (this.element.is("input") && (this.element.is("input[type=text]") || !this.element.attr('type'))) {
                this._setValues();
                this._renderControl();
                this._initObjects();
                this._wireEvents();
            } else {
                this._destroy();
                return false;
            }
        },
		/**
         * To enable or disable the state of the maskedit control.		
		 * @private
         */	
        _controlStatus: function (value) {
            //disable status
            value != true ? this.disable() : this.enable();
        },
		/**
         * To configure value for maskedit textbox control.		
		 * @private
         */	
        _setValues: function () {
            this._tempMask = null;
            this._unStrippedMask = null;
            //Predefined character definitions
            this._charMap = {
                '9': "[0-9]",
                'a': "[A-Za-z]",
                '*': "[A-Za-z0-9]",
                'A': "[A-Za-z]",
                'N': "[0-9]",
                '<': "",
                '>': "",
                'C': "",
                '?': "(.)",
                '$': "[A-Za-z ]"
            };
        },
		/**
         * Render Section For maskedit textbox control		
		 * @private
         */	
        _renderControl: function () {
            this.model.name = this.element.attr("name") != null ? this.element.attr("name") : (this.model.name != null ? this.model.name : this.element[0].id);
			if(this.element.attr("name") == null)
                this.element.attr({'name': this.model.name});
            this.wrapper = ej.buildTag("span.e-mask e-widget " + this.model.cssClass);
            this.innerWrapper = ej.buildTag("span.e-in-wrap e-box");
            this.wrapper.append(this.innerWrapper).insertBefore(this.element);
            this.innerWrapper.append(this.element);
            this._hiddenInput = ej.buildTag("input#" + this._id + "_hidden", "", {}, { type: "hidden" }).insertBefore(this.element);
            this._hiddenInput.attr("name", this.model.name);
            this.model.value = this.model.value === "" ? this.element[0].value :this.model.value;
            this.element.attr({ 'aria-invalid': false, 'value': this.model.value }).addClass('e-input');
            this._setInputMode(this.model.inputMode);
            this._setWidth(this.model.width);
            this._setHeight(this.model.height);
            this._roundedCorner(this.model.showRoundedCorner);
            this._setTextAlign(this.model.textAlign);
            this._setReadOnly(this.model.readOnly);
            this._controlStatus(this.model.enabled);
            this.previousValue = this.model.value;
        },
		/**
         * To initialize the maskedit textbox control properties.		
		 * @private
         */
        _initObjects: function () {
            this._textbox = this.element[0];
            this._keypressFlag = 0;
            this._selectedTextKeyDown = 0;
            // KeyDown trace [ 0 - None, 1 - BackSpace, 2 - Delete]
            this._keydownFlag = 0;
            // Length of the Mask
            this._maskLength = this.model.maskFormat.length;
            //Replacing all [9?$CANa*] to '_'
            this._maskModel = this.model.maskFormat.replace(/[9?$CANa*]/g, '_');
            this._validatedValue = this._maskModel;
            if (this.model.inputMode != "password") {
				this._setValue(this.model.value);
                // Setting WaterMark Text        
                this._setWaterMark();
            }

            //Check If showError Flag is true add the css class
            if (this.model.showError) {
                this.element.addClass("e-error").attr('aria-invalid',"true");
            }
        },

        //----------------private function--------------------------------------//
		/**
		 * To configure width of the maskedit textbox control.
		 * @private
         */	
        _setWidth: function (value) {
            this.wrapper.width(value);
        },
		/**
		 * To configure height of the maskedit textbox control.
		 * @private
         */
        _setHeight: function (value) {
            this.wrapper.height(value);
        },
		/**
         * To enable or disable the rounded corner behaviour. 		
		 * @private
         */	
        _roundedCorner: function (value) {
            if (value) {
                this.wrapper.removeClass('e-corner-all').addClass('e-corner-all');
            }
            else if (this.wrapper.hasClass('e-corner-all')) {
                this.wrapper.removeClass('e-corner-all');
            }
        },
		/**
		 * To configure text alignment of the maskedit textbox control.
		 * @private
         */
        _setTextAlign: function (align) {
            if (align == "right") {                
                this.wrapper.addClass('e-rtl');
            }
            else {
                this.wrapper.removeClass('e-rtl');
            }
        },
		/**
		 * To configure input mode for maskedit textbox control.
		 * @private
         */
        _setInputMode: function (type) {
            this.element.attr('type', type);
        },
		/**
         * To enable or disable the readonly behaviour. 		
		 * @private
         */	
        _setReadOnly: function (bool) {
            if (bool) this.element.attr("readonly", true);
            else this.element.removeAttr("readonly");
        },
		/**
         * To configure the custom theme for maskedit textbox control using cssClass property		
		 * @private
         */
        _setSkin: function (skin) {
            this.wrapper.removeClass(this.model.cssClass);
            this.wrapper.addClass(skin);
        },
		/**
         * To configure the watermark text to the maskedit textbox control.
		 * @private
         */
        /* Setting the WaterMark text*/
        _setWaterMark: function () {
            this._maskModel = this.model.maskFormat.replace(/[9?$CANa*]/g, '_');
            if (this.model.watermarkText && (this.model.value == '' || this.model.value == this._maskModel)) {
                this._textbox.value = this.model.watermarkText;
                this.element.addClass("e-watermark");
            } else if (this.model.value == '' && this._maskModel != '') {
                this._textbox.value = this._maskModel;
            }
        },
		/**
         * To remove the watermark text from the maskedit textbox control.
		 * @private
         */
        /* Clearing the WaterMark Text*/
        _removeWaterMark: function () {
            if (this.element.hasClass("e-watermark")) {
                this._textbox.value = this._maskModel;
                this.element.removeClass("e-watermark");
            }
        },
		/**
         * To configure alert when user press wrong key in maskedit textbox control.
		 * @private
         */
        /* Show alert when user press Wrong key*/
        _showAlert: function () {
            var ptr = this;
            this.element.addClass("e-error").attr('aria-invalid',"true").animate({ Opacity: 1 }, 700, null, function () {
                $(ptr._textbox).removeClass("e-error").attr('aria-invalid',"false");
            });
        },
		/**
         * To check whether user entered value in maskedit textbox control.
		 * @private
         */
        /* Validating User's TextBox Value*/
        _validateValue: function () {
            var tempValue = this.model.value;
            var tempModel = this._maskModel;
            var valueIndex = -1;
            // Check if Mask is Empty display the text
            if (this._maskLength == 0) {
                if (this.element.hasClass("e-watermark")) {
                    this.element.removeClass("e-watermark");
                }
                this._textbox.value = tempValue;
                return true;
            }
            else {
                var replacestring = this.model.value.toString();
                tempValue = this.model.value = replacestring.replace(/[\(\)-]/g, "");
            }

            for (var maskIndex = 0; maskIndex < this._maskLength; maskIndex++) {
                if (this._charMap[this.model.maskFormat.charAt(maskIndex)]) {
                    valueIndex++;
                    var charCode = tempValue.charCodeAt(valueIndex);
                    if (this._validateChars(charCode, maskIndex)) {
                        var strBefore = tempModel.substring(0, maskIndex);
                        var strAfter = tempModel.substring(maskIndex);
                        tempModel = strBefore + tempValue.charAt(valueIndex) + strAfter.substr(1, strAfter.length);
                    }
                }
            }
            if (this.element.hasClass("e-watermark")) {
                if (this.model.value) {
                    this.element.removeClass("e-watermark");
                    this._textbox.value = tempModel;
                }
                else
                    this._setWaterMark();
            }
            else {
                if (this.model.value) {
                    this._textbox.value = tempModel;
                }
                else
                    this._textbox.value = this.model.maskFormat.replace(/[9?$CANa*]/g, '_');
            }

        },
		/**
         * To clear selected text in maskedit textbox control.
		 * @private
         */
        /* If it is Selected Text opreation, This Function clears the Selected Text to MaskModel*/
        _selectionText: function (begin, end) {

            var replaceValue = this._maskModel.substring(begin, end);
            this._textbox.value = this._textbox.value.substring(0, begin) + replaceValue + this._textbox.value.substring(end);
            if (this._keydownFlag == 1) {
                this._setCaretPosition(begin);
                return begin;
            }
            else if (this._keydownFlag == 2) {
                this._setCaretPosition(end);
                return end;
            }
            return begin;
        },
		/**
         * To obtained CaretpPosition of the cursor in maskedit textbox control.
		 * @private
         */
        /* Get the CaretPsoition of the cursor, if it selected 1/more text call _selectionText*/
        _caretPosition: function (textbox) {
            //caretPos starts with 0
            var caretPos = 0;

            // Microsoft supports only IE for using document.selection
            if (document.selection) {
                //textbox.focus();
                // Create a Range of selected position
                var selectedRange = document.selection.createRange();
                var selectionLength = selectedRange.text.length;

                // Move selection start to 0 position
                selectedRange.moveStart('character', -textbox.value.length);

                // Get caret position by selection length, becoz now range is from Zero to current focus
                caretPos = selectedRange.text.length - selectionLength;

                if (selectionLength != 0) {
                    caretPos = this._selectionText(caretPos, selectedRange.text.length);
                    this._selectedTextKeyDown = 1;
                }
            }

            // For Firefox
            else if (textbox.selectionStart || textbox.selectionStart == '0') {
                caretPos = textbox.selectionStart;
                if (textbox.selectionStart != textbox.selectionEnd) {
                    if (this._keydownFlag)
                        caretPos = this._selectionText(textbox.selectionStart, textbox.selectionEnd);
                    this._selectedTextKeyDown = 1;
                }
            }

            // Return caret Position
            return (caretPos);
        },
		/**
         * To set the cursor in required position in maskedit textbox control.
		 * @private
         */
        /* Set the curosr at required position in textbox*/
        _setCaretPosition: function (caretPos) {

            //element will have MaskEdit Textbox
            var element = this._textbox;

            //For IE
            if (window.navigator.appName == "Microsoft Internet Explorer") {
                if (element.createTextRange) {
                    var range = element.createTextRange();
                    range.move('character', caretPos);
                    range.select();
                }
            }
            else
            /* For FireFox and Chrome */
                if (element.selectionStart || element.selectionStart == '0') {
                    element.focus();
                    element.setSelectionRange(caretPos, caretPos);
                }
                else
                    element.focus();

        },
		/**
         * To check whether user pressed character with mask format in maskedit textbox control.
		 * @private
         */
        /* Validate the User Pressed Char with the Mask Format*/
        _validateChars: function (keyChar, caretPos) {
            var charmap = this._charMap, match = false;
            var maskChar = this.model.maskFormat.substr(caretPos, 1);
            var customChar = this.model.customCharacter;
            var actualkey = String.fromCharCode(keyChar);
            $.each(charmap, function (key, value) {
                if (maskChar == key) {
                    if (customChar != null) {
                        if (key == "C")
                            value = "[" + customChar + "]";
                        else if (key == "A" || key == "N")
                            value = value.replace(("]"), "") + customChar + "]";
                    }
                    if (actualkey.match(new RegExp(value))) match = true;
                    else match = false;
                }
            });
            return match;
        },
		/**
         * To configure the next valid position on right in maskedit textbox control.
		 * @private
         */
        /* Seek to the next valid position on right*/
        _seekNext: function () {
            var currentPos = this._caretPosition(this._textbox);
            var tempPos = currentPos;
            var seekFlag = true;
            //seeking the focus to next valid position on right of MaskEdit TextBox
            while (seekFlag) {
                if (currentPos >= 0 && currentPos < this._maskLength) {
                    if (this.model.maskFormat.charAt(currentPos) != "C") {
                        if (!this._charMap[this.model.maskFormat.charAt(currentPos)]) {
                            this._setCaretPosition(currentPos);
                            currentPos++;
                        }
                    }
                    if (currentPos != tempPos) {
                        tempPos = currentPos;
                        continue;
                    }
                }
                seekFlag = false;
            }
            return currentPos;
        },
		/**
         * To configure the next valid position on left in maskedit textbox control.
		 * @private
         */
        /* Seek to the next valid position on left*/
        _seekBefore: function (caretPos) {
            var currentPos = this._caretPosition(this._textbox);
            var tempPos = --currentPos;
            var seekFlag = true;

            // Stop Seek when backspace of select more than a char
            if (this._selectedTextKeyDown == 1)
                seekFlag = false;

            //seeking the focus to next valid position on left of MaskEdit TextBox
            while (seekFlag) {
                if (currentPos >= 0 && currentPos < this._maskLength) {
                    if (!this._charMap[this.model.maskFormat.charAt(currentPos)]) {
                        this._setCaretPosition(--currentPos);
                    }
                    if (currentPos != tempPos) {
                        tempPos = currentPos;
                        continue;
                    }
                }
                seekFlag = false;
            }
            return currentPos;
        },
		/**
         * To write the user pressed character in maskedit textbox control.
		 * @private
         */
        /* Write the User Pressed Character*/
        _writeBuffer: function (key, cursorPos) {
            if (cursorPos <= this._maskLength) {
                var input = this._textbox.value;
                if (this._tempMask != null)
                    var tempkey = this._updateCasing(key, cursorPos);
                key = (tempkey == undefined) ? key : tempkey;
                var strBeforeCursor = input.substring(0, cursorPos);
                var strAfterCursor = input.substring(cursorPos);
                this._textbox.value = strBeforeCursor + key + strAfterCursor.substr(1, strAfterCursor.length);
                this._setCaretPosition(cursorPos + 1);
                //Raising ClientSide Event
                this._raiseEvents("change");
            }
        },
		/**
         * To configure the user entered character in lower or upper case character in maskedit textbox control.
		 * @private
         */
        _updateCasing: function (key, pos) {
            for (var i = 0; i < pos + 1; i++) {
                if (this._tempMask.substr(i, 1) == '<' || this._tempMask.substr(i, 1) == '>')
                    pos++;
            }
            // find case-modificator
            for (var j = pos; j > -1; j--) {
                if (this._tempMask.substr(j, 1) == '<') {
                    return key.toLowerCase();
                }
                else if (this._tempMask.substr(j, 1) == '>') {
                    return key.toUpperCase();
                }
            }
        },
		/**
         * To obtained the pure value of the textvalue, removes all the symbols in maskedit textbox control.
		 * @private
         */
        /* Return the pure value of the textvalue, removes all the symbols*/
        get_StrippedValue: function () {
            var i, value, mask = this.model.maskFormat, stripVal = "";
            value = (this.element.hasClass("e-watermark")) ? this._maskModel : this._textbox.value;
            if (mask.length == 0) return value;

            for (i in mask) {
                var char = mask[i], exp = null;

                if ("9?$a*".indexOf(char) != -1)
                    exp = this._charMap[char];
                else if (char == "A" || char == "N")
                    exp = this._charMap[char].replace(("]"), "") + this.model.customCharacter + "]";
                else if (char == "C")
                    exp = "[" + this.model.customCharacter + "]";

                if (exp && value[i] && value[i].match(new RegExp(exp)))
                    stripVal += value[i];
            }
            return stripVal;
        },
		/**
         * To obtained the textbox value as such that, Just replace all '_' to ' '(space) in maskedit textbox control.
		 * @private
         */
        /* Return the textbox value as such that, Just replace all '_' to ' '(space)*/
        get_UnstrippedValue: function () {
            var value = (this.element.hasClass("e-watermark")) ? this._maskModel : this._textbox.value, unstripVal = "";
            if (this.model.maskFormat.length == 0) return value;

            if (value != undefined)
                unstripVal = value.replace(/[_]/g, " ");
            return unstripVal;
        },
		/**
         * To configure the model value in maskedit textbox control.
		 * @private
         */
        _setValue: function (value) {
            if (ej.isNullOrUndefined(value)) value = "";
            this.model.value = value;
            if (!ej.isNullOrUndefined(this.model.maskFormat) && this.model.maskFormat != "")
                this._validateValue();
            else
                this._textbox.value = value;
            this.model.value = this.get_StrippedValue();
            this._hiddenInput.val(this.model.value);
        },
		/**
         * To set the maskFormat value with it;s validation in maskedit textbox control.
		 * @private
         */
        _setMask: function (maskValue) {
            this._maskLength = maskValue.length;
            this.model.maskFormat = maskValue;
            this._maskModel = maskValue.replace(/[9?$CANa*]/g, '_');
            this._validateValue();
        },

        //----------------public function--------------------------------------//			 
		/**
        * To enable the maskedit textbox control.		
		* @return jQuery
		* @example 
		* &lt;input id="mask" type="text" /&gt; <br> 		
		* &lt;script&gt;
		* $("#mask").ejMaskEdit({maskFormat: "99-99-9999",value:"12345678"});
		* // Create MaskEdit control
		* var maskObj = $("#mask").data("ejMaskEdit");		
		* maskObj.enable(); // enable the maskedit control		
		* &lt;/script&gt;
		 @example 
		* &lt;input id="mask" type="text" /&gt; <br> 			
		* &lt;script&gt;
		* $("#mask").ejMaskEdit({maskFormat: "99-99-9999",value:"12345678"});
		* // enable the maskedit control
		* $("#mask").ejMaskEdit("enable");		
		* &lt;/script&gt;
		* @memberof ejMaskEdit
		* @instance
         */	    
        enable: function () {
            this.element.disabled = false;
            this.element.removeAttr("disabled").removeClass('e-disable').attr({ "aria-disabled": false });
            this.model.enabled = true;
        },
		/**
        * To disable the maskedit textbox control.		
		* @return jQuery
		* @example 
		* &lt;input id="mask" type="text" /&gt; <br> 		
		* &lt;script&gt;
		* $("#mask").ejMaskEdit({maskFormat: "99-99-9999",value:"45340078"});
		* // Create MaskEdit control
		* var maskObj = $("#mask").data("ejMaskEdit");		
		* maskObj.disable(); // disable the maskedit control		
		* &lt;/script&gt;
		 @example 
		* &lt;input id="mask" type="text" /&gt; <br> 			
		* &lt;script&gt;
		* $("#mask").ejMaskEdit({maskFormat: "99-99-9999",value:"45340078"});
		* // disable the maskedit control
		* $("#mask").ejMaskEdit("disable");		
		* &lt;/script&gt;
		* @memberof ejMaskEdit
		* @instance
         */	  
        disable: function () {
            this.element.disabled = true;
            this.element.attr("disabled", "disabled").addClass('e-disable').attr({ "aria-disabled": true });
            this.model.enabled = false;
        },
		/**
        * To clear the text in maskedit textbox control.		
		* @return jQuery
		* @example 
		* &lt;input id="mask" type="text" /&gt; <br> 		
		* &lt;script&gt;
		* $("#mask").ejMaskEdit({maskFormat: "99-9999",value:"345678"});
		* // Create MaskEdit control
		* var maskObj = $("#mask").data("ejMaskEdit");		
		* maskObj.clear(); // clear the maskedit control		
		* &lt;/script&gt;
		 @example 
		* &lt;input id="mask" type="text" /&gt; <br> 			
		* &lt;script&gt;
		* $("#mask").ejMaskEdit({maskFormat: "99-9999",value:"345678"});
		* // clear the maskedit control
		* $("#mask").ejMaskEdit("clear");		
		* &lt;/script&gt;
		* @memberof ejMaskEdit
		* @instance
         */	 
        clear: function () {
            if (this.model.enabled)
                this._textbox.value = this.model.maskFormat.replace(/[9?$aCAN*]/g, '_');
        },

        /*-------------------------- Register all the events at initialize and handlers-----------------------*/
		/**
         * Wiring the events to MaskEdit textbox control		
		 * @private
         */
        _wireEvents: function () {
            this._on(this.element, 'focus', this._OnFocusHandler);
            this._on(this.element, 'blur', this._OnBlurHandler);
            this._on(this.element, 'keydown', this._OnKeyDownHandler);
            this._on(this.element, 'keypress', this._OnKeyPressHandler);
            this._on(this.element, 'keyup', this._OnKeyUpHandler);
            this._on(this.element, 'mouseover', this._OnMouseOverHandler);
            this._on(this.element, 'mouseout', this._OnMouseOutHandler);
            this._on(this.element, 'paste', this._OnPasteHandler);
            this._on(this.element, 'cut', this._OnCutHandler);
        },
		 /**
         * Section For handle the selected text cut from maskedit textbox control.
		 * @private
         */	
        /* Cut Handler */
        _OnCutHandler: function (e) {
            var selectedValue = this._maskModel.substring(this._textbox.selectionStart, this._textbox.selectionEnd);
            var beforeSelection = this._textbox.value.substring(0, this._textbox.selectionStart);
            var afterSelection = this._textbox.value.substring(this._textbox.selectionEnd);
            var cursorPosition = this._textbox.selectionStart;
            var context = this;
            setTimeout(function () {
                context._textbox.value = beforeSelection + selectedValue + afterSelection;
                context._setCaretPosition(cursorPosition);
                context._raiseEvents("change");
            }, 0);
        },
		/**
         * Section For handle the paste text to maskedit textbox control.
		 * @private
         */
        /* Paste Handler */
        _OnPasteHandler: function (e) {
            var context = this;
            setTimeout(function () {
                var text = $(context._textbox).val();
                context._setValue(text);
                context._raiseEvents("change");
            }, 0);
            return true;
        },
		/**
         * Section For handle maskedit textbox control in focused in.
		 * @private
         */
        /* On Focus Handler */
        _OnFocusHandler: function (e) {
            this.wrapper.addClass("e-focus");
            this._removeWaterMark();
            if (this.model.readOnly)
                return;
            if (this._textbox.value.indexOf("<") >= 0 || this._textbox.value.indexOf(">") >= 0) {
                this._textbox.value = this._textbox.value.replace(/[<>]/g, '');
                this._maskModel = this._textbox.value;
                this._tempMask = this.model.maskFormat;
                this.model.maskFormat = this.model.maskFormat.replace(/[<>]/g, '');
            }

            if (this._textbox.value != this._maskModel && this._unStrippedMask != null && this.model.hidePromptOnLeave)
                this._textbox.value = this._unStrippedMask;
            $.fn.selectRange = function (start, end) {
                return this.each(function () {
                    if (this.setSelectionRange) {
                        this.focus();
                        this.setSelectionRange(start, end);
                    } else if (this.createTextRange) {
                        var range = this.createTextRange();
                        range.collapse(true);
                        range.moveEnd('character', end);
                        range.moveStart('character', start);
                        range.select();
                    }
                });
            };
            if (this.model.maskFormat) {
                this.element.selectRange(0, 0);
            }
            //Raising ClientSide Event
            this._raiseEvents("focusIn");
        },
		/**
         * Section For handle the maskedit textbox control is focused out.
		 * @private
         */
        _OnBlurHandler: function (e) {
            this.wrapper.removeClass("e-focus");
            this.model.value = this.get_StrippedValue();
            if (this._textbox.value != undefined && this.model.hidePromptOnLeave) {
                this._unStrippedMask = this._textbox.value;
                this._textbox.value = this.get_UnstrippedValue();
            }
            if (this.model.inputMode != "password")
                this._setWaterMark();

            //Raising ClientSide Event
            this._raiseEvents("focusOut");
        },
		/**
         * Section For handle the keypress down event in maskedit textbox control.
		 * @private
         */
        /* On Key Down Handler */
        _OnKeyDownHandler: function (e) {
            if (this.model.readOnly) return;
            this._keypressFlag = 0;
            this._raiseEvents("onKeyDown");
            // Remove the Error Flag
            this.element.removeClass("error");

            if (this._maskLength == 0)
                return true;

            var unicode = e.keyCode ? e.keyCode : e.charCode;
            var actualkey = String.fromCharCode(unicode);

            if (unicode >= 35 && unicode <= 41) {
                if (window.navigator.appCodeName == "Mozilla" || window.navigator.appCodeName == "opera") {
                    this._keypressFlag = 1;
                }
            }
            if (e.shiftKey && (e.keyCode == 35 || e.keyCode == 36 || e.keyCode == 37 || e.keyCode == 39 || e.keyCode == 46 || e.keyCode == 127) || (e.ctrlKey && (e.keyCode == 86 || e.keyCode == 65 || e.keyCode == 67 || e.keyCode == 88))) {
                this._keypressFlag = 1;
                return;
            }
            else if (unicode == 8) { // For Backspace Key Event

                if (this.model.inputMode != "password") {
                    this._keydownFlag = 1;
                    // Seek to next valid position
                    var currentPos = this._seekBefore();
                    this._raiseEvents("change");
                    if (!this._selectedTextKeyDown) {
                        if (currentPos >= 0 && currentPos < this._maskLength)
                            this._writeBuffer("_", currentPos);
                        this._setCaretPosition(currentPos);
                    }
                    this._keydownFlag = 0;
                    this._selectedTextKeyDown = 0;
                    e.preventDefault();
                    return false;
                }
                this._keypressFlag = 1;
                return true;
            }
            else if (e.keyCode == 46 || e.keyCode == 127) { //  For Delete Key Events
                if (this.model.inputMode != "password") {
                    this._keydownFlag = 2;
                    // Seek to next valid position
                    currentPos = this._seekNext();
                    this._raiseEvents("change");
                    if (!this._selectedTextKeyDown) {
                        if (currentPos >= 0 && currentPos < this._maskLength)
                            this._writeBuffer("_", currentPos);
                    }
                    this._keydownFlag = 0;
                    this._selectedTextKeyDown = 0;
                    e.preventDefault();
                    return false;
                }
                this._keypressFlag = 1;
                return true;
            }
        },
		/**
         * Section For handle the keypress up event in maskedit textbox control.
		 * @private
         */
        /* On Key Up Handler */
        _OnKeyPressHandler: function (e) {
            if (this.model.readOnly) return;
            this._raiseEvents("keyPress");
            if (this._maskLength == 0)
                return true;

            var unicode = e.keyCode ? e.keyCode : e.charCode;
            var actualkey = String.fromCharCode(unicode);
            var currentPos = this._seekNext();
            var val = ej.browserInfo().name;
            if (this._validateChars(unicode, currentPos)) {
                if (((val == "mozilla") || (val == "opera")) && (e.keyCode == 35 || e.keyCode == 36 || e.keyCode == 37 || e.keyCode == 39)) {
                }
                else if (((val == "mozilla") || (val == "opera")) && (e.ctrlKey && (unicode == 97 || unicode == 99 || unicode == 118 || unicode == 120))) {
                }
                else
                    this._writeBuffer(actualkey, currentPos);
            }
            else
                if (((val == "mozilla") || (val == "opera")) && (e.keyCode == 35 || e.keyCode == 36 || e.keyCode == 37 || e.keyCode == 39)) {
                }
                else if (((val == "mozilla") || (val == "opera")) && (e.ctrlKey && (unicode == 97 || unicode == 99 || unicode == 118 || unicode == 120))) {
                }
                else
                    this._showAlert();

            // To stop default press actions
            if (!this._keypressFlag && unicode != 9) {//Shift + Tab Navigation
                this._keypressFlag = 0;
                e.preventDefault();
                return false;
            }
            this._keypressFlag = 0;

        },
		/**
         * Section For raise the key up event in maskedit textbox control.
		 * @private
         */
        _OnKeyUpHandler: function (e) {
            if (this._maskLength == 0) this._raiseEvents("change");
            this._raiseEvents("keyUp");
        },
		/**
         * Section For raise the mouse over event in maskedit textbox control.
		 * @private
         */
        /* On Mouse Over Handler */
        _OnMouseOverHandler: function (e) {
            // Raising ClideSide Event
            this._raiseEvents("mouseOver");
        },
		/**
         * Section For raise the mouse out event in maskedit textbox control.
		 * @private
         */
        /* On Mouse Out Handler */
        _OnMouseOutHandler: function (e) {
            // Raising ClideSide Event
            this._raiseEvents("mouseOut");
        },
		/**
         * Section For raise the corresponding event in maskedit textbox control.
		 * @private
         */
        //To raise Corresponding Client Side Events
        _raiseEvents: function (eventName) {
            var eventArgs, strippedVal = this.get_StrippedValue(), unstrippedVal=this.get_UnstrippedValue();
            this.model.value = unstrippedVal;
            if (eventName == "change") {
                if (this.previousValue != this.model.value) {
                    this.previousValue = this.model.value;
                }
                else
                    return false;
                this._hiddenInput.val(strippedVal);
            }

            if (this.model[eventName]) {
                eventArgs = { value: unstrippedVal, unmaskedValue: strippedVal };
                this._trigger(eventName, eventArgs);
            }
        },
		/**
         * Section For set the value can be set after changes happened in maskedit textbox control.
		 * @private
         */
        _OnValueChange: function () {
            if (this._textbox.value == "" && this._maskModel != "")
                this._textbox.value = this._maskModel;
            this._setValue(this._textbox.value);
        }

    });
	/**
	 * Enum for slider type.	 
	 * @enum {string}
	 * @global 
	 */
    ej.InputMode = { 	
			 /**  support for user enter character in password format. */
			Password: "password", 
			/**  support for user enter character in normal format. */
			Text: "text"			
			};  
})(jQuery, Syncfusion);;
/**
* @fileOverview Plugin to style the Menu control.
* @copyright Copyright Syncfusion Inc. 2001 - 2013. All rights reserved.
*  Use of this code is subject to the terms of our license.
*  A copy of the current license can be obtained at any time by e-mailing
*  licensing@syncfusion.com. Any infringement will be prosecuted under
*  applicable laws. 
* @version 12.1 
* @author <a href="mailto:licensing@syncfusion.com">Syncfusion Inc</a>
*/
(function ($, ej, undefined) {

    /**
	
       * @namespace ej
       * @class ejMenu
       * @requires jQuery
       * @requires jquery.easing.1.3.min.js
       * @requires ej.core.js
       * @requires ej.data.js
       * @requires ej.menu.js
       * @requires ej.checkbox.js
       * @classdesc Custom Design for Menu control.
       * @example 
       * &lt;ul id="menu"&gt;
                      * &lt;li id="Home"&gt;&lt;a&gt;Home&lt;/a&gt;&lt;/li&gt;
                      * &lt;li&gt;&lt;a&gt;Search Jobs&lt;/a&gt;
                         * &lt;ul&gt;
                            * &lt;li&gt;&lt;a&gt;Advanced Search&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs by Company&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs by Category&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs by Location&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs by Skills&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs by Designation&lt;/a&gt;&lt;/li&gt;
                          * &lt;/ul&gt;
                      * &lt;/li&gt;
                      * &lt;li id="Post Resume"&gt;&lt;a&gt;Post Resume&lt;/a&gt;&lt;/li&gt;
                      * &lt;li id="Job Seeker"&gt;&lt;a&gt;JobSeeker Login&lt;/a&gt;&lt;/li&gt;
                      * &lt;li id="Fast Forward"&gt;&lt;a&gt;Fast Forward&lt;/a&gt;
                         * &lt;ul&gt;
                            * &lt;li&gt;&lt;a&gt;Resume writing&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Certification&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Resume Spotlight&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs4u&lt;/a&gt;&lt;/li&gt;
                         * &lt;/ul&gt;
                       * &lt;/li&gt;
                       * &lt;li id="More"&gt;&lt;a&gt;More&lt;/a&gt;
                          * &lt;ul&gt;
                             * &lt;li&gt;&lt;a&gt;Mobile&lt;/a&gt;&lt;/li&gt;
                             * &lt;li&gt;&lt;a&gt;Pay check&lt;/a&gt;&lt;/li&gt;
                             * &lt;li&gt;&lt;a&gt;Blog&lt;/a&gt;&lt;/li&gt;
                          * &lt;/ul&gt;
                        * &lt;/li&gt;
                    * &lt;/ul&gt;
   
       * <br> 
       * &lt;script&gt;
       * // Create Menu
       * $("#menu").ejMenu({height: 22}); 	
       * &lt;/script&gt;
       */

    ej.widget("ejMenu", "ej.Menu", {
	
        // widget element will be automatically set in this
        element: null,

        // user defined model will be automatically set in this
        model: null,
        validTags: ["ul"],
        _setFirst: false,
        _rootCss: "e-menu",

        // default model
        defaults: {
            /**		
                * Specifies the height of the root menu.
                * @default "auto"
                * @type {string | number}
                * @example 
                * &lt;ul id="menu"&gt;
                      * &lt;li id="Home"&gt;&lt;a&gt;Home&lt;/a&gt;&lt;/li&gt;
                      * &lt;li&gt;&lt;a&gt;Search Jobs&lt;/a&gt;
                         * &lt;ul&gt;
                            * &lt;li&gt;&lt;a&gt;Advanced Search&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs by Company&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs by Category&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs by Location&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs by Skills&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs by Designation&lt;/a&gt;&lt;/li&gt;
                          * &lt;/ul&gt;
                      * &lt;/li&gt;
                      * &lt;li id="Post Resume"&gt;&lt;a&gt;Post Resume&lt;/a&gt;&lt;/li&gt;
                      * &lt;li id="Job Seeker"&gt;&lt;a&gt;JobSeeker Login&lt;/a&gt;&lt;/li&gt;
                      * &lt;li id="Fast Forward"&gt;&lt;a&gt;Fast Forward&lt;/a&gt;
                         * &lt;ul&gt;
                            * &lt;li&gt;&lt;a&gt;Resume writing&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Certification&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Resume Spotlight&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs4u&lt;/a&gt;&lt;/li&gt;
                         * &lt;/ul&gt;
                       * &lt;/li&gt;
                       * &lt;li id="More"&gt;&lt;a&gt;More&lt;/a&gt;
                          * &lt;ul&gt;
                             * &lt;li&gt;&lt;a&gt;Mobile&lt;/a&gt;&lt;/li&gt;
                             * &lt;li&gt;&lt;a&gt;Pay check&lt;/a&gt;&lt;/li&gt;
                             * &lt;li&gt;&lt;a&gt;Blog&lt;/a&gt;&lt;/li&gt;
                          * &lt;/ul&gt;
                        * &lt;/li&gt;
                    * &lt;/ul&gt;
   
       * <br> 
       * &lt;script&gt;
                
               * //To set height API value during initialization  
                *   $("#menu").ejMenu({ height: 22 }); 
                * &lt;/script&gt; 
                 * @memberof ejMenu
                * @instance
                */
            height: "",
           /**		
			* Specifies the width of the main menu.
			* @default auto
			* @type {string | number}
			* @example 
			* &lt;ul id="menu"&gt;
                      * &lt;li id="Home"&gt;&lt;a&gt;Home&lt;/a&gt;&lt;/li&gt;
                      * &lt;li&gt;&lt;a&gt;Search Jobs&lt;/a&gt;
                         * &lt;ul&gt;
                            * &lt;li&gt;&lt;a&gt;Advanced Search&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs by Company&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs by Category&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs by Location&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs by Skills&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs by Designation&lt;/a&gt;&lt;/li&gt;
                          * &lt;/ul&gt;
                      * &lt;/li&gt;
                      * &lt;li id="Post Resume"&gt;&lt;a&gt;Post Resume&lt;/a&gt;&lt;/li&gt;
                      * &lt;li id="Job Seeker"&gt;&lt;a&gt;JobSeeker Login&lt;/a&gt;&lt;/li&gt;
                      * &lt;li id="Fast Forward"&gt;&lt;a&gt;Fast Forward&lt;/a&gt;
                         * &lt;ul&gt;
                            * &lt;li&gt;&lt;a&gt;Resume writing&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Certification&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Resume Spotlight&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs4u&lt;/a&gt;&lt;/li&gt;
                         * &lt;/ul&gt;
                       * &lt;/li&gt;
                       * &lt;li id="More"&gt;&lt;a&gt;More&lt;/a&gt;
                          * &lt;ul&gt;
                             * &lt;li&gt;&lt;a&gt;Mobile&lt;/a&gt;&lt;/li&gt;
                             * &lt;li&gt;&lt;a&gt;Pay check&lt;/a&gt;&lt;/li&gt;
                             * &lt;li&gt;&lt;a&gt;Blog&lt;/a&gt;&lt;/li&gt;
                          * &lt;/ul&gt;
                        * &lt;/li&gt;
                    * &lt;/ul&gt;
   
       * <br> 
       * &lt;script&gt;	
			* //To set width API value during initialization  
      *   //To set width API value 
      *   $("#menu").ejMenu({ width: "800px",height:"30px" });  
      * &lt;/script&gt;
			 * @memberof ejMenu
			* @instance
			*/
            width: "",
           /**		
			* To enable or disable the Animation while hover or click an menu items..See {@link AnimationType}
			* @default ej.animation.Default
			* @type {enum}
			* @example 
			* &lt;ul id="menu"&gt;
                      * &lt;li id="Home"&gt;&lt;a&gt;Home&lt;/a&gt;&lt;/li&gt;
                      * &lt;li&gt;&lt;a&gt;Search Jobs&lt;/a&gt;
                         * &lt;ul&gt;
                            * &lt;li&gt;&lt;a&gt;Advanced Search&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs by Company&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs by Category&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs by Location&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs by Skills&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs by Designation&lt;/a&gt;&lt;/li&gt;
                          * &lt;/ul&gt;
                      * &lt;/li&gt;
                      * &lt;li id="Post Resume"&gt;&lt;a&gt;Post Resume&lt;/a&gt;&lt;/li&gt;
                      * &lt;li id="Job Seeker"&gt;&lt;a&gt;JobSeeker Login&lt;/a&gt;&lt;/li&gt;
                      * &lt;li id="Fast Forward"&gt;&lt;a&gt;Fast Forward&lt;/a&gt;
                         * &lt;ul&gt;
                            * &lt;li&gt;&lt;a&gt;Resume writing&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Certification&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Resume Spotlight&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs4u&lt;/a&gt;&lt;/li&gt;
                         * &lt;/ul&gt;
                       * &lt;/li&gt;
                       * &lt;li id="More"&gt;&lt;a&gt;More&lt;/a&gt;
                          * &lt;ul&gt;
                             * &lt;li&gt;&lt;a&gt;Mobile&lt;/a&gt;&lt;/li&gt;
                             * &lt;li&gt;&lt;a&gt;Pay check&lt;/a&gt;&lt;/li&gt;
                             * &lt;li&gt;&lt;a&gt;Blog&lt;/a&gt;&lt;/li&gt;
                          * &lt;/ul&gt;
                        * &lt;/li&gt;
                    * &lt;/ul&gt;
   
       * <br> 
       * &lt;script&gt;	
		* //To set animationType API value during initialization  
      *   //To set animationType API value 
      *   $("#menu").ejMenu({ height:22,animationType: ej.AnimationType.Default }); 
      * &lt;/script&gt;
			 * @memberof ejMenu
			* @instance
			*/
            animationType: "default",
            /**		
			* Specifies the orientation of normal menu. Normal menu can rendered in horizontal or vertical direction by using this API. See {@link Direction}
			* @default ej.orientation.Horizontal
			* @type {string | enum}
			* @example 
			* &lt;ul id="menu"&gt;
                      * &lt;li id="Home"&gt;&lt;a&gt;Home&lt;/a&gt;&lt;/li&gt;
                      * &lt;li&gt;&lt;a&gt;Search Jobs&lt;/a&gt;
                         * &lt;ul&gt;
                            * &lt;li&gt;&lt;a&gt;Advanced Search&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs by Company&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs by Category&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs by Location&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs by Skills&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs by Designation&lt;/a&gt;&lt;/li&gt;
                          * &lt;/ul&gt;
                      * &lt;/li&gt;
                      * &lt;li id="Post Resume"&gt;&lt;a&gt;Post Resume&lt;/a&gt;&lt;/li&gt;
                      * &lt;li id="Job Seeker"&gt;&lt;a&gt;JobSeeker Login&lt;/a&gt;&lt;/li&gt;
                      * &lt;li id="Fast Forward"&gt;&lt;a&gt;Fast Forward&lt;/a&gt;
                         * &lt;ul&gt;
                            * &lt;li&gt;&lt;a&gt;Resume writing&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Certification&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Resume Spotlight&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs4u&lt;/a&gt;&lt;/li&gt;
                         * &lt;/ul&gt;
                       * &lt;/li&gt;
                       * &lt;li id="More"&gt;&lt;a&gt;More&lt;/a&gt;
                          * &lt;ul&gt;
                             * &lt;li&gt;&lt;a&gt;Mobile&lt;/a&gt;&lt;/li&gt;
                             * &lt;li&gt;&lt;a&gt;Pay check&lt;/a&gt;&lt;/li&gt;
                             * &lt;li&gt;&lt;a&gt;Blog&lt;/a&gt;&lt;/li&gt;
                          * &lt;/ul&gt;
                        * &lt;/li&gt;
                    * &lt;/ul&gt;
   
       * <br> 
       * &lt;script&gt;	
			* //To set orientation API value during initialization  
      *   //To set orientation API value 
      *   $("#menu").ejMenu({ height:22,orientation: ej.Orientation.Horizontal });
      * &lt;/script&gt;
			 * @memberof ejMenu
			* @instance
			*/
            orientation: ej.Orientation.Horizontal,
            /**		
			* Specifies the type of the menu. Essential JavaScript Menu consists of two type of menu, they are Normal Menu and Context Menu both Normal and Context Menu mode.See {@link MenuType}
			* @default ej.menuType.NormalMenu 
			* @type {string | enum}
			* @example 
			* &lt;ul id="menu"&gt;
                      * &lt;li id="Home"&gt;&lt;a&gt;Home&lt;/a&gt;&lt;/li&gt;
                      * &lt;li&gt;&lt;a&gt;Search Jobs&lt;/a&gt;
                         * &lt;ul&gt;
                            * &lt;li&gt;&lt;a&gt;Advanced Search&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs by Company&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs by Category&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs by Location&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs by Skills&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs by Designation&lt;/a&gt;&lt;/li&gt;
                          * &lt;/ul&gt;
                      * &lt;/li&gt;
                      * &lt;li id="Post Resume"&gt;&lt;a&gt;Post Resume&lt;/a&gt;&lt;/li&gt;
                      * &lt;li id="Job Seeker"&gt;&lt;a&gt;JobSeeker Login&lt;/a&gt;&lt;/li&gt;
                      * &lt;li id="Fast Forward"&gt;&lt;a&gt;Fast Forward&lt;/a&gt;
                         * &lt;ul&gt;
                            * &lt;li&gt;&lt;a&gt;Resume writing&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Certification&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Resume Spotlight&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs4u&lt;/a&gt;&lt;/li&gt;
                         * &lt;/ul&gt;
                       * &lt;/li&gt;
                       * &lt;li id="More"&gt;&lt;a&gt;More&lt;/a&gt;
                          * &lt;ul&gt;
                             * &lt;li&gt;&lt;a&gt;Mobile&lt;/a&gt;&lt;/li&gt;
                             * &lt;li&gt;&lt;a&gt;Pay check&lt;/a&gt;&lt;/li&gt;
                             * &lt;li&gt;&lt;a&gt;Blog&lt;/a&gt;&lt;/li&gt;
                          * &lt;/ul&gt;
                        * &lt;/li&gt;
                    * &lt;/ul&gt;
   
       * <br> 
       * &lt;script&gt;	
			* //To set menuType API value during initialization  
      *   //To set menuType API value 
      *   $("#menu").ejMenu({ height:22,menuType: "normalmenu" });
      * &lt;/script&gt; 
			 * @memberof ejMenu
			* @instance
			*/
            menuType: "normalmenu",
            /**		
			*Specifies the target id of context menu. On right clicking the specified contextTarget element, context menu gets shown.
			* @default null 
			* @type {string}
			* @example
			*	&lt;div id="target" class="textarea"&gt;
			*                            HTML is written in the form of HTML elements consisting of tags enclosed in angle
			*                            brackets (like
			*                            &lt;html&gt;
			*                            ),within the web page content. HTML tags most commonly come in pairs like and ,although
			*                            some tags, known as empty elements, are unpaired, for example
			*                            &lt;img&gt;. The purpose of a web browser is to read HTML documents and compose them into
			*                            visible or audible web pages. The browser does not display the HTML tags, but uses
			*                            the tags to interpret the content of the page.
			*                        &lt;/div&gt;
			*                        &lt;ul id="contextMenu"&gt;
			*                            &lt;li&gt;&lt;a&gt;Cut&lt;/a&gt;&lt;/li&gt; 
			*                            &lt;li&gt;&lt;a&gt;Copy&lt;/a&gt;&lt;/li&gt; 
			*                            &lt;li&gt;&lt;a&gt;Paste&lt;/a&gt;&lt;/li&gt; 
			*                       &lt;li class="separator"&gt;&lt;/li&gt; 
			*                            &lt;li&gt;&lt;a&gt;Comments&lt;/a&gt;&lt;/li&gt;
			*                            &lt;li&gt;&lt;a&gt;Links&lt;/a&gt;&lt;/li&gt;
			*                            &lt;li&gt;&lt;a&gt;Clear Formatting&lt;/a&gt;&lt;/li&gt;  
			*                        &lt;/ul&gt;           
			                
      * &lt;script&gt;
		
			* //To set contextMenuTarget API value during initialization  
			* 	//To set contextMenuTarget API value 
			* 	$("#contextMenu").ejMenu({menuType:ej.MenuType.ContextMenu, height:22,contextMenuTarget:"#target"});  
      * &lt;/script&gt;
			 * @memberof ejMenu
			* @instance
			*/
            contextMenuTarget: null,
            /**		
			* Specify the CSS class to Menu to achieve custom theme.
			* @default ""
			* @type {string}
			* @example 
        * &lt;ul id="menu"&gt;
                      * &lt;li id="Home"&gt;&lt;a&gt;Home&lt;/a&gt;&lt;/li&gt;
                      * &lt;li&gt;&lt;a&gt;Search Jobs&lt;/a&gt;
                         * &lt;ul&gt;
                            * &lt;li&gt;&lt;a&gt;Advanced Search&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs by Company&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs by Category&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs by Location&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs by Skills&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs by Designation&lt;/a&gt;&lt;/li&gt;
                          * &lt;/ul&gt;
                      * &lt;/li&gt;
                      * &lt;li id="Post Resume"&gt;&lt;a&gt;Post Resume&lt;/a&gt;&lt;/li&gt;
                      * &lt;li id="Job Seeker"&gt;&lt;a&gt;JobSeeker Login&lt;/a&gt;&lt;/li&gt;
                      * &lt;li id="Fast Forward"&gt;&lt;a&gt;Fast Forward&lt;/a&gt;
                         * &lt;ul&gt;
                            * &lt;li&gt;&lt;a&gt;Resume writing&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Certification&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Resume Spotlight&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs4u&lt;/a&gt;&lt;/li&gt;
                         * &lt;/ul&gt;
                       * &lt;/li&gt;
                       * &lt;li id="More"&gt;&lt;a&gt;More&lt;/a&gt;
                          * &lt;ul&gt;
                             * &lt;li&gt;&lt;a&gt;Mobile&lt;/a&gt;&lt;/li&gt;
                             * &lt;li&gt;&lt;a&gt;Pay check&lt;/a&gt;&lt;/li&gt;
                             * &lt;li&gt;&lt;a&gt;Blog&lt;/a&gt;&lt;/li&gt;
                          * &lt;/ul&gt;
                        * &lt;/li&gt;
                    * &lt;/ul&gt;
   
       * <br> 
	   * &lt;script&gt;
			* //To Set the CSS class during initialization. 			
			* 	$("#menu").ejMenu({ height:22, cssClass:'gradient-lime ' });		
			* &lt;/script&gt;
			 * @memberof ejMenu
			* @instance
			*/
            cssClass: "",
            /**		
			* Specifies the sub menu items to be show or open only on click.
			* @default false
			* @type {boolean}
			* @example 
        * &lt;ul id="menu"&gt;
                      * &lt;li id="Home"&gt;&lt;a&gt;Home&lt;/a&gt;&lt;/li&gt;
                      * &lt;li&gt;&lt;a&gt;Search Jobs&lt;/a&gt;
                         * &lt;ul&gt;
                            * &lt;li&gt;&lt;a&gt;Advanced Search&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs by Company&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs by Category&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs by Location&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs by Skills&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs by Designation&lt;/a&gt;&lt;/li&gt;
                          * &lt;/ul&gt;
                      * &lt;/li&gt;
                      * &lt;li id="Post Resume"&gt;&lt;a&gt;Post Resume&lt;/a&gt;&lt;/li&gt;
                      * &lt;li id="Job Seeker"&gt;&lt;a&gt;JobSeeker Login&lt;/a&gt;&lt;/li&gt;
                      * &lt;li id="Fast Forward"&gt;&lt;a&gt;Fast Forward&lt;/a&gt;
                         * &lt;ul&gt;
                            * &lt;li&gt;&lt;a&gt;Resume writing&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Certification&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Resume Spotlight&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs4u&lt;/a&gt;&lt;/li&gt;
                         * &lt;/ul&gt;
                       * &lt;/li&gt;
                       * &lt;li id="More"&gt;&lt;a&gt;More&lt;/a&gt;
                          * &lt;ul&gt;
                             * &lt;li&gt;&lt;a&gt;Mobile&lt;/a&gt;&lt;/li&gt;
                             * &lt;li&gt;&lt;a&gt;Pay check&lt;/a&gt;&lt;/li&gt;
                             * &lt;li&gt;&lt;a&gt;Blog&lt;/a&gt;&lt;/li&gt;
                          * &lt;/ul&gt;
                        * &lt;/li&gt;
                    * &lt;/ul&gt;
   
       * <br> 
       * &lt;script&gt; 
			* //To set openOnClick API value during initialization  
			* 	//To set openOnClick API value 
			* 	$("#menu").ejMenu({ height:22,openOnClick: true });
			* &lt;/script&gt;
			 * @memberof ejMenu
			* @instance
			*/
            openOnClick: false,
            /**		
			* Specifies the sub menu popup direction.See {@link Direction}
			* @default ej.direction.Right
			* @type {string | enum}
			* @example 
       * &lt;ul id="menu"&gt;
                      * &lt;li id="Home"&gt;&lt;a&gt;Home&lt;/a&gt;&lt;/li&gt;
                      * &lt;li&gt;&lt;a&gt;Search Jobs&lt;/a&gt;
                         * &lt;ul&gt;
                            * &lt;li&gt;&lt;a&gt;Advanced Search&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs by Company&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs by Category&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs by Location&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs by Skills&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs by Designation&lt;/a&gt;&lt;/li&gt;
                          * &lt;/ul&gt;
                      * &lt;/li&gt;
                      * &lt;li id="Post Resume"&gt;&lt;a&gt;Post Resume&lt;/a&gt;&lt;/li&gt;
                      * &lt;li id="Job Seeker"&gt;&lt;a&gt;JobSeeker Login&lt;/a&gt;&lt;/li&gt;
                      * &lt;li id="Fast Forward"&gt;&lt;a&gt;Fast Forward&lt;/a&gt;
                         * &lt;ul&gt;
                            * &lt;li&gt;&lt;a&gt;Resume writing&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Certification&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Resume Spotlight&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs4u&lt;/a&gt;&lt;/li&gt;
                         * &lt;/ul&gt;
                       * &lt;/li&gt;
                       * &lt;li id="More"&gt;&lt;a&gt;More&lt;/a&gt;
                          * &lt;ul&gt;
                             * &lt;li&gt;&lt;a&gt;Mobile&lt;/a&gt;&lt;/li&gt;
                             * &lt;li&gt;&lt;a&gt;Pay check&lt;/a&gt;&lt;/li&gt;
                             * &lt;li&gt;&lt;a&gt;Blog&lt;/a&gt;&lt;/li&gt;
                          * &lt;/ul&gt;
                        * &lt;/li&gt;
                    * &lt;/ul&gt;
   
       * <br> 
       * &lt;script&gt; 
			* //To set subMenuDirection API value during initialization  
			* 	//To set subMenuDirection API value 
			* 	$("#menu").ejMenu({ height:22,subMenuDirection: ej.Direction.Right });
			* &lt;/script&gt;
			 * @memberof ejMenu
			* @instance
			*/
            subMenuDirection: "right",
			/**		
			* Specifies the root menu items to be aligned center in horizontal menu.
			* @default false
			* @type {boolean}
			* @example 
       * &lt;ul id="menu"&gt;
                      * &lt;li id="Home"&gt;&lt;a&gt;Home&lt;/a&gt;&lt;/li&gt;
                      * &lt;li&gt;&lt;a&gt;Search Jobs&lt;/a&gt;
                         * &lt;ul&gt;
                            * &lt;li&gt;&lt;a&gt;Advanced Search&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs by Company&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs by Category&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs by Location&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs by Skills&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs by Designation&lt;/a&gt;&lt;/li&gt;
                          * &lt;/ul&gt;
                      * &lt;/li&gt;
                      * &lt;li id="Post Resume"&gt;&lt;a&gt;Post Resume&lt;/a&gt;&lt;/li&gt;
                      * &lt;li id="Job Seeker"&gt;&lt;a&gt;JobSeeker Login&lt;/a&gt;&lt;/li&gt;
                      * &lt;li id="Fast Forward"&gt;&lt;a&gt;Fast Forward&lt;/a&gt;
                         * &lt;ul&gt;
                            * &lt;li&gt;&lt;a&gt;Resume writing&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Certification&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Resume Spotlight&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs4u&lt;/a&gt;&lt;/li&gt;
                         * &lt;/ul&gt;
                       * &lt;/li&gt;
                       * &lt;li id="More"&gt;&lt;a&gt;More&lt;/a&gt;
                          * &lt;ul&gt;
                             * &lt;li&gt;&lt;a&gt;Mobile&lt;/a&gt;&lt;/li&gt;
                             * &lt;li&gt;&lt;a&gt;Pay check&lt;/a&gt;&lt;/li&gt;
                             * &lt;li&gt;&lt;a&gt;Blog&lt;/a&gt;&lt;/li&gt;
                          * &lt;/ul&gt;
                        * &lt;/li&gt;
                    * &lt;/ul&gt;
   
       * <br> 
       * &lt;script&gt; 
			* //To set enableCenterAlign API value during initialization  
			* 	//To set enableCenterAlign API value 
			* 	$("#menu").ejMenu({ height:22,enableCenterAlign: true });
			* &lt;/script&gt;
			 * @memberof ejMenu
			* @instance
			*/
			enableCenterAlign: false,
            /**		
			* Specifies the main menu items arrows only to be shown if it contains child items.
			* @default true
			* @type {boolean}
			* @example 
       * &lt;ul id="menu"&gt;
                      * &lt;li id="Home"&gt;&lt;a&gt;Home&lt;/a&gt;&lt;/li&gt;
                      * &lt;li&gt;&lt;a&gt;Search Jobs&lt;/a&gt;
                         * &lt;ul&gt;
                            * &lt;li&gt;&lt;a&gt;Advanced Search&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs by Company&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs by Category&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs by Location&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs by Skills&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs by Designation&lt;/a&gt;&lt;/li&gt;
                          * &lt;/ul&gt;
                      * &lt;/li&gt;
                      * &lt;li id="Post Resume"&gt;&lt;a&gt;Post Resume&lt;/a&gt;&lt;/li&gt;
                      * &lt;li id="Job Seeker"&gt;&lt;a&gt;JobSeeker Login&lt;/a&gt;&lt;/li&gt;
                      * &lt;li id="Fast Forward"&gt;&lt;a&gt;Fast Forward&lt;/a&gt;
                         * &lt;ul&gt;
                            * &lt;li&gt;&lt;a&gt;Resume writing&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Certification&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Resume Spotlight&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs4u&lt;/a&gt;&lt;/li&gt;
                         * &lt;/ul&gt;
                       * &lt;/li&gt;
                       * &lt;li id="More"&gt;&lt;a&gt;More&lt;/a&gt;
                          * &lt;ul&gt;
                             * &lt;li&gt;&lt;a&gt;Mobile&lt;/a&gt;&lt;/li&gt;
                             * &lt;li&gt;&lt;a&gt;Pay check&lt;/a&gt;&lt;/li&gt;
                             * &lt;li&gt;&lt;a&gt;Blog&lt;/a&gt;&lt;/li&gt;
                          * &lt;/ul&gt;
                        * &lt;/li&gt;
                    * &lt;/ul&gt;
   
       * <br> 
       * &lt;script&gt; 
			* //To set showRooltLevelArrows API value during initialization  
			* 	//To set showRooltLevelArrows API value 
			* 	$("#menu").ejMenu({ height:22,showRooltLevelArrows: true });
			* &lt;/script&gt;
			 * @memberof ejMenu
			* @instance
			*/
            showRooltLevelArrows: true,
            /**		
			* Specifies the sub menu items arrows only to be shown if it contains child items.
			* @default true
			* @type {boolean}
			* @example 
       * &lt;ul id="menu"&gt;
                      * &lt;li id="Home"&gt;&lt;a&gt;Home&lt;/a&gt;&lt;/li&gt;
                      * &lt;li&gt;&lt;a&gt;Search Jobs&lt;/a&gt;
                         * &lt;ul&gt;
                            * &lt;li&gt;&lt;a&gt;Advanced Search&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs by Company&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs by Category&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs by Location&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs by Skills&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs by Designation&lt;/a&gt;&lt;/li&gt;
                          * &lt;/ul&gt;
                      * &lt;/li&gt;
                      * &lt;li id="Post Resume"&gt;&lt;a&gt;Post Resume&lt;/a&gt;&lt;/li&gt;
                      * &lt;li id="Job Seeker"&gt;&lt;a&gt;JobSeeker Login&lt;/a&gt;&lt;/li&gt;
                      * &lt;li id="Fast Forward"&gt;&lt;a&gt;Fast Forward&lt;/a&gt;
                         * &lt;ul&gt;
                            * &lt;li&gt;&lt;a&gt;Resume writing&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Certification&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Resume Spotlight&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs4u&lt;/a&gt;&lt;/li&gt;
                         * &lt;/ul&gt;
                       * &lt;/li&gt;
                       * &lt;li id="More"&gt;&lt;a&gt;More&lt;/a&gt;
                          * &lt;ul&gt;
                             * &lt;li&gt;&lt;a&gt;Mobile&lt;/a&gt;&lt;/li&gt;
                             * &lt;li&gt;&lt;a&gt;Pay check&lt;/a&gt;&lt;/li&gt;
                             * &lt;li&gt;&lt;a&gt;Blog&lt;/a&gt;&lt;/li&gt;
                          * &lt;/ul&gt;
                        * &lt;/li&gt;
                    * &lt;/ul&gt;
   
       * <br> 
       * &lt;script&gt; 
			* //To set showSubLevelArrows API value during initialization  
			* 	//To set showSubLevelArrows API value 
			* 	$("#menu").ejMenu({ height:22,showSubLevelArrows: true });
			* &lt;/script&gt;
			 * @memberof ejMenu
			* @instance
			*/
			
            showSubLevelArrows: true,
			enableAnimation: true,
             /**		
/**		
			* When this property sets to false, the menu list is displayed without any separators.
			* @default true
			* @type {boolean}
			* @example 
       * &lt;ul id="menu"&gt;
                      * &lt;li id="Home"&gt;&lt;a&gt;Home&lt;/a&gt;&lt;/li&gt;
                      * &lt;li&gt;&lt;a&gt;Search Jobs&lt;/a&gt;
                         * &lt;ul&gt;
                            * &lt;li&gt;&lt;a&gt;Advanced Search&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs by Company&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs by Category&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs by Location&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs by Skills&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs by Designation&lt;/a&gt;&lt;/li&gt;
                          * &lt;/ul&gt;
                      * &lt;/li&gt;
                      * &lt;li id="Post Resume"&gt;&lt;a&gt;Post Resume&lt;/a&gt;&lt;/li&gt;
                      * &lt;li id="Job Seeker"&gt;&lt;a&gt;JobSeeker Login&lt;/a&gt;&lt;/li&gt;
                      * &lt;li id="Fast Forward"&gt;&lt;a&gt;Fast Forward&lt;/a&gt;
                         * &lt;ul&gt;
                            * &lt;li&gt;&lt;a&gt;Resume writing&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Certification&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Resume Spotlight&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs4u&lt;/a&gt;&lt;/li&gt;
                         * &lt;/ul&gt;
                       * &lt;/li&gt;
                       * &lt;li id="More"&gt;&lt;a&gt;More&lt;/a&gt;
                          * &lt;ul&gt;
                             * &lt;li&gt;&lt;a&gt;Mobile&lt;/a&gt;&lt;/li&gt;
                             * &lt;li&gt;&lt;a&gt;Pay check&lt;/a&gt;&lt;/li&gt;
                             * &lt;li&gt;&lt;a&gt;Blog&lt;/a&gt;&lt;/li&gt;
                          * &lt;/ul&gt;
                        * &lt;/li&gt;
                    * &lt;/ul&gt;
   
       * <br> 
       * &lt;script&gt; 
			* //To set enableSeparator API value during initialization  
			* 	//To set enableSeparator API value 
			* 	$("#menu").ejMenu({ height:22,enableSeparator: true });
			* &lt;/script&gt;
			 * @memberof ejMenu
			* @instance
			*/
            enableSeparator: true,
           /**		
			* Enable / Disable the Menu control.
			* @default true
			* @type {boolean}
			* @example 
       * &lt;ul id="menu"&gt;
                      * &lt;li id="Home"&gt;&lt;a&gt;Home&lt;/a&gt;&lt;/li&gt;
                      * &lt;li&gt;&lt;a&gt;Search Jobs&lt;/a&gt;
                         * &lt;ul&gt;
                            * &lt;li&gt;&lt;a&gt;Advanced Search&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs by Company&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs by Category&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs by Location&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs by Skills&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs by Designation&lt;/a&gt;&lt;/li&gt;
                          * &lt;/ul&gt;
                      * &lt;/li&gt;
                      * &lt;li id="Post Resume"&gt;&lt;a&gt;Post Resume&lt;/a&gt;&lt;/li&gt;
                      * &lt;li id="Job Seeker"&gt;&lt;a&gt;JobSeeker Login&lt;/a&gt;&lt;/li&gt;
                      * &lt;li id="Fast Forward"&gt;&lt;a&gt;Fast Forward&lt;/a&gt;
                         * &lt;ul&gt;
                            * &lt;li&gt;&lt;a&gt;Resume writing&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Certification&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Resume Spotlight&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs4u&lt;/a&gt;&lt;/li&gt;
                         * &lt;/ul&gt;
                       * &lt;/li&gt;
                       * &lt;li id="More"&gt;&lt;a&gt;More&lt;/a&gt;
                          * &lt;ul&gt;
                             * &lt;li&gt;&lt;a&gt;Mobile&lt;/a&gt;&lt;/li&gt;
                             * &lt;li&gt;&lt;a&gt;Pay check&lt;/a&gt;&lt;/li&gt;
                             * &lt;li&gt;&lt;a&gt;Blog&lt;/a&gt;&lt;/li&gt;
                          * &lt;/ul&gt;
                        * &lt;/li&gt;
                    * &lt;/ul&gt;
   
       * <br> 
       * &lt;script&gt; 
			* //To set enabled API value during initialization  
			* 	//To set enabled API value 
			* 	$("#menu").ejMenu({ height:22,enabled: true });
			* &lt;/script&gt;
			 * @memberof ejMenu
			* @instance
			*/
            enabled: true,
             /**		
			* Fields used to bind the data source and it includes following field members to make databind easier.
			* @default null
			* @type {Object}
			* @example
      *&lt;ul id="menu"&gt;
       * &lt;script&gt; 
			* //To set fields API value during initialization  
			* 	//To set fields API value 
			* 	      $("#menu").ejMenu({ height:22,
            *                fields: { dataSource: window.menu, id: "id", parentId: "parentId", text: "text", spriteCssClass: "sprite" }
            *           });
             * &lt;/script&gt; 
			 * @memberof ejMenu
			* @instance
			*/
            fields: /** @lends ejMenu# */{
                /**		
                * It receives the child data for the inner level. 
                * @alias ejMenu#fields->child
				* @type Object
                */
                child: null,
                 /**		
                * datasource receives  Essential DataManager object and JSON object. 
                * @alias ejMenu#fields->dataSource
				* @type Object
                */
                dataSource: null,
                /**		
				* It receives query to retrieve data from the table (query is same as SQL).  
				* @alias ejMenu#fields->query
				* @type Object
				*/
                query: null,
                /**	
                * It receives table name to execute query on the corresponding table  
				* @alias ejMenu#fields->tableName
				* @type string
				*/
                tableName: null,
                /**	
                * Specifies the id to menu items list
               	* @alias ejMenu#fields->id
				* @type string
				*/
                id:"id",
                /**	
                * Specifies the parent id of the table.
              	* @alias ejMenu#fields->parentId
				* @type string
                */
                parentId:"parentId",
                /**	
               * Specifies the text of menu items list.
             	* @alias ejMenu#fields->text
				* @type string
				*/
                text:"text",
                /**	
                * Specifies the sprite CSS class to “li” item list
              	* @alias ejMenu#fields->spriteCssClass
				* @type string
				*/
                spriteCssClass:"spriteCssClass",
                /**	
                * Specifies the url to the anchor tag in menu item list.
             	* @alias ejMenu#fields->url
				* @type string
				*/
                url: "url",
                /**	
                * Specifies the image attribute to “img” tag inside items list. 
             	* @alias ejMenu#fields->imageAttribute
				* @type string
				*/
                imageAttribute:"imageAttribute",
                /**	
                * Specifies the html attributes to “li” item list.
                * @alias ejMenu#fields->htmlAttribute
				* @type string
				*/
                htmlAttribute:"htmlAttribute",
                /**	
                * Adds custom attributes like "target" to the anchor tag of the menu items.
                * @alias ejMenu#fields->linkAttribute
                * @type string
                 */
                linkAttribute: "linkAttribute",
                /**	
                * Specifies the image URL to “img” tag inside item list.
              	* @alias ejMenu#fields->imageUrl
				* @type string
				*/
                imageUrl:"imageUrl",
            },
            /**		
			* Specifies the menu items to be displayed in right to left direction.
			* @default false
			* @type {boolean}
			* @example 
         * &lt;ul id="menu"&gt;
                      * &lt;li id="Home"&gt;&lt;a&gt;Home&lt;/a&gt;&lt;/li&gt;
                      * &lt;li&gt;&lt;a&gt;Search Jobs&lt;/a&gt;
                         * &lt;ul&gt;
                            * &lt;li&gt;&lt;a&gt;Advanced Search&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs by Company&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs by Category&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs by Location&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs by Skills&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs by Designation&lt;/a&gt;&lt;/li&gt;
                          * &lt;/ul&gt;
                      * &lt;/li&gt;
                      * &lt;li id="Post Resume"&gt;&lt;a&gt;Post Resume&lt;/a&gt;&lt;/li&gt;
                      * &lt;li id="Job Seeker"&gt;&lt;a&gt;JobSeeker Login&lt;/a&gt;&lt;/li&gt;
                      * &lt;li id="Fast Forward"&gt;&lt;a&gt;Fast Forward&lt;/a&gt;
                         * &lt;ul&gt;
                            * &lt;li&gt;&lt;a&gt;Resume writing&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Certification&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Resume Spotlight&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs4u&lt;/a&gt;&lt;/li&gt;
                         * &lt;/ul&gt;
                       * &lt;/li&gt;
                       * &lt;li id="More"&gt;&lt;a&gt;More&lt;/a&gt;
                          * &lt;ul&gt;
                             * &lt;li&gt;&lt;a&gt;Mobile&lt;/a&gt;&lt;/li&gt;
                             * &lt;li&gt;&lt;a&gt;Pay check&lt;/a&gt;&lt;/li&gt;
                             * &lt;li&gt;&lt;a&gt;Blog&lt;/a&gt;&lt;/li&gt;
                          * &lt;/ul&gt;
                        * &lt;/li&gt;
                    * &lt;/ul&gt;
   
       * <br> 
       * &lt;script&gt; 
			* //To set rtl API value during initialization  
			* 	//To set rtl API value 
			* 	$("#menu").ejMenu({ height:22,enableRTL: true });
		* &lt;/script&gt;
			 * @memberof ejMenu
			* @instance
			*/
            enableRTL: false,
            /**		
			* Specifies the title to responsive menu.
			* @default "Menu"
			* @type {string}
			* @example 
         * &lt;ul id="menu"&gt;
                      * &lt;li id="Home"&gt;&lt;a&gt;Home&lt;/a&gt;&lt;/li&gt;
                      * &lt;li&gt;&lt;a&gt;Search Jobs&lt;/a&gt;
                         * &lt;ul&gt;
                            * &lt;li&gt;&lt;a&gt;Advanced Search&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs by Company&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs by Category&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs by Location&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs by Skills&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs by Designation&lt;/a&gt;&lt;/li&gt;
                          * &lt;/ul&gt;
                      * &lt;/li&gt;
                      * &lt;li id="Post Resume"&gt;&lt;a&gt;Post Resume&lt;/a&gt;&lt;/li&gt;
                      * &lt;li id="Job Seeker"&gt;&lt;a&gt;JobSeeker Login&lt;/a&gt;&lt;/li&gt;
                      * &lt;li id="Fast Forward"&gt;&lt;a&gt;Fast Forward&lt;/a&gt;
                         * &lt;ul&gt;
                            * &lt;li&gt;&lt;a&gt;Resume writing&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Certification&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Resume Spotlight&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs4u&lt;/a&gt;&lt;/li&gt;
                         * &lt;/ul&gt;
                       * &lt;/li&gt;
                       * &lt;li id="More"&gt;&lt;a&gt;More&lt;/a&gt;
                          * &lt;ul&gt;
                             * &lt;li&gt;&lt;a&gt;Mobile&lt;/a&gt;&lt;/li&gt;
                             * &lt;li&gt;&lt;a&gt;Pay check&lt;/a&gt;&lt;/li&gt;
                             * &lt;li&gt;&lt;a&gt;Blog&lt;/a&gt;&lt;/li&gt;
                          * &lt;/ul&gt;
                        * &lt;/li&gt;
                    * &lt;/ul&gt;
   
       * <br> 
       * &lt;script&gt; 
			* //To set titleText API value during initialization  
			* 	//To set titleText API value 
			* 	$("#menu").ejMenu({ height:22,titleText: "Menu" });
			 * &lt;/script&gt;
			 * @memberof ejMenu
			* @instance
			*/
            titleText: "Menu",
            /**     
            * Fires before context menu gets open.
            * @event
            * @name ejMenu#beforeContextOpen 	
            * @param {Object} argument Event parameters from context menu     
            * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
            * @param {object}  argument.model returns the menu model
            * @param {string}  argument.type returns the name of the event
            * @param {object}  argument.target returns the target element
            * @example 
               * &lt;ul id="menu"&gt;
                      * &lt;li id="Home"&gt;&lt;a&gt;Home&lt;/a&gt;&lt;/li&gt;
                      * &lt;li&gt;&lt;a&gt;Search Jobs&lt;/a&gt;
                         * &lt;ul&gt;
                            * &lt;li&gt;&lt;a&gt;Advanced Search&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs by Company&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs by Category&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs by Location&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs by Skills&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs by Designation&lt;/a&gt;&lt;/li&gt;
                          * &lt;/ul&gt;
                      * &lt;/li&gt;
                      * &lt;li id="Post Resume"&gt;&lt;a&gt;Post Resume&lt;/a&gt;&lt;/li&gt;
                      * &lt;li id="Job Seeker"&gt;&lt;a&gt;JobSeeker Login&lt;/a&gt;&lt;/li&gt;
                      * &lt;li id="Fast Forward"&gt;&lt;a&gt;Fast Forward&lt;/a&gt;
                         * &lt;ul&gt;
                            * &lt;li&gt;&lt;a&gt;Resume writing&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Certification&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Resume Spotlight&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs4u&lt;/a&gt;&lt;/li&gt;
                         * &lt;/ul&gt;
                       * &lt;/li&gt;
                       * &lt;li id="More"&gt;&lt;a&gt;More&lt;/a&gt;
                          * &lt;ul&gt;
                             * &lt;li&gt;&lt;a&gt;Mobile&lt;/a&gt;&lt;/li&gt;
                             * &lt;li&gt;&lt;a&gt;Pay check&lt;/a&gt;&lt;/li&gt;
                             * &lt;li&gt;&lt;a&gt;Blog&lt;/a&gt;&lt;/li&gt;
                          * &lt;/ul&gt;
                        * &lt;/li&gt;
                    * &lt;/ul&gt;
   
       * <br> 
       * &lt;script&gt; 
            *  //before context menu open event for menu
            *  $("#menu").ejMenu({height:22,
            *  beforeOpen: function (args) {}
            *  });   
            * &lt;/script&gt;
            * @memberof ejMenu
            * @instance
            */
            beforeOpen: null,
            /**     
            * Fires when context menu on open.
            * @event
            * @name ejMenu#contextOpen 	
            * @param {Object} argument Event parameters from menu     
            * @param {object}  argument.model returns the menu model
            * @param {string}  argument.type returns the name of the event
            * @param {object}  argument.target returns the target element
            * @example 
               * &lt;ul id="menu"&gt;
                      * &lt;li id="Home"&gt;&lt;a&gt;Home&lt;/a&gt;&lt;/li&gt;
                      * &lt;li&gt;&lt;a&gt;Search Jobs&lt;/a&gt;
                         * &lt;ul&gt;
                            * &lt;li&gt;&lt;a&gt;Advanced Search&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs by Company&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs by Category&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs by Location&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs by Skills&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs by Designation&lt;/a&gt;&lt;/li&gt;
                          * &lt;/ul&gt;
                      * &lt;/li&gt;
                      * &lt;li id="Post Resume"&gt;&lt;a&gt;Post Resume&lt;/a&gt;&lt;/li&gt;
                      * &lt;li id="Job Seeker"&gt;&lt;a&gt;JobSeeker Login&lt;/a&gt;&lt;/li&gt;
                      * &lt;li id="Fast Forward"&gt;&lt;a&gt;Fast Forward&lt;/a&gt;
                         * &lt;ul&gt;
                            * &lt;li&gt;&lt;a&gt;Resume writing&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Certification&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Resume Spotlight&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs4u&lt;/a&gt;&lt;/li&gt;
                         * &lt;/ul&gt;
                       * &lt;/li&gt;
                       * &lt;li id="More"&gt;&lt;a&gt;More&lt;/a&gt;
                          * &lt;ul&gt;
                             * &lt;li&gt;&lt;a&gt;Mobile&lt;/a&gt;&lt;/li&gt;
                             * &lt;li&gt;&lt;a&gt;Pay check&lt;/a&gt;&lt;/li&gt;
                             * &lt;li&gt;&lt;a&gt;Blog&lt;/a&gt;&lt;/li&gt;
                          * &lt;/ul&gt;
                        * &lt;/li&gt;
                    * &lt;/ul&gt;
   
       * <br> 
       * &lt;script&gt; 
            *  //context menu open event for menu
            *  $("#menu").ejMenu({height:22,
            *  open: function (args) {}
            *  });   
            * &lt;/script&gt;
            * @memberof ejMenu
            * @instance
            */
            open: null,
            /**     
			 * Fire when context menu on close.
			 * @event
			 * @name ejMenu#contextClose 	
			 * @param {Object} argument Event parameters from menu     
			 * @param {object}  argument.model returns the menu model
			 * @param {string}  argument.type returns the name of the event
			 * @param {object}  argument.target returns the target element
			 * @example 
          * &lt;ul id="menu"&gt;
                      * &lt;li id="Home"&gt;&lt;a&gt;Home&lt;/a&gt;&lt;/li&gt;
                      * &lt;li&gt;&lt;a&gt;Search Jobs&lt;/a&gt;
                         * &lt;ul&gt;
                            * &lt;li&gt;&lt;a&gt;Advanced Search&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs by Company&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs by Category&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs by Location&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs by Skills&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs by Designation&lt;/a&gt;&lt;/li&gt;
                          * &lt;/ul&gt;
                      * &lt;/li&gt;
                      * &lt;li id="Post Resume"&gt;&lt;a&gt;Post Resume&lt;/a&gt;&lt;/li&gt;
                      * &lt;li id="Job Seeker"&gt;&lt;a&gt;JobSeeker Login&lt;/a&gt;&lt;/li&gt;
                      * &lt;li id="Fast Forward"&gt;&lt;a&gt;Fast Forward&lt;/a&gt;
                         * &lt;ul&gt;
                            * &lt;li&gt;&lt;a&gt;Resume writing&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Certification&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Resume Spotlight&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs4u&lt;/a&gt;&lt;/li&gt;
                         * &lt;/ul&gt;
                       * &lt;/li&gt;
                       * &lt;li id="More"&gt;&lt;a&gt;More&lt;/a&gt;
                          * &lt;ul&gt;
                             * &lt;li&gt;&lt;a&gt;Mobile&lt;/a&gt;&lt;/li&gt;
                             * &lt;li&gt;&lt;a&gt;Pay check&lt;/a&gt;&lt;/li&gt;
                             * &lt;li&gt;&lt;a&gt;Blog&lt;/a&gt;&lt;/li&gt;
                          * &lt;/ul&gt;
                        * &lt;/li&gt;
                    * &lt;/ul&gt;
   
       * <br> 
       * &lt;script&gt; 
			 *  //context menu close event for menu
			 *  $("#menu").ejMenu({height:22,
             *  close: function (args) {}
			 *  });   
       * &lt;/script&gt;
			 * @memberof ejMenu
			 * @instance
			 */
            close: null,
             /**     
			 * Fires when mouse over the Menu items.
			 * @event
			 * @name ejMenu#mouseover 	
			 * @param {Object}  argument Event parameters from menu     
			 * @param {object}  argument.model returns the menu model
			 * @param {string}  argument.type returns the name of the event
			 * @param {string}  argument.text returns clicked menu item text
             * @param {object}  argument.element returns clicked menu item element
			 * @param {object}  argument.event returns the event
			 * @example 
          * &lt;ul id="menu"&gt;
                      * &lt;li id="Home"&gt;&lt;a&gt;Home&lt;/a&gt;&lt;/li&gt;
                      * &lt;li&gt;&lt;a&gt;Search Jobs&lt;/a&gt;
                         * &lt;ul&gt;
                            * &lt;li&gt;&lt;a&gt;Advanced Search&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs by Company&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs by Category&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs by Location&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs by Skills&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs by Designation&lt;/a&gt;&lt;/li&gt;
                          * &lt;/ul&gt;
                      * &lt;/li&gt;
                      * &lt;li id="Post Resume"&gt;&lt;a&gt;Post Resume&lt;/a&gt;&lt;/li&gt;
                      * &lt;li id="Job Seeker"&gt;&lt;a&gt;JobSeeker Login&lt;/a&gt;&lt;/li&gt;
                      * &lt;li id="Fast Forward"&gt;&lt;a&gt;Fast Forward&lt;/a&gt;
                         * &lt;ul&gt;
                            * &lt;li&gt;&lt;a&gt;Resume writing&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Certification&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Resume Spotlight&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs4u&lt;/a&gt;&lt;/li&gt;
                         * &lt;/ul&gt;
                       * &lt;/li&gt;
                       * &lt;li id="More"&gt;&lt;a&gt;More&lt;/a&gt;
                          * &lt;ul&gt;
                             * &lt;li&gt;&lt;a&gt;Mobile&lt;/a&gt;&lt;/li&gt;
                             * &lt;li&gt;&lt;a&gt;Pay check&lt;/a&gt;&lt;/li&gt;
                             * &lt;li&gt;&lt;a&gt;Blog&lt;/a&gt;&lt;/li&gt;
                          * &lt;/ul&gt;
                        * &lt;/li&gt;
                    * &lt;/ul&gt;
   
       * <br> 
       * &lt;script&gt; 
			 *   //mouse over event for menu
			 *  $("#menu").ejMenu({height:22,
             *  mouseover: function (args) {}
			 *  });   
       * &lt;/script&gt;
			 * @memberof ejMenu
			 * @instance
			 */
            mouseover: null,
            /**     
            * Fires when mouse out from menu items.
            * @event
            * @name ejMenu#mouseout 	
            * @param {Object}  argument Event parameters from menu     
            * @param {object}  argument.model returns the menu model
            * @param {string}  argument.type returns the name of the event
            * @param {string}  argument.text returns clicked menu item text
            * @param {object}  argument.element returns clicked menu item element
			* @param {object}  argument.event returns the event
            * @example
               * &lt;ul id="menu"&gt;
                      * &lt;li id="Home"&gt;&lt;a&gt;Home&lt;/a&gt;&lt;/li&gt;
                      * &lt;li&gt;&lt;a&gt;Search Jobs&lt;/a&gt;
                         * &lt;ul&gt;
                            * &lt;li&gt;&lt;a&gt;Advanced Search&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs by Company&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs by Category&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs by Location&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs by Skills&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs by Designation&lt;/a&gt;&lt;/li&gt;
                          * &lt;/ul&gt;
                      * &lt;/li&gt;
                      * &lt;li id="Post Resume"&gt;&lt;a&gt;Post Resume&lt;/a&gt;&lt;/li&gt;
                      * &lt;li id="Job Seeker"&gt;&lt;a&gt;JobSeeker Login&lt;/a&gt;&lt;/li&gt;
                      * &lt;li id="Fast Forward"&gt;&lt;a&gt;Fast Forward&lt;/a&gt;
                         * &lt;ul&gt;
                            * &lt;li&gt;&lt;a&gt;Resume writing&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Certification&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Resume Spotlight&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs4u&lt;/a&gt;&lt;/li&gt;
                         * &lt;/ul&gt;
                       * &lt;/li&gt;
                       * &lt;li id="More"&gt;&lt;a&gt;More&lt;/a&gt;
                          * &lt;ul&gt;
                             * &lt;li&gt;&lt;a&gt;Mobile&lt;/a&gt;&lt;/li&gt;
                             * &lt;li&gt;&lt;a&gt;Pay check&lt;/a&gt;&lt;/li&gt;
                             * &lt;li&gt;&lt;a&gt;Blog&lt;/a&gt;&lt;/li&gt;
                          * &lt;/ul&gt;
                        * &lt;/li&gt;
                    * &lt;/ul&gt;
   
       * <br> 
       * &lt;script&gt;  
            *  //mouse out event for menu
            *  $("#menu").ejMenu({height:22,
            *  mouseout: function (args) {}
            *  });   
            * &lt;/script&gt;
            * @memberof ejMenu
            * @instance
            */
            mouseout: null,
            /**     
            * Fires when mouse click on menu items.
            * @event
            * @name ejMenu#click 	
            * @param {Object}  argument Event parameters from menu     
            * @param {object}  argument.model returns the menu model
            * @param {string}  argument.type returns the name of the event
            * @param {string}  argument.text returns clicked menu item text
            * @param {object}  argument.element returns clicked menu item element
			* @param {object}  argument.event returns the event
            * @param {number}  argument.selectedItem returns the selected item
            * @example 
               * &lt;ul id="menu"&gt;
                      * &lt;li id="Home"&gt;&lt;a&gt;Home&lt;/a&gt;&lt;/li&gt;
                      * &lt;li&gt;&lt;a&gt;Search Jobs&lt;/a&gt;
                         * &lt;ul&gt;
                            * &lt;li&gt;&lt;a&gt;Advanced Search&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs by Company&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs by Category&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs by Location&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs by Skills&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs by Designation&lt;/a&gt;&lt;/li&gt;
                          * &lt;/ul&gt;
                      * &lt;/li&gt;
                      * &lt;li id="Post Resume"&gt;&lt;a&gt;Post Resume&lt;/a&gt;&lt;/li&gt;
                      * &lt;li id="Job Seeker"&gt;&lt;a&gt;JobSeeker Login&lt;/a&gt;&lt;/li&gt;
                      * &lt;li id="Fast Forward"&gt;&lt;a&gt;Fast Forward&lt;/a&gt;
                         * &lt;ul&gt;
                            * &lt;li&gt;&lt;a&gt;Resume writing&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Certification&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Resume Spotlight&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs4u&lt;/a&gt;&lt;/li&gt;
                         * &lt;/ul&gt;
                       * &lt;/li&gt;
                       * &lt;li id="More"&gt;&lt;a&gt;More&lt;/a&gt;
                          * &lt;ul&gt;
                             * &lt;li&gt;&lt;a&gt;Mobile&lt;/a&gt;&lt;/li&gt;
                             * &lt;li&gt;&lt;a&gt;Pay check&lt;/a&gt;&lt;/li&gt;
                             * &lt;li&gt;&lt;a&gt;Blog&lt;/a&gt;&lt;/li&gt;
                          * &lt;/ul&gt;
                        * &lt;/li&gt;
                    * &lt;/ul&gt;
   
       * <br> 
       * &lt;script&gt; 
            *  //click event for menu
            *  $("#menu").ejMenu({height:22,
            *  click: function (args){}
            *  });   
            * &lt;/script&gt;
            * @memberof ejMenu
            * @instance
            */
            click: null,
           /**     
            * Fires when key down on menu items.
            * @event
            * @name ejMenu#keydown 	
            * @param {Object}  argument Event parameters from menu     
            * @param {object}  argument.model returns the menu model
            * @param {string}  argument.type returns the name of the event
            * @param {string}  argument.menuText returns clicked menu item text
            * @param {object}  argument.element returns clicked menu item element
            * @param {object}  argument.event returns the event
            * @example 
               * &lt;ul id="menu"&gt;
                      * &lt;li id="Home"&gt;&lt;a&gt;Home&lt;/a&gt;&lt;/li&gt;
                      * &lt;li&gt;&lt;a&gt;Search Jobs&lt;/a&gt;
                         * &lt;ul&gt;
                            * &lt;li&gt;&lt;a&gt;Advanced Search&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs by Company&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs by Category&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs by Location&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs by Skills&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs by Designation&lt;/a&gt;&lt;/li&gt;
                          * &lt;/ul&gt;
                      * &lt;/li&gt;
                      * &lt;li id="Post Resume"&gt;&lt;a&gt;Post Resume&lt;/a&gt;&lt;/li&gt;
                      * &lt;li id="Job Seeker"&gt;&lt;a&gt;JobSeeker Login&lt;/a&gt;&lt;/li&gt;
                      * &lt;li id="Fast Forward"&gt;&lt;a&gt;Fast Forward&lt;/a&gt;
                         * &lt;ul&gt;
                            * &lt;li&gt;&lt;a&gt;Resume writing&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Certification&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Resume Spotlight&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs4u&lt;/a&gt;&lt;/li&gt;
                         * &lt;/ul&gt;
                       * &lt;/li&gt;
                       * &lt;li id="More"&gt;&lt;a&gt;More&lt;/a&gt;
                          * &lt;ul&gt;
                             * &lt;li&gt;&lt;a&gt;Mobile&lt;/a&gt;&lt;/li&gt;
                             * &lt;li&gt;&lt;a&gt;Pay check&lt;/a&gt;&lt;/li&gt;
                             * &lt;li&gt;&lt;a&gt;Blog&lt;/a&gt;&lt;/li&gt;
                          * &lt;/ul&gt;
                        * &lt;/li&gt;
                    * &lt;/ul&gt;
   
       * <br> 
       * &lt;script&gt; 
            *  //keydown event for menu
            *  $("#menu").ejMenu({height:22,
            *  keydown: function (args){}
            *  });  
            * &lt;/script&gt; 
            * @memberof ejMenu
            * @instance
            */
            keydown: null,
			/**     
            * Fires to create menu items.
            * @event
            * @name ejMenu#create  
            * @param {Object}  argument Event parameters from menu     
            * @param {object}  argument.model returns the menu model
            * @param {string}  argument.type returns the name of the event           
            * @example 
               * &lt;ul id="menu"&gt;
                      * &lt;li id="Home"&gt;&lt;a&gt;Home&lt;/a&gt;&lt;/li&gt;
                      * &lt;li&gt;&lt;a&gt;Search Jobs&lt;/a&gt;
                         * &lt;ul&gt;
                            * &lt;li&gt;&lt;a&gt;Advanced Search&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs by Company&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs by Category&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs by Location&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs by Skills&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs by Designation&lt;/a&gt;&lt;/li&gt;
                          * &lt;/ul&gt;
                      * &lt;/li&gt;
                      * &lt;li id="Post Resume"&gt;&lt;a&gt;Post Resume&lt;/a&gt;&lt;/li&gt;
                      * &lt;li id="Job Seeker"&gt;&lt;a&gt;JobSeeker Login&lt;/a&gt;&lt;/li&gt;
                      * &lt;li id="Fast Forward"&gt;&lt;a&gt;Fast Forward&lt;/a&gt;
                         * &lt;ul&gt;
                            * &lt;li&gt;&lt;a&gt;Resume writing&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Certification&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Resume Spotlight&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs4u&lt;/a&gt;&lt;/li&gt;
                         * &lt;/ul&gt;
                       * &lt;/li&gt;
                       * &lt;li id="More"&gt;&lt;a&gt;More&lt;/a&gt;
                          * &lt;ul&gt;
                             * &lt;li&gt;&lt;a&gt;Mobile&lt;/a&gt;&lt;/li&gt;
                             * &lt;li&gt;&lt;a&gt;Pay check&lt;/a&gt;&lt;/li&gt;
                             * &lt;li&gt;&lt;a&gt;Blog&lt;/a&gt;&lt;/li&gt;
                          * &lt;/ul&gt;
                        * &lt;/li&gt;
                    * &lt;/ul&gt;
   
       * <br> 
       * &lt;script&gt; 
            *  //Create event for menu
            *  $("#menu").ejMenu({height:22,
            *  create: function (args){}
            *  });   
            * &lt;/script&gt;
            * @memberof ejMenu
            * @instance
            */
            create: null,
			 /**     
            * Fires to destroy menu items.
            * @event
            * @name ejMenu#destroy 
            * @param {Object}  argument Event parameters from menu     
            * @param {object}  argument.model returns the menu model
            * @param {string}  argument.type returns the name of the event            
            * @example 
               * &lt;ul id="menu"&gt;
                      * &lt;li id="Home"&gt;&lt;a&gt;Home&lt;/a&gt;&lt;/li&gt;
                      * &lt;li&gt;&lt;a&gt;Search Jobs&lt;/a&gt;
                         * &lt;ul&gt;
                            * &lt;li&gt;&lt;a&gt;Advanced Search&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs by Company&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs by Category&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs by Location&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs by Skills&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs by Designation&lt;/a&gt;&lt;/li&gt;
                          * &lt;/ul&gt;
                      * &lt;/li&gt;
                      * &lt;li id="Post Resume"&gt;&lt;a&gt;Post Resume&lt;/a&gt;&lt;/li&gt;
                      * &lt;li id="Job Seeker"&gt;&lt;a&gt;JobSeeker Login&lt;/a&gt;&lt;/li&gt;
                      * &lt;li id="Fast Forward"&gt;&lt;a&gt;Fast Forward&lt;/a&gt;
                         * &lt;ul&gt;
                            * &lt;li&gt;&lt;a&gt;Resume writing&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Certification&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Resume Spotlight&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs4u&lt;/a&gt;&lt;/li&gt;
                         * &lt;/ul&gt;
                       * &lt;/li&gt;
                       * &lt;li id="More"&gt;&lt;a&gt;More&lt;/a&gt;
                          * &lt;ul&gt;
                             * &lt;li&gt;&lt;a&gt;Mobile&lt;/a&gt;&lt;/li&gt;
                             * &lt;li&gt;&lt;a&gt;Pay check&lt;/a&gt;&lt;/li&gt;
                             * &lt;li&gt;&lt;a&gt;Blog&lt;/a&gt;&lt;/li&gt;
                          * &lt;/ul&gt;
                        * &lt;/li&gt;
                    * &lt;/ul&gt;
   
       * <br> 
       * &lt;script&gt; 
            *  //Destroy event for menu
            *  $("#menu").ejMenu({height:22,
            *  destroy: function (args){}
            *  }); 
            * &lt;/script&gt;  
            * @memberof ejMenu
            * @instance
            */
            destroy: null
        },
        /**
        * Specify the data types for default properties 
        * @private
        */
        dataTypes: {
            animationType:"enum",
            cssClass: "string",
            titleText:"string",
            openOnClick: "boolean",
            enabled: "boolean",
            enableCenterAlign: "boolean",
            showArrow: "boolean",
            showRooltLevelArrows: "boolean",
            showSubLevelArrows: "boolean",
            enableSeparator: "boolean",
            enableRTL: "boolean",
            enableAnimation: "boolean",
            fields: {
                dataSource: "data",
                query: "data",
                child: "data"
            }
        },
        
        /**
        * To configure the properties at runtime using SetModel		
        * @private
        */
        _setModel: function (jsondata) {
            if (!(this.model.enabled) && ej.isNullOrUndefined(jsondata["enabled"])) return false;
            for (var key in jsondata) {
                switch (key) {
                    case "menuType":
                        this.model.menuType = jsondata[key];
                        this.element.removeClass("e-menu " + this.model.cssClass);
                        this._renderControl();
                        if (jsondata[key] == "contextmenu") this._contextMenuEvents();
                        break;
                    case "fields":
                        this.element.empty().insertBefore(this.wrapper);
                        this.wrapper.remove();
                        $.extend(this.model.fields, jsondata[key]);
                        this._intializeData();
                        break;
                    case "orientation": this._setOrientation(jsondata[key]); break;
                    case "showTopLevelArrow": this._addArrow(jsondata[key], this.model.showBottomLevelArrow); break;
                    case "showBottomLevelArrow": this._addArrow(this.model.showRooltLevelArrows, jsondata[key]); break;
                    case "enableSeparator": this._setSeparator(jsondata[key]); break;
                    case "height": this._setHeight(jsondata[key]); break;
                    case "width": this._setWidth(jsondata[key]); break;
                    case "cssClass": this._setSkin(jsondata[key]); break;
                    case "enableRTL": this._setRTL(jsondata[key]); break;
                    case "enableCenterAlign": this._centerAlign(jsondata[key]); break;
                    case "enabled": this.model.enabled = jsondata[key]; this._controlStatus(jsondata[key]); break;
                    case "animationType":
                        this._setAnimation(jsondata[key]);
                        break;
					case "enableAnimation":
						this.model.enableAnimation=options[key];
						break;
						
					
                }
            }
        },      
        // all events bound using this._on will be unbind automatically
        _destroy: function () {
            /// <summary>This function is  used to destroy the Menu Object</summary>
            this._cloneElement.insertBefore(this.element);
            this._cloneElement.removeClass('e-menu');
            this.element.remove();
        },
        /**
       * Create the Menu widget
       * @private
       */
        // constructor function
        _init: function () {
            /// <summary>This function is  used to Initialize the Menu Object</summary>
            this.element.css("visibility", "hidden");
            this._cloneElement = this.element.clone();
            this._setValues();
            this._intializeData();
            this.element.css("visibility", "visible");
        },

        //-------------------------Private Methods---------------------------------------------    
        /**
       * To configure the property values
       * @private
       */
        _setValues: function () {
            this._mouseOver = true;
            this._hoverOpen = true;
            this._hoverClose = true;
            this._isMenuOpen = false;
            this._hideSpeed = 100;
            this._showSpeed = 100;
            this._isSubMenuOpen = false;
            this._isContextMenuOpen = false;
            this._disabledMenuItems = new Array();
            this._delayMenuHover = 0;
            this._delaySubMenuHover = 0;
            this._delaySubMenuShow = 0;
            this._preventContextOpen = true;
            this._setAnimation(this.model.animationType);
            this._isFocused = true;
        },
        _intializeData: function () {
            if (!ej.isNullOrUndefined(this.model.fields) && this.model.fields["dataSource"] != null) {
                this._generateTemplate(this.model.fields["dataSource"]);
                this._renderMenu();
            }
            else {
                this._renderMenu();
                this._wireEvents("_on");
            }
        },
        _renderMenu: function () {
            this._renderControl();
            this._addArrow(this.model.showRooltLevelArrows, this.model.showSubLevelArrows);
            this._intializeMenu();
        },
        /**
        * Render Section For Different Types Menu		
        * @private
        */
        _renderControl: function () {
            /// <summary>This function is  used to Render the Menu Object</summary>
            var label, checkBox, checkObj, list, spanlist, i;
            this.wrapper = ej.buildTag("div.e-menu-wrap");
            if ((this.model.menuType != ej.MenuType.ContextMenu) && (this.model.orientation != "vertical")) {
                this.resWrap = ej.buildTag('span.e-menu-res-wrap');
                this.inResWrap = ej.buildTag('span.e-in-wrap e-box e-menu-res-in-wrap');
                this.label = ej.buildTag('span.e-res-title', this.model.titleText);
                this.check = ej.buildTag('span.e-check-wrap e-icon');
                this.checkBox = ej.buildTag('input#responsive.e-res-icon', '', {}, { type: 'checkbox' });
                this.wrapper.append(this.resWrap)
                this.resWrap.append(this.inResWrap);
                this.inResWrap.append(this.label).append(this.check);
                this.check.append(this.checkBox);
            }
            if (this.model.menuType != ej.MenuType.ContextMenu) {
                this.wrapper.insertBefore(this.element);
                this.wrapper.append(this.element);
                if (this.checkBox != null) {
                    this.checkBox.ejCheckBox({ size: "medium", change: this._oncheck });

                }
            }
            this.element.addClass("e-menu e-widget").attr({ "role": "menu","tabindex": 0});
            if (this.model.enableRTL) this._setRTL(this.model.enableRTL);
            if (this.model.menuType == "normalmenu") {
                this.model.orientation == "horizontal" ? this.element.addClass(this.model.cssClass + " e-horizontal") : this.element.addClass(this.model.cssClass + " e-vertical");
            }
            //For ContextMenu Mode
            else this._contextMenu_Template();
            //Adding arrows to items with sub items
            this.element.find('li:has("> ul")').find('> a,> span').addClass('aschild');
            this.element.find('>li').addClass('e-list').attr({ "role": "menuitem"});
            list = this.element.find('.e-list a.aschild');
            spanlist = this.element.find('.e-list span.aschild');
            for (i = 0; i < list.length; i++) {
                $(list[i]).siblings().attr({ "aria-hidden": true });
                $(list[i]).parent().attr({ "aria-haspopup": true, "role": "menu" });
                $(list[i]).parent().addClass("e-haschild");
                $(list[i]).siblings('ul').children('li').addClass('e-list').attr("role", "menuitem");
            }
            for (i = 0; i < spanlist.length; i++) {
                $(spanlist[i]).siblings().attr({ "aria-hidden": true });
                $(spanlist[i]).parent().attr({ "aria-haspopup": true, "role": "menu" });
                $(spanlist[i]).parent().addClass("e-haschild");
                $(spanlist[i]).siblings('ul').children('li').addClass('e-list').attr("role", "menuitem");
            }
            if (this.model.enableCenterAlign) this._centerAlign(this.model.enableCenterAlign);
            if (this.model.enableSeparator) this._setSeparator(true);
            this._controlStatus(this.model.enabled);
        },
        /**
        * Wiring the Events to Responsive Icon 	
        * @private
        */
        _oncheck: function (e) {
            var obj = this.element.parents('.e-menu-wrap').children('.e-menu');
            e.isChecked ? obj.removeClass('e-res-hide').addClass('e-res-show') : obj.removeClass('e-res-show').addClass('e-res-hide');
        },
        /**
        * Render Section For Menu through Remote Data and DataSource	
        * @private
        */
        _generateTemplate: function (data) {
            var proxy = this, queryPromise;
            if (data instanceof ej.DataManager) {
                queryPromise = data.executeQuery(this._columnToSelect(this.model.fields));
                queryPromise.done(function (e) {
					proxy._odataFlag = true;
                    proxy._generateItemTemplate(e.result);
                    proxy._wireEvents("_on");
                });
            } else {
				proxy._odataFlag = false;
                this._generateItemTemplate(proxy.model.fields['dataSource']);
                this._wireEvents("_on");
            }
        },
        /**
        * Render Section For Menu through DataSource		
        * @private
        */
        _generateItemTemplate: function (items) {
            for (var i = 0; i < items.length; i++) {
                if ((items[i][this.model.fields.parentId] == null) || (items[i][this.model.fields.parentId] == 0)) {
                    var subItem = this._menuTemplate(items[i], items, this.model.fields);
                    this.element.append(subItem);
                }
            }
        },
        /**
      * Render Section For Menu template		
      * @private
      */
        _menuTemplate: function (item, tableData, mapper) {
            var liTag, aTag, imgTag, spanTag;
            liTag = $(document.createElement('li'));
            liTag.attr("class", 'e-list');
            if (item[mapper.htmlAttribute]) this._setAttributes(item[mapper.htmlAttribute], liTag);

            if (item[mapper.id]) {
                aTag = $(document.createElement('a'));                
                if (item[mapper.imageUrl] && item[mapper.imageUrl] != "") {
                    imgTag = $(document.createElement('img'));
                    imgTag.attr('src', item[mapper.imageUrl]);
                    if (item[mapper.imageAttribute]) this._setAttributes(item[mapper.imageAttribute], imgTag);
                    aTag.append(imgTag);
                }
                else if (item[mapper.spriteCssClass] && item[mapper.spriteCssClass] != "") {
                    spanTag = $(document.createElement('span'));
                    spanTag.addClass(item[mapper.spriteCssClass]);
                    aTag.append(spanTag);
                }
                aTag.append(item[mapper.text]);
                if (item[mapper.linkAttribute]) this._setAttributes(item[mapper.linkAttribute], aTag);
                if (item[mapper.url])
                    aTag.attr('href', item[mapper.url]);
                liTag.append(aTag);
            }
            if (item[mapper.id]) {
                liTag.prop("id", item[mapper.id]);
            }
            if (!ej.isNullOrUndefined(mapper["child"])) {
                this._odataFlag = true;
                if (mapper["child"]["dataSource"] instanceof ej.DataManager) {
                    var proxy = this, queryManager = ej.Query();
                    queryManager = this._columnToSelect(mapper["child"]);
                    queryManager.where(mapper["child"]["parentId"], ej.FilterOperators.equal, item[mapper.id]);
                    var queryPromise = mapper["child"]["dataSource"].executeQuery(queryManager);
                    queryPromise.done(function (e) {
                        var childItems = e.result;
                        if (childItems && childItems.length > 0) {
                            var ul = $(document.createElement('ul'));
                            for (var i = 0; i < childItems.length; i++) {
                                var liItem = proxy._menuTemplate(childItems[i], mapper["child"]["dataSource"], mapper["child"]);
                                ul.append(liItem);
                            }
                            liTag.append(ul);
                            $(liTag).children('a').addClass('aschild');
                            if ($(liTag).parent().hasClass('e-menu') &&(proxy.model.showRooltLevelArrows))                           
                                    $(liTag).children('a.aschild').append($('<span>').addClass("e-icon e-arrows")).addClass("e-arrow-space");
                            else if(proxy.model.showSubLevelArrows) 
                               $(liTag).children('a.aschild').append($('<span>').addClass("e-icon e-arrows")).addClass("e-arrow-space");    
                        }
                    });   
                }
                else {
                    var childItems = ej.DataManager(mapper["child"]["dataSource"]).executeLocal(ej.Query().where(mapper["child"]["parentId"], ej.FilterOperators.equal, item[mapper.id]));
                    if (childItems && childItems.length > 0) {
                        var ul = $(document.createElement('ul'));
                        for (var i = 0; i < childItems.length; i++) {
                            var liItem = this._menuTemplate(childItems[i], mapper["child"]["dataSource"], mapper["child"]);
                            ul.append(liItem);
                        }
                        liTag.append(ul);
                    }
                }
            }
            else if (!this._odataFlag) {
                var childItems = ej.DataManager(mapper["dataSource"]).executeLocal(ej.Query().where(mapper["parentId"], ej.FilterOperators.equal, item[mapper.id]));
                if (childItems && childItems.length > 0) {
                    var ul = ej.buildTag('ul');
                    for (var i = 0; i < childItems.length; i++) {
                        var liItem = this._menuTemplate(childItems[i], mapper["dataSource"], mapper);
                        ul.append(liItem);
                    }
                    liTag.append(ul);
                }
            }
            return liTag;
        },
        /**
        * To configure the Attributes of Menu items
        * @private
        */
        _setAttributes: function (data, element) {
            for (var key in data) {
				if(key == "class")
					element.addClass(data[key]);
				else
                	element.attr(key, data[key]);
            }
        },
        /**
        * To configure the Top and Bottom Level Arrows	
        * @private
        */
        _addArrow: function (topArrows, bottomArrows) {
            if (topArrows)
                this.element.find('>li.e-list:has("> ul")').children('a,span').append($('<span>').addClass("e-icon e-arrows")).addClass("e-arrow-space");
            else {
                this.element.find('>li.e-list:has("> ul")').children('a,span').removeClass("e-arrow-space").children('span.e-icon').remove();
            }

            if (bottomArrows)
                this.element.find('>li.e-list > ul li.e-list:has(>ul)').children('a,span').append($('<span>').addClass("e-icon e-arrows")).addClass("e-arrow-space");
            else {
                this.element.find('>li.e-list > ul li.e-list:has(>ul)').children('a,span').removeClass("e-arrow-space").children('span.e-icon').remove();
            }

        },
        /**
         * To initialize the Menu		
		 * @private
         */
        _intializeMenu: function () {
            if (this.model.height != 0) this._setHeight(this.model.height);
            if (this.model.width != 0) this._setWidth(this.model.width);
            if (this.model.menuType == "contextmenu")
                this.model.openOnClick = false;
            if (this.model.openOnClick) {
                this._hoverOpen = false;
                this._hoverClose = false;
            }
        },
        /**
        * To configure the Orientation of Menu	
        * @private
		*/
        _setOrientation: function (val) {
            if (val == "horizontal") {
                this.element.removeClass("e-vertical e-horizontal").addClass("e-horizontal");
            } else {
                this.element.removeClass("e-horizontal e-vertical").addClass("e-vertical");
            }
        },
        /**
        * To configure the height		
        * @private
        */
        _setHeight: function (value) {
            this.element.height(value);
        },
        /**
        * To configure the width		
        * @private
        */
        _setWidth: function (value) {
            this.element.width(value);
        },
        /**
        * To enable or disable the Right to Left behaviour 			
        * @private
        */
        _setRTL: function (isRTL) {
            if (isRTL) {
                this.element.removeClass("e-rtl").addClass("e-rtl");
                this.model.subMenuDirection = "left";
            } else {
                this.element.removeClass("e-rtl");
                this.model.subMenuDirection = "right";
            }
        },
        /**
        * To enable or disable the AnimationType while Hover or Click an Menu items		
        * @private
        */
        _setAnimation: function (value) {
            value === "none" ? (this._showAnim = "none", this._hideAnim = "none") : (this._showAnim = "slideDown", this._hideAnim = "slideUp");
        },
        /**
        * To configure the control status	
        * @private
        */
        _controlStatus: function (value) {
            //disable status
            value != true ? this.disable() : this.enable();
        },
        /**
        * To configure enableCenterAlign of Menu items.		
        * @private
        */
        _centerAlign: function (enableCenterAlign) {
            if (this.model.orientation == "horizontal" && enableCenterAlign)
                this.element.css('text-align', 'center');
            else if (this.model.orientation == "horizontal") this.element.css('text-align', 'inherit');
        },
        _columnToSelect: function (mapper) {
            var column = [], queryManager = ej.Query();
            if (ej.isNullOrUndefined(mapper.query)) {
                for (var col in mapper) {
                    if (col !== "tableName" && col !== "child" && col !== "dataSource" && mapper[col])
                        column.push(mapper[col]);
                }
                if (column.length > 0)
                    queryManager.select(column);
                if (!this.model.fields["dataSource"].dataSource.url.match(mapper.tableName + "$"))
                    !ej.isNullOrUndefined(mapper.tableName) && queryManager.from(mapper.tableName);
            }
            else
                queryManager = mapper.query;
            return queryManager;
        },
        /**
        * To Find maximum z-index in the document	
        * @private
        */
        //Gets the maximum z-index in the document
        _max_zindex: function () {
            var parents = this.element.parents(), bodyEle, maxZ;
            bodyEle = $('body').children(), index = bodyEle.index(this.popup);
            bodyEle.splice(index, 1);
            $(bodyEle).each(function (i, ele) { parents.push(ele); });

            if (this.model.menuType == "contextmenu") {
                maxZ = Math.max.apply(null, $.map($('body *'), function (e, n) {
                    if ($(e).css('position') != 'static')
                        return parseInt($(e).css('z-index')) || 1;
                }));
            }
            else {
                 maxZ = Math.max.apply(maxZ, $.map(parents, function (e, n) {
                    if ($(e).css('position') != 'static') return parseInt($(e).css('z-index')) || 1;
                }));
            }
            if (!maxZ || maxZ < 10000) maxZ = 10000;
            else maxZ += 1;
            return maxZ;

        },

        _recursiveFunction: function (items, menuText) {
            var context = this;
            var isFound = false;
            $.each(items, function (key, value) {
                if (value.Text == menuText) {
                    context.selectedItem = value;
                    isFound = true;
                    return false;
                }
                else if (value.ChildItems != null) {
                    context._recursiveFunction(value.ChildItems, menuText);
                }
                if (isFound)
                    return false;
            });
        },
        /**
        * Render section for Context Menu Template	
        * @private
        */
        _contextMenu_Template: function () {
            var oldWrapper = $(".e-menu-wrap #"+this.element.context.id).get(0);
            if (oldWrapper)
                $(oldWrapper.parentElement).remove();
            this.model.orientation = "vertical";
            this._hideAnimation(this.element, this._hideAnim);
            this.element.addClass(this.model.cssClass + " e-context");
            $("body").append(this.element);
            this.wrapper.insertBefore(this.element);
            this.wrapper.append(this.element);
        },
        /**
        * To close currently visible Menu items	
        * @private
        */
        _closeMenu: function () {
            this._hideAnimation(this.element.find('li.e-list:has("> ul")').find('> ul:visible'), this._hideAnim);
        },

        _onMenuIntent: function (element, obj, canOpen) {
            obj._delayMenuHover = window.setTimeout(function () {
                if (obj._mouseOver == true && canOpen) {
                    var showanim = obj._showAnim;
                    var hideanim = obj._hideAnim;
                    var showSpeed = obj._showSpeed;
                    var hideSpeed = obj._hideSpeed;
                    obj._show(element, showanim, hideanim);
                }
            }, this._showSpeed);
        },
        /**
         * To hide Menu items 
		 * @private
         */
        _onHide: function (element, obj, canHide) {
            obj._delaySubMenuHover = window.setTimeout(function () {
                if (obj._mouseOver == false && canHide) {
                    var id = obj._id;
                    var hideanim = obj._hideAnim;
                    var hideSpeed = obj._hideSpeed;
                    obj._closeAll();
                }

            }, obj._hideSpeed);
        },
        /**
        * To configure Sub Menu Position 
        * @private
        */
        _subMenuPos: function (element, direction) {
            var parentPos = $(this.wrapper).parent().offset();
            var pos = $(element).offset();
            var posLeft = pos.left - parentPos.left;
            var subMenu = $('ul:first', element);
            var menuWidth = $(element).outerWidth();
            if (pos == null || pos == undefined)
                return false;
            var submenuWidth = subMenu.outerWidth();
            var left = document.documentElement.clientWidth + $(document).scrollLeft();
            if (this.model.menuType == "normalmenu") {
                if ($(element.parentNode).is(this.element)) {
                    if (this.model.orientation == "horizontal")
                        subMenu.css("top", $(element).outerHeight() + "px");
                    else if ((direction == "left" && posLeft > submenuWidth) || (direction == "right" && left <= pos.left + menuWidth + submenuWidth))
                        subMenu.css("left", -(submenuWidth + 4) + "px");
                    else {
                        subMenu.css("left", ($(element).outerWidth() + 4) + "px");
                    }
                } else if ((direction == "left" && posLeft > submenuWidth) || (direction == "right" && left <= pos.left + menuWidth + submenuWidth)) {
                    subMenu.css("left", -(submenuWidth + 4) + "px");
                }
            }
            else {
                left -= (pos.left + (2 * submenuWidth) + 4);
                if (left < 0) {
                    var menuLeftPos = (submenuWidth == null) ? "-206.5px" : "-" + (submenuWidth + 5) + "px";
                    subMenu.css("left", menuLeftPos);
                }
                else {
                    if (subMenu.parent('li.e-list').parent('ul').width() && direction == "right") {
                        subMenu.css("left", (subMenu.parent('li.e-list').parent('ul').width() + 4) + "px");
                    }
                    else if (pos.left > submenuWidth)
                        subMenu.css("left", -(submenuWidth + 4) + "px");
                }
            }
        },

        /**
        * To configure the custom theme for Menu using cssClass property		
        * @private
        */
        _setSkin: function (skin) {
            if (this.model.menuType == "normalmenu") {
                if (this.model.orientation == "horizontal") {
                    this.element.removeClass(this.model.cssClass + " e-horizontal");
                    this.element.addClass(skin + " e-horizontal");
                } else {
                    this.element.removeClass(this.model.cssClass + " e-vertical");
                    this.element.addClass(skin + " e-vertical");
                }

            } else {
                this.element.removeClass(this.model.cssClass + " e-context");
                this.element.addClass(skin + " e-context");
            }
        },
        /**
        * To configure seperator for Menu items		
        * @private
        */
        _setSeparator: function (separator) {
            if (separator)
                this.element.addClass("e-separator");
            else this.element.removeClass("e-separator");
        },
        /**
         * Wiring the events to Context Menu 	
		 * @private
         */
        _contextMenuEvents: function (action) {
            this[action]($(this.model.contextMenuTarget), "mouseup taphold", this._ContextMenuHandler);
            this [action](this.element, "contextmenu", this._onDefaultPreventer);
            this[action]($(this.model.contextMenuTarget), "contextmenu", this._onDefaultPreventer);
            this [action]($(document), "click", this._onContextClose);
            this[action]($(document), "keydown", this._onKeyDownHandler);
        },
        /**
        * To show the Menu items		
        * @private
        */
        _show: function (element, showanim, hideanim) {
            var siblingElement;
            var sibling = $('> ul', element);
            sibling.attr({ "aria-hidden": false });
            this._hideAnimation($(element).siblings().find(' > ul:visible'), hideanim);
            if (!($.inArray(this._disabledMenuItems, element) > -1)) {
                if (sibling.css('display') != "none") {
                    siblingElement = this.model.openOnClick ? $(sibling) : sibling.children().find('> ul');
                    this._hideAnimation(siblingElement, hideanim);
                }
                else $('> ul', element).children().find('> ul').hide();
                this._subMenuPos(element, this.model.subMenuDirection);
                sibling.css({ "z-index": this._max_zindex() + 1 });
                if ($('> ul', element).css('display') != 'block' && !$(element).hasClass("e-disable-item")) 
                    this._showAnimation(sibling, showanim);
            }
        },
        /**
        * To close all visible Menu items		
        * @private
        */
        _closeAll: function () {
            this._hideAnimation(this.element.find('li.e-list:has("> ul")').find('> ul:visible'), this._hideAnim);
            this._hideAnimation(this.element.find('> ul:visible'), this._hideAnim);
        },
        /**
        * To show the Menu items with animationType		
        * @private
        */
        _showAnimation: function (element, anim) {
            switch (anim) {
                case "slideDown":
                    element.slideDown(this.model.enableAnimation?200:0, "easeOutQuad"); break;
                case "none":
                    element.css("display","block"); break;
            }
        },
        /**
        * To hide the Menu items with animationType		
        * @private
        */
        _hideAnimation: function (element, anim) {
            switch (anim) {
                case "slideUp":
                    $(element).attr({ "aria-hidden": true });
                    element.slideUp(this.model.enableAnimation?100:0, "easeOutQuad"); break;
                case "none":
                    element.css("display","none"); break;
            }
        },

        _removeValue: function (text, disableList) {
            var $browInfo = ej.browserInfo(), elementText;
            $browInfo.version === "8.0" && $browInfo.name === "msie" ? elementText = text[0].outerText : elementText = text[0].textContent;
            var count = $(disableList).length, i = 0;
            while (i <= count)
            {
                if (disableList[i] === $.trim(elementText))
                    return i;
                    i++;
            }
        },
		/**
        * To create sub level Menu items when insert a menu item dynamically	
        * @private
        */
		_createSubLevelItem:function(target,element){
			var ulTag;
			ulTag = $(document.createElement('ul'));
			ulTag.append(element);
			target.append(ulTag);
			target.attr({ 'role': 'menu', 'aria-haspopup': 'true' });
			target.addClass("e-haschild");
			this.element.find('li:has("> ul")').find('> a,>span').addClass('aschild e-arrow-space');
			this._insertArrows(ulTag);		
		},	
		/**
        * To insert arrows to Menu items when insert a menu item dynamically	
        * @private
        */
		_insertArrows:function(ulTag){
		if (this.model.showRooltLevelArrows)
                ulTag.find('>a,>span').append($('<span>').addClass("e-icon e-arrows")).addClass("e-arrow-space");
            else 
                ulTag.find('>a,>span').removeClass("e-arrow-space").find('>span.e-icon').remove();
				
            if (this.model.showSubLevelArrows)
                ulTag.parent('li.e-list:has(>ul)').children('a,span').append($('<span>').addClass("e-icon e-arrows")).addClass("e-arrow-space");
            else 
                ulTag.parent('li.e-list:has(>ul)').children('a,span').removeClass("e-arrow-space").find('>span.e-icon').remove();
		},
		/**
        * To create a Menu items when insert a menu item dynamically	
        * @private
        */
		_createMenuItem:function(item){
            var liTag, aTag, imgTag, spanTag;
            liTag = $(document.createElement('li'));
            liTag.attr({"class":'e-list',"role":"menuitem"});
            if (item["htmlAttribute"]) this._setAttributes(item["htmlAttribute"], liTag);
            if (item["text"] && item["text"] != "") {
                aTag = $(document.createElement('a'));
                if (item["imageUrl"] && item["imageUrl"] != "") {
                    imgTag = $(document.createElement('img'));
                    imgTag.attr('src', item["imageUrl"]);
                    if (item["imageAttribute"]) this._setAttributes(item["imageAttribute"], imgTag);
                    aTag.append(imgTag);
                }
                else if (item["spriteCssClass"] && item["spriteCssClass"] != "") {
                    spanTag = $(document.createElement('span'));
                    spanTag.addClass(item["spriteCssClass"]);
                    aTag.append(spanTag);
                }
                aTag.append(item["text"]);
                if (item["linkAttribute"]) this._setAttributes(item["linkAttribute"], aTag);
                if (item["url"])
                    aTag.attr('href', item["url"]);
                liTag.append(aTag);
            }
            if (item["id"]) {
                liTag.prop("id", item["id"]);
            }
            return liTag;
		},
		/**
        * To find out each the item and their target element from item collection and tatrget element collection.	
        * @private
        */
		_insertNode:function(itemCollection,targetNode,operation){
			var item=0,targetList=0,target=0,targetCollection=[];
			typeof (targetNode) ==="string" ? targetCollection.push(this.element.find(targetNode)):typeof (targetNode) ==="undefined" ?targetCollection.push("default"):targetCollection.push(targetNode);
			for(targetList=0;targetList<targetCollection.length;targetList++){
				for(target=0;target<targetCollection[targetList].length;target++)
					for(item=0;item<itemCollection.length;item++)
						this._addItem(itemCollection[item],targetCollection[targetList][target],operation);
		   }
		},
		/**
        * To insert the menu items at their respective place.
        * @private
        */
		_addItem:function(item,target,operation){
				var element,targetElement;
				this._wireEvents("_off");
		        element = this._createMenuItem(item);
				target = target === "default" ? $("#"+item["parentId"]):$(target);
				switch(operation){
				case "insert":
					targetElement= target.children('ul');
					targetElement.length != 0 ? targetElement.append(element):this._createSubLevelItem(target,element);
					break;
				case "insertBefore":
					element.insertBefore(target);
					break;
				case "insertAfter":
					element.insertAfter(target);
					break;
					}
			this._wireEvents("_on");
		},
		/**
        * To remove the root level element while all child level elements are deleted	
        * @private
        */
		_removeItem:function(item){
		  if(item.siblings('li').length == 0 ){
					item.closest("ul").siblings('a.aschild').removeClass("aschild e-arrow-space").children('span.e-icon').remove();
					!item.closest("ul").hasClass("e-menu")?item.closest("ul").remove():item.remove();}
			else
					item.remove();
		},

        //-------------------------Public Methods---------------------------------------------//
		/**
       * Insert the menu item as child of target node.
       * @alias insert
       * @return jQuery
       * @param {ArrayObject}  item  Information about Menu item.
       * @param {string | Object}  target   Selector of target node or Object of target node.
      
       * @example 
         * &lt;ul id="menu"&gt;
                      * &lt;li id="Home"&gt;&lt;a&gt;Home&lt;/a&gt;&lt;/li&gt;
                      * &lt;li&gt;&lt;a&gt;Search Jobs&lt;/a&gt;
                         * &lt;ul&gt;
                            * &lt;li&gt;&lt;a&gt;Advanced Search&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs by Company&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs by Category&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs by Location&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs by Skills&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs by Designation&lt;/a&gt;&lt;/li&gt;
                          * &lt;/ul&gt;
                      * &lt;/li&gt;
                      * &lt;li id="Post Resume"&gt;&lt;a&gt;Post Resume&lt;/a&gt;&lt;/li&gt;
                      * &lt;li id="Job Seeker"&gt;&lt;a&gt;JobSeeker Login&lt;/a&gt;&lt;/li&gt;
                      * &lt;li id="Fast Forward"&gt;&lt;a&gt;Fast Forward&lt;/a&gt;
                         * &lt;ul&gt;
                            * &lt;li&gt;&lt;a&gt;Resume writing&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Certification&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Resume Spotlight&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs4u&lt;/a&gt;&lt;/li&gt;
                         * &lt;/ul&gt;
                       * &lt;/li&gt;
                       * &lt;li id="More"&gt;&lt;a&gt;More&lt;/a&gt;
                          * &lt;ul&gt;
                             * &lt;li&gt;&lt;a&gt;Mobile&lt;/a&gt;&lt;/li&gt;
                             * &lt;li&gt;&lt;a&gt;Pay check&lt;/a&gt;&lt;/li&gt;
                             * &lt;li&gt;&lt;a&gt;Blog&lt;/a&gt;&lt;/li&gt;
                          * &lt;/ul&gt;
                        * &lt;/li&gt;
                    * &lt;/ul&gt;
   
       * <br> 
       * &lt;script&gt;
        * $("#menu").ejMenu({height:22,}); 
       * //initialize the menu object
       *	var menuObj = $("#menu").data("ejMenu");
       *	//To insert menu item 
       *	menuObj.insert([{
	   *          id:"More",
	   *        text:"More"
	   *		}],"#Home");
      * &lt;/script&gt;
       * @example 
         * &lt;ul id="menu"&gt;
                      * &lt;li id="Home"&gt;&lt;a&gt;Home&lt;/a&gt;&lt;/li&gt;
                      * &lt;li&gt;&lt;a&gt;Search Jobs&lt;/a&gt;
                         * &lt;ul&gt;
                            * &lt;li&gt;&lt;a&gt;Advanced Search&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs by Company&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs by Category&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs by Location&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs by Skills&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs by Designation&lt;/a&gt;&lt;/li&gt;
                          * &lt;/ul&gt;
                      * &lt;/li&gt;
                      * &lt;li id="Post Resume"&gt;&lt;a&gt;Post Resume&lt;/a&gt;&lt;/li&gt;
                      * &lt;li id="Job Seeker"&gt;&lt;a&gt;JobSeeker Login&lt;/a&gt;&lt;/li&gt;
                      * &lt;li id="Fast Forward"&gt;&lt;a&gt;Fast Forward&lt;/a&gt;
                         * &lt;ul&gt;
                            * &lt;li&gt;&lt;a&gt;Resume writing&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Certification&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Resume Spotlight&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs4u&lt;/a&gt;&lt;/li&gt;
                         * &lt;/ul&gt;
                       * &lt;/li&gt;
                       * &lt;li id="More"&gt;&lt;a&gt;More&lt;/a&gt;
                          * &lt;ul&gt;
                             * &lt;li&gt;&lt;a&gt;Mobile&lt;/a&gt;&lt;/li&gt;
                             * &lt;li&gt;&lt;a&gt;Pay check&lt;/a&gt;&lt;/li&gt;
                             * &lt;li&gt;&lt;a&gt;Blog&lt;/a&gt;&lt;/li&gt;
                          * &lt;/ul&gt;
                        * &lt;/li&gt;
                    * &lt;/ul&gt;
   
       * <br> 
       * &lt;script&gt;
        * $("#menu").ejMenu({height:22,}); 
       * //To insert menu item 
       * $("#menu").ejMenu("insert",[{
	   *          id:"More",
	   *        text:"More"
	   *		}],"#Home");
      * &lt;/script&gt;
       * @memberof ejMenu
       * @instance
        */
		insert:function(item,target){
			this._insertNode(item,target,"insert");
		},	
              /**
       * Insert the menu item before the target node.
       * @alias insertBefore
       * @return jQuery
       * @param {ArrayObject}  item  Information about Menu item.
       * @param {string | Object}  target   Selector of target node or Object of target node.
      
       * @example 
       * &lt;ul id="menu"&gt;
                      * &lt;li id="Home"&gt;&lt;a&gt;Home&lt;/a&gt;&lt;/li&gt;
                      * &lt;li&gt;&lt;a&gt;Search Jobs&lt;/a&gt;
                         * &lt;ul&gt;
                            * &lt;li&gt;&lt;a&gt;Advanced Search&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs by Company&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs by Category&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs by Location&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs by Skills&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs by Designation&lt;/a&gt;&lt;/li&gt;
                          * &lt;/ul&gt;
                      * &lt;/li&gt;
                      * &lt;li id="Post Resume"&gt;&lt;a&gt;Post Resume&lt;/a&gt;&lt;/li&gt;
                      * &lt;li id="Job Seeker"&gt;&lt;a&gt;JobSeeker Login&lt;/a&gt;&lt;/li&gt;
                      * &lt;li id="Fast Forward"&gt;&lt;a&gt;Fast Forward&lt;/a&gt;
                         * &lt;ul&gt;
                            * &lt;li&gt;&lt;a&gt;Resume writing&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Certification&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Resume Spotlight&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs4u&lt;/a&gt;&lt;/li&gt;
                         * &lt;/ul&gt;
                       * &lt;/li&gt;
                       * &lt;li id="More"&gt;&lt;a&gt;More&lt;/a&gt;
                          * &lt;ul&gt;
                             * &lt;li&gt;&lt;a&gt;Mobile&lt;/a&gt;&lt;/li&gt;
                             * &lt;li&gt;&lt;a&gt;Pay check&lt;/a&gt;&lt;/li&gt;
                             * &lt;li&gt;&lt;a&gt;Blog&lt;/a&gt;&lt;/li&gt;
                          * &lt;/ul&gt;
                        * &lt;/li&gt;
                    * &lt;/ul&gt;
   
       * <br> 
       * &lt;script&gt;
        * $("#menu").ejMenu({height:22,}); 
       * //initialize the menu object
       *	var menuObj = $("#menu").data("ejMenu");
       *	//To insert menu item 
       *	menuObj.insertBefore([{
	   *          id:"More",
	   *        text:"More"
	   *		}],"#Home");
      * &lt;/script&gt;
       * @example 
       * &lt;ul id="menu"&gt;
                      * &lt;li id="Home"&gt;&lt;a&gt;Home&lt;/a&gt;&lt;/li&gt;
                      * &lt;li&gt;&lt;a&gt;Search Jobs&lt;/a&gt;
                         * &lt;ul&gt;
                            * &lt;li&gt;&lt;a&gt;Advanced Search&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs by Company&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs by Category&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs by Location&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs by Skills&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs by Designation&lt;/a&gt;&lt;/li&gt;
                          * &lt;/ul&gt;
                      * &lt;/li&gt;
                      * &lt;li id="Post Resume"&gt;&lt;a&gt;Post Resume&lt;/a&gt;&lt;/li&gt;
                      * &lt;li id="Job Seeker"&gt;&lt;a&gt;JobSeeker Login&lt;/a&gt;&lt;/li&gt;
                      * &lt;li id="Fast Forward"&gt;&lt;a&gt;Fast Forward&lt;/a&gt;
                         * &lt;ul&gt;
                            * &lt;li&gt;&lt;a&gt;Resume writing&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Certification&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Resume Spotlight&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs4u&lt;/a&gt;&lt;/li&gt;
                         * &lt;/ul&gt;
                       * &lt;/li&gt;
                       * &lt;li id="More"&gt;&lt;a&gt;More&lt;/a&gt;
                          * &lt;ul&gt;
                             * &lt;li&gt;&lt;a&gt;Mobile&lt;/a&gt;&lt;/li&gt;
                             * &lt;li&gt;&lt;a&gt;Pay check&lt;/a&gt;&lt;/li&gt;
                             * &lt;li&gt;&lt;a&gt;Blog&lt;/a&gt;&lt;/li&gt;
                          * &lt;/ul&gt;
                        * &lt;/li&gt;
                    * &lt;/ul&gt;
   
       * <br> 
       * &lt;script&gt;
        * $("#menu").ejMenu({height:22,}); 
       * //To insert menu item 
       * $("#menu").ejMenu("insertBefore",[{
	   *          id:"More",
	   *        text:"More"
	   *		}],"#Home");
      * &lt;/script&gt;
       * @memberof ejMenu
       * @instance
        */				
		insertBefore:function(item,target){
			this._insertNode(item,target,"insertBefore");
		},
		/**
       * Insert the menu item after the target node.
       * @alias insertAfter
       * @return jQuery
       * @param {ArrayObject}  item  Information about Menu item.
       * @param {string | Object}  target   Selector of target node or Object of target node.
      
       * @example 
       * &lt;ul id="menu"&gt;
                      * &lt;li id="Home"&gt;&lt;a&gt;Home&lt;/a&gt;&lt;/li&gt;
                      * &lt;li&gt;&lt;a&gt;Search Jobs&lt;/a&gt;
                         * &lt;ul&gt;
                            * &lt;li&gt;&lt;a&gt;Advanced Search&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs by Company&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs by Category&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs by Location&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs by Skills&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs by Designation&lt;/a&gt;&lt;/li&gt;
                          * &lt;/ul&gt;
                      * &lt;/li&gt;
                      * &lt;li id="Post Resume"&gt;&lt;a&gt;Post Resume&lt;/a&gt;&lt;/li&gt;
                      * &lt;li id="Job Seeker"&gt;&lt;a&gt;JobSeeker Login&lt;/a&gt;&lt;/li&gt;
                      * &lt;li id="Fast Forward"&gt;&lt;a&gt;Fast Forward&lt;/a&gt;
                         * &lt;ul&gt;
                            * &lt;li&gt;&lt;a&gt;Resume writing&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Certification&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Resume Spotlight&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs4u&lt;/a&gt;&lt;/li&gt;
                         * &lt;/ul&gt;
                       * &lt;/li&gt;
                       * &lt;li id="More"&gt;&lt;a&gt;More&lt;/a&gt;
                          * &lt;ul&gt;
                             * &lt;li&gt;&lt;a&gt;Mobile&lt;/a&gt;&lt;/li&gt;
                             * &lt;li&gt;&lt;a&gt;Pay check&lt;/a&gt;&lt;/li&gt;
                             * &lt;li&gt;&lt;a&gt;Blog&lt;/a&gt;&lt;/li&gt;
                          * &lt;/ul&gt;
                        * &lt;/li&gt;
                    * &lt;/ul&gt;
   
       * <br> 
       * &lt;script&gt;
        * $("#menu").ejMenu({height:22,}); 
       * //initialize the menu object
       *	var menuObj = $("#menu").data("ejMenu");
       *	//To insert menu item 
       *	menuObj.insertAfter([{
	   *          id:"More",
	   *        text:"More"
	   *		}],"#Home");
     * &lt;/script&gt;
       * @example 
       * &lt;ul id="menu"&gt;
                      * &lt;li id="Home"&gt;&lt;a&gt;Home&lt;/a&gt;&lt;/li&gt;
                      * &lt;li&gt;&lt;a&gt;Search Jobs&lt;/a&gt;
                         * &lt;ul&gt;
                            * &lt;li&gt;&lt;a&gt;Advanced Search&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs by Company&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs by Category&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs by Location&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs by Skills&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs by Designation&lt;/a&gt;&lt;/li&gt;
                          * &lt;/ul&gt;
                      * &lt;/li&gt;
                      * &lt;li id="Post Resume"&gt;&lt;a&gt;Post Resume&lt;/a&gt;&lt;/li&gt;
                      * &lt;li id="Job Seeker"&gt;&lt;a&gt;JobSeeker Login&lt;/a&gt;&lt;/li&gt;
                      * &lt;li id="Fast Forward"&gt;&lt;a&gt;Fast Forward&lt;/a&gt;
                         * &lt;ul&gt;
                            * &lt;li&gt;&lt;a&gt;Resume writing&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Certification&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Resume Spotlight&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs4u&lt;/a&gt;&lt;/li&gt;
                         * &lt;/ul&gt;
                       * &lt;/li&gt;
                       * &lt;li id="More"&gt;&lt;a&gt;More&lt;/a&gt;
                          * &lt;ul&gt;
                             * &lt;li&gt;&lt;a&gt;Mobile&lt;/a&gt;&lt;/li&gt;
                             * &lt;li&gt;&lt;a&gt;Pay check&lt;/a&gt;&lt;/li&gt;
                             * &lt;li&gt;&lt;a&gt;Blog&lt;/a&gt;&lt;/li&gt;
                          * &lt;/ul&gt;
                        * &lt;/li&gt;
                    * &lt;/ul&gt;
   
       * <br> 
       * &lt;script&gt;
        * $("#menu").ejMenu({height:22,}); 
       * //To insert menu item 
       * $("#menu").ejMenu("insertAfter",[{
	   *          id:"More",
	   *        text:"More"
	   *		}],"#Home");
     * &lt;/script&gt;
       * @memberof ejMenu
       * @instance
        */	
		insertAfter:function(item,target){
			this._insertNode(item,target,"insertAfter");
		},
		/**
       * Remove Menu item.
       * @alias remove
       * @return jQuery
       * @param {ArrayObject | ArrayString}  target   Selector of target node or Object of target node.
      
       * @example 
       * &lt;ul id="menu"&gt;
                      * &lt;li id="Home"&gt;&lt;a&gt;Home&lt;/a&gt;&lt;/li&gt;
                      * &lt;li&gt;&lt;a&gt;Search Jobs&lt;/a&gt;
                         * &lt;ul&gt;
                            * &lt;li&gt;&lt;a&gt;Advanced Search&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs by Company&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs by Category&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs by Location&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs by Skills&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs by Designation&lt;/a&gt;&lt;/li&gt;
                          * &lt;/ul&gt;
                      * &lt;/li&gt;
                      * &lt;li id="Post Resume"&gt;&lt;a&gt;Post Resume&lt;/a&gt;&lt;/li&gt;
                      * &lt;li id="Job Seeker"&gt;&lt;a&gt;JobSeeker Login&lt;/a&gt;&lt;/li&gt;
                      * &lt;li id="Fast Forward"&gt;&lt;a&gt;Fast Forward&lt;/a&gt;
                         * &lt;ul&gt;
                            * &lt;li&gt;&lt;a&gt;Resume writing&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Certification&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Resume Spotlight&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs4u&lt;/a&gt;&lt;/li&gt;
                         * &lt;/ul&gt;
                       * &lt;/li&gt;
                       * &lt;li id="More"&gt;&lt;a&gt;More&lt;/a&gt;
                          * &lt;ul&gt;
                             * &lt;li&gt;&lt;a&gt;Mobile&lt;/a&gt;&lt;/li&gt;
                             * &lt;li&gt;&lt;a&gt;Pay check&lt;/a&gt;&lt;/li&gt;
                             * &lt;li&gt;&lt;a&gt;Blog&lt;/a&gt;&lt;/li&gt;
                          * &lt;/ul&gt;
                        * &lt;/li&gt;
                    * &lt;/ul&gt;
   
       * <br> 
       * &lt;script&gt;
        * $("#menu").ejMenu({height:22,}); 
       * //initialize the menu object
       * var menuObj = $("#menu").data("ejMenu");
       * //To remove menu item 
       * menuObj.remove(["#Home"]);
       * &lt;/script&gt;
       * @example 
       * &lt;ul id="menu"&gt;
                      * &lt;li id="Home"&gt;&lt;a&gt;Home&lt;/a&gt;&lt;/li&gt;
                      * &lt;li&gt;&lt;a&gt;Search Jobs&lt;/a&gt;
                         * &lt;ul&gt;
                            * &lt;li&gt;&lt;a&gt;Advanced Search&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs by Company&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs by Category&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs by Location&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs by Skills&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs by Designation&lt;/a&gt;&lt;/li&gt;
                          * &lt;/ul&gt;
                      * &lt;/li&gt;
                      * &lt;li id="Post Resume"&gt;&lt;a&gt;Post Resume&lt;/a&gt;&lt;/li&gt;
                      * &lt;li id="Job Seeker"&gt;&lt;a&gt;JobSeeker Login&lt;/a&gt;&lt;/li&gt;
                      * &lt;li id="Fast Forward"&gt;&lt;a&gt;Fast Forward&lt;/a&gt;
                         * &lt;ul&gt;
                            * &lt;li&gt;&lt;a&gt;Resume writing&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Certification&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Resume Spotlight&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs4u&lt;/a&gt;&lt;/li&gt;
                         * &lt;/ul&gt;
                       * &lt;/li&gt;
                       * &lt;li id="More"&gt;&lt;a&gt;More&lt;/a&gt;
                          * &lt;ul&gt;
                             * &lt;li&gt;&lt;a&gt;Mobile&lt;/a&gt;&lt;/li&gt;
                             * &lt;li&gt;&lt;a&gt;Pay check&lt;/a&gt;&lt;/li&gt;
                             * &lt;li&gt;&lt;a&gt;Blog&lt;/a&gt;&lt;/li&gt;
                          * &lt;/ul&gt;
                        * &lt;/li&gt;
                    * &lt;/ul&gt;
   
       * <br> 
       * &lt;script&gt;
        * $("#menu").ejMenu({height:22,}); 
       * //To remove menu item 
       * $("#menu").ejMenu("remove",["#Home"]);
       * &lt;/script&gt;
       * @memberof ejMenu
       * @instance
        */
		remove:function(targetCollection){
		var target=0,innerTarget=0;   
			 for(target=0;target<targetCollection.length;target++){
				targetCollection[target] = typeof(targetCollection[target]) === "string" ? (this.element.find(targetCollection[target])):targetCollection[target];
					for(innerTarget=0;innerTarget<targetCollection[target].length;innerTarget++)
					(targetCollection[target][innerTarget].tagName === "LI" || targetCollection[target][innerTarget].tagName === "UL")? this._removeItem($(targetCollection[target][innerTarget])):targetCollection[target][innerTarget].remove();
			}
		},	
     /**
       * Shows the Context Menu .
       * @alias showContextMenu
       * @return jQuery
       * @param {number}  locationX  x co-ordinate position of context menu. 
       * @param {number}  locationY   y co-ordinate position of context menu. 
       * @param {object}  targetElement  target element 
       * @param {object}  event name of the event
       * @example 
       * &lt;ul id="menu"&gt;
                      * &lt;li id="Home"&gt;&lt;a&gt;Home&lt;/a&gt;&lt;/li&gt;
                      * &lt;li&gt;&lt;a&gt;Search Jobs&lt;/a&gt;
                         * &lt;ul&gt;
                            * &lt;li&gt;&lt;a&gt;Advanced Search&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs by Company&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs by Category&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs by Location&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs by Skills&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs by Designation&lt;/a&gt;&lt;/li&gt;
                          * &lt;/ul&gt;
                      * &lt;/li&gt;
                      * &lt;li id="Post Resume"&gt;&lt;a&gt;Post Resume&lt;/a&gt;&lt;/li&gt;
                      * &lt;li id="Job Seeker"&gt;&lt;a&gt;JobSeeker Login&lt;/a&gt;&lt;/li&gt;
                      * &lt;li id="Fast Forward"&gt;&lt;a&gt;Fast Forward&lt;/a&gt;
                         * &lt;ul&gt;
                            * &lt;li&gt;&lt;a&gt;Resume writing&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Certification&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Resume Spotlight&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs4u&lt;/a&gt;&lt;/li&gt;
                         * &lt;/ul&gt;
                       * &lt;/li&gt;
                       * &lt;li id="More"&gt;&lt;a&gt;More&lt;/a&gt;
                          * &lt;ul&gt;
                             * &lt;li&gt;&lt;a&gt;Mobile&lt;/a&gt;&lt;/li&gt;
                             * &lt;li&gt;&lt;a&gt;Pay check&lt;/a&gt;&lt;/li&gt;
                             * &lt;li&gt;&lt;a&gt;Blog&lt;/a&gt;&lt;/li&gt;
                          * &lt;/ul&gt;
                        * &lt;/li&gt;
                    * &lt;/ul&gt;
   
       * <br> 
       * &lt;script&gt;
        * $("#menu").ejMenu({height:22,}); 
       * //initialize the menu object
       *	var menuObj = $("#menu").data("ejMenu");
       *	//To show context menu
       *	menuObj.ShowContextMenu();
       * &lt;/script&gt;
       * @example 
       * &lt;ul id="menu"&gt;
                      * &lt;li id="Home"&gt;&lt;a&gt;Home&lt;/a&gt;&lt;/li&gt;
                      * &lt;li&gt;&lt;a&gt;Search Jobs&lt;/a&gt;
                         * &lt;ul&gt;
                            * &lt;li&gt;&lt;a&gt;Advanced Search&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs by Company&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs by Category&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs by Location&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs by Skills&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs by Designation&lt;/a&gt;&lt;/li&gt;
                          * &lt;/ul&gt;
                      * &lt;/li&gt;
                      * &lt;li id="Post Resume"&gt;&lt;a&gt;Post Resume&lt;/a&gt;&lt;/li&gt;
                      * &lt;li id="Job Seeker"&gt;&lt;a&gt;JobSeeker Login&lt;/a&gt;&lt;/li&gt;
                      * &lt;li id="Fast Forward"&gt;&lt;a&gt;Fast Forward&lt;/a&gt;
                         * &lt;ul&gt;
                            * &lt;li&gt;&lt;a&gt;Resume writing&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Certification&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Resume Spotlight&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs4u&lt;/a&gt;&lt;/li&gt;
                         * &lt;/ul&gt;
                       * &lt;/li&gt;
                       * &lt;li id="More"&gt;&lt;a&gt;More&lt;/a&gt;
                          * &lt;ul&gt;
                             * &lt;li&gt;&lt;a&gt;Mobile&lt;/a&gt;&lt;/li&gt;
                             * &lt;li&gt;&lt;a&gt;Pay check&lt;/a&gt;&lt;/li&gt;
                             * &lt;li&gt;&lt;a&gt;Blog&lt;/a&gt;&lt;/li&gt;
                          * &lt;/ul&gt;
                        * &lt;/li&gt;
                    * &lt;/ul&gt;
   
       * <br> 
       * &lt;script&gt;
        * $("#menu").ejMenu({height:22,}); 
       * //To show context menu
       * $("#menu").ejMenu("ShowContextMenu");
       * &lt;/script&gt;
       * @memberof ejMenu
       * @instance
        */
        //Displays the context menu
        _showContextMenu: function (locationX, locationY, targetElement, e) {
            this._closeMenu();
            this._eventArgs = e;
            if (this._trigger("beforeOpen", { target: targetElement })) return false;
            if (this._preventContextOpen) {
                this.element.css({ "left": locationX, "top": locationY });
                this.element.css({ "z-index": this._max_zindex() + 1 });
                this._showAnimation(this.element, this._showAnim);
                this._isContextMenuOpen = true;
               
                    this._trigger("open", { target: targetElement });
            }
            return false;
        },
      /**
       * Hides the Context Menu control.
       * @alias hideContextMenu
       * @return jQuery
       * @example 
       * &lt;ul id="menu"&gt;
                      * &lt;li id="Home"&gt;&lt;a&gt;Home&lt;/a&gt;&lt;/li&gt;
                      * &lt;li&gt;&lt;a&gt;Search Jobs&lt;/a&gt;
                         * &lt;ul&gt;
                            * &lt;li&gt;&lt;a&gt;Advanced Search&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs by Company&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs by Category&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs by Location&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs by Skills&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs by Designation&lt;/a&gt;&lt;/li&gt;
                          * &lt;/ul&gt;
                      * &lt;/li&gt;
                      * &lt;li id="Post Resume"&gt;&lt;a&gt;Post Resume&lt;/a&gt;&lt;/li&gt;
                      * &lt;li id="Job Seeker"&gt;&lt;a&gt;JobSeeker Login&lt;/a&gt;&lt;/li&gt;
                      * &lt;li id="Fast Forward"&gt;&lt;a&gt;Fast Forward&lt;/a&gt;
                         * &lt;ul&gt;
                            * &lt;li&gt;&lt;a&gt;Resume writing&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Certification&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Resume Spotlight&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs4u&lt;/a&gt;&lt;/li&gt;
                         * &lt;/ul&gt;
                       * &lt;/li&gt;
                       * &lt;li id="More"&gt;&lt;a&gt;More&lt;/a&gt;
                          * &lt;ul&gt;
                             * &lt;li&gt;&lt;a&gt;Mobile&lt;/a&gt;&lt;/li&gt;
                             * &lt;li&gt;&lt;a&gt;Pay check&lt;/a&gt;&lt;/li&gt;
                             * &lt;li&gt;&lt;a&gt;Blog&lt;/a&gt;&lt;/li&gt;
                          * &lt;/ul&gt;
                        * &lt;/li&gt;
                    * &lt;/ul&gt;
   
       * <br> 
       * &lt;script&gt;
        * $("#menu").ejMenu({height:22,}); 
       * //initialize the menu object
       *	var menuObj = $("#menu").data("ejMenu");
       *	//To hide context menu
       *	menuObj.hideContextMenu();
       * &lt;/script&gt;
       * @example 
       * &lt;ul id="menu"&gt;
                      * &lt;li id="Home"&gt;&lt;a&gt;Home&lt;/a&gt;&lt;/li&gt;
                      * &lt;li&gt;&lt;a&gt;Search Jobs&lt;/a&gt;
                         * &lt;ul&gt;
                            * &lt;li&gt;&lt;a&gt;Advanced Search&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs by Company&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs by Category&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs by Location&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs by Skills&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs by Designation&lt;/a&gt;&lt;/li&gt;
                          * &lt;/ul&gt;
                      * &lt;/li&gt;
                      * &lt;li id="Post Resume"&gt;&lt;a&gt;Post Resume&lt;/a&gt;&lt;/li&gt;
                      * &lt;li id="Job Seeker"&gt;&lt;a&gt;JobSeeker Login&lt;/a&gt;&lt;/li&gt;
                      * &lt;li id="Fast Forward"&gt;&lt;a&gt;Fast Forward&lt;/a&gt;
                         * &lt;ul&gt;
                            * &lt;li&gt;&lt;a&gt;Resume writing&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Certification&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Resume Spotlight&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs4u&lt;/a&gt;&lt;/li&gt;
                         * &lt;/ul&gt;
                       * &lt;/li&gt;
                       * &lt;li id="More"&gt;&lt;a&gt;More&lt;/a&gt;
                          * &lt;ul&gt;
                             * &lt;li&gt;&lt;a&gt;Mobile&lt;/a&gt;&lt;/li&gt;
                             * &lt;li&gt;&lt;a&gt;Pay check&lt;/a&gt;&lt;/li&gt;
                             * &lt;li&gt;&lt;a&gt;Blog&lt;/a&gt;&lt;/li&gt;
                          * &lt;/ul&gt;
                        * &lt;/li&gt;
                    * &lt;/ul&gt;
   
       * <br> 
       * &lt;script&gt;
        * $("#menu").ejMenu({height:22,}); 
       * //To hide context menu
       * $("#menu").ejMenu("hideContextMenu");
       * &lt;/script&gt;
       * @memberof ejMenu
       * @instance
        */
        //Hides the context Menu
        _hideContextMenu: function (e) {
            this._closeMenu();
            this.element.find(".e-mhover").removeClass("e-mhover");
            this.element.find(".e-mfocused").removeClass("e-mfocused");
            this._hideAnimation(this.element, this._hideAnim);
            this._isContextMenuOpen = false;
           
            this._trigger("close", $.extend({ events: e }, e));
        },
        /**
       * Specifies the Menu Item to be disabled by using the Menu Item Text.
       * @alias disableItem 
       * @return jQuery
	   * @param {string} itemtext  Specifies the Menu Item Text to be disabled. 
       * @example 
        * &lt;ul id="menu"&gt;
                      * &lt;li id="Home"&gt;&lt;a&gt;Home&lt;/a&gt;&lt;/li&gt;
                      * &lt;li&gt;&lt;a&gt;Search Jobs&lt;/a&gt;
                         * &lt;ul&gt;
                            * &lt;li&gt;&lt;a&gt;Advanced Search&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs by Company&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs by Category&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs by Location&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs by Skills&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs by Designation&lt;/a&gt;&lt;/li&gt;
                          * &lt;/ul&gt;
                      * &lt;/li&gt;
                      * &lt;li id="Post Resume"&gt;&lt;a&gt;Post Resume&lt;/a&gt;&lt;/li&gt;
                      * &lt;li id="Job Seeker"&gt;&lt;a&gt;JobSeeker Login&lt;/a&gt;&lt;/li&gt;
                      * &lt;li id="Fast Forward"&gt;&lt;a&gt;Fast Forward&lt;/a&gt;
                         * &lt;ul&gt;
                            * &lt;li&gt;&lt;a&gt;Resume writing&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Certification&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Resume Spotlight&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs4u&lt;/a&gt;&lt;/li&gt;
                         * &lt;/ul&gt;
                       * &lt;/li&gt;
                       * &lt;li id="More"&gt;&lt;a&gt;More&lt;/a&gt;
                          * &lt;ul&gt;
                             * &lt;li&gt;&lt;a&gt;Mobile&lt;/a&gt;&lt;/li&gt;
                             * &lt;li&gt;&lt;a&gt;Pay check&lt;/a&gt;&lt;/li&gt;
                             * &lt;li&gt;&lt;a&gt;Blog&lt;/a&gt;&lt;/li&gt;
                          * &lt;/ul&gt;
                        * &lt;/li&gt;
                    * &lt;/ul&gt;
   
       * <br> 
       * &lt;script&gt;
        * $("#menu").ejMenu({height:22,}); 
       * //initialize the menu object
       *	var menuObj = $("#menu").data("ejMenu");
       *	//To disable Menu item using item text
       *	menuObj.disableItem("Home");
       * &lt;/script&gt;
       * @example 
        * &lt;ul id="menu"&gt;
                      * &lt;li id="Home"&gt;&lt;a&gt;Home&lt;/a&gt;&lt;/li&gt;
                      * &lt;li&gt;&lt;a&gt;Search Jobs&lt;/a&gt;
                         * &lt;ul&gt;
                            * &lt;li&gt;&lt;a&gt;Advanced Search&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs by Company&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs by Category&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs by Location&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs by Skills&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs by Designation&lt;/a&gt;&lt;/li&gt;
                          * &lt;/ul&gt;
                      * &lt;/li&gt;
                      * &lt;li id="Post Resume"&gt;&lt;a&gt;Post Resume&lt;/a&gt;&lt;/li&gt;
                      * &lt;li id="Job Seeker"&gt;&lt;a&gt;JobSeeker Login&lt;/a&gt;&lt;/li&gt;
                      * &lt;li id="Fast Forward"&gt;&lt;a&gt;Fast Forward&lt;/a&gt;
                         * &lt;ul&gt;
                            * &lt;li&gt;&lt;a&gt;Resume writing&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Certification&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Resume Spotlight&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs4u&lt;/a&gt;&lt;/li&gt;
                         * &lt;/ul&gt;
                       * &lt;/li&gt;
                       * &lt;li id="More"&gt;&lt;a&gt;More&lt;/a&gt;
                          * &lt;ul&gt;
                             * &lt;li&gt;&lt;a&gt;Mobile&lt;/a&gt;&lt;/li&gt;
                             * &lt;li&gt;&lt;a&gt;Pay check&lt;/a&gt;&lt;/li&gt;
                             * &lt;li&gt;&lt;a&gt;Blog&lt;/a&gt;&lt;/li&gt;
                          * &lt;/ul&gt;
                        * &lt;/li&gt;
                    * &lt;/ul&gt;
   
       * <br> 
       * &lt;script&gt;
        * $("#menu").ejMenu({height:22,}); 
       * //To disable Menu item using item text
       * $("#menu").ejMenu("disableItem","Home");
       * &lt;/script&gt;
       * @memberof ejMenu
       * @instance
        */
        //To Disable menu item 
        disableItem: function (itemToDisable) {
           var isMenuItem = $(this.element.find('li.e-list >a ,li.e-list >span')).filter(function () { return $.trim($(this).text()) === itemToDisable; });
            if (isMenuItem.length > 0 && !($.inArray(itemToDisable, this._disabledMenuItems) > -1)) {
                isMenuItem.parent().addClass("e-disable-item").attr({ "aria-disabled": true });
                this._disabledMenuItems.push($.trim(isMenuItem.text()));
            }
        },
         /**
        * Specifies the Menu Item to be disabled by using the Menu Item Id.
		* @alias disableItembyID
		* @return jQuery
		* @param {string|number} itemid  Specifies the Menu Item id to be disabled 
		* @example 
    * &lt;ul id="menu"&gt;
                      * &lt;li id="Home"&gt;&lt;a&gt;Home&lt;/a&gt;&lt;/li&gt;
                      * &lt;li&gt;&lt;a&gt;Search Jobs&lt;/a&gt;
                         * &lt;ul&gt;
                            * &lt;li&gt;&lt;a&gt;Advanced Search&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs by Company&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs by Category&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs by Location&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs by Skills&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs by Designation&lt;/a&gt;&lt;/li&gt;
                          * &lt;/ul&gt;
                      * &lt;/li&gt;
                      * &lt;li id="Post Resume"&gt;&lt;a&gt;Post Resume&lt;/a&gt;&lt;/li&gt;
                      * &lt;li id="Job Seeker"&gt;&lt;a&gt;JobSeeker Login&lt;/a&gt;&lt;/li&gt;
                      * &lt;li id="Fast Forward"&gt;&lt;a&gt;Fast Forward&lt;/a&gt;
                         * &lt;ul&gt;
                            * &lt;li&gt;&lt;a&gt;Resume writing&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Certification&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Resume Spotlight&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs4u&lt;/a&gt;&lt;/li&gt;
                         * &lt;/ul&gt;
                       * &lt;/li&gt;
                       * &lt;li id="More"&gt;&lt;a&gt;More&lt;/a&gt;
                          * &lt;ul&gt;
                             * &lt;li&gt;&lt;a&gt;Mobile&lt;/a&gt;&lt;/li&gt;
                             * &lt;li&gt;&lt;a&gt;Pay check&lt;/a&gt;&lt;/li&gt;
                             * &lt;li&gt;&lt;a&gt;Blog&lt;/a&gt;&lt;/li&gt;
                          * &lt;/ul&gt;
                        * &lt;/li&gt;
                    * &lt;/ul&gt;
   
       * <br> 
       * &lt;script&gt;
        * $("#menu").ejMenu({height:22,}); 
		* //initialize the menu object
		*	var menuObj = $("#menu").data("ejMenu");
		*	//To disable Menu item using item id
		*	menuObj.disableItemByID("More");
    * &lt;/script&gt;
		* @example 
    * &lt;ul id="menu"&gt;
                      * &lt;li id="Home"&gt;&lt;a&gt;Home&lt;/a&gt;&lt;/li&gt;
                      * &lt;li&gt;&lt;a&gt;Search Jobs&lt;/a&gt;
                         * &lt;ul&gt;
                            * &lt;li&gt;&lt;a&gt;Advanced Search&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs by Company&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs by Category&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs by Location&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs by Skills&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs by Designation&lt;/a&gt;&lt;/li&gt;
                          * &lt;/ul&gt;
                      * &lt;/li&gt;
                      * &lt;li id="Post Resume"&gt;&lt;a&gt;Post Resume&lt;/a&gt;&lt;/li&gt;
                      * &lt;li id="Job Seeker"&gt;&lt;a&gt;JobSeeker Login&lt;/a&gt;&lt;/li&gt;
                      * &lt;li id="Fast Forward"&gt;&lt;a&gt;Fast Forward&lt;/a&gt;
                         * &lt;ul&gt;
                            * &lt;li&gt;&lt;a&gt;Resume writing&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Certification&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Resume Spotlight&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs4u&lt;/a&gt;&lt;/li&gt;
                         * &lt;/ul&gt;
                       * &lt;/li&gt;
                       * &lt;li id="More"&gt;&lt;a&gt;More&lt;/a&gt;
                          * &lt;ul&gt;
                             * &lt;li&gt;&lt;a&gt;Mobile&lt;/a&gt;&lt;/li&gt;
                             * &lt;li&gt;&lt;a&gt;Pay check&lt;/a&gt;&lt;/li&gt;
                             * &lt;li&gt;&lt;a&gt;Blog&lt;/a&gt;&lt;/li&gt;
                          * &lt;/ul&gt;
                        * &lt;/li&gt;
                    * &lt;/ul&gt;
   
       * <br> 
       * &lt;script&gt;
        * $("#menu").ejMenu({height:22,}); 
		* //To disable Menu item using item id
		* $("#menu").ejMenu("disableItemByID","More");
    * &lt;/script&gt;
		* @memberof ejMenu
		* @instance
        */
        //To disable menu item by id
        disableItemByID: function (itemId) {
            if (itemId && itemId != "") {
                var itemToDisable = this.element.find("#" + itemId) ? this.element.find("#" + itemId)[0] : undefined;
                if (itemToDisable && !($.inArray(itemToDisable, this._disabledMenuItems) > -1)) {
                    $(itemToDisable).addClass("e-disable-item").attr({ "aria-disabled": true });
                    this._disabledMenuItems.push(itemToDisable);
                }
            }
        },

       
        /**
        *Specifies the Menu Item to be enabled by using the Menu Item Text.
		* @alias enableItem
		* @return jQuery
		* @param {string} itemtext  Specifies the Menu Item Text to be enabled.
		* @example 
     * &lt;ul id="menu"&gt;
                      * &lt;li id="Home"&gt;&lt;a&gt;Home&lt;/a&gt;&lt;/li&gt;
                      * &lt;li&gt;&lt;a&gt;Search Jobs&lt;/a&gt;
                         * &lt;ul&gt;
                            * &lt;li&gt;&lt;a&gt;Advanced Search&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs by Company&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs by Category&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs by Location&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs by Skills&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs by Designation&lt;/a&gt;&lt;/li&gt;
                          * &lt;/ul&gt;
                      * &lt;/li&gt;
                      * &lt;li id="Post Resume"&gt;&lt;a&gt;Post Resume&lt;/a&gt;&lt;/li&gt;
                      * &lt;li id="Job Seeker"&gt;&lt;a&gt;JobSeeker Login&lt;/a&gt;&lt;/li&gt;
                      * &lt;li id="Fast Forward"&gt;&lt;a&gt;Fast Forward&lt;/a&gt;
                         * &lt;ul&gt;
                            * &lt;li&gt;&lt;a&gt;Resume writing&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Certification&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Resume Spotlight&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs4u&lt;/a&gt;&lt;/li&gt;
                         * &lt;/ul&gt;
                       * &lt;/li&gt;
                       * &lt;li id="More"&gt;&lt;a&gt;More&lt;/a&gt;
                          * &lt;ul&gt;
                             * &lt;li&gt;&lt;a&gt;Mobile&lt;/a&gt;&lt;/li&gt;
                             * &lt;li&gt;&lt;a&gt;Pay check&lt;/a&gt;&lt;/li&gt;
                             * &lt;li&gt;&lt;a&gt;Blog&lt;/a&gt;&lt;/li&gt;
                          * &lt;/ul&gt;
                        * &lt;/li&gt;
                    * &lt;/ul&gt;
   
       * <br> 
       * &lt;script&gt;
        * $("#menu").ejMenu({height:22,}); 
		* //initialize the menu object
		*	var menuObj = $("#menu").data("ejMenu");
		*	//To enable Menu item using item text
		* menuObj.disableItem("Home");
        * menuObj.disableItem("Search Jobs");
		*	menuObj.enableItem("Search Jobs");
     * &lt;/script&gt;
		* @example 
     * &lt;ul id="menu"&gt;
                      * &lt;li id="Home"&gt;&lt;a&gt;Home&lt;/a&gt;&lt;/li&gt;
                      * &lt;li&gt;&lt;a&gt;Search Jobs&lt;/a&gt;
                         * &lt;ul&gt;
                            * &lt;li&gt;&lt;a&gt;Advanced Search&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs by Company&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs by Category&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs by Location&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs by Skills&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs by Designation&lt;/a&gt;&lt;/li&gt;
                          * &lt;/ul&gt;
                      * &lt;/li&gt;
                      * &lt;li id="Post Resume"&gt;&lt;a&gt;Post Resume&lt;/a&gt;&lt;/li&gt;
                      * &lt;li id="Job Seeker"&gt;&lt;a&gt;JobSeeker Login&lt;/a&gt;&lt;/li&gt;
                      * &lt;li id="Fast Forward"&gt;&lt;a&gt;Fast Forward&lt;/a&gt;
                         * &lt;ul&gt;
                            * &lt;li&gt;&lt;a&gt;Resume writing&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Certification&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Resume Spotlight&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs4u&lt;/a&gt;&lt;/li&gt;
                         * &lt;/ul&gt;
                       * &lt;/li&gt;
                       * &lt;li id="More"&gt;&lt;a&gt;More&lt;/a&gt;
                          * &lt;ul&gt;
                             * &lt;li&gt;&lt;a&gt;Mobile&lt;/a&gt;&lt;/li&gt;
                             * &lt;li&gt;&lt;a&gt;Pay check&lt;/a&gt;&lt;/li&gt;
                             * &lt;li&gt;&lt;a&gt;Blog&lt;/a&gt;&lt;/li&gt;
                          * &lt;/ul&gt;
                        * &lt;/li&gt;
                    * &lt;/ul&gt;
   
       * <br> 
       * &lt;script&gt;
        * $("#menu").ejMenu({height:22}); 
		* //To enable Menu item using item text
        * $("#menu").ejMenu("disableItem","Home");
        * $("#menu").ejMenu("disableItem","Search Jobs");
		* $("#menu").ejMenu("enableItem","Search Jobs");
     * &lt;/script&gt
		* @memberof ejMenu
		* @instance
        */
        //To enable menu item 
        enableItem: function (itemToEnable) {
            var isMenuItem = $(this.element.find('li.e-list >a ,li.e-list >span')).filter(function () { return $.trim($(this).text()) === itemToEnable; });
            if (isMenuItem.length > 0 && ($.inArray(itemToEnable, this._disabledMenuItems) > -1)) {
                isMenuItem.parent().removeClass("e-disable-item").attr({ "aria-disabled": false });
                var index = this._removeValue(isMenuItem, this._disabledMenuItems);
                this._disabledMenuItems.splice(index, 1);
            }
        },
         /**
        *Specifies the Menu Item to be enabled by using the Menu Item Id.
		* @alias enableItembyID
		* @return jQuery
		* @param {string|number} itemid  Specifies the Menu Item id to be enabled. 
		* @example 
     * &lt;ul id="menu"&gt;
                      * &lt;li id="Home"&gt;&lt;a&gt;Home&lt;/a&gt;&lt;/li&gt;
                      * &lt;li&gt;&lt;a&gt;Search Jobs&lt;/a&gt;
                         * &lt;ul&gt;
                            * &lt;li&gt;&lt;a&gt;Advanced Search&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs by Company&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs by Category&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs by Location&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs by Skills&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs by Designation&lt;/a&gt;&lt;/li&gt;
                          * &lt;/ul&gt;
                      * &lt;/li&gt;
                      * &lt;li id="Post Resume"&gt;&lt;a&gt;Post Resume&lt;/a&gt;&lt;/li&gt;
                      * &lt;li id="Job Seeker"&gt;&lt;a&gt;JobSeeker Login&lt;/a&gt;&lt;/li&gt;
                      * &lt;li id="Fast Forward"&gt;&lt;a&gt;Fast Forward&lt;/a&gt;
                         * &lt;ul&gt;
                            * &lt;li&gt;&lt;a&gt;Resume writing&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Certification&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Resume Spotlight&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs4u&lt;/a&gt;&lt;/li&gt;
                         * &lt;/ul&gt;
                       * &lt;/li&gt;
                       * &lt;li id="More"&gt;&lt;a&gt;More&lt;/a&gt;
                          * &lt;ul&gt;
                             * &lt;li&gt;&lt;a&gt;Mobile&lt;/a&gt;&lt;/li&gt;
                             * &lt;li&gt;&lt;a&gt;Pay check&lt;/a&gt;&lt;/li&gt;
                             * &lt;li&gt;&lt;a&gt;Blog&lt;/a&gt;&lt;/li&gt;
                          * &lt;/ul&gt;
                        * &lt;/li&gt;
                    * &lt;/ul&gt;
   
       * <br> 
       * &lt;script&gt;
        * $("#menu").ejMenu({height:22,}); 
		* //initialize the menu object
		*	var menuObj = $("#menu").data("ejMenu");
		*	//To enable Menu item using item id
		* menuObj.disable();
		*	menuObj.enableItemByID("More");
     * &lt;/script&gt;
		* @example 
     * &lt;ul id="menu"&gt;
                      * &lt;li id="Home"&gt;&lt;a&gt;Home&lt;/a&gt;&lt;/li&gt;
                      * &lt;li&gt;&lt;a&gt;Search Jobs&lt;/a&gt;
                         * &lt;ul&gt;
                            * &lt;li&gt;&lt;a&gt;Advanced Search&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs by Company&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs by Category&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs by Location&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs by Skills&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs by Designation&lt;/a&gt;&lt;/li&gt;
                          * &lt;/ul&gt;
                      * &lt;/li&gt;
                      * &lt;li id="Post Resume"&gt;&lt;a&gt;Post Resume&lt;/a&gt;&lt;/li&gt;
                      * &lt;li id="Job Seeker"&gt;&lt;a&gt;JobSeeker Login&lt;/a&gt;&lt;/li&gt;
                      * &lt;li id="Fast Forward"&gt;&lt;a&gt;Fast Forward&lt;/a&gt;
                         * &lt;ul&gt;
                            * &lt;li&gt;&lt;a&gt;Resume writing&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Certification&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Resume Spotlight&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs4u&lt;/a&gt;&lt;/li&gt;
                         * &lt;/ul&gt;
                       * &lt;/li&gt;
                       * &lt;li id="More"&gt;&lt;a&gt;More&lt;/a&gt;
                          * &lt;ul&gt;
                             * &lt;li&gt;&lt;a&gt;Mobile&lt;/a&gt;&lt;/li&gt;
                             * &lt;li&gt;&lt;a&gt;Pay check&lt;/a&gt;&lt;/li&gt;
                             * &lt;li&gt;&lt;a&gt;Blog&lt;/a&gt;&lt;/li&gt;
                          * &lt;/ul&gt;
                        * &lt;/li&gt;
                    * &lt;/ul&gt;
   
       * <br> 
       * &lt;script&gt;
        * $("#menu").ejMenu({height:22,}); 
		* //To enable Menu item using item id
		*  $("#menu").ejMenu("disable");
		* $("#menu").ejMenu("enableItemByID","More");
     * &lt;/script&gt;
		* @memberof ejMenu
		* @instance
        */
        //To enable menu item by id
        enableItemByID: function (itemId) {
            if (itemId && itemId != "") {
                var itemToEnable = this.element.find("#" + itemId)[0];
                if (itemToEnable && ($.inArray(itemToEnable, this._disabledMenuItems) > -1)) {
                    $(itemToEnable).removeClass("e-disable-item").attr({ "aria-disabled": false });
                    for(var i = this._disabledMenuItems.length - 1; i >= 0; i--) {
                        if(this._disabledMenuItems[i].id == itemId) {
                           this._disabledMenuItems.splice(i, 1);
                        }
                    }
                }
            }
        },
       /**
        * Disables the Menu control.
		* @alias disable
		* @return jQuery
		* @example 
     * &lt;ul id="menu"&gt;
                      * &lt;li id="Home"&gt;&lt;a&gt;Home&lt;/a&gt;&lt;/li&gt;
                      * &lt;li&gt;&lt;a&gt;Search Jobs&lt;/a&gt;
                         * &lt;ul&gt;
                            * &lt;li&gt;&lt;a&gt;Advanced Search&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs by Company&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs by Category&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs by Location&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs by Skills&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs by Designation&lt;/a&gt;&lt;/li&gt;
                          * &lt;/ul&gt;
                      * &lt;/li&gt;
                      * &lt;li id="Post Resume"&gt;&lt;a&gt;Post Resume&lt;/a&gt;&lt;/li&gt;
                      * &lt;li id="Job Seeker"&gt;&lt;a&gt;JobSeeker Login&lt;/a&gt;&lt;/li&gt;
                      * &lt;li id="Fast Forward"&gt;&lt;a&gt;Fast Forward&lt;/a&gt;
                         * &lt;ul&gt;
                            * &lt;li&gt;&lt;a&gt;Resume writing&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Certification&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Resume Spotlight&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs4u&lt;/a&gt;&lt;/li&gt;
                         * &lt;/ul&gt;
                       * &lt;/li&gt;
                       * &lt;li id="More"&gt;&lt;a&gt;More&lt;/a&gt;
                          * &lt;ul&gt;
                             * &lt;li&gt;&lt;a&gt;Mobile&lt;/a&gt;&lt;/li&gt;
                             * &lt;li&gt;&lt;a&gt;Pay check&lt;/a&gt;&lt;/li&gt;
                             * &lt;li&gt;&lt;a&gt;Blog&lt;/a&gt;&lt;/li&gt;
                          * &lt;/ul&gt;
                        * &lt;/li&gt;
                    * &lt;/ul&gt;
   
       * <br> 
       * &lt;script&gt;
        * $("#menu").ejMenu({height:22,}); 
		* //initialize the menu object
		*	var menuObj = $("#menu").data("ejMenu");
		*	//To disable Menu control
		*	menuObj.disable();
    * &lt;/script&gt;
		* @example 
     * &lt;ul id="menu"&gt;
                      * &lt;li id="Home"&gt;&lt;a&gt;Home&lt;/a&gt;&lt;/li&gt;
                      * &lt;li&gt;&lt;a&gt;Search Jobs&lt;/a&gt;
                         * &lt;ul&gt;
                            * &lt;li&gt;&lt;a&gt;Advanced Search&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs by Company&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs by Category&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs by Location&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs by Skills&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs by Designation&lt;/a&gt;&lt;/li&gt;
                          * &lt;/ul&gt;
                      * &lt;/li&gt;
                      * &lt;li id="Post Resume"&gt;&lt;a&gt;Post Resume&lt;/a&gt;&lt;/li&gt;
                      * &lt;li id="Job Seeker"&gt;&lt;a&gt;JobSeeker Login&lt;/a&gt;&lt;/li&gt;
                      * &lt;li id="Fast Forward"&gt;&lt;a&gt;Fast Forward&lt;/a&gt;
                         * &lt;ul&gt;
                            * &lt;li&gt;&lt;a&gt;Resume writing&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Certification&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Resume Spotlight&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs4u&lt;/a&gt;&lt;/li&gt;
                         * &lt;/ul&gt;
                       * &lt;/li&gt;
                       * &lt;li id="More"&gt;&lt;a&gt;More&lt;/a&gt;
                          * &lt;ul&gt;
                             * &lt;li&gt;&lt;a&gt;Mobile&lt;/a&gt;&lt;/li&gt;
                             * &lt;li&gt;&lt;a&gt;Pay check&lt;/a&gt;&lt;/li&gt;
                             * &lt;li&gt;&lt;a&gt;Blog&lt;/a&gt;&lt;/li&gt;
                          * &lt;/ul&gt;
                        * &lt;/li&gt;
                    * &lt;/ul&gt;
   
       * <br> 
       * &lt;script&gt;
        * $("#menu").ejMenu({height:22,}); 
		* //To disable Menu control
		* $("#menu").ejMenu("disable");
    * &lt;/script&gt;
		* @memberof ejMenu
		* @instance
        */
        //To disable menu
        disable: function () {
            this.model.enabled = false;
            var menuItemCollection = this.element.find('>li[class!=e-separator]');
            var proxy = this;
            $.each(menuItemCollection, function (key, value) {
                $(value).addClass("e-disable-item").attr({ "aria-disabled": true });
                proxy._disabledMenuItems.push(value);
            });
        },
        /**
        * Enables the Menu control.
		* @alias enable
		* @return jQuery
		* @example 
    * &lt;ul id="menu"&gt;
                      * &lt;li id="Home"&gt;&lt;a&gt;Home&lt;/a&gt;&lt;/li&gt;
                      * &lt;li&gt;&lt;a&gt;Search Jobs&lt;/a&gt;
                         * &lt;ul&gt;
                            * &lt;li&gt;&lt;a&gt;Advanced Search&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs by Company&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs by Category&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs by Location&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs by Skills&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs by Designation&lt;/a&gt;&lt;/li&gt;
                          * &lt;/ul&gt;
                      * &lt;/li&gt;
                      * &lt;li id="Post Resume"&gt;&lt;a&gt;Post Resume&lt;/a&gt;&lt;/li&gt;
                      * &lt;li id="Job Seeker"&gt;&lt;a&gt;JobSeeker Login&lt;/a&gt;&lt;/li&gt;
                      * &lt;li id="Fast Forward"&gt;&lt;a&gt;Fast Forward&lt;/a&gt;
                         * &lt;ul&gt;
                            * &lt;li&gt;&lt;a&gt;Resume writing&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Certification&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Resume Spotlight&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs4u&lt;/a&gt;&lt;/li&gt;
                         * &lt;/ul&gt;
                       * &lt;/li&gt;
                       * &lt;li id="More"&gt;&lt;a&gt;More&lt;/a&gt;
                          * &lt;ul&gt;
                             * &lt;li&gt;&lt;a&gt;Mobile&lt;/a&gt;&lt;/li&gt;
                             * &lt;li&gt;&lt;a&gt;Pay check&lt;/a&gt;&lt;/li&gt;
                             * &lt;li&gt;&lt;a&gt;Blog&lt;/a&gt;&lt;/li&gt;
                          * &lt;/ul&gt;
                        * &lt;/li&gt;
                    * &lt;/ul&gt;
   
       * <br> 
       * &lt;script&gt;
        * $("#menu").ejMenu({height:22,}); 
		* //initialize the menu object
		*	var menuObj = $("#menu").data("ejMenu");
		*	//To enable Menu control
		*	menuObj.enable();
     * &lt;/script&gt;
		* @example 
    * &lt;ul id="menu"&gt;
                      * &lt;li id="Home"&gt;&lt;a&gt;Home&lt;/a&gt;&lt;/li&gt;
                      * &lt;li&gt;&lt;a&gt;Search Jobs&lt;/a&gt;
                         * &lt;ul&gt;
                            * &lt;li&gt;&lt;a&gt;Advanced Search&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs by Company&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs by Category&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs by Location&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs by Skills&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs by Designation&lt;/a&gt;&lt;/li&gt;
                          * &lt;/ul&gt;
                      * &lt;/li&gt;
                      * &lt;li id="Post Resume"&gt;&lt;a&gt;Post Resume&lt;/a&gt;&lt;/li&gt;
                      * &lt;li id="Job Seeker"&gt;&lt;a&gt;JobSeeker Login&lt;/a&gt;&lt;/li&gt;
                      * &lt;li id="Fast Forward"&gt;&lt;a&gt;Fast Forward&lt;/a&gt;
                         * &lt;ul&gt;
                            * &lt;li&gt;&lt;a&gt;Resume writing&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Certification&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Resume Spotlight&lt;/a&gt;&lt;/li&gt;
                            * &lt;li&gt;&lt;a&gt;Jobs4u&lt;/a&gt;&lt;/li&gt;
                         * &lt;/ul&gt;
                       * &lt;/li&gt;
                       * &lt;li id="More"&gt;&lt;a&gt;More&lt;/a&gt;
                          * &lt;ul&gt;
                             * &lt;li&gt;&lt;a&gt;Mobile&lt;/a&gt;&lt;/li&gt;
                             * &lt;li&gt;&lt;a&gt;Pay check&lt;/a&gt;&lt;/li&gt;
                             * &lt;li&gt;&lt;a&gt;Blog&lt;/a&gt;&lt;/li&gt;
                          * &lt;/ul&gt;
                        * &lt;/li&gt;
                    * &lt;/ul&gt;
   
       * <br> 
       * &lt;script&gt;
        * $("#menu").ejMenu({height:22}); 
		* //To enable Menu control
		* $("#menu").ejMenu("enable");
     * &lt;/script&gt;
		* @memberof ejMenu
		* @instance
        */
        //To enable menu
        enable: function () {
            this.model.enabled = true;
            var menuItemCollection = this.element.find('li.e-disable-item');
            $.each(menuItemCollection, function (key, value) {
                $(value).removeClass("e-disable-item").attr({ "aria-disabled": false });
            });
            this._disabledMenuItems.pop(this._disabledMenuItems);
        },

        show: function (locationX, locationY, targetElement, e) {
            /// <summary>This will set hide function of toolbar </summary>
            if (!this.model.enabled) return false;
            if (this.model.menuType == "contextmenu") 
                this._showContextMenu(locationX, locationY, targetElement, e);
              else
                this.element.css("display","block");
        },

        hide: function (e) {
            /// <summary>This will set hide function of toolbar </summary>
            if (!this.model.enabled) return false;
            if (this.model.menuType == "contextmenu")
                this._hideContextMenu(e);
            else
            {
                this._closeMenu();
                this.element.css("display", "none");
            }
        },
        
        //--------- ---------------------------------------------------//
        /**
        * Wiring the events to menu control		
        * @private
        */
        /* Register all the events at initialize*/
        _wireEvents: function (action) {
            this [action](this.element.find("li.e-list"), "mouseout", this._mouseOutHandler);
            this [action](this.element.find("li.e-list"), "mouseover", this._mouseOverHandler);
            this [action](this.element.find("li.e-list"), "click", this._onClickHandler);    
            this [action](this.element, "keydown", this._onKeyDownHandler);
            this [action](this.element, "focus", this._OnFocusHandler);
            this [action](this.element, "blur", this._OnFocusOutHandler);
            if (this.model.menuType == "contextmenu" && $(this.model.contextMenuTarget)[0] != null) {
                this._contextMenuEvents(action);
            }
            if (this.model.menuType != "contextmenu") {
                this [action]($(document), "click", this._onDocumentClick);
            }

        },

        //--------------------------------Handlers--------------------------

        /**
         * Section to track the mouse over event		
		 * @private
         */
        // handles the Mouseover event
        _mouseOverHandler: function (event) {
            var element,itemId="";
            this.element.find(".e-mhover").removeClass("e-mhover e-mfocused");
			event.currentTarget= $(event.target).closest("li")[0];
			if (!$(event.currentTarget).hasClass('e-disable-item'))
                $(event.currentTarget).addClass("e-mhover e-mfocused");
                if (event.stopPropagation)
                    event.stopPropagation();
                if (typeof (this._delaySubMenuHover) !== 'undefined') {
                    clearTimeout(this._delaySubMenuHover);
                }
                if (typeof (this._delaySubMenuHover) !== 'undefined') {
                    clearTimeout(this._delayMenuHover);
                }
                this._mouseOver = true;
                this._isMenuOpen = true;
                if ($(event.currentTarget.parentNode.parentNode).is(this.element)) {
                    this._isSubMenuOpen = false;
                }
                else {
                    this._isSubMenuOpen = true;
                }
                if (event.currentTarget.nodeName == "LI" && !$(event.currentTarget).hasClass('e-separator'))
                    element = event.currentTarget;
                else if (event.currentTarget.parentNode) {
                    if (event.currentTarget.parentNode.nodeName == "LI" && !$(event.currentTarget.parentNode).hasClass('e-separator'))
                        element = event.currentTarget.parentNode;
                    else
                        return false;
                }
                else {
                    event.preventDefault();
                    return false;
                }
                this._onMenuIntent(element, this, this._hoverOpen);
                if (!($.inArray(element, this._disabledMenuItems) > -1)) {
                    var menuText = $(element).children('a,span').text();
					itemId = !ej.isNullOrUndefined(element) ? $(element)[0].id:"";
                    var eventArgs = { "text": menuText, "element": element, "event": event ,"ID":itemId};

                    this._trigger("mouseover", $.extend({ events: eventArgs }, eventArgs));
                }
        },
        /**
        * Section to track the mouse out event		
        * @private
        */
        // handles the Mouseout event
        _mouseOutHandler: function (event) {
            var element,itemId = "";
                $(event.currentTarget).removeClass("e-mhover e-mfocused");
                if (event.stopPropagation)
                    event.stopPropagation();
                if (typeof (this._delaySubMenuHover) !== 'undefined') {
                    clearTimeout(this._delaySubMenuHover);
                }
                if (typeof (this._delaySubMenuHover) !== 'undefined') {
                    clearTimeout(this._delayMenuHover);
                }
                this._mouseOver = false;
                this._isMenuOpen = false;

                if (event.currentTarget.nodeName == "LI" && !$(event.currentTarget).hasClass('e-separator'))
                    element = event.currentTarget;
                else if (event.currentTarget.parentNode) {
                    if (event.currentTarget.parentNode.nodeName == "LI" && !$(event.currentTarget.parentNode).hasClass('e-separator'))
                        element = event.currentTarget.parentNode;
                    else
                        return false;
                }
                else {
                    event.preventDefault();
                    return false;
                }
                this._onHide(element, this, this._hoverClose);
                if (!($.inArray(element, this._disabledMenuItems) > -1)) {
                    var menuText = $(element).children('a,span').text();		
					itemId = !ej.isNullOrUndefined(element) ? $(element)[0].id:"";
                    var eventArgs = {  "text": menuText, "element": element, "event": event,"ID":itemId };
                   
                    this._trigger("mouseout", $.extend({ events: eventArgs }, eventArgs));
                }
        },
        /**
         * Section For handle click action.
		 * @private
         */
        _onClickHandler: function (event) {
            event.stopPropagation();
            var element,itemId="";
            this._isFocused = true;
            var openOnClickStart = false;
            if (!$(event.target).closest("li").hasClass('e-separator') && !$(event.target).closest("li").hasClass('e-disable-item') && $(event.target).closest("li").length > 0) {
                element = $(event.target).closest("li")[0];
            }
            else {
                event.preventDefault();
                return false;
            }
            if ($(event.target).is("a") && $(element).find(">a,>span").hasClass('aschild') && this.model.openOnClick) {
                this._isFocused = false;
            }
            if (!this._hoverOpen && $(element).find(">a,>span").hasClass('aschild') ) {
                this._show(element, this._showAnim, this._hideAnim);
                this._hoverOpen = false;
                openOnClickStart = true;
            }
            if (!($.inArray(element, this._disabledMenuItems) > -1)) {
                //Check if Context Menu, then hide the context menu firing the events
                if (this.model.menuType == "contextmenu") {
                    if (this._isContextMenuOpen) {
                        this._hideAnimation(this.element, this._hideAnim);
                        this._isContextMenuOpen = false;
                        
                        this._trigger("close", $.extend({ events: event }, event));
                    }
                }
                if (!openOnClickStart) {
                    //                    if (window.navigator.appName == "Microsoft Internet Explorer")
                    //                        this._mouseOutHandler(event, true);
                    //                    else
                    if (!$(element).find(">a,>span").hasClass("aschild")) {
                        this._closeMenu();
                        if (this.model.openOnClick)
                            this._hoverOpen = false;
                    }
                }
                var menuText = $(element).children('a,span').text();
				
				itemId = !ej.isNullOrUndefined(element) ? $(element)[0].id:"";
                var eventArgs = { "text": menuText, "element": element, "event": event, "selectedItem": this.selectedItem,"ID":itemId };            
                this._trigger("click", $.extend({ events: eventArgs }, eventArgs));
                this.selectedItem = null;
            }
        },

        /**
       * Section to watch the key down action.		
       * @private
       */
        _onKeyDownHandler: function (e) {
            var element,itemId="", hoverElement = this.element.find(".e-mhover"), focusedElement = this.element.find(".e-mfocused");
	     // tab key
            if (e.keyCode == 9) {
                this._isFocused = false;
                this._OnFocusOutHandler();
            }
            else if (e.keyCode == 37 || e.keyCode == 38 || e.keyCode == 39 || e.keyCode == 40)
                e.preventDefault();
            //key down
            if (e.keyCode == 40) {
                if (this.model.orientation != "horizontal" && this.element.find(".e-mhover").length > 0) {
                    focusedElement.removeClass("e-mfocused");
                    if (hoverElement.next().hasClass("e-separator"))
                        hoverElement.next().next().addClass("e-mfocused");
                    else if (hoverElement.is(':last-child')) {
                        hoverElement.siblings().first().addClass("e-mfocused");
                    }
                    else
                        hoverElement.next().addClass("e-mfocused");
                }
                else {
                    if (this.model.menuType == "contextmenu")
                        this.element.find("li").first().addClass("e-mfocused e-mhover");
                }
                if (this.model.orientation == "horizontal" && this.element.find(">li.e-mhover").children("ul").length > 0) {
                    this._show(hoverElement[0], this._showAnim, this._hideAnim);
                    hoverElement.removeClass("e-mhover").children("ul:first").find("li:first").addClass("e-mhover");
                }
                else {
                    if (hoverElement.is(':last-child')) {
                        hoverElement.removeClass("e-mhover").siblings().first().addClass("e-mhover");
                    }
                    else if (hoverElement.next().hasClass("e-separator"))
                        hoverElement.removeClass("e-mhover").next().next().addClass("e-mhover");
                    else {
                        hoverElement.removeClass("e-mhover").next().addClass("e-mhover");
                    }
                }
            }
            //key right
            if (e.keyCode == 39) {
                //for root level li
                if (this.model.orientation == "horizontal" && this.element.find(">li.e-list").hasClass("e-mhover")) {
				
                    if (hoverElement.is(":last-child")) {
					
                        hoverElement.removeClass("e-mhover e-mfocused");
                        if (this.element.find("li.e-list").first().hasClass("e-separator")) this.element.find("li.e-list").first().next().addClass("e-mhover e-mfocused");
                        else this.element.find("li.e-list").first().addClass("e-mhover e-mfocused");
                    }
                    else if (hoverElement.next().hasClass("e-separator")) {
					
                        hoverElement.removeClass("e-mhover e-mfocused").next().next().addClass("e-mhover e-mfocused");
                    }
                    else {
					
                        hoverElement.removeClass("e-mhover e-mfocused").next().addClass("e-mhover e-mfocused");
                    }
                }
                else if (this.element.find("li.e-mhover").children("ul").length > 0) {
                    this._show(hoverElement[0], this._showAnim, this._hideAnim);
                    hoverElement.removeClass("e-mhover").children("ul:first").find("li:first").addClass("e-mhover");
                }
                else if (hoverElement.children("ul").length <= 0) {
                    this.element.find(".e-mfocused >ul , .e-mfocused >ul li.e-list >ul").hide();
                    hoverElement.removeClass("e-mhover");
                    if (focusedElement.next().hasClass("e-separator")) {
                        focusedElement.removeClass("e-mfocused").next().next().addClass("e-mhover e-mfocused");

                    } else if (focusedElement.next().length <= 0) {
                        focusedElement.removeClass("e-mfocused").siblings().first().addClass("e-mhover e-mfocused");
                    } else focusedElement.removeClass("e-mfocused").next().addClass("e-mhover e-mfocused");
                }
            }
            //key up
            if (e.keyCode == 38) {
                if (this.model.orientation != "horizontal" && this.element.find(".e-mhover").length > 0) {
                    hoverElement.children("ul").hide();
                    focusedElement.removeClass("e-mfocused");
                    if (hoverElement.prev().hasClass("e-separator"))
                        hoverElement.prev().prev().addClass("e-mfocused");
                    else if (hoverElement.is(':first-child')) {
                        hoverElement.siblings().last().addClass("e-mfocused");
                    }
                    else
                        hoverElement.prev().addClass("e-mfocused");
                }
                else {
                    if (this.model.menuType == "contextmenu")
                    this.element.find("li").last().addClass("e-mfocused e-mhover");
                }
                if (hoverElement.is(':first-child')) {
                    hoverElement.removeClass("e-mhover e-mfocused").siblings().last().addClass("e-mhover");
                }
                else if (hoverElement.prev().hasClass("e-separator"))
                    hoverElement.removeClass("e-mhover").prev().prev().addClass("e-mhover");
                else
                    hoverElement.removeClass("e-mhover").prev().addClass("e-mhover");

            }
            //Key left
            if (e.keyCode == 37) {
                //for root level li
                if (this.model.orientation == "horizontal" && this.element.find(">li.e-list").hasClass("e-mhover")) {
                    if (hoverElement.is(":first-child")) {
                        hoverElement.removeClass("e-mhover e-mfocused");
                        if (this.element.find(">li.e-list").last().hasClass("e-separator"))
                            this.element.find(">li.e-list").last().prev().addClass("e-mhover e-mfocused");
                        else
                            this.element.find(">li.e-list").last().addClass("e-mhover e-mfocused");
                    }
                    else if (hoverElement.prev().hasClass("e-separator")) {
                        hoverElement.removeClass("e-mhover e-mfocused").prev().prev().addClass("e-mhover e-mfocused");
                    }
                    else {
                        hoverElement.removeClass("e-mhover e-mfocused").prev().addClass("e-mhover e-mfocused");
                    }
                }
                else if (this.element.find("li.e-mhover")) {
                    if (!hoverElement.parent().hasClass("e-context")) {
                    this._hideAnimation(hoverElement.parent(), this._hideAnim);
                    hoverElement.parent().closest("li.e-list").addClass("e-mhover");
                    hoverElement.removeClass("e-mhover");
                }
                }
                else {
                    this._hideAnimation(hoverElement, this._showAnim, this._hideAnim);
                    hoverElement.removeClass("e-mhover").parent().addClass("e-mhover");
                }
            }
            if (e.keyCode == 13) {
                        var menuText = $(hoverElement).children('a,span').text();
                        itemId = !ej.isNullOrUndefined($(hoverElement)[0]) ? $(hoverElement)[0].id : "";
                        var eventArgs = { "menuId": this.element[0].id, "menuText": menuText, "selectedItem": focusedElement, "itemId": itemId };
                if (this.model.menuType == "contextmenu") {
                    if (this._isContextMenuOpen && hoverElement.length > 0 && !focusedElement.hasClass("e-disable-item")) {
                        if (this.model.click)
                            this._trigger("click", $.extend({ events: eventArgs }, eventArgs));
                        this.selectedItem = null;
                        this._hideContextMenu(e);
                    }
                } else {
                    if (hoverElement.length > 0 && !hoverElement.hasClass("e-disable-item")) {
                        if ($(hoverElement).find(">a,>span").hasClass('aschild')) {
                            this._show(hoverElement[0], this._showAnim, this._hideAnim);
                            hoverElement.removeClass("e-mhover").children("ul:first").find("li:first").addClass("e-mhover");
                        }
                        else {
                this.element.find(".e-mhover >a,.e-mhover >span ").focus();
                this.element.find("li.e-list").removeClass("e-mhover e-mfocused");
                this._closeAll();
            }
                        if (ej.isNullOrUndefined($(hoverElement).find(">a").attr("href")))
                            this._trigger("click", $.extend({ events: eventArgs }, eventArgs));
            }
            }
            }
            if (e.keyCode == 27) {
                if (this.model.menuType == "contextmenu")
                    this._hideContextMenu(e);
                else
                this._closeAll();
            }
            if (e.target.nodeName == "DIV" && $(e.target).children().is(this.element))
                element = $(e.target).children()[0];
            else if (e.target.nodeName == "LI" && !$(e.target).hasClass('e-separator'))
                element = e.target;
            else if (e.target.parentNode) {
                if (e.target.parentNode.nodeName == "LI" && !$(e.target.parentNode).hasClass('e-separator'))
                    element = e.target.parentNode;
            }
            else
                return false;
            if (!($.inArray(element, this._disabledMenuItems) > -1)) {
                var menuText = $(element).children('a,span').text();
				
			   itemId = !ej.isNullOrUndefined(element) ? $(element)[0].id:"";
                var eventArgs = { "text": menuText, "element": element, "event": e,"ID":itemId };
                
                this._trigger("keydown", $.extend({ events: eventArgs }, eventArgs));
            }
            this._isFocused = false;
        },
        /**
         * Section For handle Focus action.
		 * @private
         */
        _OnFocusHandler: function (event) {
            if (!this.element.find(">li:first").hasClass("e-disable-item") && this._isFocused && this.element.find(".e-mhover").length == 0) {
                this.element.find(">li:first").addClass("e-mhover e-mfocused");
            }
            else this._isFocused = true;
        },
        /**
         * Section For handle FocusOut action.
		 * @private
         */
        _OnFocusOutHandler: function () {
            if (!this._isFocused) {
                this.element.find("li.e-list").removeClass("e-mhover e-mfocused");
                this._closeAll();
            }
            this._isFocused = true;
        },
        /**
        * Section For handle Click action on Document 
        * @private
        */
        _onDocumentClick: function (event) {
            if (this.model.openOnClick)
                this._hoverOpen = false;
            if (!$(event.target).parents("ul.e-menu").is(this.element)) {
                this._isFocused = false;
                this._OnFocusOutHandler();
            }
        },
        /**
        * Section For handle the context menu event 
        * @private
        */
        //handles the context menu event
        _ContextMenuHandler: function (e) {
            var isRightClick = false;
            if (e.type == "taphold")
                isRightClick = true;
            else if (e.button)
                isRightClick = (e.button == 2);
            else if (e.which)
                isRightClick = (e.which == 3); //for Opera
            var targetElement = e.target;
            if (isRightClick) {
                var evt = e;
                if (e.type == "taphold") {
                    if (e.options.type == "touchstart") evt = e.options.touches[0];
                    else evt = e.options;
                }
                var locationX = (evt.clientX + this.element.width() < $(window).width()) ? evt.pageX : evt.pageX - this.element.width();
                var locationY = (evt.clientY + this.element.height() < $(window).height()) ? evt.pageY : evt.pageY - this.element.height();
                var bodyPos = $("body").css("position") != "static" ? $("body").offset() : { left: 0, top: 0 };
                locationX -= bodyPos.left, locationY -= bodyPos.top;
                var showSpeed = this._showSpeed;
                this._showContextMenu(locationX, locationY, targetElement, e);
            }
            else {
                if (this._isContextMenuOpen) {
                    var hideanim = this._hideAnim;
                    var hideSpeed = this._hideSpeed;
                    this._hideContextMenu(hideanim, hideSpeed, e);
                }
            }
        },
        /**
         * Section For handles preventing the default (browser's) context menu
		 * @private
         */
        //handles preventing the default (browser's) context menu
        _onDefaultPreventer: function (e) {
            e.preventDefault();
            e.stopPropagation();
            return false;
        },
        /**
         * Section For handles context close on document click
		 * @private
         */
        //handles context close on document click
        _onContextClose: function (e) {
            var proxy = this;
            if (this._isContextMenuOpen) {
                var isRightClick = false;
                if (e.button)
                    isRightClick = (e.button == 2);
                else if (e.which)
                    isRightClick = (e.which == 3); //for Opera

                if (!isRightClick) {
					var hideanim = this._hideAnim;
					var hideSpeed = this._hideSpeed;
					this._hideContextMenu(hideanim, hideSpeed, e);                    
                    var parentElements = $(e.target).parents();
                    $.each(parentElements, function (index, value) {
                        if (value.id == proxy._ContextTargetId) {
                            return;
                        }
                    });

                }
            }
        }

    });

    /**
	 * Enum for Menu type	 
	 * @enum {string}
	 * @global 
	 */
    ej.MenuType = {
        /**  support for list of items appears as normal menu in horizontal or vertical direction. */
        NormalMenu: "normalmenu",
        /**  support for list of items appears as menu when right clicked on target area, thereby preventing browser’s default right click.. */
        ContextMenu: "contextmenu"
    };
    /**
	 * Enum for Sub Menu Popup direction	 
	 * @enum {string}
	 * @global 
	 */
    ej.Direction = {
        /**  support for Render sub menu popup in left direction. */
        Left: "left",
        /**  support for Render sub menu popup in Right direction. */
        Right: "right"
    };
    /**
	 * Enum for animationType	 
	 * @enum {string}
	 * @global 
	 */
    ej.AnimationType = {
        /**  support for disable the AnimationType while hover or click an menu items. */
        None: "none",
        /**  support for enable the AnimationType while hover or click an menu items. */
        Default: "default"
    };

    //mentions the recursive type of child
    //ej.Menu.prototype.defaults.fields.child = ej.Menu.prototype.defaults.fields;
    //ej.Menu.prototype.dataTypes.fields.child = ej.Menu.prototype.dataTypes.fields;

})(jQuery, Syncfusion);;
(function ($, ej, undefined) {

    ej.widget("ejPager", "ej.Pager", {
        _rootCSS: "e-pager",
        validTags: ["div"],
        // default model
        defaults: {
            pageSize: 12,
            pageCount: 10,
            currentPage: 1,
            enableExternalMessage: false,
            externalMessage:"",
            enableQueryString:false,
            locale: "en-US",
            masterObject: null,
            click: null,
            enableRTL: false,
            totalRecordsCount: null,
            totalPages: null
        },

        // constructor function
        _init: function () {
            this._initPrivateProperties();
            this.model.enableQueryString && this._queryStringValue();
            this.renderPager();
            this._wireEvents();
        },
        _initPrivateProperties: function () {
            this._links = [];
            this._$prev = null;
            this._$first = null;
            this._$PP = null;
            this._$NP = null;
            this._lastNP = false;
            this._lastpageCount = null;
            this._$last = null;
            this._$next = null;
            this._prevPageNo = null;
        },
        _wireEvents: function () {
            if (this.model.click != undefined)
                this._on(this.element, "click", this._pagerClickHandler);

        },
        renderPager: function () {
            var $pagerContainer = ej.buildTag('div.e-pagercontainer', "", {}, { unselectable: "on" });
            var $parentMsgBar = ej.buildTag('div.e-parentmsgbar');
            this._renderPagerContainer($pagerContainer);
            this._renderMsgBar($parentMsgBar);
            this.element.append($pagerContainer).append($parentMsgBar).addClass(this.model.enableRTL ? "e-pager e-rtl" : "e-pager");
            this.model.enableExternalMessage && this._renderPagerMessage();
        },
        _queryStringValue: function () {
            var results = new RegExp('[\\?&]page=([^&#]*)').exec(window.location.href);
            !results ? this.model.currentPage = 1 : this.model.currentPage = parseInt(results[1] || 1);
        },
        _renderPagerMessage: function () {
            var $messageDiv = ej.buildTag('div.e-pagermessage');
            if (this.model.externalMessage.toString().length)
                $messageDiv.html(this.model.externalMessage);
            else
                $messageDiv.hide();
            this.element.append($messageDiv);
        },
        _renderPagerContainer: function ($pagerContainer) {
            //Update pager styles here for next versions
            this._renderBackwardButton($pagerContainer);
            this._renderpreviousPager($pagerContainer);
            this._renderNumericItem($pagerContainer);
            this._renderForwardPager($pagerContainer);
            this._renderForwardButton($pagerContainer);
        },
        _renderMsgBar: function ($ParentMsgBar) {
            var $msgBar = ej.buildTag('span', String.format(this._getLocalizedLabels("pagerInfo"), this.model.currentPage, this.model.totalPages || 0, this.model.totalRecordsCount || 0));
            $ParentMsgBar.append($msgBar);
            $ParentMsgBar.css("text-align", ej.TextAlign.Right);
        },
        _renderpreviousPager: function ($pagerContainer) {
            this._$PP = ej.buildTag('a.e-link e-nextprevitemdisabled e-disable e-spacing e-PP', "...", {}, { title: this._getLocalizedLabels("previousPagerTooltip") , role:"link" });
            $pagerContainer.append(this._$PP);
        },
        _renderForwardPager: function ($pagerContainer) {
            this._$NP = ej.buildTag('a.e-link e-NP e-numericitem e-spacing e-default', "...", {}, { title: this._getLocalizedLabels("nextPagerTooltip"), role: "link" });
            $pagerContainer.append(this._$NP);
        },
        _renderBackwardButton: function ($pagerContainer) {
            this._$first = ej.buildTag('div.e-first e-icon e-mediaback  e-firstpagedisabled e-disable', "", {}, { unselectable: "on", title: this._getLocalizedLabels("firstPageTooltip") });
            this._$prev = ej.buildTag('div.e-prev e-icon e-arrowheadleft-2x  e-prevpagedisabled e-disable', "", {}, { unselectable: "on", title: this._getLocalizedLabels("previousPageTooltip") });
            $pagerContainer.append(this._$first);
            $pagerContainer.append(this._$prev);
        },
        _renderNumericItem: function ($pagerContainer) {
            var $numericContainer = ej.buildTag('div.e-numericcontainer e-default', "", {}, { unselectable: "on", id: "NumericContainer" });
            this._renderNumericLinks($numericContainer,this.model.pageCount);
            $pagerContainer.append($numericContainer);
        },
        _renderNumericLinks: function ($numericContainer) {
            $numericContainer.empty();
            for (var page = 1; page <= this.model.pageCount; page++) {
                var $link = ej.buildTag('a.e-link', page, {}, { role: "link" }).addClass("e-numericitem e-spacing e-default");
                if (page == this.model.currentPage)
                    $link.removeClass("e-default").addClass("e-currentitem e-active");
                $numericContainer.append($link);
            }
            this._links = $numericContainer.children();
        },
        _renderForwardButton: function ($pagerContainer) {
            this._$next = ej.buildTag('div.e-nextpage e-icon e-arrowheadright-2x  e-default', "", {}, { unselectable: "on", title: this._getLocalizedLabels("nextPageTooltip") });
            this._$last = ej.buildTag('div.e-lastpage e-icon e-mediaforward  e-default', "", {}, { unselectable: "on", title: this._getLocalizedLabels("lastPageTooltip") });
            $pagerContainer.append(this._$next);
            $pagerContainer.append(this._$last);

        },
        _applyCss: function () {
            if (this.model.currentPage > 1) {
                this._$prev.removeClass("e-prevpagedisabled").removeClass("e-disable").addClass("e-prevpage e-default");
                this._$first.removeClass("e-firstpagedisabled").removeClass("e-disable").addClass("e-firstpage e-default");
            } else {
                this._$prev.addClass("e-prevpagedisabled e-disable").removeClass("e-prevpage").removeClass("e-default");
                this._$first.addClass("e-firstpagedisabled e-disable").removeClass("e-firstpage").removeClass("e-default");
            }
            (this.model.currentPage > this.model.pageCount) ?
                 (this._$PP.removeClass("e-nextprevitemdisabled").removeClass("e-disable").addClass("e-numericitem e-default")) : (this._$PP.addClass("e-nextprevitemdisabled e-disable").removeClass("e-numericitem").removeClass("e-default"));

            if (parseInt(this._links[0].innerHTML, 10) + this.model.pageCount > this.model.totalPages)
                this._lastNP = true;
            else
                this._lastNP = false;

            if (this._lastNP == false)
                this._$NP.removeClass("e-nextprevitemdisabled").removeClass("e-disable").addClass("e-numericitem e-default");
            else
                this._$NP.addClass("e-nextprevitemdisabled e-disable").removeClass("e-numericitem").removeClass("e-default");

            this._lastpageCount = this.model.totalPages % this.model.pageCount;
            if (this._lastpageCount == 0)
                this._lastpageCount = this.model.pageCount;
            if (this.model.currentPage > (this.model.totalPages - this._LastpageCount)) {
                this._$PP.removeClass("e-nextprevitemdisabled").removeClass("e-disable").addClass("e-numericitem e-default");
                this._$NP.addClass("e-nextprevitemdisabled e-disable").removeClass("e-numericitem").removeClass("e-default");
            }
            if (this.model.currentPage == this.model.totalPages || this.model.totalRecordsCount == 0) {
                this._$last.addClass("e-lastpagedisabled e-disable").removeClass("e-lastpage").removeClass("e-default");
                this._$next.addClass("e-nextpagedisabled e-disable").removeClass("e-nextpage").removeClass("e-default");
                this._$NP.addClass("e-nextprevitemdisabled e-disable").removeClass("e-numericitem").removeClass("e-default");
            } else {
                this._$last.addClass("e-lastpage e-default").removeClass("e-lastpagedisabled").removeClass("e-disable");
                this._$next.addClass("e-nextpage e-default").removeClass("e-nextpagedisabled").removeClass("e-disable");
            }
            if (this._links.length) {
                this._links.removeClass("e-currentitem").removeClass("e-active").addClass("e-default");
                $(this._links[(this.model.currentPage - 1) % this.model.pageCount]).removeClass("e-default").addClass("e-currentitem e-active");
                $(this._links[(this._prevPageNo - 1) % this.model.pageCount]).removeClass("e-default").addClass("e-numericitem e-active");
            }
            if (this.model.pageSize >= (this.model.totalRecordsCount / this.model.pageCount) && this._$PP != null && this._$PP.length != 0) {
                this._$PP.addClass("e-nextprevitemdisabled e-disable").removeClass("e-numericitem").removeClass("e-default");
                this._$NP.addClass("e-nextprevitemdisabled e-disable").removeClass("e-numericitem").removeClass("e-default");
            }
        },
        _SetTotalPages: function () {
            this.model.totalPages = (this.model.totalRecordsCount % this.model.pageSize == 0) ? (this.model.totalRecordsCount / this.model.pageSize) : (parseInt(this.model.totalRecordsCount / this.model.pageSize, 10) + 1);
        },
        _refreshNumericItem: function () {
            if (this._links.length != 0 && this._links != null) {
                this.model.currentPage = this.model.totalPages == 1 ? 1 : this.model.currentPage;
                if (this.model.currentPage > this.model.totalPages && this.model.totalPages != 0)
                    this.model.currentPage = this.model.totalPages;
                var _pagerTarget = parseInt(this.model.currentPage / this.model.pageCount, 10);
                if (this.model.currentPage % this.model.pageCount == 0)
                    if (_pagerTarget > 0)
                        _pagerTarget = _pagerTarget - 1;
                this._links.css("display", "none");
                for (var i = 0; i < this.model.pageCount; i++) {
                    var start = (_pagerTarget * this.model.pageCount) + 1 + i;
                    if (start <= this.model.totalPages) {
                        this._links[i].style.display = '';
                        this._links[i].innerHTML = start;
                    }
                }
            }
        },
        _refreshPagerInfo: function () {
            if (this.model.totalRecordsCount == 0)
                this.model.currentPage = 0;
            this.element.find("span").text(String.format(this._getLocalizedLabels("pagerInfo"), this.model.currentPage, this.model.totalPages || 0, this.model.totalRecordsCount || 0));
        },
        _refreshExternalMessage: function () {
            if (this.model.externalMessage.toString().length)
                this.element.find(".e-pagermessage").empty().html(this.model.externalMessage).show();
            else
                this.element.find(".e-pagermessage").hide();
        },
        refreshPager: function () {
            this._SetTotalPages();
            this._refreshNumericItem();
            this._refreshPagerInfo();
            this._applyCss();
            this.model.enableExternalMessage && this._refreshExternalMessage();
            if (this.model.enableRTL)
                this.element.addClass("e-rtl");
            else
                this.element.removeClass("e-rtl");
        },

        _kDownHandler: function (e) {
            var code;
            if (e.keyCode) code = e.keyCode; // ie and mozilla/gecko
            else if (e.which) code = e.which; // ns4 and opera
            else code = e.charCode;
            e.target = null;
            if (this.model.masterObject.checkKey("firstPage", code, e))
                e.target = this._$first;
            else if (this.model.masterObject.checkKey("previousPager", code, e))
                e.target = this._$PP;
            else if (this.model.masterObject.checkKey("previousPage", code, e))
                e.target = this._$prev;
            else if (this.model.masterObject.checkKey("lastPage", code, e))
                e.target = this._$last;
            else if (this.model.masterObject.checkKey("nextPager", code, e))
                e.target = this._$NP;
            else if (this.model.masterObject.checkKey("nextPage", code, e))
                e.target = this._$next;
            else
                return false;
            this._pagerClickHandler(e);
        },

        _pagerClickHandler: function (e) {
            this._prevPageNo = this.model.currentPage;
            var $target = $(e.target);
            if ($.inArray(e.target, this._links) != -1) {
                this.model.currentPage = parseInt(e.target.innerHTML, 10);

            }
            else if ($target.hasClass("e-nextpage") && $target.hasClass("e-nextpagedisabled") != true) {
                if (this.model.currentPage % this.model.pageCount == 0) {
                    this.model.currentPage++;
                    if (this._links != undefined && this._links.length != 0)
                        this.model.currentPage = parseInt(this._links[0].innerHTML, 10) + this.model.pageCount;
                    if (parseInt(this._links[this.model.pageCount - 1].innerHTML, 10) + this.model.pageCount >= this.model.totalPages)
                        this._lastNP = true;
                }
                else
                    this.model.currentPage++;
            }
            else if ($target.hasClass("e-prevpage") && $target.hasClass("e-prevpagedisabled") != true) {
                if (this.model.currentPage % this.model.pageCount == 1)
                    this._lastNP = false;
                this.model.currentPage--;
                if (this.model.currentPage < 0) { this.model.currentPage = 0; }
            }
            else if ($target.hasClass("e-lastpage") && $target.hasClass("e-lastpagedisabled") != true) {
                this._LastpageCount = this.model.totalPages % this.model.pageCount;
                (this._LastpageCount == 0) ? (this._LastpageCount = this.model.pageCount) : null;
                this.model.currentPage = this.model.totalPages;
                this._lastNP = true;
            }
            else if ($target.hasClass("e-firstpage") && $target.hasClass("e-firstpagedisabled") != true) {
                this.model.currentPage = 1;
                this._lastNP = false;
            }
            else if ($target.hasClass("e-NP") && $target.hasClass("e-nextprevitemdisabled") != true) {
                if (this._links != undefined)
                    this.model.currentPage = parseInt(this._links[0].innerHTML, 10) + this.model.pageCount;
                if (parseInt(this._links[this.model.pageCount - 1].innerHTML, 10) + this.model.pageCount >= this.model.totalPages) {
                    this._lastNP = true;
                    if ((this.model.totalRecordsCount - this.model.pageSize) < this.model.pageSize)
                        this._LastpageCount = this.model.totalRecordsCount - this.model.pageSize;
                    else
                        this._LastpageCount = ((this.model.totalRecordsCount / this.model.pageSize) % this.model.pageCount);
                    (this._LastpageCount == 0) ? (this._LastpageCount = this.model.pageCount) : null;
                    if (this._links != undefined)
                        this.model.currentPage = parseInt(this._links[this.model.pageCount - 1].innerHTML, 10) + 1;
                }
            }
            else if ($target.hasClass("e-PP") && $target.hasClass("e-nextprevitemdisabled") != true) {
                if (this._links != undefined)
                    this.model.currentPage = parseInt(this._links[0].innerHTML, 10) - this.model.pageCount;
                this._lastNP = false;
            }
            this.goToPage(this.model.currentPage);
            return false;
        },
        goToPage: function (pageIndex) {
            if (pageIndex != this.model.currentPage)
                this._prevPageNo = this.model.currentPage;
            if (this._prevPageNo !== pageIndex && (pageIndex >= 1 && pageIndex <= this.model.totalPages)) {
                this.model.currentPage = pageIndex;
                this.model.enableQueryString && this._updateQueryString(this.model.currentPage);
                this._trigger("click", { "currentPage": pageIndex });
            }
        },
        _updateQueryString: function (value) {
            var _newUrl = this._getUpdatedURL(window.location.href, "page", value);
            if (history.pushState) {
                window.history.pushState({ path: _newUrl }, '', _newUrl);
            }
            else
                window.location.href = _newUrl;
        },

        _getUpdatedURL: function (uri, key, value) {
            var re = new RegExp("([?|&])" + key + "=.*?(&|#|$)", "i");
            if (uri.match(re)) {
                return uri.replace(re, '$1' + key + "=" + value + '$2');
            } else {
                var hash = '';
                var separator = uri.indexOf('?') !== -1 ? "&" : "?";
                if (uri.indexOf('#') !== -1) {
                    hash = uri.replace(/.*#/, '#');
                    uri = uri.replace(/#.*/, '');
                }
                return uri + separator + key + "=" + value + hash;
            }
        },
        _getLocalizedLabels: function (property) {
            return (ej.Pager.locale[this.model.locale] === undefined || ej.Pager.locale[this.model.locale][property] === undefined) ? ej.Pager.locale["en-US"][property] : ej.Pager.locale[this.model.locale][property];
        },
        _setFirst: true,
        _setModel: function (options) {
            for (var prop in options) {
                switch (prop) {
                    case "pageCount":
                        this._renderNumericLinks(this.element.find("#NumericContainer"));
                        break;
                }
            }
            this.refreshPager();
        },

        // all events bound using this._on will be unbind automatically
        _destroy: function () {
            this.element.empty().removeClass("e-pager");
        }
    });
    ej.Pager.locale = {};
    ej.Pager.locale["en-US"] = {
        pagerInfo: "{0} of {1} pages ({2} items)",
        firstPageTooltip: "Go to first page",
        lastPageTooltip: "Go to last page",
        nextPageTooltip: "Go to next page",
        previousPageTooltip: "Go to previous page",
        nextPagerTooltip: "Go to next Pager",
        previousPagerTooltip: "Go to previous Pager"
    };

})(jQuery, Syncfusion);;
/**
* @fileOverview Plugin to style the Html Button elements
* @copyright Copyright Syncfusion Inc. 2001 - 2013. All rights reserved.
*  Use of this code is subject to the terms of our license.
*  A copy of the current license can be obtained at any time by e-mailing
*  licensing@syncfusion.com. Any infringement will be prosecuted under
*  applicable laws. 
* @version 12.1 
* @author <a href="mailto:licensing@syncfusion.com">Syncfusion Inc</a>
*/
(function ($, ej, undefined) {
    // ejSlider is the plugin name 
    // "ej.Slider" is "namespace.className" will hold functions and properties

    /**
    * @namespace ej
	* @class ejSlider
	* @requires jQuery
    * @requires jquery.easing.1.3.js
    * @requires ej.core.js
    * @requires ej.slider.js
	* @classdesc Custom Design for Html Slider control.
	* @example 
	* &lt;div id="slider"&gt; &lt;/div&gt; <br> 
	* &lt;script&gt; <br>
	* // Create Slider <br>
    * $('#slider').ejSlider(); <br>
	* &lt;/script&gt; <br>
	*/

    ej.widget("ejSlider", "ej.Slider", {
        // widget element will be automatically set in this
        element: null,

        // user defined model will be automatically set in this
        model: null,
        validTags: ["div", "span"],
        _rootCSS: "e-slider",
        _setFirst: false,

        // default model
        defaults: {
            /**
                     * Specifies the orientation of the slider.See {@link orientation}
                     * @default ej.orientation.Horizontal
                     * @type {enum}
                     * @example
                     * &lt;div id="slider"&gt; &lt;/div&gt; <br>
                     * &lt;script&gt;
                     * //To set orientation API value during initialization
                     * $("#slider").ejSlider({ orientation: ej.Orientation.Vertical});
                     * &lt;/script&gt;
                     * @memberof ejSlider
                     * @instance
                     */
            orientation: "horizontal",
            /**		
			* Specifies the animation behaviour of the slider.	
			* @default true
			* @type {boolean}
			* @example 
            * &lt;div id="slider"&gt; &lt;/div&gt; <br>
            * &lt;script&gt;
			* //To set enableAnimation API value during initialization  
			* 	$("#slider").ejSlider({ enableAnimation: true});
            * &lt;/script&gt;
			* @memberof ejSlider
			* @instance
			*/
            enableAnimation: true,
            /**
			* Specifies the animationSpeed of the slider.
			* @default 500
			* @type {number}
			* @example 
            * &lt;div id="slider"&gt; &lt;/div&gt; <br>
            * &lt;script&gt;
			* //To set animationSpeed API value during initialization  
			* $("#slider").ejSlider({ animationSpeed: 500});
            * &lt;/script&gt;
			* @memberof ejSlider
			* @instance
			*/
            animationSpeed: 500,
            /**
			* Specifies the showTooltip to shows the current Slider value, while moving the Slider handle of the slider.	
			* @default true
			* @type {boolean}
			* @example 
            *&lt;/br&gt;
			*&lt;/br&gt;
            * &lt;div id="slider"&gt; &lt;/div&gt; <br>
            * &lt;script&gt;
			* //To set showTooltip API value during initialization  
			* 	$("#slider").ejSlider({ showTooltip: true});
            * &lt;/script&gt;
			* @memberof ejSlider
			* @instance
			*/
            showTooltip: true,
            /**		
			* Specify the CSS class to slider to achieve custom theme.
			* @default ""
			* @type {string}
			* @example 
            * &lt;div id="slider"&gt; &lt;/div&gt; <br>
            * &lt;script&gt;
			* //To set cssClass API value during initialization  
			* 	$("#slider").ejSlider({ cssClass: "gradient-lime"});
            * &lt;/script&gt;
			* @memberof ejSlider
			* @instance
			*/
            cssClass: "",
            /**		
			* Specifies the rounded corner behaviour for slider.
			* @default false
			* @type {boolean}
			* @example
            * &lt;div id="slider"&gt; &lt;/div&gt; <br>
            * &lt;script&gt;
			* //To set showRoundedCorner API value during initialization  
			* 	$("#slider").ejSlider({ showRoundedCorner: true});
            * &lt;/script&gt;
			* @memberof ejSlider
			* @instance
			*/
            showRoundedCorner: false,
            /**
			* Specifies the readOnly of the slider.
			* @default false
			* @type {boolean}
			* @example
            * &lt;div id="slider"&gt; &lt;/div&gt; <br>
            * &lt;script&gt;
			* //To set readOnly API value during initialization  
			* 	$("#slider").ejSlider({ readOnly: true});
            * &lt;/script&gt;
			* @memberof ejSlider
			* @instance
			*/
            readOnly: false,
            /**
			* Specifies the Right to Left Direction of the slider.
			* @default false
			* @type {boolean}
			* @example 
          *&lt;/br&gt;
			*&lt;/br&gt;
            * &lt;div id="slider"&gt; &lt;/div&gt; <br>
            * &lt;script&gt;
			* //To set enableRTL API value during initialization  
			* 	$("#slider").ejSlider({ enableRTL: false});
            * &lt;/script&gt;
			* @memberof ejSlider
			* @instance
			*/
            enableRTL: false,
            /**
			* Specifies the starting value of the slider.
			* @default 0
			* @type {number}
			* @example 
            *&lt;/br&gt;
			*&lt;/br&gt;
            * &lt;div id="slider"&gt; &lt;/div&gt; <br>
            * &lt;script&gt;
			* //To set minValue API value during initialization  
			* 	$("#slider").ejSlider({ minValue: 0});
            * &lt;/script&gt;
			* @memberof ejSlider
			* @instance
			*/
            minValue: 0,
            /**
			* Specifies the ending value of the slider.	
			* @default 100
			* @type {number}
			* @example
           *&lt;/br&gt;
			*&lt;/br&gt;
            * &lt;div id="slider"&gt; &lt;/div&gt; <br>
            * &lt;script&gt;
			* //To set maxValue API value during initialization  
			* 	$("#slider").ejSlider({ maxValue: 60});
            * &lt;/script&gt;
			* @memberof ejSlider
			* @instance
			*/
            maxValue: 100,
            /**
			* Specifies the sliderType of the slider.
			* @default ej.SliderType.Default
			* @type {enum}
			* @example 
            * &lt;div id="slider"&gt; &lt;/div&gt; <br>
            * &lt;script&gt;
			* //To set sliderType API value during initialization
			* $("#slider").ejSlider({ sliderType: ej.SliderType.Default});
            * &lt;/script&gt;
			* @memberof ejSlider
			* @instance
			*/
            sliderType: "default",
            /**
			* Specifies the value of the slider.
			* @default 0
			* @type {number}
			* @example
           *&lt;/br&gt;
			*&lt;/br&gt;
            * &lt;div id="slider"&gt; &lt;/div&gt; <br>
            * &lt;script&gt;
			* //To set value API  during initialization
			* 	$("#slider").ejSlider({ value: 60});
            * &lt;/script&gt;
			* @memberof ejSlider
			* @instance
			*/
            value: null,
            /**
			* Specifies the values of the range slider.
			* @default [minValue,maxValue]
			* @type {array}
			* @example
            *&lt;/br&gt;
			*&lt;/br&gt;
            * &lt;div id="slider"&gt; &lt;/div&gt; <br>
            * &lt;script&gt;
			* //To set values API during initialization
			* 	$("#slider").ejSlider({ values: [30,60]});
            * &lt;/script&gt;
			* @memberof ejSlider
			* @instance
			*/
            values: null,
            /**
			* Specifies the incremental step value of the slider.
			* @default 1
			* @type {number}
			* @example
          *&lt;/br&gt;
			*&lt;/br&gt;
            * &lt;div id="slider"&gt; &lt;/div&gt; <br>
            * &lt;script&gt;
			* //To set incrementStep API value during initialization  
			* 	$("#slider").ejSlider({ incrementStep: 2});
            * &lt;/script&gt;
			* @memberof ejSlider
			* @instance
			*/
            incrementStep: 1,
            /**
			* Specifies the height of the slider.
			* @default 14
			* @type {String}
			* @example
            * &lt;div id="slider"&gt; &lt;/div&gt; <br>
            * &lt;script&gt;
			* //To set height API value during initialization  
			* 	$("#slider").ejSlider({ height: 14});
            * &lt;/script&gt;
			* @memberof ejSlider
			* @instance
			*/
            height: null,
            /**
			* Specifies the width of the slider.
			* @default 100%
			* @type {String}
			* @example 
            * &lt;div id="slider"&gt; &lt;/div&gt; <br>
            * &lt;script&gt;
			* //To set width API value during initialization
			* 	$("#slider").ejSlider({ width: "300px"});
            * &lt;/script&gt;
			* @memberof ejSlider
			* @instance
			*/
            width: null,
            /**		
			* Specifies the state of the slider.
			* @default true
			* @type {boolean}
			* @example
            * &lt;div id="slider"&gt; &lt;/div&gt; <br>
            * &lt;script&gt;
			* //To set enabled API value during initialization  
			* 	$("#slider").ejSlider({ enabled: false});
            * &lt;/script&gt;
			* @memberof ejSlider
			* @instance
			*/
            enabled: true,
            /**
			* Specifies the major (large) and minor (small) ticks of the slider.	
			* @default false
			* @type {boolean}
			* @example
          *&lt;/br&gt;
			*&lt;/br&gt;
            * &lt;div id="slider"&gt; &lt;/div&gt; <br>
            * &lt;script&gt;
			* //To set enabled API value during initialization
			* 	$("#slider").ejSlider({ showScale: false});
            * &lt;/script&gt;
			* @memberof ejSlider
			* @instance
			*/
            showScale: false,
            /**		
			* Specifies the distance between two major (large) ticks from the scale of the slider.	
			* @default 10
			* @type {number}
			* @example
            *&lt;/br&gt;
			*&lt;/br&gt;
            * &lt;div id="slider"&gt; &lt;/div&gt; <br>
            * &lt;script&gt;
			* //To set largeStep API value during initialization  
			* 	$("#slider").ejSlider({ largeStep: 2});
            * &lt;/script&gt;
			* @memberof ejSlider
			* @instance
			*/
            largeStep: 10,
            /**
			* Specifies the distance between two minor (small) ticks from the scale of the slider.	
			* @default 10
			* @type {number}
			* @example
            *&lt;/br&gt;
			*&lt;/br&gt;
            * &lt;div id="slider"&gt; &lt;/div&gt; <br>
            * &lt;script&gt;
			* //To set smallStep API value during initialization  
			* 	$("#slider").ejSlider({ smallStep: 2});
            * &lt;/script&gt;
			* @memberof ejSlider
			* @instance
			*/
            smallStep: 1,
            /**
			* Specifies the showSmallTicks of the slider.
			* @default true
			* @type {boolean}
			* @example
            *&lt;/br&gt;
			*&lt;/br&gt;
            * &lt;div id="slider"&gt; &lt;/div&gt; <br>
            * &lt;script&gt;
			* //To set showSmallTicks API value during initialization  
			* 	$("#slider").ejSlider({ showSmallTicks: false});
            * &lt;/script&gt; 
			* @memberof ejSlider
			* @instance
			*/
            showSmallTicks: true,
            /**
			* Specify the enablePersistence to slider to save current model value to browser cookies for state maintains	
			* @default false
			* @type {boolean}
			* @example
            * &lt;div id="slider"&gt; &lt;/div&gt; <br>
            * &lt;script&gt;
			* //To set enablePersistence API value during initialization  
			* 	$("#slider").ejSlider({ enablePersistence: false});
            * &lt;/script&gt;
			* @memberof ejSlider
			* @instance
			*/
            enablePersistence: false,

            // Events
            /**     
			 * Fires when Slider control is started successfully.
			 * @event
			 * @name ejSlider#start 	
			 * @param {Object} argument Event parameters from slider control     
			 * @param {boolean}  argument.cancel if the event should be cancelled; otherwise, false.
			 * @param {number}  argument.sliderIndex returns current handle number or index	
			 * @param {string}  argument.id returns slider id	
			 * @param {object}  argument.model returns the slider model	
			 * @param {string}  argument.type returns the name of the event				 
			 * @param {number}  argument.value returns the slider value			 			
			 * @example 
             * &lt;div id="slider"&gt; &lt;/div&gt; <br>
             * &lt;script&gt;
			 * //start event for slider control
             * $("#slider").ejSlider({
             *    start: function (args) {}
             * });
             * &lt;/script&gt;
			 * @memberof ejSlider
			 * @instance
			 */
            start: null,
            /**     
			 * Fires when Slider control is stopped successfully.
			 * @event
			 * @name ejSlider#stop 	
			 * @param {Object} argument Event parameters from slider control     
			 * @param {boolean}  argument.cancel if the event should be cancelled; otherwise, false.
			 * @param {number}  argument.sliderIndex returns current handle number or index	
			 * @param {string}  argument.id returns slider id	
			 * @param {object}  argument.model returns the slider model	
			 * @param {string}  argument.type returns the name of the event				 
			 * @param {number}  argument.value returns the slider value			 			
			 * @example 
            * &lt;div id="slider"&gt; &lt;/div&gt; <br>
            * &lt;script&gt;
			 * //stop event for slider control
             * $("#slider").ejSlider({
             *    stop: function (args) {}
             * });
            * &lt;/script&gt;
			 * @memberof ejSlider
			 * @instance
			 */
            stop: null,
            /**     
			 * Fires when Slider control is sliding successfully.
			 * @event
			 * @name ejSlider#slide 	
			 * @param {Object} argument Event parameters from slider control     
			 * @param {boolean}  argument.cancel if the event should be cancelled; otherwise, false.
			 * @param {number}  argument.sliderIndex returns current handle number or index	
			 * @param {string}  argument.id returns slider id	
			 * @param {object}  argument.model returns the slider model	
			 * @param {string}  argument.type returns the name of the event				 
			 * @param {number}  argument.value returns the slider value			 			
			 * @example 
             * &lt;div id="slider"&gt; &lt;/div&gt; <br>
             * &lt;script&gt;
			 * //slide event for slider control
             * $("#slider").ejSlider({
             *    slide: function (args) {}
             * });
            * &lt;/script&gt;
            * @memberof ejSlider
			 * @instance
			 */
            slide: null,
            /**     
			 * Fires when Slider control value is changed successfully.
			 * @event
			 * @name ejSlider#change 	
			 * @param {Object} argument Event parameters from slider control     
			 * @param {boolean}  argument.cancel if the event should be cancelled; otherwise, false.
			 * @param {number}  argument.sliderIndex returns current handle number or index	
			 * @param {string}  argument.id returns slider id	
			 * @param {object}  argument.model returns the slider model	
			 * @param {string}  argument.type returns the name of the event				 
			 * @param {number}  argument.value returns the slider value			 			
			 * @example 
             * &lt;div id="slider"&gt; &lt;/div&gt; <br>
             * &lt;script&gt;
			 * //change event for slider control
             * $("#slider").ejSlider({
             *    change: function (args) {}
             * });
            * &lt;/script&gt;
			 * @memberof ejSlider
			 * @instance
			 */
            change: null,
            /**     
            * Fires when Slider control has been created successfully.
            * @event
            * @name ejSlider#create	
            * @param {Object} argument Event parameters from slider control     
            * @param {boolean}  argument.cancel if the event should be cancelled; otherwise, false.         
            * @param {object}  argument.model returns the slider model	
            * @param {string}  argument.type returns the name of the event                 
            * @example 
            * &lt;div id="slider"&gt; &lt;/div&gt; <br>
            * &lt;script&gt;
            * //create event for slider control
            * $("#slider").ejSlider({
            *    create: function (args) {}
            * });
           * &lt;/script&gt;
            * @memberof ejSlider
            * @instance
            */
            create: null,
            /**     
            * Fires when Slider control has been destroyed successfully.
            * @event
            * @name ejSlider#destroy	
            * @param {Object} argument Event parameters from slider control     
            * @param {boolean}  argument.cancel if the event should be cancelled; otherwise, false.         
            * @param {object}  argument.model returns the slider model	
            * @param {string}  argument.type returns the name of the event                 
            * @example 
            * &lt;div id="slider"&gt; &lt;/div&gt; <br>
            * &lt;script&gt;
            * //destroy event for slider control
            * $("#slider").ejSlider({
            *    destroy: function (args) {}
            * });
           * &lt;/script&gt;
            * @memberof ejSlider
            * @instance
            */
            destroy: null
        },
        /**
         * Specify the data types for default properties 
		 * @private
         */
        dataTypes: {
            orientation: "enum",
            enableAnimation: "boolean",
            animationSpeed: "number",
            cssClass: "string",
            showRoundedCorner: "boolean",
            readOnly: "boolean",
            enableRTL: "boolean",
            minValue: "number",
            maxValue: "number",
            sliderType: "enum",
            incrementStep: "number",
            enabled: "boolean",
            showScale: "boolean",
            largeStep: "number",
            smallStep: "number",
            showSmallTicks: "boolean",
            enablePersistence: "boolean"
        },

        observables: ["value", "values"],
        value: ej.util.valueFunction("value"),
        values: ej.util.valueFunction("values"),
        /**
        * To enable the slider  
		* @return jQuery
		* @example 
		* &lt;div id="slider"&gt; &lt;/div&gt; <br> 
		* &lt;script&gt;
		* $("#slider").ejSlider();
		* // Create slider control
		* var sliderObj = $("#slider").data("ejSlider");
		* sliderObj.enable(); // enable the slider control
		* &lt;/script&gt;
		* @example 
		* &lt;div id="slider"&gt; &lt;/div&gt; <br> 
		* &lt;script&gt;
		* $("#slider").ejSlider();
		* // enable the slider control
		* $("#slider").ejSlider("enable");
		* &lt;/script&gt;
		*@memberof ejSlider
		* @instance
        */
        enable: function () {
            if (!this.model.enabled) {
                this.model.enabled = true;
                if (this.wrapper) this.wrapper.removeClass("e-disable");
                else this.element.removeClass("e-disable");
                this._wireEvents();
            }
        },
        /**
        * To disable the slider  
		* @return jQuery
		* @example 
		* &lt;div id="slider"&gt; &lt;/div&gt; <br> 
		* &lt;script&gt;
		* $("#slider").ejSlider();
		* // Create slider control
		* var sliderObj = $("#slider").data("ejSlider");
		* sliderObj.disable(); // disable the slider control
		* &lt;/script&gt;
		* @example 
		* &lt;div id="slider"&gt; &lt;/div&gt; <br> 
		* &lt;script&gt;
		* $("#slider").ejSlider();
		* // disable the slider control
		* $("#slider").ejSlider("disable");
		* &lt;/script&gt;
		*@memberof ejSlider
		* @instance
        */
        disable: function () {
            if (this.model.enabled) {
                this.model.enabled = false;
                if (this.wrapper) this.wrapper.addClass("e-disable");
                else this.element.addClass("e-disable");
                this._unWireEvents();
            }
        },
        /**
         * To check whether the value and its type.		
		 * @private
         */
        _validateValue: function (value) {
            if (value == null) value = this.model.minValue;
            else if (typeof value === "string") value = parseFloat(value);

            if (this._isNumber(value))
                this.value(value);
            else if (!this._isNumber(this.value()))
                this.value(this.model.minValue);

            if (this.model.sliderType != "range") this._setValue();
        },
        /**
         * To check whether and configure the range slider value.		
		 * @private
         */
        _validateRangeValue: function (value) {
            if (value == null) value = new Array(this.model.minValue, this.model.maxValue);
            else if (typeof value === "string") {
                var vals = value.split(",");
                if (vals.length > 1) value = new Array(parseFloat(vals[0]), parseFloat(vals[1]));
            }

            if (typeof value === "object" && this._isNumber(value[0]) && this._isNumber(value[1]))
                this.values(new Array(value[0], value[1]));
            else if (!(typeof this.values() === "object" && this._isNumber(this.values()[0]) && this._isNumber(this.values()[1])))
                this.values(new Array(this.model.minValue, this.model.maxValue));

            if (this.model.sliderType == "range") this._setRangeValue();
        },
        /**
         * To check whether slider starting and ending value .		
		 * @private
         */
        _validateStartEnd: function () {
            if (isNaN(this.model.minValue)) this.model.minValue = 0;
            if (isNaN(this.model.maxValue)) this.model.maxValue = 100;
        },
        /**
         * To check whether slider value type.		
		 * @private
         */
        _isNumber: function (number) {
            return typeof number === "number" && !isNaN(number);
        },
        /**
         * To enable or disable the rounded corner behaviour 		
		 * @private
         */
        _outerCorner: function (boolean) {
            if (boolean) this._roundedCorner();
            else this._sharpedCorner();
        },
        /**
         * To configure the custom theme for slider using cssClass property		
		 * @private
         */
        _changeSkin: function (skin) {
            this.element.removeClass(this.model.cssClass).addClass(skin);
            if (this.model.showScale)
                this.ul.removeClass(this.model.cssClass).addClass(skin);
        },
        /**
        * To get value from slider handle
		* @return jQuery
		* @example 
		* &lt;div id="slider"&gt; &lt;/div&gt; <br>
		* &lt;script&gt;
		* $("#slider").ejSlider();
		* // Create Editors
		* var sliderObj = $("#slider").data("ejSlider");
		* sliderObj.getValue(); // getValue the slider handle
		* &lt;/script&gt;
		 @example 
		* &lt;div id="slider"&gt; &lt;/div&gt; <br> 
		* &lt;script&gt;
		* $("#slider").ejSlider();
		* // get value from slider handle
		* $("#slider").ejSlider("getValue");
		* &lt;/script&gt;
		* @memberof ejSlider
		* @instance
         */
        getValue: function () {
            /// <summary>Returns the Slider value.</summary>
            return this._getHandleValue();
        },
        /**
         * To initialize the slider		
		 * @private
         */
        // constructor function
        _init: function () {
            this._initialize();
            this._render();
        },
        /**
         * To configure the properties at runtime using SetModel		
		 * @private
         */
        _setModel: function (options) {
            if (!this.model.enabled && ej.isNullOrUndefined(options["enabled"])) return false;

            if (!ej.isNullOrUndefined(options["minValue"]) || !ej.isNullOrUndefined(options["maxValue"])) {
                if (this._isNumber(options["minValue"])) this.model.minValue = options["minValue"];
                else options["minValue"] = this.model.minValue;

                if (this._isNumber(options["maxValue"])) this.model.maxValue = options["maxValue"];
                else options["maxValue"] = this.model.maxValue;

                if (this.model.sliderType == "range" && options["values"] == undefined) this._setRangeValue();
                else if (this.model.sliderType != "range" && options["value"] == undefined) this._setValue();
            }

            var option;
            for (option in options) {
                switch (option) {
                    case "value":
                        this._validateValue(ej.util.getVal(options[option]));
                        break;
                    case "values":
                        this._validateRangeValue(ej.util.getVal(options[option]));
                        break;
                    case "height": this.model.height = options[option]; this._setDimension();
                        if (this.model.showScale) this._scaleAlignment();
                        break;
                    case "width": this.model.width = options[option]; this._setDimension();
                        if (this.model.showScale) this._scaleAlignment();
                        break;
                    case "enabled": this._disabled(!options[option]); break;
                    case "showRoundedCorner": this._outerCorner(options[option]); break;
                    case "enableRTL": this.model.enableRTL = options[option]; this._checkRTL();
                        options[option] = this.model.enableRTL;
                        break;
                    case "cssClass": this._changeSkin(options[option]); break;
                    case "showScale": this._renderScale(options[option]); break;
                    case "orientation":
                        var t = this.model.height;
                        this.model.height = this.model.width;
                        this.model.width = t;
                    case "sliderType":
                        this._sliderOptions(option, options[option]); break;
                    case "smallStep":
                    case "largeStep":
                    case "showSmallTicks":
                    case "minValue":
                    case "maxValue":
                        this._scaleOptions(option, options[option]); break;
                }
            }
        },
        /**
         * To configure destroy of the slider control.		
		 * @private
         */
        _destroy: function () {
            if (this.model.showScale) this._destroyScale();
            this.element.insertAfter(this.wrapper);
            this.wrapper.remove();
            this.element.removeClass("e-widget e-box " + this.model.cssClass).empty();
        },
        /**
         * To initialize the slider control position		
		 * @private
         */
        _initialize: function () {
            this.target = this.element[0];
            this.horDir = "left";
            this.verDir = "bottom";
            this._isFocused = false;
        },
        /**
         * Render Section For DifferentTypes		
		 * @private
         */
        _render: function () {
            this.wrapper = ej.buildTag("div.e-slider-wrap " + this.model.cssClass + "#" + this.target.id + "_wrapper", { tabindex: "0", role: "slider" })
                .insertAfter(this.element);
            this.wrapper.append(this.element);

            this.element.addClass("e-widget e-box " + this.model.cssClass);
            if (this.model.sliderType != "default") {
                this.header = ej.buildTag("div.e-range");
                this.element.append(this.header);
                if (this.model.sliderType == "range") {
                    this.secondHandle = this._createHandle();
                }
            }
            this.firstHandle = this._createHandle();
            this._setOrientation();
            this._setDimension();
            this._insertHiddenField();
            this._checkProperties();
            this._setSliderValue();
        },
        /**
         * Render Section For scale		
		 * @private
         */
        _renderScale: function (showScale) {
            if (showScale) {
                var width = "width", orien = "h";
                if (this.model.orientation == "vertical") {
                    width = "height";
                    orien = "v";
                }

                var _smallStep = this.model.smallStep;
                if (!this.model.showSmallTicks) {
                    if (this.model.largeStep > 0)
                        _smallStep = this.model.largeStep;
                    else
                        _smallStep = this.model.maxValue - this.model.minValue;
                }
                else if (_smallStep <= 0)
                    _smallStep = this.model.incrementStep;
                var count = Math.abs(this.model.maxValue - this.model.minValue) / _smallStep;

                this.ul = ej.buildTag("ul.e-scale e-" + orien + "-scale " + this.model.cssClass);
                this.wrapper.append(this.ul);

                var li, start = this.model.minValue, left = 0, tickWidth = 100 / count;
                if (orien == "v") start = this.model.maxValue;
                for (var i = 0; i <= count; i++) {
                    li = ej.buildTag("li.e-tick", "", {}, { "title": start });
                    li.css(width, tickWidth + "%");

                    if (start % this.model.largeStep == 0) {
                        var span = ej.buildTag("span.e-tick-value", "" + start);
                        li.addClass("e-large").append(span);
                    }
                    this.ul.append(li);

                    if (orien == "h") start += _smallStep;
                    else start -= _smallStep;
                    left += _smallStep;
                }

                this.ul.children().first().addClass("e-first-tick").css(width, (tickWidth / 2) + "%");
                this.ul.children().last().addClass("e-last-tick").css(width, (tickWidth / 2) + "%");

                this._scaleAlignment();
            }
            else this._destroyScale();
        },
        _destroyScale: function () {
            this.ul.remove();
            this.ul = null;
        },
        /**
         * To configure tick value position		
		 * @private
         */
        _tickValuePosition: function () {
            var width = (this.model.orientation == "vertical") ? "height" : "width";
            var left = (this.model.orientation == "vertical") ? "top" : "left";

            var firstTick = this.ul.find(".e-tick.e-first-tick");
            var first = firstTick.find(".e-tick-value");
            var other = this.ul.find(".e-tick.e-large:not(.e-first-tick)").find(".e-tick-value");

            tickWidth = firstTick[width]() * 2;
            first.css(left, -first[width]() / 2);
            other.css(left, (tickWidth - other[width]()) / 2);
        },
        /**
         * To configure scale alignment in slider		
		 * @private
         */
        _scaleAlignment: function () {
            this._tickValuePosition();

            var smallTick = 12, largeTick = 20, half = largeTick / 2;
            var height = "height", top = "top", orien = "h";
            if (this.model.orientation == "vertical") {
                height = "width";
                top = "right";
                orien = "v";
            }
            // scale
            this.ul.css(top, -(this.wrapper[height]() + half));
            if (orien == "v") this.ul.css("top", -this.wrapper.height()).css(top, half);
            this.ul[height](this.wrapper[height]() + largeTick);
            // small-ticks
            var topSize = -(largeTick - smallTick) / 2;
            if (this.model.largeStep == null && orien != "v") topSize = -topSize;
            this.ul.find(".e-tick:not(.e-large)").css(height, this.wrapper[height]() + smallTick).css(top, topSize);
            // tick-values   // 4 - distance between tick value and tick
            if (orien == "v") this.ul.children(".e-large").find(".e-tick-value").css("right", this.wrapper.width() + largeTick + 4);
        },
        /**
         * To create handle for slider		
		 * @private
         */
        _createHandle: function () {
            handle = ej.buildTag("a.e-handle e-select", "", {}, { "aria-label": "drag", "tabindex": 0, role: "slider", "aria-valuemin": this.model.minValue, "aria-valuemax": this.model.maxValue });
            this.element.append(handle);
            return handle;
        },
        /**
         * To configure dimension for slider		
		 * @private
         */
        _setDimension: function () {
            if (this.model.height) this.wrapper.height(this.model.height);
            if (this.model.width) this.wrapper.width(this.model.width);
            this._setHandleSize();
            this._handleAlignment(this.model.enableRTL);
        },
        /**
         * To configure scale alignment in slider		
		 * @private
         */
        _insertHiddenField: function () {
            this._hidden = ej.buildTag("input", "", {},
                { "type": "hidden", "name": this.element[0].id }).val(this._getHandleValue());
            this.element.append(this._hidden);
        },
        /**
         * To configure properties of the slider		
		 * @private
         */
        _checkProperties: function () {
            if (!this.model.enabled) {
                if (this.wrapper) this.wrapper.addClass("e-disable");
                else this.element.addClass("e-disable");
            }
            else this._wireEvents();

            if (this.model.showScale) this._renderScale(true);
            if (this.model.enableRTL) this._checkRTL();
            if (this.model.showRoundedCorner) this._roundedCorner();
        },
        /**
         * To configure the rounded corner behaviour 		
		 * @private
         */
        _roundedCorner: function () {
            this.element.addClass("e-corner-all");
            this.element.children(".e-handle").addClass("e-corner-all");
        },
        /**
         * To configure the shaped corner behaviour 		
		 * @private
         */
        _sharpedCorner: function () {
            this.element.removeClass("e-corner-all");
            this.element.children(".e-handle").removeClass("e-corner-all");
        },
        /**
         * To configure the handle alignment in slider control 		
		 * @private
         */
        _handleAlignment: function (rtl) {
            var mar = -(this.firstHandle.outerWidth() / 2) + "px", margin;
            if (this.model.orientation != "vertical") {
                if (!rtl) margin = "0 0 0 " + mar;
                else margin = "0 " + mar + " 0 0";
            }
            else {
                if (!rtl) margin = "0 0 " + mar + " 0";
                else margin = mar + " 0 0 0";
            }
            this.element.children('.e-handle').css("margin", margin);
        },
        /**
         * To check whether Right to Left Direction in slider control 		
		 * @private
         */
        _checkRTL: function () {
            if (this.model.showScale && this.model.orientation == "vertical" && this.model.enableRTL) this.model.enableRTL = false;
            var rtl = this.model.enableRTL, preDir = (this.model.orientation != "vertical") ? this.horDir : this.verDir;
            if (rtl) {
                this.wrapper.addClass("e-rtl");
                this.horDir = "right";
                this.verDir = "top";
            }
            else {
                this.wrapper.removeClass("e-rtl");
                this.horDir = "left";
                this.verDir = "bottom";
            }
            var currDir = (this.model.orientation != "vertical") ? this.horDir : this.verDir;

            if (preDir != currDir) {
                this.firstHandle.css(currDir, this.firstHandle.css(preDir)).css(preDir, "auto");;
                if (this.model.sliderType != "default") {
                    this.header.css(currDir, this.header.css(preDir)).css(preDir, "auto");
                    if (this.model.sliderType == "range")
                        this.secondHandle.css(currDir, this.secondHandle.css(preDir)).css(preDir, "auto");
                }
            }
            this._handleAlignment(rtl);
        },
        /**
         * To configure the direction orientation in slider control 		
		 * @private
         */
        _setOrientation: function () {
            if (this.model.orientation != "vertical") {
                this.wrapper.addClass("e-horizontal");
            }
            else {
                this.wrapper.addClass("e-vertical");
                this.firstHandle.css(this.verDir, "0");
            }
        },
        /**
         * To configure the handle size in slider control 		
		 * @private
         */
        _setHandleSize: function () {
            var size;
            if (this.model.orientation != "vertical")
                size = this.element.height() + 4;
            else
                size = this.element.width() + 4;
            this.element.find(".e-handle").height(size).width(size);
        },
        /**
         * To enable or disable the slider control state.
		 * @private
         */
        _disabled: function (boolean) {
            if (boolean) this.disable();
            else this.enable();
        },
        /**
         * To configure slider control with its options 		
		 * @private
         */
        _sliderOptions: function (prop, value) {
            this._unWireEvents();
            this._destroy();
            this.model[prop] = value;
            this._init();
        },
        /**
         * To configure the scale used in slider control 		
		 * @private
         */
        _scaleOptions: function (prop, value) {
            if (this.model.showScale) {
                this._destroyScale();
                this.model[prop] = value;
                this._renderScale(true);
            }
        },
        /**
         * To configure the showTooltip in slider control 		
		 * @private
         */
        _showTooltip: function () {
            if (this.model.showTooltip) {
                $('body .e-tooltip').remove();
                this.tooltip = ej.buildTag("div.e-tooltip " + this.model.cssClass + " e-corner-all", { role: "tooltip" });
                $("body").append(this.tooltip);
                this._setTooltipPosition();
            }
        },
        /**
         * To configure the hideTooltip slider control 		
		 * @private
         */
        _hideTooltip: function () {
            if (this.model.showTooltip)
                this.tooltip.fadeOut(800);
        },
        /**
         * To enable or disable the slider control tooltip.
		 * @private
         */
        _showhideTooltip: function (showTooltip) {
            if (this.model.showTooltip && showTooltip) {
                this._showTooltip();
                this._hideTooltip();
            }
        },
        /**
         * To configure the Tooltip position for handle slider control 		
		 * @private
         */
        _setTooltipPosition: function () {
            if (this.model.showTooltip) {
                this._updateTooltipValue();
                var top, left, remainLeft, remainTop, handle, pos, gap = 5; // gap -> distance between tooltip and slider
                handle = this._getHandle(), pos = this._getOffset(handle);

                if (this.model.orientation == "vertical") {
                    remainTop = (this.tooltip.outerHeight() - handle.outerHeight()) / 2;
                    remainLeft = handle.outerWidth() + gap;
                    top = pos.top - remainTop;
                    left = pos.left + remainLeft;
                }
                else {
                    remainLeft = (this.tooltip.outerWidth() - handle.outerWidth()) / 2;
                    remainTop = this.tooltip.outerHeight() + gap;
                    top = pos.top - remainTop;
                    left = pos.left - remainLeft;
                }
                var zindex = this._maxZindex();
                this.tooltip.css({ "top": top, "left": left, "zIndex": zindex + 1 });
            }
        },
        _getOffset: function (ele) {
            var pos = ele.offset();
            if ($("body").css("position") != "static") {
                var bodyPos = $("body").offset();
                pos.left -= bodyPos.left;
                pos.top -= bodyPos.top;
            }
            return pos;
        },

		/**
        * To calculate the max Zindex in the document.		
        * @private
        */
        _maxZindex: function () {
            var parents = this.element.parents(), bodyEle;
            bodyEle = $('body').children(), index = bodyEle.index(this.popup);
            bodyEle.splice(index, 1);
            $(bodyEle).each(function (i, ele) { parents.push(ele); });

            var maxZ = Math.max.apply(maxZ, $.map(parents, function (e, n) {
                if ($(e).css('position') != 'static') return parseInt($(e).css('z-index')) || 1;
            }));
            if (maxZ == undefined || maxZ == null || maxZ < 10000) maxZ = 10000;
            else maxZ += 1;
            return maxZ;
        },
        /**
         * To obtain the Tooltip value from handle in slider control 		
		 * @private
         */
        _updateTooltipValue: function () {
            var one = 0, two = 1;
            if (this.model.enableRTL) { one = 1, two = 0; }
            var val = this._getHandleValue();
            if (this.model.sliderType != "range")
                this.tooltip[0].innerHTML = val;
            else
                this.tooltip[0].innerHTML = val[one] + " - " + val[two];
        },
		/**
         * To configure the slider control header width based on it's type.		
		 * @private
         */
        _increaseHeaderWidth: function (animation) {
            if (this.model.sliderType != "default") {
                var size = "width", direction = this.horDir, properties = {};
                if (this.model.orientation == "vertical") { size = "height", direction = this.verDir; }

                if (this.model.sliderType == "range") {
                    properties[size] = this.handlePos - this.handlePos2 + "%";
                    properties[direction] = this.handlePos2 + "%";
                }
                else {
                    properties[size] = this.handlePos + "%";
                    properties[direction] = 0;
                }
                if (!animation) this.header.css(properties);
                else this.header.animate(properties, this.model.animationSpeed);
            }
        },
        /**
         * To configure the value based on it's slider type
		 * @private
         */
        _setSliderValue: function () {
            this._validateStartEnd();

            if (this.model.sliderType == "range")
                this._validateRangeValue(this.values());
            else
                this._validateValue(this.value());
        },

        /**
        * Section For handle the mouse hover event on slider handle.
        * @private
        */
        _hoverOnHandle: function (evt) {
            $(evt.target).addClass("e-hover");
        },
        /**
        * Section For handle the mouse leave event on slider handle.
        * @private
        */
        _leaveFromHandle: function (evt) {
            $(evt.target).removeClass("e-hover");
        },
        /**
        * Section For handle the click event on slider handle.
        * @private
        */
        _firstHandleClick: function (evt) {
            evt.preventDefault();
            this.firstHandle.focus();
            if (this._raiseEvent("start")) return false;

            this.mouseDownPos = this.handlePos;
            if (!this.model.readOnly)
                $(document).bind(ej.eventType.mouseMove, $.proxy(this._firstHandleMove, this));
            $(document).bind(ej.eventType.mouseUp, $.proxy(this._firstHandleUp, this));
            $(document).bind("mouseleave", $.proxy(this._firstHandleUp, this));
            this._showTooltip();
        },
        /**
        * Section For handle the drag event over slider handle.
        * @private
        */
        _firstHandleMove: function (evt) {
            evt.preventDefault();
            evt = evt.type == "touchmove" ? evt.originalEvent.changedTouches[0] : evt;
            var position = { x: evt.pageX, y: evt.pageY };
            this.handlePos = this._xyToPosition(position);

            if (this.model.sliderType == "range" && this.handlePos < this.handlePos2) {
                this.handlePos = this.handlePos2;
            }
            if (this.handlePos != this.preHandlePos) {
                this.preHandlePos = this.handlePos;
                this.handleVal = this._positionToValue(this.handlePos);
                this._increaseHeaderWidth(false);
                this._setHandlePosition(false, false, false);
                this._setTooltipPosition();

                this._updateModelValue();
                this._raiseEvent("slide");
            }
        },
        /**
        * Section For handle the mouse up event over slider handle.
        * @private
        */
        _firstHandleUp: function (evt) {
            evt.preventDefault();
            $(document).unbind(ej.eventType.mouseMove, $.proxy(this._firstHandleMove, this));
            $(document).unbind(ej.eventType.mouseUp, $.proxy(this._firstHandleUp, this));
            $(document).unbind("mouseleave", $.proxy(this._firstHandleUp, this));
            this._hideTooltip();

            if (this.mouseDownPos != this.handlePos) this._raiseChangeEvent();
            this._raiseEvent("stop");
        },
        /**
        * Section For handle the click event on range slider second handle.
        * @private
        */
        _secondHandleClick: function (evt) {
            evt.preventDefault();
            this.secondHandle.focus();
            if (this._raiseEvent("start")) return false;

            this.mouseDownPos2 = this.handlePos2;
            if (!this.model.readOnly)
                $(document).bind(ej.eventType.mouseMove, $.proxy(this._secondHandleMove, this));
            $(document).bind(ej.eventType.mouseUp, $.proxy(this._secondHandleUp, this));
            $(document).bind("mouseleave", $.proxy(this._secondHandleUp, this));
            this._showTooltip();
        },
        /**
        * Section For handle the drag event over range slider second handle.
        * @private
        */
        _secondHandleMove: function (evt) {
            evt.preventDefault();
            evt = evt.type == "touchmove" ? evt.originalEvent.changedTouches[0] : evt;
            var position2 = { x: evt.pageX, y: evt.pageY };
            this.handlePos2 = this._xyToPosition(position2);

            if (this.handlePos2 > this.handlePos) {
                this.handlePos2 = this.handlePos;
            }
            if (this.handlePos2 != this.preHandlePos2) {
                this.preHandlePos2 = this.handlePos2;
                this.handleVal2 = this._positionToValue(this.handlePos2);
                this._increaseHeaderWidth(false);
                this._setHandlePosition(false, false, false);
                this._setTooltipPosition();

                this._updateModelValue();
                this._raiseEvent("slide");
            }
        },
        /**
        * Section For handle the mouse up event on range slider second handle.
        * @private
        */
        _secondHandleUp: function (evt) {
            evt.preventDefault();
            $(document).unbind(ej.eventType.mouseMove, $.proxy(this._secondHandleMove, this));
            $(document).unbind(ej.eventType.mouseUp, $.proxy(this._secondHandleUp, this));
            $(document).unbind("mouseleave", $.proxy(this._secondHandleUp, this));
            this._hideTooltip();

            if (this.mouseDownPos2 != this.handlePos2) this._raiseChangeEvent();
            this._raiseEvent("stop");
        },
        /**
        * Section For handle the focus in event on slider handle.
        * @private
        */
        _focusInHandle: function (evt) {
            if (!this._isFocused) {
                this._isFocused = true;
                $(evt.target).addClass("e-focus");
                if (!this.model.readOnly)
                    $(document).bind("keydown", $.proxy(this._moveHandle, this));
                this.activeHandle = $(evt.target).is(this.firstHandle) ? 1 : 2;
                this._setZindex();
            }
        },
        /**
        * Section For handle the focus out event on slider handle.
        * @private
        */
        _focusOutHandle: function (evt) {
            this._isFocused = false;
            $(evt.target).removeClass("e-focus");
            $(document).unbind("keydown", $.proxy(this._moveHandle, this));
        },
        /**
        * Section For handle the slider handle move.
        * @private
        */
        _moveHandle: function (e) {
            var oper, val, handleNo;
            handleNo = this._getHandleIndex(this.activeHandle) - 1;

            switch (e.keyCode) {
                case 37:        // Left Key
                case 40:        // Down Key
                    e.preventDefault();
                    oper = "sub";
                    break;
                case 38:        // Up Key
                case 39:         // Right Key
                    e.preventDefault();
                    oper = "add";
                    break;
                case 36:         // Home Key
                    e.preventDefault();
                    if (this._raiseEvent("start")) return false;
                    if (this.model.sliderType != "range" && this.value() != this.model.minValue) {
                        this._changeHandleValue(this.model.minValue, this.model.enableAnimation);
                    }
                    else if (this.model.sliderType == "range") {
                        val = (this.activeHandle == 2) ? this.model.minValue : this.handleVal2;
                        if (this.values()[handleNo] != val)
                            this._changeHandleValue(val, this.model.enableAnimation);
                    }
                    break;
                case 35:         // End Key
                    e.preventDefault();
                    if (this._raiseEvent("start")) return false;
                    if (this.model.sliderType != "range" && this.value() != this.model.maxValue) {
                        this._changeHandleValue(this.model.maxValue, this.model.enableAnimation);
                    }
                    else if (this.model.sliderType == "range") {
                        val = (this.activeHandle == 1) ? this.model.maxValue : this.handleVal;
                        if (this.values()[handleNo] != val)
                            this._changeHandleValue(val, this.model.enableAnimation);
                    }
                    break;
                case 27:         // Esc Key
                    e.preventDefault();
                    this._getHandle().focusout();
                    break;
            }

            if (oper == "add" || oper == "sub") {
                if (this._raiseEvent("start")) return false;
                var hVal = (this.activeHandle == 1) ? this.handleVal : this.handleVal2;
                var value = (oper == "add") ? this._add(hVal, this.model.incrementStep, true) : this._add(hVal, this.model.incrementStep, false);
                this._changeHandleValue(value, false);
            }
        },
        /**
        * Section For handle the slider handle value change.
        * @private
        */
        _changeHandleValue: function (value, animate) {
            var position = null;
            if (this.activeHandle == 1) {
                this.handleVal = this._checkHandleValue(value);
                this.handlePos = this._checkHandlePosition(this.handleVal);

                if (this.model.sliderType == "range" && this.handlePos < this.handlePos2) {
                    this.handlePos = this.handlePos2;
                    this.handleVal = this.handleVal2;
                }
                if (this.handlePos != this.preHandlePos)
                    position = this.preHandlePos = this.handlePos;
            }
            else {
                this.handleVal2 = this._checkHandleValue(value);
                this.handlePos2 = this._checkHandlePosition(this.handleVal2);

                if (this.model.sliderType == "range" && this.handlePos < this.handlePos2) {
                    this.handlePos2 = this.handlePos;
                    this.handleVal2 = this.handleVal;
                }
                if (this.handlePos2 != this.preHandlePos2)
                    position = this.preHandlePos2 = this.handlePos2;
            }

            if (position != null) {
                this._increaseHeaderWidth(animate);
                this._setHandlePosition(animate, true, true);
            }
        },
        /**
        * Section For handle the slider bar click event.
        * @private
        */
        _sliderBarClick: function (evt) {
            if (this.model.readOnly) return false;
            if (evt.target == this.target || (this.model.sliderType != "default" && evt.target == this.header[0])) {
                evt.preventDefault();
                if (this._raiseEvent("start")) return false;
                var pos = { x: evt.pageX, y: evt.pageY },
                handlepos = this._xyToPosition(pos),
                handleVal = this._positionToValue(handlepos);

                if (this.model.sliderType == "range" && (this.handlePos - handlepos) > (handlepos - this.handlePos2)) {
                    this.handlePos2 = this.preHandlePos2 = handlepos;
                    this.handleVal2 = handleVal;
                    this.activeHandle = 2;
                }
                else {
                    this.handlePos = this.preHandlePos = handlepos;
                    this.handleVal = handleVal;
                    this.activeHandle = 1;
                }

                this._getHandle().focus();
                if (this.model.sliderType != "default") this._increaseHeaderWidth(this.model.enableAnimation);
                this._setHandlePosition(this.model.enableAnimation, true, true);
            }
        },
        /**
        * To configure the slider handle position.
        * @private
        */
        _setHandlePosition: function (animation, showTooltip, changeEvt) {
            var Handle = this._getHandle(), proxy = this, properties = {}, pos, val, direction;
            pos = (this.activeHandle == 1) ? this.handlePos : this.handlePos2;
            val = (this.activeHandle == 1) ? this.handleVal : this.handleVal2;
            Handle.attr("aria-label", val);
            direction = (this.model.orientation == "vertical") ? this.verDir : this.horDir;
            properties[direction] = pos + "%";

            if (!animation) {
                Handle.css(properties);
                this._showhideTooltip(showTooltip);
                if (changeEvt) this._raiseChangeEvent();
            }
            else Handle.animate(properties, this.model.animationSpeed, function () {
                proxy._showhideTooltip(showTooltip);
                if (changeEvt) proxy._raiseChangeEvent();
            });
        },
        /**
        * Section For handle the X,Y co-ordinate position for slider.
        * @private
        */
        _xyToPosition: function (position) {
            if (this.model.minValue == this.model.maxValue)
                return 100;
            if (this.model.orientation != "vertical") {
                var left = position.x - this.element.offset().left,
                num = this.element.width() / 100,
                val = (left / num);
            }
            else {
                var top = position.y - this.element.offset().top,
                num = this.element.height() / 100,
                val = 100 - (top / num);
            }
            val = this._stepValueCalculation(val);
            if (val < 0) val = 0;
            else if (val > 100) val = 100;
            if (this.model.enableRTL) return 100 - val;
            return val;
        },
        /**
        * Section For handle the slider handle value updation.
        * @private
        */
        _updateValue: function () {
            this.handleVal = this._checkHandleValue(this.value());
            this.handlePos = this._checkHandlePosition(this.handleVal);
            this.preHandlePos = this.handlePos;
            this.activeHandle = 1;
        },
        /**
        * Section For handle the slider handle operations.
        * @private
        */
        _setValue: function () {
            this._updateValue();

            this._increaseHeaderWidth(this.model.enableAnimation);
            this._setHandlePosition(this.model.enableAnimation, false, false);
        },
        /**
        * Section For handle slider range values.
        * @private
        */
        _updateRangeValue: function () {
            var values = this.values();
            this.handleVal = this._checkHandleValue(values[1]);
            this.handleVal2 = this._checkHandleValue(values[0]);
            this.handlePos = this._checkHandlePosition(this.handleVal);
            this.handlePos2 = this._checkHandlePosition(this.handleVal2);

            if (this.handlePos < this.handlePos2) {
                this.handlePos = this.handlePos2;
                this.handleVal = this.handleVal2;
            }
            this.preHandlePos = this.handlePos;
            this.preHandlePos2 = this.handlePos2;
        },
        /**
        * To configure slider range values.
        * @private
        */
        _setRangeValue: function () {
            this._updateRangeValue();
            this._increaseHeaderWidth(this.model.enableAnimation);
            this.activeHandle = 1;
            this._setHandlePosition(this.model.enableAnimation, false, false);
            this.activeHandle = 2;
            this._setHandlePosition(this.model.enableAnimation, false, false);
        },
        /**
         * To Check whether slider handle position.
		 * @private
         */
        _checkHandlePosition: function (value) {
            if (this.model.minValue == this.model.maxValue)
                return 100;
            var handle = this._tempStartEnd();
            if (value >= handle.start && value <= handle.end)
                value = (100 * (value - this.model.minValue)) / (this.model.maxValue - this.model.minValue);
            else if (value < handle.start) value = 0;
            else value = 100;
            return value;
        },
        /**
         * To Check whether slider handle values.
		 * @private
         */
        _checkHandleValue: function (value) {
            if (this.model.minValue == this.model.maxValue)
                return this.model.minValue;
            var handle = this._tempStartEnd();
            if (value < handle.start) value = handle.start;
            else if (value > handle.end) value = handle.end;
            return value;
        },
        /**
         * To configure slider starting and ending values.
		 * @private
         */
        _tempStartEnd: function () {
            if (this.model.minValue > this.model.maxValue)
                return {
                    start: this.model.maxValue,
                    end: this.model.minValue
                };
            else
                return {
                    start: this.model.minValue,
                    end: this.model.maxValue
                };
        },
        /**
         * To configure slider values based on slider handle present position.
		 * @private
         */
        _positionToValue: function (pos) {
            var diff = this.model.maxValue - this.model.minValue,
            val = this._round(diff * pos / 100),
            total = this._add(val, this.model.minValue, true);
            return total;
        },
        /**
         * To obtained active slider handle value.
		 * @private
         */
        _getHandle: function () {
            return (this.activeHandle == 1) ? this.firstHandle : this.secondHandle;
        },
        /**
         * To obtained slider handle index value.
		 * @private
         */
        _getHandleIndex: function (no) {
            if (this.model.sliderType == "range" && no == 1)
                return 2;
            return 1;
        },
        /**
         * To obtained slider handle values based on it's type.
		 * @private
         */
        _getHandleValue: function () {
            if (this.model.sliderType == "range") return [this.handleVal2, this.handleVal];
            else return this.handleVal;
        },
        /**
         * To configure the slider value based on it's type.
		 * @private
         */
        _updateModelValue: function () {
            var value = this._getHandleValue();
            if (this.model.sliderType == "range") this.values(value);
            else this.value(value);
            this._hidden.val(value);
        },
        /**
         * To configure slider value.
		 * @private
         */
        _add: function (a, b, addition, precision) {
            var x = Math.pow(10, precision || 3), val;
            if (addition) val = (Math.round(a * x) + Math.round(b * x)) / x;
            else val = (Math.round(a * x) - Math.round(b * x)) / x;
            return val;
        },
        /**
         * To configure slider value as rounded.
		 * @private
         */
        _round: function (a) {
            var _f = this.model.incrementStep.toString().split(".");
            return _f[1] ? parseFloat(a.toFixed(_f[1].length)) : Math.round(a);
        },
        /**
         * To configure slider change event.
		 * @private
         */
        _raiseChangeEvent: function () {
            this._updateModelValue();
            this._raiseEvent("change");
        },
        /**
         * Render section for raising event in slider .
		 * @private
         */
        _raiseEvent: function (name) {
           
                return this._trigger(name, {
                    id: this.target.id,
                    value: this._getHandleValue(),
                    sliderIndex: this._getHandleIndex(this.activeHandle)
                });
            
            
        },
        /**
         * To configure slider handle z-index value.
		 * @private
         */
        _setZindex: function () {
            if (this.model.sliderType == "range") {
                if (this.activeHandle == 1) {
                    this.firstHandle.css("z-index", 2);
                    this.secondHandle.css("z-index", 1);
                }
                else {
                    this.firstHandle.css("z-index", 1);
                    this.secondHandle.css("z-index", 2);
                }
            }
        },
        /**
         * To obtained slider handle value based on incrementStep value.
		 * @private
         */
        _stepValueCalculation: function (value) {
            if (this.model.incrementStep == 0) this.model.incrementStep = 1;
            var percentStep = this.model.incrementStep / ((this.model.maxValue - this.model.minValue) / 100);
            var remain = value % Math.abs(percentStep);
            if (remain != 0) {
                if ((percentStep / 2) > remain) value -= remain;
                else value += Math.abs(percentStep) - remain;
            }
            return value;
        },

        //-------------------- Event Wire-up -------------------------//
        /**
         * Wiring the events to slider control		
		 * @private
         */
        _wireEvents: function () {
            this._on(this.element, "mousedown", this._sliderBarClick);
            this._on(this.firstHandle, ej.eventType.mouseDown, this._firstHandleClick);
            this._on(this.firstHandle, "mouseenter", this._hoverOnHandle);
            this._on(this.firstHandle, "mouseleave", this._leaveFromHandle);
            this._on(this.firstHandle, "focus", this._focusInHandle);
            this._on(this.firstHandle, "focusout", this._focusOutHandle);

            if (this.model.sliderType == "range") {
                this._on(this.secondHandle, ej.eventType.mouseDown, this._secondHandleClick);
                this._on(this.secondHandle, "mouseenter", this._hoverOnHandle);
                this._on(this.secondHandle, "mouseleave", this._leaveFromHandle);
                this._on(this.secondHandle, "focus", this._focusInHandle);
                this._on(this.secondHandle, "focusout", this._focusOutHandle);
            }
        },
        //-------------------- Event Wire-up -------------------------//
        /**
         * unWiring the events from slider control.		
		 * @private
         */
        _unWireEvents: function () {
            this._off(this.element, "mousedown");
            this._off(this.firstHandle, ej.eventType.mouseDown);
            this._off(this.firstHandle, "mouseenter");
            this._off(this.firstHandle, "mouseleave");
            this._off(this.firstHandle, "focus");
            this._off(this.firstHandle, "focusout");

            if (this.model.sliderType == "range") {
                this._off(this.secondHandle, ej.eventType.mouseDown);
                this._off(this.secondHandle, "mouseenter");
                this._off(this.secondHandle, "mouseleave");
                this._off(this.secondHandle, "focus");
                this._off(this.secondHandle, "focusout");
            }
        }
    });
    /**
	 * Enum for slider type.	 
	 * @enum {string}
	 * @global 
	 */
    ej.SliderType = {
        /**  support for slider control to select a single value. */
        Default: "default",
        /**  support for slider control to select a single value considered from start value to current handle. */
        MinRange: "minrange",
        /**  support for slider control to select a range of value between the two handles. */
        Range: "range"
    };
})(jQuery, Syncfusion);;
/**
* @fileOverview Plugin to style the Html Button elements
* @copyright Copyright Syncfusion Inc. 2001 - 2013. All rights reserved.
*  Use of this code is subject to the terms of our license.
*  A copy of the current license can be obtained at any time by e-mailing
*  licensing@syncfusion.com. Any infringement will be prosecuted under
*  applicable laws. 
* @version 12.1 
* @author <a href="mailto:licensing@syncfusion.com">Syncfusion Inc</a>
*/
(function ($, ej, undefined) {
    /**
       * @namespace ej
       * @class ejSplitButton
       * @requires jQuery
       * @requires jquery.easing
       * @requires ej.core.js
       * @requires ej.data.js
       * @requires ej.splitbutton.js
       * @requires ej.menu.js
       * @requires ej.checkbox.js
   
       * @classdesc The Split button allows you to perform an action using clicking the button and choosing extra options from the dropdown button. The Split button also can display both text and images. 
       * @example 
       *&lt;button id="sbutton"&gt;File&lt;/button&gt;
       *&lt;ul id="target"&gt;
       *    &lt;li&gt;&lt;a href="#"&gt;Open..&lt;/a&gt;&lt;/li&gt;
       *    &lt;li&gt;&lt;a href="#"&gt;Save&lt;/a&gt;&lt;/li&gt;
       *    &lt;li&gt;&lt;a href="#"&gt;Delete&lt;/a&gt;&lt;/li&gt;
       *&lt;/ul&gt;
       * &lt;script&gt;
       * // simple control creation
       *  $("#sbutton").ejSplitButton({targetID:"target",width:100});
       * &lt;/script&gt;
       */

    // Example plugin creation code
    // ejSample is the plugin name 
    // "ej.Sample" is "namespace.className" will hold functions and properties

    ej.widget("ejSplitButton", "ej.SplitButton", {
        // widget element will be automatically set in this
        element: null,
        // user defined model will be automatically set in this
        model: null,
        validTags: ["button"],
        _setFirst: false,
        //Root Css Class
        _rootCSS: "e-splitbutton",

        // default model
        defaults: {
            /// <summary>This Contains default property of Split button </summary>
            /**		
			* Specifies the size of the Button.	See {@link ButtonSize}
			* @default ej.ButtonSize.Normal
			* @type {String | Enum}
			* @example 
			*&lt;button id="sbutton"&gt;File&lt;/button&gt;
	*&lt;ul id="target"&gt;
	*    &lt;li&gt;&lt;a href="#"&gt;Open..&lt;/a&gt;&lt;/li&gt;
	*    &lt;li&gt;&lt;a href="#"&gt;Save&lt;/a&gt;&lt;/li&gt;
	*    &lt;li&gt;&lt;a href="#"&gt;Delete&lt;/a&gt;&lt;/li&gt;
	*&lt;/ul&gt;
	* &lt;script&gt;
			* //To set size API value during initialization  
			* 	$("#sbutton").ejSplitButton({ targetID:"target",width:100, size: ej.ButtonSize.Mini});			
			  * &lt;/script&gt;
			 * @memberof ejSplitButton
			* @instance
			*/
            size: "normal",
            /**		
			* Specifies the width of the Split Button.
			* @default “28” pixel
			* @type {String | Number}
			* @example 
			*&lt;button id="sbutton"&gt;File&lt;/button&gt;
	*&lt;ul id="target"&gt;
	*    &lt;li&gt;&lt;a href="#"&gt;Open..&lt;/a&gt;&lt;/li&gt;
	*    &lt;li&gt;&lt;a href="#"&gt;Save&lt;/a&gt;&lt;/li&gt;
	*    &lt;li&gt;&lt;a href="#"&gt;Delete&lt;/a&gt;&lt;/li&gt;
	*&lt;/ul&gt;
	* &lt;script&gt;
			* //To set width API value during initialization  
			* $("#sbutton").ejSplitButton({  targetID: "target",width:100 });			
			  * &lt;/script&gt;
			 * @memberof ejSplitButton
			* @instance
			*/
            width: "",
            /**		
			* Specifies the height of the Split Button.
			* @default “28” pixel
			* @type {String | Number}
			* @example
			*&lt;button id="sbutton"&gt;File&lt;/button&gt;
	*&lt;ul id="target"&gt;
	*    &lt;li&gt;&lt;a href="#"&gt;Open..&lt;/a&gt;&lt;/li&gt;
	*    &lt;li&gt;&lt;a href="#"&gt;Save&lt;/a&gt;&lt;/li&gt;
	*    &lt;li&gt;&lt;a href="#"&gt;Delete&lt;/a&gt;&lt;/li&gt;
	*&lt;/ul&gt;
	* &lt;script&gt; 
			* //To set height API value during initialization  
			* $("#sbutton").ejSplitButton({  targetID: "target",width:100,height: 28 });			 
			  * &lt;/script&gt;
			 * @memberof ejSplitButton
			* @instance
			*/
            height: "",
            /**		
			*Specifies the disabling of Split Button if enabled is set to false.
			* @default true
			* @type {Boolean}
			* @example 
			*&lt;button id="sbutton"&gt;File&lt;/button&gt;
	*&lt;ul id="target"&gt;
	*    &lt;li&gt;&lt;a href="#"&gt;Open..&lt;/a&gt;&lt;/li&gt;
	*    &lt;li&gt;&lt;a href="#"&gt;Save&lt;/a&gt;&lt;/li&gt;
	*    &lt;li&gt;&lt;a href="#"&gt;Delete&lt;/a&gt;&lt;/li&gt;
	*&lt;/ul&gt;
	* &lt;script&gt;
			* //To set enabled API value during initialization  
			* $("#sbutton").ejSplitButton({  targetID: "target",width:100,enabled:  true });		
			  * &lt;/script&gt;
			
			 * @memberof ejSplitButton
			* @instance
			*/
            enabled: true,
            /**		
			*Specifies the text content for Split Button while initialization.
			* @default ""
			* @type {String}
			* @example 
			*&lt;button id="sbutton"&gt;File&lt;/button&gt;
	*&lt;ul id="target"&gt;
	*    &lt;li&gt;&lt;a href="#"&gt;Open..&lt;/a&gt;&lt;/li&gt;
	*    &lt;li&gt;&lt;a href="#"&gt;Save&lt;/a&gt;&lt;/li&gt;
	*    &lt;li&gt;&lt;a href="#"&gt;Delete&lt;/a&gt;&lt;/li&gt;
	*&lt;/ul&gt;
	* &lt;script&gt;
			* //To set text API value during initialization  
			* $("#sbutton").ejSplitButton({  targetID: "target",width:100, text: "New" });		 
			  * &lt;/script&gt;
			 * @memberof ejSplitButton
			* @instance
			*/
            text: null,
            /**		
			*Specifies the contentType of the Split Button.See {@link ContentType}
			* @default ej.ContentType.TextOnly
			* @type {String | Enum}
			* @example
			*&lt;button id="sbutton"&gt;File&lt;/button&gt;
	*&lt;ul id="target"&gt;
	*    &lt;li&gt;&lt;a href="#"&gt;Open..&lt;/a&gt;&lt;/li&gt;
	*    &lt;li&gt;&lt;a href="#"&gt;Save&lt;/a&gt;&lt;/li&gt;
	*    &lt;li&gt;&lt;a href="#"&gt;Delete&lt;/a&gt;&lt;/li&gt;
	*&lt;/ul&gt;
	* &lt;script&gt; 
			* //To set contentType API value during initialization  
			* $("#sbutton").ejSplitButton({ targetID: "target",width:100, contentType:  ej.ContentType.TextOnly}); 
			  * &lt;/script&gt;
			 * @memberof ejSplitButton
			* @instance
			*/
            contentType: "textonly",
            /**		
			*Specifies the imagePosition of the Split Button.See {@link imagePositions}
			* @default ej.ImagePosition.ImageRight
			* @type {String | Enum}
			* @example 
			*&lt;button id="sbutton"&gt;File&lt;/button&gt;
	*&lt;ul id="target"&gt;
	*    &lt;li&gt;&lt;a href="#"&gt;Open..&lt;/a&gt;&lt;/li&gt;
	*    &lt;li&gt;&lt;a href="#"&gt;Save&lt;/a&gt;&lt;/li&gt;
	*    &lt;li&gt;&lt;a href="#"&gt;Delete&lt;/a&gt;&lt;/li&gt;
	*&lt;/ul&gt;
	* &lt;script&gt;
			* //To set imagePositions API value during initialization  
			* $("#sbutton").ejSplitButton({targetID: "target",width:100, contentType: ej.ContentType.TextAndImage,
            *   imagePosition: ej.ImagePosition.ImageRight,prefixIcon:"e-uiLight e-icon e-handup"});
			  * &lt;/script&gt;
			* @memberof ejSplitButton
			* @instance
			*/
            imagePosition: "imageleft",
            /**		
			*Specifies the list content for Split Button while initialization
			* @default ""
			* @type {String}
			* @example 
			*&lt;button id="sbutton"&gt;File&lt;/button&gt;
	*&lt;ul id="target"&gt;
	*    &lt;li&gt;&lt;a href="#"&gt;Open..&lt;/a&gt;&lt;/li&gt;
	*    &lt;li&gt;&lt;a href="#"&gt;Save&lt;/a&gt;&lt;/li&gt;
	*    &lt;li&gt;&lt;a href="#"&gt;Delete&lt;/a&gt;&lt;/li&gt;
	*&lt;/ul&gt;
	* &lt;script&gt;

			* //To set targetID API value during initialization  
			* $("#sbutton").ejSplitButton({targetID:"target",width:100 });
			  * &lt;/script&gt;
			* @memberof ejSplitButton
			* @instance
			*/
            targetID: null,
            /**		
			*Specifies the showRoundedCorner property for Split Button while initialization.
			* @default ""
			* @type {String}
			* @example 
			*&lt;button id="sbutton"&gt;File&lt;/button&gt;
	*&lt;ul id="target"&gt;
	*    &lt;li&gt;&lt;a href="#"&gt;Open..&lt;/a&gt;&lt;/li&gt;
	*    &lt;li&gt;&lt;a href="#"&gt;Save&lt;/a&gt;&lt;/li&gt;
	*    &lt;li&gt;&lt;a href="#"&gt;Delete&lt;/a&gt;&lt;/li&gt;
	*&lt;/ul&gt;
	* &lt;script&gt;
			* //To set showRoundedCorner API value during initialization  
			* $("#sbutton").ejSplitButton({ targetID:"target",width:100,showRoundedCorner: true});
			  * &lt;/script&gt;
			* @memberof ejSplitButton
			* @instance
			*/
            showRoundedCorner: false,
            /**		
			*Specifies the image content for Split Button while initialization.
			* @default ""
			* @type {String}
			* @example 
			*&lt;button id="sbutton"&gt;File&lt;/button&gt;
	*&lt;ul id="target"&gt;
	*    &lt;li&gt;&lt;a href="#"&gt;Open..&lt;/a&gt;&lt;/li&gt;
	*    &lt;li&gt;&lt;a href="#"&gt;Save&lt;/a&gt;&lt;/li&gt;
	*    &lt;li&gt;&lt;a href="#"&gt;Delete&lt;/a&gt;&lt;/li&gt;
	*&lt;/ul&gt;
	* &lt;script&gt;
			* //To set prefixIcon API value during initialization  
			* $("#sbutton").ejSplitButton({targetID: "target",width:100,contentType: "imageonly",prefixIcon:"e-uiLight e-icon e-handup" });
			 
			  * &lt;/script&gt;
 			* @memberof ejSplitButton
			* @instance
			*/
            prefixIcon: null,
            /**		
			*Specifies the image content for Split Button while initialization.
			* @default ""
			* @type {String}
			* @example 
			*&lt;button id="sbutton"&gt;File&lt;/button&gt;
	*&lt;ul id="target"&gt;
	*    &lt;li&gt;&lt;a href="#"&gt;Open..&lt;/a&gt;&lt;/li&gt;
	*    &lt;li&gt;&lt;a href="#"&gt;Save&lt;/a&gt;&lt;/li&gt;
	*    &lt;li&gt;&lt;a href="#"&gt;Delete&lt;/a&gt;&lt;/li&gt;
	*&lt;/ul&gt;
	* &lt;script&gt;
			* //To set suffixIcon API value during initialization  
			* $("#sbutton").ejSplitButton({targetID:"target",width:100,contentType:"imageboth",prefixIcon:"e-uiLight e-icon-handup",suffixIcon:"e-uiLight e-icon-padlockclosed"});
			* &lt;/script&gt;
			* @memberof ejSplitButton
			* @instance
			*/
            suffixIcon: null,
            /**		
			*Set the root class for Split Button control theme
			* @default ""
			* @type {String}
			* @example 
			*&lt;button id="sbutton"&gt;File&lt;/button&gt;
	*&lt;ul id="target"&gt;
	*    &lt;li&gt;&lt;a href="#"&gt;Open..&lt;/a&gt;&lt;/li&gt;
	*    &lt;li&gt;&lt;a href="#"&gt;Save&lt;/a&gt;&lt;/li&gt;
	*    &lt;li&gt;&lt;a href="#"&gt;Delete&lt;/a&gt;&lt;/li&gt;
	*&lt;/ul&gt;
	* &lt;script&gt;
			* //To set cssClass API value during initialization  
			* $("#sbutton").ejSplitButton({targetID: "target",width:100,cssClass: "gradient-lime"});
			  * &lt;/script&gt;
 			* @memberof ejSplitButton
			* @instance
			*/
            cssClass: "",
            /**		
			*Specifies the enableRTL property for Split Button while initialization.
			* @default false
			* @type {Boolean}
			* @example 
			*&lt;button id="sbutton"&gt;File&lt;/button&gt;
	*&lt;ul id="target"&gt;
	*    &lt;li&gt;&lt;a href="#"&gt;Open..&lt;/a&gt;&lt;/li&gt;
	*    &lt;li&gt;&lt;a href="#"&gt;Save&lt;/a&gt;&lt;/li&gt;
	*    &lt;li&gt;&lt;a href="#"&gt;Delete&lt;/a&gt;&lt;/li&gt;
	*&lt;/ul&gt;
	* &lt;script&gt;
			* //To set enableRTL API value during initialization  
			* $("#sbutton").ejSplitButton({targetID: "target",width:100,enableRTL : true});
			  * &lt;/script&gt;
 			* @memberof ejSplitButton
			* @instance
			*/
            enableRTL: false,
            /**     
			 * Fires after Split Button control is created.
			 * @event
			 * @name ejSplitButton#create		
			 * @param {Object} argument Event parameters from button     
			 * @param {boolean}  argument.cancel returns the cancel option value
			 * @param {object}  argument.model returns the splite button model
			 * @param {string}  argument.type returns the name of the event
			 * @example 
			 *&lt;button id="sbutton"&gt;File&lt;/button&gt;
	*&lt;ul id="target"&gt;
	*    &lt;li&gt;&lt;a href="#"&gt;Open..&lt;/a&gt;&lt;/li&gt;
	*    &lt;li&gt;&lt;a href="#"&gt;Save&lt;/a&gt;&lt;/li&gt;
	*    &lt;li&gt;&lt;a href="#"&gt;Delete&lt;/a&gt;&lt;/li&gt;
	*&lt;/ul&gt;
	* &lt;script&gt;
			 * //create event for split button
             * $("#sbutton"). ejSplitButton ({
             	targetID: "target",width:100,
             *      create: function (args) {}
             * });
               * &lt;/script&gt;
			 * @memberof ejSplitButton
			 * @instance
			 */
            create: null,
            /**     
			 * Fires when Button control is clicked successfully
			 * @event
			 * @name ejSplitButton#click		
			 * @param {Object} argument Event parameters from split button     
			 * @param {boolean}  argument.cancel returns the cancel option value
			 * @param {object}  argument.model returns the split button model
			 * @param {string}  argument.type returns the name of the event
			 * @param {object} argument.target  returns the target of the current object.
			 * @param {boolean}  argument.status return the button state
			 * @example 
			 *&lt;button id="sbutton"&gt;File&lt;/button&gt;
	*&lt;ul id="target"&gt;
	*    &lt;li&gt;&lt;a href="#"&gt;Open..&lt;/a&gt;&lt;/li&gt;
	*    &lt;li&gt;&lt;a href="#"&gt;Save&lt;/a&gt;&lt;/li&gt;
	*    &lt;li&gt;&lt;a href="#"&gt;Delete&lt;/a&gt;&lt;/li&gt;
	*&lt;/ul&gt;
	* &lt;script&gt;
			 * //click event for split button
             * $("#sbutton"). ejSplitButton ({
             	targetID: "target",width:100,
             *      click: function (args) {}
             * });
               * &lt;/script&gt;
			 * @memberof ejSplitButton
			 * @instance
			 */
            click: null,
            /**     
			 * Fires when a menu item is Hovered in successfully
			 * @event
			 * @name ejSplitButton#itemMouseOver		
			 * @param {Object} argument Event parameters from split button     
			 * @param {boolean}  argument.cancel returns the cancel option value
			 * @param {object}  argument.model returns the split button model
			 * @param {string}  argument.type returns the name of the event
			 * @param {object} argument.event.element  returns the clicked menu item element
			 * @param {object} argument.event returns the event
			 * @param {String}  argument.event.ID  return the menu item id
			 * @param {String}  argument.event.Text  return the clicked menu item text
			 * @example 
			 *&lt;button id="sbutton"&gt;File&lt;/button&gt;
	*&lt;ul id="target"&gt;
	*    &lt;li&gt;&lt;a href="#"&gt;Open..&lt;/a&gt;&lt;/li&gt;
	*    &lt;li&gt;&lt;a href="#"&gt;Save&lt;/a&gt;&lt;/li&gt;
	*    &lt;li&gt;&lt;a href="#"&gt;Delete&lt;/a&gt;&lt;/li&gt;
	*&lt;/ul&gt;
	* &lt;script&gt;
			 * //itemMouseOver event for split button
             * $("#sbutton"). ejSplitButton ({
             	targetID: "target",width:100,
             *      itemMouseOver: function (args) {}
             * });
               * &lt;/script&gt;
			 * @memberof ejSplitButton
			 * @instance
			 */
            itemMouseOver: null,
            /**     
			 * Fires when a menu item is Hovered out successfully
			 * @event
			 * @name ejSplitButton#itemMouseOut		
			 * @param {Object} argument Event parameters from split button     
			 * @param {boolean}  argument.cancel returns the cancel option value
			 * @param {object}  argument.model returns the split button model
			 * @param {string}  argument.type returns the name of the event
			 * @param {object} argument.event.element  returns the clicked menu item element
			 * @param {object} argument.event returns the event
			 * @param {String}  argument.event.ID  return the menu item id
			 * @param {String}  argument.event.Text  return the clicked menu item text
			 * @example 
			 *&lt;button id="sbutton"&gt;File&lt;/button&gt;
	*&lt;ul id="target"&gt;
	*    &lt;li&gt;&lt;a href="#"&gt;Open..&lt;/a&gt;&lt;/li&gt;
	*    &lt;li&gt;&lt;a href="#"&gt;Save&lt;/a&gt;&lt;/li&gt;
	*    &lt;li&gt;&lt;a href="#"&gt;Delete&lt;/a&gt;&lt;/li&gt;
	*&lt;/ul&gt;
	* &lt;script&gt;
			 * //itemMouseOut event for split button
             * $("#sbutton"). ejSplitButton ({
             	targetID: "target",width:100,
             *      itemMouseOut: function (args) {}
             * });
               * &lt;/script&gt;
			 * @memberof ejSplitButton
			 * @instance
			 */
            itemMouseOut: null,
            /**     
            * Fires when a menu item is clicked successfully
            * @event
            * @name ejSplitButton#itemSelected		
            * @param {Object} argument Event parameters from split button     
            * @param {boolean}  argument.cancel returns the cancel option value
            * @param {object}  argument.model returns the split button model
            * @param {string}  argument.type returns the name of the event
            * @param {object} argument.event.element  returns the clicked menu item element
            * @param {object} argument.selectedItem - returns the selected item
            * @param {String}  argument.event.menuId  return the menu id
            * @param {String}  argument.event.menuText  return the clicked menu item text
            * @example 
            *&lt;button id="sbutton"&gt;File&lt;/button&gt;
   *&lt;ul id="target"&gt;
   *    &lt;li&gt;&lt;a href="#"&gt;Open..&lt;/a&gt;&lt;/li&gt;
   *    &lt;li&gt;&lt;a href="#"&gt;Save&lt;/a&gt;&lt;/li&gt;
   *    &lt;li&gt;&lt;a href="#"&gt;Delete&lt;/a&gt;&lt;/li&gt;
   *&lt;/ul&gt;
   * &lt;script&gt;
            * //itemSelected event for split button
            * $("#sbutton"). ejSplitButton ({
               targetID: "target",width:100,
            *      itemSelected: function (args) {}
            * });
              * &lt;/script&gt;
            * @memberof ejSplitButton
            * @instance
            */
            itemSelected: null,
            /**     
            * Fires when the Split Button is destroyed successfully
            * @event
            * @name ejSplitButton#destroy		
            * @param {Object} argument Event parameters from  split button     
            * @param {boolean}  argument.cancel returns the cancel option value
            * @param {object}  argument.model returns the split button model
            * @param {string}  argument.type returns the name of the event
            * @example 
            *&lt;button id="sbutton"&gt;File&lt;/button&gt;
   *&lt;ul id="target"&gt;
   *    &lt;li&gt;&lt;a href="#"&gt;Open..&lt;/a&gt;&lt;/li&gt;
   *    &lt;li&gt;&lt;a href="#"&gt;Save&lt;/a&gt;&lt;/li&gt;
   *    &lt;li&gt;&lt;a href="#"&gt;Delete&lt;/a&gt;&lt;/li&gt;
   *&lt;/ul&gt;
   * &lt;script&gt;
            * //destroy event for split button
            * $("#sbutton"). ejSplitButton ({
               targetID: "target",width:100,
            *      destroy: function (args) {}
            * });
            * &lt;/script&gt;
            * @memberof ejSplitButton
            * @instance
            */
            destroy: null
        },
        //Data Types
        dataTypes: {
            size: "string",
            enabled: "boolean",
            showRoundedCorner: "boolean",
            text: "string",
            contentType: "enum",
            imagePosition: "enum",
            targetID: "string",
            prefixIcon: "string",
            suffixIcon: "string",
            cssClass: "string",
            enableRTL: "boolean"
        },
        // sample public function
        /**
        * To disable the split button  
		* @return jQuery
        * @example 
		 *&lt;button id="sbutton"&gt;File&lt;/button&gt;
		*&lt;ul id="target"&gt;
		*    &lt;li&gt;&lt;a href="#"&gt;Open..&lt;/a&gt;&lt;/li&gt;
		*    &lt;li&gt;&lt;a href="#"&gt;Save&lt;/a&gt;&lt;/li&gt;
		*    &lt;li&gt;&lt;a href="#"&gt;Delete&lt;/a&gt;&lt;/li&gt;
		*&lt;/ul&gt;
		* &lt;script&gt;
		* //To Disable the Split Button control.		
		* $("#sbutton").ejSplitButton({targetID: "target",width:100});
		* var SptObj=$("#sbutton").data("ejSplitButton");
        * SptObj.disable();
		* &lt;/script&gt;
		* @example 
		 *&lt;button id="sbutton"&gt;File&lt;/button&gt;
		*&lt;ul id="target"&gt;
		*    &lt;li&gt;&lt;a href="#"&gt;Open..&lt;/a&gt;&lt;/li&gt;
		*    &lt;li&gt;&lt;a href="#"&gt;Save&lt;/a&gt;&lt;/li&gt;
		*    &lt;li&gt;&lt;a href="#"&gt;Delete&lt;/a&gt;&lt;/li&gt;
		*&lt;/ul&gt;
		* &lt;script&gt;
		* //To Disable the Split Button control.		
		* $("#sbutton").ejSplitButton({targetID: "target",width:100});
		* $("#sbutton").ejSplitButton("disable");
		* &lt;/script&gt;
		*@memberof ejSplitButton
		* @instance
        */
        disable: function () {
            /// <summary>This function is  used to Disable the Split button Object</summary>
            this.element.addClass("e-disable");
            this.dropbutton.addClass("e-disable").attr("aria-disabled", true);
            this.model.enabled = false;
        },
        /**
        * To Enable the split button  
		* @return jQuery
        * @example 
		 *&lt;button id="sbutton"&gt;File&lt;/button&gt;
		*&lt;ul id="target"&gt;
		*    &lt;li&gt;&lt;a href="#"&gt;Open..&lt;/a&gt;&lt;/li&gt;
		*    &lt;li&gt;&lt;a href="#"&gt;Save&lt;/a&gt;&lt;/li&gt;
		*    &lt;li&gt;&lt;a href="#"&gt;Delete&lt;/a&gt;&lt;/li&gt;
		*&lt;/ul&gt;
		* &lt;script&gt;
		* //To Enable the Split Button control.		
		* $("#sbutton").ejSplitButton({targetID: "target",width:100});
		* var SptObj=$("#sbutton").data("ejSplitButton");
        * SptObj.enable();
		* &lt;/script&gt;
		* @example 
		 *&lt;button id="sbutton"&gt;File&lt;/button&gt;
		*&lt;ul id="target"&gt;
		*    &lt;li&gt;&lt;a href="#"&gt;Open..&lt;/a&gt;&lt;/li&gt;
		*    &lt;li&gt;&lt;a href="#"&gt;Save&lt;/a&gt;&lt;/li&gt;
		*    &lt;li&gt;&lt;a href="#"&gt;Delete&lt;/a&gt;&lt;/li&gt;
		*&lt;/ul&gt;
		* &lt;script&gt;
		* //To Enable the Split Button control.		
		* $("#sbutton").ejSplitButton({targetID: "target",width:100});
		* $("#sbutton").ejSplitButton("enable");
		* &lt;/script&gt;
		*@memberof ejSplitButton
		* @instance
        */
        enable: function () {
            /// <summary>This function is  used to Enable the Split button Object</summary>
            this.element.removeClass("e-disable");
            this.dropbutton.removeClass("e-disable").attr("aria-disabled", false);
            this.model.enabled = true;
        },
        // constructor function
        /**
         * Create the split button widget
		 * @private
         */
        _init: function () {
            this._initialize();
            this._controlStatus(this.model.enabled);
            this._wireEvents();
        },
        // all events bound using this._on will be unbind automatically
        /**
        * destroy the split button widget
		* all events bound using this._on will be unbind automatically and bring the control to pre-init state.
		* @alias destroy
		* @return jQuery
        * @example 
		 *&lt;button id="sbutton"&gt;File&lt;/button&gt;
		*&lt;ul id="target"&gt;
		*    &lt;li&gt;&lt;a href="#"&gt;Open..&lt;/a&gt;&lt;/li&gt;
		*    &lt;li&gt;&lt;a href="#"&gt;Save&lt;/a&gt;&lt;/li&gt;
		*    &lt;li&gt;&lt;a href="#"&gt;Delete&lt;/a&gt;&lt;/li&gt;
		*&lt;/ul&gt;
		* &lt;script&gt;
		* //To Destroy the Split Button control.		
		* $("#sbutton").ejSplitButton({targetID: "target",width:100});
		* var SptObj=$("#sbutton").data("ejSplitButton");
        * SptObj.destroy();
		* &lt;/script&gt;
		* @example 
		 *&lt;button id="sbutton"&gt;File&lt;/button&gt;
		*&lt;ul id="target"&gt;
		*    &lt;li&gt;&lt;a href="#"&gt;Open..&lt;/a&gt;&lt;/li&gt;
		*    &lt;li&gt;&lt;a href="#"&gt;Save&lt;/a&gt;&lt;/li&gt;
		*    &lt;li&gt;&lt;a href="#"&gt;Delete&lt;/a&gt;&lt;/li&gt;
		*&lt;/ul&gt;
		* &lt;script&gt;
		* // to destroy the button		
		* $("#sbutton").ejSplitButton({targetID: "target",width:100});	
		* $("#sbutton").ejSplitButton("destroy");
		* &lt;/script&gt;
		* @memberof ejSplitButton
		* @instance
         */
        _destroy: function () {
            /// <summary>This function is  used to destroy the Split button Object</summary>
            this.element.removeClass(this.model.cssClass + " e-select e-corner-left e-btn e-disable e-split-btn");
            /*Remove dropbtn*/
            this.dropbutton.remove();
            $("#" + this.model.targetID).ejMenu('destroy');
            //this has to be worked out in Menu
            $("#" + this.model.targetID).insertAfter(this.element);
            $("#" + this.model.targetID).show();
        },
        /**
         * To configure the properties at runtime using SetModel		
		 * @private
         */
        _setModel: function (options) {
            if (!(this.model.enabled) && ej.isNullOrUndefined(options["enabled"])) return false;
            var option;
            for (option in options) {
                switch (option) {
                    case "size":
                        this._setSize(options[option]);
                        break;
                    case "width":
                        this._splitbtnWidth(options[option]);
                        break;
                    case "height":
                        this._splitbtnHeight(options[option]);
                        break;
                    case "contentType":
                        this._setContentType(options[option]);
                        break;
                    case "imagePosition":
                        this._setImagePosition(options[option]);
                        break;
                    case "text":
                        this._setText(options[option]);
                        break;
                    case "prefixIcon":
                        this._setMajorIcon(options[option]);
                        break;
                    case "suffixIcon":
                        this._setMinorIcon(options[option]);
                        break;
                    case "enabled":
                        this._controlStatus(options[option]);
                        break;
                    case "targetID":
                        this._setTargetId(options[option]);
                        break;
                    case "showRoundedCorner":
                        this._roundedCorner(options[option]);
                        break;
                    case "cssClass":
                        this._setSkin(options[option]);
                        break;
                    case "enableRTL":
                        this._setRTL(options[option]);
                        $("#" + this.model.targetID).ejMenu({ enableRTL: options[option] });
                        break;
                    case "itemMouseOver":
                        $("#" + this.model.targetID).ejMenu({ mouseOver: options[option] });
                        break;
                    case "itemMouseOut":
                        $("#" + this.model.targetID).ejMenu({ mouseOut: options[option] });
                        break;
                    case "itemSelected":
                        $("#" + this.model.targetID).ejMenu({ click: options[option] });
                        break;
                }
            }
        },
        /**
         * To configure the button text		
		 * @private
         */
        _setText: function (val) {
            /// <summary>This will set Text property of split button </summary>
            if (this.model.contentType == ej.ContentType.TextOnly) {
                this.element.html(val);
            } else {
                this.textspan.html(val);
            }
        },
        /**
         * To configure button's prefix image		
		 * @private
         */
        _setMajorIcon: function (val) {
            /// <summary>This will set Major Icon property of split button </summary>
            this.majorimgtag.removeClass(this.model.prefixIcon);
            this.majorimgtag.addClass(val);
        },
        /**
         * To configure button's suffix image		
		 * @private
         */
        _setMinorIcon: function (val) {
            /// <summary>This will set Minor Icon property of split button </summary>
            this.minorimgtag.removeClass(this.model.suffixIcon);
            this.minorimgtag.addClass(val);
        },
        _setTargetId: function (val) {
            this.model.targetID = val;
            this._renderContxtMenu();
        },
        /**
         * To configure button contents type		
		 * @private
         */
        _setContentType: function (val) {
            if (val != this.model.contentType) {
                this.element.empty();
                this.model.contentType = val;
                this._renderButtonContent();
            }
        },
        /**
         * To configure images position inside the button		
		 * @private
         */
        _setImagePosition: function (val) {
            if ((this.model.contentType == ej.ContentType.TextAndImage) && (val != this.model.imagePosition)) {
                this.element.empty();
                this.model.imagePosition = val;
                this._renderButtonContent();
            }
        },
        /**
         * To enable or disable the Right to Left behaviour 		
		 * @private
         */
        _setRTL: function (val) {
            if (val) {
                this.splitwrap.addClass("e-rtl e-btnrtl");
                this.dropbutton.css("border-right", "medium none");
                this.dropbutton.css("border-left-style", "");
            } else {
                this.splitwrap.removeClass("e-rtl e-btnrtl");
                this.dropbutton.css("border-left", "medium none");
                this.dropbutton.css("border-right-style", "");
            }
            this.model.enableRTL = val;
            this._roundedCorner(this.model.showRoundedCorner);
        },
        /**
         * To enable or disable the rounded corner behaviour 		
		 * @private
         */
        _roundedCorner: function (value) {
            if (value) {
                if (this.model.enableRTL) {
                    this.element.addClass("e-corner-right");
                    this.dropbutton.addClass("e-corner-left");

                    this.element.removeClass('e-corner-left');
                    this.dropbutton.removeClass('e-corner-right');
                }
                else {
                    this.element.addClass("e-corner-left");
                    this.dropbutton.addClass("e-corner-right");
                    this.element.removeClass("e-corner-right");
                    this.dropbutton.removeClass("e-corner-left");
                }
            }
            else {
                if (this.model.enableRTL) {
                    this.element.removeClass("e-corner-right");
                    this.dropbutton.removeClass("e-corner-left");
                }
                else {
                    this.element.removeClass('e-corner-left');
                    this.dropbutton.removeClass('e-corner-right');
                }
            }
        },
        /**
         * To configure enable/disable the splitbutton	
		 * @private
         */
        _controlStatus: function (value) {
            //disable status
            if (!value) {
                this.disable();
            } else {
                this.enable();
            }
        },
        /**
        * To configure the custom theme for button using cssClass property		
        * @private
        */
        _setSkin: function (skin) {
            this.element.removeClass(this.model.cssClass);
            this.dropbutton.removeClass(this.model.cssClass);
            this.element.addClass(skin);
            this.dropbutton.addClass(skin);
            $('#' + this.model.targetID).ejMenu('option', 'skin', skin);
        },
        /**
         * To initialize the button		
		 * @private
         */
        _initialize: function () {
            /// <summary>This function is  used to Initialize the Split button Object</summary>
            if (this.element.is("button")) {
                this._render();
            } else {
                this.element.removeClass("e-splitbutton");//need to change in src level
            }
            this._timeout = null;
        },
        /*Render Section For DifferentTypes*/
        /**
         * Render Section For DifferentTypes		
		 * @private
         */
        _render: function () {
            /// <summary>This function is  used to Render the Split button Object</summary>
            this.element.addClass(this.model.cssClass + " e-btn e-select e-split-btn").attr("role", "button");
            if ((this.model.text == null) || (this.model.text == "")) {
                this.model.text = this.element.text();
            }
            else
                this.element.attr("aria-describedby", this.model.text);
            this.element.empty();
            /*Creating Wrapper*/
            this.splitwrap = ej.buildTag('span.e-split e-widget');
            this.splitwrap.insertBefore(this.element);
            this.innerWrap = ej.buildTag('span.e-in-wrap e-box e-padding');
            this.splitwrap.append(this.innerWrap);
            this.wrapper = this.splitwrap;
            /*DropDown Button*/
            this.dropbutton = ej.buildTag('button.e-split-btn e-btn e-select ' + this.model.cssClass + ' e-drp-btn#' + this.element[0].id + 'drpbtn', "", {}, { type: "button" });
            this.dropdownimg = ej.buildTag('span.e-icon e-down-arrow');
            this.btnimgwrap = ej.buildTag('div.e-split-btn-div');
            this.btnimgwrap.append(this.dropdownimg);
            this.dropbutton.append(this.btnimgwrap);
            this.dropbutton.insertAfter(this.element);
            if (this.model.contentType == ej.ContentType.TextOnly) {
                this.dropbutton.addClass("e-btn-txt");
            } else {
                this.dropbutton.addClass("e-rht-btn");
            }

            /*width and height setting*/
            this._setSize(this.model.size);
            this.element.addClass("e-left-btn");
            this._renderButtonContent();
            this.innerWrap.append(this.element, this.dropbutton);
            /*Rendering Context menu*/
            this._renderContxtMenu();
            this._roundedCorner(this.model.showRoundedCorner);
            this._setRTL(this.model.enableRTL);
        },
        /**
         * Render section for button content
		 * @private
         */
        _renderButtonContent: function () {
            /*Image and Text*/
            this.textspan = ej.buildTag('span.e-btntxt', this.model.text);
            this.majorimgtag = ej.buildTag('span.e-icon ' + this.model.prefixIcon);
            this.minorimgtag = ej.buildTag('span.e-icon ' + this.model.suffixIcon);
            this.imgtxtwrap = ej.buildTag('div');
            /*Rendering Option*/
            if (this.model.contentType == ej.ContentType.TextAndImage) {
				switch (this.model.imagePosition) {
                    case ej.ImagePosition.ImageRight:
                    	this.imgtxtwrap.append(this.textspan, this.majorimgtag);
						break;
					case ej.ImagePosition.ImageLeft:
                    	this.imgtxtwrap.append(this.majorimgtag, this.textspan);
						break;
					case ej.ImagePosition.ImageBottom:	
						this.majorimgtag.attr("style","display:inherit");
                    	this.imgtxtwrap.append(this.textspan, this.majorimgtag);
						break;
                	case ej.ImagePosition.ImageTop:
						this.majorimgtag.attr("style","display:inherit");
                    	this.imgtxtwrap.append(this.majorimgtag, this.textspan);
						break;
                }
				this.element.append(this.imgtxtwrap);
            } else if (this.model.contentType == ej.ContentType.ImageTextImage) {
                this.imgtxtwrap.append(this.majorimgtag, this.textspan, this.minorimgtag);
                this.element.append(this.imgtxtwrap);
            } else if (this.model.contentType == ej.ContentType.ImageBoth) {
                this.imgtxtwrap.append(this.majorimgtag, this.minorimgtag);
                this.element.append(this.imgtxtwrap);
            } else if (this.model.contentType == ej.ContentType.ImageOnly) {
                this.imgtxtwrap.append(this.majorimgtag);
                this.element.append(this.imgtxtwrap);
            } else {
                this.element.addClass("e-txt");
                this.element.text(this.model.text);
            }
        },
        /*Height of the splitButton*/
        /**
         * To configure size of the button		
		 * @private
         */
        _setSize: function (val) {
            var ht = this.model.height, wd = this.model.width;
            switch (val) {
                case "mini":
                    if (ht == "") ht = "28px";
                    if (wd == "") wd = "63px";
                    this._splitbtnHeight(ht);
                    this._splitbtnWidth(wd);
                    this.element.css('font-size', '12px');
                    break;
                case "small":
                    if (ht == "") ht = "32px";
                    if (wd == "") wd = "89px";
                    this._splitbtnHeight(ht);
                    this._splitbtnWidth(wd);
                    this.element.css('font-size', '13px');
                    break;
                case "medium":
                    if (ht == "") ht = "36px";
                    if (wd == "") wd = "113px";
                    this._splitbtnHeight(ht);
                    this._splitbtnWidth(wd);
                    this.element.css('font-size', '14px');
                    break;
                case "large":
                    if (ht == "") ht = "40px";
                    if (wd == "") wd = "130px";
                    this.element.css('font-size', '15px');
                    this._splitbtnHeight(ht);
                    this._splitbtnWidth(wd);
                    break;
                default:
                    this._splitbtnHeight(ht);
                    this._splitbtnWidth(wd);
                    break;
            }
        },
        /**
         * To configure height for the split button		
		 * @private
         */
        _splitbtnHeight: function (val) {
            if ((val == "") || (val == null)) val = '30px';
            this.splitwrap.css("height", val);
        },
        /*Width of the splitButton*/
        /**
         * To configure width for the split button		
		 * @private
         */
        _splitbtnWidth: function (val) {
            this.splitwrap.css("width", val);
        },
        /*Context menu Rendering*/
        /**
         * Render section for Context menu Rendering	
		 * @private
         */
        _renderContxtMenu: function () {
            $("#" + this.model.targetID).ejMenu({
                menuType: ej.MenuType.ContextMenu,
                openOnClick: false,
                contextMenuTarget: "",
                fields: this.model.fields,
                showArrow: true,
                cssClass: this.model.cssClass,
                enableRTL: this.model.enableRTL,
                mouseover: $.proxy(this._itemMouseOver, this),
                mouseout: $.proxy(this._itemMouseOut, this),
                click: $.proxy(this._itemClick, this)
            });
        },
        /**
         * Section For handle the item click event for the split button
		 * @private
         */
        _itemClick: function (args) {
            args = { status: this.model.enabled,ID: args.ID, text: args.text};
             this._trigger("itemSelected", args);
        },
        /**
         * Section For handle the item MouseOver event for the split button
		 * @private
         */
        _itemMouseOver: function (args) {
            this._trigger("itemMouseOver", args);
        },
        /**
         * Section For handle the item MouseOut event for the split button
		 * @private
         */
        _itemMouseOut: function (args) {
            this._trigger("itemMouseOut", args);
        },
        /*-----------------------Event Handlers -----------------------------------------*/
        /**
         * Wiring the events to button control		
		 * @private
         */
        _wireEvents: function () {
            /// <summary>This function is  used to Apply Events in the Split button Object</summary>
            /*element*/
            this._on(this.element, "click", this._btnMouseClick);
            this._on(this.element, "mousedown", this._btnMouseDown);

            /*DrpBTN*/
            this._on(this.dropbutton, "click", this._btnMouseClick);
            /*DocClk*/
            this._on($(document), "mousedown", this._docrhtclk);
        },
        /*EVENT functionalities*/
        /**
         * Section For handle the click event
		 * @private
         */
        _btnMouseClick: function (e) {
            var args, targetElement;
            if (!$(e.currentTarget).hasClass("e-disable")) {
                if (e.currentTarget.id != this.element[0].id + "drpbtn") {
                    args = { status: this.model.enabled };
                    this._trigger("click", args);
                } else {
                    /*Context Menu Functionality*///need to write enableRTL
                    //the getXYpos method is created to calculate position for context menu
                    targetElement = this.dropbutton;
                    position = this._getXYpos(e);
                    if (this.contstatus) {
                        this._hidecontext(e);
                    } else {
                        if (this.model.enableRTL) {
                            $("#" + this.model.targetID).ejMenu('show', position.x - ($("#" + this.model.targetID).width() - this.dropbutton.width()), position.y, targetElement, e);
                            this.contstatus = true;
                        } else {
                            $("#" + this.model.targetID).ejMenu('show', position.x, position.y, targetElement, e);
                            this.contstatus = true;
                        }
                        this.element.bind("click", $.proxy(this._hidecontext, this));
                    }
                }
                if (this.contstatus)
                    this._on($(document), "mousedown", this._documentClick);
            }
        },
        _getXYpos:function(e)
        {
            var btnpos, btnposx, btnposy, poscur = 1;            
            btnpos = this._getOffset(this.dropbutton);
            btnposx = btnpos.left - poscur;
            btnposy = (btnpos.top + this.dropbutton.outerHeight()) - poscur;
            return { x: btnposx, y: btnposy };
        },
        _getOffset: function (ele) {
            var pos = ele.offset();
            if ($("body").css("position") != "static") {
                var bodyPos = $("body").offset();
                pos.left -= bodyPos.left;
                pos.top -= bodyPos.top;
            }
            return pos;
        },

        /**
         * Section For handle mouse down action.
		 * @private
         */
        _btnMouseDown: function (e) {
            if (!$(e.currentTarget).hasClass("e-disable")) {
                this._docrhtclk(e);
            }
        },
        /**
         * Section For handle hidecontext action.
		 * @private
         */
        _hidecontext: function (e) {
            if (!$(e.target).is(this.element) && !$(e.target).is(this.textspan) &&
					  !$(e.target).is(this.imgtxtwrap) && !$(e.target).is(this.splitwrap) && !$(e.target).parents().is($("#" + this.model.targetID))) {
                $("#" + this.model.targetID).ejMenu('hide', e);
                this.contstatus = false;
                this.element.unbind("click", $.proxy(this._hidecontext, this));
                this._off($(document), "mousedown", this._documentClick);
            }
        },
        /**
         * Section For handle on document click action.
		 * @private
         */
        _documentClick: function (e) {
            if (!$(e.target).is(this.element) && !$(e.target).is(this.textspan) &&
            !$(e.target).is(this.imgtxtwrap) && !$(e.target).is(this.splitwrap) && !$(e.target).parents().is($("#" + this.model.targetID))) {
                $("#" + this.model.targetID).ejMenu('hide', e);
                this.contstatus = false;
                this._off($(document), "mousedown", this._documentClick);
            }
        },
        /**
         * Section For handle document right click action.
		 * @private
         */
        _docrhtclk: function (e) {
            var isRightClick, targetElement;
            isRightClick = false;
            if (e.button) {
                isRightClick = (e.button == 2);
            } else if (e.which) {
                isRightClick = (e.which == 3); //for Opera
            }
            targetElement = e.target;
            if (isRightClick) {
                e.preventDefault();
            }
        },
        /**/
    });
    /**
	 * Enum for button content mode.	 
	 * @enum {string}
	 * @global 
	 */
    ej.ContentType = { TextOnly: "textonly", ImageOnly: "imageonly", ImageBoth: "imageboth", TextAndImage: "textandimage", ImageTextImage: "imagetextimage" };
    /**
	 * Enum for Button positioning	 
	 * @enum {string}
	 * @global 
	 */
    ej.ImagePosition = { ImageRight: "imageright", ImageLeft: "imageleft", ImageTop: "imagetop", ImageBottom: "imagebottom" };
    /**
	 * Enum for various button sizes	 
	 * @enum {string}
	 * @global 
	 */
    ej.ButtonSize = { Mini: "mini", Small: "small", Medium: "medium", Large: "large" };
})(jQuery, Syncfusion);;
/*!
*  filename: ej.togglebutton.js
*  version : 12.1
*  Copyright Syncfusion Inc. 2001 - 2013. All rights reserved.
*  Use of this code is subject to the terms of our license.
*  A copy of the current license can be obtained at any time by e-mailing
*  licensing@syncfusion.com. Any infringement will be prosecuted under
*  applicable laws. 
*/

/**
* @fileOverview Plugin to style the Html Button elements
* @copyright Copyright Syncfusion Inc. 2001 - 2013. All rights reserved.
*  Use of this code is subject to the terms of our license.
*  A copy of the current license can be obtained at any time by e-mailing
*  licensing@syncfusion.com. Any infringement will be prosecuted under
*  applicable laws. 
* @version 12.1 
* @author <a href="mailto:licensing@syncfusion.com">Syncfusion Inc</a>
*/
(function ($, ej, undefined) {
	
	/**
    * @namespace ej
	* @class ejToggleButton
	* @requires jQuery
	* @requires ej.togglebutton.js
    * @requires ej.checkbox.js
	* @classdesc The Toggle Button allows you to perform the toggle option by using checked and unchecked state. This Toggle Button can be helpful to user to check their states. The Toggle Button control displays both text and images.
	* @example 
	* &lt;input id="toggle" type="checkbox" /&gt; <br> 	
	* &lt;script&gt; <br>
	* // Create ToggleButton <br>
    * $('#toggle').ejToggleButton({defaultText:"Play",activeText:"Pause"}); 	<br>	
	* &lt;/script&gt; <br>
	*/    

    // Example plugin creation code
    // ejSample is the plugin name 
    // "ej.Sample" is "namespace.className" will hold functions and properties

    ej.widget("ejToggleButton", "ej.ToggleButton", {
        // widget element will be automatically set in this
        element: null,

        // user defined model will be automatically set in this
        model: null,
        validTags: ["input"],
        _setFirst: false,
        //Root Css Class
        _rootCSS: "e-togglebutton",

        // default model
        defaults: {
            /// <summary>This Contains default property of toggle button </summary>
			/**		
			* Specifies the size of the ToggleButton.	See {@link ButtonSize}
			* @default ej.ButtonSize.Normal
			* @type {enum}
			* @example 
			* &lt;input id="toggle" type="checkbox" /&gt; <br> 	
			* &lt;script&gt; <br>
			* //To set size API value during initialization  
			* 	$("#toggle").ejToggleButton({defaultText:"Play",activeText:"Pause",size: ej.ButtonSize.Mini});					 			 	
			* &lt;/script&gt; <br>
			* @memberof ejToggleButton
			* @instance
			*/		
            size: "normal",           
            /**		
			* Specifies the type of the ToggleButton.	See {@link ButtonType}
			* @default ej.ButtonType.Button
			* @type {enum}
			* @example 
			* &lt;input id="toggle" type="checkbox" /&gt; <br> 	
			* &lt;script&gt; <br>
			* //To set type API value during initialization  
			* 	$("#toggle").ejToggleButton({defaultText:"Play",activeText:"Pause",type:ej.ButtonType.Submit});					 			 	
			* &lt;/script&gt; <br>
			* @memberof ejToggleButton
			* @instance
			*/
            type: "button",
			/**		
			* Specifies the width of the ToggleButton.
			* @default 100pixel
			* @type {String}
			* @example 
			* &lt;input id="toggle" type="checkbox" /&gt; <br> 	
	* &lt;script&gt; <br>
			* //To set width API value during initialization  
			* 	$("#toggle").ejToggleButton({defaultText:"Play",activeText:"Pause",width: "100px" });						
				
	* &lt;/script&gt; <br>			
			* @memberof ejToggleButton
			* @instance
			*/
			width: "",
			
			/**		
			* Specifies the height of the ToggleButton.
			* @default 28pixel
			* @type {String}
			* @example 
			* &lt;input id="toggle" type="checkbox" /&gt; <br> 	
	* &lt;script&gt; <br>
			* //To set height API value during initialization  
			* 	$("#toggle").ejToggleButton({defaultText:"Play",activeText:"Pause",height: "28px" });						
		
	* &lt;/script&gt;		
			* @memberof ejToggleButton
			* @instance
			*/
            height: "",
			
            /**		
			* Specifies the state of the ToggleButton.
			* @default true
			* @type {boolean}
			* @example 
			* &lt;input id="toggle" type="checkbox" /&gt; <br> 	
	* &lt;script&gt; <br>

			* //To set enabled API value during initialization  
			* 	$("#toggle").ejToggleButton({defaultText:"Play",activeText:"Pause",enabled: true });						
					
			* &lt;/script&gt; <br>		
			* @memberof ejToggleButton
			* @instance
			*/
            enabled: true,
			/**		
			* Specifies the toggleState of the ToggleButton.
			* @default false
			* @type {boolean}
			* @example 
			* &lt;input id="toggle" type="checkbox" /&gt; <br> 	
	* &lt;script&gt; <br>
			* //To set toggleState API value during initialization  
			* 	$("#toggle").ejToggleButton({defaultText:"Play",activeText:"Pause",toggleState: false });	
            * &lt;/script&gt; <br>	
			* @memberof ejToggleButton
			* @instance
			*/
            toggleState: false,
			/**		
			* Specifies the defaultText of the ToggleButton.
			* @default null
			* @type {string}
			* @example 
			* &lt;input id="toggle" type="checkbox" /&gt; <br> 	
	* &lt;script&gt; <br>
			* //To set defaultText API value during initialization  
			* 	$("#toggle").ejToggleButton({defaultText:"Play",activeText:"Pause"});						
			* &lt;/script&gt; <br>		
			* @memberof ejToggleButton
			* @instance
			*/
			defaultText: null,
			
			/**		
			* Specifies the preventToggle of the ToggleButton.
			* @default false
			* @type {boolean}
			* @example 
			* &lt;input id="toggle" type="checkbox" /&gt; <br> 	
	* &lt;script&gt; <br>
			* //To set preventToggle API value during initialization  
			* 	$("#toggle").ejToggleButton({defaultText:"Play",activeText:"Pause",preventToggle: false});						
			* &lt;/script&gt; <br>		
			* @memberof ejToggleButton
			* @instance
			*/
			preventToggle: false,
			/**		
			* Specifies the activeText of the ToggleButton.
			* @default null
			* @type {string}
			* @example 
			* &lt;input id="toggle" type="checkbox" /&gt; <br> 	
	        * &lt;script&gt; <br>
			* //To set activeText API value during initialization  
			* 	$("#toggle").ejToggleButton({defaultText:"Play",activeText:"Pause"});							
	        * &lt;/script&gt; <br>	
			* @memberof ejToggleButton
			* @instance
			*/
            activeText: null,
			/**		
			* Specifies the contentType of the ToggleButton. See {@link ContentType}
			* @default ej.ContentType.TextOnly
			* @type {enum}
			* @example 
			* &lt;input id="toggle" type="checkbox" /&gt; <br> 	
	        * &lt;script&gt; <br>
			* // Set the button contentType on initialization. 			
			* 	$("#toggle").ejToggleButton({ defaultText:"Play",activeText:"Pause",contentType : ej.ContentType.TextOnly });			 
			* &lt;/script&gt; <br>
			* @memberof ejToggleButton
			* @instance
			*/
            contentType: "textonly",
			/**		
			*  Specifies the image position of the ToggleButton. This image position is applicable only with the textandimage contentType property. The images can be positioned in both imageLeft and imageRight options. See {@link imagePositions}
			* @default ej.ImagePosition.ImageLeft
			* @type {enum}
			* @example 
			* &lt;input id="toggle" type="checkbox" /&gt; <br> 	
	* &lt;script&gt; <br>
			* // Set the image position for toggle button during initialization. 			
			* 	$("#toggle").ejToggleButton(
            * {
            *    contentType: ej.ContentType.TextAndImage,
            *    imagePosition: ej.ImagePosition.ImageRight,
            *    defaultText:"Play",activeText:"Pause",
            *    defaultPrefixIcon: "e-mediaplay e-uiLight",
            *    activePrefixIcon: "e-mediapause e-uiLight"
            * });			
			* &lt;/script&gt; <br>
			* @memberof ejToggleButton
			* @instance
			*/
            imagePosition: "imageleft",
			/**		
			* Specify the rounded corner to Togglebutton
			* @default false
			* @type {boolean}
			* @example 
			* &lt;input id="toggle" type="checkbox" /&gt; <br> 	
	* &lt;script&gt; <br>
			* // Set the rounded corner during initialization. 			
			* 	$("#toggle").ejToggleButton({ defaultText:"Play",activeText:"Pause",showRoundedCorner : true });			 
			* &lt;/script&gt; <br>
			* @memberof ejToggleButton
			* @instance
			*/
            showRoundedCorner: false,
			/**					
			* Specify the enablePersistence to Togglebutton to save current model value to browser cookies for state maintains
			* @default false
			* @type {boolean}
			* @example 
			* &lt;input id="toggle" type="checkbox" /&gt; <br> 	
	* &lt;script&gt; <br>
			* // Set the rounded corner during initialization. 			
			* 	$("#toggle").ejToggleButton({ defaultText:"Play",activeText:"Pause",enablePersistence : true });			 
			* &lt;/script&gt; <br>
			
			* @memberof ejToggleButton
			* @instance
			*/
            enablePersistence: false,
			/**		
			* Specify the CSS class to toggle button to achieve custom theme.
			* @default ""
			* @type {string}
			* @example 
			* &lt;input id="toggle" type="checkbox" /&gt; <br> 	
	* &lt;script&gt; <br>
			* // Set the CSS class during initialization. 			
			* 	$("#toggle").ejToggleButton({defaultText:"Play",activeText:"Pause",cssClass : "gradient-lime" });			 
			* &lt;/script&gt; <br>
			* @memberof ejToggleButton
			* @instance
			*/
            cssClass: "",
			/**		
			* Specify the defaultPrefixIcon to toggle button to achieve custom theme.
			* @default ""
			* @type {string}
			* @example 
			* &lt;input id="toggle" type="checkbox" /&gt; <br> 	
	* &lt;script&gt; <br>
			* // Set the defaultPrefixIcon during initialization. 			
			* 	$("#toggle").ejToggleButton({ 
            *    defaultText:"Play",activeText:"Pause",
            *    contentType: "textandimage",
            *    defaultPrefixIcon: "e-mediaplay e-uiLight",
            *    activePrefixIcon: "e-mediapause e-uiLight",
            *  });			
			* &lt;/script&gt; <br>
			* @memberof ejToggleButton
			* @instance
			*/
            defaultPrefixIcon: null,
			/**		
			* Specify the defaultSuffixIcon to toggle button to achieve custom theme.
			* @default ""
			* @type {string}
			* @example 
			* &lt;input id="toggle" type="checkbox" /&gt; <br> 	
			* &lt;script&gt; <br>
			* // Set the defaultSuffixIcon during initialization. 			
			*  $("#toggle").ejToggleButton({ 
            *    defaultText:"Play",activeText:"Pause",
            *    contentType: "textandimage",
            *    defaultSuffixIcon: "e-mediaplay e-uiLight",
            *    activeSuffixIcon: "e-mediapause e-uiLight",
            *  });			
			* &lt;/script&gt; <br>
			* @memberof ejToggleButton
			* @instance
			*/
            defaultSuffixIcon: null,
			/**		
			* Specify the activePrefixIcon to toggle button to achieve custom theme.
			* @default ""
			* @type {string}
			* @example 
			*
			* &lt;input id="toggle" type="checkbox" /&gt; <br> 	
			* &lt;script&gt; <br>
			* // Set the activePrefixIcon during initialization. 			
			* 	$("#toggle").ejToggleButton({ 
            *    defaultText:"Play",activeText:"Pause",
            *    contentType: "textandimage",
            *    defaultPrefixIcon: "e-mediaplay e-uiLight",
            *    activePrefixIcon: "e-mediapause e-uiLight",
            *  });			
			* &lt;/script&gt; 
			* @memberof ejToggleButton
			* @instance
			*/
            activePrefixIcon: null,
			/**		
			* Specify the activeSuffixIcon to toggle button to achieve custom theme.
			* @default ""
			* @type {string}
			* @example 
			* &lt;input id="toggle" type="checkbox" /&gt; <br> 	
	* &lt;script&gt; <br>
			* // Set the activeSuffixIcon during initialization. 			
			* 	$("#toggle").ejToggleButton({ 
            *    contentType: "imageboth",
            *    defaultSuffixIcon: "e-mediaplay e-uiLight",
            *    activeSuffixIcon: "e-mediapause e-uiLight",
            *  });			
			* &lt;/script&gt; <br>
			* @memberof ejToggleButton
			* @instance
			*/
            activeSuffixIcon: null,
			/**		
			* Specify the Right to Left Direction to Togglebutton
			* @default false
			* @type {boolean}
			* @example 
			* &lt;input id="toggle" type="checkbox" /&gt; <br> 	
			* &lt;script&gt; <br>
			* // Set the enableRTL during initialization. 			
			* 	$("#toggle").ejToggleButton({defaultText:"Play",activeText:"Pause",enableRTL : true });			 
			* &lt;/script&gt; <br>
			* @memberof ejToggleButton
			* @instance
			*/
            enableRTL: false,
			/**     
			 * Fires when ToggleButton control is created successfully.
			 * @event
			 * @name ejToggleButton#create 	
			 * @param {Object} argument Event parameters from toggle button     
			 * @param {boolean}  argument.cancel if the event should be cancelled; otherwise, false.
			 * @param {object}  argument.model returns the toggle button model
			 * @param {string}  argument.type returns the name of the event			 
			 * @example 
			 * &lt;input id="toggle" type="checkbox" /&gt; <br> 	
	* &lt;script&gt; <br>
			 * //create event for toggle button
             * $("#toggle").ejToggleButton({
             *    defaultText:"Play",activeText:"Pause",
             *    create: function (args) {}
             * });   
             * &lt;/script&gt; <br>   
			 * @memberof ejToggleButton
			 * @instance
			 */
            create: null,
			/**     
			 * Fires when ToggleButton control is clicked successfully.
			 * @event
			 * @name ejToggleButton#click 	
			 * @param {Object} argument Event parameters from toggle button     
			 * @param {boolean}  argument.cancel if the event should be cancelled; otherwise, false.
			 * @param {boolean}  argument.isChecked return the toggle button checked state
			 * @param {object}  argument.model returns the toggle button model
			 * @param {boolean}  argument.status return the toggle button state			 
			 * @param {string}  argument.type returns the name of the event			 			
			 * @example 
			 * &lt;input id="toggle" type="checkbox" /&gt; <br> 	
	* &lt;script&gt; <br>
			 * //click event for toggle button
             * $("#toggle").ejToggleButton({
             * defaultText:"Play",activeText:"Pause",
             *    click: function (args) {}
             * }); 
             * &lt;/script&gt; <br>     
			 * @memberof ejToggleButton
			 * @instance
			 */
            click: null,
			/**     
			 * Fires when ToggleButton control state is changed successfully.
			 * @event
			 * @name ejToggleButton#change 	
			 * @param {Object} argument Event parameters from toggle button     
			 * @param {boolean}  argument.cancel if the event should be cancelled; otherwise, false.
			 * @param {boolean}  argument.isChecked return the toggle button checked state
			 * @param {object}  argument.model returns the toggle button model		
			 * @param {string}  argument.type returns the name of the event			 			
			 * @example 
			 * &lt;input id="toggle" type="checkbox" /&gt; <br> 	
	* &lt;script&gt; <br>
			 * //change event for toggle button
             * $("#toggle").ejToggleButton({
             *    defaultText:"Play",activeText:"Pause",
             *    change: function (args) {}
             * });  
             * &lt;/script&gt; <br>    
			 * @memberof ejToggleButton
			 * @instance
			 */
            change: null,
			/**     
			 * Fires when ToggleButton control is destroyed successfully.
			 * @event
			 * @name ejToggleButton#destroy 	
			 * @param {Object} argument Event parameters from toggle button     
			 * @param {boolean}  argument.cancel if the event should be cancelled; otherwise, false.
			 * @param {object}  argument.model returns the toggle button model		
			 * @param {string}  argument.type returns the name of the event			 			
			 * @example 
			 * &lt;input id="toggle" type="checkbox" /&gt; <br> 	
	* &lt;script&gt; <br>
			 * //destroy event for toggle button
             * $("#toggle").ejToggleButton({
             *    defaultText:"Play",activeText:"Pause",
             *    destroy: function (args) {}
             * });  
             * &lt;/script&gt; <br>    
			 * @memberof ejToggleButton
			 * @instance
			 */
            destroy: null
        },
		/**
         * Specify the data types for default properties 
		 * @private
         */
        //Data Types
        dataTypes: {
            size: "string",
            type: "enum",
            enabled: "boolean",
            showRoundedCorner: "boolean",
            toggleState: "boolean",
			preventToggle: "boolean",
            defaultText: "string",
            activeText: "string",
            contentType: "enum",
            imagePosition: "enum",
            defaultPrefixIcon: "string",
            defaultSuffixIcon: "string",
            activePrefixIcon: "string",
            activeSuffixIcon: "string",
            cssClass: "string",
            enableRTL: "boolean",
            enablePersistence: "boolean"
        },
		/**
        * To disable the toggle button  
		* @return jQuery
		* @example 
		* &lt;input id="toggle" type="checkbox" / &gt; <br> 
		* &lt;script&gt;
		* // Create toggle button
		* $("#toggle").ejToggleButton({defaultText:"Play",activeText:"Pause"});
		* var toggleObj = $("#toggle").data("ejToggleButton");
		* toggleObj.disable(); // disable the toggle button
		* &lt;/script&gt;
		* @example 
		* &lt;input id="toggle" type="checkbox" / &gt; <br> 
		* &lt;script&gt;
		* // disable the toggle button
		* $("#toggle").ejToggleButton({defaultText:"Play",activeText:"Pause"});
		* $("#toggle").ejToggleButton("disable");	
		* &lt;/script&gt;
		*@memberof ejToggleButton
		* @instance
        */
        // sample public function
        disable: function () {
            /// <summary>This Contains disable functionality to wire with the toggle button </summary>
            this.buttontag.addClass("e-disable").attr({ "aria-disabled": true });
            this.model.enabled = false;
        },
		/**
        * To enable the toggle button  
		* @return jQuery
		* @example 
		* &lt;input id="toggle" type="checkbox" / &gt; <br> 
		* &lt;script&gt;
		* // Create toggle button
		* $("#toggle").ejToggleButton({defaultText:"Play",activeText:"Pause"});
		* var toggleObj = $("#toggle").data("ejToggleButton");
		* toggleObj.enable(); // enable the toggle button
		* &lt;/script&gt;
		* @example 
		* &lt;input id="toggle" type="checkbox" / &gt; <br> 
		* &lt;script&gt;
		* // enable the toggle button
		* $("#toggle").ejToggleButton({defaultText:"Play",activeText:"Pause"});
		* $("#toggle").ejToggleButton("enable");	
		* &lt;/script&gt;
		*@memberof ejToggleButton
		* @instance
        */
        enable: function () {
            /// <summary>This Contains enable functionality to wire with the toggle button </summary>
            if (this.buttontag.hasClass("e-disable")) {
                this.buttontag.removeClass("e-disable").attr({ "aria-disabled": false });
                this.model.enabled = true;
            }
        },
		/**
         * Create the toggle button widget
		 * @private
         */	
        // constructor function
        _init: function () {
            /// <summary>This function contains initialization </summary>
            this._initialize();
            this._controlStatus(this.model.enabled);
            this._wireEvents();
        },
		/**
        * To destroy the toggle button  
		* @alias destroy
		* @return jQuery
		* @example 
		* &lt;input id="toggle" type="checkbox" / &gt; <br> 
		* &lt;script&gt;
		* // Create toggle button
		* $("#toggle").ejToggleButton({defaultText:"Play",activeText:"Pause"});
		* var toggleObj = $("#toggle").data("ejToggleButton");
		* toggleObj.destroy(); // destroy the toggle button
		* &lt;/script&gt;
		* @example 
		* &lt;input id="toggle" type="checkbox" / &gt; <br> 
		* &lt;script&gt;
		* // destroy the toggle button
		* $("#toggle").ejToggleButton({defaultText:"Play",activeText:"Pause"});
		* $("#toggle").ejToggleButton("destroy");	
		* &lt;/script&gt;
		*@memberof ejToggleButton
		* @instance
        */
        _destroy: function () {
            /// <summary>This Contains destroy of toggle button </summary>
            this.element.unwrap();
            this.element.removeClass('e-chkbx-hidden');
            this.labelFinder.empty();
            this.labelFinder.text(this.model.defaultText);
        },
		 /**
         * To configure the properties at runtime using SetModel		
		 * @private
         */
        _setModel: function (options) {
            if (!(this.model.enabled) && ej.isNullOrUndefined(options["enabled"])) return false;
            var option;
            for (option in options) {
                switch (option) {
                    case "size":
                        this._setSize(options[option]);
                        break;
                    case "type":
                        this._settype(options[option]);
                        break;
                    case "height":
                        this._setHeight(options[option]);
                        break;
                    case "width":
                        this._setWidth(options[option]);
                        break;
                    case "contentType":
                        this._setContentType(options[option]);
                        break;
                    case "imagePosition":
                        this._setImagePosition(options[option]);
                        break;
                    case "defaultText":
                        this._setDefaultText(options[option]);
                        break;
                    case "activeText":
                        this._setActiveText(options[option]);
                        break;
                    case "defaultPrefixIcon":
                        this._setDefaultMajorIcon(options[option]);
                        break;
                    case "defaultSuffixIcon":
                        this._setDefaultMinorIcon(options[option]);
                        break;
                    case "activePrefixIcon":
                        this._setActiveMajorIcon(options[option]);
                        break;
                    case "activeSuffixIcon":
                        this._setActiveMinorIcon(options[option]);
                        break;
                    case "enabled":
                        this._controlStatus(options[option]);
                        break;
                    case "toggleState":
                        this._tglevaluestatus(options[option]);
                        break;
                    case "showRoundedCorner":
                        this._roundedCorner(options[option]);
                        break;
                    case "cssClass":
                        this._setSkin(options[option]);
                        break;
                    case "enableRTL":
                        this._setRTL(options[option]);
                        break;
                }
            }
        },
		/**
         * To configure size of the toggle button		
		 * @private
         */	 
        _setSize: function (val) {
            this.buttontag.removeClass('e-btn-mini e-btn-medium e-btn-small e-btn-large e-btn-normal');
            this.buttontag.addClass("e-btn-" + val);
        },

        _settype: function (val) {
            this.model.type = val;            
        },

		/**
         * To configure height of the toggle button		
		 * @private
         */	
        _setHeight: function (val) {
            /// <summary>This will set Text property of toggle button </summary>
            this.buttontag.css('height', val);
        },
		/**
         * To configure width of the toggle button		
		 * @private
         */	
        _setWidth: function (val) {
            /// <summary>This will set Text property of toggle button </summary>
            this.buttontag.css('width', val);
        },
		/**
         * To configure default text to the toggle button		
		 * @private
         */	
        _setDefaultText: function (val) {
            /// <summary>This will set Text property of toggle button </summary>
            if (this.model.contentType == ej.ContentType.TextOnly) {
                this.buttontag.html(val);
            } else {
                this.defaulttxtspan.html(val);
            }
        },
		/**
         * To configure active text to the toggle button		
		 * @private
         */	
        _setActiveText: function (val) {
            /// <summary>This will set Text property of toggle button </summary>
            if (this.model.toggleState) {
                if (this.model.contentType == ej.ContentType.TextOnly) {
                    this.buttontag.html(val);
                } else {
                    this.defaulttxtspan.html(val);
                }
            }
        },
		/**
         * To configure default major icon of the toggle button		
		 * @private
         */	
        _setDefaultMajorIcon: function (val) {
            /// <summary>This will set Major Icon property of toggle button </summary>
            this.defMainIcon.removeClass(this.model.defaultPrefixIcon);
            this.defMainIcon.addClass(val);
        },
		/**
         * To configure default minor icon of the toggle button		
		 * @private
         */	
        _setDefaultMinorIcon: function (val) {
            /// <summary>This will set Minor Icon property of toggle button </summary>
            this.defMiniIcon.removeClass(this.model.defaultSuffixIcon);
            this.defMiniIcon.addClass(val);
        },
		/**
         * To configure active major icon of the toggle button		
		 * @private
         */	
        _setActiveMajorIcon: function (val) {
            /// <summary>This will set Major Icon property of toggle button </summary>
            if (this.model.toggleState) {
                this.defMainIcon.removeClass(this.model.activePrefixIcon);
                this.defMainIcon.addClass(val);
            }
        },
		/**
         * To configure active minor icon of the toggle button		
		 * @private
         */	
        _setActiveMinorIcon: function (val) {
            /// <summary>This will set Minor Icon property of toggle button </summary>
            if (this.model.toggleState) {
                this.defMiniIcon.removeClass(this.model.activeSuffixIcon);
                this.defMiniIcon.addClass(val);
            }
        },
		/**
         * To configure content type of the toggle button		
		 * @private
         */	
        _setContentType: function (val) {
            if (val != this.model.contentType) {
                this.buttontag.empty();
                this.model.contentType = val;
                this._renderButtonContent();
            }
        },
		/**
         * To configure image position of the toggle button		
		 * @private
         */	
        _setImagePosition: function (val) {
            if ((this.model.contentType == ej.ContentType.TextAndImage) && (val != this.model.imagePosition)) {
                this.buttontag.empty();
                this.model.imagePosition = val;
                this._renderButtonContent();
            }
        },
		 /**
         * To enable or disable the Right to Left behaviour 		
		 * @private
         */	 
        _setRTL: function (val) {
            if (val) {
                this.buttontag.addClass("e-rtl");
            } else {
                this.buttontag.removeClass("e-rtl");
            }
        },
		 /**
         * To enable or disable the toggle button state 		
		 * @private
         */	 
        _controlStatus: function (value) {
            //disable status
            if (!value) {
                this.disable();
            } else {
                this.enable();
            }
        },
		/**
         * To configure the custom theme for toggle button using cssClass property		
		 * @private
         */	 
        _setSkin: function (skin) {
            if (this.model.cssClass != skin) {
                this.buttontag.removeClass(this.model.cssClass);
                this.buttontag.addClass(skin);
            }
        },
		/**
         * To initialize the toggle button		
		 * @private
         */	 
        _initialize: function () {
            /// <summary>This Contains initialize of toggle button </summary>
            if (this.element.is('[type = "checkbox"]')) {
                this._render();
            } else {
                this.element.removeClass("e-togglebutton");//need to change in source level
            }
        },
		 /**
         * Render Section For DifferentTypes		
		 * @private
         */	
        /*Render Section For DifferentTypes*/
        _render: function () {
            /// <summary>This Contains rendering of toggle button </summary>
            var predecessor, labelSelector;
            /*Determine Button Type*/
            this.element.addClass('e-chkbx-hidden e-tbtn');

            /*Finding the label element in the dom*/
            predecessor = this.element.parents().last();
            labelSelector = "label[for='" + this.element[0].id + "']";
            this.labelFinder = predecessor.find(labelSelector);
            if (!this.labelFinder.length) {
                this.labelFinder = ej.buildTag('label', "Button", {}, { "for": this.element[0].id });
                this.labelFinder.insertAfter(this.element);
            }
            if ((this.model.defaultText == null) || (this.model.defaultText == "")) {
                this.model.defaultText = this.labelFinder.text();
            }
            this.labelFinder.empty();
            this.wrapper = $('<span id="' + this.element[0].id + '-wrapper" class="e-tbtn-wrap e-widget"></span>');
            $(this.element).wrap(this.wrapper);
            this.labelFinder.insertAfter(this.element);
            this.buttontag = ej.buildTag('button.e-togglebutton e-btn e-tbtn ' + this.model.cssClass + ' e-select', "", {}, { "role": "button", "tabindex": 0, "type":this.model.type});//need to change in source level
            this.labelFinder.append(this.buttontag);
            /*Initializing the content*/
            if (ej.util.isNullOrUndefined(this.model.activeText)) {
                this.model.activeText = this.model.defaultText;
            }
            if (ej.util.isNullOrUndefined(this.model.activePrefixIcon)) {
                this.model.activePrefixIcon = this.model.defaultPrefixIcon;
            }
            if (ej.util.isNullOrUndefined(this.model.activeSuffixIcon)) {
                this.model.activeSuffixIcon = this.model.defaultSuffixIcon;
            }
            /*Height Width*/
            this._setSize(this.model.size);
            this._setHeight(this.model.height);
            this._setWidth(this.model.width);
            this._setRTL(this.model.enableRTL);
            this._renderButtonContent();
            this._tglevaluestatus(this.model.toggleState);
            this._roundedCorner(this.model.showRoundedCorner);
        },
		/**
         * Render Section For toggle button content
		 * @private
         */	
        _renderButtonContent: function () {
            /*Image and Text*/
            this.imgtxtwrap = ej.buildTag('div');
            /*Text Content*/
            this.defaulttxtspan = ej.buildTag('span.e-btntxt' + '#' + this.element[0].id + 'textstatic', this.model.defaultText);
            /*Image Content*/
            if (this.model.contentType.indexOf("image") > -1) {
                this.defMainIcon = ej.buildTag('span.e-icon ' + this.model.defaultPrefixIcon + '#' + this.element[0].id + 'mainiconstatic');
                this.defMiniIcon = ej.buildTag('span.e-icon ' + this.model.defaultSuffixIcon + '#' + this.element[0].id + 'miniconstatic');
            }
            /*Rendering Option*/
            if (this.model.contentType == ej.ContentType.TextAndImage) {
				switch (this.model.imagePosition) {
                    case ej.ImagePosition.ImageRight:
                    	this.imgtxtwrap.append(this.defaulttxtspan, this.defMainIcon);
						break;
					case ej.ImagePosition.ImageLeft:
                    	this.imgtxtwrap.append(this.defMainIcon, this.defaulttxtspan);
						break;
					case ej.ImagePosition.ImageBottom:	
						this.defMainIcon.attr("style","display:inherit");
                    	this.imgtxtwrap.append(this.defaulttxtspan, this.defMainIcon);
						break;
                	case ej.ImagePosition.ImageTop:
						this.defMainIcon.attr("style","display:inherit");
                    	this.imgtxtwrap.append(this.defMainIcon, this.defaulttxtspan);
						break;
                }
				this.buttontag.append(this.imgtxtwrap);
            } else if (this.model.contentType == ej.ContentType.ImageTextImage) {
                this.imgtxtwrap.append(this.defMainIcon, this.defaulttxtspan, this.defMiniIcon);
                this.buttontag.append(this.imgtxtwrap);
            } else if (this.model.contentType == ej.ContentType.ImageOnly) {
                this.imgtxtwrap.append(this.defMainIcon);
                this.buttontag.append(this.imgtxtwrap);
            } else if (this.model.contentType == ej.ContentType.ImageBoth) {
                this.imgtxtwrap.append(this.defMainIcon, this.defMiniIcon);
                this.buttontag.append(this.imgtxtwrap);
            } else {
                this.buttontag.addClass("e-txt");
                this.buttontag.text(this.model.defaultText);
            }
            /**/
        },
		/**
         * To check whether Status of Toggle Button
		 * @private
         */	
        _tglevaluestatus: function (value) {
            /*Checking Status of Toggle Button*/
            if (value) {
                this._toggleButtonStatus(value);
                this.element.attr("checked", "checked");
            } else {
                this._toggleButtonStatus(value);
                this.element.removeAttr("checked");
            }
        },
		/**
         * To enable or disable the rounded corner behaviour 		
		 * @private
         */	
        _roundedCorner: function (value) {
            value == true ? this.buttontag.addClass('e-corner-all') : this.buttontag.removeClass('e-corner-all');
        },
        /*-----------------------Event Handlers -----------------------------------------*/
		 /**
         * Wiring the events to toggle button control		
		 * @private
         */	
        _wireEvents: function () {
            /// <summary>This Contains Events to wire with the toggle button </summary>

            this._on(this.buttontag, "click", this._tglebtnclicked);
        },
		 /**
         * Section For handle the click event
		 * @private
         */	
        _tglebtnclicked: function (e) {
		if (!this.model.preventToggle) {
            var position, end, version = 0;
            if (navigator.userAgent.search("MSIE") > 0) {
                position = navigator.userAgent.search("MSIE") + 5;
                end = navigator.userAgent.search("; Windows");
                version = navigator.userAgent.substring(position, end);
            }
            if (!this.buttontag.hasClass("e-disable")) {
                if (!this.model.toggleState) {
                    this.model.toggleState = true;
                    if ((window.navigator.appName == "Microsoft Internet Explorer") && (version == "8.0")) {
                        this._tglevaluestatus(this.model.toggleState);
                    } else {
                        this._toggleButtonStatus(this.model.toggleState);
                    }
                } else {
                    this.model.toggleState = false;
                    if ((window.navigator.appName == "Microsoft Internet Explorer") && (version == "8.0")) {
                        this._tglevaluestatus(this.model.toggleState);
                    } else {
                        this._toggleButtonStatus(this.model.toggleState);
                    }
                }
            }
            this._trigger("click", { "isChecked": this.model.toggleState, status: this.model.enabled });
        }
		},
		 /**
         * Section For handle the toggle button status
		 * @private
         */	
        _toggleButtonStatus: function (buttonstatus) {
            if (buttonstatus) {
                if (this.model.contentType == ej.ContentType.TextOnly) {
                    this.buttontag.html(this.model.activeText);
                } else {
                    this.defaulttxtspan.html(this.model.activeText);
                    this.defMainIcon.removeClass(this.model.defaultPrefixIcon).addClass(this.model.activePrefixIcon);
                    this.defMiniIcon.removeClass(this.model.defaultSuffixIcon).addClass(this.model.activeSuffixIcon);
                }
                this.buttontag.addClass("e-active").attr("aria-pressed", true);
            } else {
                if (this.model.contentType == ej.ContentType.TextOnly) {
                    this.buttontag.html(this.model.defaultText);
                } else {
                    this.defaulttxtspan.html(this.model.defaultText);
                    this.defMainIcon.removeClass(this.model.activePrefixIcon).addClass(this.model.defaultPrefixIcon);
                    this.defMiniIcon.removeClass(this.model.activeSuffixIcon).addClass(this.model.defaultSuffixIcon);
                }
                this.buttontag.removeClass("e-active").attr("aria-pressed", false);
            }
            this._trigger('change', { "isChecked": this.model.toggleState });
        },

    });
	/**
	 * Enum for toggle button content mode.	 
	 * @enum {string}
	 * @global 
	 */
    ej.ContentType = { 
			/**  Supports only for text content only */
			TextOnly: "textonly", 
			/** Supports only for image content only */
			ImageOnly: "imageonly", 
			/** Supports image for both ends of the button */
			ImageBoth: "imageboth", 
			/** Supports image with the text content */
			TextAndImage: "textandimage", 
			/** Supports image with both ends of the text */
			ImageTextImage: "imagetextimage" 
			};
	
	/**
	 * Enum for toggle button positioning	 
	 * @enum {string}
	 * @global 
	 */
    ej.ImagePosition = { 
			/**  support for aligning text in left and image in right. */
			ImageRight: "imageright", 
			/**  support for aligning text in right and image in left. */
			ImageLeft: "imageleft", 
			/**  support for aligning text in bottom and image in top. */
			ImageTop: "imagetop", 
			/**  support for aligning text in top and image in bottom. */
			ImageBottom: "imagebottom"
			};
	/**
	 * Enum for various toggle button sizes	 
	 * @enum {string}
	 * @global 
	 */
    ej.ButtonSize = { 
			/**  Creates button with inbuilt default size height, width specified */
			Normal : "normal",
			/**  Creates button with inbuilt mini size height, width specified */
			Mini: "mini", 
			/**  Creates button with inbuilt small size height, width specified */
			Small: "small",
			/**  Creates button with inbuilt medium size height, width specified */
			Medium:"medium", 
			/**  Creates button with inbuilt large size height, width specified */
			Large: "large" };
			
    ej.ButtonType = {
        /**  Creates button with button type as button */
        Button: "button",
        /**  Creates button with button type as reset */
        Reset: "reset",
        /**  Creates button with button type as submit */
        Submit: "submit"
      
    };
})(jQuery, Syncfusion);;;
/*!
*  filename: ej.toolbar.js
*  version : 12.1
*  Copyright Syncfusion Inc. 2001 - 2013. All rights reserved.
*  Use of this code is subject to the terms of our license.
*  A copy of the current license can be obtained at any time by e-mailing
*  licensing@syncfusion.com. Any infringement will be prosecuted under
*  applicable laws. 
*/

/**
* @fileOverview Plugin to style the Toolbar elements
* @copyright Copyright Syncfusion Inc. 2001 - 2013. All rights reserved.
*  Use of this code is subject to the terms of our license.
*  A copy of the current license can be obtained at any time by e-mailing
*  licensing@syncfusion.com. Any infringement will be prosecuted under
*  applicable laws. 
* @version 12.1 
* @author <a href="mailto:licensing@syncfusion.com">Syncfusion Inc</a>
*/
(function ($, ej, undefined) {
/**
    * @namespace ej
	* @class ejToolbar
	* @requires jQuery
	* @requires jquery-1.10.2.min.js
    * @requires ej.core.js
    * @requires ej.data.js
    * @requires ej.toolbar.js
	* @classdesc Custom Design for Toolbar control.
	* @example 
	* &lt;div id="toolbar1"&gt;
    *&lt;ul&gt;
    *    &lt;li id="Left" title="Left"&gt;
    *        &lt;div class="ToolbarItems LeftAlign_tool"&gt;&lt;/div&gt;
    *   &lt;/li&gt;
    *    &lt;li id="Center" title="Center"&gt;
    *        &lt;div class="ToolbarItems CenterAlign_tool"&gt;&lt;/div&gt;
    *    &lt;/li&gt;
    *    &lt;li id="Right" title="Right"&gt;
    *        &lt;div class="ToolbarItems RightAlign_tool"&gt;&lt;/div&gt;
    *    &lt;/li&gt;
    *    &lt;li id="Justify" title="Justify"&gt;
    *        &lt;div class="ToolbarItems Justify_tool"&gt;&lt;/div&gt;
    *    &lt;/li&gt;
    *&lt;/ul&gt;
    *&lt;ul&gt;
    *    &lt;li id="Bold" title="Bold"&gt;
    *        &lt;div class="ToolbarItems Bold_tool"&gt;&lt;/div&gt;
    *    &lt;/li&gt;
    *    &lt;li id="Italic" title="Italic"&gt;
    *        &lt;div class="ToolbarItems Italic_tool"&gt;&lt;/div&gt;
    *    &lt;/li&gt;
    *    &lt;li id="StrikeThrough" title="Strike Through"&gt;
    *        &lt;div class="ToolbarItems StrikeThrough_tool"&gt;&lt;/div&gt;
    *    &lt;/li&gt;
    *    &lt;li id="UndeLine" title="UnderLine"&gt;
    *        &lt;div class="ToolbarItems Underline_tool"&gt;&lt;/div&gt;
    *    &lt;/li&gt;
    *&lt;/ul&gt;
	*&lt;/div&gt;
	* &lt;script&gt;
	* // Create Toolbar
    *  $("#toolbar1").ejToolbar();	
	* &lt;/script&gt;
	*/
    // Example plugin creation code
    // ejSample is the plugin name 
    // "ej.Sample" is "namespace.className" will hold functions and properties

    ej.widget("ejToolbar", "ej.Toolbar", {
        // widget element will be automatically set in this
        element: null,

        // user defined model will be automatically set in this
        model: null,
        validTags: ["div", "span"],
        _setFirst: false,
        //Root Css Class
        _rootCSS: "e-toolbar",

        // default model
        defaults: {
            /// <summary>This Contains default property of toolbar </summary>
			/**		
			* Specifies the height of the Toolbar.
			* @default 28
			* @type {number}
			* @example 
			* &lt;div id="toolbar1"&gt;
    *&lt;ul&gt;
    *    &lt;li id="Left" title="Left"&gt;
    *        &lt;div class="ToolbarItems LeftAlign_tool"&gt;&lt;/div&gt;
    *   &lt;/li&gt;
    *    &lt;li id="Center" title="Center"&gt;
    *        &lt;div class="ToolbarItems CenterAlign_tool"&gt;&lt;/div&gt;
    *    &lt;/li&gt;
    *    &lt;li id="Right" title="Right"&gt;
    *        &lt;div class="ToolbarItems RightAlign_tool"&gt;&lt;/div&gt;
    *    &lt;/li&gt;
    *    &lt;li id="Justify" title="Justify"&gt;
    *        &lt;div class="ToolbarItems Justify_tool"&gt;&lt;/div&gt;
    *    &lt;/li&gt;
    *&lt;/ul&gt;
    *&lt;ul&gt;
    *    &lt;li id="Bold" title="Bold"&gt;
    *        &lt;div class="ToolbarItems Bold_tool"&gt;&lt;/div&gt;
    *    &lt;/li&gt;
    *    &lt;li id="Italic" title="Italic"&gt;
    *        &lt;div class="ToolbarItems Italic_tool"&gt;&lt;/div&gt;
    *    &lt;/li&gt;
    *    &lt;li id="StrikeThrough" title="Strike Through"&gt;
    *        &lt;div class="ToolbarItems StrikeThrough_tool"&gt;&lt;/div&gt;
    *    &lt;/li&gt;
    *    &lt;li id="UndeLine" title="UnderLine"&gt;
    *        &lt;div class="ToolbarItems Underline_tool"&gt;&lt;/div&gt;
    *    &lt;/li&gt;
    *&lt;/ul&gt;
	*&lt;/div&gt;
	* &lt;script&gt;
			* //To set height API value during initialization  
			* 	$("#toolbar1").ejToolbar({ height: 30 });					 
			* &lt;/script&gt;
			 * @memberof ejToolbar
			* @instance
			*/
            height: "",
			/**		
			* Specifies the width of the Toolbar.
			* @default 100
			* @type {number}
			* @example 
			* &lt;div id="toolbar1"&gt;
    *&lt;ul&gt;
    *    &lt;li id="Left" title="Left"&gt;
    *        &lt;div class="ToolbarItems LeftAlign_tool"&gt;&lt;/div&gt;
    *   &lt;/li&gt;
    *    &lt;li id="Center" title="Center"&gt;
    *        &lt;div class="ToolbarItems CenterAlign_tool"&gt;&lt;/div&gt;
    *    &lt;/li&gt;
    *    &lt;li id="Right" title="Right"&gt;
    *        &lt;div class="ToolbarItems RightAlign_tool"&gt;&lt;/div&gt;
    *    &lt;/li&gt;
    *    &lt;li id="Justify" title="Justify"&gt;
    *        &lt;div class="ToolbarItems Justify_tool"&gt;&lt;/div&gt;
    *    &lt;/li&gt;
    *&lt;/ul&gt;
    *&lt;ul&gt;
    *    &lt;li id="Bold" title="Bold"&gt;
    *        &lt;div class="ToolbarItems Bold_tool"&gt;&lt;/div&gt;
    *    &lt;/li&gt;
    *    &lt;li id="Italic" title="Italic"&gt;
    *        &lt;div class="ToolbarItems Italic_tool"&gt;&lt;/div&gt;
    *    &lt;/li&gt;
    *    &lt;li id="StrikeThrough" title="Strike Through"&gt;
    *        &lt;div class="ToolbarItems StrikeThrough_tool"&gt;&lt;/div&gt;
    *    &lt;/li&gt;
    *    &lt;li id="UndeLine" title="UnderLine"&gt;
    *        &lt;div class="ToolbarItems Underline_tool"&gt;&lt;/div&gt;
    *    &lt;/li&gt;
    *&lt;/ul&gt;
	*&lt;/div&gt;
	* &lt;script&gt;
			* //To set width API value during initialization  
			* 	//To set width API value 
			* 	$("#toolbar1").ejToolbar({ width: 30 });	
			* &lt;/script&gt;
			* @instance
			*/
            width:"",
			/**		
			* Specifies the Toolbar control state.
			* @default true
			* @type {boolean}
			* @example 
			* &lt;div id="toolbar1"&gt;
    *&lt;ul&gt;
    *    &lt;li id="Left" title="Left"&gt;
    *        &lt;div class="ToolbarItems LeftAlign_tool"&gt;&lt;/div&gt;
    *   &lt;/li&gt;
    *    &lt;li id="Center" title="Center"&gt;
    *        &lt;div class="ToolbarItems CenterAlign_tool"&gt;&lt;/div&gt;
    *    &lt;/li&gt;
    *    &lt;li id="Right" title="Right"&gt;
    *        &lt;div class="ToolbarItems RightAlign_tool"&gt;&lt;/div&gt;
    *    &lt;/li&gt;
    *    &lt;li id="Justify" title="Justify"&gt;
    *        &lt;div class="ToolbarItems Justify_tool"&gt;&lt;/div&gt;
    *    &lt;/li&gt;
    *&lt;/ul&gt;
    *&lt;ul&gt;
    *    &lt;li id="Bold" title="Bold"&gt;
    *        &lt;div class="ToolbarItems Bold_tool"&gt;&lt;/div&gt;
    *    &lt;/li&gt;
    *    &lt;li id="Italic" title="Italic"&gt;
    *        &lt;div class="ToolbarItems Italic_tool"&gt;&lt;/div&gt;
    *    &lt;/li&gt;
    *    &lt;li id="StrikeThrough" title="Strike Through"&gt;
    *        &lt;div class="ToolbarItems StrikeThrough_tool"&gt;&lt;/div&gt;
    *    &lt;/li&gt;
    *    &lt;li id="UndeLine" title="UnderLine"&gt;
    *        &lt;div class="ToolbarItems Underline_tool"&gt;&lt;/div&gt;
    *    &lt;/li&gt;
    *&lt;/ul&gt;
	*&lt;/div&gt;
	* &lt;script&gt;
			* // Enable Toolbar on initialization. 
			* 	//To set enabled API value 
			* 	$("#toolbar1").ejToolbar({ enabled:  true });			
			* &lt;/script&gt;
			 * @memberof ejToolbar
			* @instance
			*/
            enabled: true,
			/**		
			* Specifies the to show or hide.
			* @default false
			* @type {boolean}
			* @example 
			* &lt;div id="toolbar1"&gt;
    *&lt;ul&gt;
    *    &lt;li id="Left" title="Left"&gt;
    *        &lt;div class="ToolbarItems LeftAlign_tool"&gt;&lt;/div&gt;
    *   &lt;/li&gt;
    *    &lt;li id="Center" title="Center"&gt;
    *        &lt;div class="ToolbarItems CenterAlign_tool"&gt;&lt;/div&gt;
    *    &lt;/li&gt;
    *    &lt;li id="Right" title="Right"&gt;
    *        &lt;div class="ToolbarItems RightAlign_tool"&gt;&lt;/div&gt;
    *    &lt;/li&gt;
    *    &lt;li id="Justify" title="Justify"&gt;
    *        &lt;div class="ToolbarItems Justify_tool"&gt;&lt;/div&gt;
    *    &lt;/li&gt;
    *&lt;/ul&gt;
    *&lt;ul&gt;
    *    &lt;li id="Bold" title="Bold"&gt;
    *        &lt;div class="ToolbarItems Bold_tool"&gt;&lt;/div&gt;
    *    &lt;/li&gt;
    *    &lt;li id="Italic" title="Italic"&gt;
    *        &lt;div class="ToolbarItems Italic_tool"&gt;&lt;/div&gt;
    *    &lt;/li&gt;
    *    &lt;li id="StrikeThrough" title="Strike Through"&gt;
    *        &lt;div class="ToolbarItems StrikeThrough_tool"&gt;&lt;/div&gt;
    *    &lt;/li&gt;
    *    &lt;li id="UndeLine" title="UnderLine"&gt;
    *        &lt;div class="ToolbarItems Underline_tool"&gt;&lt;/div&gt;
    *    &lt;/li&gt;
    *&lt;/ul&gt;
	*&lt;/div&gt;
	* &lt;script&gt;
			* // Hide Toolbar on initialization. 
			* 	//To set hide API value 
			* 	$("#toolbar1").ejToolbar({  hide: true });			
			* &lt;/script&gt;
			 * @memberof ejToolbar
			* @instance
			*/
            hide: false,
			/**		
			* Specifies the to enableSeparator.
			* @default false
			* @type {boolean}
			* @example 
			* &lt;div id="toolbar1"&gt;
    *&lt;ul&gt;
    *    &lt;li id="Left" title="Left"&gt;
    *        &lt;div class="ToolbarItems LeftAlign_tool"&gt;&lt;/div&gt;
    *   &lt;/li&gt;
    *    &lt;li id="Center" title="Center"&gt;
    *        &lt;div class="ToolbarItems CenterAlign_tool"&gt;&lt;/div&gt;
    *    &lt;/li&gt;
    *    &lt;li id="Right" title="Right"&gt;
    *        &lt;div class="ToolbarItems RightAlign_tool"&gt;&lt;/div&gt;
    *    &lt;/li&gt;
    *    &lt;li id="Justify" title="Justify"&gt;
    *        &lt;div class="ToolbarItems Justify_tool"&gt;&lt;/div&gt;
    *    &lt;/li&gt;
    *&lt;/ul&gt;
    *&lt;ul&gt;
    *    &lt;li id="Bold" title="Bold"&gt;
    *        &lt;div class="ToolbarItems Bold_tool"&gt;&lt;/div&gt;
    *    &lt;/li&gt;
    *    &lt;li id="Italic" title="Italic"&gt;
    *        &lt;div class="ToolbarItems Italic_tool"&gt;&lt;/div&gt;
    *    &lt;/li&gt;
    *    &lt;li id="StrikeThrough" title="Strike Through"&gt;
    *        &lt;div class="ToolbarItems StrikeThrough_tool"&gt;&lt;/div&gt;
    *    &lt;/li&gt;
    *    &lt;li id="UndeLine" title="UnderLine"&gt;
    *        &lt;div class="ToolbarItems Underline_tool"&gt;&lt;/div&gt;
    *    &lt;/li&gt;
    *&lt;/ul&gt;
	*&lt;/div&gt;
	* &lt;script&gt;
			* //specifies enableSeparator Toolbar on initialization. 
			* 	//To set enableSeparator API value 
			* 	$("#toolbar1").ejToolbar({enableSeparator:true});			 
			* &lt;/script&gt;
			 * @memberof ejToolbar
			* @instance
			*/
            enableSeparator: false,
			/**		
			* Specifies the to toolbar orientation.See {@link orientation}
			* @default Horizontal
			* @type {Enum | String}
			* @example 
			* &lt;div id="toolbar1"&gt;
    *&lt;ul&gt;
    *    &lt;li id="Left" title="Left"&gt;
    *        &lt;div class="ToolbarItems LeftAlign_tool"&gt;&lt;/div&gt;
    *   &lt;/li&gt;
    *    &lt;li id="Center" title="Center"&gt;
    *        &lt;div class="ToolbarItems CenterAlign_tool"&gt;&lt;/div&gt;
    *    &lt;/li&gt;
    *    &lt;li id="Right" title="Right"&gt;
    *        &lt;div class="ToolbarItems RightAlign_tool"&gt;&lt;/div&gt;
    *    &lt;/li&gt;
    *    &lt;li id="Justify" title="Justify"&gt;
    *        &lt;div class="ToolbarItems Justify_tool"&gt;&lt;/div&gt;
    *    &lt;/li&gt;
    *&lt;/ul&gt;
    *&lt;ul&gt;
    *    &lt;li id="Bold" title="Bold"&gt;
    *        &lt;div class="ToolbarItems Bold_tool"&gt;&lt;/div&gt;
    *    &lt;/li&gt;
    *    &lt;li id="Italic" title="Italic"&gt;
    *        &lt;div class="ToolbarItems Italic_tool"&gt;&lt;/div&gt;
    *    &lt;/li&gt;
    *    &lt;li id="StrikeThrough" title="Strike Through"&gt;
    *        &lt;div class="ToolbarItems StrikeThrough_tool"&gt;&lt;/div&gt;
    *    &lt;/li&gt;
    *    &lt;li id="UndeLine" title="UnderLine"&gt;
    *        &lt;div class="ToolbarItems Underline_tool"&gt;&lt;/div&gt;
    *    &lt;/li&gt;
    *&lt;/ul&gt;
	*&lt;/div&gt;
	* &lt;script&gt;
			* // Specifies orientation of  Toolbar on initialization. 
			* 	//To set orientation API value 
			* 	$("#toolbar1").ejToolbar({ orientation: ej.Orientation.Horizontal }); 
			* &lt;/script&gt;
			 * @memberof ejToolbar
			* @instance
			*/
            orientation: "horizontal",
			/**		
			* Specifies enableRTL property for the Toolbar during initialization.
			* @default false
			* @type {boolean}
			* @example 
			* &lt;div id="toolbar1"&gt;
    *&lt;ul&gt;
    *    &lt;li id="Left" title="Left"&gt;
    *        &lt;div class="ToolbarItems LeftAlign_tool"&gt;&lt;/div&gt;
    *   &lt;/li&gt;
    *    &lt;li id="Center" title="Center"&gt;
    *        &lt;div class="ToolbarItems CenterAlign_tool"&gt;&lt;/div&gt;
    *    &lt;/li&gt;
    *    &lt;li id="Right" title="Right"&gt;
    *        &lt;div class="ToolbarItems RightAlign_tool"&gt;&lt;/div&gt;
    *    &lt;/li&gt;
    *    &lt;li id="Justify" title="Justify"&gt;
    *        &lt;div class="ToolbarItems Justify_tool"&gt;&lt;/div&gt;
    *    &lt;/li&gt;
    *&lt;/ul&gt;
    *&lt;ul&gt;
    *    &lt;li id="Bold" title="Bold"&gt;
    *        &lt;div class="ToolbarItems Bold_tool"&gt;&lt;/div&gt;
    *    &lt;/li&gt;
    *    &lt;li id="Italic" title="Italic"&gt;
    *        &lt;div class="ToolbarItems Italic_tool"&gt;&lt;/div&gt;
    *    &lt;/li&gt;
    *    &lt;li id="StrikeThrough" title="Strike Through"&gt;
    *        &lt;div class="ToolbarItems StrikeThrough_tool"&gt;&lt;/div&gt;
    *    &lt;/li&gt;
    *    &lt;li id="UndeLine" title="UnderLine"&gt;
    *        &lt;div class="ToolbarItems Underline_tool"&gt;&lt;/div&gt;
    *    &lt;/li&gt;
    *&lt;/ul&gt;
	*&lt;/div&gt;
	* &lt;script&gt;
			* // to set enableRTL Toolbar on initialization. 
			* 	//To set enableRTL API value 
			* 	$("#toolbar1").ejToolbar({ enableRTL: false });			 
			* &lt;/script&gt;
			* @memberof ejToolbar
			* @instance
			*/
            enableRTL: false,
            allowResponsive: false,
			/**		
			* Specifies showRoundedCorner property for the Toolbar during initialization.
			* @default false
			* @type {boolean}
			* @example 
			* &lt;div id="toolbar1"&gt;
    *&lt;ul&gt;
    *    &lt;li id="Left" title="Left"&gt;
    *        &lt;div class="ToolbarItems LeftAlign_tool"&gt;&lt;/div&gt;
    *   &lt;/li&gt;
    *    &lt;li id="Center" title="Center"&gt;
    *        &lt;div class="ToolbarItems CenterAlign_tool"&gt;&lt;/div&gt;
    *    &lt;/li&gt;
    *    &lt;li id="Right" title="Right"&gt;
    *        &lt;div class="ToolbarItems RightAlign_tool"&gt;&lt;/div&gt;
    *    &lt;/li&gt;
    *    &lt;li id="Justify" title="Justify"&gt;
    *        &lt;div class="ToolbarItems Justify_tool"&gt;&lt;/div&gt;
    *    &lt;/li&gt;
    *&lt;/ul&gt;
    *&lt;ul&gt;
    *    &lt;li id="Bold" title="Bold"&gt;
    *        &lt;div class="ToolbarItems Bold_tool"&gt;&lt;/div&gt;
    *    &lt;/li&gt;
    *    &lt;li id="Italic" title="Italic"&gt;
    *        &lt;div class="ToolbarItems Italic_tool"&gt;&lt;/div&gt;
    *    &lt;/li&gt;
    *    &lt;li id="StrikeThrough" title="Strike Through"&gt;
    *        &lt;div class="ToolbarItems StrikeThrough_tool"&gt;&lt;/div&gt;
    *    &lt;/li&gt;
    *    &lt;li id="UndeLine" title="UnderLine"&gt;
    *        &lt;div class="ToolbarItems Underline_tool"&gt;&lt;/div&gt;
    *    &lt;/li&gt;
    *&lt;/ul&gt;
	*&lt;/div&gt;
	* &lt;script&gt;
			* // to set showRoundedCorner for Toolbar on initialization. 
			* 	//To set showRoundedCorner API value 

			* 	$("#toolbar1").ejToolbar({ showRoundedCorner: true });			 
			* &lt;/script&gt;
			 * @memberof ejToolbar
			* @instance
			*/
            showRoundedCorner: false,
				/**		
			* Specifies dataSource value for the Toolbar during initialization.
			* @default null
			* @type {ObjectArray}
			* @example 
			* &lt;div id="toolbar1"&gt;
    *&lt;ul&gt;
    *    &lt;li id="Left" title="Left"&gt;
    *        &lt;div class="ToolbarItems LeftAlign_tool"&gt;&lt;/div&gt;
    *   &lt;/li&gt;
    *    &lt;li id="Center" title="Center"&gt;
    *        &lt;div class="ToolbarItems CenterAlign_tool"&gt;&lt;/div&gt;
    *    &lt;/li&gt;
    *    &lt;li id="Right" title="Right"&gt;
    *        &lt;div class="ToolbarItems RightAlign_tool"&gt;&lt;/div&gt;
    *    &lt;/li&gt;
    *    &lt;li id="Justify" title="Justify"&gt;
    *        &lt;div class="ToolbarItems Justify_tool"&gt;&lt;/div&gt;
    *    &lt;/li&gt;
    *&lt;/ul&gt;
    *&lt;ul&gt;
    *    &lt;li id="Bold" title="Bold"&gt;
    *        &lt;div class="ToolbarItems Bold_tool"&gt;&lt;/div&gt;
    *    &lt;/li&gt;
    *    &lt;li id="Italic" title="Italic"&gt;
    *        &lt;div class="ToolbarItems Italic_tool"&gt;&lt;/div&gt;
    *    &lt;/li&gt;
    *    &lt;li id="StrikeThrough" title="Strike Through"&gt;
    *        &lt;div class="ToolbarItems StrikeThrough_tool"&gt;&lt;/div&gt;
    *    &lt;/li&gt;
    *    &lt;li id="UndeLine" title="UnderLine"&gt;
    *        &lt;div class="ToolbarItems Underline_tool"&gt;&lt;/div&gt;
    *    &lt;/li&gt;
    *&lt;/ul&gt;
	*&lt;/div&gt;
	* &lt;script&gt;
			* // to set dataSource value during initialization. 
			* 	//To set dataSource  API value 
			* 	items = [{ edid: "1", spriteCssClass: "editTools cursor", text: "Cursor" },
			*{ edid: "2", spriteCssClass: "editTools select", text: "Select" },
			*{ edid: "3", spriteCssClass: "editTools move", text: "Move" },
			*{ edid: "4", spriteCssClass: "editTools rectselect", text: "Rectangle Select" }];

			*$("#toolbar1").ejToolbar({ dataSource:  items}); 
			* &lt;/script&gt;
			* @memberof ejToolbar
			* @instance
			*/
            dataSource: null,
				/**		
			* Specifies Specifies the query to retrieve the data from the online server. The query is used only when the online dataSource is used.
			* @default null
			* @type {object}
			* @example 
			* &lt;div id="toolbar1"&gt;
    *&lt;ul&gt;
    *    &lt;li id="Left" title="Left"&gt;
    *        &lt;div class="ToolbarItems LeftAlign_tool"&gt;&lt;/div&gt;
    *   &lt;/li&gt;
    *    &lt;li id="Center" title="Center"&gt;
    *        &lt;div class="ToolbarItems CenterAlign_tool"&gt;&lt;/div&gt;
    *    &lt;/li&gt;
    *    &lt;li id="Right" title="Right"&gt;
    *        &lt;div class="ToolbarItems RightAlign_tool"&gt;&lt;/div&gt;
    *    &lt;/li&gt;
    *    &lt;li id="Justify" title="Justify"&gt;
    *        &lt;div class="ToolbarItems Justify_tool"&gt;&lt;/div&gt;
    *    &lt;/li&gt;
    *&lt;/ul&gt;
    *&lt;ul&gt;
    *    &lt;li id="Bold" title="Bold"&gt;
    *        &lt;div class="ToolbarItems Bold_tool"&gt;&lt;/div&gt;
    *    &lt;/li&gt;
    *    &lt;li id="Italic" title="Italic"&gt;
    *        &lt;div class="ToolbarItems Italic_tool"&gt;&lt;/div&gt;
    *    &lt;/li&gt;
    *    &lt;li id="StrikeThrough" title="Strike Through"&gt;
    *        &lt;div class="ToolbarItems StrikeThrough_tool"&gt;&lt;/div&gt;
    *    &lt;/li&gt;
    *    &lt;li id="UndeLine" title="UnderLine"&gt;
    *        &lt;div class="ToolbarItems Underline_tool"&gt;&lt;/div&gt;
    *    &lt;/li&gt;
    *&lt;/ul&gt;
	*&lt;/div&gt;
	* &lt;script&gt;
			* // To set query API value on initialization. 
			* 	//To set query API value.
			* 	// DataManager creation.
			* var dataManger = ej.DataManager({
			* url: "http://mvc.syncfusion.com/Services/Northwnd.svc/"
			*});
			*  // Query creation.
			*var query = ej.Query()
			*		.from("Orders").take(6);
			*			$("#toolbar1").ejToolbar(
			*                {
            *              dataSource: dataManger,
            *               query: query
            *         });
			* &lt;/script&gt;
			* @memberof ejToolbar
			* @instance
			*/
            query: null,
				/**		
			* Specifies the mapping fields for the data items of the Toolbar
			* @default null
			* @type {string}
			* @example 
			* &lt;div id="toolbar1"&gt;
    *&lt;ul&gt;
    *    &lt;li id="Left" title="Left"&gt;
    *        &lt;div class="ToolbarItems LeftAlign_tool"&gt;&lt;/div&gt;
    *   &lt;/li&gt;
    *    &lt;li id="Center" title="Center"&gt;
    *        &lt;div class="ToolbarItems CenterAlign_tool"&gt;&lt;/div&gt;
    *    &lt;/li&gt;
    *    &lt;li id="Right" title="Right"&gt;
    *        &lt;div class="ToolbarItems RightAlign_tool"&gt;&lt;/div&gt;
    *    &lt;/li&gt;
    *    &lt;li id="Justify" title="Justify"&gt;
    *        &lt;div class="ToolbarItems Justify_tool"&gt;&lt;/div&gt;
    *    &lt;/li&gt;
    *&lt;/ul&gt;
    *&lt;ul&gt;
    *    &lt;li id="Bold" title="Bold"&gt;
    *        &lt;div class="ToolbarItems Bold_tool"&gt;&lt;/div&gt;
    *    &lt;/li&gt;
    *    &lt;li id="Italic" title="Italic"&gt;
    *        &lt;div class="ToolbarItems Italic_tool"&gt;&lt;/div&gt;
    *    &lt;/li&gt;
    *    &lt;li id="StrikeThrough" title="Strike Through"&gt;
    *        &lt;div class="ToolbarItems StrikeThrough_tool"&gt;&lt;/div&gt;
    *    &lt;/li&gt;
    *    &lt;li id="UndeLine" title="UnderLine"&gt;
    *        &lt;div class="ToolbarItems Underline_tool"&gt;&lt;/div&gt;
    *    &lt;/li&gt;
    *&lt;/ul&gt;
	*&lt;/div&gt;
	* &lt;script&gt;
			* // To set fields API value during initialization. 
			* 	//To set the fields API value.
			* 	items = [{ edid: "1", spriteCssClass: "editTools cursor", text: "Cursor" },
			*  { edid: "2", spriteCssClass: "editTools select", text: "Select" },
			*  { edid: "3", spriteCssClass: "editTools move", text: "Move" },
			*  { edid: "4", spriteCssClass: "editTools rectselect", text: "Rectangle Select"
			*  }];
			*$("#toolbar1").ejToolbar(
			*		{
			*			dataSource:  items,
			*           fields:  { id: "empid",spriteCssClass: "spriteCss"}
            *         });
			* &lt;/script&gt;
			* @memberof ejToolbar
			* @instance
			*/
            fields: /** @lends ejToolbar# */ {
				/**		
                 * Defines id for the tag.
				 * @alias ejToolbar#fields->id
				 * @type String
                 */
                id:"id",
				/**		
                 * Defines the tooltip text for the tag.
				 * @alias ejToolbar#fields->tooltipText
				 * @type String
                 */
                tooltipText:"tooltipText",
				/**		
                 * Defines the imageURL for the image location. 
				 * @alias ejToolbar#fields->imageUrl
				 * @type String
                 */
                imageUrl:"imageUrl",
				/**		
                 * Defines the text content for the tag.
				 * @alias ejToolbar#fields->text
				 * @type String
                 */
                 text:"text",
				/**		
                 * Defines the image attributes such as height, width, styles and so on.
				 * @alias ejToolbar#fields->imageAttributes
				 * @type String
                 */
                imageAttributes:"imageAttributes",
				/**		
                 * Defines the sprite css for the image tag.
				 * @alias ejToolbar#fields->spriteCssClass
				 * @type String
                 */
                spriteCssClass:"spriteCSS",
				/**		
                 * Defines the html attributes such as id, class, styles for the item.
				 * @alias ejToolbar#fields->htmlAttributes
				 * @type Object
                 */
                htmlAttributes: "htmlAttributes",
				group: null
            },
			/**		
			* Sets the root class for Toolbar control theme
			* @default ""
			* @type {string}
			* @example 
			* &lt;div id="toolbar1"&gt;
    *&lt;ul&gt;
    *    &lt;li id="Left" title="Left"&gt;
    *        &lt;div class="ToolbarItems LeftAlign_tool"&gt;&lt;/div&gt;
    *   &lt;/li&gt;
    *    &lt;li id="Center" title="Center"&gt;
    *        &lt;div class="ToolbarItems CenterAlign_tool"&gt;&lt;/div&gt;
    *    &lt;/li&gt;
    *    &lt;li id="Right" title="Right"&gt;
    *        &lt;div class="ToolbarItems RightAlign_tool"&gt;&lt;/div&gt;
    *    &lt;/li&gt;
    *    &lt;li id="Justify" title="Justify"&gt;
    *        &lt;div class="ToolbarItems Justify_tool"&gt;&lt;/div&gt;
    *    &lt;/li&gt;
    *&lt;/ul&gt;
    *&lt;ul&gt;
    *    &lt;li id="Bold" title="Bold"&gt;
    *        &lt;div class="ToolbarItems Bold_tool"&gt;&lt;/div&gt;
    *    &lt;/li&gt;
    *    &lt;li id="Italic" title="Italic"&gt;
    *        &lt;div class="ToolbarItems Italic_tool"&gt;&lt;/div&gt;
    *    &lt;/li&gt;
    *    &lt;li id="StrikeThrough" title="Strike Through"&gt;
    *        &lt;div class="ToolbarItems StrikeThrough_tool"&gt;&lt;/div&gt;
    *    &lt;/li&gt;
    *    &lt;li id="UndeLine" title="UnderLine"&gt;
    *        &lt;div class="ToolbarItems Underline_tool"&gt;&lt;/div&gt;
    *    &lt;/li&gt;
    *&lt;/ul&gt;
	*&lt;/div&gt;
	* &lt;script&gt;
			* // Sets the root class for Toolbar control theme on initialization. 
			* 	//To set the cssClass API value.
			* 	$("#toolbar1").ejToolbar({cssClass: "gradient-lime"});
			 * &lt;/script&gt;
			* @memberof ejToolbar
			* @instance
			*/
            cssClass: "",
			/**     
			 * Fires after Toolbar control is created.
			 * @event
			 * @name ejToolbar#create		
			 * @param {Object} argument Event parameters from Toolbar    
			 * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
			 * @param {object}  argument.model returns the Toolbar model
			 * @param {string}  argument.type returns the name of the event
			 * @example 
			 * &lt;div id="toolbar1"&gt;
    *&lt;ul&gt;
    *    &lt;li id="Left" title="Left"&gt;
    *        &lt;div class="ToolbarItems LeftAlign_tool"&gt;&lt;/div&gt;
    *   &lt;/li&gt;
    *    &lt;li id="Center" title="Center"&gt;
    *        &lt;div class="ToolbarItems CenterAlign_tool"&gt;&lt;/div&gt;
    *    &lt;/li&gt;
    *    &lt;li id="Right" title="Right"&gt;
    *        &lt;div class="ToolbarItems RightAlign_tool"&gt;&lt;/div&gt;
    *    &lt;/li&gt;
    *    &lt;li id="Justify" title="Justify"&gt;
    *        &lt;div class="ToolbarItems Justify_tool"&gt;&lt;/div&gt;
    *    &lt;/li&gt;
    *&lt;/ul&gt;
    *&lt;ul&gt;
    *    &lt;li id="Bold" title="Bold"&gt;
    *        &lt;div class="ToolbarItems Bold_tool"&gt;&lt;/div&gt;
    *    &lt;/li&gt;
    *    &lt;li id="Italic" title="Italic"&gt;
    *        &lt;div class="ToolbarItems Italic_tool"&gt;&lt;/div&gt;
    *    &lt;/li&gt;
    *    &lt;li id="StrikeThrough" title="Strike Through"&gt;
    *        &lt;div class="ToolbarItems StrikeThrough_tool"&gt;&lt;/div&gt;
    *    &lt;/li&gt;
    *    &lt;li id="UndeLine" title="UnderLine"&gt;
    *        &lt;div class="ToolbarItems Underline_tool"&gt;&lt;/div&gt;
    *    &lt;/li&gt;
    *&lt;/ul&gt;
	*&lt;/div&gt;
	* &lt;script&gt;
			 * //create event for Toolbar
             *  $("#toolbar1").ejToolbar({
			 *  create: function (args) {}
             * });
             * &lt;/script&gt;
			 * @memberof ejToolbar
			 * @instance
			 */		
            create: null,
			/**     
			 * Fires after Toolbar control is clicked.
			 * @event
			 * @name ejToolbar#click		
			 * @param {Object} argument Event parameters from Toolbar    
			 * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
			 * @param {object}  argument.model returns the Toolbar model
			 * @param {string}  argument.type returns the name of the event
			 * @param {object} argument.target  returns the target of the current object.
			 * @param {object} argument.currentTarget returns the target of the current object. 
			 * @param {boolean}  argument.status return the Toolbar state
			 * @example 
			 * &lt;div id="toolbar1"&gt;
    *&lt;ul&gt;
    *    &lt;li id="Left" title="Left"&gt;
    *        &lt;div class="ToolbarItems LeftAlign_tool"&gt;&lt;/div&gt;
    *   &lt;/li&gt;
    *    &lt;li id="Center" title="Center"&gt;
    *        &lt;div class="ToolbarItems CenterAlign_tool"&gt;&lt;/div&gt;
    *    &lt;/li&gt;
    *    &lt;li id="Right" title="Right"&gt;
    *        &lt;div class="ToolbarItems RightAlign_tool"&gt;&lt;/div&gt;
    *    &lt;/li&gt;
    *    &lt;li id="Justify" title="Justify"&gt;
    *        &lt;div class="ToolbarItems Justify_tool"&gt;&lt;/div&gt;
    *    &lt;/li&gt;
    *&lt;/ul&gt;
    *&lt;ul&gt;
    *    &lt;li id="Bold" title="Bold"&gt;
    *        &lt;div class="ToolbarItems Bold_tool"&gt;&lt;/div&gt;
    *    &lt;/li&gt;
    *    &lt;li id="Italic" title="Italic"&gt;
    *        &lt;div class="ToolbarItems Italic_tool"&gt;&lt;/div&gt;
    *    &lt;/li&gt;
    *    &lt;li id="StrikeThrough" title="Strike Through"&gt;
    *        &lt;div class="ToolbarItems StrikeThrough_tool"&gt;&lt;/div&gt;
    *    &lt;/li&gt;
    *    &lt;li id="UndeLine" title="UnderLine"&gt;
    *        &lt;div class="ToolbarItems Underline_tool"&gt;&lt;/div&gt;
    *    &lt;/li&gt;
    *&lt;/ul&gt;
	*&lt;/div&gt;
	* &lt;script&gt;
			 * //create event for Toolbar
             *  $("#toolbar1").ejToolbar({
			 *  click: function (args) {}
             * });
             * &lt;/script&gt;
			 * @memberof ejToolbar
			 * @instance
			 */	
            click: null,
					/**     
			 * Fires after Toolbar control is itemHovered.
			 * @event
			 * @name ejToolbar#itemHover		
			 * @param {Object} argument Event parameters from Toolbar    
			 * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
			 * @param {object}  argument.model returns the Toolbar model
			 * @param {string}  argument.type returns the name of the event
			 * @param {object} argument.target  returns the target of the current object.
			 * @param {object} argument.currentTarget returns the target of the current object. 
			 * @param {boolean}  argument.status return the Toolbar state
			 * @example 
			 * &lt;div id="toolbar1"&gt;
    *&lt;ul&gt;
    *    &lt;li id="Left" title="Left"&gt;
    *        &lt;div class="ToolbarItems LeftAlign_tool"&gt;&lt;/div&gt;
    *   &lt;/li&gt;
    *    &lt;li id="Center" title="Center"&gt;
    *        &lt;div class="ToolbarItems CenterAlign_tool"&gt;&lt;/div&gt;
    *    &lt;/li&gt;
    *    &lt;li id="Right" title="Right"&gt;
    *        &lt;div class="ToolbarItems RightAlign_tool"&gt;&lt;/div&gt;
    *    &lt;/li&gt;
    *    &lt;li id="Justify" title="Justify"&gt;
    *        &lt;div class="ToolbarItems Justify_tool"&gt;&lt;/div&gt;
    *    &lt;/li&gt;
    *&lt;/ul&gt;
    *&lt;ul&gt;
    *    &lt;li id="Bold" title="Bold"&gt;
    *        &lt;div class="ToolbarItems Bold_tool"&gt;&lt;/div&gt;
    *    &lt;/li&gt;
    *    &lt;li id="Italic" title="Italic"&gt;
    *        &lt;div class="ToolbarItems Italic_tool"&gt;&lt;/div&gt;
    *    &lt;/li&gt;
    *    &lt;li id="StrikeThrough" title="Strike Through"&gt;
    *        &lt;div class="ToolbarItems StrikeThrough_tool"&gt;&lt;/div&gt;
    *    &lt;/li&gt;
    *    &lt;li id="UndeLine" title="UnderLine"&gt;
    *        &lt;div class="ToolbarItems Underline_tool"&gt;&lt;/div&gt;
    *    &lt;/li&gt;
    *&lt;/ul&gt;
	*&lt;/div&gt;
	* &lt;script&gt;
			 * //itemHover event for Toolbar
             *  $("#toolbar1").ejToolbar({
			 *  itemHover: function (args) {}
             * });
             * &lt;/script&gt;
			 * @memberof ejToolbar
			 * @instance
			 */
            itemHover: null,
			/**     
			 * Fires after Toolbar control is itemLeave.
			 * @event
			 * @name ejToolbar#itemLeave		
			 * @param {Object} argument Event parameters from Toolbar    
			 * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
			 * @param {object}  argument.model returns the Toolbar model
			 * @param {string}  argument.type returns the name of the event
			 * @param {object} argument.target  returns the target of the current object.
			 * @param {object} argument.currentTarget returns the target of the current object. 
			 * @param {boolean}  argument.status return the Toolbar state
			 * @example 
			 * &lt;div id="toolbar1"&gt;
    *&lt;ul&gt;
    *    &lt;li id="Left" title="Left"&gt;
    *        &lt;div class="ToolbarItems LeftAlign_tool"&gt;&lt;/div&gt;
    *   &lt;/li&gt;
    *    &lt;li id="Center" title="Center"&gt;
    *        &lt;div class="ToolbarItems CenterAlign_tool"&gt;&lt;/div&gt;
    *    &lt;/li&gt;
    *    &lt;li id="Right" title="Right"&gt;
    *        &lt;div class="ToolbarItems RightAlign_tool"&gt;&lt;/div&gt;
    *    &lt;/li&gt;
    *    &lt;li id="Justify" title="Justify"&gt;
    *        &lt;div class="ToolbarItems Justify_tool"&gt;&lt;/div&gt;
    *    &lt;/li&gt;
    *&lt;/ul&gt;
    *&lt;ul&gt;
    *    &lt;li id="Bold" title="Bold"&gt;
    *        &lt;div class="ToolbarItems Bold_tool"&gt;&lt;/div&gt;
    *    &lt;/li&gt;
    *    &lt;li id="Italic" title="Italic"&gt;
    *        &lt;div class="ToolbarItems Italic_tool"&gt;&lt;/div&gt;
    *    &lt;/li&gt;
    *    &lt;li id="StrikeThrough" title="Strike Through"&gt;
    *        &lt;div class="ToolbarItems StrikeThrough_tool"&gt;&lt;/div&gt;
    *    &lt;/li&gt;
    *    &lt;li id="UndeLine" title="UnderLine"&gt;
    *        &lt;div class="ToolbarItems Underline_tool"&gt;&lt;/div&gt;
    *    &lt;/li&gt;
    *&lt;/ul&gt;
	*&lt;/div&gt;
	* &lt;script&gt;
			 * //itemLeave event for Toolbar
             *  $("#toolbar1").ejToolbar({
			 *  itemLeave: function (args) {}
             * });
            * &lt;/script&gt;
			 * @memberof ejToolbar
			 * @instance
			 */
            itemLeave: null,
				 /**    
			 * Fires when the Toolbar is destroyed successfully.
			 * @event
			 * @name ejToolbar#destroy 	
			 * @param {Object} argument Event parameters from Toolbar    
			 * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
			 * @param {object}  argument.model returns the Toolbar model
			 * @param {string}  argument.type returns the name of the event
			 * @example 
			 * &lt;div id="toolbar1"&gt;
    *&lt;ul&gt;
    *    &lt;li id="Left" title="Left"&gt;
    *        &lt;div class="ToolbarItems LeftAlign_tool"&gt;&lt;/div&gt;
    *   &lt;/li&gt;
    *    &lt;li id="Center" title="Center"&gt;
    *        &lt;div class="ToolbarItems CenterAlign_tool"&gt;&lt;/div&gt;
    *    &lt;/li&gt;
    *    &lt;li id="Right" title="Right"&gt;
    *        &lt;div class="ToolbarItems RightAlign_tool"&gt;&lt;/div&gt;
    *    &lt;/li&gt;
    *    &lt;li id="Justify" title="Justify"&gt;
    *        &lt;div class="ToolbarItems Justify_tool"&gt;&lt;/div&gt;
    *    &lt;/li&gt;
    *&lt;/ul&gt;
    *&lt;ul&gt;
    *    &lt;li id="Bold" title="Bold"&gt;
    *        &lt;div class="ToolbarItems Bold_tool"&gt;&lt;/div&gt;
    *    &lt;/li&gt;
    *    &lt;li id="Italic" title="Italic"&gt;
    *        &lt;div class="ToolbarItems Italic_tool"&gt;&lt;/div&gt;
    *    &lt;/li&gt;
    *    &lt;li id="StrikeThrough" title="Strike Through"&gt;
    *        &lt;div class="ToolbarItems StrikeThrough_tool"&gt;&lt;/div&gt;
    *    &lt;/li&gt;
    *    &lt;li id="UndeLine" title="UnderLine"&gt;
    *        &lt;div class="ToolbarItems Underline_tool"&gt;&lt;/div&gt;
    *    &lt;/li&gt;
    *&lt;/ul&gt;
	*&lt;/div&gt;
	* &lt;script&gt;
			 * //destroy event for Toolbar

             * $("#toolbar1").ejToolbar({
             *    destroy: function (args) {}
             * });
             * &lt;/script&gt;
			 * @memberof ejToolbar
			 * @instance
			 */
            destroy: null

        },
		/**
         * Specify the data types for default properties 
		 * @private
         */
        //Data Types
        dataTypes: {           
            enabled: "boolean",
            hide: "boolean",
            enableSeparator: "boolean",
            orientation: "enum",
            enableRTL: "boolean",
            showRoundedCorner: "boolean",
            allowResponsive: "boolean",
            dataSource: "data",
            query: "data",
            fields: "data",
            cssClass: "string"
        },

        // sample public function
		/**
        * To hide the Toolbar 		
		* @return jQuery
		* @example 
		* &lt;div id="toolbar1"&gt;
    *&lt;ul&gt;
    *    &lt;li id="Left" title="Left"&gt;
    *        &lt;div class="ToolbarItems LeftAlign_tool"&gt;&lt;/div&gt;
    *   &lt;/li&gt;
    *    &lt;li id="Center" title="Center"&gt;
    *        &lt;div class="ToolbarItems CenterAlign_tool"&gt;&lt;/div&gt;
    *    &lt;/li&gt;
    *    &lt;li id="Right" title="Right"&gt;
    *        &lt;div class="ToolbarItems RightAlign_tool"&gt;&lt;/div&gt;
    *    &lt;/li&gt;
    *    &lt;li id="Justify" title="Justify"&gt;
    *        &lt;div class="ToolbarItems Justify_tool"&gt;&lt;/div&gt;
    *    &lt;/li&gt;
    *&lt;/ul&gt;
    *&lt;ul&gt;
    *    &lt;li id="Bold" title="Bold"&gt;
    *        &lt;div class="ToolbarItems Bold_tool"&gt;&lt;/div&gt;
    *    &lt;/li&gt;
    *    &lt;li id="Italic" title="Italic"&gt;
    *        &lt;div class="ToolbarItems Italic_tool"&gt;&lt;/div&gt;
    *    &lt;/li&gt;
    *    &lt;li id="StrikeThrough" title="Strike Through"&gt;
    *        &lt;div class="ToolbarItems StrikeThrough_tool"&gt;&lt;/div&gt;
    *    &lt;/li&gt;
    *    &lt;li id="UndeLine" title="UnderLine"&gt;
    *        &lt;div class="ToolbarItems Underline_tool"&gt;&lt;/div&gt;
    *    &lt;/li&gt;
    *&lt;/ul&gt;
	*&lt;/div&gt;
	* &lt;script&gt;
		* &lt;div id="toolbar1"&gt;
		*&lt;ul&gt;
		*    &lt;li id="Left" title="Left"&gt;
		*        &lt;div class="ToolbarItems LeftAlign_tool"&gt;&lt;/div&gt;
		*   &lt;/li&gt;
		*    &lt;li id="Center" title="Center"&gt;
		*        &lt;div class="ToolbarItems CenterAlign_tool"&gt;&lt;/div&gt;
		*    &lt;/li&gt;
		*    &lt;li id="Right" title="Right"&gt;
		*        &lt;div class="ToolbarItems RightAlign_tool"&gt;&lt;/div&gt;
		*    &lt;/li&gt;
		*    &lt;li id="Justify" title="Justify"&gt;
		*        &lt;div class="ToolbarItems Justify_tool"&gt;&lt;/div&gt;
		*    &lt;/li&gt;
		* 	&lt;/ul&gt;
		*	&lt;ul&gt;
		*    &lt;li id="Bold" title="Bold"&gt;
		*        &lt;div class="ToolbarItems Bold_tool"&gt;&lt;/div&gt;
		*    &lt;/li&gt;
		*    &lt;li id="Italic" title="Italic"&gt;
		*        &lt;div class="ToolbarItems Italic_tool"&gt;&lt;/div&gt;
		*    &lt;/li&gt;
		*    &lt;li id="StrikeThrough" title="Strike Through"&gt;
		*        &lt;div class="ToolbarItems StrikeThrough_tool"&gt;&lt;/div&gt;
		*    &lt;/li&gt;
		*    &lt;li id="UndeLine" title="UnderLine"&gt;
		*        &lt;div class="ToolbarItems Underline_tool"&gt;&lt;/div&gt;
		*    &lt;/li&gt;
		*&lt;/ul&gt;
		*&lt;/div&gt;
		* &lt;script&gt;
		* $("#toolbar1").ejToolbar("hide");// to hide the toolbar
		* &lt;/script&gt;
		* @memberof ejToolbar
		* @instance
         */
        hide: function () {
            /// <summary>This will set hide function of toolbar </summary>
            if (!this.model.enabled) return false;
            this.element.css("display", "none");
        },
		/**
        * To show the Toolbar 		
		* @return jQuery
		* @example 
		* &lt;div id="toolbar1"&gt;
		*&lt;ul&gt;
		*    &lt;li id="Left" title="Left"&gt;
		*        &lt;div class="ToolbarItems LeftAlign_tool"&gt;&lt;/div&gt;
		*   &lt;/li&gt;
		*    &lt;li id="Center" title="Center"&gt;
		*        &lt;div class="ToolbarItems CenterAlign_tool"&gt;&lt;/div&gt;
		*    &lt;/li&gt;
		*    &lt;li id="Right" title="Right"&gt;
		*        &lt;div class="ToolbarItems RightAlign_tool"&gt;&lt;/div&gt;
		*    &lt;/li&gt;
		*    &lt;li id="Justify" title="Justify"&gt;
		*        &lt;div class="ToolbarItems Justify_tool"&gt;&lt;/div&gt;
		*    &lt;/li&gt;
		* 	&lt;/ul&gt;
		*	&lt;ul&gt;
		*    &lt;li id="Bold" title="Bold"&gt;
		*        &lt;div class="ToolbarItems Bold_tool"&gt;&lt;/div&gt;
		*    &lt;/li&gt;
		*    &lt;li id="Italic" title="Italic"&gt;
		*        &lt;div class="ToolbarItems Italic_tool"&gt;&lt;/div&gt;
		*    &lt;/li&gt;
		*    &lt;li id="StrikeThrough" title="Strike Through"&gt;
		*        &lt;div class="ToolbarItems StrikeThrough_tool"&gt;&lt;/div&gt;
		*    &lt;/li&gt;
		*    &lt;li id="UndeLine" title="UnderLine"&gt;
		*        &lt;div class="ToolbarItems Underline_tool"&gt;&lt;/div&gt;
		*    &lt;/li&gt;
		*&lt;/ul&gt;
		*&lt;/div&gt;
		* &lt;script&gt;
		* 	$("#toolbar1").ejToolbar();
		* $("#toolbar1").ejToolbar("show");// to show the toolbar
		* &lt;/script&gt;
		* @memberof ejToolbar
		* @instance
         */
        show: function () {
            /// <summary>This will set show function of toolbar </summary>
            if (!this.model.enabled) return false;
            this.element.css("display", "inline-block");
        },
		
		/**
         * Create the Toolbar widget
		 * @private
         */		 

        // constructor function
        _init: function () {
            /// <summary>This will set initialization of toolbar </summary>
            if (this.model.dataSource != null) {
                this._generateTemplate();
            }
            this._initialize();

        },
		 /**
         * To configure the properties at runtime using SetModel		
		 * @private
         */	
        _setModel: function (options) {
            if (!(this.model.enabled) && ej.isNullOrUndefined(options["enabled"])) return false;
            var option;
            for (option in options) {
                switch (option) {
                    case "height":
                        this._setHeight(options[option]);
                        break;
                    case "width":
                        this._setWidth(options[option]);
                        break;
                    case "enabled":
                        this._controlStatus(options[option]);
                        break;
                    case "hide":
                        this._controlVisibleOptions(options[option]);
                        break;
                    case "orientation":
                        this._setOrientation(options[option]);
                        break;
                    case "enableRTL":
                        this._enableRTL(options[option]);
                        break;
                    case "showRoundedCorner":
                        this._roundedCorner(options[option]);
                        break;
                    case "cssClass":
                        this._setSkin(options[option]);
                        break;
				    case "fields":
                    case "query":
                    case "dataSource":
                       this._refreshTagItems(option, options[option]);
                        break;
                }
            }
        },
		_refreshTagItems:function(key,value){
				this.model[key] = value;
				this.element.empty();
                this._generateTemplate();
				this._initialize();
		},
		/**
         * To configure the height		
		 * @private
         */	 
        _setHeight: function (val) {
            /// <summary>This will set Text property of button </summary>
            this.element.css('height',val);
        },
		/**
         * To configure the width		
		 * @private
         */	 
        _setWidth: function (val) {
            /// <summary>This will set Text property of button </summary>
            this.element.css('width',val);
        },
		/**
         * To configure the orientation		
		 * @private
         */	
        _setOrientation: function (value) {
            /// <summary>This will set orientation of toolbar </summary>
            if (value != ej.Orientation.Vertical) {
                this.element.removeClass(this.model.cssClass);
                this.element.addClass("e-toolbarspan " + this.model.cssClass);
                this.itemsContainer.removeClass("e-ul e-vertical");
                this.itemsContainer.addClass("e-ul e-horizontal");
                //this.items.removeClass("e-vertical");
                //this.items.addClass("e-horizontal");

            } else {
                this.element.removeClass("e-toolbarspan " + this.model.cssClass);
                this.element.addClass(this.model.cssClass);
                this.itemsContainer.removeClass("e-ul e-horizontal");
                this.itemsContainer.addClass("e-ul e-vertical");
                //this.items.removeClass("e-horizontal");
                //this.items.addClass("e-vertical");
            }
            this.items.children("img").addClass("e-align e-spriteimg");
            this.items.children("span").addClass("e-align e-spriteimg");
            this.items.children("div").addClass("e-align e-spriteimg");
            this.items.addClass("e-tooltxt");
        },
        /**
         * To handle the window resize
         * @private
         */
        _reSizeHandler: function () {
            this._elementWidth = (typeof window.getComputedStyle == "function") ? parseFloat(window.getComputedStyle(this.element[0]).width) : this.element.width()-1; // -1 for IE8 get exact width of an element no border and margin
            if(this.isResized && this._liTemplte.children().length > 0){
                $(this.element).append(this._liTemplte.children());
                this._renderToolbarSeparator();
                this._liTemplte.addClass("e-display-none");
                this.contstatus = false;
            }
            this._liWidth = this._spanWidth;
            this._spantag.removeClass("e-display-block").addClass("e-display-none");
            for(var i=0; i<this.itemsContainer.length; i++){
                this._liWidth += $(this.itemsContainer[i]).outerWidth(true);
                if(this._liWidth > this._elementWidth){
                    if(!this._spantag.hasClass("e-display-block"))
                        this._spantag.removeClass("e-display-none").addClass("e-display-block");
                    this._spantag.css("float",(this.model.enableRTL ? "left" : "right"));
                    if($(this.itemsContainer[i]).hasClass("e-separator"))
                        $(this.itemsContainer[i]).removeClass("e-separator");
                    this._liTemplte.append(this.itemsContainer[i]);
                }
            }
            this.isResized = true;
        },
        /**
         * To handle the overflow items of toolbar is dispalyed as context menu 
         * @private
         */
        _btnMouseClick: function (e) {
            var args, btnpos, btnposx, btnposy, targetElement, poscur = 1;
            if (!$(e.currentTarget).hasClass("e-disable")) {
                /*Context Menu Functionality*///need to write enableRTL
                targetElement = $("#" + this.element[0].id + "_target");
                btnpos = $("#" + this.element[0].id + "_target").offset();
                btnposx = btnpos.left - poscur;
                if($(this._liTemplte).width() + btnposx > $(window).width())
                    btnposx = $(window).width() - $(this._liTemplte).width() - 3; // 3px difference between popup right and window right side
                btnposy = btnpos.top + this.element.outerHeight(true) - poscur - 2; // 2px alignment in responsive image position from top
                if (this.contstatus) {
                    this._liTemplte.addClass("e-display-none");
                    this.contstatus = false;
                } else {
                    if (this.model.enableRTL) {
                        this._liTemplte.removeClass("e-display-none");
                        this._liTemplte.css({"top":btnposy+"px","left":btnposx+"px"});
                        this.contstatus = true;
                    } else {   
                        this._liTemplte.removeClass("e-display-none");
                        this._liTemplte.css({"top":btnposy+"px","left":btnposx+"px"});
                        this.contstatus = true;
                    }
                    this.element.bind("click", $.proxy(this._hidecontext, this));
                }
                if (this.contstatus)
                    this._on($(document), "mouseup", this._documentClick);
            }
        },
        /**
         * Section For handle hidecontext action.
         * @private
         */
        _hidecontext: function (e) {
            if (!$(e.target).is($("#" + this.element[0].id + "_target"))) {
                this._liTemplte.addClass("e-display-none");
                this.contstatus = false;
                this.element.unbind("click", $.proxy(this._hidecontext, this));
                this._off($(document), "mouseup", this._documentClick);
            }
        },
        /**
         * Section For handle on document click action.
         * @private
         */
        _documentClick: function (e) {
            if (!$(e.target).is($("#" + this.element[0].id + "_target")) && !($(e.target).closest("div#" +this.element[0].id + "_hiddenlist").length != 0 || $(e.target).parents().hasClass("e-ddl-popup"))) {
                this._liTemplte.addClass("e-display-none");
                this.contstatus = false;
                this._off($(document), "mouseup", this._documentClick);
            }
        },
		/**
         * To configure the custom theme for Toolbar using cssClass property			
		 * @private
         */	
        _setSkin: function (skin) {
            this.element.removeClass(this.model.cssClass);
            this.itemsContainer.removeClass(this.model.cssClass);
            this.items.removeClass(this.model.cssClass);
            this.element.addClass(skin);
            this.itemsContainer.addClass(skin);
            this.items.addClass(skin);
        },
		/**
        * destroy the Toolbar widget
		* all events bound using this._on will be unbind automatically and bring the control to pre-init state.
		* @alias destroy
		* @return jQuery
		* @example 
	   * &lt;div id="toolbar1"&gt;
		*&lt;ul&gt;
		*    &lt;li id="Left" title="Left"&gt;
		*        &lt;div class="ToolbarItems LeftAlign_tool"&gt;&lt;/div&gt;
		*   &lt;/li&gt;
		*    &lt;li id="Center" title="Center"&gt;
		*        &lt;div class="ToolbarItems CenterAlign_tool"&gt;&lt;/div&gt;
		*    &lt;/li&gt;
		*    &lt;li id="Right" title="Right"&gt;
		*        &lt;div class="ToolbarItems RightAlign_tool"&gt;&lt;/div&gt;
		*    &lt;/li&gt;
		*    &lt;li id="Justify" title="Justify"&gt;
		*        &lt;div class="ToolbarItems Justify_tool"&gt;&lt;/div&gt;
		*    &lt;/li&gt;
		* 	&lt;/ul&gt;
		*	&lt;ul&gt;
		*    &lt;li id="Bold" title="Bold"&gt;
		*        &lt;div class="ToolbarItems Bold_tool"&gt;&lt;/div&gt;
		*    &lt;/li&gt;
		*    &lt;li id="Italic" title="Italic"&gt;
		*        &lt;div class="ToolbarItems Italic_tool"&gt;&lt;/div&gt;
		*    &lt;/li&gt;
		*    &lt;li id="StrikeThrough" title="Strike Through"&gt;
		*        &lt;div class="ToolbarItems StrikeThrough_tool"&gt;&lt;/div&gt;
		*    &lt;/li&gt;
		*    &lt;li id="UndeLine" title="UnderLine"&gt;
		*        &lt;div class="ToolbarItems Underline_tool"&gt;&lt;/div&gt;
		*    &lt;/li&gt;
		*&lt;/ul&gt;
		*&lt;/div&gt;
		* &lt;script&gt;
		* &lt;script&gt;
		* //Initialize the Toolbar object.
		*$("#toolbar1").ejToolbar();
		* var toolbarObj = $("#toolbar1").data("ejToolbar");
        *  toolbarObj.destroy();       //Calls the destroy method of the Toolbar.
		* &lt;/script&gt;
				
		* @memberof ejToolbar
		* @instance
         */	 
        _destroy: function () {
            /// <summary>This will destroy toolbar </summary>
            this.element.removeClass("e-widget e-horizontal " + this.model.cssClass);
            this.element.removeClass("e-vertical " + this.model.cssClass);
            this.itemsContainer.removeClass("e-horizontal " + this.model.cssClass);
            this.itemsContainer.removeClass("e-vertical " + this.model.cssClass);
            this.items.removeClass("e-horizontal " + this.model.cssClass);
            this.items.removeClass("e-vertical " + this.model.cssClass);
        },
		/**
         * To initialize the ToolBar		
		 * @private
         */	 
        _initialize: function () {
            /// <summary>This will initialize the toolbar </summary>
            /*Rendering the controls*/
            $(this.element).attr({ "role": "toolbar", "tabindex": "0" });
            var liList = $($(this.element).children()).find("li");
            this._focusEnable = true;
            this._renderControl();
            this._roundedCorner(this.model.showRoundedCorner);
            if(this.model.allowResponsive && (this.model.orientation == "horizontal")){
                this._spantag = $("<span id='" + this.element[0].id + "_target' class='e-icon e-toolbar-res-arrow e-rel-position e-display-block'></span>");
                this._spantag.appendTo(this.element);
                this._spanWidth = this._spantag.outerWidth(true) + 8; // 8px from right of span 
                this._liTemplte = $("<div id='" + this.element[0].id + "_hiddenlist' class='e-responsive-toolbar e-display-none e-abs-position "+$(this.element)[0].className+"'></div>");
                this.isResized = false;
                this._reSizeHandler();
                this._liTemplte.insertAfter(this.element);
            }
            this._wireEvents();
            this._controlVisibleOptions(this.model.hide);
            this._enableRTL(this.model.enableRTL);
            this._controlStatus(this.model.enabled);
        },
		/**
         * To configure controlVisibleOption to show/hide the toolbar		
		 * @private
         */	
        _controlVisibleOptions: function (value) {
            /*Enable Disable Option*/
            value != false ? this.hide() : this.show();
        },
		/**
         * To configure controlStatus ie., to enable/disable the toolbar	
		 * @private
         */	
        _controlStatus: function (value) {
            //disable status
            value != true ? this.disable() : this.enable();
        },
		 /**
         * To enable or disable the rounded corner behaviour for the toolbar		
		 * @private
         */	
        _roundedCorner: function (value) {
            value != false ? this.element.addClass('e-corner-all') : this.element.removeClass('e-corner-all');
        },
		/**
         * To generate Template for the ToolBar control		
		 * @private
         */	
        _generateTemplate: function () {
            var proxy = this, queryPromise;
            this.element.css("visibility","hidden");
            if (this.model.dataSource instanceof ej.DataManager) {
                queryPromise = this.model.dataSource.executeQuery(this.model.query);
                queryPromise.done(function (e) {
                    if (proxy.model.fields.group) {
                        proxy._generateGroup(e.result);
                    }
                    else {
                        proxy.itemsSource = e.result;
                        proxy.element.append(proxy._generateTagitems());
                    }
                    proxy._initialize();
                    proxy.element.css("visibility", "visible");
                });
            } else {
                if (this.model.fields.group) {
                    proxy._generateGroup(proxy.model.dataSource);
                }
                else {
                    proxy.itemsSource = proxy.model.dataSource;
                    proxy.element.append(proxy._generateTagitems());
                }
                proxy.element.css("visibility", "visible");
            }
        },
        _generateGroup:function(value)
        {
            var proxy = this;
            var y = -1;
            var groupArray = [];
            for (var i = 0; i < value.length; i++) {
                if (this._isNewGroup(value[i].group, groupArray)) {
                    groupArray[++y] = value[i].group;
                    var index = -1;
                    proxy.itemsSource = [];
                    for (var x = i; x < value.length; x++) {
                        if (value[i].group == value[x].group) {
                            proxy.itemsSource[++index] = value[x];
                        }
                    }
                    proxy.element.append(proxy._generateTagitems());
                }
            }
            
        },
        _isNewGroup: function (value, group) {
            if (!group)
                return true;
            for (var i = 0; i < group.length; i++) {
                if (value == group[i])
                    return false;
            }
            return true;
        },
        //
		/**
         * Render Section For Toolbar	
		 * @private
         */			
        _renderControl: function () {
            /// <summary>This will render the toolbar </summary>
            this.element.addClass("e-widget");
            this._renderToolbarItems();
            this._setOrientation(this.model.orientation);
            this._renderToolbarSeparator();
            this._setHeight(this.model.height);
            this._setWidth(this.model.width);
        },
		/**
         * Render Section For ToolbarItems	
		 * @private
         */	
        _renderToolbarItems: function () {
            this.target = this.element[0];
            this.itemsContainer = this.element.children("ol,ul");
            this.itemsContainer.children("ol,ul").remove();
            this.items = this.itemsContainer.children('li');
            for (var i = 0; i < this.items.length; i++) {
                if (ej.isNullOrUndefined($(this.items[i]).attr("title")))
                    $(this.items[i]).attr("aria-label", this.items[i].id);
            }
            this._liCount = this.items.length;
        },
		/**
         * To generate Tag items for the ToolBar control		
		 * @private
         */	
        _generateTagitems: function () {
            var list, i;
            list = this.itemsSource;
            this.ultag = ej.buildTag('ul');
            for (i = 0; i < list.length; i++) {
                this.ultag.append(this._generateLi(list[i]));
            }
            return this.ultag;
        },
		/**
         * To generate Li  for the ToolBar control		
		 * @private
         */	
        _generateLi: function (toolbarItem) {
            var litag, imgtag, divtag, i;
            litag = ej.buildTag('li');
            if (toolbarItem[this.model.fields.id])
                litag.attr('id', toolbarItem[this.model.fields.id]);
            if (toolbarItem[this.model.fields.tooltipText])
                litag.attr('title', toolbarItem[this.model.fields.tooltipText]);
            if ((toolbarItem[this.model.fields.imageUrl]) && (toolbarItem[this.model.fields.imageUrl] != "")) {
                //Creating the image tag
                imgtag = ej.buildTag('img.e-align', '', {}, { 'src': toolbarItem[this.model.fields.imageUrl], 'alt': toolbarItem[this.model.fields.text] });
                if ((toolbarItem[this.model.fields.imageAttributes]) && (toolbarItem[this.model.fields.imageAttributes] != "")) {
                    for (i = 0; i < toolbarItem[this.model.fields.imageAttributes].length; i++) {
                        $.each(toolbarItem[this.model.fields.imageAttributes][i], function (index, element) {
                            imgtag.attr(index, element);
                        });
                    }
                }
                litag.append(imgtag);
            }
            if ((toolbarItem[this.model.fields.spriteCssClass]) && (toolbarItem[this.model.fields.spriteCssClass] != "")) {
                //Creating the Sprite image tag
                divtag = ej.buildTag('div.e-align ' + toolbarItem[this.model.fields.spriteCssClass] + ' e-spriteimg');
                litag.append(divtag);
            }
            if ((toolbarItem[this.model.fields.text]) && (toolbarItem[this.model.fields.text] != "")) {
                //Creating text Content inside the  tag
                litag.append(toolbarItem[this.model.fields.text]);
            }
            if ((toolbarItem[this.model.fields.htmlAttributes]) && (toolbarItem[this.model.fields.htmlAttributes] != "")) {
                for (i = 0; i < toolbarItem[this.model.fields.htmlAttributes].length; i++) {
                    $.each(toolbarItem[this.model.fields.htmlAttributes][i], function (index, element) {
                        litag.attr(index, element);
                    });
                }
            }
            return litag;
        },
	    /**
         * To renderToolbarSeparator for the ToolBar control		
		 * @private
         */	
        _renderToolbarSeparator: function () {
            /// <summary>This will render toolbar items </summary>
            /*Adding separator to each group of elements*/
            var i;
            if (this.model.enableSeparator) {
                for (i = 0; i < this.itemsContainer.length - 1; i++) {
                        $(this.itemsContainer[i]).addClass("e-separator");
                }
                if (this.model.enableRTL) {
                    if (this.itemsContainer.length == 1) {
                        for (i = 1; i < this.itemsContainer[0].children.length; i++) {
                            if (!this.model.fields.group || !this.model.dataSource || !this.model.dataSource[i].group)
                                $(this.itemsContainer[0].children[i]).addClass("e-separator");
                        }
                    }
                }
                else {
                    if (this.itemsContainer.length == 1) {
                        for (i = 0; i < this.itemsContainer[0].children.length - 1; i++) {
                            if (!this.model.fields.group || !this.model.dataSource || !this.model.dataSource[i].group)
                                $(this.itemsContainer[0].children[i]).addClass("e-separator");
                        }
                    }
                }
                
            }
        },
		/**
         * To enableRTL for the ToolBar control		
		 * @private
         */	
        _enableRTL: function (value) {
            if (value) {
                this.element.addClass('e-rtl');
                this.items.addClass('e-comnrtl');
                if (this.model.orientation == ej.Orientation.Horizontal) {
                    this.itemsContainer.addClass('e-comnrtl');
                }
            } else {
                this.element.removeClass('e-rtl');
                this.items.removeClass('e-comnrtl');
                if (this.model.orientation == ej.Orientation.Horizontal) {
                    this.itemsContainer.removeClass('e-comnrtl');
                }
            }
            this.model.enableRTL = value;
            this._renderToolbarSeparator();
        },
			/**
        * To disable an item the Toolbar 		
		* @return jQuery
		* @example 
		* &lt;div id="toolbar1"&gt;
		*&lt;ul&gt;
		*    &lt;li id="Left" title="Left"&gt;
		*        &lt;div class="ToolbarItems LeftAlign_tool"&gt;&lt;/div&gt;
		*   &lt;/li&gt;
		*    &lt;li id="Center" title="Center"&gt;
		*        &lt;div class="ToolbarItems CenterAlign_tool"&gt;&lt;/div&gt;
		*    &lt;/li&gt;
		*    &lt;li id="Right" title="Right"&gt;
		*        &lt;div class="ToolbarItems RightAlign_tool"&gt;&lt;/div&gt;
		*    &lt;/li&gt;
		*    &lt;li id="Justify" title="Justify"&gt;
		*        &lt;div class="ToolbarItems Justify_tool"&gt;&lt;/div&gt;
		*    &lt;/li&gt;
		* 	&lt;/ul&gt;
		*	&lt;ul&gt;
		*    &lt;li id="Bold" title="Bold"&gt;
		*        &lt;div class="ToolbarItems Bold_tool"&gt;&lt;/div&gt;
		*    &lt;/li&gt;
		*    &lt;li id="Italic" title="Italic"&gt;
		*        &lt;div class="ToolbarItems Italic_tool"&gt;&lt;/div&gt;
		*    &lt;/li&gt;
		*    &lt;li id="StrikeThrough" title="Strike Through"&gt;
		*        &lt;div class="ToolbarItems StrikeThrough_tool"&gt;&lt;/div&gt;
		*    &lt;/li&gt;
		*    &lt;li id="UndeLine" title="UnderLine"&gt;
		*        &lt;div class="ToolbarItems Underline_tool"&gt;&lt;/div&gt;
		*    &lt;/li&gt;
		*&lt;/ul&gt;
		*&lt;/div&gt;
		* &lt;script&gt;
		*$("#toolbar1").ejToolbar();
		* $("#toolbar1").ejToolbar("disableItem",$("li")[3]);// to disable the third item in the toolbar
		* &lt;/script&gt;
		* @memberof ejToolbar
		* @instance
         */
        //Disables the toolbarItem of given index
        disableItem: function (lielement) {
            /// <summary>This will disable toolbar item </summary>
            var current = $(lielement);
            if ((current == null) || (current.length <= 0)) {
                return;
            }
            if (!current.hasClass("e-disable")) {
                current.addClass("e-disable").attr("aria-disabled", true).removeAttr("aria-label");
            }
        },
        //Enables the toolbarItem of given index
				/**
        * To enable an item the Toolbar 		
		* @return jQuery
		* @example 
		* &lt;div id="toolbar1"&gt;
		*&lt;ul&gt;
		*    &lt;li id="Left" title="Left"&gt;
		*        &lt;div class="ToolbarItems LeftAlign_tool"&gt;&lt;/div&gt;
		*   &lt;/li&gt;
		*    &lt;li id="Center" title="Center"&gt;
		*        &lt;div class="ToolbarItems CenterAlign_tool"&gt;&lt;/div&gt;
		*    &lt;/li&gt;
		*    &lt;li id="Right" title="Right"&gt;
		*        &lt;div class="ToolbarItems RightAlign_tool"&gt;&lt;/div&gt;
		*    &lt;/li&gt;
		*    &lt;li id="Justify" title="Justify"&gt;
		*        &lt;div class="ToolbarItems Justify_tool"&gt;&lt;/div&gt;
		*    &lt;/li&gt;
		* 	&lt;/ul&gt;
		*	&lt;ul&gt;
		*    &lt;li id="Bold" title="Bold"&gt;
		*        &lt;div class="ToolbarItems Bold_tool"&gt;&lt;/div&gt;
		*    &lt;/li&gt;
		*    &lt;li id="Italic" title="Italic"&gt;
		*        &lt;div class="ToolbarItems Italic_tool"&gt;&lt;/div&gt;
		*    &lt;/li&gt;
		*    &lt;li id="StrikeThrough" title="Strike Through"&gt;
		*        &lt;div class="ToolbarItems StrikeThrough_tool"&gt;&lt;/div&gt;
		*    &lt;/li&gt;
		*    &lt;li id="UndeLine" title="UnderLine"&gt;
		*        &lt;div class="ToolbarItems Underline_tool"&gt;&lt;/div&gt;
		*    &lt;/li&gt;
		*&lt;/ul&gt;
		*&lt;/div&gt;
		* &lt;script&gt;
		*$("#toolbar1").ejToolbar();
		* $("#toolbar1").ejToolbar("enableItem",$("li")[3]);// to enable the third item in the toolbar
		* &lt;/script&gt;
		* @memberof ejToolbar
		* @instance
         */
        enableItem: function (lielement) {
            /// <summary>This will enable toolbar item </summary>
            var current = $(lielement);
            if ((current == null) || (current.length <= 0)) {
                return;
            }
            current.removeClass("e-disable").attr("aria-disabled", false);
        },
        //Disables the toolbarItem of given index
			/**
        * To Disable the Toolbar item by item id in the Toolbar 		
		* @return jQuery
		* @example 
		* &lt;div id="toolbar1"&gt;
		*&lt;ul&gt;
		*    &lt;li id="Left" title="Left"&gt;
		*        &lt;div class="ToolbarItems LeftAlign_tool"&gt;&lt;/div&gt;
		*   &lt;/li&gt;
		*    &lt;li id="Center" title="Center"&gt;
		*        &lt;div class="ToolbarItems CenterAlign_tool"&gt;&lt;/div&gt;
		*    &lt;/li&gt;
		*    &lt;li id="Right" title="Right"&gt;
		*        &lt;div class="ToolbarItems RightAlign_tool"&gt;&lt;/div&gt;
		*    &lt;/li&gt;
		*    &lt;li id="Justify" title="Justify"&gt;
		*        &lt;div class="ToolbarItems Justify_tool"&gt;&lt;/div&gt;
		*    &lt;/li&gt;
		* 	&lt;/ul&gt;
		*	&lt;ul&gt;
		*    &lt;li id="Bold" title="Bold"&gt;
		*        &lt;div class="ToolbarItems Bold_tool"&gt;&lt;/div&gt;
		*    &lt;/li&gt;
		*    &lt;li id="Italic" title="Italic"&gt;
		*        &lt;div class="ToolbarItems Italic_tool"&gt;&lt;/div&gt;
		*    &lt;/li&gt;
		*    &lt;li id="StrikeThrough" title="Strike Through"&gt;
		*        &lt;div class="ToolbarItems StrikeThrough_tool"&gt;&lt;/div&gt;
		*    &lt;/li&gt;
		*    &lt;li id="UndeLine" title="UnderLine"&gt;
		*        &lt;div class="ToolbarItems Underline_tool"&gt;&lt;/div&gt;
		*    &lt;/li&gt;
		*&lt;/ul&gt;
		*&lt;/div&gt;
		* &lt;script&gt;
		*$("#toolbar1").ejToolbar();
		*$("#toolbar1").ejToolbar("disableItemByID","left"); //Disable the Toolbar item by item id
		* &lt;/script&gt;
		* @memberof ejToolbar
		* @instance
         */
        disableItemByID: function (liid) {
            /// <summary>This will disable toolbar item by id </summary>
            var lielement;
            lielement = this.itemsContainer.find("li#" + liid);
            if ((lielement == null) || (lielement.length <= 0)) {
                return;
            }
            this.disableItem(lielement);
        },
			/**
        * To Disable the Toolbar item by item id in the Toolbar 		
		* @return jQuery
		* @example 
		* &lt;div id="toolbar1"&gt;
		*&lt;ul&gt;
		*    &lt;li id="Left" title="Left"&gt;
		*        &lt;div class="ToolbarItems LeftAlign_tool"&gt;&lt;/div&gt;
		*   &lt;/li&gt;
		*    &lt;li id="Center" title="Center"&gt;
		*        &lt;div class="ToolbarItems CenterAlign_tool"&gt;&lt;/div&gt;
		*    &lt;/li&gt;
		*    &lt;li id="Right" title="Right"&gt;
		*        &lt;div class="ToolbarItems RightAlign_tool"&gt;&lt;/div&gt;
		*    &lt;/li&gt;
		*    &lt;li id="Justify" title="Justify"&gt;
		*        &lt;div class="ToolbarItems Justify_tool"&gt;&lt;/div&gt;
		*    &lt;/li&gt;
		* 	&lt;/ul&gt;
		*	&lt;ul&gt;
		*    &lt;li id="Bold" title="Bold"&gt;
		*        &lt;div class="ToolbarItems Bold_tool"&gt;&lt;/div&gt;
		*    &lt;/li&gt;
		*    &lt;li id="Italic" title="Italic"&gt;
		*        &lt;div class="ToolbarItems Italic_tool"&gt;&lt;/div&gt;
		*    &lt;/li&gt;
		*    &lt;li id="StrikeThrough" title="Strike Through"&gt;
		*        &lt;div class="ToolbarItems StrikeThrough_tool"&gt;&lt;/div&gt;
		*    &lt;/li&gt;
		*    &lt;li id="UndeLine" title="UnderLine"&gt;
		*        &lt;div class="ToolbarItems Underline_tool"&gt;&lt;/div&gt;
		*    &lt;/li&gt;
		*&lt;/ul&gt;
		*&lt;/div&gt;
		* &lt;script&gt;
		*$("#toolbar1").ejToolbar();
		*$("#toolbar1").ejToolbar("enableItemByID","left"); //Disable the Toolbar item by item id
		* &lt;/script&gt;
		* @memberof ejToolbar
		* @instance
         */
        enableItemByID: function (liid) {
            /// <summary>This will disable toolbar item by id </summary>
            var lielement;
            lielement = this.itemsContainer.find("li#" + liid);
            if ((lielement == null) || (lielement.length <= 0)) {
                return;
            }
            this.enableItem(lielement);
        },
        //Disables all the toolbar item
		
			/**
        * To Disable all item in the Toolbar 		
		* @return jQuery
		* @example 
		* &lt;div id="toolbar1"&gt;
		*&lt;ul&gt;
		*    &lt;li id="Left" title="Left"&gt;
		*        &lt;div class="ToolbarItems LeftAlign_tool"&gt;&lt;/div&gt;
		*   &lt;/li&gt;
		*    &lt;li id="Center" title="Center"&gt;
		*        &lt;div class="ToolbarItems CenterAlign_tool"&gt;&lt;/div&gt;
		*    &lt;/li&gt;
		*    &lt;li id="Right" title="Right"&gt;
		*        &lt;div class="ToolbarItems RightAlign_tool"&gt;&lt;/div&gt;
		*    &lt;/li&gt;
		*    &lt;li id="Justify" title="Justify"&gt;
		*        &lt;div class="ToolbarItems Justify_tool"&gt;&lt;/div&gt;
		*    &lt;/li&gt;
		* 	&lt;/ul&gt;
		*	&lt;ul&gt;
		*    &lt;li id="Bold" title="Bold"&gt;
		*        &lt;div class="ToolbarItems Bold_tool"&gt;&lt;/div&gt;
		*    &lt;/li&gt;
		*    &lt;li id="Italic" title="Italic"&gt;
		*        &lt;div class="ToolbarItems Italic_tool"&gt;&lt;/div&gt;
		*    &lt;/li&gt;
		*    &lt;li id="StrikeThrough" title="Strike Through"&gt;
		*        &lt;div class="ToolbarItems StrikeThrough_tool"&gt;&lt;/div&gt;
		*    &lt;/li&gt;
		*    &lt;li id="UndeLine" title="UnderLine"&gt;
		*        &lt;div class="ToolbarItems Underline_tool"&gt;&lt;/div&gt;
		*    &lt;/li&gt;
		*&lt;/ul&gt;
		*&lt;/div&gt;
		* &lt;script&gt;
		*$("#toolbar1").ejToolbar();
		*$("#toolbar1").ejToolbar("disable"); //Disable all item in the Toolbar
		* &lt;/script&gt;
		* @memberof ejToolbar
		* @instance
         */
        disable: function () {
            /// <summary>This will disable all toolbar items </summary>
            this.items.addClass("e-disable");
            $(this.element).attr("aria-disabled", true).removeAttr("aria-label");
            if(this.model.allowResponsive && (this.model.orientation == "horizontal"))
                this._spantag.addClass("e-disable");
            this.model.enabled = false;
        },
        //Enables all the toolbar item
		/**
        * To enable all item in the Toolbar 		
		* @return jQuery
		* @example 
		* &lt;div id="toolbar1"&gt;
		*&lt;ul&gt;
		*    &lt;li id="Left" title="Left"&gt;
		*        &lt;div class="ToolbarItems LeftAlign_tool"&gt;&lt;/div&gt;
		*   &lt;/li&gt;
		*    &lt;li id="Center" title="Center"&gt;
		*        &lt;div class="ToolbarItems CenterAlign_tool"&gt;&lt;/div&gt;
		*    &lt;/li&gt;
		*    &lt;li id="Right" title="Right"&gt;
		*        &lt;div class="ToolbarItems RightAlign_tool"&gt;&lt;/div&gt;
		*    &lt;/li&gt;
		*    &lt;li id="Justify" title="Justify"&gt;
		*        &lt;div class="ToolbarItems Justify_tool"&gt;&lt;/div&gt;
		*    &lt;/li&gt;
		* 	&lt;/ul&gt;
		*	&lt;ul&gt;
		*    &lt;li id="Bold" title="Bold"&gt;
		*        &lt;div class="ToolbarItems Bold_tool"&gt;&lt;/div&gt;
		*    &lt;/li&gt;
		*    &lt;li id="Italic" title="Italic"&gt;
		*        &lt;div class="ToolbarItems Italic_tool"&gt;&lt;/div&gt;
		*    &lt;/li&gt;
		*    &lt;li id="StrikeThrough" title="Strike Through"&gt;
		*        &lt;div class="ToolbarItems StrikeThrough_tool"&gt;&lt;/div&gt;
		*    &lt;/li&gt;
		*    &lt;li id="UndeLine" title="UnderLine"&gt;
		*        &lt;div class="ToolbarItems Underline_tool"&gt;&lt;/div&gt;
		*    &lt;/li&gt;
		*&lt;/ul&gt;
		*&lt;/div&gt;
		* &lt;script&gt;
		*$("#toolbar1").ejToolbar();
		*$("#toolbar1").ejToolbar("enable"); //enable all item in the Toolbar
		* &lt;/script&gt;
		* @memberof ejToolbar
		* @instance
         */
        enable: function () {
            /// <summary>This will enable all toolbar items </summary>
            this.items.removeClass("e-disable");
            $(this.element).attr("aria-disabled", false);
            if(this.model.allowResponsive && (this.model.orientation == "horizontal"))
                this._spantag.removeClass("e-disable");
            this.model.enabled = true;
        },
        //Selects the toolbarItem of given object
		/**
        * To Select the Toolbar item		
		* @return jQuery
		* @example 
		* &lt;div id="toolbar1"&gt;
		*&lt;ul&gt;
		*    &lt;li id="Left" title="Left"&gt;
		*        &lt;div class="ToolbarItems LeftAlign_tool"&gt;&lt;/div&gt;
		*   &lt;/li&gt;
		*    &lt;li id="Center" title="Center"&gt;
		*        &lt;div class="ToolbarItems CenterAlign_tool"&gt;&lt;/div&gt;
		*    &lt;/li&gt;
		*    &lt;li id="Right" title="Right"&gt;
		*        &lt;div class="ToolbarItems RightAlign_tool"&gt;&lt;/div&gt;
		*    &lt;/li&gt;
		*    &lt;li id="Justify" title="Justify"&gt;
		*        &lt;div class="ToolbarItems Justify_tool"&gt;&lt;/div&gt;
		*    &lt;/li&gt;
		* 	&lt;/ul&gt;
		*	&lt;ul&gt;
		*    &lt;li id="Bold" title="Bold"&gt;
		*        &lt;div class="ToolbarItems Bold_tool"&gt;&lt;/div&gt;
		*    &lt;/li&gt;
		*    &lt;li id="Italic" title="Italic"&gt;
		*        &lt;div class="ToolbarItems Italic_tool"&gt;&lt;/div&gt;
		*    &lt;/li&gt;
		*    &lt;li id="StrikeThrough" title="Strike Through"&gt;
		*        &lt;div class="ToolbarItems StrikeThrough_tool"&gt;&lt;/div&gt;
		*    &lt;/li&gt;
		*    &lt;li id="UndeLine" title="UnderLine"&gt;
		*        &lt;div class="ToolbarItems Underline_tool"&gt;&lt;/div&gt;
		*    &lt;/li&gt;
		*&lt;/ul&gt;
		*&lt;/div&gt;
		* &lt;script&gt;
		*$("#toolbar1").ejToolbar();
		*$("#toolbar1").ejToolbar("selectItem",$("li")[3]);//Select the Toolbar item.
		* &lt;/script&gt;
		* @memberof ejToolbar
		* @instance
         */
        selectItem: function (lielement) {
            /// <summary>This will select a toolbar item </summary>
            var current = $(lielement);
            if ((current == null) || (current.length <= 0)) {
                return;
            }
            if (!current.hasClass("e-disable")) {
                current.removeClass("e-active");
                current.addClass("e-active");
            }
        },
        //Deselects the toolbarItem of given object
			/**
        * To Deselect the Toolbar item		
		* @return jQuery
		* @example 
		* &lt;div id="toolbar1"&gt;
		*&lt;ul&gt;
		*    &lt;li id="Left" title="Left"&gt;
		*        &lt;div class="ToolbarItems LeftAlign_tool"&gt;&lt;/div&gt;
		*   &lt;/li&gt;
		*    &lt;li id="Center" title="Center"&gt;
		*        &lt;div class="ToolbarItems CenterAlign_tool"&gt;&lt;/div&gt;
		*    &lt;/li&gt;
		*    &lt;li id="Right" title="Right"&gt;
		*        &lt;div class="ToolbarItems RightAlign_tool"&gt;&lt;/div&gt;
		*    &lt;/li&gt;
		*    &lt;li id="Justify" title="Justify"&gt;
		*        &lt;div class="ToolbarItems Justify_tool"&gt;&lt;/div&gt;
		*    &lt;/li&gt;
		* 	&lt;/ul&gt;
		*	&lt;ul&gt;
		*    &lt;li id="Bold" title="Bold"&gt;
		*        &lt;div class="ToolbarItems Bold_tool"&gt;&lt;/div&gt;
		*    &lt;/li&gt;
		*    &lt;li id="Italic" title="Italic"&gt;
		*        &lt;div class="ToolbarItems Italic_tool"&gt;&lt;/div&gt;
		*    &lt;/li&gt;
		*    &lt;li id="StrikeThrough" title="Strike Through"&gt;
		*        &lt;div class="ToolbarItems StrikeThrough_tool"&gt;&lt;/div&gt;
		*    &lt;/li&gt;
		*    &lt;li id="UndeLine" title="UnderLine"&gt;
		*        &lt;div class="ToolbarItems Underline_tool"&gt;&lt;/div&gt;
		*    &lt;/li&gt;
		*&lt;/ul&gt;
		*&lt;/div&gt;
		* &lt;script&gt;
		*$("#toolbar1").ejToolbar();
        *$("#toolbar1").ejToolbar("selectItem",$("li")[3]);//Select the Toolbar item.
		*$("#toolbar1").ejToolbar("deselectItem",$("li")[3]); //Deselect the Toolbar item.
		* &lt;/script&gt;

		* @memberof ejToolbar
		* @instance
         */
        deselectItem: function (lielement) {
            /// <summary>This will deselect toolbar item </summary>
            var current = $(lielement);
            if ((current == null) || (current.length <= 0)) {
                return;
            }
            if (!current.hasClass("e-disable")) {
                current.removeClass("e-active");
            }
        },

        //Selects the toolbarItem of given ID
		/**
        * To Select the Toolbar item by id	
		* @return jQuery
		* @example 
		* &lt;div id="toolbar1"&gt;
		*&lt;ul&gt;
		*    &lt;li id="Left" title="Left"&gt;
		*        &lt;div class="ToolbarItems LeftAlign_tool"&gt;&lt;/div&gt;
		*   &lt;/li&gt;
		*    &lt;li id="Center" title="Center"&gt;
		*        &lt;div class="ToolbarItems CenterAlign_tool"&gt;&lt;/div&gt;
		*    &lt;/li&gt;
		*    &lt;li id="Right" title="Right"&gt;
		*        &lt;div class="ToolbarItems RightAlign_tool"&gt;&lt;/div&gt;
		*    &lt;/li&gt;
		*    &lt;li id="Justify" title="Justify"&gt;
		*        &lt;div class="ToolbarItems Justify_tool"&gt;&lt;/div&gt;
		*    &lt;/li&gt;
		* 	&lt;/ul&gt;
		*	&lt;ul&gt;
		*    &lt;li id="Bold" title="Bold"&gt;
		*        &lt;div class="ToolbarItems Bold_tool"&gt;&lt;/div&gt;
		*    &lt;/li&gt;
		*    &lt;li id="Italic" title="Italic"&gt;
		*        &lt;div class="ToolbarItems Italic_tool"&gt;&lt;/div&gt;
		*    &lt;/li&gt;
		*    &lt;li id="StrikeThrough" title="Strike Through"&gt;
		*        &lt;div class="ToolbarItems StrikeThrough_tool"&gt;&lt;/div&gt;
		*    &lt;/li&gt;
		*    &lt;li id="UndeLine" title="UnderLine"&gt;
		*        &lt;div class="ToolbarItems Underline_tool"&gt;&lt;/div&gt;
		*    &lt;/li&gt;
		*&lt;/ul&gt;
		*&lt;/div&gt;
		* &lt;script&gt;
		*$("#toolbar1").ejToolbar();
		*$("#toolbar1").ejToolbar("selectItemByID","left");//Select the Toolbar item by id.
		* &lt;/script&gt;
		* @memberof ejToolbar
		* @instance
         */
        selectItemByID: function (liid) {
            /// <summary>This will select an toolbar item by id </summary>
            var lielement;
            lielement = this.itemsContainer.find("li#" + liid);
            if ((lielement == null) || (lielement.length <= 0)) {
                return;
            }
            this.selectItem(lielement);
        },
        //Deselects the toolbarItem of given ID
		/**
        * To Deselect the Toolbar item by id		
		* @return jQuery
		* @example 
		* &lt;div id="toolbar1"&gt;
		*&lt;ul&gt;
		*    &lt;li id="Left" title="Left"&gt;
		*        &lt;div class="ToolbarItems LeftAlign_tool"&gt;&lt;/div&gt;
		*   &lt;/li&gt;
		*    &lt;li id="Center" title="Center"&gt;
		*        &lt;div class="ToolbarItems CenterAlign_tool"&gt;&lt;/div&gt;
		*    &lt;/li&gt;
		*    &lt;li id="Right" title="Right"&gt;
		*        &lt;div class="ToolbarItems RightAlign_tool"&gt;&lt;/div&gt;
		*    &lt;/li&gt;
		*    &lt;li id="Justify" title="Justify"&gt;
		*        &lt;div class="ToolbarItems Justify_tool"&gt;&lt;/div&gt;
		*    &lt;/li&gt;
		* 	&lt;/ul&gt;
		*	&lt;ul&gt;
		*    &lt;li id="Bold" title="Bold"&gt;
		*        &lt;div class="ToolbarItems Bold_tool"&gt;&lt;/div&gt;
		*    &lt;/li&gt;
		*    &lt;li id="Italic" title="Italic"&gt;
		*        &lt;div class="ToolbarItems Italic_tool"&gt;&lt;/div&gt;
		*    &lt;/li&gt;
		*    &lt;li id="StrikeThrough" title="Strike Through"&gt;
		*        &lt;div class="ToolbarItems StrikeThrough_tool"&gt;&lt;/div&gt;
		*    &lt;/li&gt;
		*    &lt;li id="UndeLine" title="UnderLine"&gt;
		*        &lt;div class="ToolbarItems Underline_tool"&gt;&lt;/div&gt;
		*    &lt;/li&gt;
		*&lt;/ul&gt;
		*&lt;/div&gt;
		* &lt;script&gt;
		*$("#toolbar1").ejToolbar();
        *$("#toolbar1").ejToolbar("selectItemByID","left");//Select the Toolbar item by id.
		*$("#toolbar1").ejToolbar("deselectItemByID","left"); // To Deselect the Toolbar item by id.
		* &lt;/script&gt;
		* @memberof ejToolbar
		* @instance
         */
        deselectItemByID: function (liid) {
            /// <summary>This will deselect an toolbar item by id. </summary>
            var lielement;
            lielement = this.itemsContainer.find("li#" + liid);
            if ((lielement == null) || (lielement.length <= 0)) {
                return;
            }
            this.deselectItem(lielement);
        },
        //Remove the toolbarItem of given object
		/**
        * To Remove the Toolbar item	
		* @return jQuery
		* @example 
		* &lt;div id="toolbar1"&gt;
		*&lt;ul&gt;
		*    &lt;li id="Left" title="Left"&gt;
		*        &lt;div class="ToolbarItems LeftAlign_tool"&gt;&lt;/div&gt;
		*   &lt;/li&gt;
		*    &lt;li id="Center" title="Center"&gt;
		*        &lt;div class="ToolbarItems CenterAlign_tool"&gt;&lt;/div&gt;
		*    &lt;/li&gt;
		*    &lt;li id="Right" title="Right"&gt;
		*        &lt;div class="ToolbarItems RightAlign_tool"&gt;&lt;/div&gt;
		*    &lt;/li&gt;
		*    &lt;li id="Justify" title="Justify"&gt;
		*        &lt;div class="ToolbarItems Justify_tool"&gt;&lt;/div&gt;
		*    &lt;/li&gt;
		* 	&lt;/ul&gt;
		*	&lt;ul&gt;
		*    &lt;li id="Bold" title="Bold"&gt;
		*        &lt;div class="ToolbarItems Bold_tool"&gt;&lt;/div&gt;
		*    &lt;/li&gt;
		*    &lt;li id="Italic" title="Italic"&gt;
		*        &lt;div class="ToolbarItems Italic_tool"&gt;&lt;/div&gt;
		*    &lt;/li&gt;
		*    &lt;li id="StrikeThrough" title="Strike Through"&gt;
		*        &lt;div class="ToolbarItems StrikeThrough_tool"&gt;&lt;/div&gt;
		*    &lt;/li&gt;
		*    &lt;li id="UndeLine" title="UnderLine"&gt;
		*        &lt;div class="ToolbarItems Underline_tool"&gt;&lt;/div&gt;
		*    &lt;/li&gt;
		*&lt;/ul&gt;
		*&lt;/div&gt;
		* &lt;script&gt;
		*$("#toolbar1").ejToolbar();
		*$("#toolbar1").ejToolbar("removeItem",$("li")[3]); // Remove the Toolbar item
		* &lt;/script&gt;
		* @memberof ejToolbar
		* @instance
         */
        removeItem: function (lielement) {
            /// <summary>This will remove the toolbar item </summary>
            if (!this.model.enabled) return false;
            var current = $(lielement);
            if ((current == null) || (current.length <= 0)) {
                return;
            }
            current.remove();
        },
				/**
        * To Remove the Toolbar item by id	
		* @return jQuery
		* @example 
		* &lt;div id="toolbar1"&gt;
		*&lt;ul&gt;
		*    &lt;li id="Left" title="Left"&gt;
		*        &lt;div class="ToolbarItems LeftAlign_tool"&gt;&lt;/div&gt;
		*   &lt;/li&gt;
		*    &lt;li id="Center" title="Center"&gt;
		*        &lt;div class="ToolbarItems CenterAlign_tool"&gt;&lt;/div&gt;
		*    &lt;/li&gt;
		*    &lt;li id="Right" title="Right"&gt;
		*        &lt;div class="ToolbarItems RightAlign_tool"&gt;&lt;/div&gt;
		*    &lt;/li&gt;
		*    &lt;li id="Justify" title="Justify"&gt;
		*        &lt;div class="ToolbarItems Justify_tool"&gt;&lt;/div&gt;
		*    &lt;/li&gt;
		* 	&lt;/ul&gt;
		*	&lt;ul&gt;
		*    &lt;li id="Bold" title="Bold"&gt;
		*        &lt;div class="ToolbarItems Bold_tool"&gt;&lt;/div&gt;
		*    &lt;/li&gt;
		*    &lt;li id="Italic" title="Italic"&gt;
		*        &lt;div class="ToolbarItems Italic_tool"&gt;&lt;/div&gt;
		*    &lt;/li&gt;
		*    &lt;li id="StrikeThrough" title="Strike Through"&gt;
		*        &lt;div class="ToolbarItems StrikeThrough_tool"&gt;&lt;/div&gt;
		*    &lt;/li&gt;
		*    &lt;li id="UndeLine" title="UnderLine"&gt;
		*        &lt;div class="ToolbarItems Underline_tool"&gt;&lt;/div&gt;
		*    &lt;/li&gt;
		*&lt;/ul&gt;
		*&lt;/div&gt;
		* &lt;script&gt;
		*$("#toolbar1").ejToolbar();
		*$("#toolbar1").ejToolbar("removeItemByID","left"); // Remove the Toolbar item by id
		* &lt;/script&gt;
		* @memberof ejToolbar
		* @instance
         */
        //Remove the toolbarItem of given ID
        removeItemByID: function (liid) {
            /// <summary>This will remove the toolbar item by ID </summary>
            var lielement;
            lielement = this.itemsContainer.find("li#" + liid);
            if ((lielement == null) || (lielement.length <= 0)) {
                return;
            }
            this.removeItem(lielement);
        },
        _wireResizing: function () {
            (this.model.allowResponsive && (this.model.orientation == "horizontal")) ? $(window).bind('resize', $.proxy(this._reSizeHandler, this)) : $(window).unbind('resize', $.proxy(this._reSizeHandler, this))
        },
        /*-----------------------Event Handlers -----------------------------------------*/
        /*
		 * Wiring the events to ToolBar control		
		 * @private
         */		
        _wireEvents: function () {
            /// <summary>This will wire events for toolbar item </summary>
            var args;
            this._on(this.element, "focus", this._focusElement);
            this._on(this.element, "blur", this._targetBlur);
            this._on(this.items, "mouseenter", this._onItemHover);
            this._on(this.items, "mousedown", this._onItemClick);
            this._on(this.items, "mouseup", this._onItemClick);
            this._on(this.items, "mouseleave", this._onItemLeave);
          //this._on(this.items, "keyup", this._OnKeyUp);
          //this._on(this.items, "keydown", this._OnKeyDown);
            this._on($("#" + this.element[0].id + "_target"), "click", this._btnMouseClick);
            this._wireResizing();
        },
		 /**
         * Section For handle the Item hover event
		 * @private
         */	
        _onItemHover: function (e) {
            var currentItem = e.currentTarget, targetItem = e.target;
            if (!$(currentItem).hasClass("e-disable")) {
                this.items.removeClass("e-hover");
                $(currentItem).addClass("e-hover");
                args = { currentTarget: currentItem, target: targetItem, status: this.model.enabled };
                this._trigger("itemHover", args);
            }
        },
		 /**
         * Section For handle the Item click event
		 * @private
         */	
        _onItemClick: function (e) {
            var currentItem = e.currentTarget, targetItem = e.target;
            if (e.type == "mousedown") {
                this._focusEnable = false; 
                $(e.currentTarget).addClass('e-active');
            }
            else if (e.type == "mouseup") {
                $(e.currentTarget).removeClass('e-active');
                if (!$(currentItem).hasClass("e-disable")) {
                    args = { currentTarget: currentItem, target: targetItem, status: this.model.enabled };
                    this._activeItem = $(e.currentTarget).index();
                    this._trigger("click", args);
                }
            }
        },
		 /**
         * Section For handle the Mouse Leave on item event
		 * @private
         */	
        _onItemLeave: function (e) {
            var currentItem = e.currentTarget, targetItem = e.target;
            if (!$(currentItem).hasClass("e-disable")) {
                $(currentItem).removeClass("e-hover");
                args = { currentTarget: currentItem, target: targetItem, status: this.model.enabled };
                this._trigger("itemLeave", args);
            }
        },
		 /**
         * Section to watch the  key down event
		 * @private
         */	
        _OnKeyDown: function (e) {
            this._focusEnable = true;
            this.e = e;
            var itemsLength = this.items.length - 1, activeItem;
            switch (e.keyCode) {
                case 38: //Up
                    e.preventDefault();
                    this._removeListHover();
                    if ((this._activeItem < 0) || (this._activeItem == null) || (this._activeItem > itemsLength))
                        this._activeItem = 0;
                    else if (this._activeItem == 0)
                        this._activeItem = itemsLength;
                    else
                        this._activeItem -= 1;
                    this._beforeListHover();
                    break;
                case 40: //Down
                    e.preventDefault();
                    this._removeListHover();
                    if ((this._activeItem > itemsLength) || (this._activeItem == null) || (this._activeItem < 0))
                        this._activeItem = itemsLength;
                    else if (this._activeItem == itemsLength)
                        this._activeItem = 0;
                    else
                        this._activeItem += 1;
                    this._beforeListHover();
                    break;
                case 37: //Left
                    e.preventDefault();
                    this._removeListHover();
                    if ((this._activeItem < 0) || (this._activeItem == null) || (this._activeItem > itemsLength))
                        this._activeItem = 0;
                    else if (this._activeItem == 0)
                        this._activeItem = itemsLength;
                    else
                        this._activeItem -= 1;
                    this._beforeListHover();
                    break;
                case 39: //Right
                    e.preventDefault();
                    this._removeListHover();
                    if ((this._activeItem > itemsLength) || (this._activeItem == null) || (this._activeItem < 0))
                        this._activeItem = itemsLength;
                    else if (this._activeItem == itemsLength)
                        this._activeItem = 0;
                    else
                        this._activeItem += 1;
                    this._beforeListHover();
                    break;
                case 13: //enter key
                    break;
                case 32: //space key
                    break;
                case 33: //pup
                    e.preventDefault();
                    this._removeListHover();
                    this._activeItem = 0;
                    this._beforeListHover();
                    break;
                case 34: //pdwn
                    e.preventDefault();
                    this._removeListHover();
                    this._activeItem = itemsLength;
                    this._beforeListHover();
                    break;
                case 35: //endkey
                    e.preventDefault();
                    this._removeListHover();
                    this._activeItem = itemsLength;
                    this._beforeListHover();
                    break;
                case 36: //Home key
                    e.preventDefault();
                    this._removeListHover();
                    this._activeItem = 0;
                    this._beforeListHover();
                    break;
            }
        },
		 /**
         * Section to watch the  KeyUp event
		 * @private
         */	
        _OnKeyUp: function (e) {
            this.e = e;
            var itemsLength = this.items.length, activeItem;
            switch (e.keyCode) {
                case 38: //Up
                    break;
                case 40: //Down
                    break;
                case 37: //Left
                    break;
                case 39: //Right
                    break;
                case 13: //enter key
                    activeItem = this._getActiveItem();
                    if ($(e.target).is("div")) {
                        var currentItem = this._getTargetItem(), targetItem = this._getTargetItem();
                    } else {
                        var currentItem = $(e.target).closest("li")[0], targetItem = e.target;
                    }
                    if (!$(currentItem).hasClass("e-disable")) {
                        args = { currentTarget: currentItem, target: targetItem, status: this.model.enabled, event: this.e };
                        this._trigger("click", args);
                        this._removeListHover();
                    }
                    break;
                case 32: //space key
                    activeItem = this._getActiveItem();
                    if ($(e.target).is("div")) {
                        var currentItem = this._getTargetItem(), targetItem = this._getTargetItem();
                    } else {
                        var currentItem = $(e.target).closest("li")[0], targetItem = e.target;
                    }
                    if (!activeItem.hasClass("e-disable")) {
                        args = { currentTarget: currentItem, target: targetItem, status: this.model.enabled, event: this.e };
                        this._trigger("click", args);
                        this._removeListHover();
                    }
                    break;
                case 33: //pup
                    break;
                case 34: //pdwn
                    break;
                case 35: //endkey
                    break;
                case 36: //Home key
                    break;
                case 27:         // Esc Key
                    this.element.focusout();
                    this._removeListHover();
                    break;
                case 9:
                    this._activeItem = 0;
                    this._beforeListHover();
                    break;
            }
        },
		 /**
         * Section For handle to remove class after hover event
		 * @private
         */	
        _removeListHover: function () {
            $(this.items).removeClass("e-hover");
        },
        /**
         * Section For handle hover event before add hover event class
         * @private
         */
        _beforeListHover: function(){
            activeItem = this._getActiveItem();
            if (!activeItem.hasClass("e-disable")) {
                this._addListHover();
            }
        },
		/**
         * Section For handle to add class for hover event
		 * @private
         */	
        _addListHover: function () {
            var activeItem = this._getActiveItem();
            activeItem.addClass("e-hover");
            //activeItem[0].scrollIntoView(false);
            activeItem.focus();
        },
		/**
         *  To get the ActiveItem		
		 * @private
         */	 
        _getActiveItem: function () {
            return $($(this.items)[this._activeItem]);
        },
		/**
         * To get the TargetItem		
		 * @private
         */	 
        _getTargetItem: function () {
            return $(this.items)[this._activeItem];
        },
		/**
         * Section For handle Blur event
		 * @private
         */	
        _targetBlur: function (e) {
            e.preventDefault();
            this.element.focusout();
            this._removeListHover();
            this.element.removeClass("e-focus");
            this._off(this.element, "keyup", this._OnKeyUp);
            this._off(this.element, "keydown", this._OnKeyDown);
            // this._isFocused = false;
            /* Raise focusOut Event*/
           
                this._trigger("focusOut");
        },
		/**
         * Section For handle Focus event
		 * @private
         */	
        _focusElement: function (e) {
            this.element.addClass("e-focus");
            this._on(this.element, "keyup", this._OnKeyUp);
            this._on(this.element, "keydown", this._OnKeyDown);

            if (!this._focusEnable) {
                this._focusEnable = true;
                return;
            }
            this._removeListHover();
            this._activeItem = 0;
            this._beforeListHover();
        },
    });

})(jQuery, Syncfusion);


;
/**
* @fileOverview Plugin to style the Html Accordion elements
* @copyright Copyright Syncfusion Inc. 2001 - 2013. All rights reserved.
*  Use of this code is subject to the terms of our license.
*  A copy of the current license can be obtained at any time by e-mailing
*  licensing@syncfusion.com. Any infringement will be prosecuted under
*  applicable laws. 
* @version 12.1 
* @author <a href="mailto:licensing@syncfusion.com">Syncfusion Inc</a>
*/

(function ($, ej, undefined) {
	 /**
    * @namespace ej
	* @class ejAccordion
	* @requires jQuery
    * @requires jquery.easing.1.3.min.js
	* @requires ej.core.js
	* @requires ej.accordion.js
	* @classdesc Custom Design for Accordion control.
	* @example 
	* &lt;div id="accordion"&gt;
	*        &lt;h3&gt;
	*            &lt;a href="#"&gt;Orubase&lt;/a&gt;&lt;/h3&gt;
	*        &lt;div&gt;
	*Orubase is the only mobile application development framework built especially for developing complex line-of-business mobile applications targeting iOS, Android, and Windows Phone platforms in the shortest possible timeframe.
	*        &lt;/div&gt;
	*        &lt;h3&gt;
	*            &lt;a href="#"&gt;WinRT XAML&lt;/a&gt;&lt;/h3&gt;
	*        &lt;div&gt;
	*Essential Studio for WinRT contains all the controls you need to build line-of-business tablet applications including grid, chart, map, tree map, SSRS report viewer, rich-text editor, PDF viewer, gauges, barcode, editors, and much more. It also includes a unique set of controls for reading and writing Excel, Word, and PDF documents in Windows store apps.
	*        &lt;/div&gt;
	*        &lt;h3&gt;
	*            &lt;a href="#"&gt;Metro Studio&lt;/a&gt;&lt;/h3&gt;
	*        &lt;div&gt;
	*Syncfusion Metro Studio is a collection of over 2500 Metro-style icon templates that can be easily customized to create thousands of unique Metro icons.
	*        &lt;/div&gt;
	*    &lt;/div&gt;
	*  &lt;script type="text/javascript"&gt;
	*        $(function () {
	*            // Initialize Accordion control creation.
	*            $("#accordion").ejAccordion();
	*        });
	*    &lt;/script&gt; 
	*/
	
    ej.widget("ejAccordion", "ej.Accordion", {
        _rootCSS: "e-acrdn",
        // widget element will be automatically set in this
        element: null,

        // user defined model will be automatically set in this
        model: null,
        validTags: ["div", "span"],
        _setFirst: false,

        // default model
        defaults: {
            /// <summary>This Contains default property of Sycnfusion Accordion </summary>
			/**		
			* Specifies the collapsible state of accordion control.
			* @default false
			* @type {boolean}
			* @example 
			* &lt;div id="accordion"&gt;
	*        &lt;h3&gt;
	*            &lt;a href="#"&gt;Orubase&lt;/a&gt;&lt;/h3&gt;
	*        &lt;div&gt;
	*Orubase is the only mobile application development framework built especially for developing complex line-of-business mobile applications targeting iOS, Android, and Windows Phone platforms in the shortest possible timeframe.
	*        &lt;/div&gt;
	*        &lt;h3&gt;
	*            &lt;a href="#"&gt;WinRT XAML&lt;/a&gt;&lt;/h3&gt;
	*        &lt;div&gt;
	*Essential Studio for WinRT contains all the controls you need to build line-of-business tablet applications including grid, chart, map, tree map, SSRS report viewer, rich-text editor, PDF viewer, gauges, barcode, editors, and much more. It also includes a unique set of controls for reading and writing Excel, Word, and PDF documents in Windows store apps.
	*        &lt;/div&gt;
	*        &lt;h3&gt;
	*            &lt;a href="#"&gt;Metro Studio&lt;/a&gt;&lt;/h3&gt;
	*        &lt;div&gt;
	*Syncfusion Metro Studio is a collection of over 2500 Metro-style icon templates that can be easily customized to create thousands of unique Metro icons.
	*        &lt;/div&gt;
	*    &lt;/div&gt;
	*  &lt;script type="text/javascript"&gt;
			* // Allow to collapsible on initialization. 
			* 	//To set collapsible API value 
			* 	 $("#accordion").ejAccordion({ collapsible: true});
			* &lt;/script&gt;
			 * @memberof ejAccordion
			* @instance
			*/
            collapsible: false,
			
            ajaxSettings: { type: 'GET', cache: false, data: {}, dataType: "html", contentType: "html", async: true },
			/**		
			*  The events API binds the action for activating the accordion header. Users can activate the header by using mouse actions such as mouse-over, mouse-up, mouse-down, and so on.
			* @default "click"
			* @type {String}
			* @example			 
			* &lt;div id="accordion"&gt;
	*        &lt;h3&gt;
	*            &lt;a href="#"&gt;Orubase&lt;/a&gt;&lt;/h3&gt;
	*        &lt;div&gt;
	*Orubase is the only mobile application development framework built especially for developing complex line-of-business mobile applications targeting iOS, Android, and Windows Phone platforms in the shortest possible timeframe.
	*        &lt;/div&gt;
	*        &lt;h3&gt;
	*            &lt;a href="#"&gt;WinRT XAML&lt;/a&gt;&lt;/h3&gt;
	*        &lt;div&gt;
	*Essential Studio for WinRT contains all the controls you need to build line-of-business tablet applications including grid, chart, map, tree map, SSRS report viewer, rich-text editor, PDF viewer, gauges, barcode, editors, and much more. It also includes a unique set of controls for reading and writing Excel, Word, and PDF documents in Windows store apps.
	*        &lt;/div&gt;
	*        &lt;h3&gt;
	*            &lt;a href="#"&gt;Metro Studio&lt;/a&gt;&lt;/h3&gt;
	*        &lt;div&gt;
	*Syncfusion Metro Studio is a collection of over 2500 Metro-style icon templates that can be easily customized to create thousands of unique Metro icons.
	*        &lt;/div&gt;
	*    &lt;/div&gt;
	*  &lt;script type="text/javascript"&gt;
			* // Bind events API on initialization. 
			* $("#accordion").ejAccordion({ events: "mouseover" });
			* &lt;/script&gt;
			* @memberof ejAccordion
			* @instance
			*/
            events: "click",
			/**		
			
			* The header icon can be customized using this API. It has two properties: header as the collapsing header CSS class name, and selectedHeader as the active header CSS class name. It accepts CSS class names with string as the type.		*
			*•header: This class name set to collapsing header. This accepts the string type value.
			*•selectedHeader: This class name set to expanded header. This accepts the string type value.
			* @default "{  header: "e-acrdn-collapsicon",  selectedHeader: "e-acrdn-expandicon"   }"
			* @type {JSONObject}
			* @example	
			* &lt;div id="accordion"&gt;
	*        &lt;h3&gt;
	*            &lt;a href="#"&gt;Orubase&lt;/a&gt;&lt;/h3&gt;
	*        &lt;div&gt;
	*Orubase is the only mobile application development framework built especially for developing complex line-of-business mobile applications targeting iOS, Android, and Windows Phone platforms in the shortest possible timeframe.
	*        &lt;/div&gt;
	*        &lt;h3&gt;
	*            &lt;a href="#"&gt;WinRT XAML&lt;/a&gt;&lt;/h3&gt;
	*        &lt;div&gt;
	*Essential Studio for WinRT contains all the controls you need to build line-of-business tablet applications including grid, chart, map, tree map, SSRS report viewer, rich-text editor, PDF viewer, gauges, barcode, editors, and much more. It also includes a unique set of controls for reading and writing Excel, Word, and PDF documents in Windows store apps.
	*        &lt;/div&gt;
	*        &lt;h3&gt;
	*            &lt;a href="#"&gt;Metro Studio&lt;/a&gt;&lt;/h3&gt;
	*        &lt;div&gt;
	*Syncfusion Metro Studio is a collection of over 2500 Metro-style icon templates that can be easily customized to create thousands of unique Metro icons.
	*        &lt;/div&gt;
	*    &lt;/div&gt;
	*  &lt;script type="text/javascript"&gt;		 
			* //customIcon on initialization. 
			*   $("#accordion").ejAccordion({
			*		customIcon: {
			*			header: "header-close",
			*			selectedHeader: "header-expand"
			*		}
			*	});
			* &lt;/script&gt;
			* @memberof ejAccordion
			* @instance
			*/
            customIcon: /** @lends ejAccordion# */{
			    /**		
                 * Set class name to collapsing header.
				 * @alias ejAccordion#header
				 * @type String
				 * @example	
			* &lt;div id="accordion"&gt;
	*        &lt;h3&gt;
	*            &lt;a href="#"&gt;Orubase&lt;/a&gt;&lt;/h3&gt;
	*        &lt;div&gt;
	*Orubase is the only mobile application development framework built especially for developing complex line-of-business mobile applications targeting iOS, Android, and Windows Phone platforms in the shortest possible timeframe.
	*        &lt;/div&gt;
	*        &lt;h3&gt;
	*            &lt;a href="#"&gt;WinRT XAML&lt;/a&gt;&lt;/h3&gt;
	*        &lt;div&gt;
	*Essential Studio for WinRT contains all the controls you need to build line-of-business tablet applications including grid, chart, map, tree map, SSRS report viewer, rich-text editor, PDF viewer, gauges, barcode, editors, and much more. It also includes a unique set of controls for reading and writing Excel, Word, and PDF documents in Windows store apps.
	*        &lt;/div&gt;
	*        &lt;h3&gt;
	*            &lt;a href="#"&gt;Metro Studio&lt;/a&gt;&lt;/h3&gt;
	*        &lt;div&gt;
	*Syncfusion Metro Studio is a collection of over 2500 Metro-style icon templates that can be easily customized to create thousands of unique Metro icons.
	*        &lt;/div&gt;
	*    &lt;/div&gt;
	*  &lt;script type="text/javascript"&gt;		 
			* //header on initialization. 
			*   $("#accordion").ejAccordion({
			*			header: "collapsicon"
			*	});
			* &lt;/script&gt;
			* @memberof ejAccordion
			* @instance
                 */
                header: "e-collaps",
				 /**		
                 * Set class name to expanded header.
				 * @alias ejAccordion#selectedHeader
				 * @type String
				 * @example	
			* &lt;div id="accordion"&gt;
	*        &lt;h3&gt;
	*            &lt;a href="#"&gt;Orubase&lt;/a&gt;&lt;/h3&gt;
	*        &lt;div&gt;
	*Orubase is the only mobile application development framework built especially for developing complex line-of-business mobile applications targeting iOS, Android, and Windows Phone platforms in the shortest possible timeframe.
	*        &lt;/div&gt;
	*        &lt;h3&gt;
	*            &lt;a href="#"&gt;WinRT XAML&lt;/a&gt;&lt;/h3&gt;
	*        &lt;div&gt;
	*Essential Studio for WinRT contains all the controls you need to build line-of-business tablet applications including grid, chart, map, tree map, SSRS report viewer, rich-text editor, PDF viewer, gauges, barcode, editors, and much more. It also includes a unique set of controls for reading and writing Excel, Word, and PDF documents in Windows store apps.
	*        &lt;/div&gt;
	*        &lt;h3&gt;
	*            &lt;a href="#"&gt;Metro Studio&lt;/a&gt;&lt;/h3&gt;
	*        &lt;div&gt;
	*Syncfusion Metro Studio is a collection of over 2500 Metro-style icon templates that can be easily customized to create thousands of unique Metro icons.
	*        &lt;/div&gt;
	*    &lt;/div&gt;
	*  &lt;script type="text/javascript"&gt;		 
			* //Selected header on initialization. 
			*   $("#accordion").ejAccordion({
			*		
			*			selectedHeader: "activeicon"
			*		
			*	});
			* &lt;/script&gt;
			* @memberof ejAccordion
			* @instance
                 */
                selectedHeader: "e-expand"
            },
			/**		
			*  Adjusts the content panel height based on the given option (content, auto, or fill). By default, the panel heights are adjusted based on the content. See {@link HeightAdjustMode }
			* @default "content"
			* @type {Enum/String}
			*@example
			* &lt;div id="accordion"&gt;
	*        &lt;h3&gt;
	*            &lt;a href="#"&gt;Orubase&lt;/a&gt;&lt;/h3&gt;
	*        &lt;div&gt;
	*Orubase is the only mobile application development framework built especially for developing complex line-of-business mobile applications targeting iOS, Android, and Windows Phone platforms in the shortest possible timeframe.
	*        &lt;/div&gt;
	*        &lt;h3&gt;
	*            &lt;a href="#"&gt;WinRT XAML&lt;/a&gt;&lt;/h3&gt;
	*        &lt;div&gt;
	*Essential Studio for WinRT contains all the controls you need to build line-of-business tablet applications including grid, chart, map, tree map, SSRS report viewer, rich-text editor, PDF viewer, gauges, barcode, editors, and much more. It also includes a unique set of controls for reading and writing Excel, Word, and PDF documents in Windows store apps.
	*        &lt;/div&gt;
	*        &lt;h3&gt;
	*            &lt;a href="#"&gt;Metro Studio&lt;/a&gt;&lt;/h3&gt;
	*        &lt;div&gt;
	*Syncfusion Metro Studio is a collection of over 2500 Metro-style icon templates that can be easily customized to create thousands of unique Metro icons.
	*        &lt;/div&gt;
	*    &lt;/div&gt;
	*  &lt;script type="text/javascript"&gt;
			* //heightAdjustMode  for content div on initialization. 
			* $("#accordion").ejAccordion({ heightAdjustMode : ej.Accordion.HeightAdjustMode.Auto }); //enum 
				* &lt;/script&gt;
				*@example
			* &lt;div id="accordion"&gt;
	*        &lt;h3&gt;
	*            &lt;a href="#"&gt;Orubase&lt;/a&gt;&lt;/h3&gt;
	*        &lt;div&gt;
	*Orubase is the only mobile application development framework built especially for developing complex line-of-business mobile applications targeting iOS, Android, and Windows Phone platforms in the shortest possible timeframe.
	*        &lt;/div&gt;
	*        &lt;h3&gt;
	*            &lt;a href="#"&gt;WinRT XAML&lt;/a&gt;&lt;/h3&gt;
	*        &lt;div&gt;
	*Essential Studio for WinRT contains all the controls you need to build line-of-business tablet applications including grid, chart, map, tree map, SSRS report viewer, rich-text editor, PDF viewer, gauges, barcode, editors, and much more. It also includes a unique set of controls for reading and writing Excel, Word, and PDF documents in Windows store apps.
	*        &lt;/div&gt;
	*        &lt;h3&gt;
	*            &lt;a href="#"&gt;Metro Studio&lt;/a&gt;&lt;/h3&gt;
	*        &lt;div&gt;
	*Syncfusion Metro Studio is a collection of over 2500 Metro-style icon templates that can be easily customized to create thousands of unique Metro icons.
	*        &lt;/div&gt;
	*    &lt;/div&gt;
	*  &lt;script type="text/javascript"&gt;
			* //Pass value on string type. 
			*$("#accordion").ejAccordion({ heightAdjustMode : "auto" }); 
			* &lt;/script&gt;
			* @memberof ejAccordion
			* @instance
			*/
            heightAdjustMode : "content",
			enableAnimation: true,
			/**		
			* The given index header will activate (open). If collapsible is set to true, and a negative value is given, then all headers are collapsed. Otherwise, the first panel is activated.
			* @default 0
			* @type {Integer}
			*@example
			* &lt;div id="accordion"&gt;
	*        &lt;h3&gt;
	*            &lt;a href="#"&gt;Orubase&lt;/a&gt;&lt;/h3&gt;
	*        &lt;div&gt;
	*Orubase is the only mobile application development framework built especially for developing complex line-of-business mobile applications targeting iOS, Android, and Windows Phone platforms in the shortest possible timeframe.
	*        &lt;/div&gt;
	*        &lt;h3&gt;
	*            &lt;a href="#"&gt;WinRT XAML&lt;/a&gt;&lt;/h3&gt;
	*        &lt;div&gt;
	*Essential Studio for WinRT contains all the controls you need to build line-of-business tablet applications including grid, chart, map, tree map, SSRS report viewer, rich-text editor, PDF viewer, gauges, barcode, editors, and much more. It also includes a unique set of controls for reading and writing Excel, Word, and PDF documents in Windows store apps.
	*        &lt;/div&gt;
	*        &lt;h3&gt;
	*            &lt;a href="#"&gt;Metro Studio&lt;/a&gt;&lt;/h3&gt;
	*        &lt;div&gt;
	*Syncfusion Metro Studio is a collection of over 2500 Metro-style icon templates that can be easily customized to create thousands of unique Metro icons.
	*        &lt;/div&gt;
	*    &lt;/div&gt;
	*  &lt;script type="text/javascript"&gt;
			*  //Activate given header on initialization.
			*	$("#accordion").ejAccordion({ selectedItemIndex : 1 });
			* &lt;/script&gt;
			* @memberof ejAccordion
			* @instance
			*/
            selectedItemIndex: 0,
			/**		
			* Sets the root class for the Accordion theme. This cssClass API enables the use of custom skinning options for the Accordion control. By defining the root class using this API, we need to include this root class in the CSS.
			* @default ""
			* @type {String}
			*@example
			* &lt;div id="accordion"&gt;
	*        &lt;h3&gt;
	*            &lt;a href="#"&gt;Orubase&lt;/a&gt;&lt;/h3&gt;
	*        &lt;div&gt;
	*Orubase is the only mobile application development framework built especially for developing complex line-of-business mobile applications targeting iOS, Android, and Windows Phone platforms in the shortest possible timeframe.
	*        &lt;/div&gt;
	*        &lt;h3&gt;
	*            &lt;a href="#"&gt;WinRT XAML&lt;/a&gt;&lt;/h3&gt;
	*        &lt;div&gt;
	*Essential Studio for WinRT contains all the controls you need to build line-of-business tablet applications including grid, chart, map, tree map, SSRS report viewer, rich-text editor, PDF viewer, gauges, barcode, editors, and much more. It also includes a unique set of controls for reading and writing Excel, Word, and PDF documents in Windows store apps.
	*        &lt;/div&gt;
	*        &lt;h3&gt;
	*            &lt;a href="#"&gt;Metro Studio&lt;/a&gt;&lt;/h3&gt;
	*        &lt;div&gt;
	*Syncfusion Metro Studio is a collection of over 2500 Metro-style icon templates that can be easily customized to create thousands of unique Metro icons.
	*        &lt;/div&gt;
	*    &lt;/div&gt;
	*  &lt;script type="text/javascript"&gt;
			*  //cssClass on initialization.
			*	$("#accordion").ejAccordion({cssClass: "gradient-lime" });
			* &lt;/script&gt;
			* @memberof ejAccordion
			* @instance
			*/
            cssClass: "",
			/**		
			* Display headers and panel text from right-to-left.
			* @default false
			* @type {boolean}
			*@example
			* &lt;div id="accordion"&gt;
	*        &lt;h3&gt;
	*            &lt;a href="#"&gt;Orubase&lt;/a&gt;&lt;/h3&gt;
	*        &lt;div&gt;
	*Orubase is the only mobile application development framework built especially for developing complex line-of-business mobile applications targeting iOS, Android, and Windows Phone platforms in the shortest possible timeframe.
	*        &lt;/div&gt;
	*        &lt;h3&gt;
	*            &lt;a href="#"&gt;WinRT XAML&lt;/a&gt;&lt;/h3&gt;
	*        &lt;div&gt;
	*Essential Studio for WinRT contains all the controls you need to build line-of-business tablet applications including grid, chart, map, tree map, SSRS report viewer, rich-text editor, PDF viewer, gauges, barcode, editors, and much more. It also includes a unique set of controls for reading and writing Excel, Word, and PDF documents in Windows store apps.
	*        &lt;/div&gt;
	*        &lt;h3&gt;
	*            &lt;a href="#"&gt;Metro Studio&lt;/a&gt;&lt;/h3&gt;
	*        &lt;div&gt;
	*Syncfusion Metro Studio is a collection of over 2500 Metro-style icon templates that can be easily customized to create thousands of unique Metro icons.
	*        &lt;/div&gt;
	*    &lt;/div&gt;
	*  &lt;script type="text/javascript"&gt;
			*   // Enable enableRTL on initialization.
			*	$("#accordion").ejAccordion({ enableRTL: true});
			* &lt;/script&gt;
			* @memberof ejAccordion
			* @instance
			*/
            enableRTL: false,
			/**		
			* Displays rounded corner borders on the Accordion control's panels and headers.
			* @default false
			* @type {boolean}
			*@example
			* &lt;div id="accordion"&gt;
	*        &lt;h3&gt;
	*            &lt;a href="#"&gt;Orubase&lt;/a&gt;&lt;/h3&gt;
	*        &lt;div&gt;
	*Orubase is the only mobile application development framework built especially for developing complex line-of-business mobile applications targeting iOS, Android, and Windows Phone platforms in the shortest possible timeframe.
	*        &lt;/div&gt;
	*        &lt;h3&gt;
	*            &lt;a href="#"&gt;WinRT XAML&lt;/a&gt;&lt;/h3&gt;
	*        &lt;div&gt;
	*Essential Studio for WinRT contains all the controls you need to build line-of-business tablet applications including grid, chart, map, tree map, SSRS report viewer, rich-text editor, PDF viewer, gauges, barcode, editors, and much more. It also includes a unique set of controls for reading and writing Excel, Word, and PDF documents in Windows store apps.
	*        &lt;/div&gt;
	*        &lt;h3&gt;
	*            &lt;a href="#"&gt;Metro Studio&lt;/a&gt;&lt;/h3&gt;
	*        &lt;div&gt;
	*Syncfusion Metro Studio is a collection of over 2500 Metro-style icon templates that can be easily customized to create thousands of unique Metro icons.
	*        &lt;/div&gt;
	*    &lt;/div&gt;
	*  &lt;script type="text/javascript"&gt;
			*   // Enable showRoundedCorner on initialization.
			*	$("#accordion").ejAccordion({ showRoundedCorner: true});
			* &lt;/script&gt;
			* @memberof ejAccordion
			* @instance
			*/
            showRoundedCorner: false,
			/**		
			* Accordion headers can be expanded and collapsed on keyboard action. 
			* @default true
			* @type {boolean}
			*@example
			* &lt;div id="accordion"&gt;
	*        &lt;h3&gt;
	*            &lt;a href="#"&gt;Orubase&lt;/a&gt;&lt;/h3&gt;
	*        &lt;div&gt;
	*Orubase is the only mobile application development framework built especially for developing complex line-of-business mobile applications targeting iOS, Android, and Windows Phone platforms in the shortest possible timeframe.
	*        &lt;/div&gt;
	*        &lt;h3&gt;
	*            &lt;a href="#"&gt;WinRT XAML&lt;/a&gt;&lt;/h3&gt;
	*        &lt;div&gt;
	*Essential Studio for WinRT contains all the controls you need to build line-of-business tablet applications including grid, chart, map, tree map, SSRS report viewer, rich-text editor, PDF viewer, gauges, barcode, editors, and much more. It also includes a unique set of controls for reading and writing Excel, Word, and PDF documents in Windows store apps.
	*        &lt;/div&gt;
	*        &lt;h3&gt;
	*            &lt;a href="#"&gt;Metro Studio&lt;/a&gt;&lt;/h3&gt;
	*        &lt;div&gt;
	*Syncfusion Metro Studio is a collection of over 2500 Metro-style icon templates that can be easily customized to create thousands of unique Metro icons.
	*        &lt;/div&gt;
	*    &lt;/div&gt;
	*  &lt;script type="text/javascript"&gt;
			*   // Enable allowKeyboardNavigation on initialization.
			*	$("#accordion").ejAccordion({ allowKeyboardNavigation: true});
			* &lt;/script&gt;
			* @memberof ejAccordion
			* @instance
			*/
            allowKeyboardNavigation: true,
			/**		
			* Multiple content panels to activate at a time. 
			* @default false
			* @type {boolean}
			*@example
			* &lt;div id="accordion"&gt;
	*        &lt;h3&gt;
	*            &lt;a href="#"&gt;Orubase&lt;/a&gt;&lt;/h3&gt;
	*        &lt;div&gt;
	*Orubase is the only mobile application development framework built especially for developing complex line-of-business mobile applications targeting iOS, Android, and Windows Phone platforms in the shortest possible timeframe.
	*        &lt;/div&gt;
	*        &lt;h3&gt;
	*            &lt;a href="#"&gt;WinRT XAML&lt;/a&gt;&lt;/h3&gt;
	*        &lt;div&gt;
	*Essential Studio for WinRT contains all the controls you need to build line-of-business tablet applications including grid, chart, map, tree map, SSRS report viewer, rich-text editor, PDF viewer, gauges, barcode, editors, and much more. It also includes a unique set of controls for reading and writing Excel, Word, and PDF documents in Windows store apps.
	*        &lt;/div&gt;
	*        &lt;h3&gt;
	*            &lt;a href="#"&gt;Metro Studio&lt;/a&gt;&lt;/h3&gt;
	*        &lt;div&gt;
	*Syncfusion Metro Studio is a collection of over 2500 Metro-style icon templates that can be easily customized to create thousands of unique Metro icons.
	*        &lt;/div&gt;
	*    &lt;/div&gt;
	*  &lt;script type="text/javascript"&gt;
			*   // Enable enableMultipleOpen on initialization.
			*	$("#accordion").ejAccordion({ enableMultipleOpen: true});
			* &lt;/script&gt;
			* @memberof ejAccordion
			* @instance
			*/
            enableMultipleOpen: false,
            /**		
			* To set the Accordion headers Expand Speed. 
			* @default 300
			* @type {integer/string}
			*@example
			* &lt;div id="accordion"&gt;
	*        &lt;h3&gt;
	*            &lt;a href="#"&gt;Orubase&lt;/a&gt;&lt;/h3&gt;
	*        &lt;div&gt;
	*Orubase is the only mobile application development framework built especially for developing complex line-of-business mobile applications targeting iOS, Android, and Windows Phone platforms in the shortest possible timeframe.
	*        &lt;/div&gt;
	*        &lt;h3&gt;
	*            &lt;a href="#"&gt;WinRT XAML&lt;/a&gt;&lt;/h3&gt;
	*        &lt;div&gt;
	*Essential Studio for WinRT contains all the controls you need to build line-of-business tablet applications including grid, chart, map, tree map, SSRS report viewer, rich-text editor, PDF viewer, gauges, barcode, editors, and much more. It also includes a unique set of controls for reading and writing Excel, Word, and PDF documents in Windows store apps.
	*        &lt;/div&gt;
	*        &lt;h3&gt;
	*            &lt;a href="#"&gt;Metro Studio&lt;/a&gt;&lt;/h3&gt;
	*        &lt;div&gt;
	*Syncfusion Metro Studio is a collection of over 2500 Metro-style icon templates that can be easily customized to create thousands of unique Metro icons.
	*        &lt;/div&gt;
	*    &lt;/div&gt;
	*  &lt;script type="text/javascript"&gt;
			*   // Set expandSpeed on initialization.
			*	$("#accordion").ejAccordion({ expandSpeed: 500});
			* &lt;/script&gt;
			* @memberof ejAccordion
			* @instance
			*/
            expandSpeed: 300,
            /**		
			* To set the Accordion headers Collapse Speed. 
			* @default 300
			* @type {integer/string}
			*@example
			* &lt;div id="accordion"&gt;
	*        &lt;h3&gt;
	*            &lt;a href="#"&gt;Orubase&lt;/a&gt;&lt;/h3&gt;
	*        &lt;div&gt;
	*Orubase is the only mobile application development framework built especially for developing complex line-of-business mobile applications targeting iOS, Android, and Windows Phone platforms in the shortest possible timeframe.
	*        &lt;/div&gt;
	*        &lt;h3&gt;
	*            &lt;a href="#"&gt;WinRT XAML&lt;/a&gt;&lt;/h3&gt;
	*        &lt;div&gt;
	*Essential Studio for WinRT contains all the controls you need to build line-of-business tablet applications including grid, chart, map, tree map, SSRS report viewer, rich-text editor, PDF viewer, gauges, barcode, editors, and much more. It also includes a unique set of controls for reading and writing Excel, Word, and PDF documents in Windows store apps.
	*        &lt;/div&gt;
	*        &lt;h3&gt;
	*            &lt;a href="#"&gt;Metro Studio&lt;/a&gt;&lt;/h3&gt;
	*        &lt;div&gt;
	*Syncfusion Metro Studio is a collection of over 2500 Metro-style icon templates that can be easily customized to create thousands of unique Metro icons.
	*        &lt;/div&gt;
	*    &lt;/div&gt;
	*  &lt;script type="text/javascript"&gt;
			*   // Set collapseSpeed on initialization.
			*	$("#accordion").ejAccordion({ collapseSpeed: 500});
			* &lt;/script&gt;
			* @memberof ejAccordion
			* @instance
			*/
            collapseSpeed: 300,
			/**		
			* To activate multiple headers for the accordion, give the index value for array type for this API. The enableMultipleOpen property should be set as true. 
			* @default [0]
			* @type {Integerarray}
			*@example
			* &lt;div id="accordion"&gt;
	*        &lt;h3&gt;
	*            &lt;a href="#"&gt;Orubase&lt;/a&gt;&lt;/h3&gt;
	*        &lt;div&gt;
	*Orubase is the only mobile application development framework built especially for developing complex line-of-business mobile applications targeting iOS, Android, and Windows Phone platforms in the shortest possible timeframe.
	*        &lt;/div&gt;
	*        &lt;h3&gt;
	*            &lt;a href="#"&gt;WinRT XAML&lt;/a&gt;&lt;/h3&gt;
	*        &lt;div&gt;
	*Essential Studio for WinRT contains all the controls you need to build line-of-business tablet applications including grid, chart, map, tree map, SSRS report viewer, rich-text editor, PDF viewer, gauges, barcode, editors, and much more. It also includes a unique set of controls for reading and writing Excel, Word, and PDF documents in Windows store apps.
	*        &lt;/div&gt;
	*        &lt;h3&gt;
	*            &lt;a href="#"&gt;Metro Studio&lt;/a&gt;&lt;/h3&gt;
	*        &lt;div&gt;
	*Syncfusion Metro Studio is a collection of over 2500 Metro-style icon templates that can be easily customized to create thousands of unique Metro icons.
	*        &lt;/div&gt;
	*    &lt;/div&gt;
	*  &lt;script type="text/javascript"&gt;
			*   // selectedItems on initialization.
			*	$("#accordion").ejAccordion({ selectedItems: [0,1] });
			* &lt;/script&gt;
			* @memberof ejAccordion
			* @instance
			*/
            selectedItems: [0],
			/**		
			* To deactivate multiple headers for the accordion, give the index value for array type for this API. The enableMultipleOpen property should be set as true. 
			* @default []
			* @type {Integerarray}
			*@example
			* &lt;div id="accordion"&gt;
	*        &lt;h3&gt;
	*            &lt;a href="#"&gt;Orubase&lt;/a&gt;&lt;/h3&gt;
	*        &lt;div&gt;
	*Orubase is the only mobile application development framework built especially for developing complex line-of-business mobile applications targeting iOS, Android, and Windows Phone platforms in the shortest possible timeframe.
	*        &lt;/div&gt;
	*        &lt;h3&gt;
	*            &lt;a href="#"&gt;WinRT XAML&lt;/a&gt;&lt;/h3&gt;
	*        &lt;div&gt;
	*Essential Studio for WinRT contains all the controls you need to build line-of-business tablet applications including grid, chart, map, tree map, SSRS report viewer, rich-text editor, PDF viewer, gauges, barcode, editors, and much more. It also includes a unique set of controls for reading and writing Excel, Word, and PDF documents in Windows store apps.
	*        &lt;/div&gt;
	*        &lt;h3&gt;
	*            &lt;a href="#"&gt;Metro Studio&lt;/a&gt;&lt;/h3&gt;
	*        &lt;div&gt;
	*Syncfusion Metro Studio is a collection of over 2500 Metro-style icon templates that can be easily customized to create thousands of unique Metro icons.
	*        &lt;/div&gt;
	*    &lt;/div&gt;
	*  &lt;script type="text/javascript"&gt;
			*   // disabledItems on initialization.
			*	$("#accordion").ejAccordion({ disabledItems: [0,1] });
			* &lt;/script&gt;
			* @memberof ejAccordion
			* @instance
			*/
            disabledItems: [],
			/**		
			* To activate multiple headers for the accordion, give the index value for array type for this API. The enableMultipleOpen property should be set as true. 
			* @default []
			* @type {Integerarray}
			*@example
			* &lt;div id="accordion"&gt;
	*        &lt;h3&gt;
	*            &lt;a href="#"&gt;Orubase&lt;/a&gt;&lt;/h3&gt;
	*        &lt;div&gt;
	*Orubase is the only mobile application development framework built especially for developing complex line-of-business mobile applications targeting iOS, Android, and Windows Phone platforms in the shortest possible timeframe.
	*        &lt;/div&gt;
	*        &lt;h3&gt;
	*            &lt;a href="#"&gt;WinRT XAML&lt;/a&gt;&lt;/h3&gt;
	*        &lt;div&gt;
	*Essential Studio for WinRT contains all the controls you need to build line-of-business tablet applications including grid, chart, map, tree map, SSRS report viewer, rich-text editor, PDF viewer, gauges, barcode, editors, and much more. It also includes a unique set of controls for reading and writing Excel, Word, and PDF documents in Windows store apps.
	*        &lt;/div&gt;
	*        &lt;h3&gt;
	*            &lt;a href="#"&gt;Metro Studio&lt;/a&gt;&lt;/h3&gt;
	*        &lt;div&gt;
	*Syncfusion Metro Studio is a collection of over 2500 Metro-style icon templates that can be easily customized to create thousands of unique Metro icons.
	*        &lt;/div&gt;
	*    &lt;/div&gt;
	*  &lt;script type="text/javascript"&gt;
			*   // enabledItems on initialization.
			*	$("#accordion").ejAccordion({ enabledItems: [0,1] });
			* &lt;/script&gt;
			* @memberof ejAccordion
			* @instance
			*/
            enabledItems:[],
			/**		
			* Save current model value to browser cookies for maintaining states. When refreshing the accordion control page, the model value is applied from browser cookies or HTML 5 local storage.
			* @default false
			* @type {Boolean}
			*@example
			* &lt;div id="accordion"&gt;
	*        &lt;h3&gt;
	*            &lt;a href="#"&gt;Orubase&lt;/a&gt;&lt;/h3&gt;
	*        &lt;div&gt;
	*Orubase is the only mobile application development framework built especially for developing complex line-of-business mobile applications targeting iOS, Android, and Windows Phone platforms in the shortest possible timeframe.
	*        &lt;/div&gt;
	*        &lt;h3&gt;
	*            &lt;a href="#"&gt;WinRT XAML&lt;/a&gt;&lt;/h3&gt;
	*        &lt;div&gt;
	*Essential Studio for WinRT contains all the controls you need to build line-of-business tablet applications including grid, chart, map, tree map, SSRS report viewer, rich-text editor, PDF viewer, gauges, barcode, editors, and much more. It also includes a unique set of controls for reading and writing Excel, Word, and PDF documents in Windows store apps.
	*        &lt;/div&gt;
	*        &lt;h3&gt;
	*            &lt;a href="#"&gt;Metro Studio&lt;/a&gt;&lt;/h3&gt;
	*        &lt;div&gt;
	*Syncfusion Metro Studio is a collection of over 2500 Metro-style icon templates that can be easily customized to create thousands of unique Metro icons.
	*        &lt;/div&gt;
	*    &lt;/div&gt;
	*  &lt;script type="text/javascript"&gt;
			*   // Enable enablePersistence on initialization.
			*	$("#accordion").ejAccordion({ enablePersistence: true});
			* &lt;/script&gt;
			* @memberof ejAccordion
			* @instance
			*/
            enablePersistence: false,
			/**		
			* With the Enabled property, you can enable or disable the Accordion.
			* @default true
			* @type {Boolean}
			*@example
			* &lt;div id="accordion"&gt;
	*        &lt;h3&gt;
	*            &lt;a href="#"&gt;Orubase&lt;/a&gt;&lt;/h3&gt;
	*        &lt;div&gt;
	*Orubase is the only mobile application development framework built especially for developing complex line-of-business mobile applications targeting iOS, Android, and Windows Phone platforms in the shortest possible timeframe.
	*        &lt;/div&gt;
	*        &lt;h3&gt;
	*            &lt;a href="#"&gt;WinRT XAML&lt;/a&gt;&lt;/h3&gt;
	*        &lt;div&gt;
	*Essential Studio for WinRT contains all the controls you need to build line-of-business tablet applications including grid, chart, map, tree map, SSRS report viewer, rich-text editor, PDF viewer, gauges, barcode, editors, and much more. It also includes a unique set of controls for reading and writing Excel, Word, and PDF documents in Windows store apps.
	*        &lt;/div&gt;
	*        &lt;h3&gt;
	*            &lt;a href="#"&gt;Metro Studio&lt;/a&gt;&lt;/h3&gt;
	*        &lt;div&gt;
	*Syncfusion Metro Studio is a collection of over 2500 Metro-style icon templates that can be easily customized to create thousands of unique Metro icons.
	*        &lt;/div&gt;
	*    &lt;/div&gt;
	*  &lt;script type="text/javascript"&gt;
			*   // Enable accordion on initialization.
			*	$("#accordion").ejAccordion({ enabled: true});
			* &lt;/script&gt;
			* @memberof ejAccordion
			* @instance
			*/
            enabled: true,
            //Events
			 /**     
			 * Triggered after the AJAX content loads. Arguments have current model values.
			 * @event
			 * @name ejAccordion#ajaxLoad		
			 * @param {Object} argument Event parameters from accordion     
			 * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
			 * @param {object}  argument.model returns the accordion model
			 * @param {string}  argument.type returns the name of the event
			 * @param {string}  argument.url returns the name of the url
			 * @example 
			 * &lt;div id="accordion"&gt;
	*        &lt;h3&gt;
	*            &lt;a href="#"&gt;Orubase&lt;/a&gt;&lt;/h3&gt;
	*        &lt;div&gt;
	*Orubase is the only mobile application development framework built especially for developing complex line-of-business mobile applications targeting iOS, Android, and Windows Phone platforms in the shortest possible timeframe.
	*        &lt;/div&gt;
	*        &lt;h3&gt;
	*            &lt;a href="#"&gt;WinRT XAML&lt;/a&gt;&lt;/h3&gt;
	*        &lt;div&gt;
	*Essential Studio for WinRT contains all the controls you need to build line-of-business tablet applications including grid, chart, map, tree map, SSRS report viewer, rich-text editor, PDF viewer, gauges, barcode, editors, and much more. It also includes a unique set of controls for reading and writing Excel, Word, and PDF documents in Windows store apps.
	*        &lt;/div&gt;
	*        &lt;h3&gt;
	*            &lt;a href="#"&gt;Metro Studio&lt;/a&gt;&lt;/h3&gt;
	*        &lt;div&gt;
	*Syncfusion Metro Studio is a collection of over 2500 Metro-style icon templates that can be easily customized to create thousands of unique Metro icons.
	*        &lt;/div&gt;
	*    &lt;/div&gt;
	*  &lt;script type="text/javascript"&gt;
			 * //initial ajaxLoad event
			 *	$("#accordion").ejAccordion({ ajaxLoad: function (args) {}
			 * });
			 * &lt;/script&gt;
			 * @memberof ejAccordion
			 * @instance
			 */		
            ajaxLoad: null,
			/**     
			 * Triggered before the AJAX content is loaded in a content panel. Arguments have location of the content (URL) and current model value.
			 * @event
			 * @name ejAccordion#ajaxBeforeLoad		
			 * @param {Object} argument Event parameters from accordion     
			 * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
			 * @param {object}  argument.model returns the accordion model
			 * @param {string}  argument.type returns the name of the event
			 * @param {string}  argument.url returns current ajax content location
			 * @example 
			 * &lt;div id="accordion"&gt;
	*        &lt;h3&gt;
	*            &lt;a href="#"&gt;Orubase&lt;/a&gt;&lt;/h3&gt;
	*        &lt;div&gt;
	*Orubase is the only mobile application development framework built especially for developing complex line-of-business mobile applications targeting iOS, Android, and Windows Phone platforms in the shortest possible timeframe.
	*        &lt;/div&gt;
	*        &lt;h3&gt;
	*            &lt;a href="#"&gt;WinRT XAML&lt;/a&gt;&lt;/h3&gt;
	*        &lt;div&gt;
	*Essential Studio for WinRT contains all the controls you need to build line-of-business tablet applications including grid, chart, map, tree map, SSRS report viewer, rich-text editor, PDF viewer, gauges, barcode, editors, and much more. It also includes a unique set of controls for reading and writing Excel, Word, and PDF documents in Windows store apps.
	*        &lt;/div&gt;
	*        &lt;h3&gt;
	*            &lt;a href="#"&gt;Metro Studio&lt;/a&gt;&lt;/h3&gt;
	*        &lt;div&gt;
	*Syncfusion Metro Studio is a collection of over 2500 Metro-style icon templates that can be easily customized to create thousands of unique Metro icons.
	*        &lt;/div&gt;
	*    &lt;/div&gt;
	*  &lt;script type="text/javascript"&gt;
			 * //initial ajaxBeforeLoad event
			 *	$("#accordion").ejAccordion({ ajaxBeforeLoad: function (args) {}
			 * });
			 * &lt;/script&gt;
			 * @memberof ejAccordion
			 * @instance
			 */		
            ajaxBeforeLoad: null,
			/**     
			 * Triggered after a Accordion item is active or inactive. Argument values are activeindex, activeHeader, inActiveHeader, inActiveIndex and current model value.
			 * @event
			 * @name ejAccordion#click		
			 * @param {Object} argument Event parameters from accordion     
			 * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
			 * @param {object}  argument.model returns the accordion model
			 * @param {string}  argument.type returns the name of the event
			 * @param {number} argument.activeIndex returns active index
             * @param {object} argument.activeHeader returns current active header
             * @param {number} argument.inActiveIndex returns active index
             * @param {object} argument.inActiveHeader returns current active header
			 * @example 
			 * &lt;div id="accordion"&gt;
	*        &lt;h3&gt;
	*            &lt;a href="#"&gt;Orubase&lt;/a&gt;&lt;/h3&gt;
	*        &lt;div&gt;
	*Orubase is the only mobile application development framework built especially for developing complex line-of-business mobile applications targeting iOS, Android, and Windows Phone platforms in the shortest possible timeframe.
	*        &lt;/div&gt;
	*        &lt;h3&gt;
	*            &lt;a href="#"&gt;WinRT XAML&lt;/a&gt;&lt;/h3&gt;
	*        &lt;div&gt;
	*Essential Studio for WinRT contains all the controls you need to build line-of-business tablet applications including grid, chart, map, tree map, SSRS report viewer, rich-text editor, PDF viewer, gauges, barcode, editors, and much more. It also includes a unique set of controls for reading and writing Excel, Word, and PDF documents in Windows store apps.
	*        &lt;/div&gt;
	*        &lt;h3&gt;
	*            &lt;a href="#"&gt;Metro Studio&lt;/a&gt;&lt;/h3&gt;
	*        &lt;div&gt;
	*Syncfusion Metro Studio is a collection of over 2500 Metro-style icon templates that can be easily customized to create thousands of unique Metro icons.
	*        &lt;/div&gt;
	*    &lt;/div&gt;
	*  &lt;script type="text/javascript"&gt;
			 * //initial active event
			 *	$("#accordion").ejAccordion({ click: function (args) {}
			 * });
			 * &lt;/script&gt;
			 * @memberof ejAccordion
			 * @instance
			 */		
            click: null,
			/**     
			 * Triggered before a tab item is active. Arguments have active index and model values.
			 * @event
			 * @name ejAccordion#beforeClick		
			 * @param {Object} argument Event parameters from accordion     
			 * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
			 * @param {object}  argument.model returns the accordion model
			 * @param {string}  argument.type returns the name of the event
			 * @param {number} argument.activeIndex returns active index
			 * @example 
			 * &lt;div id="accordion"&gt;
	*        &lt;h3&gt;
	*            &lt;a href="#"&gt;Orubase&lt;/a&gt;&lt;/h3&gt;
	*        &lt;div&gt;
	*Orubase is the only mobile application development framework built especially for developing complex line-of-business mobile applications targeting iOS, Android, and Windows Phone platforms in the shortest possible timeframe.
	*        &lt;/div&gt;
	*        &lt;h3&gt;
	*            &lt;a href="#"&gt;WinRT XAML&lt;/a&gt;&lt;/h3&gt;
	*        &lt;div&gt;
	*Essential Studio for WinRT contains all the controls you need to build line-of-business tablet applications including grid, chart, map, tree map, SSRS report viewer, rich-text editor, PDF viewer, gauges, barcode, editors, and much more. It also includes a unique set of controls for reading and writing Excel, Word, and PDF documents in Windows store apps.
	*        &lt;/div&gt;
	*        &lt;h3&gt;
	*            &lt;a href="#"&gt;Metro Studio&lt;/a&gt;&lt;/h3&gt;
	*        &lt;div&gt;
	*Syncfusion Metro Studio is a collection of over 2500 Metro-style icon templates that can be easily customized to create thousands of unique Metro icons.
	*        &lt;/div&gt;
	*    &lt;/div&gt;
	*  &lt;script type="text/javascript"&gt;
			 * //initial beforeClick event
			 *	$("#accordion").ejAccordion({ beforeClick: function (args) {}
			 * });
			 * &lt;/script&gt;
			 * @memberof ejAccordion
			 * @instance
			 */		
            beforeClick: null,
			/**     
			 * Triggered after AJAX success action. Arguments have URL, content, and current model values.
			 * @event
			 * @name ejAccordion#ajaxSuccess		
			 * @param {Object} argument Event parameters from accordion     
			 * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
			 * @param {object}  argument.model returns the accordion model
			 * @param {string}  argument.type returns the name of the event
			 * @param {string}  argument.url returns current ajax content location
			 * @param {string}  argument.data returns the succesfull data sent.
			 * @param {string}  argument.content returns the ajax content.
			 * @example 
			 * &lt;div id="accordion"&gt;
	*        &lt;h3&gt;
	*            &lt;a href="#"&gt;Orubase&lt;/a&gt;&lt;/h3&gt;
	*        &lt;div&gt;
	*Orubase is the only mobile application development framework built especially for developing complex line-of-business mobile applications targeting iOS, Android, and Windows Phone platforms in the shortest possible timeframe.
	*        &lt;/div&gt;
	*        &lt;h3&gt;
	*            &lt;a href="#"&gt;WinRT XAML&lt;/a&gt;&lt;/h3&gt;
	*        &lt;div&gt;
	*Essential Studio for WinRT contains all the controls you need to build line-of-business tablet applications including grid, chart, map, tree map, SSRS report viewer, rich-text editor, PDF viewer, gauges, barcode, editors, and much more. It also includes a unique set of controls for reading and writing Excel, Word, and PDF documents in Windows store apps.
	*        &lt;/div&gt;
	*        &lt;h3&gt;
	*            &lt;a href="#"&gt;Metro Studio&lt;/a&gt;&lt;/h3&gt;
	*        &lt;div&gt;
	*Syncfusion Metro Studio is a collection of over 2500 Metro-style icon templates that can be easily customized to create thousands of unique Metro icons.
	*        &lt;/div&gt;
	*    &lt;/div&gt;
	*  &lt;script type="text/javascript"&gt;
			 * //initial ajaxSuccess event
			 *	$("#accordion").ejAccordion({ ajaxSuccess: function (args) {}
			 * });
			 * &lt;/script&gt;
			 * @memberof ejAccordion
			 * @instance
			 */		
            ajaxSuccess: null,
			/**     
			 * Triggered after AJAX load failed action. Arguments have URL, error message, and current model value.
			 * @event
			 * @name ejAccordion#ajaxError		
			 * @param {Object} argument Event parameters from accordion     
			 * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
			 * @param {object}  argument.model returns the accordion model
			 * @param {string}  argument.type returns the name of the event
			 * @param {string}  argument.url returns current ajax content location
			 * @param {string}  argument.data returns the failed data sent.
			 * @example 
			 * &lt;div id="accordion"&gt;
	*        &lt;h3&gt;
	*            &lt;a href="#"&gt;Orubase&lt;/a&gt;&lt;/h3&gt;
	*        &lt;div&gt;
	*Orubase is the only mobile application development framework built especially for developing complex line-of-business mobile applications targeting iOS, Android, and Windows Phone platforms in the shortest possible timeframe.
	*        &lt;/div&gt;
	*        &lt;h3&gt;
	*            &lt;a href="#"&gt;WinRT XAML&lt;/a&gt;&lt;/h3&gt;
	*        &lt;div&gt;
	*Essential Studio for WinRT contains all the controls you need to build line-of-business tablet applications including grid, chart, map, tree map, SSRS report viewer, rich-text editor, PDF viewer, gauges, barcode, editors, and much more. It also includes a unique set of controls for reading and writing Excel, Word, and PDF documents in Windows store apps.
	*        &lt;/div&gt;
	*        &lt;h3&gt;
	*            &lt;a href="#"&gt;Metro Studio&lt;/a&gt;&lt;/h3&gt;
	*        &lt;div&gt;
	*Syncfusion Metro Studio is a collection of over 2500 Metro-style icon templates that can be easily customized to create thousands of unique Metro icons.
	*        &lt;/div&gt;
	*    &lt;/div&gt;
	*  &lt;script type="text/javascript"&gt;
			 * //initial ajaxError event
			 *	$("#accordion").ejAccordion({ ajaxError: function (args) {}
			 * });
			 * &lt;/script&gt;
			 * @memberof ejAccordion
			 * @instance
			 */		
            ajaxError: null,
            /**     
			 * Triggered after Accordion control creation.
			 * @event
			 * @name ejAccordion#create		
			 * @param {Object} argument Event parameters from accordion     
			 * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
			 * @param {object}  argument.model returns the accordion model
			 * @param {string}  argument.type returns the name of the event			
			 * @example 
			 * &lt;div id="accordion"&gt;
	*        &lt;h3&gt;
	*            &lt;a href="#"&gt;Orubase&lt;/a&gt;&lt;/h3&gt;
	*        &lt;div&gt;
	*Orubase is the only mobile application development framework built especially for developing complex line-of-business mobile applications targeting iOS, Android, and Windows Phone platforms in the shortest possible timeframe.
	*        &lt;/div&gt;
	*        &lt;h3&gt;
	*            &lt;a href="#"&gt;WinRT XAML&lt;/a&gt;&lt;/h3&gt;
	*        &lt;div&gt;
	*Essential Studio for WinRT contains all the controls you need to build line-of-business tablet applications including grid, chart, map, tree map, SSRS report viewer, rich-text editor, PDF viewer, gauges, barcode, editors, and much more. It also includes a unique set of controls for reading and writing Excel, Word, and PDF documents in Windows store apps.
	*        &lt;/div&gt;
	*        &lt;h3&gt;
	*            &lt;a href="#"&gt;Metro Studio&lt;/a&gt;&lt;/h3&gt;
	*        &lt;div&gt;
	*Syncfusion Metro Studio is a collection of over 2500 Metro-style icon templates that can be easily customized to create thousands of unique Metro icons.
	*        &lt;/div&gt;
	*    &lt;/div&gt;
	*  &lt;script type="text/javascript"&gt;
			 * //initial create event
			 *	$("#accordion").ejAccordion({ create: function (args) {}
			 * });
			 * &lt;/script&gt;
			 * @memberof ejAccordion
			 * @instance
			 */
            create: null,
            /**     
			 * Triggered after Accordion control destroy.
			 * @event
			 * @name ejAccordion#destroy		
			 * @param {Object} argument Event parameters from accordion     
			 * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
			 * @param {object}  argument.model returns the accordion model
			 * @param {string}  argument.type returns the name of the event			
			 * @example 
			 * &lt;div id="accordion"&gt;
	*        &lt;h3&gt;
	*            &lt;a href="#"&gt;Orubase&lt;/a&gt;&lt;/h3&gt;
	*        &lt;div&gt;
	*Orubase is the only mobile application development framework built especially for developing complex line-of-business mobile applications targeting iOS, Android, and Windows Phone platforms in the shortest possible timeframe.
	*        &lt;/div&gt;
	*        &lt;h3&gt;
	*            &lt;a href="#"&gt;WinRT XAML&lt;/a&gt;&lt;/h3&gt;
	*        &lt;div&gt;
	*Essential Studio for WinRT contains all the controls you need to build line-of-business tablet applications including grid, chart, map, tree map, SSRS report viewer, rich-text editor, PDF viewer, gauges, barcode, editors, and much more. It also includes a unique set of controls for reading and writing Excel, Word, and PDF documents in Windows store apps.
	*        &lt;/div&gt;
	*        &lt;h3&gt;
	*            &lt;a href="#"&gt;Metro Studio&lt;/a&gt;&lt;/h3&gt;
	*        &lt;div&gt;
	*Syncfusion Metro Studio is a collection of over 2500 Metro-style icon templates that can be easily customized to create thousands of unique Metro icons.
	*        &lt;/div&gt;
	*    &lt;/div&gt;
	*  &lt;script type="text/javascript"&gt;
			 * //initial destroy event
			 *	$("#accordion").ejAccordion({ destroy: function (args) {}
			 * });
			 * &lt;/script&gt;
			 * @memberof ejAccordion
			 * @instance
			 */
            destroy: null
            

        },
		 /**
         * Specify the data types for default properties 
		 * @private
         */
        dataTypes: {
            cssClass: "string",
            collapsible: "boolean",
            enabled: "boolean",
            events: "string",
            heightAdjustMode : "enum",
            ajaxSettings: "data",
            customIcon: "data",
            selectedItems: "data",
            disabledItems: "data",
			enableAnimation: "boolean",
        },
		
		 /**
         * To configure the properties at runtime using SetModel		
		 * @private
         */	 
        _setModel: function (options) {
            if (!this.model.enabled && ej.isNullOrUndefined(options["enabled"])) return false;
            for (var key in options) {
                switch (key) {
                    case "events": {
                        this._off(this._headers, this.model.events);
                        this._on(this._headers, options[key], this._headerEventHandler);
                        break;
                    }
                    case "disabledItems": this.disableItems(options[key]); break;
                    case "enabledItems": this.enableItems(options[key]); break;
                    case "enabled": this._enabledAction(options[key]); break;
                    case "selectedItemIndex": this._activeTab(options[key]); break;
                    case "heightAdjustMode ": this._setHeightStyle(options[key]); break;
                    case "cssClass": this._changeSkin(options[key]); break;
                    case "showRoundedCorner": this._roundedCorner(options[key]); break;
                    case "customIcon":
                        {
                            var icons = this.model.customIcon;
                            var newIcons = options[key];
                            this._headers.find("span." + icons.header).removeClass(icons.header).addClass(newIcons.header);
                            this._headers.find("span." + icons.selectedHeader).removeClass(icons.selectedHeader).addClass(newIcons.selectedHeader);
                            break;
                        }
                    case "allowKeyboardNavigation": {
                        if (options[key]) {
                            this._off(this.element, "keydown");
                            this._on(this.element, 'keydown', this._keyPress);
                        }
                        else
                            this._off(this.element, "keydown");
                        break;
                    }
                    case "enableRTL":
                        {
                            if (options[key])
                                this.element.addClass("e-rtl");
                            else
                                this.element.removeClass("e-rtl");
                            break;
                        }
					case "enableAnimation":{
							this.model.enableAnimation=options[key];
                            break;
                        }
                    case "selectedItems": this._selectedItemsAction(options[key]); break;
                    case "expandSpeed": {
                    	this.model.expandSpeed = parseInt(this.model.expandSpeed);
                    }
                    case "collapseSpeed": {
                    	this.model.collapseSpeed = parseInt(this.model.collapseSpeed);
                    }
                }
            }
        },

        // all events bound using this._on will be unbind automatically
		 /**
        * destroy the Accordion widget
		* all events bound using this._on will be unbind automatically and bring the control to pre-init state.
		* @alias destroy
		* @return jQuery
		* @example
		
		*    &lt;div id="accordion"&gt;
		*        &lt;h3&gt;
		*            &lt;a href="#"&gt;Orubase&lt;/a&gt;&lt;/h3&gt;
		*        &lt;div&gt;
		*Orubase is the only mobile application development framework built especially for developing complex line-of-business mobile applications targeting iOS, Android, and Windows Phone platforms in the shortest possible timeframe.
		*        &lt;/div&gt;
		*        &lt;h3&gt;
		*            &lt;a href="#"&gt;WinRT XAML&lt;/a&gt;&lt;/h3&gt;
		*        &lt;div&gt;
		*Essential Studio for WinRT contains all the controls you need to build line-of-business tablet applications including grid, chart, map, tree map, SSRS report viewer, rich-text editor, PDF viewer, gauges, barcode, editors, and much more. It also includes a unique set of controls for reading and writing Excel, Word, and PDF documents in Windows store apps.
		*        &lt;/div&gt;
		*        &lt;h3&gt;
		*            &lt;a href="#"&gt;Metro Studio&lt;/a&gt;&lt;/h3&gt;
		*        &lt;div&gt;
		*Syncfusion Metro Studio is a collection of over 2500 Metro-style icon templates that can be easily customized to create thousands of unique Metro icons.
		*        &lt;/div&gt;
		*    &lt;/div&gt;
		*&lt;script&gt;
		*            $("#accordion").ejAccordion();
		*var accObj = $("#accordion").data("ejAccordion");
		*accObj.destroy(); //Calls the destroy method of Accordion.
		*&lt;/script&gt;
		* @example 
		
		*    &lt;div id="accordion"&gt;
		*        &lt;h3&gt;
		*            &lt;a href="#"&gt;Orubase&lt;/a&gt;&lt;/h3&gt;
		*        &lt;div&gt;
		*Orubase is the only mobile application development framework built especially for developing complex line-of-business mobile applications targeting iOS, Android, and Windows Phone platforms in the shortest possible timeframe.
		*        &lt;/div&gt;
		*        &lt;h3&gt;
		*            &lt;a href="#"&gt;WinRT XAML&lt;/a&gt;&lt;/h3&gt;
		*        &lt;div&gt;
		*Essential Studio for WinRT contains all the controls you need to build line-of-business tablet applications including grid, chart, map, tree map, SSRS report viewer, rich-text editor, PDF viewer, gauges, barcode, editors, and much more. It also includes a unique set of controls for reading and writing Excel, Word, and PDF documents in Windows store apps.
		*        &lt;/div&gt;
		*        &lt;h3&gt;
		*            &lt;a href="#"&gt;Metro Studio&lt;/a&gt;&lt;/h3&gt;
		*        &lt;div&gt;
		*Syncfusion Metro Studio is a collection of over 2500 Metro-style icon templates that can be easily customized to create thousands of unique Metro icons.
		*        &lt;/div&gt;
		*    &lt;/div&gt;
		*&lt;script&gt;
		* // destroy the Accordion
		* $("#Accordion").ejAccordion("destroy");	
		* &lt;/script&gt;
		* @memberof ejAccordion
		* @instance
        */	       
        _destroy: function () {
            /// <summary>This function is  used to destroy the Accordion Object</summary>
            this._removeBaseClass();
			this.element.children('.e-disable').removeClass("e-disable");
            this.element.removeClass("e-acrdn");
            this._headers.removeClass("e-active  e-state-disabled");
            this._contentPanels.removeClass("e-content-active e-disable");
            $(this._headers.find(".e-icon")).remove();
        },

        // constructor function
		/**
         * Create the Accordion widget
		 * @private
         */		 
        _init: function () {
            this._renderControl();
        },

        // render accordion control
		/**
         * Render Section For accordion control		
		 * @private
         */	  
        _renderControl: function () {
            /// <summary>This function is  used to Render the Accordion Object</summary>
            this._headers = this.element.find("> :even");
            this.element.attr("tabindex", 0).attr("role", "tablist");
            this._contentPanels = this._headers.next();
            //Index check negative values
            this._nagativeIndexCheck(this.model.selectedItemIndex);
            // Add the default class
            this._addBaseClass();
            this._setHeightStyle(this.model.heightAdjustMode );
            // Add header icons
            this._renderHeaderIcon();
            this._wireEvents();
            this._roundedCorner(this.model.showRoundedCorner);
            //Activate the header & content
            this._selectedItemsAction(this.model.selectedItems);
            this._enabledAction(this.model.enabled);
            this._setEnabledItems();
           
        },
        _setEnabledItems: function () {
            for (var index = 0; index < this.getItemsCount() ; index++) {
                if (($.inArray(index, this.model.disabledItems) < 0)&&($.inArray(index, this.model.enabledItems) < 0))
                    this.model.enabledItems.push(index);
            }
        },
        //Skin Change at run time
		/**
         * To configure the custom theme for accordion using cssClass property		
		 * @private
         */	 
        _changeSkin: function (skin) {
            if (this.model.cssClass != skin) {
                this.element.removeClass(this.model.cssClass).addClass(skin);
            }
        },
		/**
         * To configure the Selected ites for accordion using SelectedIndex property		
		 * @private
         */	 
        _selectedItemsAction: function (activeList) {
            if (this.model.enableMultipleOpen) {
                for (var seletedindex = 0; seletedindex < activeList.length; seletedindex++) {
                    this._activeTab(activeList[seletedindex], true);
                }
            }
            else
                if(!(this.collapseALL && this.model.collapsible))
                    this._activeTab(this.model.selectedItemIndex, true);
        },
		/**
         * To enable or disable the accordion state		
		 * @private
         */	 
        _enabledAction: function (flag) {
            if (flag) {
                this.element.removeClass("e-disable");
            }
            else {
                this.element.addClass("e-disable");
            }
        },
        //Add base class for headers and content panels
		/**
         * To configure the default classes for rendering Accordion control		
		 * @private
         */	 
        _addBaseClass: function () {
            this.element.addClass(this.model.cssClass + " e-widget");
            if (this.model.enableRTL)
                this.element.addClass("e-rtl");
            this._headers.addClass(" e-select").attr("role", "tab").attr("tabindex",0);
            this._contentPanels.addClass("e-content").attr("role", "tabpanel").attr("aria-hidden", true).hide(); // hide all the content panels while loding
            $(this._contentPanels[this.model.selectedItemIndex]).show();
        },
        //Remove base class for headers and content panels
		/**
         * To remove the default classes for rendering Accordion control		
		 * @private
         */	
        _removeBaseClass: function () {
            this._headers.removeClass(" e-select");
            this._contentPanels.removeClass("e-content").show(); // hide all the content panels while loding
        },
		/**
         * To enable or disable the rounded corner behaviour 		
		 * @private
         */	
        _roundedCorner: function (value) {
            if (value) {
                this._headers.addClass('e-corner-all');
                this.element.find('.e-active').removeClass('e-corner-all').addClass('e-corner-top');
                this.element.find('.e-acrdn-content-active').addClass('e-corner-bottom');
            }
            else if (this._headers.hasClass('e-corner-all')) {
                this._headers.removeClass('e-corner-all');
                this.element.find('.e-active').removeClass('e-corner-top');
                this.element.find('.e-acrdn-content-active').addClass('e-corner-bottom');
            }
        },
		/**
         * To check the negative index value 		
		 * @private
         */	
        _nagativeIndexCheck: function (index) {
            /// <summary>This function is  used to Check the nagative value of the index</summary>
            var panels;
            if (index < 0 && this.model.collapsible) {
                this.collapseALL = true;
                panels = this.getItemsCount();
                for (var pane = 0; pane < panels; pane++) {
                    this._hideTab(pane);
                }
            }
            else if (index < 0 || index >= this._headers.length)
                this.model.selectedItemIndex = 0;
        },
		/**
         * To activate the selected Item of the accordion control		
		 * @private
         */	
        _activeTab: function (index, raiseEvent) {
            /// <summary>This function is  used Activate the selected Item of the accordion control</summary>
            if ($.inArray(index, this.model.disabledItems) < 0) {
                var data = {
                    activeIndex: index
                };
                //triggering the onBeforeClick event
                if (true === (this._trigger("beforeClick", data))) {
                    return false;
                }
                if (!this.model.enableMultipleOpen)
                    this._hideTab(this.model.selectedItemIndex);
                this.model.selectedItemIndex = index;
                var icons = this.model.customIcon;
                this._headers.attr("tabindex",0).removeAttr("aria-expanded");
                this._activeHeader = this._headers.eq(index).addClass("e-active ").attr("aria-selected", true).attr("tabindex", 0).attr("aria-expanded", true);
                this._activeHeader.find(".e-icon")
                        .removeClass(icons.header)
                        .addClass(icons.selectedHeader);
                this._addSelectedItems(index);
                this._ajaxLoad();
                this._roundedCorner(this.model.showRoundedCorner);
                data = {
                    activeHeader: this._activeHeader,
                    activeIndex: this.model.selectedItemIndex
                };
                var proxy = this;
                this._activeContent = this._activeHeader.next().addClass("e-acrdn-content-active").removeAttr("aria-hidden", false).slideDown(this.model.enableAnimation?this.model.expandSpeed:0, "easeOutQuad", function () {
                    // Animation complete  //triggering the onActive event
                    if (!raiseEvent) proxy._trigger("click", data);
                }); //.animate({ height: "show" }, 'fast');
            }
        },
		/**
         * Configure to hide the previous active item of the accordion control		
		 * @private
         */	
        //This function is  used hide the prev active item of the accordion control
        _hideTab: function (index) {
            var icons = this.model.customIcon, data;
            if (index >= 0) {
			    data = {
                    inActiveIndex: index
                };
                //triggering the onBeforeClick event
			    this._trigger("beforeClick", data);
                this._activeHeader = this._headers.eq(index).removeClass("e-active ").removeAttr("aria-selected").attr("tabindex",0).attr("aria-expanded", false);
                this._activeHeader.find(".e-icon")
                        .removeClass(icons.selectedHeader)
                        .addClass(icons.header);
                data = {
                    inActiveHeader: this._activeHeader,
                    inActiveIndex: index
                };
				var proxy = this;
                this._activeHeader.next().removeClass("e-acrdn-content-active").attr("aria-hidden", true).slideUp(this.model.enableAnimation ? this.model.collapseSpeed : 0, "easeOutQuad", function () {  proxy._trigger("click", data); });//.animate({ height: "hide" }, 'fast');                                   
                this._deleteSelectedItems(index);
            }
        },
		/**
         * Configure to create the Header icon for the accordion control		
		 * @private
         */	
        //Cretae Header Icon 
        _renderHeaderIcon: function () {
            if (this._headers.find(".e-icon").length <= 0) {
                var icons = this.model.customIcon;
                if (icons) {
                    var iconSpan = ej.buildTag("span.e-icon " + icons.header).prependTo(this._headers);
                    if (!(this.collapseALL && this.model.collapsible)) {
                        this._headers.eq(this.model.selectedItemIndex).find(".e-icon")
                        .removeClass(icons.header)
                        .addClass(icons.selectedHeader);
                }
            }
            }
        },
		/**
         * To Configure ajaxload for the accordion control		
		 * @private
         */	
        //Ajax Load function 
        _ajaxLoad: function () {
            var anchor = this._activeHeader.find("a[href]");
            if (anchor.length > 0) {
                var content = $(this._contentPanels[this.model.selectedItemIndex]);
                var href = $(anchor).attr("href");
                if (href && href !== "#")
                    this._sendAjaxOptions(content, anchor[0]);
            }
        },
		/**
         * To handle the ajax request for the accordion control		
		 * @private
         */	
        //handling the ajax request
        _sendAjaxOptions: function (content, link) {
            // TODO load waiting popup
            //triggering the onBeforeLoad event
            if (true === (this._trigger("ajaxBeforeLoad", { url: link })))
                return false;
            content.addClass("e-load");
            var proxy = this;
            var curTabTitle = $(link).html();
            var _hrefLink = link.href.replace("#", "");
            var _ajaxOptions = {
                type: this.model.ajaxSettings.type, cache: this.model.ajaxSettings.cache, url: _hrefLink, data: this.model.ajaxSettings.data,
                dataType: this.model.ajaxSettings.dataType, contentType: this.model.ajaxSettings.contentType, async: this.model.ajaxSettings.async,
                "success": function (data) {
                    try {
                        proxy._ajaxSuccessHandler(data, content, link, curTabTitle);
                    } catch (e) {

                    }
                }, "error": function () {
                    try {
                        proxy._ajaxErrorHandler(data, link, proxy.model.selectedItemIndex, curTabTitle);
                    } catch (e) {

                    }
                }
            };
            this._sendAjaxRequest(_ajaxOptions);
        },
		/**
         * Configure to map the Ajax request	
		 * @private
         */	
        //Ajax Request mapping
        _sendAjaxRequest: function (ajaxOptions) {
            $.ajax({
                type: ajaxOptions.type,
                cache: ajaxOptions.cache,
                url: ajaxOptions.url,
                dataType: ajaxOptions.dataType,
                data: ajaxOptions.data,
                contentType: ajaxOptions.contentType,
                async: ajaxOptions.async,
                success: ajaxOptions.success,
                error: ajaxOptions.error,
                beforeSend: ajaxOptions.beforeSend,
                complete: ajaxOptions.complete
            });
        },
		/**
         * Configure to handle the Ajax load success event	
		 * @private
         */	
        //Ajax load success event handler
        _ajaxSuccessHandler: function (data, content, link, curTabTitle) {
            if (curTabTitle != null)
                $(link).html(curTabTitle);
            content.removeClass("e-load");
            content.html(data).addClass("e-acrdn-loaded"); //to indicate the content is already loaded
            var eventData = { data: data, url: link, content: content };
            this._trigger("ajaxSuccess", eventData);
            //triggering the onLoad event
            this._trigger("ajaxLoad", { url: link });
        },
		/**
         * Configure to handle the Ajax load failed event	
		 * @private
         */	
        //Ajax load erroe event handler
        _ajaxErrorHandler: function (data, link, index, title) {
            this._trigger("ajaxError", { data: data, url: link });
            //triggering the onLoad event
            this._trigger("ajaxLoad", { url: link });
        },
		/**
         * Configure to set the content panels height based on the height style
		 * @private
         */	
        //Set the content panels height based on the height style
        _setHeightStyle: function (heightFormat) {
            var proxy = this, maxHeight;
            if (ej.Accordion.HeightAdjustMode.Fill == heightFormat) {
                maxHeight = this._getDimension($(this.element).parent(), "height");
                $(this.element).parent().css({ "overflow": "auto" });
                proxy._headers.each(function () {
                    maxHeight -= proxy._getDimension($(this), "outerHeight");
                });
                var maxPadding = 0;
                proxy._headers.next().each(function () {
                    maxPadding = Math.max(maxPadding, proxy._getDimension($(this), "innerHeight") - proxy._getDimension($(this), "height"));
                }).height(maxHeight - maxPadding).css({ "overflow": "auto" });

            } else if (ej.Accordion.HeightAdjustMode .Auto == heightFormat) {
                maxHeight = 0;
                proxy._headers.next().each(function () {
                    maxHeight = Math.max(maxHeight, proxy._getDimension($(this), "outerHeight"));
                }).height(maxHeight);
                this.maxAutoHeight = maxHeight;
            }
        },
		/**
         * Configure to set the dimensions for accordion control header and content panel
		 * @private
         */	
        _getDimension: function (element, method) {
            var value;
            var $hidden = $(element).parents().andSelf().filter(':hidden');
            var prop = { visibility: 'hidden', display: 'block' };
            var tmp = [];
            $hidden.each(function () {
                var temp = {}, name;
                for (name in prop) {
                    temp[name] = this.style[name];
                    this.style[name] = prop[name];
                }
                tmp.push(temp);
            });
            value = /(outer)/g.test(method) ?
            $(element)[method](true) :
           $(element)[method]();

            $hidden.each(function (i) {
                var temp = tmp[i], name;
                for (name in prop) {
                    this.style[name] = temp[name];
                }
            });

            return value;
        },
		/**
         * Configure to handle header click event
		 * @private
         */	
        // Header Click event handler
        _headerEventHandler: function (event) {
            if (this.model.enabled) {
                event.preventDefault();
                var clicked = $(event.currentTarget);
                var index = this._headers.index(clicked);
                if (this.model.enableMultipleOpen && this._headers.eq(index).hasClass("e-active")) {
                    this._hideTab(index);
                }
                else if (this.model.selectedItemIndex == index && this.model.collapsible) {
                    this._hideTab(this.model.selectedItemIndex);
                    this.model.selectedItemIndex = -1
                }
                else if (!this._headers.eq(index).hasClass("e-active")) {
                    this._activeTab(index);
                }
            }
        },
		/**
         * Configure to add selected items based on index
		 * @private
         */	
        _addSelectedItems: function (index) {
            if ($.inArray(index, this.model.selectedItems) < 0)
                this.model.selectedItems.push(index);
        },
		/**
         * Configure to delete selected items based on index
		 * @private
         */	
        _deleteSelectedItems: function (index) {
            if ($.inArray(index, this.model.selectedItems) > -1) {
                position = $.inArray(index, this.model.selectedItems);
                this.model.selectedItems.splice(position, 1);
            }
        },
		/**
         * Configure to delete disabled items based on index
		 * @private
         */	
        _deleteDisabledItems: function (index) {
            if ($.inArray(index, this.model.disabledItems) > -1) {
                position = $.inArray(index, this.model.disabledItems);
                this.model.disabledItems.splice(position, 1);
            }
        },
        _addEnabledItems: function (index) {
            if ($.inArray(index, this.model.enabledItems) < 0)
                this.model.enabledItems.push(index);
        },
        _deleteEnabledItems: function (index) {
            if ($.inArray(index, this.model.enabledItems) > -1) {
                position = $.inArray(index, this.model.enabledItems);
                this.model.enabledItems.splice(position, 1);
            }
        },
		/**
         * Configure for keyboard navigation 
		 * @private
         */	
        _keyPress: function (e) {
            if (this.model.enabled) {
                var index, currentEle, targetEle = $(e.target);
                if ((targetEle.hasClass("e-select"))||(targetEle.hasClass("e-acrdn"))) {
                    index = this.model.selectedItemIndex;//this._headers.index(targetEle.siblings(".e-active")[0])
                }
                //else if(targetEle.hasClass("e-select"))
                if (e.keyCode) code = e.keyCode; // ie and mozilla/gecko
                else if (e.which) code = e.which; // ns4 and opera
                else code = e.charCode;
               if ((targetEle.hasClass("e-select"))||(targetEle.hasClass("e-acrdn"))) {
                    switch (code) {
                        //down arrow
                        case 39:
                        case 40:
							e.preventDefault();								
                            index = index == (this.getItemsCount() - 1) ? 0 : index + 1;
							this._activeTab(index);
                            currentEle = $(this._headers[index]);
                            break;
                            //up arrow
                        case 38:
                        case 37:   
							e.preventDefault();	
                            index = index == 0 ? (this.getItemsCount() - 1) : index - 1;
							this._activeTab(index);
                            currentEle = $(this._headers[index]);
                            break;
                            //End key
                        case 35:                
							e.preventDefault();	
                            currentEle = $(this._headers[this.getItemsCount() - 1]);
							this._activeTab((this.getItemsCount() - 1));
                            break;
                            //Home key
                        case 36:    
							e.preventDefault();	
                            currentEle = $(this._headers[0]);
							this._activeTab(0);
                            break;
                        case 32:
                        case 13:
							e.preventDefault();	
                            $(this._headers[index]).hasClass("e-active") && (this.model.enableMultipleOpen || this.model.collapsible) ? this._hideTab(index) : this._activeTab(index);
                            break;
                    }
                }
                else if (e.ctrlKey && !targetEle.hasClass("e-acrdn")) {
                    switch (code) {
                        case 38:    
							e.preventDefault();	
                            index = this._contentPanels.index(targetEle.parent(".e-content"));
                            currentEle = $(this._headers[index]);
							this._activeTab(index);
                            break;
                        case 33:  
							e.preventDefault();	
                            currentEle = $(this._headers[0]);
							this._activeTab(0);
                            break;
                        case 34:   
							e.preventDefault();	
                            currentEle = $(this._headers[this.getItemsCount() - 1]);
							this._activeTab((this.getItemsCount() - 1));
                            break;
                    }
                }
                if (!ej.isNullOrUndefined(currentEle)) {
                    currentEle.addClass("e-focus");
                    currentEle.focus();
                }
            }
        },
		/**
         * Wiring the events to accordion control		
		 * @private
         */		
        //-------------------- Event Wire-up -------------------------//
        _wireEvents: function () {
            //tabs Click evnts
            this._on(this._headers, this.model.events, this._headerEventHandler);
            if (this.model.allowKeyboardNavigation) {
                this._on(this.element, 'keydown', this._keyPress);
            }
        },
		/**
         * Unwiring the events to accordion control		
		 * @private
         */	
        //-------------------- Event UnWire-up -------------------------//
        _unWireEvents: function () {
            this._off(this._headers, this.model.events);
            if (this.model.allowKeyboardNavigation) {
                this._off(this.element, 'keydown');
            }
        },
        /*---------------------Public methods---------------------*/
        //client side methods
		/**
        * enable the accordion widget
		* Enables given index header and content panel.
		* @alias enableItems
		* @return jQuery
		* @example
		
		*    &lt;div id="accordion"&gt;
		*        &lt;h3&gt;
		*            &lt;a href="#"&gt;Orubase&lt;/a&gt;&lt;/h3&gt;
		*        &lt;div&gt;
		* Orubase is the only mobile application development framework built especially for developing complex line-of-business mobile applications targeting iOS, Android, and Windows Phone platforms in the shortest possible timeframe.
		*        &lt;/div&gt;
		*        &lt;h3&gt;
		*            &lt;a href="#"&gt;WinRT XAML&lt;/a&gt;&lt;/h3&gt;
		*        &lt;div&gt;
		* Essential Studio for WinRT contains all the controls you need to build line-of-business tablet applications including grid, chart, map, tree map, SSRS report viewer, rich-text editor, PDF viewer, gauges, barcode, editors, and much more. It also includes a unique set of controls for reading and writing Excel, Word, and PDF documents in Windows store apps.
		*        &lt;/div&gt;
		*        &lt;h3&gt;
		*            &lt;a href="#"&gt;Metro Studio&lt;/a&gt;&lt;/h3&gt;
		*        &lt;div&gt;
		* Syncfusion Metro Studio is a collection of over 2500 Metro-style icon templates that can be easily customized to create thousands of unique Metro icons.
		*        &lt;/div&gt;
		*    &lt;/div&gt;
		* &lt;script&gt;
*            $("#accordion").ejAccordion();
		* // Call enableItems method.
        *$("#accordion").ejAccordion("disableItems", 0); 
        *$("#accordion").ejAccordion("disableItems", 1); 
		* $("#accordion").ejAccordion("enableItems", 1);
		* &lt;/script&gt;
		* @example 
		
		*    &lt;div id="accordion"&gt;
		*        &lt;h3&gt;
		*            &lt;a href="#"&gt;Orubase&lt;/a&gt;&lt;/h3&gt;
		*        &lt;div&gt;
		*Orubase is the only mobile application development framework built especially for developing complex line-of-business mobile applications targeting iOS, Android, and Windows Phone platforms in the shortest possible timeframe.
		*        &lt;/div&gt;
		*        &lt;h3&gt;
		*            &lt;a href="#"&gt;WinRT XAML&lt;/a&gt;&lt;/h3&gt;
		*        &lt;div&gt;
		*Essential Studio for WinRT contains all the controls you need to build line-of-business tablet applications including grid, chart, map, tree map, SSRS report viewer, rich-text editor, PDF viewer, gauges, barcode, editors, and much more. It also includes a unique set of controls for reading and writing Excel, Word, and PDF documents in Windows store apps.
		*        &lt;/div&gt;
		*        &lt;h3&gt;
		*            &lt;a href="#"&gt;Metro Studio&lt;/a&gt;&lt;/h3&gt;
		*        &lt;div&gt;
		*Syncfusion Metro Studio is a collection of over 2500 Metro-style icon templates that can be easily customized to create thousands of unique Metro icons.
		*        &lt;/div&gt;
		*    &lt;/div&gt;
		*&lt;script&gt;
		* // enableItems the Accordion
		* $("#accordion").ejAccordion();
		*var accObj = $("#accordion").data("ejAccordion");
        *accObj.disableItems(0);
        *accObj.disableItems(1);
		*accObj.enableItems(1); //Calls the enable method of Accordion.	
		* &lt;/script&gt;
		* @memberof ejAccordion
		* @instance
         */	       
        enableItems: function (itemIndexes) {
            /// <summary>This function enable the given item of the Accordion control</summary>
            /// <param name="itemIndexes" type="int"></param>
            if (!this.model.enabled) return false;
            if ($.isArray(itemIndexes)) {
                for (var i = 0; i < itemIndexes.length; i++) {
                    $(this._headers[itemIndexes[i]]).removeClass("e-disable");
                    $(this._contentPanels[itemIndexes[i]]).removeClass("e-disable");
                    this._on($(this._headers[itemIndexes[i]]), this.model.events, this._headerEventHandler);
                    this.model.enabledItems.push(itemIndexes[i]);
                    this._deleteDisabledItems(itemIndexes[i]);
                }
            }
            else {
                $(this._headers[itemIndexes]).removeClass("e-disable");
                $(this._contentPanels[itemIndexes]).removeClass("e-disable");
                this._on($(this._headers[itemIndexes]), this.model.events, this._headerEventHandler);
                this.model.enabledItems.push(itemIndexes[i]);
                this._deleteDisabledItems(itemIndexes);
            }
        },
		/**
        * enable the accordion widget
		* Enables given index header and content panel.
		* @alias disableItems
		* @return jQuery
		* @example
		
		*    &lt;div id="accordion"&gt;
		*        &lt;h3&gt;
		*            &lt;a href="#"&gt;Orubase&lt;/a&gt;&lt;/h3&gt;
		*        &lt;div&gt;
		* Orubase is the only mobile application development framework built especially for developing complex line-of-business mobile applications targeting iOS, Android, and Windows Phone platforms in the shortest possible timeframe.
		*        &lt;/div&gt;
		*        &lt;h3&gt;
		*            &lt;a href="#"&gt;WinRT XAML&lt;/a&gt;&lt;/h3&gt;
		*        &lt;div&gt;
		* Essential Studio for WinRT contains all the controls you need to build line-of-business tablet applications including grid, chart, map, tree map, SSRS report viewer, rich-text editor, PDF viewer, gauges, barcode, editors, and much more. It also includes a unique set of controls for reading and writing Excel, Word, and PDF documents in Windows store apps.
		*        &lt;/div&gt;
		*        &lt;h3&gt;
		*            &lt;a href="#"&gt;Metro Studio&lt;/a&gt;&lt;/h3&gt;
		*        &lt;div&gt;
		* Syncfusion Metro Studio is a collection of over 2500 Metro-style icon templates that can be easily customized to create thousands of unique Metro icons.
		*        &lt;/div&gt;
		*    &lt;/div&gt;
		* &lt;script&gt;
*            $("#accordion").ejAccordion();
		* // Call disableItems method.
		* $("#accordion").ejAccordion("disableItems", 1);
		* &lt;/script&gt;
		* @example 
		
		*    &lt;div id="accordion"&gt;
		*        &lt;h3&gt;
		*            &lt;a href="#"&gt;Orubase&lt;/a&gt;&lt;/h3&gt;
		*        &lt;div&gt;
		*Orubase is the only mobile application development framework built especially for developing complex line-of-business mobile applications targeting iOS, Android, and Windows Phone platforms in the shortest possible timeframe.
		*        &lt;/div&gt;
		*        &lt;h3&gt;
		*            &lt;a href="#"&gt;WinRT XAML&lt;/a&gt;&lt;/h3&gt;
		*        &lt;div&gt;
		*Essential Studio for WinRT contains all the controls you need to build line-of-business tablet applications including grid, chart, map, tree map, SSRS report viewer, rich-text editor, PDF viewer, gauges, barcode, editors, and much more. It also includes a unique set of controls for reading and writing Excel, Word, and PDF documents in Windows store apps.
		*        &lt;/div&gt;
		*        &lt;h3&gt;
		*            &lt;a href="#"&gt;Metro Studio&lt;/a&gt;&lt;/h3&gt;
		*        &lt;div&gt;
		*Syncfusion Metro Studio is a collection of over 2500 Metro-style icon templates that can be easily customized to create thousands of unique Metro icons.
		*        &lt;/div&gt;
		*    &lt;/div&gt;
		*&lt;script&gt;
		* // disableItems the Accordion
		* $("#accordion").ejAccordion();
		*var accObj = $("#accordion").data("ejAccordion");
		*accObj.disableItems(1); //Calls the disableItems method of Accordion.	
		* &lt;/script&gt;
		* @memberof ejAccordion
		* @instance
         */	     
        disableItems: function (itemIndexes) {
            /// <summary>This function disable the given item of the Accordion control</summary>
            /// <param name="itemIndexes" type="int"></param>
            if (!this.model.enabled) return false;
            if ($.isArray(itemIndexes)) {
                for (var i = 0; i < itemIndexes.length; i++) {
                    $(this._headers[itemIndexes[i]]).addClass("e-disable");
                    $(this._contentPanels[itemIndexes[i]]).addClass("e-disable");
                    $(this._headers[itemIndexes[i]]).unbind(this.model.events);
                    this.model.disabledItems.push(itemIndexes[i]);
                    this._deleteEnabledItems(itemIndexes[i]);
                }
            }
            else {
                $(this._headers[itemIndexes]).addClass("e-disable");
                $(this._contentPanels[itemIndexes]).addClass("e-disable");
                $(this._headers[itemIndexes]).unbind(this.model.events);
                this.model.disabledItems.push(itemIndexes);
                this._deleteEnabledItems(itemIndexes[i]);
            }
        },
	
		/**
        * disables the accordion widget
		* disables All the header and content panel.
		* @alias disable
		* @return jQuery
		* @example
		
		*    &lt;div id="accordion"&gt;
		*        &lt;h3&gt;
		*            &lt;a href="#"&gt;Orubase&lt;/a&gt;&lt;/h3&gt;
		*        &lt;div&gt;
		* Orubase is the only mobile application development framework built especially for developing complex line-of-business mobile applications targeting iOS, Android, and Windows Phone platforms in the shortest possible timeframe.
		*        &lt;/div&gt;
		*        &lt;h3&gt;
		*            &lt;a href="#"&gt;WinRT XAML&lt;/a&gt;&lt;/h3&gt;
		*        &lt;div&gt;
		* Essential Studio for WinRT contains all the controls you need to build line-of-business tablet applications including grid, chart, map, tree map, SSRS report viewer, rich-text editor, PDF viewer, gauges, barcode, editors, and much more. It also includes a unique set of controls for reading and writing Excel, Word, and PDF documents in Windows store apps.
		*        &lt;/div&gt;
		*        &lt;h3&gt;
		*            &lt;a href="#"&gt;Metro Studio&lt;/a&gt;&lt;/h3&gt;
		*        &lt;div&gt;
		* Syncfusion Metro Studio is a collection of over 2500 Metro-style icon templates that can be easily customized to create thousands of unique Metro icons.
		*        &lt;/div&gt;
		*    &lt;/div&gt;
		* &lt;script&gt;
		* // Call disable method.
		*  $("#accordion").ejAccordion();
		* $("#accordion").ejAccordion("disable");
		* &lt;/script&gt;
		* @example 
		
		*    &lt;div id="accordion"&gt;
		*        &lt;h3&gt;
		*            &lt;a href="#"&gt;Orubase&lt;/a&gt;&lt;/h3&gt;
		*        &lt;div&gt;
		*Orubase is the only mobile application development framework built especially for developing complex line-of-business mobile applications targeting iOS, Android, and Windows Phone platforms in the shortest possible timeframe.
		*        &lt;/div&gt;
		*        &lt;h3&gt;
		*            &lt;a href="#"&gt;WinRT XAML&lt;/a&gt;&lt;/h3&gt;
		*        &lt;div&gt;
		*Essential Studio for WinRT contains all the controls you need to build line-of-business tablet applications including grid, chart, map, tree map, SSRS report viewer, rich-text editor, PDF viewer, gauges, barcode, editors, and much more. It also includes a unique set of controls for reading and writing Excel, Word, and PDF documents in Windows store apps.
		*        &lt;/div&gt;
		*        &lt;h3&gt;
		*            &lt;a href="#"&gt;Metro Studio&lt;/a&gt;&lt;/h3&gt;
		*        &lt;div&gt;
		*Syncfusion Metro Studio is a collection of over 2500 Metro-style icon templates that can be easily customized to create thousands of unique Metro icons.
		*        &lt;/div&gt;
		*    &lt;/div&gt;
		*&lt;script&gt;
		* // disable the Accordion
		* $("#accordion").ejAccordion();
		*var accObj = $("#accordion").data("ejAccordion");
		*accObj.disable(); //Calls the disable method of Accordion.	
		* &lt;/script&gt;
		* @memberof ejAccordion
		* @instance
         */	       

        disable: function () {
            /// <summary>This function disable all item of the Accordion control</summary>
            for (var i = 0; i < this._headers.length; i++) {
                $(this._headers[i]).addClass("e-disable");
                $(this._contentPanels[i]).addClass("e-disable");
                this.model.disabledItems.push(i);
            }
            this.model.enabledItems = [];
            this._unWireEvents();
        },
		/**
        * enable the accordion widget
		* Enables all the headers and content panels.
		* @alias enable
		* @return jQuery
		* @example
		
		*    &lt;div id="accordion"&gt;
		*        &lt;h3&gt;
		*            &lt;a href="#"&gt;Orubase&lt;/a&gt;&lt;/h3&gt;
		*        &lt;div&gt;
		* Orubase is the only mobile application development framework built especially for developing complex line-of-business mobile applications targeting iOS, Android, and Windows Phone platforms in the shortest possible timeframe.
		*        &lt;/div&gt;
		*        &lt;h3&gt;
		*            &lt;a href="#"&gt;WinRT XAML&lt;/a&gt;&lt;/h3&gt;
		*        &lt;div&gt;
		* Essential Studio for WinRT contains all the controls you need to build line-of-business tablet applications including grid, chart, map, tree map, SSRS report viewer, rich-text editor, PDF viewer, gauges, barcode, editors, and much more. It also includes a unique set of controls for reading and writing Excel, Word, and PDF documents in Windows store apps.
		*        &lt;/div&gt;
		*        &lt;h3&gt;
		*            &lt;a href="#"&gt;Metro Studio&lt;/a&gt;&lt;/h3&gt;
		*        &lt;div&gt;
		* Syncfusion Metro Studio is a collection of over 2500 Metro-style icon templates that can be easily customized to create thousands of unique Metro icons.
		*        &lt;/div&gt;
		*    &lt;/div&gt;
		* &lt;script&gt;
		*            $("#accordion").ejAccordion();
		* // Call enable method.
		* $("#accordion").ejAccordion("enable");
		* &lt;/script&gt;
		* @example 
		
		*    &lt;div id="accordion"&gt;
		*        &lt;h3&gt;
		*            &lt;a href="#"&gt;Orubase&lt;/a&gt;&lt;/h3&gt;
		*        &lt;div&gt;
		*Orubase is the only mobile application development framework built especially for developing complex line-of-business mobile applications targeting iOS, Android, and Windows Phone platforms in the shortest possible timeframe.
		*        &lt;/div&gt;
		*        &lt;h3&gt;
		*            &lt;a href="#"&gt;WinRT XAML&lt;/a&gt;&lt;/h3&gt;
		*        &lt;div&gt;
		*Essential Studio for WinRT contains all the controls you need to build line-of-business tablet applications including grid, chart, map, tree map, SSRS report viewer, rich-text editor, PDF viewer, gauges, barcode, editors, and much more. It also includes a unique set of controls for reading and writing Excel, Word, and PDF documents in Windows store apps.
		*        &lt;/div&gt;
		*        &lt;h3&gt;
		*            &lt;a href="#"&gt;Metro Studio&lt;/a&gt;&lt;/h3&gt;
		*        &lt;div&gt;
		*Syncfusion Metro Studio is a collection of over 2500 Metro-style icon templates that can be easily customized to create thousands of unique Metro icons.
		*        &lt;/div&gt;
		*    &lt;/div&gt;
		*&lt;script&gt;
		* // destroy the Accordion
		* $("#accordion").ejAccordion();
		*var accObj = $("#accordion").data("ejAccordion");
		*accObj.enable(); //Calls the enable method of Accordion.	
		* &lt;/script&gt;
		* @memberof ejAccordion
		* @instance
         */	       
        enable: function () {
            /// <summary>This function enable all item of the Accordion control</summary>
            for (var i = 0; i < this._headers.length; i++) {
                $(this._headers[i]).removeClass("e-disable");
                $(this._contentPanels[i]).removeClass("e-disable");
            }
            this.model.disabledItems = [];
            this._setEnabledItems();
            this._unWireEvents();
            this._wireEvents();
        },
		
        _selected: function (index) {
            /// <summary>To Activated give index of the Accordion control</summary>
            /// <param name="index" type="int"></param>
            if (!this.model.enabled) return false;
            if (index != null && this.model.selectedItemIndex != index) {
                this._activeTab(index);
            }
        },
		/**
		* Shows the hidden Accordion control.
		* @alias show
		* @return jQuery
		* @example
		
		*    &lt;div id="accordion"&gt;
		*        &lt;h3&gt;
		*            &lt;a href="#"&gt;Orubase&lt;/a&gt;&lt;/h3&gt;
		*        &lt;div&gt;
		* Orubase is the only mobile application development framework built especially for developing complex line-of-business mobile applications targeting iOS, Android, and Windows Phone platforms in the shortest possible timeframe.
		*        &lt;/div&gt;
		*        &lt;h3&gt;
		*            &lt;a href="#"&gt;WinRT XAML&lt;/a&gt;&lt;/h3&gt;
		*        &lt;div&gt;
		* Essential Studio for WinRT contains all the controls you need to build line-of-business tablet applications including grid, chart, map, tree map, SSRS report viewer, rich-text editor, PDF viewer, gauges, barcode, editors, and much more. It also includes a unique set of controls for reading and writing Excel, Word, and PDF documents in Windows store apps.
		*        &lt;/div&gt;
		*        &lt;h3&gt;
		*            &lt;a href="#"&gt;Metro Studio&lt;/a&gt;&lt;/h3&gt;
		*        &lt;div&gt;
		* Syncfusion Metro Studio is a collection of over 2500 Metro-style icon templates that can be easily customized to create thousands of unique Metro icons.
		*        &lt;/div&gt;
		*    &lt;/div&gt;
		* &lt;script&gt;
		*            $("#accordion").ejAccordion();
		* // Call show method.
		* $("#accordion").ejAccordion("show");
		* &lt;/script&gt;
		* @example 
		
		*    &lt;div id="accordion"&gt;
		*        &lt;h3&gt;
		*            &lt;a href="#"&gt;Orubase&lt;/a&gt;&lt;/h3&gt;
		*        &lt;div&gt;
		*Orubase is the only mobile application development framework built especially for developing complex line-of-business mobile applications targeting iOS, Android, and Windows Phone platforms in the shortest possible timeframe.
		*        &lt;/div&gt;
		*        &lt;h3&gt;
		*            &lt;a href="#"&gt;WinRT XAML&lt;/a&gt;&lt;/h3&gt;
		*        &lt;div&gt;
		*Essential Studio for WinRT contains all the controls you need to build line-of-business tablet applications including grid, chart, map, tree map, SSRS report viewer, rich-text editor, PDF viewer, gauges, barcode, editors, and much more. It also includes a unique set of controls for reading and writing Excel, Word, and PDF documents in Windows store apps.
		*        &lt;/div&gt;
		*        &lt;h3&gt;
		*            &lt;a href="#"&gt;Metro Studio&lt;/a&gt;&lt;/h3&gt;
		*        &lt;div&gt;
		*Syncfusion Metro Studio is a collection of over 2500 Metro-style icon templates that can be easily customized to create thousands of unique Metro icons.
		*        &lt;/div&gt;
		*    &lt;/div&gt;
		*&lt;script&gt;
		* // show the Accordion
		* $("#accordion").ejAccordion();
		*var accObj = $("#accordion").data("ejAccordion");
		*accObj.show(); //Calls the show method of Accordion.	
		* &lt;/script&gt;
		* @memberof ejAccordion
		* @instance
         */	    
        show: function () {
            /// <summary>To Show the Accordion control</summary>
            if (!this.model.enabled) return false;
            this.element.css("visibility", "visible");
        },
		/**
		* Hides the visible Accordion control.
		* @alias hide
		* @return jQuery
		* @example
		
		*    &lt;div id="accordion"&gt;
		*        &lt;h3&gt;
		*            &lt;a href="#"&gt;Orubase&lt;/a&gt;&lt;/h3&gt;
		*        &lt;div&gt;
		* Orubase is the only mobile application development framework built especially for developing complex line-of-business mobile applications targeting iOS, Android, and Windows Phone platforms in the shortest possible timeframe.
		*        &lt;/div&gt;
		*        &lt;h3&gt;
		*            &lt;a href="#"&gt;WinRT XAML&lt;/a&gt;&lt;/h3&gt;
		*        &lt;div&gt;
		* Essential Studio for WinRT contains all the controls you need to build line-of-business tablet applications including grid, chart, map, tree map, SSRS report viewer, rich-text editor, PDF viewer, gauges, barcode, editors, and much more. It also includes a unique set of controls for reading and writing Excel, Word, and PDF documents in Windows store apps.
		*        &lt;/div&gt;
		*        &lt;h3&gt;
		*            &lt;a href="#"&gt;Metro Studio&lt;/a&gt;&lt;/h3&gt;
		*        &lt;div&gt;
		* Syncfusion Metro Studio is a collection of over 2500 Metro-style icon templates that can be easily customized to create thousands of unique Metro icons.
		*        &lt;/div&gt;
		*    &lt;/div&gt;
		* &lt;script&gt;
		*            $("#accordion").ejAccordion();
		* // Call hide method.
		* $("#accordion").ejAccordion("hide");
		* &lt;/script&gt;
		* @example 
		
		*    &lt;div id="accordion"&gt;
		*        &lt;h3&gt;
		*            &lt;a href="#"&gt;Orubase&lt;/a&gt;&lt;/h3&gt;
		*        &lt;div&gt;
		*Orubase is the only mobile application development framework built especially for developing complex line-of-business mobile applications targeting iOS, Android, and Windows Phone platforms in the shortest possible timeframe.
		*        &lt;/div&gt;
		*        &lt;h3&gt;
		*            &lt;a href="#"&gt;WinRT XAML&lt;/a&gt;&lt;/h3&gt;
		*        &lt;div&gt;
		*Essential Studio for WinRT contains all the controls you need to build line-of-business tablet applications including grid, chart, map, tree map, SSRS report viewer, rich-text editor, PDF viewer, gauges, barcode, editors, and much more. It also includes a unique set of controls for reading and writing Excel, Word, and PDF documents in Windows store apps.
		*        &lt;/div&gt;
		*        &lt;h3&gt;
		*            &lt;a href="#"&gt;Metro Studio&lt;/a&gt;&lt;/h3&gt;
		*        &lt;div&gt;
		*Syncfusion Metro Studio is a collection of over 2500 Metro-style icon templates that can be easily customized to create thousands of unique Metro icons.
		*        &lt;/div&gt;
		*    &lt;/div&gt;
		*&lt;script&gt;
		* // hide the Accordion
		* $("#accordion").ejAccordion();
		*var accObj = $("#accordion").data("ejAccordion");
		*accObj.hide(); //Calls the hide method of Accordion.	
		* &lt;/script&gt;
		* @memberof ejAccordion
		* @instance
         */	    
        hide: function () {
            /// <summary>To Hide the Accordion control</summary>
            if (!this.model.enabled) return false;
            this.element.css("visibility", "hidden");
        },
		
		/**
		* Returns the total number of panels in the control.
		* @alias getItemsCount
		* @return jQuery
		* @example
		*    &lt;div id="accordion"&gt;
		*        &lt;h3&gt;
		*            &lt;a href="#"&gt;Orubase&lt;/a&gt;&lt;/h3&gt;
		*        &lt;div&gt;
		* Orubase is the only mobile application development framework built especially for developing complex line-of-business mobile applications targeting iOS, Android, and Windows Phone platforms in the shortest possible timeframe.
		*        &lt;/div&gt;
		*        &lt;h3&gt;
		*            &lt;a href="#"&gt;WinRT XAML&lt;/a&gt;&lt;/h3&gt;
		*        &lt;div&gt;
		* Essential Studio for WinRT contains all the controls you need to build line-of-business tablet applications including grid, chart, map, tree map, SSRS report viewer, rich-text editor, PDF viewer, gauges, barcode, editors, and much more. It also includes a unique set of controls for reading and writing Excel, Word, and PDF documents in Windows store apps.
		*        &lt;/div&gt;
		*        &lt;h3&gt;
		*            &lt;a href="#"&gt;Metro Studio&lt;/a&gt;&lt;/h3&gt;
		*        &lt;div&gt;
		* Syncfusion Metro Studio is a collection of over 2500 Metro-style icon templates that can be easily customized to create thousands of unique Metro icons.
		*        &lt;/div&gt;
		*    &lt;/div&gt;
		* &lt;script&gt;
		*            $("#accordion").ejAccordion();
		* // Call getItemsCount method.
		* $("#accordion").ejAccordion("getItemsCount");
		*&lt;/script&gt;
		* @example 
		
		*    &lt;div id="accordion"&gt;
		*        &lt;h3&gt;
		*            &lt;a href="#"&gt;Orubase&lt;/a&gt;&lt;/h3&gt;
		*        &lt;div&gt;
		*Orubase is the only mobile application development framework built especially for developing complex line-of-business mobile applications targeting iOS, Android, and Windows Phone platforms in the shortest possible timeframe.
		*        &lt;/div&gt;
		*        &lt;h3&gt;
		*            &lt;a href="#"&gt;WinRT XAML&lt;/a&gt;&lt;/h3&gt;
		*        &lt;div&gt;
		*Essential Studio for WinRT contains all the controls you need to build line-of-business tablet applications including grid, chart, map, tree map, SSRS report viewer, rich-text editor, PDF viewer, gauges, barcode, editors, and much more. It also includes a unique set of controls for reading and writing Excel, Word, and PDF documents in Windows store apps.
		*        &lt;/div&gt;
		*        &lt;h3&gt;
		*            &lt;a href="#"&gt;Metro Studio&lt;/a&gt;&lt;/h3&gt;
		*        &lt;div&gt;
		*Syncfusion Metro Studio is a collection of over 2500 Metro-style icon templates that can be easily customized to create thousands of unique Metro icons.
		*        &lt;/div&gt;
		*    &lt;/div&gt;
		*&lt;script&gt;
		* // getItemsCount the Accordion
		* $("#accordion").ejAccordion();
		*var accObj = $("#accordion").data("ejAccordion");
		*accObj.getItemsCount(); //Calls the getItemsCount method of Accordion.	
		* &lt;/script&gt;
		* @memberof ejAccordion
		* @instance
         */	    
        getItemsCount: function () {
            /// <summary>This function get the number of panel rendered</summary>
            if (this._headers) {
                return this._headers.length;
            }
        }
    });
	/**
	 * Enum for accordion Content height style	 
	 * @enum {string}
	 * @global 
	 */
    //Content height style enum
    ej.Accordion.HeightAdjustMode = {
		/**  Height fit to the content in the panel */
        Content: "content",
		/**  Height set to the largest content in the panel */
        Auto: "auto",
		/**  Height filled to the content of the panel */
        Fill: "fill"
    };
})(jQuery, Syncfusion);;
/**
* @fileOverview Plugin to style the Progressbar elements
* @copyright Copyright Syncfusion Inc. 2001 - 2013. All rights reserved.
*  Use of this code is subject to the terms of our license.
*  A copy of the current license can be obtained at any time by e-mailing
*  licensing@syncfusion.com. Any infringement will be prosecuted under
*  applicable laws. 
* @version 12.1 
* @author <a href="mailto:licensing@syncfusion.com">Syncfusion Inc</a>
*/

(function ($, ej, undefined) {
    /**
   * @namespace ej
   * @class ejProgressBar
   * @requires jQuery
    * @requires jquery.easing.1.3.js
    * @requires ej.core.js
	* @requires ej.progressbar.js
	* @classdesc Custom Design for Progressbar control.
   * @example
   *&lt;div id="progress"&gt;&lt;/div&gt;
   *&lt;script&gt;
   *$("#progress").ejProgressBar({value: 50});
   *&lt;/script&gt;

   */
    // ejProgressbar is the plugin name 
    // "ej.Progressbar" is "namespace.className" will hold functions and properties

    ej.widget("ejProgressBar", "ej.ProgressBar", {
        // widget element will be automatically set in this
        element: null,

        // user defined model will be automatically set in this
        model: null,
        validTags: ["div", "span"],
        _setFirst: false,
        _rootCSS: "e-progressbar",

        // default model
        defaults: {
            /// <summary>This Contains default property of Sycnfusion Tab control </summary>
            /**		
			* Sets the custom text for the ProgressBar. The text placed in the middle of the ProgressBar and it can be customizable using the class 'e-progress-text'. 
			* @default null
			* @type {String}
			* @example 
			* &lt;div id="progress"&gt;&lt;/div&gt;
			* &lt;script&gt; 
			* //To set the text API value during initialization  
			* $("#progress").ejProgressBar({ text: 'loading...',value: 50 });
			* &lt;/script&gt;
			* @memberof ejProgressBar
			* @instance
			*/
            text: "",
            /**
			* Sets the root class for ProgressBar theme. This cssClass API helps to use custom skinning option for ProgressBar control. By defining the root class using this API, we need to include this root class in CSS.
			* @default null
			* @type {String}
			* @example 
			* &lt;div id="progress"&gt;&lt;/div&gt;
			* &lt;script&gt;
			* //Initialize the ProgressBar with the cssClass value specified
			* $("#progress").ejProgressBar({ cssClass: 'gradient-lime ',value: 50  });
			* &lt;/script&gt;
			* @memberof ejProgressBar
			* @instance
			*/
            cssClass: "",
            /**
			* Sets the minimum value of the ProgressBar.
			* @default 0
			* @type {Number}
			* @example
			* &lt;div id="progress"&gt;&lt;/div&gt;
			* &lt;script&gt;
			* //Initialize the ProgressBar with the min value specified
			* $("#progress").ejProgressBar({ minValue: 50,value: 50});
			* &lt;/script&gt;
			* @memberof ejProgressBar
			* @instance
			*/
            minValue: 0,
            /**
			* Sets the maximum value of the ProgressBar.
			* @default 100
			* @type {Number}
			* @example 
			* &lt;div id="progress"&gt;&lt;/div&gt;
			* &lt;script&gt;
			* //Initialize the ProgressBar with the max value specified
			* $("#progress").ejProgressBar({ maxValue: 200 ,value: 200 });
			* &lt;/script&gt;
			* @memberof ejProgressBar
			* @instance
			*/
            maxValue: 100,
            /**
			* Sets the ProgressBar value. The value should be in between min and max values.
			* @default 0
			* @type {Number}
			* @example
			* &lt;div id="progress"&gt;&lt;/div&gt;
			* &lt;script&gt;
			* //Initialize the ProgressBar with the progress value specified
			* $("#progress").ejProgressBar({ value: 70 });
			* &lt;/script&gt;
			* @memberof ejProgressBar
			* @instance
			*/
            value: 0,
            /**
			* Sets the ProgressBar value in percentage. The value should be in between 0 to 100.
			* @default 0
			* @type {Number}
			* @example
			* &lt;div id="progress"&gt;&lt;/div&gt;
			* &lt;script&gt;
			* //Initialize the ProgressBar with the progress value specified in percent
			* $("#progress").ejProgressBar({ percentage : 35 });
			* &lt;/script&gt;
			* @memberof ejProgressBar
			* @instance
			*/
            percentage: 0,
            /**
			* Defines the height of the ProgressBar.
			* @default null
			* @type {Number/String}
			* @example 
			* &lt;div id="progress"&gt;&lt;/div&gt;
			* &lt;script&gt;
			* //Initialize the ProgressBar with the height value specified
			* $("#progress").ejProgressBar({ height: 20,value: 50 });
			* &lt;/script&gt;
			* @memberof ejProgressBar
			* @instance
			*/
            height: null,
            /**
			* Defines the width of the ProgressBar.
			* @default null
			* @type {Number/String}
			* @example 
			* &lt;div id="progress"&gt;&lt;/div&gt;
			* &lt;script&gt;
			* //Initialize the ProgressBar with the width value specified
			* $("#progress").ejProgressBar({ width: 200,value: 50});
			* &lt;/script&gt;
			* @memberof ejProgressBar
			* @instance
			*/
            width: null,
            /**
			* When this property sets to false, it disables the ProgressBar control
			* @default true
			* @type {boolean}
			* @example
			* &lt;div id="progress"&gt;&lt;/div&gt;
			* &lt;script&gt;
			* //Initialize the ProgressBar with the enabled value specified
			* $("#progress").ejProgressBar({ enabled: true,value: 50 });
			* &lt;/script&gt;
			* @memberof ejProgressBar
			* @instance
			*/
            enabled: true,
            /**
			* Sets the ProgressBar direction as right to left alignment.
			* @default false
			* @type {boolean}
			* @example
			* &lt;div id="progress"&gt;&lt;/div&gt;
			* &lt;script&gt;
			* //Initialize the Progress bar with the rtl value specified
			* $("#progress").ejProgressBar({ enableRTL: true,value: 50 });
			* &lt;/script&gt;
			* @memberof ejProgressBar
			* @instance
			*/
            enableRTL: false,
            /**
			* Save current model value to browser cookies for state maintains. While refresh the progressBar control page retains the model value apply from browser cookies
			* @default false
			* @type {boolean}
			* @example 
			* &lt;div id="progress"&gt;&lt;/div&gt;
			* &lt;script&gt;
			* //Initialize the Progress bar with the persist value specified
			* $("#progress").ejProgressBar({ enablePersistence: true,value: 50 });
			* &lt;/script&gt;
			* @memberof ejProgressBar
			* @instance
			*/
            enablePersistence: false,
            /**
			 * Event triggers when the process starts (from 0%)
			 * @event
			 * @name ejProgressBar#start
			 * @param {Object} argument Event parameters from progressbar   
			 * @param {boolean}  argument.cancel returns the cancel option value
			 * @param {object}  argument.model returns the ProgressBar model
			 * @param {object}  argument.percentage returns the current progress percentage
			 * @param {string}  argument.type returns the name of the event
			 * @param {string}  argument returns the current progress value
			 * @example
			 * &lt;div id="progress"&gt;&lt;/div&gt;
			 * &lt;script&gt;
			 * //initial start event
			 *$("#progress").ejProgressBar({
			 * value: 50,
			 *	start: function(args) {}
			 *});
             * &lt;/script&gt;
			 * @memberof ejProgressBar
			 * @instance
			 */
            start: null,
            /**     
			 * Event triggers when the process completes (at 100%)
			 * @event
			 * @name ejProgressBar#complete
			 * @param {Object} argument Event parameters from progressbar   
			 * @param {boolean}  argument.cancel returns the cancel option value
			 * @param {object}  argument.model returns the ProgressBar model
			 * @param {object}  argument.percentage returns the current progress percentage
			 * @param {string}  argument.type returns the name of the event
			 * @param {string}  argument returns the current progress value
			 * @example
			 * &lt;div id="progress"&gt;&lt;/div&gt;
			 * &lt;script&gt;
			 * //initial complete event
			 *$("#progress").ejProgressBar({
			 *	value: 50,
			 *	complete: function(args) {}
			 *});
             * &lt;/script&gt;
			 * @memberof ejProgressBar
			 * @instance
			 */
            complete: null,
            /**
			 * Event triggers when the progress value changed
			 * @event
			 * @name ejProgressBar#change
			 * @param {Object} argument Event parameters from progressbar   
			 * @param {boolean}  argument.cancel returns the cancel option value
			 * @param {object}  argument.model returns the ProgressBar model
			 * @param {object}  argument.percentage returns the current progress percentage
			 * @param {string}  argument.type returns the name of the event
			 * @param {string}  argument returns the current progress value
			 * @example
			 * &lt;div id="progress"&gt;&lt;/div&gt;
			 * &lt;script&gt;
			 * //initial complete event
			 *$("#progress").ejProgressBar({
			 *	value: 50,
			 *	change: function(args) {}
			 *});
             * &lt;/script&gt;
			 * @memberof ejProgressBar
			 * @instance
			 */
            change: null,
            /**     
            * Event triggers when the progressbar are created
            * @event
            * @name ejProgressBar#create 	
            * @param {Object} argument Event parameters from progressbar     
            * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
            * @param {object}  argument.model returns the progressbar model
            * @param {string}  argument.type returns the name of the event
            * @example 
            * &lt;div id="progress"&gt;&lt;/div&gt; <br> 
            * &lt;script&gt;
            * //create event for progressbar
             *$("#progress").ejProgressBar({
			 *	value: 50,
			 *	create: function(args) {}
			 *});
             * &lt;/script&gt;
			 * @memberof ejProgressBar
			 * @instance
			 */
            create: null,
            /**     
           * Event triggers when the progressbar are destroyed
           * @event
           * @name ejProgressBar#destroy 	
           * @param {Object} argument Event parameters from progressbar     
           * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
           * @param {object}  argument.model returns the progressbar model
           * @param {string}  argument.type returns the name of the event
           * @example 
           * &lt;div id="progress"&gt;&lt;/div&gt; <br> 
           * &lt;script&gt;
           * //destroy event for progressbar
           *$("#progress").ejProgressBar({
			 *	value: 50,
			 *	destroy: function(args) {}
			 *});
             * &lt;/script&gt;
			 * @memberof ejProgressBar
			 * @instance
			 */
            destroy: null

        },
        /**
         * Specify the data types for default properties 
		 * @private
         */

        dataTypes: {
            cssClass: "string",
            minValue: "number",
            maxValue: "number",
            enabled: "boolean",
            enableRTL: "boolean"
        },
        /**
        * Sets the Progress bar value. The value should be range between min and max values.
        * @private
        */

        _setValue: function (value) {
            /// <summary>Sets the Progress bar value. The value should be range between min and max values.</summary>
            if (value == null) value = this.model.minValue;
            else if (typeof value === "string") value = parseFloat(value);

            if (this._isNumber(value)) this.model.value = value;
            else if (!this._isNumber(this.model.value)) this.model.value = this.model.minValue;

            this.model.value = this._validateRange(this.model.value, this.model.minValue, this.model.maxValue);
            this._setProgressValue();
        },
        /**
         * Sets the Progress bar value in percentage. The value should be range between 0 to 100.
		 * @private
         */
        _setPercent: function (percent) {
            /// <summary>Sets the Progress bar value in percentage. The value should be range between 0 to 100.</summary>
            this.initial = this.model.percentage;
            if (percent == null) percent = 0;
            else if (typeof percent === "string") percent = parseFloat(percent);

            if (this._isNumber(percent)) this.model.percentage = percent;
            else if (!this._isNumber(this.model.percentage)) this.model.percentage = 0;

            this.model.percentage = this._validateRange(this.model.percentage, 0, 100);
            this.model.value = this._percentToValue(this.model.percentage);
            this._increaseProgressWidth();
        },
        /**
         * Configure to check whether the minvalue and max value are valid
		 * @private
         */
        _validateMinMax: function () {
            if (isNaN(this.model.minValue)) this.model.minValue = 0;
            if (isNaN(this.model.maxValue)) this.model.maxValue = 100;
        },
        /**
         * Sets the Progress bar text
		 * @private
         */
        _setText: function (text) {
            /// <summary>Sets the Progress bar value in percentage. The value should be range between 0 to 100.</summary>
            if (text) {
                if (this.text) this.text.html(text);
                else {
                    this.text = ej.buildTag("div.e-progress-txt", text);
                    this.element.append(this.text);
                    this._setTop();
                }
            }
            else if (this.text) {
                this.text.remove();
                this.text = null;
            }
        },
        /**
         * To configure the custom theme for progressbar using cssClass property		
		 * @private
         */
        _changeSkin: function (skin) {
            if (this.model.cssClass != skin) {
                this.element.removeClass(this.model.cssClass).addClass(skin);
            }
        },
        /**
		* Enables the progressbar control
		* @alias enable
		* @return jQuery
		*@example
		*&lt;div id="progress"&gt;&lt;/div&gt;
		*&lt;script&gt;
        * $("#progress").ejProgressBar({value: 50});
        		* // Create Progressbar instance
		* var proObj = $("#progress").data("ejProgressBar");
		* proObj.enable(); // enables the Progressbar control
		* &lt;/script&gt;
        @example
        		*&lt;div id="progress"&gt;&lt;/div&gt;
		*&lt;script&gt;
        * $("#progress").ejProgressBar({value: 50});
        // To enables the Progressbar control
		*$("#progress").ejProgressBar("enable");
		*&lt;/script&gt;
		* @memberof ejProgressBar
		* @instance
         */

        enable: function () {
            /// <summary>Enables the Progress bar control.</summary>
            if (!this.model.enabled) {
                this.element.removeClass("e-disable");
                this.model.enabled = true;
            }
        },
        /**
		* Disables the progressbar control
		* @alias disable
		* @return jQuery
		*@example
		*&lt;div id="progress"&gt;&lt;/div&gt;
		*&lt;script&gt;
        * $("#progress").ejProgressBar({value: 50});
        * // Create Progressbar instance
		* var proObj = $("#progress").data("ejProgressBar");
		* proObj.disable(); //disables the Progressbar control
		* &lt;/script&gt;
        @example
        		*&lt;div id="progress"&gt;&lt;/div&gt;
		*&lt;script&gt;
        * $("#progress").ejProgressBar({value: 50});
        //To disables the Progressbar control
		*$("#progress").ejProgressBar("disable");
		*&lt;/script&gt;
		* @memberof ejProgressBar
		* @instance
         */
        disable: function () {
            /// <summary>Disables the Progress bar control.</summary>
            if (this.model.enabled) {
                this.element.addClass("e-disable");
                this.model.enabled = false;
            }
        },
        /**
		* Returns the current progress value
		* @alias getValue
		* @return jQuery
		*@example
		*&lt;div id="progress"&gt;&lt;/div&gt;
		*&lt;script&gt;
        * $("#progress").ejProgressBar({value: 50});
        * // Create Progressbar instance
		* var proObj = $("#progress").data("ejProgressBar");
		* proObj.getValue(); // get the value of Progressbar control
		* &lt;/script&gt;
        @example
        *&lt;div id="progress"&gt;&lt;/div&gt;
		*&lt;script&gt;
        * $("#progress").ejProgressBar({value: 50});
        // To get the value of Progressbar control 
		*$("#progress").ejProgressBar("getValue");
		*&lt;/script&gt;
		* @memberof ejProgressBar
		* @instance
         */
        getValue: function () {
            /// <summary>Returns the Progress bar value.</summary>
            return this.model.value;
        },
        /**
		* Returns the current progress value in percent.
		* @alias getPercent
		* @return jQuery
		*@example
		*&lt;div id="progress"&gt;&lt;/div&gt;
		*&lt;script&gt;
        * $("#progress").ejProgressBar({value: 50});
        * // Create Progressbar instance
		* var proObj = $("#progress").data("ejProgressBar");
		* proObj.getPercentage(); // get the percent of Progressbar control
		* &lt;/script&gt;
        @example
        *&lt;div id="progress"&gt;&lt;/div&gt;
		*&lt;script&gt;
        * $("#progress").ejProgressBar({value: 50});
        // To get the percent of Progressbar control
		*$("#progress").ejProgressBar("getPercentage");
		*&lt;/script&gt;
		* @memberof ejProgressBar
		* @instance
         */
        getPercentage: function () {
            /// <summary>Returns the Progress bar value in percentage.</summary>
            return this.model.percentage;
        },
        /**
         * Create the Progressbar widget
		 * @private
         */
        // constructor function
        _init: function () {
            this._initialize();
            this._render();
        },
        /**
         * To configure the properties at runtime using SetModel		
		 * @private
         */
        _setModel: function (options) {
            /// <summary>Sets the model values to the Progress bar object</summary>
            if (!this.model.enabled && ej.isNullOrUndefined(options["enabled"])) return false;
            var option;
            for (option in options) {
                switch (option) {
                    case "value":
                        this._setValue(options[option]);
                        options[option] = this.model.value; break;
                    case "percentage":
                        this._setPercent(options[option]);
                        options[option] = this.model.percentage; break;
                    case "minValue":
                        if (!isNaN(options[option])) this._minValidation(options[option]);
                        options[option] = this.model.minValue;
                    case "maxValue":
                        if (!isNaN(options[option])) this._maxValidation(options[option]);
                        options[option] = this.model.maxValue;
                    case "text": this._setText(options[option]); break;
                    case "height": this._setHeight(options[option]); if (this.text) this._setTop(); break;
                    case "width": this._setWidth(options[option]); break;
                    case "enabled": this._disabled(!options[option]); break;
                    case "cssClass": this._changeSkin(options[option]); break;
                    case "enableRTL": this._rtl(options[option]); break;
                }
            }
        },
        /**
		* Destroy the progressbar widget
		* @alias destroy
		* @return jQuery
		*@example
		*&lt;div id="progress"&gt;&lt;/div&gt;
		*&lt;script&gt;
        * $("#progress").ejProgressBar({value: 50});
        * // Create Progressbar instance
		* var proObj = $("#progress").data("ejProgressBar");
		* proObj.destroy(); //destroy the Progressbar control
		* &lt;/script&gt;
        @example
        *&lt;div id="progress"&gt;&lt;/div&gt;
		*&lt;script&gt;
        * $("#progress").ejProgressBar({value: 50});
        // To destroy the Progressbar control
		*$("#progress").ejProgressBar("destroy");
		*&lt;/script&gt;
		* @memberof ejProgressBar
		* @instance
         */
        _destroy: function () {
            /// <summary>This function is  used to destroy the Progress bar Object</summary>
            this.element.empty();
            this.element.removeClass("e-widget " + this.model.cssClass);
        },
        /**
         * Initialization Section For Progress bar control		
		 * @private
         */

        _initialize: function () {
            this.text = null;
            this.header = null;
            this._preVal = null;
        },
        /**
         * Render Section For Progress bar control		
		 * @private
         */
        // renders the Progress bar control
        _render: function () {
            this.element.addClass("e-widget " + this.model.cssClass).attr("role", "ProgressBar");
            this._setDimention();

            this.header = ej.buildTag("div.e-progress");
            this.element.append(this.header);
            this._setText(this.model.text);

            this._setInitialValue();
            this._checkProperties();
        },
        /**
         * Configure dimensions For Progress bar control		
		 * @private
         */
        _setDimention: function () {
            this._setHeight(this.model.height);
            this._setWidth(this.model.width);
        },
        /**
         * Configure to set height For Progress bar control		
		 * @private
         */
        _setHeight: function (height) {
            if (height) this.element.height(height);
        },
        /**
         * Configure to set width For Progress bar control		
		 * @private
         */
        _setWidth: function (width) {
            if (width) this.element.width(width);
        },
        /**
         * Configure to set the value or percentage For Progress bar control initially		
		 * @private
         */
        // initially sets the value or percentage to Progress bar
        _setInitialValue: function () {
            this._validateMinMax();
            if (this.model.percentage) this._setPercent(this.model.percentage);
            else this._setValue(this.model.value);
        },
        /**
         * Configure to disable the progressbar control		
		 * @private
         */
        _disabled: function (boolean) {
            if (boolean) this.disable();
            else this.enable();
        },
        /**
         * Configure to check the properties the progressbar control		
		 * @private
         */
        _checkProperties: function () {
            if (this.model.enableRTL) this._rtl(this.model.enableRTL);
            this._minValidation(this.model.minValue);
            this._maxValidation(this.model.maxValue);
            if (!this.model.enabled) this._disabled(true);
        },
        /**
         * Configure enableRTL for the progressbar control		
		 * @private
         */
        _rtl: function (boolean) {
            if (boolean) this.element.addClass("e-rtl");
            else this.element.removeClass("e-rtl");
        },
        /**
         * Configure to valiate minvalue value of the progressbar control		
		 * @private
         */
        _minValidation: function (minvalue) {
            if (this.model.maxValue && this.model.maxValue < minvalue) this.model.maxValue = minvalue;
            if (this.model.value < minvalue)
                this.model.value = minvalue;
            this.model.minValue = minvalue;
            this._setProgressValue();
        },
        /**
         * Configure to valiate max value of the progressbar control		
		 * @private
         */
        _maxValidation: function (maxValue) {
            if (this.model.minValue && this.model.minValue > maxValue) this.model.minValue = maxValue;
            if (this.model.value > maxValue)
                this.model.value = maxValue;
            this.model.maxValue = maxValue;
            this._setProgressValue();
        },
        /**
         * Configure position of the progressbar control inner text		
		 * @private
         */

        // for the center alignment of the inner text
        _setTop: function () {
            var top = (this.element.height() - this.text.height()) / 2;
            this.text.css("top", Math.floor(top));
        },
        /**
         * Configure to change value of the progressbar control		
		 * @private
         */
        // for change the progress bar value
        _increaseProgressWidth: function () {
            this.header.css("width", this.model.percentage + "%");

            if (this.initial == 0 && this.model.percentage != this.initial)
                this._raiseEvent("start");

            if (this._preVal != this.model.value) {
                this._preVal = this.model.value;
                $(this.header).attr("aria-label", this.model.percentage);
                this._raiseEvent("change");

                if (this.model.percentage == 100)
                    this._raiseEvent("complete");
            }
        },
        /**
         * Configure to raise events of the progressbar control		
		 * @private
         */
        _raiseEvent: function (event) {
            this._trigger(event, { value: this.model.value, percentage: this.model.percentage });
        },
        /**
         * Configure to set value of the progressbar control		
		 * @private
         */
        _setProgressValue: function () {
            this.initial = this.model.percentage;
            this.model.percentage = this._valueToPercent(this.model.value);
            this._increaseProgressWidth();
        },
        /**
         * Configure to validate type of value in the progressbar control		
		 * @private
         */

        _isNumber: function (number) {
            return typeof number === "number" && !isNaN(number);
        },
        /**
         * Configure to validate value of the progressbar control between minvalue and maxValue		
		 * @private
         */
        // check whether the value in the minvalue, maxValue range
        _validateRange: function (value, minvalue, maxValue) {
            if (value < minvalue) return minvalue;
            else if (value > maxValue) return maxValue;
            return value;
        },
        /**
         * Configure to convert the value to percentage in the progressbar control		
		 * @private
         */
        // converts the value into percentage
        _valueToPercent: function (value) {
            if (this.model.maxValue <= this.model.minValue) return 100;
            value = this._validateRange(value, this.model.minValue, this.model.maxValue);
            value = (100 * (value - this.model.minValue)) / (this.model.maxValue - this.model.minValue);
            return value;
        },
        /**
         * Configure to convert the percentage to value in the progressbar control		
		 * @private
         */
        // converts the percentage into value
        _percentToValue: function (percent) {
            if (this.model.maxValue <= this.model.minValue) { return this.model.minValue; }

            if (percent >= 0 && percent <= 100) {
                var diff = this.model.maxValue - this.model.minValue;
                var val = diff * percent / 100;
                percent = val + this.model.minValue;
            }
            else if (percent < 0) percent = this.model.minValue;
            else if (percent > 100) percent = this.model.maxValue;
            return percent;
        }
    });

})(jQuery, Syncfusion);;
/**
* @fileOverview Plugin to style the Html Button elements
* @copyright Copyright Syncfusion Inc. 2001 - 2013. All rights reserved.
*  Use of this code is subject to the terms of our license.
*  A copy of the current license can be obtained at any time by e-mailing
*  licensing@syncfusion.com. Any infringement will be prosecuted under
*  applicable laws. 
* @version 12.1 
* @author <a href="mailto:licensing@syncfusion.com">Syncfusion Inc</a>
*/

(function ($, ej, undefined) {
    /**
* @namespace ej
* @class ejRating
* @requires jQuery
* @requires jquery.easing.1.3.js
* @requires ej.core.js
* @requires ej.rating.js
* @classdesc Custom Design for Html Rating control.
* @example 
* &lt;input id="rating"&gt;&lt;/input&gt; <br> 
* &lt;script&gt;
* // Create Rating
* $('#rating').ejRating(); 
* &lt;/script&gt;
*/
    ej.widget("ejRating", "ej.Rating", {
        _rootCSS: "e-rating",
        // widget element will be automatically set in this
        element: null,

        // user defined model will be automatically set in this
        model: null,
        validTags: ["input"],
        _setFirst: false,

        // default model
        defaults: {
            /**		
* Allow to render the maximum number of Rating shape(star).
* @default 5
* @type {number}
* @example
 *&lt;/br&gt;
*&lt;/br&gt;
* &lt;input id="rating"&gt;&lt;/input&gt; <br> 
* &lt;script&gt;
* //To set maxValue API value during initialization  
* 	$("#rating").ejRating({maxValue: 10 });
* &lt;/script&gt;
 * @memberof ejRating
* @instance
*/
            maxValue: 5,
            /**
* Allow to render the minimum number of Rating shape(star).
* @default 0
* @type {number}
* @example 
 *&lt;/br&gt;
*&lt;/br&gt;
* &lt;input id="rating"&gt;&lt;/input&gt; <br> 
* &lt;script&gt;
* //To set minValue API value during initialization  
* 	$("#rating").ejRating({minValue: 3 });
* &lt;/script&gt;
* @memberof ejRating
* @instance
*/
            minValue: 0,
            /**
* To specify the number of stars to be selected while rendering.
* @default 1
* @type {number}
* @example 
 *&lt;/br&gt;
*&lt;/br&gt;
* &lt;input id="rating"&gt;&lt;/input&gt; <br> 
* &lt;script&gt;
* //To set value API value during initialization  
* 	$("#rating").ejRating({value: 3 });
* &lt;/script&gt;
* @memberof ejRating
* @instance
*/
            value: 1,
            /**		
* Enables the rating control with reset button.It can be used to reset the rating control value.
* @default true
* @type {boolean}
* @example 
* &lt;input id="rating"&gt;&lt;/input&gt; <br> 
* &lt;script&gt;
* //To set allowReset API value during initialization  
* 	$("#rating").ejRating({allowReset: true});
* &lt;/script&gt;
* @memberof ejRating
* @instance
*/
            allowReset: true,
            /**		
* To specify the width of each shape in Rating control.
* @default 23
* @type {number}
* @example 
* &lt;input id="rating"&gt;&lt;/input&gt; <br> 
* &lt;script&gt;
* //To set shapeWidth API value during initialization  
* 	$("#rating").ejRating({shapeWidth: 25 });
* &lt;/script&gt;
* @memberof ejRating
* @instance
*/
            shapeWidth: 23,
            /**
* To specify the height of each shape in Rating control.
* @default 23
* @type {number}
* @example 
* &lt;input id="rating"&gt;&lt;/input&gt; <br> 
* &lt;script&gt;
* //To set shapeHeight API value during initialization  
* 	$("#rating").ejRating({shapeHeight: 25 });
* &lt;/script&gt;
* @memberof ejRating
* @instance
*/
            shapeHeight: 23,
            /**
* Specifies the orientation of Rating control.    See {@link Orientation}
* @default ej.Rating.Orientation.Horizontal
* @type {enum}
* @example 
* &lt;input id="rating"&gt;&lt;/input&gt; <br> 
* &lt;script&gt;
* //To set orientation API value during initialization  
* 	$("#rating").ejRating({orientation: ej.Rating.Orientation.Horizontal });
* &lt;/script&gt; 
* @memberof ejRating
* @instance
*/
            orientation: "horizontal",
            /**		
* Specifies the value to be increased while navigating between shapes(stars) in Rating control.
* @default 1
* @type {number}
* @example
 *&lt;/br&gt;
*&lt;/br&gt;
* &lt;input id="rating"&gt;&lt;/input&gt; <br> 
* &lt;script&gt;
* //To set incrementStep API value during initialization  
* 	$("#rating").ejRating({incrementStep: 2 });
* &lt;/script&gt;
* @memberof ejRating
* @instance
*/
            incrementStep: 1,
            /**		
* Interaction with Rating control can be prevented by enabling this API.
* @default false
* @type {boolean}
* @example 
* &lt;input id="rating"&gt;&lt;/input&gt; <br> 
* &lt;script&gt;
* //To set readOnly API value during initialization  
* 	$("#rating").ejRating({readOnly: true});
* &lt;/script&gt;
* @memberof ejRating
* @instance
*/
            readOnly: false,
            /**
* When this property is set to false, it disables the rating control.
* @default true
* @type {boolean}
* @example 
* &lt;input id="rating"&gt;&lt;/input&gt; <br> 
* &lt;script&gt;
* //To set enabled API value during initialization  
* 	$("#rating").ejRating({enabled: true });
* &lt;/script&gt;
* @memberof ejRating
* @instance
*/
            enabled: true,
            /**
* Enables the tooltip option.Currently selected value value will be displayed in tooltip.
* @default true
* @type {boolean}
* @example 
*&lt;/br&gt;
*&lt;/br&gt;
* &lt;input id="rating"&gt;&lt;/input&gt; <br> 
* &lt;script&gt;
* //To set showTooltip API value during initialization  
* 	$("#rating").ejRating({showTooltip: true});
* &lt;/script&gt;
* @memberof ejRating
* @instance
*/
            showTooltip: true,
            /**		
* Helps to provide more precise ratings.Rating control supports three precision modes - full, half, and exact. See {@link Precision}
* @default "full"
* @type {enum}
* @example
 *&lt;/br&gt;
*&lt;/br&gt;
* &lt;input id="rating"&gt;&lt;/input&gt; <br> 
* &lt;script&gt;
* //To set precision API value during initialization  
* 	$("#rating").ejRating({precision: ej.Rating.Precision.Half});
* &lt;/script&gt;
* @memberof ejRating
* @instance
*/
            precision: "full",
            /**
* Specify the CSS class to rating to achieve custom theme.
* @default ""
* @type {string}
* @example 
* &lt;input id="rating"&gt;&lt;/input&gt; <br> 
* &lt;script&gt;
* // Set the cssClass during initialization.
* 	$("#rating").ejRating({  cssClass : "gradient-lime" });
* &lt;/script&gt;
 * @memberof ejRating
* @instance
*/
            cssClass: "",
            /**
* Specifies the width of the Rating control wrapper.
* @default null
* @type {string}
* @example
* &lt;input id="rating"&gt;&lt;/input&gt; <br> 
* &lt;script&gt;
* //To set width API value during initialization  
* 	$("#rating").ejRating({width: "200" });
* &lt;/script&gt;
* @memberof ejRating
* @instance
*/
            width: null,
            /**
* Specifies the height of the Rating control wrapper.
* @default null
* @type {string}
* @example
* &lt;input id="rating"&gt;&lt;/input&gt; <br> 
* &lt;script&gt;
* //To set height API value during initialization  
* 	$("#rating").ejRating({height: "50" });
* &lt;/script&gt;
* @memberof ejRating
* @instance
*/
            height: null,
            /**
* Save current model value to browser cookies for state maintanence. While refresh the page Rating control values are retained.  
* @default false
* @type {boolean}
* @example 
* &lt;input id="rating"&gt;&lt;/input&gt; <br> 
* &lt;script&gt;
* //To set persist API value during initialization  
* 	$("#rating").ejRating({enablePersistence: true });
* &lt;/script&gt;
* @memberof ejRating
* @instance
*/
            enablePersistence: false,
            /**     
 * Fires when Rating control is created.
 * @event
 * @name ejRating#create
 * @param {Object} argument Event parameters from rating     
 * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
 * @param {object}  argument.model returns the rating model
 * @param {string}  argument.type returns the name of the event.
 * @example 
 * &lt;input id="rating"&gt;&lt;/input&gt; <br> 
* &lt;script&gt;
 * //create event for rating
 * $("#rating").ejRating({
 *    create: function (args) {}
 * });
 * &lt;/script&gt;
 * @memberof ejRating
 * @instance
 */
            create:null,
            /**     
			 * Fires when Rating control is clicked successfully.
			 * @event
			 * @name ejRating#click 
             * @param {Object} argument Event parameters from rating  
			 * @param {string} argument.value returns the current value.     
			 * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
			 * @param {object}  argument.model returns the rating model
			 * @param {string}  argument.type returns the name of the event
			 * @param {object}  argument.event returns the mouse click event args values.
			 * @example 
             * &lt;input id="rating"&gt;&lt;/input&gt; <br> 
             * &lt;script&gt;
			 * //click event for button
             * $("#rating").ejRating({
             *    click: function (args) {}
             * });
             * &lt;/script&gt;
			 * @memberof ejRating
			 * @instance
			 */
            click: null,
            /**     
 * Fires when mouse hovered over the Rating control
 * @event
 * @name ejRating#mouseover 
 * @param {Object} argument Event parameters from rating  
 * @param {string} argument.value returns the current value.     
 * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
 * @param {object}  argument.model returns the rating model
 * @param {string}  argument.type returns the name of the event
 * @param {object}  argument.event returns the mouse click event args values.
 * @param {object}  argument.index returns the current index value.
 * @example 
 * &lt;input id="rating"&gt;&lt;/input&gt; <br> 
 * &lt;script&gt;
 * //mouseover event for button
 * $("#rating").ejRating({
 *    mouseover: function (args) {}
 * });
* &lt;/script&gt;
 * @memberof ejRating
 * @instance
 */
            mouseover: null,
            /**     
* Fires when mouse hover is removed from Rating control
* @event
* @name ejRating#mouseout 
* @param {Object} argument Event parameters from rating  
* @param {string} argument.value returns the current value.     
* @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
* @param {object}  argument.model returns the rating model
* @param {string}  argument.type returns the name of the event
* @param {object}  argument.event returns the mouse click event args values.
* @example 
* &lt;input id="rating"&gt;&lt;/input&gt; <br> 
 * &lt;script&gt;
* //mouseover event for button
* $("#rating").ejRating({
*    mouseout: function (args) {}
* });
* &lt;/script&gt;
* @memberof ejRating
* @instance
*/
            mouseout: null,
            /**     
* Fires when Rating value changes.
* @event
* @name ejRating#change 
* @param {Object} argument Event parameters from rating  
* @param {string} argument.value returns the current value.     
* @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
* @param {object}  argument.model returns the rating model
* @param {string}  argument.type returns the name of the event
* @param {object}  argument.event returns the mouse click event args values.
* @example 
* &lt;input id="rating"&gt;&lt;/input&gt; <br> 
 * &lt;script&gt;
* //mouseover event for button
* $("#rating").ejRating({
*    change: function (args) {}
* });
* &lt;/script&gt;
* @memberof ejRating
* @instance
*/
            change: null,
            /**     
 * Fires when Rating control is destroyed successfully.
 * @event
 * @name ejRating#destroy 
 * @param {Object} argument Event parameters from rating      
 * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
 * @param {object}  argument.model returns the rating model
 * @param {string}  argument.type returns the name of the event
 * @example 
  * &lt;input id="rating"&gt;&lt;/input&gt; <br> 
 * &lt;script&gt;
 * //destroy event for rating
 * $("#rating").ejRating({
 *    destroy: function (args) {}
 * });
* &lt;/script&gt;
 * @memberof ejRating
 * @instance
 */
            destroy:null
        },
        /**
* Specify the data types for default properties 
* @private
*/
        dataTypes: {
            maxValue: "number",
            minValue: "number",
            allowReset: "boolean",
            shapeWidth: "number",
            shapeHeight: "number",
            orientation: "enum",
            incrementStep: "number",
            readOnly: "boolean",
            precision: "enum",
            enabled: "boolean"
        },

        observables: ["value"],
        value: ej.util.valueFunction("value"),
        /**
* To configure the properties at runtime using SetModel		
* @private
*/
        _setModel: function (options) {
            if (!this.model.enabled && ej.isNullOrUndefined(options["enabled"])) return false;
            for (var key in options) {
                switch (key) {
                    case "allowReset": {
                        if (options[key]) {
                            this._showResetButton();
                        }
                        else {
                            this._hideResetButton();
                        }
                        break;
                    }
                    case "value": {
                        this.setValue(ej.util.getVal(options[key]));
                        break;
                    }
                    case "enabled": this._enabledAction(options[key]); break;
                    case "cssClass": this._changeSkin(options[key]); break;
                    case "height": this._mainDiv.height(options[key]); break;
                    case "width": this._mainDiv.width(options[key]); break;
                    case "readOnly": {
                        this._unWireEvents();
                        break;
                    }
                    case "orientation":
                        {
                            this.model.orientation = options[key];
                            this.refresh();
                            break;
                        }
                    case "maxValue":
                        {
                            this.model.maxValue = options[key];
                            this.refresh();
                            break;
                        }
                    case "minValue":
                        {
                            this.model.minValue = options[key];
                            this.refresh();
                            break;
                        }
                    case "incrementStep":
                        {
                            this.model.incrementStep = options[key];
                            this.refresh();
                            break;
                        }
                    case "shapeWidth":
                        {
                            this.model.shapeWidth = options[key];
                            this.refresh();
                            break;
                        }
                    case "shapeHeight":
                        {
                            this.model.shapeHeight = options[key];
                            this.refresh();
                            break;
                        }
                }
            }
        },
        /**
* destroy the Rating widget
* all events bound using this._on will be unbind automatically and bring the control to pre-init state.
* @alias destroy
* @return jQuery
* @example 
* &lt;rating id="rating"&gt;Rating&lt;/rating&gt; <br> 
* &lt;script&gt;
* $("#rating").ejRating();
* // Create Rating
* var ratingObj = $("#rating").data("ejRating");
* ratingObj.destroy(); // destroy the button
* &lt;/script&gt;
* @example 
* &lt;rating id="rating"&gt;Rating&lt;/rating&gt; <br> 
* &lt;script&gt;
* $("#rating").ejRating();
* // destroy the rating
* $("#rating").ejRating("destroy");
* &lt;/script&gt;
* @memberof ejRating
* @instance
*/

        _destroy: function () {
            this.element.show();
            this._unWireEvents();
            this._mainDiv.remove();
        },

        // constructor function
        /**
 * Create the butto widget
 * @private
 */
        _init: function () {
            this._initialize();
        },
        /**
* To initialize the button		
* @private
*/
        _initialize: function () {
            this.element.hide();
            this._mainDiv = ej.buildTag("div.e-rating e-widget " + this.model.cssClass, "", {}, { tabindex: "0", role: "group", "aria-label": "Rating" });
            if (!ej.isNullOrUndefined(this.model.width))
                this._mainDiv.width(this.model.width);
            if (!ej.isNullOrUndefined(this.model.height))
                this._mainDiv.height(this.model.height);
            this._mainDiv.insertBefore(this.element);
            if (this.model.orientation == ej.Orientation.Horizontal) {
                this._mainDiv.addClass("e-horizontal");
            }
            else
                this._mainDiv.addClass("e-vertical");

            if (this.model.allowReset && !this.model.readOnly) {
                this._createReset();
            }
            this._validation();
            this._renderShape();
            this._shapes = this._mainDiv.find("div.e-shape");
            this._wireEvents();
            this._CurrentIndex = 0;
            this._initCurrentValue();
            this._enabledAction(this.model.enabled);
        },
        /**
        * To change the skin at runtime		
        * @private
        */
        _changeSkin: function (skin) {
            if (this.model.cssClass != skin) {
                this._mainDiv.removeClass(this.model.cssClass).addClass(skin);
            }
        },
        /**
* To enable or disable the Rating widget	
* @private
*/
        _enabledAction: function (flag) {
            if (flag) {
                this._mainDiv.removeClass("e-disable");
            }
            else {
                this._mainDiv.addClass("e-disable");
            }
        },
        /**
* To validate the user provided incrementStep value	
* @private
*/
        _validation: function () {
            if (this.model.incrementStep < 1) {
                this.model.incrementStep = 1;
            }
            else if (this.model.incrementStep > this.model.maxValue) {
                this.model.incrementStep = this.model.maxValue;
            }
        },
        /**
* Creates reset buttton option in the Rating widget
* @private
*/
        _createReset: function () {
            if (this._mainDiv.find("div.e-reset").length <= 0)
                ej.buildTag("div.e-reset", "", { width: this.model.shapeWidth + "px", height: this.model.shapeHeight, title: "Reset" }, { role: "button", "aria-label": "reset", "aria-hidden": false }).appendTo(this._mainDiv);
            else
                this._mainDiv.find("div.e-reset").show();
        },
        /**
* To create desired shape to be displayed in the rating control
* @private
*/
        _renderShape: function () {
            var _shapeCount = Math.round(this.model.maxValue / this.model.incrementStep);
            var _startCount = Math.round(this.model.minValue / this.model.incrementStep);
            var containerWith, containerHeight;
            if (this.model.orientation == ej.Rating.Orientation.Horizontal) {
                containerWith = (_shapeCount + 1) * this.model.shapeWidth;
                containerHeight = this.model.shapeHeight;
            } else if (this.model.orientation == ej.Rating.Orientation.Vertical) {
                containerWith = this.model.shapeWidth;
                containerHeight = this.model.shapeHeight * _shapeCount;
            }
            if (_shapeCount > 0) {
                var _ulTag = ej.buildTag("ul.e-ul", "", { width: containerWith + "px", height: containerHeight }, {});
                for (var i = _startCount ; i < _shapeCount; i++) {
                    var _liTag = ej.buildTag("li.e-shape-list", "", { width: this.model.shapeWidth + "px", height: this.model.shapeHeight }, { tabindex: -1, "aria-describedby": this.element.prop("id") + "_tooltip" });
                    ej.buildTag("div.e-shape inactive", "", { width: this.model.shapeWidth + "px", height: this.model.shapeHeight }).appendTo(_liTag);
                    _liTag.appendTo(_ulTag);
                }
                _ulTag.appendTo(this._mainDiv);
            }
        },
        /**
* To load the current value when the Rating control is rendered
* @private
*/
        _initCurrentValue: function () {
            if (ej.isNullOrUndefined(this.value()) || this.value() == ""|| this.value() == 0)
                this._CurrentIndex = 0;
            else if (this.value() > this.model.minValue) {
                var count = (this.value() / this.model.incrementStep) - (this.model.minValue / this.model.incrementStep);
                this._CurrentIndex = count;
                this._valueRefresh(this._CurrentIndex);
                for (var i = 0; i < count; i++) {
                    if (this._shapes[i]) {
                        $(this._shapes[i]).removeClass('inactive').removeClass('active').addClass('selected');
                    }
                    else {
                        this._CurrentIndex = i;
                        break;
                    }
                }
            }
            this.value(this._CurrentIndex);
        },
        /**
* To refresh the rating on every mouse out with previously selected values
* @private
*/
        _valueRefresh: function (index) {
			index = Math.ceil(index);
            var value = (this.value() / this.model.incrementStep).toFixed(1);
            if (this.model.orientation == ej.Rating.Orientation.Horizontal) {
				var len;
				if(this.model.precision=="exact")
					len = (value % 1).toFixed(1);
				else if(this.model.precision=="half")
					len = (value%1>0.5) ? 1 : 0.5;
				else
					len = 1;
				value = ((value % 1).toFixed(1) != 0 ? len : (value / this._CurrentIndex)) * this.model.shapeWidth;
				for (var i = 0; i < index; i++) {
					$(this._shapes[i]).css({ width: this.model.shapeWidth + "px" });
				}
				if (this._shapes != null)
					$(this._shapes[index - 1]).css({ width: value + "px" });
            }
            else {
                value = ((value % 1).toFixed(1) != 0 ? (value % 1).toFixed(1) : (value / this._CurrentIndex)) * this.model.shapeHeight;
                for (var i = 0; i < index; i++) {
                    $(this._shapes[i]).css({ height: this.model.shapeHeight + "px" });
                }
                if (this._shapes != null) $(this._shapes[index - 1]).css({ height: value + "px" });
            }
        },

        /**
* To refresh the rating control values.
* @private
*/
        _reset: function () {
            var prevValue;
            if (!this.model.enabled) return false;
            $(this._shapes).removeClass('active').removeClass('selected').addClass('inactive');
            prevValue = this.value();
            this.value(0);
            this._CurrentIndex = 0;
            //Update Hidden field
            this.element.value = this._CurrValue;

            ////Fire Value Change event
            this._valueChanged(prevValue);
        },
        /**
* To fill exact precision on mouse move
* @private
*/
        _fillExactPrecision: function (element, position) {
            var index = this._shapes.index(element) + (this.model.minValue / this.model.incrementStep) + 1;
            var option = this.model;
            if (option.orientation == ej.Rating.Orientation.Horizontal) {
                for (var i = 0; i < this._shapes.length; i++) {
                    if (i < index)
                        $(this._shapes[i]).css({ width: option.shapeWidth + "px" });
                    else
                        $(this._shapes[i]).css({ width: "0px" });
                }
                //Set width and title
                $(element).css({ width: position + "px" });
                this.toolTipValue = ((index - 1) * option.incrementStep + (position / option.shapeWidth) * option.incrementStep).toFixed(1);
            }
            else {
                for (var i = 0; i < index; i++) {
                    $(this._shapes[i]).css({ height: option.shapeHeight + "px" });
                }
                for (var i = index; i < this._shapes.length; i++) {
                    $(this._shapes[i]).css({ height: "0px" });
                }
                //Set width and title
                $(element).css({ height: position + "px" });
                this.toolTipValue = ((index - 1) * option.incrementStep + (position / option.shapeHeight) * option.incrementStep).toFixed(1);
            }
        },

        /**
* To fill half precision on mouse move
* @private
*/
        _fillHalfPrecision: function (element, position) {
            var index = this._shapes.index(element) + (this.model.minValue / this.model.incrementStep) + 1;
            var options = this.model;
            if (options.orientation == ej.Rating.Orientation.Horizontal) {
                for (var i = 0; i < this._shapes.length; i++) {
                    if (i < index)
                        $(this._shapes[i]).css({ width: options.shapeWidth + "px" });
                    else
                        $(this._shapes[i]).css({ width: "0px" });
                }
                if ((position + 1) % 2 == 0) {
                    position = options.shapeWidth / 2;
                    $(element).css({ width: position + "px" });
                    this.toolTipValue = ((index - 1) * options.incrementStep + (position / options.shapeWidth) * options.incrementStep).toFixed(1);
                }
                else {
                    position = options.shapeWidth;
                    $(element).css({ width: position + "px" });
                    this.toolTipValue = ((index - 1) * options.incrementStep + (position / options.shapeWidth) * options.incrementStep).toFixed(1);
                }
            }
            else {
                for (var i = 0; i < this._shapes.length; i++) {
                    if (i < index)
                        $(this._shapes[i]).css({ height: this._starHeight + "px" });
                    else
                        $(this._shapes[i]).css({ height: "0px" });
                }
                //Set width and title
                if (position <= options.shapeHeight / 2) {
                    position = options.shapeHeight / 2;
                    $(element).css({ height: position + "px" });
                    this.toolTipValue = ((index - 1) * options.incrementStep + (position / options.shapeHeight) * options.incrementStep).toFixed(1);
                }
                else {
                    position = options.shapeHeight;
                    $(element).css({ height: position + "px" });
                    this.toolTipValue = ((index - 1) * options.incrementStep + (position / options.shapeHeight) * options.incrementStep).toFixed(1);
                }
            }
        },
        /**
         * Section For handle clcik action in rating control.
		 * @private
         */
        _ClickHandler: function (e) {
            var prevValue;
            //Check if stars and rate it
            if (this.model.enabled) {
                var element;
                if (e.target.tagName == "LI")
                    element = e.target.firstChild;
                else if (e.target.parentNode.tagName == "LI")

                    element = e.target;
                if (element) {
                    var index = this._shapes.index(element) + 1;
                    for (var i = 0; i < index; i++) {
                        if (this._shapes[i]) {
                            $(this._shapes[i]).removeClass('inactive').removeClass('active').addClass('selected');
                        }
                    }
                    this._CurrentIndex = index;
                    index = index + (this.model.minValue / this.model.incrementStep);
                    var _IncrementStep = this.model.incrementStep;
                    var _CurrValue;
                    if (this.model.orientation == ej.Rating.Orientation.Horizontal)
                        _CurrValue = ((index - 1) * _IncrementStep + (element.clientWidth / this.model.shapeWidth) * _IncrementStep).toFixed(1);
                    else
                        _CurrValue = ((index - 1) * _IncrementStep + (element.clientHeight / this.model.shapeHeight) * _IncrementStep).toFixed(1);

                    prevValue = this.value();
                    //Update Hidden field        
                    this.element.value = _CurrValue;
                    this.value(_CurrValue);

                    //Fire Click event
                    //Client side Click event event handler
                   
                    this._trigger("click", { event: e, value: this.value(), prevValue: prevValue });
                    ////Fire Value Change event
                    this._valueChanged(prevValue);
                }

                //Check if reset button and reset
                if ($(e.target).hasClass('e-reset')) {
                    this._reset();
                }
            }
        },
        /**
 * Section For handle mouse over action in rating control.
 * @private
 */
        _MouseOverHandler: function (e) {
            if (this.model.enabled) {
                var element, target = e.target;
                if (e.type == 'touchmove') {
                    var coor = e.originalEvent.changedTouches[0];
                    target = document.elementFromPoint(coor.pageX, coor.pageY);
                }
                if (target.tagName == "LI")
                    element = target.firstChild;
                else if (target.parentNode.tagName == "LI")
                    element = target;
                else
                    return false;

                var index = this._shapes.index(element) + 1;
                for (var i = 0; i <= this._shapes.length; i++) {
                    if (this._shapes[i]) {
                        if (i < index)
                            $(this._shapes[i]).removeClass('inactive').removeClass('selected').addClass('active');
                        else
                            $(this._shapes[i]).removeClass('active').removeClass('selected').addClass('inactive');
                    }
                }
                //Client side onMouseOut event handler
               
                    this._trigger("mouseover", { event: e, index: index, value: this.value() });
            }
        },
        /**
 * Section For handle mouse out action in rating control.
 * @private
 */
        _MouseOutHandler: function (e) {
            if (!this.model.readOnly && this.model.enabled) {
                var index = this._CurrentIndex;
                if (index != 0) {
                    for (var i = 0; i < this._shapes.length; i++) {
                        if (this._shapes[i]) {
                            if (i < index)
                                $(this._shapes[i]).removeClass('active').removeClass('inactive').addClass('selected');
                            else
                                $(this._shapes[i]).removeClass('active').addClass('inactive');
                        }
                    }
                } else
                    $(this._shapes).removeClass('active').addClass('inactive');
                //On mouse out, keep selected the stars w.r.t current Value
                if (this.model.precision !== ej.Rating.Precision.Full)
                    this._valueRefresh(index);
                //Client side onMouseOut event handler
                
                    this._trigger("mouseout", { event: e, value: this.value() });
            }
            this._hideTooltip(1000);
        },
        /**
 * Section For handle mouse move action in rating control.
 * @private
 */
        _MouseMoveHandler: function (e) {
            if (!this.model.readOnly && this.model.enabled) {
                var element, position, target = e.target;
                if (e.type == 'touchmove') {
                    var coor = e.originalEvent.changedTouches[0];
                    target = document.elementFromPoint(coor.pageX, coor.pageY);
                }
                if (target.tagName == "LI")
                    element = target.firstChild;
                else if (target.parentNode.tagName == "LI")
                    element = target;
                else
                    return false;
                if (this.model.orientation == ej.Rating.Orientation.Horizontal)
                    if (ej.isNullOrUndefined(e.offsetX))
                        position = e.pageX - $(target).offset().left;
                    else
                        position = e.offsetX + 1;
                else {
                    if (ej.isNullOrUndefined(e.offsetY))
                        position = e.pageY - $(target).offset().top;
                    else
                        position = e.offsetY + 1;
                }
                this.toolTipValue = (this.model.minValue + (this._shapes.index(element) + 1)) * this.model.incrementStep;
                if (this.model.precision == ej.Rating.Precision.Exact)
                    this._fillExactPrecision(element, position);
                else if (this.model.precision == ej.Rating.Precision.Half)
                    this._fillHalfPrecision(element, Math.round(position));
            } else {
                this.toolTipValue = (this.value() == null || this.value() == "" ? 0 : this.value()) + ' / ' + this.model.maxValue;
            }
            $(e.target).attr("aria-label", this.toolTipValue);
            this._showhideTooltip(e.target);
        },
        /**
* To enable tooltip option
* @private
*/
        _showTooltip: function (element) {
            if (this.model.showTooltip) {
                $('.e-tooltip').remove();
                this.tooltip = ej.buildTag("div#" + this.element.prop("id") + "_tooltip.e-tooltip " + this.model.cssClass + " e-corner-all", "", {}, { role: "tooltip" });
                $("body").append(this.tooltip);
                this._isTooltip = true;
                this._setTooltipPosition(element);
            }
        },
        /**
* To hide the tooltip.
* @private
*/
        _hideTooltip: function (animationSpeed) {
            if (this.model.showTooltip) {
                $(this.tooltip).fadeOut(animationSpeed, function () {
                    $('.e-tooltip').remove();
                });
                this._isTooltip = false;
            }
        },
        /**
* Validates tooltip option is enabled or not
* @private
*/
        _showhideTooltip: function (element) {
            if (this.model.showTooltip) {
                this._showTooltip(element);
            }
        },
        /**
* To set the tooltip position
* @private
*/
        _setTooltipPosition: function (element) {
            if (this.model.showTooltip) {
                this._updateTooltipValue();
                var top, left, remainLeft, remainTop, handle, pos, gap = 5; // gap -> distance between tooltip and slider
                handle = $(element), pos = this._getOffset(handle);

                if (this.model.orientation == "vertical") {
                    remainTop = (this.tooltip.outerHeight() - handle.outerHeight()) / 2;
                    remainLeft = handle.outerWidth() + gap;
                    top = pos.top - remainTop;
                    left = pos.left + remainLeft;
                }
                else {
                    remainLeft = (this.tooltip.outerWidth() - handle.outerWidth()) / 2;
                    remainTop = this.tooltip.outerHeight() + gap;
                    top = pos.top - remainTop;
                    left = pos.left - remainLeft;
                }
                this.tooltip.css({ "top": top, "left": left });
            }
        },
        _getOffset: function (ele) {
            var pos = ele.offset();
            if ($("body").css("position") != "static") {
                var bodyPos = $("body").offset();
                pos.left -= bodyPos.left;
                pos.top -= bodyPos.top;
            }
            return pos;
        },

        /**
* To update the tooltip value based on current value.
* @private
*/
        _updateTooltipValue: function () {
            //var val = this._getCurrentHandleValue();
            //if (this.model.range != "true")
            this.tooltip[0].innerHTML = this.toolTipValue;
            //else
            //    this.tooltip[0].innerHTML = val[0] + " - " + val[1];
        },

        //-------------------- Event Wire-up -------------------------//
        /**
* Wiring the events to Rating control		
* @private
*/
        _wireEvents: function () {
            //tabs Click evnts
            if (!this.model.readOnly) {
                this._on(this._mainDiv, "mousedown", this._ClickHandler);
                this._on(this._mainDiv.find("li"), "mouseenter touchmove", this._MouseOverHandler);
            }
            this._on(this._mainDiv.find("li"), "mouseleave touchend", this._MouseOutHandler);
            //if (this.model.precision !== ej.Rating.Precision.Full) {
            this._on(this._mainDiv.find("li"), ej.eventType.mouseMove, this._MouseMoveHandler);
            //}
        },
        //-------------------- Event UnWire-up -------------------------//
        /**
* To UnWire the binded events from Rating control		
* @private
*/
        _unWireEvents: function () {
            this._mainDiv.find("li").unbind("mouseenter touchmove");
            this._mainDiv.find("li").unbind("mouseleave touchend");
            this._mainDiv.unbind("mousedown");
            if (this.model.precision !== ej.Rating.Precision.Full) {
                this._mainDiv.find("li").unbind(ej.eventType.mouseMove);
            }
        },
        /**
* Section to specify value changed		
* @private
*/
        _valueChanged: function (previous) {
            //Client side onMouseOut event handler
           
            this._trigger("change", { value: this.value(), prevValue: previous });
        },

        // ------------------Client side methods-------------------------//
        /**
* To reset the rating value
* @return jQuery
* @example 
* &lt;input id="rating"&gt;&lt;/input&gt; <br> 
* &lt;script&gt;
* $("#rating").ejRating();
* // Reset Rating
* var ratingObj = $("#rating").data("ejRating");
* ratingObj.reset(); // reset the rating values
* &lt;/script&gt;
* @example 
* &lt;input id="rating"&gt;&lt;/input&gt; <br> 
* &lt;script&gt;
* $("#rating").ejRating();
* // reset the rating values
* $("#rating").ejRating("reset");
* &lt;/script&gt;
* @memberof ejRating
* @instance
 */
        reset: function () {
            /// <summary>To Reset the Rating control</summary>
            this._reset();
        },
        /**
* To show the rating control
* @return jQuery
* @example 
* &lt;input id="rating"&gt;&lt;/input&gt; <br> 
* &lt;script&gt;
* $("#rating").ejRating();
* // Show Rating
* var ratingObj = $("#rating").data("ejRating");
* ratingObj.show(); // shows the rating control
* &lt;/script&gt;
* @example 
* &lt;input id="rating"&gt;&lt;/input&gt; <br> 
* &lt;script&gt;
* $("#rating").ejRating();
* // show the rating control
* $("#rating").ejRating("show");
* &lt;/script&gt;
* @memberof ejRating
* @instance
*/
        show: function () {
            /// <summary>To Show the Rating control</summary>
            if (!this.model.enabled) return false;
            this._mainDiv.css("visibility", "visible");
        },
        /**
* To hide the rating control
* @return jQuery
* @example 
* &lt;input id="rating"&gt;&lt;/input&gt; <br> 
* &lt;script&gt;
* $("#rating").ejRating();
* // Hide Rating
* var ratingObj = $("#rating").data("ejRating");
* ratingObj.hide(); // hides the rating control
* &lt;/script&gt;
* @example 
* &lt;input id="rating"&gt;&lt;/input&gt; <br> 
* &lt;script&gt;
* $("#rating").ejRating();
* // hide the rating control
* $("#rating").ejRating("hide");
* &lt;/script&gt;
* @memberof ejRating
* @instance
*/
        hide: function () {
            /// <summary>To Hide the Rating control</summary>
            if (!this.model.enabled) return false;
            this._mainDiv.css("visibility", "hidden");
        },
  
        _showResetButton: function () {
            /// <summary>To Show the Reset Button</summary>
            if (!this.model.enabled) return false;
            this._createReset();
        },
      
        _hideResetButton: function () {
            /// <summary>To Hide the Reset Button</summary>
            if (!this.model.enabled) return false;
            this._mainDiv.find("div.e-reset").hide()
        },
        /**
* To get the current value of rating control
* @return Current Rating value
* @example 
* &lt;input id="rating"&gt;&lt;/input&gt; <br> 
* &lt;script&gt;
* $("#rating").ejRating();
* // Get current value of rating
* var ratingObj = $("#rating").data("ejRating");
* ratingObj.getValue(); // get the current value of rting
* &lt;/script&gt;
* @example 
* &lt;input id="rating"&gt;&lt;/input&gt; <br> 
* &lt;script&gt;
* $("#rating").ejRating();
* // To get the current value of rating
* $("#rating").ejRating("getValue");
* &lt;/script&gt;
* @memberof ejRating
* @instance
*/
        getValue: function () {
            /// <summary>To get current value of Rating control</summary>
            return this.value() == null || this.value() == "" ? "" : this.value();
        },
        /**
* To set the current value of rating control
* @return jQuery
* @example 
* &lt;input id="rating"&gt;&lt;/input&gt; <br> 
* &lt;script&gt;
* $("#rating").ejRating();
* // Set current value of rating
* var ratingObj = $("#rating").data("ejRating");
* ratingObj.setValue(); // set the current value of rating
* &lt;/script&gt;
* @example 
* &lt;input id="rating"&gt;&lt;/input&gt; <br> 
* &lt;script&gt;
* $("#rating").ejRating();
* // To set the current value of rating
* $("#rating").ejRating("setValue");
* &lt;/script&gt;
* @memberof ejRating
* @instance
*/
        setValue: function (value) {
            /// <summary>To set current value of Rating control</summary>
            if (!this.model.enabled) return false;
            if (value != null) {
                this.value(value);
                $(this._shapes).removeClass('selected').addClass('inactive');
                this._initCurrentValue();
            }
        },
        /**
* user can refresh the rating control to identify changes
* @return jQuery
* @example 
* &lt;input id="rating"&gt;&lt;/input&gt; <br> 
* &lt;script&gt;
* $("#rating").ejRating();
* // Refresh the rating control
* var ratingObj = $("#rating").data("ejRating");
* ratingObj.refresh(); // refreshes the rating control
* &lt;/script&gt;
* @example 
* &lt;input id="rating"&gt;&lt;/input&gt; <br> 
* &lt;script&gt;
* $("#rating").ejRating();
* // To refresh the rating values
* $("#rating").ejRating("refresh");
* &lt;/script&gt;
* @memberof ejRating
* @instance
*/
        refresh: function () {
            this._destroy();
            this._unWireEvents();
            this._initialize();
        }
    });
    /**
 * Enum for various rating precisions	 
 * @enum {string}
 * @global 
 */
    ej.Rating.Precision = {
        /**  Allows user select full shape of Rating. */
        Full: "full",
        /**  Allows user select half shape of Rating. */
        Half: "half",
        /**  Allows user select exact shape of Rating. */
        Exact: "exact"
    }
    /**
* Enum for various rating orientations	 
* @enum {string}
* @global 
*/
    ej.Rating.Orientation = {
        /**  Renders Rating control in horizontal direction. */
        Horizontal: "horizontal",
        /**  Renders Rating control in vertical direction. */
        Vertical: "vertical"
    }
})(jQuery, Syncfusion);;
/*!
*  filename: ej.rotator.js
*  version : 12.1
*  Copyright Syncfusion Inc. 2001 - 2013. All rights reserved.
*  Use of this code is subject to the terms of our license.
*  A copy of the current license can be obtained at any time by e-mailing
*  licensing@syncfusion.com. Any infringement will be prosecuted under
*  applicable laws. 
*/

/**
* @fileOverview Plugin to style the Rotator control.
* @copyright Copyright Syncfusion Inc. 2001 - 2013. All rights reserved.
*  Use of this code is subject to the terms of our license.
*  A copy of the current license can be obtained at any time by e-mailing
*  licensing@syncfusion.com. Any infringement will be prosecuted under
*  applicable laws. 
* @version 12.1 
* @author <a href="mailto:licensing@syncfusion.com">Syncfusion Inc</a>
*/
(function ($, ej, undefined) {
    /**
   * @namespace ej
   * @class ejRotator
   * @requires jQuery
   * @requires jquery.easing.1.3.min.js
   * @requires ej.core.js
   * @requires ej.data.js
   * @requires ej.rotator.js
   * @classdesc The Rotator control displays a set of slides. Each slide may contain images or images with content, or content with user-defined transition between them.
   * @example
   * &lt;ul id="sliderContent"&gt;
   * &lt;li&gt;&lt;img src="../images/rotator/nature.jpg" /&gt;&lt;/li&gt;
   * &lt;li&gt;&lt;img src="../images/rotator/bird.jpg"/&gt;&lt;/li&gt;
   * &lt;li&gt;&lt;img src="../images/rotator/sculpture.jpg"/&gt;&lt;/li&gt;
   * &lt;li&gt;&lt;img src="../images/rotator/seaview.jpg" /&gt;&lt;/li&gt;
   * &lt;li&gt;&lt;img src="../images/rotator/snowfall.jpg"/&gt;&lt;/li&gt;
   * &lt;/ul&gt;
   * &lt;script&gt;
   * //Initialize the Rotator control in the script as follows.
   * $(function (){
   * // document ready
   * // simple Rotator creation
   * $("#sliderContent").ejRotator();
   * });
   * &lt;/script&gt;
   */
    ej.widget("ejRotator", "ej.Rotator", {
        element: null,
        model: null,
        _setFirst: false,

        defaults: {
            /**		
			* Specify the CSS class to Rotator to achieve custom theme.
			* @default ""
			* @type {string}
			* @example 
			 * &lt;ul id="sliderContent"&gt;
   * &lt;li&gt;&lt;img src="../images/rotator/nature.jpg" /&gt;&lt;/li&gt;
   * &lt;li&gt;&lt;img src="../images/rotator/bird.jpg"/&gt;&lt;/li&gt;
   * &lt;li&gt;&lt;img src="../images/rotator/sculpture.jpg"/&gt;&lt;/li&gt;
   * &lt;li&gt;&lt;img src="../images/rotator/seaview.jpg" /&gt;&lt;/li&gt;
   * &lt;li&gt;&lt;img src="../images/rotator/snowfall.jpg"/&gt;&lt;/li&gt;
   *&lt;/ul&gt;
    * &lt;script&gt;
			* // Set the CSS class during initialization. 			
			* 	$("#sliderContent").ejRotator({  cssClass : "gradient-lime" });			
			* &lt;/script&gt;
			 * @memberof ejRotator
			* @instance
			*/
            cssClass: "",
            /**		
			* Specify the list of data which contains a set of data fields. Each data value is used to render an item for the Rotator. 
			* @default null
			* @type {object}
			* @example 
			* &lt;ul id="sliderContent"&gt;
            *&lt;/ul&gt;
            * &lt;script&gt;
			* // Set the dataSource during initialization. 			
			* 	$("#sliderContent").ejRotator({ dataSource:window.items });			
			* &lt;/script&gt;
			* @memberof ejRotator
			* @instance
			*/
            dataSource: null,
            /**		
			* Retrieves data from remote data. This property is applicable only when a remote data source is used.
			* @default null
			* @type {string}
			* @memberof ejRotator
			* @instance
			*/
            query: null,
            /**		
			* Defines mapping fields for the data items of the Rotator.
			* @default null
			* @type {object}
			* @example 
			* &lt;ul id="sliderContent"&gt;
            *&lt;/ul&gt;
            * &lt;script&gt;
			* // Set the fields during initialization. 			
			* 	$("#sliderContent").ejRotator({ dataSource:window.items, fields: {text:"text",url:"url"}});
			* &lt;/script&gt;
			* @memberof ejRotator
			* @instance
			*/
            fields: /** @lends ejRotator# */{
                /**		
                 * Specifies a caption for the image.
				 * @alias ejRotator#fields->text
				 * @type String
                 */
                text: "text",
                /**		
                 * Specifies the URL for an image.
				 * @alias ejRotator#fields->url
				 * @type String
                 */
                url: "url"
            },
            /**		
           * Enables or disables the Rotator control.
           * @default true
           * @type {boolean} 
           * @example 
            * &lt;ul id="sliderContent"&gt;
   * &lt;li&gt;&lt;img src="../images/rotator/nature.jpg" /&gt;&lt;/li&gt;
   * &lt;li&gt;&lt;img src="../images/rotator/bird.jpg"/&gt;&lt;/li&gt;
   * &lt;li&gt;&lt;img src="../images/rotator/sculpture.jpg"/&gt;&lt;/li&gt;
   * &lt;li&gt;&lt;img src="../images/rotator/seaview.jpg" /&gt;&lt;/li&gt;
   * &lt;li&gt;&lt;img src="../images/rotator/snowfall.jpg"/&gt;&lt;/li&gt;
   *&lt;/ul&gt;
    * &lt;script&gt;
           * // Set the enabled during initialization. 			
           * 	$("#sliderContent").ejRotator({  enabled : true});		 
           * &lt;/script&gt;
           * @memberof ejRotator
           * @instance
           */
            enabled: true,
            /**		
           * Specifies the number of Rotator Items to be displayed.
           * @default "1"
           * @type {string | number}
           * @example 
            * &lt;ul id="sliderContent"&gt;
   * &lt;li&gt;&lt;img src="../images/rotator/nature.jpg" /&gt;&lt;/li&gt;
   * &lt;li&gt;&lt;img src="../images/rotator/bird.jpg"/&gt;&lt;/li&gt;
   * &lt;li&gt;&lt;img src="../images/rotator/sculpture.jpg"/&gt;&lt;/li&gt;
   * &lt;li&gt;&lt;img src="../images/rotator/seaview.jpg" /&gt;&lt;/li&gt;
   * &lt;li&gt;&lt;img src="../images/rotator/snowfall.jpg"/&gt;&lt;/li&gt;
   *&lt;/ul&gt;
    * &lt;script&gt;
           * // Set the displayItemsCount  during initialization. 			
           * 	$("#sliderContent").ejRotator({ displayItemsCount  : "1"});			
          * &lt;/script&gt;
            * @memberof ejRotator
           * @instance
           */
            displayItemsCount: "1",
            /**		
			* Specifies the number of Rotator Items to navigate on a single click (next/previous/play buttons). The navigateSteps property value must be less than or equal to the displayItemsCount  property value.
			* @default "1"
			* @type {string | number}
			* @example 
			 * &lt;ul id="sliderContent"&gt;
   * &lt;li&gt;&lt;img src="../images/rotator/nature.jpg" /&gt;&lt;/li&gt;
   * &lt;li&gt;&lt;img src="../images/rotator/bird.jpg"/&gt;&lt;/li&gt;
   * &lt;li&gt;&lt;img src="../images/rotator/sculpture.jpg"/&gt;&lt;/li&gt;
   * &lt;li&gt;&lt;img src="../images/rotator/seaview.jpg" /&gt;&lt;/li&gt;
   * &lt;li&gt;&lt;img src="../images/rotator/snowfall.jpg"/&gt;&lt;/li&gt;
   *&lt;/ul&gt;
    * &lt;script&gt;
			* // Set the navigateSteps during initialization. 			
			* 	$("#sliderContent").ejRotator({ navigateSteps : "1"});		 
			* &lt;/script&gt;
			 * @memberof ejRotator
			* @instance
			*/
            navigateSteps: "1",
            /**		
			* Sets the animationSpeed of slide transition.
			* @default 600
			* @type {string | number}
			* @example 
			 * &lt;ul id="sliderContent"&gt;
   * &lt;li&gt;&lt;img src="../images/rotator/nature.jpg" /&gt;&lt;/li&gt;
   * &lt;li&gt;&lt;img src="../images/rotator/bird.jpg"/&gt;&lt;/li&gt;
   * &lt;li&gt;&lt;img src="../images/rotator/sculpture.jpg"/&gt;&lt;/li&gt;
   * &lt;li&gt;&lt;img src="../images/rotator/seaview.jpg" /&gt;&lt;/li&gt;
   * &lt;li&gt;&lt;img src="../images/rotator/snowfall.jpg"/&gt;&lt;/li&gt;
   *&lt;/ul&gt;
    * &lt;script&gt;
			* // Set the animationSpeed during initialization. 			
			* 	$("#sliderContent").ejRotator({ animationSpeed : 600});			
			* &lt;/script&gt;
			 * @memberof ejRotator
			* @instance
			*/
            animationSpeed: 600,
            /**		
			* Sets the index of the slide that must be displayed first.
			* @default "1"
			* @type {string | number}
			* @example 
			 * &lt;ul id="sliderContent"&gt;
   * &lt;li&gt;&lt;img src="../images/rotator/nature.jpg" /&gt;&lt;/li&gt;
   * &lt;li&gt;&lt;img src="../images/rotator/bird.jpg"/&gt;&lt;/li&gt;
   * &lt;li&gt;&lt;img src="../images/rotator/sculpture.jpg"/&gt;&lt;/li&gt;
   * &lt;li&gt;&lt;img src="../images/rotator/seaview.jpg" /&gt;&lt;/li&gt;
   * &lt;li&gt;&lt;img src="../images/rotator/snowfall.jpg"/&gt;&lt;/li&gt;
   *&lt;/ul&gt;
    * &lt;script&gt;
			* // Set the startIndex during initialization. 			
			* 	$("#sliderContent").ejRotator({ startIndex : "1"});			
			* &lt;/script&gt;
			 * @memberof ejRotator
			* @instance
			*/
            startIndex: "0",
            /**		
			* Enable play / pause button on rotator.
			* @default false
			* @type {boolean}
			* @example 
			 * &lt;ul id="sliderContent"&gt;
   * &lt;li&gt;&lt;img src="../images/rotator/nature.jpg" /&gt;&lt;/li&gt;
   * &lt;li&gt;&lt;img src="../images/rotator/bird.jpg"/&gt;&lt;/li&gt;
   * &lt;li&gt;&lt;img src="../images/rotator/sculpture.jpg"/&gt;&lt;/li&gt;
   * &lt;li&gt;&lt;img src="../images/rotator/seaview.jpg" /&gt;&lt;/li&gt;
   * &lt;li&gt;&lt;img src="../images/rotator/snowfall.jpg"/&gt;&lt;/li&gt;
   *&lt;/ul&gt;
    * &lt;script&gt;
			* // Set the showPlayButton during initialization. 			
			* 	$("#sliderContent").ejRotator({ showPlayButton:true});			
			* &lt;/script&gt;
			 * @memberof ejRotator
			* @instance
			*/
            showPlayButton: false,
            /**		
			* Rotates the Rotator Items continuously without user interference.
			* @default false
			* @type {boolean}
			* @example 
			 * &lt;ul id="sliderContent"&gt;
   * &lt;li&gt;&lt;img src="../images/rotator/nature.jpg" /&gt;&lt;/li&gt;
   * &lt;li&gt;&lt;img src="../images/rotator/bird.jpg"/&gt;&lt;/li&gt;
   * &lt;li&gt;&lt;img src="../images/rotator/sculpture.jpg"/&gt;&lt;/li&gt;
   * &lt;li&gt;&lt;img src="../images/rotator/seaview.jpg" /&gt;&lt;/li&gt;
   * &lt;li&gt;&lt;img src="../images/rotator/snowfall.jpg"/&gt;&lt;/li&gt;
   *&lt;/ul&gt;
    * &lt;script&gt;
			* // Set the enableAutoPlay during initialization. 			
			* 	$("#sliderContent").ejRotator({ enableAutoPlay:true});		 
			* &lt;/script&gt;
			 * @memberof ejRotator
			* @instance
			*/
            enableAutoPlay: false,
            /**		
			* Turns on or off the slide buttons (next and previous) in the Rotator Items. Slide buttons are used to navigate the Rotator Items.
			* @default false
			* @type {boolean}
			* @example 
			 * &lt;ul id="sliderContent"&gt;
   * &lt;li&gt;&lt;img src="../images/rotator/nature.jpg" /&gt;&lt;/li&gt;
   * &lt;li&gt;&lt;img src="../images/rotator/bird.jpg"/&gt;&lt;/li&gt;
   * &lt;li&gt;&lt;img src="../images/rotator/sculpture.jpg"/&gt;&lt;/li&gt;
   * &lt;li&gt;&lt;img src="../images/rotator/seaview.jpg" /&gt;&lt;/li&gt;
   * &lt;li&gt;&lt;img src="../images/rotator/snowfall.jpg"/&gt;&lt;/li&gt;
   *&lt;/ul&gt;
    * &lt;script&gt;
			* // Set the showNavigateButton during initialization. 			
			* 	$("#sliderContent").ejRotator({ showNavigateButton : false});			
			* &lt;/script&gt;
			 * @memberof ejRotator
			* @instance
			*/
            showNavigateButton: true,
            /**		
			* Sets the width of a Rotator Item.
			* @default ""
			* @type {string | number}
			* @example 
			 * &lt;ul id="sliderContent"&gt;
   * &lt;li&gt;&lt;img src="../images/rotator/nature.jpg" /&gt;&lt;/li&gt;
   * &lt;li&gt;&lt;img src="../images/rotator/bird.jpg"/&gt;&lt;/li&gt;
   * &lt;li&gt;&lt;img src="../images/rotator/sculpture.jpg"/&gt;&lt;/li&gt;
   * &lt;li&gt;&lt;img src="../images/rotator/seaview.jpg" /&gt;&lt;/li&gt;
   * &lt;li&gt;&lt;img src="../images/rotator/snowfall.jpg"/&gt;&lt;/li&gt;
   *&lt;/ul&gt;
    * &lt;script&gt;
			* // Set the slideWidth during initialization. 			
			* 	$("#sliderContent").ejRotator({ slideWidth : "600px"});			
			* &lt;/script&gt;
			 * @memberof ejRotator
			* @instance
			*/
            slideWidth: "",
            /**		
			* Sets the height of a Rotator Item.
			* @default ""
			* @type {string | number}
			* @example 
			 * &lt;ul id="sliderContent"&gt;
   * &lt;li&gt;&lt;img src="../images/rotator/nature.jpg" /&gt;&lt;/li&gt;
   * &lt;li&gt;&lt;img src="../images/rotator/bird.jpg"/&gt;&lt;/li&gt;
   * &lt;li&gt;&lt;img src="../images/rotator/sculpture.jpg"/&gt;&lt;/li&gt;
   * &lt;li&gt;&lt;img src="../images/rotator/seaview.jpg" /&gt;&lt;/li&gt;
   * &lt;li&gt;&lt;img src="../images/rotator/snowfall.jpg"/&gt;&lt;/li&gt;
   *&lt;/ul&gt;
    * &lt;script&gt;
			* // Set the slideHeight during initialization. 			
			* 	$("#sliderContent").ejRotator({ slideHeight : "600px"});			
			* &lt;/script&gt;
			* @memberof ejRotator
			* @instance
			*/
            slideHeight: "",
            /**		
			* Sets the space between the Rotator Items.
			* @default ""
			* @type {string | number}
			* @example 
			 * &lt;ul id="sliderContent"&gt;
   * &lt;li&gt;&lt;img src="../images/rotator/nature.jpg" /&gt;&lt;/li&gt;
   * &lt;li&gt;&lt;img src="../images/rotator/bird.jpg"/&gt;&lt;/li&gt;
   * &lt;li&gt;&lt;img src="../images/rotator/sculpture.jpg"/&gt;&lt;/li&gt;
   * &lt;li&gt;&lt;img src="../images/rotator/seaview.jpg" /&gt;&lt;/li&gt;
   * &lt;li&gt;&lt;img src="../images/rotator/snowfall.jpg"/&gt;&lt;/li&gt;
   *&lt;/ul&gt;
    * &lt;script&gt;
			* // Set the frameSpace during initialization. 			
			* 	$("#sliderContent").ejRotator({ slideWidth:"600px",slideHeight:"400px",displayItemsCount:2, frameSpace:"10px"});			
			* &lt;/script&gt;
			 * @memberof ejRotator
			* @instance
			*/
            frameSpace: "",
            /**		
			* Resizes the Rotator when the browser is resized. 
			* @default false
			* @type {boolean}
			* @example 
			 * &lt;ul id="sliderContent"&gt;
   * &lt;li&gt;&lt;img src="../images/rotator/nature.jpg" /&gt;&lt;/li&gt;
   * &lt;li&gt;&lt;img src="../images/rotator/bird.jpg"/&gt;&lt;/li&gt;
   * &lt;li&gt;&lt;img src="../images/rotator/sculpture.jpg"/&gt;&lt;/li&gt;
   * &lt;li&gt;&lt;img src="../images/rotator/seaview.jpg" /&gt;&lt;/li&gt;
   * &lt;li&gt;&lt;img src="../images/rotator/snowfall.jpg"/&gt;&lt;/li&gt;
   *&lt;/ul&gt;
    * &lt;script&gt;
			* // Set the enableResize during initialization. 			
			* 	$("#sliderContent").ejRotator({ enableResize : false});			
			* &lt;/script&gt;
			 * @memberof ejRotator
			* @instance
			*/
            enableResize: false,
            /**		
          * Specifies the orientation for the Rotator control, that is, whether it must be rendered horizontally or vertically. See {@link Orientation}
          * @default ej.Orientation.Horizontal
          * @type {enum}
          * @example 
           * &lt;ul id="sliderContent"&gt;
   * &lt;li&gt;&lt;img src="../images/rotator/nature.jpg" /&gt;&lt;/li&gt;
   * &lt;li&gt;&lt;img src="../images/rotator/bird.jpg"/&gt;&lt;/li&gt;
   * &lt;li&gt;&lt;img src="../images/rotator/sculpture.jpg"/&gt;&lt;/li&gt;
   * &lt;li&gt;&lt;img src="../images/rotator/seaview.jpg" /&gt;&lt;/li&gt;
   * &lt;li&gt;&lt;img src="../images/rotator/snowfall.jpg"/&gt;&lt;/li&gt;
   *&lt;/ul&gt;
    * &lt;script&gt;
          * // Set the orientation during initialization. 			
          * 	$("#sliderContent").ejRotator({ orientation : ej.Orientation.Horizontal });			
          * &lt;/script&gt;
           * @memberof ejRotator
          * @instance
          */
            orientation: "horizontal",
            /**		
          * Specifies the position of the showPager in the Rotator Item. See {@link PagerPosition}
          * @default "outside"
          * @type {string | enum}
          * @example 
           * &lt;ul id="sliderContent"&gt;
   * &lt;li&gt;&lt;img src="../images/rotator/nature.jpg" /&gt;&lt;/li&gt;
   * &lt;li&gt;&lt;img src="../images/rotator/bird.jpg"/&gt;&lt;/li&gt;
   * &lt;li&gt;&lt;img src="../images/rotator/sculpture.jpg"/&gt;&lt;/li&gt;
   * &lt;li&gt;&lt;img src="../images/rotator/seaview.jpg" /&gt;&lt;/li&gt;
   * &lt;li&gt;&lt;img src="../images/rotator/snowfall.jpg"/&gt;&lt;/li&gt;
   *&lt;/ul&gt;
    * &lt;script&gt;
          * // Set the pagerPosition during initialization. 			
          * 	$("#sliderContent").ejRotator({ pagerPosition :"outside" });			
          * &lt;/script&gt;
           * @memberof ejRotator
          * @instance
          */
            pagerPosition: "outside",
            /**		
			* Turns on or off thumbnail support in the Rotator control. Thumbnail is used to navigate between slides. Thumbnail supports only single slide transition
			* You must specify the source for thumbnail elements through the thumbnailSourceID property.
			* @default false
			* @type {boolean}
			* @example 
			 * &lt;ul id="sliderContent"&gt;
   * &lt;li&gt;&lt;img src="../images/rotator/nature.jpg" /&gt;&lt;/li&gt;
   * &lt;li&gt;&lt;img src="../images/rotator/bird.jpg"/&gt;&lt;/li&gt;
   * &lt;li&gt;&lt;img src="../images/rotator/sculpture.jpg"/&gt;&lt;/li&gt;
   * &lt;li&gt;&lt;img src="../images/rotator/seaview.jpg" /&gt;&lt;/li&gt;
   * &lt;li&gt;&lt;img src="../images/rotator/snowfall.jpg"/&gt;&lt;/li&gt;
   *&lt;/ul&gt;
   * <!-- Thumbnail Source -->
   * &lt;ul id="thumbElement"&gt;
   * &lt;li&gt;&lt;img src="../images/rotator/nature.jpg" /&gt;&lt;/li&gt;
   * &lt;li&gt;&lt;img src="../images/rotator/bird.jpg"/&gt;&lt;/li&gt;
   * &lt;li&gt;&lt;img src="../images/rotator/sculpture.jpg"/&gt;&lt;/li&gt;
   * &lt;li&gt;&lt;img src="../images/rotator/seaview.jpg" /&gt;&lt;/li&gt;
   * &lt;li&gt;&lt;img src="../images/rotator/snowfall.jpg"/&gt;&lt;/li&gt;
   *&lt;/ul&gt;
    * &lt;script&gt;
			* // Set the showThumbnail during initialization. 			
			* 	$("#sliderContent").ejRotator({ showThumbnail:true, thumbnailSourceID :"thumbElement"});			
			* &lt;/script&gt;
			 * @memberof ejRotator
			* @instance
			*/
            showThumbnail: false,
            /**		
          * Turns on or off the pager support in the Rotator control. The Pager is used to navigate the Rotator Items.
          * @default true
          * @type {boolean}
          * @example 
           * &lt;ul id="sliderContent"&gt;
   * &lt;li&gt;&lt;img src="../images/rotator/nature.jpg" /&gt;&lt;/li&gt;
   * &lt;li&gt;&lt;img src="../images/rotator/bird.jpg"/&gt;&lt;/li&gt;
   * &lt;li&gt;&lt;img src="../images/rotator/sculpture.jpg"/&gt;&lt;/li&gt;
   * &lt;li&gt;&lt;img src="../images/rotator/seaview.jpg" /&gt;&lt;/li&gt;
   * &lt;li&gt;&lt;img src="../images/rotator/snowfall.jpg"/&gt;&lt;/li&gt;
   *&lt;/ul&gt;
    * &lt;script&gt;
          * // Set the pager during initialization. 			
          * 	$("#sliderContent").ejRotator({ showPager : false});			
          * &lt;/script&gt;
           * @memberof ejRotator
          * @instance
          */
            showPager: true,
            /**		
       * Pause the auto play while hover on the rotator content.
       * @default false
       * @type {boolean}
       * @example 
       * &lt;ul id="sliderContent"&gt;
       * &lt;li&gt;&lt;img src="../images/rotator/nature.jpg" /&gt;&lt;/li&gt;
       * &lt;li&gt;&lt;img src="../images/rotator/bird.jpg"/&gt;&lt;/li&gt;
       * &lt;li&gt;&lt;img src="../images/rotator/sculpture.jpg"/&gt;&lt;/li&gt;
       * &lt;li&gt;&lt;img src="../images/rotator/seaview.jpg" /&gt;&lt;/li&gt;
       * &lt;li&gt;&lt;img src="../images/rotator/snowfall.jpg"/&gt;&lt;/li&gt;
       * &lt;/ul&gt;
       * &lt;script&gt;
       * // Set the stopOnHover during initialization. 			
       * 	$("#sliderContent").ejRotator({ enableAutoPlay:true, stopOnHover:true});			
       * &lt;/script&gt;
       * @memberof ejRotator
       * @instance
       */
            stopOnHover: false,
            /**		
			* Specifies the source for thumbnail elements.
			* @default null
			* @type {object}
			* @example 
             * &lt;ul id="sliderContent"&gt;
   * &lt;li&gt;&lt;img src="../images/rotator/nature.jpg" /&gt;&lt;/li&gt;
   * &lt;li&gt;&lt;img src="../images/rotator/bird.jpg"/&gt;&lt;/li&gt;
   * &lt;li&gt;&lt;img src="../images/rotator/sculpture.jpg"/&gt;&lt;/li&gt;
   * &lt;li&gt;&lt;img src="../images/rotator/seaview.jpg" /&gt;&lt;/li&gt;
   * &lt;li&gt;&lt;img src="../images/rotator/snowfall.jpg"/&gt;&lt;/li&gt;
   *&lt;/ul&gt;

			*&lt;ul id="thumbElement"&gt;
			*&lt;li&gt;&lt;img src="../images/rotator/nature.jpg"/&gt;&lt;/li&gt;
			*&lt;li&gt;&lt;img src="../images/rotator/bird.jpg" /&gt;&lt;/li&gt;
			*&lt;li&gt;&lt;img src="../images/rotator/sculpture.jpg"/&gt;&lt;/li&gt;
			*&lt;li&gt;&lt;img src="../images/rotator/seaview.jpg"/&gt;&lt;/li&gt;
			*&lt;li&gt;&lt;img src="../images/rotator/snowfall.jpg"/&gt;&lt;/li&gt;
			*&lt;/ul&gt;
			* &lt;script&gt;
			* // Set the thumbnailSourceID during initialization. 			
			* 	$("#sliderContent").ejRotator({ showThumbnail:true, thumbnailSourceID :"thumbElement"});			 
			* &lt;/script&gt;
			 * @memberof ejRotator
			* @instance
			*/
            thumbnailSourceID: null,
            /**		
          * If the Rotator Item is an image, you can specify a caption for the Rotator Item. The caption text for each Rotator Item must be set by using the title attribute of the respective <image> tag.
          * The caption cannot be displayed if multiple Rotator Items are present.
          * @default false
          * @type {boolean}
          * @example 
          * &lt;ul id="sliderContent"&gt;
   * &lt;li&gt;&lt;img src="../images/rotator/nature.jpg" title="Nature" /&gt;&lt;/li&gt;
   * &lt;li&gt;&lt;img src="../images/rotator/bird.jpg" title="Bird" /&gt;&lt;/li&gt;
   * &lt;li&gt;&lt;img src="../images/rotator/sculpture.jpg" title="Sculpture" /&gt;&lt;/li&gt;
   * &lt;li&gt;&lt;img src="../images/rotator/seaview.jpg" title="Seaview"  /&gt;&lt;/li&gt;
   * &lt;li&gt;&lt;img src="../images/rotator/snowfall.jpg" title="Snowfall" /&gt;&lt;/li&gt;
   *&lt;/ul&gt;
    * &lt;script&gt;
          * // Set the caption during initialization. 			
          * 	$("#sliderContent").ejRotator({showCaption:true});			
          * &lt;/script&gt;
           * @memberof ejRotator
          * @instance
          */
            showCaption: false,
            /**		
          * Turns on keyboard interaction with the Rotator items. You must set this property to true to access the following keyboard shortcuts:
          * @default true
          * @type {boolean}
          * @example 
           * &lt;ul id="sliderContent"&gt;
   * &lt;li&gt;&lt;img src="../images/rotator/nature.jpg" /&gt;&lt;/li&gt;
   * &lt;li&gt;&lt;img src="../images/rotator/bird.jpg"/&gt;&lt;/li&gt;
   * &lt;li&gt;&lt;img src="../images/rotator/sculpture.jpg"/&gt;&lt;/li&gt;
   * &lt;li&gt;&lt;img src="../images/rotator/seaview.jpg" /&gt;&lt;/li&gt;
   * &lt;li&gt;&lt;img src="../images/rotator/snowfall.jpg"/&gt;&lt;/li&gt;
   *&lt;/ul&gt;
    * &lt;script&gt;
          * // Set the allowKeyboardNavigation during initialization. 			
          * 	$("#sliderContent").ejRotator({ allowKeyboardNavigation : false});		
          * &lt;/script&gt;
           * @memberof ejRotator
          * @instance
          */
            allowKeyboardNavigation: true,
            /**		
          * Specifies right to left transition of slides. 
          * @default false
          * @type {boolean}
          * @example 
           * &lt;ul id="sliderContent"&gt;
   * &lt;li&gt;&lt;img src="../images/rotator/nature.jpg" /&gt;&lt;/li&gt;
   * &lt;li&gt;&lt;img src="../images/rotator/bird.jpg"/&gt;&lt;/li&gt;
   * &lt;li&gt;&lt;img src="../images/rotator/sculpture.jpg"/&gt;&lt;/li&gt;
   * &lt;li&gt;&lt;img src="../images/rotator/seaview.jpg" /&gt;&lt;/li&gt;
   * &lt;li&gt;&lt;img src="../images/rotator/snowfall.jpg"/&gt;&lt;/li&gt;
   *&lt;/ul&gt;
    * &lt;script&gt;
          * // Set the enableRTL during initialization. 			
          * 	$("#sliderContent").ejRotator({ enableRTL : false});			
          * &lt;/script&gt;
           * @memberof ejRotator
          * @instance
          */
            enableRTL: true,
            /**		
      * Specifies the animationType type for the Rotator Item. animationType options include slide, fastSlide, slowSlide, and other custom easing animationTypes.
      * @default "slide"
      * @type {string}
      * @example 
       * &lt;ul id="sliderContent"&gt;
   * &lt;li&gt;&lt;img src="../images/rotator/nature.jpg" /&gt;&lt;/li&gt;
   * &lt;li&gt;&lt;img src="../images/rotator/bird.jpg"/&gt;&lt;/li&gt;
   * &lt;li&gt;&lt;img src="../images/rotator/sculpture.jpg"/&gt;&lt;/li&gt;
   * &lt;li&gt;&lt;img src="../images/rotator/seaview.jpg" /&gt;&lt;/li&gt;
   * &lt;li&gt;&lt;img src="../images/rotator/snowfall.jpg"/&gt;&lt;/li&gt;
   *&lt;/ul&gt;
    * &lt;script&gt;
      * // Set the animationType during initialization. 			
      * 	$("#sliderContent").ejRotator({ animationType : "slide" });			
      * &lt;/script&gt;
       * @memberof ejRotator
      * @instance
      */
            animationType: "slide",
            /**   
      * Sets the delay between the Rotator Items move  after the slide transition.
      * @default 500
      * @type {string | number}
      * @example 
       * &lt;ul id="sliderContent"&gt;
   * &lt;li&gt;&lt;img src="../images/rotator/nature.jpg" /&gt;&lt;/li&gt;
   * &lt;li&gt;&lt;img src="../images/rotator/bird.jpg"/&gt;&lt;/li&gt;
   * &lt;li&gt;&lt;img src="../images/rotator/sculpture.jpg"/&gt;&lt;/li&gt;
   * &lt;li&gt;&lt;img src="../images/rotator/seaview.jpg" /&gt;&lt;/li&gt;
   * &lt;li&gt;&lt;img src="../images/rotator/snowfall.jpg"/&gt;&lt;/li&gt;
   *&lt;/ul&gt;
    * &lt;script&gt;
      * // Set the animationSpeed during initialization.      
      *   $("#sliderContent").ejRotator({ delay : 600});     
      * &lt;/script&gt;
       * @memberof ejRotator
      * @instance
      */
            delay: 500,
            //Events............
            /**     
            * This event is fired when the Rotator control is initialized.
            * @event
            * @name ejRotator#create 	
            * @param {Object} argument Event parameters from button     
            * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
            * @param {object}  argument.model returns the rotator model
            * @param {string}  argument.type returns the name of the event
            * @example 
             * &lt;ul id="sliderContent"&gt;
   * &lt;li&gt;&lt;img src="../images/rotator/nature.jpg" /&gt;&lt;/li&gt;
   * &lt;li&gt;&lt;img src="../images/rotator/bird.jpg"/&gt;&lt;/li&gt;
   * &lt;li&gt;&lt;img src="../images/rotator/sculpture.jpg"/&gt;&lt;/li&gt;
   * &lt;li&gt;&lt;img src="../images/rotator/seaview.jpg" /&gt;&lt;/li&gt;
   * &lt;li&gt;&lt;img src="../images/rotator/snowfall.jpg"/&gt;&lt;/li&gt;
   *&lt;/ul&gt;
    * &lt;script&gt;
            * $("#sliderContent").ejRotator({
            *    create: function (args) {}
            * });   
            * &lt;/script&gt;   
            * @memberof ejRotator
            * @instance
            */
            create: null,
            /**     
            * This event is fired when the Rotator slides are changed.
            * @event
            * @name ejRotator#change 	
            * @param {Object} argument Event parameters from button     
            * @param {boolean} argument.cancel if the event should be canceled; otherwise, false.
            * @param {object}  argument.model returns the rotator model
            * @param {string}  argument.type returns the name of the event
            * @param {string}  argument.itemId the current rotator id.
            * @param {number}  argument.activeItemIndex returns the current slide index.
            * @example 
             * &lt;ul id="sliderContent"&gt;
   * &lt;li&gt;&lt;img src="../images/rotator/nature.jpg" /&gt;&lt;/li&gt;
   * &lt;li&gt;&lt;img src="../images/rotator/bird.jpg"/&gt;&lt;/li&gt;
   * &lt;li&gt;&lt;img src="../images/rotator/sculpture.jpg"/&gt;&lt;/li&gt;
   * &lt;li&gt;&lt;img src="../images/rotator/seaview.jpg" /&gt;&lt;/li&gt;
   * &lt;li&gt;&lt;img src="../images/rotator/snowfall.jpg"/&gt;&lt;/li&gt;
   *&lt;/ul&gt;
    * &lt;script&gt;
            * $("#sliderContent").ejRotator({
            *    change: function (args) {}
            * }); 
            * &lt;/script&gt;     
            * @memberof ejRotator
            * @instance
            */
            change: null,
            /**     
            * This event is fired when enableAutoPlay is started.
            * @event
            * @name ejRotator#start 	
            * @param {Object} argument Event parameters from button     
            * @param {boolean} argument.cancel if the event should be canceled; otherwise, false.
            * @param {object}  argument.model returns the rotator model
            * @param {string}  argument.type returns the name of the event
            * @param {string}  argument.itemId the current rotator id.
            * @param {number}  argument.activeItemIndex returns the current slide index.
            * @example 
            * &lt;ul id="sliderContent"&gt;
   * &lt;li&gt;&lt;img src="../images/rotator/nature.jpg" /&gt;&lt;/li&gt;
   * &lt;li&gt;&lt;img src="../images/rotator/bird.jpg"/&gt;&lt;/li&gt;
   * &lt;li&gt;&lt;img src="../images/rotator/sculpture.jpg"/&gt;&lt;/li&gt;
   * &lt;li&gt;&lt;img src="../images/rotator/seaview.jpg" /&gt;&lt;/li&gt;
   * &lt;li&gt;&lt;img src="../images/rotator/snowfall.jpg"/&gt;&lt;/li&gt;
   *&lt;/ul&gt;
    * &lt;script&gt;
           
            * $("#sliderContent").ejRotator({
            *    start: function (args) {}
            * });
            * &lt;/script&gt;      
            * @memberof ejRotator
            * @instance
            */
            start: null,
            /**     
            * This event is fired when autoplay is stopped or paused.
            * @event
            * @name ejRotator#stop 	
            * @param {Object} argument Event parameters from button     
            * @param {boolean} argument.cancel if the event should be canceled; otherwise, false.
            * @param {object}  argument.model returns the rotator model
            * @param {string}  argument.type returns the name of the event
            * @param {string}  argument.itemId the current rotator id.
            * @param {number}  argument.activeItemIndex returns the current slide index.
            * @example 
             * &lt;ul id="sliderContent"&gt;
   * &lt;li&gt;&lt;img src="../images/rotator/nature.jpg" /&gt;&lt;/li&gt;
   * &lt;li&gt;&lt;img src="../images/rotator/bird.jpg"/&gt;&lt;/li&gt;
   * &lt;li&gt;&lt;img src="../images/rotator/sculpture.jpg"/&gt;&lt;/li&gt;
   * &lt;li&gt;&lt;img src="../images/rotator/seaview.jpg" /&gt;&lt;/li&gt;
   * &lt;li&gt;&lt;img src="../images/rotator/snowfall.jpg"/&gt;&lt;/li&gt;
   *&lt;/ul&gt;
    * &lt;script&gt;
            * $("#sliderContent").ejRotator({
            *    stop: function (args) {}
            * });    
            * &lt;/script&gt;  
            * @memberof ejRotator
            * @instance
            */
            stop: null,
            /**     
			 * This event is fired when a thumbnail pager is clicked.
			 * @event
			 * @name ejRotator#thumbItemClick 	
			 * @param {Object} argument Event parameters from button     
			 * @param {boolean} argument.cancel if the event should be canceled; otherwise, false.
			 * @param {object}  argument.model returns the rotator model
			 * @param {string}  argument.type returns the name of the event
			 * @param {string}  argument.itemId the current rotator id.
			 * @param {number}  argument.activeItemIndex returns the current slide index.
			 * @example 
			  * &lt;ul id="sliderContent"&gt;
   * &lt;li&gt;&lt;img src="../images/rotator/nature.jpg" /&gt;&lt;/li&gt;
   * &lt;li&gt;&lt;img src="../images/rotator/bird.jpg"/&gt;&lt;/li&gt;
   * &lt;li&gt;&lt;img src="../images/rotator/sculpture.jpg"/&gt;&lt;/li&gt;
   * &lt;li&gt;&lt;img src="../images/rotator/seaview.jpg" /&gt;&lt;/li&gt;
   * &lt;li&gt;&lt;img src="../images/rotator/snowfall.jpg"/&gt;&lt;/li&gt;
   *&lt;/ul&gt;
   * &lt;ul id="thumbElement"&gt;
   * &lt;li&gt;&lt;img src="../images/rotator/nature.jpg" /&gt;&lt;/li&gt;
   * &lt;li&gt;&lt;img src="../images/rotator/bird.jpg"/&gt;&lt;/li&gt;
   * &lt;li&gt;&lt;img src="../images/rotator/sculpture.jpg"/&gt;&lt;/li&gt;
   * &lt;li&gt;&lt;img src="../images/rotator/seaview.jpg" /&gt;&lt;/li&gt;
   * &lt;li&gt;&lt;img src="../images/rotator/snowfall.jpg"/&gt;&lt;/li&gt;
   *&lt;/ul&gt;
    * &lt;script&gt;
             * $("#sliderContent").ejRotator({
             *    showThumbnail:true,
             *    thumbnailSourceID:"thumbElement",
             *    thumbItemClick: function (args) {}
             * });   
             * &lt;/script&gt;   
			 * @memberof ejRotator
			 * @instance
			 */
            thumbItemClick: null,
            /**     
			 * This event is fired when a pager is clicked.
			 * @event
			 * @name ejRotator#pagerClick 	
			 * @param {Object} argument Event parameters from button     
			 * @param {boolean} argument.cancel if the event should be canceled; otherwise, false.
			 * @param {object}  argument.model returns the rotator model
			 * @param {string}  argument.type returns the name of the event
			 * @param {string}  argument.itemId the current rotator id.
			 * @param {number}  argument.activeItemIndex returns the current slide index.
			 * @example 
			  * &lt;ul id="sliderContent"&gt;
   * &lt;li&gt;&lt;img src="../images/rotator/nature.jpg" /&gt;&lt;/li&gt;
   * &lt;li&gt;&lt;img src="../images/rotator/bird.jpg"/&gt;&lt;/li&gt;
   * &lt;li&gt;&lt;img src="../images/rotator/sculpture.jpg"/&gt;&lt;/li&gt;
   * &lt;li&gt;&lt;img src="../images/rotator/seaview.jpg" /&gt;&lt;/li&gt;
   * &lt;li&gt;&lt;img src="../images/rotator/snowfall.jpg"/&gt;&lt;/li&gt;
   *&lt;/ul&gt;
    * &lt;script&gt;
             * $("#sliderContent").ejRotator({
             *    pagerClick: function (args) {}
             * }); 
             * &lt;/script&gt;    
			 * @memberof ejRotator
			 * @instance
			 */
            pagerClick: null,
            /**     
           * This event is fired when the Rotator control is destroyed.
           * @event
           * @name ejRotator#destroy 	
           * @param {Object} argument Event parameters from button     
           * @param {boolean} argument.cancel if the event should be canceled; otherwise, false.
           * @param {object}  argument.model returns the rotator model
           * @param {string}  argument.type returns the name of the event
           * @example 
            * &lt;ul id="sliderContent"&gt;
  * &lt;li&gt;&lt;img src="../images/rotator/nature.jpg" /&gt;&lt;/li&gt;
  * &lt;li&gt;&lt;img src="../images/rotator/bird.jpg"/&gt;&lt;/li&gt;
  * &lt;li&gt;&lt;img src="../images/rotator/sculpture.jpg"/&gt;&lt;/li&gt;
  * &lt;li&gt;&lt;img src="../images/rotator/seaview.jpg" /&gt;&lt;/li&gt;
  * &lt;li&gt;&lt;img src="../images/rotator/snowfall.jpg"/&gt;&lt;/li&gt;
  *&lt;/ul&gt;
   * &lt;script&gt;
           * $("#sliderContent").ejRotator({
           *    destroy: function (args) {}
           * });   
           * &lt;/script&gt;   
           * @memberof ejRotator
           * @instance
           */
            destroy: null

        },
        //Data Types
        dataTypes: {
            cssClass: "string",
            dataSource: "data",
            query: "data",
            fields: "data",
            enabled: "boolean",
            displayItemsCount: "",
            navigateSteps: "",
            animationSpeed: "",
            transitionDelay: "",
            startIndex: "",
            showPlayButton: "boolean",
            enableAutoPlay: "boolean",
            slideWidth: "",
            slideHeight: "",
            frameSpace: "",
            enableResize: "boolean",
            orientation: "enum",
            pagerPosition: "enum",
            showThumbnail: "boolean",
            thumbnailSourceID: "",
            showPager: "boolean",
            showCaption: "boolean",
            enableRTL: "boolean",
            allowKeyboardNavigation: "boolean",
            animationType: "string",
            delay: ""
        },

        /**
       * Create the Rotator widget.
       * @private
       */
        _init: function () {
            this._initialize();
            if (this.model.dataSource == null)
                this._setValues();
            this._render();
            this._wireEvent();
            if (this.model.enableAutoPlay) this.play();
        },
        /**
        * To initialize the Rotator.		
        * @private
        */
        _initialize: function () {
            this.element.attr("tabindex", 0);
            this._liCount = null;
            this.transitionDelay = null;
            this._animating = false;
            this._interval = null;
            this._thumbCount = null;
            this._liSize = null;
            this._containerSize = null;
            this.containerCss = this.model.orientation == ej.Orientation.Horizontal ? "outerWidth" : "outerHeight";
            this.displayCss = this.model.orientation == ej.Orientation.Horizontal ? "left" : "top";
            $.extend(jQuery.easing, {
                slowSlide: function (x, t, b, c, d) {
                    var ts = (t /= d) * t;
                    return b + c * (ts * ts);
                },
                fastSlide: function (x, t, b, c, d) {
                    var ts = (t /= d) * t;
                    var tc = ts * t;
                    return b + c * (tc * ts + -5 * ts * ts + 10 * tc + -10 * ts + 5 * t);
                },
                slide: function (x, t, b, c, d) {
                    var ts = (t /= d) * t;
                    var tc = ts * t;
                    return b + c * (6 * tc * ts + -15 * ts * ts + 10 * tc);
                }

            })
        },
        /**
       * To configure the property values.
       * @private
       */
        _setValues: function (val) {
            this._setLicount();
            this._setSpeed(this.model.animationSpeed);
            this._setVisibleItemCount(this.model.displayItemsCount);
            this._setItemMove(this.model.navigateSteps);
            this._setFrameSpace(this.model.frameSpace);
            this._setSlideWidth(this.model.slideWidth);
            this._setSlideHeight(this.model.slideHeight);
            this._changeProperty();
            this._setInitial();
        },
        _refresh: function () {
            this._setValues();
            this._setDimension();
            this._changeSkin();
        },
        _reRenderControl: function () {
            this._setLicount();
            this._setSpeed(this.model.animationSpeed);
            this._setVisibleItemCount(this.model.displayItemsCount);
            this._setItemMove(this.model.navigateSteps);
            this._setFrameSpace(this.model.frameSpace);
            this._setSlideWidth(this.model.slideWidth);
            this._setSlideHeight(this.model.slideHeight);
            this._changeProperty();
            this._cloneItem();
            this._createCaption();
            this._changeSkin();
        },
        _refreshControl: function () {
            if (this.model.showPager) {
                this._bulletWrapper.remove();
                this._createBulletControl();
            }
            else if (this.element.parents('.e-in-wrap').siblings().hasClass("e-pager-wrap"))
                this._bulletWrapper.remove();

            if (this.model.showCaption) {
                if (this.element.siblings().hasClass("e-caption"))
                    this._caption.remove();
                this._createCaption();
            } else if (this.element.siblings().hasClass("e-caption"))
                this._caption.remove();

            if (this.model.showThumbnail) {
                if (this.element.parents('.e-in-wrap').siblings().hasClass("e-thumb")) {
                    this._thumb.remove();
                    this._thumbControl.remove();
                }
                this._createThumb();
            }
            else if (this.element.parents('.e-in-wrap').siblings().hasClass("e-thumb")) {
                this._thumb.remove();
                this._thumbControl.remove();
            }
            this._changeSkin();
        },
        /**
        * Section for check whether collection of items is local or remote data.	
        * @private
        */
        _checkDataBinding: function () {
            var source = this.model.dataSource;
            if (source != null) {
                this.element.addClass("onloading");
                if (ej.DataManager && source instanceof ej.DataManager)
                    this._initDataSource(source);
                else
                    this._renderItems(source);
            }
            this._setValues();
        },
        /**
        * Section for render items with Remote Data.	
        * @private
        */
        _initDataSource: function (source) {
            var proxy = this, queryPromise;
            queryPromise = source.executeQuery(this._getQuery());
            queryPromise.done(function (e) {
                proxy._renderItems(e.result);
            }).fail(function (e) {
                proxy.element.removeClass("onloading");
            });
        },
        _getQuery: function () {
            var column;
            if (ej.isNullOrUndefined(this.model.query)) {
                column = [], queryManager = ej.Query(), mapper = this.model.fields, col;
                for (col in mapper) {
                    if (col !== "tableName" && mapper[col])
                        column.push(mapper[col]);
                }
                if (column.length > 0)
                    queryManager.select(column);
                if (!this.model.dataSource.dataSource.url.match(mapper.tableName + "$"))
                    !ej.isNullOrUndefined(mapper.tableName) && queryManager.from(mapper.tableName);
            }
            else queryManager = this.model.query;
            return queryManager;
        },
        /**
        * Section for rendering Rotator items.
        * @private
        */
        _renderItems: function (list) {
            this._generateTagItems(list);
            this.element.removeClass("onloading");
        },
        /**
        * Section for render Rotator items with DataSource.
        * @private
        */
        _generateTagItems: function (list) {
            var i, mapField = this._getMapper();
            for (i = 0; i < list.length; i++) {
                this.element.append(this._generateLi(list[i], mapField));
            }
        },
        /**
        * Section for mapping fields.
        * @private
        */
        _getMapper: function () {
            var mapper = this.model.fields, mapFld = { _text: null, _url: null };
            mapFld._text = (mapper && mapper.text) ? mapper["text"] : "text";
            mapFld._url = (mapper && mapper.url) ? mapper["url"] : "url";
            mapFld._attr = (mapper && mapper.htmlAttr) ? mapper["htmlAttr"] : "htmlAttr";
            return mapFld;
        },
        /**
         * Section for Render rotator items.
		 * @private
         */
        _generateLi: function (list, map) {
            var li = ej.buildTag("li"), imgTag;
            imgTag = $('<img  src="' + list[map._url] + '"title="' + list[map._text] + '"/>');
            this._setAttributes((list[map._attr]), imgTag);
            li.append(imgTag);
            return li;
        },
        /**
        * Section for set attributes to Rotator items.
        * @private
        */
        _setAttributes: function (data, element) {
            if (data) {
                for (var key in data)
                    element.attr(key, data[key]);
            }
        },
        _createWrapper: function () {
            this._wrapper = ej.buildTag("div.e-inner e-" + this.model.orientation).insertBefore(this.element);
            if (this.model.dataSource != null)
                this._checkDataBinding();
            this._wrapper.append(this.element);
            this._innerWrapper = ej.buildTag("div.e-in-wrap").insertBefore(this._wrapper);
            this._innerWrapper.append(this._wrapper);
            this._outerWrapper = ej.buildTag("div.e-rotator-wrap e-widget " + this.model.cssClass).insertBefore(this._innerWrapper);
            this._outerWrapper.append(this._innerWrapper);
            if (!this.model.enabled) this._outerWrapper.addClass("e-disable").attr({ "aria-disabled": true });
        },
        _refreshTagItems: function (key, value) {
            this.model[key] = value;
            this.element.children('li').remove();
            this._checkDataBinding();
            this._cloneItem();
            this._refreshControl();
        },
        /**
         * Render Section for slide Button control.		
		 * @private
         */
        _createButtonControl: function () {
            this._buttonWrapper = ej.buildTag("div.e-nav").insertAfter(this._wrapper);
            if (this.model.showNavigateButton) {
                this._prevButton = ej.buildTag("span.e-nav-btn e-icon e-previous").attr({ 'role': 'button' }).appendTo(this._buttonWrapper);
                this._nextButton = ej.buildTag("span.e-nav-btn e-icon e-next").attr({ 'role': 'button' }).appendTo(this._buttonWrapper);
            }
            this._buttonWrapper.appendTo(this._wrapper);
            this._createAutoPlay();
            if (this.model.enabled && this.model.showNavigateButton) this._wireBtnEvents();
        },
        /**
         * Render Section for Play / Pause Button control.		
		 * @private
         */
        _createAutoPlay: function () {
            var icon;
            if (!this.model.showPlayButton) return false;
            icon = this._interval != null ? 'pause' : 'play';
            this._autoButton = ej.buildTag("span.e-nav-btn e-icon " + icon, "", {}, { role: "button" }).appendTo(this._buttonWrapper);
            if (this.model.enabled) this._wireAutoPlayEvents();
        },
        /**
         * Render Section for pager control.		
		 * @private
         */
        _createBulletControl: function () {
            var end, liElement;
            this._bulletWrapper = ej.buildTag("div.e-pager-wrap", "", {}, { role: "group", "aria-label": "showPager", tabindex: "0" }).insertAfter(this._innerWrapper);
            this._bullet = ej.buildTag("ul.e-bullet e-default");
            end = (this._liCount - (this.model.displayItemsCount * 2)) / this.model.navigateSteps;
            if ((this._liCount - (this.model.displayItemsCount * 2)) % this.model.navigateSteps != 0)
                end++;
            for (i = 1; i <= end ; i++)
                ej.buildTag("li.e-icon", "", {}, { tabindex: "-1", "aria-selected": "false" }).appendTo(this._bullet);
            this._bullet.appendTo(this._bulletWrapper);
            this._setActiveBullet();
            this._setPagerposition(this.model.pagerPosition);
            if (this.model.enabled) this._wireBulletEvents();
        },
        /**
         * Section for Rendering caption.		
		 * @private
         */
        _createCaption: function () {
            var index;
            if (!this.model.showCaption) return false;
            this._caption = ej.buildTag("div.e-caption").append($('<span>')).insertAfter(this._buttonWrapper);
            index = Math.round((-Math.round(this.element.position()[this.displayCss])) / this._liSize); index--;
            this._setCaptionText(index);
        },
        /**
         * Section for Rendering thumbnail item.		
		 * @private
         */
        _createThumb: function () {
            var liElements, i;
            if (this.model.thumbnailSourceID != null) {
                this._thumb = this.model.showPager ? ej.buildTag("div.e-thumb", "", {}, { tabindex: "0", role: "group" }).insertAfter(this._bulletWrapper) : ej.buildTag("div.e-thumb", "", {}, { tabindex: "0", role: "group" }).insertAfter(this._innerWrapper);
                this._thumbItems = ej.buildTag("ul.e-thumb-items e-ul");
                for (i = 0; i < $('#' + this.model.thumbnailSourceID).children('li').length ; i++) {
                    liElements = $('#' + this.model.thumbnailSourceID).children('li').eq(i).clone();
                    liElements.addClass("e-thumb-ele").attr('tabindex', -1).removeAttr("style").appendTo(this._thumbItems);
                }
                this._thumbItems.appendTo(this._thumb);
                $('#' + this.model.thumbnailSourceID).css("display", "none");
                this._setThumbProperty();
                this._setActiveThumb();

            }
        },
        /**
         * Section for Rendering thumbnail control.		
		 * @private
         */
        _createThumbControl: function () {
            this._thumbControl = ej.buildTag("div.e-thumb-nav").insertAfter(this._thumb);
            this._previous = ej.buildTag("span.e-thumb-btn e-icon e-previous e-disable", "", {}, { "aria-disabled": true }).appendTo(this._thumbControl);
            this._next = ej.buildTag("span.e-thumb-btn e-icon e-next e-enable", "", {}, { "aria-disabled": false }).appendTo(this._thumbControl);
            this._thumbControl.appendTo(this._outerWrapper);
            if (this.model.enabled) this._wireThumbEvents();
        },
        /**
        * To configure the custom theme for Rotator using cssClass property.		
        * @private
        */
        _setSkin: function (skin) {
            this._outerWrapper.removeClass(this.model.cssClass).addClass(skin);
        },
        /**
         * Section for clone Rotator items.		
		 * @private
         */
        _cloneItem: function () {
            var i = 0, CloneElements = ej.buildTag("ul");
            for (i = 0; i < this.model.displayItemsCount ; i++)
                this.element.children('li').eq(i).clone().addClass("clone").appendTo(this.element);
            for (i = (this._liCount - this.model.displayItemsCount) ; i < this._liCount; i++)
                this.element.children('li').eq(i).clone().addClass("clone").prependTo(CloneElements);
            for (i = 0; i < CloneElements.children('li').length; i++)
                CloneElements.children('li').eq(i).clone().prependTo(this.element);
            this._setDimension();
        },
        /**
         * Section for undo the clone Rotator items.		
		 * @private
         */
        _undoClone: function () {
            for (i = 0; i < this._liCount; i++) {
                if (this.element.children('li').eq(i).hasClass("clone")) {
                    this.element.children('li').eq(i).remove();
                    this._setLicount();
                    i--;
                }
            }
        },
        /**
        * To configure the properties at runtime using SetModel		
        * @private
        */
        _setModel: function (options) {
            if (!this.model.enabled && ej.isNullOrUndefined(options["enabled"])) return false;
            var option, valueChange = false, controlChange = false;
            for (option in options) {
                switch (option) {
                    case "fields":
                    case "query":
                    case "dataSource":
                        this._refreshTagItems(option, options[option]);
                        break;
                    case "enabled":
                        this._changeState(options[option]);
                        break;
                    case "slideWidth":
                        this._setSlideWidth(options[option]);
                        valueChange = true;
                        controlChange = true;
                        break;
                    case "slideHeight":
                        this._setSlideHeight(options[option]);
                        valueChange = true;
                        controlChange = true;
                        break;
                    case "displayItemsCount":
                        this._undoClone();
                        this._setVisibleItemCount(options[option]);
                        this._cloneItem();
                        valueChange = true;
                        controlChange = true;
                        break;
                    case "navigateSteps":
                        options[option] = this._validateItemMove(options[option]);//this._setItemMove(options[option]);
                        this.model.navigateSteps = options[option];
                        valueChange = true;
                        controlChange = true;
                        break;
                    case "pagerPosition":
                        this._setPagerposition(options[option]);
                        break;
                    case "showPager":
                        this.model.showPager = options[option];
                        controlChange = true;
                        break;
                    case "showThumbnail":
                        this.model.showThumbnail = options[option];
                        valueChange = true;
                        controlChange = true;
                        break;
                    case "thumbnailSourceID":
                        this.model.thumbnailSourceID = options[option];
                        valueChange = true;
                        controlChange = true;
                        break;
                    case "showCaption":
                        this.model.showCaption = options[option];
                        valueChange = true;
                        controlChange = true;
                        break;
                    case "enableResize":
                        this.model.enableResize = options[option];
                        this._wireResizeEvents();
                        break;
                    case "orientation":
                        this.model.orientation = options[option];
                        this._setOrientation(options[option]);
                        break;
                    case "animationSpeed":
                        this._setSpeed(options[option]);
                        break;
                    case "frameSpace":
                        this._setFrameSpace(options[option]);
                        valueChange = true;
                        controlChange = true;
                        break;
                    case "showPlayButton":
                        this.model.showPlayButton = options[option];
                        this._setAutoplay(options[option]);
                        this._changeSkin();
                        break;
                    case "startIndex":
                        this.model.startIndex = options[option];
                        valueChange = true;
                        controlChange = true;
                        break;
                    case "allowKeyboardNavigation":
                        this.model.allowKeyboardNavigation = options[option];
                        this._wireKeyboardEvents();
                        break;
                    case "showNavigateButton":
                        this.model.showNavigateButton = options[option];
                        this._setSlideButton(options[option]);
                        this._changeSkin();
                        break;
                    case "animationType":
                        this.model.animationType = options[option];
                        this._setSpeed(this.model.animationSpeed);
                        break;
                    case "cssClass": this._setSkin(options[option]); break;
                    case "enableRTL":
                        this.model.enableRTL = options[option];
                        break;
                    case "delay":
                        this._setSpeed(options[option]);
                        break;
                }
            }

            if (valueChange) this._refresh();
            if (controlChange) this._refreshControl();
        },
        /**
         * Section for set values to Rotator items.		
		 * @private
         */
        _setDimension: function () {
            this._setLicount();
            this._setUlInitial();
            this._setWidth();
            this._setHeight();
            this._setContainerPercent();
            this._changeProperty();
        },
        /**
        * Render Section For Rotator		
        * @private
        */
        _render: function () {
            this._createWrapper();
            this._cloneItem();
            this._createButtonControl();
            this._createCaption();
            if (this.model.showPager) this._createBulletControl();
            if (this.model.showThumbnail) this._createThumb();
            this._outerWrapper.find(this.element).addClass("e-ul");
            this._changeSkin();
        },
        /**
        * To configure the Rotator container width		
        * @private
        */
        _setWidth: function () {
            this._containerWidth = this.model.orientation == "horizontal" ?
            ((parseInt(this.model.slideWidth) * this.model.displayItemsCount) + (parseInt(this.model.frameSpace) * (parseInt(this.model.displayItemsCount) - 1))) : this.model.slideWidth;
            this.element.parent().css('width', this._containerWidth);
            this.element.parents('.e-in-wrap').css('width', this._containerWidth);
            this.element.parents('.e-rotator-wrap').css('width', this._containerWidth);
            this._changeProperty();
        },
        _setHeight: function () {
            this._containerHeight = this.model.orientation == "vertical" ?
            ((parseInt(this.model.slideHeight) * this.model.displayItemsCount) + (parseInt(this.model.frameSpace) * (parseInt(this.model.displayItemsCount) - 1))) : this.model.slideHeight;
            this.element.parent().css('height', this._containerHeight);
            this._changeProperty();
        },
        /**
        * To configure the rotator base element width and height 		
        * @private
        */
        _setUlInitial: function () {
            if (this.model.orientation == ej.Orientation.Horizontal) {
                this.element.css({
                    "width": this._liCount * this._liWidth,
                    "height": this._liHeight
                });
            }
            else {
                this.element.css({
                    "height": this._liCount * this._liHeight,
                    "width": this._liWidth
                });
            }
        },

        _setInitial: function () {
            this.element.css(this.displayCss, -(this._liSize * this.model.displayItemsCount) - (this.model.startIndex * this.model.navigateSteps * this._liSize));
        },
        _setBegin: function () {
            this.element.css(this.displayCss, -(this._liSize * this.model.displayItemsCount));
        },
        /**
         * To configure the rotator item width		
		 * @private
         */
        _setSlideWidth: function (val) {
            if (isNaN(parseInt(val)))
                this.model.slideWidth = this._getMaxSize("width");
            else {
                val = val.toString();
                this.model.slideWidth = this._checkValue(val, "width");
            }
            this.element.children('li').css("width", this.model.slideWidth);
            this._liWidth = this.element.children('li').outerWidth(true);
        },
        /**
         * To configure the rotator item height		
		 * @private
         */
        _setSlideHeight: function (val) {
            if (isNaN(parseInt(val)))
                this.model.slideHeight = this._getMaxSize("height");
            else {
                val = val.toString();
                this.model.slideHeight = this._checkValue(val, "height");
            }
            this.element.children('li').css("height", this.model.slideHeight);
            this._liHeight = this.element.children('li').outerHeight(true);
        },
        /**
         * To configure the showPager position.		
		 * @private
         */
        _setPagerposition: function (val) {
            if (this.model.showPager) {
                this._bullet.removeClass('e-default e-thumb-pos e-topleft e-topright e-bottomleft e-bottomright e-topCenter e-outside');
                this._bullet.addClass("e-" + val);
                if ((val == "bottomleft" || val == "bottomright") && this.model.showThumbnail && this.model.thumbnailSourceID != null)
                    this._bullet.addClass('e-thumb-pos');
            }
        },
        /**
         * To configure the caption text.		
		 * @private
         */
        _setCaptionText: function (index) {
            if (this.model.showCaption)
                this._caption.children().html(this.element.children('li').eq(++index).children('img').attr("title"));
        },
        /**
         * To configure the autoplay mode.		
		 * @private
         */
        _setAutoplay: function (val) {
            if (val)
                this._createAutoPlay();
            else if (this._autoButton != null) {
                this._setPause();
                this._autoButton.remove();
            }
        },
        /**
         * To configure the slide button.		
		 * @private
         */
        _setSlideButton: function (val) {
            if (val)
                this._createButtonControl()
            else {
                this._prevButton.remove();
                this._nextButton.remove();
            }
        },
        /**
         * To configure the orientation.		
		 * @private
         */
        _setOrientation: function (val) {
            this._wrapper.removeClass("e-" + this.model.orientation).addClass("e-" + val);
            this._setFrameSpace(this.model.frameSpace);
            this._refresh();
            if (this.model.orientation == ej.Orientation.Horizontal)
                this.element.css({ "left": -(Math.round(-(this.element.position().top) / this._liHeight) * this._liSize), "top": 0 });
            else
                this.element.css({ "top": -(Math.round(-(this.element.position().left) / this._liWidth) * this._liSize), "left": 0 });
        },
        /**
         * Section for set Active pager.		
		 * @private
         */
        _setActiveBullet: function () {
            var index = Math.round(-Math.round(this.element.position()[this.displayCss]) / (this._liSize * this.model.displayItemsCount));
            if (this.model.navigateSteps < this.model.displayItemsCount)
                index = -(Math.round(this.element.position()[this.displayCss]) + (this._liSize * this.model.displayItemsCount)) / (this._liSize * this.model.navigateSteps);
            index = this.model.navigateSteps == this.model.displayItemsCount ? index - 1 : index;
            this._bullet.children().eq(index).addClass('e-active').attr({ 'tabindex': 0, 'aria-selected': "true" });
        },
        /**
         * Section for set Active thumbnail item.		
		 * @private
         */
        _setActiveThumb: function () {
            var index = Math.round(-Math.round(this.element.position()[this.displayCss]) / this._liSize);
            index--;
            this._thumbItems.children().eq(index).addClass('e-active').attr({ 'tabindex': 0, 'aria-selected': "true" });
            this._transferThumb(index);
        },
        _checkValue: function (val, orient) {
            if ((val.indexOf("%")) > 0) {
                if (Math.round(parseInt(val)) == "100") {
                    this.model.displayItemsCount = 1;
                    this._setVisibleItemCount(this.model.displayItemsCount);
                }
                val = this._convertPercentageToPixel($(this.element[0]).parent()[orient](), parseInt(val));
            }
            return (val);
        },
        /**
         * Section for converting Percentage To Pixel.		
		 * @private
         */
        _convertPercentageToPixel: function (parent, child) {
            return Math.round((child * parent) / 100);  //round       
        },
        /**
         * Section for converting Pixel To Percentage.	
		 * @private
         */
        _convertPixelToPercentage: function (parent, child) {
            return Math.round((child / parent) * 100); //round
        },
        _changeProperty: function () {
            this.containerCss = this.model.orientation == ej.Orientation.Horizontal ? "outerWidth" : "outerHeight";
            this.displayCss = this.model.orientation == ej.Orientation.Horizontal ? "left" : "top";
            if (this.model.orientation == ej.Orientation.Horizontal) {
                this._liSize = this._liWidth;
                this._containerSize = this._containerWidth;
            }
            else {
                this._liSize = this._liHeight;
                this._containerSize = this._containerHeight;
            }
        },
        /**
         * To configure FrameSpace.	
		 * @private
         */
        _setFrameSpace: function (val) {
            if (isNaN(parseInt(val))) {
                this.model.frameSpace = "0px";
            } else {
                this.model.frameSpace = val;
                this.model.orientation == ej.Orientation.Horizontal ? this.element.children('li').css({ "margin-right": val, "margin-bottom": 0 }) : this.element.children('li').css({ "margin-bottom": val, "margin-right": 0 });
            }
        },
        /**
         * Section for finding rotator items count.	
		 * @private
         */
        _setLicount: function () {
            this._liCount = this.element.children('li').length;
        },
        /**
         * Section for finding Visible Rotator Item Count.	
		 * @private
         */
        _setVisibleItemCount: function (val) {
            this.model.displayItemsCount = val;
            this.model.navigateSteps = (this.model.navigateSteps <= this.model.displayItemsCount) ? this.model.navigateSteps : this.model.displayItemsCount;
            this._checkAndSet();
        },
        _checkAndSet: function () {
            if ((this.model.displayItemsCount > 1) || (this.model.navigateSteps > 1)) {
                if (this.model.showThumbnail) this.model.showThumbnail = false;
                if (this.model.showCaption) this.model.showCaption = false;
            }
        },
        /**
         * Section for changing the border based on visible item count.	
		 * @private
         */
        _changeSkin: function () {
            var value;
            if ((this.model.displayItemsCount > 1) || (this.model.navigateSteps > 1)) {
                this._innerWrapper.addClass("e-multiple");
                this._wrapper.removeClass("e-single");
            } else {
                this._innerWrapper.removeClass('e-multiple');
                this._wrapper.addClass("e-single");
            }
            if (this.model.showPlayButton)
                this._autoButton.css({
                    "left": (((this._innerWrapper.outerWidth(true) / 2) - (this._autoButton.width() / 2)) / this._innerWrapper.outerWidth(true)) * (100) + "%",
                    "top": (((this._innerWrapper.outerHeight(true) / 2) - (this._autoButton.height() / 2)) / this._innerWrapper.outerHeight(true)) * (100) + "%"
                });

            if (this.model.showNavigateButton) {
                value = (((this._innerWrapper.outerHeight(true) / 2) - (this._prevButton.height() / 2)) / this._innerWrapper.outerHeight(true)) * (100) + "%";
                this._prevButton.css("top", value);
                this._nextButton.css("top", value);
            }
            if ((this.model.showThumbnail) && (this.model.thumbnailSourceID != null) && (this._previous != null) && (this._next != null)) {
                value = this._outerWrapper.outerHeight(true) + (this._thumbItems.children('li').outerHeight(true) / 2) - (this._thumbItems.children('li').outerHeight(true)) - (this._previous.height() / 2);
                value = (value / this._outerWrapper.outerHeight(true)) * 100 + "%";
                this._previous.css("top", value);
                this._next.css("top", value);
            }
        },
        /**
         * To configure Number of items Move.	
		 * @private
         */
        _setItemMove: function (val) {
            this.model.navigateSteps = this._validateItemMove(val);
            this._checkAndSet();
        },
        _validateItemMove: function (val) {
            return ((val <= this.model.displayItemsCount) ? val : this.model.displayItemsCount);
        },
        /**
         * To configure Play Action.	
		 * @private
         */
        _setPlay: function () {
            if ((this.model.showPlayButton) && (this._autoButton.hasClass("play")))
                this._autoButton.removeClass("play").addClass("pause");
            this._animate = true;
            this._raiseEvent("start");
        },
        /**
         * To configure Pause Action.	
		 * @private
         */
        _setPause: function () {
            if (this._interval != null) {
                if ((this.model.showPlayButton) && (this._autoButton.hasClass("pause")))
                    this._autoButton.removeClass("pause").addClass("play");
                clearInterval(this._interval);
                this._animate = this.model.stopOnHover ? true : false;
                this._interval = null;
                this._raiseEvent("stop");
            }
        },
        _setContainerPercent: function () {
            this._containerPercent = this._convertPixelToPercentage(this.element.parents('.e-rotator-wrap').parent().width(), this.element.parent().width());
        },
        /**
         * To configure animationType speed.	
		 * @private
         */
        _setSpeed: function (val) {
            this.transitionDelay = this.model.animationType != "slideshow" ? (val + this.model.delay) : ((val * 2) + this.model.delay);
        },
        /**
         * Section for set thumbnail element properties	
		 * @private
         */
        _setThumbProperty: function () {
            var thumbwidth, itemWidth, margin;
            itemWidth = this._thumbItems.children('li').outerWidth(true);
            thumbwidth = (this.element.parent().outerWidth(true)) - (((this.element.parent().outerWidth(true)) * 12) / 100);
            this._thumb.css('width', thumbwidth);
            this._thumbCount = Math.floor(this._thumb.innerWidth() / itemWidth);
            margin = Math.round((this.element.parent().outerWidth(true) * 6) / 100) + Math.floor((thumbwidth - (this._thumbCount * itemWidth)) / 2);
            this._thumb.css({ 'margin-left': margin, 'margin-right': margin, 'width': this._thumbCount * itemWidth });
            this._thumbItems.css({ 'width': (this._thumbItems.children('li').length) * itemWidth, 'height': this._thumbItems.children('li').outerHeight(true) });
            this._checkState();
        },
        _checkState: function () {
            if (this._thumbCount < (this._liCount - (this.model.displayItemsCount * 2))) {
                if (!this.element.parents('.e-in-wrap').siblings().hasClass("e-thumb-nav"))
                    this._createThumbControl();
            }
            else if (this.element.parents('.e-in-wrap').siblings().hasClass("e-thumb-nav"))
                this._thumbControl.remove();
        },
        /**
         * Section for removing active Rotator item.	
		 * @private
         */
        _removeActiveItem: function () {
            if (this.model.showPager)
                this._bullet.children().removeClass('e-active').attr({ 'tabindex': -1, 'aria-selected': "false" });
            if (this.model.showThumbnail)
                this._thumbItems.children().removeClass('e-active').attr({ 'tabindex': -1, 'aria-selected': "false" });
        },
        /**
         * Section for set active Rotator item.	
		 * @private
         */
        _setActiveItem: function (index) {
            if (this.model.showPager)
                this._bullet.children().eq(index).addClass('e-active').attr({ 'tabindex': 0, 'aria-selected': "true" });
            if (this.model.showThumbnail) {
                this._thumbItems.children().eq(index).addClass('e-active').attr({ 'tabindex': 0, 'aria-selected': "true" });
                this._transferThumb(index);
                for (i = 0; i < this._thumbItems.children('li').length; i++)
                    if (this._thumbItems.children('li').eq(i).hasClass('e-thumbhover'))
                        this._thumbItems.children('li').eq(i).removeClass('e-thumbhover');
            }
        },
        /**
         * Section for transfering thumbnail item to particular index.	
		 * @private
         */
        _transferThumb: function (index) {
            var move, right;
            if (this.model.showThumbnail) {
                move = (index * this._thumbItems.children('li').outerWidth(true)) - (-this._thumbItems.position().left);
                right = -(this._thumbItems.position().left) + this._thumb.width();
                if ((this._thumbItems.width() - right) > move)
                    this._thumbMove(move);
                else
                    this._thumbMove((this._thumbItems.width() - right));
            }
        },

        _changeThumbControl: function (index) {
            var licount;
            if ((this.model.showThumbnail) && (this.model.thumbnailSourceID != null)) {
                licount = $('#' + this.model.thumbnailSourceID).children('li').length;
                if ((index * this._thumbItems.children('li').outerWidth(true)) > 0) {
                    if (this._previous != null && this._previous.hasClass("e-disable"))
                        this._previous.removeClass("e-disable").addClass("e-enable").attr({ "aria-disabled": false });
                }
                else if (this._previous != null && this._previous.hasClass("e-enable"))
                    this._previous.removeClass("e-enable").addClass("e-disable").attr({ "aria-disabled": true });

                if ((index * this._thumbItems.children('li').outerWidth(true)) < ((licount - this._thumbCount) * (this._thumbItems.children('li').outerWidth(true)))) {
                    if (this._next != null && this._next.hasClass("e-disable"))
                        this._next.removeClass("e-disable").addClass("e-enable").attr({ "aria-disabled": false });
                }
                else if (this._next != null && this._next.hasClass("e-enable"))
                    this._next.removeClass("e-enable").addClass("e-disable").attr({ "aria-disabled": true });
            }
        },
        /**
        * Section for find maximum item size.	
        * @private
        */
        _getMaxSize: function (display) {
            var max = 0, current = 0, i = 0, proxy = this;
            for (i = 0; i < this._liCount; i++) {
                current = this.element.children('li').eq(i)[display]();
                max = current > max ? current : max;
            }
            return max;
        },
        _calculateValue: function () {
            return (this._convertPercentageToPixel((this._containerWidth - ((this.model.displayItemsCount - 1) * parseInt(this.model.frameSpace))), (100 / this.model.displayItemsCount)));
        },
        /**
         * Section for enable / disable the rotator control.	
		 * @private
         */
        _changeState: function (val) {
            val ? this.enable() : this.disable();
        },
        /**
         * Section for find active item.	
		 * @private
         */
        _findActive: function () {
            var index, i;
            if (this.model.showPager) {
                for (i = 0; i < this._bullet.children('li').length; i++) {
                    if (this._bullet.children('li').eq(i).hasClass('e-active'))
                        return i;
                }
            }
            else if (this.model.showThumbnail) {
                for (i = 0; i < this._thumbItems.children('li').length; i++) {
                    if (this._thumbItems.children('li').eq(i).hasClass('e-active'))
                        return i;
                }
            }
            else if (!(this.model.showPager) && !(this.model.showThumbnail))
                index = Math.round(-Math.round(this.element.position()[this.displayCss]) / this._liSize);
            index--;
            return index;
        },
        /**
        * Wiring the events to rotator control		
        * @private
        */
        _wireEvent: function () {
            if (!this.model.enabled) return false;
            this._wrapper.bind("mouseenter", $.proxy(this._showControl, this));
            this._wrapper.bind("mouseleave", $.proxy(this._hideControl, this));
            this._wireKeyboardEvents();
            this._wireResizeEvents();
            this._on(this._wrapper, "swiperight swipeleft swipedown swipeup", { control: true }, this._touchOperations);
            if (this.model.thumbnailSourceID) this._on(this._thumb, "swiperight swipeleft", { control: false }, this._touchOperations);
        },
        /**
        * Wiring the Keyboard events to rotator control		
        * @private
        */
        _wireKeyboardEvents: function () {
            if (this.model.allowKeyboardNavigation) {
                this._outerWrapper.bind("focusin", $.proxy(this._onFocusIn, this));
                this._outerWrapper.bind("focusout", $.proxy(this._onFocusOut, this));
            }
            else {
                this._outerWrapper.unbind("focusin", $.proxy(this._onFocusIn, this));
                this._outerWrapper.unbind("focusout", $.proxy(this._onFocusOut, this));
            }
        },
        /**
         * Wiring the events to slide button control		
		 * @private
         */
        _wireBtnEvents: function () {
            if (this.model.showNavigateButton) {
                this._prevButton.bind("click", $.proxy(this._prevAction, this));
                this._nextButton.bind("click", $.proxy(this._nextAction, this));
            }
            else {
                this._prevButton.unbind("click", $.proxy(this._prevAction, this));
                this._nextButton.unbind("click", $.proxy(this._nextAction, this));
            }
        },
        /**
         * Wiring the events to Play / pause button control		
		 * @private
         */
        _wireAutoPlayEvents: function () {
            if (this.model.showPlayButton)
                this._on(this._autoButton, "click", this._autoPlay);
            else
                this._off(this._autoButton, "click", this._autoPlay);
        },
        /**
         * Wiring the  Resize events to Rotator control		
		 * @private
         */
        _wireResizeEvents: function () {
            if (this.model.enableResize)
                $(window).bind("resize", $.proxy(this._resizeEvent, this));
            else
                $(window).unbind("resize", $.proxy(this._resizeEvent, this));
        },
        /**
         * Wiring the events to thumbnail items.		
		 * @private
         */
        _wireThumbEvents: function () {
            if (this.model.showThumbnail) {
                this._on(this._thumbItems.children(), "click", this._thumbClick);
                this._on(this._next, "click", this._thumbNext);
                this._on(this._previous, "click", this._thumbPrev);
            }
            else {
                this._off(this._thumbItems.children(), "click", this._thumbClick);
                this._off(this._next, "click", this._thumbNext);
                this._off(this._previous, "click", this._thumbPrev);
            }
        },
        /**
         * Wiring the events to showPager.		
		 * @private
         */
        _wireBulletEvents: function () {
            if (this.model.showPager)
                this._on(this._bullet.children(), "click", this._bulletClick);
            else
                this._off(this._bullet.children(), "click", this._bulletClick);
        },
        /**
        * Section for move to next thumbnail item.		
        * @private
        */
        _thumbNext: function () {
            var proxy = this, rightValue;
            if (!this.model.enabled) return false;
            if (this._previous.hasClass("e-disable"))
                this._previous.removeClass("e-disable").addClass("e-enable").attr({ "aria-disabled": false });
            rightValue = (-Math.round(this._thumbItems.position().left)) + this._thumb.width();
            if (this._thumbItems.width() - rightValue >= this._thumbItems.children('li').outerWidth(true))
                this._thumbMove(this._thumbItems.children('li').outerWidth(true));
            if ((this._thumbItems.width() - rightValue) <= this._thumbItems.children('li').outerWidth(true))
                this._next.removeClass("e-enable").addClass("e-disable").attr({ "aria-disabled": true });
        },
        /**
         * Section for move thumbnail item.		
		 * @private
         */
        _thumbMove: function (move) {
            var proxy = this;
            proxy._thumbItems.animate({
                "left": "-=" + move + "px",
            }, 600, function () { proxy._thumbItems.stop(true, true); });
        },
        /**
         * Section for move to previous thumbnail item.		
		 * @private
         */
        _thumbPrev: function () {
            var proxy = this, leftValue, itemWidth;
            if (!this.model.enabled) return false;
            if (this._next.hasClass("e-disable"))
                this._next.removeClass("e-disable").addClass("e-enable").attr({ "aria-disabled": false });
            leftValue = Math.round(this._thumbItems.position().left);
            itemWidth = this._thumbItems.children('li').outerWidth(true);
            if ((-leftValue) >= itemWidth) this._thumbMove(-itemWidth);
            if ((-leftValue) <= itemWidth) this._previous.removeClass("e-enable").addClass("e-disable").attr({ "aria-disabled": true });
        },
        _findLength: function () {
            var proxy = this, len;
            if (this.model.showThumbnail) len = proxy._thumbItems.children('li').length - 1;
            if (this.model.showPager) len = proxy._bullet.children('li').length - 1;
            if (!(this.model.showPager) && !(this.model.showThumbnail))
                len = ((this.element[this.containerCss]()) / this._liSize) - ((this.model.displayItemsCount) * 2);
            return len;
        },
        /**
         * Section for change Active State.		
		 * @private
         */
        _changeActiveState: function (action) {
            var proxy = this, index, len;
            index = proxy._findActive();
            len = proxy._findLength();
            proxy._removeActiveItem();
            if (action == "prev") {
                if ((index > 0) && (this.model.animationType != "slideshow")) {
                    proxy._setActiveItem((index - 1));
                    if (this.model.animationType != "slideshow") this._setCaptionText((index - 1));
                }
                else {
                    proxy._setActiveItem(len);
                    if (this.model.animationType != "slideshow") this._setCaptionText((len));
                }
            }
            else if (index < len) {
                proxy._setActiveItem((index + 1));
                if (this.model.animationType != "slideshow") this._setCaptionText((index + 1));
            }
            else {
                proxy._setActiveItem(0);
                if (this.model.animationType != "slideshow") this._setCaptionText(0);
            }
        },
        /**
        * Section for move slide.		
        * @private
        */
        _moveSlide: function (move, action) {
            var proxy = this, animationType, animationSpeed, index, len, properties = {};
            animationType = this.model.animationType != "slideshow" ? this.model.animationType : "slideshow";
            animationSpeed = this.model.animationType != "slideshow" ? proxy.model.animationSpeed : 0;
            index = proxy._findActive();
            len = proxy._findLength();
            properties[this.displayCss] = proxy.element.position()[proxy.displayCss];
            properties[this.displayCss] -= move;
            index = action == "next" ? (index < len ? index + 1 : 0) : action == "prev" ? (index > 0 ? index - 1 : len) : 0;
            if (this.model.animationType == "slideshow") { proxy.element.fadeOut(proxy.model.animationSpeed / 2); if (proxy.model.showCaption) proxy._caption.fadeOut(proxy.model.animationSpeed / 2); }
            proxy.element.animate(properties, animationSpeed, animationType, function () { if (action == "end") proxy._setBegin(); proxy._animating = false; proxy._setCaptionText(index); });
            if (this.model.animationType == "slideshow") { proxy.element.fadeIn(proxy.model.animationSpeed / 2); if (proxy.model.showCaption) proxy._caption.fadeIn(proxy.model.animationSpeed / 2); }
        },
        /**
        * Section for move to next slide.		
        * @private
        */
        _moveNext: function () {
            var proxy = this, ltValue, endValue, rbValue, move;
            proxy._animating = true;
            ltValue = Math.round(proxy.element.position()[proxy.displayCss]);
            endValue = (proxy.model.displayItemsCount * proxy._liSize) - (proxy._liCount * proxy._liSize);
            rbValue = Math.round(proxy.element.position()[proxy.displayCss]) + proxy.element[proxy.containerCss]();//here width() only
            proxy._calculateItemMove();
            if ((ltValue > endValue) && (rbValue - proxy._containerSize > (proxy._move + parseInt(proxy.model.frameSpace)))) {
                return { move: proxy._move, action: "next" }
            }
            else {
                if (rbValue - proxy._containerSize <= (proxy._move + parseInt(proxy.model.frameSpace))) {
                    move = rbValue - proxy._containerSize - parseInt(proxy.model.frameSpace);
                    return { move: move, action: "end" }
                }
            }
        },
        /**
         * Section for move to previous slide.		
		 * @private
         */
        _movePrev: function () {
            var proxy = this, ltValue, rbValue, endValue, setValue;
            ltValue = Math.round(proxy.element.position()[proxy.displayCss]);
            rbValue = proxy.element.position()[proxy.displayCss]; +proxy.element[proxy.containerCss]();//here using width() only
            endValue = proxy.element[proxy.containerCss]();//here using width() only
            proxy._calculateItemMove();
            if ((rbValue < endValue) && (-ltValue > (proxy.model.displayItemsCount * proxy._liSize))) {
                return { move: -proxy._move, action: "prev" }
            }
            else {
                setValue = ((proxy.model.displayItemsCount * proxy._liSize) * 2) + ltValue - (proxy._liCount * proxy._liSize);
                proxy._move = ((proxy._liCount - (proxy.model.displayItemsCount * 2)) % proxy.model.navigateSteps) * proxy._liSize;
                if (proxy._move == 0) proxy._calculateItemMove();
                if (proxy.model.orientation == ej.Orientation.Horizontal)
                    proxy.element.css("left", setValue);
                else
                    proxy.element.css("top", setValue);
                return { move: -proxy._move, action: "prev" }
            }
        },
        /**
         * Section for enable auto play mode.		
		 * @private
         */
        _autoPlay: function (e) {
            if (!this.model.enabled) return false;
            var proxy = this, value, move;
            if (this._interval == null) {
                this._setPlay();
                move = function () {
                    proxy._animating = true;
                    value = proxy.model.enableRTL ? proxy._moveNext() : proxy._movePrev();
                    proxy._moveSlide(value.move, value.action);
                    proxy._changeActiveState(value.action);
                    proxy._changeThumbControl(proxy._findActive());
                    proxy._raiseEvent("change");
                }
                function SetInterval(move, animationSpeed) {
                    move();
                    return (setInterval(move, animationSpeed))
                }
                if (!this._animating)
                    this._interval = SetInterval(move, this.transitionDelay);
            }
            else
                this._setPause();
        },
        /**
        * Section For handle the click event on thumbnail Next button 
        * @private
        */
        _nextAction: function () {
            if (!this.model.enabled) return false;
            var proxy = this, interval, next, value;
            next = function () {
                value = proxy._moveNext();
                proxy._moveSlide(value.move, value.action);
                clearInterval(interval);
                proxy._changeActiveState("next");
                proxy._changeThumbControl(proxy._findActive());
                proxy._raiseEvent("change");
            }
            if (this._interval != null)
                this._setPause();

            if (!this._animating) {
                this._animating = true;
                interval = setInterval(next, "");
            }
        },
        /**
       * Section For handle the click event on thumbnail Previous button 
       * @private
       */
        _prevAction: function () {
            if (!this.model.enabled) return false;
            var proxy = this, interval, previous, value;
            previous = function () {
                value = proxy._movePrev();
                proxy._moveSlide(value.move, value.action);
                clearInterval(interval);
                proxy._changeActiveState("prev");
                proxy._changeThumbControl(proxy._findActive());
                proxy._raiseEvent("change");
            }
            if (this._interval != null)
                this._setPause();

            if (!this._animating) {
                this._animating = true;
                interval = setInterval(previous, "")
            }
        },
        /**
       * Section For handle the touch event 
       * @private
       */
        _touchOperations: function (e) {
            var isThumb = !e.data.control, direction, orient = this.model.orientation;
            if (isThumb)
                direction = e.type == "swipeleft" ? "left" : "right";
            else {
                if ((orient == "horizontal" && e.type == "swipeleft") ||
                    (orient == "vertical" && e.type == "swipeup"))
                    direction = "left";
                else if ((orient == "horizontal" && e.type == "swiperight") ||
                    (orient == "vertical" && e.type == "swipedown"))
                    direction = "right";
            }
            if (direction == "left")
                isThumb ? this._thumbNext() : this.slideNext();
            else if (direction == "right")
                isThumb ? this._thumbPrev() : this.slidePrevious();
        },
        /**
        * Section For handle the resize event 
        * @private
        */
        _resizeEvent: function () {
            var index, parent;
            if (!this.model.enabled) return false;
            index = Math.round(-Math.round(this.element.position().left) / this._liSize);
            parent = Math.round(this.element.parents('.e-rotator-wrap').parent().width());
            this._containerWidth = this._convertPercentageToPixel(parent, this._containerPercent);
            this._setSlideWidth(this._calculateValue().toString());
            this._setWidth();
            this.element.css("left", -(index * this._liSize));
            this._setUlInitial();
            if ((this.model.showThumbnail) && (this.model.thumbnailSourceID != null))
                this._setThumbProperty();
        },
        /**
        * Section For Calculate item move value 
        * @private
        */
        _calculateItemMove: function () {
            if (!isNaN(this.model.navigateSteps))
                this._move = this._liSize * this.model.navigateSteps;
            else {
                this.model.navigateSteps = 1;
                this._move = this._liSize * this.model.navigateSteps;
            }
        },
        /**
         * Section For handle click event on thumbnail item
		 * @private
         */
        _thumbClick: function (event) {
            var index;
            var btnStatus = this._autoButton != null ? (this._autoButton.hasClass("play") ? false : true) : (this.model.enableAutoPlay ? true : false);
            if (!this.model.enabled) return false;

            if (this._interval != null)
                this._setPause();

            this._removeActiveItem();
            index = ((event.target.className == 'e-thumb-ele e-thumbhover') || (event.target.className == 'e-thumb-ele')) ? this._thumbItems.children('li').index($(event.target)) :
            this._thumbItems.children('li').index($(event.target).parents('.e-thumb-ele'));
            this._changeThumbControl(index);
            this._setActiveItem(index);
            this._raiseEvent("thumbItemClick");            
            this._moveSlideContent(this.element.position()[this.displayCss], -((this.model.displayItemsCount * this._liSize) + (index * this.model.navigateSteps * this._liSize)), index, btnStatus);
        },
        /**
         * Section For handle click event on pager item
		 * @private
         */
        _bulletClick: function (event) {
            var index;
            var btnStatus = this._autoButton != null ? (this._autoButton.hasClass("play") ? false : true) : (this.model.enableAutoPlay ? true : false);
            if (!this.model.enabled) return false;
            if (this._interval != null)
                this._setPause();
            this._removeActiveItem();
            index = this._bullet.children('').index(event.target);
            this._changeThumbControl(index);
            this._setActiveItem(index);
            this._moveSlideContent(this.element.position()[this.displayCss], -((this.model.displayItemsCount * this._liSize) + (index * this.model.navigateSteps * this._liSize)), index, btnStatus);
            this._raiseEvent("pagerClick");

        },
        /**
         * Section For handle slide transition
		 * @private
         */
        _moveSlideContent: function (Current, Target, index, btnStatus) {
            var proxy = this, animationSpeed, animationType, properties = {};
            animationType = this.model.animationType != "slideshow" ? this.model.animationType : "slideshow";
            animationSpeed = this.model.animationType != "slideshow" ? proxy.model.animationSpeed : 0;
            properties[this.displayCss] = proxy.element.position()[proxy.displayCss];
            properties[this.displayCss] += (Target - Current);
            if (this.model.animationType == "slideshow") { proxy.element.fadeOut(proxy.model.animationSpeed / 2); if (proxy.model.showCaption) proxy._caption.fadeOut(proxy.model.animationSpeed / 2); }
            proxy.element.animate(properties, animationSpeed, animationType, function () {
                proxy._setCaptionText(index); if (btnStatus) {
                    setTimeout(function () {
                        proxy.play();
                    }, proxy.model.animationSpeed);
                }
            });
            if (this.model.animationType == "slideshow") { proxy.element.fadeIn(proxy.model.animationSpeed / 2); if (proxy.model.showCaption) proxy._caption.fadeIn(proxy.model.animationSpeed / 2); }
            proxy._raiseEvent("change");

        },
        /**
       * destroy the Rotator widget.
       * all events bound using this._on will be unbind automatically and bring the control to pre-init state.
       * @private
       */
        _destroy: function () {
            this._interval != null ? this.pause() : "";
            this._undoClone();
            this.element.removeAttr("style").insertAfter(this._outerWrapper);
            this.element.children().removeAttr("style");
            this._outerWrapper.remove();
        },
        /**
          * Section For handle events
          * @private
          */
        _raiseEvent: function (event) {

            return this._trigger(event, {
                activeItemIndex: this._findActive(),
                itemID: this.element.attr('id'),
            });

        },
        /**
        * Section For show the slide button controls with animationType.
        * @private
        */
        _showControl: function (event) {
            if (!this.model.enabled) return false;
            if (this.model.showNavigateButton) {
                this._prevButton.stop(true, true).fadeIn('slow');
                this._nextButton.stop(true, true).fadeIn('slow');
            }
            if (this.model.showPlayButton)
                this._autoButton.stop(true, true).fadeIn('slow');
            if ((this.model.showCaption) && (this._caption != null))
                this._caption.stop(true, true).fadeIn('slow');
            if (this.model.stopOnHover && this._animate)
                this.pause();
            event.stopPropagation();
        },
        /**
        * Section For hide the slide button controls with animationType.
        * @private
        */
        _hideControl: function (event) {
            if (!this.model.enabled) return false;
            if (this.model.showNavigateButton) {
                this._prevButton.stop(true, true).fadeOut('slow');
                this._nextButton.stop(true, true).fadeOut('slow');
            }
            if (this.model.showPlayButton)
                this._autoButton.stop(true, true).fadeOut('slow');
            if ((this.model.showCaption) && (this._caption != null))
                this._caption.stop(true, true).fadeOut('slow');
            if (this.model.stopOnHover && this._animate)
                this.play();
            event.stopPropagation();
        },
        /**
        * Section For Handle focusin Event.
        * @private
        */
        _onFocusIn: function () {
            if (!this.model.enabled) return false;
            this._outerWrapper.addClass("e-Focused");
            $(this._outerWrapper).bind("keydown", $.proxy(this._focuseHandle, this));
        },
        /**
         * Section For Handle focusout in Event.
		 * @private
         */
        _onFocusOut: function () {
            this._outerWrapper.removeClass("e-Focused");
            $(this._outerWrapper).unbind("keydown", $.proxy(this._focuseHandle, this));
        },
        /**
         * Section For Handle keyboard navigation.
		 * @private
         */
        _focuseHandle: function (e) {
            if (!this.model.enabled) return false;
            if (this._outerWrapper.hasClass("e-Focused")) {
                e.keyCode = ((e.altKey) && ((e.keyCode == 37) || (e.keyCode == 40))) ? 1000 : e.keyCode;
                e.keyCode = ((e.altKey) && ((e.keyCode == 38) || (e.keyCode == 39))) ? 1001 : e.keyCode;
                switch (e.keyCode) {
                    case 37:        // Left Key
                    case 40:        // Down Key
                        e.preventDefault();
                        this._prevAction();
                        break;
                    case 38:        // Up Key
                    case 39:         // Right Key     
                        e.preventDefault();
                        this._nextAction();
                        break;
                    case 32:         // Space Key
                        e.preventDefault();
                        this._autoPlay();
                        break;
                    case 1000:
                        e.preventDefault();
                        this._thumbPrev();
                        this._thumbHover("prev");
                        break;
                    case 1001:
                        e.preventDefault();
                        this._thumbNext();
                        this._thumbHover("next");
                        break;
                    case 13:
                        e.preventDefault();
                        for (i = 0; i < this._thumbItems.children('li').length; i++)
                            if (this._thumbItems.children('li').eq(i).hasClass('e-thumbhover'))
                                this.gotoIndex(i);

                }
            }
        },
        /**
         * Section For Handle mouseover event on thumbnail item.
		 * @private
         */
        _thumbHover: function (action) {
            var proxy = this, index = null, len;
            for (i = 0; i < this._thumbItems.children('li').length; i++) {
                if (this._thumbItems.children('li').eq(i).hasClass('e-thumbhover')) {
                    index = i;
                    this._thumbItems.children('li').eq(i).removeClass('e-thumbhover');
                    break;
                }
            }
            if (index == null)
                index = this._findActive();
            len = proxy._findLength();
            if (action == "prev") {
                if (index > 0)
                    this._thumbItems.children().eq(--index).addClass('e-thumbhover');
                else
                    this._thumbItems.children().eq(0).addClass('e-thumbhover');
            }
            else if (index < len)
                this._thumbItems.children().eq(++index).addClass('e-thumbhover');
            else
                this._thumbItems.children().eq(len).addClass('e-thumbhover');
        },
        //Public Methods...
        /**
       * Enables the Rotator control.
       * @alias enable
       * @return jQuery
       * @example 
        * &lt;ul id="sliderContent"&gt;
   * &lt;li&gt;&lt;img src="../images/rotator/nature.jpg" /&gt;&lt;/li&gt;
   * &lt;li&gt;&lt;img src="../images/rotator/bird.jpg"/&gt;&lt;/li&gt;
   * &lt;li&gt;&lt;img src="../images/rotator/sculpture.jpg"/&gt;&lt;/li&gt;
   * &lt;li&gt;&lt;img src="../images/rotator/seaview.jpg" /&gt;&lt;/li&gt;
   * &lt;li&gt;&lt;img src="../images/rotator/snowfall.jpg"/&gt;&lt;/li&gt;
   *&lt;/ul&gt;
    * &lt;script&gt;
       * //initialize the Rotator object
       * $("#sliderContent").ejRotator();
       *	var slideObj = $("#sliderContent").data("ejRotator");
       *	//To Enables the Rotator control.
       *	slideObj.enable();
       * &lt;/script&gt;
       * @example 
        * &lt;ul id="sliderContent"&gt;
   * &lt;li&gt;&lt;img src="../images/rotator/nature.jpg" /&gt;&lt;/li&gt;
   * &lt;li&gt;&lt;img src="../images/rotator/bird.jpg"/&gt;&lt;/li&gt;
   * &lt;li&gt;&lt;img src="../images/rotator/sculpture.jpg"/&gt;&lt;/li&gt;
   * &lt;li&gt;&lt;img src="../images/rotator/seaview.jpg" /&gt;&lt;/li&gt;
   * &lt;li&gt;&lt;img src="../images/rotator/snowfall.jpg"/&gt;&lt;/li&gt;
   *&lt;/ul&gt;
    * &lt;script&gt;
       * $("#sliderContent").ejRotator();
       * $("#sliderContent").ejRotator("enable");
       * &lt;/script&gt;
       * @memberof ejRotator
       * @instance
       */
        enable: function () {
            if (!this.model.enabled) {
                this.model.enabled = true;
                this._outerWrapper.removeClass("e-disable").attr({ "aria-disabled": false });
            }
        },
        /**
       * Disables the Rotator control.
       * @alias disable
       * @return jQuery
       * @example 
        * &lt;ul id="sliderContent"&gt;
   * &lt;li&gt;&lt;img src="../images/rotator/nature.jpg" /&gt;&lt;/li&gt;
   * &lt;li&gt;&lt;img src="../images/rotator/bird.jpg"/&gt;&lt;/li&gt;
   * &lt;li&gt;&lt;img src="../images/rotator/sculpture.jpg"/&gt;&lt;/li&gt;
   * &lt;li&gt;&lt;img src="../images/rotator/seaview.jpg" /&gt;&lt;/li&gt;
   * &lt;li&gt;&lt;img src="../images/rotator/snowfall.jpg"/&gt;&lt;/li&gt;
   *&lt;/ul&gt;
    * &lt;script&gt;
       * //initialize the Rotator object
       	* $("#sliderContent").ejRotator();
       *	var slideObj = $("#sliderContent").data("ejRotator");
       *	//To Disables the Rotator control.
       *	slideObj.disable();
      * &lt;/script&gt;
      * @example 
        * &lt;ul id="sliderContent"&gt;
   * &lt;li&gt;&lt;img src="../images/rotator/nature.jpg" /&gt;&lt;/li&gt;
   * &lt;li&gt;&lt;img src="../images/rotator/bird.jpg"/&gt;&lt;/li&gt;
   * &lt;li&gt;&lt;img src="../images/rotator/sculpture.jpg"/&gt;&lt;/li&gt;
   * &lt;li&gt;&lt;img src="../images/rotator/seaview.jpg" /&gt;&lt;/li&gt;
   * &lt;li&gt;&lt;img src="../images/rotator/snowfall.jpg"/&gt;&lt;/li&gt;
   *&lt;/ul&gt;
    * &lt;script&gt;
       	* $("#sliderContent").ejRotator();
       * //To Disables the Rotator control.
       *  $("#sliderContent").ejRotator("disable");
      * &lt;/script&gt;
       * @memberof ejRotator
       * @instance
       */
        disable: function () {
            if (this.model.enabled) {
                this.model.enabled = false;
                this._outerWrapper.addClass("e-disable").attr({ "aria-disabled": true });
                if (this._interval != null)
                    this._setPause();
            }
        },
        /**
        * This method is used to move to the next slide from the current slide. If the current slide is the last slide, then the first slide will be treated as the next slide.
		* @alias slideNext
		* @return jQuery
		* @example 
		 * &lt;ul id="sliderContent"&gt;
   * &lt;li&gt;&lt;img src="../images/rotator/nature.jpg" /&gt;&lt;/li&gt;
   * &lt;li&gt;&lt;img src="../images/rotator/bird.jpg"/&gt;&lt;/li&gt;
   * &lt;li&gt;&lt;img src="../images/rotator/sculpture.jpg"/&gt;&lt;/li&gt;
   * &lt;li&gt;&lt;img src="../images/rotator/seaview.jpg" /&gt;&lt;/li&gt;
   * &lt;li&gt;&lt;img src="../images/rotator/snowfall.jpg"/&gt;&lt;/li&gt;
   *&lt;/ul&gt;
    * &lt;script&gt;
		* //initialize the Rotator object
		* $("#sliderContent").ejRotator();
		*	var slideObj = $("#sliderContent").data("ejRotator");
		*	//Moves to the next slide.
		*	slideObj.slideNext();
		 * &lt;/script&gt;
          * @example 
        * &lt;ul id="sliderContent"&gt;
   * &lt;li&gt;&lt;img src="../images/rotator/nature.jpg" /&gt;&lt;/li&gt;
   * &lt;li&gt;&lt;img src="../images/rotator/bird.jpg"/&gt;&lt;/li&gt;
   * &lt;li&gt;&lt;img src="../images/rotator/sculpture.jpg"/&gt;&lt;/li&gt;
   * &lt;li&gt;&lt;img src="../images/rotator/seaview.jpg" /&gt;&lt;/li&gt;
   * &lt;li&gt;&lt;img src="../images/rotator/snowfall.jpg"/&gt;&lt;/li&gt;
   *&lt;/ul&gt;
    * &lt;script&gt;
       	* $("#sliderContent").ejRotator();
       * //Moves to the next slide.
       *  $("#sliderContent").ejRotator("slideNext");
      * &lt;/script&gt;
		* @memberof ejRotator
		* @instance
        */
        slideNext: function () {
            if (this.model.enabled)
                this._nextAction();
        },
        /**
        * This method is used to move to the previous slide from the current slide. If the current slide is the first slide, then the last slide will be treated as the previous slide.
		* @alias slidePrevious
		* @return jQuery
		* @example 
		 * &lt;ul id="sliderContent"&gt;
   * &lt;li&gt;&lt;img src="../images/rotator/nature.jpg" /&gt;&lt;/li&gt;
   * &lt;li&gt;&lt;img src="../images/rotator/bird.jpg"/&gt;&lt;/li&gt;
   * &lt;li&gt;&lt;img src="../images/rotator/sculpture.jpg"/&gt;&lt;/li&gt;
   * &lt;li&gt;&lt;img src="../images/rotator/seaview.jpg" /&gt;&lt;/li&gt;
   * &lt;li&gt;&lt;img src="../images/rotator/snowfall.jpg"/&gt;&lt;/li&gt;
   *&lt;/ul&gt;
    * &lt;script&gt;
		* //initialize the Rotator object
			* $("#sliderContent").ejRotator();
		*	var slideObj = $("#sliderContent").data("ejRotator");
		*	//Moves to the previous slide.
		*	slideObj.slidePrevious();
		* &lt;/script&gt;
            * @example 
        * &lt;ul id="sliderContent"&gt;
   * &lt;li&gt;&lt;img src="../images/rotator/nature.jpg" /&gt;&lt;/li&gt;
   * &lt;li&gt;&lt;img src="../images/rotator/bird.jpg"/&gt;&lt;/li&gt;
   * &lt;li&gt;&lt;img src="../images/rotator/sculpture.jpg"/&gt;&lt;/li&gt;
   * &lt;li&gt;&lt;img src="../images/rotator/seaview.jpg" /&gt;&lt;/li&gt;
   * &lt;li&gt;&lt;img src="../images/rotator/snowfall.jpg"/&gt;&lt;/li&gt;
   *&lt;/ul&gt;
    * &lt;script&gt;
       	* $("#sliderContent").ejRotator();
       * //Moves to the previous slide.
       *  $("#sliderContent").ejRotator("slidePrevious");
      * &lt;/script&gt;
		* @memberof ejRotator
		* @instance
        */
        slidePrevious: function () {
            if (this.model.enabled)
                this._prevAction();
        },
        /**
        * This method is used to move slides continuously (or start autoplay) in the specified autoplay direction.
		* @alias play
		* @return jQuery
		* @example 
		 * &lt;ul id="sliderContent"&gt;
   * &lt;li&gt;&lt;img src="../images/rotator/nature.jpg" /&gt;&lt;/li&gt;
   * &lt;li&gt;&lt;img src="../images/rotator/bird.jpg"/&gt;&lt;/li&gt;
   * &lt;li&gt;&lt;img src="../images/rotator/sculpture.jpg"/&gt;&lt;/li&gt;
   * &lt;li&gt;&lt;img src="../images/rotator/seaview.jpg" /&gt;&lt;/li&gt;
   * &lt;li&gt;&lt;img src="../images/rotator/snowfall.jpg"/&gt;&lt;/li&gt;
   *&lt;/ul&gt;
    * &lt;script&gt;
		* //initialize the Rotator object
		* $("#sliderContent").ejRotator();
		*	var slideObj = $("#sliderContent").data("ejRotator");
		*	//To Starts auto play.
		*	slideObj.play();
		  * &lt;/script&gt;
        * @example 
        * &lt;ul id="sliderContent"&gt;
   * &lt;li&gt;&lt;img src="../images/rotator/nature.jpg" /&gt;&lt;/li&gt;
   * &lt;li&gt;&lt;img src="../images/rotator/bird.jpg"/&gt;&lt;/li&gt;
   * &lt;li&gt;&lt;img src="../images/rotator/sculpture.jpg"/&gt;&lt;/li&gt;
   * &lt;li&gt;&lt;img src="../images/rotator/seaview.jpg" /&gt;&lt;/li&gt;
   * &lt;li&gt;&lt;img src="../images/rotator/snowfall.jpg"/&gt;&lt;/li&gt;
   *&lt;/ul&gt;
    * &lt;script&gt;
       	* $("#sliderContent").ejRotator();
       *	//To Starts auto play.
       *  $("#sliderContent").ejRotator("play");
      * &lt;/script&gt;
		* @memberof ejRotator
		* @instance
        */
        play: function () {
            if (this._interval == null)
                this._autoPlay();
        },
        /**
        * This method is used to pause autoplay.
		* @alias pause
		* @return jQuery
		* @example 
		 * &lt;ul id="sliderContent"&gt;
   * &lt;li&gt;&lt;img src="../images/rotator/nature.jpg" /&gt;&lt;/li&gt;
   * &lt;li&gt;&lt;img src="../images/rotator/bird.jpg"/&gt;&lt;/li&gt;
   * &lt;li&gt;&lt;img src="../images/rotator/sculpture.jpg"/&gt;&lt;/li&gt;
   * &lt;li&gt;&lt;img src="../images/rotator/seaview.jpg" /&gt;&lt;/li&gt;
   * &lt;li&gt;&lt;img src="../images/rotator/snowfall.jpg"/&gt;&lt;/li&gt;
   *&lt;/ul&gt;
    * &lt;script&gt;
		* //initialize the Rotator object
		* $("#sliderContent").ejRotator();
		*	var slideObj = $("#sliderContent").data("ejRotator");
		*	//To Pauses auto play.
		*	slideObj.pause();	
        * &lt;/script&gt;
         * @example 
        * &lt;ul id="sliderContent"&gt;
   * &lt;li&gt;&lt;img src="../images/rotator/nature.jpg" /&gt;&lt;/li&gt;
   * &lt;li&gt;&lt;img src="../images/rotator/bird.jpg"/&gt;&lt;/li&gt;
   * &lt;li&gt;&lt;img src="../images/rotator/sculpture.jpg"/&gt;&lt;/li&gt;
   * &lt;li&gt;&lt;img src="../images/rotator/seaview.jpg" /&gt;&lt;/li&gt;
   * &lt;li&gt;&lt;img src="../images/rotator/snowfall.jpg"/&gt;&lt;/li&gt;
   *&lt;/ul&gt;
    * &lt;script&gt;
       	* $("#sliderContent").ejRotator();
       *	//To Pause auto play.
       *  $("#sliderContent").ejRotator("pause");
       * &lt;/script&gt;
		* @memberof ejRotator
		* @instance
        */
        pause: function () {
            if (this._interval != null)
                this._setPause();
        },
        /**
        * This method is used to get the current slide index.
		* @alias getIndex
		* @return number
		* @example 
		 * &lt;ul id="sliderContent"&gt;
   * &lt;li&gt;&lt;img src="../images/rotator/nature.jpg" /&gt;&lt;/li&gt;
   * &lt;li&gt;&lt;img src="../images/rotator/bird.jpg"/&gt;&lt;/li&gt;
   * &lt;li&gt;&lt;img src="../images/rotator/sculpture.jpg"/&gt;&lt;/li&gt;
   * &lt;li&gt;&lt;img src="../images/rotator/seaview.jpg" /&gt;&lt;/li&gt;
   * &lt;li&gt;&lt;img src="../images/rotator/snowfall.jpg"/&gt;&lt;/li&gt;
   *&lt;/ul&gt;
    * &lt;script&gt;
		* //initialize the Rotator object
		* $("#sliderContent").ejRotator();
		*	var slideObj = $("#sliderContent").data("ejRotator");
		*	// Gets the index value of the current slide.
		*	slideObj.getIndex();
		* &lt;/script&gt;
        * @example 
        * &lt;ul id="sliderContent"&gt;
   * &lt;li&gt;&lt;img src="../images/rotator/nature.jpg" /&gt;&lt;/li&gt;
   * &lt;li&gt;&lt;img src="../images/rotator/bird.jpg"/&gt;&lt;/li&gt;
   * &lt;li&gt;&lt;img src="../images/rotator/sculpture.jpg"/&gt;&lt;/li&gt;
   * &lt;li&gt;&lt;img src="../images/rotator/seaview.jpg" /&gt;&lt;/li&gt;
   * &lt;li&gt;&lt;img src="../images/rotator/snowfall.jpg"/&gt;&lt;/li&gt;
   *&lt;/ul&gt;
    * &lt;script&gt;
       	* $("#sliderContent").ejRotator();
       *	// Gets the index value of the current slide.
       *  $("#sliderContent").ejRotator("getIndex");
       * &lt;/script&gt;
		* @memberof ejRotator
		* @instance
        */
        getIndex: function () {
            var index, count, display = "", i;
            if (this.model.displayItemsCount > 1) {
                index = Math.round(-Math.round(this.element.position()[this.displayCss]) / this._liSize); index--;
                count = this._liCount - (this.model.displayItemsCount * 2);
                index = index - (this.model.displayItemsCount - 1);
                for (i = 0; i < this.model.displayItemsCount ; i++) {
                    if (index < count) {
                        display += "  " + index.toString(); +" ";
                        index++;
                    }
                    else {
                        index = 0; i--;
                    }
                }
                return display;
            }
            else
                return this._findActive();
        },
        /**
        * This method is used to move a slide to the specified index.
		* @alias getIndex
		* @return jQuery
		* @param {number} index  index of an slide
		* @example 
		 * &lt;ul id="sliderContent"&gt;
   * &lt;li&gt;&lt;img src="../images/rotator/nature.jpg" /&gt;&lt;/li&gt;
   * &lt;li&gt;&lt;img src="../images/rotator/bird.jpg"/&gt;&lt;/li&gt;
   * &lt;li&gt;&lt;img src="../images/rotator/sculpture.jpg"/&gt;&lt;/li&gt;
   * &lt;li&gt;&lt;img src="../images/rotator/seaview.jpg" /&gt;&lt;/li&gt;
   * &lt;li&gt;&lt;img src="../images/rotator/snowfall.jpg"/&gt;&lt;/li&gt;
   *&lt;/ul&gt;
    * &lt;script&gt;
		* //initialize the Rotator object
		 * $("#sliderContent").ejRotator();
		*	var slideObj = $("#sliderContent").data("ejRotator");
		*	// Moves the slide to the specified index.
		*	slideObj.gotoIndex(3);
		*  &lt;/script&gt;
        *  @example 
        * &lt;ul id="sliderContent"&gt;
   * &lt;li&gt;&lt;img src="../images/rotator/nature.jpg" /&gt;&lt;/li&gt;
   * &lt;li&gt;&lt;img src="../images/rotator/bird.jpg"/&gt;&lt;/li&gt;
   * &lt;li&gt;&lt;img src="../images/rotator/sculpture.jpg"/&gt;&lt;/li&gt;
   * &lt;li&gt;&lt;img src="../images/rotator/seaview.jpg" /&gt;&lt;/li&gt;
   * &lt;li&gt;&lt;img src="../images/rotator/snowfall.jpg"/&gt;&lt;/li&gt;
   *&lt;/ul&gt;
    * &lt;script&gt;
       	* $("#sliderContent").ejRotator();
       *	// Moves the slide to the specified index.
       *  $("#sliderContent").ejRotator("gotoIndex",3);
       * &lt;/script&gt;
		* @memberof ejRotator
		* @instance
        */
        gotoIndex: function (index) {
            var btnStatus = this._autoButton != null ? (this._autoButton.hasClass("play") ? false : true) : (this.model.enableAutoPlay ? true : false);
            if (this._interval != null)
                this._setPause();
            this._removeActiveItem();
            if (this.model.showThumbnail) this._changeThumbControl(index);
            this._setActiveItem(index);
            this._moveSlideContent(this.element.position()[this.displayCss], -((this.model.displayItemsCount * this._liSize) + (index * this.model.navigateSteps * this._liSize)), index, btnStatus);
        },
    })
    /**
    * Enum for pager position	 
    * @enum {string}
    * @global 
    */
    ej.Rotator.PagerPosition = {
        /**  Pager is positioned at the top-left corner of an Item. */
        TopLeft: "topleft",
        /**  Pager is positioned at the top-left corner of an Item. */
        TopRight: "topright",
        /**  Pager is positioned at the top-right corner of an Item. */
        BottomLeft: "bottomleft",
        /**  Pager is positioned at the bottom-left corner of an Item. */
        BottomRight: "bottomright",
        /**  Pager is positioned at the bottom-right corner of an Item. */
        TopCenter: "topCenter",
        /**  Pager is positioned outside an Item. */
        Outside: "outside"
    };

})(jQuery, Syncfusion);;
/**
* @fileOverview Plugin to style the Html Button elements
* @copyright Copyright Syncfusion Inc. 2001 - 2013. All rights reserved.
*  Use of this code is subject to the terms of our license.
*  A copy of the current license can be obtained at any time by e-mailing
*  licensing@syncfusion.com. Any infringement will be prosecuted under
*  applicable laws. 
* @version 12.1 
* @author <a href="mailto:licensing@syncfusion.com">Syncfusion Inc</a>
*/
(function ($, ej, undefined) {
    // ejSplitter is the plugin name 
    // "ej.Splitter" is "namespace.className" will hold functions and properties
    /**
    * @namespace ej
	* @class ejSplitter
	* @param {object} options - settings for Slider.
	* @requires jQuery
    * @requires jquery.easing.1.3.js
	* @requires ej.core.js
	* @requires ej.splitter.js
	* @classdesc Custom Design for Html splitter control.<br>
	* @example
	* &lt;div id="innerSplitter"&gt;
	* 	&lt;div&gt;
	* 		&lt;div class="cont"&gt;Pane 1 &lt;/div&gt;
	* 	&lt;/div&gt;
	* 	&lt;div&gt;
	* 		&lt;div class="cont"&gt;Pane 2 &lt;/div&gt;
	* 	&lt;/div&gt;
	* &lt;/div&gt; 
	* &lt;script&gt;
	* // Create Splitter <br>
    * $('#innerSplitter').ejSplitter(); <br>
	* &lt;/script&gt;
	*/
    ej.widget("ejSplitter", "ej.Splitter", {
        // widget element will be automatically set in this
        element: null,

        // user defined model will be automatically set in this
        model: null,
        validTags: ["div", "span"],
        _rootCSS: "e-splitter",
        _setFirst: false,

        // default model
        defaults: {
            /**
			* Specify the CSS class to splitter control to achieve custom theme.
			* @default null
			* @type {string}
			* @example
	* &lt;div id="innerSplitter"&gt;
	* 	&lt;div&gt;
	* 		&lt;div class="cont"&gt;Pane 1 &lt;/div&gt;
	* 	&lt;/div&gt;
	* 	&lt;div&gt;
	* 		&lt;div class="cont"&gt;Pane 2 &lt;/div&gt;
	* 	&lt;/div&gt;
	* &lt;/div&gt; 
	        * &lt;script&gt;
			* //To set cssClass API value during initialization
			* 	$("#innerSplitter").ejSplitter({ cssClass: "gradient-lime"});
            * &lt;/script&gt;
			* @memberof ejSplitter
			* @instance
			*/
            cssClass: "",
            /**
			* Specify the orientation for spliter control.See {@link orientation}
			* @default ej.orientation.Horizontal or “horizontal”
			* @type {Enum}
			* @example
	        * &lt;div id="innerSplitter"&gt;
	        * &lt;div&gt;
	        * &lt;div class="cont"&gt;Pane 1 &lt;/div&gt;
	        * &lt;/div&gt;
	        * &lt;div&gt;
	        * &lt;div class="cont"&gt;Pane 2 &lt;/div&gt;
	        * &lt;/div&gt;
	        * &lt;/div&gt; 
	        * &lt;script&gt;
			* //To set orientation API value during initialization  
			* $("#innerSplitter").ejSplitter({ orientation: ej.Orientation.Horizontal });
            * &lt;/script&gt;
			* @memberof ejSplitter
			* @instance
			*/
            orientation: "horizontal",
			enableAnimation: true,
			/**		
			* Specify properties for each pane.
			* @default null
			* @type {array}
			* @example 
	        * &lt;div id="innerSplitter"&gt;
	        * &lt;div&gt;
	        * &lt;div class="cont"&gt;Pane 1 &lt;/div&gt;
	        * &lt;/div&gt;
	        * &lt;div&gt;
	        * &lt;div class="cont"&gt;Pane 2 &lt;/div&gt;
	        * &lt;/div&gt;
	        * &lt;/div&gt; 
    	    * &lt;script&gt;
			* //To set properties API value during initialization  
			* 	$("#innerSplitter").ejSplitter({properties: [{ paneSize: "100px" }, { paneSize: "50px"}]});
            * &lt;/script&gt;
			* @memberof ejSplitter
			* @instance
			*/
            properties: [],
            /**		
			* Specify height for splitter control.
			* @default “100%”
			* @type {string}
			* @example 
	        * &lt;div id="innerSplitter"&gt;
	        * &lt;div&gt;
	        * &lt;div class="cont"&gt;Pane 1 &lt;/div&gt;
	        * &lt;/div&gt;
	        * &lt;div&gt;
	        * &lt;div class="cont"&gt;Pane 2 &lt;/div&gt;
	        * &lt;/div&gt;
	        * &lt;/div&gt; 
    	    * &lt;script&gt;
			* //To set height API value during initialization
			* 	$("#innerSplitter").ejSplitter({height: "100%" });
            * &lt;/script&gt;
			* @memberof ejSplitter
			* @instance
			*/
            height: null,
            /**
			* Specify width for splitter control.
			* @default “100%”
			* @type {string}
			* @example 
	        * &lt;div id="innerSplitter"&gt;
	        * &lt;div&gt;
	        * &lt;div class="cont"&gt;Pane 1 &lt;/div&gt;
	        * &lt;/div&gt;
	        * &lt;div&gt;
	        * &lt;div class="cont"&gt;Pane 2 &lt;/div&gt;
	        * &lt;/div&gt;
	        * &lt;/div&gt; 
    	    * &lt;script&gt;
			* //To set width API value during initialization  
			* 	$("#innerSplitter").ejSplitter({width: 600} );
    	    * &lt;/script&gt;
			* @memberof ejSplitter
			* @instance
			*/
            width: null,
            /**
			* Specify window resizing behaviour for splitter control.
			* @default true
			* @type {boolean}
			* @example
	        * &lt;div id="innerSplitter"&gt;
	        * &lt;div&gt;
	        * &lt;div class="cont"&gt;Pane 1 &lt;/div&gt;
	        * &lt;/div&gt;
	        * &lt;div&gt;
	        * &lt;div class="cont"&gt;Pane 2 &lt;/div&gt;
	        * &lt;/div&gt;
	        * &lt;/div&gt; 
    	    * &lt;script&gt;
			* //To set windowResizing API value during initialization
			* 	$("#innerSplitter").ejSplitter({enableAutoResize: true});
    	    * &lt;/script&gt;
			* @memberof ejSplitter
			* @instance
			*/
            enableAutoResize: false,
            /**
			* Specify Right to Left Direction for splitter control.
			* @default false
			* @type {boolean}
			* @example
	        * &lt;div id="innerSplitter"&gt;
	        * &lt;div&gt;
	        * &lt;div class="cont"&gt;Pane 1 &lt;/div&gt;
	        * &lt;/div&gt;
	        * &lt;div&gt;
	        * &lt;div class="cont"&gt;Pane 2 &lt;/div&gt;
	        * &lt;/div&gt;
	        * &lt;/div&gt; 
    	    * &lt;script&gt;
			* //To set enableRTL API value during initialization  
			* 	$("#innerSplitter").ejSplitter({enableRTL: true} );
    	    * &lt;/script&gt;
			* @memberof ejSplitter
			* @instance
			*/
            enableRTL: false,
            /**
			* Specify animation speed for the Splitter pane movement while collapsing and expanding.
			* @default 300
			* @type {number}
			* @example 
	        * &lt;div id="innerSplitter"&gt;
	        * &lt;div&gt;
	        * &lt;div class="cont"&gt;Pane 1 &lt;/div&gt;
	        * &lt;/div&gt;
	        * &lt;div&gt;
	        * &lt;div class="cont"&gt;Pane 2 &lt;/div&gt;
	        * &lt;/div&gt;
	        * &lt;/div&gt; 
    	    * &lt;script&gt;
			* //To set animationSpeed API value during initialization
			* $("#innerSplitter").ejSplitter({animationSpeed: 150 });
    	    * &lt;/script&gt;
			* @memberof ejSplitter
			* @instance
			*/
            animationSpeed: 300,

            //Events
            /**
			 * Fires when before expand collapse in splitter control.
			 * @event
			 * @name ejSplitter#beforeExpandCollapse
			 * @param {Object} argument Event parameters from splitter control
			 * @param {boolean}  argument.cancel if the event should be cancelled; otherwise, false.
			 * @param {object}  argument.collapsed returns collapsed pane details.
			 * @param {object}  argument.expanded returns expanded pane details.
			 * @param {object}  argument.model returns the splitter model.
			 * @param {number}  argument.splitbarIndex returns the current split bar index.			
			 * @param {string}  argument.type returns the name of the event.
			 * @example 
	        * &lt;div id="innerSplitter"&gt;
	        * &lt;div&gt;
	        * &lt;div class="cont"&gt;Pane 1 &lt;/div&gt;
	        * &lt;/div&gt;
	        * &lt;div&gt;
	        * &lt;div class="cont"&gt;Pane 2 &lt;/div&gt;
	        * &lt;/div&gt;
	        * &lt;/div&gt; 
    	    * &lt;script&gt;
			 * //beforeExpandCollapse event for splitter control
             * $("#innerSplitter").ejSplitter({
             *    beforeExpandCollapse: function (args) {}
             * });
    	    * &lt;/script&gt;
			 * @memberof ejSplitter
			 * @instance
			 */
            beforeExpandCollapse: null,
            /**
			 * Fires when expandCollapse in splitter control pane.
			 * @event
			 * @name ejSplitter#expandCollapse
			 * @param {Object} argument Event parameters from splitter control
			 * @param {boolean}  argument.cancel if the event should be cancelled; otherwise, false.
			 * @param {object}  argument.collapsed returns collapsed pane details.
			 * @param {object}  argument.expanded returns expanded pane details.
			 * @param {object}  argument.model returns the splitter model.
			 * @param {number}  argument.splitbarIndex returns the current split bar index.			 
			 * @param {string}  argument.type returns the name of the event.
			 * @example 
	        * &lt;div id="innerSplitter"&gt;
	        * &lt;div&gt;
	        * &lt;div class="cont"&gt;Pane 1 &lt;/div&gt;
	        * &lt;/div&gt;
	        * &lt;div&gt;
	        * &lt;div class="cont"&gt;Pane 2 &lt;/div&gt;
	        * &lt;/div&gt;
	        * &lt;/div&gt; 
    	    * &lt;script&gt;
			 * //expandCollapse event for splitter control
             * $("#innerSplitter").ejSplitter({
             *    expandCollapse: function (args) {}
             * });
    	    * &lt;/script&gt;
			 * @memberof ejSplitter
			 * @instance
			 */
            expandCollapse: null,
            /**
			 * Fires when resize in splitter control pane.
			 * @event
			 * @name ejSplitter#resize
			 * @param {Object} argument Event parameters from splitter control 
			 * @param {boolean}  argument.cancel if the event should be cancelled; otherwise, false.
			 * @param {object}  argument.collapsed returns collapsed pane details.
			 * @param {object}  argument.expanded returns expanded pane details.
			 * @param {object}  argument.model returns the splitter model.
			 * @param {number}  argument.splitbarIndex returns the current split bar index.			
			 * @param {string}  argument.type returns the name of the event.
			 * @example
	        * &lt;div id="innerSplitter"&gt;
	        * &lt;div&gt;
	        * &lt;div class="cont"&gt;Pane 1 &lt;/div&gt;
	        * &lt;/div&gt;
	        * &lt;div&gt;
	        * &lt;div class="cont"&gt;Pane 2 &lt;/div&gt;
	        * &lt;/div&gt;
	        * &lt;/div&gt; 
    	    * &lt;script&gt;
			 * //resize event for splitter control
             * $("#innerSplitter").ejSplitter({
             *    resize: function (args) {}
             * });
    	    * &lt;/script&gt;
			 * @memberof ejSplitter
			 * @instance
			 */
            resize: null,
            /**
			 * Fires when splitter control pane has been created. 
			 * @event
			 * @name ejSplitter#create
			 * @param {Object} argument Event parameters from splitter control 
			 * @param {boolean}  argument.cancel if the event should be cancelled; otherwise, false.			
			 * @param {object}  argument.model returns the splitter model.			 
			 * @param {string}  argument.type returns the name of the event.
			 * @example
	        * &lt;div id="innerSplitter"&gt;
	        * &lt;div&gt;
	        * &lt;div class="cont"&gt;Pane 1 &lt;/div&gt;
	        * &lt;/div&gt;
	        * &lt;div&gt;
	        * &lt;div class="cont"&gt;Pane 2 &lt;/div&gt;
	        * &lt;/div&gt;
	        * &lt;/div&gt; 
    	    * &lt;script&gt;
			 * //create event for splitter control
             * $("#innerSplitter").ejSplitter({
             *    create: function (args) {}
             * });
    	    * &lt;/script&gt;
			 * @memberof ejSplitter
			 * @instance
			 */
            create: null,
            /**
            * Fires when splitter control pane has been destroyed. 
            * @event
            * @name ejSplitter#destroy
            * @param {Object} argument Event parameters from splitter control 
            * @param {boolean}  argument.cancel if the event should be cancelled; otherwise, false.			
            * @param {object}  argument.model returns the splitter model.			 
            * @param {string}  argument.type returns the name of the event.
            * @example
           * &lt;div id="innerSplitter"&gt;
           * &lt;div&gt;
           * &lt;div class="cont"&gt;Pane 1 &lt;/div&gt;
           * &lt;/div&gt;
           * &lt;div&gt;
           * &lt;div class="cont"&gt;Pane 2 &lt;/div&gt;
           * &lt;/div&gt;
           * &lt;/div&gt; 
           * &lt;script&gt;
            * //destroy event for splitter control
            * $("#innerSplitter").ejSplitter({
            *    destroy: function (args) {}
            * });
           * &lt;/script&gt;
            * @memberof ejSplitter
            * @instance
            */
            destroy: null
        },
        /**
         * Specify the data types for default properties 
		 * @private
         */
        dataTypes: {
            cssClass: "string",
            orientation: "enum",
            properties: "data",
            enableAutoResize: "boolean",
            enableRTL: "boolean",
            animationSpeed: "number",
			enableAnimation: "boolean",
        },
		/**
         * To initialize the maskedit textbox control.		
		 * @private
         */
        // constructor function
        _init: function () {
            this._initialize();
            this._render();
            this._wireEvents(this.model.enableAutoResize);
        },
        /**
         * To configure the properties at runtime using SetModel		
		 * @private
         */
        _setModel: function (options) {
            var option;
            for (option in options) {
                switch (option) {
                    case "cssClass ": this._changeSkin(options[option]); break;
                    case "enableAutoResize": this._windowResizing(options[option]); break;
                    case "enableRTL": this._rtl(options[option]); break;
                    case "orientation": this._setOrientation(options[option]); break;
					case "enableAnimation":{
							this.model.enableAnimation=options[key];
							break;
					}
                }
            }
        },
        /**
        * To refresh the splitter control pane resizing.
		* @return jQuery
		* @example 
		* &lt;div id="innerSplitter"&gt;
		* &lt;div&gt;
		* &lt;div class="cont"&gt;Pane 1 &lt;/div&gt;
		* &lt;/div&gt;
		* &lt;div&gt;
		* &lt;div class="cont"&gt;Pane 2 &lt;/div&gt;
		* &lt;/div&gt;
		* &lt;/div&gt;
		* &lt;script&gt;
        * $("#innerSplitter").ejSplitter();
		* // Create splitter control
		* var splitterObj = $("#innerSplitter").data("ejSplitter");
		* splitterObj.refresh(); // refresh the splitter control pane resizing.
		* &lt;/script&gt;
		* @example 
		* &lt;div id="innerSplitter"&gt;
		* &lt;div&gt;
		* &lt;div class="cont"&gt;Pane 1 &lt;/div&gt;
		* &lt;/div&gt;
		* &lt;div&gt;
		* &lt;div class="cont"&gt;Pane 2 &lt;/div&gt;
		* &lt;/div&gt;
		* &lt;/div&gt;
		* &lt;script&gt;
        * $("#innerSplitter").ejSplitter();
		* // refresh the splitter control
		* $("#innerSplitter").ejSplitter("refresh");
		* &lt;/script&gt;
		* @memberof ejSplitter
		* @instance
         */
        refresh: function () {
            this._setPanesSize();
            this._getPanesPercent();
        },
        /**
        * To collapse the splitter control pane.
		* @return jQuery
		* @example 
		* &lt;div id="innerSplitter"&gt;
		* &lt;div&gt;
		* &lt;div class="cont"&gt;Pane 1 &lt;/div&gt;
		* &lt;/div&gt;
		* &lt;div&gt;
		* &lt;div class="cont"&gt;Pane 2 &lt;/div&gt;
		* &lt;/div&gt;
		* &lt;/div&gt;
		* &lt;script&gt;
        * $("#innerSplitter").ejSplitter();
		* // Create splitter control
		* var splitterObj = $("#innerSplitter").data("ejSplitter");
		* splitterObj.collapse(); // collpase the splitter control pane.
		* &lt;/script&gt;
		* @example 
		* &lt;div id="innerSplitter"&gt;
		* &lt;div&gt;
		* &lt;div class="cont"&gt;Pane 1 &lt;/div&gt;
		* &lt;/div&gt;
		* &lt;div&gt;
		* &lt;div class="cont"&gt;Pane 2 &lt;/div&gt;
		* &lt;/div&gt;
		* &lt;/div&gt;
		* &lt;script&gt;
        * $("#innerSplitter").ejSplitter();
		* // collpase the splitter control
		* $("#innerSplitter").ejSplitter("collpase");
		* &lt;/script&gt;
		* @memberof ejSplitter
		* @instance
         */
        collapse: function (paneIndex) {
            this._clickArrow(paneIndex, true, true);
        },
        /**
        * To expand the splitter control pane.
		* @return jQuery
		* @example 
		* &lt;div id="innerSplitter"&gt;
		* &lt;div&gt;
		* &lt;div class="cont"&gt;Pane 1 &lt;/div&gt;
		* &lt;/div&gt;
		* &lt;div&gt;
		* &lt;div class="cont"&gt;Pane 2 &lt;/div&gt;
		* &lt;/div&gt;
		* &lt;/div&gt;
		* &lt;script&gt;
        * $("#innerSplitter").ejSplitter();
		* // Create splitter control
		* var splitterObj = $("#innerSplitter").data("ejSplitter");
		* splitterObj.expand(); // expand the splitter control pane.
		* &lt;/script&gt;
		* @example 
		* &lt;div id="innerSplitter"&gt;
		* &lt;div&gt;
		* &lt;div class="cont"&gt;Pane 1 &lt;/div&gt;
		* &lt;/div&gt;
		* &lt;div&gt;
		* &lt;div class="cont"&gt;Pane 2 &lt;/div&gt;
		* &lt;/div&gt;
		* &lt;/div&gt;
		* &lt;script&gt;
        * $("#innerSplitter").ejSplitter();
		* // expand the splitter control
		* $("#innerSplitter").ejSplitter("expand");
		* &lt;/script&gt;
		* @memberof ejSplitter
		* @instance
         */
        expand: function (paneIndex) {
            this._clickArrow(paneIndex, false, true);
        },
        /**
         * To configure the arrow symbol in splitter control pane..
		 * @private
         */
        _clickArrow: function (index, bool, canClick) {
            if (this._inMovement || index < 0 || index > this.panes.length) return false;
            var arrow, cls = bool ? "e-collapse" : "e-expand", splitbars = this.element.children(".e-splitbar:not(.e-shadowbar)");

            if (index == splitbars.length) arrow = this._clickArrow(index - 1, !bool, false);
            else arrow = $(splitbars[index]).children("." + cls);

            if (canClick) arrow.css("display") != "none" && arrow.mousedown();
            else return arrow;
        },

        /**
        * To add the new pane to splitter control.
		* @return jQuery
		* @param {string}  content  content of pane. 
		* @param {object}  property  pane properties.
		* @param {number}  index  index of pane.
		* @example 
		* &lt;div id="innerSplitter"&gt;
		* &lt;div&gt;
		* &lt;div class="cont"&gt;Pane 1 &lt;/div&gt;
		* &lt;/div&gt;
		* &lt;div&gt;
		* &lt;div class="cont"&gt;Pane 2 &lt;/div&gt;
		* &lt;/div&gt;
		* &lt;/div&gt;
		* &lt;script&gt;
        * $("#innerSplitter").ejSplitter();
		* // Create splitter control
		* var splitterObj = $("#innerSplitter").data("ejSplitter");
		* splitterObj.addItem("New pane 0",{ paneSize:20, minSize:20, maxSize:100},0);
		* &lt;/script&gt;
		* @example 
		* &lt;div id="innerSplitter"&gt;
		* &lt;div&gt;
		* &lt;div class="cont"&gt;Pane 1 &lt;/div&gt;
		* &lt;/div&gt;
		* &lt;div&gt;
		* &lt;div class="cont"&gt;Pane 2 &lt;/div&gt;
		* &lt;/div&gt;
		* &lt;/div&gt;
		* &lt;script&gt;
        * $("#innerSplitter").ejSplitter();
		* // expand the splitter control
		* $("#innerSplitter").ejSplitter("addItem","New pane 0",{ paneSize:20, minSize:20, maxSize:100},0);
		* &lt;/script&gt;
		* @memberof ejSplitter
		* @instance
         */
        addItem: function (content, property, index) {
            var paneCount = this.panes.length;
            index = this._getNumber(index);
            if (ej.isNullOrUndefined(index)) index = paneCount;
            if (index < 0 || index > paneCount) return "";

            var property = this._getPaneProperty(property), paneDiv, paneDivSize, requiredSize;
            property=this._checkMinMaxSize(property);
            paneDiv = ej.buildTag("div.e-pane e-" + this.model.orientation.substr(0, 1) + "-pane");
            this.element.append(paneDiv[this.containerCss](property.paneSize));
            paneDivSize = property.paneSize = paneDiv[this.containerCss]();
            paneDiv.remove();

            var start = index, end = paneCount, i, j, insert, before, direction = 2, getters = {}, taken = 0, canInsert = false;
            requiredSize = paneDivSize + this._bar;

            for (j = 0; j < direction; j++) {
                for (i = start; i < end; i++) {
                    var _paneSize = $(this.panes[i])[this.containerCss]();
                    var availableSpace = _paneSize - this.model.properties[i].minSize;

                    if (availableSpace >= requiredSize - taken) {
                        getters[i] = _paneSize - (requiredSize - taken);
                        canInsert = true;
                        break;
                    }
                    else if (availableSpace > 0) {
                        getters[i] = this.model.properties[i].minSize;
                        taken += availableSpace;
                    }
                }
                if (canInsert) break;
                else end = start, start = 0;
            }
            if (!canInsert) return "";

            for (var pos in getters)
                $(this.panes[pos])[this.containerCss](getters[pos]);

            if (index == paneCount) {
                insert = "insertBefore", before = 1;
                paneDiv.insertAfter($(this.panes[index - 1]));
            }
            else {
                insert = "insertAfter", before = 0;
                paneDiv.insertBefore($(this.panes[index]));
            }
            this.model.properties.splice(index, 0, property);
            this.panes.splice(index, 0, paneDiv);
            var splitBar = this._createSplitBar(index - before);
            splitBar[insert](paneDiv);
            paneDiv.append(content);
            this._updateModel();

            return paneDiv;
        },

        /**
       * To remove the new pane from the splitter control.
       * @return jQuery
       * @param {number}  index  index of pane. 
       * @example 
       * &lt;div id="innerSplitter"&gt;
       * &lt;div&gt;
       * &lt;div class="cont"&gt;Pane 1 &lt;/div&gt;
       * &lt;/div&gt;
       * &lt;div&gt;
       * &lt;div class="cont"&gt;Pane 2 &lt;/div&gt;
       * &lt;/div&gt;
       * &lt;/div&gt;
       * &lt;script&gt;
        * $("#innerSplitter").ejSplitter();
       * // Create splitter control
       * var splitterObj = $("#innerSplitter").data("ejSplitter");
       * splitterObj.removeItem(0); 
       * &lt;/script&gt;
		* @example 
       * &lt;div id="innerSplitter"&gt;
       * &lt;div&gt;
       * &lt;div class="cont"&gt;Pane 1 &lt;/div&gt;
       * &lt;/div&gt;
       * &lt;div&gt;
       * &lt;div class="cont"&gt;Pane 2 &lt;/div&gt;
       * &lt;/div&gt;
       * &lt;/div&gt;
       * &lt;script&gt;
        * $("#innerSplitter").ejSplitter();
       * // expand the splitter control
       * $("#innerSplitter").ejSplitter("removeItem",0);
       * &lt;/script&gt;
       * @memberof ejSplitter
       * @instance
        */
        removeItem: function (index) {
            var paneCount = this.panes.length - 1;
            index = this._getNumber(index);
            if (ej.isNullOrUndefined(index)) index = paneCount;
            if (index < 0 || index > paneCount || paneCount < 1) return null;

            var targetPane, nextPane, splitbars, removedSize;
            targetPane = $(this.panes[index]);
            removedSize = targetPane[this.containerCss]() + this._bar;
            targetPane.remove();

            splitbars = this.element.children(".e-splitbar:not(.e-shadowbar)");
            if (index == paneCount) {
                nextPane = $(this.panes[index - 1]);
                $(splitbars[index - 1]).remove();
            }
            else {
                nextPane = $(this.panes[index + 1]);
                $(splitbars[index]).remove();
            }
            nextPane[this.containerCss](nextPane[this.containerCss]() + removedSize);

            this._removeArrays(index);
            this._updateModel();
        },
        _checkMinMaxSize:function(property){
            if (property.paneSize < property.minSize)
                property.paneSize = property.minSize;
            if (property.paneSize > property.maxSize)
                property.paneSize = property.maxSize;
            return property;
        },
        /**
		 * To remove the specified pane from the splitter control pane list.
		 * @private
         */
        _removeArrays: function (index) {
            this.model.properties.splice(index, 1);
            this.panes.splice(index, 1);
            this.oldPaneSize.splice(index, 1);
            this.oldPanePercent.splice(index, 1);
            this._sizePercent.splice(index, 1);
        },
        /**
         * To return the value as a valid one.
		 * @private
         */
        _getNumber: function (value) {
            value = parseFloat(value);
            return isNaN(value) ? null : value;
        },
        /**
         * To update the pane size in percentage.
		 * @private
         */
        _updateModel: function () {
            for (var i = 0; i < this.panes.length; i++)
                this.model.properties[i].paneSize = $(this.panes[i])[this.containerCss]();
            this._getPanesPercent();
        },
        /**
         * To get the pane properties.
		 * @private
         */
        _getPaneProperty: function (property) {
            var _default = { paneSize: 10, minSize: 10, maxSize: null, collapsible: true, resizable: true };
            return $.extend(_default, property);
        },
        /**
         * To configure the custom theme for splitter control using cssClass  property		
		 * @private
         */
        _changeSkin: function (skin) {
            this.element.removeClass(this.model.cssClass).addClass(skin);
        },
        /**
         * To configure the custom theme for splitter control using cssClass  property		
		 * @private
         */
        _windowResizing: function (boolean) {
            if (boolean) this._wireEvents(boolean);
            else this._unWireEvents();
        },
        /**
         * To configure the splitter control orientation.		
		 * @private
         */
        _setOrientation: function (boolean) {
            this._unWireEvents();
            this._destroy();
            this.model.orientation = boolean;
            this._init();
        },
        /**
         * To destroy the splitter control.
		 * @private
         */
        // all events bound using this._on will be unbind automatically
        _destroy: function () {
            this.element.removeClass("e-widget " + this.model.cssClass + " e-" + this.model.orientation);
            this.element.children(".e-splitbar").remove();
            this.element.children(".e-pane").removeClass("e-pane e-" + this.model.orientation.substr(0, 1) + "-pane").height("").width("");
        },
        /**
         * To initialize the splitter control with it's property.
		 * @private
         */
        _initialize: function () {
            this.panes = [];
            this.oldPaneSize = [];
            this.oldPanePercent = [];
            this.shadowBar = null;
            this._inMovement = false;
            this.containerCss = this.model.orientation == "horizontal" ? "width" : "height";
            this.displayCss = this.model.orientation == "horizontal" ? "left" : "top";
            this._bar = 7;      // For the center splitbar size
        },
        /**
         * Render section for the splitter control.
		 * @private
         */
        _render: function () {
            this.element.addClass("e-widget " + this.model.cssClass + " e-" + this.model.orientation).attr("data-role", "splitter");
            var i, j, target = this.element[0];
            for (i = 0, j = 0; i < target.children.length; i++) {
                $(target.children[i]).addClass("e-pane");
                this.panes.push(target.children[i]);
            }
            this._setPanesProperty();
            this._insertSplitBar();
            this._setDimentions();
            this._setPanesSize();
            this._getPanesPercent();
            this._checkProperties();
			this.element.find(".e-pane").addClass("e-" + this.model.orientation.substr(0, 1) + "-pane");
        },
        /**
         * To set the splitter control pane with it's properties.
		 * @private
         */
        _setPanesProperty: function () {
            for (var p = 0; p < this.panes.length; p++) {
                if (this.model.properties[p] != undefined) {
                    this.model.properties[p].paneSize = this.model.properties[p].paneSize == undefined ? "0px" : this.model.properties[p].paneSize;
                    this.model.properties[p].minSize = isNaN(parseFloat(this.model.properties[p].minSize)) ? 10 : parseFloat(this.model.properties[p].minSize);
                    this.model.properties[p].maxSize = isNaN(parseFloat(this.model.properties[p].maxSize)) ? null : parseFloat(this.model.properties[p].maxSize);
                    this.model.properties[p].collapsible = this.model.properties[p].collapsible != false ? true : false;
                    this.model.properties[p].resizable = this.model.properties[p].resizable != false ? true : false;
                }
                else this.model.properties.push({ paneSize: "0px", minSize: 10, maxSize: null, collapsible: true, resizable: true });
            }
        },
        /**
         * To configure the split bar in splitter control.
		 * @private
         */
        _insertSplitBar: function () {
            if (this.panes.length > 1) {
                var i, splitBar;
                for (i = 0; i < this.panes.length - 1; i++) {
                    splitBar = this._createSplitBar(i);
                    splitBar.insertAfter(this.panes[i]);
                }
            }
        },
        /**
         * To create the split bar in splitter control.
		 * @private
         */
        _createSplitBar: function (i) {
            var orient = this.model.orientation.substr(0, 1), arrow1, arrow2, splitBar, accessible = false;

            splitBar = ej.buildTag("span.e-box e-splitbar e-" + orient + "-bar").attr("aria-expanded", true);

            if (this.model.properties[i].collapsible && this.model.properties[i + 1].collapsible) {
                arrow1 = ej.buildTag("span.e-icon e-collapse e-" + orient + "-arrow " + orient + "-backward");
                splitBar.append(arrow1);
                arrow2 = ej.buildTag("span.e-icon e-expand e-" + orient + "-arrow " + orient + "-forward");
                splitBar.append(arrow2);
                accessible = true;

                this._on(arrow1, "mousedown", this._collapseArrowClick);
                this._on(arrow2, "mousedown", this._expandArrowClick);
            }
            if (this.model.properties[i].resizable && this.model.properties[i + 1].resizable) {
                splitBar.addClass("e-resize");
                accessible = true;
                this._on(splitBar, ej.eventType.mouseDown, this._mouseDownOnDivider);
            }
            if (accessible) {
                splitBar.attr({ "role": "separator", "tabindex": "0" });
                this._on(splitBar, "focus focusout", this._focusOnDivider);
            }
            else splitBar.attr({ "role": "presentation" });
            return splitBar;
        },
        /**
         * To obtained the pane size in percentage.
		 * @private
         */
        _getPanesPercent: function () {
            this._sizePercent = [];
            var size = this.element[this.containerCss](), outerSize = size - ((this.panes.length - 1) * this._bar), i;

            for (i = 0; i < this.panes.length; i++) {
                if (!$(this.panes[i]).hasClass("collapsed"))
                    this.oldPaneSize[i] = $(this.panes[i])[this.containerCss]();
                this.oldPanePercent[i] = this._convertToPercent(outerSize, this.oldPaneSize[i]);
                this._sizePercent.push(this._convertToPercent(outerSize, $(this.panes[i])[this.containerCss]()));
            }
        },
        /**
         * To set the dimension for splitter control.
		 * @private
         */
        _setDimentions: function () {
            if (this.model.height)
                this.element.css("height", this.model.height);

            if (this.model.width)
                this.element.css("width", this.model.width);
        },
        /**
         * To check the splitter control properties.
		 * @private
         */
        _checkProperties: function () {
            if (this.model.enableRTL) this._rtl(this.model.enableRTL);

            this._prevSize = this.element[this.containerCss]();
        },
        /**
         * To enable or disable the Right to Left behaviour 		
		 * @private
         */
        _rtl: function (boolean) {
            if (boolean) this.element.addClass("e-rtl");
            else this.element.removeClass("e-rtl");
        },
        /**
         * To set the pane size for splitter control. 		
		 * @private
         */
        _setPanesSize: function () {
            var attr = this.containerCss,
            zeroPanes = 0,
            totalPaneSize = 0,
            totalSize = this.element[attr](),
            remainZero = false,
            bar = this._bar,
            zerothPanes = [],
            panLength, j, paneCount = this.panes.length;

            if (paneCount > 1) {
                for (j = 0; j < paneCount ; j++) {
                    $(this.panes[j]).css(attr, this.model.properties[j].paneSize);
                    bar = (j == paneCount - 1) ? 0 : bar;
                    panLength = Math.round($(this.panes[j])[attr]());

                    if (panLength <= 0) {
                        zeroPanes++;
                        zerothPanes.push(j);
                        totalPaneSize += bar;
                    }
                    else {
                        if (remainZero) {
                            $(this.panes[j]).css(attr, 0);
                            totalPaneSize += panLength + bar;
                            this.model.properties[j].paneSize = 0;
                        }
                        else {
                            totalPaneSize += panLength + bar;
                            if (totalPaneSize > totalSize) {
                                totalPaneSize -= panLength - bar;
                                var currPane = totalSize - totalPaneSize,
                                remainDivider = paneCount - j - 1;
                                currPane -= remainDivider * bar;
                                $(this.panes[j]).css(attr, currPane);
                                remainZero = true;
                                totalPaneSize += currPane + bar;
                            }
                            this.model.properties[j].paneSize = panLength;
                        }
                    }
                }
            }
            else if (paneCount == 1) {
                $(this.panes[0]).css(attr, "100%");
                totalPaneSize = totalSize;
            }

            if (paneCount > 1 && totalPaneSize != totalSize) {
                var remainingSize, lastPane = $(this.panes[paneCount - 1]);
                if (totalPaneSize > totalSize) {
                    remainingSize = totalPaneSize - totalSize;
                    lastPane.css(attr, remainingSize);
                }
                else if (totalPaneSize < totalSize) {
                    remainingSize = totalSize - totalPaneSize;
                    if (zeroPanes > 0) {
                        var z, avgWid = Math.round(remainingSize / zeroPanes);
                        for (z = 0; z < zeroPanes ; z++) {
                            $(this.panes[zerothPanes[z]]).css(attr, avgWid);
                            this.model.properties[zerothPanes[z]].paneSize = avgWid;
                        }
                    }
                    else {
                        lastPane.css(attr, Math.round(lastPane[attr]() + remainingSize));
                        this.model.properties[paneCount - 1].paneSize = lastPane[attr]();
                    }
                }
            }
            if (paneCount > 1) this._checkPaneSize();
        },
        /**
         * To return the value based on it's type.		
		 * @private
         */
        _getUnit: function (str) {
            if (str == "px") return "px";
            else if (str == "pt") return "pt";
            else if (str.substr(1) == "%") return "%";
            else return "px";
        },
        /**
         * To configure the splitter position based on it's type.
		 * @private
         */
        _getNormalValue: function (position) {
            var currentLOB, currentLOBPercent, totalValue, currentValue; //currentLOT = current left or bottom
            if (this.model.orientation == "vertical") {
                currentLOB = position.y - this.element.offset().top;
                currentLOBPercent = currentLOB / this.element.outerHeight();
                totalValue = this.element.height();
            }
            else {
                currentLOB = position.x - this.element.offset().left;
                currentLOBPercent = currentLOB / this.element.outerWidth();
                totalValue = this.element.width();
            }
            if (currentLOBPercent > 1) {
                currentLOBPercent = 1;
            }
            if (currentLOBPercent < 0) {
                currentLOBPercent = 0;
            }

            currentValue = currentLOBPercent * totalValue;

            return this._trimValue(currentValue);
        },
        /**
         * To obtained the value based on step value.
		 * @private
         */
        _trimValue: function (value) {
            var step, stepModValue, correctedValue;
            step = 1;
            stepModValue = (value) % step;
            correctedValue = value - stepModValue;
            if (Math.abs(stepModValue) * 2 >= step)
                correctedValue += (stepModValue > 0) ? step : (-step);
            return parseFloat(correctedValue.toFixed(5));
        },
        /**
         * To get the split bar index. 		
		 * @private
         */
        _getSplitbarIndex: function () {
            return this.element.children(".e-splitbar:not(.e-shadowbar)").index(this.currentSplitBar);
        },
        /**
         * To configure the pane resize in splitter control. 		
		 * @private
         */
        _paneResize: function () {
            if (this.shadowBar == null) return false;
            this.currentSplitBar = this.shadowBar.next();
            var newPosition, prevPane, nextPane, prevPaneIndex, nextPaneIndex, index = this._getSplitbarIndex();
            prevPane = this.shadowBar.prev(), nextPane = this.currentSplitBar.next();
            prevPaneIndex = index, nextPaneIndex = index + 1;

            newPosition = this.shadowBar.offset()[this.displayCss];
            newPosition = newPosition - this.currentSplitBar.offset()[this.displayCss];
            $(prevPane).css(this.containerCss, newPosition + $(prevPane)[this.containerCss]() + "px");
            $(nextPane).css(this.containerCss, $(nextPane)[this.containerCss]() - newPosition + "px");
            this.oldPaneSize[prevPaneIndex] = $(prevPane)[this.containerCss]();
            this.oldPaneSize[nextPaneIndex] = $(nextPane)[this.containerCss]();

            this.shadowBar.remove();
            this._checkPaneSize();

            var prevObj = { item: prevPane, index: prevPaneIndex, size: this.oldPaneSize[prevPaneIndex] };
            var nextObj = { item: nextPane, index: nextPaneIndex, size: this.oldPaneSize[nextPaneIndex] };

            this._updateModelValue(prevObj, nextObj);

            this._trigger("resize", {
                prevPane: prevObj,
                nextPane: nextObj,
                splitbarIndex: index
            });
        },
        /**
         * To check whether pane size in splitter control. 		
		 * @private
         */
        _checkPaneSize: function () {
            var total = 0, len, i, splitterLen, paneCount = this.panes.length;
            for (i = 0; i < paneCount; i++) {
                len = $(this.panes[i])[this.containerCss]();
                total += len + this._bar;
            }
            total -= this._bar;
			splitterLen = (typeof window.getComputedStyle == "function") ? parseFloat(window.getComputedStyle(this.element[0])[this.containerCss]) : this.element[this.containerCss]()-1;
            if (total != splitterLen) {
                var remain = splitterLen - total;
                var last = $(this.panes[paneCount - 1])[this.containerCss]();
				if(last == 0){
				for (i = paneCount-1; i >= 0; i--) {
					 if($(this.panes[i]).hasClass("expanded") && !$(this.panes[i]).hasClass("collapsed")){
					 	last = $(this.panes[i])[this.containerCss]();
						$(this.panes[i]).css(this.containerCss, Math.floor(last + remain));
						break;}
            	}}
				else
                	$(this.panes[paneCount - 1]).css(this.containerCss, Math.floor(last + remain));
            }
        },
        /**
         * To configure the maximum draggable range for pane in splitter control.
		 * @private
         */
        _maxminDraggableRange: function (shadowbarPos) {
            var prevPane, nextPane, prevPaneSize, nextPaneSize, splitbarPosition, prevPaneRange, nextPaneRange,
                prevPaneIndex, nextPaneIndex, PaneMax1, PaneMax2, PaneMin1, PaneMin2, index;
            prevPane = this.shadowBar.prev();
            this.currentSplitBar = this.shadowBar.next();
            nextPane = this.currentSplitBar.next();
            prevPaneSize = prevPane[this.containerCss]();
            nextPaneSize = nextPane[this.containerCss]();
            splitbarPosition = this.displayCss == "left" ? this.currentSplitBar[0].offsetLeft : this.currentSplitBar[0].offsetTop;
            prevPaneRange = splitbarPosition - prevPaneSize;
            nextPaneRange = nextPaneSize + splitbarPosition;

            index = this._getSplitbarIndex();
            prevPaneIndex = index;
            nextPaneIndex = index + 1;

            PaneMax1 = this.model.properties[prevPaneIndex].maxSize;
            PaneMax2 = this.model.properties[nextPaneIndex].maxSize;
            PaneMax1 = PaneMax1 != null ? parseInt(PaneMax1, 10) : null;
            PaneMax2 = PaneMax2 != null ? parseInt(PaneMax2, 10) : null;

            this.model.properties[prevPaneIndex].minSize = parseInt(this.model.properties[prevPaneIndex].minSize, 10);
            this.model.properties[nextPaneIndex].minSize = parseInt(this.model.properties[nextPaneIndex].minSize, 10);
            PaneMin1 = this.model.properties[prevPaneIndex].minSize;
            PaneMin2 = this.model.properties[nextPaneIndex].minSize;

            this.shadowBar.removeClass("e-end-indicaton");

            // checking expand limit

            if (shadowbarPos > nextPaneRange - PaneMin2) {
                this.resizedPosition = nextPaneRange - PaneMin2;
                this.shadowBar.addClass("e-end-indicaton");
            }
            else if (shadowbarPos < prevPaneRange + PaneMin1) {
                this.resizedPosition = prevPaneRange + PaneMin1;
                this.shadowBar.addClass("e-end-indicaton");
            }

            if (PaneMax1 != null) {
                if (shadowbarPos > prevPaneRange + PaneMax1) {
                    this.resizedPosition = prevPaneRange + PaneMax1;
                    this.shadowBar.addClass("e-end-indicaton");
                }
            }

            if (PaneMax2 != null) {
                if (shadowbarPos < nextPaneRange - PaneMax2) {
                    this.resizedPosition = nextPaneRange - PaneMax2;
                    this.shadowBar.addClass("e-end-indicaton");
                }
            }
        },
        /**
         * To configure the arrow click in collapse time 	.
		 * @private
         */
        _collapseArrowClick: function (event) {
            if (this.shadowBar != null) return;
            var $target = $(event.target);
            this._inMovement = true;
            this.currentSplitBar = $target.parent();
            var currBarNo, prevPane, nextPane, prevPaneIndex, nextPaneIndex, prevPaneSize, nextPaneSize, properties = {};
            var paneCount = this.panes.length;
            currBarNo = this._getSplitbarIndex();
            prevPane = this.currentSplitBar.prev();
            nextPane = this.currentSplitBar.next();
            prevPaneIndex = currBarNo;
            nextPaneIndex = currBarNo + 1;
            prevPaneSize = prevPane[this.containerCss]();
            nextPaneSize = nextPane[this.containerCss]();

            var proxy = this, collapsed, expanded;
            collapsed = { item: prevPane, index: prevPaneIndex, size: prevPaneSize };
            expanded = { item: nextPane, index: nextPaneIndex, size: nextPaneSize };

            if (this._raiseEvent("beforeExpandCollapse", collapsed, expanded, currBarNo))
                return false;

            if (!nextPane.hasClass("collapsed")) {
                this.oldPaneSize[prevPaneIndex] = prevPaneSize;

                prevPane.addClass("collapsed");
                nextPane.addClass("expanded");
                this.currentSplitBar.attr("aria-expanded", false);
                $target.parent().removeClass("e-resize");
                $target.css("display", "none");

                if (prevPaneIndex != 0) {
                    var preBar = prevPane.prev();
                    preBar.find(".e-expand").css("display", "none");
                    preBar.removeClass("e-resize");
                }

                properties[this.containerCss] = 0;
                prevPane.animate(properties,this.model.enableAnimation?this.model.animationSpeed:0);
                properties[this.containerCss] = prevPaneSize + nextPaneSize;
                nextPane.animate(properties,this.model.enableAnimation?this.model.animationSpeed:0, function () {
                    proxy._raiseEvent("expandCollapse", collapsed, expanded, currBarNo);
                });
            }
            else {
                if (prevPaneSize < this.oldPaneSize[nextPaneIndex]) {
                    $target.addClass("e-end-indicaton");
                    this._inMovement = false;
                    $(document).bind("mouseup", $.proxy(this._mouseUpOnArrow, this));
                    return false;
                }
                else {
                    prevPane.removeClass("expanded");
                    nextPane.removeClass("collapsed");
                    $target.parent().addClass("e-resize");
                    $target.siblings().css("display", "block");

                    if (nextPaneIndex != paneCount - 1) {
                        var nextBar = nextPane.next();
                        nextBar.find(".e-collapse").css("display", "block");
                        if (!nextBar.next().hasClass("collapsed")) {
                            nextBar.addClass("e-resize");
                            nextBar.attr("aria-expanded", true);
                        }
                    }

                    properties[this.containerCss] = this.oldPaneSize[nextPaneIndex];
                    nextPane.animate(properties, this.model.enableAnimation?this.model.animationSpeed:0);
                    properties[this.containerCss] = prevPaneSize - this.oldPaneSize[nextPaneIndex];
                    prevPane.animate(properties,this.model.enableAnimation?this.model.animationSpeed:0, function () {
                        proxy._raiseEvent("expandCollapse", collapsed, expanded, currBarNo);
                    });
                }
            }
        },
        /**
         * To configure the arrow click in expand time 	.
		 * @private
         */
        _expandArrowClick: function (event) {
            if (this.shadowBar != null) return;
            var $target = $(event.target);
            this._inMovement = true;
            this.currentSplitBar = $target.parent();
            var currBarNo, prevPane, nextPane, prevPaneIndex, nextPaneIndex, prevPaneSize, nextPaneSize, properties = {};
            var paneCount = this.panes.length;
            currBarNo = this._getSplitbarIndex();
            prevPane = this.currentSplitBar.prev();
            nextPane = this.currentSplitBar.next();
            prevPaneIndex = currBarNo;
            nextPaneIndex = currBarNo + 1;
            prevPaneSize = prevPane[this.containerCss]();
            nextPaneSize = nextPane[this.containerCss]();

            var proxy = this, collapsed, expanded;
            collapsed = { item: nextPane, index: nextPaneIndex, size: nextPaneSize };
            expanded = { item: prevPane, index: prevPaneIndex, size: prevPaneSize };

            if (this._raiseEvent("beforeExpandCollapse", collapsed, expanded, currBarNo))
                return false;

            if (!prevPane.hasClass("collapsed")) {
                this.oldPaneSize[nextPaneIndex] = nextPaneSize;

                prevPane.addClass("expanded");
                nextPane.addClass("collapsed");
                $target.parent().removeClass("e-resize");
                $target.css("display", "none");

                if (nextPaneIndex != paneCount - 1) {
                    var nextBar = nextPane.next();
                    nextBar.find(".e-collapse").css("display", "none");
                    nextBar.removeClass("e-resize");
                    nextBar.attr("aria-expanded", false);
                }
                
                properties[this.containerCss] = prevPaneSize + nextPaneSize;
                prevPane.animate(properties, this.model.enableAnimation?this.model.animationSpeed:0);
                properties[this.containerCss] = 0;
                nextPane.animate(properties, this.model.enableAnimation?this.model.animationSpeed:0, function () {
                    proxy._raiseEvent("expandCollapse", collapsed, expanded, currBarNo);
                });
            }
            else {
                if (nextPaneSize < this.oldPaneSize[prevPaneIndex]) {
                    $target.addClass("e-end-indicaton");
                    this._inMovement = false;
                    $(document).bind("mouseup", $.proxy(this._mouseUpOnArrow, this));
                    return false;
                }
                else {
                    prevPane.removeClass("collapsed");
                    nextPane.removeClass("expanded");
                    this.currentSplitBar.attr("aria-expanded", true);
                    $target.parent().addClass("e-resize");
                    $target.siblings().css("display", "block");

                    if (prevPaneIndex != 0) {
                        var preBar = prevPane.prev();
                        preBar.find(".e-expand").css("display", "block");
                        if (!preBar.prev().hasClass("collapsed")) preBar.addClass("e-resize");
                    }

                    properties[this.containerCss] = this.oldPaneSize[prevPaneIndex];
                    prevPane.animate(properties, this.model.enableAnimation?this.model.animationSpeed:0);
                    properties[this.containerCss] = nextPaneSize - this.oldPaneSize[prevPaneIndex];
                    nextPane.animate(properties, this.model.enableAnimation?this.model.animationSpeed:0, function () {
                        proxy._raiseEvent("expandCollapse", collapsed, expanded, currBarNo);
                    });
                }
            }
        },
        /**
         * To raise the event when it's triggered. 	.
		 * @private
         */
        _raiseEvent: function (evtName, collapsed, expanded, index) {
            if (evtName == "expandCollapse") {
                this._inMovement = false;
                this._updateModelValue(collapsed, expanded);
            }


            return this._trigger(evtName, {
                collapsed: collapsed,
                expanded: expanded,
                splitbarIndex: index
            });

        },
        /**
         * To update the model value for splitter control.
		 * @private
         */
        _updateModelValue: function (collapsed, expanded) {
            this.model.properties[collapsed.index].paneSize = collapsed.item[this.containerCss]();
            this.model.properties[expanded.index].paneSize = expanded.item[this.containerCss]();

            this._getPanesPercent();
        },
        /**
         * Section For handle the mouse up event over the arrow.
		 * @private
         */
        _mouseUpOnArrow: function (event) {
            this.element.find(".e-end-indicaton").removeClass("e-end-indicaton");
            $(document).unbind("mouseup", $.proxy(this._mouseUpOnArrow, this));
        },
        /**
         * Section For handle the key down event on splitter control pane divider.
		 * @private
         */
        _keydownOnDivider: function (e) {
            var key = e.keyCode;
            if (key == 37 || key == 38 || key == 39 || key == 40) {
                e.preventDefault();
                var oriTarget = $(e.data.target);
                if (e.ctrlKey) {
                    if (this.shadowBar == null) {
                        this.currentSplitBar = oriTarget;
                        var index = this._getSplitbarIndex();
                        if (this.model.orientation == "vertical") {
                            if (e.keyCode == 38) this.collapse(index);       // Up Key
                            else if (e.keyCode == 40) this.expand(index);    // Down Key
                        }
                        else {
                            if (e.keyCode == 37) this.collapse(index);       // Left Key
                            else if (e.keyCode == 39) this.expand(index);    // Right Key
                        }
                    }
                }
                else if (oriTarget.hasClass("e-resize")) {
                    var target = (this.shadowBar != null) ? this.shadowBar : oriTarget;
                    var offset = target.offset(), location = { pageX: offset.left, pageY: offset.top };
                    $.extend(true, e, location);
                    if ((this.model.orientation == "vertical" && (e.keyCode == 38 || e.keyCode == 40)) ||
                        (this.model.orientation == "horizontal" && (e.keyCode == 37 || e.keyCode == 39))) {
                        if (e.keyCode == 38) e.pageY -= 5;          // Up Key
                        else if (e.keyCode == 40) e.pageY += 5;     // Down Key
                        else if (e.keyCode == 37) e.pageX -= 5;     // Left Key
                        else if (e.keyCode == 39) e.pageX += 5;     // Right Key
                        this._mouseMoveOnDivider(e);
                    }
                }
            }
            else if (key == 13) {    // Enter Key
                e.preventDefault();
                this._mouseUpOnDivider();
            }
            else if (key == 27) {    // Esc Key
                e.preventDefault();
                if (this.shadowBar != null) this.shadowBar.remove();
                this.shadowBar = null;
                this._mouseUpOnDivider();
                this.element.children(".e-splitbar.e-hover").focusout();
            }
        },
        /**
         * Section For handle the focused in splitter control pane divider.
		 * @private
         */
        _focusOnDivider: function (e) {
            if (e.type == "focus") {
                if (!$(e.target).hasClass("e-hover")) {
                    $(e.target).addClass("e-hover");
                    $(document).bind("keydown", { target: e.target }, $.proxy(this._keydownOnDivider, this));
                }
            }
            else {
                this.element.children(".e-splitbar.e-hover").removeClass("e-hover");
                this._mouseUpOnDivider();
                $(document).unbind("keydown", $.proxy(this._keydownOnDivider, this));
            }
        },
        /**
         * Section For handle the mouse down over splitter control pane divider.
		 * @private
         */
        _mouseDownOnDivider: function (event) {
            event.preventDefault();
            var $target = $(event.target);
            if ($target.hasClass("e-splitbar") && $target.hasClass("e-resize")) {
                if (!$target.hasClass("e-hover")) $target.focus();
                $(document).bind(ej.eventType.mouseMove, { target: event.target }, $.proxy(this._mouseMoveOnDivider, this));
                $(document).bind(ej.eventType.mouseUp, $.proxy(this._mouseUpOnDivider, this));
                $(document).bind("mouseleave", $.proxy(this._mouseUpOnDivider, this));
            }
            else if ($target.hasClass("e-expand") || $target.hasClass("e-collapse")) {
                $target.parent().focus();
            }
        },
        /**
         * Section For handle the mouse move over splitter control pane divider.
		 * @private
         */
        _mouseMoveOnDivider: function (event) {
            event = event.type == "touchmove" ? event.originalEvent.changedTouches[0] : event;
            var position = { x: event.pageX, y: event.pageY };
            this.resizedPosition = this._getNormalValue(position);

            if (this.shadowBar == null) {
                var target = $(event.data.target);
                this.shadowBar = target.clone().addClass("e-shadowbar").removeClass("e-hover").insertBefore(target);
                this.shadowBar.children().remove();
            }
            this._maxminDraggableRange(this.resizedPosition);
            this.shadowBar.css(this.displayCss, this.resizedPosition);
        },
        /**
         * Section For handle the mouse up over splitter control pane divider.
		 * @private
         */
        _mouseUpOnDivider: function (event) {
            this._paneResize();

            $(document).unbind(ej.eventType.mouseMove, $.proxy(this._mouseMoveOnDivider, this));
            $(document).unbind(ej.eventType.mouseUp, $.proxy(this._mouseUpOnDivider, this));
            $(document).unbind("mouseleave", $.proxy(this._mouseUpOnDivider, this));
            // sets shadowBar null after removing shadowBar element
            this.shadowBar = null;
        },
        /**
         * Section For handle the window resized.
		 * @private
         */
        _windowResized: function (event) {
			var size = (typeof window.getComputedStyle == "function") ? parseFloat(window.getComputedStyle(this.element[0])[this.containerCss]) : this.element[this.containerCss]()-1;
            if (this._prevSize == size) return false;

            var paneCount = this.panes.length, outerSize = size - ((paneCount - 1) * this._bar), i, val;
            this._prevSize = size;

            for (i = 0; i < paneCount; i++) {
                val = this._convertToPixel(outerSize, this._sizePercent[i]);
                $(this.panes[i]).css(this.containerCss, val + "px");
            }
            for (i = 0; i < this.oldPaneSize.length; i++)
                this.oldPaneSize[i] = this._convertToPixel(outerSize, this.oldPanePercent[i]);

            this._checkPaneSize();
        },

        _convertToPercent: function (outer, pane) {
            return (pane * 100) / outer;
        },
        _convertToPixel: function (tot, percent) {
            return Math.floor((tot * percent) / 100);
        },

        //-------------------- Event Wire-up -------------------------//
        /**
         * Wiring the events to splitter control		
		 * @private
         */
        _wireEvents: function (boolean) {
            if (boolean) $(window).bind('resize', $.proxy(this._windowResized, this));
        },
        /**
         * unWiring the events from splitter control.		
		 * @private
         */
        _unWireEvents: function () {
            $(window).unbind('resize', $.proxy(this._windowResized, this));
        }
    });

})(jQuery, Syncfusion);;
/**
* @fileOverview Plugin to style the tab control.
* @copyright Copyright Syncfusion Inc. 2001 - 2013. All rights reserved.
*  Use of this code is subject to the terms of our license.
*  A copy of the current license can be obtained at any time by e-mailing
*  licensing@syncfusion.com. Any infringement will be prosecuted under
*  applicable laws. 
* @version 12.1 
* @author <a href="mailto:licensing@syncfusion.com">Syncfusion Inc</a>
*/
(function ($, ej, undefined) {
    /**
    * @namespace ej
	* @class ejTab
	* @requires jQuery
	* @requires jquery.easing.1.3.min.js
	* @requires ej.core.js
	* @requires ej.tab.js

	* @classdesc Custom Design for Tab control.
	*@example
	 
	*&lt;div id="tab"&gt;                  
	*&lt;ul&gt;                      
	*&lt;li&gt;&lt;a href="#javaScript"&gt;JavaScript&lt;/a&gt;&lt;/li&gt;                      
	*&lt;li&gt;&lt;a href="#cSharp"&gt;C Sharp (C#)&lt;/a&gt;&lt;/li&gt;                      
	*&lt;li&gt;&lt;a href="#vb"&gt;VB.Net&lt;/a&gt;&lt;/li&gt;                  
	*&lt;/ul&gt;                  
	*&lt;div id="javaScript"&gt; JavaScript (JS) is an interpreted computer programming language. 
	*It was originally implemented as part of web browsers so that client-side scripts could interact with the user, control the browser, communicate asynchronously, and alter the document content that was displayed. More recently, however, it has become common in both game development and the creation of desktop applications.                  
	*&lt;/div&gt;                  
	*&lt;div id="cSharp"&gt; C# is intended to be a simple, modern, general-purpose, object-oriented programming language. Its development team is led by Anders Hejlsberg. The most recent version is C# 5.0, which was released on August 15, 2012.                  
	*&lt;/div&gt;                  
	*&lt;div id="vb"&gt; The command-line compiler, VBC.EXE, is installed as part of the freeware .NET Framework SDK. Mono also includes a command-line VB.NET compiler. The most recent version is VB 2012, which was released on August 15, 2012.                  
	*&lt;/div&gt;              
	*&lt;/div&gt;
	* &lt;script type="text/javascript"&gt;         
	*$(function () {             
	* // document ready            
	* // Initialize Tab control creation             
	* $("#tab").ejTab({width:"300px"});         
	*}); &lt;/script&gt; 	
	*/
    ej.widget("ejTab", "ej.Tab", {
        _rootCSS: "e-tab",
        // widget element will be automatically set in this
        element: null,

        // user defined model will be automatically set in this
        model: null,
        validTags: ["div", "span"],
        _setFirst: false,

        // default model
        defaults: {
            /// <summary>This Contains default property of Sycnfusion Tab control </summary>
            /**		
           * Allow to collapsing the active item, while click on the active header.
           * @default false
           * @type {boolean}
           * @example 
           *&lt;div id="tab"&gt;                  
  *&lt;ul&gt;                      
  *&lt;li&gt;&lt;a href="#javaScript"&gt;JavaScript&lt;/a&gt;&lt;/li&gt;                      
  *&lt;li&gt;&lt;a href="#cSharp"&gt;C Sharp (C#)&lt;/a&gt;&lt;/li&gt;                      
  *&lt;li&gt;&lt;a href="#vb"&gt;VB.Net&lt;/a&gt;&lt;/li&gt;                  
  *&lt;/ul&gt;                  
  *&lt;div id="javaScript"&gt; JavaScript (JS) is an interpreted computer programming language. 
  *It was originally implemented as part of web browsers so that client-side scripts could interact with the user, control the browser, communicate asynchronously, and alter the document content that was displayed. More recently, however, it has become common in both game development and the creation of desktop applications.                  
  *&lt;/div&gt;                  
  *&lt;div id="cSharp"&gt; C# is intended to be a simple, modern, general-purpose, object-oriented programming language. Its development team is led by Anders Hejlsberg. The most recent version is C# 5.0, which was released on August 15, 2012.                  
  *&lt;/div&gt;                  
  *&lt;div id="vb"&gt; The command-line compiler, VBC.EXE, is installed as part of the freeware .NET Framework SDK. Mono also includes a command-line VB.NET compiler. The most recent version is VB 2012, which was released on August 15, 2012.                  
  *&lt;/div&gt;              
  *&lt;/div&gt;
  * &lt;script type="text/javascript"&gt;         
           * // Set the collapsible during initialization. 			
           * 	$("#tab").ejTab({width:"300px",collapsible: true }); 
           * &lt;/script&gt;  
            * @memberof ejTab
           * @instance
           */
            collapsible: false,
            enableAnimation: true,
            ajaxSettings: { type: 'GET', cache: false, data: {}, dataType: "html", contentType: "html", async: true },
            /**		
           * Disables the given tab headers and content panels.
           * @default []
           * @type {IntegerArray}
           * @example 
           *&lt;div id="tab"&gt;                  
  *&lt;ul&gt;                      
  *&lt;li&gt;&lt;a href="#javaScript"&gt;JavaScript&lt;/a&gt;&lt;/li&gt;                      
  *&lt;li&gt;&lt;a href="#cSharp"&gt;C Sharp (C#)&lt;/a&gt;&lt;/li&gt;                      
  *&lt;li&gt;&lt;a href="#vb"&gt;VB.Net&lt;/a&gt;&lt;/li&gt;                  
  *&lt;/ul&gt;                  
  *&lt;div id="javaScript"&gt; JavaScript (JS) is an interpreted computer programming language. 
  *It was originally implemented as part of web browsers so that client-side scripts could interact with the user, control the browser, communicate asynchronously, and alter the document content that was displayed. More recently, however, it has become common in both game development and the creation of desktop applications.                  
  *&lt;/div&gt;                  
  *&lt;div id="cSharp"&gt; C# is intended to be a simple, modern, general-purpose, object-oriented programming language. Its development team is led by Anders Hejlsberg. The most recent version is C# 5.0, which was released on August 15, 2012.                  
  *&lt;/div&gt;                  
  *&lt;div id="vb"&gt; The command-line compiler, VBC.EXE, is installed as part of the freeware .NET Framework SDK. Mono also includes a command-line VB.NET compiler. The most recent version is VB 2012, which was released on August 15, 2012.                  
  *&lt;/div&gt;              
  *&lt;/div&gt;   
  * &lt;script type="text/javascript"&gt;         
           * // Set the disabledItemIndex during initialization. 			
           * 	$("#tab").ejTab({width:"300px",disabledItemIndex: [1,2] }); 
           * &lt;/script&gt;  
            * @memberof ejTab
           * @instance
           */
            disabledItemIndex: [],
            /**   
           * Enables the given tab headers and content panels.
           * @default []
           * @type {IntegerArray}
           * @example 
           *&lt;div id="tab"&gt;                  
  *&lt;ul&gt;                      
  *&lt;li&gt;&lt;a href="#javaScript"&gt;JavaScript&lt;/a&gt;&lt;/li&gt;                      
  *&lt;li&gt;&lt;a href="#cSharp"&gt;C Sharp (C#)&lt;/a&gt;&lt;/li&gt;                      
  *&lt;li&gt;&lt;a href="#vb"&gt;VB.Net&lt;/a&gt;&lt;/li&gt;                  
  *&lt;/ul&gt;                  
  *&lt;div id="javaScript"&gt; JavaScript (JS) is an interpreted computer programming language. 
  *It was originally implemented as part of web browsers so that client-side scripts could interact with the user, control the browser, communicate asynchronously, and alter the document content that was displayed. More recently, however, it has become common in both game development and the creation of desktop applications.                  
  *&lt;/div&gt;                  
  *&lt;div id="cSharp"&gt; C# is intended to be a simple, modern, general-purpose, object-oriented programming language. Its development team is led by Anders Hejlsberg. The most recent version is C# 5.0, which was released on August 15, 2012.                  
  *&lt;/div&gt;                  
  *&lt;div id="vb"&gt; The command-line compiler, VBC.EXE, is installed as part of the freeware .NET Framework SDK. Mono also includes a command-line VB.NET compiler. The most recent version is VB 2012, which was released on August 15, 2012.                  
  *&lt;/div&gt;              
  *&lt;/div&gt;   
  * &lt;script type="text/javascript"&gt;         
           * // Set the enabledItemIndex during initialization. 
           * 	$("#tab").ejTab({width:"300px",disabledItemIndex: [0,1,2] });
           *  $("#tab").ejTab({width:"300px",enabledItemIndex: [0,1] }); 
           * &lt;/script&gt;  
            * @memberof ejTab
           * @instance
           */
            enabledItemIndex: [],
            /**		
           * The event API to bind the action for active the tab items.
           * @default "click"
           * @type {string}
           * @example 
           *&lt;div id="tab"&gt;                  
  *&lt;ul&gt;                      
  *&lt;li&gt;&lt;a href="#javaScript"&gt;JavaScript&lt;/a&gt;&lt;/li&gt;                      
  *&lt;li&gt;&lt;a href="#cSharp"&gt;C Sharp (C#)&lt;/a&gt;&lt;/li&gt;                      
  *&lt;li&gt;&lt;a href="#vb"&gt;VB.Net&lt;/a&gt;&lt;/li&gt;                  
  *&lt;/ul&gt;                  
  *&lt;div id="javaScript"&gt; JavaScript (JS) is an interpreted computer programming language. 
  *It was originally implemented as part of web browsers so that client-side scripts could interact with the user, control the browser, communicate asynchronously, and alter the document content that was displayed. More recently, however, it has become common in both game development and the creation of desktop applications.                  
  *&lt;/div&gt;                  
  *&lt;div id="cSharp"&gt; C# is intended to be a simple, modern, general-purpose, object-oriented programming language. Its development team is led by Anders Hejlsberg. The most recent version is C# 5.0, which was released on August 15, 2012.                  
  *&lt;/div&gt;                  
  *&lt;div id="vb"&gt; The command-line compiler, VBC.EXE, is installed as part of the freeware .NET Framework SDK. Mono also includes a command-line VB.NET compiler. The most recent version is VB 2012, which was released on August 15, 2012.                  
  *&lt;/div&gt;              
  *&lt;/div&gt;     
  * &lt;script type="text/javascript"&gt;         
           * // Set the events during initialization. 			
           * 	$("#tab").ejTab({width:"300px",events: "click" }); 
          * &lt;/script&gt;  
            * @memberof ejTab
           * @instance
           */
            events: "click",
            /**		
           * The idPrefix property appends the give string on runtime added tab item id’s.
           * @default "ej-tab-"
           * @type {string}
           * @example 
           *&lt;div id="tab"&gt;                  
  *&lt;ul&gt;                      
  *&lt;li&gt;&lt;a href="#javaScript"&gt;JavaScript&lt;/a&gt;&lt;/li&gt;                      
  *&lt;li&gt;&lt;a href="#cSharp"&gt;C Sharp (C#)&lt;/a&gt;&lt;/li&gt;                      
  *&lt;li&gt;&lt;a href="#vb"&gt;VB.Net&lt;/a&gt;&lt;/li&gt;                  
  *&lt;/ul&gt;                  
  *&lt;div id="javaScript"&gt; JavaScript (JS) is an interpreted computer programming language. 
  *It was originally implemented as part of web browsers so that client-side scripts could interact with the user, control the browser, communicate asynchronously, and alter the document content that was displayed. More recently, however, it has become common in both game development and the creation of desktop applications.                  
  *&lt;/div&gt;                  
  *&lt;div id="cSharp"&gt; C# is intended to be a simple, modern, general-purpose, object-oriented programming language. Its development team is led by Anders Hejlsberg. The most recent version is C# 5.0, which was released on August 15, 2012.                  
  *&lt;/div&gt;                  
  *&lt;div id="vb"&gt; The command-line compiler, VBC.EXE, is installed as part of the freeware .NET Framework SDK. Mono also includes a command-line VB.NET compiler. The most recent version is VB 2012, which was released on August 15, 2012.                  
  *&lt;/div&gt;              
  *&lt;/div&gt;
  * &lt;script type="text/javascript"&gt;         
           * // Set the idPrefix during initialization. 			
           * 	$("#tab").ejTab({width:"300px",idPrefix: "ej-tab-" }); 
           * &lt;/script&gt;  
            * @memberof ejTab
           * @instance
           */
            idPrefix: "ej-tab-",
            /**		
           * Adjust the content panel height for given option (content, auto and fill), by default panels height adjust based on the content.See {@link HeightAdjustMode}
           * @default "content"
           * @type {string | enum}
           * @example 
           *&lt;div id="tab"&gt;                  
  *&lt;ul&gt;                      
  *&lt;li&gt;&lt;a href="#javaScript"&gt;JavaScript&lt;/a&gt;&lt;/li&gt;                      
  *&lt;li&gt;&lt;a href="#cSharp"&gt;C Sharp (C#)&lt;/a&gt;&lt;/li&gt;                      
  *&lt;li&gt;&lt;a href="#vb"&gt;VB.Net&lt;/a&gt;&lt;/li&gt;                  
  *&lt;/ul&gt;                  
  *&lt;div id="javaScript"&gt; JavaScript (JS) is an interpreted computer programming language. 
  *It was originally implemented as part of web browsers so that client-side scripts could interact with the user, control the browser, communicate asynchronously, and alter the document content that was displayed. More recently, however, it has become common in both game development and the creation of desktop applications.                  
  *&lt;/div&gt;                  
  *&lt;div id="cSharp"&gt; C# is intended to be a simple, modern, general-purpose, object-oriented programming language. Its development team is led by Anders Hejlsberg. The most recent version is C# 5.0, which was released on August 15, 2012.                  
  *&lt;/div&gt;                  
  *&lt;div id="vb"&gt; The command-line compiler, VBC.EXE, is installed as part of the freeware .NET Framework SDK. Mono also includes a command-line VB.NET compiler. The most recent version is VB 2012, which was released on August 15, 2012.                  
  *&lt;/div&gt;              
  *&lt;/div&gt; 
  * &lt;script type="text/javascript"&gt;         
           * // Set the heightAdjustMode  during initialization. 			
           * 	$("#tab").ejTab({width:"300px",heightAdjustMode : ej.Tab.HeightAdjustMode.Content }); 
           * &lt;/script&gt;  
            * @memberof ejTab
           * @instance
           */
            heightAdjustMode: "content",
            /**		
			* Tab header to active for given index value. 
			* @default 0
			* @type {number}
			* @example 
      *&lt;div id="tab"&gt;                  
  *&lt;ul&gt;                      
  *&lt;li&gt;&lt;a href="#javaScript"&gt;JavaScript&lt;/a&gt;&lt;/li&gt;                      
  *&lt;li&gt;&lt;a href="#cSharp"&gt;C Sharp (C#)&lt;/a&gt;&lt;/li&gt;                      
  *&lt;li&gt;&lt;a href="#vb"&gt;VB.Net&lt;/a&gt;&lt;/li&gt;                  
  *&lt;/ul&gt;                  
  *&lt;div id="javaScript"&gt; JavaScript (JS) is an interpreted computer programming language. 
  *It was originally implemented as part of web browsers so that client-side scripts could interact with the user, control the browser, communicate asynchronously, and alter the document content that was displayed. More recently, however, it has become common in both game development and the creation of desktop applications.                  
  *&lt;/div&gt;                  
  *&lt;div id="cSharp"&gt; C# is intended to be a simple, modern, general-purpose, object-oriented programming language. Its development team is led by Anders Hejlsberg. The most recent version is C# 5.0, which was released on August 15, 2012.                  
  *&lt;/div&gt;                  
  *&lt;div id="vb"&gt; The command-line compiler, VBC.EXE, is installed as part of the freeware .NET Framework SDK. Mono also includes a command-line VB.NET compiler. The most recent version is VB 2012, which was released on August 15, 2012.                  
  *&lt;/div&gt;              
  *&lt;/div&gt;
  * &lt;script type="text/javascript"&gt;         
			* // Set the selectedItemIndex  during initialization. 			
			* 	$("#tab").ejTab({width:"300px",selectedItemIndex : 1 }); 
			* &lt;/script&gt;  
			 * @memberof ejTab
			* @instance
			*/
            selectedItemIndex: 0,
            /**		
           * Set the root class for Tab theme. This cssClass API helps to use custom skinning option for Tab control. 
           * @default ""
           * @type {string}
           * @example 
           *&lt;div id="tab"&gt;                  
  *&lt;ul&gt;                      
  *&lt;li&gt;&lt;a href="#javaScript"&gt;JavaScript&lt;/a&gt;&lt;/li&gt;                      
  *&lt;li&gt;&lt;a href="#cSharp"&gt;C Sharp (C#)&lt;/a&gt;&lt;/li&gt;                      
  *&lt;li&gt;&lt;a href="#vb"&gt;VB.Net&lt;/a&gt;&lt;/li&gt;                  
  *&lt;/ul&gt;                  
  *&lt;div id="javaScript"&gt; JavaScript (JS) is an interpreted computer programming language. 
  *It was originally implemented as part of web browsers so that client-side scripts could interact with the user, control the browser, communicate asynchronously, and alter the document content that was displayed. More recently, however, it has become common in both game development and the creation of desktop applications.                  
  *&lt;/div&gt;                  
  *&lt;div id="cSharp"&gt; C# is intended to be a simple, modern, general-purpose, object-oriented programming language. Its development team is led by Anders Hejlsberg. The most recent version is C# 5.0, which was released on August 15, 2012.                  
  *&lt;/div&gt;                  
  *&lt;div id="vb"&gt; The command-line compiler, VBC.EXE, is installed as part of the freeware .NET Framework SDK. Mono also includes a command-line VB.NET compiler. The most recent version is VB 2012, which was released on August 15, 2012.                  
  *&lt;/div&gt;              
  *&lt;/div&gt;
  * &lt;script type="text/javascript"&gt;         
           * // Set the CSS class during initialization. 			
           * 	$("#tab").ejTab({width:"300px",cssClass: "gradient-lime" }); 
          * &lt;/script&gt;  
            * @memberof ejTab
           * @instance
           */
            cssClass: "",
            /**		
           * Display the close button for each tab items. While click on the close icon particular tab item removed. 
           * @default false
           * @type {boolean}
           * @example 
           *&lt;div id="tab"&gt;                  
  *&lt;ul&gt;                      
  *&lt;li&gt;&lt;a href="#javaScript"&gt;JavaScript&lt;/a&gt;&lt;/li&gt;                      
  *&lt;li&gt;&lt;a href="#cSharp"&gt;C Sharp (C#)&lt;/a&gt;&lt;/li&gt;                      
  *&lt;li&gt;&lt;a href="#vb"&gt;VB.Net&lt;/a&gt;&lt;/li&gt;                  
  *&lt;/ul&gt;                  
  *&lt;div id="javaScript"&gt; JavaScript (JS) is an interpreted computer programming language. 
  *It was originally implemented as part of web browsers so that client-side scripts could interact with the user, control the browser, communicate asynchronously, and alter the document content that was displayed. More recently, however, it has become common in both game development and the creation of desktop applications.                  
  *&lt;/div&gt;                  
  *&lt;div id="cSharp"&gt; C# is intended to be a simple, modern, general-purpose, object-oriented programming language. Its development team is led by Anders Hejlsberg. The most recent version is C# 5.0, which was released on August 15, 2012.                  
  *&lt;/div&gt;                  
  *&lt;div id="vb"&gt; The command-line compiler, VBC.EXE, is installed as part of the freeware .NET Framework SDK. Mono also includes a command-line VB.NET compiler. The most recent version is VB 2012, which was released on August 15, 2012.                  
  *&lt;/div&gt;              
  *&lt;/div&gt;   
  * &lt;script type="text/javascript"&gt;         
           * // Set the showCloseButton during initialization. 			
           * 	$("#tab").ejTab({width:"500px",showCloseButton: true }); 
          * &lt;/script&gt;  
            * @memberof ejTab
           * @instance
           */
            showCloseButton: false,
			/**		
           * Specify to enable scrolling for Tab header. 
           * @default false
           * @type {boolean}
           * @example 
           *&lt;div id="tab"&gt;                  
  *&lt;ul&gt;                      
  *&lt;li&gt;&lt;a href="#javaScript"&gt;JavaScript&lt;/a&gt;&lt;/li&gt;                      
  *&lt;li&gt;&lt;a href="#cSharp"&gt;C Sharp (C#)&lt;/a&gt;&lt;/li&gt;                      
  *&lt;li&gt;&lt;a href="#vb"&gt;VB.Net&lt;/a&gt;&lt;/li&gt;                  
  *&lt;/ul&gt;                  
  *&lt;div id="javaScript"&gt; JavaScript (JS) is an interpreted computer programming language. 
  *It was originally implemented as part of web browsers so that client-side scripts could interact with the user, control the browser, communicate asynchronously, and alter the document content that was displayed. More recently, however, it has become common in both game development and the creation of desktop applications.                  
  *&lt;/div&gt;                  
  *&lt;div id="cSharp"&gt; C# is intended to be a simple, modern, general-purpose, object-oriented programming language. Its development team is led by Anders Hejlsberg. The most recent version is C# 5.0, which was released on August 15, 2012.                  
  *&lt;/div&gt;                  
  *&lt;div id="vb"&gt; The command-line compiler, VBC.EXE, is installed as part of the freeware .NET Framework SDK. Mono also includes a command-line VB.NET compiler. The most recent version is VB 2012, which was released on August 15, 2012.                  
  *&lt;/div&gt;              
  *&lt;/div&gt;
  * &lt;script type="text/javascript"&gt;         
           * // Set the enableTabScroll during initialization. 			
           * 	$("#tab").ejTab({width:"300px",enableTabScroll: true }); 
          * &lt;/script&gt;  
            * @memberof ejTab
           * @instance
           */
            enableTabScroll: false,
            /**		
           * Display the Reload button for each tab items. 
           * @default false
           * @type {boolean}
           * @example 
           *&lt;div id="tab"&gt;                  
  *&lt;ul&gt;                      
  *&lt;li&gt;&lt;a href="#javaScript"&gt;JavaScript&lt;/a&gt;&lt;/li&gt;                      
  *&lt;li&gt;&lt;a href="#cSharp"&gt;C Sharp (C#)&lt;/a&gt;&lt;/li&gt;                      
  *&lt;li&gt;&lt;a href="#vb"&gt;VB.Net&lt;/a&gt;&lt;/li&gt;                  
  *&lt;/ul&gt;                  
  *&lt;div id="javaScript"&gt; JavaScript (JS) is an interpreted computer programming language. 
  *It was originally implemented as part of web browsers so that client-side scripts could interact with the user, control the browser, communicate asynchronously, and alter the document content that was displayed. More recently, however, it has become common in both game development and the creation of desktop applications.                  
  *&lt;/div&gt;                  
  *&lt;div id="cSharp"&gt; C# is intended to be a simple, modern, general-purpose, object-oriented programming language. Its development team is led by Anders Hejlsberg. The most recent version is C# 5.0, which was released on August 15, 2012.                  
  *&lt;/div&gt;                  
  *&lt;div id="vb"&gt; The command-line compiler, VBC.EXE, is installed as part of the freeware .NET Framework SDK. Mono also includes a command-line VB.NET compiler. The most recent version is VB 2012, which was released on August 15, 2012.                  
  *&lt;/div&gt;              
  *&lt;/div&gt;
  * &lt;script type="text/javascript"&gt;         
           * // Set the showReloadIcon during initialization. 			
           * 	$("#tab").ejTab({width:"500px",showReloadIcon: true }); 
          * &lt;/script&gt;  
            * @memberof ejTab
           * @instance
           */
            showReloadIcon: false,
            /**		
        * Tab header display top,bottom,left or right .See {@link Position} 
        * @default "top"
        * @type {string | enum}
        * @example 
        *&lt;div id="tab"&gt;                  
  *&lt;ul&gt;                      
  *&lt;li&gt;&lt;a href="#javaScript"&gt;JavaScript&lt;/a&gt;&lt;/li&gt;                      
  *&lt;li&gt;&lt;a href="#cSharp"&gt;C Sharp (C#)&lt;/a&gt;&lt;/li&gt;                      
  *&lt;li&gt;&lt;a href="#vb"&gt;VB.Net&lt;/a&gt;&lt;/li&gt;                  
  *&lt;/ul&gt;                  
  *&lt;div id="javaScript"&gt; JavaScript (JS) is an interpreted computer programming language. 
  *It was originally implemented as part of web browsers so that client-side scripts could interact with the user, control the browser, communicate asynchronously, and alter the document content that was displayed. More recently, however, it has become common in both game development and the creation of desktop applications.                  
  *&lt;/div&gt;                  
  *&lt;div id="cSharp"&gt; C# is intended to be a simple, modern, general-purpose, object-oriented programming language. Its development team is led by Anders Hejlsberg. The most recent version is C# 5.0, which was released on August 15, 2012.                  
  *&lt;/div&gt;                  
  *&lt;div id="vb"&gt; The command-line compiler, VBC.EXE, is installed as part of the freeware .NET Framework SDK. Mono also includes a command-line VB.NET compiler. The most recent version is VB 2012, which was released on August 15, 2012.                  
  *&lt;/div&gt;              
  *&lt;/div&gt;
  * &lt;script type="text/javascript"&gt;         
        * // Set the headerPosition during initialization. 			
        * 	$("#tab").ejTab({width:"500px",headerPosition: "left" }); 
        * &lt;/script&gt;  
         * @memberof ejTab
        * @instance
        */
            headerPosition: "top",
            /**		
        * Set the width for outer panel element, if not it’s take parent width. 
        * @default null
        * @type { string | number}
        * @example 
        *&lt;div id="tab"&gt;                  
  *&lt;ul&gt;                      
  *&lt;li&gt;&lt;a href="#javaScript"&gt;JavaScript&lt;/a&gt;&lt;/li&gt;                      
  *&lt;li&gt;&lt;a href="#cSharp"&gt;C Sharp (C#)&lt;/a&gt;&lt;/li&gt;                      
  *&lt;li&gt;&lt;a href="#vb"&gt;VB.Net&lt;/a&gt;&lt;/li&gt;                  
  *&lt;/ul&gt;                  
  *&lt;div id="javaScript"&gt; JavaScript (JS) is an interpreted computer programming language. 
  *It was originally implemented as part of web browsers so that client-side scripts could interact with the user, control the browser, communicate asynchronously, and alter the document content that was displayed. More recently, however, it has become common in both game development and the creation of desktop applications.                  
  *&lt;/div&gt;                  
  *&lt;div id="cSharp"&gt; C# is intended to be a simple, modern, general-purpose, object-oriented programming language. Its development team is led by Anders Hejlsberg. The most recent version is C# 5.0, which was released on August 15, 2012.                  
  *&lt;/div&gt;                  
  *&lt;div id="vb"&gt; The command-line compiler, VBC.EXE, is installed as part of the freeware .NET Framework SDK. Mono also includes a command-line VB.NET compiler. The most recent version is VB 2012, which was released on August 15, 2012.                  
  *&lt;/div&gt;              
  *&lt;/div&gt;
  * &lt;script type="text/javascript"&gt;         
        * // Set the width during initialization. 			
        * 	$("#tab").ejTab({width: "600px" }); 
       * &lt;/script&gt;  
         * @memberof ejTab
        * @instance
        */
            width: null,
            /**		
        * Height set the outer panel element. Default this property value is null, so height take content height.
        * @default null
        * @type {string | number}
        * @example 
        *&lt;div id="tab"&gt;                  
  *&lt;ul&gt;                      
  *&lt;li&gt;&lt;a href="#javaScript"&gt;JavaScript&lt;/a&gt;&lt;/li&gt;                      
  *&lt;li&gt;&lt;a href="#cSharp"&gt;C Sharp (C#)&lt;/a&gt;&lt;/li&gt;                      
  *&lt;li&gt;&lt;a href="#vb"&gt;VB.Net&lt;/a&gt;&lt;/li&gt;                  
  *&lt;/ul&gt;                  
  *&lt;div id="javaScript"&gt; JavaScript (JS) is an interpreted computer programming language. 
  *It was originally implemented as part of web browsers so that client-side scripts could interact with the user, control the browser, communicate asynchronously, and alter the document content that was displayed. More recently, however, it has become common in both game development and the creation of desktop applications.                  
  *&lt;/div&gt;                  
  *&lt;div id="cSharp"&gt; C# is intended to be a simple, modern, general-purpose, object-oriented programming language. Its development team is led by Anders Hejlsberg. The most recent version is C# 5.0, which was released on August 15, 2012.                  
  *&lt;/div&gt;                  
  *&lt;div id="vb"&gt; The command-line compiler, VBC.EXE, is installed as part of the freeware .NET Framework SDK. Mono also includes a command-line VB.NET compiler. The most recent version is VB 2012, which was released on August 15, 2012.                  
  *&lt;/div&gt;              
  *&lt;/div&gt;
  * &lt;script type="text/javascript"&gt;         
        * // Set the height during initialization. 			
        * 	$("#tab").ejTab({width:"300px",height: "320px" }); 
       * &lt;/script&gt;  
         * @memberof ejTab
        * @instance
        */
            height: null,
            /**		
           * Display Right to Left direction for headers and panels text.
           * @default false
           * @type {boolean}
           * @example 
           *&lt;div id="tab"&gt;                  
  *&lt;ul&gt;                      
  *&lt;li&gt;&lt;a href="#javaScript"&gt;JavaScript&lt;/a&gt;&lt;/li&gt;                      
  *&lt;li&gt;&lt;a href="#cSharp"&gt;C Sharp (C#)&lt;/a&gt;&lt;/li&gt;                      
  *&lt;li&gt;&lt;a href="#vb"&gt;VB.Net&lt;/a&gt;&lt;/li&gt;                  
  *&lt;/ul&gt;                  
  *&lt;div id="javaScript"&gt; JavaScript (JS) is an interpreted computer programming language. 
  *It was originally implemented as part of web browsers so that client-side scripts could interact with the user, control the browser, communicate asynchronously, and alter the document content that was displayed. More recently, however, it has become common in both game development and the creation of desktop applications.                  
  *&lt;/div&gt;                  
  *&lt;div id="cSharp"&gt; C# is intended to be a simple, modern, general-purpose, object-oriented programming language. Its development team is led by Anders Hejlsberg. The most recent version is C# 5.0, which was released on August 15, 2012.                  
  *&lt;/div&gt;                  
  *&lt;div id="vb"&gt; The command-line compiler, VBC.EXE, is installed as part of the freeware .NET Framework SDK. Mono also includes a command-line VB.NET compiler. The most recent version is VB 2012, which was released on August 15, 2012.                  
  *&lt;/div&gt;              
  *&lt;/div&gt;
  * &lt;script type="text/javascript"&gt;         
           * // Set the rtl during initialization. 			
           * 	$("#tab").ejTab({width:"300px",enableRTL: true }); 
          * &lt;/script&gt;  
            * @memberof ejTab
           * @instance
           */
            enableRTL: false,
            /**		
           * Tab items interaction with keyboard keys, like headers active navigation.
           * @default true
           * @type {boolean}
           * @example 
           *&lt;div id="tab"&gt;                  
  *&lt;ul&gt;                      
  *&lt;li&gt;&lt;a href="#javaScript"&gt;JavaScript&lt;/a&gt;&lt;/li&gt;                      
  *&lt;li&gt;&lt;a href="#cSharp"&gt;C Sharp (C#)&lt;/a&gt;&lt;/li&gt;                      
  *&lt;li&gt;&lt;a href="#vb"&gt;VB.Net&lt;/a&gt;&lt;/li&gt;                  
  *&lt;/ul&gt;                  
  *&lt;div id="javaScript"&gt; JavaScript (JS) is an interpreted computer programming language. 
  *It was originally implemented as part of web browsers so that client-side scripts could interact with the user, control the browser, communicate asynchronously, and alter the document content that was displayed. More recently, however, it has become common in both game development and the creation of desktop applications.                  
  *&lt;/div&gt;                  
  *&lt;div id="cSharp"&gt; C# is intended to be a simple, modern, general-purpose, object-oriented programming language. Its development team is led by Anders Hejlsberg. The most recent version is C# 5.0, which was released on August 15, 2012.                  
  *&lt;/div&gt;                  
  *&lt;div id="vb"&gt; The command-line compiler, VBC.EXE, is installed as part of the freeware .NET Framework SDK. Mono also includes a command-line VB.NET compiler. The most recent version is VB 2012, which was released on August 15, 2012.                  
  *&lt;/div&gt;              
  *&lt;/div&gt;
  * &lt;script type="text/javascript"&gt;         
           * // Set the allowKeyboardNavigation during initialization. 			
           * 	$("#tab").ejTab({width:"300px",allowKeyboardNavigation: false }); 
           * &lt;/script&gt;  
            * @memberof ejTab
           * @instance
           */
            allowKeyboardNavigation: true,
            /**		
           * Tab panels and headers to display the rounded corner style.
           * @default false
           * @type {boolean}
           * @example 
           *&lt;div id="tab"&gt;                  
  *&lt;ul&gt;                      
  *&lt;li&gt;&lt;a href="#javaScript"&gt;JavaScript&lt;/a&gt;&lt;/li&gt;                      
  *&lt;li&gt;&lt;a href="#cSharp"&gt;C Sharp (C#)&lt;/a&gt;&lt;/li&gt;                      
  *&lt;li&gt;&lt;a href="#vb"&gt;VB.Net&lt;/a&gt;&lt;/li&gt;                  
  *&lt;/ul&gt;                  
  *&lt;div id="javaScript"&gt; JavaScript (JS) is an interpreted computer programming language. 
  *It was originally implemented as part of web browsers so that client-side scripts could interact with the user, control the browser, communicate asynchronously, and alter the document content that was displayed. More recently, however, it has become common in both game development and the creation of desktop applications.                  
  *&lt;/div&gt;                  
  *&lt;div id="cSharp"&gt; C# is intended to be a simple, modern, general-purpose, object-oriented programming language. Its development team is led by Anders Hejlsberg. The most recent version is C# 5.0, which was released on August 15, 2012.                  
  *&lt;/div&gt;                  
  *&lt;div id="vb"&gt; The command-line compiler, VBC.EXE, is installed as part of the freeware .NET Framework SDK. Mono also includes a command-line VB.NET compiler. The most recent version is VB 2012, which was released on August 15, 2012.                  
  *&lt;/div&gt;              
  *&lt;/div&gt;
  * &lt;script type="text/javascript"&gt;         
           * // Set the roundedCorner during initialization. 			
           * 	$("#tab").ejTab({width:"300px",showRoundedCorner: false }); 
           * &lt;/script&gt;  
            * @memberof ejTab
           * @instance
           */
            showRoundedCorner: false,
            /**		
           * Save current model value to browser cookies for state maintains. While refresh the Tab control page the model value apply from browser cookies.  
           * @default false
           * @type {boolean}
           * @example 
           *&lt;div id="tab"&gt;                  
  *&lt;ul&gt;                      
  *&lt;li&gt;&lt;a href="#javaScript"&gt;JavaScript&lt;/a&gt;&lt;/li&gt;                      
  *&lt;li&gt;&lt;a href="#cSharp"&gt;C Sharp (C#)&lt;/a&gt;&lt;/li&gt;                      
  *&lt;li&gt;&lt;a href="#vb"&gt;VB.Net&lt;/a&gt;&lt;/li&gt;                  
  *&lt;/ul&gt;                  
  *&lt;div id="javaScript"&gt; JavaScript (JS) is an interpreted computer programming language. 
  *It was originally implemented as part of web browsers so that client-side scripts could interact with the user, control the browser, communicate asynchronously, and alter the document content that was displayed. More recently, however, it has become common in both game development and the creation of desktop applications.                  
  *&lt;/div&gt;                  
  *&lt;div id="cSharp"&gt; C# is intended to be a simple, modern, general-purpose, object-oriented programming language. Its development team is led by Anders Hejlsberg. The most recent version is C# 5.0, which was released on August 15, 2012.                  
  *&lt;/div&gt;                  
  *&lt;div id="vb"&gt; The command-line compiler, VBC.EXE, is installed as part of the freeware .NET Framework SDK. Mono also includes a command-line VB.NET compiler. The most recent version is VB 2012, which was released on August 15, 2012.                  
  *&lt;/div&gt;              
  *&lt;/div&gt; 
  * &lt;script type="text/javascript"&gt;         
           * // Set the enablePersistence during initialization. 			
           * 	$("#tab").ejTab({width:"300px",enablePersistence: false }); 
           * &lt;/script&gt;  
            * @memberof ejTab
           * @instance
           */
            enablePersistence: false,
            /**		
           * When this property is set to false, it disables the tab control.
           * @default true
           * @type {boolean}
           * @example 
           *&lt;div id="tab"&gt;                  
  *&lt;ul&gt;                      
  *&lt;li&gt;&lt;a href="#javaScript"&gt;JavaScript&lt;/a&gt;&lt;/li&gt;                      
  *&lt;li&gt;&lt;a href="#cSharp"&gt;C Sharp (C#)&lt;/a&gt;&lt;/li&gt;                      
  *&lt;li&gt;&lt;a href="#vb"&gt;VB.Net&lt;/a&gt;&lt;/li&gt;                  
  *&lt;/ul&gt;                  
  *&lt;div id="javaScript"&gt; JavaScript (JS) is an interpreted computer programming language. 
  *It was originally implemented as part of web browsers so that client-side scripts could interact with the user, control the browser, communicate asynchronously, and alter the document content that was displayed. More recently, however, it has become common in both game development and the creation of desktop applications.                  
  *&lt;/div&gt;                  
  *&lt;div id="cSharp"&gt; C# is intended to be a simple, modern, general-purpose, object-oriented programming language. Its development team is led by Anders Hejlsberg. The most recent version is C# 5.0, which was released on August 15, 2012.                  
  *&lt;/div&gt;                  
  *&lt;div id="vb"&gt; The command-line compiler, VBC.EXE, is installed as part of the freeware .NET Framework SDK. Mono also includes a command-line VB.NET compiler. The most recent version is VB 2012, which was released on August 15, 2012.                  
  *&lt;/div&gt;              
  *&lt;/div&gt;
  * &lt;script type="text/javascript"&gt;         
           * // Set the enabled during initialization. 			
           * 	$("#tab").ejTab({width:"300px",enabled: false }); 
          * &lt;/script&gt;  
            * @memberof ejTab
           * @instance
           */
            enabled: true,
            //Events
            /**     
			 * Triggered after ajax content load action.
			 * @event
			 * @name ejTab#ajaxLoad 	
			 * @param {Object} argument Event parameters from button     
			 * @param {boolean} argument.cancel if the event should be canceled; otherwise, false.
			 * @param {string}  argument.url returns the name of the url
			 * @param {object}  argument.model returns the tab model.
			 * @param {string}  argument.type returns the name of the event.
			 * @param {object}  argument.prevActiveHeader returns previous active tab header.
			 * @param {number}  argument.prevActiveIndex returns previous active index.
			 * @param {object}  argument.activeHeader returns current active tab header .
			 * @param {number}  argument.activeIndex returns current active index.
			 * @example 
       *&lt;div id="tab"&gt;                  
  *&lt;ul&gt;                      
  *&lt;li&gt;&lt;a href="#javaScript"&gt;JavaScript&lt;/a&gt;&lt;/li&gt;                      
  *&lt;li&gt;&lt;a href="#cSharp"&gt;C Sharp (C#)&lt;/a&gt;&lt;/li&gt;                      
  *&lt;li&gt;&lt;a href="#vb"&gt;VB.Net&lt;/a&gt;&lt;/li&gt;                  
  *&lt;/ul&gt;                  
  *&lt;div id="javaScript"&gt; JavaScript (JS) is an interpreted computer programming language. 
  *It was originally implemented as part of web browsers so that client-side scripts could interact with the user, control the browser, communicate asynchronously, and alter the document content that was displayed. More recently, however, it has become common in both game development and the creation of desktop applications.                  
  *&lt;/div&gt;                  
  *&lt;div id="cSharp"&gt; C# is intended to be a simple, modern, general-purpose, object-oriented programming language. Its development team is led by Anders Hejlsberg. The most recent version is C# 5.0, which was released on August 15, 2012.                  
  *&lt;/div&gt;                  
  *&lt;div id="vb"&gt; The command-line compiler, VBC.EXE, is installed as part of the freeware .NET Framework SDK. Mono also includes a command-line VB.NET compiler. The most recent version is VB 2012, which was released on August 15, 2012.                  
  *&lt;/div&gt;              
  *&lt;/div&gt;
  * &lt;script type="text/javascript"&gt;         
             * $("#tab").ejTab({
              * width:"300px",
             *    ajaxLoad: function (args) {}
             * });      
             * &lt;/script&gt;  
			 * @memberof ejTab
			 * @instance
			 */
            ajaxLoad: null,
            /**     
			 * Triggered before ajax content load action.
			 * @event
			 * @name ejTab#ajaxBeforeLoad 	
			 * @param {Object} argument Event parameters from button     
			 * @param {boolean} argument.cancel if the event should be canceled; otherwise, false.
			 * @param {object}  argument.model returns the tab model.
			 * @param {string}  argument.type returns the name of the event.
		     * @param {object}  argument.prevActiveHeader returns previous active tab header.
			 * @param {number}  argument.prevActiveIndex returns previous active index.
			 * @param {object}  argument.activeHeader returns current active tab header .
			 * @param {number}  argument.activeIndex returns current active index.
			 * @example 
       *&lt;div id="tab"&gt;                  
  *&lt;ul&gt;                      
  *&lt;li&gt;&lt;a href="#javaScript"&gt;JavaScript&lt;/a&gt;&lt;/li&gt;                      
  *&lt;li&gt;&lt;a href="#cSharp"&gt;C Sharp (C#)&lt;/a&gt;&lt;/li&gt;                      
  *&lt;li&gt;&lt;a href="#vb"&gt;VB.Net&lt;/a&gt;&lt;/li&gt;                  
  *&lt;/ul&gt;                  
  *&lt;div id="javaScript"&gt; JavaScript (JS) is an interpreted computer programming language. 
  *It was originally implemented as part of web browsers so that client-side scripts could interact with the user, control the browser, communicate asynchronously, and alter the document content that was displayed. More recently, however, it has become common in both game development and the creation of desktop applications.                  
  *&lt;/div&gt;                  
  *&lt;div id="cSharp"&gt; C# is intended to be a simple, modern, general-purpose, object-oriented programming language. Its development team is led by Anders Hejlsberg. The most recent version is C# 5.0, which was released on August 15, 2012.                  
  *&lt;/div&gt;                  
  *&lt;div id="vb"&gt; The command-line compiler, VBC.EXE, is installed as part of the freeware .NET Framework SDK. Mono also includes a command-line VB.NET compiler. The most recent version is VB 2012, which was released on August 15, 2012.                  
  *&lt;/div&gt;              
  *&lt;/div&gt;
  * &lt;script type="text/javascript"&gt;         
             * $("#tab").ejTab({
              * width:"300px",
             *    ajaxBeforeLoad: function (args) {}
             * });      
             * &lt;/script&gt;  
			 * @memberof ejTab
			 * @instance
			 */
            ajaxBeforeLoad: null,
            /**     
			 * Triggered after a tab item activated.
			 * @event
			 * @name ejTab#ajaxSuccess 	
			 * @param {Object} argument Event parameters from button     
			 * @param {boolean} argument.cancel if the event should be canceled; otherwise, false.
			 * @param {object}  argument.model returns the tab model.
			 * @param {string}  argument.type returns the name of the event.
			 * @param {object}  argument.prevActiveHeader returns previous active tab header.
			 * @param {number}  argument.prevActiveIndex returns previous active index.
			 * @param {object}  argument.activeHeader returns current active tab header .
			 * @param {number}  argument.activeIndex returns current active index.
			 * @example 
       *&lt;div id="tab"&gt;                  
  *&lt;ul&gt;                      
  *&lt;li&gt;&lt;a href="#javaScript"&gt;JavaScript&lt;/a&gt;&lt;/li&gt;                      
  *&lt;li&gt;&lt;a href="#cSharp"&gt;C Sharp (C#)&lt;/a&gt;&lt;/li&gt;                      
  *&lt;li&gt;&lt;a href="#vb"&gt;VB.Net&lt;/a&gt;&lt;/li&gt;                  
  *&lt;/ul&gt;                  
  *&lt;div id="javaScript"&gt; JavaScript (JS) is an interpreted computer programming language. 
  *It was originally implemented as part of web browsers so that client-side scripts could interact with the user, control the browser, communicate asynchronously, and alter the document content that was displayed. More recently, however, it has become common in both game development and the creation of desktop applications.                  
  *&lt;/div&gt;                  
  *&lt;div id="cSharp"&gt; C# is intended to be a simple, modern, general-purpose, object-oriented programming language. Its development team is led by Anders Hejlsberg. The most recent version is C# 5.0, which was released on August 15, 2012.                  
  *&lt;/div&gt;                  
  *&lt;div id="vb"&gt; The command-line compiler, VBC.EXE, is installed as part of the freeware .NET Framework SDK. Mono also includes a command-line VB.NET compiler. The most recent version is VB 2012, which was released on August 15, 2012.                  
  *&lt;/div&gt;              
  *&lt;/div&gt;
  * &lt;script type="text/javascript"&gt;         
             * $("#tab").ejTab({
              * width: "300px",
             *    ajaxSuccess: function (args) {}
             * });      
             * &lt;/script&gt;  
			 * @memberof ejTab
			 * @instance
			 */
            ajaxSuccess: null,
            /**     
    * Triggered after a tab item activated.
    * @event
    * @name ejTab#ajaxError  
    * @param {Object} argument Event parameters from button     
    * @param {boolean} argument.cancel if the event should be canceled; otherwise, false.
    * @param {object}  argument.model returns the tab model.
    * @param {string}  argument.type returns the name of the event.
    * @param {object}  argument.prevActiveHeader returns previous active tab header.
     * @param {number}  argument.prevActiveIndex returns previous active index.
     * @param {object}  argument.activeHeader returns current active tab header .
     * @param {number}  argument.activeIndex returns current active index.
    * @example 
    *&lt;div id="tab"&gt;                  
*&lt;ul&gt;                      
*&lt;li&gt;&lt;a href="#javaScript"&gt;JavaScript&lt;/a&gt;&lt;/li&gt;                      
*&lt;li&gt;&lt;a href="#cSharp"&gt;C Sharp (C#)&lt;/a&gt;&lt;/li&gt;                      
*&lt;li&gt;&lt;a href="#vb"&gt;VB.Net&lt;/a&gt;&lt;/li&gt;                  
*&lt;/ul&gt;                  
*&lt;div id="javaScript"&gt; JavaScript (JS) is an interpreted computer programming language. 
*It was originally implemented as part of web browsers so that client-side scripts could interact with the user, control the browser, communicate asynchronously, and alter the document content that was displayed. More recently, however, it has become common in both game development and the creation of desktop applications.                  
*&lt;/div&gt;                  
*&lt;div id="cSharp"&gt; C# is intended to be a simple, modern, general-purpose, object-oriented programming language. Its development team is led by Anders Hejlsberg. The most recent version is C# 5.0, which was released on August 15, 2012.                  
*&lt;/div&gt;                  
*&lt;div id="vb"&gt; The command-line compiler, VBC.EXE, is installed as part of the freeware .NET Framework SDK. Mono also includes a command-line VB.NET compiler. The most recent version is VB 2012, which was released on August 15, 2012.                  
*&lt;/div&gt;              
*&lt;/div&gt;
* &lt;script type="text/javascript"&gt;         
          * $("#tab").ejTab({
           * width: "300px",
          *    ajaxError: function (args) {}
          * });      
          * &lt;/script&gt;  
    * @memberof ejTab
    * @instance
    */
            ajaxError: null,
            /**     
    * Triggered after a tab item activated.
    * @event
    * @name ejTab#active   
    * @param {Object} argument Event parameters from button     
    * @param {boolean} argument.cancel if the event should be canceled; otherwise, false.
    * @param {object}  argument.model returns the tab model.
    * @param {string}  argument.type returns the name of the event.
    * @param {object}  argument.prevActiveHeader returns previous active tab header.
     * @param {number}  argument.prevActiveIndex returns previous active index.
     * @param {object}  argument.activeHeader returns current active tab header .
     * @param {number}  argument.activeIndex returns current active index.
    * @example 
    *&lt;div id="tab"&gt;                  
*&lt;ul&gt;                      
*&lt;li&gt;&lt;a href="#javaScript"&gt;JavaScript&lt;/a&gt;&lt;/li&gt;                      
*&lt;li&gt;&lt;a href="#cSharp"&gt;C Sharp (C#)&lt;/a&gt;&lt;/li&gt;                      
*&lt;li&gt;&lt;a href="#vb"&gt;VB.Net&lt;/a&gt;&lt;/li&gt;                  
*&lt;/ul&gt;                  
*&lt;div id="javaScript"&gt; JavaScript (JS) is an interpreted computer programming language. 
*It was originally implemented as part of web browsers so that client-side scripts could interact with the user, control the browser, communicate asynchronously, and alter the document content that was displayed. More recently, however, it has become common in both game development and the creation of desktop applications.                  
*&lt;/div&gt;                  
*&lt;div id="cSharp"&gt; C# is intended to be a simple, modern, general-purpose, object-oriented programming language. Its development team is led by Anders Hejlsberg. The most recent version is C# 5.0, which was released on August 15, 2012.                  
*&lt;/div&gt;                  
*&lt;div id="vb"&gt; The command-line compiler, VBC.EXE, is installed as part of the freeware .NET Framework SDK. Mono also includes a command-line VB.NET compiler. The most recent version is VB 2012, which was released on August 15, 2012.                  
*&lt;/div&gt;              
*&lt;/div&gt;
* &lt;script type="text/javascript"&gt;         
          * $("#tab").ejTab({
           * width: "300px",
          *    itemActive: function (args) {}
          * });      
          * &lt;/script&gt;  
    * @memberof ejTab
    * @instance
    */
            itemActive: null,
            /**     
			 * Triggered before a tab item activated.
			 * @event
			 * @name ejTab#beforeActive 	
			 * @param {Object} argument Event parameters from button     
			 * @param {boolean} argument.cancel if the event should be canceled; otherwise, false.
			 * @param {object}  argument.model returns the tab model.
			 * @param {string}  argument.type returns the name of the event.
			 * @param {object}  argument.prevActiveHeader returns previous active tab header.
			 * @param {number}  argument.prevActiveIndex returns previous active index.
			 * @param {object}  argument.activeHeader returns current active tab header .
			 * @param {number}  argument.activeIndex returns current active index.
			 * @example 
       *&lt;div id="tab"&gt;                  
  *&lt;ul&gt;                      
  *&lt;li&gt;&lt;a href="#javaScript"&gt;JavaScript&lt;/a&gt;&lt;/li&gt;                      
  *&lt;li&gt;&lt;a href="#cSharp"&gt;C Sharp (C#)&lt;/a&gt;&lt;/li&gt;                      
  *&lt;li&gt;&lt;a href="#vb"&gt;VB.Net&lt;/a&gt;&lt;/li&gt;                  
  *&lt;/ul&gt;                  
  *&lt;div id="javaScript"&gt; JavaScript (JS) is an interpreted computer programming language. 
  *It was originally implemented as part of web browsers so that client-side scripts could interact with the user, control the browser, communicate asynchronously, and alter the document content that was displayed. More recently, however, it has become common in both game development and the creation of desktop applications.                  
  *&lt;/div&gt;                  
  *&lt;div id="cSharp"&gt; C# is intended to be a simple, modern, general-purpose, object-oriented programming language. Its development team is led by Anders Hejlsberg. The most recent version is C# 5.0, which was released on August 15, 2012.                  
  *&lt;/div&gt;                  
  *&lt;div id="vb"&gt; The command-line compiler, VBC.EXE, is installed as part of the freeware .NET Framework SDK. Mono also includes a command-line VB.NET compiler. The most recent version is VB 2012, which was released on August 15, 2012.                  
  *&lt;/div&gt;              
  *&lt;/div&gt;
  * &lt;script type="text/javascript"&gt;         
             * $("#tab").ejTab({
              * width: "300px",
             *    beforeActive: function (args) {}
             * });      
             * &lt;/script&gt;  
			 * @memberof ejTab
			 * @instance
			 */
            beforeActive: null,
            /**     
			 * Triggered after new tab item add
			 * @event
			 * @name ejTab#itemAdd 	
			 * @param {Object} argument Event parameters from button     
			 * @param {boolean} argument.cancel if the event should be canceled; otherwise, false.
			 * @param {object}  argument.model returns the tab model.
			 * @param {string}  argument.type returns the name of the event.
			 * @param {object}  argument.tabHeader returns new added tab header.
			 * @param {object}  argument.tabContent returns new added tab content panel.
			 * @example 
       *&lt;div id="tab"&gt;                  
  *&lt;ul&gt;                      
  *&lt;li&gt;&lt;a href="#javaScript"&gt;JavaScript&lt;/a&gt;&lt;/li&gt;                      
  *&lt;li&gt;&lt;a href="#cSharp"&gt;C Sharp (C#)&lt;/a&gt;&lt;/li&gt;                      
  *&lt;li&gt;&lt;a href="#vb"&gt;VB.Net&lt;/a&gt;&lt;/li&gt;                  
  *&lt;/ul&gt;                  
  *&lt;div id="javaScript"&gt; JavaScript (JS) is an interpreted computer programming language. 
  *It was originally implemented as part of web browsers so that client-side scripts could interact with the user, control the browser, communicate asynchronously, and alter the document content that was displayed. More recently, however, it has become common in both game development and the creation of desktop applications.                  
  *&lt;/div&gt;                  
  *&lt;div id="cSharp"&gt; C# is intended to be a simple, modern, general-purpose, object-oriented programming language. Its development team is led by Anders Hejlsberg. The most recent version is C# 5.0, which was released on August 15, 2012.                  
  *&lt;/div&gt;                  
  *&lt;div id="vb"&gt; The command-line compiler, VBC.EXE, is installed as part of the freeware .NET Framework SDK. Mono also includes a command-line VB.NET compiler. The most recent version is VB 2012, which was released on August 15, 2012.                  
  *&lt;/div&gt;              
  *&lt;/div&gt;
  * &lt;script type="text/javascript"&gt;         
             * $("#tab").ejTab({
              * width: "300px",
             *    itemAdd: function (args) {}
             * });      
             * &lt;/script&gt;  
			 * @memberof ejTab
			 * @instance
			 */
            itemAdd: null,
            /**     
			 * Triggered after tab item removed.
			 * @event
			 * @name ejTab#itemRemove 	
			 * @param {Object} argument Event parameters from button     
			 * @param {boolean} argument.cancel if the event should be canceled; otherwise, false.
			 * @param {object}  argument.model returns the tab model.
			 * @param {string}  argument.type returns the name of the event.
			 * @param {object}  argument.removedTab returns removed tab header.
			 * @param {object}  argument.removedPanel returns removed tab content panel.
			 * @example 
       *&lt;div id="tab"&gt;                  
  *&lt;ul&gt;                      
  *&lt;li&gt;&lt;a href="#javaScript"&gt;JavaScript&lt;/a&gt;&lt;/li&gt;                      
  *&lt;li&gt;&lt;a href="#cSharp"&gt;C Sharp (C#)&lt;/a&gt;&lt;/li&gt;                      
  *&lt;li&gt;&lt;a href="#vb"&gt;VB.Net&lt;/a&gt;&lt;/li&gt;                  
  *&lt;/ul&gt;                  
  *&lt;div id="javaScript"&gt; JavaScript (JS) is an interpreted computer programming language. 
  *It was originally implemented as part of web browsers so that client-side scripts could interact with the user, control the browser, communicate asynchronously, and alter the document content that was displayed. More recently, however, it has become common in both game development and the creation of desktop applications.                  
  *&lt;/div&gt;                  
  *&lt;div id="cSharp"&gt; C# is intended to be a simple, modern, general-purpose, object-oriented programming language. Its development team is led by Anders Hejlsberg. The most recent version is C# 5.0, which was released on August 15, 2012.                  
  *&lt;/div&gt;                  
  *&lt;div id="vb"&gt; The command-line compiler, VBC.EXE, is installed as part of the freeware .NET Framework SDK. Mono also includes a command-line VB.NET compiler. The most recent version is VB 2012, which was released on August 15, 2012.                  
  *&lt;/div&gt;              
  *&lt;/div&gt;
  * &lt;script type="text/javascript"&gt;         
             * $("#tab").ejTab({
              * width: "300px",
             *    itemRemove: function (args) {}
             * });      
             * &lt;/script&gt;  
			 * @memberof ejTab
			 * @instance
			 */
            itemRemove: null,
            /**     
			 * Triggered before a tab item remove.
			 * @event
			 * @name ejTab#beforeItemRemove 	
			 * @param {Object} argument Event parameters from button.     
			 * @param {boolean} argument.cancel if the event should be canceled; otherwise, false.
			 * @param {object}  argument.model returns the tab model.
			 * @param {string}  argument.type returns the name of the event.
			 * @param {number}  argument.index returns current tab item index			
			 * @example 
       *&lt;div id="tab"&gt;                  
  *&lt;ul&gt;                      
  *&lt;li&gt;&lt;a href="#javaScript"&gt;JavaScript&lt;/a&gt;&lt;/li&gt;                      
  *&lt;li&gt;&lt;a href="#cSharp"&gt;C Sharp (C#)&lt;/a&gt;&lt;/li&gt;                      
  *&lt;li&gt;&lt;a href="#vb"&gt;VB.Net&lt;/a&gt;&lt;/li&gt;                  
  *&lt;/ul&gt;                  
  *&lt;div id="javaScript"&gt; JavaScript (JS) is an interpreted computer programming language. 
  *It was originally implemented as part of web browsers so that client-side scripts could interact with the user, control the browser, communicate asynchronously, and alter the document content that was displayed. More recently, however, it has become common in both game development and the creation of desktop applications.                  
  *&lt;/div&gt;                  
  *&lt;div id="cSharp"&gt; C# is intended to be a simple, modern, general-purpose, object-oriented programming language. Its development team is led by Anders Hejlsberg. The most recent version is C# 5.0, which was released on August 15, 2012.                  
  *&lt;/div&gt;                  
  *&lt;div id="vb"&gt; The command-line compiler, VBC.EXE, is installed as part of the freeware .NET Framework SDK. Mono also includes a command-line VB.NET compiler. The most recent version is VB 2012, which was released on August 15, 2012.                  
  *&lt;/div&gt;              
  *&lt;/div&gt;
  * &lt;script type="text/javascript"&gt;         
             * $("#tab").ejTab({
              * width: "300px",
             *    beforeItemRemove: function (args) {}
             * });      
             * &lt;/script&gt;  
			 * @memberof ejTab
			 * @instance
			 */
            beforeItemRemove: null,
            /**     
    * Triggered before a tab item Create.
    * @event
    * @name ejTab#create   
    * @param {Object} argument Event parameters from button.     
    * @param {boolean} argument.cancel if the event should be canceled; otherwise, false.
    * @param {object}  argument.model returns the tab model.
    * @param {string}  argument.type returns the name of the event.
    * @param {number}  argument.deleteIndex returns current tab item index     
    * @example 
    *&lt;div id="tab"&gt;                  
*&lt;ul&gt;                      
*&lt;li&gt;&lt;a href="#javaScript"&gt;JavaScript&lt;/a&gt;&lt;/li&gt;                      
*&lt;li&gt;&lt;a href="#cSharp"&gt;C Sharp (C#)&lt;/a&gt;&lt;/li&gt;                      
*&lt;li&gt;&lt;a href="#vb"&gt;VB.Net&lt;/a&gt;&lt;/li&gt;                  
*&lt;/ul&gt;                  
*&lt;div id="javaScript"&gt; JavaScript (JS) is an interpreted computer programming language. 
*It was originally implemented as part of web browsers so that client-side scripts could interact with the user, control the browser, communicate asynchronously, and alter the document content that was displayed. More recently, however, it has become common in both game development and the creation of desktop applications.                  
*&lt;/div&gt;                  
*&lt;div id="cSharp"&gt; C# is intended to be a simple, modern, general-purpose, object-oriented programming language. Its development team is led by Anders Hejlsberg. The most recent version is C# 5.0, which was released on August 15, 2012.                  
*&lt;/div&gt;                  
*&lt;div id="vb"&gt; The command-line compiler, VBC.EXE, is installed as part of the freeware .NET Framework SDK. Mono also includes a command-line VB.NET compiler. The most recent version is VB 2012, which was released on August 15, 2012.                  
*&lt;/div&gt;              
*&lt;/div&gt;
* &lt;script type="text/javascript"&gt;         
          * $("#tab").ejTab({
           * width: "300px",
          *    create: function (args) {}
          * });      
          * &lt;/script&gt;  
    * @memberof ejTab
    * @instance
    */
            create: null,
            /**     
   * Triggered before a tab item destroy.
   * @event
   * @name ejTab#destroy   
   * @param {Object} argument Event parameters from button.     
   * @param {boolean} argument.cancel if the event should be canceled; otherwise, false.
   * @param {object}  argument.model returns the tab model.
   * @param {string}  argument.type returns the name of the event.
   * @param {number}  argument.deleteIndex returns current tab item index     
   * @example 
   *&lt;div id="tab"&gt;                  
*&lt;ul&gt;                      
*&lt;li&gt;&lt;a href="#javaScript"&gt;JavaScript&lt;/a&gt;&lt;/li&gt;                      
*&lt;li&gt;&lt;a href="#cSharp"&gt;C Sharp (C#)&lt;/a&gt;&lt;/li&gt;                      
*&lt;li&gt;&lt;a href="#vb"&gt;VB.Net&lt;/a&gt;&lt;/li&gt;                  
*&lt;/ul&gt;                  
*&lt;div id="javaScript"&gt; JavaScript (JS) is an interpreted computer programming language. 
*It was originally implemented as part of web browsers so that client-side scripts could interact with the user, control the browser, communicate asynchronously, and alter the document content that was displayed. More recently, however, it has become common in both game development and the creation of desktop applications.                  
*&lt;/div&gt;                  
*&lt;div id="cSharp"&gt; C# is intended to be a simple, modern, general-purpose, object-oriented programming language. Its development team is led by Anders Hejlsberg. The most recent version is C# 5.0, which was released on August 15, 2012.                  
*&lt;/div&gt;                  
*&lt;div id="vb"&gt; The command-line compiler, VBC.EXE, is installed as part of the freeware .NET Framework SDK. Mono also includes a command-line VB.NET compiler. The most recent version is VB 2012, which was released on August 15, 2012.                  
*&lt;/div&gt;              
*&lt;/div&gt;
* &lt;script type="text/javascript"&gt;         
         * $("#tab").ejTab({
          * width: "300px",
         *    destroy: function (args) {}
         * });      
         * &lt;/script&gt;  
   * @memberof ejTab
   * @instance
   */
            destroy: null

        },
        dataTypes: {
            cssClass: "string",
            collapsible: "boolean",
            events: "string",
            heightAdjustMode: "enum",
            selectedItemIndex: "number",
            enabled: "boolean",
            ajaxSettings: "data",
            disabledItemIndex: "data",
            enabledItemIndex: "data",
            enableAnimation: "boolean",
        },
        /**
        * destroy the tab widget
		* all events bound using this._on will be unbind automatically and bring the control to pre-init state.
		* @private
        */
        // all events bound using this._on will be unbind automatically
        _destroy: function () {
            /// <summary>This function is  used to destroy the Tab Object</summary>
            this._unWireEvents();
            this._removeBaseClass();
        },
        /**
        * To configure the properties at runtime using SetModel		
        * @private
        */
        _setModel: function (options) {
            /// <summary>This function is used to set the Tab model</summary>
            if (!this.model.enabled && ej.isNullOrUndefined(options["enabled"])) return false;
            for (var key in options) {
                switch (key) {
                    case "events": {
                        this._off(this.anchors, this.model.events);
                        this._on(this.anchors, options[key], this._tabItemClick);
                        break;
                    }
                    case "disabledItemIndex": {
                        this._disableItems(options[key]);
                        options[key] = this.model.disabledItemIndex;
                        break;
                    }
                    case "enabledItemIndex": this._enableItems(options[key]); break;
                    case "enabled": this._enabledAction(options[key]); break;
                    case "selectedItemIndex": this.showItem(options[key]); break;
                    case "heightAdjustMode": this._setTabsHeightStyle(options[key]); break;
                    case "cssClass": this._changeSkin(options[key]); break;
                    case "showRoundedCorner": this._roundedCorner(options[key]); break;
                    case "height": this.element.height(options[key]); break;
                    case "width": this.element.width(options[key]); break;
                    case "allowKeyboardNavigation": {
                        if (options[key])
                            this._on(this.element, 'keydown', this._keyPress);
                        else
                            this._off(this.element, "keydown");
                        break;
                    }
                    case "headerPosition":
                        {
                            this.model.headerPosition = options[key];
                            if (this.model.headerPosition == ej.Tab.Position.Top) {
                                this._removeVerticalClass();
                                this.itemsContainer.remove();
                                this.itemsContainer.insertBefore(this.element.find(">div").first());
                                this.element.find("div.e-active-content").removeClass("e-activebottom");
                            }
                            else if (this.model.headerPosition == ej.Tab.Position.Bottom) {
                                this._removeVerticalClass();
                                this.element.find("div.e-active-content").removeClass("e-activetop");
                            }
                            else if (this.model.headerPosition == ej.Tab.Position.Left || this.model.headerPosition == ej.Tab.Position.Right) {
                                this._removeHeaderClass();
                            }
                            this._refresh();
                            break;
                        }
                    case "showCloseButton":
                        {
                            if (options[key]) {
                                this._addDeleteIcon();
                                this._on(this.element.find("div.e-delete"), "click", this._tabDeleteClick);
                            } else
                                this.element.find("div.e-delete").remove();
                            break;
                        }
                    case "enableTabScroll":
                        {
                            if (options[key]) {
                                this._addScrollIcon();
                                this._on(this.element.find("div .e-scroll"), "click", this._tabScrollClick);
                                this.element.addClass("e-scrolltab");
                            } else
                                this.element.find("div .e-scroll").remove();
                            this.element.removeClass("e-scrolltab");
                            break;
                        }
                    case "enableAnimation":
                        {
                            this.model.enableAnimation = options[key];
                            break;
                        }
                    case "showReloadIcon":
                        {
                            if (options[key]) {
                                this._addReloadIcon();
                            } else
                                this.element.find("div.e-reload").remove();
                            break;
                        }
                    case "enableRTL":
                        {
                            if (options[key])
                                this.element.addClass("e-rtl");
                            else
                                this.element.removeClass("e-rtl");
                            break;
                        }
                }
            }
        },
        /**
        * Create the tab widget
        * @private
        */
        // constructor function
        _init: function () {
            this._addItemIndex = null;
            this.tabId = 0;
            this._initialize();
        },
        /**
        * To configure the custom theme for tab using cssClass property		
        * @private
        */
        //Skin Change at run time
        _changeSkin: function (skin) {
            if (this.model.cssClass != skin) {
                this.element.removeClass(this.model.cssClass).addClass(skin);
            }
        },

        /**
         * To initialize the tab		
		 * @private
         */
        // Render the tab control
        _initialize: function () {
            this.element.attr("tabindex", 0).attr("role", "tablist");
            this._itemsRefreshing();
            $(this.anchors).addClass("e-link");
            this._preTabIndex = -1;
            if (!ej.isNullOrUndefined(this.model.width))
                this.element.width(this.model.width);
            if (!ej.isNullOrUndefined(this.model.height))
                this.element.height(this.model.height);
            this._setTabPosition(this.model.headerPosition);
            if (this.model.showCloseButton)
                this._addDeleteIcon();
            if (this.model.showReloadIcon)
                this._addReloadIcon();
            if (this.model.showRoundedCorner)
                this._roundedCorner(this.model.showRoundedCorner);
            this._enabledAction(this.model.enabled);
            this.contentPanels = [];
            this._reinitialize();
            this._addBaseClass();
            if (!(this.model.headerPosition == "left" || this.model.headerPosition == "right"))
                this._setTabsHeightStyle(this.model.heightAdjustMode);
            this._disableTabs();
            this._roundedCorner(this.model.showRoundedCorner);
            if (this.model.enableTabScroll)
                this._addScrollIcon();
            this._wireEvents(this.model.events);
            // Added for rendering the Windows tab...
            this.showItem(this.model.selectedItemIndex);
            this._enabledAction(this.model.enabled);
        },
        /**
        * section for enable / disable the action
        * @private
        */
		_reinitialize:function()
		{
            var hid, href, hrefbase;
            for (var i = (this._addItemIndex != null) ? this._addItemIndex : 0; i < this.anchors.length; i++) {
                hid = this.anchors[i];
                href = $(hid).attr("href");
                hrefBase = href.split("#")[0];
                if (hrefBase && (hrefBase === location.toString().split("#")[0])) {
                    href = a.hash;
                    hid.href = href;
                }
                if (href && href !== "#") {
                    this._addContentTag(href, i);
                }
                else {
                    this.model.disabledItemIndex.push(i);
                }
                if (this._addItemIndex != null) { this._wireEvents(this.model.events); break; }
            }

			if(this.items.length==1)
			{
                this.showItem(this.model.selectedItemIndex);
            }

        },
		_itemsRefreshing: function()
		{
            this.itemsContainer = this.element.find("ol,ul").eq(0);
            this.items = this.itemsContainer.find(" > li:has(a[href])");
            this.anchors = this.items.find('a[href]');
        },
        _enabledAction: function (flag) {
            if (flag) {
                this.element.removeClass("e-disable");
            }
            else {
                this.element.addClass("e-disable");
            }
        },
        /**
        * To configure Tab position
        * @private
        */
        //Set the Tab position
        _setTabPosition: function (position) {
            if (position == ej.Tab.Position.Bottom) {
                this.itemsContainer.remove();
                this.element.append(this.itemsContainer);
                this.items.removeClass("e-bottom-line");
                this.items.addClass("e-top-line");
            }
            else if (position == ej.Tab.Position.Top) {
                this.items.removeClass("e-top-line");
                this.items.addClass("e-bottom-line");
            }
            else if (position == ej.Tab.Position.Left || position == ej.Tab.Position.Right) {
                if (this.items.length > 0) {
                    if (this.model.height)
                        this.itemsContainer.css("height", this.model.height);
                    else
                        this.itemsContainer.css("height", "100vh");
                    this.element.addClass("e-vertical");
                }
            }
        },
        /**
         * Section for Add delete icon for each tab
		 * @private
         */
        //Add delete icon for each tab
        _addDeleteIcon: function () {
            if (this.element.find("div.e-delete").length <= 0 && this.items.length > 0) {
                var deleteIcon = ej.buildTag('div.e-icon e-delete', "", {}, { role: "presentation" }).css("visibility", "hidden");
                if (this.model.headerPosition == "left" || this.model.headerPosition == "right") {
                    var delIconPosition = this.items.find("a");
                    deleteIcon.insertBefore(delIconPosition);
                }
                else
                    this.items.append(deleteIcon);
            }
        },
        _addScrollIcon: function () {
            if (this.element.find("div.e-scroll").length <= 0 && this.items.length > 0) {
                var scrollIcon = ej.buildTag('div.e-icon e-scroll', "", {}, { role: "presentation" }).css("visibility", "hidden");
                this.itemsContainer.append(scrollIcon);
                scrollIcon.css("position", "relative"); scrollIcon.css("margin-right", this.itemsContainer.width() - this.scrollPanelWidth+20 + "px");
                scrollIcon.css("margin-top", "-"+((this.itemsContainer.height()/2)+5)+"px");
                this.scrollstep = 30;
                this._on(this.element.find("div .e-scroll"), "click", this._tabScrollClick);
            }
        },
        _addScrollBackIcon: function () {
            if (this.element.find("div.e-right-scroll").length <= 0) {
                var scrollIcon = ej.buildTag('div.e-icon e-right-scroll', "", {}, { role: "presentation" }).css("visibility", "hidden");
                this.itemsContainer.prepend(scrollIcon);
                this.rightscrollstep = 30;
                scrollIcon.css("position", "absolute"); scrollIcon.css("margin-left", this.rightscrollstep + "px");
                scrollIcon.css("margin-top", "0px");
                this._on(this.element.find("div.e-right-scroll"), "click", this._tabScrollBackClick);
            }
        },
        /**
         * Section for Add reload icon for each tab
		 * @private
         */
        //Add reload icon for each tab
        _addReloadIcon: function () {
            if (this.element.find("div.e-reload").length <= 0 && this.items.length > 0) {
                var reloadIcon = ej.buildTag('div.e-icon e-reload', "", {}, { role: "presentation" }).css("visibility", "hidden");
                if (this.model.headerPosition == "left" || this.model.headerPosition == "right") {
                    var iconPosition = this.items.find("a");
                    reloadIcon.insertBefore(iconPosition);
                }
                else
                    this.items.append(reloadIcon);
            }
        },
        /**
         * Section for Add base class for headers and content panels
		 * @private
         */
        //Add base class for headers and content panels
        _addBaseClass: function () {
            this.element.addClass("e-widget " + this.model.cssClass);
            if (this.model.enableRTL)
                this.element.addClass("e-rtl");
            if (this.model.enableTabScroll) {
                this.element.addClass("e-scrolltab");
            }
            if (this.model.headerPosition == "left") {
                this.items.length > 0 && this.itemsContainer.addClass("e-left");
            }
            else if (this.model.headerPosition == "right") {
                this.items.length > 0 && this.itemsContainer.addClass("e-right");
            }
            else
                this.items.length > 0 && this.itemsContainer.addClass("e-header");
            this.items.addClass("e-select e-item").attr("role", "tab").attr("tabindex", -1).attr("aria-expanded", true).attr("aria-selected", false);
            $(this.contentPanels).addClass("e-content  e-content-item e-box").attr("role", "tabpanel").attr("aria-hidden", true);
            this.scrollPanelWidth = this.itemsContainer.width();
            if (this.model.enableTabScroll == true) {
                this.scrollstep = 0;
                this.itemsContainer.css("position", "absolute");
                this.itemsContainer.css("clip", "rect(0px," + (this.scrollPanelWidth + this.scrollstep) + "px,100px," + (this.scrollstep) + "px)");
                this.itemsContainer.css("margin-left", "-" + this.scrollstep + "px");
				this.itemsContainer.css("width",((parseInt(this.itemsContainer.css("width"))+(parseInt(this.items.css("width"))*this.items.length)+300))+"px");
                $(this.contentPanels).css("padding-top", "50px");
            }
        },
		_executeForwardScrolling: function(args)
		{		
            if ($("div#"+this._id+" .e-icon.e-scroll")[0] == args.target) {
                var step = this.scrollstep - this.rightscrollstep;
                this.offsetwidth = this.itemsContainer.width() - this.scrollPanelWidth;
                if (this.checkwidth == null || this.rightscrollstep == 30) {
                    this.itemsContainer.css("clip", "rect(0px," + (this.scrollPanelWidth + this.scrollstep) + "px,100px," + (this.scrollstep) + "px)");
                    this.itemsContainer.css("margin-left", "-" + this.scrollstep + "px");
                    $("div#"+this._id+" .e-icon.e-right-scroll").css("margin-left", this.scrollstep + "px");
                    if ($("div#"+this._id+" .e-icon.e-scroll") != "") $("div#"+this._id+" .e-icon.e-scroll").css("margin-right", (this.offsetwidth - this.scrollstep+20) + "px");
                }
                else {
                    this.itemsContainer.css("clip", "rect(0px," + (this.updatewidth + 30) + "px,100px," + (this.updatestep + 30) + "px)");
                    this.itemsContainer.css("margin-left", "-" + (this.updatestep + 30) + "px");
                    $("div#"+this._id+" .e-icon.e-right-scroll").css("margin-left", (this.updatestep + 30) + "px");
                    if ($("div#"+this._id+" .e-icon.e-scroll") != "") $("div#"+this._id+" .e-icon.e-scroll").css("margin-right", (parseInt($("div#"+this._id+" .e-icon.e-scroll").css("margin-right")) - 30) + "px");
                    this.updatestep += 30; this.updatewidth += 30;
                }
                this.scrolledwidth = this.scrollPanelWidth + this.scrollstep;
                this.checkwidth = this.scrolledwidth;
            } this.contentPanels.css("margin-left", "-1px");
        },
		_executeBackwardScrolling: function(args)
		{
            var step = this.scrollstep - this.rightscrollstep;
            var scrollwidth = this.scrolledwidth - this.rightscrollstep;
            {
                if (this.scrolledwidth != this.checkwidth)
                    this.itemsContainer.css("clip", "rect(0px," + (this.checkwidth - this.rightscrollstep + 30) + "px,100px," + (step - 30) + "px)");
                else
                    this.itemsContainer.css("clip", "rect(0px," + (this.checkwidth - this.rightscrollstep) + "px,100px," + (step - 30) + "px)");
                this.itemsContainer.css("margin-left", "-" + (step - 30) + "px");
                this.updatewidth = this.checkwidth - this.rightscrollstep + 30;
                this.updatestep = step - 30;
                this.contentPanels.css("margin-right", "-1px");
                if ($("div#"+this._id+" .e-icon.e-scroll") != "") $("div#"+this._id+" .e-icon.e-scroll").css("margin-right", (this.offsetwidth - step + 50) + "px");
                if (step <= 30) {
                    $("div#"+this._id+" .e-icon.e-right-scroll").css("display", "none");
                    this._addScrollIcon();
                    this.scrollstep = 0; this.rightscrollstep = 0;
                }
                else $("div#"+this._id+" .e-icon.e-right-scroll").css("margin-left", (step - 30) + "px");
                this.rightscrollstep = this.rightscrollstep + 30;
                this.checkwidth = this.scrolledwidth - 30;
            }
        },
        /**
         * Section for remove class for headers 
		 * @private
         */
        _removeHeaderClass: function () {
            this.itemsContainer.remove();
            this.itemsContainer.insertBefore(this.element.find(">div").first());
            this.items.removeClass("e-bottom-line e-top-line");
            $(this.contentPanels).removeClass("e-content-bottom e-corner-bottom e-activetop e-activebottom");
            this.itemsContainer.removeClass("e-header e-left e-right");
        },
        /**
         * Section for remove vertical class from Tab
		 * @private
         */
        _removeVerticalClass: function () {
            this.element.removeClass("e-vertical");
            this.itemsContainer.removeClass("e-left e-right").removeAttr("style");
        },
        /**
         * Section for remove base class for headers and content panels
		 * @private
         */
        //Remove base class for headers and content panels
        _removeBaseClass: function () {
            this.element.removeClass("e-tab e-widget");
            if ((this.model.headerPosition == "left" || this.model.headerPosition == "right"))
                this._removeVerticalClass();
            this.itemsContainer.removeClass("e-header e-clearall e-select");
            this.anchors.removeClass("e-link");
            this.items.removeClass("e-select e-item e-active e-bottom-line e-top-line");
            $(this.contentPanels).removeClass("e-content  e-content-item e-box");
            $(this.contentPanels).removeClass("e-content  e-content-item e-box e-content-bottom e-corner-bottom e-activetop e-activebottom e-active-content");
            $(this.contentPanels).css("display", "");
        },
        /**
         * Section for Create content tag for ajax load and new tab add function
		 * @private
         */
        // Create content tag for ajax load and new tab add function
        _addContentTag: function (href, index) {
            var id = this._getTabId(href);
            var panel = this.element.find("#" + id);
            if (!panel.length) {
                panel = ej.buildTag("div.e-content  e-content-item e-box e-content-bottom #" + id)
                    .insertAfter(this.contentPanels[index - 1] || this.itemsContainer);
            }
            this.contentPanels = $(this.contentPanels).add(panel);
        },
        /**
         * Section for enable/disable showRoundedCorner behaviour
		 * @private
         */
        _roundedCorner: function (value) {
            if (value) {
                this.element.addClass('e-corner-all');
                this.items.addClass('e-corner-top');
                $(this.contentPanels).addClass('e-corner-bottom');
            }
            else if (this.element.hasClass('e-corner-all')) {
                this.element.removeClass('e-corner-all');
                this.items.removeClass('e-corner-top');
                $(this.contentPanels).removeClass('e-corner-bottom');
            }
        },
        /**
         * To configure tab content panels height
		 * @private
         */
        // Set the tab content panels height
        _setTabsHeightStyle: function (heightFormat) {
            $(this.contentPanels).height("");
            if (ej.Tab.HeightAdjustMode.Fill == heightFormat) {
                var maxHeight = this._getDimension($(this.element).parent(), "height");
                $(this.element).parent().css({ "overflow": "auto" });
                for (var i = 0; i < this.items.length; i++) {
                    maxHeight -= this._getDimension($(this.items[i]), "outerHeight");
                }
                var maxPadding = 0;
                for (var i = 0; i < this.contentPanels.length; i++) {
                    maxPadding = Math.max(maxPadding, this._getDimension($(this.contentPanels[i]), "innerHeight") - this._getDimension($(this.contentPanels[i]), "height"));
                }
                $(this.contentPanels).height(maxHeight - maxPadding).css({ "overflow": "auto" });
            }
            else if (ej.Tab.HeightAdjustMode.Auto == heightFormat) {
                maxHeight = 0;
                for (var i = 0; i < this.contentPanels.length; i++) {
                    maxHeight = Math.max(maxHeight, this._getDimension($(this.contentPanels[i]), "outerHeight"));
                }
                $(this.contentPanels).height(maxHeight);
                this.maxAutoHeight = maxHeight;
            }
        },
        _getDimension: function (element, method) {
            var value;
            var $hidden = $(element).parents().andSelf().filter(':hidden');
            var prop = { visibility: 'hidden', display: 'block' };
            var hiddenCollection = [];
            $hidden.each(function () {
                var hidden = {}, name;
                for (name in prop) {
                    hidden[name] = this.style[name];
                    this.style[name] = prop[name];
                }
                hiddenCollection.push(hidden);
            });
            value = /(outer)/g.test(method) ?
            $(element)[method](true) :
           $(element)[method]();

            $hidden.each(function (i) {
                var hidden = hiddenCollection[i], name;
                for (name in prop) {
                    this.style[name] = hidden[name];
                }
            });
            return value;
        },
        // Tab to active given index value
        showItem: function (index) {
            if ($.inArray(index, this.model.disabledItemIndex) < 0) {
                var proxy = this;
                this._preTabIndex = this.model.selectedItemIndex;
                this.model.selectedItemIndex = index;
                if (this.model.selectedItemIndex >= this.contentPanels.length) {
                    this.model.selectedItemIndex = 0;
                }
                $(this.items[this.model.selectedItemIndex]).attr("aria-expanded", true).attr("aria-selected", true).attr("tabindex", 0);
                if (this.model.selectedItemIndex != null && this.model.selectedItemIndex < this.contentPanels.length) {
                    this._ajaxLoad();
                    this.hideItem(this._preTabIndex);
                    if (index >= 0)
                        if (this._onBeforeActive(index))
                            return false;
                    $(this.contentPanels[this.model.selectedItemIndex]).fadeIn(this.model.enableAnimation ? 200 : 0, "easeOutQuad", function () {
                        if (proxy._onActive())
                            return true;
                    });
                    if (!(this.model.headerPosition == "left" || this.model.headerPosition == "right"))
                        var activeClass = this.model.headerPosition == ej.Tab.Position.Top ? "e-activetop" : "e-activebottom";
                    else
                        $(this.contentPanels[this.model.selectedItemIndex]).addClass("e-active-content ");
                    $(this.items[this.model.selectedItemIndex]).addClass("e-active").removeClass("e-select");
                    $(this.contentPanels[this.model.selectedItemIndex]).addClass("e-active-content " + activeClass).removeAttr("aria-hidden", false);
                    //     $(".e-content.e-content-item.e-active-content").css("padding-top","50px");
                    //$(this.items[this._preTabIndex]).attr("aria-expanded", false).attr("tabindex", -1);
                }
            }
        },
        // Tab to hide given index value
        hideItem: function (index) {
            $(this.contentPanels[index]).fadeOut(0);
            if (!(this.model.headerPosition == "left" || this.model.headerPosition == "right"))
                var activeClass = this.model.headerPosition == ej.Tab.Position.Top ? "e-activetop" : "e-activebottom";
            $(this.items[index]).removeClass("e-active").addClass("e-select");
            $(this.contentPanels[index]).removeClass("e-active-content " + activeClass).attr("aria-hidden", true);
        },
        /**
         * To configure Ajax load
		 * @private
         */
        //Ajax load function
        _ajaxLoad: function () {
            var content = $(this.contentPanels[this.model.selectedItemIndex]);
            var link = this.anchors[this.model.selectedItemIndex];
            var href = $(link).attr("href");
            if (content.is(':empty') && href.indexOf("#") !== 0)
                this._sendAjaxOptions(content, link);
        },
        /**
         * Section for get tab id 
		 * @private
         */
        //get tab id for new added tabs
        _getTabId: function (href) {
            return !href.indexOf("#") ? href.replace("#", "") : this.model.idPrefix + this._getNextTabId();
        },
        /**
         * Section for get Next tab id 
		 * @private
         */
        _getNextTabId: function () {
            return ++this.tabId;
        },
        /**
         * Section for Disable tab 
		 * @private
         */
        // Disable tabs in given index
        _disableTabs: function () {
            for (var i = 0, li; (li = this.items[i]) ; i++) {
                if ($.inArray(i, this.model.disabledItemIndex) > -1) {
                    $(li).find("a").unbind(this.model.events);
                    this._off($(li).find("div.e-delete"), "click");
                    $(this.contentPanels[i]).addClass("e-disable");
                }
                $(li)[$.inArray(i, this.model.disabledItemIndex) != -1 &&
                    !$(li).hasClass("e-tab-selected") ? "addClass" : "removeClass"]("e-disable");
            }
        },
        /**
         * Section for handle click event
		 * @private
         */
        // Tabs event handler
        _tabItemClick: function (args) {
            if (this.model.enabled) {
                args.preventDefault(); // Prevent the ancher tag url action
                var index;
                if (this.model.selectedItemIndex == $(this.items).index($(args.currentTarget)) && this.model.collapsible) {
                    index = -1;
                }
                else {
                    index = $(this.items).index($(args.currentTarget));
                }
                if (index != this.model.selectedItemIndex)
                    this.showItem(index);
            }
        },
        /**
         * Section for handle click event on tab delete icon
		 * @private
         */
        _tabDeleteClick: function (args) {
            if (this.model.enabled) {
                var currentTab = $(args.target);
                if (currentTab.hasClass("e-delete"))
                    var index = $(this.items).index($(args.target).parent());
                if (index == this.model.selectedItemIndex && this.items.length > index)
                    this.model.selectedItemIndex++;
                this.removeItem(index);
            }
        },
        _tabScrollClick: function (args) {
            if (this.model.enabled) {
                if (($("div#"+this._id+" .e-icon.e-scroll")[0] == args.target)) {
                    this._executeForwardScrolling(args);
                    this.scrollstep = this.scrollstep + 30;
                    this._addScrollBackIcon();
					$("div#"+this._id+" .e-icon.e-right-scroll").css("display","block");
                }
            }
        },
        _tabScrollBackClick: function (args) {
            if (this.model.enabled) {
                if (($("div.e-icon.e-right-scroll")[0] == args.target)) {
                    this._executeBackwardScrolling(args);
                }
            }
        },
        /**
        * Section for handle click event
        * @private
        */
        _tabReloadClick: function (args) {
            if (this.model.enabled) {
                var currentTab = $(args.target);
                if (currentTab.hasClass("e-reload")) {
                    var link = this.anchors[this.model.selectedItemIndex];
                    var href = $(link).attr("href");
                    var content = $(this.contentPanels[this.model.selectedItemIndex]);
                    if (href.indexOf("#") !== 0)
                        this._sendAjaxOptions(content, link);
                    else
                        this.showItem(this.model.selectedItemIndex);
                }
            }
        },
        /**
        * Section for handle ajax request
        * @private
        */
        //handling the ajax request
        _sendAjaxOptions: function (content, link) {
            //load waiting popup
            if (this._onBeforeLoad(link))
                return true;
            content.addClass("e-load");
            var proxy = this;
            var curTabTitle = $(link).html();
            var hrefLink = link.href.replace("#", "");
            var ajaxOptions = {
                type: this.model.ajaxSettings.type, cache: this.model.ajaxSettings.cache, url: hrefLink, data: this.model.ajaxSettings.data,
                dataType: this.model.ajaxSettings.dataType, contentType: this.model.ajaxSettings.contentType, async: this.model.ajaxSettings.async,
                "success": function (data) {
                    try {
                        proxy._ajaxSuccessHandler(data, content, link, curTabTitle);
                    } catch (e) {

                    }
                }, "error": function () {
                    try {
                        proxy._ajaxErrorHandler(link, proxy.model.selectedItemIndex, curTabTitle);
                    } catch (e) {

                    }
                }
            };
            //$.extend(true, {}, proto.prototype.defaults, options);
            //this.model.ajaxOptions = $.extend(true, _ajaxOptions, this.model.ajaxOptions);
            this._sendAjaxRequest(ajaxOptions);
        },
        // Ajax Request function
        _sendAjaxRequest: function (ajaxOptions) {
            $.ajax({
                type: ajaxOptions.type,
                cache: ajaxOptions.cache,
                url: ajaxOptions.url,
                dataType: ajaxOptions.dataType,
                data: ajaxOptions.data,
                contentType: ajaxOptions.contentType,
                async: ajaxOptions.async,
                success: ajaxOptions.success,
                error: ajaxOptions.error,
                beforeSend: ajaxOptions.beforeSend,
                complete: ajaxOptions.complete
            });
        },
        /**
        * Section for handle ajax success
        * @private
        */
        //Ajax sucess function
        _ajaxSuccessHandler: function (data, content, link, curTabTitle) {
            if (curTabTitle != null)
                $(link).html(curTabTitle);
            content.removeClass("e-load");
            content.html(data).addClass("e-tab-loaded"); //to indicate the content is already loaded
            var eventData = { data: data, url: link, content: content };
            this._trigger("ajaxSuccess", eventData);
            if (this._onLoad(link))
                return true;
        },
        /**
        * Section for handle ajax error
        * @private
        */
        // ajax error function
        _ajaxErrorHandler: function (data, link, index, title) {
            this._trigger("ajaxError", { data: data, url: link });
            this._onLoad(link);
        },
        /**
         * Section for create content panel
		 * @private
         */
        _createContentPanel: function (id) {
            return $('<div></div>')
				.attr("id", id)
				.addClass("e-content  e-content-item e-content-bottom e-box");
        },
        /**
         * Section for Refresh conrol.
		 * @private
         */
        // Refresh conrol
        _refresh: function () {
            this._unWireEvents();
            this._initialize();
        },
        /**
         * Section to watch keypress action.		
		 * @private
         */
        _keyPress: function (e) {
            if (this.model.enabled) {
                var index, currentEle, targetEle = $(e.target);
                if (e.keyCode) code = e.keyCode; // ie and mozilla/gecko
                else if (e.which) code = e.which; // ns4 and opera
                else code = e.charCode;
                if (targetEle.hasClass("e-link") || targetEle.hasClass("e-item")) {
                    switch (code) {
                        //Right arrow
                        case 39:
                        case 38:
                            {
                                e.preventDefault();
                                this.showItem(this.model.selectedItemIndex + 1);
                                break;
                            }
                            //Left arrow
                        case 37:
                        case 40:
                            {
                                e.preventDefault();
                                if (this.model.selectedItemIndex != 0)
                                    this.showItem(this.model.selectedItemIndex - 1);
                                else if (this.model.selectedItemIndex == 0)
                                    this.showItem(this.getItemsCount() - 1);
                                break;
                            }
                            //End key
                        case 35:
                            {
                                e.preventDefault();
                                this.showItem(this.getItemsCount() - 1);
                                break;
                            }
                            //Home key
                        case 36:
                            {
                                e.preventDefault();
                                this.showItem(0);
                                break;
                            }
                    }
                }
                else if (e.ctrlKey && !targetEle.hasClass("e-tab")) {
                    switch (code) {
                        case 38:
                            e.preventDefault();
                            index = $(this.contentPanels).index(targetEle.parent(".e-content"));
                            currentEle = $(this.items[index]);
                            break;
                        case 33:
                            e.preventDefault();
                            currentEle = $(this.items[0]);
                            this.showItem(0);
                            break;
                        case 34:
                            e.preventDefault();
                            currentEle = $(this._headers[this.getItemsCount() - 1]);
                            this.showItem(this.getItemsCount() - 1);
                            break;
                    }
                }
                if (!ej.isNullOrUndefined(currentEle)) {
                    currentEle.addClass("e-focus");
                    currentEle.focus();
                }
            }
        },
        /**
         * Section to watch hover action on tab.		
		 * @private
         */
        _hoverHandler: function (args) {
            args.preventDefault();
            if (this.model.enabled) {
                var index = $(this.items).index($(args.target).parent());
                if (index == -1)
                    index = $(this.items).index($(args.target));
                args.type === "mouseout" ? $(this.element.find("div.e-delete")[index]).css("visibility", "hidden") : $(this.element.find("div.e-delete")[index]).css("visibility", "visible");
                args.type === "mouseout" ? $(this.element.find("div.e-scroll")).css("visibility", "hidden") : $(this.element.find("div.e-scroll")).css("visibility", "visible");
                args.type === "mouseout" ? $(this.element.find("div.e-right-scroll")).css("visibility", "hidden") : $(this.element.find("div.e-right-scroll")).css("visibility", "visible");
                args.type === "mouseout" ? $(this.element.find("div.e-reload")[index]).css("visibility", "hidden") : $(this.element.find("div.e-reload")[index]).css("visibility", "visible");
            }
        },
        //-------------------- Event Wire-up -------------------------//
        /**
        * Wiring the events to tab control		
        * @private
        */
        _wireEvents: function (event) {
            //tabs Click evnts
            this._on(this.items, event, this._tabItemClick);
            this._on(this.itemsContainer, "mouseover", this._hoverHandler);
            this._on(this.itemsContainer, "mouseout", this._hoverHandler);
            this._on(this.element.find("div.e-delete"), "click", this._tabDeleteClick);
            this._on(this.element.find("div.e-scroll"), "click", this._tabScrollClick);
            this._on(this.element.find("div.e-right-scroll"), "click", this._tabScrollBackClick);
            this._on(this.itemsContainer, "focusin", this._focusIn);
            this._on(this.itemsContainer, "focusout", this._focusOut);
            this._on(this.element.find("div.e-reload"), "click", this._tabReloadClick);
        },
        //-------------------- Event UnWire-up -------------------------//
        /**
        * UnWiring the events from tab control		
        * @private
        */
        _unWireEvents: function () {
            this._off(this.anchors, this.model.events);
            this._off(this.element.find("div.e-delete"), "click");
            this._off(this.element.find("div.e-scroll"), "click");
            this._off(this.element.find("div.e-right-scroll"), "click");
            this._off(this.itemsContainer, "mouseover", this._hoverHandler);
            this._off(this.itemsContainer, "mouseout", this._hoverHandler);
            this._off(this.itemsContainer, "focusin", this._focusIn);
            this._off(this.itemsContainer, "focusout", this._focusOut);
            this._off(this.element.find("div.e-reload"), "click");
        },
        //--------------------- Client side Methods ----------------------//


        _disableItems: function (indexes) {
            /// <summary>This function disable the given item of the tab control</summary>
            /// <param name="index" type="int"></param>
            if (!this.model.enabled) return false;
            if (indexes != null) {
                for (var i = 0; i < indexes.length; i++) {
                    if ($.inArray(indexes[i], this.model.disabledItemIndex) == -1)
                        this.model.disabledItemIndex.push(indexes[i]);
                }
                this.model.disabledItemIndex.sort();
                this._disableTabs();
            }
        },

        _enableItems: function (indexes) {
            /// <summary>This function enable the given item of the tab control</summary>
            /// <param name="index" type="int"></param>
            if (!this.model.enabled) return false;
            for (var i = 0; i < indexes.length; i++) {
                var index = indexes[i];
                !ej.isNullOrUndefined(this.contentPanels[index]) && $(this.contentPanels[index]).removeClass("e-disable");
                if ($.inArray(index, this.model.disabledItemIndex) == -1) {
                    return false;
                }
                this.model.disabledItemIndex = $.grep(this.model.disabledItemIndex, function (n, i) {
                    return n != index;
                });
            }
            this._refresh();
            this._disableTabs();
        },
        //Deprecate this methods with enableItems()
        /**
       * Given index header and content panel disabled.
       * @alias disable
       * @return jQuery
	   * @param {number} index  index of tab item.
       * @example 
       *&lt;div id="tab"&gt;                  
  *&lt;ul&gt;                      
  *&lt;li&gt;&lt;a href="#javaScript"&gt;JavaScript&lt;/a&gt;&lt;/li&gt;                      
  *&lt;li&gt;&lt;a href="#cSharp"&gt;C Sharp (C#)&lt;/a&gt;&lt;/li&gt;                      
  *&lt;li&gt;&lt;a href="#vb"&gt;VB.Net&lt;/a&gt;&lt;/li&gt;                  
  *&lt;/ul&gt;                  
  *&lt;div id="javaScript"&gt; JavaScript (JS) is an interpreted computer programming language. 
  *It was originally implemented as part of web browsers so that client-side scripts could interact with the user, control the browser, communicate asynchronously, and alter the document content that was displayed. More recently, however, it has become common in both game development and the creation of desktop applications.                  
  *&lt;/div&gt;                  
  *&lt;div id="cSharp"&gt; C# is intended to be a simple, modern, general-purpose, object-oriented programming language. Its development team is led by Anders Hejlsberg. The most recent version is C# 5.0, which was released on August 15, 2012.                  
  *&lt;/div&gt;                  
  *&lt;div id="vb"&gt; The command-line compiler, VBC.EXE, is installed as part of the freeware .NET Framework SDK. Mono also includes a command-line VB.NET compiler. The most recent version is VB 2012, which was released on August 15, 2012.                  
  *&lt;/div&gt;              
  *&lt;/div&gt;

  * &lt;script type="text/javascript"&gt;  
 
  * $("#tab").ejTab({width:"300px"});      
       * //initialize the tab object
       * var tabObj = $("#tab").data("ejTab");
       *	// disable the item from the Tab based on the index
       *	tabObj.disable();
       * &lt;/script&gt;  
       * @example 
       *&lt;div id="tab"&gt;                  
  *&lt;ul&gt;                      
  *&lt;li&gt;&lt;a href="#javaScript"&gt;JavaScript&lt;/a&gt;&lt;/li&gt;                      
  *&lt;li&gt;&lt;a href="#cSharp"&gt;C Sharp (C#)&lt;/a&gt;&lt;/li&gt;                      
  *&lt;li&gt;&lt;a href="#vb"&gt;VB.Net&lt;/a&gt;&lt;/li&gt;                  
  *&lt;/ul&gt;                  
  *&lt;div id="javaScript"&gt; JavaScript (JS) is an interpreted computer programming language. 
  *It was originally implemented as part of web browsers so that client-side scripts could interact with the user, control the browser, communicate asynchronously, and alter the document content that was displayed. More recently, however, it has become common in both game development and the creation of desktop applications.                  
  *&lt;/div&gt;                  
  *&lt;div id="cSharp"&gt; C# is intended to be a simple, modern, general-purpose, object-oriented programming language. Its development team is led by Anders Hejlsberg. The most recent version is C# 5.0, which was released on August 15, 2012.                  
  *&lt;/div&gt;                  
  *&lt;div id="vb"&gt; The command-line compiler, VBC.EXE, is installed as part of the freeware .NET Framework SDK. Mono also includes a command-line VB.NET compiler. The most recent version is VB 2012, which was released on August 15, 2012.                  
  *&lt;/div&gt;              
  *&lt;/div&gt;
  * &lt;script type="text/javascript"&gt;  
  * $("#tab").ejTab({width:"300px"});       
       * $("#tab").ejTab("disable");
       * &lt;/script&gt;  
       * @memberof ejTab
       * @instance
       */
        disable: function () {
            var indexes = [];
            for (var index = 0; index < this.getItemsCount() ; index++) {

                indexes.push(index);
            }
            this._disableItems(indexes);
            this.model.enabledItemIndex = [];
            this._unWireEvents();
        },
        /**
       * Given index header and content panel enabled.
       * @alias enable
       * @return jQuery
	   * @param {number} index  index of tab item.
       * @example 
       *&lt;div id="tab"&gt;                  
  *&lt;ul&gt;                      
  *&lt;li&gt;&lt;a href="#javaScript"&gt;JavaScript&lt;/a&gt;&lt;/li&gt;                      
  *&lt;li&gt;&lt;a href="#cSharp"&gt;C Sharp (C#)&lt;/a&gt;&lt;/li&gt;                      
  *&lt;li&gt;&lt;a href="#vb"&gt;VB.Net&lt;/a&gt;&lt;/li&gt;                  
  *&lt;/ul&gt;                  
  *&lt;div id="javaScript"&gt; JavaScript (JS) is an interpreted computer programming language. 
  *It was originally implemented as part of web browsers so that client-side scripts could interact with the user, control the browser, communicate asynchronously, and alter the document content that was displayed. More recently, however, it has become common in both game development and the creation of desktop applications.                  
  *&lt;/div&gt;                  
  *&lt;div id="cSharp"&gt; C# is intended to be a simple, modern, general-purpose, object-oriented programming language. Its development team is led by Anders Hejlsberg. The most recent version is C# 5.0, which was released on August 15, 2012.                  
  *&lt;/div&gt;                  
  *&lt;div id="vb"&gt; The command-line compiler, VBC.EXE, is installed as part of the freeware .NET Framework SDK. Mono also includes a command-line VB.NET compiler. The most recent version is VB 2012, which was released on August 15, 2012.                  
  *&lt;/div&gt;              
  *&lt;/div&gt;
  * &lt;script type="text/javascript"&gt;   
  * $("#tab").ejTab({width:"300px"});      
       * //initialize the tab object
       *	var tabObj = $("#tab").data("ejTab");
       *	// enable the item from the Tab based on the index
       *	tabObj.enable();
       * &lt;/script&gt;  
       * @example 
       *&lt;div id="tab"&gt;                  
  *&lt;ul&gt;                      
  *&lt;li&gt;&lt;a href="#javaScript"&gt;JavaScript&lt;/a&gt;&lt;/li&gt;                      
  *&lt;li&gt;&lt;a href="#cSharp"&gt;C Sharp (C#)&lt;/a&gt;&lt;/li&gt;                      
  *&lt;li&gt;&lt;a href="#vb"&gt;VB.Net&lt;/a&gt;&lt;/li&gt;                  
  *&lt;/ul&gt;                  
  *&lt;div id="javaScript"&gt; JavaScript (JS) is an interpreted computer programming language. 
  *It was originally implemented as part of web browsers so that client-side scripts could interact with the user, control the browser, communicate asynchronously, and alter the document content that was displayed. More recently, however, it has become common in both game development and the creation of desktop applications.                  
  *&lt;/div&gt;                  
  *&lt;div id="cSharp"&gt; C# is intended to be a simple, modern, general-purpose, object-oriented programming language. Its development team is led by Anders Hejlsberg. The most recent version is C# 5.0, which was released on August 15, 2012.                  
  *&lt;/div&gt;                  
  *&lt;div id="vb"&gt; The command-line compiler, VBC.EXE, is installed as part of the freeware .NET Framework SDK. Mono also includes a command-line VB.NET compiler. The most recent version is VB 2012, which was released on August 15, 2012.                  
  *&lt;/div&gt;              
  *&lt;/div&gt;
  * &lt;script type="text/javascript"&gt;     
  * $("#tab").ejTab({width:"300px"});    
       * $("#tab").ejTab("enable");
       * &lt;/script&gt;  
       * @memberof ejTab
       * @instance
       */
        enable: function () {
            var indexes = [];
            for (var index = 0; index < this.getItemsCount() ; index++) {
                if ($.inArray(index, this.model.enabledItemIndex) < 0) {
                    this.model.enabledItemIndex.push(index);
                    indexes.push(index);
                }
                this._enableItems(index);
            }
            this.model.disabledItemIndex = [];

        },
        /**
       * This function get the number of tab rendered
       * @alias getItemsCount
       * @return number
       * @example 
       *&lt;div id="tab"&gt;                  
  *&lt;ul&gt;                      
  *&lt;li&gt;&lt;a href="#javaScript"&gt;JavaScript&lt;/a&gt;&lt;/li&gt;                      
  *&lt;li&gt;&lt;a href="#cSharp"&gt;C Sharp (C#)&lt;/a&gt;&lt;/li&gt;                      
  *&lt;li&gt;&lt;a href="#vb"&gt;VB.Net&lt;/a&gt;&lt;/li&gt;                  
  *&lt;/ul&gt;                  
  *&lt;div id="javaScript"&gt; JavaScript (JS) is an interpreted computer programming language. 
  *It was originally implemented as part of web browsers so that client-side scripts could interact with the user, control the browser, communicate asynchronously, and alter the document content that was displayed. More recently, however, it has become common in both game development and the creation of desktop applications.                  
  *&lt;/div&gt;                  
  *&lt;div id="cSharp"&gt; C# is intended to be a simple, modern, general-purpose, object-oriented programming language. Its development team is led by Anders Hejlsberg. The most recent version is C# 5.0, which was released on August 15, 2012.                  
  *&lt;/div&gt;                  
  *&lt;div id="vb"&gt; The command-line compiler, VBC.EXE, is installed as part of the freeware .NET Framework SDK. Mono also includes a command-line VB.NET compiler. The most recent version is VB 2012, which was released on August 15, 2012.                  
  *&lt;/div&gt;              
  *&lt;/div&gt;
  * &lt;script type="text/javascript"&gt;    
  * $("#tab").ejTab({width:"300px"});     
       * //initialize the tab object
       *	var tabObj = $("#tab").data("ejTab");
       *	// get the number of tab rendered
       *	tabObj.getItemsCount();
       * &lt;/script&gt;  
       * @example 
       *&lt;div id="tab"&gt;                  
  *&lt;ul&gt;                      
  *&lt;li&gt;&lt;a href="#javaScript"&gt;JavaScript&lt;/a&gt;&lt;/li&gt;                      
  *&lt;li&gt;&lt;a href="#cSharp"&gt;C Sharp (C#)&lt;/a&gt;&lt;/li&gt;                      
  *&lt;li&gt;&lt;a href="#vb"&gt;VB.Net&lt;/a&gt;&lt;/li&gt;                  
  *&lt;/ul&gt;                  
  *&lt;div id="javaScript"&gt; JavaScript (JS) is an interpreted computer programming language. 
  *It was originally implemented as part of web browsers so that client-side scripts could interact with the user, control the browser, communicate asynchronously, and alter the document content that was displayed. More recently, however, it has become common in both game development and the creation of desktop applications.                  
  *&lt;/div&gt;                  
  *&lt;div id="cSharp"&gt; C# is intended to be a simple, modern, general-purpose, object-oriented programming language. Its development team is led by Anders Hejlsberg. The most recent version is C# 5.0, which was released on August 15, 2012.                  
  *&lt;/div&gt;                  
  *&lt;div id="vb"&gt; The command-line compiler, VBC.EXE, is installed as part of the freeware .NET Framework SDK. Mono also includes a command-line VB.NET compiler. The most recent version is VB 2012, which was released on August 15, 2012.                  
  *&lt;/div&gt;              
  *&lt;/div&gt;
  * &lt;script type="text/javascript"&gt;    
  * $("#tab").ejTab({width:"300px"});     
       * $("#tab").ejTab("getItemsCount");
       * &lt;/script&gt;  
       * @memberof ejTab
       * @instance
       */
        getItemsCount: function () {
            /// <summary>This function get the number of tab rendered</summary>
            if (this.items) {
                return this.items.length;
            }
        },
        /**
        * Add new tab items with given name, url and given index position, if index null it’s add last item.
		* @alias addItem
		* @return jQuery
		* @param {string} url  URL name / tab id.
		* @param {string} displayLabel  Tab Display name.
		* @param {number} index  Index position to placed , this is optional.
		* @example 
    *&lt;div id="tab"&gt;                  
  *&lt;ul&gt;                      
  *&lt;li&gt;&lt;a href="#javaScript"&gt;JavaScript&lt;/a&gt;&lt;/li&gt;                      
  *&lt;li&gt;&lt;a href="#cSharp"&gt;C Sharp (C#)&lt;/a&gt;&lt;/li&gt;                      
  *&lt;li&gt;&lt;a href="#vb"&gt;VB.Net&lt;/a&gt;&lt;/li&gt;
                  
  *&lt;/ul&gt;                  
  *&lt;div id="javaScript"&gt; JavaScript (JS) is an interpreted computer programming language. 
  *It was originally implemented as part of web browsers so that client-side scripts could interact with the user, control the browser, communicate asynchronously, and alter the document content that was displayed. More recently, however, it has become common in both game development and the creation of desktop applications.                  
  *&lt;/div&gt;                  
  *&lt;div id="cSharp"&gt; C# is intended to be a simple, modern, general-purpose, object-oriented programming language. Its development team is led by Anders Hejlsberg. The most recent version is C# 5.0, which was released on August 15, 2012.                  
  *&lt;/div&gt;                  
  *&lt;div id="vb"&gt; The command-line compiler, VBC.EXE, is installed as part of the freeware .NET Framework SDK. Mono also includes a command-line VB.NET compiler. The most recent version is VB 2012, which was released on August 15, 2012.                  
  *&lt;/div&gt;  
  *&lt;div id="new"&gt; The command-line compiler, VBC.EXE, is installed as part of the freeware .NET Framework SDK. Mono also includes a command-line VB.NET compiler. The most recent version is VB 2012, which was released on August 15, 2012.                  
  *&lt;/div&gt;             
  *&lt;/div&gt;
  * &lt;script type="text/javascript"&gt;         
		* //initialize the tab object
    * $("#tab").ejTab({width:"400px"});
		*	var tabObj = $("#tab").data("ejTab");
		*	// Add new tab items with given list
		*	tabObj.addItem("#new","New Item",3);
    * &lt;/script&gt;  
		* @example 
    *&lt;div id="tab"&gt;                  
  *&lt;ul&gt;                      
  *&lt;li&gt;&lt;a href="#javaScript"&gt;JavaScript&lt;/a&gt;&lt;/li&gt;                      
  *&lt;li&gt;&lt;a href="#cSharp"&gt;C Sharp (C#)&lt;/a&gt;&lt;/li&gt;                      
  *&lt;li&gt;&lt;a href="#vb"&gt;VB.Net&lt;/a&gt;&lt;/li&gt;                  
  *&lt;/ul&gt;                  
  *&lt;div id="javaScript"&gt; JavaScript (JS) is an interpreted computer programming language. 
  *It was originally implemented as part of web browsers so that client-side scripts could interact with the user, control the browser, communicate asynchronously, and alter the document content that was displayed. More recently, however, it has become common in both game development and the creation of desktop applications.                  
  *&lt;/div&gt;                  
  *&lt;div id="cSharp"&gt; C# is intended to be a simple, modern, general-purpose, object-oriented programming language. Its development team is led by Anders Hejlsberg. The most recent version is C# 5.0, which was released on August 15, 2012.                  
  *&lt;/div&gt;                  
  *&lt;div id="vb"&gt; The command-line compiler, VBC.EXE, is installed as part of the freeware .NET Framework SDK. Mono also includes a command-line VB.NET compiler. The most recent version is VB 2012, which was released on August 15, 2012.                  
  *&lt;/div&gt; 
  *&lt;div id="new"&gt; The command-line compiler, VBC.EXE, is installed as part of the freeware .NET Framework SDK. Mono also includes a command-line VB.NET compiler. The most recent version is VB 2012, which was released on August 15, 2012.                  
  *&lt;/div&gt;              
  *&lt;/div&gt;
  * &lt;script type="text/javascript"&gt;  
  * $("#tab").ejTab({width:"400px"});       
        * $("#tab").ejTab("addItem","#new","New Item",3); 
        * &lt;/script&gt;  
		* @memberof ejTab
		* @instance
        */
        addItem: function (url, displayLabel, index, cssClass) {
            /// <summary>This function new tab item in dynamicaly</summary>
            /// <param name="url" type="String">URL name / tab id</param>
            /// <param name="displayLabel" type="String"> Tab Display name</param>
            /// <param name="index" type="int">Index position to placed , this is optional</param>
            (index >= 0 && index < this.items.length) ? this._addItemIndex = index : this._addItemIndex = this.items.length;
			for(var disable_index=0;disable_index<this.model.disabledItemIndex.length;disable_index++)
			{
                if (this.model.disabledItemIndex[disable_index] >= index)
                    this.model.disabledItemIndex[disable_index]++;
            }
            if (this.model.headerPosition == "left") {
                this.items.length >= 0 && this.itemsContainer.addClass("e-left");
            }
            else if (this.model.headerPosition == "right") {
                this.items.length >= 0 && this.itemsContainer.addClass("e-right");
            }
            else
                this.items.length == 0 && this.itemsContainer.addClass("e-header");
            var liTag = ej.buildTag("li.e-select e-item");
            if (!ej.isNullOrUndefined(cssClass)) {
                var span = ej.buildTag('span').addClass(cssClass);
                liTag.append(span);
            }
            var aTag = ej.buildTag("a", displayLabel, {}, { href: url });
            aTag.appendTo(liTag);
            if (this.model.showCloseButton) {
                var deleteIcon = ej.buildTag('div.e-icon e-delete ');
                liTag.append(deleteIcon);
                this._on(deleteIcon, "click", this._tabDeleteClick);
            }
            if (index === undefined) {
                index = this.anchors.length;
            }
            var insertIndex = index >= this.items.length;
            if (insertIndex) {
                liTag.appendTo(this.itemsContainer);
            } else {
                liTag.insertBefore(this.items[index]);
            }
            if (this.model.selectedItemIndex == index) {
                this.hideItem[index];
                this.model.selectedItemIndex += 1;
            } else {
                this.hideItem[index];
                if (index < this.model.selectedItemIndex)
                    this.model.selectedItemIndex += 1;
            }
            this._itemsRefreshing();
            this._reinitialize();
            var data = {
                tabHeader: this.anchors[index],
                tabContent: this.contentPanels[index]
            };
            this._addItemIndex = null;
            this._onAdd(data);
        },
        /**
       * Remove the given index tab item.
       * @alias removeItem
       * @return jQuery
	   * @param {number} index  index of tab item.
       * @example 
       *&lt;div id="tab"&gt;                  
  *&lt;ul&gt;                      
  *&lt;li&gt;&lt;a href="#javaScript"&gt;JavaScript&lt;/a&gt;&lt;/li&gt;                      
  *&lt;li&gt;&lt;a href="#cSharp"&gt;C Sharp (C#)&lt;/a&gt;&lt;/li&gt;                      
  *&lt;li&gt;&lt;a href="#vb"&gt;VB.Net&lt;/a&gt;&lt;/li&gt;                  
  *&lt;/ul&gt;                  
  *&lt;div id="javaScript"&gt; JavaScript (JS) is an interpreted computer programming language. 
  *It was originally implemented as part of web browsers so that client-side scripts could interact with the user, control the browser, communicate asynchronously, and alter the document content that was displayed. More recently, however, it has become common in both game development and the creation of desktop applications.                  
  *&lt;/div&gt;                  
  *&lt;div id="cSharp"&gt; C# is intended to be a simple, modern, general-purpose, object-oriented programming language. Its development team is led by Anders Hejlsberg. The most recent version is C# 5.0, which was released on August 15, 2012.                  
  *&lt;/div&gt;                  
  *&lt;div id="vb"&gt; The command-line compiler, VBC.EXE, is installed as part of the freeware .NET Framework SDK. Mono also includes a command-line VB.NET compiler. The most recent version is VB 2012, which was released on August 15, 2012.                  
  *&lt;/div&gt;              
  *&lt;/div&gt;
  * &lt;script type="text/javascript"&gt;    
  * $("#tab").ejTab({width:"300px"});     
       * //initialize the tab object
       *	var tabObj = $("#tab").data("ejTab");
       *	// Remove the given index tab item.
       *	tabObj.removeItem(1);
       * &lt;/script&gt;  
       * @example 
       *&lt;div id="tab"&gt;                  
  *&lt;ul&gt;                      
  *&lt;li&gt;&lt;a href="#javaScript"&gt;JavaScript&lt;/a&gt;&lt;/li&gt;                      
  *&lt;li&gt;&lt;a href="#cSharp"&gt;C Sharp (C#)&lt;/a&gt;&lt;/li&gt;                      
  *&lt;li&gt;&lt;a href="#vb"&gt;VB.Net&lt;/a&gt;&lt;/li&gt;                  
  *&lt;/ul&gt;                  
  *&lt;div id="javaScript"&gt; JavaScript (JS) is an interpreted computer programming language. 
  *It was originally implemented as part of web browsers so that client-side scripts could interact with the user, control the browser, communicate asynchronously, and alter the document content that was displayed. More recently, however, it has become common in both game development and the creation of desktop applications.                  
  *&lt;/div&gt;                  
  *&lt;div id="cSharp"&gt; C# is intended to be a simple, modern, general-purpose, object-oriented programming language. Its development team is led by Anders Hejlsberg. The most recent version is C# 5.0, which was released on August 15, 2012.                  
  *&lt;/div&gt;                  
  *&lt;div id="vb"&gt; The command-line compiler, VBC.EXE, is installed as part of the freeware .NET Framework SDK. Mono also includes a command-line VB.NET compiler. The most recent version is VB 2012, which was released on August 15, 2012.                  
  *&lt;/div&gt;              
  *&lt;/div&gt;
  * &lt;script type="text/javascript"&gt; 
  * $("#tab").ejTab({width:"300px"});        
       * $("#tab").ejTab("removeItem",1); 
       * &lt;/script&gt;  
       * @memberof ejTab
       * @instance
       */
        removeItem: function (index) {
            /// <summary>This function Remove the given index</summary>
            /// <param name="index" type="int"></param>
            if (!this.model.enabled) return false;
            if (index != null && index > -1 && index < this.items.length) {
                if (this._onBrforeRemove({ index: index }) === true)
                    return false;
                var removedTab = $(this.items[index]).remove();
				for(var disable_index=0;disable_index<this.model.disabledItemIndex.length;disable_index++)
				{
                    if (this.model.disabledItemIndex[disable_index] > index)
                        this.model.disabledItemIndex[disable_index]--;
				else if(this.model.disabledItemIndex[disable_index]==index)
				{
                        this.model.disabledItemIndex.splice(disable_index, 1);
                    }
                }
                if (removedTab.hasClass("e-active")) {
                    this.model.selectedItemIndex = (index == 0) ? index + 1 : (index - 1);
                    this.showItem(this.model.selectedItemIndex);
                }
                this.element.find(">div.e-content")[index].remove();
                this.contentPanels.splice(index, 1);
                this.model.selectedItemIndex = index < this.model.selectedItemIndex ? this.model.selectedItemIndex - 1 : this.model.selectedItemIndex;

                if (index < 0 || index >= this.anchors.length) {
                    this.model.selectedItemIndex = 0;
                }
                if ((this.model.headerPosition == "left" || this.model.headerPosition == "right") && this.items.length == 1)
                    this._removeVerticalClass();
                else
                    this.items.length == 1 && this.itemsContainer.removeClass("e-header");
                this._unWireEvents();
                this._itemsRefreshing();
                this._wireEvents();
                this._disableTabs();
                var data = {
                    removedTab: removedTab
                };
                this._onRemove(data);
            }
        },
        /**
       * This function is to show the tab control.
       * @alias show
       * @return jQuery
       * @example 
       *&lt;div id="tab"&gt;                  
  *&lt;ul&gt;                      
  *&lt;li&gt;&lt;a href="#javaScript"&gt;JavaScript&lt;/a&gt;&lt;/li&gt;                      
  *&lt;li&gt;&lt;a href="#cSharp"&gt;C Sharp (C#)&lt;/a&gt;&lt;/li&gt;                      
  *&lt;li&gt;&lt;a href="#vb"&gt;VB.Net&lt;/a&gt;&lt;/li&gt;                  
  *&lt;/ul&gt;                  
  *&lt;div id="javaScript"&gt; JavaScript (JS) is an interpreted computer programming language. 
  *It was originally implemented as part of web browsers so that client-side scripts could interact with the user, control the browser, communicate asynchronously, and alter the document content that was displayed. More recently, however, it has become common in both game development and the creation of desktop applications.                  
  *&lt;/div&gt;                  
  *&lt;div id="cSharp"&gt; C# is intended to be a simple, modern, general-purpose, object-oriented programming language. Its development team is led by Anders Hejlsberg. The most recent version is C# 5.0, which was released on August 15, 2012.                  
  *&lt;/div&gt;                  
  *&lt;div id="vb"&gt; The command-line compiler, VBC.EXE, is installed as part of the freeware .NET Framework SDK. Mono also includes a command-line VB.NET compiler. The most recent version is VB 2012, which was released on August 15, 2012.                  
  *&lt;/div&gt;              
  *&lt;/div&gt;
  * &lt;script type="text/javascript"&gt; 
  * $("#tab").ejTab({width:"300px"});        
       * //initialize the tab object
       *	var tabObj = $("#tab").data("ejTab");
       *	// To show the tab control.
       *	tabObj.show();
       * &lt;/script&gt;  
       * @example 
       *&lt;div id="tab"&gt;                  
  *&lt;ul&gt;                      
  *&lt;li&gt;&lt;a href="#javaScript"&gt;JavaScript&lt;/a&gt;&lt;/li&gt;                      
  *&lt;li&gt;&lt;a href="#cSharp"&gt;C Sharp (C#)&lt;/a&gt;&lt;/li&gt;                      
  *&lt;li&gt;&lt;a href="#vb"&gt;VB.Net&lt;/a&gt;&lt;/li&gt;                  
  *&lt;/ul&gt;                  
  *&lt;div id="javaScript"&gt; JavaScript (JS) is an interpreted computer programming language. 
  *It was originally implemented as part of web browsers so that client-side scripts could interact with the user, control the browser, communicate asynchronously, and alter the document content that was displayed. More recently, however, it has become common in both game development and the creation of desktop applications.                  
  *&lt;/div&gt;                  
  *&lt;div id="cSharp"&gt; C# is intended to be a simple, modern, general-purpose, object-oriented programming language. Its development team is led by Anders Hejlsberg. The most recent version is C# 5.0, which was released on August 15, 2012.                  
  *&lt;/div&gt;                  
  *&lt;div id="vb"&gt; The command-line compiler, VBC.EXE, is installed as part of the freeware .NET Framework SDK. Mono also includes a command-line VB.NET compiler. The most recent version is VB 2012, which was released on August 15, 2012.                  
  *&lt;/div&gt;              
  *&lt;/div&gt;
  * &lt;script type="text/javascript"&gt;    
  * $("#tab").ejTab({width:"300px"});     
       * $("#tab").ejTab("show"); 
       * &lt;/script&gt;  
       * @memberof ejTab
       * @instance
       */
        show: function () {
            /// <summary>To Show the Tab control</summary>
            if (!this.model.enabled) return false;
            this.element.css("visibility", "visible");
        },
        /**
       * This function hides the tab control.
       * @alias hide
       * @return jQuery
       * @example 
       *&lt;div id="tab"&gt;                  
  *&lt;ul&gt;                      
  *&lt;li&gt;&lt;a href="#javaScript"&gt;JavaScript&lt;/a&gt;&lt;/li&gt;                      
  *&lt;li&gt;&lt;a href="#cSharp"&gt;C Sharp (C#)&lt;/a&gt;&lt;/li&gt;                      
  *&lt;li&gt;&lt;a href="#vb"&gt;VB.Net&lt;/a&gt;&lt;/li&gt;                  
  *&lt;/ul&gt;                  
  *&lt;div id="javaScript"&gt; JavaScript (JS) is an interpreted computer programming language. 
  *It was originally implemented as part of web browsers so that client-side scripts could interact with the user, control the browser, communicate asynchronously, and alter the document content that was displayed. More recently, however, it has become common in both game development and the creation of desktop applications.                  
  *&lt;/div&gt;                  
  *&lt;div id="cSharp"&gt; C# is intended to be a simple, modern, general-purpose, object-oriented programming language. Its development team is led by Anders Hejlsberg. The most recent version is C# 5.0, which was released on August 15, 2012.                  
  *&lt;/div&gt;                  
  *&lt;div id="vb"&gt; The command-line compiler, VBC.EXE, is installed as part of the freeware .NET Framework SDK. Mono also includes a command-line VB.NET compiler. The most recent version is VB 2012, which was released on August 15, 2012.                  
  *&lt;/div&gt;              
  *&lt;/div&gt;
  * &lt;script type="text/javascript"&gt;   
  * $("#tab").ejTab({width:"300px"});      
       * //initialize the tab object
       *	var tabObj = $("#tab").data("ejTab");
       *	// To hide the tab control..
       *	tabObj.hide();
       * &lt;/script&gt;  
       * @example 
       *&lt;div id="tab"&gt;                  
  *&lt;ul&gt;                      
  *&lt;li&gt;&lt;a href="#javaScript"&gt;JavaScript&lt;/a&gt;&lt;/li&gt;                      
  *&lt;li&gt;&lt;a href="#cSharp"&gt;C Sharp (C#)&lt;/a&gt;&lt;/li&gt;                      
  *&lt;li&gt;&lt;a href="#vb"&gt;VB.Net&lt;/a&gt;&lt;/li&gt;                  
  *&lt;/ul&gt;                  
  *&lt;div id="javaScript"&gt; JavaScript (JS) is an interpreted computer programming language. 
  *It was originally implemented as part of web browsers so that client-side scripts could interact with the user, control the browser, communicate asynchronously, and alter the document content that was displayed. More recently, however, it has become common in both game development and the creation of desktop applications.                  
  *&lt;/div&gt;                  
  *&lt;div id="cSharp"&gt; C# is intended to be a simple, modern, general-purpose, object-oriented programming language. Its development team is led by Anders Hejlsberg. The most recent version is C# 5.0, which was released on August 15, 2012.                  
  *&lt;/div&gt;                  
  *&lt;div id="vb"&gt; The command-line compiler, VBC.EXE, is installed as part of the freeware .NET Framework SDK. Mono also includes a command-line VB.NET compiler. The most recent version is VB 2012, which was released on August 15, 2012.                  
  *&lt;/div&gt;              
  *&lt;/div&gt;
  * &lt;script type="text/javascript"&gt;   
  * $("#tab").ejTab({width:"300px"});      
       * $("#tab").ejTab("hide"); 
       * &lt;/script&gt;  
       * @memberof ejTab
       * @instance
       */
        hide: function () {
            /// <summary>To Hide the Tab control</summary>
            if (!this.model.enabled) return false;
            this.element.css("visibility", "hidden");
        },
        //--------------------- End - Client side Methods ----------------------//

        //--------------------- Client side events ----------------------//
        /**
         * Section For handling before load event
		 * @private
         */
        _onBeforeLoad: function (link) {
            var data;
            if (this.model.selectedItemIndex == -1 && this.model.collapsible) {
                data = {
                    prevActiveHeader: this.items[this._preTabIndex],
                    prevActiveIndex: this._preTabIndex,
                    activeHeader: null,
                    activeIndex: null,
                    url: link
                };
            }
            else
                data = {
                    prevActiveHeader: this.items[this._preTabIndex],
                    prevActiveIndex: this._preTabIndex,
                    activeHeader: this.items[this.model.selectedItemIndex],
                    activeIndex: this.model.selectedItemIndex,
                    url: link
                };
            return this._trigger("ajaxBeforeLoad", data);
        },
        /**
        * Section For handle the focusin event
        * @private
        */
        _focusIn: function () {
            if (!this.model.readOnly && this.model.allowKeyboardNavigation)
                $(this.element).bind("keydown", $.proxy(this._keyPress, this));
        },
        /**
         * Section For handle the focus out event
		 * @private
         */
        _focusOut: function (e) {
            $(this.element).unbind("keydown", $.proxy(this._keyPress, this));
        },
        /**
         * Section For handle the load event
		 * @private
         */
        _onLoad: function (link) {
            var data;
            if (this.model.selectedItemIndex == -1 && this.model.collapsible) {
                data = {
                    prevActiveHeader: this.items[this._preTabIndex],
                    prevActiveIndex: this._preTabIndex,
                    activeHeader: null,
                    activeIndex: null,
                    url: link
                };
            }
            else
                data = {
                    prevActiveHeader: this.items[this._preTabIndex],
                    prevActiveIndex: this._preTabIndex,
                    activeHeader: this.items[this.model.selectedItemIndex],
                    activeIndex: this.model.selectedItemIndex,
                    url: link
                };
            return this._trigger("ajaxLoad", data);
        },
        /**
        * Section For handle the Active event
        * @private
        */
        _onActive: function () {
            var data;
            if (this.model.selectedItemIndex == -1 && this.model.collapsible) {
                data = {
                    prevActiveHeader: this.items[this._preTabIndex],
                    prevActiveIndex: this._preTabIndex,
                    activeHeader: null,
                    activeIndex: null
                };
            }
            else
                data = {
                    prevActiveHeader: this.items[this._preTabIndex],
                    prevActiveIndex: this._preTabIndex,
                    activeHeader: this.items[this.model.selectedItemIndex],
                    activeIndex: this.model.selectedItemIndex
                };
            return this._trigger("itemActive", data);
        },
        /**
        * Section For handle the Before itemActive event.
        * @private
        */
        _onBeforeActive: function (index) {
            if (this.model.beforeActive != null) {
                var data;
                if (this.model.selectedItemIndex == -1 && this.model.collapsible) {
                    data = {
                        prevActiveHeader: this.items[this._preTabIndex],
                        prevActiveIndex: this._preTabIndex,
                        activeHeader: null,
                        activeIndex: null
                    };
                }
                else
                    data = {
                        prevActiveHeader: this.items[this._preTabIndex],
                        prevActiveIndex: this._preTabIndex,
                        activeHeader: this.items[index],
                        activeIndex: index
                    };
                return this._trigger("beforeActive", data);
            }
        },
        /**
        * Section For trigger add event.
        * @private
        */
        _onAdd: function (data) {
            return this._trigger("itemAdd", data);
        },
        /**
         * Section For trigger remove event.
		 * @private
         */
        _onRemove: function (data) {
            return this._trigger("itemRemove", data);
        },
        /**
         * Section For trigger before remove event.
		 * @private
         */
        _onBrforeRemove: function (data) {
            return this._trigger("beforeItemRemove", data);
        }
    });
    /**
 * Enum for tab height Style.	 
 * @enum {string}
 * @global 
 */
    ej.Tab.HeightAdjustMode = {
        /**  Panel height adjusts based on the content */
        Content: "content",
        /**  All panel height will be set the tallest panel height. */
        Auto: "auto",
        /**  Content panel take based on the parent height. */
        Fill: "fill"
    };
    /**
 * Enum for Tab Position	 
 * @enum {string}
 * @global 
 */
    ej.Tab.Position = {
        /**  Tab headers display to top position. */
        Top: "top",
        /**  Tab headers display to bottom position. */
        Bottom: "bottom",
        /**  Tab headers display to left position. */
        Left: "left",
        /** Tab headers display to right position. */
        Right: "right"
    };

})(jQuery, Syncfusion);;
/*!
*  filename: ej.tagcloud.js
*  version : 12.1
*  Copyright Syncfusion Inc. 2001 - 2013. All rights reserved.
*  Use of this code is subject to the terms of our license.
*  A copy of the current license can be obtained at any time by e-mailing
*  licensing@syncfusion.com. Any infringement will be prosecuted under
*  applicable laws. 
*/

/**
* @fileOverview Plugin to style the tagCloud control.
* @copyright Copyright Syncfusion Inc. 2001 - 2013. All rights reserved.
*  Use of this code is subject to the terms of our license.
*  A copy of the current license can be obtained at any time by e-mailing
*  licensing@syncfusion.com. Any infringement will be prosecuted under
*  applicable laws. 
* @version 12.1 
* @author <a href="mailto:licensing@syncfusion.com">Syncfusion Inc</a>
*/
(function ($, ej, undefined) {
    // ejTagcloud is the plugin name 
    // "ej.Tagcloud" is "namespace.className" will hold functions and properties
    /**
    * @namespace ej
	* @class ejTagCloud
	* @requires jQuery
	* @requires ej.core.js
    * @requires ej.data.js
    * @requires ej.tagcloud.js
	* @classdesc The TagCloud allows the user to display a list of links or tags with a structured cloud format where the importance of the tags can differentiate with varied font sizes, colors, and styles.
	* @example 
	* &lt;div id="tagcloud"&gt;&lt;/div&gt; <br> 
    * &lt;script&gt; 
	*$(function () {
	*	// document ready
	*	// initialize the array of data for tagcloud input	
	* // simple tagcloud creation
	*	$("#tagcloud").ejTagCloud({ 
	*             dataSource:  window.websiteCollection,
	*             titleText: "Tech Sites"
	*       });
	*});
    * &lt;/script&gt;
    */
    ej.widget("ejTagCloud", "ej.TagCloud", {
        // widget element will be automatically set in this
        element: null,

        // user defined model will be automatically set in this
        model: null,
        validTags: ["div", "span"],
        _rootCSS: "e-tagcloud",
        _setFirst: false,

        // default model
        defaults: {
            /**		
			* Specify the CSS class to button to achieve custom theme.
			* @default ""
			* @type {string}
			* @example 
            * &lt;div id="tagcloud"&gt;&lt;/div&gt; <br> 
            * &lt;script&gt;
			* // Set the CSS class during initialization. 			
			* 	$("#tagcloud").ejTagCloud({dataSource: window.websiteCollection,cssClass  : "gradient-lime",titleText: "Tech Sites" });			 			
            * &lt;/script&gt;
			 * @memberof ejTagCloud
			* @instance
			*/
            cssClass : "",
            /**		
			* The dataSource contains the list of data to display in a cloud format. Each data contains a link url, frequency to categorize the font size and a display text.
			* @default null
			* @type {object}
			* @example 
            * &lt;div id="tagcloud"&gt;&lt;/div&gt; <br> 
            * &lt;script&gt;
			* //Initialize the TagCloud with the dataSource value specified		           
            * $("#tagcloud").ejTagCloud({ dataSource: window.websiteCollection,titleText: "Tech Sites" });	
            * &lt;/script&gt;
			 * @memberof ejTagCloud
			* @instance
			*/
            dataSource: null,
            /**		
			* Define the query to retrieve the data from online server. The query is used only when the online dataSource is used.
			* @default null
			* @type {object}
			* @example 
            * &lt;div id="tagcloud"&gt;&lt;/div&gt; <br> 
            * &lt;script&gt;
			* //Initialize the TagCloud with the query value specified		
            * var dataManger = ej.DataManager({
		    *       url:"http://mvc.syncfusion.com/Services/Northwnd.svc/"
			*});
			* var query = ej.Query().from("Orders").take(10);    
        * $("#tagcloud").ejTagCloud({
        *        titleText: "Employee List",
        *       dataSource: dataManger,
        *      query: query,
        *     fields: { text: "CustomerID" , frequency: "EmployeeID" }
        * });
    * &lt;/script&gt;
			 * @memberof ejTagCloud
			* @instance
			*/
            query: null,
            /**		
			* Defines the mapping fields for the data items of the TagCloud.
			* @default null 
			* @type {Object}
			* @param {Object} options Defines the mapping fields for the data items of the TagCloud.
			* @param {String} options.text - This component's name, sets {@link Component#text}
            * @param {String} options.url - Whether this component is vislble, sets {@link Component#url}
			* @example 
      * &lt;div id="tagcloud"&gt;&lt;/div&gt; <br> 
      * &lt;script&gt;
			  * $(function () {
     * // Initialize the TagCloud with the fields value specified    
            
     * $("#tagcloud").ejTagCloud({
       * dataSource:window.websiteCollection,
       titleText: "Tech Sites",
      * fields: { text: "text" , url:"url",frequency: "frequency"}  
     * });
      * });

     * &lt;/script&gt;
			* @memberof ejTagCloud
			* @instance
			*/
            fields: /** @lends ejTagCloud# */ { 
				/**		
                 * Defines the tag value or display text.
				 * @alias ejTagCloud#fields->text
				 * @type String
                 */
			text: "text", 
				/**		
                 * Defines the url link to navigate while click the tag.
				 * @alias ejTagCloud#fields->url
				 * @type String
                 */
			url: "url", 
				/**		
                 * Defines the frequency number to categorize the font size.
				 * @alias ejTagCloud#fields->frequency
				 * @type Number
                 */
			frequency: "frequency" 
			},
            /**		
           * Shows or hides the TagCloud title. When this set to false, it hides the TagCloud header.
           * @default true
           * @type {boolean}
           * @example 
           * &lt;div id="tagcloud"&gt;&lt;/div&gt; <br> 
           * &lt;script&gt;
           * // Set the showTitle during initialization. 			
           * 	$("#tagcloud").ejTagCloud({     dataSource: window.websiteCollection, showTitle : false  });			 
          
          * &lt;/script&gt;
            * @memberof ejTagCloud
           * @instance
           */
            showTitle: true,
            /**		
			* Sets the title text for the TagCloud. To show the title text, the showTitle property should be enabled.
			* @default "Title"
			* @type {string}
			* @example 
      * &lt;div id="tagcloud"&gt;&lt;/div&gt; <br> 
    * &lt;script&gt;
			* // Set the titleText during initialization. 			
			* 	$("#tagcloud").ejTagCloud({  dataSource:  window.websiteCollection,titleText : "Cloud Data"  });			 
		 
        * &lt;/script&gt;
			 * @memberof ejTagCloud
			* @instance
			*/
            titleText: "Title",
            /**		
			* Sets the title image for the TagCloud. To show the title image, the showTitle property should be enabled.
			* @default null
			* @type {string}
			* @example 
      * &lt;div id="tagcloud"&gt;&lt;/div&gt; <br> 
      * &lt;script&gt;
			* // Set the titleImage during initialization. 			
			* 	$("#tagcloud").ejTagCloud({  dataSource: window.websiteCollection,titleImage: '../images/bird.png' ,titleText: "Tech Sites"  });			 
			 
      * &lt;/script&gt;
			 * @memberof ejTagCloud
			* @instance
			*/
            titleImage: null,
            /**		
			* Defines the format for the TagCloud to display the tag items.See {@link Format} 
			* @default ej.Format.Cloud
			* @type {String | Enum}
			* @example 
      * &lt;div id="tagcloud"&gt;&lt;/div&gt; <br> 
        * &lt;script&gt;
			* // Set the format during initialization. 			
			* 	$("#tagcloud").ejTagCloud({ dataSource:  window.websiteCollection,format: ej.Format.Cloud,titleText: "Tech Sites" });			
			
    * &lt;/script&gt;
			 * @memberof ejTagCloud
			* @instance
			*/
            format: "cloud",
            /**		
           * Sets the TagCloud and tag items direction as right to left alignment.
           * @default false
           * @type {boolean}
           * @example 
           * &lt;div id="tagcloud"&gt;&lt;/div&gt; <br> 
          * &lt;script&gt;
           * // Set the enableRTL during initialization. 			
           * 	$("#tagcloud").ejTagCloud({ dataSource:  window.websiteCollection,enableRTL : false,titleText: "Tech Sites"  });			 
          
          * &lt;/script&gt;
            * @memberof ejTagCloud
           * @instance
           */
            enableRTL: false,
            /**		
           * Sets the minimum font size value for the tag items. The font size for the tag items will be generated in between the minimum and maximum font size values.
           * @default "10px"
           * @type {String | Number}
           * @example 
           * &lt;div id="tagcloud"&gt;&lt;/div&gt; <br> 
          * &lt;script&gt;
           * // Set the minFontSize during initialization. 			
           * 	$("#tagcloud").ejTagCloud({dataSource:  window.websiteCollection, minFontSize : "10px" ,titleText: "Tech Sites" });			 
          
            * &lt;/script&gt;
            * @memberof ejTagCloud
           * @instance
           */
            minFontSize: "10px",
            /**		
           * Sets the maximum font size value for the tag items. The font size for the tag items will be generated in between the minimum and maximum font size values.
           * @default "40px"
           * @type {String | Number}
           * @example 
           * &lt;div id="tagcloud"&gt;&lt;/div&gt; <br> 
            * &lt;script&gt;
           * // Set the maxFontSize during initialization. 			
           * 	$("#tagcloud").ejTagCloud({ dataSource:  window.websiteCollection,maxFontSize : "10px" ,titleText: "Tech Sites" });			 
           
            * &lt;/script&gt;
            * @memberof ejTagCloud
           * @instance
           */
            maxFontSize: "40px",

            //Events
            /**     
            * Event triggers when the cursor hovers on a tag item
            * @event
            * @name ejTagCloud#mouseover 	
            * @param {Object} argument Event parameters from button     
            * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
            * @param {object}  argument.model returns the tagcloud model
            * @param {string}  argument.type returns the name of the event
            * @param {string}  argument.text return current tag name
            * @param {string}  argument.url return current url link
            * @example 
            * &lt;div id="tagcloud"&gt;&lt;/div&gt; <br> 
          * &lt;script&gt;
            * //mouse over event for tagCloud
            * $("#tagcloud").ejTagCloud({
                 *  dataSource: window.websiteCollection,
                 *titleText: "Tech Sites",
            *    mouseover: function (args) {}
            * });   

           * &lt;/script&gt;   
            * @memberof ejTagCloud
            * @instance
            */
            mouseover: null,
            /**     
            * Event triggers when the cursor leaves out from a tag item
            * @event
            * @name ejTagCloud#mouseout 	
            * @param {Object} argument Event parameters from button     
            * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
            * @param {object}  argument.model returns the tagcloud model
            * @param {string}  argument.type returns the name of the event
            * @param {string}  argument.text return current tag name
            * @param {string}  argument.url return current url link
            * @example 
            * &lt;div id="tagcloud"&gt;&lt;/div&gt; <br> 
          * &lt;script&gt;
            * //mouseout event for tagCloud
            * $("#tagcloud").ejTagCloud({
                *  dataSource: window.websiteCollection,
                *  titleText: "Tech Sites",
            *    mouseout: function (args) {}
            * });           
            * &lt;/script&gt;     
            * @memberof ejTagCloud
            * @instance
            */
            mouseout: null,
            /**     
            * Event triggers when the TagCloud items are clicked
            * @event
            * @name ejTagCloud#click 	
            * @param {Object} argument Event parameters from button     
            * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
            * @param {object}  argument.model returns the tagcloud model
            * @param {string}  argument.type returns the name of the event
            * @param {string}  argument.text return current tag name
            * @param {string}  argument.url return current url link
            * @example 
            * &lt;div id="tagcloud"&gt;&lt;/div&gt; <br> 
            * &lt;script&gt;
            * //click event for tagCloud
            * $("#tagcloud").ejTagCloud({
                *  dataSource: window.websiteCollection,
                *  titleText: "Tech Sites",
            *    click: function (args) {}
            * });        
            * &lt;/script&gt;    
            * @memberof ejTagCloud
            * @instance
            */
            click: null,
            /**     
            * Event triggers when the TagCloud are created
            * @event
            * @name ejTagCloud#create 	
            * @param {Object} argument Event parameters from button     
            * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
            * @param {object}  argument.model returns the tagcloud model
            * @param {string}  argument.type returns the name of the event
            * @example 
            * &lt;div id="tagcloud"&gt;&lt;/div&gt; <br> 
            * &lt;script&gt;
            * //create event for tagCloud
            * $("#tagcloud").ejTagCloud({
                *  dataSource: window.websiteCollection,
                *  titleText: "Tech Sites",
            *    create: function (args) {}
            * });        
            * &lt;/script&gt;    
            * @memberof ejTagCloud
            * @instance
            */
            create: null,
            /**     
            * Event triggers when the TagCloud are destroyed
            * @event
            * @name ejTagCloud#destroy 	
            * @param {Object} argument Event parameters from button     
            * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
            * @param {object}  argument.model returns the tagcloud model
            * @param {string}  argument.type returns the name of the event
            * @example 
            * &lt;div id="tagcloud"&gt;&lt;/div&gt; <br> 
            * &lt;script&gt;
            * //destroy event for tagCloud
            * $("#tagcloud").ejTagCloud({
                *  dataSource: window.websiteCollection,
                *  titleText: "Tech Sites",
            *    destroy: function (args) {}
            * });        
            * &lt;/script&gt;    
            * @memberof ejTagCloud
            * @instance
            */
            destroy: null
        },
        /**
      * Specify the data types for default properties 
      * @private
      */
        dataTypes: {
            cssClass : "string",
            showTitle: "boolean",
            titleText: "string",
            titleImage: "string",
            format: "enum",
            enableRTL: "boolean",
            dataSource: "data",
            query: "data",
            fields: "data"
        },

        // constructor function
        /**
       * Create the TagCloud widget
       * @private
       */
        _init: function () {
            this._initialize();
            this._render();
        },
        /**
        * To configure the properties at runtime using SetModel		
        * @private
        */
        _setModel: function (options) {
            var option;
            for (option in options) {
                switch (option) {
                    case "fields":
                    case "query":
                    case "dataSource":
                    case "minFontSize":
                    case "maxFontSize":
                        this._refreshTagItems(option, options[option]); break;
                    case "showTitle": this._showTitle(options[option]); break;
                    case "titleText": this._title(options[option]); break;
                    case "titleImage": this._titleImage(options[option]); break;
                    case "cssClass ": this._changeSkin(options[option]); break;
                    case "format": this._format(options[option]); break;
                    case "enableRTL": this._rtl(options[option]); break;
                }
            }
        },

        _refreshTagItems: function (key, value) {
            this.model[key] = value;
            this.ul.empty();
            this._checkDataBinding();
        },
        /**
         * Section For enable or disable the title
		 * @private
         */
        _showTitle: function (boolean) {
            if (boolean) this._generateTitle();
            else {
                this.titleText.remove();
                this.titleText = null;
            }
        },
        /**
        * To configure TagCloud Title		
        * @private
        */
        _title: function (text) {
            if (this.titleText) {
                if (text) {
                    if (this.text) this.text.html(text);
                    else this._generateTextTag(text);
                }
                else if (this.text) {
                    this.text.remove();
                    this.text = null;
                }
            }
        },
        /**
         * Section For Sets the title image for the TagCloud
		 * @private
         */
        _titleImage: function (imagePath) {
            if (this.titleText) {
                if (imagePath) {
                    if (this.image) this.image.attr("src", imagePath);
                    else this._generateImageTag(imagePath);
                }
                else if (this.image) {
                    this.image.remove();
                    this.image = null;
                }
            }
        },
        /**
        * To configure the custom theme for TagCloud using cssClass  property		
        * @private
        */
        _changeSkin: function (skin) {
            if (this.model.cssClass  != skin) {
                this.element.removeClass(this.model.cssClass ).addClass(skin);
            }
        },
        /**
        * Inserts a new item into the TagCloud
		* @alias insert
		* @return jQuery
		* @example 
	
    * &lt;div id="tagcloud"&gt;&lt;/div&gt; <br> 
      * &lt;script&gt;
   *   var tagObj="";
  * var tag = { text: "google", url: "http://www.google.com", frequency: 12 };
      * $(function () {
  * // document ready
  * // initialize the array of data for tagcloud input
  * $("#tagcloud").ejTagCloud({ 
  *             dataSource: window.websiteCollection,
  *             titleText: "Tech Sites"
  * });
 * //initialize the tagCloud object
 * tagObj = $("#tagcloud").data("ejTagCloud");
* //To Inserts a new item into the TagCloud
 * tagObj.insert(tag);
  *       });     
  * &lt;/script&gt;
       
		* @example 
		
     * &lt;div id="tagcloud"&gt;&lt;/div&gt; <br> 
    * &lt;script&gt;
    * var tag = { text: "google", url: "http://www.google.com", frequency: 12 };
    *$(function () {
  * // document ready
  * // initialize the array of data for tagcloud input  
  * $("#tagcloud").ejTagCloud({ 
  *             dataSource: window.websiteCollection,titleText: "Tech Sites"
  * });  
         * $("#tagcloud").ejTagCloud("insert", tag);        
   * });
        * &lt;/script&gt;
		* @memberof ejTagCloud
		* @instance
        */
        insert: function (tag) {
            if ($.trim(tag.text)) {
                this.ul.append(this._generateLi(tag, this._getMapper()));
            }
        },
        /**
        * Inserts a new item into the TagCloud at a particular position. 
		* @alias insertAt
		* @return jQuery
		* @example 
  * &lt;div id="tagcloud"&gt;&lt;/div&gt; <br> 
      * &lt;script&gt;
      * var tag = { text: "google", url: "http://www.google.com", frequency: 12 };
      * var tagObj="";
      *$(function () {
  * // document ready 
  * $("#tagcloud").ejTagCloud({
  * dataSource:window.websiteCollection,
  * titleText: "Tech Sites"
    * });
   *  tagObj = $("#tagcloud").data("ejTagCloud");
   tagObj.insertAt(tag,2);

    * });
      * &lt;/script&gt;
		* @example 
    * &lt;div id="tagcloud"&gt;&lt;/div&gt; <br> 
      * &lt;script&gt;
* var tag = { text: "google", url: "http://www.google.com", frequency: 12 };
      *$(function () {
  * // document ready 
      * $("#tagcloud").ejTagCloud({
      * dataSource:window.websiteCollection,
      * titleText: "Tech Sites"
      * }); 
        * $("#tagcloud").ejTagCloud("insertAt", tag, 2);
        * });
        
      * &lt;/script&gt;
		* @memberof ejTagCloud
		* @instance
        */
        insertAt: function (tag, position) {
            if ($.trim(tag.text)) {
                $(this.ul.children()[position - 1]).before(this._generateLi(tag, this._getMapper()));
            }
        },
        /**
        * Removes the item from the TagCloud based on the name. It removes all the tags which have the corresponding name 
		* @alias remove
		* @return jQuery
		 * @param {string} name  name of the tag.
		* @example 
    
    * &lt;div id="tagcloud"&gt;&lt;/div&gt; <br> 
      * &lt;script&gt;
   *   var tagObj="";
  * var tag = { text: "google", url: "http://www.google.com", frequency: 12 };
      * $(function () {
  * // document ready
  * // initialize the array of data for tagcloud input
  
  
  * $("#tagcloud").ejTagCloud({ 
  *             dataSource: window.websiteCollection,
  *             titleText: "Tech Sites"
  * });
 * //initialize the tagCloud object
 * tagObj = $("#tagcloud").data("ejTagCloud");
* //To Inserts a new item into the TagCloud
 * tagObj.remove(tag);
  *       });     
  * &lt;/script&gt;
		* @example 
     * &lt;div id="tagcloud"&gt;&lt;/div&gt; <br> 
    * &lt;script&gt;
    * var tag = { text: "google", url: "http://www.google.com", frequency: 12 };
    *$(function () {
  * // document ready
  * // initialize the array of data for tagcloud input

  
  * $("#tagcloud").ejTagCloud({ 
  *             dataSource: window.websiteCollection,
  *             titleText: "Tech Sites"
  * });
  
         * $("#tagcloud").ejTagCloud("remove", tag);
        
   * });
        * &lt;/script&gt;

		* @memberof ejTagCloud
		* @instance
        */
        remove: function (txt) {
            var li = this.ul.children(), liTag, i;
            for (i = 0; i < li.length; i++) {
                liTag = $(li[i]);
                if (liTag.children()[0].innerHTML == txt)
                    liTag.remove();
            }
        },
        /**
        * Removes the item from the TagCloud based on the position. It removes the tags from the the corresponding position only. 
		* @alias removeAt
		* @return jQuery
		* @param {number} position  position of tag item.
		* @example 
        * &lt;div id="tagcloud"&gt;&lt;/div&gt; <br> 
      * &lt;script&gt;
      * var tagObj = "";
      * var tag = { text: "google", url: "http://www.google.com", frequency: 12 };
      *$(function () {       
  * // document ready   
           * $("#tagcloud").ejTagCloud({
        * dataSource:window.websiteCollection,
        * titleText: "Tech Sites"
          * });
   
    * //initialize the tagCloud object
    * var tagObj = $("#tagcloud").data("ejTagCloud");
    * // Removes the item from the TagCloud based on the position.
  
* tagObj.removeAt(2);
    * });
  
    * &lt;/script&gt;
		* @example 
     &lt;div id="tagcloud"&gt;&lt;/div&gt; <br> 
      * &lt;script&gt;
      *$(function () {
      * // document ready
      
        * $("#tagcloud").ejTagCloud({
        * dataSource:window.websiteCollection,
        * titleText: "Tech Sites"
        * });
         * $("#tagcloud").ejTagCloud("removeAt", 4);
        * });
      * &lt;/script&gt;
		* @memberof ejTagCloud
		* @instance
        */
        removeAt: function (position) {
            var li = this.ul.children();
            $(li[position - 1]).remove();
        },
        /**
        * To configure the TagCloud format		
        * @private
        */
        _format: function (format) {
            if (format == "cloud") {
                this.ul.removeClass("e-list");
                this.ul.addClass("e-cloud");
            }
            else if (format == "list") {
                this.ul.removeClass("e-cloud");
                this.ul.addClass("e-list");
            }
        },
        /**
        * destroy the tagCloud widget
		* all events bound using this._on will be unbind automatically and bring the control to pre-init state.
		* @private
        */
        _destroy: function () {
            this.element.removeClass("e-widget " + this.model.cssClass );
            this.element.empty();
        },
        /**
        * To initialize the TagCloud		
        * @private
        */
        _initialize: function () {
            this.minFreq = 0;
            this.maxFreq = 30;
            this.ul = null;
            this.titleText = null;
            this.image = null;
            this.text = null;
        },
        /**
        * Render Section For Tagcloud		
        * @private
        */
        _render: function () {
            this.element.addClass("e-widget " + this.model.cssClass );
            if (this.model.showTitle) this._generateTitle();
            this._renderWrapper();
            this._checkDataBinding();
            this._checkProperties();
        },
        /**
        * To configure the tagCloud title Text
        * @private
        */
        _generateTitle: function () {
            this.titleText = ej.buildTag("div.e-header e-title");
            if (this.model.titleImage)
                this._generateImageTag(this.model.titleImage);
            if (this.model.titleText)
                this._generateTextTag(this.model.titleText);
            if (this.ul) this.titleText.insertBefore(this.ul);
            else this.element.append(this.titleText);
        },
        /**
         * To configure the tagCloud title image
		 * @private
         */
        _generateImageTag: function (titleImage) {	
			if(!this.image)
				this.image = $('<img class="e-title-img" src="' + titleImage + '" />');
			if(this.text && !this.model.titleImage) this.image.insertBefore(this.text);			
			else this.titleText.append(this.image);
        },		
        /**
        * Section for generate Title Text 
        * @private
        */
        _generateTextTag: function (titleText) { 
			if(!this.text) this.text = ej.buildTag("span", titleText);					
			this.titleText.append(this.text);
        },		
        /**
         * Render Section For TagCloud		
		 * @private
         */
        _renderWrapper: function () {
            var format;
            format = (this.model.format == "list") ? "list" : "cloud";
            this.ul = ej.buildTag("ul.e-ul e-box e-" + format);
            this.element.append(this.ul);
        },
        /**
        * Section for rendering tagCloud items
        * @private
        */
        _renderItems: function (list) {
            this._generateTagItems(list);
            this.ul.removeClass("e-load");
        },

        _checkProperties: function () {
            if (this.model.enableRTL) this._rtl(this.model.enableRTL);
        },
        /**
        * To enable or disable the Right to Left behaviour 		
        * @private
        */
        _rtl: function (boolean) {
            if (boolean) this.element.addClass("e-rtl");
            else this.element.removeClass("e-rtl");
        },
        /**
        * Section for check whether collection of items is local or remote data	
        * @private
        */
        _checkDataBinding: function () {
            var source = this.model.dataSource;
            if (source != null) {
                this.ul.addClass("e-load");
                if (ej.DataManager && source instanceof ej.DataManager)
                    this._initDataSource(source);
                else
                    this._renderItems(source);
            }
        },
        /**
       * Section for render items with Remote Data	
       * @private
       */
        _initDataSource: function (source) {
            var proxy = this;
            var queryPromise = source.executeQuery(this._getQuery());
            queryPromise.done(function (e) {
                proxy._renderItems(e.result);
            }).fail(function (e) {
                proxy.ul.removeClass("e-load");
            });
        },
        /**
        * Section for set attributes to tagCloud items
        * @private
        */
        _setAttributes: function (data, element) {
            if (data) {
                for (var key in data)
                    element.attr(key, data[key]);
            }
        },

        _getQuery: function () {
            if (ej.isNullOrUndefined(this.model.query)) {
                var column = [], queryManager = ej.Query(), mapper = this.model.fields;
                for (var col in mapper) {
                    if (col !== "tableName" && mapper[col])
                        column.push(mapper[col]);
                }
                if (column.length > 0)
                    queryManager.select(column);
                if (!this.model.dataSource.dataSource.url.match(mapper.tableName + "$"))
                    !ej.isNullOrUndefined(mapper.tableName) && queryManager.from(mapper.tableName);
            }
            else queryManager = this.model.query;
            return queryManager;
        },

        _generateTagItems: function (list) {
            var i, mapField = this._getMapper();
            this.minFreq = Math.min.apply(Math, list.map(function (o) { return o[mapField._freq]; }))
            this.maxFreq = Math.max.apply(Math, list.map(function (o) { return o[mapField._freq]; }))

            for (i = 0; i < list.length; i++) {
                this.ul.append(this._generateLi(list[i], mapField));
            }
        },
        /**
        * Section for mapping fields
        * @private
        */
        _getMapper: function () {
            var mapper = this.model.fields, mapFld = { _text: null, _freq: null, _url: null };
            mapFld._text = (mapper && mapper.text) ? mapper["text"] : "text";
            mapFld._freq = (mapper && mapper.frequency) ? mapper["frequency"] : "frequency";
            mapFld._url = (mapper && mapper.url) ? mapper["url"] : "url";
            mapFld._attr = (mapper && mapper.htmlAttr) ? mapper["htmlAttr"] : "htmlAttr";
            return mapFld;
        },
        /**
        * Section for render tagCloud items
        * @private
        */
        _generateLi: function (list, map) {
            var li = ej.buildTag("li.e-tagitems"), aTag;
            aTag = ej.buildTag("a.e-txt", list[map._text] || list[map._url],
                    { "font-size": this._calculateFontSize(list[map._freq]) },
                    { "href": list[map._url] || "#", "target": "blank", role: "link"  });
            li.append(aTag);
            this._setAttributes((list[map._attr]), li);
            this._on(aTag, "mouseenter", this._mouseEnter);
            this._on(aTag, "mouseleave", this._mouseLeave);
            this._on(aTag, "click", this._mouseClick);
            return li;
        },

        /**
        * Section For track mouseEnter event
        * @private
        */
        _mouseEnter: function (e) {
            $(e.target).addClass("hover");
            this._raiseEvent(e, "mouseover");
        },
        /**
        * Section For track mouseLeave event
        * @private
        */
        _mouseLeave: function (e) {
            $(e.target).removeClass("hover");
            this._raiseEvent(e, "mouseout");
        },
        /**
        * Section For handle the click event
        * @private
        */
        _mouseClick: function (e) {
            $(e.target).removeClass("hover");
            this._raiseEvent(e, "click");
        },

        _raiseEvent: function (e, _type) {
                this._trigger(_type, { value: $(e.target).html(), url: $(e.target).attr('href'), eventType: _type });
        },
        /**
        * Section for calculate Font Size
        * @private
        */
        _calculateFontSize: function (frequency) {
            if (frequency) {
                var C = 2, k, fontRange, fontSize,
                minSize = parseInt(this.model.minFontSize, 10), maxSize = parseInt(this.model.maxFontSize, 10);
                k = (frequency - this.minFreq) / (this.maxFreq - this.minFreq);
                fontRange = maxSize - minSize;
                fontSize = minSize + (C * Math.floor(k * (fontRange / C)));
                return fontSize;
            }
            else return this.model.minFontSize;
        }
    });
    /**
	 * Enum for various format	 
	 * @enum {string}
	 * @global 
	 */
    ej.Format = {
        /**  Render the tagCloud items in cloud format */
        Cloud: "cloud",
        /**  Render the tagCloud items in list format */
        List: "list"
    };
})(jQuery, Syncfusion);;
/**
* @fileOverview Plugin to style the Html TileView elements
* @copyright Copyright Syncfusion Inc. 2001 - 2013. All rights reserved.
*  Use of this code is subject to the terms of our license.
*  A copy of the current license can be obtained at any time by e-mailing
*  licensing@syncfusion.com. Any infringement will be prosecuted under
*  applicable laws. 
* @version 12.1 
* @author <a href="mailto:licensing@syncfusion.com">Syncfusion Inc</a>
*/


(function ($, ej, undefined) {
    ej.widget("ejTileBase", "ej.TileBase", {
        defaults: {
            /**		
            * Section for badge specific functionalities.                      
            * @memberof ejTile
            * @member badge {object}
            * @instance
            */

            /**		
            * Section for badge specific functionalities.                      
            * @memberof ejmTile
            * @member badge {object}
            * @instance
            */
            badge: {
                //Web
                /**	
                * Specifies whether to enable badge or not.
                * @default false
                * @member badge->enabled {object}
                * @type {boolean}
                * @alias ejTile#badge->enabled
                * @example 
                * //Set the enabled property in unobtrusive way.
                * &lt;div id="tile" data-role="ejtile" data-ej-imageurl="people.png" data-ej-badge-enabled="true" &gt;
                * &lt;/div&gt;  
                * @example 
                * // Set enabled on initialization. 
                * // To set enabled API value 
                * &lt;div id="tile" &gt;&lt;/div&gt;<br>
                * &lt;script&gt; 
                * // Create Tile control 
                * $("#tile").ejTile({ imageUrl: "people.png", badge: { enabled: true } }); 
                * &lt;/script&gt;
                * @example 
                * //Get or set the enabled, after initialization:<br>
                * // Get the enabled API value.
                *  $("#tile").ejTile("option", "badge.enabled");			
                * // Set the enabled API
                * $("#tile").ejTile("option", "badge.enabled", true);            
                * @memberof ejTile
                * @instance
                */

                //Mobile
                /**	
                * Specifies whether to enable badge or not.
                * @default false
                * @member badge->enabled {object}
                * @type {boolean}
                * @alias ejmTile#badge->enabled
                * @example 
                * //Set the enabled property in unobtrusive way.
                * &lt;div id="tile" data-role="ejmtile" data-ej-imageurl="people.png" data-ej-badge-enabled="true" &gt;
                * &lt;/div&gt;  
                * @example 
                * // Set enabled on initialization. 
                * // To set enabled API value 
                * &lt;div id="tile" &gt;&lt;/div&gt;<br>
                * &lt;script&gt; 
                * // Create Tile control 
                * $("#tile").ejmTile({ imageUrl: "people.png", badge: { enabled: true } }); 
                * &lt;/script&gt;
                * @example 
                * //Get or set the enabled, after initialization:<br>
                * // Get the enabled API value.
                *  $("#tile").ejmTile("option", "badge.enabled");			
                * // Set the enabled API
                * $("#tile").ejmTile("option", "badge.enabled", true);            
                * @memberof ejmTile
                * @instance
                */
                enabled: false,
                //Web
                /**	
                * Sets value for tile badge.
                * @default 1
                * @member badge->value {object}
                * @type {number}
                * @alias ejTile#badge->value
                * @example 
                * //Set the value property in unobtrusive way.
                * &lt;div id="tile" data-role="ejtile" data-ej-imageurl="people.png" data-ej-badge-enabled="true" data-ej-badge-value="5"&gt;
                * &lt;/div&gt;  
                * @example 
                * // Set value on initialization. 
                * // To set value API value 
                * &lt;div id="tile" &gt;&lt;/div&gt;<br>
                * &lt;script&gt; 
                * // Create Tile control 
                * $("#tile").ejTile({ imageUrl: "people.png", badge: { enabled: true, value:5 } }); 
                * &lt;/script&gt;
                * @example 
                * //Get or set the value, after initialization:<br>
                * // Get the value API value.
                *  $("#tile").ejTile("option", "badge.value");			
                * // Set the value API
                * $("#tile").ejTile("option", "badge.value", 5);            
                * @memberof ejTile
                * @instance
                */

                //Mobile
                /**	
                * Sets value for tile badge.
                * @default 1
                * @member badge->value {object}
                * @type {number}
                * @alias ejmTile#badge->value
                * @example 
                * //Set the value property in unobtrusive way.
                * &lt;div id="tile" data-role="ejmtile" data-ej-imageurl="people.png" data-ej-badge-enabled="true" data-ej-badge-value="5"&gt;
                * &lt;/div&gt;  
                * @example 
                * // Set value on initialization. 
                * // To set value API value 
                * &lt;div id="tile" &gt;&lt;/div&gt;<br>
                * &lt;script&gt; 
                * // Create Tile control 
                * $("#tile").ejmTile({ imageUrl: "people.png", badge: { enabled: true, value:5 } }); 
                * &lt;/script&gt;
                * @example 
                * //Get or set the value, after initialization:<br>
                * // Get the value API value.
                *  $("#tile").ejmTile("option", "badge.value");			
                * // Set the value API
                * $("#tile").ejmTile("option", "badge.value", 5);            
                * @memberof ejmTile
                * @instance
                */
                value: 1,
                //Web
                /**	
                * Specifies maximum value for tile badge.
                * @default 100
                * @member badge->maxValue {object}
                * @type {number}
                * @alias ejTile#badge->maxValue
                * @example 
                * //Set the maxValue property in unobtrusive way.
                * &lt;div id="tile" data-role="ejtile" data-ej-imageurl="people.png" data-ej-badge-enabled="true" data-ej-badge-value="5" data-ej-badge-maxvalue="3" &gt;
                * &lt;/div&gt;  
                * @example 
                * // Set maxValue on initialization. 
                * // To set maxValue API value 
                * &lt;div id="tile" &gt;&lt;/div&gt;<br>
                * &lt;script&gt; 
                * // Create Tile control 
                * $("#tile").ejTile({ imageUrl: "people.png", badge: { enabled: true, value:5, maxValue:3 } }); 
                * &lt;/script&gt;
                * @example 
                * //Get or set the maxValue, after initialization:<br>
                * // Get the maxValue API value.
                *  $("#tile").ejTile("option", "badge.maxValue");			
                * // Set the maxValue API
                * $("#tile").ejTile("option", "badge.maxValue", 3);            
                * @memberof ejTile
                * @instance
                */

                //Mobile
                /**	
                * Specifies maximum value for tile badge.
                * @default 100
                * @member badge->maxValue {object}
                * @type {number}
                * @alias ejmTile#badge->maxValue
                * @example 
                * //Set the maxValue property in unobtrusive way.
                * &lt;div id="tile" data-role="ejmtile" data-ej-imageurl="people.png" data-ej-badge-enabled="true" data-ej-badge-value="5" data-ej-badge-maxvalue="3" &gt;
                * &lt;/div&gt;  
                * @example 
                * // Set maxValue on initialization. 
                * // To set maxValue API value 
                * &lt;div id="tile" &gt;&lt;/div&gt;<br>
                * &lt;script&gt; 
                * // Create Tile control 
                * $("#tile").ejmTile({ imageUrl: "people.png", badge: { enabled: true, value:5, maxValue:3 } }); 
                * &lt;/script&gt;
                * @example 
                * //Get or set the maxValue, after initialization:<br>
                * // Get the maxValue API value.
                *  $("#tile").ejmTile("option", "badge.maxValue");			
                * // Set the maxValue API
                * $("#tile").ejmTile("option", "badge.maxValue", 3);            
                * @memberof ejmTile
                * @instance
                */
                maxValue: 100,
                //Web
                /**	
                * Specifies minimum value for tile badge.
                * @default 1
                * @member badge->minValue {object}
                * @type {number}
                * @alias ejTile#badge->minValue
                * @example 
                * //Set the minValue property in unobtrusive way.
                * &lt;div id="tile" data-role="ejtile" data-ej-imageurl="people.png" data-ej-badge-enabled="true" data-ej-badge-value="3" data-ej-badge-minvalue="5"&gt;
                * &lt;/div&gt;  
                * @example 
                * // Set minValue on initialization. 
                * // To set minValue API value 
                * &lt;div id="tile" &gt;&lt;/div&gt;<br>
                * &lt;script&gt; 
                * // Create Tile control 
                * $("#tile").ejTile({ imageUrl: "people.png", badge: { enabled: true, value:3, minValue:5 } }); 
                * &lt;/script&gt;
                * @example 
                * //Get or set the minValue, after initialization:<br>
                * // Get the minValue API value.
                *  $("#tile").ejTile("option", "badge.minValue");			
                * // Set the minValue API
                * $("#tile").ejTile("option", "badge.minValue", 5);            
                * @memberof ejTile
                * @instance
                */

                //Mobile
                /**	
                * Specifies minimum value for tile badge.
                * @default 1
                * @member badge->minValue {object}
                * @type {number}
                * @alias ejmTile#badge->minValue
                * @example 
                * //Set the minValue property in unobtrusive way.
                * &lt;div id="tile" data-role="ejmtile" data-ej-imageurl="people.png" data-ej-badge-enabled="true" data-ej-badge-value="3" data-ej-badge-minvalue="5"&gt;
                * &lt;/div&gt;  
                * @example 
                * // Set minValue on initialization. 
                * // To set minValue API value 
                * &lt;div id="tile" &gt;&lt;/div&gt;<br>
                * &lt;script&gt; 
                * // Create Tile control 
                * $("#tile").ejmTile({ imageUrl: "people.png", badge: { enabled: true, value:3, minValue:5 } }); 
                * &lt;/script&gt;
                * @example 
                * //Get or set the minValue, after initialization:<br>
                * // Get the minValue API value.
                *  $("#tile").ejmTile("option", "badge.minValue");			
                * // Set the minValue API
                * $("#tile").ejmTile("option", "badge.minValue", 5);            
                * @memberof ejmTile
                * @instance
                */
                minValue: 1,
                //Web
                /**	
                * Specifies text instead of number for tile badge.
                * @default null
                * @member badge->text {object}
                * @type {string}
                * @alias ejTile#badge->text
                * @example 
                * //Set the text property in unobtrusive way.
                * &lt;div id="tile" data-role="ejtile" data-ej-imageurl="people.png" data-ej-badge-enabled="true" data-ej-badge-text="ten"&gt;
                * &lt;/div&gt;  
                * @example 
                * // Set text on initialization. 
                * // To set text API value 
                * &lt;div id="tile" &gt;&lt;/div&gt;<br>
                * &lt;script&gt; 
                * // Create Tile control 
                * $("#tile").ejTile({ imageUrl: "people.png", badge: { enabled: true, text:"ten" } }); 
                * &lt;/script&gt;
                * @example 
                * //Get or set the badge text, after initialization:<br>
                * // Get the badge text API value.
                *  $("#tile").ejTile("option", "badge.text");			
                * // Set the badge text API
                * $("#tile").ejTile("option", "badge.text", "ten");            
                * @memberof ejTile
                * @instance
                */

                //Mobile
                /**	
                * Specifies text instead of number for tile badge.
                * @default null
                * @member badge->text {object}
                * @type {string}
                * @alias ejmTile#badge->text
                * @example 
                * //Set the text property in unobtrusive way.
                * &lt;div id="tile" data-role="ejmtile" data-ej-imageurl="people.png" data-ej-badge-enabled="true" data-ej-badge-text="ten"&gt;
                * &lt;/div&gt;  
                * @example 
                * // Set text on initialization. 
                * // To set text API value 
                * &lt;div id="tile" &gt;&lt;/div&gt;<br>
                * &lt;script&gt; 
                * // Create Tile control 
                * $("#tile").ejmTile({ imageUrl: "people.png", badge: { enabled: true, text:"ten" } }); 
                * &lt;/script&gt;
                * @example 
                * //Get or set the badge text, after initialization:<br>
                * // Get the badge text API value.
                *  $("#tile").ejmTile("option", "badge.text");			
                * // Set the badge text API
                * $("#tile").ejmTile("option", "badge.text", "ten");            
                * @memberof ejmTile
                * @instance
                */
                text: null
            },
            //Web
            /**		
            * Specifies the tile caption in outside template content.
            * @default null
            * @member captionTemplateId {object}
            * @type {string}
            * @example 
            * //Set the captionTemplateId property in unobtrusive way.
            * &lt;div id="tile" data-role="ejtile" data-ej-imageurl="people.png" data-ej-captiontemplateid="sample" &gt;
            * &lt;/div&gt; 
            * &lt;div id="sample" &gt; Settings
            * &lt;/div&gt;
            * @example 
            * // Set captionTemplateId on initialization. 
            * // To set captionTemplateId API value 
            * &lt;div id="tile" &gt;&lt;/div&gt;<br>
            * &lt;div id="sample" &gt; Settings
            * &lt;/div&gt;
            * &lt;script&gt; 
            * // Create Tile control 
            * $("#tile").ejTile({ imageUrl: "people.png", captionTemplateId: "sample" }); 
            * &lt;/script&gt;
            * @example 
            * //Get or set the captionTemplateId, after initialization:<br>
            * // Get the captionTemplateId API value.
            *  $("#tile").ejTile("option", "captionTemplateId");			
            * // Set the captionTemplateId API
            * $("#tile").ejTile("option", "captionTemplateId", "sample");            
            * @memberof ejTile
            * @instance
            */

            //Mobile
            /**		
            * Specifies the tile text in outside template content.
            * @default null
            * @member captionTemplateId {object}
            * @type {string}
            * @example 
            * //Set the captionTemplateId property in unobtrusive way.
            * &lt;div id="tile" data-role="ejmtile" data-ej-imageurl="people.png" data-ej-captiontemplateid="sample" &gt;
            * &lt;/div&gt; 
            * &lt;div id="sample" &gt; Settings
            * &lt;/div&gt;
            * @example 
            * // Set captionTemplateId on initialization. 
            * // To set captionTemplateId API value 
            * &lt;div id="tile" &gt;&lt;/div&gt;<br>
            * &lt;div id="sample" &gt; Settings
            * &lt;/div&gt;
            * &lt;script&gt; 
            * // Create Tile control 
            * $("#tile").ejmTile({ imageUrl: "people.png", captionTemplateId: "sample" }); 
            * &lt;/script&gt;
            * @example 
            * //Get or set the captionTemplateId, after initialization:<br>
            * // Get the captionTemplateId API value.
            *  $("#tile").ejmTile("option", "captionTemplateId");			
            * // Set the captionTemplateId API
            * $("#tile").ejmTile("option", "captionTemplateId", "sample");            
            * @memberof ejmTile
            * @instance
            */
            captionTemplateId: null,
            //Web
            /**		
            * Saves current model value to browser cookies for state maintains. While refreshing the page retains the model value applies from browser cookies.
            * @default false
            * @member enablePersistence {object}
            * @type {boolean}
            * @example 
            * //Set the enablePersistence property in unobtrusive way.
            * &lt;div id="tile" data-role="ejtile"  data-ej-imageurl="people.png" data-ej-enablepersistence=true&gt;
            * &lt;/div&gt;  
            * @example 
            * // Set enablePersistence on initialization. 
            * // To set enablePersistence API value 
            * &lt;div id="tile" &gt;&lt;/div&gt;<br>
            * &lt;script&gt; 
            * // Create Tile control 
            * $("#tile").ejTile({ imageUrl: "people.png", enablePersistence:true }); 
            * &lt;/script&gt;
            * @example 
            * //Get or set the enablePersistence, after initialization:<br>
            * // Get the enablePersistence API value.
            *  $("#tile").ejTile("option", "enablePersistence");			
            * // Set the enablePersistence API
            * $("#tile").ejTile("option", "enablePersistence", true);            
            * @memberof ejTile
            * @instance
            */

            //Mobile
            /**		
            * Saves current model value to browser cookies for state maintains. While refreshing the page retains the model value applies from browser cookies.
            * @default false
            * @member enablePersistence {object}
            * @type {boolean}
            * @example 
            * //Set the enablePersistence property in unobtrusive way.
            * &lt;div id="tile" data-role="ejmtile"  data-ej-imageurl="people.png" data-ej-enablepersistence=true&gt;
            * &lt;/div&gt;  
            * @example 
            * // Set enablePersistence on initialization. 
            * // To set enablePersistence API value 
            * &lt;div id="tile" &gt;&lt;/div&gt;<br>
            * &lt;script&gt; 
            * // Create Tile control 
            * $("#tile").ejmTile({ imageUrl: "people.png", enablePersistence:true }); 
            * &lt;/script&gt;
            * @example 
            * //Get or set the enablePersistence, after initialization:<br>
            * // Get the enablePersistence API value.
            *  $("#tile").ejmTile("option", "enablePersistence");			
            * // Set the enablePersistence API
            * $("#tile").ejmTile("option", "enablePersistence", true);            
            * @memberof ejmTile
            * @instance
            */
            enablePersistence: false,
            //Web
            /**		
            * Specifies Tile imageclass, using this property we can give images for each tile through css classes.
            * @default null
            * @member imageClass {object}
            * @type {string}
            * @example 
            * //Set the imageClass property in unobtrusive way.
            * &lt;div id="tile" data-role="ejtile" data-ej-imageclass="sample"&gt;
            * &lt;/div&gt;  
            * @example 
            * // Set imageClass on initialization. 
            * // To set imageClass API value 
            * &lt;div id="tile" &gt;&lt;/div&gt;<br>
            * &lt;script&gt; 
            * // Create Tile control 
            * $("#tile").ejTile({ imageClass: "sample" }); 
            * &lt;/script&gt;
            * @example 
            * //Get or set the imageClass, after initialization:<br>
            * // Get the imageClass API value.
            *  $("#tile").ejTile("option", "imageClass");			
            * // Set the imageClass API
            * $("#tile").ejTile("option", "imageClass", "sample");            
            * @memberof ejTile
            * @instance
            */

            //Mobile
            /**		
            * Specifies Tile imageclass, using this property we can give images for each tile through css classes.
            * @default null
            * @member imageClass {object}
            * @type {string}
            * @example 
            * //Set the imageClass property in unobtrusive way.
            * &lt;div id="tile" data-role="ejmtile" data-ej-imageclass="sample"&gt;
            * &lt;/div&gt;  
            * @example 
            * // Set imageClass on initialization. 
            * // To set imageClass API value 
            * &lt;div id="tile" &gt;&lt;/div&gt;<br>
            * &lt;script&gt; 
            * // Create Tile control 
            * $("#tile").ejmTile({ imageClass: "sample" }); 
            * &lt;/script&gt;
            * @example 
            * //Get or set the imageClass, after initialization:<br>
            * // Get the imageClass API value.
            *  $("#tile").ejmTile("option", "imageClass");			
            * // Set the imageClass API
            * $("#tile").ejmTile("option", "imageClass", "sample");            
            * @memberof ejmTile
            * @instance
            */
            imageClass: null,
            //Web
            /**		
            * Specifies the position of tile image.
            * @default "center"
            * @member imagePosition {object}
            * @type {enum}
            * @example 
            * //Set the imagePosition property in unobtrusive way.
            * &lt;div id="tile" data-role="ejtile" data-ej-imageurl="people.png" data-ej-imageposition="right"&gt;
            * &lt;/div&gt;  
            * @example 
            * // Set imagePosition on initialization. 
            * // To set imagePosition API value 
            * &lt;div id="tile" &gt;&lt;/div&gt;<br>
            * &lt;script&gt; 
            * // Create Tile control 
            * $("#tile").ejTile({ imageUrl: "people.png", imagePosition: "right" }); 
            * &lt;/script&gt;
            * @example 
            * //Get or set the imagePosition, after initialization:<br>
            * // Get the imagePosition API value.
            *  $("#tile").ejTile("option", "imagePosition");			
            * // Set the imagePosition API
            * $("#tile").ejTile("option", "imagePosition", "right");            
            * @memberof ejTile
            * @instance
            */

            //Mobile
            /**		
            * Specifies the position of tile image.
            * @default "center"
            * @member imagePosition {object}
            * @type {enum}
            * @example 
            * //Set the imagePosition property in unobtrusive way.
            * &lt;div id="tile" data-role="ejmtile" data-ej-imageurl="people.png" data-ej-imageposition="right"&gt;
            * &lt;/div&gt;  
            * @example 
            * // Set imagePosition on initialization. 
            * // To set imagePosition API value 
            * &lt;div id="tile" &gt;&lt;/div&gt;<br>
            * &lt;script&gt; 
            * // Create Tile control 
            * $("#tile").ejmTile({ imageUrl: "people.png", imagePosition: "right" }); 
            * &lt;/script&gt;
            * @example 
            * //Get or set the imagePosition, after initialization:<br>
            * // Get the imagePosition API value.
            *  $("#tile").ejmTile("option", "imagePosition");			
            * // Set the imagePosition API
            * $("#tile").ejmTile("option", "imagePosition", "right");            
            * @memberof ejmTile
            * @instance
            */
            imagePosition: "center",
            //Web
            /**		
            * Specifies the tile image in outside template content.
            * @default null
            * @member imageTemplateId {object}
            * @type {string}
            * @example 
            * //Set the imageTemplateId property in unobtrusive way.
            * &lt;div id="tile" data-role="ejtile" data-ej-imagetemplateid="sample" &gt;
            * &lt;/div&gt; 
            * &lt;div id="sample" style="background-image: url('people.png');height:100px;width:100px;"&gt;
            * &lt;/div&gt;
            * @example 
            * // Set imageTemplateId on initialization. 
            * // To set imageTemplateId API value 
            * &lt;div id="tile" &gt;&lt;/div&gt;<br>
            * &lt;div id="sample" style="background-image: url('people.png');height:100%;width:100%;"&gt;
            * &lt;/div&gt;
            * &lt;script&gt; 
            * // Create Tile control 
            * $("#tile").ejTile({ imageTemplateId: "sample" }); 
            * &lt;/script&gt;
            * @example 
            * //Get or set the imageTemplateId, after initialization:<br>
            * // Get the imageTemplateId API value.
            *  $("#tile").ejTile("option", "imageTemplateId");			
            * // Set the imageTemplateId API
            * $("#tile").ejTile("option", "imageTemplateId", "sample");            
            * @memberof ejTile
            * @instance
            */

            //Mobile
            /**		
            * Specifies the tile image in outside template content.
            * @default null
            * @member imageTemplateId {object}
            * @type {string}
            * @example 
            * //Set the imageTemplateId property in unobtrusive way.
            * &lt;div id="tile" data-role="ejmtile" data-ej-imagetemplateid="sample" &gt;
            * &lt;/div&gt; 
            * &lt;div id="sample" style="background-image: url('people.png');height:100px;width:100px;"&gt;
            * &lt;/div&gt;
            * @example 
            * // Set imageTemplateId on initialization. 
            * // To set imageTemplateId API value 
            * &lt;div id="tile" &gt;&lt;/div&gt;<br>
            * &lt;div id="sample" style="background-image: url('people.png');height:100%;width:100%;"&gt;
            * &lt;/div&gt;
            * &lt;script&gt; 
            * // Create Tile control 
            * $("#tile").ejmTile({ imageTemplateId: "sample" }); 
            * &lt;/script&gt;
            * @example 
            * //Get or set the imageTemplateId, after initialization:<br>
            * // Get the imageTemplateId API value.
            *  $("#tile").ejmTile("option", "imageTemplateId");			
            * // Set the imageTemplateId API
            * $("#tile").ejmTile("option", "imageTemplateId", "sample");            
            * @memberof ejmTile
            * @instance
            */
            imageTemplateId: null,
            //Web
            /**		
            * Specifies the file name of tile image.
            * @default null
            * @member imageUrl {object}
            * @type {string}
            * @example 
            * //Set the imageUrl property in unobtrusive way.
            * &lt;div id="tile" data-role="ejtile" data-ej-imageurl="people.png"&gt;
            * &lt;/div&gt;  
            * @example 
            * // Set imageUrl on initialization. 
            * // To set imageUrl API value 
            * &lt;div id="tile" &gt;&lt;/div&gt;<br>
            * &lt;script&gt; 
            * // Create Tile control 
            * $("#tile").ejTile({ imageUrl: "people.png" }); 
            * &lt;/script&gt;
            * @example 
            * //Get or set the imageUrl, after initialization:<br>
            * // Get the imageUrl API value.
            *  $("#tile").ejTile("option", "imageUrl");			
            * // Set the imageUrl API
            * $("#tile").ejTile("option", "imageUrl", "people.png");            
            * @memberof ejTile
            * @instance
            */

            //Mobile
            /**		
            * Specifies the file name of tile image.
            * @default null
            * @member imageUrl {object}
            * @type {string}
            * @example 
            * //Set the imageUrl property in unobtrusive way.
            * &lt;div id="tile" data-role="ejmtile" data-ej-imageurl="people.png"&gt;
            * &lt;/div&gt;  
            * @example 
            * // Set imageUrl on initialization. 
            * // To set imageUrl API value 
            * &lt;div id="tile" &gt;&lt;/div&gt;<br>
            * &lt;script&gt; 
            * // Create Tile control 
            * $("#tile").ejmTile({ imageUrl: "people.png" }); 
            * &lt;/script&gt;
            * @example 
            * //Get or set the imageUrl, after initialization:<br>
            * // Get the imageUrl API value.
            *  $("#tile").ejmTile("option", "imageUrl");			
            * // Set the imageUrl API
            * $("#tile").ejmTile("option", "imageUrl", "people.png");            
            * @memberof ejmTile
            * @instance
            */
            imageUrl: null,
            /**		
            * Section for livetile specific functionalities.                      
            * @memberof ejTile
            * @member livetile {object}
            * @instance
            */

            /**		
            * Section for livetile specific functionalities.                      
            * @memberof ejmTile
            * @member livetile {object}
            * @instance
            */
            liveTile: {
                //Web
                /**	
                * Specifies whether to enable livetile or not.
                * @default false
                * @member livetile->enabled {object}
                * @type {boolean}
                * @alias ejTile#livetile->enabled
                * @example 
                * //Set the liveTile enabled property in unobtrusive way.
                * &lt;div id="tile" data-role="ejtile" data-ej-rendermode="windows" data-ej-livetile-imageurl="['people.png','sports.png']" data-ej-livetile-enabled="true" &gt;
                * &lt;/div&gt;  
                * @example 
                * // Set liveTile enabled on initialization. 
                * // To set liveTile enabled API value 
                * &lt;div id="tile" &gt;&lt;/div&gt;<br>
                * &lt;script&gt; 
                * // Create Tile control 
                * $("#tile").ejTile({ renderMode:"windows", liveTile: { enabled: true, imageUrl:['people.png','sports.png'] } }); 
                * &lt;/script&gt;
                * @example 
                * //Get or set the liveTile enabled, after initialization:<br>
                * // Get the liveTile enabled API value.
                *  $("#tile").ejTile("option", "liveTile.enabled");			
                * // Set the liveTile enabled API
                * $("#tile").ejTile("option", "liveTile.enabled", true);            
                * @memberof ejTile
                * @instance
                */

                //Mobile
                /**	
                * Specifies whether to enable livetile or not.
                * @default false
                * @member livetile->enabled {object}
                * @type {boolean}
                * @alias ejmTile#livetile->enabled
                * @example 
                * //Set the liveTile enabled property in unobtrusive way.
                * &lt;div id="tile" data-role="ejmtile" data-ej-rendermode="windows" data-ej-livetile-imageurl="['people.png','sports.png']" data-ej-livetile-enabled="true" &gt;
                * &lt;/div&gt;  
                * @example 
                * // Set liveTile enabled on initialization. 
                * // To set liveTile enabled API value 
                * &lt;div id="tile" &gt;&lt;/div&gt;<br>
                * &lt;script&gt; 
                * // Create Tile control 
                * $("#tile").ejmTile({ renderMode:"windows", liveTile: { enabled: true, imageUrl:['people.png','sports.png'] } }); 
                * &lt;/script&gt;
                * @example 
                * //Get or set the liveTile enabled, after initialization:<br>
                * // Get the liveTile enabled API value.
                *  $("#tile").ejmTile("option", "liveTile.enabled");			
                * // Set the liveTile enabled API
                * $("#tile").ejmTile("option", "liveTile.enabled", true);            
                * @memberof ejmTile
                * @instance
                */
                enabled: false,
                //Web
                /**	
                * Specifies liveTile images in css classes.
                * @default null
                * @member livetile->imageClass {object}
                * @type {string}
                * @alias ejTile#liveTile->imageClass
                * @example 
                * //Set the liveTile imageClass property in unobtrusive way.
                * &lt;div id="tile" data-role="ejtile" data-ej-rendermode="windows" data-ej-livetile-imageclass="['img1','img2','img3']" data-ej-livetile-enabled="true" &gt;
                * &lt;/div&gt;  
                * @example 
                * // Set liveTile imageClass on initialization. 
                * // To set liveTile imageClass API value 
                * &lt;div id="tile" &gt;&lt;/div&gt;<br>
                * &lt;script&gt; 
                * // Create Tile control 
                * $("#tile").ejTile({ renderMode:"windows", liveTile: { enabled: true, imageClass: ['img1','img2','img3'] } }); 
                * &lt;/script&gt;
                * @example 
                * //Get or set the liveTile imageClass, after initialization:<br>
                * // Get the liveTile imageClass API value.
                *  $("#tile").ejTile("option", "liveTile.imageClass");			
                * // Set the liveTile imageClass API
                * $("#tile").ejTile("option", "liveTile.imageClass", ['img1','img2','img3']);            
                * @memberof ejTile
                * @instance
                */

                //Mobile
                /**	
                * Specifies liveTile images in css classes.
                * @default null
                * @member livetile->imageClass {object}
                * @type {string}
                * @alias ejmTile#liveTile->imageClass
                * @example 
                * //Set the liveTile imageClass property in unobtrusive way.
                * &lt;div id="tile" data-role="ejmtile" data-ej-rendermode="windows" data-ej-livetile-imageclass="['img1','img2','img3']" data-ej-livetile-enabled="true" &gt;
                * &lt;/div&gt;  
                * @example 
                * // Set liveTile imageClass on initialization. 
                * // To set liveTile imageClass API value 
                * &lt;div id="tile" &gt;&lt;/div&gt;<br>
                * &lt;script&gt; 
                * // Create Tile control 
                * $("#tile").ejmTile({ renderMode:"windows", liveTile: { enabled: true, imageClass: ['img1','img2','img3'] } }); 
                * &lt;/script&gt;
                * @example 
                * //Get or set the liveTile imageClass, after initialization:<br>
                * // Get the liveTile imageClass API value.
                *  $("#tile").ejmTile("option", "liveTile.imageClass");			
                * // Set the liveTile imageClass API
                * $("#tile").ejmTile("option", "liveTile.imageClass", ['img1','img2','img3']);            
                * @memberof ejmTile
                * @instance
                */
                imageClass: null,
                //Web
                /**	
                * Specifies liveTile images in templates.
                * @default null
                * @member livetile->imageTemplateId {object}
                * @type {string}
                * @alias ejTile#liveTile->imageTemplateId
                * @example 
                * //Set the liveTile imageTemplateId property in unobtrusive way.
                * &lt;div id="tile" data-role="ejtile" data-ej-rendermode="windows" data-ej-livetile-imagetemplateid="['img1','img2','img3']" data-ej-livetile-enabled="true" &gt;
                * &lt;/div&gt;
                * &lt;div id="img1" style="background-image: url('people.png');height:100%;width:100%;"&gt;
                * &lt;/div&gt;
                * &lt;div id="img2" style="background-image: url('sports.png');height:100%;width:100%;"&gt;
                * &lt;/div&gt;
                * &lt;div id="img3" style="background-image: url('settings.png');height:100%;width:100%;"&gt;
                * &lt;/div&gt;                
                * @example 
                * // Set liveTile imageTemplateId on initialization. 
                * // To set liveTile imageTemplateId API value 
                * &lt;div id="tile" &gt;&lt;/div&gt;<br>
                * &lt;div id="img1" style="background-image: url('people.png');height:100%;width:100%;"&gt;
                * &lt;/div&gt;
                * &lt;div id="img2" style="background-image: url('sports.png');height:100%;width:100%;"&gt;
                * &lt;/div&gt;
                * &lt;div id="img3" style="background-image: url('settings.png');height:100%;width:100%;"&gt;
                * &lt;/div&gt;
                * &lt;script&gt; 
                * // Create Tile control 
                * $("#tile").ejTile({ renderMode:"windows", liveTile: { enabled: true, imageTemplateId: ['img1','img2','img3'] } }); 
                * &lt;/script&gt;
                * @example 
                * //Get or set the liveTile imageTemplateId, after initialization:<br>
                * // Get the liveTile imageTemplateId API value.
                *  $("#tile").ejTile("option", "liveTile.imageTemplateId");			
                * // Set the liveTile imageTemplateId API
                * $("#tile").ejTile("option", "liveTile.imageTemplateId", ['img1','img2','img3']);            
                * @memberof ejTile
                * @instance
                */

                //Mobile
                /**	
                * Specifies liveTile images in templates.
                * @default null
                * @member livetile->imageTemplateId {object}
                * @type {string}
                * @alias ejmTile#liveTile->imageTemplateId
                * @example 
                * //Set the liveTile imageTemplateId property in unobtrusive way.
                * &lt;div id="tile" data-role="ejmtile" data-ej-rendermode="windows" data-ej-livetile-imagetemplateid="['img1','img2','img3']" data-ej-livetile-enabled="true" &gt;
                * &lt;/div&gt;
                * &lt;div id="img1" style="background-image: url('people.png');height:100%;width:100%;"&gt;
                * &lt;/div&gt;
                * &lt;div id="img2" style="background-image: url('sports.png');height:100%;width:100%;"&gt;
                * &lt;/div&gt;
                * &lt;div id="img3" style="background-image: url('settings.png');height:100%;width:100%;"&gt;
                * &lt;/div&gt;                
                * @example 
                * // Set liveTile imageTemplateId on initialization. 
                * // To set liveTile imageTemplateId API value 
                * &lt;div id="tile" &gt;&lt;/div&gt;<br>
                * &lt;div id="img1" style="background-image: url('people.png');height:100%;width:100%;"&gt;
                * &lt;/div&gt;
                * &lt;div id="img2" style="background-image: url('sports.png');height:100%;width:100%;"&gt;
                * &lt;/div&gt;
                * &lt;div id="img3" style="background-image: url('settings.png');height:100%;width:100%;"&gt;
                * &lt;/div&gt;
                * &lt;script&gt; 
                * // Create Tile control 
                * $("#tile").ejmTile({ renderMode:"windows", liveTile: { enabled: true, imageTemplateId: ['img1','img2','img3'] } }); 
                * &lt;/script&gt;
                * @example 
                * //Get or set the liveTile imageTemplateId, after initialization:<br>
                * // Get the liveTile imageTemplateId API value.
                *  $("#tile").ejmTile("option", "liveTile.imageTemplateId");			
                * // Set the liveTile imageTemplateId API
                * $("#tile").ejmTile("option", "liveTile.imageTemplateId", ['img1','img2','img3']);            
                * @memberof ejmTile
                * @instance
                */
                imageTemplateId: null,
                //Web
                /**	
                * Specifies liveTile images in css classes.
                * @default null
                * @member livetile->imageUrl {object}
                * @type {string}
                * @alias ejTile#liveTile->imageUrl
                * @example 
                * //Set the liveTile imageUrl property in unobtrusive way.
                * &lt;div id="tile" data-role="ejtile" data-ej-rendermode="windows" data-ej-livetile-enabled="true" data-ej-livetile-imageurl="['people.png','sports.png','settings.png']"  &gt;
                * &lt;/div&gt;  
                * @example 
                * // Set liveTile imageUrl on initialization. 
                * // To set liveTile imageUrl API value 
                * &lt;div id="tile" &gt;&lt;/div&gt;<br>
                * &lt;script&gt; 
                * // Create Tile control 
                * $("#tile").ejTile({ renderMode: "windows", livetile: { enabled: true, imageUrl: ['people.png','sports.png','settings.png'] } }); 
                * &lt;/script&gt;
                * @example 
                * //Get or set the liveTile imageUrl, after initialization:<br>
                * // Get the liveTile imageUrl API value.
                *  $("#tile").ejTile("option", "liveTile.imageUrl");			
                * // Set the liveTile imageUrl API
                * $("#tile").ejTile("option", "liveTile.imageUrl", ['people.png','sports.png','settings.png']);            
                * @memberof ejTile
                * @instance
                */

                //Mobile
                /**	
                * Specifies liveTile images in css classes.
                * @default null
                * @member livetile->imageUrl {object}
                * @type {string}
                * @alias ejmTile#liveTile->imageUrl
                * @example 
                * //Set the liveTile imageUrl property in unobtrusive way.
                * &lt;div id="tile" data-role="ejmtile" data-ej-rendermode="windows" data-ej-livetile-enabled="true" data-ej-livetile-imageurl="['people.png','sports.png','settings.png']"  &gt;
                * &lt;/div&gt;  
                * @example 
                * // Set liveTile imageUrl on initialization. 
                * // To set liveTile imageUrl API value 
                * &lt;div id="tile" &gt;&lt;/div&gt;<br>
                * &lt;script&gt; 
                * // Create Tile control 
                * $("#tile").ejmTile({ renderMode: "windows", livetile: { enabled: true, imageUrl: ['people.png','sports.png','settings.png'] } }); 
                * &lt;/script&gt;
                * @example 
                * //Get or set the liveTile imageUrl, after initialization:<br>
                * // Get the liveTile imageUrl API value.
                *  $("#tile").ejmTile("option", "liveTile.imageUrl");			
                * // Set the liveTile imageUrl API
                * $("#tile").ejmTile("option", "liveTile.imageUrl", ['people.png','sports.png','settings.png']);            
                * @memberof ejmTile
                * @instance
                */
                imageUrl: null,
                //Web
                /**	
                * Specifies liveTile type for Tile. i.e flip, slide or carousel
                * @default "flip"
                * @member livetile->type {object}
                * @type {enum}
                * @alias ejTile#liveTile->type
                * @example 
                * //Set the liveTile type property in unobtrusive way.
                * &lt;div id="tile" data-role="ejtile" data-ej-rendermode="windows" data-ej-livetile-enabled="true" data-ej-livetile-imageurl="['people.png','sports.png','settings.png']"  data-ej-livetile-type="carousel"&gt;
                * &lt;/div&gt;  
                * @example 
                * // Set liveTile type on initialization. 
                * // To set liveTile type API value 
                * &lt;div id="tile" &gt;&lt;/div&gt;<br>
                * &lt;script&gt; 
                * // Create Tile control 
                * $("#tile").ejTile({ renderMode: "windows", liveTile: { enabled: true, imageUrl: ['people.png','sports.png','settings.png'], type:"carousel" } }); 
                * &lt;/script&gt;
                * @example 
                * //Get or set the liveTile type, after initialization:<br>
                * // Get the liveTile type API value.
                *  $("#tile").ejTile("option", "liveTile.type");			
                * // Set the liveTile type API
                * $("#tile").ejTile("option", "liveTile.type", "carousel");            
                * @memberof ejTile
                * @instance
                */

                //Mobile
                /**	
                * Specifies liveTile type for Tile. i.e flip, slide or carousel
                * @default "flip"
                * @member livetile->type {object}
                * @type {enum}
                * @alias ejmTile#liveTile->type
                * @example 
                * //Set the liveTile type property in unobtrusive way.
                * &lt;div id="tile" data-role="ejmtile" data-ej-rendermode="windows" data-ej-livetile-enabled="true" data-ej-livetile-imageurl="['people.png','sports.png','settings.png']"  data-ej-livetile-type="carousel"&gt;
                * &lt;/div&gt;  
                * @example 
                * // Set liveTile type on initialization. 
                * // To set liveTile type API value 
                * &lt;div id="tile" &gt;&lt;/div&gt;<br>
                * &lt;script&gt; 
                * // Create Tile control 
                * $("#tile").ejmTile({ renderMode: "windows", liveTile: { enabled: true, imageUrl: ['people.png','sports.png','settings.png'], type:"carousel" } }); 
                * &lt;/script&gt;
                * @example 
                * //Get or set the liveTile type, after initialization:<br>
                * // Get the liveTile type API value.
                *  $("#tile").ejmTile("option", "liveTile.type");			
                * // Set the liveTile type API
                * $("#tile").ejmTile("option", "liveTile.type", "carousel");            
                * @memberof ejmTile
                * @instance
                */
                type: "flip",
                //Web
                /**	
                * Specifies time interval between two successive livetile animation
                * @default 2000
                * @member livetile->updateInterval {object}
                * @type {number}
                * @alias ejTile#liveTile->updateInterval
                * @example 
                * //Set the liveTile updateInterval property in unobtrusive way.
                * &lt;div id="tile" data-role="ejtile" data-ej-rendermode="windows" data-ej-livetile-enabled="true" data-ej-livetile-imageurl="['people.png','sports.png','settings.png']"  data-ej-livetile-updateinterval=1000&gt;
                * &lt;/div&gt;  
                * @example 
                * // Set liveTile updateInterval on initialization. 
                * // To set liveTile updateInterval API value 
                * &lt;div id="tile" &gt;&lt;/div&gt;<br>
                * &lt;script&gt; 
                * // Create Tile control 
                * $("#tile").ejTile({ renderMode: "windows", liveTile: { enabled: true, imageUrl: ['people.png','sports.png','settings.png'], updateInterval:1000 } }); 
                * &lt;/script&gt;
                * @example 
                * //Get or set the liveTile updateInterval, after initialization:<br>
                * // Get the liveTile updateInterval API value.
                *  $("#tile").ejTile("option", "liveTile.updateInterval");			
                * // Set the liveTile updateInterval API
                * $("#tile").ejTile("option", "liveTile.updateInterval", 1000);            
                * @memberof ejTile
                * @instance
                */

                //Mobile
                /**	
                * Specifies time interval between two successive livetile animation
                * @default 2000
                * @member livetile->updateInterval {object}
                * @type {number}
                * @alias ejmTile#liveTile->updateInterval
                * @example 
                * //Set the liveTile updateInterval property in unobtrusive way.
                * &lt;div id="tile" data-role="ejmtile" data-ej-rendermode="windows" data-ej-livetile-enabled="true" data-ej-livetile-imageurl="['people.png','sports.png','settings.png']"  data-ej-livetile-updateinterval=1000&gt;
                * &lt;/div&gt;  
                * @example 
                * // Set liveTile updateInterval on initialization. 
                * // To set liveTile updateInterval API value 
                * &lt;div id="tile" &gt;&lt;/div&gt;<br>
                * &lt;script&gt; 
                * // Create Tile control 
                * $("#tile").ejmTile({ renderMode: "windows", liveTile: { enabled: true, imageUrl: ['people.png','sports.png','settings.png'], updateInterval:1000 } }); 
                * &lt;/script&gt;
                * @example 
                * //Get or set the liveTile updateInterval, after initialization:<br>
                * // Get the liveTile updateInterval API value.
                *  $("#tile").ejmTile("option", "liveTile.updateInterval");			
                * // Set the liveTile updateInterval API
                * $("#tile").ejmTile("option", "liveTile.updateInterval", 1000);            
                * @memberof ejmTile
                * @instance
                */
                updateInterval: 2000
            },
            //Web
            /**		
            * Specifies whether the tile text to be shown or hidden.
            * @default true
            * @member showText {object}
            * @type {boolean}
            * @example 
            * //Set the showText property in unobtrusive way.
            * &lt;div id="tile" data-role="ejtile"  data-ej-imageurl="people.png" data-ej-showtext=false&gt;
            * &lt;/div&gt;  
            * @example 
            * // Set showText on initialization. 
            * // To set showText API value 
            * &lt;div id="tile" &gt;&lt;/div&gt;<br>
            * &lt;script&gt; 
            * // Create Tile control 
            * $("#tile").ejTile({ imageUrl: "people.png", showText:false }); 
            * &lt;/script&gt;
            * @example 
            * //Get or set the showText, after initialization:<br>
            * // Get the showText API value.
            *  $("#tile").ejTile("option", "showText");			
            * // Set the showText API
            * $("#tile").ejTile("option", "showText", false);            
            * @memberof ejTile
            * @instance
            */

            //Mobile
            /**		
            * Specifies whether the tile text to be shown or hidden.
            * @default true
            * @member showText {object}
            * @type {boolean}
            * @example 
            * //Set the showText property in unobtrusive way.
            * &lt;div id="tile" data-role="ejmtile"  data-ej-imageurl="people.png" data-ej-showtext=false&gt;
            * &lt;/div&gt;  
            * @example 
            * // Set showText on initialization. 
            * // To set showText API value 
            * &lt;div id="tile" &gt;&lt;/div&gt;<br>
            * &lt;script&gt; 
            * // Create Tile control 
            * $("#tile").ejmTile({ imageUrl: "people.png", showText:false }); 
            * &lt;/script&gt;
            * @example 
            * //Get or set the showText, after initialization:<br>
            * // Get the showText API value.
            *  $("#tile").ejmTile("option", "showText");			
            * // Set the showText API
            * $("#tile").ejmTile("option", "showText", false);            
            * @memberof ejmTile
            * @instance
            */
            showText: true,
            //Web
            /**		
            * Changes the text of a tile.
            * @default "Text"
            * @member text {object}
            * @type {string}
            * @example 
            * //Set the text property in unobtrusive way.
            * &lt;div id="tile" data-role="ejtile" data-ej-imageurl="people.png" data-ej-text="Settings"&gt;
            * &lt;/div&gt;  
            * @example 
            * // Set text on initialization. 
            * // To set text API value 
            * &lt;div id="tile" &gt;&lt;/div&gt;<br>
            * &lt;script&gt; 
            * // Create Tile control 
            * $("#tile").ejTile({ imageUrl: "people.png", text:"Settings" }); 
            * &lt;/script&gt;
            * @example 
            * //Get or set the text, after initialization:<br>
            * // Get the text API value.
            *  $("#tile").ejTile("option", "text");			
            * // Set the text API
            * $("#tile").ejTile("option", "text", "Settings");            
            * @memberof ejTile
            * @instance
            */

            //Mobile
            /**		
            * Changes the text of a tile.
            * @default "Text"
            * @member text {object}
            * @type {string}
            * @example 
            * //Set the text property in unobtrusive way.
            * &lt;div id="tile" data-role="ejmtile" data-ej-imageurl="people.png" data-ej-text="Settings"&gt;
            * &lt;/div&gt;  
            * @example 
            * // Set text on initialization. 
            * // To set text API value 
            * &lt;div id="tile" &gt;&lt;/div&gt;<br>
            * &lt;script&gt; 
            * // Create Tile control 
            * $("#tile").ejmTile({ imageUrl: "people.png", text:"Settings" }); 
            * &lt;/script&gt;
            * @example 
            * //Get or set the text, after initialization:<br>
            * // Get the text API value.
            *  $("#tile").ejmTile("option", "text");			
            * // Set the text API
            * $("#tile").ejmTile("option", "text", "Settings");            
            * @memberof ejmTile
            * @instance
            */
            text: "Text",
            //Web
            /**		
            * Aligns the text of a tile. i.e left, right or center.
            * @default "normal"
            * @member textAlignment {object}
            * @type {enum}
            * @example 
            * //Set the textAlignment property in unobtrusive way.
            * &lt;div id="tile" data-role="ejtile" data-ej-imageurl="people.png" data-ej-textalignment="left"&gt;
            * &lt;/div&gt;  
            * @example 
            * // Set textAlignment on initialization. 
            * // To set textAlignment API value 
            * &lt;div id="tile" &gt;&lt;/div&gt;<br>
            * &lt;script&gt; 
            * // Create Tile control 
            * $("#tile").ejTile({ imageUrl: "people.png", textAlignment:"left" }); 
            * &lt;/script&gt;
            * @example 
            * //Get or set the textAlignment, after initialization:<br>
            * // Get the textAlignment API value.
            *  $("#tile").ejTile("option", "textAlignment");			
            * // Set the textAlignment API
            * $("#tile").ejTile("option", "textAlignment", "left");            
            * @memberof ejTile
            * @instance
            */

            //Mobile
            /**		
            * Aligns the text of a tile. i.e left, right or center.
            * @default "normal"
            * @member textAlignment {object}
            * @type {enum}
            * @example 
            * //Set the textAlignment property in unobtrusive way.
            * &lt;div id="tile" data-role="ejmtile" data-ej-imageurl="people.png" data-ej-textalignment="left"&gt;
            * &lt;/div&gt;  
            * @example 
            * // Set textAlignment on initialization. 
            * // To set textAlignment API value 
            * &lt;div id="tile" &gt;&lt;/div&gt;<br>
            * &lt;script&gt; 
            * // Create Tile control 
            * $("#tile").ejmTile({ imageUrl: "people.png", textAlignment:"left" }); 
            * &lt;/script&gt;
            * @example 
            * //Get or set the textAlignment, after initialization:<br>
            * // Get the textAlignment API value.
            *  $("#tile").ejmTile("option", "textAlignment");			
            * // Set the textAlignment API
            * $("#tile").ejmTile("option", "textAlignment", "left");            
            * @memberof ejmTile
            * @instance
            */
            textAlignment: "normal",
            //Web
            /**		
            * Specifies the size of a tile. i.e small, medium, large or wide.
            * @default "small"
            * @member tileSize {object}
            * @type {enum}
            * @example 
            * //Set the tileSize property in unobtrusive way.
            * &lt;div id="tile" data-role="ejtile" data-ej-imageurl="people.png" data-ej-tilesize="medium"&gt;
            * &lt;/div&gt;  
            * @example 
            * // Set tileSize on initialization. 
            * // To set tileSize API value 
            * &lt;div id="tile" &gt;&lt;/div&gt;<br>
            * &lt;script&gt; 
            * // Create Tile control 
            * $("#tile").ejTile({ imageUrl: "people.png", tileSize:"medium" }); 
            * &lt;/script&gt;
            * @example 
            * //Get or set the tileSize, after initialization:<br>
            * // Get the tileSize API value.
            *  $("#tile").ejTile("option", "tileSize");			
            * // Set the tileSize API
            * $("#tile").ejTile("option", "tileSize", "medium");            
            * @memberof ejTile
            * @instance
            */

            //Mobile
            /**		
            * Specifies the size of a tile. i.e small, medium, large or wide.
            * @default "small"
            * @member tileSize {object}
            * @type {enum}
            * @example 
            * //Set the tileSize property in unobtrusive way.
            * &lt;div id="tile" data-role="ejmtile" data-ej-imageurl="people.png" data-ej-tilesize="medium"&gt;
            * &lt;/div&gt;  
            * @example 
            * // Set tileSize on initialization. 
            * // To set tileSize API value 
            * &lt;div id="tile" &gt;&lt;/div&gt;<br>
            * &lt;script&gt; 
            * // Create Tile control 
            * $("#tile").ejmTile({ imageUrl: "people.png", tileSize:"medium" }); 
            * &lt;/script&gt;
            * @example 
            * //Get or set the tileSize, after initialization:<br>
            * // Get the tileSize API value.
            *  $("#tile").ejmTile("option", "tileSize");			
            * // Set the tileSize API
            * $("#tile").ejmTile("option", "tileSize", "medium");            
            * @memberof ejmTile
            * @instance
            */
            tileSize: "small"
        },
        dataTypes: {
            badge: { enabled: "boolean", value: "number", maxValue: "number", minValue: "number" },
            imagePosition: "enum",
            liveTile: {
                type: "enum",
                enabled: "boolean",
                updateInterval: "number",
                imageUrl: "array",
                imageClass: "array",
                imageTemplateId: "array"
            },
            showText: "boolean",
            textAlignment: "enum",
            tileSize: "enum"
        },
        //Common method to render tile control
        _tileRender: function () {
            this.element.addClass(this._prefix + "tile " + this._prefix + "tile-wrapper " + this._prefix + "tile-wrapper-" + this.model.tileSize);
            this._mainTile = ej.buildTag("div." + this._prefix + "child-tile " + this._prefix + "tile-image" + this.model.imagePosition);
            this.model.liveTile.enabled && this.model.liveTile.type=="flip"?this.element.addClass(this._prefix+'tile-'+this.model.liveTile.type):this.element.addClass(this._prefix+'tile-common');
            this.element.append(this._mainTile);
            this._image = ej.buildTag("div." + this._prefix + "fontimage " + this._prefix + "tile-image " + this._prefix + "tile-image" + this.model.imagePosition);
            this._layer = ej.buildTag("div." + this._prefix + "layer");
            if (this.model.showText) {
                this.element.addClass(this._prefix + "enabled-text");
                this._text = ej.buildTag("div").addClass(this._prefix + "tile-text " + this._prefix + "user-select " + this._prefix + "textalign-" + this.model.textAlignment);         
            }
            else
                this.element.removeClass(this._prefix + "enabled-text").addClass(this._prefix + "disabled-text");
            //Live Tile implementation
            if ((!this._isAppView() || this.model.renderMode == "windows") && this.model.liveTile.enabled) {
                this.element.addClass(this._prefix + "live-type-" + this.model.liveTile.type);
                this._mainTile.addClass(this._prefix + "live-tile-enable");
                if (!this._liveImageWrapper) {
                    this._liveImageWrapper = ej.buildTag("div." + this._prefix + "live-image-wrapper");
                    if (this.model.liveTile.imageTemplateId) {
                        this._liveImageWrapper.addClass(this._prefix + "fontimage " + this._prefix + "tile-image " + this._prefix + "tile-image" + this.model.imagePosition);
                        for (var i = 0; i < this.model.liveTile.imageTemplateId.length; i++)
                            this._liveImageWrapper.append(ej.getCurrentPage().find('#' + this.model.liveTile.imageTemplateId[i]).addClass(this._prefix + "tile-template " + this._prefix + "live-tile " + this._prefix + "fontimage " + this._prefix + "tile-image " + this._prefix + "tile-image" + this.model.imagePosition + " " + this._prefix + "tile-" + this.model.liveTile.type));
                        this._mainTile.addClass(this._prefix + "tile-template");
                    }
                    else {
                        if (this.model.liveTile.imageClass) {
                            for (var i = 0; i < this.model.liveTile.imageClass.length; i++)
                                this._liveImageWrapper.append(ej.buildTag("div").addClass(this.model.liveTile.imageClass[i]));
                        }
                        else {
                            for (var i = 0; i < this.model.liveTile.imageUrl.length; i++) {
                                var url = this._isAppView() && this.model.imagePath ? this._getAbsolutePath(this.model.imagePath) + "/windows/" + this.model.liveTile.imageUrl[i] : this._getAbsolutePath(this.model.liveTile.imageUrl[i]);
                                this._liveImageWrapper.append(ej.buildTag("div." + this._prefix + "live-tile " + this._prefix + "fontimage " + this._prefix + "tile-image " + this._prefix + "tile-image" + this.model.imagePosition + " " + this._prefix + "tile-" + this.model.liveTile.type).css({ "background-image": "url('" + url + "')" }));
                            }
                        }
                    }
                    $(this._liveImageWrapper.children()[0]).addClass(this._prefix + "tile-" + this.model.liveTile.type + 'back').removeClass(this._prefix + "tile-" + this.model.liveTile.type);          
                }
                this._mainTile.append(this._liveImageWrapper);
            }
                //Normal Tile implementation
            else {
                if (this.model.imageTemplateId)
                    this._image.append(ej.getCurrentPage().find('#' + this.model.imageTemplateId).addClass(this._prefix + "tile-template"));
                else {
                    this._image = ej.buildTag("div").addClass(this._prefix + "fontimage " + this._prefix + "tile-image " + this._prefix + "tile-image" + this.model.imagePosition);
                    if (this.model.imageClass)
                        this._image.addClass(this.model.imageClass);
                    else if (this.model.imageUrl) {
                        var url = this._isAppView() && this.model.imagePath ? (this._getAbsolutePath(this.model.imagePath) + "/" + this.model.renderMode.toLowerCase() + "/" + this.model.imageUrl) : this._getAbsolutePath(this.model.imageUrl);
                        this._image.css({ "background-image": "url('" + url + "')" });
                    }
                }
                this._mainTile.append(this._image).addClass(this._prefix + "tile-image" + this.model.imagePosition);
            }
            //Badge wrapper creation
            if (this.model.badge.enabled)
                this._setBadgeValue();
            //Text Wrapper creation
            if (this._text) {
                if (this.model.captionTemplateId) {
                    this._text.append(ej.getCurrentPage().find('#' + this.model.captionTemplateId)).addClass(this._prefix + "template-caption");
                    this._mainTile.addClass(this._prefix + "template-caption")
                }
                else
                    this._text.html(this.model.text);
                this._mainTile.append(this._text);
            }
        },
        //Delegates for Tile control
        _createDelegates: function () {
            this._onStartDelegate = $.proxy(this._isAppView() ? this._onTouchStartHandler : this._onMouseDownHandler, this);
            this._onEndDelegate = $.proxy(this._isAppView() ? this._onTouchEndHandler : this._onMouseUpHandler, this);
            this._onMoveDelegate = $.proxy(this._isAppView() ? this._onTouchMoveHandler : this._onMouseMoveHandler, this);
            this._onDocClickDelegate = $.proxy(this._onDocClickHandler, this);
            this._startAnimationDelegate = $.proxy(this["_" + this.model.liveTile.type + "Tile"], this);
        },
        //Wiring the events to Tile control	
        _wireEvents: function (remove) {
            this._createDelegates();
            ej.listenEvents([this.element, this.element, this._isAppView() ? App.document : $(document)], [ej.startEvent(), ej.endEvent(), ej.endEvent()], [this._onStartDelegate, this._onEndDelegate, this._onDocClickDelegate], remove);
            if (this.model.liveTile.enabled && (!this._isAppView() || this.model.renderMode == "windows"))
                remove ? this._stopTileAnimation() : this._startTileAnimation();
        },
        //Section for handling the onDoc click
        _onDocClickHandler: function (evt) {
            this._removeActiveClass();
        },
        //Section for removing acive class for tile
        _removeActiveClass: function () {
            (!this._isAppView() || this.model.renderMode == "windows") ? ej._removeSkewClass(this.element) : $(this.element).removeClass(this._prefix + "state-active");
        },
        //Section for start tile animation
        _startTileAnimation: function () {
            this._intervalCounter = setInterval(this._startAnimationDelegate, this.model.liveTile.updateInterval);
        },
        //Section for stop tile animation
        _stopTileAnimation: function () {
            clearInterval(this._intervalCounter);
        },
        //Section for adding flip effect to windows live tile control
        _flipTile: function () {
            var front = this.element.find("." + this._prefix + "tile-flipback");
            var back = this.element.find("." + this._prefix + "tile-flip").first();
            front.addClass(this._prefix + "tile-flip").removeClass(this._prefix + "tile-flipback");
            back.addClass(this._prefix + "tile-flipback").removeClass(this._prefix + "tile-flip");
        },
        //Section for adding slide effect to windows live tile control
        _slideTile: function () {
            this._animateEffect();
        },
        //Section for adding carousel effect to windows live tile control
        _carouselTile: function () {
            this._animateEffect();
        },
        //Common method for slide and carousel effect
        _animateEffect: function () {
            var type = this.model.liveTile.type.toLowerCase();
            var items = this.element.find("." + this._prefix + "live-tile");
            var currentitem = this.element.find("." + this._prefix + "tile-" + type + "back");
            var nextitem = currentitem.next()[0] == null ? items.first() : currentitem.next();
            var previtem = currentitem.prev()[0] == null ? items.last() : currentitem.prev();
            previtem.addClass(this._prefix + "tile-" + type).removeClass(this._prefix + "tile-" + type + "up");
            currentitem.addClass(this._prefix + "tile-" + type + "up").removeClass(this._prefix + "tile-" + type + "back");
            nextitem.removeClass(this._prefix + "tile-" + type).addClass(this._prefix + "tile-" + type + "back");
        },
        //To get the absolute path for image url
        _getAbsolutePath: function (path) {
            return this._isAppView() ? App.route.makeUrlAbsolute(path, true).toString() : path;
        },
        _isAppView: function () {
            return this._prefix == "e-m-" ? true : false;
        },
        //Public method for template update
        //Web
        /**		
        * Update the image template to another one.
        * @method updateTemplate
        * @return jQuery
        * @example 
        * &lt;div id="tile" data-role="ejtile" data-ej-imagetemplateid="sample1" &gt;
        * &lt;/div&gt; 
        * &lt;div id="sample1" style="background-image: url('people.png');height:100px;width:100px;"&gt;
        * &lt;/div&gt;
        * &lt;div id="sample2" style="background-image: url('sports.png');height:100px;width:100px;"&gt;
        * &lt;/div&gt;
        * &lt;script&gt; 
        * $(document).ready(function(){
        * $("#tile").ejTile("updateTemplate","sample2"); 
        * });
        * &lt;/script&gt;
        * @memberof ejTile
        * @instance
        */

        //Mobile
        /**		
        * Update the image template to another one.
        * @method updateTemplate
        * @return jQuery
        * @example 
        * &lt;div id="tile" data-role="ejmtile" data-ej-imagetemplateid="sample1" &gt;
        * &lt;/div&gt; 
        * &lt;div id="sample1" style="background-image: url('people.png');height:100px;width:100px;"&gt;
        * &lt;/div&gt;
        * &lt;div id="sample2" style="background-image: url('sports.png');height:100px;width:100px;"&gt;
        * &lt;/div&gt;
        * &lt;script&gt; 
        * $(document).ready(function(){
        * $("#tile").ejTile("updateTemplate","sample2"); 
        * });
        * &lt;/script&gt;
        * @memberof ejmTile
        * @instance
        */
        updateTemplate: function (id, index) {
            if (this.model.liveTile.enabled) {
                $(this._liveImageWrapper).children()[index].remove();
                var tempLive = ej.getCurrentPage().find('#' + id).addClass(this._prefix + "tile-template " + this._prefix + "live-tile " + this._prefix + "fontimage " + this._prefix + "tile-image " + this._prefix + "tile-image" + this.model.imagePosition + " " + this._prefix + "tile-" + this.model.liveTile.type);
                if ($(this._liveImageWrapper).children()[index - 1])
                    $($(this._liveImageWrapper).children()[index - 1]).after(tempLive);
                else
                    $($(this._liveImageWrapper).children()[index + 1]).before(tempLive);
            }
            else
                this._image.empty().append(ej.getCurrentPage().find('#' + id).addClass(this._prefix + "tile-template"));
        },
        //SetModel
        _setModel: function (options) {
            var refresh = false;
            for (var property in options) {
                var setModel = "_set" + property.charAt(0).toUpperCase() + property.slice(1);
                if (this[setModel])
                    this[setModel](options[property]);
                else
                    refresh = true
            }
            if (refresh)
                this._refresh();
        },
        //To set image position
        _setImagePosition: function (value) {
            if (this.model.liveTile.enabled && $($(this._mainTile.children()[0]).children()).hasClass(this._prefix + "tile-image")) {
                $($(this._mainTile.children()[0]).children()).removeClass(this._prefix + "tile-imagecenter " + this._prefix + "tile-imagetop " + this._prefix + "tile-imagebottom " + this._prefix + "tile-imageright " + this._prefix + "tile-imageleft " + this._prefix + "tile-imagetopleft " + this._prefix + "tile-imagetopright " + this._prefix + "tile-imagebottomright " + this._prefix + "tile-imagebottomleft " + this._prefix + "tile-imagefill").addClass(this._prefix + "tile-image" + value);
                this._mainTile.removeClass(this._prefix + "tile-imagecenter " + this._prefix + "tile-imagetop " + this._prefix + "tile-imagebottom " + this._prefix + "tile-imageright " + this._prefix + "tile-imageleft " + this._prefix + "tile-imagetopleft " + this._prefix + "tile-imagetopright " + this._prefix + "tile-imagebottomright " + this._prefix + "tile-imagebottomleft " + this._prefix + "tile-imagefill").addClass(this._prefix + "tile-image" + value);
                this.model.imagePosition = value;
            }
            else {
                $(this._mainTile.children()[0]).removeClass(this._prefix + "tile-imagecenter " + this._prefix + "tile-imagetop " + this._prefix + "tile-imagebottom " + this._prefix + "tile-imageright " + this._prefix + "tile-imageleft " + this._prefix + "tile-imagetopleft " + this._prefix + "tile-imagetopright " + this._prefix + "tile-imagebottomright " + this._prefix + "tile-imagebottomleft " + this._prefix + "tile-imagefill").addClass(this._prefix + "tile-image" + value);
                this._mainTile.removeClass(this._prefix + "tile-imagecenter " + this._prefix + "tile-imagetop " + this._prefix + "tile-imagebottom " + this._prefix + "tile-imageright " + this._prefix + "tile-imageleft " + this._prefix + "tile-imagetopleft " + this._prefix + "tile-imagetopright " + this._prefix + "tile-imagebottomright " + this._prefix + "tile-imagebottomleft " + this._prefix + "tile-imagefill").addClass(this._prefix + "tile-image" + value);
                this.model.imagePosition = value;
            }
        },
        //To show text
        _setShowText: function (showtext) {
            this.model.showText = showtext;
            if (showtext) {
                this.element.removeClass(this._prefix + "disabled-text").addClass(this._prefix + "enabled-text");
                this._setText(this.model.text);
            }
            else
                this.element.removeClass(this._prefix + "enabled-text").addClass(this._prefix + "disabled-text");
        },
        //To set text
        _setText: function (text) {
            if (this.model.showText) {
                if (this.element.find('.' + this._prefix + "tile-text").length)
                    this.element.find('.' + this._prefix + "tile-text").html(text);
                else
                    this.element.prepend(ej.buildTag("div." + this._prefix + "tile-text", text));
            }
        },
        //To set text position
        _setTextPosition: function (position) {
            if (position == "outer")
                this.element.removeClass(this._prefix + "text-inner").addClass(this._prefix + "text-outer")
            else
                this.element.removeClass(this._prefix + "text-outer").addClass(this._prefix + "text-inner").find("." + this._prefix + "child-tile");
        },
        //To set text alignment
        _setTextAlignment: function (value) {
            $(this.element.find("." + this._prefix + "tile-text")).removeClass(this._prefix + "textalign-left " + this._prefix + "textalign-right " + this._prefix + "textalign-normal " + this._prefix + "textalign-center").addClass(this._prefix + "textalign-" + value);
            this.model.textAlignment = value;
        },
        //To modify badge
        _setBadge: function (options) {
            if (this.model.badge.enabled) {
                this._setBadgeEnabled(true);
                for (var property in options) {
                    var setModel = "_setBadge" + property.charAt(0).toUpperCase() + property.slice(1);
                    if (this[setModel])
                        this[setModel](options[property]);
                }
            }
            else
                this._setBadgeEnabled(false);
        },
        //To set badge value
        _setBadgeValue: function () {
            var badgeValue = this.model.badge.text ? this.model.badge.text : ((this.model.badge.value <= this.model.badge.maxValue && this.model.badge.value >= this.model.badge.minValue) ? this.model.badge.value.toString() : (this.model.badge.value > this.model.badge.maxValue ? this.model.badge.maxValue.toString() + "+" : this.model.badge.minValue.toString()));
            if (this.element.find('.' + this._prefix + "tile-badge").length)
                this.element.find('.' + this._prefix + "tile-badge").html(badgeValue);
            else
                this.element.prepend(ej.buildTag("div." + this._prefix + "tile-badge", badgeValue));
        },
        //To enable or disable badge
        _setBadgeEnabled: function (enabled) {
            this.element.find("." + this._prefix + "tile-badge")[enabled ? "show" : "hide"]();
        },
        //To set tile size
        _setTileSize: function (tileSize) {
            this._refresh();
        },
        //To clear element
        _clearElement: function () {
            this.element.removeAttr("class");
            this.element.html(this._orgEle.html());
        },
        //To destroy element
        _destroy: function () {
            this._wireEvents(true);
            this._clearElement();
        }
    });
})(jQuery, Syncfusion);;
/**
* @fileOverview Plugin to style the Html TileView elements
* @copyright Copyright Syncfusion Inc. 2001 - 2013. All rights reserved.
*  Use of this code is subject to the terms of our license.
*  A copy of the current license can be obtained at any time by e-mailing
*  licensing@syncfusion.com. Any infringement will be prosecuted under
*  applicable laws. 
* @version 12.1 
* @author <a href="mailto:licensing@syncfusion.com">Syncfusion Inc</a>
*/

(function ($, ej, undefined) {
    /**
    * @namespace ej
    * @class ejTile
    * @requires jQuery
    * @requires ej.core
    * @requires ej.unobtrusive
    * @requires ej.data
    * @requires ej.touch
    * @requires ej.tilebase
    * @classdesc Custom Design for Html Tile control.
    * @example 
    * &lt;div id="tile" data-role="ejtile" data-ej-imageurl="people.png"&gt;&lt;/div&gt;<br>
    * @example 
    * &lt;div id="tile" &gt;&lt;/div&gt;<br>
    * &lt;script&gt; 
    * // Create Tile control 
    * $("#tile").ejTile({ imageUrl: "people.png" }); 
    * &lt;/script&gt;
    */

    // ejTile is the plugin name 
    // "ej.Tile" is "namespace.className" will hold functions and properties

    ej.widget("ejTile", "ej.Tile", {
        _setFirst: true,
        validTags: ["div"],
        _rootCSS: "e-tile",
        defaults: {
            /**    
            * Event triggers when the mousedown happens in the tile
            * @event
            * @name ejTile#mouseDown 	
            * @param {Object} argument Event parameters from tile     
            * @param {boolean} argument.cancel if the event should be canceled; otherwise, false.
            * @param {boolean} argument.model returns the tile model
            * @param {boolean} argument.type returns the name of the event
            * @param {boolean} argument.text returns the current tile text
            * @example 
            * //Define mouseDown event in unobtrusive way.
            * &lt;div id="tile" data-role="ejtile" data-ej-imageurl='people.png' data-ej-mousedown="mousedown" &gt;
            * &lt;/div&gt;  
            * &lt;script&gt;
            * // mouseDown event for tile
            * function mousedown(args){ 
            * //handle the event
            * }
            * &lt;/script&gt;
            * @example 
            * // Define mouseDown event on initialization. 
            * // To set mouseDown event API value 
            * &lt;div id="tile" &gt;&lt;/div&gt;<br>
            * &lt;script&gt; 
            * // Create Tile control 
            * $("#tile").ejTile({ imageUrl: "people.png", 
            * mouseDown: function (args) { 
            * //handle the event 
            * }
            * }); 
            * &lt;/script&gt;
            * @memberof ejTile
            * @instance
            */
            mouseDown: null,
            /**    
            * Event triggers when the mouseup happens in the tile
            * @event
            * @name ejTile#mouseUp 	
            * @param {Object} argument Event parameters from tile     
            * @param {boolean} argument.cancel if the event should be canceled; otherwise, false.
            * @param {boolean} argument.model returns the tile model
            * @param {boolean} argument.type returns the name of the event
            * @param {boolean} argument.text returns the current tile text
            * @example 
            * //Define mouseUp event in unobtrusive way.
            * &lt;div id="tile" data-role="ejtile" data-ej-imageurl='people.png' data-ej-mouseup="mouseup" &gt;
            * &lt;/div&gt;  
            * &lt;script&gt;
            * // mouseUp event for tile
            * function mouseup(args){ 
            * //handle the event
            * }
            * &lt;/script&gt;
            * @example 
            * // Define mouseUp event on initialization. 
            * // To set mouseUp event API value 
            * &lt;div id="tile" &gt;&lt;/div&gt;<br>
            * &lt;script&gt; 
            * // Create Tile control 
            * $("#tile").ejTile({ imageUrl: "people.png", 
            * mouseUp: function (args) { 
            * //handle the event 
            * }
            * }); 
            * &lt;/script&gt;
            * @memberof ejTile
            * @instance
            */
            mouseUp: null
        },
        //Constructor function
        _init: function () {
            this._orgEle = this.element.clone();
            this._renderEJControl();
        },
        //To render the Tile control
        _renderEJControl: function () {
            this.element.addClass("e-tile-web");
            this._prefix = "e-";
            this._tileRender();
            this._mainTile.append(this._layer);
            this._wireEvents();
        },
        //Section for handling the mousedown event
        _onMouseDownHandler: function (evt) {
            var item = this.element;
            item.addClass(ej._getSkewClass(item, evt.pageX, evt.pageY).replace('e-m-', 'e-'));
            if (this.model.mouseDown)
                this._trigger("mouseDown", { text: item.find(".e-tile-text").html() });
            ej.listenTouchEvent(this.element, ej.moveEvent(), this._onMoveDelegate, false);
            this._isMoved = false;
        },
        //Section for handling the mousemove event
        _onMouseMoveHandler: function (evt) {
            this._isMoved = true;
            this._removeActiveClass();
        },
        //Section for handling the mouseup event
        _onMouseUpHandler: function (evt) {
            if (!this._isMoved && this.model.mouseUp)
                this._trigger("mouseUp", { text: this.element.find(".e-tile-text").html() });
            this._removeActiveClass();
            this._isMoved = false;
            ej.listenTouchEvent(this.element, ej.moveEvent(), this._onMoveDelegate, true);
        },
        //To refresh the control
        _refresh: function () {
            this._destroy();
            this.element.addClass("e-tile");
            this._renderEJControl();
        }
    });
    /**
    * Enum for Tile ImagePosition
    * @enum {string}
    * @global
    */
    ej.Tile.ImagePosition = {
        /**
        Enum for center position of tile image*/
        Center: "center",
        /**
        Enum for top position of tile image*/
        Top: "top",
        /**
        Enum for bottom position of tile image*/
        Bottom: "bottom",
        /**
        Enum for right position of tile image*/
        Right: "right",
        /**
        Enum for left position of tile image*/
        Left: "left",
        /**
        Enum for topleft position of tile image*/
        TopLeft: "topleft",
        /**
        Enum for topright position of tile image*/
        TopRight: "topright",
        /**
        Enum for bottomright position of tile image*/
        BottomRight: "bottomright",
        /**
        Enum for bottomleft position of tile image*/
        BottomLeft: "bottomleft",
        /**
        Enum for fill position of tile image*/
        Fill: "fill"
    },
    /**
    * Enum for Tile TextAlignment
    * @enum {string}
    * @global
    */
    ej.Tile.TextAlignment = {
        /**
        Enum for normal alignment of tile text*/
        Normal: "normal",
        /**
        Enum for left alignment of tile text*/
        Left: "left",
        /**
        Enum for right alignment of tile text*/
        Right: "right",
        /**
        Enum for center alignment of tile text*/
        Center: "center"
    },
    ej.Tile.TextPosition = {
        /**
        Enum for inner position of tile text*/
        Inner: "inner",
        /**
        Enum for outer position of tile text*/
        Outer: "outer"
    },
    /**
    * Enum for Tile Size
    * @enum {string}
    * @global
    */
    ej.Tile.TileSize = {
        /**
        Enum for medium size of tile*/
        Medium: "medium",
        /**
        Enum for small size of tile*/
        Small: "small",
        /**
        Enum for large size of tile*/
        Large: "large",
        /**
        Enum for wide size of tile*/
        Wide: "wide"
    },
    /**
    * Enum for LiveTileType
    * @enum {string}
    * @global
    */
    ej.Tile.LiveTileType = {
        /**
        Enum for flip type of livetile*/
        Flip: "flip",
        /**
        Enum for slide type of livetile*/
        Slide: "slide",
        /**
        Enum for carousel type of livetile*/
        Carousel: "carousel"
    }
    $.extend(true, ej.Tile.prototype, ej.TileBase.prototype);
})(jQuery, Syncfusion);;
/**
* @fileOverview Plugin to style the Html div elements
* @copyright Copyright Syncfusion Inc. 2001 - 2013. All rights reserved.
*  Use of this code is subject to the terms of our license.
*  A copy of the current license can be obtained at any time by e-mailing
*  licensing@syncfusion.com. Any infringement will be prosecuted under
*  applicable laws. 
* @version 12.1 
* @author <a href="mailto:licensing@syncfusion.com">Syncfusion Inc</a>
*/
(function ($, ej, undefined) {
    /**
    * @namespace ej
    * @class ejTreeView
    * @param {object} options - settings for TreeView.
    * @requires jQuery
    * @requires jquery.easing.1.3.js
    * @requires ej.core.js
    * @requires ej.treeview.js
    * @requires ej.data.js
    * @requires ej.checkbox.js
    * @requires ej.waitingpopup.js
    * @requires ej.draggable.js
    * @classdesc Custom Design for Html div control.
    * @example 
    * &lt;ul id="treeView"&gt;
    *        &lt;li&gt;Artwork
    *            &lt;ul&gt;
    *                &lt;li&gt;Abstract
    *                    &lt;ul&gt;
    *                        &lt;li&gt;2 Acrylic Mediums&lt;/li&gt;
    *                        &lt;li&gt;Creative Acrylic&lt;/li&gt;
    *                        &lt;li&gt;Modern Painting&lt;/li&gt;
    *                        &lt;li&gt;Canvas Art&lt;/li&gt;
    *                        &lt;li&gt;Black white&lt;/li&gt;
    *                    &lt;/ul&gt;
    *                &lt;/li&gt;
    *            &lt;/ul&gt;
    *        &lt;/li&gt;
    *        &lt;li&gt;Books
    *            &lt;ul&gt;
    *                &lt;li&gt;Entertaining&lt;/li&gt;
    *                &lt;li&gt;Design&lt;/li&gt;
    *            &lt;/ul&gt;
    *        &lt;/li&gt;
    *        &lt;li&gt;Music
    *            &lt;ul&gt;
    *                &lt;li&gt;Mass&lt;/li&gt;
    *                &lt;li&gt;Folk&lt;/li&gt;
    *            &lt;/ul&gt;
    *        &lt;/li&gt;
    *    &lt;/ul&gt;
    * <br> 
    * &lt;script&gt;
    * // Create TreeView
    * $('#treeView').ejTreeView(); 	
    * &lt;/script&gt;
    */
    ej.widget("ejTreeView", "ej.TreeView", {
        _rootCSS: "e-treeview",
        // widget element will be automatically set in this
        element: null,

        // user defined model will be automatically set in this
        model: null,
        validTags: ["ul", "div"],
        _setFirst: false,

        // default model
        defaults: {
             /**		
			* This API to enable the checkbox for all nodes. 
			* @default false
			* @type {boolean}
			* @example 
            * &lt;ul id="treeView"&gt;
    *        &lt;li&gt;Artwork
    *            &lt;ul&gt;
    *                &lt;li&gt;Abstract
    *                    &lt;ul&gt;
    *                        &lt;li&gt;2 Acrylic Mediums&lt;/li&gt;
    *                        &lt;li&gt;Creative Acrylic&lt;/li&gt;
    *                        &lt;li&gt;Modern Painting&lt;/li&gt;
    *                        &lt;li&gt;Canvas Art&lt;/li&gt;
    *                        &lt;li&gt;Black white&lt;/li&gt;
    *                    &lt;/ul&gt;
    *                &lt;/li&gt;
    *            &lt;/ul&gt;
    *        &lt;/li&gt;
    *        &lt;li&gt;Books
    *            &lt;ul&gt;
    *                &lt;li&gt;Entertaining&lt;/li&gt;
    *                &lt;li&gt;Design&lt;/li&gt;
    *            &lt;/ul&gt;
    *        &lt;/li&gt;
    *        &lt;li&gt;Music
    *            &lt;ul&gt;
    *                &lt;li&gt;Mass&lt;/li&gt;
    *                &lt;li&gt;Folk&lt;/li&gt;
    *            &lt;/ul&gt;
    *        &lt;/li&gt;
    *    &lt;/ul&gt;
    * <br> 
    * &lt;script&gt;        
					
		* //To Initialize the TreeView with the showCheckbox value specified.
            * $("#treeView").ejTreeView({ showCheckbox: true });
          *  &lt;/script&gt;
			* @memberof ejTreeView
			* @instance
			*/
            showCheckbox: false,
			enableAnimation: true,
            /**		
			* To enable the drag and drop nodes. 
			* @default false
			* @type {boolean}
			* @example 
            * &lt;ul id="treeView"&gt;
    *        &lt;li&gt;Artwork
    *            &lt;ul&gt;
    *                &lt;li&gt;Abstract
    *                    &lt;ul&gt;
    *                        &lt;li&gt;2 Acrylic Mediums&lt;/li&gt;
    *                        &lt;li&gt;Creative Acrylic&lt;/li&gt;
    *                        &lt;li&gt;Modern Painting&lt;/li&gt;
    *                        &lt;li&gt;Canvas Art&lt;/li&gt;
    *                        &lt;li&gt;Black white&lt;/li&gt;
    *                    &lt;/ul&gt;
    *                &lt;/li&gt;
    *            &lt;/ul&gt;
    *        &lt;/li&gt;
    *        &lt;li&gt;Books
    *            &lt;ul&gt;
    *                &lt;li&gt;Entertaining&lt;/li&gt;
    *                &lt;li&gt;Design&lt;/li&gt;
    *            &lt;/ul&gt;
    *        &lt;/li&gt;
    *        &lt;li&gt;Music
    *            &lt;ul&gt;
    *                &lt;li&gt;Mass&lt;/li&gt;
    *                &lt;li&gt;Folk&lt;/li&gt;
    *            &lt;/ul&gt;
    *        &lt;/li&gt;
    *    &lt;/ul&gt;
    * <br> 
    * &lt;script&gt;
				
			* //To Initialize the TreeView with the allowDragAndDrop value specified.
            * $("#treeView").ejTreeView({ allowDragAndDrop: true });
            *  &lt;/script&gt;
			* @memberof ejTreeView
			* @instance
			*/
            allowDragAndDrop: false,
            /**		
			* Allow to drop the child node for dragging node. 
			* @default false
			* @type {boolean}
			* @example 
            * &lt;ul id="treeView"&gt;
    *        &lt;li&gt;Artwork
    *            &lt;ul&gt;
    *                &lt;li&gt;Abstract
    *                    &lt;ul&gt;
    *                        &lt;li&gt;2 Acrylic Mediums&lt;/li&gt;
    *                        &lt;li&gt;Creative Acrylic&lt;/li&gt;
    *                        &lt;li&gt;Modern Painting&lt;/li&gt;
    *                        &lt;li&gt;Canvas Art&lt;/li&gt;
    *                        &lt;li&gt;Black white&lt;/li&gt;
    *                    &lt;/ul&gt;
    *                &lt;/li&gt;
    *            &lt;/ul&gt;
    *        &lt;/li&gt;
    *        &lt;li&gt;Books
    *            &lt;ul&gt;
    *                &lt;li&gt;Entertaining&lt;/li&gt;
    *                &lt;li&gt;Design&lt;/li&gt;
    *            &lt;/ul&gt;
    *        &lt;/li&gt;
    *        &lt;li&gt;Music
    *            &lt;ul&gt;
    *                &lt;li&gt;Mass&lt;/li&gt;
    *                &lt;li&gt;Folk&lt;/li&gt;
    *            &lt;/ul&gt;
    *        &lt;/li&gt;
    *    &lt;/ul&gt;
    * <br> 
    * &lt;script&gt;
			
			* // Initialize the TreeView with the allowDropChild value specified.
            * $("#treeView").ejTreeView({ allowDropChild: true });
            *  &lt;/script&gt;
			* @memberof ejTreeview
			* @instance
			*/
            allowDropChild: true,
                /**		
			* Allow to drop the root node for dragging node. 
			* @default false
			* @type {boolean}
			* @example 
            * &lt;ul id="treeView"&gt;
    *        &lt;li&gt;Artwork
    *            &lt;ul&gt;
    *                &lt;li&gt;Abstract
    *                    &lt;ul&gt;
    *                        &lt;li&gt;2 Acrylic Mediums&lt;/li&gt;
    *                        &lt;li&gt;Creative Acrylic&lt;/li&gt;
    *                        &lt;li&gt;Modern Painting&lt;/li&gt;
    *                        &lt;li&gt;Canvas Art&lt;/li&gt;
    *                        &lt;li&gt;Black white&lt;/li&gt;
    *                    &lt;/ul&gt;
    *                &lt;/li&gt;
    *            &lt;/ul&gt;
    *        &lt;/li&gt;
    *        &lt;li&gt;Books
    *            &lt;ul&gt;
    *                &lt;li&gt;Entertaining&lt;/li&gt;
    *                &lt;li&gt;Design&lt;/li&gt;
    *            &lt;/ul&gt;
    *        &lt;/li&gt;
    *        &lt;li&gt;Music
    *            &lt;ul&gt;
    *                &lt;li&gt;Mass&lt;/li&gt;
    *                &lt;li&gt;Folk&lt;/li&gt;
    *            &lt;/ul&gt;
    *        &lt;/li&gt;
    *    &lt;/ul&gt;
    * <br> 
    * &lt;script&gt;
			* // Initialize the TreeView with the dropSibling value specified.
            * $("#treeView").ejTreeView({ allowDropSibling: true });
            *  &lt;/script&gt;
			* @memberof ejTreeView
			* @instance
			*/
            allowDropSibling: true,
 /**		
			* Allow to drag the node out of the controls, like one TreeView to another.
			* @default false
			* @type {boolean}
			* @example 
            * &lt;ul id="treeView"&gt;
    *        &lt;li&gt;Artwork
    *            &lt;ul&gt;
    *                &lt;li&gt;Abstract
    *                    &lt;ul&gt;
    *                        &lt;li&gt;2 Acrylic Mediums&lt;/li&gt;
    *                        &lt;li&gt;Creative Acrylic&lt;/li&gt;
    *                        &lt;li&gt;Modern Painting&lt;/li&gt;
    *                        &lt;li&gt;Canvas Art&lt;/li&gt;
    *                        &lt;li&gt;Black white&lt;/li&gt;
    *                    &lt;/ul&gt;
    *                &lt;/li&gt;
    *            &lt;/ul&gt;
    *        &lt;/li&gt;
    *        &lt;li&gt;Books
    *            &lt;ul&gt;
    *                &lt;li&gt;Entertaining&lt;/li&gt;
    *                &lt;li&gt;Design&lt;/li&gt;
    *            &lt;/ul&gt;
    *        &lt;/li&gt;
    *        &lt;li&gt;Music
    *            &lt;ul&gt;
    *                &lt;li&gt;Mass&lt;/li&gt;
    *                &lt;li&gt;Folk&lt;/li&gt;
    *            &lt;/ul&gt;
    *        &lt;/li&gt;
    *    &lt;/ul&gt;
    * <br> 
    * &lt;script&gt;
					
			* // Initialize the TreeView with the dragAndDropAcrossControl value specified.
            * $("#treeView").ejTreeView({ dragAndDropAcrossControl: true });
            *  &lt;/script&gt;
			* @memberof ejTreeView
			* @instance
			*/
            allowDragAndDropAcrossControl: true,
             /**		
			* Allow to user can edit the node text.
			* @default false
			* @type {boolean}
			* @example 
            * &lt;ul id="treeView"&gt;
    *        &lt;li&gt;Artwork
    *            &lt;ul&gt;
    *                &lt;li&gt;Abstract
    *                    &lt;ul&gt;
    *                        &lt;li&gt;2 Acrylic Mediums&lt;/li&gt;
    *                        &lt;li&gt;Creative Acrylic&lt;/li&gt;
    *                        &lt;li&gt;Modern Painting&lt;/li&gt;
    *                        &lt;li&gt;Canvas Art&lt;/li&gt;
    *                        &lt;li&gt;Black white&lt;/li&gt;
    *                    &lt;/ul&gt;
    *                &lt;/li&gt;
    *            &lt;/ul&gt;
    *        &lt;/li&gt;
    *        &lt;li&gt;Books
    *            &lt;ul&gt;
    *                &lt;li&gt;Entertaining&lt;/li&gt;
    *                &lt;li&gt;Design&lt;/li&gt;
    *            &lt;/ul&gt;
    *        &lt;/li&gt;
    *        &lt;li&gt;Music
    *            &lt;ul&gt;
    *                &lt;li&gt;Mass&lt;/li&gt;
    *                &lt;li&gt;Folk&lt;/li&gt;
    *            &lt;/ul&gt;
    *        &lt;/li&gt;
    *    &lt;/ul&gt;
    * <br> 
    * &lt;script&gt;
					
			* // Initialize the TreeView with the allowEditing value specified.
            * $("#treeView").ejTreeView({ allowEditing: true });
            *  &lt;/script&gt;
			* @memberof ejTreeView
			* @instance
			*/
            allowEditing: false,
            /**		
			* Enable the keyboard interaction with TreeView nodes.
			* @default false
			* @type {boolean}
			* @example 
            * &lt;ul id="treeView"&gt;
    *        &lt;li&gt;Artwork
    *            &lt;ul&gt;
    *                &lt;li&gt;Abstract
    *                    &lt;ul&gt;
    *                        &lt;li&gt;2 Acrylic Mediums&lt;/li&gt;
    *                        &lt;li&gt;Creative Acrylic&lt;/li&gt;
    *                        &lt;li&gt;Modern Painting&lt;/li&gt;
    *                        &lt;li&gt;Canvas Art&lt;/li&gt;
    *                        &lt;li&gt;Black white&lt;/li&gt;
    *                    &lt;/ul&gt;
    *                &lt;/li&gt;
    *            &lt;/ul&gt;
    *        &lt;/li&gt;
    *        &lt;li&gt;Books
    *            &lt;ul&gt;
    *                &lt;li&gt;Entertaining&lt;/li&gt;
    *                &lt;li&gt;Design&lt;/li&gt;
    *            &lt;/ul&gt;
    *        &lt;/li&gt;
    *        &lt;li&gt;Music
    *            &lt;ul&gt;
    *                &lt;li&gt;Mass&lt;/li&gt;
    *                &lt;li&gt;Folk&lt;/li&gt;
    *            &lt;/ul&gt;
    *        &lt;/li&gt;
    *    &lt;/ul&gt;
    * <br> 
    * &lt;script&gt;
					
			* // Initialize the TreeView with the allowKeyboardNavigation value specified.
            * $("#treeView").ejTreeView({ allowKeyboardNavigation: true });
            *  &lt;/script&gt;
			* @memberof ejTreeView
			* @instance
			*/
            allowKeyboardNavigation: true,
                /**		
             * Items API have id, text, spriteCssClass,expanded, childItems ,selected ,linkAttribute ,value, imageAttribute, htmlAttribute,imageUrl. 
             * @default null
             * @type {object}
             * @example 
             * &lt;ul id="treeView"&gt;
    *        &lt;li&gt;Artwork
    *            &lt;ul&gt;
    *                &lt;li&gt;Abstract
    *                    &lt;ul&gt;
    *                        &lt;li&gt;2 Acrylic Mediums&lt;/li&gt;
    *                        &lt;li&gt;Creative Acrylic&lt;/li&gt;
    *                        &lt;li&gt;Modern Painting&lt;/li&gt;
    *                        &lt;li&gt;Canvas Art&lt;/li&gt;
    *                        &lt;li&gt;Black white&lt;/li&gt;
    *                    &lt;/ul&gt;
    *                &lt;/li&gt;
    *            &lt;/ul&gt;
    *        &lt;/li&gt;
    *        &lt;li&gt;Books
    *            &lt;ul&gt;
    *                &lt;li&gt;Entertaining&lt;/li&gt;
    *                &lt;li&gt;Design&lt;/li&gt;
    *            &lt;/ul&gt;
    *        &lt;/li&gt;
    *        &lt;li&gt;Music
    *            &lt;ul&gt;
    *                &lt;li&gt;Mass&lt;/li&gt;
    *                &lt;li&gt;Folk&lt;/li&gt;
    *            &lt;/ul&gt;
    *        &lt;/li&gt;
    *    &lt;/ul&gt;
    * <br> 
    * &lt;script&gt;
            	
              * //To set items API value during initialization  
             *  $("#treeView").ejTreeView({items: { id: "Home",text: "Home"}});
             *  &lt;/script&gt;
             * @memberof ejTreeView
             * @instance
             */
            items: null, //id, text, spriteCssClass,expanded, childItems ,selected ,linkAttribute ,value, imageAttribute, htmlAttribute,imageUrl
            /**		
             * Fields API have dataSource, query, tableName and mapper fields. In dataSource, we can pass the JSON data object or Essential Datamanager objects.  query; it have ej.Query() object. tableName; It have data get form given table name. mapper fields;  columns of a table can be mapped to the TreeView properties, namely id, parentId, text, spriteCssClass,expanded, childItems, hasChild, selected, linkAttribute , imageAttribute, htmlAttribute, imageUrl, isChecked.
             * @default null
             * @type {object}
             * @example 
             *&lt;div id="treeView"&gt;&lt;/div&gt;
             *&lt;script&gt;

            * //To set fields API value during initialization  
             *   $("#treeView").ejTreeView(
            *{
             *  fields: { id: "id", parentId: "pid", text: "name", hasChild: "hasChild", dataSource: window.treeView, expanded: "expanded" }
            *}
            *);
             *  &lt;/script&gt;
             * @memberof ejTreeView
             * @instance
             */
            fields: /** @lends ejTreeView# */{
				/**		
                 * datasource receives  Essential DataManager object and JSON object. 
				 * @alias ejTreeView#fields->dataSource
				 * @type Object
                 */
                dataSource: null,
				/**		
                 * It receives query to retrieve data from the table (query is same as SQL). 
				 * @alias ejTreeView#fields->query
				 * @type Object
				 * @example:  ej.Query().from("Categories").select("CategoryID,CategoryName").take(3);
                 */
                query: null,
				/**		
                 * It receives table name to execute query on the corresponding table.
				 * @alias ejTreeView#fields->tableName
				 * @type String
                 */
                tableName: null,
				/**		
                 * If a parent have child node to pass the same mapper for child.
				 * @alias ejTreeView#fields->child
				 * @type String
                 */
                child: null,
				/**		
                 * Specifies the id to TreeView node items list.
				 * @alias ejTreeView#fields->id
				 * @type String
                 */
                id: "id",
				/**		
                 * Specifies the parent id of the table.
				 * @alias ejTreeView#fields->parentId
				 * @type String
                 */
                parentId: "parentId",
				/**		
                 * Specifies the text of TreeView node items list.
				 * @alias ejTreeView#fields->text
				 * @type String
                 */
                text: "text",
				/**		
                 * Specifies the sprite CSS class to “li” item list.
				 * @alias ejTreeView#fields->spriteCssClass
				 * @type String
                 */
                spriteCssClass: "spriteCssClass",
				/**		
                 *  if it’s true the corresponding node will expand
				 * @alias ejTreeView#fields->expanded
				 * @type Boolean
                 */
                expanded: "expanded",
				/**		
                 * A parent node have child, we must set this value as true.
				 * @alias ejTreeView#fields->hasChild
				 * @type Boolean
                 */
                hasChild: "hasChild",
				/**		
                 * a TreeView control has only one node selected at time.Used to select the node at initial
				 * @alias ejTreeView#fields->selected
				 * @type Boolean
                 */
                selected: "selected",
				/**		
                 * Specifies the link attribute to “a” tag in item list.
				 * @alias ejTreeView#fields->linkAttribute
				 * @type String
                 */
                linkAttribute: "linkAttribute",
				/**		
                 * Specifies the image attribute to “img” tag inside items list 
				 * @alias ejTreeView#fields->imageAttribute
				 * @type String
                 */
                imageAttribute: "imageAttribute",
				/**		
                 * Specifies the html attributes to “li” item list.
				 * @alias ejTreeView#fields->htmlAttribute
				 * @type Object
                 */
                htmlAttribute: "htmlAttribute",
				/**		
                 * Specifies the html attributes to “li” item list.
				 * @alias ejTreeView#fields->imageUrl
				 * @type String
                 */
                imageUrl: "imageUrl",
				/**		
                 * if its true Checkbox node will be checked when rendered with checkbox.
				 * @alias ejTreeView#fields->isChecked
				 * @type Boolean
                 */
                isChecked: "isChecked"
            }, //null, //id, parentId, text, spriteCssClass,expanded, childItems, hasChild, selected, linkAttribute, imageAttribute, htmlAttribute, imageUrl, isChecked, tableName
            /**		
			* Allow to check the parent node automatically if this property is enabled.
			* @default false
			* @type {boolean}
			* @example 
            * &lt;ul id="treeView"&gt;
    *        &lt;li&gt;Artwork
    *            &lt;ul&gt;
    *                &lt;li&gt;Abstract
    *                    &lt;ul&gt;
    *                        &lt;li&gt;2 Acrylic Mediums&lt;/li&gt;
    *                        &lt;li&gt;Creative Acrylic&lt;/li&gt;
    *                        &lt;li&gt;Modern Painting&lt;/li&gt;
    *                        &lt;li&gt;Canvas Art&lt;/li&gt;
    *                        &lt;li&gt;Black white&lt;/li&gt;
    *                    &lt;/ul&gt;
    *                &lt;/li&gt;
    *            &lt;/ul&gt;
    *        &lt;/li&gt;
    *        &lt;li&gt;Books
    *            &lt;ul&gt;
    *                &lt;li&gt;Entertaining&lt;/li&gt;
    *                &lt;li&gt;Design&lt;/li&gt;
    *            &lt;/ul&gt;
    *        &lt;/li&gt;
    *        &lt;li&gt;Music
    *            &lt;ul&gt;
    *                &lt;li&gt;Mass&lt;/li&gt;
    *                &lt;li&gt;Folk&lt;/li&gt;
    *            &lt;/ul&gt;
    *        &lt;/li&gt;
    *    &lt;/ul&gt;
    * <br> 
    * &lt;script&gt;
					
			* // Initialize the TreeView with the autoCheckParentNode value specified.
            * $("#treeView").ejTreeView({ autoCheckParentNode: false, showCheckbox:true });
            *  &lt;/script&gt;
			* @memberof ejTreeView
			* @instance
			*/
            autoCheckParentNode: false,
            /**		
			* Child node loaded on demand, while expand the parent node at the time the entire child loaded.
			* @default false
			* @type {boolean}
			* @example 
            * &lt;ul id="treeView"&gt;
    *        &lt;li&gt;Artwork
    *            &lt;ul&gt;
    *                &lt;li&gt;Abstract
    *                    &lt;ul&gt;
    *                        &lt;li&gt;2 Acrylic Mediums&lt;/li&gt;
    *                        &lt;li&gt;Creative Acrylic&lt;/li&gt;
    *                        &lt;li&gt;Modern Painting&lt;/li&gt;
    *                        &lt;li&gt;Canvas Art&lt;/li&gt;
    *                        &lt;li&gt;Black white&lt;/li&gt;
    *                    &lt;/ul&gt;
    *                &lt;/li&gt;
    *            &lt;/ul&gt;
    *        &lt;/li&gt;
    *        &lt;li&gt;Books
    *            &lt;ul&gt;
    *                &lt;li&gt;Entertaining&lt;/li&gt;
    *                &lt;li&gt;Design&lt;/li&gt;
    *            &lt;/ul&gt;
    *        &lt;/li&gt;
    *        &lt;li&gt;Music
    *            &lt;ul&gt;
    *                &lt;li&gt;Mass&lt;/li&gt;
    *                &lt;li&gt;Folk&lt;/li&gt;
    *            &lt;/ul&gt;
    *        &lt;/li&gt;
    *    &lt;/ul&gt;
    * <br> 
    * &lt;script&gt;
			
		* // Initialize the TreeView with the loadOnDemand value specified.
            * $("#treeView").ejTreeView({ loadOnDemand: true });
            *  &lt;/script&gt;
			* @memberof ejTreeView
			* @instance
			*/
            loadOnDemand: false,
                        /**		
            * Sets the root class for TreeView theme. This cssClass API helps to use custom skinning option for TreeView control. By defining the root class using this API, we need to include this root class in CSS.
            * @default "gradient-lime"
            * @type {string}
            * @example 
            * &lt;ul id="treeView"&gt;
    *        &lt;li&gt;Artwork
    *            &lt;ul&gt;
    *                &lt;li&gt;Abstract
    *                    &lt;ul&gt;
    *                        &lt;li&gt;2 Acrylic Mediums&lt;/li&gt;
    *                        &lt;li&gt;Creative Acrylic&lt;/li&gt;
    *                        &lt;li&gt;Modern Painting&lt;/li&gt;
    *                        &lt;li&gt;Canvas Art&lt;/li&gt;
    *                        &lt;li&gt;Black white&lt;/li&gt;
    *                    &lt;/ul&gt;
    *                &lt;/li&gt;
    *          &lt;/ul&gt;
    *        &lt;/li&gt;
    *        &lt;li&gt;Books
    *            &lt;ul&gt;
    *                &lt;li&gt;Entertaining&lt;/li&gt;
    *                &lt;li&gt;Design&lt;/li&gt;
    *            &lt;/ul&gt;
    *        &lt;/li&gt;
    *        &lt;li&gt;Music
    *            &lt;ul&gt;
    *                &lt;li&gt;Mass&lt;/li&gt;
    *                &lt;li&gt;Folk&lt;/li&gt;
    *            &lt;/ul&gt;
    *        &lt;/li&gt;
    *    &lt;/ul&gt;
    * <br> 
    * &lt;script&gt;
            * //Initialize the TreeView with the cssClass value specified
            *   $("#treeView").ejTreeView({ cssClass: 'gradient-lime'});
            *  &lt;/script&gt;
            * @memberof ejTreeView
            * @instance
            */ 
            cssClass: "",
             /**  
            * We can customize the node our own style.
            * @default null
            * @type {string}
            * @example 
            * &lt;ul id="treeView"&gt;
    *        &lt;li&gt;&lt;a class="uk-style"&gt;UK&lt;/a&gt;
    *            &lt;ul&gt;
 *    &lt;li&gt;
    *                &lt;div class="cont-list"&gt;
    *                    &lt;img src="styles/images/treeview/template-image-1.png"/&gt;
    *                        &lt;div class="cont-del"&gt;&lt;/div&gt;
    *                        &lt;div class="cont-details"&gt;&lt;/div&gt;
    *                        &lt;b&gt;Steven John&lt;/b&gt;&lt;br/&gt;
    *                        &lt;span&gt;London&lt;/span&gt;&lt;br/&gt;
    *                        &lt;span&gt;555-5665-2323&lt;/span&gt;
    *                    &lt;/div&gt;
 *  &lt;div class="treeFooter"&gt;&lt;/div&gt;
    *                &lt;/li&gt;
    *            &lt;/ul&gt;
    *        &lt;/li&gt;
    *        &lt;li&gt;&lt;a class=""usa-style""&gt;USA&lt;/a&gt;
   *            &lt;ul&gt;
 *    &lt;li&gt;
    *                &lt;div class="cont-list"&gt;
    *                    &lt;img src="styles/images/treeview/template-image-2.png"/&gt;
    *                        &lt;div class="cont-del"&gt;&lt;/div&gt;
    *                        &lt;div class="cont-details"&gt;&lt;/div&gt;
    *                        &lt;b&gt;Andrew&lt;/b&gt;&lt;br/&gt;
    *                        &lt;span&gt;Capital way&lt;/span&gt;&lt;br/&gt;
    *                        &lt;span&gt;934-8374-2455&lt;/span&gt;
    *                    &lt;/div&gt;
 *  &lt;div class="treeFooter"&gt;&lt;/div&gt;
    *                &lt;/li&gt;
 *                &lt;li&gt;
    *                &lt;div class="cont-list"&gt;
    *                    &lt;img src="styles/images/treeview/template-image-3.png"/&gt;
    *                        &lt;div class="cont-del"&gt;&lt;/div&gt;
    *                        &lt;div class="cont-details"&gt;&lt;/div&gt;
    *                        &lt;b&gt;Angelica&lt;/b&gt;&lt;br/&gt;
    *                        &lt;span&gt;Dayton&lt;/span&gt;&lt;br/&gt;
    *                        &lt;span&gt;988-4243-0806&lt;/span&gt;
    *                    &lt;/div&gt;
 *  &lt;div class="treeFooter"&gt;&lt;/div&gt;
    *                &lt;/li&gt;
    *            &lt;/ul&gt;
    *        &lt;/li&gt;
    *    &lt;/ul&gt;
    * <br> 
    * &lt;script&gt;
           * //Initialize the TreeView with the template value specified
            *   $("#treeView").ejTreeView({ template: "templatelocaldata"});
            *  &lt;/script&gt;
            * @memberof ejTreeView
            * @instance
            */
            template: null,
                       /**		
			* Display Right to Left direction for node text.
			* @default false
			* @type {boolean}
			* @example 
			* &lt;ul id="treeView"&gt;
    *        &lt;li&gt;Artwork
    *            &lt;ul&gt;
    *                &lt;li&gt;Abstract
    *                    &lt;ul&gt;
    *                        &lt;li&gt;2 Acrylic Mediums&lt;/li&gt;
    *                        &lt;li&gt;Creative Acrylic&lt;/li&gt;
    *                        &lt;li&gt;Modern Painting&lt;/li&gt;
    *                        &lt;li&gt;Canvas Art&lt;/li&gt;
    *                        &lt;li&gt;Black white&lt;/li&gt;
    *                    &lt;/ul&gt;
    *                &lt;/li&gt;
    *            &lt;/ul&gt;
    *        &lt;/li&gt;
    *        &lt;li&gt;Books
    *            &lt;ul&gt;
    *                &lt;li&gt;Entertaining&lt;/li&gt;
    *                &lt;li&gt;Design&lt;/li&gt;
    *            &lt;/ul&gt;
    *        &lt;/li&gt;
    *        &lt;li&gt;Music
    *            &lt;ul&gt;
    *                &lt;li&gt;Mass&lt;/li&gt;
    *                &lt;li&gt;Folk&lt;/li&gt;
    *            &lt;/ul&gt;
    *        &lt;/li&gt;
    *    &lt;/ul&gt;
    * <br> 
    * &lt;script&gt;
			* // Initialize the TreeView with the rtl value specified.
            * $("#treeView").ejTreeView({ enableRTL: true });
            *  &lt;/script&gt;
			* @memberof ejTreeView
			* @instance
			*/
            enableRTL: false,
              /**		
            * Node expanded on specified events
            * @default "gradient-lime"
            * @type {string}
            * @example 
           * &lt;ul id="treeView"&gt;
    *        &lt;li&gt;Artwork
    *            &lt;ul&gt;
    *                &lt;li&gt;Abstract
    *                    &lt;ul&gt;
    *                        &lt;li&gt;2 Acrylic Mediums&lt;/li&gt;
    *                        &lt;li&gt;Creative Acrylic&lt;/li&gt;
    *                        &lt;li&gt;Modern Painting&lt;/li&gt;
    *                        &lt;li&gt;Canvas Art&lt;/li&gt;
    *                        &lt;li&gt;Black white&lt;/li&gt;
    *                    &lt;/ul&gt;
    *                &lt;/li&gt;
    *            &lt;/ul&gt;
    *        &lt;/li&gt;
    *        &lt;li&gt;Books
    *            &lt;ul&gt;
    *                &lt;li&gt;Entertaining&lt;/li&gt;
    *                &lt;li&gt;Design&lt;/li&gt;
    *            &lt;/ul&gt;
    *        &lt;/li&gt;
    *        &lt;li&gt;Music
    *            &lt;ul&gt;
    *                &lt;li&gt;Mass&lt;/li&gt;
    *                &lt;li&gt;Folk&lt;/li&gt;
    *            &lt;/ul&gt;
    *        &lt;/li&gt;
    *    &lt;/ul&gt;
    * <br> 
    * &lt;script&gt;	
            * //Initialize the TreeView with the expandOn value specified
            *   $("#treeView").ejTreeView({ expandOn: 'dblclick'});
            *  &lt;/script&gt;
            * @memberof ejTreeView
            * @instance
            */
            expandOn: "dblclick",
            /**		
			* Save current model value to browser cookies for state maintains. While refresh the TreeView control page the model value apply from browser cookies and HTML5 local storage.  
			* @default false
			* @type {boolean}
			* @example 
			  * &lt;ul id="treeView"&gt;
    *        &lt;li&gt;Artwork
    *            &lt;ul&gt;
    *                &lt;li&gt;Abstract
    *                    &lt;ul&gt;
    *                        &lt;li&gt;2 Acrylic Mediums&lt;/li&gt;
    *                        &lt;li&gt;Creative Acrylic&lt;/li&gt;
    *                        &lt;li&gt;Modern Painting&lt;/li&gt;
    *                        &lt;li&gt;Canvas Art&lt;/li&gt;
    *                        &lt;li&gt;Black white&lt;/li&gt;
    *                    &lt;/ul&gt;
    *                &lt;/li&gt;
    *            &lt;/ul&gt;
    *        &lt;/li&gt;
    *        &lt;li&gt;Books
    *            &lt;ul&gt;
    *                &lt;li&gt;Entertaining&lt;/li&gt;
    *                &lt;li&gt;Design&lt;/li&gt;
    *            &lt;/ul&gt;
    *        &lt;/li&gt;
    *        &lt;li&gt;Music
    *            &lt;ul&gt;
    *                &lt;li&gt;Mass&lt;/li&gt;
    *                &lt;li&gt;Folk&lt;/li&gt;
    *            &lt;/ul&gt;
    *        &lt;/li&gt;
    *    &lt;/ul&gt;
    * <br> 
    * &lt;script&gt;
			* // Initialize the TreeView with the persist value specified.
            * $("#treeView").ejTreeView({ enablePersistence: true });
            *  &lt;/script&gt;
			* @memberof ejTreeView
			* @instance
			*/
            enablePersistence: false,
 /**		
            * Allow to check the parent node automatically if this property is enabled.
            * @default false
            * @type {boolean}
            * @example 
            * &lt;ul id="treeView"&gt;
    *        &lt;li&gt;Artwork
    *            &lt;ul&gt;
    *                &lt;li&gt;Abstract
    *                    &lt;ul&gt;
    *                        &lt;li&gt;2 Acrylic Mediums&lt;/li&gt;
    *                        &lt;li&gt;Creative Acrylic&lt;/li&gt;
    *                        &lt;li&gt;Modern Painting&lt;/li&gt;
    *                        &lt;li&gt;Canvas Art&lt;/li&gt;
    *                        &lt;li&gt;Black white&lt;/li&gt;
    *                    &lt;/ul&gt;
    *                &lt;/li&gt;
    *            &lt;/ul&gt;
    *        &lt;/li&gt;
    *        &lt;li&gt;Books
    *            &lt;ul&gt;
    *                &lt;li&gt;Entertaining&lt;/li&gt;
    *                &lt;li&gt;Design&lt;/li&gt;
    *            &lt;/ul&gt;
    *        &lt;/li&gt;
    *        &lt;li&gt;Music
    *            &lt;ul&gt;
    *                &lt;li&gt;Mass&lt;/li&gt;
    *                &lt;li&gt;Folk&lt;/li&gt;
    *            &lt;/ul&gt;
    *        &lt;/li&gt;
    *    &lt;/ul&gt;
    * <br> 
    * &lt;script&gt;	
            * // Initialize the TreeView with the enabled value specified.
            * $("#treeView").ejTreeView({ enabled: true,showCheckbox:true });
            *  &lt;/script&gt;
            * @memberof ejTreeView
            * @instance
            */
            enabled: true,
             /**		
            * Expanded nodes index collection given to integer array
            * @default []
            * @type {array}
            * @example 
            * &lt;ul id="treeView"&gt;
    *        &lt;li&gt;Artwork
    *            &lt;ul&gt;
    *                &lt;li&gt;Abstract
    *                    &lt;ul&gt;
    *                        &lt;li&gt;2 Acrylic Mediums&lt;/li&gt;
    *                        &lt;li&gt;Creative Acrylic&lt;/li&gt;
    *                        &lt;li&gt;Modern Painting&lt;/li&gt;
    *                        &lt;li&gt;Canvas Art&lt;/li&gt;
    *                        &lt;li&gt;Black white&lt;/li&gt;
    *                    &lt;/ul&gt;
    *                &lt;/li&gt;
    *            &lt;/ul&gt;
    *        &lt;/li&gt;
    *        &lt;li&gt;Books
    *            &lt;ul&gt;
    *                &lt;li&gt;Entertaining&lt;/li&gt;
    *                &lt;li&gt;Design&lt;/li&gt;
    *            &lt;/ul&gt;
    *        &lt;/li&gt;
    *        &lt;li&gt;Music
    *            &lt;ul&gt;
    *                &lt;li&gt;Mass&lt;/li&gt;
    *                &lt;li&gt;Folk&lt;/li&gt;
    *            &lt;/ul&gt;
    *        &lt;/li&gt;
    *    &lt;/ul&gt;
    * <br> 
    * &lt;script&gt;		
            * // Initialize the TreeView with the expandedNodes value specified.
            * $("#treeView").ejTreeView({ expandedNodes: [0,7]  });
            *  &lt;/script&gt;
            * @memberof ejTreeView
            * @instance
            */
            expandedNodes: [],
            /**		
            * Checked nodes index collection given to integer array
            * @default []
            * @type {array}
            * @example 
            * &lt;ul id="treeView"&gt;
    *        &lt;li&gt;Artwork
    *            &lt;ul&gt;
    *                &lt;li&gt;Abstract
    *                    &lt;ul&gt;
    *                        &lt;li&gt;2 Acrylic Mediums&lt;/li&gt;
    *                        &lt;li&gt;Creative Acrylic&lt;/li&gt;
    *                        &lt;li&gt;Modern Painting&lt;/li&gt;
    *                        &lt;li&gt;Canvas Art&lt;/li&gt;
    *                        &lt;li&gt;Black white&lt;/li&gt;
    *                    &lt;/ul&gt;
    *                &lt;/li&gt;
    *            &lt;/ul&gt;
    *        &lt;/li&gt;
    *        &lt;li&gt;Books
    *            &lt;ul&gt;
    *                &lt;li&gt;Entertaining&lt;/li&gt;
    *                &lt;li&gt;Design&lt;/li&gt;
    *            &lt;/ul&gt;
    *        &lt;/li&gt;
    *        &lt;li&gt;Music
    *            &lt;ul&gt;
    *                &lt;li&gt;Mass&lt;/li&gt;
    *                &lt;li&gt;Folk&lt;/li&gt;
    *            &lt;/ul&gt;
    *        &lt;/li&gt;
    *    &lt;/ul&gt;
    * <br> 
    * &lt;script&gt;		
            * // Initialize the TreeView with the checkedNodes value specified.
            * $("#treeView").ejTreeView({ showCheckbox: true, checkedNodes: [1, 2] });
            *  &lt;/script&gt;
            * @memberof ejTreeView
            * @instance
            */
            checkedNodes: [],
            /**		
            * Defines the width of the TreeView.
            * @default Null
            * @type {string | number}
            * @example 
           * &lt;ul id="treeView"&gt;
    *        &lt;li&gt;Artwork
    *            &lt;ul&gt;
    *                &lt;li&gt;Abstract
    *                    &lt;ul&gt;
    *                        &lt;li&gt;2 Acrylic Mediums&lt;/li&gt;
    *                        &lt;li&gt;Creative Acrylic&lt;/li&gt;
    *                        &lt;li&gt;Modern Painting&lt;/li&gt;
    *                        &lt;li&gt;Canvas Art&lt;/li&gt;
    *                        &lt;li&gt;Black white&lt;/li&gt;
    *                    &lt;/ul&gt;
    *                &lt;/li&gt;
    *            &lt;/ul&gt;
    *        &lt;/li&gt;
    *        &lt;li&gt;Books
    *            &lt;ul&gt;
    *                &lt;li&gt;Entertaining&lt;/li&gt;
    *                &lt;li&gt;Design&lt;/li&gt;
    *            &lt;/ul&gt;
    *        &lt;/li&gt;
    *        &lt;li&gt;Music
    *            &lt;ul&gt;
    *                &lt;li&gt;Mass&lt;/li&gt;
    *                &lt;li&gt;Folk&lt;/li&gt;
    *            &lt;/ul&gt;
    *        &lt;/li&gt;
    *    &lt;/ul&gt;
    * <br> 
    * &lt;script&gt;
           * //Initialize the TreeView height property with the  value specified
            *   $("#treeView").ejTreeView({ width: 300 });
            *  &lt;/script&gt;
            * @memberof ejTreeView
            * @instance
            */
            width: null,
           /**		
            * Defines the height of the TreeView.
            * @default Null
            * @type {string | number}
            * @example 
           * &lt;ul id="treeView"&gt;
    *        &lt;li&gt;Artwork
    *            &lt;ul&gt;
    *                &lt;li&gt;Abstract
    *                    &lt;ul&gt;
    *                        &lt;li&gt;2 Acrylic Mediums&lt;/li&gt;
    *                        &lt;li&gt;Creative Acrylic&lt;/li&gt;
    *                        &lt;li&gt;Modern Painting&lt;/li&gt;
    *                        &lt;li&gt;Canvas Art&lt;/li&gt;
    *                        &lt;li&gt;Black white&lt;/li&gt;
    *                    &lt;/ul&gt;
    *                &lt;/li&gt;
    *            &lt;/ul&gt;
    *        &lt;/li&gt;
    *        &lt;li&gt;Books
    *            &lt;ul&gt;
    *                &lt;li&gt;Entertaining&lt;/li&gt;
    *                &lt;li&gt;Design&lt;/li&gt;
    *            &lt;/ul&gt;
    *        &lt;/li&gt;
    *        &lt;li&gt;Music
    *            &lt;ul&gt;
    *                &lt;li&gt;Mass&lt;/li&gt;
    *                &lt;li&gt;Folk&lt;/li&gt;
    *            &lt;/ul&gt;
    *        &lt;/li&gt;
    *    &lt;/ul&gt;
    * <br> 
    * &lt;script&gt;			
            * //Initialize the TreeView height property with the  value specified
            *   $("#treeView").ejTreeView({ height: 50 });
            *  &lt;/script&gt;
            * @memberof ejTreeView
            * @instance
            */
            height: null,
           /**     
            * Fires when nodeClick successfully.
            * @event
            * @name ejTreeView#nodeClick 	
            * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
            * @param {object}  argument.model returns the TreeView model
            * @param {string}  argument.type returns the name of the event
			* @param {object}  argument.event returns the event object
			* @param {object}  argument.currentElement returns the current element of the node clicked
			* @param {object}  argument.target returns the current element target
            * @example 
            * &lt;ul id="treeView"&gt;
    *        &lt;li&gt;Artwork
    *            &lt;ul&gt;
    *                &lt;li&gt;Abstract
    *                    &lt;ul&gt;
    *                        &lt;li&gt;2 Acrylic Mediums&lt;/li&gt;
    *                        &lt;li&gt;Creative Acrylic&lt;/li&gt;
    *                        &lt;li&gt;Modern Painting&lt;/li&gt;
    *                        &lt;li&gt;Canvas Art&lt;/li&gt;
    *                        &lt;li&gt;Black white&lt;/li&gt;
    *                    &lt;/ul&gt;
    *                &lt;/li&gt;
    *            &lt;/ul&gt;
    *        &lt;/li&gt;
    *        &lt;li&gt;Books
    *            &lt;ul&gt;
    *                &lt;li&gt;Entertaining&lt;/li&gt;
    *                &lt;li&gt;Design&lt;/li&gt;
    *            &lt;/ul&gt;
    *        &lt;/li&gt;
    *        &lt;li&gt;Music
    *            &lt;ul&gt;
    *                &lt;li&gt;Mass&lt;/li&gt;
    *                &lt;li&gt;Folk&lt;/li&gt;
    *            &lt;/ul&gt;
    *        &lt;/li&gt;
    *    &lt;/ul&gt;
    * <br> 
    * &lt;script&gt;
            * //To create click event for ejTreeView
            * $("#treeView").ejTreeView({ 
            *   	nodeClick: function(args) {}
            * });    
            *  &lt;/script&gt;  
            * @memberof ejTreeView
            * @instance
            */
            nodeClick: null,
            /**     
            * Fires when beforeExpand successfully.
            * @event
            * @name ejTreeView#beforeExpand 	
            * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
            * @param {object}  argument.model returns the TreeView model
            * @param {string}  argument.type returns the name of the event
			* @param {string}  argument.value returns the value of the node
			* @param {boolean}  argument.isChildLoaded if the child node is ready to expanded state; otherwise, false.
			* @param {object}  argument.currentElement returns the current element of the node clicked
            * @example 
            * &lt;ul id="treeView"&gt;
    *        &lt;li&gt;Artwork
    *            &lt;ul&gt;
    *                &lt;li&gt;Abstract
    *                    &lt;ul&gt;
    *                        &lt;li&gt;2 Acrylic Mediums&lt;/li&gt;
    *                        &lt;li&gt;Creative Acrylic&lt;/li&gt;
    *                        &lt;li&gt;Modern Painting&lt;/li&gt;
    *                        &lt;li&gt;Canvas Art&lt;/li&gt;
    *                        &lt;li&gt;Black white&lt;/li&gt;
    *                    &lt;/ul&gt;
    *                &lt;/li&gt;
    *            &lt;/ul&gt;
    *        &lt;/li&gt;
    *        &lt;li&gt;Books
    *            &lt;ul&gt;
    *                &lt;li&gt;Entertaining&lt;/li&gt;
    *                &lt;li&gt;Design&lt;/li&gt;
    *            &lt;/ul&gt;
    *        &lt;/li&gt;
    *        &lt;li&gt;Music
    *            &lt;ul&gt;
    *                &lt;li&gt;Mass&lt;/li&gt;
    *                &lt;li&gt;Folk&lt;/li&gt;
    *            &lt;/ul&gt;
    *        &lt;/li&gt;
    *    &lt;/ul&gt;
    * <br> 
    * &lt;script&gt;
            * //To create beforeExpand event for ejTreeView
            * $("#treeView").ejTreeView({ 
            *   	beforeExpand: function(args) {}
            * });     
            *  &lt;/script&gt; 
            * @memberof ejTreeView
            * @instance
            */
            beforeExpand: null,
           /**     
            * Fires when nodeExpand successfully.
            * @event
            * @name ejTreeView#nodeExpand 	
            * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
            * @param {object}  argument.model returns the TreeView model
            * @param {string}  argument.type returns the name of the event
			* @param {string}  argument.value returns the value of the node
			* @param {boolean}  argument.isChildLoaded if the child node is ready to expanded state; otherwise, false.
			* @param {object}  argument.currentElement returns the current element of the node clicked
            * @example 
            * &lt;ul id="treeView"&gt;
    *        &lt;li&gt;Artwork
    *            &lt;ul&gt;
    *                &lt;li&gt;Abstract
    *                    &lt;ul&gt;
    *                        &lt;li&gt;2 Acrylic Mediums&lt;/li&gt;
    *                        &lt;li&gt;Creative Acrylic&lt;/li&gt;
    *                        &lt;li&gt;Modern Painting&lt;/li&gt;
    *                        &lt;li&gt;Canvas Art&lt;/li&gt;
    *                        &lt;li&gt;Black white&lt;/li&gt;
    *                    &lt;/ul&gt;
    *                &lt;/li&gt;
    *            &lt;/ul&gt;
    *        &lt;/li&gt;
    *        &lt;li&gt;Books
    *            &lt;ul&gt;
    *                &lt;li&gt;Entertaining&lt;/li&gt;
    *                &lt;li&gt;Design&lt;/li&gt;
    *            &lt;/ul&gt;
    *        &lt;/li&gt;
    *        &lt;li&gt;Music
    *            &lt;ul&gt;
    *                &lt;li&gt;Mass&lt;/li&gt;
    *                &lt;li&gt;Folk&lt;/li&gt;
    *            &lt;/ul&gt;
    *        &lt;/li&gt;
    *    &lt;/ul&gt;
    * <br> 
    * &lt;script&gt;
            * //To create expand event for ejTreeView
            * $("#treeView").ejTreeView({ 
            *   	nodeExpand: function(args) {}
            * });      
            *  &lt;/script&gt;
            * @memberof ejTreeView
            * @instance
            */
            nodeExpand: null,
 /**     
            * Fires when beforeCollapse successfully.
            * @event
            * @name ejTreeView#beforeCollapse 	
            * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
            * @param {object}  argument.model returns the TreeView model
            * @param {string}  argument.type returns the name of the event
			* @param {string}  argument.value returns the value of the node			
			* @param {object}  argument.currentElement returns the current element of the node clicked
            * @example 
            * &lt;ul id="treeView"&gt;
    *        &lt;li&gt;Artwork
    *            &lt;ul&gt;
    *                &lt;li&gt;Abstract
    *                    &lt;ul&gt;
    *                        &lt;li&gt;2 Acrylic Mediums&lt;/li&gt;
    *                        &lt;li&gt;Creative Acrylic&lt;/li&gt;
    *                        &lt;li&gt;Modern Painting&lt;/li&gt;
    *                        &lt;li&gt;Canvas Art&lt;/li&gt;
    *                        &lt;li&gt;Black white&lt;/li&gt;
    *                    &lt;/ul&gt;
    *                &lt;/li&gt;
    *            &lt;/ul&gt;
    *        &lt;/li&gt;
    *        &lt;li&gt;Books
    *            &lt;ul&gt;
    *                &lt;li&gt;Entertaining&lt;/li&gt;
    *                &lt;li&gt;Design&lt;/li&gt;
    *            &lt;/ul&gt;
    *        &lt;/li&gt;
    *        &lt;li&gt;Music
    *            &lt;ul&gt;
    *                &lt;li&gt;Mass&lt;/li&gt;
    *                &lt;li&gt;Folk&lt;/li&gt;
    *            &lt;/ul&gt;
    *        &lt;/li&gt;
    *    &lt;/ul&gt;
    * <br> 
    * &lt;script&gt;
            * //To create beforeCollapse event for ejTreeView
            * $("#treeView").ejTreeView({ 
            *   	beforeCollapse: function(args) {}
            * }); 
            *  &lt;/script&gt;     
            * @memberof ejTreeView
            * @instance
            */
            beforeCollapse: null,
           /**     
            * Fires when nodeCollapse successfully.
            * @event
            * @name ejTreeView#nodeCollapse 	
            * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
            * @param {object}  argument.model returns the TreeView model
            * @param {string}  argument.type returns the name of the event
			* @param {string}  argument.value returns the value of the node			
			* @param {object}  argument.currentElement returns the current element of the node clicked
            * @example 
            * &lt;ul id="treeView"&gt;
    *        &lt;li&gt;Artwork
    *            &lt;ul&gt;
    *                &lt;li&gt;Abstract
    *                    &lt;ul&gt;
    *                        &lt;li&gt;2 Acrylic Mediums&lt;/li&gt;
    *                        &lt;li&gt;Creative Acrylic&lt;/li&gt;
    *                        &lt;li&gt;Modern Painting&lt;/li&gt;
    *                        &lt;li&gt;Canvas Art&lt;/li&gt;
    *                        &lt;li&gt;Black white&lt;/li&gt;
    *                    &lt;/ul&gt;
    *                &lt;/li&gt;
    *            &lt;/ul&gt;
    *        &lt;/li&gt;
    *        &lt;li&gt;Books
    *            &lt;ul&gt;
    *                &lt;li&gt;Entertaining&lt;/li&gt;
    *                &lt;li&gt;Design&lt;/li&gt;
    *            &lt;/ul&gt;
    *        &lt;/li&gt;
    *        &lt;li&gt;Music
    *            &lt;ul&gt;
    *                &lt;li&gt;Mass&lt;/li&gt;
    *                &lt;li&gt;Folk&lt;/li&gt;
    *            &lt;/ul&gt;
    *        &lt;/li&gt;
    *    &lt;/ul&gt;
    * <br> 
    * &lt;script&gt;
            * //To create nodeCollapse event for ejTreeView
            * $("#treeView").ejTreeView({ 
            *   	nodeCollapse: function(args) {}
            * }); 
            *  &lt;/script&gt;     
            * @memberof ejTreeView
            * @instance
            */
            nodeCollapse: null,
             /**     
            * Fires when nodeSelect successfully.
            * @event
            * @name ejTreeView#nodeSelect 	
            * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
            * @param {object}  argument.model returns the TreeView model
            * @param {string}  argument.type returns the name of the event
			* @param {string}  argument.value returns the value of the node			
			* @param {object}  argument.currentElement returns the current element of the node clicked
            * @example 
            * &lt;ul id="treeView"&gt;
    *        &lt;li&gt;Artwork
    *            &lt;ul&gt;
    *                &lt;li&gt;Abstract
    *                    &lt;ul&gt;
    *                        &lt;li&gt;2 Acrylic Mediums&lt;/li&gt;
    *                        &lt;li&gt;Creative Acrylic&lt;/li&gt;
    *                        &lt;li&gt;Modern Painting&lt;/li&gt;
    *                        &lt;li&gt;Canvas Art&lt;/li&gt;
    *                        &lt;li&gt;Black white&lt;/li&gt;
    *                    &lt;/ul&gt;
    *                &lt;/li&gt;
    *            &lt;/ul&gt;
    *        &lt;/li&gt;
    *        &lt;li&gt;Books
    *            &lt;ul&gt;
    *                &lt;li&gt;Entertaining&lt;/li&gt;
    *                &lt;li&gt;Design&lt;/li&gt;
    *            &lt;/ul&gt;
    *        &lt;/li&gt;
    *        &lt;li&gt;Music
    *            &lt;ul&gt;
    *                &lt;li&gt;Mass&lt;/li&gt;
    *                &lt;li&gt;Folk&lt;/li&gt;
    *            &lt;/ul&gt;
    *        &lt;/li&gt;
    *    &lt;/ul&gt;
    * <br> 
    * &lt;script&gt;
            * //To create select event for ejTreeView
            * $("#treeView").ejTreeView({ 
            *   	nodeSelect: function(args) {}
            * });   
            *  &lt;/script&gt;   
            * @memberof ejTreeView
            * @instance
            */
            nodeSelect: null,
             /**     
            * Fires when nodeCheck successfully.
            * @event
            * @name ejTreeView#nodeCheck 	
            * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
            * @param {object}  argument.model returns the TreeView model
            * @param {string}  argument.type returns the name of the event
			* @param {object}  argument.event returns the event object
			* @param {string}  argument.value returns the value of the node			
			* @param {object}  argument.currentElement returns the current element of the node clicked
			* @param {boolean}  argument.isChecked it returns true when the node checkbox is checked; otherwise, false.
            * @example
            * &lt;ul id="treeView"&gt;
    *        &lt;li&gt;Artwork
    *            &lt;ul&gt;
    *                &lt;li&gt;Abstract
    *                    &lt;ul&gt;
    *                        &lt;li&gt;2 Acrylic Mediums&lt;/li&gt;
    *                        &lt;li&gt;Creative Acrylic&lt;/li&gt;
    *                        &lt;li&gt;Modern Painting&lt;/li&gt;
    *                        &lt;li&gt;Canvas Art&lt;/li&gt;
    *                        &lt;li&gt;Black white&lt;/li&gt;
    *                    &lt;/ul&gt;
    *                &lt;/li&gt;
    *            &lt;/ul&gt;
    *        &lt;/li&gt;
    *        &lt;li&gt;Books
    *            &lt;ul&gt;
    *                &lt;li&gt;Entertaining&lt;/li&gt;
    *                &lt;li&gt;Design&lt;/li&gt;
    *            &lt;/ul&gt;
    *        &lt;/li&gt;
    *        &lt;li&gt;Music
    *            &lt;ul&gt;
    *                &lt;li&gt;Mass&lt;/li&gt;
    *                &lt;li&gt;Folk&lt;/li&gt;
    *            &lt;/ul&gt;
    *        &lt;/li&gt;
    *    &lt;/ul&gt;
    * <br> 
    * &lt;script&gt; 
            * //To create select event for ejTreeView
            * $("#treeView").ejTreeView({ 
                showCheckbox:true,
            *   	nodeCheck: function(args) {}
            * }); 
            *  &lt;/script&gt;     
            * @memberof ejTreeView
            * @instance
            */
            nodeCheck: null,
           /**     
            * Fires when nodeUncheck successfully.
            * @event
            * @name ejTreeView#nodeUncheck 	
            * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
            * @param {object}  argument.model returns the TreeView model
            * @param {string}  argument.type returns the name of the event
			* @param {object}  argument.event returns the event object
			* @param {string}  argument.value returns the value of the node			
			* @param {object}  argument.currentElement returns the current element of the node clicked
			* @param {boolean}  argument.isChecked it returns true when the node checkbox is checked; otherwise, false.
            * @example 
            * &lt;ul id="treeView"&gt;
    *        &lt;li&gt;Artwork
    *            &lt;ul&gt;
    *                &lt;li&gt;Abstract
    *                    &lt;ul&gt;
    *                        &lt;li&gt;2 Acrylic Mediums&lt;/li&gt;
    *                        &lt;li&gt;Creative Acrylic&lt;/li&gt;
    *                        &lt;li&gt;Modern Painting&lt;/li&gt;
    *                        &lt;li&gt;Canvas Art&lt;/li&gt;
    *                        &lt;li&gt;Black white&lt;/li&gt;
    *                    &lt;/ul&gt;
    *                &lt;/li&gt;
    *            &lt;/ul&gt;
    *        &lt;/li&gt;
    *        &lt;li&gt;Books
    *            &lt;ul&gt;
    *                &lt;li&gt;Entertaining&lt;/li&gt;
    *                &lt;li&gt;Design&lt;/li&gt;
    *            &lt;/ul&gt;
    *        &lt;/li&gt;
    *        &lt;li&gt;Music
    *            &lt;ul&gt;
    *                &lt;li&gt;Mass&lt;/li&gt;
    *                &lt;li&gt;Folk&lt;/li&gt;
    *            &lt;/ul&gt;
    *        &lt;/li&gt;
    *    &lt;/ul&gt;
    * <br> 
    * &lt;script&gt;
            * //To create nodeUncheck event for ejTreeView
            * $("#treeView").ejTreeView({ 
                showCheckbox:true,
            *   	nodeUncheck: function(args) {}
            * });   
            *  &lt;/script&gt;   
            * @memberof ejTreeView
            * @instance
            */
            nodeUncheck: null,
            /**     
            * Fires when inlineEditValidation successfully.
            * @event
            * @name ejTreeView#inlineEditValidation 	
            * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
            * @param {object}  argument.model returns the TreeView model
            * @param {string}  argument.type returns the name of the event
			* @param {string}  argument.newText returns the new entered text for the node
            * @param {object}  argument.nodeId returns the current node element id
            * @param {string}  argument.oldText returns the old node text
            * @example 
            * &lt;ul id="treeView"&gt;
    *        &lt;li&gt;Artwork
    *            &lt;ul&gt;
    *                &lt;li&gt;Abstract
    *                    &lt;ul&gt;
    *                        &lt;li&gt;2 Acrylic Mediums&lt;/li&gt;
    *                        &lt;li&gt;Creative Acrylic&lt;/li&gt;
    *                        &lt;li&gt;Modern Painting&lt;/li&gt;
    *                        &lt;li&gt;Canvas Art&lt;/li&gt;
    *                        &lt;li&gt;Black white&lt;/li&gt;
    *                    &lt;/ul&gt;
    *                &lt;/li&gt;
    *            &lt;/ul&gt;
    *        &lt;/li&gt;
    *        &lt;li&gt;Books
    *            &lt;ul&gt;
    *                &lt;li&gt;Entertaining&lt;/li&gt;
    *                &lt;li&gt;Design&lt;/li&gt;
    *            &lt;/ul&gt;
    *        &lt;/li&gt;
    *        &lt;li&gt;Music
    *            &lt;ul&gt;
    *                &lt;li&gt;Mass&lt;/li&gt;
    *                &lt;li&gt;Folk&lt;/li&gt;
    *            &lt;/ul&gt;
    *        &lt;/li&gt;
    *    &lt;/ul&gt;
    * <br> 
    * &lt;script&gt;
            * //To create inlineEditValidation event for ejTreeView
            * $("#treeView").ejTreeView({ 
            *   	inlineEditValidation: function(args) {}
            * });  
            *  &lt;/script&gt;    
            * @memberof ejTreeView
            * @instance
            */
            inlineEditValidation: null,
          /**     
            * Fires when beforeEdit successfully.
            * @event
            * @name ejTreeView#beforeEdit 	
            * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
            * @param {object}  argument.model returns the TreeView model
            * @param {string}  argument.type returns the name of the event
			* @param {object}  argument.currentElement returns the current element of the node clicked
            * @example 
            * &lt;ul id="treeView"&gt;
    *        &lt;li&gt;Artwork
    *            &lt;ul&gt;
    *                &lt;li&gt;Abstract
    *                    &lt;ul&gt;
    *                        &lt;li&gt;2 Acrylic Mediums&lt;/li&gt;
    *                        &lt;li&gt;Creative Acrylic&lt;/li&gt;
    *                        &lt;li&gt;Modern Painting&lt;/li&gt;
    *                        &lt;li&gt;Canvas Art&lt;/li&gt;
    *                        &lt;li&gt;Black white&lt;/li&gt;
    *                    &lt;/ul&gt;
    *                &lt;/li&gt;
    *            &lt;/ul&gt;
    *        &lt;/li&gt;
    *        &lt;li&gt;Books
    *            &lt;ul&gt;
    *                &lt;li&gt;Entertaining&lt;/li&gt;
    *                &lt;li&gt;Design&lt;/li&gt;
    *            &lt;/ul&gt;
    *        &lt;/li&gt;
    *        &lt;li&gt;Music
    *            &lt;ul&gt;
    *                &lt;li&gt;Mass&lt;/li&gt;
    *                &lt;li&gt;Folk&lt;/li&gt;
    *            &lt;/ul&gt;
    *        &lt;/li&gt;
    *    &lt;/ul&gt;
    * <br> 
    * &lt;script&gt;
            * //To create beforeEdit event for ejTreeView
            * $("#treeView").ejTreeView({ 
            *   	beforeEdit: function(args) {}
            * }); 
            *  &lt;/script&gt;     
            * @memberof ejTreeView
            * @instance
            */
            beforeEdit: null,
           /**     
            * Fires when keyPress successfully.
            * @event
            * @name ejTreeView#keyPress 	
            * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
            * @param {object}  argument.model returns the TreeView model
            * @param {string}  argument.type returns the name of the event
			* @param {object}  argument.event returns the event object
			* @param {object}  argument.currentElement returns the current element of the node clicked
			* @param {string}  argument.value returns the value of the node
			* @param {string}  argument.path returns node path from root element
			* @param {number}  argument.keyCode returns the keypressed keycode value
			* @param {boolean}  argument.isExpanded it returns when the current node is in expanded state; otherwise, false.
            * @example 
            * &lt;ul id="treeView"&gt;
    *        &lt;li&gt;Artwork
    *            &lt;ul&gt;
    *                &lt;li&gt;Abstract
    *                    &lt;ul&gt;
    *                        &lt;li&gt;2 Acrylic Mediums&lt;/li&gt;
    *                        &lt;li&gt;Creative Acrylic&lt;/li&gt;
    *                        &lt;li&gt;Modern Painting&lt;/li&gt;
    *                        &lt;li&gt;Canvas Art&lt;/li&gt;
    *                        &lt;li&gt;Black white&lt;/li&gt;
    *                    &lt;/ul&gt;
    *                &lt;/li&gt;
    *            &lt;/ul&gt;
    *        &lt;/li&gt;
    *        &lt;li&gt;Books
    *            &lt;ul&gt;
    *                &lt;li&gt;Entertaining&lt;/li&gt;
    *                &lt;li&gt;Design&lt;/li&gt;
    *            &lt;/ul&gt;
    *        &lt;/li&gt;
    *        &lt;li&gt;Music
    *            &lt;ul&gt;
    *                &lt;li&gt;Mass&lt;/li&gt;
    *                &lt;li&gt;Folk&lt;/li&gt;
    *            &lt;/ul&gt;
    *        &lt;/li&gt;
    *    &lt;/ul&gt;
    * <br> 
    * &lt;script&gt;
            * //To create keyPress event for ejTreeView
            * $("#treeView").ejTreeView({ 
            *   	keyPress: function(args) {}
            * });     
            *  &lt;/script&gt; 
            * @memberof ejTreeView
            * @instance
            */
            keyPress: null,
              /**     
            * Fires when nodeDragStart successfully.
            * @event
            * @name ejTreeView#nodeDragStart 	
            * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
            * @param {object}  argument.model returns the TreeView model
            * @param {string}  argument.type returns the name of the event
			* @param {object}  argument.target returns the current element target
			* @param {object}  argument.event returns the event object
            * @example 
            * &lt;ul id="treeView"&gt;
    *        &lt;li&gt;Artwork
    *            &lt;ul&gt;
    *                &lt;li&gt;Abstract
    *                    &lt;ul&gt;
    *                        &lt;li&gt;2 Acrylic Mediums&lt;/li&gt;
    *                        &lt;li&gt;Creative Acrylic&lt;/li&gt;
    *                        &lt;li&gt;Modern Painting&lt;/li&gt;
    *                        &lt;li&gt;Canvas Art&lt;/li&gt;
    *                        &lt;li&gt;Black white&lt;/li&gt;
    *                    &lt;/ul&gt;
    *                &lt;/li&gt;
    *            &lt;/ul&gt;
    *        &lt;/li&gt;
    *        &lt;li&gt;Books
    *            &lt;ul&gt;
    *                &lt;li&gt;Entertaining&lt;/li&gt;
    *                &lt;li&gt;Design&lt;/li&gt;
    *            &lt;/ul&gt;
    *        &lt;/li&gt;
    *        &lt;li&gt;Music
    *            &lt;ul&gt;
    *                &lt;li&gt;Mass&lt;/li&gt;
    *                &lt;li&gt;Folk&lt;/li&gt;
    *            &lt;/ul&gt;
    *        &lt;/li&gt;
    *    &lt;/ul&gt;
    * <br> 
    * &lt;script&gt;
            * //nodeDragStart event for ejTreeView
            * $("#treeView").ejTreeView({ 
            *   	nodeDragStart: function(args) {}
            * });  
            *  &lt;/script&gt;    
            * @memberof ejTreeView
            * @instance
            */
            nodeDragStart: null,
             /**     
            * Fires when nodeDrag successfully.
            * @event
            * @name ejTreeView#nodeDrag 	
            * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
            * @param {object}  argument.model returns the TreeView model
            * @param {string}  argument.type returns the name of the event
			* @param {object}  argument.target returns the current element target
			* @param {object}  argument.event returns the event object
            * @example 
            * &lt;ul id="treeView"&gt;
    *        &lt;li&gt;Artwork
    *            &lt;ul&gt;
    *                &lt;li&gt;Abstract
    *                    &lt;ul&gt;
    *                        &lt;li&gt;2 Acrylic Mediums&lt;/li&gt;
    *                        &lt;li&gt;Creative Acrylic&lt;/li&gt;
    *                        &lt;li&gt;Modern Painting&lt;/li&gt;
    *                        &lt;li&gt;Canvas Art&lt;/li&gt;
    *                        &lt;li&gt;Black white&lt;/li&gt;
    *                    &lt;/ul&gt;
    *                &lt;/li&gt;
    *            &lt;/ul&gt;
    *        &lt;/li&gt;
    *        &lt;li&gt;Books
    *            &lt;ul&gt;
    *                &lt;li&gt;Entertaining&lt;/li&gt;
    *                &lt;li&gt;Design&lt;/li&gt;
    *            &lt;/ul&gt;
    *        &lt;/li&gt;
    *        &lt;li&gt;Music
    *            &lt;ul&gt;
    *                &lt;li&gt;Mass&lt;/li&gt;
    *                &lt;li&gt;Folk&lt;/li&gt;
    *            &lt;/ul&gt;
    *        &lt;/li&gt;
    *    &lt;/ul&gt;
    * <br> 
    * &lt;script&gt;
            * //To create nodeDrag event for ejTreeView
            * $("#treeView").ejTreeView({ 
            *   	nodeDrag: function(args) {}
            * });      
            *  &lt;/script&gt;
            * @memberof ejTreeView
            * @instance
            */
            nodeDrag: null,
                        /**     
            * Fires when nodeDragStop successfully.
            * @event
            * @name ejTreeView#nodeDragStop 	
            * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
            * @param {object}  argument.model returns the TreeView model
            * @param {string}  argument.type returns the name of the event
			* @param {object}  argument.target returns the current element target
			* @param {object}  argument.event returns the event object
			* @param {string}  argument.position returns the element new dragged place from its original place.
            * @example 
            * &lt;ul id="treeView"&gt;
    *        &lt;li&gt;Artwork
    *            &lt;ul&gt;
    *                &lt;li&gt;Abstract
    *                    &lt;ul&gt;
    *                        &lt;li&gt;2 Acrylic Mediums&lt;/li&gt;
    *                        &lt;li&gt;Creative Acrylic&lt;/li&gt;
    *                        &lt;li&gt;Modern Painting&lt;/li&gt;
    *                        &lt;li&gt;Canvas Art&lt;/li&gt;
    *                        &lt;li&gt;Black white&lt;/li&gt;
    *                    &lt;/ul&gt;
    *                &lt;/li&gt;
    *            &lt;/ul&gt;
    *        &lt;/li&gt;
    *        &lt;li&gt;Books
    *            &lt;ul&gt;
    *                &lt;li&gt;Entertaining&lt;/li&gt;
    *                &lt;li&gt;Design&lt;/li&gt;
    *            &lt;/ul&gt;
    *        &lt;/li&gt;
    *        &lt;li&gt;Music
    *            &lt;ul&gt;
    *                &lt;li&gt;Mass&lt;/li&gt;
    *                &lt;li&gt;Folk&lt;/li&gt;
    *            &lt;/ul&gt;
    *        &lt;/li&gt;
    *    &lt;/ul&gt;
    * <br> 
    * &lt;script&gt;
            * //To create nodeDragStop event for ejTreeView
            * $("#treeView").ejTreeView({ 
            *   	nodeDragStop: function(args) {}
            * });  
            *  &lt;/script&gt;    
            * @memberof ejTreeView
            * @instance
            */
            nodeDragStop: null,
           /**     
            * Fires when nodeDropped successfully.
            * @event
            * @name ejTreeView#nodeDropped 	
            * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
            * @param {object}  argument.model returns the TreeView model
            * @param {string}  argument.type returns the name of the event
			* @param {object}  argument.target returns the current element target
			* @param {object}  argument.event returns the event object
			* @param {object}  argument.dropedElement returns the event object
            * @example 
            * &lt;ul id="treeView"&gt;
    *        &lt;li&gt;Artwork
    *            &lt;ul&gt;
    *                &lt;li&gt;Abstract
    *                    &lt;ul&gt;
    *                        &lt;li&gt;2 Acrylic Mediums&lt;/li&gt;
    *                        &lt;li&gt;Creative Acrylic&lt;/li&gt;
    *                        &lt;li&gt;Modern Painting&lt;/li&gt;
    *                        &lt;li&gt;Canvas Art&lt;/li&gt;
    *                        &lt;li&gt;Black white&lt;/li&gt;
    *                    &lt;/ul&gt;
    *                &lt;/li&gt;
    *            &lt;/ul&gt;
    *        &lt;/li&gt;
    *        &lt;li&gt;Books
    *            &lt;ul&gt;
    *                &lt;li&gt;Entertaining&lt;/li&gt;
    *                &lt;li&gt;Design&lt;/li&gt;
    *            &lt;/ul&gt;
    *        &lt;/li&gt;
    *        &lt;li&gt;Music
    *            &lt;ul&gt;
    *                &lt;li&gt;Mass&lt;/li&gt;
    *                &lt;li&gt;Folk&lt;/li&gt;
    *            &lt;/ul&gt;
    *        &lt;/li&gt;
    *    &lt;/ul&gt;
    * <br> 
    * &lt;script&gt;
            * //To create nodeDropped event for ejTreeView
            * $("#treeView").ejTreeView({ 
            *   	nodeDropped: function(args) {}
            * });  
            *  &lt;/script&gt;    
            * @memberof ejTreeView
            * @instance
            */
            nodeDropped: null,
                   /**     
            * Fires when created successfully.
            * @event
            * @name ejTreeView#created  
            * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
            * @param {object}  argument.model returns the TreeView model
            * @param {string}  argument.type returns the name of the event
            * @example 
            * &lt;ul id="treeView"&gt;
    *        &lt;li&gt;Artwork
    *            &lt;ul&gt;
    *                &lt;li&gt;Abstract
    *                    &lt;ul&gt;
    *                        &lt;li&gt;2 Acrylic Mediums&lt;/li&gt;
    *                        &lt;li&gt;Creative Acrylic&lt;/li&gt;
    *                        &lt;li&gt;Modern Painting&lt;/li&gt;
    *                        &lt;li&gt;Canvas Art&lt;/li&gt;
    *                        &lt;li&gt;Black white&lt;/li&gt;
    *                    &lt;/ul&gt;
    *                &lt;/li&gt;
    *            &lt;/ul&gt;
    *        &lt;/li&gt;
    *        &lt;li&gt;Books
    *            &lt;ul&gt;
    *                &lt;li&gt;Entertaining&lt;/li&gt;
    *                &lt;li&gt;Design&lt;/li&gt;
    *            &lt;/ul&gt;
    *        &lt;/li&gt;
    *        &lt;li&gt;Music
    *            &lt;ul&gt;
    *                &lt;li&gt;Mass&lt;/li&gt;
    *                &lt;li&gt;Folk&lt;/li&gt;
    *            &lt;/ul&gt;
    *        &lt;/li&gt;
    *    &lt;/ul&gt;
    * <br> 
    * &lt;script&gt;
            * //To create node for ejTreeView
            * $("#treeView").ejTreeView({ 
            *       create: function(args) {}
            * });
            *  &lt;/script&gt;      
            * @memberof ejTreeView
            * @instance
            */
            create: null,
            /**     
            * Fires when destroyed successfully.
            * @event
            * @name ejTreeView#destroyed  
            * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
            * @param {object}  argument.model returns the TreeView model
            * @param {string}  argument.type returns the name of the event
            * @example 
            * &lt;ul id="treeView"&gt;
    *        &lt;li&gt;Artwork
    *            &lt;ul&gt;
    *                &lt;li&gt;Abstract
    *                    &lt;ul&gt;
    *                        &lt;li&gt;2 Acrylic Mediums&lt;/li&gt;
    *                        &lt;li&gt;Creative Acrylic&lt;/li&gt;
    *                        &lt;li&gt;Modern Painting&lt;/li&gt;
    *                        &lt;li&gt;Canvas Art&lt;/li&gt;
    *                        &lt;li&gt;Black white&lt;/li&gt;
    *                    &lt;/ul&gt;
    *                &lt;/li&gt;
    *            &lt;/ul&gt;
    *        &lt;/li&gt;
    *        &lt;li&gt;Books
    *            &lt;ul&gt;
    *                &lt;li&gt;Entertaining&lt;/li&gt;
    *                &lt;li&gt;Design&lt;/li&gt;
    *            &lt;/ul&gt;
    *        &lt;/li&gt;
    *        &lt;li&gt;Music
    *            &lt;ul&gt;
    *                &lt;li&gt;Mass&lt;/li&gt;
    *                &lt;li&gt;Folk&lt;/li&gt;
    *            &lt;/ul&gt;
    *        &lt;/li&gt;
    *    &lt;/ul&gt;
    * <br> 
    * &lt;script&gt;
            * //To create destroy event for ejTreeView
            * $("#treeView").ejTreeView({ 
            *       destroy: function(args) {}
            * }); 
            *  &lt;/script&gt;     
            * @memberof ejTreeView
            * @instance
            */
            destroy: null
            
        },
        /**
        * Specify the data types for default properties 
        * @private
        */
        dataTypes: {
            showCheckbox: "boolean",
			enableAnimation: "boolean",
            allowDragAndDrop: "boolean",
            allowDropChild: "boolean",
            allowDragAndDropAcrossControl: "boolean",
            allowEditing: "boolean",
            allowKeyboardNavigation: "boolean",
            autoCheckParentNode: "boolean",
            loadOnDemand: "boolean",
            enableRTL: "boolean",
            expandOn: "string",
            enablePersistence: "boolean",
            items: "data",
            fields: {
                dataSource: "data",
                query: "data",
                child: "data"
            },
            expandedNodes: "array",
            checkedNodes: "array"
        },
        /**
         * To configure the properties at runtime using SetModel		
		 * @private
         */
        _setModel: function (options) {
            /// <summary>This function is used to set the Treeview model</summary>
            if (!this.model.enabled && ej.isNullOrUndefined(options["enabled"])) return false;
            for (var key in options) {
                switch (key) {
                    case "cssClass": this._changeSkin(options[key]); break;
                    case "fields":
                        this._unWireEvents();
                        var f = this.element.hasClass("e-js") ? false : true, element = f ? this.element.children("ul") : this.element;
                        element.empty();
                        $.extend(this.model.fields, options[key]);
                        this._checkDataBinding();
                        if (f) {
                            var tempUl = this.element.children(".e-treeview-ul");
                            element.append(tempUl.children());
                            tempUl.remove();
                        }
                        break;
                    case "allowDragAndDropAcrossControl":
                        {
                            this.model.allowDragAndDropAcrossControl = options[key];
                            this._enableDragDrop();
                            break;
                        }
                    case "enabled": this._enabledAction(options[key]); break;
                    case "showCheckbox": {
                        if (options[key]) {
                            this.model.showCheckbox = options[key];
                            this._showCheckBox();
                        } else {
                            this.element.find('span.e-chkbox-wrap').next().remove();
                            this.element.find('span.e-chkbox-wrap').remove();
                        }
                        break;
                    }
					case "enableAnimation":{
							this.model.enableAnimation=options[key];
							break;
					}
                    case "expandedNodes": {
                        this._collapseAll();
                        this.model.expandedNodes = [];
                        for (var i = 0; i < options[key].length; i++) {
                            this._expandNode(this._liList[options[key][i]]);
                        }
                        break;
                    }
                    case "checkedNodes": {
                        if (this.model.showCheckbox) {
                            this._unCheckAll();
                            this.model.checkedNodes = [];
                            for (var j = 0; j < options[key].length; j++) {
                                var node = this._liList[options[key][j]];
                                this._nodeCheckedAction($(node).find("span.e-chkbox-wrap").first());
                            }
                        }
                        break;
                    }
                    case "expandOn": {
                        this._on(this.element, options[key], this._expandEventHandler);
                        break;
                    }
                    case "allowEditing": {
                        if (options[key]){
                            this._enableAllowEdit();
							this._allowEditable();
							}
                        else {
                            this._disableAllowEdit();
							this._preventEditable();
                        }
                        break;
                    }
                    case "allowKeyboardNavigation": {
                        if (options[key])
                            this._on(this.element,'keydown', this._KeyPress);
                        else
                            this._off(this.element,'keydown', this._KeyPress);
                        break;
                    }
                    case "allowDragAndDrop":
                        {
                            if (options[key])
                                this._enableDragDrop();
                            break;
                        }
                    case "enableRTL":
                        {
                            if (options[key])
                                this.element.addClass("e-rtl");
                            else
                                this.element.removeClass("e-rtl");
                            break;
                        }
                    case "height":
                        {
                            this.element.height(options[key]);
                            break;
                        }
                    case "width":
                        {
                            this.element.width(options[key]);
                            break;
                        }
                }
            }
        },

        // all events bound using this._on will be unbind automatically
        _destroy: function () {
            this.element.html("");
            this.element.parent().append(this._cloneElement);
            this.element.remove();
            this.element = this._cloneElement;
            this.element.removeClass("e-treeview e-widget");
        },

        // constructor function
        _init: function () {

            this._dataSource = [];
            this._fragment = [];
            this._templateType = "";
            this._id = this.element.prop("id");
            if (!ej.isNullOrUndefined(this.model.fields) && this.model.fields["dataSource"] != null)
                this._checkDataBinding();
            else
                this.initialize();
	
        },

        initialize: function () {
            this._cloneElement = $(this.element).clone();
            this._cutCopyNodeText = null;
            this.beforeEditText = null;
			this._CurrenctSelectedNodes=[];
            if (this.element.is("ul")) {
                this._createWrapper();
            }
            else {
                this.element.addClass("e-treeview-wrap e-widget");
                if (this.model.width != null) {
                    this.element.width(this.model.width);
                }
                if (this.model.height != null) {
                    this.element.height(this.model.height);
                }
                if (this.model.enableRTL)
                    this.element.addClass("e-rtl");
				if (this.model.cssClass != null)
                    this.element.addClass(this.model.cssClass);
            }
            if (this.model.fields["dataSource"] == null) {
                this._beforeBaseClass();
                this._addBaseClass();
                this._controlRender();
            }
            this._addDragableClass();
            this._finalize();
            this._enabledAction(this.model.enabled);
        },

        _beforeBaseClass: function () {
            this.element.attr("role", "tree").attr("aria-activedescendant", this._id + "_active");
            this._ulList = $("ul", this.element);
            this._liList = $("li", this.element).attr("tabindex", -1);
        },

        _createWrapper: function () {
            var mainWidget = ej.buildTag("div.e-treeview-wrap e-widget " + this.model.cssClass, "", "", { tabindex: 0 });
            !ej.isNullOrUndefined(this.element.attr("accesskey")) && mainWidget.attr("accesskey", this.element.attr("accesskey"));
            if (this.model.width != null) {
                mainWidget.width(this.model.width);
            }
            if (this.model.height != null) {
                mainWidget.height(this.model.height);
            }
            if (this.model.enableRTL)
                mainWidget.addClass("e-rtl");
            mainWidget.insertAfter(this.element);
            mainWidget.append(this.element.addClass("e-ul"));
            this.element = mainWidget;
        },
        //Skin Change at run time
        _changeSkin: function (skin) {
            if (this.model.cssClass != skin) {
                this.element.removeClass(this.model.cssClass).addClass(skin);
            }
        },
        //Enable avtion chnaged.
        _enabledAction: function (flag) {
            if (flag) {
                this.element.removeClass("e-disable");
                this._wireEvents();
            }
            else{
                this.element.addClass("e-disable");
                this._unWireEvents();
            }
        },

        // data source checking
        _checkDataBinding: function () {
            if (this.model.fields["dataSource"] instanceof ej.DataManager) {
                this._initDataSource();
            } else {
                this._ensureDataSource(this.model.fields["dataSource"]);
                this.initialize();
            }
        },

        _initDataSource: function () {
            this.element.ejWaitingPopup();
            var waitingPopup = this.element.ejWaitingPopup("instance");
            this.element.ejWaitingPopup("refresh");
            this.element.ejWaitingPopup("show");
            var query = this._columnToSelect(this.model.fields);
            var proxy = this;
            var queryPromise = this.model.fields["dataSource"].executeQuery(query);
            queryPromise.done(function (e) {
                proxy.element.ejWaitingPopup("hide");
                proxy.retriveData = e.result;
                proxy._ensureDataSource(e.result);
                proxy.initialize();
            });
        },

        //query check and create select based on the mapperField.

        _columnToSelect: function (mapper) {
            var column = [], queryManager = ej.Query();
            if (ej.isNullOrUndefined(mapper.query)) {
                for (var col in mapper) {
                    if (col !== "tableName" && col !== "child" && col !== "dataSource" && mapper[col])
                        column.push(mapper[col]);
                }
                if (column.length > 0)
                    queryManager.select(column);
                if (!this.model.fields["dataSource"].dataSource.url.match(mapper.tableName + "$"))
                    !ej.isNullOrUndefined(mapper.tableName) && queryManager.from(mapper.tableName);
            }
            else
                queryManager = mapper.query;
            return queryManager;
        },

        // data source checking
        _ensureDataSource: function (dataSource) {
            this.currentSelectedData = dataSource;
            this._renderTemplate(dataSource);
            this.element.html(this._fragment);
            this._beforeBaseClass();
        },

        //Render Treeview from template.

        _getTemplateType: function (dataSource, mapper) {
            for (var i = 0, objlen = dataSource.length; i < objlen; i++) {
                if (dataSource[i].hasOwnProperty(mapper.parentId) || dataSource[i].hasOwnProperty(mapper.hasChild))
                    return 1;
                if (dataSource[i].hasOwnProperty('child'))
                    return 2;
            }
            return 1;
        },

        _groupingObjects: function( array , f ) {
            var childs = {}, keys = [];
            for( var i=0, objlen=array.length; i<objlen; i++){
              var child = JSON.stringify( f(array[i]) );
              childs[child] = childs[child] || [];
              childs[child].push( array[i] );  
            }
            for (var i in childs)
              keys.push(childs[i]);
            return keys;
        },

        _renderTemplate: function (item) {
            this._onlineData = false;
            var proxy = this;
            if (this.model.fields["dataSource"] instanceof ej.DataManager) {
                this._dataSource = item;
            }
            else {
                //Determine the Type of Template
                this._templateType = this._getTemplateType(item, this.model.fields);
                switch (this._templateType) {
                    case 1:
                        this._dataSource = this._groupingObjects(item, function(item){ return [!ej.isNullOrUndefined(item) && [item[proxy.model.fields.parentId]] ]; });
                        break;
                    case 2:
                        this._dataSource = item; // Direct Nested child Data
                        break;
                }
            }
            this._templateNodeCreation(item, this.model.fields);
        },

        _templateNodeCreation: function (dataItems, mapper) {
            this._fragment = document.createDocumentFragment();
            var ulTag = document.createElement('ul');
            this._fragment.appendChild(ulTag);
            if (this._onlineData) {
                this._fragment.firstChild.className = "e-treeview-ul";
                this._fragment.firstChild.setAttribute('role', "group");
                $(this._fragment.firstChild).attr('style', 'display:none');
            }
            if (this.model.fields["dataSource"] != null) {
                for (var i = 0, objlen = dataItems.length; i < objlen; i++) {
                    if (ej.isNullOrUndefined(dataItems[i][this.model.fields.parentId]) || dataItems[i][this.model.fields.parentId] == 0 || this._loadOnDemandNodes ) 
                        this._fragment.firstChild.appendChild(this._tempalteSub(dataItems[i], dataItems, mapper));
                }
                $(this._fragment.firstChild).find('>li:first').addClass('first');
                $(this._fragment.firstChild).find('>li:last').addClass('last');
            }
        },

        _onlineDataSource: function (childItems, id, mapper) {
            this._templateNodeCreation(childItems, mapper);
            var element = this.element.find('li#' + id);
            element.append(this._fragment);
            element.children().find('> div:first').addClass("e-icon e-plus");
            this.model.allowEditing && this._enableAllowEdit();
            this._onlineData = false;
        },

        _tempalteSub: function (item, dataSource, mapper) {
            var liTag, spanTag, imgTag, aTag, ulTag, childItems, loadFlag = false;//, mapper = this.model.mapperField;
            var liFrag = document.createDocumentFragment();

                liTag = document.createElement('li');
                liFrag.appendChild(liTag);
                var currentLiTag = liFrag.firstChild;
                if (item[mapper.id])
                    currentLiTag.id = item[mapper.id];

                if (this._addFirst) {
                    $(currentLiTag).addClass("e-item first");
                    this._addFirst = false;
                }
                else if (this._addLast) {
                    $(currentLiTag).addClass("e-item last");
                    this._addLast = false;
                }
                else
                    $(currentLiTag).addClass("e-item");
                currentLiTag.setAttribute('role', "treeitem");
                currentLiTag.setAttribute('tabindex', -1);

                if (item[mapper.htmlAttribute]) this._setAttributes(item[mapper.htmlAttribute], currentLiTag);
                if (item[mapper.imageUrl]) {
                    imgTag = document.createElement('img');
                    imgTag.className = "e-align";
                    imgTag.src = item[mapper.imageUrl];
                    if (item[mapper.imageAttribute]) this._setAttributes(item[mapper.imageAttribute], imgTag);
                    currentLiTag.appendChild(imgTag);
                }
                else if (item[mapper.spriteCssClass]) {
                    spanTag = document.createElement('span');
                    spanTag.className = item[mapper.spriteCssClass];
                    currentLiTag.appendChild(spanTag);
                }
                if (this.model.template || item[mapper.text]) {
                    aTag = document.createElement('a');
                    if (this.model.template) {
                        if (typeof $.fn.render != "function") throw "Error : JsRender dependecy script missing";
                        aTag.innerHTML = $(this.model.template).render(item);
                    }
                    else aTag.innerHTML = item[mapper.text];
                    aTag.href = "#";
                    if (item[mapper.linkAttribute])
                        aTag.href = item[mapper.linkAttribute];
                    if (item[mapper.selected])
                        aTag.className = "e-text CanSelect e-active";
                    currentLiTag.appendChild(aTag);
                }

            this._hasExpanded = false;
            if (item[mapper.expanded]) {
                $(currentLiTag).addClass("expanded");
                this._hasExpanded = true;
            }
            if (item[mapper.isChecked])
                $(currentLiTag).addClass("ischecked");
            
            if (loadFlag)
                currentLiTag.appendChild(ulTag);

            if (!ej.isNullOrUndefined(mapper["child"])) {
                if (mapper["child"]["dataSource"] instanceof ej.DataManager) {
                    this._updateElement(currentLiTag, true);
                    currentLiTag.setAttribute('aria-selected', false);
                    currentLiTag.setAttribute('aria-expanded', false);
                    if (this.model.showCheckbox)
                        this._checkboxOnTemplate(currentLiTag.children[0]);
                    if (!this.model.loadOnDemand) {
                        var proxy = this, queryManager = ej.Query();
                        queryManager = proxy._columnToSelect(mapper["child"]);
                        queryManager.where(mapper["child"]["parentId"], ej.FilterOperators.equal, item[mapper.id]);
                        queryPromise = mapper["child"]["dataSource"].executeQuery(queryManager);
                        queryPromise.done(function (e) {

                            childItems = e.result;

                            if (childItems && childItems.length > 0) {
                                proxy._onlineData = true;
                                proxy._onlineDataSource(childItems, childItems[0][mapper.id], mapper.child);
                            }
                        });
                    }
                }
            }
            else if (this.model.fields["dataSource"] instanceof ej.DataManager) {
                this._updateElement(currentLiTag, false);
                if (this.model.showCheckbox)
                    this._checkboxOnTemplate(currentLiTag.children[0]);
            }
            else if (this.model.loadOnDemand) {
                this._updateElement(currentLiTag, true);
                currentLiTag.setAttribute('aria-selected', false);
                currentLiTag.setAttribute('aria-expanded', false);
                if (item[mapper.hasChild] || item.hasOwnProperty('child'))
                    currentLiTag.children[0].firstChild.className = "e-icon e-plus";
                if (this.model.showCheckbox) {
                    this._checkboxOnTemplate(currentLiTag.children[0]);
                    if (this._checkboxChecked && this._currentDataSource)
                        $(currentLiTag).addClass("ischecked");
                }
                if(item[mapper.expanded])
                    this._childNodeCreation(item, currentLiTag, dataSource,mapper) ;
            }
            else if (!this._onlineData) {
                    this._updateElement(currentLiTag, true);
                    if (this.model.showCheckbox)
                        this._checkboxOnTemplate(currentLiTag.children[0]);
                    this._childNodeCreation(item, currentLiTag, dataSource,mapper) ;
            }
            return currentLiTag;
        },

        _childNodeCreation: function(item, currentLiTag, dataSource,mapper) {
            var childItems = (this._templateType == 2) ? (!ej.isNullOrUndefined(item.child) && item.child.length > 0) && item.child : this._getChildNodes(this._dataSource, item);
            if (!ej.isNullOrUndefined(childItems) && childItems.length > 0) {
                currentLiTag.setAttribute('aria-selected', false);
                currentLiTag.children[0].firstChild.className = "e-icon e-plus";
                currentLiTag.setAttribute('aria-expanded', false);
                var tempFrag = document.createDocumentFragment();
                var ul = document.createElement('ul');
                tempFrag.appendChild(ul);
                tempFrag.firstChild.className = "e-treeview-ul";
                tempFrag.firstChild.setAttribute('role', "group");
                $(tempFrag.firstChild).attr('style', 'display:none');
                for (var i = 0, objlen = childItems.length; i < objlen; i++) {
                    if (i == 0) this._addFirst = true;
                    if (i == objlen - 1) this._addLast = true;
                    tempFrag.firstChild.appendChild(this._tempalteSub(childItems[i], dataSource, mapper));
                }
                currentLiTag.appendChild(ul);
            }
        },

        _checkboxOnTemplate: function (element) {
            var checkbox = ej.buildTag("input.nodecheckbox", "", {}, { type: "checkbox", name: "TreeView_Checkbox", value: "False" });
            //Create a hidden Checkbox
            var checkboxhdn = ej.buildTag("input", "", {}, { type: "hidden", name: "TreeView_Checkbox" + 1, value: "False" });
            element.insertBefore(checkbox.get(0), element.lastChild);
            element.insertBefore(checkboxhdn.get(0), element.lastChild);
            $(element.querySelector('.nodecheckbox')).ejCheckBox({ cssClass: this.model.cssClass, change: this._checkedChange });

        },

        //For Template Rendering Process
        _updateElement: function (liElement, subItem) {
            if (liElement.firstChild.nodeName == 'SPAN') {
                var spanTag = liElement.firstChild;
                var linkElement = liElement.lastChild;
            }
            else
                var linkElement = liElement.lastChild;
            if (linkElement) {
                nodeText = linkElement.innerHTML;
                textElement = linkElement.cloneNode();
                $(linkElement).remove();
                if (linkElement.className)
                    textElement.className = "e-text CanSelect e-active";
                else
                    textElement.className = "e-text CanSelect";
                textElement.innerHTML = nodeText;
                customElement = liElement.cloneNode();
                liElement.innerHTML = '';
                this._on($(textElement), "mouseenter", this._mouseEnterEvent);
                this._on($(textElement), "mouseleave", this._mouseLeaveEvent);
            }
            // Element to show expand, collapse image
            var outerdiv = ej.buildTag('div', "", {}, { role: "presentation" });
            var exCollpasediv = "";
            if (subItem) {
                exCollpasediv = ej.buildTag('div', "", {}, { role: "presentation" });
                //Outter div element 
                $(outerdiv).append(exCollpasediv).append(customElement.children).append(spanTag).append(textElement);
            }
            else
                $(outerdiv).append(customElement.children).append(spanTag).append(textElement);

            liElement.appendChild(outerdiv.get(0));
        },

        _setAttributes: function (data, element) {
            for (var key in data) {
                element.attr(key, data[key]);
            }
        },
        _addDragableClass: function () {
            if (this.model.allowDragAndDrop) {
                this._anchors = this._liList.map(function () {
                    return $("a", this)[0];
                });
                this._anchors.addClass("e-draggable e-droppable");
            }
        },
        _addBaseClass: function () {
            this._ulList.addClass("e-treeview-ul").attr("role", "group");
            this._liList.addClass("e-item").attr("role", "treeitem");
            this.element.find("ul").first().removeClass("e-treeview-ul");
        },
        //Applies the autoformat
        _controlRender: function () {
            if (this.element != null) {
                var element = this.element.find("> ul");
                var licoll = element.find('li');
                var nodecount = -1;
                for (var i = 0; i < licoll.length; i++) {
                    var listEl = $(licoll[i]);
                    var textElement;
                    var customElement;
                    var nodeText;
                    //Check for sub-tree view items.Take back up and remove them
                    var subItems = listEl.children('ul')[0];
                    if (this.model.showCheckbox == true && $(subItems).length != 0) {
                        var parentli = element.find('>li');
                        parentli.each(function (liindex) {
                            if (($(subItems).parents('li.e-item').first())[0] == $(parentli[liindex])[0])
                                nodecount = nodecount + 1;
                        });
                        this._checkboxrecursive(element, subItems, nodecount);
                    }
                    if (subItems)
                        $(listEl.children('ul')[0]).remove();

                    //Check for text elements(a) and add class if any
                    var linkElement = listEl.children('a')[0];
                    if (linkElement) {
                        nodeText = $(linkElement).text();
                        textElement = $(linkElement).clone();
                        $(linkElement).remove();
                        $(textElement).addClass('e-text');
                        $(textElement).addClass('CanSelect');
                        customElement = listEl.clone();
                        listEl.html('');
                    }
                        //Otherwise build text elements(a)
                    else {
                        nodeText = this._getText(listEl);
                        customElement = listEl.clone();
                        listEl.html('');
                        textElement = ej.buildTag("a.e-text CanSelect", nodeText, "", { href: "#", alt: ""});
                    }

                    //Span to show the images for Themes
                    var span = ej.buildTag('span');
                    if (subItems) {
                        $(span).addClass('collapseimg');
                    }
                    else {
                        $(span).addClass('child');
                    }

                    // Element to show expand, collapse image
                    var exCollpasediv = ej.buildTag('div', "", {}, { role: "presentation" });

                    //Outter div element 
                    var outerdiv = ej.buildTag('div', "", {}, { role: "presentation" });
                    $(outerdiv).append(exCollpasediv).append(span).append(customElement.children()).append(textElement);
                    listEl.prepend(outerdiv);
                    //Check for sub-tree view items and append them
                    if (subItems)
                        listEl.append(subItems);

                    //Check if ID is null for the node, then set the NodeText as ID
                    if (listEl.prop('name') == undefined) {
                        listEl.prop('name', nodeText);
                    }
                }
                if (this.model.showCheckbox) {
                    this._addCheckBox(element);
                    this.element.find(".nodecheckbox").ejCheckBox({ cssClass: this.model.cssClass, change: this._checkedChange });
                }
            }

            var liCollection = element.find('li');
            liCollection.find('>ul').hide();
            var acollection = liCollection.find('a');
            acollection.focus(function () {
                $(this).blur();
            });
            //Adding the Background position Class for all first and last Parent Nodes   
            liCollection.filter('li:last-child').addClass('last');
            $(liCollection[0]).addClass('first');

            if ($(liCollection[0]).hasClass('first') && $(liCollection[0]).hasClass('last'))
                $(liCollection[0]).removeClass('first');

            $(liCollection.filter(':has(ul)')).each(function () {
                $(this).attr("aria-expanded", false).attr("aria-selected", false)
                var liHasul = $($(this).children()).filter('ul');
                if ($(liHasul).is(':hidden')) {
                    $($(this).children().find('div')[0]).removeClass('e-icon e-minus').addClass('e-icon e-plus');
                }
                else {
                    $(this.childNodes[1]).removeClass('e-icon e-plus').addClass('e-icon e-minus');
                }
            });
        },

        _getText: function (element) {
            return $(element)
                      .clone()    //clone the element
                      .children() //select all the children
                      .remove()   //remove all the children
                      .end()  //again go back to selected element
                      .text();    //get the text of element
        },

        _finalize: function () {
            //Onfocus flag.
            this._isFocus = true;
			this._isRender = false;
            //Expand the specific nodes.
			if (this.model.loadOnDemand || !(this.model.expandedNodes instanceof Array && this.model.expandedNodes.length > 0)) {
			    var expandList = this.element.find(".expanded");
			    for (var i = 0; i < expandList.length; i++) {
			        this._expandNode(expandList[i]);
			    }
			}
            //is Checked checkbox element.
			if (this.model.loadOnDemand || !(this.model.checkedNodes instanceof Array && this.model.checkedNodes.length > 0)) {
			    this.model.showCheckbox && this._isCheckedAction();
			}

			if (!this.model.loadOnDemand) {
			    //Expanded Nodes
			    for (var i = 0; i < this.model.expandedNodes.length; i++) {
			        this._expandNode(this._liList[this.model.expandedNodes[i]]);
			    }
			    //Checked Nodes
			    if (this.model.showCheckbox) {
			        for (var j = 0; j < this.model.checkedNodes.length; j++) {
			            var node = this._liList[this.model.checkedNodes[j]];
			            this._nodeCheckedAction($(node).find("span.e-chkbox-wrap").first());
			        }
			    }
			}
            this._isRender = true;
            //Allow Editing on allowEditing = true
            this.model.allowEditing && this._enableAllowEdit();

            $(this._liList[0]).find("a.e-text").attr("id", this._id + "_active");

            this.element.attr("tabindex", 0);
        },

        _isCheckedAction: function () {
            var checkedList = this.element.find(".ischecked");
            for (var chk = 0; chk < checkedList.length; chk++) {
                this._nodeCheckedAction($(checkedList[chk]).find("span.e-chkbox-wrap").first()); //e-checkbox
            }
            this.element.find(".ischecked").removeClass("ischecked");
        },
        //Add Check box
        _addCheckBox: function (element) {
            var parentli = element.find('>li');
            for (var i = 0; i < parentli.length; i++) {
                var checkbox = ej.buildTag("input.nodecheckbox", "", {}, { type: "checkbox", name: "TreeView_Checkbox" + i , value: "False" });

                //Create a hidden Checkbox
                var checkboxhdn = ej.buildTag("input", "", {}, { type: "hidden", name: "TreeView_Checkbox" + 1, value: i });

                $(parentli[i]).children('div').children('span').first().after($(checkboxhdn));
                $(parentli[i]).children('div').children('span').first().after($(checkbox));
            }
        },

        _checkboxrecursive: function (element, subItems, nodecount) {
            var proxy = this, nodeindex;
            var ulcollection = $(subItems).find('>li');

            if ($($($(subItems).parents('li.e-item')[0]).find('>input[type= hidden]')[0]).length == 0)
                nodeindex = nodecount;
            else
                nodeindex = $($(subItems).parents('li.e-item')[0]).find('>input[type=hidden]')[0].value;

            ulcollection.each(function (ulindex) {
                var nodecountvalue = nodecount;
                var nodeindexvalue = nodeindex + "." + ulindex;
                proxy._checkboxrecursive(nodeindex, this, nodecountvalue);
                var checkbox = ej.buildTag("input.nodecheckbox", "", {}, { type: "checkbox", name: "TreeView_Checkbox" + nodeindexvalue, value: "False" });
                //Create a hidden Checkbox
                var checkboxhdn = ej.buildTag("input", "", {}, { type: "hidden", name: "TreeView_Checkbox" + nodeindexvalue, value: nodeindexvalue });
                var liEle = $(ulcollection[ulindex]);
                if (liEle.find("a.e-text ").length > 0) {
                    checkbox.insertBefore(liEle.find("a.e-text ")[0]);
                    checkboxhdn.insertBefore(liEle.find("a.e-text ")[0]);
                } else {
                    liEle.prepend(checkbox).prepend(checkboxhdn);
                }
            });
        },

        _checkedChange: function (args) {
            var treeview = $(this.element).closest(".e-treeview").data("ejTreeView");
            if (args.isChecked)
                treeview._nodeCheckedAction(this.element,args)
            else
                treeview._nodeUnCheckedAction(this.element,args)
        },

        //nodeClick event handler
        _ClickEventHandler: function (event) {
            var proxy = this;
            var element = event.target;
            if (!this._isclicked) {
                this._isclicked = true;
                this._onClick({ event: event,currentElement:element });
                if (!ej.isNullOrUndefined(element) && element.tagName.toUpperCase() == 'DIV') {
                    if ($(element).hasClass('e-plus') || $(element).hasClass('e-minus')) {
                        var expandUl = null;
                        if (element.tagName.toUpperCase() == 'SPAN') {
                            expandUl = $(element.parentNode).parent().children().filter('ul');
                            element = $(element.parentNode).children().filter('div')[0];
                        }
                        else {
                            expandUl = $(element.parentNode).parent().children().filter('ul');
                        }
                        this.isExpanded = $(expandUl).children().filter('li').length > 0 ? true : false;
                        if (!this.isExpanded && this.model.loadOnDemand) {
                            $(element).addClass("e-load");
                            this._checkboxChecked = $(element).siblings().find('input[type=checkbox]').prop('checked') ? true : false;
                            var nodeid = $($(element).parents('li.e-item')[0]).attr('id');
                            var nodeText = $($($($(element).parents('li.e-item')[0]).children('div')[0]).children('a[class*=e-text]')[0]).text();
                            var args = { id: nodeid, value: nodeText, isChildLoaded: this.isExpanded};
							this._onBeforeExpand(args);
                            this._loadOnDemand(args, element);							
                        }
                        else
                            this._expandCollapseAction(element);
                        //this._nodeSelectionAction($(element).parents('li.e-item').first().find('a[class*=e-text]').first());
                    }
                }
                if (!ej.isNullOrUndefined(element) && element.tagName.toUpperCase() == 'A') {
                    $(element).hasClass('CanSelect') && !$(element).hasClass('e-active') && this._nodeSelectionAction(element);
                }
            }


            setTimeout(function () { proxy._isclicked = false; }, 300);
            if (!ej.isNullOrUndefined(element) && element.tagName.toUpperCase() == 'A') {
                ($(element).attr("href") == "#") && event.preventDefault();
				this.element.focus();
            }

        },

        _loadOnDemand: function (args, element) {
            var childItems, proxy = this;
            if (this.model.fields["dataSource"] instanceof ej.DataManager) {
                var proxy = this, queryManager = ej.Query();
                queryManager = proxy._columnToSelect(proxy.model.fields["child"]);
                queryManager.where(proxy.model.fields["child"]["parentId"], ej.FilterOperators.equal, args.id);
                queryPromise = proxy.model.fields["child"]["dataSource"].executeQuery(queryManager);
                queryPromise.done(function (e) {
                    childItems = e.result;
                    if (childItems.length > 0) {
                        proxy._onlineData = true;
                        proxy._onlineDataSource(childItems, childItems[0][proxy.model.fields.id], proxy.model.fields.child);
                    }
                });
            } else {
                proxy._currentDataSource = false;
                childItems = this._getChildNodes(this._dataSource, args);
                !ej.isNullOrUndefined(childItems) && childItems.length > 0 && setTimeout(function () {
                    proxy._currentDataSource = true;
                    proxy._createSubNodesWhenLoadOnDemand(childItems, element, proxy.model.fields);
                }, 400) ;
            }
        },

        _createSubNodesWhenLoadOnDemand: function(childItems, element, mapper) {
            this._loadOnDemandNodes = true;
            this._templateNodeCreation(childItems, mapper);
            $(this._fragment.firstChild).attr('style', 'display:none');
            this._fragment.firstChild.className = "e-treeview-ul";
            this._fragment.firstChild.setAttribute('role', "group");
            element.parentNode.parentNode.appendChild(this._fragment);
            var liElement = $($(element).parents('li.e-item')[0]);
            liElement.attr("aria-expanded", false).attr("aria-expanded", true);
            $(element).removeClass("e-load");
            this._beforeBaseClass();
            this._isRender = false;
            this._isCheckedAction();
            this._isRender = true;
            this._enableDragDrop();
            this.model.allowEditing && this._enableAllowEdit();
            this._expandCollapseAction(element);
        },

        _getChildNodes: function (obj, rootNodes) {
            var childNodes = [], targetId = (this.model.loadOnDemand) ? rootNodes.id : rootNodes[this.model.fields.id] ;
            if(this._templateType==1) {
                for( var i=0, objlen=obj.length; i<objlen; i++){
                    if(!ej.isNullOrUndefined(obj[i][0][this.model.fields.parentId]) && obj[i][0][this.model.fields.parentId] == targetId)
                        return obj[i];
                }
            }
            else {
                for (var i = 0, objlen = obj.length; i < objlen; i++) {
                    if (obj[i][this.model.fields.id] == rootNodes.id)
                        return obj[i].child;
                    if (obj[i].hasOwnProperty('child')) {
                        childNodes = this._getChildNodes(obj[i].child, rootNodes);
                        if (!ej.isNullOrUndefined(childNodes) && childNodes.length > 0)
                            break;
                    }
                }
                return childNodes;
            }
        },

        _getPath: function (element) {
            var path = $(element).prop('name');
            var ulelement = $(element).parents('li')[0];
            while (ulelement != null && ulelement.parentNode.id != this._id) {
                path = $(ulelement).prop('name') + '/' + path;
                ulelement = $(ulelement).parents('li')[0];
            }
            path = this._id + "/" + path;
            return path;
        },

        //Action on Node Selection
        _nodeSelectionAction: function (element) {
            if ($(element) == null) return;
            //var isExpanded = this._isExpanded(element);
            if ($($(element).parents('li')).hasClass('e-node-disable')) return;
            // remove previous selection
            $($(this.element).find('.e-active')).removeClass('e-active');
            $(this.element).find("#" + this._id + "_active").removeAttr("id");
            $(this.element).find('[aria-selected=true]').attr("aria-selected", false);
            $(element).addClass('e-active');
            var currentElement = $($(element).parents('li.e-item')[0]);
            currentElement.attr("aria-selected", true).find("a.e-text").attr("id", this._id + "_active");
            var nodeText = $(element).text();
            var data = { currentElement: currentElement, value: nodeText };
            this._onNodeSelect(data);

        },
        //Action on Node Selection
        _nodeUnSelectionAction: function (element) {
            if (element[0] == null) return;
            $(element).removeClass('e-active');
        },

        //Action on Node Selection
        _nodeEnableAction: function (element) {
            if (element[0] == null) return;
            element = $(element);
            this.model.showCheckbox && $($(element.parents('li.e-item')[0]).find('input')[0]).prop('disabled', false);
            $(element.parents('li.e-item')[0]).removeClass('e-node-disable');
            $(element.parents('li.e-item')[0]).find('a').removeClass('e-node-disable');
            $(element.parents('li.e-item')[0])[0].disabled = false;
        },

        //Action on Node Selection
        _nodeDisableAction: function (element) {
            if (element[0] == null) return;
            // collapse the parent node
            element = $(element);
            this._collpaseNode(element.parents('li.e-item')[0]);
            this.model.showCheckbox && $($(element.parents('li.e-item')[0]).find('input')[0]).prop('disabled', true);
            $(element.parents('li.e-item')[0]).addClass('e-node-disable');
            $(element.parents('li.e-item')[0]).find('a').addClass('e-node-disable');
            if ($(element.parents('li.e-item')[0]).find('a').hasClass('e-active'))
                $(element.parents('li.e-item')[0]).find('a').removeClass('e-active');
            $(element.parents('li.e-item')[0])[0].disabled = true;
        },
        //Expands or collapse the selected node
        _expandCollapseAction: function (element) {
            var expandUl;
            var proxy = this;
            if ($(element.parentNode).parent().hasClass('e-node-disable')) return;
            if (!ej.isNullOrUndefined(element.tagName) && element.tagName.toUpperCase() == 'SPAN') {
                expandUl = $(element.parentNode).parent().children().filter('ul');
                element = $(element.parentNode).children().filter('div')[0];
            }
            else {
                expandUl = $(element.parentNode).parent().children().filter('ul');
            }
            var elmt;
            if (element.tagName == "DIV")
                elmt = $(element.parentNode.parentNode)[0];
            else
                elmt = element;
            var liElement = $($(element).parents('li.e-item')[0]);
            if (!this._isExpanded(elmt)) {
                liElement.attr("aria-expanded", true).attr("tabindex", 0);
                var nodeText = $($($($(element).parents('li.e-item')[0]).children('div')[0]).children('a[class*=e-text]')[0]).text();
                var data = { currentElement: liElement, value: nodeText, isChildLoaded: this.isExpanded};
				this.model.loadOnDemand ? (this.isExpanded ? (this._onBeforeExpand(data) ? true : false) : false) : (this._onBeforeExpand(data) ? true : false);				
                $(expandUl).animate({ height: 'toggle' }, this.model.enableAnimation ? 350 :0, 'linear', function () {
                    proxy._onNodeExpand(data);
                    proxy._addExpandedNodes(proxy._liList.index(liElement));
                });
                $(element).removeClass('e-icon e-plus').addClass('e-icon e-minus');
                $(element.parentNode.parentNode).addClass('e-collapse');
                $($(element.parentNode).children()[1]).removeClass('collapseimg').addClass('expandimg');
            }
            else {
                liElement.attr("aria-expanded", false).attr("tabindex", -1);
                var nodeText = $($($($(element).parents('li.e-item')[0]).children('div')[0]).children('a[class*=e-text]')[0]).text();
                var data = { currentElement: $($(element).parents('li.e-item')[0]), value: nodeText };
                if (this._onBeforeCollapse(data) === true)
                    return false;
                $(expandUl).animate({ height: 'toggle' },this.model.enableAnimation ? 200:0, 'linear', function () {
                    proxy._onNodeCollapse(data);
                    proxy._removeExpandedNodes(proxy._liList.index(data.currentElement));
                });
                $(element).removeClass('e-icon e-minus').addClass('e-icon e-plus');
                if (element.parentNode != null) {
                    $(element.parentNode.parentNode).removeClass('e-collapse');
                    $($(element.parentNode).children()[1]).addClass('collapseimg').removeClass('expandimg');
                }
            }
        },

        _isExpanded: function (element) {
            if (element.tagName == 'A')
                element = $(element.parentNode.parentNode)[0];
            var isExpanded = this._isNodeExpanded(element);
            if ($($(element).children().filter("ul")) != null)
                if ($($(element).children().filter("ul").find('li')) != null)
                    isExpanded = $($(element).children().filter("ul").find('li')).is(':not(:hidden)');
                else
                    isExpanded = false;
            return isExpanded;
        },
        _isChecked: function (element) {
            var isChecked;
            if ($(element).val() === "True")
                isChecked = true;
            else
                isChecked = false;
            return isChecked;
        },

        _nodeCheckedAction: function (element,args) {
            //var checkboxClass = element.className;
            this._CurrenctSelectedNodes=[];
            var proxy = this, checkedArray=[];
            var chkEle = $($(element).find('input.nodecheckbox[type=checkbox]')[0]);
            chkEle.addClass("checked").prop('value', 'True');
            //chkEle.ejCheckBox({ "indeterminateState": false, "indeterminate": false, "check": true });
            chkEle.ejCheckBox({ enableTriState: false});
            chkEle.ejCheckBox({ checked: true});
            checkedArray.push($(chkEle).parents("li.e-item").first().find(".e-text").first().text());
            $($(element).parents('li.e-item')[0]).children('ul').find('input.nodecheckbox[type=checkbox]').prop('checked', true).addClass("checked").prop('value', 'True');
            var chkCollection = $($(element).parents('li.e-item')[0]).children('ul').find('input.nodecheckbox[type=checkbox]');
            chkCollection.each(function (index) {
                $(this).ejCheckBox({ "checked": true });
                var liElement = $(this).parents("li.e-item").first();
                proxy._addCheckNodes(proxy._liList.index(liElement));
                checkedArray.push(liElement.find(".e-text").first().text());
                // Client side checked events
            });
            var autoCheckParentNode = this.model.autoCheckParentNode;
            var parentlicollection = $($(element).parents('li.e-item')[0]).parents('li.e-item');
            parentlicollection.each(function (index) {
                var childulcollection = $(parentlicollection[index]).children('ul').find('li.e-item');
                var count = $(parentlicollection[index]).children('ul').find('li.e-item').find('input.nodecheckbox[value="True"]').length;
                var chkEle = $($(parentlicollection[index]).find('> div')).find('input.nodecheckbox[type=checkbox]');
                if (count == childulcollection.length || autoCheckParentNode) {
                    chkEle.addClass("checked").prop('value', 'True');
                    //$(chkEle).ejCheckBox({ "indeterminateState": false });
                    //$(chkEle).ejCheckBox({ "indeterminate": false, "check": true, });
                    chkEle.ejCheckBox({ enableTriState: false });
                    chkEle.ejCheckBox({checked: true });
                    var liElement = $(chkEle).parents("li.e-item").first();
                    proxy._addCheckNodes(proxy._liList.index(liElement));
                    checkedArray.push(liElement.find(".e-text").first().text());
                } else if(chkEle.length > 0){
                    //chkEle.ejCheckBox({ "indeterminate": true });
                    //chkEle.ejCheckBox({ "indeterminateState": true });
                    chkEle.ejCheckBox({ enableTriState: true});
                    chkEle.ejCheckBox({ checkState:"indeterminate" });
                    chkEle.val("False");
                }
            });
            if(args){
                args.checknode = checkedArray;
            }
            this._onChecked(chkEle,args);
            var checkedcoll = this.element.find('input.nodecheckbox[value="True"]');
            for (var i = 0; i < checkedcoll.length; i++) {
                var chkEle = $(checkedcoll[i]).parents('.e-chkbox-wrap');
                var value = $(chkEle.next()[0])[0].value;
                var nodeid = $($(checkedcoll[i]).parents('li.e-item')[0]).attr('id');
                var text = $(checkedcoll[i]).parents("li.e-item").first().find(".e-text").first().text();

                var checkboxhdnvalue = ej.buildTag("input", "", {}, { type: "hidden", name: "TreeView_Checkbox" + value + ".Value", value: nodeid });
                //Create a hidden Checkbox
                var checkboxhdntext = ej.buildTag("input", "", {}, { type: "hidden", name: "TreeView_Checkbox" + value + ".Text", value: text });
                try {
                    $(checkedcoll[i]).append($(checkboxhdnvalue));
                    $(checkedcoll[i]).append($(checkboxhdntext));
                }
                catch (err) {
                }
            }
        },
        _nodeUnCheckedAction: function (element,args) {
            var proxy = this, uncheckedArray=[];
            element = $(element);
            var chkEle = $($(element).find('input.nodecheckbox[type=checkbox]')[0]);
            chkEle.removeClass('checked').prop('value', 'False').children().remove();
            //chkEle.ejCheckBox({ "indeterminate": false, "check": false });
            chkEle.ejCheckBox({ enableTriState: false });
            chkEle.ejCheckBox({ checked: false });
            uncheckedArray.push($(chkEle).parents("li.e-item").first().find(".e-text").first().text());
            var unCheckedList = $(element.parents('li.e-item')[0]).children('ul').find('input.nodecheckbox[type=checkbox].checked');
            unCheckedList.removeClass("checked").prop('value', 'False').children().remove();
            unCheckedList.each(function (index) {
                $(this).ejCheckBox({ "checked": false });
                var liElement = $(this).parents("li.e-item").first();
                proxy._removeCheckNodes(proxy._liList.index(liElement));
                uncheckedArray.push(liElement.find(".e-text").first().text());
            });
            var autoCheckParentNode = this.model.autoCheckParentNode;
            var parentlicollection = $(element.parents('li.e-item')[0]).parents('li.e-item');
            parentlicollection.each(function (index) {
                var count = $(parentlicollection[index]).find('input.nodecheckbox[value="True"]').length;
                var chkEle = $($(parentlicollection[index]).find('> div')).find('input.nodecheckbox[type=checkbox]');
                if (count <= 0) {
                    //chkEle.ejCheckBox({ "indeterminateState": false });
                    //chkEle.ejCheckBox({ "indeterminate": false, "check": false });
                    chkEle.ejCheckBox({ enableTriState: false });
                    chkEle.ejCheckBox({ checked: false });
                    var liElement = $(chkEle).parents("li.e-item").first();
                    proxy._removeCheckNodes(proxy._liList.index(liElement));
                    uncheckedArray.push(liElement.find(".e-text").first().text());
                    //Client side unchecked event
                }
                if (count > 1 && autoCheckParentNode) {
                    chkEle.addClass("checked").prop('value', 'True');
                    chkEle.ejCheckBox({ "checked": true });
                }
                else if (count == 1 && autoCheckParentNode) {
                    chkEle.addClass("checked").prop('value', 'False');
                    chkEle.ejCheckBox({ "checked": false });
                }
                else if (count > 0) {
                    chkEle.removeClass('checked').prop('value', 'False').children().remove();
                    //chkEle.ejCheckBox({ "indeterminate": true });
                    //chkEle.ejCheckBox({ "indeterminateState": true });
                    chkEle.ejCheckBox({ enableTriState: true });
                    chkEle.ejCheckBox({checkState: "indeterminate"});
                }
            });
            if(args){
                args.unchecknode = uncheckedArray;
            }
            this._onUnChecked(chkEle,args);
        },

        //Enables Edit
        _enableAllowEdit: function () {
            var element = this.element;
            var liCollection = $(element).find('li');
            liCollection.each(function () {
                $(this).addClass('AllowEdit');
            });
        },

        //Disables Edit
        _disableAllowEdit: function () {
            var liCollection = this.element.find('li');
            liCollection.each(function () {
                $(this).removeClass('AllowEdit');
            });
        },

        _expandNode: function (value) {
            if (value != null) {
                //if ($(element).find('ul').length > 0) {
                if (this.model.loadOnDemand)
                    this._expandCollapseAction($($(value).children('div')).children()[0]);
                else {
                if ($($($(value).children('div')).children()[0]).hasClass('e-plus')) {
                    var isExpanded = $(value).children('ul').find('li').length > 0 ? true : false;
                    if (isExpanded) {
                        this._expandCollapseAction($($(value).children('div')).children()[0]);
                    }
                }
            }
            }
            return true;
        },

        //Collapses the selected node
        _collpaseNode: function (value) {
            var element = value;
            if (element != null) {
                if ($(element).find('ul').length > 0) {
                    if ($($($(element).children('div')).children()[0]).hasClass('e-minus')) {
                        this._expandCollapseAction($($(element).children('div')).children()[0]);
                    }
                }
            }
        },

        //Expands all the nodes
        _expandAll: function () {
            var proxy = this;
            var element = this.element;
            var liCollection = $(element).find("li:not([class*='e-node-disable'])");
            var collapsedNodes = liCollection.find('>ul:hidden').parents('li.e-item');
            if (liCollection.find('>ul:hidden').find("li").length > 0)
                liCollection.find('>ul:hidden').animate({ height: 'toggle' },this.model.enableAnimation ? 350 :0);
            $(collapsedNodes).each(function () {
                if (!$(this).hasClass("e-node-disable")) {
                    var liElement = $($(this).parents('li.e-item')[0]);
                    var nodeText = $($(this).find(">div >a")[0]).text();
                    var data = { currentElement: liElement, value: nodeText };
                    if (proxy._onBeforeExpand(data) === true)
                        return false;
                    if ($(this).find('li').length > 0) {
                        $(this).addClass('expand');
                        $($($(this).children('div')[0]).children()[1]).removeClass('collapseimg').addClass('expandimg');
                        $($($(this).children('div')[0]).children()[0]).removeClass('e-icon e-plus').addClass('e-icon e-minus');
                    }
                    proxy._onNodeExpand(data);
                }
            });
        },

        //Collapse all the nodes
        _collapseAll: function () {
            var liCollection = this.element.find('li');
            this._collapseNodes(liCollection);
        },
        //Check all the nodes of the checkbox
        _checkAll: function () {
            if (this.model.showCheckbox) {
				this._CurrenctSelectedNodes=[];
                var element = this.element;
                var liCollection = $(element).find("li");
                for (var i = 0; i < liCollection.length; i++) {
                    var checkboxelement = $(liCollection[i]).children().find('input.nodecheckbox')[0];
                    if ($(checkboxelement)[0].disabled)
                        $(checkboxelement).ejCheckBox("setModel", { checked: false });
                    else {
                        $(checkboxelement).ejCheckBox("setModel", { checked: true });
                        $(checkboxelement).attr("value", "True");
                        this._onChecked(checkboxelement);
                    }
                }
            }
            else
                alert("Please Enable the ShowCheckbox property");
        },
        _unCheckAll: function () {
            if (this.model.showCheckbox) {
                var element = this.element;
                var liCollection = $(element).find("li");
                for (var i = 0; i < liCollection.length; i++) {
                    var chkEle = $(liCollection[i]).find('input.nodecheckbox')[0];
                    $(chkEle).ejCheckBox("setModel", { checked: false });
                    $(chkEle).attr("value", "False");
                    this._onUnChecked(chkEle);
                }
            }
            else
                alert("Please Enable the ShowCheckbox property");
        },

        _collapseNodes: function (liCollection) {
            var proxy = this;
            var _expandedNodes = liCollection.find('>ul:not(:hidden)').parents('li.e-item');
            liCollection.find('>ul:not(:hidden)').animate({ height: 'toggle' },this.model.enableAnimation ? 350 :0);
            $(_expandedNodes).each(function () {
                var nodeText = $($(this).find(">div >a")[0]).text();
                var data = { currentElement: this, value: nodeText };
                if (proxy._onBeforeCollapse(data) == true)
                    return false;
                //var isExpanded = false;
                if ($(this).find('li').length > 0) {
                    $(this).removeClass('e-collapse');
                    $($($(this).children('div')[0]).children()[1]).removeClass('expandimg').addClass('collapseimg');
                    $($($(this).children('div')[0]).children()[0]).removeClass('e-icon e-minus').addClass('e-icon e-plus');
                }
                proxy._onNodeCollapse(data);
            });
        },
        _showCheckBox: function () {
            var element = this.element.find("> ul");
            var licoll = element.find('li');
            for (var i = 0; i < licoll.length; i++) {
                var nodecount = -1;
                var listEl = licoll[i];
                //Check for sub-tree view items.Take back up and remove them
                var subItems = $(listEl).children('ul')[0];
                if (this.model.showCheckbox == true && $(subItems).length != 0) {
                    var parentli = element.find('>li');
                    parentli.each(function (liindex) {
                        if ($($(subItems).parents('li.e-item')[0])[0].id == $(parentli[liindex])[0].id)
                            nodecount = nodecount + 1;
                    });
                    this._checkboxrecursive(element, subItems, nodecount);
                }
            }
            this._addCheckBox(element);
            element.find(".nodecheckbox").ejCheckBox({ cssClass: this.model.cssClass, change: this._checkedChange });
        },

        //Drag and Drop Events handlers

        //dragEventHandler
        _drag: function () {
            var proxy = this;
            var pre = false;
            var _clonedElement = null;
            var dragContainment = null;
            this._treeView = this.element.parent();
            if (!this.model.allowDragAndDropAcrossControl)
                dragContainment = this.element.parent();
            //else
            //    dragContainment = this.element.parent();
            $(this._treeView).find("ul li div a").not(".e-js").ejDraggable({
                dragArea: dragContainment,
                clone: true,
                dragStart: function (args) {
                    var data = { target: args.target,event:args.event };
                    if (proxy._onDragStarts(data)) {
                        args.cancel = true;
                        _clonedElement && _clonedElement.remove();
                        return false;
                    }
                },
                drag: function (args) {
				pre=false;
                    $(".e-sibling").remove();
                    var target = args.target;
                    $(_clonedElement).css({"margin-left" : "20px" , "margin-top" : args.event.clientY-20 < 0 ? "0px" : "-20px"});
                    var data = { target: target,event:args.event };
                    if (proxy._onDrag(data))
                        return false;
                    if ($(target).hasClass('e-droppable') || $(target).parent().hasClass('e-droppable')) {
                        document.body.style.cursor = '';
                        $(_clonedElement).find('span.e-dropedStatus').removeClass().addClass("e-dropedStatus e-icon e-plus");
                        $(target).addClass("allowDrop");
                    }
                    else if(!$(target).hasClass('e-sibling')) {
                        document.body.style.cursor = 'not-allowed';
                        $(_clonedElement).find('span.e-dropedStatus').removeClass().addClass("e-dropedStatus e-icon e-minus");
                        $(target).removeClass('showline-hover');
                        $(target).removeClass('noline-hover');
                    }
                    if (target.nodeName != 'A') {
                        if (target.nodeName == 'UL' && $(target).children()[0] != null)
                        {    target = $(target).children()[0]; pre=true;}
                        if (target.nodeName != 'LI')
                            target = $(target).closest('li.e-droppable')[0] || $(target).parent();
								
                        if (target.nodeName == 'LI' && $(target).hasClass("e-droppable") && proxy.model.allowDropSibling) {
                            
							var div = ej.buildTag('div.e-sibling');
							var targetY = $(".e-treeview").has(target).offset().top + $(target).children().find('a')[0].offsetTop || -1;					
                            if(!pre)
							{
							pre=(args.event.pageY > targetY )? false : true;      
							}                 
                            pre ? $(target).prepend(div) : $(target).append(div); 							
                            if($(target).parents().hasClass("e-rtl"))
                                $(_clonedElement).find('span.e-dropedStatus').removeClass().addClass("e-dropedStatus e-icon e-insertInbetween-rtl");
                            else
                                $(_clonedElement).find('span.e-dropedStatus').removeClass().addClass("e-dropedStatus e-icon e-insertInbetween");
                        }
						
                    }
                    else
                        $(".e-sibling").remove();
                },
                dragStop: function (args) {
                    if (!args.element.dropped) {
                        _clonedElement && _clonedElement.remove();
                        document.body.style.cursor = '';
                    }
                    var target = args.target;
                    if (target.className == "e-sibling") {
                        target = $(target).parent("li")[0];
                    }
                    var targetObj = proxy;
                    var position = pre ? "Before" : "After";
                    position = target.nodeName == 'A' ? "Over" : position;
                    var data = { target: target, position: position,event:args.event };
                    if (proxy._onDragStop(data))
                        return false;
                    $(args.element).removeClass("e-active");
                    if (target.nodeName == 'A' && proxy.model.allowDropChild && $(target).hasClass('e-droppable') || (target.nodeName == 'UL' && $(target).children().length == 0)) {
                        position = "Over";
                        if (($(args.element).parent().parent().has($(target)).length == 0) && ($(target).parent().parent().has($(args.element)).length == 0 || proxy._isDescendant($(target).parents("li:last").find('>ul>li'), $(args.element).parents("li:first")[0]))) {
                            proxy._dropAsChildNode(target, args.element, args.event);
                            targetObj._applyFirstLastChildClass();
                        }
                    }
                    else {
                        if (target.nodeName == 'UL')
                            target = $(target).children()[0];
                        if (target.nodeName != 'LI')
                            target = $(target).parent('.e-droppable')[0] || $(target).parent();
                        if (target.nodeName == 'LI' && proxy.model.allowDropSibling && $(target).hasClass('e-droppable')) {
                            $(".e-sibling").remove();
                            if ($(args.element).parent().parent().has($(target)).length < 1) {
                                proxy._dropAsSublingNode(target, args.element, pre, args.event);
                                targetObj._applyFirstLastChildClass();
                            }
                        }
                    }
                    $(".e-sibling").remove();
                    $(".allowDrop").removeClass("allowDrop");
                    if (!proxy.model.allowDropChild) {
                        _clonedElement && _clonedElement.remove();
                    }
                    var data = { target: target, event: args.event, dropedElement: args.element };
                    if (proxy._onDropped(data))
                        return false;
                    document.body.style.cursor = '';
                },
                helper: function (event, ui) {
                    _clonedElement = $(event.element).clone().addClass("dragClone e-dragedNode");
                    this.spanEle = ej.buildTag('span.e-icon e-plus e-dropedStatus');
                    _clonedElement.append(this.spanEle);
                    return _clonedElement.appendTo($("body"));
                }
            });
        },
        _isDescendant: function (src, target) {
            var match = true;
            $(src).each(function (i, item) {
                if (item == target) {
                    match = false;
                    return false;
                }
                else
                    match = true;
            });
            return match;
        },
        //allowDropChildEventHandler
        _childDrop: function () {
            $(this._treeView).find("ul li div a").ejDroppable({
                accept: $(this._treeView).find("ul li div a"),
                drop: function (event, ui) {
                    $(ui.helper).hide();
                }
            });
        },
        //allowDropSiblingEventHandler
        _siblingDrop: function () {
            $(this._treeView).find("ul li").ejDroppable({
                drop: function (event, ui) {
                    $(ui.helper).hide();
                }
            });
        },

        //DropAsSubling
        _dropAsSublingNode: function (target, element, pre, event) {
            var li = event.ctrlKey ? $(element).parent().parent().clone() : $(element).parent().parent();
            var parentNode = $(element).parents('li.e-item')[1];
            pre ? $(li).insertBefore(target) : $(li).insertAfter(target);
            this._modifiCss(parentNode);
            this._applyFirstLastChildClass();
        },
        //DropAsChild
        _dropAsChildNode: function (target, element, event) {
            var li = event.ctrlKey ? $(element).parent().parent().clone() : $(element).parent().parent();
            var parentNode = $(element).parents('li.e-item')[1];
            if (target.nodeName == 'UL')
                $(target).append(li);
            else
                this._appendNode(target, li);
            this._modifiCss(parentNode);
            this._applyFirstLastChildClass();
            this._expandNode($(target).parent().parent()[0]);
        },
        _appendNode: function (selectedNode, outerLi) {
            if (selectedNode.length != 0) {
                var element = $($(selectedNode).parents('li.e-item')[0]);
                if (this._showline) {
                    if ($(element.find('li')).hasClass('last'))
                        $(element.find('li')).removeClass('last');
                    $(outerLi).addClass('last');
                }
                if (element.find('ul')[0] == null) {
                    var outerUl = ej.buildTag('ul.e-treeview-ul');
                    $(outerUl).append($(outerLi));
                    if ($(element.find('div')[1]).length == 0) {
                        var divTag = document.createElement('div');
                        divTag.setAttribute('role', 'presentation');
                        divTag.className = 'e-icon e-minus';
                        $(element.find('div')).append($(divTag));
                    }
                    if (!$(element.find('div')[1]).hasClass('e-minus')) {
                        $(element.find('div')[1]).addClass('e-icon e-minus');
                    }
                    element.append($(outerUl));
                }
                else
                    $(element.find('ul')[0]).append($(outerLi));
            }
        },

        _checkBox: function (target, li) {
            if (!$(li).hasClass("nodecheckbox") && $(li).find(".nodecheckbox").length > 0) {
                $(li).find(".e-chkbox-wrap").next().remove();
                $(li).find(".e-chkbox-wrap").remove();
            }
            if ($(target).find(".e-checkbox").length > 0) {
                var element = $(".e-treeview").has(target).find('ul')[0];
                $(element).find(".e-checkbox").next().remove();
                $(element).find(".e-checkbox").remove();
                var context = this;
                var nodecount = -1;
                var licoll = $(element).find('li');
                licoll.each(function (index) {
                    var subItems = $(this).children('ul')[0];
                    if ($(subItems).length != 0) {
                        var parentli = $(element).find('>li');
                        parentli.each(function (liindex) {
                            if ($($(subItems).parents('li.e-item')[0])[0].id == $(parentli[liindex])[0].id)
                                nodecount = nodecount + 1;
                        });
                        context._childcheckbox(element, subItems, nodecount);
                    }
                });
                this._addCheckBox($(element));
                this.element.find(".nodecheckbox").ejCheckBox({ cssClass: this.model.cssClass, change: this._checkedChange });
                $(target).closest(".e-treeview").find(".nodecheckbox").ejCheckBox({ cssClass: this.model.cssClass, change: this._checkedChange });
            }
        },
        _childcheckbox: function (element, subItems, nodecount) {
            var proxy = this, nodeindex;
            var ulcollection = $(subItems).find('>li');

            if ($($($($(subItems).parents('li.e-item')[0]).children()[0]).find('>input[type= hidden]')[0]).length == 0)
                nodeindex = nodecount;
            else
                nodeindex = $($($(subItems).parents('li.e-item')[0]).children()[0]).find('>input[type= hidden]')[0].value;

            ulcollection.each(function (ulindex) {
                var nodecountvalue = nodecount;
                var nodeindexvalue = nodeindex + "." + ulindex;
                proxy._childcheckbox(nodeindex, this, nodecountvalue);

                var checkbox = ej.buildTag("input.nodecheckbox", "", {}, { type: "checkbox", name: "TreeView_Checkbox" + nodeindexvalue, value: "False" });

                //Create a hidden Checkbox
                var checkboxhdn = ej.buildTag("input", "", {}, { type: "hidden", name: "TreeView_Checkbox.Index", value: nodeindexvalue });

                $(ulcollection[ulindex]).children('div').children('span').after($(checkboxhdn));
                $(ulcollection[ulindex]).children('div').children('span').after($(checkbox));
            });
        },
        _modifiCss: function (parentNode) {
            if ($(parentNode).find('li').length == 0) {
                $(parentNode).find('ul').remove();
                $(parentNode).removeClass('e-collapse');
                $(parentNode).find('div.e-icon').removeClass('e-icon e-minus');
            }
            else {
                if (!$($(parentNode).find('li')[$(parentNode).find('li').length - 1]).hasClass('last'))
                    $($(parentNode).find('li')[$(parentNode).find('li').length - 1]).addClass('last');
            }
        },

        _applyFirstLastChildClass: function () {
            var element = this.element;
            var liCollection = element.find('li');
            liCollection.removeClass('first');
            liCollection.removeClass('last');
            liCollection.filter('li:last-child').addClass('last');
            $(liCollection[0]).addClass('first');
            if ($(liCollection[0]).hasClass('first') && $(liCollection[0]).hasClass('last'))
                $(liCollection[0]).removeClass('first');
        },

        _expandEventHandler: function (event) {
            var target = $(event.target);
            if (target.hasClass("e-icon")) return;
            if (!(event.type === "dblclick" && this.model.allowEditing)) {
                event.preventDefault();
                var selectedNode = $(target.siblings('div'));
                var liElement = target.parent().parent();

                if (selectedNode.hasClass('e-minus'))
                    this._collpaseNode(liElement);
                else {
                    if (this.model.loadOnDemand && selectedNode.find(".e-item").length == 0) {
                        this._isclicked = false;
                        this._ClickEventHandler({ target: selectedNode[0] });
                    }
                    else this._expandNode(liElement);
                }
            }
        },

        //Double Click Handler
        _inlineEdit: function (event) {
            event.preventDefault();
            if (event.target.tagName == 'A' && !$(event.target).hasClass("e-node-disable")) {
                if ($($(event.target).parents('li.e-item')[0]).hasClass('AllowEdit')) {
                    if ($(event.target).hasClass('e-text')) {
                        $(event.target).removeClass('e-active');
                        this._inlineEditAction(event.target);
                    }
                }
            }
            return false;
        },

        //Node Editing 
        _inlineEditAction: function (element) {
			
            var editTextBox = $('#Edit_Input')[0];
            if (editTextBox == null) {
                this._createEditTextBox(element);
				
            }
            else {
                editTextBox = $('#Edit_Input')[0];
                $(editTextBox.parentNode).html(editTextBox.value);
                $(editTextBox).remove();
                this._createEditTextBox(element);
				 
            }
         
            //this.set_Selected(true);
			
        },

        //Creates Edit texbox
        _createEditTextBox: function (values) {
            var argsData = { currentElement:values };
            if (this._trigger("beforeEdit", argsData))
                return false;
            var editTextBox = $('#Edit_Input')[0];
            if (editTextBox == null) {
                var textBox = ej.buildTag('Input.Input_Text#Edit_Input', "", "", { type: 'text', value: $.trim(values.innerHTML).replace(/\n\s+/g, " "), name: 'inplace_value' });
                $(textBox).width($(values).outerWidth() + 5);
                $(textBox).height($(values).outerHeight() - 2);
                this.beforeEditText = $(values).text();
                $(values).html('');
                $(values).before(textBox);
                var size = 0;
                editTextBox = $('#Edit_Input')[0];
                if (editTextBox.value.length == '')
                    size = 3;
                else
                    size = $(values).outerWidth() + 20;

                this._mousePositionAtEnd(editTextBox);
                this._currentEditableNode = values;
                var txtbox = $(editTextBox);
                //Keypress event handler for the edit textbox
                this._on(txtbox, 'keypress', this._editTextBoxKeyPress);
                //Keydown event handler for edit textbox
                this._on(txtbox, 'keydown', this, this._pressEscKey);

                // Focus out event handler for the edit textbox
                this._on(txtbox, 'blur', this._focusout);
            }
            return editTextBox;
        },

        _editTextBoxKeyPress: function (event) {
            event.target.size = event.target.value.length + 1;
        },

        _mousePositionAtEnd: function (ctl) {
            if (ctl.focus)
                ctl.focus();
            if (ctl.select)
                ctl.select();
            return true;
        },

        //Focus out event handler for Edit textbox
        _focusout: function (e) {

            var editTextBox = $('#Edit_Input')[0];
            var currentText = editTextBox.value;
            var id = $($(editTextBox).parents('li.e-item')[0]).attr('id');
            var requestArgs = {
                nodeId: id,
                oldText: this.beforeEditText,
                newText: currentText,     
            };
            var element = $(editTextBox).siblings().closest("a");
            if (this._trigger("inlineEditValidation", requestArgs) == true) {
                this._cancelAction(editTextBox);
            }
            else
                this._saveAction(editTextBox);
            this._nodeSelectionAction(element);
        },


        //Keydown event handler for edit textbox
        _pressEscKey: function (event) {
            event.cancelBubble = true;
            event.returnValue = false;
            var editTextBox = $('#Edit_Input')[0];

            if (editTextBox != null) {
                if (event.keyCode == 13) {
                    event.stopPropagation();
                    this._focusout();
                }

                if (event.keyCode == 27)
                    this._cancelAction(editTextBox);
            }
        },
        _onFocusHandler: function (event) {
            if (this.model.allowKeyboardNavigation && this._isFocus) {
			if (this.element.find(".e-active").length == 0){
                this.element.find(".e-active").removeClass("e-active");
                nextElement = this.element.find('li').first().find('a').first();
                $(nextElement).addClass('e-active');
				}
               event.preventDefault();
                this._isFocus = false;
            }
        },

        //Key down event handler
        _KeyPress: function (e) {
            var code, node;
            var proxy = this;
            this._isFocus = true;
            var element = this.element.find("> ul");
            if (e.keyCode) code = e.keyCode; // ie and mozilla/gecko
            else if (e.which) code = e.which; // ns4 and opera
            else code = e.charCode;
            if (code == 113) {
                e.preventDefault();
                this._isFocus = false;
                var currentlySelectedItem = element.find(".e-active")[0];
                if (currentlySelectedItem != null) {
                    if ($($(currentlySelectedItem).parents('li.e-item')[0]).hasClass('AllowEdit')) {
                        $(currentlySelectedItem).removeClass('e-active');
                        this._createEditTextBox(currentlySelectedItem);
                    }
                }
            }
            if (proxy.model.allowKeyboardNavigation && (proxy.element.find('#Edit_Input').length < 1)) {
                var nextElement, selectedItem;
                selectedItem = $($(element).find('.e-active').parents('li.e-item')[0]);
                var liVisible = $(element).find('li').not(':hidden');
                // Down arrow key(next node Move)
                if (code == 40 && !e.ctrlKey && !e.altKey && !e.shiftKey) {
                    if (selectedItem.length > 0) {
                        e.preventDefault();
                        for (var i = 0; i < liVisible.length; i++) {
                            if ($($(liVisible[i]).find('a')[0]).hasClass('e-active')) {
                                if (i == liVisible.length - 1)
                                    this._nodeSelectionAction($(selectedItem.find('a')[0]).addClass('e-active'));
                                else {
                                    $($(liVisible[i]).find('a')[0]).removeClass('e-active');
                                    this._nodeSelectionAction($($(liVisible[i + 1]).find('a')[0]));
                                    break;
                                }
                            }
                        }
                        proxy._KeyPressEventHandler(nextElement, proxy, code,e);
                    }
                }
                    // Up arrow key(next node Move)
                else if (code == 38 && !e.ctrlKey && !e.altKey && !e.shiftKey) {
                    if (selectedItem.length > 0) {
                        e.preventDefault();
                        for (var i = 0; i < liVisible.length; i++) {
                            if ($($(liVisible[i]).find('a')[0]).hasClass('e-active')) {
                                if (i == 0)
                                    this._nodeSelectionAction($(selectedItem.find('a')[0]).addClass('e-active'));
                                else {
                                    $($(liVisible[i]).find('a')[0]).removeClass('e-active');
                                    this._nodeSelectionAction($($(liVisible[i - 1]).find('a')[0]).addClass('e-active'));
                                    break;
                                }
                            }
                        }
                        proxy._KeyPressEventHandler(nextElement, proxy, code,e);
                    }
                }

                    //Right arrow key(Expand)
                else if (code == 39 && !e.ctrlKey && !e.altKey && !e.shiftKey) {
                    var arrowIcon = $(selectedItem).children('div').find('div');
                    if (arrowIcon.hasClass('e-plus')) {
                        e.preventDefault();
                        if (this.model.loadOnDemand && selectedItem.find(".e-item").length == 0) {
                            this._ClickEventHandler({ target: arrowIcon[0] });
                        }
                        else this._expandNode(selectedItem);
                    }
                    else {
                        if ($(selectedItem).find("div.e-icon e-plus,div.e-icon e-minus").first().length > 0)
                            this.selectNode($(selectedItem).children("ul").find("li").first()[0]);
                        else {
                            var child = $(selectedItem).parent().find("li");
                            var index = child.index(selectedItem);
                            !ej.isNullOrUndefined(child[index + 1]) && $(selectedItem).children("ul").length > 0 && this.selectNode(child[index + 1]);
                        }
                    }
                    nextElement = $(selectedItem.find('a')[0]);
                    proxy._KeyPressEventHandler(nextElement, proxy, code,e);
                }

                    //Left arrow key(Collapse)
                else if (code == 37 && !e.ctrlKey && !e.altKey && !e.shiftKey) {
                    var arrowIcon = $(selectedItem).children('div').find('div');
                    if (arrowIcon.hasClass('e-minus')) {
                        e.preventDefault();
                        this._collpaseNode(selectedItem);
                    }
                    else if ($(selectedItem).parents().first().parents("li").first().length > 0) {
                        $(selectedItem).find("a.e-active").first().removeClass('e-active');
                        $(selectedItem).parents().first().parents("li").first().find('a').first().addClass("e-active");

                    }
                    nextElement = $(selectedItem.find('a')[0]);
                    proxy._KeyPressEventHandler(nextElement, proxy, code,e);
                }

                    //Enter Key(Expand and Collapse)
                else if (code == 13) {
                    if (selectedItem.length > 0) {
                        var arrowIcon = $(selectedItem).children('div').find('div');
                        if (arrowIcon.hasClass('e-minus'))
                            this._collpaseNode(selectedItem);
                        else {
                            if (this.model.loadOnDemand && selectedItem.find(".e-item").length == 0) {
                                this._ClickEventHandler({ target: arrowIcon[0] });
                            }
                            else this._expandNode(selectedItem);
                        }
                        nextElement = $(selectedItem.find('a')[0]);
                        proxy._KeyPressEventHandler(nextElement, proxy, code,e);
                    }
                }
                    //Space Key(toCheck and Uncheck node)
                else if (code == 32) {
                    if (selectedItem.length > 0 && this.model.showCheckbox) {
                        e.preventDefault();
                        var chkBoxEle = selectedItem.find('input.nodecheckbox[type=checkbox]').first();
                        if (chkBoxEle[0].checked)
                            this._nodeUnCheckedAction(chkBoxEle.parent().parent());
                        else
                            this._nodeCheckedAction(chkBoxEle.parent().parent());
                    }
                }
                    //Del key to delete the e-active node
                else if (code == 46) {
                    if (selectedItem.length > 0) {
                        e.preventDefault();
                        this._removeNode(selectedItem.find("a.e-text").first());
                    }
                }
                else if (e && e.ctrlKey == true) {
                    //Cut the node
                    if (code == 88) {
                        var cutNodeLi;
                        if ($(selectedItem).hasClass('first')) {
                            cutNodeLi = $(selectedItem).removeClass('first').addClass('last');
                            $(selectedItem).next().addClass('first');
                        }
                        else if (!$(selectedItem).hasClass('last'))
                            cutNodeLi = $(selectedItem).addClass('last');
                        else {
                            cutNodeLi = $(selectedItem);
                            $(selectedItem).prev().addClass('last');
                        }
                        this._cutCopyNodeText = cutNodeLi;
                        proxy._removeNode();
                        this.element.focus();
                    }

                        //Copy the node
                    else if (code == 67) {
                        var copyNodeLi = $(selectedItem).clone();
                        this._cutCopyNodeText = copyNodeLi;
                    }
                        // Paste the node
                    else if (code == 86 && this._cutCopyNodeText != null) {
                        $(selectedItem).length === 0 && (selectedItem = this.element);
                        var addChildNode;
                        var childNode = $(this._cutCopyNodeText).clone();
                        if ($(childNode.find('a')[0]).hasClass('e-active'))
                            $(childNode.find('a')[0]).removeClass('e-active');
                        if ($(childNode).hasClass('first'))
                            addChildNode = $(childNode).removeClass('first').addClass('last');
                        else if (!$(childNode).hasClass('last'))
                            addChildNode = $(childNode).addClass('last');
                        else
                            addChildNode = $(childNode);

                        if (this.model.showCheckbox) {
                            var checkbox = ej.buildTag("input.nodecheckbox", "", {}, { type: "checkbox", name: "TreeView_Checkbox" });
                            checkbox.insertAfter(childNode.find(".e-chkbox-wrap"));
                            childNode.find(".e-chkbox-wrap").remove();
                            childNode.find(".nodecheckbox").ejCheckBox({ cssClass: this.model.cssClass, change: this._checkedChange });
                        }
                        // wire mouse over event 
                        this._on(childNode.find(".e-text"), "mouseenter", this._mouseEnterEvent);
                        this._on(childNode.find(".e-text"), "mouseleave", this._mouseLeaveEvent);

                        if ($(selectedItem).children('div').children('span').hasClass('child'))
                            $(selectedItem).children('div').children('span').removeClass('child').addClass('expandimg');
                        if ($(selectedItem).children('ul').length != 0) {
                            $(selectedItem).children('ul').find('>li').last().removeClass('last');
                            $(selectedItem).find('>ul').append($(addChildNode));
                        }
                        else {
                            var wrapUl = ej.buildTag('ul.e-treeview-ul');
                            node = $(wrapUl).append($(addChildNode));
                            $(selectedItem).append($(node));
                            $(selectedItem).children('div').find('>div').first().addClass('e-icon e-minus');
                        }
                        var liCollection = $(selectedItem).parents('li.e-item');
                        liCollection.each(function (index) {
                            var licoll = $(liCollection[index]).children('ul').find('>li');
                            if (licoll.length == 1) {
                                $(licoll).addClass('last');
                            }
                        });
                        this._expandNode(selectedItem);
                    }
                }
                    //Home Key(Select the first node)
                else if (code == 36 && !e.ctrlKey && !e.altKey && !e.shiftKey) {

                    $($(selectedItem).find('a')[0]).removeClass('e-active');
                    nextElement = $($(element).find('>li')[0]).find('a')[0];
                    $(nextElement).addClass('e-active');
                    proxy._KeyPressEventHandler(nextElement, proxy, code,e);
                    e.preventDefault();
                }

                    //End Key(Select the last node)
                else if (code == 35 && !e.ctrlKey && !e.altKey && !e.shiftKey) {

                    $($(selectedItem).find('a')[0]).removeClass('e-active');
                    node = $(element).find('>li').last();
                    if ($(node).find('div').hasClass('e-minus')) {
                        nextElement = $($(node).find('li').last().find('a')[0]);
                        $(nextElement).addClass('e-active');
                    }
                    else {
                        nextElement = $(node.find('a')[0]);
                        $(nextElement).addClass('e-active');
                    }
                    proxy._KeyPressEventHandler(nextElement, proxy, code,e);
                    e.preventDefault();
                }

            }
        },
        //Remove the node
        _removeNode: function (node) {
            var selectedNode = ej.isNullOrUndefined(node) ? this.element.find('.e-active') : node;
            if (selectedNode != null) {
                var parentNode = $($(selectedNode).parents('li.e-item')[1]);
                var currentNode = $($(selectedNode).parents('li.e-item')[0]);
                var liVisible = $(parentNode).find('li').not(':hidden');
                if(liVisible.length > 0){
                    if(currentNode.next().length>0)
                        $(currentNode.next().find("a")[0]).addClass('e-active');
                    else
                        (currentNode.prev().length > 0) ? $(currentNode.prev().find("a")[0]).addClass('e-active') : $(parentNode.find("a")[0]).addClass('e-active');
                }
                else
                    (currentNode.next().length > 0) ? $(currentNode.next().find("a")[0]).addClass('e-active') : $(currentNode.prev().find("a")[0]).addClass('e-active');
                currentNode.remove();
                this._modifiCss(parentNode);
            }
        },
        _KeyPressEventHandler: function (nextElement, proxy, code,event) {
            var isExpanded;
            var element = $($(nextElement).parents('li.e-item')[0]);
            var path = proxy._getPath($(nextElement).parents('li.e-item')[0]);
            var nodeText = $(nextElement).text();
            if ($($(nextElement).parents('li.e-item')[0]).children('div').find('div').hasClass('e-minus'))
                isExpanded = true;
            else
                isExpanded = false;
            var argsData = { keyCode: code, currentElement: element, value: nodeText, isExpanded: isExpanded, path: path, event:event };
            this._trigger("keyPress", argsData);
        },

        //Document click event handler
        _documentClick: function (event) {
            if (event.target.id != 'Edit_Input')
                var editTextBox = $('#Edit_Input')[0];
            if (editTextBox != null) {
                $(editTextBox).next().html(editTextBox.value).removeAttr('style');
                $(editTextBox).remove();
                //this.element.focus();
            }
        },
        //Save Changes on the edited node
        _saveAction: function (values) {
            $(values).next().html(values.value).removeAttr('style');
            $(values).remove();
        },

        //Cancels the changes 
        _cancelAction: function (values) {
            $(values).next().html(this.beforeEditText).removeClass('e-node-hover').addClass('e-active').removeAttr('style');
            this._hideEditTextBox();
        },

        //Hides the edit textbox
        _hideEditTextBox: function () {
            var editTextBox = $('#Edit_Input')[0];
            if (editTextBox != null) {
                $(editTextBox).remove();
            }
        },

        _mouseEnterEvent: function (event) {
            if ($(event.currentTarget).hasClass("e-text") && !$(event.currentTarget).hasClass("e-node-disable"))
                $(event.currentTarget).addClass("e-node-hover");
        },
        _mouseLeaveEvent: function (event) {
            $(event.currentTarget).removeClass("e-node-hover");
        },

        _createTreeElement: function (nodeText, selectedNode, nodeUrl) {
            var value;
            if (this.model.showCheckbox) {
                if (!ej.isNullOrUndefined(selectedNode)) {
                    if ($(selectedNode.parents('li.e-item')[0]).children('ul').length == 0)
                        value = $($(selectedNode.parents('li.e-item')[0]).find('div')[0]).find('>input[type=hidden]')[0].value + '.' + 0;
                    else {
                        var prevlivalue = $($(selectedNode.parents('li.e-item')[0]).children('ul').find('li').last().find('div').find('>input[type=hidden]')[0]).val();
                        var splitvalue = prevlivalue.split('.');
                        var indexvalue = parseInt(splitvalue[splitvalue.length - 1]) + 1;
                        var previndexvalue = prevlivalue.substring(0, (prevlivalue.length) - 1);
                        value = previndexvalue + indexvalue;
                    }
                }
                else {
                    var list = this.element.find(">li");
                    value = list.length + 1;
                }

                var checkbox = ej.buildTag("input.nodecheckbox", "", {}, { type: "checkbox", name: "TreeView_Checkbox" + value + ".Checked", value: "False" });

                //Create a hidden Checkbox
                var checkboxhdn = ej.buildTag("input", "", {}, { type: "hidden", name: "TreeView_Checkbox.Index", value: value });

            }
            var textElement = ej.buildTag('a.e-text CanSelect', nodeText, "", { href: "#" , id: (this._id + "_active") });            
            if (!ej.isNullOrUndefined(nodeUrl))
                textElement.attr("href", nodeUrl);
            var textNode = $(textElement);
            this._on(textNode, "mouseenter", this._mouseEnterEvent);
            this._on(textNode, "mouseleave", this._mouseLeaveEvent);
            //Span to show the images for Themes
            var span = ej.buildTag('span.child');

            // Element to show expand, collapse image
            var exCollpasediv = ej.buildTag('div', "", {}, { role: "presentation" });
            //Outter div element 
            var outerdiv = ej.buildTag('div', "", {}, { role: "presentation" });
            $(outerdiv).append(exCollpasediv);
            $(outerdiv).append(span);
            if (this.model.showCheckbox) {
                $(outerdiv).append(checkbox);
                $(outerdiv).append(checkboxhdn);
            }
            $(outerdiv).append(textElement);

            //Check for sub-tree view items and append them
            var outerli = ej.buildTag('li.e-item#' + nodeText, "", { role: "treeitem", tabindex: -1 });           
            if (this.model.allowEditing)
                outerli.addClass('AllowEdit');
            outerli.append($(outerdiv));

            return $(outerli);
        },
        _addExpandedNodes: function (index) {
            var _nodes = this.model.expandedNodes;
            this._removeNullInArray(_nodes);
            if ($.inArray(index, _nodes) == -1) {
                this.model.expandedNodes.push(index);
            }
        },

        _removeExpandedNodes: function (index) {
            var _nodes = this.model.expandedNodes;
            if ($.inArray(index, _nodes) > -1) {
                this.model.expandedNodes.splice($.inArray(index, _nodes), 1);
                _nodes.length == 0 && (_nodes.push(-1));
            }
        },

        //-------------------- Event Wire-up -------------------------//
        _wireEvents: function () {
            this._on(this.element, "click", this._ClickEventHandler);
            this._on(this.element.find(".e-text"), "mouseenter", this._mouseEnterEvent);
            this._on(this.element.find(".e-text"), "mouseleave", this._mouseLeaveEvent);
			if(this.model.allowEditing) this._allowEditable();
            this._on(this.element, this.model.expandOn, this._expandEventHandler);
            if(this.model.allowKeyboardNavigation) this._on(this.element,"keydown", this._KeyPress);
            this._on(this.element, "focus", this._onFocusHandler);
            this._enableDragDrop();
        },

        //-------------------- Event UnWire-up -------------------------//
        _unWireEvents: function () {
            this._off(this.element, "click");
            this._off(this.element.find(".e-text"), "mouseenter");
            this._off(this.element.find(".e-text"), "mouseleave");
            this._preventEditable();
            this._off(this.element, this.model.expandOn);
            if(this.model.allowKeyboardNavigation)  this._off(this.element, 'keydown');
            this._on(this.element, "focus");
        },

        //Darg and Drop Events

        _enableDragDrop: function () {
            if (this.model.allowDragAndDrop) {
                this._drag();
                if (this.model.allowDropChild)
                    this._childDrop();
                if (this.model.allowDropSibling)
                    this._siblingDrop();
            }
        },
		
		//Allow Editable option  on nodes
		
		_allowEditable: function()
		{			
                this._on($(document), 'click', this._documentClick);
                this._on(this.element.parent(), 'dblclick', this._inlineEdit);            
		},
		
		// Prevent Editable option on nodes
		
		_preventEditable: function()
		{
			this._off($(document), 'click');
            this._off(this.element.parent(), 'dblclick');
		},
        //--------------------- Client side Methods ----------------------//
        /**
        * To refresh the TreeView  
        * @alias refresh
		* @return jQuery		
        * @example 
        *   &lt;ul id="treeView"&gt;
        *    &lt;li&gt;Artwork
        *        &lt;ul&gt;
        *           &lt;li&gt;Abstract
        *                &lt;ul&gt;
        *                    &lt;li&gt;2 Acrylic Mediums&lt;/li&gt;
        *                    &lt;li&gt;Creative Acrylic&lt;/li&gt;
        *                    &lt;li&gt;Modern Painting&lt;/li&gt;
        *                    &lt;li&gt;Canvas Art&lt;/li&gt;
        *                    &lt;li&gt;Black white&lt;/li&gt;
        *                &lt;/ul&gt;
        *           &lt;/li&gt;
        *        &lt;/ul&gt;
        *    &lt;/li&gt;
        *    &lt;li&gt;Books
        *        &lt;ul&gt;
        *            &lt;li&gt;Entertaining&lt;/li&gt;
        *            &lt;li&gt;Design&lt;/li&gt;
        *       &lt;/ul&gt;
        *    &lt;/li&gt;
        *    &lt;li&gt;Music
        *        &lt;ul&gt;
        *            &lt;li&gt;Mass&lt;/li&gt;
        *            &lt;li&gt;Folk&lt;/li&gt;
        *        &lt;/ul&gt;
        *    &lt;/li&gt;
        *&lt;/ul&gt; <br> 
        * &lt;script&gt;
        * $("#treeView").ejTreeView();
        * // Create TreeView
        * var treeObj = $("#treeView").data("ejTreeView");
        * treeObj.refresh(); // refresh the TreeView
        * &lt;/script&gt;
          * @example 
        *  &lt;ul id="treeView"&gt;
        *   &lt;li&gt;Artwork
        *       &lt;ul&gt;
        *           &lt;li&gt;Abstract
        *               &lt;ul&gt;
        *                   &lt;li&gt;2 Acrylic Mediums&lt;/li&gt;
        *                   &lt;li&gt;Creative Acrylic&lt;/li&gt;
        *                   &lt;li&gt;Modern Painting&lt;/li&gt;
        *                   &lt;li&gt;Canvas Art&lt;/li&gt;
        *                   &lt;li&gt;Black white&lt;/li&gt;
        *               &lt;/ul&gt;
        *           &lt;/li&gt;
        *       &lt;/ul&gt;
        *   &lt;/li&gt;
        *   &lt;li id="book"&gt;Books
        *       &lt;ul&gt;
        *           &lt;li&gt;Entertaining&lt;/li&gt;
        *           &lt;li&gt;Design&lt;/li&gt;
        *       &lt;/ul&gt;
        *   &lt;/li&gt;
        *   &lt;li&gt;Music
        *       &lt;ul&gt;
        *           &lt;li&gt;Mass&lt;/li&gt;
        *           &lt;li&gt;Folk&lt;/li&gt;
        *       &lt;/ul&gt;
        *   &lt;/li&gt;
        * &lt;/ul&gt; <br> 
        * &lt;script&gt;
         * $("#treeView").ejTreeView();
        * // Create TreeView
        * $("#treeView").ejTreeView("refresh");        
        * &lt;/script&gt;
        *@memberof ejTreeView
        * @instance
        */
        refresh: function () {
            this._unWireEvents();
            this._init();
        },
          /**
        * To expand all the TreeView nodes.
		* @alias expandAll
		* @return jQuery
        * @example 
        *   &lt;ul id="treeView"&gt;
        *    &lt;li&gt;Artwork
        *        &lt;ul&gt;
        *            &lt;li&gt;Abstract
        *                &lt;ul&gt;
        *                    &lt;li&gt;2 Acrylic Mediums&lt;/li&gt;
        *                    &lt;li&gt;Creative Acrylic&lt;/li&gt;
        *                    &lt;li&gt;Modern Painting&lt;/li&gt;
        *                    &lt;li&gt;Canvas Art&lt;/li&gt;
        *                    &lt;li&gt;Black white&lt;/li&gt;
        *                &lt;/ul&gt;
        *            &lt;/li&gt;
        *        &lt;/ul&gt;
        *    &lt;/li&gt;
        *    &lt;li&gt;Books
        *        &lt;ul&gt;
        *            &lt;li&gt;Entertaining&lt;/li&gt;
        *            &lt;li&gt;Design&lt;/li&gt;
        *        &lt;/ul&gt;
        *    &lt;/li&gt;
        *    &lt;li&gt;Music
        *        &lt;ul&gt;
        *            &lt;li&gt;Mass&lt;/li&gt;
        *            &lt;li&gt;Folk&lt;/li&gt;
        *        &lt;/ul&gt;
        *    &lt;/li&gt;
        * &lt;/ul&gt; <br> 
        * &lt;script&gt;
        * $("#treeView").ejTreeView();
        * // Create TreeView
        * var treeObj = $("#treeView").data("ejTreeView");
        * treeObj.expandAll(); // expandAll the TreeView nodes
        * &lt;/script&gt;
          * @example 
        *  &lt;ul id="treeView"&gt;
        *   &lt;li&gt;Artwork
        *       &lt;ul&gt;
        *           &lt;li&gt;Abstract
        *               &lt;ul&gt;
        *                   &lt;li&gt;2 Acrylic Mediums&lt;/li&gt;
        *                   &lt;li&gt;Creative Acrylic&lt;/li&gt;
        *                   &lt;li&gt;Modern Painting&lt;/li&gt;
        *                   &lt;li&gt;Canvas Art&lt;/li&gt;
        *                   &lt;li&gt;Black white&lt;/li&gt;
        *               &lt;/ul&gt;
        *           &lt;/li&gt;
        *       &lt;/ul&gt;
        *   &lt;/li&gt;
        *   &lt;li id="book"&gt;Books
        *       &lt;ul&gt;
        *           &lt;li&gt;Entertaining&lt;/li&gt;
        *           &lt;li&gt;Design&lt;/li&gt;
        *       &lt;/ul&gt;
        *   &lt;/li&gt;
        *   &lt;li&gt;Music
        *       &lt;ul&gt;
        *           &lt;li&gt;Mass&lt;/li&gt;
        *           &lt;li&gt;Folk&lt;/li&gt;
        *       &lt;/ul&gt;
        *   &lt;/li&gt;
        * &lt;/ul&gt; <br> 
        * &lt;script&gt;
         * $("#treeView").ejTreeView();
        * // Create TreeView
        * $("#treeView").ejTreeView("expandAll");        
        * &lt;/script&gt;
        *@memberof ejTreeView
        * @instance
        */
        expandAll: function () {
            this.model.enabled && this._expandAll();
        },
        /**
        * To collapse all the TreeView nodes.
		* @alias collapseAll
		* @return jQuery
        * @example 
        *   &lt;ul id="treeView"&gt;
        *    &lt;li&gt;Artwork
        *        &lt;ul&gt;
        *            &lt;li&gt;Abstract
        *                &lt;ul&gt;
        *                    &lt;li&gt;2 Acrylic Mediums&lt;/li&gt;
        *                    &lt;li&gt;Creative Acrylic&lt;/li&gt;
        *                    &lt;li&gt;Modern Painting&lt;/li&gt;
        *                    &lt;li&gt;Canvas Art&lt;/li&gt;
        *                    &lt;li&gt;Black white&lt;/li&gt;
        *                &lt;/ul&gt;
        *            &lt;/li&gt;
        *        &lt;/ul&gt;
        *    &lt;/li&gt;
        *    &lt;li&gt;Books
        *        &lt;ul&gt;
        *            &lt;li&gt;Entertaining&lt;/li&gt;
        *            &lt;li&gt;Design&lt;/li&gt;
        *        &lt;/ul&gt;
        *    &lt;/li&gt;
        *    &lt;li&gt;Music
        *        &lt;ul&gt;
        *            &lt;li&gt;Mass&lt;/li&gt;
        *            &lt;li&gt;Folk&lt;/li&gt;
        *        &lt;/ul&gt;
        *    &lt;/li&gt;
        * &lt;/ul&gt; <br> 
        * &lt;script&gt;
        * $("#treeView").ejTreeView();

        * // Create TreeView
        * var treeObj = $("#treeView").data("ejTreeView");
        * treeObj.collapseAll(); // collapseAll the TreeView nodes
        * &lt;/script&gt;
        * @example 
        *  &lt;ul id="treeView"&gt;
        *   &lt;li&gt;Artwork
        *       &lt;ul&gt;
        *           &lt;li&gt;Abstract
        *               &lt;ul&gt;
        *                   &lt;li&gt;2 Acrylic Mediums&lt;/li&gt;
        *                   &lt;li&gt;Creative Acrylic&lt;/li&gt;
        *                   &lt;li&gt;Modern Painting&lt;/li&gt;
        *                   &lt;li&gt;Canvas Art&lt;/li&gt;
        *                   &lt;li&gt;Black white&lt;/li&gt;
        *               &lt;/ul&gt;
        *           &lt;/li&gt;
        *       &lt;/ul&gt;
        *   &lt;/li&gt;
        *   &lt;li id="book"&gt;Books
        *       &lt;ul&gt;
        *           &lt;li&gt;Entertaining&lt;/li&gt;
        *           &lt;li&gt;Design&lt;/li&gt;
        *       &lt;/ul&gt;
        *   &lt;/li&gt;
        *   &lt;li&gt;Music
        *       &lt;ul&gt;
        *           &lt;li&gt;Mass&lt;/li&gt;
        *           &lt;li&gt;Folk&lt;/li&gt;
        *       &lt;/ul&gt;
        *   &lt;/li&gt;
        * &lt;/ul&gt; <br> 
        * &lt;script&gt;
         * $("#treeView").ejTreeView();
        * // Create TreeView
        * $("#treeView").ejTreeView("collapseAll");        
        * &lt;/script&gt;
        * @memberof ejTreeView
        * @instance
        */
        collapseAll: function () {
            this.model.enabled && this._collapseAll();
        },
        /**
        * To check all the TreeView nodes.
	    * @alias checkAll
		* @return jQuery
        * @example 
        *   &lt;ul id="treeView"&gt;
        *    &lt;li&gt;Artwork
        *        &lt;ul&gt;
        *            &lt;li&gt;Abstract
        *                &lt;ul&gt;
        *                   &lt;li&gt;2 Acrylic Mediums&lt;/li&gt;
        *                    &lt;li&gt;Creative Acrylic&lt;/li&gt;
        *                    &lt;li&gt;Modern Painting&lt;/li&gt;
        *                    &lt;li&gt;Canvas Art&lt;/li&gt;
        *                    &lt;li&gt;Black white&lt;/li&gt;
        *               &lt;/ul&gt;
        *            &lt;/li&gt;
        *        &lt;/ul&gt;
        *    &lt;/li&gt;
        *    &lt;li&gt;Books
        *        &lt;ul&gt;
        *            &lt;li&gt;Entertaining&lt;/li&gt;
        *            &lt;li&gt;Design&lt;/li&gt;
        *        &lt;/ul&gt;
        *    &lt;/li&gt;
        *    &lt;li&gt;Music
        *        &lt;ul&gt;
        *            &lt;li&gt;Mass&lt;/li&gt;
        *            &lt;li&gt;Folk&lt;/li&gt;
        *        &lt;/ul&gt;
        *    &lt;/li&gt;
        * &lt;/ul&gt; <br> 
        * &lt;script&gt;
         * $("#treeView").ejTreeView({showCheckbox:true});
        * // Create TreeView
        * var treeObj = $("#treeView").data("ejTreeView");
        * treeObj.checkAll(); // checkAll the TreeView nodes
        * &lt;/script&gt;
        * @example 
        *  &lt;ul id="treeView"&gt;
        *   &lt;li&gt;Artwork
        *       &lt;ul&gt;
        *           &lt;li&gt;Abstract
        *               &lt;ul&gt;
        *                   &lt;li&gt;2 Acrylic Mediums&lt;/li&gt;
        *                   &lt;li&gt;Creative Acrylic&lt;/li&gt;
        *                   &lt;li&gt;Modern Painting&lt;/li&gt;
        *                   &lt;li&gt;Canvas Art&lt;/li&gt;
        *                   &lt;li&gt;Black white&lt;/li&gt;
        *               &lt;/ul&gt;
        *           &lt;/li&gt;
        *       &lt;/ul&gt;
        *   &lt;/li&gt;
        *   &lt;li id="book"&gt;Books
        *       &lt;ul&gt;
        *           &lt;li&gt;Entertaining&lt;/li&gt;
        *           &lt;li&gt;Design&lt;/li&gt;
        *       &lt;/ul&gt;
        *   &lt;/li&gt;
        *   &lt;li&gt;Music
        *       &lt;ul&gt;
        *           &lt;li&gt;Mass&lt;/li&gt;
        *           &lt;li&gt;Folk&lt;/li&gt;
        *       &lt;/ul&gt;
        *   &lt;/li&gt;
        * &lt;/ul&gt; <br> 
        * &lt;script&gt;
         * $("#treeView").ejTreeView({showCheckbox:true});
        * // Create TreeView
        * $("#treeView").ejTreeView("checkAll");        
        * &lt;/script&gt;
        *@memberof ejTreeView
        * @instance
        */
        checkAll: function () {
            if (this.model.showCheckbox && this.model.enabled) {
                this._checkAll();
            }
        },
        /**
        * To uncheck all the TreeView nodes.
		* @alias unCheckAll
		* @return jQuery
        * @example 
        *   &lt;ul id="treeView"&gt;
        *    &lt;li&gt;Artwork
        *        &lt;ul&gt;
        *            &lt;li&gt;Abstract
        *                &lt;ul&gt;
        *                    &lt;li&gt;2 Acrylic Mediums&lt;/li&gt;
        *                    &lt;li&gt;Creative Acrylic&lt;/li&gt;
        *                    &lt;li&gt;Modern Painting&lt;/li&gt;
        *                    &lt;li&gt;Canvas Art&lt;/li&gt;
        *                    &lt;li&gt;Black white&lt;/li&gt;
        *                &lt;/ul&gt;
        *            &lt;/li&gt;
        *        &lt;/ul&gt;
        *    &lt;/li&gt;
        *    &lt;li&gt;Books
        *        &lt;ul&gt;
        *            &lt;li&gt;Entertaining&lt;/li&gt;
        *            &lt;li&gt;Design&lt;/li&gt;
        *       &lt;/ul&gt;
        *    &lt;/li&gt;
        *    &lt;li&gt;Music
        *        &lt;ul&gt;
        *            &lt;li&gt;Mass&lt;/li&gt;
        *            &lt;li&gt;Folk&lt;/li&gt;
        *        &lt;/ul&gt;
        *    &lt;/li&gt;
        * &lt;/ul&gt; <br> 
        * &lt;script&gt;
         * $("#treeView").ejTreeView({showCheckbox:true});
        * // Create TreeView
        * var treeObj = $("#treeView").data("ejTreeView");
        * treeObj.unCheckAll(); // uncheckAll the TreeView nodes
        * &lt;/script&gt;
          * @example 
        *  &lt;ul id="treeView"&gt;
        *   &lt;li&gt;Artwork
        *       &lt;ul&gt;
        *           &lt;li&gt;Abstract
        *               &lt;ul&gt;
        *                   &lt;li&gt;2 Acrylic Mediums&lt;/li&gt;
        *                   &lt;li&gt;Creative Acrylic&lt;/li&gt;
        *                   &lt;li&gt;Modern Painting&lt;/li&gt;
        *                   &lt;li&gt;Canvas Art&lt;/li&gt;
        *                   &lt;li&gt;Black white&lt;/li&gt;
        *               &lt;/ul&gt;
        *           &lt;/li&gt;
        *       &lt;/ul&gt;
        *   &lt;/li&gt;
        *   &lt;li id="book"&gt;Books
        *       &lt;ul&gt;
        *           &lt;li&gt;Entertaining&lt;/li&gt;
        *           &lt;li&gt;Design&lt;/li&gt;
        *       &lt;/ul&gt;
        *   &lt;/li&gt;
        *   &lt;li&gt;Music
        *       &lt;ul&gt;
        *           &lt;li&gt;Mass&lt;/li&gt;
        *           &lt;li&gt;Folk&lt;/li&gt;
        *       &lt;/ul&gt;
        *   &lt;/li&gt;
        * &lt;/ul&gt; <br> 
        * &lt;script&gt;
         * $("#treeView").ejTreeView({showCheckbox:true});
        * // Create TreeView
        * $("#treeView").ejTreeView("unCheckAll");        
        * &lt;/script&gt;
        *@memberof ejTreeView
        * @instance
        */
        unCheckAll: function () {
            if (this.model.showCheckbox && this.model.enabled) {
                this._unCheckAll();
            }
        },
           /**
        * To get the selected nodes in the TreeView.
		* @alias getSelectedNode
		* @return jQuery
        * @example 
        *   &lt;ul id="treeView"&gt;
        *    &lt;li&gt;Artwork
        *        &lt;ul&gt;
        *            &lt;li&gt;Abstract
        *                &lt;ul&gt;
        *                    &lt;li&gt;2 Acrylic Mediums&lt;/li&gt;
        *                    &lt;li&gt;Creative Acrylic&lt;/li&gt;
        *                    &lt;li&gt;Modern Painting&lt;/li&gt;
        *                    &lt;li&gt;Canvas Art&lt;/li&gt;
        *                    &lt;li&gt;Black white&lt;/li&gt;
        *                &lt;/ul&gt;
        *            &lt;/li&gt;
        *        &lt;/ul&gt;
        *    &lt;/li&gt;
        *    &lt;li&gt;Books
        *        &lt;ul&gt;
        *            &lt;li&gt;Entertaining&lt;/li&gt;
        *            &lt;li&gt;Design&lt;/li&gt;
        *        &lt;/ul&gt;
        *    &lt;/li&gt;
        *    &lt;li&gt;Music
        *        &lt;ul&gt;
        *            &lt;li&gt;Mass&lt;/li&gt;
        *            &lt;li&gt;Folk&lt;/li&gt;
        *        &lt;/ul&gt;
        *    &lt;/li&gt;
        * &lt;/ul&gt; <br> 
        * &lt;script&gt;
         * $("#treeView").ejTreeView();
        * // Create TreeView
        * var treeObj = $("#treeView").data("ejTreeView");
        * treeObj.getSelectedNode(); // getSelectedNode in  TreeView nodes
        * &lt;/script&gt;
          * @example 
        *  &lt;ul id="treeView"&gt;
        *   &lt;li&gt;Artwork
        *       &lt;ul&gt;
        *           &lt;li&gt;Abstract
        *               &lt;ul&gt;
        *                   &lt;li&gt;2 Acrylic Mediums&lt;/li&gt;
        *                   &lt;li&gt;Creative Acrylic&lt;/li&gt;
        *                   &lt;li&gt;Modern Painting&lt;/li&gt;
        *                   &lt;li&gt;Canvas Art&lt;/li&gt;
        *                   &lt;li&gt;Black white&lt;/li&gt;
        *               &lt;/ul&gt;
        *           &lt;/li&gt;
        *       &lt;/ul&gt;
        *   &lt;/li&gt;
        *   &lt;li id="book"&gt;Books
        *       &lt;ul&gt;
        *           &lt;li&gt;Entertaining&lt;/li&gt;
        *           &lt;li&gt;Design&lt;/li&gt;
        *       &lt;/ul&gt;
        *   &lt;/li&gt;
        *   &lt;li&gt;Music
        *       &lt;ul&gt;
        *           &lt;li&gt;Mass&lt;/li&gt;
        *           &lt;li&gt;Folk&lt;/li&gt;
        *       &lt;/ul&gt;
        *   &lt;/li&gt;
        * &lt;/ul&gt; <br> 
        * &lt;script&gt;
         * $("#treeView").ejTreeView();
        * // Create TreeView
        * $("#treeView").ejTreeView("getSelectedNode");        
        * &lt;/script&gt;
        *@memberof ejTreeView
        * @instance
        */
        //Get Selected node
        getSelectedNode: function () {
            var selectedNodeElement = this.element.find('.e-active');
            var selectedNode = $($(selectedNodeElement).parents('li.e-item')[0]);
            return selectedNode;
        },
       /**
        * To get the text of the nodes in the TreeView.
		* @alias getText
		* @return string
		* @param {object} node  object of treeview node.
        * @example 
        *   &lt;ul id="treeView"&gt;
        *   &lt;li&gt;Artwork
        *        &lt;ul&gt;
        *            &lt;li&gt;Abstract
        *                &lt;ul&gt;
        *                    &lt;li&gt;2 Acrylic Mediums&lt;/li&gt;
        *                    &lt;li&gt;Creative Acrylic&lt;/li&gt;
        *                    &lt;li&gt;Modern Painting&lt;/li&gt;
        *                    &lt;li&gt;Canvas Art&lt;/li&gt;
        *                    &lt;li&gt;Black white&lt;/li&gt;
        *                &lt;/ul&gt;
        *            &lt;/li&gt;
        *        &lt;/ul&gt;
        *    &lt;/li&gt;
        *    &lt;li id="book"&gt;Books
        *        &lt;ul&gt;
        *            &lt;li&gt;Entertaining&lt;/li&gt;
        *            &lt;li&gt;Design&lt;/li&gt;
        *        &lt;/ul&gt;
        *    &lt;/li&gt;
        *    &lt;li&gt;Music
        *        &lt;ul&gt;
        *            &lt;li&gt;Mass&lt;/li&gt;
        *            &lt;li&gt;Folk&lt;/li&gt;
        *        &lt;/ul&gt;
        *    &lt;/li&gt;
        * &lt;/ul&gt; <br> 
        * &lt;script&gt;
        * $("#treeView").ejTreeView();
        * // Create TreeView
        * var treeObj = $("#treeView").data("ejTreeView");
        * treeObj.getText($("#book")); // getText in  TreeView nodes
        * &lt;/script&gt;
          * @example 
        *  &lt;ul id="treeView"&gt;
        *   &lt;li&gt;Artwork
        *       &lt;ul&gt;
        *           &lt;li&gt;Abstract
        *               &lt;ul&gt;
        *                   &lt;li&gt;2 Acrylic Mediums&lt;/li&gt;
        *                   &lt;li&gt;Creative Acrylic&lt;/li&gt;
        *                   &lt;li&gt;Modern Painting&lt;/li&gt;
        *                   &lt;li&gt;Canvas Art&lt;/li&gt;
        *                   &lt;li&gt;Black white&lt;/li&gt;
        *               &lt;/ul&gt;
        *           &lt;/li&gt;
        *       &lt;/ul&gt;
        *   &lt;/li&gt;
        *   &lt;li id="book"&gt;Books
        *       &lt;ul&gt;
        *           &lt;li&gt;Entertaining&lt;/li&gt;
        *           &lt;li&gt;Design&lt;/li&gt;
        *       &lt;/ul&gt;
        *   &lt;/li&gt;
        *   &lt;li&gt;Music
        *       &lt;ul&gt;
        *           &lt;li&gt;Mass&lt;/li&gt;
        *           &lt;li&gt;Folk&lt;/li&gt;
        *       &lt;/ul&gt;
        *   &lt;/li&gt;
        * &lt;/ul&gt; <br> 
        * &lt;script&gt;
         * $("#treeView").ejTreeView();
        * // Create TreeView
        * $("#treeView").ejTreeView("getText",$("#book"));        
        * &lt;/script&gt;
        *@memberof ejTreeView
        * @instance
        */
        getText: function (node) {
            if (ej.isNullOrUndefined(node)) return false;
            var nodeText = $($(node).find('a')[0]).text();
            return nodeText;
        },
               /**
        * To select  the node in the TreeView.
        * @alias selectNode
		* @return jQuery
		* @param {object} node  object of treeview node.
        * @example 
        *   &lt;ul id="treeView"&gt;
        *   &lt;li&gt;Artwork
        *       &lt;ul&gt;
        *           &lt;li&gt;Abstract
        *               &lt;ul&gt;
        *                   &lt;li&gt;2 Acrylic Mediums&lt;/li&gt;
        *                   &lt;li&gt;Creative Acrylic&lt;/li&gt;
        *                   &lt;li&gt;Modern Painting&lt;/li&gt;
        *                   &lt;li&gt;Canvas Art&lt;/li&gt;
        *                   &lt;li&gt;Black white&lt;/li&gt;
        *               &lt;/ul&gt;
        *           &lt;/li&gt;
        *       &lt;/ul&gt;
        *   &lt;/li&gt;
        *   &lt;li id="book"&gt;Books
        *       &lt;ul&gt;
        *           &lt;li&gt;Entertaining&lt;/li&gt;
        *           &lt;li&gt;Design&lt;/li&gt;
        *       &lt;/ul&gt;
        *   &lt;/li&gt;
        *   &lt;li&gt;Music
        *       &lt;ul&gt;
        *           &lt;li&gt;Mass&lt;/li&gt;
        *           &lt;li&gt;Folk&lt;/li&gt;
        *       &lt;/ul&gt;
        *   &lt;/li&gt;
        * &lt;/ul&gt; <br> 
        * &lt;script&gt;
        * $("#treeView").ejTreeView();
        * // Create TreeView
        * var treeObj = $("#treeView").data("ejTreeView");
        * treeObj.selectNode($("#book")); // selectNode in  TreeView nodes
        * &lt;/script&gt;
          * @example 
        *  &lt;ul id="treeView"&gt;
        *   &lt;li&gt;Artwork
        *       &lt;ul&gt;
        *           &lt;li&gt;Abstract
        *               &lt;ul&gt;
        *                   &lt;li&gt;2 Acrylic Mediums&lt;/li&gt;
        *                   &lt;li&gt;Creative Acrylic&lt;/li&gt;
        *                   &lt;li&gt;Modern Painting&lt;/li&gt;
        *                   &lt;li&gt;Canvas Art&lt;/li&gt;
        *                   &lt;li&gt;Black white&lt;/li&gt;
        *               &lt;/ul&gt;
        *           &lt;/li&gt;
        *       &lt;/ul&gt;
        *   &lt;/li&gt;
        *   &lt;li id="book"&gt;Books
        *       &lt;ul&gt;
        *           &lt;li&gt;Entertaining&lt;/li&gt;
        *           &lt;li&gt;Design&lt;/li&gt;
        *       &lt;/ul&gt;
        *   &lt;/li&gt;
        *   &lt;li&gt;Music
        *       &lt;ul&gt;
        *           &lt;li&gt;Mass&lt;/li&gt;
        *           &lt;li&gt;Folk&lt;/li&gt;
        *       &lt;/ul&gt;
        *   &lt;/li&gt;
        * &lt;/ul&gt; <br> 
        * &lt;script&gt;
         * $("#treeView").ejTreeView();
        * // Create TreeView
        * $("#treeView").ejTreeView("selectNode",$("#book"));        
        * &lt;/script&gt;
        *@memberof ejTreeView
        * @instance
        */
        //Select the node
        selectNode: function (node) {
            var element;
            if (!ej.isNullOrUndefined(node) && this.model.enabled) {
                if ($(node).hasClass("e-text")) {
                    element = $(node);
                }
                else
                    element = $($(node).find(">div >a")[0]);
                this._nodeSelectionAction(element);
            }
        },
        /**
        * To unselect  the node in the TreeView.
	    * @alias unSelectNode
		* @return jQuery
		* @param {object} node  object of treeview node.
        * @example 
        *  &lt;ul id="treeView"&gt;
        *   &lt;li&gt;Artwork
        *       &lt;ul&gt;
        *           &lt;li&gt;Abstract
        *               &lt;ul&gt;
        *                   &lt;li&gt;2 Acrylic Mediums&lt;/li&gt;
        *                   &lt;li&gt;Creative Acrylic&lt;/li&gt;
        *                   &lt;li&gt;Modern Painting&lt;/li&gt;
        *                   &lt;li&gt;Canvas Art&lt;/li&gt;
        *                   &lt;li&gt;Black white&lt;/li&gt;
        *               &lt;/ul&gt;
        *           &lt;/li&gt;
        *        &lt;/ul&gt;
        *   &lt;/li&gt;
        *   &lt;li id="book"&gt;Books
        *       &lt;ul&gt;
        *           &lt;li&gt;Entertaining&lt;/li&gt;
        *           &lt;li&gt;Design&lt;/li&gt;
        *       &lt;/ul&gt;
        *   &lt;/li&gt;
        *   &lt;li&gt;Music
        *       &lt;ul&gt;
        *           &lt;li&gt;Mass&lt;/li&gt;
        *           &lt;li&gt;Folk&lt;/li&gt;
        *       &lt;/ul&gt;
        *   &lt;/li&gt;
        * &lt;/ul&gt; <br> 
        * &lt;script&gt;
        * $("#treeView").ejTreeView();
        * // Create TreeView
        * var treeObj = $("#treeView").data("ejTreeView");
        * treeObj.unselectNode($("#book")); // unSelectNode in  TreeView nodes
        * &lt;/script&gt;
          * @example 
        *  &lt;ul id="treeView"&gt;
        *   &lt;li&gt;Artwork
        *       &lt;ul&gt;
        *           &lt;li&gt;Abstract
        *               &lt;ul&gt;
        *                   &lt;li&gt;2 Acrylic Mediums&lt;/li&gt;
        *                   &lt;li&gt;Creative Acrylic&lt;/li&gt;
        *                   &lt;li&gt;Modern Painting&lt;/li&gt;
        *                   &lt;li&gt;Canvas Art&lt;/li&gt;
        *                   &lt;li&gt;Black white&lt;/li&gt;
        *               &lt;/ul&gt;
        *           &lt;/li&gt;
        *       &lt;/ul&gt;
        *   &lt;/li&gt;
        *   &lt;li id="book"&gt;Books
        *       &lt;ul&gt;
        *           &lt;li&gt;Entertaining&lt;/li&gt;
        *           &lt;li&gt;Design&lt;/li&gt;
        *       &lt;/ul&gt;
        *   &lt;/li&gt;
        *   &lt;li&gt;Music
        *       &lt;ul&gt;
        *           &lt;li&gt;Mass&lt;/li&gt;
        *           &lt;li&gt;Folk&lt;/li&gt;
        *       &lt;/ul&gt;
        *   &lt;/li&gt;
        * &lt;/ul&gt; <br> 
        * &lt;script&gt;
         * $("#treeView").ejTreeView();
        * // Create TreeView
        * $("#treeView").ejTreeView("unselectNode",$("#book"));        
        * &lt;/script&gt;
        *@memberof ejTreeView
        * @instance
        */
        //unselected node
        unselectNode: function (node) {
            var element;
            if (!ej.isNullOrUndefined(node) && this.model.enabled) {
                if ($(node).hasClass("e-text")) {
                    element = $(node);
                }
                else
                    element = $($(node).find(">div >a")[0]);
                this._nodeUnSelectionAction(element);
            }
        },
        /**
        * To enable the node in the TreeView.
		* @alias enableNode
		* @return jQuery
		* @param {object} node  object of treeview node.
        * @example 
        *   &lt;ul id="treeView"&gt;
        *   &lt;li&gt;Artwork
        *       &lt;ul&gt;
        *          &lt;li&gt;Abstract
        *               &lt;ul&gt;
        *                   &lt;li&gt;2 Acrylic Mediums&lt;/li&gt;
        *                   &lt;li&gt;Creative Acrylic&lt;/li&gt;
        *                   &lt;li&gt;Modern Painting&lt;/li&gt;
        *                   &lt;li&gt;Canvas Art&lt;/li&gt;
        *                   &lt;li&gt;Black white&lt;/li&gt;
        *               &lt;/ul&gt;
        *           &lt;/li&gt;
        *       &lt;/ul&gt;
        *   &lt;/li&gt;
        *   &lt;li id="book"&gt;Books
        *       &lt;ul&gt;
        *           &lt;li&gt;Entertaining&lt;/li&gt;
        *           &lt;li&gt;Design&lt;/li&gt;
        *       &lt;/ul&gt;
        *   &lt;/li&gt;
        *   &lt;li id="music"&gt;Music
        *       &lt;ul&gt;
        *           &lt;li&gt;Mass&lt;/li&gt;
        *           &lt;li&gt;Folk&lt;/li&gt;
        *       &lt;/ul&gt;
        *   &lt;/li&gt;
        * &lt;/ul&gt; <br> 
        * &lt;script&gt;
        * $("#treeView").ejTreeView();
        * // Create TreeView
        * var treeObj = $("#treeView").data("ejTreeView");
        * treeObj.disableNode($("#music"));
        * treeObj.disableNode($("#book"));
        * treeObj.enableNode($("#book")); // enableNode in  TreeView nodes
        * &lt;/script&gt;
          * @example 
        *  &lt;ul id="treeView"&gt;
        *   &lt;li&gt;Artwork
        *       &lt;ul&gt;
        *           &lt;li&gt;Abstract
        *               &lt;ul&gt;
        *                   &lt;li&gt;2 Acrylic Mediums&lt;/li&gt;
        *                   &lt;li&gt;Creative Acrylic&lt;/li&gt;
        *                   &lt;li&gt;Modern Painting&lt;/li&gt;
        *                   &lt;li&gt;Canvas Art&lt;/li&gt;
        *                   &lt;li&gt;Black white&lt;/li&gt;
        *               &lt;/ul&gt;
        *           &lt;/li&gt;
        *       &lt;/ul&gt;
        *   &lt;/li&gt;
        *   &lt;li id="book"&gt;Books
        *       &lt;ul&gt;
        *           &lt;li&gt;Entertaining&lt;/li&gt;
        *           &lt;li&gt;Design&lt;/li&gt;
        *       &lt;/ul&gt;
        *   &lt;/li&gt;
        *   &lt;li id="music"&gt;Music
        *       &lt;ul&gt;
        *           &lt;li&gt;Mass&lt;/li&gt;
        *           &lt;li&gt;Folk&lt;/li&gt;
        *       &lt;/ul&gt;
        *   &lt;/li&gt;
        * &lt;/ul&gt; <br> 
        * &lt;script&gt;
         * $("#treeView").ejTreeView();
        * // Create TreeView  
        * $("#treeView").ejTreeView("disableNode",$("#music")); 
        * $("#treeView").ejTreeView("disableNode",$("#book")); 
        * $("#treeView").ejTreeView("enableNode",$("#book"));        
        * &lt;/script&gt;
        *@memberof ejTreeView
        * @instance
        */
        //Enable the node
        enableNode: function (node) {
            var element;
            if (!ej.isNullOrUndefined(node) && this.model.enabled) {
                if ($(node).hasClass("e-text")) {
                    element = $(node);
                }
                else
                    element = $($(node).find("a.e-text")[0]);
                this._nodeEnableAction(element);
                $(node).parent().find(".nodecheckbox").ejCheckBox("enable");
            }
        },
         /**
        * To disable the node in the TreeView.
		* @alias disableNode
		* @return jQuery
		* @param {object} node  object of treeview node.
        * @example 
        *   &lt;ul id="treeView"&gt;
        *   &lt;li&gt;Artwork
        *       &lt;ul&gt;
        *           &lt;li&gt;Abstract
        *               &lt;ul&gt;
        *                   &lt;li&gt;2 Acrylic Mediums&lt;/li&gt;
        *                   &lt;li&gt;Creative Acrylic&lt;/li&gt;
        *                   &lt;li&gt;Modern Painting&lt;/li&gt;
        *                   &lt;li&gt;Canvas Art&lt;/li&gt;
        *                   &lt;li&gt;Black white&lt;/li&gt;
        *               &lt;/ul&gt;
        *           &lt;/li&gt;
        *       &lt;/ul&gt;
        *   &lt;/li&gt;
        *   &lt;li id="book"&gt;Books
        *       &lt;ul&gt;
        *           &lt;li&gt;Entertaining&lt;/li&gt;
        *           &lt;li&gt;Design&lt;/li&gt;
        *       &lt;/ul&gt;
        *   &lt;/li&gt;
        *   &lt;li&gt;Music
        *       &lt;ul&gt;
        *           &lt;li&gt;Mass&lt;/li&gt;
        *           &lt;li&gt;Folk&lt;/li&gt;
        *       &lt;/ul&gt;
        *   &lt;/li&gt;
        * &lt;/ul&gt; 
        * &lt;script&gt;
        * $("#treeView").ejTreeView();
        * // Create TreeView
        * var treeObj = $("#treeView").data("ejTreeView");
        * treeObj.disableNode($("#book")); // disableNode in  TreeView nodes
        * &lt;/script&gt;
          * @example 
        *  &lt;ul id="treeView"&gt;
        *   &lt;li&gt;Artwork
        *       &lt;ul&gt;
        *           &lt;li&gt;Abstract
        *               &lt;ul&gt;
        *                   &lt;li&gt;2 Acrylic Mediums&lt;/li&gt;
        *                   &lt;li&gt;Creative Acrylic&lt;/li&gt;
        *                   &lt;li&gt;Modern Painting&lt;/li&gt;
        *                   &lt;li&gt;Canvas Art&lt;/li&gt;
        *                   &lt;li&gt;Black white&lt;/li&gt;
        *               &lt;/ul&gt;
        *           &lt;/li&gt;
        *       &lt;/ul&gt;
        *   &lt;/li&gt;
        *   &lt;li id="book"&gt;Books
        *       &lt;ul&gt;
        *           &lt;li&gt;Entertaining&lt;/li&gt;
        *           &lt;li&gt;Design&lt;/li&gt;
        *       &lt;/ul&gt;
        *   &lt;/li&gt;
        *   &lt;li&gt;Music
        *       &lt;ul&gt;
        *           &lt;li&gt;Mass&lt;/li&gt;
        *           &lt;li&gt;Folk&lt;/li&gt;
        *       &lt;/ul&gt;
        *   &lt;/li&gt;
        * &lt;/ul&gt; <br> 
        * &lt;script&gt;
         * $("#treeView").ejTreeView();
        * // Create TreeView
        * $("#treeView").ejTreeView("disableNode",$("#book"));        
        * &lt;/script&gt;
        *@memberof ejTreeView
        * @instance
        */
        //Disable node
        disableNode: function (node) {
            var element;
            if (!ej.isNullOrUndefined(node) && this.model.enabled) {
                if ($(node).hasClass("e-text")) {
                    element = $(node);
                }
                else
                    element = $($(node).find(">div >a")[0]);
                this._nodeDisableAction(element);
                $(node).find(".nodecheckbox").ejCheckBox("disable");
            }
        },
        
        addNodes: function (collection, targetNode) {
            if (collection && collection.length > 0) {
                var mapper = this.model.fields;

                for (var i = 0; i < collection.length; i++) {
                    var node = collection[i], text = node[mapper.text];

                    if (text) {
                        var id = node[mapper.id], pid = node[mapper.parentId],
                        url = node[mapper.linkAttribute] ? node[mapper.linkAttribute] : "#",
                        parent = (targetNode && targetNode.length != 0) ? targetNode : null;

                        if (ej.isNullOrUndefined(parent)) {
                            var tar = pid ? this.element.find("#" + pid) : null;
                            parent = (tar && tar.length != 0) ? tar.children("div").find("a") : null;
                        }
                        if (ej.isNullOrUndefined(parent)) continue;

                        var nodeElement = this._createTreeElement(text, null, url);
                        id ? nodeElement.attr("id", id) : nodeElement.removeAttr("id");
                        this._appendNode(parent, nodeElement);
                        nodeElement.find(".nodecheckbox").ejCheckBox({ cssClass: this.model.cssClass, change: this._checkedChange });
                        if (this.model.allowDragAndDrop) nodeElement.find("a").addClass("e-draggable e-droppable");
                    }
                }
            }
        },
         /**
        * To add Node in the TreeView.
		* @alias addNode
		* @return jQuery
		* @param {string} newNodeText  new Treeview Node Text
		* @param {string} nodeUrl  new Treeview node Url.
		* @param {Object} target  object of Tree view node.
        * @example 
        *  &lt;ul id="treeView"&gt;
        *   &lt;li&gt;Artwork
        *       &lt;ul&gt;
        *           &lt;li&gt;Abstract
        *               &lt;ul&gt;
        *                   &lt;li&gt;2 Acrylic Mediums&lt;/li&gt;
        *                   &lt;li&gt;Creative Acrylic&lt;/li&gt;
        *                   &lt;li&gt;Modern Painting&lt;/li&gt;
        *                   &lt;li&gt;Canvas Art&lt;/li&gt;
        *                   &lt;li&gt;Black white&lt;/li&gt;
        *               &lt;/ul&gt;
        *           &lt;/li&gt;
        *       &lt;/ul&gt;
        *   &lt;/li&gt;
        *   &lt;li id="book"&gt;Books
        *       &lt;ul&gt;
        *           &lt;li&gt;Entertaining&lt;/li&gt;
        *           &lt;li&gt;Design&lt;/li&gt;
        *       &lt;/ul&gt;
        *   &lt;/li&gt;
        *   &lt;li&gt;Music
        *       &lt;ul&gt;
        *           &lt;li&gt;Mass&lt;/li&gt;
        *           &lt;li&gt;Folk&lt;/li&gt;
        *       &lt;/ul&gt;
        *   &lt;/li&gt;
        * &lt;/ul&gt; <br> 
        * &lt;script&gt;
         * $("#treeView").ejTreeView();
        * // Create TreeView
        * var treeObj = $("#treeView").data("ejTreeView");
        * treeObj.addNode("NodeNew", "Folk"); // addNode in  TreeView nodes
        * &lt;/script&gt;
        * @example 
        *  &lt;ul id="treeView"&gt;
        *   &lt;li&gt;Artwork
        *       &lt;ul&gt;
        *           &lt;li&gt;Abstract
        *               &lt;ul&gt;
        *                   &lt;li&gt;2 Acrylic Mediums&lt;/li&gt;
        *                   &lt;li&gt;Creative Acrylic&lt;/li&gt;
        *                   &lt;li&gt;Modern Painting&lt;/li&gt;
        *                   &lt;li&gt;Canvas Art&lt;/li&gt;
        *                   &lt;li&gt;Black white&lt;/li&gt;
        *               &lt;/ul&gt;
        *           &lt;/li&gt;
        *       &lt;/ul&gt;
        *   &lt;/li&gt;
        *   &lt;li id="book"&gt;Books
        *       &lt;ul&gt;
        *           &lt;li&gt;Entertaining&lt;/li&gt;
        *           &lt;li&gt;Design&lt;/li&gt;
        *       &lt;/ul&gt;
        *   &lt;/li&gt;
        *   &lt;li&gt;Music
        *       &lt;ul&gt;
        *           &lt;li&gt;Mass&lt;/li&gt;
        *           &lt;li&gt;Folk&lt;/li&gt;
        *       &lt;/ul&gt;
        *   &lt;/li&gt;
        * &lt;/ul&gt; <br> 
        * &lt;script&gt;
         * $("#treeView").ejTreeView();
        * // Create TreeView
        * $("#treeView").ejTreeView("addNode","NodeNew", "Folk");        
        * &lt;/script&gt;
        * @memberof ejTreeView
        * @instance
        */
        //Add the node
        addNode: function (newNodeText, nodeUrl, targetNode) {
            if (this.model.enabled) {
                var outerLi = null;
                var innerUl = null;
                var element = this.element;
                var childCount = element.children().length;
                var selectedNode = ej.isNullOrUndefined(targetNode) ? element.find('.e-active') : targetNode;
                if (selectedNode.length == 0) {
                    if (childCount == 0 || element.children().children().length == 0)
                        innerUl = ej.buildTag('ul');
                    else
                        innerUl = element.children()[childCount - 1];
                    outerLi = this._createTreeElement(newNodeText, null, nodeUrl);
                    $(innerUl).append(outerLi);
                    this._appendNode(selectedNode, innerUl);
                    $(outerLi).find(".nodecheckbox").ejCheckBox({ cssClass: this.model.cssClass, change: this._checkedChange });
                }
                else {
                    this._expandNode($($(selectedNode).parents('li.e-item')[0]));
                    outerLi = this._createTreeElement(newNodeText, selectedNode, nodeUrl);
                    this._appendNode(selectedNode, outerLi);
                    this.model.showCheckbox && $(outerLi).find(".nodecheckbox").ejCheckBox({ cssClass: this.model.cssClass, change: this._checkedChange });
                }
                this._applyFirstLastChildClass();
                this._enableDragDrop();
                this._modifiCss();
            }
        },
          /**
        * To remove Node in the TreeView.
		* @alias removeNode
		* @return jQuery
		* @param {Object} element  object of Tree view node.
        * @example 
        *   &lt;ul id="treeView"&gt;
        *   &lt;li&gt;Artwork
        *       &lt;ul&gt;
        *           &lt;li&gt;Abstract
        *               &lt;ul&gt;
        *                  &lt;li&gt;2 Acrylic Mediums&lt;/li&gt;
        *                   &lt;li&gt;Creative Acrylic&lt;/li&gt;
        *                   &lt;li&gt;Modern Painting&lt;/li&gt;
        *                   &lt;li&gt;Canvas Art&lt;/li&gt;
        *                   &lt;li&gt;Black white&lt;/li&gt;
        *               &lt;/ul&gt;
        *           &lt;/li&gt;
        *       &lt;/ul&gt;
        *   &lt;/li&gt;
        *   &lt;li id="book"&gt;Books
        *       &lt;ul&gt;
        *           &lt;li&gt;Entertaining&lt;/li&gt;
        *          &lt;li&gt;Design&lt;/li&gt;
        *       &lt;/ul&gt;
        *   &lt;/li&gt;
        *   &lt;li&gt;Music
        *       &lt;ul&gt;
        *           &lt;li&gt;Mass&lt;/li&gt;
        *           &lt;li&gt;Folk&lt;/li&gt;
        *       &lt;/ul&gt;
        *   &lt;/li&gt;
        * &lt;/ul&gt; <br> 
        * &lt;script&gt;
         * $("#treeView").ejTreeView();
        * // Create TreeView
        * var treeObj = $("#treeView").data("ejTreeView");
        * treeObj.removeNode($("#book")); // removeNode in  TreeView nodes
        * &lt;/script&gt;
          * @example 
        *  &lt;ul id="treeView"&gt;
        *   &lt;li&gt;Artwork
        *       &lt;ul&gt;
        *           &lt;li&gt;Abstract
        *               &lt;ul&gt;
        *                   &lt;li&gt;2 Acrylic Mediums&lt;/li&gt;
        *                   &lt;li&gt;Creative Acrylic&lt;/li&gt;
        *                   &lt;li&gt;Modern Painting&lt;/li&gt;
        *                   &lt;li&gt;Canvas Art&lt;/li&gt;
        *                   &lt;li&gt;Black white&lt;/li&gt;
        *               &lt;/ul&gt;
        *           &lt;/li&gt;
        *       &lt;/ul&gt;
        *   &lt;/li&gt;
        *   &lt;li id="book"&gt;Books
        *       &lt;ul&gt;
        *           &lt;li&gt;Entertaining&lt;/li&gt;
        *           &lt;li&gt;Design&lt;/li&gt;
        *       &lt;/ul&gt;
        *   &lt;/li&gt;
        *   &lt;li&gt;Music
        *       &lt;ul&gt;
        *           &lt;li&gt;Mass&lt;/li&gt;
        *           &lt;li&gt;Folk&lt;/li&gt;
        *       &lt;/ul&gt;
        *   &lt;/li&gt;
        * &lt;/ul&gt; <br> 
        * &lt;script&gt;
         * $("#treeView").ejTreeView();
        * // Create TreeView
        * $("#treeView").ejTreeView("removeNode",$("#book"));        
        * &lt;/script&gt;
        *@memberof ejTreeView
        * @instance
        */
        //Remove the node
        removeNode: function (node) {
            if (this.model.enabled) {
                if (!ej.isNullOrUndefined(node)) {
                    if ($(node).hasClass("e-text")) {
                        element = $(node);
                    }
                    else
                        element = $($(node).find(">div >a")[0]);
                }
                this._removeNode(element);
            }
        },
    /**
        * To getCheckedNodes in the TreeView.
	    * @alias getCheckedNodes
        * @return Object
        * @example 
        *  &lt;ul id="treeView"&gt;
        *   &lt;li&gt;Artwork
        *       &lt;ul&gt;
        *           &lt;li&gt;Abstract
        *               &lt;ul&gt;
        *                   &lt;li&gt;2 Acrylic Mediums&lt;/li&gt;
        *                   &lt;li&gt;Creative Acrylic&lt;/li&gt;
        *                   &lt;li&gt;Modern Painting&lt;/li&gt;
        *                   &lt;li&gt;Canvas Art&lt;/li&gt;
        *                   &lt;li&gt;Black white&lt;/li&gt;
        *               &lt;/ul&gt;
        *           &lt;/li&gt;
        *       &lt;/ul&gt;
        *   &lt;/li&gt;
        *   &lt;li id="book"&gt;Books
        *       &lt;ul&gt;
        *           &lt;li&gt;Entertaining&lt;/li&gt;
        *           &lt;li&gt;Design&lt;/li&gt;
        *       &lt;/ul&gt;
        *   &lt;/li&gt;
        *   &lt;li&gt;Music
        *       &lt;ul&gt;
        *           &lt;li&gt;Mass&lt;/li&gt;
        *           &lt;li&gt;Folk&lt;/li&gt;
        *       &lt;/ul&gt;
        *   &lt;/li&gt;
        * &lt;/ul&gt; <br> 
        * &lt;script&gt;
         * $("#treeView").ejTreeView({showCheckbox:true});
        * // Create TreeView
        * var treeObj = $("#treeView").data("ejTreeView");
		*treeObj.checkNode($("#book"));
        * treeObj.getCheckedNodes(); // getAllCheckedNodes in  TreeView nodes
        * &lt;/script&gt;
          * @example 
        *  &lt;ul id="treeView"&gt;
        *   &lt;li&gt;Artwork
        *       &lt;ul&gt;
        *           &lt;li&gt;Abstract
        *               &lt;ul&gt;
        *                   &lt;li&gt;2 Acrylic Mediums&lt;/li&gt;
        *                   &lt;li&gt;Creative Acrylic&lt;/li&gt;
        *                   &lt;li&gt;Modern Painting&lt;/li&gt;
        *                   &lt;li&gt;Canvas Art&lt;/li&gt;
        *                   &lt;li&gt;Black white&lt;/li&gt;
        *               &lt;/ul&gt;
        *           &lt;/li&gt;
        *       &lt;/ul&gt;
        *   &lt;/li&gt;
        *   &lt;li id="book"&gt;Books
        *       &lt;ul&gt;
        *           &lt;li&gt;Entertaining&lt;/li&gt;
        *           &lt;li&gt;Design&lt;/li&gt;
        *       &lt;/ul&gt;
        *   &lt;/li&gt;
        *   &lt;li&gt;Music
        *       &lt;ul&gt;
        *           &lt;li&gt;Mass&lt;/li&gt;
        *           &lt;li&gt;Folk&lt;/li&gt;
        *       &lt;/ul&gt;
        *   &lt;/li&gt;
        * &lt;/ul&gt; <br> 
        * &lt;script&gt;
         * $("#treeView").ejTreeView({showCheckbox:true});
        * // Create TreeView
        * $("#treeView").ejTreeView("checkNode",$("#book"));
        * $("#treeView").ejTreeView("getCheckedNodes");        
        * &lt;/script&gt;
        *@memberof ejTreeView
        * @instance
        */
        //Get all the checked  nodes
        getCheckedNodes: function () {
            var nodeCollection = this.element.find('input.nodecheckbox[value="True"]').closest('li');
            return nodeCollection;
        },
 /**
        * To Check particular node checkbox in TreeView.
		* @alias checkNode
		* @return jQuery
		* @param {Object} element  object of Tree view node.
        * @example 
        *   &lt;ul id="treeView"&gt;
        *   &lt;li&gt;Artwork
        *       &lt;ul&gt;
        *           &lt;li&gt;Abstract
        *               &lt;ul&gt;
        *                   &lt;li&gt;2 Acrylic Mediums&lt;/li&gt;
        *                   &lt;li&gt;Creative Acrylic&lt;/li&gt;
        *                   &lt;li&gt;Modern Painting&lt;/li&gt;
        *                   &lt;li&gt;Canvas Art&lt;/li&gt;
        *                   &lt;li&gt;Black white&lt;/li&gt;
        *               &lt;/ul&gt;
        *           &lt;/li&gt;
        *       &lt;/ul&gt;
        *   &lt;/li&gt;
        *   &lt;li id="book"&gt;Books
        *       &lt;ul&gt;
        *           &lt;li&gt;Entertaining&lt;/li&gt;
        *           &lt;li&gt;Design&lt;/li&gt;
        *       &lt;/ul&gt;
        *   &lt;/li&gt;
        *   &lt;li&gt;Music
        *       &lt;ul&gt;
        *           &lt;li&gt;Mass&lt;/li&gt;
        *           &lt;li&gt;Folk&lt;/li&gt;
        *       &lt;/ul&gt;
        *   &lt;/li&gt;
        * &lt;/ul&gt; <br> 
        * &lt;script&gt;
         * $("#treeView").ejTreeView({showCheckbox:true});
        * // Create TreeView
        * var treeObj = $("#treeView").data("ejTreeView");
        * treeObj.checkNode($("#book")); // checkNode in  TreeView nodes
        * &lt;/script&gt;
        * @example 
        *  &lt;ul id="treeView"&gt;
        *   &lt;li&gt;Artwork
        *       &lt;ul&gt;
        *           &lt;li&gt;Abstract
        *               &lt;ul&gt;
        *                   &lt;li&gt;2 Acrylic Mediums&lt;/li&gt;
        *                   &lt;li&gt;Creative Acrylic&lt;/li&gt;
        *                   &lt;li&gt;Modern Painting&lt;/li&gt;
        *                   &lt;li&gt;Canvas Art&lt;/li&gt;
        *                   &lt;li&gt;Black white&lt;/li&gt;
        *               &lt;/ul&gt;
        *           &lt;/li&gt;
        *       &lt;/ul&gt;
        *   &lt;/li&gt;
        *   &lt;li id="book"&gt;Books
        *       &lt;ul&gt;
        *           &lt;li&gt;Entertaining&lt;/li&gt;
        *           &lt;li&gt;Design&lt;/li&gt;
        *       &lt;/ul&gt;
        *   &lt;/li&gt;
        *   &lt;li&gt;Music
        *       &lt;ul&gt;
        *           &lt;li&gt;Mass&lt;/li&gt;
        *           &lt;li&gt;Folk&lt;/li&gt;
        *       &lt;/ul&gt;
        *   &lt;/li&gt;
        * &lt;/ul&gt; <br> 
        * &lt;script&gt;
         * $("#treeView").ejTreeView({showCheckbox:true});
        * // Create TreeView
        * $("#treeView").ejTreeView("checkNode",$("#book"));        
        * &lt;/script&gt;
        *@memberof ejTreeView
        * @instance
        */
        //Check particular node checkbox
        checkNode: function (node) {
            if (!ej.isNullOrUndefined(node) && this.model.enabled) {
                this._nodeCheckedAction($(node).find("span.e-chkbox-wrap").first());
            }
        },
       /**
        * To Uncheck particular node checkbox in TreeView.
    	* @alias unCheckNode
		* @return jQuery
		* @param {Object} element  object of Tree view node.
        * @example 
        *   &lt;ul id="treeView"&gt;
        *  &lt;li&gt;Artwork
        *       &lt;ul&gt;
        *           &lt;li&gt;Abstract
        *               &lt;ul&gt;
        *                   &lt;li&gt;2 Acrylic Mediums&lt;/li&gt;
        *                   &lt;li&gt;Creative Acrylic&lt;/li&gt;
        *                   &lt;li&gt;Modern Painting&lt;/li&gt;
        *                  &lt;li&gt;Canvas Art&lt;/li&gt;
        *                   &lt;li&gt;Black white&lt;/li&gt;
        *               &lt;/ul&gt;
        *           &lt;/li&gt;
        *      &lt;/ul&gt;
        *   &lt;/li&gt;
        *   &lt;li id="book"&gt;Books
        *       &lt;ul&gt;
        *           &lt;li&gt;Entertaining&lt;/li&gt;
        *           &lt;li&gt;Design&lt;/li&gt;
        *       &lt;/ul&gt;
        *   &lt;/li&gt;
        *   &lt;li&gt;Music
        *       &lt;ul&gt;
        *           &lt;li&gt;Mass&lt;/li&gt;
        *           &lt;li&gt;Folk&lt;/li&gt;
        *       &lt;/ul&gt;
        *   &lt;/li&gt;
        * &lt;/ul&gt; <br> 
        * &lt;script&gt;
         * $("#treeView").ejTreeView({showCheckbox:true});
        * // Create TreeView
        * var treeObj = $("#treeView").data("ejTreeView");
		* treeObj.checkAll();
        * treeObj.unCheckNode($("#book")); // uncheckNode in  TreeView nodes
        * &lt;/script&gt;
          * @example 
        *  &lt;ul id="treeView"&gt;
        *   &lt;li&gt;Artwork
        *       &lt;ul&gt;
        *           &lt;li&gt;Abstract
        *               &lt;ul&gt;
        *                   &lt;li&gt;2 Acrylic Mediums&lt;/li&gt;
        *                   &lt;li&gt;Creative Acrylic&lt;/li&gt;
        *                   &lt;li&gt;Modern Painting&lt;/li&gt;
        *                   &lt;li&gt;Canvas Art&lt;/li&gt;
        *                   &lt;li&gt;Black white&lt;/li&gt;
        *               &lt;/ul&gt;
        *           &lt;/li&gt;
        *       &lt;/ul&gt;
        *   &lt;/li&gt;
        *   &lt;li id="book"&gt;Books
        *       &lt;ul&gt;
        *           &lt;li&gt;Entertaining&lt;/li&gt;
        *           &lt;li&gt;Design&lt;/li&gt;
        *       &lt;/ul&gt;
        *   &lt;/li&gt;
        *   &lt;li&gt;Music
        *       &lt;ul&gt;
        *           &lt;li&gt;Mass&lt;/li&gt;
        *           &lt;li&gt;Folk&lt;/li&gt;
        *       &lt;/ul&gt;
        *   &lt;/li&gt;
        * &lt;/ul&gt; <br> 
        * &lt;script&gt;
         * $("#treeView").ejTreeView();
        * // Create TreeView
        * $("#treeView").ejTreeView("checkAll");
        * $("#treeView").ejTreeView("unCheckNode",$("#book"));        
        * &lt;/script&gt;
        *@memberof ejTreeView
        * @instance
        */
        //UnCheck particular node textbox
        unCheckNode: function (node) {
            if (!ej.isNullOrUndefined(node) && this.model.enabled) {
                this._nodeUnCheckedAction($(node).find("span.e-chkbox-wrap").first());
            }
        },
         /**
        * To expandNode particular node  in TreeView.
		* @alias expandNode
		* @return jQuery
		* @param {Object} element  object of Tree view node.
        * @example 
        *   &lt;ul id="treeView"&gt;
        *   &lt;li&gt;Artwork
        *       &lt;ul&gt;
        *           &lt;li&gt;Abstract
        *               &lt;ul&gt;
        *                   &lt;li&gt;2 Acrylic Mediums&lt;/li&gt;
        *                   &lt;li&gt;Creative Acrylic&lt;/li&gt;
        *                   &lt;li&gt;Modern Painting&lt;/li&gt;
        *                   &lt;li&gt;Canvas Art&lt;/li&gt;
        *                   &lt;li&gt;Black white&lt;/li&gt;
        *               &lt;/ul&gt;
        *           &lt;/li&gt;
        *       &lt;/ul&gt;
        *   &lt;/li&gt;
        *   &lt;li id="book"&gt;Books
        *       &lt;ul&gt;
        *           &lt;li&gt;Entertaining&lt;/li&gt;
        *           &lt;li&gt;Design&lt;/li&gt;
        *       &lt;/ul&gt;
        *   &lt;/li&gt;
        *   &lt;li&gt;Music
        *       &lt;ul&gt;
        *           &lt;li&gt;Mass&lt;/li&gt;
        *           &lt;li&gt;Folk&lt;/li&gt;
        *       &lt;/ul&gt;
        *   &lt;/li&gt;
        * &lt;/ul&gt; <br> 
        * &lt;script&gt;
        * $("#treeView").ejTreeView();
        * // Create TreeView
        * var treeObj = $("#treeView").data("ejTreeView");
        * treeObj.collapseAll();
        * treeObj.expandNode($("#book")); // expandNode in  TreeView nodes
        * &lt;/script&gt;
          * @example 
        *  &lt;ul id="treeView"&gt;
        *   &lt;li&gt;Artwork
        *       &lt;ul&gt;
        *           &lt;li&gt;Abstract
        *               &lt;ul&gt;
        *                   &lt;li&gt;2 Acrylic Mediums&lt;/li&gt;
        *                   &lt;li&gt;Creative Acrylic&lt;/li&gt;
        *                   &lt;li&gt;Modern Painting&lt;/li&gt;
        *                   &lt;li&gt;Canvas Art&lt;/li&gt;
        *                   &lt;li&gt;Black white&lt;/li&gt;
        *               &lt;/ul&gt;
        *           &lt;/li&gt;
        *       &lt;/ul&gt;
        *   &lt;/li&gt;
        *   &lt;li id="book"&gt;Books
        *       &lt;ul&gt;
        *           &lt;li&gt;Entertaining&lt;/li&gt;
        *           &lt;li&gt;Design&lt;/li&gt;
        *       &lt;/ul&gt;
        *   &lt;/li&gt;
        *   &lt;li&gt;Music
        *       &lt;ul&gt;
        *           &lt;li&gt;Mass&lt;/li&gt;
        *           &lt;li&gt;Folk&lt;/li&gt;
        *       &lt;/ul&gt;
        *   &lt;/li&gt;
        * &lt;/ul&gt; <br> 
        * &lt;script&gt;
         * $("#treeView").ejTreeView();
        * // Create TreeView
        * $("#treeView").ejTreeView("collapseAll");
        * $("#treeView").ejTreeView("expandNode",$("#book"));        
        * &lt;/script&gt;
        *@memberof ejTreeView
        * @instance
        */
        expandNode: function (node) {
            if (!ej.isNullOrUndefined(node) && this.model.enabled) {
                this._expandNode(node);
            }
        },
         /**
        * To collapseNode particular node  in TreeView.
		* @alias collapseNode
		* @return jQuery
		* @param {Object} element  object of Tree view node.
        * @example 
        *  &lt;ul id="treeView"&gt;
        *   &lt;li id="art"&gt;Artwork
        *       &lt;ul&gt;
        *           &lt;li&gt;Abstract
        *               &lt;ul&gt;
        *                   &lt;li&gt;2 Acrylic Mediums&lt;/li&gt;
        *                   &lt;li&gt;Creative Acrylic&lt;/li&gt;
        *                   &lt;li&gt;Modern Painting&lt;/li&gt;
        *                   &lt;li&gt;Canvas Art&lt;/li&gt;
        *                   &lt;li&gt;Black white&lt;/li&gt;
        *               &lt;/ul&gt;
        *           &lt;/li&gt;
        *       &lt;/ul&gt;
        *   &lt;/li&gt;
        *   &lt;li id="book"&gt;Books
        *       &lt;ul&gt;
        *           &lt;li&gt;Entertaining&lt;/li&gt;
        *           &lt;li&gt;Design&lt;/li&gt;
        *       &lt;/ul&gt;
        *   &lt;/li&gt;
        *   &lt;li&gt;Music
        *       &lt;ul&gt;
        *           &lt;li&gt;Mass&lt;/li&gt;
        *           &lt;li&gt;Folk&lt;/li&gt;
        *       &lt;/ul&gt;
        *   &lt;/li&gt;
        * &lt;/ul&gt; <br> 
        * &lt;script&gt;
         * $("#treeView").ejTreeView();
        * // Create TreeView
        * var treeObj = $("#treeView").data("ejTreeView");
		* treeObj.expandAll();
        * treeObj.collapseNode($("#art")); // collapseNode in  TreeView nodes
        * &lt;/script&gt;
         * @example 
        *  &lt;ul id="treeView"&gt;
        *   &lt;li id="art"&gt;Artwork
        *       &lt;ul&gt;
        *           &lt;li&gt;Abstract
        *               &lt;ul&gt;
        *                   &lt;li&gt;2 Acrylic Mediums&lt;/li&gt;
        *                   &lt;li&gt;Creative Acrylic&lt;/li&gt;
        *                   &lt;li&gt;Modern Painting&lt;/li&gt;
        *                   &lt;li&gt;Canvas Art&lt;/li&gt;
        *                   &lt;li&gt;Black white&lt;/li&gt;
        *               &lt;/ul&gt;
        *           &lt;/li&gt;
        *       &lt;/ul&gt;
        *   &lt;/li&gt;
        *   &lt;li id="book"&gt;Books
        *       &lt;ul&gt;
        *           &lt;li&gt;Entertaining&lt;/li&gt;
        *           &lt;li&gt;Design&lt;/li&gt;
        *       &lt;/ul&gt;
        *   &lt;/li&gt;
        *   &lt;li&gt;Music
        *       &lt;ul&gt;
        *           &lt;li&gt;Mass&lt;/li&gt;
        *           &lt;li&gt;Folk&lt;/li&gt;
        *       &lt;/ul&gt;
        *   &lt;/li&gt;
        * &lt;/ul&gt; <br> 
        * &lt;script&gt;
         * $("#treeView").ejTreeView();
        * // Create TreeView
        * $("#treeView").ejTreeView("expandAll");
        * $("#treeView").ejTreeView("collapseNode",$("#art"));        
        * &lt;/script&gt;
        *@memberof ejTreeView
        * @instance
        */
        //Collapses the selected node
        collapseNode: function (node) {
            if (!ej.isNullOrUndefined(node) && this.model.enabled) {
                this._collpaseNode(node);
            }
        },
      /**
        * To showNode particular node  in TreeView.
		* @alias showNode
		* @return jQuery
		* @param {Object} element  object of Tree view node.
        * @example 
        * &lt;ul id="treeView"&gt;
        *  &lt;li&gt;Artwork
        *      &lt;ul&gt;
        *          &lt;li&gt;Abstract
        *              &lt;ul&gt;
        *                  &lt;li&gt;2 Acrylic Mediums&lt;/li&gt;
        *                  &lt;li&gt;Creative Acrylic&lt;/li&gt;
        *                  &lt;li&gt;Modern Painting&lt;/li&gt;
        *                  &lt;li&gt;Canvas Art&lt;/li&gt;
        *                  &lt;li&gt;Black white&lt;/li&gt;
        *              &lt;/ul&gt;
        *          &lt;/li&gt;
        *      &lt;/ul&gt;
        *  &lt;/li&gt;
        *  &lt;li id="book"&gt;Books
        *      &lt;ul&gt;
        *          &lt;li&gt;Entertaining&lt;/li&gt;
        *          &lt;li&gt;Design&lt;/li&gt;
        *      &lt;/ul&gt;
        *  &lt;/li&gt;
        *  &lt;li&gt;Music
        *      &lt;ul&gt;
        *          &lt;li&gt;Mass&lt;/li&gt;
        *          &lt;li&gt;Folk&lt;/li&gt;
        *      &lt;/ul&gt;
        *  &lt;/li&gt;
        * &lt;/ul&gt; <br> 
        * &lt;script&gt;
         * $("#treeView").ejTreeView();
        * // Create TreeView
        * var treeObj = $("#treeView").data("ejTreeView");
        * treeObj.showNode($("#book")); // showNode in  TreeView nodes
        * &lt;/script&gt;
          * @example 
        *  &lt;ul id="treeView"&gt;
        *   &lt;li&gt;Artwork
        *       &lt;ul&gt;
        *           &lt;li&gt;Abstract
        *               &lt;ul&gt;
        *                   &lt;li&gt;2 Acrylic Mediums&lt;/li&gt;
        *                   &lt;li&gt;Creative Acrylic&lt;/li&gt;
        *                   &lt;li&gt;Modern Painting&lt;/li&gt;
        *                   &lt;li&gt;Canvas Art&lt;/li&gt;
        *                   &lt;li&gt;Black white&lt;/li&gt;
        *               &lt;/ul&gt;
        *           &lt;/li&gt;
        *       &lt;/ul&gt;
        *   &lt;/li&gt;
        *   &lt;li id="book"&gt;Books
        *       &lt;ul&gt;
        *           &lt;li&gt;Entertaining&lt;/li&gt;
        *           &lt;li&gt;Design&lt;/li&gt;
        *       &lt;/ul&gt;
        *   &lt;/li&gt;
        *   &lt;li&gt;Music
        *       &lt;ul&gt;
        *           &lt;li&gt;Mass&lt;/li&gt;
        *           &lt;li&gt;Folk&lt;/li&gt;
        *       &lt;/ul&gt;
        *   &lt;/li&gt;
        * &lt;/ul&gt; <br> 
        * &lt;script&gt;
         * $("#treeView").ejTreeView();
        * // Create TreeView
        * $("#treeView").ejTreeView("showNode",$("#book"));        
        * &lt;/script&gt;
        * @memberof ejTreeView
        * @instance
        */
        showNode: function (node) {
            if (!ej.isNullOrUndefined(node) && this.model.enabled) {
                node.css("visibility", "");
            }
        },
        /**
        * To hideNode particular node  in TreeView.
		* @alias hideNode
		* @return jQuery
		* @param {Object} element  object of Tree view node.
        * @example 
        *   &lt;ul id="treeView"&gt;
        *  &lt;li&gt;Artwork
        *      &lt;ul&gt;
        *          &lt;li&gt;Abstract
        *              &lt;ul&gt;
        *                  &lt;li&gt;2 Acrylic Mediums&lt;/li&gt;
        *                  &lt;li&gt;Creative Acrylic&lt;/li&gt;
        *                  &lt;li&gt;Modern Painting&lt;/li&gt;
        *                  &lt;li&gt;Canvas Art&lt;/li&gt;
        *                  &lt;li&gt;Black white&lt;/li&gt;
        *              &lt;/ul&gt;
        *          &lt;/li&gt;
        *      &lt;/ul&gt;
        *  &lt;/li&gt;
        *  &lt;li id="book"&gt;Books
        *      &lt;ul&gt;
        *          &lt;li&gt;Entertaining&lt;/li&gt;
        *          &lt;li&gt;Design&lt;/li&gt;
        *      &lt;/ul&gt;
        *  &lt;/li&gt;
        *  &lt;li&gt;Music
        *      &lt;ul&gt;
        *          &lt;li&gt;Mass&lt;/li&gt;
        *          &lt;li&gt;Folk&lt;/li&gt;
        *      &lt;/ul&gt;
        *  &lt;/li&gt;
        *&lt;/ul&gt; <br> 
        * &lt;script&gt;
         * $("#treeView").ejTreeView();
        * // Create TreeView
        * var treeObj = $("#treeView").data("ejTreeView");
        * treeObj.hideNode($("#book")); // hideNode in  TreeView nodes
        * &lt;/script&gt;
          * @example 
        *  &lt;ul id="treeView"&gt;
        *   &lt;li&gt;Artwork
        *       &lt;ul&gt;
        *           &lt;li&gt;Abstract
        *               &lt;ul&gt;
        *                   &lt;li&gt;2 Acrylic Mediums&lt;/li&gt;
        *                   &lt;li&gt;Creative Acrylic&lt;/li&gt;
        *                   &lt;li&gt;Modern Painting&lt;/li&gt;
        *                   &lt;li&gt;Canvas Art&lt;/li&gt;
        *                   &lt;li&gt;Black white&lt;/li&gt;
        *               &lt;/ul&gt;
        *           &lt;/li&gt;
        *       &lt;/ul&gt;
        *   &lt;/li&gt;
        *   &lt;li id="book"&gt;Books
        *       &lt;ul&gt;
        *           &lt;li&gt;Entertaining&lt;/li&gt;
        *           &lt;li&gt;Design&lt;/li&gt;
        *       &lt;/ul&gt;
        *   &lt;/li&gt;
        *   &lt;li&gt;Music
        *       &lt;ul&gt;
        *           &lt;li&gt;Mass&lt;/li&gt;
        *           &lt;li&gt;Folk&lt;/li&gt;
        *       &lt;/ul&gt;
        *   &lt;/li&gt;
        * &lt;/ul&gt; <br> 
        * &lt;script&gt;
         * $("#treeView").ejTreeView();
        * // Create TreeView
        * $("#treeView").ejTreeView("hideNode",$("#book"));        
        * &lt;/script&gt;
        *@memberof ejTreeView
        * @instance
        */
        hideNode: function (node) { 
            if (!ej.isNullOrUndefined(node) && this.model.enabled) {
                node.css("visibility", "hidden");
            }
        },
               /**
        * To show nodes  in TreeView.
	    * @alias show
		* @return jQuery
        * @example 
        *  &lt;ul id="treeView"&gt;
        *  &lt;li&gt;Artwork
        *      &lt;ul&gt;
        *          &lt;li&gt;Abstract
        *              &lt;ul&gt;
        *                  &lt;li&gt;2 Acrylic Mediums&lt;/li&gt;
        *                  &lt;li&gt;Creative Acrylic&lt;/li&gt;
        *                  &lt;li&gt;Modern Painting&lt;/li&gt;
        *                  &lt;li&gt;Canvas Art&lt;/li&gt;
        *                  &lt;li&gt;Black white&lt;/li&gt;
        *              &lt;/ul&gt;
        *          &lt;/li&gt;
        *      &lt;/ul&gt;
        *  &lt;/li&gt;
        *  &lt;li&gt;Books
        *      &lt;ul&gt;
        *          &lt;li&gt;Entertaining&lt;/li&gt;
        *          &lt;li&gt;Design&lt;/li&gt;
        *      &lt;/ul&gt;
        *  &lt;/li&gt;
        *  &lt;li&gt;Music
        *      &lt;ul&gt;
        *          &lt;li&gt;Mass&lt;/li&gt;
        *          &lt;li&gt;Folk&lt;/li&gt;
        *      &lt;/ul&gt;
        *  &lt;/li&gt;
        * &lt;/ul&gt; <br> 
        * &lt;script&gt;
         * $("#treeView").ejTreeView();
        * // Create TreeView
        * var treeObj = $("#treeView").data("ejTreeView");
        * treeObj.show(); // show the TreeView nodes
        * &lt;/script&gt;
          * @example 
        *  &lt;ul id="treeView"&gt;
        *   &lt;li&gt;Artwork
        *       &lt;ul&gt;
        *           &lt;li&gt;Abstract
        *               &lt;ul&gt;
        *                   &lt;li&gt;2 Acrylic Mediums&lt;/li&gt;
        *                   &lt;li&gt;Creative Acrylic&lt;/li&gt;
        *                   &lt;li&gt;Modern Painting&lt;/li&gt;
        *                   &lt;li&gt;Canvas Art&lt;/li&gt;
        *                   &lt;li&gt;Black white&lt;/li&gt;
        *               &lt;/ul&gt;
        *           &lt;/li&gt;
        *       &lt;/ul&gt;
        *   &lt;/li&gt;
        *   &lt;li id="book"&gt;Books
        *       &lt;ul&gt;
        *           &lt;li&gt;Entertaining&lt;/li&gt;
        *           &lt;li&gt;Design&lt;/li&gt;
        *       &lt;/ul&gt;
        *   &lt;/li&gt;
        *   &lt;li&gt;Music
        *       &lt;ul&gt;
        *           &lt;li&gt;Mass&lt;/li&gt;
        *           &lt;li&gt;Folk&lt;/li&gt;
        *       &lt;/ul&gt;
        *   &lt;/li&gt;
        * &lt;/ul&gt; <br> 
        * &lt;script&gt;
         * $("#treeView").ejTreeView();
        * // Create TreeView
        * $("#treeView").ejTreeView("show");        
        * &lt;/script&gt;
        *@memberof ejTreeView
        * @instance
        */
        show: function () {
            /// <summary>To Show the TreeView control</summary>
            this.element.css("visibility", "");
        },
         /**
        * To show nodes  in TreeView.
		* @alias hide
		* @return jQuery
        * @example 
        *  &lt;ul id="treeView"&gt;
        *  &lt;li&gt;Artwork
        *      &lt;ul&gt;
        *          &lt;li&gt;Abstract
        *              &lt;ul&gt;
        *                  &lt;li&gt;2 Acrylic Mediums&lt;/li&gt;
        *                  &lt;li&gt;Creative Acrylic&lt;/li&gt;
        *                  &lt;li&gt;Modern Painting&lt;/li&gt;
        *                  &lt;li&gt;Canvas Art&lt;/li&gt;
        *                  &lt;li&gt;Black white&lt;/li&gt;
        *              &lt;/ul&gt;
        *          &lt;/li&gt;
        *      &lt;/ul&gt;
        *  &lt;/li&gt;
        *  &lt;li&gt;Books
        *      &lt;ul&gt;
        *          &lt;li&gt;Entertaining&lt;/li&gt;
        *          &lt;li&gt;Design&lt;/li&gt;
        *      &lt;/ul&gt;
        *  &lt;/li&gt;
        *  &lt;li&gt;Music
        *      &lt;ul&gt;
        *          &lt;li&gt;Mass&lt;/li&gt;
        *          &lt;li&gt;Folk&lt;/li&gt;
        *      &lt;/ul&gt;
        *  &lt;/li&gt;
        * &lt;/ul&gt; <br> 
        * &lt;script&gt;
         * $("#treeView").ejTreeView();
        * // Create TreeView
        * var treeObj = $("#treeView").data("ejTreeView");
        * treeObj.hide(); // hide the TreeView nodes
        * &lt;/script&gt;
          * @example 
        *  &lt;ul id="treeView"&gt;
        *   &lt;li&gt;Artwork
        *       &lt;ul&gt;
        *           &lt;li&gt;Abstract
        *               &lt;ul&gt;
        *                   &lt;li&gt;2 Acrylic Mediums&lt;/li&gt;
        *                   &lt;li&gt;Creative Acrylic&lt;/li&gt;
        *                   &lt;li&gt;Modern Painting&lt;/li&gt;
        *                   &lt;li&gt;Canvas Art&lt;/li&gt;
        *                   &lt;li&gt;Black white&lt;/li&gt;
        *               &lt;/ul&gt;
        *           &lt;/li&gt;
        *       &lt;/ul&gt;
        *   &lt;/li&gt;
        *   &lt;li id="book"&gt;Books
        *       &lt;ul&gt;
        *           &lt;li&gt;Entertaining&lt;/li&gt;
        *           &lt;li&gt;Design&lt;/li&gt;
        *       &lt;/ul&gt;
        *   &lt;/li&gt;
        *   &lt;li&gt;Music
        *       &lt;ul&gt;
        *           &lt;li&gt;Mass&lt;/li&gt;
        *           &lt;li&gt;Folk&lt;/li&gt;
        *       &lt;/ul&gt;
        *   &lt;/li&gt;
        * &lt;/ul&gt; <br> 
        * &lt;script&gt;
         * $("#treeView").ejTreeView();
        * // Create TreeView
        * $("#treeView").ejTreeView("hide");        
        * &lt;/script&gt;
        *@memberof ejTreeView
        * @instance
        */
        hide: function () {
            /// <summary>To Hide the TreeView control</summary>
            this.element.css("visibility", "hidden");
        },
        /**
        * To retrive the checked status of the given TreeView node.
		* @alias isNodeChecked
		* @return jQuery
		* @param {Object} element  object of Tree view node.
        * @example 
        * &lt;ul id="treeView"&gt;
        *  &lt;li&gt;Artwork
        *      &lt;ul&gt;
        *          &lt;li&gt;Abstract
        *              &lt;ul&gt;
        *                  &lt;li&gt;2 Acrylic Mediums&lt;/li&gt;
        *                  &lt;li&gt;Creative Acrylic&lt;/li&gt;
        *                  &lt;li&gt;Modern Painting&lt;/li&gt;
        *                  &lt;li&gt;Canvas Art&lt;/li&gt;
        *                  &lt;li&gt;Black white&lt;/li&gt;
        *              &lt;/ul&gt;
        *          &lt;/li&gt;
        *      &lt;/ul&gt;
        *  &lt;/li&gt;
        *  &lt;li id="book"&gt;Books
        *      &lt;ul&gt;
        *          &lt;li&gt;Entertaining&lt;/li&gt;
        *          &lt;li&gt;Design&lt;/li&gt;
        *      &lt;/ul&gt;
        *  &lt;/li&gt;
        *  &lt;li&gt;Music
        *      &lt;ul&gt;
        *          &lt;li&gt;Mass&lt;/li&gt;
        *          &lt;li&gt;Folk&lt;/li&gt;
        *      &lt;/ul&gt;
        *  &lt;/li&gt;
        * &lt;/ul&gt; <br> 
        * &lt;script&gt;
         * $("#treeView").ejTreeView({showCheckbox:true});
        * // Create TreeView
        * var treeObj = $("#treeView").data("ejTreeView");
        * treeObj.isNodeChecked($("#book")); // isNodeChecked the TreeView nodes
        * &lt;/script&gt;
          * @example 
        *  &lt;ul id="treeView"&gt;
        *   &lt;li&gt;Artwork
        *       &lt;ul&gt;
        *           &lt;li&gt;Abstract
        *               &lt;ul&gt;
        *                   &lt;li&gt;2 Acrylic Mediums&lt;/li&gt;
        *                   &lt;li&gt;Creative Acrylic&lt;/li&gt;
        *                   &lt;li&gt;Modern Painting&lt;/li&gt;
        *                   &lt;li&gt;Canvas Art&lt;/li&gt;
        *                   &lt;li&gt;Black white&lt;/li&gt;
        *               &lt;/ul&gt;
        *           &lt;/li&gt;
        *       &lt;/ul&gt;
        *   &lt;/li&gt;
        *   &lt;li id="book"&gt;Books
        *       &lt;ul&gt;
        *           &lt;li&gt;Entertaining&lt;/li&gt;
        *           &lt;li&gt;Design&lt;/li&gt;
        *       &lt;/ul&gt;
        *   &lt;/li&gt;
        *   &lt;li&gt;Music
        *       &lt;ul&gt;
        *           &lt;li&gt;Mass&lt;/li&gt;
        *           &lt;li&gt;Folk&lt;/li&gt;
        *       &lt;/ul&gt;
        *   &lt;/li&gt;
        * &lt;/ul&gt; <br> 
        * &lt;script&gt;
         * $("#treeView").ejTreeView({showCheckbox:true});
        * // Create TreeView
        * $("#treeView").ejTreeView("isNodeChecked",$("#book"));        
        * &lt;/script&gt;
        * @memberof ejTreeView
        * @instance
        */
        isNodeChecked: function (element) {
            return this._isChecked($(element).find('input.nodecheckbox')[0]);
        },
        /**
        * To retrive the expand status of the given TreeView node.
		* @alias isExpanded
		* @return jQuery
		* @param {Object} element  object of Tree view node.
        * @example 
        *   &lt;ul id="treeView"&gt;
        *  &lt;li&gt;Artwork
        *      &lt;ul&gt;
        *          &lt;li&gt;Abstract
        *              &lt;ul&gt;
        *                  &lt;li&gt;2 Acrylic Mediums&lt;/li&gt;
        *                  &lt;li&gt;Creative Acrylic&lt;/li&gt;
        *                  &lt;li&gt;Modern Painting&lt;/li&gt;
        *                  &lt;li&gt;Canvas Art&lt;/li&gt;
        *                 &lt;li&gt;Black white&lt;/li&gt;
        *              &lt;/ul&gt;
        *          &lt;/li&gt;
        *      &lt;/ul&gt;
        *  &lt;/li&gt;
        *  &lt;li id="book"&gt;Books
        *      &lt;ul&gt;
        *          &lt;li&gt;Entertaining&lt;/li&gt;
        *          &lt;li&gt;Design&lt;/li&gt;
        *      &lt;/ul&gt;
        *  &lt;/li&gt;
        *  &lt;li&gt;Music
        *      &lt;ul&gt;
        *          &lt;li&gt;Mass&lt;/li&gt;
        *          &lt;li&gt;Folk&lt;/li&gt;
        *      &lt;/ul&gt;
        *  &lt;/li&gt;
        * &lt;/ul&gt; <br> 
        * &lt;script&gt;
         * $("#treeView").ejTreeView();
        * // Create TreeView
        * var treeObj = $("#treeView").data("ejTreeView");
        * treeObj.isExpanded($("#book")); // isExpanded the TreeView nodes
        * &lt;/script&gt;
          * @example 
        *  &lt;ul id="treeView"&gt;
        *   &lt;li&gt;Artwork
        *       &lt;ul&gt;
        *           &lt;li&gt;Abstract
        *               &lt;ul&gt;
        *                   &lt;li&gt;2 Acrylic Mediums&lt;/li&gt;
        *                   &lt;li&gt;Creative Acrylic&lt;/li&gt;
        *                   &lt;li&gt;Modern Painting&lt;/li&gt;
        *                   &lt;li&gt;Canvas Art&lt;/li&gt;
        *                   &lt;li&gt;Black white&lt;/li&gt;
        *               &lt;/ul&gt;
        *           &lt;/li&gt;
        *       &lt;/ul&gt;
        *   &lt;/li&gt;
        *   &lt;li id="book"&gt;Books
        *       &lt;ul&gt;
        *           &lt;li&gt;Entertaining&lt;/li&gt;
        *           &lt;li&gt;Design&lt;/li&gt;
        *       &lt;/ul&gt;
        *   &lt;/li&gt;
        *   &lt;li&gt;Music
        *       &lt;ul&gt;
        *           &lt;li&gt;Mass&lt;/li&gt;
        *           &lt;li&gt;Folk&lt;/li&gt;
        *       &lt;/ul&gt;
        *   &lt;/li&gt;
        * &lt;/ul&gt; <br> 
        * &lt;script&gt;
         * $("#treeView").ejTreeView();
        * // Create TreeView
        * $("#treeView").ejTreeView("isExpanded",$("#book"));        
        * &lt;/script&gt;
        * @memberof ejTreeView
        * @instance
        */
        isExpanded: function (element) {
            return this._isExpanded(element);
        },
       
        _isNodeExpanded: function (element) {
            if (!ej.isNullOrUndefined(element)) {
                var expandUl = $(element).children().filter('ul');
                var isExpanded = $(expandUl).is(':not(:hidden)');
                return isExpanded;
               
            }
        },
       /**
        * To retrive the  status of the collection of childs for the given TreeView node.
		* @alias hasChildNode
		* @return jQuery
		* @param {Object} element  object of element
		* @example 
        * &lt;ul id="treeView"&gt;
         * &lt;li&gt;Artwork
         *     &lt;ul&gt;
         *         &lt;li&gt;Abstract
         *             &lt;ul&gt;
         *                 &lt;li&gt;2 Acrylic Mediums&lt;/li&gt;
         *                &lt;li&gt;Creative Acrylic&lt;/li&gt;
         *                 &lt;li&gt;Modern Painting&lt;/li&gt;
         *              &lt;li&gt;Canvas Art&lt;/li&gt;
         *              &lt;li&gt;Black white&lt;/li&gt;
         *           &lt;/ul&gt;
         *        &lt;/li&gt;
         *    &lt;/ul&gt;
         * &lt;/li&gt;
         * &lt;li id="book"&gt;Books
         *    &lt;ul&gt;
         *         &lt;li&gt;Entertaining&lt;/li&gt;
         *        &lt;li&gt;Design&lt;/li&gt;
         *     &lt;/ul&gt;
         * &lt;/li&gt;
         * &lt;li&gt;Music
         *     &lt;ul&gt;
         *         &lt;li&gt;Mass&lt;/li&gt;
         *         &lt;li&gt;Folk&lt;/li&gt;
        *      &lt;/ul&gt;
        * &lt;/li&gt;
        * &lt;/ul&gt; <br> 
        * &lt;script&gt;
         * $("#treeView").ejTreeView();
         * // Create TreeView
        * var treeObj = $("#treeView").data("ejTreeView");
        * treeObj.hasChildNode($("#book")); // hasChildNode the TreeView nodes
        * &lt;/script&gt;
          * @example 
        *  &lt;ul id="treeView"&gt;
        *   &lt;li&gt;Artwork
        *       &lt;ul&gt;
        *           &lt;li&gt;Abstract
        *               &lt;ul&gt;
        *                   &lt;li&gt;2 Acrylic Mediums&lt;/li&gt;
        *                   &lt;li&gt;Creative Acrylic&lt;/li&gt;
        *                   &lt;li&gt;Modern Painting&lt;/li&gt;
        *                   &lt;li&gt;Canvas Art&lt;/li&gt;
        *                   &lt;li&gt;Black white&lt;/li&gt;
        *               &lt;/ul&gt;
        *           &lt;/li&gt;
        *       &lt;/ul&gt;
        *   &lt;/li&gt;
        *   &lt;li id="book"&gt;Books
        *       &lt;ul&gt;
        *           &lt;li&gt;Entertaining&lt;/li&gt;
        *           &lt;li&gt;Design&lt;/li&gt;
        *       &lt;/ul&gt;
        *   &lt;/li&gt;
        *   &lt;li&gt;Music
        *       &lt;ul&gt;
        *           &lt;li&gt;Mass&lt;/li&gt;
        *           &lt;li&gt;Folk&lt;/li&gt;
        *       &lt;/ul&gt;
        *   &lt;/li&gt;
        * &lt;/ul&gt; <br> 
        * &lt;script&gt;
         * $("#treeView").ejTreeView();
        * // Create TreeView
        * $("#treeView").ejTreeView("hasChildNode",$("#book"));        
        * &lt;/script&gt;
        *@memberof ejTreeView
        * @instance
        */
        hasChildNode: function (element) {
            var child = false;
            if (!ej.isNullOrUndefined(element)) {
                child = $(element).children().length < 2 ? false : true;
            }
            return child;
        },
        //--------------------- Client side events ----------------------//

       	_onNodeExpand: function (data) {
                this._trigger("nodeExpand", data);            
        },
        _onNodeCollapse: function (data) {
                this._trigger("nodeCollapse", data);            
        },
        _onBeforeExpand: function (data) {
                return this._trigger("beforeExpand", data);            
        },
        _onBeforeCollapse: function (data) {
                return this._trigger("beforeCollapse", data);            
        },
        _onNodeSelect: function (data) {
                this._trigger("nodeSelect", data);            
        },
        _onChecked: function (element,args) {
            var liElement = $(element).parents("li").first();
            var nodeText = liElement.find(".e-text").first().text();
			this._CurrenctSelectedNodes.push(nodeText);
            var isChecked = this._isChecked(element);
            var evt = !ej.isNullOrUndefined(args) ? !ej.isNullOrUndefined(args.event) ? args.event : "" : "";
            var checknode = !ej.isNullOrUndefined(args) ? !ej.isNullOrUndefined(args.checknode) ? args.checknode : "" : "";
            this._addCheckNodes(this._liList.index(liElement));
            var data = { currentElement: liElement, value: nodeText, currentNode:this._CurrenctSelectedNodes, currentCheckedNodes:checknode, isChecked: isChecked, event: evt };
            if (this._isRender) this._trigger("nodeCheck", data);
        },
        _onUnChecked: function (element,args) {
            var liElement = $(element).parents("li").first();
			var nodeText = liElement.find(".e-text").first().text();
			var isUnChecked = this._isChecked(element);
			var evt = !ej.isNullOrUndefined(args) ? !ej.isNullOrUndefined(args.event) ? args.event : "" : "";
            var unchecknode = !ej.isNullOrUndefined(args) ? !ej.isNullOrUndefined(args.unchecknode) ? args.unchecknode : "" : "";
			this._removeCheckNodes(this._liList.index(liElement));
			var parent = liElement.parents("li").first();
			if (!liElement.parent().parent().is(this.element) && parent.length != 0) this._removeCheckNodes(this._liList.index(parent));
			var data = { currentElement: liElement, value: nodeText, currentNode: nodeText, currentUncheckedNodes: unchecknode, isChecked: isUnChecked, event: evt };
			this._trigger("nodeUncheck", data);
        },
        _addCheckNodes: function (item) {
            var checkedArray = this.model.checkedNodes;
            this._removeNullInArray(checkedArray);
            !checkedArray instanceof Array && (checkedArray = []);
            if (checkedArray.indexOf(item) == -1) checkedArray.push(item);
        },
        _removeCheckNodes: function (item) {
            var checkedArray = this.model.checkedNodes;
            !checkedArray instanceof Array && (checkedArray = []);
            var i = checkedArray.indexOf(item);
            if (i != -1) {
                checkedArray.splice(i, 1);
                checkedArray.length == 0 && (checkedArray.push(-1));
            }
        },
        _removeNullInArray: function (array) {
            var i = array.indexOf(-1);
            if (i != -1) array.splice(i, 1);
        },
        _onDragStarts: function (data) {
                return this._trigger("nodeDragStart", data);            
        },
        _onDrag: function (data) {
                return this._trigger("nodeDrag", data);            
        },
        _onDragStop: function (data) {
                return this._trigger("nodeDragStop", data);            
        },
        _onDropped: function (data) {
                return this._trigger("nodeDropped", data);            
        },
        _onClick: function (data) {
            this._trigger("nodeClick", data);
        }
    });

    //mentions the recursive type of child
    //ej.TreeView.prototype.defaults.fields.child = ej.TreeView.prototype.defaults.fields;
    //ej.TreeView.prototype.dataTypes.fields.child = ej.TreeView.prototype.dataTypes.fields;

})(jQuery, Syncfusion);;
/*!
*  filename: ej.uploadbox.js
*  version : 12.1
*  Copyright Syncfusion Inc. 2001 - 2013. All rights reserved.
*  Use of this code is subject to the terms of our license.
*  A copy of the current license can be obtained at any time by e-mailing
*  licensing@syncfusion.com. Any infringement will be prosecuted under
*  applicable laws. 
*/

/**
* @fileOverview Plugin to style the Html Button elements
* @copyright Copyright Syncfusion Inc. 2001 - 2013. All rights reserved.
*  Use of this code is subject to the terms of our license.
*  A copy of the current license can be obtained at any time by e-mailing
*  licensing@syncfusion.com. Any infringement will be prosecuted under
*  applicable laws. 
* @version 12.1 
* @author <a href="mailto:licensing@syncfusion.com">Syncfusion Inc</a>
*/
/*global _comma_separated_list_of_variables_*/

(function ($, ej, undefined) {
    /**
* @namespace ej
* @class ejUploadbox
* @requires jQuery
* @requires jquery.easing.1.3.min.js
* @requires ej.core.js
* @requires ej.uploadbox.js
* @requires ej.dialog.js
* @requires ej.scroller.js
* @requires ej.draggable.js

* @classdesc Custom Design for Html Uploadbox control.
* @example 
* &lt;div id="uploadbox1"&gt;&lt;/div&gt; <br> 
* &lt;script&gt;
* // Create Uploadbox
* $('#uploadbox1').ejUploadbox(); 	
* &lt;/script&gt;
*/
    // Example plugin creation code
    // ejSample is the plugin name 
    // "ej.Sample" is "namespace.className" will hold functions and properties

    ej.widget("ejUploadbox", "ej.Uploadbox", {
        // widget element will be automatically set in this
        element: null,

        // user defined model will be automatically set in this
        model: null,
        validTags: ["div", "span"],
        _setFirst: false,
        //Root Css Class
        _rootCSS: "e-uploadbox",

        // default model
        defaults: {
            /// <summary>This Contains default property of upload button </summary>
            /**		
* Specifies the text content for Uploadbox browse button while initialization.
* @default "Browse"
* @type {string}
* @example 
* &lt;div id="uploadbox1"&gt;&lt;/div&gt; <br> 
* &lt;script&gt;
* //To set  browseButtonText API value 
* 	$("#uploadbox1").ejUploadbox({ buttonText:{ browse:"Choose File", upload:"Upload the File", cancel:"Cancel the Upload"}});	
* &lt;/script&gt;
* @memberof ejUploadbox
* @instance
*/
            buttonText: {
                /**		
                * Sets the text for the Browse button.
                * @alias ejUploadbox#buttonText->browse
                * @type String
                * @example 
                * &lt;div id="uploadbox1"&gt;&lt;/div&gt; <br> 
                * &lt;script&gt;
                * //To set buttonText API during initialization  
                * 	$("#uploadbox1").ejUploadbox({ buttonText: { browse: "Choose File" }});
                * &lt;/script&gt;
                * @memberof ejUploadbox#buttonText
                * @instance
                */
                browse: "Browse",
                /**		
                * Sets the text for the Upload button inside the dialog popup.
                * @alias ejUploadbox#buttonText->upload
                * @type String
                * @example 
                * &lt;div id="uploadbox1"&gt;&lt;/div&gt; <br> 
                * &lt;script&gt;
                * //To set buttonText API during initialization  
                * 	$("#uploadbox1").ejUploadbox({ buttonText: { upload: "Upload the File" }});
                * &lt;/script&gt;
                * @memberof ejUploadbox#buttonText
                * @instance
                */
                upload: "Upload",
                /**		
                * Sets the text for the Cancel button inside the dialog popup.
                * @alias ejUploadbox#buttonText->cancel
                * @type String
                * @example 
                * &lt;div id="uploadbox1"&gt;&lt;/div&gt; <br> 
                * &lt;script&gt;
                * //To set buttonText API during initialization  
                * 	$("#uploadbox1").ejUploadbox({ buttonText: { cancel: "Cancel the Upload" }});
                * &lt;/script&gt;
                * @memberof ejUploadbox#buttonText
                * @instance
                */
                cancel: "Cancel",
            },
            /**
            * Sets the text for the inside the dialog popup.
            * @default UploadBox
            * @type {string}
            * @example 
            * &lt;div id="uploadbox1"&gt;&lt;/div&gt; <br> 
            * &lt;script&gt;
            * //To set  dialogText API value during initialization
            * 	$("#uploadbox1").ejUploadbox({ dialogText:{title:"Upload File List",name:"File Name",size:"File Size",status:"File Status" }});
            * &lt;/script&gt;
            * @memberof ejUploadbox
            * @instance
            */
            dialogText: {
                /**		
                * Sets the title text dialog popup.
                * @alias ejUploadbox#dialogText->cancel
                * @type String
                * @example 
                * &lt;div id="uploadbox1"&gt;&lt;/div&gt; <br> 
                * &lt;script&gt;
                * //To set buttonText API during initialization  
                * 	$("#uploadbox1").ejUploadbox({ dialogText: { title: "Upload Box" }});
                * &lt;/script&gt;
                * @memberof ejUploadbox#dialogText
                * @instance
                */
                title: "Upload Box",
                /**		
                * Sets the Name text for the dialog popup.
                * @alias ejUploadbox#dialogText->cancel
                * @type String
                * @example 
                * &lt;div id="uploadbox1"&gt;&lt;/div&gt; <br> 
                * &lt;script&gt;
                * //To set buttonText API during initialization  
                * 	$("#uploadbox1").ejUploadbox({ dialogText: { name: "Name" }});
                * &lt;/script&gt;
                * @memberof ejUploadbox#dialogText
                * @instance
                */
                name: "Name",
                /**		
                * Sets the Size text for dialog popup.
                * @alias ejUploadbox#dialogText->cancel
                * @type String
                * @example 
                * &lt;div id="uploadbox1"&gt;&lt;/div&gt; <br> 
                * &lt;script&gt;
                * //To set buttonText API during initialization  
                * 	$("#uploadbox1").ejUploadbox({ dialogText: { size: "Size" }});
                * &lt;/script&gt;
                * @memberof ejUploadbox#dialogText
                * @instance
                */
                size: "Size",
                /**		
                * Sets the Status text for dialog popup.
                * @alias ejUploadbox#dialogText->cancel
                * @type String
                * @example 
                * &lt;div id="uploadbox1"&gt;&lt;/div&gt; <br> 
                * &lt;script&gt;
                * //To set buttonText API during initialization  
                * 	$("#uploadbox1").ejUploadbox({ dialogText: { status: "Status" }});
                * &lt;/script&gt;
                * @memberof ejUploadbox#dialogText
                * @instance
                */
                status: "Status"
            },
            /**		
* Sets the culture in uploadbox. 
* @default en-US
* @type {string}
* @example 
* &lt;div id="uploadbox1"&gt;&lt;/div&gt; <br> 
* &lt;script&gt;
* //To set  asyncUpload API value during initialization
* 	$("#uploadbox1").ejUploadbox({ locale: vi-VN });	
* &lt;/script&gt;
* @memberof ejUploadbox
* @instance
*/
            locale: "en-US",
            /**		
* Uploadbox supports both synchronous and asynchronous upload. This can be achieved by this property asyncUpload.
* @default true
* @type {boolean}
* @example 
* &lt;div id="uploadbox1"&gt;&lt;/div&gt; <br> 
* &lt;script&gt;
* //To set  asyncUpload API value during initialization
* 	$("#uploadbox1").ejUploadbox({ asyncUpload: false });	
 
* &lt;/script&gt;
* @memberof ejUploadbox
* @instance
*/
            asyncUpload: true,
            /**		
* Enables or disables Uploadbox based on boolean value.
* @default true
* @type {boolean}
* @example 
* &lt;div id="uploadbox1"&gt;&lt;/div&gt; <br> 
* &lt;script&gt;
* //To set enabled API value during initialization
* 	$("#uploadbox1").ejUploadbox({ enabled: false });	

* &lt;/script&gt;
* @memberof ejUploadbox
* @instance
*/
            enabled: true,
				/**		
				* Enables to select multiple files
				* @default false
				* @type {boolean}
				* @example 
				* &lt;div id="uploadbox1"&gt;&lt;/div&gt; <br> 
				* &lt;script&gt;
				* //To set multipleFilesSelection API value during initialization
				* 	$("#uploadbox1").ejUploadbox({ multipleFilesSelection: false });
				* &lt;/script&gt;	
				* @memberof ejUploadbox
				* @instance
				*/
            	
            multipleFilesSelection: false,
            /**		
* Uploadbox supports auto uploading of files after the file selection is done.The auto upload is performed if autoUpload is set to true.   
* @default false
* @type {boolean}
* @example 
* &lt;div id="uploadbox1"&gt;&lt;/div&gt; <br> 
* &lt;script&gt;
* //To set autoUpload API value during initialization
* 	$("#uploadbox1").ejUploadbox({ autoUpload: true });	

* &lt;/script&gt;
* @memberof ejUploadbox
* @instance
*/
            autoUpload: false,
            /**		
* Specifies to display the file details of files while selected for uploading can be done when showFileDetails set to true.  
* @default true
* @type {boolean}
* @example 
* &lt;div id="uploadbox1"&gt;&lt;/div&gt; <br> 
* &lt;script&gt;
* //To set showFileDetails API value during initialization
* 	$("#uploadbox1").ejUploadbox({ showFileDetails: false });	

* &lt;/script&gt;
* @memberof ejUploadbox
* @instance
*/
            showFileDetails: true,
				/**		
				* Specifies the file extension to allow for uploading the file while initialization. This could be mentioned in string format.  
				* @default ""
				* @type {string}
				* @example 
				* &lt;div id="uploadbox1"&gt;&lt;/div&gt; <br> 
				* &lt;script&gt;
				* //To set extensionsAllow API value during initialization
				* 	$("#uploadbox1").ejUploadbox({ extensionsAllow: ".zip"  });	
				* &lt;/script&gt;	
				* @memberof ejUploadbox
				* @instance
				*/

            extensionsAllow: "",
                /**		
				* Specifies the file extension to deny for uploading the file while initialization. This could be mentioned in string format.  
				* @default ""
				* @type {string}
				* @example 
				* &lt;div id="uploadbox1"&gt;&lt;/div&gt; <br> 
				* &lt;script&gt;
				* //To set extensionsDeny API value during initialization
				* 	$("#uploadbox1").ejUploadbox({ extensionsDeny: ".zip"  });	
				* &lt;/script&gt;	
				* @memberof ejUploadbox
				* @instance
				*/
            extensionsDeny: "",
            /**		
* Specifies the save action which has to be performed after the file is pushed for uploading. Here we have to mention the server address which has to perform this action.  
* @default ""
* @type {string}
* @example 
* &lt;div id="uploadbox1"&gt;&lt;/div&gt; <br> 
* &lt;script&gt;
* //To set saveUrl API value during initialization
* 	$("#uploadbox1").ejUploadbox({ saveUrl: "http://js.syncfusion.com/demos/beta/saveFiles.ashx"});	

* &lt;/script&gt;
* @memberof ejUploadbox
* @instance
*/
            saveUrl: "",
            /**		
* Specifies the remove action which has to be performed after the file uploading is completed. Here we have to mention the server address which has to perform this action.  
* @default ""
* @type {string}
* @example 
* &lt;div id="uploadbox1"&gt;&lt;/div&gt; <br> 
* &lt;script&gt;
* //To set removeUrl API value during initialization
* 	$("#uploadbox1").ejUploadbox({ removeUrl: "http://js.syncfusion.com/demos/beta/removeFiles.ashx"});	

* &lt;/script&gt;
* @memberof ejUploadbox
* @instance
*/
            removeUrl: "",
				/**		
				* Set the root class for Uploadbox control theme. This cssClass  API helps to use custom skinning option for Uploadbox control.   
				* @default ""
				* @type {string}
				* @example 
				* &lt;div id="uploadbox1"&gt;&lt;/div&gt; <br> 
				* &lt;script&gt;
				* //To set cssClass  API value during initialization
				* 	$("#uploadbox1").ejUploadbox({ cssClass : "gradient-lime"});	
				* &lt;/script&gt;	
				* @memberof ejUploadbox
				* @instance
				*/

            cssClass : "",
                /**		
				* Specifies the Right to Left direction property for Uploadbox control.   
				* @default false
				* @type {boolean}
				* @example 
				* &lt;div id="uploadbox1"&gt;&lt;/div&gt; <br> 
				* &lt;script&gt;
				* //To set enableRTL API value during initialization
				* 	$("#uploadbox1").ejUploadbox({ enableRTL: true});	
				* &lt;/script&gt;	
				* @memberof ejUploadbox
				* @instance
				*/
            enableRTL: false,
            /**     
* Fires when Uploadbox control is created.
* @event
* @name ejUploadbox#create	
* @param {Object} argument Event parameters from Uploadbox     
* @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
* @param {object}  argument.model returns the Uploadbox model
* @param {string}  argument.type returns the name of the event.
* @example 
* &lt;div id="uploadbox1"&gt;&lt;/div&gt; <br> 
* &lt;script&gt;
* //create event for Uploadbox
* $("#uploadbox1").ejUploadbox({
*    create: function (args) {}
* });   
 
* &lt;/script&gt;   
* @memberof ejUploadbox
* @instance
*/
            create: null,
            /**     
* Fires when the file is selected for upload successfully.
* @event
* @name ejUploadbox#fileSelect	
* @param {Object} argument Event parameters from Uploadbox     
* @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
* @param {object}  argument.model returns the Uploadbox model
* @param {string}  argument.type returns the name of the event.
* @example
* &lt;div id="uploadbox1"&gt;&lt;/div&gt; <br> 
* &lt;script&gt; 
* //fileSelect event for Uploadbox
* $("#uploadbox1").ejUploadbox({
*    fileSelect: function (args) {}
* });  
 
* &lt;/script&gt;    
* @memberof ejUploadbox
* @instance
*/
            fileSelect: null,
            /**     
* Fires when the upload progress begins.
* @event
* @name ejUploadbox#begin	
* @param {Object} argument Event parameters from Uploadbox     
* @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
* @param {object}  argument.model returns the Uploadbox model
* @param {string}  argument.type returns the name of the event.
* @example 
* &lt;div id="uploadbox1"&gt;&lt;/div&gt; <br> 
* &lt;script&gt;
* //begin event for Uploadbox
* $("#uploadbox1").ejUploadbox({
*    begin: function (args) {}
* });   
 
* &lt;/script&gt;   
* @memberof ejUploadbox
* @instance
*/
            begin: null,
            /**     
* Fires when the upload progress is cancelled.
* @event
* @name ejUploadbox#cancel	
* @param {Object} argument Event parameters from Uploadbox     
* @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
* @param {object}  argument.model returns the Uploadbox model
* @param {string}  argument.type returns the name of the event.
* @example 
* &lt;div id="uploadbox1"&gt;&lt;/div&gt; <br> 
* &lt;script&gt;
* //cancel event for Uploadbox
* $("#uploadbox1").ejUploadbox({
*    cancel: function (args) {}
* });

* &lt;/script&gt;     
* @memberof ejUploadbox
* @instance
*/
            cancel: null,
            /**     
* Fires when the file upload progress is completed.
* @event
* @name ejUploadbox#complete	
* @param {Object} argument Event parameters from Uploadbox     
* @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
* @param {object}  argument.model returns the Uploadbox model
* @param {string}  argument.type returns the name of the event.
* @param {object}  argument.files returns file entries and its details
* @example 
* &lt;div id="uploadbox1"&gt;&lt;/div&gt; <br> 
* &lt;script&gt;
* //complete event for Uploadbox
* $("#uploadbox1").ejUploadbox({
*    complete: function (args) {}
* });   

* &lt;/script&gt;   
* @memberof ejUploadbox
* @instance
*/
            complete: null,
            /**     
* Fires when the uploaded file is removed successfully.
* @event
* @name ejUploadbox#remove	
* @param {Object} argument Event parameters from Uploadbox     
* @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
* @param {object}  argument.model returns the Uploadbox model
* @param {string}  argument.type returns the name of the event.
* @param {object}  argument.status returns the file details of the file object
* @example 
* &lt;div id="uploadbox1"&gt;&lt;/div&gt; <br> 
* &lt;script&gt;
* //remove event for Uploadbox
* $("#uploadbox1").ejUploadbox({
*    remove: function (args) {}
* });  
 
* &lt;/script&gt;    
* @memberof ejUploadbox
* @instance
*/
            remove: null,
            /**     
* Fires when the Upload process ends in Error.
* @event
* @name ejUploadbox#error	
* @param {Object} argument Event parameters from Uploadbox     
* @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
* @param {object}  argument.model returns the Uploadbox model
* @param {string}  argument.type returns the name of the event.
* @param {string}  argument.action returns the action name
* @param {object}  argument.files returns the file details of the file uploaded
* @example 
* &lt;div id="uploadbox1"&gt;&lt;/div&gt; <br> 
* &lt;script&gt;
* //error event for Uploadbox
* $("#uploadbox1").ejUploadbox({
*    error: function (args) {}
* });  

* &lt;/script&gt;    
* @memberof ejUploadbox
* @instance
*/
            error: null,
            /**     
* Fires when Uploadbox control is destroyed.
* @event
* @name ejUploadbox#destroy	
* @param {Object} argument Event parameters from Uploadbox     
* @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
* @param {object}  argument.model returns the Uploadbox model
* @param {string}  argument.type returns the name of the event.
* @example 
* &lt;div id="uploadbox1"&gt;&lt;/div&gt; <br> 
* &lt;script&gt;
* //destroy event for Uploadbox
* $("#uploadbox1").ejUploadbox({
*    destroy: function (args) {}
* });  

* &lt;/script&gt;    
* @memberof ejUploadbox
* @instance
*/
            destroy: null
        },
        /**
* Specify the data types for default properties 
* @private
*/
        //Data Types
        dataTypes: {
            buttonText: "data",
            dialogText: "data",
            disbled: "boolean",
            multipleFilesSelection: "boolean",
            autoUpload: "boolean",
            showFileDetails: "boolean",
            extensionsAllow: "string",
            extensionsDeny: "string",
            saveUrl: "string",
            removeUrl: "string",
            cssClass : "string",
            enableRTL: "boolean"
        },
        // sample public function
        //Disables the upload box
        /**
* To disable the Uploadbox control  
* @return jQuery
* @example 
* &lt;div id="uploadbox1"&gt;&lt;/div&gt; <br> 
* &lt;script&gt;
* // Disable Uploadbox
* $("#uploadbox1").ejUploadbox();
* var uploadObj = $("#uploadbox1").data("ejUploadbox");
* uploadObj.disable(); // disable the Uploadbox
* &lt;/script&gt;
* @example 
* &lt;div id="uploadbox1"&gt;&lt;/div&gt; <br> 
* &lt;script&gt;
* // disable the Uploadbox
* $("#uploadbox1").ejUploadbox();
* $("#uploadbox1").ejUploadbox("disable");	
* &lt;/script&gt;
*@memberof ejUploadbox
* @instance
*/
        disable: function () {
            /// <summary>This will disable the upload button </summary>
            this.element.addClass("e-disable");
            this.model.enabled = false;
        },
        /**
* To enable the Uploadbox control  
* @return jQuery
* @example 
* &lt;div id="uploadbox1"&gt;&lt;/div&gt; <br> 
* &lt;script&gt;
* // enable Uploadbox
* $("#uploadbox1").ejUploadbox();
* var uploadObj = $("#uploadbox1").data("ejUploadbox");
* uploadObj.enable(); // enable the Uploadbox
* &lt;/script&gt;
* @example 
* &lt;div id="uploadbox1"&gt;&lt;/div&gt; <br> 
* &lt;script&gt;
* // enable the Uploadbox
* $("#uploadbox1").ejUploadbox();
* $("#uploadbox1").ejUploadbox("enable");	
* &lt;/script&gt;
*@memberof ejUploadbox
* @instance
*/
        enable: function () {
            /// <summary>This will enable upload button </summary>
            if (this.element.hasClass("e-disable")) {
                this.element.removeClass("e-disable");
                this.model.enabled = true;
            }
        },
        // constructor function
        /**
* Create the Uploadbox widget
* @private
*/
        _init: function () {
            /// <summary>This will initialize upload button </summary>
            this.s = ej.browserInfo();
            this._initialize();
            this._wireEvents();
            /*Sync Uploads*/
            if (!this.model.asyncUpload) {
                this._initObjectsSyncUpload();
            }
            //disable status
            this._controlStatus(this.model.enabled);
            this._buttonText(this.model.buttonText);
        },
        /**
* To configure the properties at runtime using SetModel		
* @private
*/
        _setModel: function (options) {
            if (!(this.model.enabled) && ej.isNullOrUndefined(options["enabled"])) return false;
            var option;
            for (option in options) {
                switch (option) {
                    case "buttonText":
                        this._buttonText(options[option]);
                        break;
                    case "dialogText":
                        this._dialogText(options[option]);
                        break;
                    case "cssClass ":
                        this._setSkin(options[option]);
                        break;
                    case "enableRTL":
                        this._setRTL(options[option]);
                        break;
                    case "enabled":
                        this._controlStatus(options[option]);
                        break;
                    case "locale":
                        this.model.locale = options[option];
                        this._buttonText(ej.Uploadbox.Locale[this.model.locale].buttonText);
                        this._dialogText(ej.Uploadbox.Locale[this.model.locale].dialogText);
                        break;
                }
            }
        },
        /**
* To ensure the status of the control;either enabled or disabled	
* @private
*/
        _controlStatus: function (value) {
            //disable status
            value != true ? this.disable() : this.enable();
        },
        /**
* To set rtl property if enabled	
* @private
*/
        _setRTL: function (val) {
            if (val) {
                this.element.addClass("e-rtl");
                if (this.s.name == "msie") {
                    this.inputupload.css('right', '-153px');
                }
            } else {
                this.element.removeClass("e-rtl");
                if (this.s.name == "msie") {
                    this.inputupload.css('right', '0px');
                }
            }
            if (this.updialog) this.updialog.ejDialog({ enableRTL: val });
        },
        _getLocalizedLabels: function (property) {
            var textType;
            if (property == "browse" || property == "upload" || property == "cancel")
                textType = "buttonText";
            else
                textType = "dialogText";
            return (ej.Uploadbox.Locale[this.model.locale] === undefined || ej.Uploadbox.Locale[this.model.locale][property] === undefined) ?
            ej.Uploadbox.Locale[this.model.locale][textType][property] :
            ej.Uploadbox.Locale[this.model.locale][property];

        },
        /**
* To set the Display Text to Buttons.
* @private
*/
        _buttonText: function (data) {
            $.extend(this.model.buttonText, data);
            this.buttondiv.val(this.model.buttonText.browse);
            this.element.find(".e-file-upload .e-uploadbtn").html(this.model.buttonText.upload);
            this.element.find(".e-file-upload .e-uploadclosebtn").html(this.model.buttonText.cancel);
        },
        /**
* To set the Text to Dialog popup.
* @private
*/
        _dialogText: function (data) {
            $.extend(this.model.dialogText, data);
            if (!(this.diaObj == undefined))
                this.diaObj.option('title', this.model.dialogText.title);
            $('.e-head-content .e-head-name').html(this.model.dialogText.name);
            $('.e-head-content .e-head-size').html(this.model.dialogText.size);
            $('.e-head-content .e-head-status').html(this.model.dialogText.status);
        },
        // all events bound using this._on will be unbind automatically
        /**
* destroy the Upload control
* all events bound using this._on will be unbind automatically and bring the control to pre-init state.
* @alias destroy
* @return jQuery
* @example 
* &lt;div id="uploadbox1"&gt;&lt;/div&gt; <br> 
* &lt;script&gt;
* // Destroy Upload
* $("#uploadbox1").ejUploadbox();
* var uploadObj = $("#uploadbox1").data("ejUploadbox");
* uploadObj.destroy(); // destroy the Upload control
* &lt;/script&gt;
* @example 
* &lt;div id="uploadbox1"&gt;&lt;/div&gt; <br> 
* &lt;script&gt;
* // destroy the Upload control
* $("#uploadbox1").ejUploadbox();
* $("#uploadbox1").ejUploadbox("destroy");	
* &lt;/script&gt;
* @memberof ejUploadbox
* @instance
*/
        _destroy: function () {
            /// <summary>This will destroy the  upload button </summary>
            if (this.element.hasClass("e-uploadbox")) {
                this.element.removeClass("e-uploadbox e-widget");
                this.element.empty();
            }
        },
        /**
* To set custom skin theme if cssClass  property is enabled.
* @private
*/
        _setSkin: function (skin) {
            this.element.removeClass(this.model.cssClass );
            this.element.addClass(skin);
        },
        /**
* To initialize the Uploadbox control	
* @private
*/
        //initialize the control
        _initialize: function () {
            /// <summary>This will initialize upload button </summary>
            /*Upload Control*/
            this.control = this.element[0];
            this.element.addClass("e-widget " + this.model.cssClass );
            this.innerdiv = ej.buildTag('div.e-selectpart e-select');
            this.element.append(this.innerdiv);
            this.buttondiv = ej.buildTag('input.e-inputbtn e-btn#' + this.control.id + '_SelectButton', '', {}, { type: 'button', value: this._getLocalizedLabels("browse") });
            this.inputupload = ej.buildTag('input.e-uploadinput', "", {}, { type: 'file', name: this.control.id });
            if (this.model.multipleFilesSelection) {
                this.inputupload.attr('multiple', 'multiple');
            }
            this.innerdiv.append(this.buttondiv);
            this.innerdiv.append(this.inputupload);
            this._Selector = this.buttondiv[0];
            this._setRTL(this.model.enableRTL);
            this.Uploadframes = []; //For IFrame
            this.UploadForms = [];
            this.UploadType = this._isXhrSupported() ? "Xhr" : "IFrame";
           
            
        },
        //Wiring Events
        /**
* Wiring the events to Uploadbox control		
* @private
*/
        _wireEvents: function () {
            /// <summary>This will wire events of  upload button </summary>
            this._on(this.element, "click", this._disableclickselect);
            this._bindInputChangeEvent();
        },
        // disable click of select button.
        /**
* To prevent click action on disable state		
* @private
*/
        _disableclickselect: function (e) {
            if (this.element.hasClass("e-disable")) {
                e.preventDefault();
            }
        },
        //Bind input change function
        /**
* This will bind input change event of upload button 	
* @private
*/
        _bindInputChangeEvent: function () {
            //file selection changed event
            this._on(this.inputupload, "change", this._inputValueChange);
        },
        //Internal Events 
        /**
* This will bind input value change event of upload button 	
* @private
*/
        _inputValueChange: function (e) {
            var input, files;
            input = $(e.target);
            this.element.find(".e-uploadbtn").removeAttr('disabled').removeClass('e-disable');
            files = this._getInputFileInfo(input);
            this._trigger("fileSelect",files);
            if (this._isAllowed(files)) {
                if (!this.model.asyncUpload) {
                    this._onSelectSyncUpload(e); //Sync Upload
                } else {
                    if (this.UploadType == "Xhr") {
                        this._onXhrSelect(e); //removed uploadcore // For XHR Upload
                    } else {
                        this._onSelectIFrame(e); //For Iframe
                    }
                }
            }
        },
        // When Auto upload is false
        /**
* This will fire on upload button	
* @private
*/
        __uploadButtonClick: function (e) {
            //removed uploadcore
            if (this.UploadType == "Xhr") {
                this._xhrOnUploadButtonClick(e); // Xhr Upload
            } else {
                this._onUploadButtonClickIFrame(e); //Iframe Upload
            }
            $(e.target).attr('disabled','disabled').addClass('e-disable');
        },
        /**
* This will fire the upload button
* @private
*/
        _actionClick: function (e) {
            var targetAction, fileItem, file;
            targetAction = $(e.target);
            fileItem = targetAction.closest(".e-upload-file");
            file = $(fileItem).data("file");
            if (targetAction.hasClass("e-file-delete")) {
                //Trigger on Remove hanlder
                this._trigger("remove", { fileStatus: file });
                //removed uploadcore
                if (this.UploadType == "Xhr") {
                    this._xhrOnRemove(e, fileItem); //XHr Remove
                } else {
                    this._onRemoveIFrame(e, fileItem); //IFrame File Upload
                }
            } else if (targetAction.hasClass("e-file-cancel")) {
                //Trigger on Cancel hanlder
                this._trigger("cancel", { fileStatus: file }); //removed uploadcore
                if (!this.model.asyncUpload) {
                    this._onCancelSyncUpload(e, fileItem); //sync
                } else {
                    if (this.UploadType == "Xhr") {
                        this._xhrOnCancel(e, fileItem); //Xhr Cancel Item
                    } else {
                        this._onCancelIFrame(e, fileItem); //IFrame Cancel Item
                    }
                }
            } else if (targetAction.hasClass("e-file-retry")) {//removed uploadcore
                if (this.UploadType == "Xhr") {
                    this._xhrOnRetry(e, fileItem); //xhr remove
                } else {
                    this._onRetryIFrame(e, fileItem); //IFrame Remove
                }
            }

        },
        /**
* This Remove file entry upload button 
* @private
*/
        _removeFileEntry: function (file) {
            file.remove();
        },
        // Internal Methods
        /**
* This will fire is on file upload button 
* @private
*/
        _isFileUpload: function (fileEntry) {
            var actiondiv = $(fileEntry).find("div.e-icon");
            return actiondiv.is(".e-file-cancel");
        },
        /*Supporting Browsers */
        /**
* Validates upload type is xhr or not
* @private
*/
        _isXhrSupported: function () {
            return (((this.s.name == "msie" && parseInt(this.s.version) < 9) || ((this.s.name == "safari" && this.s.name == "chrome") && this.s.version == "536")) ? false : (typeof (FormData) != "undefined") && (typeof (File) != "undefined"));
        },
        /*Getting File Details*/
        /**
* To get the file details of the uploaded file
* @private
*/
        _getFileName: function (input) {
            /// <summary>This will get file name in upload button </summary>
            return $.map(this._getAllFileInfo(input), function (file) {
                return file.name;
            }).join(", ");
        },
        /**
* To get the file size of the uploaded file
* @private
*/
        _getFileSize: function (input) {
            /// <summary>This will get file size in upload button </summary>
            var tempProxy = this;
            return $.map(this._getAllFileInfo(input), function (file) {
                return tempProxy._formatSize(file.size);
            }).join(", ");
        },
        /**
* Pushes the uploaded file in uplaod dialog box
* @private
*/
        _pushFile: function (files, datapart) {
            /// <summary>This will push file in upload button </summary>
            var fileListDetails, addedFile, actionlist, i, action, diaObj, addedheading;
            addedheading = $("<div class='e-head-content'><div class='e-file-head e-head-name'>" + this._getLocalizedLabels("name") + "</div><div class='e-file-head e-head-size'>" + this._getLocalizedLabels("size") + "</div><div class='e-file-head e-head-status'>" + this._getLocalizedLabels("status") + "</div></div>");
            filedialog = this.updialog;
            if (filedialog && filedialog.length != 0) {
                if (this.model.showFileDetails) {
                    this.diaObj.open();
                }
            }
            else {
                this.updialog = ej.buildTag('div.uploaddialog#' + this.element[0].id + '_dialog', "", {}, { 'title': this._getLocalizedLabels("title") });
                this.element.append(this.updialog);
            }
            fileListDetails = this.element.find(".e-ul"); /// <reference path="IframeUpload.js" />
            if (fileListDetails.length == 0) {
                addedheading.appendTo(this.updialog);
                fileListDetails = ej.buildTag('ul.e-ul').appendTo(this.updialog);
                fileListActions = ej.buildTag('div.e-file-upload').appendTo(this.updialog);
                dialogActions = ej.buildTag('button.e-uploadclosebtn e-btn e-select', this._getLocalizedLabels("cancel"), {}, { type: 'button' }).appendTo(fileListActions);
                this._on(dialogActions, "click", this._dialogclose);
            }
            var xpos = (this.element.offset().left + this.element.width()) / 3, ypos = this.element.offset().top;
            this.updialog.ejDialog({ showOnInit: false, width: 510, cssClass: this.model.cssClass, close: $.proxy(this._uploadFileListDelete, this), position: { X: xpos, Y: ypos }, enableRTL: this.model.enableRTL, content: "#" + this.element[0].id, enableResize: false });
            this.diaObj = this.updialog.data('ejDialog');
            if (!this.model.multipleFilesSelection) {
                this.element.find(".e-ul>.e-upload-file").remove();
            }
            for (i = 0; i < files.length; i++) {//localization can be given for not started
                /*things to be redefined*/
                addedFile = $("<li class='e-upload-file'><div class='e-file-list'>" +
                    "<div class='e-file-progress e-file-view'><div class='e-file-name e-file-view'><span class='e-file-name-txt'>"
                    + files[i].name + "</span></div></div><div class='e-file-size e-file-view'><span class='e-file-name-txt'>"
                    + this._formatSize(files[i].size) + "</span></div>" +
                    "<div class='e-file-percentage e-file-view'><div class='e-file-progress-bar'><div class='e-file-progress-status'></div></div></div><div class='e-action-perform'><div class='e-icon e-file-view'></div></div></div></li>").appendTo(fileListDetails).data(datapart);

                if (this._getFileSize(files[i]).toString().toLowerCase() == "nankb" || files[i].size == null) {
                    addedFile.find(".e-file-size").remove();
                }
                action = "cancel";
                addedFile.find(".e-icon").remove().addClass(action.toString());
                if (action == "cancel") {
                    actionlist = ej.buildTag('div.e-icon e-file-cancel', '', {}, { title: 'cancel' });
                } else if (action == "remove") {
                    actionlist = ej.buildTag('div.e-icon e-file-delete', '', {}, { title: 'remove' });
                } else if (action == "retry") {
                    actionlist = ej.buildTag('div.e-icon e-file-retry', '', {}, { title: 'retry' });
                }
                //appending list
                addedFile.find(".e-action-perform").append(actionlist);
                //FileList Click
                this._on(actionlist, "click", this._actionClick);
            }
            if (this.model.showFileDetails) {
                this.diaObj.open();
            }
            this._buttonText(this.model.buttonText);
            this._dialogText(this.model.dialogText);
            return addedFile;
        },
        /**
* This will push file details in upload button
* @private
*/
        _pushFileDetails: function (files) {
            var fileListDetails, addedFile, actionlist, i, action, me, diaObj, addedheading;
            addedheading = $("<div class='e-head-content'><div class='e-file-head e-head-name'>" + this._getLocalizedLabels("name") + "</div><div class='e-file-head e-head-size'>" + this._getLocalizedLabels("size") + "</div><div class='e-file-head e-head-status'>" + this._getLocalizedLabels("status") + "</div></div>");
            me = this;
            filedialog = this.updialog;
            if (filedialog && filedialog.length != 0) {
                if (this.model.showFileDetails) {
                    me.diaObj.open();
                }
            } else {
                this.updialog = ej.buildTag('div.uploaddialog#' + this.element[0].id + '_dialog', "", {}, { 'title': this._getLocalizedLabels("title") });
                this.element.append(this.updialog);
            }
            var fileListDetails, addedFile;
            fileListDetails = this.element.find(".e-ul");
            if (fileListDetails.length == 0) {
                addedheading.appendTo(this.updialog);
                fileListDetails = ej.buildTag('ul.e-ul').appendTo(this.updialog);
                fileListActions = ej.buildTag('div.e-file-upload').appendTo(this.updialog);
                dialogActions = ej.buildTag('button.e-uploadclosebtn e-btn e-select', this._getLocalizedLabels("cancel"), {}, { type: 'button' }).appendTo(fileListActions);
                this._on(dialogActions, "click", this._dialogclose);
            }
            var xpos = (this.element.position().left + this.element.find(".e-selectpart").width()) , ypos = this.element.position().top + this.element.find(".e-selectpart").height();
            if (!this.model.multipleFilesSelection) {
                this.element.find(".e-ul>.e-upload-file").remove();
            }
            addedFile = $("<li class='e-upload-file'><div class='e-file-list'><div class='e-file-progress e-file-view'><div class='e-file-name e-file-view'><span class='e-file-name-txt'>" + files.name + "</span></div></div><div class='e-file-size e-file-view'><span class='e-file-name-txt'>" + this._formatSize(0) + "\\" + this._formatSize(files.size) + "</span></div><div class='e-file-percentage e-file-view'><div class='e-file-progress-bar'><div class='e-file-progress-status'></div></div></div><div class='e-action-perform'><div class='e-icon e-file-view'></div></div></div></li>").appendTo(fileListDetails).data("file", files);
            this.updialog.ejDialog({ showOnInit: false, width: 520, height: "auto", cssClass: this.model.cssClass, close: $.proxy(this._uploadFileListDelete, this), position: { X: xpos, Y: ypos }, enableRTL: this.model.enableRTL, content: "#" + this.element[0].id, enableResize: false })
            //create object for ejDialog 
            me.diaObj = this.updialog.data("ejDialog");
            if (this.model.showFileDetails) {
                me.diaObj.open();
            }
            this._buttonText(this.model.buttonText);
            this._dialogText(this.model.dialogText);
            return addedFile;
        },
        /**
* Specifies uploading/ progress state of the file uploaded
* @private
*/
        _setProgress: function (filelist, percentage, e) {
            var progressbar, progress, filesize, loaded, total;
            progressbar = $(filelist).find(".e-file-progress-status");
            progressbar.width(percentage + "%");
           
            filesize = $(filelist).find(".e-file-size .e-file-name-txt");
            loaded = this._formatSize(e.loaded);
            total = this._formatSize(e.total);
            filesize.html(loaded + "\\" + total);
        },
        /**
* Specifies the current action of the file uploaded
* @private
*/
        _setAction: function (element, action) {
            //localization can be provided for the strings cancel,remove and retry.
            var actionlist;
            element.find(".e-icon").remove().addClass(action.toString());
            if (action == "cancel") {
                actionlist = ej.buildTag('div.e-icon e-file-cancel', '', {}, { title: 'cancel' });
            } else if (action == "remove") {
                actionlist = ej.buildTag('div.e-icon e-file-delete', '', {}, { title: 'remove' });
            } else if (action == "retry") {
                actionlist = ej.buildTag('div.e-icon e-file-retry', '', {}, { title: 'retry' });
            }
            //appending list
            element.find(".e-action-perform").append(actionlist);
            //FileList Click
            this._on(actionlist, "click", this._actionClick);
        },
        /**
* Sets status of the file uploaded
* @private
*/
        _setStatus: function (element, status) {
            var progress, upstatus = ej.buildTag('div');
            if (status == "success") {
                element.find(".file-status").addClass("e-file-status-success").html("Completed");
                element.find(".e-file-percentage").html("").attr("title", "Completed");
                upstatus.addClass("e-icon e-file-percentage-success");
                element.find(".e-file-percentage").append(upstatus);
            }
            if (status == "failed") {
                element.find(".file-status").addClass("e-file-status-failed").html("Failed");
                element.find(".e-file-percentage").html("").attr("title", "Failed");
                upstatus.addClass("e-icon e-file-percentage-failed");
                element.find(".e-file-percentage").append(upstatus);
            }
            if (status == "progress") {
                element.find(".file-status").addClass("file-status-inprogress").html("in progress");
            }
            if (status == "uploading") {
                element.find(".file-status").addClass("file-status-inprogress").html("uploading");
                progress = element.find(".e-file-percentage");
                progress.html("");
            }
        },
        /**
* Sets input field for multiple file upload
* @private
*/
        _createInputandBind: function () {
            // creating input field. 
            var tempInput = ej.buildTag('input', '', {}, { type: 'file' });
            tempInput.attr("name", this.control.id).attr("autocomplete", "off").attr("class", "e-uploadinput");
            if (this.model.multipleFilesSelection) {
                tempInput.attr("multiple", "multiple");
            }
            tempInput.appendTo(this.element.find(".e-selectpart"));
            this.inputupload = tempInput;
            this._bindInputChangeEvent();
        },
        /**
* This will show upload button
* @private
*/
        _showUploadButton: function () {
            var uploadbutton = this.element.find(".e-uploadbtn");
            if (uploadbutton.length == 0) {
                uploadbutton = ej.buildTag('button.e-uploadbtn e-btn e-select', this._getLocalizedLabels("upload"), {}, { type: "button" });
                this.element.find(".e-file-upload").append(uploadbutton);
                //File_upload_button Click
                this._on(uploadbutton, "click", this.__uploadButtonClick);
            }
			this._buttonText(this.model.buttonText);
        },
        /**
* This will reset the uplaod button
* @private
*/
        _resetFileInput: function ($element) {
            var clone = $element.clone(false, false);
            this._on(clone, "change", this._inputValueChange);
            $element.replaceWith(clone);
        },
        /**
* Ensures the file format
* @private
*/
        _isAllowed: function (files) {
            var inputfield, denied, uploadControl, args, allowExtension, denyExtension;
            inputfield = this.element.find(".e-uploadinput");
            denied = false;
            uploadControl = this;
            if (this.model.extensionsAllow != "") {
                allowExtension = this.model.extensionsAllow.split(",");
                $(files).each(function () {
                    if ($.inArray(this.extension, allowExtension) == -1) {
                        // Raise Event
                        args = { action: "ExtensionsAllow", files: files };
                        uploadControl._trigger('error', args);
                        denied = true;
                        return false;
                    }
                }
                    );
            }
            if (this.model.extensionsDeny != "") {
                denyExtension = this.model.extensionsDeny.split(",");
                $(files).each(function () {
                    if ($.inArray(this.extension, denyExtension) != -1) {
                        // Raise Event
                        args = { action: "ExtensionsDeny", files: files };
                        uploadControl._trigger('error', args);
                        denied = true;
                        return false;
                    }
                });
            }
            if (denied) {
                this._resetFileInput(inputfield);
            }
            return !denied;
        },
        /**
* This will sets the file remove status.
* @private
*/
        _fileListRemove: function () {
            fileList = this.element.find(".e-upload-file .e-file-delete");
            if (fileList.length == 0) {
                this.element.find(".e-uploadbtn").attr('disabled', 'disabled').addClass('e-disable');
                this.updialog.ejDialog('close');
            }
        },
        /**
* This will sets the uplaod disabled
* @private
*/
        _uploadHide: function () {
            fileList = this.element.find(".e-upload-file .e-file-cancel");
            if (fileList.length == 0) {
                this.element.find(".e-ul").empty();
                this.element.find(".e-uploadbtn").attr('disabled', 'disabled').addClass('e-disable');
                this.updialog.ejDialog('close');
            }
        },
        /**
* Section of handling dialog close action
* @private
*/
        _dialogclose: function () {
            var inputfield;
            this.updialog.find(".e-ul").empty();
            inputfield = this.element.find(".e-uploadinput");
            this._resetFileInput(inputfield);
            this.updialog.ejDialog('close');
        },
        /**
* Deletes the file list from the dialog box.
* @private
*/
        _uploadFileListDelete: function () {
            var inputfield;
            this.updialog.find(".e-ul").empty();
            inputfield = this.element.find(".e-uploadinput");
            this._resetFileInput(inputfield);
        },
        /* XHR Upload  */
        /**/
        /**
* Section for handling XHR upload
* @private
*/
        _onXhrSelect: function (e) {
            var inputField, files, xhrUpload, addedFile;
            inputField = $(e.target);
            files = this._getInputFileInfo(inputField);
            this._xhrBeforeUpload(files, inputField);
            xhrUpload = this;
            $.each(files, function (i, fileItem) {
                addedFile = $(fileItem).data("filelist");
                xhrUpload._setAction(addedFile, "cancel"); // XhrUpload._Uploader._setAction(addedFile, "cancel");
                if (xhrUpload.model.autoUpload) {
                    xhrUpload._xhrPerformUpload(fileItem);
                } else {
                    xhrUpload._showUploadButton();
                }
            });
        },
        /**
* Action performed before XHR upload
* @private
*/
        _xhrBeforeUpload: function (files, inputField) {
            var fileEntry, xhrUpload, formdata, addedFile;
            fileEntry = files;
            xhrUpload = this;
            $.each(fileEntry, function (i, fileItem) {
                formdata = xhrUpload._createFormObjectXhr(fileItem);
                $(fileItem).data("formobject", formdata);
                addedFile = xhrUpload._pushFileDetails(fileItem);
                $(fileItem).data("filelist", addedFile);
            });

            return fileEntry;
        },
        /**
* Action performed during XHR upload
* @private
*/
        _xhrPerformUpload: function (fileItem) {
            var isPrevented, url, xhrUpload, formdata, xhr;
            this._trigger('begin');
            url = this.model.saveUrl; //
            xhrUpload = this;
            formdata = $(fileItem).data("formobject");
            xhr = new XMLHttpRequest();
            $(fileItem).data("xhr", xhr);
            xhr.addEventListener("load", function (e) {
                xhrUpload._onRequestSuccess(xhrUpload, e, fileItem);
            }, false);
            xhr.addEventListener("error", function (e) {
                xhrUpload._onRequestError(xhrUpload, e, fileItem);
            }, false);
            xhr.upload.addEventListener("progress", function (e) {
                xhrUpload._onRequestProgress(xhrUpload, e, fileItem);
            }, false);
            xhr.open("POST", url);
            xhr.send(formdata);
        },
        /**
* secifies Action performed during XHR upload button click
* @private
*/
        _xhrOnUploadButtonClick: function (e) {
            var xhrUpload, fileEntry, started;
            xhrUpload = this;
            $(".e-ul li.e-upload-file", xhrUpload.control).each(function () {
                fileEntry = $(this);
                started = xhrUpload._isFileUpload(fileEntry);
                if (started) {
                    xhrUpload._xhrPerformUpload($(fileEntry).data("file"));
                }
            });
        },
        /**
* secifies Action performed during XHR uploaded file remove
* @private
*/
        _xhrOnRemove: function (e, fileItem) {
            var filename = $(fileItem).find(".e-file-name").text().toString().split(","), proxy = this;
            $.ajax({
                url: this.model.removeUrl,
                type: "POST",
                data: "fileNames=" + filename,
                success: function () {
                    $(fileItem).remove();
                    proxy._fileListRemove();
                }
            });
        },
        /**
* secifies cancel action performed during XHR uploaded
* @private
*/
        _xhrOnCancel: function (e, fileItem) {
            var file, xhr;
            file = $(fileItem).data("file");
            xhr = $(file).data("xhr");
            if (xhr) {
                $(file).data("xhr").abort();
            }
            $(file).data("xhr", null);
            $(fileItem).data("file", null);
            $(fileItem).remove();
            this._uploadHide();
        },
        /**
* secifies retry action performed during XHR uploaded
* @private
*/
        _xhrOnRetry: function (e, fileItem) {
            var file = $(fileItem).data("file");
            this._xhrPerformUpload(file);
        },
        /**
* section for handling request success for uploading
* @private
*/
        _onRequestSuccess: function (xhrUpload, e, fileEntry) {
            var xhr = $(fileEntry).data("xhr");
            if (xhr.status >= 200 && xhr.status <= 299) {
                xhrUpload._onXhrUploadSuccess(xhrUpload, e, fileEntry);
            }
            else {
                xhrUpload._onRequestError(xhrUpload, e, fileEntry);
            }
        },
        /**
* section for handling when upload is successful
* @private
*/
        _onXhrUploadSuccess: function (xhrUpload, e, fileEntry) {
            var addedFile, xhr, progressbar, size, fSize, filesize, args;
            addedFile = $(fileEntry).data("filelist");
            xhr = $(fileEntry).data("xhr");
            if (xhrUpload.model.removeUrl) {
                xhrUpload._setAction(addedFile, "remove");
                xhrUpload._setStatus(addedFile, "success");
            } else {
                addedFile.find(".e-icon").remove();
                xhrUpload._setStatus(addedFile, "success");
            }
            if ($(fileEntry).length > 0) {
                progressbar = $(addedFile).find(".e-file-progress-status");
                progressbar.width("100%");
                size = $(fileEntry)[0].size;
                fSize = this._formatSize(size);
                filesize = $(addedFile).find(".e-file-size .e-file-name-txt");
                filesize.html(fSize + "\\" + fSize);
            }
            args = { files: fileEntry, xhr: xhr, e: e };
            xhrUpload._trigger('complete', args);
        },
        /**
* section for handling when upload is failed
* @private
*/
        _onRequestError: function (xhrUpload, e, fileEntry) {
            var addedFile, xhr, args;
            addedFile = $(fileEntry).data("filelist");
            xhr = $(fileEntry).data("xhr");
            xhrUpload._setAction(addedFile, "retry");
            xhrUpload._setStatus(addedFile, "failed");
            args = { action: "Error", files: fileEntry, xhr: xhr, e: e };
            xhrUpload._trigger('error', args);
        },
        /**
* section for handling when upload is in progress
* @private
*/
        _onRequestProgress: function (xhrUpload, e, fileEntry) {
            var percentage, addedFile;
            percentage = Math.round(e.loaded * 100 / e.total);
            addedFile = $(fileEntry).data("filelist");
            xhrUpload._setProgress(addedFile, percentage, e);
            xhrUpload._setStatus(addedFile, "progress");
        },
        /**
* section for handling xhr form object
* @private
*/
        _createFormObjectXhr: function (file) {
            var formData = new FormData();
            formData.append(this.control.id, file.rawFile);
            return formData;
        },
        /**/
        //Utility functions
        /**/
        /**
* This will input File Info upload button 
* @private
*/
        _getInputFileInfo: function ($input) {
            var input = $input[0];
            if (input.files) {
                return this._getAllFileInfo(input.files);
            } else {
                return [{
                    name: this._GetName(input.value),
                    extension: this._getFileExtensionType(input.value),
                    size: this._getFileSizeinIE(input.value)
                }];
            }
        },
        /**
* This will get File Size IE upload button
* @private
*/
        _getFileSizeinIE: function (fileName) {
            var actievXObj, fileSize;
            actievXObj = null;
            fileSize = null;
            try {
                actievXObj = new ActiveXObject("Scripting.FileSystemObject");
            } catch (e) {
                fileSize = null;
            }
            if (actievXObj) {
                fileSize = actievXObj.getFile(fileName).size;
            }
            return fileSize;
        },
        /**
* This will get FileExtensionType of upload file
* @private
*/
        _getFileExtensionType: function (fileName) {
            return fileName.match ? (fileName.match(/\.([^\.]+)$/)[0] || "") : "";
        },
        /**
* This will return information about all files uploaded
* @private
*/
        _getAllFileInfo: function (rawFiles) {
            var tempProxy = this;
            return $.map(rawFiles, function (file) {
                return tempProxy._getFileInfo(file || rawFiles);
            });
        },
        /**
* This will return full names of files
* @private
*/
        _GetName: function (fullname) {
            var nameIndex = fullname.lastIndexOf("\\");
            return (nameIndex != -1) ? fullname.substr(nameIndex + 1) : fullname;
        },
        /**
* This will return file details in firefox older versions
* @private
*/
        _getFileInfo: function (rawFile) {
            // Older Firefox versions (before 3.6) use fileName and fileSize
            var fileName = rawFile.name || rawFile.fileName || rawFile;
            return {
                name: fileName,
                extension: this._getFileExtensionType(fileName),
                size: rawFile.size || rawFile.fileSize,
                rawFile: rawFile
            };
        },
        /**
* This will return file size type
* @private
*/
        _formatSize: function (bytes) {
            var i = -1;
            do {
                bytes = bytes / 1024;
                i++;
            } while (bytes > 99);
            return Math.max(bytes, 0.1).toFixed(1) + ['KB', 'MB', 'GB', 'TB', 'PB', 'EB'][i];
        },
        /**/
        //Start
        // IFrame Uploads
        /**
* Section for handling IFrame uploads
* @private
*/
        _onSelectIFrame: function (e) {
            var input, files, addedFile, uploadframe;
            input = $(e.target);
            files = this._getInputFileInfo(input);
            addedFile = this._beforeUploadIFrame(files);
            uploadframe = addedFile.data("iframe");
            if (this.model.autoUpload) {
                this._performUploadIFrame(addedFile);
            } else {
                this._showUploadButton();
            }
            this._off(this.inputupload, "change");
            this._bindInputChangeEvent();
        },
        /**
* Section for handling files removed from IFrame
* @private
*/
        _onRemoveIFrame: function (e, fileItem) {
            var iframe, fileNames, proxy, fileDetails;
            iframe = fileItem.data("iframe");
            fileDetails = $(fileItem).data("file");
            fileNames = fileDetails[0].name;
            proxy = this;
            if (iframe) {
                this._removeFileEntry(fileItem);
                if (this.model.removeUrl) {
                    $.ajax({
                        url: this.model.removeUrl,
                        type: "POST",
                        data: "fileNames=" + fileNames,
                        success: function () {
                            proxy._fileListRemove();
                        }
                    });
                }
            } else {
                this._removeFileEntry(fileItem);
            }
        },
        /**
* Section for handling cancel action performed during file upload
* @private
*/
        _onCancelIFrame: function (e, fileItem) {
            var iframe;
            //trigger  cancel
            this._trigger('cancel', { Status: fileItem });
            iframe = fileItem.data("iframe");
            if (iframe) {
                this._removeFileEntry(fileItem);
                if (typeof (iframe.stop) != "undefined") {
                    iframe.stop();
                } else if (iframe.document) {
                    iframe.document.execCommand("Stop");
                    iframe.contentWindow.location.href = iframe.contentWindow.location.href;
                }
                this._processServerResponse(iframe, "");
            }
            this._uploadHide();
        },
        /**
* Section for handling retry action performed during file upload
* @private
*/
        _onRetryIFrame: function (e, fileItem) {
            this._performUploadIFrame(fileItem);
        },
        /**
* Section for handling actions performed during before IFrame upload
* @private
*/
        _beforeUploadIFrame: function (files) {
            var uploadframe, uploadform, addedfile;
            //creating iframe and adding it to the upload div block.
            uploadframe = this._createFrame(this.control.id + "_Iframe" + this.Uploadframes.length);
            this.Uploadframes.push(uploadframe);
            uploadform = this._createForm(this.model.saveUrl, uploadframe[0].id);
            this.element.find("input.e-uploadinput").removeClass("e-uploadinput").css("display", "none").appendTo(uploadform);
            this._createInputandBind();
            addedfile = this._pushFile(files, { "iframe": uploadframe, "form": uploadform, "file": files });
            uploadframe.data({ "filelist": addedfile });
            this._setAction(addedfile, "cancel");
            return addedfile;
        },
        /**
* Section for handling actions performed during IFrame upload
* @private
*/
        _performUploadIFrame: function (addedFile) {
            var isPrevented, files, uploadframe, uploadform;
            this._trigger('begin');
            files = addedFile.data("file");
            this._setStatus(addedFile, "uploading");
            uploadframe = addedFile.data("iframe");
            uploadform = addedFile.data("form");
            uploadframe.appendTo(document.body);
            uploadform.appendTo(document.body);
            //error here calls the upload even if the url is wrong
            this._on(uploadframe, "load", this._removeFramesIFrame);
            uploadform.submit();
        },
        /**
* Section for handling actions performed during IFrame upload button click
* @private
*/
        _onUploadButtonClickIFrame: function (e) {
            var iframeUpload, fileEntry, started;
            iframeUpload = this;
            $(".e-ul li.e-upload-file", iframeUpload.control).each(function () {
                fileEntry = $(this);
                started = iframeUpload._isFileUpload(fileEntry);
                if (started) {
                    iframeUpload._performUploadIFrame(fileEntry);
                }
            });
        },
        /**
* Section for handling actions when IFrame files removed before upload
* @private
*/
        _removeFramesIFrame: function (e) {
            var uploadframe, response, filelist, fileEntry, args;
            uploadframe = $(e.target);
            filelist = uploadframe.data("filelist");
            try {
                response = uploadframe.contents().text();
            } catch (e) {
                response = "Server Error trying to get server response: " + e;
            }
            if (response.substring(0, 12) != "Server Error") {
                this._processServerResponse(uploadframe, response);
                this._setIframeProgress(filelist, 100, e);
                this._setStatus(filelist, "progress");
                this._successIframeUpload(filelist, response);
            }
            else { this._failureIframeUpload(filelist, response); }
        },
        //prog bar
        /**
* Specifies the progress state during IFrame upload
* @private
*/
        _setIframeProgress: function (filelist, percentage, e) {
            var progressbar, progress, filesize, loaded, total;
            progressbar = $(filelist).find(".e-file-progress-status");
            progressbar.width(percentage + "%");
        },
        /**
* Section for handling actions when IFrame upload is successfull
* @private
*/
        _successIframeUpload: function (filelist, response) {
            fileEntry = filelist.data("file");
            //trigger complete functionality
            if (this.model.removeUrl) {
                this._setAction(filelist, "remove");
                this._setStatus(filelist, "success");
            } else {
                filelist.find(".file-action").remove();
                this._setStatus(filelist, "success");
            }
            args = { files: fileEntry, responseText: response };
            this._trigger('complete', args);
        },
        /**
* Section for handling actions when IFrame upload fails
* @private
*/
        _failureIframeUpload: function (filelist, response) {
            fileEntry = filelist.data("file");
            //trigger Error functionality
            if (this.model.saveUrl) {
                this._setAction(filelist, "retry");
                this._setStatus(filelist, "failed");
            } else {
                filelist.find(".file-action").remove();
                this._setStatus(filelist, "failed");
            }
            args = { files: fileEntry, responseText: response };
            this._trigger('error', args);
        },
        /**
* Section for handling response from server
* @private
*/
        _processServerResponse: function (uploadframe) {
            var uploadform;
            uploadform = $(document.body).find("form[target='" + $(uploadframe).attr("id") + "']");
            setTimeout(function () {
                uploadform.remove();
                uploadframe.remove();
            }, 0);
        },
        /**
* Returns div block created
* @private
*/
        _createDivBlock: function (className) {
            return ej.buildTag('div.' + className);
        },
        /**
* Returns form tag as object
* @private
*/
        _createForm: function (action, target) {
            return ej.buildTag('form', '', {}, { enctype: 'multipart/form-data', method: 'POST', action: action, target: target });
        },
        /**
* Section for creating IFrame for handling IFrame uploads
* @private
*/
        _createFrame: function (id) {
            return ej.buildTag('iframe#' + id, '', { display: 'none' }, { name: id });
        },
        /**
* Returns file type input tag whwn id is specified 
* @private
*/
        _createInput: function (id) {
            return ej.buildTag('input', '', {}, { type: 'file', name: id });
        },
        //end
        /*Sync Start*/
        /**
* Section for synchronous upload
* @private
*/
        _initObjectsSyncUpload: function () {

            this.element.closest("form")
                    .attr("enctype", "multipart/form-data")
                    .attr("encoding", "multipart/form-data");
            this._wireEventsSyncUpload();
        },
        /**
* Binds events for handling sync upload 
* @private
*/
        _wireEventsSyncUpload: function () {
            var closestform = this.element.closest("form")[0];
            //form submit event. 
            this._on($(closestform), "submit", this._formSubmitSyncUpload);
            // form reset event.
            this._on($(closestform), "reset", this._formResetSyncUpload);
        },
        /**
* handles actions performed while sync upload 
* @private
*/
        _onSelectSyncUpload: function (e) {
            var input, files, selection, addedfile;
            input = $(e.target);
            files = this._getInputFileInfo(input);
            selection = $(".e-selectpart", this.control);
            this.element.find("input.e-uploadinput").removeClass("e-uploadinput").css("display", "none").appendTo(selection);
            this._createInputandBind();
            addedfile = this._pushFile(files, { "file": files, "Input": input });
        },
        /**
* Binds events for handling sync upload canceled
* @private
*/
        _onCancelSyncUpload: function (e, fileItem) {
            var inputfield = fileItem.data("Input");
            fileItem.data("file", null);
            fileItem.data("Input", null);
            fileItem.remove();
            inputfield.remove();
            this._uploadHide();
        },
        /**
* Performs upload during form submit
* @private
*/
        _formSubmitSyncUpload: function (e) {
            var input, uploader;
            input = $(".e-uploadinput", this.control);
            input.attr("name", "");
            uploader = this.control;
            setTimeout(function () {
                input.attr("name", uploader.id);
            }, 0);
        },
        /**
* Section for handling actions when form is reset.
* @private
*/
        _formResetSyncUpload: function (e) {
            $(".e-selectpart", this.control).children('input[type="file"]').each(function () {
                if (this.className == " ") {
                    $(this).remove();
                }
            });
            $(".e-ul", this.control).remove();
        }
        /*Sync End*/
    });
    ej.Uploadbox.Locale = {};

    ej.Uploadbox.Locale["en-US"] = {
        buttonText: {
            upload: "Upload",
            browse: "Browse",
            cancel: "Cancel"
        },
        dialogText: {
            title: "Upload Box",
            name: "Name",
            size: "Size",
            status: "Status"
        }
    };
})(jQuery, Syncfusion);;
/**
* @fileOverview Plugin to style the Html TextArea
* @copyright Copyright Syncfusion Inc. 2001 - 2013. All rights reserved.
*  Use of this code is subject to the terms of our license.
*  A copy of the current license can be obtained at any time by e-mailing
*  licensing@syncfusion.com. Any infringement will be prosecuted under
*  applicable laws. 
* @version 12.1 
* @author <a href="mailto:licensing@syncfusion.com">Syncfusion Inc</a>
*/
(function ($, ej, undefined) {
    /**
    * @namespace ej
	* @class ejRTE
    * @param {object} options - settings for RTE.
	* @requires jQuery
	* @requires jquery.easing.1.3.js
    * @requires jquery.js
    * @requires jquery.globalize.js
    * @requires jquery.globalize.culture.en-US.js
    * @requires ej.core.js
    * @requires ej.data.js
    * @requires ej.rte.js
    * @requires ej.button.js
    * @requires ej.togglebutton.js
    * @requires ej.splitbutton.js
    * @requires ej.checkbox.js
    * @requires ej.radiobutton.js
    * @requires ej.dropdownlist.js
    * @requires ej.dialog.js
    * @requires ej.toolbar.js
    * @requires ej.editor.js
    * @requires ej.menu.js
    * @requires ej.scroller.js
    * @requires ej.draggable.js
	* @classdesc Custom Design for Html TextArea control.
	* @example 
	* &lt;textarea   id="rteSample"&gt; 	
    * &lt;p&gt;&lt;b&gt;Description:&lt;/b&gt;&lt;/p&gt;
	    &lt;p&gt;The Rich Text Editor (RTE) control is an easy to render in
	    client side. Customer easy to edit the contents and get the HTML content for
	    the displayed content. A rich text editor control provides users with a toolbar
	    that helps them to apply rich text formats to the text entered in the text
	    area. &lt;/p&gt;&lt;/textarea &gt;     
	* &lt;script&gt;
	* // Create RTE
    * $("#rteSample").ejRTE();
	* &lt;/script&gt;
	*/
    // "ej.RTE" is "namespace.className" will hold functions and properties
    var xhr = null;
    ej.widget("ejRTE", "ej.RTE", {
        // widget element will be automatically set in this
        element: null,

        // user defined model will be automatically set in this
        model: null,
        _rootCSS: "e-rte",
        validTags: ["textarea"],
        _setFirst: false,

        // default model
        defaults: {
            /**		
			* Enables/Disables the editing of the content.
			* @default 	True
			* @type {boolean}
			* @example 
           * &lt;textarea   id="rteSample"&gt;  
    * &lt;p&gt;&lt;b&gt;Description:&lt;/b&gt;&lt;/p&gt;
        &lt;p&gt;The Rich Text Editor (RTE) control is an easy to render in
        client side. Customer easy to edit the contents and get the HTML content for
        the displayed content. A rich text editor control provides users with a toolbar
        that helps them to apply rich text formats to the text entered in the text
        area. &lt;/p&gt;&lt;/textarea &gt;     
    * &lt;script&gt;
			* // Initialize the RTE with the allowEditing value specified.
            * $("#rteSample").ejRTE({ allowEditing: false });
            * &lt;/script&gt;
			* @memberof ejRTE
			* @instance
			*/
            allowEditing: true,
            /**		
			* RTE control comments can access through keyboard shortcut keys.
			* @default 	True
			* @type {boolean}
			* @example 
            * &lt;textarea   id="rteSample"&gt;     
    * &lt;p&gt;&lt;b&gt;Description:&lt;/b&gt;&lt;/p&gt;
        &lt;p&gt;The Rich Text Editor (RTE) control is an easy to render in
        client side. Customer easy to edit the contents and get the HTML content for
        the displayed content. A rich text editor control provides users with a toolbar
        that helps them to apply rich text formats to the text entered in the text
        area. &lt;/p&gt;&lt;/textarea &gt;     
    * &lt;script&gt;
			* // Initialize the RTE with the allowKeyboardNavigation value specified.
            * $("#rteSample").ejRTE({allowKeyboardNavigation: false });
			* &lt;/script&gt;
			* @memberof ejRTE
			* @instance
			*/
            allowKeyboardNavigation: true,
            /**		
            * Sets the root class for RTE theme. This cssClass API helps to use custom skinning option for RTE control. By defining the root class using this API, we need to include this root class in CSS.
            * @default "gradient-lime"
            * @type {string}
            * @example 
            * &lt;textarea   id="rteSample"&gt;     
    * &lt;p&gt;&lt;b&gt;Description:&lt;/b&gt;&lt;/p&gt;
        &lt;p&gt;The Rich Text Editor (RTE) control is an easy to render in
        client side. Customer easy to edit the contents and get the HTML content for
        the displayed content. A rich text editor control provides users with a toolbar
        that helps them to apply rich text formats to the text entered in the text
        area. &lt;/p&gt;&lt;/textarea &gt;     
    * &lt;script&gt;
            * //Initialize the RTE with the cssClass value specified
            * 	$("#rteSample").ejRTE({ cssClass: 'gradient-lime'});
           * &lt;/script&gt;
            * @memberof ejRTE
            * @instance
            */
            cssClass: "",
            /**		
            * Defines the height of the RTE textbox.
            * @default Null
            * @type {string | number}
            * @example 
            * &lt;textarea   id="rteSample"&gt;     
    * &lt;p&gt;&lt;b&gt;Description:&lt;/b&gt;&lt;/p&gt;
        &lt;p&gt;The Rich Text Editor (RTE) control is an easy to render in
        client side. Customer easy to edit the contents and get the HTML content for
        the displayed content. A rich text editor control provides users with a toolbar
        that helps them to apply rich text formats to the text entered in the text
        area. &lt;/p&gt;&lt;/textarea &gt;     
    * &lt;script&gt;
            * //Initialize the RTE height property with the  value specified
            * 	$("#rteSample").ejRTE({ height: 250 });
            * &lt;/script&gt;
            * @memberof ejRTE
            * @instance
            */
            height: "370",
            /**		
            * Defines the width of the RTE textbox.
            * @default Null
            * @type {string |number}
            * @example 
            * &lt;textarea   id="rteSample"&gt;     
    * &lt;p&gt;&lt;b&gt;Description:&lt;/b&gt;&lt;/p&gt;
        &lt;p&gt;The Rich Text Editor (RTE) control is an easy to render in
        client side. Customer easy to edit the contents and get the HTML content for
        the displayed content. A rich text editor control provides users with a toolbar
        that helps them to apply rich text formats to the text entered in the text
        area. &lt;/p&gt;&lt;/textarea &gt;     
    * &lt;script&gt;
            * //Initialize the RTE width property with the  value specified
            * 	$("#rteSample").ejRTE({ width: 500 });
           * &lt;/script&gt;
            * @memberof ejRTE
            * @instance
            */
            width: "786",
            /**		
			* 	Enable | Disable the RTE control accessible or interaction..
			* @default 	True
			* @type {boolean}
			* @example 
            * &lt;textarea   id="rteSample"&gt;     
    * &lt;p&gt;&lt;b&gt;Description:&lt;/b&gt;&lt;/p&gt;
        &lt;p&gt;The Rich Text Editor (RTE) control is an easy to render in
        client side. Customer easy to edit the contents and get the HTML content for
        the displayed content. A rich text editor control provides users with a toolbar
        that helps them to apply rich text formats to the text entered in the text
        area. &lt;/p&gt;&lt;/textarea &gt;     
    * &lt;script&gt;
			* // Initialize the RTE with the enabled value specified.
            * $("#rteSample").ejRTE({enabled: false });
			* &lt;/script&gt;
			* @memberof ejRTE
			* @instance
			*/
            enabled: true,
            /**		
			* Set the max Length  for RTE text. 
			* @default 	400
			* @type {number}
			* @example 
            * &lt;textarea   id="rteSample"&gt;     
    * &lt;p&gt;&lt;b&gt;Description:&lt;/b&gt;&lt;/p&gt;
        &lt;p&gt;The Rich Text Editor (RTE) control is an easy to render in
        client side. Customer easy to edit the contents and get the HTML content for
        the displayed content. A rich text editor control provides users with a toolbar
        that helps them to apply rich text formats to the text entered in the text
        area. &lt;/p&gt;&lt;/textarea &gt;     
    * &lt;script&gt;
			* // Initialize the RTE with the maxLength value specified.
            * $("#rteSample").ejRTE({ maxLength: 900});
			* &lt;/script&gt;
			* @memberof ejRTE
			* @instance
			*/
            maxLength: 7000,
            /**		
			* Set the minimum width for RTE outer wrapper element. 
			* @default 	400
			* @type {number}
			* @example 
            * &lt;textarea   id="rteSample"&gt;     
    * &lt;p&gt;&lt;b&gt;Description:&lt;/b&gt;&lt;/p&gt;
        &lt;p&gt;The Rich Text Editor (RTE) control is an easy to render in
        client side. Customer easy to edit the contents and get the HTML content for
        the displayed content. A rich text editor control provides users with a toolbar
        that helps them to apply rich text formats to the text entered in the text
        area. &lt;/p&gt;&lt;/textarea &gt;     
    * &lt;script&gt;
			* // Initialize the RTE with the minWidth value specified.
            * $("#rteSample").ejRTE({ minWidth: 900});
			
			* &lt;/script&gt;
			* @memberof ejRTE
			* @instance
			*/
            minWidth: 400,
            /**		
           * Set the maximum width for RTE outer wrapper element. 
           * @default 	400
           * @type {number}
           * @example 
           * &lt;textarea   id="rteSample"&gt;  
    * &lt;p&gt;&lt;b&gt;Description:&lt;/b&gt;&lt;/p&gt;
        &lt;p&gt;The Rich Text Editor (RTE) control is an easy to render in
        client side. Customer easy to edit the contents and get the HTML content for
        the displayed content. A rich text editor control provides users with a toolbar
        that helps them to apply rich text formats to the text entered in the text
        area. &lt;/p&gt;&lt;/textarea &gt;     
    * &lt;script&gt;
           * // Initialize the RTE with the maxWidth value specified.
           * $("#rteSample").ejRTE({ maxWidth: 900});
          * &lt;/script&gt;
           * @memberof ejRTE
           * @instance
           */
            maxWidth: null,
            /**		
            * Set the minimum height for RTE outer wrapper element. 
            * @default 	280
            * @type {number}
            * @example 
            * &lt;textarea   id="rteSample"&gt;     
    * &lt;p&gt;&lt;b&gt;Description:&lt;/b&gt;&lt;/p&gt;
        &lt;p&gt;The Rich Text Editor (RTE) control is an easy to render in
        client side. Customer easy to edit the contents and get the HTML content for
        the displayed content. A rich text editor control provides users with a toolbar
        that helps them to apply rich text formats to the text entered in the text
        area. &lt;/p&gt;&lt;/textarea &gt;     
    * &lt;script&gt;
            * // Initialize the RTE with the maxWidth value specified.
            * $("#rteSample").ejRTE({ minHeight: 900});
           * &lt;/script&gt;
            * @memberof ejRTE
            * @instance
            */
            minHeight: 280,
            /**		
            * Set the maximum height for RTE outer wrapper element. 
            * @default 	280
            * @type  {string | number}
            * @example 
            * &lt;textarea   id="rteSample"&gt;     
    * &lt;p&gt;&lt;b&gt;Description:&lt;/b&gt;&lt;/p&gt;
        &lt;p&gt;The Rich Text Editor (RTE) control is an easy to render in
        client side. Customer easy to edit the contents and get the HTML content for
        the displayed content. A rich text editor control provides users with a toolbar
        that helps them to apply rich text formats to the text entered in the text
        area. &lt;/p&gt;&lt;/textarea &gt;     
    * &lt;script&gt;
            * // Initialize the RTE with the maxHeight value specified.
            * $("#rteSample").ejRTE({ maxHeight: 900});
           * &lt;/script&gt;
            * @memberof ejRTE
            * @instance
            */
            maxHeight: null,
            /**		
             * Shows toolbar in RTE. 
             * @default 	True
             * @type {boolean}
             * @example 
             * &lt;textarea   id="rteSample"&gt;     
    * &lt;p&gt;&lt;b&gt;Description:&lt;/b&gt;&lt;/p&gt;
        &lt;p&gt;The Rich Text Editor (RTE) control is an easy to render in
        client side. Customer easy to edit the contents and get the HTML content for
        the displayed content. A rich text editor control provides users with a toolbar
        that helps them to apply rich text formats to the text entered in the text
        area. &lt;/p&gt;&lt;/textarea &gt;     
    * &lt;script&gt;
             * // Initialize the RTE with the showToolbar value specified.
             * $("#rteSample").ejRTE({showToolbar: false });
             * &lt;/script&gt;
             * @memberof ejRTE
             * @instance
             */
            showToolbar: true,
            /**		
            * Shows footer in RTE. 
            * @default 	True
            * @type {boolean}
            * @example 
            * &lt;textarea   id="rteSample"&gt;     
    * &lt;p&gt;&lt;b&gt;Description:&lt;/b&gt;&lt;/p&gt;
        &lt;p&gt;The Rich Text Editor (RTE) control is an easy to render in
        client side. Customer easy to edit the contents and get the HTML content for
        the displayed content. A rich text editor control provides users with a toolbar
        that helps them to apply rich text formats to the text entered in the text
        area. &lt;/p&gt;&lt;/textarea &gt;     
    * &lt;script&gt;
            * // Initialize the RTE with the showFooter value specified.
            * $("#rteSample").ejRTE({showFooter: true });
            * &lt;/script&gt;
            * @memberof ejRTE
            * @instance
            */
            showFooter: false,
            /**		
            * Shows HtmlSource in RTE. 
            * @default 	True
            * @type {boolean}
            * @example 
            * &lt;textarea   id="rteSample"&gt;     
    * &lt;p&gt;&lt;b&gt;Description:&lt;/b&gt;&lt;/p&gt;
        &lt;p&gt;The Rich Text Editor (RTE) control is an easy to render in
        client side. Customer easy to edit the contents and get the HTML content for
        the displayed content. A rich text editor control provides users with a toolbar
        that helps them to apply rich text formats to the text entered in the text
        area. &lt;/p&gt;&lt;/textarea &gt;     
    * &lt;script&gt;
            * // Initialize the RTE with the showHtmlSource value specified.
            * $("#rteSample").ejRTE({showHtmlSource: false });
            * &lt;/script&gt;
            * @memberof ejRTE
            * @instance
            */
            showHtmlSource: true,
            /**		
            * Shows WordCount in RTE. 
            * @default 	True
            * @type {boolean}
            * @example 
            * &lt;textarea   id="rteSample"&gt;     
    * &lt;p&gt;&lt;b&gt;Description:&lt;/b&gt;&lt;/p&gt;
        &lt;p&gt;The Rich Text Editor (RTE) control is an easy to render in
        client side. Customer easy to edit the contents and get the HTML content for
        the displayed content. A rich text editor control provides users with a toolbar
        that helps them to apply rich text formats to the text entered in the text
        area. &lt;/p&gt;&lt;/textarea &gt;     
    * &lt;script&gt;
            * // Initialize the RTE with the showWordCount value specified.
            * $("#rteSample").ejRTE({showWordCount: false });
            * &lt;/script&gt;
            * @memberof ejRTE
            * @instance
            */
            showWordCount: true,
            /**		
            * Shows HtmlTagInfo in RTE. 
            * @default 	True
            * @type {boolean}
            * @example 
            * &lt;textarea   id="rteSample"&gt;     
    * &lt;p&gt;&lt;b&gt;Description:&lt;/b&gt;&lt;/p&gt;
        &lt;p&gt;The Rich Text Editor (RTE) control is an easy to render in
        client side. Customer easy to edit the contents and get the HTML content for
        the displayed content. A rich text editor control provides users with a toolbar
        that helps them to apply rich text formats to the text entered in the text
        area. &lt;/p&gt;&lt;/textarea &gt;     
    * &lt;script&gt;
            * // Initialize the RTE with the showHtmlTagInfo value specified.
            * $("#rteSample").ejRTE({showHtmlTagInfo: false });
            * &lt;/script&gt;
            * @memberof ejRTE
            * @instance
            */
            showHtmlTagInfo: true,
            /**		
            * Shows ClearAll in RTE. 
            * @DIefault 	True
            * @type {boolean}
            * @example 
            * &lt;textarea   id="rteSample"&gt;     
    * &lt;p&gt;&lt;b&gt;Description:&lt;/b&gt;&lt;/p&gt;
        &lt;p&gt;The Rich Text Editor (RTE) control is an easy to render in
        client side. Customer easy to edit the contents and get the HTML content for
        the displayed content. A rich text editor control provides users with a toolbar
        that helps them to apply rich text formats to the text entered in the text
        area. &lt;/p&gt;&lt;/textarea &gt;     
    * &lt;script&gt;
            * // Initialize the RTE with the showClearAll value specified.
            * $("#rteSample").ejRTE({showClearAll: false });
            * &lt;/script&gt;
            * @memberof ejRTE
            * @instance
            */
            showClearAll: true,
            /**		
            * Sets the iframe attribute in RTE. 
            * @default 	null
            * @type {object}
            * @example 
    * &lt;textarea   id="rteSample"&gt;     
    * &lt;p&gt;&lt;b&gt;Description:&lt;/b&gt;&lt;/p&gt;
        &lt;p&gt;The Rich Text Editor (RTE) control is an easy to render in
        client side. Customer easy to edit the contents and get the HTML content for
        the displayed content. A rich text editor control provides users with a toolbar
        that helps them to apply rich text formats to the text entered in the text
        area. &lt;/p&gt;&lt;/textarea &gt;     
    * &lt;script&gt;
            * // Initialize the RTE with the iframeAttribute value specified.
            * $("#rteSample").ejRTE({iframeAttribute: "color:#000" });
            * &lt;/script&gt;
            * @memberof ejRTE
            * @instance
            */
            iframeAttribute: "color:#5C5C5C",
            /**		
            * Sets the iframe attribute in RTE. 
            * @default 	null
            * @type {object}
            * @example 
    * &lt;textarea   id="rteSample"&gt;     
    * &lt;p&gt;&lt;b&gt;Description:&lt;/b&gt;&lt;/p&gt;
        &lt;p&gt;The Rich Text Editor (RTE) control is an easy to render in
        client side. Customer easy to edit the contents and get the HTML content for
        the displayed content. A rich text editor control provides users with a toolbar
        that helps them to apply rich text formats to the text entered in the text
        area. &lt;/p&gt;&lt;/textarea &gt;     
    * &lt;script&gt;
            * // Initialize the RTE with the iframeAttribute value specified.
            * $("#rteSample").ejRTE({showClearFormat:true });
            * &lt;/script&gt;
            * @memberof ejRTE
            * @instance
            */
            showClearFormat: true,
            /**		
            * Shows FontOption in RTE. 
            * @default 	True
            * @type {boolean}
            * @example 
            * &lt;textarea   id="rteSample"&gt;     
    * &lt;p&gt;&lt;b&gt;Description:&lt;/b&gt;&lt;/p&gt;
        &lt;p&gt;The Rich Text Editor (RTE) control is an easy to render in
        client side. Customer easy to edit the contents and get the HTML content for
        the displayed content. A rich text editor control provides users with a toolbar
        that helps them to apply rich text formats to the text entered in the text
        area. &lt;/p&gt;&lt;/textarea &gt;     
    * &lt;script&gt;
            * // Initialize the RTE with the showFontOption value specified.
            * $("#rteSample").ejRTE({showFontOption: false });
            * &lt;/script&gt;
            * @memberof ejRTE
            * @instance
            */
            showFontOption: true,
            /**		
            * Shows Dimensions in RTE. 
            * @default 	True
            * @type {boolean}
            * @example 
            * &lt;textarea   id="rteSample"&gt;     
    * &lt;p&gt;&lt;b&gt;Description:&lt;/b&gt;&lt;/p&gt;
        &lt;p&gt;The Rich Text Editor (RTE) control is an easy to render in
        client side. Customer easy to edit the contents and get the HTML content for
        the displayed content. A rich text editor control provides users with a toolbar
        that helps them to apply rich text formats to the text entered in the text
        area. &lt;/p&gt;&lt;/textarea &gt;     
    * &lt;script&gt;
            * // Initialize the RTE with the showDimensions value specified.
            * $("#rteSample").ejRTE({showDimensions: false });
            * &lt;/script&gt;
            * @memberof ejRTE
            * @instance
            */
            showDimensions: false,
            /**		
            * Sets the culture in RTE. 
            * @default 	"en-US"
            * @type {string}
            * @example 
         * &lt;textarea   id="rteSample"&gt;     
    * &lt;p&gt;&lt;b&gt;Description:&lt;/b&gt;&lt;/p&gt;
        &lt;p&gt;The Rich Text Editor (RTE) control is an easy to render in
        client side. Customer easy to edit the contents and get the HTML content for
        the displayed content. A rich text editor control provides users with a toolbar
        that helps them to apply rich text formats to the text entered in the text
        area. &lt;/p&gt;&lt;/textarea &gt;     
    * &lt;script&gt;
            * // Initialize the RTE with the culture value specified.
            * $("#rteSample").ejRTE({locale: "en-US" });
            * &lt;/script&gt;
            * @memberof ejRTE
            * @instance
            */
            locale: "en-US",
            /**		
            * Sets the name in RTE. 
            * @default 	""
            * @type {string}
            * @example 
            * &lt;textarea   id="rteSample"&gt;     
    * &lt;p&gt;&lt;b&gt;Description:&lt;/b&gt;&lt;/p&gt;
        &lt;p&gt;The Rich Text Editor (RTE) control is an easy to render in
        client side. Customer easy to edit the contents and get the HTML content for
        the displayed content. A rich text editor control provides users with a toolbar
        that helps them to apply rich text formats to the text entered in the text
        area. &lt;/p&gt;&lt;/textarea &gt;     
    * &lt;script&gt;
            * // Initialize the RTE with the name value specified.
            * $("#rteSample").ejRTE({name: "ecommentblog" });
            * &lt;/script&gt;
            * @memberof ejRTE
            * @instance
            */
            name: "",
            /**		
            * Sets the tools in RTE. 
            * @default 	null
            * @type {object}
            * @example 
            * &lt;textarea   id="rteSample"&gt;     
    * &lt;p&gt;&lt;b&gt;Description:&lt;/b&gt;&lt;/p&gt;
        &lt;p&gt;The Rich Text Editor (RTE) control is an easy to render in
        client side. Customer easy to edit the contents and get the HTML content for
        the displayed content. A rich text editor control provides users with a toolbar
        that helps them to apply rich text formats to the text entered in the text
        area. &lt;/p&gt;&lt;/textarea &gt;     
    * &lt;script&gt;
            * // Initialize the RTE with the tools value specified.
            * $("#rteSample").ejRTE({  tools: {
             *   //font: ["fontName", "fontSize", "fontColor", "backgroundColor"],
              *  formatStyle: ["format"],
               * style: ["bold", "italic", "underline", "strikethrough"],
                *alignment: ["justifyLeft", "justifyCenter", "justifyRight", "justifyFull"],
                *lists: ["unorderedList", "orderedList"],
                *indenting: ["outdent", "indent"],
                * doAction: ["undo", "redo"],
                * links: ["createLink"],
                * images: ["image", "video"],
                * tables: ["createTable", "addRowAbove", "addRowBelow", "addColumnLeft", "addColumnRight", "deleteRow", "deleteColumn", "deleteTable"]
            
            * }});
            * &lt;/script&gt;
            * @memberof ejRTE
            * @instance
            */
            tools: /** @lends ejRTE# */{
                //font: ["fontName", "fontSize", "fontColor", "backgroundColor"],
                /**		
                * Include Format styles in RTE toolbar.
                * @alias ejRTE#tools->formatStyle
                * @type Object
                  * @example 
            * &lt;textarea   id="rteSample"&gt;     
    * &lt;p&gt;&lt;b&gt;Description:&lt;/b&gt;&lt;/p&gt;
        &lt;p&gt;The Rich Text Editor (RTE) control is an easy to render in
        client side. Customer easy to edit the contents and get the HTML content for
        the displayed content. A rich text editor control provides users with a toolbar
        that helps them to apply rich text formats to the text entered in the text
        area. &lt;/p&gt;&lt;/textarea &gt;     
    * &lt;script&gt;
            * // Initialize the RTE with the tools value specified.
            * $("#rteSample").ejRTE({  tools: {
              *  formatStyle: ["format"]
            * }});
            * &lt;/script&gt;
            * @memberof ejRTE
            * @instance
                */
                formatStyle: ["format"],
                /**		
                * Include font styles in RTE toolbar.
                * @alias ejRTE#tools->style
                * @type Object
                  * @example 
            * &lt;textarea   id="rteSample"&gt;     
    * &lt;p&gt;&lt;b&gt;Description:&lt;/b&gt;&lt;/p&gt;
        &lt;p&gt;The Rich Text Editor (RTE) control is an easy to render in
        client side. Customer easy to edit the contents and get the HTML content for
        the displayed content. A rich text editor control provides users with a toolbar
        that helps them to apply rich text formats to the text entered in the text
        area. &lt;/p&gt;&lt;/textarea &gt;     
    * &lt;script&gt;
            * // Initialize the RTE with the tools value specified.
            * $("#rteSample").ejRTE({  tools: {
               * style: ["bold", "italic", "underline", "strikethrough"]
            * }});
            * &lt;/script&gt;
            * @memberof ejRTE
            * @instance
                */
                style: ["bold", "italic", "underline", "strikethrough"],
                /**		
                * Include font alignment options in RTE toolbar.
                * @alias ejRTE#tools->alignment
                * @type Object
                  * @example 
            * &lt;textarea   id="rteSample"&gt;     
    * &lt;p&gt;&lt;b&gt;Description:&lt;/b&gt;&lt;/p&gt;
        &lt;p&gt;The Rich Text Editor (RTE) control is an easy to render in
        client side. Customer easy to edit the contents and get the HTML content for
        the displayed content. A rich text editor control provides users with a toolbar
        that helps them to apply rich text formats to the text entered in the text
        area. &lt;/p&gt;&lt;/textarea &gt;     
    * &lt;script&gt;
            * // Initialize the RTE with the tools value specified.
            * $("#rteSample").ejRTE({  tools: {
                *alignment: ["justifyLeft", "justifyCenter", "justifyRight", "justifyFull"]
            * }});
            * &lt;/script&gt;
            * @memberof ejRTE
            * @instance
                */
                alignment: ["justifyLeft", "justifyCenter", "justifyRight", "justifyFull"],
                /**		
                * Include list type options in RTE toolbar.
                * @alias ejRTE#tools->lists
                * @type Object
                  * @example 
            * &lt;textarea   id="rteSample"&gt;     
    * &lt;p&gt;&lt;b&gt;Description:&lt;/b&gt;&lt;/p&gt;
        &lt;p&gt;The Rich Text Editor (RTE) control is an easy to render in
        client side. Customer easy to edit the contents and get the HTML content for
        the displayed content. A rich text editor control provides users with a toolbar
        that helps them to apply rich text formats to the text entered in the text
        area. &lt;/p&gt;&lt;/textarea &gt;     
    * &lt;script&gt;
            * // Initialize the RTE with the tools value specified.
            * $("#rteSample").ejRTE({  tools: {
                *lists: ["unorderedList", "orderedList"]
            * }});
            * &lt;/script&gt;
            * @memberof ejRTE
            * @instance
                */
                lists: ["unorderedList", "orderedList"],
                /**		
                * Include text indenting options in RTE toolbar.
                * @alias ejRTE#tools->indenting
                * @type Object
                  * @example 
            * &lt;textarea   id="rteSample"&gt;     
    * &lt;p&gt;&lt;b&gt;Description:&lt;/b&gt;&lt;/p&gt;
        &lt;p&gt;The Rich Text Editor (RTE) control is an easy to render in
        client side. Customer easy to edit the contents and get the HTML content for
        the displayed content. A rich text editor control provides users with a toolbar
        that helps them to apply rich text formats to the text entered in the text
        area. &lt;/p&gt;&lt;/textarea &gt;     
    * &lt;script&gt;
            * // Initialize the RTE with the tools value specified.
            * $("#rteSample").ejRTE({  tools: {
                *indenting: ["outdent", "indent"]
            * }});
            * &lt;/script&gt;
            * @memberof ejRTE
            * @instance
                */
                indenting: ["outdent", "indent"],
                //copyPaste: ["cut", "copy", "paste"],
                /**		
                * Include action options in RTE toolbar.
                * @alias ejRTE#tools->doAction
                * @type Object
                  * @example 
            * &lt;textarea   id="rteSample"&gt;     
    * &lt;p&gt;&lt;b&gt;Description:&lt;/b&gt;&lt;/p&gt;
        &lt;p&gt;The Rich Text Editor (RTE) control is an easy to render in
        client side. Customer easy to edit the contents and get the HTML content for
        the displayed content. A rich text editor control provides users with a toolbar
        that helps them to apply rich text formats to the text entered in the text
        area. &lt;/p&gt;&lt;/textarea &gt;     
    * &lt;script&gt;
            * // Initialize the RTE with the tools value specified.
            * $("#rteSample").ejRTE({  tools: {
                * doAction: ["undo", "redo"]
            * }});
            * &lt;/script&gt;
            * @memberof ejRTE
            * @instance
                */
                doAction: ["undo", "redo"],
                //clear: ["clearFormat", "clearAll"],
                /**		
                * Include links options in RTE toolbar.
                * @alias ejRTE#tools->links
                * @type Object
                  * @example 
            * &lt;textarea   id="rteSample"&gt;     
    * &lt;p&gt;&lt;b&gt;Description:&lt;/b&gt;&lt;/p&gt;
        &lt;p&gt;The Rich Text Editor (RTE) control is an easy to render in
        client side. Customer easy to edit the contents and get the HTML content for
        the displayed content. A rich text editor control provides users with a toolbar
        that helps them to apply rich text formats to the text entered in the text
        area. &lt;/p&gt;&lt;/textarea &gt;     
    * &lt;script&gt;
            * // Initialize the RTE with the tools value specified.
            * $("#rteSample").ejRTE({  tools: {
             
                * links: ["createLink"]
            * }});
            * &lt;/script&gt;
            * @memberof ejRTE
            * @instance
                */
                links: ["createLink"],
                /**		
                * Include image  and video options in RTE toolbar.
                * @alias ejRTE#tools->images
                * @type Object
                  * @example 
            * &lt;textarea   id="rteSample"&gt;     
    * &lt;p&gt;&lt;b&gt;Description:&lt;/b&gt;&lt;/p&gt;
        &lt;p&gt;The Rich Text Editor (RTE) control is an easy to render in
        client side. Customer easy to edit the contents and get the HTML content for
        the displayed content. A rich text editor control provides users with a toolbar
        that helps them to apply rich text formats to the text entered in the text
        area. &lt;/p&gt;&lt;/textarea &gt;     
    * &lt;script&gt;
            * // Initialize the RTE with the tools value specified.
            * $("#rteSample").ejRTE({  tools: {
             
                * images: ["image", "video"]
            * }});
            * &lt;/script&gt;
            * @memberof ejRTE
            * @instance
                */
                images: ["image", "video"],
                /**		
                * Include tables options in RTE toolbar.
                * @alias ejRTE#tools->tables
                * @type Object
                  * @example 
            * &lt;textarea   id="rteSample"&gt;     
    * &lt;p&gt;&lt;b&gt;Description:&lt;/b&gt;&lt;/p&gt;
        &lt;p&gt;The Rich Text Editor (RTE) control is an easy to render in
        client side. Customer easy to edit the contents and get the HTML content for
        the displayed content. A rich text editor control provides users with a toolbar
        that helps them to apply rich text formats to the text entered in the text
        area. &lt;/p&gt;&lt;/textarea &gt;     
    * &lt;script&gt;
            * // Initialize the RTE with the tools value specified.
            * $("#rteSample").ejRTE({  tools: {
             
                * tables: ["createTable", "addRowAbove", "addRowBelow", "addColumnLeft", "addColumnRight", "deleteRow", "deleteColumn", "deleteTable"]
            
            * }});
            * &lt;/script&gt;
            * @memberof ejRTE
            * @instance
                */
                tables: ["createTable", "addRowAbove", "addRowBelow", "addColumnLeft", "addColumnRight", "deleteRow", "deleteColumn", "deleteTable"]
                //editTable:[""],
                //scripts: ["superscript", "subscript"],
                //casing: ["upperCase", "lowerCase"],
                //paragraph: ["paragraph"],
                //custom: null
            },
            /**		
            * Sets the toolsList in RTE. 
            * @default 	null
            * @type {object}
            * @example 
            * &lt;textarea   id="rteSample"&gt;     
    * &lt;p&gt;&lt;b&gt;Description:&lt;/b&gt;&lt;/p&gt;
        &lt;p&gt;The Rich Text Editor (RTE) control is an easy to render in
        client side. Customer easy to edit the contents and get the HTML content for
        the displayed content. A rich text editor control provides users with a toolbar
        that helps them to apply rich text formats to the text entered in the text
        area. &lt;/p&gt;&lt;/textarea &gt;     
    * &lt;script&gt;
            * // Initialize the RTE with the toolsList value specified.
            * $("#rteSample").ejRTE({ toolsList: ["formatStyle", "font", "style", "scripts", "alignment", "lists", "indenting", "copyPaste", "doAction", "clear", "links", "images", "tables", "casing", "customTool"]});
            * &lt;/script&gt;
            * @memberof ejRTE
            * @instance
            */
            toolsList: ["formatStyle", "font", "style", "scripts", "alignment", "lists", "indenting", "copyPaste", "doAction", "clear", "links", "images", "tables", "casing", "customTool"],
            /**		
            * Sets the colorCode in RTE. 
            * @default 	null
            * @type {object}
            * @example 
           * &lt;textarea   id="rteSample"&gt;     
    * &lt;p&gt;&lt;b&gt;Description:&lt;/b&gt;&lt;/p&gt;
        &lt;p&gt;The Rich Text Editor (RTE) control is an easy to render in
        client side. Customer easy to edit the contents and get the HTML content for
        the displayed content. A rich text editor control provides users with a toolbar
        that helps them to apply rich text formats to the text entered in the text
        area. &lt;/p&gt;&lt;/textarea &gt;     
    * &lt;script&gt;
            * // Initialize the RTE with the colorCode value specified.
            * $("#rteSample").ejRTE({ colorCode: [
			            "000000", "FFFFFF", "C4C4C4", "ADADAD", "595959", "262626", "4f81bd", "dbe5f1", "b8cce4", "95b3d7", "366092", "244061", "c0504d", "f2dcdb", "e5b9b7", "d99694", "953734",
			            "632423", "9bbb59", "ebf1dd", "d7e3bc", "c3d69b", "76923c", "4f6128", "8064a2", "e5e0ec", "ccc1d9", "b2a2c7", "5f497a", "3f3151", "f79646", "fdeada", "fbd5b5", "fac08f",
			            "e36c09", "974806"
            ]});
            * &lt;/script&gt;
            * @memberof ejRTE
            * @instance
            */
            colorCode: [
			            "000000", "FFFFFF", "C4C4C4", "ADADAD", "595959", "262626", "4f81bd", "dbe5f1", "b8cce4", "95b3d7", "366092", "244061", "c0504d", "f2dcdb", "e5b9b7", "d99694", "953734",
			            "632423", "9bbb59", "ebf1dd", "d7e3bc", "c3d69b", "76923c", "4f6128", "8064a2", "e5e0ec", "ccc1d9", "b2a2c7", "5f497a", "3f3151", "f79646", "fdeada", "fbd5b5", "fac08f",
			            "e36c09", "974806"
            ],
            /**		
            * Sets the format in RTE. 
            * @default 	""
            * @type {string}
            * @example 
           * &lt;textarea   id="rteSample"&gt;     
    * &lt;p&gt;&lt;b&gt;Description:&lt;/b&gt;&lt;/p&gt;
        &lt;p&gt;The Rich Text Editor (RTE) control is an easy to render in
        client side. Customer easy to edit the contents and get the HTML content for
        the displayed content. A rich text editor control provides users with a toolbar
        that helps them to apply rich text formats to the text entered in the text
        area. &lt;/p&gt;&lt;/textarea &gt;     
    * &lt;script&gt;
            * // Initialize the RTE with the format value specified.
            * $("#rteSample").ejRTE({format: [
                { text: "Paragraph", value: "<p>", spriteCssClass: "e-paragraph" },
                { text: "Quotation", value: "<blockquote>", spriteCssClass: "e-quotation" },
                { text: "Heading 1", value: "<h1>", spriteCssClass: "e-h1" },
                { text: "Heading 2", value: "<h2>", spriteCssClass: "e-h2" },
                { text: "Heading 3", value: "<h3>", spriteCssClass: "e-h3" },
                { text: "Heading 4", value: "<h4>", spriteCssClass: "e-h4" },
                { text: "Heading 5", value: "<h5>", spriteCssClass: "e-h5" },
                { text: "Heading 6", value: "<h6>", spriteCssClass: "e-h6" }
            * ]});
            * &lt;/script&gt;
            * @memberof ejRTE
            * @instance
            */
            format: [
                { text: "Paragraph", value: "<p>", spriteCssClass: "e-paragraph" },
                { text: "Quotation", value: "<blockquote>", spriteCssClass: "e-quotation" },
                { text: "Heading 1", value: "<h1>", spriteCssClass: "e-h1" },
                { text: "Heading 2", value: "<h2>", spriteCssClass: "e-h2" },
                { text: "Heading 3", value: "<h3>", spriteCssClass: "e-h3" },
                { text: "Heading 4", value: "<h4>", spriteCssClass: "e-h4" },
                { text: "Heading 5", value: "<h5>", spriteCssClass: "e-h5" },
                { text: "Heading 6", value: "<h6>", spriteCssClass: "e-h6" }
            ],
            /**		
            * Sets the fontName in RTE. 
            * @default 	null
            * @type {object}
            * @example 
            * &lt;textarea   id="rteSample"&gt;     
    * &lt;p&gt;&lt;b&gt;Description:&lt;/b&gt;&lt;/p&gt;
        &lt;p&gt;The Rich Text Editor (RTE) control is an easy to render in
        client side. Customer easy to edit the contents and get the HTML content for
        the displayed content. A rich text editor control provides users with a toolbar
        that helps them to apply rich text formats to the text entered in the text
        area. &lt;/p&gt;&lt;/textarea &gt;     
    * &lt;script&gt;
            * // Initialize the RTE with the fontName value specified.
            * $("#rteSample").ejRTE({format: [
                { text: "Paragraph", value: "<p>", spriteCssClass: "e-paragraph" },
                { text: "Quotation", value: "<blockquote>", spriteCssClass: "e-quotation" },
                { text: "Heading 1", value: "<h1>", spriteCssClass: "e-h1" },
                { text: "Heading 2", value: "<h2>", spriteCssClass: "e-h2" },
                { text: "Heading 3", value: "<h3>", spriteCssClass: "e-h3" },
                { text: "Heading 4", value: "<h4>", spriteCssClass: "e-h4" },
                { text: "Heading 5", value: "<h5>", spriteCssClass: "e-h5" },
                { text: "Heading 6", value: "<h6>", spriteCssClass: "e-h6" }
           * ]});
           * &lt;/script&gt;
            * @memberof ejRTE
            * @instance
            */
            fontName: [
                       {
                           text: "Segoe UI",
                           value: "Segoe UI"
                       },
                       {
                           text: "Arial",
                           value: "Arial,Helvetica,sans-serif"
                       },
                       {
                           text: "Courier New",
                           value: "Courier New,Courier,monospace"
                       },
                       {
                           text: "Georgia",
                           value: "Georgia,serif"
                       },
                       {
                           text: "Impact",
                           value: "Impact,Charcoal,sans-serif"
                       },
                       {
                           text: "Lucida Console",
                           value: "Lucida Console,Monaco,monospace"
                       },
                       {
                           text: "Tahoma",
                           value: "Tahoma,Geneva,sans-serif"
                       },
                       {
                           text: "Times New Roman",
                           value: "Times New Roman,Times,serif"
                       },
                       {
                           text: "Trebuchet MS",
                           value: "Trebuchet MS,Helvetica,sans-serif"
                       },
                       {
                           text: "Verdana",
                           value: "Verdana,Geneva,sans-serif"
                       }
            ],
            /**		
            * Sets the fontSize in RTE. 
            * @default 	null
            * @type {object}
            * @example 
            * &lt;textarea   id="rteSample"&gt;     
    * &lt;p&gt;&lt;b&gt;Description:&lt;/b&gt;&lt;/p&gt;
        &lt;p&gt;The Rich Text Editor (RTE) control is an easy to render in
        client side. Customer easy to edit the contents and get the HTML content for
        the displayed content. A rich text editor control provides users with a toolbar
        that helps them to apply rich text formats to the text entered in the text
        area. &lt;/p&gt;&lt;/textarea &gt;     
    * &lt;script&gt;
            * // Initialize the RTE with the fontSize value specified.
            * $("#rteSample").ejRTE({ fontSize: [
                {
                    text: "1",
                    value: "1"
                },
                {
                    text: "2 (10pt)",
                    value: "2"
                },
                {
                    text: "3 (12pt)",
                    value: "3"
                },
                {
                    text: "4 (14pt)",
                    value: "4"
                },
                {
                    text: "5 (18pt)",
                    value: "5"
                },
                {
                    text: "6 (24pt)",
                    value: "6"
                },
                {
                    text: "7 (36pt)",
                    value: "7"
                }
            * ]});
            * &lt;/script&gt;	
            * @memberof ejRTE
            * @instance
            */
            fontSize: [
                {
                    text: "1",
                    value: "1"
                },
                {
                    text: "2 (10pt)",
                    value: "2"
                },
                {
                    text: "3 (12pt)",
                    value: "3"
                },
                {
                    text: "4 (14pt)",
                    value: "4"
                },
                {
                    text: "5 (18pt)",
                    value: "5"
                },
                {
                    text: "6 (24pt)",
                    value: "6"
                },
                {
                    text: "7 (36pt)",
                    value: "7"
                }
            ],
            /**		
            * Given number for rows render the insert table pop. 
            * @default 	""
            * @type {number}
            * @example 
            * &lt;textarea   id="rteSample"&gt;     
    * &lt;p&gt;&lt;b&gt;Description:&lt;/b&gt;&lt;/p&gt;
        &lt;p&gt;The Rich Text Editor (RTE) control is an easy to render in
        client side. Customer easy to edit the contents and get the HTML content for
        the displayed content. A rich text editor control provides users with a toolbar
        that helps them to apply rich text formats to the text entered in the text
        area. &lt;/p&gt;&lt;/textarea &gt;     
    * &lt;script&gt;
            * // Initialize the RTE with the 'tableRows' value specified.
            * $("#rteSample").ejRTE({tableRows: 70 });
           * &lt;/script&gt;
            * @memberof ejRTE
            * @instance
            */
            tableRows: 7,
            /**		
            * Given number for columns render the insert table pop. 
            * @default 	""
            * @type {number}
            * @example 
            * &lt;textarea   id="rteSample"&gt;     
    * &lt;p&gt;&lt;b&gt;Description:&lt;/b&gt;&lt;/p&gt;
        &lt;p&gt;The Rich Text Editor (RTE) control is an easy to render in
        client side. Customer easy to edit the contents and get the HTML content for
        the displayed content. A rich text editor control provides users with a toolbar
        that helps them to apply rich text formats to the text entered in the text
        area. &lt;/p&gt;&lt;/textarea &gt;     
    * &lt;script&gt;
            * // Initialize the RTE with the 'tableColumns' value specified.
            * $("#rteSample").ejRTE({tableColumns: 70 });
           * &lt;/script&gt; 
            * @memberof ejRTE
            * @instance
            */
            tableColumns: 9,
            /**		
            * Given number for rows render the color palete pop up.
            * @default 	""
            * @type {number}
            * @example 
            * &lt;textarea   id="rteSample"&gt;     
    * &lt;p&gt;&lt;b&gt;Description:&lt;/b&gt;&lt;/p&gt;
        &lt;p&gt;The Rich Text Editor (RTE) control is an easy to render in
        client side. Customer easy to edit the contents and get the HTML content for
        the displayed content. A rich text editor control provides users with a toolbar
        that helps them to apply rich text formats to the text entered in the text
        area. &lt;/p&gt;&lt;/textarea &gt;     
    * &lt;script&gt;
            * // Initialize the RTE with the 'colorPaletteRows' value specified.
            * $("#rteSample").ejRTE({colorPaletteRows: 70 });
            * &lt;/script&gt;
            * @memberof ejRTE
            * @instance
            */
            colorPaletteRows: 6,
            /**		
            * Given number for columns render the color palete pop up. 
            * @default 	""
            * @type {number}
            * @example 
            * &lt;textarea   id="rteSample"&gt;     
    * &lt;p&gt;&lt;b&gt;Description:&lt;/b&gt;&lt;/p&gt;
        &lt;p&gt;The Rich Text Editor (RTE) control is an easy to render in
        client side. Customer easy to edit the contents and get the HTML content for
        the displayed content. A rich text editor control provides users with a toolbar
        that helps them to apply rich text formats to the text entered in the text
        area. &lt;/p&gt;&lt;/textarea &gt;     
    * &lt;script&gt;
            * // Initialize the RTE with the 'colorPaletteColumns' value specified.
            * $("#rteSample").ejRTE({colorPaletteColumns: 70 });
            * &lt;/script&gt;
            * @memberof ejRTE
            * @instance
            */
            colorPaletteColumns: 6,
            /**		
            * Shows CustomTable in RTE. 
            * @default 	True
            * @type {boolean}
            * @example 
            * &lt;textarea   id="rteSample"&gt;     
    * &lt;p&gt;&lt;b&gt;Description:&lt;/b&gt;&lt;/p&gt;
        &lt;p&gt;The Rich Text Editor (RTE) control is an easy to render in
        client side. Customer easy to edit the contents and get the HTML content for
        the displayed content. A rich text editor control provides users with a toolbar
        that helps them to apply rich text formats to the text entered in the text
        area. &lt;/p&gt;&lt;/textarea &gt;     
    * &lt;script&gt;
            * // Initialize the RTE with the showCustomTable value specified.
            * $("#rteSample").ejRTE({showCustomTable: false });
           * &lt;/script&gt;
            * @memberof ejRTE
            * @instance
            */
            showCustomTable: true,
            /**		
            * Given string value to display in the editable area. 
            * @default 	""
            * @type {string}
            * @example 
            * &lt;textarea   id="rteSample"&gt;     
    * &lt;p&gt;&lt;b&gt;Description:&lt;/b&gt;&lt;/p&gt;
        &lt;p&gt;The Rich Text Editor (RTE) control is an easy to render in
        client side. Customer easy to edit the contents and get the HTML content for
        the displayed content. A rich text editor control provides users with a toolbar
        that helps them to apply rich text formats to the text entered in the text
        area. &lt;/p&gt;&lt;/textarea &gt;     
    * &lt;script&gt;
            * // Initialize the RTE with the 'value' value specified.
            * $("#rteSample").ejRTE({value: "The Rich Text Editor (RTE) control is an easy to render in client side. Customer easy to edit the contents, insert table, images and get the HTML content for the displayed content." });
           * &lt;/script&gt;
            * @memberof ejRTE
            * @instance
            */
            value: null,
            /**		
            * Undo operation provide the number of step limit. 
            * @default 	""
            * @type {number}
            * @example 
            * &lt;textarea   id="rteSample"&gt;     
    * &lt;p&gt;&lt;b&gt;Description:&lt;/b&gt;&lt;/p&gt;
        &lt;p&gt;The Rich Text Editor (RTE) control is an easy to render in
        client side. Customer easy to edit the contents and get the HTML content for
        the displayed content. A rich text editor control provides users with a toolbar
        that helps them to apply rich text formats to the text entered in the text
        area. &lt;/p&gt;&lt;/textarea &gt;     
    * &lt;script&gt;
            * // Initialize the RTE with the 'vaundoStackLimitlue' value specified.
            * $("#rteSample").ejRTE({undoStackLimit: 70 });
            * &lt;/script&gt;
            * @memberof ejRTE
            * @instance
            */
            undoStackLimit: 50,
            /**		
            * Shows enableResize in RTE. 
            * @default 	True
            * @type {boolean}
            * @example 
            * &lt;textarea   id="rteSample"&gt;     
    * &lt;p&gt;&lt;b&gt;Description:&lt;/b&gt;&lt;/p&gt;
        &lt;p&gt;The Rich Text Editor (RTE) control is an easy to render in
        client side. Customer easy to edit the contents and get the HTML content for
        the displayed content. A rich text editor control provides users with a toolbar
        that helps them to apply rich text formats to the text entered in the text
        area. &lt;/p&gt;&lt;/textarea &gt;     
    * &lt;script&gt;
            * // Initialize the RTE with the enableResize value specified.
            * $("#rteSample").ejRTE({enableResize: false });
           * &lt;/script&gt;
            * @memberof ejRTE
            * @instance
            */
            enableResize: true,
            /**		
            *  enablePersistence the values in RTE. 
            * @default 	True
            * @type {boolean}
            * @example 
            * &lt;textarea   id="rteSample"&gt;     
    * &lt;p&gt;&lt;b&gt;Description:&lt;/b&gt;&lt;/p&gt;
        &lt;p&gt;The Rich Text Editor (RTE) control is an easy to render in
        client side. Customer easy to edit the contents and get the HTML content for
        the displayed content. A rich text editor control provides users with a toolbar
        that helps them to apply rich text formats to the text entered in the text
        area. &lt;/p&gt;&lt;/textarea &gt;     
    * &lt;script&gt;
            * // Initialize the RTE with the enablePersistence value specified.
            * $("#rteSample").ejRTE({enablePersistence: false });
            * &lt;/script&gt;
            * @memberof ejRTE
            * @instance
            */
            enablePersistence: false,
            /**		
            * shows enableRTL in RTE. 
            * @default 	True
            * @type {boolean}
            * @example 
            * &lt;textarea   id="rteSample"&gt;     
    * &lt;p&gt;&lt;b&gt;Description:&lt;/b&gt;&lt;/p&gt;
        &lt;p&gt;The Rich Text Editor (RTE) control is an easy to render in
        client side. Customer easy to edit the contents and get the HTML content for
        the displayed content. A rich text editor control provides users with a toolbar
        that helps them to apply rich text formats to the text entered in the text
        area. &lt;/p&gt;&lt;/textarea &gt;     
    * &lt;script&gt;
            * // Initialize the RTE with the enableRTL value specified.
            * $("#rteSample").ejRTE({enableRTL: true });
           * &lt;/script&gt;
            * @memberof ejRTE
            * @instance
            */
            enableRTL: false,
            //events
            /**     
            * Fires when change successfully.
            * @event
            * @name ejRTE#change 	
            * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
            * @param {object}  argument.model returns the RTE model
            * @param {string}  argument.type returns the name of the event
            * @example 
            * &lt;textarea   id="rteSample"&gt;     
    * &lt;p&gt;&lt;b&gt;Description:&lt;/b&gt;&lt;/p&gt;
        &lt;p&gt;The Rich Text Editor (RTE) control is an easy to render in
        client side. Customer easy to edit the contents and get the HTML content for
        the displayed content. A rich text editor control provides users with a toolbar
        that helps them to apply rich text formats to the text entered in the text
        area. &lt;/p&gt;&lt;/textarea &gt;     
    * &lt;script&gt;
    * $("#rteSample").ejRTE();
            * //change event for RTE
            * $("#rteSample").ejRTE({ 
            *   	change: function(args) {}
            * });      
            * &lt;/script&gt;
            * @memberof ejRTE
            * @instance
            */
            change: null,
            /**     
            * Fires when execute successfully.
            * @event
            * @name ejRTE#execute 	
            * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
            * @param {object}  argument.model returns the RTE model
            * @param {string}  argument.type returns the name of the event
            * @example 
            * &lt;textarea   id="rteSample"&gt;     
    * &lt;p&gt;&lt;b&gt;Description:&lt;/b&gt;&lt;/p&gt;
        &lt;p&gt;The Rich Text Editor (RTE) control is an easy to render in
        client side. Customer easy to edit the contents and get the HTML content for
        the displayed content. A rich text editor control provides users with a toolbar
        that helps them to apply rich text formats to the text entered in the text
        area. &lt;/p&gt;&lt;/textarea &gt;     
    * &lt;script&gt;
    * $("#rteSample").ejRTE();
            * //execute event for RTE
            * $("#rteSample").ejRTE({ 
            *   	execute: function(args) {}
            * });   
            * &lt;/script&gt;   
            * @memberof ejRTE
            * @instance
            */
            execute: null,
            /**     
            * Fires when keydown successfully.
            * @event
            * @name ejRTE#keydown 	
            * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
            * @param {object}  argument.model returns the RTE model
            * @param {string}  argument.type returns the name of the event
            * @example 
            * &lt;textarea   id="rteSample"&gt;     
    * &lt;p&gt;&lt;b&gt;Description:&lt;/b&gt;&lt;/p&gt;
        &lt;p&gt;The Rich Text Editor (RTE) control is an easy to render in
        client side. Customer easy to edit the contents and get the HTML content for
        the displayed content. A rich text editor control provides users with a toolbar
        that helps them to apply rich text formats to the text entered in the text
        area. &lt;/p&gt;&lt;/textarea &gt;     
    * &lt;script&gt;
    * $("#rteSample").ejRTE();
            * //keydown event for RTE
            * $("#rteSample").ejRTE({ 
            *   	keydown: function(args) {}
            * });  
            * &lt;/script&gt;    
            * @memberof ejRTE
            * @instance
            */
            keydown: null,
            /**     
            * Fires when keyup successfully.
            * @event
            * @name ejRTE#keyup 	
            * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
            * @param {object}  argument.model returns the RTE model
            * @param {string}  argument.type returns the name of the event
            * @example 
            * &lt;textarea   id="rteSample"&gt;     
    * &lt;p&gt;&lt;b&gt;Description:&lt;/b&gt;&lt;/p&gt;
        &lt;p&gt;The Rich Text Editor (RTE) control is an easy to render in
        client side. Customer easy to edit the contents and get the HTML content for
        the displayed content. A rich text editor control provides users with a toolbar
        that helps them to apply rich text formats to the text entered in the text
        area. &lt;/p&gt;&lt;/textarea &gt;     
    * &lt;script&gt;
    * $("#rteSample").ejRTE();
            * //keyup event for RTE
            * $("#rteSample").ejRTE({ 
            *   	keyup: function(args) {}
            * });  
            * &lt;/script&gt;    
            * @memberof ejRTE
            * @instance
            */
            keyup: null,
            create: null,
            destroy: null
        },
        /**
         * Specify the data types for default properties 
		 * @private
         */
        dataTypes: {
            iframeAttribute:"string",
            cssClass: "string",
            name:"string",
            enabled: "boolean",
            enablePersistence: "boolean",
            toolsList: "array",
            colorCode: "array",
            format: "data",
            fontName: "data",
            fontSize: "data",
            tools: {
                formatStyle: "array",
                font: "array",
                style: "array",
                scripts: "array",
                alignment: "array",
                lists: "array",
                indenting: "array",
                copyPaste: "array",
                doAction: "array",
                clear: "array",
                links: "array",
                images: "array",
                tables: "array",
                casing: "array",
                customTool: "array"
            }
        },

        observables: ["value"],
        value:ej.util.valueFunction("value"),
        /**
        * To configure the properties at runtime using SetModel		
        * @private
        */
        _setModel: function (options) {
            if (!this.model.enabled && ej.isNullOrUndefined(options["enabled"])) return false;
            var option;
            for (option in options) {
                switch (option) {
                    case "allowEditing": this._enableEdit(options[option]); break;
                    case "showToolbar": {
                        options[option] ? ej.isNullOrUndefined(this._rteToolbar) ? this._renderToolBar() : this._rteToolbar.show() : this._rteToolbar.hide();
                        this._setIframeHeight();
                        break;
                    }
                    case "showFooter": {
                        this.model.showFooter = options[option];
                        options[option] ? ej.isNullOrUndefined(this._rteFooter) ? this._renderFooter() : this._rteFooter.show() : this._rteFooter.hide();
                        this._setIframeHeight();
                        break;
                    }
                    case "enabled": this._disabled(!options[option]); break;
                    case "height": this._rteWapper.height(options[option]); this._setIframeHeight(); break;
                    case "width": this._rteWapper.width(options[option]); break;
                    case "cssClass": this._changeSkin(options[option]); break;
                    case "showHtmlSource": this._footerElement("div.e-rte-source", options[option]); break;
                    case "showHtmlTagInfo": this._footerElement("div.e-rte-htmltaginfo", options[option]); break;
                    case "showWordCount": this._footerElement("div.e-rte-wordcount", options[option]); break;
                    case "showClearAll": this._footerElement("div.clearAll", options[option]); break;
                    case "showClearFormat": this._footerElement("div.clearFormat", options[option]); break;
                    case "enableResize": this._footerElement("div.e-rte-resize", options[option]); break;
                    case "fontName": this._fontStyleDDL.ejDropDownList({ dataSource: options[option] }); break;
                    case "fontSize": this._fontSizeDDL.ejDropDownList({ dataSource: options[option] }); break;
                    case "format": this._formatDDL.ejDropDownList({ dataSource: options[option] }); break;
                    case "value": this._getDocument().body.innerHTML = ej.util.getVal(options[option]); break;
                    case "showDimensions":
                        if (options[option]) {
                            this._rteWapper.find("#" + this._rteId + "_imgdimensions").show();
                            this._rteWapper.find("#" + this._rteId + "_videodimensions").show();
                        }
                        else {
                            this._rteWapper.find("#" + this._rteId + "_imgdimensions").hide();
                            this._rteWapper.find("#" + this._rteId + "_videodimensions").hide();
                        }
                        break;
                    case "locale":
                        this.model.locale = options[option];
                        var model = this.model;
                        this.element.ejRTE("destroy").ejRTE(model);
                        break;
                    case "tableRows": {
                        this.model.tableRows = options[option];
                        var tblDiv = this._createTable.find("div.e-rte-table").html("");
                        this._drawTable(tblDiv);
                        break;
                    }
					case "iframeAttribute":{
						this._updateIframeSkin(options[option]);
						break;
					}
                    case "tableColumns": {
                        this.model.tableColumns = options[option];
                        var tblDiv = this._createTable.find("div.e-rte-table").html("");
                        this._drawTable(tblDiv);
                        break;
                    }
                    case "showCustomTable": {
                        var custLink = this._createTable.find("div.customtable-group");
                        options[option] ? ej.isNullOrUndefined(this._customTableDialog) ? this._createCustomTable() : custLink.show() : custLink.hide();
                        break;
                    }
                    case "colorPaletteRows": {
                        this.model.colorPaletteRows = options[option];
                        this._updateColorPalette();
                        break;
                    }
                    case "colorPaletteColumns": {
                        this.model.colorPaletteColumns = options[option];
                        this._updateColorPalette();
                        break;
                    }
                    case "colorCode": {
                        this.model.colorCode = options[option];
                        this._updateColorPalette();
                        break;
                    }
                    case "enableRTL": this._enableRTL(options[option]); break;
                }
            }
        },

        // constructor function
        _init: function () {
            if (!ej.isNullOrUndefined(this.element) && this.element[0].type == "textarea") {
                this._initialize();
                this._render();
                this.model.allowEditing && this._wireEvents();
                this._updateIframeSkin(this.model.iframeAttribute);
            }
        },

        _destroy: function () {
            this.element.insertBefore(this._rteWapper);
            !ej.isNullOrUndefined(this._formatDDL) && this._formatDDL.ejDropDownList("destroy");
            !ej.isNullOrUndefined(this._fontStyleDDL) && this._fontStyleDDL.ejDropDownList("destroy");
            !ej.isNullOrUndefined(this._fontSizeDDL) && this._fontSizeDDL.ejDropDownList("destroy");
            !ej.isNullOrUndefined(this._fontColorSplit) && this._fontColorSplit.ejSplitButton("destroy");
            !ej.isNullOrUndefined(this._bgColorSplit) &&  this._bgColorSplit.ejSplitButton("destroy");
            if (!ej.isNullOrUndefined(this._customTableDialog)) {
                !ej.isNullOrUndefined(this._customTableDialog.find("#" + this._rteId + "_ddlAlignment")) && this._customTableDialog.find("#" + this._rteId + "_ddlAlignment").ejDropDownList("destroy");
                !ej.isNullOrUndefined(this._customTableDialog.find("#" + this._rteId + "_txtBorder")) && this._customTableDialog.find("#" + this._rteId + "_txtBorder").ejDropDownList("destroy");
            }
            this._rteWapper.remove();
            this.element.show();
            this._createTable = null;
        },

        _initialize: function () {
            this._toolsList = this.model.toolsList;
            this._rteId = this.element[0].id;
            this._backupArray = new Array();
            this._undoRedoPosition = 0;
            this._fontColor = "";
            this._bgColor = "";
            this._imgWidth = 0;
            this._imgHeight = 0;
            this._videoWidth = 0;
            this._videoHeight = 0;
            this._keypressFlag = true;
            this._styleItems = ej.isNullOrUndefined(this.model.tools["style"]) ? [] : this.model.tools["style"];
            this._alignItems = ej.isNullOrUndefined(this.model.tools["alignment"]) ? [] : this.model.tools["alignment"];
            this._listItems = ej.isNullOrUndefined(this.model.tools["lists"]) ? [] : this.model.tools["lists"];
            this._scriptsItems = ej.isNullOrUndefined(this.model.tools["scripts"]) ? [] : this.model.tools["scripts"];
        },

        _render: function () {
            this._renderWrapper();
            this._checkNameAttr();
            if (this.model.showToolbar) {
                this._renderToolBar();
            }
            this._renderEditArea();
            this.model.showFooter && this._renderFooter();
            this._setIframeHeight();
            this._renderAlertDialog();
        },
        _checkNameAttr: function () {
            if (!this.model.name)
                this.element.attr({ "name": this.element[0].id });
            else
                this.element.attr({ "name": this.model.name });
        },
        _enableEdit: function (args) {
            if (args) {
                this._wireEvents();
                this.model.showToolbar && this._toolBarItems.ejToolbar("option", "click", this._toolBarClick);
                this._rteIframe.contents().find("body").attr("contenteditable", true);
                //this._getDocument().designMode = "On"
            }
            else {
                this._unwireEvents();
                this.model.showToolbar && this._toolBarItems.ejToolbar("option", "click", null);
                this._rteIframe.contents().find("body").attr("contenteditable", false);
                //this._getDocument().designMode = "off"
            }
        },
        _changeSkin: function (value) {
            this._rteWapper.removeClass(this.model.cssClass).addClass(value);
            this._subControlsSetModel("cssClass", value);
        },
		
        _updateIframeSkin: function (value) {
            var target = this._rteIframe.contents().find("body");
		    this.model.enableRTL===true ? (target.attr("style", value),this._enableRTL(this.model.enableRTL)) : target.attr("style", value);
        },

        // RTE wrapper creating.
        _renderWrapper: function () {
            this._rteWapper = ej.buildTag("div.e-rte e-rte-wrapper " + this.model.cssClass + "#" + this._rteId + "_wrapper", "", {}, { role: "presentation" }).insertBefore(this.element);
            this._rteWapper.height(this.model.height).width(this.model.width);
            this._rteWapper.append(this.element.hide());
        },
        //Toolbar creating
        _renderToolBar: function () {
            this._rteToolbar = ej.buildTag("div.e-rte-toolbar #" + this._rteId + "_tools").insertAfter(this.element);
            this._crateToolbarTemplate();
            var model = {};
            model.click = this.model.allowEditing ? this._toolBarClick : null;
            model.cssClass = this.model.cssClass;
            model.enableRTL = this.model.enableRTL;
            this._toolBarItems.ejToolbar(model);

            //Init the toolbar items
            this._initToolBarItems();

        },

        _crateToolbarTemplate: function () {
            this._toolBarItems = ej.buildTag("div#" + this._rteId + "_toolbar").appendTo(this._rteToolbar).height(30);
            for (var items in this._toolsList) {
                items = this._toolsList[items];
                if (!ej.isNullOrUndefined(this.model.tools[items])) {
                    if (items == "customTool")
                        !ej.isNullOrUndefined(this.model.tools[items]) && this._customTools(this.model.tools[items]);
                    else
                        this.model.tools[items].length > 0 && this._createToolsItems(this.model.tools[items], items);
                }
            }
        },

        _createToolsItems: function (items, itemName) {
            if (itemName == "font" && !this.model.showFontOption)
                return false;
            var ulTag = ej.buildTag("ul#" + (this._rteId + itemName)), liTag;
            (itemName == "font") && ulTag.addClass("e-rte-fontgroup");
            (itemName == "formatStyle") && ulTag.addClass("e-rte-format");
            for (var i = 0; i < items.length; i++) {
                liTag = $("<li id='" + (this._rteId + items[i].replace(/ /g, '')) + "' title='" + this._getLocalizedLabels(items[i].replace(/ /g, '')) + "' ><div class='e-rte-toolbar-icon " + items[i] + "'></div></li>");
                liTag.appendTo(ulTag);
            }
            ulTag.appendTo(this._toolBarItems);
        },

        _customTools: function (toolbarItems) {
            for (var item in toolbarItems) {
            var ulTag = ej.buildTag("ul"), liTag;
			    liTag = $("<li id='" + toolbarItems[item].name + "' title='" + toolbarItems[item].tooltip + "' ><div class='" + (ej.isNullOrUndefined(toolbarItems[item].css) ? "" : toolbarItems[item].css) + "'>"+toolbarItems[item].name+"</div></li>");
                var fn = toolbarItems[item].action;
                if (typeof fn === "string") {
                    fn = ej.util.getObject(fn, window);
                }
                !ej.isNullOrUndefined(toolbarItems[item].action) && this._on(liTag, "click", fn);
                $(toolbarItems[item].template).appendTo(liTag.find("div"));
            liTag.appendTo(ulTag);
            ulTag.appendTo(this._toolBarItems);
            }
        },

        _initToolBarItems: function () {
            this._toolBarObj = this._toolBarItems.ejToolbar("instance");
            this._rteToolbar.find("#" + this._rteId + "format").length > 0 && this._renderFormat();
            this._rteToolbar.find("#" + this._rteId + "createLink").length > 0 && this._renderLinkDialog();
            this._rteToolbar.find("#" + this._rteId + "video").length > 0 && this._renderVideoDialog();
            this._rteToolbar.find("#" + this._rteId + "image").length > 0 && this._renderImageDialog();
            this._rteToolbar.find("#" + this._rteId + "createTable").length > 0 && this._renderTableDialog();
            this._rteToolbar.find("#" + this._rteId + "fontName").length > 0 && this._renderFontStyle();
            this._rteToolbar.find("#" + this._rteId + "fontSize").length > 0 && this._renderFontSize();
            this._rteToolbar.find("#" + this._rteId + "fontColor").length > 0 && this._renderFontColor();
            this._rteToolbar.find("#" + this._rteId + "backgroundColor").length > 0 && this._renderBGColor();
            this._rteToolbar.find("#" + this._rteId + "createTable").length > 0 && this._renderEditTableDialog();
            !this.model.showClearAll && this._rteToolbar.find("#" + this._rteId + "clearAll").hide();
            !this.model.showClearAll && this._rteToolbar.find("#" + this._rteId + "clearAll").hide();
            this._toolBarObj.disableItemByID(this._rteId + "undo");
            this._toolBarObj.disableItemByID(this._rteId + "redo");
            this._toolBarObj.selectItemByID(this._rteId + "justifyLeft");
            //Hide the editable table icons
            this._toolBarItems.find("#" + this._rteId + "addColumnLeft, #" + this._rteId + "addColumnRight, #" + this._rteId + "addRowAbove, #"
             + this._rteId + "addRowBelow, #" + this._rteId + "deleteRow, #" + this._rteId + "deleteColumn, #" + this._rteId + "deleteTable").hide();
        },
        //Editable Area
        _renderEditArea: function () {
            var _rteEditor = ej.buildTag("div.editarea #" + this._rteId + "_editor").appendTo(this._rteWapper);
            this._rteIframe = ej.buildTag("iframe.content-iframe #" + this._rteId + "_Iframe").appendTo(_rteEditor);
            this._setIFrames();
            if(navigator.userAgent.match(/iPad/i) != null)
                _rteEditor.addClass('e-ipadscroll');
        },

        //Sets the editor and source editors
        _setIFrames: function () {
            //var RTE = $find(this._rteId);
            this.value(this.value() == null ? $.trim(this.element[0].value) : this.value());
            this._iframeId = this._rteId + "_Iframe";
            var _htmlText = ("<!DOCTYPE html><html><head><meta charset='utf-8' /><style>html,body{padding:0;margin:-1px 5px 0;height:100%;min-height:100%; padding-top: 1px;font-size: 13px;font-family:Segoe UI;word-wrap: break-word;-webkit-nbsp-mode: space;-webkit-line-break: after-white-space;}"
                + "h1{font-size:2em;margin:.67em 0}h2{font-size:1.5em}h3{font-size:1.16em}h4{font-size:1em}h5{font-size:.83em}h6{font-size:.7em}"
                + "ul,ol{padding-left:2.5em} a{color:#00a} blockquote{margin-right: 0}"
                + ".e-rte-table{border-spacing:0;margin: 0.2em 0 0.5em;border: 1px solid;}.e-rte-table caption{border: 1px solid;border-bottom:none}"
                + ".e-rte-table,.e-rte-table td{outline:0;border-collapse: collapse; min-height:1px;vertical-align: top;text-indent: 5px;} .e-rte-table p{margin:0;padding:0;}"
                + "</style></head><body autocorrect='off' contenteditable=" + (this.model.allowEditing ? 'true' : 'false') + ">" + $.trim(this.value()) + "</body></html>");
            this._rteIframe.css({ "width": "100%" });
            //this.model.allowEditing && (this._getDocument().designMode = "On");
            this._setContent(_htmlText);
            if (!ej.isNullOrUndefined(this.model.undoStackLimit) || this.model.undoStackLimit !== "" || this.model.undoStackLimit > 0)
                this._backupArray[0] = this.getHtml();
        },

        //Set the Iframe height 
        _setIframeHeight: function () {
            var _height = (!ej.isNullOrUndefined(this._rteToolbar) && this._rteToolbar.is(':visible') ? this._rteToolbar.outerHeight() : 6) + (!ej.isNullOrUndefined(this._rteFooter) && this._rteFooter.is(':visible') ? this._rteFooter.outerHeight() : 0);
            this._rteIframe.height(_height == 0 ? this.model.height : (this._rteWapper.outerHeight() - _height) - 1);
            $("div#" + this._rteId + "_editor").css("height", $("div.e-ipadscroll#" + this._rteId + "_editor").height());
        },

        // Footer Div render 
        _renderFooter: function () {
            var options = this.model;
            this._rteFooter = ej.buildTag("div.e-rte-footer #" + this._rteId + "_footer").appendTo(this._rteWapper);
            var _leftPan = ej.buildTag("div.e-rte-footer-left");
            var _htmlSource = ej.buildTag("div.e-rte-icons e-rte-footeritems e-rte-footericon e-rte-source", "", {}, { title: this._getLocalizedLabels("viewHtml") }).appendTo(_leftPan);
            this._htmlInfo = ej.buildTag("div.e-rte-htmltaginfo e-rte-footeritems").appendTo(_leftPan);
            _leftPan.appendTo(this._rteFooter);
            var _rightPan = ej.buildTag("div.e-rte-footer-right");
            this._wordCount = ej.buildTag("div.e-rte-wordcount e-rte-footeritems").appendTo(_rightPan);
            var _clearFormat = ej.buildTag("div.e-rte-icons clearFormat e-rte-footericon e-rte-footeritems", "", {}, { title: this._getLocalizedLabels("clearFormat") }).appendTo(_rightPan);
            var _clearAll = ej.buildTag("div.e-rte-icons clearAll e-rte-footericon e-rte-footeritems", "", {}, { title: this._getLocalizedLabels("clearAll") }).appendTo(_rightPan);
            var _resize = ej.buildTag("div.e-icons e-rte-resize e-rte-footeritems").appendTo(_rightPan);
            _rightPan.appendTo(this._rteFooter);
            this._on(_htmlSource, "click", this._sourceCodeManager);
            this._on(_clearFormat, "click", this._onClearFormat);
            this._on(_clearAll, "click", this._clearAllManager);
            !this.model.showHtmlSource && _htmlSource.hide().removeClass("e-rte-footeritems");
            !this.model.showHtmlTagInfo && this._htmlInfo.hide().removeClass("e-rte-footeritems");
            !this.model.showWordCount && this._wordCount.hide().removeClass("e-rte-footeritems");
            !this.model.showClearAll && _clearAll.hide().removeClass("e-rte-footeritems");
            !this.model.showClearFormat && _clearFormat.hide().removeClass("e-rte-footeritems");
            if (options.enableResize)
                this._enableResize();
            else
                _resize.hide().removeClass("e-rte-footeritems");
            this._updateCharCount();
        },
        //Enable Footer element
        _footerElement: function (selector, visible) {
            var ele = this._rteFooter.find(selector);
            visible ? ele.show().addClass("e-rte-footeritems") : ele.hide().addClass("e-rte-footeritems");
        },
        //Update Color Palettes
        _updateColorPalette: function () {
            var _fontColor = $("ul#" + this._rteId + "_colorTable");
            var _bgColor = $("ul#" + this._rteId + "_colorBGTable");
            _fontColor.find("div.e-rte-table").remove();
            _fontColor.find("li").append(this._colorTable());
            //Bg Color table Update
            _bgColor.find("div.e-rte-table").remove();
            _bgColor.find("li").append(this._colorTable());
        },

        //Enable RTE
        _enableRTL: function (value) {
            if (value) {
                //this._rteWapper.addClass("e-rtl");
                this._rteIframe.contents().find("body").css("direction", "rtl");
            }
            else {
                this._rteIframe.contents().find("body").css("direction", "");
            }
            this._subControlsSetModel("enableRTL", value);
        },

        _subControlsSetModel: function (prop, value) {
            !ej.isNullOrUndefined(this._toolBarItems) && this._toolBarItems.ejToolbar({ prop: value });
            !ej.isNullOrUndefined(this._linkDialog) && this._linkDialog.ejDialog({ prop: value });
            !ej.isNullOrUndefined(this._imgDialog) && this._imgDialog.ejDialog({ prop: value });
            !ej.isNullOrUndefined(this._videoDialog) && this._videoDialog.ejDialog({ prop: value });
            !ej.isNullOrUndefined(this._createTable) && this._createTable.ejDialog({ prop: value });
            !ej.isNullOrUndefined(this._sourceDialog) && this._sourceDialog.ejDialog({ prop: value });
            if (!ej.isNullOrUndefined(this._customTableDialog)) {
                !ej.isNullOrUndefined(this._customTableDialog) && this._customTableDialog.ejDialog({ prop: value });
                this._customTableDialog.find(".numerictextbox").ejNumericTextbox({ prop: value });
                this._customTableDialog.find("#ddlAlignment").ejDropDownList({ prop: value });
                !ej.isNullOrUndefined(this._customTableDialog) && this._customTableDialog.find(".e-rte-btn").ejButton({ prop: value });
            }
            !ej.isNullOrUndefined(this._videoDialog) && this._videoDialog.find(".e-rte-btn").ejButton({ prop: value });
            !ej.isNullOrUndefined(this._imgDialog) && this._imgDialog.find(".e-rte-btn").ejButton({ prop: value });
            !ej.isNullOrUndefined(this._linkDialog) && this._linkDialog.find(".e-rte-btn").ejButton({ prop: value });
            !ej.isNullOrUndefined(this._fontStyleDDL) && this._fontStyleDDL.ejDropDownList({ prop: value });
            !ej.isNullOrUndefined(this._fontSizeDDL) && this._fontSizeDDL.ejDropDownList({ prop: value });
            !ej.isNullOrUndefined(this._formatDDL) && this._formatDDL.ejDropDownList({ prop: value });
            !ej.isNullOrUndefined(this._fontColorSplit) && this._fontColorSplit.ejSplitButton({ prop: value });
            !ej.isNullOrUndefined(this._bgColorSplit) && this._bgColorSplit.ejSplitButton({ prop: value });
        },
        _createHtmlSource: function () {
            if (ej.isNullOrUndefined(this._htmlSource)) {
                this._htmlSource = ej.buildTag("div.e-rte-icons e-rte-footeritems e-rte-source").appendTo(this._rteFooter);
                this._on(this._htmlSource, "click", this._sourceCodeManager);
                this._renderSourceDialog()
            }
            else
                this._htmlSource.show();
        },
        _createHtmlTagsInfo: function () {
            //var _htmlInfo = this._rteFooter.find("div.e-rte-htmltaginfo");
            if (ej.isNullOrUndefined(this._htmlInfo)) {
                this._htmlInfo = ej.buildTag("div.e-rte-htmltaginfo e-rte-footeritems").appendTo(this._rteFooter);
            }
            else
                this._htmlInfo.show();
        },
        _createWordCount: function () {
            if (ej.isNullOrUndefined(this._wordCount)) {
                this._wordCount = ej.buildTag("div.e-rte-wordcount e-rte-footeritems").appendTo(this._rteFooter);
            }
            else
                this._wordCount.show();
        },
        //Add resize icon
        _enableResize: function () {
            if (this.model.enableResize && this.model.enabled) {
                this._rteWapper.addClass("e-resizable");
                this._resizeRTE();
            }
        },
        // Resizeable 
        _resizeRTE: function () {
            var proxy = this;
            this._rteWapper.find("div.e-rte-resize").ejResizable(
                {
                    minHeight: parseInt(proxy.model.minHeight),
                    minWidth: parseInt(proxy.model.minWidth),
                    maxHeight: parseInt(proxy.model.maxHeight),
                    maxWidth: parseInt(proxy.model.maxWidth),
                    resizeStart: function (event) {
                        proxy._trigger("resizeStart", { event: event });
                    },
                    resize: function (event) {
                        var reElement = $(event.element).parents("div.e-rte-wrapper");
                        proxy._rteWapper.height($(reElement).height());
                        proxy._rteWapper.width($(reElement).width());
                        proxy._setIframeHeight();
                        proxy._trigger("resize", { event: event });
                    },
                    resizeStop: function (event) {
                        proxy._trigger("resizeStop", { event: event });
                    },
                    helper: function (event) {
                        var reElement = $(event.element).parents("div.e-rte-wrapper");
                        proxy._rteWapper.height($(reElement).height());
                        proxy._rteWapper.width($(reElement).width());
                        proxy._setIframeHeight();
                        return $(proxy._rteWapper);
                    }
                });
        },

        _getDialogModel: function () {
            return dialogModel = {
                enableResize: false,
                showOnInit: false,
                enableModal: true,
                cssClass: this.model.cssClass,
                content: "#" + this._rteId + "_wrapper",
                enableRTL: this.model.enableRTL
            };
        },

        //Render Link Dialog
        _renderLinkDialog: function () {
            var createLink = this._linkDialog = ej.buildTag("div#" + this._rteId + "_link");//.appendTo(this._rteWapper);
            var content = $("<div class='e-rte-label'><label for=" + this._rteId + "_link_url>" + this._getLocalizedLabels("linkWebUrl") + "</label></div><div class='e-rte-field'><input type='text' class='e-inputtext' id='" + this._rteId + "_link_url'/></div><div class='e-rte-label'>" +
                           "<label for="+ this._rteId + "_link_text>" + this._getLocalizedLabels("linkText") + "</label></div><div class='e-rte-field'><input type='text' class='e-inputtext' id='" + this._rteId + "_link_text'></div><div class='e-rte-label'>" +
                           "<label for=" + this._rteId + "_link_title>" + this._getLocalizedLabels("linkToolTip") + " </label></div><div class='e-rte-field'><input type='text' class='e-inputtext' id='" + this._rteId + "_link_title'></div><div class='e-rte-label'></div>" +
                           "<div class='e-rte-field'><input type='checkbox' id='" + this._rteId + "_link_target'><label for='" + this._rteId + "_link_target' style='padding: 5px;'>" + this._getLocalizedLabels("linkOpenInNewWindow") + "</label></div>" +
                           "<div class='e-rte-button e-fieldseparate'><button class='e-rte-btn' type='button' id='link_insert'>" + this._getLocalizedLabels("dialogInsert") + "</button><button class='e-rte-btn'type='button' id='link_cancel'>" + this._getLocalizedLabels("dialogCancel") + "</button></div>");
            content.appendTo(createLink);
            createLink.appendTo(this._rteWapper);
            var model = this._getDialogModel();
            model.title = this._getLocalizedLabels("createLink");
            createLink.ejDialog(model);
            this._chkTarget = createLink.find("#" + this._rteId + "_link_target");
            this._chkTarget.ejCheckBox({ enableRTL: this.model.enableRTL, cssClass: this.model.cssClass });
            createLink.find(".e-rte-btn").ejButton({ enableRTL: this.model.enableRTL, cssClass: this.model.cssClass });
            this._on(createLink.find(".e-rte-btn"), "click", this._linkBtnClick)
            this._on(this._linkDialog.find("#" + this._rteId + "_link_url"), "keypress", this._urlValidation);
        },

        //Render Link Dialog
        _renderImageDialog: function () {
            this._imgDialog = ej.buildTag("div#" + this._rteId + "_Image");
            var content = $("<div id='" + this._rteId + "_img_Tab' class='e-dialog-tab'><ul><li class='e-first'><a href='#" + this._rteId + "_general'>" + this._getLocalizedLabels("general") + " </a></li><li class='e-nofirst'><a href='#" + this._rteId + "_advanced'>" + this._getLocalizedLabels("advanced") + " </a></li></ul>" +
                "<div id='" + this._rteId + "_general'>" +
                        "<div class='e-img-field-group'><div class='e-rte-label'><label for=" + this._rteId + "_img_url>" + this._getLocalizedLabels("imageWebUrl") + " </label></div><div class='e-rte-field'><input type='text' class='e-inputtext' id='" + this._rteId + "_img_url'></div></div><div class='e-img-field-group'><div class='e-rte-label'>" +
                        "<label for=" + this._rteId + "_img_text>" + this._getLocalizedLabels("imageAltText") + " </label></div><div class='e-rte-field'><input type='text' class='e-inputtext' id='" + this._rteId + "_img_text'></div></div>" +
                        "<div id='" + this._rteId + "_imgdimensions'><div class='e-rte-label'><label>" + this._getLocalizedLabels("dimensions") + " </label></div><div class='e-rte-field'><input type='text' title='" + this._getLocalizedLabels("dimensions") + "' class='e-inputtext e-dimensions' maxlength='3' id='" + this._rteId + "_imgX'><label style='padding: 5px;'>X</label><input type='text' title='" + this._getLocalizedLabels("dimensions") + "' style='margin-right: 5px;' class='e-inputtext e-dimensions' maxlength='3' id='" + this._rteId + "_imgY'></div>" +
                 "<input type='checkbox' id='" + this._rteId + "_img_consrn'><label for='" + this._rteId + "_img_consrn'  style='padding: 5px;'>" + this._getLocalizedLabels("constrainProportions") + "</label></div>" +
                "</div>" +
                "<div id='" + this._rteId + "_advanced'>" +
                        "<div class='e-img-field-group'><div class='e-rte-label'><label for=" + this._rteId + "_img_link>" + this._getLocalizedLabels("imageLink") + " </label></div><div class='e-rte-field'><input type='text' class='e-inputtext' id='" + this._rteId + "_img_link'></div></div>"+
                        "<div class='e-img-field-group'><div class='e-rte-label'><label for=" + this._rteId + "_img_border_px>" + this._getLocalizedLabels("imageBorder") + "</label></div><div class='e-rte-field'><div class='e-img-border-field'><input type='text' title='" + this._getLocalizedLabels("imageBorder") + "' class='e-inputtext' maxlength='3' id='" + this._rteId + "_img_border_px'></div><div class='e-img-border-field'><select id='" + this._rteId + "_img_border_style'><option value='solid'>Solid</option><option value='dotted'>Dotted</option><option value='dashed'>Dashed</option><option value='double'>Double</option></select></div><div class='e-img-border-field'><input type='text' title='" + this._getLocalizedLabels("imageBorder") + "' class='e-inputtext' maxlength='7' id='" + this._rteId + "_img_border_color'></div></div></div>" +
                        "<div class='e-img-field-group'><div class='e-rte-label'><label for=" + this._rteId + "_img_style>" + this._getLocalizedLabels("imageStyle") + " </label></div><div class='e-rte-field'><input type='text' class='e-inputtext' id='" + this._rteId + "_img_style'></div></div>"+
                "</div>" +

                "</div>" +
                "<div class='e-rte-button'><button class='e-rte-btn' type='button' id='" + this._rteId + "_img_insert'>" + this._getLocalizedLabels("dialogInsert") + "</button><button class='e-rte-btn' type='button' id='" + this._rteId + "_img_cancel'>" + this._getLocalizedLabels("dialogCancel") + "</button></div>"
                );
            content.appendTo(this._imgDialog);
            this._imgDialog.appendTo(this._rteWapper);
            !this.model.showDimensions && this._rteWapper.find("#" + this._rteId + "_imgdimensions").hide();
            var model = this._getDialogModel();
            model.width = "470px";
            model.title = this._getLocalizedLabels("image");
            this._imgDialog.ejDialog(model);
            this._imgDialog.find("#" + this._rteId + "_img_Tab").ejTab({ cssClass: this.model.cssClass, width: "100%", enabled: true });
            this._imgDialog.find("#" + this._rteId + "_img_border_px").ejNumericTextbox({ enableRTL: this.model.enableRTL, cssClass: this.model.cssClass, width: "100%", minValue: 0, value: 0 });
            this._imgDialog.find("#" + this._rteId + "_img_border_style").ejDropDownList({ enableRTL: this.model.enableRTL, cssClass: this.model.cssClass, width: "100%", enabled: true });
            this._chkImgDimensions = this._imgDialog.find("#" + this._rteId + "_img_consrn");
            this._chkImgDimensions.ejCheckBox({ check: true, enableRTL: this.model.enableRTL, cssClass: this.model.cssClass });
            this._imgDialog.find(".e-rte-btn").ejButton({ enableRTL: this.model.enableRTL, cssClass: this.model.cssClass });
            this._on(this._imgDialog.find(".e-rte-btn"), "click", this._imageBtnClick)
            this._on(this._imgDialog.find("#" + this._rteId + "_img_url"), "keypress", this._urlValidation);
            this._on(this._imgDialog.find("input.e-dimensions"), "change", this._recalcImgSize);
            if(this.model.enableRTL) this._imgDialog.find("#" + this._rteId + "_img_Tab").addClass('e-rtl');
        },

        //Render the Table selection rows and columans popup
        _renderVideoDialog: function () {
            this._videoDialog = ej.buildTag("div#" + this._rteId + "_video");
            var content = $("<div><label>" + this._getLocalizedLabels("embedVideo") + "</label></div><textarea class='e-rte-video e-inputtext'  aria-label=" + this._getLocalizedLabels("embedVideo") + "></textarea>" +
               "<div id='" + this._rteId + "_videodimensions' style='margin-top:7px'><div class='e-rte-label'><label>" + this._getLocalizedLabels("dimensions") + " </label></div><div><input type='text' title='" + this._getLocalizedLabels("dimensions") + "' class='e-inputtext e-dimensions' maxlength='3' id='" + this._rteId + "_videoX'><label style='padding: 5px;'>X</label><input type='text' title='" + this._getLocalizedLabels("dimensions") + "' style='margin-right: 5px;' class='e-inputtext e-dimensions' maxlength='3' id='" + this._rteId + "_videoY'>" +
               "<input type='checkbox' id='" + this._rteId + "_video_consrn'><label for='" + this._rteId + "_video_consrn'  style='padding: 5px;'>" + this._getLocalizedLabels("constrainProportions") + "</label></div></div>" +
                "<div class='e-rte-button e-fieldseparate'><button id='video_insert'type='button' class='e-rte-btn'>" + this._getLocalizedLabels("dialogInsert") + "</button><button id='video_cancel' type='button' class='e-rte-btn'>" + this._getLocalizedLabels("dialogCancel") + "</button></div>");
            content.appendTo(this._videoDialog);
            this._videoDialog.appendTo(this._rteWapper);
            !this.model.showDimensions && this._rteWapper.find("#" + this._rteId + "_videodimensions").hide();
            var model = this._getDialogModel();
            model.width = "460px";
            model.title = this._getLocalizedLabels("video");
            this._videoDialog.ejDialog(model);
            this._chkvideoDimensions = this._videoDialog.find("#" + this._rteId + "_video_consrn");
            this._chkvideoDimensions.ejCheckBox({ checked: true, enableRTL: this.model.enableRTL, cssClass: this.model.cssClass });
            this._videoDialog.find(".e-rte-btn").ejButton({ enableRTL: this.model.enableRTL, cssClass: this.model.cssClass });
            this._on(this._videoDialog.find(".e-rte-btn"), "click", this._insertVideo);
            this._on(this._videoDialog.find("input.e-dimensions"), "change", this._recalcVideoSize);
        },

        //Render the Table selection rows and columans popup
        _renderTableDialog: function () {
            this._createTable = ej.buildTag("div.e-rte-table-picker#" + this._rteId + "_table");
            var tableCell, rowDiv, colDiv, tableDiv;
            this._tblheaderDiv = ej.buildTag("div.e-rte-tableheader");
            this._tblheaderDiv.html(this._getLocalizedLabels("createTable"));
            this._tblheaderDiv.appendTo(this._createTable);
            var tableDiv = ej.buildTag("div.e-rte-table");
            this._drawTable(tableDiv);
            tableDiv.appendTo(this._createTable);
            if (this.model.showCustomTable) {
                this._createCustomTable();
            }

            this._createTable.appendTo(this._rteWapper);
            var model = this._getDialogModel();
            model.modal = false;
            model.showHeader = false;
            model.width = "auto";
            this._createTable.ejDialog(model);
        },

        _createCustomTable: function () {
            var customTable = $("<div class='customtable-group' id='" + this._rteId + "_createCustomTableLink'><span class='e-rte-toolbar-icon customtable-image'></span><a class='customtable-link' role='button' title='" + this._getLocalizedLabels("customTable") + "'>" + this._getLocalizedLabels("customTable") + "</a></div>" +
               "<div class='customtable-group' id='" + this._rteId + "_eTblProperties'><span class='e-rte-toolbar-icon customtable-image'></span><a class='customtable-link' role='button' title='" + this._getLocalizedLabels("editTable") + "'>" + this._getLocalizedLabels("editTable") + "</a></div>");
            this._on(customTable, "click", this._openCustomTable);
            customTable.appendTo(this._createTable);
            //Create custom table Dialog.
            this._customTableDialog = ej.buildTag("div#" + this._rteId + "_customTable");
            var content = $("<div class='e-fieldgroup'><div class='e-rte-tablefields'><label for=" + this._rteId + "_txtColumns>" + this._getLocalizedLabels("tableColumns") + "</label></div><div class='e-rte-tablefields'><input type='text' class='numerictextbox' id='" + this._rteId + "_txtColumns'></div><div class='e-rte-tablefields'>" +
                 "<label for=" + this._rteId + "_txtRows>" + this._getLocalizedLabels("tableRows") + "</label></div><div class='e-rte-tablefields'><input type='text' class='numerictextbox' id='" + this._rteId + "_txtRows'></div><div class='e-rte-tablefields'>" +
                 "<label for=" + this._rteId + "_txtWidth>" + this._getLocalizedLabels("tableWidth") + "</label></div><div class='e-rte-tablefields'><input type='text' class='e-inputtext' id='" + this._rteId + "_txtWidth'></div>" +
                 "<div class='e-rte-tablefields'><label for=" + this._rteId + "_txtHeight>" + this._getLocalizedLabels("tableHeight") + "</label></div><div class='e-rte-tablefields'><input type='text' class='e-inputtext' id='" + this._rteId + "_txtHeight'></div></div>" +
                 "<div class='e-fieldseparate'><div class='e-rte-tablefields'><label for=" + this._rteId + "_txtSpacing>" + this._getLocalizedLabels("tableCellSpacing") + "</label></div><div class='e-rte-tablefields'><input type='text' class='e-inputtext' id='" + this._rteId + "_txtSpacing'></div>" +
                 "<div class='e-rte-tablefields'><label for=" + this._rteId + "_txtPadding>" + this._getLocalizedLabels("tableCellPadding") + "</label></div><div class='e-rte-tablefields'><input type='text' class='e-inputtext' id='" + this._rteId + "_txtPadding'></div>" +
                 "<div class='e-rte-tablefields'><label for=" + this._rteId + "_txtBorder>" + this._getLocalizedLabels("tableBorder") + "</label></div><div class='e-rte-tablefields'><select id='" + this._rteId + "_txtBorder'><option value='solid'>Solid</option><option value='dotted'>Dotted</option><option value='dashed'>Dashed</option><option value='double'>Double</option></select></div>" +
                 "<div class='e-rte-tablefields'><label for=" + this._rteId + "_chkCaption>" + this._getLocalizedLabels("tableCaption") + "</label></div><div class='e-rte-tablefields'><input type='checkbox' id='" + this._rteId + "_chkCaption'></div>" +
                 "<div class='e-rte-tablefields'><label for=" + this._rteId + "_ddlAlignment>" + this._getLocalizedLabels("tableAlignment") + "</label></div><div class='e-rte-tablefields'><select id='" + this._rteId + "_ddlAlignment'><option value='left'>Left</option><option value='right'>Right</option><option value='center'>Center</option></select></div></div>" +
                 "<div class='e-rte-button e-fieldseparate'><button class='e-rte-btn' id='insert'>" + this._getLocalizedLabels("dialogInsert") + "</button><button class='e-rte-btn' id='cancel'>" + this._getLocalizedLabels("dialogCancel") + "</button></div>");
            content.appendTo(this._customTableDialog);
            var model = this._getDialogModel();
            model.width = "480px";
            model.title = this._getLocalizedLabels("customTable");
            this._customTableDialog.ejDialog(model);
            this._createTable.find("div#" + this._rteId + "_eTblProperties").addClass('e-disable');
            this._customTableDialog.find(".e-rte-btn").ejButton({ enableRTL: this.model.enableRTL, cssClass: this.model.cssClass });
            this._customTableDialog.find(".numerictextbox").ejNumericTextbox({ enableRTL: this.model.enableRTL, cssClass: this.model.cssClass, width: "100%", minValue: 1, value: 3 });
            this._customTableDialog.find("#" + this._rteId + "_txtColumns").ejNumericTextbox({ maxValue: 63 });
            this._customTableDialog.find("#" + this._rteId + "_ddlAlignment").ejDropDownList({ enableRTL: this.model.enableRTL, cssClass: this.model.cssClass, width: "100%", enabled:false });
            this._customTableDialog.find("#" + this._rteId + "_txtBorder").ejDropDownList({ enableRTL: this.model.enableRTL, cssClass: this.model.cssClass, width: "100%", enabled: true });
            this._chkTblCaption = this._customTableDialog.find("#" + this._rteId + "_chkCaption");
            this._chkTblCaption.ejCheckBox({ enableRTL: this.model.enableRTL, cssClass: this.model.cssClass });
            this._on(this._customTableDialog.find(".e-rte-btn"), "click", this._insertCustomTable);
        },

        //Render Edit Table Dialog
        _renderEditTableDialog: function () {
            this._eTblDialog = ej.buildTag("div#" + this._rteId + "_eTbl");
            var rteID = this._rteId;
            var content = $("<div id='" + this._rteId + "_eTbl_Tab' class='e-dialog-tab'><ul><li class='e-first'><a href='#" + this._rteId + "_eTblTable'>" + this._getLocalizedLabels("table") + " </a></li><li class='e-nofirst'><a href='#" + this._rteId + "_eTblRow'>" + this._getLocalizedLabels("row") + " </a></li><li class='e-nofirst'><a href='#" + this._rteId + "_eTblColumn'>" + this._getLocalizedLabels("column") + " </a></li><li class='e-nofirst'><a href='#" + this._rteId + "_eTblCell'>" + this._getLocalizedLabels("cell") + " </a></li></ul>" +
                "<div id='" + this._rteId + "_eTblTable'>" +
                        "<div class='e-rte-tablefields'><label for=" + rteID + "_eTblWidth>" + this._getLocalizedLabels("tableWidth") + "</label></div><div class='e-rte-tablefields'><input type='text' class='e-inputtext' id='" + rteID + "_eTblWidth'></div>" +
                        "<div class='e-rte-tablefields'><label for=" + rteID + "_eTblHeight>" + this._getLocalizedLabels("tableHeight") + "</label></div><div class='e-rte-tablefields'><input type='text' class='e-inputtext' id='" + rteID + "_eTblHeight'></div>" +
                        "<div class='e-rte-tablefields'><label for=" + rteID + "_eTblCellSpace>" + this._getLocalizedLabels("tableCellSpacing") + "</label></div><div class='e-rte-tablefields'><input type='text' class='e-inputtext e-numerictextbox' id='" + rteID + "_eTblCellSpace'></div>" +
                        "<div class='e-rte-tablefields'><label for=" + rteID + "_eTblCellPad>" + this._getLocalizedLabels("tableCellPadding") + "</label></div><div class='e-rte-tablefields'><input type='text' class='e-inputtext e-numerictextbox' id='" + rteID + "_eTblCellPad'></div>" +
                        "<div class='e-rte-tablefields'><label for=" + rteID + "_eTblAlign>" + this._getLocalizedLabels("tableAlignment") + "</label></div><div class='e-rte-tablefields'><select id='" + rteID + "_eTblAlign' class='e-dropdownlist'><option value='left'>Left</option><option value='right'>Right</option><option value='center'>Center</option></select></div>" +
                        "<div class='e-rte-tablefields'><label for=" + rteID + "_eTblCaption>" + this._getLocalizedLabels("tableCaption") + "</label></div><div class='e-rte-tablefields'><input type='checkbox' id='" + rteID + "_eTblCaption'></div>" +
                        "<div class='e-img-field-group'><div class='e-rte-tablefields'><label for=" + rteID + "_eTblBrdrPx>" + this._getLocalizedLabels("tableBorder") + "</label></div><div class='e-rte-stylefield'><div class='e-img-border-field'><input type='text' title='" + this._getLocalizedLabels("tableBorder") + "' class='e-inputtext e-numerictextbox' maxlength='3' id='" + rteID + "_eTblBrdrPx'></div><div class='e-img-border-field'><select id='" + rteID + "_eTbl_border_style' class='e-dropdownlist'><option value='solid'>Solid</option><option value='dotted'>Dotted</option><option value='dashed'>Dashed</option><option value='solid'>Double</option></select></div><div class='e-img-border-field'><input type='text' title='" + this._getLocalizedLabels("tableBorder") + "' class='e-inputtext' maxlength='7' id='" + rteID + "_eTbl_border_color'></div></div></div>" +
                        "<div class='e-img-field-group'><div class='e-rte-tablefields'><label for=" + rteID + "_eTblStyle>" + this._getLocalizedLabels("imageStyle") + " </label></div><div class='e-rte-stylefield'><input type='text' class='e-inputtext' id='" + rteID + "_eTblStyle'></div></div>"+
                "</div>" +
                "<div id='" + rteID + "_eTblRow'>" +
                        "<div class='e-rte-tablefields'><label for=" + rteID + "_eTblRHeight>" + this._getLocalizedLabels("tableHeight") + "</label></div><div class='e-rte-tablefields'><input type='text' class='e-inputtext' id='" + rteID + "_eTblRHeight'></div>" +
                        "<div class='e-rte-tablefields'><label for=" + rteID + "_eTblRAlign>" + this._getLocalizedLabels("textAlign") + "</label></div><div class='e-rte-tablefields'><select id='" + rteID + "_eTblRAlign' class='e-dropdownlist'><option value='left'>Left</option><option value='right'>Right</option><option value='center'>Center</option></select></div>" +
                        "<div class='e-img-field-group'><div class='e-rte-tablefields'><label for=" + rteID + "_eTblRBrdrPx>" + this._getLocalizedLabels("tableBorder") + "</label></div><div class='e-rte-stylefield'><div class='e-img-border-field'><input type='text' title='" + this._getLocalizedLabels("tableBorder") + "' class='e-inputtext e-numerictextbox' maxlength='3' id='" + rteID + "_eTblRBrdrPx'></div><div class='e-img-border-field'><select id='" + rteID + "_eTblRBrdrStyle' class='e-dropdownlist'><option value='solid'>Solid</option><option value='dotted'>Dotted</option><option value='dashed'>Dashed</option><option value='solid'>Double</option></select></div><div class='e-img-border-field'><input type='text' title='" + this._getLocalizedLabels("tableBorder") + "' class='e-inputtext' maxlength='7' id='" + rteID + "_eTblRBrdrColor'></div></div></div>" +
                        "<div class='e-img-field-group'><div class='e-rte-tablefields'><label for=" + rteID + "_eTblRStyle>" + this._getLocalizedLabels("imageStyle") + " </label></div><div class='e-rte-stylefield'><input type='text' class='e-inputtext' id='" + rteID + "_eTblRStyle'></div></div>"+       
                "</div>" +
                "<div id='" + rteID + "_eTblColumn'>" +
                        "<div class='e-rte-tablefields'><label for=" + rteID + "_eTblCWidth>" + this._getLocalizedLabels("tableWidth") + "</label></div><div class='e-rte-tablefields'><input type='text' class='e-inputtext' id='" + rteID + "_eTblCWidth'></div>" +
                        "<div class='e-rte-tablefields'><label for=" + rteID + "_eTblCAlign>" + this._getLocalizedLabels("textAlign") + "</label></div><div class='e-rte-tablefields'><select id='" + rteID + "_eTblCAlign' class='e-dropdownlist'><option value='left'>Left</option><option value='right'>Right</option><option value='center'>Center</option></select></div>" +
                        "<div class='e-img-field-group'><div class='e-rte-tablefields'><label for=" + rteID + "_eTblCBrdrPx>" + this._getLocalizedLabels("tableBorder") + "</label></div><div class='e-rte-stylefield'><div class='e-img-border-field'><input type='text' title='" + this._getLocalizedLabels("tableBorder") + "' class='e-inputtext e-numerictextbox' maxlength='3' id='" + rteID + "_eTblCBrdrPx'></div><div class='e-img-border-field'><select id='" + rteID + "_eTblCBrdrStyle' class='e-dropdownlist'><option value='solid'>Solid</option><option value='dotted'>Dotted</option><option value='dashed'>Dashed</option><option value='solid'>Double</option></select></div><div class='e-img-border-field'><input type='text' title='" + this._getLocalizedLabels("tableBorder") + "' class='e-inputtext' maxlength='7' id='" + rteID + "_eTblCBrdrColor'></div></div></div>" +
                        "<div class='e-img-field-group'><div class='e-rte-tablefields'><label for=" + rteID + "_eTblCStyle>" + this._getLocalizedLabels("imageStyle") + " </label></div><div class='e-rte-stylefield'><input type='text' class='e-inputtext' id='" + rteID + "_eTblCStyle'></div></div>"+       
                "</div>" +
                "<div id='" + rteID + "_eTblCell'>" +
                        "<div class='e-rte-tablefields'><label for=" + rteID + "_eTblClWidth>" + this._getLocalizedLabels("tableWidth") + "</label></div><div class='e-rte-tablefields'><input type='text' class='e-inputtext' id='" + rteID + "_eTblClWidth'></div>" +
                        "<div class='e-rte-tablefields'><label for=" + rteID + "_eTblClAlign>" + this._getLocalizedLabels("textAlign") + "</label></div><div class='e-rte-tablefields'><select id='" + rteID + "_eTblClAlign' class='e-dropdownlist'><option value='left'>Left</option><option value='right'>Right</option><option value='center'>Center</option></select></div>" +
                        "<div class='e-img-field-group'><div class='e-rte-tablefields'><label for=" + rteID + "_eTblClBrdrPx>" + this._getLocalizedLabels("tableBorder") + "</label></div><div class='e-rte-stylefield'><div class='e-img-border-field'><input type='text' title='" + this._getLocalizedLabels("tableBorder") + "' class='e-inputtext e-numerictextbox' maxlength='3' id='" + rteID + "_eTblClBrdrPx'></div><div class='e-img-border-field'><select id='" + rteID + "_eTblClBrdrStyle' class='e-dropdownlist'><option value='solid'>Solid</option><option value='dotted'>Dotted</option><option value='dashed'>Dashed</option><option value='solid'>Double</option></select></div><div class='e-img-border-field'><input type='text' title='" + this._getLocalizedLabels("tableBorder") + "' class='e-inputtext' maxlength='7' id='" + rteID + "_eTblClBrdrColor'></div></div></div>" +
                        "<div class='e-img-field-group'><div class='e-rte-tablefields'><label for=" + rteID + "_eTblClStyle>" + this._getLocalizedLabels("imageStyle") + " </label></div><div class='e-rte-stylefield'><input type='text' class='e-inputtext' id='" + rteID + "_eTblClStyle'></div></div>"+       
                "</div>" +
                "</div>" +
                "<div class='e-rte-button'><button class='e-rte-btn' id='" + rteID + "_eTbl_apply'>" + this._getLocalizedLabels("dialogApply") + "</button><button class='e-rte-btn' id='" + rteID + "_eTbl_cancel'>" + this._getLocalizedLabels("dialogCancel") + "</button></div>"
                );
            content.appendTo(this._eTblDialog);
            this._eTblDialog.appendTo(this._rteWapper);
            var model = this._getDialogModel();
            model.width = "470px";
            model.title = this._getLocalizedLabels("editTable");
            this._eTblDialog.ejDialog(model);
            this._eTblDialog.find(".e-numerictextbox").ejNumericTextbox({ enableRTL: this.model.enableRTL, cssClass: this.model.cssClass, width: "100%", minValue: 0, value: 0 });
            this._eTblDialog.find(".e-dropdownlist").ejDropDownList({ enableRTL: this.model.enableRTL, cssClass: this.model.cssClass, width: "100%"});
            this._eTblAlign = this._getTableObj("Align").data('ejDropDownList');
            this._eTblAlign.option('enabled', false);
            this._getTableObj("_Tab").ejTab({ cssClass: this.model.cssClass, width: "100%", enabled: true });
            this._eTblCaption = this._getTableObj("Caption");
            this._eTblCaption.ejCheckBox({ enableRTL: this.model.enableRTL, cssClass: this.model.cssClass });
            this._eTblDialog.find(".e-rte-btn").ejButton({ enableRTL: this.model.enableRTL, cssClass: this.model.cssClass });
            this._on(this._eTblDialog.find(".e-rte-btn"), "click", this._eTblBtnClick);
            if(this.model.enableRTL) this._getTableObj("_Tab").addClass('e-rtl');
        },

        _drawTable: function (tableDiv) {
            for (var row = 0; row < this.model.tableRows ; row++) {
                rowDiv = ej.buildTag("div.e-rtetablerow");
                for (var col = 0; col < this.model.tableColumns; col++) {
                    tableCell = ej.buildTag("div.e-rte-tablecell e-default");
                    tableCell.appendTo(rowDiv);
                }
                rowDiv.appendTo(tableDiv);
            }
        },

        //Render Source code Dialog.
        _renderSourceDialog: function () {
            this._sourceDialog = ej.buildTag("div#" + this._rteId + "_Source");
            var content = $("<textarea class='e-rte-srctextarea e-inputtext'></textarea><div class='e-rte-button e-fieldseparate'><button id='src_update' class='e-rte-btn'>" + this._getLocalizedLabels("dialogUpdate") + "</button><button id='src_cancel' class='e-rte-btn'>" + this._getLocalizedLabels("dialogCancel") + "</button></div>");
            content.appendTo(this._sourceDialog);
            var model = this._getDialogModel();
            model.title = this._getLocalizedLabels("viewHtml");
            model.width = "auto";
            model.content = null,
            this._sourceDialog.ejDialog(model);
            this._sourceDialog.find(".e-rte-btn").ejButton({ enableRTL: this.model.enableRTL, cssClass: this.model.cssClass });
            this._on(this._sourceDialog.find(".e-rte-btn"), "click", this._srcBtnClick)
        },

        //Render Alert Dialog
        _renderAlertDialog: function () {
            this._alertWindow = ej.buildTag("div");
            var content = $("<p class='e-alerttext'></p><div class='e-rte-button e-fieldseparate'><button class='e-rte-btn e-alert-ok'>" + this._getLocalizedLabels("dialogOk") + "</button><button class='e-rte-btn e-alert-cancel'>" + this._getLocalizedLabels("dialogCancel") + "</button></div>");
            content.appendTo(this._alertWindow);
            var model = this._getDialogModel();
            model.showHeader = false;
            model.width = "auto";
            this._alertWindow.ejDialog(model);
            this._alertWindow.find(".e-rte-btn").ejButton({ enableRTL: this.model.enableRTL, cssClass: this.model.cssClass });
            this._on(this._alertWindow.find(".e-rte-btn"), "click", this._alertBtnClick)
            this._alertWindow.find(".e-alert-cancel").hide();
        },

        _alertBtnClick: function (e) {
            if ($.trim(this._alertWindow.find(".e-alerttext").html()) == this._getLocalizedLabels("deleteAlert") && $(e.target).hasClass("e-alert-ok")) {
                this._getDocument().body.innerHTML = "";
                this._toolBarObj.selectItemByID(this._rteId + "justifyLeft");
                this._setBackupData();
                this._updateCharCount();
                this._onChange();
            }
            this._alertWindow.ejDialog("close");
            this._alertWindow.find(".e-alert-cancel").hide();
        },

        _openAlert: function (alertText) {
            this._alertWindow.find(".e-alerttext").html(alertText);
            this._alertWindow.ejDialog("open");
        },

        //Render Format 
        _renderFormat: function () {
            this._formatDDL = ej.buildTag("input#" + this._rteId + "_format", "", "", { type: "text", title: this._getLocalizedLabels("format") });
            var model = {};
            model.dataSource = this.model.format;
            model.height = "25px";
            model.width = "105px";
            model.enableRTL = this.model.enableRTL;
            model.popupWidth = "175px";
            model.popupHeight = "auto";
            model.selectedItemIndex = 0;
            model.popupHide = this._formatChange;
            model.fields = { text: "text", value: "value", spriteCssClass: "spriteCssClass" };
            this._formatDDL.appendTo(this._rteToolbar.find("#" + this._rteId + "format").html(""));
            this._formatDDL.ejDropDownList(model);
            this._formatPopupStyle();
        },
        //Apply the css class for LI elemet
        _formatPopupStyle: function () {
            var _liList = $("#" + this._rteId + "_format_popup").find("li");
            for (var i = 0; i < _liList.length; i++) {
                $(_liList[i]).addClass(this.model.format[i].spriteCssClass);
            }
        },

        // Render Font style Dropdown list
        _renderFontStyle: function () {
            this._fontStyleDDL = ej.buildTag("input#" + this._rteId + "_fontName", "", "", { type: "text", title: this._getLocalizedLabels("fontName") });
            var model = {};
            model.dataSource = this.model.fontName;
            model.height = "25px";
            model.enableRTL = this.model.enableRTL;
            model.selectedItemIndex = 0;
            model.popupHide = this._fontStyleChange;
            model.fields = { text: "text", value: "value" };
            this._fontStyleDDL.appendTo(this._rteToolbar.find("#" + this._rteId + "fontName").html(""));
            this._fontStyleDDL.ejDropDownList(model);
        },

        // Render Font style Dropdown list
        _renderFontSize: function () {
            this._fontSizeDDL = ej.buildTag("input#" + this._rteId + "_fontSize", "", "", { type: "text", title: this._getLocalizedLabels("fontSize") });
            var model = {};
            model.dataSource = this.model.fontSize;
            model.height = "25px";
            model.width = "100px";
            model.enableRTL = this.model.enableRTL
            model.selectedItemIndex = 1;
            model.popupHide = this._fontSizeChange;
            model.fields = { text: "text", value: "value" };
            this._fontSizeDDL.appendTo(this._rteToolbar.find("#" + this._rteId + "fontSize").html(""));
            this._fontSizeDDL.ejDropDownList(model);
        },

        //Render Split button for font color
        _renderFontColor: function () {
            this._fontColorSplit = ej.buildTag("button#" + this._rteId + "_fontColor", "", {},{ type:"button"});
            var _liTemplte = $("<ul id='" + this._rteId + "_colorTable' class='e-rte-colorpalette'><li></li></ul>");
            _liTemplte.find("li").append(this._colorTable());
            var model = {};
            model.height = "25px";
            model.width = "50px";
            model.contentType = "textandimage";
            model.targetID = this._rteId + "_colorTable";
            model.enableRTL = this.model.enableRTL;
            model.prefixIcon = "e-fontcolor-icon";
            this._fontColorSplit.appendTo(this._rteToolbar.find("#" + this._rteId + "fontColor").html(""));
            _liTemplte.appendTo(this._rteToolbar.find("#" + this._rteId + "fontColor"));
            this._fontColorSplit.ejSplitButton(model);
            this._splitMenu = _liTemplte.ejMenu("instance");
            this._on(_liTemplte.find("div.e-rte-palettetable"), "click", this._colorPaletteClick);
            this._on(this._fontColorSplit, "click", this._fontColorClick);
            this._fontColorSplit.find("span.e-fontcolor-icon").removeClass("e-icon");
            this._fontColorSplit.find("span.e-btntxt").css("background-color", "black");
        },

        //Render Split button for font color
        _renderBGColor: function () {
            this._bgColorSplit = ej.buildTag("button#" + this._rteId + "_backgroundColor", "", {},{type:"button"});
            var _liTemplte = $("<ul id='" + this._rteId + "_colorBGTable' class='e-rte-colorpalette'><li></li></ul>");
            _liTemplte.find("li").append(this._colorTable());
            var model = {};
            model.height = "25px";
            model.width = "50px";
            model.contentType = "textandimage";
            model.targetID = this._rteId + "_colorBGTable";
            model.enableRTL = this.model.enableRTL;
            model.prefixIcon = "e-bgcolor-icon";
            this._bgColorSplit.appendTo(this._rteToolbar.find("#" + this._rteId + "backgroundColor").html(""));
            _liTemplte.appendTo(this._rteToolbar.find("#" + this._rteId + "backgroundColor"));
            this._bgColorSplit.ejSplitButton(model);
            this._bgSplitMenu = _liTemplte.ejMenu("instance");
            this._on(_liTemplte.find("div.e-rte-palettetable"), "click", this._bgColorPaletteClick);
            this._on(this._bgColorSplit, "click", this._bgColorClick);
            this._bgColorSplit.find("span.e-bgcolor-icon").removeClass("e-icon");
            this._bgColorSplit.find("span.e-btntxt").css("background-color", "white");
        },

        _colorTable: function () {
            var tableDiv = ej.buildTag("div.e-rte-table");
            var color = 0, hexCode, rowDiv;
            for (var row = 0; row < this.model.colorPaletteRows ; row++) {
                rowDiv = ej.buildTag("div.e-rtetablerow");
                for (var col = 0; col < this.model.colorPaletteColumns ; col++) {
                    if (color < this.model.colorCode.length) {
                        hexCode = "#" + this.model.colorCode[color];
                        tableCell = ej.buildTag("div.e-rte-palettetable", "", { "background-color": hexCode }, { title: hexCode, "color-code": hexCode });
                        tableCell.appendTo(rowDiv);
                        color++;
                    }
                }
                rowDiv.appendTo(tableDiv);
            }
            return tableDiv;
        },

        _recalcVideoSize: function (e) {
            var txtWidth, txtheight, newWidth, newHeight;
            txtWidth = this._videoDialog.find("#" + this._rteId + "_videoX");
            txtheight = this._videoDialog.find("#" + this._rteId + "_videoY");

            newWidth = txtWidth.val();
            newHeight = txtheight.val();

            if (this._chkvideoDimensions.ejCheckBox("isChecked") && this._videoWidth && this._videoHeight && newWidth && newHeight) {
                if (e.target.id == txtWidth[0].id) {
                    newHeight = Math.round((newWidth / this._videoWidth) * newHeight);
                    txtheight.val(newHeight);
                } else {
                    newWidth = Math.round((newHeight / this._videoHeight) * newWidth);
                    txtWidth.val(newWidth);
                }
            }
            this._videoWidth = newWidth;
            this._videoHeight = newHeight;
        },


        _recalcImgSize: function (e) {
            var txtWidth, txtheight, newWidth, newHeight;
            txtWidth = this._imgDialog.find("#" + this._rteId + "_imgX");
            txtheight = this._imgDialog.find("#" + this._rteId + "_imgY");

            newWidth = txtWidth.val();
            newHeight = txtheight.val();

            if (this._chkImgDimensions.ejCheckBox("isChecked") && this._imgWidth && this._imgHeight && newWidth && newHeight) {
                if (e.target.id == txtWidth[0].id) {
                    newHeight = Math.round((newWidth / this._imgWidth) * newHeight);
                    txtheight.val(newHeight);
                } else {
                    newWidth = Math.round((newHeight / this._imgHeight) * newWidth);
                    txtWidth.val(newWidth);
                }
            }
            this._imgWidth = newWidth;
            this._imgHeight = newHeight;
        },

        _urlValidation: function (e) {
            var _target = $(e.target);
            _target.hasClass("e-error") && _target.removeClass("e-error");
        },

        _linkBtnClick: function (sender) {
            if (sender.target.id === "link_cancel") {
                this._clearLinkFields();
                this._linkDialog.ejDialog("close");
            }
            else if (sender.target.id === "link_insert") {
                this._restoreSelection(this._selectionRange);
                if (this._txtURL.val() != "")
                    this._onInsertLink();
                else {
                    this._onUnlink();
                    this._linkDialog.ejDialog("close");
                }
                this._onChange();
                this._setBackupData();
            }
        },

        _imageBtnClick: function (sender) {
            if (sender.target.id === this._rteId + "_img_cancel") {
                this._imgDialog.ejDialog("close");
            }
            else if (sender.target.id === this._rteId + "_img_insert") {
                this._onInsertImage();
                this._onChange();
                this._setBackupData();
            }
        },

        //Source Dialog button click event
        _srcBtnClick: function (sender) {
            if (sender.target.id === "src_update") {
                var _editHTML = $.trim(this._sourceDialog.find("textarea.e-rte-srctextarea")[0].value);
                this._getDocument().body.innerHTML = _editHTML;
                this._onChange();
                this._updateCharCount();
            }
            this._sourceDialog.find("textarea.e-rte-srctextarea")[0].value = "";
            this._sourceDialog.ejDialog("close");
        },

        _eTblBtnClick: function (sender) {
            if (sender.target.id === this._rteId + "_eTbl_cancel") {
                this._eTblDialog.ejDialog("close");
            }
            else if (sender.target.id === this._rteId + "_eTbl_apply") {
                this._oneditTable();
                this._onChange();
                this._setBackupData();
            }
        },

        _insertCustomTable: function (sender) {
            if (sender.target.id === "insert") {
                var cols = this._customTableDialog.find("#" + this._rteId + "_txtColumns").ejNumericTextbox("getValue");
                var row = this._customTableDialog.find("#" + this._rteId + "_txtRows").ejNumericTextbox("getValue");
                var height = this._customTableDialog.find("#" + this._rteId + "_txtHeight").val();
                var width = this._customTableDialog.find("#" + this._rteId + "_txtWidth").val();
                var align = this._customTableDialog.find("#" + this._rteId + "_ddlAlignment").ejDropDownList("getSelectedValue");
                var spacing = this._customTableDialog.find("#" + this._rteId + "_txtSpacing").val();
                var padding = this._customTableDialog.find("#" + this._rteId + "_txtPadding").val();
                var border = this._customTableDialog.find("#" + this._rteId + "_txtBorder").ejDropDownList("getSelectedValue");
                var caption = this._chkTblCaption.ejCheckBox("isChecked");
                if (!ej.isNullOrUndefined(this._tableInsertAt) && !ej.isNullOrUndefined(this._tableInsertAt.startContainer))
                    this._restoreSelection(this._tableInsertAt);
                var tableEle = this._tableGenerator(row, cols, false, width, height, spacing, padding, align, border, caption);
                this.executeCommand('inserthtml', tableEle);
                var _tableEle = $(this._getDocument()).find("table.e-rte-table");
                _tableEle.attr("contentEditable", false);
                _tableEle.find("td").attr("contentEditable", true);
                var drpDwnObj = this._customTableDialog.find("#" + this._rteId + "_ddlAlignment").data("ejDropDownList");
                if(drpDwnObj.option('value') == "Center" )
                    _tableEle.attr('style',"margin: 0 auto");
                //this._getWindow().document.execCommand('inserthtml', false, tableEle);
                this._onChange();
                drpDwnObj.option({ value: "", enabled: false });
            }
            this._customTableDialog.ejDialog("close");
            this._clearTableFields();
        },

        _clearTableFields: function () {
            this._customTableDialog.find("#" + this._rteId + "_txtColumns").ejNumericTextbox("option", "value", 3);
            this._customTableDialog.find("#" + this._rteId + "_txtRows").ejNumericTextbox("option", "value", 3);
            this._customTableDialog.find("#" + this._rteId + "_txtHeight").val("");
            this._customTableDialog.find("#" + this._rteId + "_txtWidth").val("");
            this._customTableDialog.find("#" + this._rteId + "_ddlAlignment").ejDropDownList("clearText");
            this._customTableDialog.find("#" + this._rteId + "_txtSpacing").val("");
            this._customTableDialog.find("#" + this._rteId + "_txtPadding").val("");
            var border = this._customTableDialog.find("#" + this._rteId + "_txtBorder").ejDropDownList("clearText");
            var caption = this._chkTblCaption.ejCheckBox({ check: false });
        },

        _clearEditTableFields: function () {
            this._eTblHeight = this._eTblDialog.find("input").val("");
            this._eTblDialog.find(".e-numerictextbox").data("ejNumericTextbox").option("value", "");
            this._eTblDialog.find(".e-dropdownlist").data("ejDropDownList").option("value", "");
            this._eTblAlign.disable();
            this._eTblCaption.ejCheckBox({ check: false });
        },

        _clearLinkFields: function () {
            this._txtURL = this._linkDialog.find("#" + this._rteId + "_link_url");
            this._txtURL.removeClass("e-error");
            this._txtURL.val("http://");
            this._txtLinkText = this._linkDialog.find("#" + this._rteId + "_link_text");
            this._txtLinkText.val("");
            this._txtLinkTitle = this._linkDialog.find("#" + this._rteId + "_link_title");
            this._txtLinkTitle.val("");
            this._chkTarget.ejCheckBox({ check: false });
            this._txtURL.focus();
        },
        _clearImgFields: function () {
            this._imgDialog.find("#" + this._rteId + "_img_url").val("http://").removeClass("e-error");
            this._imgDialog.find("#" + this._rteId + "_img_text").val("");
            this._chkImgDimensions.ejCheckBox({ checked: true });
            this._imageLink = this._imgDialog.find("#" + this._rteId + "_img_link");
            this._imageLink.removeClass("e-error");
            this._imageLink.val("");
            this._imageBorderPx = this._imgDialog.find("#" + this._rteId + "_img_border_px").data('ejNumericTextbox');
            this._imageBorderPx.option('value', 0);
            this._imageBorderStyle = this._imgDialog.find("#" + this._rteId + "_img_border_style").data('ejDropDownList');
            this._imageBorderStyle.option('value', "");
            this._imageBorderColor = this._imgDialog.find("#" + this._rteId + "_img_border_color").val("");
            this._imageStyle = this._imgDialog.find("#" + this._rteId + "_img_style");
            this._imageStyle.val("");
        },

        _videoManager: function () {
            var selectNode = this._getSelectedNode();
            this._editVideo = null;
            if (selectNode && selectNode.tagName.toLowerCase() === "iframe") {
                this._editVideo = selectNode;
                this._videoWidth = parseInt(selectNode.width);
                this._videoHeight = parseInt(selectNode.height);
                this._videoDialog.find("textarea.e-rte-video").val(selectNode.outerHTML);
                this._videoDialog.find("#" + this._rteId + "_videoX").val(this._videoWidth);
                this._videoDialog.find("#" + this._rteId + "_videoY").val(this._videoHeight);
            }
            this._videoDialog.ejDialog("open");
            this._videoDialog.find("textarea.e-rte-video").focus();
        },

        _sourceCodeManager: function () {
            ej.isNullOrUndefined(this._sourceDialog) && this._renderSourceDialog();
            this._sourceDialog.find("textarea.e-rte-srctextarea")[0].value = this.getHtml();
            this._sourceDialog.ejDialog("open");
            this._sourceDialog.find("textarea.e-rte-srctextarea").focus();
        },

        _hyperLinkManager: function () {
            this._linkDialog.ejDialog("open");
            this._clearLinkFields();
            var selectNode = (this._isIE()) ? this._currentSelNode : this._getSelectedNode() ;
            (!ej.isNullOrUndefined(selectNode) && selectNode.tagName.toUpperCase() == 'A') ? this._txtLinkText.val($(selectNode).html()) : this._txtLinkText.val(this._getSelText()) ;
            this._selectedHTML = this._getSelText();
            this._hyperLinkSelection = this._getDocument().selection;
            if (selectNode && (/^(A)$/).test(selectNode.nodeName)) {
                this._txtURL.val(selectNode.href);
                this._txtLinkTitle.val(selectNode.title);
            }
            if (this._hyperLinkSelection != null)
                this._hyperLinkSelRange = this._hyperLinkSelection.createRange();
        },

        // table inser 
        _tableManager: function () {
            var tableLi = this._rteToolbar.find("#" + this._rteId + "createTable");
            this._createTable.ejDialog("option", "position", {
                X: tableLi.position().left,
                Y: tableLi.position().top + tableLi.height() + 7
            });
            this._createTable.ejDialog("open");
        },

        // Edit Table Manager
        _eTblManager: function() {
            this._clearEditTableFields();
            var rteID = this._rteId;
            var selectNode = this._currentSelNode ;
            if (!ej.isNullOrUndefined(selectNode) && selectNode.tagName.toLowerCase() === "td") {
                var parentTable = $(selectNode).closest('tbody').closest('table');
                this._getTableObj("Height").val( parentTable.attr('height') );
                this._getTableObj("Width").val( parentTable.attr('width') );
                this._getTableObj("CellSpace").data('ejNumericTextbox').option('value', parentTable[0].cellSpacing );
                this._getTableObj("CellPad").data('ejNumericTextbox').option('value', parentTable[0].cellPadding );
                var align = parentTable[0].align;
                if( !ej.isNullOrUndefined(align) && align !="") {
                    this._eTblAlign.enable();
                    this._eTblAlign.option('value', align );
                }
                var captionEle = $($(selectNode).closest('tbody')).prev('caption');
                if(captionEle.length > 0)
                    this._eTblCaption.ejCheckBox({ checked: true });
                if(parentTable[0].style != "") {
                    this._getTableObj("BrdrPx").data('ejNumericTextbox').option('value', (parentTable[0].style.borderWidth.split('px')[0] != "") ? parentTable[0].style.borderWidth.split('px')[0] : 1 );
                    var obj = this._getTableObj("_border_style").data('ejDropDownList');
                    (parentTable[0].style.borderStyle !="") ? obj.option('value', parentTable[0].style.borderStyle) : obj.option({selectedItemIndex: 0}) ;
                    this._getTableObj("_border_color").val( (parentTable.css('border-color') != "") ? parentTable.css('border-color') : "black"  );
                }
                this._getTableObj("Style").val(parentTable.attr('style'));
                
                // For Row Properties
                var parentRow = $(selectNode).closest('tr');
                this._getTableObj("RHeight").val( parentRow.css('height') );
                this._getTableObj("RAlign").data('ejDropDownList').option('value', parentRow[0].style.textAlign );
                if(parentRow[0].style != "") {
                    this._getTableObj("RBrdrPx").data('ejNumericTextbox').option('value', (parentRow[0].style.borderWidth.split('px')[0] != "") ? parentRow[0].style.borderWidth.split('px')[0] : 1 );
                    var obj = this._getTableObj("RBrdrStyle").data('ejDropDownList');
                    (parentTable[0].style.borderStyle !="") ? obj.option('value', parentTable[0].style.borderStyle) : obj.option({selectedItemIndex: 0});
                    this._getTableObj("RBrdrColor").val( (parentRow.css('border-color') != "") ? parentRow.css('border-color') : "black"  );
                }
                this._getTableObj("RStyle").val(parentRow.attr('style'));

                // For Column Properties
                this._getTableObj("CWidth").val( $(selectNode).css('width') );
                this._getTableObj("CAlign").data('ejDropDownList').option('value', selectNode.style.textAlign );
                if(selectNode.style != "") {
                    this._getTableObj("CBrdrPx").data('ejNumericTextbox').option('value', (selectNode.style.borderLeftWidth.split('px')[0] != "") ? selectNode.style.borderLeftWidth.split('px')[0] : 1);
                    var obj = this._getTableObj("CBrdrStyle").data('ejDropDownList');
                    (parentTable.css('border-style') !="") ? obj.option('value', parentTable.css('border-style')) : obj.option({selectedItemIndex: 0});
                    this._getTableObj("CBrdrColor").val( selectNode.style.borderColor );
                }
                this._getTableObj("CStyle").val($(selectNode).attr('style'));

                // For Cell Properties
                this._getTableObj("ClWidth").val( $(selectNode).css('width') );
                this._getTableObj("ClAlign").data('ejDropDownList').option('value', selectNode.style.textAlign );
                if(selectNode.style != "") {
                    this._getTableObj("ClBrdrPx").data('ejNumericTextbox').option('value', (selectNode.style.borderLeftWidth.split('px')[0] != "") ? selectNode.style.borderLeftWidth.split('px')[0] : 1);
                    var obj = this._getTableObj("ClBrdrStyle").data('ejDropDownList');
                    (parentTable.css('border-style') !="") ? obj.option('value', parentTable.css('border-style')) : obj.option({selectedItemIndex: 0});
                    this._getTableObj("ClBrdrColor").val( selectNode.style.borderColor );
                }
                this._getTableObj("ClStyle").val($(selectNode).attr('style'));
            }
        },

        _getTableObj: function(value) {
            return this._eTblDialog.find('#' + this._rteId + "_eTbl" + value);
        },

        _imageManager: function () {
            this._clearImgFields();
            var selectNode = (this._isIE()) ? this._currentSelNode : this._getSelectedNode() ;
            if (!ej.isNullOrUndefined(selectNode) && selectNode.tagName.toLowerCase() === "img") {
                this._imgWidth = parseInt(selectNode.width);
                this._imgHeight = parseInt(selectNode.height);
                this._imgDialog.find("#" + this._rteId + "_img_url").val(selectNode.src);
                this._imgDialog.find("#" + this._rteId + "_img_text").val(selectNode.alt);
                this._imgDialog.find("#" + this._rteId + "_imgX").val(this._imgWidth);
                this._imgDialog.find("#" + this._rteId + "_imgY").val(this._imgHeight);
                var linkNode = $(selectNode).parent('a');
                if(linkNode.length>0)
                        this._imageLink.val(linkNode.attr('href')) ;
                if(selectNode.style != ""){
                    this._imageBorderPx.option('value', selectNode.style.borderWidth.split('px')[0]);
                    this._imageBorderStyle.option('value', selectNode.style.borderStyle);
                    this._imageBorderColor.val(selectNode.style.borderColor);
                }
                this._imageStyle.val($(selectNode).attr('style'));
            }
            this._imgDialog.ejDialog("open");
            this._imgDialog.find("#" + this._rteId + "_img_url").focus();
        },

        _toolBarClick: function (sender) {
            var $rteEle = $(this.itemsContainer).closest("div.e-rte").find("textarea.e-rte"), rteInstcne = $rteEle.ejRTE("instance"), rteId = $rteEle.attr('id');
            var _itemId = sender.currentTarget.id.replace(rteId, "");
            if ((sender.currentTarget.id == rteId + "copy" || sender.currentTarget.id == rteId + "cut" || sender.currentTarget.id == rteId + "paste") && !rteInstcne._isIE())
                rteInstcne._openAlert(rteInstcne._getLocalizedLabels("copyPastAlert"));
            else
                rteInstcne._selectCommand(sender.currentTarget.id);
            //For Style Tools
            if (rteInstcne._styleItems.indexOf(_itemId) >= 0 || rteInstcne._scriptsItems.indexOf(_itemId) >= 0) {
                if (!$(sender.currentTarget).hasClass("e-active"))
                    this.selectItemByID(sender.currentTarget.id);
                else
                    this.deselectItemByID(sender.currentTarget.id);
            }

                //For Align Tools
            else if (rteInstcne._alignItems.indexOf(_itemId) >= 0) {
                this.selectItemByID(sender.currentTarget.id);
                var index = rteInstcne._alignItems.indexOf(_itemId)
                for (var i = 0; i < rteInstcne._alignItems.length; i++) {
                    if (i != index)
                        this.deselectItemByID(rteId + rteInstcne._alignItems[i]);
                }
            }
                //For Bullets & Numbering tools
            else if (rteInstcne._listItems.indexOf(_itemId) >= 0) {
                if (!$(sender.currentTarget).hasClass("e-active"))
                    this.selectItemByID(sender.currentTarget.id);
                else
                    this.deselectItemByID(sender.currentTarget.id);

                var index = rteInstcne._listItems.indexOf(_itemId);
                if (index == 0)
                    this.deselectItemByID(rteId + rteInstcne._listItems[1]);
                else
                    this.deselectItemByID(rteId + rteInstcne._listItems[0]);
            }
        },

        _toggleEditTable: function () {
            var selTag = this._getSelectedNode();
            if (!ej.isNullOrUndefined(this._toolBarItems)) {
                var _eTblItems = this._toolBarItems.find("#" + this._rteId + "addColumnLeft, #" + this._rteId + "addColumnRight, #" + this._rteId + "addRowAbove, #"
                 + this._rteId + "addRowBelow, #" + this._rteId + "deleteRow, #" + this._rteId + "deleteColumn, #" + this._rteId + "deleteTable");
                
                if(selTag && selTag.tagName && selTag.tagName.toLowerCase() == "td" || selTag.tagName.toLowerCase() == "table" || selTag.tagName.toLowerCase() == "tr"){
                    _eTblItems.show();
                    this._createTable.find("div#" + this._rteId + "_eTblProperties").removeClass('e-disable');
                }
                else if(!ej.isNullOrUndefined(this._createTable)){
                    _eTblItems.hide();
                    this._createTable.find("div#" + this._rteId + "_eTblProperties").addClass('e-disable');
                    
                }
            }
            this._setIframeHeight();
        },

        //Format changes
        _formatChange: function (args) {
            var $rteEle = $(this.element).closest("div.e-rte").find("textarea.e-rte"), rteInstcne = $rteEle.ejRTE("instance"), rteId = $rteEle.attr('id');
            this.getSelectedValue() !== "" && rteInstcne._onFormatBlock(this.getSelectedValue());
            rteInstcne._focus();
            rteInstcne._onChange();
        },

        // Font style chnage event
        _fontStyleChange: function (args) {
            var $rteEle = $(this.element).closest("div.e-rte").find("textarea.e-rte"), rteInstcne = $rteEle.ejRTE("instance"), rteId = $rteEle.attr('id');
            this.getSelectedValue() !== "" && rteInstcne._onFontName(this.getSelectedValue());
            rteInstcne._onChange();
        },

        // Font style chnage event
        _fontSizeChange: function (args) {
            var $rteEle = $(this.element).closest("div.e-rte").find("textarea.e-rte"), rteInstcne = $rteEle.ejRTE("instance"), rteId = $rteEle.attr('id');
            this.getSelectedValue() !== "" && rteInstcne._onFontSize(this.getSelectedValue());
            rteInstcne._onChange();
        },

        //Color select 
        _colorPaletteClick: function (e) {
            var value;
            if ((value = e.target.getAttribute('color-code'))) {
                this._focus();
                this._onFontColor(value);
                this._splitMenu.hide(e);
                this._bgSplitMenu.hide(e);
                this._fontColor = value;
                this._fontColorSplit.find("span.e-btntxt").css("background-color", value);
            }
        },

        //Color select 
        _bgColorPaletteClick: function (e) {
            var value;
            if ((value = e.target.getAttribute('color-code'))) {
                this._focus();
                this._onBGColor(value);
                this._splitMenu.hide(e);
                this._bgSplitMenu.hide(e);
                this._bgColor = value;
                this._bgColorSplit.find("span.e-btntxt").css("background-color", value);
            }
        },

        _fontColorClick: function (e) {
            this._onFontColor(this._fontColor);
            this._splitMenu.hide(e);
        },

        _bgColorClick: function (e) {
            this._onBGColor(this._bgColor);
            this._bgSplitMenu.hide(e);
        },

        _selectCommand: function (selectedId) {
            var rteId = this._rteId;
            if (!(selectedId == rteId + "format" || selectedId == rteId + "fontName" || selectedId == rteId + "fontSize" || selectedId == rteId + "fontColor" || selectedId == rteId + "backgroundColor")) {
                this._focus();
            }
            switch (selectedId) {
                case rteId + "bold":
                    this._onBold();
                    break;
                case rteId + "italic":
                    this._onItalics();
                    break;
                case rteId + "underline":
                    this._onUnderLine();
                    break;
                case rteId + "strikethrough":
                    this._onStrikeThrough();
                    break;
                case rteId + "justifyLeft":
                    this._onJustifyLeft();
                    break;
                case rteId + "justifyRight":
                    this._onJustifyRight();
                    break;
                case rteId + "justifyCenter":
                    this._onJustifyCenter();
                    break;
                case rteId + "justifyFull":
                    this._onJustifyFull();
                    break;
                case rteId + "cut":
                    this._onCut();
                    break;
                case rteId + "copy":
                    this._onCopy();
                    break;
                case rteId + "paste":
                    this._onPaste();
                    break;
                case rteId + "clearFormat":
                    this._onClearFormat();
                    break;
                case rteId + "clearAll":
                    this._clearAllManager();
                    break;
                case rteId + "orderedList":
                    this._onOrderList();
                    break;
                case rteId + "unorderedList":
                    this._onUnorderList();
                    break;
                case rteId + "undo":
                    this._undo();
                    break;
                case rteId + "redo":
                    this._redo();
                    break;
                case rteId + "indent":
                    this._onIndent();
                    this._indentdepth++;
                    break;
                case rteId + "outdent":
                    this._onOutdent();
                    if (this._indentdepth > 0)
                        this._indentdepth--;
                    break;
                case rteId + "createLink":
                    this._hyperLinkManager();
                    break;
                case rteId + "image":
                    this._imageManager();
                    break;
                case rteId + "createTable":
                    this._tableManager();
                    break;
                case rteId + "addRowAbove":
                    this.insertRow(true);
                    break;
                case rteId + "addRowBelow":
                    this.insertRow(false);
                    break;
                case rteId + "addColumnLeft":
                    this.insertColumn(true);
                    break;
                case rteId + "addColumnRight":
                    this.insertColumn(false);
                    break;
                case rteId + "deleteRow":
                    this.removeRow();
                    break;
                case rteId + "deleteColumn":
                    this.removeColumn();
                    break;
                case rteId + "deleteTable":
                    this.removeTable();
                    break;
                case rteId + "superscript":
                    this._onSuperScript();
                    break;
                case rteId + "subscript":
                    this._onSubScript();
                    break;
                case rteId + "upperCase":
                    this._onUpperCase();
                    break;
                case rteId + "lowerCase":
                    this._onLowerCase();
                    break;
                case rteId + "video":
                    this._videoManager();
                    break;
                default:
                    this._onChange();
                    break;
            }
            if (selectedId != rteId + "createLink" && selectedId != rteId + "image" && selectedId != rteId + "video" && selectedId != rteId + "createTable") {
                this._onChange();
            }
            if ((selectedId != rteId + "undo") && (selectedId != rteId + "redo"))
                this._setBackupData();
            var selectEle = this._getSelectedNode();
            this._updateTagInfo(selectEle);
        },
        _focus: function () {
            this._getWindow().document.body.focus();
            //this._selectRange();
            this._restoreSelection(this._selectionRange);
        },
        _getWindow: function () {
            var iframe = document.getElementById(this._iframeId);
            return iframe.contentWindow;
        },
        _getDocument: function () {
            var iframe = document.getElementById(this._iframeId);
            var iframeObject = iframe.contentWindow.document;
            return iframeObject;
        },
        // Iframe command function
        _onBold: function () {
            var args;
            this.executeCommand('bold', args);
        },

        _onItalics: function () {
            var args;
            this.executeCommand('italic', args);
        },
        _onStrikeThrough: function () {
            var args;
            this.executeCommand('strikethrough', args);
        },

        _onUnderLine: function () {
            var args;
            this.executeCommand('underline', args);
        },

        _onJustifyLeft: function () {
            var args;
            this.executeCommand('JustifyLeft', args);
        },

        _onJustifyRight: function () {
            var args;
            this.executeCommand('JustifyRight', args);
        },

        _onJustifyCenter: function () {
            var args;
            this.executeCommand('JustifyCenter', args);
        },

        _onJustifyFull: function () {
            this._alignFlag = true;
            this.executeCommand('justifyfull');
        },

        _onCut: function () {
            var args;
            this.executeCommand('cut', args);
        },
        _onCopy: function () {
            var args;
            this.executeCommand('copy', args);
        },
        _onPaste: function () {
            var args;
            this.executeCommand('paste', args);

        },
        _onOrderList: function () {
            var args;
            this.executeCommand('insertorderedlist', args);
        },
        _onUnorderList: function () {
            var args;
            this.executeCommand('insertunorderedlist', args);
        },
        _onUndo: function () {
            var args;
            this.executeCommand('undo', null);
        },
        _onDelete: function () {
            var args;
            this.executeCommand('delete', null);
        },

        _onRedo: function () {
            var args;
            this.executeCommand('redo', null);
        },

        _onClearFormat: function () {
            var args;
            this.executeCommand('removeformat', args);
            //this.executeCommand('foreColor', 'black');
            this._setBackupData();
            this._onChange();
            this._updateIframeSkin(this.model.iframeAttribute);
        },

        _onFormatBlock: function (formatBlock) {
            this._focus();
            this.executeCommand('formatBlock', formatBlock);
        },

        _onFontName: function (fontsname) {
            this.executeCommand("fontName", fontsname);
        },

        _onFontSize: function (Size) {
            this.executeCommand('fontSize', Size);
        },

        _onFontColor: function (color) {
            this.executeCommand('foreColor', color);
            this._onChange();
            this._updateIframeSkin(this.model.iframeAttribute);
        },

        _onBGColor: function (color) {
            this.executeCommand('backColor', color);
            this._onChange();
        },

        _onIndent: function () {
            this._getWindow().document.body.style.wordWrap = "break-word";
			var node = document.getSelection ? this._getWindow().document.getSelection().focusNode : this._getWindow().document.selection.createRange().parentElement(); 
			if (node && (/^(OL|UL|LI)$/).test(node.nodeName))
                this._listIndent(node);
            else if (node && (/^(#text)$/).test(node.nodeName) && (/^(OL|UL|LI)$/).test($(node).parent()[0].nodeName)) {
                this._listIndent($(node).parents("li")[0], node);
            }
            else
                this.executeCommand('indent', false);
            node && $(node).focus();
            this._updateIframeSkin(this.model.iframeAttribute);
        },

        _onOutdent: function () {
			var node = document.getSelection ? this._getWindow().document.getSelection().focusNode : this._getWindow().document.selection.createRange().parentElement();            
			if (node && (/^(OL|UL|LI)$/).test(node.nodeName))
                this._listOutdent(node);
            else if (node && (/^(#text)$/).test(node.nodeName) && (/^(OL|UL|LI)$/).test($(node).parent()[0].nodeName)) {
                this._listOutdent($(node).parents("li")[0], node);
            }
            else
                this.executeCommand('outdent', false);
        },

        _onSelectAll: function () {
            this.executeCommand('selectall');
        },
        _onHyperLink: function (val) {
            this.executeCommand('createlink', val);
        },

        _onInsertLink: function () {
            var oEl;
            var sHtml;
            if (!this._isUrl($.trim(this._txtURL.val()))) {
                this._txtURL.addClass("e-error");
                return false;
            }
            var selectedNode = (this._isIE()) ? this._currentSelNode : this._getSelectedNode() ;
            if( !ej.isNullOrUndefined(selectedNode) && selectedNode.tagName.toUpperCase() == 'A'){
                selectedNode.href = this._txtURL.val();
                selectedNode.title = this._txtLinkTitle.val();
                selectedNode.innerHTML = this._txtLinkText.val();
                (this._chkTarget.ejCheckBox("isChecked")) ? selectedNode.target = "_blank" : selectedNode.target = "_self" ;
            }
            else {
                oEl = ej.buildTag("a", "", {}, { href: this._txtURL.val(), title: this._txtLinkTitle.val() });
                (this._chkTarget.ejCheckBox("isChecked")) ? oEl[0].target = "_blank" : oEl[0].target = "_self";
                
                var oSelection;
                var oSelRange;
                this._focus();
                if (this._isIE()) {
                    if (this._selectedHTML.length == 0) {
                        this._selectedHTML = this._txtLinkText.val();
                    }
                    oEl[0].innerHTML = this._selectedHTML;
                    sHtml = oEl[0].outerHTML;
                }
                else {
                    oEl.html(this._txtLinkText.val());
                    sHtml = $('<div>').append($(oEl).clone()).html();
                }
                if (sHtml) {
                    if (this._isIE()) {
                        this._pasteHtml(sHtml);
                    }
                    else {
                        this._getDocument().execCommand('inserthtml', false, sHtml);
                    }
                    this._focus();
                }
            }
            this._linkDialog.ejDialog("close");
        },

        _setImageProperties: function(imgEle, imgX, imgY, imageStyle, borderPx, borderStyle, borderColor) {
            if (this.model.showDimensions) {
                (imgX != "") && (imgEle[0].width = imgX);
                (imgY != "") && (imgEle[0].height = imgY);
            }
            // Apply Image Style 
            imgEle = $(imgEle).attr('style', imageStyle);
            //Apply Border to Image
            border = borderPx + "px ";
            (!ej.isNullOrUndefined(borderStyle) && borderStyle != "" ) ? border += borderStyle+" " : border += "solid " ;
            (!ej.isNullOrUndefined(borderColor) && borderColor != "") ? border += borderColor : border += "black" ;
            imgEle = imgEle.css('border', border);
        },

        _onInsertImage: function () {
            var url = this._imgDialog.find("#" + this._rteId + "_img_url");
            var altText = this._imgDialog.find("#" + this._rteId + "_img_text");
            var imgX = this._imgDialog.find("#" + this._rteId + "_imgX").val();
            var imgY = this._imgDialog.find("#" + this._rteId + "_imgY").val();
            var proxy = this;
            this._restoreSelection(this._selectionRange);
            var element,border, nodeName="", imageLink = proxy._imageLink.val(), borderPx = proxy._imageBorderPx.getValue(), borderStyle = proxy._imageBorderStyle.getSelectedValue(), borderColor = proxy._imageBorderColor.val(), imageStyle = proxy._imageStyle.val();
            var selectedNode = (proxy._isIE()) ? proxy._currentSelNode : proxy._getSelectedNode() ;
            (!ej.isNullOrUndefined(selectedNode)) && (nodeName =  selectedNode.nodeName.toUpperCase()); 

            if(nodeName != 'IMG') {
                //Url Validation
                var _img = $("<img>", {
                src: url.val(),
                error: function () { url.addClass("e-error"); },
                success: function () {
                    var imgEle = ej.buildTag("img", "", {}, { alt: altText.val(), src: url.val() });
                    proxy._setImageProperties(imgEle, imgX, imgY, imageStyle,  borderPx, borderStyle, borderColor );
                    if( imageLink != "" ) {
                        if(proxy._isUrl($.trim(imageLink))) {
                            var link = ej.buildTag("a", "", {}, { href: imageLink });
                            $(imgEle[0].outerHTML).appendTo(link);
                            imgEle = link;
                        }
                        else {
                            proxy._imageLink.addClass("e-error");
                            return false;
                        }
                    }
                    var element = imgEle[0].outerHTML;
                    proxy.executeCommand('inserthtml', element);
                }
                });
            }
            else {
                selectedNode.src = url.val();
                selectedNode.alt = altText.val();
                proxy._setImageProperties($(selectedNode), imgX, imgY, imageStyle,  borderPx, borderStyle, borderColor );
                var linkNode = $(selectedNode).parent('a');
                if( imageLink != "" ) {
                    if(proxy._isUrl($.trim(imageLink))) {
                        if(linkNode.length>0)
                            linkNode.attr('href', imageLink);
                        else {
                            var link = ej.buildTag("a", "", {}, { href: imageLink });
                            $(selectedNode.outerHTML).appendTo(link);
                            $(selectedNode).replaceWith(link);
                        }
                    }
                    else {
                        proxy._imageLink.addClass("e-error");
                        return false;
                    }
                }
                else 
                    (linkNode.length>0) && (linkNode.attr('href', "")) ;
            }
            proxy._imgDialog.ejDialog("close");
        },
        //Insert video
        _insertVideo: function (sender) {
            var videoLink = $.trim(this._videoDialog.find("textarea.e-rte-video")[0].value);
            var videoX = this._videoDialog.find("#" + this._rteId + "_videoX").val();
            var videoY = this._videoDialog.find("#" + this._rteId + "_videoY").val();
            this._restoreSelection(this._selectionRange);
            var ele = $(videoLink);
            if (sender.target.id === "video_insert" && videoLink !== "") {
                var _index, _link = ele[0].src;
                if (!ej.isNullOrUndefined(_link) && _link.indexOf("http:") == -1) {
                    _index = videoLink.indexOf("src=") + 5;
                    videoLink = videoLink.substr(0, _index) + "http:" + videoLink.substr(_index);
                }
                var _index, ele = $(videoLink)[0];
                if (this.model.showDimensions) {
                    (videoX != "") && (ele.width = videoX);
                    (videoX != "") && (ele.height = videoY);
                }
                this._editVideo && $(this._editVideo).remove();
                this.executeCommand('inserthtml', ele.outerHTML);
                //this._getWindow().document.execCommand('inserthtml', false, ele.outerHTML);
                this._onChange();
                this._setBackupData();
                this._videoDialog.ejDialog("close");
            }
            else if (sender.target.id === "video_cancel")
                this._videoDialog.ejDialog("close");
            else
                this._openAlert(this._getLocalizedLabels("videoError"));
            this._videoDialog.find("textarea.e-rte-video")[0].value = "";
            this._videoDialog.find("#" + this._rteId + "_videoX").val("");
            this._videoDialog.find("#" + this._rteId + "_videoY").val("");
            this._chkvideoDimensions.ejCheckBox({ check: true });
        },

        _oneditTable: function() {
            var selectedNode = this._currentSelNode ;
            // Table Properties
            var parentTable = $(selectedNode).closest('tbody').closest('table');
            var captionEle = $($(selectedNode).closest('tbody')).prev('caption');
            (this._eTblCaption.ejCheckBox("isChecked")) ?  (captionEle.length == 0) && ej.buildTag('caption').append($('<br type="_moz">')).insertBefore($($(selectedNode).closest('tbody'))) : (captionEle.length > 0) && captionEle.remove();
            var tblAlign = this._eTblAlign.getSelectedValue();
            var style = this._getTableObj("Style").val();
            if(tblAlign == 'center') style += ";margin: 0 auto;";
            (style) && parentTable.attr('style', style);
            parentTable.attr({
                width : this._getTableObj("Width").val(),
                height : this._getTableObj("Height").val(),
                cellPadding : this._getTableObj("CellPad").data('ejNumericTextbox').getValue(),
                cellSpacing : this._getTableObj("CellSpace").data('ejNumericTextbox').getValue(),
                align : tblAlign,
            }).css({
                'border-width' : this._getTableObj("BrdrPx").data('ejNumericTextbox').getValue() + "px" ,
                'border-style' : this._getTableObj("_border_style").data('ejDropDownList').getSelectedValue() ,
                'border-color' : this._getTableObj("_border_color").val()
            });
            
            // Row Properties
            var parentRow = $(selectedNode).closest('tr');
            parentRow.attr({
                style : this._getTableObj("RStyle").val()
            }).css({
                'height' : this._getTableObj("RHeight").val(),
                'text-align' : this._getTableObj("RAlign").data('ejDropDownList').getSelectedValue(),
                'border-width' : this._getTableObj("RBrdrPx").data('ejNumericTextbox').getValue() + "px" ,
                'border-style' : this._getTableObj("RBrdrStyle").data('ejDropDownList').getSelectedValue() ,
                'border-color' : this._getTableObj("RBrdrColor").val()
            });

            // Column Properties
            var border = this._getTableObj("CBrdrPx").data('ejNumericTextbox').getValue() + "px ";
            border += this._getTableObj("CBrdrStyle").data('ejDropDownList').getSelectedValue() +" ";
            border += this._getTableObj("CBrdrColor").val();
            var curRow = $(selectedNode).closest("tr");
            var allRows = curRow.closest("table").find("tr");
            var colIndex = curRow.find("td").index(selectedNode);
            for (var i = 0; i < allRows.length; i++) {
                var element = $($(allRows[i]).find("td")[colIndex]);
                if(allRows.length > 1) {
                    element.attr({
                        style : this._getTableObj("CStyle").val()
                    }).css({
                        'width' : this._getTableObj("ColumnHeight").val(),
                        'text-align' : this._getTableObj("CAlign").data('ejDropDownList').getSelectedValue(),
                        'border-left' : border,
                        'border-right' : border
                    });
                    if(i==0) element.css('border-top', border);
                    if(i==allRows.length-1) element.css('border-bottom', border);
                }
                else
                    $(selectedNode).css('border', border);
            }

            // Cell Prperties
            $(selectedNode).attr({
                style : this._getTableObj("ClStyle").val()
            }).css({
                'width' : this._getTableObj("CellHeight").val(),
                'text-align' : this._getTableObj("ClAlign").data('ejDropDownList').getSelectedValue(),
                'border-width' : this._getTableObj("ClBrdrPx").data('ejNumericTextbox').getValue() + "px" ,
                'border-style' : this._getTableObj("ClBrdrStyle").data('ejDropDownList').getSelectedValue() ,
                'border-color' : this._getTableObj("ClBrdrColor").val()
            });
            this._eTblDialog.ejDialog("close");
        },

        //Clears all the contents
        _clearAllManager: function () {
            this._alertWindow.find(".e-alert-cancel").show();
            this._openAlert(this._getLocalizedLabels("deleteAlert"));
        },

        _onUnlink: function () {
            var args;
            this.executeCommand('unlink', args);
        },
        _onSuperScript: function () {
            var args;
            this.executeCommand('SuperScript', args);
        },
        _onSubScript: function () {
            var args;
            this.executeCommand('SubScript', args);
        },
        _onInsertTable: function (rows, cols, designTime) {
            var tableEle = this._tableGenerator(rows, cols, designTime);
            this.executeCommand('inserthtml', tableEle);
        },

        _onUpperCase: function () {
            if (!document.getSelection){
                if (this._getWindow().document.selection.type.toLowerCase() == "control") {
                    return;
                }
                this._getDocument().selection.createRange().pasteHTML(this._seleText.toUpperCase());
            }
            else {
                var selection = this._getWindow().getSelection();
                var range = selection.getRangeAt(0);
                var node = range.commonAncestorContainer;
                var children = range.commonAncestorContainer.children;
                if (children != null) {
                    var contents = range.extractContents();
                    var result = this._changeCase(contents, "Upper");
                    var resultCopy = $(result).clone();
                    var tempDiv = document.createElement('div');
                    $(tempDiv).append($(resultCopy));
                    if (tempDiv.innerHTML != null && tempDiv.innerHTML != "")
                        this._getWindow().document.execCommand('inserthtml', false, tempDiv.innerHTML);
                }
                else {
                    var replaceString = node.nodeValue.substring(range.startOffset, range.endOffset).toUpperCase();
                    var startString = node.nodeValue.substr(0, range.startOffset);
                    var endString = node.nodeValue.substring(range.endOffset);
                    node.nodeValue = startString + replaceString + endString;
                }
            }

        },

        _changeCase: function (parentNode, toChangeCase) {
            var s = parentNode.childNodes;
            if (s.length > 0) {
                for (var t = 0; t < s.length; t++) {
                    if (s[t].nodeType == 3) {
                        s[t].nodeValue = toChangeCase == "Upper" ? s[t].nodeValue.toUpperCase() : s[t].nodeValue.toLowerCase();
                    }
                    else {
                        if (s[t].nodeType == 1 && s[t].tagName.toUpperCase() != "FONT") {
                            this._changeCase(s[t], toChangeCase);
                        }
                    }
                }
            }
            return s;
        },

		
        	_onLowerCase: function () {
			if (!document.getSelection) {
			if (this._getWindow().document.selection.type.toUpperCase() == "control") {
			return;
			}
			this._getDocument().selection.createRange().pasteHTML(this._seleText.toLowerCase());
			}
			else 
			{
            var selection = this._getWindow().getSelection();
            var range = selection.getRangeAt(0);
            var node = range.commonAncestorContainer;
            var children = range.commonAncestorContainer.children;
            if (children != null) {
                var contents = range.extractContents();
                var result = this._changeCase(contents, "Lower");
                var resultCopy = $(result).clone();
                var tempDiv = document.createElement('div');
                $(tempDiv).append($(resultCopy));
                if (tempDiv.innerHTML != null && tempDiv.innerHTML != "")
                    this._getWindow().document.execCommand('inserthtml', false, tempDiv.innerHTML);
            }
            else {
                var replaceString = node.nodeValue.substring(range.startOffset, range.endOffset).toLowerCase();
                var startString = node.nodeValue.substr(0, range.startOffset);
                var endString = node.nodeValue.substring(range.endOffset);
                node.nodeValue = startString + replaceString + endString;
            }
		}
        },

        _listIndent: function (node, selection) {
            var sibling, newList;
            sibling = node.previousSibling;

            if (sibling && this._isListNode(sibling)) {
                sibling.appendChild(node);
                return true;
            }
            if (sibling && sibling.nodeName == 'LI' && this._isListNode(sibling.lastChild)) {
                sibling.lastChild.appendChild(node);
                this._combineLists(node.lastChild, sibling.lastChild, node);
                return true;
            }

            sibling = node.nextSibling;

            if (sibling && this._isListNode(sibling)) {
                sibling.insertBefore(node, sibling.firstChild);
                return true;
            }

            if (sibling && sibling.nodeName == 'LI' && this._isListNode(node.lastChild)) {
                return false;
            }

            sibling = node.previousSibling;
            if (sibling && sibling.nodeName == 'LI') {
                newList = ej.buildTag(node.parentNode.nodeName)[0];
                sibling.appendChild(newList);
                newList.appendChild(node);
                this._combineLists(node.lastChild, newList, node);
                return true;
            }
            return true;
        },

        _combineLists: function (from, to, li) {
            var node;

            if (this._isListNode(from)) {
                while ((node = li.lastChild.firstChild)) {
                    to.appendChild(node);
                }

                $(from).remove();
            }
        },
        _isListNode: function (node) {
            return node && (/^(OL|UL)$/).test(node.nodeName);
        },
        _isFirstChild: function (node) {
            return node.parentNode.firstChild == node;
        },

        _isLastChild: function (node) {
            return node.parentNode.lastChild == node;
        },
        _removeEmptyList: function (node) {
            if (node && $(node).children().length === 0)
                $(node).remove();
        },
        _listOutdent: function (node) {
            var ul = $(node.parentNode), ulParent = ul[0].parentNode, newBlock;
            node = $(node);
            if (this._isFirstChild(node[0]) && this._isLastChild(node[0])) {
                if (ulParent.nodeName == "LI") {
                    node.insertAfter(ulParent);
                    this._removeEmptyList(ulParent);
                    ul.remove();
                } else if (this._isListNode(ulParent)) {
                    ul.remove();
                } else {
                    this.executeCommand('outdent', false);
                    ////ulParent.insertBefore(createNewTextBlock(node[0]), ul[0]);
                    //ul.remove();
                }

                return true;
            } else if (this._isFirstChild(node[0])) {
                if (ulParent.nodeName == "LI") {
                    node.insertAfter(ulParent);
                    node[0].appendChild(ul[0]);
                    this._removeEmptyList(ulParent);
                } else if (this._isListNode(ulParent)) {
                    ulParent.insertBefore(node[0], ul[0]);
                } else {
                    this.executeCommand('outdent', false);
                    //ulParent.insertBefore(createNewTextBlock(node[0]), ul[0]);
                    //node.remove();
                }

                return true;
            } else if (this._isLastChild(node[0])) {
                if (ulParent.nodeName == "LI") {
                    node.insertAfter(ulParent);
                } else if (this._isListNode(ulParent)) {
                    node.insertAfter(ul[0]);
                } else {
                    this.executeCommand('outdent', false);
                    //node.insertAfter(createNewTextBlock(node[0]), ul[0]);
                    //node.remove();
                }

                return true;
            } else {
                this.executeCommand('outdent', false);
            }

            return false;
        },

        _updateIndent: function (node) {
		if (!ej.isNullOrUndefined(this._toolBarObj)) {
            this._toolBarObj.enableItemByID(this._rteId + "indent");
            if (node && (/^(LI)$/).test(node.nodeName)) {
                if ($(node).parents(node.parentNode.nodeName).last().find("li").index(node) === 0)
                    this._toolBarObj.disableItemByID(this._rteId + "indent");
            }
		  }
        },

        //Get char Count
        _updateCharCount: function () {
            if (this.model.showFooter && this.model.showWordCount) {
                var iframeHtml = this._getText();
                this._wordCount.html(this._getLocalizedLabels("characters") + " : " + $.trim(iframeHtml).length);
            }
        },

        //Update tag info 
        _updateTagInfo: function (element) {
            var text = "", pTag, curNode = element.tagName ? element.tagName.toLowerCase() : "html";
            if (this.model.showFooter && this.model.showHtmlTagInfo && curNode != "body") {
                var parentNodes = $(element).parents();
                for (var i = parentNodes.length - 1; i >= 0; i--) {
                    pTag = parentNodes[i].tagName.toLowerCase();
                    if (pTag != "body" && pTag != "html")
                        text += " » " + pTag;
                }
                text += " » " + curNode;
                this._htmlInfo.html(text);
            }
        },

        //Update Model value 
        _updateValue: function () {
            this.value(this._getDocument().body.innerHTML);
			this.element.val(this._getDocument().body.innerHTML);
        },

        //Clear Formats
        _setClearFormat: function () {
            var selItemslength = this._rteToolbar.find(".e-active").length;
        },


        //Updates the status of the toolbar items
        _updateToolbarStatus: function () {
            if (this.model.showToolbar) {
                //Style Tools
                for (var i = 0; i < this._styleItems.length; i++) {
                    if (this._getCommandStatus(this._styleItems[i].toLowerCase()))
                        this._toolBarObj.selectItemByID(this._rteId + this._styleItems[i]);
                    else
                        this._toolBarObj.deselectItemByID(this._rteId + this._styleItems[i]);
                }

                //Align Tools
                for (var i = 0; i < this._alignItems.length; i++) {
                    if (this._getCommandStatus(this._alignItems[i].toLowerCase()))
                        this._toolBarObj.selectItemByID(this._rteId + this._alignItems[i]);
                    else
                        this._toolBarObj.deselectItemByID(this._rteId + this._alignItems[i]);
                }

                //Formatting Tools
                for (var i = 0; i < this._scriptsItems.length; i++) {
                    if (this._getCommandStatus(this._scriptsItems[i].toLowerCase()))
                        this._toolBarObj.selectItemByID(this._rteId + this._scriptsItems[i]);
                    else
                        this._toolBarObj.deselectItemByID(this._rteId + this._scriptsItems[i]);
                }


                //List Tools
                if (this._getCommandStatus('InsertOrderedList'))
                    this._toolBarObj.selectItemByID(this._rteId + "orderedList");
                else
                    this._toolBarObj.deselectItemByID(this._rteId + "orderedList");

                if (this._getCommandStatus('InsertUnorderedList'))
                    this._toolBarObj.selectItemByID(this._rteId + "unorderedList");
                else
                    this._toolBarObj.deselectItemByID(this._rteId + "unorderedList");
            }

        },

        _updateFormat: function () {
            if (this._formatDDL && this.model.showToolbar && !ej.isNullOrUndefined(this.model.tools["formatStyle"])) {
                var formatName = $.trim(this._getCommandValue('formatblock').replace(/'/g, ""));
                if (!this._isIE())
                    formatName = "<" + formatName + ">";
                if (ej.isNullOrUndefined(formatName) || formatName == "")
                    this._formatDDL.ejDropDownList({ "value": this._getLocalizedLabels("format") });
                else
                    this._formatDDL.ejDropDownList("setSelectedValue", formatName);
            }
        },
        // Update font options
        _updateFontOptionStatus: function () {
            if (this.model.showFontOption && this.model.showToolbar && !ej.isNullOrUndefined(this.model.tools["font"])) {
                var fontName = $.trim(this._getCommandValue('fontname').replace(/"/g, "").replace(/'/g, "").replace(/, /g, ","));
                (ej.isNullOrUndefined(fontName) || fontName == "") && (fontName = "Segoe UI");
                this._fontStyleDDL.ejDropDownList("setSelectedValue", fontName);
                // Font Size DDL
                var fontSize = this._getCommandValue('fontsize');
                (ej.isNullOrUndefined(fontSize) || fontSize == "") && (fontSize = 2);
                this._fontSizeDDL.ejDropDownList("setSelectedValue", fontSize);
            }
        },

        _getSelText: function () {
            var selectedtext = '';
            var selection;
            var textnode;
            if (!this._isIE()) {
                if (window.getSelection) {
                    selection = this._getWindow().getSelection();
                    var range = document.createRange();
                    range = selection.getRangeAt(0);
                    textnode = document.createTextNode(range.toString());
                    selectedtext = textnode.nodeValue;
                }
                else if (document.getSelection) {
                    selectedtext = this._getWindow().document.getSelection();
                }
            }
            else
                selectedtext = this._seleText;
            return selectedtext;
        },

        _isIE: function () {
            var _ie = false, browserInfo = ej.browserInfo();
            if (browserInfo.name == 'msie') {
                _ie = true;
            }
            return _ie;
        },
        //Vaidate the input URL string
        _isUrl: function (url) {
            var regexp = /(ftp|http|https):\/\/(\w+:{0,1}\w*@)?(\S+)(:[0-9]+)?(\/|\/([\w#!:.?+=&%@!\-\/]))?/;
            return regexp.test(url);
        },
        _pasteHtml: function (html) {
            var sel, range;
            this._focus();
            if (window.getSelection) {
                // IE9 and non-IE
                sel = this._getWindow().getSelection();
                if (sel.getRangeAt && sel.rangeCount) {
                    range = sel.getRangeAt(0);
                    range.deleteContents();
                    // some browsers (IE9, for one)
                    var el = document.createElement("div");
                    el.innerHTML = html;
                    var frag = this._getDocument().createDocumentFragment(), node, lastNode;
                    while ((node = el.firstChild)) {
                        lastNode = frag.appendChild(node);
                    }
                    range.insertNode(frag);

                    // Preserve the selection
                    if (lastNode) {
                        range = range.cloneRange();
                        range.setStartAfter(lastNode);
                        range.collapse(true);
                        sel.removeAllRanges();
                        sel.addRange(range);
                    }
                }
            } else if (this._getDocument().selection && this._getDocument().selection.type != "Control") {
                // IE < 9
                if(this._ieSelectionRange)
                    this._ieSelectionRange.pasteHTML(html);
                else
                    this._getDocument().body.insertAdjacentHTML("afterBegin", html);
            }
        },
        //get locale text
        _getLocalizedLabels: function (property) {
            return ej.RTE.Locale[this.model.locale][property] === undefined ? ej.RTE.Locale["en-US"][property] : ej.RTE.Locale[this.model.locale][property];
        },

        //get Selection Node
        _getSelectedNode: function () {
            var temp=this._getRange();
            var element, rng = (this._ieSelectionRange && temp.offsetLeft == 0 && temp.offsetTop == 0 && temp.boundingHeight == 0) ? this._ieSelectionRange : temp;
            var startContainer = rng.startContainer, endContainer = rng.endContainer;
            var startOffset = rng.startOffset, endOffset = rng.endOffset;
            if (!rng) {
                return this._getDocument().selection;
            }

            if (rng.setStart) {
                element = rng.commonAncestorContainer;
                //Image tage and link selction
                if (!rng.collapsed) {
                    if (startContainer == endContainer) {
                        if (endOffset - startOffset < 2) {
                            if (startContainer.hasChildNodes()) {
                                element = startContainer.childNodes[startOffset];
                            }
                        }
                    }
                    if (startContainer.nodeType === 3 && endContainer.nodeType === 3) {
                        if (startContainer.length === startOffset) {
                            startContainer = this._skipEmptyNode(startContainer.nextSibling, true);
                        } else {
                            startContainer = startContainer.parentNode;
                        }

                        if (endOffset === 0) {
                            endContainer = this._skipEmptyNode(endContainer.previousSibling, false);
                        } else {
                            endContainer = endContainer.parentNode;
                        }

                        if (startContainer && startContainer === endContainer) {
                            return startContainer;
                        }
                    }
                }

                if (element && element.nodeType == 3) {
                    return element.parentNode;
                }
                return element;
            }
            element = rng.item ? rng.item(0) : rng.parentElement();
            return element;
        },

        _skipEmptyNode: function (node, forwards) {
            var orig = node;

            while (node && node.nodeType === 3 && node.length === 0) {
                node = forwards ? node.nextSibling : node.previousSibling;
            }

            return node || orig;
        },

        _getSelection: function () {

            var selCurrent;
            if (this._isIE()) {
                selCurrent = this._getDocument().selection.createRange();
                selCurrent.type = this._getDocument().selection.type;
            }
            else {
                selCurrent = window.getSelection();
                if (selCurrent.rangeCount) {
                    sText = selCurrent.getRangeAt(0);
                }
                selCurrent.type = this._getWindow().getSelection().type;
            }
            return selCurrent;
        },

        _saveSelection: function () {
            var win = this._getWindow();
            var doc = win.document;
            var sel = win.getSelection ? win.getSelection() : doc.selection;
            var range;

            if (sel) {
                if (sel.createRange) {
                    range = sel.createRange();
                } else if (sel.getRangeAt(0)) {
                    range = sel.getRangeAt(0);
                } else if (sel.anchorNode && sel.focusNode && doc.createRange) {
                    // Older WebKit browsers
                    range = doc.createRange();
                    range.setStart(sel.anchorNode, sel.anchorOffset);
                    range.setEnd(sel.focusNode, sel.focusOffset);

                    // Handle the case when the selection was selected backwards (from the end to the start in the
                    // document)
                    if (range.collapsed !== sel.isCollapsed) {
                        range.setStart(sel.focusNode, sel.focusOffset);
                        range.setEnd(sel.anchorNode, sel.anchorOffset);
                    }
                }
            }
            return range;
        },

        _restoreSelection: function (range) {
            var win = this._getWindow();
            var doc = win.document;
            var sel = win.getSelection ? win.getSelection() : doc.selection;
            if (sel && range) {
                if (range.select) {
                    range.select();
                } else if (sel.removeAllRanges && sel.addRange) {
                    sel.removeAllRanges();
                    sel.addRange(range);
                }
            }
        },

        _getRange: function () {
            if (this._isIE()) {
                return this._ieSelection =  (window.getSelection && !ej.isNullOrUndefined(this._selectionRange)) ? this._selectionRange : this._saveSelection() ;
            }
            else if (this._getWindow().getSelection().rangeCount > 0)
                return this._getWindow().getSelection().getRangeAt(0);
            else {
                var iframe = document.getElementsByTagName("iframe")[0];
                var win = iframe.contentWindow;
                var range = win.document.createRange();
                range.setStart(win.document.body, 0);
                range.setEnd(win.document.body, 0);
                win.document.body.focus();
                return range;
            }
        },
        //Sets Back up data
        _setBackupData: function () {
            var htmlTxt = this.getHtml();
            var bkuplen = this._backupArray.length;
            if (bkuplen == this.model.undoStackLimit + 1)
                this._backupArray.splice(0, 1);
            if (htmlTxt != this._backupArray[bkuplen - 1] && this.model.undoStackLimit > 0)
                this._backupArray.push(htmlTxt);
            this._undoRedoPosition = this._backupArray.length - 1;
            if (this.model.showToolbar && this._rteToolbar.find("#" + this._rteId + "undo") && this._backupArray.length > 1)
                this._toolBarObj.enableItemByID(this._rteId + "undo");
        },
        _getSelectedHtmlString: function () {
            if (typeof document.selection != "undefined")
                return this._seleText;
            else {
                var container = document.createElement("div");
                container.appendChild(this._getRange().cloneContents());
                return container.innerHTML;
            }
        },

        _getText: function () {
            var _body = this._getDocument().body;
            if (_body != null) {
                if (!this._isIE())
                    return _body.textContent;
                else
                    return _body.innerText;
            }
        },
        _validateMaxLength: function (e) {
            if (this.model.maxLength != null) {    // MaxLength function - Restricts the user for entering text in RTE
                var len = this.model.maxLength;
                var usertext = this._getText();
                if (len <= usertext.length) {
                    if (!(e.keyCode < 47 || e.ctrlKey && e.keyCode == 65 || e.ctrlKey && e.keyCode == 67 || e.ctrlKey && e.keyCode == 86 || e.ctrlKey && e.keyCode == 88)) {
                        this._keypressFlag = false;
                        this._cancelEvent(e);
                    }
                }
            }
        },
        _cancelEvent: function (e) {
            e.returnValue = false;
            e.stopPropagation();
            e.preventDefault();
            return false;
        },

        _alignToolUpdate: function (name) {
            for (var i = 0; i < this._alignItems.length; i++) {
                if (this._alignItems[i] === name)
                    this._toolBarObj.selectItemByID(this._rteId + "underline");
                else
                    this._toolBarObj.deselectItemByID(this._rteId + "underline");
            }
        },

        //Gets the status of the formats of the current text
        _getCommandStatus: function (commandName) {
            var state = this._getDocument().queryCommandState(commandName);
            return state;
        },

        //Gets the value of the font-size and font-name of the current text
        _getCommandValue: function (commandName) {
            var value = this._getDocument().queryCommandValue(commandName);
            if (this._isIE()) {
                if (commandName == "fontname") {
                    for (var i = 0; i < this.model.fontName.length; i++) {
                        if (value == this.model.fontName[i].text) {
                            value = this.model.fontName[i].value;
                            break;
                        }
                    }
                    if (ej.isNullOrUndefined(value) || value == "")
                        value = this.model.fontName[0].value;
                }
                if (commandName == "formatblock") {
                    for (var i = 2; i < this.model.format.length; i++) {
                        if (value == this.model.format[i].text) {
                            value = this.model.format[i].value;
                            break;
                        }
                    }
                    if (value == "Normal")
                        value = this.model.format[0].value;
                }
            }
            return value;
        },

        //Undo 
        _undo: function () {
            if (this._undoRedoPosition > 0) {
                this.setHtml(this._backupArray[this._undoRedoPosition - 1]);
                this._undoRedoPosition--;
                this._updateToolbarStatus();
                this._updateFontOptionStatus();
                this._updateFormat();
                this._updateCharCount();
                this._toolBarObj.enableItemByID(this._rteId + "redo");
            }
            if (this._undoRedoPosition == 0) {
                this._toolBarObj.disableItemByID(this._rteId + "undo");
                this._toolBarObj.enableItemByID(this._rteId + "redo");
            }
            this._focus();
        },

        //Redo 
        _redo: function () {
            if (this._undoRedoPosition != this._backupArray.length) {
                if (this._backupArray[this._undoRedoPosition + 1] != null) {
                    var redoString = this._backupArray[this._undoRedoPosition + 1];
                    this.setHtml(redoString);
                    this._undoRedoPosition++;
                }
                else {
                    var redoString = this._backupArray[this._backupArray.length - 2];
                    this.setHtml(redoString);
                    this._undoRedoPosition = this._backupArray.length - 2;
                }
                this._updateToolbarStatus();
                this._updateFontOptionStatus();
                this._updateFormat();
                this._updateCharCount();
                if (!ej.isNullOrUndefined(this._toolBarObj)) {
                    if (this._backupArray.length - 1 > 0)
                        this._toolBarObj.enableItemByID(this._rteId + "undo");
                    if (this._undoRedoPosition == this._backupArray.length - 1)
                        this._toolBarObj.disableItemByID(this._rteId + "redo");
                }
            }
            this._focus();
        },

        //set content
        _setContent: function (htmltxt) {
            var editdoc = this._getDocument();
            editdoc.open();
            editdoc.write(htmltxt);
            editdoc.close();
            editdoc.EditMode = true;
        },

        //table start to create 
        _tableCellStart: function (e) {
            this._tableInsertAt = this._saveSelection();
        },

        //Select the table Cell on mouse over
        _tableCellSelect: function (e) {
            var cellEle, target = $(e.target);
            var row = target.parent().index();
            var col = target.index();
            this._createTable.find("div.e-rte-tablecell").each(function (index) {
                cellEle = $(this);
                if (cellEle.index() <= col && cellEle.parent().index() <= row) {
                    cellEle.addClass("e-active");
                }
                else {
                    cellEle.removeClass("e-active");
                }
            });
            this._tblheaderDiv.html((col + 1) + "x" + (row + 1) + " Table");
            $(this._getDocument()).find("table.e-rte-tableremove").remove();
            if (!ej.isNullOrUndefined(this._tableInsertAt)) {
                this._restoreSelection(this._tableInsertAt);
                this._onInsertTable((row + 1), (col + 1), true);
            }
        },
        _tableCellLeave: function (e) {
            this._createTable.find("div.e-rte-tablecell").removeClass("e-active");
            $(this._getDocument()).find("table.e-rte-tableremove").remove();
            this._tblheaderDiv.html(this._getLocalizedLabels("createTable"));
        },
        _tableCellDown: function (e) {
            var target = $(e.target);
            var row = target.parent().index() + 1;
            var col = target.index() + 1;
            this._focus();
            $(this._getDocument()).find("table.e-rte-tableremove").remove();
            if (!ej.isNullOrUndefined(this._tableInsertAt)) {
                this._restoreSelection(this._tableInsertAt);
                this._onInsertTable(row, col, false);
            }
            var _tableEle = $(this._getDocument()).find("table.e-rte-table");
            _tableEle.attr("contentEditable", false);
            _tableEle.find("td").attr("contentEditable", true);
            this._createTable.ejDialog("close");
            this._setBackupData();
            this._onChange();
        },

        _tableGenerator: function (rows, cols, designTime, width, height, spacing, padding, align, border, caption) {
            var tblClass = "e-rte-table", colTemplate = "<td><br _moz_dirty=''/></td>";
            if (designTime)
                tblClass += " e-rte-tableremove";
            width = ej.isNullOrUndefined(width) || width == "" ? "width='100%'" : "width=" + width + "'";
            height = ej.isNullOrUndefined(height) || height == "" ? "" : "height='" + height + "'";
            spacing = ej.isNullOrUndefined(spacing) || spacing == "" ? "" : "cellspacing='" + spacing + "'";
            padding = ej.isNullOrUndefined(padding) || padding == "" ? "" : "cellpadding='" + padding + "'";
            align = ej.isNullOrUndefined(align) ? "" : " align='" + align + "'";
            switch (border) {

                case "dotted": borderstyle = "dotted";
                    break;
                case "double":
                    this._border = "style='border:3px double black'";
                    borderstyle = "double";
                    return "<table class='" + tblClass + "' " + width + " " + height + " " + this._border + " " + spacing + " " + padding + " " + align + ">" + (caption ? "<caption></caption>" : "") + Array(rows + 1).join("<tr>" + Array(cols + 1).join("<td style='border:3px " + borderstyle + " black'><br _moz_dirty=''/></td>") + "</tr>") + "</table>";
                    break;
                case "dashed": borderstyle = "dashed";
                    break;
                case "solid": borderstyle = "solid";
                    break;
                default: borderstyle = "solid";
                    break;
            }
            if (borderstyle != "double")
            {
                this._border = "style='border:1px " + borderstyle + " black'";
                return "<table class='" + tblClass + "' " + width + " " + height + " " + this._border + " " + spacing + " " + padding + " " + align + ">" + (caption ? "<caption></caption>" : "") + Array(rows + 1).join("<tr>" + Array(cols + 1).join("<td style='border:1px " + borderstyle + " black'><br _moz_dirty=''/></td>") + "</tr>") + "</table>";
            }             
          
            return "<table class='" + tblClass + "' " + width + " " + height + " " + this._border + " " + spacing + " " + padding + " " + align + ">" + (caption ? "<caption></caption>" : "") + Array(rows + 1).join("<tr>" + Array(cols + 1).join(colTemplate) + "</tr>") + "</table>";
        },
        //custom table link click event 
        _openCustomTable: function (e) {
            if(e.currentTarget.id == this._rteId + "_createCustomTableLink" )
                 this._customTableDialog.ejDialog("open");
            else {
                if(!this._createTable.find("div#" + this._rteId + "_eTblProperties").hasClass('e-disable')) {                    
                    this._eTblManager();
                    this._eTblDialog.ejDialog("open");
                    this._on(this._getTableObj("Width"), "focusout", this._widthFocusOut);
                }
            }
        },
        _documentClick: function (e) {
            if ((!ej.isNullOrUndefined(this._createTable)) && ($(e.target).parents("#" + this._rteId + "tables").length <= 0)){
                this._createTable.ejDialog("isOpened") && this._createTable.ejDialog("close");
                this._createTable.find("div#" + this._rteId + "_eTblProperties").addClass('e-disable');
            }
            else
                $(this._createTable.find("div#" + this._rteId + "_eTblProperties").hasClass('e-disable')).addClass('e-disable');
            if(this._isIE() && !window.getSelection){
                var checkdropdown = e.target.id;
                if(checkdropdown == "")
                    checkdropdown = e.target.parentNode.id;
                if(checkdropdown == (this._rteId + "_fontSize") || checkdropdown == (this._rteId + "_fontName") || checkdropdown == (this._rteId + "_format") || checkdropdown == (this._rteId + "_fontSize_dropdown") || checkdropdown == (this._rteId + "_fontName_dropdown") || checkdropdown == (this._rteId + "_format_dropdown") )
                    this._restoreSelection(this._selectionRange);
            }
        },
        
        // iframe events 
        _iframeFocus: function (e) {
            !ej.isNullOrUndefined(this._createTable) && this._createTable.ejDialog("isOpened") && this._createTable.ejDialog("close");
            !ej.isNullOrUndefined(this._fontSizeDDL) && this._fontSizeDDL.ejDropDownList("hidePopup");
            !ej.isNullOrUndefined(this._fontStyleDDL) && this._fontStyleDDL.ejDropDownList("hidePopup");
            !ej.isNullOrUndefined(this._formatDDL) && this._formatDDL.ejDropDownList("hidePopup");
            !ej.isNullOrUndefined(this._createTable) && this._createTable.ejDialog("close");
            !ej.isNullOrUndefined(this._splitMenu) && this._splitMenu.hide(e);
            !ej.isNullOrUndefined(this._bgSplitMenu) && this._bgSplitMenu.hide(e);
        },
        _iframeKeyDown: function (e) {
            var _toolEle;
            if (this.model.showToolbar && this.model.allowKeyboardNavigation) {
                if (e && e.ctrlKey == true) {
                    /// Ctrl-b is pressed
                    if (e.keyCode == 66) {
                        e.preventDefault();
                        _toolEle = this._toolBarItems.find("#" + this._rteId + "bold");
                        if (!_toolEle.hasClass("e-disable")) {
                            this._onBold();
                            if (!_toolEle.hasClass("e-active"))
                                this._toolBarObj.selectItemByID(this._rteId + "bold");
                            else
                                this._toolBarObj.deselectItemByID(this._rteId + "bold");
                        }
                        return false;
                    }
                        /// Ctrl-i is pressed
                    else if (e.keyCode == 73) {
                        e.preventDefault();
                        _toolEle = this._toolBarItems.find("#" + this._rteId + "italic");
                        if (!_toolEle.hasClass("e-disable")) {
                            this._onItalics();
                            if (!_toolEle.hasClass("e-active"))
                                this._toolBarObj.selectItemByID(this._rteId + "italic");
                            else
                                this._toolBarObj.deselectItemByID(this._rteId + "italic");
                        }
                        return false;
                    }
                        /// Ctrl-u is pressed.
                    else if (e.keyCode == 85) {
                        e.preventDefault();
                        _toolEle = this._toolBarItems.find("#" + this._rteId + "underline");
                        if (!_toolEle.hasClass("e-disable")) {
                            this._onUnderLine();
                            if (!_toolEle.hasClass("e-active"))
                                this._toolBarObj.selectItemByID(this._rteId + "underline");
                            else
                                this._toolBarObj.deselectItemByID(this._rteId + "underline");
                        }
                        return false;
                    }
                        //justify Left Crtl-l
                    else if (e.keyCode == 76 && !e.altKey) {
                        e.preventDefault();
                        _toolEle = this._toolBarItems.find("#" + this._rteId + "justifyLeft");
                        if (!_toolEle.hasClass("e-disable")) {
                            this._onJustifyLeft();
                            this._alignToolUpdate("justifyLeft");
                        }
                        return false;
                    }
                        //justify right Crtl-r
                    else if (e.keyCode == 82) {
                        e.preventDefault();
                        _toolEle = this._toolBarItems.find("#" + this._rteId + "justifyRight");
                        if (!_toolEle.hasClass("e-disable")) {
                            this._onJustifyRight();
                            this._alignToolUpdate("justifyRight");
                        }
                        return false;
                    }
                        //justify center Crtl-e
                    else if (e.keyCode == 69 && !e.altKey) {
                        e.preventDefault();
                        _toolEle = this._toolBarItems.find("#" + this._rteId + "justifyCenter");
                        if (!_toolEle.hasClass("e-disable")) {
                            this._onJustifyCenter();
                            this._alignToolUpdate("justifyCenter");
                        }
                        return false;
                    }
                        //justify center Crtl-J
                    else if (e.keyCode == 74 && !e.altKey) {
                        e.preventDefault();
                        _toolEle = this._toolBarItems.find("#" + this._rteId + "justifyFull");
                        if (!_toolEle.hasClass("e-disable")) {
                            this._onJustifyFull();
                            this._alignToolUpdate("justifyFull");
                        }
                        return false;
                    }
                        //Undo
                    else if (e.keyCode == 90 && !e.altKey) {
                        e.preventDefault();
                        _toolEle = this._toolBarItems.find("#" + this._rteId + "undo");
                        if (!_toolEle.hasClass("e-disable")) {
                            this._onUndo();
                        }
                        return false;
                    }
                        //REDO
                    else if (e.keyCode == 89) {
                        e.preventDefault();
                        _toolEle = this._toolBarItems.find("#" + this._rteId + "rddo");
                        if (!_toolEle.hasClass("e-disable")) {
                            this._onRedo();
                        }
                        return false;
                    }
                        //COPY
                    else if (e.keyCode == 67) {
                        _toolEle = this._toolBarItems.find("#" + this._rteId + "copy");
                        if (_toolEle.hasClass("e-disable")) {
                            this._keypressFlag = false;
                            e.preventDefault();
                        }
                    }
                        //CUT
                    else if (e.keyCode == 88) {
                        _toolEle = this._toolBarItems.find("#" + this._rteId + "cut");
                        if (_toolEle.hasClass("e-disable")) {
                            this._keypressFlag = false;
                            e.preventDefault();
                        }

                    }
                        //PASTE
                    else if (e.keyCode == 86) {
                        _toolEle = this._toolBarItems.find("#" + this._rteId + "paste");
                        if (_toolEle.hasClass("e-disable")) {
                            this._keypressFlag = false;
                            e.preventDefault();
                        }
                    }
                        //INDENT
                    else if (e.keyCode == 77) {

                        if (e.shiftKey == true) {
                            e.preventDefault();
                            _toolEle = this._toolBarItems.find("#" + this._rteId + "outdent");
                            if (!_toolEle.hasClass("e-disable")) {
                                this._onOutdent();
                                if (this._indentdepth > 0)
                                    this._indentdepth--;
                            }
                            return false;
                        }
                        else {
                            e.preventDefault();
                            _toolEle = this._toolBarItems.find("#" + this._rteId + "indent");
                            if (!_toolEle.hasClass("e-disable")) {
                                this._onIndent();
                                this._indentdepth++;
                            }
                            return false;
                        }
                    }
                        //Hyperlink
                    else if (e.keyCode == 75) {
                        e.preventDefault();
                        _toolEle = this._toolBarItems.find("#" + this._rteId + "createLink");
                        if (!_toolEle.hasClass("e-disable")) {
                            this._hyperLinkManager();
                        }
                        return false;

                    }
                }

                    // Delete and Delete All
                else if (e.keyCode == 127) {
                    if ((this.getHtml() == this.getSelHtmlString()) || (this._currentIFrame.get_Text() == this._currentIFrame.getSelText())) {
                        var isClearAll = this.ClearAllManager();
                        if (!isClearAll) {
                            e.preventDefault();
                            return isClearAll;
                        }
                    }
                }
				else if(e.keyCode==8) {
                    ele = this._getSelectedNode();
                    if ($(ele).parents("BLOCKQUOTE").length>0) {
                        this._onOutdent();
                        e.preventDefault();
                    }       
				}
			}
			
            if (this.model.maxLength != null) {
                this._validateMaxLength(e);
            }
            this._trigger("keydown", { keyCode: e.keyCode });
            // if (e.keyCode != 16 || e.keyCode != 17 || e.keyCode != 18)
                // this._onChange();
        },
        //Key press event
        _iframeKeypress: function (e) {
            if (!this._keypressFlag) {
                e.preventDefault();
                this._keypressFlag = true;
                return false;
            }
        },
        _iframeKeyUp: function (e) {
            this._toggleEditTable();
            this._setBackupData();
            this._updateToolbarStatus();
            this._updateFontOptionStatus();
            this._updateFormat();
            this._updateCharCount();
            var selectEle = this._getSelectedNode();
            this._updateTagInfo(selectEle);
			this._onChange();
			this._ieSelectionRange = this._ieSelection;
            this._trigger("keyup", { keyCode: e.keyCode });
        },
        _iframeMouseUp: function (e) {
            this._toggleEditTable();
            this._updateToolbarStatus();
            this._updateFontOptionStatus();
            this._updateFormat();
            this._ieSelectionRange = this._ieSelection;
        },
        _iframeMouseDown: function (e) {
            this._updateTagInfo(e.target);
            this._updateIndent(e.target);
            this._currentSelNode = e.target; //For IE
        },
        _iframeFocusOut: function (e) {
            this._onChange();
            this._selectionRange = this._saveSelection();
            this._seleText = ej.isNullOrUndefined(this._getWindow().document.selection) ? this._selectionRange .toString(): this._getWindow().document.selection.createRange().text;
        },
        _widthFocusOut:function(e) {
            var dialog, drpdwnId, widthId;
            if(e.currentTarget.id === this._rteId+ "_txtWidth"){
                dialog = this._customTableDialog;
                drpdwnId = "#" + this._rteId + "_ddlAlignment";
                widthId = "#" + this._rteId + "_txtWidth";
            }
            else {
                dialog = this._eTblDialog;
                drpdwnId = "#" + this._rteId + "_eTblAlign";
                widthId = "#" + this._rteId + "_eTblWidth";
            }
            var val = dialog.find(widthId).val();
            var obj = $(drpdwnId).data('ejDropDownList');
            var option = ( val && !isNaN(val) || (val.slice(-2) =='px') || (val.slice(-2) =='em') || (val.slice(-2) =='pt') || (val.slice(-1)=='%') ) ? true : false ;
            obj.option('enabled', option);
        },
        //-------------------- Event Wire-up -------------------------//
        _wireEvents: function () {
            var _iframe = $(this._getDocument());
            if (!ej.isNullOrUndefined(this._createTable)) {
                this._on(this._createTable, "mouseenter", "div.e-rte-tablecell", this._tableCellStart);
                this._on(this._createTable, "mousemove", "div.e-rte-tablecell", this._tableCellSelect);
                this._on(this._createTable, "mouseleave", "div.e-rte-table", this._tableCellLeave);
                this._on(this._createTable, "mousedown", "div.e-rte-tablecell", this._tableCellDown);
                this._on($(document), "click", this._documentClick);
            }
            this._on(_iframe, "focus", this._iframeFocus);
            this._on(_iframe, "keydown", this._iframeKeyDown);
            this._on(_iframe, "keypress", this._iframeKeypress);
            this._on(_iframe, "keyup", this._iframeKeyUp);
            this._on(_iframe, "mouseup", this._iframeMouseUp);
            this._on(_iframe, "mousedown", this._iframeMouseDown);
            this._on(_iframe, "focusout", this._iframeFocusOut);
            if(!ej.isNullOrUndefined(this._customTableDialog))
            this._on(this._customTableDialog.find("#" + this._rteId + "_txtWidth"), "focusout", this._widthFocusOut);
            
        },

        _unwireEvents: function () {
            var _iframe = $(this._getDocument());
            if (!ej.isNullOrUndefined(this._createTable)) {
                this._off(this._createTable.find("div.e-rte-tablecell"), "mousemove");
                this._off(this._createTable.find("div.e-rte-table"), "mouseleave");
                this._off(this._createTable.find("div.e-rte-tablecell"), "mousedown");
                this._off($(document), "click");
            }
            this._off(_iframe, "focus");
            this._off(_iframe, "keydown");
            this._off(_iframe, "keyup");
            this._off(_iframe, "mouseup");
            this._off(_iframe, "mousedown");
            this._off(_iframe, "blur");
            if (!ej.isNullOrUndefined(this._customTableDialog))
            this._off(this._customTableDialog.find("#" + this._rteId + "_txtWidth"), "focusout");
        },

        /*----------------------- Event handler------------------------------------------*/
        _onChange: function () {
		_prevhtml = $.trim(this.value());
		if( _prevhtml!== this.getHtml()){
            this._updateValue();
            var args = { text: this._getText(), htmlText: this.getHtml() };
            this._trigger("change", args);
			}
        },

        /*----------------------- Public Methods-----------------------------------------*/

        //Disables toolbar item
        /**
        * disable the RTE toolbar item.
        * @example 
        * &lt;textarea   id="rteSample"&gt; 	   
        * &lt;p&gt;&lt;b&gt;Description:&lt;/b&gt;&lt;/p&gt;
	    * &lt;p&gt;The Rich Text Editor (RTE) control is an easy to render in
	    client side. Customer easy to edit the contents and get the HTML content for
	    the displayed content. A rich text editor control provides users with a toolbar
	    that helps them to apply rich text formats to the text entered in the text
	    area. &lt;/p&gt;&lt;/textarea &gt;
        * &lt;script&gt;
        * $("#rteSample").ejRTE();
        * // Create RTE
        * var rteeObj  = $("#rteSample").data("ejRTE");
        * rteeObj.disableToolbarItem("rteSamplecreateTable"); // disable toolbar item
        * &lt;/script&gt;
		* @example 
        * &lt;textarea   id="rteSample"&gt; 	   
        * &lt;p&gt;&lt;b&gt;Description:&lt;/b&gt;&lt;/p&gt;
	    * &lt;p&gt;The Rich Text Editor (RTE) control is an easy to render in
	    client side. Customer easy to edit the contents and get the HTML content for
	    the displayed content. A rich text editor control provides users with a toolbar
	    that helps them to apply rich text formats to the text entered in the text
	    area. &lt;/p&gt;&lt;/textarea &gt;
        * &lt;script&gt;
        * $("#rteSample").ejRTE();
        * // Create RTE
        * $("#rteSample").ejRTE("disableToolbarItem","rteSamplecreateTable");// disable toolbar item        
        * &lt;/script&gt;
        *@memberof ejRTE
        * @instance
        */
        disableToolbarItem: function (id) {
            !ej.isNullOrUndefined(id) && this._toolBarObj.disableItemByID(id);
        },
        /**
        * enable  the RTE toolbar item.
        * @example 
        * &lt;textarea   id="rteSample"&gt; 	            
        *      &lt;p&gt;&lt;b&gt;Description:&lt;/b&gt;&lt;/p&gt;
        &lt;p&gt;The Rich Text Editor (RTE) control is an easy to render in
        client side. Customer easy to edit the contents and get the HTML content for
        the displayed content. A rich text editor control provides users with a toolbar
        that helps them to apply rich text formats to the text entered in the text
        area. &lt;/p&gt;&lt;/textarea &gt;
        * &lt;script&gt;
         * $("#rteSample").ejRTE();
        * // Create RTE
        * var rteeObj  = $("#rteSample").data("ejRTE");
        * rteeObj.enableToolbarItem("rteSamplecreateTable"); // enable toolbar item
        * &lt;/script&gt;
		* @example 
        * &lt;textarea   id="rteSample"&gt; 	   
        * &lt;p&gt;&lt;b&gt;Description:&lt;/b&gt;&lt;/p&gt;
	    * &lt;p&gt;The Rich Text Editor (RTE) control is an easy to render in
	    client side. Customer easy to edit the contents and get the HTML content for
	    the displayed content. A rich text editor control provides users with a toolbar
	    that helps them to apply rich text formats to the text entered in the text
	    area. &lt;/p&gt;&lt;/textarea &gt;
        * &lt;script&gt;
        * $("#rteSample").ejRTE();
        * // Create RTE
        * $("#rteSample").ejRTE("enableToolbarItem","rteSamplecreateTable");// enable toolbar item        
        * &lt;/script&gt;
        *@memberof ejRTE
        * @instance
        */
        //To enable toolbar item
        enableToolbarItem: function (id) {
            !ej.isNullOrUndefined(id) && this._toolBarObj.enableItemByID(id);
        },
        /**
        * removes  the RTE toolbar item.
        * @example 
        * &lt;textarea   id="rteSample"&gt; 	          
        *        &lt;p&gt;&lt;b&gt;Description:&lt;/b&gt;&lt;/p&gt;
        &lt;p&gt;The Rich Text Editor (RTE) control is an easy to render in
        client side. Customer easy to edit the contents and get the HTML content for
        the displayed content. A rich text editor control provides users with a toolbar
        that helps them to apply rich text formats to the text entered in the text
        area. &lt;/p&gt;&lt;/textarea &gt;
        * &lt;script&gt;
      * $("#rteSample").ejRTE();
        * // Create RTE
        * var rteeObj  = $("#rteSample").data("ejRTE");
        * rteeObj.removeToolbarItem("rteSamplecreateTable"); // remove toolbar item
        * &lt;/script&gt;
		* @example 
        * &lt;textarea   id="rteSample"&gt; 	   
        * &lt;p&gt;&lt;b&gt;Description:&lt;/b&gt;&lt;/p&gt;
	    * &lt;p&gt;The Rich Text Editor (RTE) control is an easy to render in
	    client side. Customer easy to edit the contents and get the HTML content for
	    the displayed content. A rich text editor control provides users with a toolbar
	    that helps them to apply rich text formats to the text entered in the text
	    area. &lt;/p&gt;&lt;/textarea &gt;
        * &lt;script&gt;
        * $("#rteSample").ejRTE();
        * // Create RTE
        * $("#rteSample").ejRTE("removeToolbarItem","rteSamplecreateTable");// remove toolbar item        
        * &lt;/script&gt;
        *@memberof ejRTE
        * @instance
        */
        //Removes toolbar item
        removeToolbarItem: function (id) {
            !ej.isNullOrUndefined(id) && this._toolBarObj.removeItemByID(id);
        },
        /**
        * Displays the RTE control.
        * @example 
        * &lt;textarea   id="rteSample"&gt; 	             
        *     &lt;p&gt;&lt;b&gt;Description:&lt;/b&gt;&lt;/p&gt;
        &lt;p&gt;The Rich Text Editor (RTE) control is an easy to render in
        client side. Customer easy to edit the contents and get the HTML content for
        the displayed content. A rich text editor control provides users with a toolbar
        that helps them to apply rich text formats to the text entered in the text
        area. &lt;/p&gt;&lt;/textarea &gt;
        * &lt;script&gt;
         * $("#rteSample").ejRTE();
        * // Create RTE
        * var rteeObj  = $("#rteSample").data("ejRTE");
        * rteeObj.show(); // shows rte
        * &lt;/script&gt;
		* @example 
        * &lt;textarea   id="rteSample"&gt; 	   
        * &lt;p&gt;&lt;b&gt;Description:&lt;/b&gt;&lt;/p&gt;
	    * &lt;p&gt;The Rich Text Editor (RTE) control is an easy to render in
	    client side. Customer easy to edit the contents and get the HTML content for
	    the displayed content. A rich text editor control provides users with a toolbar
	    that helps them to apply rich text formats to the text entered in the text
	    area. &lt;/p&gt;&lt;/textarea &gt;
        * &lt;script&gt;
        * $("#rteSample").ejRTE();
        * // Create RTE
        * $("#rteSample").ejRTE("show");// shows rte       
        * &lt;/script&gt;
        *@memberof ejRTE
        * @instance
        */
        //Displays the control
        show: function () {
            this._rteWapper.show();
        },
        /**
        * Hides the RTE control.
        * @example 
        * &lt;textarea   id="rteSample"&gt; 	                 
        * &lt;p&gt;&lt;b&gt;Description:&lt;/b&gt;&lt;/p&gt;
        &lt;p&gt;The Rich Text Editor (RTE) control is an easy to render in
        client side. Customer easy to edit the contents and get the HTML content for
        the displayed content. A rich text editor control provides users with a toolbar
        that helps them to apply rich text formats to the text entered in the text
        area. &lt;/p&gt;&lt;/textarea &gt;
        * &lt;script&gt;
       * $("#rteSample").ejRTE();
        * // Create RTE
        * var rteeObj  = $("#rteSample").data("ejRTE");
        * rteeObj.hide(); // hides rte
        * &lt;/script&gt;
		* @example 
        * &lt;textarea   id="rteSample"&gt; 	   
        * &lt;p&gt;&lt;b&gt;Description:&lt;/b&gt;&lt;/p&gt;
	    * &lt;p&gt;The Rich Text Editor (RTE) control is an easy to render in
	    client side. Customer easy to edit the contents and get the HTML content for
	    the displayed content. A rich text editor control provides users with a toolbar
	    that helps them to apply rich text formats to the text entered in the text
	    area. &lt;/p&gt;&lt;/textarea &gt;
        * &lt;script&gt;
        * $("#rteSample").ejRTE();
        * // Create RTE
        * $("#rteSample").ejRTE("hide");// hides rte       
        * &lt;/script&gt;
        *@memberof ejRTE
        * @instance
        */
        //Hides the control
        hide: function () {
            this._rteWapper.hide();
        },
        /**
        * gets the HTML string from the RTE control.
        * @example 
        * &lt;textarea   id="rteSample"&gt; 	                
        *  &lt;p&gt;&lt;b&gt;Description:&lt;/b&gt;&lt;/p&gt;
        &lt;p&gt;The Rich Text Editor (RTE) control is an easy to render in
        client side. Customer easy to edit the contents and get the HTML content for
        the displayed content. A rich text editor control provides users with a toolbar
        that helps them to apply rich text formats to the text entered in the text
        area. &lt;/p&gt;&lt;/textarea &gt;
        * &lt;script&gt;
        * $("#rteSample").ejRTE();
        * // Create RTE
        * var rteeObj  = $("#rteSample").data("ejRTE");
        * rteeObj.getHtml(); // gets the html string from rte
        * &lt;/script&gt;
		* @example 
        * &lt;textarea   id="rteSample"&gt; 	   
        * &lt;p&gt;&lt;b&gt;Description:&lt;/b&gt;&lt;/p&gt;
	    * &lt;p&gt;The Rich Text Editor (RTE) control is an easy to render in
	    client side. Customer easy to edit the contents and get the HTML content for
	    the displayed content. A rich text editor control provides users with a toolbar
	    that helps them to apply rich text formats to the text entered in the text
	    area. &lt;/p&gt;&lt;/textarea &gt;
        * &lt;script&gt;
        * $("#rteSample").ejRTE();
        * // Create RTE
        * $("#rteSample").ejRTE("getHtml");// getHtml from rte        
        * &lt;/script&gt;
        *@memberof ejRTE
        * @instance
        */
        getHtml: function () {
            return $.trim(this._getDocument().body.innerHTML);
        },
        /**
        * sets the HTML string from the RTE control.
        * @example 
        * &lt;textarea   id="rteSample"&gt; 	              
        *   &lt;/textarea &gt;
        * &lt;script&gt;
         * $("#rteSample").ejRTE();
        * // Create RTE
        * var rteeObj  = $("#rteSample").data("ejRTE");
        * rteeObj.setHtml("The Rich Text Editor (RTE) control is an easy to render in client side."); // sets the html string to rte
        * &lt;/script&gt;
		* @example 
        * &lt;textarea   id="rteSample"&gt; 	   
        * &lt;p&gt;&lt;b&gt;Description:&lt;/b&gt;&lt;/p&gt;
	    * &lt;p&gt;The Rich Text Editor (RTE) control is an easy to render in
	    client side. Customer easy to edit the contents and get the HTML content for
	    the displayed content. A rich text editor control provides users with a toolbar
	    that helps them to apply rich text formats to the text entered in the text
	    area. &lt;/p&gt;&lt;/textarea &gt;
        * &lt;script&gt;
        * $("#rteSample").ejRTE();
        * // Create RTE
        * $("#rteSample").ejRTE("setHtml","The Rich Text Editor (RTE) control is an easy to render in client side.");// sets the html string to rte    
	       
        * &lt;/script&gt;
        *@memberof ejRTE
        * @instance
        */
        //Set the 
        setHtml: function (value) {
            this._getDocument().body.innerHTML = value;
        },
        /**
        * gets the content as string from the RTE control.
        * @example 
        * &lt;textarea   id="rteSample"&gt; 	               
        *   &lt;p&gt;&lt;b&gt;Description:&lt;/b&gt;&lt;/p&gt;
        &lt;p&gt;The Rich Text Editor (RTE) control is an easy to render in
        client side. Customer easy to edit the contents and get the HTML content for
        the displayed content. A rich text editor control provides users with a toolbar
        that helps them to apply rich text formats to the text entered in the text
        area. &lt;/p&gt;&lt;/textarea &gt;
        * &lt;script&gt;
       * $("#rteSample").ejRTE();
        * // Create RTE
        * var rteeObj  = $("#rteSample").data("ejRTE");
        * rteeObj.getText(); // gets the content as string from rte
        * &lt;/script&gt;
		* @example 
        * &lt;textarea   id="rteSample"&gt; 	   
        * &lt;p&gt;&lt;b&gt;Description:&lt;/b&gt;&lt;/p&gt;
	    * &lt;p&gt;The Rich Text Editor (RTE) control is an easy to render in
	    client side. Customer easy to edit the contents and get the HTML content for
	    the displayed content. A rich text editor control provides users with a toolbar
	    that helps them to apply rich text formats to the text entered in the text
	    area. &lt;/p&gt;&lt;/textarea &gt;
        * &lt;script&gt;
        * $("#rteSample").ejRTE();
        * // Create RTE
        * $("#rteSample").ejRTE("getText");// getText from rte        
        * &lt;/script&gt;
        *@memberof ejRTE
        * @instance
        */
        //Get plan text
        getText: function () {
            return this._getText();
        },
        /**
        * gets the content as string from the RTE control.
        * @example 
        * &lt;textarea   id="rteSample"&gt; 	             
        *     &lt;p&gt;&lt;b&gt;Description:&lt;/b&gt;&lt;/p&gt;
        &lt;p&gt;The Rich Text Editor (RTE) control is an easy to render in
        client side. Customer easy to edit the contents and get the HTML content for
        the displayed content. A rich text editor control provides users with a toolbar
        that helps them to apply rich text formats to the text entered in the text
        area. &lt;/p&gt;&lt;/textarea &gt;
        * &lt;script&gt;
        * $("#rteSample").ejRTE();
        * // Create RTE
        * var rteeObj  = $("#rteSample").data("ejRTE");
        * rteeObj.executeCommand("bold", true); // gets the content as string from rte
        * &lt;/script&gt;
        *@memberof ejRTE
        * @instance
        */
        //Execute Command
        executeCommand: function (cmdName, args) {
            if (this._trigger("execute", { commandName: cmdName }))
                return false;
            var anchor = "";
            if (cmdName == "underline") {
                if (window.getSelection) {
                    anchor = $(this._getWindow().getSelection().anchorNode).parents("a");
                    if (anchor.length > 0) {
                        $(anchor).css("text-decoration", ($(anchor).css("text-decoration") == "none") ? "underline" : "none");
                        return;
                    }
                }
            }
            else if (cmdName.toLowerCase() == "inserthtml") {
                if (this._isIE()) {
                    this._pasteHtml(args);
                }
                else
                    this._getDocument().execCommand(cmdName, false, args);
                return;
            }
            this._focus();
            if (this._isIE() && cmdName == "fontName")
                args = args.substring(0, args.indexOf(","));
            this._getDocument().execCommand(cmdName, false, args);
            this._setBackupData();
        },
        /**
        * focus the RTE control.
        * @example 
        * &lt;textarea   id="rteSample"&gt; 	               
        *   &lt;p&gt;&lt;b&gt;Description:&lt;/b&gt;&lt;/p&gt;
        &lt;p&gt;The Rich Text Editor (RTE) control is an easy to render in
        client side. Customer easy to edit the contents and get the HTML content for
        the displayed content. A rich text editor control provides users with a toolbar
        that helps them to apply rich text formats to the text entered in the text
        area. &lt;/p&gt;&lt;/textarea &gt;
        * &lt;script&gt;
         * $("#rteSample").ejRTE();
        * // Create RTE
        * var rteeObj  = $("#rteSample").data("ejRTE");
        * rteeObj.focus(); // gets the content as string from rte
        * &lt;/script&gt;
		* @example 
        * &lt;textarea   id="rteSample"&gt; 	   
        * &lt;p&gt;&lt;b&gt;Description:&lt;/b&gt;&lt;/p&gt;
	    * &lt;p&gt;The Rich Text Editor (RTE) control is an easy to render in
	    client side. Customer easy to edit the contents and get the HTML content for
	    the displayed content. A rich text editor control provides users with a toolbar
	    that helps them to apply rich text formats to the text entered in the text
	    area. &lt;/p&gt;&lt;/textarea &gt;
        * &lt;script&gt;
        * $("#rteSample").ejRTE();
        * // Create RTE
        * $("#rteSample").ejRTE("focus");// enable toolbar item        
        * &lt;/script&gt;
        *@memberof ejRTE
        * @instance
        */
        //focus the editable area.
        focus: function () {
            this._focus();
        },
        /**
        * gets the command status of the given RTE control.
        * @example 
        * &lt;textarea   id="rteSample"&gt;                   
        *  &lt;p&gt;&lt;b&gt;Description:&lt;/b&gt;&lt;/p&gt;
        &lt;p&gt;The Rich Text Editor (RTE) control is an easy to render in
        client side. Customer easy to edit the contents and get the HTML content for
        the displayed content. A rich text editor control provides users with a toolbar
        that helps them to apply rich text formats to the text entered in the text
        area. &lt;/p&gt;&lt;/textarea &gt;
        * &lt;script&gt;
         * $("#rteSample").ejRTE();
        * // Create RTE
        * var rteeObj  = $("#rteSample").data("ejRTE");
        * rteeObj.getCommandStatus(arguments); // gets the content as string from rte
        * &lt;/script&gt;
        *@memberof ejRTE
        * @instance
        */
        //get the command status
        getCommandStatus: function (value) {
            if (ej.isNullOrUndefined(value) || value != "")
                return this._getCommandStatus(value);
        },
        /**
        * gets the command status of the given RTE control.
        * @example 
        * &lt;textarea   id="rteSample"&gt;     
    * &lt;p&gt;&lt;b&gt;Description:&lt;/b&gt;&lt;/p&gt;
        &lt;p&gt;The Rich Text Editor (RTE) control is an easy to render in
        client side. Customer easy to edit the contents and get the HTML content for
        the displayed content. A rich text editor control provides users with a toolbar
        that helps them to apply rich text formats to the text entered in the text
        area. &lt;/p&gt;&lt;/textarea &gt;     
    * &lt;script&gt;
    * $("#rteSample").ejRTE();
        * // Create RTE
        * var rteeObj  = $("#rteSample").data("ejRTE");
        * rteeObj.getSelectedHtml(); // gets the content as string from rte
        * &lt;/script&gt;
		* @example 
        * &lt;textarea   id="rteSample"&gt; 	   
        * &lt;p&gt;&lt;b&gt;Description:&lt;/b&gt;&lt;/p&gt;
	    * &lt;p&gt;The Rich Text Editor (RTE) control is an easy to render in
	    client side. Customer easy to edit the contents and get the HTML content for
	    the displayed content. A rich text editor control provides users with a toolbar
	    that helps them to apply rich text formats to the text entered in the text
	    area. &lt;/p&gt;&lt;/textarea &gt;
        * &lt;script&gt;
        * $("#rteSample").ejRTE();
        * // Create RTE
        * $("#rteSample").ejRTE("getSelectedHtml");// getSelectedHtml from rte        
        * &lt;/script&gt;
        *@memberof ejRTE
        * @instance
        */
        //currently selected text html
        getSelectedHtml: function () {
            return this._getSelectedHtmlString();
        },
        
        //Tables methods
        //insert table Rows
        insertRow: function (before, cell) {
            //$("#rte1_Iframe").contents().find("#test").parents("tr").find("td").length
            var seletedCell = ej.isNullOrUndefined(cell) ? this._getSelectedNode() : cell;
            var colTemplate = "<td " + this._border + "contentEditable='true'><br _moz_dirty=''/></td>", curRow = $(seletedCell).closest("tr");
            var cols = curRow.find("td").length;
            var newRow = $("<tr>" + Array(cols + 1).join(colTemplate) + "</tr>");
            before ? newRow.insertBefore(curRow) : newRow.insertAfter(curRow);
            curRow.find("td:first").focus();
        },
        
        //Insert table Columns
        insertColumn: function (before, cell) {
            var seletedCell = ej.isNullOrUndefined(cell) ? this._getSelectedNode() : cell;
            var colTemplate = "<td " + this._border + "contentEditable='true'><br _moz_dirty=''/></td>", curRow = $(seletedCell).closest("tr"), curCell;
            var allRows = curRow.closest("table").find("tr");
            var colIndex = curRow.find("td").index(seletedCell);
            for (var i = 0; i < allRows.length; i++) {
                curCell = $(allRows[i]).find("td")[colIndex];
                before ? $(colTemplate).insertBefore(curCell) : $(colTemplate).insertAfter(curCell);
            }
        },
       
        //Remove table Row
        removeRow: function (cell) {
            var seletedCell = ej.isNullOrUndefined(cell) ? this._getSelectedNode() : cell;
            curRow = $(seletedCell).closest("tr").remove();
        },
      
        //Remove table Column
        removeColumn: function (cell) {
            var seletedCell = ej.isNullOrUndefined(cell) ? this._getSelectedNode() : cell;
            var curRow = $(seletedCell).closest("tr");
            var allRows = curRow.closest("table").find("tr");
            var colIndex = curRow.find("td").index(seletedCell);
            for (var i = 0; i < allRows.length; i++) {
                $($(allRows[i]).find("td")[colIndex]).remove();
            }
        },
        
        //Remove table 
        removeTable: function (table) {
            var seletedTable = ej.isNullOrUndefined(table) ? $(this._getSelectedNode()).closest("table") : table;
            $(seletedTable).remove();
        }
    });

    ej.RTE.Locale = {};

    ej.RTE.Locale["en-US"] = {
        bold: "Bold",
        italic: "Italic",
        underline: "Underline",
        strikethrough: "Strikethrough",
        superscript: "Superscript",
        subscript: "Subscript",
        justifyCenter: "Align text center",
        justifyLeft: "Align text left",
        justifyRight: "Align text right",
        justifyFull: "Justify",
        unorderedList: "Insert unordered list",
        orderedList: "Insert ordered list",
        indent: "Indent",
        outdent: "Outdent",
        cut: "Cut",
        copy: "Copy",
        paste: "Paste",
        paragraph: "Paragraph",
        undo: "Undo",
        redo: "Redo",
        upperCase: "Upper Case",
        lowerCase: "Lower Case",
        clearAll: "Clear All",
        clearFormat: "Clear Format",
        createLink: "Insert/Edit hyperlink",
        image: "Insert image",
        video: "Insert video",
        editTable: "Edit Table Properties",
        embedVideo: "Paste your embed code below",
        viewHtml: "View HTML",
        fontName: "Select font family",
        fontSize: "Select font size",
        fontColor: "Select color",
        format: "Format",
        backgroundColor: "Background color",
        style: "Styles",
        deleteAlert: "Are you sure you want to clear all the contents?",
        copyPastAlert: "Your browser doesn't support direct access to the clipboard. Please use the Ctrl+X/C/V keyboard shortcuts instead.",
        videoError: "The text area can not be empty",
        imageWebUrl: "Web Address",
        imageAltText: "Image description",
        dimensions: "Dimensions",
        constrainProportions: "Constrain Proportions",
        linkWebUrl: "Web Address",
        imageLink: "Image as Link",
        imageBorder: "Image Border",
        imageStyle: "Style",
        linkText: "Text",
        linkToolTip: "ToolTip",
        linkOpenInNewWindow: "Open link in new window",
        tableColumns: "No.of Columns",
        tableRows: "No.of Rows",
        tableWidth: "Width",
        tableHeight: "Height",
        tableCellSpacing: "Cell spacing",
        tableCellPadding: "Cell padding",
        tableBorder: "Border",
        tableCaption: "Caption",
        tableAlignment: "Alignment",
        textAlign: "Text align",
        dialogUpdate: "Update",
        dialogInsert: "Insert",
        dialogCancel: "Cancel",
        dialogApply: "Apply",
        dialogOk: "Ok",
        createTable: "Create table",
        addColumnLeft: "Add column on the left",
        addColumnRight: "Add column on the right",
        addRowAbove: "Add row above",
        addRowBelow: "Add row below",
        deleteRow: "Delete the row",
        deleteColumn: "Delete the column",
        deleteTable: "Delete the table",
        customTable: "Create custom table...",
        characters: "Characters",
        general: "General",
        advanced: "Advanced",
        table: "Table",
        row: "Row",
        column: "Column",
        cell: "Cell"
    };

})(jQuery, Syncfusion);;
/**
* @fileOverview Plugin provides support to display color picker within your web page and allows to pick the color.
* @copyright Copyright Syncfusion Inc. 2001 - 2014. All rights reserved.
*  Use of this code is subject to the terms of our license.
*  A copy of the current license can be obtained at any time by e-mailing
*  licensing@syncfusion.com. Any infringement will be prosecuted under
*  applicable laws. 
* @version 12.3 
* @author <a href="mailto:licensing@syncfusion.com">Syncfusion Inc</a>
*/
(function ($, ej, undefined) {
    /**
  * @namespace ej
  * @classdesc Custom Design for Html color picker control.
  * @class ejColorPicker
  * @param {object} options - settings for color picker
  * @requires jQuery
  * @requires jquery.easing.1.3.js
  * @requires ej.core.js
  * @requires ej.colorpicker.js
  * @requires ej.button.js
  * @requires ej.splitbutton.js
  * @requires ej.menu.js
  * @requires ej.slider.js
  * @example   
  * &lt;input type="text" id="colorPick"/&gt; <br> 
  * &lt;script&gt;
  * // Create Color picker
  * $('#colorPick').ejColorPicker({ value: "#278787" });
  * &lt;/script&gt;
  */
    ej.widget("ejColorPicker", "ej.ColorPicker", {
        _rootCSS: "e-colorpicker",
        // widget element will be automatically set in this
        element: null,

        // user defined model will be automatically set in this
        model: null,
        validTags: ["input", "div"],
        _setFirst: false,

        // default model
        defaults: {

            //colorpicker  
            /**		
			* Enable or Disable the opacity for color picker
			* @default true
			* @type {boolean}
			* @example   
            * &lt;input type="text" id="colorPick"/&gt; <br> 
            * &lt;script&gt;
            * //To set enableOpacity API during initialization  
            * $('#colorPick').ejColorPicker({ value: "#278787", enableOpacity: false });
            * &lt;/script&gt;
			* @memberof ejColorPicker
			* @instance
			*/
            enableOpacity: true,
            /**		
           * Specifies the opacity value for color picker
           * @default 100
           * @type {number}
           * @example   
           * &lt;input type="text" id="colorPick"/&gt; <br> 
           * &lt;script&gt;
           * //To set opacityValue API during initialization  
           * $('#colorPick').ejColorPicker({ value: "#278787", opacityValue: 20 });
           * &lt;/script&gt;
           * @memberof ejColorPicker
           * @instance
           */
            opacityValue: 100,

            //colorpalette
            /**		
           * Specifies the number of columns in color palette
           * @default 10
           * @type {number}
           * @example   
           * &lt;input type="text" id="colorPick"/&gt; <br> 
           * &lt;script&gt;
           * //To set columns API during initialization  
           * $('#colorPick').ejColorPicker({ value: "#278787", modelType: "palette", columns: 5});
           * &lt;/script&gt;
           * @memberof ejColorPicker
           * @instance
           */
            columns: 10,
            //colorpalette
            /**		
           * Specifies the palette type, these types are basicPalette and customPalette See {@link Palette}
           * @default  ej.Palette.BasicPalette
           * @type {enum}
           * @example   
           * &lt;input type="text" id="colorPick"/&gt; <br> 
           * &lt;script&gt;
           * //To set palette API during initialization 
           * $('#colorPick').ejColorPicker({ value: "#278787", modelType: "palette", palette: "basicPalette"});
           * &lt;/script&gt;
           * @memberof ejColorPicker
           * @instance
           */
            palette: "basicPalette",
         
             /**		
            * Specifies the custom colors for customPalette
            * @default  empty
            * @type {data}
            * @example   
            * &lt;input type="text" id="colorPick"/&gt; <br> 
            * &lt;script&gt;
            * //To set custom API during initialization
            * $('#colorPick').ejColorPicker({ value: "#278787", modelType: "palette", palette: "customPalette", custom: ["ffffff", "ffccff", "ff99ff", "ff66ff", "ff33ff", "ff00ff", "ccffff", "ccccff"]});
            * &lt;/script&gt;
            * @memberof ejColorPicker
            * @instance
            */
            custom: [],

            //common API's
            /**		
            * Specifies the model type, these types are default, picker and palette  See {@link ModelType} 
            * @default  default
            * @type {enum}
            * @example   
            * &lt;input type="text" id="colorPick"/&gt; <br> 
            * &lt;script&gt;
            * //To set modelType API during initialization
            * $('#colorPick').ejColorPicker({ value: "#278787", modelType: "palette"});
            * &lt;/script&gt;
            * @memberof ejColorPicker
            * @instance
            */
            modelType: "default",
            /**		
            * Enable or disables the preview of old color and current color 
            * @default  true
            * @type {boolean}
            * @example   
            * &lt;input type="text" id="colorPick"/&gt; <br> 
            * &lt;script&gt;
            * //To set showPreview API during initialization
            * $('#colorPick').ejColorPicker({ value: "#278787", showPreview: false});
            * &lt;/script&gt;
            * @memberof ejColorPicker
            * @instance
            */
            showPreview: true,
            /**		
            * Specifies the selected color value, using this we can set the color value 
            * @default  null
            * @type {string}
            * @example   
            * &lt;input type="text" id="colorPick"/&gt; <br> 
            * &lt;script&gt;
            * //To set value API during initialization
            * $('#colorPick').ejColorPicker({ value: "#278787"});
            * &lt;/script&gt;
            * @memberof ejColorPicker
            * @instance
            */
            value: null,
            /**		
            * Specifies the display in inline format
            * @default  false
            * @type {boolean}
            * @example   
            * &lt;input type="text" id="colorPick"/&gt; <br> 
            * &lt;script&gt;
            * //To set displayInline API during initialization
            * $('#colorPick').ejColorPicker({ value: "#278787",  displayInline: true});
            * &lt;/script&gt;
            * @memberof ejColorPicker
            * @instance
            */
            displayInline: false,
            /**		
            * Specifies the text, which is show on button
            * @default buttonText.apply= "Apply",  buttonText.cancel= "Cancel"
            * @type {string}
            * @example   
            * &lt;input type="text" id="colorPick"/&gt; <br> 
            * &lt;script&gt;
            * //To set buttonText API during initialization
            * $('#colorPick').ejColorPicker({ value: "#278787",   buttonText: {apply: "Set", cancel: "Close" }});
            * &lt;/script&gt;
            * @memberof ejColorPicker
            * @instance
            */
            buttonText: {
                apply: "Apply",
                cancel: "Cancel"
            },
            /**		
            * Specifies the tooltip text content
            * @default 
            * tooltipText: {
              *  switcher: "Switcher",
               * addButton: "Add Color",
               * basic: "Basic",
               * monoChrome: "Mono Chrome",
               * flatColors: "Flat Color",
               * seaWolf: "Sea Wolf",
               * webColors: "Web Colors",
               * sandy: "Sandy",
               * pinkShades: "Pink Shades",
               * misty: "Misty",
               * citrus: "Citrus",
               * vintage: "Vintage",
               *  moonLight: "Moon Light",
               * candyCrush: "Candy Crush",
               * currentColor: "Current Color",
               * selectedColor: "Selected Color"
            * }
            * @type {object}
            * @example   
            * &lt;input type="text" id="colorPick"/&gt; <br> 
            * &lt;script&gt;
            * //To set tooltipText API during initialization
            * $('#colorPick').ejColorPicker({ value: "#278787",   tooltipText: { switcher: "Switch",  currentColor: "New Color", selectedColor: "Old Color" }});
            * &lt;/script&gt;
            * @memberof ejColorPicker
            * @instance
            */
            tooltipText: {
                switcher: "Switcher",
                addButton: "Add Color",
                basic: "Basic",
                monoChrome: "Mono Chrome",
                flatColors: "Flat Color",
                seaWolf: "Sea Wolf",
                webColors: "Web Colors",
                sandy: "Sandy",
                pinkShades: "Pink Shades",
                misty: "Misty",
                citrus: "Citrus",
                vintage: "Vintage",
                moonLight: "Moon Light",
                candyCrush: "Candy Crush",
                currentColor: "Current Color",
                selectedColor: "Selected Color"
            },
            /**		
            * Specifies the selected Recent colors
            * @default false
            * @type {boolean}
            * @example   
            * &lt;input type="text" id="colorPick"/&gt; <br> 
            * &lt;script&gt;
            * //To set showRecentColors API during initialization
            * $('#colorPick').ejColorPicker({ value: "#278787",   showRecentColors: true});
            * &lt;/script&gt;
            * @memberof ejColorPicker
            * @instance
            */
            showRecentColors: false,
            /**		
            * Specifies the toolIcon
            * @default null
            * @type {string}
            * @example   
            * &lt;input type="text" id="colorPick"/&gt; <br> 
            * &lt;script&gt;
            * //To set toolIcon API during initialization
            * $('#colorPick').ejColorPicker({ value: "#278787",  toolIcon: "e-fontcolor-icon"});
            * &lt;/script&gt;
            * &lt;style&gt;
            * .e-colorwidget .e-tool .e-fontcolor-icon:before
            * {
            * content: "\e632";
            * margin-top: 9px;
            * font-size: 10px;
            * margin-left: 5px;
            * }
            * &lt;/style&gt;
            * @memberof ejColorPicker
            * @instance
            */
            toolIcon: null,
            /**		
            * Specifies the cssClass for color picker to achieve custom theme
            * @default emptyString
            * @type {string}
            * @example   
            * &lt;input type="text" id="colorPick"/&gt; <br> 
            * &lt;script&gt;
            * //To set cssClass API during initialization
            * $('#colorPick').ejColorPicker({ value: "#278787",  cssClass : "gradient-lime"});
            * &lt;/script&gt;
            * @memberof ejColorPicker
            * @instance
            */
            cssClass: "",
            /**		
            * Specifies the color picker control state.
            * @default true
            * @type {boolean}
            * @example   
            * &lt;input type="text" id="colorPick"/&gt; <br> 
            * &lt;script&gt;
            * //To set enabled API during initialization
            * $('#colorPick').ejColorPicker({ value: "#278787",  enabled: false});
            * &lt;/script&gt;
            * @memberof ejColorPicker
            * @instance
            */
            enabled: true,

            //Events
            /**     
			 * Fires after Color value has been changed sucessfully.If the user want to perform any operation after the color value changed then the user can make use of this change event. 
			 * @event
			 * @name ejColorPicker#change
			 * @param {Object} argument Event parameters from color picker     
			 * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
			 * @param {object}  argument.model returns the color picker model
			 * @param {string}  argument.type returns the name of the event
			 * @param {string}  argument.value return the changed color value
			 * @example   
             * &lt;input type="text" id="colorPick"/&gt; <br> 
             * &lt;script&gt;
			 * //change event for color picker
             * $('#colorPick').ejColorPicker({              
             * value: "#278787", 
             * change: function (args) {
			 * // Write a code block to perform operation after changing the color.
			 * }
             *});
            * &lt;/script&gt;
            * @memberof ejColorPicker
            * @instance
            */
            change: null,
            /**     
			 * Fires after Color value has been selected sucessfully. If the user want to perform any operation after the color value selected then the user can make use of this select event. 
			 * @event
			 * @name ejColorPicker#select
			 * @param {Object} argument Event parameters from color picker     
			 * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
			 * @param {object}  argument.model returns the color picker model
			 * @param {string}  argument.type returns the name of the event
			 * @param {string}  argument.value return the selected color value
			 * @example   
             * &lt;input type="text" id="colorPick"/&gt; <br> 
             * &lt;script&gt;
			 * //select event for color picker
             * $('#colorPick').ejColorPicker({              
             * value: "#278787", 
             * select: function (args) {
			 * // Write a code block to perform operation after selecting the color.
			 * }
             *});
            * &lt;/script&gt;
            * @memberof ejColorPicker
            * @instance
            */
            select: null,
            /**     
			 * Fires after opening the color picker popup
			 * @event
			 * @name ejColorPicker#open
			 * @param {Object} argument Event parameters from color picker     
			 * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
			 * @param {object}  argument.model returns the color picker model
			 * @param {string}  argument.type returns the name of the event			 
			 * @example   
             * &lt;input type="text" id="colorPick"/&gt; <br> 
             * &lt;script&gt;
			 * //open event for color picker
             * $('#colorPick').ejColorPicker({              
             * value: "#278787", 
             * open: function (args) {
			 * // Write a code block to perform operation after opening the color picker popup.
			 * }
             *});
            * &lt;/script&gt;
            * @memberof ejColorPicker
            * @instance
            */
            open: null,
            /**     
			 * Fires after closing the color picker popup.
			 * @event
			 * @name ejColorPicker#close
			 * @param {Object} argument Event parameters from color picker     
			 * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
			 * @param {object}  argument.model returns the color picker model
			 * @param {string}  argument.type returns the name of the event			 
			 * @example   
             * &lt;input type="text" id="colorPick"/&gt; <br> 
             * &lt;script&gt;
			 * //close event for color picker
             * $('#colorPick').ejColorPicker({              
             * value: "#278787", 
             * close: function (args) {
			 * // Write a code block to perform operation after closing the color picker popup.
			 * }
             *});
             * &lt;/script&gt;
             * @memberof ejColorPicker
             * @instance
             */
            close: null,
            /**     
			 * Fires after Color picker control is created. If the user want to perform any operation after the color picker control creation then the user can make use of this create event. 
			 * @event
			 * @name ejColorPicker#create		
			 * @param {Object} argument Event parameters from color picker     
			 * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
			 * @param {object}  argument.model returns the color picker model
			 * @param {string}  argument.type returns the name of the event			 
			 * @example   
             * &lt;input type="text" id="colorPick"/&gt; <br> 
             * &lt;script&gt;
			 * //create event for color picker
             * $('#colorPick').ejColorPicker({              
             * value: "#278787", 
             * create: function (args) {
			 * // Write a code block to perform operation after creating the color picker.
			 * }
             *});
            * &lt;/script&gt;
            * @memberof ejColorPicker
            * @instance
            */
            create: null,
            /**     
            * Fires after Color picker control is destroyed. If the user want to perform any operation after the color picker control destroyed then the user can make use of this destroy event. 
            * @event
            * @name ejColorPicker#destroy		
            * @param {Object} argument Event parameters from color picker     
            * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
            * @param {object}  argument.model returns the color picker model
            * @param {string}  argument.type returns the name of the event            
            * @example   
            * &lt;input type="text" id="colorPick"/&gt; <br> 
            * &lt;script&gt;
            * //create event for color picker
            * $('#colorPick').ejColorPicker({              
            * value: "#278787", 
            * destroy: function (args) {
            * // Write a code block to perform operation after creating the color picker.
            * }
            *});
           * &lt;/script&gt;
           * @memberof ejColorPicker
           * @instance
           */
            destroy: null,
        },
        datetypes: {
            columns: "number",
            cssClass: "string",
            opacityValue: "number",
            displayInline: "boolean",
            showRecentColors: "boolean",
            enabled: "boolean",
            showPreview: "boolean",
            enableOpacity: "boolean",
            buttonText: "data",
            custom: "data"
        },

        observables: ["value"],
        value: ej.util.valueFunction("value"),

        _setModel: function (jsondata) {
            if (!(this.model.enabled) && ej.isNullOrUndefined(jsondata["enabled"])) return false;
            for (var key in jsondata) {
                switch (key) {
                    //colorpicker API's                    
                    case "enableOpacity":
                        this.model.enableOpacity = jsondata[key];
                        this._previewSlider(this.model.enableOpacity);
                        break;
                    case "opacityValue":
                        this.model.opacityValue = jsondata[key];
                        this._changeAlpha();
                        this._opacity.option('value', this.model.opacityValue);
                        this._updateValue();
                        break;
                        //ColorPalette API's
                    case "custom":
                        this.model.custom = jsondata[key];
                        this._reInitialize();
                        break;
                    case "palette":
                        this.model.palette = jsondata[key];
                        this._reInitialize();
                        break;
                    case "columns":
                        this.model.columns = jsondata[key];
                        this._reInitialize();
                        break;
                        //common API's
                    case "value":
                        this.setValue(ej.util.getVal(jsondata[key]));
                        break;
                    case "modelType":
                        this.model.modelType = jsondata[key];
                        this.model.modelType == "default" ? this._defaultRender() : this._reInitialize();
                        break;
                    case "tooltipText":
                        this._toolTipText(jsondata[key]);
                        break;
                    case "showPreview":
                        this.model.showPreview = jsondata[key];
                        this._previewPane(this.model.showPreview);
                        break;
                    case "buttonText":
                        this._buttonText(jsondata[key]);
                        break;
                    case "displayInline":
                        this._setDisplayInline(jsondata[key]);
                        break;
                    case "cssClass":
                        this.model.cssClass = jsondata[key];
                        break;
                    case "enabled":
                        this.model.enabled = jsondata[key];
                        this._enabled(this.model.enabled);
                        break;
                    case "showRecentColors":
                        this.model.showRecentColors = jsondata[key];
                        this._previewColor(this.model.showRecentColors);
                        break;
                }
            }
        },
        _defaultRender: function () {
            this._pickerType();
            this.PaletteWrapper = this._layoutType(this.model.palette);
            this.PaletteWrapper.addClass('e-hide');
            if (this.model.displayInline && !this.element.is('input'))
                this._footer.removeClass('e-hide');
            this.popupList.prepend(this.PaletteWrapper);
        },
        _pickerType: function () {
            this._modelType = "picker";
            this.PaletteWrapper.removeAttr('style');
            this.PaletteWrapper.addClass('e-hide');
            this._gradient.removeClass('e-hide');
            this._gradient.fadeIn(200);
            this._presetTag.parents('.e-split.e-widget').addClass('e-hide');
            this._switcher.removeAttr('class');
            this._switcher.addClass('e-color-image e-pickerModel');
            this._switch = true;
            this._rgbValue();
            this._hueGradient();
            this._updateUI();
            this._templateWrapper.find('#' + this._id + "hsv").ejButton("enable");
            this._switchEvents();
            this._unSwitchEvents();
            this._hideUnBindEvents();
            this._showBindEvents();
            this.model.modelType == "default" ? this._changeTag.removeClass('e-hide') : this._changeTag.addClass('e-hide');
            this.popupList.prepend(this._gradient);
        },
        _paletteType: function () {
            this._gradient.removeAttr('style');
            this._presetTag.parents('.e-split.e-widget').removeClass('e-hide');
            this.PaletteWrapper.removeClass('e-hide');
            this.PaletteWrapper.fadeIn(200);
            this._templateWrapper.find('#' + this._id + "hsv").ejButton("disable");
            this._switch = false;
            this._disableHSVButton();
            this._cellSelect();
            this._switchEvents();
            this._unSwitchEvents();
            this.model.modelType == "default" ? this._changeTag.removeClass('e-hide') : this._changeTag.addClass('e-hide');
            this.popupList.prepend(this.PaletteWrapper);
        },
        _reInitialize: function () {
            this._destroyPalette(false);
        },
        _destroyPalette: function (presets) {
            if (this.model.modelType == "default" || this.model.modelType == "picker" || this.model.modelType == "palette")
                this.PaletteWrapper.remove();
            if (presets || this._temp != this.model.columns && this.model.palette !== "customPalette") this.PaletteWrapper = this._presetType(this._presetsId);
            else if (this.model.modelType == "palette") {
                this._modelType = "palette";
                this.PaletteWrapper = this._layoutType(this.model.palette);
                this._hideUnBindEvents();
                this._showBindEvents();
            }
            if (this.model.modelType == "picker") {
                this._pickerType();
                if (this.model.displayInline && !this.element.is('input'))
                    this._footer.addClass('e-hide');
            }
            else if (this.model.modelType == "palette") {
                this._gradient.addClass('e-hide');
                this._paletteType();
                presets || this.model.palette == "customPalette" ? "" : this._splitObj.option('prefixIcon', 'e-color-image e-basic');
            }
            else if (this.model.modelType == "default")
                if (this._modelType == "palette") {
                    this._gradient.addClass('e-hide');
                    this._paletteType();
                }
            this._temp = this.model.columns;
            if (this.model.palette == "customPalette" || this.model.modelType == "picker")
                this._presetTag.parents('.e-split.e-widget').addClass('e-hide');
            else
                this._presetTag.parents('.e-split.e-widget').removeClass('e-hide');
        },
        _previewColor: function (color) {
            if (color) {
                this._swatchesArea.fadeIn(200);
                this._bindRecentEvent();
            }
            else {
                this._swatchesArea.fadeOut(200);
                this._unBindRecentEvent();
            }
        },
        _buttonText: function (data) {
            $.extend(this.model.buttonText, data);
            this._buttonTag.html(this.model.buttonText.apply);
            this._cancelTag.html(this.model.buttonText.cancel);
        },
        _toolTipText: function (data) {
            $.extend(this.model.tooltipText, data);
            this._addTitleText();
        },
        _previewPane: function (showPreview) {
            showPreview ? this._previewTag.removeClass("e-hide") : this._previewTag.addClass("e-hide");
        },
        _previewSlider: function (slider) {
            slider ? this._opacity.enable() : this._opacity.disable();
        },
        _destroy: function () {
            if (this.isPopupOpen) this.hide();
            this.popupContainer.remove();
            if (this.wrapper) {
                this.element.insertAfter(this.wrapper);
                this.wrapper.remove();
                $('#' + this._id + "_Presets").parent('.e-menu-wrap').remove();
            }
            if (this.element.is("input")) this.element.attr('type', 'text');
            this.element.removeClass('e-colorpicker e-input');
        },
        _init: function () {
            this._browser = ej.browserInfo();
            this._isFocused = false;
            this.isPopupOpen = false;
            this._dataBind = false;
            this._modelType = "picker";
            this.model.palette === "basicPalette" ? this._presetsId = "e-presets30" : "";
            this._renderControl();
            this.model.palette === "customPalette" ? this._presetTag.parents('.e-split.e-widget').addClass('e-hide') : "";
            this.popupContainer.find('button.e-presets').ejSplitButton({ targetID: this._id + "_Presets" });
            if (this.value()) {
                this.setValue(this.value());
                if (this._switch) this._rgbValue();
                this._alphaGradient(this.value());
            }
            this._hsvValue();
            this._alphaGradient(this.RGBToHEX(this.rgb))
            this._hueGradient();
            this._addTitleText();
            this._temp = this.model.columns;
        },
        _renderControl: function () {
            this._createWrapper();
            this._renderPopupPanelWrapper();
            this._renderPopupElement();
            this._selectedButton = this._groupTag.find('.e-click');
            this._setDisplayInline(this.model.displayInline);
            this._previewPane(this.model.showPreview);
            this._previewColor(this.model.showRecentColors);
            if (this._switch) this._previewSlider(this.model.enableOpacity);
            this._wireEvents();
            this._switchEvents();
        },

        _createWrapper: function () {
            if (this.element.is("input")) {
                this.element.addClass('e-input');
                this.spanElement = ej.buildTag("span.e-selected-color");     //style
                this.wrapper = ej.buildTag("span.e-colorwidget e-picker e-widget #" + this._id + "Wrapper" + this.model.cssClass).attr({ 'tabindex': '0', "aria-expanded": false, "aria-haspopup": true, "aria-owns": this._id + "_popup" });
                this.container = ej.buildTag("span.e-in-wrap e-box");
                this.drpbtnspan = ej.buildTag("span.e-icon e-down-arrow", "", {}, { "aria-label": "select" });
                this.dropdownbutton = ej.buildTag("span.e-select#" + this._id + "_dropdown", "", {}, { "role": "button" }).append(this.drpbtnspan);
                this.iconWrapper = ej.buildTag("span.e-tool-icon " + this.model.toolIcon);
                this.container.insertAfter(this.element);
                if (!ej.isNullOrUndefined(this.model.toolIcon)) {
                    this.iconWrapper.append(this.spanElement);
                    this.container.addClass('e-tool');
                    this.container.append(this.iconWrapper);
                }
                else
                    this.container.append(this.spanElement);
                this.container.append(this.element, this.dropdownbutton);
                this.wrapper.insertBefore(this.container);
                this.wrapper.append(this.container);
                this.element.attr({ 'type': 'hidden' }).val(this.value())
            }
            this._checkNameAttr();
        },
        _addTitleText: function () {
            this._switcher.attr('title', this.model.tooltipText.switcher);
            this._spanTag.attr('title', this.model.tooltipText.addButton);
            this._presetLi.find("#e-presets00").attr('title', this.model.tooltipText.webColors);
            this._presetLi.find("#e-presets01").attr('title', this.model.tooltipText.vintage);
            this._presetLi.find("#e-presets02").attr('title', this.model.tooltipText.seaWolf);
            this._presetLi.find("#e-presets10").attr('title', this.model.tooltipText.sandy);
            this._presetLi.find("#e-presets11").attr('title', this.model.tooltipText.pinkShades);
            this._presetLi.find("#e-presets12").attr('title', this.model.tooltipText.moonLight);
            this._presetLi.find("#e-presets20").attr('title', this.model.tooltipText.monoChrome);
            this._presetLi.find("#e-presets21").attr('title', this.model.tooltipText.misty);
            this._presetLi.find("#e-presets22").attr('title', this.model.tooltipText.flatColors);
            this._presetLi.find("#e-presets30").attr('title', this.model.tooltipText.basic);
            this._presetLi.find("#e-presets31").attr('title', this.model.tooltipText.candyCrush);
            this._presetLi.find("#e-presets32").attr('title', this.model.tooltipText.citrus);
            this._currentTag.attr('title', this.model.tooltipText.currentColor);
            this._previousTag.attr('title', this.model.tooltipText.selectedColor);
        },
        _renderPopupPanelWrapper: function () {
            this.popupContainer = ej.buildTag("div.e-colorpicker e-box e-popup e-widget " + this.model.cssClass + "#" + this._id + "_popup", "", { "role": "grid", "aria-readonly": "true", "tabIndex": 0, "style": "visibility:hidden" });
            $('body').append(this.popupContainer);

            this.popupList = ej.buildTag("div.e-popupWrapper");
            //render color area
            this._gradient = ej.buildTag("div.e-container");
            //gradient Area            
            this._colorArea = ej.buildTag("div.e-hsv-color");
            this._gradientArea = ej.buildTag("div.e-hsv-gradient")//style
            this._handleArea = ej.buildTag("div.e-draghandle e-color-image").attr({ 'tabindex': '0' });
            this._colorArea.append(this._gradientArea, this._handleArea);
            //slider area
            this._picker = ej.buildTag("div.e-gradient");
            this._hueSlider = ej.buildTag("div.e-widget e-hue e-state-default#" + this._id + "hue");
            this._alphaSlider = ej.buildTag("div.e-widget e-opacity e-state-default#" + this._id + "alpha");
            this._picker.append(this._hueSlider, this._alphaSlider);
            //gradient and slider container            
            this._gradient.append(this._colorArea, this._picker);
            //first level block design       
            this.popupList.append(this._gradient);

            //second block
            this._footerBlock = ej.buildTag("div.e-footerContainer");
            //container for code switcher and input element
            this._templateWrapper = ej.buildTag("div.e-buttons");

            //buttons area
            this._groupTag = ej.buildTag("div.e-groupButton");

            this._formEle = ej.buildTag("div.e-form");
            this._rgb = ej.buildTag("button.e-rgbButton e-click#" + this._id + "rgba");
            this._hexCode = ej.buildTag("button.e-hexButton#" + this._id + "hex");
            this._hsva = ej.buildTag("button.e-hsvButton#" + this._id + "hsv");
            this._groupTag.append(this._rgb, this._hexCode, this._hsva);

            //input area
            this._codeEditor = ej.buildTag("div.e-codeeditor");
            this._inputTag = ej.buildTag("input.e-color-code", "", {}, { "type": "text", 'tabindex': '0', "maxLength": "22" });
            this._codeEditor.append(this._inputTag);

            this._formEle.append(this._groupTag, this._codeEditor);
            //container for preview area
            this._previewTag = ej.buildTag("div.e-preview").attr({ 'tabindex': '0' });
            this._currentTag = ej.buildTag("div.e-current").attr({ 'tabindex': '0' });
            this._previousTag = ej.buildTag("div.e-previous").attr({ 'tabindex': '0' });
            this._previewTag.append(this._currentTag, this._previousTag);

            this._templateWrapper.append(this._formEle, this._previewTag);

            //swatches area
            this._swatchesArea = ej.buildTag("div.e-color-labels");
            var blockCount = 11;     //No.of blocks for save the selected colors
            this._divTag = ej.buildTag("div.e-recent-color");
            this._addTag = ej.buildTag('div.e-colorblock e-block');
            this._spanTag = ej.buildTag('div.e-color e-color-image e-add');
            this._addTag.append(this._spanTag)
            this._divTag.append(this._addTag);
            for (var count = 0; count < blockCount; count++) {
                this._liTag = ej.buildTag('div.e-colorblock e-block');
                var spanTag = ej.buildTag('div.e-color e-color-image e-empty');
                this._liTag.append(spanTag);
                this._divTag.prepend(this._liTag);
            }
            this._swatchesArea.append(this._divTag);

            //footer area             

            this._footer = ej.buildTag('div.e-footer');
            this._swatches = ej.buildTag('div.e-element');
            this._changeTag = ej.buildTag('div.e-switcher');
            this._switcher = ej.buildTag('div.e-color-image');
            this._presetTag = ej.buildTag('button.e-presets#' + this._id + "-split");
            this._presets = ej.buildTag('div');
            this._buttonTag = ej.buildTag("button.e-applyButton");
            this._cancelTag = ej.buildTag("button.e-cancelButton");
            this._changeTag.append(this._switcher);
            this._presetTag.append(this._presets);
            this._swatches.append(this._changeTag, this._presetTag);
            this._footer.append(this._swatches, this._buttonTag, this._cancelTag);

            this._footerBlock.append(this._templateWrapper, this._swatchesArea, this._footer);


            //for this design purpose                                   

            this.PaletteWrapper = this._layoutType(this.model.palette);
            this.popupList.append(this.PaletteWrapper, this._footerBlock);
            this._presetContainer = ej.buildTag('ul.e-presetWrapper' + "#" + this._id + "_Presets", "", {}, { "style": "top:87px" });
            this._presetLi = ej.buildTag('li.e-item');
            this._presetLi.append(this._renderPresets());
            this._presetContainer.append(this._presetLi);
            this.popupList.append(this._presetContainer);
            this.popupContainer.append(this.popupList);

            //IE Support
            if (this._browser.name = "msie" && (this._browser.version == "9.0" || this._browser.version == "8.0"))
                this._hueSlider.addClass('e-color-image e-filter');
            else
                this._hueSlider.addClass('e-common');
        },
        _colorPresetsClick: function (e) {
            this._presetsId = e.currentTarget.id;
            if (this._presetsId === "e-presets00") this._splitObj.option('prefixIcon', 'e-color-image e-webcolor');
            else if (this._presetsId === "e-presets01") this._splitObj.option('prefixIcon', 'e-color-image e-vintage');
            else if (this._presetsId === "e-presets02") this._splitObj.option('prefixIcon', 'e-color-image e-seawolf');
            else if (this._presetsId === "e-presets10") this._splitObj.option('prefixIcon', 'e-color-image e-sandy');
            else if (this._presetsId === "e-presets11") this._splitObj.option('prefixIcon', 'e-color-image e-pink');
            else if (this._presetsId === "e-presets12") this._splitObj.option('prefixIcon', 'e-color-image e-moonlight');
            else if (this._presetsId === "e-presets20") this._splitObj.option('prefixIcon', 'e-color-image e-monochrome');
            else if (this._presetsId === "e-presets21") this._splitObj.option('prefixIcon', 'e-color-image e-misty');
            else if (this._presetsId === "e-presets22") this._splitObj.option('prefixIcon', 'e-color-image e-flat');
            else if (this._presetsId === "e-presets30") this._splitObj.option('prefixIcon', 'e-color-image e-basic');
            else if (this._presetsId === "e-presets31") this._splitObj.option('prefixIcon', 'e-color-image e-candy');
            else if (this._presetsId === "e-presets32") this._splitObj.option('prefixIcon', 'e-color-image e-citrus');
            this._destroyPalette(true, false);
        },

        _renderPresets: function () {
            var tableDiv = ej.buildTag("div.e-presets-table");
            var spanTag = ej.buildTag("span.e-presetHeader");
            spanTag.html("Swatches");
            tableDiv.append(spanTag);
            var color = 0, hexCode, rowDiv;
            for (var row = 0; row < 4 ; row++) {
                rowDiv = ej.buildTag("ul.e-tablerow");
                for (var col = 0; col < 3 ; col++) {
                    tableCell = ej.buildTag("li.e-color-image e-preset-row" + "#" + "e-presets" + [row] + [col]);
                    tableCell.appendTo(rowDiv);
                }
                rowDiv.appendTo(tableDiv);
            }
            return tableDiv;
        },
        _renderPopupElement: function () {
            var proxy = this;
            this._hsv =
            {
                h: 360, s: 100, v: 100
            };
            this._templateWrapper.find('#' + this._id + "rgba").ejButton({ text: "RGBA", type: "button" });
            this._templateWrapper.find('#' + this._id + "hex").ejButton({ text: "HEX", type: "button" });
            this._templateWrapper.find('#' + this._id + "hsv").ejButton({ text: "HSVA", type: "button" });
            this._applyObj = this.popupContainer.find('button.e-applyButton').ejButton({ text: this.model.buttonText.apply, type: "button" }).data('ejButton');
            this._cancelObj = this.popupContainer.find('button.e-cancelButton').ejButton({ text: this.model.buttonText.cancel, type: "button" }).data('ejButton');
            this._splitObj = this.popupContainer.find('#' + this._id + "-split").ejSplitButton({ size: "normal", showRoundedCorner: true, contentType: "imageonly" }).data('ejSplitButton');
            this._presetTag.parents('.e-split.e-widget').css({ "height": "27px" });
            this.model.custom.length == 0 ? this._splitObj.option('prefixIcon', "e-color-image e-basic") : "";
            this._splitObj._getXYpos = function (e) {
                $("#" + this.model.targetID).ejMenu({ animationType: "none" });
                $("#" + this.model.targetID).ejMenu('hide', e);
                btnpos = this.dropbutton.offset();
                btnposx = btnpos.left - this.dropbutton.prev().outerWidth() - 1;
                btnposy = (btnpos.top - $("#" + this.model.targetID).outerHeight()) - 1;
                return { x: btnposx, y: btnposy }
            }
            this._colorSlider = this.popupContainer.find("#" + this._id + "hue").ejSlider({ orientation: "Vertical", minValue: 0, maxValue: 360, change: function (e) { proxy._changeHue(e); }, slide: function (e) { proxy._changeHue(e); } }).data('ejSlider');
            this._opacity = this.popupContainer.find("#" + this._id + "alpha").ejSlider({ value: this.model.opacityValue, orientation: "Vertical", incrementStep: 5, value: 100, change: function (e) { proxy._changeAlpha(e); }, slide: function (e) { proxy._changeAlpha(e); } }).data('ejSlider');
            this._colorSlider.firstHandle.css({ "height": "13px", "margin": "0px 0px -12px -2px", "width": "13px" });
            this._opacity.firstHandle.css({ "height": "13px", "margin": "0px 0px -12px -2px", "width": "13px" });
            this.popupContainer.css({ "visibility": "visible", "display": "none" });
            if (this.model.modelType == "picker") {
                this._modelType = "picker";
                this._gradient.removeClass('e-hide');
                this._changeTag.addClass('e-hide');
                this.PaletteWrapper.addClass('e-hide');
                this._presetTag.parents('.e-split.e-widget').addClass('e-hide');
                this._switch = true;
            } else if (this.model.modelType == "palette") {
                this._modelType = "palette"
                this._changeTag.addClass('e-hide');
                this._presetTag.parents('.e-split.e-widget').removeClass('e-hide');
                this.PaletteWrapper.removeClass('e-hide');
                this._gradient.addClass('e-hide');
                this._templateWrapper.find('#' + this._id + "hsv").ejButton("disable");
                this._switch = false;
            } else if (this.model.modelType == "default") {
                this._modelType = "picker";
                this._gradient.removeClass('e-hide');
                this._switcher.addClass('e-pickerModel');
                this.PaletteWrapper.addClass('e-hide');
                this._presetTag.parents('.e-split.e-widget').addClass('e-hide');
                this._switch = true;
            }
        },
        _layoutType: function (type) {
            if (typeof type === "string" && type == "basicPalette")
                this._collection = this._paletteGenerate(ej.Colors.basic, this.model.columns);
            else if (typeof type === "string" && type == "customPalette")
                this._collection = this._paletteGenerate(this.model.custom, this.model.columns);
            type == "customPalette" ? this._collection.addClass('e-custom') : "";
            return this._collection;
        },
        _presetType: function (type) {
            if (type === "e-presets00") this._collection = this._paletteGenerate(ej.Colors.webColors, this.model.columns);
            else if (type === "e-presets01") this._collection = this._paletteGenerate(ej.Colors.vintage, this.model.columns);
            else if (type === "e-presets02") this._collection = this._paletteGenerate(ej.Colors.seaWolf, this.model.columns);
            else if (type === "e-presets10") this._collection = this._paletteGenerate(ej.Colors.sandy, this.model.columns);
            else if (type === "e-presets11") this._collection = this._paletteGenerate(ej.Colors.pinkShades, this.model.columns);
            else if (type === "e-presets12") this._collection = this._paletteGenerate(ej.Colors.moonLight, this.model.columns);
            else if (type === "e-presets20") this._collection = this._paletteGenerate(ej.Colors.monoChrome, this.model.columns);
            else if (type === "e-presets21") this._collection = this._paletteGenerate(ej.Colors.misty, this.model.columns);
            else if (type === "e-presets22") this._collection = this._paletteGenerate(ej.Colors.flat, this.model.columns);
            else if (type === "e-presets30") this._collection = this._paletteGenerate(ej.Colors.basic, this.model.columns);
            else if (type === "e-presets31") this._collection = this._paletteGenerate(ej.Colors.candyCrush, this.model.columns);
            else if (type === "e-presets32") this._collection = this._paletteGenerate(ej.Colors.citrus, this.model.columns);
            return this._collection;
        },
        _paletteGenerate: function (colors, columns) {
            var color;
            this._PresetTable = ej.buildTag("div.e-palette-color").attr({ "role": "presentation" });
            this._tag = ej.buildTag("div.e-row");
            for (color = 0; color < colors.length; color++) {
                if (color && color % columns == 0)
                    this._tag = ej.buildTag("div.e-row");
                this._td = ej.buildTag("div.e-item").attr({ 'role': 'gridcell', "aria-label": "#" + $.trim(colors[color]), "value": "#" + $.trim(colors[color]), "style": "background-color:" + "#" + $.trim(colors[color]), "tabindex": "0" });
                this._tag.append(this._td);
                this._PresetTable.append(this._tag);
            }
            return this._PresetTable;
        },
        _checkNameAttr: function () {
            if (!this.element.attr("name"))
                this.element.attr({ "name": this.element[0].id });
        },
        _enabled: function (bool) {
            bool ? this.enable() : this.disable();
        },
        _setDisplayInline: function (isDisplayInline) {
            this.model.displayInline = isDisplayInline;
            if (isDisplayInline && this.element.is("input")) {
                this.popupContainer.insertAfter(this.wrapper);
                this._setPopupPosition();
            }
            else if (isDisplayInline) {
                this.element.append(this.popupContainer);
                this.popupContainer.find('button.e-applyButton').css({ "display": "none" });
                this.popupContainer.find('button.e-cancelButton').css({ "display": "none" });
                this._footer.css({ "display": "none" });
            }
            else {
                this.popupContainer.css('display', 'none');
                $('body').append(this.popupContainer);
                this._isOpen = false;
                this._bindIconClick();
                this._isFocused = false;
                this.isPopupOpen = false;
                this.wrapper.removeClass("e-focus");
            }
            if (isDisplayInline) {
                this.show();
                if (this.element.is("input")) this._off($(this.dropdownbutton), "mousedown", this._iconClick);
            }
        },
        _bindIconClick: function () {
            var count = $._data($(this.dropdownbutton)[0], "events");
            ej.isNullOrUndefined(count) ? this._on(this.dropdownbutton, "mousedown", this._iconClick) : count.mousedown.length == 0 ? this._on(this.dropdownbutton, "mousedown", this._iconClick) : "";
        },
        _unBindIconClick: function () {
            this._off(this.dropdownbutton, "mousedown", this._iconClick);
        },
        _setPopupPosition: function () {
            var elementObj = this.wrapper;
            var pos = elementObj.offset(),
            left = pos.left,
            totalHeight = elementObj.outerHeight(),
            border = (totalHeight - elementObj.height()) / 2,
            maxZ = this._getZindexPartial();
            this.popupContainer.css({
                "left": left + 3 + "px",
                'position': 'absolute',
                "top": pos.top + totalHeight - border + 3 + "px",
                "z-index": maxZ
            });
        },
        _getZindexPartial: function () {
            var parents = this.element.parents(), bodyEle;
            bodyEle = $('body').children(), index = bodyEle.index(this.popupList);
            bodyEle.splice(index, 1);
            $(bodyEle).each(function (i, ele) { parents.push(ele); });

            var maxZ = Math.max.apply(maxZ, $.map(parents, function (e, n) {
                if ($(e).css('position') != 'static')
                    return parseInt($(e).css('z-index')) || 1;
            }));
            if (!maxZ || maxZ < 10000) maxZ = 10000;
            else maxZ += 1;
            return maxZ;
        },
        setValue: function (value) {
            var color;
            this.value(value);
            if (value) {
                color = this._HexToHSV(value);
                this._oldValue = this.rgb;
                if (this._switch) {
                    this._valueOperation();
                    this._colorSlider.option('value', parseInt(this._hsv.h));
                }
                this._inputTagValue(this._selectedButton);
                if (!this.model.displayInline || this.element.is("input"))
                    this.spanElement.css({ "background-color": this._formRGBA(this.rgb) });
                if (!this._switch) {
                    this._updateUI();
                    this.element.val(this.value());
                }
            }
        },
        /**
       * Enable the color picker control
       * @return jQuery
       
       * @example   
       * &lt;input type="text" id="colorPick"/&gt; <br> 
       * &lt;script&gt;
       * // Create Color picker
       * $('#colorPick').ejColorPicker({ value: "#278787" });
       * // Create ColorPicker instance
       * var colorObj = $("#colorPick").data("ejColorPicker");
       * colorObj.enable(); // enables the colorPicker
       * &lt;/script&gt; 
       
       * @example   
       * &lt;input type="text" id="colorPick"/&gt; <br> 
       * &lt;script&gt;
       * // Create Color picker
       * $('#colorPick').ejColorPicker({ value: "#278787" });
        * // enable the colorPicker
       * $("#colorPick").ejColorPicker("enable");
       * &lt;/script&gt;       
       
       * @memberof ejColorPicker
       * @instance
       */
        enable: function () {
            if (this.wrapper.hasClass("e-disable")) {
                this.wrapper.removeClass('e-disable');
                this.popupList.removeClass('e-disable');
                this.dropdownbutton.removeClass('e-disable');
                var temp = this._switch;
                this._switch = true;
                this._cancelObj.enable();
                this._colorSlider.enable();
                this._opacity.enable();
                this._splitObj.enable();
                this._applyObj.enable();
                this._wireEvents();
                this._switchEvents();
                this._switch = temp;
                $(this._inputTag).removeAttr('readonly');
                this._on(this.dropdownbutton, "mousedown", this._iconClick);
                if (this._switcher.hasClass('e-paletteModel')) {
                    this._on($(document), "keydown", this._keyDown);
                    this._on(this._collection, "mousedown", this._onSelect);
                } else
                    this._on($(document), "keydown", this._onKeyDown);

            }
        },
        /**
        * Disables the color picker control
		* @return jQuery
		
        * @example   
        * &lt;input type="text" id="colorPick"/&gt; <br> 
        * &lt;script&gt;
        * // Create Color picker
        * $('#colorPick').ejColorPicker({ value: "#278787" });
        * // Create ColorPicker instance
		* var colorObj = $("#colorPick").data("ejColorPicker");
		* colorObj.disable(); // disables the colorPicker
        * &lt;/script&gt; 
        
        * @example   
        * &lt;input type="text" id="colorPick"/&gt; <br> 
        * &lt;script&gt;
        * // Create Color picker
        * $('#colorPick').ejColorPicker({ value: "#278787" });
         * // disables the colorPicker
        * $("#colorPick").ejColorPicker("disable");
        * &lt;/script&gt;       
		
        * @memberof ejColorPicker
	    * @instance
		*/
        disable: function () {
            if (!this.wrapper.hasClass("e-disable")) {
                this.wrapper.addClass('e-disable');
                this.popupList.addClass('e-disable');
                this.dropdownbutton.addClass('e-disable');
                var temp = this._switch;
                this._switch = false;
                this._cancelObj.disable();
                this._colorSlider.disable();
                this._opacity.disable();
                this._splitObj.disable();
                this._applyObj.disable();
                this._unWireEvents();
                this._unSwitchEvents();
                this._switch = temp;
                this._unBindIconClick();
                $(this._inputTag).attr('readonly', 'readonly');
                if (this._switcher.hasClass('e-paletteModel')) {
                    this._off($(document), "keydown", this._keyDown);
                    this._off(this._collection, "mousedown", this._onSelect);
                } else
                    this._off($(document), "keydown", this._onKeyDown);
            }
            if (this.isPopupOpen && !this.model.displayInline)
                this.hide();
        },
        /**
        * Gets the selected color in RGB format
		* @return jQuery
		
        * @example   
        * &lt;input type="text" id="colorPick"/&gt; <br> 
        * &lt;script&gt;
        * // Create Color picker
        * $('#colorPick').ejColorPicker({ value: "#278787" });
        * // Create ColorPicker instance
		* var colorObj = $("#colorPick").data("ejColorPicker");
		* var color=colorObj.getColor(); // gets the selected color in RGB format
        * alert("Red="+color.r+", Green="+color.g+", Blue="+color.b);
        * &lt;/script&gt; 
        
        * @example   
        * &lt;input type="text" id="colorPick"/&gt; <br> 
        * &lt;script&gt;
        * // Create Color picker
        * $('#colorPick').ejColorPicker({ value: "#278787" });
         * // gets the selected color in RGB format
        * var color=$("#colorPick").ejColorPicker("getColor");
        * alert("Red="+color.r+", Green="+color.g+", Blue="+color.b);
        * &lt;/script&gt;       
		
        * @memberof ejColorPicker
	    * @instance
		*/
        getColor: function () {
            return this.rgb;
        },
        /**
        * Gets the selected color value as string
		* @return jQuery
		
        * @example   
        * &lt;input type="text" id="colorPick"/&gt; <br> 
        * &lt;script&gt;
        * // Create Color picker
        * $('#colorPick').ejColorPicker({ value: "#278787" });
        * // Create ColorPicker instance
		* var colorObj = $("#colorPick").data("ejColorPicker");
		* alert(colorObj.getValue()); // gets the selected color value as string
        * &lt;/script&gt; 
        
        * @example   
        * &lt;input type="text" id="colorPick"/&gt; <br> 
        * &lt;script&gt;
        * // Create Color picker
        * $('#colorPick').ejColorPicker({ value: "#278787" });
         * // gets the selected color value as string
        * alert($("#colorPick").ejColorPicker("getValue"));
        * &lt;/script&gt;       
		
        * @memberof ejColorPicker
	    * @instance
		*/
        getValue: function () {
            return this.value();
        },
        _alphaGradient: function (value) {
            var browser = ej.browserInfo();
            var value = ej.isNullOrUndefined(value) ? "#000000" : value;
            if (browser.name == "mozilla")
                this.popupList.find("#" + this._id + "alpha").attr({ "style": "background:-moz-linear-gradient(center top," + value + ",#fff) repeat scroll 0 0 rgba(0, 0, 0, 0);" });
            else if ((browser.name == "msie") && (browser.version == "9.0" || browser.version == "8.0"))
                this.popupList.find("#" + this._id + "alpha").attr({ "style": "filter: progid:DXImageTransform.Microsoft.gradient( startColorstr=" + value + ", endColorstr='#ffffff',GradientType=0 );" });
            else if (browser.name == "msie")
                this.popupList.find("#" + this._id + "alpha").attr({ "style": "background:-ms-linear-gradient(top," + value + ",#fff) repeat scroll 0 0 rgba(0, 0, 0, 0);" });
            else if (browser.name == "chrome" || browser.name == "safari")
                this.popupList.find("#" + this._id + "alpha").attr({ "style": "background:-webkit-linear-gradient(top," + value + ",#fff) repeat scroll 0 0 rgba(0, 0, 0, 0);" });
            else if (browser.name == "opera")
                this.popupList.find("#" + this._id + "alpha").attr({ "style": "background:-o-linear-gradient(top," + value + ",#fff) repeat scroll 0 0 rgba(0, 0, 0, 0);" });
            if (browser.name == "msie" && browser.version == "8.0")
                this.popupList.find("#" + this._id + "alpha").children(".e-handle").css({ "background": this._formRGB(this.rgb), "filter": "alpha(opacity=" + this.rgb.a * 100 + ")" });
        },
        _hueGradient: function () {
            var temp = this._hsv;
            var value = { h: this._hsv.h, s: 100, v: 100 };
            this.popupList.find("#" + this._id + "hue").children(".e-handle").css({ "background": this._formRGB(this.HSVToRGB(value)) });
            this._hsv = temp;
        },
        _changeEvent: function () {
            var colorCode;
            if (this._change) {
                colorCode = this.RGBToHEX(this.rgb);
                this._alphaGradient(colorCode);
                this._trigger("change", { value: colorCode });
            }
        },
        _selectEvent: function () {
            this.value(this.RGBToHEX(this.rgb));
            this.element.is("input") ? this.spanElement.css({ "background-color": this._formRGBA(this.rgb) }) : "";
            if (this._previousColor !== this.value()) {
                this.element.attr({ 'value': this.value() })
                this._trigger("select", { value: this.value() });
            }
        },
        _changeHue: function (e) {
            this._hsv.h = Math.round(e.value);
            this._hueGradient();
            this._hsvValue();
            this._changeEvent();
        },
        _changeAlpha: function (e) {
            this.rgb.a = ej.isNullOrUndefined(e) ? this.model.opacityValue / 100 : e.value / 100;
            if (this._browser.name = "msie" && this._browser.version == "8.0") {
                this._currentTag.css({ "background-color": this._formRGB(this.rgb), "filter": "alpha(opacity=" + this.rgb.a * 100 + ")" });
                this.popupList.find("#" + this._id + "alpha").children(".e-handle").css({ "background": this._formRGB(this.rgb), "filter": "alpha(opacity=" + this.rgb.a * 100 + ")" });
            }
            else {
                this._currentTag.css("background-color", this._formRGBA(this.rgb));
                this.popupList.find("#" + this._id + "alpha").children(".e-handle").css({ "background": this._formRGBA(this.rgb) });
            }
            this._inputTagValue(this._selectedButton);
        },
        _updateValue: function () {
            if (this._browser.name = "msie" && this._browser.version == "8.0") this.spanElement.css({ "background-color": this._formRGB(this.rgb), "filter": "alpha(opacity=" + this.rgb.a * 100 + ")" });
            else this.spanElement.css({ "background-color": this._formRGBA(this.rgb) });
        },
        _wireEvents: function () {
            if (this.element.is('input')) {
                this._on(this.wrapper, "blur", this._targetBlur);
                this._on(this.wrapper, "focus", this._targetFocus);
                this._on(this.wrapper, "keydown", this._popupShown);
            }
            this._on(this._cancelTag, "click", this.hide);
            this._on(this._changeTag, "click", this._switchModel);
            this._on(this._groupTag, "click", this._groupButton);
            this._on(this._addTag, "click", this._addColor);
            this._on(this._buttonTag, "click", this._buttonClick);
            this._on(this._presetContainer.find("li.e-preset-row"), "mousedown", this._colorPresetsClick);

        },
        _unWireEvents: function () {
            if (!this.model.displayInline && !this.element.is('input')) {
                this._off(this.wrapper, "blur", this._targetBlur);
                this._off(this.wrapper, "focus", this._targetFocus);
                this._off(this.wrapper, "keydown", this._popupShown);
            }
            this._off(this._cancelTag, "click", this.hide);
            this._off(this._changeTag, "click", this._switchModel);
            this._off(this._groupTag, "click", this._groupButton);
            this._off(this._addTag, "click", this._addColor);
            this._off(this._buttonTag, "click", this._buttonClick);
            this._off(this._presetContainer.find("li.e-preset-row"), "mousedown", this._colorPresetsClick);
        },
        _popupShown: function (e) {
            if (e.altKey && e.keyCode == 40) { this._showHidePopup(); return false; }
        },
        _recentColor: function (e) {
            this._divTag.find('.e-select').removeClass('e-select').addClass('e-block');
            var hexCode = e.target.attributes.value;
            var rgbCode = e.target.attributes.style;
            if (ej.isNullOrUndefined(hexCode) || ej.isNullOrUndefined(rgbCode)) { this._change = false; return false; }
            $(e.target.parentNode).addClass('e-select').removeClass('e-block');
            var alpha = rgbCode.value.replace(/^(background-color:rgb|background-color:rgba)\(/, '').replace(/\)$/, '').replace(/\s/g, '').split(',');
            this._opacity.option('value', parseFloat(alpha[3]) * 100);
            this.rgb.a = parseFloat(alpha[3]);
            this._HexToHSV(hexCode.value)
            this._valueOperation();
            this._inputTagValue(this._selectedButton);
            this._colorSlider.option('value', parseInt(this._hsv.h));
            this._hueGradient();
            this._changeEvent();
            if (!this.model.displayInline || this.element.is("input"))
                this.wrapper.focus();
        },

        _handleClick: function (e) {
            e.preventDefault();
            this.mouseDownPos = this._handlePos;
            this._gradientArea.bind("mousemove", $.proxy(this._handleMovement, this));
            $(document).bind("mouseup", $.proxy(this._handleUp, this));
        },
        _handleMove: function (e) {
            $(this._handleArea).focus();
            this._handleMovement(e);
        },
        _handleMovement: function (e) {
            //value 100 used to specify the highest value of s and v.
            //the value 146 specify the gradient area height
            //the value 152 specify the gradient area width
            if (!this.model.enabled) return false;
            if (!this.model.displayInline || this.element.is("input"))
                this.wrapper.focus();
            this._hsv.v = parseInt(100 * (this._gradientArea.height() - Math.max(0, Math.min(this._gradientArea.height(), (e.pageY - this._gradientArea.offset().top)))) / this._gradientArea.height(), 10);
            this._hsv.s = parseInt(100 * (Math.max(0, Math.min(this._gradientArea.width(), (e.pageX - this._gradientArea.offset().left)))) / this._gradientArea.width(), 10);
            this._hsvValue();
            this._change = true;
            this._changeEvent();
        },
        _handleUp: function () {
            $(document).unbind('mouseup', this._handleUp);
            this._gradientArea.unbind('mousemove', this._handleMovement);
            return false;
        },
        _handlePosition: function () {
            this._handlePos = this._handleArea ? {
                left: parseInt(this._gradientArea.width() * this._hsv.s / 100, 10) + "px",
                top: parseInt(this._gradientArea.height() * (100 - this._hsv.v) / 100, 10) + "px"
            } : "";
            this._handleArea.css({ "left": this._handlePos.left, "top": this._handlePos.top });
        },
        _addColor: function () {
            this._previousColor = this.value();
            var collection = this._divTag.find('> div');
            value = this._selectedButton.html() != "HSVA" ? this._inputTag.val() : this._formRGBA(this.HSVToRGB(this._hsv));
            if (value !== "" && this._change) {
                if (this.model.showRecentColors && collection.length <= 12) {
                    $($(collection)[collection.length - 2]).remove();
                    this._generateLi();
                }
            }
            if (!this.model.displayInline || this.element.is("input")) this.wrapper.focus();
        },
        _buttonClick: function (e) {
            this._change = true;
            this._previousColor = this.value();
            var value = this._inputTag.val(), collection = this._divTag.find('div');
            this.model.opacityValue = this.rgb.a * 100;
            this.value(this.RGBToHEX(this.rgb));
            this._updatePreviewColor();
            if (this.model.displayInline && this.element.is("input") || !this.model.displayInline) {
                this._selectEvent();
                this._change ? this.hide() : "";
                this.wrapper.focus();
            }
            this._opacity.option('value', this.model.opacityValue);
        },
        _generateLi: function () {
            this._liTag = ej.buildTag('div.e-colorblock e-block').attr({ "value": this.RGBToHEX(this.rgb), "tabindex": "0" });
            var spanTag = ej.buildTag('div.e-color e-set').attr({ "style": "background-color:" + this._formRGBA(this.rgb), "value": this.RGBToHEX(this.rgb), "title": this.RGBToHEX(this.rgb) });
            this._liTag.append(spanTag);
            this._divTag.prepend(this._liTag);
        },
        _colorCodeValue: function (e) {
            var newValue = "", codeValue = this._inputTag.val(), color, value, code, count;
            value = $.trim(codeValue);
            value.length == 5 ? this._inputTag.removeClass('e-error') : "";
            if ((e.shiftKey && e.keyCode >= 35 && e.keyCode <= 40) || (e.ctrlKey && (e.keyCode == 88 || e.keyCode == 86)) || e.keyCode == 190)
                this._keyPressFlag = 1;
            else if ((!e.crtlKey && !e.shiftKey) && ((e.keyCode >= 65 && e.keyCode < 71) || (e.keyCode >= 35 && e.keyCode <= 40) || (e.keyCode >= 48 && e.keyCode <= 57) || e.keyCode == 13 || e.keyCode == 8 || e.keyCode == 46 || e.type === "blur"))
                this._keyPressFlag = 1;
            else this._keyPressFlag = 0;
            if (this.model.enableOpacity && (e.keyCode == 188 || e.keyCode == 71 || e.keyCode == 82) || (e.shiftKey && (e.keyCode == 57 || e.keyCode == 48)))
                this._keyPressFlag = 1;
            if (this._keyPressFlag == 1) {
                this._inputTag.removeClass('e-error');
                if (e.keyCode === 13 || e.type === "blur") {
                    this._inputTag.removeClass('e-error');
                    var regex = /^\#([a-fA-F0-9]{6}|[a-fA-F0-9]{3})$/;
                    code = value.match(regex);
                    if (!ej.isNullOrUndefined(code)) {
                        if (code[1].length === 3) {
                            for (count = 0; count < code[1].length; count++) {
                                newValue += code[1][count] + code[1][count];
                            }
                        }
                        else if (code[1].length === 6)
                            newValue = code[1];
                        value = this.hexCodeToRGB("#" + newValue);
                        this._inputTag.val("#" + newValue);
                        this.rgb = this.RGBToHSV(value);
                        this._change = true;
                    }
                    else {
                        var rgbRegex = /^rgba?\((\d+),\s*(\d+),\s*(\d+)(?:,\s*(\d+(?:\.\d+)?))?\)$/;
                        code = value.match(rgbRegex);
                        if (!ej.isNullOrUndefined(code)) {
                            this._rgbaColor(code);
                        }
                        else {
                            var hsvRegex = /^hsva?\((\d+),\s*(\d+),\s*(\d+)(?:,\s*(\d+(?:\.\d+)?))?\)$/;
                            code = value.match(hsvRegex);
                            if (!ej.isNullOrUndefined(code)) {
                                this._hsvColor(code);
                            }
                            else {
                                //this._inputTag.addClass('e-error');
                                this._showAlert();
                                this._change = false;
                                return false;
                            }
                        }
                    }
                    if (this._change) {
                        this._valueOperation();
                        this._colorSlider.option('value', parseInt(this._hsv.h));
                        this._hueGradient();
                        this._inputTag.removeClass("e-error");
                    }
                }
            }
            else {
                if (e.keyCode != 9)
                    e.preventDefault();
                if (!e.shiftKey && !e.ctrlKey)
                    this._showAlert();
            }
        },
        _rgbaColor: function (code) {
            var rgb = {}, color, newValue;
            if (code[0].split('(')[0] == "rgba" && !ej.isNullOrUndefined(code[4])) {
                rgb.r = code[1]; rgb.g = code[2]; rgb.b = code[3]; rgb.a = code[4];
            }
            else if (code[0].split('(')[0] == "rgb" && ej.isNullOrUndefined(code[4])) {
                rgb.r = code[1]; rgb.g = code[2]; rgb.b = code[3]; rgb.a = this.rgb.a;
                this._inputTag.val("rgba(" + rgb.r + "," + rgb.g + "," + rgb.b + "," + rgb.a + ")");
            }
            else {
                this._inputTag.addClass('e-error');
                this._change = false;
                return false;
            }
            this.rgb.a = parseFloat(rgb.a);
            newValue = this.RGBToHEX(rgb);
            this._HexToHSV(newValue);
            this._inputTag.removeClass("e-error");
            this._change = true;
        },
        _hsvColor: function (code) {
            var hsv = {};
            if (!ej.isNullOrUndefined(code[4])) {
                hsv.h = code[1]; hsv.s = code[2]; hsv.v = code[3]; hsv.a = code[4];
            }
            else {
                this._inputTag.addClass('e-error');
                this._change = false;
                return false;
            }
            this.rgb.a = parseFloat(hsv.a);
            this.rgb = this.HSVToRGB(hsv);
            this._inputTag.removeClass("e-error");
            this._change = true;
        },
        _showAlert: function () {
            var ptr = this;
            this._inputTag.addClass("e-error").attr('aria-invalid', "true").animate({ Opacity: 1 }, 700, null, function () {
                $(ptr._inputTag).removeClass("e-error").attr('aria-invalid', "false");
            });
        },
        _iconClick: function (e) {
            e.preventDefault();
            this._showHidePopup();
            this.wrapper.focus();
        },
        _showHidePopup: function () {
            if (this.model.displayInline) return false;
            if (!this.isPopupOpen)
                this.show();
            else {
                this.hide();
                this.wrapper.focus();
            }
        },
        /**
        * Hides the colorpicker popup, if in opended state.
		* @return jQuery
		
        * @example   
        * &lt;input type="text" id="colorPick"/&gt; <br> 
        * &lt;script&gt;
        * // Create Color picker
        * $('#colorPick').ejColorPicker({ value: "#278787" });
        * // Create ColorPicker instance
		* var colorObj = $("#colorPick").data("ejColorPicker");
		* colorObj.hide(); // hide the colorpicker popup
        * &lt;/script&gt; 
        
        * @example   
        * &lt;input type="text" id="colorPick"/&gt; <br> 
        * &lt;script&gt;
        * // Create Color picker
        * $('#colorPick').ejColorPicker({ value: "#278787" });
         * // hide the colorpicker popup
        * $("#colorPick").ejColorPicker("hide");
        * &lt;/script&gt;       
		
        * @memberof ejColorPicker
	    * @instance
		*/
        hide: function () {
            if (!this.isPopupOpen || this.model.displayInline) return false;
            this.isPopupOpen = false;
            this._dataBind = false;
            this.wrapper.focus();
            this.popupContainer.slideUp(200, "easeOutQuad");
            if (!this.model.displayInline)
                this._off($(document), "mousedown", this._onDocumentClick);
            this._off(this._inputTag, "keydown", this._colorCodeValue);
            this._off(this._inputTag, "blur", this._colorCodeValue);
            this._modelType == "palette" ? this._off($(document), "keydown", this._keyDown) : this._off($(document), "keydown", this._onKeyDown);
            this._trigger("close");
        },
        /**
        * Open the colorpicker popup.
		* @return jQuery
		
        * @example   
        * &lt;input type="text" id="colorPick"/&gt; <br> 
        * &lt;script&gt;
        * // Create Color picker
        * $('#colorPick').ejColorPicker({ value: "#278787" });
        * // Create ColorPicker instance
		* var colorObj = $("#colorPick").data("ejColorPicker");
		* colorObj.show(); // open the colorpicker popup
        * &lt;/script&gt; 
        
        * @example   
        * &lt;input type="text" id="colorPick"/&gt; <br> 
        * &lt;script&gt;
        * // Create Color picker
        * $('#colorPick').ejColorPicker({ value: "#278787" });
         * // open the colorpicker popup
        * $("#colorPick").ejColorPicker("show");
        * &lt;/script&gt;       
		
        * @memberof ejColorPicker
	    * @instance
		*/
        show: function () {
            if (!this.model.displayInline || this.element.is("input")) this.wrapper.focus();
            if (this.isPopupOpen || !this.model.enabled) return false;
            this.isPopupOpen = true;
            if (this.model.modelType == "palette") this._cellSelect();
            this._previousColor = this.value();
            this.popupContainer.children().find('.e-focus').removeClass('e-focus');
            if (!this.model.displayInline) this._setPopupPosition();
            var proxy = this;
            this.popupContainer.slideDown(200, "easeOutQuad", function () {
                proxy.isFocused = true;
                if (!proxy.model.displayInline)
                    proxy._on($(document), "mousedown", proxy._onDocumentClick);
            });
            if (!this._dataBind)
                this._modelType == "palette" ? this._on($(document), "keydown", this._keyDown) : this._on($(document), "keydown", this._onKeyDown);
            this._on(this._inputTag, "keydown", this._colorCodeValue);
            this._on(this._inputTag, "blur", this._colorCodeValue);
            this._trigger("open");
            this._dataBind = true;
        },
        _showBindEvents: function () {
            this._modelType == "palette" ? this._on($(document), "keydown", this._keyDown) : this._on($(document), "keydown", this._onKeyDown);
        },
        _hideUnBindEvents: function () {
            this._off($(document), "keydown");
            this._modelType == "palette" ? this._off($(document), "keydown", this._onKeyDown) : this._off($(document), "keydown", this._keyDown);
        },
        _switchEvents: function () {
            if (this._switch) {
                this._on(this._gradientArea, "mousedown", this._handleMove);
                this._on(this._handleArea, "mousedown", this._handleClick);
                this._on(this._gradientArea, "mousedown", this._handleClick);
            }
            else {
                this._on(this._collection, "mousedown", this._onSelect);
            }
        },
        _unSwitchEvents: function () {
            if (!this._switch) {
                this._off(this._gradientArea, "mousedown", this._handleMove);
                this._off(this._handleArea, "mousedown", this._handleClick);
                this._off(this._gradientArea, "mousedown", this._handleClick);
            }
            else {
                this._off(this._collection, "mousedown", this._onSelect);
            }
        },
        _groupButton: function (e) {
            if ($(e.target).hasClass('e-disable')) return false;
            if ($(e.target).hasClass("e-button")) {
                var element = this._groupTag.find('.e-btn.e-select');
                this._inputTagValue($(e.target));
                this._selectedButton.html() !== "HEX" ? this._inputTag.attr('maxLength', '22') : this._inputTag.attr('maxLength', '7');
                this._groupTag.find('.e-click').removeClass('e-click');
                $(e.target).addClass('e-click');
            }
        },
        _inputTagValue: function (type) {
            if (type.html() == "RGBA") this._inputTag.val(this._formRGBA(this.rgb));
            else if (type.html() == "HEX") this._inputTag.val(this.RGBToHEX(this.rgb));
            else if (type.html() == "HSVA") {
                if (this.model.modelType != "palette")
                    this._inputTag.val("hsva(" + Math.round(this._hsv.h) + "," + Math.round(this._hsv.s) + "," + Math.round(this._hsv.v) + "," + this.rgb.a + ")");
            }
            this._selectedButton = type;
        },
        _bindRecentEvent: function () {
            this._on(this._divTag, "click", this._recentColor);
        },
        _unBindRecentEvent: function () {
            this._off(this._divTag, "click", this._recentColor);
        },
        _handlePlacement: function (prop, value, bool) {
            var hsv = this._hsv;
            hsv[prop] += value * (bool ? 1 : 3);
            if (hsv[prop] < 0) { hsv[prop] = 0; }
            prop === "s" ? this._hsv.s = hsv[prop] : this._hsv.v = hsv[prop];
            this._hsvValue();
            this._changeEvent();
        },
        _onKeyDown: function (e) {
            if (!this.model.enabled) return false;
            var key = e.keyCode, active;
            this._change = true;
            if (e.shiftKey && key == 9) {
                active = document.activeElement;
                $(active).hasClass('e-draghandle') ? this._focusWrapper(e) : "";
            }
            if ((!e.altKey && !e.shiftKey)) {
                switch (key) {
                    case 39:
                        if (e.target.className === "e-draghandle e-color-image" || (this.element.is('input') && e.target == this.wrapper[0])) {
                            e.preventDefault();
                            this._handlePlacement("s", 1, e.ctrlKey);
                        }
                        break;
                    case 38:
                        if (e.target.className === "e-draghandle e-color-image" || (this.element.is('input') && e.target == this.wrapper[0])) {
                            e.preventDefault();
                            this._handlePlacement("v", 1, e.ctrlKey);
                        }
                        break;
                    case 37:
                        if (e.target.className === "e-draghandle e-color-image" || (this.element.is('input') && e.target == this.wrapper[0])) {
                            e.preventDefault();
                            this._handlePlacement("s", -1, e.ctrlKey);
                        }
                        break;
                    case 40:
                        if (e.target.className === "e-draghandle e-color-image" || (this.element.is('input') && e.target == this.wrapper[0])) {
                            e.preventDefault();
                            this._handlePlacement("v", -1, e.ctrlKey);
                        }
                        break;
                    case 13:
                        e.preventDefault();
                        this._buttonClick(e);
                        break;
                    case 27:        // Esc Key
                        e.preventDefault();
                        this.hide();
                        break;
                    case 9:
                        active = document.activeElement;
                        $(active).hasClass('e-colorwidget') ? this._focusPopup(e) : "";
                        break;
                }
            }
        },
        _focusPopup: function (e) {
            $(this._handleArea).focus();
            e.preventDefault();
        },
        _focusWrapper: function (e) {
            $(this.wrapper).focus();
            e.preventDefault();
        },
        _onDocumentClick: function (e) {
            if (!$(e.target).is(this.popupContainer) && !$(e.target).parents(".e-colorpicker").is(this.popupContainer) &&
              !$(e.target).is(this.wrapper) && !$(e.target).parents(".e-colorpickerwidget").is(this.wrapper) && !$(e.target).parents('.e-presetWrapper').is(this._presetContainer)) {
                this.hide();
                this._isFocused = false;
                this.wrapper.removeClass('e-focus');
            }
        },
        _range: function (range, value) {
            if (value === "") return value = 0;
            else if (value > range) return range;
            else return value;
        },
        _hsvValue: function () {
            var colorCode, hsv;
            this._change = true;
            this._hsv.v = this._hsv.v >= 100 ? 100 : this._hsv.v;
            this._hsv.s = this._hsv.s >= 100 ? 100 : this._hsv.s;
            this.hsv = this._hsv;
            this.rgb = this.HSVToRGB(this.hsv);
            this._valueOperation();
            this._inputTagValue(this._selectedButton);
        },
        _formRGB: function (value) {
            if (!ej.isNullOrUndefined(value)) return "rgb(" + value.r + "," + value.g + "," + value.b + ")";
        },
        _formRGBA: function (value) {
            if (!ej.isNullOrUndefined(value)) return "rgba(" + value.r + "," + value.g + "," + value.b + "," + value.a + ")";
        },
        _rgbValue: function (e) {
            var rgbColor, colorCode;
            value = this._HexToRGB(this.value());
            if (!ej.isNullOrUndefined(value)) {
                this.rgb = value;
                this._change = true;
                this.RGBToHSV(this.rgb);
                this._colorSlider.option('value', parseInt(this._hsv.h));
                this._opacity.option('value', this.rgb.a * 100);
            }
            this._valueOperation();
            this._inputTagValue(this._selectedButton);
        },
        _valueOperation: function () {
            this._handlePosition();
            this._alphaGradient(this.value());
            this._inputTag.removeClass("e-error");
            this._updateUI();
        },
        _HexToHSV: function (hex) {
            return this.RGBToHSV(this._HexToRGB(hex));
        },
        _HexToRGB: function (hex) {
            if (!ej.isNullOrUndefined(hex)) {
                var reg = "^#([A-Fa-f0-9]{6}|[A-Fa-f0-9]{3})$", hex, validate = hex.match(reg);
                if (ej.isNullOrUndefined(validate)) { this._change = false; return false; }
                if (validate[1].length == 3)
                    hex = "#" + validate[1][0] + validate[1][0] + validate[1][1] + validate[1][1] + validate[1][2] + validate[1][2];
                this._change = true;
                hex = parseInt(((hex.indexOf('#') > -1) ? hex.substring(1) : hex), 16);
                value = ej.isNullOrUndefined(this.rgb) ? this.model.opacityValue / 100 : this.rgb.a;
                return this.rgb = { r: hex >> 16, g: (hex & 0x00FF00) >> 8, b: (hex & 0x0000FF), a: value };
            }
        },
        RGBToHSV: function (rgb) {
            var hsv = { h: 0, s: 0, v: 0 };
            var min = Math.min(rgb.r, rgb.g, rgb.b);
            var max = Math.max(rgb.r, rgb.g, rgb.b);
            var differ = max - min;
            hsv.v = max;
            hsv.v *= 100 / 255;
            if (differ === 0) {
                this._hsv = hsv;
                return this.HSVToRGB(hsv);
            }
            hsv.s = max != 0 ? 255 * differ / max : 0;
            if (hsv.s != 0) {
                if (rgb.r == max) hsv.h = (rgb.g - rgb.b) / differ;
                else if (rgb.g == max) hsv.h = 2 + (rgb.b - rgb.r) / differ;
                else hsv.h = 4 + (rgb.r - rgb.g) / differ;
            } else hsv.h = -1;
            hsv.h *= 60;
            if (hsv.h < 0) hsv.h += 360;
            hsv.s *= 100 / 255;
            this._hsv = hsv;
            return this.HSVToRGB(hsv);
        },
        /**
        * Convert color value from HSV to RGB
		* @return jQuery
		
        * @example   
        * &lt;input type="text" id="colorPick"/&gt; <br> 
        * &lt;script&gt;
        * // Create Color picker
        * $('#colorPick').ejColorPicker({ value: "#278787" });
        * // Create ColorPicker instance
		* var colorObj = $("#colorPick").data("ejColorPicker");
		* var color=colorObj.HSVToRGB("{h:230,s:98,v:98}"); // Convert color value from HSV to RGB
        * alert("Red="+color.r+", Green="+color.g+", Blue="+color.b);
        * &lt;/script&gt; 
        
        * @example   
        * &lt;input type="text" id="colorPick"/&gt; <br> 
        * &lt;script&gt;
        * // Create Color picker
        * $('#colorPick').ejColorPicker({ value: "#278787" });
        * // Convert color value from HSV to RGB        
        * var color=$("#colorPick").ejColorPicker("HSVToRGB","{h:230,s:98,v:98}");
        * alert("Red="+color.r+", Green="+color.g+", Blue="+color.b);
        * &lt;/script&gt;       
		
        * @memberof ejColorPicker
	    * @instance
		*/
        HSVToRGB: function (hsv) {
            var rgb = {};
            var h = Math.round(hsv.h);
            var s = Math.round(hsv.s * 255 / 100);
            var v = Math.round(hsv.v * 255 / 100);
            if (s == 0) {
                rgb.r = rgb.g = rgb.b = v;
            } else {
                var t1 = v;
                var t2 = (255 - s) * v / 255;
                var t3 = (t1 - t2) * (h % 60) / 60;
                if (h == 360) h = 0;
                if (h < 60) { rgb.r = t1; rgb.b = t2; rgb.g = t2 + t3 }
                else if (h < 120) { rgb.g = t1; rgb.b = t2; rgb.r = t1 - t3 }
                else if (h < 180) { rgb.g = t1; rgb.r = t2; rgb.b = t2 + t3 }
                else if (h < 240) { rgb.b = t1; rgb.r = t2; rgb.g = t1 - t3 }
                else if (h < 300) { rgb.b = t1; rgb.g = t2; rgb.r = t2 + t3 }
                else if (h < 360) { rgb.r = t1; rgb.g = t2; rgb.b = t1 - t3 }
                else { rgb.r = 0; rgb.g = 0; rgb.b = 0 }
            }
            this._hsv = hsv;
            var value = ej.isNullOrUndefined(this.rgb) ? this.model.opacityValue / 100 : this.rgb.a;
            return { r: Math.round(rgb.r), g: Math.round(rgb.g), b: Math.round(rgb.b), a: value }
        },
        _HSVToHex: function (hsv) {
            return this.RGBToHEX(this.HSVToRGB(hsv));
        },
        _toHEX: function (rgb) {
            if (rgb.indexOf("#") !== -1) return rgb;
            rgb = rgb.match(/^rgb\((\d+),\s*(\d+),\s*(\d+)\)$/);
            return "#" + this._hex(rgb[1]) + this._hex(rgb[2]) + this._hex(rgb[3]);
        },

        //Process the rgb object values to color code
        /**
        * Convert color value from RGB to HEX
		* @return jQuery
		
        * @example   
        * &lt;input type="text" id="colorPick"/&gt; <br> 
        * &lt;script&gt;
        * // Create Color picker
        * $('#colorPick').ejColorPicker({ value: "#278787" });
        * // Create ColorPicker instance
		* var colorObj = $("#colorPick").data("ejColorPicker");
		* alert(colorObj.RGBToHEX(colorObj.getColor())); // Convert color value from RGB to HEX
        * &lt;/script&gt; 
        
        * @example   
        * &lt;input type="text" id="colorPick"/&gt; <br> 
        * &lt;script&gt;
        * // Create Color picker
        * $('#colorPick').ejColorPicker({ value: "#278787" });
        * // Create ColorPicker instance
		* var colorObj = $("#colorPick").data("ejColorPicker");
         * // Convert color value from RGB to HEX
        * alert($("#colorPick").ejColorPicker("RGBToHEX",colorObj.getColor()));
        * &lt;/script&gt;       
		
        * @memberof ejColorPicker
	    * @instance
		*/
        RGBToHEX: function (rgb) {
            if (!ej.isNullOrUndefined(rgb))
                return "#" + this._hex(rgb.r) + this._hex(rgb.g) + this._hex(rgb.b);
        },
        _hex: function (x) {
            return ("0" + parseInt(x).toString(16)).slice(-2);
        },
        _colorValue: function (value) {
            this._color = value.indexOf("#") != -1 ? this.hexCodeToRGB(value) : "";
            return "rgb(" + this._color.r + ", " + this._color.g + ", " + this._color.b + ")";
        },
        //Process the color code value to RGB
        /**
        * To Convert color value from hexCode to RGB
		* @return jQuery
		
        * @example   
        * &lt;input type="text" id="colorPick"/&gt; <br> 
        * &lt;script&gt;
        * // Create Color picker
        * $('#colorPick').ejColorPicker({ value: "#278787" });
        * // Create ColorPicker instance
		* var colorObj = $("#colorPick").data("ejColorPicker");
		* var color=colorObj.hexCodeToRGB("#278787"); // Convert color value from hexCode to RGB
        * alert("Red="+color.r+", Green="+color.g+", Blue="+color.b);
        * &lt;/script&gt; 
        
        * @example   
        * &lt;input type="text" id="colorPick"/&gt; <br> 
        * &lt;script&gt;
        * // Create Color picker
        * $('#colorPick').ejColorPicker({ value: "#278787" });
         * // Convert color value from hexCode to RGB
        * var color=$("#colorPick").ejColorPicker("hexCodeToRGB","#278787");
        * alert("Red="+color.r+", Green="+color.g+", Blue="+color.b);
        * &lt;/script&gt;       
		
        * @memberof ejColorPicker
	    * @instance
		*/
        hexCodeToRGB: function (colorCode) {
            var result = /^#?([a-f\d]{2})([a-f\d]{2})([a-f\d]{2})$/i.exec(colorCode);
            return result ? {
                r: parseInt(result[1], 16),
                g: parseInt(result[2], 16),
                b: parseInt(result[3], 16),
                a: this.rgb.a
            } : null;
        },

        _updatePreviewColor: function () {
            if (this._browser.name = "msie" && this._browser.version == "8.0")
                this._previousTag.css({ "background-color": this._formRGB(this.rgb), "filter": "alpha(opacity=" + this.rgb.a * 100 + ")" });
            else
                this._previousTag.css({ "background-color": this._formRGBA(this.rgb) });
            this._oldValue = this.rgb;
        },
        _updateUI: function () {
            var value = this._hsv;
            if (this._switch) {
                var hsv = { h: this._hsv.h, s: 100, v: 100 };
                this._gradientArea.css({ "background-color": this._colorValue(this._HSVToHex(hsv)) });
            } else this._cellSelect();
            if (this._browser.name = "msie" && this._browser.version == "8.0") {
                this._currentTag.css({ "background-color": this._formRGB(this.rgb), "filter": "alpha(opacity=" + this.rgb.a * 100 + ")" });
                this._previousTag.css({ "background-color": this._formRGB(this._oldValue), "filter": "alpha(opacity=" + this.rgb.a + ")" });
            }
            else {
                this._currentTag.css({ "background-color": this._formRGBA(this.rgb) });
                this._previousTag.css({ "background-color": this._formRGBA(this._oldValue) });
                this.popupList.find("#" + this._id + "alpha").children(".e-handle").css({ "background": this._formRGBA(this.rgb) });
            }
            this._hsv = value;
        },

        _targetFocus: function (e) {
            e.preventDefault();
            if (!this._isFocused) {
                this._isFocused = true;
                this.wrapper.addClass("e-focus");
            }
        },
        _targetBlur: function (e) {
            this._isFocused = false;
            if (!this.isPopupOpen) this.wrapper.removeClass("e-focus");
        },
        _switchModel: function () {
            this.value(this.RGBToHEX(this.rgb));
            var proxy = this;
            if (this._modelType == "palette") {
                this._modelType = "picker";
                this._switcher.removeClass("e-paletteModel").addClass("e-pickerModel");
                this.PaletteWrapper.fadeOut(300, function () {
                    proxy._presetTag.parents('.e-split.e-widget').addClass('e-hide');
                    proxy._gradient.fadeIn(300);
                });
                this._switch = true;
                this._rgbValue();
                this._hueGradient();
                this._templateWrapper.find('#' + this._id + "hsv").ejButton("enable");
            } else {
                this._modelType = "palette";
                this._switcher.removeClass("e-pickerModel").addClass("e-paletteModel");
                this._switch = false;
                this._gradient.fadeOut(300, function () {
                    proxy._presetTag.parents('.e-split.e-widget').removeClass('e-hide');
                    if (ej.isNullOrUndefined(proxy.PaletteWrapper)) {
                        proxy.PaletteWrapper = proxy._layoutType(proxy.model.palette);
                        proxy._splitObj.option('prefixIcon', 'e-color-image e-basic');
                        proxy.popupList.prepend(proxy.PaletteWrapper);
                    }
                    proxy.PaletteWrapper.fadeIn(300);
                });
                this._cellSelect();
                this._disableHSVButton();
            }
            if (!this.model.displayInline || this.element.is(":input")) this.wrapper.focus();
            this._switchEvents();
            this._unSwitchEvents();
            if (this.isPopupOpen) {
                this._hideUnBindEvents();
                this._showBindEvents();
            }
        },
        _disableHSVButton: function () {
            if ($(this._groupTag.find('.e-click')).attr('id') == this._id + "hsv") {
                this._inputTagValue(this._templateWrapper.find("#" + this._id + "rgba"));
                this._templateWrapper.find("#" + this._id + "rgba").addClass('e-click');
                this._templateWrapper.find('#' + this._id + "hsv").removeClass('e-click');
            }
            this._templateWrapper.find('#' + this._id + "hsv").ejButton("disable");
        },
        //palette Block
        _cellSelect: function () {
            var code, element, proxy = this;
            this._removeClass();
            this._collection.find('.e-item').each(function () {
                code = $(this).css("background-color");
                if (code && code.replace(/ /g, '') === proxy._formRGB(proxy.rgb)) {
                    element = this;
                }
            });
            $(element).addClass("e-state-selected").attr("aria-selected", true);
        },
        _removeClass: function () {
            this._collection.find('.e-item').removeClass('e-state-selected').removeAttr("aria-selected");
        },
        _position: function (items, element, columns) {
            items = Array.prototype.slice.call(items);
            var n = items.length,
            index = items.indexOf(element);
            if (index < 0) return columns < 0 ? items[n - 1] : items[0];
            index += columns;
            return items[index < 0 ? index += n : index %= n];
        },
        _onSelect: function (e) {
            this._collection.find('.e-item').removeClass("e-state-selected").removeAttr('aria-selected');
            if (e.target.style.backgroundColor != "") {
                this._HexToRGB(this._toHEX(e.target.style.backgroundColor));
                this._updateUI();
                this._inputTagValue(this._selectedButton);
                if (!this.model.displayInline || this.element.is("input")) this.wrapper.focus();
            }
            this._changeEvent();
            e.preventDefault();
        },
        _keyDown: function (e) {
            this._change = true;
            if (!this.model.enabled) return false;
            var selected = "",
                key = e.keyCode,
                items = this._collection.find('.e-item'),
                element = items.filter(".e-state-selected").get(0),
                columnSize = this.model.palette === "webSafe" ? 18 : this.model.columns;
            if (!e.altKey && (key == 37 || key == 38 || key == 39 || key == 40) && (e.target.className !== "e-color-code"))
                this._removeClass();
            if (e.keyCode == 9) return false;
            switch (!e.altKey && key) {
                case 40:
                    if (e.target.className !== "e-color-code") {
                        e.preventDefault();
                        selected = this._position(items, element, columnSize);
                    }
                    break;
                case 37:
                    if (e.target.className !== "e-color-code") {
                        e.preventDefault();
                        selected = this._position(items, element, -1);
                    }
                    break;
                case 38:
                    if (e.target.className !== "e-color-code") {
                        e.preventDefault();
                        selected = this._position(items, element, -columnSize);
                    }
                    break;
                case 39:
                    if (e.target.className !== "e-color-code") {
                        e.preventDefault();
                        selected = this._position(items, element, 1);
                    }
                    break;
                case 13: //Enter Key                    
                    this._collection.find('.e-item').removeClass('e-state-selected').removeAttr("aria-selected");
                    this._buttonClick(e);
                    this._updateUI();
                    this.hide();
                    if (this.model.displayInline || this.element.is("input")) this.wrapper.focus();
                    break;
                case 27:  //Esc Key
                    this.hide();
                    if (this.model.displayInline || this.element.is("input")) this.wrapper.focus();
                    break;
            }
            if (selected) {
                $(selected).addClass('e-state-selected').attr("aria-selected", true);
                this._currentTag.css({ "background-color": this._formRGB(this._HexToRGB(selected.attributes.value.value)) });
                this._inputTagValue(this._selectedButton);
                this._changeEvent();
            }
        },
    })
    /**
    * Enum for ColorPicker Palette	 
    * @enum {String}
    * @global 
    */
    ej.Palette = {
        /**  Represents the basic Palette. This is default Type*/
        BasicPalette: "basicPalette",
        /**  Represents the custom Palette. User will customize the palette*/
        CustomPalette: "customPalette",
    }
    /**
    * Enum for ColorPicker ModelType	 
    * @enum {String}
    * @global 
    */
    ej.ModelType = {
        /**  Color picker is a Default type*/
        Default: "default",
        /**  Represents Palette Type*/
        Palette: "palette",
        /** Represents Picker Type */
        Picker: "picker"
    }
    ej.Colors = {
        basic: ["ffffff", "facbcb", "fccb98", "faf39a", "fbf8cd", "a6d38b", "aadee8", "d1ecf2", "cdcae5", "eecde1", "cccbcb", "f16667", "f69668", "f8ee6b", "f7ec37", "89c987", "75cddd", "8bd3e1", "9494c8", "9494c8", "b3b2b3", "ec2024", "f7971d", "ffcb67", "f5ea14", "74bf44", "69c8c9", "46c7f4", "6666ad", "b76cab", "676767", "971b1e", "ca6828", "ca9732", "979937", "0d9948", "339898", "4857a6", "62449a", "973794", "000000", "2f1110", "973620", "663433", "343416", "183319", "023334", "22205f", "3b2f8d", "310e31"],
        monoChrome: ["ffffff", "e3e3e3", "c6c6c6", "aaaaaa", "8e8e8e", "717171", "555555", "393939", "1c1c1c", "000000", "f9e6e7", "f4d0d2", "efbabc", "e9a4a7", "e48e92", "df787c", "da6267", "d44c52", "cf363c", "ca2027", "fff4ca", "ffeb9e", "fff0b4", "ffeb9e", "ffe788", "ffe272", "ffd947", "ffd531", "ffd01b", "ffcc05", "e4f4eb", "ccead9", "b4e0c7", "9cd6b5", "84cca3", "6dc190", "55b77e", "3dad6c", "25a35a", "0d9948", "e8f4f4", "d6e3eb", "c4d1e3", "b3c0da", "a1aed1", "8f9dc9", "7d8bc0", "6c7ab7", "5a68af", "4857a6"],
        flat: ["7477b8", "488bca", "18b1d4", "1db369", "78c162", "acc063", "ffe84e", "f6b757", "f79853", "ed6346", "E87F3D", "E4C45D", "B7A575", "999999", "67809F", "002228", "00A578", "F9A41F", "F3770B", "D7401B", "FFCB36", "82CC2C", "36B595", "6370AD", "D4264E", "004D8E", "22A04B", "F3A414", "C77405", "F3420B", "1ABC9C", "3498DB", "9B59B6", "E67E22", "E74C3C", "3A738A", "EBD9A7", "89AD78", "FF766D", "C76160", "BF3542", "CDC5BA", "EBE3D6", "3C3C3C", "2E2E2E", "77A7FB", "E57368", "FBCB43", "34B67A", "FFFFFF"],
        seaWolf: ["0EEAFF", "15A9FA", "1B76FF", "1C3FFD", "2C1DFF", "0B3C4C", "0E5066", "13647F", "127899", "1A8BB2", "74B8E8", "659EBB", "3C9FFF", "26466F", "2472FF", "0069A4", "009BF2", "004165", "49A0B4", "274C5F", "000000", "7A5848", "E0A088", "F9DEC6", "3A2A22", "DC3522", "D9CB9E", "374140", "2A2C2B", "1E1E20", "CB3937", "FE6B2C", "654E44", "6DD16F", "70FE2C", "275673", "4681A6", "FDDEC9", "F22816", "400101", "071C2F", "388494", "E6A934", "F3DB5F", "534329", "206956", "47683B", "E1BFA6", "BF7950", "903932"],
        webColors: ["0066aa", "00bbdd", "338800", "77bb00", "ffcc99", "990c0c", "0303c9", "336699", "669933", "cccccc", "EEEEEE", "E7C36F", "F7B230", "E35B20", "000033", "7D7A74", "BD524A", "FCB200", "8CFCC2", "2ACD6B", "666666", "666553", "FFFEEC", "B2B2A4", "AAA4B2", "9CA5E3", "5A668C", "BBA469", "CFC295", "FFFFFF", "DBBF56", "2E94B3", "808080", "E96656", "14A168", "DE185B", "D8806F", "DBE186", "D8CC63", "DCC527", "4E6C89", "E2BDAD", "EC6053", "81BBAD", "DFCDA5", "453394", "66398A", "313E7D", "336694", "788E91"],
        sandy: ["c0a986", "ecd9c3", "dfc79b", "f6d58e", "ecdaad", "fff3e0", "7f6b4a", "ffd694", "7f7a70", "ccac76", "E6E2AF", "A79A71", "EFECCA", "806F4C", "2F2F2E", "997F1A", "CCB65F", "FFD291", "6B674A", "635F3A", "7F693A", "FFEBC1", "FFD275", "7F7561", "CCA85E", "D29854", "4A4034", "C9AD8D", "4A351D", "968169", "E6E39F", "9A9757", "FFFDC9", "94909A", "E2E0E6", "960010", "EB1517", "CD7C29", "9A571C", "1F7A94", "7F6826", "7F724C", "FFE499", "FFD04C", "CCA63D", "FFA669", "92FFB6", "FFF352", "E8C269", "D7E8CB"],
        pinkShades: ["F6B1C3", "F0788C", "DE264C", "BC0D35", "A20D1E", "E12256", "BB1C48", "7B132F", "3B0917", "FA2660", "FFB7B5", "9A423F", "FF6D68", "BB5855", "CC5753", "E88161", "D66C60", "C2646E", "996072", "705262", "FFA1BD", "FF8FB7", "FF82AE", "E9719B", "CC6882", "F250C7", "BF1774", "BF2696", "AC60AA", "BB90C5", "BF1553", "F20775", "F2F2F2", "A68F63", "261414", "FFFFFF", "7F3443", "FF6887", "7F3443", "CC536C", "D06AA9", "E65F41", "650017", "BC1620", "FA427E", "3B1132", "84476E", "B83D65", "E6E0E8", "FF6EE8"],
        misty: ["5C7A84", "3D5372", "7C9196", "50748A", "ADBFBF", "010735", "052159", "194073", "376B8C", "FFFFFF", "985999", "C811CC", "892EFF", "FF6852", "DBA211", "0A0D0C", "85A67C", "46593E", "BBD9AD", "202616", "BF8E63", "734327", "A66C4B", "593A2F", "BFBFBF", "8DB0B6", "1B778A", "F46C1B", "881801", "192129", "81808C", "ABAABF", "0C0E09", "6A7366", "37402F", "5D6663", "84867B", "A4A66A", "BABBB1", "20211C", "6B9695", "646E8C", "6B8196", "61787F", "648C80", "8E9FBA", "89A8C8", "799ED1", "7FAEE7", "849EBD"],
        citrus: ["FAEA41", "E7F03E", "E3C647", "FAC541", "F0AB3E", "CCCA1F", "FFF300", "FFCB0D", "FF9500", "804A00", "6A692A", "FFFCA0", "FFFFFF", "CF664E", "EFAC66", "EFF299", "F2DC6D", "F29727", "F2600C", "592202", "214001", "4F7302", "1A2601", "BCD97E", "C0D904", "AAFEFF", "359D6D", "E5FF45", "65FCCF", "ABDC4B", "42B200", "C6FF00", "F2E304", "FFB200", "FF8600", "52EC04", "04E206", "94D507", "ECE404", "E2C904", "DA321C", "FF7913", "FBD028", "C0D725", "9FC131", "547312", "ADBF26", "DEB329", "F1DB47", "E08214"],
        vintage: ["684132", "fdbe30", "eaac21", "87783c", "3e4028", "ffc706", "cd5648", "5bafa9", "828282", "363636", "424862", "fb9a63", "bfc4d5", "f6fbf4", "febc98", "657050", "FCF082", "D8D98B", "A2AB80", "4D584A", "5ADED4", "4DAAAB", "26596A", "163342", "6C98A1", "010A26", "28403B", "557359", "AEBF8A", "C7D9AD", "AFFCCB", "CB4243", "D2997E", "36857E", "4AC6BB", "28394B", "191313", "AF0A18", "DC373D", "122438", "43734A", "A6A26D", "D9B448", "BF8C2C", "734002", "26010F", "866F53", "ACBD91", "7BAB87", "546859"],
        moonLight: ["241D37", "2A233D", "322B45", "362F49", "D4BA73", "261225", "592040", "8C3063", "A64985", "73345D", "A3C8FF", "85B6FF", "000040", "213190", "050859", "FFFFFF", "6AAED9", "4184BF", "224573", "A3C8FF", "CFC496", "B3B391", "889486", "61797B", "366577", "123340", "436E73", "7B8C61", "D7D996", "F0EBB4", "CFC496", "B3B391", "889486", "61797B", "366577", "123340", "436E73", "7B8C61", "D7D996", "F0EBB4", "341F36", "D9B5E0", "9889AB", "4D4E66", "1B2129", "5CBBE3", "FCF1BC", "5C8182", "383A47", "B4F257"],
        candyCrush: ["0779f4", "30da00", "fb8006", "f9d802", "a71df7", "f70200", "fd49ae", "682e07", "9b2424", "5e7693", "F9AB3B", "EF5627", "FF0000", "00A398", "803C2C", "DE5711", "FFF026", "FF0048", "14A0CC", "00B229", "FFFFBE", "F7CD99", "FF77A1", "9886E8", "97CACB", "EAEDE5", "FFD127", "FF870C", "EC4610", "9A1900", "993460", "CC1464", "C300FF", "FFFFBC", "CCB914", "FFFEE2", "B24C5F", "FF274F", "0A94CC", "679DB2", "C2FFE6", "16B271", "5FFFBC", "B2442F", "FFA190", "E89359", "FFFB75", "F36EFF", "5999E8", "73EB86"],
    }
})(jQuery, Syncfusion);;
/*!
*  filename: ej.captcha.js
*  version : 12.3
*  Copyright Syncfusion Inc. 2001 - 2013. All rights reserved.
*  Use of this code is subject to the terms of our license.
*  A copy of the current license can be obtained at any time by e-mailing
*  licensing@syncfusion.com. Any infringement will be prosecuted under
*  applicable laws. 
*/

/**
* @fileOverview Plugin to style the Html Captcha elements
* @copyright Copyright Syncfusion Inc. 2001 - 2013. All rights reserved.
*  Use of this code is subject to the terms of our license.
*  A copy of the current license can be obtained at any time by e-mailing
*  licensing@syncfusion.com. Any infringement will be prosecuted under
*  applicable laws. 
* @version 12.3 
* @author <a href="mailto:licensing@syncfusion.com">Syncfusion Inc</a>
*/

 (function ($, ej, undefined) {
	/**
    * @namespace ej
	* @classdesc Custom Design for Html Captcha control.
	* @class ejCaptcha
	* @param {object} options - settings for Captcha
	* @requires jQuery
	* @requires ej.core.js
    * @requires ej.captcha.js
	* @requires ej.button.js
	* @example 
	* &lt;captcha id="captcha1"&gt;Captcha&lt;/Captcha&gt; <br> 
	* &lt;script&gt;
	* // Create Captcha
    * $('#Captcha1').ejCaptcha(); 	
	* &lt;/script&gt;
	*/
       ej.widget("ejCaptcha", "ej.Captcha", {
        // widget element will be automatically set in this
        element: null,
        // user defined model will be automatically set in this
        model: null,
        validTags: ["div"],
        _setFirst: false,
        //Root Css Class
        _rootCSS: "e-captcha",

        // default model
        defaults: {
            /// <summary>This Contains default property of Captcha </summary>
			/**		
			* Specifies the target input element that will verify the Captcha.	
			* @default null
			* @type {string}
			* @example 
			* &lt;div id="captcha1"&gt;&lt;/div&gt; <br> 
			* &lt;script&gt;
			* //To set target input API value during initialization  
			* 	$("#captcha1").ejCaptcha({  targetInput: "input1"});	
			* &lt;/script&gt;
			* @memberof ejCaptcha
			* @instance
			*/	
            targetInput: "",
			/**		
			* Specifies the target button of the Captcha to validate the entered text and captcha text.	
			* @default null
			* @type {string}
			* @example 
			* &&lt;div id="captcha1"&gt;&lt;/div&gt; <br> 
			* &lt;button id="button1"&gt;Submit&lt;/button&gt; <br>
			* &lt;script&gt;
			* //To set target button API value during initialization  
			* 	$("#captcha1").ejCaptcha({  targetButton: "button1"});	
			* &lt;/script&gt;
			* @memberof ejCaptcha
			* @instance
			*/	
            targetButton:"",
			/**		
			* Specifies the height of the Captcha.
			* @default 50
			* @type {number}
			* @example 
			* &&lt;div id="captcha1"&gt;&lt;/div&gt; <br> 
			* &lt;script&gt;
			* //To set height API value during initialization  
			* 	$("#captcha1").ejCaptcha({  height: 50});	
			* &lt;/script&gt;			
			* @memberof ejCaptcha
			* @instance
			*/
            height :50,
			/**		
			* Specifies the width of the Captcha.
			* @default 150
			* @type {number}
			* @example 
			* &&lt;div id="captcha1"&gt;&lt;/div&gt; <br> 
			* &lt;script&gt;
			* //To set width API value during initialization  
			* 	$("#captcha1").ejCaptcha({  width: 150});	
			* &lt;/script&gt;				
			* @memberof ejCaptcha
			* @instance
			*/
            width :150,
			/**		
			* Specifies the character set of the Captcha that will be used to generate captcha text randomly.
			* @default "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789"
			* @type {string}
			* @example 
			* &&lt;div id="captcha1"&gt;&lt;/div&gt; <br> 
			* &lt;script&gt;
			* //To set character set API value during initialization  
			* 	$("#captcha1").ejCaptcha({  characterSet: "ABCD1234"});	
			* &lt;/script&gt;				
			* @memberof ejCaptcha
			* @instance
			*/
            characterSet : "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789",
			/**		
			* Specifies the maximum number of characters used in the Captcha.
			* @default 8
			* @type {number}
			* @example 
			* &&lt;div id="captcha1"&gt;&lt;/div&gt; <br> 
			* &lt;script&gt;
			* //To set maximum length API value during initialization  
			* 	$("#captcha1").ejCaptcha({  maximumLength: 8});	
			* &lt;/script&gt;				
			* @memberof ejCaptcha
			* @instance
			*/
            maximumLength :8,  
			/**		
			* Specifies the minimum number of characters used in the Captcha.
			* @default 4
			* @type {number}
			* @example 
			* &&lt;div id="captcha1"&gt;&lt;/div&gt; <br> 
			* &lt;script&gt;
			* //To set minimum length API value during initialization  
			* 	$("#captcha1").ejCaptcha({  minimumLength: 6});	
			* &lt;/script&gt;				
			* @memberof ejCaptcha
			* @instance
			*/
            minimumLength:4,
			/**		
			* Specifies the case sensitivity for the characters typed in the Captcha.
			* @default true
			* @type {boolean}
			* @example 
			* &&lt;div id="captcha1"&gt;&lt;/div&gt; <br> 
			* &lt;script&gt;
			* //To set case sensitivity API value during initialization  
			* 	$("#captcha1").ejCaptcha({  enableCaseSensitivity: true});	
			* &lt;/script&gt;				
			* @memberof ejCaptcha
			* @instance
			*/
            enableCaseSensitivity :true,
			/**		
			* Set the Captcha validation automatically.
			* @default false
			* @type {boolean}
			* @example 
			* &&lt;div id="captcha1"&gt;&lt;/div&gt; <br> 
			* &lt;script&gt;
			* //To set auto validation API value during initialization  
			* 	$("#captcha1").ejCaptcha({  enableAutoValidation: true});	
			* &lt;/script&gt;				
			* @memberof ejCaptcha
			* @instance
			*/
            enableAutoValidation:false,
			/**		
			* Specifies the error message to be displayed when the Captcha mismatch.
			* @default null
			* @type {string}
			* @example 
			* &&lt;div id="captcha1"&gt;&lt;/div&gt; <br> 
			* &lt;script&gt;
			* //To set error message API value during initialization  
			* 	$("#captcha1").ejCaptcha({  customErrorMessage: "InValid Captcha"});	
			* &lt;/script&gt;				
			* @memberof ejCaptcha
			* @instance
			*/			
            customErrorMessage: "",
			/**		
			* Sets the Captcha with audio support, that enables to dictate the captcha text.
			* @default false
			* @type {boolean}
			* @example 
			* &&lt;div id="captcha1"&gt;&lt;/div&gt; <br> 
			* &lt;script&gt;
			* //To set enable audio API value during initialization  
			* 	$("#captcha1").ejCaptcha({  enableAudio: true});	
			* &lt;/script&gt;				
			* @memberof ejCaptcha
			* @instance
			*/
            enableAudio: false,
			/**		
			* Sets the Captcha with a refresh button.
			* @default false
			* @type {boolean}
			* @example 
			* &&lt;div id="captcha1"&gt;&lt;/div&gt; <br> 
			* &lt;script&gt;
			* //To set enable refresh API value during initialization  
			* 	$("#captcha1").ejCaptcha({  enableRefreshImage: true});	
			* &lt;/script&gt;				
			* @memberof ejCaptcha
			* @instance
			*/
            enableRefreshImage: false,
			/**		
			* Sets the Captcha direction as right to left alignment.
			* @default false
			* @type {boolean}
			* @example 
			* &&lt;div id="captcha1"&gt;&lt;/div&gt; <br> 
			* &lt;script&gt;
			* //To set enable RTL API value during initialization  
			* 	$("#captcha1").ejCaptcha({  enableRTL: true});	
			* &lt;/script&gt;				
			* @memberof ejCaptcha
			* @instance
			*/
            enableRTL:false,
			/**		
			* Specifies the method to map values to Captcha.
			* @default null
			* @type {string}
			* @example 
			* &&lt;div id="captcha1"&gt;&lt;/div&gt; <br> 
			* &lt;script&gt;
			* //To set request mapper API value during initialization  
			* 	$("#captcha1").ejCaptcha({  requestMapper: "GetCurrentItem"});	
			* &lt;/script&gt;				
			* @memberof ejCaptcha
			* @instance
			*/
            requestMapper:"",
			/**		
			* Fires when captch refresh begins.
			* @event
			* @name ejCaptcha#refreshBegin 	
			* @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
			* @param {object}  argument.model returns the Captcha model
			* @param {string}  argument.type returns the name of the event		  
			* @example
			* &&lt;div id="captcha1"&gt;&lt;/div&gt; <br> 
			* &lt;script&gt;
			* //Refresh begin event of Captcha control 
			* 	$("#captcha1").ejCaptcha({  
				refreshBegin: function(args) {}
				});	
			* &lt;/script&gt;				
			* @memberof ejCaptcha
			* @instance
			*/
            refreshBegin:"",
			/**		
			* Fires after captch refresh succeeded.
			* @event
			* @name ejCaptcha#refreshSuccess 	
			* @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
			* @param {object}  argument.model returns the Captcha model
			* @param {string}  argument.type returns the name of the event		  
			* @example
			* &&lt;div id="captcha1"&gt;&lt;/div&gt; <br> 
			* &lt;script&gt;
			* //Refresh success event of Captcha control 
			* 	$("#captcha1").ejCaptcha({  
				refreshSuccess: function(args) {}
				});	
			* &lt;/script&gt;				
			* @memberof ejCaptcha
			* @instance
			*/
            refreshSuccess:"",
			/**		
			* Fires when captch refresh fails to load.
			* @event
			* @name ejCaptcha#refreshFailure 	
			* @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
			* @param {object}  argument.model returns the Captcha model
			* @param {string}  argument.type returns the name of the event		  
			* @example
			* &&lt;div id="captcha1"&gt;&lt;/div&gt; <br> 
			* &lt;script&gt;
			* //Refresh failure event of Captcha control 
			* 	$("#captcha1").ejCaptcha({  
				refreshFailure: function(args) {}
				});	
			* &lt;/script&gt;				
			* @memberof ejCaptcha
			* @instance
			*/
            refreshFailure:"",
			/**		
			* Fires after captch refresh completed.
			* @event
			* @name ejCaptcha#refreshComplete 	
			* @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
			* @param {object}  argument.model returns the Captcha model
			* @param {string}  argument.type returns the name of the event		  
			* @example
			* &&lt;div id="captcha1"&gt;&lt;/div&gt; <br> 
			* &lt;script&gt;
			* //Refresh complete event of Captcha control 
			* 	$("#captcha1").ejCaptcha({  
				refreshComplete: function(args) {}
				});	
			* &lt;/script&gt;				
			* @memberof ejCaptcha
			* @instance
			*/
            refreshComplete: "",
			/**		
			* Specifies the method with values to be mapped in the Captcha.
			* @default null
			* @type {string}
			* @example 
			* &&lt;div id="captcha1"&gt;&lt;/div&gt; <br> 
			* &lt;script&gt;
			* //To set mapper API value during initialization  
			* 	$("#captcha1").ejCaptcha({  mapper: "GetCurrentItem});	
			* &lt;/script&gt;				
			* @memberof ejCaptcha
			* @instance
			*/
            mapper:"",
        },
        /**	
    * Specify the data types for default properties 
    * @private
    */
        //Data Types
        dataTypes: {
            targetInput: "string",
            targetButton:"string",
            height :"number",
            width :"number",
            characterSet : "string",
            maximumLength :"number",             
            minimumLength:"number",            
            enableCaseSensitivity :"boolean",
            enableAutoValidation :"boolean",            
            customErrorMessage: "string",
            requestMapper: "string",
            enableAudio: "boolean",
            enableRefreshImage:"boolean",
            enableRTL:"boolean",            
            mapper:"string",
        },
        /**
* Create the Captcha widget
* @private
*/
        _init: function () {
            /// <summary>This will initialize captcha </summary>
            this._initialize();
            this._wireEvents();
        },       
         
        /*----------------------- Private Methods-----------------------------------------*/
        //initialize the control
        _initialize: function () {         
        $("#"+ this._id + "_RefreshButton").ejButton({size: "normal", showRoundedCorner : true, contentType: "imageonly", prefixIcon: "e-captcha refreshicon",type:"button"});
        $("#"+ this._id + "_PlayAudio").ejButton({size: "normal", showRoundedCorner : true, contentType: "imageonly", prefixIcon: "e-captcha audioicon",type:"button"});
            this._control = $("#" + this._id).get(0);
			if(!this.model.targetInput)
            this.model.targetInput = this._id + "_ValidText";
            this._target  = $("#"+ this.model.targetInput).get(0);
            this._captchaImage = $("#"+ this._id + "_CaptchaImage").get(0);
            this._message = $("#"+ this._id + "_CaptchaMessage").get(0);
            this._refreshButton = $("#"+ this._id + "_RefreshButton").get(0);
            this._form = $(this._target).parents('form')[0];				
            this._audioObject = $("#"+ this._id + "_AudioObject").get(0);
            this._audioButton = $("#"+ this._id + "_PlayAudio").get(0);
            this._audioPluginObject = null;
            this._audioType = "audio/wav";                                       
            this._submitButton=$("#"+this.model.targetButton).get(0);
            if(!this._submitButton)
            this._submitButton=$("[id$="+this.model.targetButton+"]").get(0);
            this._hiddenField = $("#"+ this._id + "_Hidden").get(0);
            
            if (this._audioObject) {
                if (!this._isSupportAudio(this._audioType)) {
                    this._appendAudioPlugin();
                }
            }
        },
        //Wiring Events
        /**
* Wiring the events to Captcha control		
* @private
*/
        _wireEvents: function (val) {            
            this._FormSubmitDelegate = $.proxy(this.formSubmit, this);
            this._CaptchaRefreshDelegate = $.proxy(this.refresh, this);
            this._PlayAudioDelegate = $.proxy(this.playAudio, this);

            if (this._audioButton)
                $(this._audioButton).on('click', this._PlayAudioDelegate);
            if (this._submitButton)
                $(this._submitButton).on('click', this._FormSubmitDelegate);
            if (this._refreshButton)
                $(this._refreshButton).on('click', this._CaptchaRefreshDelegate);
        },  

        //Remove Handlers
        _unwireEvents: function () {
            if (this._audioButton)
                $(this._audioButton).unbind('click', this._PlayAudioDelegate);
            if (this._submitButton)
                $(this._submitButton).unbind('click', this._FormSubmitDelegate);
            if (this._refreshButton)
                $(this._refreshButton).unbind('click', this._CaptchaRefreshDelegate);
        },

         //Play Audio
            playAudio : function(){

                try{
                    if (this._audioPluginObject) {
                        this._audioPluginObject.Play();
                    } else {
                        if (this._audioObject && this._audioObject.play) {                           
                            this._audioObject.play();
                        }
                    }
                } catch (e) { }
            },

            //Is Support audio
            _isSupportAudio: function (type) {
                if (!document.createElement("audio").canPlayType) {
                    return false;
                } else {
                    return document.createElement("audio").canPlayType(type).match(/maybe|probably/i) ? true : false;
                }
            },
            //Append audio plugin 
            _appendAudioPlugin: function () {
                $(this._audioPluginObject).remove();
                var obj = this._createAudioPluginObject();
                this._audioObject.parentNode.appendChild(obj);
                this._audioPluginObject = $("#"+this._id + "_AudioPlugin").get(0);
            },
            //Create Audio Plugin Object
            _createAudioPluginObject : function(){
                var a = document.createElement("embed");
                a.setAttribute("id", this._id + "_AudioPlugin");
                a.setAttribute("src", this.model.audioUrl);
                a.setAttribute("name", "AudioPlugin");
                a.setAttribute("enablejavascript", "true");
                a.setAttribute("type", "audio/wav");
                a.setAttribute("autostart", "false");
                a.setAttribute("pluginspage", "http://www.apple.com/quicktime/download/");
                a.style.top = 0;
                a.style.left = 0;
                a.style.width = "0px";
                a.style.height = "0px";
                a.style.position = "absolute";
                return a;
            },

            //On ajax Success
            _onSuccess: function (captchaResult) {
                var jsonCaptcha;               
                    jsonCaptcha = JSON.parse(captchaResult);
                this._validSuccess(jsonCaptcha);               
            },

             _onSuccessAsp: function (captchaResult) {
                var jsonCaptcha;               
                    jsonCaptcha = JSON.parse(captchaResult.d);

                    this._validSuccess(jsonCaptcha);              
            },

            //To display validation message
            _displayMessages: function () {
                if (this.model.isValid)
                {
                    $("#" + this._id + "_CaptchaMessage").html("");
                }
                else
				{      
                    $("#" + this._id + "_CaptchaMessage").html(this.model.customErrorMessage);
                    $("#" + this._id + "_ValidText").addClass("error");
                }

            },
            _validSuccess: function(jsonCaptcha)
            {
			try {
                    if (this._audioPluginObject) {
                        this._audioPluginObject.Pause();
                    } else if (this._audioObject && this._audioObject.pause) {
                        this._audioObject.pause();
                    }
                } catch (e) { }

                this._captchaImage.src = jsonCaptcha.NewChallenge;
                if(this._audioObject)
                this._audioObject.src = jsonCaptcha.AudioLink;

                this.model.audioURL=jsonCaptcha.AudioLink;

                if (this._audioPluginObject) {
                    this._appendAudioPlugin();
                }

                if (jsonCaptcha.Validation == "false")
                    this.model.isValid = false;
                else
                    this.model.isValid = true;
                if (this.model.enableAutoValidation)
                    this._displayMessages(); 
				if($("#"+this._id + "_ValidText"))
                   $("#"+this._id + "_ValidText").val("");
                   this.model.encryptedCode=jsonCaptcha.Script;
                $(this._hiddenField).val(this.model.encryptedCode);
            },
           _refreshSuccess: function(jsonCaptcha)
           {
              try {
                    if (this._audioPluginObject) {
                        this._audioPluginObject.Pause();
                    } else if (this._audioObject && this._audioObject.pause) {
                        this._audioObject.pause();
                    }
                } catch (e) { }

                this._captchaImage.src = jsonCaptcha.NewChallenge;
                if (this._audioObject) {
                    this._audioObject.src = jsonCaptcha.AudioLink;
                }
                this.model.audioUrl=jsonCaptcha.AudioLink;

                if (this._audioPluginObject) {
                    this._appendAudioPlugin();
                }
                this.model.encryptedCode=jsonCaptcha.EncryptedText;
                $(this._hiddenField).val(jsonCaptcha.EncryptedText);    
           },
            //On captcha refresh ajax Success
            _onRefreshSuccess: function (captchaRefresh) {
                var jsonCaptcha;
                jsonCaptcha = JSON.parse(captchaRefresh);
                  this._refreshSuccess(jsonCaptcha)           

            },
            _onRefreshSuccessAsp: function (captchaRefresh)
            {
            var jsonCaptcha;
                jsonCaptcha = JSON.parse(captchaRefresh.d);
                this._refreshSuccess(jsonCaptcha)             
            },
  
            /*-----------------------Event Handlers -----------------------------------------*/
            //Hanldes the form submit event
            formSubmit: function(event) {
                var proxy = this;
                var data = {
                    Height: this.model.height,
                    Width: this.model.width,
                    CharacterSet: this.model.characterSet,
                    MaximumLength: this.model.maximumLength,                     
                    MinimumLength: this.model.minimumLength,
                    CaseSensitive: this.model.enableCaseSensitivity,
                    EnableRefreshImage: this.model.enableRefreshImage,
                    EnableAudio: this.model.enableAudio,
                    ResponseText: $(this._target).val(),
                    EncrytedText: this.model.encryptedCode,
                    Id: this._id,
                    RequestType: "CaptchaRequest"
                };
                if (this.model.requestMapper) {
                    $.ajax({
                        type: 'Post',
                        url: this.model.requestMapper,
                        data: data,
                        dataType: "html",
                        async: false,
                        success: function(CaptchaResult) {
                            proxy._onSuccess(CaptchaResult);
                        },
                        error: function(e) {

                        }
                    });
                }
				else
                {
                    if (this.model.pathName)
                    {                   
                       $.ajax({
                        type: 'POST',
                        url: this.model.pathName+"/"+this.model.mapper,                        
                       data: '{encryptedCode: "'+this.model.encryptedCode+'",characterSet:"'+this.model.characterSet+'",height:"'+this.model.height+'",targerValue:"'+$(this._target).val()+'",minimumLength:"'+this.model.minimumLength+'",maximumLength:"'+this.model.maximumLength+'"}',                        
                        contentType: "application/json; charset=utf-8",
                        dataType: "json",         
                        async: false,
                        success: function(CaptchaResult) {
                            proxy._onSuccessAsp(CaptchaResult);
                            return;
                        },
                        error: function(e) {
                        proxy._trigger("refreshFailure");
                            return;
                        }
                    });
                    }
                   }
                //Check if invalid and stop form submit
                if (!this.model.isValid && this.model.enableAutoValidation) {
                    event.preventDefault();
                    event.stopPropagation();
                    return false;
                }
            },
            //Hanlder for captcha refresh event
            refresh : function (event) {
                var proxy = this;
                var data = {
                    Height: this.model.height,
                    Width: this.model.width,
                    CharacterSet: this.model.characterSet,
                    MaximumLength: this.model.maximumLength, 
                    MinimumLength: this.model.minimumLength,                   
                    CaseSensitive: this.model.enableCaseSensitivity,
                    ResponseText: $(this._target).val(),
                    EncrytedText: this.model.encryptedCode,
                    EnableRefreshImage: this.model.enableRefreshImage,
                    EnableAudio:this.model.enableAudio,                 
                    Id: this._id,
                    RequestType: "CaptchaRequest"
                };
                var handler = this.model.refreshBegin;
                var eventArgs = this.model;
                if (handler) {
                    handler(proxy, eventArgs);
                    if (eventArgs.getCancel()) {
                        return false;
                    }
                }
                if (this.model.requestMapper) {          
                    $.ajax({
                        type: 'Post',
                        url: this.model.requestMapper,
                        data: data,
                        dataType: "html",
                        async: false,
                        success: function (CaptchaRefresh) {
                            proxy._onRefreshSuccess(CaptchaRefresh);
                            proxy._trigger("refreshSuccess")
                                return;
                        },
                        error: function (errData) {                            
                            proxy._trigger("refreshFailure")
                            return;
                        },
                        complete: function (jqXHR, textStatus) {                           
                            proxy._trigger("refreshComplete")
                            return;
                        }
                    });
                }
                 else 
                 {  
                    if (this.model.pathName)
                     {
                         $.ajax({
                        type: 'POST',
                        url: this.model.pathName+"/"+this.model.mapper,                        
                        data: '{encryptedCode: "'+this.model.encryptedCode+'",characterSet:"'+this.model.characterSet+'",height:"'+this.model.height+'",targerValue:"'+$(this._target).val()+'",minimumLength:"'+this.model.minimumLength+'",maximumLength:"'+this.model.maximumLength+'"}',                        
                        contentType: "application/json; charset=utf-8",
                        dataType: "json",                   
                        success: function (CaptchaRefresh) {
                            proxy._onRefreshSuccessAsp(CaptchaRefresh);
                            proxy._trigger("refreshSuccess")
                                return;
                        },
                        error: function (errData) {                            
                            proxy._trigger("refreshFailure")
                            return;
                        },
                        complete: function (jqXHR, textStatus) {                           
                            proxy._trigger("refreshComplete")
                            return;
                        }
                    });
                   }
                 }
            } 
    })      
})(jQuery, Syncfusion);;
(function ($, ej, undefined) {
    ej.gridFeatures = ej.gridFeatures || {};
    ej.gridFeatures.common = {
        /**
		 * Refresh the grid contents. The template refreshment is based on the argument passed along with this method
         * @alias ejGrid#refreshContent
		 * @return jQuery
		 * @param {boolean} [templateRefresh] When templateRefresh is set true, template and grid contents both are refreshed in grid else only grid content is refreshed
		 * @example 
		 * &lt;div id="Grid"&gt;&lt;/div&gt; 
		 * &lt;script&gt;
		 * // Create grid object.
		 * var gridObj = $("#Grid").data("ejGrid");
		 * gridObj.refreshContent(); // Refreshes the grid contents only
		 * gridObj.refreshContent(true); // Refreshes the template and grid contents
		 * &lt;/script&gt;
		 * @example 
		 * &lt;div id="Grid"&gt;&lt;/div&gt; 
		 * &lt;script&gt;
		 * // Refreshes the grid contents only
		 * $("#Grid").ejGrid("refreshContent");        
		 * // Refreshes the template and grid contents
		 * $("#Grid").ejGrid("refreshContent", true);        
		 * &lt;/script&gt;
         */
        refreshContent: function ( refreshTemplate) {
             refreshTemplate && this.refreshTemplate();
            var args = {};
            args.requestType = ej.Grid.Actions.Refresh;
            this._processBindings(args);
        },

        /**
		 * Resolves row height issue when unbound column is used with FrozenColumn
         * @alias ejGrid#rowHeightRefresh
		 * @return jQuery
		 * @example 
		 * &lt;div id="Grid"&gt;&lt;/div&gt; 
		 * &lt;script&gt;
		 * // Create grid object.
		 * var gridObj = $("#Grid").data("ejGrid");
		 * gridObj.rowHeightRefresh(); // Resolves row height issue
		 * &lt;/script&gt;
		 * @example 
		 * &lt;div id="Grid"&gt;&lt;/div&gt; 
		 * &lt;script&gt;         
		 * // Resolves row height issue
		 * $("#Grid").ejGrid("rowHeightRefresh");   
		 * &lt;/script&gt;
         */
        rowHeightRefresh: function () {
            if (this.model.scrollSettings.frozenColumns > 0 && (this._isUnboundColumn || this.getContentTable().find(".e-templatecell") != null)) {
                var maxHeight = 0;
                var frozenRow = this.model.scrollSettings.frozenRows;
                var rowIndex = frozenRow > 0 ? frozenRow - 1 : 0;
                if (!ej.isNullOrUndefined(this._gridContent.find(".e-frozencontentdiv tr")[rowIndex].getClientRects()[0]))
                    var frozenContentHeight = this._gridContent.find(".e-frozencontentdiv tr")[rowIndex].getClientRects()[0].height;
                if (!ej.isNullOrUndefined(this._gridContent.find(".e-movablecontentdiv tr")[rowIndex].getClientRects()[0]))
                    var movableContentHeight = this._gridContent.find(".e-movablecontentdiv tr")[rowIndex].getClientRects()[0].height;
                if (frozenContentHeight != movableContentHeight) {
                    if (this._gridContent.find("style").length > 0)
                        this._gridContent.find("style").remove()
                    var arr = [frozenContentHeight, movableContentHeight];
                    maxHeight = ej.max(arr) + (frozenRow > 0 ? 0 : 1);
                    if ((this.model.editSettings.allowEditing || this.model.editSettings.allowAdding) && this.model.editSettings.editMode == "normal" && this._isUnboundColumn)
                        $("<style type='text/css'>.e-rowcell { height: " + maxHeight + "px;  }.e-grid .e-editedrow .e-rowcell{  padding-top:0.7em;padding-bottom:0.7em;}  </style>").prependTo(this._gridContent);
                    else
                        $("<style type='text/css'>.e-rowcell{ height: " + maxHeight + "px} </style>").prependTo(this._gridContent);
                    if (frozenRow > 0) {
                        this._getRowHeights()
                        this.model.scrollSettings.height = this._rowHeightCollection[Math.floor(this.model.scrollSettings.height / this._rowHeightCollection[1])] + 18;
                        this.getContent().ejScroller("option", { "height": this.model.scrollSettings.height });
                    }
                }
            }
        },

        /**
		 * Refresh the grid with new data source
         * @alias ejGrid#dataSource
		 * @return jQuery
         * @param {array} datasource Pass new data source to the grid
		 * @example 
		 * &lt;script&gt;
		 * // Create grid object.
		 * var gridObj = $("#Grid").data("ejGrid");
         * // Refreshes the grid data source
		 * gridObj.dataSource(data); 
		 * &lt;/script&gt;
		 * @example 
		 * &lt;script&gt;
		 * // Refreshes the grid data source
		 * $("#Grid").ejGrid("dataSource", data);        
		 * &lt;/script&gt;
         */
        dataSource: function (dataSource) {
            this._dataSource(dataSource);
            this._refreshDataSource(dataSource);
            if (this.model.scrollSettings.allowVirtualScrolling) {
                this._refreshVirtualContent();
                this.getContent().first().ejScroller("refresh");
                if (this.getContent().ejScroller("isVScroll"))
                    this.element.find(".e-gridheader").addClass("e-scrollcss");
                else
                    this.element.find(".e-gridheader").removeClass("e-scrollcss");
            }
        },
        _refreshDataSource: function (dataSource) {
            if (dataSource instanceof ej.DataManager)
                this._dataManager = dataSource;
            else
                this._dataManager = ej.DataManager(dataSource);
            this.refreshContent(true);
            if (!ej.isNullOrUndefined(this. getPager())) {
                this.getPager().ejPager("model.currentPage", 1);
                this._refreshGridPager();
            }
        },
        /**
		 * Hide columns from the grid based on the header text
         * @alias ejGrid#hideColumns
		 * @return jQuery
		 * @param {array/string} headerText you can pass either array of header text of various columns or a header text of a column to hide 
		 * @example 
		 * &lt;script&gt;
		 * // Create grid object.
		 * var gridObj = $("#Grid").data("ejGrid");
		 * gridObj.hideColumns("Order ID"); // Hides column based on the given header text of the column
		 * gridObj.hideColumns(["Order ID", "Customer ID"]); // Hide columns based on the array of header text of the columns given
		 * &lt;/script&gt;
		 * @example 
		 * &lt;script&gt;
		 * // Hide column based on the given header text of the column
		 * $("#Grid").ejGrid("hideColumns", "Order ID"); 
		 * // Hide columns based on the array of header text of the columns given
		 * $("#Grid").ejGrid("hideColumns", ["Order ID", "Customer ID"]); 			
		 * &lt;/script&gt;
         */
        hideColumns: function (c) {
            var i, count = 0, args = {}, index;
            if ($.isArray(c)) {
                for (i = 0; i < c.length; i++) {
                    index = $.inArray(c[i], this._visibleColumns);
                    if (index != -1) {
                        this._hiddenColumns.push(c[i]);
                        this._visibleColumns.splice(index, 1);
                    }
                }
            } else {
                index = $.inArray(c, this._visibleColumns);
                if (index != -1) {
                    this._hiddenColumns.push(c);
                    this._visibleColumns.splice(index, 1);
                }
            }
            for (i = 0; i < this.model.columns.length; i++) {
                if ($.inArray(this.model.columns[i].headerText, this._hiddenColumns) != -1) {
                    this.model.columns[i].visible = false;
                    count++;
                }
                if (this._hiddenColumns.length == count)
                    break;
            }
            this.refreshTemplate();
            args.requestType = "refresh";
            this.sendDataRenderingRequest(args);
            this._hideHeaderColumn(this._hiddenColumns);
            this.model.allowScrolling && this.getScrollObject().refresh();
        },
        /**
         * Show columns in the grid based on the header text 
         * @alias ejGrid#showColumns
         * @return jQuery
         * @param {array/string} headerText you can pass either array of header text of various columns or a header text of a column to show 
         * @example 
         * &lt;script&gt;
         * // Create grid object.
         * var gridObj = $("#Grid").data("ejGrid");
         * gridObj.showColumns("Order ID"); // Shows column based on the given header text of the column
         * gridObj.showColumns(["Order ID", "Customer ID"]); // Shows columns based on the array of header text of the columns given
         * &lt;/script&gt;
         * @example 
         * &lt;script&gt;
         * // Shows column based on the given header text of the column
         * $("#Grid").ejGrid("showColumns", "Order ID"); 
         * // Shows columns based on the array of header text of the columns given
         * $("#Grid").ejGrid("showColumns", ["Order ID", "Customer ID"]); 			
         * &lt;/script&gt;
         */
        showColumns: function (c) {
            var i, count = 0, args = {}, index, column;
            if ($.isArray(c)) {
                for (i = 0; i < c.length; i++) {
                    index = $.inArray(c[i], this._hiddenColumns);
                    if (index != -1) {
                        this._hiddenColumns.splice(index, 1);
                        this._visibleColumns.push(c[i]);
                    }
                }
            } else {
                index = $.inArray(c, this._hiddenColumns);
                if (index != -1) {
                    this._hiddenColumns.splice(index, 1);
                    this._visibleColumns.push(c);
                }
            }
            for (i = 0; i < this.model.columns.length; i++) {
                if ($.inArray(this.model.columns[i].headerText, this._visibleColumns) != -1) {
                    this.model.columns[i].visible = true;
                    count++;
                }
                if (this._visibleColumns.length == count)
                    break;
            }
            this.refreshTemplate();
            args.requestType = "refresh";
            this.sendDataRenderingRequest(args);
            this._showHeaderColumn(this._visibleColumns);
        },
        /**
		 * Refresh the template of the grid
         * @alias ejGrid#refreshTemplate
		 * @return jQuery
		 * @example 
		 * &lt;script&gt;
		 * // Create grid object.
		 * var gridObj = $("#Grid").data("ejGrid");
         * // Refreshes the template of the grid control
		 * gridObj.refreshTemplate(); 
		 * &lt;/script&gt;
		 * @example 
		 * &lt;script&gt;
		 * // Refreshes the template of the grid control.
		 * $("#Grid").ejGrid("refreshTemplate");        
		 * &lt;/script&gt;
         */
         refreshTemplate: function () {
            this.addInitTemplate();
            if (this.model.editSettings.allowEditing || this.model.editSettings.allowAdding) {
                if (this.model.editSettings.editMode == "normal")
                    this.addEditingTemplate();
                else if (this.model.editSettings.editMode == "dialog" || this.model.editSettings.editMode == "externalform" ||
                    this.model.editSettings.editMode == "inlineform")
                    this.addDialogEditingTemplate();
                else if (this.model.editSettings.editMode == "dialogtemplate" || this.model.editSettings.editMode == "externalformtemplate" ||
                    this.model.editSettings.editMode == "inlineformtemplate")
                    this.addExternalDialogEditingTemplate();
            }
            if (this.model.allowGrouping) this.addGroupingTemplate();
        },
        set_dropColumn: function (from, to) {
            if (this.model.allowReordering && from != to) {
                this.model.columns.splice(to, 0, this.model.columns.splice(from, 1)[0]);
                var columns = this.model.columns;
                var $headerCell = this.getHeaderTable().find(".e-headercelldiv");
                var $attributeCollection = { "attributes": [] };
                var $tmp = {};
                var fromIndex = from < to ? from : to;
                var toIndex = from < to ? to : from;
                var item = 0;
                for (var i = fromIndex; i <= toIndex; i++) {
                    var cellAttributes = $headerCell[i].attributes;
                    for (var j = 0; j < cellAttributes.length; j++) {
                        $tmp[cellAttributes[j].name] = cellAttributes[j].value;
                    }
                    $attributeCollection.attributes.push($tmp);
                    $tmp = $attributeCollection.attributes[item];
                    for (var attr in $tmp) {
                        $headerCell[i].removeAttribute(attr);
                    }
                    $tmp = {};
                    item++;
                }
                var spliceFrom = from < to ? $attributeCollection.attributes.length - 1 : 0;
                var spliceTo = from < to ? 0 : $attributeCollection.attributes.length - 1;
                $attributeCollection.attributes.splice(spliceFrom, 0, $attributeCollection.attributes.splice(spliceTo, 1)[0]);
                //Add attributes
                var j = 0;
                for (var i = fromIndex; i <= toIndex; i++) {

                    var $attrColl = $attributeCollection.attributes[j];
                    for (var attr in $attrColl) {
                        $headerCell[i].setAttribute(attr, $attrColl[attr]);
                    }
                    j++;
                }
				this._columnsWidthCollection.splice(to,0,this._columnsWidthCollection.splice(from,1)[0]);
                this._fieldColumnNames = this._headerColumnNames = [];
                for (var count = 0; count < columns.length; count++) {
                    this._fieldColumnNames[columns[count].headerText] = columns[count].field;
                    this._headerColumnNames[columns[count].field] = columns[count].headerText;
                    $($headerCell[count]).text(columns[count].headerText);
                }
                var args = {};
                args.requestType = ej.Grid.Actions.Reorder;
                this.addInitTemplate();
                this.sendDataRenderingRequest(args);
            }
        },
        /**
		 * Get the pager of grid..
         * @alias ejGrid#getPager
		 * @return gridPager
		 * @example 
		 * &lt;script&gt;
		 * // Create grid object.
		 * var gridObj = $("#Grid").data("ejGrid");
         * // Gets grid pager of grid control
		 * gridObj.getPager(); 
		 * &lt;/script&gt;
		 * @example 
		 * &lt;script&gt;
		 * // Gets grid pager of grid control
		 * $("#Grid").ejGrid("getPager");        
		 * &lt;/script&gt;
         */
         getPager: function () {
            return this._gridPager;
        },
        /**
		 * Get the footer table element of grid
         * @alias ejGrid#getFooterTable
		 * @return gridFooterTable
		 * @example 
		 * &lt;script&gt;
		 * // Create grid object.
		 * var gridObj = $("#Grid").data("ejGrid");
         * // Gets grid footer table of grid control
		 * gridObj.getFooterTable(); 
		 * &lt;/script&gt;
		 * @example 
		 * &lt;script&gt;
		 * // Gets grid footer table of grid control
		 * $("#Grid").ejGrid("getFooterTable");        
		 * &lt;/script&gt;
         */
        getFooterTable: function () {
            return this._gridFooterTable;
        },

        setGridFooterTable: function (value) {
            this._gridFooterTable = value;
        },
        /**
		 * Get the footer content of grid.
         * @alias ejGrid#getFooterContent
		 * @return gridFooterContent
		 * @example 
		 * &lt;script&gt;
		 * // Create grid object.
		 * var gridObj = $("#Grid").data("ejGrid");
         * // Gets grid footer content of grid control
		 * gridObj.getFooterContent(); 
		 * &lt;/script&gt;
		 * @example 
		 * &lt;script&gt;
		 * // Gets grid footer content of grid control
		 * $("#Grid").ejGrid("getFooterContent");        
		 * &lt;/script&gt;
         */
        getFooterContent: function () {
            return this._gridFooterContent;
        },

        setGridFooterContent: function (value) {
            this._gridFooterContent = value;
        },
        /**
		 * Get the scroller object of grid.
         * @alias ejGrid#getScrollObject
		 * @return scrollObject
		 * @example 
		 * &lt;script&gt;
		 * // Create grid object.
		 * var gridObj = $("#Grid").data("ejGrid");
         * // Gets scroll object of grid control
		 * gridObj.getScrollObject(); 
		 * &lt;/script&gt;
		 * @example 
		 * &lt;script&gt;
		 * // Gets scroll object of grid control
		 * $("#Grid").ejGrid("getScrollObject");        
		 * &lt;/script&gt;
         */
        getScrollObject: function () {
            if (this._scrollObject == null || ej.isNullOrUndefined(this._scrollObject.model))
                this._scrollObject = this.getContent().ejScroller("instance");
            return this._scrollObject;
        },
        setGridPager: function (value) {
            this._gridPager = value;
        },
        /**
         * Get the row height of grid.
         * @alias ejGrid#getRowHeight
         * @return rowHeight
         * @example 
         * &lt;script&gt;
         * // Create grid object.
         * var gridObj = $("#Grid").data("ejGrid");
         * // Gets the row height of the grid
         * gridObj.getRowHeight();  
         * &lt;/script&gt;
         * @example 
         * &lt;script&gt;
         * // Gets the row height of the grid
         * $("#Grid").ejGrid("getRowHeight");   
         * &lt;/script&gt;
         */
        getRowHeight: function () {
            var rowHeight = -1;
            var trColl = this.getContentTable().find('tr:not(.e-virtualrow)');
            if (trColl.length > 1) {
                if (trColl[0].getBoundingClientRect() && this.getContentTable().find("tr")[0].getBoundingClientRect().height) {
                    rowHeight = trColl[0].getBoundingClientRect().height;
                } else
                    rowHeight = trColl[0].offsetHeight;
            }
            return rowHeight == -1 ? 32 : rowHeight;
        },
        /**
		 * Get the current page index in grid pager.
         * @alias ejGrid#getCurrentIndex
		 * @return Current PageIndex
		 * @example 
		 * &lt;script&gt;
		 * // Create grid object.
		 * var gridObj = $("#Grid").data("ejGrid");
         * // Gets the current page index in grid
		 * gridObj.getCurrentIndex();  
		 * &lt;/script&gt;
		 * @example 
		 * &lt;script&gt;
		 * // Gets the current page index in grid
		 * $("#Grid").ejGrid("getCurrentIndex");   
		 * &lt;/script&gt;
         */
        getCurrentIndex: function () {
            return ((this._currentPage() - 1) * (this.model.pageSettings.pageSize));
        },
        /**
		 * Get the column details based on the given column index in grid
         * @alias ejGrid#getColumnByIndex
		 * @return Column Object
		 * @param {number} columnIndex Pass the index of the column to get the corresponding column object
		 * @example 
		 * &lt;script&gt;
		 * // Create grid object.
		 * var gridObj = $("#Grid").data("ejGrid");
         * // Gets the column details based on the given column index
		 * gridObj.getColumnByIndex(1); 
		 * &lt;/script&gt;
		 * @example 
		 * &lt;script&gt;
		 * // Gets the column details based on the given column index
		 * $("#Grid").ejGrid("getColumnByIndex", 1);        
		 * &lt;/script&gt;
         */
        getColumnByIndex: function (index) {
            if (index < this.model.columns.length)
                return this.model.columns[index];
            return null;
        },
        set_currentPageIndex: function (val) {
            var pageSetting = this.model.pageSettings;
            var recordCount = this.model.filterSettings.filteredColumns.length == 0 ? this._searchCount == null ? this._gridRecordsCount : this._searchCount : this._filteredRecordsCount;
            if (pageSetting.totalPages == null)
                pageSetting.totalPages = Math.ceil(recordCount / pageSetting.pageSize);
            if (val > pageSetting.totalPages || val < 1 || val == this._currentPage())
                return false;
            if (ej.isNullOrUndefined(this._prevPageNo))
                this._prevPageNo = this._currentPage();
            this._currentPage(val);
            if (this._currentPage() != this._prevPageNo) {
                var args = {};
                args.requestType = "paging";
                this._isVirtualRecordsLoaded = false;
                this.gotoPage(this._currentPage(), args);
                var model = this._refreshVirtualPagerInfo();
                this._showPagerInformation(model);
                return true;
            }
            else
                return false;
        },
        /**
		 * Expand or collapse the row based on the row state in grid
         * @alias ejGrid#expandCollapse
		 * @return jQuery
		 * @param {object} $target Pass the target object to expand/collapse the row based on its row state
		 * @example 
		 * &lt;script&gt;
		 * // Create grid object.
		 * var gridObj = $("#Grid").data("ejGrid");
         * // Expands or collapses the row based on the row state
		 * gridObj.expandCollapse($("tr td.recordplusexpand > div").first());  
		 * &lt;/script&gt;
		 * @example 
		 * &lt;script&gt;
		 * // Expands or collapses the row based on the row state
		 * $("#Grid").ejGrid("expandCollapse", $("tr td.recordplusexpand > div").first());        
		 * &lt;/script&gt;
         */
        expandCollapse: function ($target) {
            if ($target.prop("tagName") == "DIV" && ($target.parent().hasClass("e-recordplusexpand") || $target.parent().hasClass("e-recordpluscollapse") || $target.parent().hasClass("e-detailrowcollapse") || $target.parent().hasClass("e-detailrowexpand")))
                $target = $target.parent();
            if ($target.hasClass("e-recordplusexpand") && this.model.groupSettings.groupedColumns.length) {
                var cellIndex = $target.index();
                var $rows = $target.closest('tr').next();
                $rows.hide();
                $target.removeClass("e-recordplusexpand").addClass("e-recordpluscollapse").find("div").removeClass("e-gdiagonalnext").addClass("e-gnextforward");
            } else if ($target.hasClass("e-recordpluscollapse") && this.model.groupSettings.groupedColumns.length) {
                var cellIndex = $target.index();
                var $rows = $target.closest('tr').next();
                var toExpandRows = [];
                var $row = $rows;
                if ($($row[0].cells[cellIndex]).hasClass("e-indentcell")) {
                    if ($row.children(".e-indentcell").length == ($target.parent().children('.e-indentcell').length) + 1) {
                        $row.show();
                        var $expand = $row.children(".e-recordplusexpand");
                        if ($expand != null && $expand.length > 0) {
                            toExpandRows.push($expand);
                        }
                    }
                }
                $target.removeClass("e-recordpluscollapse").addClass("e-recordplusexpand").find("div").removeClass("e-gnextforward").addClass("e-gdiagonalnext");
                for (var i = 0; i < toExpandRows.length; i++) {
                    toExpandRows[i].removeClass("e-recordplusexpand").addClass("e-recordpluscollapse").find("div").removeClass("e-gdiagonalnext").addClass("e-gnextforward");
                    this.expandCollapse(toExpandRows[i]);
                }
            } else if ($target.hasClass("e-detailrowexpand")) {
                var cellIndex = $target.index(), proxy = this;
                var rowIndexValue;
                if (this.model.groupSettings.groupedColumns.length > 0)
                    rowIndexValue = this.getIndexByRow($target.closest('tr')) - $target.closest('tr').parents('tr').prevAll('tr').find('td.e-summaryrow').parent().length;
                else
                    rowIndexValue = this.getIndexByRow($target.closest('tr')) - $target.closest('tr').prevAll("tr.e-detailrow").length;
                var $rows = $target.closest('tr').next();
                $rows.hide(0, function () {
                    var args = { masterRow: $target.closest('tr'), detailsRow: $rows, masterData: proxy._currentJsonData[rowIndexValue] };
                    var foreignKeyData = proxy._getForeignKeyData(args.masterData);
                    if (!ej.isNullOrUndefined(foreignKeyData))
                        args.foreignKeyData = foreignKeyData;
                    proxy._trigger("detailsCollapse", args);

                });
                $target.removeClass("e-detailrowexpand").addClass("e-detailrowcollapse").find("div").addClass("e-gnextforward").removeClass("e-gdiagonalnext");
            } else if ($target.hasClass("e-detailrowcollapse")) {
                var cellIndex = $target.index(), proxy = this;
                var rowIndexValue;
                if (this.model.groupSettings.groupedColumns.length > 0)
                    rowIndexValue = this.getIndexByRow($target.closest('tr')) - $target.closest('tr').parents('tr').prevAll('tr').find('td.e-summaryrow').parent().length;
                else
                    rowIndexValue = this.getIndexByRow($target.closest('tr')) - $target.closest('tr').prevAll("tr.e-detailrow").length;
                var detailrow = $target.closest('tr').next();
                if (detailrow.hasClass("e-detailrow"))
                    $rows = detailrow;
                else {
                    var detailtr = ej.buildTag("tr.e-detailrow", "", { 'display': 'none' }, {});
                    var indenttd = ej.buildTag("td.e-detailindentcell");
                    var detailstd = ej.buildTag("td.e-detailcell", "", {}, { colspan: this.model.columns.length });
                    var detaildiv = ej.buildTag("div");
                    var count = $($target.closest('tr')).parents('.e-grid').length;
                    detaildiv.attr("id", "child" + count + "_grid" + rowIndexValue);
                    $(detailtr).append(indenttd);
                    $(detailtr).append(detailstd);
                    var rowData = this._currentJsonData[rowIndexValue];
                    if (this.model.detailsTemplate)
                    $(detailtr).append(detailstd.append(this._renderEjTemplate(this.model.detailsTemplate, rowData)));

                    $($target.closest('tr')).after(detailtr);
                    this._trigger("detailsDataBound", { detailsElement: detailtr, data: rowData }); //    $(tbody).append(trchild);
                    if (this.model.childGrid) {
                        var temp = this.model.childGrid.queryString;
                        var temp1 = ej.Query().where(temp, "equal", rowData[temp], true);
                        this.model.childGrid.query = temp1;
                        $(detailtr).append(detailstd.append(detaildiv));
                    }
                    $rows = detailtr;
                }
                var toExpandRows = [];
                var $row = $rows;
                if ($($row[0].cells[cellIndex]).hasClass("e-detailindentcell")) {
                    $row.show(0, function () {
                        var args = { masterRow: $target.closest('tr'), detailsRow: $rows, masterData: proxy._currentJsonData[rowIndexValue] };
                        var foreignKeyData = proxy._getForeignKeyData(args.masterData);
                        if (!ej.isNullOrUndefined(foreignKeyData))
                            args.foreignKeyData = foreignKeyData;
                        proxy._trigger("detailsExpand", args);
                    });
                    this.model.childGrid && !ej.isNullOrUndefined(detaildiv) && detaildiv.ejGrid(this.model.childGrid);
                    var $expand = $row.children(".e-detailrowexpand");
                    if ($expand != null && $expand.length > 0) {
                        toExpandRows.push($expand);
                    }
                }
                $target.removeClass("e-detailrowcollapse").addClass("e-detailrowexpand").find("div").addClass("e-gdiagonalnext").removeClass("e-gnextforward");
                for (var i = 0; i < toExpandRows.length; i++) {
                    toExpandRows[i].removeClass("e-detailrowexpand").addClass("e-detailrowcollapse").find("div").removeClass("e-gdiagonalnext").addClass("e-gnextforward");
                    this.expandCollapse(toExpandRows[i]);
                }
            }
        },
        _refreshGridPager: function () {
            if (this. getPager() != null) {
                var pagerModel = this.getPager().ejPager("model"), model = {};
                model.currentPage = this._currentPage();
                if (this._filteredRecordsCount == 0 && this.model.currentViewData.length == 0 && (ej.isNullOrUndefined(this._prevPageNo) || this._prevPageNo)) {
                    model.currentPage = 0;
                    this._prevPageNo = pagerModel.currentPage;
                    this.model.pageSettings.currentPage = 0;
                } else if (pagerModel.currentPage == 0 && (ej.isNullOrUndefined(this._prevPageNo) || this._prevPageNo))
                    model.currentPage = this._prevPageNo;
                model.totalRecordsCount = this.model.filterSettings.filteredColumns.length == 0 ? this._searchCount == null ? this._gridRecordsCount : this._searchCount : this._filteredRecordsCount;
                if (ej.util.isNullOrUndefined(model.currentPage))
                    model.currentPage = this._currentPage();
                this.getPager().ejPager("option", model).ejPager("refreshPager");
            }
        },
        _showHeaderColumn: function (showColumns, field) {
            var $head = this.getHeaderTable().find("thead");
            var $headerCell = $head.find(".e-headercell");
            var $filterBarCell = $head.find(".e-filterbar").find(".e-filterbarcell");
            var $col = this.getHeaderTable().find("colgroup").find("col"), column;
            for (var i = 0; i < showColumns.length; i++) {
                if (field)
                    column = this.getColumnByField(showColumns[i]);
                else
                    column = this.getColumnByHeaderText(showColumns[i]);
                var index = $.inArray(column, this.model.columns);
                var frznCol = this.model.scrollSettings.frozenColumns;
                if (frznCol != 0 && index >= frznCol)
                    var thIndex = $headerCell.eq(index).removeClass("e-hide").index() + frznCol;
                else
                    var thIndex = $headerCell.eq(index).removeClass("e-hide").index();
                $filterBarCell.eq(thIndex).removeClass("e-hide");
                $col.eq(index).css("display", "");
            }
        },
        _hideHeaderColumn: function (hiddenColumns, field) {
            var $head = this.getHeaderTable().find("thead");
            var $headerCell = $head.find(".e-headercell");
            var $filterBarCell = $head.find(".e-filterbar").find(".e-filterbarcell");
            var $col = this.getHeaderTable().find("colgroup").find("col"), column;
            for (var i = 0; i < hiddenColumns.length; i++) {
                if (field)
                    column = this.getColumnByField(hiddenColumns[i]);
                else
                    column = this.getColumnByHeaderText(hiddenColumns[i]);
                var index = $.inArray(column, this.model.columns);
                var frznCol = this.model.scrollSettings.frozenColumns;
                if (frznCol != 0 && index >= frznCol)
                    var thIndex = $headerCell.eq(index).addClass("e-hide").index() + frznCol;
                else
                    var thIndex = $headerCell.eq(index).addClass("e-hide").index();
                $filterBarCell.eq(thIndex).addClass("e-hide");
                if ($col.length > this.model.columns.length)
                    $col = $col.slice($col.length - this.model.columns.length);
                $col.eq(index).css("display", "none");
            }
        },
        _checkSkipAction: function (args) {
            switch (args.requestType) {
                case ej.Grid.Actions.Save:
                case ej.Grid.Actions.Delete:
                    return true;
            }
            return false;
        },
        _unboundTemplateRendering: function (unboundTemplateId) {
            return $("#" + unboundTemplateId).html();
        },
        _processBindings: function (args) {
            this.model.query = this.model.enablePersistence ? new ej.Query() : this.commonQuery.clone();
            if (!this._checkSkipAction(args) && this._trigger("actionBegin", args))
                return true;
            if (this.model.editSettings.editMode == "batch" && args.requestType != "batchsave" && args.requestType != "cancel" && !this._confirmedValue && this._bulkChangesAcquired() && this.model.editSettings.showConfirmDialog) {
                this._confirmDialog.find(".e-content").html(this._getLocalizedLabels("BatchSaveLostChanges"));
                this._confirmDialog.ejDialog("open");
                this._requestArgs = args;
                return false;
            }
            this._ensureDataSource(args);
            if (this.model.enableRTL) {
                !this.element.hasClass("e-rtl") && this.element.addClass("e-rtl");
            } else {
                this.element.hasClass("e-rtl") && this.element.removeClass("e-rtl")
            }
            if (args.requestType == ej.Grid.Actions.Delete && this.model.groupSettings.groupedColumns.length == 0)
                args.tr.remove();
            if (!(this._dataSource() instanceof ej.DataManager) && this.model.groupSettings.groupedColumns.length != 0)
                this._setAggregates();
            if (this._dataSource() instanceof ej.DataManager && args.requestType != ej.Grid.Actions.BeginEdit && args.requestType != ej.Grid.Actions.Cancel && args.requestType != ej.Grid.Actions.Add) {
                var queryPromise = this._dataSource().executeQuery(this.model.query);
                var waitingPopup = this.element.ejWaitingPopup("instance");
                this.element.ejWaitingPopup("show");
                    queryPromise.done(ej.proxy(function (e) {
                        this.element.ejWaitingPopup("hide");
                        this.model.currentViewData = e.result;
                        this._processData(e, args);
                    }, this));

                    queryPromise.fail(ej.proxy(function (e) {
                        this.element.ejWaitingPopup("hide");
                        e = [];
                        this.model.currentViewData = [];
                        this._processData(e, args);
                    }, this));
            } else
                this.sendDataRenderingRequest(args);
        },
        _createUnboundElement: function (column) {
            var divElement = document.createElement("div");
            divElement.id = this._id + column.headerText.replace(/\s+/g, "") + "_UnboundTemplate";
            var $div = ej.buildTag("div.e-unboundcelldiv"), commands = column["commands"];
            for (var unbounType = 0; unbounType < commands.length; unbounType++) {
                var $button = ej.buildTag("input.e-" + commands[unbounType].type + "button", "", {}, { type: "button" });
                $button.val(commands[unbounType].type);
                $div.append($button);
            }
            $("body").append($(divElement).html($div).hide());
            return divElement;
        },
        _refreshUnboundTemplate: function ($target) {
            if (this._isUnboundColumn) {
                var index = 0;
                for (var column = 0; column < this.model.columns.length; column++) {
                    if (this.model.columns[column]["isUnbound"]) {
                        var $unboundDivs = $target.find(".e-unboundcell.e-" + this.model.columns[column]["headerText"].replace(/\s+/g, "")).find(".e-unboundcelldiv");
                        var commands = $.extend(true,[],this.model.columns[column].commands);
                        for (var j = 0; j < commands.length; j++) {
                            var width = ej.isNullOrUndefined(commands[j].buttonOptions.width) ? "52" : commands[j].buttonOptions.width;
                            var height = ej.isNullOrUndefined(commands[j].buttonOptions.height) ? "30" : commands[j].buttonOptions.height;
                            commands[j].buttonOptions.width = ej.isNullOrUndefined(commands[j].buttonOptions.width) ? "52" : commands[j].buttonOptions.width;
                            commands[j].buttonOptions.height = ej.isNullOrUndefined(commands[j].buttonOptions.height) ? "28" : commands[j].buttonOptions.height;
                            commands[j].buttonOptions.cssClass = this.model.cssClass;
                            commands[j].buttonOptions.enableRTL = this.model.enableRTL;
                            var $buttons = $unboundDivs.find(".e-" + commands[j].type + "button");
                            if ($target.closest(".e-editcell").length) {
                                if (commands[j].type == "save" || commands[j].type == "cancel")
                                    $buttons.show();
                                else {
                                    $buttons.hasClass("e-deletebutton") && $buttons.hide();
                                    $buttons.hasClass("e-editbutton") && $buttons.hide();
                                }
                            } else {
                                if (commands[j].type == "save" || commands[j].type == "cancel")
                                    $buttons.hide();
                                else {
                                    $buttons.hasClass("e-deletebutton") && $buttons.show();
                                    $buttons.hasClass("e-editbutton") && $buttons.show();
                                }
                            }
                            if ($($buttons[0]).data("ejButton"))
                                $buttons.filter(".e-button").ejButton("destroy");
                            $buttons.ejButton(commands[j].buttonOptions);
                        }
                    } else
                        continue;
                }
            }
        },
        _gridTemplate: function (self, templateId) {
            return self._renderEjTemplate("#" + templateId, this.data);
        },
        _createTemplateElement: function (column) {
            var scriptElement = document.createElement("script");
            scriptElement.id = (this._id + column.headerText + $.inArray(column, this.model.columns) + "_Template").split(" ").join("");
            scriptElement.type = "text/x-jsrender";
			column["template"] = column["template"] == "True" ? true : column["template"];
            if (typeof column["template"] != "boolean") {
                if (column.template.slice(0, 1) !== "#")
                    scriptElement.text = column.template;
                else
                    scriptElement.text = $(column["template"]).html();
            }
            else
                scriptElement.text = $(column["templateID"]).html();
            $("body").append(scriptElement);
            return scriptElement;
        },
        _renderGridPager: function () {
            var $div = $(document.createElement('div'));
            var pagerModel = {};
            this.model.pageSettings.click = this._gPagerClickHandler;
            this.model.pageSettings.totalRecordsCount = this._gridRecordsCount;
            this.model.pageSettings.enableRTL = this.model.enableRTL;
            this.model.pageSettings.locale = this.model.locale;
            this.model.pageSettings.enableQueryString = this.model.pageSettings.enableQueryString;
            if (this.model.allowFiltering && this.model.filterSettings.filterType == "filterbar")
                pagerModel.enableExternalMessage = true;
            $.extend(pagerModel, this.model.pageSettings);
            pagerModel.currentPage = this._currentPage();
            pagerModel.masterObject = this;
            this.setGridPager($div);
            $div.ejPager(pagerModel);
            $div.ejPager("refreshPager");
            pagerModel = $div.ejPager("model");
            this.model.pageSettings.totalPages = pagerModel.totalPages;
            if (this._currentPage() !== pagerModel.currentPage)
                this._currentPage(pagerModel.currentPage);
            if (this.model.pageSettings.enableTemplates) {
                if (!this.model.pageSettings.showDefaults) {
                        $div.children().remove();
                    }
					var $tempHtml;
					if ($(this.model.pageSettings.template)[0].type == 'text/x-jsrender')
					    $tmpHtml = $(this.model.pageSettings.template).hide().html()
				    else
					    $tempHtml = $(this.model.pageSettings.template)[0];
                      var $divPagerContainer = $div.find('.e-pagercontainer');
                      var $customDiv = ej.buildTag('div.e-pagercontainer ');
                      $customDiv.append($tmpHtml);
                      $div.append($customDiv)
                }
              return $div;
        },

        _renderContext: function (e) {
            var menuitems = this.model.contextMenuSettings.contextMenuItems;
            var item, item2, i = 0;
            var ul = ej.buildTag('ul', "", {}, { id: 'context' });
            for (i; i < menuitems.length; i++) {
                item = menuitems[i];
                item2 = this._items(item, ul);
                ul.append(item2);
            }
            var customitems = this.model.contextMenuSettings.customContextMenuItems;
            var custom, custom2, j = 0;
            for (j; j < customitems.length; j++) {
                custom = customitems[j];
                custom2 = this._items(custom, ul);
                ul.append(custom2);
            }
            this.element.append(ul);
            $(ul).ejMenu({
                menuType: ej.MenuType.ContextMenu,
                openOnClick: false,
                contextMenuTarget: "#" + this._id,
                click: $.proxy(this._clickevent, this),
                width: "auto",
                beforeOpen: $.proxy(this._menu, this)
            });

            this._conmenu = ul.data("ejMenu");
        },
        _clickevent: function (sender) {
            var args = sender.events.text;
            var tr = $(this._contexttarget.parentNode);
            var c = $(this._contexttarget);
            var columnName = c.attr("ej-mappingname");
            switch (args) {
                case "Add Record": this._startAdd();
                    break;
                case "Edit Record": this.startEdit(tr);
                    break;
                case "Delete Record": this._deleteRow(tr);
                    break;
                case "Save": this.endEdit();
                    break;
                case "Cancel": this.cancelEdit();
                    break;
                case "Sort In Descending Order": var columnSortDirection = ej.sortOrder.Descending;
                    this.sortColumn(columnName, columnSortDirection);
                    break;
                case "Sort In Ascending Order": var columnSortDirection = ej.sortOrder.Ascending;
                    this.sortColumn(columnName, columnSortDirection);
                    break;
                case "Grouping": this.groupColumn(columnName);
                    break;
                case "Ungrouping": this.ungroupColumn(columnName);
                    break;
                case "Next Page":
                case "Previous Page":
                case "Last Page":
                case "First Page": this._page(sender);
                    break;
                default: this._trigger("contextClick", sender);
            }

        },

        _menu: function (sender) {
            var context = this._conmenu.element;
            this._contexttarget = sender.target;
            var td = $(this._contexttarget);
            if (td.hasClass("e-rowcell") && this.model.allowSelection)
                this.selectRows(this.getIndexByRow(td.parent()));
            var element = sender.target.parentNode.getAttribute("aria-sort");
            var target = sender.target.className;
            var sorting = $(context);
            var value = sender.target.getAttribute("ej-mappingname");
            switch (target) {
                case "e-pager e-js": var a = $(context.find(".e-page"));
                    context.find(".e-head").css("display", "none");
                    context.find(".e-content").css("display", "none");
                    context.find(".e-savcan").css("display", "none");
                    a.css("display", "block");
                    break;
                case "e-headercelldiv": var a = $(context.find(".e-head"));
                    context.find(".e-page").css("display", "none");
                    context.find(".e-content").css("display", "none");
                    context.find(".e-savcan").css("display", "none");
                    a.css("display", "block");
                    break;
                case "e-rowcell e-selectionbackground e-active": var a = $(context.find(".e-content"));
                    context.find(".e-head").css("display", "none");
                    context.find(".e-page").css("display", "none");
                    context.find(".e-savcan").css("display", "none");
                    a.css("display", "block");
                    break;
               default: var a = $(context);
                    context.find(".e-head").css("display", "none");
                    context.find(".e-page").css("display", "none");
                    context.find(".e-savcan").css("display", "none");
                    context.find(".e-content").css("display", "none");
            }
            switch (element) {
                case "ascending": sorting.find(".ascending").parent().css("display", "none");
                    break;
                case "descending": sorting.find(".descending").parent().css("display", "none");
                    break;
            }
            if (this.model.isEdit) {
                var a = $(context.find(".e-savcan"));
                context.find(".e-head").css("display", "none");
                context.find(".e-content").css("display", "none");
                context.find(".e-page").css("display", "none");
                a.css("display", "block");
            }

            if (this.model.pageSettings.currentPage == 1) {
                sorting.find(".previous").parent().css("display", "none");
                sorting.find(".first").parent().css("display", "none");
            }
            else if (this.model.pageSettings.currentPage == this.model.pageSettings.totalPages) {
                sorting.find(".last").parent().css("display", "none");
                sorting.find(".nextpage").parent().css("display", "none");
            }
            if (this.model.groupSettings.groupedColumns.indexOf(value) != -1)
                a.find(".group").parent().css("display", "none");
            else if (this.model.groupSettings.groupedColumns.indexOf(value) == -1)
                a.find(".ungroup").parent().css("display", "none");
            if (this.model.contextOpen)
                this._trigger("contextOpen", sender);
        },

        _items: function (item) {
            if (item.indexOf("Record") != -1) {
                var li = ej.buildTag('li', "", {}, { class: "e-content" });
                li.css("display", "none");
            }
            else if (item.indexOf("Page") != -1) {
                var li = ej.buildTag('li', "", {}, { class: "e-page" });
                if (item.indexOf("Next") != -1)
                    var div = ej.buildTag('div', "", {}, { class: "nextpage" });
                else if (item.indexOf("Previous") != -1)
                    var div = ej.buildTag('div', "", {}, { class: "previous" });
                else if (item.indexOf("Last") != -1)
                    var div = ej.buildTag('div', "", {}, { class: "last" });
                else if (item.indexOf("First") != -1)
                    var div = ej.buildTag('div', "", {}, { class: "first" });
                li.css("display", "none");
            }
            else if (item == "Save" || item == "Cancel") {
                var li = ej.buildTag('li', "", {}, { class: "e-savcan" });
                li.css("display", "none");
            }
            else {
                var li = ej.buildTag('li', "", {}, { class: "e-head" });
                if (item.indexOf("Ascending") != -1)
                    var div = ej.buildTag('div', "", {}, { class: "ascending" });
                else if (item.indexOf("Descending") != -1)
                    var div = ej.buildTag('div', "", {}, { class: "descending" });
                else if (item == "Grouping")
                    var div = ej.buildTag('div', "", {}, { class: "group" });
                else if (item == "Ungrouping")
                    var div = ej.buildTag('div', "", {}, { class: "ungroup" });
                li.css("display", "none");
            }
            li.append(div);
            var a = document.createElement("a");
            a.innerHTML = item;
            li.append(a);
            return li;
        },

        _page: function (send) {
            if (send.events.text == "Next Page") {
                var b = this.model.pageSettings.currentPage;
                ++b;
                this.gotoPage(b);
            }
            else if (send.events.text == "Previous Page") {
                var b = this.model.pageSettings.currentPage;
                if (b > 1) {
                    --b;
                    this.gotoPage(b);
                }
                else
                    this.gotoPage(b);
            }
            else if (send.events.text == "Last Page") {
                var b = this.model.pageSettings.totalPages
                this.gotoPage(b);
            }
            else
                this.gotoPage(1);


        },

        /**
		 * Send a paging request to specified page in grid
         * @alias ejGrid#gotoPage
		 * @return jQuery
		 * @param {number} pageIndex Pass the page index to perform paging at specified page index
		 * @example 
		 * &lt;script&gt;
		 * // Create grid object.
		 * var gridObj = $("#Grid").data("ejGrid");
         * // Sends a paging request to the grid with specified page index
		 * gridObj.gotoPage(3);
		 * &lt;/script&gt;
		 * @example 
		 * &lt;script&gt;
		 * // Sends a paging request to the grid with specified page index
		 * $("#Grid").ejGrid("gotoPage", 3);        
		 * &lt;/script&gt;
         */
        gotoPage: function (pageIndex) {
            if (!this.model.allowPaging && (!this.model.allowScrolling && !this.model.scrollSettings.allowVirtualScrolling))
                return;
            var args = {}, returnValue;
            args.previousPage = this._currentPage();
            this._currentPage(pageIndex);
            args.endIndex = ((this._currentPage() * this.model.pageSettings.pageSize) > this._gridRecordsCount) ? (this._gridRecordsCount) : (this._currentPage() * this.model.pageSettings.pageSize);
            args.startIndex = (this._currentPage() * this.model.pageSettings.pageSize) - this.model.pageSettings.pageSize;
            args.currentPage = pageIndex;
            if (this.model.allowPaging) {
                //this.model.pageSettings.currentPage = pageIndex;
                //this. getPager().ejPager("refreshPager");
                args.requestType = ej.Grid.Actions.Paging;
            }
            if (this.model.scrollSettings.allowVirtualScrolling && this.model.allowScrolling) {
                args.requestType = ej.Grid.Actions.VirtualScroll;
            }
            returnValue = this._processBindings(args);
            if (returnValue)
                this._currentPage(args.previousPage);
            this._primaryKeyValues = [];
        },
        _gPagerClickHandler: function (sender) {
            if (this._prevPageNo == sender.currentPage)
                return;
            this.model.masterObject.gotoPage(sender.currentPage);
            return false;
        },
        _processData: function (e, args) {
            if (e.count == 0 && this.model.currentViewData.length)
                this._gridRecordsCount = e.result.length;
            else
                this._gridRecordsCount = e.count;
            if (this.getPager() != null)
                this.model.pageSettings.totalRecordsCount = this._gridRecordsCount;
            if ((args.requestType == ej.Grid.Actions.Filtering || ej.Grid.Actions.Save || (this.model.filterSettings.filteredColumns.length > 0 && args.requestType == ej.Grid.Actions.Refresh)))
                this._filteredRecordsCount = e.count;
            this.sendDataRenderingRequest(args);
        },
        _renderGridFooter: function () {
            if (this.model.summaryRows.length > 0) {
                var _$gridFooter = ej.buildTag("div.e-gridfooter");
                var $table = ej.buildTag("table.e-gridsummary", "", {}, { cellspacing: "0.25px" });
                this.setGridFooterContent(_$gridFooter);
                if (this.model.scrollSettings.frozenColumns > 0) {
                    var $frozenFooterDiv = ej.buildTag("div.e-frozenfooterdiv"), $movableFooter = ej.buildTag("div.e-movablefooter")
                        , $tableClone = $table.clone(), $movableFooterDiv = ej.buildTag("div.e-movablefooterdiv");
                    $movableFooter.append($movableFooterDiv);
                    $table.append(this.getHeaderTable().first().find('colgroup').clone());
                    $tableClone.append(this.getHeaderTable().last().find('colgroup').clone());
                    $frozenFooterDiv.append($table);
                    $movableFooterDiv.append($tableClone);
                    this.setGridFooterTable($table.add($tableClone));
                    this._createSummaryRows(this.getFooterTable());
                    _$gridFooter.append($frozenFooterDiv.add($movableFooter));
                    _$gridFooter.find(".e-frozenfooterdiv").outerWidth(this.getHeaderContent().find(".e-frozenheaderdiv").width())
                          .end().find(".e-movablefooterdiv").outerWidth(this.getContent().find(".e-movablecontentdiv").width());
                }
                else {
                    $table.append(this.getHeaderTable().find('colgroup').clone());
                    this.setGridFooterTable($table);
                    this._createSummaryRows(this.getFooterTable());
                    _$gridFooter.append($table);
                }
                return _$gridFooter;
            } else
                throw "summary row collection is missing";
        },
        _createSummaryRows: function (table, summaryData, aggregates) {
            var col = table.find("col");
            if (table.find("tbody").length > 0)
                table.find("tbody").remove();
            var $tBody = ej.buildTag('tbody'), proxy = this, $tBodyClone = $tBody.clone();
            var summaryCol = this.model.summaryRows;
            if (!ej.isNullOrUndefined(summaryData) && this._isCaptionSummary)
                summaryCol = this._captionSummary();
            $.each(summaryCol, function (indx, row) {
                if (row.showTotalSummary === false && ej.isNullOrUndefined(summaryData)) return true;
                var $tr = ej.buildTag('tr');
                for (var i = 0; i < proxy.model.groupSettings.groupedColumns.length; i++) {
                    $tr.prepend(ej.buildTag('td').addClass("e-indentcell"));
                }
                if (proxy.model.detailsTemplate != null) {
                    if (proxy.model.groupSettings.groupedColumns.length != 0)
                        $tr.children("td.e-indentcell").last().after("<td class='e-summaryrow'></td>");
                    else
                        $tr.prepend("<td class='e-summaryrow'></td>");
                }
                var $cells = proxy.getHeaderTable().find('td').clone().addClass("e-summaryrow");
                $cells.first().html(row.title);
                proxy._hideSummaryColumn($cells, col);
                if (proxy.model.scrollSettings.frozenColumns > 0) {
                    var $trClone = $tr.clone();
                    $tBody.append($tr.append($cells.slice(0, proxy.model.scrollSettings.frozenColumns)));
                    $tBodyClone.append($trClone.append($cells.slice(proxy.model.scrollSettings.frozenColumns)));
                }
                else
                    $tBody.append($tr.append($cells));
                $.each(row.summaryColumns, function (cindx, col) {
                    var value = aggregates ? aggregates[indx][col.displayColumn] : proxy._getSummaryValues(col, summaryData), prefix = col.prefix ? col.prefix : "";
                    var index = proxy.getColumnIndexByField(col.displayColumn), suffix = col.suffix ? col.suffix : "";
                    $($cells[index]).html(prefix + (col.format ? proxy.formatting(col.format, value) : value) + suffix).css("text-align", proxy.model.columns[index].textAlign);
                });
            });
            if (this.model.scrollSettings.frozenColumns > 0) {
                table.first().append($tBody);
                table.last().append($tBodyClone);
            }
            else
                table.append($tBody);
        },
        _getSummaryValues: function (summaryCol, summaryData) {
            var $value, jsonData;
            if (!ej.isNullOrUndefined(summaryData))
                jsonData = summaryData;
            else if (this.model.filterSettings.filteredColumns.length > 0)
                jsonData = this._filteredRecords;
            else
                jsonData = this._dataSource();
            var dbMgr = jsonData instanceof ej.DataManager ? jsonData : ej.DataManager(jsonData);
            switch (summaryCol.summaryType) {
                case ej.Grid.SummaryType.Maximum:
                    var obj = ej.max(jsonData, summaryCol.dataMember);
                    $value = obj[summaryCol.dataMember];
                    break;
                case ej.Grid.SummaryType.Minimum:
                    var obj = ej.min(jsonData, summaryCol.dataMember);
                    $value = obj[summaryCol.dataMember];
                    break;
                case ej.Grid.SummaryType.Average:
                    $value = ej.avg(jsonData, summaryCol.dataMember);
                    break;
                case ej.Grid.SummaryType.Sum:
                    $value = ej.sum(jsonData, summaryCol.dataMember);
                    break;
                case ej.Grid.SummaryType.Count:
                    $value = jsonData.length;
                    break;
                case ej.Grid.SummaryType.TrueCount:
                    var predicate = ej.Predicate(summaryCol.dataMember, "equal", true);
                    $value = dbMgr.executeLocal(ej.Query().where(predicate)).length;
                    break;
                case ej.Grid.SummaryType.FalseCount:
                    var predicate = ej.Predicate(summaryCol.dataMember, "equal", false);
                    $value = dbMgr.executeLocal(ej.Query().where(predicate)).length;
                    break;
                case ej.Grid.SummaryType.Custom:
                    var fn = summaryCol.customSummaryValue;
                    if (fn) {
                        if (typeof fn === "string")
                            fn = ej.util.getObject(fn, window);
                        if ($.isFunction(fn))
                            $value = fn.call();
                    }
                    break;
            }
            return $value;
        },
        _hideCaptionSummaryColumn: function () {
            var headerColumn = this.getHeaderTable().find('.e-headercelldiv[ej-mappingname]').first();
            var captionTd = this.getContentTable().find('.e-groupcaption').clone();
            var groupCaptionParent = this.getContentTable().find('.e-groupcaption').parent();
            var colLength = this.model.columns.length - 1;
            if (this._isCaptionSummary) {
                this.getContentTable().find('.e-summaryrow').remove();
                this.getFooterTable().find("tbody td").slice(-colLength).removeClass("e-groupcaptionsummary").addClass("e-summaryrow");
				if(this.getFooterTable() != null){
                     this.getContentTable().find('.e-recordplusexpand').parent().children('.e-indentcell').remove();
				}
                if (!this.model.groupSettings.showGroupedColumn) {
                    if (!headerColumn.is(':visible')) {
                        this.getContentTable().find('.e-groupcaption').addClass("e-hide");
                        for (i = 0; i < captionTd.length; i++) {
                            groupCaptionParent.eq(i).children().not('.e-hide,.e-recordplusexpand').filter('td.e-groupcaptionsummary:first').replaceWith(captionTd.eq(i));
                        }
                    }
                }
            }
            this.getContentTable().find('.e-recordtable').find('.e-indentcell').remove();
        },
        _hideSummaryColumn: function (td, col) {
            if (col.length > this.model.columns.length)
                col = col.slice(col.length - this.model.columns.length);
            if (!this.model.groupSettings.showGroupedColumn && this.model.showSummary) {
                for (i = 0; i < this.model.columns.length; i++) {
                    for (j = 0; j < this.model.groupSettings.groupedColumns.length || j < this._hiddenColumns.length; j++) {
                        var headerColumn = this.getHeaderTable().find('.e-headercelldiv[ej-mappingname]');
                        if (!headerColumn.eq(i).is(':visible')) {
                            col.eq(i).css("display", "none");
                            $(td[i]).addClass("e-hide");
                            break;
                        }
                        else {
                            if (col.eq(i).css("display") == "none")
                                col.eq(i).css("display", "");
                        }
                    }
                }
            }
            else {
                for (i = 0; i < this.model.columns.length; i++) {
                    if (!this.model.columns[i]["visible"]) {
                        col.eq(i).css("display", "none");
                        $(td[i]).addClass("e-hide");
                    }
                    else {
                        if (col.eq(i).css("display") == "none")
                            col.eq(i).css("display", "");
                    }
                }
            }
        },
        _getRowHeights: function () {
            var trs = this.getRows();
            if (trs !== null) {
                this._rowHeightCollection = [];
                if ((this.model.scrollSettings.frozenColumns > 0 && trs[0] !== undefined) || (typeof trs[0].item !== "undefined" && typeof trs[0].length == "number"))
                    trs = trs[0];
                for (var i = 0 ; i < trs.length ; i++) {
                    this._rowHeightCollection[i] = trs[i].offsetTop;
                }
            }
            return this._rowHeightCollection;
        },
        _initScrolling: function () {
            var frozen = [], unfrozen = [], hideColumns = 0;
            for (var columnCount = 0; columnCount < this.model.columns.length; columnCount++) {
                if (this.model.columns[columnCount].visible === false && columnCount < this.model.scrollSettings.frozenColumns)
                    hideColumns++;
                if (this.model.columns[columnCount]["isFrozen"] === true)
                    frozen.push(this.model.columns[columnCount]);
                else
                    unfrozen.push(this.model.columns[columnCount]);
            }
            this.model.scrollSettings.frozenColumns += hideColumns;
            if (frozen.length > 0) {
                var freeze = this.model.scrollSettings.frozenColumns;
                this.model.columns = $.merge($.merge([], frozen), unfrozen);
                this.model.scrollSettings.frozenColumns = frozen.length;
                if(frozen.length != freeze && freeze != 0)
                    this.model.scrollSettings.frozenColumns = freeze;
            }
            if ((this.model.scrollSettings.frozenColumns > 0 || this.model.scrollSettings.frozenRows > 0) && (this.model.allowGrouping || this.model.rowTemplate != null || this.model.detailsTemplate != null || this.model.scrollSettings.allowVirtualScrolling || this.model.editSettings.editMode == "batch")) {
                this._renderAlertDialog();
                this._alertDialog.find(".e-content").text(this._getLocalizedLabels("FrozenNotSupportedException"));
                this._alertDialog.ejDialog("open");
                return;
            }
            if (this.model.scrollSettings.allowVirtualScrolling && this.model.allowScrolling) {
                this.model.pageSettings.pageSize = this.model.pageSettings.pageSize == 12 ? Math.round(this.model.scrollSettings.height / 32) + 1 : this.model.pageSettings.pageSize;
                this.model.pageSettings.totalPages = Math.ceil(this._gridRecordsCount / this.model.pageSettings.pageSize);
            }
            if (this.model.width || this.model.height) {
                this.model.allowScrolling = true;
                if (this.model.width) this.model.scrollSettings.width = this.model.width;
                if (this.model.height) this.model.scrollSettings.height = this.model.height;
            }
        },
        _checkScrollActions: function (requestType) {
            if (requestType == ej.Grid.Actions.Grouping || requestType == ej.Grid.Actions.Ungrouping || requestType == ej.Grid.Actions.Add || requestType == ej.Grid.Actions.Cancel
                || requestType == ej.Grid.Actions.Save || requestType == ej.Grid.Actions.Delete || requestType == ej.Grid.Actions.Filtering || requestType == ej.Grid.Actions.Paging || requestType == ej.Grid.Actions.Refresh)
                return true;
            return false;
        },
        _refreshScroller: function (args) {
            var gridContent = this.getContent().first(), direction;
            if (ej.isNullOrUndefined(gridContent.data("ejScroller")))
                return;
            if (this.model.scrollSettings.frozenColumns > 0) {
                direction = this.model.enableRTL ? "margin-right" : "margin-left";
                gridContent.find(".e-movablecontent").css(direction, gridContent.find(".e-frozencontentdiv").width() + "px");
                this.getHeaderContent().find(".e-movableheader").removeAttr("style").css(direction, this.getHeaderContent().find(".e-frozenheaderdiv").width() + "px");
                this.refreshScrollerEvent();
                gridContent.find(".e-movablecontent").scrollLeft(this.getHeaderContent().find(".e-movableheader").scrollLeft());
            }
            if (this.model.scrollSettings.frozenRows > 0) {
                this._initFrozenRows();
                var temp = this.getScrollObject().model.scrollTop;
                if (args.requestType == ej.Grid.Actions.Add)
                    this.getScrollObject().scrollY(0, true);
                if (!ej.isNullOrUndefined(this.getScrollObject()._v))
                    this.getScrollObject()._v.skipChange = true;
            }
            if (args.requestType == "beginedit") {
                var temp = this.getScrollObject().model.scrollTop;
                this.getScrollObject().scrollY(0, true);
            }
            gridContent.ejScroller("refresh");
            gridContent.ejScroller("model.enableRTL", this.model.enableRTL);
            if (gridContent.ejScroller("isVScroll") &&  !this.getScrollObject().model.autoHide )
                this.element.find(".e-gridheader").addClass("e-scrollcss");
            else
                this.element.find(".e-gridheader").removeClass("e-scrollcss");
            this._getRowHeights();
            if (temp && args.requestType != ej.Grid.Actions.Add) {
                this._currentTopFrozenRow = 0;
                this.getScrollObject().scrollY(temp, true);
            }
            if (args.requestType == "virtualscroll") {
                var top = this.getScrollObject().model.scrollTop + this.getScrollObject().model.height - (this.getScrollObject().model.height * .3);
                this.getScrollObject().scrollY(top, true);
            }
        },
        _renderScroller: function () {
            if (!this.model.scrollSettings)
                this.model.scrollSettings = {};

            if (this.model.scrollSettings.width || this.model.width)
                this.element.width(this.model.scrollSettings.width || this.model.width);

            var $content = this.getContent().attr("tabindex", "0"), staticWidth, direction, gridRows = this.getRows();
            
            if (this.model.scrollSettings.frozenColumns > 0) {
                scrollWidth = this.getContent().find(".e-frozencontentdiv").width() + 20;
                if (scrollWidth > this.model.scrollSettings.width) {
                    this.getContent().remove();
                    this.getHeaderTable().eq(1).remove();
                    this._alertDialog.find(".e-content").text(this._getLocalizedLabels("FrozenColumnsViewAlert"));
                    this._alertDialog.ejDialog("open");
                    return;
                }
                staticWidth = this.getContent().find(".e-frozencontentdiv").width();
                direction = this.model.enableRTL ? "margin-right" : "margin-left";
                this.getContent().find(".e-movablecontent").css(direction, staticWidth + "px");
                this.getHeaderContent().find(".e-movableheader").css(direction, staticWidth + "px");
                this.model.scrollSettings["targetPane"] = ".e-movablecontent";
            }
            this._initFrozenRows();
            if (this.model.scrollSettings.autoHide)
                this.model.scrollSettings["show"] = $.proxy(this._showHideScroller,this);
            $content.ejScroller(this.model.scrollSettings);
            if (!this.model.scrollSettings.autoHide) 
                this._showHideScroller();
            this.refreshScrollerEvent();
            var proxy = this;
            if (proxy.model.scrollSettings.allowVirtualScrolling) {
                var model = this._refreshVirtualPagerInfo();
                this._showPagerInformation(model);
                $content.ejScroller({
                    scroll: function (e) {
                        e["reachedEnd"] = this.content()[0].scrollHeight - e.scrollTop == this.content()[0].clientHeight;
                        (e.source == "button" || e.source == "key" || e.source == "wheel") && proxy._virtualScroll(e);
                    },
                    thumbEnd: function (e) {
                        if (!$(e.originalEvent.target).hasClass("e-rowcell"))
                            e["reachedEnd"] = this.content()[0].scrollHeight - e.scrollData.sTop == this.content()[0].clientHeight;
                            proxy._virtualScroll(e);
                    }
                });
            }
        },
        _showHideScroller: function () {
            if (this.getBrowserDetails().browser != "msie" && this.model.scrollSettings.frozenColumns == 0) {
                this.getHeaderContent().find("div table").first().width(this.getContentTable().width());
                this.getHeaderTable().width(this.getContentTable().width());
            }
            if (this.getContent().ejScroller("isVScroll")) {
                this.getHeaderContent().find("div").first().addClass("e-headercontent");
                !this.model.scrollSettings.autoHide && this.getHeaderContent().addClass("e-scrollcss")
            } else {
                this.element.find(".e-gridheader").removeClass("e-scrollcss");
                this.getHeaderContent().find("div").first().removeClass("e-headercontent");
            }
        },
        _initFrozenRows: function () {
            var gridRows = this.getRows();
            if (!this.model.currentViewData || this.model.currentViewData.length == 0)
                return;
            if (this.model.scrollSettings.frozenRows > 0 && gridRows != null) {
                this.model.scrollSettings["scroll"] = $.proxy(this._scroll, this);
                this.getContent().find(".e-frozeny").removeClass("e-frozeny")
                    .end().find(".e-frozenrow").removeClass("e-frozenrow");
                if (!ej.isNullOrUndefined(gridRows[0][this.model.scrollSettings.frozenRows - 1]) && !ej.isNullOrUndefined(gridRows[1][this.model.scrollSettings.frozenRows - 1]) && this.model.scrollSettings.frozenColumns > 0)
                    $(gridRows[0][this.model.scrollSettings.frozenRows - 1].cells).add(gridRows[1][this.model.scrollSettings.frozenRows - 1].cells).addClass("e-frozeny").parent().addClass("e-frozenrow");
                else
                    $(gridRows[this.model.scrollSettings.frozenRows - 1].cells).addClass("e-frozeny").parent().addClass("e-frozenrow");
                this.model.scrollSettings.height = this._rowHeightCollection[Math.floor(this.model.scrollSettings.height / this._rowHeightCollection[1])] + 18;
            }
            else
                delete this.model.scrollSettings["scroll"];
        },
        refreshScrollerEvent: function () {
            var proxy = this;
            this.getContent().find(".e-content,.e-movablecontent").scroll(ej.proxy(function (e) {
                if (this.model.scrollSettings.targetPane)
                    this.getHeaderContent().find(".e-movableheader").scrollLeft($(e.currentTarget).scrollLeft());
                else
                    this.getHeaderContent().find("div").first().scrollLeft($(e.currentTarget).scrollLeft());
                if (this.model.scrollSettings.frozenRows > 0 && this.model.editSettings.editMode.indexOf("inlineform") != -1 && this.model.isEdit) {
                    var scrollTop = e.target.scrollTop;
                    this.getContent().find(".e-content").scrollTop(0);
                    this.getScrollObject().scrollY(this.getScrollObject().model.scrollTop + scrollTop, true);
                }
            }, this));
            this.element.find(".e-gridheader").find(".e-headercontent,.e-movableheader").scroll(ej.proxy(function (e) {
                var $currentTarget = $(e.currentTarget);
                if (this.model.scrollSettings.targetPane) {
                    this.getContent().find(".e-movablecontent").scrollLeft($currentTarget.scrollLeft());
                    this.model.showSummary && this.getFooterContent().find(".e-movablefooter").scrollLeft($currentTarget.scrollLeft());;
                }
                else {
                    this.model.showSummary && this.getFooterContent().scrollLeft($currentTarget.scrollLeft());
                    this.getContent().find(".e-content").first().scrollLeft($currentTarget.scrollLeft());
                }
            }, this));
        },
        _renderByFrozenDesign: function () {
            var $div = $(document.createElement('div')), col = this._getMetaColGroup().find("col"), colgroups = {};
            colgroups["colgroup1"] = $div.append(ej.buildTag("colgroup").append(col.splice(0, this.model.scrollSettings.frozenColumns))).html();
            colgroups["colgroup2"] = $div.html(ej.buildTag("colgroup").append(col)).html();
            this.getContent().find("div").first().get(0).innerHTML = $.render[this._id + "_FrozenTemplate"]({ datas: this.model.currentViewData }, colgroups);
            this.setGridContentTable(this.getContent().find(".e-table").attr("role", "grid"));
        },
        addFrozenTemplate: function () {
            var template = "<div class='e-frozencontentdiv'>"
            + "<table cellspacing='0.25px' class='e-table'>{{:~colgroup1}}<tbody>"
            + "{{for datas tmpl='" + this._id + "_JSONFrozenTemplate'/}}"
            + "</tbody></table></div>"
            + "<div class='e-movablecontent'><div class='e-movablecontentdiv'><table cellspacing='0.25px' class='e-table'>{{:~colgroup2}}<tbody>"
            + "{{for datas tmpl='" + this._id + "_JSONTemplate'/}}"
            + "</tbody></table></div></div>", templates = {};
            templates[this._id + "_FrozenTemplate"] = template;
            $.templates(templates);
        },
        _getTopRow: function (offsetTop) {
            var currentTopRow = this.model.scrollSettings.frozenRows, i = 0;
            if (offsetTop > 10) {
                for (var i = 0; i < this._rowHeightCollection.length; i++) {
                    if (this._rowHeightCollection[i] > offsetTop) {
                        currentTopRow = this.model.scrollSettings.frozenRows + i - 1;
                        break;
                    }
                }
            }
            return { imaginaryIndex: currentTopRow, actualIndex: i };
        },
        _showHideRow: function (from, to, action, scrollPosition) {
            var rows = this.getRows();
            if (this.model.scrollSettings.frozenColumns > 0)
                $(rows[0]).slice(from, to).add($(rows[1]).slice(from, to).toArray())[action]();
            else
                $(rows).slice(from, to)[action]();
            this._currentTopFrozenRow = action == "show" ? from : to;
            this.getScrollObject()._changevHandlerPosition(scrollPosition);
        },
        _scroll: function (args) {
            if (args.scrollData != null && args.scrollData.dimension != "width") {
                args.cancel = true;
                var rows = this.getRows(), indexes = this._getTopRow(args.scrollTop), currentTopRow = indexes.imaginaryIndex, frozenRows;
                if (currentTopRow > this._currentTopFrozenRow)
                    this._showHideRow(this.model.scrollSettings.frozenRows, currentTopRow, "hide", this._rowHeightCollection[indexes.actualIndex]);
                else if (currentTopRow < this._currentTopFrozenRow)
                    this._showHideRow(currentTopRow, this._currentTopFrozenRow + 1, "show", this._rowHeightCollection[indexes.actualIndex]);
                args.model.scrollTop = args.scrollTop;
            }
            else {
                if (this.getScrollObject()._v != null)
                    this.getScrollObject()._v.skipChange = true;
            }
        },
        _renderAlertDialog: function () {
            var $contentDiv = ej.buildTag('div.e-content', this._getLocalizedLabels("frozenColumnsMessage"))
                , $buttons = ej.buildTag('span.e-buttons', "<input type='button' id=" + this._id + 'ConfirmDialogOK' + " value='" + this._getLocalizedLabels("OkButton") + "'/>");
            this._alertDialog = ej.buildTag('div#' + this._id + 'AlertDialog');
            this._alertDialog.append($contentDiv).append($buttons);
            this.element.append(this._alertDialog);
            $buttons.find("input").ejButton({
                cssClass: this.model.cssClass,
                showRoundedCorner: true,
                size: "mini",
                click: $.proxy(function (args) {
                    this._alertDialog.ejDialog("close");
                }, this)
            });
            this._renderFDialog(this._id + 'AlertDialog');
            this._alertDialog.ejDialog({ width: "auto", enableModal: true });
        },
        _renderFDialog: function (id) {
            $("#" + id).ejDialog({ showOnInit: false, "enableRTL": this.model.enableRTL, "cssClass": this.model.cssClass, "showHeader": false, width: 260, enableResize: false, allowKeyboardNavigation: false, content: "#" + this._id });
        },
        _virtualScroll: function (e) {
            if (e != null) {
                var flag = 0;
                var recordCount = this.model.filterSettings.filteredColumns.length == 0 ? this._searchCount == null ? this._gridRecordsCount : this._searchCount : this._filteredRecordsCount;
                var pageInfo = this.model.pageSettings;
                var tbody = this.getContentTable()[0].tBodies[0];
                var virtualRows = $(tbody).find('tr.e-virtualrow');
                pageInfo.totalPages = Math.ceil(recordCount / pageInfo.pageSize);
                if (e.scrollTop !== undefined)
                    e.model.scrollTop = e.scrollTop;
                if (e.reachedEnd != undefined) e.model.reachedEnd = e.reachedEnd;
                var currentPageNo = this._calculateCurrenPage(virtualRows, this.getContentTable(), e.model);
                if (currentPageNo > pageInfo.totalPages)
                    currentPageNo = pageInfo.totalPages;
                if (pageInfo.currentPage != currentPageNo && $.inArray((currentPageNo - 1) * pageInfo.pageSize, this.virtualLoadedPages) == -1) {
                    this._isVirtualRecordsLoaded = false;
                }
                if (!this._isVirtualRecordsLoaded) {                    
                    if ($.inArray((currentPageNo - 1) * pageInfo.pageSize, this.virtualLoadedPages) == -1) {
                        if (this.model.scrollSettings.virtualScrollMode == "continuous" && !e.reachedEnd)
                            return
                        if (currentPageNo == pageInfo.totalPages && $.inArray((currentPageNo - 2) * pageInfo.pageSize, this.virtualLoadedPages) == -1) {
                            flag++;
                            this.set_currentPageIndex(currentPageNo - 1);
                        }
                        if (flag == 1) this._lastRow = true;
                        this.set_currentPageIndex(currentPageNo);
                    }
                    pageInfo.currentPage = currentPageNo;
                }
                else
                    pageInfo.currentPage = currentPageNo;
                var model = this._refreshVirtualPagerInfo();
                this._showPagerInformation(model);
            }
        },

        _createPagerStatusBar: function () {
            var $statusBar = this.element.find(".e-pagerstatusbar");
            if ($statusBar.length)
                $statusBar.remove();
            var $pagermsgDiv = ej.buildTag('div.e-pagermsgdiv');
            this.$pagerStatusBarDiv = ej.buildTag('div.e-pagerstatusbar').append($pagermsgDiv);
            this.$pagerStatusBarDiv.appendTo(this.element);
            this.$pagerStatusBarDiv.css("display", "none");
        },
        _refreshVirtualContent: function (currentPage) {
            var rowHeight = this.getRowHeight();
            var recordsCount = this.model.filterSettings.filteredColumns.length == 0 ? this._searchCount == null ? this._gridRecordsCount : this._searchCount : this._filteredRecordsCount;
            if (currentPage != null) {
                this._currentPage(currentPage);
                var model = this._refreshVirtualPagerInfo();
                this._showPagerInformation(model);
            }
            var currentIndex = this.getCurrentIndex();
            var tbody = this.getContentTable()[0].tBodies[0];
            if (this.getCurrentIndex() > 0) {
                var virtualTRTop = document.createElement("tr");
                $(virtualTRTop).addClass("e-virtualrow").css("height", rowHeight * currentIndex).prependTo(tbody);
            } if (currentIndex + this.model.pageSettings.pageSize <= recordsCount) {
                var virtualTRBottom = document.createElement("tr");
                var virtualHeight = this.model.scrollSettings.virtualScrollMode == "normal" ? rowHeight * (recordsCount - (currentIndex + this.model.pageSettings.pageSize)) : 1;
                $(virtualTRBottom).addClass("e-virtualrow").css("height",virtualHeight).appendTo($(tbody));
            }
            this.virtualLoadedPages = new Array();
            this.virtualLoadedPages.push(currentIndex >= 0 ? currentIndex : 0);
            var focusTR = $(tbody).find('tr:not(.e-virtualrow)').attr('name', currentIndex >= 0 ? currentIndex : 0)[0];
            if (focusTR && focusTR.previousSibling && ($(focusTR.previousSibling).hasClass("e-virtualrow") || focusTR.previousSibling.offsetTop > (currentIndex * this.getContent().height()))) {
                this.getContent().children("div").first().scrollTop(this.getContent().find(".content").scrollTop() - (this.getContent().find(".content").scrollTop() - focusTR.offsetTop));
                this._isVirtualRecordsLoaded = true;
            }
        },

        _calculateCurrenPage: function (virtualRows, target, args) {
            var pageSize = this.model.pageSettings.pageSize;
            var currentPage, diff, proxy = this, trEle;
            if (args.scrollTop >= this._scrollValue) {
                currentPage = Math.ceil((args.scrollTop + this.model.scrollSettings.height) / this.getRowHeight() / pageSize);
                if (args.virtualScrollMode == "continuous" && args.reachedEnd) {
                    currentPage = this.virtualLoadedPages[this.virtualLoadedPages.length - 1] / pageSize + 2;
                }
            }
            else
                currentPage = Math.floor((args.scrollTop + this.model.scrollSettings.height) / this.getRowHeight() / pageSize);
            if ($.inArray((currentPage - 1) * pageSize, this.virtualLoadedPages) != -1) {
                var viewTr = this.getContentTable().find("tr").filter(function () { if (this.offsetHeight + this.offsetTop >= args.scrollTop && this.offsetHeight + this.offsetTop <= args.scrollTop + proxy.model.scrollSettings.height) return true; })
                diff = args.scrollTop >= this._scrollValue ? ":last" : ":first";
                trEle = viewTr.filter(diff).eq(0);
                currentPage = trEle.hasClass("e-virtualrow") ? currentPage : parseInt(trEle.attr("name"), 10) / pageSize + 1;
            }
            this._scrollValue = args.scrollTop;
            for (var index = 0; index < virtualRows.length; index++) {
                var val = virtualRows[index];
                if (val.offsetTop + val.offsetHeight >= args.scrollTop) {
                    var prevVirtualPage = this._calculatePrevPage(virtualRows, target, args);
                    this._prevPageNo = prevVirtualPage;
                    if ((currentPage != this.model.pageSettings.totalPages) && currentPage != prevVirtualPage + 1 && $(val).next().is("tr[name=" + (currentPage - 1) * pageSize + "]"))
                        currentPage -= 1;
                    if (currentPage == 0)
                        currentPage = 1;
                    return currentPage > this.model.pageSettings.totalPages ? this.model.pageSettings.totalPages : currentPage;
                }
            }
            return currentPage;
        },
        _calculatePrevPage: function (virtualRows, target, args) {
            for (var i = 0; i < virtualRows.length; i++) {
                var val = virtualRows[i];
                if (val.offsetTop + val.offsetHeight >= args.scrollTop) {
                    var trElement = $(val).prevAll('tr[name]')[0];
                    if (trElement != null) {
                        return Math.ceil(parseInt($(trElement).attr('name'), 10) / this.model.pageSettings.pageSize) + 1;
                    }
                }
            }
            return -1;
        },
        _refreshVirtualPagerInfo: function () {
            var model = {};
            model.pageSize = this.model.pageSettings.pageSize;
            model.currentPage = this._currentPage();
            model.totalRecordsCount = this.model.filterSettings.filteredColumns.length == 0 ? this._searchCount == null ? this._gridRecordsCount : this._searchCount : this._filteredRecordsCount;
            model.totalPages = Math.ceil(model.totalRecordsCount / model.pageSize);

            return model;
        },
        _showPagerInformation: function (model) {
            var from = (model.currentPage - 1) * model.pageSize;
            $(this.$pagerStatusBarDiv).find("div:first").html(String.format(this._getLocalizedLabels("PagerInfo"), model.currentPage, model.totalPages, model.totalRecordsCount), from, from + model.pageSize);
            $(this.$pagerStatusBarDiv).css('display', 'block');
        },
		_cellMerging: function (args) {
            args.colMerge = function (range) {
                if (this.cell.className.indexOf("e-colmerge") == -1) {
                    this.cell.className += " e-colmerge";
                    if (this.model.columns.length - this.cell.cellIndex < range)
                        range = this.model.columns.length - this.cell.cellIndex;
                    this.cell.colSpan = range;
                    for (var i = 1; i < range; i++) {
                        if (!ej.isNullOrUndefined(this.cell.parentElement.children[this.cell.cellIndex + i]))
                            this.cell.parentElement.children[this.cell.cellIndex + i].className += " e-hide";
                    }
                }
            };
            args.rowMerge = function (range) {
                if (this.cell.className.indexOf("e-rowmerge") == -1) {
                    this.cell.className += " e-rowmerge";
                    if (this.model.currentViewData.length - this.cell.parentElement.rowIndex < range)
                        range = this.model.currentViewData.length - this.cell.parentElement.rowIndex;
                    this.cell.rowSpan = range;
                    for (var i = 0; i < range - 1; i++) {
                        var ele = this.cell.parentNode.parentNode;
                        if (!ej.isNullOrUndefined(ele.children[this.cell.parentElement.rowIndex + i].nextSibling)) {
                            if (!($(".e-grid").children().is('.e-dialog')) || ($(".e-grid").find('.e-dialog').attr("style").indexOf("display: none")) != -1)
                                ele.children[this.cell.parentElement.rowIndex + i].nextSibling.children[this.cell.cellIndex].className += " e-merged e-hide";
                        }
                        else
                            break;
                    }
                }
            };
            args.merge = function (col, row) {
                if (col > 1 && row > 1) {
                    if (this.cell.className.indexOf("e-colmerge") == -1) {
                        this.cell.className += " e-colmerge";
						if (this.model.currentViewData.length - this.cell.parentElement.rowIndex < row)
							row = this.model.currentViewData.length - this.cell.parentElement.rowIndex;
                        if (!($(".e-grid").children().is('.e-dialog')) || ($(".e-grid").find('.e-dialog').attr("style").indexOf("display: none")) != -1) {
                            for (var i = 0; i < row ; i++) {
                                var ele = this.cell.parentNode.parentNode;
                                if (!ej.isNullOrUndefined(ele.children[this.cell.parentElement.rowIndex + i])) {
                                    var selectCell = ele.children[this.cell.parentElement.rowIndex + i].children[this.cell.cellIndex];
									if (this.model.columns.length - selectCell.cellIndex < col)
										col = this.model.columns.length - selectCell.cellIndex;
                                    selectCell.colSpan = col;
                                    for (var j = 1; j < col; j++) {
                                        if (!ej.isNullOrUndefined(selectCell.parentElement.children[this.cell.cellIndex + j]))
                                            selectCell.parentElement.children[this.cell.cellIndex + j].className += " e-hide";
                                    }
                                }
                                else
                                    break;
                            }
                        }
                        else {
                            this.cell.colSpan = col;
                            for (var j = 1; j < col; j++) {
                                if (!ej.isNullOrUndefined(this.cell.nextSibling))
                                    parentElement.children[this.cell.cellIndex + j].className += " e-hide";
                            }
                        }
                        args.rowMerge(row);
                    }
                }
                else {
                    if (col > 1)
                        args.colMerge(col);
                    if (row > 1)
                        args.rowMerge(row);
                }
            };
        },
        _replacingContent: function () {
            var temp = document.createElement('div');
            var currentIndex = this.getCurrentIndex();
            var contentTable = this.getContentTable()[0];
            var colGroup = $(contentTable).find("colgroup").first();
            var rowHeight = this.getRowHeight();
            colGroup.replaceWith(this._getMetaColGroup());
            this.model.detailsTemplate != null && colGroup.prepend(this._getIndentCol());
            var tbody = contentTable.tBodies[0];
            var cloneEle = $(tbody).clone();
            var elementTbody = $("<tbody></tbody>").append($.render[this._id + "_JSONTemplate"](this.model.currentViewData));
            var proxy = this;
            this.virtualLoadedPages.push(currentIndex >= 0 ? currentIndex : 0);
            var orderedVirtualPages = ej.dataUtil.mergeSort(ej.distinct(this.virtualLoadedPages));
            var tempTbody = $("<tbody></tbody>");
            $(elementTbody).children('tr').attr('name', currentIndex);
            var minValue = ej.dataUtil.min(orderedVirtualPages);
            var maxValue = ej.dataUtil.max(orderedVirtualPages);
            $(cloneEle).find('tr.e-virtualrow').remove();
            $(cloneEle).append($(elementTbody).children('tr'));
            $.each(orderedVirtualPages, function (index, val) {
                $(tempTbody).append($(cloneEle).find('tr[name=' + val + ']'));
                if (val != 0) {
                    var prevValue = val == minValue ? minValue : orderedVirtualPages[index - 1];
                    var virtualTRMiddle = document.createElement("tr");
                    var middleRows = val - prevValue - proxy.model.pageSettings.pageSize;
                    if (middleRows > 0)
                        $(virtualTRMiddle).addClass("e-virtualrow").css("height", rowHeight * middleRows).insertBefore($(tempTbody).children('tr[name=' + val + ']:first'));
                }
                if (val == maxValue) {
                    var virtualTRBottom = document.createElement("tr");
                    var bottomRows = proxy._gridRecordsCount - maxValue - proxy.model.pageSettings.pageSize;
                    if (bottomRows > 0)
                        $(virtualTRBottom).addClass("e-virtualrow").css("height", rowHeight * bottomRows).appendTo(tempTbody);
                }
            });
            if (minValue > 0) {
                var virtualTRTop = document.createElement("tr");
                $(virtualTRTop).addClass("e-virtualrow").css("height", rowHeight * minValue).prependTo(tempTbody);
            }
            var focusTR = $(tbody).html($(tempTbody).html()).find("tr[name=" + currentIndex + "]")[0];
            if ((focusTR && focusTR.previousSibling && ($(focusTR.previousSibling).hasClass("e-virtualrow") || focusTR.previousSibling.offsetTop > (currentIndex * this.getContent().height()))
            && (this._gridRecordsCount - currentIndex > this.model.pageSettings.pageSize || focusTR.offsetParent.offsetHeight - focusTR.offsetTop < this.getContent().height())) || this._lastRow) {
                if (this._lastRow) this._lastRow = false;
                this._isVirtualRecordsLoaded = true;
                this.getContent().children("div").first().scrollTop(this.getContent().find(".content").scrollTop() - (this.getContent().find(".content").scrollTop() - focusTR.offsetTop));
            }
            $(temp).append($("<table></table>").append(tempTbody));
            $(contentTable).get(0).replaceChild(this.model.rowTemplate != null ? temp.firstChild.lastChild : temp.firstChild.firstChild, $(contentTable).get(0).lastChild);
            this._currentJsonData = this.model.currentViewData;
            this._gridRows = $(contentTable).get(0).rows;
            var lastVirtualRow = $(contentTable).find(".e-virtualrow").last();
            var lastVirtualRowHeight = this.model.scrollSettings.virtualScrollMode == "normal" ? (lastVirtualRow.height() - ($(contentTable).height() - (this._gridRecordsCount * rowHeight))) : 1;
            lastVirtualRow.css("height",lastVirtualRowHeight);           
            this._eventBindings();
        },
        _refreshPagerTotalRecordsCount: function () {
            if (this.model.filterSettings.filteredColumns.length)
                this.getPager().ejPager({ totalRecordsCount: this._searchCount == null ? this._filteredRecordsCount : this._searchCount, currentPage: this._currentPage() });
            else
                this.getPager().ejPager({ totalRecordsCount: this._searchCount == null ? this._gridRecordsCount : this._searchCount, currentPage: this._currentPage() });
        },
        _maxZindex: function () {
            var maxZ = 1;
            maxZ = Math.max.apply(null, $.map($('body *'), function (e, n) {
                if ($(e).css('position') == 'absolute')
                    return parseInt($(e).css('z-index')) || 1;
            }));
            if (maxZ == undefined || maxZ == null)
                maxZ = 1;
            return maxZ;
        },
        _keyPressed: function (action, target, e) {
            var $target = $(target);
            if (!this.model.allowKeyboardNavigation || (target.tagName == 'INPUT' && e.code != 40 && e.code != 38 && e.code != 13 && e.code != 27 && e.code != 9) || String.fromCharCode(e.code).toLowerCase() == this.element[0].accessKey.toLowerCase())
                return true;
			if(this.model.allowFiltering && ($target.hasClass('e-filtertext') && e.code==13))
				return true;
            if ((this.model.editSettings.editMode == "dialog" || this.model.editSettings.editMode == "dialogtemplate") && $(target).closest("#" + this._id + "EditForm").length)
                return true;
            else if ($(target).parent().siblings("#" + this._id + "EditForm").length)
                return true;
            if (e.code == 13 && $target.parent().hasClass("e-unboundcelldiv"))
                return true;
            if (e.code == 13 && target.tagName == 'INPUT' && $target.parent().is("#" + this._id + "_search"))
                action = "searchRequest";
            if (this.getPager() != null)
                var pager = this.getPager().ejPager("model"), pageIndex = pager.currentPage;
            var returnValue = false, curEl, $target = $(target);

            switch (action) {
                case "insertRecord":
                    if (ej.gridFeatures.edit)
                        this._toolbarOperation(this._id + "_add");
                    break;
                case "searchRequest":
                    this.search($target.val());
                    break;
                case "saveRequest":
                    if (ej.gridFeatures.edit) {
                           $target.blur();
                        if (this.model.editSettings.editMode == "batch")
                            this._moveCurrentCell("down");
                        else
                            this._toolbarOperation(this._id + "_update");
                    }
                    break;
                case "cancelRequest":
                    if (ej.gridFeatures.edit)
                        this._toolbarOperation(this._id + "_cancel");
                    break;
                case "deleteRecord":
                    if (ej.gridFeatures.edit)
                        this._toolbarOperation(this._id + "_delete");
                    break;
                case "editRecord":
                    if (ej.gridFeatures.edit)
                        this._toolbarOperation(this._id + "_edit");
                    break;
                case "totalGroupCollapse":
                    if (ej.gridFeatures.group)
                        this.collapseAll();
                    break;
                case "totalGroupExpand":
                    if (ej.gridFeatures.group)
                        this.expandAll();
                    break;
                case "selectedGroupExpand":
                    if (ej.gridFeatures.group) {
                        this._$currentTr = $(this.getRows()).eq(this._selectedRow());
                        curEl = this._$currentTr.parents("tr").first().prev().find(".e-recordpluscollapse");
                        this.expandCollapse(curEl);
                    }
                    break;
                case "selectedGroupCollapse":
                    if (ej.gridFeatures.group) {
                        this._$currentTr = $(this.getRows()).eq(this._selectedRow());
                        curEl = this._$currentTr.parents("tr").first().prev().find(".e-recordplusexpand");
                        this.expandCollapse(curEl);
                    }
                    break;
                case "firstRowSelection":
                    if (ej.gridFeatures.selection)
                        this.selectRows(0);
                    break;
                case "lastRowSelection":
                    if (ej.gridFeatures.selection)
                        this.selectRows($(this.getRows()).length - 1);
                    break;
                case "upArrow":
                    if (ej.gridFeatures.selection) {
                        if ((target["type"] == "text" || target["type"] == "checkbox") && this.model.isEdit && this.model.editSettings.editMode != "batch")
                            return true;
                        if (this._selectedRow() != 0) {
                            this.selectRows(this._selectedRow() - 1);
                            if (this.model.editSettings.editMode == "batch")
                                this._moveCurrentCell("up");
                        }
                        if (this._previousRowCellIndex && this._previousRowCellIndex[0][0] != 0) {
                            this.selectCells([[this._previousRowCellIndex[0][0] - 1, this._previousRowCellIndex[0][1]]]);
                            if (this.model.editSettings.editMode == "batch")
                                this._moveCurrentCell("down");
                        }
                    }
                    break;
                case "downArrow":
                    if (ej.gridFeatures.selection) {
                        if ((target["type"] == "text" || target["type"] == "checkbox") && this.model.isEdit && this.model.editSettings.editMode != "batch")
                            return true;
                        var lastRow = $(this.getRows()).length - 1;
                        if (this.model.scrollSettings.frozenColumns > 0 && !ej.isNullOrUndefined(this.getRows()[0]))
                            lastRow = this.getRows()[0].length - 1;
                        if (this._selectedRow() != lastRow) {
                            this.selectRows(this._selectedRow() + 1);
                            if (this.model.editSettings.editMode == "batch")
                                this._moveCurrentCell("down");
                        }
                        if (this._previousRowCellIndex && this._previousRowCellIndex[0][0] != lastRow && this._allowcellSelection) {
                            this.selectCells([[this._previousRowCellIndex[0][0] + 1, this._previousRowCellIndex[0][1]]]);
                            if (this.model.editSettings.editMode == "batch")
                                this._moveCurrentCell("down");
                        }
                    }
                    break;
                case "rightArrow":
                    if (ej.gridFeatures.selection && this._allowcellSelection) {
                        if ((target["type"] == "text" || target["type"] == "checkbox") && this.model.isEdit && this.model.editSettings.editMode != "batch")
                            return true;
                        var lastRow = $(this.getRows()).length - 1;
                        if (this._previousRowCellIndex && this._previousRowCellIndex[0][1] == this.model.columns.length - 1 && this._previousRowCellIndex[0][0] != lastRow) {
                            this.selectCells([[this._previousRowCellIndex[0][0] + 1, [0]]]);
                        } else if (this._previousRowCellIndex && this._previousRowCellIndex[0][1] != this.model.columns.length - 1)
                            this.selectCells([[this._previousRowCellIndex[0][0], [parseInt(this._previousRowCellIndex[0][1]) + 1]]]);

                    }
                    break;
                case "leftArrow":
                    if (ej.gridFeatures.selection && this._allowcellSelection) {
                        if ((target["type"] == "text" || target["type"] == "checkbox") && this.model.isEdit && this.model.editSettings.editMode != "batch")
                            return true;
                        if (this._previousRowCellIndex && this._previousRowCellIndex[0][1] == 0 && this._previousRowCellIndex[0][0] != 0)
                            this.selectCells([[this._previousRowCellIndex[0][0] - 1, [this.model.columns.length - 1]]]);
                        else if (this._previousRowCellIndex && this._previousRowCellIndex[0][1] != 0)
                            this.selectCells([[this._previousRowCellIndex[0][0], [parseInt(this._previousRowCellIndex[0][1]) - 1]]]);

                    }
                    break;
                case "firstCellSelection":
                    if (ej.gridFeatures.selection && this._allowcellSelection) {
                        if ((target["type"] == "text" || target["type"] == "checkbox") && this.model.isEdit && this.model.editSettings.editMode != "batch")
                            return true;
                        var lastRow = $(this.getRows()).length - 1;
                        lastRow > -1 && this.selectCells([[0, [0]]]);
                    }
                    break;
                case "lastCellSelection":
                    if (ej.gridFeatures.selection && this._allowcellSelection) {
                        if ((target["type"] == "text" || target["type"] == "checkbox") && this.model.isEdit && this.model.editSettings.editMode != "batch")
                            return true;
                        var lastRow = $(this.getRows()).length - 1;
                        lastRow > -1 && this.selectCells([[lastRow, [this.model.columns.length - 1]]]);
                    }
                    break;
                case "nextPage":
                    if (this.getPager() != null)
                        pageIndex = pageIndex + 1;
                    break;
                case "previousPage":
                    if (this.getPager() != null)
                        pageIndex = pageIndex - 1;
                    break;
                case "lastPage":
                    if (this.getPager() != null)
                        pageIndex = pager.totalPages;
                    break;
                case "firstPage":
                    if (this.getPager() != null)
                        pageIndex = 1;
                    break;
                case "nextPager":
                    if (this.getPager() != null)
                        pageIndex = Math.ceil(pager.currentPage / pager.pageCount) * pager.pageCount + 1;
                    break;
                case "previousPager":
                    if (this.getPager() != null)
                        pageIndex = (Math.floor(pager.currentPage / pager.pageCount) - 1) * pager.pageCount + 1;
                    break;
                case "moveCellLeft":
                    if (this.model.editSettings.editMode == "batch")
                        returnValue = this._moveCurrentCell("left");
                    else
                        returnValue = true;
                    break;
                case "moveCellRight":
                    if (this.model.editSettings.editMode == "batch" && $target)
                        returnValue = this._moveCurrentCell("right");
                    else
                        returnValue = true;
                    break;
                default:
                    returnValue = true;
            }
            if (this. getPager() != null && pager.currentPage !== pageIndex)
                this.getPager().ejPager("goToPage", pageIndex);
            return returnValue;
        },
        _findColumnsWidth: function () {
            var j = this.getHeaderTable().find(".e-headercell");            
            for (var i = 0; i < this.model.columns.length; i++)
                this._columnsWidthCollection[i] = j.eq(i).outerWidth();
        },
        _calculateWidth: function () {
            var j = this.getHeaderTable().find("colgroup").find("col"), width = 0;
            for (var i = 0; i < j.length; i++)
                width += j.eq(i).width();
            return width;

        },
        _initIndicators: function () {
            var indicatorId = this._id + "_ColumnDropIndicator";
            if ($("#" + indicatorId).length)
                $("#" + indicatorId).remove();
            this._Indicator = document.createElement("DIV");
            $(this._Indicator).attr('id', indicatorId).addClass("e-columndropindicator").addClass("e-dropAcceptor").appendTo(document.body);
            $(this._Indicator).css({ "display": "none" });

        },
        /**
         * Re-order the column in grid
         * @alias ejGrid#reorderColumns
         * @return jQuery
         * @param {string} fromFieldName Pass the from field name of the column needs to be changed
         * @param {string} toFieldName Pass the to field name of the column needs to be changed
         * @example 
         * &lt;script&gt;
         * // Create grid object.
         * var gridObj = $("#Grid").data("ejGrid");
         * // Reorders the column based on the given index
         * gridObj.reorderColumns("OrderID", "CustomerID"); 
         * &lt;/script&gt;
         * @example 
         * &lt;script&gt;
         * // Reorders the column based on the given index
         * $("#Grid").ejGrid("reorderColumns", "OrderID", "CustomerID");
         * &lt;/script&gt;
         */
        reorderColumns: function (fromfname, tofname) {
            var fromindex = this.model.columns.indexOf(this.getColumnByField(fromfname));
            var toindex = this.model.columns.indexOf(this.getColumnByField(tofname));
            this.set_dropColumn(fromindex, toindex);
        },
        /**
		 * Add or remove columns in grid column collections
         * @alias ejGrid#columns
		 * @return jQuery
		 * @param {array/string} columndetails Pass array of columns or string of field name to add/remove the column in grid 
		 * @param {string} [action] Pass add/remove action to be performed. By default "add" action will perform
		 * @example 
		 * &lt;script&gt;
		 * // Create grid object.
		 * var gridObj = $("#Grid").data("ejGrid");
         * // remove grid column
		 * gridObj.columns("OrderID", "remove");
         * // Add new column into grid or modified already existing column in the grid.
		 * gridObj.columns("CustomerID", "add"); 
		 * &lt;/script&gt;
		 * @example 
		 * &lt;script&gt;
		 * // remove grid column
		 * $("#Grid").ejGrid("columns","OrderID", "remove");   
		 * // Add new column into grid or modified already existing column in the grid.			
		 * $("#Grid").ejGrid("columns","CustomerID", "add");    			
		 * &lt;/script&gt;
         */
        columns: function (details, action) {
            if (ej.isNullOrUndefined(details)) return;
            var isString = false;
            if (typeof details === "string") {
                details = [details];
                isString = true;
            }
            else if (details instanceof Array && details.length && typeof details[0] === "string")
                isString = true;
            for (i = 0; i < details.length; i++) {
                var index = $.inArray(this.getColumnByField(isString ? details[i] : details[i]["field"]), this.model.columns);
                if (action == "add" || ej.isNullOrUndefined(action)) {
                    if (index == -1)
                        this.model.columns.push(isString ? { field: details[i] } : details[i]);
                    else
                        this.model.columns[index] = isString ? { field: details[i] } : details[i];
                }
                else {
                    if (index != -1)
                        this.model.columns.splice(index, 1);
                }
            }
            var $header = this.element.find(".e-gridheader");
            $header.find("div").first().empty().append(this._renderGridHeader().find("table"));
            this._headerCellgDragDrop();
            this.refreshContent(true);
        },
        _enableRowHover: function () {
            if (this.model.enableRowHover)
                this._on(this.element, "mouseenter mouseleave", ".e-gridcontent tr td", this._rowHover);
            else
                this._off(this.element, "mouseenter mouseleave", ".e-gridcontent tr td");
        },
        _rowHover: function (e) {
            if (!this.model.enableRowHover)
                return;
            var $target = $(e.target);
            var $gridRows = $(this.getRows());
            if (($target.hasClass("e-rowcell") && $target.closest("#" + this._id + "EditForm").length) || !$target.hasClass("e-rowcell"))
                return;
            if (e.type == "mouseenter" && !this._dragActive) {
                if (this.model.scrollSettings.frozenColumns > 0 && !ej.isNullOrUndefined($gridRows[0]) && !ej.isNullOrUndefined($gridRows[1]))
                    $gridRows = $($gridRows[0]).add($gridRows[1]);
                $gridRows.removeClass("e-hover");
                var index = this.getIndexByRow($target.parent());
                index != -1 && this.getRowByIndex(index).addClass("e-hover");
            } else {
                if (this.model.scrollSettings.frozenColumns > 0 && !ej.isNullOrUndefined($gridRows[0]) && !ej.isNullOrUndefined($gridRows[1]))
                    $gridRows = $($gridRows[0]).add($gridRows[1]);
                $gridRows.removeClass("e-hover");
            }
            return false;
        },
        _rightClickHandler: function (e) {
            e.preventDefault();
            if (e.which == 3) {
                var args = {};
                $target = $(e.target);
                $gridRow = $(this.getRows());
                if (this.getContentTable().has($target).length) {
                    var index = $gridRow.index($target.parent());
                    if (index == -1)
                        return;
                    args = { rowIndex: index, row: this.getRowByIndex(index), data: this._currentJsonData[index], cellIndex: $target.index(), cellValue: $target.html(), cell: $target };
                }
                else if (this.getHeaderTable().has($target).length) {
                    var index = 0
                    $th = this.getHeaderTable().find('th').not('.e-detailheadercell,.e-grouptopleftcell,.e-filterbarcell');
                    if ($target.is('.e-headercelldiv'))
                        index = $th.index($target.closest('.e-headercell'));
                    else
                        index = $th.index($target);
                    if (index == -1)
                        return;
                    args = { headerIndex: index, headerText: this.getColumnFieldNames()[index], headerCell: $th.eq(index), column: this.getColumnByIndex(index) }
                }
                else if ($target.is('.e-pager') || (this. getPager() != null && this.getPager().has($target).length)) {
                    args = { pager: this.model.pageSettings }
                }
                this._trigger("rightClick", args);
            }
        },
        _touchGrid: function (e) {
            var curPage = this._currentPage();
            var lastPage = this._gridRecordsCount / this.model.pageSettings.pageSize;
            var totPage = lastPage == 0 ? lastPage : (parseInt(lastPage, 10) + 1);
            switch (e.type) {
                case "swipeleft":
                    if (this.model.allowPaging && curPage != totPage && !this.model.isEdit)
                        this.element.ejGrid("gotoPage", curPage + 1);
                    break;
                case "swiperight":
                    if (this.model.allowPaging && curPage > 1 && !this.model.isEdit)
                        this.element.ejGrid("gotoPage", curPage - 1);
                    break;
            }
        },
        _recorddblClickHandler: function () {
            var args = {};
            args = { rowIndex: this._selectedRow(), row: this.getRowByIndex(this._selectedRow()), data: this._currentJsonData[this._selectedRow()], cell: $target };
            this._trigger("recordDoubleClick", args);
        },
        _recordClick: function (e) {
            var args = {};
            $target = $(e.target);
            if (!$target.is('.e-rowcell') && !$target.closest("td").is(".e-rowcell"))
                return;
            var index = this.getIndexByRow($target.closest('tr'));
            args = { rowIndex: index, row: this.getRowByIndex(index), data: this._currentJsonData[index], cell: $target.closest("td") };
            this._trigger("recordClick", args);
        },
        _headerMouseDown: function (e) {
            if (this.model.allowResizing)
                this._resizer._mouseDown(e);
            if (($(e.target).hasClass("e-headercelldiv") && !$(e.target).parent().hasClass("e-grouptopleftcell")) || $(e.target).hasClass("e-headercell")) {
                var $headercell = $(e.target).hasClass("e-headercelldiv") ? $(e.target).parent() : $(e.target);
                this.model.enableHeaderHover && $headercell.removeClass("e-hover e-headercell-hover").addClass("e-headercellactive e-active");
            }
        },
        _headerHover: function (e) {
            var $target = $(e.target);
            if (e.type == "mouseover" || e.type == "mousemove" || e.type == "touchmove" || e.type == "MSPointerMove") {
                if (this.model.allowResizing || this.model.allowResizeToFit)
                    this._resizer._mouseHover(e);

                if (this.model.enableHeaderHover && !this._dragActive && (($target.hasClass("e-headercelldiv") && !$target.parent().hasClass("e-grouptopleftcell")) || $target.hasClass("e-headercell"))) {
                    if ($target.hasClass("e-headercelldiv"))
                        $target = $target.parent();
                    this.getHeaderTable().find(".e-columnheader").find(".e-headercell-hover").removeClass("e-headercell-hover").removeClass("e-hover");
                    $target.addClass("e-headercell-hover e-hover");
                }

            } else
                this.model.enableHeaderHover && this.getHeaderTable().find(".e-columnheader").find(".e-headercell-hover").removeClass("e-headercell-hover").removeClass("e-hover");
        },
        _colgroupRefresh: function () {
            var gridheaderCol = $(this.getHeaderTable()).find('colgroup')[0];
            var gridcontentCol = $(this.getContentTable()).find('colgroup')[0];
            var headerColClone = $(gridheaderCol).clone();
            var contentColClone = $(gridcontentCol).clone();
            $(gridcontentCol).remove();
            $(gridheaderCol).remove();
            $(headerColClone).prependTo(this.getHeaderTable());
            $(contentColClone).prependTo(this.getContentTable());
        },
        _headerdblClickHandler: function (e) {
            if (this.model.allowResizeToFit)
                this._resizer._columnResizeToFit(e);
        },        
       
        _mouseUp: function (e) {
            if (this.model.allowResizing)
                this._resizer._mouseUp(e);
        },

        _mouseMove: function (e) {
            if (this.model.allowResizing)
                this._resizer._mouseMove(e);
        },
        _setModel: function (options) {
            for (var prop in options) {
                switch (prop) {
                    case "enableResponsiveRow":
                        if (options[prop])
                            this.element.addClass("e-responsive");
                        else
                            this.element.removeClass("e-responsive"); 
                        break;  
                    case "pageSettings":
                        var pageModel = this.getPager().ejPager("model");
                        if (ej.isNullOrUndefined(options[prop]["currentPage"]) || pageModel.currentPage != this._currentPage()) {
                            for (var pageProp in options[prop]) {
                                if (pageProp != "currentPage" && options[prop][pageProp] === pageModel[pageProp])
                                    delete options[prop][pageProp];
                            }
                            if ($.isEmptyObject(options[prop]))
                                break;
							options[prop]["currentPage"] = this._currentPage();
                            this.getPager().ejPager("option", options[prop]);
							this._currentPage(this._currentPage()>pageModel.totalPages?pageModel.totalPages:this._currentPage());
                            this.refreshContent();
                        }
                        break;
                    case "allowPaging":
                        this.model.allowPaging = options[prop];
                        if (options[prop] && this.element.children(".e-pager").length == 0) {
                            this.element.append(this._renderGridPager());
                            this.refreshContent();
                            this.getPager().ejPager("refreshPager");
                        } else {
                            this.getPager().remove();
                            this.setGridPager(null);
                            this.refreshContent();
                            if (this.model.filterSettings.filterType == "filterbar" && this.model.allowFiltering)
                                this._createPagerStatusBar();
                        }
                        break;
                    case "allowGrouping":
                        if (options[prop] && this.element.children(".e-groupdroparea").length == 0) {
                            this.model.allowGrouping = options[prop];
                            this.addGroupingTemplate();
                            this.element.prepend(this._renderGroupDropArea());
                            this._enableGroupingEvents();
                            this._headerCellgDragDrop();
                        } else
                            this.element.children(".e-groupdroparea").remove();
                        break;
                    case "groupSettings":
                        $.extend(this.model.groupSettings, options[prop]);
                        this.refreshTemplate();
						if (!ej.isNullOrUndefined(options[prop]["groupedColumns"])) {
                            var args = {};
                            args.columnName = this.model.groupSettings.groupedColumns[this.model.groupSettings.groupedColumns.length - 1];
                            args.requestType = ej.Grid.Actions.Grouping;
                            $(".e-grid").find(".e-groupdroparea").empty();
                            for (var i = 0; i < this.model.groupSettings.groupedColumns.length - 1; i++)
                                this._addColumnToGroupDrop(this.model.groupSettings.groupedColumns[i]);
                            this._processBindings(args);
                        }
                        if (this.model.groupSettings.showToggleButton) {
                            var $table = this._renderGridHeader().find("table");
                            this.element.children(".e-gridheader").find("table").replaceWith($table);
                            this.setGridHeaderTable($table);
                            this.setWidthToColumns();
                        }
                        if (!this.model.groupSettings.enableDropAreaAnimation)
                            this.expandGroupDropArea();
                        else
                            this.collapseGroupDropArea();
                        break;
                    case "cssClass":
                        this.element.removeClass(this.model.cssClass).addClass(options[prop]);
                        break;
                    case "allowFiltering":
                    case "filterSettings":
                        if (prop == "filterSettings")
                            $.extend(this.model.filterSettings, options[prop]);
                        else
                            this.model.allowFiltering = options[prop];
                        this._$fDlgIsOpen && (this._isExcelFilter ? this._excelFilter.closeXFDialog() : this._closeFilterDlg())
                        if (!this.model.allowFiltering) {
                            if (this.model.filterSettings.filterType == ej.Grid.FilterType.FilterBar)
                                this.getHeaderTable().find(".e-filterbar").remove();
                            else if (this.model.filterSettings.filterType == ej.Grid.FilterType.Menu || this.model.filterSettings.filterType == ej.Grid.FilterType.Excel)
                                this.getHeaderTable().find(".e-columnheader").find(".e-filtericon").remove()
                                    .end().find(".e-headercellfilter").removeClass("e-headercellfilter");
                            this.model.filterSettings.filteredColumns = [];
                            this.refreshContent();
                        } else {
                            if (this.model.filterSettings.filterType == ej.Grid.FilterType.FilterBar) {
                                this.getHeaderTable().find(".e-filterbar").remove();
                                this.getHeaderTable().find(".e-columnheader").find(".e-filtericon").remove()
                                    .end().find(".e-headercellfilter").removeClass("e-headercellfilter");
                                this._renderFiltering();
                                if (this.model.filterSettings.showFilterBarStatus && !this.model.allowPaging)
                                    this._createPagerStatusBar();
                                else if (this.model.filterSettings.showFilterBarStatus && this.model.allowPaging)
                                    this.getPager().ejPager({ enableExternalMessage: true });
                                var $filterbar = this.getHeaderTable().find(".e-filterbar");
                                for (var i = 0; i < this.model.groupSettings.groupedColumns.length; i++)
                                    $filterbar.prepend(this._getEmptyFilterBarCell());
                            } else if (this.model.filterSettings.filterType == ej.Grid.FilterType.Menu || this.model.filterSettings.filterType == ej.Grid.FilterType.Excel) {
                                this.getHeaderTable().find(".e-columnheader").find(".e-filtericon").remove()
                                    .end().find(".e-headercellfilter").removeClass("e-headercellfilter");
                                this.getHeaderTable().find(".e-filterbar").remove();
                                this.getHeaderTable().find(".e-columnheader").find(".e-headercell").addClass("e-headercellfilter").append(ej.buildTag('div.e-filtericon e-icon e-filterset'));
                                if (this.model.filterSettings.filterType == ej.Grid.FilterType.Excel) {
                                    this._isExcelFilter = true;
                                    this._renderExcelFilter();                                                                        
                                } else if (this._isExcelFilter) {
                                    this._isExcelFilter = false;
                                    this._excelFilter.resetExcelFilter();
                                    this._excelFilter = null;
                                }
                                this._renderFilterDialogs();
                                this.model.filterSettings.filteredColumns = [];
                                this.refreshContent();
                            }
                            this._enableFilterEvents();
                        }
                        break;
                    case "enableRowHover":
                        this.model.enableRowHover = options[prop];
                        this._enableRowHover();
                        break;
                    case "allowScrolling":
                    case "scrollSettings":
                        var $content = this.getContent();
                        if (!ej.util.isNullOrUndefined(options["scrollSettings"]))
                            $.extend(this.model.scrollSettings, options["scrollSettings"]);
                        if (options["scrollSettings"]["frozenColumns"] !== undefined || options["scrollSettings"]["frozenRows"] !== undefined || 
                            options["scrollSettings"]["allowVirtualScrolling"] !== undefined || options["scrollSettings"]["virtualScrollMode"] !== undefined) {
                            var model = this.model;
                            model.query = this.commonQuery.clone();
                            if(options["scrollSettings"]["virtualScrollMode"] != undefined)
                                model.pageSettings.currentPage = 1;
                            this.element.ejGrid("destroy").ejGrid(model);
                        }
                        else {
                            if (!ej.util.isNullOrUndefined(options["allowScrolling"]))
                                this.model.allowScrolling = options["allowScrolling"];
                            !ej.util.isNullOrUndefined($content.data("ejScroller")) && $content.ejScroller("destroy");
                            if (this.model.allowScrolling) {
                                this.getHeaderTable().parent().addClass("e-headercontent");
                                this._renderScroller();
                            } else {
                                this.element.children(".e-gridheader").removeClass("e-scrollcss");
                                this.element.get(0).style.width.length == 0 && this.element.css("width", "auto");
                            }
                        }
                        break;
                    case "locale":
                        this.model.locale = options[prop];
                        var model = this.model;
                        model.query = this.commonQuery.clone();
                        this.element.ejGrid("destroy").ejGrid(model);
                        break;
                    case "dataSource":
                        this.resetModelCollections();
                        this._refreshDataSource(this._dataSource());
                        if (this._gridRecordsCount && this.model.allowFiltering) {
                            this._initColumns(this.model.currentViewData[0] != undefined ? this.model.currentViewData[0] : this.model.currentViewData.value);
                            this._renderFilterDialogs();
                        }
                        break;
					case "selectedRowIndex":
                        if (this._selectedRow() != -1 && $.inArray(this._selectedRow(), this._selectedRowsIndexes) == -1)
                            this.selectRows(this._selectedRow());
                        else if (this._selectedRow() == -1) {
                            this.clearSelection();
                            this._selectedRowsIndexes = [];
                        }
                        break;                                                                     
                    case "editType":
                        if (this._selectedRow() != -1 && $.inArray(this._selectedRow(), this._selectedRowsIndexes) == -1)
                            this.selectRows(this._selectedRow());
                        break;
                    case "editSettings":
                        $.extend(this.model.editSettings, options[prop]);
                        this._processEditing();
                        this.refreshBatchEditMode();
                        this._tdsOffsetWidth = [];
                        $("#" + this._id + "_dialogEdit_wrapper,#" + this._id + "_dialogEdit").remove();
                        $("#" + this._id + "_externalEdit").remove();
                        if (this.model.editSettings.allowEditing || this.model.editSettings.allowAdding) {
                            if (this.model.editSettings.editMode == "dialog" || this.model.editSettings.editMode == "dialogtemplate")
                                this.element.append(this._renderDialog());
                            else if (this.model.editSettings.editMode == "externalform" || this.model.editSettings.editMode == "externalformtemplate")
                                this.element.append(this._renderExternalForm());
                        }
                        this._enableEditingEvents();
                        break;
                    case "showSummary":
                    case "summaryRows":
                        if (prop == "showSummary" && !options[prop])
                            this.element.children(".e-gridfooter").remove();
                        else {
                            var $content = this.element.find(".e-gridcontent").first();
                            this._renderGridFooter().insertAfter($content);
                        }
                        break;
                    case "enableAltRow":
                        this.model.enableAltRow = options[prop];
                        this.addInitTemplate();
                        this.refreshContent();
                        break;
                    case "toolbarSettings":
                        $.extend(this.model.toolbarSettings, options[prop]);
                        this.element.children(".e-gridtoolbar").remove();
                        if (this.model.toolbarSettings.showToolbar)
                            this._renderToolBar().insertBefore(this.element.find(".e-gridheader").first());
                        break;
				    case "selectionSettings":
                        $.extend(this.model.selectionSettings, options[prop]);
						this._allowrowSelection = this._allowcellSelection = this._allowcolumnSelection = false;						
                        this._initSelection();
                        break;
                    case "sortSettings":
                        $.extend(this.model.sortSettings, options[prop]);
                        this.refreshContent();
                        break;
                    case "enableRTL":
                        this.model.enableRTL = options[prop];
                        this.refreshContent();
                        break;
                    case "allowSelection":
                        if (options[prop])
                            this._off(this.element, "click")._on(this.element, "click", this._clickHandler);
                        break;
                    case "query":
                        this.commonQuery = $.extend(true, {}, options[prop]);
                        break;
                }
            }

        },
        /**
         * Reset the model collections like pageSettings, groupSettings, filterSettings, sortSettings and summaryRows.
         * @alias ejGrid#resetModelCollections
         * @return jQuery		 
         * @example 
         * &lt;script&gt;
         * // Create grid object.
         * var gridObj = $("#Grid").data("ejGrid");
         * // Reset model collections
         * gridObj.resetModelCollections(); 
         * &lt;/script&gt;
         * @example 
         * &lt;script&gt;
         * // Reset model collections
         * $("#Grid").ejGrid("resetModelCollections");
         * &lt;/script&gt;
         */
        resetModelCollections: function () {
            this.model.groupSettings.groupedColumns = [];
            this.model.filterSettings.filteredColumns = [];
            this.model.sortSettings.sortedColumns = [];
            this.model.pageSettings.currentPage = this.defaults.pageSettings.currentPage;
            this.model.summaryRows = [];
        }
    };
})(jQuery, Syncfusion);;
(function ($, ej, undefined) {
    ej.gridFeatures = ej.gridFeatures || {};
    ej.gridFeatures.edit = {
        _processEditing: function () {
            var query = this._columnToSelect();
            this.model.query._fromTable != "" && query.from(this.model.query._fromTable);
            if (this._dataSource() instanceof ej.DataManager && query.queries.length) {
                var queryPromise = this._dataSource().executeQuery(query);
                queryPromise.done(ej.proxy(function (e) {
                    this._dropDownManager = ej.DataManager(e.result);
                    if (this.model.editSettings.editMode == "normal") this.addEditingTemplate();
                    else if (this.model.editSettings.editMode == "batch") this.addBatchEditTemplate();
                    else if (this.model.editSettings.editMode == "dialog" ||
                        this.model.editSettings.editMode == "externalform" ||
                        this.model.editSettings.editMode == "inlineform")
                        this.addDialogEditingTemplate();
                    else this.addExternalDialogEditingTemplate();
                }, this));
            } else {
                if (this.model.editSettings.editMode == "normal") this.addEditingTemplate();
                else if (this.model.editSettings.editMode == "batch") this.addBatchEditTemplate();
                else if (this.model.editSettings.editMode == "dialog" ||
                    this.model.editSettings.editMode == "externalform" ||
                    this.model.editSettings.editMode == "inlineform")
                    this.addDialogEditingTemplate();
                else this.addExternalDialogEditingTemplate();
            }
        },
        addEditingTemplate: function () {
            if (this.model.columns.length == 0)
                return;
            var $tbody = ej.buildTag('tbody');
            var $tr = ej.buildTag('tr');
            var $td = ej.buildTag('td', "", {}, { colSpan: this.model.scrollSettings.frozenColumns > 0 ? this.model.scrollSettings.frozenColumns : this.model.columns.length });
            var $form = ej.buildTag('form', "", {}, { id: this._id + "EditForm", "class": "gridform" });
            var $table = ej.buildTag('table.e-table', "", {}, { cellspacing: 0.25 });
            var $innerTbody = ej.buildTag('tbody');
            var $innerTr = ej.buildTag('tr');
            $tbody.append($tr);
            $tr.append($td);
            $td.append($form);
            var $colGroup = $(document.createElement('colgroup'));
            $form.append($table);
            $table.append($colGroup);
            $innerTbody.append($innerTr).appendTo($table);
            if (this.model.scrollSettings.frozenColumns > 0) {
                var $tbodyClone = $tbody.clone();
                $tbodyClone.find("td").first().prop("colSpan", this.model.columns.length - this.model.scrollSettings.frozenColumns);
            }
            for (var columnCount = 0; columnCount < this.model.columns.length; columnCount++) {
                var $innerTd = ej.buildTag('td.e-rowcell');
                $innerTr.append($innerTd.get(0));
                if (ej.isNullOrUndefined(this.model.columns[columnCount]["isUnbound"]) && ej.isNullOrUndefined(this.model.columns[columnCount]["template"])) {
                    this._initCellEditType(columnCount, $innerTd);
                } else if (this.model.columns[columnCount]["template"]) {
                    var helpers = {};
                    helpers["_" + this._id + "ColumnTemplating"] = ej.proxy(this._gridTemplate, null, this);
                    $.views.helpers(helpers);
                    $("#" + this._id + this.model.columns[columnCount].headerText + columnCount + "_Template").remove();
                    var scriptElement = this._createTemplateElement(this.model.columns[columnCount]);
                    $innerTd.addClass("e-templatecell").html("{{:~_" + this._id + "ColumnTemplating('" + scriptElement.id + "')}}");
                } else if (this.model.columns[columnCount]["isUnbound"]) {
                    var helpers = {};
                    helpers["_" + this._id + "UnboundTemplate"] = this._unboundTemplateRendering;
                    $.views.helpers(helpers);
                    $("#" + this._id + this.model.columns[columnCount].headerText + "_UnboundTemplate").remove();
                    divElement = this._createUnboundElement(this.model.columns[columnCount]);
                    $innerTd.addClass("e-unboundcell").addClass("e-" + this.model.columns[columnCount]["headerText"].replace(/\s+/g, "")).html("{{:~_" + this._id + "UnboundTemplate('" + divElement.id + "')}}");
                    this.model.scrollSettings.frozenColumns > 0 && $innerTd.addClass("e-frozenunbound");
                    this._isUnboundColumn = true;
                }
                if (this.model.columns[columnCount]["textAlign"] != undefined)
                    $innerTd.css("text-align", this.model.columns[columnCount]["textAlign"]);
                this.model.columns[columnCount]["allowEditing"] == false && $innerTd.find(".e-field").attr("disabled", true).addClass("e-disable");
                if (this.model.columns[columnCount]["isPrimaryKey"] === true) {
                    $innerTd.find(".e-field").attr("disabled", true).addClass("e-disable");
                    this._primaryKeys.push($.trim(this.model.columns[columnCount].field));
                    this._primaryKeys = $.unique(this._primaryKeys);
                }
                if (this.model.columns[columnCount]["isIdentity"] === true)
                    $innerTd.find(".e-field").addClass("e-identity");
                var $col = $(document.createElement('col'));
                if (this.model.columns[columnCount]["priority"]) {
                    $innerTd.addClass("e-table-priority-" + this.model.columns[columnCount]["priority"]);
                    $col.addClass("e-table-priority-" + this.model.columns[columnCount]["priority"]);
                }
                if (this.model.columns[columnCount]["visible"] === false) {
                    $col.css("display", "none");
                    $innerTd.addClass("e-hide");
                }
                if (!ej.isNullOrUndefined(this.model.columns[columnCount]["cssClass"])) {
                    $innerTd.addClass(this.model.columns[columnCount]["cssClass"]);
                }
                !this.model.groupSettings.showGroupedColumn && $innerTd.addClass("{{for ~groupedColumns}}" +
                    " {{if #data == '" + this.model.columns[columnCount]["field"] + "'}}e-hide{{/if}}" +
                    "{{/for}}") && $col.css("display", "none");
                $colGroup.append($col);
                if (columnCount == this.model.scrollSettings.frozenColumns - 1) {
                    $innerTr = $tbodyClone.find("tr").last();
                    $colGroup = $tbodyClone.find("colgroup");
                    $.templates(this._id + "_JSONFrozenEditingTemplate", $tbody.html());
                    $tbody = $tbodyClone;
                }
            }
            $.templates(this._id + "_JSONEditingTemplate", $tbody.html());
        },

        addDialogEditingTemplate: function () {
            if (this.model.columns.length == 0)
                return;
            var $tbody = ej.buildTag('div');
            var $form = ej.buildTag('form.gridform', "", {}, { id: this._id + "EditForm" });
            var $table = ej.buildTag('table', "", {}, { cellspacing: "14px" });
            var $innerTr, $labelTd, $valueTd, trElement, tdElement;
            for (var columnCount = 0; columnCount < this.model.columns.length; columnCount++) {
                if (this.model.editSettings.editMode == "dialog") {
                    trElement = 'tr';
                    tdElement = 'td';
                }
                else trElement = tdElement = 'div';
                $innerTr = ej.buildTag(trElement);
                $labelTd = ej.buildTag(tdElement, "", { "text-align": "right" }).addClass("e-label");
                $valueTd = ej.buildTag(tdElement, "", { "text-align": "left" }).addClass("e-rowcell");
                 if (this.model.columns[columnCount]["priority"] && this.model.editSettings.editMode == "inlineform") 
                    $innerTr.addClass("e-table-priority-" + this.model.columns[columnCount]["priority"]);
                $innerTr.append($labelTd.get(0)).append($valueTd.get(0));
                if (this.model.columns[columnCount].headerText == undefined)
                    this.model.columns[columnCount].headerText = this.model.columns[columnCount].field;
                $labelTd.append("<label for='" + this.model.columns[columnCount].field + "'>" + this.model.columns[columnCount].headerText + "</label>");
                if (ej.isNullOrUndefined(this.model.columns[columnCount]["isUnbound"]) && ej.isNullOrUndefined(this.model.columns[columnCount]["template"]))
                    this._initCellEditType(columnCount, $valueTd);
                else if (this.model.columns[columnCount]["template"]) {
                    var helpers = {};
                    helpers["_" + this._id + "ColumnTemplating"] = ej.proxy(this._gridTemplate, null, this);
                    $.views.helpers(helpers);
                    $("#" + this._id + this.model.columns[columnCount].headerText + columnCount + "_Template").remove();
                    var scriptElement = this._createTemplateElement(this.model.columns[columnCount]);
                    $valueTd.addClass("e-templatecell").html("{{:~_" + this._id + "ColumnTemplating('" + scriptElement.id + "')}}");
                } else if (this.model.columns[columnCount]["isUnbound"]) {
                    var helpers = {};
                    helpers["_" + this._id + "UnboundTemplate"] = this._unboundTemplateRendering;
                    $.views.helpers(helpers);
                    $("#" + this._id + this.model.columns[columnCount].headerText + "_UnboundTemplate").remove();
                    divElement = this._createUnboundElement(this.model.columns[columnCount]);
                    $valueTd.addClass("e-unboundcell").addClass("e-" + this.model.columns[columnCount]["headerText"].replace(/\s+/g, "")).html("{{:~_" + this._id + "UnboundTemplate('" + divElement.id + "')}}");
                    this.model.scrollSettings.frozenColumns > 0 && $innerTd.addClass("e-frozenunbound");
                    this._isUnboundColumn = true;
                }
                this.model.columns[columnCount]["allowEditing"] == false && $valueTd.find(".e-field").attr("disabled", true).addClass("e-disable");
                if (this.model.columns[columnCount]["isPrimaryKey"] === true)
                    $valueTd.find(".e-field").attr("disabled", true).addClass("e-disable");
                if (this.model.columns[columnCount]["visible"] === false)
                    $innerTr.addClass("e-hide");
                 if (!ej.isNullOrUndefined(this.model.columns[columnCount]["cssClass"])) {
                    $innerTd.addClass(columns[i]["cssClass"]);
                }
                if (this.model.editSettings.editMode == "dialog") {
                    $form.append($table);
                    $table.append($innerTr);
                } else
                    $form.append($innerTr);
                $form.appendTo($tbody);
                if (this.model.columns[columnCount]["isPrimaryKey"] === true) {
                    $valueTd.find(".e-field").attr("disabled", true).addClass("e-disable");
                    this._primaryKeys.push($.trim(this.model.columns[columnCount].field));
                    this._primaryKeys = $.unique(this._primaryKeys);
                }
            }
            if (this.model.editSettings.editMode == "dialog") $form.append($table);
            $tbody = this.renderDiaglogButton($form, $tbody);
            $.templates(this._id + "_JSONDialogEditingTemplate", $tbody.html());
        },
        _editEventTrigger: function (args) {
            if (args.requestType == "save" || args.requestType == "delete") {
                var params = {
                    data: args.data,
                    action: args.action !== undefined ? args.action : args.requestType,
                };
                if (!ej.isNullOrUndefined(args.foreignKeyData))
                    params.foreignKeyData = args.foreignKeyData;
                this._trigger("end" + params.action.charAt(0).toUpperCase() + params.action.slice(1), params);
            }
        },
        _compiledDropDownTemplate: function (valueField, textField, colType, format) {
            var helpers = { _gridFormatting: this.formatting };
            $.views.helpers(helpers);
            var $select = ej.buildTag('select');
            var $option = ej.buildTag("option", ej.isNullOrUndefined(textField) ? "{{:text}}" : colType == "date" && format != null ? "{{:~_gridFormatting('" + format + "'," + textField + ")}}" : "{{:" + textField + "}}", {}, { value: ej.isNullOrUndefined(valueField) ? "{{:value}}" : "{{:" + valueField + "}}" });
            $select.append($option);
            return $.templates($select.html());
        },
        _initCellEditType: function (columnCount, element) {
            if (this.model.columns[columnCount]["foreignKeyValue"])
                this.model.columns[columnCount]["editType"] = "dropdownedit";
            if (this.model.columns[columnCount]["editTemplate"])
                this.model.columns[columnCount]["editType"] = "edittemplate";
            if (ej.isNullOrUndefined(this.model.columns[columnCount]["editType"]))
                if (this.model.columns[columnCount]["type"] == "date" || this.model.columns[columnCount]["type"] == "datetime")
                    this.model.columns[columnCount]["editType"] = this.model.columns[columnCount]["type"] + "picker";
                else
                    this.model.columns[columnCount]["editType"] = "stringedit";
            if (this.model.isResponsive)
                element.attr("data-cell", this.model.columns[columnCount]["headerText"]);
            switch (this.model.columns[columnCount]["editType"]) {
                case "edittemplate":
                    var temp = this.model.columns[columnCount].editTemplate.create;
                    if (typeof temp == "string") {
                        var temp1 = ej.util.getObject(temp, window);
                        if (!$.isFunction(temp1)) {
                            if ($(temp).length == 1 && $(temp).get(0).tagName == "SCRIPT")
                                var $edittemplate = $($(temp).html()).attr({ id: this._id + this.model.columns[columnCount].field, name: this.model.columns[columnCount].field });
                            else
                                var $edittemplate = $(temp).attr({ id: this._id + this.model.columns[columnCount].field, name: this.model.columns[columnCount].field });
                        }
                        else
                            var $edittemplate = $(temp1()).attr({ id: this._id + this.model.columns[columnCount].field, name: this.model.columns[columnCount].field });
                    }
                    else
                        var $edittemplate = $(temp()).attr({ id: this._id + this.model.columns[columnCount].field, name: this.model.columns[columnCount].field });
                    element.append($edittemplate);
                    break;
                case "stringedit":
                    element.html(ej.buildTag('input.e-field e-ejinputtext', "", {}, { value: "{{:#data['" + this.model.columns[columnCount].field + "']}}", id: this._id + this.model.columns[columnCount].field, name: this.model.columns[columnCount].field }));
                    break;
                case "booleanedit":
                    element.html('{{if ' + this.model.columns[columnCount].field + '}} <input class="e-field e-checkbox" type ="checkbox" id=' + this._id + this.model.columns[columnCount].field + ' name=' + this.model.columns[columnCount].field + ' checked="checked"></input>{{else}} <input class="e-field e-checkbox" type ="checkbox" id=' + this._id + this.model.columns[columnCount].field + ' name=' + this.model.columns[columnCount].field + ' > {{/if}}');
                    if (this.model.editSettings.editMode == "normal")
                        element.addClass("e-boolcell");
                    break;
                case "numericedit":
                    var $numericText = ej.buildTag('input.e-numerictextbox e-field', "", {}, { type: "text", value: "{{:#data['" + this.model.columns[columnCount].field + "']}}", id: this._id + this.model.columns[columnCount].field, name: this.model.columns[columnCount].field });
                    element.append($numericText);
                    break;
                case "datepicker":
                case "datetimepicker":
                    var $datePicker = ej.buildTag('input.e-' + this.model.columns[columnCount]["editType"] + ' e-field', "", {}, { type: "text", value: "{{:#data['" + this.model.columns[columnCount].field + "']}}", id: this._id + this.model.columns[columnCount].field, name: this.model.columns[columnCount].field });
                    element.append($datePicker);
                    break;
                case "dropdownedit":
                    var currColumn = this.model.columns[columnCount];
                    if (ej.isNullOrUndefined(currColumn.dataSource)) {
                        var arrayOfDatas;
                        var query = new ej.Query().select(currColumn.field);
                        if (ej.isNullOrUndefined(this._dropDownManager) || (!this._dataManager.dataSource.offline && this._dataManager.dataSource.json))
                            arrayOfDatas = this._dataManager.executeLocal(query);
                        else
                            arrayOfDatas = this._dropDownManager.executeLocal(query);
                        var selectedItems = [];
                        var uniqueData = uniqueData = ej.dataUtil.mergeSort(ej.distinct(arrayOfDatas));
                        for (var index = 0; index < uniqueData.length; index++)
                            selectedItems.push({ text: uniqueData[index], value: uniqueData[index] });
                    }
                    else
                        selectedItems = currColumn.dataSource;
                    var dropDownTemplate;
                    var fieldName = ej.isNullOrUndefined(currColumn.foreignKeyField) ? currColumn.field : currColumn.foreignKeyField;
                    if (currColumn.foreignKeyValue)
                        dropDownTemplate = this._compiledDropDownTemplate(fieldName, currColumn.foreignKeyValue, currColumn.type, currColumn.format);
                    else
                        dropDownTemplate = this._compiledDropDownTemplate();
                    element.get(0).innerHTML = ["<select>", dropDownTemplate.render(selectedItems), "</select>"].join("");
                    element.find("select").prop({ id: this._id + currColumn.field, name: currColumn.field }).addClass("e-field e-dropdownlist");
                    break;
            }
        },
        addBatchEditTemplate: function () {
            if (this.model.columns.length == 0)
                return;
            var $outerDiv = ej.buildTag('div', "", { display: "none" }, { id: this._id + "_BulkEditTemplate" }), i, columnCount, $innerDiv;
            for (i = 0, columnCount = this.model.columns.length; i < columnCount; i++) {
                if (!this.model.columns[i]["template"] || !this.model.columns[i]["isUnbound"]) {
                    $innerDiv = ej.buildTag('div', "", {}, { id: this.model.columns[i].field + "_BulkEdit" });
                    this._initCellEditType(i, $innerDiv);
                    $outerDiv.append($innerDiv);
                }
                if (this.model.columns[i]["isPrimaryKey"] === true) {
                    this._primaryKeys.push($.trim(this.model.columns[i].field));
                    this._primaryKeys = $.unique(this._primaryKeys);
                }
                if (this.model.columns[i]["isIdentity"] === true)
                    $innerDiv.find(".e-field").addClass("e-identity");
            }
            if ($outerDiv.children().length)
                this._bulkEditTemplate = $outerDiv;

        },
        addExternalDialogEditingTemplate: function () {
            if (this.model.columns.length == 0)
                return;
            for (var columnCount = 0; columnCount < this.model.columns.length; columnCount++) {
                if (this.model.columns[columnCount]["isPrimaryKey"] === true) {
                    this._primaryKeys.push($.trim(this.model.columns[columnCount].field));
                    this._primaryKeys = $.unique(this._primaryKeys);
                }
            }
            var $tbody = ej.buildTag('div', "", { 'display': 'none' });
            var $form = ej.buildTag('form.gridform', "", {}, { id: this._id + "EditForm" });
            var cloneElement;
            if (this.model.editSettings.editMode == "dialogtemplate" && this.model.editSettings.dialogEditorTemplateID != null)
                cloneElement = this.model.editSettings.dialogEditorTemplateID;
            else if (this.model.editSettings.editMode == "externalformtemplate" && this.model.editSettings.externalFormTemplateID != null)
                cloneElement = this.model.editSettings.externalFormTemplateID;
            else
                cloneElement = this.model.editSettings.inlineFormTemplateID;

            $form.html($(cloneElement).html());
            $tbody = this.renderDiaglogButton($form, $tbody);
            $.templates(this._id + "_JSONdialogTemplateMode", $tbody.html());
        },
        _editdblClickHandler: function (e) {
            var $target = $(e.target);
            if ($target.hasClass("e-rowcell")) {
                if (!this.model.isEdit) {
                    this._$currentTr = (this.model.scrollSettings.frozenColumns > 0 || this.model.scrollSettings.frozenRows > 0)
                        ? this.getRowByIndex($target.closest('tr').index())
                        : $target.closest('tr');
                    this.startEdit(this._$currentTr);
                }
            }
            return false;
        },
        _columnToSelect: function () {
            var column = [];
            for (var i = 0; i < this.model.columns.length; i++) {
                if (this.model.columns[i]["editType"] === ej.Grid.EditingType.Dropdown && ej.isNullOrUndefined(this.model.columns[i]["dataSource"]))
                    column.push(this.model.columns[i].field);
            }
            if (column.length)
                return ej.Query().select(column);
            return ej.Query();
        },
        _renderExternalForm: function () {
            var $externalform = ej.buildTag("div", "", { display: "none" }, { id: this._id + "_externalEdit", 'class': "e-form-container" });
            var $eformHeader = ej.buildTag("div", "", "", { id: this._id + "_eFormHeader", 'class': "e-form-titlebar" });
            var $eformTitle = ej.buildTag("span", "", "", { 'class': "e-form-title" });
            var $eformToggleBtn = ej.buildTag("div", "", "", { id: this._id + "_eFormToggleBtn", 'class': "e-form-togglebtn" });
            var $eformToggleIcon = ej.buildTag("span", "", "", { 'class': "e-form-toggle-icon e-icon" });
            $eformToggleBtn.append($eformToggleIcon);
            $eformHeader.append($eformTitle).append($eformToggleBtn);

            var $eformContent = ej.buildTag("div", "", "", { id: this._id + "_eFormContent", 'class': "e-form-content" });
            var $eform = ej.buildTag("div", "", "", { id: this._id + "_externalForm", 'class': "e-externalform" });
            var $contentOuterDiv = ej.buildTag("div", "", "", { 'class': "e-externalformedit" });
            $eform.append($contentOuterDiv);
            $eformContent.append($eform);
            return $externalform.append($eformHeader).append($eformContent);;
        },
        _buttonClick: function (e) {
            if (e.type == "close") {
                if (!this.model.isEdit)
                    return;
                this.model.isEdit = false;
                this.element.ejGrid("cancelEdit");
                this.refreshToolbar();
                return;
            }
            if (e.keyCode !== undefined && e.keyCode != 13 || this.model == null)
                return true;
            if (this.model.editSettings.editMode == "dialog" || this.model.editSettings.editMode == "dialogtemplate") {
                if (e.target.id == "EditDialog_" + this._id + "_Save") {
                    if (this.element.ejGrid("endEdit").length !== undefined)
                        $("#" + this._id + "_dialogEdit").ejDialog("close");
                } else if (e.target.id == "EditDialog_" + this._id + "_Cancel") {
                    this.element.ejGrid("cancelEdit");
                    $("#" + this._id + "_dialogEdit").ejDialog("close");
                }
            }
            else if (this.model.editSettings.editMode == "externalform" || this.model.editSettings.editMode == "externalformtemplate") {
                if ($(e.target).hasClass("e-form-toggle-icon")) {
                    this.element.ejGrid("cancelEdit");
                    $("#" + this._id + "_externalEdit").css("display", "none");
                }
                else {
                    if (e.target.id == "EditExternalForm_" + this._id + "_Save") {
                        if (this.element.ejGrid("endEdit").length !== undefined)
                            $("#" + this._id + "_externalEdit").css("display", "none");
                    } else if (e.target.id == "EditExternalForm_" + this._id + "_Cancel") {
                        this.element.ejGrid("cancelEdit");
                        $("#" + this._id + "_externalEdit").css("display", "none");
                    }
                }
            }
            else if (this.model.editSettings.editMode == "inlineform" || this.model.editSettings.editMode == "inlineformtemplate") {
                if (e.target.id == "InlineEditForm_" + this._id + "_Save")
                    this.element.ejGrid("endEdit");
                else if (e.target.id == "InlineEditForm_" + this._id + "_Cancel")
                    this.element.ejGrid("cancelEdit");
            }
            else
                this.element.ejGrid("cancelEdit");
        },
        _enableEditingEvents: function () {
            if (this.model.editSettings.allowEditing || this.model.editSettings.allowAdding) {
                if (this.model.editSettings.allowEditing && this.model.editSettings.editMode != "batch" && this.model.editSettings.allowEditOnDblClick)
                    this._on(this.element, "dblclick", ".e-gridcontent", this._editdblClickHandler);
                else
                    this._off(this.element, "dblclick", ".e-gridcontent");
                this._off($("#" + this._id + "_dialogEdit"), "click keypress", "#EditDialog_" + this._id + "_Save ,#EditDialog_" + this._id + "_Cancel");
                if (this.model.editSettings.editMode == "dialog" || this.model.editSettings.editMode == "dialogtemplate") {
                    this._on($("#" + this._id + "_dialogEdit"), "click keypress", "#EditDialog_" + this._id + "_Save ,#EditDialog_" + this._id + "_Cancel", this._buttonClick);
                }
                else if (this.model.editSettings.editMode == "externalform" || this.model.editSettings.editMode == "externalformtemplate") {
                    this._on($("#" + this._id + "_externalEdit"), "click keypress", "#EditExternalForm_" + this._id + "_Save ,#EditExternalForm_" + this._id + "_Cancel", this._buttonClick);
                    $(this.element).on("click", ".e-form-toggle-icon", $.proxy(this._buttonClick, this));
                }
                else if (this.model.editSettings.editMode == "batch")
                    this._on($(document), "mousedown", this._saveCellHandler);

                else if (this.model.editSettings.editMode == "inlineform" || this.model.editSettings.editMode == "inlineformtemplate")
                    $(this.element).on("click keypress", "#InlineEditForm_" + this._id + "_Save ,#InlineEditForm_" + this._id + "_Cancel", $.proxy(this._buttonClick, this));

            } else {
                this._off($("#" + this._id + "_dialogEdit"), "click", "#EditDialog_" + this._id + "_Save ,#EditDialog_" + this._id + "_Cancel");
                $(this.element).off("click", ".e-icon");
                this._off($("#" + this._id + "_externalEdit"), "click", "#EditExternalForm_" + this._id + "_Save ,#EditExternalForm_" + this._id + "_Cancel");
                $(this.element).off("click", ".e-form-toggle-icon");
                $(this.element).off("click", "#InlineEditForm_" + this._id + "_Save ,#InlineEditForm_" + this._id + "_Cancel");
            }
        },

        _deleteRow: function ($tr) {
            if (!this.model.editSettings.allowDeleting)
                return;
            if (this.model.editSettings.editMode == "batch")
                this._bulkDelete(this.getIndexByRow($tr));
            else {
                if (this._primaryKeys.length == 0 && !this.model.editSettings.allowEditing && !this.model.editSettings.allowAdding) {
                    for (i = 0; i < this.model.columns.length; i++) {
                        if (this.model.columns[i]["isPrimaryKey"] === true) {
                            this._primaryKeys.push($.trim(this.model.columns[i].field));
                            this._primaryKeys = $.unique(this._primaryKeys);
                        }
                    }
                }
                if (this._selectedRow() == -1 && ej.isNullOrUndefined($tr)) {
                    alert(this._getLocalizedLabels("DeleteOperationAlert"));
                    return;
                }
                if (ej.isNullOrUndefined($tr))
                    $tr = this.getRowByIndex(this._selectedRow());
                this._primaryKeyValues = [];
                for (var index = 0; index < this._primaryKeys.length; index++) {
                    var column = this.getColumnByField(this._primaryKeys[index]);
                    var index = $.inArray(column, this.model.columns);
                    this._primaryKeyValues.push($tr.find(".e-rowcell").eq(index).html());
                }
                var deleteManager = ej.DataManager(this._currentJsonData);
                var query = new ej.Query();
                for (var i = 0; i < this._primaryKeys.length; i++)
                    query = query.where(this._primaryKeys[i], ej.FilterOperators.equal, this._primaryKeyValues[i]);
                currentData = deleteManager.executeLocal(query);
                var args = {};
                args.tr = $tr;
                args.data = currentData[0];
                var foreignKeyData = this._getForeignKeyData(args.data);
                if (!ej.isNullOrUndefined(foreignKeyData))
                    args.foreignKeyData = foreignKeyData;
                args.requestType = ej.Grid.Actions.Delete;
                if (this._trigger("actionBegin", args))
                    return true;
                this._cDeleteData = currentData;
                var promise;
                if (this._dataSource() instanceof ej.DataManager) {
                    promise = this._dataManager.remove(this._primaryKeys[0], currentData[0][this._primaryKeys[0]]);
                    var proxy = this;
                    if ($.isFunction(promise.promise)) {
                        promise.done(function(e) {
                            proxy._processBindings(args);
                            proxy._primaryKeyValues = [];
                            proxy._cDeleteData = null;
                        });
                        promise.fail(function(e) {

                        });
                    } else
                        this._processBindings(args);
                } else
                    this._processBindings(args);
                if (promise == undefined || !$.isFunction(promise.promise)) {
                    this._primaryKeyValues = [];
                    this._cDeleteData = null;
                }
            }
        },
        /**
		 * Send an edit record request in grid
         * @alias ejGrid#startEdit
		 * @return jQuery
		 * @param {object} $tr Pass the tr- selected row element to be edited in grid
		 * @example 
		 * &lt;script&gt;
		 * // Create grid object.
		 * var gridObj = $("#Grid").data("ejGrid");
         * // Sends an edit record request to the grid
		 * gridObj.startEdit($(".gridcontent tr").first()); 
		 * &lt;/script&gt;
		 * @example 
		 * &lt;script&gt;
		 * // Sends an edit record request to the grid
		 * $("#Grid").ejGrid("startEdit", $(".gridcontent tr").first());        
		 * &lt;/script&gt;
         */
        startEdit: function ($tr) {
            if (!this.model.editSettings.allowEditing)
                return;
            if (this._selectedRow() == -1 && ej.isNullOrUndefined($tr)) {
                alert(this._getLocalizedLabels("EditOperationAlert"));
                return;
            }
            if (ej.isNullOrUndefined($tr)) {
                this._currentTrIndex = this._selectedRow();
                this._$currentTr = this.getRowByIndex(this._currentTrIndex);
            } else {
                this._currentTrIndex = $tr.index();
                this._$currentTr = $tr;
            }
            for (var index = 0; index < this._primaryKeys.length; index++) {
                var column = this.getColumnByField(this._primaryKeys[index]);
                var indx = $.inArray(column, this.model.columns);
                this._primaryKeyValues.push(this._$currentTr.find(".e-rowcell").eq(indx).html());
            }
            var args = { row: this._$currentTr, rowIndex: this._currentTrIndex, primaryKey: this._primaryKeys, primaryKeyValue: this._primaryKeyValues };
            var cancel = this._trigger("beginEdit", args);
            if (cancel) {
                this._primaryKeyValues = [];
                return;
            }
            args.requestType = ej.Grid.Actions.BeginEdit;
            this._processBindings(args);

        },
         _startAdd: function() {
            if (!this.model.editSettings.allowAdding)
                return;
            if (this.model.editSettings.editMode == "batch")
                this._bulkAddRow();
            else {
                var cloneData = {};
                for (var i = 0; i < this.model.columns.length; i++)
                    cloneData[this.model.columns[i].field] = !ej.isNullOrUndefined(this.model.columns[i].defaultValue) ? this.model.columns[i].defaultValue : "";
                var args = {};
                args.data = cloneData;
                var foreignKeyData = this._getForeignKeyData(args.data);
                if (!ej.isNullOrUndefined(foreignKeyData))
                    args.foreignKeyData = foreignKeyData;
                args.requestType = "add";
                this.clearSelection();
                var returnValue = this._processBindings(args);
                if (!returnValue)
                    this._selectedRow(0);
                var groupedColumns = this.model.groupSettings.groupedColumns.length;
                if (groupedColumns > 1) {
                    var $editCol = this.getContentTable().find(".e-addedrow").find("table").find("colgroup").children();
                    $($editCol.slice(0, groupedColumns - 1)).css('width', this.getHeaderTable().find('colgroup').children()[0].style.width);
                }
            }
        },
        /**
		 * Send a save request in grid.
         * @alias ejGrid#endEdit
		 * @return jQuery
		 * @example 
		 * &lt;script&gt;
		 * // Create grid object.
		 * var gridObj = $("#Grid").data("ejGrid");
         * // Sends a save request to the grid
		 * gridObj.endEdit(); 
		 * &lt;/script&gt;
		 * @example 
		 * &lt;script&gt;
		 * // Sends a save request to the grid
		 * $("#Grid").ejGrid("endEdit");        
		 * &lt;/script&gt;
         */
        endEdit: function () {
            if (this.model.isEdit) {
                var formElement, $formElement;
                if (!this.editFormValidate())
                    return true;
                var obj = {};
                var subElements = [];
                var editedRowWrap;
                if (this.model.editSettings.editMode == "batch")
                    this.saveCell();
                else {
                    formElement = this.model.scrollSettings.frozenColumns > 0 ? this.element.find(".gridform") : document.getElementById(this._id + "EditForm");
                    $formElement = $(formElement);
                    if (this.model.editSettings.editMode == "normal" || this.model.editSettings.editMode == "inlineform" || this.model.editSettings.editMode == "inlineformtemplate")
                        editedRowWrap = $formElement.closest('tr');
                    else
                        editedRowWrap = $formElement.closest('div');
                    formElement = this.model.scrollSettings.frozenColumns > 0 ? formElement[0] : formElement;
                    for (var index = 0; index < formElement.length; index++) {
                        if (editedRowWrap.hasClass("e-addedrow") && $(formElement[index]).hasClass("e-identity"))
                            continue;
                        var columnName = formElement[index].name, $element = $(formElement[index]);
                        if (columnName != undefined) {
                            if (columnName == "") {
                                if (formElement[index].id.indexOf("Save") != -1 || formElement[index].id.indexOf("Cancel")!= -1)
                                    columnName = "";
                                else
                                    columnName = formElement[index].id.replace(this._id, "");
                            }
                            if (columnName != "" && obj[columnName] == null) {
                                var column = this.getColumnByField(columnName), value = formElement[index].value;
                                if (!ej.isNullOrUndefined(column) && (column.editType == "edittemplate")) {
                                    if (formElement[index].type == 'hidden') index++;
                                    if (formElement[index].type != 'hidden') {
                                        var temp1 = column.editTemplate.read; $element = $(formElement[index]);
                                        if (typeof temp1 == "string")
                                            temp1 = ej.util.getObject(temp1, window);
                                        value = temp1($element);
                                    }
                                }
                                else if ($(formElement[index]).hasClass("e-datepicker")) {
                                    value = $element.ejDatePicker("model.value");
                                    subElements.push({ element: $element, pluginName: "ejDatePicker" });
                                }
                                else if ($(formElement[index]).hasClass("e-datetimepicker")) {
                                    value = $element.ejDateTimePicker("model.value");
                                    subElements.push({ element :$element, pluginName:"ejDateTimePicker"});
                                }
                                else if ($element.is(".e-numerictextbox"))
                                    value = $element.ejNumericTextbox("getValue");
                                else if ($element.data("ejDropDownList")) {
                                    value = $element.ejDropDownList("getSelectedValue");
                                    subElements.push({ element: $element, pluginName: "ejDropDownList" });
                                }
                                if (column == null)
                                    value = !isNaN(parseFloat(value)) && isFinite(value) ? parseFloat(value) : value;
                                else if (column.type == "number" && value.length)
                                    value = parseFloat(value);
                                var originalvalue;
                            if (formElement[index].type != "checkbox")
                                originalvalue = value;
                            else
                                originalvalue = $(formElement[index]).is(':checked');
                            if (columnName.indexOf(".") != -1)
                                ej.createObject(columnName, originalvalue, obj);
                            else
                                obj[columnName] = originalvalue;
                            }
                        }
                        if (index == formElement.length - 1 && $formElement.length > 1 && $formElement.index(formElement) == 0) {
                            formElement = $formElement[1];
                            index = -1;
                        }

                    }
                    var args = { data: obj };
                    var foreignKeyData = this._getForeignKeyData(args.data);
                    if (!ej.isNullOrUndefined(foreignKeyData))
                        args.foreignKeyData = foreignKeyData;
                    args.requestType = ej.Grid.Actions.Save;
                    args.selectedRow = this._selectedRow();
                    var currentData;
                    if (this._trigger("actionBegin", args))
                        return true;
                    for (var ele = 0; ele < subElements.length; ele++) {
                        var element = subElements[ele].element;
                        element.data(subElements[ele].pluginName)["destroy"]();
                    }
                    if (editedRowWrap.hasClass("e-editedrow")) {
                        this._cModifiedData = obj;
                        args.action = "edit";
                    } else if (editedRowWrap.hasClass("e-addedrow")) {
                        this._cAddedRecord = obj;
                        args.action = "add";
                    }
                    this._updateAction(args);
                }
            }
        },
        _updateAction: function (args) {
            var promise;
            if (this._dataSource() instanceof ej.DataManager) {
                if (!ej.isNullOrUndefined(this._cModifiedData))
                    promise = this._dataManager.update(this._primaryKeys[0], args.data);
                else
                    promise = this._dataManager.insert(args.data);
                var proxy = this;
                if ($.isFunction(promise.promise)) {
                    promise.done(function (e) {
                        proxy._processBindings(args);
                        proxy._cModifiedData = null;
                        proxy._cAddedRecord = null;
                        proxy._primaryKeyValues = [];
                    });
                    promise.fail(function (e) {
                    });
                } else
                    proxy._processBindings(args);
            } else
                this._processBindings(args);
            if (promise == undefined || !$.isFunction(promise.promise)) {
                this._cModifiedData = null;
                this._cAddedRecord = null;
                this._primaryKeyValues = [];
            }
        },
        /**
		 * Send a cancel request in grid.
         * @alias ejGrid#cancelEdit
		 * @return jQuery
		 * @example 
		 * &lt;script&gt;
		 * // Create grid object.
		 * var gridObj = $("#Grid").data("ejGrid");
         * // Sends a cancel request to the grid
		 * gridObj.cancelEdit(); 
		 * &lt;/script&gt;
		 * @example 
		 * &lt;script&gt;
		 * // Sends a cancel request to the grid
		 * $("#Grid").ejGrid("cancelEdit");        
		 * &lt;/script&gt;
         */
        cancelEdit: function () {
            var args = {};
            $("#" + this._id + "EditForm").find(".e-datepicker").ejDatePicker("destroy");
            args.requestType = ej.Grid.Actions.Cancel;
            this._processBindings(args);
            this._primaryKeyValues = [];
            this._currentData = null;
        },
        /**
         * Refresh the toolbar items in grid.
         * @alias ejGrid#refreshToolbar
         * @return jQuery
         * @example 
         * &lt;div id="Grid"&gt;&lt;/div&gt; 
         * &lt;script&gt;
         * // Create grid object.
         * var gridObj = $("#Grid").data("ejGrid");
         * // Refreshes the toolbar items state
         * gridObj.refreshToolbar(); 
         * &lt;/script&gt;
         * @example 
         * &lt;div id="Grid"&gt;&lt;/div&gt; 
         * &lt;script&gt;
         * // Refreshes the toolbar items state
         * $("#Grid").ejGrid("refreshToolbar");        
         * &lt;/script&gt;
         */
        refreshToolbar: function () {
            var $toolbar = $("#" + this._id + "_toolbarItems");
            var lis = $toolbar.find("li");
            $toolbar.ejToolbar("enableItem", lis);
            for (var i = 0; i < lis.length; i++) {
                switch (lis[i].id) {
                    case this._id + "_add":
                    case this._id + "_edit":
                    case this._id + "_delete":
                        if (this.model.isEdit) {
                            $(lis[i]).hasClass("e-hover") && $(lis[i]).removeClass("e-hover");
                            this._disabledToolItems.push(lis[i]);
                        }
                        break;
                    case this._id + "_update":
                    case this._id + "_cancel":
                        if (!this.model.isEdit) {
                            $(lis[i]).hasClass("e-hover") && $(lis[i]).removeClass("e-hover");
                            this._disabledToolItems.push(lis[i]);
                        }
                        break;
                }
            }

            $toolbar.ejToolbar("disableItem", this._disabledToolItems);
            $toolbar.ejToolbar("model.enableRTL", this.model.enableRTL);
            this._disabledToolItems = $();

        },
        _getHiddenCount: function (elements) {
            var count = 0;
            for (var i = 0; i < elements.length; i++) {
                if (elements.eq(i).hasClass("e-hide"))
                    count++;
            }
            return count;
        },
        _edit: function (args) {
            var editingManager = ej.DataManager(this._currentJsonData);
            var queryManager = new ej.Query();
            if (this.model.allowFiltering)
                this._previousFilterCount = this._filteredRecordsCount;
            for (var index = 0; index < this._primaryKeys.length; index++)
                queryManager = queryManager.where(this._primaryKeys[index], ej.FilterOperators.equal, this._primaryKeyValues[index]);
            this._currentData = editingManager.executeLocal(queryManager);
            var temp = document.createElement('div');
            var $temp = $(temp), $tempSecondTR;
            if (this.model.editSettings.editMode == "normal") {
                temp.innerHTML = ['<table>', $.render[this._id + "_JSONEditingTemplate"](this._currentData, { groupedColumns: this.model.groupSettings.groupedColumns }), '</table>'].join("");
                var $tr = $temp.find("tr").first(), detailCount = 0, $tempFirstTR, firstHidden = this.model.columns.length
                , $currentTrFr = args.row.first();
                if (this.model.scrollSettings.frozenColumns > 0) {
                    $temp.prepend(['<table>', $.render[this._id + "_JSONFrozenEditingTemplate"](this._currentData, { groupedColumns: this.model.groupSettings.groupedColumns }), '</table>'].join(""));
                    $tr.splice(0, 0, $temp.find("table").first().find("tr").first().get(0));
                    $currentTrLa = args.row.last();
                    $tempLastTR = $tr.last();
                }
                $tempFirstTR = $temp.find("tr").first();
                $temp.find('td').not(".e-rowcell").addClass("e-editcell e-normaledit");
                var $select = $temp.find("select.e-field");
                for (var i = 0; i < $select.length; i++)
                    $select.eq(i).val(this._currentData[0][$select[i].name]);
                if (this.model.groupSettings.groupedColumns.length >= 2) {
                    var $indentCell = args.row.find("td.e-indentcell");
                    $temp.find("tr").first().prepend($indentCell);
                }
                if (this.model.detailsTemplate != null) {
                    detailCount++;
                    $temp.find(".e-editcell").find("tr").prepend(args.row.find("[class*='detailrow']"));
                }
                if (this.model.scrollSettings.frozenColumns > 0) {
                    $temp.find(".e-editcell").get(1).colSpan = this.model.columns.length - this.model.scrollSettings.frozenColumns - args.row.last().find(".e-hide").length + detailCount;
                    firstHidden = this.model.scrollSettings.frozenColumns;
                    $currentTrLa.hasClass("e-alt_row") && $tempLastTR.addClass("e-alt_row")
                }
                $temp.find(".e-editcell").get(0).colSpan = firstHidden - $currentTrFr.find("td").not(":visible").length + detailCount;
                $currentTrFr.hasClass("e-alt_row") && $tempFirstTR.addClass("e-alt_row");
                $currentTrFr.empty().replaceWith($tempFirstTR.addClass("e-editedrow"));
                if (this.model.scrollSettings.frozenColumns > 0)
                    $currentTrLa.empty().replaceWith($tempLastTR.addClass("e-editedrow"));
                this._refreshUnboundTemplate($tr.find(".gridform"));
                this._gridRows = this.getContentTable().first().find(".e-rowcell").closest("tr").toArray();
                if (this.model.scrollSettings.frozenColumns > 0)
                    this.getScrollObject().scrollY(this.getScrollObject().model.scrollTop, true);
                if (this.model.scrollSettings.frozenColumns > 0) 
                    this._gridRows = [this._gridRows, this.getContentTable().last().find(".e-rowcell").closest("tr").toArray()];
            }
            else if (this.model.editSettings.editMode == "inlineform" || this.model.editSettings.editMode == "inlineformtemplate") {
                var hideCount = !this.model.groupSettings.showGroupedColumn ? this.model.groupSettings.groupedColumns.length : 0;
                var detailCount = 0;
                temp.innerHTML = this.model.editSettings.editMode == "inlineform" ? $.render[this._id + "_JSONDialogEditingTemplate"](this._currentData) : $.render[this._id + "_JSONdialogTemplateMode"](this._currentData);
                var $select = $temp.find("select.e-field");
                for (var i = 0; i < $select.length; i++)
                    $select.eq(i).val(this._currentData[0][$select[i].name]);
                var tr = ej.buildTag('tr');
                var td = ej.buildTag('td');
                tr.addClass("e-editedrow");
                td.addClass("e-inlineformedit");
                td.html($(temp).children());
                tr.append(td);
                if (this.model.scrollSettings.frozenColumns > 0) {
                    var $trClone = tr.clone();
                    $trClone.find("td").empty().prop("colspan", this.model.scrollSettings.frozenColumns);
                    args.row.eq(1).after(tr).end().eq(0).after($trClone);
                    this._gridRows = [this._gridRows, this.getContentTable().last().find("tr").toArray()];
                }
                else
                    args.row.after(tr);
                this._gridRows = this.getContentTable().first().find(".e-rowcell,.e-inlineformedit").closest("tr").toArray();
                if (this.model.scrollSettings.frozenColumns > 0)
                    td.prop("colspan", this.model.columns.length - this.model.scrollSettings.frozenColumns - tr.find("form").children().not(":visible").length - hideCount + detailCount);
                else
                    td.prop("colspan", this.model.columns.length -  tr.find("form").children().not(":visible").length - hideCount + detailCount);
                if (this.model.scrollSettings.frozenColumns > 0)
                    this._gridRows = [this._gridRows, this.getContentTable().last().find("tr").toArray()];
                $("#" + this._id + "_inlineFormTitle").text(this._getLocalizedLabels("EditFormTitle") + this._currentData[0][this._primaryKeys[0]]);
                args.row.find("input").attr('disabled', 'disabled').addClass("e-disable");
            }
            else {
                $temp.addClass("e-editedrow");
                temp.innerHTML = this.model.editSettings.editMode == "dialog" || this.model.editSettings.editMode == "externalform" ? $.render[this._id + "_JSONDialogEditingTemplate"](this._currentData) : $.render[this._id + "_JSONdialogTemplateMode"](this._currentData);
                var $select = $temp.find("select.e-field");
                for (var i = 0; i < $select.length; i++)
                    $select.eq(i).val(this._currentData[0][$select[i].name]);
                if (this.model.editSettings.editMode == "dialog" || this.model.editSettings.editMode == "dialogtemplate") {
                    $("#" + this._id + "_dialogEdit").html($(temp));
                    var model = {};
                    model.cssClass = this.model.cssClass;
                    model.enableRTL = this.model.enableRTL;
                    model.width = "auto";
                    model.enableResize = this.phoneMode;
                    model.close = $.proxy(this._buttonClick, this);
                    model.content = "#" + this._id;
                    model.enableModal = true;
                    model.allowKeyboardNavigation = false;
                    model.title = this._getLocalizedLabels("EditFormTitle") + this._currentData[0][this._primaryKeys[0]];
                    $("#" + this._id + "_dialogEdit").ejDialog(model);
                    $("#" + this._id + "_dialogEdit").ejDialog("open");
                }
                else {
                    $("#" + this._id + "_externalEdit").css("display", "block").css('z-index', this._maxZindex() + 1);
                    $(".e-externalformedit").html($(temp));
                    $("#" + this._id + "_eFormHeader").find(".e-form-title").text(this._getLocalizedLabels("EditFormTitle") + this._currentData[0][this._primaryKeys[0]]);
                    this._externalFormPosition();
                    args.row.find("input").attr('disabled', 'disabled').addClass("e-disable");
                }
            }
        },
        _add: function (args) {
            var temp = document.createElement('div');
            if ((!(this._dataSource() instanceof ej.DataManager) || this._dataManager.dataSource.url == undefined) && this.model.groupSettings.groupedColumns.length == 0 && this.model.scrollSettings.frozenColumns == 0 && this.model.scrollSettings.frozenRows == 0)
                !(this._dataSource() instanceof ej.DataManager) ? this._dataSource().splice(0, 1) : this._dataSource().dataSource.json.splice(0, 1);
            this._previousFilterCount = this._filteredRecordsCount;
            if (this.model.editSettings.editMode == "normal") {
                var $tempFirstTR, $temp = $(temp), frozenColSpan = this.model.columns.length;
                temp.innerHTML = ['<table>', $.render[this._id + "_JSONEditingTemplate"](args.data, { groupedColumns: this.model.groupSettings.groupedColumns }), '</table>'].join("");
                var $select = $(temp).find('select.e-field');
                for (var i = 0; i < $select.length; i++)
                    $select.eq(i).val(args.data[$select[i].name]);
                if (this.model.scrollSettings.frozenColumns > 0) {
                    $tempLastTR = $temp.find("table").first().find("tr").first();
                    $temp.prepend(['<table>', $.render[this._id + "_JSONFrozenEditingTemplate"](args.data, { groupedColumns: this.model.groupSettings.groupedColumns }), '</table>'].join(""));
                }
                $tempFirstTR = $temp.find("tr").first();
                var td = $(temp).find(".e-editcell").get(0);
                $(temp).find('td').first().addClass("e-editcell");
                if (this.model.pageSettings.pageSize <= this.model.currentViewData.length && this.model.groupSettings.groupedColumns.length == 0)
                    this.getContentTable().get(0).lastChild.removeChild(this.getContentTable().get(0).lastChild.lastChild);
                if (this.model.detailsTemplate != null && $(this.getContentTable().get(0).lastChild.lastChild).children('.e-detailrowexpand').length)
                    this.getContentTable().get(0).lastChild.removeChild(this.getContentTable().get(0).lastChild.lastChild);
                if (this.model.currentViewData.length == 0 || this.getContentTable().find('td.e-rowcell').length == 0)
                    this.getContentTable().find('tr').first().replaceWith($(temp).find("tr").first().addClass("e-addedrow e-normaledit"));
                else {
                    if (this.model.editSettings.rowPosition == "top")
                        this.getContentTable().first().find('tbody').first().prepend($tempFirstTR.addClass("e-addedrow e-normaledit"));
                    else if (this.model.editSettings.rowPosition == "bottom")
                        this.getContentTable().first().find('tbody').first().append($tempFirstTR.addClass("e-addedrow e-normaledit"));
                    if (this.model.scrollSettings.frozenColumns > 0)
                        this.getContentTable().last().find('tbody').first().prepend($tempLastTR.addClass("e-addedrow e-normaledit"));
                }
                var $editTr = this.getContentTable().find("tr.e-addedrow");
                if (this.model.detailsTemplate != null)
                    $editTr.find('tr').first().prepend(ej.buildTag('td.e-detailrowcollapse'));
                var hideCount = !this.model.groupSettings.showGroupedColumn ? this.model.groupSettings.groupedColumns.length : 0;
                if (this.model.groupSettings.groupedColumns.length) {
                    for (var i = 0; i < this.model.groupSettings.groupedColumns.length; i++) {
                        if (i == 0)
                            $editTr.prepend(ej.buildTag("td.e-indentcell"));
                        else {
                            $editTr.find("tr").prepend(ej.buildTag("td.e-indentcell"));
                        }
                    }
                    if (this.model.groupSettings.groupedColumns.length >= 2)
                        $editTr.find("td.e-editcell").prop("colspan", (this.model.columns.length + (this.model.groupSettings.groupedColumns.length - 1) - this._hiddenColumns.length - hideCount));
                    else
                        $editTr.find("td.e-editcell").prop("colspan", (this.model.columns.length - this._hiddenColumns.length - hideCount));
                } else if (this.model.detailsTemplate != null)
                    $editTr.find(".e-editcell").prop("colspan", (this.model.columns.length - this._hiddenColumns.length - hideCount + 1));
                else {
                    if (this.model.scrollSettings.frozenColumns > 0) {
                        $editTr.find(".e-editcell").last().prop("colspan", (this.model.columns.length - this.model.scrollSettings.frozenColumns - $tempLastTR.find("td").not(":visible").length - hideCount));
                        frozenColSpan = this.model.scrollSettings.frozenColumns;
                    }
                    $editTr.find(".e-editcell").first().prop("colspan", (frozenColSpan - $tempFirstTR.find("td").not(":visible").length - hideCount));

                }
                this._refreshUnboundTemplate($editTr.find(".gridform"));
                this._gridRows = this.getContentTable().first().find(".e-rowcell").closest("tr").toArray();
                if (this.model.scrollSettings.frozenColumns > 0)
                    this._gridRows = [this._gridRows, this.getContentTable().last().find(".e-rowcell").closest("tr").toArray()];
            }
            else if (this.model.editSettings.editMode == "inlineform" || this.model.editSettings.editMode == "inlineformtemplate") {
                temp.innerHTML = this.model.editSettings.editMode == "inlineform" ? $.render[this._id + "_JSONDialogEditingTemplate"](args.data) : $.render[this._id + "_JSONdialogTemplateMode"](args.data);
                var $select = $(temp).find('select.e-field');
                for (var i = 0; i < $select.length; i++)
                    $select.eq(i).val(args.data[$select[i].name]);
                var hideCount = !this.model.groupSettings.showGroupedColumn ? this.model.groupSettings.groupedColumns.length : 0;
                var detailCount = 0;
                var tr = ej.buildTag('tr');
                var td = ej.buildTag('td');
                tr.addClass("e-addedrow");
                td.addClass("e-inlineformedit e-editcell");
                td.html($(temp).children());
                tr.append(td);
                var hideCount = !this.model.groupSettings.showGroupedColumn ? this.model.groupSettings.groupedColumns.length : 0;
                if (this.model.groupSettings.groupedColumns.length) {
                    for (var i = 0; i < this.model.groupSettings.groupedColumns.length; i++) {
                        tr.prepend(ej.buildTag("td.e-indentcell"));
                    }
                } else
                    tr.find("td.e-editcell").prop("colspan", (this.model.columns.length - this._hiddenColumns.length - hideCount));
                if (this.model.scrollSettings.frozenColumns > 0) {
                    var $trClone = tr.clone(), $divs = td.find(".gridform").children();
                    $trClone.find("td").empty().prop("colSpan", this.model.scrollSettings.frozenColumns - this._getHiddenCount($divs.slice(0, this.model.scrollSettings.frozenColumns)));
                    td.prop("colSpan", this.model.columns.length - this.model.scrollSettings.frozenColumns - this._getHiddenCount($divs.slice(this.model.scrollSettings.frozenColumns)));
                    this.getContentTable().first().find('tbody').first().prepend($trClone);
                    this.getContentTable().last().find('tbody').first().prepend(tr);
                }
                else {
                    td.prop("colspan", this.model.columns.length - this._hiddenColumns.length - hideCount + detailCount);
                    if (this.model.currentViewData.length == 0 || this.getContentTable().find('td.e-rowcell').length == 0)
                        this.getContentTable().find('tr').first().replaceWith($(tr));
                    else
                        if (this.model.pageSettings.pageSize <= this.model.currentViewData.length && this.model.groupSettings.groupedColumns.length == 0)
                            this.getContentTable().get(0).lastChild.removeChild(this.getContentTable().get(0).lastChild.lastChild);
                    if (this.model.editSettings.rowPosition == "top")
                        this.getContentTable().first().find('tbody').first().prepend(tr);
                    else if (this.model.editSettings.rowPosition == "bottom")
                        this.getContentTable().first().find('tbody').first().append(tr);
                }
                
                if (this.model.detailsTemplate != null && $(this.getContentTable().get(0).lastChild.lastChild).children('.e-detailrowexpand').length)
                    this.getContentTable().get(0).lastChild.removeChild(this.getContentTable().get(0).lastChild.lastChild);
                $("#" + this._id + "_inlineFormTitle").text(this._getLocalizedLabels("AddFormTitle"));
                this._refreshUnboundTemplate($("#" + this._id + "EditForm"));
                this._gridRows = this.getContentTable().first().find(".e-rowcell,.e-inlineformedit").closest("tr").toArray();
                if (this.model.scrollSettings.frozenColumns > 0)
                    this._gridRows = [this._gridRows, this.getContentTable().last().find(".e-rowcell").closest("tr").toArray()];
            }
            else {
                $(temp).addClass("e-addedrow");
                temp.innerHTML = this.model.editSettings.editMode == "dialog" || this.model.editSettings.editMode == "externalform" ? $.render[this._id + "_JSONDialogEditingTemplate"](args.data) : $.render[this._id + "_JSONdialogTemplateMode"](args.data);
                var $select = $(temp).find('select.e-field');
                for (var i = 0; i < $select.length; i++)
                    $select.eq(i).val(args.data[$select[i].name]);
                if (this.model.editSettings.editMode == "dialog" || this.model.editSettings.editMode == "dialogtemplate") {
                    $("#" + this._id + "_dialogEdit").html($(temp));
                    var model = {};
                    model.cssClass = this.model.cssClass;
                    model.width = "auto";
                    model.enableResize = this.phoneMode;
                    model.content = "#" + this._id;
                    model.enableModal = true;
                    model.close = $.proxy(this._buttonClick, this);
                    model.enableRTL = this.model.enableRTL;
                    model.allowKeyboardNavigation = false;
                    model.title = this._getLocalizedLabels("AddFormTitle");
                    $("#" + this._id + "_dialogEdit").ejDialog(model);
                    $("#" + this._id + "_dialogEdit").ejDialog("open");
                }
                else {
                    $("#" + this._id + "_externalEdit").css("display", "block").css('z-index', this._maxZindex() + 1);
                    $(".e-externalformedit").html($(temp));
                    $("#" + this._id + "_eFormHeader").find(".e-form-title").text(this._getLocalizedLabels("AddFormTitle"));
                    this._externalFormPosition();
                }
            }
            if (this.model.allowPaging) {
                if (this.model.filterSettings.filteredColumns.length)
                    this.getPager().ejPager({ totalRecordsCount: this._searchCount == null ? this._filteredRecordsCount : this._searchCount, currentPage: this._currentPage() });
                else
                    this.getPager().ejPager({ totalRecordsCount: this._searchCount == null ? this._gridRecordsCount : this._searchCount, currentPage: this._currentPage() });
                this._refreshGridPager();
            }
        },
        editFormValidate: function () {
            if ($.isFunction($.validator)) {
                if (this.model.scrollSettings.frozenColumns > 0) {
                    var forms = this.element.find(".gridform");
                    return forms.length > 1 ? forms.eq(0).validate().form() && forms.eq(1).validate().form() : forms.validate().form();
                }
                return $("#" + this._id + "EditForm").validate().form();
            }
            return true;
        },
        _refreshAltRow: function () {
            var $gridRows = this.getContentTable().get(0).rows;
            for (var r = 0; r < $gridRows.length; r++) {
                var $row = $($gridRows[r]);
                $row.hasClass("e-alt_row") && $row.removeClass("e-alt_row");
                if ($row.index() % 2 != 0)
                    $row.addClass("e-alt_row");
            }
        },
        _editCompleteAction: function (args) {
            var $form = this.element.find(".gridform");
            this.model.isEdit = true;
            this.setWidthToColumns();
            if (ej.Grid.Actions.Add == args.requestType) {
                $form.find(".e-field:disabled").not(".e-identity").removeAttr("disabled").removeClass("e-disable");
                for (var i = 0; i < this.model.groupSettings.groupedColumns.length - 1; i++)
                    $form.find("colgroup").prepend(this._getIndentCol());
            }
            if (this._tdsOffsetWidth.length == 0 || this.model.groupSettings.groupedColumns.length)
                this._setoffsetWidth();
            this.model.editSettings.editMode != "dialogtemplate" && this._refreshEditForm(args);
            if (this.model.scrollSettings.frozenColumns > 0 && (this.model.editSettings.editMode == "inlineform" || this.model.editSettings.editMode == "inlineformtemplate")) {
                if (args.requestType == "beginedit")
                    args.row.eq(0).next().find("td").height(args.row.eq(1).next().find("td").height());
                else
                    this.getContentTable().first().find("tr").first().find("td").height(this.getContentTable().last().find("tr").first().find("td").height());
            }
            if (this.model.scrollSettings.frozenRows > 0 && args.requestType == "beginedit")
                this._initFrozenRows();
            if ($.isFunction($.validator))
                this._initValidator();
        },
        _refreshEditForm: function (args) {
            var form = this.model.scrollSettings.frozenColumns > 0 ? this.element.find(".gridform") : document.getElementById(this._id + "EditForm");
            var elementFocused = false,columnIndex;
            var $formElement = $(form).find("input,select"), percent = 86;
            if (this._isUnboundColumn || this.getContentTable().find(".e-templatecell") != null)
                $formElement = $formElement.filter(function () { return (!$(this).closest("td").hasClass("e-unboundcell") && !$(this).closest("td").hasClass("e-templatecell")) })
            for (var i = 0; i < $formElement.length; i++) {
                var $element = $formElement.eq(i);
                var inputWidth, column = this.getColumnByField($element.prop("name"));
                if (this.model.editSettings.editMode == "externalform" || this.model.editSettings.editMode == "externalformtemplate" || this.model.editSettings.editMode == "inlineform" || this.model.editSettings.editMode == "inlineformtemplate")
                    $element.parent().css("width", ej.dataUtil.max(this._tdsOffsetWidth) + "px");
                if (column != null)
                    columnIndex = $.inArray(column, this.model.columns);
                if (this.model.editSettings.editMode == "batch") {
                    percent = 95;
                }
                else if (this.model.editSettings.editMode == "normal")
                    percent = 96;
                inputWidth = this._tdsOffsetWidth[i] * (percent / 100);
                if (column !=null && columnIndex !== undefined && (columnIndex < this.model.columns.length && this.model.columns[columnIndex].editTemplate)) {
                    var temp = { rowdata: this._currentJsonData[this._bulkEditCellDetails.rowIndex], column: this.model.columns, element: $element };
                    var temp1 = this.model.columns[columnIndex].editTemplate.write;
                    if (!ej.isNullOrUndefined(args) && args.requestType == "add") temp.rowdata = {};
                    if (typeof temp1 == "string")
                        temp1 = ej.util.getObject(temp1, window);
                    temp1(temp);
                }
                else if ($element.hasClass("e-numerictextbox")) {
                    var params = { width: inputWidth }, value = $element.val(), customParams = this.getColumnByField($element.prop("name"));
                    if (customParams["width"] && typeof customParams["width"] == "string" && customParams["width"].indexOf("%") != -1 && (this.model.editSettings.editMode == "normal" || this.model.editSettings.editMode == "batch"))
                        params.width = "100%";
                    params.enableRTL = this.model.enableRTL;
                    params.showSpinButton = true;
                    params.cssClass = this.model.cssClass;
                    if (value.length)
                        params.value = parseFloat(value);
                    if ($element.hasClass("e-disable"))
                        params.enabled = false;
                    if (!ej.isNullOrUndefined(customParams["editParams"]))
                        $.extend(params, customParams["editParams"]);
                    $element.ejNumericTextbox(params);
                    $element.prop("name", $element.prop("name").replace(this._id, ""));
                } else if ($element.hasClass("e-datepicker")) {
                    var params = { width: inputWidth }, column = this.getColumnByField($element.prop("name"));
                    if (column["width"] && typeof column["width"] == "string" && column["width"].indexOf("%") != -1 && (this.model.editSettings.editMode == "normal" || this.model.editSettings.editMode == "batch"))
                        params.width = "100%";
                    params.enableRTL = this.model.enableRTL;
                    params.cssClass = this.model.cssClass;
                    params.displayDefaultDate = true;
                    params.showPopupButton = false;
                    if ($element.val().length)
                        params.value = new Date($element.val());
                    if ($element.hasClass("e-disable"))
                        params.enabled = false;
                    if (column["format"] !== undefined && column.format.length > 0) {
                        var toformat = new RegExp("\\{0(:([^\\}]+))?\\}", "gm");
                        var formatVal = toformat.exec(column.format);
                        params.dateFormat = formatVal[2];
                    }
                    if (!ej.isNullOrUndefined(column["editParams"]))
                        $.extend(params, column["editParams"]);
                    $element.ejDatePicker(params);
                }
                else if ($element.hasClass("e-datetimepicker")) {
                    var column = this.getColumnByField($element.prop("name")),
                        params = {
                            width: inputWidth,
                            enableRTL: this.model.enableRTL,
                            cssClass: this.model.cssClass,
                            showPopupButton: false
                        };
                    if (column["width"] && typeof column["width"] == "string" && column["width"].indexOf("%") != -1 && (this.model.editSettings.editMode == "normal" || this.model.editSettings.editMode == "batch"))
                        params.width = "100%";
                    if ($element.val().length)
                        params.value = new Date($element.val());
                    if ($element.hasClass("e-disable"))
                        params.enabled = false;
                    if (column["format"] !== undefined && column.format.length > 0) {
                        var toformat = new RegExp("\\{0(:([^\\}]+))?\\}", "gm");
                        var formatVal = toformat.exec(column.format);
                        params.dateTimeFormat = formatVal[2];
                    }
                    if (!ej.isNullOrUndefined(column["editParams"]))
                        $.extend(params, column["editParams"]);
                    $element.ejDateTimePicker(params);
                }
                else if ($element.hasClass("e-dropdownlist")) {
                    var column = this.getColumnByField($element.prop("name")),
                        params = {
                            width: inputWidth,
                            enableRTL: this.model.enableRTL
                        };
                    if (column["width"] && typeof column["width"] == "string" && column["width"].indexOf("%") != -1 && (this.model.editSettings.editMode == "normal" || this.model.editSettings.editMode == "batch"))
                        params.width = "100%";
                    if (!ej.isNullOrUndefined(column["editParams"]))
                        $.extend(params, column["editParams"]);
                    $element.ejDropDownList(params);
                    $element.ejDropDownList("setSelectedValue", $element.val());
                    if ($element.hasClass("e-disable"))
                        $element.ejDropDownList("disable");
                }
                    //else if ($element.hasClass("checkbox"))
                    //{
                    //    var value = $element.prop("checked");
                    //    $element.ejCheckBox({
                    //        id: $element[0].id,
                    //        checked: value
                    //    });
                    //}
                else {
                    switch ($element.prop('tagName')) {
                        case "INPUT":
                            if ($element.attr("type") != "checkbox") {
                                $element.css("text-align", $element.attr("name") != null && this.getColumnByField($element.attr("name")) != null ?
                                this.getColumnByField($element.attr("name")).textAlign : "center");
                                if (this.model.editSettings.editMode == "batch")
                                    $element.width('100%').height(28);
                                else if (this.model.editSettings.editMode == "normal")
                                    $element.css('width', '100%').css("height", "30px");
                                else
                                    $element.outerWidth(inputWidth).height(28);

                            }
                            else
                                $element.width(inputWidth > 0 ? ($element.width() > inputWidth ? inputWidth : $element.width()) : 1);
                            break;
                        case "SELECT":
                            $element.width(inputWidth).height(28);
                            break;
                    }
                }
                if (!$element.is(":disabled") && !elementFocused && (!$element.is(":hidden") || typeof $element.data("ejDropDownList") == "object")) {
                    this._focusElements($element.closest('td'));
                    elementFocused = true;
                }
            }

        },
        _focusElements: function ($currentCell) {
            if ($currentCell.length) {
                var $childElem = $currentCell.children();
                if ($childElem[0].tagName.toLowerCase() == "select" || $childElem[0].tagName.toLowerCase() == "input") {
                    $childElem.focus().select();
                    $childElem[0].focus();
                }
                else if ($childElem.find(".e-field.e-dropdownlist").length)
                    $childElem.find(".e-ddl").focus();
                else
                    $childElem.find('input:visible,select').first().select().focus();
            }
        },
        _renderToolBar: function () {
            var $div = ej.buildTag('div.e-gridtoolbar', "", {}, { id: this._id + "_toolbarItems" });
            var $ul = ej.buildTag("ul");
            (!ej.isNullOrUndefined(this.model.toolbarSettings.toolbarItems) && this.model.toolbarSettings.toolbarItems.length) && this._renderLi($ul);
            $div.append($ul);
            var $customUl = ej.buildTag("ul");
            $div.append($customUl);
            (!ej.isNullOrUndefined(this.model.toolbarSettings.customToolbarItems) && this.model.toolbarSettings.customToolbarItems.length) && this._renderCustomLi($customUl);
            var model = {};
            model.click = this._toolBarClick;
            model.cssClass = this.model.cssClass;
            model.enableRTL = this.model.enableRTL;
            model.enableSeprator = false;
            $div.ejToolbar(model);
            $div.ejToolbar("disableItem", this._disabledToolItems);
            this._disabledToolItems = $();
            return $div;
        },
        _renderCustomLi: function ($ul) {
            for (var i = 0; i < this.model.toolbarSettings.customToolbarItems.length; i++) {
                var $li = ej.buildTag("li", "", {}, { title: this.model.toolbarSettings.customToolbarItems[i] });
                switch (typeof this.model.toolbarSettings.customToolbarItems[i]) {
                    case "string":
                        var $item = ej.buildTag("a.e-toolbaricons e-icon", "", {}).addClass(this.model.toolbarSettings.customToolbarItems[i]);
                        break;
                    case "object":
                        $li.attr("title", this.model.toolbarSettings.customToolbarItems[i]["templateID"].replace("#", ""));
                        var $item = $(this.model.toolbarSettings.customToolbarItems[i]["templateID"]).hide().html();
                        break;
                }
                $li.html($item);
                $ul.append($li);
            }
        },
        _renderLi: function ($ul) {
            for (var i = 0; i < this.model.toolbarSettings.toolbarItems.length; i++) {
                var $li = ej.buildTag("li", "", {}, { id: this._id + "_" + this.model.toolbarSettings.toolbarItems[i], title: this._getLocalizedLabels(this.model.toolbarSettings.toolbarItems[i].slice(0, 1).toUpperCase() + this.model.toolbarSettings.toolbarItems[i].slice(1)), "tabindex": "0" });
                this._renderLiContent($li, this.model.toolbarSettings.toolbarItems[i]);
                $ul.append($li);
            }
        },
        _renderLiContent: function ($li, item) {
            var $a, $input;
            switch (item) {
                case "add":
                    $a = ej.buildTag("a.e-addnewitem e-toolbaricons e-icon e-addnew", "", {});
                    break;
                case "edit":
                    $a = ej.buildTag("a.e-edititem e-toolbaricons e-icon e-edit", "", {});
                    break;
                case "delete":
                    $a = ej.buildTag("a.e-deleteitem e-toolbaricons e-icon e-delete", "", {});
                    break;
                case "update":
                    $a = ej.buildTag("a.e-saveitem e-toolbaricons e-disabletool e-icon e-save", "", {});
                    this._disabledToolItems.push($li.get(0));
                    break;
                case "cancel":
                    $a = ej.buildTag("a.e-cancel e-toolbaricons e-disabletool e-icon e-gcancel", "", {});
                    this._disabledToolItems.push($li.get(0));
                    break;
				
                case "search":
                    $input = ej.buildTag("input.e-ejinputtext", "", {}, { type: "text" });
                    $a = ej.buildTag("a.e-searchitem e-toolbaricons e-disabletool e-icon e-searchfind", "", {});
                    $li.append($input);
                    this.model.allowSearching = true;
                    break;
				case "printGrid":
					 $a = ej.buildTag("a.e-print e-toolbaricons e-icon", "", {});
					 break;
                case "excelExport":
                    $a = ej.buildTag("a.e-toolbaricons e-excelIcon e-icon", "", {});
                    break;
                case "wordExport":
                    $a = ej.buildTag("a.e-toolbaricons e-wordIcon e-icon", "", {});
                    break;
                case "pdfExport":
                    $a = ej.buildTag("a.e-toolbaricons e-pdfIcon e-icon", "", {});
                    break;
            }
            $li.append($a);

        },
        _toolBarClick: function (Sender) {
            var $gridEle = $(this.itemsContainer).closest(".e-grid"), gridInstance = $gridEle.ejGrid("instance"), gridId = $gridEle.attr('id');
            if (Sender.event == undefined && Sender.target.tagName == "INPUT" && Sender.currentTarget.id == gridId + "_search")
                return;
            $.isFunction($.fn.ejDatePicker) && $("#" + gridId + "EditForm").find(".e-datepicker").ejDatePicker("hide");
            var currentTarget = Sender.currentTarget; var target = Sender.target;
            var args = { itemName: currentTarget.title, itemId: currentTarget.id, currentTarget: currentTarget, target: target, itemIndex: $(currentTarget).index(), itemCurrentTarget: currentTarget.outerHTML, itemTarget: target.outerHTML, toolbarData: Sender };
            if ($gridEle.ejGrid("instance")._trigger("toolbarClick", args))
                return;
            switch (args.itemId) {
                case gridId + "_add":
                    gridInstance._toolbarOperation(gridId + "_add");
                    break;
                case gridId + "_edit":
                    gridInstance._toolbarOperation(gridId + "_edit");
                    break;
                case gridId + "_delete":
                    gridInstance._toolbarOperation(gridId + "_delete");
                    break;
                case gridId + "_update":
                    gridInstance._toolbarOperation(gridId + "_update");
                    break;
                case gridId + "_cancel":
                    if (gridInstance.model.editSettings.editMode == "batch")
                        gridInstance.cancelEdit();
                    else
                        gridInstance._toolbarOperation(gridId + "_cancel");
                    break;
                case gridId + "_search":
                    gridInstance._toolbarOperation(gridId + "_search", $(Sender.currentTarget).find("input").val());
                    break;
				case gridId + "_printGrid":
                    gridInstance._toolbarOperation(gridId + "_printGrid");
                    break;
                case gridId + "_excelExport":
                    gridInstance._toolbarOperation(gridId + "_excelExport");
                    break;
                case gridId + "_wordExport":
                    gridInstance._toolbarOperation(gridId + "_wordExport");
                    break;
                case gridId + "_pdfExport":
                    gridInstance._toolbarOperation(gridId + "_pdfExport");
                    break;
            }
            return false;
        },

        _toolbarOperation: function (operation, searchEle) {
            var $gridEle = this.element, gridObject = $gridEle.ejGrid("instance"), batchEnable = gridObject.model.editSettings.editMode == "batch", gridId = $gridEle.attr('id'), fieldName;
            switch (operation) {
                case gridId + "_add":
                    if (batchEnable)
                        gridObject._bulkAddRow();
                    else
                        gridObject._startAdd();
                    break;
                case gridId + "_edit":
                    if (batchEnable) {
					    if (gridObject._bulkEditCellDetails.columnIndex == -1) {
                            alert(this._getLocalizedLabels("EditOperationAlert"));
                            return;
                        }
                        fieldName = gridObject.model.columns[gridObject._bulkEditCellDetails.columnIndex].field;
                        fieldName && gridObject.editCell(gridObject._bulkEditCellDetails.rowIndex, fieldName);
                    }
                    else
                        gridObject.startEdit();
                    break;
                case gridId + "_delete":
                    if (batchEnable)
                        gridObject._bulkDelete();
                    else
                        gridObject._deleteRow();
                    break;
                case gridId + "_update":
                    if (batchEnable)
                        this.model.editSettings.showConfirmDialog ? this._confirmDialog.find(".e-content").html(this._getLocalizedLabels("BatchSaveConfirm")).end().ejDialog("open") : this.batchSave();
                    else
                        gridObject.endEdit();
                    break;
                case gridId + "_cancel":
                    if (batchEnable) {
                        if ($("#" + gridId + "ConfirmDialog").ejDialog("isOpened") === true)
                            this._triggerConfirm();
                        else
                            gridObject.cancelEditCell();
                    }
                    else
                        gridObject.cancelEdit();
                    break;
                case gridId + "_search":
                    $gridEle.ejGrid("search", searchEle);
                    break;
				case gridId + "_printGrid":
                    this.print();
                    break;
                case gridId + "_excelExport":
                    gridObject.export("ExportToExcel", 'excelExporting',true);
                    break;
                case gridId + "_wordExport":
                    gridObject.export("ExportToWord", 'wordExporting',true);
                    break;
                case gridId + "_pdfExport":
                    gridObject.export("ExportToPdf", 'pdfExporting',true);
                    break;
            }
            return false;
        },
        renderDiaglogButton: function (form, tbody) {
            var btnId;
            if (this.model.editSettings.editMode == "externalform" || this.model.editSettings.editMode == "externalformtemplate")
                btnId = "EditExternalForm_";
            else if (this.model.editSettings.editMode == "inlineform" || this.model.editSettings.editMode == "inlineformtemplate") {
                btnId = "InlineEditForm_";
                var inlineTitleBar = ej.buildTag("div", "", "", { id: this._id + "_inlineFormTitle", 'class': "e-inlineform-titlebar" });
                inlineTitleBar.appendTo(tbody);
            }
            else
                btnId = "EditDialog_";
            var savebtn = ej.buildTag('input', "", { 'margin-left': '30px' }, { type: "button", id: btnId + this._id + "_Save" });
            savebtn.ejButton({ cssClass: this.model.cssClass, enableRTL: this.model.enableRTL, text: this._getLocalizedLabels("SaveButton"), width: "100" });
            var cancelbtn = ej.buildTag('input', "", { 'margin-left': '19px', 'margin-right': '13px' }, { type: "button", id: btnId + this._id + "_Cancel" });
            cancelbtn.ejButton({ cssClass: this.model.cssClass, enableRTL: this.model.enableRTL, text: this._getLocalizedLabels("CancelButton"), width: "100" });
            var btnDiv = (this.model.editSettings.editMode != "dialog" && this.model.editSettings.editMode != "dialogtemplate") ? ej.buildTag('div', "", "", { 'class': "e-editform-btn" }) : ej.buildTag('div');
            btnDiv.append(savebtn);
            btnDiv.append(cancelbtn);
            form.appendTo(tbody);
            if (this.model.editSettings.editMode != "dialog" && this.model.editSettings.editMode != "dialogtemplate")
                btnDiv.appendTo(tbody);
            else
                form.append(btnDiv);
            return tbody;
        },
        _externalFormPosition: function () {
            var pos = $(this.element).offset();
            var width = $(this.element).width();
            var height = $(this.element).height();
            var DivElement = $("#" + this._id + "_externalEdit");
            switch (this.model.editSettings.formPosition) {
                case "topright":
                    $(DivElement).find('.e-form-toggle-icon').removeClass('e-bottomleft').addClass('e-topright');
                    $(DivElement).css({ "left": (pos.left + width + 1) + "px", "top": pos.top + "px", "position": "absolute", "width": "290px"  });
                    $("#" + this._id + "_eFormContent").height("auto");
                    break;
                case "bottomleft":
                    $(DivElement).find('.e-form-toggle-icon').removeClass('e-topright').addClass('e-bottomleft');
                    $(DivElement).css({ "left": (pos.left) + "px", "top": (pos.top + height + 1) + "px" });
                    $("#" + this._id + "_eFormContent").width("100%");
                    break;
            }
        },
        _setoffsetWidth: function () {
            var tds, $form = this.model.scrollSettings.frozenColumns > 0 ? this.element.find(".gridform") : $("#" + this._id + "EditForm");
            if (this._gridRecordsCount == 0 && this.model.editSettings.editMode != "batch")
                return;
            if (this.model.editSettings.editMode == "batch")
                tds = $form.closest("td");
            else if (this.model.editSettings.editMode == "inlineform" || this.model.editSettings.editMode == "inlineformtemplate" || this.model.editSettings.editMode == "externalform" || this.model.editSettings.editMode == "externalformtemplate")
                tds = $form.find(".e-rowcell").not(".e-unboundcell,.e-templatecell");
            else
                tds = $form.find("tr").find(".e-rowcell").not(".e-unboundcell,.e-templatecell");
            for (var i = 0; i < tds.length; i++)
                this._tdsOffsetWidth[i] = tds.get(i).offsetWidth;
        },
        _bulkChangesAcquired: function () {
            if (this._batchChanges.added.length > 0 || this._batchChanges.changed.length || this._batchChanges.deleted.length)
                return true;
            return false;
        },
        _renderDialog: function () {
            var $dialog = ej.buildTag("div.e-dialog e-dialog-content e-shadow e-widget-content", "", { display: "none" }, { id: this._id + "_dialogEdit" });
            return $dialog;
        },
        /**
         * Get the data of currently edited cell value in "batch" edit mode
         * @alias ejGrid#getCurrentEditCellData
         * @return jQuery
         * @example 
         * &lt;script&gt;
         * // Create grid object.
         * var gridObj = $("#Grid").data("ejGrid");
         * // Get data of currently edited cell value
         * gridObj.getCurrentEditCellData(); 
         * &lt;/script&gt;
         * @example 
         * &lt;script&gt;
         * // Get data of currently edited cell value
         * $("#Grid").ejGrid("getCurrentEditCellData");        
         * &lt;/script&gt;
         */
        getCurrentEditCellData: function () {
            if (this.model.isEdit && $("#" + this._id + "EditForm").length) {
                var $element = $("#" + this._id + this._bulkEditCellDetails.fieldName), cellValue;
                switch (this._bulkEditCellDetails.cellEditType) {
                    case ej.Grid.EditingType.String:
                        cellValue = $element.val();
                        break;
                    case ej.Grid.EditingType.Numeric:
                        cellValue = $element.ejNumericTextbox("getValue");
                        $element.ejNumericTextbox("getValue");
                        break;
                    case ej.Grid.EditingType.Dropdown:
                        cellValue = this._bulkEditCellDetails.isForeignKey ? { "value": $element.ejDropDownList("getSelectedValue"), "text": $element.ejDropDownList("getSelectedItem") } : $element.ejDropDownList("getSelectedValue");
                        break;
                    case ej.Grid.EditingType.Boolean:
                        cellValue = $element.is(':checked');
                        break;
                    case ej.Grid.EditingType.DatePicker:
                        cellValue = $element.ejDatePicker("model.value");
                        break;
                    case ej.Grid.EditingType.DateTimePicker:
                        cellValue = $element.ejDateTimePicker("model.value");
                        break;
                }
                if (typeof cellValue == "string" && cellValue.length && this.model.columns[this._bulkEditCellDetails.columnIndex].type == "number")
                    cellValue = parseFloat(cellValue);
                return cellValue;
            }
            return null;
        },
        cancelEditCell: function () {
            if (this.model.isEdit) {
                var tr = this.getRows()[this._bulkEditCellDetails.rowIndex], cellData = {}, cell;
                cellData[this._bulkEditCellDetails.fieldName] = this._bulkEditCellDetails.cellValue;
                if ($(tr).hasClass("e-insertedrow"))
                    cell = tr.cells[this._bulkEditCellDetails.columnIndex + this.model.groupSettings.groupedColumns.length];
                else
                    cell = tr.cells[this._bulkEditCellDetails.columnIndex];
                $(cell).removeClass("e-editedbatchcell").empty().html($($.templates.Grid_JSONTemplate.render(cellData))[0].cells[this._bulkEditCellDetails.columnIndex].innerHTML);
                this.model.isEdit = false;
                this.element.focus();
            }
        },
        saveCell: function (preventSaveEvent) {
            if (this.model.isEdit) {
                if (!this.editFormValidate())
                    return true;
                var $form = $("#" + this._id + "EditForm"), $targetTR = $form.closest("tr"), $targetTD = $form.closest("td"), $toolBar, formattedValue
                , args = {}, column = this.model.columns[this._bulkEditCellDetails.columnIndex], $element = $("#" + this._id + this._bulkEditCellDetails.fieldName);
                args = {
                    columnName: column.field,
                    value: this.getCurrentEditCellData(),
                    rowData: this._bulkEditCellDetails.rowData,
                    previousValue: this._bulkEditCellDetails.cellValue,
                    columnObject: column,
                    cell: $targetTD,
                    isForeignKey: this._bulkEditCellDetails.isForeignKey
                };
                if (!preventSaveEvent && this._trigger("cellSave", args)) {
                    this._focusElements(args.cell);
                    this._bulkEditCellDetails.cancelSave = true;
                    return;
                }
                if (this._bulkEditCellDetails.cellEditType == "datetimepicker" || this._bulkEditCellDetails.cellEditType == "dropdownedit" || this._bulkEditCellDetails.cellEditType == "datepicker")
                    $element[$element.data("ejWidgets")[0]]("destroy");
                if (!ej.isNullOrUndefined(column.format)) {
                    formattedValue = this.formatting(column.format, args.isForeignKey ?
                                              (!isNaN(parseFloat(args.value.text)) && isFinite(args.value.text)
                                              ? parseFloat(args.value.text)
                                              : args.value.text) : args.value);
                    args.cell.empty().html(formattedValue);
                }
                if (this._bulkEditCellDetails.cellEditType == "edittemplate") {
                    var temp1 = column.editTemplate.read;
                    if (typeof temp1 == "string")
                        temp1 = ej.util.getObject(temp1, window);
                    args.value = temp1($element);
                    args.rowData[this._bulkEditCellDetails["fieldName"]] = args.value;
                    args.cell.empty().html(args.value);
                }
                else {
                    if (args.columnObject.type == "boolean" || args.columnObject.editType == "booleanedit") {
                        var cellData = {};
                        cellData[args.columnObject.field] = args.value;
                        args.cell.empty().html($($.templates[this._id +"_JSONTemplate"].render(cellData))[0].cells[this._bulkEditCellDetails.columnIndex].innerHTML);
                    }
                    else
                        args.cell.empty().html(args.isForeignKey ? args.value.text : args.value);
                }
                var isValueModified = ((this._bulkEditCellDetails.cellEditType == "datepicker" || this._bulkEditCellDetails.cellEditType == "datetimepicker") && args.value instanceof Date && args.previousValue instanceof Date) ? (args.value.getTime() !== args.previousValue.getTime()) : (args.value !== args.previousValue);
                this.model.isEdit = false;
                if (isValueModified) {
                    this._enableSaveCancel();
                    args.cell.addClass("e-updatedtd e-icon e-gupdatenotify");
                    if (typeof args.rowData[this._bulkEditCellDetails["fieldName"]] == "string" && args.rowData[this._bulkEditCellDetails["fieldName"]].length)
                        args.rowData[this._bulkEditCellDetails["fieldName"]] = args.isForeignKey ? args.value.value.toString() : args.value.toString();
                    else
                        args.rowData[this._bulkEditCellDetails["fieldName"]] = args.isForeignKey ? (!isNaN(parseInt(args.value.value)) ? parseInt(args.value.value) : args.value.value) : args.value;
                    if ($.inArray(args.rowData, this._batchChanges.changed) == -1 && $.inArray(args.rowData, this._batchChanges.added) == -1)
                        this._batchChanges.changed.push(args.rowData);
                }
                $targetTR.removeClass("e-editedrow");
                args.cell.removeClass("e-editedbatchcell");
            }
        },
        _enableSaveCancel: function () {
            if (this.model.toolbarSettings.showToolbar) {
                $toolBar = this.element.find("#" + this._id + "_toolbarItems");
                $toolBar.ejToolbar("enableItemByID", this._id + "_update");
                $toolBar.ejToolbar("enableItemByID", this._id + "_cancel");
            }
        },

        setCellValue: function (index, fieldName, cellValue) {
            var data = this.getDataByIndex(index), tr = this.getRows()[index], dataIndex
                , columnIndex = this.getColumnIndexByField(fieldName);
            data[fieldName] = cellValue;
            if ($(tr).hasClass("e-insertedrow"))
                tr.cells[columnIndex + this.model.groupSettings.groupedColumns.length].innerHTML = cellValue;
            else {
                $.inArray(data, this._batchChanges.changed) == -1 && this._batchChanges.changed.push(data);
                tr.cells[columnIndex].innerHTML = cellValue;
            }
            this._enableSaveCancel();
        },
        setDefaultData: function (defaultData) {
            if (ej.isNullOrUndefined(defaultData)) {
                var fieldNames = this.getColumnFieldNames();
                defaultData = $.extend(true, {}, this._bulkEditCellDetails._data[0]);
                for (var field in defaultData) {
                    var index = $.inArray(field, fieldNames);
                    index == -1 && delete defaultData[field];
                    if (defaultData.hasOwnProperty(field)) {
                        if (!ej.isNullOrUndefined(this.model.columns[index]["defaultValue"])) {
                            defaultData[field] = this.model.columns[index]["defaultValue"];
                        }
                        else {
                            switch (typeof defaultData[field]) {
                                case "number":
                                    defaultData[field] = 0;
                                    break;
                                case "string":
                                    defaultData[field] = "";
                                    break;
                                case "boolean":
                                    defaultData[field] = false;
                                    break;
                                case "object":
                                    if ($.isArray(defaultData[field]))
                                        defaultData[field] = new Array();
                                    else
                                        defaultData[field] = "";
                            }
                        }
                    }
                }
            }
            this._bulkEditCellDetails.defaultData = defaultData;
        },
        _bulkDelete: function (index) {
            if (ej.isNullOrUndefined(index))
                index = this._selectedRow();
		    if (index == -1) {
                alert(this._getLocalizedLabels("DeleteOperationAlert"));
                return;
            }
            var tr = this.getRows()[index], $tr = $(tr), data = this.getDataByIndex(index), args = {};
            args = {
                primaryKey: this._primaryKeys,
                rowIndex: index,
                rowData: data,
                row: $tr
            };
            if (this._trigger("beforeBatchDelete", args))
                return;
            if ($tr.hasClass("e-insertedrow")) {
                $tr.remove();
                index = $.inArray(tr, this._bulkEditCellDetails.insertedTrCollection);
                if (index != -1) {
                    this._bulkEditCellDetails.insertedTrCollection.splice(index, 1);
                    this._batchChanges.added.splice(index, 1);
                }
            }
            else {
                $tr.hide();
                this._batchChanges.deleted.push(data);
            }
            this._enableSaveCancel();
            this._selectedRow(-1);
            args = {};
            this._trigger("batchDelete", args);
        },
        _bulkAddRow: function (defaultData) {
            var args = {}, $tr, editCellIndex, rows = this.getRows();
            if (!ej.isNullOrUndefined(defaultData))
                this._bulkEditCellDetails.defaultData = defaultData;
            ej.isNullOrUndefined(this._bulkEditCellDetails.defaultData) && this.setDefaultData();
            args = {
                defaultData: $.extend(true, {}, this._bulkEditCellDetails.defaultData),
                primaryKey: this._primaryKeys,
            };
            if (this._trigger("beforeBatchAdd", args))
                return;
            $tr = $($.render[this._id + "_JSONTemplate"](args.defaultData)).addClass("e-insertedrow");
            rows != null && $(rows[0]).hasClass("e-alt_row") && $tr.removeClass("e-alt_row");
            if (this.model.editSettings.rowPosition == "top")
                this.getContentTable().first().find('tbody').first().prepend($tr);
            else if (this.model.editSettings.rowPosition == "bottom")
                this.getContentTable().first().find('tbody').first().append($tr);
			if (this._gridRecordsCount === 0)
                this.getContentTable().find("tbody .emptyrecord").first().remove();
            this._gridRows = this.getContentTable().find("td.e-rowcell").closest("tr").toArray();
            for (var i = 0; i < this.model.groupSettings.groupedColumns.length; i++)
                $tr.prepend(ej.buildTag("td.e-indentcell"));
            this._bulkEditCellDetails.insertedTrCollection.push($tr.get(0));
            this._batchChanges.added.push(args.defaultData);
            this._enableSaveCancel();
            var rowindex = this._gridRows.length - 1;
            if (this.model.editSettings.rowPosition == "bottom")
            {
                editCellIndex = this._findNextEditableCell(0);
                this.selectRows(rowindex);
                this.editCell(rowindex, this.model.columns[editCellIndex].field);
            }
            else
            {
                editCellIndex = this._findNextEditableCell(0);
                this.selectRows(0);
                this.editCell(0, this.model.columns[editCellIndex].field);
            }
            $tr.find(".e-rowcell").addClass("e-updatedtd e-icon e-gupdatenotify");
            args = { defaultData: args.defaultData };
            $.extend(args, {
                columnObject: this.model.columns[editCellIndex],
                columnIndex: editCellIndex,
                row: $tr,
                primaryKey: this._primaryKeys,
                cell: $($tr[0].cells[editCellIndex])
            });
            this._trigger("batchAdd", args);
        },
        getDataByIndex: function (rowIndex) {
            var $tr = $(this.getRows()[rowIndex]), insertedRowIndex, currentRowData, index;
            if ($tr.hasClass("e-insertedrow")) {
                insertedRowIndex = $.inArray($tr[0], this._bulkEditCellDetails.insertedTrCollection);
                return this._batchChanges.added[insertedRowIndex];
            }
            else
                return this._bulkEditCellDetails._data[rowIndex - this._bulkEditCellDetails.insertedTrCollection.length];

        },
        /**
         * It is used to refresh and reset the changes made in "batch" edit mode
         * @alias ejGrid#refreshBatchEditChanges
         * @return jQuery
         * @example 
         * &lt;script&gt;
         * // Create grid object.
         * var gridObj = $("#Grid").data("ejGrid");
         * gridObj.refreshBatchEditChanges(); 
         * // It is used to refresh and reset the changes made in batch edit mode
         * &lt;/script&gt;
         * @example 
         * &lt;script&gt;
         * // It is used to refresh and reset the changes made in batch edit mode
         * $("#Grid").ejGrid("refreshBatchEditChanges");
         * &lt;/script&gt;
         */
        refreshBatchEditChanges: function () {
            this._bulkEditCellDetails = {
                cellValue: null,
                rowIndex: -1,
                _data: null,
                columnIndex: -1,
                fieldName: null,
                cellEditType: "",
                cancelSave: false,
                defaultData: null,
                insertedTrCollection: [],
                rowData: null,
                isForeignKey: false
            };
            this._batchChanges = {
                added: [],
                deleted: [],
                changed: []
            };
        },
        refreshBatchEditMode: function () {
            if (this.model.editSettings.editMode == "batch") {
                this.refreshBatchEditChanges();
                this._bulkEditCellDetails._data = $.extend(true, [], this.getCurrentViewData());
            }
        },
        /**
         * Cancel the modified changes in grid control when edit mode is "batch".
         * @alias ejGrid#batchCancel
         * @return jQuery
         * @example 
         * &lt;script&gt;
         * // Create grid object.
         * var gridObj = $("#Grid").data("ejGrid");
         * gridObj.batchCancel();
         * // Cancel added, edited, and deleted changes made in grid
         * &lt;/script&gt;
         * @example 
         * &lt;script&gt;
         * // Cancel added, edited, and deleted changes made in grid
         * $("#Grid").ejGrid("batchCancel");
         * &lt;/script&gt;
         */
        batchCancel: function () {
            this.cancelEdit();
        },
        /**
		 * Save the modified changes to data source in grid control when edit mode is "batch".
         * @alias ejGrid#batchSave
		 * @return jQuery
		 * @example 
		 * &lt;script&gt;
		 * // Create grid object.
		 * var gridObj = $("#Grid").data("ejGrid");
         * // Save added, edited, and deleted changes to source of data
		 * gridObj.batchSave(); 
		 * &lt;/script&gt;
		 * @example 
		 * &lt;script&gt;
		 * // Save added, edited, and deleted changes to source of data
		 * $("#Grid").ejGrid("batchSave");
		 * &lt;/script&gt;
         */
        batchSave: function () {
            var args = {}, deferedObject, gridObject = this;
            this.saveCell();
            args["batchChanges"] = this.getBatchChanges();
            if (this._trigger("beforeBatchSave", args))
                return;
            args = {};
            args.requestType = "batchsave";
            var deferedObject = this._dataManager.saveChanges(this.getBatchChanges(), this._primaryKeys[0], this.model.query._fromTable);
            if (this._dataManager instanceof ej.DataManager && !this._dataManager.dataSource.offline) {
                deferedObject.done(function (e) {
                    gridObject._processBindings(args);
                });
            }
            else
                this._processBindings(args);

        },
        /**
         * Get the batch changes of edit, delete and add operations of grid.
         * @alias ejGrid#getBatchChanges
         * @return jQuery
         * @example
         * &lt;script&gt;
         * // Create grid object.
         * var gridObj = $("#Grid").data("ejGrid");
         * // Gets the edit, delete, and add changes of a grid
         * gridObj.getBatchChanges(); 
         * &lt;/script&gt;
         * @example
         * &lt;script&gt;
         * // Gets the edit, delete, and add changes of a grid
         * $("#Grid").ejGrid("getBatchChanges");        
         * &lt;/script&gt;
         */
        getBatchChanges: function () {
            return this._batchChanges;
        },
        /**
         * Edit a particular cell based on the row index and field name provided in "batch" edit mode.
         * @alias ejGrid#editCell
         * @return jQuery
         * @param { number } index Pass row index to edit particular cell
         * @param { string } fieldName Pass the field name of the column to perform batch edit 
         * @example
         * &lt;script&gt;
         * // Create grid object.
         * var gridObj = $("#Grid").data("ejGrid");
         * // Edit particular cell based on row index and column field name
         * gridObj.editCell(2, "OrderID"); 
         * &lt;/script&gt;
         * @example
         * &lt;script&gt;
         * // Edit particular cell based on row index and column field name
         * $("#Grid").ejGrid("editCell", 2, "OrderID");
         * &lt;/script&gt;
         */
        editCell: function (index, fieldName) {
            var $form = $("#" + this._id + "EditForm");
            this.model.isEdit && this.saveCell();
            if ($.isFunction($.validator) && $form.length && $form.validate().errorList.length)
                return;
            var $targetTR = $(this.getRows()[index]), columnIndex = this.getColumnIndexByField(fieldName), $targetTd = $targetTR.find(".e-rowcell").eq(columnIndex), column = this.model.columns[columnIndex], rowData = this.getDataByIndex(index);
            var args = {
                validationRules: ej.isNullOrUndefined(column.validationRules) ? {} : $.extend(true, {}, column.validationRules),
                columnName: column.field,
                value: rowData[fieldName],
                rowData: rowData,
                primaryKey: this._primaryKeys,
                columnObject: column,
                cell: $targetTd,
                isForeignKey: !ej.isNullOrUndefined(column.foreignKeyValue) && this.model.editSettings.editMode == "batch" ? true : false,
            }, isEditable = true;
            if (this._trigger("cellEdit", args))
                return;
			if($targetTR.hasClass("e-insertedrow")) args.requestType="add";		
            if ($.inArray(fieldName, this._primaryKeys) != -1 || args.columnObject.allowEditing === false || args.columnObject.template || args.columnObject.isUnbound) {
                $.extend(this._bulkEditCellDetails, {
                    cellValue: args.value,
                    rowIndex: index,
                    fieldName: fieldName,
                    rowData: args.rowData,
                    columnIndex: columnIndex,
                    isForeignKey: ej.isNullOrUndefined(args.columnObject.foreignKeyValue) ? false : true
                });
                isEditable = false;
            }
            if ($targetTR.hasClass("e-insertedrow") && (!args.columnObject.template && !args.columnObject.isUnbound))
                isEditable = true;
            if (isEditable) {
                $.extend(this._bulkEditCellDetails, {
                    rowIndex: index,
                    cellValue: args.value,
                    columnIndex: columnIndex,
                    fieldName: fieldName,
                    cellEditType: args.columnObject.editType,
                    rowData: rowData,
                    isForeignKey: ej.isNullOrUndefined(args.columnObject.foreignKeyValue) ? false : true
                });
                this._renderBulkEditObject(args, $targetTd);
                $targetTR.addClass("e-editedrow");
                args.cell.addClass("e-editedbatchcell");
            }
        },
        _findNextEditableCell: function (columnIndex) {
            var endIndex = this.model.columns.length;
            for (var i = columnIndex; i < endIndex; i++) {
                if (!this.model.columns[i].template && !this.model.columns[i].isUnbound)
                    return i;
            }
            return -1;
        },
        _findNextCell: function (columnIndex, direction) {
            var splittedColumn, visibleColumns = [], predicate, rows = this.getRows();
            predicate = ej.Predicate("visible", "equal", true).and("template", "notequal", true);
            if (!$(rows[this._bulkEditCellDetails.rowIndex]).hasClass("e-insertedrow"))
                predicate = predicate.and("allowEditing", "notequal", false);
            splittedColumn = direction == "right" ? this.model.columns.slice(columnIndex) : this.model.columns.slice(0, columnIndex + 1).reverse();
            visibleColumns = ej.DataManager(splittedColumn).executeLocal(ej.Query().where(predicate));
            if (visibleColumns.length == 0 && (!(direction == "left" && this._bulkEditCellDetails.rowIndex == 0) && !(direction == "right" && this._bulkEditCellDetails.rowIndex + 1 == this.getRows().length))) {
                splittedColumn = direction == "right" ? this.model.columns.slice(0, columnIndex) : this.model.columns.slice(columnIndex).reverse();
                visibleColumns = ej.DataManager(splittedColumn).executeLocal(ej.Query().where(predicate));
                this._bulkEditCellDetails.rowIndex = visibleColumns.length && direction == "right" ? this._bulkEditCellDetails.rowIndex + 1 : this._bulkEditCellDetails.rowIndex - 1;
            }
            return visibleColumns.length ? $.inArray(visibleColumns[0], this.model.columns) : -1;
        },
        _moveCurrentCell: function (direction) {
            var editCellIndex, rowIndex = this._bulkEditCellDetails.rowIndex, currentRow, $form = $("#" + this._id + "EditForm");
            if (this._bulkEditCellDetails.rowIndex == -1 && this._bulkEditCellDetails.columnIndex == -1)
                return true;
            switch (direction) {
                case "right":
                    if ((this._bulkEditCellDetails.rowIndex == this.getRows().length - 1 && this._bulkEditCellDetails.columnIndex == this.model.columns.length - 1) || (!this.element.is(document.activeElement) && $form.length == 0))
                        return true;
                    if (this._bulkEditCellDetails.columnIndex == this.model.columns.length - 1) {
                        editCellIndex = 0;
                        this._bulkEditCellDetails.rowIndex = this._bulkEditCellDetails.rowIndex + 1;
                    }
                    else
                        editCellIndex = this._bulkEditCellDetails.columnIndex + 1;
                    if (this.model.columns[editCellIndex].template === true || this.model.columns[editCellIndex].visible === false || this.model.columns[editCellIndex].allowEditing === false)
                        editCellIndex = this._findNextCell(editCellIndex, direction);
                    this._bulkEditCellDetails.rowIndex != rowIndex && this.selectRows(this._bulkEditCellDetails.rowIndex);
                    editCellIndex != -1 && this.editCell(this._bulkEditCellDetails.rowIndex, this.model.columns[editCellIndex].field);
                    break;
                case "left":
                    if ((this._bulkEditCellDetails.rowIndex == 0 && this._bulkEditCellDetails.columnIndex == 0) || (!this.element.is(document.activeElement) && $form.length == 0))
                        return true;
                    if (this._bulkEditCellDetails.columnIndex == 0) {
                        editCellIndex = this.model.columns.length - 1;
                        this._bulkEditCellDetails.rowIndex = this._bulkEditCellDetails.rowIndex - 1;
                        this.selectRows(this._bulkEditCellDetails.rowIndex);
                    }
                    else
                        editCellIndex = this._bulkEditCellDetails.columnIndex - 1;
                    if (this.model.columns[editCellIndex].template === true || this.model.columns[editCellIndex].visible === false || this.model.columns[editCellIndex].allowEditing === false)
                        editCellIndex = this._findNextCell(editCellIndex, direction);
                    this._bulkEditCellDetails.rowIndex != rowIndex && this.selectRows(this._bulkEditCellDetails.rowIndex);
                    editCellIndex != -1 && this.editCell(this._bulkEditCellDetails.rowIndex, this.model.columns[editCellIndex].field);
                    break;
                case "up":
                    if (this._bulkEditCellDetails.rowIndex == 0)
                        return;
                    editCellIndex = this._bulkEditCellDetails.columnIndex;
                    this.selectRows(this._bulkEditCellDetails.rowIndex - 1);
                    this.editCell(this._bulkEditCellDetails.rowIndex - 1, this.model.columns[this._bulkEditCellDetails.columnIndex].field);
                    break;
                case "down":
                    if (this._bulkEditCellDetails.rowIndex == this.getRows().length - 1)
                        return;
                    editCellIndex = this._bulkEditCellDetails.columnIndex;
                    this.selectRows(this._bulkEditCellDetails.rowIndex + 1);
                    this.editCell(this._bulkEditCellDetails.rowIndex + 1, this.model.columns[this._bulkEditCellDetails.columnIndex].field);
                    break;

            }
            addedRow = !$(this.getRows()[this._bulkEditCellDetails.rowIndex]).hasClass("e-insertedrow");
            if (editCellIndex != -1 && (this.model.columns[editCellIndex].isUnbound || (this.model.columns[editCellIndex].isPrimaryKey && addedRow) || this.model.columns[editCellIndex].template))
                this.element.focus();
            return false;
        },
        _renderBulkEditObject: function (cellEditArgs, $td) {
            var $form = ej.buildTag("form", "", {}, { id: this._id + "EditForm" }), $bulkEditTemplate = this._bulkEditTemplate, mappingName = this._id + cellEditArgs.columnObject.field, $element, htmlString, cellData = {};
            cellData[cellEditArgs.columnObject.field] = cellEditArgs.value; var args={requestType:cellEditArgs.requestType};
            $td.empty();
            htmlString = $bulkEditTemplate.find("#" + cellEditArgs.columnObject.field + "_BulkEdit").html();
            $element = $($.templates(htmlString).render(cellData));
            if ($element.get(0).tagName == "SELECT") {
                $element.val(cellData[cellEditArgs.columnObject.field]);
                $element.val() == null && $element.val($element.find("option").first().val());
            }
            $form.append($element);
            $td.append($form);
            this._setoffsetWidth();
            this._refreshEditForm(args);
            if ($.isFunction($.validator) && !$.isEmptyObject(cellEditArgs.validationRules)) {
                this._initValidator();
                this.setValidationToField(cellEditArgs.columnObject.field, cellEditArgs.validationRules);
            }
            this.model.isEdit = true;
        },
        _triggerConfirm: function (args) {
            if (args !== undefined && document.activeElement.value == this._getLocalizedLabels("OkButton")) {
                if (this._confirmDialog.find(".e-content").text() == this._getLocalizedLabels("BatchSaveConfirm"))
                    this.batchSave();
                else {
                    this._confirmedValue = true;
                    this._processBindings(this._requestArgs);
                }
            }
            else {
                if (this._confirmDialog.find(".e-content").text() != this._getLocalizedLabels("BatchSaveConfirm")) {
                    switch (this._requestArgs.requestType) {
                        case "grouping":
                            this.model.groupSettings.groupedColumns.pop();
                            break;
                        case "ungrouping":
                            this.model.groupSettings.groupedColumns.push(this._requestArgs.columnName);
                            break;
                        case "sorting":
                            this._cSortedDirection = this._cSortedColumn = null;
                            break
                        case "filtering":
                            this.model.filterSettings.filteredColumns.reverse().splice(0, this._requestArgs.currentFilterObject);
                            this.model.filterSettings.filteredColumns.reverse();
                            break;
                        case "paging":
                            this._currentPage(this._requestArgs.previousPage);
                            this.getPager().ejPager("model.currentPage", this._requestArgs.previousPage);
                            break

                    }
                }
                this._confirmedValue = false;
            }
            this._requestArgs = null;
            this._confirmDialog.ejDialog("close");
        },
        _saveCellHandler: function (e) {
            var $target = $(e.target);
            e.stopPropagation();
            if ($target.closest(".e-popup").length == 0 && $target.closest(".e-rowcell").find("#" + this._id + "EditForm").length == 0)
                this.saveCell();
        },
        _initValidator: function () {
            var gridObject = this, elements = this.model.scrollSettings.frozenColumns > 0 ? this.element.find(".gridform") : $("#" + this._id + "EditForm");
            for (var i = 0; i < elements.length ; i++) {
                elements.eq(i).validate({
                    errorClass: 'e-field-validation-error',
                    errorElement: 'div',
                    wrapper: "div",
                    errorPlacement: function (error, element) {
                        var $td = element.closest("td"), $container = $(error).addClass("e-error")
                        , $tail = ej.buildTag("div.e-errortail e-toparrow"), $container;
                        $td.find(".e-error").remove();
                        if (element.parent().hasClass("e-in-wrap"))
                            $container.insertAfter(element.closest(".e-widget"));
                        else
                            $container.insertAfter(element);
                        $container.prepend($tail)
                        gridObject.model.editSettings.editMode != "dialog" && $container.offset({ left: element.offset().left, top: element.offset().top + element.height() });     
                        $container.fadeIn("slow");
                    },

                });
            }
        },
        setValidation: function () {
            for (var i = 0; i < this.model.columns.length; i++) {
                if (!ej.isNullOrUndefined(this.model.columns[i]["validationRules"])) {
                    this.setValidationToField(this.model.columns[i].field, this.model.columns[i].validationRules);
                }
            }
        },
        /**
		 * Set validation to a field during editing.
         * @alias ejGrid#setValidationToField
		 * @return jQuery
		 * @param {string} fieldName Specify the field name of the column to set validation rules
		 * @param {object} rules Specify the validation rules for the field
		 * @example 
		 * &lt;script&gt;
		 * // Create grid object.
		 * var gridObj = $("#Grid").data("ejGrid");
		 * // It is used to set validation to a field during editing
		 * gridObj.setValidationToField("OrderID", { required: true }); 
		 * &lt;/script&gt;
		 * @example 
		 * &lt;script&gt;
		 * // It is used to set validation to a field during editing
		 * $("#Grid").ejGrid("setValidationToField", "OrderID", { required: true });
		 * &lt;/script&gt;
         */
        setValidationToField: function (name, rules) {
            var ele = $("#" + this._id + name).length > 0 ? $("#" + this._id + name) : $("#" + name);
            ele.rules("add", rules);
            var validator = $("#" + this._id + "EditForm").validate();
            validator.settings.messages[name] = {};
            if (!ej.isNullOrUndefined(rules["required"])) {
                if (!ej.isNullOrUndefined(rules["messages"] && rules["messages"]["required"]))
                    var message = rules["messages"]["required"];
                else
                    var message = $.validator.messages.required;
                if (message.indexOf("This field") == 0)
                    message = message.replace("This field", this.getColumnByField(name).headerText);
                else
                    message = this.getColumnByField(name).headerText + " " + message;
                validator.settings.messages[name]["required"] = message;
                if (ele.hasClass("e-datepicker"))
                    ele.ejDatePicker("instance")._removeWatermark()
            }
        },
        _renderConfirmDialog: function () {
            var $contentDiv = ej.buildTag('div.e-content', this._getLocalizedLabels("BatchSaveConfirm"))
            , $buttons = ej.buildTag('span.e-buttons', "<input type='button' id=" + this._id + 'ConfirmDialogOK' + " value='" + this._getLocalizedLabels("OkButton") + "' /> "
                + "<input type='button' id=" + this._id + 'ConfirmDialogCancel' + " value='" + this._getLocalizedLabels("CancelButton") + "' />");

            this._confirmDialog = ej.buildTag('div#' + this._id + 'ConfirmDialog');
            this._confirmDialog.append($contentDiv).append($buttons);
            this.element.append(this._confirmDialog);
            $buttons.find("input").ejButton({
                cssClass: this.model.cssClass,
                showRoundedCorner: true,
                size: "mini",
                click: $.proxy(this._triggerConfirm, this)
            });
            this._renderFDialog(this._id + 'ConfirmDialog');
            this._confirmDialog.ejDialog({ width: "auto", enableModal: true });
        },
        _unboundClickHandler: function (e) {
            var $target = $(e.target);
            var $editTrLen = 0;
            if ($target.hasClass("e-unboundcelldiv"))
                return;
            if ($target.hasClass("e-cancelbutton"))
                this.model.isEdit = false;
            if (this.model.editSettings.editMode == "inlineform" || this.model.editSettings.editMode == "inlinetemplate")
                $editTrLen = $("#" + this._id).find(".e-editedrow").length;
            this.model.allowSelection && this.selectRows(this.getIndexByRow($target.closest("tr")) - $editTrLen);
            $.isFunction($.fn.ejDatePicker) && $("#" + this._id + "EditForm").find(".e-datepicker").ejDatePicker("hide");
            if ($target.hasClass("e-editbutton")) {
                var index = this.getIndexByRow($target.closest("tr"));
                if (this.model.isEdit)
                    this.cancelEdit();
                var $tr = this.getRowByIndex(index);
                this.startEdit($tr);
            } else if ($target.hasClass("e-deletebutton")) {
                var index = this.getIndexByRow($target.closest("tr"));
                var $tr = this.getRowByIndex(index);
                this._deleteRow($tr);
            }
            else if ($target.hasClass("e-savebutton"))
                this.endEdit();
            else if ($target.hasClass("e-cancelbutton"))
                this.cancelEdit();
        },
          /**
         * Add a new record in grid control even allowAdding is set as false.
         * @alias ejGrid# addRecord
         * @return jQuery
         * @example 
         * &lt;script&gt;
         * // Create grid object.
         * var gridObj = $("#Grid").data("ejGrid");
         * // Sends an add new record request to the grid
         * gridObj.addRecord(); 
         * &lt;/script&gt;
         * @example 
         * &lt;script&gt;
         * // add new record to the grid
         * $("#Editing").ejGrid("addRecord",{OrderID:12333})       
         * &lt;/script&gt;
         */
        addRecord: function (data,serverChange) {
            if (data) {
                if (!serverChange) {
                    this._dataManager.insert(data);
                    this.getContentTable().prepend($.render[this._id + "_JSONTemplate"](data));
                    if (this.model.allowPaging && this.getPager() != null) {
                        var model = this.getPager().ejPager("model");
                        model.totalRecordsCount++;
                        this.getRowByIndex(model.pageSize - 1).remove();
                        this.getPager().ejPager("refreshPager");
                    }
                } else {
                    var args = { data: data };
                    args.action = "add";
                    args.selectedRow = this._selectedRow();
                    this._cAddedRecord = data;
                    args.requestType = ej.Grid.Actions.Save;
                    this._updateAction(args);
                }
            } else
                this._startAdd();
        },
        /**
         * Update a edited record in grid control even allowEditing is set as false.
         * @alias ejGrid#updateRecord
         * @return jQuery
         * @param {string} fieldName Pass the primary key field Name of the column
         * @param {array} data Pass the edited json data of record need to be update.
         * @example 
         * &lt;script&gt;
         * // Create grid object.
         * var gridObj = $("#Grid").data("ejGrid");
         * // Sends a update record request to the grid
         * gridObj.updateRecord("OrderID", { OrderID: 10249, EmployeeID: 3 }); 
         * &lt;/script&gt;
         * @example 
         * &lt;script&gt;
         * // Sends a update record request to the grid
         * $("#Grid").ejGrid("updateRecord", "OrderID", { OrderID: 10249, EmployeeID: 3 });        
         * &lt;/script&gt;
         */
        updateRecord: function (keyField, data, action) {
            this._updateDeleteRecord(keyField, data, "update");
        },
        _updateDeleteRecord: function (keyField, data, action) {
            var dataMgr = ej.DataManager(this._currentJsonData), dataFilter, index, $row, $newrow;
            dataFilter = dataMgr.executeLocal(ej.Query().where(keyField, ej.FilterOperators.equal, data[keyField]));
            if (dataFilter.length) {
                index = $.inArray(dataFilter[0], this._currentJsonData);
                if (index != -1) {
                    $row = this.getRowByIndex(index);
                    if (action == "update") {
                        $newrow = $($.render[this._id + "_JSONTemplate"](data));
                        $row.replaceWith($newrow);
                        !$row.hasClass("e-alt_row") && $newrow.removeClass("e-alt_row");
                    }
                    else {
                        this._deleteRow($row);
                    }
                }
            }
            this._dataManager[action](keyField, data);
        },
        /**
         * Delete a record in grid control even allowDeleting is set as false
         * @alias ejGrid#deleteRecord
         * @return jQuery
         * @param {string} fieldName Pass the primary key field Name of the column
         * @param {array} data Pass the json data of record need to be delete.
         * @example 
         * &lt;script&gt;
         * // Create grid object.
         * var gridObj = $("#Grid").data("ejGrid");
         * // Sends a delete record request to the grid
         * gridObj.deleteRecord("OrderID", { OrderID: 10249, EmployeeID: 3 }); 
         * &lt;/script&gt;
         * @example 
         * &lt;script&gt;
         * // Sends a delete record request to the grid
         * $("#Grid").ejGrid("deleteRecord", "OrderID", { OrderID: 10249, EmployeeID: 3 });        
         * &lt;/script&gt;
         */
        deleteRecord: function (keyField, data) {
            this._updateDeleteRecord(keyField, data, "remove");
        },
    };
})(jQuery, Syncfusion);;
(function ($, ej, undefined) {
    ej.gridFeatures = ej.gridFeatures || {};
    ej.gridFeatures.sort = {
        _addSortElementToColumn: function (field, direction) {
            var column = this.getColumnByField(field), imageDirection;
            var index = $.inArray(column, this.model.columns);
            var $headerCellDiv = this.getHeaderTable().find("thead tr:not('.e-stackedHeaderRow')").find(".e-headercell").not(".e-detailheadercell").eq(index).find(".e-headercelldiv");
            $headerCellDiv.find(".e-ascending,.e-descending").remove();
            imageDirection = direction != "descending" ? "e-rarrowup-2x" : "e-rarrowdown-2x";
            $headerCellDiv.append(this._createSortElement().addClass("e-" + (direction || "ascending") + " " + imageDirection));
            $headerCellDiv.parent().attr("aria-sort", direction);
        },
        _removeSortElementFromColumn: function (field) {
            var column = this.getColumnByField(field);
            var index = $.inArray(column, this.model.columns);
            var $headerCellDiv = this.getHeaderTable().find("thead").find(".e-headercell").not(".e-detailheadercell").eq(index).find(".e-headercelldiv");
            $headerCellDiv.find(".e-ascending,.e-descending").remove();
            $headerCellDiv.parent().removeAttr("aria-sort");
        },
        _sortCompleteAction: function (args) {
            var imageDirection;
            this.getHeaderTable().find(".e-columnheader").find(".e-headercelldiv")
                    .find(".e-ascending,.e-descending").remove();
            this.getHeaderTable().find("[aria-sort]").removeAttr("aria-sort");
            for (var i = 0; i < this.model.sortSettings.sortedColumns.length; i++)
                this._addSortElementToColumn(this.model.sortSettings.sortedColumns[i].field, this.model.sortSettings.sortedColumns[i].direction);
            if (this.model.groupSettings.groupedColumns.length && this._$curSElementTarget != null) {
                var $element = this._checkEinGroupDrop($.trim(this._$curSElementTarget.text()));
                if (!ej.isNullOrUndefined($element)) {
                    imageDirection = args.columnSortDirection == "ascending" ? "e-rarrowup-2x" : "e-rarrowdown-2x";
                    $element.find(".e-ascending,.e-descending").removeClass().addClass("e-icon e-" + args.columnSortDirection + " " + imageDirection);
                }
            }
            this.multiSortRequest = false;
        },
        /**
         * Remove a column or collection of columns from a sorted column collections in grid.
         * @alias ejGrid#removeSortedColumns
         * @return jQuery
         * @param {array/string} fieldName Pass array of field names of the columns to remove a collection of sorted columns or pass a string of field name to remove a column from sorted column collections
         * @example 
         * &lt;script&gt;
         * // Create grid object.
         * var gridObj = $("#Grid").data("ejGrid");
         * // Removes a column from sorted column collections
         * gridObj.removeSortedColumns("OrderID"); 
         * // Removes specified collection of columns from sorted column collections
         * gridObj.removeSortedColumns(["CustomerID", "ShipCity"]); 
         * &lt;/script&gt;
         * @example 
         * &lt;script&gt;
         * // Removes a column from sorted column collections
         * $("#Grid").ejGrid("removeSortedColumns", "OrderID");        
         * // Removes specified collection of columns from sorted column collections
         * $("#Grid").ejGrid("removeSortedColumns", ["CustomerID", "ShipCity"]);        
         * &lt;/script&gt;
         */
        removeSortedColumns: function (fieldName) {
            if ($.isArray(fieldName)) {
                for (var i = 0; i < fieldName.length; i++) {
                    this._removeSortedColumnFromCollection(fieldName[i]);
                }
            }
            else
                this._removeSortedColumnFromCollection(fieldName);
            this.multiSortRequest = true;
            this.sortColumn(null, null);
        },
        _removeSortedColumnFromCollection: function (fieldName) {
            for (var i = 0; i < this.model.sortSettings.sortedColumns.length; i++) {
                if (this.model.sortSettings.sortedColumns[i].field == fieldName) {
                    this.model.sortSettings.sortedColumns.splice(i, 1);
                    break;
                }
            }
        },
        /**
		 * Clear the sorting from columns in the grid
         * @alias ejGrid#clearSorting
		 * @return jQuery
		 * @example 
		 * &lt;script&gt;
		 * // Create grid object.
		 * var gridObj = $("#Grid").data("ejGrid");
         * // Clears the sorting from columns in the grid
		 * gridObj.clearSorting(); 
		 * &lt;/script&gt;
		 * @example 
		 * &lt;script&gt;
		 * // Clears the sorting from columns in the grid
		 * $("#Grid").ejGrid("clearSorting");        
		 * &lt;/script&gt;
         */
        clearSorting: function () {
            var proxy = this;
            this.model.sortSettings.sortedColumns = $.grep(this.model.sortSettings.sortedColumns, function (value, index) {
                if ($.inArray(value.field, proxy.model.groupSettings.groupedColumns) != -1)
                    return true;
                return false;
            });
            this._$prevSElementTarget = null;
            this._$curSElementTarget = null;
            this.refreshContent();
        },
        /**
         * Send a sorting request in grid.
         * @alias ejGrid#sortColumn
         * @return jQuery
         * @param {string} columnName Pass the field name of the column as columnName for which sorting have to be performed
         * @param {string} [sortingDirection] Pass the sort direction ascending/descending by which the column have to be sort. By default it is sorting in an ascending order
         * @example 
         * &lt;script&gt;
         * // Create grid object.
         * var gridObj = $("#Grid").data("ejGrid");
         * gridObj.sortColumn("OrderID", "ascending"); // Sends a sorting request to the grid with specified columnName and sortDirection
         * &lt;/script&gt;
         * @example 
         * &lt;script&gt;
         * // Sends a sorting request to the grid with specified columnName and sortDirection
         * $("#Grid").ejGrid("sortColumn", "OrderID", "ascending");        
         * &lt;/script&gt;
         */
        sortColumn: function (columnName, columnSortDirection) {
            if (!this.model.allowSorting || $.inArray(columnName, this._disabledSortableColumns) != -1 || (columnName != null && columnName.length == 0))
                return;
            var args = {};
            if (!this.multiSortRequest) {
                var proxy = this;
                this.model.sortSettings.sortedColumns = $.grep(this.model.sortSettings.sortedColumns, function (value, index) {
                    if ($.inArray(value.field, proxy.model.groupSettings.groupedColumns) != -1)
                        return true;
                    return false;
                });
            }
            args.requestType = ej.Grid.Actions.Sorting;
            this._cSortedColumn = args.columnName = columnName;
            this._cSortedDirection = args.columnSortDirection = columnSortDirection === undefined ? ej.sortOrder.Ascending : columnSortDirection;
            if (this._cSortedColumn !== null) {
                this._removeSortedColumnFromCollection(columnName);
                this.model.sortSettings.sortedColumns.push({ field: this._cSortedColumn, direction: this._cSortedDirection });
            }
            var returnValue = this._processBindings(args);
            if (returnValue)
                this._cSortedDirection = this._cSortedColumn = null;
            this._primaryKeyValues = [];
        },
        _createSortElement: function () {
            return ej.buildTag('span.e-icon', "&nbsp;");
        },
        _renderMultiTouchDialog: function () {
            this._customPop = ej.buildTag("div.e-gridpopup", "", { display: "none" });
            var $content = ej.buildTag("div.e-content"), $downTail = ej.buildTag("div.e-downtail e-tail");
            if (this.model.allowMultiSorting) {
                var $selElement = ej.buildTag("span.e-sortdirect e-icon");
                $content.append($selElement);
            }
            if (this.model.selectionType == "multiple") {
                var $selElement = ej.buildTag("span.e-rowselect e-icon");
                $content.append($selElement);
            }
            this._customPop.append($content);
            this._customPop.append($downTail);
            this.element.append(this._customPop);
        },

    };
})(jQuery, Syncfusion);;
(function ($, ej, undefined) {
    ej.gridFeatures = ej.gridFeatures || {};
    ej.gridFeatures.filter = {
        /**
		 * Get the filter bar of grid
         * @alias ejGrid#getFilterBar
		 * @return gridFilterBar
		 * @example 
		 * &lt;script&gt;
		 * // Create grid object.
		 * var gridObj = $("#Grid").data("ejGrid");
         * // Gets filter bar of grid control
		 * gridObj.getFilterBar(); 
		 * &lt;/script&gt;
		 * @example 
		 * &lt;script&gt;
		 * // Gets filter bar of grid control
		 * $("#Grid").ejGrid("getFilterBar");        
		 * &lt;/script&gt;
         */
        getFilterBar: function () {
            return this._gridFilterBar;
        },

        setGridFilterBar: function (value) {
            this._gridFilterBar = value;
        },
        /**
		 * Send a filtering request to grid.
         * @alias ejGrid#filterColumn
		 * @return jQuery
		 * @param {string} fieldName Pass the field name of the column 
		 * @param {string} filterOperator string/integer/dateTime operator
		 * @param {string/number} filterValue Pass the value to be filtered in a column
		 * @param {string} predicate Pass the predicate as and/or
		 * @param {boolean} [matchcase] Pass the match case valueas true/false
		 * @example 
		 * &lt;script&gt;
		 * // Create grid object.
		 * var gridObj = $("#Grid").data("ejGrid");
         * // Sends a filtering request to the grid
		 * gridObj.filterColumn("OrderID","equal","10248","and", true);
		 * &lt;/script&gt;
         * @example
		 * &lt;script&gt;
		 * // Sends a filtering request to the grid
		 * $("#Grid").ejGrid("filterColumn","OrderID","equal","10248","and", true);
		 * &lt;/script&gt;
         */
        filterColumn: function (fieldName, filterOperator, filterValue, predicate, matchcase, actualFilterValue) {
            if (!this.model.allowFiltering)
                return;
            var args = {};
            args.requestType = ej.Grid.Actions.Filtering;
            args.currentFilterObject = [];
            if (!$.isArray(filterOperator))
                filterOperator = $.makeArray(filterOperator);
            if (!$.isArray(filterValue))
                filterValue = $.makeArray(filterValue);
            var firstLoop = false;
            var filterCol = this._filterCollection;
            if (ej.util.isNullOrUndefined(this._currentFilterColumn))
                this._currentFilterColumn = this.getColumnByField(fieldName);
            for (var index = 0; index < filterOperator.length; index++) {
                var filterObject = {
                    field: fieldName,
                    operator: filterOperator[index],
                    value: filterValue[index],
                    matchcase: matchcase,
                    predicate: predicate,
                    actualFilterValue: actualFilterValue
                };
                if (this.model.filterSettings.filterType == "filterbar")
                    this._$colType = this._currentFilterColumn.type;
                if (this.model.filterSettings.filteredColumns.length == 0 && filterObject.value !== "") {
                    if (this._$colType == "date" && (filterOperator == "equal" || filterOperator == "notequal") && typeof (filterObject.value) !== "string")
                        this._setDateFilters(filterObject);
                    else
                        this.model.filterSettings.filteredColumns.push(filterObject);
                } else {
                    var proxy = this;
                    if (!firstLoop) {
                        var dataManger = ej.DataManager(this.model.filterSettings.filteredColumns);
                        var query = new ej.Query().where("field", ej.FilterOperators.equal, filterObject.field);
                        var object = dataManger.executeLocal(query);
                        for (var i = 0; i < object.length; i++) {
                            var objectIndex = $.inArray(object[i], this.model.filterSettings.filteredColumns)
                            if (objectIndex != -1)
                                this.model.filterSettings.filteredColumns.splice(objectIndex, 1);
                        }
                    }
                    if (filterObject.value !== "") {
                        if (this._$colType == "date" && (filterOperator == "equal" || filterOperator == "notequal") && typeof (filterObject.value) !== "string")
                            this._setDateFilters(filterObject);
                        else
                            this.model.filterSettings.filteredColumns.push(filterObject);
                    }
                }
                firstLoop = true;
                args.currentFilterObject.push(filterObject);
            }
            args.filterCollection = this.model.filterSettings.filteredColumns;
            args.currentFilteringColumn = fieldName;
            var returnValue = this._processBindings(args);
            if (returnValue) {
                this.model.filterSettings.filteredColumns.reverse().splice(0, filterOperator.length);
                this.model.filterSettings.filteredColumns.reverse();
            }

        },
        /**
         * Send a search request to grid with specified string passed in it
         * @alias ejGrid#search
         * @return jQuery
         * @param {string} searchString Pass the string to search in Grid records
         * @example 
         * &lt;script&gt;
         * // Create grid object.
         * var gridObj = $("#Grid").data("ejGrid");
         * // Sends a search request to the grid
         * gridObj.search("France"); 
         * &lt;/script&gt;
         * @example
         * &lt;script&gt;
         * // Sends a search request to the grid
         * $("#Grid").ejGrid("search", "France");        
         * &lt;/script&gt;
         */
        search: function (searchString) {
            var args = {};
            args.requestType = ej.Grid.Actions.Search;
            args.keyValue = searchString;
            this._searchString = searchString;
            this._processBindings(args);
            this._primaryKeyValues = [];
        },
        _filterBarHandler: function (e) {
            var keycode = e.keyCode;
            if ((this.model.filterSettings["filterBarMode"] == "immediate" || keycode == 13) && keycode != 9) {
                var $target = $(e.target);
                this.filterStatusMsg = "";
                var fieldName = $target.prop("id").replace("_filterBarcell", "");
                var column = this.getColumnByField(fieldName);
                if (column == null)
                    return;
                this._currentFilterColumn = column;
                if (this._currentFilterColumn != this._oldFilterColumn)
                    this.filterValueOldLength = 0;
                this._currentFilterbarValue = $target.val();
                this.filterValueCurrentLength = this._currentFilterbarValue.length;
                if (((this.filterValueCurrentLength == 0 && this.filterValueOldLength == 0) || this._currentFilterbarValue == this.OldfilterValue) && this._currentFilterColumn == this._oldFilterColumn) {
                    this._showFilterMsg();
                    return;
                }
                this._skipFilterProcess = this._checkForSkipInput();
                if (!this._skipFilterProcess) {
                    this._processFilter();
                } else {
                    if (this._currentFilterColumn.type == "string") {
                        this.filterStatusMsg = "Invalid Filter Data";
                        this._showFilterMsg();
                    } else {
                        this._skipFilterProcess = false;
                        this._showFilterMsg();
                        return;
                    }
                }
            }
        },
        _renderExcelFilter: function () {
            var model = {
                instance:this,
                showSortOptions: this.model.allowSorting,
                allowFormatFiltering: this.model.filterSettings.allowFormatFiltering,
                allowCaseSensitive: this.model.filterSettings.enableCaseSensitivity,
                maxFilterLimit: this.model.filterSettings.maxFilterChoices,
                filterHandler: ej.proxy(this._filterHandler,this)
            };
            this._excelFilter = new ej.excelFilter(model);
        },
        _filterHandler: function (args) {
            var arg = {},fQMgr;
            arg.requestType = args.action == "sorting" ? args.action : "filtering";
            var temp = this.model.filterSettings.filteredColumns;
            if (args.action == "filtering") {
                fQMgr = ej.DataManager(this.model.filterSettings.filteredColumns);
                var query = new ej.Query().where("field", ej.FilterOperators.equal, args.fieldName);
                var object = fQMgr.executeLocal(query);
                for (var i = 0; i < object.length; i++) {
                    var objectIndex = $.inArray(object[i], this.model.filterSettings.filteredColumns)
                    if (objectIndex != -1)
                        this.model.filterSettings.filteredColumns.splice(objectIndex, 1);
                }
                ej.merge(this.model.filterSettings.filteredColumns, args.filterCollection);
                args.currentFilterCollection = args.filterCollection;   
            }
            else if (args.action == "clearfiltering") {
                var filterObj = args.filterDetails;
                delete this._excelFilter._predicates[0][args.fieldName];
                this.filterColumn(filterObj.field, filterObj.operator, filterObj.value, filterObj.predicate);
                return;
            }
            else if (args.action == "sorting") {
                var sortObj = args.sortDetails;
                if (ej.gridFeatures.sort)
                    this.sortColumn(sortObj.field, sortObj.direction);
                this._excelFilter.closeXFDialog();
                return;
            }
            
            arg.currentFilteringColumn = args.fieldName;           
            arg.predicated = args.ejpredicate;
            var returnValue = this._processBindings(arg);
            if (returnValue)
                this.model.filterSettings.filteredColumns = temp;
        },
        _renderFiltering: function () {
            var $headerTable = this.getHeaderTable();
            var $tr = ej.buildTag('tr.e-filterbar'), $trClone;
            if (this.model.detailsTemplate) $tr.append(ej.buildTag('th.e-filterbarcell e-mastercell'));
            for (var column = 0; column < this.model.columns.length; column++) {
                var $input = ej.buildTag('input.e-ejinputtext e-filtertext', "", {}, { title: this.model.columns[column]["field"] + "'s filterbar cell", type: "text", id: this.model.columns[column]["field"] + "_filterBarcell" });
                var $th = ej.buildTag('th.e-filterbarcell'), $div = ej.buildTag('div.e-filterdiv'), $span = ej.buildTag('span.e-cancel e-icon e-hide');
                if (this.model.columns[column]["allowFiltering"] === false) {
                    $input.attr("disabled", true).addClass("e-disable");
                    this._disabledFilterableColumns.push(this.model.columns[column]["headerText"]);
                }
                this.model.columns[column]["visible"] === false && $th.addClass("e-hide");
                $div.append($input).append($span);
                $tr.append($th.append($div));
                if (column == this.model.scrollSettings.frozenColumns - 1) {
                    $trClone = $tr.clone();
                    $headerTable.find("thead").first().append($trClone);
                    $tr.empty();
                }
            }
            $headerTable.find("thead").last().append($tr);
            this.setGridFilterBar($tr);
        },
        _closeFilterDlg: function () {
            $("#" + this._id + "_" + this._$colType + "Dlg").ejDialog("close");
            this._$fDlgIsOpen = false;
        },
        _filterBarClose: function (e) {
            var $target = $(e.target);
            if (e.type == "click" && $target.hasClass("e-cancel")) {
                var $targetText = $target.prev();
                $targetText.focus().val("");
                $targetText.trigger("keyup");
                e.stopPropagation();
            }
            if (e.type == "focusin" && $target.hasClass("e-filtertext")) {
                $target = $(e.target).next();
                this.getFilterBar().find(".e-cancel").addClass("e-hide");
                $target.removeClass("e-hide");
            }
        },
        _processFilter: function () {
            if (!this._alreadyFilterProcessed) {
                this._alreadyFilterProcessed = true;
                this._startTimer();
            } else {
                this._stopTimer();
                this._startTimer();
            }
        },
        _startTimer: function () {
            var proxy = this;
            this._timer = window.setTimeout(
                function () {
                    proxy._onTimerTick();
                },
                1500);
        },
        _stopTimer: function () {
            this._oldFilterColumn = this._currentFilterColumn;
            this.filterValueOldLength = this.filterValueCurrentLength;
            if (this._timer != null)
                window.clearTimeout(this._timer);
        },

        _onTimerTick: function () {
            this.OldfilterValue = this._currentFilterbarValue;
            this._findPredicate();
            var result = null;
            var matchcase = this._currentFilterColumn.type == "string" ? false : true;
            if (this._currentFilterColumn.foreignKeyValue && this._currentFilterColumn.dataSource && this._currentFilterbarValue != "")
                result = this._fltrForeignKeyValue(this._operator, this._currentFilterbarValue, matchcase,
                         this._currentFilterColumn.dataSource, this._currentFilterColumn.foreignKeyField,
                         this._currentFilterColumn.foreignKeyValue, this._currentFilterColumn.type);
            if (!this._skipFilterProcess) {
                if (ej.isNullOrUndefined(result))
                    this.filterColumn(this._currentFilterColumn.field, this._operator, this._currentFilterbarValue, this._predicate, matchcase);
                else
                    this.filterColumn(this._currentFilterColumn.field, result.operator, result.value, result.value.length > 1 ? "or" : this._predicate, matchcase, result.filterValue);
            }
            else
                this.filterStatusMsg = "Invalid Filter Data";
            this._showFilterMsg();
            this._stopTimer();
        },

        _findPredicate: function () {
            var _value = this._currentFilterbarValue.replace(/ && /i, " and ").replace(" || ", " or ");
            var _predicateFinder = _value.split(' ');
            this._predicate = "and";
            if (_predicateFinder.length != 0) {
                if ($.isFunction(ej.Predicate[_predicateFinder[1]])) {
                    this._skipFilterProcess = false;
                    this._predicate = _predicateFinder[1];
                    var valuesArray = _value.split(" " + _predicateFinder[1] + " ");
                    var tempOperator = [];
                    var filterValues = [];
                    for (var i = 0; i < valuesArray.length; i++) {
                        this._validateFilterValue(valuesArray[i]);
                        tempOperator.push(this._operator);
                        if (this._currentFilterColumn.type == "number")
                            filterValues.push(this._currentFilterbarValue);
                        else if (this._currentFilterColumn.type == "string")
                            filterValues.push(valuesArray[i]);
                    }
                    this._currentFilterbarValue = filterValues;
                    this._operator = tempOperator;
                } else
                    this._validateFilterValue($.trim(this._currentFilterbarValue));
            } else
                this._validateFilterValue($.trim(this._currentFilterbarValue));
        },

        _validateFilterValue: function (_value) {
            switch (this._currentFilterColumn.type) {
                case "number":
                    this._operator = ej.FilterOperators.equal;
                    var stringSkipInput = new Array(">", "<", "=", "!");
                    for (var i = 0; i < _value.length; i++) {
                        if (jQuery.inArray(_value[i], stringSkipInput) != -1) {
                            break;
                        }
                    }
                    if (i != _value.length) {
                        this._getOperator(_value.substring(i));
                        if (i != 0)
                            this._currentFilterbarValue = _value.substring(0, i);
                    }
                    if (this._currentFilterbarValue != "" && _value.length >= 1)
                        this._currentFilterbarValue = parseFloat(this._currentFilterbarValue);
                    else
                        this._currentFilterbarValue = _value.length > 1 ? parseFloat(_value) : _value;
                    break;
                case "date":
                    this._operator = ej.FilterOperators.equal;
                    this._getOperator(_value);
                    var _format;
                    if (this._currentFilterColumn.format == "")
                        _format = Globalize.culture().calendar.patterns.d; //System Date format
                    else
                        _format = this._currentFilterColumn.format.split(':')[1].replace('}', "");
                    if (this._currentFilterbarValue != "")
                        this._currentFilterbarValue = Globalize.parseDate(this._currentFilterbarValue, _format);
                    break;
                case "datetime":
                    this._operator = ej.FilterOperators.equal;
                    this._getOperator(_value);
                    var _format;
                    if (this._currentFilterColumn.format == "")
                        _format = Globalize.culture().calendar.patterns.f; //System DateTime format
                    else
                        _format = this._currentFilterColumn.format.split(':')[1].replace('}', "");
                    if (this._currentFilterbarValue != "")
                        this._currentFilterbarValue = Globalize.parseDate(this._currentFilterbarValue, _format);
                    break;
                case "string":
                    this._operator = ej.FilterOperators.startsWith;
                    break;
                case "boolean":
                    if (this._currentFilterbarValue.toLowerCase() == "true" || this._currentFilterbarValue == "1")
                        this._currentFilterbarValue = true;
                    else if (this._currentFilterbarValue.toLowerCase() == "false" || this._currentFilterbarValue == "0")
                        this._currentFilterbarValue = false;
                    this._operator = ej.FilterOperators.equal;
                    break;
                default:
                    this._operator = ej.FilterOperators.equal;
            }
        },
        _getOperator: function (_value) {
            if (_value.charAt(0) == "=") {
                this._operator = ej.FilterOperators.equal;
                this._currentFilterbarValue = _value.substring(1);
            }
            if (ej.data.operatorSymbols[_value.charAt(0)] !== undefined || ej.data.operatorSymbols[_value.slice(0, 2)] !== undefined) {
                this._operator = ej.data.operatorSymbols[_value.charAt(0)];
                this._currentFilterbarValue = _value.substring(1);
                if (this._operator === undefined) {
                    this._operator = ej.data.operatorSymbols[_value.slice(0, 2)];
                    this._currentFilterbarValue = _value.substring(2);
                }
            }
            if (this._operator == ej.FilterOperators.lessThan || this._operator == ej.FilterOperators.greaterThan) {
                if (this._currentFilterbarValue.charAt(0) == "=") {
                    this._operator = this._operator + "orequal";
                    this._currentFilterbarValue = this._currentFilterbarValue.substring(1);
                }
            }

        },

        _checkForSkipInput: function () {
            var isSkip = false;
            var skipInput = new Array("=", " ", "!");
            var context = this;
            if (this._currentFilterColumn.type == "number") {
                if (ej.data.operatorSymbols[this._currentFilterbarValue] !== undefined || $.inArray(this._currentFilterbarValue, skipInput) != -1)
                    isSkip = true;
            }
            if (this._currentFilterColumn.type == "string") {
                var stringSkipInput = new Array(">", "<", "=", "!");
                for (var i = 0; i < this._currentFilterbarValue.length; i++) {
                    if ($.inArray(this._currentFilterbarValue[i], stringSkipInput) != -1)
                        isSkip = true;
                }
            }
            return isSkip;
        },
        _showFilterMsg: function () {
            var index = $.inArray(this._currentFilterColumn, this.filterColumnCollection);
            if (this._currentFilterbarValue !== "" && index == -1)
                this.filterColumnCollection.push(this._currentFilterColumn);
            if (this._currentFilterbarValue == "" && index != -1) {
                this.filterColumnCollection.splice(index, 1);
            }
            if ((!this._skipFilterProcess || this.filterColumnCollection.length > 0) && this.filterStatusMsg != "Invalid Filter Data") {
                for (var index = 0; index < this.filterColumnCollection.length; index++) {
                    if (index > 0)
                        this.filterStatusMsg += " && ";
                    this.filterStatusMsg += this.filterColumnCollection[index].headerText + ": " + $("#" + this.filterColumnCollection[index].field + "_filterBarcell").val();
                }
            }

            if (this.model.allowPaging)
                this.getPager().ejPager("model.externalMessage", this.filterStatusMsg);
            else {
                this.$pagerStatusBarDiv.find("div").html(this.filterStatusMsg);
                if (this.filterStatusMsg.length)
                    this.$pagerStatusBarDiv.css("display", "block");
                else
                    this.$pagerStatusBarDiv.hide();
            }
            if (this.filterStatusMsg == "Invalid Filter Data") {
                index = $.inArray(this._currentFilterColumn, this.filterColumnCollection);
                this.filterColumnCollection.splice(index, 1);
            }
            this.filterStatusMsg = "";
        },
        _renderFilterDialogs: function () {
            var $strDlg, $numDlg, $boolDlg, $dateDlg, $datetimeDlg;

            $.each(this.model.columns, ej.proxy(function (indx, col) {
                if (col.type == "string" && !$strDlg) {
                    $strDlg = true;
                    !this._isExcelFilter ? this._renderStringDialog() : this._excelFilter.renderDialog("string");
                } else if (col.type == "number" && !$numDlg) {
                    $numDlg = true;
                    !this._isExcelFilter ? this._renderNumDialog() : this._excelFilter.renderDialog("number");
                } else if (col.type == "date" && !$dateDlg) {
                    $dateDlg = true;
                    !this._isExcelFilter ?  this._renderDateDialog(col.field) : this._excelFilter.renderDialog("date");
                } else if (col.type == "datetime" && !$datetimeDlg) {
                    $datetimeDlg = true;
                    this._renderDateTimeDialog();
                } else if (col.type == "boolean" && !$boolDlg) {
                    $boolDlg = true;
                    !this._isExcelFilter ? this._renderBoolDialog() : this._excelFilter.renderDialog("boolean");
                }
            }, this));
        },
        _renderStringDialog: function () {
            var $id = this._id + "_stringDlg";
            if ($("#" + $id).length > 0) return;
            var $content = ej.buildTag("div#" + $id + ".e-dlgcontainer");
            $content.appendTo("body");
            this._renderFDialog($id);
            this._renderDlgContent($content, "string");

        },
        _renderBoolDialog: function () {
            var $id = this._id + "_booleanDlg";
            if ($("#" + $id).length > 0) return;
            var $content = ej.buildTag("div#" + $id + ".e-dlgcontainer");
            $content.appendTo("body");
            this._renderFDialog($id);
            if (!this.model.filterSettings.showPredicate)
                $("#" + $id).ejDialog({ minHeight: 90 });
            else
                $("#" + $id).ejDialog({ minHeight: 136 });
            this._renderDlgContent($content, "boolean");
        },
        _renderNumDialog: function () {
            var $id = this._id + "_numberDlg";
            if ($("#" + $id).length > 0) return;
            var $content = ej.buildTag("div#" + $id + ".e-dlgcontainer");
            $content.appendTo("body");
            this._renderFDialog($id);
            this._renderDlgContent($content, "number");

        },
        _renderDateDialog: function (col) {
            var $id = this._id + "_dateDlg";
            if ($("#" + $id).length > 0) return;
            var $content = ej.buildTag("div#" + $id + ".e-dlgcontainer");
            $content.appendTo("body");
            this._renderFDialog($id);
            this._renderDlgContent($content, "date",col);

        },
        _renderDateTimeDialog: function () {
            var $id = this._id + "_datetimeDlg";
            if ($("#" + $id).length > 0) return;
            var $content = ej.buildTag("div#" + $id + ".e-dlgcontainer");
            $content.appendTo("body");
            this._renderFDialog($id);
            this._renderDlgContent($content, "datetime");
        },
        _renderFDialog: function (id) {
            $("#" + id).ejDialog({ showOnInit: false, "enableRTL": this.model.enableRTL, "cssClass": this.model.cssClass, "showHeader": false, width: 260, enableResize: false, allowKeyboardNavigation: false, content: "#" + this._id });
        },
        _renderDlgContent: function (content, type,col) {
            content.addClass("e-grid");
            var $predicate = ej.buildTag("div.e-predicate"), $operator = ej.buildTag("div.e-operator"), $value = ej.buildTag("div.e-value");
            var $strOp = this._getLocalizedLabels("StringMenuOptions");
            var $numOp = this._getLocalizedLabels("NumberMenuOptions");
            var $drdown = ej.buildTag("input#" + this._id + type + "_ddinput", {}, {}, { "type": "text" });
            var $drdownDiv = ej.buildTag("div#" + this._id + type + "_dropdown");
            var $drdownUl = ej.buildTag("ul");
            var $radio = ej.buildTag("input", {}, {}, { "type": "radio", "name": this._id + "_predicate" + type, "value": "or" });
            var $andRadio = ej.buildTag("input", {}, {}, { "type": "radio", "name": this._id + "_predicate" + type, "value": "and", "checked": "checked" });
            var $cbox;
            $predicate.append($andRadio)
                .append(ej.buildTag("span.e-caption").html(this._getLocalizedLabels("PredicateAnd")))
                .append($radio)
                .append(ej.buildTag("span.e-caption").html(this._getLocalizedLabels("PredicateOr")));
            !this.model.filterSettings.showPredicate && $predicate.hide();
            if (type == "string") {
                $cbox = ej.buildTag("input", {}, {}, { "type": "checkbox" });
                $predicate.append($cbox)
                    .append(ej.buildTag("span.e-caption").html(this._getLocalizedLabels("MatchCase")));
                $.each($strOp, function (indx, operator) {
                    $drdownUl.append(ej.buildTag("li", {}, {}, { "value": operator.value }).html(operator.text));
                });
            }
            if (type == "number" || type == "date" || type == "datetime") {
                $.each($numOp, function (indx, operator) {
                    $drdownUl.append(ej.buildTag("li", {}, {}, { "value": operator.value }).html(operator.text));
                });
            }
            if (type != "boolean") {
                $drdownDiv.append($drdownUl);
                $operator.append($drdown);
                $operator.append($drdownDiv);
            }
            var $tBox = ej.buildTag("input", {}, {}, { "type": "text" });
            var $tchkBox = ej.buildTag("input", {}, {}, { "type": "checkbox" });
            var $filter = ej.buildTag("input.e-filter", {}, {}, { "type": "button", "value": this._getLocalizedLabels("Filter") });
            var $clear = ej.buildTag("input.e-clear", {}, {}, { "type": "button", "value": this._getLocalizedLabels("Clear") });
            $value.append(ej.buildTag("span.e-caption").html(this._getLocalizedLabels("Filter") + " : "));
            content.append($predicate);
            if (type == "boolean")
                $value.append($tchkBox);
            else {
                $value.append(ej.buildTag("br")).append($tBox);
                content.append($operator);
            }
            content.append($value);
            content.append(ej.buildTag("div.e-dlgBtns").append($filter)
                .append($clear));
            if (type != "boolean")
                $drdown.ejDropDownList({"cssClass": this.model.cssClass, "enableRTL": this.model.enableRTL, "targetID": this._id + type + "_dropdown", width: "100%", height: "26px", selectedItemIndex: 0});
            $radio.ejRadioButton({ "cssClass": this.model.cssClass, "enableRTL": this.model.enableRTL });
            $andRadio.ejRadioButton({ "cssClass": this.model.cssClass, "enableRTL": this.model.enableRTL, checked: true });
            if ($cbox)
                $cbox.ejCheckBox({ "cssClass": this.model.cssClass, "enableRTL": this.model.enableRTL });
            content.css("display", "none");
            this._createButton("filter", $filter);
            this._createButton("clear", $clear);
            if (type == "number")
                $tBox.ejNumericTextbox({ "cssClass": this.model.cssClass, "enableRTL": this.model.enableRTL, "value": 0, showSpinButton: false, height: "26px", decimalPlaces: 2, width: "100%" });
            else if (type == "date") {
                $tBox.attr("id", this._id + col)
                $tBox.ejDatePicker({ "cssClass": this.model.cssClass, "enableRTL": this.model.enableRTL, width: "100%" });
            }
            else if (type == "datetime")
                $tBox.ejDateTimePicker({ "cssClass": this.model.cssClass, "enableRTL": this.model.enableRTL, width: "100%" });
            else if (type == "boolean")
                $tchkBox.ejCheckBox({ "cssClass": this.model.cssClass, "enableRTL": this.model.enableRTL });
            else if (type == "string") {
                $tBox.attr("id", this._id + "_acString");
                $tBox.ejAutocomplete({
                    "cssClass": this.model.cssClass, "enableRTL": this.model.enableRTL, "dataSource": this._dataSource(), width: "100%", height: 26, enableDistinct: true, focusIn: function (args) {
                        var $dropdown = this.element.closest(".e-dialog").find(".e-dropdownlist");
                        this.model.filterType = $dropdown.val();
                    }
                });
            }
        },
        _createButton: function (name, element) {
            var $func = name == "filter" ? ej.proxy(this._fltrBtnHandler, this) : ej.proxy(this._fltrClrHandler, this);
            element.ejButton({ "cssClass": this.model.cssClass, "enableRTL": this.model.enableRTL, "click": $func });
        },
        _getIdField: function () {
            var $key;
            $.each(this.model.columns, function (indx, col) {
                if (col.key) {
                    $key = col.field;
                    return false;
                }
            });
            return $key;
        },
        _filterCompleteAction: function () {
            if (this.model.allowPaging)
                this._refreshGridPager(); 
            if (this.model.filterSettings.filterType == "menu" || this._isExcelFilter) {
                if (!this._isExcelFilter)
                    this._closeFilterDlg();
                else if (this._excelFilter._openedFltr) {
                    this._excelFilter.closeXFDialog();
                }
                var column = this.getColumnByField(this._$curFieldName);
                var index = $.inArray(column, this.model.columns), proxy = this, _addicon = false;
                $.each(this.model.filterSettings.filteredColumns, function (indx, col) {
                    if (col.field == proxy._$curFieldName) {
                        _addicon = true;
                        return false;
                    }
                    else if (col.isComplex) {
                        if (col.predicates[0].field == proxy._$curFieldName) {
                            _addicon = true;
                            return false;
                        }
                    }
                });
                var $fIcon = this.getHeaderTable().find("thead").find(".e-headercell").eq(index).find(".e-filtericon");
                if (_addicon)
                    $fIcon.addClass("e-filteredicon e-filternone");
                else
                    $fIcon.removeClass("e-filteredicon e-filternone");
            }
        },
        _setFilterFieldValues: function (id) {
            var $fVal = "", proxy = this;
            var flchk = -1;
            $.each(this.model.filterSettings.filteredColumns, function (indx, value) {
                if (value.field == proxy._$curFieldName)
                    flchk = indx;
            });
            if (flchk == -1 && this._$colType != "boolean")
            {
                if (this._$colType == "string")
                    $(".e-predicate input[type='checkbox']").ejCheckBox({checked: false});
                $(".e-predicate input[name ="+this._id+"_predicate"+this._$colType+"]:first").ejRadioButton({checked:true});
                $("#" + this._id + this._$colType + "_ddinput").ejDropDownList({ selectedItemIndex: 0,change: function (args) { 
						this.element.closest(".e-dialog").find(".e-autocomplete").val($fVal);
					}
				});
            }
            if (this._$curFieldName != this._$prevFieldName) {
                $.each(this.model.filterSettings.filteredColumns, function (indx, col) {
                    if (col.field == proxy._$curFieldName) {
						var index;
						var option= proxy._$colType == "number" || proxy._$colType == "date" ? "Number" : "String";
						var $dlist = proxy._getLocalizedLabels(option + "MenuOptions");
                        for (index = 0; index < $dlist.length; index++){
                            if ($dlist[index].value.toLowerCase() == col.operator) 
                               break;
                        }
                        if (proxy._$colType == "string")
                            $(".e-predicate input[type='checkbox']").ejCheckBox({checked: col.matchcase});
                        $("input[value="+col.predicate+"]").ejRadioButton({checked:true});    
                        $("#" + proxy._id + proxy._$colType + "_ddinput").ejDropDownList({ selectedItemIndex: index });
                        $fVal = col.actualFilterValue != null ? col.actualFilterValue : col.value;
                        return false;
                    }
                });
                if (this._$colType == "boolean") {
                    if ($fVal && $fVal != "")
                        $(id).find(".e-value input").attr("checked", true);
                    else
                        $(id).find(".e-value input").attr("checked", false);
                } else
                    $(id).find(".e-value input").val($fVal);
            }
        },
        _fltrBtnHandler: function (e) {
            var $par = $("#" + this._id + "_" + this._$colType + "Dlg");
            var $input = $par.find(".e-value input"), $operator, result;
            var value = $input.val(), matchcase = undefined, filterValue;
            if (this._$colType == "number") {
                $input = $input.filter(".e-numerictextbox");
                value = parseFloat($input.ejNumericTextbox("getValue"));
                matchcase = true;
            }
            if (this._$colType == "string")
                matchcase = $par.find(".e-predicate input[type='checkbox']").is(":checked");
            if (this._$colType == "date") {
                value = $("#" + this._id + this._$colForeignKeyField).val();
                value = Globalize.parseDate(value, this._$colFormat);
                matchcase = true;
            }
            if (this._$colType == "boolean") {
                value = $input.ejCheckBox("model.checked") != null ? $input.ejCheckBox("model.checked") : false;
                $operator = "equal";
            } else
                $operator = $("#" + this._id + this._$colType + "_ddinput").ejDropDownList("getSelectedValue").toLowerCase();
            if (this._$colForeignKeyValue && this._$colDropdownData)
                result = this._fltrForeignKeyValue($operator, value, matchcase, this._$colDropdownData, this._$colForeignKeyField, this._$colForeignKeyValue);
            if (ej.isNullOrUndefined(result))
                this.filterColumn(this._$curFieldName, $operator, value, $par.find(".e-predicate input[type='radio']:checked").attr("value"), matchcase);
            else {
                if ($par.find(".e-predicate input[type='radio']:checked").css("display") == "none" && result.value.length > 1)
                    this.filterColumn(this._$curFieldName, result.operator, result.value, "or", matchcase, result.filterValue);
                else
                    this.filterColumn(this._$curFieldName, result.operator, result.value, $par.find(".e-predicate input[type='radio']:checked").attr("value"), matchcase, result.filterValue);
            }
        },
        _fltrClrHandler: function (e) {
            var id = "#" + this._id + "_" + this._$colType + "Dlg";
            if (this._$colType == "boolean")
                $(id).find(".e-value input").ejCheckBox("model.checked", false);
            else
                if (this._$colType == "number")
                    $(id).find('.e-numerictextbox').ejNumericTextbox("model.value","");
                else
                    $(id).find(".e-value input").val("");
            if (this._$colType == "date") {
                for (var j = 0; j < this.model.filterSettings.filteredColumns.length; j++) {
                    if (this.model.filterSettings.filteredColumns[j].isComplex) {
                        var preobject = this.model.filterSettings.filteredColumns[j].predicates;
                        if (this.model.filterSettings.filteredColumns.length == 1) {
                            this.model.filterSettings.filteredColumns = preobject;
                        }
                        else {
                            this.model.filterSettings.filteredColumns[j] = preobject[0];
                            this.model.filterSettings.filteredColumns.push(preobject[1]);
                        }
                    }
                }
            }
            this.filterColumn(this._$curFieldName, "", "", "or");
        },

        _fltrForeignKeyValue: function (operator, value, matchcase, dataSource, fieldName, mapFieldName, colType) {
            if (ej.isNullOrUndefined(matchcase))
                matchcase = true;
            var operatorCol = [], query, filterValue;
            var data = ej.DataManager(dataSource);
            if (colType == "date") {
                var $prevDate = new Date(value.setDate(value.getDate() - 1));
                var $nextDate = new Date(value.setDate(value.getDate() + 2));
                if (operator == "equal" || operator == "notequal") {
                    if (operator == "equal")
                        query = new ej.Query().where(ej.Predicate(mapFieldName, ">", $prevDate, !matchcase)
                            .and(mapFieldName, "<", $nextDate, !matchcase)).select(fieldName);
                    else
                        query = new ej.Query().where(ej.Predicate(mapFieldName, "<=", $prevDate, !matchcase)
                           .or(mapFieldName, ">=", $nextDate, !matchcase)).select(fieldName);
                }
                else
                    query = new ej.Query().where(mapFieldName, operator, value, !matchcase).select(fieldName);
            }
            else
                query = new ej.Query().where(mapFieldName, operator, value, !matchcase).select(fieldName);
            filterValue = value;
            value = data.executeLocal(query);
            operatorCol.push("equal");
            for (i = 1; i < value.length ; i++) {
                operatorCol.push("equal");
            }
            return { operator: operatorCol, value: value, filterValue: filterValue };
        },
        _setDateFilters: function (filterObject) {
            var $prevDate = new Date(filterObject.value.setDate(filterObject.value.getDate() - 1));
            var $nextDate = new Date(filterObject.value.setDate(filterObject.value.getDate() + 2));
            var $prevObj = $.extend({}, filterObject);
            var $nextObj = $.extend({}, filterObject);
            $prevObj.value = $prevDate;
            $nextObj.value = $nextDate;
            if (filterObject.operator == "equal") {
                $prevObj.operator = "greaterthan";
                $prevObj.predicate = "and";
                $nextObj.operator = "lessthan";
                $nextObj.predicate = "and";
            } else {
                $prevObj.operator = "lessthanorequal";
                $prevObj.predicate = "or";
                $nextObj.operator = "greaterthanorequal";
                $nextObj.predicate = "or";
            }
            pred = ej.Predicate($prevObj.field, $prevObj.operator, $prevObj.value, false);
            predicate = pred[$nextObj.predicate](ej.Predicate($nextObj.field, $nextObj.operator, $nextObj.value, false));
            this.model.filterSettings.filteredColumns.push(predicate);
        },
    };
})(jQuery, Syncfusion);;
(function ($, ej, undefined) {
    ej.gridFeatures = ej.gridFeatures || {};
    ej.gridFeatures.group = {
        _renderGroupDropArea: function () {
            if (!this.model.groupSettings.enableDropAreaAutoSizing)
                this.model.groupSettings.enableDropAreaAnimation = this.model.groupSettings.enableDropAreaAutoSizing;
            var dragLabel = this.model.groupSettings.enableDropAreaAnimation ? "" : this._getLocalizedLabels("GroupDropArea");
            var $div = ej.buildTag("div.e-groupdroparea", dragLabel);
            this.model.groupSettings.enableDropAreaAutoSizing && $div.append(ej.buildTag("div.e-animatebutton e-icon").addClass(this.model.groupSettings.enableDropAreaAnimation ? "e-animatebuttondown e-gdownarrow" : "e-animatebuttonup e-guparrow"));
            return $div;
        },
        _getColGroup: function (gridObjectId) {
            var gridObject = this.getRsc("helpers", gridObjectId);
            if (gridObject.model.groupSettings.groupedColumns.length == 1)
                var level = this.data.items.level === undefined ? 0 : this.data.items.level - 1;
            else
                var level = this.data.items.level === undefined ? gridObject.model.groupSettings.groupedColumns.length : this.data.items.level - 1;
            var $div = $(document.createElement("div"));
            var $colGroup;
            if (gridObject._isCaptionSummary)
                $colGroup = gridObject._getCaptionColGroup(level);
            else {
                $colGroup = gridObject._getMetaColGroup();
                if (level != gridObject.model.groupSettings.groupedColumns.length && gridObject.model.groupSettings.groupedColumns.length > 1)
                    $colGroup.prepend(gridObject._getIndentCol());
            }
            $div.html($colGroup);
            return $div.html();
        },
        _colSpanAdjust: function (gridObjectId, type, captionDetails) {
            var gridObject, groupData;
            if (ej.isNullOrUndefined(gridObjectId)) {
                gridObject = this;
                groupData = captionDetails;
            }
            else {
                gridObject = this.getRsc("helpers", gridObjectId);
                groupData = this;
            }
            if (gridObject.model.groupSettings.groupedColumns.length == 1) {
                var level = groupData.data.items.level === undefined ? 1 : groupData.data.items.level - 1;
                if (type == "groupcaption")
                    gridObject._currentJsonData = gridObject._currentJsonData.concat(groupData.data.items);
            } else {
                var level = groupData.data.items.level === undefined ? gridObject.model.groupSettings.groupedColumns.length : groupData.data.items.level - 1;
                if (type == "groupcaption" && groupData.data.items.level === undefined)
                    gridObject._currentJsonData = gridObject._currentJsonData.concat(groupData.data.items);
            }
            var hideGroupColumnCount = !gridObject.model.groupSettings.showGroupedColumn ? gridObject.model.groupSettings.groupedColumns.length : 0;
            var colspan = gridObject.model.columns.length + gridObject.model.groupSettings.groupedColumns.length - level - gridObject._hiddenColumns.length - hideGroupColumnCount;
            colspan = gridObject.model.detailsTemplate != null ? colspan + 1 : colspan;

            if (gridObject._isCaptionSummary && type == "groupcaption") {
                var index = [];
                var row = gridObject._captionSummary();
                $.each(row[0].summaryColumns, function (cindx, col) {
                    index.push(gridObject.getColumnIndexByField(col.displayColumn));
                });
                if (index.length > 0)
                    colspan = ej.min(index);
                if (!gridObject.model.groupSettings.showGroupedColumn) {
                    colspan = groupData.data.items.childLevels != null ? (groupData.data.items.childLevels + 2) : 1;
                }
                else {
                    colspan = gridObject.model.detailsTemplate != null ? colspan + 1 : colspan;
                    colspan = groupData.data.items.childLevels != null ? colspan + (groupData.data.items.childLevels + 1) : colspan;
                }
            }
            return colspan;
        },
        _captionFormat: function (gridObjectId) {
            var gridObject = this.getRsc("helpers", gridObjectId);
            var keyValue;
            var column = gridObject.getColumnByField(this.data.field);
            if (column.foreignKeyValue && column.dataSource)
                keyValue = gridObject._foreignKeyBinding(column, this.data.key);
            else
                keyValue = this.data.key;
            var formatted = column.format ? gridObject.formatting(column.format, keyValue) : keyValue;
            return column.headerText + ": " + formatted + " - " + this.data.count + " " + gridObject._getLocalizedLabels("GroupCaptionFormat");
        },
        _getCaptionColGroup: function (level) {
            var cloneColGroup = this.getHeaderTable().find("colgroup").clone();
            var colColl = cloneColGroup.find("col");
            var indentCol = colColl.length - this.model.columns.length;
            if (this.model.detailsTemplate != null)
                indentCol = indentCol - 1;
            cloneColGroup.find("col:lt(" + indentCol + ")").remove();
            if (level > 0 && level != this.model.groupSettings.groupedColumns.length) {
                if (this.model.groupSettings.groupedColumns.length > 2 && level != this.model.groupSettings.groupedColumns.length - 1) {
                    for (var i = 0; i < this.model.groupSettings.groupedColumns.length - level; i++) {
                        cloneColGroup.prepend(this._getIndentCol());
                    }
                }
                else
                    cloneColGroup.prepend(this._getIndentCol());
            }
            return cloneColGroup;
        },
        _groupSummaryRow: function (item, aggregates, gridObjectId) {
            var gridObject = this.getRsc("helpers", gridObjectId);
            if (gridObject.model.showSummary) {
                if (!ej.isNullOrUndefined(item.level) && !gridObject._isCaptionSummary) return;
                if (gridObject.getFooterTable() == null)
                    gridObject._renderGridFooter();
                gridObject._createSummaryRows(gridObject.getFooterTable(), item.records == null ? item : item.records,aggregates);
                if (gridObject._isCaptionSummary) {
                    var index = [];
                    var row = gridObject._captionSummary();
                    $.each(row[0].summaryColumns, function (cindx, col) {
                        index.push(gridObject.getColumnIndexByField(col.displayColumn));
                    });
                    if (index.length > 0)
                        colIndex = ej.min(index);
                    var colLength = gridObject.model.columns.length;
                    if (!gridObject.model.groupSettings.showGroupedColumn) {
                        gridObject.getFooterTable().find("tbody td").slice(-(colLength - 1)).removeClass("e-summaryrow").addClass("e-groupcaptionsummary");
                    }
                    else {
                        gridObject.getFooterTable().find("tbody td").slice(-(colLength - colIndex)).removeClass("e-summaryrow").addClass("e-groupcaptionsummary");
                    }
                }
                if (!gridObject.model.groupSettings.showGroupedColumn) {
                  var groupedcolumns = gridObject.model.groupSettings.groupedColumns;
                  var count=0;
                  var gridfooterrow = gridObject.getFooterTable().children('tbody').find('tr');
                  for (var j = 0; j < gridObject.model.summaryRows.length; j++) {
                  for (var k = 0; k < gridObject.model.summaryRows[j].summaryColumns.length; k++) {
                  for (var i = 0; i < groupedcolumns.length; i++) {
                      if (groupedcolumns[i] == gridObject.model.summaryRows[j].summaryColumns[k].displayColumn){
                      count++;
                      if (gridObject.model.summaryRows[j].summaryColumns.length == count)
                      { 
                          $(gridfooterrow[j]).addClass("e-hide")
                      }}}}
                  count=0;
                   }}
                return gridObject.getFooterTable().find("tbody").html();
            }
        },
        addGroupingTemplate: function () {
            var tbody = document.createElement('tbody');
            var expandTd = "<td class='e-recordplusexpand'><div class='e-icon e-gdiagonalnext'></div></td>";
            var proxy = this;
            var helpers = {};
            helpers["_" + proxy._id + "ColSpanAdjust"] = this._colSpanAdjust;
            helpers["_" + proxy._id + "Colgroup"] = this._getColGroup;
            helpers["_" + proxy._id + "CaptionFormat"] = this._captionFormat;
            helpers["_" + proxy._id + "GroupSummaryRow"] = this._groupSummaryRow;
            helpers[proxy._id + "Object"] = this;
            $.views.helpers(helpers);
            var cpationTd = expandTd + "<td class='e-groupcaption' colspan='{{:~_" + proxy._id + "ColSpanAdjust('" + proxy._id + "Object" + "','groupcaption') }}'>{{html:~_" + proxy._id + "CaptionFormat('" + proxy._id + "Object')}}</td";
            if (this._isCaptionSummary && this.model.showSummary)
                cpationTd = cpationTd + "{{:~_" + proxy._id + "GroupSummaryRow(items, aggregates,'" + proxy._id + "Object')}}";
            var captionTr = "<tr>" + cpationTd + "</tr>";
            var $tbody = ej.buildTag("tbody");
            $tbody.html("{{if items.GROUPGUID}}" +
                "{{for items tmpl='" + proxy._id + "_GroupingTemplate'/}}" +
                "{{else}}" +
                "{{for items tmpl='" + proxy._id + "_JSONTemplate'/}}" +
                "{{/if}}");
            var indentTd = "<td class='e-indentcell'></td>";
            var table = "<table class='e-table {{if items.GROUPGUID}}{{else}}e-recordtable{{/if}}' cellspacing='0.25px' >" +
                "{{:~_" + proxy._id + "Colgroup('" + proxy._id + "Object')}}" +
                $tbody.html() + (this._isCaptionSummary ? "</table>" : "{{:~_" + proxy._id + "GroupSummaryRow(items,aggregates,'" + proxy._id + "Object')}}" +
            "</table>");
            var tableTd = "<td class='e-tabletd' colspan='{{:~_" + proxy._id + "ColSpanAdjust('" + proxy._id + "Object" + "','table')}}'>" + table + "</td>";
            var tr = "<tr>" + indentTd + tableTd + "</tr>";
            $.templates(proxy._id + "_GroupingTemplate", captionTr + tr);
        },
        _getGroupTopLeftCell: function () {
            var $th = ej.buildTag("th.e-grouptopleftcell");
            $th.append(ej.buildTag("div.e-headercelldiv", "&nbsp;"));
            return $th;
        },
        _getEmptyFilterBarCell: function () {
            return ej.buildTag("th.e-filterbarcell");
        },
        _groupingAction: function () {
            var $groupTopCell = this.getHeaderTable().find("thead").find(".e-columnheader").find(".e-grouptopleftcell"), $col = this.getHeaderTable().find("colgroup").find("col");
            var groupColumn = $groupTopCell.length;
            if (groupColumn) {
                this.getHeaderTable().find("colgroup").replaceWith(this._getMetaColGroup());
                (this.model.detailsTemplate != null || this.model.childGrid != null) && this.getHeaderTable().find("colgroup").prepend(this._getIndentCol());
                $groupTopCell.remove();
                this.getHeaderTable().find("thead").find(".e-filterbar").find(".e-filterbarcell:lt(" + groupColumn + ")").remove();
            }
            !this.model.allowResizeToFit && this.setWidthToColumns();
            for (var i = 0; i < this.model.groupSettings.groupedColumns.length; i++) {
                this.getHeaderTable().find("colgroup").prepend(this._getIndentCol());
                this.getHeaderTable().find("thead").find(".e-columnheader").prepend(this._getGroupTopLeftCell());
                this.getHeaderTable().find("thead").find(".e-filterbar").prepend(this._getEmptyFilterBarCell());

            }
            this.getHeaderTable().find("thead").find("th.e-grouptopleftcell").last().addClass("e-lastgrouptopleftcell");
        },
        /**
		 * Send a column grouping request in grid.
         * @alias ejGrid#groupColumn
		 * @return jQuery
		 * @param {string} fieldName Pass the field Name of the column to be grouped in grid control
		 * @example 
		 * &lt;script&gt;
		 * // Create grid object.
		 * var gridObj = $("#Grid").data("ejGrid");
         * // Sends a group column request to the grid
		 * gridObj.groupColumn("OrderID"); 
		 * &lt;/script&gt;
		 * @example 
		 * &lt;script&gt;
		 * // Sends a group column request to the grid
		 * $("#Grid").ejGrid("groupColumn", "OrderID");        
		 * &lt;/script&gt;
         */
        groupColumn: function (columnName) {
            if (!this.model.allowGrouping || $.inArray(columnName, this._disabledGroupableColumns) != -1)
                return;
            if (ej.isNullOrUndefined(this.getColumnByField(columnName)) || $.inArray(columnName, this.model.groupSettings.groupedColumns) != -1)
                return;
            this.model.groupSettings.groupedColumns.push(columnName);
            for (var i = 0; i < this.model.sortSettings.sortedColumns.length; i++) {
                if (this.model.sortSettings.sortedColumns[i].field == columnName) {
                    break;
                }
            }
            this.model.sortSettings.sortedColumns.length == i && this.model.sortSettings.sortedColumns.push({ field: columnName, direction: ej.sortOrder.Ascending });
            var args = {};
            args.columnName = columnName;
            args.requestType = ej.Grid.Actions.Grouping;
            var returnValue = this._processBindings(args);
            if (returnValue)
                this.model.groupSettings.groupedColumns.pop();
            this._primaryKeyValues = [];
        },
        /**
         * Un-group a column from grouped columns collection in grid
         * @alias ejGrid#ungroupColumn
         * @return jQuery
         * @param {string} fieldName Pass the field Name of the column to be ungrouped from grouped column collection
         * @example 
         * &lt;script&gt;
         * // Create grid object.
         * var gridObj = $("#Grid").data("ejGrid");
         * // Sends an ungroup column request to the grid
         * gridObj.ungroupColumn("OrderID"); 
         * &lt;/script&gt;
         * @example 
         * &lt;script&gt;
         * // Sends an ungroup column request to the grid
         * $("#Grid").ejGrid("ungroupColumn", "OrderID");        
         * &lt;/script&gt;
         */
        ungroupColumn: function (columnName) {
            if (!this.model.allowGrouping)
                return;
            if ($.inArray(columnName, this.model.groupSettings.groupedColumns) != -1)
                this.model.groupSettings.groupedColumns.splice($.inArray(columnName, this.model.groupSettings.groupedColumns), 1);
            else
                return null;
            var args = {};
            for (var i = 0; i < this.model.sortSettings.sortedColumns.length; i++) {
                if (this.model.sortSettings.sortedColumns[i].field == columnName) {
                    this.model.sortSettings.sortedColumns.splice(i, 1);
                    break;
                }
            }
            args.columnName = columnName;
            args.requestType = ej.Grid.Actions.Ungrouping;
            var returnValue = this._processBindings(args);
            if (returnValue)
                this.model.groupSettings.groupedColumns.push(columnName);
            this._primaryKeyValues = [];
        },
        /**
         * Collapse the group drop area in grid
         * @alias ejGrid#collapseGroupDropArea
         * @return jQuery
         * @example 
         * &lt;script&gt;
         * // Create grid object.
         * var gridObj = $("#Grid").data("ejGrid");
         * // Collapse the group drop area of the grid
         * gridObj.collapseGroupDropArea(); 
         * &lt;/script&gt;
         * @example 
         * &lt;script&gt;
         * // Collapse the group drop area of the grid
         * $("#Grid").ejGrid("collapseGroupDropArea");        
         * &lt;/script&gt;
         */
        collapseGroupDropArea: function () {
            var $groupDropArea = this.element.find(".e-groupdroparea").first(), proxy = this;
            this.model.groupSettings.groupedColumns.length == 0 && this.model.groupSettings.enableDropAreaAnimation && $groupDropArea.animate({ height: "10px" }, 200, function () {
                if (proxy.model != null && proxy.model.groupSettings.groupedColumns.length == 0) {
                    $(this).html("").append(ej.buildTag("div.e-animatebutton e-animatebuttondown e-icon e-gdownarrow"));
                    $(this).dequeue().css("height", "auto");
                }
            });
        },
        /**
         * Expand the group drop area in grid.
         * @alias ejGrid#expandGroupDropArea
         * @return jQuery
         * @example 
         * &lt;script&gt;
         * // Create grid object.
         * var gridObj = $("#Grid").data("ejGrid");
         * // Expands the group drop area of the grid
         * gridObj.expandGroupDropArea(); 
         * &lt;/script&gt;
         * @example 
         * &lt;script&gt;
         * // Expands the group drop area of the grid
         * $("#Grid").ejGrid("expandGroupDropArea");        
         * &lt;/script&gt;
         */
        expandGroupDropArea: function () {
            var $groupDropArea = this.element.find(".e-groupdroparea").first(), proxy = this;
            this.model.groupSettings.groupedColumns.length == 0 && proxy.model.groupSettings.enableDropAreaAnimation && $groupDropArea.animate({ height: "30px" }, 150, function () {
                proxy.model.groupSettings.groupedColumns.length == 0 && $groupDropArea.html(proxy._getLocalizedLabels("GroupDropArea")).append(ej.buildTag("div.e-animatebutton e-animatebuttonup e-icon e-guparrow"));
                $groupDropArea.dequeue().css("height", "30px");
            });
        },
        _enableGroupingEvents: function () {
            if (this.model.allowGrouping) {
                this._on(this.element, "click", ".e-groupdroparea,.e-groupheadercell", this._groupHeaderCellClick);
                //to avoid ungrouping issue.
                this._on(this.element, "mousedown", ".e-groupheadercell", function (e) {
                    return false;
                });
            }

        },
        _recalculateIndentWidth: function () {
            var proxy = this;
            var browserDetails = this.getBrowserDetails();
            var indentWidth = this.getHeaderTable().find(".e-lastgrouptopleftcell").width(), newWidth = indentWidth, perPixel = indentWidth / 30, $col;
            if (perPixel >= 1)
                newWidth = (30 / perPixel);
            this.getHeaderTable().find("colgroup").find("col").slice(0, this.model.groupSettings.groupedColumns.length).css("width", newWidth + "px");
            indentWidth = this.getHeaderTable().find(".e-lastgrouptopleftcell").width();
            if (indentWidth > 30 || (this._isCaptionSummary && indentWidth >= 30)) {
                if (this._isCaptionSummary) {
                    var colgroup = this.getContentTable().find("table").filter(":not(.e-recordtable)").children("colgroup");
                    $.each(colgroup, function (index, item) {
                        var indentCol = $(item).find("col").length - proxy.model.columns.length;
                        if (proxy.model.detailsTemplate != null)
                        {
                            if (indentCol > 0)
                            indentCol = indentCol - 1;
                        }
                        $(item).find("col").slice(0, indentCol).css("width", newWidth + "px");
                    });
                }
                else
                    this.getContentTable().find("table").filter(":not(.e-recordtable)").children("colgroup").find("col:first-child").css("width", indentWidth + "px");
                $col = this.getContentTable().find("colgroup").first().find("col").slice(0, this.model.groupSettings.groupedColumns.length);
                if (browserDetails.browser != "msie")
                    $col.css("width", newWidth + "px");
                else
                    $col.first().css("width", ((indentWidth / this.element.width()) * 100) + "%");
            } else
                this.getContentTable().find("colgroup").first().find("col").slice(0, this.model.groupSettings.groupedColumns.length).css("width", newWidth + "px");

        },
        /**
		 * Get the column field name from the given header text in grid.
         * @alias ejGrid#getFieldNameByHeaderText
		 * @return fieldName
		 * @param {string} headerText Pass header text of the column to get its corresponding field name
		 * @example 
		 * &lt;script&gt;
		 * // Create grid object.
		 * var gridObj = $("#Grid").data("ejGrid");
         * // Gets the column field name from the given headerText
		 * gridObj.getFieldNameByHeaderText("Order ID"); 
		 * &lt;/script&gt;
		 * @example 
		 * &lt;script&gt;
		 * // Gets the column field name from the given headerText
		 * $("#Grid").ejGrid("getFieldNameByHeaderText", "Order ID");
		 * &lt;/script&gt;
         */
        getFieldNameByHeaderText: function (headerText) {
            if (ej.isNullOrUndefined(this._fieldColumnNames[headerText]))
                return null;
            return this._fieldColumnNames[headerText];
        },
        /**
         * Get the column header text from the given field name in grid.
         * @alias ejGrid#getHeaderTextByFieldName
         * @return headerText
         * @param {string} field Pass field name of the column to get its corresponding header text
         * @example 
         * &lt;script&gt;
         * // Create grid object.
         * var gridObj = $("#Grid").data("ejGrid");
         * gridObj.getHeaderTextByFieldName("OrderID"); // Gets the column header text from the given field name
         * &lt;/script&gt;
         * @example 
         * &lt;script&gt;
         * // Gets the column header text from the given field name
         * $("#Grid").ejGrid("getHeaderTextByFieldName", "OrderID");
         * &lt;/script&gt;
         */
        getHeaderTextByFieldName: function (field) {
            if (ej.isNullOrUndefined(this._headerColumnNames[field]))
                return null;
            return this._headerColumnNames[field];
        },
        /**
         * Expand all the group caption rows in grid.
         * @alias ejGrid#expandAll
         * @return jQuery
         * @example 
         * &lt;script&gt;
         * // Create grid object.
         * var gridObj = $("#Grid").data("ejGrid");
         * // Expand all the group caption rows
         * gridObj.expandAll(); 
         * &lt;/script&gt;
         * @example 
         * &lt;script&gt;
         * // Expand all the group caption rows
         * $("#Grid").ejGrid("expandAll");        
         * &lt;/script&gt;
         */
        expandAll: function () {
            var recordPlus = this.element.find(".e-recordpluscollapse");
            for (var i = 0; i < recordPlus.length; i++)
                this.expandCollapse($(recordPlus[i]));
        },
        /**
		 * Collapse all the group caption rows in grid
         * @alias ejGrid#collapseAll
		 * @return jQuery
		 * @example 
		 * &lt;script&gt;
		 * // Create grid object.
		 * var gridObj = $("#Grid").data("ejGrid");
         * // Collapse all the group caption rows
		 * gridObj.collapseAll(); 
		 * &lt;/script&gt;
		 * @example 
		 * &lt;script&gt;
		 * // Collapse all the group caption rows
		 * $("#Grid").ejGrid("collapseAll");        
		 * &lt;/script&gt;
         */
        collapseAll: function () {
            var recordPlus = this.element.find(".e-recordplusexpand");
            for (var i = 0; i < recordPlus.length; i++)
                this.expandCollapse($(recordPlus[i]));
        },
        _group: function (args) {
            if (this.model.groupSettings.groupedColumns.length && this.model.currentViewData) {
                this._currentJsonData = [];
                var $tbody = this.getContentTable().children('tbody');
                $tbody.empty();
                var temp = document.createElement('div');
                if (!this.model.groupSettings.showGroupedColumn) {
                    this._hideHeaderColumn(this.model.groupSettings.groupedColumns, true);
                    this.getContentTable().children("colgroup").replaceWith(this.getHeaderTable().find('colgroup').clone());
                }
                if (args.requestType == "reorder")
                    this._isReorder = true;
                else
                    this._isReorder = false;
                var $col = this.getContentTable().children("colgroup").find('col');
                var length = $col.length - this.model.columns.length;
                if (this.model.detailsTemplate != null)
                    length--;
                if ($col.length > this.model.columns.length)
                    this.getContentTable().children("colgroup").find('col:lt(' + length + ')').remove();
                this.getContentTable().find("colgroup").first().replaceWith(this._getMetaColGroup());
                var dlen;
                if (this.model.detailsTemplate != null) {
                    dlen = this.model.groupSettings.groupedColumns.length + 1;
                }
                else
                    dlen = this.model.groupSettings.groupedColumns.length;
                for (var i = 0; i < dlen; i++)
                    this.getContentTable().children("colgroup").prepend(this._getIndentCol());
                temp.innerHTML = ['<table>', $.render[this._id + "_GroupingTemplate"](this.model.currentViewData, { groupedColumns: this.model.groupSettings.groupedColumns }), '</table>'].join("");
                this.getContentTable().get(0).replaceChild(temp.firstChild.firstChild, this.getContentTable().get(0).lastChild);
                this._hideCaptionSummaryColumn();
                this._groupingAction();
                this._gridRows = this.getContentTable().find(".e-recordtable").find("tbody").find("tr").toArray();
                this._eventBindings();
            }
        },
        _ungroup: function () {
            var $header = this.element.children(".e-gridheader");
            var $filterInput = $header.find(".e-filterbar").find("th").find("input");
            $header.find("div").first().empty().append(this._renderGridHeader().find("table"));
            this.setGridHeaderContent($header);
            if (this.model.allowFiltering && this.model.filterSettings.filterType == "filterbar") {
                this._renderFiltering();
                var filterThNew = this.element.children(".e-gridheader").find(".e-filterbar").find("th").find("input");
                for (var i = 0; i < $filterInput.length; i++)
                    filterThNew.eq(i).val($filterInput.eq(i).val());
            }
            if (!this.model.groupSettings.showGroupedColumn)
                this._hideHeaderColumn(this.model.groupSettings.groupedColumns, true);
            this.element.find(".e-gridcontent").children("div").first().empty().append(this._renderGridContent().find("table").first());
            this.setGridContent(this.element.find(".e-gridcontent"));
            if (this.model.groupSettings.groupedColumns.length != 0)
                this._gridRows = this.getContentTable().find(".e-recordtable").find("tbody").find("tr").toArray();
            else
                this._gridRows = this.getContentTable().get(0).rows;
            this._eventBindings();
        },
        _groupHeaderCellClick: function (e) {
            var $target = $(e.target);
            if ($target.hasClass("e-ungroupbutton")) {
                var columnName = $.trim($target.parent().text());
                var field = this.getFieldNameByHeaderText(columnName);
                this.ungroupColumn(field);
            } else if ($target.hasClass("e-togglegroupbutton")) {
                var columnName = $.trim($target.parent().text()), field;
                field = this.getFieldNameByHeaderText(columnName);
                $target.hasClass("e-toggleungroup") && this.ungroupColumn(field);
            } else if ($target.hasClass("e-animatebutton")) {
                if (!this.model.groupSettings.enableDropAreaAnimation) {
                    this.model.groupSettings.enableDropAreaAnimation = true;
                    this.collapseGroupDropArea();
                } else {
                    this.expandGroupDropArea();
                    this.model.groupSettings.enableDropAreaAnimation = false;
                }
            }

            return false;
        },
        _captionSummary: function () {
            var summary = null;
            for (k = 0; k < this.model.summaryRows.length; k++) {
                if (this.model.summaryRows[k].showCaptionSummary == true) {
                    summary = $(this.model.summaryRows[k]);
                    break;
                }
            }
            return summary;
        },
        _dropAreaHover: function (e) {
            var $target = $(e.target), proxy = this;
            if (e.type == "mouseenter") {
                if (this._dragActive) {
                    if ($target.hasClass("e-groupdroparea"))
                        $target.addClass("e-hover");
                } else
                    $target.removeClass("e-hover");
                $target.hasClass("e-groupheadercell") && this.model.groupSettings.showUngroupButton && $target.find(".e-ungroupbutton").show(150);
            } else if (e.type == "mouseleave") {
                if ($target.hasClass("e-groupdroparea")) {
                    $target.find(".e-ungroupbutton").hide(150);
                    $target.removeClass("e-hover");
                }
                $target.hasClass("e-groupheadercell") && this.model.groupSettings.showUngroupButton && $target.find(".e-ungroupbutton").hide(150);
            }
            return false;
        },
        _groupingCompleteAction: function (args) {
            var $groupDrop = this.element.children(".e-groupdroparea");
            if (this.model.groupSettings.groupedColumns.length && $groupDrop.find(".e-grid-icon").length == 0)
                $groupDrop.empty();
            if (this._initialRender) {
                for (var i = 0; i < this.model.groupSettings.groupedColumns.length; i++)
                    this._addColumnToGroupDrop(this.model.groupSettings.groupedColumns[i]);
                this._refreshGridPager();
            } else
                this._addColumnToGroupDrop(args.columnName);
            this.getHeaderTable().find(".e-columnheader").find(".e-headercelldiv").find(".e-ascending,.e-descending").remove();
            for (var i = 0; i < this.model.sortSettings.sortedColumns.length; i++)
                this._addSortElementToColumn(this.model.sortSettings.sortedColumns[i].field, this.model.sortSettings.sortedColumns[i].direction);
            if (!this._initialRender && ej.gridFeatures.dragAndDrop)
                this._groupHeaderCelldrag();
            this.model.allowScrolling && this.getContentTable().parent().scrollLeft(this.getHeaderTable().parent().scrollLeft() - 1);
            this.element.children(".e-cloneproperties").remove();
        },
        _ungroupingCompleteAction: function (args) {
            var $groupDrop = this.element.children(".e-groupdroparea");
            this._removeColumnFromDrop(args.columnName);
            this.getHeaderTable().find(".e-columnheader").find(".e-headercelldiv").find(".e-ascending,.e-descending").remove();
            for (var i = 0; i < this.model.sortSettings.sortedColumns.length; i++)
                this._addSortElementToColumn(this.model.sortSettings.sortedColumns[i].field, this.model.sortSettings.sortedColumns[i].direction);
            if (this.model.groupSettings.groupedColumns.length == 0) {
                $groupDrop.html(this.model.groupSettings.enableDropAreaAnimation ? "" : this._getLocalizedLabels("GroupDropArea"));
                this.model.groupSettings.enableDropAreaAutoSizing && $groupDrop.append(ej.buildTag("div.e-animatebutton e-icon").addClass(this.model.groupSettings.enableDropAreaAnimation ? "e-animatebuttondown e-gdownarrow" : "e-animatebuttonup e-guparrow"));
                $groupDrop.css("height", "auto");
            }
            if (ej.gridFeatures.dragAndDrop)
                this._headerCellgDragDrop();
            this.model.allowScrolling && this.getContentTable().parent().scrollLeft(this.getHeaderTable().parent().scrollLeft() + 10);
            this.element.children(".e-cloneproperties").remove();
        },
        _getToggleButton: function () {
            return ej.buildTag("span.e-togglegroupbutton e-icon e-groupbutton", "&nbsp;");
        },
        _checkEinHeader: function (field) {
            var $headerCell = this.element.children(".e-gridheader").find("thead").find(".e-columnheader").find(".e-headercell");
            for (var i = 0; i < $headerCell.length; i++) {
                if ($.trim($headerCell.eq(i).text()) == field)
                    return $headerCell.eq(i);
            }
            return null;

        },

        _checkEinGroupDrop: function (field) {
            var $groupHeaderCell = this.element.children(".e-groupdroparea").find(".e-grid-icon");
            for (var i = 0; i < $groupHeaderCell.length; i++) {
                if ($.trim($groupHeaderCell.eq(i).text()) == field)
                    return $groupHeaderCell.eq(i);
            }
            return null;
        },

        _addColumnToGroupDrop: function (field) {
            var $groupedColumn = ej.buildTag("div.e-grid-icon e-groupheadercell"), $groupDropArea = this.element.find(".e-groupdroparea").first();
            var $childDiv = ej.buildTag("div"), imageDirection = "e-rarrowup-2x";
            var column = this.getColumnByField(field)
            if (column.disableHtmlEncode)
                $groupedColumn.append($childDiv.text(column.headerText));
            else
                $groupedColumn.append($childDiv.html(column.headerText));
            var $headerCell = this._checkEinHeader(column.headerText);
            if (this.model.groupSettings.showToggleButton) {
                $childDiv.append(this._getToggleButton().addClass("e-toggleungroup"));
                $headerCell.find(".e-headercelldiv").find(".e-togglegroupbutton").remove().end().append(this._getToggleButton().addClass("e-toggleungroup"));
            }
            var direction = "ascending";
            if ($headerCell.find(".e-ascending,.e-descending").length) {
                direction = $headerCell.find(".e-ascending,.e-descending").hasClass("e-ascending") ? "ascending" : "descending";
                imageDirection = direction == "ascending" ? "e-rarrowup-2x" : "e-rarrowdown-2x";
            }
            $childDiv.append(this._createSortElement().addClass("e-" + direction + " " + imageDirection));
            this.model.groupSettings.showUngroupButton && $childDiv.append(ej.buildTag("span.e-ungroupbutton e-icon e-cancel", " ", {}, { title: "Click here to ungroup" }));
            $groupDropArea.append($groupedColumn).css("height", "auto");
            var left = $groupedColumn.offset().left, $cloned = $groupedColumn.clone().css("position", "absolute"), proxy = this;
            $groupedColumn.css("visibility", "hidden")
            $groupDropArea.append($cloned).dequeue();
            $cloned.css({ "left": left + 150 }).animate({ left: left }, 150, function (e) {
                $groupedColumn.css("visibility", "visible");
                $cloned.remove();
            });
        },
        _removeColumnFromDrop: function (field) {
            var headerText = this.getHeaderTextByFieldName(field), proxy = this, $groupDropArea = this.element.children(".e-groupdroparea");
            var $groupHeaderCell = $groupDropArea.css("height", "30px").find(".e-grid-icon");
            for (var i = 0; i < $groupHeaderCell.length; i++) {
                if ($.trim($groupHeaderCell.eq(i).text()) == headerText) {
                    if (this.model.groupSettings.groupedColumns.length == 0) {
                        this.collapseGroupDropArea();
                    } else
                        $groupHeaderCell.eq(i).remove();
                }
            }
        },
        _setAggreatedCollection: function (clonedQuery) {
            if (this._dataSource() instanceof ej.DataManager)
                return;
            var data;            
            data = ej.DataManager(this._dataSource()).executeLocal(clonedQuery).result;
            this._aggregatedCollection = data;
        },        
        _setAggregates: function (data, collection) {
            var indx, pred, query = new ej.Query();
            data = data || this.model.currentViewData, collection = collection || this._aggregatedCollection;
            var dLen = data.length, cLen;
            if (dLen != 0)
            pred = ej.Predicate("field", "equal", data[0].field)["and"]("key", "equal", data[0].key);
            for (indx = 1; indx < dLen; indx++) {
                pred = pred["or"](ej.Predicate("field", "equal", data[indx].field)["and"]("key", "equal", data[indx].key));
            }
            collection = pred ? new ej.DataManager(collection).executeLocal(query.where(pred)) : collection;
            cLen = collection.length;
            for (indx = 0; indx < cLen; indx++) {
                if(indx > 0  && indx < cLen - 1) continue;
                data[indx].count = collection[indx].count; /*Updating the count*/
                if (data[indx].items.GROUPGUID)
                    this._setAggregates(data[indx].items, collection[indx].items);
                if (this.model.showSummary) { /*Adding extra property which will contain aggregates*/
                    var agg = data[indx]["aggregates"] = [];
                    var rows = this.model.summaryRows, scolumns, summaryData;
                    for (var row = 0, rlen = rows.length; row < rlen; row++) {
                        scolumns = rows[row].summaryColumns, obj = {};
                        for (var col = 0, clen = scolumns.length; col < clen; col++) {
                            summaryData = collection[indx].items.level ? collection[indx].items.records : collection[indx].items;
                            obj[scolumns[col].dataMember] = this._getSummaryValues(scolumns[col], summaryData);
                        }
                        agg.push(obj);
                    }
                }
            }
        },
    };
})(jQuery, Syncfusion);;
(function ($, ej, undefined) {
    ej.gridFeatures = ej.gridFeatures || {};
    ej.gridFeatures.dragAndDrop = {
        _headerCellgDragDrop: function () {
            var proxy = this;
            this.dragHeaderElement();
            var $droppableElements = this.element.children("div.e-groupdroparea");
            $droppableElements.ejDroppable({
                accept: $droppableElements,
                drop: function (event, ui) {
                    if (ej.isNullOrUndefined(ui.helper) || !ui.helper.is(":visible"))
                        return;
                    var column = proxy.getColumnByHeaderText($.trim(ui.helper.text()));
                    ui.helper.remove();
                    if ($.inArray(column["headerText"], proxy._disabledGroupableColumns) != -1)
                        return;
                    if (!ej.isNullOrUndefined(column.field))
                        proxy.groupColumn(column.field);
                }
            });
        },
        _headerCellreorderDragDrop: function () {
            var proxy = this;
            this.dragHeaderElement();
            var $droppableElements = this.element.find(".e-headercelldiv");
            $droppableElements.ejDroppable({
                accept: $droppableElements,
                drop: function (event, ui) {
                    if (ej.isNullOrUndefined(ui.helper) || !ui.helper.is(":visible"))
                        return;
                    var column = proxy.getColumnByHeaderText($.trim(ui.helper.text()));
                    ui.helper.remove();
                    if ($.inArray(column["headerText"], proxy._disabledGroupableColumns) != -1)
                        return;
                    if (!ej.isNullOrUndefined(column.field)) {
                        var toColumn = proxy.getColumnByHeaderText($.trim($(event.dropTarget).text()));
                        proxy.reorderColumns(column.field, toColumn.field);

                    }
                }
            });
        },

        dragHeaderElement: function () {
            var proxy = this;
            var $dragableElements = this.element.children("div.e-gridheader").find("th.e-headercell").not(".e-headertemplate");
            var $visualElement = ej.buildTag('div.e-cloneproperties e-grid', "", { 'height': '20px', 'z-index': 2 }),column;
            //header element columnDrag
            $dragableElements.ejDraggable({
                cursorAt: { top: -50, left: -2 },
                helper: function (event, ui) {
                    if (proxy.element.find(".e-dragclone").length > 0) proxy.element.find(".e-dragclone").remove();
                    var $th;
                    if ($(event.sender.target).hasClass("e-headercell"))
                        $th = $(event.sender.target);
                    else
                        $th = $(event.sender.target).closest("th");
                    column = proxy.getColumnByField($th.find(".e-headercelldiv").attr("ej-mappingname"));
                    return $visualElement.text($th.text()).clone().width($th.outerWidth() + 2).height($th.height() + 2).css({ "font-size": parseInt(($th.height() + 3) / 2) }).addClass("e-dragclone").appendTo(proxy.element);
                },
                dragStart: function (args) {
                    var target = args.target;
                    var data = { target: target, draggableType: "headercell", column: column };
                    if (proxy._resizer != null && proxy._resizer._expand) {
                        $(".e-dragclone").remove();
                        return false;
                    }
                    proxy._dragActive = true;
                    var $target = proxy.element.find(".e-groupdroparea");
                    if (proxy.model.allowGrouping)
                        proxy.expandGroupDropArea();
                    if (proxy._trigger("columnDragStart", data))
                        return false;
                },
                drag: function (args) {
                    var $target = $(args.target);
                    var data = { target: $target, draggableType: "headercell", column: column };
                    if (proxy._trigger("columnDrag", data))
                        return false;

                    if ($target.hasClass('e-headercelldiv') && proxy.model.allowReordering) {
                        document.body.style.cursor = '';
                        $target.addClass("e-allowDrop");
                        proxy.getHeaderTable().find(".e-reorderindicate").removeClass("e-reorderindicate");
                        $target.parent().addClass("e-reorderindicate");
                    }
                    if ($target.hasClass('e-groupdroparea') || $target.closest('.e-groupdroparea').length) {
                        document.body.style.cursor = '';
                        $target.addClass("e-allowDrop");
                    } else
                        document.body.style.cursor = 'not-allowed';
                },
                dragStop: function (args) {
                    if (!args.element.dropped) {
                        var $target = $(args.target);
                        var data = { target: $target, draggableType: "headercell", column: column };
                        proxy._trigger("columnDrop", data);
                        proxy.element.find(".e-groupdroparea").removeClass("e-hover");
                        proxy.getHeaderTable().find(".e-columnheader").find(".e-headercellactive").removeClass("e-headercellactive").removeClass("e-active");
                        $(".e-dragclone").remove();
                        proxy._dragActive = false;
                        proxy.getHeaderTable().find(".e-reorderindicate").removeClass("e-reorderindicate");
                        if (proxy.model.allowGrouping)
                            proxy.collapseGroupDropArea();
                        document.body.style.cursor = '';
                        $(proxy._Indicator).css('display', 'none');
                    }
                }
            });
        },
        _groupHeaderCelldrag: function () {
            //grouped header cell drag.
            var $visualElement = ej.buildTag('div.e-cloneproperties e-grid', "", { 'height': '20px', 'z-index': 2 });
            var proxy = this;
            var $groupedHeaderCells = this.element.children(".e-groupdroparea").find(".e-groupheadercell");
            $groupedHeaderCells.ejDraggable({
                cursorAt: { top: -35, left: -2 },
                helper: function (event, ui) {
                    var $div = $(event.sender.target).closest(".e-grid-icon");
                    return $visualElement.text($(event.sender.target).closest(".e-groupheadercell").text()).clone().width($div.width() + 2).height($div.height() + 2).addClass("e-dragclone").appendTo(proxy.element);
                },
                dragStart: function (args) {
                    var target = args.target;
					args.model.cursorAt = { top: -35, left: -2 };
                    var data = { target: target, draggableType: "groupheadercell" };
                    if (proxy._trigger("columnDragStart", data))
                        return false;
                },
                drag: function (args) {
                    $(".Sibling").remove();
                    var $target = $(args.target);
                    var data = { target: $target, draggableType: "groupheadercell" };
                    if (proxy._trigger("columnDrag", data))
                        return false;
                    if ($target.closest('div.e-gridcontent').length) {
                        document.body.style.cursor = '';
                        $target.addClass("e-allowDrop");
                    } else
                        document.body.style.cursor = 'not-allowed';
                },
                dragStop: function (args) {
                    if (!args.element.dropped) {
                        var $target = $(args.target);
                        var data = { target: $target, draggableType: "groupheadercell" };
                        proxy._trigger("columnDrop", data);
                        $(".e-dragclone").remove();
                        document.body.style.cursor = '';
                    }
                }
            });

            //grid content drop
            var $contentDroppableElements = this.element.children(".e-gridcontent");
            $contentDroppableElements.ejDroppable({
                accept: proxy.element.children("div.e-groupdroparea").find(".e-groupheadercell"),
                drop: function (event, ui) {
                    if (ej.isNullOrUndefined(ui.helper) || !ui.helper.is(":visible") || !ui.draggable.hasClass("e-groupheadercell"))
                        return;
                    var field = proxy.getFieldNameByHeaderText($.trim(ui.draggable.text()));
                    ui.helper.remove();
                    if (!ej.isNullOrUndefined(field))
                        proxy.ungroupColumn(field);


                }
            });
        },
    };
})(jQuery, Syncfusion);;
(function ($, ej, undefined) {
    ej.gridFeatures = ej.gridFeatures || {};
    ej.gridFeatures.selection = {
        /**
         * Select rows in grid.
         * @alias ejGrid#selectRows
         * @return jQuery
         * @param { number } fromIndex It is used to set the starting index of row for selecting rows.
         * @param { number } toIndex It is used to set the ending index of row for selecting rows.
         * @example 
         * &lt;script&gt;
         * // Create grid object.
         * var gridObj = $("#Grid").data("ejGrid");
         * // Selects rows based on the given index
         * gridObj.selectRows(1, 4); 
         * &lt;/script&gt;
         * @example 
         * &lt;script&gt;
         * // Selects rows based on the given index
         * $("#Grid").ejGrid("selectRows", 1, 4);
         * &lt;/script&gt;
         */
        selectRows: function (rowIndex, toIndex) {
            if (!this.model.allowSelection)
                return false;
            var $gridRows = $(this.getRows());
            var args = {};
            if (this._selectedRow() != -1)
                args = { rowIndex: this._selectedRow(), row: $gridRows.eq(this._selectedRow()), data: this._currentJsonData[this._selectedRow()] };

            if (this._trigger("rowSelecting", args))
                return;
            if ((this.model.editSettings.allowEditing || this.model.editSettings.allowAdding) && this.model.isEdit && this.model.enableAutoSaveOnSelectionChange) {
                if (this.model.editSettings.editMode == "externalform" || this.model.editSettings.editMode == "externalformtemplate")
                    $("#" + this._id + "_externalEdit").css("display", "none");
                if (this.endEdit())
                    return;
                var $gridRows = $(this.getRows());
            }
            if (ej.isNullOrUndefined(toIndex) || ej.isNullOrUndefined(rowIndex)) {
                rowIndex = ej.isNullOrUndefined(rowIndex) ? toIndex : rowIndex;
                switch (this.model.selectionType) {
                    case ej.Grid.SelectionType.Multiple:
                        if (this.multiSelectCtrlRequest) {
                            var selectedRowIndex = $.inArray(rowIndex, this._selectedRowsIndexes);
                            selectedRowIndex != -1 && this.clearSelection(rowIndex) && this._selectedRowsIndexes.splice(selectedRowIndex, 0);
                            if (selectedRowIndex == -1) {
                                this._selectedRowsIndexes.push(rowIndex);
                                this.getRowByIndex(rowIndex).attr("aria-selected", "true").find('.e-rowcell, .e-detailrowcollapse, .e-detailrowexpand').addClass("e-selectionbackground e-active");
                            }
                            break;
                        }
                    case ej.Grid.SelectionType.Single:
                        this.clearSelection();
                        this.clearColumnSelection();
                        this._selectedRowsIndexes = [];
                        this._selectedRowsIndexes.push(rowIndex);
                        this.getRowByIndex(rowIndex).attr("aria-selected", "true").find('.e-rowcell, .e-detailrowcollapse, .e-detailrowexpand').addClass("e-selectionbackground e-active");
                        break;
                }
            } else {
                if (this.model.selectionType == ej.Grid.SelectionType.Multiple) {
                    this.clearSelection();
                    this.clearColumnSelection();
                    this._selectedRowsIndexes = [];
                    rows = rowIndex - toIndex < 0 ? this.getRowByIndex(rowIndex, toIndex + 1) : this.getRowByIndex(toIndex, rowIndex + 1);
                    for (var i = 0; i < rows.length; i++)
                        this._selectedRowsIndexes.push(rows[i].rowIndex);
                    $(rows).attr("aria-selected", "true").find('.e-rowcell, .e-detailrowcollapse, .e-detailrowexpand').addClass("e-selectionbackground e-active");
                }
            }
            this._previousIndex = this._selectedRowsIndexes.length ? rowIndex : undefined;
            if (this._selectedRow() !== rowIndex)
                this._selectedRow(rowIndex);
            var args = { rowIndex: this._selectedRow(), row: this.getRowByIndex(this._selectedRow()), data: this._currentJsonData[this._selectedRow()] };
            this._trigger("rowSelected", args);
            // this.multiSelectCtrlRequest = false;
        },

        /**
         * Select cells in grid.
         * @alias ejGrid#selectCells
         * @return jQuery
         * @param { object } rowCellIndexes It is used to set the starting index of row and indexes of cells for that corresponding row for selecting cells.
         * @example 
         * &lt;script&gt;
         * // Create grid object.
         * var gridObj = $("#Grid").data("ejGrid");
         * // Selects cells based on the given index
         * gridObj.selectCells([[1, [4, 3, 2]]]); 
         * &lt;/script&gt;
         * @example 
         * &lt;script&gt;
         * // Selects cells based on the given index
         * $("#Grid").ejGrid("selectCells", [[1, [4, 3, 2]]]);
         * &lt;/script&gt;
         */
        selectCells: function (rowCellIndexes) {
            if (!this.model.allowSelection && !this._allowcellSelection)
                return false;
            switch (this.model.selectionType) {
                case ej.Grid.SelectionType.Multiple:
                    if (this.multiSelectCtrlRequest) {
                        var selectedCellIndex = $.inArray(rowCellIndexes[0][0], this._rowIndexesColl);
                        if (selectedCellIndex != -1)
                            this._selectedRowCellIndexes[selectedCellIndex].cellIndex.push(parseInt(rowCellIndexes[0][1].toString()));
                        else {
                            if ($.inArray(rowCellIndexes[0][0], this._rowIndexesColl) == -1)
                                this._rowIndexesColl.push(rowCellIndexes[0][0]);
                            this._selectedRowCellIndexes.push({ rowIndex: rowCellIndexes[0][0], cellIndex: rowCellIndexes[0][1] });
                        }
                        $(this.getRowByIndex(rowCellIndexes[0][0]).find(".e-rowcell:eq(" + rowCellIndexes[0][1] + ")")).addClass("e-cellselectionbackground e-activecell");
                        break;
                    }
                    else if (this._multiSelectShiftRequest) {
                        this.clearCellSelection();
                        this.clearColumnSelection();
                        this._selectedRowCellIndexes = [];
                        var previousRowIndex = this._previousRowCellIndex[0][0];
                        var previousCellIndex = parseInt(this._previousRowCellIndex[0][1]);
                        var currentRowIndex = rowCellIndexes[0][0];
                        var currentCellIndex = parseInt(rowCellIndexes[0][1]);
                        var newCellColl = [];
                        var min, max;
                        if (currentRowIndex > previousRowIndex) {
                            for (var i = previousRowIndex; i <= currentRowIndex; i++) {
                                newCellColl = [];
                                min = i == previousRowIndex ? previousCellIndex : 0;
                                max = i == currentRowIndex ? currentCellIndex : this.model.columns.length;
                                for (var j = min; j <= max; j++) {
                                    newCellColl.push(j);
                                    $(this.getRowByIndex(i).find(".e-rowcell:eq(" + j + ")")).addClass("e-cellselectionbackground e-activecell");
                                }
                                this._selectedRowCellIndexes.push({ rowIndex: i, cellIndex: newCellColl });
                            }
                        } else {
                            for (var i = previousRowIndex; i >= currentRowIndex; i--) {
                                newCellColl = [];
                                min = i == previousRowIndex ? previousCellIndex : this.model.columns.length;
                                max = i == currentRowIndex ? currentCellIndex : 0;
                                for (var j = min; j >= max; j--) {
                                    newCellColl.push(j);
                                    $(this.getRowByIndex(i).find(".e-rowcell:eq(" + j + ")")).addClass("e-cellselectionbackground e-activecell");
                                }
                                this._selectedRowCellIndexes.push({ rowIndex: i, cellIndex: newCellColl });
                            }
                        }
                        break;
                    }
                case ej.Grid.SelectionType.Single:
                    this.clearCellSelection();
                    this.clearColumnSelection();
                    this._selectedRowCellIndexes = [];
                    if ($.inArray(rowCellIndexes[0][0], this._rowIndexesColl) == -1)
                       this._rowIndexesColl.push(rowCellIndexes[0][0]);
                    this._selectedRowCellIndexes.push({ rowIndex: rowCellIndexes[0][0], cellIndex: rowCellIndexes[0][1] });
                    $(this.getRowByIndex(rowCellIndexes[0][0]).find(".e-rowcell:eq(" + rowCellIndexes[0][1] + ")")).addClass("e-cellselectionbackground e-activecell");
                    break;
            }
            if (!this._multiSelectShiftRequest) this._previousRowCellIndex = rowCellIndexes;
        },

        /**
         * Select columns in grid.
         * @alias ejGrid#selectColumns
         * @return jQuery
         * @param { number } fromIndex It is used to set the starting index of column for selecting columns.
         * @example 
         * &lt;script&gt;
         * // Create grid object.
         * var gridObj = $("#Grid").data("ejGrid");
         * // Selects columns based on the given index
         * gridObj.selectColumns(1,4); 
         * &lt;/script&gt;
         * @example 
         * &lt;script&gt;
         * // Selects columns based on the given index
         * $("#Grid").ejGrid("selectColumns", 1, 4);
         * &lt;/script&gt;
         */
        selectColumns: function (columnIndex, toIndex) {
            if (!this.model.allowcolumnSelection && !this.model.allowSelection)
                return false;
            var gridRows = this.getRows();
            if (ej.isNullOrUndefined(toIndex) || ej.isNullOrUndefined(columnIndex)) {
                columnIndex = ej.isNullOrUndefined(columnIndex) ? toIndex : columnIndex;
                switch (this.model.selectionType) {
                    case ej.Grid.SelectionType.Multiple:
                        if (this.multiSelectCtrlRequest) {
                            var selectedColumnIndex = $.inArray(columnIndex, this._selectedColumnIndexes);
                            selectedColumnIndex != -1 && this.clearColumnSelection(columnIndex) && this._selectedColumnIndexes.splice(selectedColumnIndex, 0);
                            if (selectedColumnIndex == -1) {
                                this._selectedColumnIndexes.push(columnIndex);
                                for (var i = 0; i < gridRows.length; i++) {
                                    $(gridRows[i].cells[columnIndex]).addClass("e-columnselection");
                                }
                                $(this.getHeaderTable().find("th.e-headercell:visible")[columnIndex]).addClass("e-columnselection");
                            }
                            break;
                        }
                    case ej.Grid.SelectionType.Single:
                        this.clearSelection();
                        this.clearCellSelection();
                        this.clearColumnSelection();
                        this._selectedColumnIndexes = [];
                        this._selectedColumnIndexes.push(columnIndex);
                        for (var i = 0; i < gridRows.length; i++) {
                            $(gridRows[i].cells[columnIndex]).addClass("e-columnselection");
                        }
                        $(this.getHeaderTable().find("th.e-headercell:visible")[columnIndex]).addClass("e-columnselection");
                        break;
                }
            } else {
                if (this.model.selectionType == ej.Grid.SelectionType.Multiple) {
                    this.clearColumnSelection();
                    this._selectedColumnIndexes = [];
                    var startIndex = columnIndex > toIndex ? toIndex : columnIndex;
                    var endIndex = columnIndex > toIndex ? columnIndex + 1 : toIndex + 1;
                    for (var i = startIndex; i < endIndex; i++) {
                        for (var j = 0; j < gridRows.length; j++) {
                            $(gridRows[j].cells[i]).addClass("e-columnselection");
                        }
                        $(this.getHeaderTable().find("th.e-headercell:visible")[i]).addClass("e-columnselection");
                        this._selectedColumnIndexes.push(i);
                    }
                }
            }
            this._previousColumnIndex = this._selectedColumnIndexes.length ? columnIndex : undefined;
        },
        /**
         * Clear all the row selection or at specific row selection based on the index provided
         * @alias ejGrid#clearSelection
         * @return jQuery
         * @param {number} [index] If index of the row is specified then it will remove the selection from the particular row else it will clears all of the row selection
         * @example 
         * &lt;script&gt;
         * // Create grid object.
         * var gridObj = $("#Grid").data("ejGrid");
         * gridObj.clearSelection(2);  // Removes the selection based on the row index
         * gridObj.clearSelection();  // clears all of the row selection
         * &lt;/script&gt;
         * @example 
         * &lt;script&gt;
         * // Removes the selection based on the row index
         * $("#Grid").ejGrid("clearSelection", 2);   
         * // clears all of the row selection
         * $("#Grid").ejGrid("clearSelection");        
         * &lt;/script&gt;
         */
        clearSelection: function (index) {
            if (this._selectedRow() >= -1) {
                var $gridRows = $(this.getRows()), index;
                if (!ej.isNullOrUndefined(index)) {
                    this.getRowByIndex(index).removeAttr("aria-selected").find(".e-selectionbackground").removeClass("e-selectionbackground").removeClass("e-active");
                    index = $.inArray(index, this._selectedRowsIndexes);
                    if (index != -1)
                        this._selectedRowsIndexes.splice(index, 1);
                } else {
                    if (this.model.scrollSettings.frozenColumns > 0)
                        $gridRows = $($gridRows[0]).add($gridRows[1]);
                    $gridRows.removeAttr("aria-selected").find(".e-rowcell, .e-detailrowcollapse, .e-detailrowexpand").removeClass("e-selectionbackground").removeClass("e-active");
                }
                if (!this._selectedRowsIndexes.length)
                    this._selectedRow(-1);
            }
            return true;
        },
        /**
         * It is used to clear all the cell selection.
         * @alias ejGrid#clearCellSelection
         * @return jQuery
         * @example 
         * &lt;script&gt;
         * // Create grid object.
         * var gridObj = $("#Grid").data("ejGrid");
         * gridObj.clearCellSelection();  // clears all of the cell selection
         * &lt;/script&gt;
         * @example 
         * &lt;script&gt;         
         * // clears all of the cell selection
         * $("#Grid").ejGrid("clearCellSelection");        
         * &lt;/script&gt;
         */
        clearCellSelection: function () {
            if (this._allowcellSelection) {
                var $gridRows = $(this.getRows());
                $gridRows.find(".e-rowcell, .e-detailrowcollapse, .e-detailrowexpand").removeClass("e-cellselectionbackground").removeClass("e-activecell");
                this._selectedRowCellIndexes = [];
                this._rowIndexesColl = [];
            }
            return true;
        },

        /**
        * It is used to clear all the row selection or at specific row selection based on the index provided.
        * @alias ejGrid#clearColumnSelection
        * @return jQuery
        * @param {number} [index] If index of the column is specified then it will remove the selection from the particular column else it will clears all of the column selection
        * @example 
        * &lt;script&gt;
        * // Create grid object.
        * var gridObj = $("#Grid").data("ejGrid");
        * gridObj.clearColumnSelection(2);  // Removes the selection based on the column index
        * gridObj.clearColumnSelection();  // clears all of the column selection
        * &lt;/script&gt;
        * @example 
        * &lt;script&gt;
        * // Removes the selection based on the column index
        * $("#Grid").ejGrid("clearColumnSelection", 2);   
        * // clears all of the column selection
        * $("#Grid").ejGrid("clearColumnSelection");        
        * &lt;/script&gt;
        */
        clearColumnSelection: function (index) {
            if (this._allowcolumnSelection) {
                var $gridRows = $(this.getRows());
                if (!ej.isNullOrUndefined(index)) {
                    for (var i = 0; i < $gridRows.length; i++) {
                        $($gridRows[i].cells[index]).removeClass("e-columnselection");
                    }
                    $(this.getHeaderTable().find("th.e-headercell:visible")[index]).removeClass("e-columnselection");
                    this._selectedColumnIndexes.splice(0, index);

                } else {
                    $gridRows.find(".e-rowcell").removeClass("e-columnselection");
                    $(this.getHeaderTable().find("th.e-headercell:visible")).removeClass("e-columnselection");
                    this._selectedColumnIndexes = [];
                }
            }
            return true;
        },
        /**
         * Get the selected records details in grid.
         * @alias ejGrid#getSelectedRecords
         * @return selectedRecords
         * @example 
         * &lt;script&gt;
         * // Create grid object.
         * var gridObj = $("#Grid").data("ejGrid");
         * // Gets the selected row list
         * gridObj.getSelectedRecords();
         * &lt;/script&gt;
         * @example 
         * &lt;script&gt;
         * // Gets the selected row list
         * $("#Grid").ejGrid("getSelectedRecords");        
         * &lt;/script&gt;
         */
        getSelectedRecords: function () {
            var records = [];
            for (var i = 0; i < this._selectedRowsIndexes.length; i++) {
                if (this._selectedRowsIndexes[i] != -1)
                    records.push(this._currentJsonData[this._selectedRowsIndexes[i]]);
            }
            return records;
        },
        _setCurrentRow: function (requestType) {
            if (requestType == ej.Grid.Actions.Refresh || requestType == ej.Grid.Actions.Filtering || requestType == ej.Grid.Actions.Sorting || requestType == ej.Grid.Actions.Delete || requestType == ej.Grid.Actions.Save || requestType == ej.Grid.Actions.Cancel || requestType == ej.Grid.Actions.Paging) {
                this._selectedRow(-1);
                this._selectedRowsIndexes = [];
            }
        },
        _renderMultiTouchDialog: function () {
            this._customPop = ej.buildTag("div.e-gridpopup", "", { display: "none" });
            var $content = ej.buildTag("div.e-content"), $downTail = ej.buildTag("div.e-downtail e-tail");
            if (this.model.allowMultiSorting) {
                var $selElement = ej.buildTag("span.e-sortdirect e-icon");
                $content.append($selElement);
            }
            if (this.model.selectionType == ej.Grid.SelectionType.Multiple) {
                var $selElement = ej.buildTag("span.e-rowselect e-icon");
                $content.append($selElement);
            }
            this._customPop.append($content);
            this._customPop.append($downTail);
            this.element.append(this._customPop);
        },
    };
})(jQuery, Syncfusion);;
(function ($, ej, undefined) {
	ej.gridFeatures = ej.gridFeatures || {};
   /**   
    * @classdesc The grid can be easily configured to the DOM element, such as div. you can create a grid with a highly customizable look and feel.
    * @class ejGrid
    * @param {object} options - settings for grid
    * @requires jQuery
    * @requires jquery.easing.min.js
    * @requires jquery.globalize.min.js    
    * @requires jsrender.min.js
    * @requires ej.core.js
    * @requires ej.data.js
    * @requires ej.grid.js
    * @requires ej.pager.js
    * @requires ej.scroller.js
    * @requires ej.waitingpopup.js
    * @requires ej.radiobutton.js
    * @requires ej.dropdownlist.js
    * @requires ej.dialog.js
    * @requires ej.button.js
    * @requires ej.autocomplete.js
    * @requires ej.checkbox.js
    * @requires ej.datepicker.js
    * @requires ej.datetimepicker.js
    * @requires ej.editor.js
    * @requires ej.toolbar.js
    * @example
    * &lt;div id="Grid"&gt;&lt;/div&gt; 
    * &lt;script&gt;
    * // Create Grid
    * $('#Grid').ejGrid({
    *     dataSource: window.gridData
    * });         
    * &lt;/script&gt;
    */
    ej.widget("ejGrid", "ej.Grid", /** @lends ejGrid# */ {
        //Root Css Class
        _rootCSS: "e-grid",
        // widget element will be automatically set in this
        element: null,
        validTags: ["div"],
        // user defined model will be automatically set in this
        model: null,
        keyConfigs: /** @lends ejGrid# */{
            focus: "e",
            insertRecord: "45", //Insert
            deleteRecord: "46", // delete
            editRecord: "113", //F2
            saveRequest: "13", // enter
            cancelRequest: "27", //Esc
            nextPage: "34", // PgDn
            previousPage: "33", // PgUp
            lastPage: "ctrl+alt+34", //"CtrlAltPgDn",
            firstPage: "ctrl+alt+33", //"CtrlPlusAltPlusPgUp",
            nextPager: "alt+34", //"AltPlusPgDown",
            previousPager: "alt+33", //"AltPlusPgUp",
            firstCellSelection: "36", //"Home",
            lastCellSelection: "35", //"End",
            firstRowSelection: "ctrl+36", //"CtrlPlusHome",
            lastRowSelection: "ctrl+35", //"CtrlPlusEnd",
            upArrow: "38", //Up arrow
            downArrow: "40", //Down arrow
            rightArrow: "39", //Right arrow
            leftArrow: "37", //Left arrow
            moveCellRight: "9", //tab
            moveCellLeft: "shift+9", //shifttab
            selectedGroupExpand: "alt+40", //"AltPlusDownArrow",
            totalGroupExpand: "ctrl+40", //"CtrlPlusDownArrow",
            selectedGroupCollapse: "alt+38", //"AltPlusUpArrow",
            totalGroupCollapse: "ctrl+38" //"CtrlPlusUpArrow",
        },
        _ignoreOnPersist: [
            "query", "isEdit", "toolbarClick", "queryCellInfo", "mergeCellInfo", "currentViewData", "enableAltRow", "enableRTL","contextClick","contextOpen",
            "rowDataBound", "rowTemplate", "detailsDataBound", "detailsTemplate", "childGrid", "summaryRows", "toolbarSettings",
            "editSettings", "allowMultiSorting", "enableAutoSaveOnSelectionChange", "locale", "allowScrolling", "allowCellMerging",
            "allowTextWrap", "cssClass", "dataSource", "groupSettings.enableDropAreaAnimation", "enableRowHover", "showSummary",
            "enableHeaderHover", "allowKeyboardNavigation", "scrollSettings.frozenRows", "scrollSettings.frozenColumns", "enableTouch", "editSettings.rowPosition", "contextMenuSettings.enableContextMenu",
        ],
        _ignoreOnExport: [
            "isEdit", "toolbarClick", "query", "queryCellInfo", "selectionType", "currentViewData", "enableAltRow", "enableRTL", "rowDataBound", "rowTemplate",
            "detailsDataBound", "detailsTemplate", "editSettings", "pageSettings", "enableAutoSaveOnSelectionChange", "localization", "allowScrolling",
            "cssClass", "dataSource", "groupSettings.enableDropAreaAnimation", "enableRowHover", "allowSummary",
            "enableHeaderHover", "allowKeyboardNavigation"
        ],
        observables: ["dataSource", "selectedRowIndex", "pageSettings.currentPage"],
        _dataSource: ej.util.valueFunction("dataSource"),
        _selectedRow: ej.util.valueFunction("selectedRowIndex"),
        _currentPage: ej.util.valueFunction("pageSettings.currentPage"),
        // default model
        defaults: /** @lends ejGrid# */ {

            /**		
             * This specifies the grid to show the paginated data. Also enables pager control at the bottom of grid for dynamic navigation through data source. Paging can be further customized through “pageSettings” property.
             * @default false
             * @type {boolean}
             * @example            
             * &lt;div id="Grid"&gt;&lt;/div&gt; 
             * &lt;script&gt;
             * $("#Grid").ejGrid({
             *     dataSource:window.gridData,
             *     allowPaging:true                      
             * });
             * &lt;/script&gt;                 
             */
            allowPaging: false,
            /**		
             * Gets or sets a value that indicates whether to enable column chooser on grid. On enabling feature able to show/hide grid columns                       
             * @default false
             * @type {boolean}
             * @example            
             * &lt;div id="Grid"&gt;&lt;/div&gt; 
             * &lt;script&gt;
             * $("#Grid").ejGrid({
             *     dataSource:window.gridData,
             *     showColumnChooser:true                      
             * });
             * &lt;/script&gt;                 
             */
            showColumnChooser: false,
            /**		
             * Gets or sets a value that indicates whether to  enable the dynamic sorting behaviour on grid data. Sorting can be done through clicking on particular column header.
             * @default false
             * @type {boolean}
             * @example  
             * &lt;div id="Grid"&gt;&lt;/div&gt; 
             * &lt;script&gt;
             * $("#Grid").ejGrid({
             *    dataSource:window.gridData,
             *    allowSorting:true                       
             * });
             * &lt;/script&gt; 
             */
            allowSorting: false,
            /**		
            * Gets or sets a value that indicates stacked header should be shown on grid layout when the property “stackedHeaderRows” is set.  
            * @default false
            * @type {boolean}
            * @example  
            * &lt;div id="Grid"&gt;&lt;/div&gt; 
            * &lt;script&gt;
            * $("#Grid").ejGrid({
            *    dataSource:window.gridData,
            *    allowSorting:true,
            *    showStackedHeader:true,
            *    stackedHeaderRows:[{stackedHeaderColumn:[{stackedHeaderText:"ID & Freight",column:"CustomerID"}
			*	 ,{stackedHeaderText:"Frieght",column:"Freight,EmployeeID,OrderDate"}
			*	 ,{stackedHeaderText:"Date & Location Top Level",column:"ShipCity"}
			*	   ]}
			*	  ],
            *    columns: [
            *       { field: "CustomerID", headerText: "Customer ID", width: 80 },
            *        { field: "EmployeeID", headerText: "Employee ID", width: 75, textAlign: ej.TextAlign.Right },
            *       { field: "Freight", width: 75, format: "{0:C}", textAlign: ej.TextAlign.Right },
            *        { field: "OrderDate", headerText: "Order Date", width: 80, format: "{0:MM/dd/yyyy}", textAlign: ej.TextAlign.Right },
            *       { field: "ShipCity", headerText: "Ship City", width: 110 }
            *       ]
            * });
            * &lt;/script&gt; 
            */
            showStackedHeader: false,
            /**		
            * Gets or sets an object that indicates to managing the collection of stacked header rows for the grid.
            * @remark
            * We must set the showStackedHeader as true
            * @default []
            * @name ejGrid#stackedHeaderRows
            * @type {array}
            * @example  
            * &lt;div id="Grid"&gt;&lt;/div&gt; 
            * &lt;script&gt;
            * $("#Grid").ejGrid({
            *    dataSource:window.gridData,
            *    allowSorting:true,
            *    showStackedHeader:true,
            *    stackedHeaderRows:[{stackedHeaderColumn:[{stackedHeaderText:"ID & Freight",column:"CustomerID"}
			*	 ,{stackedHeaderText:"Frieght",column:"Freight,EmployeeID,OrderDate"}
			*	 ,{stackedHeaderText:"Date & Location Top Level",column:"ShipCity"}
			*	   ]}
			*	  ],
            *    columns: [
            *       { field: "CustomerID", headerText: "Customer ID", width: 80 },
            *        { field: "EmployeeID", headerText: "Employee ID", width: 75, textAlign: ej.TextAlign.Right },
            *       { field: "Freight", width: 75, format: "{0:C}", textAlign: ej.TextAlign.Right },
            *        { field: "OrderDate", headerText: "Order Date", width: 80, format: "{0:MM/dd/yyyy}", textAlign: ej.TextAlign.Right },
            *       { field: "ShipCity", headerText: "Ship City", width: 110 }
            *       ]
            * });
            * &lt;/script&gt; 
            */
            /**		
             * Gets or sets a value that indicates whether to add stacked header columns into the stacked header rows
             * @remarks
             * We must enable the showStackedHeader 
             * @name ejGrid#stackedHeaderRows->stackedHeaderColumn
             * @default []
             * @type {array}
             * @example  
             * &lt;div id="Grid"&gt;&lt;/div&gt; 
             * &lt;script&gt;
             * $("#Grid").ejGrid({
             *    dataSource:window.gridData,
             *    allowSorting:true,
             *    showStackedHeader:true,
             *    stackedHeaderRows:[{stackedHeaderColumn:[{stackedHeaderText:"ID & Freight",column:"CustomerID"}
             *	 ,{stackedHeaderText:"Frieght",column:"Freight,EmployeeID,OrderDate"}
             *	 ,{stackedHeaderText:"Date & Location Top Level",column:"ShipCity"}
             *	   ]}
             *	  ],
             *    columns: [
             *       { field: "CustomerID", headerText: "Customer ID", width: 80 },
             *        { field: "EmployeeID", headerText: "Employee ID", width: 75, textAlign: ej.TextAlign.Right },
             *       { field: "Freight", width: 75, format: "{0:C}", textAlign: ej.TextAlign.Right },
             *        { field: "OrderDate", headerText: "Order Date", width: 80, format: "{0:MM/dd/yyyy}", textAlign: ej.TextAlign.Right },
             *       { field: "ShipCity", headerText: "Ship City", width: 110 }
             *       ]
             * });
             * &lt;/script&gt; 
             **/
             /**		
             * Gets or sets a value that indicates the header text for the particular stacked header column.
             * @remarks
             * We must enable the showStackedHeader 
             * @name ejGrid#stackedHeaderRows->stackedHeaderColumn->stackedHeaderText
             * @default null
             * @type {string}
             * @example  
             * &lt;div id="Grid"&gt;&lt;/div&gt; 
             * &lt;script&gt;
             * $("#Grid").ejGrid({
             *    dataSource:window.gridData,
             *    allowSorting:true,
             *    showStackedHeader:true,
             *    stackedHeaderRows:[{stackedHeaderColumn:[{stackedHeaderText:"ID & Freight",column:"CustomerID"}
             *	 ,{stackedHeaderText:"Frieght",column:"Freight,EmployeeID,OrderDate"}
             *	 ,{stackedHeaderText:"Date & Location Top Level",column:"ShipCity"}
             *	   ]}
             *	  ],
             *    columns: [
             *       { field: "CustomerID", headerText: "Customer ID", width: 80 },
             *        { field: "EmployeeID", headerText: "Employee ID", width: 75, textAlign: ej.TextAlign.Right },
             *       { field: "Freight", width: 75, format: "{0:C}", textAlign: ej.TextAlign.Right },
             *        { field: "OrderDate", headerText: "Order Date", width: 80, format: "{0:MM/dd/yyyy}", textAlign: ej.TextAlign.Right },
             *       { field: "ShipCity", headerText: "Ship City", width: 110 }
             *       ]
             * });
             * &lt;/script&gt; 
             **/
            /**		
            * Gets or sets a value that indicates the header text for the particular stacked header column.
            * @remarks
            * We must enable the showStackedHeader 
            * @name ejGrid#stackedHeaderRows->stackedHeaderColumn->column
            * @default null
            * @type {string}
            * @example  
            * &lt;div id="Grid"&gt;&lt;/div&gt; 
            * &lt;script&gt;
            * $("#Grid").ejGrid({
            *    dataSource:window.gridData,
            *    allowSorting:true,
            *    showStackedHeader:true,
            *    stackedHeaderRows:[{stackedHeaderColumn:[{stackedHeaderText:"ID & Freight",column:"CustomerID"}
            *	 ,{stackedHeaderText:"Frieght",column:"Freight,EmployeeID,OrderDate"}
            *	 ,{stackedHeaderText:"Date & Location Top Level",column:"ShipCity"}
            *	   ]}
            *	  ],
            *    columns: [
            *       { field: "CustomerID", headerText: "Customer ID", width: 80 },
            *        { field: "EmployeeID", headerText: "Employee ID", width: 75, textAlign: ej.TextAlign.Right },
            *       { field: "Freight", width: 75, format: "{0:C}", textAlign: ej.TextAlign.Right },
            *        { field: "OrderDate", headerText: "Order Date", width: 80, format: "{0:MM/dd/yyyy}", textAlign: ej.TextAlign.Right },
            *       { field: "ShipCity", headerText: "Ship City", width: 110 }
            *       ]
            * });
            * &lt;/script&gt; 
            **/
			stackedHeaderRows: [],
            /**		
             * Gets or sets a value that indicates whether to enable dynamic filtering behaviour on grid. Filtering can be used to limit the records displayed using required criteria and this can be further customized through “filterSettings” property
             * @default false
             * @type {boolean}
             * @example  
             * &lt;div id="Grid"&gt;&lt;/div&gt; 
             * &lt;script&gt;
             * $("#Grid").ejGrid({
             *    dataSource:window.gridData,
             *    allowFiltering:true                       
             * });
             * &lt;/script&gt;
             */

            allowFiltering: false,
            /**		
             * Gets or sets a value that indicates whether user can select rows on grid. On enabling feature, selected row will be highlighted. 
             * @default true
             * @type {boolean}
             * @example 
             * &lt;div id="Grid"&gt;&lt;/div&gt; 
             * &lt;script&gt;
             * $("#Grid").ejGrid({
             *    dataSource:window.gridData,
             *    allowSelection:true                      
             * });
             * &lt;/script&gt; 
             */
            allowSelection: true,
            /**		
             * Gets or sets a value that indicates whether to enable dynamic grouping behaviour. Grouping can be done by drag on drop desired columns to grid’s GroupDropArea. This can be further customized through “groupSettings” property.
             * @default false
             * @type {boolean}
             * @example 
             * &lt;div id="Grid"&gt;&lt;/div&gt; 
             * &lt;script&gt;
             * $("#Grid").ejGrid({
             *    dataSource:window.gridData,
             *    allowGrouping:true                      
             * });
             * &lt;/script&gt; 
             */
            allowGrouping: false,
            /**		
             * Gets or sets a value that indicates summary rows should be shown on grid layout when the property “summaryRows” is set
             * @remarks
             * showSummary must be set as true.
             * summaryRows and summaryColumns must be given. See {@link ejGrid#summaryRows}
             * @default false
             * @type {boolean}
             * @example 
             * &lt;div id="Grid"&gt;&lt;/div&gt; 
             * &lt;script&gt;
             * $("#Grid").ejGrid({
             *     dataSource:window.gridData,
             *     columns: [{ field: "OrderID"}, { field: "Freight" }, { field: "ShipCity" }],
             *     showSummary:true,
             *     summaryRows:[{ title: "sum", summaryColumns: [{summaryType:ej.Grid.SummaryType.Count,displayColumn:"Freight",dataMember:"Freight"}]}]       
             * });
             * &lt;/script&gt;  
             */
            showSummary: false,
            /**		
             * Gets or sets a value that indicates whether to enable dynamic resizabiliy of columns. . Resize the width of the columns by simply click and move the particular column header line
             * @remarks
             * When allowResizing is enabled then we must specify the following properties
             * 1. allowScrolling must be set as true
             * 2. Provide the scrollSettings after enable the allowScrolling. See{@link ejGrid#scrollSettings}            
             * @default false
             * @type {boolean}
             * @example 
             * &lt;div id="Grid"&gt;&lt;/div&gt; 
             * &lt;script&gt;
             * $("#Grid").ejGrid({
             *     dataSource:window.gridData,
             *     allowResizing:true,
             *     allowScrolling:true,
             *     scrollSettings:{width:300,height:300}
             * });
             * &lt;/script&gt; 
             */
            allowResizing: false,
            /**		
            * Gets or sets a value that indicates whether the Content will wrap to the next line if the content exceeds the boundary of the Column Cells.
            * @default false
            * @type {boolean}
            * @example 
            * &lt;div id="Grid"&gt;&lt;/div&gt; 
            * &lt;script&gt;
            * $("#Grid").ejGrid({
            *    dataSource:window.gridData,
            *    allowTextWrap:true                      
            * });
            * &lt;/script&gt; 
            */
            allowTextWrap: false,
			/**		
             * Gets or sets a value that indicates whether to customizing cell based on our needs.
             * @remarks
             * Provide the mergeSettings after enable the allowCellMerging. See{@link ejGrid#mergeSettings}            
             * @default false
             * @type {boolean}
             * @example
             * &lt;div id="Grid"&gt;&lt;/div&gt; 
             * &lt;script&gt;
             * $("#Grid").ejGrid({
             *   dataSource:window.gridData,
             *   allowCellMerging:true,
             *   mergeSettings:{rowIndex:1,columnRange:[2,3]}
             * });
             * &lt;/script&gt; 
             */
            allowCellMerging: false,
            /**		
             * Gets or sets a  value that indicates whether to enable mouse over effect on corresponding grid row.
             * @default true
             * @type {boolean}
             * @example 
             * &lt;div id="Grid"&gt;&lt;/div&gt; 
             * &lt;script&gt;
             * $("#Grid").ejGrid({
             *    dataSource:window.gridData,
             *    enableRowHover:true
             * });
             * &lt;/script&gt; 
             */
            enableRowHover: true,
            /**		
             * Gets or sets a value that indicates whether to persist the grid model state in page using applicable medium i.e., HTML5 localStorage or cookies
             * @default false
             * @type {boolean}
             * @example  
             * &lt;div id="Grid"&gt;&lt;/div&gt; 
             * &lt;script&gt;
             * $("#Grid").ejGrid({
             *     dataSource:window.gridData,
             *     allowGrouping: true,
             *     enablePresistence:true
             * });
             * &lt;/script&gt; 
             */
            enablePersistence: false,
            /**		
             * Gets or sets a value that indicates to select the row while initializing the grid
             * @remarks
             * allowSelection must be set as true.
             * @default -1
             * @type {number}
             * @example 
             * &lt;div id="Grid"&gt;&lt;/div&gt; 
             * &lt;script&gt;
             * $("#Grid").ejGrid({
             *     dataSource:window.gridData,
             *     selectedRowIndex:1
             * });
             * &lt;/script&gt; 
             */
            selectedRowIndex: -1,
            /**		
             * Gets or sets a value that indicates whether to enable dynamic searching behaviour in grid. Currently search box can be enabled through “toolbarSettings”
             * @remarks
             * While enabling allowSearching then we must enable toolbar in the grid and add search option in the toolbaritem. See {@link ejGrid#toolbarSettings}
             * @default false
             * @type {boolean}
             * @example 
             * &lt;div id="Grid"&gt;&lt;/div&gt; 
             * &lt;script&gt;
             * $("#Grid").ejGrid({
             *     dataSource:window.gridData,
             *     allowSearching: true,
             *     toolbarSettings:{showToolbar:true,toolbarItems:[ej.Grid.ToolBarItems.Search]}
             * });
             * &lt;/script&gt; 
             */
            allowSearching: false,
            /**		
             * Gets or sets a value that indicates whether to enable mouse over effect  on the corresponding column header cell of the grid
             * @default false
             * @type {boolean}
             * @example 
             * &lt;div id="Grid"&gt;&lt;/div&gt; 
             * &lt;script&gt;
             * $("#Grid").ejGrid({
             *    dataSource:window.gridData,
             *    enableHeaderHover:true
             * });
             * &lt;/script&gt; 
             */
            enableHeaderHover: false,
            /**		
             * Gets or sets a value that indicates whether to enable the columns reordering behaviour in the grid.  Reordering can be done through by drag and drop the particular column from one index to another index within the grid.
             * @default false
             * @type {boolean}
             * @example  			
             * &lt;div id="Grid"&gt;&lt;/div&gt; 
             * &lt;script&gt;
             * $("#Grid").ejGrid({
             *    dataSource:window.gridData,
             *    allowReordering:true
             * });
             * &lt;/script&gt; 
             */
            allowReordering: false,
            /**		
             * Gets or sets a value that indicates whether to enable keyboard support for performing grid actions. selectionType – Gets or sets a value that indicates whether to enable single row or multiple rows selection behaviour in grid. Multiple selection can be done through by holding CTRL and clicking the grid rows
             * @default true
             * @type {boolean}
             * @example 
             * &lt;div id="Grid"&gt;&lt;/div&gt; 
             * &lt;script&gt;
             * $("#Grid").ejGrid({
             *     dataSource:window.gridData,
             *     allowKeyboardNavigation:false
             * });
             * &lt;/script&gt; 
             */
            allowKeyboardNavigation: true,
            /**		
             * To Disable the mouse swipe property as false.
             * @default true
             * @type {boolean}
             * @example 
             * &lt;div id="Grid"&gt;&lt;/div&gt; 
             * &lt;script&gt;
             * $("#Grid").ejGrid({
             *     dataSource:window.gridData,
             *     enableTouch:false
             * });
             * &lt;/script&gt; 
             */
            enableTouch: true,
            /**		
             * The row selection behavior of grid. Accepting types are "single" and "multiple".
             * @remarks
             * allowSelection must be set as true while using selectionType.
             * @default ej.Grid.SelectionType.Single
             * @type {enum}
             * @example 
             * &lt;div id="Grid"&gt;&lt;/div&gt; 
             * &lt;script&gt;
             * $("#Grid").ejGrid({
             *    dataSource:window.gridData,
             *    selectionType:"multiple"
             * });
             * &lt;/script&gt; 
             */
            selectionType: "single",
            /**		
			 * Gets or sets the data to render  the grid with records
			 * @default null
			 * @type {object}
			 * @example 
             * &lt;div id="Grid"&gt;&lt;/div&gt; 
			 * &lt;script&gt;
             * $("#Grid").ejGrid({
             *    dataSource:window.gridData
             * });
             * &lt;/script&gt; 
			 */
            dataSource: null,
            /**		
			 * Gets or sets a value that indicates to render the grid with custom theme. allowScrolling – Gets or sets a value that indicates whether to enable the scrollbar in the grid and view the records by scroll through the grid manually
			 * @default ""
			 * @type {string}
			 * @example 
			 * &lt;div id="Grid"&gt;&lt;/div&gt;
             * &lt;style type="text/css"&gt;
             *    .gradient-green {
             *        font-family: cursive;
             *    }
             *    .gradient-green .e-alt_row {
             *        background: none repeat scroll 0 0 #71A409;
             *    }
             * &lt;/style&gt;
			 * &lt;script&gt;
             * $("#Grid").ejGrid({
             *    dataSource:window.gridData,
             *    cssClass: "gradient-green"
             * });
             * &lt;/script&gt; 
			 */
            cssClass: "",
            /**		
             * Gets or sets a value that indicates whether to enable the scrollbar in the grid and view the records by scroll through the grid manually
             * @remarks
             * Provide the scrollSettings after enable the allowScrolling. See{@link ejGrid#scrollSettings}            
             * @default false
             * @type {boolean}
             * @example
             * &lt;div id="Grid"&gt;&lt;/div&gt; 
             * &lt;script&gt;
             * $("#Grid").ejGrid({
             *   dataSource:window.gridData,
             *   allowScrolling:true,
             *   scrollSettings:{width:300,height:100}
             * });
             * &lt;/script&gt; 
             */
            allowScrolling: false,
            /**		
             * Gets or sets a value that indicates whether to customizing the user interface (UI) as locale-specific in order to display regional data i.e. in a language and culture specific to a particular country or region.
             * @default "en-US"
             * @type {string}
             * @example 
             * &lt;div id="Grid"&gt;&lt;/div&gt; 
             * &lt;script&gt;
             * ej.Pager.locale["es-ES"] = {
             *    pagerInfo: "{0} de {1} páginas ({2} artículos)"
             * };
             * $("#Grid").ejGrid({
             *   dataSource:window.gridData,
             *   allowPaging:true,
             *   locale : "es-ES" 
             * });
             * &lt;/script&gt;             
             */
            locale: "en-US",
            /**		
             * Gets or sets a value that indicates whether to enable the save action in the grid  through row selection
             * @remarks
             * When enableAutoSaveOnSelectionChange property value as false, then we must include edit option in toolbar and also set allowEditing as true.
             * @default true
             * @type {boolean}
             * @example 
             * &lt;div id="Grid"&gt;&lt;/div&gt; 
             * &lt;script&gt;
             * $("#Grid").ejGrid({
             *     dataSource:window.gridData,
             *     editSettings: { allowEditing: true },
             *     toolbarSettings: { showToolbar: true, toolbarItems: ["cancel", "save"]},
             *     columns: [{ field: "OrderID", isPrimaryKey: true }, { field: "CustomerID" }, { field: "ShipCity" }],
             *     enableAutoSaveOnSelectionChange: false
             * });
             * &lt;/script&gt; 
             */
            enableAutoSaveOnSelectionChange: true,
            /**		
             * Gets or sets a value that indicates whether to enable multi columns sorting behaviour in grid. Sort multiple columns by holding CTRL and click on the corresponding column header.
             * @remarks
             * allowSorting must be set as true while enabling allowMultiSorting. 
             * @default false
             * @type {boolean}
             * @example 
             * &lt;div id="Grid"&gt;&lt;/div&gt; 
             * &lt;script&gt;
             * $("#Grid").ejGrid({
             *   dataSource:window.gridData,
             *   allowSorting:true,
             *   allowMultiSorting:true
             * });
             * &lt;/script&gt; 
             */
            allowMultiSorting: false,
            /**		
             * Gest or sets an object that indicates whether to customize the editing behaviour of the grid.
             * @type {Object}
             */
            editSettings: /** @lends ejGrid# */ {
                /**		
                 * Gets or sets a value that indicates whether  to enable the edit action in the editing mode.
                 * @remarks
                 * When allowEditing is enabled then we must set columns property "isPrimaryKey" as true for primary column.
                 * We can edit the record in the grid by the following ways.
                 *   1. Enable toolbar in the grid and add editing option in the toolbaritem and click the "edit" icon which is in the toolbar. See {@link ejGrid#toolbarSettings}
                 *   2. Press function key F2.
                 *   3. Double click on the record.
                 * @alias ejGrid#editSettings->allowEditing
                 * @default false
                 * @type {boolean}
                 * @example
                 * &lt;div id="Grid"&gt;&lt;/div&gt; 
                 * &lt;script&gt;
                 * $("#Grid").ejGrid({
                 *     dataSource: window.gridData,
                 *     columns: [{ field: "OrderID", isPrimaryKey: true }, { field: "CustomerID" }, { field: "ShipCity" }],
                 *     editSettings: { allowEditing: true },
                 *     toolbarSettings: { showToolbar: true, toolbarItems: ["edit"] }
                 * });
                 * &lt;/script&gt;              
                 */
                allowEditing: false,
                /**		
                 * Gets or sets a value that indicates whether  to enable insert action in the editing mode.
                 * @remarks
                 * When allowAdding is enabled then we must set columns property "isPrimaryKey" as true for primary column.
                 * We can add a new record in the grid by the following ways.
                 *   1. Enable toolbar in the grid and include add record option in the toolbaritem and click the "add" icon which is in the toolbar. See {@link ejGrid#toolbarSettings}
                 *   2. Press Insert key.                 
                 * @alias ejGrid#editSettings->allowAdding
                 * @default false
                 * @type {boolean}
                 * @example
                 * &lt;div id="Grid"&gt;&lt;/div&gt; 
                 * &lt;script&gt;
                 * $("#Grid").ejGrid({
                 *     dataSource: window.gridData,
                 *     columns: [{ field: "OrderID", isPrimaryKey: true }, { field: "CustomerID" }, { field: "ShipCity" }],
                 *     editSettings: { allowAdding: true },
                 *     toolbarSettings: { showToolbar: true, toolbarItems: ["add"] }                             
                 * });
                 * &lt;/script&gt;                 
                 */
                allowAdding: false,
                /**		
                 * Gets or sets a value that indicates whether  to enable the delete action in the editing mode.
                 * @remarks
                 * When allowDeleting is enabled then we must set columns property "isPrimaryKey" as true for primary column.
                 * We can delete a record in the grid by the following ways.
                 *   1. Enable toolbar in the grid and include delete option in the toolbaritem and click the "delete" icon which is in the toolbar. See {@link ejGrid#toolbarSettings}
                 *   2. Press Delete key. 
                 * @alias ejGrid#editSettings->allowDeleting
                 * @default false
                 * @type {boolean}
                 * @example
                 * &lt;div id="Grid"&gt;&lt;/div&gt; 
                 *&lt;script&gt;
                 * $("#Grid").ejGrid({
                 *     dataSource: window.gridData,
                 *     columns: [{ field: "OrderID", isPrimaryKey: true }, { field: "CustomerID" }, { field: "ShipCity" }],
                 *     editSettings: { allowDeleting: true },
                 *     toolbarSettings: { showToolbar: true, toolbarItems: ["delete"] }      
                 * });
                 * &lt;/script&gt;                  
                 */
                allowDeleting: false,
                /**		
                 * Gets or sets a value that indicates whether to define the mode of editing See {@link editMode}
                 * @remarks
                 * When allowAdding/allowEditing is enabled then we must set columns property "isPrimaryKey" as true for primary column.
                 * We must enable allowEditing or allowAdding property
                 * If allowAdding is enable then we must enable toolBar in the grid and include the "add" option as the toolBarItem. See {@link ejGrid#toolbarSettings}
                 * @alias ejGrid#editSettings->editMode
                 * @default ej.Grid.EditMode.Normal
                 * @type {enum}
                 * @example
                 * &lt;div id="Grid"&gt;&lt;/div&gt;
                 * &lt;script&gt;
                 * $("#Grid").ejGrid({
                 *     dataSource: window.gridData,
                 *     columns: [{ field: "OrderID", isPrimaryKey: true }, { field: "CustomerID" }, { field: "ShipCity" }],
                 *     editSettings: { allowEditing: true, editMode: ej.Grid.EditMode.Dialog },
                 *     toolbarSettings: { showToolbar: true, toolbarItems: ["edit"] }                             
                 * });
                 * &lt;/script&gt;                          
                 */
                editMode: "normal",
                /**		
                 * This specifies to set the position of an adding new row either in the top or bottom of the grid
                 * @remarks
                 * When allowAdding/allowEditing is enabled then we must set columns property "isPrimaryKey" as true for primary column.
                 * We must enable allowEditing or allowAdding property
                 * If allowAdding is enable then we must enable toolBar in the grid and include the "add" option as the toolBarItem. See {@link ejGrid#toolbarSettings}
                 * @alias ejGrid#editSettings->rowPosition
                 * @default ej.Grid.RowPosition.top
                 * @type {enum}
                 * @example
                 * &lt;div id="Grid"&gt;&lt;/div&gt;
                 * &lt;script&gt;
                 * $("#Grid").ejGrid({
                 *     dataSource: window.gridData,
                 *     columns: [{ field: "OrderID", isPrimaryKey: true }, { field: "CustomerID" }, { field: "ShipCity" }],
                 *     editSettings: { allowEditing: true, allowAdding:true, rowPosition:"bottom" },
                 *     toolbarSettings: { showToolbar: true, toolbarItems: ["add"]  },                             
                 * });
                 * &lt;/script&gt;                          
                 */
                rowPosition: "top",
                /**		
                 * This specifies the id of the template. This template can be used to display the data that you require to be edited using the Dialog Box
                 * @remarks
                 * When dialogEditorTemplateID is used then we must set columns property "isPrimaryKey" as true for primary column.
                 * We must enable allowEditing or allowAdding property
                 * If allowAdding is enable then we must enable toolBar in the grid and include the "add" option as the toolBarItem. See {@link ejGrid#toolbarSettings}
                 * While using dialogEditorTemplateID then editMode must be "ej.Grid.EditMode.DialogTemplate". See {@link editMode}
                 * Provide the template in the script tag and select type as "text/template" and assign the script id as the "dialogEditorTemplateID"
                 * @alias ejGrid#editSettings->dialogEditorTemplateID
                 * @default null
                 * @type {string}
                 * @example               
                 * &lt;div id="Grid"&gt;&lt;/div&gt;
                 * &lt;script id="template" type="text/template"&gt;
                 *    &lt;table&gt;
                 *        &lt;tr&gt;
                 *            &lt;td&gt;OrderID&lt;/td&gt;
                 *            &lt;td&gt;
                 *                &lt;input id="OrderID" name="OrderID" value="{{: OrderID}}" disabled="disabled" /&gt;&lt;/td&gt;
                 *        &lt;/tr&gt;
                 *        &lt;tr&gt;
                 *            &lt;td&gt;CustomerID&lt;/td&gt;
                 *            &lt;td&gt;
                 *                &lt;input id="CustomerID" name="CustomerID" value="{{: CustomerID}}" /&gt;&lt;/td&gt;
                 *        &lt;/tr&gt;
                 *        &lt;tr&gt;
                 *            &lt;td&gt;EmployeeID&lt;/td&gt;
                 *            &lt;td&gt;
                 *                &lt;input id="EmployeeID" name="EmployeeID" value="{{: EmployeeID}}" /&gt;&lt;/td&gt;
                 *        &lt;/tr&gt;
                 *    &lt;/table&gt;
                 * &lt;/script&gt;
                 * &lt;script&gt;
                 * $("#Grid").ejGrid({
                 *     dataSource: window.gridData,
                 *     columns: [{ field: "OrderID", isPrimaryKey: true }, { field: "CustomerID" }, { field: "ShipCity" }],
                 *     editSettings: { allowEditing: true, editMode: ej.Grid.EditMode.DialogTemplate, dialogEditorTemplateID: "#template" },
                 *     toolbarSettings: { showToolbar: true, toolbarItems: ["edit"] }                             
                 * });
                 * &lt;/script&gt;                
                 */
                dialogEditorTemplateID: null,
                /**		
                 * Gets or sets a value that indicates whether  to enable the editing action while double click on the record
                 * @remarks
                 * When allowEditOnDblClick is set as false then we must enable toolbar in the grid and also include "edit" option in toolbaritem. Because if false is set then, record is not edited while double click on the row.
                 * We must enable allowEditing property
                 * When allowEditOnDblClick is used then we must set columns property "isPrimaryKey" as true for primary column.
                 * @alias ejGrid#editSettings->allowEditOnDblClick
                 * @default true
                 * @type {boolean}
                 * @example
                 * Defining editEvent with edit option:
                 * &lt;div id="Grid"&gt;&lt;/div&gt; 
                 * &lt;script&gt; 
                 * $("#Grid").ejGrid({
                 *     dataSource: window.gridData,
                 *     columns: [{ field: "OrderID", isPrimaryKey: true }, { field: "CustomerID" }, { field: "ShipCity" }],
                 *     editSettings: { allowEditing: true, allowEditOnDblClick: false },
                 *     toolbarSettings: { showToolbar: true, toolbarItems: ["edit"] }      
                 * });
                 * &lt;/script&gt; 
                 */
                allowEditOnDblClick: true,
                /**		
                 * - This specifies the id of the template. This template can be used to display the data that you require to be edited using the External edit form
                 * @remarks
                 * We must enable allowEditing or allowAdding property
                 * If allowAdding is enable then we must enable toolBar in the grid and include the "add" option as the toolBarItem. See {@link ejGrid#toolbarSettings}
                 * When externalFormTemplateID is used then we must set columns property "isPrimaryKey" as true for primary column.
                 * While using externalFormTemplateID then editMode must be "ej.Grid.EditMode.ExternalFormTemplate". See {@link editMode}
                 * Provide the template in the script tag and select type as "text/template" and assign the script id as the "externalFormTemplateID"
                 * @alias ejGrid#editSettings->externalFormTemplateID
                 * @default null
                 * @type {string}
                 * @example               
                 * &lt;div id="Grid"&gt;&lt;/div&gt;
                 * &lt;script id="template" type="text/template"&gt;
                 *    &lt;table&gt;
                 *        &lt;tr&gt;
                 *            &lt;td&gt;OrderID&lt;/td&gt;
                 *            &lt;td&gt;
                 *                &lt;input id="OrderID" name="OrderID" value="{{: OrderID}}" disabled="disabled" /&gt;&lt;/td&gt;
                 *        &lt;/tr&gt;
                 *        &lt;tr&gt;
                 *            &lt;td&gt;CustomerID&lt;/td&gt;
                 *            &lt;td&gt;
                 *                &lt;input id="CustomerID" name="CustomerID" value="{{: CustomerID}}" /&gt;&lt;/td&gt;
                 *        &lt;/tr&gt;
                 *        &lt;tr&gt;
                 *            &lt;td&gt;EmployeeID&lt;/td&gt;
                 *            &lt;td&gt;
                 *                &lt;input id="EmployeeID" name="EmployeeID" value="{{: EmployeeID}}" /&gt;&lt;/td&gt;
                 *        &lt;/tr&gt;
                 *    &lt;/table&gt;
                 * &lt;/script&gt;
                 * &lt;script&gt;
                 * $("#Grid").ejGrid({
                 *     dataSource: window.gridData,
                 *     columns: [{ field: "OrderID", isPrimaryKey: true }, { field: "CustomerID" }, { field: "ShipCity" }],
                 *     editSettings: { allowEditing: true, editMode: ej.Grid.EditMode.ExternalFormTemplate, externalFormTemplateID: "#template" },
                 *     toolbarSettings: { showToolbar: true, toolbarItems: ["edit"] }                             
                 * });
                 * &lt;/script&gt;                
                 */
                externalFormTemplateID: null,
                /**		
                 * This specifies the id of the template. This template can be used to display the data that you require to be edited using the Inline edit form 
                 * @remarks
                 * We must enable allowEditing or allowAdding property
                 * If allowAdding is enable then we must enable toolBar in the grid and include the "add" option as the toolBarItem. See {@link ejGrid#toolbarSettings}
                 * When inlineFormTemplateID is used then we must set columns property "isPrimaryKey" as true for primary column.
                 * While using inlineFormTemplateID then editMode must be "ej.Grid.EditMode.InlineFormTemplate". See {@link editMode}
                 * Provide the template in the script tag and select type as "text/template" and assign the script id as the "inlineFormTemplateID"
                 * @alias ejGrid#editSettings->inlineFormTemplateID
                 * @default null
                 * @type {string}
                 * @example               
                 * &lt;div id="Grid"&gt;&lt;/div&gt;
                 * &lt;script id="template" type="text/template"&gt;
                 *    &lt;table&gt;
                 *        &lt;tr&gt;
                 *            &lt;td&gt;OrderID&lt;/td&gt;
                 *            &lt;td&gt;
                 *                &lt;input id="OrderID" name="OrderID" value="{{: OrderID}}" disabled="disabled" /&gt;&lt;/td&gt;
                 *        &lt;/tr&gt;
                 *        &lt;tr&gt;
                 *            &lt;td&gt;CustomerID&lt;/td&gt;
                 *            &lt;td&gt;
                 *                &lt;input id="CustomerID" name="CustomerID" value="{{: CustomerID}}" /&gt;&lt;/td&gt;
                 *        &lt;/tr&gt;
                 *        &lt;tr&gt;
                 *            &lt;td&gt;EmployeeID&lt;/td&gt;
                 *            &lt;td&gt;
                 *                &lt;input id="EmployeeID" name="EmployeeID" value="{{: EmployeeID}}" /&gt;&lt;/td&gt;
                 *        &lt;/tr&gt;
                 *    &lt;/table&gt;
                 * &lt;/script&gt;
                 * &lt;script&gt;
                 * $("#Grid").ejGrid({
                 *     dataSource: window.gridData,
                 *     columns: [{ field: "OrderID", isPrimaryKey: true }, { field: "CustomerID" }, { field: "ShipCity" }],
                 *     editSettings: { allowEditing: true, editMode: ej.Grid.EditMode.InlineTemplateForm, inlineFormTemplateID: "#template" },
                 *     toolbarSettings: { showToolbar: true, toolbarItems: ["edit"] }                             
                 * });
                 * &lt;/script&gt;                
                 */
                inlineFormTemplateID: null,
                /**		
                 * This specifies  to set the position of an External edit form either in the top-right or bottom-left of the grid
                 * @remarks
                 * We must enable allowEditing or allowAdding property.
                 * If allowAdding is enable then we must enable toolBar in the grid and include the "add" option as the toolBarItem. See {@link ejGrid#toolbarSettings}
                 * When formPosition is used then editMode must be "ej.Grid.EditMode.ExternalFormTemplate" or "ej.Grid.editMode.ExternalForm".
                 * We must set columns property "isPrimaryKey" as true for primary column.
                 * @alias ejGrid#editSettings->formPosition
                 * @default ej.Grid.FormPosition.BottomLeft
                 * @type {enum}
                 * @example
                 * &lt;div id="Grid"&gt;&lt;/div&gt;
                 * &lt;script&gt;
                 * $("#Grid").ejGrid({
                 *     dataSource: window.gridData,
                 *     columns: [{ field: "OrderID", isPrimaryKey: true }, { field: "CustomerID" }, { field: "ShipCity" }],
                 *     editSettings: { allowAdding: true, allowEditing: true, editMode: ej.Grid.EditMode.ExternalForm, formPosition: ej.Grid.FormPosition.BottomLeft },
                 *     toolbarSettings: { showToolbar: true, toolbarItems: ["edit"] }                             
                 * });
                 * &lt;/script&gt;    
                 */
                formPosition: "bottomleft",
                /**		
                 * Gets or sets a value that indicates whether the confirm dialog has to be shown while saving or discarding the batch changes  
                 * @remarks
                 * We must set the editMode as batch.
                 * We must enable allowEditing,allowAdding or allowDeleting property                 
                 * @alias ejGrid#editSettings->showConfirmDialog
                 * @default true
                 * @type {boolean}
                 * @example
                 * &lt;div id="Grid"&gt;&lt;/div&gt;
                 * &lt;script&gt;
                 * $("#Grid").ejGrid({
                 *     dataSource: window.gridData,
                 *     columns: [{ field: "OrderID", isPrimaryKey: true }, { field: "CustomerID" }, { field: "ShipCity" }],
                 *     editSettings: { allowEditing: true, editMode: ej.Grid.EditMode.Batch, showConfirmDialog:false },
                 *     toolbarSettings: { showToolbar: true, toolbarItems: ["edit","update","cancel"] }                             
                 * });
                 * &lt;/script&gt;                          
                 */
                showConfirmDialog: true
            },
            /**		
            * This property is used to configure the selection behaviour of the grid.            
            * @type Object
            */
            selectionSettings: /** @lends ejGrid# */ {

                /**		
                * Gets or sets  a value that indicates whether to add the default selection actions as a seleciton mode.See {@link selectionMode}
                * @remarks
                * We must enable allowSelection property
                * @alias ejGrid#selectionSettings->selectionMode
                * @default ["row"]
                * @type {enum}
                * @example
                * &lt;div id="Grid"&gt;&lt;/div&gt; 
                * &lt;script&gt;
                * $("#Grid").ejGrid({
                *    dataSource:window.gridData,
                *    allowSelection: true,   
                *    selectionSettings: {selectionMode: ["row","cell","column"] }
                * });
                * &lt;/script&gt;
                */
                selectionMode: ["row"],

                /**		
                * Gets or sets a value that indicates whether to enable the toggle selction behaviour for row, cell and column.
                * @remarks
                * When enableToggle is defined in selectionSettings then "allowSelection" must be set as true.
                * @alias ejGrid#selectionSettings->enableToggle
                * @default false
                * @type {boolean}
                * @example
                * &lt;div id="Grid"&gt;&lt;/div&gt; 
                * &lt;script&gt;
                * $("#Grid").ejGrid({
                *     dataSource:  window.gridData,
                *     allowSelection: true,   
                *     selectionSettings: {enableToggle: true }
                * });
                * &lt;/script&gt;
                */
                enableToggle: false
            },

            /**		
             * Gets or sets an object that indicates whether to modify the pager default configuration.     
             * @type Object
             */
            pageSettings: /** @lends ejGrid# */ {
                /**		
                 * Gets or sets a value that indicates whether to define the number of records displayed per page
                 * @remarks
                 * When pageSize is defined in pageSettings then "allowPaging" must be set as true.
                 * @alias ejGrid#pageSettings->pageSize
                 * @default 12
                 * @type {number}
                 * @example
                 * &lt;div id="Grid"&gt;&lt;/div&gt; 
                 * &lt;script&gt;
                 * $("#Grid").ejGrid({
                 *     dataSource: window.gridData,
                 *     allowPaging: true,   
                 *     pageSettings: { pageSize: 2 }
                 * });
                 * &lt;/script&gt;
                 */
                pageSize: 12,
                /**		
                 * Gets or sets a value that indicates whether to define the number of pages displayed in the pager for navigation
                 * @remarks
                 * When pageCount is defined in pageSettings then "allowPaging" must be set as true.
                 * @alias ejGrid#pageSettings->pageCount
                 * @default 8
                 * @type {number}
                 * @example
                 * &lt;div id="Grid"&gt;&lt;/div&gt; 
                 * &lt;script&gt;
                 * $("#Grid").ejGrid({
                 *     dataSource: window.gridData,
                 *     allowPaging: true,    
                 *     pageSettings: { pageCount: 1 }
                 * });
                 * &lt;/script&gt;
                 */
                pageCount: 8,
                /**		
                 * Gets or sets a value that indicates whether to define which page to display currently in the grid
                 * @remarks
                 * When currentPage is defined in pageSettings then "allowPaging" must be set as true.
                 * The currentPage must be less than or equal to total number of pages.
                 * @alias ejGrid#pageSettings->currentPage
                 * @default 1
                 * @type {number}
                 * @example
                 * &lt;div id="Grid"&gt;&lt;/div&gt; 
                 * &lt;script&gt;
                 * $("#Grid").ejGrid({
                 *     dataSource:  window.gridData,
                 *     allowPaging: true,   
                 *     pageSettings: { currentPage: 1 }
                 * });
                 * &lt;/script&gt;
                 */
                currentPage: 1,
                /**		
                 * Get the value of total number of pages in the grid. The totalPages value is calculated based on pagesize and totalrecords of grid
                 * @remarks
                 * There is no need to set value for this property.
                 * @alias ejGrid#pageSettings->totalPages
                 * @default null
                 * @type {number}  
                 * @example
                 * &lt;div id="Grid"&gt;&lt;/div&gt;
                 * &lt;div id="print"&gt;&lt;/div&gt;
                 * &lt;script&gt;
                 * $("#Grid").ejGrid({
                 *     dataSource:  window.gridData,
                 *     allowPaging: true    
                 * });
                 * var value = $("#Grid").ejGrid("option", "pageSettings.totalPages");
                 * $("#print").text("TotalPages: " + value);
                 * &lt;/script&gt;
                 */
                totalPages: null,
                /**
				* Gets or sets a value that indicates whether to  enables pager template for the grid.
                * @remarks
                * If enableTemplates property set as true, need to give templateId property.
                * @alias ejGrid#pageSettings->totalPages
                * @default false
                * @type {bool}  
                * @example
				* &lt;div id="Grid"&gt;&lt;/div&gt; 
				* &lt;script id="pagerTemplate" type="text/x-jsrender"&gt;
				*  &lt;input type="text" id="txtPageNumber"  value="" style="width: 45px; border: none; cursor: text" /&gt;
				* &lt;input type="button" value="Go" style="border: none; cursor: pointer" id="btnGo" onclick="gotoPage(this)" /&gt;
				* &lt;/script&gt;
				* &lt;script&gt;
				* $("#Grid").ejGrid({
				*  dataSource:window.gridData,
				*  pageSettings:{enableTemplates:true,template:"#pagerTemplate"}
				* });
				* &lt;/script&gt;
				*/
                enableTemplates: false,
                /**
				* Gets or sets a value that indicates whether to  enables default pager for the grid.
                * @alias ejGrid#pageSettings->totalPages
                * @default true
                * @type {bool}  
                * @example
				* &lt;div id="Grid"&gt;&lt;/div&gt; 
				* &lt;script id="pagerTemplate" type="text/x-jsrender"&gt;
				*  &lt;input type="text" id="txtPageNumber"  value="" style="width: 45px; border: none; cursor: text" /&gt;
				* &lt;input type="button" value="Go" style="border: none; cursor: pointer" id="btnGo" /&gt;
				* &lt;/script&gt;
				* &lt;script&gt;
				* $("#Grid").ejGrid({
				*  dataSource:window.gridData,
				*  pageSettings:{showDefaults:false,enableTemplates:true,templateID:"#pagerTemplate"}
				* });
				* &lt;/script&gt;
				*/
                showDefaults: false,
                /**
				* Gets or sets a value that indicates to add the template as a pager template for grid.
                * @remarks
                * If enableTemplates property set as true, need to give templateId property.
                * @alias ejGrid#pageSettings->totalPages
                * @default null
                * @type {string}  
                * @example
				* &lt;div id="Grid"&gt;&lt;/div&gt; 
				* &lt;script id="pagerTemplate" type="text/x-jsrender"&gt;
				*  &lt;input type="text" id="txtPageNumber"  value="" style="width: 45px; border: none; cursor: text" /&gt;
				* &lt;input type="button" value="Go" style="border: none; cursor: pointer" id="btnGo" /&gt;
				* &lt;/script&gt;
				* &lt;script&gt;
				* $("#Grid").ejGrid({
				*  dataSource:window.gridData,
				*  pageSettings:{enableTemplates:true,template:"#pagerTemplate"}
				* });
				* &lt;/script&gt;
				*/
                template: null,
                /**		
                 * Get the value of total number of records which is bound to  the grid. The totalRecordsCount value is calculated based on dataSource bound to the grid.
                 * @remarks
                 * There is no need to set value for this property.
                 * @alias ejGrid#pageSettings->totalRecordsCount
                 * @default null
                 * @type {number}
                 * @example
                 * &lt;div id="Grid"&gt;&lt;/div&gt;
                 * &lt;div id="print"&gt;&lt;/div&gt;
                 * &lt;script&gt;
                 * $("#Grid").ejGrid({
                 *     dataSource:  window.gridData,
                 *     allowPaging: true
                 * });
                 * var value = $("#Grid").ejGrid("option", "pageSettings.totalRecordsCount");
                 * $("#print").text("TotalRecordsCount: " + value);
                 * &lt;/script&gt;
                 */
                totalRecordsCount: null,
                /**		
                 * Gets or sets a value that indicates whether  to pass the current page information as a query string along with the url while navigating to other page.
                 * @remarks
                 * When currentPage is defined in pageSettings then "allowPaging" must be set as true.
                 * @alias ejGrid#pageSettings->enableQueryString
                 * @default false
                 * @type {boolean}
                 * @example
                 * &lt;div id="Grid"&gt;&lt;/div&gt; 
                 * &lt;script&gt;
                 * $("#Grid").ejGrid({
                 *     dataSource:  window.gridData,
                 *     allowPaging: true,   
                 *     pageSettings: {enableQueryString: true }
                 * });
                 * &lt;/script&gt;
                 */
                enableQueryString: false
            },
            /**		
             * Gets or sets an object that indicates whether to customize the grouping behaviour of the grid. 
             * @type {Object}             
             */
            groupSettings: /** @lends ejGrid# */ {
                /**		
                 * – Gets or sets a value that indicates whether to showthe group button image(toggle button)in the column header and also in the grouped column in the group drop area .  It can be used to group/ungroup the columns by click on the toggle button. 
                 * @remarks 
                 * When showToggleButton is enabled then "allowGrouping" must be set as true.
                 * @alias ejGrid#groupSettings->showToggleButton
                 * @default false
                 * @type {boolean}
                 * @example
                 * &lt;div id="Grid"&gt;&lt;/div&gt; 
                 * &lt;script&gt;
                 * $("#Grid").ejGrid({
                 *     dataSource:window.gridData,
                 *     allowGrouping:true,
                 *     groupSettings:{showToggleButton:true}
                 * });
                 * &lt;/script&gt;   
                 */
                showToggleButton: false,
                 /**		
                  * Gets or sets a value that indicates whether to  enable the animation effects to the group drop area
                  * @remarks
                  * allowGrouping must be set as true.
                  * @alias ejGrid#groupSettings->enableDropAreaAnimation
                  * @default true
                  * @type {boolean}
                  * @example
                  * &lt;div id="Grid"&gt;&lt;/div&gt; 
                  * &lt;script&gt;
                  * $("#Grid").ejGrid({
                  *    dataSource:window.gridData,
                  *    allowGrouping: true,
                  *    enableDropAreaAnimation:true                        
                  * });
                  * &lt;/script&gt;            
                  */
                enableDropAreaAnimation: true,
                 /**		
                  * Gets or sets a value that indicates whether to   hide the grouped columns from the grid
                  * @remarks
                  * When showGroupedColumn is enabled then "allowGrouping" must be set as true.
                  * @alias ejGrid#groupSettings->showGroupedColumn
                  * @default false
                  * @type {boolean}
                  * @example
                  * &lt;div id="Grid"&gt;&lt;/div&gt; 
                  * &lt;script&gt;
                  * $("#Grid").ejGrid({
                  *     dataSource:window.gridData,
                  *     allowGrouping:true,
                  *     groupSettings:{showGroupedColumn:false}
                  * });
                  * &lt;/script&gt; 
                  */
                showGroupedColumn: true,
                 /**		
                  * Gets or sets a value that indicates whether to  enable the close button in the grouped column  which is in the group drop area to ungroup the grouped column
                  * @remarks
                  * When showUngroupButton is enabled then "allowGrouping" and showToggleButton must be set as true.
                  * @alias ejGrid#groupSettings->showUngroupButton
                  * @default false
                  * @type {boolean}
                  * @example
                  * &lt;div id="Grid"&gt;&lt;/div&gt; 
                  * &lt;script&gt;
                  * $("#Grid").ejGrid({
                  *     dataSource:window.gridData,
                  *     allowGrouping:true,
                  *     groupSettings:{showToggleButton: true, showUngroupButton:true}
                  * });
                  * &lt;/script&gt; 
                  */
                showUngroupButton: true,
                /**		
                 * Gets or sets a value that indicates whether to  enable animation button option in the group drop area of the grid.
                 * @remarks
                 * When enableDropAreaAutoSizing is enabled then "allowGrouping" must be set as true.
                 * @alias ejGrid#groupSettings->enableDropAreaAutoSizing
                 * @default false
                 * @type {boolean}
                 * @example
                 * &lt;div id="Grid"&gt;&lt;/div&gt; 
                 * &lt;script&gt;
                 * $("#Grid").ejGrid({
                 *     dataSource:window.gridData,
                 *     allowGrouping:true,
                 *     groupSettings:{enableDropAreaAutoSizing: true}
                 * });
                 * &lt;/script&gt; 
                 */
                enableDropAreaAutoSizing: true,
                /**		
                 * Gets or sets a value that indicates whether to add grouped columns programmatically at initial load
                 * @remarks
                 * When groupedColumns is specified then "allowGrouping" must be set as true.
                 * @alias ejGrid#groupSettings->groupedColumns
                 * @default []
                 * @type {object}
                 * @example
                 * &lt;div id="Grid"&gt;&lt;/div&gt; 
                 * &lt;script&gt;
                 * $("#Grid").ejGrid({
                 *     dataSource:window.gridData,
                 *     allowGrouping:true,
                 *     groupSettings:{groupedColumns:["OrderID"]}
                 * });
                 * &lt;/script&gt; 
                 */
                groupedColumns: []
            },

            /**		
           * Gets or sets an object that indicates whether to customize the context menu behaviour of the grid.
           * @type {Object}             
           */

            contextMenuSettings: {
                /**
                 * Gets or sets a value that indicates whether to enable the context menu action in the grid.
                 * @remarks
                 * We must provide the Contextmenu Items. See{@link toolbarItems}
                 * If the above options added in contextmenu as a contextmenu item then we must enable allowEditing, allowDeleting , allowAdding , allowGrouping , allowSorting and allowPaging.
                 * @alias ejGrid#contextMenuSettings->enableContextMenu
                 * @default false
                 * @type {boolean}
                 * @example
                 * &lt;div id="Grid"&gt;&lt;/div&gt; 
                 * &lt;script&gt;
                 * $("#Grid").ejGrid({
                 *    dataSource: window.gridData,
                 *    columns:[{field: "OrderID", isPrimaryKey: true},{field: "EmployeeID"}],
                 *    editSettings: { allowDeleting: true, allowEditing: true, allowAdding: true },
                 *    allowGrouping: true,
                 *    allowSorting: true,
                 *    allowPaging: true,
                 *    contextMenuSettings: { enableContextMenu: true }
                 * });
                 * &lt;/script&gt; 
                 */
                enableContextMenu: false,
                /**		
                 * Gets or sets a value that indicates whether to add the default context menu actions as a context menu items
                 *If enableContextMenu is true it will show all the items related to the target, if u want selected items from contextmenu u have to mention in the contextMenuItems
                 * @remarks
                 * We must enable enableContextMenu property
                 * @alias ejGrid#contextMenuSettings->contextMenuItems
                 * @default []
                 * @type {array}
                 * @example
                 * &lt;div id="Grid"&gt;&lt;/div&gt; 
                 * &lt;script&gt;
                 * $("#Grid").ejGrid({
                 *    dataSource:window.gridData,
                 *    columns:[{field: "OrderID", isPrimaryKey: true},{field: "EmployeeID"}],
                 *    editSettings: { allowDeleting: true, allowEditing: true, allowAdding: true },
                 *    allowGrouping: true,
                 *    allowSorting: true,
                 *    allowPaging: true,
                 *    contextMenuSettings: { enableContextMenu: true, contextMenuItems:["Add Record,Edit Record,Delete Record"]  }
                 * });
                 * &lt;/script&gt;
                 */
                contextMenuItems: ["Add Record", "Edit Record", "Delete Record", "Sort In Ascending Order", "Sort In Descending Order", "Next Page", "Last Page", "Previous Page", "First Page", "Save", "Cancel", "Grouping", "Ungrouping"],
                /**		
                 * Gets or sets a value that indicates whether to add custom contextMenu items within the toolbar to perform any action in the grid
                 * @remarks
                 * We must enable enableContextMenu property
                 * @alias ejGrid#contextMenuSettings->customContextMenuItems
                 * @default []
                 * @type {array}
                 * @example
                 * &lt;div id="Grid"&gt;&lt;/div&gt; 
                 * &lt;style type="text/css" class="cssStyles"&gt;
                 * &lt;/style&gt;
                 * &lt;script&gt;
                 * $("#Grid").ejGrid({
                 *   dataSource:window.gridData, 
                 *   contextMenuSettings: { enableContextMenu: true, customContextMenuItems:["Hidden Columns,Visible Columns"]  }
                 * });
                 * &lt;/script&gt;
                 */
                customContextMenuItems: []
            },
            /**		
             * Gets or sets an object that indicates whether to customize the filtering behaviour of the grid
             * @type {Object}             
             */
            filterSettings: /** @lends ejGrid# */ {
                /**		
                 * This specifies the grid to show the filterBar or filterMenu to the grid records. See {@link filterType}
                 * @remarks
                 * When filterType is specified then "allowFiltering" must be set as true.
                 * @alias ejGrid#filterSettings->filterType
                 * @default ej.Grid.FilterType.FilterBar
                 * @type {enum}
                 * @example
                 * &lt;div id="Grid"&gt;&lt;/div&gt; 
                 * &lt;script&gt;
                 * $("#Grid").ejGrid({
                 *    dataSource:window.gridData,
                 *    allowFiltering: true, 
                 *    filterSettings: {  filterType: "menu" }                     
                 * });
                 * &lt;/script&gt;
                 */
                filterType: "filterbar",
                /**		
                 * This specifies the grid to starts the filter action while typing in the filterBar or after pressing the enter key. based on the filterBarMode. See {@link filterBarMode}
                 * @remarks
                 * When filterBarMode is specified then "allowFiltering" must be set as true and filterType must be "filterbar".
                 * @alias ejGrid#filterSettings->filterBarMode
                 * @default ej.Grid.FilterBarMode.Immediate
                 * @type {enum} 
                 * @example
                 * &lt;div id="Grid"&gt;&lt;/div&gt; 
                 * &lt;script&gt;
                 * $("#Grid").ejGrid({
                 *    dataSource:window.gridData,
                 *    allowFiltering: true, 
                 *    filterSettings:{ filterBarMode: ej.Grid.FilterBarMode.OnEnter }                
                 * });
                 * &lt;/script&gt;
                 */
                filterBarMode: "immediate",
                /**		
                 * This specifies the grid to show  the filter text within the grid pager itself.
                 * @remarks
                 * When filterBarMode is specified then "allowFiltering" must be set as true and filterType must be "filterbar".
                 * @alias ejGrid#filterSettings->showFilterBarMessage
                 * @default true
                 * @type {boolean}
                 * @example
                 * &lt;div id="Grid"&gt;&lt;/div&gt; 
                 * &lt;script&gt;
                 * $("#Grid").ejGrid({
                 *    dataSource:window.gridData,
                 *    allowFiltering: true, 
                 *    filterSettings: {  showFilterBarStatus: true }                        
                 * });
                 * &lt;/script&gt;
                 */
                showFilterBarStatus: true,
                statusBarWidth: 450,
                /**		
                 * Gets or sets a value that indicates whether to enable the predicate options in the filtering menu  
                 * @remarks
                 * When showPredicate is specified then "allowFiltering" must be set as true and filterType must be "menu".
                 * @alias ejGrid#filterSettings->showPredicate
                 * @default false
                 * @type {boolean}
                 * @example
                 * &lt;div id="Grid"&gt;&lt;/div&gt; 
                 * &lt;script&gt;
                 * $("#Grid").ejGrid({
                 *    dataSource:window.gridData,
                 *     allowFiltering: true, 
                 *     filterSettings: { showPredicate:true, filterType:"menu"}                       
                 * });
                 * &lt;/script&gt;
                 */
                showPredicate: false,
                /**		
                 * Gets or sets a value that indicates whether to define the filtered columns details programmatically at initial load   
                 * @remarks
                 * When filteredColumns is specified then "allowFiltering" must be set as true and filterType must be "menu".
                 * @alias ejGrid#filterSettings->filteredColumns
                 * @default []
                 * @type {object}                 
                 */
                filteredColumns: [],                
                /**		
                * Gets or sets a value that indicates the maximum number of filter choices that can be showed in the excel styled filter menu. 
                * @remarks
                * When maxFilterChoices is specified then "allowFiltering" must be set as true and filterType must be "excel".
                * @alias ejGrid#filterSettings->maxFilterChoices
                * @default 1000
                * @type {number}
                * @example
                * &lt;div id="Grid"&gt;&lt;/div&gt; 
                * &lt;script&gt;
                * $("#Grid").ejGrid({
                *    dataSource:window.gridData,
                *     allowFiltering: true, 
                *     filterSettings: { maxFilterChoices:200, filterType:"excel"}                       
                * });
                * &lt;/script&gt;
                */
                maxFilterChoices: 1000,
                /**		
                * Gets or sets a value that indicates to perform the filter operation with case sensitive in excel styled filter menu mode
                * @remarks
                * When enableCaseSensitivity is specified then "allowFiltering" must be set as true and filterType must be "excel".
                * @alias ejGrid#filterSettings->enableCaseSensitivity
                * @default false
                * @type {boolean}
                * @example
                * &lt;div id="Grid"&gt;&lt;/div&gt; 
                * &lt;script&gt;
                * $("#Grid").ejGrid({
                *    dataSource:window.gridData,
                *     allowFiltering: true, 
                *     filterSettings: { enableCaseSensitivity:true, filterType:"excel"}                       
                * });
                * &lt;/script&gt;
                */
                enableCaseSensitivity:false
            },
            /**		
             * Gets or sets a value that indicates whether to customize the sorting behaviour of the grid.
             * @type {Object}           
             */
            sortSettings: /** @lends ejGrid# */ {
                /**		
                 * Gets  or sets a value that indicates whether to define the sorted Columns details programmatically at initial load
                 * @remarks
                 * allowSorting must be set as true.
                 * @alias ejGrid#sortSettings->sortedColumns
                 * @default []
                 * @type {object}
                 * @example                 
                 * &lt;div id="Grid"&gt;&lt;/div&gt; 
                 * &lt;div id="print"&gt;&lt;/div&gt; 
                 * &lt;script&gt;
                 * $("#Grid").ejGrid({
                 *    dataSource:window.gridData,
                 *    columns:[{field: "OrderID"},{field: "EmployeeID"}],
                 *    allowSorting: true,
                 *    sortSettings: {sortedColumns: [{field:"EmployeeID", direction:"ascending"}] }                  
                 * });
                 * var sortColumn = $("#Grid").ejGrid("option", "sortSettings.sortedColumns");
                 * $("#print").text(sortColumn);
                 * &lt;/script&gt;
                 */

                /**
                 * Gets or sets a value that indicates whether to define the field name of the column to be sort
                 * @remarks
                 * allowSorting must be set as true.
                 * @name ejGrid#sortSettings->sortedColumns->field 	          
                 * @default -
                 * @type {string}
                 * @example
                 * &lt;div id="Grid"&gt;&lt;/div&gt;          
                 * &lt;script&gt;
                 * $("#Grid").ejGrid({
                 *   dataSource:window.gridData,
                 *   columns:[{field: "OrderID"},{field: "EmployeeID"}],
                 *   sortSettings: {sortedColumns: [{field:"EmployeeID"}] }             
                 * });
                 * &lt;/script&gt;  
                 */

                /**
                 * Gets  or sets a value that indicates whether to define the direction to sort the column.
                 * @remarks
                 * allowSorting must be set as true.
                 * @name ejGrid#sortSettings->sortedColumns->direction 	          
                 * @default -
                 * @type {string}
                 * @example
                 * &lt;div id="Grid"&gt;&lt;/div&gt;          
                 * &lt;script&gt;
                 * $("#Grid").ejGrid({
                 *   dataSource:window.gridData,
                 *   columns:[{field: "OrderID"},{field: "EmployeeID"}],
                 *   sortSettings: {sortedColumns: [{field:"EmployeeID", direction:"descending"}] }                    
                 * });
                 * &lt;/script&gt;  
                 */
                sortedColumns: []
            },
            /**		
             * Gets or sets an object that indicates whether to enable the toolbar in the grid and add toolbar items
             * @type {Object}             
             */
            toolbarSettings: /** @lends ejGrid# */ {
                /**		
                 * Gets or sets a value that indicates whether to enable  toolbar in the grid.
                 * @remarks
                 * We must provide the toolbarItems. See{@link toolbarItems}
                 * If editing options added in toolbar as a toolbar item then we must enable allowEditing, allowDeleting and allowAdding.
                 * @alias ejGrid#toolbarSettings->showToolbar
                 * @default false
                 * @type {boolean}
                 * @example
                 * &lt;div id="Grid"&gt;&lt;/div&gt; 
                 * &lt;script&gt;
                 * $("#Grid").ejGrid({
                 *    dataSource: window.gridData,
                 *    columns:[{field: "OrderID", isPrimaryKey: true},{field: "EmployeeID"}],
                 *    editSettings: { allowEditing: true },
                 *    toolbarSettings: { showToolbar: true, toolbarItems: [ej.Grid.ToolBarItems.Edit] }
                 * });
                 * &lt;/script&gt; 
                 */
                showToolbar: false,
                /**		
                 * Gets or sets  a value that indicates whether  to add the default editing actions as a toolbar items
                 * @remarks
                 * We must enable showToolbar property
                 * @alias ejGrid#toolbarSettings->toolbarItems
                 * @default []
                 * @type {enum}
                 * @example
                 * &lt;div id="Grid"&gt;&lt;/div&gt; 
                 * &lt;script&gt;
                 * $("#Grid").ejGrid({
                 *    dataSource:window.gridData,
                 *    columns:[{field: "OrderID", isPrimaryKey: true},{field: "EmployeeID"}],
                 *    editSettings: { allowDeleting: true, allowEditing: true, allowAdding: true },
                 *    toolbarSettings: { showToolbar: true, toolbarItems: [ej.Grid.ToolBarItems.Add, ej.Grid.ToolBarItems.Edit, ej.Grid.ToolBarItems.Delete, ej.Grid.ToolBarItems.Update, ej.Grid.ToolBarItems.Cancel] }
                 * });
                 * &lt;/script&gt;
                 */
                toolbarItems: [],
                /**		
                 * Gets or sets a value that indicates whether  to add custom toolbar items within the toolbar to perform any action in the grid 
                 * @remarks
                 * We must enable showToolbar property
                 * @alias ejGrid#toolbarSettings->customToolbarItems
                 * @default []
                 * @type {object}
                 * @example
                 * &lt;div id="Grid"&gt;&lt;/div&gt; 
                 * &lt;style type="text/css" class="cssStyles"&gt;
                 *   .Expand:before
                 *   {
                 *     content:"\e627";
                 *   }
                 * &lt;/style&gt;
                 * &lt;script&gt;
                 * $("#Grid").ejGrid({
                 *   dataSource:window.gridData, 
                 *   toolbarSettings:{showToolbar:true,customToolbarItems:["Expand"]}
                 * });
                 * &lt;/script&gt;
                 */
                customToolbarItems: []
            },
            /**		
             * Gets or sets an object that indicates whether to customize the scrolling behaviour of the grid.
             * @type {Object}
             */
            scrollSettings: /** @lends ejGrid# */
            {
                /**		
                 * This specify the grid to show the horizontal scroll bar, to scroll and view the grid contents
                 * @remarks
                 * We must enable allowScrolling property
                 * @alias ejGrid#scrollSettings->width
                 * @default 250
                 * @type {number}
                 * @example
                 * &lt;div id="Grid"&gt;&lt;/div&gt; 
                 * &lt;script&gt;
                 * $("#Grid").ejGrid({
                 *    dataSource:window.gridData,
                 *    allowScrolling: true,
                 *    scrollSettings:{ width:300 }
                 * });
                 * &lt;/script&gt; 
                 */
                width: "auto",
                /**		
                 * This specify the grid to show the vertical scroll bar, to scroll and view the grid contents.
                 * @remarks
                 * We must enable allowScrolling property
                 * @alias ejGrid#scrollSettings->height
                 * @default 0
                 * @type {number}
                 * @example
                 * &lt;div id="Grid"&gt;&lt;/div&gt; 
                 * &lt;script&gt;
                 * $("#Grid").ejGrid({
                 *    dataSource:window.gridData,
                 *    allowScrolling: true,
                 *    scrollSettings:{ height:100 }
                 * });
                 * &lt;/script&gt; 
                 */
                height: 0,
                /**		
                 * This specify the grid to to view data that you require without buffering the entire load of a huge database
                 * @remarks
                 * 1.We must enable allowScrolling property
                 * 2.Sets the height and width size
                 * @alias ejGrid#scrollSettings->allowVirtualScrolling
                 * @default false
                 * @type {boolean}
                 * @example
                 * &lt;div id="Grid"&gt;&lt;/div&gt; 
                 * &lt;script&gt;
                 * $("#Grid").ejGrid({
                 *    dataSource:window.gridData,
                 *    allowScrolling: true,
                 *    scrollSettings:{width:300,height:100,allowVirtualScrolling:true}
                 * });
                 * &lt;/script&gt; 
                 */
                allowVirtualScrolling: false,
                /**		
                * This is used to define the mode of virtual scrolling in grid. See {@link virtualScrollMode}
                * @remarks                 
                * We must enable allowScrolling and allowVirtualScrolling property                 
                * @alias ejGrid#scrollSettings->virtualScrollMode
                * @default ej.Grid.VirtualScrollMode.Normal
                * @type {enum}
                * @example
                * &lt;div id="Grid"&gt;&lt;/div&gt;
                * &lt;script&gt;
                * $("#Grid").ejGrid({
                *     dataSource: window.gridData,                     
                *     allowScrolling:true,
                *     scrollSettings:{ width:500 , height: 550 , allowVirtualScrolling:true, virtualScrollMode:ej.Grid.VirtualScrollMode.Normal }
                * });
                * &lt;/script&gt;                          
                */
                virtualScrollMode:"normal",
                /**		
                 * This specify the grid to freeze particular rows at the time of scrolling.
                 * @remarks
                 * We must enable allowScrolling property
                 * @alias ejGrid#scrollSettings->frozenRows
                 * @default 0
                 * @type {number}
                 * @example
                 * &lt;div id="Grid"&gt;&lt;/div&gt; 
                 * &lt;script&gt;
                 * $("#Grid").ejGrid({
                 *     dataSource:window.gridData,
                 *     allowScrolling: true,               
                 *     scrollSettings:{width:300,height:200,frozenRows:2 }
                 * });
                 * &lt;/script&gt; 
                 */
                frozenRows: 0,
                /**		
                 * This specify the grid to freeze particular columns  at the time of scrolling.
                 * @remarks
                 * We must enable allowScrolling property
                 * @alias ejGrid#scrollSettings->frozenColumns
                 * @default 0
                 * @type {number}
                 * @example
                 * &lt;div id="Grid"&gt;&lt;/div&gt; 
                 * &lt;script&gt;
                 * $("#Grid").ejGrid({
                 *     dataSource:window.gridData,
                 *     allowScrolling: true,               
                 *     scrollSettings:{width:500,height:100,frozenColumns:2 }
                 * });  
                 * &lt;/script&gt; 
                 */
                frozenColumns: 0
            },
            /**		
             * Gets or sets an object that indicates to managing the collection of summary rows for the grid.
             * @remarks
             * We must enable showSummary property
             * @default []
             * @name ejGrid#summaryRows
             * @type {array}
             * @example
             * &lt;div id="Grid"&gt;&lt;/div&gt; 
             * &lt;script&gt;
             * $("#Grid").ejGrid({
             *      dataSource:window.gridData,
             *      showSummary: true,
             *      columns:[{field: "OrderID"},{field: "Freight"}],
             *      summaryRows: [{
             *         title: "Sum",
             *         summaryColumns: [{
             *            summaryType: ej.Grid.SummaryType.Sum,
             *            displayColumn: "Freight",
             *            dataMember: "Freight"
             *         }]
             *      }]                                    
             * });
             * &lt;/script&gt; 
             */

            /**
             * This specifies the grid to show the title for the summary rows.  
             * @remarks
             * showSummary must be true and summaryRows must be given while using title.
             * @name ejGrid#summaryRows->title 	          
             * @default ""
             * @type {string}
             * @example
             * &lt;div id="Grid"&gt;&lt;/div&gt; 
             * &lt;script&gt;
             * $("#Grid").ejGrid({
             *      dataSource:window.gridData,  
             *      showSummary: true,
             *      columns:[{field: "OrderID"},{field: "Freight"}],
             *      summaryRows: [{
             *         title: "Sum",
             *         summaryColumns: [{
             *            summaryType: ej.Grid.SummaryType.Sum,
             *            displayColumn: "Freight",
             *            dataMember: "Freight"
             *         }]
             *      }]                                    
             * });
             * &lt;/script&gt;
             */

            /**
             * Gets or sets a value that indicates whether to show the summary value within the group caption area for the corresponding summary column while grouping the column
             * @remarks
             * showSummary and allowGrouping must be true. summaryRows must be given while using showCaptionSummary.
             * @name ejGrid#summaryRows->showCaptionSummary 	          
             * @default false
             * @type {boolean}
             * @example
             * &lt;div id="Grid"&gt;&lt;/div&gt; 
             * &lt;script&gt;
             * $("#Grid").ejGrid({
             *      dataSource:window.gridData,  
             *      columns:[{field: "OrderID"},{field: "Freight"}],
             *      allowGrouping: true,
             *      showSummary: true,
             *      summaryRows: [{
             *         title: "Sum",
             *         summaryColumns: [{
             *            summaryType: ej.Grid.SummaryType.Sum,
             *            displayColumn: "Freight",
             *            dataMember: "Freight"
             *         }],
             *         showCaptionSummary: true
             *      }]                                    
             * });
             * &lt;/script&gt;           
             */

            /**
             * Gets or sets a value that indicates whether to show the total summary value  the for the corresponding summary column. The summary row is added after the grid content.
             * @remarks
             * showSummary and allowGrouping must be true. summaryRows must be given while using showTotalSummary.
             * @name ejGrid#summaryRows->showTotalSummary 	          
             * @default true
             * @type {boolean}
             * @example
             * &lt;div id="Grid"&gt;&lt;/div&gt; 
             * &lt;script&gt;
             * $("#Grid").ejGrid({
             *      dataSource:window.gridData, 
             *      columns:[{field: "OrderID"},{field: "Freight"}],
             *      showSummary: true,
             *      summaryRows: [{
             *         title: "Sum",
             *         summaryColumns: [{
             *            summaryType: ej.Grid.SummaryType.Sum,
             *            displayColumn: "Freight",
             *            dataMember: "Freight"
             *         }],
             *         showTotalSummary: true
             *      }]                                    
             * });
             * &lt;/script&gt; 
             */

            /**
             * Gets or sets a value that indicates whether to add summary columns into the summary rows.
             * @remarks            
             * showSummary must be true and summaryRows must be given while using summaryColumns.
             * @name ejGrid#summaryRows->summaryColumns 	          
             * @default []
             * @type {object}
             * @example
             * &lt;div id="Grid"&gt;&lt;/div&gt; 
             * &lt;script&gt;
             * $("#Grid").ejGrid({
             *      dataSource:window.gridData, 
             *      columns:[{field: "OrderID"},{field: "Freight"}],
             *      showSummary: true,
             *      summaryRows: [{
             *         title: "Sum",
             *         summaryColumns: [{
             *            summaryType: ej.Grid.SummaryType.Sum,
             *            displayColumn: "Freight",
             *            dataMember: "Freight"
             *         }]
             *      }]                                    
             * });
             * &lt;/script&gt; 
             */

            /**
             * Gets or sets a value that indicates the type of calculations to be performed for the corresponding summary column
             * @remarks            
             * showSummary must be true and summaryRows must be given while using summaryColumns and their properties.
             * @name ejGrid#summaryRows->summaryColumns->summaryType     
             * @default []
             * @type {object}
             * @example
             * &lt;div id="Grid"&gt;&lt;/div&gt; 
             * &lt;script&gt;
             * $("#Grid").ejGrid({
             *      dataSource:window.gridData, 
             *      columns:[{field: "OrderID"},{field: "Freight"}],
             *      showSummary: true,
             *      summaryRows: [{
             *         title: "Sum",
             *         summaryColumns: [{
             *            summaryType: ej.Grid.SummaryType.Sum,
             *            displayColumn: "Freight",
             *            dataMember: "Freight"
             *         }]
             *      }]                                    
             * });
             * &lt;/script&gt; 
             */

            /**
             * Gets or sets a value that indicates to define the target column at which to display the summary.
             * @remarks            
             * showSummary must be true and summaryRows must be given while using summaryColumns and their properties.
             * @name ejGrid#summaryRows->summaryColumns->displayColumn 	          
             * @default null
             * @type {string}
             * @example
             * &lt;div id="Grid"&gt;&lt;/div&gt; 
             * &lt;script&gt;
             * $("#Grid").ejGrid({
             *      dataSource:window.gridData,
             *      columns:[{field: "OrderID"},{field: "Freight"}],
             *      showSummary: true,
             *      summaryRows: [{
             *         title: "Sum",
             *         summaryColumns: [{
             *            summaryType: ej.Grid.SummaryType.Sum,
             *            displayColumn: "Freight",
             *            dataMember: "Freight"
             *         }]
             *      }]                                    
             * });
             * &lt;/script&gt; 
             */

            /**
             * This specifies summary column used to perform the summary calculation
             * @remarks            
             * showSummary must be true and summaryRows must be given while using summaryColumns and their properties.
             * @name ejGrid#summaryRows->summaryColumns->dataMember 	          
             * @default null
             * @type {string}
             * @example
             * &lt;div id="Grid"&gt;&lt;/div&gt; 
             * &lt;script&gt;
             * $("#Grid").ejGrid({
             *      dataSource:window.gridData,
             *      columns:[{field: "OrderID"},{field: "Freight"}],
             *      showSummary: true,
             *      summaryRows: [{
             *         title: "Sum",
             *         summaryColumns: [{
             *            summaryType: ej.Grid.SummaryType.Sum,
             *            displayColumn: "Freight",
             *            dataMember: "Freight"
             *         }]
             *      }]                                    
             * });
             * &lt;/script&gt; 
             */

            /**
             * Gets or sets a value that indicates the format for the text applied on the column
             * @remarks            
             * showSummary must be true and summaryRows must be given while using summaryColumns and their properties.
             * @name ejGrid#summaryRows->summaryColumns->format 	          
             * @default null
             * @type {string}
             * @example
             * &lt;div id="Grid"&gt;&lt;/div&gt; 
             * &lt;script&gt;
             * $("#Grid").ejGrid({
             *      dataSource:window.gridData,
             *      columns:[{field: "OrderID"},{field: "Freight"}],
             *      showSummary: true,
             *      summaryRows: [{
             *         title: "Sum",
             *         summaryColumns: [{
             *            summaryType: ej.Grid.SummaryType.Sum,
             *            displayColumn: "Freight",
             *            dataMember: "Freight",
             *            format: "{0:C2}"
             *         }]
             *      }]                                    
             * });
             * &lt;/script&gt; 
             */

            /**
             * Gets or sets a value that indicates the text displayed after the summary column value
             * @remarks            
             * showSummary must be true and summaryRows must be given while using summaryColumns and their properties.
             * @name ejGrid#summaryRows->summaryColumns->suffix 	          
             * @default null
             * @type {string}
             * @example
             * &lt;div id="Grid"&gt;&lt;/div&gt; 
             * &lt;script&gt;
             * $("#Grid").ejGrid({
             *      dataSource:window.gridData,
             *      columns:[{field: "OrderID"},{field: "Freight"}],
             *      showSummary: true,
             *      summaryRows: [{
             *         title: "Sum",
             *         summaryColumns: [{
             *            summaryType: ej.Grid.SummaryType.Sum,
             *            displayColumn: "Freight",
             *            dataMember: "Freight",
             *            suffix: "/-"
             *         }]
             *      }]                                    
             * });
             * &lt;/script&gt; 
             */

            /**
             * Gets or sets a value that indicates the text displayed before the summary column value
             * @remarks            
             * showSummary must be true and summaryRows must be given while using summaryColumns and their properties.
             * @name ejGrid#summaryRows->summaryColumns->prefix 	          
             * @default null
             * @type {string}
             * @example
             * &lt;div id="Grid"&gt;&lt;/div&gt; 
             * &lt;script&gt;
             * $("#Grid").ejGrid({
             *      dataSource:window.gridData,
             *      showSummary: true,
             *      summaryRows: [{
             *         title: "Sum",
             *         summaryColumns: [{
             *            summaryType: ej.Grid.SummaryType.Sum,
             *            displayColumn: "Freight",
             *            dataMember: "Freight",
             *            prefix : "Currency:"
             *         }]
             *      }]                                    
             * });
             * &lt;/script&gt; 
             */

            /**
             * Gets or sets a value that indicates the text displayed in the summary column as a value
             * @remarks            
             * showSummary must be true and summaryRows must be given while using summaryColumns and their properties.
             * The summaryType property must be set as Custom.
             * @name ejGrid#summaryRows->summaryColumns->customSummaryValue 	          
             * @default null
             * @type {string}
             * @example
             * &lt;div id="Grid"&gt;&lt;/div&gt; 
             * &lt;script&gt;
             * $("#Grid").ejGrid({
             *      dataSource:window.gridData,
             *      columns:[{field: "OrderID"},{field: "Freight"}],
             *      showSummary: true,
             *      summaryRows: [{
             *         title: "Sum",
             *         summaryColumns: [{
             *            summaryType: ej.Grid.SummaryType.Custom,
             *            displayColumn: "Freight",
             *            dataMember: "Freight",
             *            customSummaryValue : "Currency"
             *         }]
             *      }]                                    
             * });    
             * &lt;/script&gt; 
             */
            summaryRows: [],
            /**		
             * Align content in the grid control from right to left by setting the property as true.
             * @default false
             * @type {boolean}
             * @example
             * &lt;div id="Grid"&gt;&lt;/div&gt; 
             * &lt;script&gt;
             * $("#Grid").ejGrid({
             *   dataSource:window.gridData,
             *   enableRTL:true,
             * });
             * &lt;/script&gt; 
             */
            enableRTL: false,
            /**		
             * Gets or sets a value that indicates whether to enable the alternative rows differentiation in the grid records based on corresponding theme.
             * @default true
             * @type {boolean}
             * @example
             * &lt;div id="Grid"&gt;&lt;/div&gt; 
             * &lt;script&gt;
             * $("#Grid").ejGrid({
             *   dataSource:window.gridData,
             *   enableAltRow:true,
             * });
             * &lt;/script&gt; 
             */
            enableAltRow: true,
            currentViewData: null,
            /**		
             * This specifies the grid to add the details row for the corresponding master row
             * @default null
             * @type {string}
             * @example
             * &lt;div id="Grid"&gt;&lt;/div&gt; 
             * &lt;script id="templateData" type="text/x-jsrender"&gt;
             * &lt;table&gt;
             * &lt;tr&gt;
             * &lt;td&gt;
             * &lt;img src="styles/images/Employees/{{:EmployeeID}}.png" alt="{{:EmployeeID}}"/&gt;
             * &lt;/td&gt;
             * &lt;/tr&gt;
             * &lt;/table&gt;
             * &lt;/script&gt;
             * &lt;script&gt;
             * $("#Grid").ejGrid({
             *   dataSource:window.gridData,
             *   detailsTemplate:"#templateData",
             *   detailsDataBound: "detailGridData",
             * });             
             * &lt;/script&gt;             
             */
            detailsTemplate: null,
            /**		
            * This specifies the grid to add the grid control inside the grid row of the parent with expand/collapse options
            * @default null
            * @type {string}
            * @example 	       
            * &lt;div id="Grid"&gt;&lt;/div&gt;
            * &lt;script&gt;
            * $("#Grid").ejGrid({
            *   dataSource:window.gridData,
            *    childGrid: {
            * dataSource: window.employeeView,
            * queryString: "EmployeeID",
            * }
            * });
            * &lt;/script&gt;
            */
            childGrid: null,
            /**		
             * Gets or sets a value that indicates to render the grid with template rows. The template row must be a table row. That table row must have the JavaScript render binding format ({{:columnName}}) then the grid data source binds the data to the corresponding table row of the template.
             * @default null
             * @type {string}
             * @example
             * &lt;div id="Grid"&gt;&lt;/div&gt; 
             * &lt;script id="templateData" type="text/x-jsrender"&gt;
             * &lt;tr&gt;
             * &lt;td&gt;
             * &lt;img src="styles/images/Employees/{{:EmployeeID}}.png" alt="{{:EmployeeID}}"/&gt;
             * &lt;/td&gt;
             * &lt;td&gt;
             * {{:EmployeeID}}
             * &lt;/td&gt;
             * &lt;/tr&gt;
             * &lt;/script&gt;
             * &lt;script&gt;
             * $("#Grid").ejGrid({
             *  dataSource:window.gridData,
             *   rowTemplate:"#templateData",
             *   columns:[{headerText:"Employeephoto"},{field:"EmployeeID",headerText:"EmployeeID"}],
             * });
             * &lt;/script&gt;
             */
            rowTemplate: null,
            /**		
             * Triggered detail template row is initialized.
             * @remarks
             * Template must be bind to detailsTemplate property while using this event
             * @event
			 * @name ejGrid#detailsDataBound 	
			 * @param {Object} argument Event parameters from grid              
			 * @param {boolean}  argument.cancel Returns the cancel option value.
			 * @param {object}  argument.detailsElement Returns details row element.
             * @param {object}  argument.data Returns the details row data.
			 * @param {object}  argument.model Returns the grid model.
			 * @param {string}  argument.type Returns the name of the event.
			 * @example
             * &lt;div id="Grid"&gt;&lt;/div&gt;
             * &lt;script&gt;
             * $("#Grid").ejGrid({
             *    detailsDataBound: function (args){}
             * });
             * &lt;/script&gt;
             */
            detailsDataBound: null,
            /**    
             * Triggered every time a request is made to access row information, element and data.
             * @event
             * @name ejGrid#rowDataBound 	
             * @param {Object} argument Event parameters from grid              
             * @param {object}  argument.row Returns grid row.
             * @param {boolean}  argument.cancel Returns the cancel option value.
             * @param {object}  argument.data Returns current row record object (JSON).
             * @param {object} argument.foreignKeyData Returns the foreign key record object (JSON).
             * @param {object}  argument.model Returns the grid model.
             * @param {string}  argument.type Returns the name of the event.
             * @example 
             * &lt;div id="Grid"&gt;&lt;/div&gt;
             * &lt;script&gt;
             * $("#Grid").ejGrid({
             *    rowDataBound: function (args){}
             * });
             * &lt;/script&gt;
             */
            rowDataBound: null,
            /**    
             * Triggered every time a request is made to access particular cell information, element and data.
             * @event
             * @name ejGrid#queryCellInfo 	
             * @param {Object} argument Event parameters from grid              
             * @param {object}  argument.cell Returns grid cell.
             * @param {boolean}  argument.cancel Returns the cancel option value.
             * @param {object}  argument.data Returns current row record object (JSON).
             * @param {string}  argument.text Returns the text value in the cell.
             * @param {object} argument.column Returns the column object.
             * @param {object} argument.foreignKeyData Returns the foreign key record object (JSON).
             * @param {object}  argument.model Returns the grid model.
             * @param {string}  argument.type Returns the name of the event.
             * @example
             * &lt;div id="Grid"&gt;&lt;/div&gt; 
             * &lt;script&gt;
             * $("#Grid").ejGrid({
             *    queryCellInfo: function (args){}
             * });
             * &lt;/script&gt;
             */
            queryCellInfo: null,
			 /**    
             * Triggered every time a request is made to access particular cell information, element and data.
             * @event
             * @name ejGrid#mergeCellInfo 	
             * @param {Object} argument Event parameters from grid              
             * @param {object}  argument.cell Returns grid cell.
             * @param {boolean}  argument.cancel Returns the cancel option value.
             * @param {object}  argument.data Returns current row record object (JSON).
             * @param {string}  argument.text Returns the text value in the cell.
             * @param {object} argument.column Returns the column object.
             * @param {object} argument.foreignKeyData Returns the foreign key record object (JSON).
             * @param {object}  argument.model Returns the grid model.
             * @param {string}  argument.type Returns the name of the event.
             * @example
             * &lt;div id="Grid"&gt;&lt;/div&gt; 
             * &lt;script&gt;
             * $("#Grid").ejGrid({
             *    mergeCellInfo: function (args){}
             * });
             * &lt;/script&gt;
             */
            mergeCellInfo: null,
            /**    
             * Triggered when the grid is rendered completely.
             * @event
             * @name ejGrid#create 	
             * @param {Object} argument Event parameters from grid             
             * @param {boolean}  argument.cancel Returns the cancel option value.
             * @param {object}  argument.model Returns the grid model.
             * @param {string}  argument.type Returns the name of the event.
             * @example
             * &lt;div id="Grid"&gt;&lt;/div&gt;
             * &lt;script&gt;
             * $("#Grid").ejGrid({
             *    create: function (args){}
             * });
             * &lt;/script&gt;
             */
            create: null,
            /**    
             * Triggered for every grid action before its starts.
             * @event
             * @name ejGrid#actionBegin              
             * @param {Object} argument Event parameters when grid is initialized:          
             * @param {boolean}  argument.cancel Returns the cancel option value.
             * @param {object}  argument.model Returns the grid model.
             * @param {string}  argument.type Returns the name of the event.
             
             * @param {Object} argument Event parameters when grid paging action starts:
             * @param {boolean}  argument.cancel Returns the cancel option value.
             * @param {number} argument.currentPage  Returns the current selected page number.
             * @param {number} argument.previousPage  Returns the previous selected page number.
             * @param {number} argument.endIndex Returns the end row index of that current page.
             * @param {object}  argument.model Returns the grid model.
             * @param {string}  argument.requestType Returns request type.
             * @param {number} argument.startIndex  Returns the start row index of that current page.
             * @param {string}  argument.type Returns the name of the event.
 
             * @param {Object} argument Event parameters when grid sorting action starts:
             * @param {boolean}  argument.cancel Returns the cancel option value.
             * @param {string}  argument.columnName Returns the current grouped column field name.
             * @param {object}  argument.model Returns the grid model.
             * @param {string}  argument.requestType Returns request type.
             * @param {string}  argument.type Returns the name of the event. 
             * @param {string}  argument.columnSortDirection  Returns the column sort direction.
 
             * @param {Object} argument Event parameters when grid grouping action starts:
             * @param {boolean}  argument.cancel Returns the cancel option value.
             * @param {string}  argument.columnName Returns the current grouped column field name.
             * @param {object}  argument.model Returns the grid model.
             * @param {string}  argument.requestType Returns request type.
             * @param {string}  argument.type Returns the name of the event.        
 
             * @param {Object} argument Event parameters when grid record editing action starts:
             * @param {boolean}  argument.cancel Returns the cancel option value.
             * @param {object}  argument.row  Returns current edited row.
             * @param {object}  argument.model Returns the grid model.
             * @param {string} argument.originalEventType Returns the current action event type.
             * @param {string} argument.primaryKey Returns primary key.
             * @param {string} argument.primaryKeyValue Returns primary key value.
             * @param {string}  argument.requestType Returns request type.
             * @param {number}  argument.rowIndex Returns the edited row index.
             * @param {string}  argument.type Returns the name of the event.
 
             * @param {Object} argument Event parameters when grid record save action starts:
             * @param {boolean}  argument.cancel Returns the cancel option value.
             * @param {object}  argument.data Returns the record object (JSON).
             * @param {object}  argument.foreignKeyData Returns the foreign key record object (JSON).
             * @param {object}  argument.model Returns the grid model.
             * @param {number}  argument.selectedRow Returns the selected row index.
             * @param {string}  argument.requestType Returns request type.
             * @param {string}  argument.type Returns the name of the event. 
 
             * @param {Object} argument Event parameters when grid record cancel action starts:
             * @param {boolean}  argument.cancel Returns the cancel option value.
             * @param {object}  argument.model Returns the grid model.
             * @param {string}  argument.requestType Returns request type.
             * @param {string}  argument.type Returns the name of the event. 
 
             * @param {Object} argument Event parameters when grid record delete action starts:
             * @param {boolean}  argument.cancel Returns the cancel option value.
             * @param {object}  argument.data Returns the record object (JSON).
             * @param {object}  argument.foreignKeyData Returns the foreign key record object (JSON).
             * @param {object}  argument.model Returns the grid model.
             * @param {string}  argument.requestType Returns request type.
             * @param {object}  argument.tr Returns selected row for delete.
             * @param {string}  argument.type Returns the name of the event.
 
             * @param {Object} argument Event parameters when add new record action starts:
             * @param {boolean}  argument.cancel Returns the cancel option value.
             * @param {object}  argument.data Returns the record object (JSON).
             * @param {object}  argument.model Returns the grid model.
             * @param {string}  argument.requestType Returns request type.
             * @param {string}  argument.type Returns the name of the event.
 
             * @param {Object} argument Event parameters when grid filtering action starts:
             * @param {boolean}  argument.cancel Returns the cancel option value.
             * @param {object}  argument.currentFilteringColumn Returns current filtering column field name.
             * @param {object}  argument.filterCollection Returns filter details.
             * @param {object}  argument.model Returns the grid model.
             * @param {string}  argument.requestType Returns request type.
             * @param {string}  argument.type Returns the name of the event.
 
             * @example 
             * &lt;div id="Grid"&gt;&lt;/div&gt; 
             * &lt;script&gt;
             * $("#Grid").ejGrid({
             *    actionBegin: function (args){}
             * });
             * &lt;/script&gt;
             */
            actionBegin: null,
            /**    
             * Triggered for every grid action success event.
             * @event
             * @name ejGrid#actionComplete 	
             * @param {Object} argument Arguments in actionComplete when grid is initialized.     
             * @param {boolean} argument.cancel Returns the cancel option value.
             * @param {object} argument.model Returns the grid model.
             * @param {string} argument.requestType Returns request type.
             * @param {string} argument.type Returns the name of the event.
             
             * @param {Object} argument Arguments in actionComplete after grid paging action is completed.
             * @param {boolean} argument.cancel Returns the cancel option value.
             * @param {number} argument.currentPage Returns the current selected page number.
             * @param {number} argument.previousPage Returns the previous selected page number.
             * @param {number} argument.endIndex Returns the end row index of that current page.
             * @param {object} argument.model Returns the grid model.				
             * @param {string} argument.originalEventType Returns current action event type.
             * @param {string} argument.requestType Returns request type.
             * @param {number} argument.startIndex Returns the start row index of the current page.
             * @param {object} argument.target Returns grid element.
             * @param {string} argument.type Returns the name of the event.
             
             * @param {object} argument Arguments in actionComplete after grid sorting action is completed.				
             * @param {boolean} argument.cancel Returns the cancel option value.
             * @param {string} argument.columnName Returns the current sorted column field name.
             * @param {string} argument.columnSortDirection Returns the column sort direction.
             * @param {object} argument.model Returns the grid model.
             * @param {string} argument.originalEventType Returns current action event type.
             * @param {string} argument.requestType Returns request type.
             * @param {object} argument.target Returns grid element.
             * @param {string} argument.type Returns the name of the event.
             
             * @param {object} argument Arguments in actionComplete after grid grouping action is completed.
             * @param {boolean} argument.cancel Returns the cancel option value.
             * @param {string} argument.columnName Returns the current grouped column field name.
             * @param {object} argument.model Returns the grid model.
             * @param {string} argument.originalEventType Returns current action event type.
             * @param {string} argument.requestType Returns request type.
             * @param {object} argument.target Returns grid element.
             * @param {string} argument.type Returns the name of the event.
             
             * @param {object} argument Arguments in actionComplete after grid record editing action is completed.
             * @param {boolean} argument.cancel Returns the cancel option value.
             * @param {object} argument.row Returns current edited row.
             * @param {object} argument.model Returns the grid model.
             * @param {string} argument.originalEventType Returns current action event type.
             * @param {string} argument.primaryKey Returns primary key.
             * @param {string} argument.primaryKeyValue Returns primary key value.
             * @param {string} argument.requestType Returns request type.
             * @param {number} argument.rowIndex Returns the edited row index.
             * @param {object} argument.target Returns grid element.
             * @param {string} argument.type Returns the name of the event.
             
             * @param {object} argument Arguments in actionComplete after grid record save action is completed.
             * @param {object} argument.cancel Returns the cancel option value.
             * @param {object} argument.data Returns the record object (JSON).
             * @param {number} argument.selectedRow Returns the selectedRow index.
             * @param {object} argument.foreignKeyData Returns the foreign key record object (JSON).
             * @param {object} argument.model Returns the grid model.
             * @param {string} argument.originalEventType Returns current action event type.
             * @param {string} argument.requestType Returns request type.
             * @param {object} argument.target Returns grid element.
             * @param {string} argument.type Returns the name of the event.
             
             * @param {object} argument Arguments in actionComplete after grid record cancel action is completed.
             * @param {boolean} argument.cancel Returns the cancel option value.
             * @param {object} argument.model Returns the grid model.
             * @param {string} argument.originalEventType Returns current action event type.
             * @param {string} argument.requestType Returns request type.
             * @param {object} argument.target Returns grid element.
             * @param {string} argument.type Returns the name of the event.
             
             * @param {object} argument Arguments in actionComplete after grid record delete action is completed.
             * @param {boolean} argument.cancel Returns the cancel option value.
             * @param {object} argument.data Returns the record object (JSON).
             * @param {object} argument.foreignKeyData Returns the foreign key record object (JSON).
             * @param {object} argument.model Returns the grid model.
             * @param {string} argument.originalEventType Returns current action event type.
             * @param {string} argument.requestType Returns request type.
             * @param {object} argument.target Returns grid element.
             * @param {object} argument.tr Returns selected row for delete.
             * @param {string} argument.type Returns the name of the event.
             
             * @param {object} argument Arguments in actionComplete after add new record action is completed.
             * @param {boolean} argument.cancel Returns the cancel option value.
             * @param {object} argument.data Returns empty record object (JSON).
             * @param {object} argument.foreignKeyData Returns the foreign key record object (JSON).
             * @param {object} argument.model Returns the grid model.
             * @param {string} argument.originalEventType Returns current action event type.
             * @param {string} argument.requestType Returns request type.
             * @param {object} argument.target Returns grid element.
             * @param {string} argument.type Returns the name of the event.
             
             * @param {object} argument Arguments in actionComplete after grid filtering action is completed.
             * @param {boolean} argument.cancel Returns the cancel option value.
             * @param {string} argument.currentFilteringColumn Returns current filtering column field name.
             * @param {object} argument.filterCollection Returns filter details.
             * @param {object} argument.model Returns the grid model.
             * @param {string} argument.originalEventType Returns current action event type.
             * @param {string} argument.requestType Returns request type.
             * @param {object} argument.target Returns grid element.
             * @param {string} argument.type Returns the name of the event.
             * @example
             * &lt;div id="Grid"&gt;&lt;/div&gt; 
             * &lt;script&gt;
             * $("#Grid").ejGrid({
             *    actionComplete: function (args) {}
             * }); 
             * &lt;/script&gt;
             */
            actionComplete: null,
            /**    
             * Triggered before the record is going to be edited.
             * @remarks
             * allowEditing and allowSelection must be set as true
             * @event
             * @name ejGrid#beginEdit 	
             * @param {Object} argument Arguments when beginEdit event is triggered.  
             * @param {boolean} argument.cancel Returns the cancel option value.
             * @param {object} argument.row Returns the current edited row.
             * @param {object} argument.model Returns the grid model.
             * @param {string} argument.primaryKey Returns primary key.
             * @param {string} argument.primaryKeyValue Returns primary key value.
             * @param {number} argument.rowIndex Returns the edited row index.
             * @param {string} argument.type Returns the name of the event.
             * @example
             * &lt;div id="Grid"&gt;&lt;/div&gt; 
             * &lt;script&gt;
             * $("#Grid").ejGrid({
             *    beginEdit: function (args) {}
             * });
             * &lt;/script&gt;
             */
            beginEdit: null,
		    /**    
             * Triggered after the record is edited.
             * @event
             * @name ejGrid#endEdit 	
             * @param {Object} argument Arguments when endEdit event is triggered.  
             * @param {object} argument.model Returns the grid model.
		     * @param {object} argument.data Returns modified data.             
             * @param {string} argument.type Returns the name of the event.
             * @example 
             * &lt;div id="Grid"&gt;&lt;/div&gt; 
             * &lt;script&gt;
             * $("#Grid").ejGrid({
             *    endEdit: function (args) {}
             * });
             * &lt;/script&gt;
             */
            endEdit: null,
	        /**    
             * Triggered after the record is added.
             * @event
             * @name ejGrid#endAdd 	
             * @param {Object} argument Arguments when endAdd event is triggered.  
             * @param {object} argument.model Returns the grid model.
		     * @param {object} argument.data Returns added data.
             * @param {string} argument.type Returns the name of the event.             
             * @example 
             * &lt;div id="Grid"&gt;&lt;/div&gt; 
             * &lt;script&gt;
             * $("#Grid").ejGrid({
             *    endAdd: function (args) {}
             * });
             * &lt;/script&gt;
             */
            endAdd: null,
		    /**    
             * Triggered after the record is deleted.
             * @event
             * @name ejGrid#endDelete 	
             * @param {Object} argument Arguments when endDelete event is triggered.  
             * @param {object} argument.model Returns the grid model.
		     * @param {object} argument.data Returns deleted data.
             * @param {string} argument.type Returns the name of the event.             
             * @example
             * &lt;div id="Grid"&gt;&lt;/div&gt; 
             * &lt;script&gt;
             * $("#Grid").ejGrid({
             *    endDelete: function (args) {}
             * });
             * &lt;/script&gt;
             */
            endDelete: null,
			 /**    
             * Triggered before the batch add.
             * @event
             * @name ejGrid#beforeBatchAdd 	
             * @param {Object} argument Arguments when beforeBatchAdd event is triggered.  
             * @param {object} argument.model Returns the grid model.
		     * @param {object} argument.data Returns deleted data.
             * @param {string} argument.type Returns the name of the event.
             * @param {object} argument.defaultData Returns the default data object.
             * @param {string} argument.primaryKey Returns primaryKey.
             * @example
             * &lt;div id="Grid"&gt;&lt;/div&gt; 
             * &lt;script&gt;
             * $("#Grid").ejGrid({
             *    beforeBatchAdd: function (args) {}
             * });
             * &lt;/script&gt;
             */
            beforeBatchAdd: null,
            /**    
             * Triggered before the batch save.
             * @event
             * @name ejGrid#beforeBatchSave 	
             * @param {Object} argument Arguments when beforeBatchSave event is triggered.  
             * @param {object} argument.model Returns the grid model.
		     * @param {object} argument.data Returns deleted data.
             * @param {string} argument.type Returns the name of the event.
             * @param {object} argument.batchChanges Returns the changed record object.
             * @example
             * &lt;div id="Grid"&gt;&lt;/div&gt; 
             * &lt;script&gt;
             * $("#Grid").ejGrid({
             *    beforeBatchSave: function (args) {}
             * });
             * &lt;/script&gt;
             */
            beforeBatchSave: null,
            /**    
             * Triggered before the batch delete.
             * @event
             * @name ejGrid#beforeBatchDelete 	
             * @param {Object} argument Arguments when beforeBatchDelete event is triggered.  
             * @param {object} argument.model Returns the grid model.
		     * @param {object} argument.data Returns deleted data.
             * @param {string} argument.type Returns the name of the event.
             * @param {string} argument.primaryKey Returns primaryKey.
             * @param {number} argument.rowIndex Returns the row index.
             * @param {object} argument.rowData Returns the row data.
             * @param {object} argument.row Returns the row element.
             * @example
             * &lt;div id="Grid"&gt;&lt;/div&gt; 
             * &lt;script&gt;
             * $("#Grid").ejGrid({
             *    beforeBatchDelete: function (args) {}
             * });
             * &lt;/script&gt;
             */
            beforeBatchDelete: null,
            /**    
             * Triggered when record batch add.
             * @event
             * @name ejGrid#batchAdd 	
             * @param {Object} argument Arguments when batchAdd event is triggered.  
             * @param {object} argument.model Returns the grid model.
		     * @param {object} argument.data Returns deleted data.
             * @param {string} argument.type Returns the name of the event.
             * @param {object} argument.columnObject Returns the column object.
             * @param {number} argument.columnIndex Returns the column index.
             * @param {object} argument.row Returns the row element.
             * @param {object} argument.primaryKey Returns the primaryKey.            
             * @param {object} argument.cell Returns the cell object.   
             * @example
             * &lt;div id="Grid"&gt;&lt;/div&gt; 
             * &lt;script&gt;
             * $("#Grid").ejGrid({
             *    batchAdd: function (args) {}
             * });
             * &lt;/script&gt;
             */
            batchAdd: null,
            /**    
             * Triggered when record batch delete.
             * @event
             * @name ejGrid#batchDelete 	
             * @param {Object} argument Arguments when batchDelete event is triggered.  
             * @param {object} argument.model Returns the grid model.
		     * @param {object} argument.data Returns deleted data.
             * @param {string} argument.type Returns the name of the event.              
             * @example
             * &lt;div id="Grid"&gt;&lt;/div&gt; 
             * &lt;script&gt;
             * $("#Grid").ejGrid({
             *    batchDelete: function (args) {}
             * });
             * &lt;/script&gt;
             */
            batchDelete: null,
            /**    
             * Triggered when record cell save.
             * @event
             * @name ejGrid#cellSave 	
             * @param {Object} argument Arguments when cellSave event is triggered.  
             * @param {object} argument.model Returns the grid model.
		     * @param {object} argument.data Returns deleted data.
             * @param {string} argument.type Returns the name of the event.
             * @param {string} argument.columnName Returns the column name.
             * @param {string} argument.value Returns the cell value.
             * @param {object} argument.rowData Returns the row data object.
             * @param {string} argument.previousValue Returns the previous value of the cell.
             * @param {object} argument.columnObject Returns the column object.
             * @param {object} argument.cell Returns the cell object.
             * @param {boolean} argument.isForeignKey Returns isForeignKey option value.
             * @example
             * &lt;div id="Grid"&gt;&lt;/div&gt; 
             * &lt;script&gt;
             * $("#Grid").ejGrid({
             *    cellSave: function (args) {}
             * });
             * &lt;/script&gt;
             */
            cellSave: null,            
            /**    
             * Triggered when record cell edit.
             * @event
             * @name ejGrid#cellEdit 	
             * @param {Object} argument Arguments when cellEdit event is triggered.  
             * @param {object} argument.model Returns the grid model.
		     * @param {object} argument.data Returns deleted data.
             * @param {string} argument.type Returns the name of the event.
             * @param {object} argument.validationRules Returns the validation rules.
             * @param {string} argument.columnName Returns the column name.
             * @param {string} argument.value Returns the cell value.
             * @param {object} argument.rowData Returns the row data object.
             * @param {string} argument.previousValue Returns the previous value of the cell.
             * @param {object} argument.columnObject Returns the column object.
             * @param {object} argument.cell Returns the cell object.
             * @param {boolean} argument.isForeignKey Returns isForeignKey option value.
             * @example
             * &lt;div id="Grid"&gt;&lt;/div&gt; 
             * &lt;script&gt;
             * $("#Grid").ejGrid({
             *    cellEdit: function (args) {}
             * });
             * &lt;/script&gt;
             */
            cellEdit: null,
            /**    
             * Triggered when column resize start.
             * @event
             * @name ejGrid#resizeStart 	
             * @param {Object} argument Arguments when resizeStart event is triggered.  
             * @param {object} argument.model Returns the grid model.
		     * @param {object} argument.data Returns deleted data.
             * @param {string} argument.type Returns the name of the event.             
             * @param {number} argument.columnIndex Returns the column index.            
             * @param {object} argument.column Returns the column object.            
             * @param {object} argument.target Returns the grid object.
             * @param {number} argument.oldWidth Returns the old width value.            
             * @example
             * &lt;div id="Grid"&gt;&lt;/div&gt; 
             * &lt;script&gt;
             * $("#Grid").ejGrid({
             *    resizeStart: function (args) {}
             * });
             * &lt;/script&gt;
             */
            resizeStart: null,
            /**    
             * Triggered when column resize end.
             * @event
             * @name ejGrid#resizeEnd 	
             * @param {Object} argument Arguments when resizeEnd event is triggered.  
             * @param {object} argument.model Returns the grid model.
		     * @param {object} argument.data Returns deleted data.
             * @param {string} argument.type Returns the name of the event.             
             * @param {number} argument.columnIndex Returns the column index.            
             * @param {object} argument.column Returns the column object.            
             * @param {object} argument.target Returns the grid object.
             * @param {number} argument.oldWidth Returns the old width value. 
             * @param {number} argument.newWidth Returns the new width value.
             * @param {number} argument.extra Returns the extra width value.            
             * @example
             * &lt;div id="Grid"&gt;&lt;/div&gt; 
             * &lt;script&gt;
             * $("#Grid").ejGrid({
             *    resizeEnd: function (args) {}
             * });
             * &lt;/script&gt;
             */
            resizeEnd: null,
            /**    
             * Triggered after column resized.
             * @event
             * @name ejGrid#resized 	
             * @param {Object} argument Arguments when resized event is triggered.  
             * @param {object} argument.model Returns the grid model.
		     * @param {object} argument.data Returns deleted data.
             * @param {string} argument.type Returns the name of the event.             
             * @param {number} argument.columnIndex Returns the column index.            
             * @param {object} argument.column Returns the column object.            
             * @param {object} argument.target Returns the grid object.
             * @param {number} argument.oldWidth Returns the old width value. 
             * @param {number} argument.newWidth Returns the new width value.                      
             * @example
             * &lt;div id="Grid"&gt;&lt;/div&gt; 
             * &lt;script&gt;
             * $("#Grid").ejGrid({
             *    resized: function (args) {}
             * });
             * &lt;/script&gt;
             */
            resized: null,
            /**    
             * Triggered initial load.
             * @event
             * @name ejGrid#load 	
             * @param {Object} argument Arguments when load event is triggered.  
             * @param {object} argument.model Returns the grid model.
		     * @param {object} argument.data Returns deleted data.
             * @param {string} argument.type Returns the name of the event.                                       
             * @example
             * &lt;div id="Grid"&gt;&lt;/div&gt; 
             * &lt;script&gt;
             * $("#Grid").ejGrid({
             *    load: function (args) {}
             * });
             * &lt;/script&gt;
             */
            load: null,
            /**    
             * Triggered when grid going to destroy.
             * @event
             * @name ejGrid#destroy 	
             * @param {Object} argument Arguments when destroy event is triggered.  
             * @param {object} argument.model Returns the grid model.
		     * @param {object} argument.data Returns deleted data.
             * @param {string} argument.type Returns the name of the event.                                       
             * @example
             * &lt;div id="Grid"&gt;&lt;/div&gt; 
             * &lt;script&gt;
             * $("#Grid").ejGrid({
             *    destroy: function (args) {}
             * });
             * &lt;/script&gt;
             */
            destroy: null,
            /**    
             * Triggered before the row is going to be selected.
             * @remarks
             * allowSelection must be set as true
             * @event
             * @name ejGrid#rowSelecting
             * @param {Object} argument Arguments when rowSelecting event is triggered.  
             * @param {number} argument.rowIndex Returns the selected row index value.
             * @param {object} argument.row Returns the selected row element.
             * @param {object} argument.data Returns current record object (JSON).
             * @param {boolean} argument.cancel Returns the cancel option value.
             * @param {object} argument.model Returns the grid model.
             * @param {string} argument.type Returns the name of the event.
             * @example
             * &lt;div id="Grid"&gt;&lt;/div&gt; 
             * &lt;script&gt;
             * $("#Grid").ejGrid({
             *    rowSelecting: function (args) {}
             * });
             * &lt;/script&gt;
             */
            rowSelecting: null,
            /**    
             * Triggered after the row is selected.
             * @remarks
             * allowSelection must be set as true
             * @event
             * @name ejGrid#rowSelected 	
             * @param {Object} argument Arguments when rowSelected event is triggered.  
             * @param {boolean} argument.cancel Returns the cancel option value.
             * @param {object} argument.data Returns current record object (JSON).
             * @param {object} argument.foreignKeyData Returns the foreign key record object (JSON).
             * @param {number} argument.rowIndex Returns the row index of the selected row.
             * @param {object} argument.row Returns the current selected row.
             * @param {object} argument.model Returns the grid model.
             * @param {string} argument.type Returns the name of the event.
             * @example 
             * &lt;div id="Grid"&gt;&lt;/div&gt; 
             * &lt;script&gt;
             * $("#Grid").ejGrid({
             *    rowSelected: function (args) {}
             * });
             * &lt;/script&gt;
             */
            rowSelected: null,
            /**    
			 * Triggered when column dragging begins.
			 * @event
             * @remarks
             * allowGrouping or allowReordering must be set as true
			 * @name ejGrid#columnDragStart 	
			 * @param {Object} argument Arguments when columnDragStart event is triggered.  
			 * @param {boolean} argument.cancel Returns the cancel option value.
			 * @param {object} argument.draggableType Returns draggable element type.
             * @param {object} argument.column Returns the draggable column object.
			 * @param {object} argument.model Returns the grid model.
			 * @param {object} argument.target Returns drag start element.
			 * @param {string} argument.type Returns the name of the event.
			 * @example
             * &lt;div id="Grid"&gt;&lt;/div&gt; 
             * &lt;script&gt;
			 * $("#Grid").ejGrid({
			 *     columnDragStart: function (args) {}
			 * });
             * &lt;/script&gt;
			 */
            columnDragStart: null,
            /**    
             * Triggered when the column is being dragged.
             * @remarks
             * allowGrouping or allowReordering must be set as true
             * @event
             * @name ejGrid#columnDrag 	
             * @param {Object} argument Arguments when columnDrag event is triggered.  
             * @param {boolean} argument.cancel Returns the cancel option value.
             * @param {object} argument.draggableType Returns draggable element type.
             * @param {object} argument.column Returns the draggable column object.
             * @param {object} argument.model Returns the grid model.
             * @param {object} argument.target Returns target elements based on mouse move position.
             * @param {string} argument.type Returns the name of the event.
             * @example
             * &lt;div id="Grid"&gt;&lt;/div&gt; 
             * &lt;script&gt;
             * $("#Grid").ejGrid({
             *    columnDrag: function (args) {}
             * });
             * &lt;/script&gt;
             */
            columnDrag: null,
            /**    
             * Triggered when the column is dropped.
             * @remarks
             * allowGrouping or allowReordering must be set as true
             * @event
             * @name ejGrid#columnDrop 	
             * @param {Object} argument Arguments when columnDrop event is triggered.  
             * @param {boolean} argument.cancel Returns the cancel option value.
             * @param {object} argument.draggableType Returns draggable element type.
             * @param {object} argument.column Returns the draggable column object.
             * @param {object} argument.model Returns the grid model.
             * @param {object} argument.target Returns dropped dragged element.
             * @param {string} argument.type Returns the name of the event.
             * @example
             * &lt;div id="Grid"&gt;&lt;/div&gt;
             * &lt;script&gt;
             * $("#Grid").ejGrid({
             *  columnDrop: function (args) {}
             * });
             * &lt;/script&gt;
             */
            columnDrop: null,
            /**    
             * Triggered when record is clicked.
             * @remarks
             * allowSelection must be set as true
             * @event
             * @name ejGrid#recordClick 	
             * @param {Object} argument Arguments when recordClick event is triggered.  
             * @param {boolean} argument.cancel Returns the cancel option value.
             * @param {object} argument.data Returns current record object (JSON).
             * @param {number} argument.rowIndex Returns the row index of the selected row.
             * @param {object} argument.row Returns the current selected row.
             * @param {object} argument.model Returns the grid model.
             * @param {string} argument.type Returns the name of the event.
             * @example
             * &lt;div id="Grid"&gt;&lt;/div&gt; 
             * &lt;script&gt;
             * $("#Grid").ejGrid({
             *    recordClick: function (args) {}
             * });
             * &lt;/script&gt;
             */
            recordClick: null,
            /**    
             * Triggered when record is double clicked.
             * @remarks
             * allowSelection must be set as true
             * @event
             * @name ejGrid#recordDoubleClick 	
             * @param {Object} argument Arguments when recordDoubleClick event is triggered.  
             * @param {boolean} argument.cancel Returns the cancel option value.
             * @param {object} argument.data Returns current record object (JSON).
             * @param {number} argument.rowIndex Returns the row index of the selected row.
             * @param {object} argument.row Returns the current selected row.
             * @param {object} argument.model Returns the grid model.
             * @param {string} argument.type Returns the name of the event.
             * @example
             * &lt;div id="Grid"&gt;&lt;/div&gt; 
             * &lt;script&gt;
             * $("#Grid").ejGrid({
             *    recordDoubleClick: function (args) {}
             * });
             * &lt;/script&gt;
             */
            recordDoubleClick: null,
            /**    
             * Triggered when right clicked on grid element.
             * @event
             * @name ejGrid#rightClick 	
             * @param {Object} argument Arguments when rightClick event is triggered.  
             * @param {boolean} argument.cancel Returns the cancel option value.
             * @param {object} argument.currentData Returns current record object (JSON).
             * @param {number} argument.rowIndex Returns the row index of the selected row.
             * @param {object} argument.row Returns the current selected row.
             * @param {object} argument.data Returns the selected row data object.
             * @param {number} argument.cellIndex Returns the cell index of the selected cell.
             * @param {string} argument.cellValue Returns the cell value.
             * @param {object} argument.cell Returns the cell object.
             * @param {object} argument.model Returns the grid model.
             * @param {string} argument.type Returns the name of the event.
             * @example
             * &lt;div id="Grid"&gt;&lt;/div&gt; 
             * &lt;script&gt;
             * $("#Grid").ejGrid({
             *    rightClick: function (args) {}
             * });
             * &lt;/script&gt;
             */
            rightClick: null,
            /**    
             * Triggered when detail template row is clicked to collapse.
             * @remarks
             * detailsTemplate and detailsDataBound must be specified.
             * @event
             * @name ejGrid#detailsCollapse 	
             * @param {Object} argument Arguments when detailsCollapse event is triggered.  
             * @param {boolean} argument.cancel Returns the cancel option value.
             * @param {object} argument.detailsRow Returns detail row element.
             * @param {object} argument.masterData Returns master row of detail row record object (JSON).
             * @param {object} argument.foreignKeyData Returns the foreign key record object (JSON).
             * @param {object} argument.masterRow Returns master row element.
             * @param {object} argument.model Returns the grid model.
             * @param {string} argument.type Returns the name of the event.
             * @example
             * &lt;div id="Grid"&gt;&lt;/div&gt; 
             * &lt;script&gt;
             * $("#Grid").ejGrid({
             *    detailsCollapse: function (args) {}
             * });
             * &lt;/script&gt;
             */
            detailsCollapse: null,
            /**    
             * Triggered when detail template row is clicked to expand.
             * @remarks
             * detailsTemplate and detailsDataBound must be specified.
             * @event
             * @name ejGrid#detailsExpand 	
             * @param {Object} argument Arguments when detailsExpand event is triggered.  
             * @param {boolean} argument.cancel Returns the cancel option value.
             * @param {object} argument.detailsRow Returns detail row element.
             * @param {object} argument.masterData Returns master row of detail row record object (JSON).
             * @param {object} argument.foreignKeyData Returns the foreign key record object (JSON).
             * @param {object} argument.masterRow Returns master row element.
             * @param {object} argument.model Returns the grid model.
             * @param {string} argument.type Returns the name of the event.
             * @example 
             * &lt;div id="Grid"&gt;&lt;/div&gt; 
             * &lt;script&gt;
             * $("#Grid").ejGrid({
             *    detailsExpand: function (args) {}
             * });
             * &lt;/script&gt;
             */
            detailsExpand: null,
            /**    
			 * Triggered when toolbar item is clicked in grid.
             * @remarks
             * showToolbar must be set as true and also add the toolbarItems.
			 * @event
			 * @name ejGrid#toolBarClick 	
			 * @param {Object} argument Arguments when toolBarClick event is triggered.  
			 * @param {boolean} argument.cancel Returns the cancel option value.
			 * @param {object} argument.currentTarget Returns the current item.
			 * @param {object} argument.model Returns the grid model.
			 * @param {boolean} argument.status Returns the status of toolbar item which denotes its enabled state
			 * @param {object} argument.target Returns the target item.
			 * @param {string} argument.type Returns the name of the event.
			 * @example
             * &lt;div id="Grid"&gt;&lt;/div&gt; 
             * &lt;script&gt;
			 * $("#Grid").ejGrid({
			 *    toolbarClick: function (args) {}
			 * });
             * &lt;/script&gt;
			 */
            toolbarClick: null,
            /**    
			 * Triggered before the context menu is opened.
             * @remarks
             * enableContextMenu must be set as true and also add the contextMenuItems.
			 * @event
			 * @name ejGrid#contextOpen	
			 * @param {Object} argument Arguments when contextOpen event is triggered.  
             * @param {boolean} argument.cancel Returns the cancel option value.
             * @param {object} argument.currentTarget Returns the current item.
             * @param {object} argument.model Returns the grid model.
             * @param {boolean} argument.status Returns the status of contextmenu item which denotes its enabled state
             * @param {object} argument.target Returns the target item.
             * @param {string} argument.type Returns the name of the event.
			 * @example
             * &lt;div id="Grid"&gt;&lt;/div&gt; 
             * &lt;script&gt;
			 * $("#Grid").ejGrid({
			 *    contextOpen: function (args) {}
			 * });
             * &lt;/script&gt;
			 */

            contextOpen: null,
            /**    
            * Triggered when context menu item is clicked
            * @remarks
            * enableContextMenu must be set as true and also add the contextMenuItems.
			* @event
            * @name ejGrid#contextClick 	
            * @param {Object} argument Arguments when contextClick event is triggered.  
            * @param {boolean} argument.cancel Returns the cancel option value.
            * @param {object} argument.currentTarget Returns the current item.
            * @param {object} argument.model Returns the grid model.
            * @param {boolean} argument.status Returns the status of contextmenu item which denotes its enabled state
            * @param {object} argument.target Returns the target item.
            * @param {string} argument.type Returns the name of the event.
            * @example
            * &lt;div id="Grid"&gt;&lt;/div&gt; 
            * &lt;script&gt;
            * $("#Grid").ejGrid({
            *    contextClick: function (args) {}
            * });
            * &lt;/script&gt;
            */
            contextClick: null,

            /**		
             * Gets or sets an object  that indicates to render the grid with specified columns
             * @name ejGrid#columns 
             * @default []
             * @type {array}
             * @example 
             * &lt;div id="Grid"&gt;&lt;/div&gt; 
             * &lt;script&gt;
             * $("#Grid").ejGrid({
             *   dataSource:window.gridData
             * });
             * var value = $("#Grid").ejGrid("option", "columns");
             * &lt;/script&gt; 
             */

            /**
             * Gets or sets a value that indicates to display the columns in the grid mapping  with column name of the dataSource.
             * @remarks
             * We must need to specify the field property of the column because the columns which are specified in the field property is only bound to the grid.
             * @name ejGrid#columns->field 	          
             * @default ""
             * @type {string}
             * @example
             * &lt;div id="Grid"&gt;&lt;/div&gt; 
             * &lt;script&gt;
             * $("#Grid").ejGrid({
             *   dataSource:window.gridData,
             *   columns:[{field:"OrderID"}]
             * });
             * &lt;/script&gt; 
             */

            /**
			 * Gets or sets a value that indicates to display the title of that particular column.
             * @remarks
             * It is optional if the headerText is not mentioned then it will consider the field name as headerText
             * @name ejGrid#columns->headerText 	          
             * @default ""
             * @type {string}
             * @example
             * &lt;div id="Grid"&gt;&lt;/div&gt; 
             * &lt;script&gt;
             * $("#Grid").ejGrid({
             *   dataSource:window.gridData,
             *   columns:[{field:"OrderID",headerText:"Order ID"},{field:"CustomerID",headerText:"Customer ID"}]
             * });
             * &lt;/script&gt; 
			 */

            /**
			 * Gets or sets a value that indicates whether to enable dynamic grouping behaviour for particular column.
             * @remarks
             * It is optional if we need to disable the grouping behaviour for particular column then set this property as false.
             * @name ejGrid#columns->allowGrouping 	          
             * @default true
             * @type {boolean}
             * @example
             * &lt;div id="Grid"&gt;&lt;/div&gt; 
             * &lt;script&gt;
             * $("#Grid").ejGrid({
             *   dataSource:window.gridData,
             *   allowGrouping:true,
             *   columns:[{field:"OrderID"},{field:"CustomerID",allowGrouping:false},{field:"ShipCity"}]
             * });
             * &lt;/script&gt; 
			 */

            /**
             * Gets or sets a value that indicates whether to enable dynamic filtering behaviour for particular column.
             * @remarks
             * It is optional if we need to disable the filtering behaviour for particular column then set this property as false.
             * @name ejGrid#columns->allowFiltering 	          
             * @default true
             * @type {boolean}
             * @example
             * &lt;div id="Grid"&gt;&lt;/div&gt; 
             * &lt;script&gt;
             * $("#Grid").ejGrid({
             *   dataSource:window.gridData,
             *   allowFiltering:true,
             *   columns:[{field:"OrderID"},{field:"CustomerID",allowFiltering:false},{field:"ShipCity"}]
             * });
             * &lt;/script&gt; 
             */

            /**
             * Gets or sets a value that indicates whether to enable dynamic sorting behaviour for particular column.
             * @remarks
             * It is optional if we need to disable the sorting behaviour for particular column then set this property as false.
             * @name ejGrid#columns->allowSorting 	          
             * @default true
             * @type {boolean}
             * @example
             * &lt;div id="Grid"&gt;&lt;/div&gt; 
             * &lt;script&gt;
             * $("#Grid").ejGrid({
             *   dataSource:window.gridData,
             *   allowSorting:true,
             *   columns:[{field:"OrderID"},{field:"CustomerID",allowSorting:false},{field:"ShipCity"}] 
             * });
             * &lt;/script&gt; 
             */

            /**
             * Gets or sets a value that indicates whether to enable editing behaviour for particular column.
             * @remarks
             * It is optional if we need to disable the editing behaviour for particular column then set this property as false.
             * @name ejGrid#columns->allowEditing 	          
             * @default true
             * @type {boolean}
             * @example
             * &lt;div id="Grid"&gt;&lt;/div&gt; 
             * &lt;script&gt;
             * $("#Grid").ejGrid({
             *   dataSource:window.gridData,
             *   editSettings:{allowEditing:true},
             *   columns:[{field:"OrderID"},{field:"CustomerID",allowEditing:false},{field:"ShipCity"}] 
             * });
             * &lt;/script&gt; 
             */

            /**
             * Gets or sets a value that indicates whether to bind the column which are not in the datasource 
             * @remarks
             * If we need to bind the column to the grid which is not in the datasource then we must enable isUnBound property.
             * There is no need to specify field property if the column is unBound.
             * @name ejGrid#columns->isUnbound 	          
             * @default false
             * @type {boolean}
             * @example
             * &lt;div id="Grid"&gt;&lt;/div&gt; 
             * &lt;script&gt;
             * $("#Grid").ejGrid({
             *   dataSource:window.gridData,
             *   columns:[{field:"OrderID",isUnbound:false},{field:"CustomerID"},{field:"ShipCity",headerText:"ShipCity"}]
             * });
             * &lt;/script&gt; 
             */

            /**
             * Gets or sets a value that indicates whether to  enables column template for a particular column.
             * @remarks
             * Need to bind the templateID, if the columnTemplate is set as true.
             * @name ejGrid#columns->template 	          
             * @default false
             * @type {boolean}
             * @example
             * &lt;div id="Grid"&gt;&lt;/div&gt; 
             * &lt;script id="columnTemplate" type="text/x-jsrender"&gt;
             * &lt;img src="styles/images/Employees/{{:EmployeeID}}.png" alt="{{:EmployeeID}}"/&gt;
             * &lt;/script&gt;
             * &lt;script&gt;
             * $("#Grid").ejGrid({
             *  dataSource:window.gridData,
             *  columns:[{headerText:"Employee",template:true,templateID:"#columnTemplate"},{field:"EmployeeID"}]
             * });
             * &lt;/script&gt;
             */

            /**
             * Gets or sets a value that indicates to render the element(based on edit type)  for editing the grid record. . See {@link editingType}
             * @remarks
             * allowEditing must be set as true.
             * It is optional.
             * @name ejGrid#columns->editType 	          
             * @default ej.Grid.EditingType.String
             * @type {enum}
             * @example
             * &lt;div id="Grid"&gt;&lt;/div&gt; 
             * &lt;script&gt;
             * $("#Grid").ejGrid({
             *   dataSource:window.gridData,
             *   editSettings:{allowEditing:true},
             *   columns:[{field:"OrderID"},{field:"CustomerID"},{field:"Freight",editType:ej.Grid.EditingType.Numeric, editParams: { decimalCount: 2 }}]
             * });
             * &lt;/script&gt; 
             */

            /**
            * Gets or sets a template that displays a custom editor used to edit column values. See {@link editTemplate}
            * @remarks
            * allowEditing must be set as true.
            * It is optional.
            * @default null
            * @type {object}
            * @name ejGrid#columns->editTemplate 	          
            * @example
            * &lt;div id="Grid"&gt;&lt;/div&gt; 
            * &lt;script&gt;
            * $("#Grid").ejGrid({
            *   dataSource:window.gridData,
            *   editSettings:{allowEditing:true},
            *   columns:[{field:"OrderID"},{field:"CustomerID"},
            *             { field: "ShipName",editTemplate: { create: function () { return "&lt;input&gt;"; }, read: function (args) { return args.ejMaskEdit("getValue"); }, write: function (args) { args.element.ejMaskEdit({ width: "100%" }); } } }]
            * });
            * &lt;/script&gt; 
            */

            /**
             * Gets or sets a value that indicates to align the text within the column. See {@link textAlign}
             * @remarks
             * It is optional.
             * @name ejGrid#columns->textAlign 	          
             * @default ej.TextAlign.Left
             * @type {enum}
             * @example
             * &lt;div id="Grid"&gt;&lt;/div&gt; 
             * &lt;script&gt;
             * $("#Grid").ejGrid({
             *   dataSource:window.gridData,
             *   columns:[{field:"OrderID",textAlign:ej.TextAlign.Center},{field:"CustomerID",textAlign:ej.TextAlign.Right},{field:"ShipCity"}]
             * });
             * &lt;/script&gt; 
             */

            /**
             * Gets or sets a value that indicates the column is act as a primary key(read-only) of the grid. The editing is performed based on the primary key column
             * @remarks
             * When editing is enable then we must set isPrimaryKey property as true for the primary key column.
             * @name ejGrid#columns->isPrimaryKey 	          
             * @default false
             * @type {boolean}
             * @example
             * &lt;div id="Grid"&gt;&lt;/div&gt; 
             * &lt;script&gt;
             * $("#Grid").ejGrid({             
             *   dataSource:window.gridData,
             *   editSettings:{allowEditing:true},
             *   columns:[{field:"OrderID",isPrimaryKey:true},{field:"CustomerID"},{field:"ShipCity"}]
             * });
             * &lt;/script&gt; 
             */

            /**
             * Gets or sets a value that indicates thecolumn has an identity in the database.            
             * @name ejGrid#columns->isIdentity 	          
             * @default false
             * @type {boolean}
             * @example
             * &lt;div id="Grid"&gt;&lt;/div&gt; 
             * &lt;script&gt;
             * $("#Grid").ejGrid({
             *   dataSource:window.gridData,
             *   columns:[{field:"OrderID"},{field:"CustomerID",isIdentity:true},{field:"ShipCity"}]
             * });
             * &lt;/script&gt; 
             */

            /**
             * Gets or sets a value that indicates whether this column is visible in the grid.
             * @remarks
             * It is optional.
             * @name ejGrid#columns->visible 	          
             * @default true
             * @type {boolean}
             * @example
             * &lt;div id="Grid"&gt;&lt;/div&gt; 
             * &lt;script&gt;
             * $("#Grid").ejGrid({
             *   dataSource:window.gridData,
             *   columns:[{field:"OrderID"},{field:"CustomerID",visible:false},{field:"ShipCity"}]
             * });
             * &lt;/script&gt; 
             */

            /**
			 * Gets or sets a value that indicates to bind the external datasource to the particular column when columnEditType as "dropdownedit" and also it is used to bind the datasource to the foreign key column while editing the grid. 
             * //Where data is array of JSON objects of text and value for the drop-down and array of JSON objects for foreign key column.
             * @remarks
             * We must specify the dataSource property.
             * @name ejGrid#columns->dataSource 	          
             * @default null
             * @type {array}
             * @example
             * &lt;div id="Grid"&gt;&lt;/div&gt;  
             * &lt;script&gt;
             * $("#Grid").ejGrid({
             *   dataSource:window.gridData,
             *   columns:[{field:"OrderID"},{field:"CustomerID",visible:false},{field:"ShipCity"}]
             * });
             * &lt;/script&gt;
             */

            /**
             * Gets or sets a value that indicates to provide custom css for an individual column.
             * @name ejGrid#columns->cssClass 	          
             * @default -
             * @type {string}
             * @example
             * &lt;div id="Grid"&gt;&lt;/div&gt; 
             * &lt;style class="temp"&gt;
             * .temp{
             * color:green;}
             * &lt;/style&gt;
             * &lt;script&gt;
             * $("#Grid").ejGrid({
             *   dataSource:window.gridData,
             *   columns:[{field:"OrderID"},{field:"CustomerID",cssClass:"temp"},{field:"ShipCity"}]
             * });
             * &lt;/script&gt; 
             */

            /**
             * Gets or sets a value that indicates to display the specified default value while adding a new record to the grid
             * @remarks
             * It is optional.allowAdding must be true while using this property.
             * @name ejGrid#columns->defaultValue 	          
             * @default -
             * @type {string/number/boolean/date}
             * @example
             * &lt;div id="Grid"&gt;&lt;/div&gt;          
             * &lt;script&gt;
             * $("#Grid").ejGrid({
             *   dataSource:window.gridData,
             *   editSettings: {allowAdding: true},
             *   toolbarSettings: { showToolbar: true, toolbarItems: [ej.Grid.ToolBarItems.Add] },
             *   columns:[{field:"OrderID"},{field:"CustomerID"},{field:"ShipCity",defaultValue:"ABC"}]
             * });
             * &lt;/script&gt;
             */

            /**
			 * Gets or sets a value that indicates to add the template within the header element of the particular column.
             * @remarks
             * It is optional.
             * @name ejGrid#columns->headerTemplateID 	          
             * @default null
             * @type {string}
             * @example
             * &lt;div id="Grid"&gt;&lt;/div&gt; 
             * &lt;div id="customerTemplate"&gt;
             * &lt;span class="e-userlogin e-icon headericon"&gt;&lt;/span&gt;
             *  CUS ID
             * &lt;/div&gt;
             * &lt;script&gt;
             * $("#Grid").ejGrid({
             *   dataSource:window.gridData,
             *   columns:[{field:"OrderID",headerTemplateID: "#customerTemplate"},{field:"CustomerID"},{field:"ShipCity"}]
             * });
             * &lt;/script&gt;
             */

            /**
			 * Gets or sets a value that indicates to define the width for a particular column in the grid.
             * @remarks
             * It is optional.
             * @name ejGrid#columns->width 	          
             * @default -
             * @type {number}
             * @example
             * &lt;div id="Grid"&gt;&lt;/div&gt;          
             * &lt;script&gt;
             * $("#Grid").ejGrid({
             *   dataSource:window.gridData,
             *   columns:[{field:"OrderID",width:50},{field:"CustomerID",width:60},{field:"ShipCity",width:90}]
             * });
             * &lt;/script&gt;
             */

            /**
             * Gets or sets a value that indicates to define foreign key field name of the grid datasource.
             * @remarks
             * It is optional. If the foreignKeyField name is not specified then it will take the field name from the "field" property.
             * We must specify the data source for column dataSource property
             * @name ejGrid#columns->foreignKeyField  	          
             * @default null
             * @type {string}
             * @example
             * &lt;div id="Grid"&gt;&lt;/div&gt;          
             * &lt;script&gt;
             * $("#Grid").ejGrid({
             *   dataSource:window.gridData,
             *   columns:[{field:"OrderID"},{field:"EmployeeID",foreignKeyField:"EmployeeID",foreignKeyValue:"FirstName",headerText:"FirstName",dataSource:window.employeeData },{field:"ShipCity"}]
             * });
             * &lt;/script&gt;
             */

            /**
             * Gets or sets a value that indicates to bind the field which is in foreign column datasource based on the foreignKeyField
             * @remarks
             * We must specify the data source for column dataSource property
             * @name ejGrid#columns->foreignKeyValue  	          
             * @default null
             * @type {string}
             * @example
             * &lt;div id="Grid"&gt;&lt;/div&gt;  
             * &lt;script&gt;
             * $("#Grid").ejGrid({
             *   dataSource:window.gridData,
             *   columns:[{field:"OrderID"},{field:"EmployeeID",foreignKeyField:"EmployeeID",foreignKeyValue:"FirstName",dataSource:window.employeeData,headerText:"FirstName"},{field:"ShipCity"}]
             * });
             * &lt;/script&gt; 
             */

            /**
             * Gets or sets a value that indicates to define constraints for saving data to the database.
             * @remarks
             * allowEditing or allowAdding must be set as true. It is must to add the validations script. They are jquery.validate.min.js and jquery.validate.unobtrusive.min.js
             * @name ejGrid#columns->validationRules 	          
             * @type {object}
             * @example
             * &lt;div id="Grid"&gt;&lt;/div&gt;          
             * &lt;script&gt;
             * $("#Grid").ejGrid({
             *   dataSource:window.gridData,
             *   editSettings: {allowEditing: true, allowAdding: true},
             *   columns:[{field:"OrderID", validationRules: { required: true, number: true }},{field:"CustomerID"},{field:"ShipCity"}] 
             * });
             * &lt;/script&gt; 
             */

            /**
             * Gets or sets a value that indicates the format for the text applied on the column
             * @name ejGrid#columns->format 	          
             * @default -
             * @type {string}
             * @example
             * &lt;div id="Grid"&gt;&lt;/div&gt;          
             * &lt;script&gt;
             * $("#Grid").ejGrid({
             *   dataSource:window.gridData,
             *   columns:[{field:"OrderID"},{field:"CustomerID"},{field:"Freight",format:"{0:C}"}]
             * });
             * &lt;/script&gt;  
             */

            /**
             * Gets or sets a value that indicates the attribute values to the td element of a particular column
             * @name ejGrid#columns->customAttributes 	          
             * @default -
             * @type {object}
             * @example
             * &lt;div id="Grid"&gt;&lt;/div&gt;          
             * &lt;script&gt;
             * $("#Grid").ejGrid({
             *   dataSource:window.gridData,
             *   columns:[{field:"OrderID"},{field:"CustomerID",customAttributes:{"style":"color:red"}},{field:"Freight"}]
             * });
             * &lt;/script&gt;  
             */

            /**
             * Gets or sets a value that indicates to add the template as a particular column data .
             * @remarks
             * If you want to use templateID for the column template, then columnTemplate must be set as true.
             * @name ejGrid#columns->templateID 	          
             * @default null
             * @type {string}
             * @example
             * &lt;div id="Grid"&gt;&lt;/div&gt; 
             * &lt;script id="columnTemplate" type="text/x-jsrender"&gt;
             * &lt;img src="styles/images/Employees/{{:EmployeeID}}.png" alt="{{:EmployeeID}}"/&gt;
             * &lt;/script&gt;
             * &lt;script&gt;
             * $("#Grid").ejGrid({
             *  dataSource:window.gridData,
             *  columns:[{headerText:"Employee",template:true,templateID:"#columnTemplate"},{field:"EmployeeID"}]
             * });
             * &lt;/script&gt;
             */

            /**
             * Gets or sets a value that indicates to render the records with an html elements
             * @remarks
             * It is optional if we need to disable the htmlencode behaviour for particular column then set this property as false
             * @name ejGrid#columns->disableHtmlEncode	          
             * @default false
             * @type {boolean}
             * @example
             * &lt;div id="Grid"&gt;&lt;/div&gt; 
             * &lt;script&gt;
             * $("#Grid").ejGrid({
             *     dataSource:window.gridData,
             *     columns:[{field:"OrderID",headerText:"Order ID",disableHtmlEncode:true}
             *   });
             * &lt;/script&gt;
            **/

            /**  
             * You can use this property to freeze selected columns in grid at the time of scrolling.
             * @remarks
             * We must enable allowScrolling property
             * @name ejGrid#columns->isFrozen            
             * @default false
             * @type {boolean}
             * @example
             * &lt;div id="Grid"&gt;&lt;/div&gt; 
             * &lt;script&gt;
             * $("#Grid").ejGrid({             
             *   dataSource:window.gridData,
             *   allowScrolling: true,               
             *   scrollSettings:{width:500,height:100 }
             *   columns:[{field:"OrderID",isFrozen:true},{field:"CustomerID"},{field:"ShipCity"}]
             * });
             * &lt;/script&gt; 
             **/

            /**
             * Gets or sets an object  that indicates to define a command column in the grid.
             * @remarks
             * isUnbound must be true while declaring commands button.
             * @name ejGrid#columns->commands 	          
             * @default []
             * @type {array}
             * @example
             * &lt;div id="Grid"&gt;&lt;/div&gt; 
             * &lt;script&gt;
             * $("#Grid").ejGrid({
             *     dataSource:window.gridData,
             *     editSettings:{allowEditing:true,allowAdding:true,allowDeleting:true},
             *     columns:[
             *            { field: "OrderID", isPrimaryKey: true },  
             *            { field: "EmployeeID" },
             *            {
             *               headerText: "Manage Records",
             *               commands: [
             *                   { type: ej.Grid.UnboundType.Edit, buttonOptions: { text: "Edit" } },
             *                   { type: ej.Grid.UnboundType.Delete, buttonOptions: { text: "Delete" } },
             *                   { type: ej.Grid.UnboundType.Save, buttonOptions: { text: "Save" } },
             *                   { type: ej.Grid.UnboundType.Cancel, buttonOptions: { text: "Cancel" } }
             *                ],
             *               isUnbound: true,
             *               width: 130
             *            }
             *     ] 
             * });
             * &lt;/script&gt; 
             */

            /**
             * Gets or sets a value that indicates to add the command column button. See {@link unboundType}
             * @remarks
             * isUnbound must be true while declaring commands button.
             * @name ejGrid#columns->commands->type	          
             * @default -
             * @type {enum}
             * @example
             * &lt;div id="Grid"&gt;&lt;/div&gt; 
             * &lt;script&gt;
             * $("#Grid").ejGrid({
             *     dataSource:window.gridData,
             *     editSettings:{allowEditing:true,allowAdding:true,allowDeleting:true},
             *     columns:[
             *            { field: "OrderID", isPrimaryKey: true },  
             *            { field: "EmployeeID" },
             *            {
             *               headerText: "Manage Records",
             *               commands: [
             *                   { type: ej.Grid.UnboundType.Edit, buttonOptions: { text: "Edit" } },
             *                   { type: ej.Grid.UnboundType.Delete, buttonOptions: { text: "Delete" } },
             *                   { type: ej.Grid.UnboundType.Save, buttonOptions: { text: "Save" } },
             *                   { type: ej.Grid.UnboundType.Cancel, buttonOptions: { text: "Cancel" } }
             *                ],
             *               isUnbound: true, width: 130
             *            }
             *     ] 
             * });
             * &lt;/script&gt; 
             */

            /**
             * Gets or sets an object that indicates to define all the button options which are available in ejButton.
             * @remarks
             * isUnBound must be true while declaring commands button.
             * @name ejGrid#columns->commands->buttonOptions
             * @default -
             * @type {object}
             * @example
             * &lt;div id="Grid"&gt;&lt;/div&gt; 
             * &lt;script&gt;
             * $("#Grid").ejGrid({
             *     dataSource:window.gridData,
             *     editSettings:{allowEditing:true,allowAdding:true,allowDeleting:true},
             *     columns:[
             *            { field: "OrderID", isPrimaryKey: true },  
             *            { field: "EmployeeID" },
             *            {
             *               headerText: "Manage Records",
             *               commands: [
             *                   { type: ej.Grid.UnboundType.Edit, buttonOptions: { text: "Edit" } },
             *                   { type: ej.Grid.UnboundType.Delete, buttonOptions: { text: "Delete" } },
             *                   { type: ej.Grid.UnboundType.Save, buttonOptions: { text: "Save" } },
             *                   { type: ej.Grid.UnboundType.Cancel, buttonOptions: { text: "Cancel" } }
             *                ],
             *               isUnbound: true, width: 130
             *            }
             *     ] 
             * });
             * &lt;/script&gt; 
             */

            /**
             * Gets or sets a value that indicates to customize ejNumericTextbox of an editable column. See {@link editingType}
             * @name ejGrid#columns->numericEditParams 	          
             * @default -
             * @type {object}
             * @example
             * &lt;div id="Grid"&gt;&lt;/div&gt; 
             * &lt;script&gt;
             * $("#Grid").ejGrid({
             *     dataSource:window.gridData,
             *     editSettings:{allowEditing:true,allowAdding:true,allowDeleting:true},
             *     columns:[{ field: "OrderID"}, { field: "Freight", editType: ej.Grid.EditingType.Numeric, editParams: { decimalCount: 2 }}]
             *   });
             * &lt;/script&gt;  
             */

            columns: [],
			/**		
            * Query the dataSource from the table for Grid.
            * @name ejGrid#query
            * @default null
            * @type {object}
            * @example       
            * &lt;div id="Grid"&gt;&lt;/div&gt; <br> 
            * &lt;script type="text/javascript"&gt;
			* var queryOrder=ej.Query().select(["OrderID", "CustomerID"]); 
            * $("#Grid").ejGrid({
            *   dataSource: window.gridData,
            *   query: queryOrder
			* });  
            * &lt;/script&gt;
            */
            query: null,
            isEdit: false,
            /**		
            * Gets or sets a value that indicates whether the grid design has be to made responsive.
            * @name ejGrid#isResponsive
            * @default false
            * @type boolean
            * @example       
            * &lt;div id="Grid"&gt;&lt;/div&gt; <br> 
            * &lt;script type="text/javascript"&gt; 
            * $("#Grid").ejGrid({
            *   dataSource: window.gridData,
                isResponsive: true 
			* });  
            * &lt;/script&gt;
            */
            isResponsive: false,
             /**		
            * Gets or sets a value that indicates whether the grid rows has to be rendered as detail view in mobile mode
            * @name ejGrid#enableResponsiveRow
            * @default false
            * @type boolean
            * @example       
            * &lt;div id="Grid"&gt;&lt;/div&gt; <br> 
            * &lt;script type="text/javascript"&gt; 
            * $("#Grid").ejGrid({
            *   dataSource: window.gridData,
                isResponsive: true,
                enableResponsiveRow: true    
			* });  
            * &lt;/script&gt;
            */
            enableResponsiveRow: false,
        },
        dataTypes: {
            dataSource: "data",
            query:"data",
            columns: "array",
            summaryRows: "array",
            toolbarSettings: {
                toolbarItems: "array",
                customToolbarItems: "array"
            },
            contextMenuSettings: {
                contextMenuItems: "array",
                customContextMenuItems: "array"
            },
            selectionSettings: {
                selectionMode: "array"
            },
            sortSettings: {
                sortedColumns: "array"
            },
            filterSettings: {
                filteredColumns: "array",
                filterType: "enum",
                filterBarMode: "enum",
            },
            groupSettings: {
                groupedColumns: "array"
            },
            editSettings: {
                editMode: "enum",
                formPosition: "enum",
                rowPosition: "enum",
            }
        },
        /**
         * Get the content table element of grid
         * @return gridContentTable
         * @example 
         * &lt;script&gt;
         * // Create grid object.
         * var gridObj = $("#Grid").data("ejGrid");
         * // Gets content table of grid control
         * gridObj.getContentTable();
         * &lt;/script&gt;
         * @example 
         * &lt;script&gt;
         * // Gets content table of grid control
         * $("#Grid").ejGrid("getContentTable");        
         * &lt;/script&gt;
         */
        getContentTable: function () {
            return this._gridContentTable;
        },

        setGridContentTable: function (value) {
            this._gridContentTable = value;
        },
        /**
		 * Get the content div element of grid.
		 * @return gridContent
		 * @example 
		 * &lt;script&gt;
		 * // Create grid object.
		 * var gridObj = $("#Grid").data("ejGrid");
         * // Gets content of grid control
		 * gridObj.getContent(); 
		 * &lt;/script&gt;
		 * @example 
		 * &lt;script&gt;
		 * // Gets content of grid control
		 * $("#Grid").ejGrid("getContent");        
		 * &lt;/script&gt;
         */
        getContent: function () {
            return this._gridContent;
        },

        setGridContent: function (value) {
            this._gridContent = value;
        },
        /**
          * Get the header content div element of grid..
          * @return gridHeaderContent
          * @example 
          * &lt;script&gt;
          * // Create grid object.
          * var gridObj = $("#Grid").data("ejGrid");
          * // Gets grid header content of grid control
          * gridObj.getHeaderContent(); 
          * &lt;/script&gt;
          * @example 
          * &lt;script&gt;
          * // Gets grid header content of grid control
          * $("#Grid").ejGrid("getHeaderContent");        
          * &lt;/script&gt;
          */
        getHeaderContent: function () {
            return this._gridHeaderContent;
        },

        setGridHeaderContent: function (value) {
            this._gridHeaderContent = value;
        },
        /**
		 * Get the header table element of grid
		 * @return gridHeaderTable
		 * @example 
		 * &lt;script&gt;
		 * // Create grid object.
		 * var gridObj = $("#Grid").data("ejGrid");
		 * gridObj.getHeaderTable(); 
         * // Gets header table of grid control
		 * &lt;/script&gt;
		 * @example 
		 * &lt;script&gt;
		 * // Gets header table of grid control
		 * $("#Grid").ejGrid("getHeaderTable");        
		 * &lt;/script&gt;
         */
        getHeaderTable: function () {
            return this._gridHeaderTable;
        },

        setGridHeaderTable: function (value) {
            this._gridHeaderTable = value;
        },
        /**
         * Get the rows(tr element)of grid  which is displayed in the current page.
         * @return gridRows
         * @example 
         * &lt;script&gt;
         * // Create grid object.
         * var gridObj = $("#Grid").data("ejGrid");
         * // Gets grid rows of grid control
         * gridObj.getRows(); 
         * &lt;/script&gt;
         * @example 
         * &lt;script&gt;
         * // Gets grid rows of grid control
         * $("#Grid").ejGrid("getRows");        
         * &lt;/script&gt;
         */
        getRows: function () {
            return this._gridRows;
        },
        /**
		 * Get the rows(tr element) from the given from and to row index in grid
		 * @return gridRows
		 * @param {number} from Pass the from index from which the rows to be returned
		 * @param {number} to Pass the to index to which the rows to be returned
		 * @example 
		 * &lt;script&gt;
		 * // Create grid object.
		 * var gridObj = $("#Grid").data("ejGrid");
         * // Gets the rows from the specified row index 
		 * gridObj.getRowByIndex(3, 6);  
		 * &lt;/script&gt;
		 * @example 
		 * &lt;script&gt;
		 * // Gets the rows from the specified row index
		 * $("#Grid").ejGrid("getRowByIndex", 3, 6);   
		 * &lt;/script&gt;
         */
        getRowByIndex: function (from, to) {
            try {
                var gridRows = this.getRows(), $gridRows = $(gridRows), $row = $();
                if (ej.isNullOrUndefined(to)) {
                    if (this.model.scrollSettings.frozenColumns > 0) {
                        $row.push(gridRows[0][from]);
                        $row.push(gridRows[1][from]);
                        return $row;
                    }
                    return $(gridRows[from]);
                } else {
                    if (this.model.scrollSettings.frozenColumns > 0) {
                        $row.push($gridRows.eq(0).slice(from, to));
                        $row.push($gridRows.eq(0).slice(from, to));
                        return $row;
                    }
                    return $($gridRows.slice(from, to));
                }
            } catch (e) {
                return $();
            }
        },
        /**
         * Get the column index of the given field in grid.
         * @return Column Index
         * @param {string} fieldName Pass the field name of the column to get the corresponding column index
         * @example 
         * &lt;div id="Grid"&gt;&lt;/div&gt; 
         * &lt;script&gt;
         * // Create grid object.
         * var gridObj = $("#Grid").data("ejGrid");
         * // Gets the column index based on the given field name
         * gridObj.getColumnIndexByField("OrderID"); 
         * &lt;/script&gt;
         * @example 
         * &lt;div id="Grid"&gt;&lt;/div&gt; 
         * &lt;script&gt;
         * // Gets the column index based on the given field name
         * $("#Grid").ejGrid("getColumnIndexByField", "OrderID");        
         * &lt;/script&gt;
         */
        getColumnIndexByField: function (field) {
            for (var column = 0; column < this.model.columns.length; column++) {
                if (this.model.columns[column]["field"] == field)
                    break;
            }
            return column;
        },
        /**
		 * Get the row index based on the given tr element in grid.
		 * @return row index
		 * @param {object} $tr Pass the tr element in grid content to get its row index
		 * @example 
		 * &lt;script&gt;
		 * // Create grid object.
		 * var gridObj = $("#Grid").data("ejGrid");
         * // Gets the row index based on the given row
		 * gridObj.getIndexByRow($(".gridcontent tr").first()); 
		 * &lt;/script&gt;
		 * @example 
		 * &lt;script&gt;
		 * // Gets the row index based on the given row
		 * $("#Grid").ejGrid("getIndexByRow", $(".gridcontent tr").first());        
		 * &lt;/script&gt;
         */
        getIndexByRow: function ($tr) {
            var gridRows = this.getRows(), $gridRows = $(gridRows), rowIndex;
            if (this.model.scrollSettings.frozenColumns > 0) {
                rowIndex = $(gridRows[0]).index($tr);
                if (rowIndex == -1)
                    rowIndex = $(gridRows[1]).index($tr);
                return rowIndex;
            } else
                return $gridRows.index($tr);
        },
        /**
         * Get the names of all the visible column collections in grid
         * @return visibleColumns
         * @example 
         * &lt;script&gt;
         * // Create grid object.
         * var gridObj = $("#Grid").data("ejGrid");
         * // Gets the names of all the visible column collections
         * gridObj.getVisibleColumnNames(); 
         * &lt;/script&gt;
         * @example 
         * &lt;script&gt;
         * // Gets the names of all the visible column collections
         * $("#Grid").ejGrid("getVisibleColumnNames");        
         * &lt;/script&gt;
         */
        getVisibleColumnNames: function (headerText) {
            return this._visibleColumns;
        },
        /**
        * Get the names of all the hidden column collections in grid.
        * @return hiddenColumns
        * @example 
        * &lt;script&gt;
        * // Create grid object.
        * var gridObj = $("#Grid").data("ejGrid");
        * // Gets names of all the hidden column collections
        * gridObj.getHiddenColumnNames(); 
        * &lt;/script&gt;
        * @example 
        * &lt;script&gt;
        * // Gets names of all the hidden column collections
        * $("#Grid").ejGrid("getHiddenColumnNames");        
        * &lt;/script&gt;
        */
        getHiddenColumnNames: function (headerText) {
            return this._hiddenColumns;
        },
        /**
         * Get the column details based on the given field in grid
         * @return Column Object
         * @param {string} fieldName Pass the field name of the column to get the corresponding column object
         * @example 
         * &lt;script&gt;
         * // Create grid object.
         * var gridObj = $("#Grid").data("ejGrid");
         * // Gets the column details based on the given field name
         * gridObj.getColumnByField("OrderID");
         * &lt;/script&gt;
         * @example 
         * &lt;script&gt;
         * // Gets the column details based on the given field name
         * $("#Grid").ejGrid("getColumnByField", "OrderID");        
         * &lt;/script&gt;
         */
        getColumnByField: function (field) {
            for (var column = 0; column < this.model.columns.length; column++) {
                if (this.model.columns[column]["field"] == field)
                    break;
            }
            return column == this.model.columns.length ? null : this.model.columns[column];
        },
        /**
         * Get the column details based on the given header text in grid.
         * @return Column Object
         * @param {string} headerText Pass the header text of the column to get the corresponding column object
         * @example 
         * &lt;script&gt;
         * // Create grid object.
         * var gridObj = $("#Grid").data("ejGrid");
         * // Gets the column details based on the given headerText
         * gridObj.getColumnByHeaderText("Order ID"); 
         * &lt;/script&gt;
         * @example 
         * &lt;script&gt;
         * // Gets the column details based on the given headerText
         * $("#Grid").ejGrid("getColumnByHeaderText", "Order ID");        
         * &lt;/script&gt;
         */
        getColumnByHeaderText: function (headerText) {
            for (var column = 0; column < this.model.columns.length; column++) {
                if (this.model.columns[column]["headerText"] == headerText)
                    break;
            }
            return column == this.model.columns.length ? null : this.model.columns[column];
        },
        /**
         * Get the current page data source of grid..
         * @return currentJsonData
         * @example 
         * &lt;script&gt;
         * // Create grid object.
         * var gridObj = $("#Grid").data("ejGrid");
         * // Gets current view data of grid control
         * gridObj.getCurrentViewData(); 
         * &lt;/script&gt;
         * @example 
         * &lt;script&gt;
         * // Gets current view data of grid control
         * $("#Grid").ejGrid("getCurrentViewData");        
         * &lt;/script&gt;
         */
        getCurrentViewData: function () {
            return this._currentJsonData;
        },
        /**
		 * Get the list of field names from column collection in grid.
		 * @return columnFieldNames
		 * @example 
		 * &lt;script&gt;
		 * // Create grid object.
		 * var gridObj = $("#Grid").data("ejGrid");
         * // Gets the column field names based on the given index
		 * gridObj.getColumnFieldNames(); 
		 * &lt;/script&gt;
		 * @example 
		 * &lt;script&gt;
		 * // Gets the column field names based on the given index
		 * $("#Grid").ejGrid("getColumnFieldNames");        
		 * &lt;/script&gt;
         */
        getColumnFieldNames: function () {
            var columnNames = [];
            for (var column = 0; column < this.model.columns.length; column++) {
                if (this.model.columns[column]["field"])
                    columnNames.push(this.model.columns[column]["field"]);
            }
            return columnNames;
        },
        /**
         * Get the browser details
         * @return browserDetails
         * @example 
         * &lt;div id="Grid"&gt;&lt;/div&gt; 
         * &lt;script&gt;
         * // Create grid object.
         * var gridObj = $("#Grid").data("ejGrid");
         * // Gets the browser details of the application being run
         * gridObj.getBrowserDetails(); 
         * &lt;/script&gt;
         * @example 
         * &lt;div id="Grid"&gt;&lt;/div&gt; 
         * &lt;script&gt;
         * // Gets the browser details of the application being run
         * $("#Grid").ejGrid("getBrowserDetails");        
         * &lt;/script&gt;
         */
        getBrowserDetails: function () {
            var b = navigator.userAgent.match(/(firefox|chrome|opera|msie|safari)\s?\/?(\d+(.\d+)*)/i);
            if (!!navigator.userAgent.match(/Trident\/7\./))
                return { browser: "msie", version: $.uaMatch(navigator.userAgent).version };
            return { browser: b[1].toLowerCase(), version: b[2] };
        },
        _initPrivateProperties: function () {
            this._click = 0;
            this._gridHeaderTable = null;
            this._gridWidth = this.element.width();
            this._id = this.element.attr("id");
            this._gridRows = null;
            this._gridContentTable = null;
            this._gridContent = null;
            this._gridHeaderContent = null;
            this._gridFooterContent = null;
            this._gridFooterTable = null;
            this._gridRecordsCount = this._dataSource() !== null ? (this.model.pageSettings.totalRecordsCount == null ? this._dataSource().length : this.model.pageSettings.totalRecordsCount) : 0;
            this._links = null;
            this._gridPager = null;
            this._cSortedColumn = null;
            this._cSortedDirection = null;
            this._$curSElementTarget = null;
            this._gridFilterBar = null;
            this._$curFieldName = null;
            this._$prevFieldName = null;
            this._$fDlgIsOpen = false;
            this._$colType = null;
            this._$colFormat = null;
            this._$prevColType = null;
            this._$prevSElementTarget = null;
            this._currentFilterColumn = null;
            this._filteredRecordsCount = null;
            this._filteredRecords = [];
            this.filterColumnCollection = [];
            this._previousFilterCount = null;
            this._excelFilter = null;
            this._isExcelFilter = this.model.filterSettings.filterType == "excel";
            this._primaryKeys = [];
            this._primaryKeyValues = [];
            this._modifiedRecords = [];
            this._addedRecords = [];
            this._tdsOffsetWidth = [];
            this._deletedRecords = [];
            this._disabledToolItems = $();
            this._validationRules = {};
            this._groupedColumns = [];
            this._currentJsonData = [];
            this._groupingColumnIndex = 0;
            this._dataManager = this._dataSource() instanceof ej.DataManager ? this._dataSource() : this._dataSource() != null ? ej.DataManager(this._dataSource()) : null;
            if (this._dataManager != null && this.model.allowScrolling && this.model.scrollSettings.allowVirtualScrolling && this.model.pageSettings.totalRecordsCount != null && this._dataManager.dataSource.json != null)
                this._dataManager.dataSource.json.splice(this.model.pageSettings.totalRecordsCount);
            this._disabledSortableColumns = [];
            this._disabledGroupableColumns = [];
            this._disabledFilterableColumns = [];
            this._hiddenColumns = [];
            this._visibleColumns = [];
            this._sortedColumns = [];
            this.multiSortRequest = false;
            this.multiSelectCtrlRequest = false;
            this._multiSelectShiftRequest = false;
            this._enableSelectMultiTouch = false;
            this._enableSortMultiTouch = false;
            this._initialRender = false;
            this._fieldColumnNames = {};
            this._headerColumnNames = {};
            this._selectedRowsIndexes = [];
            this._isReorder = false;
            this._searchString = "";
            this._searchCount = null;
            this._columnsWidthCollection = [];
            this._Indicator = null;
            this._resizer = null;
            this._bulkEditCellDetails = {
                cellValue: null,
                rowIndex: -1,
                columnIndex: -1,
                fieldName: null,
                _data: null,
                cellEditType: "",
                cancelSave: false,
                defaultData: null,
                insertedTrCollection: [],
                rowData: null
            };
            this._batchChanges = {
                added: [],
                deleted: [],
                changed: []
            };
            this._bulkEditTemplate = $();
            this._confirmDialog = null;
            this._confirmedValue = false;
            this._lastRow = false;
            this._isVirtualRecordsLoaded = false;
            this._scrollValue = 0;
            this._currentTopFrozenRow = this.model.scrollSettings.frozenRows;
            this._rowHeightCollection = [];
            this._scrollObject = null;
            this._customPop = null;
            this._selectedRowCellIndexes = [];
            this._rowIndexesColl = [];
            this._selectedColumnIndexes = [];
            this.commonQuery = $.extend(true, {}, this.model.query);
            if (ej.gridFeatures.group) {
                this._rowCol = this._captionSummary();
                this._isCaptionSummary = (this._rowCol != null && this._rowCol.length) > 0 ? true : false;
            }
            this.phoneMode = this.model.isResponsive && document.documentElement.clientWidth < 360 ? true : false;
            if (this.model.selectionSettings.selectionMode.length > 0)
			    this._initSelection();
        },
        _init: function () {
            this._trigger("load");
            if (ej.isNullOrUndefined(this.model.query) || !(this.model.query instanceof ej.Query))
                this.model.query = ej.Query();
            if (ej.gridFeatures.common)
                this._initScrolling();
            this._initPrivateProperties();
            if (this.model.enableResponsiveRow)
                this.element.addClass("e-responsive");
            if (ej.gridFeatures.edit) {
                if (this.model.editSettings.allowEditing || this.model.editSettings.allowAdding)
                    this._processEditing();
            }
            this._checkDataBinding();
        },
        _initColumns: function (object) {
            while (object.items != undefined)
                object = object.items[0];
            if (this.model.columns.length == 0 && object) {
                for (var field in object) {
                    if (object.hasOwnProperty(field) && (typeof (object[field]) != "object" || object[field] instanceof Date)) {
                        var value = object[field];
                        this.model.columns.push({
                            field: field,
                            type: value != null ? (value.getDay ? (value.getHours() > 0 || value.getMinutes() > 0 || value.getSeconds() > 0 || value.getMilliseconds() > 0 ? "datetime" : "date") : typeof (value)) : null
                        });
                    }
                }
                this.model.columns.length && this._renderAfterColumnInitialize();
            } else {
                for (var index = 0; index < this.model.columns.length; index++) {
                    if (ej.isNullOrUndefined(this.model.columns[index].type)) {
                        var $field;
                        if (this.model.columns[index].dataSource && this.model.columns[index].foreignKeyValue)
                            $field = this.model.columns[index].dataSource[0][this.model.columns[index].foreignKeyValue];
                        else
                            $field = !ej.isNullOrUndefined(this.model.columns[index].field) ? ej.getObject(this.model.columns[index].field, object) : null;
                        this.model.columns[index].type = $field != null ? ($field.getDay ? ($field.getHours() > 0 || $field.getMinutes() > 0 || $field.getSeconds() > 0 || $field.getMilliseconds() > 0 ? "datetime" : "date") : typeof ($field)) : null;
                    }
                    else if (this.model.columns[index]["type"] == "date" && this.model.columns[index].format == undefined)
                        if (ej.isNullOrUndefined(Globalize))
                            $.extend(this.model.columns[index], { format: "{0:" + Globalize.culture().calendars.standard.patterns.d + "}" });
                        else
                            $.extend(this.model.columns[index], { format: "{0:M/d/yyyy}" });
                    else if (this.model.columns[index]["type"] == "datetime" && this.model.columns[index].format == undefined)
                        if (ej.isNullOrUndefined(Globalize))
                            $.extend(this.model.columns[index], { format: "{0:" + Globalize.culture().calendars.standard.patterns.d + " " + Globalize.culture().calendars.standard.patterns.t + "}" });
                        else
                            $.extend(this.model.columns[index], { format: "{0:M/d/yyyy h:mm tt}" });
                }
            }
        },
		_initSelection: function(){	
            var mode = this.model.selectionSettings.selectionMode;
            for (i = 0; i < mode.length; i++) {			
                this["_allow" + mode[i] + "Selection"] = true;
            }
		},
        _checkDataBinding: function () {
            if (this.model.enablePersistence)
                this.model.query = new ej.Query();
            this._initialRenderings();
            this._initialRender = true;
            this.model.enableRTL && this.element.addClass("e-rtl");
            if (this.model.cssClass != null)
                this.element.addClass(this.model.cssClass);
            if (this.model.showColumnChooser)
                this._renderColumnChooser();
            if (this.model.allowGrouping)
                this.element.append(this._renderGroupDropArea());
            if (this.model.toolbarSettings.showToolbar)
                this.element.append(this._renderToolBar());
            var columns = this.model.columns;
            if (columns && columns.length) {
                var expands = [];
                if (typeof columns[0] === "string")
                    for (var i = 0; i < columns.length; i++)
                        columns[i] = { field: columns[i] };
                for (var i = 0; i < columns.length; i++) {
                    if (!columns[i].field || columns[i].field.indexOf('.') === -1) continue;
                    this._getExpands(columns[i].field, expands);
                }
                this.model.query.expand(expands);
                this._renderAfterColumnInitialize();
            }
            if (this.model.allowPaging)
                this.element.append(this._renderGridPager());
			if (this.model.contextMenuSettings.enableContextMenu)
                 this.element.append(this._renderContext());
            if ($.isFunction($.fn.ejWaitingPopup)) {
                this.element.ejWaitingPopup({ showOnInit: false });
                $("#" + this._id + "_WaitingPopup").addClass("e-gridwaitingpopup");
            }
            if (this._dataSource() instanceof ej.DataManager) {
                this.element.ejWaitingPopup("show");
                if (this._dataSource().ready != undefined) {
                    var proxy = this;
                    this._dataSource().ready.done(function (args) {
                        proxy._initDataSource();
                        proxy.model.dataSource = ej.DataManager(args.result);
                    });
                } else {
                    this.element.ejWaitingPopup("show");
                    this._initDataSource();
                }
            } else {
                this._ensureDataSource();
                this._trigger("actionBegin");
                this._initGridRender();
            }
        },
		 _renderColumnChooser: function () {
            var $columnButton = ej.buildTag("button .e-ccButton", 'Columns', { 'float': 'right' });
            this.element.append($columnButton);
            $columnButton.ejButton({
                prefixIcon: "e-down-arrow",
                imagePosition:"imageright",
				contentType:"textandimage",
				type: 'button',
                width:90
            });
			var buttHeight=$columnButton.outerHeight();
			$columnButton.css('margin-top',0-(buttHeight));
            var $mainDiv = ej.buildTag("div");
            var $outerDiv = ej.buildTag("div .e-grid e-columnChooser",'',{},{id:this._id+"ccDiv"});
            var $searchBox = ej.buildTag("div.e-searchbox e-fields").append(ej.buildTag("input#" + this._id + "_SearchBox.e-ejinputtext e-filtertext", {}, {}, { "type": "text" }))
            var $sapnDiv = ej.buildTag('span .e-searchfind e-icon')
            $searchBox.append($sapnDiv);
            var $listOuterDiv=ej.buildTag('div','', {'height':'228px'},{id:this._id+"liScrollerDiv"})
            var $listBox = ej.buildTag("div", '', { 'margin-left': '0px','width':'250px' });
            for (var index = 0; index < this.model.columns.length; index++) {
                var column = this.model.columns[index];
                var $innerDiv = ej.buildTag('div', '', {}, { 'class': 'e-columnChooserListDiv' });
                var styleAttr = {}
                var $input = ej.buildTag('input', '', styleAttr, { 'id': column.field, 'value': column.field, 'type': 'checkbox' });
                var label = ej.buildTag('label', column.field, { 'font-size': '13px' }, { 'for': column.field });
                $innerDiv.append($input);
                $innerDiv.append(label);
                $listBox.append($innerDiv);
                $input.ejCheckBox({
                    checked: !ej.isNullOrUndefined(column.visible) ? column.visible : true
                });
            }
            $listOuterDiv.append($listBox)
            $outerDiv.append($searchBox);
            $outerDiv.append($listOuterDiv);
           
            var $splitterDiv = ej.buildTag('div .e-columnChooserSplitter', '', { 'margin-top': '-4px','border-bottom':'0px' });
            $outerDiv.append($splitterDiv);
            $buttonDiv = ej.buildTag('div', '', { 'float': 'right', 'margin-top': '10px','margin-right':'-10px' });
            var $addButton = ej.buildTag("button", 'Done');
            var $cancelButton = ej.buildTag("button", 'Cancel', { 'margin-right': '20px','margin-left':'6px' });
            $buttonDiv.append($addButton);
            $buttonDiv.append($cancelButton);
            $outerDiv.append($buttonDiv);
            $addButton.ejButton({
                click: $.proxy(this._addButtonCC, this),
                showRoundedCorner:true,
                width:66
            });
            $cancelButton.ejButton({
                click: $.proxy(this._cancelButtonHandler, this),
                showRoundedCorner:true,
                width:66
            });
              $outerDiv.insertBefore(this.element)
              $outerDiv.ejDialog({ width: 'auto', beforeClose: $.proxy(this._columnChooserBeforeClose, this), showOnInit: false, allowKeyboardNavigation: false, enableResize: false, "enableRTL": this.model.enableRTL, "cssClass": this.model.cssClass, showHeader: false, width: 260 });
		 },
		 _columnChooserBeforeClose: function () {
		     $(".e-columnChoosertail").remove();
		     $(".e-columnChoosertailAlt").remove();
		     for (index = 0; index < this.model.columns.length; index++) {
		         var chkState = !ej.isNullOrUndefined(this.model.columns[index].visible) ? this.model.columns[index].visible : true
		         var chkBox = $("#" + this.model.columns[index].field).ejCheckBox('model.checked', chkState);
		     }
			  $("#" + this._id + "_SearchBox").val('');
			 var args={};
			 args.target={};args.target.value='';
			 this._columnChooserSearch(args);
		 },
		 _columnChooserSearch: function (e) {
            if (e.type == 'click') {
                e.target.value = '';
                $("#"+this._id+"_SearchBox").val('');
            }
            var val = e.target.value;
			var span=$("#"+this._id+"_SearchBox").next('span')
            if (val != '') {
                span.removeClass("e-searchfind");
                span.addClass("e-cancel");
            }
            else {
                span.removeClass("e-cancel");
                span.addClass("e-searchfind");
            }
            $(".e-cancel").bind('click', $.proxy(this._columnChooserSearch, this));
            var currentCheckedItemsData = this.model.columns;
            var columnCollection = [];
            if (val != '') 
                currentCheckedItemsData = ej.DataManager(this.model.columns).executeLocal(ej.Query().where("field", ej.FilterOperators.startsWith, val,true));
            currentCheckedItemsData.forEach(function (obj) {
                columnCollection.push(obj.field);
            });
            if (!ej.isNullOrUndefined($("#nomatches")[0]))
                $("#nomatches").remove();
            var div = $("#"+this._id+"ccDiv").find("#"+this._id+"liScrollerDiv")

            for (var index = 0; index < this.model.columns.length; index++) {
                var column = this.model.columns[index];
                if (columnCollection.indexOf(column.field) == -1)
                    $("#ej" + column.field).parent('div').css('display', 'none');
                else
                    $("#ej" + column.field).parent('div').css('display', 'block');

            }

            if (columnCollection.length == 0) {
                var $labeldiv = ej.buildTag('div#nomatches', '');
                var $label = ej.buildTag('span', 'No Matches Found');
                $labeldiv.append($label);
                $(div).append($labeldiv);
            }
            $("#"+this._id+"liScrollerDiv").ejScroller('refresh')
        },
		_addButtonCC: function () {
            this._visibleColumns = [];
            this._hiddenColumns = [];
            for (index = 0; index < this.model.columns.length; index++) {
                var chkBox = $("#" + this.model.columns[index].field).data("ejCheckBox");
                var column = this.getColumnByField(chkBox.model.value);
                if (chkBox.model.checked)
                    this._visibleColumns.push(column.headerText);
                else
                    this._hiddenColumns.push(column.headerText);
            }
            this.showColumns(this._visibleColumns);
            this.hideColumns(this._hiddenColumns);
            $("#"+this._id+"ccDiv").ejDialog('close');
            $(".e-columnChoosertail").remove();
        },
		_cancelButtonHandler: function () {
		    $("#" + this._id + "ccDiv").ejDialog('close');
		    $(".e-columnChoosertailAlt").remove();
            $(".e-columnChoosertail").remove();
		},
        _initDataSource: function () {
            this._ensureDataSource();
            this._trigger("actionBegin");
            var queryPromise = this._dataSource().executeQuery(this.model.query);
            if (this._dataManager.dataSource.table != null)
                this._dataManager.dataSource.table.css("display", "none");
            queryPromise.done(ej.proxy(function (e) {
                this.element.ejWaitingPopup("hide");
                this.model.currentViewData = e.result;
                if (this.model.pageSettings.totalRecordsCount != null && this.model.filterSettings.filteredColumns.length == 0)
                    this._gridRecordsCount = this.model.pageSettings.totalRecordsCount;
                else if (e.count == 0 && e.result.length)
                    this._gridRecordsCount = e.result.length;
                else
                    this._gridRecordsCount = e.count;
                if (this.getPager() != null)
                    this.model.pageSettings.totalRecordsCount = this._gridRecordsCount;
                this._initGridRender();
            }, this));
        },
        _initialRenderings: function () {
            if (this.model.groupSettings.groupedColumns.length) {
                var sortedColumns = new Array();
                for (var i = 0; i < this.model.sortSettings.sortedColumns.length; i++) {
                    if (ej.isNullOrUndefined(this.model.sortSettings.sortedColumns[i].direction))
                        this.model.sortSettings.sortedColumns[i].direction = ej.sortOrder.Ascending;
                    sortedColumns.push(this.model.sortSettings.sortedColumns[i].field);
                }
                for (var i = 0; i < this.model.groupSettings.groupedColumns.length; i++) {
                    if ($.inArray(this.model.groupSettings.groupedColumns[i], sortedColumns) == -1)
                        this.model.sortSettings.sortedColumns.push({ field: this.model.groupSettings.groupedColumns[i], direction: ej.sortOrder.Ascending });
                }
            }
        },
        _getExpands: function (field, arr) {
            var splits = field.split('.'), tmp = "";
            splits.splice(splits.length - 1, 1);
            for (var i = 0; i < splits.length; i++, tmp = "") {
                for (var j = 0; j < i; j++)
                    tmp += splits[j] + "/";
                tmp = tmp + splits[i];
                if (arr.indexOf(tmp) === -1)
                    arr.push(tmp);
            }
        },
        _renderAfterColumnInitialize: function () {
            this.element.append(this._renderGridHeader());
            if (this.model.allowFiltering && this.model.filterSettings.filterType == "filterbar")
                this._renderFiltering();
            if (this.model.allowPaging)
                this.element.append(this.element.find(".e-pager").first());
        },
        _ensureDataSource: function (args) {
            if (this._dataSource() == null && !(this._dataSource() instanceof ej.DataManager))
                return;
            this.model.query.requiresCount();
            var queryManagar = this.model.query;
            if (!(this._dataSource() instanceof ej.DataManager))
                this.model.currentViewData = this._dataSource();
            if ((!(this._dataSource() instanceof ej.DataManager) || this._dataManager.dataSource.url == undefined) && (this.model.editSettings.allowEditing || this.model.editSettings.allowAdding) && (!ej.isNullOrUndefined(this._cModifiedData) || !ej.isNullOrUndefined(this._cAddedRecord))) {
                if (ej.isNullOrUndefined(this._cAddedRecord)) {
                    for (var index = 0; index < this._primaryKeys.length; index++)
                        queryManagar = queryManagar.where(this._primaryKeys[index], ej.FilterOperators.equal, this._primaryKeyValues[index]);
                    var currentData = this._dataManager.executeLocal(queryManagar);
                    if (!(this._dataSource() instanceof ej.DataManager))
                        $.extend(this._dataSource()[$.inArray(currentData.result[0], this._dataSource())], this._cModifiedData);
                    else
                        $.extend(this._dataSource().dataSource.json[$.inArray(currentData.result[0], this._dataSource().dataSource.json)], this._cModifiedData);
                    this._cModifiedData = null;
                } else {
                    (this._dataSource() instanceof ej.DataManager) ? this._dataSource().dataSource.json.unshift(this._cAddedRecord) : this._dataSource().unshift(this._cAddedRecord);
                    this._cAddedRecord = null;
                }
                queryManagar.queries = [];
                this.model.isEdit = false;
            }

            if (this.model.editSettings.allowDeleting && !ej.isNullOrUndefined(this._cDeleteData) && (!(this._dataSource() instanceof ej.DataManager) || this._dataManager.dataSource.url == undefined)) {
                if (!(this._dataSource() instanceof ej.DataManager)) {
                    var index = $.inArray(this._cDeleteData[0], this._dataSource());
                    this._dataSource().splice(index, 1);
                }
                else {
                    var index = $.inArray(this._cDeleteData[0], this._dataSource().dataSource.json);
                    this._dataSource().dataSource.json.splice(index, 1);
                }
            }

            if (this.model.sortSettings.sortedColumns.length) {
                for (var i = this.model.sortSettings.sortedColumns.length - 1; i >= 0; i--)
                    queryManagar.sortBy(this.model.sortSettings.sortedColumns[i].field, this.model.sortSettings.sortedColumns[i].direction);

            }

            if (this.model.allowSearching && this._searchString.length) {
                queryManagar.search(this._searchString, this.getColumnFieldNames(), ej.FilterOperators.contains, true);
                if (args.requestType == "searching")
                    this._currentPage(1);
            }

            if (this.model.allowFiltering && this.model.filterSettings.filteredColumns.length) {
                var predicate, firstFilterCondition = this.model.filterSettings.filteredColumns[0];
                if (this._isExcelFilter) {
                    var predicates = this._excelFilter._predicates[0];
                    for (var prop in predicates) {
                        var obj = predicates[prop], isTake = obj["from"]!=undefined;
                        if (isTake) 
                           queryManagar.skip(obj["from"] == "top" ? 0 : this._gridRecordsCount - obj["take"]).take(obj["take"]);                                     
                        else
                            predicate = predicate != undefined ? predicate["and"](obj) : obj;
                    }
                }
                else {
                    if (!firstFilterCondition.isComplex)
                        predicate = ej.Predicate(firstFilterCondition.field, firstFilterCondition.operator, firstFilterCondition.value, !firstFilterCondition.matchcase);
                    else
                        predicate = firstFilterCondition;
                    for (var i = 1; i < this.model.filterSettings.filteredColumns.length; i++) {
                        if (!this.model.filterSettings.filteredColumns[i].isComplex)
                            predicate = predicate[this.model.filterSettings.filteredColumns[i].predicate](this.model.filterSettings.filteredColumns[i].field, this.model.filterSettings.filteredColumns[i].operator, this.model.filterSettings.filteredColumns[i].value, !this.model.filterSettings.filteredColumns[i].matchcase);
                        else
                            predicate = predicate["and"](this.model.filterSettings.filteredColumns[i]);
                    }
                }
                predicate && queryManagar.where(predicate);
                if (!(this._dataSource() instanceof ej.DataManager)) {
                    var fresults = this._dataManager.executeLocal(queryManagar);
                    this._filteredRecordsCount = isTake ? fresults.result.length : fresults.count;
                    var lastPage = (this._filteredRecordsCount % this.model.pageSettings.pageSize == 0) ? (this._filteredRecordsCount / this.model.pageSettings.pageSize) : (parseInt(this._filteredRecordsCount / this.model.pageSettings.pageSize, 10) + 1);
                    if (this._currentPage() > lastPage)
                        this._currentPage(lastPage);
                } else if (!ej.isNullOrUndefined(args) && args.requestType == ej.Grid.Actions.Filtering)
                    this._currentPage(1);
                if (this.model.showSummary && this.model.summaryRows.length > 0) {
                    var fQueryMgr = new ej.Query();
                    this._filteredRecords = this._dataManager.executeLocal(fQueryMgr.where(predicate));
                }
            }


            if (this.model.allowPaging || (this.model.scrollSettings.allowVirtualScrolling && this.model.allowScrolling)) {
                if (this._currentPage() == 0) {
                    if (this._prevPageNo == 0 || this._prevPageNo == null)
                        this._currentPage(1);
                    else
                        this._currentPage(this._prevPageNo);
                }
                queryManagar.page(this._currentPage(), this.model.pageSettings.pageSize);
            }

            if (this.model.allowGrouping) {
                var cloned = queryManagar.clone();
                if (this.model.allowPaging && this.model.groupSettings.groupedColumns.length) {
                    cloned.queries = cloned.queries.slice(0, cloned.queries.length - 1);
                }
                for (var i = 0; i < this.model.groupSettings.groupedColumns.length; i++) {
                    queryManagar.group(this.model.groupSettings.groupedColumns[i]);
                    cloned.group(this.model.groupSettings.groupedColumns[i]);
                }
                if (this.model.groupSettings.groupedColumns.length)
                    this._setAggreatedCollection(cloned);
            }
            if (args != undefined && args.requestType == "add" && (!(this._dataSource() instanceof ej.DataManager) || this._dataManager.dataSource.url == undefined) && this.model.groupSettings.groupedColumns.length == 0 && this.model.scrollSettings.frozenColumns == 0 && this.model.scrollSettings.frozenRows == 0)
                !(this._dataSource() instanceof ej.DataManager) ? this._dataSource().unshift(args.data) : this._dataSource().dataSource.json.unshift(args.data);
            if ((!(this._dataSource() instanceof ej.DataManager) || this._dataManager.dataSource.url == undefined)) {
                var result = this._dataManager.executeLocal(queryManagar);
                this.model.currentViewData = result.result;
                this._gridRecordsCount = result.count;
                this._searchCount = this._searchString.length ? result.count : null;
            }

        },
        _initGridRender: function () {
            this.addInitTemplate();
            if (this.model.scrollSettings.frozenColumns > 0)
                this.addFrozenTemplate();
            this.model.allowGrouping && this.addGroupingTemplate();
            if (this.model.allowResizing || this.model.allowResizeToFit)
                this._resizer = new ej.Grid.gridResize(this);
			if (this.model.allowTextWrap == true)
                this.element.addClass("e-wrap");
            this.render();
            if (this.model.allowGrouping && ej.gridFeatures.dragAndDrop)
                this._headerCellgDragDrop();
            if (this.model.allowReordering && ej.gridFeatures.dragAndDrop) {
                this._headerCellreorderDragDrop();
                this._initIndicators();
            }
            this._wireEvents();           
            this._initialRender = false;
            if (this.model.width && !this.model.allowScrolling)
                this.element.width(this.model.width);
            if (this.model.editSettings.allowEditing || this.model.editSettings.allowAdding)
                this._processEditing();
            if (this.model.allowFiltering && (this.model.filterSettings.filterType == "menu" || this.model.filterSettings.filterType == "excel")) {
                this._isExcelFilter && this._renderExcelFilter();
                this._renderFilterDialogs();
            }
        },
        _getMetaColGroup: function () {
            var $colgroup = ej.buildTag("colgroup");
            for (var i = 0; i < this.model.columns.length; i++) {
                var $col = $(document.createElement("col"));
                this.model.columns[i]["visible"] === false && $col.css("display", "none");
                !ej.isNullOrUndefined(this.model.columns[i]["cssClass"]) && $col.addClass(this.model.columns[i]["cssClass"]);
                if (!this.model.groupSettings.showGroupedColumn && $.inArray(this.model.columns[i]["field"], this.model.groupSettings.groupedColumns) != -1)
                    $col.css("display", "none");
                $colgroup.append($col);
            }
            return $colgroup;
        },
        _alternateRow: function () {
            return this.getIndex() % 2 == 0 ? "" : "e-alt_row";
        },
        addInitTemplate: function () {
            var headerCellDiv = this.element.find(".e-headercelldiv"), templates = {};
            var tbody = document.createElement('tbody'), $tbody = $(tbody);
            if (this.model.rowTemplate == null) {
                var tr = document.createElement('tr'),
                    $tr = $(tr),
                    columns = this.model.columns,
                    i;
                if (this._gridRecordsCount)
                    this._initColumns(this.model.currentViewData[0] != undefined ? this.model.currentViewData[0] : this.model.currentViewData.value);
                var helpers = { _gridFormatting: this.formatting };
                $.views.helpers(helpers);

                var viewHelper = {};
                viewHelper["_foreignKey"] = this._foreignKeyBinding; // move to common methods
                $.views.helpers(viewHelper);

                if (this.model.detailsTemplate || this.model.childGrid || this.model.detailsDataBound) {
                    var $tdDetailCell = ej.buildTag("td.e-detailrowcollapse", "<div class='e-icon e-gnextforward'></div>");
                    $tr.append($tdDetailCell);
                }
                for (i = 0; i < this.model.columns.length; i++) {
                    var $tdCell = ej.buildTag("td.e-rowcell");
                    if (this.model.isResponsive)
                        $tdCell.attr("data-cell", this.model.columns[i]["headerText"]);
                    columns[i]["visible"] == false && $tdCell.addClass("e-hide");
                    !this.model.groupSettings.showGroupedColumn && $tdCell.addClass("{{for ~groupedColumns}}" +
                        " {{if #data == '" + this.model.columns[i]["field"] + "'}}e-hide{{/if}}" +
                        "{{/for}}");
                    if (columns[i]["template"]) {
                        var viewHelper = {};
                        viewHelper["_" + this._id + "ColumnTemplating"] = ej.proxy(this._gridTemplate, null, this);
                        $.views.helpers(viewHelper);
                        $("#" + this._id + columns[i].headerText + i + "_Template").remove();
                        var scriptElement = this._createTemplateElement(columns[i]);
                        this.model.columns[i]["allowGrouping"] = this.model.columns[i]["allowFiltering"] = this.model.columns[i]["allowSorting"] = false;
                        $tdCell.addClass("e-templatecell").html("{{:~_" + this._id + "ColumnTemplating('" + scriptElement.id + "')}}");
                    } else {
                        var splits = (columns[i].field || "").split("."), sLen = splits.length - 1, braces = "";
                        while (sLen) {
                            braces += "(";
                            sLen--;
                        }
						var columnType=columns[i].type || columns[i].editType
                        switch (columnType) {
                            case "boolean":
							case "booleanedit":
                                $tdCell.addClass("e-boolrowcell").html('{{if ' + columns[i].field + '}} <input type ="checkbox" disabled="disabled" checked="checked"></input>{{else}} <input type ="checkbox" disabled="disabled"  ></input> {{/if}}');
                                break;
                            default:
                                if (columns[i].disableHtmlEncode)
                                    $tdCell.html("{{html:" + "#data['" + splits.join("'] || {})['") + "']}}");
                                else
                                    $tdCell.html("{{:" + braces + "#data['" + splits.join("'] || {})['") + "']}}");
                        }
                        if (columns[i]["format"] != undefined && (!columns[i]["foreignKeyValue"] && !columns[i]["dataSource"]))
                            $tdCell.html("{{:~_gridFormatting('" + columns[i]["format"] + "',#data['" + columns[i]["field"] + "'])}}");
                        if (columns[i]["foreignKeyValue"] && columns[i]["dataSource"]) {
                            var foreignKeyColumn = columns[i]["field"] != columns[i]["foreignKeyField"] ? columns[i]["field"] : columns[i]["foreignKeyField"];
                            $tdCell.html("{{:~_foreignKey('" + JSON.stringify(columns[i]) + "',#data['" + foreignKeyColumn + "'],'" + this._id + "Object" + "')}}");
                        }
                        if (columns[i]["isUnbound"]) {
                            var viewHelper = {};
                            viewHelper["_" + this._id + "UnboundTemplate"] = this._unboundTemplateRendering;
                            $.views.helpers(viewHelper);
                            this.model.columns[i]["allowGrouping"] = this.model.columns[i]["allowFiltering"] = this.model.columns[i]["allowSorting"] = false;
                            $("#" + this._id + columns[i].headerText + "_UnboundTemplate").remove();
                            divElement = this._createUnboundElement(columns[i]);
                            $tdCell.addClass("e-unboundcell").addClass("e-" + columns[i]["headerText"].replace(/\s+/g, "")).html("{{:~_" + this._id + "UnboundTemplate('" + divElement.id + "')}}");
                            this.model.scrollSettings.frozenColumns > 0 && $tdCell.addClass("e-frozenunbound");
                            this._isUnboundColumn = true;
                        }

                    }
                    if (!this.phoneMode && columns[i]["textAlign"] != undefined) {
                        $tdCell.css("text-align", columns[i]["textAlign"]);
                        $(headerCellDiv[i]).css("text-align", columns[i]["textAlign"]);
                    }
                    if (!ej.isNullOrUndefined(columns[i]["cssClass"])) {
                        $tdCell.addClass(columns[i]["cssClass"]);
                    }
                    if (!ej.isNullOrUndefined(columns[i]["priority"]))
                        $tdCell.addClass("e-table-priority-" + columns[i]["priority"]);
                    if (!ej.isNullOrUndefined(columns[i]["customAttributes"]))
                        $tdCell.attr(columns[i]["customAttributes"]);
                    $tdCell.attr("role", "gridcell");
                    $tr.append($tdCell);
                    if (this.model.enableAltRow) {
                        helpers["_" + this._id + "AlternateRow"] = this._alternateRow;
                        $.views.helpers(helpers);
                        $tr.addClass("{{:~_" + this._id + "AlternateRow()}}");
                    }
                    $tr.attr("role", "row");
                    if (this.model.scrollSettings.frozenColumns > 0 && this.model.scrollSettings.frozenColumns == i + 1) {
                        tbody.appendChild(tr);
                        templates[this._id + "_JSONFrozenTemplate"] = $tbody.html();
                        $tr.empty();
                        $tbody.empty();
                    }
                }
                tbody.appendChild(tr);
            }
            templates[this._id + "_JSONTemplate"] = this.model.rowTemplate != null ? $(this.model.rowTemplate).html() : $tbody.html();
            $.templates(templates);
        },
        /**
         * Creates a grid control
         * @return jQuery
         * @example 
         * &lt;div id="Grid"&gt;&lt;/div&gt; 
         * &lt;script&gt;
         * // Create grid object.
         * var gridObj = $("#Grid").data("ejGrid");
         * // It renders the grid.
         * gridObj.render(); 
         * &lt;/script&gt;
         * @example 
         * &lt;div id="Grid"&gt;&lt;/div&dgt; 
         * &lt;script&gt;
         * // It renders the grid.
         * $("#Grid").ejGrid("render");        
         * &lt;/script&gt;
         */
        // Grid rendering parts
        render: function () {
            this._renderGridContent().insertAfter(this.element.children(".e-gridheader"));
            this.model.allowResizeToFit && this.setWidthToColumns();
            if (this.model.allowGrouping && ej.gridFeatures.dragAndDrop)
                this._groupHeaderCelldrag();
            this.model.showSummary = this.model.summaryRows.length > 0 || this.model.showSummary;
            if (this.model.showSummary) {
                this._renderGridFooter().insertAfter(this.getContent());
                this._hideCaptionSummaryColumn();
            }
            this._initialEndRendering();

        },
        _createStackedRow: function (stackedHeaderRow, frozenHeader) {
            var $tr = ej.buildTag('tr.e-columnheader e-stackedHeaderRow');
            var sHeader = [];
            for (var c = 0; c < this.model.columns.length; c++) {
                var column = this.model.columns[c];
                if (column.visible != false) {
                    var headerText = ''
                    var sColumn = stackedHeaderRow.stackedHeaderColumn;
                    for (var col = 0; col < sColumn.length; col++) {
                        if (sColumn[col].column.indexOf(column.field) != -1)
                            headerText = sColumn[col].stackedHeaderText;
                    }
                    sHeader.push(headerText);
                }
            }
            var colsPanList = []
            for (var i = 0; i < sHeader.length; i++) {
                var colSpan = 1;
                for (var j = i + 1; j < sHeader.length; j++) {
                    if (sHeader[i] == sHeader[j]) {
                        colSpan++;
                    }
                    else
                        break;
                }
                colsPanList.push({ sapnCount: colSpan, headerText: sHeader[i] })
                i += colSpan - 1;
            }
            var $tr = ej.buildTag('tr.e-columnheader e-stackedHeaderRow');
            var frzCol = this.model.scrollSettings.frozenColumns;
            if (this.model.allowScrolling && frzCol > 0) {
                var frozenColspanList = [];
                var forzenColumn = 0, index = 0, frzHideCol = 0;
                for (var i = 0; i < this.model.columns.length; i++) {
                    var col = this.model.columns[i];                    
                    if ( i < frzCol && col.visible == false)
                        frzHideCol++;
                }                
                forzenColumn = frzCol - frzHideCol;
                while (forzenColumn > 0) {
                    var spanC = colsPanList[index].sapnCount;
                    if (colsPanList[index].sapnCount < forzenColumn) {
                        frozenColspanList.push(colsPanList[index])
                        if (!frozenHeader)
                            colsPanList.splice(index, 1);
                        else
                            index++;
                    }
                    else if (colsPanList[index].sapnCount > forzenColumn) {
                        colsPanList[index].sapnCount = colsPanList[index].sapnCount - forzenColumn
                        if (frozenHeader)
                            frozenColspanList.push({ sapnCount: forzenColumn, headerText: colsPanList[index].headerText });
                    }
                    else {
                        frozenColspanList.push(colsPanList[index])
                        if (!frozenHeader)
                            colsPanList.splice(index, 1);
                    }
                    forzenColumn -= spanC;
                }
                if (frozenHeader)
                    colsPanList = frozenColspanList
            }
            for (var c = 0; c < colsPanList.length; c++) {
                var $th = ej.buildTag('th.e-headercell e-default', colsPanList[c].headerText, {}, { 'colspan': colsPanList[c].sapnCount });
                $tr.append($th);
            }
            return $tr;
        },
        _renderGridHeaderInternalDesign: function (columns, frozenHeader) {
            var $table = ej.buildTag('table.e-table', "", {}, { cellspacing: "0.25px", role: "grid" });
            var $thead = ej.buildTag('thead');
            var $tbody = ej.buildTag('tbody.e-hide');
            var $columnHeader = ej.buildTag('tr.e-columnheader');
            var $colGroup = $(document.createElement('colgroup'));
            var $rowBody = $(document.createElement('tr'));
            if (this.model.detailsTemplate || this.model.childGrid || this.model.detailsDataBound) {
                $columnHeader.append(ej.buildTag('th.e-headercell e-detailheadercell', '<div></div>'));
                $colGroup.append(this._getIndentCol());
            }
            if (this.model.showStackedHeader) {
                for (var index = 0; index < this.model.stackedHeaderRows.length; index++) {
                    var $tr = this._createStackedRow(this.model.stackedHeaderRows[index], frozenHeader);
                    $thead.append($tr);
                }
            }
            for (var columnCount = 0; columnCount < columns.length; columnCount++) {
                var $headerCell = ej.buildTag('th.e-headercell e-default', "", {}, { role: "columnheader" });
                var bodyCell = document.createElement('td');
                var $headerCellDiv = ej.buildTag('div.e-headercelldiv', columns[columnCount]["headerText"] === undefined ? columns[columnCount]["headerText"] = columns[columnCount]["field"] : columns[columnCount]["headerText"], {}, { "ej-mappingname": columns[columnCount]["field"] });
                if (columns[columnCount].disableHtmlEncode)
                    $headerCellDiv.text(columns[columnCount]["headerText"]);
                $headerCell.append($headerCellDiv);
                if (this.model.allowFiltering && (this.model.filterSettings.filterType == "menu" || this.model.filterSettings.filterType == "excel") &&
                (columns[columnCount]["allowFiltering"] == undefined ||
                    columns[columnCount]["allowFiltering"] === true))
                    $headerCell.append(ej.buildTag('div.e-filtericon e-icon e-filterset'));
                var col = document.createElement('col');
                if (columns[columnCount]["priority"])
                    $(bodyCell).addClass("e-table-priority-" + columns[columnCount]["priority"]);
                $rowBody.append(bodyCell);
                $columnHeader.append($headerCell);
                $colGroup.append(col);
                if (columns[columnCount]["visible"] === false)
                    $headerCell.addClass("e-hide") && $(col).css("display", "none") && this._hiddenColumns.push(columns[columnCount].headerText);
                else {
                    this._visibleColumns.push(columns[columnCount].headerText);
                    columns[columnCount]["visible"] = true;
                }
				if (columns[columnCount]["textAlign"] != undefined)
                    $headerCellDiv.css("text-align", columns[columnCount]["textAlign"]);
                columns[columnCount]["allowSorting"] === false && this._disabledSortableColumns.push(columns[columnCount].field);
                columns[columnCount]["allowGrouping"] === false && this._disabledGroupableColumns.push(columns[columnCount].headerText);
                if (!ej.isNullOrUndefined(columns[columnCount]["cssClass"])) {
                    $headerCell.addClass(this.model.columns[columnCount]["cssClass"]);
                    $(col).addClass(this.model.columns[columnCount]["cssClass"]);
                }
                if (!ej.isNullOrUndefined(this.model.columns[columnCount]["headerTemplateID"])) {
                    $headerCellDiv.html($(this.model.columns[columnCount]["headerTemplateID"]).hide().html()).parent().addClass("e-headertemplate");
                    var index = $.inArray(this.model.columns[columnCount].headerText, this._disabledGroupableColumns);
                    index == -1 && this._disabledGroupableColumns.push(this.model.columns[columnCount].headerText);
                }
                if (this.model.groupSettings.showToggleButton && $.inArray(columns[columnCount].headerText, this._disabledGroupableColumns) == -1) {
                    if ($.inArray(columns[columnCount].field, this.model.groupSettings.groupedColumns) != -1)
                        $headerCellDiv.append(this._getToggleButton().addClass("e-toggleungroup"));
                    else
                        $headerCellDiv.append(this._getToggleButton().addClass("e-togglegroup"));
                }
				if (this.model.isResponsive)
                    $headerCell.attr("title", columns[columnCount].headerText);
                if (columns[columnCount]["priority"]) {
                    $headerCell.attr("data-priority", columns[columnCount]["priority"]).addClass("e-table-priority-" + columns[columnCount]["priority"]);
                    $(col).addClass("e-table-priority-" + columns[columnCount]["priority"]);
                }
                this._columnsWidthCollection.push(columns[columnCount]["width"]);
                if (columns[columnCount]["width"] == undefined && this.model.commonWidth !== undefined)
                    this._columnsWidthCollection[columnCount] = this.model.commonWidth;
                this._fieldColumnNames[columns[columnCount].headerText] = columns[columnCount].field;
                this._headerColumnNames[columns[columnCount].field] = columns[columnCount].headerText;
                this.model.allowFiltering && ("menu" == this.model.filterSettings.filterType || this._isExcelFilter) && $headerCell.addClass("e-headercellfilter");
            }
            $thead.append($columnHeader);
            $tbody.append($rowBody);
            $table.append($colGroup).append($thead).append($tbody);
            return $table;
        },
        _renderGridHeader: function () {
            var $div = ej.buildTag('div.e-gridheader'), temp, $frozenDiv, $movableDiv;
            var $innerDiv = ej.buildTag('div');
            if (this.model.allowScrolling && !this.model.scrollSettings.autoHide)
                $innerDiv.addClass("e-headercontent");
            this.setGridHeaderContent($div);
            this._hiddenColumns = [];
            this._columnsWidthCollection = [];
            this._visibleColumns = [];
            this._disabledGroupableColumns = [];
            this._fieldColumnNames = {};
            this._headerColumnNames = {};
            if (this.model.scrollSettings.frozenColumns > 0) {
                $frozenDiv = ej.buildTag("div.e-frozenheaderdiv", this._renderGridHeaderInternalDesign(this.model.columns.slice(0, this.model.scrollSettings.frozenColumns), true));
                $movableDiv = ej.buildTag("div.e-movableheader", ej.buildTag("div.e-movableheaderdiv", this._renderGridHeaderInternalDesign(this.model.columns.slice(this.model.scrollSettings.frozenColumns), false)));
                $innerDiv.append($frozenDiv).append($movableDiv);
            } else
                $innerDiv.append(this._renderGridHeaderInternalDesign(this.model.columns));
            $div.html($innerDiv);
			if (this.model.isResponsive)
                $div.addClass("e-textover");
            this.setGridHeaderTable(this.getHeaderContent().find(".e-table"));
            return $div;
        },
        _renderGridContent: function () {
            var $div = ej.buildTag('div.e-gridcontent');
            var $innderDiv = ej.buildTag('div');
            var $table = ej.buildTag('table.e-table', "", {}, { cellspacing: "0.25px" });
            var $tbody = $(document.createElement('tbody'));
            $table.append(this.getHeaderTable().find('colgroup').clone()).append($tbody);
            $innderDiv.html($table);
            $div.html($innderDiv);
            this.setGridContentTable($table);
            this.setGridContent($div);
            $table.attr("role", "grid");
            if (this._dataSource() == null) {
                var $emptyTd = ej.buildTag('td', this._getLocalizedLabels("EmptyRecord"), {}, { colSpan: this.model.columns.length });
                $tbody.append($(document.createElement("tr")).append($emptyTd));
                this.setWidthToColumns();
            } else {
                var args = {};
                if (this.model.groupSettings.groupedColumns.length) {
                    if (this._initialRender)
                        args.columnName = this.model.groupSettings.groupedColumns[this.model.groupSettings.groupedColumns.length - 1];
                    args.requestType = ej.Grid.Actions.Grouping;
                } else
                    args.requestType = ej.Grid.Actions.Refresh;
                this.sendDataRenderingRequest(args);
            }
            if (this._isCaptionSummary && args.requestType == "grouping" && this.model.groupSettings.groupedColumns.length > 1) {
                var colgroup = this.getContentTable().find(".e-table").not(".e-recordtable").children("colgroup");
                var $cols1 = $(this.getContentTable().find(".e-recordtable")[0]).children("colgroup").find("col");
                for (i = 0; i < colgroup.length; i++) {
                    var colCount = $(colgroup[i]).find("col").length;
                    $(colgroup[i]).find("col:gt(" + (colCount - $cols1.length - 1) + ")").remove();
                    $(colgroup[i]).append($cols1.clone());
                }
            }
            return $div;
        },
        /**
         * Print the grid control
         * @return jQuery
         * @example 
         * &lt;div id="Grid"&gt;&lt;/div&gt; 
         * &lt;script&gt;
         * // Create grid object.
         * var gridObj = $("#Grid").data("ejGrid");
         * // It prints the grid.
         * gridObj.print(); 
         * &lt;/script&gt;
         * @example 
         * &lt;div id="Grid"&gt;&lt;/div&dgt; 
         * &lt;script&gt;
         * // It prints the grid.
         * $("#Grid").ejGrid("print");        
         * &lt;/script&gt;
         */
			print:function(){
			var allowPaging=this.model.allowPaging;
			var elementClone=this.element.clone();	
			if(allowPaging){
				this.model.allowPaging=false;
				this.refreshContent();
				this.getPager().css('display','none');
				elementClone=this.element.clone();
				this.model.allowPaging=true;
				if(!this.model.allowScrolling){
					this.refreshContent();
					this.getPager().css('display','block');
					}
				}			
				
			if(this.model.allowScrolling){
				var scrollWidth=this.model.scrollSettings.width,scrollHeight=this.model.scrollSettings.height;
			    var scrollContent=this.getContent().find('.e-content')[0];
				$(elementClone.find('.e-gridcontent')).height(scrollContent.scrollHeight);
			    $(elementClone.find('.e-gridcontent')).ejScroller({width:scrollContent.scrollWidth,height:scrollContent.scrollHeight});
			    elementClone.width(scrollContent.scrollWidth);
				if(this.model.allowPaging){
					this.refreshContent();
					this.getPager().css('display','block');
					}
			}
			if (this.model.toolbarSettings.showToolbar)
			    elementClone.find(".e-gridtoolbar").remove();
			ej.print(elementClone);	
			},

        export: function (action, serverEvent, multipleExport) {
            var modelClone = $.extend(true,{},this.model);
            var attr = { action: action, method: 'post' };
            var form = ej.buildTag('form', "", null, attr);
            var gridob = this;
            if (this._ignoreOnExport) {
                for (var i = 0; i < this._ignoreOnExport.length; i++) {
                    delete modelClone[this._ignoreOnExport[i]];
                }
            }
            if (ej.raiseWebFormsServerEvents) {
                var args = { model: modelClone, originalEventType: serverEvent };
                var clientArgs = { model: JSON.stringify(modelClone) };
                ej.raiseWebFormsServerEvents(serverEvent, args, clientArgs);
            }
            else {
                var gridobjectArray = {};
                if (multipleExport && $('body').find('.e-grid').length > 1) {
                    form.attr('action', "Multiple" + action);
                    $('body').find('.e-grid').each(function (index, object) {
                        var gridObject = $(object).data('ejGrid');
                        var modelClone = JSON.parse(JSON.stringify(gridObject.model));
                        if (gridob._ignoreOnExport) {
                            for (var i = 0; i < gridob._ignoreOnExport.length; i++) {
                                delete modelClone[gridob._ignoreOnExport[i]];
                            }
                            gridobjectArray[index] = JSON.stringify(modelClone);
                            var inputAttr = { name: 'GridModel', type: 'hidden', value: JSON.stringify(modelClone) }
                            var input = ej.buildTag('input', "", null, inputAttr);
                            form.append(input);
                        }
                    });
                }
                else {
                    var inputAttr = { name: 'GridModel', type: 'hidden', value: JSON.stringify(modelClone) }
                    var input = ej.buildTag('input', "", null, inputAttr);
                    form.append(input);
                    form.append(this);
                }
                $('body').append(form);
                form.submit();
            }
            return true;
        },
        sendDataRenderingRequest: function (args) {
        if (args.requestType == "add" ||( this.model.currentViewData!=null && this.model.currentViewData.length)){
                switch (args.requestType) {
                    case ej.Grid.Actions.Refresh:
                    case ej.Grid.Actions.Paging:
                    case ej.Grid.Actions.Sorting:
                    case ej.Grid.Actions.Filtering:
                    case ej.Grid.Actions.Save:
                    case ej.Grid.Actions.Cancel:
                    case ej.Grid.Actions.Delete:
                    case ej.Grid.Actions.Search:
                    case ej.Grid.Actions.Reorder:
                    case ej.Grid.Actions.BatchSave:
                        if (this.model.groupSettings.groupedColumns.length == 0) {
                            var temp = document.createElement('div'),temp1, insertIndex = -1, isRemoteAdaptor = false;
                            if (!this.model.isResponsive)
                                this.getContentTable().find("colgroup").first().replaceWith(this._getMetaColGroup());
                            (this.model.detailsTemplate != null || this.model.childGrid != null || this.model.detailsDataBound != null) && this.getContentTable().find("colgroup").first().prepend(this._getIndentCol());
                            var currentPage = this._currentPage();
                            if (this.model.editSettings.editMode == "externalform" || this.model.editSettings.editMode == "externalformtemplate")
                                $("#" + this._id + "_externalEdit").css("display", "none");
                            if (this.model.scrollSettings.frozenColumns > 0)
                                temp.innerHTML = this._renderByFrozenDesign();
                            else {
                                temp.innerHTML = ['<table>', $.render[this._id + "_JSONTemplate"](this.model.currentViewData), '</table>'].join("");
								if(args.data) {
									temp1 = document.createElement('div');
									temp1.innerHTML = ['<table>', $.render[this._id + "_JSONTemplate"](args.data), '</table>'].join("");
									if (this._dataSource() instanceof ej.DataManager && args.requestType == ej.Grid.Actions.Save) {
									    insertIndex = this._getDataIndex(this.model.currentViewData, args.data);
									    isRemoteAdaptor = this._dataSource().adaptor instanceof ej.remoteSaveAdaptor;
								}
								}
                                var tableEle = this.getContentTable().get(0);
                                var tbodyEle = tableEle.lastChild;
                                var rindex = this.getContentTable().first().find('tbody').first();
                                if ((args.requestType == "save" || args.requestType == "cancel") && this.model.editSettings.editMode != "batch") {
                                    rowIndex = !ej.isNullOrUndefined(args.selectedRow) ? args.selectedRow : this._selectedRow();
                                    var a = this._currentTrIndex;
                                    if (rowIndex == -1)
                                        rowIndex = a;
                                    if (this.model.detailsTemplate != null) {
                                        rowTr = $(tbodyEle.childNodes[rowIndex]);
                                        rowEle = $(tbodyEle.childNodes).not('.e-detailrow');
                                        for (var i = 0; i < rowEle.length; i++) {
                                            if (rowTr.is(rowEle[i]))
                                                rowIndex = i;
                                        }
                                    }

                                    if (this.model.editSettings.editMode == "normal" || this.model.editSettings.editMode == "inlineform" || this.model.editSettings.editMode == "inlineformtemplate") {                                      
                                        $oldChild = this.getContentTable().find('.e-addedrow').get(0);
                                        $editedTr = this.getContentTable().find('.e-editedrow');
                                        $newChild = ($editedTr.length || args.requestType == "cancel") ? temp.firstChild.firstChild.firstChild : temp1.firstChild.firstChild.lastChild;
                                        if ($editedTr.length) {
                                            $newChild = temp.firstChild.firstChild.childNodes[rowIndex];
                                            if (this.model.editSettings.editMode == "inlineform" || this.model.editSettings.editMode == "inlineformtemplate") {
                                                $oldChild = $editedTr.prev('tr').get(0);
                                                $editedTr.remove();
                                            } else
                                                $oldChild = $editedTr.get(0);
                                            if (args.requestType == "cancel") {
                                                $($oldChild).replaceWith($($newChild));
                                            } else if (!ej.isNullOrUndefined(this._filteredRecordsCount) && this._filteredRecordsCount < this._previousFilterCount) {
                                                if (this.model.detailsTemplate != null && $($oldChild).next('tr.e-detailrow').length)
                                                    tbodyEle.removeChild($($oldChild).next('tr.e-detailrow').get(0));
                                                $($oldChild).remove();
                                                if (this.model.pageSettings.pageSize <= this.model.currentViewData.length && this.model.groupSettings.groupedColumns.length == 0)
                                                    tbodyEle.appendChild(temp.firstChild.firstChild.lastChild);
                                            } else {
                                                tbodyEle.replaceChild($newChild, $oldChild);
                                            }
                                        } else {
                                            if (args.requestType == "cancel" || this._dataSource() instanceof ej.DataManager || this._currentPage() != 1 || (args.requestType == "save" && !ej.isNullOrUndefined(this._filteredRecordsCount) && this._filteredRecordsCount == this._previousFilterCount)) {
                                                if (!ej.isNullOrUndefined($oldChild)) {
                                                    $($oldChild).remove();
                                                    if (this._dataSource() instanceof ej.DataManager && insertIndex != -1) {
                                                        if (insertIndex == 0)
                                                            tbodyEle.insertBefore($newChild, tbodyEle.children[insertIndex]);
                                                        else
                                                            $($newChild).insertAfter(tbodyEle.children[insertIndex - 1]);                                                        
                                                    }
                                                    else if ((!(this._dataSource() instanceof ej.DataManager) || isRemoteAdaptor) && this._currentPage() != 1 && args.requestType == "save")
                                                        $(tbodyEle).prepend($(temp.firstChild.firstChild.firstChild));
                                                    if (this.model.pageSettings.pageSize <= this.model.currentViewData.length && this.model.groupSettings.groupedColumns.length == 0 &&
                                                        ((this._dataSource() instanceof ej.DataManager && insertIndex == -1 && (!isRemoteAdaptor && args.requestType != "save")) ||
                                                        (args.requestType != "save" && !(this._dataSource() instanceof ej.DataManager))))
                                                        tableEle.lastChild.appendChild(temp.firstChild.firstChild.lastChild);
                                                }
                                                if (args.requestType == "cancel" && this._selectedRow() != -1)
                                                    this.clearSelection();

                                            } else if (this.model.currentViewData.length == 1) {
                                                $(tbodyEle).empty();
                                                tbodyEle.appendChild($newChild);
                                            } else if (this.model.editSettings.rowPosition == "bottom") {
                                                rindex.prepend($oldChild);
                                                tbodyEle.replaceChild($newChild, $oldChild);
                                            } else
                                                tbodyEle.replaceChild($newChild, $oldChild);
                                        }
                                    } else if (this.model.editSettings.editMode == "dialog" || this.model.editSettings.editMode == "dialogtemplate" || this.model.editSettings.editMode == "externalform" || this.model.editSettings.editMode == "externalformtemplate") {
                                        $editedTr = this.element.find('.e-editedrow');
                                        if (args.requestType == "cancel" || (!$editedTr.length && !ej.isNullOrUndefined(this._filteredRecordsCount) && this._filteredRecordsCount == this._previousFilterCount)) {
                                            $newChild = temp.firstChild.firstChild.childNodes[rowIndex];
                                            $oldChild = tbodyEle.childNodes[rowIndex];
                                            $($oldChild).replaceWith($($newChild));
                                            this.clearSelection();
                                            this.model.allowPaging && this._refreshGridPager();

                                        } else if ($editedTr.length) {
                                            $newChild = temp.firstChild.firstChild.childNodes[rowIndex];
                                            $oldChild = tbodyEle.childNodes[rowIndex];
											if (this.model.allowCellMerging != null) {
                                                if ($($oldChild.childNodes).hasClass("e-merged")) {
                                                    var index = $($oldChild).children('.e-merged').index();
                                                    var count = $oldChild.children[index].colSpan;
                                                    for (var i = 0 ; i < count; i++) {
                                                        $newChild.childNodes[index + i].className += " e-merged e-hide";
                                                        $newChild.childNodes[index].colSpan = i + 1;
                                                    }
                                                }
                                            }
                                            if (this.model.detailsTemplate != null)
                                                $oldChild = $(tbodyEle.childNodes).not('.e-detailrow').eq(rowIndex).get(0);
                                            if (!ej.isNullOrUndefined(this._filteredRecordsCount) && this._filteredRecordsCount < this._previousFilterCount) {
                                                if (this.model.detailsTemplate != null && $($oldChild).next('tr.e-detailrow').length)
                                                    tbodyEle.removeChild($($oldChild).next('tr.e-detailrow').get(0));
                                                $($oldChild).remove();
                                                if (this.model.pageSettings.pageSize <= this.model.currentViewData.length)
                                                    tbodyEle.appendChild(temp.firstChild.firstChild.lastChild);
                                            } else
                                                tbodyEle.replaceChild($newChild, $oldChild);
                                        } else if (this.model.currentViewData.length == 1 && this.getContentTable().find('td.e-rowcell').length == 0) {
                                            $newChild = temp.firstChild.firstChild.firstChild;
                                            $(tbodyEle).empty();
                                            tbodyEle.appendChild($newChild);
                                        } else {
                                            newChild = ($editedTr.length || args.requestType == "cancel") ? temp.firstChild.firstChild.firstChild : temp1.firstChild.firstChild.lastChild;
                                            if (!(this._dataSource() instanceof ej.DataManager)) {
                                                if (this._currentPage() == 1)
                                            this.getContentTable().find('tbody').first().prepend($(newChild));
                                                else
                                                    this.getContentTable().find('tbody').first().prepend($(temp.firstChild.firstChild.firstChild));
                                            if (this.model.pageSettings.pageSize <= this.model.currentViewData.length)
                                                tbodyEle.removeChild(tbodyEle.lastChild);
                                            }
                                            else if (insertIndex != -1) {
                                                if (insertIndex == 0)
                                                    tbodyEle.insertBefore(newChild, tbodyEle.children[insertIndex]);
                                                else
                                                    $(newChild).insertAfter(tbodyEle.children[insertIndex - 1]);
                                                if (this.model.allowPaging && (this.model.pageSettings.pageSize <= this.model.currentViewData.length || insertIndex == this.model.pageSettings.pageSize - 1))
                                                    tbodyEle.removeChild(tbodyEle.lastChild);
                                            }
                                            if (this.model.detailsTemplate != null && $(tableEle.lastChild.lastChild).children('.e-detailrowexpand').length)
                                                tbodyEle.removeChild(tbodyEle.lastChild);
                                        }
                                    }
                                    if (this.model.enableAltRow)
                                        this._refreshAltRow();
                                } else if (args.requestType == "delete") {
                                    if (this._isUnboundColumn) {
                                    $editedrow = this.element.find('.e-editedrow');
                                    $oldChild = this.getContentTable().find('.e-editedrow').get(0);
                                    $newChild = ($editedrow.length) ? temp.firstChild.firstChild.firstChild : temp1.firstChild.firstChild.lastChild;

                                    if ($editedrow.length != 0 && (this.model.editSettings.editMode == "normal" || this.model.editSettings.editMode == "externalform" || this.model.editSettings.editMode == "externalformtemplate")) {
                                        $($oldChild).replaceWith($($newChild));
                                    }
                                    else if (this.model.editSettings.editMode == "inlineform" || this.model.editSettings.editMode == "inlineformtemplate") {
                                        $oldChild = $editedrow.prev('tr').get(0);
                                        $editedrow.remove();
                                    }
                                    else
                                        $oldChild = $editedrow.get(0);
                                    }
                                    if (this.model.pageSettings.pageSize <= this.model.currentViewData.length)
                                        tbodyEle.appendChild(temp.firstChild.firstChild.lastChild);
                                    if (this.model.detailsTemplate != null) {
                                        var visibleRow = this.getContentTable().find('.e-detailrow:visible');
                                        $.each(visibleRow, function (indx, item) {
                                            if (visibleRow.eq(indx).closest('tr').prev().children('.e-detailrowexpand').length == 0)
                                                visibleRow.eq(indx).remove();
                                        });
                                    }
                                    if (this.model.enableAltRow)
                                        this._refreshAltRow();
                                } else
                                    this.getContentTable().get(0).replaceChild(this.model.rowTemplate != null ? temp.firstChild.lastChild : temp.firstChild.firstChild, this.getContentTable().get(0).lastChild);

                            }
                            this._currentJsonData = this.model.currentViewData;
                            this._gridRows = this.getContentTable().get(0).rows;
                            if (this.model.scrollSettings.frozenColumns > 0)
                                this._gridRows = [this._gridRows, this.getContentTable().get(1).rows];
                            this._eventBindings();
                            var model = {};
                            if ((args.requestType == "sorting" || args.requestType == "filtering") && this.model.scrollSettings.allowVirtualScrolling) {
                                if (args.requestType == "filtering") {
                                    this.getContent().first().ejScroller("refresh").ejScroller("isVScroll") ? this.element.find(".gridheader").addClass("e-scrollcss") : this.element.find(".gridheader").removeClass("e-scrollcss");
                                    var model = this._refreshVirtualPagerInfo();
                                }
                                this._refreshVirtualContent(currentPage, model);
                                args.requestType == "filtering" && this.getContent().first().ejScroller("refresh");
                            }
                            if (this.model.allowPaging) {
                                if (this.model.filterSettings.filteredColumns.length)
                                    this.getPager().ejPager({ totalRecordsCount: this._searchCount == null ? this._filteredRecordsCount : this._searchCount, currentPage: this._currentPage() });
                                else
                                    this.getPager().ejPager({ totalRecordsCount: this._searchCount == null ? this._gridRecordsCount : this._searchCount, currentPage: this._currentPage() });
                                this._refreshGridPager();
                            }
                            break;
                        }
                    case ej.Grid.Actions.Grouping:
                        this._group(args);
                        break;
                    case ej.Grid.Actions.BeginEdit:
                        this._edit(args);
                        break;
                    case ej.Grid.Actions.Add:
                        this._add(args);
                        break;
                    case ej.Grid.Actions.Ungrouping:
                        this._ungroup(args);
                        break;
                    case ej.Grid.Actions.VirtualScroll:
                        !this._isVirtualRecordsLoaded && this._replacingContent();
                        break;
                }
            } else
                this.getContentTable().find('tbody').empty().first().append(this._getEmptyTbody());
            this._completeAction(args);
        },
        _completeAction: function (args) {
            this.model.isEdit = false;
            this._confirmedValue = false;
            if (ej.Grid.Actions.Grouping == args.requestType && ej.isNullOrUndefined(args.columnName))
                return;
            this.setWidthToColumns();
            if (ej.Grid.Actions.UnGrouping == args.requestType || this.model.groupSettings.groupedColumns.length && !$("#" + this._id + "EditForm").length)
                this._recalculateIndentWidth();
            if (ej.Grid.Actions.Paging == args.requestType || ej.Grid.Actions.BatchSave == args.requestType)
                this._refreshGridPager();
            else if ((ej.Grid.Actions.Sorting == args.requestType && this.model.allowSorting) || ej.Grid.Actions.Refresh == args.requestType) {
                if (ej.gridFeatures.sort && this.getHeaderTable() !== null)
                    this._sortCompleteAction(args);
                if(this.model.allowPaging){		
					var pageModel=this.getPager().ejPager("option");
					this._currentPage(pageModel.currentPage);
					delete pageModel.currentPage;
					$.extend(this.model.pageSettings, pageModel);
					delete this.model.pageSettings.masterObject;
					this._refreshGridPager();
				}
                if (!this._initialRender && (this.model.scrollSettings.frozenRows > 0 || this.model.scrollSettings.frozenColumns > 0))
                    this._refreshScroller(args);
            } else if (ej.Grid.Actions.Delete == args.requestType || ej.Grid.Actions.Save == args.requestType || ej.Grid.Actions.Search == args.requestType) {
                this._editEventTrigger(args);
                if (this.model.allowPaging)
                    this._refreshPagerTotalRecordsCount();
            } else if (ej.Grid.Actions.Filtering == args.requestType)   
                this._filterCompleteAction();
            else if (ej.Grid.Actions.BeginEdit == args.requestType || ej.Grid.Actions.Add == args.requestType)
                this._editCompleteAction(args);
            else if (ej.Grid.Actions.Grouping == args.requestType || ej.Grid.Actions.Ungrouping == args.requestType)
                this["_" + args.requestType + "CompleteAction"](args);
            if (this.model.toolbarSettings.showToolbar)
                this.refreshToolbar();
            if (!this._initialRender && this.model.showSummary && this.model.summaryRows.length > 0) {
                this.getFooterTable().find('colgroup').remove();
                this.getFooterTable().append(this.getHeaderTable().find("colgroup").clone());
                this._createSummaryRows(this.getFooterTable());
            }
            if (!this._initialRender && ej.gridFeatures.selection)
                this._setCurrentRow(args.requestType);
            this.model.editSettings.editMode == "batch" && this.refreshBatchEditMode();
            if (!this._initialRender && this.model.allowScrolling && (this._checkScrollActions(args.requestType) || (this.model.editSettings.editMode.indexOf("inline") != -1 && args.requestType == "beginedit")) ||
                (this.model.scrollSettings.virtualScrollMode == "continuous" && args.requestType == "virtualscroll"))
                this._refreshScroller(args);
            if (this._customPop != null && args.requestType != "sorting") {
                this._customPop.hide();
            }
            if (this.model.allowScrolling && (ej.Grid.Actions.Filtering == args.requestType || ej.Grid.Actions.Paging == args.requestType || ej.Grid.Actions.Sorting == args.requestType ||
                ej.Grid.Actions.Grouping == args.requestType || ej.Grid.Actions.Ungrouping == args.requestType)) {
                this.getContentTable().find("tr:last").find("td").addClass("e-lastrowcell");
            }
            this._trigger("actionComplete", args);
            if ((ej.Grid.Actions.BeginEdit == args.requestType || ej.Grid.Actions.Add == args.requestType) && $.isFunction($.validator))
                this.setValidation();
        },
        _getForeignKeyData: function (data) {
            var proxy = this;
            var column;
            for (i = 0; i < this.model.columns.length; i++) {
                if (this.model.columns[i].foreignKeyValue && this.model.columns[i].dataSource) {
                    var fieldName = ej.isNullOrUndefined(proxy.model.columns[i]["foreignKeyField"]) ? proxy.model.columns[i]["field"] : proxy.model.columns[i]["foreignKeyField"];
                    this.model.columns[i].dataSource.filter(function (col) {
                        var value = data[proxy.model.columns[i]["field"]];
                        var fValue = !isNaN(parseFloat(value)) && isFinite(value) ? parseFloat(value) : value;
                        if (col[fieldName] == fValue)
                            return column = col;
                    });
                }
            }
            return column;
        },
        _foreignKeyBinding: function (curColumn, cellValue, gridId) {
            if (typeof (curColumn) == "string")
                curColumn = JSON.parse(curColumn);
            var cellData;
            curColumn.dataSource.filter(function (col) {
                if (col[curColumn.foreignKeyField] == cellValue)
                    return cellData = curColumn.type == "date" ? new Date(col[curColumn.foreignKeyValue]) : col[curColumn.foreignKeyValue];
            });
            if (curColumn.format) {
                var gridObj = ej.isNullOrUndefined(gridId) ? this : this.getRsc("helpers", gridId);
                cellData = gridObj.formatting(curColumn.format, cellData);
            }
            return cellData;
        },
        _getDataIndex: function (data, item) {
            var flag = 0, _plen;
            for (var d = 0, len = data.length; d < len; d++) {
                for(var key = 0, _plen = this._primaryKeys.length; key < _plen; key++) {
                 if (data[d][this._primaryKeys[key]] != item[this._primaryKeys[key]])
                     continue;                                  
                 else if(key == _plen-1)
                     flag = 1;
              }
               if(flag) return d;      
            }
            return -1;
        },
        _eventBindings: function () {
            var rowLength = this.model.frozenColumns > 0 ? this._gridRows[0].length : this._gridRows.length;
            if (ej.gridFeatures.common)
                this._refreshUnboundTemplate(this.getContentTable());
            if (this._gridRecordsCount != 0) {
                if (this.model.queryCellInfo != null || this.model.rowDataBound != null || this.model.mergeCellInfo != null) {
                    for (var row = 0; row < rowLength; row++)
                        this._rowEventTrigger(this.getRowByIndex(row), this._currentJsonData[row]);
                }
            }
        },
        _rowEventTrigger: function (row, data) {
            var args = { row: row, data: data };
            this._trigger("rowDataBound", args);
            var tdCells = row.cells;
            var $tdRowcells = $(row).find(".e-rowcell");
            for (var cellIndex = 0; cellIndex < $tdRowcells.length; cellIndex++) {
                var args = { cell: $tdRowcells[cellIndex], data: data, text: $tdRowcells[cellIndex].innerHTML };
                var foreignKeyData = this._getForeignKeyData(args.Data);
                if ($($tdRowcells[cellIndex]).hasClass("e-rowcell"))
                    args.column = this.model.columns[cellIndex];
                if (!ej.isNullOrUndefined(foreignKeyData))
                    args.foreignKeyData = foreignKeyData;
				if (this.model.allowCellMerging == true) {
                    this._cellMerging(args);
                    this._trigger("mergeCellInfo", args);
                }
                this._trigger("queryCellInfo", args);
            }

        },
        formatting: function (formatstring, str) {
            formatstring = formatstring.replace(/%280/g, "\"").replace(/&lt;/g, "<").replace(/&gt;/g, ">");
            var s = formatstring;
            var frontHtmlidx, FrontHtml, RearHtml, lastidxval;
            frontHtmlidx = formatstring.split("{0:");
            lastidxval = formatstring.split("}");
            FrontHtml = frontHtmlidx[0];
            RearHtml = lastidxval[1];
            if (typeof (str) == "string" && $.isNumeric(str))
                str = Number(str);
            if (formatstring.indexOf("{0:") != -1) {
                var toformat = new RegExp("\\{0(:([^\\}]+))?\\}", "gm");
                var formatVal = toformat.exec(formatstring);
                if (formatVal != null && str != null) {
                    if (FrontHtml != null && RearHtml != null)
                        str = FrontHtml + Globalize.format(str, formatVal[2]) + RearHtml;
                    else
                        str = Globalize.format(str, formatVal[2]);
                } else if (str != null)
                    str = str;
                else
                    str = "";
                return str;
            } else if (this.data != null && this.data.Value == null) {
                $.each(this.data, function (dataIndex, dataValue) {
                    s = s.replace(new RegExp('\\{' + dataIndex + '\\}', 'gm'), dataValue);
                });
                return s;
            } else {
                return this.data.Value;
            }
        },
        
        setWidthToColumns: function () {
            var $cols1 = this.getContentTable().children("colgroup").find("col");
            var $cols2 = this.getHeaderTable().children("colgroup").find("col");
            var width = this.element.width(), frozenWidth = 0;
            if (this.model.groupSettings.groupedColumns.length && !this.model.allowScrolling && this.model.groupSettings.showGroupedColumn) {
                var browserDetails = this.getBrowserDetails();
                if (browserDetails.browser == "msie" && parseInt(browserDetails.version, 10) > 8)
                    $cols1.first().css("width", ((30 / width) * 100) + "%");
            }
            var colCount = this.model.columns.length;
            if ($cols1.length > colCount) $cols1.splice(0, ($cols1.length - colCount));
            if ($cols2.length > colCount) $cols2.splice(0, ($cols2.length - colCount));

            for (var i = 0; i < $cols2.length; i++) {
                if (this.model.allowResizeToFit && this.model.columns[i]["width"] === undefined) {
                    hCellIndex = this.model.groupSettings.groupedColumns.length ? (i + this.model.groupSettings.groupedColumns.length) : i;
                    var contentWidth = this._resizer._getContentWidth(i);
                    var headerWidth = this._resizer._getHeaderContentWidth(this.getHeaderTable().find('.e-headercelldiv').eq(hCellIndex));
                    var finalWidth = contentWidth > headerWidth ? contentWidth : headerWidth;
                    finalWidth = finalWidth + (finalWidth * 0.2);
                    this._columnsWidthCollection[i] = finalWidth;
                }
                if (!ej.isNullOrUndefined(this._columnsWidthCollection[i])) {
                    $cols1.eq(i).width(this._columnsWidthCollection[i]);
                    $cols2.eq(i).width(this._columnsWidthCollection[i]);
                } else if (this.model.allowScrolling) {
                    var colWidth = (width / this.model.columns.length).toFixed(2) + "px";
                    $cols1.eq(i).css("width", colWidth);
                    $cols2.eq(i).css("width", colWidth);
                    this._columnsWidthCollection[i] = parseFloat((width / this.model.columns.length).toFixed(2));
                }
            }
            if (this.model.isEdit) {
                var clonedCol = $cols1.clone(), $colGroup = this.model.scrollSettings.frozenColumns > 0 ? this.getContent().find(".gridform").find("colgroup") : $("#" + this._id + "EditForm").find("colgroup");
                this.model.scrollSettings.frozenColumns > 0 && $colGroup.first().empty().append(clonedCol.splice(0, this.model.scrollSettings.frozenColumns));
                $colGroup.last().empty().append(clonedCol);
                this.model.detailsTemplate != null && $colGroup.prepend(this._getIndentCol());
            }
            if (this.model.groupSettings.groupedColumns.length) {
                var $grouedColGroup = this.getContentTable().find(".e-recordtable").children("colgroup");
                for (var i = 0; i < $grouedColGroup.length; i++) {
                    var clonedCol = $cols1.clone();
                    if (this.model.detailsTemplate != null || this.model.childGrid != null) clonedCol.splice(0, 0, this._getIndentCol()[0]);
                    $grouedColGroup.eq(i).empty().append(clonedCol);
                }
            }
            if (this.model.scrollSettings.frozenColumns > 0) {
                var totalWidth = 0, frozenWidth;
                for (var i = 0; i < this._columnsWidthCollection.length; i++) {
                    totalWidth += this._columnsWidthCollection[i];
                    if (this.model.scrollSettings.frozenColumns - 1 == i)
                        frozenWidth = Math.ceil(totalWidth);
                }
                this.getContent().find(".e-frozencontentdiv").outerWidth(frozenWidth)
                    .end().find(".e-movablecontentdiv").outerWidth(totalWidth - frozenWidth);
                this.getHeaderContent().find(".e-frozenheaderdiv").outerWidth(frozenWidth)
                    .end().find(".e-movableheaderdiv").outerWidth(totalWidth - frozenWidth);
            }
        },
        _initialEndRendering: function () {
            // use this method to add behaviour after grid render.
            if (this.model.editSettings.allowEditing || this.model.editSettings.allowAdding) {
                if (this.model.editSettings.editMode == "dialog" || this.model.editSettings.editMode == "dialogtemplate")
                    this.element.append(this._renderDialog());
                else if (this.model.editSettings.editMode == "externalform" || this.model.editSettings.editMode == "externalformtemplate")
                    this.element.append(this._renderExternalForm());
            }
            if (this._selectedRow() != -1)
                this.selectRows(this._selectedRow());
            this.model.editSettings.editMode == "batch" && this._renderConfirmDialog();
            this.model.scrollSettings.frozenColumns > 0 && this._renderAlertDialog();
            if (this.model.allowMultiSorting || this.model.selectionType == "mulitple")
                this._renderMultiTouchDialog();
            if (this.model.scrollSettings.frozenColumns > 0 && !this.model.allowScrolling) {
                this.getContent().remove();
                this.getHeaderTable().eq(1).remove();
                this._alertDialog.find(".e-content").text(this._getLocalizedLabels("FrozenColumnsScrollAlert"));
                this._alertDialog.ejDialog("open");
                return;
            }
            this.model.scrollSettings.allowVirtualScrolling && this._createPagerStatusBar();
            this._getRowHeights();
            this.model.allowScrolling && this._renderScroller();
            if (this.model.scrollSettings.allowVirtualScrolling) {
                this._refreshVirtualContent();
                this.getContent().first().ejScroller("refresh");
                if (this.getContent().ejScroller("isVScroll"))
                    this.element.find(".e-gridheader").addClass("e-scrollcss");
                else
                    this.element.find(".e-gridheader").removeClass("e-scrollcss");
            }
            if (this.model.allowFiltering && this.model.filterSettings.filterType == "filterbar" && !this.model.allowPaging && !this.model.scrollSettings.allowVirtualScrolling)
                this._createPagerStatusBar();
            if (ej.gridFeatures.common)
                this.rowHeightRefresh()
        },
        _getRowHeights: function () {
            var trs = this.getRows();
            if (trs !== null) {
                this._rowHeightCollection = [];
                if ((this.model.scrollSettings.frozenColumns > 0 && trs[0] !== undefined) || (typeof trs[0].item !== "undefined" && typeof trs[0].length == "number"))
                    trs = trs[0];
                for (var i = 0; i < trs.length; i++) {
                    this._rowHeightCollection[i] = trs[i].offsetTop;
                }
            }
            return this._rowHeightCollection;
        },
        _getEmptyTbody: function () {
            var $emptyTd = ej.buildTag('td.emptyrecord', this._getLocalizedLabels("EmptyRecord"), {}, { colSpan: this.model.columns.length });
            return $(document.createElement("tr")).append($emptyTd);
        },
        _getIndentCol: function () {
            return ej.buildTag("col", "", { width: "30px" });
        },
        _createSortElement: function () {
            return ej.buildTag('span.e-icon', "&nbsp;");
        },
        _wireEvents: function () {
            this._on(this.element, ($.isFunction($.fn.tap) && this.model.enableTouch) ? "tap" : "click", this._clickHandler);
            this._on(this.element, ($.isFunction($.fn.tap) && this.model.enableTouch) ? "tap" : "click", ".e-gridheader", this._mouseClickHandler);
            if (ej.gridFeatures.common) {
                this._on(this.element, "dblclick", ".e-gridcontent", this._recorddblClickHandler);
                if (this.model.rightClick)
                    this._on(this.element, "contextmenu", this._rightClickHandler);
                this._on(this.element, "click", ".e-gridcontent", this._recordClick);
                this._enableRowHover();
                if (this.model.enableTouch)
                    this._on(this.element, "swipeleft swiperight", ".e-gridcontent .e-table", $.proxy(this._touchGrid, this));
                else
                    this.element.addClass(".e-touch");
                this._on(this.element, "mousedown", ".e-gridheader", this._headerMouseDown);
                this._on(this.element, "mouseover mouseleave", ".e-gridheader", this._headerHover);
                this._on(this.element, ej.eventType.mouseMove, ".e-gridheader", this._headerHover);
                this.model.allowResizeToFit && this._on(this.element, "dblclick", ".e-gridheader", this._headerdblClickHandler);
                if (this.model.allowResizing) {
                    this._on(this.element, ej.eventType.mouseMove, this._mouseMove);
                    this._on(this.element, "mouseup", this._mouseUp);
                }
				if (this.model.allowKeyboardNavigation) {
					this.element[0].tabIndex = 0;
					this.element[0].accessKey = "e";
					this._on(this.element, "keyup", this._keyDownHandler);
                }
            }
            if (ej.gridFeatures.edit) {
                this._enableEditingEvents();
                this._on(this.element, "click", ".e-gridcontent .e-unboundcelldiv", this._unboundClickHandler);
            }
            if (this.model.allowGrouping) {
                this._enableGroupingEvents();
                this._on(this.element, "mouseenter mouseleave", ".e-groupdroparea,.e-groupheadercell", this._dropAreaHover);
               
            }
            if (!this.model.editSettings.allowEditing)
                this._on(this.element, "dblclick", ".e-gridcontent", this._recorddblClickHandler);
            this._enableFilterEvents();
        },
        _enableFilterEvents: function () {
            if (this.model.allowMultiSorting || this.model.selectionType == "multiple" || this.model.allowFiltering)
                this._on($(document), "click", this._docClickHandler);
            if (this.model.allowFiltering) {
                var proxy = this, $target;
                this._off(this.element, "keyup", ".e-filterbar input")._on(this.element, "keyup", ".e-filterbar input", this._filterBarHandler);
                this._on(this.element, "focus click", ".e-filterbar", this._filterBarClose);
            }
        },
        _docClickHandler: function (e) {
            var details = this.getBrowserDetails();
            if (this._customPop != null && this.element.find(e.target).length == 0)
                this._customPop.hide();
            if (this.model.allowFiltering) {
                if (this.model.filterSettings.filterType == "menu" || this.model.filterSettings.filterType == "excel") {
                    if (this._$colType && ($(e.target).find(".e-grid.e-dlgcontainer").length > 1 || $(e.target).find(".e-excelfilter").length > 1))
                        if (details.browser == "msie")
                            e.target.tagName != "BODY" && (!this.isExcelFilter ? this._closeFilterDlg() : this._excelFilter.closeXFDialog(e));
                        else
                            !this._isExcelFilter ? this._closeFilterDlg() : this._excelFilter.closeXFDialog(e);
                } else if (!$(e.target).hasClass("e-filtertext"))
                    this.getFilterBar().find(".e-cancel").addClass("e-hide");
            }

        },
        _mouseClickHandler: function (e) {
            if (this.getHeaderTable().find('.e-columnheader').css('cursor') == "col-resize")
                return;
            if ($(e.target).is(".e-ascending, .e-descending"))
                e.target = e.target.parentNode;
            var $target = $(e.target);
            if (this._$fDlgIsOpen && this.model.allowFiltering && (this.model.filterSettings.filterType == "menu" || this._isExcelFilter)) {
                $.fx.off = true;
                if (this._isExcelFilter)
                    this._excelFilter.closeXFDialog();
                else
                    $("#" + this._id + "_" + this._$prevColType + "Dlg").ejDialog("close");
                $.fx.off = false;
            }
            this.getHeaderTable().find(".e-columnheader").find(".e-headercellactive").removeClass("e-headercellactive").removeClass("e-active");
            if ($target.hasClass("e-headercelldiv") || (!$target.hasClass("e-togglegroupbutton") && $target.closest(".e-headercelldiv").length && $.inArray($target[0].tagName, ["SELECT", "INPUT", "TEXTAREA"]) == -1)) {
                if (!this.model.allowSorting || ej.gridFeatures.sort === undefined)
                    return;
                $target = $target.hasClass("e-headercelldiv") ? $target : $target.closest(".e-headercelldiv");
                var columnName = $target.attr("ej-mappingname");
                var columnSortDirection = ej.sortOrder.Ascending;
                this._$prevSElementTarget = this._$curSElementTarget;
                this._$curSElementTarget = $target;
                if ($target.find('span').hasClass("e-ascending"))
                    var columnSortDirection = ej.sortOrder.Descending;
                else
                    var columnSortDirection = ej.sortOrder.Ascending;
                if (e["pointerType"] == "touch" && this._customPop != null && !this._customPop.is(":visible") && this._customPop.find(".e-sortdirect").hasClass("e-spanclicked"))
                    this._customPop.show();
                if (e["pointerType"] == "touch" && this._customPop != null && (this._customPop.find(".e-rowselect").is(":visible") || !this._customPop.find(".e-sortdirect").hasClass("e-spanclicked")) && this.model.allowMultiSorting) {
                    var $offset = $target.offset();
                    this._customPop.removeAttr("style");
                    this._customPop.offset({ left: $offset.left, top: $offset.top - this.getHeaderTable().find(".e-columnheader").height() }).find(".e-sortdirect").show().end()
                        .find(".e-rowselect").hide().end().show();
                }
                if (this.model.allowMultiSorting && (e.ctrlKey || this._enableSortMultiTouch))
                    this.multiSortRequest = true;
                if (e.shiftKey && $.inArray(columnName, this.model.groupSettings.groupedColumns) == -1) {
                    this._removeSortedColumnFromCollection(columnName);
                    this.multiSortRequest = true;
                    columnName = null;
                }
                this.sortColumn(columnName, columnSortDirection);
            } else if ($target.hasClass("e-togglegroupbutton") && this.model.allowGrouping) {
                var field = $target.parent().attr("ej-mappingname");
                $target.hasClass("e-togglegroup") && this.groupColumn(field);
                $target.hasClass("e-toggleungroup") && this.ungroupColumn(field);
            } else if ($target.hasClass("e-filtericon") || $target.hasClass("e-filteredicon")) {
                var columnName = $target.parent().find(".e-headercelldiv").attr("ej-mappingname");
                this._$prevFieldName = this._$curFieldName;
                if (this.model.allowFiltering) {
                    var proxy = this;
                    $.each(this.model.columns, function (indx, col) {
                        if (col.field == columnName) {
                            proxy._$colType = col.type;
                            proxy._$curFieldName = col.field;
                            proxy._$colFormat = col.format;
                            proxy._$colForeignKeyField = col.foreignKeyField ? col.foreignKeyField : col.field;
                            proxy._$colForeignKeyValue = col.foreignKeyValue;
                            proxy._$colDropdownData = col.dataSource;
                        }
                    });
                     if (!this._isExcelFilter) {
                        var $id = "#" + this._id + "_" + this._$colType + "Dlg";
                        if (this._$colType == "string") {
                            if (this._$colForeignKeyValue && this._$colDropdownData)
                                $("#" + this._id + "_acString").ejAutocomplete({ fields: { text: proxy._$colForeignKeyValue, key: proxy._$colForeignKeyField }, dataSource: proxy._$colDropdownData });
                            else
                                $("#" + this._id + "_acString").ejAutocomplete({ fields: { text: proxy._$curFieldName, key: this._getIdField() }, dataSource: this._dataSource() });
                        } else if (this._$colType == "date") {
                            if (this._$colFormat != undefined) {
                                this._$colFormat = this._$colFormat.replace("{0:", "").replace("}", "");
                                $($id).find(".e-datewidget input").ejDatePicker({ dateFormat: this._$colFormat.replace("{0:", "").replace("}", "") });
                            }
                            else
                                $($id).find(".e-datewidget input").ejDatePicker({ dateFormat: "MM/dd/yyyy" });
                        }
                         else if (this._$colType == "datetime") {
                        if (this._$colFormat != undefined) {
                            this._$colFormat = this._$colFormat.replace("{0:", "").replace("}", "");
                            $($id).find(".e-datetimewidget input").ejDateTimePicker({ dateFormat: this._$colFormat.replace("{0:", "").replace("}", "") });
                        }
                        else
                            $($id).find(".e-datetimewidget input").ejDateTimePicker({ dateFormat: "MM/dd/yyyy hh:mm:ss" });
                    }
                        this._setFilterFieldValues($id);
                    }
                    $($id).ejDialog({ position: { X: "", Y: "" } });
                    var docWidth = $(document).width(), dlgWidth =  document.documentElement.clientWidth < 800 ? 130 : 250, xPos = $target.position().left + 18, yPos = $target.position().top + 2;
                    var  marginLeft = e.clientX - xPos; 
                    if (docWidth - e.clientX <= dlgWidth)
                        xPos = docWidth - dlgWidth - marginLeft;
                    if (!this._isExcelFilter)
                        $($id).ejDialog({ position: { X: xPos, Y: yPos }, width: dlgWidth })
                        .ejDialog("open");
                    else
                        this._excelFilter.openXFDialog({ field: columnName, displayName: this.getHeaderTextByFieldName(columnName), dataSource: this._dataSource(), position: { X: xPos, Y: yPos }, type: this._$colType, format: this._$colFormat });
                    if (this._$colType == "number")
                        $($id).find(".e-numerictextbox").ejNumericTextbox({ width: "100%" });
                    this._$prevColType = this._$colType;
                    this._$fDlgIsOpen = true;
                }
            }

            e.preventDefault();

        },
        _clickHandler: function (e) {
           var $target = $(e.target), fieldName, $form = $("#" + this._id + "EditForm"), index, columnIndex, rowIndex;
            if ($target.closest(".e-grid").attr("id") !== this._id) return;
            if ($target.closest("#" + this._id + "EditForm").length)
                return;
            if ($target.hasClass("e-rowcell") || $target.closest("td").is(".e-rowcell") || ($target.hasClass("e-headercell") && ((e.clientY - $target.offset().top) < ($target.height() / 4)))) {
                if (this._bulkEditCellDetails.cancelSave) {
                    this._bulkEditCellDetails.cancelSave = false;
                    return;
                }
                if (this.model.editSettings.editMode == "batch" && ($.isFunction($.validator) && $form.length && $form.validate().errorList.length > 0))
                    return;
                this.model.editSettings.editMode == "batch" && this.element.focus();
                index = $target.closest("tr").hasClass("e-insertedrow") ? this.model.groupSettings.groupedColumns.length : 0;
                columnIndex = $target.hasClass("e-rowcell") ? $target.index() : $target.parent().index() - index;
                rowIndex = this.getIndexByRow($target.closest("tr"));
                this._bulkEditCellDetails.columnIndex = columnIndex;
                this._bulkEditCellDetails.rowIndex = rowIndex;
                if (this.model.allowSelection && ej.gridFeatures.selection) {
                    if (this.model.selectionType == "multiple") {
                        if (e.ctrlKey || this._enableSelectMultiTouch) {
                            this.multiSelectCtrlRequest = true;
                        }
                        if (e.shiftKey) {
                            this._multiSelectShiftRequest = true;
                            if (this._allowcellSelection && rowIndex > -1)
                                this.selectCells([[rowIndex, [columnIndex]]]);
                            if (this._allowrowSelection) 
                                    this.selectRows(this._previousIndex, this.getIndexByRow($target.closest('tr')));
                            if (this._allowcolumnSelection && $target.hasClass("e-headercell") && ((e.clientY - $target.offset().top) < ($target.height() / 4)))
                                this.selectColumns(this._previousColumnIndex, $target.index());
                        }
                        if (e["pointerType"] == "touch" && this._customPop != null && !this._customPop.is(":visible") && this._customPop.find(".e-rowselect").hasClass("e-spanclicked"))
                            this._customPop.show();
                        if (e["pointerType"] == "touch" && this._customPop != null && (this._customPop.find(".e-sortdirect").is(":visible") || !this._customPop.find(".e-rowselect").hasClass("e-spanclicked")) && this.model.selectionType == "multiple") {
                            this._customPop.removeAttr("style");
                            var offset = $target.offset();
                            this._customPop.offset({ top: 0, left: 0 }).offset({ left: offset.left, top: offset.top - this.getRowHeight() }).find(".e-sortdirect").hide().end()
                                .find(".e-rowselect").show().end().show();
                        }
                    }
                    if (!this._multiSelectShiftRequest) {
                        if (this._allowcellSelection && rowIndex > -1) {
                            if (this.model.selectionSettings.enableToggle && this._selectedRowCellIndexes.length == 1 &&
                                this._selectedRowCellIndexes[0].rowIndex == rowIndex && this._selectedRowCellIndexes[0].cellIndex.length == 1 &&
                                parseInt(this._selectedRowCellIndexes[0].cellIndex) == columnIndex)
                                this.clearCellSelection();
                            else
                                this.selectCells([[rowIndex, [columnIndex]]]);
                        }
                        if (this._allowrowSelection && rowIndex > -1) {
                            if (this.model.selectionSettings.enableToggle && this.getSelectedRecords().length == 1 && $.inArray(this.getIndexByRow($target.closest('tr')), this._selectedRowsIndexes) != -1)
                                this.clearSelection(this.getIndexByRow($target.closest('tr')));
                            else
                                this.selectRows(this.getIndexByRow($target.closest('tr')));
                        }
                        if (this._allowcolumnSelection && $target.hasClass("e-headercell") && ((e.clientY - $target.offset().top) < ($target.height() / 4))) {
                            if (this.model.selectionSettings.enableToggle && this._selectedColumnIndexes.length == 1 && $.inArray($target.index(), this._selectedColumnIndexes) != -1)
                                this.clearColumnSelection($target.index());
                            else
                                this.selectColumns($target.index());
                        }
                        this.multiSelectCtrlRequest = false;
                    }
                    this._multiSelectShiftRequest = false;
                }
				
                fieldName = this.model.columns[this._bulkEditCellDetails.columnIndex]["field"];
                if (fieldName)
                    this.model.editSettings.allowEditing && this.model.editSettings.editMode == ej.Grid.EditMode.Batch && this.editCell($.inArray($target.closest("tr").get(0), this.getRows()), fieldName);
            }
            if ($target.hasClass("e-rowselect") || $target.hasClass("e-sortdirect")) {
                if (!$target.hasClass("e-spanclicked")) {
                    $target.addClass("e-spanclicked");
                    if ($target.hasClass("e-rowselect"))
                        this._enableSelectMultiTouch = true;
                    if ($target.hasClass("e-sortdirect"))
                        this._enableSortMultiTouch = true;
                } else {
                    $target.removeClass("e-spanclicked");
                    if ($target.hasClass("e-rowselect"))
                        this._enableSelectMultiTouch = false;
                    if ($target.hasClass("e-sortdirect"))
                        this._enableSortMultiTouch = false;
                    this._customPop.hide();
                }
            }
			if ($target.hasClass('e-ccButton')||$target.parents('button').hasClass("e-ccButton")) {
			    var dlgWidth = 230, xPos;
			    xPos = $("#" + this._id).position().left + $("#" + this._id).width() - dlgWidth;
			    var dialogObj = $("#" + this._id + "ccDiv").data('ejDialog')
			    if (dialogObj && dialogObj.isOpened()) {
			        dialogObj.close();
			        $(".e-columnChoosertail").remove();
			        $(".e-columnChoosertailAlt").remove();
			    }
			    else {
			        $("#" + this._id + "ccDiv").ejDialog({ width: '230px', height: '309px', position: { X: xPos, Y: $target.offset().top + 35 } })
                       .ejDialog("open");
					var maxZindex=parseInt($("#"+this._id+"ccDiv_wrapper").css('z-index'));
			        var $tailDiv = ej.buildTag("div.e-columnChoosertail", '', { 'display': 'block', 'position': 'absolute', 'left': xPos + 200, 'top': $target.offset().top + 15 });
			        var $tailDiv2 = ej.buildTag("div.e-columnChoosertailAlt", '', { 'display': 'block', 'z-index': maxZindex+2, 'position': 'absolute', 'left': xPos + 200, 'top': $target.offset().top + 16 });
			        $tailDiv.insertBefore($("#" + this._id + "ccDiv_wrapper"));
			        $tailDiv2.insertBefore($("#" + this._id + "ccDiv_wrapper"));
			    }
			    $("#" + this._id + "liScrollerDiv").ejScroller({ height: '228', width: '230', buttonSize: 0 })
			    $("#" + this._id + "liScrollerDiv").ejScroller('refresh');
			    $(".e-ejinputtext").bind('keyup', $.proxy(this._columnChooserSearch, this))
           }
            if (ej.gridFeatures.common) {
                this.expandCollapse($target);
            if (this.model.childGrid && $target.hasClass("e-gdiagonalnext") || $target.hasClass("e-gnextforward"))
                e.stopPropagation();
            }
            if (this.model.allowFiltering && (this.model.filterSettings.filterType == "menu" || this._isExcelFilter) && !$target.is(".e-filtericon") && $target.closest(".e-dlgcontainer").length != 1)
                !this._isExcelFilter ? this._closeFilterDlg() : this._excelFilter.closeXFDialog(e);
        },
        /**
         * Destroy the grid widget
		 * all events bound using this._on will be unbind automatically and bring the control to pre-init state.
		 * @alias ejGrid#destroy
		 * @return jQuery
		 * @example 
		 * &lt;script&gt;
		 * var gridObj = $("#Grid").data("ejGrid");
		 * gridObj.destroy(); // destroy the Grid
		 * &lt;/script&gt;
		 * @example 
		 * &lt;script&gt;
		 * // destroy the Grid
		 * $("#Grid").ejGrid("destroy");        
		 * &lt;/script&gt;
         */
        _destroy: function () {
            /// <summary>This function is  used to destroy the Grid Object</summary>
            this.element.find(".e-gridheader").find(".e-headercontent,.e-movableheader")
                .add(this.getContent().find(".e-content,.e-movablecontent")).unbind('scroll');
            var editForm = $("#" + this._id + "EditForm");
            if (editForm.length) {
                var $formEle = editForm.find('.e-field'), $element;
                for (var i = 0; i < $formEle.length; i++) {
                    $element = $($formEle[i]);
                    if ($element.hasClass('e-datetimepicker'))
                        $element.ejDateTimePicker("destroy");
                    else if ($element.hasClass('e-datepicker'))
                        $element.ejDatePicker("destroy");
                    else if ($element.hasClass('e-dropdownlist'))
                        $element.ejDropDownList("destroy");
                }
                editForm.remove();
            }
            if (this._confirmDialog)
                this._confirmDialog.ejDialog("destroy");
            if (this.model.allowFiltering && this.model.filterSettings.filterType == "menu") {
                var proxy = this, $colType;
                $.each(this.model.columns, function (indx, col) {
                    $colType = col.type;
                    $("#" + proxy._id + $colType + "_ddinput_popup_wrapper").remove();
                    if ($colType == "string")
                        $("#" + proxy._id + "_stringDlg").find('.e-autocomplete').ejAutocomplete("destroy");
                    else if ($colType == "date")
                        $("#" + proxy._id + "_dateDlg").find('.e-datepicker').ejDatePicker("destroy");
                    else if ($colType == "datetime")
                        $("#" + proxy._id + "_datetimeDlg").find('.e-datetimepicker').ejDateTimePicker("destroy");
                    else if ($colType == "number")
                        $("#" + proxy._id + "_numberDlg").find('.e-numerictextbox').ejNumericTextbox("destroy");
                });
            }
            this.element.empty().removeClass("e-grid " + this.model.cssClass);
            this.element.ejWaitingPopup("destroy");
        },
        _getLocalizedLabels: function (property) {
            return (ej.Grid.locale[this.model.locale] === undefined || ej.Grid.locale[this.model.locale][property] === undefined) ? ej.Grid.locale["en-US"][property] : ej.Grid.locale[this.model.locale][property];
        },
    });
    if (ej.gridFeatures.common)
        $.extend(ej.Grid.prototype, ej.gridFeatures.common);
    if (ej.gridFeatures.edit)
        $.extend(ej.Grid.prototype, ej.gridFeatures.edit);
    if (ej.gridFeatures.filter)
        $.extend(ej.Grid.prototype, ej.gridFeatures.filter);
    if (ej.gridFeatures.group)
        $.extend(ej.Grid.prototype, ej.gridFeatures.group);
    if (ej.gridFeatures.selection)
        $.extend(ej.Grid.prototype, ej.gridFeatures.selection);
    if (ej.gridFeatures.sort)
        $.extend(ej.Grid.prototype, ej.gridFeatures.sort);
    if (ej.gridFeatures.dragAndDrop)
        $.extend(ej.Grid.prototype, ej.gridFeatures.dragAndDrop);

    ej.Grid.locale = {};

    ej.Grid.locale["en-US"] = {
        EmptyRecord: "No records to display",
        GroupDropArea: "Drag a column header here to group its column",
        DeleteOperationAlert: "No records selected for delete operation",
        EditOperationAlert: "No records selected for edit operation",
        SaveButton: "Save",
        OkButton: "OK",
        CancelButton: "Cancel",
        EditFormTitle: "Details of ",
        AddFormTitle: "Add New Record",
        Notactionkeyalert: "This Key-Combination is not avaiable",
        Keyconfigalerttext: "This Key-Combination has already been assigned to ",
        GroupCaptionFormat: "items",
        BatchSaveConfirm: "Are you sure you want to save changes?",
        BatchSaveLostChanges: "Unsaved changes will be lost. Are you sure you want to continue?",
        PagerInfo: "{0} of {1} pages ({2} items)",
        FrozenColumnsViewAlert: "Frozen columns should be in grid view area",
        FrozenColumnsScrollAlert: "Enable allowScrolling while using frozen Columns",
        FrozenNotSupportedException: "Frozen Columns and Rows are not supported for Grouping, Row Template, Detail Template and Batch Editing",
        Add: "Add",
        Edit: "Edit",
        Delete: "Delete",
        Update: "Update",
        Cancel: "Cancel",
        PrintGrid: "Print",
        ExcelExport: "Excel Export",
        WordExport: "Word Export",
        PdfExport: "PDF Export",
        StringMenuOptions: [{ text: "StartsWith", value: "StartsWith" }, { text: "EndsWith", value: "EndsWith" }, { text: "Contains", value: "Contains" }, { text: "Equal", value: "Equal" }, { text: "NotEqual", value: "NotEqual" }],
        NumberMenuOptions: [{ text: "LessThan", value: "LessThan" }, { text: "GreaterThan", value: "GreaterThan" }, { text: "LessThanOrEqual", value: "LessThanOrEqual" }, { text: "GreaterThanOrEqual", value: "GreaterThanOrEqual" }, { text: "Equal", value: "Equal" }, { text: "NotEqual", value: "NotEqual" }],
        PredicateAnd: "AND",
        PredicateOr: "OR",
        Filter: "Filter",
        MatchCase: "Match Case",
        Clear: "Clear"
    };
    ej.Grid.Actions = {
        /** Used to specify paging action in grid   */
        Paging: "paging",
        /** Used to specify sorting action in grid   */
        Sorting: "sorting",
        /** Used to specify filtering action in grid   */
        Filtering: "filtering",
        /** Used to specify begin edit action in grid   */
        BeginEdit: "beginedit",
        /** Used to specify saving action in grid   */
        Save: "save",
        /** Used to specify adding action in grid   */
        Add: "add",
        /** Used to specify deleting action in grid   */
        Delete: "delete",
        /** Used to specify cancelling action in grid   */
        Cancel: "cancel",
        /** Used to specify grouping action in grid   */
        Grouping: "grouping",
        /** Used to specify un-grouping action in grid   */
        Ungrouping: "ungrouping",
        /** Used to specify refresh action in grid   */
        Refresh: "refresh",
        /** Used to specify reordering action in grid   */
        Reorder: "reorder",
        /** Used to specify searching action in grid   */
        Search: "searching",
        /** Used to specify batch save action in grid   */
        BatchSave: "batchsave",
        /** Used to specify virtual scroll action in grid   */
        VirtualScroll: "virtualscroll"
    };
    /**
     * Enum for various grid summary type	 
     * @enum {string}
     * @global 
     */
    ej.Grid.SummaryType = {
        /**  Creates grid with summary type as Average */
        Average: "average",
        /**  Creates grid with summary type as Minimum */
        Minimum: "minimum",
        /**  Creates grid with summary type as Maximum */
        Maximum: "maximum",
        /**  Creates grid with summary type as Count */
        Count: "count",
        /**  Creates grid with summary type as Sum */
        Sum: "sum",
        /**  Creates grid with summary type as TrueCount */
        TrueCount: "truecount",
        /**  Creates grid with summary type as FalseCount */
        FalseCount: "falsecount",
        /**  Creates grid with summary type as Custom */
        Custom: "custom"
    };
    /**
	 * Enum for specifying edit mode in grid 
	 * @enum {string}
	 * @global 
	 */
    ej.Grid.EditMode = {
        /**  Creates grid with editMode as Normal */
        Normal: "normal",
        /**  Creates grid with editMode as Dialog */
        Dialog: "dialog",
        /**  Creates grid with editMode as DialogTemplate */
        DialogTemplate: "dialogtemplate",
        /**  Creates grid with editMode as Batch */
        Batch: "batch",
        /**  Creates grid with editMode as ExternalForm */
        ExternalForm: "externalform",
        /**  Creates grid with editMode as ExternalFormTemplate */
        ExternalFormTemplate: "externalformtemplate",
        /**  Creates grid with editMode as InlineForm */
        InlineForm: "inlineform",
        /**  Creates grid with editMode as InlineTemplateForm */
        InlineTemplateForm: "inlineformtemplate"
    };

    /**
      * Enum for specifying row position in grid 
      * @enum {string}
      * @global 
      */
    ej.Grid.Rowposition = {
        /** Add new row in the top of the grid */
        Top: "top",
        /** Add new row in the bottom of the grid */
        Bottom: "bottom",
    };
    /**
	 * Enum for specifying form position in grid 
	 * @enum {string}
	 * @global 
	 */
    ej.Grid.FormPosition = {
        /**  Creates grid with formPosition as BottomLeft */
        BottomLeft: "bottomleft",
        /**  Creates grid with formPosition as TopRight */
        TopRight: "topright"
    };
    /**
	 * Enum for specifying various editing type in grid	 
	 * @enum {string}
	 * @global 
	 */
    ej.Grid.EditingType = {
        /**  Allows to set edit type as string edit type */
        String: "stringedit",
        /**  Allows to set edit type as boolean edit type */
        Boolean: "booleanedit",
        /**  Allows to set edit type as numeric edit type */
        Numeric: "numericedit",
        /**  Allows to set edit type as drop down edit type */
        Dropdown: "dropdownedit",
        /**  Allows to set edit type as date picker edit type */
        DatePicker: "datepicker",
        /**  Allows to set edit type as date time picker edit type */
        DateTimePicker: "datetimepicker",
    };
    /**
	 * Enum for specifying various unbound type in grid	 
	 * @enum {string}
	 * @global 
	 */
    ej.Grid.UnboundType = {
        /** Used to specify unbound type as Edit   */
        Edit: "edit",
        /** Used to specify unbound type as Save   */
        Save: "save",
        /** Used to specify unbound type as Delete   */
        Delete: "delete",
        /** Used to specify unbound type as Cancel   */
        Cancel: "cancel"
    };
    /**
	 * Enum for specifying various tool bar items in grid	 
	 * @enum {string}
	 * @global 
	 */
    ej.Grid.ToolBarItems = {
        /** Used to add toolbar item for adding records    */
        Add: "add",
        /** Used to add toolbar item for editing records    */
        Edit: "edit",
        /** Used to add toolbar item for deleting records    */
        Delete: "delete",
        /** Used to add toolbar item for updating records    */
        Update: "update",
        /** Used to add toolbar item for cancelling records    */
        Cancel: "cancel",
        /** Used to add toolbar item for searching records    */
        Search: "search",
        /** Used to add toolbar item for printing grid    */
        PrintGrid: "printGrid",
         /** Used to add toolbar item for exproting grid to excel    */
        ExcelExport: "excelExport",
        /** Used to add toolbar item for exporting grid to word    */
        WordExport: "wordExport",
        /** Used to add toolbar item for exporting grid to pdf    */
        PdfExport: "pdfExport"
    };
    /**
     * Enum for various grid filtering type	 
     * @enum {string}
     * @global 
     */
    ej.Grid.FilterType = {
        /**  Creates grid with filtering type as Menu */
        Menu: "menu",
        /**  Creates grid with filtering type as FilterBar */
        FilterBar: "filterbar",
        /** Creates grid with filtering type as Excel */
        Excel: "excel"
    };

    /**
	 * Enum for specifying various filter bar mode in grid	 
	 * @enum {string}
	 * @global 
	 */
    ej.Grid.FilterBarMode = {
        /** Used to set filter bar mode as Immediate mode */
        Immediate: "immediate",
        /** Used to set filter bar mode as OnEnter mode */
        OnEnter: "onenter"
    };
    /**
     * Enum for various selection type in grid 
     * @enum {string}
     * @global 
     */
    ej.Grid.SelectionType = {
        /**  Support for Single selection only in grid */
        Single: "single",
        /**  Support for multiple selections in grid */
        Multiple: "multiple"
    };
     /**
     * Enum for various virtual scroll mode in grid 
     * @enum {string}
     * @global 
     */
    ej.Grid.VirtualScrollMode = {
        /** Used to set the Normal mode virtual paging*/
        Normal: "normal",
        /** Used to set the Continuous mode virtual paging*/
        Continuous: "continuous"
    };
    /**
    * Enum for various selection mode in grid 
    * @enum {string}
    * @global 
    */
    ej.Grid.SelectionMode = {
	    /**  Support for Row selection in grid */
        Row: "row",
        /**  Support for Cell selection in grid */
        Cell: "cell",       
        /**  Support for Column selection in grid */
        Column: "column"
    };
})(jQuery, Syncfusion);;
(function ($, ej, undefined) {
    ej.Grid = ej.Grid || {};
     ej.Grid.gridResize = function (instance) {
        this.$headerTable = instance.getHeaderTable();
        this.gridInstance = instance;
        this._colMinWidth = 15;
        this._$visualElement = $();
        this._currentCell = -1;
        this._allowStart = false;
        this._oldWidth = null;
        this._orgX = null;
        this._orgY = null;
        this._extra = null;
        this._expand = false;
        this._target = null;
    }

    ej.Grid.gridResize.prototype = {
        _mouseHover: function (e) {
            if (this._$visualElement.is(":visible"))
                return;
            if ($(e.target).is(".e-headercelldiv"))
                e.target = e.target.parentNode;
            var $target = $(e.target);
            if ($target.hasClass("e-headercell")) {
                var _resizableCell = e.target;
                var location = _resizableCell.getBoundingClientRect(), _x = 0, _y = 0;
                if (e.type = "mousemove") {
                    _x = e.clientX;
                    _y = e.clientY;
                }
                else if (e.type = "touchmove") {
                    _x = evt.originalEvent.changedTouches[0].clientX;
                    _y = evt.originalEvent.changedTouches[0].clientY;
                }
                else if (e.type = "MSPointerMove") {
                    _x = e.originalEvent.clientX;
                    _y = e.originalEvent.clientY;
                }
                if (this.gridInstance.model.scrollSettings&&this.gridInstance.model.scrollSettings.frozenColumns)
                    var _nlx = this.gridInstance.getHeaderContent().width() + this.gridInstance.element.children(".e-gridheader").find(".e-columnheader").offset().left;
                else
                    var _nlx = this.gridInstance.getHeaderTable().width() + this.gridInstance.element.children(".e-gridheader").find(".e-columnheader").offset().left;
                if (((_x >= (location.left + document.documentElement.scrollLeft + _resizableCell.offsetWidth - 5)) || ((_x <= (location.left + 3)))) && (_x < _nlx) && (_x >= location.left) && (_y <= location.top + document.documentElement.scrollTop + e.target.offsetHeight)) {
                    $target.parent().css("cursor", "col-resize");
                    this._currentCell = this.gridInstance.getHeaderContent().find(".e-headercell").index(_resizableCell);
                    this._allowStart = true;
                }
                else {
                    $target.parent().css("cursor", "pointer");
                    this._allowStart = false;
                    this._currentCell = -1;
                }
            }
        },
        _start: function (_x, _y) {
            var _myrow = this.gridInstance.getHeaderTable().find(".e-columnheader");
            var _cells = _myrow.find(".e-headercell"), _mycel;
            if (this._currentCell != -1 && this._currentCell < _cells.length)
                _mycel = _cells[this._currentCell];
            if (typeof (_mycel) == 'undefined')
                return;
            var _j = _mycel.getBoundingClientRect();
            this._tableY = _j.top + parseInt(navigator.userAgent.indexOf("WebKit") != -1 ? document.body.scrollTop : document.documentElement.scrollTop);
            if (this._allowStart) {
                this._$visualElement = $(document.createElement('div'));
                _height = this.gridInstance.element.find(".e-gridcontent").first().height() + this.gridInstance.element.find(".e-gridheader").height();
                this._$visualElement.addClass("e-reSizeColbg").appendTo(this.gridInstance.element).css({ height: _height + 'px', cursor: 'col-resize' });
                this._$visualElement.css({ left: _x, top: this._tableY });
                this._oldWidth = _mycel.offsetWidth;
                this._orgX = _x;
                this._orgY = _y;
                this._extra = _x - this._orgX;
                this._expand = true;
            }
            else {
                this._currentCell = -1;
            }
        },
        _mouseMove: function (e) {
            if (this._expand) {
                var _x = 0, _y = 0;
                if (e.type = "mousemove") {
                    _x = e.clientX;
                    _y = e.clientY;
                }
                else if (e.type = "touchmove") {
                    _x = evt.originalEvent.changedTouches[0].clientX;
                    _y = evt.originalEvent.changedTouches[0].clientY;
                }
                else if (e.type = "MSPointerMove") {
                    _x = e.originalEvent.clientX;
                    _y = e.originalEvent.clientY;
                }
                if (navigator.userAgent.indexOf("WebKit") != -1) {
                    _x = e.pageX;
                    _y = e.pageY;
                }
                _x += document.documentElement.scrollLeft;
                e.preventDefault();
                this._moveVisual(_x);
            }
            else
                this._mouseHover(e);
        },
        _getCellIndex: function (e) {
            var $target = $(e._target);
            var targetCell = e._target;
            var hCellIndex = targetCell.cellIndex;
            var cellIndex = hCellIndex;
            if (e.gridInstance.model.groupSettings.groupedColumns.length) {
                cellIndex = hCellIndex - e.gridInstance.model.groupSettings.groupedColumns.length;
            }
            return cellIndex;
        },
        _reSize: function (_x, _y) {
            // Function used for Resizing the column
            var proxy = this;
            var resized = false, $content;
            if (this.gridInstance.model.scrollSettings&&this.gridInstance.model.scrollSettings.frozenColumns)
                this._initialTableWidth = this.gridInstance.getHeaderTable().first().parent().width() + this.gridInstance.getHeaderTable().last().parent().width();
            else
                this._initialTableWidth = this.gridInstance.getHeaderTable().parent().width();
            !this.gridInstance.model.enableRTL && this._getResizableCell();
            var _rowobj = this.gridInstance.getHeaderTable().find(".e-columnheader");
            if (this._currentCell != -1 && this._expand) {
                this._expand = false;
                var _childTH = _rowobj.find(".e-headercell").filter(":visible");
                var _outerCell = _childTH[this._currentCell];
                var _oldWidth = _outerCell.offsetWidth;
                var _extra = _x - this._orgX;
                //Check whether the column minimum width reached
                if (parseInt(_extra) + parseInt(_oldWidth) > this._colMinWidth) {
                    if (_extra != 0)
                        _rowobj.css("cursor", 'default');
                    this._resizeColumnUsingDiff(_oldWidth, _extra);
                    $content = this.gridInstance.element.find(".e-gridcontent").first();
                    this.gridInstance.model.allowScrolling && $content.ejScroller("refresh");
                    var browser = this.gridInstance.getBrowserDetails();
                    if (browser.browser == "msie" && this.gridInstance.model.allowScrolling) {
                        var oldWidth = this.gridInstance.getContentTable().width(), newwidth = this.gridInstance._calculateWidth();
                        if (this.gridInstance.model.scrollSettings&&this.gridInstance.model.scrollSettings.frozenColumns > 0) {
                            this.gridInstance.getHeaderTable().last().width(newwidth - this.gridInstance.getHeaderContent().find(".e-frozenheaderdiv").width());
                            this.gridInstance.getContentTable().last().width(newwidth - this.gridInstance.getContent().find(".e-frozencontentdiv").width());
                            this.gridInstance.model.showSummary && this.gridInstance.getFooterTable().last().width(newwidth - this.gridInstance.getFooterContent().find(".e-frozenfootertdiv").width());
                        }
                        else {
                            this.gridInstance.getHeaderTable().width(newwidth);
                            this.gridInstance.model.showSummary && this.gridInstance.getFooterTable().width(newwidth);
                        }
                        if (parseInt(browser.version, 10) > 8 && this.gridInstance.model.groupSettings && this.gridInstance.model.groupSettings.groupedColumns.length) {
                            if (newwidth > oldWidth) {
                                this.gridInstance.getContentTable().width(newwidth);
                                this.gridInstance.getContentTable().children("colgroup").find("col").first().css("width", (20 / $content.find("table").first().width()) * 100 + "%");
                            }
                            else {
                                this.gridInstance.getContentTable().css("width", "100%");
                                this.gridInstance._groupingAction();
                                this.gridInstance.getContentTable().children("colgroup").find("col").first().css("width", ((this.gridInstance.getHeaderTable().find("colgroup").find("col").first().width() / $content.find("table").first().width()) * 100).toFixed(2) + "%");
                            }
                        }
                        this.gridInstance.getHeaderTable().parent().scrollLeft($content.find(".e-content").scrollLeft() - 1);
                    }					
                    this.gridInstance._colgroupRefresh();
                    if (this.gridInstance.model.groupSettings.groupedColumns.length && this.gridInstance.model.groupSettings.showGroupedColumn)
                        this.gridInstance._recalculateIndentWidth();
                    if (!(browser.browser == "msie") && this.gridInstance.model.allowScrolling && this.gridInstance.model.scrollSettings.frozenColumns == 0) {
                        this.gridInstance.getHeaderTable().width(this.gridInstance._calculateWidth());
                        this.gridInstance.getContentTable().width(this.gridInstance._calculateWidth());
                    }
                    if (this.gridInstance.model.allowResizing && this.gridInstance.getHeaderTable().find(".e-columnheader").css('cursor') == 'default') {
                        var cellIndex = this._getCellIndex(this);
                        var newWidth = _oldWidth + _extra;
                        var args = { columnIndex: cellIndex, column: this.gridInstance.model.columns[cellIndex], oldWidth: _oldWidth, newWidth: newWidth };
                        this.gridInstance._trigger("resized", args);
                    }
                }

            }
            this._target = null;
            this._$visualElement.remove();
            this._expand = false;
            this._currentCell = -1;
            this._allowStart = false;

        },
        _getFrozenResizeWidth: function () {
            var $frozenColumnsCol = this.gridInstance.getHeaderTable().find('colgroup').find("col").slice(0, this.gridInstance.model.scrollSettings?this.gridInstance.model.scrollSettings.frozenColumn:0), width = 0;
            for (var i = 0; i < $frozenColumnsCol.length; i++) {
                if ($frozenColumnsCol.eq(i).css("display") != "none")
                    width += parseInt($frozenColumnsCol[i].style.width.replace("px", ""));
            }
            return width;
        },
        _diaplayFinder: function () {
            return $(this).css('display') != 'none';
        },
        _resizeColumnUsingDiff: function (_oldWidth, _extra) {
            var proxy = this;
            var $headerCols = this.gridInstance.getHeaderTable().find('colgroup').find("col"), $headerCol = $headerCols.filter(this._diaplayFinder).eq(!this.gridInstance.model.allowGrouping ? this._currentCell : this._currentCell + this.gridInstance.model.groupSettings.groupedColumns.length)
                    , $ContentCol, $footerCol, $frozenCols = $headerCols.slice(0, this.gridInstance.model.scrollSettings ? this.gridInstance.model.scrollSettings.frozenColumns : 0);
            var _inlineWidth = $headerCol[0].style.width;
            var indent = _oldWidth / parseInt(_inlineWidth);
            _extra = _extra / indent
            var _newWidth = parseInt(_extra) + parseInt(_inlineWidth);
            if (_newWidth > 0 && _extra != 0) {
                if (_newWidth < this._colMinWidth)
                    _newWidth = this._colMinWidth;
                var _extra = _newWidth - _oldWidth;
                if (this.gridInstance.model.scrollSettings&&this.gridInstance.model.scrollSettings.frozenColumns) {
                    if (this._currentCell >= 0 && this._currentCell < this.gridInstance.model.scrollSettings.frozenColumns && this._getFrozenResizeWidth() + _extra > this.gridInstance.element.find(".e-headercontent").first().width())
                        return;
                    $ContentCol = this.gridInstance.getContentTable().find('colgroup').find("col").filter(this._diaplayFinder).eq(this._currentCell);
                }
                else
                    $ContentCol = this.gridInstance.getContentTable().find('colgroup').first().find("col").filter(this._diaplayFinder).eq(!this.gridInstance.model.allowGrouping ? this._currentCell : this._currentCell + this.gridInstance.model.groupSettings.groupedColumns.length);
                if (this.gridInstance.model.showSummary) {
                    $footerCol = this.gridInstance.getFooterTable().find('colgroup').find("col").filter(this._diaplayFinder).eq(this._currentCell);
                    $footerCol.outerWidth(_newWidth);
                }
                $headerCol.outerWidth(_newWidth);
                if (this.gridInstance.model.isEdit) {
                    var $editCol = this.gridInstance.getContentTable().find(".e-editedrow,.e-addedrow").find("table").find("colgroup");
                    $($editCol[0].children[this._currentCell - this.gridInstance.model.groupSettings ? this.gridInstance.model.groupSettings.groupedColumns.length : 0]).width(_newWidth);
                }
                if (this.gridInstance.model.groupSettings && this.gridInstance.model.groupSettings.groupedColumns.length) {
                    var $tables = this.gridInstance.getContentTable().find(".e-recordtable");
                    var $colGroup = $tables.find("colgroup");
                    var cellIndex = this._currentCell + this.gridInstance._hiddenColumns.length;
                    for (var i = 0 ; i < $colGroup.length; i++)
                        $($colGroup[i].children[cellIndex]).width(_newWidth);
                }
                $ContentCol.width(_newWidth);
                this.gridInstance._findColumnsWidth();
                if (this.gridInstance.model.scrollSettings&&this.gridInstance.model.scrollSettings.frozenColumns) {
                    var frozenColumns = this.gridInstance.getContentTable().find('colgroup').find("col").slice(0, this.gridInstance.model.scrollSettings.frozenColumns)
                        , width = 0, direction;
                    for (i = 0; i < frozenColumns.length; i++)
                        width += parseInt(frozenColumns[i].style.width.replace("px", ""));
                    this.gridInstance.getHeaderContent().find(".e-frozenheaderdiv").width(width);
                    direction = this.gridInstance.model.enableRTL ? "margin-right" : "margin-left";
                    this.gridInstance.getContent().find(".e-frozencontentdiv").width(width).next().css(direction, width + "px");
                    this.gridInstance.getHeaderContent().find(".e-frozenheaderdiv").width(width).next().css(direction, width + "px");
                    this.gridInstance.model.showSummary && this.gridInstance.getFooterContent().find(".e-frozenfooterdiv").width(width);
                }
                this.gridInstance.getHeaderTable().find(".e-columnheader").css("cursor", "default");
            }
        },
        _triggerResizeEvents: function (event,_x) {
            var _rowobj = this.gridInstance.getHeaderTable().find(".e-columnheader");
            var _childTH = _rowobj.find(".e-headercell").filter(":visible");
            var _outerCell = _childTH[this._currentCell];
            var _oldWidth = _outerCell.offsetWidth;            
            var cellIndex = this._getCellIndex(this);
            if (event == "resizeStart") {
                var args = { columnIndex: cellIndex, column: this.gridInstance.model.columns[cellIndex], target: $(_outerCell), oldWidth: _oldWidth };
                this.gridInstance._trigger("resizeStart", args);
            }
            else {
                var _extra = _x - this._orgX;
                var newWidth = _oldWidth + _extra;
                this.gridInstance._colgroupRefresh();
                var args = { columnIndex: cellIndex, column: this.gridInstance.model.columns[cellIndex], target: $(_outerCell), oldWidth: _oldWidth, newWidth: newWidth, extra: _extra };
                this.gridInstance._trigger("resizeEnd", args);
            }
        },
        _mouseUp: function (e) {
            if (this._expand) {
                var _x = e.clientX, _y = e.clientY;
                if (navigator.userAgent.indexOf("WebKit") != -1) {
                    _x = e.pageX;
                    _y = e.pageY;
                }
                if (this.gridInstance.model.allowResizing && this.gridInstance.getHeaderTable().find(".e-columnheader").css('cursor') == 'col-resize') {
                    this._triggerResizeEvents("resizeEnd",_x);
                }
                _x += document.documentElement.scrollLeft;
                this._reSize(_x, _y);
            }
        },
        _getResizableCell: function () {
            var row = this.gridInstance.getHeaderTable().find(".e-columnheader");
            var cell = row.find(".e-headercell").not(".e-hide");
            var scrollLeft = navigator.userAgent.indexOf("WebKit") != -1 ? document.body.scrollLeft : document.documentElement.scrollLeft;
            for (var i = 0; i < cell.length; i++) {
                point = cell[i].getBoundingClientRect();
                var xlimit = point.left + scrollLeft + 5;
                if (xlimit > this._orgX) {
                    this._currentCell = i - 1;
                    return;
                }
            }
        },
        _moveVisual: function (_x) {
            /// Used to move the visual element in mouse move
            var _bounds = this.gridInstance.getHeaderContent().find("div").first()[0].getBoundingClientRect();
            if ((_bounds.left + document.documentElement.scrollLeft + _bounds.width < _x) || (_x < _bounds.left + document.documentElement.scrollLeft))
                this._$visualElement.remove();
            else if (this._currentCell != -1)
                this._$visualElement.css({ left: _x, top: this._tableY });
        },
        _mouseDown: function (e) {
            if (this._allowStart && ($(e.target).closest("tr").css("cursor") == 'col-resize')) {
                var _x = e.clientX, _y = e.clientY;
                if (navigator.userAgent.indexOf("WebKit") != -1) {
                    _x = e.pageX;
                    _y = e.pageY;
                }
                if (this.gridInstance.model.allowResizing && this.gridInstance.getHeaderTable().find(".e-columnheader").css('cursor') == 'col-resize') {
                    if ($(e.target).is(".e-headercelldiv"))
                        e.target = e.target.parentNode;
                    this._target = e.target;
                    this._triggerResizeEvents("resizeStart",_x);
                }
                 var gridobj = this;
                _x += document.documentElement.scrollLeft;
                if (e.button != 2)
                if (this.gridInstance.model.allowResizing && this.gridInstance.model.allowResizeToFit) {
                        gridobj.gridInstance._click++;
                             setTimeout(function () {
                                  if (gridobj.gridInstance._click == 1) {
                                       gridobj._start(_x, _y);
                                  }
                                  else if (gridobj.gridInstance._click == 2) {
                                       gridobj._columnResizeToFit(e);
                                  }
                                 gridobj.gridInstance._click = 0;
                           }, 400);
                }
                else
                    this._start(_x, _y);
                e.preventDefault();
            }
            return false;
        },
        _columnResizeToFit: function (e) {
            if (this.gridInstance.model.allowResizeToFit && this.gridInstance.getHeaderTable().find(".e-columnheader").css('cursor') == 'col-resize') {
                if ($(e.target).is(".e-headercelldiv"))
                    e.target = e.target.parentNode;
                var $target = $(e.target);
                if ($target.hasClass("e-headercell")) {
                    var targetCell = e.target;
                    var hCellIndex = targetCell.cellIndex;
                    var cellIndex = hCellIndex;
                    this._orgX = e.pageX;
                    this._getResizableCell();
                    if (hCellIndex != this._currentCell) {
                        hCellIndex = cellIndex = this._currentCell;
                        targetCell = e.target.previousSibling;
                    }
                    var $cellDiv = $(targetCell).children('.e-headercelldiv');
                    if (this.gridInstance.model.groupSettings.groupedColumns.length) {
                        cellIndex = hCellIndex - this.gridInstance.model.groupSettings.groupedColumns.length;
                    }
                    var finalWidth = 0, headerWidth = 0, contentWidth = 0;
                    contentWidth = this._getContentWidth(cellIndex);
                    headerWidth = this._getHeaderContentWidth($cellDiv);
                    finalWidth = headerWidth > contentWidth ? headerWidth : contentWidth;
                    finalWidth = finalWidth + (finalWidth * 0.2);
                    var oldWidth = this.gridInstance.getHeaderTable().find('col').eq(hCellIndex).width();
                    var args = { columnIndex: cellIndex, column: this.gridInstance.model.columns[cellIndex], target: $target, oldWidth: oldWidth };
                    this.gridInstance._trigger("resizeStart", args);
                    this.gridInstance.getHeaderTable().find('col').eq(hCellIndex).width(finalWidth);
                    if (this.gridInstance.model.groupSettings.groupedColumns.length) {
                        var $colGroups = this.gridInstance.getContentTable().find('.e-recordtable').find('colgroup');
                        $.each($colGroups, function (indx, colgroup) {
                            $(colgroup).find('col').eq(cellIndex).width(finalWidth);
                        });
                    }
                    else
                        this.gridInstance.getContentTable().find('col').eq(cellIndex).width(finalWidth);
                    var newWidth = this.gridInstance.getHeaderTable().find('col').eq(hCellIndex).width();
                    this.gridInstance._colgroupRefresh();
                    args = { columnIndex: cellIndex, column: this.gridInstance.model.columns[cellIndex], target: $target, oldWidth: oldWidth, newWidth: newWidth, extra: Math.abs(newWidth - oldWidth) };
                    this.gridInstance._trigger("resizeEnd", args);
                    this.gridInstance._columnsWidthCollection[cellIndex] = finalWidth;
                    this.gridInstance.model.columns[cellIndex]["width"] = finalWidth;
                    args = { columnIndex: cellIndex, column: this.gridInstance.model.columns[cellIndex], target: $target, oldWidth: oldWidth, newWidth: newWidth };
                    this.gridInstance._trigger("resized", args);
                }
            }
        },
        _getContentWidth: function (cellindx) {
            var contentWidth = 0;
            var $span = ej.buildTag('span', {}, {}), proxy = this.gridInstance, tdWidth;
            $.each(proxy._gridRows, function (indx, row) {
                var td = $(row).find('td.e-rowcell').eq(cellindx);
                var content = $(td).html();
                if (proxy.model.columns[cellindx]["isUnbound"])
                    $span.html($(content).children());
                else
                    $span.html(content);
                $(td).html($span);
                tdWidth = td.find('span:first').width();
                if (tdWidth > contentWidth)
                    contentWidth = tdWidth;
                $(td).html(content);
            });
            proxy._refreshUnboundTemplate(this.gridInstance.getContentTable());
            return contentWidth;
        },
        _getHeaderContentWidth: function ($cellDiv) {
            var headerWidth = 0, $span = ej.buildTag('span', {}, {});
            var content = $cellDiv.html();
            $span.html(content);
            $cellDiv.html($span);
            headerWidth = $cellDiv.find('span:first').width();
            $cellDiv.html(content);
            return headerWidth;
        },
    };
})(jQuery, Syncfusion);;
(function ($, ej, undefined) {
	ej.scheduleFeatures = ej.scheduleFeatures || {};

	var horizontalHeaderTemplate = function () {
	    this._horiDateHeader = ("{{if multiRes == true}}<td class = 'e-horizontalheader'><div class='e-horizontalresheader'>RESOURCES</div></td>{{else}}<td></td>{{/if}}" +
                                "<td class='e-horizontaldaterender' style='height:100%;'><div class='e-horizontaltimecells' style='height:{{:height}}px;'>" +
                                "<div class='e-headerdays' style='position:absolute;'>" +
                                "<table class ='e-table' cellpadding='0px' cellspacing='0px' style='width:100%;border-collapse:separate;'><tbody>{{if view !== 'month'}}" +
                                "<tr style='width:100%;'> <td>{{if view === 'day'}}<table style='width:100%;'>{{else}}<table cellspacing='0px'>{{/if}}<tbody><tr> {{for cols ~headercellWidth = headercellWidth}}<td class='e-headercells {{:cellToday}} e-horizontalheadertddate'><div class='{{:currentDateClass}} e-horizontalheaderdiv'>{{:currentDay}}</div></td>{{/for}} </tr></tbody></table></td> </tr>" +
                                "<tr style='width:100%;position:absolute;'> <td><table class ='e-horizontaltabletimecell' cellspacing='0px' style='border-collapse:collapse;'><tbody><tr> {{for timeTdCount }}{{for ~name=time ~dayend=timecellclass}}<td class='e-horizontaltimetd {{:~dayend}}'><div style='width:{{:cellWidth}}'><span class ='e-horizontaltimespan'>{{:~name.slice(0,2)}}</span><div class ='e-horizontaltimespandiv'>{{:~name.slice(3,5)}}</div></div></td><td class='e-horizontalalternatetd'><div style='width:{{:cellWidth}}'></div></td>{{/for}}{{/for}}  </tr></tbody></table></td> </tr>" +
                                "{{/if}}" +
                                "{{if view == 'month'}}<tr>{{for cols}}<td class='e-headercells e-horizontmonthheaderdatetd {{:cellToday}}'><div class='e-dateheadercell e-horizontmonthdatealign'>{{:currentDate}}</div></td>{{/for}}</tr>" +
                                "<tr>{{for cols}}<td class ='e-horizontmonthheaderdaytd'><div class='e-dateheadercell e-horizontmonthdatealign' style='width:{{:cellWidth}}'>{{:currentDay}}</div></td>{{/for}}</tr>" +
                                "{{/if}}</tbody></table></div></div></td>");
	    return this._horiDateHeader;
	};

	var horizontalResourceHeaderTemplate = function () {
	    this._horizontalResHeader = ("<div class ='e-horizontresdiv'><table class='e-table e-resourceheadertable' height='100%' width='100%' cellpadding='0px' cellspacing='0px'>" +
            "<tbody>{{for trs}}<tr>{{for ~resname=name ~marginleft=marginleft }}{{if classname == 'e-parentnode'}}<td class='{{:classname}}' style='width:25px;'></td>{{else}}<td style='width:25px;'></td>{{/if}}<td class='{{:classname}}' style='vertical-align:middle;'>" +
            "<div style='text-align:left;margin-left:{{:~marginleft}};'>{{if userResTemplId == true}}{{:userResHeader}}{{else}}{{:~resname}}{{/if}}</div>" +
            "</td>{{if classname == 'e-parentnode'}}<td id={{:idnum}} class='{{:classname}} {{:classname}}category e-resourceicon e-resourcecollapse' style='width:25px;vertical-align:middle'></td>{{else}}<td class='{{:classname}} {{:classname}}category' style='width:25px;'></td>{{/if}}{{/for}}</tr>{{/for}}<tr class='e-resfootertr' style='height:18px;'><td></td></tr></tbody></table></div>");
	    return this._horizontalResHeader;
	};



	var horizontalCellsTemplate = function () {
	    this._cellsTemplate = ("<table class='e-table e-workcellstab' height='100%' cellpadding='0px' cellspacing='0px'>" +
        "<tbody>{{for cellrows ~cols=cols ~view=view ~classname=classname ~tdWidth=tdWidth}}" +
        "{{if classname == 'e-parentnode'}} <tr style='width:100%'>{{for ~cols ~tdWidth=tdWidth}}{{if #index % 24 == 0 && #index != 0 && ~view !== 'month' }}<td class='e-resourceheadercells e-workcells e-parentworkcell e-dayend' style='width:{{:cellWidth}}'>{{else}}<td class='e-resourceheadercells e-workcells e-parentworkcell' style='width:{{:cellWidth}}'>{{/if}}<div class='e-appointwrapper'></div></td>{{if ~view !== 'month'}}<td class='e-resourceheadercells e-workcells e-parentworkcell e-alternateworkcell' style='width:{{:cellWidth}}'><div class='e-appointwrapper'></div></td>{{/if}}{{/for}}</tr>" +
        "{{else}}<tr style='width:100%'>{{for ~cols ~tdWidth=tdWidth}}{{if #index % 24 == 0 && #index != 0 && ~view !== 'month' }}<td class='e-workcells e-childworkcell e-dayend' style='width:{{:cellWidth}}'>{{else}}<td class='e-workcells e-childworkcell' style='width:{{:cellWidth}}'>{{/if}}<div class='e-appointwrapper'></div></td>{{if ~view !== 'month'}}<td class='e-workcells e-childworkcell e-alternateworkcell' style='width:{{:cellWidth}}'><div class='e-appointwrapper'></div></td>{{/if}}{{/for}}" +
        "</tr>{{/if}}" +
        "{{/for}}</tbody></table>");
	    return this._cellsTemplate;
	};

	ej.scheduleFeatures.horizontal = {

	    _renderHorizontalTemplates: function () {
	        this.horizontalHeader = $.templates(this._createTemplate(horizontalHeaderTemplate(), "_HorizontalHeader"));
	        this.cellTemplate = $.templates(this._createTemplate(horizontalCellsTemplate(), "_HorizontalCells"));
	        this.horizontalResHeader = $.templates(this._createTemplate(horizontalResourceHeaderTemplate(), "_HorizontalResources"));
	    },

	    _horizontalViewNavigation: function (e) {
            if (e.type == "click") {
	            if (this.currentView() !== "day" && this.currentView() !== "month") {
	                this.element.find(".e-headercells").children().removeClass("e-activeview");
	                $(e.currentTarget).children().addClass("e-activeview");
	                var currentDateIndex = e.currentTarget.cellIndex;
	                this.currentDate(this._getRenderDates()[currentDateIndex]);
	                var cellOffsetLeft = this.element.find(".e-workcells")[currentDateIndex * 48].offsetLeft;
	                $(this.element.find(".e-horizontaltimecells").find("table tr")[2]).css("left", (-cellOffsetLeft) + "px");
	                $('div#' + this._id + "_scroller").ejScroller({ scrollLeft: cellOffsetLeft });
	            }
	            else if (this.currentView() == "month") {
	                var currentDateIndex = e.currentTarget.cellIndex;
	                if (this.model.views.indexOf("Day") == -1) return false;
	                this.currentDate(this.monthDays[currentDateIndex]);
	                newView = "day";
	                this._navView = true;
	                this._viewChange(e,this.currentView(), newView);
	            }
	        }
	        else if (e.type == "dblclick") {
	            if (this.currentView() === "week" || this.currentView() === "workweek") {
				var _target = $(e.target);
	                var dayTarget = _target.hasClass("e-headercells") ? _target : _target.parent().hasClass("e-headercells") ? _target.parent() : _target.parent().parent();
	                if (this.model.views.indexOf("Day") == -1 || !dayTarget.hasClass("e-headercells")) return false;
	                this.currentDate(this._getRenderDates()[dayTarget.index()]);
	                newView = "day";
	                this._navView = true;
	                this._viewChange(e, this.currentView(), newView);
	            }
	        }

	    },

	    _renderHorizontalHeaderDays: function (localizeDays) {
	        if (this.currentView() !== "month") {
	            var curday, temp;
	            for (var i = 0; i < this._dateRender.length; i++) {
	                curday = (this.currentView() === "day") ? (ej.isNullOrUndefined(this.model.resources) && ej.isNullOrUndefined(this.model.group)) ? localizeDays.calendar.days.names[new Date(this._dateRender[i]).getDay()] : localizeDays.calendar.days.names[new Date(this._dateRender[i]).getDay()].slice(0, 3) : localizeDays.calendar.days.names[new Date(this._dateRender[i]).getDay()].slice(0, 3);
	                temp = this.currentView() === "day" ? (ej.isNullOrUndefined(this.model.resources) && ej.isNullOrUndefined(this.model.group)) ? curday + " " + Globalize.format(new Date(this._dateRender[i]), "dd MMMM yyyy") : curday + " " + Globalize.format(new Date(this._dateRender[i]), "dd MMM yy") : curday + " " + new Date(this._dateRender[i]).getDate();
	                $(this.element.find(".e-headercells")[i]).find("div")[0].innerHTML = temp;
	            }
	        }
	    },

	    _horizontalScroll: function (e, proxy) {
            if (proxy.currentView() !== "day" && proxy.currentView() !== "month") {
	            var currentDateIndex = $(proxy.element.find(".e-headercells").find(".e-activeview").parent()).index();
	            var index = Math.floor((e.scrollLeft / (proxy.element.find(".e-workcells").width() + 1)) / 48);
	            if (currentDateIndex !== index)
	                proxy.element.find(".e-headercells").children().removeClass("e-activeview");
	            $(proxy.element.find(".e-headercells")[index]).children().addClass("e-activeview");
	        }
	    },

	    _getHorizontalheaderCellCount: function (weekcols) {
	        var cols = [];
	        for (var i = 0; i < this._dateRender.length; i++) {
	            curday = (this.currentView() === "month" || this.currentView() === "day") ? (ej.isNullOrUndefined(this.model.resources) && ej.isNullOrUndefined(this.model.group)) ? weekcols.calendar.days.names[new Date(this._dateRender[i]).getDay()] : weekcols.calendar.days.names[new Date(this._dateRender[i]).getDay()].slice(0, 3) : weekcols.calendar.days.names[new Date(this._dateRender[i]).getDay()].slice(0, 3);
	            var headerCellClass = (Globalize.format(new Date(this._dateRender[i]), this._pattern.d) === Globalize.format(new Date(), this._pattern.d)) ? "e-headerToday" : "";
	            temp = this.currentView() === "month" ? { currentDay: curday, currentDateClass: "", cellToday: headerCellClass, cellWidth: this.model.cellWidth } : this.currentView() === "day" ? (ej.isNullOrUndefined(this.model.resources) && ej.isNullOrUndefined(this.model.group)) ? { currentDay: curday + " " + Globalize.format(new Date(this._dateRender[i]), "dd MMMM yyyy"), currentDateClass: "", cellToday: headerCellClass, cellWidth: this.model.cellWidth } : { currentDay: curday + " " + Globalize.format(new Date(this._dateRender[i]), "dd MMM yy"), currentDateClass: "", cellToday: headerCellClass, cellWidth: this.model.cellWidth } : { currentDay: curday + " " + new Date(this._dateRender[i]).getDate(), currentDateClass: new Date(this._dateRender[i]).getDate() === new Date(this.currentDate()).getDate() ? "e-activeview" : "", cellToday: headerCellClass, cellWidth: this.model.cellWidth };
	            cols.push(temp);
	        }
	        return cols;
	    },

	    _getHoriStartEndTime: function (localizeAmPm) {
	        var startendTimes = [];
	        var daysCount = this.currentView() == "week" ? 7 : this.currentView() == "day" ? 1 : 5;
	        for (var day = 0; day < daysCount; day++) {
	            for (var i = this.model.startHour; i < this.model.endHour; i++) { // * 2
	                if (this.model.timeMode === "12") {
	                    timelist = ((i === this.model.startHour || i === 12) && (localizeAmPm.AM && localizeAmPm.PM)) ? Globalize.format(new Date(new Date().setHours(i)), "hh tt") : Globalize.format(new Date(new Date().setHours(i)), "hh 00");
	                    startendTimes.push({ time: timelist, id: i, timecellclass: ((i % 48) == 0 && day != 0) ? "e-dayend" : "", cellWidth: (!this.model.cellWidth) ? "30px" : this.model.cellWidth });
	                }
	                else
	                    startendTimes.push({ time: (i <= 9 ? "0" + i : i) + " 00", timecellclass: ((i % 48) == 0 && day != 0) ? "e-dayend" : "", cellWidth: (!this.model.cellWidth) ? "30px" : this.model.cellWidth });
	            }
	        }
	        return startendTimes;
	    },

	    _getHorizontalDayAppointments: function (appointmentsList, dateCount, day, res) {
	        for (var app = 0; app < appointmentsList.length; app++) {
	            if (!ej.isNullOrUndefined(this.model.group) && ej.scheduleFeatures.resources) {
	                if (Globalize.format(new Date(dateCount[day]), this._pattern.d) == Globalize.format(new Date(appointmentsList[app][this.model.appointmentSettings["startTime"]]), this._pattern.d) &&
                        Globalize.format(new Date(dateCount[day]), this._pattern.d) <= Globalize.format(new Date(appointmentsList[app][this.model.appointmentSettings["endTime"]]), this._pattern.d) &&
                        (appointmentsList[app][this.model.appointmentSettings.resourceFields.toString().split(',')[this.model.resources.length - 1]] == this.res1[res].id) &&
                        Globalize.format(new Date(appointmentsList[app][this.model.appointmentSettings["startTime"]]), this._pattern.d) <= Globalize.format(new Date(appointmentsList[app][this.model.appointmentSettings["endTime"]]), this._pattern.d))
	                    this._appList.push(appointmentsList[app]);
	            }
	            else {
	                if (Globalize.format(new Date(dateCount[day]), this._pattern.d) == Globalize.format(new Date(appointmentsList[app][this.model.appointmentSettings["startTime"]]), this._pattern.d) &&
                        Globalize.format(new Date(dateCount[day]), this._pattern.d) <= Globalize.format(new Date(appointmentsList[app][this.model.appointmentSettings["endTime"]]), this._pattern.d) &&
                        Globalize.format(new Date(appointmentsList[app][this.model.appointmentSettings["startTime"]]), this._pattern.d) <= Globalize.format(new Date(appointmentsList[app][this.model.appointmentSettings["endTime"]]), this._pattern.d))
	                    this._appList.push(appointmentsList[app]);
	            }
	        }

	    },

	    _renderHorizontalApp: function (rStartTime, rEndTime, record, day, cellHeight, timeScaleRowCount, startHour, endHour) {
	        var headerCell = this.element.find(".e-workcellstab tr");
	        var newResCollection = this._getResourceCollection();
	        var appHeight, appWidth, topPosition, leftPosition, bottomPosition;
	        if (rStartTime <= rEndTime) {
	            var startTime = ((new Date(rStartTime).getHours()) < startHour) ? (new Date(rStartTime).getHours() + (-new Date(rStartTime).getHours()) + (-new Date(rStartTime).getMinutes())) : (new Date(rStartTime).getHours() + (-startHour));
	            var endTime = ((new Date(rEndTime).getHours()) > endHour) ? (new Date(rEndTime).getHours() + (-new Date(rEndTime).getHours() + (endHour - startHour)) + (-new Date(rEndTime).getMinutes())) : (new Date(rEndTime).getHours() + (-startHour));
	            endTime = (new Date(rEndTime).getHours() == endHour) ? (new Date(endTime).getMinutes() + (-new Date(rEndTime).getMinutes())) : endTime;
	            appHeight = 20;
	            topPosition = 0;
	            var level = 0;
	            if (this.model.startHour <= new Date(rStartTime).getHours()) {

	                var groupIndex = (!ej.isNullOrUndefined(this.model.group) && this.model.group.resources.length > 1) ? this._findResourceIndex(newResCollection[0].dataSource, "id", record[this.model.appointmentSettings.resourceFields.toString().split(',')[this.model.resources.length - 2]]) + 1 : 0;
	                var resValue = (!ej.isNullOrUndefined(this.model.group) && !ej.isNullOrUndefined(this.model.resources)) ? this._findResourceIndex(this.level_Resources, "id", record[this.model.appointmentSettings.resourceFields.toString().split(',')[this.model.resources.length - 1]]) + 1 : 1;

	                var overlapList = (ej.scheduleFeatures.resources && !ej.isNullOrUndefined(this.model.resources) && !ej.isNullOrUndefined(this.model.group)) ? this._overlapApp(record[this.model.appointmentSettings["id"]], record[this.model.appointmentSettings["startTime"]], record[this.model.appointmentSettings["endTime"]], record[this.model.appointmentSettings.resourceFields.toString().split(',')[this.model.resources.length - 1]]) : this._overlapApp(record[this.model.appointmentSettings["id"]], record[this.model.appointmentSettings["startTime"]], record[this.model.appointmentSettings["endTime"]], null);
	                var proxy = this;

	                var level = (ej.scheduleFeatures.resources && !ej.isNullOrUndefined(this.model.resources) && !ej.isNullOrUndefined(this.model.group)) ? this._getOverlapCount(record, day, this._dateRender, record[this.model.appointmentSettings.resourceFields.toString().split(',')[this.model.resources.length - 1]]) : this._getOverlapCount(record, day, this._dateRender, null);
	                topPosition = (topPosition + (level * appHeight));

	                if (record[this.model.appointmentSettings["allDay"]]) {
	                    diffTime = new Date(Globalize.format(new Date(rEndTime), this._pattern.d)) - new Date(Globalize.format(new Date(rStartTime), this._pattern.d)); // recordEnd.AddDays(1).Subtract(recordStart);
	                    var diffDays = Math.round(diffTime / 86400000) + 1;
	                    if (diffDays > this._dateRender.length) // if no of days excceeded the renderdates
	                        diffDays = new Date(this._dateRender[this._dateRender.length - 1]).getDay() + 1 - new Date(rStartTime).getDay();
	                    diffMinutes = (diffDays * 24 * 60); // minutes
	                    cellIndex = ((this.model.endHour - this.model.startHour) * timeScaleRowCount) * day;
	                    appWidth = ((this.element.find(".e-workcells").outerWidth() / (30)) * diffMinutes);
	                    leftPosition = "0";
	                }
	                else if (new Date(Globalize.format(new Date(record[this.model.appointmentSettings["startTime"]]), this._pattern.d)) < new Date(Globalize.format(new Date(record[this.model.appointmentSettings["endTime"]]), this._pattern.d))) {
	                    diffTime = new Date(Globalize.format(new Date(rEndTime), this._pattern.d)) - new Date(Globalize.format(new Date(rStartTime), this._pattern.d));
	                    var diffDays = Math.round(diffTime / 86400000);
	                    diffMinutes = (((diffDays) * 24 * 60) - (new Date(rStartTime).getHours() * 60) + new Date(rStartTime).getMinutes()) + ((new Date(rEndTime).getHours() * 60) + new Date(rEndTime).getMinutes());
	                    cellIndex = (((this.model.endHour - this.model.startHour) * timeScaleRowCount) * day) + (startTime * timeScaleRowCount);
	                    appWidth = ((this.element.find(".e-workcells").outerWidth() / (30)) * diffMinutes);
	                    if (diffDays > this._dateRender.length) // if no of days excceeded the renderdates
	                    {
	                        diffDays = new Date(this._dateRender[this._dateRender.length - 1]).getDay() + 1 - new Date(rStartTime).getDay();
	                        diffMinutes = ((diffDays) * 24 * 60); // minutes
	                        appWidth = ((this.element.find(".e-workcells").outerWidth() / (30)) * diffMinutes) - (cellIndex * 25) + 10;
	                    }
	                    leftPosition = (parseFloat((new Date(rStartTime).getMinutes())) * (this.element.find(".e-workcells")[0].offsetWidth) * timeScaleRowCount / 60);

	                }
	                else {
	                    leftPosition = (parseFloat((new Date(rStartTime).getMinutes())) * (this.element.find(".e-workcells")[0].offsetWidth) * timeScaleRowCount / 60);
	                    diffInMinutes = ((new Date(rEndTime).getHours() * 60) + new Date(rEndTime).getMinutes()) - ((new Date(rStartTime).getHours() * 60) + new Date(rStartTime).getMinutes());
	                    appWidth = ((this.element.find(".e-workcells").width() / 30) * diffInMinutes) + ((new Date(rEndTime).getHours() - new Date(rStartTime).getHours()) * timeScaleRowCount) - 2;
	                    cellIndex = (((this.model.endHour - this.model.startHour) * timeScaleRowCount) * day) + (startTime * timeScaleRowCount);
	                }
	                var appointColor = (!ej.isNullOrUndefined(this.model.resources) && ej.scheduleFeatures.resources) ? this._getResourceColor(record) : { appointData: "", appointCustomcss: "" };
	                //

	                //
	                //var minDate = new Date(this._dateRender[this._dateRender.length - 1]);
	                //var maxDate = new Date(this._dateRender[0]);
	                //for (var i = 0; i < overlapList.length; i++) {
	                //    var date = new Date(overlapList[i][proxy.model.appointmentSettings["startTime"]]);
	                //    if (date < minDate)
	                //        minDate = date;
	                //    var date1 = new Date(overlapList[i][proxy.model.appointmentSettings["endTime"]]);
	                //    if (date1 > maxDate)
	                //        maxDate = date1;
	                //}
	                //if (overlapList.length > 0) {
	                //    if (this._categorybar) {
	                //        this._renderCategory(headerCell, groupIndex, record, cellHeight, minDate, maxDate, appointColor, timeScaleRowCount, day, startTime, endTime);
	                //        this._categorybar = false;
	                //    }
	                //    else {
	                //        if (new Date(record[this.model.appointmentSettings["startTime"]]) >= new Date(maxDate) && record[this.model.appointmentSettings["endTime"]] <= minDate)
	                //            this._renderCategory(headerCell, groupIndex, record, cellHeight, minDate, maxDate, appointColor, timeScaleRowCount, day, startTime, endTime);
	                //    }
	                //}
	                //else {
	                //    this._renderCategory(headerCell, groupIndex, record, cellHeight, new Date(rStartTime), new Date(rEndTime), appointColor, timeScaleRowCount, day, startTime, endTime);
	                //}

	                //

	                //
	                if ((topPosition + appHeight) < cellHeight) {
	                    //user-defined Template data rendering
	                    var userAppTemplId = (this.model.appointmentTemplateId) ? true : false;
	                    var userTempHtml = this._getUserAppointmentTemplate(record);
	                    var startTimeDisp = this.model.timeMode == "12" ? Globalize.format(new Date(record[this.model.appointmentSettings["startTime"]]), this._pattern.t) : Globalize.format(new Date(record[this.model.appointmentSettings["startTime"]]), "HH:mm");
	                    var endTimeDisp = this.model.timeMode == "12" ? Globalize.format(new Date(record[this.model.appointmentSettings["endTime"]]), this._pattern.t) : Globalize.format(new Date(record[this.model.appointmentSettings["endTime"]]), "HH:mm");

	                    //   var category = "e-" + record[this.model.appointmentSettings["categorize"]];
	                    var timeCellhtml = this.appTemplate.render({ id: record[this.model.appointmentSettings["id"]], appHeight: appHeight, rtl: this.model.enableRTL, subject: record[this.model.appointmentSettings["subject"]], startTime: startTimeDisp, endTime: endTimeDisp, left: leftPosition + "px", top: topPosition, appWidth: appWidth + "px", userAppTemplId: userAppTemplId, userTemplate: userTempHtml, appClass: "e-appointment", appointData: appointColor.appointData, appointCustomcss: appointColor.appointCustomcss, orientation: this.model.orientation });
	                    $($(headerCell[resValue + groupIndex - 1]).find("div.e-appointwrapper")[cellIndex]).append(timeCellhtml);
	                    if (!ej.isNullOrUndefined(this.model.group) && this.model.group.resources.length > 1)
	                        $($($(headerCell.find("td.e-resourceheadercells")).parent()[groupIndex - 1]).find("div.e-appointwrapper")[cellIndex]).append("<div class='e-categorybar' id='categorybar_" + record[this.model.appointmentSettings["id"]] + "' style='position:absolute;height:" + (cellHeight - 3) + "px;background:" + appointColor.appointData + ";width:" + (appWidth + 1) + "px;left:" + leftPosition + "px;'></div>");
	                }
	                else {
	                    if ($("#" + this._id + "_" + Globalize.format(new Date(record[this.model.appointmentSettings["startTime"]]), "MM_dd_yyyy") + "_" + (resValue + groupIndex - 1) + "_" + cellIndex).length == 0) {
	                        $($(headerCell[resValue + groupIndex - 1]).find("div.e-appointwrapper")[cellIndex]).append('<div id="' + this._id + "_" + Globalize.format(new Date(record[this.model.appointmentSettings["startTime"]]), "MM_dd_yyyy") + "_" + (resValue + groupIndex - 1) + "_" + cellIndex + '" class="e-icon e-appsoverflow"></div>');
	                        if (!ej.isNullOrUndefined(this.model.group) && this.model.group.resources.length > 1)
	                            $($($(headerCell.find("td.e-resourceheadercells")).parent()[groupIndex - 1]).find("div.e-appointwrapper")[cellIndex]).append("<div class='e-categorybar' id='categorybar_" + record[this.model.appointmentSettings["id"]] + "' style='position:absolute;height:" + (cellHeight - 3) + "px;background:" + appointColor.appointData + ";width:" + (appWidth + 1) + "px;left:" + leftPosition + "px;'></div>");
	                        this.element.find("#" + this._id + "_" + Globalize.format(new Date(record[this.model.appointmentSettings["startTime"]]), "MM_dd_yyyy") + "_" + (resValue + groupIndex - 1) + "_" + cellIndex).css(this.model.enableRTL ? "right" : "left", (this.element.find(".e-workcells").width() + leftPosition - 25) + "px").css("top", topPosition + "px");
	                    }
	                }

	            }
	        }
	    },

	    _renderCategory: function (headerCell, groupIndex, record, cellHeight, minDate, maxDate, appointColor, timeScaleRowCount, day, startTime, endTime) {
	        var diffTime, diffDays, diffMinutes, cellIndex, appWidth, leftPosition;
	        if (record[this.model.appointmentSettings["allDay"]]) {
	            diffTime = new Date(Globalize.format(new Date(maxDate), this._pattern.d)) - new Date(Globalize.format(new Date(minDate), this._pattern.d)); // recordEnd.AddDays(1).Subtract(recordStart);
	            var diffDays = Math.round(diffTime / 86400000) + 1;
	            if (diffDays > this.dateRender.length) // if no of days excceeded the renderdates
	                diffDays = new Date(this.dateRender[this.dateRender.length - 1]).getDay() + 1 - new Date(minDate).getDay();
	            diffMinutes = (diffDays * 24 * 60); // minutes
	            cellIndex = ((this.model.endHour - this.model.startHour) * timeScaleRowCount) * day;
	            appWidth = ((this.element.find(".e-workcells").width() / (30 - 1.1)) * diffMinutes) + 3;
	            leftPosition = "0";
	        }
	        else if (new Date(Globalize.format(new Date(record[this.model.appointmentSettings["startTime"]]), this._pattern.d)) < new Date(Globalize.format(new Date(record[this.model.appointmentSettings["endTime"]]), this._pattern.d))) {
	            diffTime = new Date(Globalize.format(new Date(maxDate), this._pattern.d)) - new Date(Globalize.format(new Date(minDate), this._pattern.d));
	            var diffDays = Math.round(diffTime / 86400000) + 1;
	            diffMinutes = ((diffDays - 1) * 24 * 60); // minutes
	            cellIndex = (((this.model.endHour - this.model.startHour) * timeScaleRowCount) * day) + (startTime * timeScaleRowCount);
	            //  appWidth = Math.round((this.element.find(".e-workcells").width() / (30 - 1)) * diffMinutes) - (cellIndex * this.element.find(".e-workcells").width()) + "px";
	            appWidth = ((this.element.find(".e-workcells").width() / (30 - 1.1)) * diffMinutes) + (new Date(maxDate).getMinutes() * ((this.element.find(".e-workcells")[0].offsetWidth) * timeScaleRowCount) / 60) + ((new Date(maxDate).getHours() - this.model.startHour) * timeScaleRowCount);
	            if (diffDays > this.dateRender.length) // if no of days excceeded the renderdates
	            {
	                diffDays = new Date(this.dateRender[this.dateRender.length - 1]).getDay() + 1 - new Date(minDate).getDay();
	                diffMinutes = ((diffDays) * 24 * 60); // minutes
	                appWidth = ((this.element.find(".e-workcells").width() / (30 - 1.1)) * diffMinutes) - (cellIndex * 25) + 10;
	            }
	            leftPosition = (parseFloat((new Date(minDate).getMinutes())) * (this.element.find(".e-workcells")[0].offsetWidth) * timeScaleRowCount / 60);

	        }
	        else {
	            leftPosition = (parseFloat((new Date(minDate).getMinutes())) * (this.element.find(".e-workcells")[0].offsetWidth) * timeScaleRowCount / 60);
	            diffInMinutes = ((new Date(maxDate).getHours() * 60) + new Date(maxDate).getMinutes()) - ((new Date(minDate).getHours() * 60) + new Date(minDate).getMinutes());
	            appWidth = ((this.element.find(".e-workcells").width() / 30) * diffInMinutes) + ((new Date(maxDate).getHours() - new Date(minDate).getHours()) * timeScaleRowCount) - 2;
	            cellIndex = (((this.model.endHour - this.model.startHour) * timeScaleRowCount) * day) + (startTime * timeScaleRowCount);
	        }
	        if (!ej.isNullOrUndefined(this.model.group) && this.model.group.resources.length > 1)
	            $($($(headerCell.find("td.e-resourceheadercells")).parent()[groupIndex - 1]).find("div.e-appointwrapper")[cellIndex]).append("<div class='e-categorybar' style='position:absolute;height:" + (cellHeight - 3) + "px;background:" + appointColor.appointData + ";width:" + (appWidth + 1) + "px;left:" + leftPosition + "px;'></div>");

	    },

	    _getHorizontalOverlapCount: function (appointments, dateRender, day, record, resValue) {
	        var filterData = [];
	        for (var i = 0; i < appointments.length; i++) {
	            if (ej.scheduleFeatures.resources && !ej.isNullOrUndefined(this.model.resources) && !ej.isNullOrUndefined(this.model.group)) {
	                if ((Globalize.format(new Date(appointments[i][this.model.appointmentSettings["startTime"]]), this._pattern.d) <= Globalize.format(new Date(dateRender[day]), this._pattern.d)) &&
                    (Globalize.format(new Date(appointments[i][this.model.appointmentSettings["endTime"]]), this._pattern.d) >= Globalize.format(new Date(dateRender[day]), this._pattern.d)) &&
                    (((new Date(record[this.model.appointmentSettings["startTime"]]) < new Date(appointments[i][this.model.appointmentSettings["startTime"]]) && new Date(record[this.model.appointmentSettings["endTime"]]) >= new Date(appointments[i][this.model.appointmentSettings["startTime"]])) ||
                        (new Date(record[this.model.appointmentSettings["startTime"]]) < new Date(appointments[i][this.model.appointmentSettings["endTime"]]) && new Date(record[this.model.appointmentSettings["endTime"]]) > new Date(appointments[i][this.model.appointmentSettings["endTime"]])) ||
                        (new Date(record[this.model.appointmentSettings["startTime"]]) >= new Date(appointments[i][this.model.appointmentSettings["startTime"]]) && new Date(record[this.model.appointmentSettings["startTime"]]) < new Date(appointments[i][this.model.appointmentSettings["endTime"]])) ||
                        (new Date(record[this.model.appointmentSettings["endTime"]]) > new Date(appointments[i][this.model.appointmentSettings["startTime"]]) && new Date(record[this.model.appointmentSettings["endTime"]]) < new Date(appointments[i][this.model.appointmentSettings["endTime"]])))
                        && ((resValue != null) ? (resValue == appointments[i][this.model.resources[this.model.resources.length - 1]["field"]]) : resValue == null))) {
	                    filterData.push(appointments[i]);
	                }
	            }
	            else {
	                if ((Globalize.format(new Date(appointments[i][this.model.appointmentSettings["startTime"]]), this._pattern.d) <= Globalize.format(new Date(dateRender[day]), this._pattern.d)) &&
                    (Globalize.format(new Date(appointments[i][this.model.appointmentSettings["endTime"]]), this._pattern.d) >= Globalize.format(new Date(dateRender[day]), this._pattern.d)) &&
                    (((new Date(record[this.model.appointmentSettings["startTime"]]) < new Date(appointments[i][this.model.appointmentSettings["startTime"]]) && new Date(record[this.model.appointmentSettings["endTime"]]) >= new Date(appointments[i][this.model.appointmentSettings["startTime"]])) ||
                        (new Date(record[this.model.appointmentSettings["startTime"]]) < new Date(appointments[i][this.model.appointmentSettings["endTime"]]) && new Date(record[this.model.appointmentSettings["endTime"]]) > new Date(appointments[i][this.model.appointmentSettings["endTime"]])) ||
                        (new Date(record[this.model.appointmentSettings["startTime"]]) >= new Date(appointments[i][this.model.appointmentSettings["startTime"]]) && new Date(record[this.model.appointmentSettings["startTime"]]) < new Date(appointments[i][this.model.appointmentSettings["endTime"]])) ||
                        (new Date(record[this.model.appointmentSettings["endTime"]]) > new Date(appointments[i][this.model.appointmentSettings["startTime"]]) && new Date(record[this.model.appointmentSettings["endTime"]]) < new Date(appointments[i][this.model.appointmentSettings["endTime"]]))))) {
	                    filterData.push(appointments[i]);
	                }
	            }
	        }
	        return filterData;
	    },

	    _moveScroller: function () {
	        if (this.currentView() == "week" || this.currentView() == "workweek") {
	            if (this.element.find(".e-headercells").children().hasClass("e-activeview")) {
	                var currentDateIndex = $(this.element.find(".e-headercells").find(".e-activeview").parent()).index();
	                var cellOffsetLeft = this.element.find(".e-workcells")[currentDateIndex * 48].offsetLeft;
	                $(this.element.find(".e-horizontaltimecells").find("table tr")[2]).css("left", (-cellOffsetLeft) + "px");
	                $('div#' + this._id + "_scroller").ejScroller({ scrollLeft: cellOffsetLeft });
	            }
	        }
	    },


	    _renderHorizontalHeaderBar: function () {
	        var view = this.currentView(); var weekcols = Globalize.culture(this.model.locale);
	        var viewColCount = view == "day" ? 1 : view == "week" ? 7 : 5;
	        this.datesColumn = this._getHeaderAllDayCellsCount();
	        if (view === 'month') {
	            this.datesColumn = [], this.monthDays = [], this.totdays = [];
	            for (var i = 0; i < this._dateRender.length; i++) {
	                if (new Date(this.currentDate()).getMonth() === new Date(this._dateRender[i]).getMonth())
	                    this.totdays.push(new Date(this._dateRender[i]));
	            }
	            var date = this.totdays.length;
	            for (var i = 0; i < date; i++) {
	                curday = weekcols.calendar.days.names[new Date(this.totdays[i]).getDay()];
	                temp = { currentDate: Globalize.format(new Date(this.totdays[i]), "dd"), currentDateClass: "", currentDay: weekcols.calendar.days.names[new Date(this.totdays[i]).getDay()].toString().slice(0, 1), cellWidth: (!this.model.cellWidth && this.currentView() != "month") ? "30px" : this.model.cellWidth };
	                this.datesColumn.push(temp);
	                this.monthDays.push(new Date(this.totdays[i]));
	            }
	        }
	        var multiResources = (!ej.isNullOrUndefined(this.model.resources) && !ej.isNullOrUndefined(this.model.group)) ? true : false;
	        var html = this.horizontalHeader.render({ cols: this.datesColumn, multiRes: multiResources, view: view, timeTdCount: this._getStartEndTime(), colspan: view == "month" ? 1 : (this._getStartEndTime().length / viewColCount) * 2, height: view == 'month' ? 63 : 59, width: this.model.width, headercellWidth: view == "day" ? "100%" : "55px" });
	        this.aTR.append(html);
	    },

	    _renderHorizontalContent: function ($contentAreaDiv, $contentTable, $contentBody, $contentTR, $WorkCellTD, $scrollerbarDiv) {
	        var trCount, resHtml;
	        if (!ej.isNullOrUndefined(this.model.resources) && !ej.isNullOrUndefined(this.model.group)) {
	            resHtml = this._renderHorizontalResources();
	            trCount = this._horiResources;
	        }
	        else {
	            var tdWidth = this.currentView() == "month" ? "24px" : "25px";
	            resHtml = "";
	            trCount = [{ classname: "e-childnode", tdWidth: tdWidth }];
	        }
	        //Work cells
	        var workCellhtml = this.cellTemplate.render({ cellrows: trCount, view: this.currentView(), cols: this.currentView() == 'month' ? this.datesColumn : this._getStartEndTime() });
	        
	        this.wTR.append(ej.buildTag('td').append(resHtml)).append(ej.buildTag('td').append($contentAreaDiv.append($contentTable.append($contentBody.append($contentTR.append($WorkCellTD.append($scrollerbarDiv.append(this.$WorkCellDiv.append(workCellhtml)))))))));
	    },

	    _getResourceHeadTemplate: function (record) {
	        var userTemp = $.templates($(this.model.resourceHeaderTemplateId).html());
	        var userTempHtml = userTemp.render(record);
	        return userTempHtml;
	    },

	    _renderHorizontalResources: function(){
	        this._resCollect = []; this._horizontalResRender = []; this._horiResources = [];
	        this._renderMultipleResourceHeaderTemplate(null, null);
	        var view = this.currentView();
	        var viewColCount = view == "day" ? 1 : view == "week" || view == "month" ? 7 : 5;
	        var tdWidth = this.currentView() == "month" ? "24px" : "25px";
	        var userResTemplId = (this.model.resourceHeaderTemplateId) ? true : false;
	        var resTempHtml = "";
	        //     var resTempHtml = this._getResourceHeadTemplate(record);
	        if (!ej.isNullOrUndefined(this.model.group)) {
	            if (this.model.group.resources.length === 1) {
	                var index;
	                for (var i = 0; i < this.model.group.resources.length; i++) {
	                    index = this._findResourceIndex(this.model.resources, "name", this.model.group.resources[i]);
	                }
	                for (var i = 0; i < this._resourceInfo[index].dataSource.length; i++) {
	                    this._horiResources.push({ name: this._resourceInfo[index].dataSource[i][this._resourceInfo[index].text], idnum: this._resourceInfo[index].dataSource[i][this._resourceInfo[index].id], classname: "e-childnode", marginleft: "20px", width: "10px", backgroundcolor: "", border: "1px solid #bbbcbb", bordertop: "", tdWidth: tdWidth, userResHeader: this._getResourceHeadTemplate({ name: this._resourceInfo[index].dataSource[i][this._resourceInfo[index].text], classname: "e-childnode", id: this._resourceInfo[index].dataSource[i][this._resourceInfo[index].id], color: this._resourceInfo[index].dataSource[i][this._resourceInfo[index].color] }), userResTemplId: userResTemplId });
	                    this._resCollect.push(this._resourceInfo[index].dataSource[i]);
	                }
	                this._horizontalResRender.push(this._resCollect);
	            }
	            else {
	                var newResCollection = this._getResourceCollection();
	                var res = [], query, resApp, resourceHeaderHtml;

	                var d = 0;
	                var k = 0; var group = [];
	                
	                for (var j = 0; j < newResCollection[d].dataSource.length; j++) {
	                    this._horiResources.push({ name: newResCollection[d].dataSource[j].text, idnum: newResCollection[d].dataSource[j].id, classname: "e-parentnode", marginleft: "20px", width: "10px", backgroundcolor: "#bbbcbb", border: "1px solid #bbbcbb", bordertop: "1px solid #bbbcbb", tdWidth: tdWidth, userResHeader: this._getResourceHeadTemplate({ name: newResCollection[d].dataSource[j].text, classname: "e-parentnode", id: newResCollection[d].dataSource[j].id, color: newResCollection[d].dataSource[j].color }), userResTemplId: userResTemplId });
	                    this._resCollect.push(newResCollection[d].dataSource[j]);
	                    k++;

	                    for (i = 0; i < newResCollection[k].dataSource.length; i++) {
	                        p = k;
	                        if (newResCollection[k - 1].dataSource[j].id == newResCollection[k].dataSource[i].groupId) {
	                            group.push(newResCollection[k].dataSource[i]);
	                            this._horiResources.push({ name: newResCollection[k].dataSource[i].text, idnum: newResCollection[k].dataSource[i].id + "_" + newResCollection[k].dataSource[i].groupId, classname: "e-childnode", marginleft: "45px", width: "20px", backgroundcolor: "", border: "1px dotted #bbbcbb", bordertop: "", tdWidth: tdWidth, userResHeader: this._getResourceHeadTemplate({ name: newResCollection[k].dataSource[i].text, classname: "e-childnode", id: newResCollection[k].dataSource[i].id, color: newResCollection[k].dataSource[i].color }), userResTemplId: userResTemplId });
	                            this._resCollect.push(newResCollection[k].dataSource[i]);
	                            while (k < newResCollection.length - 1) {
	                                for (var m = 0; m < newResCollection[k + 1].dataSource.length; m++) {
	                                    if (newResCollection[k].dataSource[i].id == newResCollection[k + 1].dataSource[m].groupId) {
	                                        group.push(newResCollection[k + 1].dataSource[m]);
	                                        this._horiResources.push({ name: newResCollection[k + 1].dataSource[m].text, classname: "e-childnode", marginleft: "55px", width: "20px", backgroundcolor: "", border: "1px dotted #bbbcbb", bordertop: "", tdWidth: tdWidth, userResHeader: this._getResourceHeadTemplate({ name: newResCollection[k + 1].dataSource[m].text, classname: "e-childnode", id: newResCollection[k + 1].dataSource[m].id, color: newResCollection[k + 1].dataSource[m].color }), userResTemplId: userResTemplId });
	                                        this._resCollect.push(newResCollection[k + 1].dataSource[m]);
	                                    }
	                                }
	                                k++;
	                            }

	                        }
	                        k = p;
	                    }
	                    k = 0;
	                }

	                //           debugger;

	                // for (var resCollect = 0; resCollect < newResCollection.length - 1; resCollect++) {
	                //     for (var i = 0; i < newResCollection[resCollect].dataSource.length; i++) {
	                //         var j = i + 1;
	                //         this._horiResources.push({ name: newResCollection[j - 1].dataSource[i].text, classname: "e-parentnode", marginleft: "50px", width: "10px", backgroundcolor: "#bbbcbb", border: "1px solid #bbbcbb", bordertop: "1px solid #bbbcbb" });
	                //         this._resCollect.push(newResCollection[j - 1].dataSource[i]);

	                //    //     while (j <= newResCollection.length - 1) {
	                //             //    query = new ej.Query().where("groupId", ej.FilterOperators.equal, newResCollection[j-1].dataSource[i][newResCollection[j-1].id]);
	                //             //    resApp = new ej.DataManager(newResCollection[j].dataSource).executeLocal(query); //this._resourceManager1.executeLocal(query);
	                //             //    if (resApp.length > 0) {
	                //             //        for (var a = 0; a < resApp.length; a++) {
	                //             //            if (newResCollection[j - 1].dataSource[i].id == resApp[a].groupId) {
	                //             //                this._horiResources.push({ name: resApp[a].text, classname: "e-childnode", marginleft: "80px", width: "20px", backgroundcolor: "", border: "1px dotted #bbbcbb", bordertop: "" });
	                //             //                this._resCollect.push(resApp[a]);
	                //             //            }
	                //             //        }
	                //             //    }
	                //             //j++;
	                //   //      }
	                //     }
	                //}
	                this._horizontalResRender.push(this._resCollect);
	            }
	        }
	        else {
	            for (var k = 0; k < this.model.resources.length; k++) {
	                this._horizontalResRender.push(this.model.resources[k].resourceSettings.dataSource);
	                this._resCollect.push(this.model.resources[k].resourceSettings.dataSource);
	            }
	        }
	        var html = this.horizontalResHeader.render({ trs: this._horiResources });
	        return html;
	    },

	    _renderHorizontalCurrentTime: function () {
	        if (this.currentView() !== "month") {
	            if (this.element.find("td.e-headerToday").length > 0) {
	                this.element.find('.e-horizontaltabletimecell').append('<div id="' + this._id + '_HighlightCurrentTimeline" class="e-highlightcurrenttimeline" style="height: 92%; width: 2px; position: absolute; background-color: red; "></div>')
	                $('#' + this._id + '_HighlightCurrentTimeline').css("top", this.element.find('.e-headerdays tr')[1].offsetTop);
	            }
	        }
	        else {
	            var monthDays = [];
	            for (var k = 0; k < this._dateRender.length; k++) {
	                if (new Date(this.currentDate()).getMonth() === new Date(this._dateRender[k]).getMonth())
	                    monthDays.push(new Date(this._dateRender[k]));
	            }
	            var target = this.element.find(".e-workcells");
	            for (var i = 0; i < monthDays.length; i++) {
	                if (Globalize.format(new Date(monthDays[i]), this._pattern.d) === Globalize.format(new Date(), this._pattern.d))
	                    $(target[i]).addClass("e-monthcurrenttime");
	            }
	        }
	    },

	    _horizontalTimePosition: function (temp, now) {
	        if (temp.element.find("td.e-headerToday").length > 0) {
	            var headertodaypos = temp.element.find("td.e-headerToday")[0].cellIndex;
	            var startTime = (now.getHours() - temp.model.startHour);
	            var cellIndex = (((temp.model.endHour - temp.model.startHour) * 2) * headertodaypos) + (startTime * 2);
	            var leftPosition = (parseFloat((now.getMinutes())) * (temp.element.find(".e-workcells")[0].offsetWidth) * 2 / 60);
	            $('#' + temp._id + '_HighlightCurrentTimeline').css("left", (cellIndex * temp.element.find(".e-workcells")[0].offsetWidth) + leftPosition);
	        }
	    },

	    _horizontalBusinessHighlight: function (renderDates, b_startHour, b_endHour, timeRowsCount, target) {
	        var tdLength = (ej.scheduleFeatures.resources && !ej.isNullOrUndefined(this.model.group)) ? this._horiResources.length : 1;
	        if (this.currentView() !== "month") {
	            for (var res = 0; res < tdLength; res++) {
	                for (var k = 0; k < renderDates.length; k++) {
	                    for (var i = (b_startHour * timeRowsCount) ; i < (b_endHour * timeRowsCount) ; i++) {
	                        if (new Date(renderDates[k]).getDay() !== 0 && new Date(renderDates[k]).getDay() !== 6)
	                            $($(this.element.find(".e-workcellstab tr")[res]).find(".e-workcells")[i + (((this.model.endHour - this.model.startHour) * timeRowsCount) * k)]).addClass("e-businesshighlightworkcells");
	                    }
	                }
	            }
	        }
	        else {
	            var monthDays = this.monthDays;
	            var target = this.element.find(".e-workcells");target.removeClass("e-monthcellhighlight");
	            for (var res = 0; res < tdLength; res++) {
	                for (var i = 0; i < monthDays.length; i++) {
	                    if (new Date(monthDays[i]).getDay() !== 0 && new Date(monthDays[i]).getDay() !== 6)
	                        $($(this.element.find(".e-workcellstab tr")[res]).find(".e-workcells")[i]).addClass("e-monthcellhighlight");
	                }
	            }
	        }
	    },

	    _onResourceClick: function (e) {
	        var target = $(e.currentTarget);
	        if (target.hasClass("e-resourceicon e-resourcecollapse")) {
	            target.removeClass("e-resourceicon e-resourcecollapse");
	            target.addClass("e-resourceicon e-resourceexpand");
	        }
	        else {
	            target.removeClass("e-resourceicon e-resourceexpand");
	            target.addClass("e-resourceicon e-resourcecollapse");
	        }
	        var index = $(e.currentTarget).parent().index(); var newResCollection = this._getResourceCollection();
	        var query = new ej.Query().where("groupId", ej.FilterOperators.equal, parseInt(e.currentTarget.id));
	        var resApp = new ej.DataManager(newResCollection[newResCollection.length - 1].dataSource).executeLocal(query);
	        for (var i = index + 1; i <= (resApp.length + index) ; i++) {
	            $(this.element.find(".e-resourceheadertable").find("tr")[i]).toggle();
	            $(this.element.find(".e-workcellstab").find("tr")[i]).toggle();
	        }
	    },

	};

})(jQuery, Syncfusion);;
(function ($, ej, undefined) {

    ej.ExcelFilter = ej.ExcelFilter || {};

    ej.excelFilter = function (options) {
        this._ctrlInstance = options["instance"];
        this.id = this._ctrlInstance._id;
        this._dialogContainer = null;
        this._showSort = options["showSortOptions"] || false;
        this._maxCount = options["maxFilterLimit"] || 1000;
        this._formatFiltering = false;
        this._locale = this._ctrlInstance.model.locale || "en-US";
        this._filterHandler = options["filterHandler"] || null;
        this._cssClass = options["cssClass"] || null;
        this._matchCase = options["allowCaseSensitive"] || false;
        this._title = options["title"] || this._getLocalizedLabel("title");
        this.fName = null;
        this._displayName = null;
        this._dataSource = null;
        this._$colType = null;
        this._$key = 0;
        this.filteredColumn = null;
        this.sortedColumns = null;        
        this._chkList = null;
        this._listsWrap = null;
        this._menuWrap = null;
        this._localJSON = null;
        this._actualCount = 0;
        this._totalRcrd = 0;
        this._searchCount = 0;
        this._currentData = null;
        this._openedFltr = null;
        this._predicates = [];
        this.cFilteredCols = [];
        this._noDlg = [];
        this._sepAftr = ["sortDesc","notequal", "between", "top10","endswith","contains"];
        this._posType = ["number", "date", "string", "boolean"];
        this._reqInProgess = false;
        this._isFiltered = false;
        this._selectAll = "<div class='e-ftrchk'><input type='checkbox' class='e-selectall' value='selectall' class='e-ftrchk' /><label class='e-ftrchk'>(Select All)</label></div>";
        this._blank = "<div class='e-ftrchk'><input type='checkbox' id='blanks' class='e-ftrchk e-blanks' /><label class='e-ftrchk' for='blanks'>(Blanks)</label></div>";
        this.numberMenuOpt = [
              { id: 1, text: this._getLocalizedLabel("SortNoSmaller"), sprite: "e-sortasc e-icon e-fnsort", htmlAttribute: { "ejfnrole": "sortAsc" } },
              { id: 2, text: this._getLocalizedLabel("SortNoLarger"), sprite: "e-sortdesc e-icon e-fnsort", htmlAttribute: { "ejfnrole": "sortDesc" } },
              { id: 3, text: this._getLocalizedLabel("ClearFilter"), sprite: "e-filternone e-icon", htmlAttribute: { "ejfnrole": "clearfilter" } },
              { id: 4, text: this._getLocalizedLabel("NumberFilter"),htmlAttribute: { "ejfnrole": "popup" }, child: this._getLocalizedLabel("NumberMenuOptions") }];
        this.stringMenuOpt  = this.booleanMenuOpt = [
              { id: 1, text: this._getLocalizedLabel("SortTextAscending"), sprite: "e-sortasc e-icon e-fnsort", htmlAttribute: { "ejfnrole": "sortAsc" } },
              { id: 2, text: this._getLocalizedLabel("SortTextDescending"), sprite: "e-sortdesc e-icon e-fnsort", htmlAttribute: { "ejfnrole": "sortDesc" } },
              { id: 3, text: this._getLocalizedLabel("ClearFilter"), sprite: "e-filternone e-icon", htmlAttribute: { "ejfnrole": "clearfilter" } },
              { id: 4, text: this._getLocalizedLabel("TextFilter"), htmlAttribute: { "ejfnrole": "popup" }, child: this._getLocalizedLabel("StringMenuOptions") }];
        this.dateMenuOpt = [
              { id: 1, text: this._getLocalizedLabel("SortDateOldest"), sprite: "e-sortasc e-icon e-fnsort", htmlAttribute: { "ejfnrole": "sortAsc" } },
              { id: 2, text: this._getLocalizedLabel("SortDateNewest"), sprite: "e-sortdesc e-icon e-fnsort", htmlAttribute: { "ejfnrole": "sortDesc" } },
              { id: 3, text: this._getLocalizedLabel("ClearFilter"), sprite: "e-filternone e-icon", htmlAttribute: { "ejfnrole": "clearfilter" } },
              { id: 4, text: this._getLocalizedLabel("DateFilter"), htmlAttribute: { "ejfnrole": "popup" }, child: this._getLocalizedLabel("DateMenuOptions") }
        ];
        
        return this;
    };
    ej.excelFilter.prototype = {
        getField: function () {
            return this.fName;
        },
        getFormat: function () {
            return this._$format;
        },
        /*Main Dialog*/
        renderDialog: function (type) {
            this._$colType = type;
            var dlgId = this.id + type + "_excelDlg";           
            //if ($("#" + dlgId).length != 0)
            //    return;
            var $dlg = ej.buildTag("div#" + dlgId + ".e-excelfilter e-dlgcontainer e-shadow");
            var $ul = ej.buildTag("ul#" + this.id + type + "_MenuItem");
            var menuData = this._getMenuData(type);
            $ul = this._createLiTag($ul, menuData, false);
            var $searchBox = ej.buildTag("span.e-searchbox e-fields").append(ej.buildTag("input#" + this.id + "_SearchBox.e-ejinputtext e-searchinput", {}, {}, { "type": "text", "placeholder":"Search" })).append(ej.buildTag("span.e-search e-icon"))
            var $lbox = ej.buildTag("div#" + this.id + type + "_CheckBoxList.e-checkboxlist e-fields").append(ej.buildTag("div"));
            var $btns = this._createBtn();
            var $sBox = ej.buildTag("div.e-searchcontainer");            
            var $status = ej.buildTag("div.e-status e-fields e-hide", this._getLocalizedLabel("CheckBoxStatusMsg"));
            $dlg.append($ul);            
            $sBox.append($searchBox);
            $sBox.append($status);
            $sBox.append($lbox);
            $sBox.append($btns);
            $dlg.append($sBox);
            $dlg.appendTo("body");
            $dlg.css("display", "none");
            this._renderCustomFDlg(type);
            this._dialogContainer = $dlg;
            if(this._cssClass!=null)
                $dlg.addClass(this._cssClass);
            if (!this._showSort) {
                $ul.find(".e-fnsort").closest("li").css("display", "none");
                $ul.find("li.e-separator:first").css("display", "none");
            }
            this._lsitBoxTemplate();
            this._renderSubCtrls(type);
            this._wireEvents();
        },
        _renderSubCtrls: function (type) {
            $("#" + this.id + type + "_MenuItem").ejMenu({ orientation: "vertical", width: 266, click: ej.proxy(this._menuHandler, this), enableRTL: this._ctrlInstance.model.enableRTL });
            $("#" + this.id + type + "_OkBtn").ejButton({ text: this._getLocalizedLabel("Ok"), showRoundedCorner: true, width: 60, click: ej.proxy(this._fltrBtnHandler, this), enabled: true });
            $("#" + this.id + type + "_CancelBtn").ejButton({ text: this._getLocalizedLabel("Cancel"), showRoundedCorner: true, width: 60, click: ej.proxy(this.closeXFDialog, this) });
            $("#" + this.id + type + "_CheckBoxList").ejScroller({ height: 130, width: 234, scroll: ej.proxy(this._virtualize,this) });
            $("#" + this.id + type + "_CheckBoxList").ejWaitingPopup({ showOnInit: false });
        },
        openXFDialog: function (options) {
            this.fName = options["field"];
            this._dataSource = options["dataSource"];
            this._$colType = options["type"] || "string";
            this._$format = options["format"] || "";
            this.filteredColumn = options["filteredColumns"] || this._ctrlInstance.model.filterSettings.filteredColumns;
            this.sortedColumns = options["sortedColumns"] || this._ctrlInstance.model.sortSettings.sortedColumns;
            this._displayName = options["displayName"];
            this._$key = options["key"] || 0;
            if (this._openedFltr == null || !this._openedFltr.is($("#" + this.id + this._$colType + "_excelDlg"))) {
                this.closeXFDialog();
                this._openedFltr = $("#" + this.id + this._$colType + "_excelDlg");
            }
            this._listsWrap = $("#" + this.id + this._$colType + "_CheckBoxList");
            this._menuWrap = $("#" + this.id + this._$colType + "_MenuItem");
            this._searchBox = this._openedFltr.find(".e-searchbox input");          
            this._setPosition(this._openedFltr, options["position"]);          
            this._openedFltr.fadeIn(300, "easeOutQuad", function () {
            });
            this._isFiltered = this._predicates[this._$key] != undefined && this._predicates[this._$key][this.fName] != undefined;
            this._$format = (this._$colType == "date") ? "{0:MM/dd/yyyy hh:mm:ss}" : this._$format;
            this._processListData();            
            this._actualCount > this._maxCount ? this._openedFltr.find("div.e-status").removeClass("e-hide") : this._openedFltr.find("div.e-status").addClass("e-hide");            
            this._setDisable();            
        },
        closeXFDialog: function (e) {
            if (e != null) {
                var $target = $(e.target);
                if ($target.closest("#"+ this.id + this._$colType + "_CustomFDlg").length)
                    return;
            }            
            if (this._openedFltr) {
                if (!this._openedFltr.hasClass("e-dlgcustom")) {
                    this._openedFltr.fadeOut(300, "easeOutQuad", function () {
                    });
                    this._listsWrap.ejWaitingPopup("hide");
                }
                else {
                    this._openedFltr.ejDialog("close");
                }
            }           
        },
        _setPosition: function (ele, pos) {
            ele.css("position", "absolute");
            ele.css("left", pos["X"]).css("top", pos["Y"]);                      
        },
        _setDisable: function () {
            var $clr = this._menuWrap.find("li[ejfnrole='clearfilter']");
            var $sort = this._menuWrap.find("li[ejfnrole *= 'sort']"), $checked = this._listsWrap.find("input").filter(":checked:not(.e-selectall)").length;
            !this._isFiltered ? $clr.addClass("e-disable-item") : $clr.removeClass("e-disable-item");
            if (this._showSort) {
                var sQM = ej.DataManager(this.sortedColumns).executeLocal(new ej.Query().where("field", "equal", this.fName));
                ( sQM.length && sQM[0]["direction"] == "ascending") ? $sort.filter("[ejfnrole='sortAsc']").addClass("e-disable-item") : $sort.filter("[ejfnrole='sortAsc']").removeClass("e-disable-item");
                (sQM.length && sQM[0]["direction"] == "descending") ? $sort.filter("[ejfnrole='sortDesc']").addClass("e-disable-item") : $sort.filter("[ejfnrole ='sortDesc']").removeClass("e-disable-item");
            }
            if (this._$colType == "boolean")
                this._menuWrap.find("li[aria-haspopup=true]").addClass("e-hide");
            $("#" + this.id + this._$colType + "_OkBtn").ejButton({ enabled: $checked != 0 });
            this._searchBox.val(""); this._searchBox.siblings().addClass("e-search").removeClass("e-cancel");
        },
        _createBtn: function (isCDlg) {
            var id = isCDlg ? this.id + this._$colType + "Custom" : this.id + this._$colType;
            var $divCon = ej.buildTag("div.e-btncontainer e-fields");
            var $div = ej.buildTag("div");
           $div.append(ej.buildTag("input#" + id + "_OkBtn.e-fltrbtn",{},{},{ type : "button" })).append(ej.buildTag("input#" + id + "_CancelBtn.e-fltrbtn",{},{},{ type : "button" }));
            return $divCon.append($div);
        },
        _menuHandler: function (args) {
            var role = $(args.element).attr("ejfnrole");
            var arg = {};
            if (this._showSort && (role == "sortAsc" || role == "sortDesc")) {
                var sOrder, sDetails;
                sOrder = role == "sortAsc" ? "ascending" : "descending";
                sDetails = { field: this.fName, direction: sOrder };
                arg = { originalEvent: args.event, action: "sorting", sortDetails: sDetails };
            }
            else if (role == "clearfilter") {
                var ftrDetails = { field: this.fName, operator: "", value: "", predicate: "or" };
                arg = { originalEvent: args.event, fieldName: this.fName, action: "clearfiltering", filterDetails: ftrDetails };
                var indx = $.inArray(this.fName, this.cFilteredCols);
                indx!=-1 && this.cFilteredCols.splice(indx, 1);
            }
            else if(role == "popup") {
                return;
            }
            else if (role == "operator") {
                this._openCustomFilter(args.text.replace(/ /g ,"").replace("...","").toLowerCase());
            }
            if (role != "operator") {
                this._filterHandler(arg);
                this.closeXFDialog();
            }
        },
        _searchBoxFocus: function (e) {
            var $target = $(e.target);
            if ($target.hasClass("e-cancel")) {
                $target.prev().val("");
                $target.next().addClass("e-cancel")
                $target.addClass("e-search");
                $target.prev().trigger("keyup");
            }
            if (e.type == "focusin") {
                $target.next().addClass("e-cancel");
                $target.next().removeClass("e-search");
            }
        },
        _search: function (e) {
            var enterText = e.target.value, args = {}, parsed, operator, $target = $(e.target);
            parsed = parseFloat(enterText) ? parseFloat(enterText) : enterText;
            operator ="contains" ;
            parsed = (parsed == "" || parsed == undefined) ? undefined : parsed;
            this._processListData({ type: "searching", value: parsed, operator: operator });
            if ($target.val() == "") {
                $target.next().addClass("e-search");
                $target.next().removeClass("e-cancel");
            } else {
                $target.next().addClass("e-cancel");
                $target.next().removeClass("e-search");
            }
        },
        _getLocalizedLabel: function (property) {
            return (ej.ExcelFilter.locale[this._locale] !== undefined && ej.ExcelFilter.locale[this._locale][property] !== undefined) ? ej.ExcelFilter.locale[this._locale][property] : ej.ExcelFilter.locale["en-US"][property];
        },
        _getMenuData: function (type) {
            return type != undefined ? this[type + "MenuOpt"] : [];
        },
        /*CheckBox list*/
        _lsitBoxTemplate: function () {
            var helper = {}, temp = {};            
           helper[this.id + "_getField"] = ej.proxy(this.getField, this);
            helper[this.id + "_getFormat"] = ej.proxy(this.getFormat, this);
            helper[this.id + "_checkBoxState"] = ej.proxy(this._setCheckState, this);
            helper[this.id + "_formatData"] = ej.proxy(this._formatting, this);
            helper[this.id + "_genId"] = this._genCheckID;
            $.views.helpers(helper);            
            var formatData = (this._formatFiltering || this._$colType == "date") ? "{{:~" + this.id + "_formatData(~" + this.id + "_getFormat(),#data[~" + this.id + "_getField()])}}" : "{{:#data[~" + this.id + "_getField()]}}";
            var valueFormat = (this._formatFiltering || this._$colType == "date") ? "{{:~" + this.id + "_formatData(~" + this.id + "_getFormat(),#data[~" + this.id + "_getField()])}}" : "{{:#data[~" + this.id + "_getField()]}}";
            var genID = this.id + this._$colType + "{{:~" + this.id + "_genId()}}";
            temp[this.id + this._$colType + "_listBox_Template"] = "<div class='e-ftrchk'><input type='checkbox' id='" + genID + "' value='" + valueFormat + "' class='e-ftrchk' {{:~" + this.id + "_checkBoxState(#data[~" + this.id + "_getField()])}}/><label class='e-ftrchk' for='" + genID + "'>" + formatData + "</label></div>";
            $.templates(temp);
        },
      
        _processListData: function (params) {
            var result, promise, args = {}, query = new ej.Query(); this._searchCount = 0;
            var predicates = this._predicates[this._$key], pred;
            for (var prop in predicates) {
                if (prop == this.fName)
                    continue;
                var obj = predicates[prop], from = obj["from"];
                if (from)
                    query.skip(from == "top" ? 0 : this._dataSource.length - (this._dataSource.length - obj["take"])).take(obj["take"]);
                else
                    pred = pred != undefined ? pred["and"](obj) : obj;
            }
            args.columnName = this.fName;
            query.requiresCount();
            pred && query.where(pred);
            if (params && params.type == "searching") {
                params.value && query.where(this.fName, params.operator, params.value, !this._matchCase);
                promise = ej.DataManager(this._localJSON).executeLocal(query);
                args.data = this._currentData = promise.result;
                this._totalRcrd = this._searchCount =  promise.count;
                this._setCheckBoxList(args);
            }
            else if (!(this._dataSource instanceof ej.DataManager)) {
                promise = ej.DataManager(this._dataSource).executeLocal(query);
                args.data = this._localJSON = this._currentData = promise.result;
                this._totalRcrd = promise.count;
                this._setCheckBoxList(args);
            }
            else {
                proxy = this;
                this._listsWrap.ejWaitingPopup("show");
                if (!this._reqInProgess) {
                    this._reqInProgess = true;
                    promise = this._dataSource.executeQuery(query);
                    promise.done(function (e) {
                        args.data = proxy._localJSON = proxy._currentData = e.result;
                        proxy._totalRcrd = e.count;
                        proxy._listsWrap.ejWaitingPopup("hide");
                        proxy._setCheckBoxList(args);
                        proxy._reqInProgess = false;
                    });
                }
            }           
        },
        _setCheckBoxList: function (args) {
            if (this._currentData.length != 0) {
                var sorts = ej.distinct(args.data, args.columnName, true);
                var sortedData = ej.mergeSort(sorts, args.columnName), flag = this._isFiltered;
                this._actualCount = sorts.length; sortedData.length = this._maxCount > this._actualCount ?this._actualCount : this._maxCount;
                this._blank = (ej.DataManager(sortedData).executeLocal(new ej.Query().where(ej.Predicate(args.columnName, "equal", null)))) ? "" : this._blank;
                this._listsWrap.find("div:first").html([this._selectAll, $.render[this.id + this._$colType + "_listBox_Template"](sortedData), this._blank].join(""));                
                this._chkList = this._listsWrap.find("input:checkbox").not(".e-selectall"), $inView = this._chkList.slice(0, 20);
                $inView.ejCheckBox({ change: ej.proxy(this._checkHandler, this) });
                $inView.filter(":checked").ejCheckBox({ checked: true });
                this._listsWrap.find(".e-selectall").ejCheckBox({  change: ej.proxy(this._selectAllHandler, this) });
				this._listsWrap.find(".e-selectall").attr("id",this.id + this._$colType + "SelectAll");
				this._listsWrap.find(".e-selectall").closest("span").siblings("label").attr("for", this.id + this._$colType + "SelectAll");
            }
            else
                this._listsWrap.find("div:first").html(ej.buildTag("div.e-ftrchk", this._getLocalizedLabel("NoResult"), {}, {}));
            if (!this._isFiltered || this._actualCount == this._chkList.filter(":checked").length)
                this._listsWrap.find(".e-selectall").ejCheckBox({ checked: true });
            this._listsWrap.ejScroller({ scrollTop:0 }).ejScroller("refresh");
        },
     
        _createLiTag: function ($ul, menuData, isChild) {
            proxy = this;
            $.each(menuData, function (index, obj) {
                var $li = ej.buildTag("li", {}, {}, obj["htmlAttribute"] || (isChild && { "ejfnrole": "operator" }) || {}), $child;
                var apd = (isChild && $.inArray(obj.value,proxy._noDlg)==-1) ? "..." : "";
                var $a = ej.buildTag("a", obj.text + apd, {}, {});
                obj["sprite"] != undefined && $a.append(ej.buildTag("span", {}, {}, { "class": obj["sprite"] }))
                if (obj["child"] != undefined)
                    $child = proxy._createLiTag(ej.buildTag("ul.e-shadow"), obj["child"], true);

                obj["child"] == undefined ? $ul.append($li.append($a)) : $ul.append($li.append($a).append($child))
                if ($.inArray(obj.value || (obj.htmlAttribute && obj.htmlAttribute.ejfnrole), proxy._sepAftr) != -1)
                    $ul.append(ej.buildTag("li.e-separator"));
            });
            return $ul;
        },
        _setCheckState: function (value) {           
            var val = this._$colType == "boolean" ? value.toString() : value;
            if (!this._isFiltered)
                return "checked";            
            else {
                var fC = ej.DataManager(this.filteredColumn).executeLocal(ej.Query().where("field", "equal", this.fName)), fQ;
                fQ = ej.DataManager(fC).executeLocal(ej.Query().where("value", "equal", val));
                return fQ.length != 0 ? "checked" : "";            
            }
        },
        _genCheckID: function () {
            return "CheckBox" + this.getIndex();
        },
        _formatting: function (format, value) {
            if (this._$colType == "date" && (format =="" || !this._formatFiltering))
                format = "{0:MM/dd/yyyy hh:mm:ss}";
            if (value == undefined || value === "" || value == null)
                return "(Blanks)";
            if (this._formatFiltering || this._$colType == "date") {
                var formatter = ej.Grid.prototype.formatting;                
                    return  format!="" ? formatter(format, value):value;                
            } 
        },
        _checkHandler: function (args) {
            $("#" + args.model.id).prop("checked", args.isChecked);
            var $checkedlist = this._listsWrap.find("input").filter(":checked:not(.e-selectall)"), clen = $checkedlist.length;
            var $selectall = this._listsWrap.find(".e-selectall"), canCheck;
            if (clen == this._chkList.length) {
                $selectall.prop("checked", true);
                canCheck = true;
            }
            else {
                $selectall.removeProp("checked");
                canCheck = false;
            }
            $selectall.ejCheckBox({ checked: canCheck });
            $("#" + this.id + this._$colType + "_OkBtn").ejButton({ enabled: clen != 0 });
        },
        _selectAllHandler: function (args) {
            
            if (args.isChecked) {
                this._chkList.filter(":not(:checked)").ejCheckBox({ checked: args.isChecked });
                this._chkList.prop("checked", true);
            }
            else {
                this._chkList.filter(function () { if ($(this).hasClass("e-checkbox") && $(this).prop("checked")) return this; }).ejCheckBox({ checked: args.isChecked });
                this._chkList.removeProp("checked"); this._chkList.removeAttr("checked");
            }
            $("#" + this.id + this._$colType + "_OkBtn").ejButton({ enabled: args.isChecked });
        },
        /*Custom Dialog*/
        _renderCustomFDlg: function (type) {
            var dlgId = this.id + type + "_CustomFDlg";
            if ($("#" + dlgId).length)
                return;
            var $dlg = ej.buildTag("div#" + dlgId + ".e-excelfilter e-dlgcustom");
            $dlg.addClass(this._cssClass);
            this._renderCDlgContent($dlg, type);
            $dlg.ejDialog({ showOnInit: false, enableResize: false, enableModal:true, allowKeyboardNavigation: false, title: this._title, width: 370,content:"#" + this.id, enableRTL:this._ctrlInstance.model.enableRTL , closeIconTooltip:"Close" });
        },

        _renderCDlgContent: function (content, type) {
            var $div1 = ej.buildTag("div.e-dlgfields", "Show rows where", {}, {});
            var $id = this.id + this._$colType;
            var $fset = ej.buildTag("fieldset.e-fieldset");
            var $op1 = ej.buildTag("tr.e-fields"), $op2 = ej.buildTag("tr.e-fields"), $op3 = ej.buildTag("tr.e-fields e-top");
            var $dp1 = ej.buildTag("input#" + $id + "_CustomDrop1"), $dp2 = ej.buildTag("input#" + $id + "_CustomDrop2"), $dp3 = ej.buildTag("input#" + $id + "_CustomDrop3");
            var $in1 = ej.buildTag("input#" + $id + "_CustomValue1.e-ejinputtext", {}, {}, { "type": "text" }), $in2 = ej.buildTag("input#" + $id + "_CustomValue2.e-ejinputtext", {}, {}, { "type": "text" }), $in3 = ej.buildTag("input#" + $id + "_CustomValue3.e-ejinputtext", {}, {}, { "type": "text" });
            var $pred = ej.buildTag("tr.e-predicate");
            var radioAnd = ej.buildTag("input#" + $id + "_CustomAndPredicate", {}, {}, { "type": "radio", "name": "predicate" , "value":"and" }), radioOr = ej.buildTag("input#" + $id + "_CustomOrPredicate", {}, {}, { "type": "radio", "name": "predicate", "value":"or" });
            var matchCase = ej.buildTag("input#" + $id + "_CustomMatchPredicate", {}, {}, { "type": "checkbox" });
            content.append($div1);
            var table = ej.buildTag("table.e-optable");

            $op1.append(ej.buildTag("td.e-operator").append($dp1))
                .append(ej.buildTag("td.e-value").append($in1));
            $pred.append(ej.buildTag("td", {}, {}, {}).append(radioAnd).append(ej.buildTag("label.e-caption", this._getLocalizedLabel("PredicateAnd"), {}, { "for": $id + "_CustomAndPredicate" }))
                .append(radioOr).append(ej.buildTag("label.e-caption", this._getLocalizedLabel("PredicateOr"), {}, { "for": $id + "_CustomOrPredicate" })));
            $op2.append(ej.buildTag("td.e-operator").append($dp2))
                .append(ej.buildTag("td.e-value").append($in2));
            table.append($op1)
            .append($pred)
            .append($op2);
			if(type == "string")
			    $pred.append(ej.buildTag("td", {}, {}, {}).append(matchCase).append(ej.buildTag("label.e-caption", this._getLocalizedLabel("MatchCase"), {}, { "for": $id + "_CustomMatchPredicate" })));
            if (type == "number") {
                $op3.append(ej.buildTag("td.e-operator").append($dp3))
                .append(ej.buildTag("td.e-value").append($in3));
                table.append($op3)
            }
            $fset.append(ej.buildTag("legend"))
                .append(table)
            content.append(ej.buildTag("div.e-dlgfields").append($fset));
            content.append(ej.buildTag("div.e-dlgfields").append(this._createBtn(true)));
            content.appendTo("body");
            var uppertype = type.replace(type.charAt(0), type.charAt(0).toUpperCase());
            $([$dp1, $dp2, $dp3]).ejDropDownList({ fields: { text: "text", value: "value" }, height: 27, width: 120, enableRTL: this._ctrlInstance.model.enableRTL });
            if (type == "number") {
                $([$in1, $in2, $in3]).ejNumericTextbox({ showSpinButton: false, height: "27px", width: "177px", enableRTL: this._ctrlInstance.model.enableRTL });
            }
            else if (type == "date") {
                $([$in1, $in2]).ejDatePicker({ "cssClass": this._ctrlInstance.model.cssClass, height: "27px", width: "177px", "enableRTL": this._ctrlInstance.model.enableRTL });
            }
            else if (type == "string") {
                $([$in1, $in2]).ejAutocomplete({ "cssClass": this._ctrlInstance.model.cssClass, "enableRTL": this._ctrlInstance.model.enableRTL, enableDistinct: true, width:"177px",height:"27px" });
            }
            $([radioAnd, radioOr]).ejRadioButton({ "cssClass": this._ctrlInstance.model.cssClass, "enableRTL": this._ctrlInstance.model.enableRTL });
            matchCase.ejCheckBox({ });
            $("#" + $id + "Custom_OkBtn").ejButton({ text: this._getLocalizedLabel("Ok"), showRoundedCorner: true, width:"23%", click: ej.proxy(this._fltrBtnHandler, this), enabled: true });
            $("#" + $id + "Custom_CancelBtn").ejButton({ text: this._getLocalizedLabel("Cancel"), showRoundedCorner: true, width: "23%", click: ej.proxy(this.closeXFDialog, this) })
        },
                     
        _openCustomFilter: function (operator) {
            var oper = operator != "top10" ? this._$colType: operator;
            var type = oper.replace(oper.charAt(0), oper.charAt(0).toUpperCase());
            var id = this.id + this._$colType;
            this.closeXFDialog();
            this._openedFltr = $("#" + id + "_CustomFDlg");
            this._openedFltr.ejDialog("open");            
            this._openedFltr.find("legend").html(this._displayName);            
            var sl = (["Number","Date"].indexOf(type) != -1) ? 6 : 5;
            $("#" + id + "_CustomDrop1").ejDropDownList({ dataSource: this._getLocalizedLabel(type + "MenuOptions").slice(0,sl) });
            $("#" + id + "_CustomDrop2").ejDropDownList({ dataSource: this._getLocalizedLabel(type + "MenuOptions").slice(0, sl) });
			if (this._$colType == "number") {
                var decimalPlace = 0;
                if(operator == "top10"){
                    this._openedFltr.find(".e-optable tr").not(".e-top").addClass("e-hide");   
                    this._openedFltr.find(".e-optable tr.e-top").removeClass("e-hide");
                    $("#" + id + "_CustomDrop3").ejDropDownList({ dataSource: this._getLocalizedLabel(type + "MenuOptions") });
                }
                else{                   
                    decimalPlace = parseInt(this._$format.split("{0:C")[1]) || parseInt(this._$format.split("{0:c")[1]);                    
                    this._openedFltr.find(".e-optable tr.e-top").addClass("e-hide");
                    this._openedFltr.find(".e-optable tr").not(".e-top").removeClass("e-hide"); 
                    $("#" + id + "_CustomValue1").ejNumericTextbox("model.decimalPlaces", decimalPlace);
                    $("#" + id + "_CustomValue2").ejNumericTextbox("model.decimalPlaces", decimalPlace);
                }
            }
            else {
                this._openedFltr.find(".e-optable tr.e-top").addClass("e-hide");
                this._openedFltr.find(".e-optable tr").not(".e-top").removeClass("e-hide");
            }
            if(this._$colType == "string")
                this._openedFltr.find(".e-autocomplete").ejAutocomplete({
                    fields: { text: this.fName }, dataSource: this._dataSource, focusIn: function (args) {
                        var type = this.element.closest("td").siblings().find(".e-dropdownlist").ejDropDownList("getSelectedValue");
                        this.model.filterType = type == "" ? this.model.filterType : type;
                    }
                });
            this._setFilteredData(id, operator);
        },

        _setFilteredData: function ($id, op) {
            var indx = $.inArray(this.fName, this.cFilteredCols);
            var fQM = [], optrs = [], fLen;
            var between = ["greaterthanorequal", "lessthanorequal"];
            if (op == "top10")
                return;
            if (indx != -1)
                fQM = ej.DataManager(this._ctrlInstance.model.filterSettings.filteredColumns).executeLocal(ej.Query().where("field", "equal", this.fName));
            fLen = fQM.length;
            var drops = this._openedFltr.find(".e-dropdownlist"), inputs = this._openedFltr.find(".e-value input:visible"), $pred = this._openedFltr.find(".e-predicate");            
            if (indx != -1 && fLen && fQM[0]["from"]!=undefined)
                optrs = [op, ""];
            else if (indx != -1 && fLen && fQM[0]["isCustom"])
                optrs = [fQM[0]["operator"], fQM[1] ? fQM[1]["operator"] : ""];
            else if (op == "between" || op == "customfilter")
                optrs = op != "customfilter" ? between : ["", ""];           
            else
                optrs = [op, ""];
           
            for (var i = 0; i < (indx != -1 ? fLen : 2) ; i++) {
                var opt = /\D*/.exec(optrs[i])[0];
                $(drops[i]).prop("value", opt);
                $(drops[i]).ejDropDownList("setSelectedValue", opt);
                if ($(inputs[i]).hasClass("e-datepicker"))   
                    $("#" + inputs[i].id ).ejDatePicker("model.value", (indx != -1 && fLen && fQM[i]["isCustom"]) ? fQM[i]["value"] : null );                                
                else
                    $(inputs[i]).val((indx != -1 && fLen && fQM[i]["isCustom"]) ? fQM[i]["value"] : "");
                var $pre = (indx != -1 && fLen && fQM[i]["isCustom"]) ? fQM[i]["condition"] : "and";
				if(this._$colType == "string"){
					var $match = (indx != -1 && fLen && fQM[i]["isCustom"]) ? fQM[i]["matchcase"] : false;
					var $active = $pred.find("input[type='checkbox']").attr("checked", $match).parent();
					$match == false ? $active.find(".e-chk-image").toggleClass("e-chk-activeicon",false) : $active.find(".e-chk-image").toggleClass("e-chk-activeicon",true);
				}
                $pred.find("input[type='radio']").filter("[value=" + $pre + "]").prop("checked" , true).parent().find(".e-rad-icon").removeClass("e-rad-select").addClass("e-rad-active");
				var $select= $pred.find("input[type='radio']").not("[value=" + $pre + "]").parent().find(".e-rad-icon");
				$select.hasClass("e-rad-active") ? $select.removeClass("e-rad-active").addClass("e-rad-select") : $select;
            }            
        },
        _setDateObject: function (filterObject) {
            if (filterObject.value != null) {
                var $prevDate = new Date(filterObject.value.setDate(filterObject.value.getDate() - 1));
                var $nextDate = new Date(filterObject.value.setDate(filterObject.value.getDate() + 2));
                var $prevObj = $.extend({}, filterObject);
                var $nextObj = $.extend({}, filterObject);
                $prevObj.value = $prevDate;
                $nextObj.value = $nextDate;
                if (filterObject.operator == "equal") {
                    $prevObj.operator = "greaterthan";
                    $prevObj.predicate = "and";
                    $nextObj.operator = "lessthan";
                    $nextObj.predicate = "and";
                } else {
                    $prevObj.operator = "lessthanorequal";
                    $prevObj.predicate = "or";
                    $nextObj.operator = "greaterthanorequal";
                    $nextObj.predicate = "or";
                }
                var predicate = ej.Predicate($prevObj.field, $prevObj.operator, $prevObj.value, $prevObj.matchcase);
                predicate = predicate[$nextObj.predicate]($nextObj.field, $nextObj.operator, $nextObj.value, $nextObj.matchcase);
                filterObject.ejpredicate = predicate;
                return filterObject;
            }
            else
                return filterObject;
        },
        _getCDlgFields: function () {
            var dropDowns = this._openedFltr.find(".e-dropdownlist:visible");
            var $match = this._openedFltr.find(".e-checkbox"), matchcase,valueColl = [];
            var $inputs = this._openedFltr.find(".e-value input:visible");
            for (var i = 0,len = dropDowns.length; i < len; i++) {
                var dvalue = $(dropDowns[i]).ejDropDownList("getSelectedValue"), value;
                if (this._$colType == "number")
                    value = $inputs.eq(i).ejNumericTextbox("model.value");
                if (this._$colType == "string") {
                    value = $inputs.eq(i).val();
                    matchcase = $match.is(":checked");
                }
                if (this._$colType == "date") 
                    value = $inputs.eq(i).ejDatePicker("model.value");
                if (dvalue == "top" || dvalue == "bottom") {
                    valueColl.push({ field: this.fName, value: "", predicate: this._openedFltr.find(".e-predicate input[type='radio']:checked").val(), operator:"notequal", matchcase: matchcase, isCustom: true, take: value , from:dvalue });
                }
                else if (dvalue != "") {
                    var filterObj = { field: this.fName, value: value, predicate: this._openedFltr.find(".e-predicate input[type='radio']:checked").val(), operator: dvalue.toLowerCase(), matchcase: matchcase, isCustom: true };
                    valueColl.push((this._$colType == "date" && ["equal", "notequal"].indexOf(dvalue.toLowerCase()) != -1) ? this._setDateObject(filterObj) : filterObj);
                }
                else
                    return valueColl;
            }
            return valueColl;
        },
      
        _fltrBtnHandler: function (args) {
            var matchcase, valColl = [], arg = {}, predicate, fObj;
            if (!this._openedFltr.hasClass("e-dlgcustom")) {
                if (!this._isFiltered && this._listsWrap.find(".e-selectall").ejCheckBox("model.checked") && this._searchCount == 0) 
                    return this.closeXFDialog();                                  
                var checked = this._listsWrap.find("input").filter(":checked:not(.e-selectall)");
                var len = checked.length, cVal;
                var optr = this._colType == "string" ? "startswith" : "equal";
                for (var i = 0; i < len; i++) {
                    cVal = $(checked[i]).hasClass("e-blanks") ? null : checked[i].value;
                     fObj = { field: this.fName, value: cVal, predicate: "or", operator: optr, matchcase: this._matchCase };
                     if (this._$colType == "date") {
                         fObj.value = Globalize.parseDate(checked[i].value, this._$format.replace("{0:", "").replace("}", ""));
                         valColl.push(this._setDateObject(fObj));
                     }
                     else
                         valColl.push(fObj);
                }
            }
            else {
                valColl = this._getCDlgFields();
                $.inArray(this.fName, this.cFilteredCols) == -1 && this.cFilteredCols.push(this.fName);
            }
            var firstVal = valColl[0], isTake = firstVal["from"];
            if (firstVal.value != null) {
                predicate = firstVal["ejpredicate"] ? firstVal["ejpredicate"] : ej.Predicate(firstVal.field, firstVal.operator, firstVal.value, !firstVal.matchcase);
                for (var j = 1, jlen = valColl.length; j < jlen; j++) {
                    predicate = valColl[j].ejpredicate != undefined ? predicate[valColl[j].predicate](valColl[j].ejpredicate) : predicate[valColl[j].predicate](valColl[j].field, valColl[j].operator, valColl[j].value, !valColl[j].matchcase);
                }
                arg = { action: "filtering", filterCollection: valColl, fieldName: this.fName, ejpredicate: predicate };
                if (this._predicates[this._$key] == undefined)
                    this._predicates[this._$key] = {};
                this._predicates[this._$key][this.fName] = !isTake ? predicate : { from: firstVal["from"], take: firstVal["take"] };
                this.closeXFDialog();
                this._filterHandler(arg);
            }
        },
        _virtualize: function (e) {
            var chks = this._chkList.not(".e-checkbox").filter(function () { if (this.offsetTop > e.scrollTop - this.offsetHeight && e.scrollTop + 200 > this.offsetTop + this.offsetHeight) return this; });
            if (chks.length == 0) return;
            chks.filter(":checked").ejCheckBox({ checked: true }); chks.filter(":not(:checked)").ejCheckBox({ checked: false });
            chks.ejCheckBox({ change: ej.proxy(this._checkHandler, this) });
        },
        resetExcelFilter: function (template) {
            var _i, _type, _id = this.id,_len, _$id;
            this._predicates = [];
            this._isFiltered = false;
            this.cFilteredCols = [];
            for (_i = 0, _len = this._posType.length; _i < _len; _i++) {
                _type = this._posType[_i], _$id = _id + _type;
                var cDlg = $("#" + _$id + "_CustomFDlg");               
                $("#" + _$id + "_CheckBoxList").ejWaitingPopup("destroy"); $("#" + _$id + "_excelDlg").remove();
                cDlg.find(".e-dropdownlist").ejDropDownList("destroy");
                cDlg.find(".e-button").ejButton("destroy");
                if (_type == "string" || _type == "boolean") cDlg.find(".e-autocomplete").ejAutocomplete("destroy");
                if (_type == "number") cDlg.find(".e-numerictextbox").ejNumericTextbox("destroy");
                if (_type == "date") cDlg.find(".e-datepicker").ejDatePicker("destroy");
                cDlg.ejDialog("destroy");
                $("#" + _$id + "_CustomFDlg").remove();
            }
        },
        _wireEvents: function () {
            this._ctrlInstance._on(this._dialogContainer, "focus click", ".e-searchbox", ej.proxy(this._searchBoxFocus, this));
            this._ctrlInstance._on(this._dialogContainer, "keyup", ".e-searchbox input", ej.proxy(this._search, this));
        }
    };

    ej.ExcelFilter.locale = ej.ExcelFilter.locale || {};
    ej.ExcelFilter.locale["en-US"] = {
        SortNoSmaller: "Sort Smallest to Largest",
        SortNoLarger: "Sort Largest to Smallest",
        SortTextAscending: "Sort A to Z",
        SortTextDescending: "Sort Z to A",
        SortDateOldest: "Sort by Oldest",
        SortDateNewest:"Sort by Newest",
        ClearFilter: "Clear Filter",
        NumberFilter: "Number Filters",
        TextFilter: "Text Filters",
        DateFilter: "Date Filters",
        StringMenuOptions: [{ text: "Equal", value: "equal" }, { text: "Not Equal", value: "notequal" }, { text: "Starts With", value: "startswith" }, { text: "Ends With", value: "endswith" }, { text: "Contains", value: "contains" }, { text: "Custom Filter", value: "customfilter" }],
        NumberMenuOptions: [{ text: "Equal", value: "equal" }, { text: "Not Equal", value: "notequal" }, { text: "Less Than", value: "lessthan" }, { text: "Less Than Or Equal", value: "lessthanorequal" }, { text: "Greater Than", value: "greaterthan" }, { text: "Greater Than Or Equal", value: "greaterthanorequal" }, { text: "Between", value: "between" }, { text: "Custom Filter", value: "customfilter" }],
        DateMenuOptions: [{ text: "Equal", value: "equal" }, { text: "Not Equal", value: "notequal" }, { text: "Less Than", value: "lessthan" }, { text: "Less Than Or Equal", value: "lessthanorequal" }, { text: "Greater Than", value: "greaterthan" }, { text: "Greater Than Or Equal", value: "greaterthanorequal" }, { text: "Between", value: "between" }, { text: "Custom Filter", value: "customfilter" }],
        Top10MenuOptions: [{ text: "Top", value: "top" }, { text: "Bottom", value: "bottom" }],
        title:"Custom Filter",
        PredicateAnd: "AND",
        PredicateOr: "OR",
        Ok: "OK",
        MatchCase: "Match Case",
        Cancel: "Cancel",
        NoResult: "No Matches Found",
        CheckBoxStatusMsg: "Not all items showing"
    };

})(jQuery, Syncfusion);;
(function ($, ej, undefined) {
	ej.scheduleFeatures = ej.scheduleFeatures || {};

	var resourceHeadTemplate = function () {
	    this._resHeadTemplate = (
        "{{for resourceName}}{{for ~cols=colspan}}<td class='e-resourcecells 'style='height:27px;' colspan='{{:~cols}}'> {{for ~resource=resourceNam}}<div align='center'>{{if userResTemplId == true}}{{:userResHeader}}{{else}}{{:~resource}}{{/if}}</div>{{/for}}  </td>{{/for}}{{/for}}" +
        "<td class='e-emptytd e-headerresourcecells'></td>");
	    return this._resHeadTemplate;
	};

	ej.scheduleFeatures.resources = {

	    __getMultipleResourceCellsCount: function (count) {
	        if (this.model.group.resources.length === 1) {
	            var index;
	            for (var i = 0; i < this.model.group.resources.length; i++) {
	                index = this._findResourceIndex(this.model.resources, "name", this.model.group.resources[i]);
	            }
	            resCount = this._resourceInfo[index].dataSource.length; // this.model.resources[index].dataSource.length;
	        }
	        else {
	            var newResCollection = this._getResourceCollection(), resources = [];
	            var groupData, prevGroupDataId, query, resApp;
	            for (var j = 0; j < newResCollection.length; j++) {
	                resources = [];
	                if (j == 0) {
	                    for (var i = 0; i < newResCollection[j].dataSource.length; i++) {
	                        resources.push({ resourceNam: newResCollection[j].dataSource[i][newResCollection[j].text] });
	                    }
	                }
	                else {
	                    if (j > 1) {
	                        newResCollection[j - 1].dataSource = [];
	                        newResCollection[j - 1].dataSource = this.res;
	                    }
	                    this.res = [];
	                    groupData = ej.group(newResCollection[j].dataSource, newResCollection[j].groupId);
	                    prevGroupDataId = ej.group(newResCollection[j - 1].dataSource, newResCollection[j - 1].id);
	                    this._resourceManager1 = groupData instanceof ej.DataManager ? groupData : ej.DataManager(groupData);
	                    for (var l = 0; l < prevGroupDataId.length; l++) {
	                        for (var k = 0; k < prevGroupDataId[l].items.length; k++) {
	                            query = new ej.Query().where("key", ej.FilterOperators.equal, prevGroupDataId[l].items[k][newResCollection[j].id]);
	                            resApp = this._resourceManager1.executeLocal(query);
	                            if (resApp.length > 0) {
	                                for (var a = 0; a < resApp[0].items.length; a++) {
	                                    resources.push({ resourceNam: resApp[0].items[a][newResCollection[j].text] });
	                                    this.res.push(resApp[0].items[a]);
	                                }
	                            }
	                        }
	                    }
	                }
	            }
	            resCount = resources.length;
	        }
	        count = count * resCount; // this.res1.length;
	        return { resCount: resCount, count: count };
	    },
	    /**
                * To configure the dataSource values to the resource fields.
                * @private
                */
	    _initResourceData: function (fields, i) {
	        var query = this._columnToSelect(fields);
	        var proxy = this;
	        var queryPromise = fields.dataSource.executeQuery(query);
	        queryPromise.done(function (e) {
	            fields.dataSource = e.result;
	            proxy._resourceInfo.push(fields);
	            if (i == proxy.model.resources.length - 1)
	                proxy._renderInitSchedule();
	        });
	    },

	    _showMultipleResourceDetails: function (temp) {
	        var prevdata, query, resApp1, fieldCollection = [];
	        if (this.model.resources.length > 0) {
	            var newResCollection = this._getResourceCollection();
	            for (var i = 0; i < newResCollection.length; i++) {
	                this._appointmentAddWindow.find('.owner_' + i).ejAutocomplete("clearText");
	            }
	            for (var j = 0; j < this.render_Resources.length - 1; j++) {
	                prevdata = this.render_Resources[j + 1] instanceof ej.DataManager ? this.render_Resources[j + 1] : ej.DataManager(this.render_Resources[j + 1]);
	                query = new ej.Query().where(newResCollection[j + 1].groupId, ej.FilterOperators.equal, temp[this.model.appointmentSettings.resourceFields.toString().split(',')[j]]);
	                resApp1 = prevdata.executeLocal(query);
	                fieldCollection = [];
	                for (var k = 0; k < resApp1.length; k++) {

	                    fieldCollection.push({ text: resApp1[k][newResCollection[j + 1].text], value: resApp1[k][newResCollection[j + 1].id], color: resApp1[k][newResCollection[j + 1].color] });
	                }
	                this._appointmentAddWindow.find('.owner_' + (j + 1)).ejAutocomplete("clearText");
	                this._appointmentAddWindow.find('.owner_' + (j + 1)).ejAutocomplete("option", "dataSource", fieldCollection);

	            }
	            for (var i = 0; i < newResCollection.length; i++) {
	                this._appointmentAddWindow.find('.owner_' + i).data("ejAutocomplete").selectValueByKey(temp[this.model.appointmentSettings.resourceFields.toString().split(',')[i]]);
	            }
	        }
	    },

	    _renderAppWindowResources: function (control) {
	        var newResCollection = this._getResourceCollection(), fieldCollection = [];
	        for (var i = 0; i < newResCollection.length; i++) {
	            fieldCollection = [];
	            for (var j = 0; j < this.render_Resources[i].length; j++) {
	                fieldCollection.push({ text: this.render_Resources[i][j][newResCollection[i].text], value: this.render_Resources[i][j][newResCollection[i].id], color: this.render_Resources[i][j][newResCollection[i].color] });
	            }
	            if (!this.model.resources[i].allowMultiple)
	                control.find('.owner_' + i).ejAutocomplete({
	                    enableRTL: this.model.enableRTL,
	                    showPopupButton: true,
	                    width: "100%", cssClass: this.model.cssClass,
	                    multiSelectMode: "none",
	                    dataSource: fieldCollection, fields: { text: "text", key: "value" }, change: this._onResourceChange,
	                    template: this.resourceTemplate
	                });
	            else
	                control.find('.owner_' + i).ejAutocomplete({
	                    enableRTL: this.model.enableRTL,
	                    showPopupButton: true,
	                    width: "100%", cssClass: this.model.cssClass,
	                    multiSelectMode: "visualmode",
	                    dataSource: fieldCollection, fields: { text: "text", key: "value" }, change: this._onResourceChange,
	                    template: this.resourceTemplate
	                });
	        }
	    },

	    _onResourceChange: function (e) {
	        var scheduleDiv = $("#" + this._id.toString().split("_")[0]), proxy = scheduleDiv.ejSchedule("instance");
	        var controlId = parseInt(this._id.toString().split("_")[2]);
	        var controlMode = proxy._appointmentAddWindow.find('.owner_' + controlId).ejAutocomplete("option", "multiSelectMode")
	        var selectedItems = this.getSelectedItems();
	        var resourceCollection = proxy._getResourceCollection();
	        if (controlId < proxy.render_Resources.length - 1 && selectedItems.length > 0) {
	            var j = controlId + 1;
	            var data = proxy.render_Resources[j - 1] instanceof ej.DataManager ? proxy.render_Resources[j - 1] : ej.DataManager(proxy.render_Resources[j - 1]);
	            var prevdata = proxy.render_Resources[j] instanceof ej.DataManager ? proxy.render_Resources[j] : ej.DataManager(proxy.render_Resources[j]);
	            if (controlMode == "none") {
	                if (!ej.isNullOrUndefined(e.value)) {
	                    var textCheck = new ej.Query().where(resourceCollection[j - 1].text, ej.FilterOperators.equal, e.value);
	                    var result = data.executeLocal(textCheck);
	                }
	                var currentVal = ((!e.value) || ej.isNullOrUndefined(e.value) || result.length <= 0) ? selectedItems[0].value : selectedItems[1].value;
	                if (!ej.isNullOrUndefined(currentVal)) {
	                    var query = new ej.Query().where(resourceCollection[j - 1].id, ej.FilterOperators.equal, currentVal);
	                    var resApp = data.executeLocal(query);

	                    var query = new ej.Query().where(resourceCollection[j].groupId, ej.FilterOperators.equal, resApp[0][resourceCollection[j - 1].id]);
	                    var resApp1 = prevdata.executeLocal(query);
	                    var fieldCollection = [];
	                    for (var k = 0; k < resApp1.length; k++) {
	                        fieldCollection.push({ text: resApp1[k][resourceCollection[j].text], value: resApp1[k][resourceCollection[j].id], color: resApp1[k][resourceCollection[j].color] });
	                    }
	                    proxy._appointmentAddWindow.find('.owner_' + j).ejAutocomplete("clearText");
	                    proxy._appointmentAddWindow.find('.owner_' + j).ejAutocomplete("option", "dataSource", fieldCollection); //.data("ejAutocomplete").selectValueByKey(resApp1[0].id);
	                    proxy._appointmentAddWindow.find('.owner_' + j).data("ejAutocomplete").selectValueByKey(resApp1[0][resourceCollection[j].id]);
	                    selectedItems = resApp1[0][resourceCollection[j].id];
	                }
	            }
	            else {
	                var predicate;
	                predicate = ej.Predicate(resourceCollection[j - 1].id, ej.FilterOperators.equal, selectedItems[0].value);
	                for (var i = 1; i < selectedItems.length; i++) {
	                    predicate = predicate["or"](resourceCollection[j - 1].id, ej.FilterOperators.equal, selectedItems[i].value);
	                }
	                var query = new ej.Query().where(predicate);
	                var resApp = data.executeLocal(query);

	                if (resApp.length > 0) {
	                    var prev_predicate;
	                    prev_predicate = ej.Predicate(resourceCollection[j].groupId, ej.FilterOperators.equal, resApp[0][resourceCollection[j - 1].id]);
	                    for (var i = 1; i < resApp.length; i++) {
	                        prev_predicate = prev_predicate["or"](resourceCollection[j].groupId, ej.FilterOperators.equal, resApp[i][resourceCollection[j - 1].id]);
	                    }
	                    var query = new ej.Query().where(prev_predicate);
	                    var resApp1 = prevdata.executeLocal(query);

	                    var fieldCollection = [];
	                    for (var k = 0; k < resApp1.length; k++) {
	                        fieldCollection.push({ text: resApp1[k][resourceCollection[j].text], value: resApp1[k][resourceCollection[j].id], color: resApp1[k][resourceCollection[j].color] });
	                    }
	                    proxy._appointmentAddWindow.find('.owner_' + j).ejAutocomplete("clearText");
	                    proxy._appointmentAddWindow.find('.owner_' + j).ejAutocomplete("option", "dataSource", fieldCollection); //.data("ejAutocomplete").selectValueByKey(resApp1[0].id);
	                    proxy._appointmentAddWindow.find('.owner_' + j).data("ejAutocomplete").selectValueByKey(resApp1[0][resourceCollection[j].id]);
	                    selectedItems.push(resApp1[0][resourceCollection[j].id]);
	                }


	            }
	        }
	    },

	    _getResourceCollection: function () {
	        var resIndexCollection = [], index;
	        if (!ej.isNullOrUndefined(this.model.resources)) {
	            if (!ej.isNullOrUndefined(this.model.group)) {
	                for (var i = 0; i < this.model.group.resources.length; i++) {
	                    index = this._findResourceIndex(this.model.resources, "name", this.model.group.resources[i]);
	                    resIndexCollection.push({ index: index, name: this.model.resources[i].name });
	                }
	                resIndexCollection = resIndexCollection.sort(function (a, b) {
	                    var x = a.index; var y = b.index;
	                    return ((x < y) ? -1 : ((x > y) ? 1 : 0));
	                });
	            }
	            else {
	                for (var i = 0; i < this.model.resources.length; i++) {
	                    resIndexCollection.push({ index: i });
	                }
	            }
	            var newResCollection = [];
	            for (var k = 0; k < resIndexCollection.length; k++) {
	                newResCollection.push(this._resourceInfo[resIndexCollection[k].index]);
	            }
	            return newResCollection;
	        }

	    },

	    _renderMultipleResourceHeaderTemplate: function (tbody, cols) {
	        this.res1 = []; this.render_Resources = []; this.level_Resources = [];
	        var view = this.currentView();
	        var viewColCount = view == "day" ? 1 : view == "week" || view == "month" ? 7 : 5;
	        if (!ej.isNullOrUndefined(this.model.group)) {
	            var userResTemplId = (this.model.resourceHeaderTemplateId) ? true : false;
	            if (this.model.group.resources.length === 1) {
	                var index;
	                for (var i = 0; i < this.model.group.resources.length; i++) {
	                    index = this._findResourceIndex(this.model.resources, "name", this.model.group.resources[i]);
	                }
	                var resources = [];
	                for (var i = 0; i < this._resourceInfo[index].dataSource.length; i++) {
	                    resources.push({ resourceNam: this._resourceInfo[index].dataSource[i][this._resourceInfo[index].text], colspan: viewColCount, userResHeader: this._getResourceHeadTemplate({ name: this._resourceInfo[index].dataSource[i][this._resourceInfo[index].text], classname: "e-childnode", id: this._resourceInfo[index].dataSource[i][this._resourceInfo[index].id], color: this._resourceInfo[index].dataSource[i][this._resourceInfo[index].color] }), userResTemplId: userResTemplId });
	                    this.res1.push(this._resourceInfo[index].dataSource[i]);
	                    this.level_Resources.push(this.res1[i]);
	                }
	                this.render_Resources.push(this.res1);
	                if (this.model.orientation == "vertical" && tbody !== null && cols != null) {
	                    leftCellwidth = view == "month" ? 50 : 56;
	                    var resourceHeaderHtml = this.resourceHeadTemplate.render({ cols: cols, resourceName: resources, leftWidth: leftCellwidth });
	                    tbody.append(ej.buildTag('tr.e-headerbar').append(resourceHeaderHtml));
	                }
	            }
	            else {
	                var newResCollection = this._getResourceCollection();
	                var res = [], resources = [], prevGroupDataId, query, resApp, tres = [], group, leftCellwidth, resourceHeaderHtml;
	                for (var j = 0; j < newResCollection.length; j++) {
	                    resources = []; this.level_Resources = [];
	                    if (j >= 0 && j < newResCollection.length - 1) {
	                        prevGroupDataId = ej.group(newResCollection[j + 1].dataSource, newResCollection[j + 1].groupId);
	                        this._resourceManager1 = prevGroupDataId instanceof ej.DataManager ? prevGroupDataId : ej.DataManager(prevGroupDataId);

	                        for (var i = 0; i < newResCollection[j].dataSource.length; i++) {
	                            query = new ej.Query().where("key", ej.FilterOperators.equal, newResCollection[j].dataSource[i][newResCollection[j].id]);
	                            resApp = this._resourceManager1.executeLocal(query);
	                            if (resApp.length > 0) {
	                                for (var a = 0; a < resApp[0].items.length; a++) {
	                                    if (newResCollection[j].dataSource[i].id == resApp[0].items[a].groupId) {
	                                        res.push(resApp[0].items[a]);
	                                        (j == newResCollection.length - 2) ? this.res1.push(resApp[0].items[a]) : this.res1 = [];
	                                    }
	                                }
	                            }
	                            tres = [];
	                            for (m = j + 1; m < newResCollection.length; m++) {
	                                group = ej.group(newResCollection[m].dataSource, newResCollection[m].groupId);
	                                this._resourceData = group instanceof ej.DataManager ? group : ej.DataManager(group);
	                                if (m > j + 1) {
	                                    this.tres = [];
	                                    for (var l = 0; l < tres.length; l++) {
	                                        query = new ej.Query().where("key", ej.FilterOperators.equal, tres[l].id);
	                                        resApp = this._resourceData.executeLocal(query);
	                                        if (resApp.length > 0) {
	                                            for (var a = 0; a < resApp[0].items.length; a++) {
	                                                this.tres.push(resApp[0].items[a]);
	                                            }
	                                        }
	                                    }
	                                } else {
	                                    query = new ej.Query().where("key", ej.FilterOperators.equal, newResCollection[m - 1].dataSource[i].id);
	                                    resApp = this._resourceData.executeLocal(query);
	                                    if (resApp.length > 0) {
	                                        for (var a = 0; a < resApp[0].items.length; a++) {
	                                            tres.push(resApp[0].items[a]);
	                                        }
	                                    }
	                                }

	                            }

	                            if (tres.length > 0) {
	                                if (j == 0 && !ej.isNullOrUndefined(this.tres))
	                                    resources.push({ resourceNam: newResCollection[j].dataSource[i][newResCollection[j].text], colspan: this.tres.length * viewColCount, userResHeader: this._getResourceHeadTemplate({ name: newResCollection[j].dataSource[i][newResCollection[j].text], classname: "e-childnode", id: newResCollection[j].dataSource[i][newResCollection[j].id], color: newResCollection[j].dataSource[i][newResCollection[j].color] }), userResTemplId: userResTemplId });
	                                else
	                                    resources.push({ resourceNam: newResCollection[j].dataSource[i][newResCollection[j].text], colspan: tres.length * viewColCount, userResHeader: this._getResourceHeadTemplate({ name: newResCollection[j].dataSource[i][newResCollection[j].text], classname: "e-parentnode", id: newResCollection[j].dataSource[i][newResCollection[j].id], color: newResCollection[j].dataSource[i][newResCollection[j].color] }), userResTemplId: userResTemplId });
	                                this.level_Resources.push(newResCollection[j].dataSource[i]);
	                            }
	                            res = [];
	                        }
	                    }
	                    else {
	                        for (var i = 0; i < this.res1.length; i++) {
	                            resources.push({ resourceNam: this.res1[i][newResCollection[j].text], colspan: viewColCount, userResHeader: this._getResourceHeadTemplate({ name: this.res1[i][newResCollection[j].text], classname: "e-childnode", id: this.res1[i][newResCollection[j].id], color: this.res1[i][newResCollection[j].color] }), userResTemplId: userResTemplId });
	                            this.level_Resources.push(this.res1[i]);
	                        }
	                    }
	                    this.render_Resources.push(this.level_Resources);
	                    if (this.model.orientation == "vertical" && tbody !== null && cols != null) {
	                        leftCellwidth = view == "month" ? 50 : 56;
	                        resourceHeaderHtml = this.resourceHeadTemplate.render({ cols: cols, resourceName: resources, leftWidth: leftCellwidth });
	                        tbody.append(ej.buildTag('tr.e-headerbar').append(resourceHeaderHtml));
	                    }
	                }
	            }

	        }
	        else {
	            for (var k = 0; k < this.model.resources.length; k++) {
	                this.render_Resources.push(this.model.resources[k].resourceSettings.dataSource);
	            }
	        }
	        return tbody;
	    },

	    _getMultipleResourceColsCount: function (cols) {
	        if (this.model.group.resources.length === 1) {
	            for (var i = 0; i < this.model.group.resources.length; i++) {
	                index = this._findResourceIndex(this.model.resources, "name", this.model.group.resources[i]);
	            }
	            if (this._resourceInfo[index].dataSource.length > 0) {
	                cols = cols * this._resourceInfo[index].dataSource.length;
	            }
	        }
	        else {
	            var newResCollection = this._getResourceCollection();
	            if (newResCollection[newResCollection.length - 1].dataSource.length > 0) {
	                cols = cols * this.res1.length;
	            }
	        }
	        return cols;
	    },

	    _getMultipleResourceAppList: function (app, dateCount, appointmentsList, res, day) {
	        if ((Globalize.format(new Date(appointmentsList[app][this.model.appointmentSettings["startTime"]]), this._pattern.d) <= Globalize.format(new Date(dateCount[day]), this._pattern.d)) && (appointmentsList[app][this.model.appointmentSettings.resourceFields.toString().split(',')[this.model.resources.length - 1]] == this.res1[res][this.model.resources[this.model.resources.length - 1].resourceSettings.id]) && (Globalize.format(new Date(appointmentsList[app][this.model.appointmentSettings["endTime"]]), this._pattern.d) >= Globalize.format(new Date(dateCount[day]), this._pattern.d)) && (Globalize.format(new Date(appointmentsList[app][this.model.appointmentSettings["startTime"]]), this._pattern.d) == Globalize.format(new Date(appointmentsList[app][this.model.appointmentSettings["endTime"]]), this._pattern.d)) && (appointmentsList[app][this.model.appointmentSettings["allDay"]] === false) && (Globalize.format(new Date(appointmentsList[app][this.model.appointmentSettings["startTime"]]), this._pattern.d) <= Globalize.format(new Date(appointmentsList[app][this.model.appointmentSettings["endTime"]]), this._pattern.d))) {
	            var id = this._processed[app].ParentId;
	            var query = new ej.Query().where(this.model.appointmentSettings["id"], ej.FilterOperators.equal, parseInt(id));
	            var list = this._currentAppointmentData;
	            var templist = new ej.DataManager(list).executeLocal(query);
	            if (this.currentView() == "day") {
	                (((new Date(templist[0][this.model.appointmentSettings["endTime"]]) - new Date(templist[0][this.model.appointmentSettings["startTime"]])) / 3600000) > 24) ? this._setAllDayPos(appointmentsList[app], day, res) : this._appList.push(appointmentsList[app]);
	            }
	            else { this._appList.push(appointmentsList[app]); }
	        }
	    },

	    _getMultipleResourceAlldayApp: function (renderDate, day, res, i) {
	        if ((Globalize.format(new Date(renderDate[day]), this._pattern.d) == Globalize.format(new Date(this._processed[i][this.model.appointmentSettings["startTime"]]), this._pattern.d)) && (this._processed[i][this.model.appointmentSettings.resourceFields.toString().split(',')[this.model.resources.length - 1]] == this.res1[res][this.model.resources[this.model.resources.length - 1].resourceSettings.id]) &&
				(this._processed[i][this.model.appointmentSettings["allDay"]] === true || (((new Date(this._processed[i][this.model.appointmentSettings["endTime"]]) - new Date(this._processed[i][this.model.appointmentSettings["startTime"]])) / 3600000) > 24))) {
	            //level = this._getOverlapCount(this._processed[i], day, renderDate);
	            this._setAllDayPos(this._processed[i], day, res);
				this._filtered.push(this._processed[i]);
	        }
	    },

	    _getMonthAppList: function (renderDate, day, appointments, app, res) {
	        if ((appointments[app][this.model.appointmentSettings["allDay"]] == true) || (((new Date(appointments[app][this.model.appointmentSettings["endTime"]]) - new Date(appointments[app][this.model.appointmentSettings["startTime"]])) / 3600000) > 24)) {
	            if ((Globalize.format(new Date(renderDate[day]), this._pattern.d) == Globalize.format(new Date(appointments[app][this.model.appointmentSettings["startTime"]]), this._pattern.d)) &&
                (Globalize.format(new Date(renderDate[day]), this._pattern.S) <= Globalize.format(new Date(appointments[app][this.model.appointmentSettings["endTime"]]), this._pattern.S)) &&
                (appointments[app][this.model.appointmentSettings.resourceFields.toString().split(',')[this.model.resources.length - 1]] == this.res1[res][this.model.resources[this.model.resources.length - 1].resourceSettings.id]) &&
                (Globalize.format(new Date(appointments[app][this.model.appointmentSettings["startTime"]]), this._pattern.S) <= Globalize.format(new Date(appointments[app][this.model.appointmentSettings["endTime"]]), this._pattern.S))) {
	                this.filterData.push(appointments[app]);
	            }
	        } else {
	            if ((Globalize.format(new Date(renderDate[day]), this._pattern.d) == Globalize.format(new Date(appointments[app][this.model.appointmentSettings["startTime"]]), this._pattern.d)) &&
                (Globalize.format(new Date(renderDate[day]), this._pattern.d) <= Globalize.format(new Date(appointments[app][this.model.appointmentSettings["endTime"]]), this._pattern.d)) &&
                (appointments[app][this.model.appointmentSettings.resourceFields.toString().split(',')[this.model.resources.length - 1]] == this.res1[res][this.model.resources[this.model.resources.length - 1].resourceSettings.id]) &&
                (Globalize.format(new Date(appointments[app][this.model.appointmentSettings["startTime"]]), this._pattern.d) <= Globalize.format(new Date(appointments[app][this.model.appointmentSettings["endTime"]]), this._pattern.d))) {
	                this.filterData.push(appointments[app]);
	            }
	        }
	        return this.filterData;
	    },

	    _getResourceColor: function (record) {
	        var newResCollection = this._getResourceCollection();
	        var resourceDataIndex, appointData = "", appointCustomcss = "";
	        if (!ej.isNullOrUndefined(this.model.group)) {
	            resourceDataIndex = this._findResourceIndex(this.render_Resources[0], newResCollection[0].id, record[this.model.appointmentSettings.resourceFields.split(',')[0]]);
	            if (!ej.isNullOrUndefined(resourceDataIndex)) {
	                appointData = !ej.isNullOrUndefined(this.render_Resources[0][resourceDataIndex][newResCollection[0].color]) ? this.render_Resources[0][resourceDataIndex][newResCollection[0].color] : appointData;
	                appointCustomcss = !ej.isNullOrUndefined(this.render_Resources[0][resourceDataIndex][newResCollection[0].appointmentClass]) ? this.render_Resources[0][resourceDataIndex][newResCollection[0].appointmentClass] : appointCustomcss;
	            }
	        }
	        else {
	            resourceDataIndex = this._findResourceIndex(this.render_Resources[this.render_Resources.length - 1], newResCollection[newResCollection.length - 1].id, record[this.model.appointmentSettings.resourceFields.split(',')[this.model.appointmentSettings.resourceFields.split(',').length - 1]]);
	            if (!ej.isNullOrUndefined(resourceDataIndex)) {
	                appointData = !ej.isNullOrUndefined(this.render_Resources[this.render_Resources.length - 1][resourceDataIndex][newResCollection[newResCollection.length - 1].color]) ? this.render_Resources[this.render_Resources.length - 1][resourceDataIndex][newResCollection[newResCollection.length - 1].color] : appointData;
	                appointCustomcss = !ej.isNullOrUndefined(this.render_Resources[this.render_Resources.length - 1][resourceDataIndex][newResCollection[newResCollection.length - 1].appointmentClass]) ? this.render_Resources[this.render_Resources.length - 1][resourceDataIndex][newResCollection[newResCollection.length - 1].appointmentClass] : appointCustomcss;
	            }
	        }
	        return { appointData: appointData, appointCustomcss: appointCustomcss };
	    },

	    _getResourceDetails: function (_target) {
	        if (!ej.isNullOrUndefined(this.model.resources)) {
	            if (this.model.resources.length > 0) {
	                var noOfDays = this.model.currentView === "week" || this.model.currentView === "month" ? 7 : this.model.currentView === "workweek" ? 5 : 1;
	                var index = this.model.currentView === "month" ? _target.index() : this._cellIndex;

	                index = this.model.orientation == "vertical" ? parseInt(index / noOfDays) : this._workCellIndex;
	                var res1 = this.model.orientation == "vertical" ? this.res1 : this._resCollect;

	                var group, resourceData, query, result; this._resourceCollection = [];
	                var newResCollection = this._getResourceCollection();

	                for (var i = newResCollection.length - 1; i >= 0; i--) {
	                    if (i == newResCollection.length - 1) {

	                        if (!ej.isNullOrUndefined(this.model.group)) {
	                            this._resourceCollection.push({ id: res1[index][newResCollection[i].id] });
	                            group = res1[index][newResCollection[i].groupId];
	                        }
	                        else {
	                            this._resourceCollection.push({ id: newResCollection[newResCollection.length - 1].dataSource[index][newResCollection[newResCollection.length - 1].id] });
	                            group = newResCollection[newResCollection.length - 1].dataSource[index][newResCollection[newResCollection.length - 1].groupId];
	                        }
	                    }
	                    else {
	                        resourceData = newResCollection[i].dataSource instanceof ej.DataManager ? newResCollection[i].dataSource : ej.DataManager(newResCollection[i].dataSource);
	                        query = new ej.Query().where(newResCollection[i].id, ej.FilterOperators.equal, group);
	                        result = resourceData.executeLocal(query);
	                        this._resourceCollection.push({ id: result[0][newResCollection[i].id] });
	                        group = result[0][newResCollection[i].groupId];
	                    }
	                }
	                return this._resourceCollection;
	            }
	        }
	    },

	    _getResourceFields: function (obj) {
	        for (var j = 0, i = this._resourceCollection.length - 1; j < this._resourceCollection.length; j++, i--) {
	            obj[this.model.appointmentSettings.resourceFields.split(',')[j]] = this._resourceCollection[i].id;
	        }
	        return obj;
	    },

	    _getResourceSaveData: function (autoVal, obj, maxId, temp, starttime, endtime) {
	        var newResCollection = this._getResourceCollection();
	        var args = [], newAppointment, data, prevdata;
	        var resources = this.model.appointmentSettings.resourceFields.toString().split(',');
	        if (autoVal[autoVal.length - 1].length > 0) {
	            for (var j = 0; j < autoVal[autoVal.length - 1].length; j++) {
	                newAppointment = this._addAppData(obj, maxId, new Date(temp.StartDate.setHours(parseInt(starttime[0]), parseInt(starttime[1]))), new Date(temp.EndDate.setHours(parseInt(endtime[0]), parseInt(endtime[1]))), maxId);

	                if (autoVal[autoVal.length - 1].length > 1) {
	                    obj[this.model.appointmentSettings["id"]] = maxId;
	                    maxId++;
	                }
	                newAppointment[resources[resources.length - 1]] = autoVal[resources.length - 1][j].value;
	                var id = autoVal[resources.length - 1][j].value;
	                for (var i = resources.length - 1; i > 0; i--) {
	                    data = this.render_Resources[i] instanceof ej.DataManager ? this.render_Resources[i] : ej.DataManager(this.render_Resources[i]);
	                    query = new ej.Query().where(newResCollection[i].id, ej.FilterOperators.equal, id);
	                    appointId = data.executeLocal(query);
	                    prevdata = this.render_Resources[i - 1] instanceof ej.DataManager ? this.render_Resources[i - 1] : ej.DataManager(this.render_Resources[i - 1]);
	                    query1 = new ej.Query().where(newResCollection[i - 1].id, ej.FilterOperators.equal, appointId[0][newResCollection[i].groupId]);
	                    appointGroupId = prevdata.executeLocal(query1);
	                    id = appointGroupId[0][newResCollection[i - 1].id];
	                    newAppointment[resources[i - 1]] = id;

	                }
	                args.push(newAppointment);
	            }
	        }
	        else {
	            newAppointment = this._addAppData(obj, maxId, new Date(temp.StartDate.setHours(parseInt(starttime[0]), parseInt(starttime[1]))), new Date(temp.EndDate.setHours(parseInt(endtime[0]), parseInt(endtime[1]))), maxId);
	            newAppointment[resources[resources.length - 1]] = 0;
	            args.push(newAppointment);
	        }
	        return args;
	    },

	    _getDragDropResourceData: function (proxy, cellIndex, _newAddList) {
	        if (!ej.isNullOrUndefined(cellIndex)) {
	            var noOfCols = proxy.model.currentView === "week" || proxy.model.currentView === "month" ? 7 : proxy.model.currentView === "workweek" ? 5 : 1;
	            var newRes = proxy.model.orientation == "vertical" ? proxy.res1[Math.floor(cellIndex / noOfCols)] : proxy._resCollect[cellIndex];
	            _newAddList[proxy.model.appointmentSettings.resourceFields.split(',')[proxy.model.appointmentSettings.resourceFields.split(',').length - 1]] = newRes.id;
	            _newAddList[proxy.model.appointmentSettings.resourceFields.split(',')[proxy.model.appointmentSettings.resourceFields.split(',').length - 2]] = newRes.groupId;
	        }
	        return _newAddList;
	    },

	    _renderResourceData: function (targetElement, index) {
	        if (!ej.isNullOrUndefined(this.model.resources)) {
	            if (this.model.resources.length > 0) {
	                var noOfDays = this.model.currentView === "week" || this.model.currentView === "month" ? 7 : this.model.currentView === "workweek" ? 5 : 1;
	                index = this.model.currentView === "month" ? targetElement.index() : index;
	                index = this.model.orientation == "vertical" ? parseInt(index / noOfDays) : this._workCellIndex;
	                var res1 = this.model.orientation == "vertical" ? this.res1 : this._resCollect;
	                var group, resourceData, query, result, prevdata, resApp1, fieldCollection = []; this._resIdCollection = [];
	                var newResCollection = this._getResourceCollection();
	                for (var i = 0; i < newResCollection.length; i++) {
	                    this._appointmentAddWindow.find('.owner_' + i).ejAutocomplete("clearText");
	                }
	                this.resourceId = [];
	                for (var i = newResCollection.length - 1; i >= 0; i--) {
	                    if (i == newResCollection.length - 1) {
	                        if (!ej.isNullOrUndefined(this.model.group)) {
	                            this.resourceId.push({ id: res1[index][newResCollection[i].id], value: res1[index][newResCollection[i].text] });
	                            group = res1[index][newResCollection[i].groupId];
	                        }
	                        else {
	                            this.resourceId.push({ id: this.render_Resources[i][index][newResCollection[i].id], value: this.render_Resources[i][index][newResCollection[i].text] });
	                            group = this.render_Resources[i][index][newResCollection[i].groupId];
	                        }
	                        if (this.model.resources.length == 1)
	                            this._appointmentAddWindow.find('.owner_' + i).data("ejAutocomplete").selectValueByText(this.resourceId[0].value);
	                    }
	                    else {
	                        resourceData = newResCollection[i].dataSource instanceof ej.DataManager ? newResCollection[i].dataSource : ej.DataManager(newResCollection[i].dataSource);
	                        query = new ej.Query().where(newResCollection[i].id, ej.FilterOperators.equal, group);
	                        result = resourceData.executeLocal(query);
	                        this._appointmentAddWindow.find('.owner_' + i).data("ejAutocomplete").selectValueByText(result[0][newResCollection[i].text]);
	                        this.resourceId.push({ id: result[0][newResCollection[i].id], value: result[0][newResCollection[i].text] });
	                        group = result[0][newResCollection[i].groupId];
	                    }
	                }
	                if (this.model.resources.length > 1) {
	                    for (var j = 0, l = this.render_Resources.length - 1; j < this.render_Resources.length - 1; j++, l--) {
	                        prevdata = this.render_Resources[j + 1] instanceof ej.DataManager ? this.render_Resources[j + 1] : ej.DataManager(this.render_Resources[j + 1]);
	                        query = new ej.Query().where(newResCollection[j + 1].groupId, ej.FilterOperators.equal, this.resourceId[l][newResCollection[l].id]);
	                        resApp1 = prevdata.executeLocal(query);
	                        fieldCollection = [];
	                        for (var k = 0; k < resApp1.length; k++) {
	                            fieldCollection.push({ text: resApp1[k][newResCollection[j + 1].text], value: resApp1[k][newResCollection[j + 1].id], color: resApp1[k][newResCollection[j + 1].color] });
	                        }
	                        this._appointmentAddWindow.find('.owner_' + (j + 1)).ejAutocomplete("clearText");
	                        this._appointmentAddWindow.find('.owner_' + (j + 1)).ejAutocomplete("option", "dataSource", fieldCollection); //.data("ejAutocomplete").selectValueByKey(resApp1[0].id);

	                    }
	                    for (var j = 0, l = this.render_Resources.length - 1; j < this.render_Resources.length - 1; j++, l--) {
	                        this._appointmentAddWindow.find('.owner_' + (j + 1)).data("ejAutocomplete").selectValueByKey(this.resourceId[l - 1][newResCollection[l - 1].id]);
	                    }
	                }
	            }
	        }
	    },

	    /**
        * To configure the dataSource values of resources
        * @private
        */
	    _bindResourcesData: function () {
	        if (!ej.isNullOrUndefined(this.model.resources)) {
	            // this._levelresourceData = [], 
	            this._resourceInfo = []; var resFlag = false;
	            for (var i = 0; i < this.model.resources.length; i++) {
	                if (!ej.isNullOrUndefined(this.model.resources[i].resourceSettings)) {
	                    if (this.model.resources[i].resourceSettings.dataSource instanceof ej.DataManager)
	                        this._initResourceData(this.model.resources[i].resourceSettings, i);
	                    else {
	                        this._resourceInfo.push(this.model.resources[i].resourceSettings);
	                        resFlag = true;
	                        //this.model.resources[i].resourceSettings.dataSource
	                    }
	                }
	            }
	            if (resFlag)
	                this._renderInitSchedule();
	        }
	    },

	    _renderResourceElements: function ($appWindow) {
	        for (var i = 0; i < this.model.resources.length; i++) {
	            $appWindow += "<tr><td class='e-leftfields e-textlabel'>" + this.model.resources[i].title + ":</td><td colspan='3' class='e-rightfields'>" +
						  "<input id='" + this._id + "_ownerfield_" + i + "'  class='owner_" + i + "' type='text' value=''/><div id='ownerlist'></div></td></tr>";
	        }
	        return $appWindow;
	    },

	    _renderResourcesTemplate: function () {
	        //resource header template rendering
	        if (this.model.orientation == "vertical")
	            this.resourceHeadTemplate = $.templates(this._createTemplate(resourceHeadTemplate(), "_ResourceHeader"));

	        //resource autocomplete template rendering
	        if (!ej.isNullOrUndefined(this.model.resources))
	            this.resourceTemplate = "<div class='e-resourcediv' style='height:15px; margin-right:5px; margin-top:3px; float:left; width:15px; background-color: ${color}'></div> <div>${text}</div>";
	    }
	};
})(jQuery, Syncfusion);;
/**
* @fileOverview Plugin to style the Html Schedule elements
* @copyright Copyright Syncfusion Inc. 2001 - 2014. All rights reserved.
*  Use of this code is subject to the terms of our license.
*  A copy of the current license can be obtained at any time by e-mailing
*  licensing@syncfusion.com. Any infringement will be prosecuted under
*  applicable laws. 
* @version 12.2
* @author <a href="mailto:licensing@syncfusion.com">Syncfusion Inc</a>
*/

(function ($, ej, undefined) {
    ej.scheduleFeatures = ej.scheduleFeatures || {};

    var monthCellsTemplate = function () {
        this._monthCellsTemplate = ("<table class='e-table e-workcellstab'  height='100%'width='100%' cellpadding='0px' cellspacing='0px'>" +
        "<tbody>{{for cellrows ~cols1=cols1}}" +
        "<tr>{{for ~cwidth=cwidth ~cheight=cheight ~cols1}}" +
        "<td class='e-monthcells' width='{{:~cwidth}}' height='{{:~cheight}}'>" +
        "<div class='e-monthheader'></div><div class='e-appointwrapper'>" +
        "</td>{{/for}}" +
        "</tr>{{/for}}" +
        "</tbody></table>");
        return this._monthCellsTemplate;
    };

    var appointmentTemplate = function () {
        this._appTemplate = ("<div class='{{:appClass}} {{:appointCustomcss}}' id='Appointment_{{:id}}' role='presentation' style='{{if rtl == true}}right{{else}}left{{/if}}: {{:left}}; width: {{:appWidth}}; background: {{:appointData}};  color:{{:appointtextcolor}}; height: {{:appHeight}}px; top: {{:top}}px;' >"
        + "{{if appClass == 'e-appointment'}}" +
        "{{if orientation == 'vertical'}}<div class='e-tophandle'></div><div class='e-bottomhandle'></div>{{else}}<div class='e-lefthandle'></div><div class='e-righthandle'></div>{{/if}}" +
        "{{else}}<div class='e-lefthandle'></div><div class='e-righthandle'></div>" +
        "{{/if}}{{if userAppTemplId == false}}" +
       "<div class='e-appointinnertext'><div class='e-apptext'>{{:subject}} </div><div class='e-apptime'> {{:startTime}} - {{:endTime}}</div>"
        +"{{if value == true}}"+
        "<div class='e-multipleCategory'>{{:multiDiv}}</div>{{/if}}</div>" +
        "{{else}}{{:userTemplate}}{{/if}}</div>");
        return this._appTemplate;
    };

     var categorizeMultipleTemplate = function () {
        this._categorizeMultipleTemplate = (
            "{{for renderData }}<div style='background:{{:value}};height:11px;width:6px;float:left'>" +
            "</div>{{/for}}"
            );
        return this._categorizeMultipleTemplate;
    };

    var headerTemplate = function () {
        this._headerTemplate = (
        "{{for cols ~view=view ~colspan=colspan}}" +
        "<td class='e-headercells' colspan='{{:~colspan}}'>" +
        "{{if ~view == 'day' }}<div class='e-dateheadercell e-dayheadercell'> <span class='e-headerdaydisplay'>{{:currentDay}}</span></div> {{else}} <div class='e-dateheadercell'> <span class='e-headerdaydisplay'>{{:currentDay}}</span></div>{{/if}}" +
        "</td>" +
        "{{/for}}<td class='e-emptytd'></td>");
        return this._headerTemplate;
    };
    var leftHeaderTemplate = function () {
        this._leftIndent = ("<table class='e-table e-leftindenttable' width='{{:cswidth}}' cellpadding='0px' cellspacing='0px'>" +
        "<tbody> {{for trs ~tdClass=tdClass ~divClass=divClass}} <tr>{{for ~tdClass=tdClass ~divClass=divClass}} <td class='{{:~tdClass}}'style='height:27px;'> <div class='{{:~divClass}}'></div></td>{{/for}}</tr>" +
        "{{/for}}</tbody></table>");
        return this._leftIndent;
    };
    var alldayTemplate = function () {
        this._alldayTemplate = (
     "{{for cols ~colspan=colspan}} <td class='e-alldaycells' style='height:27px;' colspan='{{:~colspan}}'> </td>" +
        "{{/for}}<td class='e-emptyalldaytd'></td>");
        return this._alldayTemplate;
    };

    var workCellsTemplate = function () {
        this._workCellsTemplate = ("<table class='e-table e-workcellstab' height='100%'width='100%' cellpadding='0px' cellspacing='0px'>" +
        "<tbody >{{for cellrows ~cols1=cols1}}" +
        "<tr>{{for ~index=#index ~cwidth=cwidth ~cheight=cheight ~cols1 }}" +
        "{{if (~index+1) % 2 == 0 }}<td class='e-workcells' width='{{:~cwidth}}' height='{{:~cheight}}'>{{else}}<td class='e-workcells e-alternatecells'width='{{:~cwidth}}' height='{{:~cheight}}'>{{/if}}<div class='e-appointwrapper'></div></td>" +
        "{{/for}}" +
        "</tr>" +
        "{{/for}}</tbody></table>");
        return this._workCellsTemplate;
    };

    var timeCellsTemplate = function () {
        this._timeCellsTemplate = ("<div class='e-scrolltimecells'><table class='e-table e-timecelltable' cellpadding='0px' cellspacing='0px'>" +
        "<tbody>{{for timerows ~cols1=cols1 ~view1=viewclass}}" +
        "<tr>{{for ~name=time ~id=id ~view=~view1 }} " +
        "{{if ~view == 0}}<td class='e-timecells'><div class='e-timecelldivs'>{{:~name.slice(0,2)}}</div>" +
        "</td> <td class='e-timecells' rowspan='1'>{{for ~cols1}}{{if (#index+1)%2 ==0}}<div class='e-ampmdisplay'></div>{{else}}<div id='{{:~id}}' class='e-ampmdisplay e-ampmstyle'>{{:~name.slice(3,5)}}</div> {{/if}}{{/for}} </td>{{else}}<td class='e-timecells' style='width:50px;'><div class='e-timecelldivs'></div></td>{{/if}}{{/for}}</tr>{{/for}}</tbody></table></div>");
        return this._timeCellsTemplate;
    };

    /**
    * @namespace ej
    * @class ejSchedule
    * @requires jQuery
    * @requires jquery.easing.min.js
    * @requires jquery.globalize.min.js    
    * @requires jsrender.min.js
    * @requires ej.core.js
    * @requires ej.data.js
    * @requires ej.schedule.js
    * @requires ej.scroller.js
    * @requires ej.radiobutton.js
    * @requires ej.editor.js
    * @requires ej.dropdownlist.js
    * @requires ej.dialog.js
    * @requires ej.button.js
    * @requires ej.checkbox.js
    * @requires ej.datepicker.js
    * @requires ej.timepicker.js
    * @requires ej.autcomplete.js
    * @classdesc Custom Design for Html Schedule control.
    * @example 
    * &lt;div id="Schedule"&gt;&lt;/div&gt; <br> 
    * &lt;script&gt;
    * $('#Schedule').ejSchedule();         
    * &lt;/script&gt;
    */

    ej.widget("ejSchedule", "ej.Schedule", {
        // Root Css Class
        _rootCSS: "e-schedule",
        // widget element will be automatically set in this
        element: null,
        validTags: ["div"],
        // user defined model will be automatically set in this
        model: null,

        // default model
        defaults: {
            // time zone collection object to set the list of time zones.
            timezoneCollection: [
            { text: "UTC -12:00", value: "UTC -12:00" },
            { text: "UTC -11:00", value: "UTC -11:00" },
            { text: "UTC -10:00", value: "UTC -10:00" },
            { text: "UTC -09:00", value: "UTC -09:00" },
            { text: "UTC -08:00", value: "UTC -08:00" },
            { text: "UTC -07:00", value: "UTC -07:00" },
            { text: "UTC -06:00", value: "UTC -06:00" },
            { text: "UTC -05:00", value: "UTC -05:00" },
            { text: "UTC -04:30", value: "UTC -04:30" },
            { text: "UTC -04:00", value: "UTC -04:00" },
            { text: "UTC -03:30", value: "UTC -03:30" },
            { text: "UTC -03:00", value: "UTC -03:00" },
            { text: "UTC -02:00", value: "UTC -02:00" },
            { text: "UTC -01:00", value: "UTC -01:00" },
            { text: "UTC +00:00", value: "UTC +00:00" },
            { text: "UTC +01:00", value: "UTC +01:00" },
            { text: "UTC +02:00", value: "UTC +02:00" },
            { text: "UTC +03:00", value: "UTC +03:00" },
            { text: "UTC +03:30", value: "UTC +03:30" },
            { text: "UTC +04:00", value: "UTC +04:00" },
            { text: "UTC +04:30", value: "UTC +04:30" },
            { text: "UTC +05:00", value: "UTC +05:00" },
            { text: "UTC +05:30", value: "UTC +05:30" },
            { text: "UTC +05:45", value: "UTC +05:45" },
            { text: "UTC +06:00", value: "UTC +06:00" },
            { text: "UTC +06:30", value: "UTC +06:30" },
            { text: "UTC +07:00", value: "UTC +07:00" },
            { text: "UTC +08:00", value: "UTC +08:00" },
            { text: "UTC +09:00", value: "UTC +09:00" },
            { text: "UTC +09:30", value: "UTC +09:30" },
            { text: "UTC +10:00", value: "UTC +10:00" },
            { text: "UTC +11:00", value: "UTC +11:00" },
            { text: "UTC +12:00", value: "UTC +12:00" },
            { text: "UTC +13:00", value: "UTC +13:00" },
            ],
            /**		
            * Defines the views to show in schedule control. 
            * @default [ "Day" , "Week" , "WorkWeek" , "Month" ]
            * @type {Array.string}
            * @example 
            * &lt;div id="Schedule"&gt;&lt;/div&gt; <br>
            * &lt;script&gt;
            * 	$("#Schedule").ejSchedule({ views: ["Day","Month"], currentView:"day"});
            * &lt;/script&gt;
            * @memberof ejSchedule
            * @instance
            */
            views: ["Day", "Week", "WorkWeek", "Month"],
            /**		
            * Defines the view of the schedule; this will mention the view of the schedule.     See {@link CurrentView}
            * @default ej.Schedule.CurrentView.Week
            * @type {enum}
            * @example 
            * &lt;div id="Schedule"&gt;&lt;/div&gt; <br>
            * &lt;script&gt;
            * 	$("#Schedule").ejSchedule({ currentView: ej.Schedule.CurrentView.Day});
            * &lt;/script&gt;
            * @memberof ejSchedule
            * @instance
            */
            currentView: "week",
            /**		
            * Defines the time mode of the schedule; this will mention the timemode of the schedule.     See {@link TimeMode}
            * @default ej.Schedule.TimeMode.Hour12
            * @type {enum}
            * @example 
            * &lt;div id="Schedule"&gt;&lt;/div&gt; <br>
            * &lt;script&gt;
            * 	$("#Schedule").ejSchedule({ timeMode: ej.Schedule.TimeMode.Hour24});
            * &lt;/script&gt;
            * @memberof ejSchedule
            * @instance
            */
            timeMode: "12",
            /**		
            * Defines the timeZone of the schedule.
            * @default null
            * @type {string}
            * @example 
            * &lt;div id="Schedule"&gt;&lt;/div&gt; <br> 
            * &lt;script&gt;
            * 	$("#Schedule").ejSchedule({ timeZone: "UTC +2:00"});
            * &lt;/script&gt;
            * @memberof ejSchedule
            * @instance
            */
            timeZone: null,
            /**		
            * Apply the start hour to the work area of schedule.
            * @default 0
            * @type {number}
            * @example 
            * &lt;div id="Schedule"&gt;&lt;/div&gt; <br> 
            * &lt;script&gt;
            * 	$("#Schedule").ejSchedule({ startHour: 9});
            * &lt;/script&gt;
            * @memberof ejSchedule
            * @instance
            */
            startHour: 0,
            /**		
            * Apply the end hour to the work area of schedule.
            * @default 24
            * @type {number}
            * @example 
            * &lt;div id="Schedule"&gt;&lt;/div&gt; <br> 
            * &lt;script&gt;
            * 	$("#Schedule").ejSchedule({ endHour: 18});
            * &lt;/script&gt;
            * @memberof ejSchedule
            * @instance
            */
            endHour: 24,
            /**		
            * Enable or disable the highlighting business hour behavior for schedule.
            * @default true
            * @type {boolean}
            * @example 
            * &lt;div id="Schedule"&gt;&lt;/div&gt; <br> 
            * &lt;script&gt;
            * 	$("#Schedule").ejSchedule({ highlightBusinessHours: false});
            * &lt;/script&gt;
            * @memberof ejSchedule
            * @instance
            */
            highlightBusinessHours: true,
            /**		
            * Enable or disable the quick window open behavior for schedule.
            * @default true
            * @type {boolean}
            * @example
            * &lt;div id="Schedule"&gt;&lt;/div&gt; <br> 
            * &lt;script&gt;
            * 	$("#Schedule").ejSchedule({ showQuickWindow: false});
            * &lt;/script&gt;
            * @memberof ejSchedule
            * @instance
            */
            showQuickWindow: true,
            /**		
            * Apply the business start hour of the schedule.
            * @default 9
            * @type {number}
            * @example 
            * &lt;div id="Schedule"&gt;&lt;/div&gt; <br> 
            * &lt;script&gt;
            * 	$("#Schedule").ejSchedule({ highlightBusinessHours : true, businessStartHour: 7});
            * &lt;/script&gt;  
            * @memberof ejSchedule
            * @instance
            */
            businessStartHour: 9,
            /**		
            * Apply the business end hour of the schedule.
            * @default 18
            * @type {number}
            * @example 
            * &lt;div id="Schedule"&gt;&lt;/div&gt; <br> 
            * &lt;script&gt;
            * 	$("#Schedule").ejSchedule({ highlightBusinessHours : true, businessEndHour: 16});
            * &lt;/script&gt;
            * @memberof ejSchedule
            * @instance
            */
            businessEndHour: 18,
            /**		
            * Defines the width of the schedule.
            * @default "800px"
            * @type {string}
            * @example 
            * &lt;div id="Schedule"&gt;&lt;/div&gt; <br> 
            * &lt;script&gt;
            * 	$("#Schedule").ejSchedule({ width: "700px"});
            * &lt;/script&gt;
            * @memberof ejSchedule
            * @instance
            */
            width: "800px",
            /**		
            * Defines the height of the schedule.
            * @default "800px"
            * @type {string}
            * @example 
            * &lt;div id="Schedule"&gt;&lt;/div&gt; <br> 
            * &lt;script&gt;
            * 	$("#Schedule").ejSchedule({ height: "700px"});
            * &lt;/script&gt;
            * @memberof ejSchedule
            * @instance
            */
            height: "800px",
		    /**		
            * Defines the Cell height of the schedule.
            * @default "20px"
            * @type {string}
            * @example 
            * &lt;div id="Schedule"&gt;&lt;/div&gt; <br> 
            * &lt;script&gt;
            * 	$("#Schedule").ejSchedule({ cellHeight: "100px"});
            * &lt;/script&gt;
            * @memberof ejSchedule
            * @instance
            */
            cellHeight: "20px",
            /**		
            * Defines the Cell Width of the schedule.
            * @default ""
            * @type {string}
            * @example 
            * &lt;div id="Schedule"&gt;&lt;/div&gt; <br> 
            * &lt;script&gt;
            * 	$("#Schedule").ejSchedule({ cellWidth: "100px"});
            * &lt;/script&gt;
            * @memberof ejSchedule
            * @instance
            */
            cellWidth: "",
            /**		
            * Apply the current date of the schedule.
            * @default new Date()
            * @type {date}
            * @example 
            * &lt;div id="Schedule"&gt;&lt;/div&gt; <br> 
            * &lt;script&gt;
            * 	$("#Schedule").ejSchedule({ currentDate: new Date()});
            * &lt;/script&gt;
            * @memberof ejSchedule
            * @instance
            */
            currentDate: new Date(),
            /**		
            * Specify the CSS class to schedule to achieve custom theme.
            * @default ""
            * @type {string}
            * @example 
            * &lt;div id="Schedule"&gt;&lt;/div&gt; <br> 
            * &lt;script&gt;
            * 	$("#Schedule").ejSchedule({ cssClass: "gradient-lime"});
            * &lt;/script&gt;
            * @memberof ejSchedule
            * @instance
            */
            cssClass: "",
            /**		
            * Defines the locale for schedule.
            * @default "en-US"
            * @type {string}
            * @example 
            * &lt;div id="Schedule"&gt;&lt;/div&gt; <br> 
            * &lt;script&gt;
            * 	$("#Schedule").ejSchedule({ locale: "en-US"});
            * &lt;/script&gt;
            * @memberof ejSchedule
            * @instance
            */
            locale: "en-US",
            /**		
            * Enables or disables the resize behavior of schedule.
            * @default false
            * @type {boolean}
            * @example 
            * &lt;div id="Schedule"&gt;&lt;/div&gt; <br> 
            * &lt;script&gt;
            * 	$("#Schedule").ejSchedule({ enableResize: false});
            * &lt;/script&gt;
            * @memberof ejSchedule
            * @instance
            */
            enableResize: false,
            /**		
            * Enables or disables the rtl direction for schedule.
            * @default false
            * @type {boolean}
            * @example 
            * &lt;div id="Schedule"&gt;&lt;/div&gt; <br> 
            * &lt;script&gt;
            * 	$("#Schedule").ejSchedule({ enableRTL: true});
            * &lt;/script&gt;
            * @memberof ejSchedule
            * @instance
            */
            enableRTL: false,
            /**		
            * Enables or disables the previous next appointment navigation of schedule.
            * @default true
            * @type {boolean}
            * @example 
            * &lt;div id="Schedule"&gt;&lt;/div&gt; <br> 
            * &lt;script&gt;
            * 	$("#Schedule").ejSchedule({ 
            *       enableAppointmentNavigation: true,
            *       currentDate:new Date(2014,1,1),
            *       appointmentSettings: { dataSource: window.Default, //collection of object from dataSource.
            *               id: "Id",
            *               subject: "Subject",
            *               description: "Description",
            *               startTime: "StartTime",
            *               endTime: "EndTime",
            *               recurrence: "Recurrence",
            *               recurrenceRule: "RecurrenceRule",
            *               allDay: "AllDay"
            *             }
            *       });
            * &lt;/script&gt;
            * @memberof ejSchedule
            * @instance
            */
            enableAppointmentNavigation: true,
            /**		
            * An element’s id which can be used for appointment template rendering.
            * @default null
            * @type {string}
            * @example 
            * &lt;div id="Schedule"&gt;&lt;/div&gt; <br> 
            * &lt;script&gt;
            * 	$("#Schedule").ejSchedule({ appointmentTemplateId: "#templatename"});
            * &lt;/script&gt;
            * @memberof ejSchedule
            * @instance
            */
            appointmentTemplateId: null,
            /**		
            * An element’s id which can be used for resource header template rendering.
            * @default null
            * @type {string}
            * @example 
            * &lt;div id="Schedule"&gt;&lt;/div&gt; <br> 
            * &lt;script&gt;
            * 	$("#Schedule").ejSchedule({ resourceHeaderTemplateId: "#resTemplate"});
            * &lt;/script&gt;
            * @memberof ejSchedule
            * @instance
            */
            resourceHeaderTemplateId: null,
            /**		
            * Enables or disables the appointment drag and drop behavior of schedule.
            * @default true
            * @type {boolean}
            * @example 
            * &lt;div id="Schedule"&gt;&lt;/div&gt; <br> 
            * &lt;script&gt;
            * 	$("#Schedule").ejSchedule({ 
            *     allowDragDrop: true,
            *     appointmentSettings: { dataSource: window.Default, //collection of object from dataSource.
            *               id: "Id",
            *               subject: "Subject",
            *               description: "Description",
            *               startTime: "StartTime",
            *               endTime: "EndTime",
            *               recurrence: "Recurrence",
            *               recurrenceRule: "RecurrenceRule",
            *               allDay: "AllDay"
            *             }   
            *        });
            * &lt;/script&gt;
            * @memberof ejSchedule
            * @instance
            */
            allowDragDrop: true,
            /**		
            * Enables or disables the appointment resize behavior of schedule.
            * @default true
            * @type {boolean}
            * @example 
            * &lt;div id="Schedule"&gt;&lt;/div&gt; <br> 
            * &lt;script&gt;
            * 	$("#Schedule").ejSchedule({ 
            *     enableAppointmentResize: true,
            *     appointmentSettings: { dataSource: window.Default, //collection of object from dataSource.
            *               id: "Id",
            *               subject: "Subject",
            *               description: "Description",
            *               startTime: "StartTime",
            *               endTime: "EndTime",
            *               recurrence: "Recurrence",
            *               recurrenceRule: "RecurrenceRule",
            *               allDay: "AllDay"
            *             }   
            *       });
            * &lt;/script&gt;
            * @memberof ejSchedule
            * @instance
            */
            enableAppointmentResize: true,
            /**		
            * Enables or disables the current time indicator of schedule.
            * @default true
            * @type {boolean}
            * @example 
            * &lt;div id="Schedule"&gt;&lt;/div&gt; <br> 
            * &lt;script&gt;
            * 	$("#Schedule").ejSchedule({ showCurrentTimeIndicator: true});
            * &lt;/script&gt;
            * @memberof ejSchedule
            * @instance
            */
            showCurrentTimeIndicator: true,
            /**		
            * This property is used to set the reminder options of schedule. 
            * @default 
            * @type {Object}
            * @example       
            * &lt;div id="Schedule"&gt;&lt;/div&gt; <br> 
            * &lt;script&gt;
            * $("#Schedule").ejSchedule({
            *   reminderSettings: {
            *       enable: true,
            *       }
            *    });
            * &lt;/script&gt;
            * @memberof ejSchedule
            * @instance
            */  
            reminderSettings: {
                /**		
                * Enables or disables the reminder settings option for schedule.
                * @name ejSchedule#reminderSettings->enable
                * @default false
                * @type {boolean}
                * @example 
                * &lt;div id="Schedule"&gt;&lt;/div&gt; <br> 
                * &lt;script&gt;
                * 	$("#Schedule").ejSchedule({ reminderSettings:{enable: true}});
                * &lt;/script&gt;
                * @memberof ejSchedule
                * @instance
                */
                enable: false,
                /**		
                * Set the alert timing in reminder settings option of schedule.
                * @name ejSchedule#reminderSettings->alertBefore
                * @default 5
                * @type {number}
                * @example 
                * &lt;div id="Schedule"&gt;&lt;/div&gt; <br> 
                * &lt;script&gt;
                * 	$("#Schedule").ejSchedule({ reminderSettings:{enable: true, alertBefore:6}});
                * &lt;/script&gt;
                * @memberof ejSchedule
                * @instance
                */
                alertBefore: 5
            },
            /**		
            * This property is used to bind the contextmenu items of schedule. 
            * @default Object
            * @type {Object}
            * @example       
            * &lt;div id="Schedule"&gt;&lt;/div&gt; <br> 
            * &lt;script&gt;
            * $("#Schedule").ejSchedule({
            *   contextMenuSettings: {
            *       enable: true,
            *       }
            *    });
            * &lt;/script&gt;
            * @memberof ejSchedule
            * @instance
            */
            contextMenuSettings: /** @lends ejSchedule# */{
                /**		
                * Enables or disables the context menu option for schedule.
                * @name ejSchedule#contextMenuSettings->enable
                * @default false
                * @type {boolean}
                * @example 
                * &lt;div id="Schedule"&gt;&lt;/div&gt; <br> 
                * &lt;script&gt;
                * 	$("#Schedule").ejSchedule({ contextMenuSettings:{enable: true}});
                * &lt;/script&gt;
                * @memberof ejSchedule
                * @instance
                */
                enable: false,
                /**		
                * Context menu collection for appointment and cells of schedule.
                * @name ejSchedule#contextMenuSettings->menuItems
                * @default Object
                * @type {object}
                * @example 
                * &lt;div id="Schedule"&gt;&lt;/div&gt; <br> 
                * &lt;script&gt;
                * 	$("#Schedule").ejSchedule({ contextMenuSettings:{enable: true,
                *       menuItems:{appointment: [
                *        { id: "open", text: "Open Appointment" },
                *        { id: "delete", text: "Delete Appointment" }
                *    ],
                *    cells: [
                *        { id: "new", text: "New Appointment" },
                *        { id: "recurrence", text: "New Recurring Appointment" },
                *        { id: "today", text: "Today" },
                *        { id: "gotodate", text: "Go to date" },
                *        { id: "settings", text: "Settings" },
                *        { id: "view", text: "View", parentId: "settings" },
                *        { id: "timemode", text: "TimeMode", parentId: "settings" },
                *        { id: "view_Day", text: "Day", parentId: "view" },
                *        { id: "view_Week", text: "Week", parentId: "view" },
                *        { id: "view_Workweek", text: "Workweek", parentId: "view" },
                *        { id: "view_Month", text: "Month", parentId: "view" },
                *        { id: "timemode_Hour12", text: "12 Hours", parentId: "timemode" },
                *        { id: "timemode_Hour24", text: "24 Hours", parentId: "timemode" },
                *        { id: "businesshours", text: "Business Hours", parentId: "settings" }
                *    ]}
                *   }
                *   });
                * &lt;/script&gt;
                * @memberof ejSchedule
                * @instance
                */
                menuItems: {
                    // Context menu collection object for appointment.
                    appointment: [
                        { id: "open", text: "Open Appointment" },
                        { id: "delete", text: "Delete Appointment" }
                    ],
                    // Context menu collection object for cells.
                    cells: [
                        { id: "new", text: "New Appointment" },
                        { id: "recurrence", text: "New Recurring Appointment" },
                        { id: "today", text: "Today" },
                        { id: "gotodate", text: "Go to date" },
                        { id: "settings", text: "Settings" },
                        { id: "view", text: "View", parentId: "settings" },
                        { id: "timemode", text: "TimeMode", parentId: "settings" },
                        { id: "view_Day", text: "Day", parentId: "view" },
                        { id: "view_Week", text: "Week", parentId: "view" },
                        { id: "view_Workweek", text: "Workweek", parentId: "view" },
                        { id: "view_Month", text: "Month", parentId: "view" },
                        { id: "timemode_Hour12", text: "12 Hours", parentId: "timemode" },
                        { id: "timemode_Hour24", text: "24 Hours", parentId: "timemode" },
                        { id: "businesshours", text: "Business Hours", parentId: "settings" }
                    ]
                }
            },
            /**		
            * Groups the specified resources in the schedule control.
            * @default null
            * @type {Object}
            * @example 
            * &lt;div id="Schedule"&gt;&lt;/div&gt; <br> 
            * &lt;script&gt;
            * 	$("#Schedule").ejSchedule({ 
            *       group: {
            *           resources: ["Rooms","Owners"]
            *       },    
            *       resources: [{
            *        field: "ownerId",
            *        title: "Owner",
            *        name: "Owners", allowMultiple: true,
            *        resourceSettings: { dataSource: [
            *        { text: "Room1", id: 1, color: "#f8a398" }], 
            *        text: "text", id: "id",  color: "color" }
            *      },
            *       {
            *           field: "roomId",
            *           title: "Room",
            *           name: "Rooms", allowMultiple: false,
            *           resourceSettings: { dataSource: [
            *           { text: "Andrew", id: 1, groupId: 1, color: "#f8a398" },
            *           { text: "Jerry", id: 2, groupId: 1, color: "#56ca85"}],
            *             text: "text", id: "id", groupId: "groupId", color: "color"
            *           }
            *        }],
            *       appointmentSettings: {
            *           dataSource: window.ResourcesData,
            *           id: "Id",
            *           subject: "Subject",
            *           startTime: "StartTime",
            *           endTime: "EndTime",
            *           allDay: "AllDay",
            *           recurrence: "Recurrence",
            *           recurrenceRule: "RecurrenceRule",
            *           resourceFields: "roomId,ownerId"
            *       } 
            *    });  
            * &lt;/script&gt;
            * @memberof ejSchedule
            * @instance
            */
            group: null,
            /**		
            * Binds the resource collection to the schedule.
            * @default null
            * @type {Object}
            * @example 
            * &lt;div id="Schedule"&gt;&lt;/div&gt; <br> 
            * &lt;script&gt;
            * 	$("#Schedule").ejSchedule({ 
            *      resources: [
            *      {
            *        field: "ownerId",
            *        title: "Owner",
            *        name: "Owners", allowMultiple: true,
            *        resourceSettings: { dataSource: [
            *        { text: "Andrew", id: 1, groupId: 1, color: "#f8a398" },
            *        { text: "Cruise", id: 3, groupId: 2, color: "#56ca85" },
            *        { text: "Jerry", id: 5, groupId: 1, color: "#51a0ed" }],    
            *        text: "text", id: "id", groupId: "groupId", color: "color" }
            *      }],    
            *      appointmentSettings: { dataSource: window.ResourcesData, //collection of object from dataSource.
            *               id: "Id",
            *               subject: "Subject",
            *               startTime: "StartTime",
            *               endTime: "EndTime",
            *               recurrence: "Recurrence",
            *               recurrenceRule: "RecurrenceRule",
            *               allDay: "AllDay",
            *               resourceFields: "ownerId"
            *             } 
            *      });  
            * &lt;/script&gt;
            * @memberof ejSchedule
            * @instance
            */
            resources: null,
            /**     
            * Enables or disables the keyboard interaction of schedule.
            * @default true
            * @type {boolean}
            * @example 
            * &lt;div id="Schedule"&gt;&lt;/div&gt; <br> 
            * &lt;script&gt;
            * 	$("#Schedule").ejSchedule({ allowKeyboardNavigation: true});
            * &lt;/script&gt;
            * @memberof ejSchedule
            * @instance
            */
            allowKeyboardNavigation: true,
            /**		
            * This property is used to bind the appointmentSettings data fields to the schedule. 
            * @default Object
            * @type {Object}
            * @example       
            * &lt;div id="Schedule"&gt;&lt;/div&gt; <br> 
            * &lt;script&gt;
            * $("#Schedule").ejSchedule({
            *     appointmentSettings: { dataSource: window.Default, //collection of object from dataSource.
            *               id: "Id",
            *               subject: "Subject",
            *               description: "Description",
            *               startTime: "StartTime",
            *               endTime: "EndTime",
            *               recurrence: "Recurrence",
            *               recurrenceRule: "RecurrenceRule",
            *               allDay: "AllDay"
            *             }                                      
            *    });
            * &lt;/script&gt;
            * @memberof ejSchedule
            * @instance
            */
            appointmentSettings: /** @lends ejSchedule# */{
            /**		
            * Render schedule appointments by the specified dataSource.
            * @name ejSchedule#appointmentSettings->dataSource
            * @default null
            * @type {object}
            * @example       
            * &lt;div id="Schedule"&gt;&lt;/div&gt; <br> 
            * &lt;script&gt;
            * $("#Schedule").ejSchedule({
            *     appointmentSettings: { dataSource: window.Default, //collection of object from dataSource.
            *               id: "Id",
            *               subject: "Subject",
            *               description: "Description",
            *               startTime: "StartTime",
            *               endTime: "EndTime",
            *               recurrence: "Recurrence",
            *               recurrenceRule: "RecurrenceRule",
            *               allDay: "AllDay"
            *             }                                      
            *    });
            * &lt;/script&gt;
            * @memberof ejSchedule
            * @instance
            */
            dataSource: null,
            /**		
            * Query the dataSource from the table for schedule.
            * @name ejSchedule#appointmentSettings->query
            * @default null
            * @type {object}
            * @example       
            * &lt;div id="Schedule"&gt;&lt;/div&gt; <br> 
            * &lt;script&gt;
            * $("#Schedule").ejSchedule({
            *     appointmentSettings: { dataSource: window.Default, //collection of object from dataSource.
            *               id: "Id",
            *               subject: "Subject",
            *               description: "Description",
            *               startTime: "StartTime",
            *               endTime: "EndTime",
            *               recurrence: "Recurrence",
            *               recurrenceRule: "RecurrenceRule",
            *               allDay: "AllDay"
            *             }                                      
            *    });
            * &lt;/script&gt;
            * @memberof ejSchedule
            * @instance
            */
            query: null,
            /**		
            * Specify the tablename to retrive the data for schedule.
            * @name ejSchedule#appointmentSettings->tableName
            * @default null
            * @type {string}
            * @example       
            * &lt;div id="Schedule"&gt;&lt;/div&gt; <br> 
            * &lt;script&gt;
            * $("#Schedule").ejSchedule({
            *     appointmentSettings: { tableName:"tablename",
            *               dataSource: window.Default, //collection of object from dataSource.
            *               id: "Id",
            *               subject: "Subject",
            *               description: "Description",
            *               startTime: "StartTime",
            *               endTime: "EndTime",
            *               recurrence: "Recurrence",
            *               recurrenceRule: "RecurrenceRule",
            *               allDay: "AllDay"
            *             }                                      
            *    });
            * &lt;/script&gt;
            * @memberof ejSchedule
            * @instance
            */
            tableName: null,
            /**		
            * Bind string value to id field of schedule.
            * @name ejSchedule#appointmentSettings->id
            * @default null
            * @type {string}
            * @example       
            * &lt;div id="Schedule"&gt;&lt;/div&gt; <br> 
            * &lt;script&gt;
            * $("#Schedule").ejSchedule({
            *     appointmentSettings: { id: "Id",
            *               dataSource: window.Default, //collection of object from dataSource.                *               
            *               subject: "Subject",
            *               description: "Description",
            *               startTime: "StartTime",
            *               endTime: "EndTime",
            *               recurrence: "Recurrence",
            *               recurrenceRule: "RecurrenceRule",
            *               allDay: "AllDay"
            *             }                                      
            *    });
            * &lt;/script&gt;
            * @memberof ejSchedule
            * @instance
            */
            id: null,
            /**		
            * Bind string value to subject field of schedule.
            * @name ejSchedule#appointmentSettings->subject
            * @default null
            * @type {string}
            * @example       
            * &lt;div id="Schedule"&gt;&lt;/div&gt; <br> 
            * &lt;script&gt;
            * $("#Schedule").ejSchedule({
            *     appointmentSettings: { subject: "Subject",
            *               dataSource: window.Default, //collection of object from dataSource.
            *               id: "Id",
            *               description: "Description",
            *               startTime: "StartTime",
            *               endTime: "EndTime",
            *               recurrence: "Recurrence",
            *               recurrenceRule: "RecurrenceRule",
            *               allDay: "AllDay"
            *             }                                      
            *    });
            * &lt;/script&gt;
            * @memberof ejSchedule
            * @instance
            */
            subject: null,
            /**		
            * Bind string value to description field of schedule.
            * @name ejSchedule#appointmentSettings->description
            * @default null
            * @type {string}
            * @example       
            * &lt;div id="Schedule"&gt;&lt;/div&gt; <br> 
            * &lt;script&gt;
            * $("#Schedule").ejSchedule({
            *     appointmentSettings: { description: "Description",
            *               dataSource: window.Default, //collection of object from dataSource.
            *               id: "Id",
            *               subject: "Subject",               
            *               startTime: "StartTime",
            *               endTime: "EndTime",
            *               recurrence: "Recurrence",
            *               recurrenceRule: "RecurrenceRule",
            *               allDay: "AllDay"
            *             }                                      
            *    });
            * &lt;/script&gt;
            * @memberof ejSchedule
            * @instance
            */
            description: null,
            /**		
            * Bind string value to startTime field of schedule.
            * @name ejSchedule#appointmentSettings->startTime
            * @default null
            * @type {string}
            * @example       
            * &lt;div id="Schedule"&gt;&lt;/div&gt; <br> 
            * &lt;script&gt;
            * $("#Schedule").ejSchedule({
            *     appointmentSettings: { startTime: "StartTime",
            *               dataSource: window.Default, //collection of object from dataSource.
            *               id: "Id",
            *               subject: "Subject",
            *               description: "Description",               
            *               endTime: "EndTime",
            *               recurrence: "Recurrence",
            *               recurrenceRule: "RecurrenceRule",
            *               allDay: "AllDay"
            *             }                                      
            *    });
            * &lt;/script&gt;
            * @memberof ejSchedule
            * @instance
            */
            startTime: null,
            /**		
            * Bind string value to endTime field of schedule.
            * @name ejSchedule#appointmentSettings->endTime
            * @default null
            * @type {string}
            * @example       
            * &lt;div id="Schedule"&gt;&lt;/div&gt; <br> 
            * &lt;script&gt;
            * $("#Schedule").ejSchedule({
            *     appointmentSettings: { endTime: "EndTime",
            *               dataSource: window.Default, //collection of object from dataSource.
            *               id: "Id",
            *               subject: "Subject",
            *               description: "Description",
            *               startTime: "StartTime",
            *               recurrence: "Recurrence",
            *               recurrenceRule: "RecurrenceRule",
            *               allDay: "AllDay"
            *             }                                      
            *    });
            * &lt;/script&gt;
            * @memberof ejSchedule
            * @instance
            */
            endTime: null,
            /**		
            * Bind string value to recurrence field of schedule.
            * @name ejSchedule#appointmentSettings->recurrence
            * @default null
            * @type {string}
            * @example       
            * &lt;div id="Schedule"&gt;&lt;/div&gt; <br> 
            * &lt;script&gt;
            * $("#Schedule").ejSchedule({
            *     appointmentSettings: { recurrence: "Recurrence",
            *               dataSource: window.Default, //collection of object from dataSource.
            *               id: "Id",
            *               subject: "Subject",
            *               description: "Description",
            *               startTime: "StartTime",
            *               endTime: "EndTime",
            *               recurrenceRule: "RecurrenceRule",
            *               allDay: "AllDay"
            *             }                                      
            *    });
            * &lt;/script&gt;
            * @memberof ejSchedule
            * @instance
            */
            recurrence: null,
            /**		
            * Bind string value to recurrenceRule field of schedule.
            * @name ejSchedule#appointmentSettings->recurrenceRule
            * @default null
            * @type {string}
            * @example       
            * &lt;div id="Schedule"&gt;&lt;/div&gt; <br> 
            * &lt;script&gt;
            * $("#Schedule").ejSchedule({
            *     appointmentSettings: { recurrenceRule: "RecurrenceRule",
            *               dataSource: window.Default, //collection of object from dataSource.
            *               id: "Id",
            *               subject: "Subject",
            *               description: "Description",
            *               startTime: "StartTime",
            *               endTime: "EndTime",
            *               recurrence: "Recurrence",
            *               allDay: "AllDay"
            *             }                                      
            *    });
            * &lt;/script&gt;
            * @memberof ejSchedule
            * @instance
            */
            recurrenceRule: null,
            /**		
            * Bind string value to allday field of schedule.
            * @name ejSchedule#appointmentSettings->allDay
            * @default null
            * @type {string}
            * @example       
            * &lt;div id="Schedule"&gt;&lt;/div&gt; <br> 
            * &lt;script&gt;
            * $("#Schedule").ejSchedule({
            *     appointmentSettings: { dataSource: window.Default, //collection of object from dataSource.
            *               id: "Id",
            *               subject: "Subject",
            *               description: "Description",
            *               startTime: "StartTime",
            *               endTime: "EndTime",
            *               recurrence: "Recurrence",
            *               recurrenceRule: "RecurrenceRule",
            *               allDay: "AllDay"
            *             }                                      
            *    });
            * &lt;/script&gt;
            * @memberof ejSchedule
            * @instance
            */
            allDay: null,
            /**		
            * Binds the resource related fields to the schedule.
            * @default null
            * @type {string}
            * @example 
            * &lt;div id="Schedule"&gt;&lt;/div&gt; <br> 
            * &lt;script&gt;
            * 	$("#Schedule").ejSchedule({ 
            *      resources: [
            *      {
            *        field: "ownerId",
            *        title: "Owner",
            *        name: "Owners", allowMultiple: true,
            *        resourceSettings: { dataSource: [
            *        { text: "Andrew", id: 1, groupId: 1, color: "#f8a398" },
            *        { text: "Cruise", id: 3, groupId: 2, color: "#56ca85" },
            *        { text: "Jerry", id: 5, groupId: 1, color: "#51a0ed" }],    
            *        text: "text", id: "id", groupId: "groupId", color: "color" }
            *      }],    
            *      appointmentSettings: { dataSource: window.ResourcesData, //collection of object from dataSource.
            *               id: "Id",
            *               subject: "Subject",
            *               startTime: "StartTime",
            *               endTime: "EndTime",
            *               recurrence: "Recurrence",
            *               recurrenceRule: "RecurrenceRule",
            *               allDay: "AllDay",
            *               resourceFields: "ownerId"
            *             } 
            *      });  
            * &lt;/script&gt;
            * @memberof ejSchedule
            * @instance
            */
            resourceFields: null,
                /**		
           * Bind string value to categorize field of schedule.
           * @name ejSchedule#appointmentSettings->categorize
           * @default null
           * @type {string}
           * @example       
           * &lt;div id="Schedule"&gt;&lt;/div&gt; <br> 
           * &lt;script&gt;
           * $("#Schedule").ejSchedule({
           *     appointmentSettings: { dataSource: window.Default, //collection of object from dataSource.
           *               id: "Id",
           *               subject: "Subject",
           *               description: "Description",
           *               startTime: "StartTime",
           *               endTime: "EndTime",
           *               recurrence: "Recurrence",
           *               recurrenceRule: "RecurrenceRule",
           *               allDay: "AllDay",
           *               categorize: "Categorize"
           *             }                                      
           *    });
           * &lt;/script&gt;
           * @memberof ejSchedule
           * @instance
           */
            categorize:null
            },
            /**		
            * Defines the orientation type of the schedule; Renders the schedule either in vertical or horizontal mode as specified through this property.
            * @default ej.Schedule.Orientation.Vertical
            * @type {enum}
            * @example 
            * &lt;div id="Schedule"&gt;&lt;/div&gt; <br>
            * &lt;script&gt;
            * 	$("#Schedule").ejSchedule({ orientation: ej.Schedule.Orientation.Horizontal});
            * &lt;/script&gt;
            * @memberof ejSchedule
            * @instance
            */
            orientation: "vertical",
            /**		
            * This property is used to bind the Categorize items of schedule. 
            * @default Object
            * @type {Object}
            * @example       
            * &lt;div id="Schedule"&gt;&lt;/div&gt; <br> 
            * &lt;script&gt;
            * $("#Schedule").ejSchedule({
            *   categorizeSettings: {
            *       enable: true,
            *       }
            *    });
            * &lt;/script&gt;
            * @memberof ejSchedule
            * @instance
            */
            categorizeSettings: /** @lends ejSchedule# */{
            /**		
            * Enables or disables the Categorize option for schedule.
            * @name ejSchedule#categorizeSettings->enable
            * @default false
            * @type {boolean}
            * @example 
            * &lt;div id="Schedule"&gt;&lt;/div&gt; <br> 
            * &lt;script&gt;
            * 	$("#Schedule").ejSchedule({ categorizeSettings:{enable: false}});
            * &lt;/script&gt;
            * @memberof ejSchedule
            * @instance
            */
             enable: false,
            /**		
            * allowMultiple option enables/disables the multiple selection of categories for the appointments.
            * @name ejSchedule#categorizeSettings->allowMultiple
            * @default false
            * @type {boolean}
            * @example 
            * &lt;div id="Schedule"&gt;&lt;/div&gt; <br> 
            * &lt;script&gt;
            * 	$("#Schedule").ejSchedule({ categorizeSettings:{allowMultiple: false}});
            * &lt;/script&gt;
            * @memberof ejSchedule
            * @instance
            */
             allowMultiple: false,
            /**		
            * Categorize collection for the schedule.
            * @name ejSchedule#categorizeSettings->dataSource
            * @default Object
            * @type {object}
            * @example 
            * &lt;div id="Schedule"&gt;&lt;/div&gt; <br> 
            * &lt;script&gt;
            * 	$("#Schedule").ejSchedule({ categorizeSettings:{enable: true,allowMultiple:false,
            *  dataSource:[
            *  { text: "Blue Category", id: 1, color: "#7499e1", fontColor: "green" },
            *  { text: "Green Category", id: 2, color: "#7cce6e", fontColor: "green" },
            *  { text: "Orange Category", id: 3, color: "#f19d5a", fontColor: "green" },
            *  { text: "Purple Category", id: 4, color: "#937bd1", fontColor: "green" },
            *  { text: "Red Category", id: 5, color: "#d98889", fontColor: "green" },
            *  { text: "Yellow Category", id: 6, color: "#f8f264", fontColor: "green" }                
            *  ],
            *   });
            * &lt;/script&gt;
            * @memberof ejSchedule
            * @instance
            */
             dataSource:
                [
                { text: "Blue Category", id: 1, color: "#7499e1", fontColor: "green" },
                { text: "Green Category", id: 2, color: "#7cce6e", fontColor: "green" },
                { text: "Orange Category", id: 3, color: "#f19d5a", fontColor: "green" },
                { text: "Purple Category", id: 4, color: "#937bd1", fontColor: "green" },
                { text: "Red Category", id: 5, color: "#d98889", fontColor: "green" },
                { text: "Yellow Category", id: 6, color: "#f8f264", fontColor: "green" }
                ],
                /**		
            * Bind string value to text field of schedule.
            * @name ejSchedule#categorizeSettings->text
            * @default null
            * @type {string}
            * @example       
            * &lt;div id="Schedule"&gt;&lt;/div&gt; <br> 
            * &lt;script&gt;
            * $("#Schedule").ejSchedule({
            *   categorizeSettings:{
            *    enable:true,
            *    allowMultiple:false,
            *      dataSource:[
            *     { text: "Blue Category", id: 1, color: "#7499e1", fontColor: "green" },
            *     { text: "Green Category", id: 2, color: "#7cce6e", fontColor: "green" },
            *     { text: "Orange Category", id: 3, color: "#f19d5a", fontColor: "green" },
            *     { text: "Orange Category", id: 3, color: "#f19d5a", fontColor: "green" },
            *     { text: "Orange Category", id: 3, color: "#f19d5a", fontColor: "green" },
            *     { text: "Purple Category", id: 4, color: "#937bd1", fontColor: "green" },
            *     { text: "Red Category", id: 5, color: "#d98889", fontColor: "green" },
            *     { text: "Yellow Category", id: 6, color: "#f8f264", fontColor: "green" }                
            *     ],            
            *      text: "text",
            *      id:"id",
            *      color:"color",
            *      fontColor:"fontColor"
            *       }
            *      }                                      
            *    });
            * &lt;/script&gt;
            * @memberof ejSchedule
            * @instance
            */

            text: "text",
            /**		
            * Bind string value to id field of schedule.
            * @name ejSchedule#categorizeSettings->id
            * @default null
            * @type {string}
            * @example       
            * &lt;div id="Schedule"&gt;&lt;/div&gt; <br> 
            * &lt;script&gt;
            * $("#Schedule").ejSchedule({
            *   categorizeSettings:{
            *    enable:true,
            *    allowMultiple:false,
            *      dataSource:[
            *     { text: "Blue Category", id: 1, color: "#7499e1", fontColor: "green" },
            *     { text: "Green Category", id: 2, color: "#7cce6e", fontColor: "green" },
            *     { text: "Orange Category", id: 3, color: "#f19d5a", fontColor: "green" },
            *     { text: "Orange Category", id: 3, color: "#f19d5a", fontColor: "green" },
            *     { text: "Orange Category", id: 3, color: "#f19d5a", fontColor: "green" },
            *     { text: "Purple Category", id: 4, color: "#937bd1", fontColor: "green" },
            *     { text: "Red Category", id: 5, color: "#d98889", fontColor: "green" },
            *     { text: "Yellow Category", id: 6, color: "#f8f264", fontColor: "green" }                
            *     ],            
            *      text: "text",
            *      id:"id",
            *      color:"color",
            *      fontColor:"fontColor"
            *       }
            *      }                                      
            *    });
            * &lt;/script&gt;
            * @memberof ejSchedule
            * @instance
            */
            id: "id",
                    /**		
            * Bind string value to color field of schedule.
            * @name ejSchedule#categorizeSettings->color
            * @default null
            * @type {string}
            * @example       
            * &lt;div id="Schedule"&gt;&lt;/div&gt; <br> 
            * &lt;script&gt;
            * $("#Schedule").ejSchedule({
            *   categorizeSettings:{
            *    enable:true,
            *    allowMultiple:false,
            *      dataSource:[
            *     { text: "Blue Category", id: 1, color: "#7499e1", fontColor: "green" },
            *     { text: "Green Category", id: 2, color: "#7cce6e", fontColor: "green" },
            *     { text: "Orange Category", id: 3, color: "#f19d5a", fontColor: "green" },
            *     { text: "Orange Category", id: 3, color: "#f19d5a", fontColor: "green" },
            *     { text: "Orange Category", id: 3, color: "#f19d5a", fontColor: "green" },
            *     { text: "Purple Category", id: 4, color: "#937bd1", fontColor: "green" },
            *     { text: "Red Category", id: 5, color: "#d98889", fontColor: "green" },
            *     { text: "Yellow Category", id: 6, color: "#f8f264", fontColor: "green" }                
            *     ],            
            *      text: "text",
            *      id:"id",
            *      color:"color",
            *      fontColor:"fontColor"
            *       }
            *      }                                      
            *    });
            * &lt;/script&gt;
            * @memberof ejSchedule
            * @instance
            */
            color: "color",
                    /**		
            * Bind string value to fontColor field of schedule.
            * @name ejSchedule#categorizeSettings->fontColor
            * @default null
            * @type {string}
            * @example       
            * &lt;div id="Schedule"&gt;&lt;/div&gt; <br> 
            * &lt;script&gt;
            * $("#Schedule").ejSchedule({
            *   categorizeSettings:{
            *    enable:true,
            *    allowMultiple:false,
            *      dataSource:[
            *     { text: "Blue Category", id: 1, color: "#7499e1", fontColor: "green" },
            *     { text: "Green Category", id: 2, color: "#7cce6e", fontColor: "green" },
            *     { text: "Orange Category", id: 3, color: "#f19d5a", fontColor: "green" },
            *     { text: "Orange Category", id: 3, color: "#f19d5a", fontColor: "green" },
            *     { text: "Orange Category", id: 3, color: "#f19d5a", fontColor: "green" },
            *     { text: "Purple Category", id: 4, color: "#937bd1", fontColor: "green" },
            *     { text: "Red Category", id: 5, color: "#d98889", fontColor: "green" },
            *     { text: "Yellow Category", id: 6, color: "#f8f264", fontColor: "green" }                
            *     ],            
            *      text: "text",
            *      id:"id",
            *      color:"color",
            *      fontColor:"fontColor"
            *       }
            *      }                                      
            *    });
            * &lt;/script&gt;
            * @memberof ejSchedule
            * @instance
            */
            fontColor:"fontColor"
            },
        /**    
        * Triggered after the cell is clicked.
        * @event
        * @name ejSchedule#cellClick 	
        * @param {Object} argument.object Returns the object of cellClick event.  
        * @param {boolean} argument.cancel Returns the cancel option value.
        * @param {object} argument.endTime Returns the end time of the clicked cell.
        * @param {object} argument.model Returns the schedule model.
        * @param {object} argument.startTime Returns the start time of the clicked cell.
        * @param {object} argument.target Returns the target of the clicked cell.
        * @param {string} argument.type Returns the name of the event.
        * @example
        * &lt;div id="Schedule"&gt;&lt;/div&gt; <br> 
        * &lt;script&gt;
        * $("#Schedule").ejSchedule({
        *    cellClick: function (args) {}
        * });
        * &lt;/script&gt;
        * @memberof ejSchedule
        * @instance
        */
        cellClick: null,
        /**    
        * Triggered after the appointment is clicked.
        * @event
        * @name ejSchedule#appointmentClick 	
        * @param {Object} argument.object Returns the object of appointmentClick event.  
        * @param {object} argument.appointment Returns the clicked appointment object.
        * @param {boolean} argument.cancel Returns the cancel option value.
        * @param {object} argument.model Returns the schedule model.
        * @param {string} argument.type Returns the name of the event.
        * @example 
        * &lt;div id="Schedule"&gt;&lt;/div&gt; <br> 
        * &lt;script&gt;
        * $("#Schedule").ejSchedule({
        *     appointmentSettings: { dataSource: window.Default, //collection of object from dataSource.
        *               id: "Id",
        *               subject: "Subject",
        *               description: "Description",
        *               startTime: "StartTime",
        *               endTime: "EndTime",
        *               recurrence: "Recurrence",
        *               recurrenceRule: "RecurrenceRule",
        *               allDay: "AllDay"
        *             },   
        *    appointmentClick: function (args) {}
        * });
        * &lt;/script&gt;
        * @memberof ejSchedule
        * @instance
        */
        appointmentClick: null,
        /**    
        * Triggered after the cell is clicked twice.
        * @event
        * @name ejSchedule#cellDoubleClick 	
        * @param {Object} argument.object Returns the object of cellDoubleClick event.  
        * @param {boolean} argument.cancel Returns the cancel option value.
        * @param {object} argument.endTime Returns the end time of the double clicked cell.
        * @param {object} argument.model Returns the schedule model.
        * @param {object} argument.startTime Returns the start time of the double clicked cell.
        * @param {object} argument.target Returns the target of the double clicked cell.
        * @param {string} argument.type Returns the name of the event.
        * @example 
        * &lt;div id="Schedule"&gt;&lt;/div&gt; <br> 
        * &lt;script&gt;
        * $("#Schedule").ejSchedule({
        *    cellDoubleClick: function (args) {}
        * });
        * &lt;/script&gt;
        * @memberof ejSchedule
        * @instance
        */
        cellDoubleClick: null,
        /**    
        * Triggered before the appointment window is opened.
        * @event
        * @name ejSchedule#appointmentWindowOpen 	
        * @param {Object} argument.object returns the object of appointmentWindowOpen event while select detail option from quick window.  
        * @param {boolean} argument.cancel Returns the cancel option value.
        * @param {object} argument.endTime Returns the end time of the double clicked cell.
        * @param {object} argument.model Returns the schedule model.
        * @param {string} argument.originalEventType Returns the action name which triggers window open.
        * @param {object} argument.startTime Returns the start time of the double clicked cell.
        * @param {object} argument.target Returns the target of the double clicked cell.
        * @param {string} argument.type Returns the name of the event.
        
        * @param {Object} argument.object Returns the object of edit appointmentWindowOpen event while select edit appointment/edit series option.  
        * @param {object} argument.appointment Returns the edit appointment object.
        * @param {boolean} argument.cancel Returns the cancel option value.
        * @param {boolean} argument.edit Returns the edit occurrence option value.
        * @param {object} argument.model Returns the schedule model.
        * @param {string} argument.type Returns the name of the event.
        
        * @example 
        * &lt;div id="Schedule"&gt;&lt;/div&gt; <br> 
        * &lt;script&gt;
        * $("#Schedule").ejSchedule({
        *    appointmentWindowOpen: function (args) {}
        * });
        * &lt;/script&gt;
        * @memberof ejSchedule
        * @instance
        */
        appointmentWindowOpen: null,
        /**    
        * Triggered after the appointment is saved.
        * @event
        * @name ejSchedule#appointmentSaved	
        * @param {Object} argument.object Returns the object of appointmentSaved event using appointment window.  
        * @param {object} argument.appointment Returns the saved appintment object.
        * @param {boolean} argument.cancel Returns the cancel option value.
        * @param {object} argument.model Returns the schedule model.
        * @param {string} argument.type Returns the name of the event.
        
        * @param {Object} argument.object Returns the object of appointmentSaved event using quick window.  
        * @param {boolean} argument.cancel Returns the cancel option value.
        * @param {object} argument.appointment Returns the saved appintment object.
        * @param {object} argument.model Returns the schedule model.
        * @param {string} argument.type Returns the name of the event.
        
        * @example 
        * &lt;div id="Schedule"&gt;&lt;/div&gt; <br> 
        * &lt;script&gt;
        * $("#Schedule").ejSchedule({
        *    appointmentSaved: function (args) {}
        * });
        * &lt;/script&gt;
        * @memberof ejSchedule
        * @instance
        */
        appointmentSaved: null,
        /**    
        * Triggered after the appointment is edited.
        * @event
        * @name ejSchedule#appointmentEdited	
        * @param {Object} argument.object Returns the object of appointmentEdited event.  
        * @param {object} argument.appointment Returns the edited appintment object.
        * @param {boolean} argument.cancel Returns the cancel option value.
        * @param {object} argument.model Returns the schedule model.
        * @param {string} argument.type Returns the name of the event.
        * @example 
        * &lt;div id="Schedule"&gt;&lt;/div&gt; <br> 
        * &lt;script&gt;
        * $("#Schedule").ejSchedule({
        *     appointmentSettings: { dataSource: window.Default, //collection of object from dataSource.
        *               id: "Id",
        *               subject: "Subject",
        *               description: "Description",
        *               startTime: "StartTime",
        *               endTime: "EndTime",
        *               recurrence: "Recurrence",
        *               recurrenceRule: "RecurrenceRule",
        *               allDay: "AllDay"
        *             },
        *    appointmentEdited: function (args) {}
        * });
        * &lt;/script&gt;
        * @memberof ejSchedule
        * @instance
        */
        appointmentEdited: null,
        /**    
        			* Triggered after the appointment is deleted.
        			* @event
        			* @name ejSchedule#appointmentDeleted	
        			* @param {Object} argument.object Returns the object of appointmentDeleted event.
        			* @param {boolean} argument.cancel Returns the cancel option value.
                    * @param {object} argument.appointment Returns the deleted appintment object.
        			* @param {object} argument.model Returns the schedule model.
        			* @param {string} argument.type Returns the name of the event.
        			* @example 
                    * &lt;div id="Schedule"&gt;&lt;/div&gt; <br> 
                    * &lt;script&gt;
                    * $("#Schedule").ejSchedule({
                    *     appointmentSettings: { dataSource: window.Default, //collection of object from dataSource.
                    *               id: "Id",
                    *               subject: "Subject",
                    *               description: "Description",
                    *               startTime: "StartTime",
                    *               endTime: "EndTime",
                    *               recurrence: "Recurrence",
                    *               recurrenceRule: "RecurrenceRule",
                    *               allDay: "AllDay"
                    *             },
        			*    appointmentDeleted: function (args) {}
        			* });
                    * &lt;/script&gt;
        * @memberof ejSchedule
        * @instance
        */
        appointmentDeleted: null,
        /**    
        			* Triggered after the navigate the view or date of the schedule.
        			* @event
        			* @name ejSchedule#navigation
        			* @param {Object} argument.object Returns the object of schedule view navigation event.
                    * @param {object} argument.Date Returns the current date object.
        			* @param {boolean} argument.cancel Returns the cancel option value.
        			* @param {object} argument.model Returns the schedule model.
                    * @param {string} argument.currentView Returns the current view value.
                    * @param {string} argument.prevView Returns the previous view value.
                    * @param {object} argument.target Returns the target of the action.
        			* @param {string} argument.type Returns the name of the event.
        
                    * @param {Object} argument.object Returns the object of previous/next date navigation event.  
        			* @param {boolean} argument.cancel Returns the cancel option value.
        			* @param {object} argument.model Returns the schedule model.
                    * @param {object} argument.currentDate Returns the new date of the schedule.
                    * @param {object} argument.prevDate Returns the previous date of the schedule.
                    * @param {object} argument.target Returns the target of the action.
        			* @param {string} argument.type Returns the name of the event.
        
                    * @param {Object} argument.object Returns the navigation event while change the date using calendar in schedule.  
        			* @param {boolean} argument.cancel Returns the cancel option value.
        			* @param {object} argument.model Returns the schedule model.
                    * @param {object} argument.currentDate Returns the new date of the schedule.
                    * @param {object} argument.prevDate Returns the previous date of the schedule.
                    * @param {object} argument.target Returns the target of the action.
        			* @param {string} argument.type Returns the name of the event.
        
        			* @example 
                    * &lt;div id="Schedule"&gt;&lt;/div&gt; <br> 
                    * &lt;script&gt;
                    * $("#Schedule").ejSchedule({
        			*    navigation: function (args) {}
        			* });
                    * &lt;/script&gt;
        * @memberof ejSchedule
        * @instance
        */
        navigation: null,
        /**     
        * Triggered when the appointment is start dragging. 
        * @event
        * @name ejSchedule#dragStart		
        * @param {Object} argument.object Returns the object of dargStart event.
			* @param {boolean} argument.cancel Returns the cancel option value.
			* @param {object} argument.model Returns the schedule model.
            * @param {object} argument.target Returns the target of the dragging appointment.
			* @param {string} argument.type Returns the name of the event.
        * @example 
        * &lt;div id="Schedule"&gt;&lt;/div&gt; <br> 
            * &lt;script&gt;
            * $("#Schedule").ejSchedule({
            *     appointmentSettings: { dataSource: window.Default, //collection of object from dataSource.
            *               id: "Id",
            *               subject: "Subject",
            *               description: "Description",
            *               startTime: "StartTime",
            *               endTime: "EndTime",
            *               recurrence: "Recurrence",
            *               recurrenceRule: "RecurrenceRule",
            *               allDay: "AllDay"
            *             },
        *    dragStart: function (args) {}
        * });
        * &lt;/script&gt;
        * @memberof ejSchedule
        * @instance
        */
        dragStart: null,
        /**     
        * Triggered when the appointment is drag over.
        * @event
        * @name ejSchedule#dragOver
            * @param {Object} argument.object Returns the object of dragOver event.  
			* @param {boolean} argument.cancel Returns the cancel option value.
			* @param {object} argument.model Returns the schedule model.
            * @param {object} argument.target Returns the target of the drag over appointment.
			* @param {string} argument.type Returns the name of the event.
        * @example 
        * &lt;div id="Schedule"&gt;&lt;/div&gt; <br> 
            * &lt;script&gt;
            * $("#Schedule").ejSchedule({
            *     appointmentSettings: { dataSource: window.Default, //collection of object from dataSource.
            *               id: "Id",
            *               subject: "Subject",
            *               description: "Description",
            *               startTime: "StartTime",
            *               endTime: "EndTime",
            *               recurrence: "Recurrence",
            *               recurrenceRule: "RecurrenceRule",
            *               allDay: "AllDay"
            *             },
        *    dragOver: function (args) {}
        * });
        * &lt;/script&gt;
        * @memberof ejSchedule
        * @instance
        */
        dragOver: null,
        /**     
        * Triggered when the appointment drop. 
        * @event
        * @name ejSchedule#dragDrop
            * @param {Object} argument.object Returns the object of dragDrop event.
            * @param {object} argument.appointment Returns the dropped appointment object.
			* @param {boolean} argument.cancel Returns the cancel option value.
			* @param {object} argument.model Returns the schedule model.
			* @param {string} argument.type Returns the name of the event.
        * @example 
        * &lt;div id="Schedule"&gt;&lt;/div&gt; <br> 
            * &lt;script&gt;
            * $("#Schedule").ejSchedule({
            *     appointmentSettings: { dataSource: window.Default, //collection of object from dataSource.
            *               id: "Id",
            *               subject: "Subject",
            *               description: "Description",
            *               startTime: "StartTime",
            *               endTime: "EndTime",
            *               recurrence: "Recurrence",
            *               recurrenceRule: "RecurrenceRule",
            *               allDay: "AllDay"
            *             },
        *    dragDrop: function (args) {}
        * });
        * &lt;/script&gt;
        * @memberof ejSchedule
        * @instance
        */
        dragDrop: null,
        /**     
        * Triggered when start resizing the appointment. 
        * @event
        * @name ejSchedule#resizeStart
            * @param {Object} argument.object Returns the object of resizeStart event.
			* @param {boolean} argument.cancel Returns the cancel option value.
            * @param {object} argument.element Returns the resize element value.
			* @param {object} argument.model Returns the schedule model.
			* @param {string} argument.type Returns the name of the event.
        * @example 
        * &lt;div id="Schedule"&gt;&lt;/div&gt; <br> 
            * &lt;script&gt;
            * $("#Schedule").ejSchedule({
            *     appointmentSettings: { dataSource: window.Default, //collection of object from dataSource.
            *               id: "Id",
            *               subject: "Subject",
            *               description: "Description",
            *               startTime: "StartTime",
            *               endTime: "EndTime",
            *               recurrence: "Recurrence",
            *               recurrenceRule: "RecurrenceRule",
            *               allDay: "AllDay"
            *             },
        *    resizeStart: function (args) {}
        * });
        * &lt;/script&gt;
        * @memberof ejSchedule
        * @instance
        */
        resizeStart: null,
        /**     
        * Triggered when resizing the appointment. 
        * @event
        * @name ejSchedule#resizing
            * @param {Object} argument.object Returns the object of resizing event.
			* @param {boolean} argument.cancel Returns the cancel option value.
            * @param {object} argument.element Returns the resize element value.
			* @param {object} argument.model Returns the schedule model.
			* @param {string} argument.type Returns the name of the event.
        * @example 
        * &lt;div id="Schedule"&gt;&lt;/div&gt; <br> 
            * &lt;script&gt;
            * $("#Schedule").ejSchedule({
            *     appointmentSettings: { dataSource: window.Default, //collection of object from dataSource.
            *               id: "Id",
            *               subject: "Subject",
            *               description: "Description",
            *               startTime: "StartTime",
            *               endTime: "EndTime",
            *               recurrence: "Recurrence",
            *               recurrenceRule: "RecurrenceRule",
            *               allDay: "AllDay"
            *             },
        *    resizing: function (args) {}
        * });
        * &lt;/script&gt;
        * @memberof ejSchedule
        * @instance
        */
        resizing: null,
        /**     
        * Triggered when stop resizing the appointment. 
        * @event
        * @name ejSchedule#resizeStop
            * @param {Object} argument.object Returns the object of resizeStop event
            * @param {object} argument.appointment Returns the resized appointment value.
			* @param {boolean} argument.cancel Returns the cancel option value.
			* @param {object} argument.model Returns the schedule model.
            * @param {object} argument.target Returns the target of the resized appointment.
			* @param {string} argument.type Returns the name of the event.
        * @example 
        * &lt;div id="Schedule"&gt;&lt;/div&gt; <br>
            * &lt;script&gt;
            * $("#Schedule").ejSchedule({
            *     appointmentSettings: { dataSource: window.Default, //collection of object from dataSource.
            *               id: "Id",
            *               subject: "Subject",
            *               description: "Description",
            *               startTime: "StartTime",
            *               endTime: "EndTime",
            *               recurrence: "Recurrence",
            *               recurrenceRule: "RecurrenceRule",
            *               allDay: "AllDay"
            *             },
        *    resizeStop: function (args) {}
        * });
        * &lt;/script&gt;
        * @memberof ejSchedule
        * @instance
        */
        resizeStop: null,
        /**    
        * Triggered after the context menu is clicked.
        * @event
        * @name ejSchedule#menuItemClick
        * @param {Object} argument.object Returns the object of menuItemClick event.  
        * @param {boolean} argument.cancel Returns the cancel option value.
        * @param {object} argument.events Returns the object of menu item event.
        * @param {object} argument.model Returns the schedule model.
        * @param {string} argument.type Returns the name of the event.
        * @example
        * &lt;div id="Schedule"&gt;&lt;/div&gt; <br> 
        * &lt;script&gt;
        * $("#Schedule").ejSchedule({
        *    menuItemClick: function (args) {}
        * });
        * &lt;/script&gt;
        * @memberof ejSchedule
        * @instance
        */
        menuItemClick: null,
        /**    
        * Triggered before the context menu open.
        * @event
        * @name ejSchedule#beforeContextMenuOpen
        * @param {Object} argument.object Returns the object of beforeContextMenuOpen event.  
        * @param {boolean} argument.cancel Returns the cancel option value.
        * @param {object} argument.events Returns the object of before open menu target.
        * @param {object} argument.model Returns the schedule model.
        * @param {string} argument.type Returns the name of the event.
        * @example
        * &lt;div id="Schedule"&gt;&lt;/div&gt; <br> 
        * &lt;script&gt;
        * $("#Schedule").ejSchedule({
        *    beforeContextMenuOpen: function (args) {}
        * });
        * &lt;/script&gt;
        * @memberof ejSchedule
        * @instance
        */
        beforeContextMenuOpen: null,
        /**    
        * Triggers when the reminder is raised for an appointment.
        * @event
        * @name ejSchedule#reminder
        * @param {boolean} argument.cancel Returns the cancel option value.
        * @param {object} argument.model Returns the schedule model.
        * @param {string} argument.type Returns the name of the event.
        * @param {object} argument.reminderAppointment Returns the appointment object for which the reminder is raised.
        * @example
        * &lt;div id="Schedule"&gt;&lt;/div&gt; <br> 
        * &lt;script&gt;
        * $("#Schedule").ejSchedule({
        *    reminder: function (args) {}
        * });
        * &lt;/script&gt;
        * @memberof ejSchedule
        * @instance
        */
        reminder: null
    },
        dataTypes: {
            views: "array",
            timezoneCollection: "array",
            //timeMode: "enum",
            //currentView: "enum",
            resources: "array",
            resources: {
                resourceSettings:"arary",
            },
            group: {
                resources: "array"
            },
            appointmentSettings: {
                dataSource: "data",
                query: "data"
            },
            contextMenuSettings: {
                menuItems: {
                    appointment: "array",
                    cells:"array",
                }
            }
        },
    _ignoreFromAppointmentOnExport:["dataSource","query","tableName","resourceFields"],
    observables: ["appointmentSettings.dataSource", "currentView", "currentDate"],
    currentView: ej.util.valueFunction("currentView"),
    currentDate: ej.util.valueFunction("currentDate"),
    dataSource: ej.util.valueFunction("appointmentSettings.dataSource"),

    // constructor function
    /**
    * Create the schedule widget
    * @private
    */
    _init: function () {
            this._trigger("load");
            this.element.addClass("e-schedule"); //While resize the window the root class has been removed. 
            if (ej.isNullOrUndefined(this.model.appointmentSettings.query) || !(this.model.appointmentSettings.query instanceof ej.Query))
                this.model.appointmentSettings.query = ej.Query();
            (this.model.cssClass != null) && this.element.addClass(this.model.cssClass);
            this._initPrivateProperties();
            
            if (ej.scheduleFeatures.resources && !ej.isNullOrUndefined(this.model.resources))
                this._bindResourcesData();
            else
                this._renderInitSchedule();
            this._wireEvents();
            
            !ej.isNullOrUndefined(this.model.appointmentSettings) && (this._dataManager = this.dataSource() instanceof ej.DataManager ? this.dataSource() : this.dataSource() != null ? ej.DataManager(this.dataSource()) : null);
            this._bindAppointmentsData();
        this.model.enableRTL && this.element.addClass("e-rtl");
        //Reminder options
        if (this.model.reminderSettings.enable) {
            this._checkReminders();
            this._reminderInterval = window.setInterval($.proxy(this._checkReminders, this), 60000);
        }
    },

    /**
    * To configure the properties at runtime using SetModel		
    * @private
    */
    _setModel: function (options) {
        var flag = false, _resFlag = false;
        for (var prop in options) {
            switch (prop) {
                case "views":
                    this.hTD.find("table.e-viewstable").remove();
                    this.hTD.find("div.e-viewsdiv").append(this._renderViewNavigation(options[prop]));
                    break;
                case "currentView":
                    this._navView = true;
                    this._viewChange(null, null, ej.util.getVal(options[prop]));
                    break;
                case "currentDate":
                    this._setCurrentDate = true;
                    this._navigateClick(null);
                    break;
                case "timeMode":
                    this.model.timeMode = options[prop];
                    this._appointmentAddWindow.find('.startendtime').ejTimePicker({timeFormat: (this.model.timeMode == "12") ? this._pattern.t : "HH:mm"});
                    if (this.currentView() !== "month") {
                        var viewclass = this.currentView() == "month" ? 1 : 0; //"e-monthtimecells" : "e-timecelldivs";
                        var timeCellhtml = this.timeCellTemplate.render({ timerows: this._getStartEndTime(), cols1: [1, 2], viewclass: viewclass });
                        this.$TimeCellTD.children().remove();
                        this.$TimeCellTD.append(timeCellhtml); if (this.model.showCurrentTimeIndicator) $(".e-highlightcurrenttimeline").remove(); 
                        this.element.find(".e-scrolltimecells").css("top", -(this.element.find(".e-draggableworkarea").scrollTop()) + "px");
                        this._highlightCurrentTime(this.model.showCurrentTimeIndicator);
                        if (!ej.isNullOrUndefined(this.dataSource()))
                           this._renderAppointmentAll();
                    }
                    break;
                case "timeZone":
                    this.model.timeZone = options[prop];
                    this.element.find("div.e-categorybar").remove();
                    this.element.find("div.e-appointment,div.e-alldayappointment,div.e-monthappointment").remove();
                    this._bindAppointmentsData();
                    break;
                case "startHour":
                    this.model.startHour = options[prop];
                    this._setNewStartEndHours();
                    break;
                case "endHour":
                    this.model.endHour = options[prop];
                    this._setNewStartEndHours();
                    break;
                case "highlightBusinessHours":
                    this.model.highlightBusinessHours = options[prop];
                    (this.model.highlightBusinessHours) ? this._highlightBusinessHours() : this.element.find(".e-workcells").removeClass("e-businesshighlightworkcells") && this.element.find(".e-monthcells").removeClass("e-monthcellhighlight");
                    break;
                case "businessStartHour":
                    this.model.businessStartHour = options[prop];
                    this.currentView() !== "month" ? this.element.find(".e-workcells").removeClass("e-businesshighlightworkcells") : this.element.find(".e-monthcells").removeClass("e-monthcellhighlight");
                    this.model.highlightBusinessHours && this._highlightBusinessHours();
                    break;
                case "businessEndHour":
                    this.model.businessEndHour = options[prop];
                    this.currentView() !== "month" ? this.element.find(".e-workcells").removeClass("e-businesshighlightworkcells") : this.element.find(".e-monthcells").removeClass("e-monthcellhighlight");
                    this.model.highlightBusinessHours && this._highlightBusinessHours();
                    break;
                case "height":
                    this.model.height = options[prop];
                    if (this.model.showCurrentTimeIndicator) $(".e-highlightcurrenttimeline").remove(); 
                    flag = true;
                    break;
                case "width":
                    this.model.width = options[prop];
                    if (this.model.showCurrentTimeIndicator) $(".e-highlightcurrenttimeline").remove(); 
                    flag = true;
                    break;
                case "cssClass":
                    this.element.removeClass(this.model.cssClass).addClass(options[prop]);
                    break;
                case "locale":
                    this.model.locale = options[prop];
                    var model = this.model;
                    this.element.ejSchedule("destroy").ejSchedule(model);
                    break;
                case "enableResize":
                    this.model.enableResize = options[prop];
                    if (this.model.enableResize) {
                        !this._isDevice() ? this._on($(window), "resize", this._scheduleResize) : this._on($(window), "orientationchange", this._scheduleResize);
                        this._wireResizeEvents();
                    }
                    break;
                case "showCurrentTimeIndicator":
                    this.model.showCurrentTimeIndicator = options[prop];
                    this._highlightCurrentTime(this.model.showCurrentTimeIndicator);
                    if (!this.model.showCurrentTimeIndicator) this.currentView() !== "month" ? $(".e-highlightcurrenttimeline").remove() : $(this.element.find(".e-monthcells")).removeClass("e-monthcurrenttime");
                    break;
                case "enableRTL":
                    options[prop] ? this.element.addClass("e-rtl") : this.element.removeClass("e-rtl");
                    this.model.enableRTL = options[prop];
                    var model = this.model;
                    this.element.ejSchedule("destroy").ejSchedule(model);
                    break;
                case "appointmentTemplateId":
                    this.model.appointmentTemplateId = options[prop];
                    this.element.find("div.e-appointment,div.e-alldayappointment,div.e-monthappointment").remove();
                    this.element.find("div.e-categorybar").remove();
                    if (!ej.isNullOrUndefined(this.dataSource()))
                    this._renderAppointmentAll();
                    break;
                case "allowKeyboardNavigation":
                    this.model.allowKeyboardNavigation = options[prop];
                    if (this.model.allowKeyboardNavigation) {
                        this._on(this.element, "keydown", $.proxy(this._onKeyDown, this));
                        this._on($(document), "keydown", $.proxy(this._documentOnKeyDown, this));
                    }
                    else {
                        this._off(this.element, "keydown");
                        this._off($(document), "keydown");
                    }
                    break;
                case "appointmentSettings":
                    $.extend(this.model.appointmentSettings, options[prop]);
                    this.element.find("div.e-categorybar").remove();
                    this.element.find("div.e-appointment,div.e-alldayappointment,div.e-monthappointment,div.e-appsoverflow").remove();
                    this._bindAppointmentsData();
                    break;
                case "contextMenuSettings":
                    $.extend(this.model.contextMenuSettings, options[prop]);
                    if (this.model.contextMenuSettings.enable && ej.isNullOrUndefined(this._contextMenu))
                        this._showContextMenu(this.model.contextMenuSettings.enable);
                    break;
                case "reminderSettings":
                    if (options[prop].enable)
                        this._checkReminders() && (this._reminderInterval = window.setInterval($.proxy(this._checkReminders, this), 60000));
                    else
                        window.clearInterval(this._reminderInterval);
                    break;
                case "enableAppointmentNavigation":
                    this.model.enableAppointmentNavigation = options[prop];
                    if(!this.model.enableAppointmentNavigation)
                        this.element.find(".e-prev,.e-next").remove();
                    else{
                        this._nextPrevAppointment(this.model.enableAppointmentNavigation);
                        if (!ej.isNullOrUndefined(this.dataSource()))
                            this._renderAppointmentAll();
                    }
                    break;
                case "resources":
                    if (ej.scheduleFeatures.resources && !ej.isNullOrUndefined(this.model.resources)) {
                        $.extend(this.model.resources, options[prop]);
                        flag = true; _resFlag = true;
                    }
                    break;
                case "categorizeSettings":
                    $.extend(this.model.categorizeSettings, options[prop]);
                    if (!ej.isNullOrUndefined(this.model.categorizeSettings)) {                      
                        if (this._appointmentAddWindow.find('.e-categorizevalue').length == 0) {
                            $appWindow = "<tr><td class='e-leftfields e-textlabel'>" + this._getLocalizedLabels("Categorize") + ":</td><td colspan='3' class='e-rightfields'>" + "<input id= '" + this._id + "_categorize' class='e-categorizevalue' type='text' value=''/><div id='ownerlist'></div></td></tr>";
                            this._appointmentAddWindow.find('#' + this._id + "summary").after($appWindow);
                            this._renderTemplates();
                            var control = this._appointmentAddWindow;
                            this._renderCategoryItems(control);
                        }
                        this._showContextMenu(this.model.contextMenuSettings.enable);
                        if (!ej.isNullOrUndefined(this.dataSource()))
                            this._renderAppointmentAll();
                    }
                    break;

                case "group":
                    if (ej.scheduleFeatures.resources && !ej.isNullOrUndefined(this.model.resources)) {
                        $.extend(this.model.group, options[prop]);
                        flag = true; _resFlag = true;
                    }
                    break;
                case "cellWidth":
                    this.model.cellWidth = options[prop];
                    if (this.model.showCurrentTimeIndicator) $(".e-highlightcurrenttimeline").remove();
                    flag = true;
                    break;
                case "cellHeight":
                    this.model.cellHeight = options[prop];
                    if (this.model.showCurrentTimeIndicator) $(".e-highlightcurrenttimeline").remove();
                    flag = true;
                    break;
                case "orientation":
                    this.model.orientation = options[prop];
                    flag = true;
                    break;
                case "resourceHeaderTemplateId":
                    this.model.resourceHeaderTemplateId = options[prop];
                    if (ej.scheduleFeatures.resources && !ej.isNullOrUndefined(this.model.group) && !ej.isNullOrUndefined(this.model.resources)) {
                        if (this.model.orientation == "horizontal") {
                            var resDiv = $(this.wTR.find("td")[0]);
                            resDiv.find("div.e-horizontresdiv").remove();
                            var resHtml = this._renderHorizontalResources();
                            resDiv.html(resHtml);
                        }
                        else {
                            this.aTR.empty();
                            this._renderHeaderAllDayTemplate();
                            this._headerbarWidth();
                            this.element.find(".e-schedulesettings").html(this._getLocalizedLabels("AllDay"));
                            this._renderAllDayAppointments();
                        }
                    }
                    break;
            }
        }
        if (flag) {
            this.element.empty();
            (!_resFlag) ? this._renderInitSchedule() : this._bindResourcesData();
            if (!ej.isNullOrUndefined(this.dataSource()))
            this._renderAppointmentAll();
        }
    },

    //Touch Schedule:
    /**
    * To configure the touch for schedule		
    * @private
    */
    _touchSchedule: function (e) {
        var event;
        switch (e.type) {
            case "swipeleft":
                event = { currentTarget: this.element.find('.e-navigatenext') };
                this._navigateClick(event);
                break;
            case "swiperight":
                event = { currentTarget: this.element.find('.e-navigateprevious') };
                this._navigateClick(event);
                break;
        }
    },

    /**
    * To configure the private properties of schedule		
    * @private
    */
    _initPrivateProperties: function () {
        this._appId = 0;
        this._parentId = 0;
        this._currentAction = "";
        this._reminderClose = false;
        this._currentAppointmentData = null;
        this._processed = [];
        this._dateRender = [];
        this._rRule = {};
        this._dayNames = ["SU", "MO", "TU", "WE", "TH", "FR", "SA"];
        var culture = Globalize.culture(this.model.locale);
        this._dayShortNames = culture.calendar.days.namesAbbr;
        this._dayFullNames = culture.calendar.days.names;
        this._monthNames = culture.calendar.months.names;
        this._pattern = culture.calendar.patterns;
        this._navView = false;
        this._editKey = false;
        this._cellClick = false;
        this._gotoDate = true;
        this._recurChecked = false;
        this._currentRowIndex = 0;
        this._currentColIndex = 0;
        this._setCurrentDate = false;
        this._browserInfo = ej.browserInfo();
        this._leftResizing = false;
        this._rightResizing = false;
        this._topResizing = false;
        this._bottomResizing = false;
        this._focused = null;
    },
    /**
    * destroy the schedule widget
    * all events bound using this._on will be unbind automatically and bring the control to pre-init state.
    * @alias destroy
    * @return jQuery
    * @example 
    * &lt;div id="Schedule"&gt;&lt;/div&gt; <br> 
    * &lt;script&gt;
    * $('#Schedule').ejSchedule();
        * var schObj = $("#Schedule").data("ejSchedule");
    * schObj.destroy(); // destroy the schedule
    * &lt;/script&gt;
    * @memberof ejSchedule
    * @instance
    */
    _destroy: function () {
        //Appointment window and Edit window controls destory
        this._destorySubControls($("#" + this._id + "_AddEditForm"));
        this._destorySubControls($("#" + this._id + "_RecurForm"));
        this.element.find("#" + this._id + "_navigator").ejDatePicker("destroy");
        //Clear  set interval
        window.clearInterval(this._intervalID);
        //Remove dialogs
        $("#" + this._id + "AppointmentAddEditWindow_wrapper,#" + this._id + "AppointmentQuickWindow_wrapper,#" + this._id + "AppDetailsWindow_wrapper").remove();
        //Remove templates
        $("#" + this._id + "_WorkCells_Template,#" + this._id + "_MonthCells_Template,#" + this._id + "_TimeCells_Template,#" + this._id + "_HeaderDate_Template,#" + this._id + "_Appointment_Template,#" + this._id + "_AlldayCells_Template").remove();
        this.element.empty().removeClass("e-schedule");
    },

    /**
    * Destroy subcontrols.
    * @private
    */
    _destorySubControls: function (formelement) {
        $formEle = formelement.find('.e-timepicker,.e-datepicker,.e-dropdownlist,.e-autocomplete');
        for (var index = 0; index < $formEle.length; index++) {
            $element = $($formEle[index]);
            if ($element.hasClass('e-timepicker'))
                $element.ejTimePicker("destroy");
            else if ($element.hasClass('e-datepicker'))
                $element.ejDatePicker("destroy");
            else if ($element.hasClass('e-dropdownlist'))
                $element.ejDropDownList("destroy");
            else if ($element.hasClass('e-autocomplete'))
                $element.ejAutocomplete("destroy");
        }
    },

    /**
    * Wiring the events to schedule control		
    * @private
    */
    _wireEvents: function () {
        this._on((this.element, $(window)), "mousedown", $.proxy(this._onScheduleMouseDown, this));
        this._on(this.element, "click", "div.e-navigateprevious,div.e-navigatenext", $.proxy(this._navigateClick, this));
        this._on(this.element, "click", ".e-commonviewbutton", $.proxy(this._viewChange, this));
        this._on(this.element, "click", ".e-datedisplay", $.proxy(this._viewCalendar, this));
        this._on(this.element, "click", "div.e-appsoverflow", $.proxy(this._appOverflowClick, this));
        //next/prev event
        this._on(this.element, "click", ".e-prev.enable,.e-next.enable", $.proxy(this._nextPrev, this));
        //Work Cell events
        this._on(this.element, "dblclick", ".e-headercells", $.proxy(this._navigateView, this));
        this._on(this.element, "click", ".e-headercells,.e-timecells,td.e-monthcells div.e-monthheader", $.proxy(this._navigateView, this));
        this._on(this.element, "mouseover", "td.e-workcells,td.e-monthcells", $.proxy(this._onCellMouseOver, this));
        this._on(this.element, "mouseout", "td.e-workcells,td.e-monthcells", $.proxy(this._onCellMouseOver, this));
        this._on(this.element, $.isFunction($.fn.tap) ? "tap" : "click", ".e-workcells,.e-alldaycells,.e-alldayappointment,.e-monthcells,div.e-monthappointment", $.proxy(this._cellSelection, this));
        this._on(this.element, ej.eventType.mouseDown, "td.e-workcells,td.e-monthcells", $.proxy(this._mouseButtonClick, this));
        this._on(this.element, "keydown click", "td.e-workcells,td.e-monthcells", $.proxy(this._shiftSelection, this));
        // Appointments events
        this._on(this.element, "dblclick", ".e-workcells,.e-alldaycells,.e-alldayappointment,div.e-monthappointment,.e-monthcells", $.proxy(this._appointmentWindow, this));
        this._on(this.element, "mouseover", ".e-appointment,.e-alldayappointment,.e-monthappointment", $.proxy(this._onAppointMouseOver, this));
        this._on(this.element, "mouseout", ".e-appointment,.e-alldayappointment,.e-monthappointment", $.proxy(this._onAppointMouseOver, this));
        // Keydown events
        if (this.model.allowKeyboardNavigation) {
            this._on(this.element, "keydown", $.proxy(this._onKeyDown, this));
            this._on($(document), "keydown", $.proxy(this._documentOnKeyDown, this));
        }
        if (this.model.orientation == "horizontal")
            this._on(this.element, "click", "td.e-parentnodecategory", $.proxy(this._onResourceClick, this));
        //Touch event.
        this._on(this.element, "swipeleft swiperight", ".e-contentarearow .e-workcellstab", $.proxy(this._touchSchedule, this));
        // Rezize events
        if (this.model.enableResize)
            !this._isDevice() ? this._on($(window), "resize", this._scheduleResize) : this._on($(window), "orientationchange", this._scheduleResize);
    },
    _mouseButtonClick: function (e) {
        if (!(e.altKey) && !(e.shiftKey) && !(e.ctrlKey) && (e.which != 3)) {
            if (e.which == 1 || e.type == "touchstart") {
                var oEl = e.target || document.activeElement || e.srcElement;
                this._mouseSelect(e, oEl);
            }
        }
    },
    _shiftSelection: function (e) {
        if (e.shiftKey == true && e.which == 1) {
            var oE1 = this._focused;
            this.model.showQuickWindow && (this._quickAppointWindow.ejDialog("close") && this._quickAppDetailsWindow.ejDialog("close"));
            this.element.find("e.workcells,e.alldaycells,.e-monthcells").removeClass("e-mousehover");
            this.element.find(".e-workcells,.e-alldaycells,.e-monthcells").removeClass("e-selectedCell");
            this.element.find('.e-selectedAppointment').removeClass("e-selectedAppointment");
            var oRowCell = this.element.find('div.e-draggableworkarea tr');
            var sourceRowIndex, sourceColumnIndex;
            sourceRowIndex = $(oE1).parent()[0].rowIndex;
            sourceColumnIndex = $(oE1)[0].cellIndex;
            var view = this.currentView();
            var rowcount = oRowCell.length - 1;
            var i, j, k;
            var currentRowIndex = $(e.target).parent()[0].rowIndex;
            var currentColumnIndex = $(e.target)[0].cellIndex;
            if ($(e.target).hasClass('e-workcells')) {
                var viewCount = view == "day" ? 1 : view == "workweek" ? 5 : 7;
                var resIndex = Math.floor($(e.target).index() / viewCount);
                var resmaxcolindex = (viewCount * (resIndex + 1));
                var resmincolindex = (viewCount * resIndex);
                if (sourceColumnIndex >= parseInt(resmincolindex) && sourceColumnIndex <= parseInt(resmaxcolindex) - 1 && currentColumnIndex != "undefined") {
                    if (sourceColumnIndex == currentColumnIndex) {
                        if (currentRowIndex > sourceRowIndex) {
                            for (k = parseInt(sourceRowIndex) ; k <= currentRowIndex; k++) {
                                $(oRowCell[k].cells[sourceColumnIndex]).addClass("e-selectedCell");
                            }
                        }
                        else if (currentRowIndex < sourceRowIndex) {
                            for (k = parseInt(sourceRowIndex) ; k >= currentRowIndex; k--) {
                                $(oRowCell[k].cells[sourceColumnIndex]).addClass("e-selectedCell");
                            }
                        }
                        else {
                            $(oRowCell[currentRowIndex].cells[currentColumnIndex]).addClass("e-selectedCell");
                        }
                    }
                    else if (currentColumnIndex > sourceColumnIndex) {
                        var difference = currentColumnIndex - sourceColumnIndex;
                        for (i = parseInt(sourceRowIndex) ; i <= rowcount; i++) {
                            $(oRowCell[i].cells[sourceColumnIndex]).addClass("e-selectedCell");
                        }
                        if (difference > 1) {
                            for (k = 1; k <= difference - 1; k++) {
                                for (var m = 0; m <= rowcount; m++) {
                                    $(oRowCell[m].cells[currentColumnIndex - k]).addClass("e-selectedCell");
                                }
                            }
                        }
                        for (j = 0; j <= currentRowIndex; j++) {
                            $(oRowCell[j].cells[currentColumnIndex]).addClass("e-selectedCell");
                        }
                    }
                    else if (currentColumnIndex < sourceColumnIndex) {
                        var difference = sourceColumnIndex - currentColumnIndex;
                        for (i = parseInt(sourceRowIndex) ; i >= 0 ; i--) {
                            $(oRowCell[i].cells[sourceColumnIndex]).addClass("e-selectedCell");
                        }
                        if (difference > 1) {
                            for (k = 1; k <= difference - 1; k++) {
                                for (var m = 0; m <= rowcount; m++) {
                                    $(oRowCell[m].cells[[currentColumnIndex + k]]).addClass("e-selectedCell");
                                }
                            }
                        }
                        for (j = parseInt(currentRowIndex) ; j <= rowcount; j++) {
                            $(oRowCell[j].cells[currentColumnIndex]).addClass("e-selectedCell");
                        }
                    }
                    this._currentRowIndex = currentRowIndex;
                    this._currentColIndex = currentColumnIndex;
                    $(oRowCell[currentRowIndex].cells[currentColumnIndex]).addClass("e-selectedCell").attr("tabIndex", 0).focus();
                }
                else { return false; }
            }
            else if ($(e.target).hasClass('e-monthcells')) {
                var resIndex = Math.floor($(e.target).index() / 7);
                var mincolindex = 7 * (parseInt(resIndex));
                var maxcolindex = 7 * (resIndex + 1);
                if (sourceColumnIndex >= parseInt(mincolindex) && sourceColumnIndex <= parseInt(maxcolindex) - 1 && currentColumnIndex != "undefined") {
                    if (currentRowIndex == sourceRowIndex) {
                        if (currentColumnIndex < sourceColumnIndex) {
                            for (k = sourceColumnIndex; k >= currentColumnIndex; k--) {
                                $(oRowCell[currentRowIndex].cells[k]).addClass("e-selectedCell");
                            }
                        }
                        else if (currentColumnIndex > sourceColumnIndex) {
                            for (k = sourceColumnIndex; k <= currentColumnIndex; k++) {
                                $(oRowCell[currentRowIndex].cells[k]).addClass("e-selectedCell");
                            }
                        }
                        else if (currentColumnIndex == sourceColumnIndex) {
                            $(oRowCell[currentRowIndex].cells[currentColumnIndex]).addClass("e-selectedCell");
                        }
                    }
                    else if (currentRowIndex > sourceRowIndex) {
                        var difference = (currentRowIndex - sourceRowIndex);
                        for (i = sourceColumnIndex; i < maxcolindex; i++) {
                            $(oRowCell[sourceRowIndex].cells[i]).addClass("e-selectedCell");
                        }
                        if (difference > 1) {
                            for (k = 1; k <= difference - 1; k++) {
                                for (i = mincolindex ; i < maxcolindex; i++) {
                                    $(oRowCell[sourceRowIndex + k].cells[i]).addClass("e-selectedCell");
                                }
                            }
                        }
                        for (j = mincolindex; j <= currentColumnIndex; j++) {
                            $(oRowCell[currentRowIndex].cells[j]).addClass("e-selectedCell");
                        }
                    }
                    else if (currentRowIndex < sourceRowIndex) {
                        var difference = (sourceRowIndex - currentRowIndex);
                        for (i = sourceColumnIndex; i >= mincolindex; i--) {
                            $(oRowCell[sourceRowIndex].cells[i]).addClass("e-selectedCell");
                        }
                        if (difference > 1) {
                            for (k = 1; k <= difference - 1; k++) {
                                for (i = mincolindex; i < maxcolindex; i++) {
                                    $(oRowCell[sourceRowIndex - k].cells[i]).addClass("e-selectedCell");
                                }
                            }
                        }
                        for (j = maxcolindex - 1; j >= currentColumnIndex; j--) {
                            $(oRowCell[currentRowIndex].cells[j]).addClass("e-selectedCell");
                        }
                    }
                    this._currentRowIndex = currentRowIndex;
                    this._currentColIndex = currentColumnIndex;
                    $(oRowCell[currentRowIndex].cells[currentColumnIndex]).addClass("e-selectedCell").attr("tabIndex", 0).focus();
                }
                else { return false; }
            }
        }
    },

    _mouseSelect: function (event, oEl) {
        this.model.showQuickWindow && (this._quickAppointWindow.ejDialog("close") && this._quickAppDetailsWindow.ejDialog("close"));
        var oRowCell = this.element.find('div.e-draggableworkarea tr');
        if ($(oEl).hasClass("e-selectedAppointment")) {
            this.element.find('.e-selectedAppointment').removeClass("e-selectedAppointment");
            $(oRowCell[0].cells[0]).addClass("e-selectedCell").attr("tabIndex", 0).focus();
        }
        var sourceRowIndex, sourceColumnIndex, lastRowIndex, lastColumnIndex;
        var target = this.element.find('div.e-draggableworkarea');
        sourceRowIndex = $(oEl).parent()[0].rowIndex;
        sourceColumnIndex = $(oEl)[0].cellIndex;
        var view = this.currentView();
        var rowcount = oRowCell.length - 1;
        this._currentRowIndex = sourceRowIndex;
        this._currentColIndex = sourceColumnIndex;
        if ($(oEl).hasClass('e-workcells') || $(oEl).hasClass('e-monthcells')) {
            oRowCell.bind(ej.eventType.mouseMove, function (e) {
                e.preventDefault();
                var currTarget = e.target;
                if (e.type == 'touchmove') {
                    var coor = e.originalEvent.changedTouches[0];
                    currTarget = document.elementFromPoint(coor.pageX, coor.pageY);
                }
                if ($(currTarget).hasClass('e-workcells') || $(currTarget).hasClass('e-monthcells')) {
                    var i, j, k;
                    var currentRowIndex = $(currTarget).parent()[0].rowIndex;
                    var currentColumnIndex = $(currTarget)[0].cellIndex;
                    if ($(currTarget).hasClass('e-workcells')) {
                        var viewCount = view == "day" ? 1 : view == "workweek" ? 5 : 7;
                        var resIndex = Math.floor($(currTarget).index() / viewCount);
                        var resmaxcolindex = (viewCount * (resIndex + 1));
                        var resmincolindex = (viewCount * resIndex);
                        if (sourceColumnIndex >= parseInt(resmincolindex) && sourceColumnIndex <= parseInt(resmaxcolindex) - 1 && currentColumnIndex != "undefined") {
                            if (sourceColumnIndex == currentColumnIndex && (currentRowIndex <= sourceRowIndex || currentRowIndex >= sourceRowIndex)) {
                                if ($(oRowCell[currentRowIndex].cells[currentColumnIndex]).hasClass("e-selectedCell")) {
                                    if (currentColumnIndex > lastColumnIndex) {
                                        for (i = rowcount; i >= 0; i--)
                                            $(oRowCell[i].cells[currentColumnIndex - 1]).removeClass("e-selectedCell");
                                        if (currentRowIndex > sourceRowIndex) {
                                            for (i = 0; i <= rowcount; i++)
                                                $(oRowCell[i].cells[currentColumnIndex]).removeClass("e-selectedCell");
                                        }
                                        else if (currentRowIndex < sourceRowIndex) {
                                            for (i = 0; i < currentRowIndex; i++)
                                                $(oRowCell[i].cells[currentColumnIndex]).removeClass("e-selectedCell");
                                        }
                                        else if (currentRowIndex == sourceRowIndex) {
                                            for (i = 0; i <= rowcount; i++) {
                                                if (i != sourceRowIndex)
                                                    $(oRowCell[i].cells[currentColumnIndex]).removeClass("e-selectedCell");
                                            }
                                        }
                                    }
                                    else if (currentColumnIndex < lastColumnIndex) {
                                        for (i = rowcount; i >= 0; i--)
                                            $(oRowCell[i].cells[currentColumnIndex + 1]).removeClass("e-selectedCell");
                                        if (currentRowIndex > sourceRowIndex) {
                                            for (i = rowcount; i > currentRowIndex; i--)
                                                $(oRowCell[i].cells[currentColumnIndex]).removeClass("e-selectedCell");
                                        }
                                        else if (currentRowIndex < sourceRowIndex) {
                                            for (i = rowcount; i > sourceRowIndex; i--)
                                                $(oRowCell[i].cells[currentColumnIndex]).removeClass("e-selectedCell");
                                        }
                                        else if (currentRowIndex == sourceRowIndex) {
                                            for (i = 0; i <= rowcount; i++) {
                                                if (i != sourceRowIndex)
                                                    $(oRowCell[i].cells[currentColumnIndex]).removeClass("e-selectedCell");
                                            }
                                        }
                                    }
                                    else if (currentColumnIndex == lastColumnIndex) {
                                        if (currentRowIndex > sourceRowIndex) {
                                            for (i = rowcount; i > currentRowIndex; i--)
                                                $(oRowCell[i].cells[currentColumnIndex]).removeClass("e-selectedCell");
                                        }
                                        else if (currentRowIndex < sourceRowIndex) {
                                            for (i = 0; i < currentRowIndex; i++)
                                                $(oRowCell[i].cells[currentColumnIndex]).removeClass("e-selectedCell");
                                        }
                                        else if (currentRowIndex == sourceRowIndex) {
                                            for (i = 0; i <= rowcount; i++) {
                                                if (i != sourceRowIndex)
                                                    $(oRowCell[i].cells[currentColumnIndex]).removeClass("e-selectedCell");
                                            }
                                        }
                                    }
                                }
                                else {
                                    if (currentRowIndex > sourceRowIndex) {
                                        for (i = 0; i <= rowcount; i++)
                                            $(oRowCell[i].cells[lastColumnIndex]).removeClass("e-selectedCell");
                                        for (j = 0; j < sourceRowIndex; j++)
                                            $(oRowCell[j].cells[sourceColumnIndex]).removeClass("e-selectedCell");
                                        for (k = parseInt(sourceRowIndex) ; k <= currentRowIndex; k++) {
                                            $(oRowCell[k].cells[sourceColumnIndex]).addClass("e-selectedCell");
                                        }
                                    }
                                    else if (currentRowIndex < sourceRowIndex) {
                                        for (i = 0; i < rowcount; i++)
                                            $(oRowCell[i].cells[lastColumnIndex]).removeClass("e-selectedCell");
                                        for (j = rowcount; j > sourceRowIndex; j--)
                                            $(oRowCell[j].cells[sourceColumnIndex]).removeClass("e-selectedCell");
                                        for (k = parseInt(sourceRowIndex) ; k >= currentRowIndex; k--) {
                                            $(oRowCell[k].cells[sourceColumnIndex]).addClass("e-selectedCell");
                                        }
                                    }
                                }
                                $(oRowCell[currentRowIndex].cells[currentColumnIndex]).addClass("e-selectedCell").attr("tabIndex", 0).focus();
                            }
                            else if (currentColumnIndex > sourceColumnIndex) {
                                if ($(oRowCell[currentRowIndex].cells[currentColumnIndex]).hasClass("e-selectedCell")) {
                                    if (currentColumnIndex < lastColumnIndex) {
                                        for (i = rowcount; i >= 0; i--)
                                            $(oRowCell[i].cells[lastColumnIndex]).removeClass("e-selectedCell");
                                        for (j = rowcount; j > currentRowIndex; j--)
                                            $(oRowCell[j].cells[currentColumnIndex]).removeClass("e-selectedCell");
                                    }
                                    else if (currentColumnIndex == lastColumnIndex) {
                                        for (i = rowcount; i > currentRowIndex; i--)
                                            $(oRowCell[i].cells[currentColumnIndex]).removeClass("e-selectedCell");
                                    }
                                    $(oRowCell[currentRowIndex].cells[currentColumnIndex]).addClass("e-selectedCell").attr("tabIndex", 0).focus();
                                } else {
                                    for (var l = 0; l < sourceRowIndex; l++) {
                                        $(oRowCell[l].cells[sourceColumnIndex]).removeClass("e-selectedCell");
                                    }
                                    var difference = currentColumnIndex - sourceColumnIndex;
                                    for (i = parseInt(sourceRowIndex) ; i <= rowcount; i++) {
                                        $(oRowCell[i].cells[sourceColumnIndex]).addClass("e-selectedCell");
                                    }
                                    if (difference > 1) {
                                        for (k = 1; k <= difference - 1; k++) {
                                            for (var m = 0; m <= rowcount; m++) {
                                                $(oRowCell[m].cells[currentColumnIndex - k]).addClass("e-selectedCell");
                                            }
                                        }
                                    }
                                    for (j = 0; j <= currentRowIndex; j++) {
                                        $(oRowCell[j].cells[currentColumnIndex]).addClass("e-selectedCell");
                                    }
                                    $(oRowCell[currentRowIndex].cells[currentColumnIndex]).addClass("e-selectedCell").attr("tabIndex", 0).focus();
                                }
                            }
                            else if (currentColumnIndex < sourceColumnIndex) {
                                if ($(oRowCell[currentRowIndex].cells[currentColumnIndex]).hasClass("e-selectedCell")) {
                                    if (currentColumnIndex > lastColumnIndex) {
                                        for (i = rowcount; i >= 0; i--)
                                            $(oRowCell[i].cells[lastColumnIndex]).removeClass("e-selectedCell");
                                        for (j = 0; j < currentRowIndex; j++)
                                            $(oRowCell[j].cells[currentColumnIndex]).removeClass("e-selectedCell");
                                    }
                                    else if (currentColumnIndex == lastColumnIndex) {
                                        for (i = 0; i < currentRowIndex; i++)
                                            $(oRowCell[i].cells[currentColumnIndex]).removeClass("e-selectedCell");
                                    }
                                    $(oRowCell[currentRowIndex].cells[currentColumnIndex]).addClass("e-selectedCell").attr("tabIndex", 0).focus();
                                } else {
                                    for (var l = rowcount; l > sourceRowIndex; l--) {
                                        $(oRowCell[l].cells[sourceColumnIndex]).removeClass("e-selectedCell");
                                    }
                                    var difference = sourceColumnIndex - currentColumnIndex;
                                    for (i = parseInt(sourceRowIndex) ; i >= 0 ; i--) {
                                        $(oRowCell[i].cells[sourceColumnIndex]).addClass("e-selectedCell");
                                    }
                                    if (difference > 1) {
                                        for (k = 1; k <= difference - 1; k++) {
                                            for (var m = 0; m <= rowcount; m++) {
                                                $(oRowCell[m].cells[[currentColumnIndex + k]]).addClass("e-selectedCell");
                                            }
                                        }
                                    }
                                    for (j = rowcount ; j >= parseInt(currentRowIndex) ; j--) {
                                        $(oRowCell[j].cells[currentColumnIndex]).addClass("e-selectedCell");
                                    }
                                    $(oRowCell[currentRowIndex].cells[currentColumnIndex]).addClass("e-selectedCell").attr("tabIndex", 0).focus();
                                }
                            }
                        }
                        else { return false; }
                    }
                    else if ($(currTarget).hasClass('e-monthcells')) {
                        var resIndex = Math.floor($(currTarget).index() / 7);
                        var mincolindex = 7 * (parseInt(resIndex));
                        var maxcolindex = 7 * (resIndex + 1);
                        if (sourceColumnIndex >= parseInt(mincolindex) && sourceColumnIndex <= parseInt(maxcolindex) - 1 && currentColumnIndex != "undefined") {
                            if ($(oRowCell[currentRowIndex].cells[currentColumnIndex]).hasClass("e-selectedCell")) {
                                if (currentRowIndex == sourceRowIndex) {
                                    if (currentRowIndex < lastRowIndex) {
                                        for (i = maxcolindex; i > currentColumnIndex; i--)
                                            $(oRowCell[currentRowIndex].cells[i]).removeClass("e-selectedCell");
                                        for (j = maxcolindex; j >= 0; j--)
                                            $(oRowCell[currentRowIndex + 1].cells[j]).removeClass("e-selectedCell");
                                    }
                                    else if (currentRowIndex > lastRowIndex) {
                                        for (i = currentColumnIndex; i < maxcolindex; i++)
                                            $(oRowCell[currentRowIndex - 1].cells[i]).removeClass("e-selectedCell");
                                        for (j = 0; j < currentColumnIndex; j++)
                                            $(oRowCell[currentRowIndex].cells[j]).removeClass("e-selectedCell");
                                    }
                                    else if (currentRowIndex == lastRowIndex) {
                                        if (currentColumnIndex > sourceColumnIndex) {
                                            for (i = maxcolindex; i > currentColumnIndex; i--)
                                                $(oRowCell[currentRowIndex].cells[i]).removeClass("e-selectedCell");
                                        }
                                        else if (currentColumnIndex < sourceColumnIndex) {
                                            for (i = 0; i < currentColumnIndex; i++)
                                                $(oRowCell[currentRowIndex].cells[i]).removeClass("e-selectedCell");
                                        }
                                        if (currentColumnIndex == sourceColumnIndex) {
                                            for (i = 0; i <= maxcolindex; i++) {
                                                if (i != sourceColumnIndex)
                                                    $(oRowCell[currentRowIndex].cells[i]).removeClass("e-selectedCell");
                                            }
                                        }
                                    }
                                }
                                else if (currentRowIndex > sourceRowIndex) {
                                    if (currentRowIndex < lastRowIndex) {
                                        for (i = maxcolindex; i > currentColumnIndex; i--)
                                            $(oRowCell[currentRowIndex].cells[i]).removeClass("e-selectedCell");
                                        for (j = maxcolindex; j >= 0; j--)
                                            $(oRowCell[currentRowIndex + 1].cells[j]).removeClass("e-selectedCell");
                                    }
                                    else if (currentRowIndex == lastRowIndex) {
                                        for (j = maxcolindex; j > currentColumnIndex; j--)
                                            $(oRowCell[currentRowIndex].cells[j]).removeClass("e-selectedCell");
                                    }
                                }
                                else if (currentRowIndex < sourceRowIndex) {
                                    if (currentRowIndex > lastRowIndex) {
                                        for (i = currentColumnIndex; i < maxcolindex; i++)
                                            $(oRowCell[currentRowIndex - 1].cells[i]).removeClass("e-selectedCell");
                                        for (j = 0; j < currentColumnIndex; j++)
                                            $(oRowCell[currentRowIndex].cells[j]).removeClass("e-selectedCell");

                                    }
                                    else if (currentRowIndex == lastRowIndex) {
                                        for (j = 0; j < currentColumnIndex; j++)
                                            $(oRowCell[currentRowIndex].cells[j]).removeClass("e-selectedCell");
                                    }
                                }
                                $(oRowCell[currentRowIndex].cells[currentColumnIndex]).addClass("e-selectedCell").attr("tabIndex", 0).focus();
                            }
                            else {
                                if (currentRowIndex == sourceRowIndex) {
                                    if (currentColumnIndex < sourceColumnIndex) {
                                        for (i = maxcolindex; i >= 0; i--) {
                                            $(oRowCell[currentRowIndex + 1].cells[i]).removeClass("e-selectedCell");
                                        }
                                        for (j = maxcolindex; j > currentColumnIndex; j--) {
                                            $(oRowCell[currentRowIndex].cells[j]).removeClass("e-selectedCell");
                                        }
                                        for (k = sourceColumnIndex; k >= currentColumnIndex; k--) {
                                            $(oRowCell[currentRowIndex].cells[k]).addClass("e-selectedCell");
                                        }
                                    }
                                    else if (currentColumnIndex > sourceColumnIndex) {
                                        for (i = 0; i < maxcolindex; i++) {
                                            if (currentRowIndex != 0)
                                            $(oRowCell[currentRowIndex - 1].cells[i]).removeClass("e-selectedCell");
                                        }
                                        for (j = 0; j < sourceColumnIndex; j++) {
                                            $(oRowCell[currentRowIndex].cells[j]).removeClass("e-selectedCell");
                                        }
                                        for (k = sourceColumnIndex; k <= currentColumnIndex; k++) {
                                            $(oRowCell[currentRowIndex].cells[k]).addClass("e-selectedCell");
                                        }
                                    }
                                    else if (currentColumnIndex == sourceColumnIndex) {
                                        $(oRowCell[currentRowIndex].cells[currentColumnIndex]).addClass("e-selectedCell");
                                    }
                                }
                                else if (currentRowIndex > sourceRowIndex) {
                                    for (var l = mincolindex; l < sourceColumnIndex; l++) {
                                        $(oRowCell[sourceRowIndex].cells[l]).removeClass("e-selectedCell");
                                    }
                                    var difference = (currentRowIndex - sourceRowIndex);
                                    for (i = sourceColumnIndex; i < maxcolindex; i++) {
                                        $(oRowCell[sourceRowIndex].cells[i]).addClass("e-selectedCell");
                                    }
                                    if (difference > 1) {
                                        for (k = 1; k <= difference; k++) {
                                            for (i = mincolindex ; i < maxcolindex; i++) {
                                                $(oRowCell[sourceRowIndex + k].cells[i]).addClass("e-selectedCell");
                                            }
                                        }
                                    }
                                    for (j = mincolindex; j <= currentColumnIndex; j++) {
                                        $(oRowCell[currentRowIndex].cells[j]).addClass("e-selectedCell");
                                    }
                                }
                                else if (currentRowIndex < sourceRowIndex) {
                                    for (var l = maxcolindex; l > sourceColumnIndex; l--) {
                                        $(oRowCell[sourceRowIndex].cells[l]).removeClass("e-selectedCell");
                                    }
                                    var difference = (sourceRowIndex - currentRowIndex);
                                    for (i = sourceColumnIndex; i >= mincolindex; i--) {
                                        $(oRowCell[sourceRowIndex].cells[i]).addClass("e-selectedCell");
                                    }
                                    if (difference > 1) {
                                        for (k = 1; k <= difference; k++) {
                                            for (i = mincolindex; i < maxcolindex; i++) {
                                                $(oRowCell[sourceRowIndex - k].cells[i]).addClass("e-selectedCell");
                                            }
                                        }
                                    }
                                    for (j = maxcolindex - 1; j >= currentColumnIndex; j--) {
                                        $(oRowCell[currentRowIndex].cells[j]).addClass("e-selectedCell");
                                    }
                                }
                                $(oRowCell[currentRowIndex].cells[currentColumnIndex]).addClass("e-selectedCell").attr("tabIndex", 0).focus();
                            }
                        }
                        else { return false; }
                    }
                    lastRowIndex = currentRowIndex;
                    lastColumnIndex = currentColumnIndex;
                }
            });
            oRowCell.bind(ej.eventType.mouseUp, function (e) {
                oRowCell.unbind(ej.eventType.mouseMove);
            });
            this.element.find("e.workcells,e.alldaycells,.e-monthcells").removeClass("e-mousehover");
            this.element.find(".e-workcells,.e-alldaycells,.e-monthcells").removeClass("e-selectedCell");
            this.element.find('.e-selectedAppointment').removeClass("e-selectedAppointment");
        }
    },
    /**
    * Section For handling mouse down action on the schedule control.
    * @private
    */
    _onScheduleMouseDown: function (e) {
        var _targ = $(e.target);
        var appointdiv = _targ.hasClass("e-appointment,e-alldayappointment,e-monthappointment") ? _targ : _targ.parents(".e-appointment,.e-alldayappointment,.e-monthappointment");
        if (!_targ.hasClass("e-workcells") && !appointdiv.hasClass("e-appointment") && !appointdiv.hasClass("e-alldayappointment") && !appointdiv.hasClass("e-monthappointment") && !_targ.hasClass("e-alldaycells") && !_targ.hasClass("e-monthcells") && !(_targ).parents(".e-scheduledialog").hasClass("e-scheduledialog")) {
            this.model.showQuickWindow && (this._quickAppointWindow.ejDialog("close") && this._quickAppDetailsWindow.ejDialog("close"));
            this.element.find(".e-workcells").removeClass("e-mousehover");
        }
    },
    /**
    * Section For handle mouse hover action in the schedule cells.
    * @private
    */
    _onCellMouseOver: function (e) {
        var element = $(e.currentTarget);
        if (e.type === "mouseover") {
            this.element.find(".e-mousehover").removeClass("e-mousehover");
            if ((element.hasClass("e-workcells") || element.hasClass("e-monthcells")) && !element.hasClass("e-resourceheadercells"))
                element.addClass("e-mousehover");
        }
        else if (e.type === "mouseout") {
            if (element.hasClass("e-workcells") || element.hasClass("e-monthcells"))
                element.removeClass("e-mousehover");
        }
    },
    /**
    * Section For handle mouse hover action in the schedule appointments.
    * @private
    */
    _onAppointMouseOver: function (e) {
        if (this.model.enableAppointmentResize) {
            var appointdiv = $(e.target).hasClass("e-appointment,e-alldayappointment,e-monthappointment") ? $(e.target) : $(e.target).parents(".e-appointment,.e-alldayappointment,.e-monthappointment");
            if (appointdiv.hasClass("e-appointment") && this.model.orientation == "vertical") {
                if (e.type === "mouseover") {
                    appointdiv.find(".e-bottomhandle,.e-tophandle").addClass("e-icon e-nsexpand");
                    appointdiv.find(".e-bottomhandle").addClass("e-appointresizer-bottom");
                    appointdiv.find(".e-tophandle").addClass("e-appointresizer-top");
                }
                else if (e.type === "mouseout") {
                    appointdiv.find(".e-bottomhandle,.e-tophandle").removeClass("e-icon e-nsexpand");
                }

            }
            else if (((appointdiv.hasClass("e-alldayappointment") || appointdiv.hasClass("e-monthappointment")) && this.model.orientation == "vertical") || (this.model.orientation == "horizontal")) {
                if (e.type === "mouseover") {
                    appointdiv.find(".e-lefthandle,.e-righthandle").addClass("e-icon e-ewexpand");
                    appointdiv.find(".e-lefthandle").addClass("e-appointresizer-left");
                    appointdiv.find(".e-righthandle").addClass("e-appointresizer-right");
                }
                else if (e.type === "mouseout") {
                    appointdiv.find(".e-lefthandle,.e-righthandle").removeClass("e-icon e-ewexpand");
                }
            }
        }
    },
    /**
    * Section For handle cell selection action of the schedule.
    * @private
    */
    _cellSelection: function (e) {
        if (e.shiftKey)
            return false;
        this.element.find("e.workcells,e.alldaycells,.e-monthcells").removeClass("e-mousehover");
        this.element.find(".e-workcells,.e-alldaycells,.e-monthcells").removeClass("e-selectedCell");
        this.element.find('.e-selectedAppointment').removeClass("e-selectedAppointment");
        var localizeDays = Globalize.culture(this.model.locale);
            var _target = !ej.isNullOrUndefined(e.target) ? $(e.target) : $(e);
            if ((_target.hasClass("e-workcells") || _target.hasClass("e-alldaycells") || _target.hasClass("e-monthcells")) && !_target.hasClass("e-resourceheadercells")) {
                this.model.showQuickWindow && this._quickAppDetailsWindow.ejDialog("close");
                this._cellIndex = _target.hasClass("e-workcells") ? _target.index() : _target.hasClass("e-alldaycells") ? _target.index() : 7 - ((parseInt(_target.index() / 7) + 1) * 7 - _target.index()) + (_target.parent().index() * 7);
                if (this.model.orientation == "horizontal") {
                    this._cellIndex = this.model.currentView !== "month" ? Math.floor(this._cellIndex / ((this.model.endHour - this.model.startHour) * 2)) : this._cellIndex;
                    this._workCellIndex = _target.parent().index();
                }
                var renderDate = (this.model.orientation == "horizontal" && this.model.currentView == "month") ? this.monthDays : this.model.orientation == "vertical" ? this.dateRender : this._dateRender;
                var curDate = new Date(renderDate[this._cellIndex]);
                if (_target.hasClass("e-workcells") && this.model.currentView !== "month") {
                    var time = this.model.orientation == "vertical" ? this.model.startHour + (_target.parent().index() / 2) : this.model.startHour + ((_target.index() - (((this.model.endHour - this.model.startHour) * 2) * this._cellIndex)) / 2);
                    var timemin = time.toString().split(".");
                    this.cur_StartTime = new Date(curDate).setHours(parseInt(timemin[0]), parseInt(timemin[1]) == 5 ? 30 : 00);
                    this.cur_EndTime = new Date(new Date(this.cur_StartTime).setMinutes(new Date(this.cur_StartTime).getMinutes() + 30));
                    this._appointmentAddWindow.find(".allday").ejCheckBox({ checked: false });
                }
                else if (_target.hasClass("e-workcells") && this.model.orientation == "horizontal" && this.model.currentView == "month") {
                    this.cur_StartTime = new Date(new Date(curDate).setHours(0, 0));
                    this.cur_EndTime = new Date(new Date(curDate).setHours(23, 59));
                    this._appointmentAddWindow.find(".allday").ejCheckBox({ checked: true });
                }
                else {
                    //           this._cellIndex = index;
                    this.cur_StartTime = new Date(new Date(curDate).setHours(0, 0));
                    this.cur_EndTime = new Date(new Date(curDate).setHours(23, 59));
                    this._appointmentAddWindow.find(".allday").ejCheckBox({ checked: true });
                }
                if (this.model.showQuickWindow) {
                    this._quickAppointWindow.find("#" + this._id + "_quickAppwindow").css("display", "block");
                    //add the dialog code to display while clicking on the cells
                    this._quickAppointWindow.find('.e-quickstartend').html(localizeDays.calendar.days.names[new Date(curDate).getDay()] + ", " + Globalize.format(new Date(curDate), "MMMM") + " " + new Date(curDate).getDate() + ", " + Globalize.format(new Date(this.cur_StartTime), this.model.timeMode == "12" ? this._pattern.t : "HH:mm") + " - " + Globalize.format(new Date(this.cur_EndTime), this.model.timeMode == "12" ? this._pattern.t : "HH:mm"));
                    var topElmt = $(e).hasClass("e-workcells") || $(e).hasClass("e-alldaycells") || $(e).hasClass("e-monthcells") ? $(e) : $(e.target);
                    (e.type != "beforeOpen") ? this._quickAppointWindow.ejDialog("open") : "";
                    this._quickAppointWindow.find('.subject').val("");
                    var x = (this.model.orientation == "vertical") ? ej.isNullOrUndefined(this.model.group) ? this.currentView() === "day" ? (this.cellwidth / 2) - this.element.find(".e-leftindent").width() : topElmt.offset().left - this._quickAppointWindow.width() / 3 : this.currentView() === "day" ? topElmt.offset().left : topElmt.offset().left - (this._quickAppointWindow.width() * (40 / 100)) : ej.isNullOrUndefined(this.model.group) ? topElmt.offset().left - this._quickAppointWindow.width() / 2.5  : topElmt.offset().left - (this._quickAppointWindow.width() * (40 / 100));
                    var y = topElmt.offset().top - this._quickAppointWindow.height() - 10;
                    this._quickAppointWindow.ejDialog({ position: { X: x, Y: y } });
                    if (ej.scheduleFeatures.resources && !ej.isNullOrUndefined(this.model.resources))
                        this._getResourceDetails(_target);
                }
                var event = { startTime: new Date(this.cur_StartTime), endTime: this.cur_EndTime, target: e };
                if (this._trigger("cellClick", event)) return false;
                _target.addClass("e-selectedCell").attr("tabIndex", 0).focus();
                this.model.showQuickWindow && this._quickAppointWindow.find('.subject').focus();
                this._cellClick = true;
                this._appointmentAddWindow.ejDialog({ title: this._getLocalizedLabels("CreateAppointmentTitle") });
            }
            else {
                if (!_target.hasClass("e-resourceheadercells")) {
                    this._cellClick = false;
                    this._quickAppDetailsWindow && this._quickAppDetailsWindow.find("" + "#" + this._id + "_quickAppDetailswindow").css("display", "block");
                    this.model.showQuickWindow && this._quickAppointWindow.ejDialog("close");
                    var appoint = _target.hasClass("e-appointment") || _target.hasClass("e-monthappointment") || _target.hasClass("e-alldayappointment") ? _target : _target.parents(".e-appointment,.e-alldayappointment,.e-monthappointment");
                    if (appoint.length <= 0) return false;
                    appoint.attr("tabIndex", 0).focus();
                    appoint.addClass("e-selectedAppointment");

                    this._appId = parseInt(appoint[0].id.split("_")[1]);
                    var query = new ej.Query().where(this.model.appointmentSettings["id"], ej.FilterOperators.equal, this._appId);
                    var templist = new ej.DataManager(this._processed).executeLocal(query);
                    var value = (((this._formatTime(templist[0][this.model.appointmentSettings["endTime"]]) - this._formatTime(templist[0][this.model.appointmentSettings["startTime"]])) / 3600000) > 24) ? this._getAppointmentByParentId(this._appId) : templist[0];
                    this._quickAppDetailsWindow && this._quickAppDetailsWindow.find('.e-quicksubject').html(templist[0][this.model.appointmentSettings["subject"]]);
                    var temp = $.extend({}, templist[0]);
                    if (this.model.timeZone != null) {
                        var timezone = this.model.timeZone.split(" ")[1].split(":");
                        var hour = parseInt(timezone[0]);
                        var mins = parseInt(timezone[1]);
                        dateMinutes = new Date().getTimezoneOffset();
                        var obj = this._minToHour(dateMinutes + ((hour * 60) + mins));
                        var newStartDate = new Date(temp.StartTime); var newEndDate = new Date(temp.EndTime);
                        temp.StartTime = new Date(newStartDate.setHours(newStartDate.getHours() + obj.h, newStartDate.getMinutes() + obj.m));
                        temp.EndTime = new Date(newEndDate.setHours(newEndDate.getHours() + obj.h, newEndDate.getMinutes() + obj.m));
                    }
                    var startTimeDisp = this.model.timeMode == "12" ? Globalize.format(this._formatTime(temp[this.model.appointmentSettings["startTime"]]), this._pattern.t) : Globalize.format(this._formatTime(temp[this.model.appointmentSettings["startTime"]]), "HH:mm");
                    var endTimeDisp = this.model.timeMode == "12" ? Globalize.format(this._formatTime(temp[this.model.appointmentSettings["endTime"]]), this._pattern.t) : Globalize.format(this._formatTime(temp[this.model.appointmentSettings["endTime"]]), "HH:mm");

                    var diffTime = new Date().getTime() - (this._formatTime(temp[this.model.appointmentSettings["startTime"]]).getTime());
                    var diffMinute = 60 * 1000;
                    var diffHour = 60 * diffMinute;
                    var diffDays = 24 * diffHour;

                    var result = Math.abs(new Date() - this._formatTime(temp[this.model.appointmentSettings["startTime"]]));
                    var h = Math.floor(result / 1000 / 60 / 60); h = (h <= 9 ? '0' + h : h);
                    var m = new Date(result).getUTCMinutes(); m = (m <= 9 ? '0' + m : m);

                    var dueIn = Math.ceil(diffTime / diffMinute - 1) < 0 && h < 24 ? this._getLocalizedLabels("EventBeginsin") + " " + h + " " + this._getLocalizedLabels("Hours") + " " + m + " " + this._getLocalizedLabels("Minutes") :
                        Math.ceil(diffTime / diffMinute - 1) < 0 && h >= 24 ? this._getLocalizedLabels("EventBeginsin") + " " + Math.abs(Math.ceil(diffTime / diffDays)) + " " + this._getLocalizedLabels("Days") : this._getLocalizedLabels("Overdue");
                    if (this.model.showQuickWindow) {
                        this._quickAppDetailsWindow.find('.e-rightfields').html(dueIn);
                        var startdate = localizeDays.calendar.days.names[this._formatTime(temp[this.model.appointmentSettings["startTime"]]).getDay()] + ", " + Globalize.format(this._formatTime(temp[this.model.appointmentSettings["startTime"]]), "MMMM") + " " + this._formatTime(temp[this.model.appointmentSettings["startTime"]]).getDate() + ", " + startTimeDisp + " - " + endTimeDisp;
                        var enddate = localizeDays.calendar.days.names[this._formatTime(temp[this.model.appointmentSettings["startTime"]]).getDay()] + ", " + Globalize.format(this._formatTime(temp[this.model.appointmentSettings["startTime"]]), "MMMM") + " " + this._formatTime(temp[this.model.appointmentSettings["startTime"]]).getDate() + " - " + localizeDays.calendar.days.names[this._formatTime(temp[this.model.appointmentSettings["endTime"]]).getDay()] + ", " + Globalize.format(this._formatTime(temp[this.model.appointmentSettings["endTime"]]), "MMMM") + " " + this._formatTime(temp[this.model.appointmentSettings["endTime"]]).getDate() + ", " + startTimeDisp + " - " + endTimeDisp;
                        (((this._formatTime(temp[this.model.appointmentSettings["endTime"]]) - this._formatTime(temp[this.model.appointmentSettings["startTime"]])) / 3600000) < 24) ? this._quickAppDetailsWindow.find('.e-quickstartendtime').html(startdate) : this._quickAppDetailsWindow.find('.e-quickstartendtime').html(enddate);

                        if (temp[this.model.appointmentSettings["recurrence"]] == 1)
                            this._quickAppDetailsWindow.find('.e-editseries').removeClass("e-disable").on('click', $.proxy(this._editClick, this));

                        else
                            this._quickAppDetailsWindow.find('.e-editseries').addClass("e-disable").off('click');
                        (e.type != "beforeOpen") ? this._quickAppDetailsWindow.ejDialog("open") : "";
                        var leftOffsetValue = $(appoint).offset().left - (this._quickAppDetailsWindow.width() * (40 / 100));
                        var left = this.currentView() == "day" ? leftOffsetValue + ($(appoint).width() / 2) : leftOffsetValue;
                        var top = $(appoint).offset().top - this._quickAppDetailsWindow.height() - 10;
                        this._quickAppDetailsWindow.ejDialog({ position: { X: left, Y: top } });
                        this._quickAppDetailsWindow.focus();
                    }
                    var data = { appointment: templist[0] };
                    if (this._trigger("appointmentClick", data))
                        return false;
                    this._appointmentAddWindow.ejDialog({ title: this._getLocalizedLabels("Editevent") });
                }
            }
            if (!$(_target).hasClass("e-monthcells"))
                this.element.find(".e-scrolltimecells").css("top", (-this.element.find('.e-draggableworkarea').scrollTop()) + "px");
    },
    /**
   * Section For Check the reminder with 60000 ms interval.
   * @private
   */
    _checkReminders: function (args) {
        this._todayAppointments = [];
        for (var i = 0; i < this._processed.length; i++) {
            if ((Globalize.format(new Date(this._processed[i][this.model.appointmentSettings["startTime"]]), this._pattern.d) <= Globalize.format(new Date(), this._pattern.d))) {
                this._todayAppointments.push(this._processed[i]);
            }
        }
        var proxy = this;
        jQuery.each(this._todayAppointments, function (val, i) {
            var reminderValue = proxy.model.reminderSettings.alertBefore;
            var startTime = proxy._formatTime(i[proxy.model.appointmentSettings["startTime"]]);
            if (reminderValue != -1 && reminderValue != null) {
                var currTime = new Date().getTime();
                var difference = (currTime - new Date(startTime).getTime()) / (1000 * 60);
                if (Math.round(difference) == (reminderValue * -1) && difference <= 0) {
                    proxy._trigger("reminder", { reminderAppointment: i });
                }
            }
        });
    },
    /**
    * Section For render the context menu of schedule.
    * @private
    */
    _showContextMenu: function (flag) {
        var gotoFlag = false, cellCollection = this.model.contextMenuSettings.menuItems.cells;
        if (flag) {
            this._contextMenu = ej.buildTag("ul#" + this._id + "contextMenu");
            this.element.append(this._contextMenu);
            this._contextMenu.ejMenu({
                fields: { id: "id", text: "text", parentId: "parentId" },
                menuType: ej.MenuType.ContextMenu,
                openOnClick: false,
                contextMenuTarget: "#" + this._id,
                width: "auto",
                click: $.proxy(this._onMenuSelect, this),
                beforeOpen: $.proxy(this._beforeopen, this)
            });
            if (this.model.categorizeSettings.enable && this.model.contextMenuSettings.menuItems.appointment) {  
                var menuItems = this.model.contextMenuSettings.menuItems.appointment;
                var dataLength = this.model.categorizeSettings.dataSource.length;
                for (var i = 0; i < menuItems.length; i++) 
                {
                    if (menuItems[i].id == 'categorize')
                        var menuId = i;                      
                }
                    for (var i = 0; i < dataLength; i++) {
                        var dataSet = this.model.categorizeSettings;
                        this.model.contextMenuSettings.menuItems.appointment.push({ text: dataSet.dataSource[i][dataSet.text], id: "categorize_" + dataSet.dataSource[i][dataSet.id], fontColor: dataSet.dataSource[i][dataSet.fontColor], color: dataSet.dataSource[i][dataSet.color], parentId: this.model.contextMenuSettings.menuItems.appointment[menuId].id.toString() });
                    }
                }
            
            if (this._gotoDate) {
                if (this.model.contextMenuSettings.enable) {
                    for (var id in cellCollection) {
                        if (cellCollection[id].id == "gotodate") {
                            gotoFlag = true;
                            break;
                        }
                    }
                    if (gotoFlag) {
                        this._renderGotoWindow();
                        this._gotoDate = false;
                    }
                }
            }
        }
    },

    /**
    * Section For handle before context menu open action.
    * @private
    */
    _beforeopen: function (args) {
        if (this.model.contextMenuSettings.enable) {
            $(document).unbind(ej.eventType.mouseMove);
            if (this._trigger("beforeContextMenuOpen", { events: args })) return false;
            var targetClass = $(args.target), appFlag;
            var menuObj = this._contextMenu.data("ejMenu");
            args.cancel = (targetClass.hasClass("e-workcells") || targetClass.hasClass("e-alldaycells") || targetClass.hasClass("e-appointinnertext") || targetClass.hasClass("e-monthcells") || targetClass.hasClass("e-apptime") || targetClass.hasClass("e-apptext")) ? false : true;
            appFlag = (targetClass.hasClass("e-appointinnertext") || targetClass.hasClass("e-apptime") || targetClass.hasClass("e-apptext")) ? false : true;
            (appFlag) ? menuObj.option("fields.dataSource", this.model.contextMenuSettings.menuItems.cells) : menuObj.option("fields.dataSource", this.model.contextMenuSettings.menuItems.appointment);
            this._cellSelection(args);
            this._target = args;
            this._quickAppointWindow.ejDialog("close");
            this._quickAppDetailsWindow.ejDialog("close");

            if (this.model.categorizeSettings.enable && appFlag == false)
            {
                var findDiv = this._contextMenu.find('li#categorize').find('li').children();

                if (findDiv.find('div').length == 0) {
                    for (var i = 0; i < this.model.categorizeSettings.dataSource.length; i++) {
                        this._contextMenu.find('li#categorize').find('li#categorize_' + this.model.categorizeSettings.dataSource[i].id).children().append('<div><div style="border: 1px solid black;height: 12px;margin-left: 2px;margin-top: 2px;width: 12px;background-color:' + this.model.categorizeSettings.dataSource[i].color + '"></div></div>');
                    }
                }
                for (var i = 1; i <= this.model.categorizeSettings.dataSource.length; i++) {
                    if (this._contextMenu.find('li#categorize').find('li#categorize_' + i).children().find('div:first-child.e-categorizeMenuSelected'))
                        this._contextMenu.find('li#categorize').find('li#categorize_' + i).children().find('div:first-child').removeClass('e-categorizeMenuSelected');
                }
                var query = new ej.Query().where(this.model.appointmentSettings["id"], ej.FilterOperators.equal, this._appId);
                var addItem = ej.DataManager(this._processed).executeLocal(query);
                if (addItem[0][this.model.appointmentSettings["categorize"]]) {
                    var categorizeItems = addItem[0][this.model.appointmentSettings["categorize"]].split(',');
                    if (this.model.categorizeSettings.allowMultiple) {
                        if (categorizeItems[0].length > 0) {
                            for (var i = 0; i < categorizeItems.length; i++) {
                                this._contextMenu.find('li#categorize').find('li#categorize_' + categorizeItems[i]).children().find('div:first-child').addClass('e-categorizeMenuSelected');
                            }
                        }
                    }
                    else {
                        if (categorizeItems[0].length > 0) {
                            var lastValue = categorizeItems.length - 1;
                            this._contextMenu.find('li#categorize').find('li#categorize_' + categorizeItems[lastValue]).children().find('div:first-child').addClass('e-categorizeMenuSelected');
                        }
                    }
                }
            }
            else {
                if(this._contextMenu.find('li#categorize'))
                    this._contextMenu.find('li#categorize').css("display", "none");               
            }
            
        }
        else {
            var obj = this._contextMenu.data("ejMenu");
            obj.hide();
            return false;
        }
    },
    /**
    * Section For handle menu select from context menu of schedule.
    * @private
    */
    _onMenuSelect: function (args) {
        if (this._trigger("menuItemClick", { events: args })) return false;
        var id = args.events.ID.split("_");
        switch (id[0]) {
            case "new":
                this._appointmentWindow(this._target);
                break;
            case "recurrence":
                this._appointmentWindow(this._target);
                this._appointmentAddWindow.find(".recurrence").ejCheckBox({ checked: true });
                this._repeat(args = { isChecked: true });
                break;
            case "open":
                this._appointmentWindow(this._target);
                break;
            case "today":
                this._navigateClick(this.currentDate(new Date()));
                break;
            case "view":
                this.currentView(id[1].toLowerCase());
                this._renderCurrentView();
                break;
            case "timemode":
                this.model.timeMode = (id[1] == "Hour12") ? "12" : "24";
                if (this.currentView() !== "month") {
                    var viewclass = this.currentView() == "month" ? 1 : 0; //"e-monthtimecells" : "e-timecelldivs";
                    var timeCellhtml = this.timeCellTemplate.render({ timerows: this._getStartEndTime(), cols1: [1, 2], viewclass: viewclass });
                    this.$TimeCellTD.children().remove();
                    this.$TimeCellTD.append(timeCellhtml); if (this.model.showCurrentTimeIndicator) $(".e-highlightcurrenttimeline").remove(); 
                    this.element.find(".e-scrolltimecells").css("top", -(this.element.find(".e-draggableworkarea").scrollTop()) + "px");
                    this._highlightCurrentTime(this.model.showCurrentTimeIndicator);
                    this._renderAppointmentAll();
                }
                break;
            case "businesshours":
                this.model.highlightBusinessHours = (this.model.highlightBusinessHours) ? false : true;
                (this.model.highlightBusinessHours) ? this._highlightBusinessHours() : this.element.find(".e-workcells").removeClass("e-businesshighlightworkcells") && this.element.find(".e-monthcells").removeClass("e-monthcellhighlight");
                break;
            case "gotodate":
                this._gotoWindow.ejDialog("open");
                break;
            case "delete":
                this._deleteAppoint();
                break;
            case "print":
                this._printAppointment();
                break;
            case "categorize":
                if (ej.isNullOrUndefined(id[1]))
                    this.element.find(".e-appointment,.e-alldayappointment,.e-monthappointment").removeClass("e-selectedAppointment");
                else
                {
                    if (this.model.categorizeSettings.enable)
                    {
                        var getId = parseInt(id[1]); var idValue = "";
                        var query = new ej.Query().where(this.model.appointmentSettings["id"], ej.FilterOperators.equal, this._appId);
                        var addItem = ej.DataManager(this._processed).executeLocal(query);
                        if (addItem[0][this.model.appointmentSettings["categorize"]]) {
                            var categorizeItems = addItem[0][this.model.appointmentSettings["categorize"]].split(',');
                            if (!this.model.categorizeSettings.allowMultiple) {
                                if (categorizeItems[categorizeItems.length - 1] == getId)
                                    idValue = "";
                                else
                                    idValue = getId.toString();
                            }
                            else {
                                if (categorizeItems[0].length > 0) {
                                    for (var i = 0; i < categorizeItems.length; i++) {
                                        if (categorizeItems[i] == getId) {
                                            var findId = true;
                                            break;
                                        }
                                    }
                                    if (findId == true) {
                                        for (var i = 0; i < categorizeItems.length; i++) {
                                            if (categorizeItems[categorizeItems.length - 1] != getId) {
                                                if (categorizeItems[i] != getId) {
                                                    if (categorizeItems.length - 1 != i)
                                                        idValue = idValue + categorizeItems[i] + ",";
                                                    else
                                                        idValue = idValue + categorizeItems[i];
                                                }
                                            }
                                            else {
                                                if (categorizeItems[i] != getId) {
                                                    if (categorizeItems.length - 2 != i)
                                                        idValue = idValue + categorizeItems[i] + ",";
                                                    else
                                                        idValue = idValue + categorizeItems[i];
                                                }
                                            }
                                        }
                                    }
                                    else
                                        idValue = addItem[0][this.model.appointmentSettings["categorize"]] + "," + getId;
                                }
                                //else
                                //{
                                //    idValue = getId.toString();
                                //}
                            }
                        }
                        else {
                            idValue = getId.toString();
                        }
                        var query = new ej.Query().where(this.model.appointmentSettings["id"], ej.FilterOperators.equal, this._appId);
                        var _newAddList = new ej.DataManager(this._processed).executeLocal(query)[0];
                        _newAddList[this.model.appointmentSettings["categorize"]] = idValue;
                        this.element.find("#Appointment_" + this._appId).remove();
                        new ej.DataManager(this._currentAppointmentData).update(this.model.appointmentSettings["id"], _newAddList);
                        this._renderSingleApp(_newAddList, _newAddList);
                        break;

                    }
                }               
        }
    },
    /**
    * Section For handle the goto ok button click.
    * @private
    */
    _gotoOk: function (args) {
        if ($(args.currentTarget).hasClass("e-gotoOk")) {
            var date = this._gotoWindow.find("#" + this._id + "gotodate").ejDatePicker("option", "value");
            var view = this._gotoWindow.find("#" + this._id + "show").ejDropDownList("option", "itemValue");
            this.currentDate(new Date(date));
            this.currentView(view.toLowerCase());
            this._renderCurrentView();
        }
        this._gotoCancel();
    },
    /**
    * Section For handle the goto cancel button click.
    * @private
    */
    _gotoCancel: function () {
        this._gotoWindow.find("#" + this._id + "gotodate").ejDatePicker({ value: new Date() });
        this._gotoWindow.find("#" + this._id + "show").ejDropDownList({ value: "Day" });
        this._gotoWindow.ejDialog("close");
    },
    /**
    * Section For render the goto window controls of schedule.
    * @private
    */
    _renderGoto: function () {
        this._gotoWindow.find("#" + this._id + "gotodate").ejDatePicker({ enableRTL: this.model.enableRTL, width: '100%', value: this.currentDate(), locale: this.model.locale, cssClass: this.model.cssClass });
        this._gotoWindow.find("#" + this._id + "show").ejDropDownList({ enableRTL: this.model.enableRTL, dataSource: [{ text: this._getLocalizedLabels("Day"), value: "day" }, { text: this._getLocalizedLabels("Week"), value: "week" }, { text: this._getLocalizedLabels("WorkWeek"), value: "workweek" }, { text: this._getLocalizedLabels("Month"), value: "month"}], width: "100%", selectedItemIndex: 0, cssClass: this.model.cssClass });
        this._gotoWindow.find(".e-gotobutton").ejButton({ enableRTL: this.model.enableRTL, showRoundedCorner: true, width: "60px", cssClass: this.model.cssClass });
    },
    /**
    * Section For render the gotowindow of schedule.
    * @private
    */
    _renderGotoWindow: function () {
        this._gotoWindow = ej.buildTag('div.e-gotodialog#' + this._id + "gotoWindow");
        var $goWindow = "<div><table width='100%' cellpadding='5'><tbody><tr><td>" + this._getLocalizedLabels("Date") + ":</td><td><input id='" + this._id + "gotodate' type='text' value='' /></td></tr>" +
                "<tr><td>" + this._getLocalizedLabels("Showin") + ":</td><td><input type='text' id='" + this._id + "show' /></td></tr></tbody></table>";
        $goWindow += "<div class='e-floatright'><table><tr><td><div><button id='" + this._id + "gotook' class='e-gotobutton e-gotoOk'>" + this._getLocalizedLabels("Ok") + "</button></div></td>" +
                              "<td><div><button id='" + this._id + "gotocancel' class='e-gotobutton e-gotoCancel'>" + this._getLocalizedLabels("Cancel") + "</button></div></td></tr></table></div></div>";
        this._on(this._gotoWindow, "click", ".e-gotobutton", $.proxy(this._gotoOk, this));
        this._gotoWindow.append($goWindow);
        this.element.append(this._gotoWindow);
        this._renderGoto();
        this._gotoWindow.ejDialog({
            width: 300,
            cssClass: this.model.cssClass,
            showOnInit: false,
            enableRTL: this.model.enableRTL, enableModal: true,
            title: this._getLocalizedLabels("Gotodate"),
            content: "#" + this._id,
            enableResize: false,
            allowKeyboardNavigation: false,
            close: $.proxy(this._gotoCancel, this)
        });
        $("#" + this._id + "gotoWindow_wrapper").addClass("e-scheduledialog").find(".e-titlebar").addClass("e-dialogheader");
    },

    /**
    * Section For handle current time indicator shown in the schedule.
    * @private
    */
    _highlightCurrentTime: function (_showIndicator) {
        if (_showIndicator) {
            var flag = false, index = 0;
            for (var i = 0; i < this._dateRender.length; i++) {
                if (Globalize.format(new Date(this._dateRender[i]), this._pattern.d) === Globalize.format(new Date(), this._pattern.d)) {
                    flag = true;
                    (this.currentView() != "month" && this.model.enableRTL == true) ? index = (this._dateRender.length - 1) - i : index = i;
                }
            }
            if (this.currentView() !== "month") {
                if (flag) {
                    var temp = this;
                    var view = this.currentView() === "week" ? 7 : this.currentView() === "workweek" ? 5 : 1;
                    var timeWidth = this.element.find('.e-draggableworkarea')[0].offsetWidth / (view * this.res1.length);
                    if (new Date().getHours() >= temp.model.startHour && new Date().getHours() <= temp.model.endHour) {
                        if (temp.model.orientation == "vertical") {
                            for (var i = 0; i < this.res1.length; i++) {
                                this.element.find('.e-draggableworkarea').append('<div id="' + this._id + i + '_HighlightCurrentTimeline" class="e-highlightcurrenttimeline"></div>');
                                $('#' + this._id + i + '_HighlightCurrentTimeline').append('<div class="e-currenttime"></div>').append(ej.buildTag("div.e-icon e-timeidicatorcircle e-lefttimeindicator")).append(ej.buildTag("div.e-icon e-timeidicatorcircle e-righttimeindicator"));
                                $('#' + this._id + i + '_HighlightCurrentTimeline').css("width", timeWidth + "px").css("left", ((index + (view * i)) * timeWidth) + "px");
                                $('#' + this._id + i + '_HighlightCurrentTimeline').find('.e-righttimeindicator').css("margin-left", (timeWidth - 3) + "px");
                            }
                        }
                        else {
                            this._renderHorizontalCurrentTime();
                        }
                    }
                    function ChangeCurrentTimePosition() {
                        var now = new Date();
                        now.setMinutes(now.getMinutes());
                        var position;
                        if (now.getHours() >= temp.model.startHour && now.getHours() <= temp.model.endHour) {
                            if (temp.model.orientation == "vertical") {
                                var topVal = ((((now.getHours() - temp.model.startHour) * 60) + now.getMinutes()) * (temp.element.find(".e-workcells").height() + 1) * 2) / 60;
                                for (var i = 0; i < temp.res1.length; i++)
                                    $('#' + temp._id + i + '_HighlightCurrentTimeline').css("top", topVal - 1);
                                var currentTime = temp.model.timeMode == "12" ? Globalize.format(new Date(), "hh:mm tt") : Globalize.format(new Date(), "HH:mm");
                                for (var i = 0; i < temp.res1.length; i++) {
                                    $('#' + temp._id + i + '_HighlightCurrentTimeline').find(".e-currenttime").html(currentTime);
                                    if (temp.model.timeMode == "12") $('#' + temp._id + i + '_HighlightCurrentTimeline').find(".e-currenttime").addClass("e-timewidth");
                                }
                            }
                            else {
                                temp._horizontalTimePosition(temp, now);
                            }
                        }
                    }
                    ChangeCurrentTimePosition();
                    this._intervalID = window.setInterval(ChangeCurrentTimePosition, 60000);
                }
                else {
                    clearInterval(this._intervalID);
                    $(".e-highlightcurrenttimeline").remove();
                }
            }
            else {
                if (flag) {
                    if (this.model.orientation == "vertical") {
                        var rowPosition = Math.ceil((index + 1) / 7); var cellId;
                        if (!this.model.group) this.res1 = ["0"];
                        for (var i = 0; i < this.res1.length; i++) {
                            cellId = (i * (7) + (new Date(this._dateRender[index]).getDay() + 1)) + ((rowPosition - 1) * ((this.res1.length) * 7));
                            $(this.element.find(".e-monthcells")[Math.floor(cellId) - 1]).addClass("e-monthcurrenttime");
                        }
                    }
                    else
                        this._renderHorizontalCurrentTime();
                }
                else
                    this.model.orientation == "vertical" ? this.element.find(".e-monthcells").removeClass("e-monthcurrenttime") : this.element.find(".e-workcells").removeClass("e-monthcurrenttime");
            }
        }
    },

    _getPreviousNextAppointment: function () {
            var dateCount = this._dateRender;
            var appointmentsList = this._processed;
            this._previousList = [];
            this._nextList = [];
            this.appointment;
            for (var app = 0; app < appointmentsList.length; app++) {
                if (new Date(appointmentsList[app][this.model.appointmentSettings["startTime"]]) < new Date(dateCount[0]) && new Date(appointmentsList[app][this.model.appointmentSettings["endTime"]]) < new Date(dateCount[0])) {
                    this._previousList.push(appointmentsList[app]);
                }
                else if (new Date(appointmentsList[app][this.model.appointmentSettings["startTime"]]) > new Date(dateCount[(dateCount.length - 1)]) && new Date(appointmentsList[app][this.model.appointmentSettings["endTime"]]) > new Date(dateCount[(dateCount.length - 1)])) {
                    this._nextList.push(appointmentsList[app]);
                }
            }
            this.appointment = { previous: this._previousList, next: this._nextList };
        },

        _nextPrevAppointment: function (flag) {
            if (flag) {
                var base = this.element.find("#" + this._id + "_scroller");
                this.height = base.height();
                this.width = base.width();
                var scroll = this.element.find(".e-vscroll").width();
                var timecell = this.element.find(".e-scrolltimecells").width();
                var top = (this.height / 2) - (150 / 2), left;
                var prev = ej.buildTag("div.e-prev enable", this._getLocalizedLabels("PreviousAppointment"), {}, {});
                var next = ej.buildTag("div.e-next enable", this._getLocalizedLabels("NextAppointment"), {}, {});
                left = (this.model.orientation == "vertical") ? ((this.currentView() != "month") ? -4:-10):-60;
                if (!this.model.enableRTL) {
                    prev.css("top", (top + 40) + "px").css("transform", "rotate(90deg)").css("left", left + "px");
                    next.css("top", (top + 40) + "px").css("transform", "rotate(-90deg)").css("left", (this.width - scroll -((this.model.orientation == "vertical")?35:90)) + "px");
                }
                else {
                    next.css("top", (top + 40) + "px").css("transform", "rotate(90deg)").css("left", (left - ((this.currentView() != "month") ? 39 : 51)) + "px");
                    prev.css("top", (top + 40) + "px").css("transform", "rotate(-90deg)").css("left", (this.width - timecell - ((this.currentView() != "month") ? 31 : 41)) + "px");
                }
                base.append(prev).append(next);
            }
        },

        _nextPrev: function (e) {
            if ($(e.currentTarget).hasClass("e-prev")) {
                this.appointment.previous.sort(function (a, b) {
                    return new Date(b.StartTime) - new Date(a.StartTime);
                });
                this._navigateClick(this.currentDate(new Date(this.appointment.previous[0].StartTime)));
            }
            else if ($(e.currentTarget).hasClass("e-next")) {
                this.appointment.next.sort(function (a, b) {
                    return new Date(a.StartTime) - new Date(b.StartTime);
                });
                this._navigateClick(this.currentDate(new Date(this.appointment.next[0].StartTime)));
            }
        },

        _nextPrevButton: function (app) {
            if (app.length != 0)
                this.element.find(".e-prev,.e-next").css("display", "none");
            else {
                this._getPreviousNextAppointment();
                this.element.find(".e-prev,.e-next").css("display", "block");
                if (this.appointment.previous.length == 0)
                    this.element.find(".e-prev").removeClass("enable").css("opacity", "0.4");
                else
                    this.element.find(".e-prev").addClass("enable").css("opacity", "1");
                if (this.appointment.next.length == 0)
                    this.element.find(".e-next").removeClass("enable").css("opacity", "0.4");
                else
                    this.element.find(".e-next").addClass("enable").css("opacity", "1");
            }

        },

    /**
    * Section For handle view navigate action of the schedule.
    * @private
    */
        _navigateView: function (e) {
            var newView, oldview = this.currentView();
            var _target = $(e.target);
            if (this.model.orientation == "vertical") {
                if (this.currentView() === "month") {
                    if (_target.hasClass("e-monthheader")) {
                        if (this.model.views.indexOf("Day") == -1) return false;
                        var cellIndex = _target.parent().index();
                        var index = (cellIndex > 6) ? (cellIndex % 7) : cellIndex;
                        this.currentDate(this._getRenderDates()[((_target.parent().parent().index() * 7) +index)]);
						newView = "day";
                    }
                    else if (_target.hasClass("e-timecells")) {
                        var target = _target.hasClass("e-timecells") ? _target : _target.parent();
                        if (this.model.views.indexOf("Week") == -1) return false;
                        this.currentDate(this._getRenderDates()[(target.parent().index() * 7)]);
                        newView = "week";
                    }
                }
                else if (this.currentView() === "week" || this.currentView() === "workweek") {
                    var dayTarget = _target.hasClass("e-headercells") ? _target : _target.parent().hasClass("e-headercells") ? _target.parent() : _target.parent().parent();
                    if (this.model.views.indexOf("Day") == -1 || !dayTarget.hasClass("e-headercells")) return false;
                    var cellIndex = dayTarget.index();
                    var cols = this.currentView() === "workweek" ? 5 : 7;
                    var index = (cellIndex > cols - 1) ? (cellIndex % cols) : cellIndex;
                    this.currentDate(this._getRenderDates()[index]);
                    newView = "day";
                }
                this._navView = true;
                this._viewChange(e, oldview, newView);
            }
            else if (this.model.orientation == "horizontal" && ej.scheduleFeatures.horizontal)
                this._horizontalViewNavigation(e);
    },

    /**
    * Section For handle over flow icon click of the schedule.
    * @private
    */
    _appOverflowClick: function (e) {
        //to stop the day view navigation while we click overflow app
        if (this.model.views.indexOf("Day") !== -1) {
            var overflowId = $(e.target)[0].id.toString().split("_");
            this.currentDate(new Date(overflowId[3], overflowId[1] - 1, overflowId[2]));
            this._navView = true;
            this._viewChange(e, this.currentView(), "day");
        }
        else return false;
    },
    /**
    * Section For handle calendar navigation of the schedule.
    * @private
    */
    _viewCalendar: function (e) {
        var obj = this.element.find("#" + this._id + "_navigator").data("ejDatePicker");
        if (obj) {
            obj.show();
            this.element.find("#" + this._id + "_navigator").ejDatePicker({ value: new Date(this.currentDate()) });
            var controlmargin = this.element.position();
            if (this.model.enableRTL)
                $("div.e-datepicker").css("top", (controlmargin.top + 47) + "px").css("left", (controlmargin.left + this.element.width() - 280) + "px");
            else
                $("div.e-datepicker").css("top", (controlmargin.top + 47) + "px").css("left", (controlmargin.left + 55) + "px");
        }
        this.element.find(".e-workcells").removeClass("e-selectedCell");
        this.element.find(".e-workcells").removeClass("e-mousehover");
        this.element.find(".e-appointment,.e-alldayappointment,.e-monthappointment").removeClass("e-selectedAppointment");
    },
    /**
    * Section For change the current view of the schedule.
    * @private
    */
    _viewChange: function (e, oldview, newView) {
        var flag = true;
        if (oldview !== newView || (!oldview && !newView)) {
            if (!this._navView) {
                flag = (this.currentView() == (e.currentTarget.className.split("e-")[2]).split(" ")[0]) ? false : true;
                this.currentView($(e.currentTarget).hasClass("e-activeview") ? this.currentView() : (e.currentTarget.className.split("e-")[2]).split(" ")[0]);
            }
            else
                this.currentView(newView);
            if (flag) this._renderCurrentView();
        }
        var data = { prevView: oldview, currentView: this.currentView(), Date: new Date(this.currentDate()), target: e, type: "Scheduleview" };
        if (this._trigger("navigation", data))
            return false;
    },
    /**
    * Section For render the current view of the schedule.
        * @private
        */
        _renderCurrentView: function () { //clear the old view and render with newly selected view
            //clear old view 
            this.element.find(".e-commonviewbutton").blur();
            this.element.find(".e-commonviewbutton").removeClass("e-activeview");
            if (this.model.orientation == "vertical")
                this.element.find(".e-headerdaydisplay,.e-datedisplay,.e-headrealldaytable").children().remove();
            this.wTR.children().remove();
            this.aTR.children().remove();
            this._getRenderDates();
            //set new view
            this.element.find(".e-" + this.currentView()).addClass("e-activeview").attr("tabIndex", 0).focus();
            this._renderHeaderAllDayTemplate();
            this._renderContentAreaTemplate();
            this._renderScroller();
            if (this.model.orientation == "vertical") {
                this._headerbarWidth();
                this.element.find(".e-schedulesettings").html(this._getLocalizedLabels("AllDay"));
                this.element.find(".e-ampmdisplay").height(this.element.find(".e-workcells").height());
            }
            else if (this.model.orientation == "horizontal" && ej.scheduleFeatures.horizontal)
                this._moveScroller();
            this._renderHeaderDays();
            if (!ej.isNullOrUndefined(this.dataSource())) {
                this._dataProcessing(this._currentAppointmentData);
                this._renderAppointmentAll();
            }
            (this.model.highlightBusinessHours) ? this._highlightBusinessHours() : this.element.find(".e-workcells").removeClass("e-businesshighlightworkcells");
            this._highlightCurrentTime(this.model.showCurrentTimeIndicator);
            this._navView = false;
    },
    /**
    * Section For render the header day cells of the schedule.
        * @private
        */
        _renderHeaderDays: function () {
            this.dateRender = [];
            var dateList = this._getRenderDates();
            this.res1 = (!ej.isNullOrUndefined(this.model.group)) ? this.res1 : [0];
            for (var i = 0; i < this.res1.length; i++) {
                for (var j = 0; j < dateList.length; j++) {
                    this.dateRender.push(new Date(dateList[j]));
                }
            }
            //            var dateRender = this.dateRender; // this._getRenderDates();
            var localizeDays = Globalize.culture(this.model.locale);
            var dateVal = this._getCurrentDateHeaderValue();
            this.element.find("div.e-datedisplay")[0].innerHTML = dateVal;
            if (this.model.orientation == "vertical") {
                if (this.currentView() !== "month") {
                    var curday, temp;
                    for (var i = 0; i < this.dateRender.length; i++) {
                        curday = (this.currentView() === "day") ? (ej.isNullOrUndefined(this.model.resources) && ej.isNullOrUndefined(this.model.group)) ? localizeDays.calendar.days.names[new Date(this.dateRender[i]).getDay()] : localizeDays.calendar.days.names[new Date(this.dateRender[i]).getDay()].slice(0, 3) : localizeDays.calendar.days.names[new Date(this.dateRender[i]).getDay()].slice(0, 3);
                        temp = this.currentView() === "day" ? (ej.isNullOrUndefined(this.model.resources) && ej.isNullOrUndefined(this.model.group)) ? curday + " " + Globalize.format(new Date(this.dateRender[i]), "dd MMMM yyyy") : curday + " " + Globalize.format(new Date(this.dateRender[i]), "dd MMM yy") : curday + " " + new Date(this.dateRender[i]).getDate();
                        this.element.find(".e-headerdaydisplay")[i].innerHTML = temp;
                    }
                    this.element.find(".e-alldaycells").css('height', "27px");
                    this.element.find(".e-headercells").addClass("e-dateheaderselect");
                }
                else
                    this._renderMonthTimeCells();
            }
            else if(this.model.orientation == "horizontal" && ej.scheduleFeatures.horizontal)
                this._renderHorizontalHeaderDays(localizeDays);
    },
    /**
    * Section For render the month time cells of the schedule.
        * @private
        */
        _renderMonthTimeCells: function () {
            var timeCellIndex = 0; var dateRender = this._dateRender, k = 0, resCount = !ej.isNullOrUndefined(this.model.group) ? this.res1.length * 7 : 7;
            for (var i = 0; i < dateRender.length; i++) {
                for (var j = k; j < resCount; j = j + 7) {
                    $(this.element.find("div.e-monthheader"))[j].innerHTML = (Globalize.format(dateRender[i], "dd")).toString();
                }
                k++;
                if ((i + 1) % 7 == 0) {
                    k = resCount;
                    resCount = resCount + ((!ej.isNullOrUndefined(this.model.group) ? this.res1.length : 1) * 7);
                    start = Globalize.format(dateRender[i - 6], "MMM dd");
                    end = Globalize.format(dateRender[i], "MMM dd");
                    $(this.element.find(".e-timecelldivs"))[timeCellIndex].innerHTML = start + " - " + end;
                    timeCellIndex++;
                }
                if (new Date(dateRender[i]).getMonth() !== new Date(this.currentDate()).getMonth())
                    $(this.element.find("div.e-monthheader")[i]).addClass("e-monthdateshighlight");
            else
                $(this.element.find("div.e-monthheader")[i]).removeClass("e-monthdateshighlight");
        }
        this.element.find(".e-timecells").addClass("e-dateheaderselect").height(this.element.find(".e-monthcells").height());
        if (j != 6) this.element.find(".e-timecells").addClass("e-timecellpadding");
    },
    /**
    * Section For handle the date navigation of the schedule.
    * @private
    */
    _navigateClick: function (e) {
        var data;
        if (e != null && !this._setCurrentDate) {
            var dat = new Date(this.currentDate());
            var day = this.currentView() === "day" ? 1 : 7;
            if ($(e.currentTarget).hasClass("e-navigateprevious")) {
                this.currentDate(this.currentView() === "month" ? new Date(dat.setMonth(new Date(this.currentDate()).getMonth() - 1)) : new Date(dat.setDate(new Date(this.currentDate()).getDate() - day)));
                this.element.find(".e-navigateprevious").attr("tabIndex", 0).focus();
            }
            else if ($(e.currentTarget).hasClass("e-navigatenext")) {
                this.currentDate(this.currentView() === "month" ? new Date(dat.setMonth(new Date(this.currentDate()).getMonth() + 1)) : new Date(dat.setDate(new Date(this.currentDate()).getDate() + day)));
                this.element.find(".e-navigatenext").attr("tabIndex", 0).focus();
            }
            if ($(e.currentTarget).hasClass("e-navigateprevious") || $(e.currentTarget).hasClass("e-navigatenext"))
                data = { prevDate: new Date(dat.getDate() - 1), currentDate: dat, target: e, type: "navigatedate" };
            if (this._trigger("navigation", data))
                return false;
        }
        this.element.find(".e-workcells,.e-alldaycells,.e-monthcells").removeClass("e-selectedCell");
        this.element.find('.e-selectedAppointment').removeClass("e-selectedAppointment"); this.element.find(".e-appsoverflow").remove();
        this.element.find(".e-monthcells").removeClass("e-monthcurrenttime");
        if (this.model.showCurrentTimeIndicator) $(".e-highlightcurrenttimeline").remove(); 
        this.model.showQuickWindow && (this._quickAppointWindow.ejDialog("close") && this._quickAppDetailsWindow.ejDialog("close"));
        //  this.element.find("td.e-workcells").removeClass("icon-AddGrad");
        if (this.currentView() === "month")
            this._renderCurrentView();
        else {
            this._renderHeaderDays();
            if (!ej.isNullOrUndefined(this.dataSource())) {
                this._dataProcessing(this._currentAppointmentData);
                this._renderAppointmentAll();
            }
            (this.model.highlightBusinessHours) ? this._highlightBusinessHours() : this.element.find(".e-workcells").removeClass("e-businesshighlightworkcells");
            this._highlightCurrentTime(this.model.showCurrentTimeIndicator);
        }
        this._setCurrentDate = false;
    },
    /**
    * Section For handle the device type.
    * @private
    */
    _isDevice: function () {
        // return (/mobile|tablet|android|kindle/i.test(navigator.userAgent.toLowerCase()));
        // comment above line temporary. Due to the below code event wont bind for tablet device
        return (/mobile|tablet|android|kindle/i.test(navigator.userAgent.toLowerCase()));
    },
    /**
    * Section For handle the resize of the schedule.
    * @private
    */
    _scheduleResize: function (e) {
        this.model.showQuickWindow && (this._quickAppDetailsWindow.ejDialog("close") || this._quickAppointWindow.ejDialog("close"));
        if (this.model.enableResize) {
            var schedule = this;
            if (this.resizeTO) clearTimeout(this.resizeTO);
            this.resizeTO = setTimeout(function () {
                if (!schedule._isDevice()) {
                    var scheduleWidth = $($(schedule.element.find("div")[0]).parent()).width() + 20; // $(this.element.find("table")[0]).width + 20;
                    var scheduleHeight = $($(schedule.element.find("div")[0]).parent()).height(); // this.element.find("table")[0].offsetHeight;
                    if (scheduleWidth > $(this).width()) {
                        schedule._scWidth = $(this).width() - 20;
                    }
                    else if (scheduleWidth < $(this).width()) {
                        schedule._scWidth = schedule.model.width;
                    }
                    if (scheduleHeight > $(this).height()) {
                        schedule._scHeight = $(this).height() - 20;
                    }
                    else if (scheduleHeight < $(this).height()) {
                        schedule._scHeight = schedule.model.height;
                    }
                }
                else {
                    schedule._scHeight = schedule.element.height();
                    schedule._scWidth = schedule.element.width();
                }
                schedule.element.find(".e-highlightcurrenttimeline").remove();
                schedule.element.children().remove();
                schedule._renderInitSchedule();
                schedule._renderAppointmentAll();
            }, 500);
        }
    },
    /**
    * Section For render the templates used in the schedule.
    * @private
    */
    _renderTemplates: function () {
        //content area template
        if (this.model.orientation == "vertical") {
            this.workcellTemp = $.templates(this._createTemplate(workCellsTemplate(), "_WorkCells"));
            this.monthcellTemp = $.templates(this._createTemplate(monthCellsTemplate(), "_MonthCells"));
            //timecells template rendering
            this.timeCellTemplate = $.templates(this._createTemplate(timeCellsTemplate(), "_TimeCells"));
            //header day template rendering
            this.headTemplate = $.templates(this._createTemplate(headerTemplate(), "_HeaderDate"));
            //allday template rendering
            this.alldayTemp = $.templates(this._createTemplate(alldayTemplate(), "_AlldayCells"));
            this.leftIndent = $.templates(this._createTemplate(leftHeaderTemplate(), "_LeftIndentCells"));
        }
        if (ej.scheduleFeatures.resources && !ej.isNullOrUndefined(this.model.resources))
            this._renderResourcesTemplate();        

        // appointment template rendering
        this.appTemplate = $.templates(this._createTemplate(appointmentTemplate(), "_Appointment"));

        //horizontal header template
        if (ej.scheduleFeatures.horizontal && this.model.orientation === "horizontal")
            this._renderHorizontalTemplates();
        if (!ej.isNullOrUndefined(this.model.categorizeSettings) && this.model.categorizeSettings.enable ) {
            this._renderCategorizeTemplate();
            this._categorizeMultipleTemplate = $.templates(this._createTemplate(categorizeMultipleTemplate(), "_CategorizeMultiple"));
        }
    },
        /**
        * To configure start and end hour of the schedule		
    * @private
    */
    _setNewStartEndHours: function () {
        if (this.currentView() !== "month") {
            this.wTR.children().remove();
            this._renderContentAreaTemplate();
            this._renderScroller();
            this.element.find(".e-ampmdisplay").height(this.element.find(".e-workcells").height());
            (this.model.highlightBusinessHours) ? this._highlightBusinessHours() : this.element.find(".e-workcells").addClass("e-businesshighlightworkcells");
            if (this.model.showCurrentTimeIndicator) $(".e-highlightcurrenttimeline").remove(); 
            this._highlightCurrentTime(this.model.showCurrentTimeIndicator);
            this._renderAppointmentAll();
        }
    },
    /**
    * Section For render the schedule control initially.
    * @private
    */
    _renderInitSchedule: function () {
        this.model.width = this.model.width ? this.element.width(this.model.width).width() : this.element.width();
        this.model.height = this.model.height ? this.element.height(this.model.height).height() : this.element.height();
        this.scWidth = this._scWidth ? parseInt(this._scWidth) : parseInt(this.model.width);
        this.scHeight = this._scHeight ? parseInt(this._scHeight) : parseInt(this.model.height);

        this.element.addClass("e-scheduleouter").height(this.scHeight).width(this.scWidth).attr("role", "presentation");
        $(this.element).attr("tabIndex", 1);
        var $scheduleDiv = ej.buildTag('div', "", {}, { height: this.scHeight, width: this.scWidth });
        var $scheduleTable = ej.buildTag('table.e-table', "", {}, { cellpadding: "0px", cellspacing: "0px" }).addClass("e-scheduleoutertable");
        var $tBody = ej.buildTag('tbody');
        var hTR = ej.buildTag("tr.e-scheduleheader");
        this.aTR = ej.buildTag("tr");
        this.model.views.toString().toLowerCase().split(',').indexOf(this.currentView()) == -1 && this.currentView(this.model.views[0].toLowerCase());
        this._getRenderDates();
        this.wTR = ej.buildTag("tr.e-contentarearow");
        this._renderTemplates();
        this._renderHeaderBar(hTR); //header dates div(prev & next)
        this._renderHeaderAllDayTemplate(); //header and all-day div
        this._renderContentAreaTemplate(); //contentarea div

        $scheduleDiv.append($scheduleTable.append($tBody.append(hTR).append(this.aTR).append(this.wTR)));
        this.element.append($scheduleDiv);
        this.element.find(".e-" + this.currentView()).addClass("e-activeview");
        this.element.find("#" + this._id + "_navigator").ejDatePicker({ enableRTL: this.model.enableRTL, buttonText: this._getLocalizedLabels("Today"), select: $.proxy(this._onselect, this), locale: this.model.locale, cssClass: this.model.cssClass });
        this.element.find("span.e-datewidget").css("display", "none");
        if(this.model.orientation == "vertical"){
            this._headerbarWidth(); (this.currentView() === "month") && this._renderMonthTimeCells();
        }
        this.element.find(".e-headercells").addClass("e-dateheaderselect");
        (this.model.highlightBusinessHours) ? this._highlightBusinessHours() : this.element.find(".e-workcells").removeClass("e-businesshighlightworkcells");
        this._renderAppointmentWindow();
        //Render Recurrence edit dialog.
        this._renderRecurEditWindow();
        this.model.showQuickWindow && (this._renderQuickAppWindow() || this._renderQuickAppDetailsView());
        this.element.find(".e-schedulesettings").html(this._getLocalizedLabels("AllDay"));
        // render scroller
        this._renderScroller();
        this.model.currentView == "month" ? this.element.find(".e-timecells").height($(this.element.find(".e-monthcells")).height()) : this.element.find(".e-ampmdisplay").height($(this.element.find(".e-workcells")).height());
        this._highlightCurrentTime(this.model.showCurrentTimeIndicator);
        this._showContextMenu(this.model.contextMenuSettings.enable);
        if (this.model.orientation == "horizontal" && ej.scheduleFeatures.horizontal)
            this._moveScroller();
    },
    _headerbarWidth: function () {
        var cols = this.currentView() == "workweek" ? 5 : this.currentView() == "day" ? 1 : 7;
        var col = (!ej.isNullOrUndefined(this.model.group) && !ej.isNullOrUndefined(this.model.resources)) ? (this.res1.length * cols) : cols;
        this.element.find(".e-headrealldaytable").width(this.model.cellWidth ? (parseInt(this.model.cellWidth) * col) + 18 + col : "100%");
    },
    /**
    * Section For render the scroller of the schedule.
    * @private
    */
    _renderScroller: function () {
        var _scrollDiv = $('div#' + this._id + "_scroller");
        var hghtdiff = this.model.orientation === "vertical" ? this.element.find(".e-scheduleheader").height() + this.element.find(".e-headrealldaytable").height() : this.currentView() != "month" ? this.element.find(".e-scheduleheader").height() + this.element.find(".e-horizontalheader").height() + 18 : this.element.find(".e-scheduleheader").height() + this.element.find(".e-horizontalheader").height(); var proxy = this; var wdthdiff = (this.currentView() === "month") ? 0 : 4;
        var scrollWidth = this.model.orientation === "vertical" ? this.scWidth - this.element.find(".e-scrolltimecells").width() + wdthdiff : (!ej.isNullOrUndefined(this.model.group) && !ej.isNullOrUndefined(this.model.resources)) ? this.scWidth - 200 : this.scWidth;
        var cols = this.currentView() == "workweek" ? 5 : this.currentView() == "day" ? 1 : 7;
        var col = (!ej.isNullOrUndefined(this.model.group) && !ej.isNullOrUndefined(this.model.resources)) ? (this.res1.length * cols) : cols;
        _scrollDiv.ejScroller({
            enableRTL: this.model.enableRTL, height: this.scHeight - Math.floor(hghtdiff), width: Math.floor(scrollWidth), cssClass: this.model.cssClass,
            thumbStart: function (e) {
                var target = $(e.originalEvent.target);
                if (target.parents(".e-appointment").hasClass("e-appointment") || target.parents(".e-monthappointment").hasClass("e-monthappointment") || target.hasClass("e-categorybar")) return false;
            },
            scroll: function (e) {
                (proxy.model.orientation === "vertical") ? proxy.element.find(".e-scrolltimecells").css("top", (-e.scrollTop) + "px") : proxy.currentView() != "month" ? $(proxy.element.find(".e-horizontaltimecells").find("table tr")[2]).css("left", (-e.scrollLeft) + "px") : proxy.element.find(".e-headerdays").css("left", (-e.scrollLeft) + "px"); // proxy.element.find(".e-horizontaltimecells").css("left", (-e.scrollLeft) + "px");
                if (proxy.model.orientation == "vertical") {
                    proxy.element.find(".e-headrealldaytable").css("left", (-e.scrollLeft) + "px");
                    //  changing AM/PM label while scrolling
                    if (proxy.model.timeMode === "12" && proxy.currentView() !== "month") {
                        var top = (e.scrollTop / (proxy.element.find(".e-workcells").height() * 2));
                        top = (Math.round(top) < top) ? Math.round(top) + 1 : Math.round(top);
                        var localizeAmPm = Globalize.culture(proxy.model.locale).calendar;
                        if (localizeAmPm.AM && localizeAmPm.PM) {
                            proxy.element.find('.e-ampmstyle').each(function (i, val) {
                                if (i == (top))
                                    (parseInt($(val)[0].id) < 12) ? $(val).text(localizeAmPm.AM[0]) : $(val).text(localizeAmPm.PM[0]);
                                else if (($(val).text() == localizeAmPm.AM[0] && i > 0) || ($(val).text() == localizeAmPm.PM[0] && parseInt($(val)[0].id) > 12))
                                    $(val).text("00");
                            });
                        }
                    }
                }
                else if (ej.scheduleFeatures.horizontal && proxy.model.orientation == "horizontal")
                    proxy._horizontalScroll(e, proxy);
            }
        });
        this._nextPrevAppointment(this.model.enableAppointmentNavigation);
        if (this.model.orientation == "vertical") {
            _scrollDiv.ejScroller("isVScroll") ? this.element.find(".e-emptytd,.e-emptyalldaytd").show() : this.element.find(".e-emptytd,.e-emptyalldaytd").hide();
                if (this.model.cellWidth) {
                (this.currentView() == "day") && this.element.find(".e-headerdaydisplay").css("width", this.model.cellWidth);

                var rows = this._dateRender.length / 7;
                var overallWidth = parseInt(this.model.cellWidth) * col;
                var overallHeight = parseInt(this.model.cellHeight) * rows;
                var dragWidth = this.scWidth - this.element.find(".e-scrolltimecells").width() + wdthdiff;
                var dragHeight = this.scHeight - hghtdiff;

                if (this.currentView() == "month") {
                    this.element.find(".e-draggableworkarea").width((this.scWidth > overallWidth + this.element.find(".e-scrolltimecells").width() + 18) ? overallWidth : _scrollDiv.ejScroller("isVScroll") ? dragWidth - 18 : dragWidth);
                    this.element.find(".e-draggableworkarea").height(this.scHeight > (overallHeight + hghtdiff) ? overallHeight : _scrollDiv.ejScroller("isHScroll") ? dragHeight - 18 : dragHeight);
                    this.element.find(".e-emptytd").show();
                }
                else
                    this.element.find(".e-draggableworkarea").width((this.scWidth > overallWidth + this.element.find(".e-scrolltimecells").width() + 18) ? overallWidth : dragWidth - 18);
            }
            else {
                    this.element.find(".e-draggableworkarea").height(this.scHeight - hghtdiff);
                    this.element.find(".e-scrolltimecells").height(this.scHeight - hghtdiff);
            }
        }
        else {
            (!ej.isNullOrUndefined(this.model.group) && !ej.isNullOrUndefined(this.model.resources)) ? this.element.find(".e-horizontresdiv").css("height", (this.scHeight - hghtdiff + (this.currentView() != "month" ? 18 : 0)) + "px") : "";
            !(_scrollDiv.ejScroller("isHScroll")) ? this.element.find(".e-draggableworkarea").css("height", "100%") && this.element.find(".e-resfootertr").css("height", "0px") : this.currentView() == "month" ? this.element.find(".e-draggableworkarea").css("height", this.scHeight - hghtdiff - 18) : "";
            if (this.currentView() == "month")
                this.element.find(".e-draggableworkarea").width(scrollWidth);
        }
    },
    /**
    * Section For handle the business hour highlight of the schedule.
    * @private
    */

    _highlightBusinessHours: function () {
        if (this.currentView() !== "month") {
            this.element.find(".e-workcells").removeClass("e-businesshighlightworkcells");
            var renderDates = this.dateRender;
            var target = this.element.find(".e-workcells");
            var b_startHour = this.model.businessStartHour >= this.model.startHour ? this.model.businessStartHour - this.model.startHour : 0;
            var b_endHour = this.model.businessEndHour <= this.model.endHour ? this.model.businessEndHour - this.model.startHour : this.model.endHour - this.model.startHour;
            var timeRowsCount = 60 / 30;
            this.res1 = (!ej.isNullOrUndefined(this.model.group) && !ej.isNullOrUndefined(this.model.resources)) ? this.res1 : [0];
            var colCount = this.currentView() === "week" ? 7 * this.res1.length : this.currentView() === "workweek" ? 5 * this.res1.length : 1 * this.res1.length;
            if (this.model.orientation == "vertical") {
                for (var i = (b_startHour * timeRowsCount) ; i < (b_endHour * timeRowsCount) ; i++) {
                    for (var j = 0; j < colCount; j++) {
                        if (new Date(renderDates[j]).getDay() !== 0 && new Date(renderDates[j]).getDay() !== 6)
                            $(target[(i * colCount) + j]).addClass("e-businesshighlightworkcells");
                    }
                }
            }
            else
                this._horizontalBusinessHighlight(this._dateRender, b_startHour, b_endHour, timeRowsCount, target);
        }
        else {
            var target, monthDays;
            if (this.model.orientation == "vertical") {
                monthDays = this.dateRender;
                target = this.element.find(".e-monthcells");
                target.removeClass("e-monthcellhighlight");
                for (var i = 0; i < monthDays.length; i++) {
                    if (new Date(monthDays[i]).getDay() !== 0 && new Date(monthDays[i]).getDay() !== 6)
                        $(target[i]).addClass("e-monthcellhighlight");
                }
            }
            else
                this._horizontalBusinessHighlight(null, null, null, null, null);
        }
    },
    /**
    * Section For handle the calendar date select and change the current date of the schedule.
    * @private
    */
    _onselect: function (e) {
        var proxy = this;
        var currentdate = new Date(this.currentDate());
        var selectdate = Globalize.parseDate(e.value, this._pattern.d);
        this.currentDate(selectdate);
        var data = { prevDate: currentdate, currentDate: this.currentDate(), target: e }
        if (this._trigger("navigation", data))
            return false;
        proxy._navigateClick(e);
    },
    /**
    * Section For render the header bar of the schedule.
    * @private
    */
    _renderHeaderBar: function (hTR) {
        var colsVal = this.model.orientation == "vertical" ? 1 : 2;
        this.hTD = ej.buildTag('td', "", {}, { colspan: colsVal });
        var commonDiv = ej.buildTag('div.e-datecommondiv').append(ej.buildTag('div.e-icon e-navigateprevious', "", {}, { title: this._getLocalizedLabels("Previous") })).append(ej.buildTag('div', "", {}, { "aria-live": "assertive", tabindex: 0 }).html(this._getCurrentDateHeaderValue()).addClass("e-datedisplay")).append(ej.buildTag("input#" + this._id + "_navigator").css("display", "none").addClass("dateCalendar")).append(ej.buildTag('div.e-icon e-navigatenext', "", {}, { title: this._getLocalizedLabels("Next") }));
        var buttonsDiv = ej.buildTag('div.e-viewsdiv');
        hTR.append(this.hTD.append(ej.buildTag("div").css("width", this.scWidth).append(commonDiv).append(buttonsDiv.append(this._renderViewNavigation(this.model.views)))));
    },
    /**
    * Section For render the views of the schedule.
    * @private
    */
    _renderViewNavigation: function (views) {
        var $viewTable = ej.buildTag('table.e-table e-viewstable', "", {}, { cellspacing: '0px', cellpadding: '0px' });
        var $viewBody = ej.buildTag('tbody');
        var $viewTR = ej.buildTag('tr');
        var _viewCollection = views, viewClassname; //this.model.views; // ["Day", "Week", "WorkWeek", "Month", "Agenda"];
        for (var i = 0; i < _viewCollection.length; i++) {
            viewClassname = (_viewCollection.length == 1) ? "e-" + _viewCollection[i].toLowerCase() + " e-singleview" : (i == 0) ? "e-" + _viewCollection[i].toLowerCase() + " e-firstview" : i == (_viewCollection.length - 1) ? "e-" + _viewCollection[i].toLowerCase() + " e-lastview" : "e-" + _viewCollection[i].toLowerCase();
            $viewTR.append(ej.buildTag('td').append(ej.buildTag('div', this._getLocalizedLabels(_viewCollection[i]), {}, {}).addClass("e-commonviewbutton").addClass(viewClassname)));
        }
        $viewTable.append($viewBody.append($viewTR))
        return $viewTable;
    },
    /**
    * Section For render the recurrence window of the schedule.
    * @private
    */
    _recurrenceWindow: function () {
        var recurtype = ["Daily", "Weekly", "Monthly", "Yearly", "EveryWeekDay"];
        var culture = Globalize.culture(this.model.locale);
        var weekshort = this._dayShortNames;
        var weekfull = this._dayFullNames;
        var months = this._monthNames;
        var startday = ["First", "Second", "Third", "Fourth", "Last"];
        var $recurWin = "<div class='e-recurwindow e-windowmargin' style='display:none'>" +
                    "<form id='" + this._id + "_RecurForm'><table class='e-table' width='100%' cellpadding='7'><tbody>";
        $recurWin += "<tr><td width='16%' class='e-textlabel'>" + this._getLocalizedLabels("Recurrence") + ":</td>" +
                    "<td><input class='recurtype' id='recurtype' type='text' name='RecurrenceType' value='" + this._getLocalizedLabels(recurtype[0]) + "' /><div id='recurtypelist'><ul>";
        for (var type = 0; type < recurtype.length; type++) {
            $recurWin += "<li>" + this._getLocalizedLabels(recurtype[type]) + "</li>";
        }
        $recurWin += "</ul></div></td></tr>";
        $recurWin += "<tr id='" + this._id + "repeatEvery'><td class='e-textlabel'>" + this._getLocalizedLabels("Every") + ":</td><td><table><tr><td><div class='e-floatleft'><input id='recurevery' class='e-recurevery' type='text' /></div></td>" +
                    "<td><div id='" + this._id + "recurtypes' class='e-appcheckbox'>" + this._getLocalizedLabels("RecurrenceDay") + "</div></td></tr></table></td></tr>";

        $recurWin += "<tr id='" + this._id + "weeklyrecur' style='display:none'><td id='recurweek' class='e-textlabel'><div>" + this._getLocalizedLabels("RepeatOn") + ":</div></td>" +
                    "<td id='recurweekcount'><table class='e-table' cellpadding='3'><tr>";
        for (var weekcount = 0; weekcount < weekshort.length; weekcount++) {
            $recurWin += "<td><div class='e-weekday e-_" + weekshort[weekcount] + "'><input id='" + weekshort[weekcount] + "' class='weekdays e-weekly" + this._dayNames[weekcount] + "'name='weekdays' type='checkbox'/><label class='e-textmargin' for='" + weekshort[weekcount] + "'>" + weekshort[weekcount] + "</label></div></td>";
        }
        $recurWin += "</tr></table></td></tr>";
        $recurWin += "<tr id='" + this._id + "monthlyrecur' style='display:none'><td id='recurmonthtext' class='e-recurendslabel'><div class='e-recurendsalign'>" + this._getLocalizedLabels("RepeatBy") + ":</div></td>" +
                    "<td id='recurmonthcount'><table class='e-table' cellpadding='3'><tr><td><div><input id='" + this._id + "monthday' class='e-monthdaytype' name='monthday' type='radio'/><label class='e-textmargin' for='" + this._id + "monthday'>" + this._getLocalizedLabels("Day") + "</label></div></td>" +
                    "<td><div><input id='monthdate' class='e-monthdate' type='text'/></div></td></tr>" +
                    "<tr><td><div><input id='" + this._id + "monthon' class='e-monthposition' name='monthday' type='radio'/><label class='e-textmargin' for='" + this._id + "monthon'>" + this._getLocalizedLabels("The") + "</label></div></td>" +
                    "<td><div><input id='monthsrt' class='monthsrt' type='text' name='monthsrt' value=''/><div id='monthsrtlist'><ul>";
        for (var day = 0; day < startday.length; day++) {
            $recurWin += "<li>" + this._getLocalizedLabels(startday[day]) + "</li>";
        }
        $recurWin += "</div></div></td>";
        $recurWin += "<td><div class='e-appcheckbox'><input id='monthsrtday' class='e-monthsrtday' type='text' name='monthsrtday' value=''/>" +
            "<div id='monthsrtdaylist'><ul>";
        for (var monthday = 0; monthday < weekfull.length; monthday++) {
            $recurWin += "<li>" + weekfull[monthday] + "</li>";
        }
        $recurWin += "</div></div></td></tr></table></td></tr>";
        $recurWin += "<tr id='" + this._id + "yearlyrecur' style='display:none'><td id='recuryeartext' class='e-recurendslabel'><div class='e-recurendsalign'>" + this._getLocalizedLabels("RepeatBy") + ":</div></td>" +
                    "<td id='recuryearcount' ><table class='e-table' cellpadding='3'><tr>" +
                    "<td><div><input id='" + this._id + "yearday' class='e-yearrecurday' name='yearday' type='radio'/><label class='e-textmargin' for='" + this._id + "yearday'>" + this._getLocalizedLabels("The") + "</label></div></td>" +
                    "<td><div class='e-controlalign'><input id='yearmonth' class='yearmonth' type='text' name='yearmonth' value=''/></div></td>" +
                    "<div id='yearmonthlist'><ul>";
            for (var monthcount = 0; monthcount < (months.length - 1); monthcount++) {
                $recurWin += "<li>" + months[monthcount] + "</li>";
            }
            $recurWin += "</ul></div><td><input id='yeardate' class='yeardate' text='text'/></td></tr>" +
                    "<tr><td><div><input id='" + this._id + "yearother' class='e-yearrecurposi' name='yearday' type='radio'/><label class='e-textmargin' for='" + this._id + "yearother'>" + this._getLocalizedLabels("The") + "</label></div></td>" +
                    "<td><div><input id='yearsrt' class='yearsrt' type='text' name='yearsrt' value=''/><div id='yearsrtlist'><ul>";
        for (var yearstartday = 0; yearstartday < startday.length; yearstartday++) {
            $recurWin += "<li>" + this._getLocalizedLabels(startday[yearstartday]) + "</li>";
        }
        $recurWin += "</ul></div></div></td><td><div class='e-controlalign'><input id='yearsrtday' class='yearsrtday' type='text' name='yearsrtday' value=''/>" +
            "<div id='yearsrtdaylist'><ul>";
        for (var yearweek = 0; yearweek < weekfull.length; yearweek++) {
            $recurWin += "<li>" + weekfull[yearweek] + "</li>";
            }
            $recurWin += "</ul></div></div></td><td><div><span>" + this._getLocalizedLabels("OfEvery").split(" ")[0] + "</span></div></td><td><div><input id='yearsrtmonth' class='yearsrtmonth' type='text' name='yearsrtmonth' value=''/>" +
                    "<div id='yearsrtmonthlist'><ul>";
            for (var yearmonth = 0; yearmonth < (months.length - 1); yearmonth++) {
                $recurWin += "<li>" + months[yearmonth] + "</li>";
            }
            $recurWin += "</ul></div></div></td></tr></table></td></tr>";
        $recurWin += "<tr><td class='e-textlabel'>" + this._getLocalizedLabels("StartsOn") + ":</td>" +
                    "<td><input id='recurstartdate' class='recurstartdate' type='text' name='RecurStartDate' value=''/></td></tr>";
        $recurWin += "<tr><td class='e-recurendslabel'><div class='e-recurendsalign'>" + this._getLocalizedLabels("Ends") + ":</div></td>" +
                    "<td><table class='e-table' cellpadding='3'><tr><td><div><input id='repeatendnever' class='recurends e-recurnoend' type='radio' name='repeatend' value='Never'/><label class='e-textmargin' for='repeatendnever'>" + this._getLocalizedLabels("Never") + "</label></div></td></tr>" +
                    "<tr><td><div><input id='" + this._id + "repeatendafter' class='recurends e-recurafter' type='radio' name='repeatend'/><label class='e-textmargin' for='" + this._id + "repeatendafter'>" + this._getLocalizedLabels("After") + "</label>" +
                    "</div></td><td><div><input id='dailyoccurance' class='e-recurcount' type='text'/></div></td>" +
                    "<td><span style='margin-left: -80px;'>" + this._getLocalizedLabels("Occurence") + "</span></td></tr>" +
                    "<td><div><input id='" + this._id + "repeatendon' class='recurends e-recuruntil' type='radio' name='repeatend'/><label class='e-textmargin' for='" + this._id + "repeatendon'>" + this._getLocalizedLabels("On") + "</label></div></td>" +
                    "<td><input id='daily' class='e-until' type='text' name='daily' value=''/></td></tr></table></td></tr>";
        $recurWin += "<tr style='display:none'><td><span class='e-textlabel'>" + this._getLocalizedLabels("Summary") + ":</span></td><td><span class=e-recurRule></span></td></tr></tbody></table></form>";
        $recurWin += "<div class='e-floatright'><table class='e-table' cellpadding='5'><tr><td><div class='e-btnmargin e-floatleft'><button id='donerecur' class='recurbutton'>" + this._getLocalizedLabels("Done") + "</button></div></td>" +
                    "<td><div class='e-btnmargin e-floatleft'><button id='cancelrecur' class='e-cancelrecur recurbutton'>" + this._getLocalizedLabels("Cancel") + "</button></div></td></tr></tabled></div></div>";
        return $recurWin;
    },
    /**
    * Section For render the appointment window of the schedule.
    * @private
    */
    _renderAppointmentWindow: function () {
        this._appointmentAddWindow = ej.buildTag('div.e-scheduledialog#' + this._id + "AppointmentAddEditWindow");
        var $appWindow = "<div class='e-appwindow e-windowmargin'>" +
                            "<form id='" + this._id + "_AddEditForm'><table class='e-table' width='100%' cellpadding='7'><tbody>";
        $appWindow += "<tr><td class='e-leftfields e-textlabel'>" + this._getLocalizedLabels("AppointmentSubject") + ":</td><td colspan='3' class='e-rightfields'>" +
                          "<input id='" + this._id + "subject' tabIndex='0' class='subject e-inputtext' name='" + this.model.appointmentSettings["subject"] + "' type='text' /><input id='id' type='text' name='" + this.model.appointmentSettings["id"] + "' value='' style='display: none'/></td></tr>";
        $appWindow += "<tr><td class='e-textlabel'>" + this._getLocalizedLabels("StartTime") + ":</td><td width='25%'><input id='" + this._id + "startdate' class='startenddate' type='text' name='StartDate' value='' /></td><td width=22%'><input type='text' id='" + this._id + "starttime' class='startendtime' name='StartTime'/></td>" +
                          "<td width='28%'><input id='startutc' class='startutc' type='text' value=''/><div id='Timezoneliststr'></div></td></tr>";
        $appWindow += "<tr><td class='e-textlabel'>" + this._getLocalizedLabels("EndTime") + ":</td><td><input id='" + this._id + "enddate' class='startenddate' type='text' name='EndDate' value='' /></td><td><input type=''text' id='" + this._id + "endtime' class='startendtime' name='EndTime'/></td>" +
                          "<td><input id='endutc' class='endutc' type='text' value=''/><div id='Timezonelistend'></div></td></tr>";
        $appWindow += "<tr><td colspan='4'><table class='e-table' cellpadding='3'><tr><td><div class='e-allday e-checkboxmargin'><input id='" + this._id + "allday' tabIndex='0' class='allday' type='checkbox' name='" + this.model.appointmentSettings["allDay"] + "'/></div></td><td><div class='e-alldaytext'>" +
                          "<label for='" + this._id + "allday'>" + this._getLocalizedLabels("AllDay") + "</label></div></td><td><div class='e-checkboxmargin e-recurrence'>" +
                          "<input id='" + this._id + "recurrence' tabIndex='0' class='recurrence' type='checkbox' name='" + this.model.appointmentSettings["recurrence"] + "' value='recurrence'/></div></td><td><label class='e-repeatlbl' for='" + this._id + "recurrence'>" +
                          this._getLocalizedLabels("Recurrence") + "</label></td></tr></table></td></tr>";
            $appWindow += "<tr id='" + this._id + "summary' style='display:none'><td class='e-textlabel'><label>" + this._getLocalizedLabels("Summary") + ":</label></td><td colspan='3'><table><tr><td><label class='e-recurFinalRule'></label></td><td><div class='e-recuredit'><a class='e-recuredit' href='#'>" + this._getLocalizedLabels("Edit") + "</a><span class='e-icon e-scheduleedit'></span></div></td></tr></table></td></tr>";         
            if (this.model.categorizeSettings.enable) {
                $appWindow += "<tr><td class='e-leftfields e-textlabel'>" + this._getLocalizedLabels("Categorize") + ":</td><td colspan='3' class='e-rightfields'>" + "<input id= '" + this._id + "_categorize' class='e-categorizevalue' type='text' value=''/><div id='ownerlist'></div></td></tr>";
            }

            if (!ej.isNullOrUndefined(this.model.resources) && this.model.resources.length > 0 && ej.scheduleFeatures.resources)
                $appWindow = this._renderResourceElements($appWindow);
              $appWindow += "<tr><td colspan='4'><textarea class='e-description e-inputtext' id='description' name='" + this.model.appointmentSettings["description"] + "'></textarea></td></tr>";
            $appWindow += "</tbody></table></form>";
            $appWindow += "<div class='e-floatright'><table class='e-table' cellpadding='5'><tr><td><div class='e-btnmargin e-floatleft'><button id='done' class='e-btndone e-appsave'>" + this._getLocalizedLabels("Done") + "</button></div></td>" +
                          "<td><div class='e-btnmargin e-floatleft'><button id='cancel' class='e-appointcancel cancel'>" + this._getLocalizedLabels("Cancel") + "</button></div></td></tr></table></div></div>";
            this._appointmentAddWindow.append($appWindow).append(this._recurrenceWindow());
        this.element.append(this._appointmentAddWindow);
        this._renderControls();
        this._appointmentAddWindow.ejDialog({
            width: 600,
            cssClass: this.model.cssClass,
            showOnInit: false,
            enableRTL: this.model.enableRTL, enableModal: true,
            title: this._getLocalizedLabels("CreateAppointmentTitle"),
            content: "#" + this._id,
            enableResize: false,
            allowKeyboardNavigation: false,
            close: $.proxy(this._onclose, this)
        });
        $("#" + this._id + "AppointmentAddEditWindow_wrapper").appendTo("body").addClass("e-scheduledialog").find(".e-titlebar").addClass("e-dialogheader");
        this._on(this._appointmentAddWindow, "click", "a.e-recuredit", $.proxy(this._editRecurrence, this));
    },
    /**
    * Section For render the quick window.
    * @private
    */
    _renderQuickAppWindow: function () {
        this._quickAppointWindow = ej.buildTag('div.e-scheduledialog#' + this._id + "AppointmentQuickWindow");
        var $quickWin = "<div id='" + this._id + "_quickAppwindow' class='e-tableAppoint'>" +
                    "<table cellpadding='7' class='e-qappwindow'><tbody>";
        $quickWin += "<tr><td colspan='2'><div class='e-quickstartend'></div> <div title='" + this._getLocalizedLabels("Close") + "'class='e-icon e-scheduleclose e-schedclosemargin e-pointercursor'></div></td></tr><tr><td class='e-subjectlabel'>" + this._getLocalizedLabels("Event") + ":</td><td colspan='2' class='e-rightfields'>" +
                          "<input id='" + this._id + "subject' class='subject e-inputtext' style='height:22px;' type='text' /></td></tr></tbody></table>";
        $quickWin += "<div style='margin-top:5px; padding-bottom: 30px;'><div tabIndex='0' class='e-detailedapp e-detailedwindow'> " + this._getLocalizedLabels("Detailed") + " >></div><div style='margin-right:-3px;' class='e-floatright'><div class='e-quickAppDone'><button id='done' class='e-btndone qAppDone' style='font-size:12px !important;'>" + this._getLocalizedLabels("CreateAppointmentTitle") + "</button></div>" +
                          "</div></div></div>";
        this._quickAppointWindow.append($quickWin);
        this.element.append(this._quickAppointWindow);
        this._quickAppointWindow.append(ej.buildTag("div.e-quickarrow").append("<div class='e-outerarrowline'></div>").append("<div class='e-innerarrow'></div>"));
        var control = this._quickAppointWindow;
        //control.find('.subject').ejMaskEdit({ name: "Subject", inputMode: ej.inputMode.Text, width: "100%", height: "25px", cssClass: this.model.cssClass });
        control.find('.e-btndone').ejButton({ showRoundedCorner: true, width: "130px", click: $.proxy(this._saveAppointment, this), cssClass: this.model.cssClass, enableRTL: this.model.enableRTL });
        this._on(control.find('.e-scheduleclose'), "click", $.proxy(this._quickWindowClose, this));
        this._on(control.find('.e-detailedapp'), "click", $.proxy(this._appointmentWindow, this));
        this._quickAppointWindow.ejDialog({
            width: 330,
            cssClass: this.model.cssClass,
            enableRTL: this.model.enableRTL,
            showOnInit: false,
            enableResize: false,
            allowDraggable: false, showHeader: false,
            allowKeyboardNavigation: false
        });
        $("#" + this._id + "AppointmentQuickWindow_wrapper").addClass("e-scheduledialog").find("#" + this._id + "AppointmentQuickWindow").addClass("e-dialogouter");
    },
    /**
    * Section For handle the details of quick app window.
    * @private
    */
    _renderQuickAppDetailsView: function () {
        this._quickAppDetailsWindow = ej.buildTag('div.e-scheduledialog#' + this._id + "AppDetailsWindow");
        var $quickAppWin = "<div id='" + this._id + "_quickAppDetailswindow' class='e-tableAppoint'>" +
                          "<table cellpadding='1' class='e-qappwindow'><tbody>";
        $quickAppWin += "<tr><td colspan='2'><div class='e-quicksubject' style='float:left;'></div> <div title='" + this._getLocalizedLabels("Close") + "' class='e-icon e-scheduleclose e-schedclosemargin e-pointercursor'></div></td></tr><tr><td colspan='2' class='e-rightfields'>" +
                          "</td></tr><tr><td colspan='2'><div class='e-quickstartendtime'></div></td></tr></tbody></table>";
        $quickAppWin += "<div style='margin-top:5px;'><div title='" + this._getLocalizedLabels("Delete") + "' class='e-icon e-scheduledelete e-pointercursor' tabIndex='0'></div><div style='margin-right:-15px;' class='e-floatright'><div class='e-editevents'><div class='e-editevent' tabIndex='0'>" + this._getLocalizedLabels("Editevent") + " | " + "</div><div class='e-editseries' tabIndex='0'>" + this._getLocalizedLabels("Editseries") + " >></div> </div>" +
                          "</div></div></div>";
        this._quickAppDetailsWindow.append($quickAppWin);
        this.element.append(this._quickAppDetailsWindow);
        this._quickAppDetailsWindow.append(ej.buildTag("div.e-quickarrow").append("<div class='e-outerarrowline'></div>").append("<div class='e-innerarrow'></div>"));
        this._on(this._quickAppDetailsWindow.find('.e-scheduleclose'), "click", $.proxy(this._quickAppDetailWindowClose, this));
        this._on(this._quickAppDetailsWindow.find('.e-scheduledelete'), "click", $.proxy(this._deleteAppoint, this));
        this._on(this._quickAppDetailsWindow.find('.e-editevent,.e-editseries'), "click", $.proxy(this._editClick, this));
        this._quickAppDetailsWindow.ejDialog({
            width: 270,
            cssClass: this.model.cssClass,
            enableRTL: this.model.enableRTL,
            showOnInit: false,
            enableResize: false,
            allowDraggable: false, showHeader: false,
            allowKeyboardNavigation: false
        });
        $("#" + this._id + "AppDetailsWindow_wrapper").addClass("e-scheduledialog").find("#" + this._id + "AppDetailsWindow").addClass("e-dialogouter");
    },
    /**
    * Section For edit click in quick window of the schedule.
    * @private
    */
    _editClick: function (e) {
        var query = new ej.Query().where(this.model.appointmentSettings["id"], ej.FilterOperators.equal, this._appId);
        var templist = new ej.DataManager(this._processed).executeLocal(query);
        if ($(e.target).hasClass("e-editevent")) {
            if (templist[0][this.model.appointmentSettings["recurrence"]] == 1) {
                this._parentId = templist[0]["ParentId"];
                this._currentAction = ej.Schedule.Actions.EditOcurrence;
                this._showAppointmentDetails(this._appId, true);
            }
            else if (ej.isNullOrUndefined(templist[0][this.model.appointmentSettings["recurrence"]])) {
                this._currentAction = ej.Schedule.Actions.Save;
                this._showAppointmentDetails(this._appId, false);
            }
            else {
                this._currentAction = ej.Schedule.Actions.Save;
                this._showAppointmentDetails(templist[0]["ParentId"]);
            }
        }
        else if ($(e.target).hasClass("e-editseries")) {
            if (templist[0][this.model.appointmentSettings["recurrence"]] == 1) {
                this._parentId = templist[0]["ParentId"];
                this._currentAction = ej.Schedule.Actions.EditSeries;
                this._showAppointmentDetails(this._parentId);
            }

        }
        this.model.showQuickWindow && this._quickAppDetailsWindow.ejDialog("close");
    },
    /**
    * Section For handle the delete appointments of the schedule.
    * @private
    */
    _deleteAppoint: function () {
        if (this._appId) {
            this._currentAction = ej.Schedule.Actions.Delete;
            var parentData = this._getAppointmentByParentId(this._appId);
            if (ej.isNullOrUndefined(parentData[this.model.appointmentSettings["recurrenceRule"]]))
                this._deleteApp(parseInt(this._appId));
            else {
                this._parentId = parseInt(parentData[this.model.appointmentSettings["id"]]);
                this._recurEditWindow.find(".e-editonly").ejRadioButton({ checked: true });
                this._recurEditWindow.ejDialog("open");
                this._recurEditWindow.focus();
            }
        }
        this.model.showQuickWindow && this._quickAppDetailsWindow.ejDialog("close");
    },
    /**
    * Section For handle the quick window close of the schedule.
    * @private
    */
    _quickAppDetailWindowClose: function () {
        this._quickAppDetailsWindow.ejDialog("close");
        this._processFocus();
    },
    /**
    * Section For handle the all day option check in the appoinment window.
    * @private
    */
    _alldayCheck: function () {
        if (this._appointmentAddWindow.find(".allday").ejCheckBox("option", "checked")) {
           this._appointmentAddWindow.find("#" + this._id + "starttime").ejTimePicker({value:(this.model.timeMode=="12")?"12:00 AM":"00:00", enabled: false });
           this._appointmentAddWindow.find("#" + this._id + "endtime").ejTimePicker({value:(this.model.timeMode=="12")?"11:59 PM":"23:30", enabled: false });
           this._appointmentAddWindow.find(".startutc,.endutc").ejDropDownList({ enabled: false });
        }
        else {
            this._appointmentAddWindow.find(".startendtime").ejTimePicker({ enabled: true });
            this._appointmentAddWindow.find(".startutc,.endutc").ejDropDownList({ enabled: true });
        }
    },
    /**
    * Section For render the recurrence edit window of the schedule .
    * @private
    */
    //Render Recurrence window 
    _renderRecurEditWindow: function () {
        this._recurEditWindow = ej.buildTag('div.e-scheduledialog#' + this._id + "RecurrenceEdit")
        var title = ej.buildTag('div.e-recurtitle', this._getLocalizedLabels("RecurrenceEditTitle"));
        title.append(ej.buildTag('div.e-icon e-scheduleclose e-schedclosemargin e-pointercursor', "", {}, { title: this._getLocalizedLabels("Close") }));
        this._recurEditWindow.append(title);
        var msgDiv = ej.buildTag('div.e-recureditmsg', this._getLocalizedLabels("RecurrenceEditMessage"));
        //<input type="radio" value="Never" name="repeatend" class="recurends e-radiobtn e-js e-input" id="repeatendnever" tabindex="">
        var editOnly = ej.buildTag('div.e-recureditopt').append(ej.buildTag('input.e-editonly', "", {}, { type: "radio", name: "recurEdit", id: this._id + "_editOnly" })).append(ej.buildTag('label.e-recureditlbl', this._getLocalizedLabels("RecurrenceEditOnly"), {}, { "for": this._id + "_editOnly" }));
        var editSer = ej.buildTag('div.e-recureditopt').append(ej.buildTag('input.e-editseries', "", {}, { type: "radio", name: "recurEdit", id: this._id + "_editSeries" })).append(ej.buildTag('label.e-recureditlbl', this._getLocalizedLabels("RecurrenceEditSeries"), {}, { "for": this._id + "_editSeries" }));
        var btns = ej.buildTag('div.e-recureditbtns').append(ej.buildTag('button.e-recurdone', this._getLocalizedLabels("Done"), {}, { name: "done", value: this._getLocalizedLabels("Done") })).append(ej.buildTag('button.e-recurcancel', this._getLocalizedLabels("Cancel"), {}, { name: "cancel", value: this._getLocalizedLabels("Cancel") }));
        this._recurEditWindow.append(msgDiv).append(editOnly).append(editSer).append(btns);
        this._recurEditWindow.find('.e-editonly,.e-editseries').ejRadioButton({ cssClass: this.model.cssClass });
        btns.find('.e-recurdone,.e-recurcancel').ejButton({ cssClass: this.model.cssClass, enableRTL: this.model.enableRTL });
        this._on(this._recurEditWindow, "click", ".e-recurdone,.e-recurcancel", $.proxy(this._recurBtnClick, this));
        this._on(this._recurEditWindow, "click", "div.e-scheduleclose", $.proxy(this._recrConformClose, this));
        this._recurEditWindow.appendTo(this.element);
        this._recurEditWindow.ejDialog({
            width: 360,
            cssClass: this.model.cssClass,
            enableRTL: this.model.enableRTL,
            showOnInit: false,
            enableModal: true,
            enableResize: false,
            showHeader: false,
            content: "#" + this._id,
            allowKeyboardNavigation: false,
            close: $.proxy(this._processFocus, this)
        });
        $("#" + this._id + "RecurrenceEdit_wrapper").appendTo("body");
    },
    /**
    * Section For handle the recurrence window close.
    * @private
    */
    // Recurrence Conform window close
    _recrConformClose: function () {
        this._recurEditWindow.ejDialog("close");
        this._processFocus();
    },
    /**
    * Section For handle the recurrence edit click in the appointment window.
    * @private
    */
    //Recurrence edit button click events
    _recurBtnClick: function (args) {
        if ($(args.target).hasClass("e-recurdone")) {
            if (this._recurEditWindow.find(".e-editonly").ejRadioButton("option", "checked")) {
                if (this._currentAction == ej.Schedule.Actions.Delete) {
                    var _newAddList = new ej.DataManager(this._processed).executeLocal(new ej.Query().where(this.model.appointmentSettings["id"], ej.FilterOperators.equal, parseInt(this._appId)))[0];
                    if (_newAddList[this.model.appointmentSettings["recurrenceRule"]].toString().indexOf("RECUREDITID") != -1)
                        this._deleteApp(parseInt(this._appId));
                    else {
                        var deleApp = this._deleteOcurrence(this._parentId, _newAddList[this.model.appointmentSettings["startTime"]]);
                        var promise = this._dataManager.update(this.model.appointmentSettings["id"], deleApp);
                        var proxy = this;
                        if (promise && $.isFunction(promise.promise) && this.dataSource() instanceof ej.DataManager) {
                            promise.done(function (e) {
                                new ej.DataManager(proxy._processed).remove(proxy.model.appointmentSettings["id"], parseInt(proxy._appId));
                                proxy.element.find("#Appointment_" + proxy._appId).remove();
                                proxy.element.find("#categorybar_" + proxy._appId).remove();
                                proxy._trigger("appointmentDeleted", { appointment: _newAddList });
                            });
                            promise.fail(function (e) {
                            });
                        }
                        else {
                            var dele = new ej.DataManager(this._processed).remove(this.model.appointmentSettings["id"], parseInt(this._appId));
                            this.element.find("#Appointment_" + this._appId).remove();
                            this.element.find("#categorybar_" + this._appId).remove();
                            this._trigger("appointmentDeleted", { appointment: _newAddList });
                        }
						this._getApps();
                        if (this._app.length < 1) {
                             if (this.model.enableAppointmentNavigation)
                                 this._nextPrevButton(this._app);
                            }
                    }
                }
                else {
                    this._currentAction = ej.Schedule.Actions.EditOcurrence;
                    this._showAppointmentDetails(this._appId, true);
                }
            }
            else {
                if (this._currentAction == ej.Schedule.Actions.Delete)
                    this._deleteApp(parseInt(this._parentId));
                else {
                    this._currentAction = ej.Schedule.Actions.EditSeries;
                    this._showAppointmentDetails(this._parentId);
                }
            }
        }
        this._recurEditWindow.ejDialog("close");
    },
    /**
    * Section For handle the recurrence select in the appointment window.
    * @private
    */
    _repeat: function (e) {
        if (e.isChecked || this._recurChecked) {
            if (this._currentAction == ej.Schedule.Actions.EditSeries || this._appointmentAddWindow.find(".e-recurFinalRule").html() != "") {
                this._appointmentAddWindow.find("#" + this._id + "summary").css("display", "table-row");
            }
            else {
                this._appointmentAddWindow.find(".e-appwindow").toggle();
                this._appointmentAddWindow.find(".e-recurwindow").css("margin-left", 700).toggle().animate({ "margin-left": 10 }, 400);
                this._recurClearFields();
                var strDate = new Date(this._appointmentAddWindow.find('.startenddate').ejDatePicker("option", "value"));
                this._appointmentAddWindow.find('.recurtype').ejDropDownList({ selectedItemIndex: 0 });
                this._appointmentAddWindow.find('.recurtype').ejDropDownList({ selectedItemIndex: 1 });
                this._appointmentAddWindow.find(".recurstartdate").ejDatePicker({ value: strDate });
                this._appointmentAddWindow.find('.e-until').ejDatePicker({ value: new Date(strDate.setDate(strDate.getDate() + 7 * 10)) });
                    this._appointmentAddWindow.find(".e-recurnoend").ejRadioButton({ checked: true });
                }
            }
            else
                (this._currentAction == ej.Schedule.Actions.EditSeries || this._appointmentAddWindow.find(".e-recurFinalRule").html() != "") && this._appointmentAddWindow.find("#" + this._id + "summary").css("display", "none");
            this._recurChecked = false;
        },
    /**
    * Section For handle the recurrence type change in the recurrence window.
    * @private
    */
    _recurtype: function (e) {
        this._rRuleFreq = "";
        var _rule = "";
        var strDate = new Date(this._appointmentAddWindow.find("#" + this._id + "startdate").ejDatePicker("option", "value"));
        if (e.itemId != null) {
            this._appointmentAddWindow.find("#" + this._id + "repeatEvery").css("display", "table-row");
            if (e.itemId == 0) {
                this._appointmentAddWindow.find("#" + this._id + "recurtypes").get(0).innerHTML = this._getLocalizedLabels("RecurrenceDay");
                this._appointmentAddWindow.find("#" + this._id + "weeklyrecur,#" + this._id + "monthlyrecur,#" + this._id + "yearlyrecur").css("display", "none");
                this._appointmentAddWindow.find("#" + this._id + "dailyrecur").css("display", "table-row");
                this._rRuleFreq = "DAILY";
                _rule = this._getLocalizedLabels("RecurrenceDay").toLowerCase();
            }
            else if (e.itemId == 1) {
                //_rule = this._weeklyRecurrnce();
                this._appointmentAddWindow.find("#" + this._id + "recurtypes").get(0).innerHTML = this._getLocalizedLabels("RecurrenceWeek");
                var curDay = this._dayNames[strDate.getDay()];
                this._appointmentAddWindow.find(".e-weekly" + curDay).ejCheckBox({ checked: true });
                _rule = this._getLocalizedLabels("RecurrenceWeek").toLowerCase() + " " + this._getLocalizedLabels("On").toLowerCase() + " " + this._dayFullNames[strDate.getDay()];
                this._rRuleFreq = "WEEKLY";
                this._appointmentAddWindow.find("#" + this._id + "monthlyrecur,#" + this._id + "yearlyrecur,#" + this._id + "dailyrecur").css("display", "none");
                this._appointmentAddWindow.find("#" + this._id + "weeklyrecur").css("display", "table-row");
            }
            else if (e.itemId == 2) {
                this._appointmentAddWindow.find("#" + this._id + "recurtypes").get(0).innerHTML = this._getLocalizedLabels("RecurrenceMonth");
                this._appointmentAddWindow.find("#" + this._id + "weeklyrecur,#" + this._id + "yearlyrecur,#" + this._id + "dailyrecur").css("display", "none");
                this._appointmentAddWindow.find("#" + this._id + "monthlyrecur").css("display", "table-row");
                this._appointmentAddWindow.find('.e-monthdate').ejNumericTextbox({ value: strDate.getDate() });
                this._appointmentAddWindow.find('.e-monthsrtday').ejDropDownList({ selectedItemIndex: strDate.getDay() });
                this._appointmentAddWindow.find('.monthsrt').ejDropDownList({ selectedItemIndex: this._getWeekIndex(strDate) });
                this._appointmentAddWindow.find('.e-monthdaytype').ejRadioButton({ checked: true });
                //var index = this._getWeekIndex(strDate); 
                this._rRuleFreq = "MONTHLY";
                _rule = this._getLocalizedLabels("RecurrenceMonth").toLowerCase();
            }
            else if (e.itemId == 3) {
                this._appointmentAddWindow.find("#" + this._id + "recurtypes").get(0).innerHTML = this._getLocalizedLabels("RecurrenceYear");
                this._appointmentAddWindow.find("#" + this._id + "weeklyrecur,#" + this._id + "monthlyrecur,#" + this._id + "dailyrecur").css("display", "none");
                this._appointmentAddWindow.find("#" + this._id + "yearlyrecur").css("display", "table-row");
                this._appointmentAddWindow.find('.yearmonth').ejDropDownList({ selectedItemIndex: strDate.getMonth() });
                this._appointmentAddWindow.find('.yeardate').ejNumericTextbox({ value: strDate.getDate() });
                this._appointmentAddWindow.find('.yearsrtday').ejDropDownList({ selectedItemIndex: strDate.getDay() });
                this._appointmentAddWindow.find('.yearsrtmonth').ejDropDownList({ selectedItemIndex: strDate.getMonth() });
                this._appointmentAddWindow.find('.yearsrt').ejDropDownList({ selectedItemIndex: this._getWeekIndex(strDate) });
                this._appointmentAddWindow.find('.e-yearrecurday').ejRadioButton({ checked: true });
                this._rRuleFreq = "YEARLY";
                _rule = this._getLocalizedLabels("RecurrenceYear").toLowerCase();
            }
            else if (e.itemId == 4) {
                this._rRuleFreq = "WEEKDAYS";
                this._appointmentAddWindow.find("#" + this._id + "weeklyrecur,#" + this._id + "yearlyrecur,#" + this._id + "dailyrecur,#" + this._id + "repeatEvery,#" + this._id + "monthlyrecur").css("display", "none");
                _rule = e.text;
            }
            this._appointmentAddWindow.find(".e-recurRule").html(_rule);
        }
    },
    /**
    * Section For generate the dates for all the view of the schedule.
    * @private
    */
    _getRenderDates: function () {
        this._dateRender = []; var firstDay, newDate;
        if (this.currentView() === "week" || this.currentView() === "workweek") {
            firstDay = this._firstweekdate();
            for (var i = 0; i < 7; i++) {
                newDate = new Date(firstDay);
                newDate.setDate(firstDay.getDate() + i);
                this._dateRender.push(new Date(newDate.getFullYear(), newDate.getMonth(), newDate.getDate()).getTime());
            }
            if (this.currentView() === "workweek")
                this._dateRender = this._dateRender.slice(1, 6);
        }
        else if (this.currentView() === "month") {
            this._dateRender = this._monthdates();
        }
        else {
            this._dateRender.push(this.currentDate());
        }
        return this._dateRender;
    },
    /**
    * Section For render the current date/date period in the header bar.
    * @private
    */
    _getCurrentDateHeaderValue: function () {
        var localizedDay = Globalize.culture(this.model.locale);
        var dateValue, value = this._dateRender;
        if (this.currentView() === "day")
            dateValue = localizedDay.calendar.days.names[new Date(value[0]).getDay()].slice(0, 3) + " " + new Date(value[0]).getDate() + " " + Globalize.format(new Date(value[0]), "MMM") + " " + new Date(value[0]).getFullYear(); //  Globalize.format(new Date(value[0]), "MMMM dd yyyy");
        else if (this.currentView() === "week" || this.currentView() === "workweek") {
            if (new Date(value[0]).getYear() === new Date(value[value.length - 1]).getYear()) {
                if (new Date(value[0]).getMonth() === new Date(value[value.length - 1]).getMonth()) {
                    dateValue = Globalize.format(new Date(value[0]), "dd") + " - " + Globalize.format(new Date(value[value.length - 1]), "dd MMM yyyy");
                }
                else if (new Date(value[0]).getMonth() !== new Date(value[value.length - 1]).getMonth()) {
                    dateValue = this.model.enableRTL ? Globalize.format(new Date(value[value.length - 1]), "MMM dd") + " - " + Globalize.format(new Date(value[0]), "dd MMM yyyy") :
                           Globalize.format(new Date(value[0]), "dd MMM") + " - " + Globalize.format(new Date(value[value.length - 1]), "dd MMM yyyy");
                }
            }
            else {
                dateValue = Globalize.format(new Date(value[0]), "MMM dd yyyy") + " - " + Globalize.format(new Date(value[value.length - 1]), "MMM dd yyyy");
            }
            //  dateValue = new Date(value[0]).getDate() + " " + Globalize.format(new Date(value[0]), "MMM") + (new Date(value[0]).getFullYear) + "/" + " - " + localizedDay.calendar.days.names[new Date(value[value.length - 1]).getDay()].slice(0, 3) + " " + (new Date(value[value.length - 1]).getMonth() + 1) + "/" + new Date(value[value.length - 1]).getDate();
        }
        else {
            dateValue = Globalize.format(new Date(this.currentDate()), "MMM yyyy");
        }
        return dateValue;
    },

    /**
    * Section For find the first day of the week.
    * @private
    */
    _firstweekdate: function () {
        var firstdayofweek = new Date(this.currentDate());
        var day = firstdayofweek.getDay();
        if (day != 0) {
            firstdayofweek.setDate(firstdayofweek.getDate() - day);
        }
        return firstdayofweek;
    },
    /**
    * Section For generate the allday cell counts of the schedule.
    * @private
        */
    _getHeaderAllDayCellsCount: function () {
            var weekcols = Globalize.culture(this.model.locale);
            var temp, count, cols, resCount = 1, curday; this.dateRender = [];
            cols = [];
            if (this.model.orientation == "vertical") {
                count = this._getColumnCount().length; // this.model.currentView === "month" ? dateRender.length / 5 : dateRender.length;
                if (!ej.isNullOrUndefined(this.model.resources) && !ej.isNullOrUndefined(this.model.group) && ej.scheduleFeatures.resources) {
                    var getCount = this.__getMultipleResourceCellsCount(count);
                    resCount = getCount.resCount; count = getCount.count;
                }
                for (var i = 0; i < resCount; i++) {
                    for (var j = 0; j < this._dateRender.length; j++)
                        this.dateRender.push(this._dateRender[j]);
                }
                for (var i = 0; i < count; i++) {
                    curday = (this.currentView() === "month" || this.currentView() === "day") ? (ej.isNullOrUndefined(this.model.resources) && ej.isNullOrUndefined(this.model.group)) ? weekcols.calendar.days.names[new Date(this.dateRender[i]).getDay()] : weekcols.calendar.days.names[new Date(this.dateRender[i]).getDay()].slice(0, 3) : weekcols.calendar.days.names[new Date(this.dateRender[i]).getDay()].slice(0, 3);
                    var headerCellClass = (Globalize.format(new Date(this.dateRender[i]), this._pattern.d) === Globalize.format(new Date(), this._pattern.d)) ? "e-headerToday" : "";
                    temp = this.currentView() === "month" ? { currentDay: curday, currentDateClass: "", cellToday: headerCellClass } : this.currentView() === "day" ? (ej.isNullOrUndefined(this.model.resources) && ej.isNullOrUndefined(this.model.group)) ? { currentDay: curday + " " + Globalize.format(new Date(this.dateRender[i]), "dd MMMM yyyy"), currentDateClass: "", cellToday: headerCellClass } : { currentDay: curday + " " + Globalize.format(new Date(this.dateRender[i]), "dd MMM yy"), currentDateClass: "", cellToday: headerCellClass } : { currentDay: curday + " " + new Date(this.dateRender[i]).getDate(), currentDateClass: new Date(this.dateRender[i]).getDate() === new Date(this.currentDate()).getDate() ? "e-activeview" : "", cellToday: headerCellClass };
                    cols.push(temp);
                }
            }
            else 
                cols = this._getHorizontalheaderCellCount(weekcols);
            return cols;
        },

        _findResourceIndex: function (data, attr, value) {
            for (var i = 0; i < data.length; i++) {
                if (data[i][attr] === value) {
                    return i;
                }
            }
        },

        /**
        * Section For render all day header template.
        * @private
        */
        _renderHeaderAllDayTemplate: function () {
            if (ej.scheduleFeatures.horizontal && this.model.orientation == "horizontal") {
                if (!ej.isNullOrUndefined(this.model.resources) && ej.scheduleFeatures.resources)
                    this._renderMultipleResourceHeaderTemplate(null, null);
                this._renderHorizontalHeaderBar();
            }
            else {
                var headerDiv = ej.buildTag('div', "", {}, { height: "100%" }).css("position", "relative").css("overflow", "hidden");
                var resCount = 0;
                var table = ej.buildTag('table.e-table e-headrealldaytable', "", {}, { cellspacing: "0px", cellpadding: "0px" });
                var tbody = ej.buildTag('tbody');
                var headTR = ej.buildTag('tr.e-dateheader');
                var cols = this._getHeaderAllDayCellsCount();
                var html = this.headTemplate.render({ cols: cols, view: this.currentView(), colspan: 1 });
                //multiple resources
                if (!ej.isNullOrUndefined(this.model.resources) && ej.scheduleFeatures.resources) {
                    tbody = this._renderMultipleResourceHeaderTemplate(tbody, cols);
                    resCount = !ej.isNullOrUndefined(this.model.group) ? this.model.group.resources.length : 0;
                }
                var inner_table = ej.buildTag('table.e-table', "", {}, { cellspacing: "0px", cellpadding: "0px" });
                var tb = ej.buildTag('tbody');
                var headT = ej.buildTag('tr.e-headertr');

                headTrCount = (this.currentView() == "month") ? 1 : 2;
                var rowCount = resCount + headTrCount; var rows = [];
                for (var i = 0; i < rowCount; i++) {
                    if (this.currentView() == "month")
                        rows.push({ tdClass: (i == rowCount - 1) ? "e-monthleftcells" : "e-resourceheadercells", divClass: (i == rowCount - 1) ? "e-dateheadercell" : "" });
                    else
                        rows.push({ tdClass: (i == rowCount - 1) ? "e-leftindent" : (i == rowCount - 2) ? "e-settingsindent" : "e-resourceheadercells", divClass: (i == rowCount - 1) ? "e-schedulesettings e-textellipsis" : (i == rowCount - 2) ? "e-dateheadercell" : "" });
                }
                var widths = this.currentView() == "month" ? "50px" : "56px";
                var leftIndentHtml = this.leftIndent.render({ trs: rows, cswidth: widths });

                this.aTR.append(ej.buildTag('td').append(inner_table.append(tb.append(headT.append(ej.buildTag('td.empty', "", {}, { width: this.currentView() == "month" ? "50px" : "56px" }).css("border-bottom", (this.currentView() != "month") ? "1px solid #bbbcbb" : "").append(leftIndentHtml)).append(ej.buildTag('td').append(headerDiv.append(table.append(tbody.append(headTR.append(html))))))))));
                if (this.currentView() !== "month") {
                    var alldayTR = ej.buildTag('tr');
                    var allHtml = this.alldayTemp.render({ cols: cols, colspan: 1 });
                    tbody.append(alldayTR.append(allHtml));
                }
            }
        },
    /**
    * Section For generate the current month dates with previous and next month exceeding dates.
    * @private
    */
    _monthdates: function () {
        var currentDate = new Date(this.currentDate());
        var header = new Date(currentDate.getFullYear(), currentDate.getMonth(), 1);
        var day = header.getDay() + 1;
        header.setDate(header.getDate() - day);
        //nextmonth days.
        var daysInMonth = new Date(currentDate.getFullYear(), currentDate.getMonth() + 1, 0).getDate();
            var daysInNextMonth = (day + daysInMonth) % 7 != 0 ? (7 - (day + daysInMonth) % 7) : 0;
            var rows = parseInt((day + daysInMonth + daysInNextMonth) / 7);
            var dates = [];
            for (var row = 0; row < (rows * 7); row++) {
                dates.push(new Date(header.setDate(header.getDate() + 1)));
            }
            return dates;
    },
    /**
    * Section For get the start and end time of the schedule.
    * @private
    */
    _getStartEndTime: function () {
        var timerows, startendTimes = [], timelist;
        var localizeAmPm = Globalize.culture(this.model.locale).calendar;
        if (this.model.orientation === "vertical") {
            if (this.currentView() !== "month") {
                for (var i = this.model.startHour; i < this.model.endHour; i++) {
                    if (this.model.timeMode === "12") {
                        timelist = ((i === this.model.startHour || i === 12) && (localizeAmPm.AM && localizeAmPm.PM)) ? Globalize.format(new Date(new Date().setHours(i)), "hh tt") : Globalize.format(new Date(new Date().setHours(i)), "hh 00");
                        startendTimes.push({ time: timelist, id: i });
                    }
                    else
                        startendTimes.push({ time: (i <= 9 ? "0" + i : i) + " 00" });
                }
            }
            else {
                var row = this._dateRender.length;
                for (var i = 0; i < (row / 7) ; i++) {
                    startendTimes.push(i);
                }
            }
        }
        else if (this.model.orientation == "horizontal" && ej.scheduleFeatures.horizontal)
            startendTimes = this._getHoriStartEndTime(localizeAmPm);
        return startendTimes;
    },
    /**
    * Section For get the work cells count of the schedule.
    * @private
    */
    _getWorkCellsCount: function () {
            var workcells = [];
            if (this.currentView() !== "month") {
                var strEndTime = this._getStartEndTime();
                for (var i = 0; i < (2 * strEndTime.length); i++) {
                    workcells.push({ crow: i, cwidth: this.model.cellWidth, cheight: this.model.cellHeight });
                }
            }
            else {
                var row = this._dateRender.length;
                for (var i = 0; i < (row / 7); i++) {
                    workcells.push({ crow: i, cwidth: this.model.cellWidth, cheight: this.model.cellHeight });
                }
            }
        return workcells;
    },
    /**
    * Section For render the content area template of the schedule.
    * @private
    */
    _renderContentAreaTemplate: function () {
        var $contentAreaDiv = ej.buildTag('div.e-workareadiv');
        var $contentTable = ej.buildTag('table.e-table e-worktable', "", {}, { height: "100%", cellpadding: "0px", cellspacing: "0px", width: "100%" });
        var $contentBody = ej.buildTag('tbody');
        var $contentTR = ej.buildTag('tr');
        var $WorkCellTD = ej.buildTag('td');
        var $scrollerbarDiv = ej.buildTag('div#' + this._id + "_scroller");
        this.$WorkCellDiv = ej.buildTag('div').css('position', "relative").attr('class', 'e-draggableworkarea').css("height", "100%");
        
        if (ej.scheduleFeatures.horizontal && this.model.orientation == "horizontal")
            this._renderHorizontalContent($contentAreaDiv, $contentTable, $contentBody, $contentTR, $WorkCellTD, $scrollerbarDiv);
        else {
      //      this.$WorkCellDiv.css("height", "100%");
            var timeCellClassName = this.currentView() === "month" ? "e-monthleftindent" : "e-workleftindent";
            this.$TimeCellTD = ej.buildTag('td', "", {}, {}).addClass(timeCellClassName);
            
            var colspan = [], index;
            var cols = this.currentView() === "day" ? 1 : this.currentView() === "week" || this.currentView() === "month" ? 7 : 5;
            if (!ej.isNullOrUndefined(this.model.resources) && !ej.isNullOrUndefined(this.model.group) && ej.scheduleFeatures.resources)
                cols = this._getMultipleResourceColsCount(cols);
            for (var i = 0; i < cols; i++) colspan.push(i);

            //time cells
            var timeCellhtml;
            var viewclass = this.currentView() == "month" ? 1 : 0; //"e-monthtimecells" : "e-timecelldivs";
            timeCellhtml = this.timeCellTemplate.render({ timerows: this._getStartEndTime(), cols1: [1, 2], viewclass: viewclass });

            //Work cells
            var workCellhtml;
            if (this.currentView() !== "month")
                workCellhtml = this.workcellTemp.render({ cellrows: this._getWorkCellsCount(), cols1: colspan, orientation: this.model.orientation });
            else {
                workCellhtml = this.monthcellTemp.render({ cellrows: this._getWorkCellsCount(), cols1: colspan, orientation: this.model.orientation });
            }
            this.wTR.append(ej.buildTag('td').append($contentAreaDiv.append($contentTable.append($contentBody.append($contentTR.append(this.$TimeCellTD.append(timeCellhtml)).append($WorkCellTD.append($scrollerbarDiv.append(this.$WorkCellDiv.append(workCellhtml)))))))));
        }
        
    },
    /**
    * Section For get the column count of the schedule.
    * @private
    */
    _getColumnCount: function () {
        var weekcells = [];
        var daysCount = this.currentView() === "week" || this.currentView() === "month" ? 7 : this.currentView() === "workweek" ? 5 : 1;
        for (var i = 0; i < daysCount; i++) {
            weekcells.push(i);
        }
            return weekcells;
        },
        
        /**
        * To configure the dataSource values
        * @private
        */
    _bindAppointmentsData: function () {
        if (!ej.isNullOrUndefined(this.model.appointmentSettings) && this.dataSource() != null) {
            if (this.dataSource() instanceof ej.DataManager && !this.dataSource().dataSource.offline) {
                this._initDataSource(); //Code to bind fields from database
            }
            else {
                this._currentAppointmentData = this.dataSource();
                !ej.isNullOrUndefined(this.dataSource().dataSource) && this.dataSource().dataSource.json && (this._currentAppointmentData = this.dataSource().dataSource.json);
                this._dataCount = this._currentAppointmentData.length;
                this._dataProcessing(this._currentAppointmentData);
                this._renderAppointmentAll();
            }
            }
        },
        /**
        * To configure the dataSource values to the fields.
        * @private
        */
    _initDataSource: function () {
        var query = this._columnToSelect(this.model.appointmentSettings);
        var proxy = this;
        var queryPromise = this.dataSource().executeQuery(query);
        queryPromise.done(function (e) {
            //utc timezone conversion
            var tempstarttime, tempendtime, localutc;
            localutc = new Date().getTimezoneOffset();
            var obj = proxy._minToHour(localutc);
            for (var a = 0; a < e.result.length; a++) {
                tempstarttime = e.result[a].StartTime;
                tempendtime = e.result[a].EndTime;
                tempstarttime.setHours(tempstarttime.getHours() - obj.h, tempstarttime.getMinutes() - obj.m);
                tempendtime.setHours(tempendtime.getHours() - obj.h, tempendtime.getMinutes() - obj.m);
            }
            proxy._currentAppointmentData = e.result;
            //proxy._dataManager = ej.DataManager(e.result);
            proxy._dataCount = e.result.length;
            proxy._overlapList = [];
            proxy._dataProcessing(e.result);
            proxy._renderAppointmentAll();
        });
    },
    /**
    * To configure the column values of the table
    * @private
    */
    _columnToSelect: function (mapper) {
        var column = [], queryManager = ej.Query();
        if (ej.isNullOrUndefined(mapper.query)) {
            column = this._getColumns(mapper);
            if (column.length > 0)
                queryManager.select(column);
            if (!ej.isNullOrUndefined(this.dataSource())) {
                if (!this.dataSource().dataSource.url.match(mapper.tableName + "$"))
                    !ej.isNullOrUndefined(mapper.tableName) && queryManager.from(mapper.tableName);
            }
        }
        else
            queryManager = mapper.query;
        return queryManager;
    },
    /**
    * returns the columns name in array list.
    * @private
    */
    _getColumns: function (mapper) {
        var column = []
        for (var col in mapper) {
            if (col !== "tableName" && col !== "query" && col !== "dataSource")
                column.push(mapper[col]);
        }
        return column;
    },
    /**
    * Section For sort the appointments based on the appointment id.
    * @private
    */
    _sortAppById: function (appointments) {

        var proxy = this;
        appointments.sort(function (a, b) {
            var x = a[proxy.model.appointmentSettings["id"]]; var y = b[proxy.model.appointmentSettings["id"]];
            return ((x < y) ? -1 : ((x > y) ? 1 : 0));
        });
        return appointments;
    },
    /**
    * Section For handle the datasource dataprocessing.
    * @private
    */
    _dataProcessing: function (appointments) {
        this._processed = [];
        if(appointments.length > 0){
            var appId = this._sortAppById(appointments);
            this._maxId = appId[appId.length - 1][this.model.appointmentSettings["id"]] + 1;
            var recurEndDate;
            for (var app = 0; app < appointments.length; app++) {
                this._appointmentProcessing(appointments[app]);
            }
            //this._maxId = null;
            this._processed = this._sortAppById(this._processed);
            return this._processed;
        }
    },
    /**
    * Section For processing the appointments.
    * @private
    */
    _appointmentProcessing: function (appointment) {
        var day_start=this._formatTime(appointment[this.model.appointmentSettings["startTime"]]);
        this._rRule = {}, _count = 0, _lastDay = false;
        (!ej.isNullOrUndefined(appointment[this.model.appointmentSettings["recurrenceRule"]])) && this.recurrenceRuleSplit(appointment[this.model.appointmentSettings["recurrenceRule"]]);
        if (appointment[this.model.appointmentSettings["recurrence"]] == 1 && ej.isNullOrUndefined(this._rRule.recurEditId)) {
            var byDay = "", _interval = ej.isNullOrUndefined(this._rRule.interval) ? 1 : this._rRule.interval;
            !ej.isNullOrUndefined(this._rRule.weekDays) && (byDay = this._rRule.weekDays.split(','));
            // Find Recurrence End date:
            if (!ej.isNullOrUndefined(this._rRule.until)) {
                recurEndDate = new Date(this._rRule.until);
                recurEndDate.setDate(recurEndDate.getDate() + 1);
                }
                else if (!ej.isNullOrUndefined(this._rRule.count))
                    switch (this._rRule.freq) {
                    case "DAILY":
                        recurEndDate = new Date(new Date(day_start).setDate(day_start.getDate() + this._rRule.count * (_interval)));
                        break;
                    case "WEEKLY":
                        var dayCount = day_start.getDate() + (((this._rRule.count / byDay.length) * 7)) * (_interval);
                        recurEndDate = new Date(new Date(day_start).setDate(Math.round(dayCount) < dayCount ? dayCount + 1 : dayCount));
                        break;
                    case "MONTHLY":
                        recurEndDate = new Date(new Date(day_start).setMonth(day_start.getMonth() + this._rRule.count * (_interval)));
                        break;
                    case "YEARLY":
                        recurEndDate = new Date(new Date(day_start).setFullYear(day_start.getFullYear() + this._rRule.count * (_interval)));
                        break;
                }
                else {
                    recurEndDate = new Date(new Date(this.currentDate()).setDate(day_start.getDate() + 42 * (_interval)));
                }
            switch (this._rRule.freq) {
                case "WEEKLY":
                    byDay = this._rRule.weekDays.split(',');
                    if (byDay.indexOf(this._dayNames[new Date(day_start).getDay()]) == -1)
                        day_start = this._weeklyDayFind(day_start, byDay);
                    break;
                case "MONTHLY":
                    if (!ej.isNullOrUndefined(this._rRule.monthDay)) {
                        _lastDay = new Date(day_start).getDate() == new Date(day_start.getFullYear(), day_start.getMonth() + 1, 0).getDate() ? true : false;
                        while (new Date(day_start).getDate() != this._rRule.monthDay) {
                            day_start = new Date(day_start.setDate(day_start.getDate() + 1));
                        }
                    }
                    else
                        day_start = new Date((this._getWeekIndex(day_start) + 1) > this._rRule.setPositions && this._rRule.setPositions != -1 ? new Date(day_start.getFullYear(), day_start.getMonth() + 1, 1) : day_start);
                    break;
                case "YEARLY":
                    if (!ej.isNullOrUndefined(this._rRule.monthDay)) {
                        day_start = (day_start.getMonth() + 1 >= this._rRule.month) && day_start.getDate() > this._rRule.monthDay ? new Date(day_start.getFullYear() + 1, this._rRule.month - 1, this._rRule.monthDay) : new Date(day_start.getFullYear(), this._rRule.month - 1, this._rRule.monthDay);
                    }
                    else if (!ej.isNullOrUndefined(this._rRule.setPositions)) {
                        day_start = new Date(day_start.getFullYear(), this._rRule.month - 1, day_start.getDate());
                        day_start = new Date((this._getWeekIndex(day_start) + 1) > this._rRule.setPositions ? new Date(day_start.getFullYear(), day_start.getMonth() + 1, 1) : day_start);
                    }
                    break;
            }

            while (day_start <= recurEndDate) {
                var orgStrTime = this._formatTime(appointment[this.model.appointmentSettings["startTime"]]);
                var newAppointment = {}, orgEndTime = this._formatTime(appointment[this.model.appointmentSettings["endTime"]]);
                ((this._rRule.freq == "MONTHLY" || this._rRule.freq == "YEARLY") && !ej.isNullOrUndefined(this._rRule.setPositions)) && (day_start = this._dayOfWeekInMonth(day_start, this._rRule.weekDays, this._rRule.setPositions));
                var strTime = new Date(day_start.getFullYear(), day_start.getMonth(), day_start.getDate(), orgStrTime.getHours(), orgStrTime.getMinutes(), 0);
                var endTime = new Date(day_start.getFullYear(), day_start.getMonth(), day_start.getDate(), orgEndTime.getHours(), orgEndTime.getMinutes(), 0);
                if (!ej.isNullOrUndefined(this._rRule.exDate)) {
                    if (this._rRule.exDate.indexOf(Globalize.format(new Date(day_start), this._pattern.d)) == -1) {
                        newAppointment = this._addAppData(appointment, this._maxId, strTime, endTime, appointment[this.model.appointmentSettings["id"]]);
                    }
                }
                else
                    newAppointment = this._addAppData(appointment, this._maxId, strTime, endTime, appointment[this.model.appointmentSettings["id"]]);
                switch (this._rRule.freq) {
                    case "DAILY":
                        day_start = new Date(day_start.setDate(day_start.getDate() + _interval));
                        break;
                    case "WEEKLY":
                        var curDay = this._dayNames[new Date(day_start).getDay()];
                        if (byDay.length - 1 != byDay.indexOf(curDay)) {
                            day_start = this._weeklyDayFind(day_start, byDay);
                        }
                        else {
                            day_start = new Date(day_start.setDate(day_start.getDate() + (day_start.getDay() * -1)));
                            day_start = new Date(day_start.setDate(day_start.getDate() + (_interval * 7) - 1));
                            day_start = this._weeklyDayFind(day_start, byDay);
                        }
                        break;
                    case "MONTHLY":
                        day_start = _lastDay ? new Date(day_start.getFullYear(), (day_start.getMonth() + _interval) + 1, 0) :
                            ej.isNullOrUndefined(this._rRule.setPositions) ? new Date(day_start.setMonth(day_start.getMonth() + _interval)) : new Date(day_start.getFullYear(), day_start.getMonth() + _interval, 1);
                        break;
                    case "YEARLY":
                        day_start = new Date(day_start.setFullYear(day_start.getFullYear() + _interval));
                        break;
                }
                if ((Globalize.format(this._formatTime(appointment[this.model.appointmentSettings["startTime"]]), this._pattern.d) < Globalize.format(this._formatTime(appointment[this.model.appointmentSettings["endTime"]]), this._pattern.d)) && (((this._formatTime(appointment[this.model.appointmentSettings["endTime"]]) - this._formatTime(appointment[this.model.appointmentSettings["startTime"]])) / 3600000) < 24) && (appointment[this.model.appointmentSettings["allDay"]] !== true)) {
                    endTime.setDate(endTime.getDate() + 1);

                    this._multipleDayApp(newAppointment, this._maxId, this._formatTime(newAppointment[this.model.appointmentSettings["startTime"]]), endTime);

                }
                else
                    (!$.isEmptyObject(newAppointment)) && this._processed.push(newAppointment);
                this._maxId++;
                _count++;
                if (!ej.isNullOrUndefined(this._rRule.count) && _count == this._rRule.count)
                    break;
            }
        }

        if ((Globalize.format(this._formatTime(appointment[this.model.appointmentSettings["startTime"]]), this._pattern.d) < Globalize.format(this._formatTime(appointment[this.model.appointmentSettings["endTime"]]), this._pattern.d)) && (((this._formatTime(appointment[this.model.appointmentSettings["endTime"]]) - this._formatTime(appointment[this.model.appointmentSettings["startTime"]])) / 3600000) < 24) && (appointment[this.model.appointmentSettings["allDay"]] !== true) && appointment[this.model.appointmentSettings["recurrence"]] != 1) {
            this._multipleDayApp(appointment, this._maxId, this._formatTime(appointment[this.model.appointmentSettings["startTime"]]), this._formatTime(appointment[this.model.appointmentSettings["endTime"]]));
        }
        else if (((this._formatTime(appointment[this.model.appointmentSettings["endTime"]]) - this._formatTime(appointment[this.model.appointmentSettings["startTime"]])) / 3600000) > 24) {
            if ((appointment[this.model.appointmentSettings["allDay"]] === true) || ((appointment[this.model.appointmentSettings["allDay"]] !== true) && ((this._formatTime(appointment[this.model.appointmentSettings["endTime"]]) - this._formatTime(appointment[this.model.appointmentSettings["startTime"]])) / 3600000) > 24)) {
                if (this.currentView() == "day") {
                    this._multipleDaySplit(appointment, this._maxId, this._formatTime(appointment[this.model.appointmentSettings["startTime"]]), this._formatTime(appointment[this.model.appointmentSettings["endTime"]]));
                }
                else if (this.currentView() === "workweek") {
                    this._multipleWorkWeekSplit(appointment, this._maxId, this._formatTime(appointment[this.model.appointmentSettings["startTime"]]), this._formatTime(appointment[this.model.appointmentSettings["endTime"]]));
                }
                else {
                    this._multipleWeekSplit(appointment, this._maxId, this._formatTime(appointment[this.model.appointmentSettings["startTime"]]), this._formatTime(appointment[this.model.appointmentSettings["endTime"]]));
                }
            }
        }
        else if ((appointment[this.model.appointmentSettings["recurrence"]] == 0 || !ej.isNullOrUndefined(this._rRule.recurEditId))) {
            ej.isNullOrUndefined(appointment.ParentId) && (appointment.ParentId = appointment[this.model.appointmentSettings["id"]]);
            this._processed.push(appointment);
        }
        else if ((appointment[this.model.appointmentSettings["recurrence"]] == undefined) || (appointment[this.model.appointmentSettings["allDay"]] == undefined)) {
            this._processed.push(appointment);
        }
    },
    /**
    * Section For converting time while data from sql data source.
    * @private
    */
    //Convert time format.
    _formatTime:function(appointment)
    {
    if(new Date(appointment).toString()=="Invalid Date")
        time = new Date(parseInt(appointment.match(/\d+/).toString()));
    else
        time=new Date(appointment);
    return time;
    },
    /**
    * Section For split the multiple day appointment.
    * @private
    */
    //Split two appointments for multiple day.
    _multipleDayApp: function (app, maxId, startTime, endTime) {
        var day_start = new Date(startTime);
        var nextDay = new Date(new Date(startTime).getFullYear(), new Date(startTime).getMonth(), new Date(startTime).getDate());
        var day_end = new Date(new Date(nextDay).setHours(new Date(nextDay).getHours() + 24));
        day_end = new Date(new Date(day_end).setMinutes(new Date(day_end).getMinutes() - 1));

        while (day_start >= new Date(startTime) && day_start <= day_end && day_end <= new Date(endTime)) {
            appointment = this._addAppData(app, maxId, day_start, day_end, app[this.model.appointmentSettings["id"]]);
            this._processed.push(appointment);
            day_start = new Date(new Date(day_end).setMinutes(new Date(day_end).getMinutes() + 1));
            day_end = this._formatTime(app[this.model.appointmentSettings["endTime"]]);
            maxId++;
            this._maxId = maxId;
        }
    },
    _multipleDaySplit: function (app, maxId, startTime, endTime) {
        var day_start = new Date(startTime);
        var nextDay = new Date(new Date(startTime).getFullYear(), new Date(startTime).getMonth(), new Date(startTime).getDate());
        var day_end = new Date(new Date(nextDay).setHours(new Date(endTime).getHours()));
        day_end = new Date(new Date(day_end).setMinutes(new Date(endTime.getMinutes())));
        while (day_start >= new Date(startTime) && day_start <= day_end && day_end <= new Date(endTime)) {
            appointment = this._addAppData(app, maxId, day_start, day_end, app[this.model.appointmentSettings["id"]]);
            this._processed.push(appointment);
            day_start = new Date(new Date(day_start).setHours(new Date(startTime).getHours() + 24));
            day_start = new Date(new Date(day_start).setMinutes(new Date(startTime).getMinutes()));
            day_end = new Date(new Date(day_start).setHours(new Date(endTime).getHours()));
            day_end = new Date(new Date(day_end).setMinutes(new Date(endTime).getMinutes()));
            maxId++;
            this._maxId = maxId;
        }
    },

    _multipleWorkWeekSplit: function (app, maxId, startTime, endTime) {
        var sunday = new Date((new Date(startTime)).setDate(startTime.getDate() + 1));
        var saturday = new Date((new Date(startTime)).setDate(startTime.getDate() + 2));
        var day_start = (new Date(startTime).getDay() == 0) ? sunday : (new Date(startTime).getDay() == 6) ? saturday : new Date(startTime);
        var x = 6 - day_start.getDay();
        var nextDay = new Date(new Date(startTime).getFullYear(), new Date(startTime).getMonth(), new Date(startTime).getDate() + x);
        var day_end = new Date(new Date(nextDay).setHours(new Date(endTime).getHours()));
        day_end = new Date(new Date(day_end).setMinutes(new Date(endTime.getMinutes())));
        if (day_end > new Date(endTime)) {
            day_end = this._formatTime(app[this.model.appointmentSettings["endTime"]]);
        }
        else if ((day_end.getDay() == 6) || (day_end.getDay() == 7)) {
            var sunday = new Date((new Date(day_end)).setDate(day_end.getDate() - 2));
            var saturday = new Date((new Date(day_end)).setDate(day_end.getDate() - 1));
            day_end = (new Date(day_end).getDay() == 6) ? saturday : sunday;
        }
        else { day_end = day_end; }

        while (day_start >= new Date(startTime) && day_start <= day_end && day_end <= new Date(endTime)) {
            appointment = this._addAppData(app, maxId, day_start, day_end, app[this.model.appointmentSettings["id"]]);
            this._processed.push(appointment);
            var next1 = new Date(new Date(day_end).getFullYear(), new Date(day_end).getMonth(), new Date(day_end).getDate() + 3);
            day_start = new Date(new Date(next1).setHours(new Date(startTime).getHours()));
            day_start = new Date(new Date(day_start).setMinutes(new Date(startTime).getMinutes()));
            var q = 5 - day_start.getDay();
            var next = new Date(new Date(day_start).getFullYear(), new Date(day_start).getMonth(), new Date(day_start).getDate() + q);
            day_end = new Date(new Date(next).setHours(new Date(endTime).getHours()));
            day_end = new Date(new Date(day_end).setMinutes(new Date(endTime).getMinutes()));
            if (day_end > new Date(endTime)) {
                day_end = this._formatTime(app[this.model.appointmentSettings["endTime"]]);
            } else { day_end = day_end; }
            maxId++;
            this._maxId = maxId;
        }
    },

    _multipleWeekSplit: function (app, maxId, startTime, endTime) {
        var day_start = new Date(startTime);
        var x = 6 - day_start.getDay();
        var nextDay = new Date(new Date(startTime).getFullYear(), new Date(startTime).getMonth(), new Date(startTime).getDate() + x);
        var day_end = new Date(new Date(nextDay).setHours(new Date(endTime).getHours()));
        day_end = new Date(new Date(day_end).setMinutes(new Date(endTime.getMinutes())));
        if (day_end > new Date(endTime)) {
            day_end = this._formatTime(app[this.model.appointmentSettings["endTime"]]);
        } else {
            day_end = day_end;
        }

        while (day_start >= new Date(startTime) && day_start <= day_end && day_end <= new Date(endTime)) {

            appointment = this._addAppData(app, maxId, day_start, day_end, app[this.model.appointmentSettings["id"]]);
            this._processed.push(appointment);
            var next1 = new Date(new Date(day_end).getFullYear(), new Date(day_end).getMonth(), new Date(day_end).getDate() + 1);
            day_start = new Date(new Date(next1).setHours(new Date(startTime).getHours()));
            day_start = new Date(new Date(day_start).setMinutes(new Date(startTime).getMinutes()));
            var q = 6 - day_start.getDay();
            var next = new Date(new Date(day_start).getFullYear(), new Date(day_start).getMonth(), new Date(day_start).getDate() + q);
            day_end = new Date(new Date(next).setHours(new Date(endTime).getHours()));
            day_end = new Date(new Date(day_end).setMinutes(new Date(endTime).getMinutes()));
            if (day_end > new Date(endTime)) {
                day_end = this._formatTime(app[this.model.appointmentSettings["endTime"]]);
            } else {
                day_end = day_end;
            }
            maxId++;
            this._maxId = maxId;
        }
    },

    /**
    * Section For handle the recurrence appointment delete.
    * @private
    */
    //Delete ocurrence 
    _deleteOcurrence: function (appId, date) {
        var query = new ej.Query().where(this.model.appointmentSettings["id"], ej.FilterOperators.equal, appId);
        var _editApp = new ej.DataManager(this._currentAppointmentData).executeLocal(query);
        if (!ej.isNullOrUndefined(_editApp) && !ej.isNullOrUndefined(_editApp[0][this.model.appointmentSettings["recurrenceRule"]])) {
            var _rRule = _editApp[0][this.model.appointmentSettings["recurrenceRule"]];
            this.recurrenceRuleSplit(_rRule);
            if (ej.isNullOrUndefined(this._rRule.exDate)) {
                _editApp[0][this.model.appointmentSettings["recurrenceRule"]] = _rRule + ";EXDATE=" + Globalize.format(new Date(date), this._pattern.d);
            }
            else {
                var _rules = _rRule.split(';');
                recurRule = "";
                for (var i = 0; i < _rules.length; i++) {
                    if (recurRule != "")
                        recurRule += ";";
                    var keyValuePair = _rules[i].replace(/ /g, '').split('=');
                    if (keyValuePair[0] == "EXDATE") {
                        _rules[i] = _rules[i] + "," + Globalize.format(new Date(date), this._pattern.d);
                    }
                    recurRule += _rules[i];
                }
                _editApp[0][this.model.appointmentSettings["recurrenceRule"]] = recurRule;
            }
            return _editApp;
        }
    },
    /**
    * Section For get the week index of the month.
    * @private
    */
    //Find week index in a month.
    _getWeekIndex: function (curDate) {
        var index = 0;
        var firstDate = new Date(curDate.getFullYear(), curDate.getMonth(), 1);
        var dayIndex = curDate.getDay();
        if (dayIndex != firstDate.getDay())
            do {
                firstDate = new Date(firstDate.setDate(firstDate.getDate() + 1));
            } while (dayIndex != firstDate.getDay())
        while (firstDate.getDate() != curDate.getDate()) {
            firstDate = new Date(firstDate.setDate(firstDate.getDate() + 7));
            index++;
        }
        return index;
    },
    /**
    * create the appointment data with JSON object.
    * @private
    */
    //create the appointment data with JSON object.
    _addAppData: function (app, id, startTime, endTime, parentId) {
        var data = {};
        for (var item in this.model.appointmentSettings) {
            if (item != "dataSource" && item != "query" && item != "tableName" && item != "resourceFields")
                data[this.model.appointmentSettings[item]] = app[this.model.appointmentSettings[item]];
                if(item == "resourceFields"){
                    for(var val in !ej.isNullOrUndefined(this.model.appointmentSettings.resourceFields) && this.model.appointmentSettings.resourceFields.split(','))
                    data[this.model.appointmentSettings.resourceFields.split(',')[val]] = app[this.model.appointmentSettings.resourceFields.split(',')[val]];
                }
        }
        (!ej.isNullOrUndefined(id)) && (data[this.model.appointmentSettings["id"]] = id);
        (!ej.isNullOrUndefined(startTime)) && (data[this.model.appointmentSettings["startTime"]] = startTime);
        (!ej.isNullOrUndefined(endTime)) && (data[this.model.appointmentSettings["endTime"]] = endTime);
        (!ej.isNullOrUndefined(parentId)) && (data["ParentId"] = parentId);
        return data;
    },
    /**
    * Section for find the number of week and the day name.
    * @private
    */
    // find the number week with day name.
    _dayOfWeekInMonth: function (strDate, dayName, weekIndex) {
        var dayIndex = this._dayNames.indexOf(dayName);
        var curDate = weekIndex == -1 ? new Date(strDate.getFullYear(), strDate.getMonth() + 1, 0) : new Date(strDate.getFullYear(), strDate.getMonth(), 1);
        while (curDate.getDay() != dayIndex) {
            curDate = new Date(weekIndex == -1 ? curDate.setDate(curDate.getDate() - 1) : curDate.setDate(curDate.getDate() + 1));
        }
        return new Date(weekIndex == 1 || weekIndex == -1 ? curDate : curDate.setDate(curDate.getDate() + ((weekIndex - 1) * 7)));
    },
    /**
    * Section for find the day name.
    * @private
    */
    // Find the day name 
    _weeklyDayFind: function (strDate, days) {
        var newdayStr = strDate;
        do {
            newdayStr = new Date(newdayStr.setDate(newdayStr.getDate() + 1));
            curDay = this._dayNames[new Date(newdayStr).getDay()];
        } while (days.indexOf(curDay) == -1);
        return newdayStr;
    },
    /**
    * Section for render the appointments.
    * @private
    */
    _renderAppointments: function () {
        var dateCount = this._dateRender;
        this._tempapp = [];
        this.cellwidth = this.element.find(".e-workcells").width();
        var appointments = this._currentAppointmentData;
        this.leftPosition = 0;
        if (this.model.timeZone != null) {
            var timezone = this.model.timeZone.split(" ")[1].split(":");
            var hour = parseInt(timezone[0]);
            var mins = parseInt(timezone[1]);
            dateMinutes = new Date().getTimezoneOffset();
            var obj = this._minToHour(dateMinutes + ((hour * 60) + mins));
            for (var a = 0; a < appointments.length; a++) {
                temp = $.extend({}, appointments[a]);
                sHour = new Date(temp.StartTime);
                eHour = new Date(temp.EndTime);
                temp.StartTime = new Date(sHour.setHours(sHour.getHours() + obj.h, sHour.getMinutes() + obj.m));
                temp.EndTime = new Date(eHour.setHours(eHour.getHours() + obj.h, eHour.getMinutes() + obj.m));
                this._tempapp.push(temp);
            }
            this._dataProcessing(this._tempapp);
        }
            //else
            //    this._dataProcessing(this.dataSource());
            var appointmentsList = this._processed;
            this._filtered = [];
            if (!this.model.group) this.res1 = ["0"];
            this.resIndex = 0; //this._categorybar = true;
            for (var res = 0; res < this.res1.length; res++) {
                //if (ej.scheduleFeatures.horizontal && this.model.orientation == "horizontal" && ej.scheduleFeatures.resources && !ej.isNullOrUndefined(this.model.group) && !ej.isNullOrUndefined(this.model.resources) && this.model.group.resources.length > 1) {
                //    if(this.resIndex != this.res1[res][this.model.resources[this.model.resources.length-1].resourceSettings.id])
                //        this._categorybar = true;
                //    this.resIndex = this.res1[res][this.model.resources[this.model.resources.length - 1].resourceSettings.id]
                //}
                for (var day = 0; day < dateCount.length; day++) {
                    this._getDayAppointments(day, res);
                    //this._appList = [];
                    //for (var app = 0; app < appointmentsList.length; app++) {
                    //    if (!ej.isNullOrUndefined(this.model.group)) {
                    //        if ((Globalize.format(new Date(appointmentsList[app][this.model.appointmentSettings["startTime"]]), this._pattern.d) <= Globalize.format(new Date(dateCount[day]), this._pattern.d)) && (appointmentsList[app][this.model.appointmentSettings[this.model.resources[this.model.resources.length - 1].field]] == this.res1[res].id) && (Globalize.format(new Date(appointmentsList[app][this.model.appointmentSettings["endTime"]]), this._pattern.d) >= Globalize.format(new Date(dateCount[day]), this._pattern.d)) && (Globalize.format(new Date(appointmentsList[app][this.model.appointmentSettings["startTime"]]), this._pattern.d) == Globalize.format(new Date(appointmentsList[app][this.model.appointmentSettings["endTime"]]), this._pattern.d)) && (appointmentsList[app][this.model.appointmentSettings["allDay"]] === false) && (Globalize.format(new Date(appointmentsList[app][this.model.appointmentSettings["startTime"]]), this._pattern.d) <= Globalize.format(new Date(appointmentsList[app][this.model.appointmentSettings["endTime"]]), this._pattern.d))) {
                    //            this._appList.push(appointmentsList[app]);
                    //        }
                    //    }
                    //    else {
                    //        if ((Globalize.format(new Date(appointmentsList[app][this.model.appointmentSettings["startTime"]]), this._pattern.d) <= Globalize.format(new Date(dateCount[day]), this._pattern.d)) && (Globalize.format(new Date(appointmentsList[app][this.model.appointmentSettings["endTime"]]), this._pattern.d) >= Globalize.format(new Date(dateCount[day]), this._pattern.d)) && (Globalize.format(new Date(appointmentsList[app][this.model.appointmentSettings["startTime"]]), this._pattern.d) == Globalize.format(new Date(appointmentsList[app][this.model.appointmentSettings["endTime"]]), this._pattern.d)) && (appointmentsList[app][this.model.appointmentSettings["allDay"]] === false) && (Globalize.format(new Date(appointmentsList[app][this.model.appointmentSettings["startTime"]]), this._pattern.d) <= Globalize.format(new Date(appointmentsList[app][this.model.appointmentSettings["endTime"]]), this._pattern.d))) {
                    //            this._appList.push(appointmentsList[app]);
                    //        }
                    //    }
                    //}

                    for (var i = 0; i < this._appList.length; i++) {
                        this._renderApp(this._appList[i], day, res);
                        this._filtered.push(this._appList[i]);
                    }
                }
              }
        },

    _getDayAppointments: function (day, res) {
        this._appList = [];
        var dateCount = this._dateRender, appointmentsList = this._processed;
        if (this.model.orientation == "vertical") {
            for (var app = 0; app < appointmentsList.length; app++) {
                if (!ej.isNullOrUndefined(this.model.group) && ej.scheduleFeatures.resources)
                    this._getMultipleResourceAppList(app, dateCount, appointmentsList, res, day);
                else {
                    var allDayCheck = appointmentsList[app][this.model.appointmentSettings["allDay"]];
                if ((Globalize.format(new Date(appointmentsList[app][this.model.appointmentSettings["startTime"]]), this._pattern.d) <= Globalize.format(new Date(dateCount[day]), this._pattern.d)) && (Globalize.format(new Date(appointmentsList[app][this.model.appointmentSettings["endTime"]]), this._pattern.d) >= Globalize.format(new Date(dateCount[day]), this._pattern.d)) && (Globalize.format(new Date(appointmentsList[app][this.model.appointmentSettings["startTime"]]), this._pattern.d) == Globalize.format(new Date(appointmentsList[app][this.model.appointmentSettings["endTime"]]), this._pattern.d)) && (((ej.isNullOrUndefined(allDayCheck)||allDayCheck == "")? false:allDayCheck=== true ? true:allDayCheck.toLowerCase()==='false'? false : allDayCheck) === false) && (Globalize.format(new Date(appointmentsList[app][this.model.appointmentSettings["startTime"]]), this._pattern.d) <= Globalize.format(new Date(appointmentsList[app][this.model.appointmentSettings["endTime"]]), this._pattern.d))) {
                    var id = (appointmentsList[app][this.model.appointmentSettings["recurrence"]] == undefined || appointmentsList[app].ParentId == undefined) ? this._processed[app].Id : this._processed[app].ParentId;
                    var query = new ej.Query().where(this.model.appointmentSettings["id"], ej.FilterOperators.equal, parseInt(id));
                    var list = this._currentAppointmentData;
                    var templist = new ej.DataManager(list).executeLocal(query);
                    (((new Date(templist[0][this.model.appointmentSettings["endTime"]]) - new Date(templist[0][this.model.appointmentSettings["startTime"]])) / 3600000) > 24) ? this._setAllDayPos(appointmentsList[app], day, res) : this._appList.push(appointmentsList[app]);
                   }
                }
            }
        }
        else if (this.model.orientation == "horizontal" && ej.scheduleFeatures.horizontal)
            this._getHorizontalDayAppointments(appointmentsList, dateCount, day, res);
    },

        /**
    * Section for handle the utc drop down.
    * @private
    */
    _utc: function (e) {
        if (this.element.hasClass("startutc"))
            $(".endutc").val(e.text);
        else
            $(".startutc").val(e.text);
    },
    /**
    * Section to create handle for appointments in row wise.
    * @private
    */
    _rowHandlesOver: function (e) {
        $(e.target).addClass("e-rowcursor");
    },
    /**
    * Section to create handle for appointments in column wise.
    * @private
    */
    _colHandlesOver: function (e) {
        $(e.target).addClass("e-columncursor");
    },
    /**
        * Section for render the appointments from dataSource.
        * @private
        */
    _renderApp: function (record, day, res) {
        var cellHeight = this.element.find(".e-workcells").height();
        var timeScaleRowCount = (60 / 30);
        var startHour = parseInt(this.model.startHour) < parseInt(this.model.endHour) ? parseInt(this.model.startHour) : 0;
        var endHour = parseInt(this.model.startHour) < parseInt(this.model.endHour) ? parseInt(this.model.endHour) : 24;
        var dateCount = this._dateRender;
        var rStartTime = this._formatTime(record[this.model.appointmentSettings["startTime"]]);
        var rEndTime = this._formatTime(record[this.model.appointmentSettings["endTime"]]);

        if (this.model.orientation == "vertical") {         
            var appWidth, appHeight;
           
            if (rStartTime <= rEndTime) {
                var startTime = ((new Date(rStartTime).getHours()) < startHour) ? (new Date(rStartTime).getHours() + (-new Date(rStartTime).getHours()) + (-new Date(rStartTime).getMinutes())) : (new Date(rStartTime).getHours() + (-startHour));
                var endTime = ((new Date(rEndTime).getHours()) > endHour) ? (new Date(rEndTime).getHours() + (-new Date(rEndTime).getHours() + (endHour - startHour)) + (-new Date(rEndTime).getMinutes())) : (new Date(rEndTime).getHours() + (-startHour));
                endTime = (new Date(rEndTime).getHours() == endHour) ? (new Date(endTime).getMinutes() + (-new Date(rEndTime).getMinutes())) : endTime;

                if (endHour <= rEndTime.getHours()) {
                    var endDate = new Date(rEndTime.setHours(endHour, 00, 00));
                    appHeight = (parseFloat(((new Date(endDate).getHours() * 60) + (new Date(endDate).getMinutes())) - ((new Date(rStartTime).getHours() * 60) + (new Date(rStartTime).getMinutes()))) * ((cellHeight) * timeScaleRowCount) / 60) - 3;
                }
                else
                    appHeight = (parseFloat(((new Date(rEndTime).getHours() * 60) + (new Date(rEndTime).getMinutes())) - ((new Date(rStartTime).getHours() * 60) + (new Date(rStartTime).getMinutes()))) * ((cellHeight) * timeScaleRowCount) / 60) - 2;
                var topPosition = parseFloat((new Date(rStartTime).getMinutes())) * (cellHeight) * timeScaleRowCount / 60;

                if (this.model.startHour <= new Date(rStartTime).getHours()) {

                    var overlapList = this._overlapApp(record[this.model.appointmentSettings["id"]], record[this.model.appointmentSettings["startTime"]], record[this.model.appointmentSettings["endTime"]], null);
                    var proxy = this;
                    if (overlapList.length > 1) {
                        //overlapList.push(record);
                        overlapList.sort(function (a, b) {
                        var d1 = proxy._formatTime(a[proxy.model.appointmentSettings["startTime"]]);
                        var d2 = proxy._formatTime(b[proxy.model.appointmentSettings["startTime"]]);
                            return d1 - d2;
                        });
                        var index = overlapList.indexOf(record);
                        appWidth = (100 / overlapList.length) + 2;
                        this.currentView() === "day" ? appWidth -= 7 : appWidth;
                        this.leftPosition = ((100 / overlapList.length) * (index)) + "%";
                    }
                    else {
                        this.leftPosition = "0";
                        appWidth = 95;
                    }

                    appWidth = (this.currentView() === "week" || this.currentView() === "workweek") ? (appWidth - 10) + "%" : appWidth + "%";

                var colorAppointment = this._appointmentColor(record);
                //user-defined Template data rendering
                var userAppTemplId = (this.model.appointmentTemplateId) ? true : false;
                var userTempHtml = this._getUserAppointmentTemplate(record);
                var startTimeDisp = this.model.timeMode == "12" ? Globalize.format(this._formatTime(record[this.model.appointmentSettings["startTime"]]), this._pattern.t) : Globalize.format(this._formatTime(record[this.model.appointmentSettings["startTime"]]), "HH:mm");
                var endTimeDisp = this.model.timeMode == "12" ? Globalize.format(this._formatTime(record[this.model.appointmentSettings["endTime"]]), this._pattern.t) : Globalize.format(this._formatTime(record[this.model.appointmentSettings["endTime"]]), "HH:mm");

                    //   var category = "e-" + record[this.model.appointmentSettings["categorize"]];
                var timeCellhtml = this.appTemplate.render({ id: record[this.model.appointmentSettings["id"]], appHeight: appHeight, rtl: this.model.enableRTL, subject: record[this.model.appointmentSettings["subject"]], startTime: startTimeDisp, endTime: endTimeDisp, left: this.leftPosition, top: topPosition, appWidth: appWidth, userAppTemplId: userAppTemplId, userTemplate: userTempHtml, appClass: "e-appointment", appointData: colorAppointment.appointColor.appointData, appointCustomcss: colorAppointment.appointColor.appointCustomcss, appointtextcolor: colorAppointment.appointColor.appointtextcolor, value: colorAppointment.value, multiDiv: colorAppointment.multiDiv, orientation: this.model.orientation });
                    $(this.element.find("div.e-appointwrapper")[(startTime * 2 * dateCount.length * this.res1.length) + day + (dateCount.length * res)]).append(timeCellhtml);
                }
                //                $(this.element.find("td.e-workcells")[(startTime * 2 * dateCount.length) + day]).addClass("icon-AddGrad");
            }
        }
        else if (this.model.orientation == "horizontal" && ej.scheduleFeatures.horizontal)
            this._renderHorizontalApp(rStartTime, rEndTime, record, day, cellHeight, timeScaleRowCount, startHour, endHour);
    },

    /**
    * Section for get the appointments template from user.
    * @private
    */
    _getUserAppointmentTemplate: function (record) {
        var userTemp = $.templates($(this.model.appointmentTemplateId).html());
        var userTempHtml = userTemp.render(record);
        return userTempHtml;
    },
    /**
    * Section for handle the overlap appointments.
    * @private
    */
    _overlapApp: function (id, appStartTime, appEndTime, resValue) {
        this.overlapList = [];
        if (this.model.orientation == "vertical") {
            for (var i = 0; i < this._appList.length; i++) {

                if ((new Date(this._appList[i][this.model.appointmentSettings["endTime"]]) > new Date(appStartTime)) && (new Date(this._appList[i][this.model.appointmentSettings["startTime"]]) < new Date(appEndTime))
                    && (Globalize.format(new Date(this._appList[i][this.model.appointmentSettings["startTime"]]), this._pattern.d) === Globalize.format(new Date(appStartTime), this._pattern.d))) {
                    this.overlapList.push(this._appList[i]);
                }
            }
        }
        else if (this.model.orientation == "horizontal" && ej.scheduleFeatures.horizontal) {
            for (var i = 0; i < this._appList.length; i++) {
                if (ej.scheduleFeatures.resources && !ej.isNullOrUndefined(this.model.resources)) {
                    if ((new Date(this._appList[i][this.model.appointmentSettings["endTime"]]) > new Date(appStartTime)) && (new Date(this._appList[i][this.model.appointmentSettings["startTime"]]) < new Date(appEndTime))
                        && (this._appList[i][this.model.resources[this.model.resources.length - 1]["field"]] == resValue)
                        && (Globalize.format(new Date(this._appList[i][this.model.appointmentSettings["startTime"]]), this._pattern.d) === Globalize.format(new Date(appStartTime), this._pattern.d))) {
                        this.overlapList.push(this._appList[i]);
                    }
                }
                else {
                    if ((new Date(this._appList[i][this.model.appointmentSettings["endTime"]]) > new Date(appStartTime)) && (new Date(this._appList[i][this.model.appointmentSettings["startTime"]]) < new Date(appEndTime))
                        && (Globalize.format(new Date(this._appList[i][this.model.appointmentSettings["startTime"]]), this._pattern.d) === Globalize.format(new Date(appStartTime), this._pattern.d))) {
                        this.overlapList.push(this._appList[i]);
                    }
                }
            }
        }
        return this.overlapList;
    },
    /**
    * Section for render allday appointments of the schedule.
    * @private
    */
    _renderAllDayAppointments: function () {
        var appHeight = 16;
            var appWidth = this.cellwidth - 20;
            var renderDate = this._dateRender;
            var level = 1; this.level = 0;
            if (!this.model.group) this.res1 = ["0"];
            for (var res = 0; res < this.res1.length; res++) {
                for (var day = 0; day < renderDate.length; day++) {
                    for (var i = 0; i < this._processed.length; i++) {
                        if (!ej.isNullOrUndefined(this.model.resources) && !ej.isNullOrUndefined(this.model.group) && ej.scheduleFeatures.resources)
                            this._getMultipleResourceAlldayApp(renderDate, day, res, i);
                        else {
                            if ((Globalize.format(new Date(renderDate[day]), this._pattern.d) == Globalize.format(new Date(this._processed[i][this.model.appointmentSettings["startTime"]]), this._pattern.d)) &&
                    (this._processed[i][this.model.appointmentSettings["allDay"]] === true || (((new Date(this._processed[i][this.model.appointmentSettings["endTime"]]) - new Date(this._processed[i][this.model.appointmentSettings["startTime"]])) / 3600000) > 24))) {
                                this._setAllDayPos(this._processed[i], day, res);
								this._filtered.push(this._processed[i]);
                                var id = (this._processed[i][this.model.appointmentSettings["recurrence"]] == undefined || this._processed[i].ParentId == undefined) ? this._processed[i].Id : this._processed[i].ParentId;
                                var query = new ej.Query().where(this.model.appointmentSettings["id"], ej.FilterOperators.equal, parseInt(id));
                                var list = this._currentAppointmentData;
                                var templist = new ej.DataManager(list).executeLocal(query);
                                if ((new Date(this._processed[i][this.model.appointmentSettings["endTime"]])) < (templist[0][this.model.appointmentSettings["endTime"]])) {
                                    this.element.find(".e-alldayappointment").children(".e-righthandle").remove();
                                }
                                if ((new Date(this._processed[i][this.model.appointmentSettings["startTime"]])) > (templist[0][this.model.appointmentSettings["startTime"]])) {
                                    this.element.find(".e-alldayappointment").children(".e-lefthandle").remove();
                                }
                            }
                        }
                    }
                }
            }
    },
    /**
    * Section for get the overlap appointments count of the schedule.
    * @private
    */
    _getOverlapCount: function (record, day, dateRender, resValue) {
        var len = dateRender.length;
        var appointments = this._processed;
        var filterData = [];
        if (this.model.orientation == "vertical") {
            for (var i = 0; i < appointments.length; i++) {
                if ((Globalize.format(new Date(appointments[i][this.model.appointmentSettings["startTime"]]), this._pattern.d) <= Globalize.format(new Date(dateRender[day]), this._pattern.d)) &&
                (Globalize.format(new Date(appointments[i][this.model.appointmentSettings["endTime"]]), this._pattern.d) >= Globalize.format(new Date(dateRender[day]), this._pattern.d)) &&
                (appointments[i][this.model.appointmentSettings["allDay"]] == true || (Globalize.format(new Date(appointments[i][this.model.appointmentSettings["startTime"]]), this._pattern.d) < Globalize.format(new Date(appointments[i][this.model.appointmentSettings["endTime"]]), this._pattern.d))) &&
                (Globalize.format(new Date(appointments[i][this.model.appointmentSettings["startTime"]]), this._pattern.d) <= Globalize.format(new Date(appointments[i][this.model.appointmentSettings["endTime"]]), this._pattern.d))) {
                    filterData.push(appointments[i]);
                }
            }
        }
        else if (this.model.orientation == "horizontal" && ej.scheduleFeatures.horizontal)
            filterData = this._getHorizontalOverlapCount(appointments, dateRender, day, record, resValue);
        var proxy = this;
        filterData.sort(function (a, b) {
            var d1 = proxy._formatTime(a[proxy.model.appointmentSettings["startTime"]]);
            var d2 = proxy._formatTime(b[proxy.model.appointmentSettings["startTime"]]);
            return d1 - d2;
        });
        var index = filterData.length > 0 ? filterData.indexOf(record) : 0;
        return index == -1 ? 0 : index;
    },
    /**
        * Section for handle the allday appointments position.
        * @private
        */
        _setAllDayPos: function (record, day, res) {
            var level = this._getOverlapCount(record, day, this._dateRender, null);
            var start = Globalize.format(this._formatTime(record[this.model.appointmentSettings["startTime"]]), this._pattern.d);
            var end = Globalize.format(this._formatTime(record[this.model.appointmentSettings["endTime"]]), this._pattern.d);
        if (new Date(start) <= new Date(end)) {
            var dateRender = this._dateRender;
            var appWidth, left = 0, top = 15, appHeight = 15;
            var r_start = Globalize.format(new Date(dateRender[0]), this._pattern.d);
            var r_end = Globalize.format(new Date(dateRender[dateRender.length - 1]), this._pattern.d);
            end = new Date(end) > new Date(r_end) ? r_end : end;
                var noOfDays = Math.ceil((new Date(start) - new Date(r_start)) / (1000 * 3600) / 24);
                var diffInDays = Math.ceil((new Date(end) - new Date(start)) / (1000 * 3600) / 24) + 1;
                var cellWidth = this.element.find(".e-alldaycells").width();
                var daysCount = this.currentView() === "week" ? 7 : this.currentView() === "workweek" ? 5 : 1;
                var alldayTr = (!ej.isNullOrUndefined(this.model.group) && !ej.isNullOrUndefined(this.model.resources)) ? this.model.group.resources.length : 0;
                var tdIndex = this.currentView() === "day" ? 0 : this.currentView() === "workweek" ? new Date(start).getDay()-1 : new Date(start).getDay();
                left = (this.model.enableRTL) ? this.element.find(".e-headrealldaytable").width() - this.element.find(".e-alldaycells")[tdIndex + (res * daysCount)].offsetLeft - this.element.find(".e-alldaycells").width() : this.element.find(".e-alldaycells")[tdIndex + (res * daysCount)].offsetLeft;
                appWidth = (diffInDays * cellWidth) - ((diffInDays * cellWidth)*(4/100)); 
                top = (level * top) + (3 * (level + 1)) + this.element.find(".e-headercells").height() + 3 + (alldayTr * $($(".e-headerbar")[0]).height());

                var colorAppointment = this._appointmentColor(record);

                //user-defined Template data rendering
                var userAppTemplId = (this.model.appointmentTemplateId) ? true : false;
            var userTempHtml = this._getUserAppointmentTemplate(record);
               
               var startTimeDisp = this.model.timeMode == "12" ? Globalize.format(this._formatTime(record[this.model.appointmentSettings["startTime"]]), this._pattern.t) : Globalize.format(this._formatTime(record[this.model.appointmentSettings["startTime"]]), "HH:mm");
                var endTimeDisp = this.model.timeMode == "12" ? Globalize.format(this._formatTime(record[this.model.appointmentSettings["endTime"]]), this._pattern.t) : Globalize.format(this._formatTime(record[this.model.appointmentSettings["endTime"]]), "HH:mm");
        //    var category = "e-" + record[this.model.appointmentSettings["categorize"]];
            var alldayCellhtml = this.appTemplate.render({ id: record[this.model.appointmentSettings["id"]], top: top, appHeight: appHeight, rtl: this.model.enableRTL, subject: record[this.model.appointmentSettings["subject"]], startTime: startTimeDisp, endTime: endTimeDisp , appWidth: appWidth + "px", allday: true, left: left + "px", userAppTemplId: userAppTemplId, userTemplate: userTempHtml, appClass: "e-alldayappointment", appointData: colorAppointment.appointColor.appointData, appointCustomcss: colorAppointment.appointColor.appointCustomcss, appointtextcolor: colorAppointment.appointColor.appointtextcolor, value: colorAppointment.value, multiDiv: colorAppointment.multiDiv, orientation: this.model.orientation });
            if (this.level < level) {
                var cellHeight = (level + 1) * (appHeight + 3) + 3;
                this.element.find(".e-alldaycells").css('height', cellHeight + "px");
            }
            this.element.find(".e-headrealldaytable").append(alldayCellhtml);
            this.level = this.level < level ? level : this.level;
        }
        if (level > 0)
            this._reRenderScroller();
        },
        _reRenderScroller: function () {
            var hghtdiff = this.element.find(".e-scheduleheader").height() + this.element.find(".e-headrealldaytable").height();
            var _scrollDiv = $('div#' + this._id + "_scroller");
            _scrollDiv.ejScroller({
                height: this.scHeight - hghtdiff,
            });
            var wdthdiff = (this.currentView() === "month") ? 0 : 4;
            var cols = this.currentView() == "workweek" ? 5 : this.currentView() == "day" ? 1 : 7;
            var col = (!ej.isNullOrUndefined(this.model.group) && !ej.isNullOrUndefined(this.model.resources)) ? (this.res1.length * cols) : cols;
            this.element.find(".e-draggableworkarea").width(((this.scWidth > (parseInt(this.model.cellWidth) * col) + this.element.find(".e-scrolltimecells").width() + 18) ? (parseInt(this.model.cellWidth) * col) : this.scWidth - this.element.find(".e-scrolltimecells").width() + wdthdiff - 18));
        },
    /**
    * Section for render the appointments in the month view of schedule.
    * @private
    */
    _renderMonthAppointment: function () {
        this.filterData = []; var filtered = [];
        var renderDate = (this.model.orientation == "vertical") ? this._dateRender : this.monthDays;
        
            // var appointments = this.model.appointmentSettings["dataSource"];
            var appointments = this._processed; //this._dataProcessing(this.dataSource());
            if (!this.model.group) this.res1 = ["0"];

            for (var res = 0; res < this.res1.length; res++) {
                for (var day = 0; day < renderDate.length; day++) {
                    this.filterData = [];
                    for (var app = 0; app < appointments.length; app++) {
                        if (!ej.isNullOrUndefined(this.model.resources) && !ej.isNullOrUndefined(this.model.group) && ej.scheduleFeatures.resources)
                            this._getMonthAppList(renderDate, day, appointments, app, res);
                        else {
                            if ((appointments[app][this.model.appointmentSettings["allDay"]] == true) || (((new Date(appointments[app][this.model.appointmentSettings["endTime"]]) - new Date(appointments[app][this.model.appointmentSettings["startTime"]])) / 3600000) > 24)) {
                                if ((Globalize.format(new Date(renderDate[day]), this._pattern.d) == Globalize.format(new Date(appointments[app][this.model.appointmentSettings["startTime"]]), this._pattern.d)) &&
                    (Globalize.format(new Date(renderDate[day]), this._pattern.S) <= Globalize.format(new Date(appointments[app][this.model.appointmentSettings["endTime"]]), this._pattern.S)) &&
                    (Globalize.format(new Date(appointments[app][this.model.appointmentSettings["startTime"]]), this._pattern.S) <= Globalize.format(new Date(appointments[app][this.model.appointmentSettings["endTime"]]), this._pattern.S))) {
                                    this.filterData.push(appointments[app]);
                                }
                            } else {
                                if ((Globalize.format(new Date(renderDate[day]), this._pattern.d) == Globalize.format(new Date(appointments[app][this.model.appointmentSettings["startTime"]]), this._pattern.d)) &&
         (Globalize.format(new Date(renderDate[day]), this._pattern.d) <= Globalize.format(new Date(appointments[app][this.model.appointmentSettings["endTime"]]), this._pattern.d)) &&
         (Globalize.format(new Date(appointments[app][this.model.appointmentSettings["startTime"]]), this._pattern.d) <= Globalize.format(new Date(appointments[app][this.model.appointmentSettings["endTime"]]), this._pattern.d))) {
                                    this.filterData.push(appointments[app]);
                                }
                            }
                        }
                    }
                    for (var i = 0; i < this.filterData.length; i++) {
                        //var level = this._getIndex(day, filterData[i], renderDate, appointments);
                        this._renderAppointmentMonth(this.filterData[i], day, res);
                        filtered.push(this.filterData[i]);
                    }
                }
            }
            if(this.model.enableAppointmentNavigation)
                this._nextPrevButton(filtered);
        },
        /**
    * Wiring the events to resize appointments .
    * @private
    */
    _wireResizeEvents: function () {
        if (this.model.enableAppointmentResize) {
            this._on(this.element.find("div.e-lefthandle,div.e-righthandle,div.e-tophandle,div.e-bottomhandle"), "mousedown", $.proxy(this._appointmentResizeStart, this));
            this._on(this.element.find("div.e-lefthandle,div.e-righthandle"), "mouseover", $.proxy(this._colHandlesOver, this));
            this._on(this.element.find("div.e-tophandle,div.e-bottomhandle"), "mouseover", $.proxy(this._rowHandlesOver, this));
        }
    },
    /**
    * Unwiring the events to resize appointments .
    * @private
    */
    _unWireResizeEvents: function () {
        if (this.model.enableAppointmentResize) {
            //Unwire events 
            this._off(this.element.find("div.e-lefthandle,div.e-righthandle,div.e-tophandle,div.e-bottomhandle"), "mousedown", $.proxy(this._appointmentResizeStart, this));
            this._off(this.element.find("div.e-lefthandle,div.e-righthandle"), "mouseover", $.proxy(this._colHandlesOver, this));
            this._off(this.element.find("div.e-tophandle,div.e-bottomhandle"), "mouseover", $.proxy(this._rowHandlesOver, this));
        }
    },
    /**
    * Section for get the index to render the appointment.
    * @private
    */
    _getIndex: function (day, record, dateRender, appointments, resValue) {
        var proxy = this;
        appointments.sort(function (a, b) {
            var d1 = proxy._formatTime(a[proxy.model.appointmentSettings["startTime"]]);
            var d2 = proxy._formatTime(b[proxy.model.appointmentSettings["startTime"]]);
            return d1 - d2;
        });
        var len = dateRender.length;
        var data = [];
        for (var app = 0; app < appointments.length; app++) {
            if (this.model.orientation == "horizontal" && ej.scheduleFeatures.resources && !ej.isNullOrUndefined(this.model.group)) {
                if ((Globalize.format(new Date(appointments[app][this.model.appointmentSettings["startTime"]]), this._pattern.d) <= Globalize.format(new Date(dateRender[day]), this._pattern.d)) &&
                (Globalize.format(new Date(appointments[app][this.model.appointmentSettings["endTime"]]), this._pattern.d) >= Globalize.format(new Date(dateRender[day]), this._pattern.d)) &&
                (appointments[app][this.model.resources[this.model.resources.length - 1]["field"]] == resValue) &&
                (Globalize.format(new Date(appointments[app][this.model.appointmentSettings["startTime"]]), this._pattern.d) <= Globalize.format(new Date(appointments[app][this.model.appointmentSettings["endTime"]]), this._pattern.d))) {
                    data.push(appointments[app]);
                }
            }
            else {
                if ((Globalize.format(new Date(appointments[app][this.model.appointmentSettings["startTime"]]), this._pattern.d) <= Globalize.format(new Date(dateRender[day]), this._pattern.d)) &&
                (Globalize.format(new Date(appointments[app][this.model.appointmentSettings["endTime"]]), this._pattern.d) >= Globalize.format(new Date(dateRender[day]), this._pattern.d)) &&
                (Globalize.format(new Date(appointments[app][this.model.appointmentSettings["startTime"]]), this._pattern.d) <= Globalize.format(new Date(appointments[app][this.model.appointmentSettings["endTime"]]), this._pattern.d))) {
                    data.push(appointments[app]);
                }
            }
        }
        this.overlapList = data;
        var index = data.length > 0 ? data.indexOf(record) : 0;
        return index;
    },
    /**
        * Render the appointments in the month view.
        * @private
        */
    _renderAppointmentMonth: function (record, day, res) {

        // var renderDate = this._dateRender;
        var renderDate = (this.model.orientation == "vertical") ? this._dateRender : this.monthDays;
        var overlapCount = (ej.scheduleFeatures.resources && !ej.isNullOrUndefined(this.model.group) && !ej.isNullOrUndefined(this.model.resources)) ? this._getIndex(day, record, renderDate, this._processed, record[this.model.appointmentSettings.resourceFields.toString().split(',')[this.model.resources.length - 1]]) : this._getIndex(day, record, renderDate, this._processed, null);
        var startTime = new Date(record[this.model.appointmentSettings["startTime"]]);
        var endTime = new Date(record[this.model.appointmentSettings["endTime"]]);
        var appHeight = 18;
        var appWidth, topPosition = 0;
        this.leftPosition = 0;
        this.CellWidth = this.model.orientation == "vertical" ? this.element.find(".e-monthcells").width() : this.element.find(".e-workcells").width();
        this.CellHeight = this.model.orientation == "vertical" ? this.element.find(".e-monthcells").height() : this.element.find(".e-workcells").height();

        var start = (Globalize.format(startTime, this._pattern.d));
        var end = (Globalize.format(endTime, this._pattern.d));
        var r_end = new Date(renderDate[renderDate.length - 1]);
        end = new Date(end) > new Date(r_end) ? r_end : end;
        var maxDays = 7 - (new Date(start).getDay());
        var diffInDays = Math.ceil((new Date(end) - new Date(start)) / (1000 * 3600) / 24) + 1;
        

        if (startTime <= endTime) {
            if (this.model.orientation == "vertical") {
                diffInDays = diffInDays >= maxDays ? maxDays : diffInDays;
                var rowPosition = Math.ceil((day + 1) / 7);
                if (Globalize.format(startTime, this._pattern.d) != Globalize.format(endTime, this._pattern.d))
                    appWidth = (diffInDays * this.CellWidth) - 15 - diffInDays;
                else
                    appWidth = (this.CellWidth * 6) / 7;
            }
            else
                appWidth = (diffInDays * this.CellWidth);
             var resIndex = (!ej.isNullOrUndefined(this.model.resources) && !ej.isNullOrUndefined(this.model.group) && ej.scheduleFeatures.resources) ? this._findResourceIndex(this.res1, this.model.resources[this.model.resources.length - 1].resourceSettings.id, record[this.model.appointmentSettings.resourceFields.toString().split(',')[this.model.resources.length - 1]]) : 0;
            var cellId = this.model.orientation == "vertical" ? (resIndex * (7) + (new Date(startTime).getDay() + 1)) + ((rowPosition - 1) * (this.res1.length * 7)) : day + 1;    // (7 * (rowPosition - 1)) + (new Date(startTime).getDay() + 1);

            topPosition = (overlapCount * appHeight);
            this.leftPosition = 0;

            if(!ej.isNullOrUndefined(this.model.resources))
            var newResCollection = this._getResourceCollection();
            //var groupIndex = this._findResourceIndex(newResCollection[0].dataSource, "id", record[this.model.appointmentSettings.resourceFields.toString().split(',')[this.model.resources.length - 2]]);
            //var resValue = this._findResourceIndex(this.level_Resources, "id", record[this.model.appointmentSettings.resourceFields.toString().split(',')[this.model.resources.length - 1]]);
            var groupIndex = (!ej.isNullOrUndefined(this.model.group) && this.model.group.resources.length > 1) ? this._findResourceIndex(newResCollection[0].dataSource, newResCollection[0].id, record[this.model.appointmentSettings.resourceFields.toString().split(',')[this.model.resources.length - 2]]) + 1 : 0;
            var resValue = (!ej.isNullOrUndefined(this.model.group) && !ej.isNullOrUndefined(this.model.resources)) ? this._findResourceIndex(this.level_Resources, this.model.resources[this.model.resources.length - 1].resourceSettings.id, record[this.model.appointmentSettings.resourceFields.toString().split(',')[this.model.resources.length - 1]]) + 1 : 1;
            var colorAppointment = this._appointmentColor(record);
            if ((overlapCount * (appHeight + 21)) < (this.CellHeight)) {
                appHeight -= 2;
                if (topPosition > 1) topPosition -= 1;
                if (this.leftPosition > 1) this.leftPosition -= 1;

                

                //user-defined Template data rendering
                var userAppTemplId = (this.model.appointmentTemplateId) ? true : false;
                var userTempHtml = this._getUserAppointmentTemplate(record);

                var startTimeDisp = this.model.timeMode == "12" ? Globalize.format(this._formatTime(record[this.model.appointmentSettings["startTime"]]), this._pattern.t) : Globalize.format(this._formatTime(record[this.model.appointmentSettings["startTime"]]), "HH:mm");
                var endTimeDisp = this.model.timeMode == "12" ? Globalize.format(this._formatTime(record[this.model.appointmentSettings["endTime"]]), this._pattern.t) : Globalize.format(this._formatTime(record[this.model.appointmentSettings["endTime"]]), "HH:mm");
                //   var category = "e-" + record[this.model.appointmentSettings["categorize"]];
                var timeCellhtml = this.appTemplate.render({ id: record[this.model.appointmentSettings["id"]], appHeight: appHeight, rtl: this.model.enableRTL, subject: record[this.model.appointmentSettings["subject"]], startTime: startTimeDisp, endTime: endTimeDisp, left: this.leftPosition + "px", top: topPosition, appWidth: appWidth + "px", userAppTemplId: userAppTemplId, userTemplate: userTempHtml, appClass: "e-monthappointment", appointData: colorAppointment.appointColor.appointData, appointCustomcss: colorAppointment.appointColor.appointCustomcss, appointtextcolor: colorAppointment.appointColor.appointtextcolor, value: colorAppointment.value, multiDiv: colorAppointment.multiDiv });
                if (this.model.orientation == "vertical")
                    $(this.element.find("div.e-appointwrapper")[cellId - 1]).append(timeCellhtml);
                else if (this.model.orientation == "horizontal" && ej.scheduleFeatures.horizontal) {
                    $($(this.element.find(".e-workcellstab tr")[resValue + groupIndex - 1]).find("div.e-appointwrapper")[cellId - 1]).append(timeCellhtml);
                    $($($(this.element.find(".e-workcellstab tr").find("td.e-resourceheadercells")).parent()[groupIndex - 1]).find("div.e-appointwrapper")[cellId - 1]).append("<div class='e-categorybar' id='categorybar_'" + record[this.model.appointmentSettings["id"]] + " style='position:absolute;height:" + (this.CellHeight - 3) + "px;background:" + colorAppointment.appointColor.appointData + ";width:" + (appWidth + 1) + "px;left:" + this.leftPosition + "px;'></div>");
                }
                //this.element.find("div.e-draggableworkarea").append(timeCellhtml);
                timeCellhtml = "";
            }
            else {
                if ($("#" + this._id + "_" + Globalize.format(new Date(record[this.model.appointmentSettings["startTime"]]), "MM_dd_yyyy") + "_" + (resValue + groupIndex - 1) + "_" + (cellId - 1)).length == 0) {
                    if (this.model.orientation == "vertical")
                        $(this.element.find("div.e-appointwrapper")[cellId - 1]).append('<div id="' + this._id + "_" + Globalize.format(new Date(record[this.model.appointmentSettings["startTime"]]), "MM_dd_yyyy") + "_" + (resValue + groupIndex - 1) + "_" + (cellId - 1) + '" class="e-icon e-appsoverflow"></div>');
                    else {
                        $($(this.element.find(".e-workcellstab tr")[resValue + groupIndex - 1]).find("div.e-appointwrapper")[cellId - 1]).append('<div id="' + this._id + "_" + Globalize.format(new Date(record[this.model.appointmentSettings["startTime"]]), "MM_dd_yyyy") + "_" + (resValue + groupIndex - 1) + "_" + (cellId - 1) + '" class="e-icon e-appsoverflow"></div>');
                    }
                    this.element.find("#" + this._id + "_" + Globalize.format(new Date(record[this.model.appointmentSettings["startTime"]]), "MM_dd_yyyy") + "_" + (resValue + groupIndex - 1) + "_" + (cellId - 1)).css(this.model.enableRTL ? "right" : "left", (this.CellWidth + this.leftPosition - 25) + "px").css("top", topPosition + "px");
                }
            }
        }
    },
    
    /**
    * Section for handle the quick window close.
    * @private
    */
    _quickWindowClose: function () {
        this._quickAppointWindow.ejDialog("close");
        this._processFocus();
    },
    /**
    * Section for create the appointment window.
    * @private
    */
    _appointmentWindow: function (e) {
        var event = { startTime: new Date(this.cur_StartTime), endTime: this.cur_EndTime, target: e };
        var startTime, endTime, _target = !ej.isNullOrUndefined(e.target) ? $(e.target) : $(e);
        if (this._trigger("cellDoubleClick", event) || _target.hasClass("e-categorybar"))
            return false;
        if ((_target.hasClass("e-workcells") || _target.hasClass("e-alldaycells") || _target.hasClass("e-monthcells") || this._cellClick || _target.hasClass("e-detailedapp")) && !_target.hasClass("e-resourceheadercells")) {
            this._currentAction = ej.Schedule.Actions.Add;
            this._appointmentAddWindow.find(".e-recurwindow").css("display", "none");
            this._appointmentAddWindow.find(".e-appwindow").css("display", "block");
            var index = this._cellIndex;
            if (this.model.orientation === "horizontal")
                index = Math.floor(index / ((this.model.endHour - this.model.startHour) * 2));
            if (!this._cellClick) {
                this.currentDate(new Date(this.dateRender[index]));
                if (_target.hasClass("e-workcells")) {
                    var time = this.model.startHour + (_target.parent().index() / 2);
                    var timemin = time.toString().split(".");
                    startTime = new Date(this.currentDate()).setHours(parseInt(timemin[0]), parseInt(timemin[1]) == 5 ? 30 : 00);
                    endTime = new Date(new Date(startTime).setMinutes(new Date(startTime).getMinutes() + 30));
                }
                else if (_target.hasClass("e-monthcells")) {
                    startTime = new Date(new Date(this.currentDate()).setHours(9));
                    startTime = new Date(startTime).setMinutes(00); //.setHours(00)).setMinutes(00);
                    endTime = new Date(new Date(startTime).setMinutes(30)); //.setHours(23)).setMinutes(59));
                }
                else {
                    startTime = new Date(this.currentDate()); //.setHours(00)).setMinutes(00);
                    endTime = new Date(startTime); //.setHours(23)).setMinutes(59));
                    this._appointmentAddWindow.find(".allday ").ejCheckBox({ checked: true });
                }
            } else {
                startTime = new Date(this.cur_StartTime);
                endTime = new Date(this.cur_EndTime);
            }
            if (this._trigger("appointmentWindowOpen", event))
                return false;
            this._appointmentAddWindow.find("#" + this._id + "subject").val(this.model.showQuickWindow ? this._quickAppointWindow.find(".subject").val() : "");
            this._appointmentAddWindow.find("#" + this._id + "startdate").ejDatePicker("option", "value", Globalize.format(new Date(startTime), this._pattern.d));
            this._appointmentAddWindow.find("#" + this._id + "enddate").ejDatePicker("option", "value", Globalize.format(new Date(endTime), this._pattern.d));
            this._appointmentAddWindow.find("#" + this._id + "starttime").ejTimePicker("option", "value", Globalize.format(new Date(startTime), this._pattern.t));
            this._appointmentAddWindow.find("#" + this._id + "endtime").ejTimePicker("option", "value", Globalize.format(new Date(endTime), this._pattern.t));
            ($(e.currentTarget).hasClass("e-alldaycells") || this.currentView() == "month") && this._appointmentAddWindow.find(".allday").ejCheckBox({ checked: true });
            this._alldayCheck();
            if (this.model.timeZone == null) {
                document.getElementById('startutc').value = "UTC " + Globalize.format(new Date(), "zzz");
                document.getElementById('endutc').value = "UTC " + Globalize.format(new Date(), "zzz");
            }
            else {
                document.getElementById('startutc').value = this.model.timeZone;
                document.getElementById('endutc').value = this.model.timeZone;
            }
            //render resource related data in the appointment window
            if (ej.scheduleFeatures.resources && !ej.isNullOrUndefined(this.model.resources))
                this._renderResourceData(_target, this._cellIndex);
            this._appointmentAddWindow.ejDialog("open");
            this._appointmentAddWindow.find(".subject").focus();
            this._cellClick = false;
        }
        else {
            if (!_target.hasClass("e-resourceheadercells")) {
                var target = (this._editKey) ? $(e) : _target.hasClass("e-appointment") || _target.hasClass("e-monthappointment") || _target.hasClass("e-alldayappointment") ? _target : _target.parents('.e-monthappointment,.e-appointment,.e-alldayappointment');
                if (!ej.isNullOrUndefined(target[0].id)) {
                    this._currentAction = ej.Schedule.Actions.Save;
                    this._appId = parseInt(target[0].id.split("_")[1]);
                    var query = new ej.Query().where(this.model.appointmentSettings["id"], ej.FilterOperators.equal, this._appId);
                    var templist = new ej.DataManager(this._processed).executeLocal(query);
                    if (templist[0][this.model.appointmentSettings["recurrence"]] == 1) {
                        this._parentId = templist[0]["ParentId"];
                        this._recurEditWindow.find(".e-editonly").ejRadioButton({ checked: true });
                        this._recurEditWindow.ejDialog("open");
                        this._recurEditWindow.focus();
                    }
                    else {
                    if (templist[0][this.model.appointmentSettings["allDay"]] || (((this._formatTime(templist[0][this.model.appointmentSettings["endTime"]]) - this._formatTime(templist[0][this.model.appointmentSettings["startTime"]])) / 3600000) > 24)) {
                        var query = new ej.Query().where(this.model.appointmentSettings["id"], ej.FilterOperators.equal, this._appId);
                        var temp = new ej.DataManager(this._processed).executeLocal(query);
                        this._appId = temp[0].ParentId;
                    }
                        this._showAppointmentDetails(this._appId);
                    }
                    this.model.showQuickWindow && this._quickAppDetailsWindow.ejDialog("close");
                }
                this._editKey = false;
            }
        }
        this.model.showQuickWindow && this._quickAppointWindow.ejDialog("close");
    },
    /**
    * Section for toggle the appointment window.
    * @private
    */
    _editRecurrence: function (args) {
        this._appointmentAddWindow.find(".e-appwindow").toggle();
        this._appointmentAddWindow.find(".e-recurwindow").css("margin-left", 700).toggle().animate({ "margin-left": 10 }, 300);
    },
    /**
    * Section for show all the required details to the window while editing.
    * @private
    */
    _showAppointmentDetails: function (appId, editOcurrence) {
        var query = new ej.Query().where(this.model.appointmentSettings["id"], ej.FilterOperators.equal, parseInt(appId));
        var list = editOcurrence == true ? this._processed : this._currentAppointmentData;
        var templist = new ej.DataManager(list).executeLocal(query);
        this._appointmentAddWindow.find('.subject').val(templist[0][this.model.appointmentSettings["subject"]]);
        this._appointmentAddWindow.find('.id').val(templist[0][this.model.appointmentSettings["id"]]);
        this._appointmentAddWindow.find('.e-description').val(templist[0][this.model.appointmentSettings["description"]] ? templist[0][this.model.appointmentSettings["description"]] : "");
        var temp = $.extend({}, templist[0]);
        if (this.model.timeZone != null && !editOcurrence) {
            var timezone = this.model.timeZone.split(" ")[1].split(":");
            var hour = parseInt(timezone[0]);
            var mins = parseInt(timezone[1]);
            dateMinutes = new Date().getTimezoneOffset();
            var obj = this._minToHour(dateMinutes + ((hour * 60) + mins));
            var newStartDate = new Date(temp.StartTime); var newEndDate = new Date(temp.EndTime);
            temp.StartTime = new Date(newStartDate.setHours(newStartDate.getHours() + obj.h, newStartDate.getMinutes() + obj.m));
            temp.EndTime = new Date(newEndDate.setHours(newEndDate.getHours() + obj.h, newEndDate.getMinutes() + obj.m));
        }
        this._appointmentAddWindow.find("#" + this._id + "startdate").ejDatePicker("option", "value", Globalize.format(this._formatTime(temp[this.model.appointmentSettings["startTime"]]), this._pattern.d));
        this._appointmentAddWindow.find("#" + this._id + "enddate").ejDatePicker("option", "value", Globalize.format(this._formatTime(temp[this.model.appointmentSettings["endTime"]]), this._pattern.d));
        this._appointmentAddWindow.find("#" + this._id + "starttime").ejTimePicker("option", "value", Globalize.format(this._formatTime(temp[this.model.appointmentSettings["startTime"]]), this._pattern.t));
        this._appointmentAddWindow.find("#" + this._id + "endtime").ejTimePicker("option", "value", Globalize.format(this._formatTime(temp[this.model.appointmentSettings["endTime"]]), this._pattern.t));
        if (this.model.timeZone == null) {
            this._appointmentAddWindow.find("#startutc").val("UTC " + Globalize.format(this._formatTime(temp[this.model.appointmentSettings["startTime"]]), "zzz"));
            this._appointmentAddWindow.find("#endutc").val("UTC " + Globalize.format(this._formatTime(temp[this.model.appointmentSettings["endTime"]]), "zzz"));
        }
        else {
            this._appointmentAddWindow.find("#startutc").val(this.model.timeZone);
            this._appointmentAddWindow.find("#endutc").val(this.model.timeZone);
        }
        templist[0][this.model.appointmentSettings["allDay"]] == 1 && (this._appointmentAddWindow.find(".allday ").ejCheckBox({ checked: true }));
        if (templist[0][this.model.appointmentSettings["recurrence"]] == 1) {
            if (editOcurrence == true) {
                (!ej.isNullOrUndefined(templist[0][this.model.appointmentSettings["recurrenceRule"]]) && templist[0][this.model.appointmentSettings["recurrenceRule"]].toString().indexOf("RECUREDITID") != -1) && (this._currentAction = ej.Schedule.Actions.Save);
                this._appointmentAddWindow.find(".recurrence").ejCheckBox({ checked: true, enabled: false });
                this._appointmentAddWindow.find(".e-repeatlbl").addClass('e-disable');
                this._appointmentAddWindow.find("#" + this._id + "summary").css("display", "none");
                this._appointmentAddWindow.find(".e-recurFinalRule").html("");
                this._recRule = templist[0][this.model.appointmentSettings["recurrenceRule"]];
            }
            else {
                this.recurrenceRuleSplit(templist[0][this.model.appointmentSettings["recurrenceRule"]]);
                this._appointmentAddWindow.find(".recurstartdate").ejDatePicker({ value: Globalize.format(this._formatTime(temp[this.model.appointmentSettings["startTime"]]), this._pattern.d) });
                this._appointmentAddWindow.find("#" + this._id + "summary").css("display", "table-row");
                this._appointmentAddWindow.find(".e-recurFinalRule").html(this._showRecurrenceFields(appId));
                this._appointmentAddWindow.find(".recurrence").ejCheckBox({ checked: true });
                    this._appointmentAddWindow.find(".e-recuredit").css("display", "inline");
                }
            }
        if (this.model.categorizeSettings.enable && !ej.isNullOrUndefined(temp[this.model.appointmentSettings["categorize"]])) {
            this._showCategorizeDetails(temp);
        }
            if (!ej.isNullOrUndefined(this.model.resources) && ej.scheduleFeatures.resources)
                this._showMultipleResourceDetails(temp);
            var data = { appointment: templist[0], edit: editOcurrence };
            if (this._trigger("appointmentWindowOpen", data))
                return false;
        this._alldayCheck();
        this._appointmentAddWindow.ejDialog("open");
        this._appointmentAddWindow.find(".subject").focus();
    },
    /**
    * Section for show the recurrence details in the recurrence window.
    * @private
    */
    _showRecurrenceFields: function (appId) {
        var _recurRule = "", _recurOpt = "", _recurType = this._appointmentAddWindow.find('.recurtype');
        _recurRule = ej.isNullOrUndefined(this._rRule.interval) ? this._getLocalizedLabels("Every") + " " : this._getLocalizedLabels("Every") + " " + this._rRule.interval + " ";
        switch (this._rRule.freq) {
            case "DAILY":
                _recurType.ejDropDownList({ selectedItemIndex: 0 });
                _recurRule += this._getLocalizedLabels("RecurrenceDay").toLowerCase();
                break;
            case "WEEKLY":
                _recurType.ejDropDownList({ selectedItemIndex: 1 });
                this._appointmentAddWindow.find('.weekdays').ejCheckBox({ checked: false });
                var _days = this._rRule.weekDays.split(',');
                for (var i = 0; i < _days.length; i++) {
                    this._appointmentAddWindow.find('.e-weekly' + _days[i]).ejCheckBox({ checked: true });
                    _recurOpt += this._dayShortNames[this._dayNames.indexOf(_days[i])] + (i == _days.length - 1 ? "" : ", ");
                }
                _recurRule += this._getLocalizedLabels("RecurrenceWeek").toLowerCase() + " " + this._getLocalizedLabels("On").toLowerCase() + " " + _recurOpt;
                break;
            case "MONTHLY":
                _recurType.ejDropDownList({ selectedItemIndex: 2 });
                if (!ej.isNullOrUndefined(this._rRule.monthDay)) {
                    this._appointmentAddWindow.find('.e-monthdaytype').ejRadioButton({ checked: true });
                    this._appointmentAddWindow.find('.e-monthdate').ejNumericTextbox({ value: parseInt(this._rRule.monthDay) });
                    _recurOpt = parseInt(this._rRule.monthDay);
                }
                else {
                    this._appointmentAddWindow.find('.e-monthposition').ejRadioButton({ checked: true });
                    this._appointmentAddWindow.find('.e-monthsrtday').ejDropDownList({ selectedItemIndex: this._dayNames.indexOf(this._rRule.weekDays) });
                    this._appointmentAddWindow.find('.monthsrt').ejDropDownList({ selectedItemIndex: this._rRule.setPositions - 1 });
                    _recurOpt = this._appointmentAddWindow.find('.monthsrt').ejDropDownList("option", "value").toLowerCase() + " " + this._dayFullNames[this._dayNames.indexOf(this._rRule.weekDays)];
                }
                _recurRule += this._getLocalizedLabels("RecurrenceMonth").toLowerCase() + " " + this._getLocalizedLabels("On").toLowerCase() + " " + _recurOpt;
                break;
            case "YEARLY":
                _recurType.ejDropDownList({ selectedItemIndex: 3 });
                if (!ej.isNullOrUndefined(this._rRule.monthDay)) {
                    this._appointmentAddWindow.find('.e-yearrecurday').ejRadioButton({ checked: true });
                    this._appointmentAddWindow.find('.yearmonth').ejDropDownList({ selectedItemIndex: this._rRule.month - 1 });
                    this._appointmentAddWindow.find('.yeardate').ejNumericTextbox({ value: parseInt(this._rRule.monthDay) });
                    _recurOpt = this._monthNames[this._rRule.month - 1] + " " + parseInt(this._rRule.monthDay);
                }
                else {
                    this._appointmentAddWindow.find('.e-yearrecurposi').ejRadioButton({ checked: true });
                    this._appointmentAddWindow.find('.yearsrtday').ejDropDownList({ selectedItemIndex: this._dayNames.indexOf(this._rRule.weekDays) });
                    this._appointmentAddWindow.find('.yearsrtmonth').ejDropDownList({ selectedItemIndex: this._rRule.month - 1 });
                    this._appointmentAddWindow.find('.yearsrt').ejDropDownList({ selectedItemIndex: this._rRule.setPositions - 1 });
                    _recurOpt = this._appointmentAddWindow.find('.yearsrt').ejDropDownList("option", "value").toLowerCase() + " " + this._dayFullNames[this._dayNames.indexOf(this._rRule.weekDays)] + " " + this._monthNames[this._rRule.month - 1];
                }
                _recurRule += this._getLocalizedLabels("RecurrenceYear").toLowerCase() + " " + this._getLocalizedLabels("On").toLowerCase() + " " + _recurOpt;
                break;
        }
        // Recurrence interavel
        this._appointmentAddWindow.find('.e-recurevery').ejNumericTextbox({ value: ej.isNullOrUndefined(this._rRule.interval) ? 1 : this._rRule.interval });

        //Recurrence end 
        if (!ej.isNullOrUndefined(this._rRule.count)) {
            this._appointmentAddWindow.find(".e-recurafter").ejRadioButton({ checked: true });
            this._appointmentAddWindow.find('.e-recurcount').ejNumericTextbox({ value: this._rRule.count });
            _recurRule += ", " + this._rRule.count + this._getLocalizedLabels("Times").toLowerCase();
        }
        else if (!ej.isNullOrUndefined(this._rRule.until)) {
            this._appointmentAddWindow.find(".e-recuruntil").ejRadioButton({ checked: true });
            this._appointmentAddWindow.find('.e-until').ejDatePicker({ value: new Date(this._rRule.until) });
            _recurRule += " " + this._getLocalizedLabels("Until").toLowerCase() + " " + Globalize.format(new Date(this._rRule.until), this._pattern.d);
        }
        else
            this._appointmentAddWindow.find(".e-recurnoend").ejRadioButton({ checked: true });

        return _recurRule;
    },
    /**
    * Section for handle the closing of recurrence window.
    * @private
    */
    _closerecur: function (args) {
        if (args.model.text == "Cancel") {
            this._appointmentAddWindow.find(".e-recurFinalRule").html() == "" && this._appointmentAddWindow.find("#" + this._id + "recurrence").ejCheckBox({ checked: false });
        }
        else {
            var interval, monthDate, endRecur = "", recurOption = "";
            interval = this._findInterval(this._appointmentAddWindow.find('.e-recurevery'));
            switch (this._rRuleFreq) {
                case "DAILY":
                    this._recRule = "FREQ=" + this._rRuleFreq + ";INTERVAL=" + interval;
                    break;
                case "WEEKDAYS":
                    this._recRule = "FREQ=WEEKLY;BYDAY=MO,TU,WE,TH,FR";
                    break;
                case "WEEKLY":
                    var _weekChkList = this._appointmentAddWindow.find("input[name='weekdays']");
                    var _weekRule = "", _dayName = "", _byDay = "";
                    for (var i = 0; i < _weekChkList.length; i++) {
                        $(_weekChkList[i]).ejCheckBox("option", "checked") == true ? _weekRule += (_weekRule != "" ? "," : "") + this._dayNames[i].toUpperCase() : "";
                        $(_weekChkList[i]).ejCheckBox("option", "checked") == true ? _dayName += (_dayName != "" ? ", " : "") + this._dayShortNames[i] : "";
                    }
                    var _rule = this._getLocalizedLabels("RecurrenceWeek").toLowerCase() + " on " + _dayName;
                    this._appointmentAddWindow.find(".e-recurRule").html(_rule);
                    this._recRule = "FREQ=" + this._rRuleFreq + ";INTERVAL=" + interval + ";" + "BYDAY=" + _weekRule;
                    break;
                case "MONTHLY":
                    if (this._appointmentAddWindow.find('.e-monthdaytype').ejRadioButton("option", "checked")) {
                        monthDate = this._appointmentAddWindow.find('.e-monthdate').ejNumericTextbox("option", "value");
                        this._recRule = "FREQ=" + this._rRuleFreq + ";BYMONTHDAY=" + monthDate + ";INTERVAL=" + interval
                        recurOption = " " + this._getLocalizedLabels("On").toLowerCase() + " " + monthDate;
                    }
                    else {
                        var _setPos = this._appointmentAddWindow.find('.monthsrt').ejDropDownList("option", "selectedItemIndex");
                        var dayName = this._appointmentAddWindow.find('.e-monthsrtday').ejDropDownList("option", "selectedItemIndex");
                        _setPos = _setPos < 4 ? (_setPos + 1) : -1;
                        this._recRule = "FREQ=" + this._rRuleFreq + ";BYDAY=" + this._dayNames[dayName] + ";BYSETPOS=" + _setPos + ";INTERVAL=" + interval;
                        recurOption = " " + this._getLocalizedLabels("On").toLowerCase() + " " + this._appointmentAddWindow.find('.monthsrt').ejDropDownList("option", "value").toLowerCase() + " " + this._dayFullNames[dayName];
                    }
                    break;
                case "YEARLY":
                    if (this._appointmentAddWindow.find("#" + this._id + "yearday").ejRadioButton("option", "checked")) {
                        var _monthIndex = this._appointmentAddWindow.find('.yearmonth').ejDropDownList("option", "selectedItemIndex");
                        var _monthDate = this._findInterval(this._appointmentAddWindow.find('.yeardate'));
                        this._recRule = "FREQ=" + this._rRuleFreq + ";BYMONTHDAY=" + _monthDate + ";BYMONTH=" + (_monthIndex + 1) + ";INTERVAL=" + interval;
                        recurOption = " " + this._getLocalizedLabels("On").toLowerCase() + " " + this._monthNames[_monthIndex] + " " + _monthDate;
                    }
                    else {
                        var _setPos = this._appointmentAddWindow.find('.yearsrt').ejDropDownList("option", "selectedItemIndex");
                        var _monthIndex = this._appointmentAddWindow.find('.yearsrtmonth').ejDropDownList("option", "selectedItemIndex");
                        var dayName = this._appointmentAddWindow.find('.yearsrtday').ejDropDownList("option", "selectedItemIndex");
                        _setPos = _setPos < 4 ? (_setPos + 1) : -1;
                        this._recRule = "FREQ=" + this._rRuleFreq + ";BYDAY=" + this._dayNames[dayName] + ";BYMONTH=" + (_monthIndex + 1) + ";BYSETPOS=" + _setPos + ";INTERVAL=" + interval;
                        recurOption = " " + this._getLocalizedLabels("On").toLowerCase() + " " + this._appointmentAddWindow.find('.yearsrt').ejDropDownList("option", "value").toLowerCase() + " " + this._dayFullNames[dayName] + " " + this._monthNames[_monthIndex];
                    }
                    break;
            }
            if (this._appointmentAddWindow.find('.e-recurafter').ejRadioButton("option", "checked")) {
                var _count = this._findInterval(this._appointmentAddWindow.find('.e-recurcount'));
                this._recRule += ";COUNT=" + _count;
                endRecur = _count ? ", " + _count + " " + this._getLocalizedLabels("Times") : ", ";
            }
            else if (this._appointmentAddWindow.find('.e-recuruntil').ejRadioButton("option", "checked")) {
                var date = Globalize.format(new Date(this._appointmentAddWindow.find('.e-until').ejDatePicker("option", "value")), this._pattern.d);
                this._recRule += ";UNTIL=" + date;
                endRecur = " " + this._getLocalizedLabels("Until").toLowerCase() + " " + date;
            }
            var _every = (this._rRuleFreq != "WEEKDAYS" ? this._getLocalizedLabels("Every") : "") + " " + (interval > 1 ? interval + " " : "");
            var _rRuleStr = _every + this._appointmentAddWindow.find(".e-recurRule").html() + recurOption + endRecur;
            this._appointmentAddWindow.find("#" + this._id + "summary").css("display", "table-row");
            this._appointmentAddWindow.find(".e-recurFinalRule").html(_rRuleStr);
            this._appointmentAddWindow.find(".e-recuredit").css("display", "inline");
            recurOption = "";
        }
        this._appointmentAddWindow.find(".e-recurwindow").toggle();
        this._appointmentAddWindow.find(".e-appwindow").css("margin-left", 700).toggle().animate({ "margin-left": 10 }, 300);
        this._appointmentAddWindow.find('.done').focus();
    },
    /**
    Recurrence startdate value change
    @private
    */
    _recurDateChange: function (args) {
        this._appointmentAddWindow.find("#" + this._id + "startdate").ejDatePicker("option", "value", args.value);
        this._appointmentAddWindow.find("#" + this._id + "enddate").ejDatePicker("option", "value", args.value);
    },

    /**
    * Section to find the interval.
    * @private
    */
    _findInterval: function (numericElement) {
        var interval;
        interval = numericElement.ejNumericTextbox("option", "value");
        return interval = ej.isNullOrUndefined(interval) ? 0 : interval;
    },
    /**
    * Section for cancel the appointment window.
    * @private
    */
    _cancel: function () {
        this._appointmentAddWindow.ejDialog("close");
    },
    /**
    * Section for handle the start and end time in appointment window.
    * @private
    */
    _timeChange: function (e) {
        if (this._currentAction != ej.Schedule.Actions.Add) return false;
        var obj = this._appointmentAddWindow.find("#" + this._id + "starttime").data("ejTimePicker");
        if (obj.showDropdown && obj._id == this._id + "starttime") {
            var endtime = $("#" + this._id + "starttime_popup").find("li.e-active").next("li").html();
            if (!ej.isNullOrUndefined(endtime)) document.getElementById(this._id + "endtime").value = endtime;
            else document.getElementById(this._id + "endtime").value = e.value;
        }
    },
    /**
    * Section for render all the controls used in the schedule.
    * @private
    */
    _renderControls: function (e) {
        var control = this._appointmentAddWindow;
        //control.find('.subject').ejMaskEdit({ enableRTL: this.model.rtl, name: this.model.appointmentSettings["subject"], inputMode: ej.inputMode.Text, width: "100%", cssClass: this.model.cssClass });
        control.find('.allday').ejCheckBox({ enableRTL: this.model.enableRTL, cssClass: this.model.cssClass, change: $.proxy(this._alldayCheck, this) });
        control.find('.startenddate').ejDatePicker({ enableRTL: this.model.enableRTL, width: '100%', value: this.currentDate(), locale: this.model.locale, cssClass: this.model.cssClass });
        control.find('.startendtime').ejTimePicker({ enableRTL: this.model.enableRTL, width: "100%", value: this.currentDate(), locale: this.model.locale, change: $.proxy(this._timeChange, this), cssClass: this.model.cssClass, timeFormat: (this.model.timeMode == "12") ? this._pattern.t : "HH:mm" });
        control.find('.startutc').ejDropDownList({ enableRTL: this.model.enableRTL, dataSource: this.model.timezoneCollection, fields: { text: "text", value: "value" }, width: "100%", change: this._utc, cssClass: this.model.cssClass });
        control.find('.endutc').ejDropDownList({ enableRTL: this.model.enableRTL, dataSource: this.model.timezoneCollection, fields: { text: "text", value: "value" }, width: "100%", change: this._utc, cssClass: this.model.cssClass });
        if (!ej.isNullOrUndefined(this.model.resources) && ej.scheduleFeatures.resources) 
            this._renderAppWindowResources(control);
        if (this.model.categorizeSettings.enable)
            this._renderCategoryItems(control);
        control.find('.recurrence').ejCheckBox({ enableRTL: this.model.enableRTL, change: $.proxy(this._repeat, this), cssClass: this.model.cssClass });
        control.find('.e-btndone').ejButton({ enableRTL: this.model.enableRTL, showRoundedCorner: true, width: "100px", click: $.proxy(this._saveAppointment, this), cssClass: this.model.cssClass });
        control.find('.cancel').ejButton({ enableRTL: this.model.enableRTL, showRoundedCorner: true, width: "100px", click: $.proxy(this._cancel, this), cssClass: this.model.cssClass });
        control.find('.recurtype').ejDropDownList({ enableRTL: this.model.enableRTL, targetID: "recurtypelist", width: "100%", select: $.proxy(this._recurtype, this), cssClass: this.model.cssClass });
        control.find('.dailyrecurtype').ejRadioButton({ enableRTL: this.model.enableRTL, cssClass: this.model.cssClass });
        control.find('.e-dailyeveryday').ejRadioButton({ enableRTL: this.model.enableRTL, checked: true, cssClass: this.model.cssClass });
        control.find('.everydaycount').ejNumericTextbox({ enableRTL: this.model.enableRTL, name: "everydaycount", minValue: 1, value: 1, width: "100px", float: "left", cssClass: this.model.cssClass });
        control.find('.recurends').ejRadioButton({ enableRTL: this.model.enableRTL, cssClass: this.model.cssClass });
        control.find('.e-recurcount').ejNumericTextbox({ enableRTL: this.model.enableRTL, name: "dailyoccurance", minValue: 1, value: 10, width: "65px", cssClass: this.model.cssClass, change: $.proxy(this._endAfterChange, this) });
        control.find('.weekcount').ejNumericTextbox({ enableRTL: this.model.enableRTL, name: "weekcount", minValue: 1, value: 1, width: "100px", cssClass: this.model.cssClass });
        control.find('.e-recurevery').ejNumericTextbox({ enableRTL: this.model.enableRTL, name: "recurevery", minValue: 1, value: 1, width: "100px", cssClass: this.model.cssClass });
        control.find('.weekdays').ejCheckBox({ enableRTL: this.model.enableRTL, cssClass: this.model.cssClass });
        control.find('.e-monthposition').ejRadioButton({ enableRTL: this.model.enableRTL, cssClass: this.model.cssClass });
        control.find('.e-monthdaytype').ejRadioButton({ enableRTL: this.model.enableRTL, checked: true, cssClass: this.model.cssClass });
        control.find('.e-monthdate').ejNumericTextbox({ enableRTL: this.model.enableRTL, name: "monthdate", minValue: 1, maxValue: 31, value: 1, width: "70px", cssClass: this.model.cssClass, change: $.proxy(this._monthDateChange, this) });
        control.find('.monthnumber').ejNumericTextbox({ enableRTL: this.model.enableRTL, name: "monthnumber", minValue: 1, value: 1, width: "70px", cssClass: this.model.cssClass });
        control.find('.yearcount').ejNumericTextbox({ enableRTL: this.model.enableRTL, name: "yearcount", minValue: 1, value: 1, width: "50px", cssClass: this.model.cssClass });
        control.find('.e-yearrecurposi,.e-yearrecurday').ejRadioButton({ enableRTL: this.model.enableRTL, cssClass: this.model.cssClass });
        control.find('.yeardate').ejNumericTextbox({ enableRTL: this.model.enableRTL, name: "yeardate", minValue: 1, maxValue: 31, value: 1, width: "70px", cssClass: this.model.cssClass, change: $.proxy(this._yearDateChange, this) });
        control.find('.recurbutton').ejButton({ enableRTL: this.model.enableRTL, showRoundedCorner: true, width: "100px", click: $.proxy(this._closerecur, this), cssClass: this.model.cssClass });
        control.find('.monthsrt').ejDropDownList({ enableRTL: this.model.enableRTL, targetID: "monthsrtlist", width: "100px", cssClass: this.model.cssClass, change: $.proxy(this._monthDayChange, this) });
        control.find('.e-monthsrtday').ejDropDownList({ enableRTL: this.model.enableRTL, targetID: "monthsrtdaylist", width: "100px", cssClass: this.model.cssClass, change: $.proxy(this._monthDayChange, this) });
        control.find('.yearsrt').ejDropDownList({ enableRTL: this.model.enableRTL, targetID: "yearsrtlist", width: "100px", cssClass: this.model.cssClass, change: $.proxy(this._yearDayChange, this) });
        control.find('.yearmonth').ejDropDownList({ enableRTL: this.model.enableRTL, targetID: "yearmonthlist", width: "100px", cssClass: this.model.cssClass, change: $.proxy(this._yearDateChange, this) });
        control.find('.yearsrtday').ejDropDownList({ enableRTL: this.model.enableRTL, targetID: "yearsrtdaylist", width: "100px", cssClass: this.model.cssClass, change: $.proxy(this._yearDayChange, this) });
        control.find('.yearsrtmonth').ejDropDownList({ enableRTL: this.model.enableRTL, targetID: "yearsrtmonthlist", width: "100px", cssClass: this.model.cssClass, change: $.proxy(this._yearDayChange, this) });
        control.find('.recurstartdate').ejDatePicker({ enableRTL: this.model.enableRTL, locale: this.model.locale, cssClass: this.model.cssClass, change: $.proxy(this._recurDateChange, this) });
        control.find('.e-until').ejDatePicker({ enableRTL: this.model.enableRTL, locale: this.model.locale, cssClass: this.model.cssClass, change: $.proxy(this._endOnChange, this) });
    },
    _renderCategoryItems: function (control) {
        if (this.model.categorizeSettings.allowMultiple) {
            control.find('.e-categorizevalue').ejAutocomplete({
                enableRTL: this.model.enableRTL,
                showPopupButton: true,
                width: "100%", cssClass: this.model.cssClass,
                multiSelectMode: "visualmode",
                dataSource: this.model.categorizeSettings.dataSource, fields: { text: "text", value: "color", key: "id", fontColor: "fontColor" },
                template: this._categorizeTemplate,
            });
        }
        else {
            control.find('.e-categorizevalue').ejAutocomplete({
                enableRTL: this.model.enableRTL,
                showPopupButton: true,
                width: "100%", cssClass: this.model.cssClass,
                multiSelectMode: "none",
                dataSource: this.model.categorizeSettings.dataSource, fields: { text: "text", value: "color", key: "id", fontColor: "fontColor" },
                template: this._categorizeTemplate,
            });
        }
    },
    _appointmentColor: function(record){
        var appointColor = (this.model.categorizeSettings.enable && !ej.isNullOrUndefined(record[this.model.appointmentSettings["categorize"]])) ? this._getCategorizeColor(record) : (!ej.isNullOrUndefined(this.model.resources) && ej.scheduleFeatures.resources) ? this._getResourceColor(record) : { appointData: "", appointCustomcss: "", appointtextcolor: "" };

        if (this.model.categorizeSettings.enable && this.model.categorizeSettings.allowMultiple && !ej.isNullOrUndefined(record[this.model.appointmentSettings["categorize"]])) 
        {
            var categorizeItems = record[this.model.appointmentSettings["categorize"]].split(",");
            var categorizeItemsLength = categorizeItems.length;
            var value = (categorizeItemsLength > 1) ? true : false;
            var multiDiv = this._categorizeMultipleDiv(record, categorizeItems, categorizeItemsLength);
        }
        if (!ej.isNullOrUndefined(multiDiv) && appointColor.appointData.length > 0 && this.model.categorizeSettings.allowMultiple)
            return { appointColor: appointColor, value: value, multiDiv: multiDiv };
        else if (appointColor.appointData.length > 0 && !this.model.categorizeSettings.allowMultiple || !ej.isNullOrUndefined(this.model.resources) && ej.scheduleFeatures.resources)
            return { appointColor: appointColor };
        else {
            appointData = "", appointCustomcss = "", appointtextcolor = "", value = "", appointColor = "";
            return { appointColor: appointColor, value: value, multiDiv: multiDiv };
        }
    },
    _categorizeMultipleDiv: function (record, categorizeItems, categorizeItemsLength)
    {
        //var categorizeItems = record[this.model.appointmentSettings["categorize"]].split(",");
        //var categorizeItemsLength = categorizeItems.length - 1; 
        var renderData = [];
        for (i = 0; i <= categorizeItemsLength-1; i++)
        {
            if (i != categorizeItemsLength-1)
            {
                var dataSet = this.model.categorizeSettings;
                var categorizeDataIndex = this._findResourceIndex(dataSet.dataSource, "id", parseInt(categorizeItems[i]));
                if (ej.isNullOrUndefined(categorizeDataIndex)) {
                    record[this.model.appointmentSettings["categorize"]] = "";
                    return null;
                }
                else
                renderData.push({ text: dataSet.dataSource[categorizeDataIndex][dataSet.text], value: dataSet.dataSource[categorizeDataIndex][dataSet.color], key: dataSet.dataSource[categorizeDataIndex][dataSet.id], fontColor: dataSet.dataSource[categorizeDataIndex][dataSet.fontColor] })
                
            }           
        }
        var template = this._categorizeMultipleTemplate.render({renderData:renderData});
        return template;
    },
   _showCategorizeDetails: function (temp)
   {
        var categorizeItems = temp[this.model.appointmentSettings["categorize"]].split(",");
        for (i = 0; i < categorizeItems.length; i++) {
            this._appointmentAddWindow.find('.e-categorizevalue').data("ejAutocomplete").selectValueByKey(parseInt(categorizeItems[i]));
            }        
    },
   _getCategorizeSaveData: function (categorizeVal, obj, maxId, temp, starttime, endtime)
    {
        var args = [], newAppointment;
        newAppointment = this._addAppData(obj, maxId, new Date(temp.StartDate.setHours(parseInt(starttime[0]), parseInt(starttime[1]))), new Date(temp.EndDate.setHours(parseInt(endtime[0]), parseInt(endtime[1]))), maxId);
        if (categorizeVal.length > 0)
        {
            var categorizeItemsLength = categorizeVal[categorizeVal.length - 1].length - 1; var idValue = "";
            for (var i = 0; i <= categorizeItemsLength; i++) {
                var getId = categorizeVal[categorizeVal.length - 1][i].id;
                  
                    if (i != categorizeItemsLength) {
                            idValue = idValue + getId.toString() + ",";
                        }
                        else {
                            idValue = idValue + getId.toString();
                        }
                      
            }
            newAppointment[this.model.appointmentSettings["categorize"]] = idValue;
        }
        else
        {
            newAppointment[this.model.appointmentSettings["categorize"]] = "";
        }
        args.push(newAppointment);
        return args;
    },
    _renderCategorizeTemplate: function () {

        if (!ej.isNullOrUndefined(this.model.categorizeSettings))
            this._categorizeTemplate = "<div class='e-categorizeTemplate' style='height:15px;margin-right:5px;margin-top:3px;float:left;width:15px;background-color: ${color}'></div> <div>${text}</div>";
    },
    _getCategorizeColor: function (record)
    {
        var categorizeDataIndex = null, appointData = "", appointCustomcss = "", appointtextcolor="";
        var categorizeItems = record[this.model.appointmentSettings["categorize"]].split(",");
        var categorizeItemsLength = categorizeItems.length - 1;
        var dataSet = this.model.categorizeSettings;
        categorizeDataIndex = this._findResourceIndex(dataSet.dataSource, "id", parseInt(categorizeItems[categorizeItemsLength]));
        if (ej.isNullOrUndefined(categorizeDataIndex)) {
            record[this.model.appointmentSettings["categorize"]] = "";
            return { appointData: appointData, appointCustomcss: appointCustomcss, appointtextcolor: appointtextcolor };                
        }
        else {
                appointData = !ej.isNullOrUndefined(dataSet.dataSource[categorizeDataIndex][dataSet.color]) ? dataSet.dataSource[categorizeDataIndex][dataSet.color] : appointData;
                appointtextcolor = !ej.isNullOrUndefined(dataSet.dataSource[categorizeDataIndex][dataSet.fontColor]) ? dataSet.dataSource[categorizeDataIndex][dataSet.fontColor] : appointtextcolor;
           }
        
            return { appointData: appointData, appointCustomcss: appointCustomcss, appointtextcolor: appointtextcolor }
    },

    /**
    * To check the radio button while change the year date.
    * @private
    */
    _yearDateChange: function (e) {
        this._appointmentAddWindow.find("#" + this._id + "yearday").ejRadioButton("option", "checked", true);
    },
    /**
    * To check the radio button while change the year day.
    * @private
    */
    _yearDayChange: function (e) {
        this._appointmentAddWindow.find("#" + this._id + "yearother").ejRadioButton("option", "checked", true);
    },
    /**
    * To check the radio button while change the month date.
    * @private
    */
    _monthDateChange: function (e) {
        this._appointmentAddWindow.find("#" + this._id + "monthday").ejRadioButton("option", "checked", true);
    },
    /**
    * To check the radio button while change the month day.
    * @private
    */
    _monthDayChange: function (e) {
        this._appointmentAddWindow.find("#" + this._id + "monthon").ejRadioButton("option", "checked", true);
    },
    /**
    * To check the radio button while change the end after value.
    * @private
    */
    _endAfterChange: function (e) {
        this._appointmentAddWindow.find("#" + this._id + "repeatendafter").ejRadioButton("option", "checked", true);
    },
    /**
    * To check the radio button while change the end date in recurrence window.
    * @private
    */
    _endOnChange: function (e) {
        this._appointmentAddWindow.find("#" + this._id + "repeatendon").ejRadioButton("option", "checked", true);
    },
    /**
    * Section to convert the minutes to hours.
    * @private
    */
    _minToHour: function (mins) {
        var hours = parseInt(mins / 60);
        var mins = mins % 60;
        var obj = {
            "h": hours,
            "m": mins
        };
        return obj;
    },
    /**
    * Section for handle the closing of appointment window.
    * @private
    */
    _onclose: function (args) {
        this._clearFields();
        this._processFocus();
    },
    
    /**
    * Section for handle printing of appointment save of schedule.
    * @private
        */
        _printAppointment: function (){
            var $element = ej.buildTag("div");
            var query = new ej.Query().where(this.model.appointmentSettings["id"], ej.FilterOperators.equal, this._appId);
            var templist = new ej.DataManager(this._processed).executeLocal(query);
            var appointmentTable = "<table align='center' style='margin-top:50px;margin-left:50;'>";
            var proxy = this;
            $.each(templist[0], function (i, val) {
                if (i != "ParentId" && !ej.isNullOrUndefined(val) && !ej.isNullOrUndefined(i) && i != "null") {
                    if (i == proxy.model.appointmentSettings.startTime || i == proxy.model.appointmentSettings.endTime)
                        val = proxy._formatTime(val);
                    appointmentTable += "<tr style='height:30px'><td style='width:300px'><b>" + i + ":</b></td><td  style='width:600px'>" + val + "</td></tr>";
                }
            });
            appointmentTable += "</table>";
            var ownerTextName = "";
            if (!ej.isNullOrUndefined(this.model.appointmentSettings.resourceFields) && this.model.appointmentSettings.resourceFields != "") {
                var s = proxy.model.appointmentSettings.resourceFields.split(',');
                $.each(this.model.resources[this.model.resources.length - 1].resourceSettings.dataSource, function (i, val) {
                    if (val.id == templist[0][s[s.length - 1]])
                        ownerTextName = val.text;
                });
            }
            ownerTextName = (ownerTextName == "") ? "Unknown" : ownerTextName;
            var appointmentWindowData = "<font size='20' style='margin-top:50px;margin-left:50;font-family:Times New Roman'><b>" + ownerTextName + "</b></font></br><hr style='margin-left:50' size='10'>"
            var printElement = appointmentWindowData + appointmentTable;
            $element.append(printElement);
            ej.print($element);
            return false;
    },
    /**
    * Section for handle the appointment save of schedule.
    * @private
        */
        _saveAppointment: function () {
            var appointments = this._sortAppById(this._processed), maxId, query, query1, appointId, appointGroupId, _editApp;
            if (this._cellClick && this.model.showQuickWindow) {
                var obj = {};
                maxId = appointments.length > 0 ? appointments[appointments.length - 1][this.model.appointmentSettings["id"]] + 1 : 1;
                obj[this.model.appointmentSettings["id"]] = maxId;
                obj["ParentId"] = maxId;
            obj[this.model.appointmentSettings["subject"]] = this._quickAppointWindow.find('.subject').val();
            obj[this.model.appointmentSettings["startTime"]] = new Date(this.cur_StartTime);
            obj[this.model.appointmentSettings["endTime"]] = new Date(this.cur_EndTime);
            if (this.currentView() === "month" || (new Date(this.cur_StartTime).getHours() == 0 && new Date(this.cur_EndTime).getHours() == 23 && new Date(this.cur_EndTime).getMinutes() == 59))
                obj[this.model.appointmentSettings["allDay"]] = true;
            else
                obj[this.model.appointmentSettings["allDay"]] = false;
            obj[this.model.appointmentSettings["recurrence"]] = false;
            if(!ej.isNullOrUndefined(this.model.group) && !ej.isNullOrUndefined(this.model.resources) && ej.scheduleFeatures.resources)
                obj = this._getResourceFields(obj);
            var args = { appointment: obj };
            if (this._trigger("appointmentSaved", args)) {
                this._quickAppointWindow.ejDialog("close");
                return false;
            }
            this._currentAction == ej.Schedule.Actions.Add;
            promise = this._dataManager.insert(args.appointment); // Add new appointment
            var proxy = this;
            if ($.isFunction(promise.promise) && this.dataSource() instanceof ej.DataManager) {
                    promise.done(function (e) {
                        //proxy._appointmentProcessing(args.appointment);
                        proxy._processed.push(args.appointment);
                        proxy._renderSingleApp(obj, args.appointment);
                        proxy._currentAppointmentData.push(args.appointment);
                    });
                    promise.fail(function (e) {
                    });
            }
            else {
                this._processed.push(args.appointment);
                this._renderSingleApp(obj, args.appointment);
            }
                //this._dataManager.insert(args.appointment); // Add new appointment
                //this._processed.push(args.appointment);
            this._cellClick = false;
            this._quickAppointWindow.ejDialog("close");
        }
        else {
            var formelement;
            obj = {};
                temp = {};
                //var appId = this._sortAppById(appointments);
                maxId = (this._currentAction == ej.Schedule.Actions.Add || this._currentAction == ej.Schedule.Actions.EditOcurrence) && appointments.length > 0 ? appointments[appointments.length - 1][this.model.appointmentSettings["id"]] + 1 : parseInt(this._appId);
            formelement = $("#" + this._id + "AppointmentAddEditWindow").find("#" + this._id + "_AddEditForm").get(0); var autoVal = []; var categorizeVal = [];
                for (var index = 0; index < formelement.length; index++) {
                    var columnName = formelement[index].name, $element = $(formelement[index]);
                    if (columnName != undefined) {
                    if (columnName == "")
                        columnName = formelement[index].id.replace(this._id, "");
                    if (columnName != "" && obj[columnName] == null) {
                        var value = formelement[index].value;
                        if (columnName == "Id") {
                            if (value == "")
                                value = parseInt(maxId);
                            else
                                value = parseInt(value);
                        }
                        if ($(formelement[index]).hasClass("e-datepicker"))
                            value = $element.ejDatePicker("option", "value");
                        else if ($(formelement[index]).hasClass("e-datepicker"))
                                value = $element.ejTimePicker("option", "value");
                            else if ($element.data("ejDropDownList"))
                                value = $element.ejDropDownList("model.value");
                            else if ($element.data("ejAutocomplete")) {
                                autoVal.push($element.data("ejAutocomplete").getSelectedItems());
                            }
                            if (columnName != "StartDate" && columnName != "EndDate" && columnName != "StartTime" && columnName != "EndTime") {
                                if (formelement[index].type != "checkbox")
                                    obj[columnName] = value;
                            else
                                obj[columnName] = $(formelement[index]).is(':checked');
                        }
                        else {
                                temp[columnName] = columnName == "StartDate" || columnName == "EndDate" ? new Date(value) : value;
                        }
                    }
                    else {
                        if ($element.data("ejAutocomplete")) {
                            if (columnName == this._id + "_categorize")
                            {
                                categorizeVal.push($element.data("ejAutocomplete").getSelectedItems());
                                if (!this.model.categorizeSettings.allowMultiple)
                                {
                                    if (value.length!=0)
                                    {
                                        var dataSet = this.model.categorizeSettings;
                                        categorizeDataIndex = this._findResourceIndex(dataSet.dataSource, "text", value);
                                        categorizeVal[0].push({ text: dataSet.dataSource[categorizeDataIndex][dataSet.text], color: dataSet.dataSource[categorizeDataIndex][dataSet.color], id: dataSet.dataSource[categorizeDataIndex][dataSet.id], fontColor: dataSet.dataSource[categorizeDataIndex][dataSet.fontColor] });
                                        categorizeVal[0].shift();
                                    }
										
                                }									  
                            }
                            else {
                                autoVal.push($element.data("ejAutocomplete").getSelectedItems());
                            }
                        }
                    }
                }}
                var starttime = this._timeFormat(temp.StartTime).split(":");
            var endtime = this._timeFormat(temp.EndTime).split(":");
            obj[this.model.appointmentSettings["startTime"]] = new Date(temp.StartDate.setHours(parseInt(starttime[0]), parseInt(starttime[1])));
            obj[this.model.appointmentSettings["endTime"]] = new Date(temp.EndDate.setHours(parseInt(endtime[0]), parseInt(endtime[1])));
            (obj[this.model.appointmentSettings["recurrence"]]) && (obj[this.model.appointmentSettings["recurrenceRule"]] = this._recRule);
            // timezone calculation
            if (obj.startutc != "") {
                var utc = obj.startutc.split(" ")[1].split(":");
                var hour = parseInt(utc[0]);
                var minutes = parseInt(utc[1]);
                var localtime = new Date().getTimezoneOffset();
                var local = this._minToHour(localtime + ((hour * 60) + minutes));
                new Date(obj.StartTime.setHours(obj.StartTime.getHours() + local.h, obj.StartTime.getMinutes() + local.m));
                new Date(obj.EndTime.setHours(obj.EndTime.getHours() + local.h, obj.EndTime.getMinutes() + local.m));
            }
            //Update the edited date
            if (this._currentAction == ej.Schedule.Actions.EditOcurrence) {
                var _editApp = this._deleteOcurrence(this._parentId, new Date(temp["StartDate"]));
                new ej.DataManager(this._processed).remove(this.model.appointmentSettings["id"], this._appId);
                obj[this.model.appointmentSettings["recurrence"]] = 1;
                obj[this.model.appointmentSettings["recurrenceRule"]] = _editApp[0][this.model.appointmentSettings["recurrenceRule"]] + ";RECUREDITID=" + _editApp[0][this.model.appointmentSettings["id"]];
            }
            else if (this._currentAction == ej.Schedule.Actions.EditSeries) {
                var query = new ej.Query().where(this.model.appointmentSettings["id"], ej.FilterOperators.equal, this._parentId);
                var _editApp = new ej.DataManager(this._currentAppointmentData).executeLocal(query);
                obj[this.model.appointmentSettings["id"]] = _editApp[0][this.model.appointmentSettings["id"]]; maxId = _editApp[0][this.model.appointmentSettings["id"]]; 
                obj[this.model.appointmentSettings["recurrenceRule"]] = this._recRule == "" ? _editApp[0][this.model.appointmentSettings["recurrenceRule"]] : this._recRule;
            }
            if (this._currentAction != ej.Schedule.Actions.Add) {
                if (this._trigger("appointmentEdited", { appointment: obj })) {
                    this._appointmentAddWindow.ejDialog("close");
                    return false;
                }
            }
            else {
                if (this._trigger("appointmentSaved", { appointment: obj })) {
                    this._appointmentAddWindow.ejDialog("close");
                        return false;
                    }
                }
                var args = [], newAppointment;
                if (!ej.isNullOrUndefined(this.model.resources) && ej.scheduleFeatures.resources)
                    args = this._getResourceSaveData(autoVal, obj, maxId, temp, starttime, endtime);
                else if (!ej.isNullOrUndefined(this.model.categorizeSettings)) {
                    args = this._getCategorizeSaveData(categorizeVal, obj, maxId, temp, starttime, endtime);
                }
                else {
                    var parId = this._currentAction == ej.Schedule.Actions.EditOcurrence || (!ej.isNullOrUndefined(obj[this.model.appointmentSettings["recurrenceRule"]]) && obj[this.model.appointmentSettings["recurrenceRule"]].toString().indexOf("RECUREDITID") != -1) ? this._parentId : maxId;
                    this._maxId = maxId;
                    newAppointment = this._addAppData(obj, this._maxId, new Date(temp.StartDate.setHours(parseInt(starttime[0]), parseInt(starttime[1]))), new Date(temp.EndDate.setHours(parseInt(endtime[0]), parseInt(endtime[1]))), parId);
                    args.push(newAppointment);
                }
                if (this._currentAction == ej.Schedule.Actions.Add || this._currentAction == ej.Schedule.Actions.EditOcurrence) {
                    var proxy = this, promise;
                    promise = this._dataManager.saveChanges({ added: args, changed: [], deleted: [] });
                    if ($.isFunction(promise.promise) && this.dataSource() instanceof ej.DataManager) {
                        promise.done(function (e) {
                            for (var i = 0; i < args.length; i++) {
                                proxy._processed.push(args[i]);
                                proxy._renderSingleApp(obj, args[i]);
                                proxy._currentAppointmentData.push(args[i]);
                            }
                        });
                        promise.fail(function (e) {
                        });
                    }
                    else {
                        for (var i = 0; i < args.length; i++) {
                            this._appointmentProcessing(args[i]);
                            !(this.dataSource() instanceof ej.DataManager) && this._renderSingleApp(obj, args[i]);
                        }
                    }
                }
                else if (this._currentAction == ej.Schedule.Actions.Save || this._currentAction == ej.Schedule.Actions.EditSeries) {
                    var newApp = [], editedApp = [];
                    for (var i = 0; i < args.length; i++) {
                        query = new ej.Query().where(this.model.appointmentSettings["id"], ej.FilterOperators.equal, args[i].ParentId);
                        _editApp = this.dataSource() instanceof ej.DataManager ? ej.DataManager(this._processed).executeLocal(query) : this._dataManager.executeLocal(query);
                        (!ej.isNullOrUndefined(_editApp) && _editApp.length > 0) ? editedApp.push(args[i]) : newApp.push(args[i]);
                    }
                    var promise = this._dataManager.saveChanges({ added: newApp, changed: editedApp, deleted: [] }, this.model.appointmentSettings["id"]);
                    if (editedApp.length > 0) { //updating edited appointments
                        var proxy = this;
                        if ($.isFunction(promise.promise) && this.dataSource() instanceof ej.DataManager) {
                            promise.done(function (e) {
                                for (var i = 0; i < editedApp.length; i++) {
                                    (editedApp[i][proxy.model.appointmentSettings["recurrence"]] || (Globalize.format(new Date(editedApp[i][proxy.model.appointmentSettings["startTime"]]), proxy._pattern.d) < Globalize.format(new Date(editedApp[i][proxy.model.appointmentSettings["endTime"]]), proxy._pattern.d)) && (((new Date(editedApp[i][proxy.model.appointmentSettings["endTime"]]) - new Date(editedApp[i][proxy.model.appointmentSettings["startTime"]])) / 3600000) < 24) && (editedApp[i][proxy.model.appointmentSettings["allDay"]] !== true)) ? proxy._dataProcessing(proxy._currentAppointmentData) : new ej.DataManager(proxy._processed).update(proxy.model.appointmentSettings["id"], editedApp[i]) && proxy.element.find("#Appointment_" + proxy._appId).remove() && proxy.element.find("#categorybar_" + proxy._appId).remove();
                                    proxy._renderSingleApp(editedApp[i], editedApp[i]);
                                }
                            });
                            promise.fail(function (e) {
                            });
                        }
                        else {
                            for (var i = 0; i < editedApp.length; i++) {
                                if (!(editedApp[i][this.model.appointmentSettings["recurrence"]] || (Globalize.format(new Date(editedApp[i][this.model.appointmentSettings["startTime"]]), this._pattern.d) < Globalize.format(new Date(editedApp[i][this.model.appointmentSettings["endTime"]]), this._pattern.d)) && (((new Date(editedApp[i][this.model.appointmentSettings["endTime"]]) - new Date(editedApp[i][this.model.appointmentSettings["startTime"]])) / 3600000) < 24) && (editedApp[i][this.model.appointmentSettings["allDay"]] !== true)))
                                    new ej.DataManager(this._processed).update(this.model.appointmentSettings["id"], editedApp[i]) && this.element.find("#Appointment_" + this._appId).remove();
                                proxy._dataProcessing(proxy._currentAppointmentData);
                                proxy.element.find("#categorybar_" + proxy._appId).remove()
                                proxy._renderSingleApp(editedApp[i], editedApp[i]);
                            }
                        }
                    }
                    if (newApp.length > 0) { // for newly added appointments during editing
                        if ($.isFunction(promise.promise) && this.dataSource() instanceof ej.DataManager) {
                            promise.done(function (e) {
                                for (var i = 0; i < newApp.length; i++) {
                                    proxy._processed.push(newApp[i]);
                                    proxy._renderSingleApp(newApp[i], newApp[i]);
                                }
                            });
                            promise.fail(function (e) {
                            });
                        }
                        else {
                            for (var i = 0; i < newApp.length; i++) {
                                this._appointmentProcessing(newApp[i]);
                                !(this.dataSource() instanceof ej.DataManager) && this._renderSingleApp(newApp[i], newApp[i]);
                            }
                        }
                    }
                }
            }
            this._clearFields();
            this._appointmentAddWindow.ejDialog("close");
            if (this.element.find('div.e-draggableworkarea tr').find('td').hasClass("e-selectedCell"))
            this._processFocus();
        this._recRule = "";

    },

    _renderSingleApp: function (obj, args) {
    this.element.find(".e-prev,.e-next").css("display", "none");
    if (obj[this.model.appointmentSettings["recurrence"]] || this.currentView() === "month" || (((this._formatTime(obj[this.model.appointmentSettings["endTime"]]) - this._formatTime(obj[this.model.appointmentSettings["startTime"]])) / 3600000) > 24))
            this._renderAppointmentAll();
            else {
                var strDate = this._formatTime(obj[this.model.appointmentSettings["startTime"]]);
                var index = this.currentView() === "day" ? 0 : $.inArray(new Date(strDate.getFullYear(), strDate.getMonth(), strDate.getDate()).getTime(), this._dateRender);
                var resIndex = (!ej.isNullOrUndefined(this.model.resources) && !ej.isNullOrUndefined(this.model.group) && ej.scheduleFeatures.resources) ? this._findResourceIndex(this.res1, "id", args[this.model.appointmentSettings.resourceFields.toString().split(',')[this.model.resources.length-1]]) : 0;
                this._getDayAppointments(index, resIndex);
                (this.currentView() !== "month" ? (index >= 0 && obj[this.model.appointmentSettings["allDay"]] && this.model.orientation == "vertical" ? this._setAllDayPos(args, index, resIndex) : this._renderApp(args, index, resIndex)) : this._renderAppointmentMonth(args, this._cellIndex));
                if (this.overlapList.length > 1) {
//                    this._categorybar = true;
                    for (var i = 0; i < this.overlapList.length; i++) {
                        this.element.find("#Appointment_" + this.overlapList[i][this.model.appointmentSettings["id"]]).remove();
                        this.element.find("#categorybar_" + this.overlapList[i][this.model.appointmentSettings["id"]]).remove();
                        this.currentView() == "month" ? this._renderAppointmentMonth(this.overlapList[i], this._cellIndex) : this._renderApp(this.overlapList[i], index, resIndex);
                    }
                }
                this._unWireResizeEvents(); // Unwire appointments resize events.
                this._wireResizeEvents(); // wire appointments resize events.
                this._dragdropAppointments(); // Wire appointments  Drag events.
        }
    },

    /**
    * Convert timeformat into 24 hour mode.
    * @private
    */
    _timeFormat: function (time) {
        var culture = Globalize.culture(this.model.locale);
        var hours = Number(time.match(/^(\d+)/)[1]);
        var minutes = Number(time.match(/:(\d+)/)[1]);
        if (!ej.isNullOrUndefined(time.split(" ")[1])) {
            var AMPM = time.match(/\s(.*)$/)[1];
            if (AMPM == culture.calendar.PM[0] && hours < 12) hours = hours + 12;
            if (AMPM == culture.calendar.AM[0] && hours == 12) hours = hours - 12;
        }
        var sHours = hours.toString();
        var sMinutes = minutes.toString();
        if (hours < 10) sHours = "0" + sHours;
        if (minutes < 10) sMinutes = "0" + sMinutes;
        return sHours + ":" + sMinutes;
    },
    /**
    * Section for handle the escape key process.
    * @private
    */
    _processEscapeKey: function () {
        this._processFocus();
        this.element.find("#" + this._id + "_navigator").ejDatePicker("hide");
    },
    /**
    * Section for handle the focus process of the schedule.
    * @private
    */
    _processFocus: function () {
        var selectApp = this.element.find("div.e-selectedAppointment");
        if (selectApp.length != 0)
            selectApp.focus();
        if (this.element.find("td.e-selectedCell").hasClass("e-workcells")) {
            var oRowCell = this.element.find('div.e-draggableworkarea tr');
            var rowcount = oRowCell.length - 1; // this.get_RowCount();
            var omincolIndex, omaxcolIndex, rowInx, maxrowInx, checkcolindex;
            var curr_View = this.currentView();
            var proxy = this;
            this.element.find('td.e-selectedCell').each(function (index, tdItem) {
                var targetId = $(tdItem).index();
                if (targetId != null && !isNaN(targetId)) {
                    if (omincolIndex == null) {
                        omincolIndex = targetId;
                        rowInx = $(tdItem).parent()[0].rowIndex;
                    }
                    else if (omincolIndex < targetId) {
                        omincolIndex = omincolIndex;
                    }
                    else {
                        if (omincolIndex != targetId) {
                            omincolIndex = targetId;
                            rowInx = $(tdItem).parent()[0].rowIndex;
                        }
                    }
                    if (omaxcolIndex == null) {
                        omaxcolIndex = targetId;
                        if (curr_View != "month") {
                            checkcolindex = parseInt(omaxcolIndex);
                            for (var i = 0; i <= rowcount; i++) {
                                if ($(oRowCell[i].cells[checkcolindex]).hasClass("e-selectedCell")) {
                                    maxrowInx = i;

                                }
                            }
                        }
                    }
                    else if (omaxcolIndex > targetId) {
                        omaxcolIndex = omaxcolIndex;
                    }
                    else {
                        if (omaxcolIndex != targetId) {
                            omaxcolIndex = targetId;
                            if (curr_View != "month") {
                                checkcolindex = parseInt(omaxcolIndex);
                                for (var i = 0; i <= rowcount; i++) {
                                    if ($(oRowCell[i].cells[checkcolindex]).hasClass("e-selectedCell")) {
                                        maxrowInx = i;

                                    }
                                }
                            }
                        }
                    }
                }
            });
            if (curr_View != "month") {
                $(oRowCell[maxrowInx].cells[omaxcolIndex]).attr("tabIndex", 0).focus();
            }
            else {
                var tdsel = oRowCell.find('td.e-selectedCell');
                var tdlen = parseInt(tdsel.length) - 1;
                $(tdsel[tdlen]).attr("tabIndex", 0).focus();
            }
        }
        else {
            this.element.focus();
            this.element.find("td.e-selectedCell").attr("tabIndex", 0).focus();
        }
    },
    /**
    * Section to clear all the fields of apointment window.
    * @private
    */
    _clearFields: function () {
        this._appointmentAddWindow.find('.subject').val("");
        this._appointmentAddWindow.find('.e-description').val("");
        this._appointmentAddWindow.find("#" + this._id + "startdate").ejDatePicker("option", "value", Globalize.format(new Date(), this._pattern.d));
        this._appointmentAddWindow.find("#" + this._id + "enddate").ejDatePicker("option", "value", Globalize.format(new Date(), this._pattern.d));
        this._appointmentAddWindow.find('.startendtime').ejTimePicker({ enabled: true });
        this._appointmentAddWindow.find("#" + this._id + "starttime").ejTimePicker("option", "value", Globalize.format(new Date(), this._pattern.t));
        this._appointmentAddWindow.find("#" + this._id + "endtime").ejTimePicker("option", "value", Globalize.format(new Date(), this._pattern.t));
        this._appointmentAddWindow.find(".recurrence,.weekdays").ejCheckBox({ checked: false, enabled: true });
        this._appointmentAddWindow.find(".e-repeatlbl").removeClass('e-disable');
        this._appointmentAddWindow.find(".allday").ejCheckBox({ checked: false });
        this._appointmentAddWindow.find(".startutc,.endutc").ejDropDownList("clearText");
        this._appointmentAddWindow.find("#" + this._id + "summary").css("display", "none");
        this._appointmentAddWindow.find(".e-recurFinalRule").html("");
        this._appointmentAddWindow.find(".e-recuredit").hide();
        this._appointmentAddWindow.find(".e-categorizevalue").ejAutocomplete("clearText");
    },
    /**
    * To configure the model for numeric text box.
    * @private
    */
    _getNumericModel: function () {
        return numericModel = {
            width: "100px",
            minValue: 1,
            value: 1,
            cssClass: this.model.cssClass,
            enableRTL: this.model.enableRTL,
            localize: this.model.locale
        };
    },
    /**
    * Section to clear all the recurrence fields.
    * @private
    */
    _recurClearFields: function () {
        var list = this._appointmentAddWindow.find("input[name='weekdays']");
        for (var i = 0; i < list.length; i++) {
            $(list[i]).ejCheckBox({ checked: false });
        }
        this._appointmentAddWindow.find('.e-recurevery').ejNumericTextbox({ value: 1 });
        this._appointmentAddWindow.find('.e-recurcount').ejNumericTextbox({ value: 10 });
        this._appointmentAddWindow.find(".e-recurRule").html("");
    },
    /**
    * Section to delete the appointment.
    * @private
    */
    _getApps: function () {
        var dateCount = this._dateRender;
        this._app = [];var appointmentsList = this._processed;
        for (var res = 0; res < this.res1.length; res++) {
            for (var day = 0; day < dateCount.length; day++) {
                for (var app = 0; app < appointmentsList.length; app++) {
                    if ((Globalize.format(new Date(appointmentsList[app][this.model.appointmentSettings["startTime"]]), this._pattern.d) <= Globalize.format(new Date(dateCount[day]), this._pattern.d)) && (Globalize.format(new Date(appointmentsList[app][this.model.appointmentSettings["endTime"]]), this._pattern.d) >= Globalize.format(new Date(dateCount[day]), this._pattern.d)) && (Globalize.format(new Date(appointmentsList[app][this.model.appointmentSettings["startTime"]]), this._pattern.d) == Globalize.format(new Date(appointmentsList[app][this.model.appointmentSettings["endTime"]]), this._pattern.d)) && (Globalize.format(new Date(appointmentsList[app][this.model.appointmentSettings["startTime"]]), this._pattern.d) <= Globalize.format(new Date(appointmentsList[app][this.model.appointmentSettings["endTime"]]), this._pattern.d))) {
                        this._app.push(appointmentsList[app]);

                    }
                }
            }
        }

    },
    _deleteApp: function (id) {
        var args;
        var query = new ej.Query().where(this.model.appointmentSettings["id"], ej.FilterOperators.equal, id);
        var removeItem = ej.DataManager(this._processed).executeLocal(query);
        var p_id = removeItem.length == 0 ? id : this._getAppointmentByParentId(id);
        var promise = this._dataManager.remove(this.model.appointmentSettings["id"], id);
        if(ej.isNullOrUndefined(promise)) { promise = this._dataManager.remove(this.model.appointmentSettings["id"], p_id); }
        var proxy = this;
        if ($.isFunction(promise.promise)) {
            promise.done(function (e) {
                //       removeItem = e;
                (removeItem[0][proxy.model.appointmentSettings["recurrence"]]) && proxy._deleteRecurrence(id);
                new ej.DataManager(proxy._currentAppointmentData).remove(proxy.model.appointmentSettings["id"], id);
                (removeItem[0][proxy.model.appointmentSettings["recurrence"]]) ? proxy._dataProcessing(proxy._currentAppointmentData) && proxy._renderAppointmentAll() : new ej.DataManager(proxy._processed).remove(proxy.model.appointmentSettings["id"], id) && proxy.element.find("#Appointment_" + id).remove() && proxy.element.find("#categorybar_" + id).remove();;
            });
            promise.fail(function (e) {
            });
        }
        else {
            (promise[0][this.model.appointmentSettings["recurrence"]]) && this._deleteRecurrence(id);
            ((promise[0][this.model.appointmentSettings["recurrence"]]) || (((new Date(promise[0][this.model.appointmentSettings["endTime"]]) - new Date(promise[0][this.model.appointmentSettings["startTime"]])) / 3600000) > 24)) ? this._dataProcessing(this._currentAppointmentData) && this._renderAppointmentAll() : new ej.DataManager(this._processed).remove(this.model.appointmentSettings["id"], id) && this.element.find("#Appointment_" + id).remove() && this.element.find("#categorybar_" + id).remove();;
        }
        this._getApps();
        if (this._app.length < 1) {
            if (this.model.enableAppointmentNavigation)
                this._nextPrevButton(this._app);
        }
        args = { appointment: promise };
        this._trigger("appointmentDeleted", args);
    },

    _deleteRecurrence: function (id) {
        var query = ej.Query().where(ej.Predicate(this.model.appointmentSettings["recurrenceRule"], ej.FilterOperators.contains, "RECUREDITID=" + id, true));
        var _newAddList = new ej.DataManager(this._processed).executeLocal(query);
        for (var i = 0; i < _newAddList.length; i++) {
            this._deleteApp(_newAddList[i][this.model.appointmentSettings["id"]]);
            this.element.find("#Appointment_" + id).remove();
            this.element.find("#categorybar_" + id).remove();
        }
    },


    /**
    * Section for handle the appointment resize start process.
    * @private
    */
    _appointmentResizeStart: function (e) {
        if (this.model.enableAppointmentResize) {
            var data = { element: $(e.target).parent() };
            if (this._trigger("resizeStart", data))
                return false;
            e.preventDefault();
            this.model.showQuickWindow && this._quickAppointWindow.ejDialog("close") && this._quickAppDetailsWindow.ejDialog("close");
            this.resizeStart = true;
            if ($(e.target).hasClass("e-lefthandle") || $(e.target).hasClass("e-righthandle")) {
                this.resize = e.pageX;
                this.appoint_width = $(e.target).parent().width();
                this.appoint_left = parseInt($(e.target).parent()[0].style.left);
                this._appointleft = parseInt($($(e.target).parent()[0]).offset().left);
                $(e.target).hasClass("e-lefthandle") ? this._leftResizing = true : this._rightResizing = true;
            }
            else if ($(e.target).hasClass("e-tophandle") || $(e.target).hasClass("e-bottomhandle")) {
                this.resize = e.pageY;
                this.appoint_height = $(e.target).parent().height();
                this.appoint_top = parseInt($(e.target).parent()[0].style.top);
                $(e.target).hasClass("e-tophandle") ? this._topResizing = true : this._bottomResizing = true;
            }
            this._curElmt = $(e.target).parent();
            this.element.find(".e-workcells").removeClass("e-selectedCell");
            this.element.find(".e-workcells").removeClass("e-mousehover");
            this._on($(document), ej.eventType.mouseMove, $.proxy(this._appointmentResizing, this));
            this._on($(document), ej.eventType.mouseUp, $.proxy(this._appointmentResizeStop, this));
            if ($(e.target).hasClass("e-rowcursor") || $(e.target).hasClass("e-columncursor")) // to prevent the drag n drop functionality
                return false;
        }
    },
    /**
    * Section for handle the appointments while resizing.
    * @private
    */
    _appointmentResizing: function (e) {
        var target = $(e.target);
        var data = { element: target.parent() };
        if (this._trigger("resizing", data))
            return false;
        if (this._topResizing) {
            this._curElmt.height(this.appoint_height + this.resize - e.pageY + 3);
            this._curElmt.css("top", this.appoint_top - (this.resize - e.pageY) - 3);
        }
        else if (this._bottomResizing) {
            this._curElmt.height(this.appoint_height + (e.pageY - this.resize) + 2);
        }
        else if (this._rightResizing) {
            this._curElmt.width(this.appoint_width + (e.pageX - this.resize) + 2);
        }
        else if (this._leftResizing) {
            this._curElmt.width(this.appoint_width + this.resize - e.pageX + 3);
            this._curElmt.css("left", this.appoint_left - (this.resize - e.pageX) - 3);
        }
    },
    /**
    * Section for handle the appointment after stop the resize.
    * @private
    */
    _appointmentResizeStop: function (e) {
        try {
            var dragArea = this.element.find(".e-draggableworkarea");
            this.resizeStart = false;
            $(document).unbind(ej.eventType.mouseMove);
                $(document).unbind(ej.eventType.mouseUp);
                var model = this.model.appointmentSettings;
                var appList = this._currentAppointmentData;
                var appointDiv = $(e.target).hasClass("e-appointment") || $(e.target).hasClass("e-monthappointment") || $(e.target).hasClass("e-alldayappointment") ? $(e.target) : $(e.target).parents(".e-appointment,.e-alldayappointment,.e-monthappointment");
                var appId = appointDiv[0].id.split("_")[1];
                var tempApp = new ej.DataManager(this._processed).executeLocal(new ej.Query().where(this.model.appointmentSettings["id"], ej.FilterOperators.equal, appId));
                var appoint = ((tempApp[0][model["recurrence"]] && tempApp[0][model["recurrenceRule"]].toString().indexOf("RECUREDITID") == -1) || (((this._formatTime(tempApp[0][model["endTime"]]) - this._formatTime(tempApp[0][model["startTime"]])) / 3600000) > 24)) ? this._getAppointmentByParentId(appId) : tempApp[0];
                //var appoint = this._getAppointmentByParentId(appId);
                var noOfDays = this.model.currentView === "week" || this.model.currentView === "month" ? 7 : this.model.currentView === "workweek" ? 5 : 1;

                if (appoint != null || appoint !== undefined) {
                var bigApps = this._formatTime(appoint[this.model.appointmentSettings["startTime"]]).getDate() < new Date(appoint[this.model.appointmentSettings["endTime"]]).getDate() ? true : false;
                var newStartTime = this._formatTime(appoint[this.model.appointmentSettings["startTime"]]);
                var newEndTime = this._formatTime(appoint[this.model.appointmentSettings["endTime"]]);
                var StartTime = this._formatTime(appoint[this.model.appointmentSettings["startTime"]]);
                var EndTime = this._formatTime(appoint[this.model.appointmentSettings["endTime"]]);
                var resizeElmntHeight = $(appointDiv[0]).height();

                if (this.currentView() !== "month") {
                    var appTop = $(appointDiv[0]).offset().top; 
                    var cellHeight = this.element.find(".e-workcells").height();
                    var resizeMax = dragArea.height() + dragArea.scrollTop();
                    var topPos = appTop - dragArea.offset().top + dragArea.scrollTop();
                    var resizeMaxTop = (topPos - (cellHeight / ((60 / 30)))) < resizeMax ? topPos : resizeMax - resizeElmntHeight;
                }

                if (this._topResizing) {

                    if (resizeMaxTop > 0) {
                        var newTopTime;
                        newTopTime = resizeMaxTop * 60 / ((60 / 30) * (cellHeight + 1));
                        newStartTime.setHours(parseInt(this.model.startHour) + parseInt(newTopTime / 60));
                        newStartTime.setMinutes(parseInt(newTopTime % 60));
                        newStartTime.setMinutes((Math.round(newStartTime.getMinutes() / 10.0) * 10));
                    }
                    this._topResizing = false;
                }
                else if (this._bottomResizing) {
                    if (resizeMaxTop > 0) {
                        var newTopTime;
                        var newcellCount = ($(appointDiv[0]).height() / cellHeight);
                        newTopTime = (resizeMaxTop + (newcellCount * cellHeight)) * 60 / ((60 / 30) * (cellHeight + 1));
                        newEndTime.setHours(parseInt(this.model.startHour) + parseInt(newTopTime / 60));
                        newEndTime.setMinutes(parseInt(newTopTime % 60));
                        newEndTime.setMinutes((Math.round(newEndTime.getMinutes() / 10.0) * 10));
                    }
                    this._bottomResizing = false;
                }
                else if (this._leftResizing) {
                    if (this.model.orientation == "vertical") {
                        var index = (!this.model.group) ? 0 : this._findResourceIndex(this.res1, "id", appoint[this.model.appointmentSettings.resourceFields.split(',')[this.model.resources.length - 1]]);
                        if (this.currentView() !== "month") {
                            if ($(appointDiv[0]).offset().left > $($(".e-alldaycells")[index * noOfDays]).offset().left) {
                                newStartTime = new Date(this._dateRender[0]);
                                var diffLeft = $(appointDiv[0]).offset().left - $(this.element.find(".e-alldaycells")[index * noOfDays]).offset().left;
                                var diffDays = Math.round(diffLeft / (this.element.find(".e-alldaycells").width() + 1));
                                newStartTime.setDate(new Date(newStartTime).getDate() + diffDays);
                                newStartTime.setHours(new Date(StartTime).getHours());
                                newStartTime.setMinutes(new Date(StartTime).getMinutes());
                            }
                        }
                        else {
                            if ($(appointDiv[0]).offset().left > $($(".e-monthcells")[index * noOfDays]).offset().left) {
                                newStartTime = new Date(new Date(this.model.currentDate).getFullYear(), new Date(this.model.currentDate).getMonth(), 1);
                                var daysInPrevMonth = newStartTime.getDay();
                                newStartTime.setDate(newStartTime.getDate() - daysInPrevMonth);
                                var diffDays = Math.round($(appointDiv[0]).offset().left / (this.element.find(".e-monthcells").width() + 1));
                                // diffDays = (diffDays % 7) - 2;
                                newStartTime.setDate(newStartTime.getDate() + parseInt(diffDays));
                                var newTopDate = Math.round($(appointDiv[0]).offset().top / (this.element.find(".e-monthcells").height()) - 1);
                                newStartTime.setDate(newStartTime.getDate() + (parseInt(newTopDate) * 7));
                                newStartTime.setHours(new Date(StartTime).getHours());
                                newStartTime.setMinutes(new Date(StartTime).getMinutes());
                            }
                        }

                    }
                    else {
                        var newStartTime = new Date(appoint[this.model.appointmentSettings["startTime"]]);
                        var diffLeft = $(appointDiv[0]).offset().left - this._appointleft;
                        if (this.currentView() == "month") {
                            var diffDays = Math.round(diffLeft / (this.element.find(".e-workcells").width() + 1));
                            newStartTime.setDate(new Date(newStartTime).getDate() + diffDays);
                        }
                        else {
                            var diffMinutes = (diffLeft * 60) / ((this.element.find(".e-workcells").width() * 2) + 1);
                            newStartTime.setHours(parseInt(new Date(newStartTime).getHours()) + parseInt(diffMinutes / 60));
                            newStartTime.setMinutes(new Date(newStartTime).getMinutes() + parseInt(diffMinutes % 60));
                            newStartTime.setMinutes(Math.round(newStartTime.getMinutes() / 10.0) * 10);
                        }
                        newStartTime = new Date(newStartTime);
                    }
                    this._leftResizing = false;
                }
                else if (this._rightResizing) {
                    if (this.model.orientation == "vertical") {
                        var index = (!this.model.group) ? 0 : this._findResourceIndex(this.res1, "id", appoint[this.model.appointmentSettings.resourceFields.split(',')[this.model.resources.length - 1]]);
                        if (this.currentView() !== "month") {
                            var diffLeft = appointDiv[0].offsetLeft + $(appointDiv[0]).width(); //  ($(appointDiv[0]).offset().left - $(this.element.find(".e-alldaycells")[(index + 1) * noOfDays]).offset().left) + $(appointDiv[0]).width();
                            var diffDays = Math.floor(diffLeft / (this.element.find(".e-alldaycells").width())) - 1 - (index * noOfDays);
                            var cellsLeft = (index == this.res1.length - 1) ? $(this.element.find(".e-alldaycells")[(index + 1) * noOfDays - 1]).offset().left + this.element.find(".e-alldaycells").width() : $(this.element.find(".e-alldaycells")[(index + 1) * noOfDays]).offset().left;
                            if (cellsLeft >= diffLeft) {
                                newEndTime = new Date(this._dateRender[0]);
                                newEndTime.setDate(new Date(newEndTime).getDate() + diffDays);
                                newEndTime.setHours(new Date(EndTime).getHours());
                                newEndTime.setMinutes(new Date(EndTime).getMinutes());
                            }
                        }
                        else {
                            var appWidth = $(appointDiv[0]).offset().left + $(appointDiv[0]).width();
                            var diffWidth = $(appointDiv[0]).width() - this.appoint_width;
                            var diffDays = Math.round(diffWidth / (this.element.find(".e-monthcells").width()));
                            var diffLeft = $(this.element.find(".e-monthcells")[(index + 1) * noOfDays]).offset().left;
                            if (appWidth > diffLeft)
                                newEndTime.setDate(new Date(newEndTime).getDate() + diffDays);
                            newEndTime.setHours(new Date(EndTime).getHours());
                            newEndTime.setMinutes(new Date(EndTime).getMinutes());
                        }
                    }
                    else {
                        var newEndTime = new Date(EndTime);
                        var diffWidth = $(appointDiv[0]).width() - this.appoint_width;
                        if (this.currentView() == "month") {
                            var diffDays = Math.round(diffWidth / (this.element.find(".e-workcells").width() + 1));
                            newEndTime.setDate(new Date(newEndTime).getDate() + diffDays);
                        }
                        else {
                            var diffMinutes = (diffWidth * 60) / ((this.element.find(".e-workcells").width() * 2) + 1);
                            newEndTime.setHours(new Date(newEndTime).getHours() + parseInt(diffMinutes / 60));
                            newEndTime.setMinutes(new Date(newEndTime).getMinutes() + parseInt(diffMinutes % 60));
                            newEndTime.setMinutes(Math.round(newEndTime.getMinutes() / 10.0) * 10);
                        }
                        newEndTime = new Date(newEndTime);
                    }
                    this._rightResizing = false;
                }

                //Recurrence applointment
                if (appoint[model["recurrence"]] == 1 && appoint[model["recurrenceRule"]].toString().indexOf("RECUREDITID") == -1) {
                    var _newAddList = {};
                    var query = new ej.Query().where(this.model.appointmentSettings["id"], ej.FilterOperators.equal, appId);
                    var _appList = new ej.DataManager(this._processed).executeLocal(query)[0];
                    var _newAddList = {};
                    $.extend(_newAddList, appoint);
                    var _currStr = new Date(_appList[model["startTime"]]);
                    var _currEnd = new Date(_appList[model["endTime"]]);
                    var _resizeStrTime = new Date(_currStr.getFullYear(), _currStr.getMonth(), _currStr.getDate()).setHours(newStartTime.getHours(), newStartTime.getMinutes(), newStartTime.getMilliseconds()); //newStartTime.setTime(new Date(_newAddList[model["startTime"]]).getTime());
                    var _resizeEndTime = new Date(_currEnd.getFullYear(), _currEnd.getMonth(), _currEnd.getDate()).setHours(newEndTime.getHours(), newEndTime.getMinutes(), newEndTime.getMilliseconds()); //newStartTime.setTime(new Date(_newAddList[model["startTime"]]).getTime());
                    this._deleteOcurrence(parseInt(appoint[model["id"]]), new Date(_appList[model["startTime"]]));
                    this._processed = this._sortAppById(this._processed);
                    var maxId = this._processed.length > 0 ? this._processed[this._processed.length - 1][model["id"]] + 1 : 1;
                    _newAddList[model["recurrenceRule"]] = appoint[model["recurrenceRule"]] + ";RECUREDITID=" + _newAddList[model["id"]];
                    _newAddList[model["startTime"]] = new Date(_resizeStrTime);
                    _newAddList[model["endTime"]] = new Date(_resizeEndTime);
                    _newAddList[model["id"]] = maxId;
                    var args = { data: _newAddList };
                    promise = this._dataManager.insert(args.data); // Add new appointment
                    var proxy = this;
                    if ($.isFunction(promise.promise) && this.dataSource() instanceof ej.DataManager) {
                        promise.done(function (e) {
                            proxy._processed.splice(proxy._processed.indexOf(_appList), 1);
                            proxy._processed.push(args.data);
                            proxy._renderSingleApp(_newAddList, _newAddList);
                        });
                        promise.fail(function (e) {
                            proxy._renderAppointmentAll();
                        });
                    }
                    else {
                        this._processed.splice(this._processed.indexOf(_appList), 1);
                        this._processed.push(args.data);
                        this._renderSingleApp(_newAddList, _newAddList);
                    }
                }
                else {
                    var query = new ej.Query().where(this.model.appointmentSettings["id"], ej.FilterOperators.equal, appoint[this.model.appointmentSettings["id"]]);
                    var _updateList = {}, object = new ej.DataManager(this._currentAppointmentData).executeLocal(query);
                    $.extend(_updateList, object);
                    _updateList[0][this.model.appointmentSettings["startTime"]] = newStartTime;
                    _updateList[0][this.model.appointmentSettings["endTime"]] = newEndTime;
                    promise = this._dataManager.update(this.model.appointmentSettings["id"], _updateList[0]);
                    var proxy = this;
                    if (promise && $.isFunction(promise.promise) && this.dataSource() instanceof ej.DataManager) {
                        promise.done(function (e) {
                            new ej.DataManager(proxy._processed).update(proxy.model.appointmentSettings["id"], _updateList);
                            proxy.element.find("#Appointment_" + _updateList[0][proxy.model.appointmentSettings["id"]]).remove();
                            proxy.element.find("#categorybar_" + _updateList[0][proxy.model.appointmentSettings["id"]]).remove();
                            proxy._renderSingleApp(_updateList[0], _updateList[0]);
                        });
                        promise.fail(function (e) {
                            proxy._renderAppointmentAll();
                        });
                    }
                    else {
                        new ej.DataManager(this._processed).update(proxy.model.appointmentSettings["id"], _updateList);
                        this.element.find("#Appointment_" + appId).remove();
                        if (((this._formatTime(tempApp[0][model["endTime"]]) - this._formatTime(tempApp[0][model["startTime"]])) / 3600000) > 24) {
                            proxy._dataProcessing(appList);
                        }
                        proxy._renderSingleApp(_updateList[0], _updateList[0]);
                    }

                }
                var data = { appointment: appoint, target: e };
                if (this._trigger("resizeStop", data))
                    return false;
                //this._dataProcessing(appList);
                //this._renderSingleApp(appList[0], { data: appList });
                //this._renderAppointmentAll();
            }
            this.element.find("div.e-tophandle,div.e-bottomhandle").removeClass("e-rowcursor");
            this.element.find("div.e-lefthandle,div.e-righthandle").removeClass("e-columncursor");
        } catch (e) {

        }
    },
    /**
    * Split the Recurrence rule string.
    * @private
    */
    // Split the Recurrence rule string 
    recurrenceRuleSplit: function (rRule) {
        var _rule, _rRuleSplit = rRule.split(";");
        for (var r = 0; r < _rRuleSplit.length; r++) {
            _rule = _rRuleSplit[r].split("=");
            switch ($.trim(_rule[0]).toUpperCase()) {
                case "FREQ":
                    this._rRule.freq = $.trim(_rule[1]).toUpperCase();
                    break;
                case "INTERVAL":
                    this._rRule.interval = parseInt(_rule[1], 10);
                    break;
                case "COUNT":
                    this._rRule.count = parseInt(_rule[1], 10);
                    break;
                case "UNTIL":
                    this._rRule.until = new Date(_rule[1]);
                    break;
                case "BYDAY":
                    this._rRule.weekDays = _rule[1];
                    break;
                case "BYMONTHDAY":
                    this._rRule.monthDay = parseInt(_rule[1], 10);
                    break;
                case "BYMONTH":
                    this._rRule.month = parseInt(_rule[1], 10);
                    break;
                case "BYSETPOS":
                    this._rRule.setPositions = parseInt(_rule[1], 10);
                    break;
                case "WKST":
                    this._rRule.weekStart = _rule[1];
                    break;
                case "EXDATE":
                    var _exDates = _rule[1].split(",");
                    this._rRule.exDate = [];
                    for (var i = 0; i < _exDates.length; i++) {
                        this._rRule.exDate[i] = Globalize.format(new Date(_exDates[i]), this._pattern.d);
                    }
                    //this._rRule.exDate = _rule[1];
                    break;
                case "RECUREDITID":
                    this._rRule.recurEditId = parseInt(_rule[1]);
                    break;
            }
        }
        return this._rRule;
    },
    /**
    * Section for handle the drag and drop appointments.
    * @private
    */
    _dragdropAppointments: function () {
        this._topResizing = false;
        this._bottomResizing = false;
        this._leftResizing = false;
        this._rightResizing = false;
        if (this.model.allowDragDrop) {
            var proxy = this;

            // normal appointments
            var $dragableElements = this.element.find(".e-appointment");
            $dragableElements.ejDraggable({
                handle: ".e-appointment",
                dragArea: proxy.element.find("div.e-draggableworkarea"),
                cursorAt: { top: 20, left: 0 },
                dragStart: function (args) {
                    proxy._dragStart(args);
                },
                drag: function (args) {
                    proxy._dragOver(args);
                },
                dragStop: function (args) {
                    proxy._dragStop(args);
                },
                helper: function (event, ui) {
                    var appointdiv = $(event.sender.target).hasClass("e-appointment") ? $(event.sender.target) : $(event.sender.target).parents(".e-appointment");
                    if (appointdiv.hasClass("e-appointment"))
                        return appointdiv.addClass("e-draggableAppointment").css("width", appointdiv[0].offsetWidth + "px").appendTo(proxy.element);
                }

            });

            // allday appointments
                var $dragableElements = this.element.find(".e-alldayappointment");
                $dragableElements.ejDraggable({
                    handle: ".e-alldayappointment",
                    dragArea: proxy.element.find(".e-headrealldaytable tr")[(!ej.isNullOrUndefined(this.model.group) && !ej.isNullOrUndefined(this.model.resources)) ? this.model.group.resources.length + 1 : 2],
                    cursorAt: { top: 17, left: 0 },
                    dragStart: function (args) {
                        proxy._dragStart(args);
                },
                drag: function (args) {
                    proxy._dragOver(args);
                },
                dragStop: function (args) {
                    proxy._dragStop(args);
                },
                helper: function (event, ui) {
                    var appointdiv = $(event.sender.target).hasClass("e-alldayappointment") ? $(event.sender.target) : $(event.sender.target).parents(".e-alldayappointment");
                    if (appointdiv.hasClass("e-alldayappointment"))
                        return appointdiv.addClass("e-draggableAppointment").css("width", appointdiv[0].offsetWidth + "px").appendTo(proxy.element);
                }

            });

            // month appointments
            var $dragableElements = this.element.find(".e-monthappointment");
            $dragableElements.ejDraggable({
                handle: ".e-monthappointment",
                dragArea: proxy.element.find(".e-workareadiv"),
                cursorAt: { top: 17, left: 0 },
                dragStart: function (args) {
                    proxy._dragStart(args);
                },
                drag: function (args) {
                    proxy._dragOver(args);
                },
                dragStop: function (args) {
                    proxy._dragStop(args);
                },
                helper: function (event, ui) {
                    var appointdiv = $(event.sender.target).hasClass("e-monthappointment") ? $(event.sender.target) : $(event.sender.target).parents(".e-monthappointment");
                    if (appointdiv.hasClass("e-monthappointment"))
                        return appointdiv.addClass("e-draggableAppointment").css("width", appointdiv[0].offsetWidth + "px").appendTo(proxy.element);
                }

            });
        }
    },
    /**
    * Section for handle the drag start of appointment.
    * @private
    */
    _dragStart: function (args) {
        var target = args.element;
        var data = { target: target };
        this._trigger("dragStart", data);
        this.model.showQuickWindow && (this._quickAppointWindow.ejDialog("close") && this._quickAppDetailsWindow.ejDialog("close"));
    },
    /**
    * Section for handle the drag over of appointment.
    * @private
    */
    _dragOver: function (args) {
        var $target = $(args.element);
        var data = { target: $target };
        this._trigger("dragOver", data);
        this.element.find(".e-workcells").removeClass("e-selectedCell");
        this.element.find(".e-workcells").removeClass("e-mousehover");
    },
    /**
    * Section for handle the drag stop of appointment.
        * @private
        */
    _dragStop: function (args) {
            this.model.showQuickWindow && (this._quickAppDetailsWindow.ejDialog("close"));
            var proxy = this;
            var model = proxy.model.appointmentSettings; var dragArea = proxy.element.find(".e-draggableworkarea");;
            var appId = args.element[0].id.split("_")[1];
            var tempApp = new ej.DataManager(this._processed).executeLocal(new ej.Query().where(this.model.appointmentSettings["id"], ej.FilterOperators.equal, appId));
            var appoint = ((tempApp[0][model["recurrence"]] && tempApp[0][model["recurrenceRule"]].toString().indexOf("RECUREDITID") == -1) || (((this._formatTime(tempApp[0][model["endTime"]]) - this._formatTime(tempApp[0][model["startTime"]])) / 3600000) > 24)) ? this._getAppointmentByParentId(appId) : tempApp[0];
            //var appoint = proxy._getAppointmentByParentId(appId);
            var appList = this._currentAppointmentData;
            if (this.model.orientation == "vertical") {
                if ($(args.element[0]).hasClass("e-appointment")) {
                    var dragMax = dragArea.height() + dragArea.scrollTop();
                    var dragElmntHeight = args.element[0].offsetHeight;
                    var cellHeight = this.element.find(".e-workcells").height();
                    var topPos = $(args.element[0]).offset().top - dragArea.offset().top + dragArea.scrollTop();
                    var dragMaxTop = (topPos - (cellHeight / (60 / 30))) < dragMax ? topPos : dragMax - dragElmntHeight;
                    if (dragMaxTop > 0) {
                        var newTopTime;
                        newTopTime = dragMaxTop * 60 / ((60 / 30) * (cellHeight + 1));

                        var twoDayApps = proxy._formatTime(appoint[proxy.model.appointmentSettings["startTime"]]).getDate() < proxy._formatTime(appoint[proxy.model.appointmentSettings["endTime"]]).getDate() ? true : false;
                        var startTime = proxy._formatTime(appoint[proxy.model.appointmentSettings["startTime"]]);
                        var endTime = proxy._formatTime(appoint[proxy.model.appointmentSettings["endTime"]]);
                        var duration = (endTime - startTime) / 1000;
                        var hours = Math.floor(duration / 3600);
                        var minutes = Math.floor((duration % 3600) / 60);
                        var newStartTime, newEndTime, cellstrtime, cellendtime;

                        if (proxy.currentView() !== "month") {
                            var firstDayOfWeek = new Date(proxy._firstweekdate());
                            if (proxy.currentView() === "workweek") {
                                var _day = new Date(firstDayOfWeek).getDay();
                                var _dayAdd = _day == 0 ? 1 : 0;
                                firstDayOfWeek.setDate(firstDayOfWeek.getDate() + _dayAdd);
                            }
                            newStartTime = firstDayOfWeek;

                            //finding cellindex by left calculation
                            var dragAreaLength = dragArea.width();
                            var noOfCols = proxy.currentView() === "week" ? 7 : proxy.currentView() === "workweek" ? 5 : 1;
                            noOfCols = !ej.isNullOrUndefined(proxy.model.group) ? noOfCols * proxy.res1.length : noOfCols;
                            var cellWidth = Math.round(dragAreaLength / noOfCols);
                            var originalLeft = args.element[0].offsetLeft - dragArea.offset().left;
                            var cellIndex = Math.round(originalLeft / cellWidth);
                            newStartTime = (this.model.enableRTL)?new Date(this.dateRender[noOfCols-cellIndex-1]):new Date(this.dateRender[cellIndex]);
                            newEndTime = newStartTime;
                        }
                        else {
                            newStartTime = proxy._formatTime(appoint[proxy.model.appointmentSettings["startTime"]]);
                            newEndTime = new Date(newStartTime);
                        }

                        if (twoDayApps) {
                            newStartTime = proxy._getNewStartTime(newTopTime, newStartTime);
                            cellstrtime = new Date(newStartTime);
                            newEndTime = proxy._getNewEndTime(newStartTime, hours, minutes);
                            cellendtime = new Date(newEndTime);
                        }
                        else {
                            newStartTime.setHours(parseInt(proxy.model.startHour) + Math.round(newTopTime / 60));
                            newStartTime.setMinutes(parseInt(newTopTime % 60));
                            newStartTime = (new Date(newStartTime).getMinutes() >= 30) ? new Date(new Date(newStartTime).setHours(new Date(newStartTime).getHours() - 1)) : new Date(newStartTime);
                            newStartTime.setMinutes(Math.round(newStartTime.getMinutes() / 10.0) * 10);
                            cellstrtime = new Date(newStartTime);

                            var totMin = (proxy._formatTime(appoint[proxy.model.appointmentSettings["endTime"]]).getHours() * 60 + proxy._formatTime(appoint[proxy.model.appointmentSettings["endTime"]]).getMinutes()) -
                     (proxy._formatTime(appoint[proxy.model.appointmentSettings["startTime"]]).getHours() * 60 + proxy._formatTime(appoint[proxy.model.appointmentSettings["startTime"]]).getMinutes());


                            newEndTime.setHours(new Date(newStartTime).getHours() + parseInt(totMin / 60));
                            newEndTime.setMinutes(new Date(newStartTime).getMinutes() + parseInt(totMin % 60));
                            newEndTime.setMinutes(Math.round(newEndTime.getMinutes() / 10.0) * 10);
                            cellendtime = new Date(newEndTime);
                        }
                    }
                    else {
                        var dragAreaLength = dragArea.width();
                        var noOfCols = proxy.currentView() === "week" ? 7 : proxy.currentView() === "workweek" ? 5 : 1;
                        noOfCols = !ej.isNullOrUndefined(proxy.model.group) ? noOfCols * proxy.res1.length : noOfCols;
                        var cellWidth = Math.round(dragAreaLength / noOfCols);
                        var originalLeft = args.element[0].offsetLeft - dragArea.offset().left;
                        var cellIndex = Math.round(originalLeft / cellWidth);
                        newStartTime = (this.model.enableRTL) ? new Date(this.dateRender[noOfCols - cellIndex - 1]) : new Date(this.dateRender[cellIndex]);
                        newStartTime.setHours(0);
                        newStartTime.setMinutes(0);
                        cellstrtime = new Date(newStartTime);
                        newEndTime = newStartTime;
                        newEndTime.setHours(23);
                        newEndTime.setMinutes(59);
                        cellendtime = new Date(newEndTime);
                        var allday = true;
                    }
                }
                else if ($(args.element[0]).hasClass("e-alldayappointment")) {

                    var appoint = proxy._getAppointmentByParentId(appId);
                    newStartTime = this._formatTime(appoint[this.model.appointmentSettings["startTime"]]);
                    newEndTime = this._formatTime(appoint[this.model.appointmentSettings["endTime"]]);
                    var diffLeft = args.element[0].offsetLeft - this.element.find(".e-alldaycells").offset().left - 1; // ui.position.left - ui.originalPosition.left;
                    diffLeft = (this.model.enableRTL) ? Math.abs(diffLeft) : diffLeft;
                    var cellIndex = Math.round(diffLeft / (this.element.find(".e-alldaycells").width() + 1));
                    var count = Math.round($(args.element[0]).width() / (this.element.find(".e-alldaycells").width() + 1));
                    var overall = Math.ceil((newEndTime - newStartTime) / (1000 * 60 * 60 * 24));
                    var endInd = Math.round($(args.element[0]).width() / (this.element.find(".e-alldaycells").width() + 1));
                    var cols = proxy.currentView() === "week" ? 7 : proxy.currentView() === "workweek" ? 5 : 1;
                    if (cellIndex < 0) {
                        if (newStartTime <= new Date(this._dateRender[0])) {
                            cellstrtime = new Date(newStartTime);
                            cellstrtime.setDate(cellstrtime.getDate() + cellIndex);
                            cellstrtime.setHours(new Date(proxy._formatTime(appoint[proxy.model.appointmentSettings["startTime"]])).getHours());
                            cellstrtime.setMinutes(new Date(proxy._formatTime(appoint[proxy.model.appointmentSettings["startTime"]])).getMinutes());
                            cellstrtime = new Date(cellstrtime);
                        }
                        else {
                            var startdate = new Date(proxy._dateRender[0]);
                            startdate.setDate(startdate.getDate() + cellIndex);
                            startdate.setHours(new Date(proxy._formatTime(appoint[proxy.model.appointmentSettings["startTime"]])).getHours());
                            startdate.setMinutes(new Date(proxy._formatTime(appoint[proxy.model.appointmentSettings["startTime"]])).getMinutes());
                            cellstrtime = new Date(startdate);
                        }
                    } else {
                        if (newStartTime < new Date(this._dateRender[0])) {
                            var Index = (cellIndex > cols - 1) ? (cellIndex % cols) : cellIndex;
                            newStartTime.setDate(new Date(proxy._formatTime(appoint[proxy.model.appointmentSettings["startTime"]])).getDate() + Index);
                            newStartTime.setHours(new Date(proxy._formatTime(appoint[proxy.model.appointmentSettings["startTime"]])).getHours());
                            newStartTime.setMinutes(new Date(proxy._formatTime(appoint[proxy.model.appointmentSettings["startTime"]])).getMinutes());
                            cellstrtime = new Date(newStartTime);
                        }
                        else { 
                            var Index = (cellIndex > cols - 1) ? (cellIndex % cols) : cellIndex;
                            cellstrtime = new Date(new Date(this._dateRender[Index]));
                            cellstrtime.setHours(new Date(proxy._formatTime(appoint[proxy.model.appointmentSettings["startTime"]])).getHours());
                            cellstrtime.setMinutes(new Date(proxy._formatTime(appoint[proxy.model.appointmentSettings["startTime"]])).getMinutes());
                        }
                    }
                    var starttime = new Date(cellstrtime);
                    starttime.setDate(starttime.getDate() + overall - 1);
                    starttime.setHours(new Date(proxy._formatTime(appoint[proxy.model.appointmentSettings["endTime"]])).getHours());
                    starttime.setMinutes(new Date(proxy._formatTime(appoint[proxy.model.appointmentSettings["endTime"]])).getMinutes());
                    cellendtime = new Date(starttime);
                }

                else if ($(args.element[0]).hasClass("e-monthappointment")) {
                    var oldStartTime = proxy._formatTime(appoint[proxy.model.appointmentSettings["startTime"]]);
                    var oldEndTime = proxy._formatTime(appoint[proxy.model.appointmentSettings["endTime"]]);
                    newStartTime = new Date(new Date(proxy.model.currentDate).getFullYear(),
                                              new Date(proxy.model.currentDate).getMonth(), 1);
                    var daysInPrevMonth = newStartTime.getDay();
                    newStartTime.setDate(newStartTime.getDate() - daysInPrevMonth);
                    var diffLeft = args.element[0].offsetLeft - this.element.find(".e-monthcells").offset().left;
                    diffLeft = (this.model.enableRTL) ? Math.abs(diffLeft) : diffLeft;
                    var cellIndex = Math.round(diffLeft / (this.element.find(".e-monthcells").width() + 1));                    
                    var diffTop = args.element[0].offsetTop - this.element.find(".e-monthcells").offset().top;
                    var newTopDate = Math.round(diffTop / (this.element.find(".e-monthcells").height() - 1));
                    //if (!ej.isNullOrUndefined(this.model.group)) {
                    //    cellIndex = cellIndex - 1; newTopDate = newTopDate - 2;
                    //}
                    var index = (newTopDate * 7) + (cellIndex - (Math.floor(cellIndex / 7) * 7));
                    newStartTime = new Date(proxy._dateRender[index]);
                    newStartTime.setHours(oldStartTime.getHours(), oldStartTime.getMinutes(), oldStartTime.getSeconds());
                    cellstrtime = new Date(newStartTime);
                    var diffTime = newStartTime.getTime() - new Date(oldStartTime).getTime();
                    newEndTime = new Date(new Date(oldEndTime).getTime() + diffTime);
                    cellendtime = new Date(newEndTime);
                }
            }
            else if (this.model.orientation == "horizontal" && ej.scheduleFeatures.horizontal) {
                var diffTimeLeft;
                var cellstrtime = new Date(appoint[proxy.model.appointmentSettings["startTime"]]);
                var cellendtime = new Date(appoint[proxy.model.appointmentSettings["endTime"]]);
                var colIndex = Math.floor((args.element[0].offsetLeft - dragArea.offset().left) / this.element.find(".e-workcells").width());
                var rowIndex = Math.floor((args.element[0].offsetTop - dragArea.offset().top) / (this.element.find(".e-workcells").outerHeight())); 
                var target = $(this.element.find(".e-workcellstab tr")[rowIndex]).find("td")[colIndex];

                if (!$(target).hasClass("e-resourceheadercells")) {
                    var newStartTime = new Date(this._dateRender[0]); // new Date(appoint[proxy.model.appointmentSettings["startTime"]]);

                    diffTimeLeft = ((args.element[0].offsetLeft + dragArea.scrollLeft() - dragArea.offset().left) * (30)) / this.element.find(".e-workcells").outerWidth();

                    newStartTime.setHours(parseInt(proxy.model.startHour) + Math.round(diffTimeLeft / 60));
                    newStartTime.setMinutes(parseInt(diffTimeLeft % 60));
                    newStartTime = (new Date(newStartTime).getMinutes() >= 30) ? new Date(new Date(newStartTime).setHours(new Date(newStartTime).getHours() - 1)) : new Date(newStartTime);
                    newStartTime.setMinutes(Math.round(newStartTime.getMinutes() / 10.0) * 10);
                    cellstrtime = new Date(newStartTime);

                    var startTime = proxy._formatTime(appoint[proxy.model.appointmentSettings["startTime"]]);
                    var endTime = proxy._formatTime(appoint[proxy.model.appointmentSettings["endTime"]]);
                    var duration = (endTime - startTime) / 1000;
                    var hours = Math.floor(duration / 3600);
                    var minutes = Math.floor((duration % 3600) / 60);
                    newEndTime = proxy._getNewEndTime(newStartTime, hours, minutes);
                    cellendtime = new Date(newEndTime);

                    this._cellIndex = Math.floor((args.element[0].offsetLeft + dragArea.scrollLeft() - dragArea.offset().left) / this.element.find(".e-workcells").width());
                    var cellIndex = rowIndex;
                    if ($(args.element[0]).hasClass("e-monthappointment")) {
                        var oldStartTime = new Date(appoint[proxy.model.appointmentSettings["startTime"]]);
                        var oldEndTime = new Date(appoint[proxy.model.appointmentSettings["endTime"]]);

                        newStartTime = new Date(proxy.monthDays[this._cellIndex]);
                        newStartTime.setHours(oldStartTime.getHours(), oldStartTime.getMinutes(), oldStartTime.getSeconds());
                        cellstrtime = new Date(newStartTime);
                        var diffTime = newStartTime.getTime() - new Date(oldStartTime).getTime();
                        newEndTime = new Date(new Date(oldEndTime).getTime() + diffTime);
                        cellendtime = new Date(newEndTime);
                    }
                }
            }
            //Recurrence appointment
            if (appoint[model["recurrence"]] == 1 && appoint[model["recurrenceRule"]].indexOf("RECUREDITID") == -1) {
                var query = new ej.Query().where(this.model.appointmentSettings["id"], ej.FilterOperators.equal, appId);
                var _appList = new ej.DataManager(this._processed).executeLocal(query)[0];
                var _newAddList = {};
                $.extend(_newAddList, appoint);
                proxy._deleteOcurrence(parseInt(appoint[model["id"]]), _appList[model["startTime"]]);
                this._processed = this._sortAppById(this._processed);
                var maxId = this._processed.length > 0 ? this._processed[this._processed.length - 1][model["id"]] + 1 : 1;
                _newAddList[model["recurrenceRule"]] = appoint[model["recurrenceRule"]] + ";RECUREDITID=" + _newAddList[model["id"]];
                _newAddList[model["startTime"]] = cellstrtime;
                _newAddList[model["endTime"]] = cellendtime;
                _newAddList[model["id"]] = maxId;
                _newAddList["ParentId"] = maxId;
                if (!ej.isNullOrUndefined(allday)) _newAddList[model["allDay"]] = allday;
                if (!ej.isNullOrUndefined(this.model.group) && ej.scheduleFeatures.resources)
                    _newAddList = this._getDragDropResourceData(proxy, cellIndex, _newAddList);
                var argsAdd = { data: _newAddList };
                promise = proxy._dataManager.insert(argsAdd.data); // Add new appointment
                if ($.isFunction(promise.promise) && proxy.dataSource() instanceof ej.DataManager) {
                    promise.done(function (e) {
                        proxy._processed.splice(proxy._processed.indexOf(_appList), 1);
                        proxy._appointmentProcessing(_newAddList);
                        proxy._renderSingleApp(_newAddList, argsAdd.data);
                    });
                    promise.fail(function (e) {
                        proxy._renderAppointmentAll();
                    });
                }
                else {
                    proxy._processed.splice(proxy._processed.indexOf(_appList), 1);
                    this._processed.push(argsAdd.data);
                    this._renderSingleApp(_newAddList, argsAdd.data);
                }
            }
            else {
                var query = new ej.Query().where(proxy.model.appointmentSettings["id"], ej.FilterOperators.equal, appoint[proxy.model.appointmentSettings["id"]]);
                var object = new ej.DataManager(this._currentAppointmentData).executeLocal(query);
                object[0][proxy.model.appointmentSettings["startTime"]] = cellstrtime;
                object[0][proxy.model.appointmentSettings["endTime"]] = cellendtime;
                if (!ej.isNullOrUndefined(allday)) object[0][proxy.model.appointmentSettings["allDay"]] = allday;
                if (this.model.orientation == "horizontal" && !ej.isNullOrUndefined(this.model.group) && ej.scheduleFeatures.resources)
                    proxy.element.find("#categorybar_" + appoint[proxy.model.appointmentSettings["id"]]).remove();
                if (!ej.isNullOrUndefined(this.model.group) && ej.scheduleFeatures.resources)
                    object[0] = this._getDragDropResourceData(proxy, cellIndex, object[0]);
                promise = proxy._dataManager.update(proxy.model.appointmentSettings["id"], object[0]);
                if (promise && $.isFunction(promise.promise) && proxy.dataSource() instanceof ej.DataManager) {
                    promise.done(function (e) {
                        new ej.DataManager(proxy._processed).update(proxy.model.appointmentSettings["id"], object);
                        proxy._renderSingleApp(object[0], object[0]);
                    });
                    promise.fail(function (e) {
                        proxy._renderAppointmentAll();
                    });
                }
                else {
                    new ej.DataManager(proxy._processed).update(appId, object);
                    this.element.find("#Appointment_" + appId).remove();
                    if (((this._formatTime(object[0][model["endTime"]]) - this._formatTime(object[0][model["startTime"]])) / 3600000) > 24) {
                        proxy._dataProcessing(appList);
                    }
                    proxy._renderSingleApp(object[0], object[0]);
                }
                //new ej.DataManager(proxy._processed).update(proxy.model.appointmentSettings["id"], appList);
            }
            $(args.element[0]).remove();
            // proxy._dataProcessing(appList);
            //proxy._renderAppointmentAll();
            proxy._trigger("dragDrop", { appointment: object });
    },
    /**
    * To get the appointments parent Id.
    * @private
    */
    _getAppointmentByParentId: function (appId) {
        var tempApp, appoint, appList = this._currentAppointmentData; // _processed;
        var tempApp = new ej.DataManager(this._processed).executeLocal(new ej.Query().where(this.model.appointmentSettings["id"], ej.FilterOperators.equal, appId));
        var appoint = new ej.DataManager(appList).executeLocal(new ej.Query().where(this.model.appointmentSettings["id"], ej.FilterOperators.equal, parseInt(tempApp[0]["ParentId"])));
        return !$.isEmptyObject(appoint) ? appoint[0] : appoint;
    },
    /**
    * To render all the appointments.
    * @private
    */
    _renderAppointmentAll: function () {
        this.element.find("div.e-appointment,div.e-alldayappointment,div.e-monthappointment").remove();
        this.element.find("div.e-categorybar").remove();
        if (this.currentView() !== "month") {
                this._renderAppointments();
                if (this.model.orientation == "vertical") this._renderAllDayAppointments();
				if(this.model.enableAppointmentNavigation)
                    this._nextPrevButton(this._filtered);
        }
        else
            this._renderMonthAppointment();
        this._wireResizeEvents();
        this._dragdropAppointments();
    },
    /**
    * Section to get the new end time after drag and drop the appointment.
    * @private
    */
    _getNewEndTime: function (newStartTime, hours, minutes) {
        var newEndTime = new Date(newStartTime);
        newEndTime.setHours(newEndTime.getHours() + hours);
        newEndTime.setMinutes(newEndTime.getMinutes() + minutes);
        return newEndTime;
    },
    /**
    * Section to get the new start time after drag and drop the appointment.
    * @private
    */
    _getNewStartTime: function (newTopTime, newStartTime) {
        newStartTime.setHours((newStartTime.getHours()) + parseInt(this.model.startHour) + parseInt(newTopTime / 60));
        newStartTime.setMinutes((newStartTime.getMinutes()) + parseInt(newTopTime % 60));
        newStartTime.setMinutes(Math.round(newStartTime.getMinutes() / 10.0) * 10);
        return newStartTime;
    },

    /**
    * Section to create the templates used in the schedule.
    * @private
    */
    _createTemplate: function (elt, templateId) {
        var scriptElt = document.createElement("script");
        scriptElt.id = (this._id + templateId + "_Template");
        scriptElt.type = "text/x-jsrender";
        scriptElt.text = elt;
        $("body").append(scriptElt);
        return scriptElt;
    },
    /**
    * Section for handle the keyboard support of the schedule.
    * @private
    */
    _documentOnKeyDown: function (event) {
        if (this.model.allowKeyboardNavigation) {
            var oEl = event.target || document.activeElement || event.srcElement;
            if (event.which == 27) {
                this._processEscapeKey();
            }
            else if (event.which == 13) {
                this._processEnterKey(oEl);
            }
            else if (event.shiftKey == false && event.which == 9) {
                this._processWindowTab(oEl);
            }
        }
    },
    /**
    * Section for handle the window tabbing process.
    * @private
    */
    _processWindowTab: function (oEl) {
        if ($(oEl).hasClass("qAppDone"))
            this._quickAppointWindow.find(".subject").focus();
        else if ($(oEl).hasClass("e-appointcancel") || $(oEl).hasClass("e-cancelrecur"))
            this._appointmentAddWindow.find(".subject").focus();
        else if ($(oEl).hasClass("e-editseries"))
            this.model.showQuickWindow && this._quickAppDetailsWindow.focus();
    },
    /**
    * Section for handle all the keyboard navigation process.
    * @private
    */
    _onKeyDown: function (event) {
        if (this.model.allowKeyboardNavigation) {
            this.model.showQuickWindow && (this._quickAppointWindow.ejDialog("close") && this._quickAppDetailsWindow.ejDialog("close"));
            var oRowCell = this.element.find('div.e-draggableworkarea tr');
            var oEl = event.target || document.activeElement || event.srcElement;
            if (event.shiftKey == false && event.which == 38) { //up arrow navigation
                this._stopDefaultAction(event);
                this._processUpArrow(oEl, oRowCell);
            }
            else if (event.shiftKey == false && event.which == 40) { //down arrow for cell navigation
                this._stopDefaultAction(event);
                this._processDown(oEl, oRowCell);
            }
            else if (event.ctrlKey == false && event.shiftKey == false && event.which == 39) { //right arrow cell navigation
                this._stopDefaultAction(event);
                this._processRight(oEl, oRowCell);
            }
            else if (event.ctrlKey == false && event.shiftKey == false && event.which == 37) { //left arrow cell navigation
                this._stopDefaultAction(event);
                this._processLeft(oEl, oRowCell);
            }
            else if (event.altKey == true && event.which == 67) { //Alt+C To show/hide the calendar pane
                this._stopDefaultAction(event);
                this.element.find(".e-datedisplay").click();
            }
            else if (event.ctrlKey == true && event.which == 37) { //Previous button
                this._stopDefaultAction(event);
                this.element.find("div.e-navigateprevious").click();
                this.element.find('div.e-navigateprevious').attr("tabIndex", 0).focus();
                this.element.find("#" + this._id + "_navigator").ejDatePicker({ value: new Date(this.currentDate()) });
            }
            else if (event.ctrlKey == true && event.which == 39) { //Next button
                this._stopDefaultAction(event);
                this.element.find("div.e-navigatenext").click();
                this.element.find('div.e-navigatenext').attr("tabIndex", 0).focus();
                this.element.find("#" + this._id + "_navigator").ejDatePicker({ value: new Date(this.currentDate()) });
            }
            else if ((event.altKey == true && event.which == 107) || (event.altKey == true && event.which == 187) || (event.altKey == true && event.which == 61)) { //Alt + "+" to navigate through the toolbar items
                this._stopDefaultAction(event);
                this._navigateToolbarRight(event);
            }
            else if ((event.altKey == true && event.which == 189) || (event.altKey == true && event.which == 109) || (event.altKey == true && event.which == 45) || (event.altKey == true && event.which == 173)) { // Alt + "-" to navigate back through the toolbar items
                this._stopDefaultAction(event);
                this._navigateToolbarLeft(event);
            }
            else if (event.ctrlKey == true && event.which == 67) { //Cancel Ctrl+C
                this._stopDefaultAction(event);
                this._cancel();
            }
            else if (event.which == 32) { //spacebar
                this._stopDefaultAction(event);
                if ($(document.activeElement)[0] === this.element.find("div.e-navigateprevious")[0]) {
                    this.element.find("div.e-navigateprevious").click();
                    this.element.find('div.e-navigateprevious').attr("tabIndex", 0).focus();
                }
                else if ($(document.activeElement)[0] === this.element.find("div.e-navigatenext")[0]) {
                    this.element.find("div.e-navigatenext").click();
                    this.element.find('div.e-navigatenext').attr("tabIndex", 0).focus();
                }
                this.element.find("#" + this._id + "_navigator").ejDatePicker({ value: new Date(this.currentDate()) });
            }
            else if (event.altKey == true && event.which == 78) { //New appointment window
                this._stopDefaultAction(event);
                this._newAppWindow(oEl, oRowCell);
                this.element.find("#" + this._id + "_navigator").ejDatePicker("hide");
            }
            else if (event.ctrlKey == true && event.which == 69 && this.element.find(".e-selectedAppointment").length > 0) { //(E)Edit appointment window
                this._stopDefaultAction(event);
                var elmt = this.element.find(".e-selectedAppointment");
                this._editKey = true;
                this._cellClick = false;
                this._appointmentWindow(elmt);
                this.element.find("#" + this._id + "_navigator").ejDatePicker("hide");
            }
            else if (event.which == 46) { //Delete
                this._stopDefaultAction(event);
                var delap = this.element.find('.e-selectedAppointment');
                if (delap.length > 0) {
                    var appId = delap[0].id.split("_")[1];
                    var parentId = this._getAppointmentByParentId(appId);
                    if (delap.length != 0 && ej.isNullOrUndefined(parentId[this.model.appointmentSettings["recurrenceRule"]]))
                        this._deleteApp(parseInt(appId));
                    else
                        this._deleteApp(parseInt(parentId[this.model.appointmentSettings["id"]]));
                }
            }
            else if (event.shiftKey == true && event.which == 9) { //Shift + tab for reverse navigation of appointments
                this._stopDefaultAction(event);
                this._reverseTab(oEl, event, oRowCell);
                this.element.find("#" + this._id + "_navigator").ejDatePicker("hide");
            }
            else if (event.shiftKey == false && event.which == 9) { // For tab navigation of appointments
                this._stopDefaultAction(event);
                this._processTab(oEl, event, oRowCell);
                this.element.find("#" + this._id + "_navigator").ejDatePicker("hide");
            }
            else if (event.shiftKey == true && event.which == 38) { //shift + up arrow
                this._stopDefaultAction(event);
                this._processShiftUp(oEl, event, oRowCell);
            }
            else if (event.shiftKey == true && event.which == 40) { //shift + down arrow
                this._stopDefaultAction(event);
                this._processShiftDown(oEl, event, oRowCell);
            }
            else if (event.shiftKey == true && event.which == 37) { //shift+left
                this._stopDefaultAction(event);
                this._processShiftLeft(oEl, event, oRowCell);
            }
            else if (event.shiftKey == true && event.which == 39) { //shift+right
                this._stopDefaultAction(event);
                this._processShiftRight(oEl, event, oRowCell);
            }
            if (!$(oEl).hasClass("e-monthcells"))
                this.element.find(".e-scrolltimecells").css("top", (-this.element.find('.e-draggableworkarea').scrollTop()) + "px");
        }
    },
    /**
    * Section for handle the shift up key process.
    * @private
    */
    _processShiftUp: function (oEl, event, oRowCell) {
        if ($(oEl).hasClass('e-workcells')) {
            var rowcount = oRowCell.length - 1;
            var currrowindex = $(oEl).parent()[0].rowIndex;
            var currcolindex = $(oEl)[0].cellIndex;
            if (currrowindex > 0 && currrowindex <= rowcount && currrowindex != "undefined") {
                var newcur = parseInt(currrowindex) - 1;
                if (newcur >= 0) {
                    if ($(oRowCell[newcur].cells[currcolindex]).hasClass("e-selectedCell")) {
                        $(oRowCell[newcur].cells[currcolindex]).attr("tabIndex", 0).focus();
                        $(oRowCell[currrowindex].cells[currcolindex]).removeClass("e-selectedCell");
                    }
                    else {
                        $(oRowCell[currrowindex].cells[currcolindex]).addClass("e-selectedCell");
                        $(oRowCell[newcur].cells[currcolindex]).addClass("e-selectedCell").attr("tabIndex", 0).focus();
                    }
                }
                this._currentRowIndex = newcur;
                this._currentColIndex = currcolindex;
            }
            else {
                $(oRowCell[currrowindex].cells[currcolindex]).attr("tabIndex", 0).focus();
                this._currentRowIndex = currrowindex;
                this._currentColIndex = currcolindex;
            }

        }
        else if ($(oEl).hasClass('e-monthcells')) {
            var resourceIndex = Math.floor($(oEl).index() / 7);
            var rowcount = oRowCell.length - 1, resmaxcolindex = 7 * (resourceIndex + 1), mincolindex = 7 * (parseInt(resourceIndex));
            var currrowindex = $(oEl).parent()[0].rowIndex;
            var currcolindex = $(oEl)[0].cellIndex;
            if (currrowindex > 0 && currrowindex <= rowcount && currrowindex != "undefined") {
                var newcur = parseInt(currrowindex) - 1;
                if (newcur >= 0) {
                    if ($(oRowCell[newcur].cells[currcolindex]).hasClass("e-selectedCell")) {
                        for (var i = currcolindex; i >= mincolindex; i--) {
                            $(oRowCell[currrowindex].cells[i]).attr("tabIndex", 0).focus();
                            $(oRowCell[currrowindex].cells[i]).removeClass("e-selectedCell");
                        }
                        for (var j = resmaxcolindex - 1; j >= currcolindex; j--) {
                            if (newcur >= 0) {
                                $(oRowCell[newcur].cells[j]).attr("tabIndex", 0).focus();
                                $(oRowCell[newcur].cells[j]).removeClass("e-selectedCell");
                            }
                        }
                        $(oRowCell[newcur].cells[currcolindex]).addClass("e-selectedCell");
                    }
                    else {
                        for (var i = currcolindex; i >= mincolindex; i--)
                            $(oRowCell[currrowindex].cells[i]).addClass("e-selectedCell").attr("tabIndex", 0).focus();
                        for (var j = resmaxcolindex - 1; j >= currcolindex; j--) {
                            if (newcur >= 0)
                                $(oRowCell[newcur].cells[j]).addClass("e-selectedCell").attr("tabIndex", 0).focus();
                        }
                    }
                }
                else
                    $(oRowCell[newcur + 1].cells[currcolindex]).attr("tabIndex", 0).focus();
            }
            else
                return false;
        }
        else
            return false;

    },
    /**
    * Section for handle the shift down key process.
    * @private
    */
    _processShiftDown: function (oEl, event, oRowCell) {

        if ($(oEl).hasClass('e-workcells')) {
            var rowcount = oRowCell.length - 1; // this.get_RowCount();
            var currrowindex = $(oEl).parent()[0].rowIndex;
            var currcolindex = $(oEl)[0].cellIndex;
            if (currrowindex >= 0 && currrowindex <= rowcount && currrowindex != "undefined") {
                $(oRowCell[currrowindex].cells[currcolindex]).addClass("e-selectedCell").attr("tabIndex", 0).focus();
                var newcur = parseInt(currrowindex) + 1;
                if (newcur <= rowcount) {
                    if ($(oRowCell[newcur].cells[currcolindex]).hasClass("e-selectedCell")) {
                        $(oRowCell[newcur].cells[currcolindex]).attr("tabIndex", 0).focus();
                        $(oRowCell[currrowindex].cells[currcolindex]).removeClass("e-selectedCell");
                    }
                    else {
                        $(oRowCell[currrowindex].cells[currcolindex]).addClass("e-selectedCell");
                        $(oRowCell[newcur].cells[currcolindex]).addClass("e-selectedCell").attr("tabIndex", 0).focus();
                    }
                    this._currentRowIndex = newcur;
                    this._currentColIndex = currcolindex;
                }
                else {
                    $(oRowCell[currrowindex].cells[currcolindex]).attr("tabIndex", 0).focus();
                    this._currentRowIndex = currrowindex;
                    this._currentColIndex = currcolindex;
                }

            }
        }
        else if ($(oEl).hasClass('e-monthcells')) {
            var resourceIndex = Math.floor($(oEl).index() / 7);
            var rowcount = oRowCell.length - 1, resmaxcolindex = 7 * (resourceIndex + 1), mincolindex = 7 * (parseInt(resourceIndex));
            var currrowindex = $(oEl).parent()[0].rowIndex;
            var currcolindex = $(oEl)[0].cellIndex;
            if (currrowindex >= 0 && currrowindex <= rowcount && currrowindex != "undefined") {
                var newcur = parseInt(currrowindex) + 1;
                if (newcur < (this._dateRender.length) / 7) {
                    if ($(oRowCell[newcur].cells[currcolindex]).hasClass("e-selectedCell")) {
                        for (var i = currcolindex; i <= resmaxcolindex - 1; i++) {
                            $(oRowCell[currrowindex].cells[i]).attr("tabIndex", 0).focus();
                            $(oRowCell[currrowindex].cells[i]).removeClass("e-selectedCell");
                        }
                        for (var j = mincolindex; j <= currcolindex; j++) {
                            if (newcur < (this._dateRender.length) / 7) {
                                $(oRowCell[newcur].cells[j]).attr("tabIndex", 0).focus();
                                $(oRowCell[newcur].cells[j]).removeClass("e-selectedCell");
                            }
                        }
                        $(oRowCell[newcur].cells[currcolindex]).addClass("e-selectedCell");
                    }
                    else {
                        for (var i = currcolindex; i <= resmaxcolindex - 1; i++)
                            $(oRowCell[currrowindex].cells[i]).addClass("e-selectedCell").attr("tabIndex", 0).focus();
                        for (var j = mincolindex; j <= currcolindex; j++) {
                            if (newcur < (this._dateRender.length)/7)
                                $(oRowCell[newcur].cells[j]).addClass("e-selectedCell").attr("tabIndex", 0).focus();
                        }
                    }
                }
                else
                    $(oRowCell[newcur - 1].cells[currcolindex]).attr("tabIndex", 0).focus();
            }
            else
                return false;
        }
        else
            return false;
    },
    /**
    * Section for handle the shift left key process.
    * @private
    */
    _processShiftLeft: function (oEl, event, oRowCell) {
        var rowcount = oRowCell.length - 1;
        if ($(oEl).hasClass('e-workcells')) {
            var targetElement = oEl;
            var viewCount = this.model.currentView == "day" ? 1 : this.model.currentView == "workweek" ? 5 : 7;
            var resIndex = Math.floor($(targetElement).index() / viewCount);
            var resmaxcolindex = (viewCount * resIndex); // this.model.currentView == "day" ? 1 : this.model.currentView == "workweek" ? 5 : 7;
            var currColInx = $(targetElement)[0].cellIndex;
            var currRowInx = $(targetElement).parent()[0].rowIndex;
                var leftColInx = currColInx - 1;
                if (currColInx > 0 && leftColInx > parseInt(resmaxcolindex) - 1 && currColInx != "undefined") {
                    if ($(oRowCell[currRowInx].cells[leftColInx]).hasClass("e-selectedCell")) {
                        for (var i = 0; i <= parseInt(currRowInx); i++)
                            $(oRowCell[i].cells[currColInx]).removeClass("e-selectedCell");
                        for (var j = rowcount; j >= parseInt(currRowInx); j--)
                            $(oRowCell[j].cells[leftColInx]).removeClass("e-selectedCell");
                        $(oRowCell[currRowInx].cells[leftColInx]).addClass("e-selectedCell").attr("tabIndex", 0).focus();
                    }
                    else {
                        for (var i = 0; i <= parseInt(currRowInx); i++)
                            $(oRowCell[i].cells[currColInx]).addClass("e-selectedCell");
                        for (var j = rowcount; j >= parseInt(currRowInx); j--)
                            $(oRowCell[j].cells[leftColInx]).addClass("e-selectedCell");
                        $(oRowCell[currRowInx].cells[leftColInx]).attr("tabIndex", 0).focus();
                    }
                this._currentRowIndex = currRowInx;
                this._currentColIndex = leftColInx;
            }
            else {
                $(oRowCell[currRowInx].cells[leftColInx + 1]).attr("tabIndex", 0).focus();
                this._currentRowIndex = currRowInx;
                this._currentColIndex = leftColInx + 1;
            }
        }
        else if ($(oEl).hasClass('e-monthcells')) {
            var resIndex = Math.floor($(oEl).index() / 7);
            var rowcount = oRowCell.length - 1, mcolind = 7 * (parseInt(resIndex) + 1); //7,
            var mincolindex = 7 * (parseInt(resIndex)); //0;
            var currrowindex = $(oEl).parent()[0].rowIndex;
            var currcolindex = $(oEl)[0].cellIndex;
            if (currcolindex >= mincolindex && currcolindex <= mcolind && currcolindex != "undefined") {
                newcur = parseInt(currcolindex) - 1;
                if ($(oRowCell[currrowindex].cells[newcur]).hasClass("e-selectedCell")) {
                    if (currcolindex == mincolindex && currrowindex >= 1) {
                        var newrow = parseInt(currrowindex) - 1;
                        var newcol = parseInt(mcolind) - 1;
                        if (newrow >= 0) {
                            $(oRowCell[newrow].cells[newcol]).attr("tabIndex", 0).focus();
                            $(oRowCell[newrow].cells[currcolindex]).removeClass("e-selectedCell");
                            $(oRowCell[currrowindex].cells[mincolindex]).removeClass("e-selectedClass");
                        }
                    }
                    else if (currcolindex < mcolind) {
                        newcur = parseInt(currcolindex) - 1;
                        $(oRowCell[currrowindex].cells[newcur]).attr("tabIndex", 0).focus();
                        $(oRowCell[currrowindex].cells[currcolindex]).removeClass("e-selectedCell");
                    }
                }
                else {
                    if (currcolindex == mincolindex && currrowindex >= 0) {
                        var newrow = parseInt(currrowindex) - 1;
                        var newcol = parseInt(mcolind) - 1;
                        if (newrow >= 0)
                            $(oRowCell[newrow].cells[newcol]).addClass("e-selectedCell").attr("tabIndex", 0).focus();
                        else
                            $(oRowCell[newrow + 1].cells[currcolindex]).attr("tabIndex", 0).focus();
                    }
                    else if (currcolindex < mcolind) {
                        newcur = parseInt(currcolindex) - 1;
                        $(oRowCell[currrowindex].cells[newcur]).addClass("e-selectedCell").attr("tabIndex", 0).focus();
                    }
                }
            }
            else
                return;
        }
        else
            return false;
    },
    /**
    * Section for handle the shift right key process.
    * @private
    */
    _processShiftRight: function (oEl, event, oRowCell) {
        if ($(oEl).hasClass('e-workcells')) {
            var rowcount = oRowCell.length - 1;
            var viewCount = this.model.currentView == "day" ? 1 : this.model.currentView == "workweek" ? 5 : 7;
            var resIndex = Math.floor($(oEl).index() / viewCount);
            var resmaxcolindex = (viewCount * (resIndex+1));
            var currrowindex = $(oEl).parent()[0].rowIndex;
            var currcolindex = $(oEl)[0].cellIndex;
                var newcur = parseInt(currcolindex) + 1;
                if (currcolindex >= 0 && currcolindex < parseInt(resmaxcolindex) - 1 && currcolindex != "undefined") {
                    if ($(oRowCell[currrowindex].cells[newcur]).hasClass("e-selectedCell")) {
                        for (var i = parseInt(currrowindex); i <= rowcount; i++)
                            $(oRowCell[i].cells[currcolindex]).removeClass("e-selectedCell");
                        for (var j = 0; j <= currrowindex; j++)
                            $(oRowCell[j].cells[newcur]).removeClass("e-selectedCell").attr("tabIndex", 0).focus();
                        $(oRowCell[currrowindex].cells[newcur]).addClass("e-selectedCell");
                    }
                    else {
                        for (var i = parseInt(currrowindex); i <= rowcount; i++)
                            $(oRowCell[i].cells[currcolindex]).addClass("e-selectedCell");
                        for (var j = 0; j <= currrowindex; j++)
                            $(oRowCell[j].cells[newcur]).addClass("e-selectedCell").attr("tabIndex", 0).focus();
                }
                this._currentRowIndex = currrowindex;
                this._currentColIndex = newcur;
            }
            else
                $(oRowCell[currrowindex].cells[currcolindex]).attr("tabIndex", 0).focus();
        }
        else if ($(oEl).hasClass('e-monthcells')) {
            var resIndex = Math.floor($(oEl).index() / 7);
            var rowcount = oRowCell.length - 1, mcolind = resmaxcolindex = 7 * (parseInt(resIndex) + 1), mincolindex = 7 * (parseInt(resIndex));;
            var currrowindex = $(oEl).parent()[0].rowIndex;
            var currcolindex = $(oEl)[0].cellIndex;
            var newcur = parseInt(currcolindex) + 1;
            if (currcolindex >= 0 && currcolindex <= parseInt(resmaxcolindex) - 1 && currcolindex != "undefined") {
                if ($(oRowCell[currrowindex].cells[newcur]).hasClass("e-selectedCell")) {
                    if (newcur < mcolind) {
                        $(oRowCell[currrowindex].cells[newcur]).attr("tabIndex", 0).focus();
                        $(oRowCell[currrowindex].cells[currcolindex]).removeClass("e-selectedCell");
                    }
                    if (currcolindex == parseInt(resmaxcolindex) - 1 && currrowindex < (this._dateRender.length) / 7) {
                        var newrow = parseInt(currrowindex) + 1;
                        currcolindex = mincolindex;
                        if (newrow < (this._dateRender.length) / 7) {
                            $(oRowCell[newrow].cells[currcolindex]).removeClass("e-selectedCell").attr("tabIndex", 0).focus();
                            if (newcur < mcolind) {
                                $(oRowCell[newrow].cells[newcur]).attr("tabIndex", 0).focus();
                                $(oRowCell[newrow].cells[currcolindex]).removeClass("e-selectedCell");
                            }
                        }
                    }
                }
                else {
                    if (newcur < mcolind)
                        $(oRowCell[currrowindex].cells[newcur]).addClass("e-selectedCell").attr("tabIndex", 0).focus();
                    if (currcolindex == parseInt(resmaxcolindex) - 1 && currrowindex < (this._dateRender.length) / 7) {
                        var newrow = parseInt(currrowindex) + 1;
                        currcolindex = mincolindex;
                        if (newrow < (this._dateRender.length) / 7) {
                            $(oRowCell[newrow].cells[currcolindex]).addClass("e-selectedCell").attr("tabIndex", 0).focus();
                            if (newcur < mcolind)
                                $(oRowCell[newrow].cells[newcur]).addClass("e-selectedCell").attr("tabIndex", 0).focus();
                        }
                        else
                            $(oRowCell[newrow - 1].cells[newcur - 1]).attr("tabIndex", 0).focus();
                    }
                }
            }
            else
                return false;
        }
        else
            return false;
    },
    /**
    * Section for handle the enter key process.
    * @private
    */
    _processEnterKey: function (oEl) {
        if ($(oEl).hasClass("e-workcells") || $(oEl).hasClass("e-monthcells") || $(oEl).hasClass("e-alldaycells")) {
            var tds = this.element.find('td.e-selectedCell');
            var nt = tds.length - 1;
            if (nt == 0) {
                this._cellSelection(oEl);
            }
            else {
                this._multipleCellAppointCreation(oEl);
            }
        }
        else if ($(oEl).hasClass("e-detailedapp e-detailedwindow")) {
            this._appointmentWindow(oEl);
        }
        else if ($(oEl).parent().hasClass("e-allday")) {
            var e = this._appointmentAddWindow.find("#" + this._id + "allday").ejCheckBox("option", "checked");
            var checkedVal = e ? false : true;
            this._appointmentAddWindow.find(".allday").ejCheckBox({ checked: checkedVal });
        }
        else if ($(oEl).parent().hasClass("e-recurrence")) {
            var e = this._appointmentAddWindow.find("#" + this._id + "recurrence").ejCheckBox("option", "checked");
            var checkedVal = e ? false : true;
            this._appointmentAddWindow.find(".recurrence").ejCheckBox({ checked: checkedVal });
            this._recurChecked = checkedVal ? true : false;
            this._repeat(oEl);
        }
        else if ($(oEl).hasClass("e-scheduledelete")) {
            this.model.showQuickWindow && this._quickAppDetailsWindow.find('.e-scheduledelete').click();
        }
        else if ($(oEl).hasClass("e-editevent"))
            this.model.showQuickWindow && this._quickAppDetailsWindow.find('.e-editevent').click();
        else if ($(oEl).hasClass(".e-editseries"))
            this.model.showQuickWindow && this._quickAppDetailsWindow.find('.e-editseries').click();
        else if ($(oEl).hasClass("e-appointment") || $(oEl).hasClass("e-alldayappointment") || $(oEl).hasClass("e-monthappointment"))
            this._cellSelection(oEl);
        else if ($(oEl).parent().hasClass("e-weekday")) {
            var e = this._appointmentAddWindow.find("#" + $(oEl).children()[0].id).ejCheckBox("option", "checked");
            var checkedVal = e ? false : true;
            this._appointmentAddWindow.find("#" + $(oEl).children()[0].id).ejCheckBox({ checked: checkedVal });
        }
    },
    /**
    * To find the minimum and maximum cell index of the schedule.
    * @private
    */
    _findMinMaxCellIndex: function () {
        var oRowCell = this.element.find('div.e-draggableworkarea tr');
        var rowcount = oRowCell.length - 1;
        var omincolIndex, omaxcolIndex, rowInx, maxrowInx, checkcolindex;
        var curr_View = this.currentView();
        var viewCount = curr_View == "day" ? 1 : curr_View == "workweek" ? 5 : 7;
        var proxy = this;
        this.element.find('td.e-selectedCell').each(function (index, tdItem) {
            var targetId = $(tdItem).index();
            if (targetId != null && !isNaN(targetId)) {
                if (omincolIndex == null) {
                    omincolIndex = targetId;
                    rowInx = $(tdItem).parent()[0].rowIndex;
                }
                else if (omincolIndex < targetId) {
                    omincolIndex = omincolIndex;
                }
                else {
                    if (omincolIndex != targetId) {
                        omincolIndex = targetId;
                        rowInx = $(tdItem).parent()[0].rowIndex;
                    }
                }
                if (omaxcolIndex == null) {
                    omaxcolIndex = targetId;
                    if (curr_View != "month") {
                        checkcolindex = parseInt(omaxcolIndex);
                        for (var i = 0; i <= rowcount; i++) {
                            if ($(oRowCell[i].cells[checkcolindex]).hasClass("e-selectedCell")) {
                                maxrowInx = i;

                            }
                        }
                    }
                }
                else if (omaxcolIndex > targetId) {
                    omaxcolIndex = omaxcolIndex;
                }
                else {
                    if (omaxcolIndex != targetId) {
                        omaxcolIndex = targetId;
                        if (curr_View != "month") {
                            checkcolindex = parseInt(omaxcolIndex);
                            for (var i = 0; i <= rowcount; i++) {
                                if ($(oRowCell[i].cells[checkcolindex]).hasClass("e-selectedCell")) {
                                    maxrowInx = i;

                                }
                            }
                        }
                    }
                }
            }
        });
        var _minMaxIndexes = { rowInx: rowInx, maxrowInx: maxrowInx, omincolIndex: omincolIndex, omaxcolIndex: omaxcolIndex };
        return _minMaxIndexes;
        //}

    },
    /**
    * Section for handle the multiple cell appointment creation.
    * @private
    */
    _multipleCellAppointCreation: function (target) {
        var indexes = this._findMinMaxCellIndex(); var renderDates = this.dateRender;
        var omincolIndex = indexes.omincolIndex, omaxcolIndex = indexes.omaxcolIndex, rowInx = indexes.rowInx, maxrowInx = indexes.maxrowInx;
        var minStartDate = renderDates[omincolIndex]; var maxEndDate = renderDates[omaxcolIndex];
        var localizeDays = Globalize.culture(this.model.locale);
        if ($(target).hasClass("e-workcells")) {
            var minStartTime = (this.model.startHour + (rowInx / 2)).toString().split(".");
            var maxEndTime = (this.model.startHour + (maxrowInx / 2)).toString().split(".");
            this.cur_StartTime = new Date(minStartDate).setHours(parseInt(minStartTime[0]), parseInt(minStartTime[1]) == 5 ? 30 : 00);
            this.cur_EndTime = new Date(maxEndDate).setHours(parseInt(maxEndTime[0]), parseInt(maxEndTime[1]) == 5 ? 30 : 00);
            this.cur_EndTime = new Date(new Date(this.cur_EndTime).setMinutes(new Date(this.cur_EndTime).getMinutes() + 30));
        }
        else if ($(target).hasClass("e-monthcells")) {
            var selCell = this.element.find('td.e-selectedCell');
            var nsc = selCell.length - 1;
            var startIndex = ($(selCell[0]).parent().index() * 7) + ($(selCell[0]).index() - (Math.floor($(selCell[0]).index() / 7) * 7));
            var endIndex = ($(selCell[nsc]).parent().index() * 7) + ($(selCell[nsc]).index() - (Math.floor($(selCell[nsc]).index() / 7) * 7)); // $(selCell[nsc]).index();
            var startDate = new Date(this._dateRender[startIndex]);
            var endDate = new Date(this._dateRender[endIndex]);
            this.cur_StartTime = new Date(new Date(startDate).setHours(9));
            this.cur_StartTime = new Date(this.cur_StartTime).setMinutes(00); //.setHours(00)).setMinutes(00);
            this.cur_EndTime = new Date(new Date(endDate).setHours(9)); //.setHours(23)).setMinutes(59)); 
            this.cur_EndTime = new Date(new Date(this.cur_EndTime).setMinutes(30));
        }
        if (this.model.showQuickWindow) {
            this._quickAppointWindow.find("#" + this._id + "_quickAppwindow").css("display", "block");

            //add the dialog code to display while clicking on the cells
            if (((new Date(this.cur_EndTime) - new Date(this.cur_StartTime)) / 3600000) < 24)
                this._quickAppointWindow.find('.e-quickstartend').html(localizeDays.calendar.days.names[new Date(this.cur_StartTime).getDay()] + ", " + Globalize.format(new Date(this.cur_StartTime), "MMMM") + " " + new Date(this.cur_StartTime).getDate() + ", " + Globalize.format(new Date(this.cur_StartTime), this.model.timeMode == "12" ? this._pattern.t : "HH:mm") + " - " + Globalize.format(new Date(this.cur_EndTime), this.model.timeMode == "12" ? this._pattern.t : "HH:mm"));
                else
                    this._quickAppointWindow.find('.e-quickstartend').html(localizeDays.calendar.days.names[new Date(this.cur_StartTime).getDay()] + ", " + Globalize.format(new Date(this.cur_StartTime), "MMMM") + " " + new Date(this.cur_StartTime).getDate() + ", " + Globalize.format(new Date(this.cur_StartTime), this.model.timeMode == "12" ? this._pattern.t : "HH:mm") + " - " + localizeDays.calendar.days.names[new Date(this.cur_EndTime).getDay()] + ", " + Globalize.format(new Date(this.cur_EndTime), "MMMM") + " " + new Date(this.cur_EndTime).getDate() + ", " + Globalize.format(new Date(this.cur_EndTime), this.model.timeMode == "12" ? this._pattern.t : "HH:mm"));
                this._quickAppointWindow.find('.subject').val(""); var x = target.offsetLeft; var y = target.offsetTop;
                this._quickAppointWindow.ejDialog({ position: { X: x, Y: y} });
                this._quickAppointWindow.ejDialog("open");
                this._quickAppointWindow.find(".subject").focus();
            }
        this._cellClick = true;

    },
    /**
    * Section for handle the cell selection process using keyboard.
    * @private
    */
    _addCellSelection: function (oRowCell) {
        var tds = this.element.find('td.e-selectedCell');
        var nt = tds.length - 1;
        if (nt > 0) {
            this.element.find('td.e-selectedCell').removeClass("e-selectedCell");
            $(oRowCell[this._currentRowIndex].cells[this._currentColIndex]).addClass("e-selectedCell").attr("tabIndex", 0).focus();
        }
    },
    /**
    * Section for handle the up arrow key process.
    * @private
    */
    _processUpArrow: function (oEl, oRowCell) {
        if ($(oEl).hasClass("e-selectedAppointment")) {
            this.element.find('.e-selectedAppointment').removeClass("e-selectedAppointment");
            $(oRowCell[0].cells[0]).addClass("e-selectedCell").attr("tabIndex", 0).focus();
        }
        else if ($(oEl).hasClass("e-selectedCell"))
            this._addCellSelection(oRowCell);
        var allda = this.element.find(".e-alldaycells");
        if ($(oEl).hasClass("e-workcells")) {
            var rowcount = oRowCell.length - 1;
            var currrowindex = this.element.find("td.e-selectedCell").parent()[0].rowIndex;
            var currcolindex = this.element.find("td.e-selectedCell")[0].cellIndex;
            var newcur = parseInt(currrowindex) - 1;
            if (currrowindex > 0 && currrowindex <= rowcount && currrowindex != "undefined") {
                $(oRowCell[currrowindex].cells[currcolindex]).removeClass("e-selectedCell").blur();
                $(oRowCell[newcur].cells[currcolindex]).addClass("e-selectedCell").attr("tabIndex", 0).focus();
            }
            else if (currrowindex == 0) {
                var currrowindex = this.element.find("td.e-selectedCell").parent()[0].rowIndex;
                var currcolindex = this.element.find("td.e-selectedCell")[0].cellIndex;
                $(oRowCell[currrowindex].cells[currcolindex]).removeClass("e-selectedCell").blur();
                $(this.element).attr("tabIndex", 0).focus();
                $(allda[currcolindex]).addClass("e-selectedCell").attr("tabIndex", 0).focus();
            }
        }
        else if ($(oEl).hasClass("e-alldaycells")) {
            return false;
        }

        else if ($(oEl).hasClass("e-monthcells")) {
            var rowcount = oRowCell.length - 1;
            var currrowindex = this.element.find("td.e-selectedCell").parent()[0].rowIndex;
            var currcolindex = this.element.find("td.e-selectedCell")[0].cellIndex;
            var newcur = parseInt(currrowindex) - 1;
            if (currrowindex >= 0 && currrowindex <= rowcount && currrowindex != "undefined") {
                if (newcur >= 0) {
                    $(oRowCell[currrowindex].cells[currcolindex]).removeClass("e-selectedCell").blur();
                    $(oRowCell[newcur].cells[currcolindex]).addClass("e-selectedCell").attr("tabIndex", 0).focus();
                }
                else
                    $(oRowCell[0].cells[currcolindex]).addClass("e-selectedCell").attr("tabIndex", 0).focus();
            }
            else
                return false;
        }
        else
            return false;

    },
    /**
    * Section for handle the down arrow key process.
    * @private
    */
    _processDown: function (oEl, oRowCell) {
        if ($(oEl).hasClass("e-selectedAppointment")) {
            this.element.find('.e-selectedAppointment').removeClass("e-selectedAppointment");
            $(oRowCell[0].cells[0]).addClass("e-selectedCell").attr("tabIndex", 0).focus();
        }
        else if ($(oEl).hasClass("e-selectedCell"))
            this._addCellSelection(oRowCell);
        if ($(oEl).hasClass("e-workcells")) {
            var rowcount = oRowCell.length - 1;
            var currrowindex = this.element.find("td.e-selectedCell").parent()[0].rowIndex;
            var currcolindex = this.element.find("td.e-selectedCell")[0].cellIndex;
            var newcur = parseInt(currrowindex) + 1;
            if (currrowindex >= 0 && currrowindex <= rowcount && currrowindex != "undefined") {
                if (newcur <= rowcount) {
                    $(oRowCell[currrowindex].cells[currcolindex]).removeClass("e-selectedCell").blur();
                    $(oRowCell[newcur].cells[currcolindex]).addClass("e-selectedCell").attr("tabIndex", 0).focus();
                }
                else
                    $(oRowCell[currrowindex].cells[currcolindex]).attr("tabIndex", 0).focus();
            }
        }
        else if ($(oEl).hasClass("e-alldaycells")) {
            var allDayInx = this.element.find('td.e-selectedCell')[0].cellIndex - 1;
            var allda = this.element.find(".e-alldaycells");
            $(allda[allDayInx]).removeClass("e-selectedCell").blur();
            $(oRowCell[0].cells[allDayInx]).addClass("e-selectedCell").attr("tabIndex", 0).focus();
        }

        else if ($(oEl).hasClass("e-monthcells")) {
            var rowcount = oRowCell.length - 1;
            var currrowindex = this.element.find("td.e-selectedCell").parent()[0].rowIndex;
            var currcolindex = this.element.find("td.e-selectedCell")[0].cellIndex;
            var newcur = parseInt(currrowindex) + 1;
            if (currrowindex >= 0 && currrowindex < rowcount && currrowindex != "undefined") {
                if (newcur <= 7) {
                    $(oRowCell[currrowindex].cells[currcolindex]).removeClass("e-selectedCell").blur();
                    $(oRowCell[newcur].cells[currcolindex]).addClass("e-selectedCell").attr("tabIndex", 0).focus();
                }
                else
                    $(oRowCell[newcur - 1].cells[currcolindex]).addClass("e-selectedCell").attr("tabIndex", 0).focus();
            }
        }
        else
            return false;
    },
    /**
    * Section for handle the right arrow key process.
    * @private
    */
    _processRight: function (oEl, oRowCell) {
        if ($(oEl).hasClass("e-selectedAppointment")) {
            this.element.find('.e-selectedAppointment').removeClass("e-selectedAppointment");
            $(oRowCell[0].cells[0]).addClass("e-selectedCell").attr("tabIndex", 0).focus();
        }
        else if ($(oEl).hasClass("e-selectedCell"))
            this._addCellSelection(oRowCell);
        if ($(oEl).hasClass("e-workcells") || $(oEl).hasClass("e-monthcells")) {
            var colIndex = this.currentView() == "day" ? 1 : this.currentView() == "workweek" ? 5 : 7;
            maxcolindex = (!ej.isNullOrUndefined(this.model.resources) && !ej.isNullOrUndefined(this.model.group) && ej.scheduleFeatures.resources) ? this.render_Resources[this.render_Resources.length - 1].length * colIndex : colIndex;
            var currrowindex = this.element.find("td.e-selectedCell").parent()[0].rowIndex;
            var currcolindex = this.element.find("td.e-selectedCell")[0].cellIndex;
            var newcur = parseInt(currcolindex) + 1;
            if (currcolindex >= 0 && currcolindex < parseInt(maxcolindex) - 1 && currcolindex != "undefined") {
                $(oRowCell[currrowindex].cells[currcolindex]).removeClass("e-selectedCell").blur();
                $(oRowCell[currrowindex].cells[newcur]).addClass("e-selectedCell").attr("tabIndex", 0).focus();
            }
            else if (currcolindex == parseInt(maxcolindex) - 1) {
                var maxrowInx1 = $(oEl).parent()[0].rowIndex;
                this.element.find("div.e-navigatenext").click();
                $(oRowCell[maxrowInx1].cells[0]).addClass("e-selectedCell").attr("tabIndex", 0).focus();
            }
            else
                return false;
        }
        else if ($(oEl).hasClass("e-alldaycells")) {
            var allDayInx = this.element.find('td.e-selectedCell')[0].cellIndex - 1;
            var allda = this.element.find(".e-alldaycells");
            var nall = parseInt(allda.length) - 1;
            var newallDayInx = parseInt(allDayInx) + 1;
            if (allDayInx >= 0 && allDayInx < nall) {
                $(allda[allDayInx]).removeClass("e-selectedCell").blur();
                $(allda[newallDayInx]).addClass("e-selectedCell").attr("tabIndex", 0).focus();
            }
            else if (allDayInx == nall) {
                this.element.find("div.e-navigatenext").click();
                $(allda[0]).addClass("e-selectedCell").attr("tabIndex", 0).focus();
            }
            else
                return false;
        }

    },
    /**
    * Section for handle the left arrow key process.
    * @private
    */
    _processLeft: function (oEl, oRowCell) {
        if ($(oEl).hasClass("e-selectedAppointment")) {
            this.element.find('.e-selectedAppointment').removeClass("e-selectedAppointment");
            $(oRowCell[0].cells[0]).addClass("e-selectedCell").attr("tabIndex", 0).focus();
        }
        else if ($(oEl).hasClass("e-selectedCell"))
            this._addCellSelection(oRowCell);
        if ($(oEl).hasClass("e-workcells") || $(oEl).hasClass("e-monthcells")) {
            var colIndex = this.currentView() == "day" ? 1 : this.currentView() == "workweek" ? 5 : 7;
            maxcolindex = (!ej.isNullOrUndefined(this.model.resources) && !ej.isNullOrUndefined(this.model.group) && ej.scheduleFeatures.resources) ? this.render_Resources[this.render_Resources.length - 1].length * colIndex : colIndex;
            var currrowindex = this.element.find("td.e-selectedCell").parent()[0].rowIndex;
            var currcolindex = this.element.find("td.e-selectedCell")[0].cellIndex;
            var newcur = parseInt(currcolindex) - 1;
            if (currcolindex > 0 && currcolindex <= parseInt(maxcolindex) - 1 && currcolindex != "undefined") {
                $(oRowCell[currrowindex].cells[currcolindex]).removeClass("e-selectedCell").blur();
                $(oRowCell[currrowindex].cells[newcur]).addClass("e-selectedCell").attr("tabIndex", 0).focus();
            }
            else if (currcolindex == 0) {
                var maxrowInx1 = $(oEl).parent()[0].rowIndex;
                this.element.find("div.e-navigateprevious").click();
                $(oRowCell[maxrowInx1].cells[maxcolindex - 1]).addClass("e-selectedCell").attr("tabIndex", 0).focus();
            }
            else
                return false;
        }
        else if ($(oEl).hasClass("e-alldaycells")) {
            var maxcolindex = this.currentView() == "day" ? 1 : this.currentView() == "workweek" ? 5 : 7;
            var allDayInx = this.element.find('td.e-selectedCell')[0].cellIndex - 1;
            var allda = this.element.find(".e-alldaycells");
            var nall = parseInt(allda.length) - 1;
            var newallDayInx = parseInt(allDayInx) - 1;
            if (allDayInx > 0 && allDayInx <= nall) {
                $(allda[allDayInx]).removeClass("e-selectedCell").blur();
                $(allda[newallDayInx]).addClass("e-selectedCell").attr("tabIndex", 0).focus();
            }
            else if (allDayInx == 0) {
                this.element.find("div.e-navigateprevious").click();
                $(allda[maxcolindex - 1]).addClass("e-selectedCell").attr("tabIndex", 0).focus();
            }
            else
                return false;
        }
    },
    /**
    * Section for handle the navigate toolbar right key process.
    * @private
    */
    _navigateToolbarRight: function (event) {
        var views = this.model.views;
        var curViewIndex = this.element.find(".e-" + this.currentView()).parent().index();
        if (curViewIndex < views.length - 1) {
            var newView = views[curViewIndex + 1].toLowerCase(); this._navView = true;
            this._viewChange(event, this.currentView(), newView);
            this.element.find("#" + this._id + "_navigator").ejDatePicker({ value: new Date(this.currentDate()) });
        }
    },
    /**
    * Section for handle the navigate toolbar left key process.
    * @private
    */
    _navigateToolbarLeft: function (event) {
        var curViewIndex = this.element.find(".e-" + this.currentView()).parent().index();
        if (curViewIndex > 0) {
            var newView = this.model.views[curViewIndex - 1].toLowerCase(); this._navView = true;
            this._viewChange(event, this.currentView(), newView);
            this.element.find("#" + this._id + "_navigator").ejDatePicker({ value: new Date(this.currentDate()) });
        }
    },
       
    /**
    * Section to open the new appointment window using keyboard.
    * @private
    */
    _newAppWindow: function (contextElement, oRowCell) {
        if ($(contextElement).hasClass("e-selectedCell")) {
            this._currentAction = ej.Schedule.Actions.Add;
            var targetElement = contextElement == null ? event : contextElement;
            var rowIndex = $(targetElement).parent().index();
            var columnIndex = $(targetElement).index();
            var resourceIndex = 0, renderDates = this.dateRender;
            var s = this.element.find('td.e-selectedCell');
            var ns = parseInt(s.length) - 1;
            var index = $(targetElement).hasClass("e-workcells") ? $(targetElement).index() : $(targetElement).hasClass("e-alldaycells") ? $(targetElement).index() - 1 : ($(targetElement).parent().index() * 7) + ($(targetElement).index() - (Math.floor($(targetElement).index() / 7) * 7)); //($(targetElement).parent().index() * 7) + $(targetElement).index();
            if (ns == 0) {
                startdate = new Date(renderDates[index]);
                if ($(targetElement).hasClass("e-workcells")) {
                    var time = this.model.startHour + (rowIndex / 2);
                    var timemin = time.toString().split(".");
                    startTime = new Date(startdate).setHours(parseInt(timemin[0]), parseInt(timemin[1]) == 5 ? 30 : 00);
                    endTime = new Date(new Date(startTime).setMinutes(new Date(startTime).getMinutes() + 30));
                }
                else if ($(targetElement).hasClass("e-alldaycells")) {
                    startTime = new Date(startdate.setHours(0, 0));
                    endTime = new Date(startTime.setHours(0, 0));
                    this._appointmentAddWindow.find(".allday ").ejCheckBox({ checked: true });
                }
                else if ($(targetElement).hasClass("e-monthcells")) {
                    startTime = new Date(new Date(startdate).setHours(9));
                    startTime = new Date(startTime).setMinutes(00); //.setHours(00)).setMinutes(00);
                    endTime = new Date(new Date(startTime).setMinutes(30)); //.setHours(23)).setMinutes(59));
                }
            }
            else if (ns > 0) {
                var indexes = this._findMinMaxCellIndex();
                var omincolIndex = indexes.omincolIndex, omaxcolIndex = indexes.omaxcolIndex, rowInx = indexes.rowInx, maxrowInx = indexes.maxrowInx;
                if ($(contextElement).hasClass('e-workcells')) {
                    var minStartDate = renderDates[omincolIndex]; var maxEndDate = renderDates[omaxcolIndex];
                    var minStartTime = (this.model.startHour + (rowInx / 2)).toString().split(".");
                    var maxEndTime = (this.model.startHour + (maxrowInx / 2)).toString().split(".");
                    startTime = new Date(new Date(minStartDate).setHours(parseInt(minStartTime[0]), parseInt(minStartTime[1]) == 5 ? 30 : 00));
                    endTime = new Date(maxEndDate).setHours(parseInt(maxEndTime[0]), parseInt(maxEndTime[1]) == 5 ? 30 : 00);
                    endTime = new Date(new Date(endTime).setMinutes(new Date(endTime).getMinutes() + 30));
                }
                else if ($(contextElement).hasClass('e-monthcells')) {
                    var selCell = this.element.find('td.e-selectedCell');
                    var nsc = selCell.length - 1;
                    var startIndex = ($(selCell[0]).parent().index() * 7) + ($(selCell[0]).index() - (Math.floor($(selCell[0]).index() / 7) * 7));
                    var endIndex = ($(selCell[nsc]).parent().index() * 7) + ($(selCell[nsc]).index() - (Math.floor($(selCell[nsc]).index() / 7) * 7)); // $(selCell[nsc]).index();
                    var startDate = new Date(this._dateRender[startIndex]);
                    var endDate = new Date(this._dateRender[endIndex]);
                    startTime = new Date(new Date(startDate).setHours(9));
                    startTime = new Date(new Date(startTime).setMinutes(00)); //.setHours(00)).setMinutes(00);
                    endTime = new Date(new Date(endDate).setHours(9)); //.setHours(23)).setMinutes(59)); 
                    endTime = new Date(new Date(endTime).setMinutes(30));
                }
            }
            if (ej.scheduleFeatures.resources && !ej.isNullOrUndefined(this.model.resources))
                this._renderResourceData($(targetElement), index);
            this._appointmentAddWindow.find("#" + this._id + "startdate").ejDatePicker("option", "value", Globalize.format(new Date(startTime), this._pattern.d));
            this._appointmentAddWindow.find("#" + this._id + "enddate").ejDatePicker("option", "value", Globalize.format(new Date(endTime), this._pattern.d));
            this._appointmentAddWindow.find("#" + this._id + "starttime").ejTimePicker("option", "value", Globalize.format(new Date(startTime), this._pattern.t));
            this._appointmentAddWindow.find("#" + this._id + "endtime").ejTimePicker("option", "value", Globalize.format(new Date(endTime), this._pattern.t));
            ($(contextElement).hasClass("e-alldaycells") || this.currentView() == "month") && this._appointmentAddWindow.find(".allday").ejCheckBox({ checked: true });
            this._alldayCheck();
            if (this.model.timeZone == null) {
                this._appointmentAddWindow.find("#startutc").val("UTC " + Globalize.format(new Date(startTime), "zzz"));
                this._appointmentAddWindow.find("#endutc").val("UTC " + Globalize.format(new Date(endTime), "zzz"));
            }
            else {
                this._appointmentAddWindow.find("#startutc").val(this.model.timeZone);
                this._appointmentAddWindow.find("#endutc").val(this.model.timeZone);
            }
           
            this._appointmentAddWindow.ejDialog("open");
            this._appointmentAddWindow.find('.subject').focus();
            this._cellClick = false;
        }

    },
    /**
    * Section for handle the tab key process.
    * @private
    */
    _processTab: function (oEl, event, oRowCell) {
        var appo = this.element.find('div.e-appointment,div.e-alldayappointment,div.e-monthappointment');
        if (oEl.id == this._id || $(oEl).hasClass("e-datedisplay")) {
            if (appo.length != 0)
                $(appo[0]).addClass("e-selectedAppointment").attr("tabIndex", 0).focus();
            else
                $(oRowCell[0].cells[0]).addClass("e-selectedCell").attr("tabIndex", 0).focus();
        }
        if ($(oEl).hasClass("e-selectedCell")) {
            if (appo.length != 0) {
                var selcel = this.element.find('td.e-selectedCell');
                var n = parseInt(selcel.length) - 1;
                if (n > 0) {
                    for (var i = 0; i <= n; i++) {
                        selcel[i].blur();
                        $(selcel[i]).removeClass("e-selectedCell");
                    }
                }
                else
                    $(oEl).removeClass("e-selectedCell").blur();
                $(appo[0]).addClass("e-selectedAppointment").attr("tabIndex", 0).focus();
            }
        }
        if ($(oEl).hasClass("e-navigateprevious")) {
            this.element.find('.e-navigatenext').attr("tabIndex", 0).focus();
        }
        else if ($(oEl).hasClass("e-navigatenext"))
            $(oRowCell[0].cells[0]).addClass("e-selectedCell").attr("tabIndex", 0).focus();
        if ($(oEl).hasClass("e-appointment") || $(oEl).hasClass("e-alldayappointment") || $(oEl).hasClass("e-monthappointment")) {
            var appcount = appo.length;
            var appindex = this.element.find("div.e-appointment,div.e-alldayappointment,div.e-monthappointment").index($(".e-selectedAppointment"));
            if (appindex < appcount && appindex >= 0) {
                $(appo[appindex]).removeClass("e-selectedAppointment").blur();
                var newappindex = parseInt(appindex) + 1;
                if (newappindex < appcount)
                    $(appo[newappindex]).addClass("e-selectedAppointment").attr("tabIndex", 0).focus();
                else if (newappindex == appcount)
                    this.element.find('.e-navigateprevious').attr("tabIndex", 0).focus();
            }
            else if (appindex == appcount)
                this.element.find('.e-navigateprevious').attr("tabIndex", 0).focus();
        }
    },
    /**
    * Section for handle the shift tab key process.
    * @private
    */
    _reverseTab: function (oEl, event, oRowCell) {
        var appo = this.element.find('div.e-appointment,div.e-alldayappointment,div.e-monthappointment');
        if (oEl.id == this._id || $(oEl).hasClass("e-datedisplay")) {
            var len = parseInt(appo.length) - 1;
            if (appo.length != 0)
                $(appo[len]).addClass("e-selectedAppointment").attr("tabIndex", 0).focus();
        }
        var appcount = appo.length;
        if ($(oEl).hasClass("e-selectedCell")) {
            var len = appo.length - 1;
            if (appo.length != 0) {
                var selcel = this.element.find('td.e-selectedCell');
                var n = parseInt(selcel.length) - 1;
                if (n > 0) {
                    for (var i = 0; i <= n; i++)
                        $(selcel[i]).removeClass("e-selectedCell").blur();
                }
                else
                    $(oEl).removeClass("e-selectedCell").blur();
                $(appo[len]).addClass("e-selectedAppointment").attr("tabIndex", 0).focus();
            }
        }
        if ($(oEl).hasClass("e-navigatenext"))
            $(this.element).find('.e-navigateprevious').attr("tabIndex", 0).focus();
        else if ($(oEl).hasClass("e-navigateprevious"))
            $(oRowCell[0].cells[0]).addClass("e-selectedCell").attr("tabIndex", 0).focus();
        if ($(oEl).hasClass("e-appointment") || $(oEl).hasClass("e-alldayappointment") || $(oEl).hasClass("e-monthappointment")) {
            var appindex = this.element.find("div.e-appointment,div.e-alldayappointment,div.e-monthappointment").index($(".e-selectedAppointment"));
            if (appindex < appcount && appindex > 0) {
                $(appo[appindex]).removeClass("e-selectedAppointment").blur();
                var newappindex = parseInt(appindex) - 1;
                if (newappindex < appcount && appindex >= 0)
                    $(appo[newappindex]).addClass("e-selectedAppointment").attr("tabIndex", 0).focus();
                else if (newappindex == 0)
                    $(this.element).find('.e-navigatenext').attr("tabIndex", 0).focus();
            }
            else if (appindex == 0) {
                $(appo[appindex]).removeClass("e-selectedAppointment").blur();
                $(this.element).find('.e-navigatenext').attr("tabIndex", 0).focus();
            }
        }
    },
    /**
    * Section to stop the default action.
    * @private
    */
    _stopDefaultAction: function (e) {
        e.cancelBubble = true; e.returnValue = false; e.stopPropagation(); e.preventDefault();
    },
    /**
    * To get the localized labels.
    * @private
    */
    //get locale text
    _getLocalizedLabels: function (property) {
        return ej.Schedule.Locale[this.model.locale][property] === undefined ? ej.Schedule.Locale["en-US"][property] : ej.Schedule.Locale[this.model.locale][property];
    },
    /**
    * It is used to save the appointment. The appointment obj is based on the argument passed along with this method.  
    * @return jQuery
    * @param {object} argument.obj obj of the appointment
    * @example 
    * &lt;div id="Schedule"&gt;&lt;/div&gt; <br>
    * &lt;script&gt;
    * $('#Schedule').ejSchedule({
        *     appointmentSettings: { dataSource: window.Default, //collection of object from dataSource.
        *               id: "Id",
        *               subject: "Subject",
        *               description: "Description",
        *               startTime: "StartTime",
        *               endTime: "EndTime",
        *               recurrence: "Recurrence",
        *               recurrenceRule: "RecurrenceRule",
        *               allDay: "AllDay"
        *             }                            
        *   });
        * var schObj = $("#Schedule").data("ejSchedule");
    * schObj.saveAppointment(obj); //obj contains collection of appointment. 
    * &lt;/script&gt;
    * @memberof ejSchedule
    * @instance
    */
    //public methods
    saveAppointment: function (obj) {
        var appointments = this._sortAppById(this._processed), maxId;
        maxId = appointments.length > 0 ? appointments[appointments.length - 1][this.model.appointmentSettings["id"]] + 1 : 1;
        if (ej.isNullOrUndefined(obj[this.model.appointmentSettings["id"]]) || obj[this.model.appointmentSettings["id"]] == "") {
            obj[this.model.appointmentSettings["id"]] = maxId;
            this._dataManager.insert(obj);
            this._appointmentProcessing(obj);
        }
        else {
            this._dataManager.update(this.model.appointmentSettings["id"], obj);
            obj[this.model.appointmentSettings["recurrence"]] ? this._dataProcessing(this._currentAppointmentData) : new ej.DataManager(this._processed).update(this.model.appointmentSettings["id"], obj);
        }
        this._renderAppointmentAll();
    },
    /**
    * It is used to print the schedule. 
    * @return jQuery
    * @example 
    * &lt;div id="Schedule"&gt;&lt;/div&gt; <br>
    * &lt;script&gt;
    * $('#Schedule').ejSchedule({
        *     appointmentSettings: { dataSource: window.Default, //collection of object from dataSource.
        *               id: "Id",
        *               subject: "Subject",
        *               description: "Description",
        *               startTime: "StartTime",
        *               endTime: "EndTime",
        *               recurrence: "Recurrence",
        *               recurrenceRule: "RecurrenceRule",
        *               allDay: "AllDay"
        *             }                            
        *   });
        * var schObj = $("#Schedule").data("ejSchedule");
    * schObj.print();
    * &lt;/script&gt;
    * @memberof ejSchedule
    * @instance
    */
    //public methods
    print:function(){
    ej.print(this.element);
    },
    /**
    * It is used to delete the appointment. The id is based on the argument passed along with this method  
    * @return jQuery
    * @param {number} argument.id id value of the appointment
    * @example 
    * &lt;div id="Schedule"&gt;&lt;/div&gt; <br> 
    * &lt;script&gt;
    * $('#Schedule').ejSchedule({
        *     appointmentSettings: { dataSource: window.Default, //collection of object from dataSource.
        *               id: "Id",
        *               subject: "Subject",
        *               description: "Description",
        *               startTime: "StartTime",
        *               endTime: "EndTime",
        *               recurrence: "Recurrence",
        *               recurrenceRule: "RecurrenceRule",
        *               allDay: "AllDay"
        *             }                            
        *   });
        * var schObj = $("#Schedule").data("ejSchedule");
        * schObj.deleteAppointment(105); //Appointments id number.
    * &lt;/script&gt;
    * @memberof ejSchedule
    * @instance
    */
    deleteAppointment: function (id) {
        this._deleteApp(id);
        //this._renderAppointmentAll();
    },
    /**
    * It is used to get the appointment list of schedule control. 
        * @return Appointments
    * @example 
    * &lt;div id="Schedule"&gt;&lt;/div&gt; <br> 
    * &lt;script&gt;
    * $('#Schedule').ejSchedule({
        *     appointmentSettings: { dataSource: window.Default, //collection of object from dataSource.
        *               id: "Id",
        *               subject: "Subject",
        *               description: "Description",
        *               startTime: "StartTime",
        *               endTime: "EndTime",
        *               recurrence: "Recurrence",
        *               recurrenceRule: "RecurrenceRule",
        *               allDay: "AllDay"
        *             }                            
        *   });
        * var schObj = $("#Schedule").data("ejSchedule");
        * var appointments=schObj.getAppointments(); // Gets the appointments list of schedule control
    * &lt;/script&gt;
    * @memberof ejSchedule
    * @instance
    */
    getAppointments: function () {
        var appList = this.dataSource();
        return appList;
    },
    /**
    * It used search the appointments from appointment list of schedule control. 
    * @return Appointments
    * @param {string} argument.searchString searchString value of the search word in the appointments.
    * @param {string} argument.fields fields value of which feilds need search.
    * @param {string} argument.filterOperator filterOperator value of the search operation need to done.
    * @param {boolean} argument.ignoreCase ignoreCase value of the case.  
    * @example 
    * &lt;div id="Schedule"&gt;&lt;/div&gt; <br> 
    * &lt;script&gt;
    * $('#Schedule').ejSchedule({
        *     appointmentSettings: { dataSource: window.Default, //collection of object from dataSource.
        *               id: "Id",
        *               subject: "Subject",
        *               description: "Description",
        *               startTime: "StartTime",
        *               endTime: "EndTime",
        *               recurrence: "Recurrence",
        *               recurrenceRule: "RecurrenceRule",
        *               allDay: "AllDay"
        *             }                            
        *   });
        * var schObj = $("#Schedule").data("ejSchedule");
        * var appointments=schObj.searchAppointments("gold"); // Gets the appointments list of schedule control
    * &lt;/script&gt;
    * @memberof ejSchedule
    * @instance
    */
    searchAppointments: function (searchString, fields, filterOperator, ignoreCase) {
        var queryManagar = ej.Query();
        var result, _fields = ej.isNullOrUndefined(fields) ? this._getColumns(this.model.appointmentSettings) : fields;
        filterOperator = ej.isNullOrUndefined(filterOperator) ? "contains" : filterOperator;
        ignoreCase = ej.isNullOrUndefined(ignoreCase) ? true : ignoreCase;
        if (typeof searchString == "string") {
            if (searchString.length) {
                queryManagar.search(searchString, _fields, filterOperator, ignoreCase);
                if (!(this.dataSource() instanceof ej.DataManager))
                    result = this._dataManager.executeLocal(queryManagar);
            }
        }
        else if (typeof searchString == "object")
            result = this.filterAppointments(searchString);
        return result;
    },
   /**
    * It used search the appointments from appointment list of schedule control. 
    * @return Appointments
    * @param {object} argument.filterConditions filterConditions value of the filter collections for appointments. 
    * @example 
    * &lt;div id="Schedule"&gt;&lt;/div&gt; <br> 
    * &lt;script&gt;
    * $('#Schedule').ejSchedule({
        *     appointmentSettings: { dataSource: window.Default, //collection of object from dataSource.
        *               id: "Id",
        *               subject: "Subject",
        *               description: "Description",
        *               startTime: "StartTime",
        *               endTime: "EndTime",
        *               recurrence: "Recurrence",
        *               recurrenceRule: "RecurrenceRule",
        *               allDay: "AllDay"
        *             }                            
        *   });
        * var schObj = $("#Schedule").data("ejSchedule");
        * var filter=[{
        *    field: "Subject",
        *    operator: "contains",
        *    value: "gold",
        *    predicate: "or"
        *}];
        * var appointments=schObj.filterAppointments(filter); // Gets the appointments list of schedule control
    * &lt;/script&gt;
    * @memberof ejSchedule
    * @instance
    */
    filterAppointments: function (filterConditions) {
        var result, queryManagar = ej.Query();
        if(!ej.isNullOrUndefined(filterConditions)) {
            var firstFilterCondition = filterConditions[0];
            predicate = ej.Predicate(firstFilterCondition.field, firstFilterCondition.operator, firstFilterCondition.value, !firstFilterCondition.matchcase);
            for (var i = 1; i < filterConditions.length; i++)
                predicate = predicate[filterConditions[i].predicate](filterConditions[i].field, filterConditions[i].operator, filterConditions[i].value, !(filterConditions[i].matchcase));
            queryManagar.where(predicate);
            if (!(this.dataSource() instanceof ej.DataManager))
                result = this._dataManager.executeLocal(queryManagar);
        }
        return result;
    },
    /**
    * It used to Export the appointments from  of schedule control. 
    * @return null
    * @param {string} argument.page to refer the controller action name to redirect. (For MVC)
    * @param {string} argument.serverEvents to refer the server event name.(For ASP)
    * @example 
    * &lt;div id="Schedule"&gt;&lt;/div&gt; <br> 
    * &lt;script&gt;
    * $('#Schedule').ejSchedule({
        *     appointmentSettings: { dataSource: window.Default, //collection of object from dataSource.
        *               id: "Id",
        *               subject: "Subject",
        *               description: "Description",
        *               startTime: "StartTime",
        *               endTime: "EndTime",
        *               recurrence: "Recurrence",
        *               recurrenceRule: "RecurrenceRule",
        *               allDay: "AllDay"
        *             }                            
        *   });
        * var schObj = $("#Schedule").data("ejSchedule");
        * schObj.export("ActionName","ExportToICS"); // To Export the Appointment
    * &lt;/script&gt;
    * @memberof ejSchedule
    * @instance
    */
    export: function (page,serverEvent) {
        this._exportSchedule(page,serverEvent);
    },
    /**
    * Section to export the file.
    * @private
    */
    _exportSchedule: function (action, serverEvent) {
        var modelClone = JSON.parse(JSON.stringify(this.model));
        var appointmentClone = JSON.parse(JSON.stringify(this.model.appointmentSettings));
        var attr = { action: action, method: 'post' };
        var form = ej.buildTag('form', "", null, attr);
        if (this._ignoreFromAppointmentOnExport) {
            for (var i = 0; i < this._ignoreFromAppointmentOnExport.length; i++) {
                delete appointmentClone[this._ignoreFromAppointmentOnExport[i]];
            }
        }
        if (ej.raiseWebFormsServerEvents) {
            var args = { model: modelClone, originalEventType: serverEvent };
            var clientArgs = { model: JSON.stringify(appointmentClone) };
            ej.raiseWebFormsServerEvents(serverEvent, args, clientArgs);
        }
        else {
            var inputAttr = { name: 'ScheduleModel', type: 'hidden', value: JSON.stringify(appointmentClone) }
            var input = ej.buildTag('input', "", null, inputAttr);
            form.append(input);
            form.append(this);
            $('body').append(form);
            form.submit();
        }
        return true;
    }
});

    if (ej.scheduleFeatures.resources)
        $.extend(ej.Schedule.prototype, ej.scheduleFeatures.resources);
    if (ej.scheduleFeatures.horizontal)
        $.extend(ej.Schedule.prototype, ej.scheduleFeatures.horizontal);


//Schedule Localization 
ej.Schedule.Locale = {};

//Default english localized values
ej.Schedule.Locale["en-US"] = {
    ReminderWindowTitle: "Reminder window",
    CreateAppointmentTitle: "Create Appointment",
    RecurrenceEditTitle: "Edit repeat appointment",
    RecurrenceEditMessage: "How would you like to change the event in the series?",
    RecurrenceEditOnly: "Only this appointment",
    RecurrenceEditSeries: "Entire series",
    PreviousAppointment: "Previous Appointment",
    NextAppointment:"Next Appointment",
    AppointmentSubject: "Subject",
    StartTime: "Start Time",
    EndTime: "End Time",
    AllDay: "All day",
    Today: "Today",
    Recurrence: "Repeat",
    Done: "Done",
    Cancel: "Cancel",
    Ok: "Ok",
    RepeatBy: "Repeat by",
    RepeatEvery: "Repeat every",
    RepeatOn: "Repeat on",
    StartsOn: "Starts on",
    Ends: "Ends",
    Summary: "Summary",
    Daily: "Daily",
    Weekly: "Weekly",
    Monthly: "Monthly",
    Yearly: "Yearly",
    Every: "Every",
    EveryWeekDay: "Every weekday",
    Never: "Never",
    After: "After",
    Occurence: "Occurrence(s)",
    On: "On",
    Edit: "Edit",
    RecurrenceDay: "Day(s)",
    RecurrenceWeek: "Week(s)",
    RecurrenceMonth: "Month(s)",
    RecurrenceYear: "Year(s)",
    The: "The",
    OfEvery: "of every",
    First: "First",
    Second: "Second",
    Third: "Third",
    Fourth: "Fourth",
    Last: "Last",
    WeekDay: "Weekday",
    WeekEndDay: "Weekendday",
    Subject: "Subject",
    Categorize: "Categories",
    DueIn: "Due In",
    DismissAll: "Dismiss All",
    Dismiss: "Dismiss",
    OpenItem: "Open Item",
    Snooze: "Snooze",
    Day: "Day",
    Week: "Week",
    WorkWeek: "Work Week",
    Month: "Month",
    Agenda: "Agenda",
    Detailed: "Detailed",
    EventBeginsin: "Appointment Begins in",
    Editevent: "Edit Appointment",
    Editseries: "Edit series",
    Times: "times",
    Until: "until",
    Eventwas: "Appointment was",
    Hours: "hrs",
    Minutes: "mins",
    Overdue: "Overdue Appointment",
    Days: "day(s)",
    Event: "Subject",
    Select: "select",
    Previous: "prev",
    Next: "next",
    Close: "close",
    Delete: "delete",
    Date: "Date",
    Showin: "Show in",
    Gotodate: "Go To Date",
    RESOURCES: "RESOURCES" 
};
/**
* Enum for various schedule actions while create/edit appointments	 
* @enum {string}
* @global 
*/
ej.Schedule.Actions = {
    /**  support to know the action as add while create appointment in the schedule */
    Add: "add",
    /**  support to know the action as save while create appointment in the schedule */
    Save: "save",
    /**  support to know the action as delete while delete appointment in the schedule */
    Delete: "delete",
    /**  support to know the action as edit occurance while editin the occurrence appointment */
    EditOcurrence: "editOcurrence",
    /**  support to know the action as edit series while editing the complete series appointment */
    EditSeries: "editSeries"
};
/**
* Enum for various schedule view	 
* @enum {string}
* @global 
*/
ej.Schedule.CurrentView = {
    /**  Creates schedule with current view as day */
    Day: "day",
    /**  Creates schedule with current view as week */
    Week: "week",
    /**  Creates schedule with current view as work week */
    Workweek: "workweek",
    /**  Creates schedule with current view as month */
    Month: "month"
};
/**
* Enum for various schedule timemode	 
* @enum {string}
* @global 
*/
ej.Schedule.TimeMode = {
    /**  Creates schedule with time mode as twelve hour */
    Hour12: "12",
    /**  Creates schedule with time mode as twenty four hour */
    Hour24: "24"
};

    /**
    * Enum for schedule orientation	 
    * @enum {string}
    * @global 
    */
ej.Schedule.Orientation = {
    /**  Creates schedule in vertical orientation */
    Vertical: "vertical",
    /**  Creates schedule in horizontal orientation */
    Horizontal: "horizontal"
};

})(jQuery, Syncfusion);;
/**
* @fileOverview Plugin to create the Gantt
* @copyright Copyright Syncfusion Inc. 2001 - 2013. All rights reserved.
*  Use of this code is subject to the terms of our license.
*  A copy of the current license can be obtained at any time by e-mailing
*  licensing@syncfusion.com. Any infringement will be prosecuted under
*  applicable laws. 
* @version 12.1 
* @author <a href="mailto:licensing@syncfusion.com">Syncfusion Inc</a>
*/

(function ($, ej, undefined) {
    /**
* @namespace ej
* @class ejGantt
* @requires jQuery
* @requires jquery.globalize.js
* @requires jquery.easing.1.3.js
* @requires jsrender.js
* @requires ej.web.all.js
* @classdesc Custom Design for Html Gantt control.
* @example 
* &lt;gantt id="gantt"&gt;Gantt&lt;/gantt&gt; <br> 
* &lt;script&gt;
* // Create Gantt
* $('#gantt').ejGantt(); 	
* &lt;/script&gt;
*/
   
    ej.widget("ejGantt", "ej.Gantt", {

        // widget element will be automatically set in this
        _rootCSS: "e-gantt",

        element: null,

        validTags: ["div"],
        // user defined model will be automatically set in this
        model: null,

        defaults: {
            /**		
            * Enables or disables sorting. When enabled, we can sort the column by clicking on the column.
            * @default false
            * @type {boolean}
            * @example 
            * &lt;div id="gantt"&gt;&lt;/div&gt; 
            * &lt;script&gt;        
            * 	$("#gantt").ejGantt({ allowSorting:  true });            
            * &lt;/script&gt;
            * @memberof ejGantt
			* @instance
            */
            allowSorting: false,

            /**		
            * Enables or disables the ability to resize column.
            * @default false
            * @type {boolean}
            * @example 
            * &lt;div id="gantt"&gt;&lt;/div&gt; 
            * &lt;script&gt;                   
            * 	$("#gantt").ejGantt({ allowColumnResize:  true });			* 
            * &lt;/script&gt;
            * @memberof ejGantt
			* @instance
            */
            allowColumnResize: false,

            /**		
            * Enables or disables the interactive selection of a row.
            * @default true
            * @type {boolean}
            * @example 
            * &lt;div id="gantt"&gt;&lt;/div&gt; 
            * &lt;script&gt;   
            * 	$("#gantt").ejGantt({ allowSelection:  true });			* 
            * &lt;/script&gt;
            * @memberof ejGantt
			* @instance
            */
            allowSelection: true,

            /**		
            * Collection of data or hierarchical data  to represent in gantt
            * @default 
            * @type {array}
            * @example 
            * &lt;div id="gantt"&gt;&lt;/div&gt; 
            * &lt;script&gt; 		
            * 	$("#gantt").ejGantt(
            *  {
            *     dataSource:[{Id:2,TaskName:"Testing",startDate:"12/1/2000",Duration:5 }]	
            *  });		
            * &lt;/script&gt;
            * @memberof ejGantt
			* @instance
            */
            dataSource:null,

            /**		
            * It receives query to retrieve data from the table (query is same as SQL). 
            * @default 
            * @type {object}
            * @example
            * &lt;div id="gantt"&gt;&lt;/div&gt; 
            * &lt;script&gt;  			
            * 	$("#gantt").ejGantt(
            *  {
            *     query:ej.Query().from("Categories").select("CategoryID,CategoryName").take(3);	
            *  });		
            * &lt;/script&gt;
            * @memberof ejGantt
			* @instance
            */
            query: null,

           /**		
           * Specifies the mapping property path for task Id in datasource
           * @default ""
           * @type {string}
           * @example 
           * &lt;div id="gantt"&gt;&lt;/div&gt; 
           * &lt;script&gt;  				
           * 	$("#gantt").ejGantt({  taskIdMapping : "ID" });	
           * &lt;/script&gt;
           * @memberof ejGantt
			* @instance
           */
            taskIdMapping: "",

            /**		
            * Specifies the mapping property path for parent task Id in selfreference datasource
            * @default ""
            * @type {string}
            * @example 
            * &lt;div id="gantt"&gt;&lt;/div&gt; 
            * &lt;script&gt;  		
            * 	$("#gantt").ejGantt({  parentTaskIdMapping : "ID" });	            
            * &lt;/script&gt;
            * @memberof ejGantt
			* @instance
            */
            parentTaskIdMapping: "",
            
            /**		
            * Specifies the mapping property path for task name in datasource
            * @default ""
            * @type {string}
            * @example 
            * &lt;div id="gantt"&gt;&lt;/div&gt; 
            * &lt;script&gt;		
            * 	$("#gantt").ejGantt({  taskNameMapping : "Name" });	
            * &lt;/script&gt;
            * @memberof ejGantt
			* @instance
            */
            taskNameMapping: "",

            /**		
            * Specifies the mapping property path for start date of a task  in datasource
            * @default ""
            * @type {string}
            * @example 
            * &lt;div id="gantt"&gt;&lt;/div&gt; 
            * &lt;script&gt;		 			
            * 	$("#gantt").ejGantt({  startDateMapping : "startdate" });	         
            * &lt;/script&gt;
            * @memberof ejGantt
			* @instance
            */
            startDateMapping: "",

            /**		
           * Specifies the mapping property path for end Date of a task  in datasource
           * @default ""
           * @type {string}
           * @example 
           * &lt;div id="gantt"&gt;&lt;/div&gt; 
           * &lt;script&gt;				
           * 	$("#gantt").ejGantt({  endDateMapping : "EndDate" });	
           * &lt;/script&gt;
           * @memberof ejGantt
		   * @instance
           */
            endDateMapping: "",

            /**		
           * Specifies the mapping property path for baseline start date of a task  in datasource
           * @default ""
           * @type { string}
           * @example 
           * &lt;div id="gantt"&gt;&lt;/div&gt; 
           * &lt;script&gt;			
           * 	$("#gantt").ejGantt({  baselineStartDateMapping : "BaselineStartDate" });		
           * &lt;/script&gt;
           * @memberof ejGantt
			* @instance
           */
            baselineStartDateMapping: "",

            /**		
            *Specifies the mapping property path for baseline end date in datasource
            * @default ""
            * @type {string}
            * @example 
            * &lt;div id="gantt"&gt;&lt;/div&gt; 
            * &lt;script&gt;			
            * 	$("#gantt").ejGantt({  baselineEndDateMapping : "BaselineEndDate" });
            * &lt;/script&gt;
            * @memberof ejGantt
			* @instance
            */
            baselineEndDateMapping: "",

            /**		
            * Specifies the mapping property path for sub tasks in datasource 
            * @default ""
            * @type {string}
            * @example 
            * &lt;div id="gantt"&gt;&lt;/div&gt; 
            * &lt;script&gt;				
            * 	$("#gantt").ejGantt({  childMapping : "Children" });
            * &lt;/script&gt;
            * @memberof ejGantt
			* @instance
            */
            childMapping: "",

           /**		
           * Specifies the mapping property path for duration of a task in datasource
           * @default ""
           * @type {string}
           * @example 
           * &lt;div id="gantt"&gt;&lt;/div&gt; 
           * &lt;script&gt;		
           * 	$("#gantt").ejGantt({  durationMapping : "Duration" });
           * &lt;/script&gt;
           * @memberof ejGantt
			* @instance
           */
            durationMapping: "",

            /**		
           * Specifies the mapping property path for milestone in datasource
           * @default ""
           * @type {string}
           * @example 
           * &lt;div id="gantt"&gt;&lt;/div&gt; 
           * &lt;script&gt;				
           * 	$("#gantt").ejGantt({  milestoneMapping : "milestone" });	
           * &lt;/script&gt;
           * @memberof ejGantt
			* @instance
           */
            milestoneMapping: "",

           /**		
           *Specifies the mapping property path for  progress percentage  of a task in datasource
           * @default ""
           * @type {string}
           * @example 
           * &lt;div id="gantt"&gt;&lt;/div&gt; 
           * &lt;script&gt;			
           * 	$("#gantt").ejGantt({  progressMapping : "progress" });			
           * &lt;/script&gt;
           * @memberof ejGantt
			* @instance
           */
            progressMapping: "Progress",

            /**		
           *Specifies the mapping property path for predecessors  of a task in datasource
           * @default ""
           * @type {string}
           * @example 
           * &lt;div id="gantt"&gt;&lt;/div&gt; 
           * &lt;script&gt;				
           * 	$("#gantt").ejGantt({  predecessorMapping : "predecessor" });		
           * &lt;/script&gt;
           * @memberof ejGantt
			* @instance
           */
            predecessorMapping: "",

            /**		
            * Specifies the mapping property path for resources of a task in datasource
            * @default ""
            * @type {string}
            * @example 
            * &lt;div id="gantt"&gt;&lt;/div&gt; 
            * &lt;script&gt;				
            * 	$("#gantt").ejGantt({  resourceInfoMapping : "resources" });	
            * &lt;/script&gt;
            * @memberof ejGantt
			* @instance
            */
            resourceInfoMapping: "",

            /**		
           * Collection of data  regarding resources involved in entire project       
           * @default []
           * @type {array}
           * @example 
           * &lt;div id="gantt"&gt;&lt;/div&gt; 
           * &lt;script&gt;			
           * 	$("#gantt").ejGantt(
           *  {
           *    resources:[{id:1; name:"jack" }]	
           *  });		
           * &lt;/script&gt;
           * @memberof ejGantt
			* @instance
           */
            resources: [],

            /**		
           * Collection of holidays with date, background and label information to be displayed in gantt. 
           * @default []
           * @type {array}
           * @example 
           * &lt;div id="gantt"&gt;&lt;/div&gt; 
           * &lt;script&gt;				
           * 	$("#gantt").ejGantt(
           *  {
           *    holidays:[{day:"12/2/2000",background:"cyan",label:"local holiday" }]	
           *  });		
           * &lt;/script&gt;
           * @memberof ejGantt
		   * @instance
           */
            holidays: [],


            /**		
           * Specifies whether to highlight the weekends in gantt .
           * @default true
           * @type {boolean}
           * @example 
           * &lt;div id="gantt"&gt;&lt;/div&gt; 
           * &lt;script&gt;	
           * 	$("#gantt").ejGantt({ highlightWeekends:  true });			* 
           * &lt;/script&gt;
           * @memberof ejGantt
			* @instance
           */
            highlightWeekends: true,

            /**		
           * Specifies start date of the gantt schedule. By default, start date will be rounded to its previous Sunday.
           * @default null
           * @type {string}
           * @example 
           * &lt;div id="gantt"&gt;&lt;/div&gt; 
           * &lt;script&gt;				
           * 	$("#gantt").ejGantt(
           *  {
           *     scheduleStartDate:"12/2/2000"
           *  });		
           * &lt;/script&gt;
           * @memberof ejGantt
			* @instance
           */
            scheduleStartDate: null,


            /**		
           * Specifies end date of the gantt schedule. By default, end date will be rounded to its next Saturday.
           * @default null
           * @type {string}
           * @example 
           * &lt;div id="gantt"&gt;&lt;/div&gt; 
           * &lt;script&gt;	 			
           * 	$("#gantt").ejGantt(
           *  {
           *     scheduleEndDate:"12/2/2000"
           *  });		
           * &lt;/script&gt;
           * @memberof ejGantt
			* @instance
           */
            scheduleEndDate: null,

            /**		
           * Indicates whether we can edit the progress of a task interactively in gantt chart.
           * @default true
           * @type {boolean}
           * @example 
           * &lt;div id="gantt"&gt;&lt;/div&gt; 
           * &lt;script&gt;	
           * 	$("#gantt").ejGantt({ enableProgressBarResizing:  true });			
           * &lt;/script&gt;
           * @memberof ejGantt
			* @instance
           */
            enableProgressBarResizing: true,

            /**		
           * Specifies the height of a single row in gantt. Also, we need to set same height in the CSS style with class name e-rowcell. 
           * @default "30"
           * @type {number}
           * @example 
           * &lt;div id="gantt"&gt;&lt;/div&gt; 
           * &lt;script&gt;				
           * 	$("#gantt").ejGantt({  
           *			rowHeight : 30,
           * 			});
           * &lt;/script&gt;
           * @memberof ejGantt
			* @instance
           */
            rowHeight: 30,

           /**		
           * Specifies whether to include weekends while calculating the duration of a task. 
           * @default true
           * @type {boolean}
           * @example 
           * &lt;div id="gantt"&gt;&lt;/div&gt; 
           * &lt;script&gt;	
           * 	$("#gantt").ejGantt({ includeWeekend:  true });			
           * &lt;/script&gt;
           * @memberof ejGantt
			* @instance
           */
            includeWeekend: true,

            /**		
			* Specifies the toolbarSettings options.
			* @type {object}
            * @memberof ejGantt
			* @instance
			*/
            toolbarSettings: {

                /**		
                * Specifies the state of enabling or disabling toolbar
                 * @alias ejGantt#toolbarSettings->showToolBar
                * @default true
                * @type {boolean}
                * @example 
                * &lt;div id="gantt"&gt;&lt;/div&gt; 
                * &lt;script&gt;	
                * 	$("#gantt").ejGantt({ showToolBar:  true });			 
                * &lt;/script&gt;
                * @memberof ejGantt
			    * @instance
                */
                showToolbar: false,


                /**		
                * Specifies the list of toolbar items to rendered in toolbar
                * @alias ejGantt#toolbarSettings->toolbarItems
                * @default []
                * @type {array}
                * @example 
                * &lt;div id="gantt"&gt;&lt;/div&gt; 
                * &lt;script&gt;	
                * 	$("#gantt").ejGantt({ toolbarItems: [ej.Gantt.ToolbarItems.Add,ej.Gantt.ToolbarItems.Edit] });			 
                * &lt;/script&gt;
                * @memberof ejGantt
			    * @instance
                */
                toolbarItems: []
            },

            /**		
            * Specifies the options for striplines
            * @default "[]"
            * @type {string}
            * @example 
            * &lt;div id="gantt"&gt;&lt;/div&gt; 
            * &lt;script&gt;			
            * 	$("#gantt").ejGantt(stripLines: [
            *    {
            *     day: "12/2/2000,
            *     label: "Project Release",
            *     lineStyle: "dotted",
            *     lineColor: "Darkblue",
            *     lineWidth: 2
            *   }
            * ]); 
            * &lt;/script&gt;
            * @memberof ejGantt
			* @instance
            */
            stripLines: [],            
            /**		
            * Specifies the working time schedule fo day           
            * @default "TimeScale24"
            * @type {enum}
            * @example 
            * &lt;div id="gantt"&gt;&lt;/div&gt; 
            * &lt;script&gt;				
            * 	$("#gantt").ejGantt({  
            *			workingTimeScale : "TimeScale24Hours" 
            });
            * &lt;/script&gt;
            * @memberof ejGantt
			* @instance
            */
            workingTimeScale: 'TimeScale8Hours',


            /**		
            * Specifies whether rounding off the day working time edits
          
            * @default "true"   
            * @type {boolean}
            * @example 
            * &lt;div id="gantt"&gt;&lt;/div&gt; 
            * &lt;script&gt;				
            * 	$("#gantt").ejGantt({  
            *			roundOffDayworkingTime : true });
            * &lt;/script&gt;
            * @memberof ejGantt
            * @instance
            */
            roundOffDayworkingTime: true,

            /**		
            * Specifies the duration unit for each tasks whether days or hours
           
            * @default "day"
            * @type {enum}
            * @example 
            * &lt;div id="gantt"&gt;&lt;/div&gt; 
            * &lt;script&gt;				
            * 	$("#gantt").ejGantt({  
            *			durationUnit : day });
            * &lt;/script&gt;
            * @memberof ejGantt
            * @instance
            */
            durationUnit: 'day',
            /**		
            * Specified the format for weekview for schedule header
            * @alias ejGantt#scheduleHeaderSettings->weekHeaderFormat
            * @default "MMM dd , yyyy"
            * @type {string}
            * @example 
            * &lt;div id="gantt"&gt;&lt;/div&gt; 
            * &lt;script&gt;				
            * 	$("#gantt").ejGantt({  
            *			scheduleHeaderSettings:{weekHeaderFormat : "MMM dd , yyyy" }
            });
            * &lt;/script&gt;              
            */
            /**		
			* Specifies the options for customizing schedule header. 
			* @type {object}
            * @memberof ejGantt
			* @instance
			*/
            scheduleHeaderSettings:
            {
                /**		
               * Specified the format for week view in schedule header
               * @alias ejGantt#scheduleHeaderSettings->weekHeaderFormat
               * @default "ddd"
               * @type {string}
               * @example 
               * &lt;div id="gantt"&gt;&lt;/div&gt; 
               * &lt;script&gt;				
               * 	$("#gantt").ejGantt({  
               *			scheduleHeaderSettings:{weekHeaderFormat : "MMM dd , yyyy" }
               });
               * &lt;/script&gt;                                  */
                
                weekHeaderFormat: "MMM dd , yyyy",

                /**		
                * Specified the format for day view in schedule header
                * @alias ejGantt#scheduleHeaderSettings->dayHeaderFormat
                * @default "ddd"
                * @type {string}
                * @example 
                * &lt;div id="gantt"&gt;&lt;/div&gt; 
                * &lt;script&gt;				
                * 	$("#gantt").ejGantt({  
                *			scheduleHeaderSettings:{dayHeaderFormat : "ddd" }
                });
                * &lt;/script&gt;              
		
                */
                dayHeaderFormat: "ddd",

                /**		
               * Specified the format for year view in schedule header
               * @alias ejGantt#scheduleHeaderSettings->yearHeaderFormat
               * @default "yyyy"
               * @type {string}
               * @example 
               * &lt;div id="gantt"&gt;&lt;/div&gt; 
               * &lt;script&gt;				
               * 	$("#gantt").ejGantt({  
               *			scheduleHeaderSettings:{yearHeaderFormat : "yyyy" }
               });
               * &lt;/script&gt;               
               */
                yearHeaderFormat: "yyyy",

                /**		
               * Specified the format for month view in schedule header
               * @alias ejGantt#scheduleHeaderSettings->monthHeaderFormat
               * @default "MMM"
               * @type {string}
               * @example 
               * &lt;div id="gantt"&gt;&lt;/div&gt; 
               * &lt;script&gt;				
               * 	$("#gantt").ejGantt({  
               *			scheduleHeaderSettings:{monthHeaderFormat : "MMM" }
               });
               * &lt;/script&gt;              
               */
                monthHeaderFormat: "MMM",


                /**		
                * Specified the background for weekends in gantt
                * @alias ejGantt#scheduleHeaderSettings->hourHeaderFormat
                * @default "#F2F2F2"
                * @type {string}
                * @example 
                * &lt;div id="gantt"&gt;&lt;/div&gt; 
                * &lt;script&gt;			
                * 	$("#gantt").ejGantt({  
                *			scheduleHeaderSettings:{weekendBackground : #F2F2F2}
                });
                * &lt;/script&gt;               
                */
                hourHeaderFormat: 'HH',

                /**		
               * Specifies the schedule mode
               * @alias ejGantt#scheduleHeaderSettings->scheduleHeaderType
               * @default "week"
               * @type {enum}
               * @example 
               * &lt;div id="gantt"&gt;&lt;/div&gt; 
               * &lt;script&gt;				
               * 	$("#gantt").ejGantt({  
               *			scheduleHeaderSettings:{scheduleHeaderType : "week"}
               });
               * &lt;/script&gt;              
               */
                scheduleHeaderType: 'week',


                /**		
                * Specified the background for weekends in gantt
                * @alias ejGantt#scheduleHeaderSettings->weekendBackground
                * @default "#F2F2F2"
                * @type {string}
                * @example 
                * &lt;div id="gantt"&gt;&lt;/div&gt; 
                * &lt;script&gt;			
                * 	$("#gantt").ejGantt({  
                *			scheduleHeaderSettings:{weekendBackground : #F2F2F2}});
                * &lt;/script&gt;
                */
                weekendBackground: '#F2F2F2'


            },

            /**		
            * Specifies the background of the taskbar in gantt
            * @default ""
            * @type {string}
            * @example 
            * &lt;div id="gantt"&gt;&lt;/div&gt; 
            * &lt;script&gt;	
            * 	$("#gantt").ejGantt({  
            *			taskbarBackground : #F2F2F2});
            * &lt;/script&gt;
            * @memberof ejGantt
			   * @instance
            */
            taskbarBackground: "",

            /**		
            *Specifies the background of progressbar in gantt
            * @default ""
            * @type {string}
            * @example 
            * &lt;div id="gantt"&gt;&lt;/div&gt; 
            * &lt;script&gt;	
            * 	$("#gantt").ejGantt({  
            *			progressbarBackground : #F2F2F2});
            * &lt;/script&gt;
            * @memberof ejGantt
			   * @instance
            */
            progressbarBackground: "",

            /**		
            * Specifies the background of connector lines in Gantt
            * @default ""
            * @type {string}
            * @example 
            * &lt;div id="gantt"&gt;&lt;/div&gt; 
            * &lt;script&gt;				
            * 	$("#gantt").ejGantt({  
            *			connectorLineBackground : #F2F2F2});
            * &lt;/script&gt;
            * @memberof ejGantt
			   * @instance
            */
            connectorLineBackground: "",

            /**		
            * Specifies the background of parent taskbar in gantt
            * @default ""
            * @type {string}
            * @example 
            * &lt;div id="gantt"&gt;&lt;/div&gt; 
            * &lt;script&gt;			
            * 	$("#gantt").ejGantt({  
            *			parentTaskbarBackground : #F2F2F2});
            * &lt;/script&gt;
            * @memberof ejGantt
			   * @instance
            */
            parentTaskbarBackground: "",

            /**		
            *Specifies the background of parent progressbar in gantt
            * @default ""
            * @type {string}
            * @example 
            * &lt;div id="gantt"&gt;&lt;/div&gt; 
            * &lt;script&gt;			
            * 	$("#gantt").ejGantt({  
            *			parentProgressbarBackground : #F2F2F2});
            * &lt;/script&gt;
            * @memberof ejGantt
			   * @instance
            */

            parentProgressbarBackground: "",

            /**		
            * Specifies the width of the connector lines in gantt
            * @default "1"
            * @type {number}
            * @example 
            * &lt;div id="gantt"&gt;&lt;/div&gt; 
            * &lt;script&gt; 			
            * 	$("#gantt").ejGantt({  
            *			connectorlineWidth : 1 });
             * &lt;/script&gt;
             * @memberof ejGantt
			   * @instance
            */
            connectorlineWidth: 1,

            /**		
            *Specifies whether to display task name beside task bar.
            * @default "true"
            * @type {boolean}
            * @example 
            * &lt;div id="gantt"&gt;&lt;/div&gt; 
            * &lt;script&gt;			
            * 	$("#gantt").ejGantt({  
            *			showTaskNames : true});
            * &lt;/script&gt;
            * @memberof ejGantt
			   * @instance
            */
            showTaskNames: true,


            /**		
            *Specifies whether to show grid cell tooltip.
            * @default "true"
            * @type {boolean}
            * @example 
            * &lt;div id="gantt"&gt;&lt;/div&gt; 
            * &lt;script&gt; 			
            * 	$("#gantt").ejGantt({  
            *			showGridCellTooltip : true});
            * &lt;/script&gt;
            * @memberof ejGantt
			   * @instance
            */
            showGridCellTooltip: false,

            /**		
            *Specifies whether to show grid cell tooltip over expander cell alone.
            * @default "true"
            * @type {boolean}
            * @example 
            * &lt;div id="gantt"&gt;&lt;/div&gt; 
            * &lt;script&gt;			
            * 	$("#gantt").ejGantt({  
            *			showGridExpandCellTooltip : true});
            * &lt;/script&gt;
            * @memberof ejGantt
			   * @instance
            */
            showGridExpandCellTooltip: false,

            /**		
            * Specifies whether display task progress inside taskbar.
            * @default "true"
            * @type {boolean}
            * @example 
            * &lt;div id="gantt"&gt;&lt;/div&gt; 
            * &lt;script&gt;			
            * 	$("#gantt").ejGantt({  
            *			showProgressStatus : true});
            * &lt;/script&gt;
            * @memberof ejGantt
			   * @instance
            */
            showProgressStatus: true,

            /**		
            * Specifies whether to display resource names for a task beside taskbar.
            * @default "true"
            * @type {boolean}
            * @example 
            * &lt;div id="gantt"&gt;&lt;/div&gt; 
            * &lt;script&gt; 			
            * 	$("#gantt").ejGantt({  
            *			showResourceNames : true});
            * &lt;/script&gt;
            * @memberof ejGantt
			   * @instance
            */
            showResourceNames: true,

            /**		
            * Enables or disables tooltip while editing (dragging/resizing) the taskbar.
            * @default "true"
            * @type {boolean}
            * @example 
            * &lt;div id="gantt"&gt;&lt;/div&gt; 
            * &lt;script&gt; 			
            * 	$("#gantt").ejGantt({  
            *			enableTaskbarDragTooltip : true});
            * &lt;/script&gt;
            * @memberof ejGantt
			   * @instance
            */
            enableTaskbarDragTooltip: true,

            /**		
            * Enables or disables tooltip for taskbar.
            * @default true
            * @type {boolean}
            * @example 
            * &lt;div id="gantt"&gt;&lt;/div&gt; 
            * &lt;script&gt;			
            * 	$("#gantt").ejGantt({  enableTaskbarTooltip : true });	
            * &lt;/script&gt;
            * @memberof ejGantt
			   * @instance
            */
            enableTaskbarTooltip: true,

            /**		
            * Specifies the editSettings options in gantt. 
            * @type {object}
            * @memberof ejGantt
			   * @instance
            */
            editSettings: {

            /**		
            * specifies the option for enabling / disablig editing in gantt grid part
            * @alias ejGantt#editSettings->allowEditing
            * @default false
            * @type {boolean}
            * @example 
            * &lt;div id="gantt"&gt;&lt;/div&gt; 
            * &lt;script&gt;			
            * 	$("#gantt").ejGantt({  editSettings:{allowEditing : true} });	
            * &lt;/script&gt;
            * @memberof ejGantt
			   * @instance
            */
              allowEditing: false,

            /**		
            * enables/disables add gantt in toolbar
            * @alias ejGantt#editSettings->allowAdding
            * @default false
            * @type {boolean}
            * @example 
            * &lt;div id="gantt"&gt;&lt;/div&gt; 
            * &lt;script&gt; 			
            * 	$("#gantt").ejGantt({  editSettings:{allowAdding : true} });
            * &lt;/script&gt;
            * @memberof ejGantt
			   * @instance
            */
            allowAdding: false,

            /**		
            * enables/disables delete gantt in toolbar
            * @alias ejGantt#editSettings->allowDeleting
            * @default false
            * @type {boolean}
            * @example 
            * &lt;div id="gantt"&gt;&lt;/div&gt; 
            * &lt;script&gt;			
            * 	$("#gantt").ejGantt({  editSettings:{allowDeleting : true} });	
            * &lt;/script&gt;
            * @memberof ejGantt
			   * @instance
            */
             allowDeleting: false,


            /**		
            * specifies the editmode in gantt , "normal" is for dialog editing ,"cellEditing" is for cell type editing
            * @alias ejGantt#editSettings->editMode
            * @default normal
            * @type {string}
            * @example 
            * &lt;div id="gantt"&gt;&lt;/div&gt; 
            * &lt;script&gt;			
            * 	$("#gantt").ejGantt({  editSettings:{editMode : true} });
            * &lt;/script&gt;
            * @memberof ejGantt
			   * @instance
            */
                editMode: "normal",
            },

            ganttRecords: [],
            parentRecords: [],

            /**		
            * Enables/Disables Keyboard navigation in gantt
            * @default true
            * @type {boolean}
            * @example 
            * &lt;div id="gantt"&gt;&lt;/div&gt; 
            * &lt;script&gt; 			
            * 	$("#gantt").ejGantt({ allowKeyboardNavigation : true});			
            * &lt;/script&gt;
            * @memberof ejGantt
			   * @instance
            */
            allowKeyboardNavigation: true,

            /**		
             * Specify the CSS class to gantt to achieve custom theme.
             * @default ""
             * @type {string}
             * @example 
             * &lt;div id="gantt"&gt;&lt;/div&gt; 
             * &lt;script&gt;		
             * 	$("#gantt").ejGantt({  cssClass : "gradient-lime" });
             * &lt;/script&gt;
             * @memberof ejGantt
			   * @instance
            */
            cssClass: "",

            /**		
            * Specify the locale for gantt
            * @default "en-US"
            * @type {string}
            * @example 
            * &lt;div id="gantt"&gt;&lt;/div&gt; 
            * &lt;script&gt;		
            * 	$("#gantt").ejGantt({  locale : "it-IT" });
            * &lt;/script&gt;
            * @memberof ejGantt
			   * @instance
            */
            locale: "en-US",

            /**		
            * Specifies enabling / diabling multiple soting for gantt columns
            * @default false
            * @type {boolean}
            * @example 
            * &lt;div id="gantt"&gt;&lt;/div&gt; 
            * &lt;script&gt;			
            * 	$("#gantt").ejGantt({ allowMultiSorting : true});			
            * &lt;/script&gt;
            * @memberof ejGantt
			   * @instance
            */
            allowMultiSorting: false,

            /**		
            *  Specifes the sorting options for gantt.
            * @type {object}
            * @memberof ejGantt
			   * @instance
            */
            sortSettings: {
           /**		
           * Specifes the sorted columns columns for gantt
           * @alias ejGantt#sortSettings->sortedColumns
           * @default []
           * @type {array}
           * @example 
           * &lt;div id="gantt"&gt;&lt;/div&gt; 
           * &lt;script&gt;		
           * 	$("#gantt").ejGantt({ sortSettings{sortedColumns : []}});	           
           * &lt;/script&gt;
           * @memberof ejGantt
			   * @instance
            */
                sortedColumns: []
            },

           
            /**		
            * Enables/Disables enableAltRow row effect in gantt
            * @default true
            * @type {boolean}
            * @example 
            * &lt;div id="gantt"&gt;&lt;/div&gt; 
            * &lt;script&gt;			
            * 	$("#gantt").ejGantt({ enableAltRow : true});			
            * &lt;/script&gt;
            * @memberof ejGantt
			   * @instance
            */
            enableAltRow: true,

            /**		
            * Enables/Disables virtualization for rendering gantt items.
            * @default false
            * @type {boolean}
            * @example 
            * &lt;div id="gantt"&gt;&lt;/div&gt; 
            * &lt;script&gt;			
            * 	$("#gantt").ejGantt({ enableVirtualization : true});			
            * &lt;/script&gt;
            * @memberof ejGantt
			   * @instance
            */
            enableVirtualization: false,

            /**		
             * Specified the height of the progressbar in taskbar
             * @default "100"
             * @type {number}
             * @example 
             * &lt;div id="gantt"&gt;&lt;/div&gt; 
             * &lt;script&gt;		
             * 	$("#gantt").ejGantt({  
             *			progressbarHeight : 100 });
             * &lt;/script&gt;
             * @memberof ejGantt
			   * @instance
            */
            progressbarHeight: 100,

            /**		
             * Specifies the template for tooltip on mouseaction on taskbars
             * @default null
             * @type {string}
             * @example 
             * &lt;div id="gantt"&gt;&lt;/div&gt; 
             * &lt;script&gt;			
             * 	$("#gantt").ejGantt(
             *  {
             *     taskbarTooltipTemplate: ""
             *  });		
             * &lt;/script&gt;
             * @memberof ejGantt
			   * @instance
            */
            taskbarTooltipTemplate: null,

            /**		
             * Specifies the template for tooltip on resizing progressbar
             * @default null
             * @type {string}
             * @example 
             * &lt;div id="gantt"&gt;&lt;/div&gt; 
             * &lt;script&gt;		
             * 	$("#gantt").ejGantt(
             *  {
             *     progressbarTooltipTemplate: ""
             *  });		
             * &lt;/script&gt;
             * @memberof ejGantt
			   * @instance
            */
            progressbarTooltipTemplate: null,

            /**		
             * Specifies the template for tooltip on mouseaction on taskbars
             * @default null
             * @type {string}
             * @example 
             * &lt;div id="gantt"&gt;&lt;/div&gt; 
             * &lt;script&gt;		
             * 	$("#gantt").ejGantt(
             *  {
             *     taskbarTooltipTemplateId: ""
             *  });		
             * &lt;/script&gt;
             * @memberof ejGantt
			   * @instance
            */
            taskbarTooltipTemplateId: null,
            /**		
            * Specifies the dateFormat for gantt , given format is displayed in tooltip , grid .
            * @default ""
            * @type {string}
            * @example 
            * &lt;div id="gantt"&gt;&lt;/div&gt; 
            * &lt;script&gt;		
            * 	$("#gantt").ejGantt(
            *  {
            *     dateFormat: "dd/MM/yyyy"
            *  });		
            * &lt;/script&gt;
            * @memberof ejGantt
			   * @instance
            */
            dateFormat: "",

            /**		
            * Specifies the mapping property name for resource ID in resource Collection in gantt
            * @default ""
            * @type {string}
            * @example 
            * &lt;div id="gantt"&gt;&lt;/div&gt; 
            * &lt;script&gt;	
            * 	$("#gantt").ejGantt(
            *  {
            *     resourceIdMapping: "peopleID"
            *  });		
            * &lt;/script&gt;
            * @memberof ejGantt
			   * @instance
            */
            resourceIdMapping: "",

            /**		
            * Specifies the mapping property path for resource name of a task in gantt
            * @default ""
            * @type {string}
            * @example 
            * &lt;div id="gantt"&gt;&lt;/div&gt; 
            * &lt;script&gt;		
            * 	$("#gantt").ejGantt(
            *  {
            *     resourceNameMapping: "EmployeeName"
            *  });		
            * &lt;/script&gt;
            * @memberof ejGantt
			   * @instance
            */
            resourceNameMapping: "",

             /**		
             * Specifies the template ID for customized tooltip for progressbar editing in gantt
             * @default null
             * @type {string}
             * @example 
             * &lt;div id="gantt"&gt;&lt;/div&gt; 
             * &lt;script&gt;	
             * 	$("#gantt").ejGantt(
             *  {
             *     progressbarTooltipTemplateId: "tooltiptemplateID"
             *  });		
             * &lt;/script&gt;
             * @memberof ejGantt
			   * @instance
             */
            progressbarTooltipTemplateId: null,

            /**		
            * Specifies the template Id for customized tooltip for taskbar editing in gantt
            * @default null
            * @type {string}
            * @example 
            * &lt;div id="gantt"&gt;&lt;/div&gt; 
            * &lt;script&gt;		
            * 	$("#gantt").ejGantt(
            *  {
            *     taskbarEditingTooltipTemplateId: "TooltipTemplateId"
            *  });		
            * &lt;/script&gt;
            * @memberof ejGantt
			   * @instance
            */
            taskbarEditingTooltipTemplateId: null,

            /**		
            * Specifies the template Id for customized tooltip for taskbar editing in gantt
            * @default null
            * @type {string}
            * @example 
            * &lt;div id="gantt"&gt;&lt;/div&gt; 
            * &lt;script&gt;			
            * 	$("#gantt").ejGantt(
            *  {
            *     taskbarEditingTooltipTemplate: "tooltiptemplate"
            *  });		
            * &lt;/script&gt;
            * @memberof ejGantt
			   * @instance
            */
            taskbarEditingTooltipTemplate: null,

          /**		
          * Specifies the selected row Index in gantt , the row with given index is highlighted
          * @default  "-1"
          * @type {number}
          * @example 
          * &lt;div id="gantt"&gt;&lt;/div&gt; 
          * &lt;script&gt;			
          * 	$("#gantt").ejGantt(
          *  {
          *     selectedRowIndex:2
          *  });		
          * &lt;/script&gt;
          * @memberof ejGantt
			   * @instance
          */
          selectedRowIndex: -1,


         /**		
         * Enables or Disables gantt chart editing in gantt
         * @default  "true"
         * @type {boolean}
         * @example 
         * &lt;div id="gantt"&gt;&lt;/div&gt; 
         * &lt;script&gt;		
         * 	$("#gantt").ejGantt(
         *  {
         *     allowGanttChartEditing:true
         *  });		
         * &lt;/script&gt;
         * @memberof ejGantt
			   * @instance
         */
            allowGanttChartEditing: true,

         /**		
         * Specifies the size option of gantt control.
         * @type {object}
         * @memberof ejGantt
			   * @instance
         */
            sizeSettings: {
                /**		
                * Specifies the height of gantt control
                * @alias ejGantt#sizeSettings->height
                * @default  "450px"
                * @type {string}
                * @example 
                * &lt;div id="gantt"&gt;&lt;/div&gt; 
                * &lt;script&gt;			
                * 	$("#gantt").ejGantt(
                *  {
                *     sizeSettings:{height: "700px"}
                *  });		
                * &lt;/script&gt;
               
                */
                height: "450px",

                /**		
                * Specifies the width of gantt control
                * @alias ejGantt#sizeSettings->width
                * @default  "1000px"
                * @type {string}
                *  @example 
                * &lt;div id="gantt"&gt;&lt;/div&gt; 
                * &lt;script&gt;		
                * 	$("#gantt").ejGantt(
                *  {
                *     sizeSettings:{width: "700px"}
                *  });		
                * &lt;/script&gt;
              
                */
                width: "1000px"
            },

         
           selectedItem: null,

          /**		
          * Specifies the weekendBackground color in gantt
          * @default  "#F2F2F2"
          * @type {string}
          * @example 
          * &lt;div id="gantt"&gt;&lt;/div&gt; 
          * &lt;script&gt;		
          * 	$("#gantt").ejGantt(
          *  {
          *     weekendBackground: "blue"
          *  });		
          * &lt;/script&gt;
          * @memberof ejGantt
			   * @instance
          */
            weekendBackground: '#F2F2F2',

            /**		
           * Specifies the baseline background color in gantt
           * @default  "#fba41c"
           * @type {string}
           * @example 
           * &lt;div id="gantt"&gt;&lt;/div&gt; 
            * &lt;script&gt;	
           * 	$("#gantt").ejGantt(
           *  {
           *     baselineColor: "blue"
           *  });		
           * &lt;/script&gt;
           * @memberof ejGantt
			   * @instance
           */
            baselineColor: '#fba41c',

            /**		
           * Specifies the tree expander column in gantt
           * @default  0
           * @type {number}
           * @example 
           * &lt;div id="gantt"&gt;&lt;/div&gt; 
           * &lt;script&gt;		
           * 	$("#gantt").ejGantt(
           *  {
           *     treeColumnIndex: 1
           *  });		
           * &lt;/script&gt;
           * @memberof ejGantt
		   * @instance
           */
            treeColumnIndex: 0,
            
            /**		
           * Specifies the fields to be included in the edit dialog in gantt
           * @default  []
           * @type {Array}
           * @example 
           * &lt;div id="gantt"&gt;&lt;/div&gt; 
           * &lt;script&gt;			
           * 	$("#gantt").ejGantt(
           *  {
           *     editDialogFields: [{ field: "taskId", editType: "stringedit" }]
           *  });		
           * &lt;/script&gt;
           * @memberof ejGantt
			   * @instance
           */
            editDialogFields: [],
            

             /**		
           * Specifies the fields to be included in the add dialog in gantt
           * @default  []
           * @type {Array}
           * @example 
           * &lt;div id="gantt"&gt;&lt;/div&gt; 
           * &lt;script&gt;			
           * 	$("#gantt").ejGantt(
           *  {
           *     addDialogFields: [{ field: "taskId", editType: "stringedit" }]
           *  });		
           * &lt;/script&gt;
           * @memberof ejGantt
			   * @instance
           */
            addDialogFields: [],
            /**		
            * Enables or Disables rendering baselines in Gantt , when enabled baseline is rendered in gantt
            * @default  "false"
            * @type {boolean}
            * @example 
            * &lt;div id="gantt"&gt;&lt;/div&gt; 
            * &lt;script&gt;		
            * 	$("#gantt").ejGantt(
            *  {
            *     renderBaseline: false
            *  });		
            * &lt;/script&gt;
            * @memberof ejGantt
			   * @instance
            */
            renderBaseline: false,

            /**		
            * Enables or disables the contextmenu for gantt , when enabled contextmenu appears on right clicking gantt
            * @default  "false"
            * @type {boolean}
            * @example 
            * &lt;div id="gantt"&gt;&lt;/div&gt; 
            * &lt;script&gt;			
            * 	$("#gantt").ejGantt(
            *  {
            *     enableContextMenu: false
            *  });		
            * &lt;/script&gt;
            * @memberof ejGantt
			   * @instance
            */
            enableContextMenu: false,

           

            /**		
            * Enables or disables the option for dynamically updating the Gantt size on window resizing
            * @default  "false"
            * @type {boolean}
            * @example 
            * &lt;div id="gantt"&gt;&lt;/div&gt; 
            * &lt;script&gt;			
            * 	$("#gantt").ejGantt(
            *  {
            *     enableResize: false
            *  });		
            * &lt;/script&gt;
            * @memberof ejGantt
		    * @instance
            */
            enableResize:false,

            /**		
            * Enables or disables the collapse all tasks for gantt , when enabled all sub tasks will disappears in gantt
            * @default  "false"
            * @type {boolean}
            * @example 
            * &lt;div id="gantt"&gt;&lt;/div&gt; 
            * &lt;script&gt;			
            * 	$("#gantt").ejGantt(
            *  {
            *     enableCollapseAll: false
            *  });		
            * &lt;/script&gt;
            * @memberof ejGantt
			   * @instance
            */
            enableCollapseAll: false,
            /**    
			* Triggered before the row is going to be selected.
			* @event
			* @name ejGantt#rowSelecting 
			* @param {Object} argument Arguments when rowSelecting event is triggered.  
			* @param {boolean} argument.cancel Returns the cancel option value.
			* @param {object} argument.rowElement Returns the selecting row element.
			* @param {string} argument.recordIndex Returns the index of selecting row record.
			* @param {object} argument.data Returns the data selecting record.
			* @example 
			* &lt;div id="gantt"&gt;&lt;/div&gt; 
            * &lt;script&gt;
			* $("#gantt").ejGantt({
			*    rowSelecting: function (args) {}
			* });
			* &lt;/script&gt;
            * @memberof ejGantt
			   * @instance
            */
            rowSelecting: null,

            /**    
			* Triggered after the row is selected.
			* @event
			* @name ejGantt#rowSelected 
			* @param {Object} argument Arguments when rowSelected event is triggered.  
			* @param {boolean} argument.cancel Returns the cancel option value.
			* @param {object} argument.rowElement Returns the selecting row element.
			* @param {string} argument.recordIndex Returns the index of selecting row record.
			* @param {object} argument.data Returns the data of selected record.
			* @example 
			* &lt;div id="gantt"&gt;&lt;/div&gt; 
            * &lt;script&gt;
			* $("#gantt").ejGantt({
			*    rowSelected: function (args) {}
			* });
			* &lt;/script&gt;
            * @memberof ejGantt
			   * @instance
            */
            rowSelected: null,

            /**    
            * Triggered while rendering each cell in the grid tree
            * @event
            * @name ejGantt#queryCellInfo 
            * @param {Object} argument Arguments when queryCellInfo event is triggered.  
            * @param {boolean} argument.cancel Returns the cancel option value.
            * @param {object} argument.rowElement Returns the selecting row element.
            * @param {string} argument.cellValue Returns the value of cell.
            * @param {object} argument.data Returns the data of current cell record.
            * @param {object} argument.column Returns the column of cell belongs.
            * @example 
            * &lt;div id="gantt"&gt;&lt;/div&gt; 
            * &lt;script&gt;
            * $("#gantt").ejGantt({
            *    queryCellInfo: function (args) {}
            * });
            * &lt;/script&gt;
            * @memberof ejGantt
			   * @instance
            */
            queryCellInfo: null,

            /**    
            * Triggered while rendering each taskbar in the gantt chart
            * @event
            * @name ejGantt#queryTaskbarInfo 
            * @param {Object} argument Arguments when queryTaskbarInfo event is triggered.  
            * @param {boolean} argument.cancel Returns the cancel option value.
            * @param {object} argument.TaskbarBackground Returns the taskbar background of current item.
            * @param {string} argument.ProgressbarBackground Returns the progressbar background of current item.
            * @param {object} argument.data Returns the data of the record.
            * @example 
            * &lt;div id="gantt"&gt;&lt;/div&gt; 
            * &lt;script&gt;
            * $("#gantt").ejGantt({
            *    queryTaskbarInfo: function (args) {}
            * });
            * &lt;/script&gt;
            * @memberof ejGantt
			   * @instance
            */
            queryTaskbarInfo: null,

            /**    
            * Triggered while enter the edit mode in the grid tree cell
            * @event
            * @name ejGantt#beginEdit 
            * @param {Object} argument Arguments when beginEdit event is triggered.  
            * @param {boolean} argument.cancel Returns the cancel option value.
            * @param {object} argument.rowElement Returns the row element of editing cell.
            * @param {object} argument.cellElement Returns the Element of editing cell.
            * @param {object} argument.data Returns the data of current cell record.
            * @param {number} argument.columnIndex Returns the column Index of cell belongs.
            * @example 
            * &lt;div id="gantt"&gt;&lt;/div&gt; 
            * &lt;script&gt;
            * $("#gantt").ejGantt({
            *    beginEdit: function (args) {}
            * });
            * &lt;/script&gt;
            * @memberof ejGantt
			   * @instance
            */
            beginEdit: null,

            /**    
            * Triggered after save the modified cellValue in treegrid
            * @event
            * @name ejGantt#endEdit 
            * @param {Object} argument Arguments when endEdit event is triggered.  
            * @param {boolean} argument.cancel Returns the cancel option value.
            * @param {object} argument.rowElement Returns the row element of editing cell.
            * @param {object} argument.cellElement Returns the Element of editing cell.
            * @param {object} argument.data Returns the data of edited cell record.
            * @param {string} argument.columnName Returns the column name of edited cell belongs.
            * @param {object} argument.columnObject Returns the column object of edited cell belongs.
            * @example 
            * &lt;div id="gantt"&gt;&lt;/div&gt; 
            * &lt;script&gt;
            * $("#gantt").ejGantt({
            *    endEdit: function (args) {}
            * });
            * &lt;/script&gt;
            * @memberof ejGantt
			   * @instance
            */
            endEdit: null,


            /**    
            * Triggered while rendering each row
            * @event
            * @name ejGantt#rowDataBound 
            * @param {Object} argument Arguments when rowDataBound event is triggered.  
            * @param {boolean} argument.cancel Returns the cancel option value.
            * @param {object} argument.rowElement Returns the row element of editing cell.
            * @param {object} argument.data Returns the data of edited cell record..
            * @example 
            * &lt;div id="gantt"&gt;&lt;/div&gt; 
            * &lt;script&gt;
            * $("#gantt").ejGantt({
            *    rowDataBound: function (args) {}
            * });
            * &lt;/script&gt;
            * @memberof ejGantt
			   * @instance
            */
            rowDataBound: null,

            /**    
            * Triggered  while expanding the gantt record
            * @event
            * @name ejGantt#expanding 
            * @param {Object} argument Arguments when expanding event is triggered.  
            * @param {boolean} argument.cancel Returns the cancel option value.
            * @param {object} argument.recordIndex Returns the row index of record.
            * @param {object} argument.data Returns the data of edited cell record..
            * @example 
            * &lt;div id="gantt"&gt;&lt;/div&gt; 
            * &lt;script&gt;
            * $("#gantt").ejGantt({
            *    expanding: function (args) {}
            * });
            * &lt;/script&gt;
            * @memberof ejGantt
			   * @instance
            */
            expanding: null,


            /**    
            * Triggered  after expand the record
            * @event
            * @name ejGantt#expanded 
            * @param {Object} argument Arguments when expanded event is triggered.  
            * @param {boolean} argument.cancel Returns the cancel option value.
            * @param {object} argument.recordIndex Returns the row index of record.
            * @param {object} argument.data Returns the data of edited cell record..
            * @example 
            * &lt;div id="gantt"&gt;&lt;/div&gt; 
            * &lt;script&gt;
            * $("#gantt").ejGantt({
            *    expanded: function (args) {}
            * });
            * &lt;/script&gt;
            * @memberof ejGantt
			   * @instance
            */
            expanded: null,

            /**    
            * Triggered while collapsing the gantt record
            * @event
            * @name ejGantt#collapsing 
            * @param {Object} argument Arguments when collapsing event is triggered.  
            * @param {boolean} argument.cancel Returns the cancel option value.
            * @param {object} argument.recordIndex Returns the row index of collapsing record.
            * @param {object} argument.data Returns the data of edited cell record..
            * @example 
            * &lt;div id="gantt"&gt;&lt;/div&gt; 
            * &lt;script&gt;
            * $("#gantt").ejGantt({
            *    collapsing: function (args) {}
            * });
            * &lt;/script&gt;
            * @memberof ejGantt
			   * @instance
            */
            collapsing: null,

            /**    
            * Triggered after collapsing the gantt record
            * @event
            * @name ejGantt#collapsed 
            * @param {Object} argument Arguments when collapsed event is triggered.  
            * @param {boolean} argument.cancel Returns the cancel option value.
            * @param {object} argument.recordIndex Returns the row index of collapsed record.
            * @param {object} argument.data Returns the data of edited cell record..
            * @example 
            * &lt;div id="gantt"&gt;&lt;/div&gt; 
            * &lt;script&gt;
            * $("#gantt").ejGantt({
            *    collapsed: function (args) {}
            * });
            * &lt;/script&gt;
            * @memberof ejGantt
			   * @instance
            */
            collapsed: null,


            /**    
            * Triggered after collapsing the gantt record
            * @event
            * @name ejGantt#actionBegin 
            * @param {Object} argument Event parameters when gantt is initialized:          
			* @param {boolean}  argument.cancel Returns the cancel option value.
			* @param {object}  argument.model Returns the grid model.
			* @param {string}  argument.type Returns the name of the event.

            * @param {Object} argument Event parameters while perform sorting in grid tree action starts:
            * @param {boolean}  argument.cancel Returns the cancel option value.
            * @param {string}  argument.columnName Returns the current grouped column field name.
			* @param {object}  argument.model Returns the gantt model.
            * @param {string}  argument.requestType Returns request type.
            * @param {string}  argument.type Returns the name of the event. 
            * @param {string}  argument.columnSortDirection Returns the direction of sorting ascending or descending
            
            * @param {Object} argument Event parameters while searching action starts:
            * @param {boolean}  argument.cancel Returns the cancel option value.
            * @param {string}  argument.keyValue Returns the value of searching element.
			* @param {object}  argument.model Returns the gantt model.
            * @param {string}  argument.requestType Returns request type.
            * @param {string}  argument.type Returns the name of the event. 
             
            * @param {Object} argument Event parameterswhile performing the delete operation starts:
            * @param {boolean}  argument.cancel Returns the cancel option value.
            * @param {string}  argument.data Returns the data od deleting element.
			* @param {object}  argument.model Returns the gantt model.
            * @param {string}  argument.requestType Returns request type.
            * @param {string}  argument.type Returns the name of the event. 

            * @example 
            * &lt;div id="gantt"&gt;&lt;/div&gt; 
            * &lt;script&gt;
            * $("#gantt").ejGantt({
            *    actionBegin: function (args) {}
            * });
            * &lt;/script&gt;
            * @memberof ejGantt
			   * @instance
            */
            actionBegin: null,

            /**    
            * Triggered after collapsing the gantt record
            * @event
            * @name ejGantt#actionComplete 
            * @param {Object} argument Event parameters when gantt is initialized:          
			* @param {boolean}  argument.cancel Returns the cancel option value.
			* @param {object}  argument.model Returns the grid model.
			* @param {string}  argument.type Returns the name of the event.

            * @param {Object} argument Event parameters after perform the sorting in grid tree is completed:
            * @param {boolean}  argument.cancel Returns the cancel option value.
            * @param {string}  argument.columnName Returns the current grouped column field name.
			* @param {object}  argument.model Returns the gantt model.
            * @param {string}  argument.requestType Returns request type.
            * @param {string}  argument.type Returns the name of the event. 
            * @param {string}  argument.columnSortDirection Returns the direction of sorting ascending or descending
            
            * @param {Object} argument Event parameters after searching completed:
            * @param {boolean}  argument.cancel Returns the cancel option value.
            * @param {string}  argument.keyValue Returns the value of searched element.
			* @param {object}  argument.model Returns the gantt model.
            * @param {string}  argument.requestType Returns request type.
            * @param {string}  argument.type Returns the name of the event. 
             
            * @param {Object} argument Event parameterswhile performing after completing the delete operation is completed:
            * @param {boolean}  argument.cancel Returns the cancel option value.
            * @param {string}  argument.data Returns the data od deleted element.
			* @param {object}  argument.model Returns the gantt model.
            * @param {string}  argument.requestType Returns request type.
            * @param {string}  argument.type Returns the name of the event. 

            * @example 
            * &lt;div id="gantt"&gt;&lt;/div&gt; 
            * &lt;script&gt;
            * $("#gantt").ejGantt({
            *    actionComplete: function (args) {}
            * });
            * &lt;/script&gt;
            * @memberof ejGantt
			   * @instance
            */
            actionComplete: null,

            /**    
            * Triggered while editing the gantt chart (dragging, resizing the taskbar )
            * @event
            * @name ejGantt#taskbarEditing 
            * @param {Object} argument Arguments when taskbarEditing event is triggered.  
            * @param {boolean} argument.cancel Returns the cancel option value.
			* @param {object}  argument.model Returns the gantt model.
			* @param {object}  argument.rowData Returns the row object being edited.
			* @param {object}  argument.editingFields Returns the field values of record being edited.
            * @param {string}  argument.type Returns the name of the event. 
            * @example 
            * &lt;div id="gantt"&gt;&lt;/div&gt; 
            * &lt;script&gt;
            * $("#gantt").ejGantt({
            *    taskbarEditing: function (args) {}
            * });
            * &lt;/script&gt;
            * @memberof ejGantt
			   * @instance
            */
            taskbarEditing: null,

            /**    
            * Triggered while after completing the editing operation in taskbar
            * @event
            * @name ejGantt#taskbarEdited 
            * @param {Object} argument Arguments when taskbarEdited event is triggered.  
            * @param {boolean} argument.cancel Returns the cancel option value.
            * @param {object} argument.data Returns the data of edited record.
            * @param {object} argument.previousData Returns the previous data value of edited record.  
            * @param {boolean} argument.dragging Returns 'true' if taskbar is dragged.  
            * @param {boolean} argument.leftResizing Returns 'true' if taskbar is left resized.  
            * @param {boolean} argument.rightResizing Returns 'true' if taskbar is right resized.  
            * @param {boolean} argument.progressResizing Returns 'true' if taskbar is progress resized.  
			* @param {object}  argument.editingFields Returns the field values of record being edited.
			* @param {object}  argument.model Returns the gantt model.
            * @example 
            * &lt;div id="gantt"&gt;&lt;/div&gt; 
            * &lt;script&gt;
            * $("#gantt").ejGantt({
            *    taskbarEdited: function (args) {}
            * });
            * &lt;/script&gt;
            * @memberof ejGantt
			   * @instance
            */
            taskbarEdited: null,

            /**    
            * Triggered while gantt is loaded
            * @event
            * @name ejGantt#load 
            * @param {Object} argument Arguments when load event is triggered.             
            * @param {boolean} argument.cancel Returns the cancel option value.
			* @param {object}  argument.model Returns the gantt model
            * @param {string}  argument.type Returns the name of the event. 
            * @example 
            * &lt;div id="gantt"&gt;&lt;/div&gt; 
            * &lt;script&gt;
            * $("#gantt").ejGantt({
            *    load: function (args) {}
            * });
            * &lt;/script&gt;
            * @memberof ejGantt
			   * @instance
            */
            load: null,

            /**    
            * Triggered while Context Menu is rendered in Gantt control
            * @event
            * @name ejGantt#contextMenuOpen 
            * @param {Object} argument Arguments when context menu is rendered.  
            * @param {boolean} argument.cancel Returns the cancel option value.
            * @param {boolean} argument.contextMenuItems Returns the default context menu items to which we add custom items.
			* @param {object}  argument.model Returns the gantt model.
            * @param {string}  argument.requestType Returns request type.
            * @param {string}  argument.type Returns the name of the event. 
            * @example 
            * &lt;div id="gantt"&gt;&lt;/div&gt; 
            * &lt;script&gt;
            * $("#gantt").ejGantt({
            *    contextMenuOpen: function (args) {}
            * });
            * &lt;/script&gt;
            * @memberof ejGantt
			   * @instance
            */
            contextMenuOpen: null,

        },

        dataTypes: {
            allowSorting: "boolean",
            allowColumnResize: "boolean",
            allowSelection: "boolean",
           // dataSource: "data", //inline or remote data
            taskIdMapping: "string",
            taskNameMapping: "string",
            startDateMapping: "string",
            endDateMapping: "string",
            baselineStartDateMapping: "string",
            baselineEndDateMapping: "string",
            childMapping: "string",
            durationMapping: "string",
            milestoneMapping: "string",
            progressMapping: "string",
            predecessorMapping: "string",
            resourceInfoMapping: "string",
            resources: "array",
            holidays: "array",
            highlightWeekends: "boolean",
            enableProgressBarResizing: "boolean",
            rowHeight: "number",
            includeWeekend: "boolean",
            toolbarSettings: {
                toolbarItems: "array"
            },
            stripLines: "array",
            scheduleHeaderSettings: "data",
            taskbarBackground: "string",
            progressbarBackground: "string",
            connectorLineBackground: "string",
            parentTaskbarBackground: "string",
            parentProgressbarBackground: "string",
            connectorlineWidth: "number",
            showTaskNames: "boolean",
            showProgressStatus: "boolean",
            showResourceNames: "boolean",
            enableTaskbarDragTooltip: "boolean",
            enableTaskbarTooltip: "boolean",
            editSettings: "data",
            ganttRecords: "array",
            parentRecords: "array",
            allowKeyboardNavigation: "boolean",
            cssClass: "string",
            locale: "string",
            allowMultiSorting: "boolean",
            showGridExpandCellTooltip: "boolean",
            sortSettings: {
                sortedColumns: "array"
            },
            enableAltRow: "boolean",
            enableVirtualization: "boolean",
            progressbarHeight: "number",
            dateFormat: "string",
            resourceIdMapping: "string",
            resourceNameMapping: "string",
          //  selectedRowIndex: "number",
            allowGanttChartEditing: "boolean",
            size: {
                height: "string",
                width: "string"
            },
            selectedItem: "data",
            weekendBackground: "string",
            baselineColor: "string",
            renderBaseline: "boolean",
            showGridCellTooltip: "boolean",
            treeColumnIndex: "number",
            editDialogFields: "array",
            enableResize:"boolean"
        },

        observables: ["selectedItem", "selectedRowIndex","dataSource"],
        selectedItem: ej.util.valueFunction("selectedItem"),
        selectedRowIndex: ej.util.valueFunction("selectedRowIndex"),
        dataSource: ej.util.valueFunction("dataSource"),

        getColumns: function() {
            return this._columns;
        },

        setColumns: function(value) {
            this._columns = value;
        },

        _init: function () {
            var proxy = this,
                model = proxy.model, startDate, endDate;
            proxy._initPrivateProperties();
            proxy._setCultureInfo();
            proxy.element.addClass("e-gantt-core");

            if (model.isdesignMode) {
                model.taskIdMapping = "Id";
                model.taskNameMapping = "Name";
                model.startDateMapping = "StartDate";
                model.durationMapping = "Duration";
                model.progressMapping = "Progress";
                model.scheduleStartDate = "02/23/2014";
                model.scheduleEndDate = "03/10/2014";
                this.dataSource ([{ Id: 1, Name: "Task 1", StartDate: "02/23/2014", Duration: 5, Progress: 40 },
                { Id: 2, Name: "Task 2", StartDate: "02/24/2014",Duration:5,Progress:40 },
                { Id: 3, Name: "Task 3", StartDate: "02/25/2014", Duration: 5, Progress: 40 }]); 
            }
            
            if (model.scheduleStartDate != null && model.scheduleEndDate != null) {
                startDate = proxy._getDateFromFormat(model.scheduleStartDate);
                endDate = proxy._getDateFromFormat(model.scheduleEndDate);
            }
            else {
                startDate = proxy._getDateFromFormat(new Date());
                endDate = proxy._getDateFromFormat(new Date(startDate));
                endDate.setDate(endDate.getDate() + 20);
            }

            if (model.scheduleHeaderSettings.scheduleHeaderType == ej.Gantt.ScheduleHeaderType.Week)
                proxy._calculateWeekSplit(startDate, endDate);
            if (model.scheduleHeaderSettings.scheduleHeaderType == ej.Gantt.ScheduleHeaderType.Year)
                proxy._calculateYearSplit(startDate, endDate);//method for year split

            if (model.scheduleHeaderSettings.scheduleHeaderType == ej.Gantt.ScheduleHeaderType.Month)
                proxy._calculateMonthSplit(startDate, endDate);//method for Month split

            if (model.scheduleHeaderSettings.scheduleHeaderType == ej.Gantt.ScheduleHeaderType.Day)
                proxy._calculateDaySplit(startDate, endDate);

            if (model.durationUnit == "hours") {
                model.durationUnit = "hour";
            }
            if (this.dataSource() !== null) {
                this._checkDataBinding();
            }
            else {
                proxy.setColumns(proxy.createTreeGridColumns());
                proxy._initialize();
            }
            //
            //Call collapse All
            if(model.enableCollapseAll)
                proxy._collapseAll();

        },
        
        _checkDataBinding: function () {
            var proxy = this, model = proxy.model;

            if (this.dataSource() instanceof ej.DataManager) {
                this._initDataSource();
            }
            else if (this.dataSource().length > 0) {
                if ((model.taskIdMapping.length > 0) && (model.parentTaskIdMapping.length > 0)) {

                    //cloning the datasource
                    var dataArray = this.dataSource().slice(0);
                    var data = [];
                    $.each(dataArray, function (indx, value) {
                        data.push(jQuery.extend(true, {}, value));
                    });
                    proxy._reconstructDatasource(data);
                    proxy._createGanttRecords(proxy.secondaryDatasource);
                }
                else {
                    proxy._createGanttRecords(this.dataSource());
                }
                proxy._initialize();
            }
        },

        _initDataSource: function () {
            var proxy = this, model = proxy.model, retriveData;
            
            var query = this._columnToSelect();
            var proxy = this;
            var queryPromise = this.dataSource().executeQuery(query);
            queryPromise.done(ej.proxy(function (e) {
                retriveData = e.result;
                proxy._reconstructDatasource(e.result);
                proxy._createGanttRecords(proxy.secondaryDatasource);
                proxy._initialize();
            }));
        },

        _columnToSelect: function (mapper) {
            var proxy = this,
                model = proxy.model,
                column = [], queryManager = ej.Query();

                queryManager = model.query;
            return queryManager;
        },

        _reconstructDatasource: function (datasource) {
            var proxy = this, model = proxy.model,
                childItems, data,tempRecord=[];
            
            for (var i = 0; i < datasource.length; i++) {
                var data = {};
                data = ej.DataManager(datasource).executeLocal(ej.Query().where(model.parentTaskIdMapping,
                    ej.FilterOperators.equal,
                    datasource[i][model.taskIdMapping]));
               

                if ((ej.isNullOrUndefined(datasource[i][model.parentTaskIdMapping]) || datasource[i][model.parentTaskIdMapping] == 0)
                    && data.length > 0) {
                    datasource[i][model.childMapping] = data;
                    proxy.secondaryDatasource.push(datasource[i]);
                }
                else if (ej.isNullOrUndefined(datasource[i][model.parentTaskIdMapping]) && !(data && data.length > 0)) {
                    proxy.secondaryDatasource.push(datasource[i]);
                }

                else if (data && data.length > 0 && !(ej.isNullOrUndefined(data[0][model.parentTaskIdMapping]))) {
                    proxy._appendChildDataItems(proxy.secondaryDatasource, data);
                }

                if (data && data.length > 0) {

                    proxy._datasourceChildItems = proxy._reconstructDatasource(data);
                    if (ej.isNullOrUndefined(datasource[i][model.childMapping])) {
                        datasource[i][model.childMapping] = proxy._datasourceChildItems;
                        proxy._getParentIndex(datasource);
                    }
                    proxy._datasourceChildItems = [];

                }
                
                else if (!(ej.isNullOrUndefined(datasource[i][model.parentTaskIdMapping]))) {
                    tempRecord.push(datasource[i]);
                }
            }

            if (tempRecord && tempRecord.length > 0) {
                return tempRecord;
            }
        },


        _appendChildDataItems: function (parentDatasource, data) {
            var proxy = this,
                model = proxy.model;

            for (var count = 0; count < parentDatasource.length; count++) {

                if (parentDatasource[count][model.taskIdMapping] === data[0][model.parentTaskIdMapping]) {
                    parentDatasource[count][model.childMapping] = data;
                }
                else if (!(ej.isNullOrUndefined(parentDatasource[count][model.childMapping])) &&
                    parentDatasource[count][model.childMapping].length > 0) {
                    proxy._appendChildDataItems(parentDatasource[count], data);
                }
            }

            if ((ej.isNullOrUndefined(parentDatasource.length)) && parentDatasource) {
                if (parentDatasource[model.taskIdMapping] === data[0][model.parentTaskIdMapping]) {
                    parentDatasource[model.childMapping] = data;
                }
                else if (!(ej.isNullOrUndefined(parentDatasource[model.childMapping])) &&
                    parentDatasource[model.childMapping].length > 0) {
                    proxy._appendChildDataItems(parentDatasource[model.childMapping], data);
                }
            }

        },
        _getParentIndex: function (datasource) {
            var proxy = this, model = proxy.model,
                dataArr = proxy.secondaryDatasource, parentIndex = 0;

            if (datasource.length > 0) {
                parentIndex = datasource[0][model.parentTaskIdMapping];
            }
            for (var i = 0; i < dataArr.length; i++) {
                if (dataArr[i][model.taskIdMapping] == parentIndex && ej.isNullOrUndefined(dataArr[i][model.childMapping])) {
                    dataArr[i][model.childMapping] = (datasource);
                    return true;
                }
                else {
                    var childItems = dataArr[i][model.childMapping];
                    if (childItems) {
                        for (var i = 0; i < childItems.length; i++) {
                            if (childItems[i][model.taskIdMapping] == parentIndex && ej.isNullOrUndefined(childItems[i][model.childMapping])) {
                                childItems[i][model.childMapping] = (datasource);
                                return true;
                            }
                        }
                    }
                }
            }

        },

        reRenderChart: function (scheduleMode) {
            var proxy = this, model = proxy.model,
                startDate = proxy._getDateFromFormat(model.scheduleStartDate),
                endDate = proxy._getDateFromFormat(model.scheduleEndDate);
            model.ganttRecords = [];
            model.parentRecords = [];
            $("#ejGanttChart" + proxy._id).empty();

            switch (scheduleMode) {
                case ej.Gantt.ScheduleHeaderType.Week:
                    model.scheduleHeaderSettings.scheduleHeaderType = ej.Gantt.ScheduleHeaderType.Week;
                    proxy._perDayWidth = 30;
                    proxy._calculateWeekSplit(startDate, endDate);
                    break;

                case ej.Gantt.ScheduleHeaderType.Day:
                    model.scheduleHeaderSettings.scheduleHeaderType = ej.Gantt.ScheduleHeaderType.Day;
                    model.scheduleHeaderSettings.dayHeaderFormat = "dd, MMM yy";
                    proxy._perHourWidth = 20;
                    if (model.workingTimeScale == ej.Gantt.workingTimeScale.TimeScale24Hours)
                        proxy._perDayWidth = (proxy._perHourWidth * 24);//24 hours
                    else if (model.workingTimeScale == ej.Gantt.workingTimeScale.TimeScale8Hours)
                        proxy._perDayWidth = (proxy._perHourWidth * 24);//9 hours
                    proxy._calculateDaySplit(startDate, endDate);
                    break;

                case ej.Gantt.ScheduleHeaderType.Month:
                    model.scheduleHeaderSettings.scheduleHeaderType = ej.Gantt.ScheduleHeaderType.Month;
                    model.scheduleHeaderSettings.monthHeaderFormat = "MMM yyyy";
                    model.scheduleHeaderSettings.weekHeaderFormat = "M/dd";
                    proxy._perDayWidth = 10;
                    proxy._perWeekWidth = 70;
                    proxy._calculateMonthSplit(startDate, endDate);
                    break;

                case ej.Gantt.ScheduleHeaderType.Year:
                    model.scheduleHeaderSettings.scheduleHeaderType = ej.Gantt.ScheduleHeaderType.Year;
                    proxy._perDayWidth = 3;
                    proxy._perMonthWidth = 90;
                    proxy._calculateYearSplit(startDate, endDate);
                    break;
            }

            proxy.element.ejGantt("destroy").ejGantt(model)
            proxy._isRenddered = false;           

        },

        _updateGanttRecords: function() {
            var proxy = this, model = proxy.model, count = model.ganttRecords.length;
            for (var i = 0; i < count; i++) {
                model.ganttRecords[i].left = (model.ganttRecords[i].left/proxy._prevDayWidth)*proxy._perDayWidth;
                model.ganttRecords[i].width = (model.ganttRecords[i].width / proxy._prevDayWidth) * proxy._perDayWidth;
                model.ganttRecords[i].width = model.ganttRecords[i].isMilestone == true ? 26 : model.ganttRecords[i].width,
                model.ganttRecords[i].progressWidth = model.ganttRecords[i]._calculateProgressWidth(model.ganttRecords[i].width, model.ganttRecords[i].status);
            }

        },
        
        _initPrivateProperties: function () {

            var proxy = this;
            model = proxy.model;
            scheduleMode = model.scheduleHeaderSettings.scheduleHeaderType;
            proxy._isFromGantt = true;
            proxy._ganttHeight = proxy.element.height() ? proxy.element.height() + "px" : proxy.model.sizeSettings.height,
            proxy._ganttWidth = proxy.element.width() ? proxy.element.width() + "px" : proxy.model.sizeSettings.width,
            proxy._disabledToolItems = [],
            proxy._id = proxy.element.attr('id'),
            proxy._scheduleWeeks = [],
            proxy._duplicate = false,          
            proxy._wrongenddate=false,
            proxy._scheduleYears = [],//new property for year schedule modes

            proxy._scheduleMonths = [], //new property for month

            proxy._scheduleYears = [],//property for schedule years
            proxy._scheduleDays = [],//property for schedule days

            proxy._projectStartDate, //property instead of scheduleWeeks[0] and scheduleYears[0]
            proxy._projectEndDate,

            proxy._columns = [],
            proxy._rows = [],
            proxy._storedIndex = -1,
            proxy._perDayWidth,
            proxy._prevDayWidth = 0,


            proxy._perMonthWidth,//new property for month
            proxy._perWeekWidth,//new property for month-week schedule

            proxy._$treegridPane = null,
            proxy._$ganttchartPane = null,
            proxy._$treegrid = null,
            proxy._$ganttchart = null,
            proxy._$ejGantt = null,
            proxy._$treegridHelper = null,
            proxy._$ganttchartHelper = null;
            proxy._tdsOffsetWidth = [];
            proxy._ids = [];
            proxy._predecessorsCollection = [];
            proxy._connectorLinesCollection = [];            
            proxy._updatedConnectorLineCollection = [];
            proxy._connectorlineIds = [];
            proxy._updatedGanttRecords = [];
            proxy._currentViewData = [];
            proxy._gridRows = null;

            //variables used for ExpandCollapse action in non Virtualization 
            proxy._collapsedRecordCount = 0;
            proxy._expandedRecordsCount = 0;
            proxy._totalCollapseRecordCount = 0;
            proxy._isInExpandCollapse = false;

            proxy._hiddenRecordCount = 0;
            
            proxy.prevPredecessorTO = null;
            proxy.prevPredecessorFrom = null;
            proxy.prevPredecessorcount = 0;
            proxy._isValidationEnabled = true;
            proxy._isRenddered = false;
            proxy._holidays = proxy._getHoliday();
            proxy._stringHolidays = proxy._getStringHolidays();
            proxy._isinBeginEdit = false;
            proxy._isMileStoneEdited = false;
            proxy._cellEditType = null;
            proxy._isinAddnewRecord = false;

            //variables for context menu items
            proxy._contextMenuItems = proxy._getContextMenuItems();
            proxy._subContextMenuItems = proxy._getsubContextMenuItems();

            //private variable decalartion for localization
            proxy._columnHeaderTexts = null;
            proxy._editDialogTexts = null;
            proxy._months = null;  
            proxy._days = null;
            proxy._emptyRecordText = null;
            proxy._commonColumnWidth = 150;
            proxy._searchString = "",
            proxy._emptyDataColumns = [],
            proxy.secondaryDatasource = [];
            proxy._datasourceChildItems = [];
            proxy._$predecessorTable = null;
            proxy._mappingItems = [];
            proxy._editedDialogRecord = null;
            if (scheduleMode == "week") {
                proxy._perDayWidth = 30;
            }
            else if (scheduleMode == "year") {
                proxy._perDayWidth = 3;
                proxy._perMonthWidth = 90;
            }
            else if (scheduleMode == "month") {
                proxy._perDayWidth = 10;
                proxy._perWeekWidth = 70;
            }
            else if (scheduleMode == "day") {
                proxy._perHourWidth = 20;
                if (model.workingTimeScale == ej.Gantt.workingTimeScale.TimeScale24Hours)
                    proxy._perDayWidth = (proxy._perHourWidth * 24);//24 hours
                else if (model.workingTimeScale == ej.Gantt.workingTimeScale.TimeScale8Hours)
                    proxy._perDayWidth = (proxy._perHourWidth * 24);//9 hours
            }
        },
        
        _setCultureInfo:function() {
             var proxy = this,
                model = proxy.model,
                culture = model.locale;

            model.dateFormat = !model.dateFormat ? ((ej.Gantt.localization[culture] && ej.Gantt.localization[culture]["calendars"] &&
                    ej.Gantt.localization[culture]["calendars"]["standard"]["patterns"]) ?
                (ej.Gantt.localization[culture]["calendars"]["standard"]["patterns"]["d"]) :
                Globalize.culture(culture) ? Globalize.culture(culture)["calendars"]["standard"]["patterns"]["d"] :
                    ej.Gantt.localization["en-US"]["calendars"]["standard"]["patterns"]["d"]) : model.dateFormat;

            
            //EmptyRecord text to be displayed with dataSource is null
            proxy._emptyRecordText = ej.Gantt.localization[culture] ?
                ej.Gantt.localization[culture]["emptyRecord"] :
                ej.Gantt.localization["en-US"]["emptyRecord"];
            
            //ColumnHeaderText to be displayed in treegrid
            proxy._columnHeaderTexts = ej.Gantt.localization[culture] ?
                ej.Gantt.localization[culture]["columnHeaderTexts"] :
                ej.Gantt.localization["en-US"]["columnHeaderTexts"];
            
            //EditDialog Text to be displayed in popup window
            proxy._editDialogTexts = ej.Gantt.localization[culture] ?
                ej.Gantt.localization[culture]["editDialogTexts"] :
                ej.Gantt.localization["en-US"]["editDialogTexts"];
            
            //Month names to be displayed in schedule
            proxy._months = (ej.Gantt.localization[culture] && ej.Gantt.localization[culture]["calendars"]) ?
                ej.Gantt.localization[culture]["calendars"]["standard"]["months"]["namesAbbr"] :
                Globalize.culture(culture) ? Globalize.culture(culture)["calendars"]["standard"]["months"]["namesAbbr"] :
                ej.Gantt.localization["en-US"]["calendars"]["standard"]["months"]["namesAbbr"];
            
            //Day names to displayed in schedule
            proxy._days = (ej.Gantt.localization[culture] && ej.Gantt.localization[culture]["calendars"]) ?
                ej.Gantt.localization[culture]["calendars"]["standard"]["months"]["namesAbbr"] :
                Globalize.culture(culture) ? Globalize.culture(culture)["calendars"]["standard"]["days"]["namesAbbr"] :
                ej.Gantt.localization["en-US"]["calendars"]["standard"]["days"]["namesAbbr"];

        },

        //Populate available fields
        _getMappingItems: function () {
            var proxy = this,
                model = proxy.model,
                mappingItems = [],
                length = proxy._columns.length;

            for (var i = 0; i < length; i++) {
                mappingItems.push(proxy._columns[i].mappingName);
            }
            return mappingItems;
        },

        //populate the context menu with items
        _getContextMenuItems: function () {
            var contextMenuItems = [];

            contextMenuItems = [{
                iconPath: "url(../images/MenuItem_Details.png)",
                headerText: "Task Details...",
                eventHandler: null,
                isDefault:true,

            }, {
                iconPath: "url(../images/MenuItem_Add.png)",
                headerText: "Add New Task",
                eventHandler: null,
                isDefault: true
            }
            , {
                iconPath: "url(../images/MenuItem_Indent.png)",
                headerText: "Indent",
                eventHandler: null,
                isDefault: true
            }, {
                iconPath: "url(../images/MenuItem_Outdent.png)",
                headerText: "Outdent",
                eventHandler: null,
                isDefault: true
            }, {
                iconPath: "url(../images/MenuItem_Delete.png)",
                headerText: "Delete",
                eventHandler: null,
                isDefault: true
            },         
           ];
            return contextMenuItems;
        },

        //populate sub context menu items
        _getsubContextMenuItems: function () {
            var contextMenuItems = [];

            contextMenuItems = [{
                iconPath: null,
                headerText: "Above",
                eventHandler: null
            }, {
                iconPath: null,
                headerText: "Below",
                eventHandler: null
            }
            ];
            return contextMenuItems;
        },


        _initialize: function () {
            var proxy = this, model = proxy.model,
            startDate, endDate;

            proxy._$treegridPane = ej.buildTag('div.e-treegridPane');
            proxy._$ganttchartPane = ej.buildTag('div.e-ganttchartPane');
            proxy._$ejGantt = ej.buildTag('div.e-gantt-Spliter#e-ejSpliter' + proxy._id, "", {}, { 'unselectable': "on" });     

            proxy.setColumns(proxy.createTreeGridColumns());
            proxy._updatePredecessors();

            proxy._trigger("load");
            proxy._render();
            proxy._updatedGanttRecords = proxy.getUpdatedRecords();
            proxy._currentViewData = proxy.getCurrentViewData();
            proxy._connectorLinesCollection = [];
            proxy._createConnectorLinesCollection("Load");
            proxy._$treegridHelper.ejTreeGrid("renderRecords");
            proxy._renderGanttChart();
            proxy._$ganttchartHelper && proxy._$ganttchartHelper.ejGanttChart("renderConnectorLines", proxy._connectorLinesCollection);
            //  proxy._$ganttchartHelper && proxy._$ganttchartHelper.ejGanttChart("createGridLinesTable");
            proxy._isRenddered = true;
            proxy._gridRows = proxy.getRows();
            proxy._ganttChartRows = proxy.getGanttChartRows();
            proxy._createSubContextMenuTemplate();
            proxy._createContextMenuTemplate();
            proxy._wireEvents();
            if (model.enableResize) {
                proxy.element.parent().css({ 'overflow': 'hidden' });
            }
        },

        _render: function() {
            var proxy = this, element = proxy.element, model = proxy.model;
            if (proxy.model.toolbarSettings.showToolbar) {
                element.append(proxy._renderToolBar());
            }
            proxy._ganttHeight = parseInt(proxy._ganttHeight) - proxy._getToolBarHeight();
            proxy._$treegrid = ej.buildTag("div.ejTreeGrid#ejTreeGrid" + proxy._id, "", { "height": proxy._ganttHeight }, {});
            proxy._$ganttchart = ej.buildTag("div.ejGanttChart#ejGanttChart" + proxy._id, "", { "height": proxy._ganttHeight }, {});
            proxy._$ganttchartPane.append(proxy._$ganttchart);
            proxy._$treegridPane.append(proxy._$treegrid);
            proxy._$ejGantt.append(proxy._$treegridPane);
            proxy._$ejGantt.append(proxy._$ganttchartPane);    
            
            element.append(proxy._$ejGantt);
            proxy._renderGantt();
            proxy._renderTreeGrid();
            proxy._$treegridPane.css({ 'overflow': 'hidden' });
            if (model.editSettings.allowEditing || model.editSettings.editMode == "normal" || model.editSettings.editMode == "dialogTemplate") {
                proxy.editDialogTemplate();
                element.append(proxy._renderDialog());
            }
            if (model.editSettings.allowAdding) {
                proxy.addDialogTemplate();
                var $dialog = ej.buildTag("div.e-dialog e-dialog-content e-shadow e-widget-content", "", { display: "none" }, { id: this._id + "_dialogAdd" });
                element.append($dialog);
            }
        },

        _renderDialog: function () {
            var $dialog = ej.buildTag("div.e-dialog e-dialog-content e-shadow e-widget-content", "", { display: "none" }, { id: this._id + "_dialogEdit" });
            return $dialog;
        },

        editDialogTemplate: function () {

            var proxy = this,
                model = proxy.model,
                columns = proxy._columns,
                length = columns.length,
                emptyDataColumns = [], model = proxy.model,column,predecessorColoumnIndex;

            proxy._mappingItems = proxy._getMappingItems();

            if (length == 0)
                return;

            if (model.editDialogFields.length > 0) {
                var filteredColumns = [];
                var count=0,resultColumn;
                for(;count<model.editDialogFields.length;count++)
                {
                    resultColumn = $.grep(proxy._columns, function (val) {
                        return val.mappingName === model.editDialogFields[count].field;
                    });
                    resultColumn.length && filteredColumns.push(resultColumn[0]);
                } 
                columns = filteredColumns;
            }
           
            var $tbody = ej.buildTag('div', "", {}, { 'unselectable': 'on' });
            $form = ej.buildTag('form', "", { 'height': 'auto', 'width': 'auto', 'font-family': 'Segoe UI', 'font-size': '14px' }, { id: proxy._id + "EditForm" });
            $table = ej.buildTag('table', "", { width: "500px" }, { 'unselectable': "on" });
            $tr = ej.buildTag('tr');
            for (var i = 0; i < columns.length; i++) {
                if ($.inArray(columns[i].mappingName, proxy._mappingItems) != -1) {
                    if (columns[i].mappingName !== model.predecessorMapping)
                    {
                        if ($tr.children("td").length < 2) {
                            $td = ej.buildTag('td', "", { "text-align": "left", 'padding': '5px 15px 0', 'font-weight': 'normal', "outline": "none" }, { 'unselectable': "on" });
                            $innerTable = ej.buildTag('table', "", { "outline": "none" }, { 'unselectable': "on" });

                            $inTr = ej.buildTag('tr');

                            $inTd = ej.buildTag('td.editLabel', "", { "outline": "none" }, { 'unselectable': "on" });
                            $inTd.append("<label for='" + columns[i].field + "' style='font-weight:normal;'>" + proxy._columnHeaderTexts[columns[i].field] + "</label>");

                            $inTr.append($inTd);

                            $innerTable.append($inTr);

                            $inTr2 = ej.buildTag('tr');
                            $inTd2 = ej.buildTag('td.e-editValue');

                            //column = ej.TreeGrid.getColumnByField(proxy._columns, columns[i].field);
                            column = columns[i];
                            proxy.initCellEditType(column, $inTd2);

                            $inTr2.append($inTd2);
                            $innerTable.append($inTr2);

                            $td.append($innerTable);
                            $tr.append($td);

                            //if (i % 2 === 0 && proxy._columnHeaderTexts[columns[i].field] === "Resources") {
                            //    $td.attr("colspan", "2");
                            //    $table.append($tr);
                            //    $tr = $tempTr = ej.buildTag('tr');
                            //}
                        }
                        else {
                            $table.append($tr);
                            $tr = $tempTr = ej.buildTag('tr');
                            i--;
                        }
                    }else
                    {
                        predecessorColoumnIndex=i;
                    }
                }
            }
            $table.append($tr);
            //      $("body").append($table);
            $tr = ej.buildTag("tr");

            if (model.predecessorMapping && predecessorColoumnIndex) {

                $td = ej.buildTag('td', "", { "text-align": "left", 'padding': '10px 15px 0', 'font-weight': 'normal',"outline":"none" }, { colspan: 2, 'unselectable': "on" });

                $innerTable = ej.buildTag('table#predecessoreditTable', "", { "width": "100%", "outline": "none" }, { 'unselectable': "on" });

                $inTr = ej.buildTag('tr');
                $inTd = ej.buildTag('td.editLabel', "", {"outline":"none"}, { 'unselectable': "on" });
                
                $inTd.append("<span class='e-addpre e-icon e-enable e-edit-dialog' style='cursor:pointer'>Add New</span>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<span class='e-deletepre e-icon e-disable e-edit-dialog' style='cursor:pointer'>&nbsp;&nbsp;Remove</span>");

                $inTr.append($inTd);
                $innerTable.append($inTr);

                $inTr2 = ej.buildTag('tr');
                $inTd2 = ej.buildTag('td.e-editValue');

                $preDiv = ej.buildTag("div#treegrid" + proxy._id + "predecessoredit", "", { "width": "100%", "height": "150px" }, {});

                $inTd2.append($preDiv);
             
                $inTr2.append($inTd2);
                $innerTable.append($inTr2);

                $td.append($innerTable);
                $tr.append($td);
                $table.append($tr);
            }

            
            $form.append($table);
            $form.appendTo($tbody);
           
            $tbody = proxy.renderDiaglogButton($form, $tbody);
            $.templates(proxy._id + "_JSONDialogEditingTemplate", $tbody.html());
        },
        addDialogTemplate: function () {

            var proxy = this,
                model = proxy.model,
                columns = proxy._columns,
                length = columns.length,
                emptyDataColumns = [], model = proxy.model, column, predecessorColoumnIndex;

            proxy._mappingItems = proxy._getMappingItems();

            if (length == 0)
                return;

            if (model.addDialogFields.length > 0) {
                var filteredColumns = [];
                var count = 0, resultColumn;
                for (; count < model.addDialogFields.length; count++) {
                    resultColumn = $.grep(proxy._columns, function (val) {
                        return val.mappingName === model.addDialogFields[count].field;
                    });
                    resultColumn.length && filteredColumns.push(resultColumn[0]);
                }
                columns = filteredColumns;
            }

            var $tbody = ej.buildTag('div', "", {}, { 'unselectable': 'on' });
            $form = ej.buildTag('form', "", { 'height': 'auto', 'width': 'auto', 'font-family': 'Segoe UI', 'font-size': '14px' }, { id: proxy._id + "AddForm" });
            $table = ej.buildTag('table', "", { width: "500px" }, { 'unselectable': "on" });
            $tr = ej.buildTag('tr');
            for (var i = 0; i < columns.length; i++) {
                if ($.inArray(columns[i].mappingName, proxy._mappingItems) != -1) {
                    if (columns[i].mappingName !== model.predecessorMapping) {
                        if ($tr.children("td").length < 2) {
                            $td = ej.buildTag('td', "", { "text-align": "left", 'padding': '5px 15px 0', 'font-weight': 'normal', "outline": "none" }, { 'unselectable': "on" });
                            $innerTable = ej.buildTag('table', "", { "outline": "none" }, { 'unselectable': "on" });

                            $inTr = ej.buildTag('tr');

                            $inTd = ej.buildTag('td.editLabel', "", { "outline": "none" }, { 'unselectable': "on" });
                            $inTd.append("<label for='" + columns[i].field + "' style='font-weight:normal;'>" + proxy._columnHeaderTexts[columns[i].field] + "</label>");

                            $inTr.append($inTd);

                            $innerTable.append($inTr);

                            $inTr2 = ej.buildTag('tr');
                            $inTd2 = ej.buildTag('td.e-editValue');

                            column = columns[i];
                            proxy.initCellEditType(column, $inTd2,"Add");

                            $inTr2.append($inTd2);
                            $innerTable.append($inTr2);

                            $td.append($innerTable);
                            $tr.append($td);

                        }
                        else {
                            $table.append($tr);
                            $tr = $tempTr = ej.buildTag('tr');
                            i--;
                        }
                    } else {
                        predecessorColoumnIndex = i;
                    }
                }
            }
            $table.append($tr);

            $tr = ej.buildTag("tr");

            if (model.predecessorMapping && predecessorColoumnIndex) {

                $td = ej.buildTag('td', "", { "text-align": "left", 'padding': '10px 15px 0', 'font-weight': 'normal', "outline": "none" }, { colspan: 2, 'unselectable': "on" });

                $innerTable = ej.buildTag('table#predecessoreditTable', "", { "width": "100%", "outline": "none" }, { 'unselectable': "on" });

                $inTr = ej.buildTag('tr');
                $inTd = ej.buildTag('td.editLabel', "", { "outline": "none" }, { 'unselectable': "on" });

                $inTd.append("<span class='e-addpre e-icon e-enable e-add-dialog' style='cursor:pointer'>Add New</span>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<span class='e-deletepre e-icon e-disable e-add-dialog' style='cursor:pointer'>&nbsp;&nbsp;Remove</span>");

                $inTr.append($inTd);
                $innerTable.append($inTr);

                $inTr2 = ej.buildTag('tr');
                $inTd2 = ej.buildTag('td.e-editValue');

                $preDiv = ej.buildTag("div#treegrid" + proxy._id + "predecessorAdd", "", { "width": "100%", "height": "150px" }, {});

                $inTd2.append($preDiv);

                $inTr2.append($inTd2);
                $innerTable.append($inTr2);

                $td.append($innerTable);
                $tr.append($td);
                $table.append($tr);
            }


            $form.append($table);
            $form.appendTo($tbody);

            $tbody = proxy.renderDiaglogButton($form, $tbody);
            $.templates(proxy._id + "_JSONDialogAddingTemplate", $tbody.html());
        },
        _clearContextMenu: function () {
            //variables for context menu items
         
            $('.e-contextmenu').remove();
            $('.e-innerContextmenu').remove();
        },

        //JSRender Template for ContextMenu
        _createContextMenuTemplate: function () {
            var proxy = this;
            helpers = {
                getHeaderName: $.proxy(proxy._getHeaderName, proxy),
            };
            $.views.helpers(helpers);
            var menuItemList = "<li style='list-style-type:none;width:150px;height:25px;display:inline-block;'>";
            var listChild = "<div class='e-menuitem' id={{:headerText}}  style='left:0px;padding-top:3px;height:25px;width:150px;cursor:pointer;display:block;position:absolute;'  {{if !isDefault}}onClick='{{:eventHandler}}'{{/if}}>" +
                            "<div class='e-icon' style='left:10px;top:4px;height:20px;position:absolute;" +
                            "background-image:{{:iconPath}};background-repeat:no-repeat;'/>" +
                            "<div style='left:35px;position:absolute;'>" +
                            "<span style='font-family:Segoe UI;font-size:12px;padding-left:5px;'>{{:headerText}}</span></div>" +
                            "{{if ~getHeaderName(#data)}}<span class='e-icon e-expander' style='height:16px;width:16px;right:5px;top:7px;position:absolute;'></span>{{/if}}";

            menuItemList += listChild;
            menuItemList += "</div></li>";
            var templates = {};
            templates[proxy._id + "contextMenuTemplate"] = menuItemList;
            $.templates(templates);
        },

        //helper methods for context menu template
        _getHeaderName: function (data) {
            if (data.headerText == "Add New Task" && this._subContextMenuItems.length > 0) {
                return true;
            }
            return false;
        },

        //JSRender Template for Inner Context Menu
        _createSubContextMenuTemplate: function () {
            var proxy = this,
                helpers = {
                    getHeaderName: proxy._getHeaderName
                };
            $.views.helpers(helpers);

            var menuItemList = "<li style='list-style-type:none;width:120px;height:25px;display:inline-block;'>";
            var listChild = "<div class='e-menuitem' id={{:headerText}} style='left:0px;padding-top:3px;" +
                "height:25px;width:120px;cursor: pointer;display:block;position:absolute;'>" +
                "<div style='left:10px;height:20px;width:16px;position:absolute;padding-top:5px;" +
                "background-image:{{:iconPath}};background-repeat:no-repeat;'/>" +
               "<div style='left:10px;position:absolute;'>" +
                "<span style='font-family:Segoe UI;font-size:12px;padding-left:5px;'>{{:headerText}}" +
                "</span></div></div>";


            menuItemList += listChild;
            menuItemList += "</li>";

            var templates = {};
            templates[proxy._id + "subContextMenuTemplate"] = menuItemList;
            $.templates(templates);
        },

        initCellEditType: function (column, element,editTarget) {

            var proxy = this,
                model = proxy.model,
                id,    
                helpers = {};
            id = editTarget ? editTarget : "";
            if (ej.isNullOrUndefined(column["editType"]))
                column["editType"] = "stringedit";

            switch (column["editType"]) {
                case "stringedit":
                    if (column.mappingName === model.predecessorMapping) {
                        helpers["_" + proxy._id + "predecessor"] = proxy._getPredecessorsValue;
                        $.views.helpers(helpers);
                        element.html(ej.buildTag('input.e-field e-ejinputtext', "", {}, { value: "{{:~_" + proxy._id + "predecessor('" + proxy._id + "Object','" + model.predecessorMapping + "','" + column.field + "')}}", id: proxy._id + column.field, name: column.field }));
                    } else {
                        element.html(ej.buildTag('input.e-field e-ejinputtext', "", {}, { value: "{{:#data['" + column.field + "']}}", id: proxy._id + column.field + id, name: column.field, dialog: editTarget }));
                    }
                    break;
                case "booleanedit":
                    element.html('{{if ' + column.field + '}} <input class="e-field e-checkbox" type ="checkbox" id=' + proxy._id + column.field + ' name=' + column.field + ' checked="checked"></input>{{else}} <input class="e-field e-checkbox" type ="checkbox" id=' + proxy._id + column.field + ' name=' + column.field + ' > {{/if}}');
                    break;
                case "numericedit":
                    var $numericText = ej.buildTag('input.e-numerictextbox e-field', "", {}, { type: "text", value: "{{:#data['" + column.field + "']}}", id: proxy._id + column.field + id, name: column.field, dialog: editTarget });
                    element.append($numericText);
                    break;
                case "datepicker":
                case "datetimepicker":
                    var $datePicker = ej.buildTag('input.e-' + column["editType"] + ' e-field', "", {}, { type: "text", value: "{{:#data['" + column.field + "']}}", id: proxy._id + column.field + id, name: column.field, dialog: editTarget });
                    element.append($datePicker);
                    break;
                case "dropdownedit":
                    var $dropDownList = ej.buildTag('input.e-field e-dropdownlist' + ' e-field', "", {}, { type: "text", value: "{{:#data['" + column.field + "']}}", id: proxy._id + column.field + id, name: column.field, dialog: editTarget });
                    element.append($dropDownList);                    
                    break;
            }
        },


        _getPredecessorsValue: function (gridObject, mappingName) {
            return this.data.item ? this.data.item[mappingName] : "";
        },

        _updateEditFormData:function()
        {
            var proxy = this;
        },
        _onHoliDayCheckBefore: function (startDate) {
            var proxy = this,
                holidays = proxy.model.holidays,
                includeWeekend = proxy.model.includeWeekend;


            if (holidays && holidays.length) {
                var holidayIndex = 0,
                    holidayslength = holidays.length,
                    holiday, flag;
                    for (holidayIndex = 0; holidayIndex < holidayslength; holidayIndex++) {
                        holiday = proxy._getDateFromFormat(holidays[holidayIndex].day);
                        if (holiday.getTime() == startDate.getTime()) {
                            startDate.setDate(startDate.getDate() - 1);
                            if (this.model.includeWeekend === false) {
                                proxy._setBeforeWeekend(startDate);
                            }
                        }
                        else {
                            return startDate;
                        }
                    }
            }
            else {
                return startDate;
            }
        },
        _onHoliDayCheck: function (startDate)
        {
            var proxy = this,
                holidays = proxy.model.holidays,
                includeWeekend = proxy.model.includeWeekend;
                

            if (holidays && holidays.length) {
                var holidayIndex = 0,
                    holidayslength = holidays.length,
                    holiday, flag;
                do {
                    for (holidayIndex = 0; holidayIndex < holidayslength; holidayIndex++) {
                        holiday = proxy._getDateFromFormat(holidays[holidayIndex].day);
                        if (holiday.getTime()==startDate.getTime()) {
                            startDate.setDate(startDate.getDate() + 1);
                            if (this.model.includeWeekend === false) {
                                if (startDate.getDay() == 0) {
                                    startDate.setDate(startDate.getDate() + 1);
                                } else if (startDate.getDay() == 6) {
                                    startDate.setDate(startDate.getDate() + 2);
                                }
                            }
                        }
                        else
                        {
                            return startDate;
                        }
                    }
                }while(1);

            }
            else
            {
                return startDate;
            }
        },
        _editStartDateChange:function(element,args)
        {
            var proxy = this,
                id = proxy._id,
                model = proxy.model,
                targetId = element.attr("id"),
                holidays = this.model.holidays,
                dialog = element.attr("dialog"),
                selectedItem = this.selectedItem(),
                editedObj = this._editedDialogRecord,
                startDateId, endDateId, durationId;
               

                if (dialog === "Add")
                {
                    startDateId = "#" + id + "startDateAdd";
                    endDateId = "#" + id + "endDateAdd";
                    durationId = "#" + id + "durationAdd";
                }
                else
                {
                    startDateId = "#" + id + "startDate";
                    endDateId = "#" + id + "endDate";
                    durationId = "#" + id + "duration";
                }

                var startDate = $(startDateId).val();
                endDate = $(endDateId).val();
                duration = parseInt($(durationId).val());

                if (!startDate)
                {
                    startDate = editedObj?editedObj.startDate:undefined;
                }
                if (isNaN(duration)) {
                    duration = editedObj ? editedObj.duration : undefined;
                }
                if (!endDate) {
                    endDate = editedObj ? editedObj.endDate : undefined;
                }

                if ((targetId === id + "startDate") || (targetId === id + "startDateAdd"))
                { 
                    startDate = proxy._getDateFromFormat(args.value);
                    var tempEndDate = new Date();
                    stratDate=proxy._onHoliDayCheck(startDate);
                    //check start date is in week end days
                    if (this.model.includeWeekend === false) {
                        if (startDate.getDay() === 0) {
                            startDate.setDate(startDate.getDate() + 1);
                        }
                        else if (startDate.getDay() === 6) {
                            startDate.setDate(startDate.getDate() + 2);
                        }
                    }
                    tempEndDate = startDate;
                    startDate = Globalize.format(startDate, this.model.dateFormat);
                    //$("#" + id + "startDate").val(startDate);
                    if (editedObj)
                        editedObj.startDate = startDate;

                if (model.dateFormat.indexOf("hh")!=-1)
                    $(startDateId).ejDateTimePicker("option", "value", startDate);
                else
                    $(startDateId).ejDatePicker("option", "value", startDate);

                if (!endDate || isNaN(duration))
                    return;

                if (model.durationMapping) {

                    //check week end is inside the period

                    if (this.model.includeWeekend === false) {
                        for (count = 1; count < duration; count++) {
                            if (tempEndDate.getDay() == 0)
                                tempEndDate.setDate(tempEndDate.getDate() + 1);
                            else if (tempEndDate.getDay() == 6)
                                tempEndDate.setDate(tempEndDate.getDate() + 2);
                            tempEndDate.setDate(tempEndDate.getDate() + 1);
                        }
                    } else {
                        tempEndDate.setDate(tempEndDate.getDate() + duration - 1);
                    }

                    if (this.model.includeWeekend === false) {
                        if (tempEndDate.getDay() == 0)
                            tempEndDate.setDate(tempEndDate.getDate() + 1);
                        else if (tempEndDate.getDay() == 6)
                            tempEndDate.setDate(tempEndDate.getDate() + 2);
                    }

                    //holiday collection checking in end date
                    if (holidays && holidays.length) {
                        var holidayIndex = 0,
                            holidayslength = holidays.length,
                            holiday;
                        for (holidayIndex = 0; holidayIndex < holidayslength; holidayIndex++) {
                            holiday = new Date(holidays[holidayIndex].day);
                            if (holiday >= proxy._getDateFromFormat(new Date(startDate), this.model.dateFormat) && holiday <= tempEndDate) {
                                tempEndDate.setDate(tempEndDate.getDate() + 1);
                                if (this.model.includeWeekend === false) {
                                    if (tempEndDate.getDay() == 0) {
                                        tempEndDate.setDate(tempEndDate.getDate() + 1);
                                    } else if (tempEndDate.getDay() == 6) {
                                        tempEndDate.setDate(tempEndDate.getDate() + 2);
                                    }
                                }
                            }
                        }
                    }

                    if (this.model.includeWeekend === false) {
                        if (tempEndDate.getDay() == 0)
                            tempEndDate.setDate(tempEndDate.getDate() + 1);
                        else if (tempEndDate.getDay() == 6)
                            tempEndDate.setDate(tempEndDate.getDate() + 2);
                    }

                    tempEndDate = Globalize.format(tempEndDate, this.model.dateFormat);
                    //temp =this._calculateFormatedDate(temp, this.model.dateFormat);
                    if (editedObj)
                        editedObj.endDate = tempEndDate;

                    if (model.dateFormat.indexOf("hh") != -1)
                        $(endDateId).ejDateTimePicker("option", "value", new Date(tempEndDate));
                    else
                        $(endDateId).ejDatePicker("option", "value", new Date(tempEndDate));

                    //$("#" + id + "endDate").val(tempEndDate);
                    //$("#" + id + "endDate").attr("value", tempEndDate);
                }
                else {
                    var dateDiff = proxy._daydiff(proxy._getDateFromFormat(this.selectedItem().startDate), proxy._getDateFromFormat(editedObj.startDate));
                    editedObj.duration -= dateDiff;
                }
            }

            else if((targetId=== id + "endDate") ||(targetId===id + "endDateAdd"))
            {
                endDate = proxy._getDateFromFormat(args.value);

                endDate = proxy._onHoliDayCheck(endDate);
                if (this.model.includeWeekend === false) {
                    if (endDate.getDay() === 0) {
                        endDate.setDate(endDate.getDate() - 2);
                    }
                    else if (endDate.getDay() === 6) {
                        endDate.setDate(endDate.getDate() - 1);
                    }
                }
                endDate = Globalize.format(endDate, this.model.dateFormat);

                if (editedObj)
                    editedObj.endDate = endDate;

                if (model.dateFormat.indexOf("hh") != -1)
                    $(endDateId).ejDateTimePicker("option", "value", new Date(endDate));
                else
                    $(endDateId).ejDatePicker("option", "value", new Date(endDate));
             
                if (!startDate || isNaN(duration))
                    return;

                startDate = proxy._getDateFromFormat(startDate);
                endDate = proxy._getDateFromFormat(endDate);

                if (startDate.getTime() <= endDate.getTime())
                {
                        
                    var tempDuration = 0;
                    dateDiff = proxy._daydiff(proxy._getDateFromFormat(startDate), proxy._getDateFromFormat(endDate));

                    if (this.model.includeWeekend === false) {
                        offset = proxy._calculateDateDifference(new Date(startDate), dateDiff);
                        offset += 1;
                    }
                    else
                        offset = dateDiff + 1;

                    for (var count = 0; count < holidays.length; count++) {
                        if (new Date(holidays[count].day) >= new Date(startDate) && new Date(holidays[count].day) <= new Date(endDate)) {
                            offset--;
                        }
                    }
                    tempDuration = offset;
                    if (editedObj)
                        editedObj.duration = tempDuration;

                    $(durationId).ejNumericTextbox("option", "value", tempDuration);
                }
                else {
                    if (editedObj)
                        editedObj.endDate = args.prevDate;

                    if (model.dateFormat.indexOf("hh") != -1)
                        $(endDateId).ejDateTimePicker("option", "value", new Date(args.prevDate));
                    else
                        $(endDateId).ejDatePicker("option", "value", new Date(args.prevDate));

                }
            }
            else if ((targetId === id + "duration") || (targetId === id + "durationAdd"))
            {
                startDate = proxy._getDateFromFormat(startDate);
                var tempEndDate = new Date(startDate);

                if (!endDate || !startDate)
                    return;

                duration = args.value;

                //check week end is inside the period
                if (this.model.includeWeekend === false) {
                    for (count = 1; count < duration; count++) {
                        if (tempEndDate.getDay() == 0)
                            tempEndDate.setDate(tempEndDate.getDate() + 1);
                        else if (tempEndDate.getDay() == 6)
                            tempEndDate.setDate(tempEndDate.getDate() + 2);
                        tempEndDate.setDate(tempEndDate.getDate() + 1);
                    }
                } else {
                    tempEndDate.setDate(tempEndDate.getDate() + duration - 1);
                }

                if (this.model.includeWeekend === false) {
                    if (tempEndDate.getDay() == 0)
                        tempEndDate.setDate(tempEndDate.getDate() + 1);
                    else if (tempEndDate.getDay() == 6)
                        tempEndDate.setDate(tempEndDate.getDate() + 2);
                }
                //holiday collection checking in end date
                if (holidays && holidays.length) {
                    var holidayIndex = 0,
                        holidayslength = holidays.length,
                        holiday;
                    for (holidayIndex = 0; holidayIndex < holidayslength; holidayIndex++) {
                        holiday = new Date(holidays[holidayIndex].day);
                        if (holiday >= proxy._getDateFromFormat(startDate, this.model.dateFormat) && holiday <= tempEndDate) {
                            tempEndDate.setDate(tempEndDate.getDate() + 1);
                            if (this.model.includeWeekend === false) {
                                if (tempEndDate.getDay() == 0) {
                                    tempEndDate.setDate(tempEndDate.getDate() + 1);
                                } else if (tempEndDate.getDay() == 6) {
                                    tempEndDate.setDate(tempEndDate.getDate() + 2);
                                }
                            }
                        }
                    }
                }

                if (this.model.includeWeekend === false) {
                    if (tempEndDate.getDay() == 0)
                        tempEndDate.setDate(tempEndDate.getDate() + 1);
                    else if (tempEndDate.getDay() == 6)
                        tempEndDate.setDate(tempEndDate.getDate() + 2);
                }
                tempEndDate = Globalize.format(tempEndDate, this.model.dateFormat);

                if (editedObj) {
                    editedObj.endDate = tempEndDate;
                    editedObj.duration = duration;
                }
                if (model.dateFormat.indexOf("hh") != -1)
                    $(endDateId).ejDateTimePicker("option", "value", new Date(tempEndDate));
                else
                    $(endDateId).ejDatePicker("option", "value", new Date(tempEndDate));
            }
        },

        _taskNameCollection:function()
        {
            var proxy = this,
            records = proxy._updatedGanttRecords,
            names = [];
            for(var i=0;i<records.length;i++)
            {
               
                var temp = { id: records[i].taskId.toString(), text: (records[i].taskId.toString()+"-"+records[i].taskName), value: records[i].taskName };
                names.push(temp);
            }
            return names;
        }
        ,

        _idFromPredecessor:function(pre)
        {
            var str = "123FS+2";
            var preArray = pre.split(',');
            var preIdArray = [];
           

            for (var j = 0; j < preArray.length; j++)
            {
                var strArray = [];
                for (var i = 0; i < preArray[j].length; i++) {
                    if (!isNaN(preArray[j].charAt(i))) {
                        strArray.push(preArray[j].charAt(i));
                    }
                    else {
                        break;
                    }
                }
                preIdArray.push((strArray.join('')));
            }
            return preIdArray;
        },
        _preIdChangByName:function()
        {
            var proxy = this;
            var $form = $("#" + proxy._id + "PreEditForm");
            $targetTr = $form.closest("tr");
            $val = $form.find(".onEdit");
            var temp = $val.val();
            var names = proxy._taskNameCollection();

            var id = $.map(names, function (obj, index) {
                if (obj.text === $val.val()) {
                    return obj.id;
                }
            });
            id = id[0];
            $targetTr.find(".id").html(id);
        },
        _preBeginEdit:function(args)
        {
            if (args.columnIndex === 0)
            {
                args.cancel = true;

                // args.
            }
        },
        _preEndEdit:function(args)
        {
            var proxy = this;
            if (args.columnName === "name")
            {
                var ds = args.model.dataSource;
                var names = proxy._taskNameCollection();
                var id = $.map(names, function (obj, index) {
                    if (obj.text === args.value) {
                        return obj.id;
                    }
                });
                id = id[0];
                args.model.selectedItem.id = id;
                
                var curent = args.model.dataSource.indexOf(args.model.selectedItem.item);

                if (curent === -1) {
                    args.model.dataSource.push(args.model.selectedItem.item);
                    curent = args.model.dataSource.indexOf(args.model.selectedItem.item);
                }
                args.model.dataSource[curent].id = id;
                $currentRow = args.rowElement;
                $currentRow[0].cells[0].innerText = id;
                if($currentRow[0].cells[2].innerText.length<=0)
                {
                    $currentRow[0].cells[2].innerText = "Finish-Start";
                    args.model.dataSource[curent].type = "Finish-Start";
                }
                proxy.enbleDisableAddPreButton('enable');
            }
        },
        _deleteDisableEnable:function(args)
        {
            var proxy = this;
            $(".e-deletepre").unbind("click", $.proxy(proxy._deletepredecessor, proxy));
            $(".e-deletepre").bind("click", $.proxy(proxy._deletepredecessor, proxy));
            $(".e-deletepre").removeClass("e-disable");
            $(".e-deletepre").addClass("e-enable");
        },
        _preDblClickHandler:function(e)
        {
            var proxy = this;
            $target = $(e.target);
            $sample = $("#" + proxy._id + "PreEditForm");
            //proxy._on(proxy.element, "click", ".e-tdprecontent", proxy._preClickHandler);
            // proxy._off(proxy.element, "dblclick", ".e-tdprecontent", proxy._preDblClickHandler);
            if ($sample.length === 0)
            {
                $form = ej.buildTag("form.#" + proxy._id + "PreEditForm", "", {}, {});
                $form.submit(function (event) { event.preventDefault()});
                var s = $target.attr("edittype");
                var field = $target.attr("field");
                var types = [{ id: "SS", text: "Start-Start", value: "Start-Start" }, { id: "SF", text: "Start-Finish", value: "Start-Finish" }, { id: "FS", text: "Finish-Start", value: "Finish-Start" }, { id: "FF", text: "Finish-Finish", value: "Finish-Finish" }];

                switch(field)
                {
                    case "taskName":
                        $attchelemet = ej.buildTag("input.e-field e-dropdownlist e-field onEdit", "", {}, { type: "text", id: proxy._id + 'taskNameEdit',value:e.target.innerHTML });
                        $form.append($attchelemet);
                        $target.empty().append($form);
                        $target.addClass("onEdit");
                        $attchelemet.ejDropDownList({
                            dataSource:proxy._taskNameCollection(),
                            width: "auto",
                            change: $.proxy(proxy._preIdChangByName, proxy),
                        });
                        $attchelemet.focus();
                        break;
                    case "type":
                        $attchelemet = ej.buildTag("input.e-field e-dropdownlist e-field onEdit", "", {}, { type: "text", id: proxy._id + 'TyepeEdit', value: e.target.innerHTML });
                        $form.append($attchelemet);
                        $target.empty().append($form);
                        $target.addClass("onEdit");
                        $attchelemet.ejDropDownList({
                            dataSource:types,
                            width: "auto"
                        });
                        $attchelemet.focus();
                        break;
                    case "offset":
                        $attchelemet = ej.buildTag('input.e-numerictextbox e-field onEdit', "", {}, { type: "text", id: proxy._id + 'OffsetEdit' });
                        var cellValue;
                        if (isNaN(parseInt(e.target.innerHTML))) {
                            cellValue = 0;
                        }
                        else
                            cellValue = parseInt(e.target.innerHTML);
                        $form.append($attchelemet);
                        $target.empty().append($form);
                        $target.addClass("onEdit");
                        $attchelemet.ejNumericTextbox({
                            width:"auto",                  
                            showSpinButton: true,
                            value: cellValue
                        });

                        $attchelemet.focus();
                        break;
                    case "id":
                        break;

                }
            }
        },
        _predecessorEditCollection:function(){
        
            var proxy = this,
               model = proxy.model;
            var gridId = "#" + proxy._id + "predecessor";
            var types = [{ id: "SS", text: "Start-Start", value: "Start-Start" }, { id: "SF", text: "Start-Finish", value: "Start-Finish" }, { id: "FS", text: "Finish-Start", value: "Finish-Start" }, { id: "FF", text: "Finish-Finish", value: "Finish-Finish" } ];
            var data = [];
            var selectedItem = model.selectedItem;
            var names = proxy._taskNameCollection();
            if (!ej.isNullOrUndefined(selectedItem) && !ej.isNullOrUndefined(selectedItem.predecessor) && !ej.isNullOrUndefined(selectedItem.item[model.predecessorMapping]))
            {
                var predecessor = selectedItem.predecessor;
                //var item = selectedItem.item;
                //var prede = proxy._idFromPredecessor(item.predecessor);

                for (var i = 0; i < predecessor.length; i++) {
                    var id = predecessor[i].from;

                    if (selectedItem.taskId.toString()!==id.toString())
                    {
                        var taskName, type, lags = 0;
                        for (var j = 0; j < names.length; j++) {
                            if (names[j].id === id) {
                                taskName = names[j].text;
                                break;
                            }
                        }
                        for (var k = 0; k < types.length; k++) {
                            if (types[k].id == predecessor[i].predecessorsType) {
                                type = types[k].text;
                            }
                        }
                        lags = predecessor[i].offset;
                        var obj = { id: id, name: taskName, type: type, offset: lags };
                        data.push(obj);
                    }
                }
            }
            return data;
        },
        _refreshEditForm: function (args) {
            var proxy = this,
                elementFocused = false,
                $formElement,
                percent = 86,
                i = 0, form, length,
                $element,
                inputWidth = "165px",
                model = proxy.model,
                width,
                params = {},
                value,
                column,
                customParams,
                toformat, newIdInstant,
                formatVal, editTreeGridId;
            selectedItem = model.selectedItem;
            
          
            if (args.requestType == "add") {
                form = document.getElementById(proxy._id + "AddForm");
                editTreeGridId = "#treegrid" + proxy._id + "predecessorAdd";
            }
            else {
                form = document.getElementById(proxy._id + "EditForm");
                editTreeGridId = "#treegrid" + proxy._id + "predecessoredit";
            }

            $formElement = $(form).find("input,select");
            length = $formElement.length;
            for (i; i < length; i++) {
                $element = $formElement.eq(i);
                //  inputWidth = proxy._tdsOffsetWidth[i] * (percent / 100);
                params = {};
                inputWidth = 210;
                if ($element.hasClass("e-numerictextbox")) {
               
                    width = inputWidth;
                    value = $element.val();
                    params.width = width;                    
                    params.showSpinButton = true;
                    params.cssClass = model.cssClass;
                    //for parent editing 
                    var tempId = $element[0].id;
                    tempId=tempId.replace(proxy._id, '');
                    if (selectedItem !== null && selectedItem.hasChildRecords && (tempId == "duration" || tempId == "status"))
                    {
                        params.readOnly = true;
                    }
                    if (tempId == "status") {
                        params.maxValue = 100;
                    }
                    else {
                        params.maxValue = 100000;
                        params.change = $.proxy(proxy._editStartDateChange, proxy, $element);
                    }
                    params.minValue = 0;
                    if (value.length)
                        params.value = parseFloat(value);
                    else
                        params.value = 0;
                    customParams = ej.TreeGrid.getColumnByField(proxy._columns, $element.prop("name"));
                    if (!ej.isNullOrUndefined(customParams["editParams"]))
                        $.extend(params, customParams["editParams"]);
                    $element.ejNumericTextbox(params);
                    $element.prop("name", $element.prop("name").replace(proxy._id, ""));
                }
                else if ($element.hasClass("e-datepicker")) {
                    width = inputWidth;
                    params.width = width;                    
                    params.cssClass = model.cssClass;
                    params.dateFormat = model.dateFormat;
                    params.locale = model.locale;
                    params.displayDefaultDate = true;
                    var tempId = $element[0].id;
                    tempId = tempId.replace(proxy._id, '');
                    if (selectedItem !== null && selectedItem.hasChildRecords && (tempId == "startDate" || tempId == "endDate") && args.requestType !== "add" ) {
                        params.readOnly = true;
                    }
                    params.select = $.proxy(proxy._editStartDateChange, proxy, $element);

                    if ($element.val().length)
                        params.value = proxy._getDateFromFormat($element.val());
                    else
                        params.value = proxy._getDateFromFormat(model.scheduleStartDate);

                    column = ej.TreeGrid.getColumnByField(proxy._columns, $element.prop("name"));
                    if (column["format"] !== undefined && column.format.length > 0) {
                        toformat = new RegExp("\\{0(:([^\\}]+))?\\}", "gm");
                        formatVal = toformat.exec(column.format);
                        params.dateFormat = formatVal[2];
                    }
                    if (!ej.isNullOrUndefined(column["editParams"]))
                        $.extend(params, column["editParams"]);

                    if (model.dateFormat.indexOf("hh") != -1)
                        $element.ejDateTimePicker(params);
                    else
                        $element.ejDatePicker(params);
                }
                else if ($element.hasClass("e-datetimepicker")) {
                    width = inputWidth;
                    params = {
                         width: width,
                        rtl: model.rtl,
                        locale:model.locale,
                        cssClass: model.css,
                        showButton: false
                    };
                    if ($element.val().length)
                        params.value = proxy._getDateFromFormat($element.val());
                    else
                        params.value = new Date();
                    column = ej.TreeGrid.getColumnByField(proxy._columns, $element.prop("name"));
                    if (column["format"] !== undefined && column.format.length > 0) {
                        toformat = new RegExp("\\{0(:([^\\}]+))?\\}", "gm");
                        formatVal = toformat.exec(column.format);
                        params.dateTimeFormat = formatVal[2];
                    }
                    if (!ej.isNullOrUndefined(column["editParams"]))
                        $.extend(params, column["editParams"]);

                    if (model.dateFormat.indexOf("hh") != -1)
                        $element.ejDateTimePicker(params);
                    else
                        $element.ejDatePicker(params);

                }
                else if ($element.hasClass("e-dropdownlist")) {
                    column = ej.TreeGrid.getColumnByField(proxy._columns, $element.prop("name"));
                    var dataSource = column.dropdownData;
                    var checkReadonly=false;
                    if (selectedItem !== null && selectedItem.hasChildRecords && args.requestType !== "add")
                    {
                        checkReadonly=true;
                    }

                    $element.ejDropDownList({
                        width:inputWidth,
                        showCheckbox: true,
                        dataSource: dataSource,
                        readOnly: checkReadonly,
                        fields: { id: model.resourceIdMapping, text: model.resourceNameMapping, value: model.resourceNameMapping },
                        selectedItems: proxy.getIndexofresourceInfo(dataSource,args.data)
                    });
                    $element.ejDropDownList("setSelectedValue", $element.val());
                } else {
                    switch ($element.prop('tagName')) {
                        case "INPUT":

                            if ($element.attr("type") != "checkbox") {
                                //For ID field id is read only

                                if ($element.attr("name") === "taskId") {
                                    if (args.requestType !== "add") {
                                        $element.attr("readonly", "readonly");
                                    }
                                    else if (args.requestType === "add") {
                                        newIdInstant = proxy._updatedGanttRecords.length;
                                        newIdInstant = proxy._getNewTaskIdFordialogEdit(newIdInstant);
                                        $element.val(newIdInstant);
                                    }
                                }
                                else if ($element.attr("name") === "taskName" && args.requestType === "add") {
                                    $element.val("New Task" + newIdInstant);
                                }

                                var tempId = $element[0].id;
                                tempId = tempId.replace(proxy._id, '');

                                $element.css("text-align", $element.attr("name") != null && ej.TreeGrid.getColumnByField(proxy._columns, $element.prop("name")) != null ?
                                    ej.TreeGrid.getColumnByField(proxy._columns, $element.attr("name")).textAlign : "center");
                                $element.outerWidth(inputWidth).height(25);
                            } else
                                $element.width("165px");//(inputWidth > 0 ? ($element.width() > inputWidth ? inputWidth : $element.width()) : 1);
                            break;

                        case "SELECT":
                            $element.width(inputWidth).height(23);
                            break;
                    }
                }
                if (!$element.is(":disabled") && !elementFocused && (!$element.is(":hidden") || typeof $element.data("ejDropDownList") == "object")) {
                    if (!proxy._isEnterKeyPressed) {
                        proxy._focusElements($element.closest('td'));
                        elementFocused = true;
                    }
                }
            }
           

            //render prdecessor table in edit dialog
            if (model.predecessorMapping && $(editTreeGridId).length) {
                var types = [{ id: "SS", text: "Start-Start", value: "Start-Start" }, { id: "SF", text: "Start-Finish", value: "Start-Finish" }, { id: "FS", text: "Finish-Start", value: "Finish-Start" }, { id: "FF", text: "Finish-Finish", value: "Finish-Finish" }];
                var ds = [];
                if (args.requestType !== "add")
                {
                    ds=proxy._predecessorEditCollection();
                }
                $(editTreeGridId).ejTreeGrid({
                    dataSource:ds,
                    allowSorting: false,
                    allowAdding: true,
                    columns: [{ headerText: "ID", field: "id", width: "50px",allowEditing:false },
                        { headerText: "TaskName", field: "name", editType: ej.TreeGrid.EditingType.Dropdown, dropdownData: proxy._taskNameCollection(), width: "170px" },
                        { headerText: "Type", field: "type", editType: ej.TreeGrid.EditingType.Dropdown, dropdownData: types, width: "80px" },
                        { headerText: "Offset", field: "offset", editType: ej.TreeGrid.EditingType.Numeric, width: "50px" }],
                    enableAltRow: true,
                    editSettings: {
                        allowAdding: true,
                        allowDeleting: true,
                        allowEditing: true,
                        editMode: "cellEditing",
                    },
                    endEdit: $.proxy(proxy._preEndEdit, proxy),
                    beginEdit: $.proxy(proxy._preBeginEdit, proxy),
                    rowSelected: $.proxy(proxy._deleteDisableEnable, proxy),
                    treeColumnIndex:5,
                });
                
                if (model.selectedItem && model.selectedItem.hasChildRecords && args.requestType!=="add")
                {
                    proxy.enbleDisableDeletePreButton('disable');
                    proxy.enbleDisableAddPreButton('disable');
                }
                else {
                    proxy.enbleDisableDeletePreButton('disable');
                    proxy.enbleDisableAddPreButton('enable');
                }
            }
        },

        enbleDisableDeletePreButton:function(args)
        {
            var proxy = this;
            if(args==="enable")
            {
                $(".e-deletepre").unbind("click", $.proxy(proxy._deletepredecessor, proxy));
                $(".e-deletepre").bind("click", $.proxy(proxy._deletepredecessor, proxy));
                $('.e-deletepre').addClass('e-enable');
                $('.e-deletepre').removeClass('e-disable');
            }
            else {
                $(".e-deletepre").unbind("click", $.proxy(proxy._deletepredecessor, proxy));
                $('.e-deletepre').addClass('e-disable');
                $('.e-deletepre').removeClass('e-enable');
            }
        },
        enbleDisableAddPreButton:function(args)
        {
            var proxy = this;
            if (args === "enable") {
               $(".e-addpre").unbind("click", $.proxy(proxy._addpredecessor, proxy));
               $(".e-addpre").bind("click", $.proxy(proxy._addpredecessor, proxy));
                $('.e-addpre').addClass('e-enable');
                $('.e-addpre').removeClass('e-disable');
            }
            else {
                $(".e-addpre").unbind("click", $.proxy(proxy._addpredecessor, proxy));
                $('.e-addpre').addClass('e-disable');
                $('.e-addpre').removeClass('e-enable');
            }
        },
        _disableAddButton: function (treeGridId) {
            var proxy = this;
            var table = $(treeGridId +"e-table");
            var flag = false;
            var len = table[0].rows.length;
            proxy.enbleDisableAddPreButton('disable');
            for (var j = 0; j < len; j++)
            {
                var currentRow = table[0].rows[j];
                if(currentRow.cells.length===4)
                if(currentRow.cells[1].innerText==="")
                {
                    flag = true;
                }
            }
            if (!flag) {
                proxy.enbleDisableAddPreButton('enable');
            }
        },
        _addpredecessor: function (e) {
            var proxy = this;
            var args = {},treeGridId;
            args["requestType"] = ej.TreeGrid.Actions.Save;
            args["_cAddedRecord"] = {item:{ id: "", name: "", type: "", offset: "" },hasChildRecords:false,level:0};
            proxy.enbleDisableAddPreButton('disable');
            if ($(e.target).hasClass("e-edit-dialog"))
                treeGridId="#treegrid" + proxy._id + "predecessoredit";
            else
                treeGridId = "#treegrid" + proxy._id + "predecessorAdd";

            var selectedIndex = $(treeGridId).ejTreeGrid("option", "selectedRowIndex");
            $(treeGridId).ejTreeGrid("clearSelection", selectedIndex);
            $(treeGridId).ejTreeGrid("processBindings", args);
        },

        _deletepredecessor:function(e)
        {
            var proxy = this, treeGridId;

            if ($(e.target).hasClass("e-edit-dialog"))
                treeGridId = "#treegrid" + proxy._id + "predecessoredit";
            else
                treeGridId = "#treegrid" + proxy._id + "predecessorAdd";

            var selectedItem = $(treeGridId).ejTreeGrid("option", "selectedRowIndex");
            //var currentData = $("#treegrid" + proxy._id + "predecessoredit").ejTreeGrid("option", "currentViewData"); currentViewData
            var args = {};
            args["requestType"] = ej.TreeGrid.Actions.Delete;
            if (selectedItem !== -1) {
                $(treeGridId).ejTreeGrid("deleteRow");
                proxy.enbleDisableDeletePreButton('disable');
            }
            proxy._disableAddButton(treeGridId);
        },


        getColumnIndexByField: function (field) {

            var proxy = this,
                columns = proxy.model.columns,
                length = columns.length,
                column = 0;

            for (column; column < length; column++) {

                if (columns[column]["field"] == field) break;

            }

            return column;
        },

        getIndexofresourceInfo: function (dataSource,data) {
            var resourceInfo = data.resourceInfo,
                count = 0, length = resourceInfo && resourceInfo.length, resourceIndex = [];

            for (count; count < length; count++)
                resourceIndex.push(dataSource.indexOf(resourceInfo[count]));

            return resourceIndex;

        },
        
        _focusElements: function ($currentCell) {
            if ($currentCell.length) {
                $currentCell.focus();
                var $childElem = $currentCell.children();
                if ($childElem[0].tagName.toLowerCase() == "select" || $childElem[0].tagName.toLowerCase() == "input") {
                    $childElem.focus().select();
                    $childElem[0].focus();
                }
                else if ($childElem.find(".e-field.e-dropdownlist").length)
                    $childElem.find(".e-ddl").focus();
                else
                    $childElem.find('input,select').select().focus();
            }
        },

        renderDiaglogButton: function (form, tbody) {

            var btnId,
                proxy = this,
                model = proxy.model;
            if (form[0].id === (proxy._id + "EditForm"))
                btnId = "EditDialog_";
            else
                btnId = "AddDialog_";

            var savebtn = ej.buildTag('input', "",
                 {
                     'border-radius':'3px'
                 },
                 { type: "button", id: btnId + proxy._id + "_Save" }),
             saveText = proxy._editDialogTexts["saveButton"];
            
            savebtn.ejButton(
                {
                    cssClass: model.cssClass,
                    text: saveText,
                    width: "70"
                });

            var cancelText = proxy._editDialogTexts["cancelButton"],
             cancelbtn = ej.buildTag('input', "",
                 {
                     'margin-left': '20px',
                     'border-radius': '3px'
                 },
                 {
                     type: "button",
                     id: btnId + proxy._id + "_Cancel"
                 });
            
            cancelbtn.ejButton(
                {
                    cssClass: model.cssClass,
                    text: cancelText,
                    width: "70"
                });

            //Sample delete button
            var delbtn = ej.buildTag('input', "",
                {
                    'margin-left': '20px',
                    'border-radius': '3px'
                },
                {
                    type: "button",
                    id: "EditDialog_" + proxy._id + "_Delete"
                });

            delbtn.ejButton(
                {
                    cssClass: model.cssClass,
                    text: "Delete",
                    width: "70"
                });



            var btnDiv = ej.buildTag('div', "", { 'margin-top': '20px','float':'right','margin-bottom':'15px' ,'margin-right':'15px'}, { 'class': "e-editform-btn" });

            btnDiv.append(savebtn);
            if(form[0].id===(proxy._id+"EditForm") && model.editSettings.allowDeleting)
                btnDiv.append(delbtn);
            btnDiv.append(cancelbtn);
            form.appendTo(tbody);
            btnDiv.appendTo(tbody);
            
            return tbody;
        },
        _toolbarItemLeave:function(args)
        {
            var i = 0;
        },
        _renderToolBar: function() {
            var proxy = this,
                model = proxy.model,
                $div = ej.buildTag("div.e-gantttoolbar#e-gantttoolbar" + proxy._id, "", { 'height': '50px' }, { id: proxy._id + "_toolbarItems"}),
                $ul = ej.buildTag("ul", "", { 'width': proxy._ganttWidth}, {});
            (!ej.isNullOrUndefined(model.toolbarSettings.toolbarItems) && model.toolbarSettings.toolbarItems.length) && proxy._renderLi($ul);
            $div.append($ul);
            var $customUl = ej.buildTag("ul");
            $div.append($customUl);
            var helper = {};
            helper.click = proxy._toolBarClick;
            helper.cssClass = "";
            helper.rtl = false;
            helper.itemSeparator = false;
            helper.itemLeave =proxy._toolbarItemLeave
            helper.fields = {
                id: "",
                tooltipText: "",
                imageUrl: "",
                text: "",
                imageAttributes: "",
                spriteCSS: "",
                htmlAttributes: "",
            };
            $div.ejToolbar(helper);
            $div.ejToolbar("disableItem", proxy._disabledToolItems);
            proxy._disabledToolItems = $();
            return $div;
        },

        _renderLi: function ($ul) {
            var proxy = this,
                toolBarItems = proxy.model.toolbarSettings.toolbarItems,
                length = toolBarItems.length,
                i = 0,
                $li;
            for (i; i < length; i++) {

                $li = ej.buildTag("li", "", {}, {
                    id: proxy._id + "_" + toolBarItems[i],                    
                    "tabindex": "0"
                });

                if (toolBarItems[i] !== "search")
                    $li.css({ 'width': '30px', 'height': '24px' });

                proxy._renderLiContent($li, toolBarItems[i]);
                $ul.append($li);
            }
        },

        _renderLiContent: function ($li, item) {
            var $a, $input;
            switch (item) {

                case "add":
                    $a = ej.buildTag("span.e-addnewitem e-toolbaricons e-icon e-gantt-add", "", {});
                    //title: toolBarItems[i],
                    $li.attr("title","Add");
                    if (!this.model.editSettings.allowAdding)
                        this._disabledToolItems.push($li.get(0));
                    break;

                case "edit":
                    $a = ej.buildTag("span.e-edititem e-toolbaricons e-icon e-gantt-edit", "", {});
                    $li.attr("title", "Edit");
                    if (!this.selectedItem()) {
                        this._disabledToolItems.push($li.get(0));
                    }
                    
                    break;

                case "delete":
                    $li.attr("title", "Delete");
                    $a = ej.buildTag("span.e-deleteitem e-toolbaricons e-icon e-gantt-delete", "", {});
                    
                    if (!this.selectedItem()) {
                        this._disabledToolItems.push($li.get(0));
                    }
                    
                    break;

                case "update":
                    $li.attr("title", "Update");
                    $a = ej.buildTag("span.e-saveitem e-toolbaricons e-icon e-gantt-save e-disabletool", "", {});
                    this._disabledToolItems.push($li.get(0));
                    break;

                case "cancel":
                    $li.attr("title", "Cancel");
                    $a = ej.buildTag("span.e-cancel e-toolbaricons e-icon e-gantt-cancel e-disabletool", "", {});
                    this._disabledToolItems.push($li.get(0));
                    break;

                case "indent":
                    $li.attr("title", "Indent");
                    $input = ej.buildTag("span.e-indent e-toolbaricons e-icon e-disabletool e-gantt-indent", "", {}, {});
                    $li.append($input);
                    this._disabledToolItems.push($li.get(0));
                    break;

                case "outdent":
                    $li.attr("title", "Outdent");
                    $input = ej.buildTag("span.e-outdent e-toolbaricons e-icon e-disabletool e-gantt-outdent", "", {}, {});
                    $li.append($input);
                    this._disabledToolItems.push($li.get(0));
                    break;

                case "expandAll":
                    $li.attr("title", "ExpandAll");
                    $input = ej.buildTag("span.e-expandall e-toolbaricons e-icon e-gantt-expandall", "", {}, {});
                    $li.append($input);
                    break;

                case "collapseAll":
                    $li.attr("title", "CollapseAll");
                    $input = ej.buildTag("span.e-collapseall e-toolbaricons e-icon e-gantt-collapseall", "", {}, {});
                    $li.append($input); 
                    break;

                case "search":
                    $input = ej.buildTag("input.e-ejinputtext", "", { 'margin': '4px','text-indent':'4px' }, { type: "text", placeholder: "Search" }, { 'color': "#E8E9E9" });
                    $li.css({ 'float': 'right', 'padding': '2px' });
                    $li.attr("title", "Search");
                    $li.addClass('e-search');
                    $li.append($input);
                    this.model.allowSearching = true;

                    $input.keydown($.proxy(this._keyDown, this));

                    break;
            }
            $li.append($a);
        },


        _keyDown: function (e) {

            if (e.which ===13) { //13 is key code for Enter Key 
                this._searchString = e.currentTarget.value;
                this._toolbarOperation(this._id + "_search", e.currentTarget.value);
            }          
        },

        
        _updateToolbarOptions:function(searchString){

            var proxy = this,
                model = proxy.model,
                $addElement = $($('.e-addnewitem')[0]).parent(),
                $editElement = $($('.e-edititem')[0]).parent(),
                $deleteElement = $($('.e-deleteitem')[0]).parent(),
                $indentElement = $($('.e-indent')[0]).parent(),
                $outdentElement = $($('.e-outdent')[0]).parent();
                

            if (searchString.length) {

                var disabledItems = [];

                if (model.editSettings.allowAdding) {
                    $addElement && disabledItems.push($addElement.removeClass('e-disable')[0]);
                }

                if (model.editSettings.allowEditing) {
                    $editElement && disabledItems.push($editElement.removeClass('e-disable')[0]);
                }

                if (model.editSettings.allowDeleting) {
                    $deleteElement && disabledItems.push($deleteElement.removeClass('e-disable')[0]);
                }

                
                if ($indentElement && $indentElement.length)
                    $indentElement && disabledItems.push($indentElement.removeClass('e-disable')[0]);

                if ($outdentElement && $outdentElement.length)
                    $outdentElement && disabledItems.push($outdentElement.removeClass('e-disable')[0]);

                $('.e-gantttoolbar').ejToolbar('disableItem', disabledItems);

            } else {
                var enabledItems = [];

                if (model.editSettings.allowAdding) {
                    $addElement && enabledItems.push($addElement[0]);
                }

                if (model.selectedItem) {

                    if (model.editSettings.allowEditing) {
                        $editElement && enabledItems.push($editElement[0]);
                    }

                    if (model.editSettings.allowDeleting) {
                        $deleteElement && enabledItems.push($deleteElement[0]);
                    }

                    proxy.updateIndentOutdentOption(model.selectedItem);
                }

                $('.e-gantttoolbar').ejToolbar('enableItem', enabledItems);
            }
        },

        _renderGantt: function() {
            
            var proxy = this,                                                                                
                leftPaneSize,
                rightPaneSize,
                ganttWidth = parseInt(proxy._ganttWidth);           

            //calculate treegrid pane size
            leftPaneSize = (2*proxy._commonColumnWidth+3) / ganttWidth * 100;
            leftPaneSize = leftPaneSize < 30 ? leftPaneSize : 30;

            //Here 7 is the splitter span width
            rightPaneSize = 100 - leftPaneSize-( 7 / ganttWidth * 100);           
            

            $("#e-ejSpliter" + proxy._id).ejSplitter({
                height: proxy._ganttHeight,
                width: proxy._ganttWidth,
                orientation: ej.Orientation.Horizontal,
                properties: [{ paneSize: leftPaneSize + "%",collapsible:false }, { paneSize: rightPaneSize + "%",collapsible:false }],
                enableAutoResize: false,
                resize: $.proxy(proxy._onResize,proxy)
            });
        },

        _renderTreeGrid: function() {
            var proxy = this, model = proxy.model;
            proxy._$treegridHelper = $("#ejTreeGrid" + proxy._id).ejTreeGrid({
                enableAltRow: model.enableAltRow,
                allowColumnResize: model.allowColumnResize,
                enableVirtualization: model.enableVirtualization,
                allowSorting: model.allowSorting,
                allowMultiSorting: model.allowMultiSorting,
                sortSettings: model.sortSettings,
                allowSelection: model.allowSelection,
                selectionType:"single",
                selectedRowIndex: this.selectedRowIndex(),
                allowSearching: model.allowSearching,
                parentIdMapping:model.parentTaskIdMapping,
                allowKeyboardNavigation: model.allowKeyboardNavigation,
                cssClass: model.cssClass,
                locale: model.locale,
                editSettings: model.editSettings,
                rowTemplate: model.rowTemplate,
                rowDataBound: model.rowDataBound,
                queryCellInfo: model.queryCellInfo,
                dataSource:this.dataSource(),
                flatRecords: model.ganttRecords,
                parentRecords: model.parentRecords,
                records: proxy._rows,
                ids: proxy._ids,
                columns: proxy._columns,
                dateFormat: model.dateFormat,
                resourceInfoMapping: model.resourceInfoMapping,
                resourceNameMapping: model.resourceNameMapping,
                resourceIdMapping:model.resourceIdMapping,
                taskIdMapping: model.taskIdMapping,
                showGridCellTooltip: model.showGridCellTooltip,
                showGridExpandCellTooltip: model.showGridExpandCellTooltip,
                taskNameMapping: model.taskNameMapping,
                startDateMapping: model.startDateMapping,
                endDateMapping: model.endDateMapping,
                childMapping: model.childMapping,
                durationMapping: model.durationMapping,
                progressMapping: model.progressMapping,
                predecessorMapping: model.predecessorMapping,
                rowHeight: model.rowHeight,
                emptyRecordText: proxy._emptyRecordText,
                isFromGantt: proxy._isFromGantt,
                treeColumnIndex: model.treeColumnIndex,
                workingTimeScale:model.workingTimeScale,
            });
        },

        _renderGanttChart: function() {
            var proxy = this,
                model = this.model;
            proxy._$ganttchartHelper = $("#ejGanttChart" + proxy._id).ejGanttChart(
                {
                    dataSource: this.dataSource(),
                    scheduleStartDate: model.scheduleStartDate,
                    scheduleEndDate: model.scheduleEndDate,
                    startDateMapping: model.startDateMapping,
                    endDateMapping: model.endDateMapping,
                    progressMapping: model.progressMapping,
                    durationMapping: model.durationMapping,
                    childPropertyName: model.childMapping,
                    predecessorMapping: model.predecessorMapping,
                    enableVirtualization: model.enableVirtualization,
                    highlightWeekends: model.highlightWeekends,
                    milestoneMapping: model.milestoneMapping,
                    enableRowHover: model.enableRowHover,
                    perDayWidth: proxy._perDayWidth,
                    workingTimeScale: model.workingTimeScale,
                    roundOffDayworkingTime: model.roundOffDayworkingTime,
                    durationUnit: model.durationUnit,
                    
                    perMonthWidth: proxy._perMonthWidth,//new property for year - month schedule
                    perWeekWidth: proxy._perWeekWidth,//new property for week - month schedule
                    perHourWidth: proxy._perHourWidth,

                    includeWeekend: model.includeWeekend,
                    connectorlineWidth: model.connectorlineWidth,
                    scheduleHeaderSettings: model.scheduleHeaderSettings,
                    taskbarBackground: model.taskbarBackground,
                    progressbarBackground: model.progressbarBackground,
                    connectorLineBackground: model.connectorLineBackground,
                    parentTaskbarBackground: model.parentTaskbarBackground,
                    parentProgressbarBackground: model.parentProgressbarBackground,
                    holidays: model.holidays,
                    dateFormat: model.dateFormat,
                    locale: model.locale,
                    enableTaskbarTooltip: model.enableTaskbarTooltip,
                    enableTaskbarDragTooltip:model.enableTaskbarDragTooltip,
                    ganttRecords: model.ganttRecords,
                    parentRecords: model.parentRecords,
                    rows: proxy._rows,
                    scheduleWeeks: proxy._scheduleWeeks,

                    projectStartDate: proxy._projectStartDate, //new property instead of scheduleWeeks[0] and scheduleYears[0]
                    projectEndDate: proxy._projectEndDate,

                    scheduleYears: proxy._scheduleYears, //new property for year
                    scheduleMonths: proxy._scheduleMonths, //new property for month
                    scheduleDays:proxy._scheduleDays,
                    progressbarHeight: model.progressbarHeight,
                    tooltipTemplate: model.taskbarTooltipTemplate,
                    tooltipTemplateId: model.taskbarTooltipTemplateId,
                    progressbarTooltipTemplate: model.progressbarTooltipTemplate,
                    progressbarTooltipTemplateId: model.progressbarTooltipTemplateId,
                    queryTaskbarInfo: model.queryTaskbarInfo,
                    showTaskNames: model.showTaskNames,
                    showProgressStatus: model.showProgressStatus,
                    showResourceNames: model.showResourceNames,
                    resourceInfoMapping: model.resourceInfoMapping,
                    resourceNameMapping: model.resourceNameMapping,
                    enableProgressBarResizing: model.enableProgressBarResizing,
                    allowGanttChartEditing: model.allowGanttChartEditing,
                    taskbarEditingTooltipTemplateId: model.taskbarEditingTooltipTemplateId,
                    taskbarEditingTooltipTemplate:model.taskbarEditingTooltipTemplate,
                    _ids: proxy._ids,
                    stripLines: model.stripLines,
                    _predecessorCollection: proxy._predecessorsCollection,
                    weekendBackground: model.weekendBackground,
                    allowKeyboardNavigation: model.allowKeyboardNavigation,
                    updatedGanttRecords: proxy._updatedGanttRecords,
                    renderBaseline: model.renderBaseline,
                    baselineColor:model.baselineColor,
                    editSettings: model.editSettings,
                    columnHeaderTexts: proxy._columnHeaderTexts,
                    localizedDays: proxy._days,
                    localizedMonths:proxy._months,
                    rowHeight: model.rowHeight
                });
        },

        createTreeGridColumns: function () {

            var proxy = this, model = proxy.model, column, columns = [];

            var mapping = (model.taskIdMapping)//?model.taskIdMapping:"Task Id";

            if (mapping.length) {
                column = {
                    field: "taskId",
                    headerText: proxy._columnHeaderTexts["taskId"],
                    width: 30,
                    editType: ej.Gantt.EditingType.String,
                    mappingName: mapping
                };

                columns.push(column);
                //proxy._emptyDataColumns.push(column.field);
            }

            mapping = model.taskNameMapping;// "Task Name";

            if (mapping.length) {
                column = {
                    field: "taskName",
                    headerText: proxy._columnHeaderTexts["taskName"],
                    width: 150,
                    editType: ej.Gantt.EditingType.String,
                    mappingName: mapping
                };

                columns.push(column);
                //proxy._emptyDataColumns.push(column.field);
            }

            mapping = model.startDateMapping;// "Start Date";

            if (mapping.length) {
                column = {
                    field: "startDate",
                    headerText:proxy._columnHeaderTexts["startDate"],
                    width: 150,
                    editType: ej.Gantt.EditingType.DatePicker,
                    mappingName: mapping
                };

                columns.push(column);
                //proxy._emptyDataColumns.push(column.field);
            }


            mapping = model.endDateMapping;// "End Date";

            if (mapping.length) {
                column = {
                    field: "endDate",
                    headerText: proxy._columnHeaderTexts["endDate"],
                    width: 150,
                    editType: ej.Gantt.EditingType.DatePicker,
                    mappingName:mapping
                };

                columns.push(column);
                //proxy._emptyDataColumns.push(column.field);
            }

            mapping = model.resourceInfoMapping;//"Resources";

            if (mapping.length) {
                column = {
                    field: "resourceInfo",
                    headerText:proxy._columnHeaderTexts["resourceInfo"],
                    width: 150,
                    editType: ej.Gantt.EditingType.Dropdown,
                    mappingName: mapping,
                    dropdownData: model.resources //model.resources is null
                };
                columns.push(column);
                //proxy._emptyDataColumns.push(column.field);
            }

            mapping = model.durationMapping;// "Duration";

            if (mapping.length) {
                column = {
                    field: "duration",
                    headerText: proxy._columnHeaderTexts["duration"],
                    width: 150,
                    editType: ej.Gantt.EditingType.Numeric,
                    mappingName:mapping
                };

                columns.push(column);
                //proxy._emptyDataColumns.push(column.field);
            }

            mapping = model.progressMapping;// "Progress";

            if (mapping.length) {
                column = {
                    field: "status",
                    headerText: proxy._columnHeaderTexts["status"],
                    width: 150,
                    editType: ej.Gantt.EditingType.Numeric,
                    mappingName:mapping
                };
                columns.push(column);
                //proxy._emptyDataColumns.push(column.field);
            }
            mapping = model.predecessorMapping;//)?model.predecessorMapping: "Predecessor";

            if (mapping.length) {
                column = {
                    field: "predecessor",
                    headerText:proxy._columnHeaderTexts["predecessor"],
                    width: 150,
                    editType: ej.Gantt.EditingType.String,
                    mappingName: mapping
                };
                columns.push(column);
                //proxy._emptyDataColumns.push(column.field);
            }
            
            //mapping = (model.baselineStartDateMapping )?model.baselineStartDateMapping: "Baseline StartDate";

            //if (mapping.length) {
            //    column = {
            //        field: "baselineStartDate",
            //        headerText: proxy._columnHeaderTexts["baselineStartDate"],
            //        width: 150,
            //        editType: ej.Gantt.EditingType.DatePicker,
            //        mappingName: mapping
            //    };
            //    columns.push(column);
            //    proxy._emptyDataColumns.push(column.field);
            //}
            //mapping = (model.baselineEndDateMapping)?model.baselineEndDateMapping: "Baseline EndDate";

            //if (mapping.length) {
            //    column = {
            //        field: "baselineEndDate",
            //        headerText:proxy._columnHeaderTexts["baselineEndDate"],
            //        width: 150,
            //        editType: ej.Gantt.EditingType.DatePicker,
            //        mappingName: mapping
            //    };
            //    columns.push(column);
            //    proxy._emptyDataColumns.push(column.field);
            //}
            return columns;
        },
        

        _createGanttRecords: function(data) {
            var proxy = this,
                model = proxy.model,
                ganttRecords = model.ganttRecords,
                length = data.length,
                count = 0,
                parentGanttRecord,
                parentRecords = model.parentRecords,
                enableSorting = model.allowSorting,
                enableAltRow = model.enableAltRow,
                rows = proxy._rows,
                ids = proxy._ids;


            for (count; count < length; count++) {
                parentGanttRecord = proxy._createGanttRecord(data[count], 0, null,undefined,"Load");
                proxy._storedIndex++;
                //if (enableAltRow)
                parentGanttRecord.isAltRow = proxy._storedIndex % 2 == 0 ? false : true;
                parentGanttRecord.index = proxy._storedIndex;
                rows[proxy._storedIndex] = parentGanttRecord;
                ids[proxy._storedIndex] = parentGanttRecord["taskId"].toString();
                ganttRecords.push(parentGanttRecord);
                //if (enableSorting)
                parentRecords.push(parentGanttRecord);
                parentGanttRecord.childRecords && proxy._addNestedGanttRecords(parentGanttRecord.childRecords);
            }

            for (var i = 0; i < ganttRecords.length; i++) {
                
                if (ganttRecords[i].isMilestone) {
                    if (ganttRecords[i].parentItem) {
                        ganttRecords[i].startDate = ganttRecords[i].parentItem.endDate;
                    } else {
                        ganttRecords[i].startDate = ganttRecords[i - 1].endDate;
                    }
                    ganttRecords[i].left = ganttRecords[i]._calculateLeft(proxy._projectStartDate, proxy._perDayWidth,
                        proxy._holidays, model);
                }

                if (ganttRecords[i].parentItem) {

                    if (!ganttRecords[i].predecessor) {
                        proxy._updateParentItem(ganttRecords[i]);
                    }

                    if (!ganttRecords[i].hasChildRecords)
                        ganttRecords[i]._updateParentProgress(ganttRecords[i].parentItem,model.progressMapping);
                }
                
            }
        },
        

        //AddNested Gantt Records
        _addNestedGanttRecords: function (data) {
            var proxy = this,
                rows = proxy._rows,
                ids = proxy._ids,
                model = proxy.model,
                ganttRecords = model.ganttRecords,
                count = 0,
                length = data.length,
                records = [],
                record,
                enableAltRow = model.enableAltRow;
            
            for (count; count < length; count++) {
                record = data[count];
                proxy._storedIndex++;
                //if (enableAltRow)
                record.isAltRow = proxy._storedIndex % 2 == 0 ? false : true;;
                record.index = proxy._storedIndex;
                rows[proxy._storedIndex] = record;
                ids[proxy._storedIndex] = record["taskId"].toString();
                ganttRecords.push(record);
                records = record.childRecords;
                if (records) {
                    var j = 0, recordlength = records.length, childGanttRecord = null;
                    for (j; j < recordlength; j++) {
                        childGanttRecord = records[j];
                        proxy._storedIndex++;
                        //if (enableAltRow)
                        childGanttRecord.isAltRow = proxy._storedIndex % 2 == 0 ? false : true;
                        childGanttRecord.index = proxy._storedIndex;
                        rows[proxy._storedIndex] = childGanttRecord;
                        ids[proxy._storedIndex] = childGanttRecord["taskId"].toString();
                        ganttRecords.push(childGanttRecord);
                        childGanttRecord.childRecords && proxy._addNestedGanttRecords(childGanttRecord.childRecords);
                    }
                }

            }
            
        },
        
        _createGanttRecord: function(data, level, parentItem, expanded,creatAt) {
            var proxy = this, ganttRecord, model = proxy.model, //column = proxy._emptyDataColumns,
                child = data[model.childMapping],
                startDate = data[model.startDateMapping],
                endDate = data[model.endDateMapping],
                duration = data[model.durationMapping] ? data[model.durationMapping] : 0,
                status =data[model.progressMapping]?data[model.progressMapping]:0,
                startDateType = proxy._getDateFromFormat(startDate),
                endDateType = proxy._getDateFromFormat(endDate),
                resourceInfo = data[model.resourceInfoMapping],
                predecessors = data[model.predecessorMapping],
                baselineStartDate = data[model.baselineStartDateMapping],
                baselineStartDateType = proxy._getDateFromFormat(baselineStartDate, model.dateFormat),
                baselineEndDate = data[model.baselineEndDateMapping],
                baselineEndDateType = proxy._getDateFromFormat(baselineEndDate, model.dateFormat);
            
            var sTime = startDateType ? new Date(startDateType.getFullYear(), startDateType.getMonth(), startDateType.getDate()).getTime() : null,
                eTime = endDateType ? new Date(endDateType.getFullYear(), endDateType.getMonth(), endDateType.getDate()).getTime() : null;

            ganttRecord = new ej.Gantt.GanttRecord();
            ganttRecord.taskId = data[model.taskIdMapping],
            ganttRecord.taskName = data[model.taskNameMapping],
            ganttRecord.startDate = proxy.getFormatedDate(startDateType),
            ganttRecord.endDate = proxy.getFormatedDate(endDateType),

            ganttRecord.baselineStartDate = proxy.getFormatedDate(baselineStartDateType),
            ganttRecord.baselineEndDate = proxy.getFormatedDate(baselineEndDateType),
            ganttRecord.duration = duration;
            if (endDateType && (ej.isNullOrUndefined(duration) || sTime != eTime)) {
                ganttRecord._calculateDuration(startDateType, endDateType, model.includeWeekend, model.holidays, data[model.milestoneMapping],model);
                duration = ganttRecord.duration;
            }
            ganttRecord.isMilestone = duration == 0 ? true : (data[model.milestoneMapping]) ? data[model.milestoneMapping] : false,
            ganttRecord.duration = (ganttRecord.isMilestone && duration != 0) ? 0 : duration,
            ganttRecord.status = status,
            ganttRecord.predecessor = predecessors && ganttRecord._calculatePredecessor(predecessors),
            ganttRecord.resourceInfo = resourceInfo && ganttRecord._setResourceInfo(data[(model.resourceInfoMapping)], model.resourceIdMapping, model.resourceNameMapping, model.resources),
            ganttRecord.parentItem = parentItem;
            var tempData;
            if (model.parentTaskIdMapping && creatAt)
            {
                tempData = ej.DataManager(this.dataSource()).executeLocal(ej.Query().where(model.taskIdMapping,
                ej.FilterOperators.equal,
                ganttRecord.taskId));
                ganttRecord.item = tempData[0];
            }
            else
            {
                ganttRecord.item = data;
            }


            
            ganttRecord.isSelected = proxy.selectedItem() ? proxy.selectedItem()[(model.taskIdMapping)?model.taskIdMapping:"Task Id"] == ganttRecord.taskId ? true : false : false,
            ganttRecord.level = level,
            ganttRecord.left = ganttRecord._calculateLeft(proxy._projectStartDate, proxy._perDayWidth, proxy._holidays, model),
            ganttRecord.width = ganttRecord._calculateWidth(proxy._perDayWidth, proxy._holidays,model),
            ganttRecord.width = ganttRecord.width == 0 ? 26 : ganttRecord.width,
            ganttRecord.progressWidth = ganttRecord._calculateProgressWidth(ganttRecord.width, status),
            ganttRecord.childRecords = child && proxy._createChildRecords(child, level + 1, ganttRecord),
            ganttRecord.hasChildRecords = child ? true : false,
            ganttRecord.expanded = expanded !== undefined ? expanded : child ? child.length > 0 : false;
            if (model.enableVirtualization === false)
            {
                ganttRecord.isExpanded=true;
            }
            if (model.renderBaseline && ganttRecord.baselineStartDate && ganttRecord.baselineEndDate) {
                ganttRecord.baselineLeft = ganttRecord._calculateBaselineLeft(proxy._projectStartDate, proxy._perDayWidth, model.includeWeekend, model.dateFormat, model.endDateMapping, proxy._holidays);
                ganttRecord.baselineWidth = ganttRecord._calculateTaskbarWidth(proxy._perDayWidth, model);
            }
            else
                model.renderBaseline = false;
           

            if (predecessors)
                proxy._predecessorsCollection.push(ganttRecord);
            
            return ganttRecord;
        },
        
        _getHoliday: function () {
            if (this.model.holidays != null) {
                var holidayList = this.model.holidays,
                    holidays = [],
                    holidayLength = holidayList.length;
                if (holidayLength > 0) {
                    for (var i = 0; i < holidayLength; i++) {
                        holidays[i] = this.getDateType(holidayList[i].day);
                    }
                    return holidays;
                }
            }
            return false;
        },

        _getStringHolidays: function() {
            if (this._holidays != null && this._holidays.length > 0) {
                var holidayList = this._holidays,
                    holidays = [],
                    holidayLength = holidayList.length;
                if (holidayLength > 0) {
                    for (var i = 0; i < holidayLength; i++) {
                        holidays[i] = Globalize.format(holidayList[i], this.model.dateFormat);
                    }
                    return holidays;
                }
            }
            return false;
        },
        
        //create childRecords for particular GanttRecord
        _createChildRecords: function (data, level, parentItem) {
            var ganttRecords = [],
                proxy = this,
                i = 0,
                length = data.length;

            for (i = 0; i < length; i++) {
                if (data[i]) {
                    var parentRecord = proxy._createGanttRecord(data[i], level, parentItem,undefined,"Load");
                    ganttRecords.push(parentRecord);
                }
            }
            return ganttRecords;
        },

        _createResourceInfoCollection: function (data) {
            var proxy = this,
                count = 0,
                length = data.length;
            for (count; count < length; count++) {
                proxy._resourceInfoCollection.push(proxy._createResourceInfo(data[count]));
            }
        },

        _createResourceInfo: function (data) {
            var model = this.model, resourceInfo = {};
            resourceInfo.resourceId = data[model.resourceIdMapping];
            resourceInfo.resourceName = data[model.resourceNameMapping];
            return resourceInfo;
        },

        _calculateWeekSplit: function (startdate, enddate) {
            var proxy = this;
            var roundOffStartDate = startdate.getDay() == 0 ? (startdate.getDate()) - 7 :
                (startdate.getDate()) - startdate.getDay();
            startdate.setDate(roundOffStartDate);
            do {
                proxy._scheduleWeeks.push(new Date(startdate));
                startdate.setDate(startdate.getDate() + 7);
            } while (!(startdate >= enddate));

            proxy._projectStartDate= new Date(proxy._scheduleWeeks[0]);
            proxy._projectEndDate = new Date(proxy._scheduleWeeks[proxy._scheduleWeeks.length - 1]);
        },

        //method for calculatting years (year-month schedule mode)
        _calculateYearSplit: function (startdate, enddate) {
            var proxy = this;
            startdate.setMonth(0);
            do {
                proxy._scheduleYears.push(new Date(startdate));
                startdate.setMonth(startdate.getMonth() + 12);
            } while (!(startdate >= enddate));

            var projectStartDate, projectEndDate;
            projectStartDate = new Date(proxy._scheduleYears[0].getFullYear(), 0, 1);
            projectEndDate = new Date(proxy._scheduleYears[proxy._scheduleYears.length - 1].getFullYear(), 11, 31);
            proxy._projectStartDate = projectStartDate;
            proxy._projectEndDate = projectEndDate;
        },

        //Method for calculating days (day-hour schedule mode)
        _calculateDaySplit: function (startdate, enddate) {
            var proxy = this;
            do {
                proxy._scheduleDays.push(new Date(startdate));
                startdate.setDate(startdate.getDate() + 1);
            } while ((startdate <= enddate));
            proxy._projectStartDate = new Date(proxy._scheduleDays[0]);
            proxy._projectEndDate = new Date(proxy._scheduleDays[proxy._scheduleDays.length - 1]);
        },

        //new method for calculatting months
        _calculateMonthSplit: function (startdate, enddate) {
            var proxy = this;
            dt = new Date(startdate.getFullYear(), startdate.getMonth() - 1, 1);
            startdate = dt;

            do {
                proxy._scheduleMonths.push(new Date(startdate));
                if (startdate.getMonth() == 0 || startdate.getMonth() == 2 || startdate.getMonth() == 4 || startdate.getMonth() == 6 || startdate.getMonth() == 7 || startdate.getMonth() == 9 || startdate.getMonth() == 11)
                    startdate.setDate(startdate.getDate() + 31);
                else if (startdate.getMonth() == 3 || startdate.getMonth() == 5 || startdate.getMonth() == 8 || startdate.getMonth() == 10)
                    startdate.setDate(startdate.getDate() + 30);
                else if (startdate.getMonth() == 1) {
                    if(startdate.getFullYear()%4==0)
                        startdate.setDate(startdate.getDate() + 29);
                    else
                        startdate.setDate(startdate.getDate() + 28);
                }
            } while (!(startdate >= enddate));

            var projectStartDate, projectEndDate;
            projectStartDate = new Date(proxy._scheduleMonths[0]);
            projectEndDate = new Date(proxy._scheduleMonths[proxy._scheduleMonths.length - 1]);
            projectEndDate = new Date(projectEndDate.getFullYear(), projectEndDate.getMonth() + 1, 0);
            proxy._projectStartDate = projectStartDate;
            proxy._projectEndDate = projectEndDate;
        },



        _updatePredecessors: function () {
            var proxy = this,
                count = 0,
                predecessorsCollection = proxy._predecessorsCollection,
                length = predecessorsCollection.length,
                ganttRecord,
                connectorsCollection,
            i = 0,
            successorGanttRecord,
            connectorCount;
            for (count; count < length; count++) {
                ganttRecord = predecessorsCollection[count];
                connectorsCollection = ganttRecord["predecessor"];
                connectorCount = connectorsCollection.length;
                for (i=0; i < connectorCount; i++) {
                    var connector = connectorsCollection[i];
                    successorGanttRecord = proxy.model.ganttRecords[proxy._ids.indexOf(connector.from)];
                    connector.to = ganttRecord['taskId'].toString();
                    if (successorGanttRecord) {
                        if (successorGanttRecord.predecessor) {
                            successorGanttRecord.predecessor.push(connector);
                        } else {
                            successorGanttRecord.predecessor = [];
                            successorGanttRecord.predecessor.push(connector);
                            predecessorsCollection.push(successorGanttRecord);
                        }
                    }

                }                
            }
        },
        
        getDateType: function (date) {
            if (date != null) {
                if (typeof (date) == "object") {
                    return date;
                }
                else {
                    var hyphenRegex = new RegExp("\\-", "g"),
                    dotRegex = new RegExp("\\.", "g");
                    date = date.replace(hyphenRegex, "/");
                    date = date.replace(dotRegex, "/");
                    return new Date(date);
                }
            }
            return null;
        },

        getFormatedDate: function(date) {
            return Globalize.format(date, this.model.dateFormat);
        },

        _getProgressWidth: function (parentwidth, percent) {
            return (parentwidth * percent) / 100;
        },

        
        _wireEvents: function () {
            var proxy = this,
                model=proxy.model,
                treeGrid = $("#ejTreeGrid" + proxy._id),
                ganttChart = $("#ejGanttChart" + proxy._id),
                gantt = proxy._$ejGantt;

            proxy._on(proxy.element, "click", proxy.clickHandler);
            proxy._on(proxy.element, "contextmenu", proxy._rightClick);
            if (model.enableResize) {
                proxy._on($(window), "resize", proxy.windowResize);
            }
            //proxy._on(proxy.element, "dblclick", ".e-tdprecontent", proxy._preDblClickHandler);
            //proxy._on(proxy.element, "click", ".e-tdprecontent", proxy._preClickHandler);
            treeGrid.ejTreeGrid({ rowSelecting: $.proxy(proxy.rowSelecting, proxy) });
            treeGrid.ejTreeGrid({ rowSelected: $.proxy(proxy.rowSelected, proxy) });
            //treeGrid.ejTreeGrid({ rowHover: $.proxy(proxy._treeGridRowHover, proxy) });
            treeGrid.ejTreeGrid({ actionBegin: $.proxy(proxy.actionBegin, proxy) });
            treeGrid.ejTreeGrid({ actionComplete: $.proxy(proxy.actionComplete, proxy) });

            if (model.queryCellInfo) {
                treeGrid.ejTreeGrid({ queryCellInfo: $.proxy(proxy._queryCellInfo, proxy) });
            }
            
            if (model.rowDataBound) {
                treeGrid.ejTreeGrid({ rowDataBound: $.proxy(proxy._rowDataBound, proxy) });
            }
            
            treeGrid.ejTreeGrid({ beginEdit: $.proxy(proxy._beginEdit, proxy) });
            treeGrid.ejTreeGrid({ endEdit: $.proxy(proxy._endEdit, proxy) });            
            
            treeGrid.ejTreeGrid({ expanding: $.proxy(proxy.expanding, proxy) });
            treeGrid.ejTreeGrid({ collapsing: $.proxy(proxy.collapsing, proxy) });
            treeGrid.ejTreeGrid({ expanded: $.proxy(proxy.expanded, proxy) });
            treeGrid.ejTreeGrid({ collapsed: $.proxy(proxy.collapsed, proxy) });
            treeGrid.ejTreeGrid({ refreshRow: $.proxy(proxy.refreshRow, proxy) });
            treeGrid.ejTreeGrid({ cancelEditCell: $.proxy(proxy.cancelEditCell, proxy) });
            
            
            ganttChart.ejGanttChart({ rowSelecting: $.proxy(proxy.rowSelecting, proxy) });
            ganttChart.ejGanttChart({ rowSelected: $.proxy(proxy.rowSelected, proxy) });
            ganttChart.ejGanttChart({ rowHover: $.proxy(proxy._ganttChartRowHover, proxy) });
            ganttChart.ejGanttChart({ actionBegin: $.proxy(proxy.actionBegin, proxy) });
            ganttChart.ejGanttChart({ actionComplete: $.proxy(proxy.chartactionComplete, proxy) });
            ganttChart.ejGanttChart({ refreshRow: $.proxy(proxy.refreshRowData, proxy) }); 
            ganttChart.ejGanttChart({ zooming: $.proxy(proxy.zoomingChart, proxy) });
            ganttChart.ejGanttChart({ queryTaskbarInfo: $.proxy(proxy.queryTaskbarInfo, proxy) });
            ganttChart.ejGanttChart({ expanding: $.proxy(proxy.expanding, proxy) });
            ganttChart.ejGanttChart({ collapsing: $.proxy(proxy.collapsing, proxy) });
            ganttChart.ejGanttChart({ expanded: $.proxy(proxy.expanded, proxy) });
            ganttChart.ejGanttChart({ collapsed: $.proxy(proxy.collapsed, proxy) });

            ganttChart.ejGanttChart({ taskbarEditing: $.proxy(proxy.taskbarEditing, proxy) });
            ganttChart.ejGanttChart({ taskbarEdited: $.proxy(proxy.taskbarEdited, proxy) });

            gantt.ejSplitter({ allowResizing: $.proxy(proxy._onResize, proxy) });
            
            proxy._enableEditingEvents();
        },
 

        _onResize:function(args){
            var proxy = this;
            
            proxy._$treegridHelper.ejTreeGrid("refreshScroller", args.prevPane.size);
            proxy._$ganttchartHelper.ejGanttChart("refreshScroller", args.nextPane.size);

        },

        clickHandler: function (e) {
            var proxy = this;
            //var $target = e.target;

            //if ($target.closest(".e-popup").length == 0 &&
            //  $target.closest(".e-rowcell").find("#" + proxy._id + "EditForm").length == 0) {
                
            //}


            proxy._clearContextMenu();

            if (e.toElement) {
                e.toElement.tabIndex = 0;
                e.toElement.focus();
            }

            if (proxy._isinBeginEdit) {

                var $target = $(e.target);

                if (!$target.closest('td').hasClass("e-editedcell") && !$target.hasClass('e-date') && !$target.hasClass('e-arrow') && !$target.hasClass('e-down-arrow')) {
                    proxy._$treegridHelper.ejTreeGrid("cancelEditCell");
                    proxy._isinBeginEdit = false;
                }
                else if ($target.closest('td').hasClass("e-editedcell") && !$target.hasClass('e-date') && !$target.hasClass('e-arrow') && !$target.hasClass('e-down-arrow'))
                {
                    $target.focus();
                }
            }
        },

        windowResize: function (e) {
            var proxy = this, leftPaneSize,
                rightPaneSize,
                ejGanttSplit = ("#e-ejSpliter" + this._id),
                toolbar = ("#" + this._id + "_toolbarItems"),
                ganttchart = ("#ejGanttChart" + this._id),
                treegrid = ("#ejTreeGrid" + this._id),
                ganttbody = ("#" + this._id + "viewerbodyContianer" + "ejGanttChart" + this._id),
                gantthead = ("#" + this._id + "viewerheaderContainer" + "ejGanttChart" + this._id),
                treegridhelp = ("#ejTreeGrid" + this._id),
                treegridcontent = ("#ejTreeGrid" + this._id + "e-gridcontent"),
                treegridPanel = (".e-treegridPane"),
                ganttchartPanel = (".e-ganttchartPane"),

                width = $(window).width(),
                height = $(window).height(),
                ganttObject = $(proxy._id).ejGantt("instance"),

                data, ganttWidth, treeGridWidth, top, totalWidth, gridWidth, chartWidth, chartPaneWidth;

            $(ejGanttSplit).width(width);
            $(ejGanttSplit).height(height - $(toolbar).height());
            data = $(ejGanttSplit).ejSplitter("option", "properties");
            $(ejGanttSplit).ejSplitter("refresh");
            $(toolbar).width(width);
            $(".e-ul").width(width);

            //calculate treegrid pane size
            leftPaneSize = ((2 * 150) + 3);
           
            //Here 7 is the splitter span width
            rightPaneSize = width - leftPaneSize - 7;

            ganttWidth = $(ganttchart).width();
            treeGridWidth = $(treegrid).width();

            $(ganttchart).height(height - $(toolbar).height());

            $(ganttbody).ejScroller({
                width: ganttWidth,
                height: (height - $(toolbar).height() - $(gantthead).height() - 3),
            });

            top = $(treegridcontent).ejScroller("option", "scrollTop");
            $(ganttbody).ejScroller("option", "scrollTop", top);
            $(ganttbody).ejScroller("refresh");

            $(treegridhelp).height(height - $(toolbar).height());
            $(treegridcontent).height((height - $(toolbar).height() - $(gantthead).height() - 3));
            $(treegridcontent).ejScroller("refresh");

            totalWidth = $(ejGanttSplit)[0].getBoundingClientRect().width;
            gridWidth = $(treegridPanel)[0].getBoundingClientRect().width;
            chartWidth = $(ganttchartPanel)[0].getBoundingClientRect().width;

            chartPaneWidth = totalWidth - gridWidth - 9; // (9 -> 7 from splitter & 2 from Border)
            $(ganttchartPanel).css("width", chartPaneWidth);
           
                proxy._$treegridHelper.ejTreeGrid("updateViewPortHeight");
                proxy._$treegridHelper.ejTreeGrid("processBindings");

                var tempArgs = {};
                tempArgs.requestType = ej.TreeGrid.Actions.Refresh;
                proxy._$treegridHelper.ejTreeGrid("sendDataRenderingRequest", tempArgs);
                
                proxy._$ganttchartHelper.ejGanttChart("refreshHelper", proxy._currentViewData, proxy._updatedGanttRecords);

                proxy._$ganttchartHelper.ejGanttChart("updateViewPortHeight");
                
    
        },

        
        // handler for mouse right click
        _rightClick: function (e) {
            var proxy = this,
                $target = $(e.target),
                div = $target[0].parentNode,
                $ganttGridRows,
                row,
                recordIndexr,
                item,selectIndex,
                args = {};
            proxy._$treegridHelper.ejTreeGrid("disableTooltip");
           
            if ($(div).hasClass("e-chartcell") || $(div).hasClass("e-ganttrowcell")) {
                $ganttGridRows = proxy._$ganttchartHelper.ejGanttChart("getGanttChartRows");
                args.target = "ejTreeGrid";
            } else {
                $ganttGridRows = $(proxy._$treegridHelper.ejTreeGrid("getRows"));
                args.target = "ejGanttChart";
            }
            row = $target.closest('tr');
            recordIndexr = $ganttGridRows.index(row);
            proxy._currentViewData = proxy.getCurrentViewData();
            item = proxy._currentViewData[recordIndexr];
            recordIndexr = proxy._updatedGanttRecords.indexOf(item);
            args.data = item;
            args.recordIndex = recordIndexr;

            if (item && proxy.model.enableContextMenu) {
                e.preventDefault();
                proxy._contextMenuItems = [];
                proxy._contextMenuItems = proxy._getContextMenuItems();
                proxy._subContextMenuItems = [];
                proxy._subContextMenuItems = proxy._getsubContextMenuItems();
                proxy._updateIndentOutdentContextmenuOption(recordIndexr, item);
                proxy._clearContextMenu();
                if (proxy._contextMenuItems.length > 0)
                    proxy._renderContextMenu(e, recordIndexr, item);

                proxy.rowSelected(args);
                proxy._$treegridHelper.ejTreeGrid("selectRows", recordIndexr);
                proxy._$ganttchartHelper.ejGanttChart("selectRows", recordIndexr);
            }
            else if (proxy.dataSource()==null || proxy.dataSource().length == 0) {
                proxy._contextMenuItems = [];
                proxy._subContextMenuItems = [];
                proxy._contextMenuItems = proxy._getContextMenuItems();
                proxy._updateIndentOutdentContextmenuOption(recordIndexr, item);
                proxy._clearContextMenu();
                if (proxy._contextMenuItems.length > 0)
                    proxy._renderContextMenu(e, recordIndexr, item);
            }
        },

        _updateIndentOutdentContextmenuOption: function (recordIndex, ganttRecord) {
            var proxy = this;
            if (recordIndex === 0) {;
                proxy._contextMenuItems.splice(2, 2);
            }
            else if(recordIndex>0){
                if (ganttRecord.level === 0) {
                    proxy._contextMenuItems.splice(3, 1);
                }
                else {
                    var previousGanttRecord = proxy._updatedGanttRecords[recordIndex - 1];
                    if (ganttRecord.level - previousGanttRecord.level === 1) {
                        proxy._contextMenuItems.splice(2, 1);
                    }
                    if(ganttRecord.isMilestone)
                        proxy._contextMenuItems.splice(3, 1);
                    if(previousGanttRecord.isMilestone)
                        proxy._contextMenuItems.splice(2, 1);
                }
            }
            else if (recordIndex < 0) {
                proxy._contextMenuItems.splice(0, 1);
                proxy._contextMenuItems.splice(1, 3);
            }

            var addIndex = -1, editIndex = -1, deleteIndex = -1;

            if (!model.editSettings.allowAdding) {
                $.each(proxy._contextMenuItems, function (idx, val) {
                    if (val.headerText === "Add New Task") {
                        addIndex = idx;
                    }
                });
                if(addIndex !== -1)
                    proxy._contextMenuItems.splice(addIndex, 1);
            }

            if (!model.editSettings.allowDeleting)
            {
                $.each(proxy._contextMenuItems, function (idx, val) {
                    if (val.headerText === "Delete") {
                        deleteIndex = idx;
                    }
                });
                if (deleteIndex !== -1)
                    proxy._contextMenuItems.splice(deleteIndex, 1);
            }

            if (!model.editSettings.allowEditing)
            {
                $.each(proxy._contextMenuItems, function (idx, val) {
                    if (val.headerText === "Task Details...") {
                        editIndex = idx;
                    }});
                proxy._contextMenuItems.splice(editIndex, 1);
            }

            if( proxy._searchString.length>0 && addIndex!==-1)
            {
                proxy._contextMenuItems.splice(addIndex, 1);
            }


        },
      
        queryTaskbarInfo: function (args) {        
            this._trigger("queryTaskbarInfo", args);
        },
        

        zoomingChart: function (args) {
            var proxy = this;
            proxy.reRenderChart(args.delta);
        },

        refreshRowData: function (args) {
            var proxy = this;
            proxy._$treegridHelper.ejTreeGrid("refreshRow", args.index);
        },

        expanding: function (args) {
            var proxy = this;           
            if (!proxy._trigger("expanding", args)) {
                if (proxy.model.enableVirtualization) {                    
                    ej.TreeGrid.sendExpandCollapseRequest(proxy, args);
                } else {
                    proxy._isInExpandCollapse = true;
                    
                    ej.TreeGrid.sendExpandCollapseRequest(proxy, args);
                    proxy._isInExpandCollapse = false;
                    var expandedCount = proxy._expandedRecordsCount;
                    proxy._$ganttchartHelper.ejGanttChart("reRenderGanttGridLinesTable", expandedCount);
                }
            }
        },

        collapsing: function (args) {
            var proxy = this;
            if (!proxy._trigger("collapsing", args)) {
                if (proxy.model.enableVirtualization) {                    
                    ej.TreeGrid.sendExpandCollapseRequest(proxy,args);
                } else {
                    proxy._gridRows = proxy.getRows();
                    proxy._isInExpandCollapse = true;
                    //proxy.sendExpandCollapseRequest(args);
                    ej.TreeGrid.sendExpandCollapseRequest(proxy, args);
                    proxy._isInExpandCollapse = false;
                    var collapsedCount = proxy._collapsedRecordCount; 
                    proxy._$ganttchartHelper.ejGanttChart("reRenderGanttGridLinesTable", -collapsedCount);
                }
            }
        },

        expanded: function (args) {
            var proxy = this;            
            proxy._updatedGanttRecords = proxy.getUpdatedRecords();
            proxy._currentViewData = proxy.getCurrentViewData();
            if (proxy.model.enableVirtualization) {
                if (proxy.model.predecessorMapping) {
                    proxy._isValidationEnabled = false;
                    proxy._$ganttchartHelper.ejGanttChart("clearConnectorLines");
                    proxy._connectorlineIds = [];
                    proxy._connectorLinesCollection = [];
                    proxy._createConnectorLinesCollection();
                }
                proxy._$ganttchartHelper.ejGanttChart("refreshHelper", proxy._currentViewData, proxy._updatedGanttRecords);
            }
            proxy._trigger("expanded", args);
        },

        collapsed: function (args) {
            var proxy = this;
            proxy._updatedGanttRecords = proxy.getUpdatedRecords();
            proxy._currentViewData = proxy.getCurrentViewData();
            if (proxy.model.enableVirtualization) {
                if (proxy.model.predecessorMapping) {
                    proxy._isValidationEnabled = false;
                    proxy._connectorlineIds = [];
                    proxy._connectorLinesCollection = [];
                    proxy._$ganttchartHelper.ejGanttChart("clearConnectorLines");
                    proxy._createConnectorLinesCollection();
                }
                proxy._$ganttchartHelper.ejGanttChart("refreshHelper", proxy._currentViewData, proxy._updatedGanttRecords);
            }
            proxy._trigger("collapsed", args);
        },

        refreshRow: function (args) {
            var proxy = this;
            proxy._$ganttchartHelper.ejGanttChart("refreshRow", args.recordIndex);
        },

        _enableEditingEvents: function () {
            var proxy = this,
                model = proxy.model;

            if (model.editSettings.allowEditing|| model.editSettings.editMode == "dialogTemplate" ||
                (model.toolbarSettings.showToolbar && ((model.editSettings.allowAdding && model.toolbarSettings.toolbarItems.indexOf("add") !== -1) ||
                    (model.editSettings.allowEditing && model.toolbarSettings.toolbarItems.indexOf("index") !== -1)))) {
                
                proxy._on($("#" + proxy._id + "_dialogEdit"),
                    "click keypress", "#EditDialog_" + proxy._id + "_Save ,#EditDialog_" + proxy._id + "_Cancel,#EditDialog_" + proxy._id + "_Delete", proxy._buttonClick);

                proxy._on($("#" + proxy._id + "_dialogAdd"),
                   "click keypress", "#AddDialog_" + proxy._id + "_Save ,#AddDialog_" + proxy._id + "_Cancel", proxy._buttonClick);
                // $(proxy.element).on("click", ".e-icon", $.proxy(proxy._buttonClick, proxy));
            }
        },
        
        cancelEditCell: function () {

            var proxy = this;
            
            if (proxy.model.toolbarSettings.toolbarItems.indexOf("cancel") !== -1) {
                proxy._disabledToolItems.push($($('.e-cancel')[0]).parent()[0]);
            }

            if (proxy.model.toolbarSettings.toolbarItems.indexOf("save") !== -1) {
                proxy._disabledToolItems.push($($('.e-saveitem')[0]).parent()[0]);
            }

            $('.e-gantttoolbar').ejToolbar('disableItem', proxy._disabledToolItems);
        },

        _keyPressed: function (action, target, e) {
        },

        _buttonClick: function (e) {

            if (e.keyCode !== undefined && e.keyCode != 13)
                return true;
            
            var proxy = this,   
                model = proxy.model,
                args = {};
            
            if (model.editSettings.allowEditing || model.editSettings.editMode == "dialogTemplate" ||
                (model.toolbarSettings.showToolbar && ((model.editSettings.allowAdding && model.toolbarSettings.toolbarItems.indexOf("add") !== -1) ||
                    (model.editSettings.allowEditing && model.toolbarSettings.toolbarItems.indexOf("index") !== -1)))) {

                if (e.target.id == "EditDialog_" + proxy._id + "_Save") {
                    if (!proxy._sendSaveRequest("Edit"))
                        $("#" + proxy._id + "_dialogEdit").ejDialog("close");
                } else if (e.target.id == "EditDialog_" + proxy._id + "_Cancel") {
                    $("#" + proxy._id + "_dialogEdit").ejDialog("close");
                }
                    //For delete option in editing dialog window
                else if (e.target.id == "EditDialog_" + proxy._id + "_Delete") {

                    proxy._$treegridHelper.ejTreeGrid("deleteRow");
                    $("#" + proxy._id + "_dialogEdit").ejDialog("close");
                }
                else if (e.target.id == "AddDialog_" + proxy._id + "_Save") {
                    if (!proxy._sendSaveRequest("Add"))
                        if (!this._duplicate&&!this._wrongenddate) {
                            $("#" + proxy._id + "_dialogAdd").ejDialog("close");                            
                        }
                }
                else if (e.target.id == "AddDialog_" + proxy._id + "_Cancel") {
                  //  proxy._ids.splice(proxy._ids.length - 1, 1);
                    $("#" + proxy._id + "_dialogAdd").ejDialog("close");
                }
            }
            else {
                proxy._sendCancelRequest();
            }

            return false;
        },

        rowSelecting: function (args) {
            var proxy = this;
            if (proxy.model.rowSelecting) {
                if (proxy._trigger("rowSelecting", args))
                    return true;
            }
            return true;
        },

        rowSelected:function(args){
            var proxy = this;
            if (args.target == "ejTreeGrid")
                proxy._$ganttchartHelper.ejGanttChart("selectRows", args.recordIndex);
            if (args.target == "ejGanttChart")
                proxy._$treegridHelper.ejTreeGrid("selectRows", args.recordIndex);
            var idx = args.recordIndex;
            this.selectedItem(args.data);
            this.selectedRowIndex(idx);
            if (proxy.model.toolbarSettings.showToolbar) {

                if (proxy.model.toolbarSettings.toolbarItems.indexOf("indent") !== -1 || proxy.model.toolbarSettings.toolbarItems.indexOf("outdent") !== -1) {

                    proxy.updateIndentOutdentOption(args.data);
                }

                var enableItems = [];

                if (proxy.model.editSettings.allowDeleting) {
                    if (proxy.model.toolbarSettings.toolbarItems.indexOf("delete") !== -1)
                        enableItems.push($($('.e-deleteitem')[0]).parent()[0]);
                }

                if (proxy.model.editSettings.allowEditing) {
                    if (proxy.model.toolbarSettings.toolbarItems.indexOf("edit") !== -1)
                        enableItems.push($($('.e-edititem')[0]).parent()[0]);
                }

                $('.e-gantttoolbar').ejToolbar('enableItem', enableItems);
            }

            if (proxy.model.rowSelected)
                proxy._trigger("rowSelected", args);           
        },

        _treeGridRowHover: function (args) {
            var proxy = this;
            if (args.reason == "mouseenter")
                proxy._$ganttchartHelper.ejGanttChart("addRowHover", args.index);
            else if (args.reason == "mouseleave")
                proxy._$ganttchartHelper.ejGanttChart("removeRowHover");
        },

        updateIndentOutdentOption: function (ganttRecord) {
            var proxy = this;
            proxy._updatedGanttRecords = proxy.getUpdatedRecords();
            var recordIndex = proxy._updatedGanttRecords.indexOf(ganttRecord),
                previousGanttRecord;

            var indentIconElement =  $($('.e-indent')[0]).parent()[0],
                outentIconElement = $($('.e-outdent')[0]).parent()[0],
                disabledItems = [];

            if (recordIndex === 0) {
                if (indentIconElement)
                    $('.e-gantttoolbar').ejToolbar('disableItem', indentIconElement);

                if (outentIconElement)
                    $('.e-gantttoolbar').ejToolbar('disableItem', outentIconElement);
                
                $('.e-gantttoolbar').ejToolbar('disableItem', disabledItems);
            }
            else {
                if (ganttRecord.level === 0) {
                    if (indentIconElement)
                        $('.e-gantttoolbar').ejToolbar('enableItem', indentIconElement);
                    if (outentIconElement)
                        $('.e-gantttoolbar').ejToolbar('disableItem', outentIconElement);
                }
                else {
                    previousGanttRecord = proxy._updatedGanttRecords[recordIndex - 1];
                    if (ganttRecord.level - previousGanttRecord.level === 1) {
                        if (outentIconElement)
                            $('.e-gantttoolbar').ejToolbar('enableItem', outentIconElement);
                        if (indentIconElement)
                            $('.e-gantttoolbar').ejToolbar('disableItem', indentIconElement);
                    }
                    else {
                        if (indentIconElement)
                            $('.e-gantttoolbar').ejToolbar('enableItem', indentIconElement);
                        if (outentIconElement)
                            $('.e-gantttoolbar').ejToolbar('enableItem', outentIconElement);
                    }
                    if (ganttRecord.isMilestone) {
                        if (outentIconElement)
                            $('.e-gantttoolbar').ejToolbar('disableItem', outentIconElement);
                    }
                    if (previousGanttRecord.isMilestone) {
                        if (outentIconElement)
                            $('.e-gantttoolbar').ejToolbar('disableItem', indentIconElement);
                    }
                }
            }
        },

        _ganttChartRowHover: function (args) {
            var proxy = this;
            if (args.reason == "mouseenter")
                proxy._$treegridHelper.ejTreeGrid("addRowHover", args.index);
            else if (args.reason == "mouseleave")
                proxy._$treegridHelper.ejTreeGrid("removeRowHover");
        },
        _removeChildRecordsPredecessor:function(record)
        {
            var proxy = this;
            if (record.hasChildRecords) {
                var length = record.childRecords.length, index = 0;
                for (index; index < length; index++) {
                    record.childRecords[index].predecessor && proxy._removePredecessor(record.childRecords[index].predecessor, record.childRecords[index]);
                    if(record.childRecords[index].hasChildRecords)
                    {
                        proxy._removeChildRecordsPredecessor(record.childRecords[index]);
                    }
                }
            }
        },
        actionComplete: function (args) {
            var proxy = this,model = proxy.model;
            proxy._updatedGanttRecords = proxy.getUpdatedRecords();
            proxy._currentViewData = proxy.getCurrentViewData();
            proxy._gridRows = proxy.getRows();
            proxy._ganttChartRows = proxy.getGanttChartRows();

            if (args.requestType === "searching" || args.requestType === "sorting") {
                proxy._$ganttchartHelper.ejGanttChart("clearConnectorLines");
                var visibleRecordCount = proxy._$ganttchartHelper.ejGanttChart("getVisibleRecordsCount");
                if (!model.enableVirtualization)
                {
                    var filteredCollapseCount=0;
                    if (args.requestType === "searching" || proxy._searchString)
                    {
                        filteredCollapseCount = proxy._currentViewData.filter(function (record) {
                            return (proxy.getExpandStatus(record) === false);
                        });
                        proxy._$ganttchartHelper.ejGanttChart("refreshHelper", proxy._currentViewData, proxy._updatedGanttRecords, filteredCollapseCount.length);
                        proxy._$ganttchartHelper.ejGanttChart("reRenderGanttGridLinesTable", (proxy._currentViewData.length - visibleRecordCount - filteredCollapseCount.length));
                    }
                    else
                    {
                        proxy._$ganttchartHelper.ejGanttChart("refreshHelper", proxy._currentViewData, proxy._updatedGanttRecords, proxy._totalCollapseRecordCount);
                        proxy._$ganttchartHelper.ejGanttChart("reRenderGanttGridLinesTable", (proxy._currentViewData.length - visibleRecordCount-proxy._totalCollapseRecordCount));
                    }
                }
                else
                {
                    proxy._$ganttchartHelper.ejGanttChart("refreshHelper", proxy._currentViewData, proxy._updatedGanttRecords);
                }


                proxy._isValidationEnabled = false;
                proxy._connectorlineIds = [];
                proxy._connectorLinesCollection = [];
                proxy._createConnectorLinesCollection();
            }

            if (args.requestType === "delete") {

                this.dataSource(proxy._$treegridHelper.ejTreeGrid("getDataSource"));


                model.selectedItem = null;
                this.selectedRowIndex(-1);
                var childCount = 0;
                //Collapse record counts
                if (!args.data.expanded && args.data.hasChildRecords)
                {
                    childCount = proxy._$treegridHelper.ejTreeGrid("getChildCount", args.data, 0);
                    proxy._totalCollapseRecordCount -= childCount;
                    proxy._$ganttchartHelper.ejGanttChart("reRenderGanttGridLinesTable", -1);
                } 
                //Delete from parent collection if item is in parentsCollection
                if (model.parentRecords.indexOf(args.data) !== -1) {
                    model.parentRecords.splice(model.parentRecords.indexOf(args.data), 1);
                }
                //Delete predecessor from child records
                if (args.data.hasChildRecords) {
                    proxy._removeChildRecordsPredecessor(args.data)
                }

                //Delete Predecessor from record
                args.data.predecessor && proxy._removePredecessor(args.data.predecessor, args.data);

                //Update parent item if item has parent
                if (args.data.parentItem)
                {
                    parentRecord=args.data.parentItem;
                    if(args.data.parentItem.childRecords.length>0)
                    {
                        proxy._updateParentItem(args.data);
                    }else
                    {
                        parentRecord.expanded=false;
                        parentRecord.hasChildRecords = false;
                        proxy.refreshGanttRecord(parentRecord);
                    }
                }
                proxy._rows = proxy._$treegridHelper.ejTreeGrid("getFlatRecords");
                proxy._ids = proxy._ids = proxy._$treegridHelper.ejTreeGrid("getUpdatedIds");

                if (proxy.model.predecessorMapping) {
                    proxy._connectorlineIds = [];
                    proxy._connectorLinesCollection = [];
                    proxy._predecessorsCollection = [];
                    proxy._$ganttchartHelper.ejGanttChart("clearConnectorLines");
                    proxy._createConnectorLinesCollection();
                }
                //Scroller is not correctly updated before connector line cleared.
                proxy._$ganttchartHelper.ejGanttChart("refreshHelper", proxy.getCurrentViewData(), proxy.getUpdatedRecords(), proxy._totalCollapseRecordCount);
                proxy._isValidationEnabled = false;


                if (proxy.model.toolbarSettings.showToolbar) {
                    if (proxy.model.toolbarSettings.toolbarItems.indexOf("delete") !== -1) {
                        $('.e-gantttoolbar').ejToolbar('disableItem', $($('.e-deleteitem')[0]).parent()[0]);
                        if (proxy.model.toolbarSettings.toolbarItems.indexOf("indent") !== -1) 
                            $('.e-gantttoolbar').ejToolbar('disableItem', $($('.e-indent')[0]).parent()[0]);
                        if (proxy.model.toolbarSettings.toolbarItems.indexOf("outdent") !== -1) 
                            $('.e-gantttoolbar').ejToolbar('disableItem', $($('.e-outdent')[0]).parent()[0]);
                        if (proxy.model.toolbarSettings.toolbarItems.indexOf("edit") !== -1)
                            $('.e-gantttoolbar').ejToolbar('disableItem', $($('.e-edititem')[0]).parent()[0]);

                    }
                }
                proxy.updateAltRow(proxy.getUpdatedRecords, 0, 1);//for set e-alt-row style after deletion of element
                
            }

            if (args.requestType === "selection") {
                proxy._$ganttchartHelper.ejGanttChart("selectRows", args.recordIndex);
            }

            if (args.requestType === "rowHover") {
                proxy._$ganttchartHelper.ejGanttChart("addRowHover", args.index);
            }
            
            if (args.requestType === "scroll") {
                
                proxy._$ganttchartHelper.ejGanttChart("onScrollHelper", args.delta);
            }

            if (args.requestType === "save") {
                if (args._cAddedRecord) {

                    //For change the id collection after new record is created//
                    proxy._$treegridHelper.ejTreeGrid("renderRecord", args);
                    this.dataSource(proxy._$treegridHelper.ejTreeGrid("getDataSource"));
                    proxy._ids = proxy._$treegridHelper.ejTreeGrid("getUpdatedIds");
                    proxy.model.ganttRecords = proxy._$treegridHelper.ejTreeGrid("getUpdatedRecords");
                    proxy._updatedGanttRecords = proxy._$treegridHelper.ejTreeGrid("getUpdatedRecords");
                    proxy._rows = proxy._$treegridHelper.ejTreeGrid("getFlatRecords");
                    proxy._currentViewData = proxy._$treegridHelper.ejTreeGrid("getCurrentViewData");
                    proxy._$ganttchartHelper.ejGanttChart("setUpdatedRecords", proxy._currentViewData, proxy._updatedGanttRecords, proxy._rows, proxy._ids);
                    proxy._$ganttchartHelper.ejGanttChart("renderRecord", args, proxy._totalCollapseRecordCount);
                    //for soritng
                    if (!args._cAddedRecord.parentItem)
                    {
                        model.parentRecords.splice(args._cAddedRecord.index, 0, args._cAddedRecord);
                    }
              
                    //if (!proxy.model.enableVirtualization)
                    proxy._$ganttchartHelper.ejGanttChart("reRenderGanttGridLinesTable", 1);
                    //}

                    if(args._cAddedRecord.parentItem)
                    {
                        proxy._updateParentItem(args._cAddedRecord);
                    }
                } else if (args._cModifiedData) {
                    args._cModifiedData.left = args._cModifiedData._calculateLeft(proxy._projectStartDate, proxy._perDayWidth, proxy._holidays, model);
                    args._cModifiedData.width = args._cModifiedData._calculateWidth(proxy._perDayWidth, proxy._holidays, model);
                    args._cModifiedData.progressWidth = args._cModifiedData._getProgressWidth(args._cModifiedData.width, args._cModifiedData.status);
                    
                    proxy.refreshGanttRecord(args._cModifiedData);

                    if (args._cModifiedData.parentItem)
                        proxy._updateParentItem(args._cModifiedData);

                    if (args._cModifiedData.predecessor) {
                        if (args.previousValue) {
                            proxy._removeConnectorLine(args.previousValue, args._cModifiedData);
                        }
                    }
                }
                
                if (proxy.model.predecessorMapping) {

                    if ((args._cAddedRecord && args._cAddedRecord.predecessor)
                        || (args._cModifiedData && args._cModifiedData.predecessor)) {
                        proxy._isValidationEnabled = true;
                    }
                    if (args._cAddedRecord) {
                        proxy._$ganttchartHelper.ejGanttChart("clearConnectorLines");
                        proxy._connectorlineIds = [];
                        proxy._connectorLinesCollection = [];
                        proxy._createConnectorLinesCollection();
                        proxy._addConnectorLine(args._cAddedRecord);
                    } else {
                        if (args._cModifiedData) {
                            if (args.previousValue) {
                                proxy._removeConnectorLine(args.previousValue, args._cModifiedData);
                            }
                            proxy._addConnectorLine(args._cModifiedData);
                            if (args._cModifiedData.predecessor) {
                                proxy._isMileStoneEdited = args._cModifiedData.isMilestone;
                                proxy._updatedConnectorLineCollection = [];
                                proxy._connectorlineIds = [];
                                proxy._validatePredecessor(args._cModifiedData);
                                if (proxy._updatedConnectorLineCollection.length > 0) {
                                    proxy._$ganttchartHelper.ejGanttChart("appendConnectorLine", proxy._updatedConnectorLineCollection);
                                }
                            }
                            if (args._cModifiedData.parentItem)
                                proxy._updateParentItem(args._cModifiedData);
                        }
                    }
                }
            }

            proxy._ganttActionCompleteTrigger(args);
        },

        chartactionComplete: function (args) {
            var proxy = this;

            proxy._updatedGanttRecords = proxy.getUpdatedRecords();
            proxy._currentViewData = proxy.getCurrentViewData();

            if (args.requestType === "rowHover") {
                proxy._$treegridHelper.ejTreeGrid('addRowHover', args.index);
            }
            
            if (args.requestType === "scroll") {

                proxy._isRenddered = false;
                if (args.vscrollExsist && args.delta !== undefined)
                    proxy._$treegridHelper.ejTreeGrid('onScrollHelper', args.delta);

                proxy._updatedGanttRecords = proxy.getUpdatedRecords();
                proxy._currentViewData = proxy.getCurrentViewData();

                if (proxy.model.enableVirtualization) {

                    if (proxy.model.predecessorMapping) {

                        if (proxy._connectorLinesCollection.length > 0) {
                            proxy._clearConnectorLines();
                        }
                        proxy._isValidationEnabled = false;
                        proxy._connectorlineIds = [];
                        proxy._connectorLinesCollection = [];
                        proxy._createConnectorLinesCollection();
                    }
                }
                proxy._isRenddered = true;
            }
            
            if (args.requestType === "selection") {
                proxy._$treegridHelper.ejTreeGrid("selectRows", args.recordIndex);
            }
            //for validate the predecessor on mouse hover and mouse up of second drag point
            if (args.requestType === "validatePredecessor") {
                args.data[this.model.taskIdMapping] = args.data.taskID;
                var flag= proxy._editedPredecessorValidation(args);
                args.predecessorValidation = flag;
            }

            //on mouse up if validation is true then draw connector line
            if (args.requestType === "drawConnectorLine") {

                var currentPredecessor = args.toItem._calculatePredecessor(args.predecessor);
                args.toItem.item[proxy.model.predecessorMapping] = args.predecessorString[0];

                if (!args.fromItem.predecessor) {// for parent item
                    args.fromItem['predecessor'] = currentPredecessor;
                }
                else {
                    args.fromItem.predecessor.push(currentPredecessor[0]);
                }
                if (args.toItem.predecessor)
                {
                    args.toItem.predecessor.push(currentPredecessor[0]);
                }
                else
                {
                    args.toItem.predecessor=[];
                    args.toItem.predecessor.push(currentPredecessor[0]);
                }

                proxy._isMileStoneEdited = args.toItem.isMilestone;
                proxy._updatedConnectorLineCollection = [];
                proxy._connectorlineIds = [];
                proxy._validatePredecessor(args.toItem);
                if (proxy._updatedConnectorLineCollection.length > 0) {
                    proxy._$ganttchartHelper.ejGanttChart("appendConnectorLine", proxy._updatedConnectorLineCollection);
                }
            }
            
            proxy._ganttActionCompleteTrigger(args);
        },

        _clearConnectorLines: function () {
            var proxy = this,
                count = 0,
                length = proxy._connectorLinesCollection.length,
                connectorLine,
                childId,
                childGanttRecord,
                predecessors,
                predecessorslength,
                i = 0,
                predecessor;
            
            for (count; count < length; count++) {

                connectorLine = proxy._connectorLinesCollection[count];
                childId = connectorLine && connectorLine.ConnectorLineId.split("child")[1];
                childGanttRecord = proxy._rows[proxy._ids.indexOf(childId)];
                predecessors = childGanttRecord.predecessor;
                predecessorslength =predecessors&& predecessors.length;
                
                for (i = 0; i < predecessorslength; i++) {

                    predecessor = predecessors[i];

                    if (predecessor.to === childId && predecessor.isdrawn) {
                        predecessor.isdrawn = false;
                    }
                }
            }
            
            proxy._$ganttchartHelper.ejGanttChart("clearConnectorLines");
        },
        

        taskbarEditing: function (args) {
            
            this._trigger("taskbarEditing", args);
        },
        
        _refreshParent: function (item) {
            this.refreshGanttRecord(item);
            if (item.parentItem) {
                this._refreshParent(item.parentItem);
            }
        },

        taskbarEdited: function (args) {
            var proxy = this,
                ganttRecord = args.data, model = proxy.model,
                projectStartDate = new Date(proxy._projectStartDate),
                remainDays,
                day,
                offset = 0;
            
            proxy._isValidationEnabled = false;
            
            if (args.dragging) {
                if (args.previousData.left !== ganttRecord.left) {
                    remainDays = ganttRecord.left % proxy._perDayWidth,
                           day = (ganttRecord.left - remainDays) / proxy._perDayWidth,
                           offset = 0;
                    
                    if (remainDays > 0 && (model.workingTimeScale != ej.Gantt.workingTimeScale.TimeScale24Hours)) {
                        offset = remainDays >= proxy._perDayWidth / 2 ? 1 : 0;
                    }
                    var remainingHours = Math.round(remainDays / (proxy._perDayWidth / 24));//Hour calculation

                    projectStartDate.setDate(projectStartDate.getDate() + day + offset);
                    if (model.scheduleHeaderSettings.scheduleHeaderType == ej.Gantt.ScheduleHeaderType.Day) {
                        projectStartDate.setHours(remainingHours);
                    }
                    else if (model.dateFormat.indexOf("hh:") &&
                     model.workingTimeScale == ej.Gantt.workingTimeScale.TimeScale8Hours) {
                        var remainingHours = Math.round(remainDays / (proxy._perDayWidth / 9));
                        projectStartDate.setHours(8);
                    }
                      
                    if ((model.workingTimeScale != ej.Gantt.workingTimeScale.TimeScale24Hours)) {
                        if (remainDays >= (proxy._perDayWidth / 2)) {
                            ganttRecord.left = ganttRecord.left + proxy._perDayWidth;//- remainDays;
                            if (model.roundOffDayworkingTime)
                                ganttRecord.left -= remainDays;
                        } else {
                            ganttRecord.left = ganttRecord.left;// - remainDays;
                            if (model.roundOffDayworkingTime)
                                ganttRecord.left -= remainDays;
                        }
                    }
                    
                    if (this._stringHolidays &&
                     this._stringHolidays.indexOf(ganttRecord._getFormatedDate(projectStartDate, model.dateFormat)) != -1) {
                        projectStartDate.setDate(projectStartDate.getDate() + 1);
                        ganttRecord.left += proxy._perDayWidth;
                    }
                    
                    if (this.model.includeWeekend == false) {
                        if (projectStartDate.getDay() === 0) {
                            projectStartDate.setDate(projectStartDate.getDate() + 1);
                            ganttRecord.left += proxy._perDayWidth;
                        } else if (projectStartDate.getDay() === 6) {
                            projectStartDate.setDate(projectStartDate.getDate() + 2);
                            ganttRecord.left += (2 * proxy._perDayWidth);
                        }
                    }

                    if (this._stringHolidays &&
                    this._stringHolidays.indexOf(ganttRecord._getFormatedDate(projectStartDate, model.dateFormat)) != -1) {
                        projectStartDate.setDate(projectStartDate.getDate() + 1);
                        ganttRecord.left += proxy._perDayWidth;
                    }
                    
                    ganttRecord.startDate = ganttRecord._getFormatedDate(projectStartDate, model.dateFormat);//_calculateFormatedDate
                    ganttRecord._calculateEndDate(model.includeWeekend, this._holidays, model, proxy._perDayWidth);
                    ganttRecord.width = ganttRecord._calculateTaskbarWidth(proxy._perDayWidth,model);
                    ganttRecord.progressWidth = ganttRecord._getProgressWidth(ganttRecord.width, ganttRecord.status);
                    proxy._isValidationEnabled = true;
                }

            } else if (args.rightResizing) {
                
                if (args.previousData.width !== ganttRecord.width) {
                    day = ganttRecord.width / proxy._perDayWidth;
                    var rem = ganttRecord.width % proxy._perDayWidth;
                    offset = 0;
                    if (rem > 0 && (model.workingTimeScale != "TimeScale24Hours")) {
                        offset = rem >= (proxy._perDayWidth / 2) ? 1 : 0;
                    }

                    var totalLength = ganttRecord.left + ganttRecord.width;
                    var remainingLength = totalLength % proxy._perDayWidth;

                    var remainingHours = Math.round(remainingLength / (proxy._perDayWidth / 24));//hour calculation

                    var startDate = proxy._getDateFromFormat(ganttRecord.startDate);

                    if ((model.scheduleHeaderSettings.scheduleHeaderType == ej.Gantt.ScheduleHeaderType.Day &&
                        model.durationUnit == ej.Gantt.DurationUnit.Hour)) {//model.dateFormat.indexOf("hh") == -1
                        var startDateHour = (ganttRecord.left % proxy._perDayWidth) / proxy._perHourWidth;
                        var hours = Math.round(totalLength / (proxy._perHourWidth));
                        hours -= startDateHour;

                        var endDate = ganttRecord._dateUpdate(startDate, 0, hours);
                        startDate.setHours(startDateHour);
                    }

                    else if ((model.scheduleHeaderSettings.scheduleHeaderType == ej.Gantt.ScheduleHeaderType.Week
                         && model.dateFormat.indexOf("hh") != -1)) {
                        if (model.durationUnit == ej.Gantt.DurationUnit.Hour) {
                            var startDateHour = Math.round((ganttRecord.left % proxy._perDayWidth) / (proxy._perDayWidth / 24));
                            var remainingHourDiff = Math.round(totalLength / (proxy._perDayWidth / 24));
                            remainingHourDiff -= startDateHour;
                            endDate = ganttRecord._dateUpdate(startDate, 0, remainingHourDiff);
                        }
                    }

                    else
                        var endDate = ganttRecord._dateUpdate(startDate, day - 1 + offset, 0);

                    if (model.scheduleHeaderSettings.scheduleHeaderType == ej.Gantt.ScheduleHeaderType.Day ||
                        (model.scheduleHeaderSettings.scheduleHeaderType == ej.Gantt.ScheduleHeaderType.Week && model.dateFormat.indexOf("hh") != -1))
                        endDate.setHours(Math.round(remainingHours));
                    else if (model.dateFormat.indexOf("hh:") &&
                       model.workingTimeScale == ej.Gantt.workingTimeScale.TimeScale8Hours) {
                        endDate.setHours(Math.round(17));
                    }

                    if (model.holidays && model.holidays.length) {
                        var holidayIndex = 0,
                            holidayslength = model.holidays.length,
                            holiday;
                        for (holidayIndex = 0; holidayIndex < holidayslength; holidayIndex++) {
                            holiday = new Date(this._holidays[holidayIndex]);
                            if (endDate === holiday) {
                                endDate.setDate(endDate.getDate() - 1);
                            }
                        }
                    }

                    if (this.model.includeWeekend == false) {
                        if (endDate.getDay() == 0) {
                            endDate.setDate(endDate.getDate() - 2);
                        } else if (endDate.getDay() == 6) {
                            endDate.setDate(endDate.getDate() - 1);
                        }
                    }
                    
                    if (this._stringHolidays &&
                   this._stringHolidays.indexOf(ganttRecord._getFormatedDate(endDate, model.dateFormat)) != -1) {
                        endDate.setDate(endDate.getDate() + 1);
                        if (endDate.getDay() == 6) {
                            endDate.setDate(endDate.getDate() + 2);
                        }
                    }
                    
                    ganttRecord.endDate = ganttRecord._calculateFormatedDate(endDate, model.dateFormat);
                    ganttRecord._calculateDuration(startDate, endDate, model.includeWeekend, model.holidays, ganttRecord.isMilestone,model);
                    ganttRecord.width = ganttRecord._calculateTaskbarWidth(proxy._perDayWidth,model);
                    ganttRecord.progressWidth = ganttRecord._getProgressWidth(ganttRecord.width, ganttRecord.status);
                    proxy._isValidationEnabled = true;
                }

            } else if (args.leftResizing) {
                if (args.previousData.left !== ganttRecord.left) {
                    remainDays = ganttRecord.left % proxy._perDayWidth;
                    day = (ganttRecord.left - remainDays) / proxy._perDayWidth;
                    offset = 0;
                    if (remainDays > 0 && (model.workingTimeScale != "TimeScale24Hours")) {
                        offset = remainDays >= proxy._perDayWidth / 2 ? 1 : 0;
                    }
                    var remainingHours = Math.round(remainDays / (proxy._perDayWidth / 24));//Hour calculation

                    projectStartDate.setDate(projectStartDate.getDate() + day + offset);

                    if (model.scheduleHeaderSettings.scheduleHeaderType == ej.Gantt.ScheduleHeaderType.Day ||
                        (model.scheduleHeaderSettings.scheduleHeaderType == ej.Gantt.ScheduleHeaderType.Week && model.dateFormat.indexOf("hh") != -1)) {
                        if (model.workingTimeScale == ej.Gantt.workingTimeScale.TimeScale8Hours &&
                            (remainingHours >= 17||model.durationUnit==ej.Gantt.DurationUnit.Day))
                            projectStartDate.setHours(8);
                        else
                            projectStartDate.setHours(remainingHours);
                    }
                    else if (model.dateFormat.indexOf("hh:") &&
                       model.workingTimeScale == ej.Gantt.workingTimeScale.TimeScale8Hours) {
                        var remainingHours = Math.round(remainDays / (proxy._perDayWidth / 9));
                        projectStartDate.setHours(8);
                    }

                    if (this._stringHolidays &&
                    this._stringHolidays.indexOf(ganttRecord._getFormatedDate(projectStartDate, model.dateFormat)) != -1) {
                        projectStartDate.setDate(projectStartDate.getDate() + 1);
                        ganttRecord.left += proxy._perDayWidth;
                    }
                    
                    if (this.model.includeWeekend == false) {
                        if (projectStartDate.getDay() === 0) {
                            projectStartDate.setDate(projectStartDate.getDate() + 1);
                            ganttRecord.left += proxy._perDayWidth;
                        } else if (projectStartDate.getDay() === 6) {
                            projectStartDate.setDate(projectStartDate.getDate() + 2);
                            ganttRecord.left += (2 * proxy._perDayWidth);
                        }
                    }
                    
                    if (this._stringHolidays &&
                    this._stringHolidays.indexOf(ganttRecord._getFormatedDate(projectStartDate, model.dateFormat)) != -1) {
                        projectStartDate.setDate(projectStartDate.getDate() + 1);
                        ganttRecord.left += proxy._perDayWidth;
                    }

                    
                    if (model.workingTimeScale != ej.Gantt.workingTimeScale.TimeScale24Hours &&
                        model.scheduleHeaderSettings.scheduleHeaderType == ej.Gantt.ScheduleHeaderType.Day &&
                        model.durationUnit != ej.Gantt.DurationUnit.Hour) {

                        if (remainDays >= (proxy._perDayWidth / 2)) {
                            ganttRecord.left = ganttRecord.left + proxy._perDayWidth;// - remainDays;
                            if (model.roundOffDayworkingTime)
                                ganttRecord.left -= remainDays;
                        } else {
                            ganttRecord.left = ganttRecord.left;//- (remainingHours * 20);
                            if (model.roundOffDayworkingTime)
                                ganttRecord.left -= remainDays;
                        }
                    }
                    else if (model.scheduleHeaderSettings.scheduleHeaderType != ej.Gantt.ScheduleHeaderType.Day &&
                        model.dateFormat.indexOf("hh") == -1) {
                        if (remainDays >= (proxy._perDayWidth / 2)) {
                            ganttRecord.left = ganttRecord.left + proxy._perDayWidth;// - remainDays;
                            if (model.roundOffDayworkingTime)
                                ganttRecord.left -= remainDays;
                        } else {
                            ganttRecord.left = ganttRecord.left;//- (remainingHours * 20);
                            if (model.roundOffDayworkingTime)
                                ganttRecord.left -= remainDays;
                        }
                    }
                      

                    else if (model.scheduleHeaderSettings.scheduleHeaderType == ej.Gantt.ScheduleHeaderType.Day
                        && (model.workingTimeScale == ej.Gantt.workingTimeScale.TimeScale8Hours ||
                        model.workingTimeScale == ej.Gantt.workingTimeScale.TimeScale24Hours || model.durationUnit == ej.Gantt.DurationUnit.Hour)) {
                        var remainingHourDiff = ganttRecord.left % proxy._perHourWidth;

                        if (remainingHourDiff >= (proxy._perHourWidth / 2)) {
                            ganttRecord.left = ganttRecord.left + proxy._perHourWidth;// - remainDays;

                            ganttRecord.left -= remainingHourDiff;
                        } else {
                            ganttRecord.left -= remainingHourDiff;
                        }
                    }

                    ganttRecord.startDate = ganttRecord._calculateFormatedDate(projectStartDate, model.dateFormat);
                    ganttRecord._calculateDuration(projectStartDate, proxy._getDateFromFormat(ganttRecord.endDate), model.includeWeekend, model.holidays, ganttRecord.isMilestone,model);
                    ganttRecord.width = ganttRecord._calculateTaskbarWidth(proxy._perDayWidth, model);
                    ganttRecord.progressWidth = ganttRecord._getProgressWidth(ganttRecord.width, ganttRecord.status);
                    proxy._isValidationEnabled = true;
                }
            } else if (args.progressResizing) {
                ganttRecord.status = ganttRecord._getProgressPercent(ganttRecord.width, ganttRecord.progressWidth);
                if (ganttRecord.parentItem) {
                    ganttRecord._updateParentProgress(ganttRecord.parentItem, model.progressMapping);
                }
                proxy._refreshParent(ganttRecord);
                //proxy.refreshGanttRecord(ganttRecord);
                //proxy.refreshGanttRecord(ganttRecord.parentItem);
            }
            if (proxy._isValidationEnabled) {
                proxy._updateEditedGanttRecord(ganttRecord);
                if (ganttRecord.predecessor) {
                    proxy._isMileStoneEdited = ganttRecord.isMilestone;
                    proxy._updatedConnectorLineCollection = [];
                    proxy._connectorlineIds = [];
                    proxy._editedTaskBarItem = ganttRecord;
                    proxy._validatePredecessor(ganttRecord);
                    //proxy._$ganttchartHelper.ejGanttChart("clearConnectorLines");
                    //proxy._createConnectorLinesCollection();

                    if (proxy._updatedConnectorLineCollection.length > 0) {
                        proxy._$ganttchartHelper.ejGanttChart("appendConnectorLine", proxy._updatedConnectorLineCollection);
                    }
                }
                proxy._editedTaskBarItem = null;
                if (ganttRecord.parentItem)
                    proxy._updateParentItem(ganttRecord);
                proxy.refreshGanttRecord(ganttRecord);
            }
            proxy._trigger("taskbarEdited", args);
        },

        actionBegin: function (args) {

            var proxy = this;

            if (!proxy._trigger("actionBegin", args)) {

                if (args.requestType === "beginedit") {
                    proxy._sendEditRequest();
                }   
                
            }

        },

        _ganttActionCompleteTrigger: function (args) {
            this._trigger("actionComplete", args);
        },

        _queryCellInfo: function (args) {
            this._trigger("queryCellInfo", args);
        },

        _beginEdit: function (args) {
            var proxy = this;
            proxy._trigger("beginEdit", args);
            proxy._isinBeginEdit = true;
            if (proxy.model.toolbarSettings.showToolbar) {

                var toolBarEnabledItems = [];

                var cancelElement = $($('.e-cancel')[0]).parent()[0];
                var saeveElement = $($('.e-saveitem')[0]).parent()[0];

                if (cancelElement) {
                    toolBarEnabledItems.push(cancelElement);
                }

                if (saeveElement) {
                    toolBarEnabledItems.push(saeveElement);
                }               
                
                $('.e-gantttoolbar').ejToolbar('enableItem', toolBarEnabledItems);
            }
        },

        _endEdit: function (args) {
            var proxy = this;
            
            proxy._isinBeginEdit = false;

            if (proxy._trigger("endEdit", args)) {
                var predecessorString = "";
                args.data[args.columnName] = args.previousValue;
                if (args.columnName === "predecessor") {
                    predecessorString = proxy._predecessorToString(args.data.predecessor, args.data);
                    args.data.item[args.columnObject.mappingName] = predecessorString;
                } else {
                    args.data.item[args.columnObject.mappingName] = args.previousValue;
                }
            }
            else {

                if (args.isModified) {
                    proxy._currentViewData = proxy._$treegridHelper.ejTreeGrid("getCurrentViewData");
                    proxy._isValidationEnabled = true;
                    proxy._updateEditedGanttRecords(args);
                }
            }
            proxy.refreshGanttRecord(args.data);
            proxy._$treegridHelper.ejTreeGrid("selectRows", this.selectedRowIndex());
            proxy._$ganttchartHelper.ejGanttChart("selectRows", this.selectedRowIndex());
            proxy._$ganttchartHelper.ejGanttChart("updateSelectedItem", this.selectedRowIndex());

            delete args.isModified;
            if (proxy.model.toolbarSettings.showToolbar) {
                var toolBarDisableItems = [];

                if (proxy.model.toolbarSettings.toolbarItems.indexOf("cancel") !== -1) {
                    toolBarDisableItems.push($($('.e-cancel')[0]).parent()[0]);
                }

                if (proxy.model.toolbarSettings.toolbarItems.indexOf("delete") !== -1) {
                    toolBarDisableItems.push($($('.e-saveitem')[0]).parent()[0]);
                }
                
                $('.e-gantttoolbar').ejToolbar('disableItem', toolBarDisableItems);
            }
            // proxy._trigger("endEdit", args);
        },

        

        _rowDataBound: function (args) {
            this._trigger("rowDataBound", args);
        },
        _refreshToolBar: function () {
            var $toolbar = $("#" + this._id + "_toolbarItems");
            $toolbar.find("li").removeClass("e-hover");
        },

        _toolBarClick: function (sender) {
            var proxy = this,
                model=proxy.model,
                $gridEle = $(this.itemsContainer).closest(".e-gantt"),
                gridInstance = $gridEle.ejGantt("instance"),
                gridId = $gridEle.attr('id');

            gridInstance._clearContextMenu();
            if (sender.event == undefined && sender.target.tagName == "INPUT" && sender.currentTarget.id == gridId + "_search")
                return false;

            if (gridInstance.model.dateFormat.indexOf("hh") != -1)
                $.isFunction($.fn.ejDateTimePicker) && $("#" + gridId + "EditForm").find(".e-datepicker").ejDateTimePicker("hide");
            else
                $.isFunction($.fn.ejDatePicker) && $("#" + gridId + "EditForm").find(".e-datepicker").ejDatePicker("hide");


            if ($gridEle.ejGantt("instance")._trigger("toolBarClick", sender))
                return false;
            
            switch (sender.currentTarget.id) {

                case gridId + "_add":
                    gridInstance._toolbarOperation(gridId + "_add");
                    break;

                case gridId + "_edit":
                    gridInstance._toolbarOperation(gridId + "_edit");
                    break;

                case gridId + "_delete":
                    gridInstance._toolbarOperation(gridId + "_delete");
                    break;

                case gridId + "_update":
                    gridInstance._toolbarOperation(gridId + "_update");
                    break;

                case gridId + "_cancel":
                    gridInstance._toolbarOperation(gridId + "_cancel");
                    break;

                case gridId + "_search":
                    gridInstance._toolbarOperation(gridId + "_search", $(sender.currentTarget).find("input").val());
                    break;

                case gridId + "_indent":
                    gridInstance._toolbarOperation(gridId + "_indent");
                    break;

                case gridId + "_outdent":
                    gridInstance._toolbarOperation(gridId + "_outdent");
                    break;
                    
                case gridId + "_expandAll":
                    gridInstance._toolbarOperation(gridId + "_expandAll");
                    break;
                 
                case gridId + "_collapseAll":
                    gridInstance._toolbarOperation(gridId + "_collapseAll");
                    break;
            }
            return false;
        },

        _toolbarOperation: function (operation, searchEle) {
            var $gridEle = this.element, gridObject = $gridEle.ejGantt("instance"), gridId = $gridEle.attr('id'), proxy = this;
            switch (operation) {

                case gridId + "_add":
                    proxy._sendAddRequest();
                    break;

                case gridId + "_edit":
                    proxy._sendEditRequest();
                    break;

                case gridId + "_delete":
                    proxy._$treegridHelper.ejTreeGrid("deleteRow");
                    break;

                case gridId + "_update":
                    proxy._$treegridHelper.ejTreeGrid("endEdit");
                    //gridObject.sendSaveRequest();

                    if (proxy.model.toolbarSettings.toolbarItems.indexOf("cancel") !== -1) {
                        proxy._disabledToolItems.push($($('.e-cancel')[0]).parent()[0]);
                    }

                    if (proxy.model.toolbarSettings.toolbarItems.indexOf("save") !== -1) {
                        proxy._disabledToolItems.push($($('.e-saveitem')[0]).parent()[0]);
                    }
                    $('.e-gantttoolbar').ejToolbar('disableItem', proxy._disabledToolItems);
                    
                    proxy._disabledToolItems = [];
                    
                    break;

                case gridId + "_cancel":
                    proxy._$treegridHelper.ejTreeGrid("cancelEditCell");

                    if (proxy.model.toolbarSettings.toolbarItems.indexOf("cancel") !== -1) {
                        proxy._disabledToolItems.push($($('.e-cancel')[0]).parent()[0]);
                    }

                    if (proxy.model.toolbarSettings.toolbarItems.indexOf("save") !== -1) {
                        proxy._disabledToolItems.push($($('.e-saveitem')[0]).parent()[0]);
                    }
                    
                    $('.e-gantttoolbar').ejToolbar('disableItem', proxy._disabledToolItems);
                    proxy._disabledToolItems = [];
                    break;

                case gridId + "_search":
                    proxy._$treegridHelper.ejTreeGrid("search", searchEle);
                    //proxy.model.enableContextMenu = searchEle.length == 0 ? true : false;
                    proxy._searchString = searchEle;

                    proxy._updateToolbarOptions(searchEle);
                    
                    break;

                case gridId + "_indent":
                    proxy._sendIndentRequest();
                    break;

                case gridId + "_outdent":
                    proxy._sendOutdentRequest();
                    break;

                case gridId + "_expandAll":
                    proxy._expandAll();
                    break;

                case gridId + "_collapseAll":
                    proxy._collapseAll();                    
                    break;
            }
            proxy._refreshToolBar();
            return false;
        },
        
        _setModel: function (options) {
            
            var proxy = this, option,
                model = proxy.model;
            
            for (option in options) {
                switch (option) {
                    case "sortSettings":
                        proxy._sendSortingRequest(options[option]);
                        break;
                    case "destroy":
                        proxy.destroy();
                        break;
                    case "allowGanttChartEditing":
                        proxy._ganttChartEditing(options[option]);
                        break;
                    case "enableProgressBarResizing":
                        proxy._resizeProgressbar(options[option]);
                        break;
                    case "enableTaskbarTooltip":
                        proxy._showTooltip(options[option]);
                        break;
                    case "enableTaskbarDragTooltip":
                        proxy._showEditingTooltip(options[option]);
                        break;
                    case "enableAltRow":
                        proxy._updateAltRow(options[option]);
                        break;
                    case "locale":
                        model.ganttRecords = [];
                        model.parentRecords = [];
                        proxy._clearContextMenu();
                        model.locale = options[option];
                        proxy.element.ejGantt("destroy").ejGantt(model);
                        break;
                    case "allowSelection":
                        proxy._updateRowSelection(options[option]);
                        break;
                    case "editSettings":
                        $.extend(model.editSettings, options[option]);
                        proxy._updateEditMode(options[option]);
                        break;
                    case "taskbarBackground":
                        proxy._updateTaskbarBackground(options[option]);
                        break;
                    case "progressbarBackground":
                        proxy._updateProgressbarBackground(options[option]);
                        break;
                    case "parentTaskbarBackground":
                        proxy._updateParentTaskbarBackground(options[option]);
                        break;
                    case "parentProgressbarBackground":
                        proxy._updateParentProgressbarBackground(options[option]);
                        break;
                    case "showTaskNames":
                        proxy._updateRenderTaskNames(options[option]);
                        break;
                    case "showProgressStatus":
                        proxy._updateRendeProgressStatus(options[option]);
                        break;
                    case "showResourceNames":
                        proxy._updateRenderResourceNames(options[option]);
                        break;
                    case "progressbarHeight":
                        proxy._updateProgressbarHeight(options[option]);
                        break;
                    case "baselineColor":
                        proxy._updateBaselineColor(options[option]);
                        break;
                    case "renderBaseline":
                        proxy._updateRenderBaseline(options[option]);
                        break;
                    case "allowColumnResize":
                        proxy._updateAllowColumnResize(options[option]);
                        break;
                    case "allowKeyboardNavigation":
                        proxy._updateAllowKeyboardNavigation(options[option]);
                        break;
                    case "selectedItem":
                        proxy._updateSelectedItem(options[option]);
                        break;
                    case "dataSource":
                        model.dataSource = [];
                        model.ganttRecords = [];
                        model.parentRecords = [];
                        model.dataSource = (options[option]);
                        proxy.element.ejGantt("destroy").ejGantt(model);
                        break;
                    case "showGridCellTooltip":
                        proxy._updateShowGridCellTooltip(options[option]);
                        break;
                    case "showGridExpandCellTooltip":
                        proxy._updateShowGridExpandCellTooltip(options[option]);
                        break;
                    case "highlightWeekends":
                        proxy._updateHighlightWeekends(options[option]);
                        break;
                    case "connectorLineBackground":
                        proxy._updateConnectorLineBackground(options[option]);
                        break;
                    case "connectorlineWidth":
                        proxy._updateConnectorlineWidth(options[option]);
                        break;
                    case "weekendBackground":
                        proxy._updateWeekendBackground(options[option]);
                        break;
                    case "enableContextMenu":
                        proxy.model.enableContextMenu = options[option];
                        break;
                    case "enableResize":
                        debugger;
                        if (model.enableResize) {
                            proxy.model.enableResize = options[option];
                            proxy._on($(window), "resize", proxy.windowResize);
                        }
                        else {
                            proxy._off($(window), "resize", proxy.windowResize);
                        }
                        break;
                    case "selectedRowIndex":
                        if (this.selectedRowIndex() !== -1 && !ej.isNullOrUndefined(this.selectedRowIndex())) {
                            var args = {};
                            args.recordIndex = this.selectedRowIndex();
                            args.data = proxy.model.ganttRecords[args.recordIndex];
                            args.target = "ejTreeGrid";
                            proxy.rowSelected(args);
                            proxy._$treegridHelper.ejTreeGrid("selectRows", args.recordIndex);
                            proxy._$ganttchartHelper.ejGanttChart("selectRows", args.recordIndex);
                            proxy._$ganttchartHelper.ejGanttChart("updateSelectedItem", args.recordIndex);
                        }
                        else if (this.selectedRowIndex() === -1) {
                            proxy._$treegridHelper.ejTreeGrid("clearSelection");
                        }
                        break;
                }
            }
        },

        _updateWeekendBackground: function (bgcolor) {
            var proxy = this;
            this.model.weekendBackground = bgcolor;
            proxy._$ganttchartHelper.ejGanttChart("updateWeekendBackground", bgcolor);

        },

        _updateConnectorlineWidth: function (lineWidth) {
            var proxy = this;
            this.model.connectorlineWidth = lineWidth;
            proxy._$ganttchartHelper.ejGanttChart("updateConnectorlineWidth", lineWidth);
            proxy._$ganttchartHelper.ejGanttChart("clearConnectorLines");
            proxy._$ganttchartHelper.ejGanttChart("renderConnectorLines",
                proxy._connectorLinesCollection);

        },

        _updateConnectorLineBackground: function (bgcolor) {
            var proxy = this;
            this.model.connectorLineBackground = bgcolor;
            proxy._$ganttchartHelper.ejGanttChart("updateConnectorLineBackground", bgcolor);
            proxy._$ganttchartHelper.ejGanttChart("clearConnectorLines");
            proxy._$ganttchartHelper.ejGanttChart("renderConnectorLines",
                proxy._connectorLinesCollection);

        },

        _updateHighlightWeekends: function (bool) {
            var proxy = this;
            this.model.highlightWeekends = bool;
            proxy._$ganttchartHelper.ejGanttChart("updateHighlightWeekends", bool);
        },

        _updateShowGridCellTooltip: function (bool) {
            var proxy = this;
            proxy._$treegridHelper.ejTreeGrid("updateShowGridCellTooltip", bool);
        },

        _updateShowGridExpandCellTooltip: function (bool) {
            var proxy = this;
            proxy._$treegridHelper.ejTreeGrid("updateShowGridExpandCellTooltip", bool);
        },

        _updateSelectedItem: function (item) {
            var proxy = this,
                args = {};
            if (item[proxy.model.taskIdMapping] != null) {


                args.recordIndex =proxy._ids.indexOf((item[proxy.model.taskIdMapping]).toString());
                args.item = proxy.model.ganttRecords[args.recordIndex];
                args.target = "ejTreeGrid";
                proxy.rowSelected(args);
                proxy._$treegridHelper.ejTreeGrid("selectRows", args.recordIndex);
                proxy._$ganttchartHelper.ejGanttChart("selectRows", args.recordIndex);
                proxy._$ganttchartHelper.ejGanttChart("updateSelectedItem", args.recordIndex);
            }
        },

        _updateAllowKeyboardNavigation: function (bool) {
            var proxy = this;
            proxy._$treegridHelper.ejTreeGrid("updateAllowKeyboardNavigation", bool);
            proxy._$ganttchartHelper.ejGanttChart("updateAllowKeyboardNavigation", height);
        },
        
        _updateAllowColumnResize: function (bool) {
            var proxy = this;
            proxy._$treegridHelper.ejTreeGrid("updateAllowColumnResize", bool);
        },
        
        _updateProgressbarHeight: function (height) {
            var proxy = this;
            this.model.progressbarHeight = height;
            proxy._$ganttchartHelper.ejGanttChart("updateProgressbarHeight", height);
        },
        
        _updateBaselineColor: function (color) {
            var proxy = this;
            this.model.baselineColor = color;
            proxy._$ganttchartHelper.ejGanttChart("updateBaselineColor", color);
        },
        
        _updateRenderBaseline: function (bool) {
            var proxy = this;
            this.model.renderBaseline = bool;
            proxy._$ganttchartHelper.ejGanttChart("updateRenderBaseline", bool);
            proxy._$ganttchartHelper.ejGanttChart("renderConnectorLines",
                proxy._connectorLinesCollection);
        },
        
        _updateRenderTaskNames: function (bool) {
            var proxy = this;
            this.model.showTaskNames = bool;
            proxy._$ganttchartHelper.ejGanttChart("updateRenderTaskNames", bool);
        },
        
        _updateRendeProgressStatus: function (bool) {
            var proxy = this;
            this.model.showProgressStatus = bool;
            proxy._$ganttchartHelper.ejGanttChart("updateRendeProgressStatus", bool);
        },
        
        _updateRenderResourceNames: function (bool) {
            var proxy = this;
            this.model.showResourceNames = bool;
            proxy._$ganttchartHelper.ejGanttChart("updateRenderResourceNames", bool);
        },
        
        _updateTaskbarBackground: function (bgcolor) {
            var proxy = this;
            this.model.taskbarBackground = bgcolor;
            proxy._$ganttchartHelper.ejGanttChart("updateTaskbarBackground", bgcolor);
        },

        _updateProgressbarBackground: function (bgcolor) {
            var proxy = this;
            this.model.progressbarBackground = bgcolor;
            proxy._$ganttchartHelper.ejGanttChart("updateProgressbarBackground", bgcolor);
        },

        _updateParentTaskbarBackground: function (bgcolor) {
            var proxy = this;
            this.model.parentTaskbarBackground = bgcolor;
            proxy._$ganttchartHelper.ejGanttChart("updateParentTaskbarBackground", bgcolor);
        },

        _updateParentProgressbarBackground: function (bgcolor) {
            var proxy = this;
            this.model.parentProgressbarBackground = bgcolor;
            proxy._$ganttchartHelper.ejGanttChart("updateParentProgressbarBackground", bgcolor);
        },
        
        _updateEditMode: function (edit) {
            var proxy = this;
            proxy._$treegridHelper.ejTreeGrid("updateEditMode", edit);
            
        },
        
        _updateRowSelection: function (bool) {
            var proxy = this;
            proxy._$treegridHelper.ejTreeGrid("updateRowSelection", bool);
        },

        _updateAltRow: function (bool) {
            var proxy = this;
            proxy._$treegridHelper.ejTreeGrid("updateAltRow", bool);
        },
        
        /**
       * To Open the dialog to add new task to the gantt
       * @return jQuery
       * @example 
       * &lt;gantt id="gantt"&gt;Gantt&lt;/gantt&gt; <br> 
       * &lt;script&gt;
       * // Create Gantt
       * var ganttObj = $("#gantt").data("ejGantt");
       * gantObj.ejGantt("openAddDialog"); // To open the add dialog
       * &lt;/script&gt;
       * @example 
       *  &lt;gantt id="gantt"&gt;Gantt&lt;/gantt&gt; <br> 
       * &lt;script&gt;
       * // open Add dialog
       * $("#gantt").ejGantt("openAddDialog");	
       * &lt;/script&gt;
       * @memberof ejGantt
       * @instance
        */
        openAddDialog: function() {
            this._sendAddRequest();
        },

        /**
        * To Open the dialog to edit existing task to the gantt
        * @return jQuery
        * @example 
        * &lt;gantt id="gantt"&gt;Gantt&lt;/gantt&gt; <br> 
        * &lt;script&gt;
        * // Create Gantt
        * var ganttObj = $("#gantt").data("ejGantt");
        * gantObj.ejGantt("openEditDialog"); // To open the add dialog
        * &lt;/script&gt;
        * @example 
        *  &lt;gantt id="gantt"&gt;Gantt&lt;/gantt&gt; <br> 
        * &lt;script&gt;
        * // open Add dialog
        * $("#gantt").ejGantt("openEditDialog");	
        * &lt;/script&gt;
        * @memberof ejGantt
        * @instance
         */
        openEditDialog:function() {
            this._sendEditRequest();
        },
        
        /**
        * To search an item with search string provided at the run time
        * @return jQuery
        * @example 
        * &lt;gantt id="gantt"&gt;Gantt&lt;/gantt&gt; <br> 
        * &lt;script&gt;
        * // Create Gantt
        * var ganttObj = $("#gantt").data("ejGantt");
        * gantObj.ejGantt("searchItem",$("#text").val()); // To search a task
        * &lt;/script&gt;
        * @example 
        *  &lt;gantt id="gantt"&gt;Gantt&lt;/gantt&gt; <br> 
        * &lt;script&gt;
        * // To search a task
        * $("#gantt").ejGantt("searchItem",$("#text").val());	
        * &lt;/script&gt;
        * @memberof ejGantt
        * @instance
         */
        searchItem: function (searchString) {
            var proxy = this;
            proxy._$treegridHelper.ejTreeGrid("search", searchString);
        },
        

        /**
        * To delete a selected item in gantt
        * @return jQuery
        * @example 
        * &lt;gantt id="gantt"&gt;Gantt&lt;/gantt&gt; <br> 
        * &lt;script&gt;
        * // Create Gantt
        * var ganttObj = $("#gantt").data("ejGantt");
        * gantObj.ejGantt("deleteItem"); // To delete a task
        * &lt;/script&gt;
        * @example 
        *  &lt;gantt id="gantt"&gt;Gantt&lt;/gantt&gt; <br> 
        * &lt;script&gt;
        * // To delete an item
        * $("#gantt").ejGantt("deleteItem");	
        * &lt;/script&gt;
        * @memberof ejGantt
        * @instance
         */
        deleteItem: function () {
            var proxy = this;
            proxy._$treegridHelper.ejTreeGrid("deleteRow");
        },


        /**
          * To Expand all the parent items in gantt
          * @return jQuery
          * @example 
          * &lt;gantt id="gantt"&gt;Gantt&lt;/gantt&gt; <br> 
          * &lt;script&gt;
          * // Create Gantt
          * var ganttObj = $("#gantt").data("ejGantt");
          * gantObj.ejGantt("expandAllItems"); // To expand all parent items in gantt
          * &lt;/script&gt;
          * @example 
          *  &lt;gantt id="gantt"&gt;Gantt&lt;/gantt&gt; <br> 
          * &lt;script&gt;
          * // To expand all items
          * $("#gantt").ejGantt("expandAllItems");	
          * &lt;/script&gt;
          * @memberof ejGantt
          * @instance
           */

        expandAllItems:function() {
            this._expandAll();
        },

        /**
          * To collapse all the parent items in gantt
          * @return jQuery
          * @example 
          * &lt;gantt id="gantt"&gt;Gantt&lt;/gantt&gt; <br> 
          * &lt;script&gt;
          * // Create Gantt
          * var ganttObj = $("#gantt").data("ejGantt");
          * gantObj.ejGantt("collapseAllItems"); // To collapse all parent items in gantt
          * &lt;/script&gt;
          * @example 
          *  &lt;gantt id="gantt"&gt;Gantt&lt;/gantt&gt; <br> 
          * &lt;script&gt;
          * // To expand all items
          * $("#gantt").ejGantt("collapseAllItems");	
          * &lt;/script&gt;
          * @memberof ejGantt
          * @instance
           */
        collapseAllItems:function() {
            this._collapseAll();
        },
        
          /**
          * To indent a selected item in gantt
          * @return jQuery
          * @example 
          * &lt;gantt id="gantt"&gt;Gantt&lt;/gantt&gt; <br> 
          * &lt;script&gt;
          * // Create Gantt
          * var ganttObj = $("#gantt").data("ejGantt");
          * gantObj.ejGantt("indentItem"); // To indent a selected item in gantt
          * &lt;/script&gt;
          * @example 
          *  &lt;gantt id="gantt"&gt;Gantt&lt;/gantt&gt; <br> 
          * &lt;script&gt;
          * // To indent a selected item in gantt
          * $("#gantt").ejGantt("indentItem");	
          * &lt;/script&gt;
          * @memberof ejGantt
          * @instance
           */
        indentItem:function() {
            this._sendIndentRequest();
        },

        /**
       * To outdent a selected item in gantt
       * @return jQuery
       * @example 
       * &lt;gantt id="gantt"&gt;Gantt&lt;/gantt&gt; <br> 
       * &lt;script&gt;
       * // Create Gantt
       * var ganttObj = $("#gantt").data("ejGantt");
       * gantObj.ejGantt("outdentItem"); // To outdent a selected item in gantt
       * &lt;/script&gt;
       * @example 
       *  &lt;gantt id="gantt"&gt;Gantt&lt;/gantt&gt; <br> 
       * &lt;script&gt;
       * // To outdent a selected item in gantt
       * $("#gantt").ejGantt("outdentItem");	
       * &lt;/script&gt;
       * @memberof ejGantt
       * @instance
        */
        outdentItem:function() {
            this._sendOutdentRequest();
        },
        

        /**
        * To cancel the edited state of an item in gantt
        * @return jQuery
        * @example 
        * &lt;gantt id="gantt"&gt;Gantt&lt;/gantt&gt; <br> 
        * &lt;script&gt;
        * // Create Gantt
        * var ganttObj = $("#gantt").data("ejGantt");
        * gantObj.ejGantt("cancelEdit"); // To cancel edited
        * &lt;/script&gt;
        * @example 
        *  &lt;gantt id="gantt"&gt;Gantt&lt;/gantt&gt; <br> 
        * &lt;script&gt;
        * // To outdent a selected item in gantt
        * $("#gantt").ejGantt("cancelEdit");	
        * &lt;/script&gt;
        * @memberof ejGantt
        * @instance
         */
        cancelEdit:function() {
            this.sendCancelRequest();
        },
        

        /**
       * To expand and collapse an item in gantt using item's ID
       * @return jQuery
       * @example 
       * &lt;gantt id="gantt"&gt;Gantt&lt;/gantt&gt; <br> 
       * &lt;script&gt;
       * // Create Gantt
       * var ganttObj = $("#gantt").data("ejGantt");
       * gantObj.ejGantt("expandCollapseRecord" , "23"); // To expand collapse an item
       * &lt;/script&gt;
       * @example 
       *  &lt;gantt id="gantt"&gt;Gantt&lt;/gantt&gt; <br> 
       * &lt;script&gt;
       * // To expand collapse an item
       * $("#gantt").ejGantt("expandCollapseRecord" , "23");	
       * &lt;/script&gt;
       * @memberof ejGantt
       * @instance
        */
        expandCollapseRecord: function (taskId) {
            var proxy = this, args = {},
                record = proxy.model.ganttRecords[proxy._ids.indexOf(taskId)],
                expand = record.expanded == true ? false : true;
            args.data = record;
            args.recordIndex = proxy._ids.indexOf(taskId);
            args.expanded = expand;
            args.requestType = "expandCollapse";
            proxy._$treegridHelper.ejTreeGrid("sendExpandCollapseRequest", args);
        },



        /**
       * To save the edited state of an item in gantt
       * @return jQuery
       * @example 
       * &lt;gantt id="gantt"&gt;Gantt&lt;/gantt&gt; <br> 
       * &lt;script&gt;
       * // Create Gantt
       * var ganttObj = $("#gantt").data("ejGantt");
       * gantObj.ejGantt("saveEdit"); // To save edited state of an item
       * &lt;/script&gt;
       * @example 
       *  &lt;gantt id="gantt"&gt;Gantt&lt;/gantt&gt; <br> 
       * &lt;script&gt;
       * // To expand collapse an item
       * $("#gantt").ejGantt("saveEdit");	
       * &lt;/script&gt;
       * @memberof ejGantt
       * @instance
        */
        saveEdit: function () {
            var proxy = this;
            proxy._$treegridHelper.ejTreeGrid("endEdit");
        },
        
        _showTooltip: function (boolValue) {
            var proxy = this;
            this.model.enableTaskbarTooltip = boolValue;
            proxy._$ganttchartHelper.ejGanttChart("showTooltip", boolValue);
        },

        _showEditingTooltip: function (boolValue) {
            var proxy = this;
            this.model.enableTaskbarDragTooltip = boolValue;
            proxy._$ganttchartHelper.ejGanttChart("showEditingTooltip", boolValue);
        },
        
        _ganttChartEditing:function(boolValue) {
            var proxy = this;
            proxy.model.allowGanttChartEditing = boolValue;
            proxy._$ganttchartHelper.ejGanttChart("ganttChartEditing", boolValue);
        },
        
        _resizeProgressbar: function (boolValue) {
            var proxy = this;
            proxy.model.enableProgressBarResizing = boolValue;
            proxy._$ganttchartHelper.ejGanttChart("resizeProgressbar", boolValue);

        },
        
        _sendSortingRequest: function(sortSettings) {
            var proxy = this, columnName, columnSortDirection;
            columnName = sortSettings.sortedColumns[0].field;
            columnSortDirection = sortSettings.sortedColumns[0].direction;
            proxy._$treegridHelper.ejTreeGrid("sortColumn", columnName, columnSortDirection);
        },
        
        sendCancelRequest: function () {
            var proxy = this;
            proxy._$treegridHelper.ejTreeGrid("cancelEditCell");

            if (proxy.model.toolbarSettings.toolbarItems.indexOf("cancel") !== -1) {
                proxy._disabledToolItems.push($($('.e-cancel')[0]).parent()[0]);
            }

            if (proxy.model.toolbarSettings.toolbarItems.indexOf("save") !== -1) {
                proxy._disabledToolItems.push($($('.e-saveitem')[0]).parent()[0]);
            }

            $('.e-gantttoolbar').ejToolbar('disableItem', proxy._disabledToolItems);
            proxy._disabledToolItems = [];
        },
        
        _sendAddRequest: function () {
            
            var proxy = this,                
                columns = proxy._columns,
                length = columns.length,
                columnCount = 0,
                cloneData = {},
                args = {};
            proxy._isinAddnewRecord = true;
            
            for (columnCount; columnCount < length; columnCount++)
                cloneData[columns[columnCount].mappingName] = "";

            args.data = cloneData;
            args.requestType = "add";            
          //  proxy._$treegridHelper.ejTreeGrid("clearSelection");
            proxy.renderedEditDialog(args);
        },

        _sendEditRequest: function ($tr) {
            var proxy = this,
                model = proxy.model,
                args,
                data;
            
            if (ej.isNullOrUndefined($tr)) {
                proxy._currentTrIndex = this.selectedRowIndex();
                proxy._$currentTr = $(proxy._$treegridHelper.ejTreeGrid("getRows")).eq(this.selectedRowIndex());
            } else {
                proxy._currentTrIndex = $tr.index();
                proxy._$currentTr = $tr;
            }
            data = proxy.getUpdatedRecords()[this.selectedRowIndex()];

            args = { rowElement: proxy._$currentTr, data: data };
            proxy._trigger("beginEdit", args);
            args.requestType = "beginedit";
            proxy.renderedEditDialog(args);
        },
        _updateAddData:function(obj)
        {
            var proxy = this,model=this.model,id,startDate;
            var cloneObj = {};
           
            if (!obj[model.taskIdMapping])
            {
                 id= proxy._updatedGanttRecords.length;
                 id = proxy._getNewTaskId(id);
                 var indx = proxy._ids.indexOf(id.toString());
                 proxy._ids.splice(indx,1);
                 obj[model.taskIdMapping] = id;
            }

            if (model.taskNameMapping && !obj[model.taskNameMapping]) {
                obj[model.taskNameMapping] = "New Task" + obj[model.taskIdMapping];
            }

            if (!obj[model.startDateMapping]) {
                obj[model.startDateMapping] = model.scheduleStartDate;
            }

            if (model.durationMapping && ej.isNullOrUndefined(obj[model.durationMapping])) {
                if (!obj[model.endDateMapping])
                obj[model.durationMapping] = "5";
            }
            if (model.progressMapping) {
                obj[model.progressMapping] = obj[model.progressMapping] > 100 ?
                    100 : obj[model.progressMapping];
            }

            if (!obj[model.endDateMapping] && model.endDateMapping) {

                if (!obj[model.durationMapping]) {
                    var startDate = proxy._getDateFromFormat(model.scheduleStartDate);
                    startDate.setDate(startDate.getDate() + 4);
                    obj[model.endDateMapping] = Globalize.format(startDate, this.model.dateFormat);
                }
            }
        },
        _sendSaveRequest: function (saveType) {
            var obj = {},
                doc = document,
                proxy = this,
                model=proxy.model,
                index = 0,
                formelement,
                length,
                editedRowWrap,
                columnName,
                $element,
                startdate,
                enddate,
                column,
                value,dialogId,treeGridId
            args = {};
            if(saveType==="Add")
            {
                formelement = doc.getElementById(proxy._id + "AddForm");
                length = formelement && formelement.length;
                dialogId = "Add";
                treeGridId = "#treegrid" + proxy._id + "predecessorAdd";
                $element = $("#" + proxy._id + "taskId" + dialogId);             
                column = ej.TreeGrid.getColumnByField(proxy._columns, "taskId");
                proxy._cellEditType = column.editType;
                if (column && $element.length>0)
                    value = proxy.getCurrentEditCellData($element, column.dropdownData);

                if (proxy._ids.indexOf(value) != -1 && value)
                    this._duplicate = true;
                else if (value) {
                    this._duplicate = false;
                }
            }
            else
            {
                formelement = doc.getElementById(proxy._id + "EditForm");
                length = formelement && formelement.length;
                treeGridId = "#treegrid" + proxy._id + "predecessoredit";
                dialogId = "";
            }
            editedRowWrap = $(formelement).closest('div');


            if (!this.editFormValidate())
                return true;            
                      
            if (!this._duplicate) {
                for (index = 0; index < length; index++) {
                    columnName = formelement[index].name;
                    $element = $(formelement[index]);
                    if (columnName) {
                        if (!obj[columnName])
                            column = ej.TreeGrid.getColumnByField(proxy._columns, columnName);

                        if (column && formelement[index].id === (proxy._id + columnName + dialogId)) {
                            proxy._cellEditType = column.editType;
                            value = proxy.getCurrentEditCellData($element, column.dropdownData); // formelement[index].value;
                            if (proxy._isinAddnewRecord === false && column.mappingName === proxy.model.predecessorMapping) {
                                args.previousValue = proxy._updatePredecessorValue(obj, value);
                            }
                            if (formelement[index].type != "checkbox")
                                obj[column.mappingName] = value;
                        }
                    }
                }
                date1 = proxy._getDateFromFormat(obj[model.startDateMapping]);
                date2 = proxy._getDateFromFormat(obj[model.endDateMapping]);
                var diffdays=0;
                if (date1 && date2)
                    diffdays = date2.getTime() - date1.getTime();
                if (diffdays < 0)
                    this._wrongenddate = true;
                else
                    this._wrongenddate = false;
                if (!this._wrongenddate) {
                    // when adding new task resource value changes.
                if (obj[model.resourceIdMapping] && editedRowWrap.hasClass("e-addedrow"))
                {
                        var resource = [];
                        $.each(obj[model.resourceIdMapping], function (indx, value) { resource.push(value[model.resourceIdMapping]); })
                        obj[model.resourceIdMapping] = resource;
                    }

                    if (!obj[model.taskIdMapping] && saveType === "Edit")
                        obj[model.taskIdMapping] = proxy._editedDialogRecord.taskId;

                if (saveType === "Add")
                {
                        proxy._updateAddData(obj);
                    }

                    args.data = obj;


                    //getting predecessor from edit dialog table    
                    var prdecessor_validation_flag = false;
                    var results = [], resultFlag = false;;
                if (model.predecessorMapping )
                {
                        if (saveType === "Edit") {
                            resultFlag = model.editDialogFields.length === 0 ? true : false;
                            results = $.grep(model.editDialogFields, function (val, idx) {
                                return (val.field === model.predecessorMapping);
                            });
                    }else if(saveType === "Add")
                    {
                        resultFlag = model.addDialogFields.length=== 0 ? true : false;
                            results = $.grep(model.addDialogFields, function (val, idx) {
                                return (val.field === model.predecessorMapping);
                            });
                        }
                    }

                if (results.length > 0 || resultFlag)
                {
                        var pre = [];
                        var taskNameFlag = true;
                        pre = $(treeGridId).ejTreeGrid("option", "dataSource");
                        $.each(pre, function (idx, value) {
                        if(!(value.id.length>0 && value.name.length && value.type.length))
                        {
                                taskNameFlag = false;
                            }
                        });

                        if (taskNameFlag) {
                            var predecessorColl = [], predecessorString = [];//from,to,offset,predecessorType,isdrawn
                            var types = [{ id: "SS", text: "Start-Start", value: "Start-Start" }, { id: "SF", text: "Start-Finish", value: "Start-Finish" }, { id: "FS", text: "Finish-Start", value: "Finish-Start" }, { id: "FF", text: "Finish-Finish", value: "Finish-Finish" }];
                            var tempType;
                            for (var i = 0; i < pre.length; i++) {
                                var predecessor = {};
                                predecessor["from"] = obj.taskID;
                                predecessor["to"] = pre[i].id;
                                predecessor["isdrawn"] = false;
                            predecessor["offset"] =isNaN(parseInt(pre[i].offset)) ? 0 : parseInt(pre[i].offset);
                                for (var j = 0; j < types.length; j++) {
                                    if (pre[i].type == types[j].text) {
                                        predecessor["predecessorType"] = types[j].id;
                                        tempType = types[j].id;
                                        break;
                                    }
                                }
                                var tem = pre[i].id + tempType//+data[i].lags.toString();
                            pre[i].offset=isNaN(parseInt(pre[i].offset)) ? 0 : parseInt(pre[i].offset);
                                if (pre[i].offset !== 0) {
                                    if (pre[i].offset < 0)
                                        tem += pre[i].offset.toString();
                                    else
                                        tem += "+" + pre[i].offset.toString();
                                }
                                predecessorString.push(tem);
                                predecessorColl.push(predecessor);
                            }
                            args.predecessorString = predecessorString;

                            //befor save validate the edit form data
                            prdecessor_validation_flag = proxy._editedPredecessorValidation(args);

                        } else {
                            //Break here and give notification to users
                            alert("Fill all the columns in predecessor table")
                            return true;
                        }

                        if (prdecessor_validation_flag) {
                            if (proxy._isinAddnewRecord === false) {// && proxy.model.predecessorMapping
                                args.previousValue = proxy._updatePredecessorValue(obj, predecessorString.join(','));
                            }
                            obj[model.predecessorMapping] = predecessorString.join(',');
                        }
                        else {
                            alert("Cyclic Dependency Occured, Please Check The Predecessor");
                            return true;
                        }
                    }

                    args.requestType = ej.TreeGrid.Actions.Save;
                    args.recordIndex = (this.selectedRowIndex() !== -1) ? this.selectedRowIndex() : -1;

                    if (proxy._trigger("actionBegin", args))
                        return true;


                    if (editedRowWrap.hasClass("e-editedrow")) {
                        args._cModifiedData = proxy._updateRecord(obj);
                        if (proxy._editedDialogRecord) {
                            if (!args._cModifiedData.startDate) {
                                args._cModifiedData.startDate = proxy._editedDialogRecord.startDate;
                            }
                            if (!args._cModifiedData.endDate) {
                                args._cModifiedData.endDate = proxy._editedDialogRecord.endDate;
                            }
                            if (!args._cModifiedData.duration) {
                                args._cModifiedData.duration = proxy._editedDialogRecord.duration;
                            }
                            proxy._editedDialogRecord = null;
                        }

                        if (args.data[model.endDateMapping] && !args.data[model.durationMapping]) {
                            args._cModifiedData._calculateDuration(proxy._getDateFromFormat(args._cModifiedData.startDate), proxy._getDateFromFormat(args._cModifiedData.endDate), model.includeWeekend, model.holidays, args._cModifiedData.isMilestone, model);
                        }

                    }
                    else if (editedRowWrap.hasClass("e-addedrow")) {

                        var parentItem, Level = 0, cIndex;
                        if (model.selectedItem) {
                            if (model.selectedItem.hasChildRecords) {//this is parent item
                                Level = model.selectedItem.level + 1
                                parentItem = model.selectedItem;
                                cIndex = 0;
                            }
                            else if ((!model.selectedItem.hasChildRecords && model.selectedItem.parentItem)) {
                                Level = model.selectedItem.level;
                                parentItem = model.selectedItem.parentItem;
                                cIndex = parentItem.childRecords.indexOf(model.selectedItem) + 1;
                            }
                            else {
                                Level = 0;
                                parentItem = null;
                            }
                        }
                        args._cAddedRecord = proxy._createGanttRecord(obj, Level, parentItem, false);
                        if (model.predecessorMapping && args._cAddedRecord.predecessor && args._cAddedRecord.parentItem) {
                            var predecessorIdArray = proxy._idFromPredecessor(args._cAddedRecord.item[model.predecessorMapping]);
                            if (predecessorIdArray.indexOf(args._cAddedRecord.parentItem.taskId.toString()) !== -1) {
                                alert("Cyclic Dependency Occured, Please Check The Predecessor");
                                return true;
                            }
                        }
                    }

                    if (args._cAddedRecord) {
                        args._cAddedRecord.taskId =args._cAddedRecord.taskId;
                        args._cAddedRecord.index = args.recordIndex;
                        //proxy._$treegridHelper.ejTreeGrid("processBindings", args);
                    }

                    proxy.actionComplete(args);
                    proxy._cModifiedData = null;
                    proxy._cAddedRecord = null;
                    proxy._primaryKeyValues = [];
                    proxy._isinAddnewRecord = false;
                    return false;
                }
                else {
                    alert("Invalid End date");                           
                }
            }
            else {
                alert("Duplicate ID");
                         
            }
                
        },




        _recordReturnerByID:function(id)
        {
            var proxy = this,
                records = proxy._rows;
            for (var i = 0; i < records.length; i++) {
                if (records[i].taskId.toString() === id.toString()) {
                    return records[i]
                }
            }
            return null;

        },
        _editedPredecessorValidation: function (args) {

            var proxy = this,
                records = proxy._rows,
                flag = true,
                recordId = args.data[this.model.taskIdMapping],
                predecessor = args.predecessorString.join(','),predecessorIdArray;
            if (predecessor.length > 0) {
                predecessorIdArray = proxy._idFromPredecessor(predecessor);
                for (var count = 0; count < predecessorIdArray.length; count++) {
                    //Check edited item has parent item in predecessor collection
                    var editingData = proxy._recordReturnerByID(recordId);
                    if (editingData && editingData.parentItem) {
                        if (predecessorIdArray.indexOf(editingData.parentItem.taskId.toString()) !== -1) {
                            return false;
                        }
                    }
                    // Check if predecessor exist more then one 
                    var tempIdArray = predecessorIdArray.slice(0),checkArray=[],countFlag=true;
                    $.each(tempIdArray, function (index, value) { 
                        if(checkArray.indexOf(value)===-1)
                            checkArray.push(value);
                        else
                            countFlag = false;
                    });
                    if (countFlag === false) {
                        return false;

                    }
                    //Cyclick check  
                    currentId = predecessorIdArray[count];
                    var visitedIdArray = [];
                    var predecessorCollection = predecessorIdArray.slice(0);
                    predecessorCollection.splice(count, 1);


                    while (currentId !== null) {
                        var currentIdArray = [];
                        if (visitedIdArray.indexOf(currentId) === -1)
                        {
                            currentRecord = proxy._recordReturnerByID(currentId);
                          //  var currentPredecessor="";
                            if (!ej.isNullOrUndefined(currentRecord.predecessor) && currentRecord.predecessor.length>0)
                            {
                                $.each(currentRecord.predecessor, function (index, value) {
                                    if (currentRecord.taskId.toString() !== value.from)
                                        currentIdArray.push(value.from.toString());
                                });
                            //    currentPredecessor=currentRecord.predecessor[0].from
                            }
                            if (recordId.toString() === currentRecord.taskId.toString() || currentIdArray.indexOf(recordId.toString()) !== -1) { // || predecessorCollection.indexOf(currentPredecessor) !== -1
                                //cycylic occurs//break;
                                return false;
                            }
                            visitedIdArray.push(currentId);
                            if (!ej.isNullOrUndefined(currentRecord.predecessor) && currentRecord.predecessor.length > 0) {
                                var result;
                                if (currentIdArray.length > 1) {
                                    result=this._predecessorValidation(currentIdArray, args);
                                }
                                else
                                    currentId = currentRecord.predecessor[0].from;
                                if (result === false) {
                                    return false;
                                }
                            }
                            else
                                break;
                        }
                        else
                        {
                            break;
                        }
                    }
                }
            }
            return flag;
        },

        _predecessorValidation: function (predecessor,args) {
            var proxy = this;
            var recordId=args.data[this.model.taskIdMapping];
            for (var count = 0; count < predecessor.length; count++) {
                currentId = predecessor[count];
                var visitedIdArray = [];
                var predecessorCollection = predecessor.slice(0);
                predecessorCollection.splice(count, 1);

                while (currentId !== null) {
                    var currentIdArray = [];
                    if (visitedIdArray.indexOf(currentId) === -1) {
                        currentRecord = proxy._recordReturnerByID(currentId);
                        //  var currentPredecessor="";
                        if (!ej.isNullOrUndefined(currentRecord.predecessor) && currentRecord.predecessor.length > 0) {
                            $.each(currentRecord.predecessor, function (index, value) {
                                if (currentRecord.taskId.toString() !== value.from)
                                    currentIdArray.push(value.from.toString());
                            });
                            //    currentPredecessor=currentRecord.predecessor[0].from
                        }
                        if (recordId.toString() === currentRecord.taskId.toString() || currentIdArray.indexOf(recordId.toString()) !== -1) { // || predecessorCollection.indexOf(currentPredecessor) !== -1
                            //cycylic occurs//break;
                            return false;
                        }
                        visitedIdArray.push(currentId);
                        if (!ej.isNullOrUndefined(currentRecord.predecessor) && currentRecord.predecessor.length > 0) {
                            currentId = currentRecord.predecessor[0].from;
                        }
                        else
                            break;
                    }
                    else {
                        break;
                    }
                }
            }
            return true;
            
        },
        _sendCancelRequest: function () {
            var proxy = this,
                model=proxy.model,
                args = {};

            if (model.dateFormat.indexOf("hh") != -1)
                $("#" + proxy._id + "EditForm").find(".e-datepicker").ejDateTimePicker("destroy");
            else
                $("#" + proxy._id + "EditForm").find(".e-datepicker").ejDatePicker("destroy");

            args.requestType = ej.TreeGrid.Actions.Cancel;
            proxy.processBindings(args);
            proxy._primaryKeyValues = [];
            proxy.model.currentData = null;
        },
        
        _updatePredecessorValue: function (obj, value) {
            var proxy = this,
                prevPredecessors,
                modifiedPredecessors,
                length=0,
                predecessor,
                ganttPredecessor = [],
                count = 0;
            prevPredecessors = proxy.selectedItem()["predecessor"];

            if (value.length > 0) {
                modifiedPredecessors = proxy.selectedItem()._calculatePredecessor(value);
            }
            if (modifiedPredecessors)
                length = modifiedPredecessors.length;

            var index = 0, prevPrdecessorlength=0;
            if(prevPredecessors)
                prevPrdecessorlength = prevPredecessors.length;


            for (index = 0; index < prevPrdecessorlength; index++) {
                predecessor = prevPredecessors[index];
                if (predecessor.from === proxy.selectedItem().taskId.toString()) {
                    ganttPredecessor.push(predecessor);
                }
            }
            for (count = 0; count < length; count++) {
                ganttPredecessor.push(modifiedPredecessors[count]);
            }
            proxy.selectedItem().predecessor = ganttPredecessor;
            if (proxy.model.predecessorMapping)
                proxy.selectedItem().item[proxy.model.predecessorMapping] = value;

            return prevPredecessors;          //
        },


        getCurrentEditCellData: function ($element, dropdownData) {
            var proxy = this, model = proxy.model, date;
            if ($("#" + proxy._id + "EditForm").length || $("#" + proxy._id + "AddForm").length) {
                var cellValue="";// $element = $("#" + proxy._id + proxy._columnFieldName), 
                switch (proxy._cellEditType) {
                    case ej.TreeGrid.EditingType.String:
                        cellValue = $element.val();
                        break;
                    case ej.TreeGrid.EditingType.Numeric:
                        cellValue = $element.ejNumericTextbox("getValue");
                        $element.ejNumericTextbox("getValue");
                        break;
                    case ej.TreeGrid.EditingType.Dropdown:
                        cellValue = proxy._getSelectedItem($element.ejDropDownList("model.selectedItems"), dropdownData);
                        break;
                    case ej.TreeGrid.EditingType.Boolean:
                        cellValue = $element.is(':checked');
                        break;

                    case ej.TreeGrid.EditingType.DatePicker:
                        if (model.dateFormat.indexOf("hh") != -1)
                            date = $element.ejDateTimePicker("model.value");
                        else
                            date = $element.ejDatePicker("model.value");
                        cellValue = Globalize.format(date, model.dateFormat);
                        break;
                    case ej.TreeGrid.EditingType.DateTimePicker:
                        cellValue = $element.ejDateTimePicker("model.value");
                        break;
                }

                //if (typeof cellValue == "string" && cellValue.length )//&& model.columns[proxy._cellEditingDetails.columnIndex].type == "number")
                //    cellValue = parseFloat(cellValue);
                return cellValue;
            }
            return null;
        },


        _getSelectedItem: function (indexArray, dropDownData) {
            var count = 0,
                length = indexArray.length,
                    selectedItems = [];
            if (dropDownData) {
                for (count = 0; count < length; count++) {
                    $.each(dropDownData, function (index, resourceInfo) {
                        if (indexArray[count] === index) {
                            selectedItems.push(resourceInfo);
                        }
                    });
                }
            }

            return selectedItems;
        },
        _updateRecord: function (data) {
            var proxy = this,model=this.model,
                columns=proxy._columns,
               length,
                index = 0,
                ganttRecord = proxy.getUpdatedRecords()[this.selectedRowIndex()];

            if (model.editDialogFields.length > 0) {
                var filteredColumns = [];
                var count = 0, resultColumn;
                for (; count < model.editDialogFields.length; count++) {
                    resultColumn = $.grep(proxy._columns, function (val) {
                        return val.mappingName === model.editDialogFields[count].field;
                    });
                    resultColumn.length && filteredColumns.push(resultColumn[0]);
                }
                columns = filteredColumns;
            }
            length = columns.length;

            for (index = 0; index < length; index++) {

                if (columns[index].field.toString() === "predecessor")
                    continue;
                    //for predecessor from edit dialog window 
                
                ganttRecord[columns[index].field] = data[columns[index].mappingName];
                ganttRecord.item[columns[index].mappingName] = data[columns[index].mappingName];//field
            }
            
            return ganttRecord;
        },

        _sendIndentRequest: function () {

            var proxy = this,
                model = proxy.model,
                data = proxy.getCurrentData(), args = {}, eveargs = {};

            if (ej.isNullOrUndefined(data)) {
                alert("There is no gantt Record was not selected to perform the Indent ");
            }
            else {

                proxy._updatedGanttRecords = proxy.getUpdatedRecords();
                var previousGanttRecord = proxy._updatedGanttRecords[this.selectedRowIndex() - 1];
                if (previousGanttRecord.isMilestone)
                    return;
                args.data = data;
                args.requestType = "beginedit"
                if (!proxy._trigger("actionBegin", args)) {

                    if (data.parentItem) {
                        var childIndex = data.parentItem.childRecords.indexOf(data);
                        data.parentItem.childRecords.splice(childIndex, 1);
                        if(!model.parentTaskIdMapping)
                            data.parentItem.item[model.childMapping].splice(childIndex, 1);
                    } else {
                        var dataSourceIndex = this.dataSource().indexOf(data.item);
                        if (!model.parentTaskIdMapping)
                            this.dataSource().splice(dataSourceIndex, 1);
                    }
                    if (data.level === previousGanttRecord.level) {
                        if (previousGanttRecord.childRecords) {
                            if (!previousGanttRecord.expanded) {
                                previousGanttRecord.childRecords.splice(previousGanttRecord.childRecords.length, 0, data);
                                if (!model.parentTaskIdMapping)
                                    previousGanttRecord.item[model.childMapping].splice(previousGanttRecord.childRecords.length, 0, data.item);
                                else
                                    data.item[model.parentTaskIdMapping] = previousGanttRecord.taskId;
                            }
                        } else {
                            previousGanttRecord.childRecords = [];
                            previousGanttRecord.childRecords.push(data);
                            data.parentItem = previousGanttRecord;
                            previousGanttRecord.item[model.childMapping] = [];
                            if (!model.parentTaskIdMapping)
                                previousGanttRecord.item[model.childMapping].push(data.item);
                            else
                                data.item[model.parentTaskIdMapping] = previousGanttRecord.taskId;
                        }
                    } else if (previousGanttRecord.level - data.level === 1) {
                        previousGanttRecord.parentItem.childRecords.splice(previousGanttRecord.parentItem.childRecords.length, 0, data);
                        if (!model.parentTaskIdMapping)
                            previousGanttRecord.parentItem.item[model.childMapping].splice(previousGanttRecord.parentItem.childRecords.length, 0, data.item);
                        else
                            data.item[model.parentTaskIdMapping] = previousGanttRecord.parentItem.taskId;
                        data.parentItem = previousGanttRecord.parentItem;
                    } else if (previousGanttRecord.level - data.level > 1) {

                        var parentItem = proxy.getParentItem(previousGanttRecord, data.level);
                        parentItem.childRecords.splice(parentItem.childRecords.length, 0, data);
                        if (!model.parentTaskIdMapping)
                            parentItem.item[model.childMapping].splice(parentItem.childRecords.length, 0, data.item);
                        else
                            data.item[model.parentTaskIdMapping] = parentItem.taskId;
                        data.parentItem = parentItem;

                    }
                }
                previousGanttRecord.hasChildRecords = previousGanttRecord.childRecords ? previousGanttRecord.childRecords.length > 0 : false;
                previousGanttRecord.expanded = previousGanttRecord.childRecords ? previousGanttRecord.childRecords.length > 0 : false;
                data.level += 1;
                proxy.updateLevel(data, +1);
                proxy.refreshGanttRecord(data);
                proxy.updateIndentOutdentOption(data);
                //update parent recordsCollections
                if (previousGanttRecord.level === 0) {
                    if (model.parentRecords.indexOf(previousGanttRecord) === -1) {
                        model.parentRecords.push(previousGanttRecord);
                    }
                }
                if (data.level > 0) {
                    if (model.parentRecords.indexOf(data) !== -1) {
                        var idx = model.parentRecords.indexOf(data);
                        model.parentRecords.splice(idx, 1);
                    }
                }
                //Update parentitem
                if (data.parentItem) {
                   // data._updateParentProgress(data.parentItem, model.progressMapping);
                    proxy._updateParentItem(data);
                }
                //Remove child predecessor from Parent item
                if (previousGanttRecord.predecessor && previousGanttRecord.predecessor.length > 0 && proxy.model.predecessorMapping && previousGanttRecord.hasChildRecords) {
                    proxy._updatePredecessorOnIndentOutdent(previousGanttRecord);
                }

                proxy.refreshGanttRecord(previousGanttRecord);
                eveargs.data = data;
                eveargs.requestType = "endedit";
                this._trigger("actionComplete", eveargs);
            }
        },
        
        _updatePredecessorOnIndentOutdent:function(parentRecord)
        {

            var proxy = this, len = parentRecord.predecessor.length,
                predecessorCollection = parentRecord.predecessor,
                predecessorIndex,
                childRecord,
                id,
                updatedPredecessor = [];
            for (var count = 0; count < len; count++) {
                //remove predecessor connect with parent item
                if (predecessorCollection[count].to === parentRecord.taskId.toString()) {
                    childRecord = proxy._rows[proxy._ids.indexOf(predecessorCollection[count].from)];
                    predecessorIndex = childRecord.predecessor.indexOf(predecessorCollection[count]);
                    childRecord.predecessor.splice(predecessorIndex, 1);
                    id = "parent" + childRecord['taskId'] + "child" + parentRecord['taskId'];
                    proxy._$ganttchartHelper.ejGanttChart("removeConnectorline", id);
                }
                    //remove predecessor if parent record as predecessor of it's child record
                else if (predecessorCollection[count].from === parentRecord.taskId.toString()) {
                    childRecord = proxy._rows[proxy._ids.indexOf(predecessorCollection[count].to)];
                    if (parentRecord.childRecords.indexOf(childRecord) !== -1) {
                        predecessorIndex = childRecord.predecessor.indexOf(predecessorCollection[count]);
                        childRecord.predecessor.splice(predecessorIndex, 1);
                        var stringPredecessor = proxy._predecessorToString(childRecord.predecessor, childRecord);
                        childRecord.item[model.predecessorMapping] = stringPredecessor;
                        id = "parent" + parentRecord['taskId'] + "child" + childRecord['taskId'];
                        proxy._$ganttchartHelper.ejGanttChart("removeConnectorline", id);
                        proxy.refreshGanttRecord(childRecord);
                    }
                    else
                    {
                        updatedPredecessor.push(predecessorCollection[count]);
                    }
                }
            }
            parentRecord.predecessor = updatedPredecessor;
            parentRecord.item[model.predecessorMapping] = "";
            if (updatedPredecessor.length > 0) {
                proxy._updatedConnectorLineCollection = [];
                proxy._connectorlineIds = [];
                proxy._validatePredecessor(parentRecord);
                if (proxy._updatedConnectorLineCollection.length > 0) {
                    proxy._$ganttchartHelper.ejGanttChart("appendConnectorLine", proxy._updatedConnectorLineCollection);
                }
            }
        },
        
        _removePredecessor:function(predecessor,record)
        {
            var proxy = this, length = predecessor.length, count = 0, model = proxy.model, parentGanttRecord, childGanttRecord;
            for (count; count < length; count++)
            {
                if (predecessor[count].from !== record.taskId.toString())
                {
                    childGanttRecord = proxy._rows[proxy._ids.indexOf(predecessor[count].from)];  //index st
                }
                else if (predecessor[count].to !== record.taskId.toString())
                {
                    childGanttRecord = proxy._rows[proxy._ids.indexOf(predecessor[count].to)]
                }

                if (childGanttRecord) {
                    var cIndex = childGanttRecord.predecessor.indexOf(predecessor[count]);
                    if (cIndex !== -1)
                        childGanttRecord.predecessor.splice(cIndex, 1);

                    if (childGanttRecord.item[model.predecessorMapping]) {
                        childGanttRecord.item[model.predecessorMapping] = proxy._predecessorToString(childGanttRecord.predecessor, childGanttRecord);
                        proxy.refreshGanttRecord(childGanttRecord);
                    }
                }
            }

        },

        _predecessorToString:function(predecessorCollection,record)
        {
            var proxy = this, predecessorString=[],count=0,length=predecessorCollection.length;
            
            for (count; count < length;count++)
            {
                if(record.taskId.toString()!==predecessorCollection[count].from)
                    {
                        var tem = predecessorCollection[count].from + predecessorCollection[count].predecessorsType;
                        predecessorCollection[count].offset = isNaN(predecessorCollection[count].offset) ? 0 : predecessorCollection[count].offset;

                        if (predecessorCollection[count].offset !== 0) {
                            if (predecessorCollection[count].offset < 0)
                                tem += predecessorCollection[count].offset.toString();
                            else if (predecessorCollection[count].offset > 0)
                                tem += "+" + predecessorCollection[count].offset.toString();
                        }
                        predecessorString.push(tem);
                }
            }
           
            return predecessorString.join(',');
        },
        //get the parentItem of selected ganttRecord to perform the indent operation
        
        getParentItem:function(ganttRecord,level) {

            var proxy = this;

            if (ganttRecord.parentItem.level === level)
                return ganttRecord.parentItem;

            return proxy.getParentItem(ganttRecord.parentItem, level);
        },

        _sendOutdentRequest: function () {
            var proxy = this,
                model = proxy.model,
                data = proxy.getCurrentData(), args = {}, eveargs = {};

            if (ej.isNullOrUndefined(data)) {
                alert("There is no gantt Record was not selected to perform the Outdent ");
            } else {
                args.data = data;
                args.requestType = "actionBegin"
                if (!proxy._trigger("actionBegin", args)) {

                if (data.level === 0)
                    return;
                else {                 

                        proxy._updatedGanttRecords = proxy.getUpdatedRecords();
                        if (data.isMilestone)
                            return;

                        //Remove the Gantt Record from it's parent GanttRecord
                        if (data.parentItem.childRecords) {

                            var childIndex = data.parentItem.childRecords.indexOf(data);

                            data.parentItem.childRecords.splice(childIndex, 1);
                            if(!model.parentTaskIdMapping)
                                data.parentItem.item[model.childMapping].splice(childIndex, 1);

                            if (data.parentItem.childRecords.length == 0) {

                                data.parentItem.childRecords = null;
                                data.parentItem.expanded = false;
                                data.parentItem.hasChildRecords = false;
                            }
                        }

                        //Add Gantt Record to ParentItem of it's parent item
                        if (data.parentItem.parentItem) {

                            var parentIndex = data.parentItem.parentItem.childRecords.indexOf(data.parentItem);

                            data.parentItem.parentItem.childRecords.splice(parentIndex + 1, 0, data);
                            if (!model.parentTaskIdMapping)
                                data.parentItem.parentItem.item[model.childMapping].splice(parentIndex + 1, 0, data.item);
                            else
                                data.item[model.parentTaskIdMapping] = data.parentItem.parentItem.taskId;

                            proxy.refreshGanttRecord(data.parentItem.parentItem);


                        } else {
                            parentIndex = this.dataSource().indexOf(data.parentItem.item);
                            delete data.item[model.parentTaskIdMapping];
                            this.dataSource().splice(parentIndex + 1, 0, data.item);
                        }

                        if (data.hasChildRecords) {
                            proxy._updateChildLevel(data);
                        }
                        data.level -= 1; //Decrease the Indent level of the Gantt Record                               
                        childIndex !== -1 && proxy._removeChildRecords(data, childIndex);
                       //update parentitem of selected item after changing of its child 
                        if (data.parentItem)
                        {
                            data.parentItem.hasChildRecords && proxy._updateParentItem(data.parentItem.childRecords[0]);
                            proxy.refreshGanttRecord(data.parentItem);
                        }
                        data.parentItem = data.parentItem.parentItem;
                        if (data.hasChildRecords) {
                            proxy._refreshChildLevel(data);
                        }

                        data.hasChildRecords = data.childRecords ? data.childRecords.length > 0 : false;
                        data.expanded = data.childRecords ? data.childRecords.length > 0 : false;


                        if (data.hasChildRecords) {
                            var count = 0,
                                length = data.childRecords.length,
                                record = null;
                            for (count; count < length; count++) {
                                record = data.childRecords[count];
                                if (record) {
                                    proxy.refreshGanttRecord(record);
                                }
                            }
                            proxy._updateParentItem(data.childRecords[0]);
                            data._updateParentProgress(data, model.progressMapping);
                        }  
                    }
                }
                proxy.updateIndentOutdentOption(data);
                //update parent recordsCollections
                if (data.level === 0) {
                    if (model.parentRecords.indexOf(data) === -1) {
                        model.parentRecords.push(data);
                    }
                }
                //update predecessor
                if (data.predecessor && data.predecessor.length > 0 && proxy.model.predecessorMapping && data.hasChildRecords)
                {
                    proxy._updatePredecessorOnIndentOutdent(data)
                }
                proxy.refreshGanttRecord(data);
                eveargs.data = data;
                eveargs.requestType = "actionComplete";
                this._trigger("actionComplete", eveargs);
            }
        },
        
        _refreshChildLevel: function (ganttRecord) {

            var proxy = this,
                count = 0,
                childRecords = ganttRecord.childRecords,
                length = childRecords.length,
                record;

            for (count = 0; count < length; count++) {
                record = childRecords[count];                
                proxy.refreshGanttRecord(record);
                if (record.hasChildRecords) {
                    proxy._refreshChildLevel(record);
                }
            }
        },

        _updateChildLevel:function(ganttRecord) {

            var proxy = this,
                count = 0,
                childRecords = ganttRecord.childRecords,
                length = childRecords.length,
                record;
            
            for (count = 0; count < length; count++) {
                record = childRecords[count];                
                record.level = record.level - 1;
               // proxy.refreshGanttRecord(record);
                if (record.hasChildRecords) {
                    proxy._updateChildLevel(record);
                }
            }
        },

        _removeChildRecords:function(ganttRecord,index){

            if (ganttRecord.parentItem) {
                var proxy = this,
                    model = proxy.model,
                    childIndex = 0,
                    record,
                    addedChildGanttRecordCount = 0,
                    length = 0;
                parentChildRecords = ganttRecord.parentItem.childRecords,
                parentChildRecordsLength = parentChildRecords && parentChildRecords.length;

                for (childIndex = index; childIndex < parentChildRecordsLength; childIndex++) {

                    record = parentChildRecords[childIndex];


                    if (record) {

                        if (ganttRecord.childRecords) {

                            length = ganttRecord.childRecords.length;
                            ganttRecord.childRecords.splice(length, 0, record);
                            if (!model.parentTaskIdMapping)
                                ganttRecord.item[model.childMapping].splice(length, 0, record.item);
                            else
                                record.item[model.parentTaskIdMapping] = ganttRecord.taskId;
                        } else {
                            ganttRecord.childRecords = [];
                            ganttRecord.item[model.childMapping] = [];
                            ganttRecord.childRecords.push(record);
                            if (!model.parentTaskIdMapping)
                                ganttRecord.item[model.childMapping].push(record.item);
                            else
                                record.item[model.parentTaskIdMapping] = ganttRecord.taskId;
                        }
                        record.parentItem = ganttRecord;                                             
                        addedChildGanttRecordCount++;
                    }
                }

                if (ganttRecord.parentItem.childRecords) {

                    ganttRecord.parentItem.childRecords.splice(index, addedChildGanttRecordCount);
                    if(!model.parentTaskIdMapping)
                        ganttRecord.parentItem.item[model.childMapping].splice(index, addedChildGanttRecordCount);
                    if (ganttRecord.parentItem.childRecords.length == 0) {
                        ganttRecord.parentItem.childRecords = null;
                        ganttRecord.parentItem.expanded = false;
                        ganttRecord.parentItem.hasChildRecords = false;
                        proxy.refreshGanttRecord(ganttRecord.parentItem);
                    }
                }
            }
        },

        updateChildGanttRecordLevel: function (data,index) {
            var childIndex = 0, proxy = this, model = proxy.model,
                parentChildRecords = data.parentItem&& data.parentItem.parentItem&& data.parentItem.parentItem.childRecords,
                parentChildRecordsLength =parentChildRecords&&parentChildRecords.length,
                addedChildRecordCount = 0;
            for (childIndex = index; childIndex < parentChildRecordsLength; childIndex++) {
                var childGanttRecord = parentChildRecords[childIndex];
                if (childGanttRecord.childRecords) {
                    childGanttRecord.childRecords.splice(childGanttRecord.childRecords.length, 0, childGanttRecord);
                    childGanttRecord.item[model.childMapping].splice(childGanttRecord.childRecords.length, 0, childGanttRecord.item);
                }
                else {
                    childGanttRecord.childRecords = [];
                    childGanttRecord.item[model.childMapping] = [];
                    if (childGanttRecord) {
                        childGanttRecord.childRecords.push(childGanttRecord);
                        childGanttRecord.item[model.childMapping].push(childGanttRecord.item);
                    }
                }
                proxy.refreshGanttRecord(data.parentItem);
                data.parentItem = childGanttRecord;
                ++addedChildGanttRecordCount;
            }
            data.parentItem.childRecords.splice(index, addedChildGanttRecordCount);
            if (data.parentItem.childRecords.length == 0) {
                data.parentItem.childRecords = null;
                data.parentItem.expanded = false;
                data.parentItem.hasChildRecords = false;
            }

            proxy.refreshGanttRecord(data.parentItem);

        },

        updateLevel: function (childGanttRecord,offset) {
            var proxy = this,
                count = 0,
                childRecords = childGanttRecord.childRecords,
                length =childRecords&& childRecords.length,
                ganttRecord;
            for (count; count < length; count++) {
                ganttRecord = childRecords[count];
                ganttRecord.level += offset;
                proxy.refreshGanttRecord(ganttRecord);
                if (ganttRecord.childRecords) 
                    proxy.updateLevel(ganttRecord, offset);                
            }
        },

        addNewGanttRecord:function() {
            var proxy = this, eventArgs = {},newRecord;
            newRecord = this.model.ganttRecords[this.model.ganttRecords.length].item;
            eventArgs.requestType = "save";
            eventArgs._cAddedRecord = jQuery.extend(true, {}, newRecord);
            eventArgs.recordIndex = index + 1;
            proxy.subContextMenuAction(eventArgs);
            proxy._clearContextMenu();
        },
        getExpandStatus: function (record) {

            var parentRecord = record.parentItem;

            if (parentRecord != null) {

                if (parentRecord.expanded === false) {
                    return false;
                } else if (parentRecord.parentItem) {

                    if (parentRecord.parentItem.expanded === false) {
                        return false;
                    } else {
                        return this.getExpandStatus(parentRecord.parentItem);
                    }

                } else return true;
            } else return true;
        },
        subContextMenuAction: function (args) {
            var proxy = this, eventArgs = {}, model = proxy.model, selectionIndex;
            proxy._updatedGanttRecords = proxy.getUpdatedRecords();
            proxy._currentViewData = proxy.getCurrentViewData();
            proxy._gridRows = proxy.getRows();
            proxy._ganttChartRows = proxy.getGanttChartRows();
            if (args.requestType === "save") {

                if (args._cAddedRecord) {

                    eventArgs.selectedItem = model.selectedItem;

                    args._cAddedRecord.predecessor = null;
                    if (proxy.model.predecessorMapping)
                    args._cAddedRecord.item[model.predecessorMapping] = null;//args._cAddedRecord.item.Predecessor

                    eventArgs.data = args._cAddedRecord.item;
                    if (eventArgs.data.duration === 0)
                    {
                        eventArgs.data.duration = 1;
                    }
                    eventArgs.requestType = ej.TreeGrid.Actions.Save;
                    eventArgs.recordIndex = args.recordIndex;

                    eventArgs._cAddedRecord = proxy._createGanttRecord(eventArgs.data, args._cAddedRecord.level, null);//level
                    eventArgs._cAddedRecord.taskId = proxy._getNewTaskId(eventArgs._cAddedRecord.taskId);
                    var index = proxy._ids.indexOf(eventArgs._cAddedRecord.taskId.toString());
                    proxy._ids.splice(index,1);

                    if(model.taskIdMapping)
                    eventArgs._cAddedRecord.item[model.taskIdMapping] = eventArgs._cAddedRecord.taskId;
                    eventArgs._cAddedRecord.taskName = "New Task " + eventArgs._cAddedRecord.taskId;
                    if (model.taskNameMapping)
                    eventArgs._cAddedRecord.item[model.taskNameMapping] = "New Task " + eventArgs._cAddedRecord.taskId;
                    eventArgs._cAddedRecord.item[model.childMapping] = null;

                    if (args._cAddedRecord.hasChildRecords && args.position==="Below" ) { 
                        eventArgs._cAddedRecord.parentItem = model.selectedItem;
                        eventArgs._cAddedRecord.level = model.selectedItem.level + 1;
                        eventArgs._cAddedRecord.expanded = false;
                    }
                    else if (args._cAddedRecord.hasChildRecords && args.position === "Above") {

                        if (args._cAddedRecord.parentItem)
                        {
                            eventArgs._cAddedRecord.parentItem = args._cAddedRecord.parentItem;
                            eventArgs._cAddedRecord.level = args._cAddedRecord.parentItem.level + 1;
                            eventArgs._cAddedRecord.expanded = false;
                        }
                        else
                        {
                            eventArgs._cAddedRecord.parentItem = null;
                            eventArgs._cAddedRecord.expanded = false;
                        }
                        
                    }
                    else if (!args._cAddedRecord.hasChildRecords && args._cAddedRecord.parentItem) {
                        eventArgs._cAddedRecord.parentItem = args._cAddedRecord.parentItem;
                        eventArgs._cAddedRecord.level = args._cAddedRecord.parentItem.level + 1;
                        eventArgs._cAddedRecord.expanded = false;
                    }
                    else
                    {
                        eventArgs._cAddedRecord.parentItem = null;

                    }
                    //if (eventArgs._cAddedRecord.parentItem && !model.enableVirtualization)
                    //{
                    //    eventArgs._cAddedRecord.isExpanded = eventArgs._cAddedRecord.parentItem.expanded;
                    //}




                    //if (args._cAddedRecord.parentItem) {
                    //    eventArgs._cAddedRecord.parentItem = args._cAddedRecord.parentItem;
                    //    eventArgs._cAddedRecord.parentItem.childRecords.push(eventArgs._cAddedRecord);
                    //}
                    eventArgs._cAddedRecord.hasChildRecords = false;
                    eventArgs._cAddedRecord.childRecords = null;
                    eventArgs._cAddedRecord.index = args.recordIndex;
                    eventArgs._cAddedRecord.isAltRow = (eventArgs._cAddedRecord.index+1) % 2 == 0 ? false : true;
                    eventArgs.actionRequestType = ej.TreeGrid.Actions.ContextMenuAdd;
                    eventArgs.position = args.position;

                    if (!model.enableVirtualization) {
                        eventArgs._cAddedRecord.isExpanded = proxy.getExpandStatus(eventArgs._cAddedRecord);
                        if (!eventArgs._cAddedRecord.isExpanded) {
                            proxy._totalCollapseRecordCount += 1;
                        }
                    }

                    proxy.actionComplete(eventArgs);

                    //proxy._$treegridHelper.ejTreeGrid("renderInitialRecord", eventArgs);
                    //proxy._$treegridHelper.ejTreeGrid("renderRecords", eventArgs);

                    //proxy._$ganttchartHelper.ejGanttChart("reRenderGanttGridLinesTable", 1);
                }
            }
            //proxy._connectorlineIds = [];
            //proxy._connectorLinesCollection = [];
            //proxy._$ganttchartHelper.ejGanttChart("clearConnectorLines");
            //proxy._createConnectorLinesCollection();
            //proxy._ganttActionCompleteTrigger(args);
            selectionIndex = proxy._currentViewData.indexOf(model.selectedItem);
            //if (args.recordIndex != 0 && eventArgs._cAddedRecord.isExpanded)
            //    selectionIndex = args.recordIndex + 1;
            //else
            //    selectionIndex = args.recordIndex;
            proxy._$treegridHelper.ejTreeGrid("selectRows", selectionIndex);
            proxy._$ganttchartHelper.ejGanttChart("selectRows", selectionIndex);
        },

        _getNewTaskId: function (taskId) {
            var proxy = this, tempTaskId;
            if (proxy._ids.indexOf(taskId.toString()) == -1) {
                              proxy._ids.push(taskId.toString());
                                return taskId;
            } else {
                taskId++;
                tempTaskId = proxy._getNewTaskId(taskId);
                return tempTaskId;
            }
        },

        _getNewTaskIdFordialogEdit: function (taskId) {
            var proxy = this, tempTaskId;
            if (proxy._ids.indexOf(taskId.toString()) == -1) {              
                return taskId;
            } else {
                taskId++;
                tempTaskId = proxy._getNewTaskIdFordialogEdit(taskId);
                return tempTaskId;
            }
        },
        _updateNewTaskId: function () {
            var proxy = this, model = proxy.model;
            proxy._ids = [];
            $.each(proxy._rows, function (index, value) {
                if (value && value.taskId)
                    proxy._ids.push(value.taskId.toString());
            });
        },
        contextMenuAction: function (args) {
            var proxy = this;
            if (args.requestType == "contextMenuAdd")
                proxy._sendAddRequest();
            if (args.requestType == "contextMenuIndent")
                proxy._sendIndentRequest();
            if (args.requestType == "contextMenuOutdent")
                proxy._sendOutdentRequest();
            if (args.requestType == "contextMenuDelete")
                proxy._$treegridHelper.ejTreeGrid("deleteRow");
            if (args.requestType == "contextMenuTaskDetails")
                proxy._sendEditRequest();
        },

        _renderContextMenu: function (e, index, item) {

            var proxy = this,
                eventArgs = {},
                args = {},
                posx,
                posy,
                contextMenu,
                contextMenuUList,
                subContextMenu,
                subContextMenuUList;
            //Initialize the contextmenu item to avoid duplicates
            //proxy._contextMenuItems = proxy._getContextMenuItems();
            //proxy._subContextMenuItems = proxy._getsubContextMenuItems();
            if (!e) e = window.event;
            if (e.pageX || e.pageY) {
                posx = e.pageX;
                posy = e.pageY;
            } else if (e.clientX || e.clientY) {
                posx = e.clientX + document.body.scrollLeft
                    + document.documentElement.scrollLeft;
                posy = e.clientY + document.body.scrollTop
                    + document.documentElement.scrollTop;
            }

            args.requestType = "ContextMenuOpen";
            args.contextMenuItems = proxy._contextMenuItems;
            args.innerContextMenuItems = proxy._subContextMenuItems;
            proxy._trigger("contextMenuOpen", args);
            contextMenu = ej.buildTag("div.e-contextmenu", "", {
                'height': 'auto',
                'width': '150px',
                'position': 'absolute',
                'left': posx + 'px',
                'top': posy + 'px',
                'background-color': '#F0F0F0',
                'border': '1px solid #99bce8',
                'z-index': '10',
            }, {});

            contextMenuUList = ej.buildTag("ul", $.render[proxy._id + "contextMenuTemplate"](args.contextMenuItems), {
                'height': 'auto',
                'width': 'auto',
                'top': '0px',
                'display': 'inline'
            }, {
                'data-icon': false,
                'type': 'none',
                'data-role': 'list-divider'
            });

            subContextMenu = ej.buildTag("div.e-innerContextmenu", "", {
                'height': 'auto',
                'width': '120px',
                'position': 'absolute',
                'left': (posx + 152) + 'px',
                'top': (posy + 30) + 'px',
                'background-color': '#F0F0F0',
                'border': '1px solid #99bce8',
                'z-index': '10',
                'visibility': 'hidden',
            }, {});

            subContextMenuUList = ej.buildTag("ul", $.render[proxy._id + "subContextMenuTemplate"](args.innerContextMenuItems), {
                'height': 'auto',
                'width': 'auto',
                'top': '0px',
                'display': 'inline'
            }, {
                'data-icon': false,
                'type': 'none',
                'data-role': 'list-divider'
            });
            contextMenu.append(contextMenuUList);
            $(document.body).append(contextMenu);

            if (args.innerContextMenuItems.length > 0) {
                subContextMenu.append(subContextMenuUList);
                $(document.body).append(subContextMenu);
            }
         


            $(contextMenu).css({ 'height': 'auto' });
            proxy._on(contextMenu, "contextmenu", function (argsE) {
                argsE.preventDefault();
            });

            //click event handlers for contextmenu items and subcontextmenu items

                $('.e-menuitem').click(function () { 
                    var choice = $(this).attr('id');
                    switch (choice) {
                        case "Task":
                            eventArgs.requestType = "contextMenuTaskDetails";
                            proxy.contextMenuAction(eventArgs);
                            proxy._clearContextMenu();
                            break;
                        case "Indent": eventArgs.requestType = "contextMenuIndent";
                            proxy.contextMenuAction(eventArgs);
                            proxy._clearContextMenu();
                            break;
                        case "Outdent": eventArgs.requestType = "contextMenuOutdent";
                            proxy.contextMenuAction(eventArgs);
                            proxy._clearContextMenu();
                            break;
                        case "Delete": eventArgs.requestType = "contextMenuDelete";
                            proxy.contextMenuAction(eventArgs);
                            proxy._clearContextMenu();
                            break;
                        case "Above": eventArgs.requestType = "save";
                            eventArgs._cAddedRecord = jQuery.extend(true, {}, item);
                            eventArgs.recordIndex = index == 0 ? 0 : index;
                            eventArgs.position = "Above";
                            proxy.subContextMenuAction(eventArgs);
                            proxy._clearContextMenu();
                            break;
                        case "Below": eventArgs.requestType = "save";
                            eventArgs._cAddedRecord = jQuery.extend(true, {}, item);
                            eventArgs.recordIndex = index;
                            eventArgs.position = "Below";
                            proxy.subContextMenuAction(eventArgs);
                            proxy._clearContextMenu();
                            break;
                        case "Open":
                            eventArgs.requestType = "contextMenuAdd";
                            proxy.contextMenuAction(eventArgs);
                            proxy._clearContextMenu();
                            break;
                        case "Add":
                            proxy._setInitialData();
                            break;
                        default:
                            proxy._clearContextMenu();
                            break;
                    }
                });
            //Mouseenter and mouse leave handler for contextmenu items and Subcontextmenuitems
                $(".e-menuitem").mouseenter(function () {
                    $(this).addClass("e-contextmenu-mouseover");
                    if ($(this).attr('id') == "Add" || $(this).attr('id') == "Above" || $(this).attr('id') == "Below" || $(this).attr('id') == "Open") {
                        $('.e-innerContextmenu').css({ 'visibility': 'visible' });
                        $('#Add').addClass("e-contextmenu-mouseover");
                    }
                    else {
                        $('.e-innerContextmenu').css({ 'visibility': 'hidden' });
                    }
                });
                $(".e-menuitem").mouseleave(function () {
                    $(this).removeClass("e-contextmenu-mouseover");
                });
            e.preventDefault();
        },

        _setInitialData: function () {
            var eventArgs = {}, proxy = this,
                model = proxy.model, data = [], item, initialData = {};
            proxy._ids = [];
            initialData[model.startDateMapping] = model.scheduleStartDate ? model.scheduleStartDate : null;
            initialData[model.durationMapping] = 5;
            initialData[model.taskNameMapping] = "New Task 1";
            initialData[model.taskIdMapping] = "1";
            data.push(initialData);

            if (proxy.dataSource()==null || proxy.dataSource().length == 0) {
                //proxy._createGanttRecords(data);
                item = proxy._createGanttRecord(data[0], 0, null);
                item.index = 0;
                eventArgs.requestType = "save";
                eventArgs._cAddedRecord = jQuery.extend(true, {}, item);
                eventArgs.recordIndex = 0;
                proxy.actionComplete(eventArgs);
                proxy._clearContextMenu();
            }
        },

        sendExpandCollapseRequest: function (args) {
            
            var proxy = this,
                ganttRecord = args.data,
                model = proxy.model,
                index = proxy._currentViewData.indexOf(ganttRecord),
                $row = $($(proxy._gridRows)[index]),
                height = 0,
                isModified = ganttRecord.expanded !== args.expanded;
           
            if (args.expanded) {
                ganttRecord.expanded = args.expanded;
                proxy._expandedRecordsCount = 0;
                proxy.expandGanttRecord(ganttRecord, args.expanded, args);
                if (isModified)
                    proxy._totalCollapseRecordCount -= proxy._expandedRecordsCount;
                //$row.find(".e-treegridcollapse").removeClass('e-treegridcollapse').addClass('e-treegridexpand');
                //$row.removeClass('e-treegridrowcollapse').addClass('e-treegridrowexpand');                
                if (model.enableAltRow)
                    proxy.updateAltRow(ganttRecord, index, 1);
                
            }
            else {
                proxy._collapsedRecordCount = 0;
                proxy.collapseGanttRecord(ganttRecord, args.expanded);
                if (isModified)
                    proxy._totalCollapseRecordCount += proxy._collapsedRecordCount;
                ganttRecord.expanded = args.expanded;
                $row.find(".e-treegridexpand").removeClass('e-treegridexpand').addClass('e-treegridcollapse');
                $row.removeClass('e-treegridrowexpand').addClass('e-treegridrowcollapse');               
                    
                if (model.enableAltRow)
                    proxy.updateAltRow(ganttRecord, index, proxy._collapsedRecordCount + 1);                
            }
            height = (proxy._updatedGanttRecords.length - proxy._totalCollapseRecordCount) * model.rowHeight;
            proxy._$ganttchart.ejGanttChart("updateHeight", height);
            if (args.expanded)
                proxy._trigger("expanded", args);
            else
                proxy._trigger("collapsed", args);
        },

        expandGanttRecord: function (ganttRecord, expanded, args) {
            
            var proxy = this,
                rowClassName = ".gridrowtaskId" + ganttRecord.taskId.toString() + "level" + (ganttRecord.level + 1).toString(),
                $rows = $(rowClassName),
                count = 0,
                length = $rows.length,
                row,
                index,
                record,
                $expandedRows = $(rowClassName),
                index = proxy._currentViewData.indexOf(ganttRecord),
                $row = $($(proxy._gridRows)[index]);

            if (ganttRecord.hasChildRecords && ganttRecord.childRecords.length > 0) {
                $row.find(".e-treegridcollapse").removeClass('e-treegridcollapse').addClass('e-treegridexpand');
                $row.removeClass('e-treegridrowcollapse').addClass('e-treegridrowexpand');
            }
          
            proxy._expandedRecordsCount += $expandedRows.length / 2;
            if ($expandedRows.length === 0) {
                if (proxy.model.allowSearching && proxy._searchString.length>0) {
                    args && proxy._$treegridHelper.ejTreeGrid("sendExpandCollapseRequest", args);
                }
            }

            $expandedRows.css({ 'display': 'table-row' });
            proxy.updateConnectorLines($expandedRows, expanded);
            for (count; count < length / 2; count++) {
                row = $rows[count];
                index = $(proxy._gridRows).index(row);
                if (index !== -1) {
                    record = proxy._updatedGanttRecords[index];
                    record.isExpanded = true;
                    if (record.expanded)
                        proxy.expandGanttRecord(record, expanded);
                }
            }
        },

        updateConnectorLines: function ($gridRows, expanded) {
            var proxy = this,
                length = $gridRows.length / 2,
                count = 0,
                row,
                index,
                record;
            
            for (count = 0; count < length; count++) {
                row = $gridRows[count];
                index = $(proxy._gridRows).index(row);
                if (index !== -1) {
                    record = proxy._currentViewData[index];
                    record.isExpanded = expanded;                  
                }
            }
            proxy._connectorlineIds = [];
            proxy._connectorLinesCollection = [];
            proxy._$ganttchartHelper.ejGanttChart("clearConnectorLines");
            proxy._createConnectorLinesCollection();            
        },

        collapseGanttRecord: function (ganttRecord, expanded) {
            var proxy = this,
                rowClassName = ".gridrowtaskId" + ganttRecord.taskId.toString() + "level" + (ganttRecord.level + 1).toString(),
                $rows = $(rowClassName),
                count = 0,
                length = $rows.length,
                row,
                index,
                record,
                $collapsedRows = $(rowClassName);
            proxy._collapsedRecordCount += $collapsedRows.length / 2;
            $collapsedRows.css({ 'display': 'none' });

            proxy.updateConnectorLines($collapsedRows, expanded);

            for (count; count < length / 2; count++) {
                row = $rows[count];
                index = $(proxy._gridRows).index(row);
                //if (index === -1) {
                //    index = $(proxy._ganttChartRows).index($rows[count + $collapsedRows.length / 2]);
                //    $(row).addClass('e-hide');
                //}
                if (index !== -1) {
                    record = proxy.getUpdatedRecords()[index];
                    if (record.expanded)
                        proxy.collapseGanttRecord(record, expanded);
                }
            }
        },

        updateAltRow: function (ganttRecord, recordIndex, offset) {
            var proxy = this,
                count = 0,
                currentViewData = proxy._currentViewData,
                length = currentViewData.length,
                isAltRow = ganttRecord.isAltRow,
                record,
                $gridRows = $(proxy._gridRows);
            for (count = recordIndex + offset; count < length; count++) {
                if ($($gridRows[count]).css('display') === "none")
                    continue;
                record = currentViewData[count];
                record.isAltRow = !isAltRow;
                isAltRow = record.isAltRow;
                if (isAltRow)
                    $($gridRows[count]).addClass('e-alt-row');
                else
                    $($gridRows[count]).removeClass('e-alt-row');
            }
        },
        renderedEditDialog: function (args) {
            var proxy = this;
            switch (args.requestType) {
                case ej.TreeGrid.Actions.Add:
                    var temp = document.createElement('div');
                    $(temp).addClass("e-addedrow");
                    temp.innerHTML = $.render[proxy._id + "_JSONDialogAddingTemplate"](args.data);
                    $("#" + proxy._id + "_dialogAdd").html($(temp));
                    var evntArgs = {};
                    evntArgs.cssClass = this.model.cssClass,
                    evntArgs.enableModal = true,
                    evntArgs.width = "auto",
                    evntArgs.enableResize = false,
                    evntArgs.contentSelector = "#" + proxy._id,
                    evntArgs.rtl = proxy.model.rtl,
                    evntArgs.allowKeyboardNavigation = false,
                    title = proxy._editDialogTexts["addFormTitle"];
                    evntArgs.title = title;
                    $("#" + proxy._id + "_dialogAdd").ejDialog(evntArgs);
                    $("#" + proxy._id + "_dialogAdd").ejDialog("open");
                    proxy._editedDialogRecord = null;
                    break;
                case ej.TreeGrid.Actions.BeginEdit:
                    temp = document.createElement('div');
                    $(temp).addClass("e-editedrow");
                    temp.innerHTML = $.render[proxy._id + "_JSONDialogEditingTemplate"](args.data);
                    $("#" + proxy._id + "_dialogEdit").html($(temp));
                    var evntArgs = {};
                    evntArgs.cssClass = proxy.model.cssClass;
                    evntArgs.rtl = model.rtl;
                    evntArgs.width = "auto";
                    evntArgs.enableResize = false;
                    evntArgs.contentSelector = "#" + proxy._id;
                    evntArgs.enableModal = true;
                    evntArgs.allowKeyboardNavigation = false;
                    var title = proxy._editDialogTexts["editFormTitle"] + " " + proxy.selectedItem()["taskName"];
                    evntArgs.title = title;
                    $("#" + proxy._id + "_dialogEdit").ejDialog(evntArgs);
                    $("#" + proxy._id + "_dialogEdit").ejDialog("open");
                    proxy._editedDialogRecord=jQuery.extend(true, {}, this.selectedItem());
                    break;                    
            }
            proxy._refreshEditForm(args);
            //proxy.setValidation();
        },

        editFormValidate: function () {
            if ($.isFunction($.validator))
                return $("#" + this._id + "EditForm").validate().form();
            return true;
        },

        addFormValidate: function () {
            if ($.isFunction($.validator))
                return $("#" + this._id + "AddForm").validate().form();
            return true;
        },
        getCurrentData: function () {
            var proxy = this,
                updatedRecords = proxy.getUpdatedRecords();
            return updatedRecords && updatedRecords[this.selectedRowIndex()];
        },

        getCurrentViewData: function () {
            return this._$treegridHelper.ejTreeGrid("getCurrentViewData");
        },

        getRows: function () {
            return this._$treegridHelper.ejTreeGrid("getRows");
        },
        
        getGanttChartRows: function() {
            return this._$ganttchartHelper.ejGanttChart("getGanttChartRows");
        },

        getUpdatedRecords: function () {
            return this._$treegridHelper.ejTreeGrid("getUpdatedRecords");
        },

        _getToolBarHeight: function () {
            var toolbarH = 0;
            if (this.model.toolbarSettings.showToolbar) {
                toolbarH = $(".e-gantttoolbar").height();
            }
            return toolbarH;
        },

        _initValidator: function () {
            var gridObject = this;
            $("#" + this._id + "EditForm").validate({
                errorClass: 'e-field-validation-error',
                errorElement: 'div',
                wrapper: "div",
                errorPlacement: function (error, element) {
                    var $td = element.closest("td"), $container = $(error).addClass("e-error")
                    , $tail = ej.buildTag("div.e-errortail e-toparrow");
                    $td.find(".e-error").remove();
                    element.hasClass("e-numerictextbox") ? $container.insertAfter(element.closest(".e-numeric")) : $container.insertAfter(element);
                    $container.prepend($tail);
                    gridObject.model.editSettings.editMode != "normal" && $container.offset({ left: element.offset().left });
                    $container.fadeIn("slow");
                },

            });
        },

        setValidation: function () {
            this._initValidator();
            for (var i = 0; i < this._columns.length; i++) {
                if (!ej.isNullOrUndefined(this._columns[i]["validationRules"])) {
                    this.setValidationToField(this._columns[i].field, this._columns[i].validationRules);
                }
            }
        },

        setValidationToField: function (name, rules) {
            var ele = $("#" + this._id + name).length > 0 ? $("#" + this._id + name) : $("#" + name);
            ele.rules("add", rules);
            var validator = $("#" + this._id + "EditForm").validate();
            if (!ej.isNullOrUndefined(rules["required"])) {
                validator.settings.messages["Name"] = {};
                validator.settings.messages["Name"]["required"] = ej.TreeGrid.getColumnByField(proxy._columns, name).headerText + " is required";
            }
        },

        _createConnectorLinesCollection: function (args) {

            var proxy = this,
                recordLength = proxy._currentViewData.length,
                count,
                prdecessorIndex = -1,
                predecessorsCollection,
                ganttRecord,
                connectorLinesCollection = proxy._predecessorsCollection,
                connectorLinesLength = connectorLinesCollection.length,
                x = connectorLinesLength / 2,
                connectorLinesCount,
                enableVirtualization = proxy.model.enableVirtualization,
                record;


            if (model.enableVirtualization && args === "Load") {
                //THis is done for validate all the record during load time
                recordLength = proxy._updatedGanttRecords.length;
                for (count = 0; count < recordLength; count++) {
                    ganttRecord = proxy._updatedGanttRecords[count];
                        predecessorsCollection = ganttRecord["predecessor"];
                        if (predecessorsCollection) {
                            record = ganttRecord;
                            proxy._addPredecessorsCollection(ganttRecord, predecessorsCollection);
                        }
                }
            }
            else {

                for (count = 0; count < recordLength; count++) {
                    ganttRecord = proxy._currentViewData[count];
                    if (enableVirtualization === false && ganttRecord.isExpanded === false)
                        continue;
                    if (proxy._currentViewData.indexOf(ganttRecord) !== -1) {
                        predecessorsCollection = ganttRecord["predecessor"];
                        if (predecessorsCollection) {
                            record = ganttRecord;
                            proxy._addPredecessorsCollection(ganttRecord, predecessorsCollection);
                        }
                    }
                }

            }

            if (proxy._connectorLinesCollection.length > 0) {

                proxy._$ganttchartHelper && proxy._$ganttchartHelper.ejGanttChart("renderConnectorLines", proxy._connectorLinesCollection);

            }
        },

        _addPredecessorsCollection: function (ganttRecord, predecessorsCollection) {
            var proxy = this,
                predecessorsLength,
                predecessorCount,
                predecessor,
                parentGanttRecord,
                childGanttRecord,
                model = proxy.model,
                connectorObj = {},
                predecessorValue = [];
            if (predecessorsCollection) {

                predecessorsLength = predecessorsCollection.length;

                for (predecessorCount = 0; predecessorCount < predecessorsLength; predecessorCount++) {

                    predecessor = predecessorsCollection[predecessorCount];


                        parentGanttRecord = proxy._rows[proxy._ids.indexOf(predecessor.from)];  
                        childGanttRecord = proxy._rows[proxy._ids.indexOf(predecessor.to)];  

                        if (!model.enableVirtualization) {
                            if ((parentGanttRecord && parentGanttRecord.isExpanded === false) || 
                                (childGanttRecord && childGanttRecord.isExpanded === false))
                                continue;
                        }

                        if (proxy._isValidationEnabled && !proxy._isInExpandCollapse && childGanttRecord)
                            proxy._validateChildGanttRecord(parentGanttRecord, childGanttRecord, predecessor);
                }
				// for correct re draw the alreadey created line in multiple predecessor for same task
                var count = 0;
                 $.each(predecessorsCollection, function (inx, value) {

                    if (value.to === ganttRecord.taskId.toString()) {
                        count++;
                    }
                    return count;
                });
				
				
				
                    for (predecessorCount = 0; predecessorCount < predecessorsLength; predecessorCount++) {

                        predecessor = predecessorsCollection[predecessorCount];
						if (count > 1)
                        {
                            predecessor.isdrawn = false;
                        }
						
                        if (predecessor.isdrawn) {
                            predecessor.isdrawn = false;
                        }
                        else {

                            parentGanttRecord = proxy._rows[proxy._ids.indexOf(predecessor.from)];
                            childGanttRecord = proxy._rows[proxy._ids.indexOf(predecessor.to)];

                            if (proxy._updatedGanttRecords.indexOf(parentGanttRecord) === -1 || proxy._updatedGanttRecords.indexOf(childGanttRecord) == -1)
                                continue;

                            if (!model.enableVirtualization) {
                                if (parentGanttRecord.isExpanded === false || childGanttRecord.isExpanded === false)
                                    continue;
                            }

                            //if (proxy._isValidationEnabled && !proxy._isInExpandCollapse)
                            //    proxy._validateChildGanttRecord(parentGanttRecord, childGanttRecord, predecessor);

                        if (parentGanttRecord && childGanttRecord) {

                            connectorObj = proxy._createConnectorLineObject(parentGanttRecord, childGanttRecord, predecessor);
                          
                            if (connectorObj) {

                                if (proxy._connectorlineIds.length > 0 && proxy._connectorlineIds.indexOf(connectorObj.ConnectorLineId) == -1) {

                                    proxy._connectorLinesCollection.push(connectorObj);
                                    proxy._connectorlineIds.push(connectorObj.ConnectorLineId);
                                }
                                else if (proxy._connectorlineIds.length == 0) {
                                    proxy._connectorLinesCollection.push(connectorObj);
                                    proxy._connectorlineIds.push(connectorObj.ConnectorLineId);
                                }
                                else if (proxy._connectorlineIds.indexOf(connectorObj.ConnectorLineId) != -1) {
                                    var index = proxy._connectorlineIds.indexOf(connectorObj.ConnectorLineId);
                                    proxy._connectorLinesCollection[index] = connectorObj;
                                }
                                predecessor.isdrawn = true;
                            }
                        }
                    }
                }

            }
          
        },

        _createConnectorLineObject: function (parentGanttRecord, childGanttRecord, predecessor) {
            var connectorObj = {},
                proxy = this,
                model = proxy.model,
                parentTop,
                childTop,
                parentIndex = proxy._updatedGanttRecords.indexOf(parentGanttRecord),
                childIndex = proxy._updatedGanttRecords.indexOf(childGanttRecord),
                count = 0,
                index = 0;
            
            if (parentIndex === -1 || childIndex === -1){
                return;
            }
            if (!model.enableVirtualization) {
                var collapsedCount = 0;
                if (childIndex > parentIndex) {
                    
                    for (count = 0; count < parentIndex; count++) {
                        if (proxy._updatedGanttRecords[count].isExpanded===false)
                            ++collapsedCount;
                    }
                    index = parentIndex;
                    parentIndex -= collapsedCount;
                    
                    for (index; index < childIndex; index++) {
                        if (proxy._updatedGanttRecords[index].isExpanded === false)
                            ++collapsedCount;
                    }
                    childIndex -= collapsedCount;
                }
                else if (parentIndex > childIndex) {
                    for (count = 0; count < childIndex; count++) {
                        if (proxy._updatedGanttRecords[count].isExpanded === false)
                            ++collapsedCount;
                    }
                    index = childIndex;
                    childIndex -= collapsedCount;
                    
                    for (index ; index < parentIndex; index++) {
                        if (proxy._updatedGanttRecords[index].isExpanded === false)
                            ++collapsedCount;
                    }
                    parentIndex -= collapsedCount;
                }
            }

            if (parentIndex != -1 && childIndex != -1) {
                parentTop = parentIndex * model.rowHeight;
                childTop = childIndex * model.rowHeight;
                connectorObj["ParentLeft"] = parentGanttRecord['left'];
                connectorObj["ChildLeft"] = childGanttRecord['left'];
                connectorObj["ParentWidth"] = parentGanttRecord['width'] == 0 ? 26 : parentGanttRecord['width'];
                connectorObj["ChildWidth"] = childGanttRecord['width'] == 0 ? 26 : childGanttRecord['width'];
                connectorObj["ParentTop"] = parentTop;
                connectorObj["ChildTop"] = childTop;
                connectorObj["Height"] = parentTop > childTop ? parentTop - childTop : childTop - parentTop;
                connectorObj["Type"] = predecessor.predecessorsType;
                connectorObj["ConnectorLineId"] = "parent"+ parentGanttRecord['taskId'] + "child" + childGanttRecord['taskId'];                
                connectorObj["milestoneParent"] = parentGanttRecord.isMilestone ? true : false;
                connectorObj["milestoneChild"] = childGanttRecord.isMilestone ? true : false;
                
                return connectorObj;
            }
        },

        //Validate the child Gantt Record

        _validateChildGanttRecord: function (parentGanttRecord, childGanttRecord, predecessor) {
            var proxy = this,
                model = proxy.model,
                childStartDate,
                parentStartDate,
                childEndDate,
                parentEndDate,
                offset,holiday
                predecessorOffset = 0;
                predecessorOffset = predecessor.offset?parseInt(predecessor.offset):0;

                if (proxy._editedTaskBarItem === childGanttRecord)
                {
                    return;
                }
            switch (predecessor.predecessorsType) {

                case 'SS':

                    parentStartDate = proxy._getDateFromFormat(parentGanttRecord.startDate);
                    childStartDate = proxy._getDateFromFormat(childGanttRecord.startDate);

                        if (childGanttRecord.isMilestone) {
                            parentStartDate.setDate(parentStartDate.getDate() - 1);
                        }

                        if (proxy.model.includeWeekend === false) {
                            proxy._setBeforeWeekend(parentStartDate);
                        }

                        if (predecessorOffset) {

                            parentStartDate.setDate(parentStartDate.getDate() + (Math.floor(predecessorOffset)));

                            if (proxy.model.includeWeekend === false) {

                                if (predecessorOffset < 0) {
                                    proxy._setBeforeWeekend(parentStartDate);
                                }

                                if (predecessorOffset > 0) {
                                    proxy._setAfterWeekend(parentStartDate);
                                }
                            }
                        }

                        if (proxy._stringHolidays &&
                          proxy._stringHolidays.indexOf(childGanttRecord._getFormatedDate(parentStartDate, model.dateFormat)) != -1) {
                            parentStartDate.setDate(parentStartDate.getDate() + 1);
                        }

                        childGanttRecord.startDate = childGanttRecord._getFormatedDate(parentStartDate, model.dateFormat);

                            var currentTaskId = childGanttRecord['taskId'].toString(),
                                 predecessorsCollection = childGanttRecord['predecessor'],
                               childPredecessor = predecessorsCollection.filter(function (data) { return data.to === currentTaskId });

                            if (childPredecessor.length > 1) {

                                var minStartDate = proxy._getPredecessorDate(childGanttRecord, childPredecessor, predecessor.predecessorsType);


                                if (childGanttRecord.isMilestone) {
                                    minStartDate.setDate(minStartDate.getDate() - 1);
                                }

                                if (model.includeWeekend === false) {
                                    proxy._setBeforeWeekend(minStartDate);
                                }
                                childGanttRecord.startDate = childGanttRecord._getFormatedDate(minStartDate, model.dateFormat);

                            }
                            childGanttRecord.left = childGanttRecord._calculateLeft(proxy._projectStartDate,
                                    proxy._perDayWidth, proxy._holidays, model);
                        childGanttRecord.width = childGanttRecord._calculateWidth(proxy._perDayWidth, proxy._holidays, model);
                    break;

                case 'SF':

                    parentStartDate = proxy._getDateFromFormat(parentGanttRecord.startDate);
                    childEndDate = proxy._getDateFromFormat(childGanttRecord.endDate);                    

                        parentStartDate.setDate(parentStartDate.getDate() - 1);

                        if (proxy.model.includeWeekend === false) {
                            proxy._setBeforeWeekend(parentStartDate);
                        }

                        if (predecessorOffset) {

                            parentStartDate.setDate(parentStartDate.getDate() + (Math.floor(predecessorOffset)));

                            if (proxy.model.includeWeekend === false) {

                                if (predecessorOffset < 0) {
                                    proxy._setBeforeWeekend(parentStartDate);
                                }

                                if (predecessorOffset > 0) {
                                    proxy._setAfterWeekend(parentStartDate);
                                }
                            }
                        }


                        if (proxy._stringHolidays &&
                            proxy._stringHolidays.indexOf(childGanttRecord._getFormatedDate(parentStartDate, model.dateFormat)) != -1) {
                            parentStartDate.setDate(parentStartDate.getDate() + 1);
                        }

                        childGanttRecord.endDate = childGanttRecord._getFormatedDate(parentStartDate, model.dateFormat);

                        if (childGanttRecord.isMilestone)
                            childGanttRecord.startDate = childGanttRecord._getFormatedDate(parentStartDate, model.dateFormat);
                        else
                        {
                            var tempEndDate = proxy._getDateFromFormat(childGanttRecord.endDate);
                            for(var i=0;i<childGanttRecord.duration-1;i++)
                            {
                                tempEndDate.setDate(tempEndDate.getDate() - 1);
                                if (proxy.model.includeWeekend === false) {
                                    if (predecessorOffset < 0) {
                                        proxy._setBeforeWeekend(tempEndDate);
                                    }
                                }
                                tempEndDate=proxy._onHoliDayCheckBefore(tempEndDate);
                            }
                            childGanttRecord.startDate = childGanttRecord._getFormatedDate(tempEndDate, model.dateFormat);
                        }
                            var currentTaskId = childGanttRecord['taskId'].toString(),
                                       predecessorsCollection = childGanttRecord['predecessor'],
                                     childPredecessor = predecessorsCollection.filter(function (data) { return data.to === currentTaskId });

                            if (childPredecessor.length > 0) {

                                var minStartDate = proxy._getPredecessorDate(childGanttRecord, childPredecessor, predecessor.predecessorsType);

                                minStartDate.setDate(minStartDate.getDate() - 1);

                                if (proxy.model.includeWeekend === false) {
                                    proxy._setBeforeWeekend(minStartDate);
                                }

                                childGanttRecord.endDate = childGanttRecord._getFormatedDate(minStartDate, model.dateFormat);
                                if (childGanttRecord.isMilestone)
                                    childGanttRecord.startDate = childGanttRecord._getFormatedDate(minStartDate, model.dateFormat);
                                else {
                                    var tempEndDate = proxy._getDateFromFormat(childGanttRecord.endDate);
                                    for (var i = 0; i < childGanttRecord.duration - 1; i++) {
                                        tempEndDate.setDate(tempEndDate.getDate() - 1);
                                        if (proxy.model.includeWeekend === false) {
                                            if (predecessorOffset < 0) {
                                                proxy._setBeforeWeekend(tempEndDate);
                                            }
                                        }
                                        tempEndDate = proxy._onHoliDayCheckBefore(tempEndDate);
                                    }
                                    childGanttRecord.startDate = childGanttRecord._getFormatedDate(tempEndDate, model.dateFormat);
                                }
                            }
                            childGanttRecord.left = childGanttRecord._calculateLeft(proxy._projectStartDate, proxy._perDayWidth, proxy._holidays, model);
                        childGanttRecord.width = childGanttRecord._calculateWidth(proxy._perDayWidth, proxy._holidays, model);

                    break;

                case 'FS':

                    parentEndDate = proxy._getDateFromFormat(parentGanttRecord.endDate);
                    childStartDate = proxy._getDateFromFormat(childGanttRecord.startDate);

                        if (!childGanttRecord.isMilestone) {
                            parentEndDate.setDate(parentEndDate.getDate() + 1);
                        }

                        if (predecessorOffset < 0) {
                            predecessorOffset *= -1;
                            offset = -1;
                            if (model.includeWeekend === false) {
                                proxy._setBeforeWeekend(parentEndDate);
                            }
                        } else {

                            if (model.includeWeekend === false) {
                                proxy._setAfterWeekend(parentEndDate);
                            }
                            offset = 1;
                        }


                        if (predecessorOffset) {

                            for (var i = 0; i < predecessorOffset; i++) {

                                if (proxy.model.includeWeekend === false) {
                                    if (offset < 0) {
                                        proxy._setBeforeWeekend(parentEndDate);
                                    }

                                    if (offset > 0) {
                                        proxy._setAfterWeekend(parentEndDate);
                                    }
                                }
                                parentEndDate.setDate(parentEndDate.getDate() + offset);
                            }

                            if (proxy.model.includeWeekend === false) {
                                proxy._setAfterWeekend(parentEndDate);
                            }
                        }

                        if (proxy._stringHolidays &&
                            proxy._stringHolidays.indexOf(childGanttRecord._getFormatedDate(parentEndDate, model.dateFormat)) != -1) {
                            parentEndDate.setDate(parentEndDate.getDate() + 1);
                        }

                        childGanttRecord.startDate = childGanttRecord._getFormatedDate(parentEndDate, model.dateFormat);

                        if (proxy._editedTaskBarItem!==childGanttRecord) {
                            var predecessorsCollection = childGanttRecord['predecessor'],
                                currentTaskId = childGanttRecord['taskId'].toString(),
                                childPredecessor = predecessorsCollection&&predecessorsCollection.filter(function (data) { return data.to === currentTaskId });

                            if (childPredecessor && childPredecessor.length > 1 && offset>0) {

                                var maxEndDate = proxy._getPredecessorDate(childGanttRecord, childPredecessor, predecessor.predecessorsType);

                                if (!childGanttRecord.isMilestone) {
                                    maxEndDate.setDate(maxEndDate.getDate() + 1);
                                }

                                if (model.includeWeekend === false) {
                                    proxy._setAfterWeekend(maxEndDate);
                                }

                                childGanttRecord.startDate = childGanttRecord._getFormatedDate(maxEndDate, model.dateFormat);
                            }
                       }

                            childGanttRecord.left = childGanttRecord._calculateLeft(proxy._projectStartDate, proxy._perDayWidth, proxy._holidays, model);
                        childGanttRecord.width = childGanttRecord._calculateWidth(proxy._perDayWidth,proxy._holidays, model);
                    break;

                case 'FF':
                    parentEndDate = proxy._getDateFromFormat(parentGanttRecord.endDate);
                    childEndDate = proxy._getDateFromFormat(childGanttRecord.endDate);

                        if (proxy.model.includeWeekend === false) {
                            proxy._setAfterWeekend(parentEndDate);
                        }

                        if (predecessorOffset) {
                            parentEndDate.setDate(parentEndDate.getDate() + (Math.floor(predecessorOffset)));

                            if (proxy.model.includeWeekend === false) {
                                if (predecessorOffset < 0) {
                                    proxy._setBeforeWeekend(parentEndDate);
                                }

                                if (predecessorOffset > 0) {
                                    proxy._setAfterWeekend(parentEndDate);
                                }
                            }
                        }
                        if (proxy._stringHolidays &&
                           proxy._stringHolidays.indexOf(childGanttRecord._getFormatedDate(parentEndDate, model.dateFormat)) != -1) {
                            parentEndDate.setDate(parentEndDate.getDate() + 1);
                        }

                        childGanttRecord.endDate = childGanttRecord._getFormatedDate(parentEndDate, model.dateFormat);
                        //End Date is changed by start date and duration
                        if (childGanttRecord.isMilestone)
                            childGanttRecord.startDate = childGanttRecord._getFormatedDate(parentEndDate, model.dateFormat);
                        else {
                            var tempEndDate = proxy._getDateFromFormat(childGanttRecord.endDate);
                            for (var i = 0; i < childGanttRecord.duration - 1; i++) {
                                tempEndDate.setDate(tempEndDate.getDate() - 1);
                                if (proxy.model.includeWeekend === false) {
                                    if (predecessorOffset < 0) {
                                        proxy._setBeforeWeekend(tempEndDate);
                                    }
                                }
                                tempEndDate = proxy._onHoliDayCheckBefore(tempEndDate);
                            }
                            childGanttRecord.startDate = childGanttRecord._getFormatedDate(tempEndDate, model.dateFormat);
                        }

                            var currentTaskId = childGanttRecord['taskId'].toString(),
                                        predecessorsCollection = childGanttRecord['predecessor'],
                                      childPredecessor =predecessorsCollection&& predecessorsCollection.filter(function (data) { return data.to === currentTaskId });

                            if (childPredecessor && childPredecessor.length > 1) {

                                var maxEndDate = proxy._getPredecessorDate(childGanttRecord, childPredecessor, predecessor.predecessorsType);

                                if (model.includeWeekend === false) {
                                    proxy._setAfterWeekend(maxEndDate);
                                }

                                if (predecessorOffset >= 0 && (maxEndDate.getTime() !== parentEndDate.getTime())) {
                                    childGanttRecord.endDate = childGanttRecord._getFormatedDate(maxEndDate, model.dateFormat);
                                    if (childGanttRecord.isMilestone)
                                        childGanttRecord.startDate = childGanttRecord._getFormatedDate(maxEndDate, model.dateFormat);
                                    else {
                                        var tempEndDate = proxy._getDateFromFormat(childGanttRecord.endDate);
                                        for (var i = 0; i < childGanttRecord.duration - 1; i++) {
                                            tempEndDate.setDate(tempEndDate.getDate() - 1);
                                            if (proxy.model.includeWeekend === false) {
                                                proxy._setBeforeWeekend(tempEndDate);
                                            }
                                            tempEndDate = proxy._onHoliDayCheckBefore(tempEndDate);
                                        }
                                        childGanttRecord.startDate = childGanttRecord._getFormatedDate(tempEndDate, model.dateFormat);
                                    }
                                }
                            }

                            childGanttRecord.left = childGanttRecord._calculateLeft(proxy._projectStartDate, proxy._perDayWidth, proxy._holidays, model);
                            childGanttRecord.width = childGanttRecord._calculateWidth(proxy._perDayWidth, proxy._holidays, model);
                    break;
            }

            if (childGanttRecord.parentItem)
                proxy._updateParentItem(childGanttRecord);

            if (proxy._isRenddered) {
                proxy.refreshGanttRecord(childGanttRecord);
            }

        },

        _getPredecessorDate: function (ganttRecord, predecessorsCollection, predecessorType) {
            var minStartDate = null, proxy = this, model = proxy.model,
                    maxEndDate = null,
                    tempMaxEndDate = null,
                    length,
                    i,
                    dateDiff = 0,
                    predecessorOffset,
                    connectorLineObject,
                    offset,
                    validatedpredecessor = predecessorsCollection.filter(function (data) {
                        return data.to === ganttRecord.taskId.toString();
                    });


            if (validatedpredecessor) {

                length = validatedpredecessor.length;

                

                    for (i = 0; i < length; i++) {

                        predecessor = validatedpredecessor[i];

                        parentGanttRecord = proxy._rows[proxy._ids.indexOf(predecessor.from)];
                        childGanttRecord = proxy._rows[proxy._ids.indexOf(predecessor.to)];

                        if (minStartDate === null) {
                            minStartDate = proxy._getDateFromFormat(parentGanttRecord.startDate);
                            if (parseInt(predecessor.offset)) {
                                minStartDate.setDate(minStartDate.getDate() + parseInt(predecessor.offset));
                            }
                        }

                        if (maxEndDate === null) {
                            maxEndDate = proxy._getDateFromFormat(parentGanttRecord.endDate);
                            //if (parseInt(predecessor.offset)) {
                            //    maxEndDate.setDate(maxEndDate.getDate() + parseInt(predecessor.offset));
                            //}
                            tempMaxEndDate = proxy._getDateFromFormat(parentGanttRecord.endDate);
                        }

                        if (proxy._getDateFromFormat(parentGanttRecord.startDate).getTime() < minStartDate.getTime()) {
                            minStartDate = proxy._getDateFromFormat(parentGanttRecord.startDate);
                            if (parseInt(predecessor.offset)) {
                                minStartDate.setDate(minStartDate.getDate() + parseInt(predecessor.offset));
                            }
                        }

                        if (proxy._getDateFromFormat(parentGanttRecord.endDate).getTime() > maxEndDate.getTime()) {
                            maxEndDate = proxy._getDateFromFormat(parentGanttRecord.endDate);
                            
                        }
                        tempMaxEndDate.setDate(tempMaxEndDate.getDate() + parseInt(predecessor.offset));
                        var tempMaxDate = proxy._getDateFromFormat(tempMaxEndDate);
                        if (model.includeWeekend === false && tempMaxDate) {
                            proxy._setAfterWeekend(tempMaxDate);
                        }
                        if (parseInt(predecessor.offset)) {
                            if (maxEndDate.getTime() < tempMaxDate.getTime())
                            maxEndDate.setDate(maxEndDate.getDate() + parseInt(predecessor.offset));
                        }
                        (tempMaxEndDate.setDate(tempMaxEndDate.getDate() - parseInt(predecessor.offset)))
                        if (model.includeWeekend === false) {
                            proxy._setAfterWeekend(maxEndDate);
                            proxy._setAfterWeekend(tempMaxEndDate);
                        }
                }
            }
            if (predecessorType == "FS" || predecessorType == "FF") {
                return maxEndDate;
            }
            else if (predecessorType == "SF" || predecessorType == "SS") {
                return minStartDate;
            }

        },

        _renderPredecessor: function (predecessor, item) {
            //var proxy = this,
            //model = proxy.model,
            //length = predecessor.length, predecessorValues = [];

            //for (var i = 0; i < length; i++) {
            //    if (predecessor[i].offset < 0) {
            //        predecessorValues.push(predecessor[i].from + "" + predecessor[i].predecessorsType + ""
            //            + predecessor[i].offset + "d");
            //    }
            //    else if (predecessor[i].offset >= 1) {
            //        predecessorValues.push(predecessor[i].from + "" + predecessor[i].predecessorsType + "+"
            //            + predecessor[i].offset + "d");
            //    }
            //    else {
            //        predecessorValues.push(predecessor[i].from);
            //    }
            //}
            //if (predecessorValues.length > 0) {
            //    item.item[model.predecessorMapping] = predecessorValues.join(',');
            //    //this.refreshRow(itemIndex, false);
            //}
        },

        _setBeforeWeekend: function(date) {
            if (date.getDay() == 0)
                date.setDate(date.getDate() - 2);
            else if (date.getDay() == 6)
                date.setDate(date.getDate() - 1);
        },
        
        _setAfterWeekend: function(date) {
            if (date.getDay() == 0)
                date.setDate(date.getDate() + 1);
            else if (date.getDay() == 6)
                date.setDate(date.getDate() + 2);
        },
        
        _updateEditedGanttRecords: function (args) {
            var proxy = this, model = proxy.model,
                ganttRecord = args.data,
                startDate = proxy._getDateFromFormat(ganttRecord.startDate),
                isPredecessorsModified = false;

            if (args.columnName === "startDate") {
               
                if (this._stringHolidays &&
                      this._stringHolidays.indexOf(ganttRecord._getFormatedDate(startDate, model.dateFormat)) != -1) {
                    startDate.setDate(startDate.getDate() + 1);
                }
                
                if (proxy.model.includeWeekend === false) {
                    this._setAfterWeekend(startDate);
                    if (this._stringHolidays &&
                      this._stringHolidays.indexOf(ganttRecord._getFormatedDate(startDate, model.dateFormat)) != -1) {
                        startDate.setDate(startDate.getDate() + 1);
                    }
                }
                ganttRecord.startDate = proxy.getFormatedDate(startDate);
                ganttRecord.left = ganttRecord._calculateGanttRecordLeft(startDate, proxy._projectStartDate, proxy._perDayWidth);
                ganttRecord.width = ganttRecord._calculateWidth(proxy._perDayWidth, proxy._holidays, model);// model.holidays
                ganttRecord.progressWidth = ganttRecord._calculateProgressWidth(ganttRecord.width, ganttRecord.status);
                if (ganttRecord.parentItem)
                    proxy._updateParentItem(ganttRecord);
            }

            else if (args.columnName === "endDate") {
                //validate start date
                var endDate = proxy._getDateFromFormat(ganttRecord.endDate);
                if (startDate.getTime() <= endDate.getTime()) {

                    if (this._stringHolidays &&
                          this._stringHolidays.indexOf(ganttRecord._getFormatedDate(endDate, model.dateFormat)) != -1) {
                        endDate.setDate(endDate.getDate() + 1);
                        if (endDate.getDay() == 6) {
                            endDate.setDate(endDate.getDate() + 2);
                        }
                    }

                    if (model.includeWeekend === false) {
                        if (endDate.getDay() == 0) {
                            endDate.setDate(endDate.getDate() - 2);
                        } else if (endDate.getDay() == 6) {
                            endDate.setDate(endDate.getDate() - 1);
                        }
                    }
                    ganttRecord.endDate = ganttRecord._calculateFormatedDate(endDate, model.dateFormat);
                    ganttRecord._calculateDuration(startDate, endDate, model.includeWeekend, model.holidays, ganttRecord.isMilestone, model);
                    ganttRecord.width = ganttRecord._calculateTaskbarWidth(proxy._perDayWidth, model);
                    ganttRecord.progressWidth = ganttRecord._calculateProgressWidth(ganttRecord.width, ganttRecord.status);
                    if (ganttRecord.parentItem)
                        proxy._updateParentItem(ganttRecord);
                }
                else
                {
                    ganttRecord.endDate = args.previousValue;
                    ganttRecord.item[model.endDateMapping] = args.previousValue;
                }
            }
            
            else if (args.columnName == "duration") {
                var currentDuration = ganttRecord.duration,
                    offset = currentDuration - args.previousValue,
                modifiedEndDate = proxy._calculateUpdatedDate(proxy._getDateFromFormat(ganttRecord.endDate), offset);

                if (currentDuration === 0) {
                    ganttRecord.isMilestone = true;
                } else {
                    ganttRecord.isMilestone = false;
                }

                ganttRecord.endDate = modifiedEndDate;
                ganttRecord.width = ganttRecord._calculateWidth(proxy._perDayWidth, proxy.holidays, model);
                ganttRecord.progressWidth = ganttRecord._calculateProgressWidth(ganttRecord.width, ganttRecord.status);
                if(model.endDateMapping)
                ganttRecord.item[model.endDateMapping] = modifiedEndDate;
                if (ganttRecord.parentItem)
                    proxy._updateParentItem(ganttRecord);
            }

            else if (args.columnName == "predecessor") {
                isPredecessorsModified = true;
                if (args.previousValue) {
                    proxy._removeConnectorLine(args.previousValue, ganttRecord);
                }
                proxy._addConnectorLine(ganttRecord);
            }

            else if (args.columnName == "status") {
                ganttRecord.status = ganttRecord.status > 100 ? 100 : ganttRecord.status;
                ganttRecord.progressWidth = proxy._getProgressWidth(ganttRecord.width, ganttRecord.status);
                if (ganttRecord.parentItem) {
                    if (ganttRecord.parentItem)
                        proxy._updateParentItem(ganttRecord);
                    
                }
            }

            //proxy._updateEditedGanttRecord(ganttRecord);
            if (ganttRecord.predecessor) {
                if (!isPredecessorsModified)
                    proxy._$ganttchartHelper.ejGanttChart("updateConnectorLineOffset", ganttRecord);
                proxy._isMileStoneEdited = ganttRecord.isMilestone;
                proxy._updatedConnectorLineCollection = [];
                proxy._connectorlineIds = [];
                proxy._validatePredecessor(ganttRecord);
                if (proxy._updatedConnectorLineCollection.length > 0) {
                    proxy._$ganttchartHelper.ejGanttChart("appendConnectorLine", proxy._updatedConnectorLineCollection);
                }
            }
            proxy._isValidationEnabled = false;
            
            proxy.refreshGanttRecord(ganttRecord);
        },

        _removeConnectorLine: function (predecessor,ganttRecord) {
            var proxy = this,
                connectorLineId,
                parentGanttRecord,
                record,
                i = 0,
                length = predecessor && predecessor.length,
                index = -1;
                
            for (i = 0; i < length; i++) {
                parentGanttRecord = proxy._rows[proxy._ids.indexOf(predecessor[i].from)];
                record = proxy._rows[proxy._ids.indexOf(predecessor[i].to)];
                
                if (ganttRecord.taskId.toString() === predecessor[i].to.toString()) {
                    
                    index = parentGanttRecord.predecessor.indexOf(predecessor[i]);
                    index >= 0 && parentGanttRecord.predecessor.splice(index, 1);
                }

                if (parentGanttRecord && record) {
                    connectorLineId = "parent"+parentGanttRecord['taskId'] + "child" + record['taskId'];
                    proxy._$ganttchartHelper.ejGanttChart("removeConnectorline", connectorLineId);
                }
            }
        },

        _addConnectorLine:function(ganttRecord) {
            var proxy = this,
                count = 0,
                predecessors = ganttRecord.predecessor,
                length,
                predecessor,
                successorGanttRecord;

            if (predecessors) {

                length = predecessors.length;

                for (count = 0; count < length; count++) {

                    predecessor = predecessors[count];
                    
                    if (predecessor.to === ganttRecord.taskId.toString()) {
                        successorGanttRecord = proxy._rows[proxy._ids.indexOf(predecessor.from)];
                    }

                    if (successorGanttRecord) {
                        if (successorGanttRecord.predecessor) {
                            successorGanttRecord.predecessor.push(predecessor);
                        } else {
                            successorGanttRecord.predecessor = [];
                            successorGanttRecord.predecessor.push(predecessor);
                            proxy._predecessorsCollection.push(successorGanttRecord);
                        }
                    }
                }
            }
        },

        _calculateUpdatedDate: function (modifiedDate, offset) {
            var proxy = this, model = proxy.model,
                i = 0;
            if (offset > 0) {
                for (i; i < offset; i++) {
                    if (model.durationUnit == ej.Gantt.DurationUnit.Day) {
                        modifiedDate = proxy.dateUpdate(modifiedDate, 1, 0);
                    }
                    else if (model.durationUnit == ej.Gantt.DurationUnit.Hour) {
                        modifiedDate = proxy.dateUpdate(modifiedDate, 0, 1);
                    }

                    if (modifiedDate.getDay() == 0)
                        modifiedDate.setDate(modifiedDate.getDate() + 1);
                    else if (modifiedDate.getDay() == 6)
                        modifiedDate.setDate(modifiedDate.getDate() + 2);
                }
            } else {
                var dateOffset = (-1) * offset;
                for (i = 0; i < dateOffset; i++) {
                    if (model.durationUnit == ej.Gantt.DurationUnit.Day) {
                        modifiedDate = proxy.dateUpdate(modifiedDate, -1, 0);
                    }
                    else if (model.durationUnit == ej.Gantt.DurationUnit.Hour) {
                        modifiedDate = proxy.dateUpdate(modifiedDate, 0, -1);
                    }

                    if (modifiedDate.getDay() == 0)
                        modifiedDate.setDate(modifiedDate.getDate() - 2);
                    else if (modifiedDate.getDay() == 6)
                        modifiedDate.setDate(modifiedDate.getDate() - 1);
                }
            }
            modifiedDate = proxy.getFormatedDate(modifiedDate);
            return modifiedDate;
        },

        _updateEditedGanttRecord: function (ganttRecord) {
            var proxy = this;
            if (ganttRecord.predecessor) {
                var predecessorsCollection = ganttRecord.predecessor,
                    count = 0,
                    length = predecessorsCollection.length,
                    predecessor,
                    parentGanttRecord,
                    record,
                    connectorLineId,
                    connectorLineObject;
                for (count = 0; count < length; count++) {
                    proxy._updatedConnectorLineCollection = [];
                    proxy._connectorlineIds = [];
                    predecessor = predecessorsCollection[count];
                    if (predecessor.to === ganttRecord['taskId'].toString()) {

                        parentGanttRecord = proxy._rows[proxy._ids.indexOf(predecessor.from)];
                        record = proxy._rows[proxy._ids.indexOf(predecessor.to)];
                        
                        if (parentGanttRecord && record) {
                            connectorLineId = "parent" + parentGanttRecord['taskId'] + "child" + record['taskId'];
                            proxy._$ganttchartHelper.ejGanttChart("removeConnectorline", connectorLineId);
                            connectorLineObject = proxy._createConnectorLineObject(parentGanttRecord, record, predecessor);
                            if (connectorLineObject) {
                                if (proxy._connectorlineIds.length > 0 && proxy._connectorlineIds.indexOf(connectorLineObject.ConnectorLineId) == -1) {

                                    proxy._updatedConnectorLineCollection.push(connectorLineObject);
                                    proxy._connectorlineIds.push(connectorLineObject.ConnectorLineId);
                                }
                                else if (proxy._connectorlineIds.length == 0) {
                                    proxy._updatedConnectorLineCollection.push(connectorLineObject);
                                    proxy._connectorlineIds.push(connectorLineObject.ConnectorLineId);
                                }
                                proxy._$ganttchartHelper.ejGanttChart("appendConnectorLine", proxy._updatedConnectorLineCollection);
                            }
                        }
                        
                    }
                }
            }
        },
        

        _updateGanttRecord: function (ganttRecord) {
            var proxy = this,
                model = proxy.model;            
            ganttRecord.left = ganttRecord._calculateLeft(proxy._projectStartDate, proxy._perDayWidth, proxy.holidays, model);
            ganttRecord.width = ganttRecord._calculateWidth(proxy._perDayWidth,model.holidays, model);
            proxy.refreshGanttRecord(ganttRecord);            
            if (ganttRecord.parentItem)
                proxy._updateParentItem(ganttRecord);
        },

        refreshGanttRecord: function (ganttRecord) {
            var proxy = this;
            proxy._currentViewData = proxy.getCurrentViewData();
            var index = proxy._currentViewData.indexOf(ganttRecord);
            proxy._$ganttchartHelper.ejGanttChart("refreshRow", index);
            proxy._$treegridHelper.ejTreeGrid("refreshRow", index);   
        },

        _updateParentItem: function (ganttRecord) {

            var proxy = this,
                parentGanttRecord = ganttRecord.parentItem,                
                model = proxy.model,                
                prevStartDate = proxy._getDateFromFormat(parentGanttRecord.startDate),
                currentStartDate = proxy._getDateFromFormat(ganttRecord.startDate),
                prevEndDate = proxy._getDateFromFormat(parentGanttRecord.endDate),
                currentEndDate = proxy._getDateFromFormat(ganttRecord.endDate);
                            
            
            if (currentStartDate.getTime() < prevStartDate.getTime()) {
                parentGanttRecord.startDate = ganttRecord.startDate;
                if (model.startDateMapping)
                parentGanttRecord.item[model.startDateMapping] = parentGanttRecord.startDate;                
            }

            if (currentEndDate.getTime() > prevEndDate.getTime()) {                
                dateDiff = proxy._daydiff(prevEndDate, currentEndDate);
                parentGanttRecord.endDate = ganttRecord.endDate;
                if(model.endDateMapping)
                parentGanttRecord.item[model.endDateMapping] = parentGanttRecord.endDate;                
            }

            var count = 0,
                childRecords = parentGanttRecord.childRecords,
                length = childRecords && childRecords.length,
                childGanttRecord,
                minStartDate = null,
                maxEndDate = null,
                parentProgress = 0,
                milestoneCount = 0,
                    totalDuration = 0,
                    completedDuration = 0;

            for (count = 0; count < length; count++) {

                childGanttRecord = childRecords[count];

                if (minStartDate === null) {
                    minStartDate =proxy._getDateFromFormat(childGanttRecord.startDate);
                }
                if (maxEndDate === null) {
                    maxEndDate = proxy._getDateFromFormat(childGanttRecord.endDate);
                }
                if (proxy._getDateFromFormat(childGanttRecord.endDate).getTime() > maxEndDate.getTime()) {
                    maxEndDate =proxy._getDateFromFormat(childGanttRecord.endDate);
                }
                if (proxy._getDateFromFormat(childGanttRecord.startDate).getTime() < minStartDate.getTime()) {
                    minStartDate = proxy._getDateFromFormat(childGanttRecord.startDate);
                }

                if (!childGanttRecord.isMilestone) {
                    //converting days into hours .
                    //By now 24 hours is default//...
                    if (model.durationUnit == ej.Gantt.DurationUnit.Hour) {
                        totalDuration += (childGanttRecord.duration*8);
                        //summation of progress done of child
                        completedDuration += ((childGanttRecord.duration*8) * childGanttRecord.status) / 100;
                    }
                    else {
                        totalDuration += (childGanttRecord.duration * 24);
                        //summation of progress done of child
                        completedDuration += ((childGanttRecord.duration * 24) * childGanttRecord.status) / 100;
                    }


                }

                else
                    milestoneCount++;
            }

            if (prevStartDate.getTime() !== minStartDate.getTime()) {
                parentGanttRecord.startDate = parentGanttRecord._getFormatedDate(minStartDate, model.dateFormat);
                if (model.startDateMapping)
                parentGanttRecord.item[model.startDateMapping] = parentGanttRecord.startDate;
            }
            
            if (prevEndDate.getTime() != maxEndDate.getTime()) {                                                       
                parentGanttRecord.endDate = parentGanttRecord._getFormatedDate(maxEndDate, model.dateFormat);
                if(model.endDateMapping)
                parentGanttRecord.item[model.endDateMapping] = parentGanttRecord.endDate;                
            }

            if (!(model.endDateMapping.length==0 && model.workingTimeScale == ej.Gantt.workingTimeScale.TimeScale8Hours &&
                model.durationUnit == ej.Gantt.DurationUnit.Hour &&
                model.scheduleHeaderSettings.scheduleHeaderType == ej.Gantt.ScheduleHeaderType.Day)) {
                parentGanttRecord._calculateDuration(proxy._getDateFromFormat(parentGanttRecord.startDate),
                    proxy._getDateFromFormat(parentGanttRecord.endDate), model.includeWeekend, model.holidays,
                    parentGanttRecord[model.milestoneMapping], model);

            }

            if ((model.workingTimeScale == ej.Gantt.workingTimeScale.TimeScale8Hours ||
                model.workingTimeScale == ej.Gantt.workingTimeScale.TimeScale24Hours) &&
            model.scheduleHeaderSettings.scheduleHeaderType == ej.Gantt.ScheduleHeaderType.Day ){
                var beginDate = proxy._getDateFromFormat(parentGanttRecord.startDate);
                //beginDate.setHours(8);
                var diffDate = (beginDate - proxy._projectStartDate) / 86400000;
            }
            else if (model.workingTimeScale == ej.Gantt.workingTimeScale.TimeScale8Hours && model.roundOffDayworkingTime) {
                var diffDate = Math.round((proxy._getDateFromFormat(parentGanttRecord.startDate) - proxy._projectStartDate) / 86400000);
            }
            else {
                var diffDate = Math.round((proxy._getDateFromFormat(parentGanttRecord.startDate) - proxy._projectStartDate) / 86400000);
            }
            parentGanttRecord.left = (diffDate * proxy._perDayWidth);
            parentGanttRecord.width = parentGanttRecord._calculateWidth(proxy._perDayWidth,  proxy._holidays, model);//model.holidays
            
            //Calculate progressWidth after Left and width calculation
            parentProgress = totalDuration > 0 ? (completedDuration / totalDuration) * 100 : 0;
            parentGanttRecord.progressWidth = proxy._getProgressWidth(parentGanttRecord.width, parentProgress);
            parentGanttRecord.status = Math.round(parentProgress);

            if (model.progressMapping)
                parentGanttRecord.item[model.progressMapping] = parentGanttRecord.status;

            if (proxy._isRenddered)
                proxy.refreshGanttRecord(parentGanttRecord);

            if (parentGanttRecord.parentItem) {
                proxy._updateParentItem(parentGanttRecord);
            }

            
        },

        _validatePredecessor: function (childGanttRecord) {

            var proxy = this,model=proxy.model;

            if (childGanttRecord.predecessor) {

                var predecessorsCollection = childGanttRecord['predecessor'],
                    count,                    
                    parentGanttRecord,
                    record=null,
                    connectorLineId,
                    isValidated = false,
                    predecessor,
                    connectorLineObject,
                    lowestOffset;

                var currentTaskId = childGanttRecord['taskId'].toString(),

                    predecessors = predecessorsCollection.filter(function (data) { return data.to === currentTaskId }),
                    successors = predecessorsCollection.filter(function (data) { return data.from === currentTaskId });
               var predecessorLength = predecessors.length;

                for (count = 0; count < predecessorLength; count++) {
                    predecessor = predecessors[count];
                    parentGanttRecord = proxy._rows[proxy._ids.indexOf(predecessor.from)];//THis is going to exception when id is not corrct order
                    record = proxy._rows[proxy._ids.indexOf(predecessor.to)];
                    connectorLineId = "parent" + parentGanttRecord['taskId'] + "child" + record['taskId'];
                    if (proxy._$ganttchartHelper)
                        proxy._$ganttchartHelper.ejGanttChart("removeConnectorline", connectorLineId);

                    if (childGanttRecord.taskId.toString() === predecessor.to)
                        proxy._validateChildGanttRecord(parentGanttRecord, record, predecessor);


                    if (childGanttRecord.taskId.toString() === predecessor.from)
                        proxy._validateChildGanttRecord(parentGanttRecord, record, predecessor);

                }
                for (count = 0; count < predecessorLength; count++) {
                predecessor = predecessors[count];
                parentGanttRecord = proxy._rows[proxy._ids.indexOf(predecessor.from)];//THis is going to exception when id is not corrct order
                record = proxy._rows[proxy._ids.indexOf(predecessor.to)];
             
					if ((model.enableVirtualization === false && (parentGanttRecord.isExpanded === false || record.isExpanded == false)))//(model.enableVirtualization === false && !parentGanttRecord.isExpanded && record.isExpanded === false)
						continue;
                    connectorLineObject = proxy._createConnectorLineObject(parentGanttRecord, record, predecessor);
                    if (connectorLineObject) {
                        if (proxy._connectorlineIds.length > 0 && proxy._connectorlineIds.indexOf(connectorLineObject.ConnectorLineId) == -1) {

                            proxy._updatedConnectorLineCollection.push(connectorLineObject);
                            proxy._connectorlineIds.push(connectorLineObject.ConnectorLineId);
                        }
                        else if (proxy._connectorlineIds.length == 0) {
                            proxy._updatedConnectorLineCollection.push(connectorLineObject);
                            proxy._connectorlineIds.push(connectorLineObject.ConnectorLineId);
                        }
                        else if (proxy._connectorlineIds.indexOf(connectorLineObject.ConnectorLineId) != -1) {
                            var index = proxy._connectorlineIds.indexOf(connectorLineObject.ConnectorLineId);
                            proxy._updatedConnectorLineCollection[index] = connectorLineObject;
                        }
                    }                   

                }

                var length = successors.length;

                for (count = 0; count < length; count++) {

                    predecessor = successors[count];
                    parentGanttRecord = proxy._rows[proxy._ids.indexOf(predecessor.from)];
                    record = proxy._rows[proxy._ids.indexOf(predecessor.to)];
                    connectorLineId = "parent" + parentGanttRecord['taskId'] + "child" + record['taskId'];
                    if (proxy._$ganttchartHelper)
                        proxy._$ganttchartHelper.ejGanttChart("removeConnectorline", connectorLineId);
                    proxy._validateChildGanttRecord(parentGanttRecord, record, predecessor);
                      if ((model.enableVirtualization === false && (parentGanttRecord.isExpanded === false || record.isExpanded == false)))
                        continue;
					connectorLineObject = proxy._createConnectorLineObject(parentGanttRecord, record, predecessor);
                    if (connectorLineObject) {
                        if (proxy._connectorlineIds.length > 0 && proxy._connectorlineIds.indexOf(connectorLineObject.ConnectorLineId) == -1) {

                            proxy._updatedConnectorLineCollection.push(connectorLineObject);
                            proxy._connectorlineIds.push(connectorLineObject.ConnectorLineId);
                        }
                        else if (proxy._connectorlineIds.length == 0) {
                            proxy._updatedConnectorLineCollection.push(connectorLineObject);
                            proxy._connectorlineIds.push(connectorLineObject.ConnectorLineId);
                        }
                        else if (proxy._connectorlineIds.indexOf(connectorLineObject.ConnectorLineId) != -1) {
                            var index = proxy._connectorlineIds.indexOf(connectorLineObject.ConnectorLineId);
                            proxy._updatedConnectorLineCollection[index] = connectorLineObject;
                        }
                    }

                    record &&  proxy._validatePredecessor(record);

                }                
            }
        },
       
        _calculateDateDifference: function (emptyDate, dateDiff) {
            var differenceCount = dateDiff >= 0 ? dateDiff : (-1 * dateDiff),
                offset = 0;
            
            if (dateDiff > 0)
                offset = differenceCount - this._calculateWeekendCount(emptyDate, dateDiff);
            else
                offset = dateDiff + this._calculateWeekendCount(emptyDate, dateDiff);
            return offset;
        },

        _calculateWeekendCount: function (emptyDate, dateDiff) {
            var differenceCount = dateDiff >= 0 ? dateDiff : (-1 * dateDiff),
                weekendCount = 0,
                i = 0;
            
            if (dateDiff > 0) {
                for (i = 0; i < dateDiff; i++) {
                    if (emptyDate.getDay() == 0) {
                        weekendCount++;
                        emptyDate.setDate(emptyDate.getDate() + 1);
                    }
                    else if (emptyDate.getDay() == 6) {
                        weekendCount += 2;
                        ++i;
                        emptyDate.setDate(emptyDate.getDate() + 2);
                    } else
                        emptyDate.setDate(emptyDate.getDate() + 1);
                }
            }
            else {
                for (i = 0; i < differenceCount; i++) {
                    emptyDate.setDate(emptyDate.getDate() - 1);
                    if (emptyDate.getDay() == 0 || emptyDate.getDay() == 6) {
                        weekendCount += 1;
                    }
                }
            }
            return weekendCount;
        },

        _daydiff: function (first, second) {
            return ((new Date(second) - new Date(first)) / (1000 * 60 * 60 * 24));
        },

        


        dateUpdate: function (date, days, hours) {
            return new Date(
                date.getFullYear(),
                date.getMonth(),
                date.getDate() + days,
                date.getHours() + hours,
                date.getMinutes(),
                date.getSeconds(),
                date.getMilliseconds()
            );
        },
        
        //Method for collapse all the GanttRecords

        _collapseAll: function () {
            var proxy = this,
                model = proxy.model,
                count = 0,
                ganttRecord,
                args = {},
                enableVirtualization = model.enableVirtualization,
                $gridRows = proxy.getRows(),
                $rowElement = null;

            proxy._gridRows = $gridRows;
            proxy._isInExpandCollapse = true;
             
            //loop excecute for iterate the GanttRecords preseent in the Gantt Control            
            for (count = 0; count < proxy.model.parentRecords.length; count++) {
                if (proxy.model.parentRecords[count].expanded) {
                    ganttRecord = proxy.model.parentRecords[count];
                    args.data = ganttRecord;
                    args.recordIndex = count;
                    args.expanded = false;

                    if (enableVirtualization) {
                        if (ganttRecord.hasChildRecords && ganttRecord.expanded) {
                            ej.TreeGrid.sendExpandCollapseRequest(proxy, args);
                        }
                    } else {
                        if (ganttRecord.hasChildRecords && ganttRecord.expanded) {
                            ej.TreeGrid.sendExpandCollapseRequest(proxy, args);
                        }
                        else
                            ej.TreeGrid.sendExpandCollapseRequest(proxy, args);
                        var collapsedCount = proxy._collapsedRecordCount;
                        proxy._$ganttchartHelper.ejGanttChart("reRenderGanttGridLinesTable", -collapsedCount);
                    }
                }
            }

            proxy._isInExpandCollapse = false;
        },
        
        
        //Method for expand all the GanttRecords
        
        _expandAll: function () {
            
            var proxy = this,
                model = proxy.model,
                count = 0,
                ganttRecord,
                args = {},
                enableVirtualization = model.enableVirtualization;

            proxy._isInExpandCollapse = true;

            //loop excecute for iterate the GanttRecords preseent in the Gantt Control

            for (count = 0; count < proxy._updatedGanttRecords.length; count++) {
                ganttRecord = proxy._updatedGanttRecords[count];
                args.data = ganttRecord;
                args.recordIndex = count;
                args.expanded = true;

                if (enableVirtualization) {
                    if (ganttRecord.hasChildRecords && !ganttRecord.expanded) {
                             ej.TreeGrid.sendExpandCollapseRequest(proxy, args);
                    }
                } else {
                    if (ganttRecord.hasChildRecords && !ganttRecord.expanded) {
                        ej.TreeGrid.sendExpandCollapseRequest(proxy, args);
                        var expandedCount = proxy._expandedRecordsCount;
                        proxy._$ganttchartHelper.ejGanttChart("reRenderGanttGridLinesTable", expandedCount);
                    }
                }
            }

            proxy._isInExpandCollapse = false;
        },
        _getDateFromFormat: function (date) {
            
            if (typeof date === "object") {
                return new Date(date);
            }
            if (date) {
                return Globalize.parseDate(date, this.model.dateFormat) == null ?
                    new Date(date) : Globalize.parseDate(date, this.model.dateFormat);
            }
        },

        /**
        * destroy the gantt widget
		* all events bound using this._on will be unbind automatically and bring the control to pre-init state.
		* @alias destroy
		* @return jQuery
		* @example 
		* &lt;gantt id="gantt"&gt;Gantt&lt;/gantt&gt; <br> 
		* &lt;script&gt;
		* // Create gantt
		* var gantt = $("#gantt").data("ejGantt");
		* gantt.destroy(); // destroy the gantt
		* &lt;/script&gt;
		* @example 
		* &lt;gantt id="gantt"&gt;Gantt&lt;/gantt&gt; <br> 
		* &lt;script&gt;
		* // enable the gantt
		* $("#gantt").ejGantt("destroy");	
		* &lt;/script&gt;
		* @memberof ejGantt
		* @instance
         */
        _destroy: function () {
            var proxy = this;
            proxy.element.empty().removeClass("e-gantt-core e-gantt " + proxy.model.cssClass);
        }
        


    });
    
    ej.Gantt.EditingType = {
        String: "stringedit",
        Boolean: "booleanedit",
        Numeric: "numericedit",
        Dropdown: "dropdownedit",
        DatePicker: "datepicker",
        DateTimePicker:"datetimepicker"
    };

    ej.Gantt.ToolbarItems = {
        Add: "add",
        Edit: "edit",
        Delete: "delete",
        Update: "update",
        Cancel: "cancel",
        Search: "search",
        Indent: "indent",
        Outdent: "outdent",
        ExpandAll: "expandAll",
        CollapseAll:"collapseAll"
    };

    ej.Gantt.ScheduleHeaderType = {
        Year: "year",
        Month: "month",
        Week: "week",
        Day: "day"
    };

    ej.Gantt.DurationUnit = {
        Day: "day",
        Hour: "hour",
    };

    ej.Gantt.workingTimeScale = {
        TimeScale8Hours: "TimeScale8Hours",
        TimeScale24Hours: "TimeScale24Hours",
    };


    ej.Gantt.localization = {};

    ej.Gantt.localization["en-US"] = {

        //string to display with dataSource contains 0 objects
        emptyRecord: "No records to display",

        //headerText to be displayed in treegrid
        columnHeaderTexts: {
            taskId: "ID",
            taskName: "Task Name",
            startDate: "Start Date",
            endDate: "End Date",
            resourceInfo: "Resources",
            duration: "Duration",
            status: "Progress",
            predecessor: "Predecessor",
            baselineStartDate: "Baseline Start Date",
            baselineEndDate: "Baseline End Date"
        },

        //string to display in dialog 
        editDialogTexts: {
            addFormTitle: "New Task",
            editFormTitle: "Edit Task",
            saveButton: "Save",
            cancelButton: "Cancel"
        },

        calendars: {
            standard: {
                days: {
                    // full name of days
                    names: ["Sunday", "Monday", "Tuesday", "Wednesday", "Thursday", "Friday", "Saturday"],
                    // abbreviated names of days
                    namesAbbr: ["Sun", "Mon", "Tue", "Wed", "Thu", "Fri", "Sat"],
                },
                months: {
                    // full name of months
                    names: ["January", "February", "March", "April", "May", "June", "July", "August", "September", "October", "November", "December"],
                    // abbreviated name of months
                    namesAbbr: ["Jan", "Feb", "Mar", "Apr", "May", "Jun", "Jul", "Aug", "Sep", "Oct", "Nov", "Dec"]
                },

                // set of predefined date and time patterns used by the culture.
                patterns: {
                    d: "M/d/yyyy",
                    D: "dddd, MMMM dd, yyyy",
                    F: "dddd, MMMM dd, yyyy h:mm:ss tt",
                    g: "M/d/yyyy h:mm tt",
                    G: "M/d/yyyy h:mm:ss tt",
                    m: "MMMM dd",
                    M: "MMMM dd",
                    s: "yyyy'-'MM'-'ddTHH':'mm':'ss",
                    t: "h:mm tt",
                    T: "h:mm:ss tt",
                    u: "yyyy'-'MM'-'dd HH':'mm':'ss'Z'",
                    y: "MMMM, yyyy",
                    Y: "MMMM, yyyy"
                }
            }
        }
    };

    /*-----Initialize the GanttRecord object--------------*/

    ej.Gantt.GanttRecord = function() {
        var proxy = this;
        proxy.taskId = null,
        proxy.taskName = null,
        proxy.startDate = null,
        proxy.endDate = null,
        proxy.duration = null,
        proxy.isMilestone = false,
        proxy.status = null,
        proxy.predecessor = null,
        proxy.resourceInfo = null,
        proxy.parentItem = null,
        proxy.isSelected = false,
        proxy.childRecords = null,
        proxy.hasChildRecords = false,
        proxy.expanded = false,
        proxy.level = 0,
        proxy.left = 0,
        proxy.width = 0,
        proxy.progressWidth = 0,
        proxy.item = null,
        proxy.baselineLeft = 0,
        proxy.baselineWidth = 0,
        proxy.baselineStartDate = null,
        proxy.baselineEndDate = null,
        proxy.isReadOnly = false;
    }

    ej.Gantt.GanttRecord.prototype = {

        //calculate the left position of the taskbar
        _calculateLeft: function (projectStartDate, dayWidth, holidays, model) {

            var proxy = this,
                diffDate,
                enddatecount = 0,
                beginDate = proxy._getDateFromFormat(proxy.startDate, model.dateFormat),
                endDate = proxy._getDateFromFormat(proxy.endDate, model.dateFormat),
                endDateHours = endDate && endDate.getHours(),
                endDateMinutes = endDate && endDate.getMinutes(),
                duration = 0, timeScale = model.workingTimeScale;

            if (endDate) {
                endDateHours = endDate.getHours();
                endDateMinutes = endDate.getMinutes();
            }

            if (model.durationUnit == ej.Gantt.DurationUnit.Hour) {
                if (model.durationMapping) {
                    if (timeScale == ej.Gantt.workingTimeScale.TimeScale24Hours) {
                        endDateHours = (parseInt(proxy.duration % 24) > 0) ? parseInt(proxy.duration % 24) : 0;
                        duration = endDateHours > 0 ? parseInt(proxy.duration / 24) + 1 : parseInt(proxy.duration / 24);

                    }
                    else if (timeScale == ej.Gantt.workingTimeScale.TimeScale8Hours) {
                        endDateHours = (parseInt(proxy.duration % 8) > 0) ? parseInt(proxy.duration % 8) : 0;
                        duration = endDateHours > 0 ? parseInt(proxy.duration / 8) + 1 : parseInt(proxy.duration / 8);
                    }
                }

            }
            else {
                duration = parseInt(proxy.duration);
            }

            if (endDate == null) {

                endDate = proxy._getDateFromFormat(proxy.startDate, model.dateFormat);
                endDate.setDate(endDate.getDate() + duration);
            }


            if (model.includeWeekend == false) {

                if (beginDate.getDay() == 0) {
                    beginDate.setDate(beginDate.getDate() + 1);
                }
                if (beginDate.getDay() == 6) {
                    beginDate.setDate(beginDate.getDate() - 1);
                }

                proxy._updateHoliday(beginDate, holidays, model.includeWeekend);

                var emptyDate = new Date(beginDate);
                for (var i = 0; i < duration; i++) {
                    if (emptyDate.getDay() == 0) {
                        enddatecount++;
                    } else if (emptyDate.getDay() == 6) {
                        enddatecount += 2;
                        emptyDate.setDate(emptyDate.getDate() + 2);
                    }
                    emptyDate.setDate(emptyDate.getDate() + 1);
                }
                endDate = new Date(beginDate);
                enddatecount = duration === 0 ? 1 : enddatecount;
                endDate.setDate(endDate.getDate() + (duration + enddatecount - 1));


                if (endDate.getDay() == 0) {
                    endDate.setDate(endDate.getDate() + 1);
                }
                if (endDate.getDay() == 6) {
                    endDate.setDate(endDate.getDate() + 2);
                }
            }

            else if (model.includeWeekend == true) {
                proxy._updateHoliday(beginDate, holidays, model.includeWeekend);
                endDate = new Date(beginDate);
                if (duration > 0) {
                    endDate.setDate(endDate.getDate() + (duration - 1));
                }

            }

            if (endDateHours) {
                if ((timeScale == ej.Gantt.workingTimeScale.TimeScale8Hours) &&
                    model.durationUnit == ej.Gantt.DurationUnit.Hour) {
                    if (endDateHours > 0)
                        endDate.setHours(8 + endDateHours);
                    else
                        endDate.setHours(17);
                }
                else
                    endDate.setHours(endDateHours);
            }
            else if ((timeScale == ej.Gantt.workingTimeScale.TimeScale8Hours && model.durationUnit == ej.Gantt.DurationUnit.Hour)) {
                endDate.setHours(17);
            }

            else if ((timeScale == ej.Gantt.workingTimeScale.TimeScale24Hours && model.dateFormat.indexOf("hh") != -1) ||
                (timeScale == ej.Gantt.workingTimeScale.TimeScale24Hours && model.durationUnit == ej.Gantt.DurationUnit.Hour)) {
                endDate.setHours(24);
            }

            // endDate.setMinutes(endDateMinutes);
            proxy.startDate = proxy._getFormatedDate(beginDate, model.dateFormat);
            proxy.endDate = proxy._getFormatedDate(endDate, model.dateFormat);

            if (model.endDateMapping)
                proxy.item[model.endDateMapping] = proxy.endDate;

            if (model.startDateMapping)
                proxy.item[model.startDateMapping] = proxy.startDate;
            diffDate = Math.round((beginDate - projectStartDate) / 86400000);

            if ((timeScale == ej.Gantt.workingTimeScale.TimeScale24Hours)
                && model.scheduleHeaderSettings.scheduleHeaderType != ej.Gantt.ScheduleHeaderType.Day) {
                diffDate = ((beginDate - projectStartDate) / 86400000);
            }

            else if ((timeScale == ej.Gantt.workingTimeScale.TimeScale8Hours ) &&
                model.scheduleHeaderSettings.scheduleHeaderType == ej.Gantt.ScheduleHeaderType.Day &&
                model.dateFormat.indexOf("hh") == -1) {
                diffDate = ((beginDate - projectStartDate) / 86400000);
            }

            else if ((timeScale == ej.Gantt.workingTimeScale.TimeScale24Hours || timeScale == ej.Gantt.workingTimeScale.TimeScale8Hours) &&
                  model.scheduleHeaderSettings.scheduleHeaderType == ej.Gantt.ScheduleHeaderType.Day &&
                   model.dateFormat.indexOf("hh") != -1) {

                diffDate = ((beginDate - projectStartDate) / 86400000);
            }

            else if ((timeScale == ej.Gantt.workingTimeScale.TimeScale8Hours) && model.roundOffDayworkingTime) {
                diffDate = Math.round((beginDate - projectStartDate) / 86400000);
            }


            return (diffDate * dayWidth);
        },

        //calculate the left margin of the baseLine element.
        _calculateBaselineLeft: function (projectStartDate, dayWidth, includeWeekend, dateformat) {

            var diffDate,
                proxy = this,
                beginDate = Globalize.parseDate(proxy.startDate, dateformat),
                duration = parseInt(proxy.duration),
                enddate = proxy.endDate;

            if (enddate == null) {
                enddate = new Date(beginDate);
                enddate.setDate(enddate.getDate() + duration);
            }
            diffDate = Math.ceil((beginDate - projectStartDate) / 86400000);
            return (diffDate * dayWidth);
        },

        _updateHoliday: function (date, holidays, includeWeekend) {
            var j = 0,
                length = holidays && holidays.length;

            if (length > 0) {

                for (j = 0; j < length; j++) {

                    if (holidays[j].getDate() == date.getDate()) {
                        date.setDate(date.getDate() + 1);
                    }

                }

                if ((includeWeekend == false) && (date.getDay() == 6)) {
                    date.setDate(date.getDate() + 2);
                }
            }
        },

        _calculateGanttRecordLeft: function (startDate, projectStartDate, dayWidth) {
            var proxy = this,
                diffDate = proxy._daydiff(projectStartDate, startDate);
            return (diffDate * dayWidth);
        },

        _calculateReverseLeft: function (projectStartDate, dayWidth, includeWeekend, dateformat) {
            var diffDate,
                proxy = this,
                enddatecount = 0,
                startdate = proxy.startDate,
                duration = proxy.duration && parseInt(proxy.duration);

            if (startdate == null) {

                startdate = proxy._getDateFromFormat(proxy.endDate);
                startdate.setDate(startdate.getDate() - duration);
            }
            if (includeWeekend == false) {

                var emptyDate = proxy._getDateFromFormat(proxy.endDate, dateformat);

                for (var i = 0; i < duration; i++) {

                    if (emptyDate.getDay() == 0) {
                        enddatecount += 2;
                        emptyDate.setDate(emptyDate.getDate() - 2);
                    }
                    else if (emptyDate.getDay() == 6) {
                        enddatecount++;
                    }
                    emptyDate.setDate(emptyDate.getDate() - 1);
                }
                startdate = proxy._getDateFromFormat(proxy.endDate, dateformat);
                enddatecount = duration === 0 ? 1 : enddatecount;
                startdate.setDate(startdate.getDate() - (duration + enddatecount - 1));
            }
            proxy.startDate = proxy._getFormatedDate(startdate, dateformat);
            diffDate = Math.ceil((startdate - projectStartDate) / 86400000);
            return (diffDate * dayWidth);
        },

        _calculateReverseWidth: function (dayWidth, dateFormat, includeWeekend, holidays) {

            var proxy = this,
                enddatecount = 0,
                diffDate,
                emptyDate,
                i = 0,
                startdate = proxy.startDate,
                duration = proxy.duration && parseInt(proxy.duration),
                enddate = proxy.endDate;

            if (startdate == null) {
                startdate = proxy._getDateFromFormat(enddate, dateFormat);
                startdate.setDate(startdate.getDate() - duration + 1);

            }

            if (includeWeekend == false) {

                emptyDate = proxy._getDateFromFormat(enddate, dateFormat);

                for (i = 0; i < duration; i++) {

                    if (emptyDate.getDay() == 0) {

                        enddatecount += 2;
                        emptyDate.setDate(emptyDate.getDate() - 2);

                    } else if (emptyDate.getDay() == 6) {
                        enddatecount++;
                    }

                    emptyDate.setDate(emptyDate.getDate() - 1);

                }

                startdate = proxy._getDateFromFormat(enddate, dateFormat);
                startdate.setDate(startdate.getDate() - (duration + enddatecount - 1));
            }

            if (holidays != null && holidays.length > 0) {

                emptyDate = proxy._getDateFromFormat(startdate, dateFormat);

                for (i = 0; i < duration; i++) {

                    emptyDate.setDate(emptyDate.getDate() + 1);

                    for (var j = 0; j < holidays.length; j++) {

                        if (new Date(holidays[j].day).getDate() == emptyDate.getDate()) {
                            enddatecount++;
                        }

                    }

                    if ((includeWeekend == false) && (emptyDate.getDay() == 0 || emptyDate.getDay() == 6)) {
                        enddatecount++;
                    }

                }

                enddate = proxy._getDateFromFormat(proxy.startDate, dateFormat);
                enddate.setDate(enddate.getDate() + (duration + enddatecount - 1));
            }

            diffDate = Math.ceil((proxy._getDateFromFormat(enddate, dateFormat) - proxy._getDateFromFormat(proxy.startDate, dateFormat)) / 86400000) + 1;

            return (diffDate) * dayWidth;
        },

        _calculateWidth: function (dayWidth, holidays, model) {

            var proxy = this,
                diffDate,
                startdate = proxy.startDate,
                enddate = proxy.endDate,
                duration = proxy.duration;

            if (enddate == null) {
                duration = parseInt(duration);
                enddate = proxy._getFormatedDate(startdate, model.dateFormat);
                enddate.setDate(enddate.getDate() + duration - 1);
            }

            proxy._calculateEndDate(model.includeWeekend, holidays, model, dayWidth);

            if (model.durationUnit == ej.Gantt.DurationUnit.Hour) {
                if (model.workingTimeScale == ej.Gantt.workingTimeScale.TimeScale24Hours) {
                    if (model.durationUnit == ej.Gantt.DurationUnit.Hour && model.dateFormat.indexOf("hh") == -1) {
                        diffDate = proxy.duration;
                    }
                    else {
                        diffDate = proxy._hourdiff(proxy._getDateFromFormat(proxy.startDate, model.dateFormat),
                        proxy._getDateFromFormat(proxy.endDate, model.dateFormat));
                    }
                }
                else if (model.workingTimeScale == ej.Gantt.workingTimeScale.TimeScale8Hours) {
                    if (model.durationUnit == ej.Gantt.DurationUnit.Hour && model.dateFormat.indexOf("hh") == -1) {
                        diffDate = (proxy.duration / 8) * 24 + (proxy.duration % 8);
                    }
                    else {
                        diffDate = proxy._hourdiff(proxy._getDateFromFormat(proxy.startDate, model.dateFormat),
                        proxy._getDateFromFormat(proxy.endDate, model.dateFormat));
                    }
                }
                return (diffDate * (dayWidth / 24));
            }
            else {
                if ((model.workingTimeScale == ej.Gantt.workingTimeScale.TimeScale24Hours)
                      && model.scheduleHeaderSettings.scheduleHeaderType != ej.Gantt.ScheduleHeaderType.Day &&
                    (model.dateFormat.indexOf("hh:") != -1)) {
                    diffDate = ((proxy._getDateFromFormat(proxy.endDate, model.dateFormat) -
                        proxy._getDateFromFormat(proxy.startDate, model.dateFormat)) / 86400000);// + 1;//Math.ceil
                }

                else if ((model.workingTimeScale == ej.Gantt.workingTimeScale.TimeScale24Hours)
                  && model.scheduleHeaderSettings.scheduleHeaderType != ej.Gantt.ScheduleHeaderType.Day &&
                   (model.dateFormat.indexOf("hh:") == -1)) {
                    diffDate = ((proxy._getDateFromFormat(proxy.endDate, model.dateFormat) -
                        proxy._getDateFromFormat(proxy.startDate, model.dateFormat)) / 86400000) + 1;//Math.ceil
                }
                    //Calculating width for 8 hour timescale  for dateformats without time
                else if ((model.workingTimeScale == ej.Gantt.workingTimeScale.TimeScale8Hours) &&
                model.scheduleHeaderSettings.scheduleHeaderType == ej.Gantt.ScheduleHeaderType.Day &&
                    model.durationUnit != ej.Gantt.DurationUnit.Hour) {

                    var tStartDate = new Date(proxy.startDate),
                        tEndDate = new Date(proxy.endDate);
                    tStartDate.setHours(8);
                    tEndDate.setHours(17);

                    diffDate = ((tEndDate - tStartDate) / 86400000);
                }

                else if ((model.workingTimeScale == ej.Gantt.workingTimeScale.TimeScale8Hours) &&
                         model.scheduleHeaderSettings.scheduleHeaderType == ej.Gantt.ScheduleHeaderType.Day &&
                         model.durationUnit == ej.Gantt.DurationUnit.Hour) {
                    var tStartDate = new Date(proxy.startDate),
                       tEndDate = new Date(proxy.endDate);

                    diffDate = ((tEndDate - tStartDate) / 86400000);
                }

                else if (model.workingTimeScale == ej.Gantt.workingTimeScale.TimeScale8Hours || model.roundOffDayworkingTime)
                    diffDate = Math.round((proxy._getDateFromFormat(proxy.endDate, model.dateFormat) -
                        proxy._getDateFromFormat(proxy.startDate, model.dateFormat)) / 86400000) + 1;


                return (diffDate) * dayWidth;
            }
        },


        _calculateTaskbarWidth: function (dayWidth, model) {
            var proxy = this,
                diffDate = 0,
                dateFormat = model.dateFormat,
                timeScale = model.workingTimeScale;

            if (model.durationUnit == ej.Gantt.DurationUnit.Hour) {
                if (model.workingTimeScale == ej.Gantt.workingTimeScale.TimeScale24Hours) {

                    if (model.durationUnit == ej.Gantt.DurationUnit.Hour && model.dateFormat.indexOf("hh") == -1) {
                        diffDate = proxy.duration;
                    }
                    else {
                        diffDate = proxy._hourdiff(proxy._getDateFromFormat(proxy.startDate, dateFormat),
                        proxy._getDateFromFormat(proxy.endDate, dateFormat));
                    }
                }
                else if (model.workingTimeScale == ej.Gantt.workingTimeScale.TimeScale8Hours) {
                    if (model.durationUnit == ej.Gantt.DurationUnit.Hour && model.dateFormat.indexOf("hh") == -1) {
                        diffDate = proxy.duration;
                    }
                    else {
                        diffDate = proxy._hourdiff(proxy._getDateFromFormat(proxy.startDate, dateFormat),
                        proxy._getDateFromFormat(proxy.endDate, dateFormat));
                    }
                }
                return (diffDate * (dayWidth / 24));
            }
            else {
                if (timeScale == ej.Gantt.workingTimeScale.TimeScale24Hours &&
                    model.scheduleHeaderSettings.scheduleHeaderType != ej.Gantt.ScheduleHeaderType.Day && model.dateFormat.indexOf("hh") == -1) {
                    diffDate = proxy._daydiff(proxy._getDateFromFormat(proxy.startDate, dateFormat),
                    proxy._getDateFromFormat(proxy.endDate, dateFormat)) + 1;

                }

                else if (timeScale == ej.Gantt.workingTimeScale.TimeScale24Hours) {
                    diffDate = proxy._daydiff(proxy._getDateFromFormat(proxy.startDate, dateFormat),
                    proxy._getDateFromFormat(proxy.endDate, dateFormat));
                }//+ 1;

                else if (timeScale == ej.Gantt.workingTimeScale.TimeScale8Hours) {
                    diffDate = Math.round(proxy._daydiff(proxy._getDateFromFormat(proxy.startDate, dateFormat),
                        proxy._getDateFromFormat(proxy.endDate, dateFormat)) + 1);
                }
                else {
                    diffDate = proxy._daydiff(proxy._getDateFromFormat(proxy.startDate, dateFormat),
                        proxy._getDateFromFormat(proxy.endDate, dateFormat));//+ 1;
                }
                return (diffDate * dayWidth);
            }
        },

        _getFormatedDate: function (dateObject, dateformat) {
            return Globalize.format(dateObject, dateformat);
        },

        _getDateFromFormat: function (dateString, dateformat) {
            if (dateString) {
                return Globalize.parseDate(dateString, dateformat) == null ?
                    new Date(dateString) : Globalize.parseDate(dateString, dateformat);
            }
        },

        _calculateProgressWidth: function (parentWidth, percent) {
            return (parentWidth * percent) / 100;
        },

        _setResourceInfo: function (resourceIdcollection, resourceIdMapping, resourceNameMapping, resourcecollection) {

            var count = 0,
            length = resourceIdcollection.length,
            resources = [];

            for (count; count < length; count++) {

                var resource = resourcecollection.filter(function (resourceInfo) {
                    return resourceIdcollection[count] === resourceInfo[resourceIdMapping];
                });

                resource.length && resources.push(resource[0]);
            }
            return resources;
        },

        _updateEndDate: function (prevStartDate, includeWeekend, dateFormat) {
            var proxy = this,
                formatedPrevStartDate = new Date(prevStartDate),
                formatedStartDate = proxy._getDateFromFormat(proxy.startDate, dateFormat),
                diff = proxy._daydiff(formatedPrevStartDate, formatedStartDate),
            endDate = proxy._getDateFromFormat(proxy.endDate, dateFormat),
            weekendCount = 0;

            if (includeWeekend == false) {
                var count = 0;
                for (count; count <= diff; count++) {
                    endDate.setDate(endDate.getDate() + 1);
                    if (endDate.getDay() == 0 || endDate.getDay() == 6)
                        weekendCount += 1;
                }
            }

            var modifiedEndDate = proxy._getDateFromFormat(proxy.endDate, dateFormat);
            modifiedEndDate.setDate(modifiedEndDate.getDate() + diff + weekendCount);
            proxy.endDate = proxy._getFormatedDate(modifiedEndDate, dateFormat);
        },

        _calculateFormatedDate: function (date, format) {
            return Globalize.format(date, format);
        },

        _updateParentProgress: function (parent,progressMapping) {
            var parentProgress = 0,
                childCount = parent.childRecords && parent.childRecords.length,
                totalDuration = 0,
                completedDuration = 0;


            if (parent.childRecords) {
                for (var i = 0; i < childCount; i++) {

                    //converting days into hours .
                    //By now 24 hours is default
                    totalDuration += (parent.childRecords[i].duration * 24);

                    //summation of progress done of child
                    completedDuration += ((parent.childRecords[i].duration * 24) * parent.childRecords[i].status) / 100;
                  
                }

                parentProgress = (completedDuration / totalDuration) * 100;
                if (isNaN(parentProgress))
                    parentProgress = 0;
                parent.progressWidth = this._getProgressWidth(parent.width, parentProgress);
                parent.status = Math.round(parentProgress);
                if (progressMapping)
                    parent.item[progressMapping] = parent.status;

            }
            if (parent.parentItem) {
                this._updateParentProgress(parent.parentItem, progressMapping);
            }
        },


        _getProgressWidth: function (parentwidth, percent) {
            return (parentwidth * percent) / 100;
        },

        _calculatePredecessor: function (string) {
            var collection = [],
                match,
                obj = {},
                values,
                to = this.taskId.toString(),
                offsetvalue;
            string.split(',').forEach(function (el) {
                values = el.split('+');
                offsetvalue = '+';
                if (el.indexOf('-') >= 0) {
                    values = el.split('-');
                    offsetvalue = '-';
                }
                match = values[0].match(/(\d+|[A-z]+)/g);
                obj = {
                    from: match[0],
                    predecessorsType: match.length > 1 ? match[1].toUpperCase() : "FS",
                    offset: values.length > 1 ? offsetvalue + "" + values[1] : "",
                    isdrawn: false,
                    to: to
                };
                collection.push(obj);
            });
            return collection;
        },

        _updateGanttRecord: function (startDateMappingName, endDateMappingName, durationMappingName, includeWeekend, scheduleStartDay, perDayWidth, dateFormat, holidays) {
            var proxy = this,
                remainDays = proxy.left % perDayWidth,
                day = (proxy.left - remainDays) / perDayWidth,holiday
                startDate = new Date(scheduleStartDay);

            startDate.setDate(scheduleStartDay.getDate() + day);

            if (startDate.getDay() == 0)
                startDate.setDate(startDate.getDate() + 1);
            else if (startDate.getDay() == 6)
                startDate.setDate(startDate.getDate() + 2);

            proxy.startDate = proxy._getFormatedDate(startDate, dateFormat);
            proxy.left = proxy._calculateLeft(scheduleStartDay, perDayWidth, includeWeekend, dateFormat, endDateMappingName);
            proxy.width = proxy._calculateWidth(perDayWidth, includeWeekend, holidays, dateFormat);
            if (proxy.parentItem)
                proxy._updateParentItem(proxy, startDateMappingName, endDateMappingName, durationMappingName, includeWeekend, scheduleStartDay, perDayWidth, dateFormat, holidays);
        },

        _calculateUpdatedDate: function (modifiedDate, offset) {
            var proxy = this,
                i = 0;
            if (offset > 0) {
                for (i; i < offset; i++) {
                    modifiedDate = proxy._dataSubtract(modifiedDate, 1);
                    if (modifiedDate.getDay() == 0)
                        modifiedDate.setDate(modifiedDate.getDate() + 1);
                    else if (modifiedDate.getDay() == 6)
                        modifiedDate.setDate(modifiedDate.getDate() + 2);
                }
            } else {
                var dateOffset = (-1) * offset;
                for (i = 0; i < dateOffset; i++) {
                    modifiedDate = proxy._dataSubtract(modifiedDate, -1);
                    if (modifiedDate.getDay() == 0)
                        modifiedDate.setDate(modifiedDate.getDate() - 2);
                    else if (modifiedDate.getDay() == 6)
                        modifiedDate.setDate(modifiedDate.getDate() - 1);
                }
            }
            modifiedDate = proxy._getFormatedDate(modifiedDate);
            return modifiedDate;
        },

        _dataSubtract: function (date, days) {
            return new Date(
                date.getFullYear(),
                date.getMonth(),
                date.getDate() + days,
                date.getHours(),
                date.getMinutes(),
                date.getSeconds(),
                date.getMilliseconds()
            );
        },

        _calculateEndDate: function (includeWeekend, holidays,model, perDayWidth) {
            var proxy = this,
                count = 0,
                startDate = proxy._getDateFromFormat(proxy.startDate, model.dateFormat),
                endDate = proxy._getDateFromFormat(proxy.endDate, model.dateFormat),
                endDateHours = 0, endDateMinutes = 0,
                duration = 0,// proxy.duration && parseInt(proxy.duration),
                remainingHours = 0, totalLength, remainingLength;

            if (endDate) {
                endDateHours = endDate.getHours();
                endDateMinutes = endDate.getMinutes();
            }

            if (model.durationUnit == ej.Gantt.DurationUnit.Hour) {
                if (model.workingTimeScale == ej.Gantt.workingTimeScale.TimeScale24Hours) {
                    if (!model.endDateMapping) {
                        var totalHours = startDate.getHours() + proxy.duration;
                        endDateHours = (parseInt(totalHours % 24) > 0) ? parseInt(totalHours % 24) : 0;
                    } else {
                        if (endDate) {
                            endDateHours = endDate.getHours();
                            endDateMinutes = endDate.getMinutes();
                        } else {
                            endDateHours = (parseInt(proxy.duration % 24) > 0) ? parseInt(proxy.duration % 24) : 0;
                        }
                    }
                    duration = endDateHours > 0 ? parseInt((startDate.getHours() + proxy.duration) / 24) + 1 :
                        parseInt(startDate.getHours() + proxy.duration) / 24;
                   
                }
                else if (model.workingTimeScale == ej.Gantt.workingTimeScale.TimeScale8Hours) {
                    if (!model.endDateMapping) {
                        var totalHours = startDate.getHours() + proxy.duration;
                        endDateHours = (parseInt(totalHours % 8) > 0) ? parseInt(totalHours % 8) : 0;
                        endDateHours = proxy._calculateTimeScale8Hour(startDate.getHours(), proxy.duration) % 24;
                    }
                    else {
                        if (endDate) {
                            endDateHours = endDate.getHours();
                            endDateMinutes = endDate.getMinutes();
                        } else {
                            endDateHours = proxy._calculateTimeScale8Hour(startDate.getHours(), proxy.duration) % 24;
                        }
                    }
                    var endHour = proxy._calculateTimeScale8Hour(startDate.getHours(), proxy.duration);
                    var calculatedEndHour = parseInt(endHour / 24) < 1 ? 1 : parseInt(endHour / 24);
                    var checkEndDate = proxy._dateUpdate(startDate, 0, (endHour % 24));
                    if (checkEndDate.getHours() >= 8) {
                        calculatedEndHour += 1;
                    }
                    duration = endDateHours > 0 ? Math.round(calculatedEndHour)
                        : parseInt(proxy.duration) / 8;

                }
                 
                //duration = parseInt(proxy.duration);
            }
            else {
                duration = parseInt(proxy.duration);
            }

            
            if (proxy.width > 0) {
                totalLength = proxy.left + proxy.width;
                remainingLength = totalLength % perDayWidth;
                remainingHours = Math.round(remainingLength / (perDayWidth / 24));//hour calculation
            }

            if (duration === 0) {
                proxy.endDate = proxy.startDate;
                return;
            }

            if (includeWeekend === false) {
                for (count = 1; count < duration; count++) {
                    if (startDate.getDay() == 0)
                        startDate.setDate(startDate.getDate() + 1);
                    else if (startDate.getDay() == 6)
                        startDate.setDate(startDate.getDate() + 2);

                    startDate.setDate(startDate.getDate() + 1);
                }
            } else {
                startDate.setDate(startDate.getDate() + duration - 1);
            }

            if (includeWeekend === false) {
                if (startDate.getDay() == 0)
                    startDate.setDate(startDate.getDate() + 1);
                else if (startDate.getDay() == 6)
                    startDate.setDate(startDate.getDate() + 2);
            }

            if (holidays && holidays.length) {
                var holidayIndex = 0,
                    holidayslength = holidays.length,
                    holiday;
                for (holidayIndex = 0; holidayIndex < holidayslength; holidayIndex++) {
                    holiday = new Date(holidays[holidayIndex]);
                    if (holiday >= proxy._getDateFromFormat(proxy.startDate, model.dateFormat) && holiday <= startDate) {
                        startDate.setDate(startDate.getDate() + 1);
                        if (includeWeekend === false) {
                            if (startDate.getDay() == 0) {
                                startDate.setDate(startDate.getDate() + 1);
                            } else if (startDate.getDay() == 6) {
                                startDate.setDate(startDate.getDate() + 2);
                            }
                        }
                    }
                }
            }

            if (includeWeekend === false) {
                if (startDate.getDay() == 0)
                    startDate.setDate(startDate.getDate() + 1);
                else if (startDate.getDay() == 6)
                    startDate.setDate(startDate.getDate() + 2);
            }
            if (remainingHours > 0)
                startDate.setHours(remainingHours);//endDateHours
            if (endDateHours > 0) {
                if ((model.workingTimeScale == ej.Gantt.workingTimeScale.TimeScale8Hours) &&
                    model.durationUnit == ej.Gantt.DurationUnit.Hour) {
                    startDate.setHours(endDateHours);
                }
                else
                    startDate.setHours(endDateHours);
            }
            
            if (model.dateFormat.indexOf("hh:") &&
                      model.workingTimeScale == ej.Gantt.workingTimeScale.TimeScale8Hours &&
                model.scheduleHeaderSettings.scheduleHeaderType != ej.Gantt.ScheduleHeaderType.Day) {
                startDate.setHours(17);
            }
            else if (model.workingTimeScale == ej.Gantt.workingTimeScale.TimeScale24Hours
                && model.durationUnit == ej.Gantt.DurationUnit.Hour) {
                if (endDateHours > 0) {
                    startDate.setHours(endDateHours);
                }
                else
                    startDate.setHours(24);
            }

            
            proxy.endDate = proxy._calculateFormatedDate(startDate, model.dateFormat);
            if (model.endDateMapping)
                proxy.item[model.endDateMapping] = proxy.endDate;
        },

        _calculateTimeScale8Hour: function (startHour, duration) {
            var endHour = startHour;

            do {
                if ((startHour >= 8 && startHour < 13) || (startHour > 13 && startHour <= 17)) {
                    endHour++;
                    duration--;
                    startHour = startHour == 24 ? 0 : startHour + 1;
                }
                else {
                    startHour = startHour == 24 ? 0 : startHour + 1;
                    
                    endHour++;
                }
            } while (duration > 0);

            return endHour;
        },

        _dateUpdate: function (date, days,hours) {
            return new Date(
                date.getFullYear(),
                date.getMonth(),
                date.getDate() + days,
                date.getHours() + hours,
                date.getMinutes(),
                date.getSeconds(),
                date.getMilliseconds()
            );
        },


        _calculateDuration: function (startDate, endDate, includeWeekend, holidays, milestone,model) {
            var proxy = this,
            dateDiff;

            if (model.durationUnit == ej.Gantt.DurationUnit.Hour) {
                dateDiff = proxy._hourdiff(startDate, endDate);
                if (dateDiff < 0) {
                    //dateDiff = dateDiff * -1;
                }
                if (model.workingTimeScale == ej.Gantt.workingTimeScale.TimeScale8Hours) {
                }
            }
            else {
                dateDiff = proxy._daydiff(startDate, endDate);
            }

            if (includeWeekend === false)
                offset = proxy._calculateDateDifference(new Date(startDate), dateDiff);
            else {
                if (model.durationUnit != ej.Gantt.DurationUnit.Hour)
                    offset = dateDiff + 1;
                else
                    offset = dateDiff;
            }

            for (var count = 0; count < holidays.length; count++) {
                if (new Date(holidays[count].day) >= startDate && new Date(holidays[count].day) <= endDate) {
                    if (model.durationUnit == ej.Gantt.DurationUnit.Hour)
                        offset -= 24;
                    else
                        offset--;
                }
            }

            if (model.workingTimeScale == ej.Gantt.workingTimeScale.TimeScale24Hours ||
                model.durationUnit == ej.Gantt.DurationUnit.Hour) {

                var decimal = (offset - parseInt(offset, 10));
                decimal = (decimal * 10);
                if (decimal == 5) { offset = (parseInt(offset, 10) + 0.5); }

                else {
                    offset = Math.round(offset);
                }
            }
            //else if (model.scheduleHeaderSettings.scheduleHeaderType == ej.Gantt.ScheduleHeaderType.Day) {

            //    offset = Math.round(offset) - 1;
            //}
            else
                offset = Math.round(offset);

            if (ej.isNullOrUndefined(milestone))
                proxy.duration = offset;
            else
                proxy.duration = (offset == 0 && milestone == false) ? 1 : offset;
        },

        _calculateDateDifference: function (emptyDate, dateDiff) {
            var differenceCount = dateDiff >= 0 ? dateDiff : (-1 * dateDiff),
                offset = 0;

            if (dateDiff > 0)
                offset = differenceCount - this._calculateWeekendCount(emptyDate, dateDiff) + 1;
            else
                offset = dateDiff + this._calculateWeekendCount(emptyDate, dateDiff);
            return offset;
        },

        _calculateWeekendCount: function (emptyDate, dateDiff) {
            var differenceCount = dateDiff >= 0 ? dateDiff : (-1 * dateDiff),
                weekendCount = 0,
                i = 0;

            if (dateDiff > 0) {
                for (i = 0; i < dateDiff; i++) {
                    if (emptyDate.getDay() == 0) {
                        weekendCount++;
                        emptyDate.setDate(emptyDate.getDate() + 1);
                    }
                    else if (emptyDate.getDay() == 6) {
                        weekendCount += 2;
                        ++i;
                        emptyDate.setDate(emptyDate.getDate() + 2);
                    } else
                        emptyDate.setDate(emptyDate.getDate() + 1);
                }
            }
            else {
                for (i = 0; i < differenceCount; i++) {
                    emptyDate.setDate(emptyDate.getDate() - 1);
                    if (emptyDate.getDay() == 0 || emptyDate.getDay() == 6) {
                        weekendCount += 1;
                    }
                }
            }
            return weekendCount;
        },

        _daydiff: function (first, second) {
            return ((second - first) / (1000 * 60 * 60 * 24));
        },
               
        _hourdiff: function (first, second) {
            //if(first.getTime()>second.getTime())
            //    return ((first - second) / (1000 * 60 * 60));
            //else
                return ((second - first) / (1000 * 60 * 60));
        },

        _getProgressPercent: function (parentwidth, progresswidth) {
            return Math.ceil(((progresswidth / parentwidth) * 100).toFixed(2));
        },

    };

    ej.Gantt._getPredecessorsValue = function (mappingName) {
        return this.data.item ? this.data.item[mappingName] : "";
    };    

    ej.Gantt._getCellValue = function (columnName) {

        var cellValue = this.data[columnName];

        if (cellValue) {
            return cellValue;
        } else {
            return this.data.item && this.data.item[columnName];
        }


    };

})(jQuery, Syncfusion);;
/**
 * @fileOverview Plugin to style the Html TreeGrid elements
 * @copyright Copyright Syncfusion Inc. 2001 - 2014. All rights reserved.
 *  Use of this code is subject to the terms of our license.
 *  A copy of the current license can be obtained at any time by e-mailing
 *  licensing@syncfusion.com. Any infringement will be prosecuted under
 *  applicable laws. 
 * @version 12.1 
 * @author <a href="mailto:licensing@syncfusion.com">Syncfusion Inc</a>
 */

(function ($, ej, undefined) {
    /**
* @namespace ej
* @class ejTreeGrid
* @requires jQuery
* @requires jquery.globalize.js
* @requires jquery.easing.1.3.js
* @requires jsrender.js
* @requires ej.web.all.js
* @classdesc Custom Design for Html Tree grid control.
* @example 
* &lt;treegrid id="treegrid"&gt;treegrid&lt;/treegrid&gt; <br> 
* &lt;script&gt;
* // Create TreeGrid
* $('#treegrid').ejTreeGrid(); 	
* &lt;/script&gt;
*/
    ej.widget("ejTreeGrid", "ej.TreeGrid", {
        // widget element will be automatically set in this
        _rootCSS: "e-treegrid",

        // widget element will be automatically set in this
        element: null,

        validTags: ["div"],

        // user defined model will be automatically set in this
        model: null,

        keyConfigs: {
            focus: "e",
            editRecord: "113", //F2
            saveRequest: "13", // enter
            cancelRequest: "27", //Esc         
            firstRowSelection: "36", //"CtrlPlusHome",
            lastRowSelection: "35", //"CtrlPlusEnd",
            leftArrow:"37",//Left arrow
            upArrow: "38", //Up arrow
            rightArrow:"39",//Right arrow
            downArrow: "40", //Down arrow
            moveCellRight: "9", //tab
            moveCellLeft: "shift+9", //shifttab
            selectedRowExpand: "alt+40", //"AltPlusDownArrow",
            totalRowExpand: "ctrl+40", //"CtrlPlusDownArrow",
            selectedRowCollapse: "alt+38", //"AltPlusUpArrow",
            totalRowCollapse: "ctrl+38" //"CtrlPlusUpArrow"
        },
      
        defaults: {
            /**		
			* Enables or disables sorting. When enabled, we can sort the column by clicking on the column. 
			* @default false
			* @type {boolean}
			* @example 
			* &lt;treegrid id="treegrid"&gt;treegrid&lt;/treegrid&gt; <br> 
			* &lt;script&gt;
			 * 	$("#treegrid").ejTreeGrid({  editSettings:{allowSorting : true} });
			* &lt;/script&gt;
			* @memberof ejTreeGrid
			* @instance
			*/
            allowSorting: false,
            /**		
             * Enables or disables the ability to resize column.
             * @default false
             * @type {boolean}
             * @example 
             * &lt;div id="treegrid"&gt;&lt;/div&gt; 
             * &lt;script&gt;                   
             * 	$("#treegrid").ejTreeGrid({ allowColumnResize:  true });			* 
             * &lt;/script&gt;
             * @memberof ejTreeGrid
             * @instance
             */
            allowColumnResize: false,
           
            /**		
            * Enables or disables the interactive selection of a row.
            * @default true
            * @type {boolean}
            * @example 
            * &lt;div id="treegrid"&gt;&lt;/div&gt; 
            * &lt;script&gt;   
            * 	$("#treegrid").ejTreeGrid({ allowSelection:  true });			* 
            * &lt;/script&gt;
            * @memberof ejTreeGrid
            * @instance
            */
            allowSelection: true,
           
            /**		
           * Collection of data or hierarchical data  to represent in  TreeGrid
           * @default 
           * @type {array}
           * @example 
           * &lt;div id="treegrid"&gt;&lt;/div&gt; 
           * &lt;script&gt; 		
           * 	$("#treegrid").ejTreeGrid(
           *  {
           *     dataSource:[{Id:2,TaskName:"Testing",startDate:"12/1/2000",Duration:5 }]	
           *  });		
           * &lt;/script&gt;
           * @memberof ejTreeGrid
           * @instance
           */

            dataSource: null,
           
            /**		
            * It receives query to retrieve data from the table (query is same as SQL). 
            * @default 
            * @type {object}
            * @example
            * &lt;div id="treegrid"&gt;&lt;/div&gt; 
            * &lt;script&gt;  			
            * 	$("#treegrid").ejTreeGrid(
            *  {
            *     query:ej.Query().from("Categories").select("CategoryID,CategoryName").take(3);	
            *  });		
            * &lt;/script&gt;
            * @memberof ejTreeGrid
            * @instance
            */
            query: ej.Query(),
            /**		
           * Specifies the mapping property path for child Id in self reference datasource
           * @default ""
           * @type {string}
           * @example 
           * &lt;div id="treegrid"&gt;&lt;/div&gt; 
           * &lt;script&gt;  		
           * 	$("#treegrid").ejTreeGrid({  IdMapping : "ID" });	            
           * &lt;/script&gt;
           * @memberof ejTreeGrid
           * @instance
           */
            idMapping: "",


            /**		
            * Specifies the mapping property path for parent Id in self reference datasource
            * @default ""
            * @type {string}
            * @example 
            * &lt;div id="treegrid"&gt;&lt;/div&gt; 
            * &lt;script&gt;  		
            * 	$("#treegrid").ejTreeGrid({  parentIdMapping : "ID" });	            
            * &lt;/script&gt;
            * @memberof ejTreeGrid
            * @instance
            */
            parentIdMapping: "",
            
            /**		
           * Specifies whether to show grid cell tooltip.
           * @default "true"
           * @type {boolean}
           * @example 
           * &lt;div id="treegrid"&gt;&lt;/div&gt; 
           * &lt;script&gt; 			
           * 	$("#treegrid").ejTreeGrid({  
           *			showGridCellTooltip : true});
           * &lt;/script&gt;
           * @memberof ejTreeGrid
           * @instance
           */
            showGridCellTooltip: false,

            /**		
            *Specifies whether to show grid cell tooltip over expander cell alone.
            * @default "true"
            * @type {boolean}
            * @example 
            * &lt;div id="treegrid"&gt;&lt;/div&gt; 
            * &lt;script&gt;			
            * 	$("#treegrid").ejTreeGrid({  
            *			showGridExpandCellTooltip : true});
            * &lt;/script&gt;
            * @memberof ejTreeGrid
            * @instance
            */

            showGridExpandCellTooltip: false,

            /**		
           * Enables/Disables enableAltRow row effect in TreeGrid
           * @default true
           * @type {boolean}
           * @example 
           * &lt;div id="treegrid"&gt;&lt;/div&gt; 
           * &lt;script&gt;			
           * 	$("#treegrid").ejTreeGrid({ enableAltRow : true});			
           * &lt;/script&gt;
           */
    enableAltRow: false,
            /* editing API's */
            /**		
            * Specifies the editSettings options in treegrid. 
            * @type {object}
            * @memberof ejTreeGrid
            * @instance
            */
            editSettings: {
                /**		
           * specifies the option for enabling / disablig editing in treegrid part
           * @alias ejTreeGrid#editSettings->allowEditing
           * @default false
           * @type {boolean}
           * @example 
           * &lt;div id="treegrid"&gt;&lt;/div&gt; 
           * &lt;script&gt;			
           * 	$("#treegrid").ejTreeGrid({  editSettings:{allowEditing : true} });	
           * &lt;/script&gt;
           * @memberof ejTreeGrid
           * @instance
           */
                allowEditing: false,
                /**		
           * enables/disables add treegrid in toolbar
           * @alias ejTreeGrid#editSettings->allowAdding
           * @default false
           * @type {boolean}
           * @example 
           * &lt;div id="treegrid"&gt;&lt;/div&gt; 
           * &lt;script&gt; 			
           * 	$("#treegrid").ejTreeGrid({  editSettings:{allowAdding : true} });
           * &lt;/script&gt;
           * @memberof ejTreeGrid
           * @instance
           */
                allowAdding: false,
                /**		
            * enables/disables delete treegrid in toolbar
            * @alias ejTreeGrid#editSettings->allowDeleting
            * @default false
            * @type {boolean}
            * @example 
            * &lt;div id="treegrid"&gt;&lt;/div&gt; 
            * &lt;script&gt;			
            * 	$("#treegrid").ejTreeGrid({  editSettings:{allowDeleting : true} });	
            * &lt;/script&gt;
            * @instance
            */
                allowDeleting: false,
                /**		
            * specifies the editmode in treegrid , "normal" is for dialog editing ,"cellEditing" is for cell type editing
            * @alias ejTreeGrid#editSettings->editMode
            * @default normal
            * @type {string}
            * @example 
            * &lt;div id="treegrid"&gt;&lt;/div&gt; 
            * &lt;script&gt;			
            * 	$("#treegrid").ejTreeGrid({  editSettings:{editMode : true} });
            * &lt;/script&gt;
            * @instance
            */
                editMode: "normal"
            },

            enableAltRow: true,         
                       
            /**		
            * Enables/Disables virtualization for rendering TreeGrid items.
            * @default false
            * @type {boolean}
            * @example 
            * &lt;div id="treegrid"&gt;&lt;/div&gt; 
            * &lt;script&gt;			
            * 	$("#treegrid").ejTreeGrid({ enableVirtualization : true});			
            * &lt;/script&gt;
            * @memberof ejTreeGrid
            * @instance
            */
            enableVirtualization: false,
                    
            /**		
           * Specifies enabling / diabling multiple sorting for tree grid columns
           * @default false
           * @type {boolean}
           * @example 
           * &lt;div id="treegrid"&gt;&lt;/div&gt; 
           * &lt;script&gt;			
           * 	$("#treegrid").ejTreeGrid({ allowMultiSorting : true});			
           * &lt;/script&gt;
           * @memberof ejTreeGrid
           * @instance
           */
            allowMultiSorting: false,
          
            /**		
            *  Specifes the sorting options for Treegrid.
            * @type {object}
            * @memberof ejTreeGrid
            * @instance
            */
            sortSettings: {
                /**		
           * Specifes the sorted columns for Tree grid
           * @alias ejTreeGrid#sortSettings->sortedColumns
           * @default []
           * @type {array}
           * @example 
           * &lt;div id="treegrid"&gt;&lt;/div&gt; 
           * &lt;script&gt;		
           * 	$("#treegrid").ejTreeGrid({ sortSettings{sortedColumns : []}});	           
           * &lt;/script&gt;
           * @instance
            */
                sortedColumns: []
            },

           

            /**		
           * Specifes the single or multiple row selection           
           * @default "single"
           * @type {string}
           * @example 
           * &lt;div id="treegrid"&gt;&lt;/div&gt; 
           * &lt;script&gt;		
           * 	$("#treegrid").ejTreeGrid({ selectionType:"multiple" });	           
           * &lt;/script&gt;
           * @memberof ejTreeGrid
           * @instance
            */

            selectionType: "single",
           
            /**		
          * Specifies the selected row Index in Treegrid , the row with given index is highlighted
          * @default  "-1"
          * @type {number}
          * @example 
          * &lt;div id="treegrid"&gt;&lt;/div&gt; 
          * &lt;script&gt;			
          * 	$("#treegrid").ejTreeGrid(
          *  {
          *     selectedRowIndex:2
          *  });		
          * &lt;/script&gt;
          * @memberof ejTreeGrid
          * @instance
          */
            selectedRowIndex: -1,
            
            selectedItem: null,
          
            /**		
            * Enables/Disables Keyboard navigation in treegrid
            * @default true
            * @type {boolean}
            * @example 
            * &lt;div id="treegrid"&gt;&lt;/div&gt; 
            * &lt;script&gt; 			
            * 	$("#treegrid").ejTreeGrid({ allowKeyboardNavigation : true});			
            * &lt;/script&gt;
            * @memberof ejTreeGrid
            * @instance
            */
            
            allowKeyboardNavigation: true,

           
            /**		
             * Specify the CSS class to tree grid to achieve custom theme.
             * @default ""
             * @type {string}
             * @example 
             * &lt;div id="treegrid"&gt;&lt;/div&gt; 
             * &lt;script&gt;		
             * 	$("#treegrid").ejTreeGrid({  cssClass : "gradient-lime" });
             * &lt;/script&gt;
             * @memberof ejTreeGrid
             * @instance
            */
            cssClass: "",
           
            locale: "en-US",

            isEdit: false,

            allowScrolling: false,
            groupSettings: {
                groupedColumns:[]
            },

            /* event function */
            /**    
            * Triggered while rendering each row
            * @event
            * @name ejTreeGrid#rowDataBound 
            * @param {Object} argument Arguments when rowDataBound event is triggered.  
            * @param {boolean} argument.cancel Returns the cancel option value.
            * @param {object} argument.rowElement Returns the row element of editing cell.
            * @param {object} argument.data Returns the data of edited cell record..
            * @example 
            * &lt;div id="treegrid"&gt;&lt;/div&gt; 
            * &lt;script&gt;
            * $("#treegrid").ejTreeGrid({
            *    rowDataBound: function (args) {}
            * });
            * &lt;/script&gt;
            * @instance
            */
            rowDataBound: null,
            /**    
            * Triggered while rendering each cell in the grid tree
            * @event
            * @name ejTreeGrid#queryCellInfo 
            * @param {Object} argument Arguments when queryCellInfo event is triggered.  
            * @param {boolean} argument.cancel Returns the cancel option value.
            * @param {object} argument.rowElement Returns the selecting row element.
            * @param {string} argument.cellValue Returns the value of cell.
            * @param {object} argument.data Returns the data of current cell record.
            * @param {object} argument.column Returns the column of cell belongs.
            * @example 
            * &lt;div id="treegrid"&gt;&lt;/div&gt; 
            * &lt;script&gt;
            * $("#treegrid").ejTreeGrid({
            *    queryCellInfo: function (args) {}
            * });
            * &lt;/script&gt;
            * @instance
            */
            queryCellInfo: null,

            /**    
			* Triggered before the row is going to be selected.
			* @event
			* @name ejTreeGrid#rowSelecting 
			* @param {Object} argument Arguments when rowSelecting event is triggered.  
			* @param {boolean} argument.cancel Returns the cancel option value.
			* @param {object} argument.rowElement Returns the selecting row element.
			* @param {string} argument.recordIndex Returns the index of selecting row record.
			* @param {object} argument.data Returns the data selecting record.
			* @example 
			* &lt;div id="treegrid"&gt;&lt;/div&gt; 
            * &lt;script&gt;
			* $("#treegrid").ejTreeGrid({
			*    rowSelecting: function (args) {}
			* });
			* &lt;/script&gt;
            * @memberof ejTreeGrid
			   * @instance
            */
            rowSelecting: null,
            /**    
			* Triggered after the row is selected.
			* @event
			* @name ejTreeGrid#rowSelected 
			* @param {Object} argument Arguments when rowSelected event is triggered.  
			* @param {boolean} argument.cancel Returns the cancel option value.
			* @param {object} argument.rowElement Returns the selecting row element.
			* @param {string} argument.recordIndex Returns the index of selecting row record.
			* @param {object} argument.data Returns the data of selected record.
			* @example 
			* &lt;div id="treegrid"&gt;&lt;/div&gt; 
            * &lt;script&gt;
			* $("#treegrid").ejTreeGrid({
			*    rowSelected: function (args) {}
			* });
			* &lt;/script&gt;
            * @memberof ejTreeGrid
			   * @instance
            */
            rowSelected: null,
            /**    
           * Triggered while enter the edit mode in the grid tree cell
           * @event
           * @name ejTreeGrid#beginEdit 
           * @param {Object} argument Arguments when beginEdit event is triggered.  
           * @param {boolean} argument.cancel Returns the cancel option value.
           * @param {object} argument.rowElement Returns the row element of editing cell.
           * @param {object} argument.cellElement Returns the Element of editing cell.
           * @param {object} argument.data Returns the data of current cell record.
           * @param {number} argument.columnIndex Returns the column Index of cell belongs.
           * @example 
           * &lt;div id="treegrid"&gt;&lt;/div&gt; 
           * &lt;script&gt;
           * $("#treegrid").ejTreeGrid({
           *    beginEdit: function (args) {}
           * });
           * &lt;/script&gt;
           * @memberof ejTreeGrid
              * @instance
           */
            beginEdit: null,

            /**    
            * Triggered after the save the modified cellValue in treegrid
            * @event
            * @name ejTreeGrid#endEdit 
            * @param {Object} argument Arguments when endEdit event is triggered.  
            * @param {boolean} argument.cancel Returns the cancel option value.
            * @param {object} argument.rowElement Returns the row element of editing cell.
            * @param {object} argument.cellElement Returns the Element of editing cell.
            * @param {object} argument.data Returns the data of edited cell record.
            * @param {string} argument.columnName Returns the column name of edited cell belongs.
            * @param {object} argument.columnObject Returns the column object of edited cell belongs.
            * @example 
            * &lt;div id="treegrid"&gt;&lt;/div&gt; 
            * &lt;script&gt;
            * $("#treegrid").ejTreeGrid({
            *    endEdit: function (args) {}
            * });
            * &lt;/script&gt;
            * @memberof ejTreeGrid
			   * @instance
            */
            endEdit: null,

            /**    
            * Triggered  while expanding the TreeGrid record
            * @event
            * @name ejTreeGrid#expanding 
            * @param {Object} argument Arguments when expanding event is triggered.  
            * @param {boolean} argument.cancel Returns the cancel option value.
            * @param {object} argument.recordIndex Returns the row index of record.
            * @param {object} argument.data Returns the data of edited cell record..
            * @example 
            * &lt;div id="treegrid"&gt;&lt;/div&gt; 
            * &lt;script&gt;
            * $("#treegrid").ejTreeGrid({
            *    expanding: function (args) {}
            * });
            * &lt;/script&gt;
            * @memberof ejTreeGrid
			   * @instance
            */
            expanding: null,

            /**    
            * Triggered  after expand the record
            * @event
            * @name ejTreeGrid#expanded 
            * @param {Object} argument Arguments when expanded event is triggered.  
            * @param {boolean} argument.cancel Returns the cancel option value.
            * @param {object} argument.recordIndex Returns the row index of record.
            * @param {object} argument.data Returns the data of edited cell record..
            * @example 
            * &lt;div id="treegrid"&gt;&lt;/div&gt; 
            * &lt;script&gt;
            * $("#treegrid").ejTreeGrid({
            *    expanded: function (args) {}
            * });
            * &lt;/script&gt;
            * @memberof ejTreeGrid
			   * @instance
            */
            expanded: null,
            /**    
            * Triggered while collapsing the TreeGrid record
            * @event
            * @name ejTreeGrid#collapsing 
            * @param {Object} argument Arguments when collapsing event is triggered.  
            * @param {boolean} argument.cancel Returns the cancel option value.
            * @param {object} argument.recordIndex Returns the row index of collapsing record.
            * @param {object} argument.data Returns the data of edited cell record..
            * @example 
            * &lt;div id="treegrid"&gt;&lt;/div&gt; 
            * &lt;script&gt;
            * $("#TreeGrid").ejTreeGrid({
            *    collapsing: function (args) {}
            * });
            * &lt;/script&gt;
            * @memberof ejTreeGrid
			   * @instance
            */
            collapsing: null,
            /**    
            * Triggered after collapsed the TreeGrid record
            * @event
            * @name ejTreeGrid#collapsed 
            * @param {Object} argument Arguments when collapsed event is triggered.  
            * @param {boolean} argument.cancel Returns the cancel option value.
            * @param {object} argument.recordIndex Returns the row index of collapsed record.
            * @param {object} argument.data Returns the data of edited cell record..
            * @example 
            * &lt;div id="treegrid"&gt;&lt;/div&gt; 
            * &lt;script&gt;
            * $("#treegrid").ejTreeGrid({
            *    collapsed: function (args) {}
            * });
            * &lt;/script&gt;
            * @memberof ejTreeGrid
			   * @instance
            */
            collapsed: null,

            currentViewData: [],
            flatRecords: [],
            parentRecords: [],
            records: [],

          
            /**		
            * It is to specify data values to the field. 
            * @default 
            * @type {array}
            * @example
            * &lt;div id="treegrid"&gt;&lt;/div&gt; 
            * &lt;script&gt;  			
            * 	$("#treegrid").ejTreeGrid(
            *  {
            *     columns: [{ field: "Name", headerText: "Name", isTemplateColumn: true, templateID: "customColumnTemplate" },
                          { field: "Type", headerText: "Type" },
                          { field: "DateCreated", headerText: "Date Created" },
                          { field: "DateModified", headerText: "Date Modified" }]	
            *  });		
            * &lt;/script&gt;
            * @memberof ejTreeGrid
            * @instance
            */

            columns: [],
            commonWidth: 150,

            

            dateFormat: "MM/dd/yyyy",

          
            /**		
           * Specifies the height of a single row in gantt. Also, we need to set same height in the CSS style with class name e-rowcell. 
           * @default "30"
           * @type {number}
           * @example 
           * &lt;div id="treegrid"&gt;&lt;/div&gt; 
           * &lt;script&gt;				
           * 	$("#treegrid").ejTreeGrid({  
           *			rowHeight : 30,
           * 			});
           * &lt;/script&gt;
           * @instance
           */
            rowHeight: 30,

            emptyRecordText: "Empty Records To Display",
            /**		
           * Specifies the tree expander column in tree grid
           * @default  0
           * @type {number}
           * @example 
           * &lt;div id="treegrid"&gt;&lt;/div&gt; 
           * &lt;script&gt;		
           * 	$("#treegrid").ejTreeGrid(
           *  {
           *     treeColumnIndex: 1
           *  });		
           * &lt;/script&gt;
           * @memberof ejTreeGrid
           * @instance
           */
            treeColumnIndex: 0,

            //API for toggle between datepicker and datetimepicker
            workingTimeScale: 'TimeScale8Hours',

            /**		
            * Specifies the mapping property path for sub tasks in datasource 
            * @default ""
            * @type {string}
            * @example 
            * &lt;div id="treegrid"&gt;&lt;/div&gt; 
            * &lt;script&gt;				
            * 	$("#treegrid").ejTreeGrid({  childMapping : "Children" });
            * &lt;/script&gt;
            * @instance
            */
            childMapping: "",           
                      
           
            /**		
             * Enables or disables the collapse all menus for tree grid , when enabled all sub menus will disappears in tree grid
             * @default  "false"
             * @type {boolean}
             * @example 
             * &lt;div id="treegrid"&gt;&lt;/div&gt; 
             * &lt;script&gt;			
             * 	$("#treegrid").ejTreeGrid(
             *  {
             *     enableCollapseAll: false
             *  });		
             * &lt;/script&gt;
             * @memberof ejTreeGrid
                * @instance
             */
            enableCollapseAll:false,

             /**		
             * Specifies the row template for each TreeGrid row
             * @default  ""
             * @type {string}
             * @example 
             * &lt;div id="treegrid"&gt;&lt;/div&gt; 
             * &lt;script&gt;			
             * 	$("#treegrid").ejTreeGrid(
             *  {
             *     rowTemplateID: "customTemplate"
             *  });		
             * &lt;/script&gt;
             * @memberof ejTreeGrid
             * @instance
             */
            rowTemplateID: "",

             /**		
             * Specifies the alternate row template for each alternate TreeGrid row
             * @default  ""
             * @type {string}
             * @example 
             * &lt;div id="treegrid"&gt;&lt;/div&gt; 
             * &lt;script&gt;			
             * 	$("#treegrid").ejTreeGrid(
             *  {
             *     altRowTemplateID: "altRowCustomTemplate"
             *  });		
             * &lt;/script&gt;
             * @memberof ejTreeGrid
             * @instance
             */
            altRowTemplateID: ""
        },

        dataTypes: {
            columns: "array",
            sortSettings: {
                sortedColumns: "array"
            },
          //  dataSource: "data",
            currentViewData: "array",
            flatRecords: "array",
            parentRecords: "array",
            records: "array"

        },

        observables: ["selectedRowIndex","dataSource"],
        selectedRowIndex: ej.util.valueFunction("selectedRowIndex"),
        dataSource: ej.util.valueFunction("dataSource"),
        //GET THE GRID HEADER TABLE FOR COLUMNRESIZE
        getHeaderTable: function () {
            return this._$gridHeaderTable;
        },

        //SET THE GRID HEADER TABLE
        setGridHeaderTable: function (value) {
            this._$gridHeaderTable = value;
        },

        //GET THE GRIDHEADERCONTENT
        getHeaderContent: function () {
            return this._$gridHeaderContent;
        },

        //SET THE GRIDHEADERCONTENT
        setGridHeaderContent: function (value) {
            this._$gridHeaderContent = value;
        },

        //GET THE GRID CONTENT TABLE
        getContentTable: function () {
            return this._$gridContentTable;
        },

        getContent: function () {
            return this._$gridContent;
        },
        
        //SET THE GRID CONTENT TABLE
        setGridContentTable: function (value) {
            this._$gridContentTable = value;
        },

        //GET TREEGRID ROWS
        getRows: function () {
            return this._$gridRows;
        },

        //SET TREEGRID ROWS
        setGridRows: function (value) {
            this._$gridRows = value;
        },

        //GET CURRENTVIEWPORT DATA
        getCurrentViewData: function () {
            return this.model.currentViewData;
        },

        //GET FLAT RECORDS
        getFlatRecords: function () {
            return this.model.records;
        },

        //SET THE HEIGHT BASED ON THE RECORD COUNT
        setHeight: function (value) {
            this._totalHeight = value;
        },

        //GET THE UPDATEDRECORDS[SORTED,FILTERED,EXPANDCOLLAPSED]
        getUpdatedRecords: function () {
            return this._updatedRecords;
        },

        //GET THE UPDATED DATASOURCE
        getDataSource: function () {
            return this.dataSource();
        },

        //get the ids collection after perform the addition/delete operation
        //GET THE IDS COLLECTION AFTER PERFORM THE ADDITION/DELETE OPERATION
        getUpdatedIds: function () {
            return this.model.ids;
        },

        //GET THE RECORDS COUNT
        getRecordsCount: function () {
            return this._recordsCount;
        },

        //SET THE RECORDS COUNT
        setRecordsCount: function (value) {
            this._recordsCount = value;
        },

        //GET THE EXPANDCOLLAPSE RECORDS COUNT
        getExpandCollapseRecordsCount: function () {
            return this._expandCollapseRecordCount;
        },
        setCollapsedRecordCount:function(value){
            this._totalCollapseRecordCount=value; 
        },
        getCollapsedRecordCount: function () {
            return this._totalCollapseRecordCount;
        },
        
        _colgroupRefresh: function () {
            var gridheaderCol = $(this.getHeaderTable()).find('colgroup')[0];
            var gridcontentCol = $(this.getContentTable()).find('colgroup')[0];
            var headerColClone = $(gridheaderCol).clone();
            var contentColClone = $(gridcontentCol).clone();
            $(gridcontentCol).remove();
            $(gridheaderCol).remove();
            $(headerColClone).prependTo(this.getHeaderTable());
            $(contentColClone).prependTo(this.getContentTable());
        },
		
		/*CONSTRUCTOR FUNCTION*/
        _init: function () {

            var proxy = this;

            proxy._initPrivateProperties();
            
            if (proxy.model.isdesignMode) {

                proxy.element.css({ "height": "250px" });
                if (proxy.model.columns.length === 0) {
                    proxy.model.columns = [
                        { "field": "Column1", "HeaderText": "Column1" },
                        { "field": "Column2", "HeaderText": "Column2" },
                        { "field": "Column3", "HeaderText": "Column3" }
                    ];

                }
                if (proxy.model.childMapping.length == 0) {
                    proxy.model.childMapping = "Children";
                }
            }

            proxy._processEditing();
            proxy._initDatasource();

            proxy._createTooltipTemplate();
       
            //Render Records in collapsed state
            if (!proxy.model.isFromGantt && proxy.model.enableCollapseAll)
            {
                proxy.collapseAll();
            }

        },
        //#region RUNTIME UPDATE METHOD FOR TREEGRID
        //METHOD FOR UPDATE THE TREEGRID IN RUN TIME
        _setModel: function (options) {

            var proxy = this,
                model = proxy.model;

            for (var prop in options) {

                switch (prop) {

                    case "enableAltRow":
                        model.enableAltRow = options[prop];
                        proxy._addInitTemplate();
                        proxy.refreshContent();
                        break;

                    case "enableVirtualization":
                        model.enableVirtualization = options[prop];
                        proxy.refreshContent();
                        break;

                    case "allowColumnResize":
                        model.allowColumnResize = options[prop];
                        proxy._enableColumnResizeEvents();
                        break;

                    case "allowSorting":
                        model.allowSorting = options[prop];
                        break;

                    case "sortSettings":
                        $.extend(model.sortSettings, options[prop]);
                        proxy.refreshContent();
                        break;

                    case "allowSelection":
                        proxy.clearSelection();
                        model.allowSelection = options[prop];
                        break;

                    case "selectionType":
                        proxy.clearSelection();
                        model.selectionType = options[prop];
                        break;

                    case "selectedRowIndex":
                        if (this.selectedRowIndex() != -1 && $.inArray(this.selectedRowIndex(), proxy._selctedRowsIndexes == -1) && !ej.isNullOrUndefined(this.selectedRowIndex()))
                            proxy.selectRows(this.selectedRowIndex());
                        break;

                    case "editSettings":
                        $.extend(model.editSettings, options[prop]);
                        proxy._processEditing();
                        proxy._enableEditingEvents();
                        break;

                    case "allowKeyboardNavigation":
                        model.allowKeyboardNavigation = options[prop];
                        break;

                    case "dataSource":
                        proxy.resetModelCollections();
                        proxy._initDatasource();
                        args = {};
                        $(".e-tooltiptable").remove();
                        args["requestType"] = ej.TreeGrid.Actions.RefreshDataSource;
                        proxy._ensureDataSource(args);
                        if (this.selectedRowIndex() !== -1 && this.selectedRowIndex() < proxy.model.currentViewData.length)
                        {
                            proxy.selectRows(this.selectedRowIndex());
                        }
                        break;

                    case "columns":
                        $.extend(model.columns, options[prop]);
                        proxy._addInitTemplate();
                        proxy.refreshContent();
                        break;
                    
                    case "rowHeight":
                        model.rowHeight = options[prop];
                        proxy._addInitTemplate();
                        proxy.refreshContent();
                        break;

                    case "treeColumnIndex":
                        model.treeColumnIndex = options[prop];
                        proxy._addInitTemplate();
                        proxy.refreshContent();
                        break;

                }

            }

        },
                

        //REFRESH THE TREEGRIDCONTENT
       
        refreshContent: function () {

            var proxy = this,
                args = {};     
            args.requestType = ej.TreeGrid.Actions.Refresh;
            proxy.processBindings(args);
        },


        //RESET THE COLLECTIONS WHILE CHANGING THE DATASOURCE
       
        resetModelCollections: function () {

            var proxy = this,
            model = proxy.model;

           // model.sortSettings.sortedColumns = [];
            model.currentViewData = [];
            model.flatRecords = [];
            model.parentRecords = [];
            model.records = [];

        },        

        //#endregion

        //DECLARE THE PRIVATE VARIABLES
        _initPrivateProperties: function () {

            var proxy = this;

            proxy._id = proxy.element.attr("id"),
            proxy._columnsWidthCollection = [],
            proxy._cSortedColumn = null,
            proxy._cSortedDirection = null,
            proxy._dataManager = null,
            proxy._disabledSortableColumns = [],
            proxy._filteredRecords = [],
            proxy._fieldColumnNames = [],
            proxy._multiSortRequest = null,
            proxy._$gridContainer = null,
            proxy._$gridContent = null,
            proxy._$gridContentTable = null,
            proxy._$gridHeaderTable = null,
            proxy._$gridHeaderContainer = null;
            proxy._gridHeight = proxy.element.height(),
            proxy._recordsCount = null,
            proxy._$gridRows = null,
            proxy._gridRows = null,
            proxy._gridWidth = proxy.element.width(),
            proxy._headerColumnNames = [],
            proxy._hiddenColumns = [],
            proxy._multiSelectCtrlRequest = false,
            proxy._multiSelectShiftRequest = false,
            proxy._offset = 0,
            proxy._previousIndex = -1,
            proxy._prevRBottom = 0,
            proxy._prevRTop = 0,
            proxy._prevScrollTop = 0,
            proxy._prevScrollLeft = 0,
            proxy._prevVBottom = 0,
            proxy._resizer = null,
            proxy._scrollTop = 0,
            proxy._searchString = "",
            proxy._selectedRowsIndexes = [],
            proxy._sortedRecords = [],
            proxy._storedIndex = -1,
            proxy._$tableContent = null,
            proxy._tempsortedrecords = [],
            proxy._totalHeight = 0,
            proxy._viewPortHeight = 0,
            proxy._recordIndexCount=0,
            proxy._visibleColumns = [],
            proxy._visibleRange = null,
            proxy._vScrollDir = 1,
            proxy._vScrollDist = 0,
            proxy._cellEditTemplate = $(),
            proxy._cellEditingDetails = {
                cellValue: null,
                rowIndex: -1,
                columnIndex: -1,
                fieldName: null,
                _data: null,
                cellEditType: "",
                cancelSave: false,
                defaultData: null,
                insertedTrCollection: [],
                data: null
            },
            proxy._tdsOffsetWidth = [],
            proxy._isEnterKeyPressed = false,
            proxy._updatedRecords = [],
            proxy._scrollBarHeight = 18,
            proxy._expandCollapseRecordCount = 0,
            proxy._removedCount = 0,
            proxy._$gridHeaderContent = null,
            proxy._expandedRecordsCount = 0,
            proxy._totalCollapseRecordCount = 0,
            proxy._collapsedRecordCount = 0,
            proxy.secondaryDatasource = [],
            proxy._dropDownTemplate = proxy._compiledDropDownTemplate(),
            proxy._dataManager = proxy.dataSource() instanceof ej.DataManager ? proxy.dataSource() :
                proxy.dataSource() != null ? ej.DataManager(proxy.dataSource()) : null;
            
        },


        //DATASOURCE CHECKING
        _checkDataBinding: function () {

            var proxy = this,
                model = proxy.model;

            proxy.element.addClass('e-treegrid-core');
            proxy.element.attr("tabindex", "0");
            proxy._renderAfterColumnInitialize();
            proxy._viewPortHeight = proxy._getViewPortHeight(); 
            proxy._ensureDataSource();
            proxy._initGridRender();
            proxy._$gridContent = $("#" + proxy._id + "e-gridcontent");
            proxy._$gridContainer = $("#" + proxy._id + "e-gridcontainer");
            proxy._$tableContent = $("#" + proxy._id + "e-table");            
            proxy._wireEvents();            
            
            proxy.setWidthToColumns();
            
            proxy._renderScroller();
            
            if (model.isFromGantt) {
                proxy._addEmptyColumntoGrid();
            }
           
        },

        _initDatasource: function () {
            var proxy = this,
                model = proxy.model,retriveData;

            if (this.dataSource() instanceof ej.DataManager) {
           
                var queryPromise = this.dataSource().executeQuery(model.query);
                queryPromise.done(ej.proxy(function (e) {
                    retriveData = e.result;
                    proxy._reconstructDatasource(e.result);
                    proxy._createRecords(proxy.secondaryDatasource);
                    proxy._checkDataBinding();
                }));
            }
           
            else if (this.dataSource()) {
                if (model.idMapping && model.parentIdMapping) {
                    if ((model.idMapping.length > 0) && (model.parentIdMapping.length > 0)) {

                        //cloning the datasource
                        var dataArray = this.dataSource().slice(0);
                        proxy._reconstructDatasource(dataArray);
                        proxy._createRecords(proxy.secondaryDatasource);
                    }
                }
                else if (model.flatRecords.length === 0 && this.dataSource() !== null) {
                    proxy._createRecords(this.dataSource());
                }
                proxy._checkDataBinding();
            }
            
        },

        _reconstructDatasource: function (datasource) {
            var proxy = this, model = proxy.model,
                childItems, data, tempRecord = [];

            for (var i = 0; i < datasource.length; i++) {
                var data = {};
                data = ej.DataManager(datasource).executeLocal(ej.Query().where(model.parentIdMapping,
                    ej.FilterOperators.equal,
                    datasource[i][model.idMapping]));


                if ((ej.isNullOrUndefined(datasource[i][model.parentIdMapping]) || datasource[i][model.parentIdMapping] == 0)
                    && data.length > 0) {
                    datasource[i][model.childMapping] = data;
                    proxy.secondaryDatasource.push(datasource[i]);
                }
                else if (ej.isNullOrUndefined(datasource[i][model.parentIdMapping]) && !(data && data.length > 0)) {
                    proxy.secondaryDatasource.push(datasource[i]);
                }

                else if (data && data.length > 0 && !(ej.isNullOrUndefined(data[0][model.parentIdMapping]))){
                    proxy._appendChildDataItems(proxy.secondaryDatasource, data);
                }

                if (data && data.length > 0) {

                    proxy._datasourceChildItems = proxy._reconstructDatasource(data);
                    if (ej.isNullOrUndefined(datasource[i][model.childMapping])) {
                        datasource[i][model.childMapping] = proxy._datasourceChildItems;
                        proxy._getParentIndex(datasource);
                    }
                    proxy._datasourceChildItems = [];

                }

                else if (!(ej.isNullOrUndefined(datasource[i][model.parentIdMapping]))) {
                    tempRecord.push(datasource[i]);
                }
            }

            if (tempRecord && tempRecord.length > 0) {
                return tempRecord;
            }
        },

        _appendChildDataItems: function (parentDatasource,data) {
            var proxy = this,
                model = proxy.model;
            
            for (var count = 0; count < parentDatasource.length; count++) {
                
                if (parentDatasource[count][model.idMapping] === data[0][model.parentIdMapping]) {
                    parentDatasource[count][model.childMapping] = data;
                }
                else if (!(ej.isNullOrUndefined(parentDatasource[count][model.childMapping])) &&
                    parentDatasource[count][model.childMapping].length > 0) {
                    proxy._appendChildDataItems(parentDatasource[count], data);
                }
            }

            if ((ej.isNullOrUndefined(parentDatasource.length)) && parentDatasource) {
                if (parentDatasource[model.idMapping] === data[0][model.parentIdMapping]) {
                    parentDatasource[model.childMapping] = data;
                }
                else if (!(ej.isNullOrUndefined(parentDatasource[model.childMapping])) &&
                    parentDatasource[model.childMapping].length > 0) {
                    proxy._appendChildDataItems(parentDatasource[model.childMapping], data);
                }
            }

        },
        
        _getParentIndex: function (datasource) {
            var proxy = this, model = proxy.model,
                dataArr = proxy.secondaryDatasource, parentIndex = 0;

            if (datasource.length > 0) {
                parentIndex = datasource[0][model.parentIdMapping];
            }
            for (var i = 0; i < dataArr.length; i++) {
                if (dataArr[i][model.idMapping] == parentIndex && ej.isNullOrUndefined(dataArr[i][model.childMapping])) {
                    dataArr[i][model.childMapping] = (datasource);
                    return true;
                }
                else {
                    var childItems = dataArr[i][model.childMapping];
                    if (childItems) {
                        for (var i = 0; i < childItems.length; i++) {
                            if (childItems[i][model.idMapping] == parentIndex && ej.isNullOrUndefined(childItems[i][model.childMapping])) {
                                childItems[i][model.childMapping] = (datasource);
                                return true;
                            }
                        }
                    }
                }
            }

        },
        //PROCESSBINDING
        processBindings: function (args) {

            var proxy = this,
                model = proxy.model;

            model.query = new ej.Query();

            if (proxy._trigger("actionBegin", args)) {
                return true;
            }

            proxy._ensureDataSource(args);

        },

        _ensureDataSource: function (args) {

            var proxy = this,
                model = proxy.model,
                sortcolumns = model.sortSettings.sortedColumns,
                isupdatedHeight = false,
                index,
                childIndex;

            //if (model.dataSource == null) {
            //    return;
            //}

            model.query.requiresCount();
            proxy._queryManagar = model.query;

            if ((model.editSettings.allowEditing || model.editSettings.allowAdding) &&
                args && args.requestType == ej.TreeGrid.Actions.Save &&
                (!ej.isNullOrUndefined(args._cModifiedData) || !ej.isNullOrUndefined(args._cAddedRecord))) {

                if (ej.isNullOrUndefined(args._cAddedRecord)) {
                    $.extend(model.flatRecords[$.inArray(model.selectedItem, model.flatRecords)],
                        args._cModifiedData);
                } else {

                    var parentItem;

                    if (this.selectedRowIndex() === -1) {

                        args._cAddedRecord.hasChildRecords = false;
                        model.flatRecords.unshift(args._cAddedRecord);
                        model.records.unshift(args._cAddedRecord);
                        this.dataSource().unshift(args._cAddedRecord.item);

                        if (model.isFromGantt) {
                            model.ids.unshift(args._cAddedRecord.taskId);
                        }
                        args.recordIndex = 0;

                    } else {

                        args._cAddedRecord.level = model.selectedItem.level;
                        index = model.selectedItem ? model.flatRecords.indexOf(model.selectedItem) : 0;

                        if (args._cAddedRecord.level === 0) {
                            index = index > 0 ? index - 1 : index;
                        }

                        if (args.actionRequestType != ej.TreeGrid.Actions.ContextMenuAdd) {
                            args.recordIndex = index > 0 ? index - 1 : 0;
                        }

                        model.flatRecords.splice(index, 0, args._cAddedRecord);

                        var rowIndex = model.records.indexOf(model.selectedItem);
                        model.records.splice(rowIndex, 0, args._cAddedRecord);

                        if (model.isFromGantt) {
                            model.ids.splice(rowIndex, 0, args._cAddedRecord.taskId.toString());
                        }

                        if (args._cAddedRecord.level > 0) {

                            parentItem = model.selectedItem.parentItem;

                            if (parentItem) {

                                childIndex = parentItem.childRecords ?
                                    parentItem.childRecords.indexOf(model.selectedItem) : -1;

                                if (childIndex !== -1) {
                                    childIndex = childIndex === 0 ? 0 : childIndex - 1;
                                }

                                parentItem.childRecords.splice(childIndex, 0, args._cAddedRecord);

                            }
                        }

                        proxy.setRecordsCount(model.flatRecords.length);
                        proxy.setHeight(model.flatRecords.length * model.rowHeight);
                        proxy.updateHeight();
                        isupdatedHeight = true;

                    }
                    proxy._queryManagar.queries = [];
                    model.isEdit = false;
                    proxy._updatedGanttRecords = model.flatRecords;

                }
            }

            if (args && args.requestType === ej.TreeGrid.Actions.ExpandCollapse) {

                isupdatedHeight = true;
               // proxy.clearSelection(proxy.selectedRowIndex());
                proxy._filteredRecords = [];
                proxy._expandCollapseRecordCount = 0;
                proxy.updateExpandCollapseRecords(args.data, args.expanded, args.recordIndex);
                proxy.setRecordsCount(model.flatRecords.length);
                proxy.setHeight(model.flatRecords.length * model.rowHeight);
                proxy._updatedRecords = model.flatRecords;

                if (!model.enableVirtualization) {
                    if (model.allowSearching && proxy._searchString.length) {
                        proxy._updateFilteredRecords(proxy._tempfilteredrecords);
                        proxy.setRecordsCount(proxy._filteredRecords.length);
                        proxy.setHeight(proxy._filteredRecords * model.rowHeight);
                        proxy._updatedRecords = proxy._filteredRecords;
                    }

                    if (model.allowSorting && model.sortSettings.sortedColumns.length) {
                        proxy._updateExpandCollapseForSortedRecords(args.data, args.expanded);
                        proxy.setHeight(proxy._sortedRecords * model.rowHeight);
                        proxy._updatedRecords = proxy._sortedRecords;
                    }

                }
                proxy.updateHeight();

                if (model.enableAltRow) {
                    proxy._updateAltRow();
                }

            }

            if (model.editSettings.allowDeleting && args && args.requestType === ej.TreeGrid.Actions.Delete) {

                var deletedRow = args.data,
                    recordIndex,
                    rowindex;
                proxy._removedCount = 0;
                var childRecordsCount = proxy._getFilteredChildRecordsCount(deletedRow);
                childRecordsCount += 1;
                proxy._removedCount = 0;

                if (deletedRow.parentItem) {

                    var childRecords = deletedRow.parentItem.childRecords;
                    childIndex = childRecords.indexOf(deletedRow);

                    deletedRow.parentItem.childRecords.splice(childIndex, 1);
                    if(!model.parentIdMapping)
                        deletedRow.parentItem.item[proxy.model.childMapping].splice(childIndex, 1);
                }
                //Delete from paren collection if item is parent
                var parentIndex=model.parentRecords.indexOf(deletedRow);
                if (parentIndex!== -1)
                {
                    model.parentRecords.splice(parentIndex, 1);
                }
                recordIndex = proxy.model.flatRecords.indexOf(deletedRow);
                if (recordIndex!==-1)
                    proxy.model.flatRecords.splice(recordIndex, childRecordsCount);
                proxy.setRecordsCount(model.flatRecords.length);
                proxy.setHeight(model.flatRecords.length * model.rowHeight);
                isupdatedHeight = false;
                index = model.records.indexOf(deletedRow);
                var deletedRecordCount = proxy.getChildCount(deletedRow, 0);
                model.records.splice(index, deletedRecordCount + 1);
                model.ids && model.ids.splice(index, deletedRecordCount + 1);
                proxy._updatedRecords = model.flatRecords;
                if(!model.parentIdMapping)
                {
                    rowindex = this.dataSource().indexOf(deletedRow.item);
                    if (rowindex !== -1)
                        proxy.dataSource().splice(rowindex, 1);
                }
                else
                {
                    //remove child item
                    if (deletedRow.hasChildRecords && deletedRow.childRecords.length > 0)
                    {
                        proxy._removeChildItem(deletedRow);
                    }
                    //remove deleting item
                    var idx = this.dataSource().indexOf(deletedRow.item);
                    if (idx !== -1)
                        this.dataSource().splice(idx, 1);
                }
                model.selectedItem = null;
                this.selectedRowIndex(-1);

            }

            if ((model.allowSearching && proxy._searchString.length) || (args && args.requestType==="searching")) {

                proxy._queryManagar.search(proxy._searchString, proxy.getColumnFieldNames(),
                    ej.FilterOperators.contains, true);

                proxy._filteredRecords = model.records.filter(function (record) {
                    return record.hasChildRecords === false;
                });

                var dataManager = new ej.DataManager(proxy._filteredRecords);
                proxy._tempfilteredrecords = dataManager.executeLocal(proxy._queryManagar).result;
                proxy._updateFilteredRecords(proxy._tempfilteredrecords);
                proxy.setRecordsCount(proxy._filteredRecords.length);
                proxy._updatedRecords = proxy._filteredRecords;

                if (model.allowSorting && model.sortSettings.sortedColumns.length > 0) {
                    proxy._sortingRecords();
                }
                proxy.setHeight(proxy._updatedRecords.length * model.rowHeight);
                proxy.updateHeight();
                isupdatedHeight = true;

                if (model.enableAltRow) {

                    proxy._updateAltRow();
                }

            } else if (model.allowSorting && sortcolumns.length > 0) {

                isupdatedHeight = proxy._sortingRecords();
            }

            if (!isupdatedHeight) {

                proxy.setRecordsCount(model.flatRecords.length);
                proxy.setHeight(model.flatRecords.length * model.rowHeight);
                proxy._updatedRecords = model.flatRecords;
                proxy.updateHeight();

            }

            if (!args) {
                args = {};
                args.requestType = ej.TreeGrid.Actions.Refresh;
            }

            proxy._updateCurrentViewData();

            if ((!model.isFromGantt && args.requestType === "save") || args.requestType === "sorting" || args.requestType === "searching" || args.requestType === "delete" || args.requestType === "refreshDataSource") {
                proxy.renderRecords(args);
                //proxy.onScrollHelper($(".e-content").scrollTop());
            }

        },
        //remove child record in data source on self reference
        _removeChildItem: function (record) {
            var currentRecord, proxy = this;
            for (var i = 0; i < record.childRecords.length; i++) {
                currentRecord = record.childRecords[i];
                var idx = this.dataSource().indexOf(currentRecord.item);
                this.dataSource().splice(idx, 1);
                if (currentRecord.hasChildRecords) {
                    proxy._removeChildItem(currentRecord);
                }
            }
        },

        //CHECK EDITING OPTIONS 
        _processEditing:function(){

            var proxy = this,
                model = proxy.model;

            if (model.editSettings.allowEditing) {
                if (model.editSettings.editMode === "cellEditing") {
                    proxy._addCellEditTemplate();
                }
            }

        },

        //#region TREEGRIDHEADER RENDERIGN

        //TREEGRID HEADER RENDER INITIALIZATION
        _renderAfterColumnInitialize:function(){
            var proxy = this;
            proxy.element.append(proxy._renderGridHeader());
        },


        //METHOD FOR RENDERING TREEGRID HEADER
        _renderGridHeader: function () {

            var proxy = this,
                model = proxy.model,
                doc = document,
                $innerDiv = ej.buildTag('div.e-gridheadercontainer', "", { "overflow": "hidden" }, {}),
                $table = ej.buildTag('table.e-table', "", {}, { "cellspacing": "0px", "border-spacing": "0px" }),
                $thead = ej.buildTag('thead'),
                $tbody = ej.buildTag('tbody.e-hide'),
                $columnHeader = ej.buildTag('tr.e-columnheader'),
                $colGroup = $(doc.createElement('colgroup')),
                $rowBody = $(doc.createElement('tr')),
                columnCount = 0,
                columns = model.columns,
                length = columns.length,
                $headerCell,
                bodyCell,
                $headerCellDiv,
                col,
                $div = model.isFromGantt ? ej.buildTag('div.e-gridheader#' + proxy._id + "e-gridheader", "", {}, "") :
                    ej.buildTag('div.e-gridheader#' + proxy._id + "e-gridheader", "",
                {
                    "border-top-style": "solid",
                    "border-left-style": "solid",
                    "border-right-style": "solid",
                    "border-top-width": "1px",
                    "border-left-width": "1px",
                    "border-right-width": "1px",
                }, "");
            

            proxy.setGridHeaderContent($div);
            proxy._$gridHeaderContainer = $innerDiv;
            proxy._hiddenColumns = [];

            //CHECK SORTING ENABLED OR NOT
            if (model.allowSorting) {
                $columnHeader.css({ 'cursor': 'pointer' });
            }

            for (columnCount; columnCount < length; columnCount++) {

                $headerCell = ej.buildTag('th.e-headercell', "", { "height": "45px" }, {});
                bodyCell = doc.createElement('td');

                $headerCellDiv = ej.buildTag(
                    'div.e-headercelldiv',
                    columns[columnCount]["headerText"] == undefined ?
                    columns[columnCount]["headerText"] = columns[columnCount]["field"] :
                    columns[columnCount]["headerText"], {}, {'unselectable':"on"}
                );

                $headerCell.append($headerCellDiv);

                col = doc.createElement('col');

                $rowBody.append(bodyCell);
                $columnHeader.append($headerCell);
                $colGroup.append(col);


                //CHECK VISIBLE PROPERTY OF THE COLUMN
                if (columns[columnCount]["visible"] === false) {
                    $headerCell.addClass("e-hide") && $(col).css("display", "none") &&
                    proxy._hiddenColumns.push(columns[columnCount].headerText);

                } else {
                    proxy._visibleColumns.push(columns[columnCount].headerText);
                    columns[columnCount]["visible"] = true;
                }

                if (columns[columnCount]["allowSorting"] === false) {
                    proxy._disabledSortableColumns.push(columns[columnCount].headerText);
                    $headerCell.css({ 'cursor': 'normal' });
                }

                if (!ej.isNullOrUndefined(columns[columnCount]["headerTemplateID"])) {
                    $headerCellDiv.html($(columns[columnCount]["headerTemplateID"]).hide().html()).parent().addClass("headertemplate");
                }

                if (columns[columnCount]["width"] == undefined && model.commonWidth !== undefined) {
                    proxy._columnsWidthCollection[columnCount] = model.commonWidth;
                } else {
                    proxy._columnsWidthCollection.push(columns[columnCount]["width"]);
                }

                proxy._fieldColumnNames[columns[columnCount].headerText] = columns[columnCount].field;
                proxy._headerColumnNames[columns[columnCount].field] = columns[columnCount].headerText;

            }
            $thead.append($columnHeader);
            $tbody.append($rowBody);
            $table.append($colGroup).append($thead).append($tbody);

            $innerDiv.html($table);
            $div.html($innerDiv);

            proxy.setGridHeaderTable($table);

            return $div;
        },

        //#endregion

        //#region TREEGRIDCONTENT RENDERING
        //TREEGRID CONTENT RENDER INITIALIZATION
        _initGridRender: function () {

            var proxy = this;
            proxy._addInitTemplate();
            proxy._renderGrid();

            //CHECK COLUMN RESIZE IS ENABLED OR NOT AND INITIALIZE THE RESIZER
            if (proxy.model.allowColumnResize) {
                proxy._resizer = new ej.Grid.gridResize(proxy);
            }

        },


        //ADD GRIDCONTENT TO THE TREEGRID
        _renderGrid: function () {
            
            var proxy = this;
            proxy._renderGridContent().insertAfter(proxy.element.children(".e-gridheader"));
            proxy._initialEndRendering();

        },


        //RENDER THE GRIDCONTENT
        _renderGridContent:function(){
            
            var proxy = this,
                doc = document,
                model = proxy.model,
                $tbody = $(doc.createElement('tbody')),
                $div = ej.buildTag("div.e-gridcontent#" + proxy._id + "e-gridcontent", "",
                {                    
                    "height": proxy._viewPortHeight
                },
                {"unselectable":"on"}),
                $innderDiv = ej.buildTag('div.e-gridcontainer#' + proxy._id + "e-gridcontainer", "", {}, {});

            //CHECK VIRTUALIZATION ENABLED OR NOT
            if (model.enableVirtualization) {
                $innderDiv.css({ 'height': proxy._totalHeight });
            }

            var $table = ej.buildTag('table.e-table#' + proxy._id + "e-table", "",
                {top: "0px" }, { 'cellspacing': '0px' });

            $table.append(proxy.getHeaderTable().find('colgroup').clone()).append($tbody);
            $innderDiv.html($table);
            $div.html($innderDiv);
            proxy.setGridContentTable($table);

            //CHECK DATASOURCE IS DEFINED OR NOT
            if (this.dataSource() === null) {
                var $emptyTd = ej.buildTag('td', model.emptyRecordText, {}, { colSpan: model.columns.length });
                $tbody.append($(doc.createElement("tr")).append($emptyTd));
                $innderDiv.css('height', model.rowHeight);
            } else
            {
                proxy.renderRecords();
            }

            if (!model.isFromGantt) {
                $div.css({
                    "border-bottom-style": "solid",
                    "border-bottom-width": "1px",
                    "border-left-style": "solid",
                    "border-left-width": "1px",
                    "border-right-style": "solid",
                    "border-right-width": "1px",
                    "-moz-box-sizing": "border-box",
                    "width": "auto",
                    "height":"auto"
                });
            }

            $table.css({ 'position': 'relative' });
            return $div;

        },


        //AFTER INITIALIZATION END RENDERING
        _initialEndRendering: function () {

            var proxy = this,
                index = proxy.selectedRowIndex();

            if (index != -1) {

                if (!proxy._rowSelectingEventTrigger()) {

                    proxy.selectRows(index);
                    proxy._rowSelectedEventTrigger(index);

                }
            }
        },

        //#endregion
        
        //#region TEMPLATE

        //ADD CELLEDITING TEMPLATE
        _addCellEditTemplate: function () {

            var proxy = this,
                model = proxy.model,
                columns = model.columns,
                columnCount = 0,
                $outerDiv = ej.buildTag('div', "", { display: 'none' }, { id: proxy._id + "_CellEditTemplate" }),
                $innerDiv,
                length =columns&& columns.length;

            for (columnCount = 0; columnCount < length; columnCount++) {

                $innerDiv = ej.buildTag('div', "", {}, { id: columns[columnCount].field + "_CellEdit" });
                ej.TreeGrid._initCellEditType(proxy, $innerDiv, proxy._id, columnCount);
                $outerDiv.append($innerDiv);

            }

            if ($outerDiv.children().length) {
                proxy._cellEditTemplate = $outerDiv;
            }

        },


        //ADD TEMPLATE FOR RENDERING GRIDCONTENT
        _addInitTemplate:function(){

            var proxy = this,
                model = proxy.model,
                columns = model.columns,
                columnCount = 0,
                length = columns.length,
                columnName,
                TD,
                helpers = {};

            //CHECK THE COLUMNS ARRAY CONTAIN THE COLUMN OBJECT
            if (columns.length === 0) {
                return false;
            }

            helpers["_" + proxy._id + "rowClassName"] = ej.TreeGrid._getrowClassName;
            helpers["_" + proxy._id + "expandStatus"] = $.proxy(proxy._getExpandStatusRecord, proxy);         
            helpers["_" + proxy._id + "cellValue"] = proxy._getCellValue;
            

            helpers["_" + proxy._id + "itemDetails"] = $.proxy(proxy._convertItemToString, proxy);
            $.views.helpers(helpers);

            
            if (model.rowTemplateID.length > 0) {


                var temp = document.createElement('tr');

                var altRowTemplate = model.altRowTemplateID.length > 0 ? $("#" + model.altRowTemplateID)[0].innerHTML : $("#" + model.rowTemplateID)[0].innerHTML;
                var TR = "{{if isAltRow}}" + altRowTemplate
                    + "{{else}}" + $("#" + model.rowTemplateID)[0].innerHTML + "{{/if}}";

            }

            else {

                var TR = "<tr class='e-treegridrows {{:~_" + proxy._id + "rowClassName()}} ";

                if (model.enableAltRow) {
                    TR += "{{if isAltRow}}e-alt-row{{/if}} ";
                }

                TR += "{{if expanded}}e-treegridrowexpand {{else hasChildRecords}} " +
                        "e-treegridrowcollapse {{/if}} '" +
                        "style='height:" + model.rowHeight + "px;display:{{:~_" + proxy._id + "expandStatus(#data)}}'" +
                        "role='row'>";

                for (columnCount = 0; columnCount < length; columnCount++) {

                    columnName = columns[columnCount].field;

                    //CHECK THE TEMPLATE COLUMN OR NOT
                    if (columns[columnCount]["isTemplateColumn"]) {

                        var templateId = columns[columnCount]["templateID"],
                            TEMPLATE = columns[columnCount]["template"];

                        if (templateId) {

                            TD = "<td style=''" +
                                 "class='{{if isSelected}}e-rowcell e-selectionbackground e-intend {{else}}e-rowcell e-intend {{/if}}'" +
                                 "role='gridcell'" +
                                 "'tabindex='-1'" +
                                 "aria-label={{:~_" + proxy._id + "itemDetails(#data)}}>" +
                                  $("#" + templateId)[0].innerHTML +
                                 "</td>";

                        } else if (TEMPLATE) {

                            TD = "<td " +
                                 "class='{{if isSelected}}e-rowcell e-selectionbackground e-intend {{else}}" +
                                 "e-rowcell e-intend {{/if}}'" +
                                 "role='gridcell'" +
                                 "tabindex='-1'" +
                                 "aria-label={{:~_" + proxy._id + "itemDetails(#data)}}>" +
                                 TEMPLATE +
                                 "</td>";
                        }

                        //EXPAND COLLAPSE COLUMN BASED ON THE TREECOLUMNINDEX
                    } else if (columnCount === model.treeColumnIndex) {

                        TD = "<td " +
                             "class='{{if isSelected}}e-rowcell e-selectionbackground e-intend {{else}}e-rowcell e-intend {{/if}}'" +
                             "role='gridcell'" +
                             "tabindex='-1'" +
                             "aria-label={{:~_" + proxy._id + "itemDetails(#data)}}>" +
                             "<div  style='height:20px;' unselectable='on'>" +
                             "{{if hasChildRecords}}" +
                             "<div " +
                             "class='intend'" +
                             "style='height:1px; float:left; width:{{:level*20}}" + "px; display:inline-block;'>" +
                             "</div>" +
                             "{{else !hasChildRecords}}" +
                             "<div " +
                             "class='intend'" +
                             "style='height:1px; float:left; width:{{:(level+1)*20}}" + "px; display:inline-block;'>" +
                             "</div>" +
                             "{{/if}}" +
                             "<div " +
                             "class='{{if expanded}}e-treegridexpand e-icon{{else hasChildRecords}}" +
                             "e-treegridcollapse e-icon{{/if}}'" +
                             "style='float: left;display:inline-block; unselectable='on''>" +//unselectable on icon div
                             "</div>" +
                             "<div class='e-cell'" +
                             "style='display:inline-block;width:100%' unselectable='on'>" +//unselectable on
                             "{{:#data['" + columnName + "']}}" +
                             "</div>" +
                             "</div>" +
                             "</td>";
                        //CHECK RESOURCE COLUMN FROM GANTT OR NOT
                    } else if (model.isFromGantt && columns[columnCount].mappingName === model.resourceInfoMapping) {
                        helpers = {};
                        helpers["_" + proxy._id + "resourceName"] = proxy._getResourceName;
                        $.views.helpers(helpers);

                        TD = "<td  " +
                             "class='{{if isSelected}}e-rowcell e-selectionbackground{{else}}e-rowcell {{/if}}'" +
                             "role='gridcell'" +
                             "tabindex='-1'" +
                             "aria-label={{:~_" + proxy._id + "itemDetails(#data)}}>" +
                             "{{:~_" + proxy._id + "resourceName('" + proxy._id + "Object','"
                             + model.resourceNameMapping + "','" + columnName + "')}}" +
                             "</td>";
                        //CHECK PREDECESSOR COLUMN FROM GANTT OR NOT
                    } else if (model.isFromGantt && columns[columnCount].mappingName === model.predecessorMapping) {

                        helpers = {};
                        helpers["_" + proxy._id + "predecessor"] = proxy._getPredecessorsValue;
                        $.views.helpers(helpers);

                        TD = "<td " +
                            "class='{{if isSelected}}e-rowcell e-selectionbackground{{else}}e-rowcell {{/if}}' " +
                            "role='gridcell'" +
                            "tabindex='-1'" +
                            "aria-label={{:~_" + proxy._id + "itemDetails(#data)}}>" +
                            "{{:~_" + proxy._id + "predecessor('" + proxy._id + "Object','"
                            + model.predecessorMapping + "','" + columnName + "')}}" +
                            "</td>";

                    }
                    else {
                        TD = "<td " +
                            "class='{{if isSelected}}e-rowcell e-selectionbackground{{else}}e-rowcell{{/if}}'" +
                            "role='gridcell'" +
                            "tabindex='-1'" +
                            "aria-label={{:~_" + proxy._id + "itemDetails(#data)}} unselectable='on'>" +
                            "{{:~_" + proxy._id + "cellValue('" + columnName + "')}}" +
                            "</td>";//unselectable for 
                    }

                    TR += TD;
                }


                if (model.isFromGantt) {
                    TR += "<td class='e-extendcolumn'></td></tr>";
                }
            }

                var templates = {};
                templates[proxy._id + "_Template"] = TR;
                $.templates(templates);

            

        },
        
        //#endregion TEMPLATE

        //#region TEMPLATE METHODS

        //CONVERT RECORD OBJECT TO STRING
        _convertItemToString: function (data) {

            var str = '',
                labelObject = {},
                columns = this.model.columns;

            for (var i = 0; i < columns.length; i++) {
                labelObject[columns[i].field] = data.item[columns[i].field];
            }

            for (var p in labelObject) {
                if (labelObject.hasOwnProperty(p)) {
                    str += p + '' + labelObject[p] + '\n';
                }
            }

            return str;
        },

        _getCellValue: function (columnName) {

            var cellValue = this.data[columnName];

            if (cellValue || cellValue===0) {
                return cellValue;
            } else {
                return this.data.item && this.data.item[columnName];
            }


        },

        //GET RESOURCE NAME FOR RESOURCE COLUMN IN THE GANTT CONTROL
        _getResourceName: function (gridObject, mappingName, columnName) {

            var proxy = this,
                data = proxy.data[columnName],
                count = 0,
                length = data && data.length,
                resourceName = [];

            if (data && data.length > 0) {
                for (count; count < length; count++) {
                    resourceName.push(data[count][mappingName]);
                }
            }
            return resourceName;
        },


        //GET PREDECESSOR VALUE FOR PREDECESSOR COLUMN IN THE GANTT CONTROL
        _getPredecessorsValue: function (gridObject, mappingName) {

            return this.data.item[mappingName];

        },


        //GET THE EXPAND COLLAPSE STATUS OF TABLE ROW
        _getExpandStatusRecord: function (data) {

            var proxy = this;

            if (proxy.getExpandStatus(data)) {

                return 'table-row';

            }

            return 'none';
        },

        //#endregion TEMPLATE METHODS

        //#region CONVERT HIERARCHICAL RECORDS TO FLAT RECORDS
        //CREATE FLATRECORDS FROM HIERARCHICAL DATASOURCE
        _createRecords: function (dataSource) {

            var proxy = this,
                model = proxy.model,
                flatRecords = model.flatRecords,
                length = dataSource.length,
                count = 0,
                parentRecord,
                parentRecords = model.parentRecords,
                rows = model.records;                

            for (count; count < length; count++) {

                parentRecord = proxy._createRecord(dataSource[count], 0, null);
                proxy._storedIndex++;

                //CHECK ALTROW ENABLED OR NOT
                //if (enableAltRow) {     //  DYNAMICALY CHANGE THE ALTROW 
                    parentRecord.isAltRow = proxy._storedIndex % 2 == 0 ? false : true;
                //}

                //CHECK SORTING ENABLED OR NOT
                //if (enableSorting) {//  DYNAMICALY CHANGE THE ALTROW 
                    parentRecords.push(parentRecord);
                //}

                parentRecord.index = proxy._storedIndex;
                rows[proxy._storedIndex] = parentRecord;
                flatRecords.push(parentRecord);
                parentRecord.childRecords && proxy._addNestedRecords(parentRecord.childRecords);

            }
        },


        //CREATE RECORD OBJECT
        _createRecord: function (data, level, parentItem, expanded) {

            var proxy = this,
                record,
                childDataSource = data[proxy.model.childMapping];

            //CLONE THE DATA OBJECT
            record = $.extend({}, data);

            record.parentItem = parentItem;
            record.item = data;
            record.childRecords = childDataSource && proxy._createChildRecords(childDataSource, level + 1, record);
            record.hasChildRecords = childDataSource ? true : false;
            record.expanded = expanded !== undefined ? expanded : childDataSource ? childDataSource.length > 0 : false;
            record.isSelected = false;
            record.level = level;

            return record;
        },


        //CREATE CHILD GANTT RECORDS
        _createChildRecords: function (childDataSource, level, parentItem) {

            var proxy = this,
                records = [],
                count = 0,
                length = childDataSource.length,
                record = null,
                childRecord;

            for (count = 0; count < length; count++) {

                record = childDataSource[count];

                if (record) {
                    childRecord = proxy._createRecord(record, level, parentItem);
                    records.push(childRecord);
                }
            }

            return records;

        },


        //ADD NESTED RECORDS TO FLATRECORDS
        _addNestedRecords: function (childDataSource) {

            var proxy = this,
                model = proxy.model,
                flatRecords = model.flatRecords,
                count = 0,
                length = childDataSource.length,
                rows = model.records,
                records = [],
                record;
                

            for (count = 0; count < length; count++) {

                record = childDataSource[count];
                proxy._storedIndex++;

                //CHECK ALTROW ENABLED OR NOT
                //if (enableAltRow) {//DYNAMICALLY CHANGE THE ALT ROW
                    record.isAltRow = proxy._storedIndex % 2 == 0 ? false : true;
                //}

                record.index = proxy._storedIndex;
                rows[proxy._storedIndex] = record;
                flatRecords.push(record);
                records = record.childRecords;

                if (records) {

                    var j = 0,
                        recordlength = records.length,
                        childRecord = null;

                    for (j = 0; j < recordlength; j++) {

                        childRecord = records[j];
                        proxy._storedIndex++;
                        childRecord.index=proxy._storedIndex;
                        //CHECK ALTROW ENABLED OR NOT
                        //if (enableAltRow) {  //DYNAMICALLY CHANGE THE ALT ROW
                            childRecord.isAltRow = proxy._storedIndex % 2 == 0 ? false : true;
                        //}

                        rows[proxy._storedIndex] = childRecord;
                        flatRecords.push(childRecord);
                        childRecord.childRecords && proxy._addNestedRecords(childRecord.childRecords);
                    }
                }
            }
        },

        //#endregion CONVERT HIERARCHICAL RECORDS TO FLAT RECORDS

        //#region METHOD FOR EVENTS

        //INITIALIZE THE EVENTS
        _wireEvents: function () {

            var proxy = this,
                model = proxy.model;

            proxy._$gridContent.bind("scroll", $.proxy(proxy._onScroll, proxy));
            proxy._on(proxy.element, "click", proxy._onClick);
            proxy._on(proxy.element, "click", "#" + proxy._id + "e-gridheader", proxy._onHeaderClick);
            proxy._on(proxy.element, "mousedown", "#" + proxy._id + "e-gridheader", proxy._headerMouseDown);

            if (proxy.model.showGridCellTooltip && !proxy.model.showGridExpandCellTooltip) {

                if (ej.browserInfo().name == "msie") {
                    proxy._on(proxy.element, "mouseenter", ".e-rowcell", proxy._mouseHover);
                }
                else {
                    proxy._on(proxy.element, "mousemove", ".e-rowcell", proxy._mouseHover);
                }
            }
            else {
                proxy._on(proxy.element, "mouseenter", ".e-rowcell", proxy._mouseHover);
            }
            proxy._on(proxy.element, "mouseleave", ".e-rowcell", proxy._cellMouseLeave);
            proxy._enableColumnResizeEvents();

            //MOUSE WHEEL EVENT FOR GANTT CONTROL
            if (model.isFromGantt) {
                proxy._on(proxy.element, "mousewheel DOMMouseScroll", proxy._mouseWheel);
            }

            proxy._enableEditingEvents();
        },

        _mouseHover: function (e) {
            e.preventDefault();
            var $target = $(e.target), proxy = this,
                posx = 0,
                posy = 0,
                $ganttGridRows = proxy.getRows(),
                row = $target.closest('tr'),
                div = $target[0].parentNode,
                progressHandler,
                recordIndexr = $ganttGridRows.index(row),
                item = proxy.model.currentViewData[recordIndexr],
                tooltipElement, tooltiptable, tooltipbody;

            

            proxy._cellMouseLeave();

            if (!e) e = window.event;
            if (e.originalEvent.pageX || e.originalEvent.pageY) {
                posx = e.originalEvent.pageX;
                posy = e.originalEvent.pageY;
            }
            else if (e.originalEvent.clientX || e.originalEvent.clientY) {
                posx = e.originalEvent.clientX + document.body.scrollLeft
                    + document.documentElement.scrollLeft;
                posy = e.originalEvent.clientY + document.body.scrollTop
                    + document.documentElement.scrollTop;
            }

            if ((posx + 200) > proxy._windowWidth) {
                posx -= (posx + 200) - proxy._windowWidth;
            }

            if ((posy + 150) > proxy._windowHeight) {
                posy -= (posy + 150) - proxy._windowHeight;
            }

            if (item) {

                tooltipElement = {
                    ttiptaskname: item.taskName
                };


                tooltiptable = ej.buildTag("table.e-tooltiptable", "", { 'padding': '0' }, { 'cellspacing': '1' });
                if (proxy.model.showGridExpandCellTooltip == false) {
                    if ($target[0].firstChild && $target[0].firstChild.textContent) {
                        tooltipbody = ej.buildTag("tbody", $target[0].firstChild.textContent, {}, {});
                    }
                    else if ((div.childNodes.length > 0 && div.childNodes[1] && div.childNodes[1].textContent)) {
                        tooltipbody = ej.buildTag("tbody", div.childNodes[1].textContent, {}, {});

                    }
                }
                else if (proxy.model.showGridExpandCellTooltip == true) {
                    if (($target[0].firstChild && $target[0].firstChild.childNodes.length > 0)
                        && ($target[0].firstChild && $target[0].firstChild.textContent)) {
                        tooltipbody = ej.buildTag("tbody", $target[0].firstChild.textContent, {}, {});

                    }
                }

                tooltiptable.append(tooltipbody);
                if (tooltipbody) {
                    if (!(proxy.model.showGridExpandCellTooltip) && proxy.model.showGridCellTooltip == true && $target[0].firstChild
                        || (proxy.model.showGridExpandCellTooltip && proxy.model.showGridCellTooltip == true)) {

                        proxy._mouseOverTooltip = ej.buildTag("div.e-tooltipgantt#tooltipgantt" + proxy._id + "", tooltiptable,
                            {
                                "top": (posy + 10) + "px", "left": (posx + 15) + "px",
                                'position': 'absolute',
                                'z-index': '5',
                                'padding': '0',
                                'border-radius': '3px'
                            }, {});

                        $(document.body).append(proxy._mouseOverTooltip);
                    }
                }
            }
        },

        _cellMouseLeave: function () {

            var proxy = this;
            $("#tooltipgantt").remove();
            $(proxy._mouseOverTooltip).remove();
        },

        disableTooltip: function () {
            this._cellMouseLeave();
        },

        _createTooltipTemplate: function () {

            var proxy = this,
                td,
                columnHeaderTexts = proxy.model.columnHeaderTexts,
                helpers = {
                    _getTaskName: proxy._tooltipTaskName
                };

            $.views.helpers(helpers);
            var parentTr = "<tr class='e-tooltip_rowcell'>";

            td = "{{if ~_getTaskName()}}" +
                "<td class='e-tooltiptaskname' style='height:auto;width:auto;font-weight:normal;'>{{:ttiptaskname}}</td>{{/if}}";

            parentTr += td;
            parentTr += "</tr>";

            var templates = {};
            templates[proxy._id + "tooltipTemplate"] = parentTr;
            $.templates(templates);
        },

        _tooltipTaskName: function () {
            return this.data.ttiptaskname;
        },

        //SCROLL EVENT FOR VIRTUALIZATION
        _onScroll: function (args) {

            var proxy = this,
                hScrollDist;

            if (proxy.model.showGridCellTooltip) {
                proxy._cellMouseLeave();
            }

            if (args.scrollLeft != undefined) {
                proxy._scrollLeft = args.scrollLeft;
                hScrollDist = Math.abs(proxy._scrollLeft - proxy._prevScrollLeft);
            }

            

            if (hScrollDist) {

                //CHANGE SCROLL LEFT VALUE OF HEADER  BASED ON THE SCROLLING OF CONTENT DIV
                proxy._$gridHeaderContainer.scrollLeft(proxy._scrollLeft);
                proxy._prevScrollLeft = proxy._scrollLeft;
            }
            
            if (args.scrollTop!=undefined) {

                

                //CHECK VIRTUALIZATION IS ENABLED OR NOT
                if (proxy.model.enableVirtualization) {

                    proxy._scrollTop = args.scrollTop; //proxy._$gridContent.scrollTop();
                    proxy._vScrollDir = proxy._prevScrollTop <= proxy._scrollTop ? 1 : -1;
                    proxy._vScrollDist = Math.abs(proxy._scrollTop - proxy._prevScrollTop);
                    //console.log("VScrollDist:" + proxy._vScrollDist);
               
                    if (proxy._vScrollDist) {
                        proxy._updateCurrentViewData();
                        proxy._prevScrollTop = proxy._scrollTop;
                    }
                }
            }
        },


        //CLICK HANDLER FOR PERFORM ROWSELECTION AND EXPANDCOLLAPSE THE RECORD
        _onClick: function (e) {

            var proxy = this,
                model = proxy.model,
                $target = $(e.target);
            proxy.element[0].focus();
            if ($target.hasClass('e-treegridexpand') || $target.hasClass("e-treegridcollapse")) {

                var $tr = $target.closest('tr'),
                    $gridRows = proxy.getRows(),
                    rowIndex = $gridRows.index($tr),
                    record = model.currentViewData && model.currentViewData[rowIndex],
                    recordIndex = proxy._updatedRecords.indexOf(record),
                    args = {},
                    isExpandCollapseEnabeled;

                args.data = record;
                args.recordIndex = recordIndex;
                args.expanded = $tr.hasClass("e-treegridrowexpand") ? false : true;

                if (args.expanded) {

                    isExpandCollapseEnabeled = proxy._trigger("expanding", args);

                } else {

                    isExpandCollapseEnabeled = proxy._trigger("collapsing", args);
                    
                }

                if (!isExpandCollapseEnabeled&&!model.isFromGantt) {
                    ej.TreeGrid.sendExpandCollapseRequest(proxy,args);
                  
                }
                
            } else if ($target.hasClass("e-rowcell") || $target.parent().hasClass("e-rowcell")
                || ($target.closest("td").hasClass("e-rowcell")) && $target.hasClass("e-cell")) {

                if (!model.allowSelection) return;
                
                if (model.selectionType == ej.TreeGrid.SelectionType.Multiple) {

                    if (e.ctrlKey) proxy._multiSelectCtrlRequest = true;
                    
                    if (e.shiftKey) {

                        proxy._multiSelectShiftRequest = true;
                        proxy.selectRows(proxy._previousIndex, proxy.getRows().index($target.closest('tr')));
                        
                    }
                }
                
                if (!proxy._multiSelectShiftRequest) {

                    recordIndex = proxy.getRows().index($target.closest('tr'));

                    if (recordIndex != -1) {
                        
                        record = model.currentViewData[recordIndex];                      
                        var args = { rowElement: $tr, recordIndex: this.selectedRowIndex() };

                        if (!proxy._trigger('rowSelecting', args)) {

                            model.selectedItem = record;
                            this.selectedRowIndex(proxy._updatedRecords.indexOf(record));
                            proxy.selectRows(this.selectedRowIndex());
                            args = {
                                rowElement: $tr,
                                data: proxy.model.selectedItem,
                                target: "ejTreeGrid",
                                recordIndex: this.selectedRowIndex(),
                            };
                            proxy._trigger("rowSelected", args);                            
                        }
                    }
                }
                
                proxy._multiSelectShiftRequest = false;
            }
            
        },


        //CLICK HANDLER FOR PERFORM THE SORTING ACTION
        _onHeaderClick: function (e) {

            var proxy = this,
                model = proxy.model;
            proxy._cellMouseLeave();

            if ($(e.target).is(".e-ascending, .e-descending")) e.target = e.target.parentNode;

            var $target = $(e.target),
                columnName,
                columnSortDirection;

            proxy.getHeaderTable().find(".e-columnheader").find(".e-headercellactive").removeClass("e-headercellactive");

            if ($target.hasClass("e-headercelldiv")) {

                if (!model.allowSorting) return false;

                columnName = $.trim($target.text());

                if ($target.find('span').hasClass("e-ascending")) {
                    columnSortDirection = ej.sortOrder.Descending;
                } else {
                    columnSortDirection = ej.sortOrder.Ascending;
                }

                if (model.allowMultiSorting && e.ctrlKey) {
                    proxy._multiSortRequest = true;
                }

                proxy.sortColumn(columnName, columnSortDirection);
            }

            return false;
        },


        //HEADER MOUSE DOWN EVENT FOR COLUMN RESIZING
        _headerMouseDown: function (e) {

            var proxy = this,
                model = proxy.model;
            proxy._cellMouseLeave();

            if (model.allowColumnResize) proxy._resizer._mouseDown(e);

            if (($(e.target).hasClass("headercelldiv") && $(e.target).hasClass("headercell"))) {

                var $headercell = $(e.target).hasClass("headercelldiv") ? $(e.target).parent() : $(e.target);
                model.headerEffects && $headercell.addClass("headercellactive");

            }
        },


        //EVENT FOR COLUMN RESIZE
        _enableColumnResizeEvents: function () {

            var proxy = this;

            if (proxy.model.allowColumnResize) {
                proxy._on(proxy.element, "mousemove", proxy._mouseMove);
                proxy._on(proxy.element, "mouseup", proxy._mouseUp);
            } else {
                proxy._off(proxy.element, "mousemove", proxy._mouseMove);
                proxy._off(proxy.element, "mouseup", proxy._mouseUp);
            }
        },


        _mouseMove: function (e) {

            var proxy = this;

            if (proxy.model.allowColumnResize) {
                proxy._resizer._mouseMove(e);
            }
        },


        _mouseUp: function (e) {

            var proxy = this;

            if (proxy.model.allowColumnResize) {

                if (proxy.model.isEdit && $(e.target).hasClass('e-headercelldiv')) {
                    proxy.saveCell(); 
                }                
                proxy._resizer._mouseUp(e);
                //proxy.getHeaderTable().css({ 'width': '100%' });
                //proxy.setWidthToColumns();
            }
        },




        //ENABLE THE EDITING EVENTS
        _enableEditingEvents: function () {

            var proxy = this,
                model = proxy.model;

            if (model.editSettings.allowEditing) {

                if (model.editSettings.editMode == "cellEditing") {
                    proxy._on(proxy.element, "mousedown", ".e-gridcontent", proxy._saveCellHandler);
                }

                proxy._on(this.element, "dblclick", ".e-gridcontent", proxy._editdblClickHandler);
            } else {

                if (model.editSettings.editMode == "cellEditing") {
                    proxy._off(proxy.element, "mousedown", ".e-gridcontent", proxy._saveCellHandler);
                }

                proxy._off(this.element, "dblclick", ".e-gridcontent", proxy._editdblClickHandler);
            }
        },


        //ENABLE THE MOUSEDOWNEVENT HANDLER FOR SAVE CELL
        _saveCellHandler: function (e) {

            var $target = $(e.target),
                proxy = this;
           e.stopPropagation();

            if ($target.closest(".e-popup").length == 0 &&
                $target.closest(".e-rowcell").find("#" + proxy._id + "EditForm").length == 0) {
                proxy.saveCell();
            }

        },

        //DOUBLE CLICK EVENT HANDLER FOR DIALOG EDITING
        _editdblClickHandler: function (e) {

            var proxy = this,
                model = proxy.model,
                $form = $("#" + proxy._id + "EditForm"),
                args = {};

            if (model.editSettings.editMode === "normal") {
                args.requestType = ej.TreeGrid.Actions.BeginEdit;
                proxy._trigger("actionBegin", args);
            }

            if (proxy._cellEditingDetails.cancelSave) {

                proxy._cellEditingDetails.cancelSave = false;
                return;

            }

            if ($form.length === 0) {

                model.editSettings.editMode == "cellEditing" && proxy.element.focus();
                proxy._cellEditingDetails.columnIndex = proxy.getCellIndex(e);
                proxy._cellEditingDetails.rowIndex = proxy.getRowIndex(e);
                fieldName = proxy._cellEditingDetails.columnIndex >= 0 && model.columns[proxy._cellEditingDetails.columnIndex].field;
                if (fieldName) {
                    model.editSettings.allowEditing &&
                        model.editSettings.editMode == ej.TreeGrid.EditMode.CellEditing &&
                        proxy.cellEdit(proxy._cellEditingDetails.rowIndex, fieldName);
                }
            }
        },


        //PERFORM SCROLLING IN TREEGRID
        _mouseWheel: function (e) {

            var proxy = this,
                delta = null,
                ori = e;

            e = e.originalEvent;

            try {
                e.preventDefault ? e.preventDefault() : ori.preventDefault();
            }
            catch (err) {
            }
			
			 if (proxy._$gridContent.height() > proxy._$gridContainer.height()) { 
                proxy._$gridContent.ejScroller("scrollY", 0, true);
                return;
            }

            if (e.wheelDelta) {
                delta = -e.wheelDelta / 120;
                if (window.opera) {
                    if (parseFloat(window.opera.version, 10) < 10)
                        delta = -delta;
                }
            } else if (e.detail) {
                delta = e.detail / 3;
            }
	
            if (!delta) return false;

            var scrollTop = proxy._$gridContent.ejScroller("option", "scrollTop"),

                change = scrollTop + (delta * 57);  

            var h = parseInt(proxy._updatedRecords.length * 30) - (proxy._viewPortHeight-18);

            //console.log("TreeGrid ScrollTop:" + scrollTop + "Height :" + h);
            
            if (change> h) {
                change = h;                
            }

            if (change < 0) {
                change = 0;
            }
            
                proxy._updateScrollTop(change);
           
            return true;

        },
        _getCollapsedParentItem:function(record)
        {
            var parentRecord = record.parentItem;
            if (parentRecord) {
                if (parentRecord.expanded === false) {
                    return parentRecord;
                } else {
                    return this._getCollapsedParentItem(parentRecord)
                }
            }
            else {
                return null;
            }
        },
        //KEY PRESSED EVENT FOR KEY BOARD INTERACTION IN TREEGRID CONTROL
        _keyPressed: function (action, target, e) {

                var proxy = this,
                $treegridEle = proxy.element,
                gridObject = $treegridEle.ejTreeGrid("instance"),
                model = gridObject.model;
            //CHECK THE ALLOWKEYBOARDNAVIGATION
            if (!model.allowKeyboardNavigation){
                return false;
            }

            var returnValue = false,
                $target = $(target);
            $form = $("#" + proxy._id + "EditForm");
            switch (action) {

                //save the current edited cellValue on Enter key press
                case "saveRequest":
                   
                    if (model.editSettings.editMode == "cellEditing" && $form.length>0) {
                        gridObject._moveCurrentCell("down");
                    }

                    break;

                    //cancel the cell editing in the current cell Esc key press
                case "cancelRequest":
                    if (model.editSettings.editMode == "cellEditing") {
                        gridObject.cancelEditCell();
                    }

                    break;

                    //edit the cell in treegrid by F2 key press
                case "editRecord":

                    if (model.editSettings.allowEditing && model.editSettings.editMode == "cellEditing") {

                        gridObject._cellEditingDetails.rowIndex = this.selectedRowIndex();
                        gridObject._cellEditingDetails.columnIndex = proxy.getCellIndex(e, target);

                        if (gridObject._cellEditingDetails.columnIndex !== -1) {
                            fieldName = gridObject.model.columns[gridObject._cellEditingDetails.columnIndex].field;
                            fieldName && gridObject.cellEdit(gridObject._cellEditingDetails.rowIndex, fieldName);
                        }

                    }

                    break;

                    //select the first row while press the Home key
                case "firstRowSelection":
                    if (!proxy._rowSelectingEventTrigger()) {
                        proxy.selectRows(0);
                        recordIndex = proxy._updatedRecords.indexOf(model.selectedItem);
                        rowTop = recordIndex !== -1 && recordIndex * model.rowHeight;// - (proxy.getCollapsedRecordCount() * model.rowHeight);
                        var currentHeight = proxy._$gridContainer.height();
                        if (currentHeight > proxy._viewPortHeight) {
                            args = {
                                requestType: "scroll",
                                delta: rowTop
                            }
                            proxy._scrollTop = rowTop;
                            proxy._completeAction(args);
                            proxy._$gridContent.children(".e-content").scrollTop(rowTop);
                        }
                        if (model.enableVirtualization) {
                            proxy.selectRows(0);
                            proxy.element[0].focus();
                        }
                        proxy._rowSelectedEventTrigger(0);
                    }
                    break;

                case "lastRowSelection":
                    if (!proxy._rowSelectedEventTrigger()) {
                        lastRowIndex = proxy._updatedRecords.length - 1;
                        var previousItem = proxy._updatedRecords[lastRowIndex],
                               parentItem,
                               rowClassName, length;
                        if (!model.enableVirtualization && previousItem.parentItem) {
                            parentItem = proxy._getCollapsedParentItem(previousItem);
                            if (parentItem) {
                                length = proxy.getChildCount(parentItem, 0);
                                proxy.selectRows(lastRowIndex - length);
                            }
                            else {
                                proxy.selectRows(lastRowIndex);
                            }
                        }
                        else {
                            proxy.selectRows(lastRowIndex);
                        }

                        recordIndex = model.selectedItem && proxy._updatedRecords.indexOf(model.selectedItem);
                        rowTop = lastRowIndex !== -1 && recordIndex * model.rowHeight;

                        if (rowTop + proxy.model.rowHeight >= proxy._viewPortHeight) {
                            scrollTop = rowTop + proxy.model.rowHeight - (proxy._viewPortHeight - proxy._scrollBarHeight) - (proxy.getCollapsedRecordCount() * model.rowHeight);
                            if (scrollTop >= 0) {
                                args = {
                                    requestType: "scroll",
                                    delta: scrollTop
                                };
                            proxy._scrollTop = scrollTop;
                            proxy._completeAction(args); //actionComplete event trigger for perform the scrolling gantt chartcontrol.
                        }
                            if (model.enableVirtualization) {
                                proxy.selectRows(recordIndex);
                                proxy.element[0].focus();
                               
                            }
                        }
                        proxy._rowSelectedEventTrigger(recordIndex);
                    }
                    break;
                    //set the edit mode to previous cell by press shift +tab key
                case "moveCellLeft":

                    if (model.editSettings.allowEditing && model.editSettings.editMode == "cellEditing" && $form.length>0) {
                        returnValue = proxy._moveCurrentCell("left");
                    }

                    break;

                    //set the edit mode to next cell by tab key
                case "moveCellRight":
                    if (model.editSettings.allowEditing && model.editSettings.editMode == "cellEditing" && $target && $form.length>0) {
                        returnValue = proxy._moveCurrentCell("right");
                    }

                    break;

                    //select the next row by press down arrow key
                case "downArrow":
                    if (!proxy._rowSelectingEventTrigger()) {
                        // debugger;
                        //get the last row index of the ganttchart control
                        var lastRowIndex = proxy._updatedRecords.length - 1;

                        if (proxy.selectedRowIndex() != lastRowIndex) {

                            if (!model.enableVirtualization && !model.selectedItem.expanded) {
                                var child = proxy.getChildCount(model.selectedItem, 0);
                                var index = proxy.selectedRowIndex() + child + 1;
                                if (lastRowIndex >= index)
                                    proxy.selectRows(proxy.selectedRowIndex() + child + 1);

                            }
                            else {
                                proxy.selectRows(proxy.selectedRowIndex() + 1);
                            }
                            model.selectedItem.isSelected = true;
                            var recordIndex = model.selectedItem && proxy._updatedRecords.indexOf(model.selectedItem),
                                rowTop = lastRowIndex !== -1 && recordIndex * model.rowHeight,
                                scrollTop,
                                containerWidth,
                                scrollerExist = proxy._$gridContent.children(".e-content").length;
                            if (scrollerExist)
                                scrollTop = proxy._$gridContent.children(".e-content").scrollTop();
                            else
                                scrollTop = proxy._$gridContent.scrollTop();

                            containerWidth = scrollTop + proxy._viewPortHeight - proxy._scrollBarHeight - model.rowHeight + (proxy.getCollapsedRecordCount() * model.rowHeight);

                            if (containerWidth < rowTop) {
                                args = {
                                    requestType: "scroll",
                                    delta: (scrollTop+model.rowHeight),
                                };
                                proxy._completeAction(args);
                                if (scrollerExist)
                                    proxy._$gridContent.children(".e-content").scrollTop((scrollTop + model.rowHeight));
                                else
                                    proxy._$gridContent.scrollTop((scrollTop + model.rowHeight));

                                if (model.enableVirtualization) {
                                    proxy.element[0].focus();
                                }
                            }
                            proxy._rowSelectedEventTrigger(proxy.selectedRowIndex()); //for diasable completeaction
                        }
                    }
                    break;
                // expand the child task by press right arrow
                case "rightArrow":
                    var rowIndex = proxy.selectedRowIndex(),
                    record = model.currentViewData && model.currentViewData[rowIndex],
                    args = {},
                    isExpandCollapseEnabeled;
                    args.data = record;
                    args.recordIndex = rowIndex;
                    args.expanded = true;
                    if (!proxy._rowSelectingEventTrigger() && record.hasChildRecords) {
                        if (proxy.selectedRowIndex() >= 0) {
                            isExpandCollapseEnabeled = proxy._trigger("expanding", args);
                            if (!isExpandCollapseEnabeled && !model.isFromGantt) {
                                ej.TreeGrid.sendExpandCollapseRequest(proxy, args);
                            }
                        }
                    }
                    break;
                //collapse the child task by press left arrow 
                case "leftArrow":
                    var rowIndex = proxy.selectedRowIndex(),
                    record = model.currentViewData && model.currentViewData[rowIndex],
                    args = {},
                    isExpandCollapseEnabeled;
                    args.data = record;
                    args.recordIndex = rowIndex;
                    args.expanded = false;
                    if (!proxy._rowSelectingEventTrigger()&&record.hasChildRecords) {
                        if (proxy.selectedRowIndex()>=0) {
                                    isExpandCollapseEnabeled = proxy._trigger("collapsing", args); 
                            if (!isExpandCollapseEnabeled&&!model.isFromGantt) {
                                ej.TreeGrid.sendExpandCollapseRequest(proxy,args);
                            }
                         }
                     }
                    break;
                    //select the previous row by press down arrow key
                case "upArrow":
                    if (!proxy._rowSelectingEventTrigger()) {
                        if (proxy.selectedRowIndex() !== 0) {
                            var previousItem = proxy._updatedRecords[proxy.selectedRowIndex() - 1],
                                parentItem,
                                rowClassName, length;
                            if (!model.enableVirtualization && previousItem.parentItem) {
                                parentItem = proxy._getCollapsedParentItem(previousItem);
                                if (parentItem) {
                                    length = proxy.getChildCount(parentItem, 0);
                                    proxy.selectRows(proxy.selectedRowIndex() - length - 1);
                                }
                                else {
                                    proxy.selectRows(proxy.selectedRowIndex() - 1);
                                }
                            }
                            else {
                                proxy.selectRows(proxy.selectedRowIndex() - 1);
                            }

                            model.selectedItem.isSelected = true;
                            recordIndex = proxy._updatedRecords.indexOf(model.selectedItem);
                            rowTop = recordIndex !== -1 && (recordIndex * model.rowHeight) - (proxy.getCollapsedRecordCount() * model.rowHeight);
                            var scrollerExist = proxy._$gridContent.children(".e-content").length, scrollTop;
                            if (scrollerExist)
                                scrollTop = proxy._$gridContent.children(".e-content").scrollTop();
                            else
                                scrollTop = proxy._$gridContent.scrollTop();
                            if (scrollTop) {
                                if (scrollTop > rowTop) {
                                    if (rowTop >= 0) {
                                        args = {
                                            requestType: "scroll",
                                            delta: rowTop
                                        };
                                    }
                                    else
                                    {
                                        args = {
                                            requestType: "scroll",
                                            delta: 0
                                        };
                                    }
                                    if (scrollerExist)
                                        proxy._$gridContent.children(".e-content").scrollTop(args.delta);
                                    else
                                        proxy._$gridContent.scrollTop(args.delta);

                                    proxy._completeAction(args);
                                    if (model.enableVirtualization) {
                                        proxy.selectRows(proxy.selectedRowIndex());
                                        proxy.element[0].focus();
                                    }
                                }
                            }
                            proxy._rowSelectedEventTrigger(proxy.model.selectedRowIndex);
                        }
                    }
                    break;

                default:
                    returnValue = true;
            }

            return returnValue;
        },

        //ROWSELECTING EVENT METHOD
        _rowSelectingEventTrigger: function () {

            var proxy = this,
                model = proxy.model,
                args = {},
                selectedItem,
                $gridRows;

            if (model.currentViewData.length <= 0) {
                this.selectedRowIndex(-1);
            }
            if (this.selectedRowIndex() !== -1 && model.currentViewData.length > 0) {

                $gridRows = proxy.getRows();
                selectedItem = model.currentViewData[this.selectedRowIndex()];
                model.selectedItem = selectedItem;
                args = {
                    rowElement: $gridRows.eq(this.selectedRowIndex()),
                    data: selectedItem,
                    recordIndex: this.selectedRowIndex(),
                };

            }
            return proxy._trigger('rowSelecting', args);
        },


        //ROWSELECTED EVENT METHOD
        _rowSelectedEventTrigger: function (index) {

            var proxy = this,model=proxy.model,
                args = {},
                $gridRows = proxy.getRows();

            if (model.currentViewData.length > 0) {
                args = {
                    rowElement: $gridRows.eq(index),
                    data: proxy.model.selectedItem,
                    target: "ejTreeGrid",
                    recordIndex: index
                };
                proxy._trigger("rowSelected", args);
            }

        },


        //QUERYCELLINFO,ROWDATABOUND EVENT BINDING FOR TREEGRID
        _eventBindings: function () {

            var proxy = this,
                model = proxy.model,
                count = 0,
                gridRows = proxy._gridRows,
                length = gridRows.length;

            
            if (model.queryCellInfo != null || model.rowDataBound != null) {

                for (count; count < length; count++) {

                    proxy._rowEventTrigger(gridRows[count], model.flatRecords[count]);

                }
            }
        },

        //ROWDATABOUND EVENT TRIGGER
        _rowEventTrigger: function (row, data) {

            var proxy = this;

            proxy._trigger("rowDataBound", {
                rowElement: row,
                data: data
            });

            var tdCells = row.cells,
                $tdRowcells = $(row).find(".e-rowcell"),
                cellIndex = 0,
                length = tdCells.length,
                column,
                columns = proxy.model.columns;

            for (cellIndex; cellIndex < length; cellIndex++) {

                if ($(tdCells[cellIndex]).hasClass("e-rowcell")) {

                    column = columns[$tdRowcells.index(tdCells[cellIndex])];

                }

                if (column) {

                    proxy._cellEventTrigger(tdCells[cellIndex], data, column);

                }
            }
        },


        //QUERYCELLINFO EVENT TRIGGER
        _cellEventTrigger: function (cell, data, column) {

            var args = {
                cellElement: cell,
                data: data,
                column: column,

            };

            args.cellValue = args.column && data[args.column.field];

            this._trigger("queryCellInfo", args);
        },


        //ACTION COMPLETE EVENT TRIGGER
        _completeAction: function (args) {

            var proxy = this,
                model = proxy.model,
                sortcolumns = model.sortSettings.sortedColumns;

            model.isEdit = false;

            if ((ej.TreeGrid.Actions.Sorting == args.requestType && model.allowSorting)
                || (ej.TreeGrid.Actions.Refresh == args.requestType)) {

                if (model.allowSorting) {

                    if (!proxy._multiSortRequest) {
                        proxy.getHeaderTable().find(".e-columnheader").
                            find(".e-headercelldiv").find(".e-ascending,.e-descending").remove();

                        proxy.getHeaderTable().find("[aria-sort]").removeAttr("aria-sort");
                    }

                    for (var i = 0; i < sortcolumns.length; i++) {
                        proxy._addSortElementToColumn(sortcolumns[i].field, sortcolumns[i].direction);
                    }

                }

                proxy._multiSortRequest = false;
            } else if (ej.TreeGrid.Actions.BeginEdit == args.requestType || ej.TreeGrid.Actions.Add == args.requestType) {

                var $form = $("#" + proxy._id + "EditForm");

                proxy.model.isEdit = true;
                proxy.setWidthToColumns();

                if (ej.TreeGrid.Actions.Add == args.requestType) {

                    $form.find(".e-field:disabled").not(".e-identity").removeAttr("disabled").removeClass("e-disable");

                }

            }
            else if (args.requestType === "scroll")
            {
                proxy._updateCurrentViewData();

                if (!proxy.model.isFromGantt && args.delta>=0 && proxy.isVScroll() && proxy.model.enableVirtualization)
                {
                    if (!proxy._$gridContent.ejScroller("isHScroll") && args.delta>=18) {
                        args.delta -= proxy._scrollBarHeight;
                        proxy._scrollTop = args.delta;
                    }
                    proxy._$gridContent.ejScroller("scrollY", args.delta, true);
                }
            }
            proxy._trigger("actionComplete", args);
        },


        //#endregion

        //#region METHOD FOR DATA PROCESSING

        //RENDERING THE DATA
        sendDataRenderingRequest: function (args) {

            var proxy = this,
                temp;

            if (proxy.model.currentViewData.length) {

                switch (args.requestType) {

                    case ej.TreeGrid.Actions.Delete:
                    case ej.TreeGrid.Actions.Refresh:
                    case ej.TreeGrid.Actions.Save:
                    case ej.TreeGrid.Actions.Sorting:
                    case ej.TreeGrid.Actions.Searching:
                    case ej.TreeGrid.Actions.ExpandCollapse:
                    case ej.TreeGrid.Actions.RefreshDataSource:
                        temp = document.createElement('div');
                        proxy.getContentTable().find("colgroup").first().replaceWith(proxy._getMetaColGroup());
                        var $tbody = proxy.getContentTable().children('tbody');
                        $tbody.empty();                        
                        temp.innerHTML = ['<table>', $.render[proxy._id + "_Template"](proxy.model.currentViewData),
                            '</table>'].join("");
                        proxy.getContentTable().get(0).replaceChild(temp.firstChild.firstChild,
                            proxy.getContentTable().get(0).lastChild);
                        proxy._gridRows = proxy.getContentTable().get(0).rows;
                        proxy.setGridRows($(proxy.getContentTable().get(0).rows));
                        proxy._eventBindings();
                        break;
                }

            } else {

                proxy.getContentTable().find('tbody').empty().append(proxy._getEmptyTbody());
                proxy._gridRows = null;
                proxy.setGridRows(null);

            }

            proxy._completeAction(args);            

        },


        //SORTING THE DATA BASED ON THE PARTICULAR FIELDS
        /**
      * To sorting the data based on the particular fields
      * @return jQuery
      * @example 
      * &lt;treegrid id="treegrid"&gt;treegrid&lt;/treegrid&gt; <br> 
      * &lt;script&gt;
      * // Create Tree Grid
      * var treegridObj = $("#treegrid").data("ejTreeGrid");
      * treegridObj.ejTreeGrid("sortColumn"); // To sort the data
      * &lt;/script&gt;
      * @example 
      *  &lt;treegrid id="treegrid"&gt;treegrid&lt;/treegrid&gt; <br> 
      * &lt;script&gt;
      * // refresh the content
      * $("#treegrid").ejTreeGrid("sortColumn");	
      * &lt;/script&gt;
      * @memberof ejTreeGrid
      * @instance
       */
        sortColumn: function (columnName, columnSortDirection) {

            var proxy = this,
                model = proxy.model,
                sortColumns = model.sortSettings.sortedColumns,
                args = {},
                count = 0,
                length = sortColumns.length;

            if (!model.allowSorting || $.inArray(columnName, proxy._disabledSortableColumns) != -1
                || columnName.length == 0) {
                return;
            }

            if (!proxy._multiSortRequest) {
                model.sortSettings.sortedColumns = [];
            }

            args.requestType = ej.TreeGrid.Actions.Sorting;
            args.columnName = columnName;

            proxy._cSortedColumn = proxy.getFieldNameByHeaderText(columnName);
            proxy._cSortedDirection = args.columnSortDirection = columnSortDirection === undefined ?
                ej.sortOrder.Ascending : columnSortDirection;

            for (count; count < length; count++) {
                
                if (sortColumns[count].field == proxy._cSortedColumn) {
                    sortColumns.splice(count, 1);
                    break;
                }

            }

            model.sortSettings.sortedColumns.push({
                field: proxy._cSortedColumn,
                direction: proxy._cSortedDirection
            });

            var returnValue = proxy.processBindings(args);

            if (returnValue) {

                proxy._cSortedDirection = proxy._cSortedColumn = null;

            }
        },


        //DELETE THE PARTICULAR RECORD 
        deleteRow: function ($tr) {

            var proxy = this,
                deletedRowIndex,
                deletedRow,
                args = {},
                eveArgs = {};


            if (ej.isNullOrUndefined($tr)) {
                $tr = proxy.getRowByIndex(proxy.selectedRowIndex());
            }

            deletedRowIndex = proxy.getRows().index($tr);
            deletedRow = proxy.model.currentViewData[deletedRowIndex];

            args.tr = $tr;
            args.data = deletedRow;
            args.requestType = ej.TreeGrid.Actions.Delete;   

            if (proxy._trigger("actionBegin", args)) {
                return true;
            }

            proxy.processBindings(args);
           // proxy._completeAction(args);

        },


        //SEARCH THE TASK FROM TREE GRID CONTROL
        /**
        * To search an item with search string provided at the run time
        * @return jQuery
        * @example 
        * &lt;treegrid id="treegrid"&gt;TreeGrid&lt;/treegrid&gt; <br> 
        * &lt;script&gt;
        * // Create treegrid
        * var treegridObj = $("#treegrid").data("ejTreeGrid");
        * treegridObj.ejTreeGrid("searchItem",$("#text").val()); // To search a task
        * &lt;/script&gt;
        * @example 
        *  &lt;treegrid id="treegrid"&gt;TreeGrid&lt;/treegrid&gt; <br> 
        * &lt;script&gt;
        * // To search a task
        * $("#treegrid").ejTreeGrid("searchItem",$("#text").val());	
        * &lt;/script&gt;
        * @memberof ejTreeGrid
        * @instance
         */
        search: function (searchString) {

            var proxy = this,
                args = {};

            args.requestType = ej.TreeGrid.Actions.Searching;
            args.keyValue = searchString;

            proxy._searchString = searchString;

            proxy.processBindings(args);
        },

        //SEND SAVE REQUEST
        endEdit: function () {

            var proxy = this;

            if (proxy.model.editSettings.editMode == "cellEditing")
                return proxy.saveCell();

            return true;

        },

        //#endregion

        //#region PUBLIC METHODS

        //CALCULATE THE VISIBLE RANGE OF RECORDS TO DISPLAYED IN CURRENT VIEWPORT
        getVisibleRange: function () {

            var proxy = this,
                top = proxy._scrollTop / proxy.model.rowHeight,
                coeff = top - Math.floor(top),
                topIndex, bottomIndex,
                model = proxy.model;

            //console.log("TreeGrid ScrollTop:" + proxy._scrollTop);

            proxy._offset = coeff * model.rowHeight;
            topIndex = Math.floor(top);
            bottomIndex = proxy._getRowPosition(proxy._scrollTop + proxy._viewPortHeight);
            topIndex = Math.max(0, topIndex);
            bottomIndex = Math.min(proxy.getRecordsCount(), bottomIndex);

            proxy._visibleRange = {
                top: topIndex,
                bottom: bottomIndex
            };

            return proxy._visibleRange;

        },


        //SET THE COLUMNWIDTH TO EACH COLUMN IN TREEGRID
        setWidthToColumns: function () {

            var proxy = this,
                model = proxy.model,
                $cols1 = proxy.getContentTable().children("colgroup").find("col"),
                $cols2 = proxy.getHeaderTable().children("colgroup").find("col"),
                width = proxy._gridWidth,
                colCount = model.columns.length;

            if ($cols1.length > colCount) $cols1.splice(0, ($cols1.length - colCount));
            if ($cols2.length > colCount) $cols2.splice(0, ($cols2.length - colCount));

            for (var i = 0; i < $cols2.length; i++) {

                if (!ej.isNullOrUndefined(proxy._columnsWidthCollection[i])) {

                    $cols1.eq(i).width(proxy._columnsWidthCollection[i]);
                    $cols2.eq(i).width(proxy._columnsWidthCollection[i]);

                } else {
                    var colWidth = (width / colCount).toFixed(2) + "px";
                    $cols1.eq(i).css("width", colWidth);
                    $cols2.eq(i).css("width", colWidth);
                    proxy._columnsWidthCollection[i] = (width / colCount).toFixed(2);
                }
            }
        },


        //ENTER EDIT MODE INTO THE TREEGRID CELL
        cellEdit: function (index, fieldName) {

            var proxy = this,
                $form,
                model = proxy.model;

            if (proxy.model.showGridCellTooltip) {
                proxy._cellMouseLeave();
            }

            proxy.model.isEdit && proxy.saveCell();

            $form = $("#" + proxy._id + "EditForm");

            if ($form.length > 0) return true;

            if (!proxy.model.selectedItem) return false;

            if (model.isFromGantt && ((fieldName === "status" ||
                fieldName === "startDate" ||
                fieldName === "endDate" ||
                fieldName === "duration" ||
                fieldName === "predecessor") &&
                proxy.model.selectedItem.hasChildRecords) ||
                (fieldName === "baselineStartDate" ||
                    fieldName === "baselineEndDate") || (fieldName==="taskId")) {

                return false;

            }

            if (proxy.getRows()[index]) {
                var $targetTr = $(proxy.getRows()[index]);
                columnIndex = proxy.getColumnIndexByField(fieldName),
                $targetTd = $targetTr.find(".e-rowcell").eq(columnIndex),
                column = proxy.model.columns[columnIndex],
                data = proxy.model.selectedItem,
                beginEditArgs = {},
                args = {
                    validationRules: ej.isNullOrUndefined(column.validationRules) ? {} : $.extend(true, {},
                        column.validationRules),
                    columnName: column.field,
                    value: !ej.isNullOrUndefined(data[fieldName]) ? data[fieldName] : data.item && data.item[fieldName],
                    data: data,
                    columnObject: column,
                    cell: $targetTd
                },
                isEditable = true;

            if (model.isFromGantt && args.columnName === "predecessor") {

                var predecessor = args.data.item[proxy.model.predecessorMapping];
                args.value = predecessor ? predecessor : "";

            }

            beginEditArgs.data = data;
            beginEditArgs.columnIndex = columnIndex;
            beginEditArgs.rowElement = $targetTr;
            beginEditArgs.cellElement = $targetTd;

          

                proxy._cellEventTrigger($targetTr[0].cells[proxy._cellEditingDetails.columnIndex], data, args.columnObject);

                if (proxy._trigger("beginEdit", beginEditArgs)) return false;

                if ($.inArray(fieldName, proxy._primaryKeys) != -1 ||
                    args.columnObject.allowEditing === false) {

                    $.extend(proxy._cellEditingDetails, {
                        cellValue: args.value,
                        rowIndex: index,
                        fieldName: fieldName,
                        data: args.data,
                        columnIndex: columnIndex
                    });

                    isEditable = false;
                }

                if ($targetTr.hasClass("e-insertedrow") && (!args.columnObject.isTemplateColumn)) isEditable = true;

                if (isEditable) {

                    $.extend(proxy._cellEditingDetails, {
                        rowIndex: index,
                        cellValue: args.value,
                        columnIndex: columnIndex,
                        fieldName: fieldName,
                        cellEditType: args.columnObject.editType,
                        data: data
                    });
                }

                proxy._renderCellEditObject(args, $targetTd);
                $targetTr.addClass("e-editedrow");                
                args.cell&&args.cell.addClass("e-editedcell");

                return true;
            }
        },


        //SAVE THE EDITED CELL
        saveCell: function () {

            var proxy = this,
                model = proxy.model;

            if ($("#" + proxy._id + "EditForm").length > 0) {

                if (!proxy.editFormValidate()) return true;

                if (proxy._cellEditingDetails.columnIndex >= 0) {

                    var $form = $("#" + proxy._id + "EditForm"),
                        $targetTr = $form.closest("tr"),
                        $targetTd = $form.closest("td"),
                        formattedValue, args = {},
                        column = model.columns[proxy._cellEditingDetails.columnIndex],
                        $element = $("#" + proxy._id + proxy._cellEditingDetails.fieldName),
                        cellElement;

                    args = {
                        columnName: column.field,
                        value: proxy.getCurrentEditCellData(),
                        data: proxy._cellEditingDetails.data,
                        previousValue: proxy._cellEditingDetails.cellValue,
                        columnObject: column,
                        cellElement: $targetTd,
                        rowElement: $targetTr,
                    };

                    if (proxy._cellEditingDetails.cellEditType == "datetimepicker" ||
                        proxy._cellEditingDetails.cellEditType == "dropdownedit" ||
                        proxy._cellEditingDetails.cellEditType == "datepicker") {

                        $element.data("ejWidgets") && $element[$element.data("ejWidgets")[0]]("destroy");                        

                    }

                    if (proxy._cellEditingDetails.cellEditType === "datepicker")
                        args.value = proxy.getFormatedDate(args.value);

                    if (!ej.isNullOrUndefined(column.format)) {

                        formattedValue = proxy.formatting(column.format, args.value);
                        args.cellElement.empty().html(formattedValue);

                    } else {

                        if (args.columnObject.type == "boolean") {

                            var cellData = {};
                            cellData[args.columnObject.field] = args.value;
                            args.cellElement.empty().html(
                                $($.templates.Grid_JSONTemplate.render(cellData))[0].cells[proxy._cellEditingDetails.columnIndex].innerHTML
                            );

                        } else args.cellElement.empty().html(args.value);
                    }

                    var isValueModified = ((proxy._cellEditingDetails.cellEditType == "datepicker" ||
                            proxy._cellEditingDetails.cellEditType == "datetimepicker") &&
                        args.value instanceof Date && args.previousValue instanceof Date) ?
                        (args.value.getTime() !== args.previousValue.getTime()) :
                        (args.value !== args.previousValue);

                    model.isEdit = false;

                    var fieldName = proxy._cellEditingDetails["fieldName"];

                    if (model.isFromGantt && fieldName === "predecessor") {

                        var ganttPredecessor = [],
                        prevPredecessors = [],
                        modifiedPredecessors = [],
                        count = 0,
                        length, predecessor;

                        args.previousValue = args.data.predecessor;
                        prevPredecessors = args.previousValue;

                        if (args.value.length > 0) {
                            modifiedPredecessors = args.data._calculatePredecessor(args.value);
                        }

                        length = modifiedPredecessors.length;

                        var index = 0,
                            prevPrdecessorlength = args.previousValue && args.previousValue.length;

                        for (index = 0; index < prevPrdecessorlength; index++) {

                            predecessor = prevPredecessors[index];

                            if (predecessor.from === args.data.taskId.toString()) {
                                ganttPredecessor.push(predecessor);
                            }

                        }

                        for (count = 0; count < length; count++) {

                            ganttPredecessor.push(modifiedPredecessors[count]);

                        }

                        args.data[proxy._cellEditingDetails["fieldName"]] = ganttPredecessor;
                        args.data.item[model.predecessorMapping] = args.value;

                    } else if (isValueModified) {
                        if (model.isFromGantt) {

                            switch (fieldName) {

                                case "taskId":
                                    args.data.item[model.taskIdMapping] = args.value;
                                    break;
                                case "taskName":
                                    args.data.item[model.taskNameMapping] = args.value;
                                    break;
                                case "startDate":
                                    args.data.item[model.startDateMapping] = args.value;
                                    break;
                                case "endDate":
                                    args.data.item[model.endDateMapping] = args.value;
                                    break;
                                case "resourceInfo":
                                    args.data.item[model.resourceInfoMapping] = proxy._getResourceId( args.value);                                    
                                    break;
                                case "duration":
                                    args.data.item[model.durationMapping] = args.value;
                                    break;
                                case "status":
                                    args.data.item[model.progressMapping] = args.value;
                                    break;
                                default:
                                    args.data.item[proxy._cellEditingDetails[fieldName]] = args.value;
                                    break;
                            }

                        } else {
                            args.data.item[proxy._cellEditingDetails.fieldName] = args.value;
                        }
                        
                        args.data[proxy._cellEditingDetails.fieldName] = args.value;
                        
                    }

                    if (model.isFromGantt && (args.columnName === "startDate" ||
                        args.columnName === "endDate" ||
                        args.columnName === "duration" ||
                        args.columnName === "predecessor" ||
                        args.columnName == "status")) {

                        args.isModified = true;

                    }
                    cellElement = $($.render[proxy._id + "_Template"](args.data))[0].cells[proxy._cellEditingDetails.columnIndex];
                    args.cellElement.removeClass("e-editedcell").replaceWith(cellElement);
                    proxy._trigger("endEdit", args);
                    proxy._cellEventTrigger(args.rowElement[0].cells[proxy._cellEditingDetails.columnIndex], args.data, args.columnObject);
                    $targetTr.removeClass("e-editedrow");
                }
            }

            return false;
        },


        //VALIDATE THE MODIFIED CELLVALUE
        editFormValidate: function () {

            if ($.isFunction($.validator))
                return $("#" + this._id + "EditForm").validate({
                    onsubmit: false
                }).form();

            return true;
        },


        //GET THE CURRENT CELL DATA
        getCurrentEditCellData: function () {

            var proxy = this,
                model = proxy.model;

            if ($("#" + proxy._id + "EditForm").length) {

                var $element = $("#" + proxy._id + proxy._cellEditingDetails.fieldName),
                    cellValue;

                switch (proxy._cellEditingDetails.cellEditType) {

                    case ej.TreeGrid.EditingType.String:
                        cellValue = $element.val();
                        break;

                    case ej.TreeGrid.EditingType.Numeric:
                        cellValue = $element.ejNumericTextbox("getValue");
                        $element.ejNumericTextbox("getValue");
                        break;

                    case ej.TreeGrid.EditingType.Dropdown:
                        if (model.isFromGantt) {
                            cellValue = proxy._getSelectedItem($element.ejDropDownList("model.selectedItems"));
                        } else
                            cellValue = $element.ejDropDownList("model.value");
                        break;

                    case ej.TreeGrid.EditingType.Boolean:
                        cellValue = $element.is(':checked');
                        break;

                    case ej.TreeGrid.EditingType.DatePicker:
                        if (model.dateFormat.indexOf("hh") == -1)
                            cellValue = $element.ejDatePicker("model.value");
                        else
                            cellValue = $element.ejDateTimePicker("model.value");

                        break;

                    case ej.TreeGrid.EditingType.DateTimePicker:
                        cellValue = $element.ejDateTimePicker("model.value");
                        break;

                }
                if (typeof cellValue == "string" && cellValue.length &&
                    model.columns[proxy._cellEditingDetails.columnIndex].type == "number") {

                    cellValue = parseFloat(cellValue);

                }

                return cellValue;
            }

            return null;
        },


        _getResourceId:function(dataSource){

            var resourceIds = [],
                count = 0, length = dataSource.length;

            for (count = 0; count < length; count++) {
                resourceIds.push(dataSource[count][this.model.resourceIdMapping]);
            }
            return resourceIds;
        },

        _getSelectedItem: function (indexArray) {

            var proxy = this,
                count = 0,
                length = indexArray.length,
                dropDownData = proxy.model.columns[proxy._cellEditingDetails.columnIndex].dropdownData,
                selectedItems = [];

            if (dropDownData) {

                for (count = 0; count < length; count++) {

                    $.each(dropDownData, function (index, resourceInfo) {
                        if (indexArray[count] === index) {
                            selectedItems.push(resourceInfo);
                        }
                    });

                }
            }

            return selectedItems;
        },

        //GET THE FORMATED DATE 
        getFormatedDate: function (date) {
            return Globalize.format(date, this.model.dateFormat);
        },


        //GET THE FORMATTED TEXT
        formatting: function (formatstring, str) {

            formatstring = formatstring.replace(/%280/g, "\"").replace(/&lt;/g, "<").replace(/&gt;/g, ">");

            var proxy = this,
                s = formatstring,
                frontHtmlidx, frontHtml, rearHtml, lastidxval;

            frontHtmlidx = formatstring.split("{0:");
            lastidxval = formatstring.split("}");
            frontHtml = frontHtmlidx[0];
            rearHtml = lastidxval[1];

            if (formatstring.indexOf("{0:") != -1) {

                var toformat = new RegExp("\\{0(:([^\\}]+))?\\}", "gm"),
                    formatVal = toformat.exec(formatstring);

                if (formatVal != null && str != null) {

                    if (frontHtml != null && rearHtml != null) str = frontHtml + Globalize.format(str, formatVal[2]) + rearHtml;
                    else str = Globalize.format(str, formatVal[2]);

                } else if (str != null) str = str;
                else str = "";

                return str;
            } else if (proxy.data != null && proxy.data.Value == null) {

                $.each(proxy.data, function (dataIndex, dataValue) {
                    s = s.replace(new RegExp('\\{' + dataIndex + '\\}', 'gm'), dataValue);
                });

                return s;

            } else return proxy.data.Value;

        },


        //GET THE COLUMNINDEX BY USIGN THE FIELD NAME
        getColumnIndexByField: function (field) {

            var proxy = this,
                columns = proxy.model.columns,
                length = columns.length,
                column = 0;

            for (column; column < length; column++) {
                if (columns[column]["field"] == field)
                    break;
            }

            return column;
        },

        //GET THE BROWSER DETAILS
        getBrowserDetails: function () {
            var b = navigator.userAgent.match(/(firefox|chrome|opera|msie|safari)\s?\/?(\d+(.\d+)*)/i);
            if (!!navigator.userAgent.match(/Trident\/7\./))
                return { browser: "msie", version: jQuery.uaMatch(navigator.userAgent).version };
            return { browser: b[1].toLowerCase(), version: b[2] };
        },


        setValidation: function () {

            var proxy = this,
                columns = proxy.model.columns,
                length = columns.length,
                count = 0;

            proxy._initValidator();

            for (count; count < length; count++) {

                if (!ej.isNullOrUndefined(columns[count]["validationRules"])) {

                    proxy.setValidationToField(columns[count].field, columns[count].validationRules);

                }
            }
        },

        //SET THE VALIDATION TO THE PARTICULAR FIELD
        setValidationToField: function (name, rules) {

            var proxy = this,
                ele = $("#" + proxy._id + name).length > 0 ? $("#" + proxy._id + name) : $("#" + name);
            ele.rules("add", rules);

            var validator = $("#" + proxy._id + "EditForm").validate({
                onsubmit: false

            });

            if (!ej.isNullOrUndefined(rules["required"])) {

                validator.settings.messages[name] = {};
                validator.settings.messages[name]["required"] = ej.TreeGrid.getColumnByField(proxy.model.columns, name).headerText + " is required";

            }
        },


        //GET THE FIELDNAME OF COLUMN BY USING THE HEADERTEXT
        getFieldNameByHeaderText: function (headerText) {

            var proxy = this;

            if (ej.isNullOrUndefined(proxy._fieldColumnNames[headerText])) return null;

            return proxy._fieldColumnNames[headerText];
        },


        //CLEAR THE SELECTION

        clearSelection: function (index) {

            var proxy = this,
                model = proxy.model;

            if (this.selectedRowIndex() != -1) {

                var $gridRows = proxy.getRows(),
                    index;

                if (!ej.isNullOrUndefined(index)) {
                    for (i = 0; i < this._selectedRowsIndexes.length; i++)
                        proxy.getRowByIndex(this._selectedRowsIndexes[i]).removeAttr("aria-selected").find(".e-selectionbackground").removeClass("e-selectionbackground").removeClass("e-active");                   
                    var record=model.flatRecords[index];
                    if (record){   // For delete all records
                        record.isSelected = false;
                    }
                    else {

                        if ($gridRows) {
                            $gridRows.removeAttr("aria-selected").find(".e-rowcell").removeClass("e-selectionbackground").removeClass("e-active");
                        }
                        var dataManager = new ej.DataManager(model.flatRecords);

                        var query = new ej.Query();
                        query.search(true, ['isSelected'], ej.FilterOperators.Equals, true);

                        var records = dataManager.executeLocal(query);
                        var length = records && records.length;

                        if (length > 0) {

                            var count = 0;

                            for (count = 0; count < length; count++) {
                                records[count].isSelected = false;
                            }

                        }

                    }
                    if (!proxy._selectedRowsIndexes.length) {
                        proxy.selectedRowIndex(-1);
                        proxy.model.selectedItem = null;//For single selection only
                    }
                }

            }
        },


        //UPDATE EXPAND COLLAPSE RECORDS
        updateExpandCollapseRecords: function (record, expand, index) {

            var proxy = this;

            record.expanded = expand;
            proxy._removedCount = 0;
            proxy._expandCollapseRecordCount = proxy.getChildRecordsCount(record);

            if (expand) {

                proxy._tempRecordIndex = proxy.model.records.indexOf(record);

                if (record.hasChildRecords) {
                    proxy._addNestedExpandCollapseRecords(record.childRecords, record.level, record, index);
                }

            } else {

                proxy.model.flatRecords.splice(index + 1, proxy._expandCollapseRecordCount);

            }
        },

        //COUNT THE CHILDRECORDS COUNT
        getChildRecordsCount: function (record) {

            var proxy = this,
                childRecords = record.childRecords,
                count = 0,
                length = childRecords ? childRecords.length : 0;

            for (count; count < length; count++) {

                if (proxy._filteredRecords.length > 0) {

                    var recordIndex = proxy._filteredRecords.indexOf(childRecords[count]);

                    if (recordIndex != -1) {

                        ++proxy._removedCount;

                        if (childRecords[count].hasChildRecords && childRecords[count].expanded) {
                            proxy.getChildRecordsCount(childRecords[count]);
                        }

                    }
                } else {

                    ++proxy._removedCount;

                    if (childRecords[count].hasChildRecords && childRecords[count].expanded) {
                        proxy.getChildRecordsCount(childRecords[count]);
                    }

                }
            }

            return proxy._removedCount;
        },


        //GET THE EXPAND STATUS OF RECORD
        getExpandStatus: function (record) {

            var parentRecord = record.parentItem;

            if (parentRecord != null) {

                if (parentRecord.expanded === false) {
                    return false;
                } else if (parentRecord.parentItem) {

                    if (parentRecord.parentItem.expanded === false) {
                        return false;
                    } else {
                        return this.getExpandStatus(parentRecord.parentItem);
                    }

                } else return true;
            } else return true;
        },


        //RETURN THE COLUMN FIELD NAMES
        getColumnFieldNames: function () {

            var columnNames = [],
                columns = this.model.columns,
                count = 0,
                length = columns.length;

            for (count; count < length; count++) {

                if (columns[count]["field"]) {
                    columnNames.push(columns[count]["field"]);
                }

            }

            return columnNames;
        },


        //RENDERED  RECORDS
        renderRecords: function (args) {

            var proxy = this;

            if (!args) {

                args = {};
                args.requestType = ej.TreeGrid.Actions.Refresh;

            }

            this.sendDataRenderingRequest(args);
        },       


        //SELECT THE ROW IN TREEGRID
        selectRows: function (rowIndex, toIndex) {

            var proxy = this,
                model = proxy.model;

            var $gridRows = proxy.getRows(),
            selectedItem;

            if (model.editSettings.allowEditing || model.editSettings.allowAdding) {
                if ($("#" + proxy._id + "EditForm").length > 0 && proxy.endEdit()) {
                    $gridRows = proxy.getRows();
                    return;
                }
            }

            if (ej.isNullOrUndefined(toIndex) || ej.isNullOrUndefined(rowIndex)) {

                rowIndex = ej.isNullOrUndefined(rowIndex) ? toIndex : rowIndex;        

                    switch (model.selectionType) {

                        case ej.TreeGrid.SelectionType.Multiple:

                            if (proxy._multiSelectCtrlRequest) {
                                var selectedRowIndex = $.inArray(rowIndex, proxy._selectedRowsIndexes);
                                selectedRowIndex != -1 && proxy.clearSelection(rowIndex) && proxy._selectedRowsIndexes.splice(selectedRowIndex, 0);

                                if (selectedRowIndex == -1) {
                                    proxy._selectedRowsIndexes.push(rowIndex);                                   
                                    proxy.getRowByIndex(rowIndex).attr("aria-selected", "true").find('.e-rowcell').addClass("e-selectionbackground e-active").attr("tabindex", "0");                                    
                                }
                                break;
                            }
                            

                        case ej.TreeGrid.SelectionType.Single:
                                proxy.clearSelection(proxy._previousIndex);
                            if (rowIndex <= proxy._updatedRecords.length - 1) {
                                proxy._selectedRowsIndexes = [];
                                proxy._selectedRowsIndexes.push(rowIndex);
                                proxy.getRowByIndex(rowIndex).attr("aria-selected", "true").find('.e-rowcell').addClass("e-selectionbackground e-active").attr("tabindex", 0);
                                if (proxy._updatedRecords[rowIndex]) {
                                    proxy._updatedRecords[rowIndex].isSelected = true;  //Selection issue Changes
                                    model.selectedItem = proxy._updatedRecords[rowIndex];
                                }
                                }
                            break;
                    }
              
            } else {

                if (model.selectionType == ej.TreeGrid.SelectionType.Multiple)
                {
                proxy.clearSelection();
                proxy._selectedRowsIndexes = [];
                var rows = rowIndex - toIndex < 0 ? $gridRows.slice(rowIndex, toIndex + 1) : $gridRows.slice(toIndex, rowIndex + 1),
                    count = 0,
                    length = rows.length;

                for (count; count < length; count++) {
                    proxy._selectedRowsIndexes.push(rows[count].rowIndex);
                    model.flatRecords[rows[count].rowIndex].isSelected = true;
                }
                    $(rows).attr("aria-selected", "true").find('.e-rowcell').addClass("e-selectionbackground e-active").attr("tabindex", "0");                                       
            }
            }
            proxy._previousIndex = proxy._selectedRowsIndexes.length ? rowIndex : undefined;

            if (this.selectedRowIndex() != rowIndex)
            this.selectedRowIndex(rowIndex);

            proxy._multiSelectCtrlRequest = false;
            return false;
        },


        //GET THE CURRENTCELLINDEX
        getCellIndex: function (e, target) {

            var $target = $(e.target);

            if (target) $target = $(target);

            if ($target.prop("tagName") == "TD") {
                return $target.index();
            } else if ($target.prop('tagName') == "DIV" && $target.hasClass('e-cell')) {

                var $td = $target.closest('td');
                return $td.index();

            }

            return -1;
        },


        //GET THE CURRENT ROW INDEX
        getRowIndex: function (e) {

            var proxy = this,
                $target = $(e.target),
                $gridRows = proxy.getRows(),
                index = $gridRows.index($target.parent());

            if (index == -1) {

                var row = $target.closest('tr');
                index = $gridRows.index(row);

            }

            return index;
        },


        //GET THE ROW ELEMENT BY INDEX
        getRowByIndex: function (from, to) {
            try {
                var proxy = this;

                var $gridRows = proxy.getRows(),
                    gridRows=proxy._gridRows,
                    $row = $();

                if (proxy.model.enableVirtualization) {
                    var recordstart = proxy._updatedRecords[from];
                    from = proxy.model.currentViewData.indexOf(recordstart);            
                }

                if (ej.isNullOrUndefined(to)) {
                    if (proxy.model.enableVirtualization) {
                        var recordend = proxy._updatedRecords[to];
                        to = proxy.model.currentViewData.indexOf(recordend);
                    }
                    return $(gridRows[from]);
                } else {                    
                    return $($gridRows.slice(from, to));
                }
            } catch (e) {
                return $();
            }
        },

        //cancel the editMode of the cell
        cancelEditCell: function (e) {

            var proxy = this,
                tr = proxy._gridRows[proxy.selectedRowIndex()],//proxy._cellEditingDetails.rowIndex
                cellData = {},
                cell;

            cellData[proxy._cellEditingDetails.fieldName] = proxy._cellEditingDetails.cellValue,
            data = proxy.model.selectedItem;

            if ($(tr).hasClass("e-insertedrow")) cell = tr.cells[proxy._cellEditingDetails.columnIndex];
            else cell = tr.cells[proxy._cellEditingDetails.columnIndex];

            $(cell).removeClass("e-editedcell").empty().html(
                $($.render[proxy._id + "_Template"](data))[0].cells[proxy._cellEditingDetails.columnIndex].innerHTML
            );

            proxy.element.focus();
            proxy._trigger("cancelEditCell");
        },

        //RERENDERED THE EDITED RECORD
        refreshRow: function (index) {
            var proxy = this,
                data = proxy.model.currentViewData[index];
            if (data) {
                $(proxy._gridRows[index]).replaceWith($($.render[proxy._id + "_Template"](data)));
            }

            proxy.setGridRows($(proxy.getContentTable().get(0).rows));
        },

        //removeRecord: function (args) {
        //    var proxy = this,
        //    deletedRow = args.data, childIndex;
        //    var childRecordsCount = proxy._getFilteredChildRecordsCount(deletedRow);
        //    childRecordsCount += 1;var count=0;
        //    for (var i = 0; i < childRecordsCount; i++) {
        //        proxy.getRows().eq(args.recordIndex).remove();
        //        proxy.setGridRows($(proxy.getContentTable().get(0).rows));
        //    }
        //},

        //RENDERE THE ADDED NEW RECORD
        renderRecord: function (args) {
            var proxy = this,
                addedGridRow = $.render[this._id + "_Template"](args._cAddedRecord),
                model = proxy.model, spliceIndex, spliceIndexRecords;

            if (proxy._gridRows != null && proxy._gridRows.length > 0) {
                var insertIndex = model.currentViewData.indexOf(model.selectedItem);
                if (args.position) {
                    if (args.position == "Above") {
                        proxy.getRows().eq(insertIndex).before(addedGridRow);
                       // spliceIndex = args.recordIndex;
                    }
                    else if (args.position == "Below") {
                        proxy.getRows().eq(insertIndex).after(addedGridRow);
                        //spliceIndex = args.recordIndex + 1;
                    }
                }
                else {
                    if (insertIndex === -1) {
                        args._cAddedRecord.index = 0;
                        proxy.getRows().eq(0).before(addedGridRow);
                        //spliceIndex = 0;
                    } else {
                        proxy.getRows().eq(insertIndex).after(addedGridRow);
                        //spliceIndex = args.recordIndex + 1;
                    }
                }

                if (model.isFromGantt) {
                    proxy.setGridRows($(proxy.getContentTable().get(0).rows));

                    args._cAddedRecord.hasChildRecords = false;
                    args._cAddedRecord.expanded = false;

                    //For update selfreference datasource
                    if (model.parentIdMapping) {
                        if (args._cAddedRecord.parentItem)
                            args._cAddedRecord.item[model.parentIdMapping] = args._cAddedRecord.parentItem.taskId;
                        else
                            delete args._cAddedRecord.item[model.parentIdMapping];

                        this.dataSource().push(args._cAddedRecord.item);
                    }
                    //update child record in parent item
                    if (args._cAddedRecord.parentItem && model.selectedItem) {

                        if (args._cAddedRecord.parentItem.hasChildRecords && (args.position == "Below" || !args.position)) {

                            var tempIndex = model.flatRecords.indexOf(args._cAddedRecord.parentItem);
                            var childIndex = model.flatRecords[tempIndex].childRecords.indexOf(model.selectedItem);
                            if (childIndex === -1)
                            {
                                model.flatRecords[tempIndex].childRecords.splice(0, 0, args._cAddedRecord);
                                if (!model.parentIdMapping)
                                    model.flatRecords[tempIndex].item[model.childMapping].splice(0, 0, args._cAddedRecord.item);
                            }
                            else {
                                model.flatRecords[tempIndex].childRecords.splice(childIndex + 1, 0, args._cAddedRecord);
                                if (!model.parentIdMapping)
                                    model.flatRecords[tempIndex].item[model.childMapping].splice(childIndex + 1, 0, args._cAddedRecord.item);
                            } 
                        }
                        else if (args._cAddedRecord.parentItem.hasChildRecords && args.position == "Above") {

                            var tempIndex = model.flatRecords.indexOf(args._cAddedRecord.parentItem);
                            var childIndex = model.flatRecords[tempIndex].childRecords.indexOf(model.selectedItem);

                            if (childIndex === -1) {
                                model.flatRecords[tempIndex].childRecords.push(args._cAddedRecord);
                                if (!model.parentIdMapping)
                                    model.flatRecords[tempIndex].item[model.childMapping].push(args._cAddedRecord.item);
                            }
                            else {
                                model.flatRecords[tempIndex].childRecords.splice(childIndex, 0, args._cAddedRecord);
                                if (!model.parentIdMapping)
                                    model.flatRecords[tempIndex].item[model.childMapping].splice(childIndex, 0, args._cAddedRecord.item);
                            }
                        }
                    }
                    else if (!args._cAddedRecord.parentItem && model.selectedItem)
                    {
                        model.parentRecords.splice(args._cAddedRecord.index, 0, args._cAddedRecord);
                        var idx = this.dataSource().indexOf(model.selectedItem.item);
                        if(args.position == "Above")
                        {
                            if (!model.parentIdMapping)
                                this.dataSource().splice(idx, 0, args._cAddedRecord.item);
                        }
                        else if (args.position == "Below" || !args.position)
                        {
                            if (!model.parentIdMapping)
                                this.dataSource().splice(idx + 1, 0, args._cAddedRecord.item);
                        }
                    }
                    else if (!model.selectedItem && args._cAddedRecord)
                    {
                        if (!model.parentIdMapping)
                            this.dataSource().splice(0, 0, args._cAddedRecord.item);
                    }

                    //splice index for records and id
                    if (model.selectedItem)
                    {
                        var selectedIndex = model.records.indexOf(model.selectedItem);
                        var flatIndex = model.flatRecords.indexOf(model.selectedItem);
                        if (args.position == "Above") {
                            spliceIndexRecords = selectedIndex;
                            spliceIndex = flatIndex;
                        }
                        else
                        {
                            spliceIndexRecords = selectedIndex + 1;
                            spliceIndex = flatIndex+1;
                        }
                    }
                    else
                    {
                        spliceIndexRecords = 0;
                        spliceIndex = 0;
                    }
                    
                    if (proxy.getExpandStatus(args._cAddedRecord) && model.enableVirtualization){
                        if (spliceIndex === model.flatRecords.length) {
                            model.flatRecords.push(args._cAddedRecord);
                        }
                        else {
                            proxy._recordIndexCount = 0;
                            model.flatRecords.splice(spliceIndex, 0, args._cAddedRecord);
                        }
                    } else if (!model.enableVirtualization) {

                        if (spliceIndex === model.flatRecords.length) {
                            model.flatRecords.push(args._cAddedRecord);
                        }
                        else {
                            proxy._recordIndexCount = 0;
                            model.flatRecords.splice(spliceIndex, 0, args._cAddedRecord);
                        }
                    }
                    if (spliceIndexRecords === model.records.length)
                    {
                        model.records.push(args._cAddedRecord);
                        model.ids.push(args._cAddedRecord.taskId.toString());
                    }
                    else
                    {
                        model.records.splice(spliceIndexRecords, 0, args._cAddedRecord);
                        model.ids.splice(spliceIndexRecords, 0, args._cAddedRecord.taskId.toString());
                    }
                    proxy.setRecordsCount(model.flatRecords.length);
                    proxy.setHeight(model.flatRecords.length * model.rowHeight);
                    proxy._updatedRecords = model.flatRecords;
                    proxy._updateCurrentViewData();
                }
            }

            else if (proxy._gridRows == null || proxy._gridRows.length==0) {
                //model.ids = [];
                if (this.selectedRowIndex() === -1) {

                    args._cAddedRecord.hasChildRecords = false;
                    args.recordIndex = 0;
                }

                model.flatRecords.push(args._cAddedRecord);
                if (this.dataSource() == null) {
                    this.dataSource([]);
                }
                this.dataSource().push(args._cAddedRecord.item);
                model.records.push(args._cAddedRecord);
                model.ids.push(args._cAddedRecord.taskId.toString());
                proxy._updatedGanttRecords = model.flatRecords;

                proxy._updatedRecords = model.flatRecords;
                proxy.setRecordsCount(model.flatRecords.length);
                proxy.setHeight(model.flatRecords.length * model.rowHeight);

                proxy._updateCurrentViewData();

                proxy.getContentTable().find('tbody').empty().append(proxy._getEmptyTbody());
                var temp = document.createElement('div');
                proxy.getContentTable().find("colgroup").first().replaceWith(proxy._getMetaColGroup());
                var $tbody = proxy.getContentTable().children('tbody');
                $tbody.empty();
                temp.innerHTML = ['<table>', $.render[proxy._id + "_Template"](proxy.model.currentViewData),
                    '</table>'].join("");
                proxy.getContentTable().get(0).replaceChild(temp.firstChild.firstChild,
                    proxy.getContentTable().get(0).lastChild);

                proxy._gridRows = proxy.getContentTable().get(0).rows;
                proxy.setGridRows($(proxy.getContentTable().get(0).rows));
                proxy._eventBindings();

            }

        },

        ////Adding new item to datasource
        //_appendToDatasource: function (index, item, datasource) {
        //    //proxy = this,
        //    //model = proxy.model;

        //    //for (var i = 0; i < datasource.length; i++) {
        //    //    if (proxy._recordIndexCount == index) {
        //    //        datasource.splice(i, 0, item);
        //    //        proxy._recordIndexCount = 0;
        //    //        break;
        //    //    }
        //    //    else if (datasource[i][model.childMapping] &&
        //    //        datasource[i][model.childMapping].length > 0) {
        //    //        proxy._recordIndexCount++;
        //    //        proxy._appendToDatasource(index, item, datasource[i][model.childMapping])
        //    //    }
        //    //    proxy._recordIndexCount++;
        //    //}

        //  //  if()


        //},
        
        onScrollHelper: function (scrolltop) {
            var proxy = this;            
            proxy._$gridContent.ejScroller("scrollY", scrolltop, true);
            var isHorizontalScroll = proxy._$gridContent.ejScroller("isHScroll");
            if (!isHorizontalScroll)
                proxy._$gridContent.scrollTop(scrolltop);
        },

        isVScroll: function () {
            if (this._$gridContent.hasClass("e-scroller")) {
                return this._$gridContent.ejScroller("isVScroll");
            }
            else
            {
                return false;
            }
        },

        //REFRESH THE SCROLLER FOR TREEGRIDCONTENT
        refreshScroller:function(panSize){
            
            var proxy = this;                       
            proxy._$gridContent.removeClass("e-borderbox");
            //proxy._$gridContent.css({ 'width': panSize + "px" });
            var scrollTop = proxy._$gridContent.ejScroller("option", "scrollTop");
            proxy._$gridContent.ejScroller("option", { "persist": true });
            proxy._$gridContent.ejScroller("option", { "width": panSize });
            
            proxy._$gridContent.css("height", proxy._viewPortHeight);
            proxy._$gridContent.ejScroller("refresh");
            var isHorizontalScroll = proxy._$gridContent.ejScroller("isHScroll");

            if (proxy.model.isFromGantt)
                if (!isHorizontalScroll) {
                    proxy._$gridContent.addClass("e-borderbox");
                }
                else {
                    proxy._$gridContent.removeClass("e-borderbox");
                }
            proxy._$gridContent.ejScroller("option", { "scrollTop": scrollTop });

            if(!isHorizontalScroll)
            proxy._$gridContent.scrollTop(scrollTop);
            //else {
            //    proxy._$gridContent.scrollTop(0)
            //}
        },

        refreshHeight:function(){

            var proxy = this;
            proxy._$gridContent.ejScroller("refresh");            

        },

        //#endregion

        //#region PRIVATE METHODS

        //CALCULATE THE VIEWPORTHEIGHT
        _getViewPortHeight: function () {

            var proxy = this,
                height = 0,borderTopWidth=0;
            borderTopWidth = proxy._$gridHeaderContent.css("border-top-width");
            borderTopWidth = borderTopWidth === 'thin' ? 1 : borderTopWidth;
            height = proxy.element.height() -
                proxy._$gridHeaderContent.height() -
                parseFloat(borderTopWidth) -
                parseFloat(proxy._$gridHeaderContent.css("border-bottom-width"));

            return height;
        },

        updateViewPortHeight: function () {
            var proxy = this;
            proxy._viewPortHeight = proxy._getViewPortHeight();
        },

        _updateScrollCss: function () {
            var proxy = this;
            if (!proxy.model.isFromGantt) {
                var isVscroll = proxy.isVScroll();
                if (isVscroll) {
                        proxy._$gridHeaderContent.addClass("e-scrollcss");
                } else {
                    proxy._$gridHeaderContent.removeClass("e-scrollcss");
                }
            }
        },



        //UPDATE CURRENT VIEWPORT DATA FROM FLATRECORDS/UPDATED RECORDS
        _updateCurrentViewData: function () {

            var proxy = this,
                model = proxy.model,
                args = {},
                tMargin,
                isRangeModified = false;

            //CHECK VIRTUALIZATION IS ENABLED OR NOT 
            if (model.enableVirtualization) {

                proxy.getVisibleRange();
                model.currentViewData = proxy._updatedRecords.slice(proxy._visibleRange.top,
                    proxy._visibleRange.bottom);



                if (proxy._vScrollDist !== 0) {

                    tMargin = proxy._scrollTop - proxy._offset;

                    //console.log("Margin :" + tMargin);

                    if ((proxy._prevRTop - proxy._visibleRange.top !== 0) ||
                        (proxy._prevRBottom - proxy._visibleRange.bottom !== 0)) {

                        args.requestType = ej.TreeGrid.Actions.Refresh;
                        proxy.sendDataRenderingRequest(args);
                        isRangeModified = true;

                    }

                        proxy._$tableContent.css({
                            "top": tMargin
                        });

                    proxy._prevRTop = proxy._visibleRange.top;
                    proxy._prevRBottom = proxy._visibleRange.bottom;

                }

            } else {

                model.currentViewData = proxy._updatedRecords;

            }

            if (proxy._isEnterKeyPressed) {

                proxy._isEnterKeyPressed = false;
                if (model.allowSelection && model.editSettings.allowEditing) {

                    if (!isRangeModified) proxy._cellEditingDetails.rowIndex += 1;

                    if (!proxy._rowSelectingEventTrigger()) {

                        proxy.selectRows(proxy._cellEditingDetails.rowIndex);
                        proxy._rowSelectedEventTrigger(proxy._cellEditingDetails.rowIndex);

                    }

                    proxy.cellEdit(proxy._cellEditingDetails.rowIndex, proxy.model.columns[proxy._cellEditingDetails.columnIndex].field);

                }
            }
        },


        //GET THE TABLE ROW POSITION
        _getRowPosition: function (y) {
            return Math.ceil((y) / this.model.rowHeight);
        },


        //GET THE COLUMNGROUP FOR HEADER TABLE
        _getMetaColGroup: function () {

            var $colgroup = this.getHeaderTable().find('colgroup').clone();
            return $colgroup;

        },


        //CREATE EMPTY ROW WHILE DATASOURCE OBJECT AS NULL
        _getEmptyTbody: function () {

            var proxy = this,
                $emptyTd = ej.buildTag('td', proxy.model.emptyRecordText, {}, { colSpan: proxy.model.columns.length });
            return $(document.createElement("tr")).append($emptyTd);

        },


        //ADD SORTICON TO THE COLUMN HEADER
        _addSortElementToColumn: function (field, direction) {

            var proxy = this,
                column = ej.TreeGrid.getColumnByField(proxy.model.columns, field),
                index = $.inArray(column, proxy.model.columns),
                $headerCellDiv = proxy.getHeaderTable().find("thead").
                find(".e-headercell").eq(index).find(".e-headercelldiv");
            $headerCellDiv.find(".e-ascending,.e-descending").remove();
            $headerCellDiv.append(proxy._createSortElement().addClass("e-" + direction).addClass("e-icon"));
            $headerCellDiv.parent().attr("aria-sort", direction);
        },


        //CREATE SORT ELEMENT        
        _createSortElement: function () {
            return ej.buildTag('span', "&nbsp;");
        },


        //RENDERED EDITED CELL
        _renderCellEditObject: function (cellEditArgs, $td) {

            var proxy = this,
                $form = ej.buildTag("form.#" + proxy._id + "EditForm", "", {}, {}),
                $cellEditTemplate = proxy._cellEditTemplate,
                $element,
                htmlString,
                cellData = {};

            cellData[cellEditArgs.columnObject.field] = cellEditArgs.value,
            columnIndex = proxy.getColumnIndexByField(cellEditArgs.columnObject.field);

            if (columnIndex===proxy.model.treeColumnIndex) {
                $td.find('.e-cell').empty();
            } else $td.empty();

            htmlString = $cellEditTemplate.find("#" + cellEditArgs.columnObject.field + "_CellEdit").html();
            if (cellEditArgs.columnObject.field === "predecessor")
                $element = $($.templates(htmlString).render(cellEditArgs.data));
            else
                 $element = $($.templates(htmlString).render(cellData));

            if ($element.get(0).tagName == "SELECT") {

                $element.val(cellData[cellEditArgs.columnObject.field]);
                $element.val() == null && $element.val($element.find("option").first().val());

            }

            $form.append($element);

           if (columnIndex===proxy.model.treeColumnIndex) {
                $td.find('.e-cell').append($form);
            } else $td.append($form);

            proxy._setoffsetWidth();
            proxy._refreshEditForm();

            if ($.isFunction($.validator) && !$.isEmptyObject(cellEditArgs.validationRules)) {

                proxy._initValidator();
                proxy.setValidationToField(cellEditArgs.columnObject.field, cellEditArgs.validationRules);

            }
            proxy.model.isEdit = true;

        },


        //SET WIDTH FOR EDITING ELEMENTS
        _setoffsetWidth: function () {

            var proxy = this,
                tds, $form = $("#" + proxy._id + "EditForm"),
                i = 0,
                length, child;

            if (proxy.model.editSettings.editMode == "cellEditing") {
                tds = $form.closest("td");
            } else {
                tds = $form.find("tr").find(".e-rowcell");
            }

            length = tds.length;

            for (i; i < length; i++) {

                child = $(tds.children()[0]).children();

                if (child.length > 1) {
                    proxy._tdsOffsetWidth[i] = tds.get(i).offsetWidth - child[0].offsetWidth - child[1].offsetWidth;
                } else {
                    proxy._tdsOffsetWidth[i] = tds.get(i).offsetWidth;
                }
            }

        },


        //get Index of resourceInfo
        getIndexofresourceInfo: function (dataSource) {

            var proxy = this,
                data = proxy._cellEditingDetails.data,
                resourceInfo = data.resourceInfo,
                count = 0,
                length = resourceInfo && resourceInfo.length,
                resourceIndex = [];

            for (count; count < length; count++)
                resourceIndex.push(dataSource.indexOf(resourceInfo[count]));

            return resourceIndex;

        },

        //REFRESH THE EDITED FORM
        _refreshEditForm: function () {

            var proxy = this,
                form = document.getElementById(proxy._id + "EditForm"),
                elementFocused = false,
                $formElement = $(form).find("input,select"),
                percent = 86,
                i = 0,
                length = $formElement.length,
                $element, inputWidth,
                model = proxy.model,
                width,
                params = {},
                value,
                column,
                customParams,
                toformat,
                formatVal;

            for (i; i < length; i++) {

                $element = $formElement.eq(i);

                if (model.editSettings.editMode == "cellEditing") percent = 95;

                inputWidth = proxy._tdsOffsetWidth[i] * (percent / 100);

                if ($element.hasClass("e-numerictextbox")) {

                    width = inputWidth;
                    value = $element.val();
                    params.width = width;
                    params.height = "21px";
                    params.showSpinButton = true;
                    params.cssClass = model.cssClass;
                  
                    if (value.length) params.value = parseFloat(value);

                    customParams = ej.TreeGrid.getColumnByField(model.columns, $element.prop("name"));

                    if (model.isFromGantt && (customParams.field === "duration" || customParams.field === "status"))
                    {
                        params.minValue = 0;
                        if (customParams.field == "status")
                            params.maxValue = 100;
                    }

                    if (!ej.isNullOrUndefined(customParams["editParams"])) $.extend(params, customParams["editParams"]);

                    $element.ejNumericTextbox(params);
                    $element.prop("name", $element.prop("name").replace(proxy._id, ""));

                } else if ($element.hasClass("e-datepicker")) {

                    //inputWidth = proxy._tdsOffsetWidth[i] * (90 / 100);
                    width = inputWidth;
                    params.width = width;
                    params.height = "21px";
                    params.locale = model.locale;
                    params.cssClass = model.cssClass;
                    params.displayDefaultDate = true;
                    params.dateFormat = model.dateFormat;

                    if ($element.val().length) params.value = new Date($element.val());

                    column = ej.TreeGrid.getColumnByField(model.columns, $element.prop("name"));

                    if (column["format"] !== undefined && column.format.length > 0) {

                        toformat = new RegExp("\\{0(:([^\\}]+))?\\}", "gm");
                        formatVal = toformat.exec(column.format);
                        params.dateFormat = formatVal[2];
                    }

                    if (!ej.isNullOrUndefined(column["editParams"])) $.extend(params, column["editParams"]);

                    if (model.dateFormat.indexOf("hh") == -1)
                        $element.ejDatePicker(params);
                    else {
                        $element.ejDateTimePicker(params);
                    }
                } else if ($element.hasClass("e-datetimepicker")) {

                    width = inputWidth;
                    params = {
                        width: width,
                        height: "21px",
                        locale:model.locale,
                        cssClass: model.css,
                        showButton: false
                    };

                    if ($element.val().length) params.value = new Date($element.val());

                    column = ej.TreeGrid.getColumnByField(model.columns, $element.prop("name"));

                    if (column["format"] !== undefined && column.format.length > 0) {

                        toformat = new RegExp("\\{0(:([^\\}]+))?\\}", "gm");
                        formatVal = toformat.exec(column.format);
                        params.dateTimeFormat = formatVal[2];

                    }

                    if (!ej.isNullOrUndefined(column["editParams"])) $.extend(params, column["editParams"]);

                    if (model.dateFormat.indexOf("hh") == -1)
                        $element.ejDatePicker(params);
                    else
                        $element.ejDateTimePicker(params);

                } else if ($element.hasClass("e-dropdownlist")) {
                    var dataSource = model.columns[proxy._cellEditingDetails.columnIndex].dropdownData;

                    if(model.isFromGantt){
                        $element.ejDropDownList({
                            width: inputWidth + "px",
                            height: 21,
                            showCheckbox: true,
                            dataSource: dataSource,
                            popupWidth:"auto",
                            fields: {
                                id: model.resourceIdMapping,
                                text: model.resourceNameMapping,
                                value: model.resourceNameMapping
                            },
                            selectedItems: proxy.getIndexofresourceInfo(dataSource)
                        });
                        
                    } else {
                        $element.css({ 'position': 'absolute', 'margin-top': '-13px' });
                        $element.ejDropDownList({
                            width: inputWidth + "px",
                            height: "21px",                            
                            dataSource: dataSource,
                        });                        
                    }
                    $element.ejDropDownList("setSelectedValue", $element.val());

                } else {

                    switch ($element.prop('tagName')) {
                        case "INPUT":

                            if ($element.attr("type") != "checkbox") {

                                $element.css("text-align", $element.attr("name") != null &&
                                    ej.TreeGrid.getColumnByField(model.columns, $element.attr("name")) != null ?
                                    ej.TreeGrid.getColumnByField(model.columns, $element.attr("name")).textAlign :
                                    "center");

                                $element.outerWidth(inputWidth).height(19);

                            } else $element.width(inputWidth > 0 ? ($element.width() > inputWidth ? inputWidth : $element.width()) : 1);

                            break;

                        case "SELECT":
                            $element.width(inputWidth).height(20);
                            break;
                    }
                }

                if (!$element.is(":disabled") && !elementFocused &&
                    (!$element.is(":hidden") || typeof $element.data("ejDropDownList") == "object")) {

                    if (!proxy._isEnterKeyPressed) {

                        proxy._focusElements($element.closest('td'));
                        elementFocused = true;

                    }
                }
            }
        },


        //INITIALIZE THE VALIDATOR
        _initValidator: function() {

            var proxy = this;

            $("#" + proxy._id + "EditForm").validate({
                onclick: false,
                onfocusout: false,
                errorClass: 'e-field-validation-error',
                errorElement: 'div',
                wrapper: "div",

                errorPlacement: function(error, element) {
                    var $td = element.closest("td"),
                        $container = $(error).addClass("e-error"),
                        $tail = ej.buildTag("div.e-errortail e-toparrow");

                    $td.find(".e-error").remove();
                    element.hasClass("e-numerictextbox") ? $container.insertAfter(element.closest(".e-numeric")) :
                                                          $container.insertAfter(element);
                    $container.prepend($tail);
                    proxy.model.editSettings.editMode != "normal" && $container.offset({
                        left: element.offset().left
                    });

                    $container.fadeIn("slow");
                }
            });
        },


        //SET FOCUS TO EDITED ELEMENT
        _focusElements: function ($currentCell) {
            var proxy = this;

            if ($currentCell.length) {

                $currentCell.focus();
                var $childElem = $currentCell.children();

                if ($childElem[0]) {

                    if ($childElem[0].tagName.toLowerCase() == "select" || 
                        $childElem[0].tagName.toLowerCase() == "input") {

                        $childElem.focus().select();
                        $childElem[0].focus();

                    } else if ($childElem.find(".e-field.e-dropdownlist").length) {

                        $childElem.find(".e-ddl").focus();

                    } else if ($childElem.find(".e-datewidget.e-widget").length) {

                        $childElem.find(".e-datewidget.e-widget").focus();
                    }
                    else {

                        $childElem.find('input,select').select().focus();
                    }
                }
            }
			   proxy._$gridHeaderContainer.scrollLeft(proxy._$gridContent.children('.e-content').scrollLeft());
        },


        //UPDATE THE GRID CONTAINER HEIGHT
        updateHeight: function (height) {

            var proxy = this;

            if (proxy._$gridContainer && proxy._$gridContent) {
                if (height) {
                    proxy._$gridContainer.css({
                        "height": height + "px",
                        "width": "auto"
                    });
                    proxy._$gridContent.ejScroller("refresh");
                } else {
                    proxy._$gridContainer.css({
                        "height": proxy._totalHeight + "px"
                    });
                }                
                if (proxy._$gridContent.hasClass("e-scroller")) {
                    proxy._$gridContent.ejScroller("refresh");
                    proxy._updateScrollCss();
                }
            }
        },


        //ADD EXPANDED CHILDRECORDS TO FLAT RECORDS
        _addNestedExpandCollapseRecords: function (data, level, parentItem, index) {

            var proxy = this,
                model = proxy.model,
                records = model.flatRecords,
                rows = model.records,
                count = 0,
                length = data.length,
                parentRecord;


            for (count; count < length; count++) {
                
                index++;
                proxy._tempRecordIndex++;
                parentRecord = rows[proxy._tempRecordIndex];

                if ($.inArray(parentRecord, records) === -1) {
                    records.splice(index, 0, parentRecord);
                }

                if (parentRecord && parentRecord.isExpanded === false) {
                    parentRecord.isExpanded = true;
                }

                if (parentRecord.hasChildRecords) {

                    if (parentRecord.expanded) {

                        var len = parentRecord.childRecords.length,
                            k = 0,
                            childRecord;

                        for (k; k < len; k++) {

                            index++;
                            proxy._tempRecordIndex++;
                            childRecord = rows[proxy._tempRecordIndex];
                            records.splice(index, 0, childRecord);

                            if (childRecord.hasChildRecords) {

                                if (childRecord.expanded) {
                                    index = proxy._addNestedExpandCollapseRecords(childRecord.childRecords, level + 1, childRecord, index);
                                } else proxy._tempRecordIndex += proxy.getChildCount(childRecord, 0);//childRecord.childRecords.length;

                            }
                        }
                    } else {

                        proxy._tempRecordIndex += proxy.getChildCount(parentRecord,0);  //parentRecord.childRecords.length;// here inner task are not counted
                    }
                }
            }
            return index;
        },

        getChildCount:function(record,count)
        {
            var currentRecord, proxy = this;
            if (!record.hasChildRecords)
                return 0;
            for (var i = 0; i < record.childRecords.length;i++)
            {
                currentRecord= record.childRecords[i];
                count++;
                if(currentRecord.hasChildRecords)
                {
                    count=proxy.getChildCount(currentRecord, count);
                }
            }
            return count;
        },
        //UPDATE THE FILTERED RECORDS FOR ADDING PARENTRECORDS
        _updateFilteredRecords: function (data) {

            var proxy = this,
                count = 0,
                length = data.length,
                record,virtualization=proxy.model.enableVirtualization;

            proxy._filteredRecords = [];
            proxy._filteredGanttRecords = [];

            for (count; count < length; count++) {

                record = data[count];

                if (record.parentItem != null) {

                    proxy._addParentRecord(record.parentItem);

                    if ($.inArray(record, proxy._filteredRecords) === -1) {
                        if (virtualization)
                        {
                            if (proxy.getExpandStatus(record))
                            {
                                proxy._filteredRecords.push(record);
                            }
                        }
                        else
                        {
                            proxy._filteredRecords.push(record);
                        }
                           
                    }

                } else {

                    if ($.inArray(record, proxy._filteredRecords) === -1) {
                        if (virtualization) {
                            if (proxy.getExpandStatus(record)) {
                                proxy._filteredRecords.push(record);
                            }
                        }
                        else {
                            proxy._filteredRecords.push(record);
                        }
                    }
                }
            }
        },

                
        //ADD PARENTRECORD FOR FILTERED OTH LEVEL ITEMS
        _addParentRecord: function (record) {

            var proxy = this,
            virtualization = proxy.model.enableVirtualization;

            if (record.parentItem) {
                proxy._addParentRecord(record.parentItem);

                if ($.inArray(record, proxy._filteredRecords) === -1) {

                    if (virtualization) {
                        if (proxy.getExpandStatus(record)) {
                            proxy._filteredRecords.push(record);
                        }
                    }
                    else {
                        proxy._filteredRecords.push(record);
                    }
                     

                }
            } else {

                if ($.inArray(record, proxy._filteredRecords) === -1) {

                    if (virtualization) {
                        if (proxy.getExpandStatus(record)) {
                            proxy._filteredRecords.push(record);
                        }
                    }
                    else {
                        proxy._filteredRecords.push(record);
                    }

                }
            }
        },


        //UPDATEEXPANDCOLLAPSE FOR SORTED RECORDS
        _updateExpandCollapseForSortedRecords: function (Record, expanded) {

            var proxy = this,
                count = 0,
                length = proxy._sortedRecords.length,
                record,
                index = proxy._sortedRecords.indexOf(Record);

            if (expanded) {

                proxy._addNestedSortRecords(Record, index);

            } else {

                var removedCount = 0;

                for (count = index + 1; count < length; count++) {

                    record = proxy._sortedRecords[count];

                    if (record.level === Record.level) {
                        break;
                    } else if (!proxy.getExpandStatus(record)) {
                        removedCount++;
                    }

                }

                proxy._sortedRecords.splice(index + 1, removedCount);

            }
        },


        //ADD NESTEDSORTRECORDS
        _addNestedSortRecords: function (record, index) {

            var proxy = this,
                childRecords,
                sortColumns = proxy.model.sortSettings.sortedColumns,
                length;

            if (sortColumns.length) {

                length = sortColumns.length - 1;
                proxy._queryManagar.queries = [];

                for (var i = length; i >= 0; i--) {

                    proxy._queryManagar.sortBy(sortColumns[i].field, sortColumns[i].direction);

                }

            }
            if (record.hasChildRecords) {

                var dataManager = ej.DataManager(record.childRecords),
                    count,
                    childrecord;

                childRecords = dataManager.executeLocal(proxy._queryManagar).result;
                length = childRecords.length;

                for (count = 0; count < length; count++) {

                    childrecord = childRecords[count];

                    if (proxy._filteredRecords.length) {

                        if ($.inArray(childrecord, proxy._filteredRecords) !== -1) {
                            proxy._sortedRecords.splice(++index, 0, childrecord);
                        }

                    } else {
                        proxy._sortedRecords.splice(++index, 0, childrecord);
                    }

                    if (childrecord.hasChildRecords && childrecord.expanded) {
                        proxy._addNestedSortRecords(childrecord, index);
                    }
                }
            }
        },


        //UPDATE ALTROW
        _updateAltRow: function () {

            var proxy = this,
                count = 0,
                currentViewData = proxy._updatedRecords,
                length = currentViewData.length,
                record;

            for (count = 0; count < length; count++) {
                record = currentViewData[count];
                record.isAltRow = count % 2 == 0 ? true : false;
            }

        },


        
        //COUNT THE NUMBER OF CHILD RECORDS PRESENT IN THE GANTTRECORD
        _getFilteredChildRecordsCount: function (Record) {

            var proxy = this,
                model=proxy.model,
                length = Record&&Record.childRecords && Record.childRecords.length,
                i = 0,
                record;

            for (i = 0; i < length; i++) {

                record = Record.childRecords[i];
                if (model.enableVirtualization)
                {
                    if(proxy.getExpandStatus(record))
                    {
                        proxy._removedCount++;
                    }
                }
                else
                {
                    proxy._removedCount++;
                }
                   
                if (record.childRecords) {
                    proxy._getFilteredChildRecordsCount(record);
                }
            }

            return proxy._removedCount;
        },


        //SORTING THE RECORDS
        _sortingRecords: function () {

            var proxy = this,
                model = proxy.model,
                sortcolumns = model.sortSettings.sortedColumns;

            proxy._queryManagar.queries = [];

            if (sortcolumns.length) {

                var length = sortcolumns.length - 1;

                for (var i = length; i >= 0; i--) {

                    var column;
                    if(model.isFromGantt)
                        column = ej.TreeGrid.getColumnByMappingName(model.columns, sortcolumns[i].field);
                    if (column) {
                        proxy._queryManagar.sortBy(column.field, sortcolumns[i].direction);
                        sortcolumns[i].field = column.field;
                    }
                    else
                        proxy._queryManagar.sortBy(sortcolumns[i].field, sortcolumns[i].direction);
                }

            }

            var dataManager = new ej.DataManager(model.parentRecords),
                records = dataManager.executeLocal(proxy._queryManagar).result;

            proxy._sortedRecords = [];
            proxy._storedIndex = -1;

            proxy._createSortedRecords(records);
            proxy._tempsortedrecords = proxy._sortedRecords;
           // proxy._updateSortedRecords(proxy._tempsortedrecords);
            proxy.setRecordsCount(proxy._sortedRecords.length);
            proxy._updatedRecords = proxy._sortedRecords;

            return true;
        },


        //CREATE SORTED GANTT RECORDS
        _createSortedRecords: function (data) {

            var proxy = this,
                sortedRecords = proxy._sortedRecords,
                dataManager,
                enableAltRow = proxy.model.enableAltRow;

            $.each(data, function (index, record) {

                if (proxy._searchString.length > 0) {

                    if (proxy._filteredRecords.length > 0 && proxy._filteredRecords.indexOf(record) !== -1) {

                        proxy._storedIndex++;

                        if (enableAltRow) {

                            record.isAltRow = proxy._storedIndex % 2 == 0 ? false : true;

                        }

                        sortedRecords[proxy._storedIndex] = record;

                    }
                } else {

                    proxy._storedIndex++;

                    if (enableAltRow) {

                        record.isAltRow = proxy._storedIndex % 2 == 0 ? false : true;

                    }

                    sortedRecords[proxy._storedIndex] = record;

                }

                if (record.hasChildRecords && record.expanded) {

                    dataManager = ej.DataManager(record.childRecords);
                    var childRecords = dataManager.executeLocal(proxy._queryManagar).result;
                    proxy._createSortedRecords(childRecords);

                }
                else if (record.hasChildRecords && !record.expanded && !model.enableVirtualization)
                {
                    proxy._setChildRecords(record, sortedRecords);
                }
            });
        },

        _setChildRecords:function(record,sortedRecords)
        {
            var proxy = this,
            enableAltRow = proxy.model.enableAltRow;
            var length = record.childRecords.length;

            dataManager = ej.DataManager(record.childRecords);
            var childRecords = dataManager.executeLocal(proxy._queryManagar).result;
            for(var count=0;count<length;count++)
            {
                if (proxy._filteredRecords.length > 0) {
                    if (proxy._filteredRecords.length > 0 && proxy._filteredRecords.indexOf(childRecords[count]) !== -1) {
                        proxy._storedIndex++;
                        if (enableAltRow) {
                            childRecords[count].isAltRow = proxy._storedIndex % 2 == 0 ? false : true;
                        }
                        sortedRecords[proxy._storedIndex] = childRecords[count];
                        if (childRecords[count].hasChildRecords) {
                            this._setChildRecords(childRecords[count], sortedRecords);
                        }
                    }
                }else
                {
                    proxy._storedIndex++;
                    if (enableAltRow) {
                        childRecords[count].isAltRow = proxy._storedIndex % 2 == 0 ? false : true;
                    }
                    sortedRecords[proxy._storedIndex] = childRecords[count];
                    if (childRecords[count].hasChildRecords) {
                        this._setChildRecords(childRecords[count], sortedRecords);
                    }
                }
            }
        },    
        //UPDAT THE SORTED RECORDS
        _updateSortedRecords: function (data) {

            var proxy = this,
                count = 0,
                length = data.length,
                record,model=this.model;

            for (count; count < length; count++) {

                record = data[count];

                if (!model.enableVirtualization) {

                    if (proxy._filteredRecords.length > 0) {

                        if ($.inArray(record, proxy._filteredRecords) !== -1 && $.inArray(record, proxy._sortedRecords) === -1) {
                            proxy._sortedRecords.push(record);
                        }

                    } else {

                        if ($.inArray(record, proxy._sortedRecords) === -1) {
                            proxy._sortedRecords.push(record);
                        }

                    }

                    if (record.hasChildRecords && record.childRecords.length > 0) {

                        var dataManager = ej.DataManager(record.childRecords),
                            sortedChildRecords = dataManager.executeLocal(proxy._queryManagar).result;
                        proxy._updateSortedRecords(sortedChildRecords);

                    }
                }
            }
        },


        //REMOVE SORT ICON FORM COLUMN HEADER
        _removeSortElementFromColumn: function (field) {

            var proxy = this,
                column = ej.TreeGrid.getColumnByField(proxy.model.columns, field),
                index = $.inArray(column, proxy.model.columns),
                $headerCellDiv = proxy.getHeaderTable().find("thead").find(".e-headercell").eq(index).find(".e-headercelldiv");

            $headerCellDiv.find(".ascending,.descending").remove();
            $headerCellDiv.parent().removeAttr("aria-sort");
        },


        //ADD EMPTYCOLUMN TO THE TREEGRID
        _addEmptyColumntoGrid: function () {

            $('.e-gridheadercontainer table colgroup').append(ej.buildTag('col', "", {
                "width": "auto"
            },
            {}));

            $('.e-gridheadercontainer table thead tr').append(ej.buildTag('th.e-headercell'));
        },


        //UPDATE THE SCROLLTOP OF THE GRIDCONTAINER
        _updateScrollTop: function (y) {

            var args = {},
                proxy = this;

            args.requestType = "scroll";
            args.delta = y;

            var isHorizontalScroll = proxy._$gridContent.ejScroller("isHScroll");

            if (!isHorizontalScroll) {
                proxy._$gridContent.scrollTop(y);//grid container
            }
            else
            {
                proxy._$gridContent.scrollTop(0)
            }
            proxy._$gridContent.ejScroller("scrollY", y, true);
            proxy._completeAction(args);
            
        },


        //MOVE CURRENT CELL FOR KEYBOARD INTERACTION
        _moveCurrentCell: function (direction) {

            var proxy = this,
                rowIndex = proxy._cellEditingDetails.rowIndex,
                columnIndex = proxy._cellEditingDetails.columnIndex,
                model = proxy.model;

            if (rowIndex == -1 && columnIndex == -1) return false;

            if (proxy.model.isEdit) {

                proxy.saveCell();
                proxy.model.isEdit = false;

            }

            switch (direction) {
                

                case "right":
                    if (!proxy._rowSelectingEventTrigger()) {

                        if (!proxy.selectRows(proxy._cellEditingDetails.rowIndex)) {

                            if (rowIndex == proxy._gridRows.length - 1 && columnIndex == model.columns.length - 1) {
                                return true;
                            }

                            if (columnIndex == model.columns.length - 1) {

                                proxy._cellEditingDetails.columnIndex = 0;
                                proxy._cellEditingDetails.rowIndex = rowIndex + 1;

                            } else proxy._cellEditingDetails.columnIndex = columnIndex + 1;

                            
                            proxy.cellEdit(proxy._cellEditingDetails.rowIndex,
                                model.columns[proxy._cellEditingDetails.columnIndex].field);
                        }
                    }

                    break;

                case "left":

                    if ((rowIndex == 0 && columnIndex == 0)) return true;

                    if (columnIndex == 0) {

                        proxy._cellEditingDetails.columnIndex = model.columns.length - 1;
                        proxy._cellEditingDetails.rowIndex = rowIndex - 1;

                    } else {
                        proxy._cellEditingDetails.columnIndex = columnIndex - 1;
                    }

                    proxy.selectRows(proxy._cellEditingDetails.rowIndex);
                    proxy.cellEdit(proxy._cellEditingDetails.rowIndex, model.columns[proxy._cellEditingDetails.columnIndex].field);

                    break;

                case "up":

                    if (rowIndex == 0) return true;

                    proxy._cellEditingDetails.rowIndex = rowIndex - 1;
                    proxy.selectRows(proxy._cellEditingDetails.rowIndex);
                    proxy._rowSelectedEventTrigger(proxy._cellEditingDetails.rowIndex);
                    proxy.cellEdit(proxy._cellEditingDetails.rowIndex, model.columns[proxy._cellEditingDetails.columnIndex].field);

                    break;

                case "down":

                    if (rowIndex + 1 == proxy.getRows().length - 1) {

                        if (proxy.model.enableVirtualization) {

                            proxy._isEnterKeyPressed = true;
                            proxy._updateScrollTop(proxy._$gridContent.scrollTop() + model.rowHeight - proxy._offset);
                        }
                    }

                    if (!proxy._isEnterKeyPressed) {

                        if (proxy._updatedRecords.length > (rowIndex + 1))
                        if (!proxy.selectRows(rowIndex + 1)) {

                            proxy._rowSelectedEventTrigger(proxy._cellEditingDetails.rowIndex+1);
                            proxy._cellEditingDetails.rowIndex = rowIndex + 1;
                            proxy.cellEdit(proxy._cellEditingDetails.rowIndex, model.columns[proxy._cellEditingDetails.columnIndex].field);
                            args = {};
                            args.requestType = "scroll";
                            args.delta = proxy._$gridContent.children('.e-content').scrollTop();
                            proxy._trigger("actionComplete", args);
                            //$(".e-ganttviewerbodyContianer").children('.e-content').scrollTop((proxy._$gridContent.children('.e-content').scrollTop()));
                        }

                    }

                    break;
            }

            return false;

        },

        //FIND THE COLUMNWIDTH OF THE TREEGRID COLUMN
        _findColumnsWidth: function (extra) {

            var proxy = this,
                length = proxy.model.columns.length,
                j = proxy.getHeaderTable().find("colgroup").find("col");
            
            for (var i = 0; i < length; i++)
                proxy._columnsWidthCollection[i] = j.eq(i).width();

        },

        //CALCULATE THE TOTAL WIDTH OF THE COLUMNS
        _calculateWidth: function () {

            var j = this.getHeaderTable().find("colgroup").find("col"),
                width = 0;

            for (var i = 0; i < j.length; i++)
                width += j.eq(i).width();

            return width;
        },


        //TEMPLATE FOR DROP DOWNLIST
        _compiledDropDownTemplate: function () {

            var $select = ej.buildTag('select'),
                $option = ej.buildTag("option", "{{:text}}", {},
                {
                    value: "{{:value}}"
                });

            $select.append($option);

            return $.templates($select.html());
        },


        _renderScroller: function () {

            var proxy = this,
            model = proxy.model;
            if (this.dataSource() === null)
                return;
            var containerHeight = model.flatRecords.length * model.rowHeight;//flatrecords*rowheight//
            var contentHeight = proxy._viewPortHeight < 0 ? containerHeight : proxy._viewPortHeight-18;//viewportheight ||
            proxy._$gridContent.css({ "height": contentHeight + "px" });
            proxy._$gridContainer.css({ "height": containerHeight + "px" });
            
            if (!proxy._$gridContent.hasClass("e-scroller")) {
                $div = ej.buildTag('div', {}, { "width": "auto", "height": "auto" });
                $div.append(proxy._$gridContainer);
                proxy._$gridContent.append($div);
                var height = proxy._gridHeight ? contentHeight : "auto";
                var width = proxy.element.width(); //is set for minus border width.

                proxy._$gridContent.ejScroller({
                    height:proxy.model.isFromGantt ? 0 :height,
                    width: proxy.model.isFromGantt?width:width-2,
                    scroll: $.proxy(proxy._onScroll, proxy)
                });
                proxy._$gridContent.ejScroller("refresh");

            }
            else {
                proxy._$gridContent.ejScroller("refresh");
          }
            proxy._$gridContent.ejScroller("model.keyConfigs", { up: "", down: "" });
            proxy._updateScrollCss();
            var isHorizontalScroll = proxy._$gridContent.ejScroller("isHScroll");
            if (proxy.model.isFromGantt)
                if (!isHorizontalScroll) {
                    proxy._$gridContent.addClass("e-borderbox");
                }
                else {
                    proxy._$gridContent.removeClass("e-borderbox");
                }

        },
        

        
        //ALL EVENTS BOUND USING THIS._ON WILL BE UNBIND AUTOMATICALLY
        _destroy: function () {

            var proxy = this;
            proxy.element.empty().removeClass("e-treegrid-core e-treegrid " + proxy.model.cssClass);
            
        },

        /**
          * To collapse all the parent items in tree grid
          * @return jQuery
          * @example 
          * &lt;treegrid id="treegrid"&gt;Treegrid&lt;/treegrid&gt; <br> 
          * &lt;script&gt;
          * // Create treegrid
          * var treegridObj = $("#treegrid").data("ejTreeGrid");
          * treegridObj.ejTreeGrid("collapseAll"); // To collapse all parent items in tree grid
          * &lt;/script&gt;
          * @example 
          *  &lt;treegrid id="treegrid"&gt;Treegrid&lt;/treegrid&gt; <br> 
          * &lt;script&gt;
          * // To expand all items
          * $("#treegrid").ejTreeGrid("collapseAll");	
          * &lt;/script&gt;
          * @memberof ejTreeGrid
          * @instance
           */

        collapseAll: function () {
            var proxy = this,
                model = proxy.model,
                count = 0,
                ganttRecord,
                args = {},
                enableVirtualization = model.enableVirtualization,
                $gridRows = proxy.getRows(),
                $rowElement = null;

            proxy._gridRows = $gridRows;
            //loop excecute for iterate the GanttRecords preseent in the Gantt Control

            for (count = 0; count < proxy.model.parentRecords.length; count++) {
                ganttRecord = proxy.model.parentRecords[count];
                args.data = ganttRecord;
                args.recordIndex = count;
                args.expanded = false;

                if (enableVirtualization) {
                    if (ganttRecord.hasChildRecords && ganttRecord.expanded) {
                        ej.TreeGrid.sendExpandCollapseRequest(proxy, args);
                    }
                } else {
                    if (ganttRecord.hasChildRecords && ganttRecord.expanded) {
                        ej.TreeGrid.sendExpandCollapseRequest(proxy, args);
                    }
                    else
                        ej.TreeGrid.sendExpandCollapseRequest(proxy, args);
                }
            }
        },

        //#endregion       

    });

    //ENUM FOR EACH ACTION IN TREEGRID
    ej.TreeGrid.Actions = {
        Sorting: "sorting",
        BeginEdit: "beginedit",
        Save: "save",
        Add: "add",
        Delete: "delete",
        Cancel: "cancel",
        Refresh: "refresh",
        Searching: "searching",
        ExpandCollapse: "expandcollapse",
        Selection: "selection",
        rowHover: "rowHover",
        Scroll: "scroll",
        ContextMenuAdd: "contextMenuAdd",
        RefreshDataSource:"refreshDataSource"
    };

    //ENUM FOR SELECTION MODE IN TREEGRID
    ej.TreeGrid.SelectionType = {
        Single: "single",
        Multiple: "multiple"
    };

    //ENUM FOR DIFFERENT EDIT MODE IN TREEGRID
    ej.TreeGrid.EditMode = {
        Normal: "normal",
        CellEditing: "cellEditing"
    };

    /* enum for five different columnEditType for treegrid */
    ej.TreeGrid.EditingType = {
        String: "stringedit",
        Boolean: "booleanedit",
        Numeric: "numericedit",
        Dropdown: "dropdownedit",
        DatePicker: "datepicker"        
    };

    
    ej.TreeGrid = ej.TreeGrid || {};


    //INITIALIZE THE CELLEDIT ELEMENT FOR DIFFERENT TYPES OF EDITORS
    ej.TreeGrid._initCellEditType = function (instance, $element, id, columnCount) {

        var proxy=instance,
            model=proxy.model,
            columns=model.columns,
            column=columns[columnCount];

        if (ej.isNullOrUndefined(columns[columnCount]["editType"])) {
            column["editType"] = "stringedit";
        }

        switch (column["editType"]) {
            
            case "stringedit":

                if(model.isFromGantt){
                    
                    var helpers={};
                    
                    helpers["_" + id + "cellValue"] = ej.TreeGrid._getCellValue;
                    
                    if(column.field==="predecessor"){
                        helpers["_" + id + "predecessorCell"] = ej.Gantt._getPredecessorsValue;
                    }
                    
                    $.views.helpers(helpers);


                    if(column.field==="predecessor"){ //get the predecessor value from item of GanttRecord
                     
                        var $input=ej.buildTag('input.e-field e-ejinputtext',
                                       "",{},
                                       {
                                           value:"{{:~_" + id + "predecessorCell('"+model.predecessorMapping+"')}}",
                                           id: id + column.field, 
                                           name: column.field
                                       }
                                       );
                    }else { /*get the cellValue if the column does not exist in the GanttRecord object and
                           present in the item of GanttRecord*/
                        $input= ej.buildTag('input.e-field e-ejinputtext', 
                                            "",
                                            {},
                                            {
                                                value: "{{:~_" + proxy._id + "cellValue('" + columns[columnCount].field + "')}}",
                                                id: proxy._id + model.columns[columnCount].field,
                                                name: model.columns[columnCount].field
                                            });
                       
                    }

                }else { /*Get the cellvalue of the normal gridcell*/
                    $input=ej.buildTag('input.e-field e-ejinputtext',
                                          "",
                                          {}, 
                                          {
                                              value: "{{:#data['" + column.field + "']}}", 
                                              id: id + column.field, 
                                              name: column.field
                                          });
                }    
                $element.html($input);
            
                break;

            case "booleanedit":
                
                $element.html('{{if ' + columns[columnCount].field + '}} ' +
                        '<input class="e-field e-checkbox" type ="checkbox" ' +
                        'id=' + proxy._id + columns[columnCount].field +
                        'name=' + columns[columnCount].field +
                        'checked="checked"></input>' +
                        '{{else}} ' +
                        '<input class="e-field e-checkbox"' +
                        ' type ="checkbox" ' +
                        'id=' + proxy._id + columns[columnCount].field +
                        'name=' + columns[columnCount].field + ' > ' +
                        '{{/if}}');

                break;
            
            case "numericedit":

                var $numericText = ej.buildTag('input.e-numerictextbox e-field', "", {}, {
                    type: "text",
                    value: "{{:#data['" + columns[columnCount].field + "']}}",
                    id: proxy._id + columns[columnCount].field,
                    name: columns[columnCount].field
                });
                $element.append($numericText);

                break;

            case "datepicker":
            case "datetimepicker":
                
                var $datePicker = ej.buildTag('input.e-' + columns[columnCount]["editType"] + ' e-field', "",
                                             {}, 
                                             {
                                                 type: "text",
                                                 value: "{{:#data['" + columns[columnCount].field + "']}}",
                                                 id: id + columns[columnCount].field,
                                                 name: columns[columnCount].field
                                             });
                $element.append($datePicker);

                break;

            case "dropdownedit":

                if(model.isFromGantt){
                    var $dropDownList = ej.buildTag('input.e-field e-dropdownlist' + ' e-field',
                        "", 
                        {}, 
                        {
                            type: "text",
                            value: "{{:#data['" + columns[columnCount].field + "']}}",
                            id: id + columns[columnCount].field,
                            name: columns[columnCount].field
                        });
                    $element.append($dropDownList);
                }else {
                    if (!ej.isNullOrUndefined(column.dropdownData)) {
                        var selectedItems = column.dropdownData;
                        var dropDownTemplate = proxy._compiledDropDownTemplate();
                        $element.get(0).innerHTML = ["<select>", dropDownTemplate.render(selectedItems), "</select>"].join("");
                        $element.find("select").prop({ id: proxy._id + column.field, name: column.field }).addClass("e-field e-dropdownlist");
                    }
                }

                break;
        }
    };
    
    ej.TreeGrid._getCellValue= function (columnName) {

        var cellValue = this.data[columnName];

        if (cellValue) {
            return cellValue;
        } else {
            return this.data.item && this.data.item[columnName];
        }


    };
    
    //GET ROW CLASS NAME FOR EXPAND COLLAPSE ACTION
    ej.TreeGrid._getrowClassName = function () {

        var rowClass = "gridrowIndex",
            proxy = this;

        if (proxy.data.parentItem) {// && proxy.data.parentItem.index

            rowClass += proxy.data.parentItem.index.toString();

        }

        rowClass += "level";
        rowClass += proxy.data.level.toString();

        return rowClass;
    };

    //GET COLUMN BY FIELD NAME
    ej.TreeGrid.getColumnByField = function (columns, field) {

        var column = 0;

        for (column; column < columns.length; column++) {

            if (columns[column]["field"] == field) break;

        }

        return column == columns.length ? null : columns[column];
    };


    ej.TreeGrid.getColumnByMappingName = function (columns, field) {
        var column = 0;
        for (column; column < columns.length; column++) {
            if (columns[column]["mappingName"] == field) break;
        }
        return column == columns.length ? null : columns[column];
    };

    //EXPAND COLLAPSE THE PARTICULAR RECORD OBJECT
    ej.TreeGrid.sendExpandCollapseRequest = function (element, args) {

        var proxy = element,
            model = proxy.model;

        args.requestType = ej.TreeGrid.Actions.ExpandCollapse;

        if (model.enableVirtualization) {

            if (proxy._isFromGantt) {
                proxy._$treegridHelper.ejTreeGrid("processBindings", args);
            } else {
                proxy.processBindings(args);
            }

            if (args.expanded) {

                proxy._trigger("expanded", args);

            } else {

                proxy._trigger("collapsed", args);

            }

            if (proxy._isFromGantt) {
                proxy._$treegridHelper.ejTreeGrid("sendDataRenderingRequest", args);
                proxy._updatedGanttRecords = proxy.getUpdatedRecords();
                proxy._currentViewData = proxy.getCurrentViewData();
                if (proxy.model.enableVirtualization) {
                    if (proxy.model.predecessorMapping) {
                        proxy._isValidationEnabled = false;
                        proxy._connectorlineIds = [];
                        proxy._connectorLinesCollection = [];
                        proxy._$ganttchartHelper.ejGanttChart("clearConnectorLines");
                        proxy._createConnectorLinesCollection();
                    }
                   proxy._$ganttchartHelper.ejGanttChart("refreshHelper", proxy._currentViewData, proxy._updatedGanttRecords);
                   proxy._$treegridHelper.ejTreeGrid("refreshHeight");                    
                }
                
            } else {
                proxy.sendDataRenderingRequest(args);
                proxy._$gridContent.ejScroller("refresh");
            }
            

        } else {

            var record = args.data,
                index,
                $row,
                height = 0,
                isModified = record.expanded !== args.expanded;

            if (proxy._isFromGantt) {
                index = proxy._currentViewData.indexOf(record)
            } else {
                index = proxy.model.currentViewData.indexOf(record);
            }

            $row = $(proxy.getRows()[index]);

            if (args.expanded) {//expand event 
                record.expanded = args.expanded;
                proxy._expandedRecordsCount = 0;
                ej.TreeGrid.expandRecord(proxy, record, args.expanded, args);

                if (isModified) {
                    proxy._totalCollapseRecordCount -= proxy._expandedRecordsCount;
                }

                $row.find(".e-treegridcollapse").removeClass('e-treegridcollapse').addClass('e-treegridexpand');
                $row.removeClass('e-treegridrowcollapse').addClass('e-treegridrowexpand');                

                if (model.enableAltRow) {
                    ej.TreeGrid.updateAltRow(proxy, record, index, 1);
                }
            } else {

                proxy._collapsedRecordCount = 0;
                ej.TreeGrid.collapseRecord(proxy, record, args.expanded);
                
                if (isModified)
                    proxy._totalCollapseRecordCount += proxy._collapsedRecordCount;

                record.expanded = args.expanded;
                $row.find(".e-treegridexpand").removeClass('e-treegridexpand').addClass('e-treegridcollapse');
                $row.removeClass('e-treegridrowexpand').addClass('e-treegridrowcollapse');               
                    
                if (model.enableAltRow)
                    ej.TreeGrid.updateAltRow(proxy, record, index, proxy._collapsedRecordCount + 1);
            }


            var filteredCollapseCount = 0;
            if (proxy._searchString) {
                filteredCollapseCount = proxy.getUpdatedRecords().filter(function (record) {
                    return (proxy.getExpandStatus(record) === false);
                });
                height = (proxy.getUpdatedRecords().length - filteredCollapseCount.length) * model.rowHeight;
            } else
                height = (proxy.getUpdatedRecords().length - proxy._totalCollapseRecordCount) * model.rowHeight;
            if (proxy._isFromGantt) {
                proxy._$treegridHelper.ejTreeGrid("updateHeight", height);
                proxy._$ganttchart.ejGanttChart("updateHeight", height);
                proxy._$ganttchart.ejGanttChart("setCollapsedRecordCount", proxy._totalCollapseRecordCount);
                proxy._$treegridHelper.ejTreeGrid("setCollapsedRecordCount", proxy._totalCollapseRecordCount);
                
            } else {
                proxy.updateHeight(height);
            }

            

            if (args.expanded)
                proxy._trigger("expanded", args);
            else
                proxy._trigger("collapsed", args);
        }
        if (proxy._isFromGantt) {
            var treeGrid = proxy._$treegridHelper.ejTreeGrid("instance");
            var chart = proxy._$ganttchart.ejGanttChart("instance");
            var top = treeGrid._$gridContent.ejScroller("option", "scrollTop");
            chart._$bodyContainer.ejScroller("option", "scrollTop", top);
        }
    };

    //EXPAND THE RECORD
    ej.TreeGrid.expandRecord = function (controlObject, Record, expanded, args) {
  
        var proxy = controlObject,
            rowClassName = ".gridrowIndex" + Record.index.toString() + "level" + (Record.level + 1).toString(),
            $rows = $(rowClassName),
            count = 0,
            length = $rows.length,
            row,
            index,
            record,
            model = proxy.model,
            $expandedRows = $(rowClassName),
            index,
            $gridRows = proxy.getRows(),
            $row = $($gridRows[index]);

        if (proxy._isFromGantt) {
            index = proxy._currentViewData.indexOf(Record);


        } else {
            index = model.currentViewData.indexOf(Record);
        }

        if (Record.hasChildRecords && Record.childRecords.length > 0) {
            $row.find(".e-treegridcollapse").removeClass('e-treegridcollapse').addClass('e-treegridexpand');
            $row.removeClass('e-treegridrowcollapse').addClass('e-treegridrowexpand');
        }

        proxy._expandedRecordsCount += proxy._isFromGantt ? $expandedRows.length / 2 : $expandedRows.length;

        if ($expandedRows.length === 0) {
            if (proxy.model.allowSearching && proxy._searchString && proxy._searchString.length > 0) {
                args && ej.TreeGrid.sendExpandCollapseRequest(proxy, args);
            }
        }

        $expandedRows.css({ 'display': 'table-row' });

        if (proxy._isFromGantt) {
         
            length = length / 2;
        }

        for (count; count < length; count++) {
            row = $rows[count];
            index = $gridRows.index(row);
            if (index !== -1) {
                record = proxy._isFromGantt ? proxy._updatedGanttRecords[index] : proxy._updatedRecords[index];
                record.isExpanded = true;
                if (record.expanded)
                    ej.TreeGrid.expandRecord(proxy,record, expanded);
            }
        }

        if (proxy._isFromGantt)
        proxy.updateConnectorLines($expandedRows, expanded);
    },
    
    //COLLAPSE THE RECORD
    ej.TreeGrid.collapseRecord = function (controlObject, Record, expanded) {

        var proxy = controlObject,
            model = proxy.model,
            rowClassName = ".gridrowIndex" + Record.index.toString() + "level" + (Record.level + 1).toString(),
            $rows = $(rowClassName),
            count = 0,
            length = $rows.length,
            row,
            index,
            record,
            $collapsedRows = $(rowClassName),
        $gridRows = proxy.getRows();

        proxy._collapsedRecordCount += proxy._isFromGantt ? $collapsedRows.length / 2 : $collapsedRows.length;
        $collapsedRows.css({ 'display': 'none' });

        if (proxy._isFromGantt) {

            proxy.updateConnectorLines($collapsedRows, expanded);
            length = length / 2;

        }

        for (count; count < length; count++) {

            row = $rows[count];
            var nn = proxy.getRows();
            index = proxy.getRows().index(row);


            if (index !== -1) {
                record = proxy.getUpdatedRecords()[index];

                if (record.expanded)
                    ej.TreeGrid.collapseRecord(proxy,record, expanded);
            }
        }
    },

    //UPDATE THE ALT ROW FUNCTION
    ej.TreeGrid.updateAltRow = function (controlObject, Record, recordIndex, offset) {

        var proxy = controlObject,
            count = 0,
            model = proxy.model,
            currentViewData = proxy._isFromGantt ? proxy._currentViewData : model.currentViewData,
            length = currentViewData.length,
            isAltRow = Record.isAltRow,
            record,
            $gridRows = proxy.getRows();

        for (count = recordIndex + offset; count < length; count++) {

            if ($($gridRows[count]).css('display') === "none")
                continue;

            record = currentViewData[count];
            record.isAltRow = !isAltRow;
            isAltRow = record.isAltRow;

            if (isAltRow)
                $($gridRows[count]).addClass('e-alt-row');
            else
                $($gridRows[count]).removeClass('e-alt-row');

        }
    };     

    jQuery.uaMatch = function (ua) {
        ua = ua.toLowerCase();

        var match = /(chrome)[ \/]([\w.]+)/.exec(ua) ||
            /(webkit)[ \/]([\w.]+)/.exec(ua) ||
            /(opera)(?:.*version|)[ \/]([\w.]+)/.exec(ua) ||
            /(msie) ([\w.]+)/.exec(ua) ||
            ua.indexOf("compatible") < 0 && /(mozilla)(?:.*? rv:([\w.]+)|)/.exec(ua) ||
            [];

        return {
            browser: match[1] || "",
            version: match[2] || "0"
        };
    };

})(jQuery, Syncfusion);;;;
(function ($, ej, undefined) {
    // ejSample is the plugin name 
    // "ej.Sample" is "namespace.className" will hold functions and properties
    /**
    * @namespace ej
    * @classdesc The ribbon can be easily configured to the DOM element, such as div. you can create a ribbon with a highly customizable look and feel.
    * @class ejRibbon
    * @param {object} options - settings for ribbon 
    * @requires jQuery
    * @requires jquery.easing.min.js
    * @requires jquery.globalize.min.js    
    * @requires ej.core.js
    * @requires ej.data.js
    * @requires ej.dropdownlist.js
    * @requires ej.button.js
    * @requires ej.tab.js
    * @requires ej.splitbutton.js
    * @requires ej.ribbon.js
    * @example 
   * @example 
       *&lt;ul id="menu"&gt;
	   *&lt;li&gt;&lt;a&gt;FILE &lt;/a&gt;
       *&lt;ul&gt;
       *&lt;li&gt;&lt;a&gt;New&lt;/a&gt;&lt;/li&gt;
       *&lt;li&gt;&lt;a&gt;Open&lt;/a&gt;&lt;/li&gt;
       *&lt;li&gt;&lt;a&gt;Save&lt;/a&gt;&lt;/li&gt;
       *&lt;li&gt;&lt;a&gt;Save as&lt;/a&gt;&lt;/li&gt;
       *&lt;li&gt;&lt;a&gt;Print&lt;/a&gt;&lt;/li&gt;
	   *&lt;/ul&gt;&lt;/li&gt;&lt;/ul&gt;
       *&lt;div id="Ribbon"&gt;&lt;/div&gt; 
	   *&lt;button id="btn"&gt;Home button&lt;/button&gt;
       *&lt;script type="text/javascript"&gt;   
       *$(function () {
       *     $("#Ribbon").ejRibbon({
	   * // Set the width during initialization. 	
       *      width: "100%",
       *        applicationTab: { Type: "ApplicationMenu", itemID: "menu", menuSettings: { openOnClick: false } },
       *        tabs: [{
       *            id: "home", text: "HOME", groups: [{
       *                 text: "New", alignType: ej.Ribbon.alignType.rows,type:"custom",contentID:"btn"
       *            }]
       *         }]
       *   });
       * });	
       * $('#Ribbon').ejRibbon("ejRibbon");        	   
       * &lt;/script&gt;  
       * @memberof ejRibbon
       * @instance
       */
    ej.widget("ejRibbon", "ej.Ribbon", {
        element: null,
        model: null,
        _rootCSS: "e-ribbon",
        defaults: {
		/**		
        * Specify the width to the ribbon control.we can set width in string or number format. 
        * @default null
		* @alias ejRibbon#width
        * @type { string | number}
        * @example 
       *&lt;ul id="menu"&gt;
	   *&lt;li&gt;&lt;a&gt;FILE &lt;/a&gt;
       *&lt;ul&gt;
       *&lt;li&gt;&lt;a&gt;New&lt;/a&gt;&lt;/li&gt;
       *&lt;li&gt;&lt;a&gt;Open&lt;/a&gt;&lt;/li&gt;
       *&lt;li&gt;&lt;a&gt;Save&lt;/a&gt;&lt;/li&gt;
       *&lt;li&gt;&lt;a&gt;Save as&lt;/a&gt;&lt;/li&gt;
       *&lt;li&gt;&lt;a&gt;Print&lt;/a&gt;&lt;/li&gt;
	   *&lt;/ul&gt;&lt;/li&gt;&lt;/ul&gt;
       *&lt;div id="Ribbon"&gt;&lt;/div&gt; 
	   *&lt;button id="btn"&gt;Home button&lt;/button&gt;
       *&lt;script type="text/javascript"&gt;   
       *$(function () {
       *     $("#Ribbon").ejRibbon({
	   * // Set the width during initialization. 	
       *      width: "100%",
       *        applicationTab: { Type: "ApplicationMenu", itemID: "menu", menuSettings: { openOnClick: false } },
       *        tabs: [{
       *            id: "home", text: "HOME", groups: [{
       *                 text: "New", alignType: ej.Ribbon.alignType.rows,type:"custom",contentID:"btn"
       *            }]
       *         }]
       *   });
       * });	        
       * &lt;/script&gt;  
       * @memberof ejRibbon
       * @instance
       */
            width: "",
			/**		
        * Specify the application tab to contain application menu in the ribbon control.
        * @default Object
		* @alias ejRibbon#applicationTab
        * @type {object}
        * @example 
        *&lt;ul id="menu"&gt;
	   *&lt;li&gt;&lt;a&gt;FILE &lt;/a&gt;
       *&lt;ul&gt;
       *&lt;li&gt;&lt;a&gt;New&lt;/a&gt;&lt;/li&gt;
       *&lt;li&gt;&lt;a&gt;Open&lt;/a&gt;&lt;/li&gt;
       *&lt;li&gt;&lt;a&gt;Save&lt;/a&gt;&lt;/li&gt;
       *&lt;li&gt;&lt;a&gt;Save as&lt;/a&gt;&lt;/li&gt;
       *&lt;li&gt;&lt;a&gt;Print&lt;/a&gt;&lt;/li&gt;
	   *&lt;/ul&gt;&lt;/li&gt;&lt;/ul&gt;
       *&lt;div id="Ribbon"&gt;&lt;/div&gt; 
	   *&lt;button id="btn"&gt;Home button&lt;/button&gt;
       *&lt;script type="text/javascript"&gt;   
       *$(function () {
       *     $("#Ribbon").ejRibbon({
	   * // Set the width during initialization. 	
       *      width: "100%",
       *        applicationTab: { Type: "ApplicationMenu", itemID: "menu", menuSettings: { openOnClick: false } },
       *        tabs: [{
       *            id: "home", text: "HOME", groups: [{
       *                 text: "New", alignType: ej.Ribbon.alignType.rows,type:"custom",contentID:"btn"
       *            }]
       *         }]
       *   });
       * });	        
       * &lt;/script&gt;  
       * @memberof ejRibbon
       * @instance
       */
            applicationTab: {
			/**		
        * To specify the type of application tab item.Specify the type of application tab as "ApplicationMenu" in the ribbon control.
        * @default null
		* @alias ejRibbon#applicationTab->Type
        * @type {string}
        * @example 
         *&lt;ul id="menu"&gt;
	   *&lt;li&gt;&lt;a&gt;FILE &lt;/a&gt;
       *&lt;ul&gt;
       *&lt;li&gt;&lt;a&gt;New&lt;/a&gt;&lt;/li&gt;
       *&lt;li&gt;&lt;a&gt;Open&lt;/a&gt;&lt;/li&gt;
       *&lt;li&gt;&lt;a&gt;Save&lt;/a&gt;&lt;/li&gt;
       *&lt;li&gt;&lt;a&gt;Save as&lt;/a&gt;&lt;/li&gt;
       *&lt;li&gt;&lt;a&gt;Print&lt;/a&gt;&lt;/li&gt;
	   *&lt;/ul&gt;&lt;/li&gt;&lt;/ul&gt;
       *&lt;div id="Ribbon"&gt;&lt;/div&gt; 
	   *&lt;button id="btn"&gt;Home button&lt;/button&gt;
       *&lt;script type="text/javascript"&gt;   
       *$(function () {
       *     $("#Ribbon").ejRibbon({
	   * // Set the width during initialization. 	
       *      width: "100%",
       *        applicationTab: { Type: "ApplicationMenu", itemID: "menu", menuSettings: { openOnClick: false } },
       *        tabs: [{
       *            id: "home", text: "HOME", groups: [{
       *                 text: "New", alignType: ej.Ribbon.alignType.rows,type:"custom",contentID:"btn"
       *            }]
       *         }]
       *   });
       * });	        
       * &lt;/script&gt;  
       * @memberof ejRibbon
       * @instance
       */
			        Type:"",
					/**		
        * Specify the ID of 'ul' list to create application menu in the ribbon control.
        * @default null
		* @alias ejRibbon#applicationTab->itemID
        * @type {string}
        * @example 
         *&lt;ul id="menu"&gt;
	   *&lt;li&gt;&lt;a&gt;FILE &lt;/a&gt;
       *&lt;ul&gt;
       *&lt;li&gt;&lt;a&gt;New&lt;/a&gt;&lt;/li&gt;
       *&lt;li&gt;&lt;a&gt;Open&lt;/a&gt;&lt;/li&gt;
       *&lt;li&gt;&lt;a&gt;Save&lt;/a&gt;&lt;/li&gt;
       *&lt;li&gt;&lt;a&gt;Save as&lt;/a&gt;&lt;/li&gt;
       *&lt;li&gt;&lt;a&gt;Print&lt;/a&gt;&lt;/li&gt;
	   *&lt;/ul&gt;&lt;/li&gt;&lt;/ul&gt;
       *&lt;div id="Ribbon"&gt;&lt;/div&gt; 
	   *&lt;button id="btn"&gt;Home button &lt;/button&gt;
       *&lt;script type="text/javascript"&gt;   
       *$(function () {
       *     $("#Ribbon").ejRibbon({
	   * // Set the width during initialization. 	
       *      width: "100%",
       *        applicationTab: { Type: "ApplicationMenu", itemID: "menu", menuSettings: { openOnClick: false } },
       *        tabs: [{
       *            id: "home", text: "HOME", groups: [{
       *                 text: "New", alignType: ej.Ribbon.alignType.rows,type:"custom",contentID:"btn"
       *            }]
       *         }]
       *   });
       * });	        
       * &lt;/script&gt;  
       * @memberof ejRibbon
       * @instance
       */
					itemID: "",
					/**		
        * Specify the menu members,events using the menu settings for the menu in the application tab.
        * @default object
		* @alias ejRibbon#applicationTab->menuSettings
        * @type {object}
        * @example 
                 *&lt;ul id="menu"&gt;
	   *&lt;li&gt;&lt;a&gt;FILE &lt;/a&gt;
       *&lt;ul&gt;
       *&lt;li&gt;&lt;a&gt;New&lt;/a&gt;&lt;/li&gt;
       *&lt;li&gt;&lt;a&gt;Open&lt;/a&gt;&lt;/li&gt;
       *&lt;li&gt;&lt;a&gt;Save&lt;/a&gt;&lt;/li&gt;
       *&lt;li&gt;&lt;a&gt;Save as&lt;/a&gt;&lt;/li&gt;
       *&lt;li&gt;&lt;a&gt;Print&lt;/a&gt;&lt;/li&gt;
	   *&lt;/ul&gt;&lt;/li&gt;&lt;/ul&gt;
       *&lt;div id="Ribbon"&gt;&lt;/div&gt; 
	   *&lt;button id="btn"&gt;Home button&lt;/button&gt;
       *&lt;script type="text/javascript"&gt;   
       *$(function () {
       *     $("#Ribbon").ejRibbon({
	   * // Set the width during initialization. 	
       *      width: "100%",
       *        applicationTab: { Type: "ApplicationMenu", itemID: "menu", menuSettings: { openOnClick: false } },
       *        tabs: [{
       *            id: "home", text: "HOME", groups: [{
       *                 text: "New", alignType: ej.Ribbon.alignType.rows,type:"custom",contentID:"btn"
       *            }]
       *         }]
       *   });
       * });	        
       * &lt;/script&gt;  
       * @memberof ejRibbon
       * @instance
       */
					menuSettings:{}
			},
			/**		
        * Specify the tabs and its groups,controls to the ribbon control.
        * @default array
		* @alias ejRibbon#tabs
        * @type {array}
        * @example 
                 *&lt;ul id="menu"&gt;
	   *&lt;li&gt;&lt;a&gt;FILE &lt;/a&gt;
       *&lt;ul&gt;
       *&lt;li&gt;&lt;a&gt;New&lt;/a&gt;&lt;/li&gt;
       *&lt;li&gt;&lt;a&gt;Open&lt;/a&gt;&lt;/li&gt;
       *&lt;li&gt;&lt;a&gt;Save&lt;/a&gt;&lt;/li&gt;
       *&lt;li&gt;&lt;a&gt;Save as&lt;/a&gt;&lt;/li&gt;
       *&lt;li&gt;&lt;a&gt;Print&lt;/a&gt;&lt;/li&gt;
	   *&lt;/ul&gt;&lt;/li&gt;&lt;/ul&gt;
       *&lt;div id="Ribbon"&gt;&lt;/div&gt; 
	   *&lt;button id="btn"&gt;Home button&lt;/button&gt;
	   *&lt;button id="insert"&gt;Insert button&lt;/button&gt;
       *&lt;script type="text/javascript"&gt;   
       *$(function () {
       *     $("#Ribbon").ejRibbon({
	   * // Set the width during initialization. 	
       *      width: "100%",
       *        applicationTab: { Type: "ApplicationMenu", itemID: "menu", menuSettings: { openOnClick: false } },
       *        tabs: [{
       *            id: "home", text: "HOME", groups: [{
       *                 text: "New", alignType: ej.Ribbon.alignType.rows,type:"custom",contentID:"btn"
       *            }]
       *         },
	            {
				id: "insert", text: "INSERT", groups: [{
       *                 text: "Insert group", alignType: ej.Ribbon.alignType.rows,type:"custom",contentID:"insert"
       *            }]
	            }
	   ]
       *   });
       * });	        
       * &lt;/script&gt;  
       * @memberof ejRibbon
       * @instance
       */
            tabs: [{
			/**		
        * Specify the ID for each tab's content panel.
        * @default null
		* @alias ejRibbon#tabs->id
        * @type {string}
        * @example 
       *&lt;ul id="menu"&gt;
	   *&lt;li&gt;&lt;a&gt;FILE &lt;/a&gt;
       *&lt;ul&gt;
       *&lt;li&gt;&lt;a&gt;New&lt;/a&gt;&lt;/li&gt;
       *&lt;li&gt;&lt;a&gt;Open&lt;/a&gt;&lt;/li&gt;
       *&lt;li&gt;&lt;a&gt;Save&lt;/a&gt;&lt;/li&gt;
       *&lt;li&gt;&lt;a&gt;Save as&lt;/a&gt;&lt;/li&gt;
       *&lt;li&gt;&lt;a&gt;Print&lt;/a&gt;&lt;/li&gt;
	   *&lt;/ul&gt;&lt;/li&gt;&lt;/ul&gt;
       *&lt;div id="Ribbon"&gt;&lt;/div&gt; 
	   *&lt;button id="btn"&gt;Home button&lt;/button&gt;
       *&lt;script type="text/javascript"&gt;   
       *$(function () {
       *     $("#Ribbon").ejRibbon({
	   * // Set the width during initialization. 	
       *      width: "100%",
       *        applicationTab: { Type: "ApplicationMenu", itemID: "menu", menuSettings: { openOnClick: false } },
       *        tabs: [{
       *            id: "home", text: "HOME", groups: [{
       *                 text: "New", alignType: ej.Ribbon.alignType.rows,type:"custom",contentID:"btn"
       *            }]
       *         }]
       *   });
       * });	        
       * &lt;/script&gt;  
       * @memberof ejRibbon
       * @instance
       */
			   id:null,
			   /**		
        * Specify the text of the tab in the ribbon control.
        * @default null
		* @alias ejRibbon#tabs->text
        * @type {string}
        * @example 
        *&lt;ul id="menu"&gt;
	   *&lt;li&gt;&lt;a&gt;FILE &lt;/a&gt;
       *&lt;ul&gt;
       *&lt;li&gt;&lt;a&gt;New&lt;/a&gt;&lt;/li&gt;
       *&lt;li&gt;&lt;a&gt;Open&lt;/a&gt;&lt;/li&gt;
       *&lt;li&gt;&lt;a&gt;Save&lt;/a&gt;&lt;/li&gt;
       *&lt;li&gt;&lt;a&gt;Save as&lt;/a&gt;&lt;/li&gt;
       *&lt;li&gt;&lt;a&gt;Print&lt;/a&gt;&lt;/li&gt;
	   *&lt;/ul&gt;&lt;/li&gt;&lt;/ul&gt;
       *&lt;div id="Ribbon"&gt;&lt;/div&gt; 
	   *&lt;button id="btn"&gt;Home button&lt;/button&gt;
       *&lt;script type="text/javascript"&gt;   
       *$(function () {
       *     $("#Ribbon").ejRibbon({
	   * // Set the width during initialization. 	
       *      width: "100%",
       *        applicationTab: { Type: "ApplicationMenu", itemID: "menu", menuSettings: { openOnClick: false } },
       *        tabs: [{
       *            id: "home", text: "HOME", groups: [{
       *                 text: "New", alignType: ej.Ribbon.alignType.rows,type:"custom",contentID:"btn"
       *            }]
       *         }]
       *   });
       * });	        
       * &lt;/script&gt;  
       * @memberof ejRibbon
       * @instance
       */
			   text:null,
			   /**		
        * Specify single group or multiple groups and its contents to each tab in the ribbon control.
        * @default array
		* @alias ejRibbon#tabs->groups
        * @type {array}
        * @example 
        *&lt;ul id="menu"&gt;
	   *&lt;li&gt;&lt;a&gt;FILE &lt;/a&gt;
       *&lt;ul&gt;
       *&lt;li&gt;&lt;a&gt;New&lt;/a&gt;&lt;/li&gt;
       *&lt;li&gt;&lt;a&gt;Open&lt;/a&gt;&lt;/li&gt;
       *&lt;li&gt;&lt;a&gt;Save&lt;/a&gt;&lt;/li&gt;
       *&lt;li&gt;&lt;a&gt;Save as&lt;/a&gt;&lt;/li&gt;
       *&lt;li&gt;&lt;a&gt;Print&lt;/a&gt;&lt;/li&gt;
	   *&lt;/ul&gt;&lt;/li&gt;&lt;/ul&gt;
       *&lt;div id="Ribbon"&gt;&lt;/div&gt; 
	   *&lt;button id="btn"&gt;Home button&lt;/button&gt;
       *&lt;script type="text/javascript"&gt;   
       *$(function () {
       *     $("#Ribbon").ejRibbon({
	   * // Set the width during initialization. 	
       *      width: "100%",
       *        applicationTab: { Type: "ApplicationMenu", itemID: "menu", menuSettings: { openOnClick: false } },
       *        tabs: [{
       *            id: "home", text: "HOME", groups: [{
       *                 text: "New", alignType: ej.Ribbon.alignType.rows,type:"custom",contentID:"btn"
       *            }]
       *         }]
       *   });
       * });	        
       * &lt;/script&gt;  
       * @memberof ejRibbon
       * @instance
       */
			   groups:[{
			     /**		
        * Specify the text to the groups in the ribbon control.
        * @default null
		* @alias ejRibbon#tabs->groups->text
        * @type {string}
        * @example 
        *&lt;ul id="menu"&gt;
	   *&lt;li&gt;&lt;a&gt;FILE &lt;/a&gt;
       *&lt;ul&gt;
       *&lt;li&gt;&lt;a&gt;New&lt;/a&gt;&lt;/li&gt;
       *&lt;li&gt;&lt;a&gt;Open&lt;/a&gt;&lt;/li&gt;
       *&lt;li&gt;&lt;a&gt;Save&lt;/a&gt;&lt;/li&gt;
       *&lt;li&gt;&lt;a&gt;Save as&lt;/a&gt;&lt;/li&gt;
       *&lt;li&gt;&lt;a&gt;Print&lt;/a&gt;&lt;/li&gt;
	   *&lt;/ul&gt;&lt;/li&gt;&lt;/ul&gt;
       *&lt;div id="Ribbon"&gt;&lt;/div&gt; 
	   *&lt;button id="btn"&gt;Home button&lt;/button&gt;
       *&lt;script type="text/javascript"&gt;   
       *$(function () {
       *     $("#Ribbon").ejRibbon({
	   * // Set the width during initialization. 	
       *      width: "100%",
       *        applicationTab: { Type: "ApplicationMenu", itemID: "menu", menuSettings: { openOnClick: false } },
       *        tabs: [{
       *            id: "home", text: "HOME", groups: [{
       *                 text: "New", alignType: ej.Ribbon.alignType.rows,type:"custom",contentID:"btn"
       *            }]
       *         }]
       *   });
       * });	        
       * &lt;/script&gt;  
       * @memberof ejRibbon
       * @instance
       */
			   text:null,
			    /**		
        * Specify the custom items such as div,table,controls using the type "custom".
        * @default null
		* @alias ejRibbon#tabs->groups->type
        * @type {string}
        * @example 
		*&lt;ul id="menu"&gt;
	   *&lt;li&gt;&lt;a&gt;FILE &lt;/a&gt;
       *&lt;ul&gt;
       *&lt;li&gt;&lt;a&gt;New&lt;/a&gt;&lt;/li&gt;
       *&lt;li&gt;&lt;a&gt;Open&lt;/a&gt;&lt;/li&gt;
       *&lt;li&gt;&lt;a&gt;Save&lt;/a&gt;&lt;/li&gt;
       *&lt;li&gt;&lt;a&gt;Save as&lt;/a&gt;&lt;/li&gt;
       *&lt;li&gt;&lt;a&gt;Print&lt;/a&gt;&lt;/li&gt;
	   *&lt;/ul&gt;&lt;/li&gt;&lt;/ul&gt;
       *&lt;div id="Ribbon"&gt;&lt;/div&gt; 
	   *&lt;button id="btn"&gt;Home button&lt;/button&gt;
       *&lt;script type="text/javascript"&gt;   
       *$(function () {
       *     $("#Ribbon").ejRibbon({
	   * // Set the width during initialization. 	
       *      width: "100%",
       *        applicationTab: { Type: "ApplicationMenu", itemID: "menu", menuSettings: { openOnClick: false } },
       *        tabs: [{
       *            id: "home", text: "HOME", groups: [{
       *                 text: "New", alignType: ej.Ribbon.alignType.rows,type:"custom",contentID:"btn"
       *            }]
       *         }]
       *   });
       * });	        
       * &lt;/script&gt;  
       * @memberof ejRibbon
       * @instance
       */
			   type:null,
			    /**		
        * Specify the ID of custom items to place into the groups.
        * @default null
		* @alias ejRibbon#tabs->groups->contentID
        * @type {string}
        * @example 
		*&lt;ul id="menu"&gt;
	   *&lt;li&gt;&lt;a&gt;FILE &lt;/a&gt;
       *&lt;ul&gt;
       *&lt;li&gt;&lt;a&gt;New&lt;/a&gt;&lt;/li&gt;
       *&lt;li&gt;&lt;a&gt;Open&lt;/a&gt;&lt;/li&gt;
       *&lt;li&gt;&lt;a&gt;Save&lt;/a&gt;&lt;/li&gt;
       *&lt;li&gt;&lt;a&gt;Save as&lt;/a&gt;&lt;/li&gt;
       *&lt;li&gt;&lt;a&gt;Print&lt;/a&gt;&lt;/li&gt;
	   *&lt;/ul&gt;&lt;/li&gt;&lt;/ul&gt;
       *&lt;div id="Ribbon"&gt;&lt;/div&gt; 
	   *&lt;button id="btn"&gt;Home button&lt;/button&gt;
       *&lt;script type="text/javascript"&gt;   
       *$(function () {
       *     $("#Ribbon").ejRibbon({
	   * // Set the width during initialization. 	
       *      width: "100%",
       *        applicationTab: { Type: "ApplicationMenu", itemID: "menu", menuSettings: { openOnClick: false } },
       *        tabs: [{
       *            id: "home", text: "HOME", groups: [{
       *                 text: "New", alignType: ej.Ribbon.alignType.rows,type:"custom",contentID:"btn"
       *            }]
       *         }]
       *   });
       * });	        
       * &lt;/script&gt;  
       * @memberof ejRibbon
       * @instance
       */
			   contentID:null,
			   /**		
        * Specify the alignment of controls in the groups in 'row' type or 'column' type.Value for row type is "ej.Ribbon.alignType.rows" and Value for column type is "ej.Ribbon.alignType.columns"
        * @default null
		* @alias ejRibbon#tabs->groups->alignType
        * @type {enum}
        * @example 
        *&lt;ul id="menu"&gt;
	   *&lt;li&gt;&lt;a&gt;FILE &lt;/a&gt;
       *&lt;ul&gt;
       *&lt;li&gt;&lt;a&gt;New&lt;/a&gt;&lt;/li&gt;
       *&lt;li&gt;&lt;a&gt;Open&lt;/a&gt;&lt;/li&gt;
       *&lt;li&gt;&lt;a&gt;Save&lt;/a&gt;&lt;/li&gt;
       *&lt;li&gt;&lt;a&gt;Save as&lt;/a&gt;&lt;/li&gt;
       *&lt;li&gt;&lt;a&gt;Print&lt;/a&gt;&lt;/li&gt;
	   *&lt;/ul&gt;&lt;/li&gt;&lt;/ul&gt;
       *&lt;div id="Ribbon"&gt;&lt;/div&gt; 
	   *&lt;button id="btn"&gt;Home button&lt;/button&gt;
       *&lt;script type="text/javascript"&gt;   
       *$(function () {
       *     $("#Ribbon").ejRibbon({
	   * // Set the width during initialization. 	
       *      width: "100%",
       *        applicationTab: { Type: "ApplicationMenu", itemID: "menu", menuSettings: { openOnClick: false } },
       *        tabs: [{
       *            id: "home", text: "HOME", groups: [{
       *                 text: "New", alignType: ej.Ribbon.alignType.rows,type:"custom",contentID:"btn"
       *            }]
       *         }]
       *   });
       * });	        
       * &lt;/script&gt;  
       * @memberof ejRibbon
       * @instance
       */
			   alignType:null,
			     /**		
        * Specify the group expander for groups in the ribbon control.Set "true" to enable the group expander.
        * @default false
		* @alias ejRibbon#tabs->groups->enableGroupExpander
        * @type {boolean}
        * @example 
        *&lt;ul id="menu"&gt;
	   *&lt;li&gt;&lt;a&gt;FILE &lt;/a&gt;
       *&lt;ul&gt;
       *&lt;li&gt;&lt;a&gt;New&lt;/a&gt;&lt;/li&gt;
       *&lt;li&gt;&lt;a&gt;Open&lt;/a&gt;&lt;/li&gt;
       *&lt;li&gt;&lt;a&gt;Save&lt;/a&gt;&lt;/li&gt;
       *&lt;li&gt;&lt;a&gt;Save as&lt;/a&gt;&lt;/li&gt;
       *&lt;li&gt;&lt;a&gt;Print&lt;/a&gt;&lt;/li&gt;
	   *&lt;/ul&gt;&lt;/li&gt;&lt;/ul&gt;
       *&lt;div id="Ribbon"&gt;&lt;/div&gt; 
	   *&lt;button id="btn"&gt;Home button&lt;/button&gt;
       *&lt;script type="text/javascript"&gt;   
       *$(function () {
       *     $("#Ribbon").ejRibbon({
	   * // Set the width during initialization. 	
       *      width: "100%",
       *        applicationTab: { Type: "ApplicationMenu", itemID: "menu", menuSettings: { openOnClick: false } },
       *        tabs: [{
       *            id: "home", text: "HOME", groups: [{
       *                 text: "New", alignType: ej.Ribbon.alignType.rows,type:"custom",enableGroupExpander: true,contentID:"btn"
       *            }]
       *         }]
       *   });
       * });	        
       * &lt;/script&gt;  
       * @memberof ejRibbon
       * @instance
       */
			   enableGroupExpander:false,
			    /**		
        * Specify the syncfusion button,split button,dropdown list controls to the groups in the ribbon control.
        * @default array
		* @alias ejRibbon#tabs->groups->content
        * @type {array}
        * @example 
       *&lt;ul id="menu"&gt;
	   *&lt;li&gt;&lt;a&gt;FILE &lt;/a&gt;
       *&lt;ul&gt;
       *&lt;li&gt;&lt;a&gt;New&lt;/a&gt;&lt;/li&gt;
       *&lt;li&gt;&lt;a&gt;Open&lt;/a&gt;&lt;/li&gt;
       *&lt;li&gt;&lt;a&gt;Save&lt;/a&gt;&lt;/li&gt;
       *&lt;li&gt;&lt;a&gt;Save as&lt;/a&gt;&lt;/li&gt;
       *&lt;li&gt;&lt;a&gt;Print&lt;/a&gt;&lt;/li&gt;
	   *&lt;/ul&gt;&lt;/li&gt;&lt;/ul&gt;
       *&lt;div id="Ribbon"&gt;&lt;/div&gt; 
       *&lt;script type="text/javascript"&gt;   
       *$(function () {
       *     $("#Ribbon").ejRibbon({
	   * // Set the width during initialization. 	
       *      width: "100%",
       *        applicationTab: { Type: "ApplicationMenu", itemID: "menu", menuSettings: { openOnClick: false } },
       *        tabs: [{
       *            id: "home", text: "HOME", groups: [{
       *                 text: "New", alignType: ej.Ribbon.alignType.rows,content: [{
       *                     groups: [{
       *                         id: "new",
       *                         text: "New",
	   *                         toolTip: "New",
       *                         type: ej.Ribbon.type.button,
       *                         buttonSettings: {
       *                         width: 160,
       *                         height: 50
       *                         }
       *                     }]
       *                 }]
       *            }]
       *         }]
       *   });
       * });	        
       * &lt;/script&gt;  
       * @memberof ejRibbon
       * @instance
       */
			   content:[{
			   /**		
        * Specify the controls such as syncfusion button,split button,dropdown list controls in the subgroup of the ribbon tab .
        * @default array
		* @alias ejRibbon#tabs->groups->content->groups
        * @type {array}
        * @example 
			   *&lt;ul id="menu"&gt;
	   *&lt;li&gt;&lt;a&gt;FILE &lt;/a&gt;
       *&lt;ul&gt;
       *&lt;li&gt;&lt;a&gt;New&lt;/a&gt;&lt;/li&gt;
       *&lt;li&gt;&lt;a&gt;Open&lt;/a&gt;&lt;/li&gt;
       *&lt;li&gt;&lt;a&gt;Save&lt;/a&gt;&lt;/li&gt;
       *&lt;li&gt;&lt;a&gt;Save as&lt;/a&gt;&lt;/li&gt;
       *&lt;li&gt;&lt;a&gt;Print&lt;/a&gt;&lt;/li&gt;
	   *&lt;/ul&gt;&lt;/li&gt;&lt;/ul&gt;
       *&lt;div id="Ribbon"&gt;&lt;/div&gt; 
       *&lt;script type="text/javascript"&gt;   
       *$(function () {
       *     $("#Ribbon").ejRibbon({
	   * // Set the width during initialization. 	
       *      width: "100%",
       *        applicationTab: { Type: "ApplicationMenu", itemID: "menu", menuSettings: { openOnClick: false } },
       *        tabs: [{
       *            id: "home", text: "HOME", groups: [{
       *                 text: "New", alignType: ej.Ribbon.alignType.rows,content: [{
       *                     groups: [{
       *                         id: "new",
       *                         text: "New",
	   *                         toolTip: "New",
       *                         type: ej.Ribbon.type.button,
       *                         buttonSettings: {
       *                         width: 160,
       *                         height: 50
       *                         }
       *                     }]
       *                 }]
       *            }]
       *         }]
       *   });
       * });	        
       * &lt;/script&gt;  
       * @memberof ejRibbon
       * @instance
       */
			   groups:[{
			    /**		
        * Specify the Id for button,split button,dropdown list controls in the sub groups.
        * @default null
		* @alias ejRibbon#tabs->groups->content->groups->id
        * @type {string}
        * @example 
        *&lt;ul id="menu"&gt;
	   *&lt;li&gt;&lt;a&gt;FILE &lt;/a&gt;
       *&lt;ul&gt;
       *&lt;li&gt;&lt;a&gt;New&lt;/a&gt;&lt;/li&gt;
       *&lt;li&gt;&lt;a&gt;Open&lt;/a&gt;&lt;/li&gt;
       *&lt;li&gt;&lt;a&gt;Save&lt;/a&gt;&lt;/li&gt;
       *&lt;li&gt;&lt;a&gt;Save as&lt;/a&gt;&lt;/li&gt;
       *&lt;li&gt;&lt;a&gt;Print&lt;/a&gt;&lt;/li&gt;
	   *&lt;/ul&gt;&lt;/li&gt;&lt;/ul&gt;
       *&lt;div id="Ribbon"&gt;&lt;/div&gt; 
       *&lt;script type="text/javascript"&gt;   
       *$(function () {
       *     $("#Ribbon").ejRibbon({
	   * // Set the width during initialization. 	
       *      width: "100%",
       *        applicationTab: { Type: "ApplicationMenu", itemID: "menu", menuSettings: { openOnClick: false } },
       *        tabs: [{
       *            id: "home", text: "HOME", groups: [{
       *                 text: "New", alignType: ej.Ribbon.alignType.rows,content: [{
       *                     groups: [{
       *                         id: "new",
       *                         text: "New",
	   *                         toolTip: "New",
       *                         type: ej.Ribbon.type.button,
       *                         buttonSettings: {
       *                         width: 160,
       *                         height: 50
       *                         }
       *                     }]
       *                 }]
       *            }]
       *         }]
       *   });
       * });	        
       * &lt;/script&gt;  
       * @memberof ejRibbon
       * @instance
       */
	           id:null,
			   /**		
        * Specify the text for button,split button controls in the sub groups.
        * @default null
		* @alias ejRibbon#tabs->groups->content->groups->text
        * @type {string}
        * @example 
       *&lt;ul id="menu"&gt;
	   *&lt;li&gt;&lt;a&gt;FILE &lt;/a&gt;
       *&lt;ul&gt;
       *&lt;li&gt;&lt;a&gt;New&lt;/a&gt;&lt;/li&gt;
       *&lt;li&gt;&lt;a&gt;Open&lt;/a&gt;&lt;/li&gt;
       *&lt;li&gt;&lt;a&gt;Save&lt;/a&gt;&lt;/li&gt;
       *&lt;li&gt;&lt;a&gt;Save as&lt;/a&gt;&lt;/li&gt;
       *&lt;li&gt;&lt;a&gt;Print&lt;/a&gt;&lt;/li&gt;
	   *&lt;/ul&gt;&lt;/li&gt;&lt;/ul&gt;
       *&lt;div id="Ribbon"&gt;&lt;/div&gt; 
       *&lt;script type="text/javascript"&gt;   
       *$(function () {
       *     $("#Ribbon").ejRibbon({
	   * // Set the width during initialization. 	
       *      width: "100%",
       *        applicationTab: { Type: "ApplicationMenu", itemID: "menu", menuSettings: { openOnClick: false } },
       *        tabs: [{
       *            id: "home", text: "HOME", groups: [{
       *                 text: "New", alignType: ej.Ribbon.alignType.rows,content: [{
       *                     groups: [{
       *                         id: "new",
       *                         text: "New",
	   *                         toolTip: "New",
       *                         type: ej.Ribbon.type.button,
       *                         buttonSettings: {
       *                         width: 160,
       *                         height: 50
       *                         }
       *                     }]
       *                 }]
       *            }]
       *         }]
       *   });
       * });	        
       * &lt;/script&gt;  
       * @memberof ejRibbon
       * @instance
       */
			   text:null,
			    /**		
        * Specify the tooltip for button,split,dropdown controls in the sub groups.
        * @default null
		* @alias ejRibbon#tabs->groups->content->groups->toolTip
        * @type {string}
        * @example 
        *&lt;ul id="menu"&gt;
	   *&lt;li&gt;&lt;a&gt;FILE &lt;/a&gt;
       *&lt;ul&gt;
       *&lt;li&gt;&lt;a&gt;New&lt;/a&gt;&lt;/li&gt;
       *&lt;li&gt;&lt;a&gt;Open&lt;/a&gt;&lt;/li&gt;
       *&lt;li&gt;&lt;a&gt;Save&lt;/a&gt;&lt;/li&gt;
       *&lt;li&gt;&lt;a&gt;Save as&lt;/a&gt;&lt;/li&gt;
       *&lt;li&gt;&lt;a&gt;Print&lt;/a&gt;&lt;/li&gt;
	   *&lt;/ul&gt;&lt;/li&gt;&lt;/ul&gt;
       *&lt;div id="Ribbon"&gt;&lt;/div&gt; 
       *&lt;script type="text/javascript"&gt;   
       *$(function () {
       *     $("#Ribbon").ejRibbon({
	   * // Set the width during initialization. 	
       *      width: "100%",
       *        applicationTab: { Type: "ApplicationMenu", itemID: "menu", menuSettings: { openOnClick: false } },
       *        tabs: [{
       *            id: "home", text: "HOME", groups: [{
       *                 text: "New", alignType: ej.Ribbon.alignType.rows,content: [{
       *                     groups: [{
       *                         id: "new",
       *                         text: "New",
	   *                         toolTip: "New",
       *                         type: ej.Ribbon.type.button,
       *                         buttonSettings: {
       *                         width: 160,
       *                         height: 50
       *                         }
       *                     }]
       *                 }]
       *            }]
       *         }]
       *   });
       * });	        
       * &lt;/script&gt;  
       * @memberof ejRibbon
       * @instance
       */
			   toolTip:null,
			    /**		
        * Specify the type as "ej.Ribbon.type.button" or "ej.Ribbon.type.splitButton" or "ej.Ribbon.type.dropDownList" to render button,split,dropdown controls.
        * @default null
		* @alias ejRibbon#tabs->groups->content->groups->type
        * @type {enum}
        * @example 
        *&lt;ul id="menu"&gt;
	   *&lt;li&gt;&lt;a&gt;FILE &lt;/a&gt;
       *&lt;ul&gt;
       *&lt;li&gt;&lt;a&gt;New&lt;/a&gt;&lt;/li&gt;
       *&lt;li&gt;&lt;a&gt;Open&lt;/a&gt;&lt;/li&gt;
       *&lt;li&gt;&lt;a&gt;Save&lt;/a&gt;&lt;/li&gt;
       *&lt;li&gt;&lt;a&gt;Save as&lt;/a&gt;&lt;/li&gt;
       *&lt;li&gt;&lt;a&gt;Print&lt;/a&gt;&lt;/li&gt;
	   *&lt;/ul&gt;&lt;/li&gt;&lt;/ul&gt;
       *&lt;div id="Ribbon"&gt;&lt;/div&gt; 
       *&lt;script type="text/javascript"&gt;   
       *$(function () {
       *     $("#Ribbon").ejRibbon({
	   * // Set the width during initialization. 	
       *      width: "100%",
       *        applicationTab: { Type: "ApplicationMenu", itemID: "menu", menuSettings: { openOnClick: false } },
       *        tabs: [{
       *            id: "home", text: "HOME", groups: [{
       *                 text: "New", alignType: ej.Ribbon.alignType.rows,content: [{
       *                     groups: [{
       *                         id: "new",
       *                         text: "New",
	   *                         toolTip: "New",
       *                         type: ej.Ribbon.type.button,
       *                         buttonSettings: {
       *                         width: 160,
       *                         height: 50
       *                         }
       *                     }]
       *                 }]
       *            }]
       *         }]
       *   });
       * });	        
       * &lt;/script&gt;  
       * @memberof ejRibbon
       * @instance
       */
			   type:null,
			      /**		
        * Specify the seperator to the control which is in row type group.The seperator seperate the control from the next control in the group.Set "true" to enable the seperator. 
        * @default false
		* @alias ejRibbon#tabs->groups->content->groups->enableSeparator
        * @type {boolean}
        * @example 
        *&lt;ul id="menu"&gt;
	   *&lt;li&gt;&lt;a&gt;FILE &lt;/a&gt;
       *&lt;ul&gt;
       *&lt;li&gt;&lt;a&gt;New&lt;/a&gt;&lt;/li&gt;
       *&lt;li&gt;&lt;a&gt;Open&lt;/a&gt;&lt;/li&gt;
       *&lt;li&gt;&lt;a&gt;Save&lt;/a&gt;&lt;/li&gt;
       *&lt;li&gt;&lt;a&gt;Save as&lt;/a&gt;&lt;/li&gt;
       *&lt;li&gt;&lt;a&gt;Print&lt;/a&gt;&lt;/li&gt;
	   *&lt;/ul&gt;&lt;/li&gt;&lt;/ul&gt;
       *&lt;div id="Ribbon"&gt;&lt;/div&gt; 
       *&lt;script type="text/javascript"&gt;   
       *$(function () {
       *     $("#Ribbon").ejRibbon({
	   * // Set the width during initialization. 	
       *      width: "100%",
       *        applicationTab: { Type: "ApplicationMenu", itemID: "menu", menuSettings: { openOnClick: false } },
       *        tabs: [{
       *            id: "home", text: "HOME", groups: [{
       *                 text: "New", alignType: ej.Ribbon.alignType.rows,content: [{
       *                     groups: [{
       *                         id: "new",
       *                         text: "New",
	   *                         toolTip: "New",
       *                         enableSeparator: true,
       *                          buttonSettings: {
       *                          width: 100,
       *                          }
       *                     },
	   * 						{
       *                         id: "font",
       *                         text: "Font",
	   *                         toolTip: "Font",
       *                         buttonSettings: {
       *                         width: 150,
       *                         }
       *                     }],
	   *						 defaults: {
       *                        type: ej.Ribbon.type.button,
       *                        height: 70
       *                    }
       *                 }]
       *            }]
       *         }]
       *   });
       * });	        
       * &lt;/script&gt;  
       * @memberof ejRibbon
       * @instance
       */
			   enableSeparator:false,
			   /**		
        * Specify the size for button,split button controls.set "true" for big size and "false" for small size.
        * @default null
		* @alias ejRibbon#tabs->groups->content->groups->isBig
        * @type {boolean}
        * @example 
        *&lt;ul id="menu"&gt;
	   *&lt;li&gt;&lt;a&gt;FILE &lt;/a&gt;
       *&lt;ul&gt;
       *&lt;li&gt;&lt;a&gt;New&lt;/a&gt;&lt;/li&gt;
       *&lt;li&gt;&lt;a&gt;Open&lt;/a&gt;&lt;/li&gt;
       *&lt;li&gt;&lt;a&gt;Save&lt;/a&gt;&lt;/li&gt;
       *&lt;li&gt;&lt;a&gt;Save as&lt;/a&gt;&lt;/li&gt;
       *&lt;li&gt;&lt;a&gt;Print&lt;/a&gt;&lt;/li&gt;
	   *&lt;/ul&gt;&lt;/li&gt;&lt;/ul&gt;
       *&lt;div id="Ribbon"&gt;&lt;/div&gt; 
       *&lt;script type="text/javascript"&gt;   
       *$(function () {
       *     $("#Ribbon").ejRibbon({
	   * // Set the width during initialization. 	
       *      width: "100%",
       *        applicationTab: { Type: "ApplicationMenu", itemID: "menu", menuSettings: { openOnClick: false } },
       *        tabs: [{
       *            id: "home", text: "HOME", groups: [{
       *                 text: "New", alignType: ej.Ribbon.alignType.rows,content: [{
       *                     groups: [{
       *                         id: "new",
       *                         text: "New",
	   *                         toolTip: "New",
	   *                         isBig:true,
       *                         type: ej.Ribbon.type.button,
       *                         buttonSettings: {
       *                         width: 100
       *                         }
       *                     }]
       *                 }]
       *            }]
       *         }]
       *   });
       * });	        
       * &lt;/script&gt;  
       * @memberof ejRibbon
       * @instance
       */
			   isBig:null,
			     /**		
        * Specify the css class property to apply styles to the button,split,dropdown controls in the groups.
        * @default null
		* @alias ejRibbon#tabs->groups->content->groups->cssClass
        * @type {string}
        * @example 
        *&lt;ul id="menu"&gt;
	   *&lt;li&gt;&lt;a&gt;FILE &lt;/a&gt;
       *&lt;ul&gt;
       *&lt;li&gt;&lt;a&gt;New&lt;/a&gt;&lt;/li&gt;
       *&lt;li&gt;&lt;a&gt;Open&lt;/a&gt;&lt;/li&gt;
       *&lt;li&gt;&lt;a&gt;Save&lt;/a&gt;&lt;/li&gt;
       *&lt;li&gt;&lt;a&gt;Save as&lt;/a&gt;&lt;/li&gt;
       *&lt;li&gt;&lt;a&gt;Print&lt;/a&gt;&lt;/li&gt;
	   *&lt;/ul&gt;&lt;/li&gt;&lt;/ul&gt;
       *&lt;div id="Ribbon"&gt;&lt;/div&gt; 
	   *&lt;style&gt;
       *.e-cssclass {
       *      margin-left: 20px;
       *}
       *&lt;/style&gt;
       *&lt;script type="text/javascript"&gt;   
       *$(function () {
       *     $("#Ribbon").ejRibbon({
	   * // Set the width during initialization. 	
       *      width: "100%",
       *        applicationTab: { Type: "ApplicationMenu", itemID: "menu", menuSettings: { openOnClick: false } },
       *        tabs: [{
       *            id: "home", text: "HOME", groups: [{
       *                 text: "New", alignType: ej.Ribbon.alignType.rows,content: [{
       *                     groups: [{
       *                         id: "new",
       *                         text: "New",
	   *                         toolTip: "New",
	   *                         cssClass:"e-cssclass",
	   *                         isBig:true,
       *                         type: ej.Ribbon.type.button,
       *                         buttonSettings: {
       *                         width: 100,
       *                         }
       *                     }]
       *                 }]
       *            }]
       *         }]
       *   });
       * });	        
       * &lt;/script&gt;  
       * @memberof ejRibbon
       * @instance
       */
			   cssClass:null,
			   /**		
        * Specify the syncfusion button or split button members,events using this buttonSettings.
        * @default object
		* @alias ejRibbon#tabs->groups->content->groups->buttonSettings
        * @type {object}
        * @example 
        *&lt;ul id="menu"&gt;
	   *&lt;li&gt;&lt;a&gt;FILE &lt;/a&gt;
       *&lt;ul&gt;
       *&lt;li&gt;&lt;a&gt;New&lt;/a&gt;&lt;/li&gt;
       *&lt;li&gt;&lt;a&gt;Open&lt;/a&gt;&lt;/li&gt;
       *&lt;li&gt;&lt;a&gt;Save&lt;/a&gt;&lt;/li&gt;
       *&lt;li&gt;&lt;a&gt;Save as&lt;/a&gt;&lt;/li&gt;
       *&lt;li&gt;&lt;a&gt;Print&lt;/a&gt;&lt;/li&gt;
	   *&lt;/ul&gt;&lt;/li&gt;&lt;/ul&gt;
       *&lt;div id="Ribbon"&gt;&lt;/div&gt; 
       *&lt;script type="text/javascript"&gt;   
       *$(function () {
       *     $("#Ribbon").ejRibbon({
	   * // Set the width during initialization. 	
       *      width: "100%",
       *        applicationTab: { Type: "ApplicationMenu", itemID: "menu", menuSettings: { openOnClick: false } },
       *        tabs: [{
       *            id: "home", text: "HOME", groups: [{
       *                 text: "New", alignType: ej.Ribbon.alignType.rows,content: [{
       *                     groups: [{
       *                         id: "new",
       *                         text: "New",
	   *                         toolTip: "New",
	   *                         isBig:true,
       *                         type: ej.Ribbon.type.button,
       *                         buttonSettings: {
       *                         width: 100,
       *                         }
       *                     }]
       *                 }]
       *            }]
       *         }]
       *   });
       * });	        
       * &lt;/script&gt;  
       * @memberof ejRibbon
       * @instance
       */
			   buttonSettings:{},
			     /**		
        * Specify the syncfusion dropdown list members,events using this dropdownSettings.
        * @default object
		* @alias ejRibbon#tabs->groups->content->groups->dropdownSettings
        * @type {object}
        * @example 
        *&lt;ul id="menu"&gt;
	   *&lt;li&gt;&lt;a&gt;FILE &lt;/a&gt;
       *&lt;ul&gt;
       *&lt;li&gt;&lt;a&gt;New&lt;/a&gt;&lt;/li&gt;
       *&lt;li&gt;&lt;a&gt;Open&lt;/a&gt;&lt;/li&gt;
       *&lt;li&gt;&lt;a&gt;Save&lt;/a&gt;&lt;/li&gt;
       *&lt;li&gt;&lt;a&gt;Save as&lt;/a&gt;&lt;/li&gt;
       *&lt;li&gt;&lt;a&gt;Print&lt;/a&gt;&lt;/li&gt;
	   *&lt;/ul&gt;&lt;/li&gt;&lt;/ul&gt;
       *&lt;div id="Ribbon"&gt;&lt;/div&gt; 
	   *&lt;style&gt;
       *.e-cssclass {
       *      margin-left: 20px;
       *}
       *&lt;/style&gt;
       *&lt;script type="text/javascript"&gt;  
       *var fontfamily = ["Segoe UI", "Arial", "Times New Roman", "Tahoma", "Helvetica"];	   
       *$(function () {
       *     $("#Ribbon").ejRibbon({
	   * // Set the width during initialization. 	
       *      width: "100%",
       *        applicationTab: { Type: "ApplicationMenu", itemID: "menu", menuSettings: { openOnClick: false } },
       *        tabs: [{
       *            id: "home", text: "HOME", groups: [{
       *                 text: "New", alignType: ej.Ribbon.alignType.rows,content: [{
       *                     groups: [{
       *                        id: "fontfamily",
       *                        toolTip: "Font",
	   *						type: ej.Ribbon.type.dropDownList,
       *                        dropdownSettings: {
       *                             dataSource: fontfamily,
       *                             value: "Segoe UI",
       *                             width: 150
       *                         }
       *                     }]
       *                 }]
       *            }]
       *         }]
       *   });
       * });	        
       * &lt;/script&gt;  
       * @memberof ejRibbon
       * @instance
       */
			   dropdownSettings:{},
			   }],
			     /**		
        * Specify the height,width,type,isBig property to the controls in the group commonly.
        * @default object
		* @alias ejRibbon#tabs->groups->content->defaults
        * @type {object}
        * @example 
       *&lt;ul id="menu"&gt;
	   *&lt;li&gt;&lt;a&gt;FILE &lt;/a&gt;
       *&lt;ul&gt;
       *&lt;li&gt;&lt;a&gt;New&lt;/a&gt;&lt;/li&gt;
       *&lt;li&gt;&lt;a&gt;Open&lt;/a&gt;&lt;/li&gt;
       *&lt;li&gt;&lt;a&gt;Save&lt;/a&gt;&lt;/li&gt;
       *&lt;li&gt;&lt;a&gt;Save as&lt;/a&gt;&lt;/li&gt;
       *&lt;li&gt;&lt;a&gt;Print&lt;/a&gt;&lt;/li&gt;
	   *&lt;/ul&gt;&lt;/li&gt;&lt;/ul&gt;
       *&lt;div id="Ribbon"&gt;&lt;/div&gt; 
       *&lt;script type="text/javascript"&gt;   
       *$(function () {
       *     $("#Ribbon").ejRibbon({
	   * // Set the width during initialization. 	
       *      width: "100%",
       *        applicationTab: { Type: "ApplicationMenu", itemID: "menu", menuSettings: { openOnClick: false } },
       *        tabs: [{
       *            id: "home", text: "HOME", groups: [{
       *                 text: "New", alignType: ej.Ribbon.alignType.rows,content: [{
       *                     groups: [{
       *                         id: "new",
       *                         text: "New",
	   *                         toolTip: "New",
       *                          buttonSettings: {
       *                          width: 100,
       *                          }
       *                     },
	   * 						{
       *                         id: "font",
       *                         text: "Font",
	   *                         toolTip: "Font",
       *                         buttonSettings: {
       *                         width: 150,
       *                         }
       *                     }],
	   *						 defaults: {
       *                        type: ej.Ribbon.type.button,
       *                        height: 70
       *                    }
       *                 }]
       *            }]
       *         }]
       *   });
       * });	        
       * &lt;/script&gt;  
       * @memberof ejRibbon
       * @instance
       */
			   defaults:{},
			   }],
			   }]
			}],
			   /**		
        * Specify the contextual tabs and tabset to the ribbon control with the background color and border color.Please refer the tabs section for adding tabs into the contextual tab and contextual tab set.
        * @default object
		* @alias ejRibbon#contextualTabs
        * @type {object}
		* @example 
       *&lt;ul id="menu"&gt;
	   *&lt;li&gt;&lt;a&gt;FILE &lt;/a&gt;
       *&lt;ul&gt;
       *&lt;li&gt;&lt;a&gt;New&lt;/a&gt;&lt;/li&gt;
       *&lt;li&gt;&lt;a&gt;Open&lt;/a&gt;&lt;/li&gt;
       *&lt;li&gt;&lt;a&gt;Save&lt;/a&gt;&lt;/li&gt;
       *&lt;li&gt;&lt;a&gt;Save as&lt;/a&gt;&lt;/li&gt;
       *&lt;li&gt;&lt;a&gt;Print&lt;/a&gt;&lt;/li&gt;
	   *&lt;/ul&gt;&lt;/li&gt;&lt;/ul&gt;
       *&lt;div id="Ribbon"&gt;&lt;/div&gt; 
	   *&lt;button id="btn"&gt;Home button&lt;/button&gt;
	   *&lt;button id="design"&gt;Design button&lt;/button&gt;
	   *&lt;button id="div1"&gt;First button&lt;/button&gt;
	   *&lt;button id="div2"&gt;Second button&lt;/button&gt;
       *&lt;script type="text/javascript"&gt;   
       *$(function () {
       *     $("#Ribbon").ejRibbon({
	   * // Set the width during initialization. 	
       *      width: "100%",
       *        applicationTab: { Type: "ApplicationMenu", itemID: "menu", menuSettings: { openOnClick: false } },
       *        tabs: [{
       *            id: "home", text: "HOME", groups: [{
       *                 text: "New", alignType: ej.Ribbon.alignType.rows,type:"custom",contentID:"btn"
       *            }]
       *         }],
	   *         contextualTabs:
       *                      {
       *               tabs: [
       *                        {
       *                           id: "Design", text: "DESIGN",backgroundColor:"#FCFBEB",borderColor: "#F2CC1C", groups: [{
       *                                     text: "Table Style Options", type: "custom", contentID: "design"
       *                                 }
       *                                 ]
       *                            }],
	   *               tabSet: [
       *                         {
       *                            borderColor: "lightblue",backgroundColor:"blue",
       *                             tabs:[{
       *                                 id: "tabset1", text: "Tabset1", 
       *                                 groups: [
       *                                 { text: "Tabset1 Styles", type: "custom", contentID: "div1"
       *                                 }]
       *                            },
       *                             {
       *                                 id: "tabset2", text: "Tabset2",
       *                                 groups: [
       *                                     { text: "Tabset2 Styles", type: "custom", contentID: "div2"
       *                                     }
       *                                 ]
       *                            }]
       * }
       * ]}
       *   });
       * });	        
       * &lt;/script&gt;  
       * @memberof ejRibbon
       * @instance
       */
            contextualTabs: {
			 /**	
			 * Specify the contextual tabs to the ribbon control with the background color and border color.Please refer the tabs section for adding tabs into the contextual tab.
           * @default array
		   * @alias ejRibbon#contextualTabs->tabs
           * @type {array}
			* @example 
       *&lt;ul id="menu"&gt;
	   *&lt;li&gt;&lt;a&gt;FILE &lt;/a&gt;
       *&lt;ul&gt;
       *&lt;li&gt;&lt;a&gt;New&lt;/a&gt;&lt;/li&gt;
       *&lt;li&gt;&lt;a&gt;Open&lt;/a&gt;&lt;/li&gt;
       *&lt;li&gt;&lt;a&gt;Save&lt;/a&gt;&lt;/li&gt;
       *&lt;li&gt;&lt;a&gt;Save as&lt;/a&gt;&lt;/li&gt;
       *&lt;li&gt;&lt;a&gt;Print&lt;/a&gt;&lt;/li&gt;
	   *&lt;/ul&gt;&lt;/li&gt;&lt;/ul&gt;
       *&lt;div id="Ribbon"&gt;&lt;/div&gt; 
	   *&lt;button id="btn"&gt;Home button&lt;/button&gt;
	   *&lt;button id="design"&gt;Design button&lt;/button&gt;
	   *&lt;button id="div1"&gt;First button&lt;/button&gt;
	   *&lt;button id="div2"&gt;Second button&lt;/button&gt;
       *&lt;script type="text/javascript"&gt;   
       *$(function () {
       *     $("#Ribbon").ejRibbon({
	   * // Set the width during initialization. 	
       *      width: "100%",
       *        applicationTab: { Type: "ApplicationMenu", itemID: "menu", menuSettings: { openOnClick: false } },
       *        tabs: [{
       *            id: "home", text: "HOME", groups: [{
       *                 text: "New", alignType: ej.Ribbon.alignType.rows,type:"custom",contentID:"btn"
       *            }]
       *         }],
	   *         contextualTabs:
       *                      {
       *               tabs: [
       *                        {
       *                           id: "Design", text: "DESIGN",backgroundColor:"#FCFBEB",borderColor: "#F2CC1C", groups: [{
       *                                     text: "Table Style Options", type: "custom", contentID: "design"
       *                                 }
       *                                 ]
       *                            }],
	   *               tabSet: [
       *                         {
       *                            borderColor: "lightblue",backgroundColor:"blue",
       *                             tabs:[{
       *                                 id: "tabset1", text: "Tabset1", 
       *                                 groups: [
       *                                 { text: "Tabset1 Styles", type: "custom", contentID: "div1"
       *                                 }]
       *                            },
       *                             {
       *                                 id: "tabset2", text: "Tabset2",
       *                                 groups: [
       *                                     { text: "Tabset2 Styles", type: "custom", contentID: "div2"
       *                                     }
       *                                 ]
       *                            }]
       * }
       * ]}
       *   });
       * });	        
       * &lt;/script&gt;  
       * @memberof ejRibbon
       * @instance
       */
            tabs: [],
			  /**		
        * Specify the contextual tabset to the ribbon control with the background color and border color.Please refer the tabs section for adding tabs into the contextual tab set.
        * @default array
		* @alias ejRibbon#contextualTabs->tabSet
        * @type {array}
		* @example 
       *&lt;ul id="menu"&gt;
	   *&lt;li&gt;&lt;a&gt;FILE &lt;/a&gt;
       *&lt;ul&gt;
       *&lt;li&gt;&lt;a&gt;New&lt;/a&gt;&lt;/li&gt;
       *&lt;li&gt;&lt;a&gt;Open&lt;/a&gt;&lt;/li&gt;
       *&lt;li&gt;&lt;a&gt;Save&lt;/a&gt;&lt;/li&gt;
       *&lt;li&gt;&lt;a&gt;Save as&lt;/a&gt;&lt;/li&gt;
       *&lt;li&gt;&lt;a&gt;Print&lt;/a&gt;&lt;/li&gt;
	   *&lt;/ul&gt;&lt;/li&gt;&lt;/ul&gt;
       *&lt;div id="Ribbon"&gt;&lt;/div&gt; 
	   *&lt;button id="btn"&gt;Home button&lt;/button&gt;
	   *&lt;button id="design"&gt;Design button&lt;/button&gt;
	   *&lt;button id="div1"&gt;First button&lt;/button&gt;
	   *&lt;button id="div2"&gt;Second button&lt;/button&gt;
       *&lt;script type="text/javascript"&gt;   
       *$(function () {
       *     $("#Ribbon").ejRibbon({
	   * // Set the width during initialization. 	
       *      width: "100%",
       *        applicationTab: { Type: "ApplicationMenu", itemID: "menu", menuSettings: { openOnClick: false } },
       *        tabs: [{
       *            id: "home", text: "HOME", groups: [{
       *                 text: "New", alignType: ej.Ribbon.alignType.rows,type:"custom",contentID:"btn"
       *            }]
       *         }],
	   *         contextualTabs:
       *                      {
       *               tabs: [
       *                        {
       *                           id: "Design", text: "DESIGN",backgroundColor:"#FCFBEB",borderColor: "#F2CC1C", groups: [{
       *                                     text: "Table Style Options", type: "custom", contentID: "design"
       *                                 }
       *                                 ]
       *                            }],
	   *               tabSet: [
       *                         {
       *                            borderColor: "lightblue",backgroundColor:"blue",
       *                             tabs:[{
       *                                 id: "tabset1", text: "Tabset1", 
       *                                 groups: [
       *                                 { text: "Tabset1 Styles", type: "custom", contentID: "div1"
       *                                 }]
       *                            },
       *                             {
       *                                 id: "tabset2", text: "Tabset2",
       *                                 groups: [
       *                                     { text: "Tabset2 Styles", type: "custom", contentID: "div2"
       *                                     }
       *                                 ]
       *                            }]
       * }
       * ]}
       *   });
       * });	        
       * &lt;/script&gt;  
       * @memberof ejRibbon
       * @instance
       */
			                tabSet:[{
							  /**		
        * Specify the backgroundColor of the contextual tabset in the ribbon control.
        * @default null
		* @alias ejRibbon#contextualTabs->tabSet->backgroundColor
        * @type {string}
		* @example 
       *&lt;ul id="menu"&gt;
	   *&lt;li&gt;&lt;a&gt;FILE &lt;/a&gt;
       *&lt;ul&gt;
       *&lt;li&gt;&lt;a&gt;New&lt;/a&gt;&lt;/li&gt;
       *&lt;li&gt;&lt;a&gt;Open&lt;/a&gt;&lt;/li&gt;
       *&lt;li&gt;&lt;a&gt;Save&lt;/a&gt;&lt;/li&gt;
       *&lt;li&gt;&lt;a&gt;Save as&lt;/a&gt;&lt;/li&gt;
       *&lt;li&gt;&lt;a&gt;Print&lt;/a&gt;&lt;/li&gt;
	   *&lt;/ul&gt;&lt;/li&gt;&lt;/ul&gt;
       *&lt;div id="Ribbon"&gt;&lt;/div&gt; 
	   *&lt;button id="btn"&gt;Home button&lt;/button&gt;
	   *&lt;button id="design"&gt;Design button&lt;/button&gt;
	   *&lt;button id="div1"&gt;First button&lt;/button&gt;
	   *&lt;button id="div2"&gt;Second button&lt;/button&gt;
       *&lt;script type="text/javascript"&gt;   
       *$(function () {
       *     $("#Ribbon").ejRibbon({
	   * // Set the width during initialization. 	
       *      width: "100%",
       *        applicationTab: { Type: "ApplicationMenu", itemID: "menu", menuSettings: { openOnClick: false } },
       *        tabs: [{
       *            id: "home", text: "HOME", groups: [{
       *                 text: "New", alignType: ej.Ribbon.alignType.rows,type:"custom",contentID:"btn"
       *            }]
       *         }],
	   *         contextualTabs:
       *                      {
       *               tabs: [
       *                        {
       *                           id: "Design", text: "DESIGN",backgroundColor:"#FCFBEB",borderColor: "#F2CC1C", groups: [{
       *                                     text: "Table Style Options", type: "custom", contentID: "design"
       *                                 }
       *                                 ]
       *                            }],
	   *               tabSet: [
       *                         {
       *                            borderColor: "lightblue",backgroundColor:"blue",
       *                             tabs:[{
       *                                 id: "tabset1", text: "Tabset1", 
       *                                 groups: [
       *                                 { text: "Tabset1 Styles", type: "custom", contentID: "div1"
       *                                 }]
       *                            },
       *                             {
       *                                 id: "tabset2", text: "Tabset2",
       *                                 groups: [
       *                                     { text: "Tabset2 Styles", type: "custom", contentID: "div2"
       *                                     }
       *                                 ]
       *                            }]
       * }
       * ]}
       *   });
       * });	        
       * &lt;/script&gt;  
       * @memberof ejRibbon
       * @instance
       */
							backgroundColor:null,
							   /**		
        * Specify the borderColor of the contextual tabset in the ribbon control.
        * @default null
		* @alias ejRibbon#contextualTabs->tabSet->borderColor
        * @type {string}
		* @example 
       *&lt;ul id="menu"&gt;
	   *&lt;li&gt;&lt;a&gt;FILE &lt;/a&gt;
       *&lt;ul&gt;
       *&lt;li&gt;&lt;a&gt;New&lt;/a&gt;&lt;/li&gt;
       *&lt;li&gt;&lt;a&gt;Open&lt;/a&gt;&lt;/li&gt;
       *&lt;li&gt;&lt;a&gt;Save&lt;/a&gt;&lt;/li&gt;
       *&lt;li&gt;&lt;a&gt;Save as&lt;/a&gt;&lt;/li&gt;
       *&lt;li&gt;&lt;a&gt;Print&lt;/a&gt;&lt;/li&gt;
	   *&lt;/ul&gt;&lt;/li&gt;&lt;/ul&gt;
       *&lt;div id="Ribbon"&gt;&lt;/div&gt; 
	   *&lt;button id="btn"&gt;Home button&lt;/button&gt;
	   *&lt;button id="design"&gt;Design button&lt;/button&gt;
	   *&lt;button id="div1"&gt;First button&lt;/button&gt;
	   *&lt;button id="div2"&gt;Second button&lt;/button&gt;
       *&lt;script type="text/javascript"&gt;   
       *$(function () {
       *     $("#Ribbon").ejRibbon({
	   * // Set the width during initialization. 	
       *      width: "100%",
       *        applicationTab: { Type: "ApplicationMenu", itemID: "menu", menuSettings: { openOnClick: false } },
       *        tabs: [{
       *            id: "home", text: "HOME", groups: [{
       *                 text: "New", alignType: ej.Ribbon.alignType.rows,type:"custom",contentID:"btn"
       *            }]
       *         }],
	   *         contextualTabs:
       *                      {
       *               tabs: [
       *                        {
       *                           id: "Design", text: "DESIGN",backgroundColor:"#FCFBEB",borderColor: "#F2CC1C", groups: [{
       *                                     text: "Table Style Options", type: "custom", contentID: "design"
       *                                 }
       *                                 ]
       *                            }],
	   *               tabSet: [
       *                         {
       *                            borderColor: "lightblue",backgroundColor:"blue",
       *                             tabs:[{
       *                                 id: "tabset1", text: "Tabset1", 
       *                                 groups: [
       *                                 { text: "Tabset1 Styles", type: "custom", contentID: "div1"
       *                                 }]
       *                            },
       *                             {
       *                                 id: "tabset2", text: "Tabset2",
       *                                 groups: [
       *                                     { text: "Tabset2 Styles", type: "custom", contentID: "div2"
       *                                     }
       *                                 ]
       *                            }]
       * }
       * ]}
       *   });
       * });	        
       * &lt;/script&gt;  
       * @memberof ejRibbon
       * @instance
       */
			                borderColor:null,
							/**	
			 * Specify the tabs to present in the contectual tab set.Please refer the tabs section for adding tabs into the contextual tab set.
           * @default array
		   * @alias ejRibbon#contextualTabs->tabSet->tabs
           * @type {array}
			* @example 
       *&lt;ul id="menu"&gt;
	   *&lt;li&gt;&lt;a&gt;FILE &lt;/a&gt;
       *&lt;ul&gt;
       *&lt;li&gt;&lt;a&gt;New&lt;/a&gt;&lt;/li&gt;
       *&lt;li&gt;&lt;a&gt;Open&lt;/a&gt;&lt;/li&gt;
       *&lt;li&gt;&lt;a&gt;Save&lt;/a&gt;&lt;/li&gt;
       *&lt;li&gt;&lt;a&gt;Save as&lt;/a&gt;&lt;/li&gt;
       *&lt;li&gt;&lt;a&gt;Print&lt;/a&gt;&lt;/li&gt;
	   *&lt;/ul&gt;&lt;/li&gt;&lt;/ul&gt;
       *&lt;div id="Ribbon"&gt;&lt;/div&gt; 
	   *&lt;button id="btn"&gt;Home button&lt;/button&gt;
	   *&lt;button id="design"&gt;Design button&lt;/button&gt;
	   *&lt;button id="div1"&gt;First button&lt;/button&gt;
	   *&lt;button id="div2"&gt;Second button&lt;/button&gt;
       *&lt;script type="text/javascript"&gt;   
       *$(function () {
       *     $("#Ribbon").ejRibbon({
	   * // Set the width during initialization. 	
       *      width: "100%",
       *        applicationTab: { Type: "ApplicationMenu", itemID: "menu", menuSettings: { openOnClick: false } },
       *        tabs: [{
       *            id: "home", text: "HOME", groups: [{
       *                 text: "New", alignType: ej.Ribbon.alignType.rows,type:"custom",contentID:"btn"
       *            }]
       *         }],
	   *         contextualTabs:
       *                      {
       *               tabs: [
       *                        {
       *                           id: "Design", text: "DESIGN",backgroundColor:"#FCFBEB",borderColor: "#F2CC1C", groups: [{
       *                                     text: "Table Style Options", type: "custom", contentID: "design"
       *                                 }
       *                                 ]
       *                            }],
	   *               tabSet: [
       *                         {
       *                            borderColor: "lightblue",backgroundColor:"blue",
       *                             tabs:[{
       *                                 id: "tabset1", text: "Tabset1", 
       *                                 groups: [
       *                                 { text: "Tabset1 Styles", type: "custom", contentID: "div1"
       *                                 }]
       *                            },
       *                             {
       *                                 id: "tabset2", text: "Tabset2",
       *                                 groups: [
       *                                     { text: "Tabset2 Styles", type: "custom", contentID: "div2"
       *                                     }
       *                                 ]
       *                            }]
       * }
       * ]}
       *   });
       * });	        
       * &lt;/script&gt;  
       * @memberof ejRibbon
       * @instance
       */
							tabs:[]
							}]				
			},
			/**     
    * Triggered after the ribbon tab item selected in the ribbon control.
    * @event
    * @name ejRibbon#onTabSelect   
    * @param {Object} argument Event parameters from button     
    * @param {boolean} argument.cancel if the event should be canceled; otherwise, false.
    * @param {object}  argument.model returns the ribbon model.
    * @param {string}  argument.type returns the name of the event.
    * @param {object}  argument.prevActiveHeader returns previous active tab header.
     * @param {number}  argument.prevActiveIndex returns previous active index.
     * @param {object}  argument.activeHeader returns current active tab header .
     * @param {number}  argument.activeIndex returns current active index.
    * @example 
      *&lt;ul id="menu"&gt;
	   *&lt;li&gt;&lt;a&gt;FILE &lt;/a&gt;
       *&lt;ul&gt;
       *&lt;li&gt;&lt;a&gt;New&lt;/a&gt;&lt;/li&gt;
       *&lt;li&gt;&lt;a&gt;Open&lt;/a&gt;&lt;/li&gt;
       *&lt;li&gt;&lt;a&gt;Save&lt;/a&gt;&lt;/li&gt;
       *&lt;li&gt;&lt;a&gt;Save as&lt;/a&gt;&lt;/li&gt;
       *&lt;li&gt;&lt;a&gt;Print&lt;/a&gt;&lt;/li&gt;
	   *&lt;/ul&gt;&lt;/li&gt;&lt;/ul&gt;
       *&lt;div id="Ribbon"&gt;&lt;/div&gt; 
	   *&lt;button id="btn"&gt;Home button&lt;/button&gt;
       *&lt;script type="text/javascript"&gt;   
       *$(function () {
       *     $("#Ribbon").ejRibbon({
	   * // Set the width during initialization. 	
       *      width: "100%",
       *        applicationTab: { Type: "ApplicationMenu", itemID: "menu", menuSettings: { openOnClick: false } },
       *        tabs: [{
       *            id: "home", text: "HOME", groups: [{
       *                 text: "New", alignType: ej.Ribbon.alignType.rows,type:"custom",contentID:"btn"
       *            }]
       *         }],
	   *     onTabSelect: function (args) {}
       *   });
       * });	        
       * &lt;/script&gt;  
       * @memberof ejRibbon
       * @instance
       */
            onTabSelect: null,
	   /**     
	   * Triggered when tab is clicked successfully in the ribbon control.
	   * @event
	   * @name ejRibbon#onTabClick 	
	   * @param {Object} argument Event parameters from button     
	   * @param {boolean} argument.cancel if the event should be canceled; otherwise, false.
	   * @param {object}  argument.model returns the ribbon model.
	   * @param {string}  argument.type returns the name of the event.
	   * @param {object}  argument.prevActiveHeader returns previous active tab header.
	   * @param {number}  argument.prevActiveIndex returns previous active index.
	   * @param {object}  argument.activeHeader returns current active tab header .
	   * @param {number}  argument.activeIndex returns current active index.
	   * @example 
       *&lt;ul id="menu"&gt;
	   *&lt;li&gt;&lt;a&gt;FILE &lt;/a&gt;
       *&lt;ul&gt;
       *&lt;li&gt;&lt;a&gt;New&lt;/a&gt;&lt;/li&gt;
       *&lt;li&gt;&lt;a&gt;Open&lt;/a&gt;&lt;/li&gt;
       *&lt;li&gt;&lt;a&gt;Save&lt;/a&gt;&lt;/li&gt;
       *&lt;li&gt;&lt;a&gt;Save as&lt;/a&gt;&lt;/li&gt;
       *&lt;li&gt;&lt;a&gt;Print&lt;/a&gt;&lt;/li&gt;
	   *&lt;/ul&gt;&lt;/li&gt;&lt;/ul&gt;
       *&lt;div id="Ribbon"&gt;&lt;/div&gt; 
	   *&lt;button id="btn"&gt;Home button&lt;/button&gt;
       *&lt;script type="text/javascript"&gt;   
       *$(function () {
       *     $("#Ribbon").ejRibbon({
	   * // Set the width during initialization. 	
       *      width: "100%",
       *        applicationTab: { Type: "ApplicationMenu", itemID: "menu", menuSettings: { openOnClick: false } },
       *        tabs: [{
       *            id: "home", text: "HOME", groups: [{
       *                 text: "New", alignType: ej.Ribbon.alignType.rows,type:"custom",contentID:"btn"
       *            }]
       *         }],
	   *      onTabClick: function (args) {}
       *   });
       * });	        
       * &lt;/script&gt;  
       * @memberof ejRibbon
       * @instance
       */
            onTabClick: null,
			/**     
			 * Triggered after the new ribbon tab item add.
			 * @event
			 * @name ejRibbon#onTabAdd 	
			 * @param {Object} argument Event parameters from button     
			 * @param {boolean} argument.cancel if the event should be canceled; otherwise, false.
			 * @param {object}  argument.model returns the ribbon model.
			 * @param {string}  argument.type returns the name of the event.
			 * @param {object}  argument.tabHeader returns new added tab header.
			 * @param {object}  argument.tabContent returns new added tab content panel.
			 * @example 
     *&lt;ul id="menu"&gt;
	   *&lt;li&gt;&lt;a&gt;FILE &lt;/a&gt;
       *&lt;ul&gt;
       *&lt;li&gt;&lt;a&gt;New&lt;/a&gt;&lt;/li&gt;
       *&lt;li&gt;&lt;a&gt;Open&lt;/a&gt;&lt;/li&gt;
       *&lt;li&gt;&lt;a&gt;Save&lt;/a&gt;&lt;/li&gt;
       *&lt;li&gt;&lt;a&gt;Save as&lt;/a&gt;&lt;/li&gt;
       *&lt;li&gt;&lt;a&gt;Print&lt;/a&gt;&lt;/li&gt;
	   *&lt;/ul&gt;&lt;/li&gt;&lt;/ul&gt;
       *&lt;div id="Ribbon"&gt;&lt;/div&gt; 
	   *&lt;button id="btn"&gt;Home button&lt;/button&gt;
       *&lt;script type="text/javascript"&gt;   
       *$(function () {
       *     $("#Ribbon").ejRibbon({
	   * // Set the width during initialization. 	
       *      width: "100%",
       *        applicationTab: { Type: "ApplicationMenu", itemID: "menu", menuSettings: { openOnClick: false } },
       *        tabs: [{
       *            id: "home", text: "HOME", groups: [{
       *                 text: "New", alignType: ej.Ribbon.alignType.rows,type:"custom",contentID:"btn"
       *            }]
       *         }],
	   *       onTabAdd: function (args) {}
       *   });
       * });	        
       * &lt;/script&gt;  
       * @memberof ejRibbon
       * @instance
       */
            onTabAdd: null,
			/**     
			 * Triggered after the tab item removed from the ribbon control.
			 * @event
			 * @name ejRibbon#onTabRemove 	
			 * @param {Object} argument Event parameters from button     
			 * @param {boolean} argument.cancel if the event should be canceled; otherwise, false.
			 * @param {object}  argument.model returns the ribbon model.
			 * @param {string}  argument.type returns the name of the event.
			 * @param {object}  argument.removedTab returns removed tab header.
			 * @param {object}  argument.removedPanel returns removed tab content panel.
			 * @example 
      *&lt;ul id="menu"&gt;
	   *&lt;li&gt;&lt;a&gt;FILE &lt;/a&gt;
       *&lt;ul&gt;
       *&lt;li&gt;&lt;a&gt;New&lt;/a&gt;&lt;/li&gt;
       *&lt;li&gt;&lt;a&gt;Open&lt;/a&gt;&lt;/li&gt;
       *&lt;li&gt;&lt;a&gt;Save&lt;/a&gt;&lt;/li&gt;
       *&lt;li&gt;&lt;a&gt;Save as&lt;/a&gt;&lt;/li&gt;
       *&lt;li&gt;&lt;a&gt;Print&lt;/a&gt;&lt;/li&gt;
	   *&lt;/ul&gt;&lt;/li&gt;&lt;/ul&gt;
       *&lt;div id="Ribbon"&gt;&lt;/div&gt; 
	   *&lt;button id="btn"&gt;Home button&lt;/button&gt;
       *&lt;script type="text/javascript"&gt;   
       *$(function () {
       *     $("#Ribbon").ejRibbon({
	   * // Set the width during initialization. 	
       *      width: "100%",
       *        applicationTab: { Type: "ApplicationMenu", itemID: "menu", menuSettings: { openOnClick: false } },
       *        tabs: [{
       *            id: "home", text: "HOME", groups: [{
       *                 text: "New", alignType: ej.Ribbon.alignType.rows,type:"custom",contentID:"btn"
       *            }]
       *         }],
	   *       onTabRemove: function (args) {}
       *   });
       * });	        
       * &lt;/script&gt;  
       * @memberof ejRibbon
       * @instance
       */
            onTabRemove: null,
			/**     
			 * Triggered before the ribbon tab item remove.
			 * @event
			 * @name ejRibbon#beforeTabRemove 	
			 * @param {Object} argument Event parameters from button.     
			 * @param {boolean} argument.cancel if the event should be canceled; otherwise, false.
			 * @param {object}  argument.model returns the ribbon model.
			 * @param {string}  argument.type returns the name of the event.
			 * @param {number}  argument.index returns current tab item index in the ribbon control.		
			 * @example 
      *&lt;ul id="menu"&gt;
	   *&lt;li&gt;&lt;a&gt;FILE &lt;/a&gt;
       *&lt;ul&gt;
       *&lt;li&gt;&lt;a&gt;New&lt;/a&gt;&lt;/li&gt;
       *&lt;li&gt;&lt;a&gt;Open&lt;/a&gt;&lt;/li&gt;
       *&lt;li&gt;&lt;a&gt;Save&lt;/a&gt;&lt;/li&gt;
       *&lt;li&gt;&lt;a&gt;Save as&lt;/a&gt;&lt;/li&gt;
       *&lt;li&gt;&lt;a&gt;Print&lt;/a&gt;&lt;/li&gt;
	   *&lt;/ul&gt;&lt;/li&gt;&lt;/ul&gt;
       *&lt;div id="Ribbon"&gt;&lt;/div&gt; 
	   *&lt;button id="btn"&gt;Home button&lt;/button&gt;
       *&lt;script type="text/javascript"&gt;   
       *$(function () {
       *     $("#Ribbon").ejRibbon({
	   * // Set the width during initialization. 	
       *      width: "100%",
       *        applicationTab: { Type: "ApplicationMenu", itemID: "menu", menuSettings: { openOnClick: false } },
       *        tabs: [{
       *            id: "home", text: "HOME", groups: [{
       *                 text: "New", alignType: ej.Ribbon.alignType.rows,type:"custom",contentID:"btn"
       *            }]
       *         }],
	   *       beforeTabRemove: function (args) {}
       *   });
       * });	        
       * &lt;/script&gt;  
       * @memberof ejRibbon
       * @instance
       */
            beforeTabRemove: null,
			  /**     
    * Triggered before the ribbon control create.
    * @event
    * @name ejRibbon#create   
    * @param {Object} argument Event parameters from button.     
    * @param {boolean} argument.cancel if the event should be canceled; otherwise, false.
    * @param {object}  argument.model returns the ribbon model.
    * @param {string}  argument.type returns the name of the event.
    * @param {number}  argument.deleteIndex returns current ribbon tab item index     
    * @example 
      *&lt;ul id="menu"&gt;
	   *&lt;li&gt;&lt;a&gt;FILE &lt;/a&gt;
       *&lt;ul&gt;
       *&lt;li&gt;&lt;a&gt;New&lt;/a&gt;&lt;/li&gt;
       *&lt;li&gt;&lt;a&gt;Open&lt;/a&gt;&lt;/li&gt;
       *&lt;li&gt;&lt;a&gt;Save&lt;/a&gt;&lt;/li&gt;
       *&lt;li&gt;&lt;a&gt;Save as&lt;/a&gt;&lt;/li&gt;
       *&lt;li&gt;&lt;a&gt;Print&lt;/a&gt;&lt;/li&gt;
	   *&lt;/ul&gt;&lt;/li&gt;&lt;/ul&gt;
       *&lt;div id="Ribbon"&gt;&lt;/div&gt; 
	   *&lt;button id="btn"&gt;Home button&lt;/button&gt;
       *&lt;script type="text/javascript"&gt;   
       *$(function () {
       *     $("#Ribbon").ejRibbon({
	   * // Set the width during initialization. 	
       *      width: "100%",
       *        applicationTab: { Type: "ApplicationMenu", itemID: "menu", menuSettings: { openOnClick: false } },
       *        tabs: [{
       *            id: "home", text: "HOME", groups: [{
       *                 text: "New", alignType: ej.Ribbon.alignType.rows,type:"custom",contentID:"btn"
       *            }]
       *         }],
	   *       create: function (args) {}
       *   });
       * });	        
       * &lt;/script&gt;  
       * @memberof ejRibbon
       * @instance
       */
            create: null,
			/**     
			 * Triggered when the control in the group is clicked successfully .
			 * @event
			 * @name ejRibbon#groupClick 	   
			 * @param {boolean} argument.cancel if the event should be canceled; otherwise, false.
			 * @param {object}  argument.model returns the ribbon model.
			 * @param {string}  argument.type returns the name of the event.
			 * @param {number}  argument.target returns the control clicked in the group.		
			 * @example 
      *&lt;ul id="menu"&gt;
	   *&lt;li&gt;&lt;a&gt;FILE &lt;/a&gt;
       *&lt;ul&gt;
       *&lt;li&gt;&lt;a&gt;New&lt;/a&gt;&lt;/li&gt;
       *&lt;li&gt;&lt;a&gt;Open&lt;/a&gt;&lt;/li&gt;
       *&lt;li&gt;&lt;a&gt;Save&lt;/a&gt;&lt;/li&gt;
       *&lt;li&gt;&lt;a&gt;Save as&lt;/a&gt;&lt;/li&gt;
       *&lt;li&gt;&lt;a&gt;Print&lt;/a&gt;&lt;/li&gt;
	   *&lt;/ul&gt;&lt;/li&gt;&lt;/ul&gt;
       *&lt;div id="Ribbon"&gt;&lt;/div&gt; 
	   *&lt;button id="btn"&gt;Home button&lt;/button&gt;
       *&lt;script type="text/javascript"&gt;   
       *$(function () {
       *     $("#Ribbon").ejRibbon({
	   * // Set the width during initialization. 	
       *      width: "100%",
       *        applicationTab: { Type: "ApplicationMenu", itemID: "menu", menuSettings: { openOnClick: false } },
       *        tabs: [{
       *            id: "home", text: "HOME", groups: [{
       *                 text: "New", alignType: ej.Ribbon.alignType.rows,type:"custom",contentID:"btn"
       *            }]
       *         }],
	   *       groupClick: function (args) {}
       *   });
       * });	        
       * &lt;/script&gt;  
       * @memberof ejRibbon
       * @instance
       */
            groupClick: null,
			/**     
			 * Triggered when the expand/collapse button is clicked successfully .
			 * @event
			 * @name ejRibbon#onToggleButtonClick 	   
			 * @param {boolean} argument.cancel if the event should be canceled; otherwise, false.
			 * @param {object}  argument.model returns the ribbon model.
			 * @param {string}  argument.type returns the name of the event.
			 * @param {number}  argument.target returns the expand/collapse button.		
			 * @example 
      *&lt;ul id="menu"&gt;
	   *&lt;li&gt;&lt;a&gt;FILE &lt;/a&gt;
       *&lt;ul&gt;
       *&lt;li&gt;&lt;a&gt;New&lt;/a&gt;&lt;/li&gt;
       *&lt;li&gt;&lt;a&gt;Open&lt;/a&gt;&lt;/li&gt;
       *&lt;li&gt;&lt;a&gt;Save&lt;/a&gt;&lt;/li&gt;
       *&lt;li&gt;&lt;a&gt;Save as&lt;/a&gt;&lt;/li&gt;
       *&lt;li&gt;&lt;a&gt;Print&lt;/a&gt;&lt;/li&gt;
	   *&lt;/ul&gt;&lt;/li&gt;&lt;/ul&gt;
       *&lt;div id="Ribbon"&gt;&lt;/div&gt; 
	   *&lt;button id="btn"&gt;Home button&lt;/button&gt;
       *&lt;script type="text/javascript"&gt;   
       *$(function () {
       *     $("#Ribbon").ejRibbon({
	   * // Set the width during initialization. 	
       *      width: "100%",
       *        applicationTab: { Type: "ApplicationMenu", itemID: "menu", menuSettings: { openOnClick: false } },
       *        tabs: [{
       *            id: "home", text: "HOME", groups: [{
       *                 text: "New", alignType: ej.Ribbon.alignType.rows,type:"custom",contentID:"btn"
       *            }]
       *         }],
	   *       onToggleButtonClick: function (args) {}
       *   });
       * });	        
       * &lt;/script&gt;  
       * @memberof ejRibbon
       * @instance
       */
            onToggleButtonClick: null,
			/**     
			 * Triggered when the groupexpander in the group is clicked successfully .
			 * @event
			 * @name ejRibbon#groupExpand 	   
			 * @param {boolean} argument.cancel if the event should be canceled; otherwise, false.
			 * @param {object}  argument.model returns the ribbon model.
			 * @param {string}  argument.type returns the name of the event.
			 * @param {number}  argument.target returns the clicked groupexpander.		
			 * @example 
      *&lt;ul id="menu"&gt;
	   *&lt;li&gt;&lt;a&gt;FILE &lt;/a&gt;
       *&lt;ul&gt;
       *&lt;li&gt;&lt;a&gt;New&lt;/a&gt;&lt;/li&gt;
       *&lt;li&gt;&lt;a&gt;Open&lt;/a&gt;&lt;/li&gt;
       *&lt;li&gt;&lt;a&gt;Save&lt;/a&gt;&lt;/li&gt;
       *&lt;li&gt;&lt;a&gt;Save as&lt;/a&gt;&lt;/li&gt;
       *&lt;li&gt;&lt;a&gt;Print&lt;/a&gt;&lt;/li&gt;
	   *&lt;/ul&gt;&lt;/li&gt;&lt;/ul&gt;
       *&lt;div id="Ribbon"&gt;&lt;/div&gt; 
	   *&lt;button id="btn"&gt;Home button&lt;/button&gt;
       *&lt;script type="text/javascript"&gt;   
       *$(function () {
       *     $("#Ribbon").ejRibbon({
	   * // Set the width during initialization. 	
       *      width: "100%",
       *        applicationTab: { Type: "ApplicationMenu", itemID: "menu", menuSettings: { openOnClick: false } },
       *        tabs: [{
       *            id: "home", text: "HOME", groups: [{
       *                 text: "New", alignType: ej.Ribbon.alignType.rows,type:"custom",contentID:"btn"
       *            }]
       *         }],
	   *       groupExpand: function (args) {}
       *   });
       * });	        
       * &lt;/script&gt;  
       * @memberof ejRibbon
       * @instance
       */
            groupExpand: null,
            applicationTabClick: null,
			/**		
        * Specify the index of the ribbon tab to select the given index tab item in the ribbon control.
        * @default 1
		* @alias ejRibbon#selectedItemIndex
        * @type {number}
        * @example 
		*&lt;ul id="menu"&gt;
	   *&lt;li&gt;&lt;a&gt;FILE &lt;/a&gt;
       *&lt;ul&gt;
       *&lt;li&gt;&lt;a&gt;New&lt;/a&gt;&lt;/li&gt;
       *&lt;li&gt;&lt;a&gt;Open&lt;/a&gt;&lt;/li&gt;
       *&lt;li&gt;&lt;a&gt;Save&lt;/a&gt;&lt;/li&gt;
       *&lt;li&gt;&lt;a&gt;Save as&lt;/a&gt;&lt;/li&gt;
       *&lt;li&gt;&lt;a&gt;Print&lt;/a&gt;&lt;/li&gt;
	   *&lt;/ul&gt;&lt;/li&gt;&lt;/ul&gt;
       *&lt;div id="Ribbon"&gt;&lt;/div&gt; 
	   *&lt;button id="btn"&gt;Home button&lt;button&gt;
       *&lt;script type="text/javascript"&gt;   
       *$(function () {
       *     $("#Ribbon").ejRibbon({
       *      width: "100%",
	   * // Set the selectedItemIndex during initialization. 
	   *        selectedItemIndex : 2,
       *        applicationTab: { Type: "ApplicationMenu", itemID: "menu", menuSettings: { openOnClick: false } },
       *        tabs: [{
       *            id: "home", text: "HOME", groups: [{
       *                 text: "New", alignType: ej.Ribbon.alignType.rows,type:"custom",contentID:"btn"
       *            }]
       *         },
	   *        {
	   *		id: "insert", text: "INSERT", groups: [{
       *                 text: "Insert group", alignType: ej.Ribbon.alignType.rows,type:"custom",contentID:"insert"
       *            }]
	   *        }
	   *       ]
       *   });
       * });	        
       * &lt;/script&gt;  
       * @memberof ejRibbon
       * @instance
       */
            selectedItemIndex: 1,
			/**		
        * Specify the index or indexes to enable the given index tab or indexes tabs in the ribbon control.
        * @default null
		* @alias ejRibbon#enabledItemIndex
        * @type {array}
        * @example 
		*&lt;ul id="menu"&gt;
	   *&lt;li&gt;&lt;a&gt;FILE &lt;/a&gt;
       *&lt;ul&gt;
       *&lt;li&gt;&lt;a&gt;New&lt;/a&gt;&lt;/li&gt;
       *&lt;li&gt;&lt;a&gt;Open&lt;/a&gt;&lt;/li&gt;
       *&lt;li&gt;&lt;a&gt;Save&lt;/a&gt;&lt;/li&gt;
       *&lt;li&gt;&lt;a&gt;Save as&lt;/a&gt;&lt;/li&gt;
       *&lt;li&gt;&lt;a&gt;Print&lt;/a&gt;&lt;/li&gt;
	   *&lt;/ul&gt;&lt;/li&gt;&lt;/ul&gt;
       *&lt;div id="Ribbon"&gt;&lt;/div&gt; 
	   *&lt;button id="btn"&gt;Home button&lt;button&gt;
	   *&lt;button id="insert"&gt;Insert button&lt;/button&gt;
       *&lt;script type="text/javascript"&gt;   
       *$(function () {
       *     $("#Ribbon").ejRibbon({
       *      width: "100%",
	   * // Set the enabledItemIndex during initialization. 
	   *  enabledItemIndex: [1,2],
       *        applicationTab: { Type: "ApplicationMenu", itemID: "menu", menuSettings: { openOnClick: false } },
       *        tabs: [{
       *            id: "home", text: "HOME", groups: [{
       *                 text: "New", alignType: ej.Ribbon.alignType.rows,type:"custom",contentID:"btn"
       *            }]
       *         },
	   *        {
	   *		id: "insert", text: "INSERT", groups: [{
       *                 text: "Insert group", alignType: ej.Ribbon.alignType.rows,type:"custom",contentID:"insert"
       *            }]
	   *        }
	   *]
       *   });
       * });	        
       * &lt;/script&gt;  
       * @memberof ejRibbon
       * @instance
       */
            enabledItemIndex: [],
			/**		
        * Specify the index or indexes to disable the given index tab or indexes tabs in the ribbon control.
        * @default 0
		* @alias ejRibbon#disabledItemIndex
        * @type {array}
        * @example 
        *&lt;ul id="menu"&gt;
	   *&lt;li&gt;&lt;a&gt;FILE &lt;/a&gt;
       *&lt;ul&gt;
       *&lt;li&gt;&lt;a&gt;New&lt;/a&gt;&lt;/li&gt;
       *&lt;li&gt;&lt;a&gt;Open&lt;/a&gt;&lt;/li&gt;
       *&lt;li&gt;&lt;a&gt;Save&lt;/a&gt;&lt;/li&gt;
       *&lt;li&gt;&lt;a&gt;Save as&lt;/a&gt;&lt;/li&gt;
       *&lt;li&gt;&lt;a&gt;Print&lt;/a&gt;&lt;/li&gt;
	   *&lt;/ul&gt;&lt;/li&gt;&lt;/ul&gt;
       *&lt;div id="Ribbon"&gt;&lt;/div&gt; 
	   *&lt;button id="btn"&gt;Home button&lt;button&gt;
	   *&lt;button id="insert"&gt;Insert button&lt;/button&gt;
       *&lt;script type="text/javascript"&gt;   
       *$(function () {
       *     $("#Ribbon").ejRibbon({
       *      width: "100%",
	   * // Set the disabledItemIndex during initialization. 
	   *  disabledItemIndex: [1,2],
       *        applicationTab: { Type: "ApplicationMenu", itemID: "menu", menuSettings: { openOnClick: false } },
       *        tabs: [{
       *            id: "home", text: "HOME", groups: [{
       *                 text: "New", alignType: ej.Ribbon.alignType.rows,type:"custom",contentID:"btn"
       *            }]
       *         },
	   *        {
	   *		id: "insert", text: "INSERT", groups: [{
       *                 text: "Insert group", alignType: ej.Ribbon.alignType.rows,type:"custom",contentID:"insert"
       *            }]
	   *        }
	   *]
       *   });
       * });	        
       * &lt;/script&gt;  
       * @memberof ejRibbon
       * @instance
       */
            disabledItemIndex: [0],            
            locale: "en-US",
			/**     
   * Triggered before the ribbon control destroy.
   * @event
   * @name ejRibbon#destroy   
   * @param {Object} argument Event parameters from button.     
   * @param {boolean} argument.cancel if the event should be canceled; otherwise, false.
   * @param {object}  argument.model returns the ribbon model.
   * @param {string}  argument.type returns the name of the event.
   * @param {number}  argument.deleteIndex returns current ribbon tab item index     
   * @example 
       *&lt;ul id="menu"&gt;
	   *&lt;li&gt;&lt;a&gt;FILE &lt;/a&gt;
       *&lt;ul&gt;
       *&lt;li&gt;&lt;a&gt;New&lt;/a&gt;&lt;/li&gt;
       *&lt;li&gt;&lt;a&gt;Open&lt;/a&gt;&lt;/li&gt;
       *&lt;li&gt;&lt;a&gt;Save&lt;/a&gt;&lt;/li&gt;
       *&lt;li&gt;&lt;a&gt;Save as&lt;/a&gt;&lt;/li&gt;
       *&lt;li&gt;&lt;a&gt;Print&lt;/a&gt;&lt;/li&gt;
	   *&lt;/ul&gt;&lt;/li&gt;&lt;/ul&gt;
       *&lt;div id="Ribbon"&gt;&lt;/div&gt; 
	   *&lt;button id="btn"&gt;Home button&lt;/button&gt;
       *&lt;script type="text/javascript"&gt;   
       *$(function () {
       *     $("#Ribbon").ejRibbon({
	   * // Set the width during initialization. 	
       *      width: "100%",
       *        applicationTab: { Type: "ApplicationMenu", itemID: "menu", menuSettings: { openOnClick: false } },
       *        tabs: [{
       *            id: "home", text: "HOME", groups: [{
       *                 text: "New", alignType: ej.Ribbon.alignType.rows,type:"custom",contentID:"btn"
       *            }]
       *         }],
	   *        destory: function (args) {}
       *   });
       * });	        
       * &lt;/script&gt;  
       * @memberof ejRibbon
       * @instance
       */
            destory: null,
        },
        dataTypes: {
            tabs: "array",            
            disabledItemIndex: "data",
            enabledItemIndex: "data",
            selectedItemIndex: "number",
        },
		/**
         * destroy the ribbon widget
		 * all events bound using this._on will be unbind automatically and bring the control to pre-init state.
		 * @alias ejRibbon#destroy
		 * @return jQuery
		 * @example 
		 * &lt;script&gt;
		 * var ribbonObj = $("#Ribbon").data("ejRibbon");
		 * ribbonObj.destroy(); // destroy the ribbon
		 * &lt;/script&gt;
		 * @example 
		 * &lt;script&gt;
		 * // destroy the ribbon
		 * $("#Ribbon").ejRibbon("destroy");        
		 * &lt;/script&gt;
         */
        _destroy: function () {
            this.element.ejTab('destroy');
            this.element.find('.e-contextul').removeAttr("style");
        },
        _setModel: function (options) {
            for (var key in options) {
                switch (key) {
                    case "selectedItemIndex":
                        if (this.model.selectedItemIndex > 0 && this._tabObj.model.disabledItemIndex.indexOf(this.model.selectedItemIndex) === -1 && this._ribbonTabs.eq(this.model.selectedItemIndex).is(':visible'))
                            this._tabObj.option({ selectedItemIndex: this.model.selectedItemIndex });
                        break;
                    case "disabledItemIndex":
                        var conParent, disableLen = this.model.disabledItemIndex.length;
                        var curDisabled = this.model.disabledItemIndex, aTag = this.element.find('.e-link'), activeTab;
                        this.model.disabledItemIndex.push(0);
                        this._tabObj.option({ disabledItemIndex: this.model.disabledItemIndex });
                        for (var k = 0; k < disableLen; k++) {
                            conParent = this._ribbonTabs.eq(curDisabled[k]).parent().parent();
                            if (conParent.hasClass('e-contextli') || conParent.hasClass('e-contextliset'))
                                conParent.addClass("e-disable");
                            if (this._tabObj.model.selectedItemIndex === curDisabled[k]) {
                                activeTab = aTag.eq(curDisabled[k]).attr("href");
                                $(activeTab).append($("#" + this._id + "_disabled"));
                                $(activeTab).css("position", "relative");
                                $("#" + this._id + "_disabled").show();
                            }
                        }
                        break;
                    case "enabledItemIndex":
                        var conParent, enableLen = this.model.enabledItemIndex.length;
                        for (var k = 0; k < enableLen; k++) {
                            if (this.model.enabledItemIndex[k] === 0 || !(this._ribbonTabs.eq(this.model.enabledItemIndex[k]).is(':visible'))) {
                                this.model.enabledItemIndex.splice(k, 1);
                                --enableLen;
                                --k;
                            }
                        }
                         						    
                        this._conTabsRemove();
                        this._tabObj.option({ enabledItemIndex: this.model.enabledItemIndex });
                        this._contextualTabs();
                        for (var i = 0; i < this.model.enabledItemIndex.length; i++) {
                            this._tabText.eq(this.model.enabledItemIndex[i]).parent().removeClass("e-disable");
                            this._tabText.eq(this.model.enabledItemIndex[i]).parent().css("position", "");
                            conParent = this._ribbonTabs.eq(this.model.enabledItemIndex[i]).parent().parent();
                            if (conParent.hasClass('e-contextli') || conParent.hasClass('e-contextliset'))
                                conParent.removeClass("e-disable");
                        }

                        $("#" + this._id + "_disabled").hide();
                        this._setTabContentHeight();
                        

                        break;
                }
            }
        },
		
		_setTabContentHeight: function()
		{
			var contents = this.element.find(".e-groupdiv"), contentHeight = [];
			for(var i = 0; i < contents.length; i++)
				contentHeight.push($(contents[i]).height());
			contentHeight.sort();
			this.element.find(".e-content").height(contentHeight[contentHeight.length - 1]);
		},
		
        _init: function () {
            if (!ej.isNullOrUndefined(this.element)) {
                this._render();
                this._wireEvents();
            }
        },
        _render: function () {
            this._renderTab();
            this._initPrivateProperties();
            this._groupingControls();
            this._customization();
			$("#" + this._id).append(ej.buildTag("div#" + this._id + "_disabled"));
    	    var content = $("#" + this._id).find(".e-content");
            $("#" + this._id + "_disabled").height($(content).height()).width(this.model.width);
			$("#" + this._id + "_disabled").css({top:0, left:0 , position: "absolute"}).addClass("e-disable");
			$("#" + this._id + "_disabled").hide();
        },
        _onGroupClick: function (args) {
            this._trigger("groupClick", args);
        },
        _onTabClick: function (args) {
            var expandCollapse = this.element.find('.e-rarrowdown-2x'), active = this.element.find('li.e-active');
            var collapse = this.element.find('.e-collapseactive'), aTag, href;
            var activeContent = this.element.find('.e-active-content');

            if (active.length > 0) {
                active.removeClass("e-active");
                activeContent.removeClass("e-active-content").hide();
            }
            else {
                if (collapse.length > 0) {
                    if (args.activeHeader.textContent === collapse.find('.e-link').text()) {
                        collapse.addClass("e-active").removeClass("e-collapseactive");
                        activeContent.removeClass("e-collapse-content").slideDown("slow");
                    }
                    else {
                        aTag = this.element.find('.e-link');
                        collapse.removeClass("e-collapseactive");
                        this.element.find('.e-collapse-content').removeClass("e-collapse-content");
                        activeContent.removeClass("e-active-content");
                        href = aTag.eq(args.activeIndex).attr("href");
                        aTag.eq(args.activeIndex).addClass("e-active");
                        $(href).slideDown("slow");
                    }
                }
            }
            if (expandCollapse.length > 0) {
                expandCollapse.parent().removeClass("e-collapsed").addClass("e-expanded");
                expandCollapse.removeClass("e-rarrowdown-2x").addClass("e-rarrowup-2x");
                this._isCollapsed = false;
            }
            this._trigger("onTabClick", args);
        },
		_onTabSelect: function (args) {
		    this.model.selectedItemIndex=args.activeIndex;
			this._trigger("onTabSelect", args);
		},
		_onTabAdd: function (args) {
			this._trigger("onTabAdd", args);
		},
		_onTabRemove: function (args) {
			this._trigger("onTabRemove", args);
		},
		_beforeTabRemove: function (args) {
			this._trigger("beforeTabRemove", args);
		},
		_create: function (args) {
			this._trigger("create", args);
		},
		 _OnKeyDown: function (e) {
		 if(e.keyCode === 38 || e.keyCode === 39){
		   if(this.model.tabs.length===this._tabObj.model.selectedItemIndex)
			  e.stopImmediatePropagation();
			}
        },
        _collapse: function () {
            var activeContent = this.element.find('.e-active-content'), togglecontent = this.element.find("#" + this._id + "_togglebutton");
            activeContent.slideUp("fast", "swing");
            this.element.find('li.e-active').removeClass("e-active").addClass("e-collapseactive e-select");
            activeContent.addClass("e-collapse-content");
			this._tabObj.option({ selectedItemIndex: 0 });
            togglecontent.removeClass("e-expanded").addClass("e-collapsed");
            togglecontent.find("span").removeClass("e-rarrowup-2x").addClass("e-rarrowdown-2x");
            this._isCollapsed = true;
        },
        _expand: function () {
            var activeContent = this.element.find('.e-active-content'), togglecontent = this.element.find("#" + this._id + "_togglebutton");
            activeContent.slideDown("fast", "swing");
            this.element.find('.e-collapseactive').addClass("e-active").removeClass("e-collapseactive");
            activeContent.removeClass("e-collapse-content");
            togglecontent.removeClass("e-collapsed").addClass("e-expanded");
            togglecontent.find("span").removeClass("e-rarrowdown-2x").addClass("e-rarrowup-2x");
            this._isCollapsed = false;
        },
        _onGroupExpandClick: function (e) {
            this._trigger("groupExpand", e);
        },
        _onApplicationTabClick: function (args) {
            this._trigger("applicationTabClick", args);
        },
        _initPrivateProperties: function () {
            this._tabUl = this.element.find('.e-header');
            this._applicationTab = this.element.find('.e-apptab');
            this._ribbonTabs = this._tabUl.find(":not('a')");
            this._tabText = this.element.find('.e-link');
            this._id = this.element.attr("id");
            this._tabObj = this.element.data("ejTab");
            this._isCollapsed = false;
			this._contextualTabSet = "";
        },
        _getLocalizedLabels: function (property) {
            return ej.Ribbon.Locale[this.model.locale][property] === undefined ? ej.Ribbon.Locale["en-US"][property] : ej.Ribbon.Locale[this.model.locale][property];
        },
       _customization: function () {
            var toggleelm, model = {},li;
            this._contextualTabs();
            toggleelm = '<div id=' + this._id + '_togglebutton class="e-expanded"><span class="e-icon e-rarrowup-2x"></span></div>';
            li = ej.buildTag("li#" + this._id + "_" + "expandCollapse.e-expandcollapse", toggleelm);
            this._tabUl.append(li);
            this._applicationTab.click($.proxy(this._onApplicationTabClick, this));
			this._applicationTab.find('.e-link').hide();
            if (this.model.applicationTab.menuSettings)
                model = this.model.applicationTab.menuSettings;
            $("#" + this.model.applicationTab.itemID).appendTo(this._applicationTab);
            $("#" + this.model.applicationTab.itemID).ejMenu(model);            
            $("#" + this._id + "_" + this.model.applicationTab.Type.replace(/\s/g, '')).remove();
            this.element.find('.e-content').css("box-sizing","content-box");
        },
		_conTabsRemove:function () {
			var cntxTabs=this.element.find(".e-contextli,.e-contextliset"),$cntxTab;
			for(var i=0;i<cntxTabs.length;i++){
				$cntxTab = $(cntxTabs[i]);
				$cntxTab.is(":visible") ? $cntxTab.find("li").insertAfter($cntxTab.eq(0)) : $cntxTab.find("li").hide().insertAfter($cntxTab.eq(0));
				$cntxTab.remove();
			}				
	    },
		_contextualTabs:function () {
			var curContextualTab, curcontextualTabSet, li, ul;
			if (this.model.contextualTabs.tabs) {
                for (var i = 0; i < this.model.contextualTabs.tabs.length; i++) {
                    curContextualTab = this.model.contextualTabs.tabs[i];
                    this._contextualTab = this._tabUl.children(".e-contextualtab").eq(0);
					this._contextualTab.find("a").css("color",curContextualTab.borderColor);
                    li = ej.buildTag("li.e-contextli");
					if(this._contextualTab.is(":hidden")){
						li.hide();
						this._contextualTab.show();
					}
					if(this._contextualTab.hasClass("e-disable")){
						li.addClass("e-disable");
					}
					this._contextualTab.css("background",curContextualTab.backgroundColor).css("border-color",curContextualTab.borderColor);
                    this._contextualTab.before(li);
                    ul = ej.buildTag("ul.e-contextul");
                    ul.append(this._contextualTab);
                    ul.appendTo(this.element.find('.e-contextli').eq(i));
                }
            }
            if (this.model.contextualTabs.tabSet) {
                for (var i = 0; i < this.model.contextualTabs.tabSet.length; i++) {
                    curcontextualTabSet = this.model.contextualTabs.tabSet[i];
                    li = ej.buildTag("li.e-contextliset"); //.hide();
                    this._contextualTabSet = this._tabUl.children('.e-contextualtabset').eq(0);
					this._contextualTabSet.before(li);
					this._tabUl.children('.e-contextualtabset').find('a').css("color",curcontextualTabSet.borderColor);
                    ul = ej.buildTag("ul.e-contextul", "", {
                        'background': curcontextualTabSet.backgroundColor,
                        'border-color': curcontextualTabSet.borderColor
                    });
                    for (var j = 0; j < curcontextualTabSet.tabs.length; j++) {
                        ul.append(this._tabUl.children('.e-contextualtabset').eq(0));
                    }
                    ul.appendTo(this.element.find('.e-contextliset').eq(i));
                }
            }
		},
        showContextualTab: function (text) {
            this.element.find(".e-contexttitle").filter(function (index) { return $(this).text() === text; }).parent().parent().show();
        },
        hideContextualTab: function (text) {
            this.element.find(".e-contexttitle").filter(function (index) { return $(this).text() === text; }).parent().parent().hide();
            this._tabObj.option({ selectedItemIndex: this._tabObj.model.selectedItemIndex + 1 });
        },
		  /**
       * It is used to hide the given text tab in the ribbon control.
       * @alias hideTab
       * @return jQuery
	   * @param {srting} string  text of the tab item.  
       * @example 	   
       *&lt;ul id="menu"&gt;
	   *&lt;li&gt;&lt;a&gt;FILE &lt;/a&gt;
       *&lt;ul&gt;
       *&lt;li&gt;&lt;a&gt;New&lt;/a&gt;&lt;/li&gt;
       *&lt;li&gt;&lt;a&gt;Open&lt;/a&gt;&lt;/li&gt;
       *&lt;li&gt;&lt;a&gt;Save&lt;/a&gt;&lt;/li&gt;
       *&lt;li&gt;&lt;a&gt;Save as&lt;/a&gt;&lt;/li&gt;
       *&lt;li&gt;&lt;a&gt;Print&lt;/a&gt;&lt;/li&gt;
	   *&lt;/ul&gt;&lt;/li&gt;&lt;/ul&gt;
       *&lt;div id="Ribbon"&gt;&lt;/div&gt; 
	   *&lt;button id="btn"&gt;Home button&lt;/button&gt;
       *&lt;script type="text/javascript"&gt;   
       *$(function () {
       *     $("#Ribbon").ejRibbon({
	   * // Set the width during initialization. 	
       *      width: "100%",
       *        applicationTab: { Type: "ApplicationMenu", itemID: "menu", menuSettings: { openOnClick: false } },
       *        tabs: [{
       *            id: "home", text: "HOME", groups: [{
       *                 text: "New", alignType: ej.Ribbon.alignType.rows,type:"custom",contentID:"btn"
       *            }]
       *         }]
       *   });
       * });	        
       * //initialize the Ribbon object
       *	var ribbonObj = $("#Ribbon").data("ejRibbon");
       *	// hide the given text tab item in the ribbon control.
       *	ribbonObj.hideTab("HOME");
       * &lt;/script&gt;  
       * @example 
	   *&lt;ul id="menu"&gt;
	   *&lt;li&gt;&lt;a&gt;FILE &lt;/a&gt;
       *&lt;ul&gt;
       *&lt;li&gt;&lt;a&gt;New&lt;/a&gt;&lt;/li&gt;
       *&lt;li&gt;&lt;a&gt;Open&lt;/a&gt;&lt;/li&gt;
       *&lt;li&gt;&lt;a&gt;Save&lt;/a&gt;&lt;/li&gt;
       *&lt;li&gt;&lt;a&gt;Save as&lt;/a&gt;&lt;/li&gt;
       *&lt;li&gt;&lt;a&gt;Print&lt;/a&gt;&lt;/li&gt;
	   *&lt;/ul&gt;&lt;/li&gt;&lt;/ul&gt;
       *&lt;div id="Ribbon"&gt;&lt;/div&gt; 
	   *&lt;button id="btn"&gt;Home button&lt;/button&gt;
       *&lt;script type="text/javascript"&gt;   
       *$(function () {
       *     $("#Ribbon").ejRibbon({
	   * // Set the width during initialization. 	
       *      width: "100%",
       *        applicationTab: { Type: "ApplicationMenu", itemID: "menu", menuSettings: { openOnClick: false } },
       *        tabs: [{
       *            id: "home", text: "HOME", groups: [{
       *                 text: "New", alignType: ej.Ribbon.alignType.rows,type:"custom",contentID:"btn"
       *            }]
       *         }]
       *   });
       * });	               
       * $("#Ribbon").ejRibbon("hideTab","HOME"); 
       * &lt;/script&gt;  
       * @memberof ejRibbon
       * @instance
       */
        hideTab: function (text) {
			var index = this._tabText.map(function (index) { if($(this).text() === text)return index; })[0], updatedindex,disLen=this._tabObj.model.disabledItemIndex.length,hide=true;
			for(var i=0;i<disLen;i++){
			if(this._tabObj.model.disabledItemIndex[i]===index)
			  hide=false;
			}
			if(hide){
			if(this._isContextualTab(index))
				this._tabText.filter(function (index) { return $(this).text() === text; }).parents(".e-contextli").first().hide();						
			else
				this._tabText.filter(function (index) { return $(this).text() === text; }).parent().hide();
			updatedindex = this._getVisibleItemIndex(this._tabText);
			}
			if(updatedindex)
				this._tabObj.option("selectedItemIndex", updatedindex);			
        },
		 /**
       * It is used to show the given text tab in the ribbon control.
       * @alias showTab
       * @return jQuery
	   * @param {srting} string  text of the tab item.  
       * @example 	   
       *&lt;ul id="menu"&gt;
	   *&lt;li&gt;&lt;a&gt;FILE &lt;/a&gt;
       *&lt;ul&gt;
       *&lt;li&gt;&lt;a&gt;New&lt;/a&gt;&lt;/li&gt;
       *&lt;li&gt;&lt;a&gt;Open&lt;/a&gt;&lt;/li&gt;
       *&lt;li&gt;&lt;a&gt;Save&lt;/a&gt;&lt;/li&gt;
       *&lt;li&gt;&lt;a&gt;Save as&lt;/a&gt;&lt;/li&gt;
       *&lt;li&gt;&lt;a&gt;Print&lt;/a&gt;&lt;/li&gt;
	   *&lt;/ul&gt;&lt;/li&gt;&lt;/ul&gt;
       *&lt;div id="Ribbon"&gt;&lt;/div&gt; 
	   *&lt;button id="btn"&gt;Home button&lt;/button&gt;
       *&lt;script type="text/javascript"&gt;   
       *$(function () {
       *     $("#Ribbon").ejRibbon({
	   * // Set the width during initialization. 	
       *      width: "100%",
       *        applicationTab: { Type: "ApplicationMenu", itemID: "menu", menuSettings: { openOnClick: false } },
       *        tabs: [{
       *            id: "home", text: "HOME", groups: [{
       *                 text: "New", alignType: ej.Ribbon.alignType.rows,type:"custom",contentID:"btn"
       *            }]
       *         }]
       *   });
       * });	        
       * //initialize the ribbon object
       *	var ribbonObj = $("#Ribbon").data("ejRibbon");
       *	// hide the given text tab item in the ribbon control.
       *	ribbonObj.showTab("HOME");
       * &lt;/script&gt;  
       * @example 
	  *&lt;ul id="menu"&gt;
	   *&lt;li&gt;&lt;a&gt;FILE &lt;/a&gt;
       *&lt;ul&gt;
       *&lt;li&gt;&lt;a&gt;New&lt;/a&gt;&lt;/li&gt;
       *&lt;li&gt;&lt;a&gt;Open&lt;/a&gt;&lt;/li&gt;
       *&lt;li&gt;&lt;a&gt;Save&lt;/a&gt;&lt;/li&gt;
       *&lt;li&gt;&lt;a&gt;Save as&lt;/a&gt;&lt;/li&gt;
       *&lt;li&gt;&lt;a&gt;Print&lt;/a&gt;&lt;/li&gt;
	   *&lt;/ul&gt;&lt;/li&gt;&lt;/ul&gt;
       *&lt;div id="Ribbon"&gt;&lt;/div&gt; 
	   *&lt;button id="btn"&gt;Home button&lt;/button&gt;
       *&lt;script type="text/javascript"&gt;   
       *$(function () {
       *     $("#Ribbon").ejRibbon({
	   * // Set the width during initialization. 	
       *      width: "100%",
       *        applicationTab: { Type: "ApplicationMenu", itemID: "menu", menuSettings: { openOnClick: false } },
       *        tabs: [{
       *            id: "home", text: "HOME", groups: [{
       *                 text: "New", alignType: ej.Ribbon.alignType.rows,type:"custom",contentID:"btn"
       *            }]
       *         }]
       *   });
       * });	               
       * $("#Ribbon").ejRibbon("showTab","HOME"); 
       * &lt;/script&gt;  
       * @memberof ejRibbon
       * @instance
       */
		showTab: function (text) {
			var index = this._tabText.map(function (index) { if($(this).text() === text)return index; })[0];
			if(this._isContextualTab(index))
				this._tabText.filter(function (index) { return $(this).text() === text; }).parents(".e-contextli").first().show();						
			else
				this._tabText.filter(function (index) { return $(this).text() === text; }).parent().show();
        },
		_getVisibleItemIndex:function(items){
			var i = 0;   
			while(i < items.length){
			if($(items[i]).is(":visible"))
				return i;
			i++;
			}			
		},
		_isContextualTab:function(index){
		     var contextualtabs = this.model.contextualTabs.tabs, i=0;
			 if(contextualtabs){
			     while(i < contextualtabs.length){
					if(this.model.tabs[index-1].id === contextualtabs[i].id)
						return true;
					i++;
				 }
			 }
			 return false;
		},
		/**
       * It is used to remove the given index tab item from the ribbon control.
       * @alias removeTab
       * @return jQuery
	   * @param {number} index  index of tab item.  
       * @example 	   
       *&lt;ul id="menu"&gt;
	   *&lt;li&gt;&lt;a&gt;FILE &lt;/a&gt;
       *&lt;ul&gt;
       *&lt;li&gt;&lt;a&gt;New&lt;/a&gt;&lt;/li&gt;
       *&lt;li&gt;&lt;a&gt;Open&lt;/a&gt;&lt;/li&gt;
       *&lt;li&gt;&lt;a&gt;Save&lt;/a&gt;&lt;/li&gt;
       *&lt;li&gt;&lt;a&gt;Save as&lt;/a&gt;&lt;/li&gt;
       *&lt;li&gt;&lt;a&gt;Print&lt;/a&gt;&lt;/li&gt;
	   *&lt;/ul&gt;&lt;/li&gt;&lt;/ul&gt;
       *&lt;div id="Ribbon"&gt;&lt;/div&gt; 
	   *&lt;button id="btn"&gt;Home button&lt;/button&gt;
       *&lt;script type="text/javascript"&gt;   
       *$(function () {
       *     $("#Ribbon").ejRibbon({
	   * // Set the width during initialization. 	
       *      width: "100%",
       *        applicationTab: { Type: "ApplicationMenu", itemID: "menu", menuSettings: { openOnClick: false } },
       *        tabs: [{
       *            id: "home", text: "HOME", groups: [{
       *                 text: "New", alignType: ej.Ribbon.alignType.rows,type:"custom",contentID:"btn"
       *            }]
       *         }]
       *   });
       * });	        
       * //initialize the Ribbon object
       *	var ribbonObj = $("#Ribbon").data("ejRibbon");
       *	// Remove the given index tab item from the ribbon control.
       *	ribbonObj.removeTab(1);
       * &lt;/script&gt;  
       * @example 
	  *&lt;ul id="menu"&gt;
	   *&lt;li&gt;&lt;a&gt;FILE &lt;/a&gt;
       *&lt;ul&gt;
       *&lt;li&gt;&lt;a&gt;New&lt;/a&gt;&lt;/li&gt;
       *&lt;li&gt;&lt;a&gt;Open&lt;/a&gt;&lt;/li&gt;
       *&lt;li&gt;&lt;a&gt;Save&lt;/a&gt;&lt;/li&gt;
       *&lt;li&gt;&lt;a&gt;Save as&lt;/a&gt;&lt;/li&gt;
       *&lt;li&gt;&lt;a&gt;Print&lt;/a&gt;&lt;/li&gt;
	   *&lt;/ul&gt;&lt;/li&gt;&lt;/ul&gt;
       *&lt;div id="Ribbon"&gt;&lt;/div&gt; 
	   *&lt;button id="btn"&gt;Home button&lt;/button&gt;
       *&lt;script type="text/javascript"&gt;   
       *$(function () {
       *     $("#Ribbon").ejRibbon({
	   * // Set the width during initialization. 	
       *      width: "100%",
       *        applicationTab: { Type: "ApplicationMenu", itemID: "menu", menuSettings: { openOnClick: false } },
       *        tabs: [{
       *            id: "home", text: "HOME", groups: [{
       *                 text: "New", alignType: ej.Ribbon.alignType.rows,type:"custom",contentID:"btn"
       *            }]
       *         }]
       *   });
       * });	               
       * $("#Ribbon").ejRibbon("removeTab",1); 
       * &lt;/script&gt;  
       * @memberof ejRibbon
       * @instance
       */
        removeTab: function (tabIndex) {
            var curTab, href;
            if (this._tabObj.model) {
                if (tabIndex > 0) {
                    this._ribbonTabs = this._tabUl.find(".e-link").parent();
                    href = this._ribbonTabs.eq(tabIndex).children().attr("href");
                    curTab = this._ribbonTabs.eq(tabIndex);
                    if (curTab.hasClass("e-contextualtab"))
                        curTab.parent().remove();
                    else if (curTab.hasClass("e-contextualtabset")) {
                        if (curTab.siblings('.e-contextualtabset').length === 0)
                            curTab.parent().remove();
                        else
                            curTab.remove();
                    }
                    else
                        curTab.remove();
                    $(href).remove();
                    if (this._tabObj.model.selectedItemIndex === tabIndex) {
                        this._ribbonTabs = this._tabUl.find(".e-link").parent();
                        curTab = this._ribbonTabs.eq(tabIndex);
                        curTab.addClass("e-active");
                        href = curTab.children().attr("href");
                        $(href).addClass("e-active-content e-activetop").show();
                    }
                }
            }
        },
		/**
       * It is used to set new text to the given text tab in the ribbon control.
       * @alias setTabText
       * @return jQuery
	   * @param {string} string  current text of the tab item.  
	   * @param {string} string  new text of the tab item.  
       * @example 	   
       *&lt;ul id="menu"&gt;
	   *&lt;li&gt;&lt;a&gt;FILE &lt;/a&gt;
       *&lt;ul&gt;
       *&lt;li&gt;&lt;a&gt;New&lt;/a&gt;&lt;/li&gt;
       *&lt;li&gt;&lt;a&gt;Open&lt;/a&gt;&lt;/li&gt;
       *&lt;li&gt;&lt;a&gt;Save&lt;/a&gt;&lt;/li&gt;
       *&lt;li&gt;&lt;a&gt;Save as&lt;/a&gt;&lt;/li&gt;
       *&lt;li&gt;&lt;a&gt;Print&lt;/a&gt;&lt;/li&gt;
	   *&lt;/ul&gt;&lt;/li&gt;&lt;/ul&gt;
       *&lt;div id="Ribbon"&gt;&lt;/div&gt; 
	   *&lt;button id="btn"&gt;Home button&lt;/button&gt;
       *&lt;script type="text/javascript"&gt;   
       *$(function () {
       *     $("#Ribbon").ejRibbon({
	   * // Set the width during initialization. 	
       *      width: "100%",
       *        applicationTab: { Type: "ApplicationMenu", itemID: "menu", menuSettings: { openOnClick: false } },
       *        tabs: [{
       *            id: "home", text: "HOME", groups: [{
       *                 text: "New", alignType: ej.Ribbon.alignType.rows,type:"custom",contentID:"btn"
       *            }]
       *         }]
       *   });
       * });	        
       * //initialize the Ribbon object
       *	var ribbonObj = $("#Ribbon").data("ejRibbon");
       *	// Set new text to the given text tab in the ribbon control.
       *	ribbonObj.setTabText("HOME","NEW");
       * &lt;/script&gt;  
       * @example 
	   *&lt;ul id="menu"&gt;
	   *&lt;li&gt;&lt;a&gt;FILE &lt;/a&gt;
       *&lt;ul&gt;
       *&lt;li&gt;&lt;a&gt;New&lt;/a&gt;&lt;/li&gt;
       *&lt;li&gt;&lt;a&gt;Open&lt;/a&gt;&lt;/li&gt;
       *&lt;li&gt;&lt;a&gt;Save&lt;/a&gt;&lt;/li&gt;
       *&lt;li&gt;&lt;a&gt;Save as&lt;/a&gt;&lt;/li&gt;
       *&lt;li&gt;&lt;a&gt;Print&lt;/a&gt;&lt;/li&gt;
	   *&lt;/ul&gt;&lt;/li&gt;&lt;/ul&gt;
       *&lt;div id="Ribbon"&gt;&lt;/div&gt; 
	   *&lt;button id="btn"&gt;Home button&lt;/button&gt;
       *&lt;script type="text/javascript"&gt;   
       *$(function () {
       *     $("#Ribbon").ejRibbon({
	   * // Set the width during initialization. 	
       *      width: "100%",
       *        applicationTab: { Type: "ApplicationMenu", itemID: "menu", menuSettings: { openOnClick: false } },
       *        tabs: [{
       *            id: "home", text: "HOME", groups: [{
       *                 text: "New", alignType: ej.Ribbon.alignType.rows,type:"custom",contentID:"btn"
       *            }]
       *         }]
       *   });
       * });	               
       * $("#Ribbon").ejRibbon("setTabText","HOME","NEW"); 
       * &lt;/script&gt;  
       * @memberof ejRibbon
       * @instance
       */
        setTabText: function (oldText, newText) {
            this._tabText.filter(function (index) { return $(this).text() === oldText; }).text(newText);
        },
		 /**
       * It is used to get text of the given index tab in the ribbon control.
       * @alias getTabText
       * @return Text of the given index tab in the ribbon control.
	   * @param {number} number  index of the tab item.  
       * @example 	   
       *&lt;ul id="menu"&gt;
	   *&lt;li&gt;&lt;a&gt;FILE &lt;/a&gt;
       *&lt;ul&gt;
       *&lt;li&gt;&lt;a&gt;New&lt;/a&gt;&lt;/li&gt;
       *&lt;li&gt;&lt;a&gt;Open&lt;/a&gt;&lt;/li&gt;
       *&lt;li&gt;&lt;a&gt;Save&lt;/a&gt;&lt;/li&gt;
       *&lt;li&gt;&lt;a&gt;Save as&lt;/a&gt;&lt;/li&gt;
       *&lt;li&gt;&lt;a&gt;Print&lt;/a&gt;&lt;/li&gt;
	   *&lt;/ul&gt;&lt;/li&gt;&lt;/ul&gt;
       *&lt;div id="Ribbon"&gt;&lt;/div&gt; 
	   *&lt;button id="btn"&gt;Home button&lt;/button&gt;
       *&lt;script type="text/javascript"&gt;   
       *$(function () {
       *     $("#Ribbon").ejRibbon({
	   * // Set the width during initialization. 	
       *      width: "100%",
       *        applicationTab: { Type: "ApplicationMenu", itemID: "menu", menuSettings: { openOnClick: false } },
       *        tabs: [{
       *            id: "home", text: "HOME", groups: [{
       *                 text: "New", alignType: ej.Ribbon.alignType.rows,type:"custom",contentID:"btn"
       *            }]
       *         }]
       *   });
       * });	        
       * //initialize the Ribbon object
       *	var ribbonObj = $("#Ribbon").data("ejRibbon");
       *	// To get text of the given index tab in the ribbon control.
       *	ribbonObj.getTabText(1);
       * &lt;/script&gt;  
       * @example 
	   *&lt;ul id="menu"&gt;
	   *&lt;li&gt;&lt;a&gt;FILE &lt;/a&gt;
       *&lt;ul&gt;
       *&lt;li&gt;&lt;a&gt;New&lt;/a&gt;&lt;/li&gt;
       *&lt;li&gt;&lt;a&gt;Open&lt;/a&gt;&lt;/li&gt;
       *&lt;li&gt;&lt;a&gt;Save&lt;/a&gt;&lt;/li&gt;
       *&lt;li&gt;&lt;a&gt;Save as&lt;/a&gt;&lt;/li&gt;
       *&lt;li&gt;&lt;a&gt;Print&lt;/a&gt;&lt;/li&gt;
	   *&lt;/ul&gt;&lt;/li&gt;&lt;/ul&gt;
       *&lt;div id="Ribbon"&gt;&lt;/div&gt; 
	   *&lt;button id="btn"&gt;Home button&lt;/button&gt;
       *&lt;script type="text/javascript"&gt;   
       *$(function () {
       *     $("#Ribbon").ejRibbon({
	   * // Set the width during initialization. 	
       *      width: "100%",
       *        applicationTab: { Type: "ApplicationMenu", itemID: "menu", menuSettings: { openOnClick: false } },
       *        tabs: [{
       *            id: "home", text: "HOME", groups: [{
       *                 text: "New", alignType: ej.Ribbon.alignType.rows,type:"custom",contentID:"btn"
       *            }]
       *         }]
       *   });
       * });	            
       * $("#Ribbon").ejRibbon("getTabText",1); 
       * &lt;/script&gt;  
       * @memberof ejRibbon
       * @instance
       */
        getTabText: function (index) {
            var tabText = this._tabText.eq(index).text();
            return tabText;
        },
		 /**
       * It is used to check given text tab in the ribbon control is visible or not.
       * @alias isVisible
       * @return true if it is visible,false if it is invisible
	   * @param {srting} string  text of the tab item.  
       * @example 	   
       *&lt;ul id="menu"&gt;
	   *&lt;li&gt;&lt;a&gt;FILE &lt;/a&gt;
       *&lt;ul&gt;
       *&lt;li&gt;&lt;a&gt;New&lt;/a&gt;&lt;/li&gt;
       *&lt;li&gt;&lt;a&gt;Open&lt;/a&gt;&lt;/li&gt;
       *&lt;li&gt;&lt;a&gt;Save&lt;/a&gt;&lt;/li&gt;
       *&lt;li&gt;&lt;a&gt;Save as&lt;/a&gt;&lt;/li&gt;
       *&lt;li&gt;&lt;a&gt;Print&lt;/a&gt;&lt;/li&gt;
	   *&lt;/ul&gt;&lt;/li&gt;&lt;/ul&gt;
       *&lt;div id="Ribbon"&gt;&lt;/div&gt; 
	   *&lt;button id="btn"&gt;Home button&lt;/button&gt;
       *&lt;script type="text/javascript"&gt;   
       *$(function () {
       *     $("#Ribbon").ejRibbon({
	   * // Set the width during initialization. 	
       *      width: "100%",
       *        applicationTab: { Type: "ApplicationMenu", itemID: "menu", menuSettings: { openOnClick: false } },
       *        tabs: [{
       *            id: "home", text: "HOME", groups: [{
       *                 text: "New", alignType: ej.Ribbon.alignType.rows,type:"custom",contentID:"btn"
       *            }]
       *         }]
       *   });
       * });	        
       * //initialize the Ribbon object
       *	var ribbonObj = $("#Ribbon").data("ejRibbon");
       *	//  To check given text tab in the ribbon control is visible or not.
       *	ribbonObj.isVisible("HOME");
       * &lt;/script&gt;  
       * @example 
	  *&lt;ul id="menu"&gt;
	   *&lt;li&gt;&lt;a&gt;FILE &lt;/a&gt;
       *&lt;ul&gt;
       *&lt;li&gt;&lt;a&gt;New&lt;/a&gt;&lt;/li&gt;
       *&lt;li&gt;&lt;a&gt;Open&lt;/a&gt;&lt;/li&gt;
       *&lt;li&gt;&lt;a&gt;Save&lt;/a&gt;&lt;/li&gt;
       *&lt;li&gt;&lt;a&gt;Save as&lt;/a&gt;&lt;/li&gt;
       *&lt;li&gt;&lt;a&gt;Print&lt;/a&gt;&lt;/li&gt;
	   *&lt;/ul&gt;&lt;/li&gt;&lt;/ul&gt;
       *&lt;div id="Ribbon"&gt;&lt;/div&gt; 
	   *&lt;button id="btn"&gt;Home button&lt;/button&gt;
       *&lt;script type="text/javascript"&gt;  
       *$(function () {
       *     $("#Ribbon").ejRibbon({
	   * // Set the width during initialization. 	
       *      width: "100%",
       *        applicationTab: { Type: "ApplicationMenu", itemID: "menu", menuSettings: { openOnClick: false } },
       *        tabs: [{
       *            id: "home", text: "HOME", groups: [{
       *                 text: "New", alignType: ej.Ribbon.alignType.rows,type:"custom",contentID:"btn"
       *            }]
       *         }]
       *   });
       * });	              
       * $("#Ribbon").ejRibbon("isVisible","HOME"); 
       * &lt;/script&gt;  
       * @memberof ejRibbon
       * @instance
       */
        isVisible: function (text) {
            var tab = this._tabText.filter(function (index) { return $(this).text() === text; }).parent();
            return (tab.is(':visible'));
        },
		 /**
       * It is used to check given text tab in the ribbon control is enabled or not.
       * @alias isEnable
       * @return true if it is in enabled state,false if it is in disabled state.
	   * @param {srting} string  text of the tab item.  
       * @example 	   
       *&lt;ul id="menu"&gt;
	   *&lt;li&gt;&lt;a&gt;FILE &lt;/a&gt;
       *&lt;ul&gt;
       *&lt;li&gt;&lt;a&gt;New&lt;/a&gt;&lt;/li&gt;
       *&lt;li&gt;&lt;a&gt;Open&lt;/a&gt;&lt;/li&gt;
       *&lt;li&gt;&lt;a&gt;Save&lt;/a&gt;&lt;/li&gt;
       *&lt;li&gt;&lt;a&gt;Save as&lt;/a&gt;&lt;/li&gt;
       *&lt;li&gt;&lt;a&gt;Print&lt;/a&gt;&lt;/li&gt;
	   *&lt;/ul&gt;&lt;/li&gt;&lt;/ul&gt;
       *&lt;div id="Ribbon"&gt;&lt;/div&gt; 
	   *&lt;button id="btn"&gt;Home button&lt;/button&gt;
       *&lt;script type="text/javascript"&gt;   
       *$(function () {
       *     $("#Ribbon").ejRibbon({
	   * // Set the width during initialization. 	
       *      width: "100%",
       *        applicationTab: { Type: "ApplicationMenu", itemID: "menu", menuSettings: { openOnClick: false } },
       *        tabs: [{
       *            id: "home", text: "HOME", groups: [{
       *                 text: "New", alignType: ej.Ribbon.alignType.rows,type:"custom",contentID:"btn"
       *            }]
       *         }]
       *   });
       * });	        
       * //initialize the Ribbon object
       *	var ribbonObj = $("#Ribbon").data("ejRibbon");
       *	//  To check given text tab in the ribbon control is enabled or not.
       *	ribbonObj.isEnable("HOME");
       * &lt;/script&gt;  
       * @example 
	   *&lt;ul id="menu"&gt;
	   *&lt;li&gt;&lt;a&gt;FILE &lt;/a&gt;
       *&lt;ul&gt;
       *&lt;li&gt;&lt;a&gt;New&lt;/a&gt;&lt;/li&gt;
       *&lt;li&gt;&lt;a&gt;Open&lt;/a&gt;&lt;/li&gt;
       *&lt;li&gt;&lt;a&gt;Save&lt;/a&gt;&lt;/li&gt;
       *&lt;li&gt;&lt;a&gt;Save as&lt;/a&gt;&lt;/li&gt;
       *&lt;li&gt;&lt;a&gt;Print&lt;/a&gt;&lt;/li&gt;
	   *&lt;/ul&gt;&lt;/li&gt;&lt;/ul&gt;
       *&lt;div id="Ribbon"&gt;&lt;/div&gt; 
	   *&lt;button id="btn"&gt;Home button&lt;/button&gt;
       *&lt;script type="text/javascript"&gt;   
       *$(function () {
       *     $("#Ribbon").ejRibbon({
	   * // Set the width during initialization. 	
       *      width: "100%",
       *        applicationTab: { Type: "ApplicationMenu", itemID: "menu", menuSettings: { openOnClick: false } },
       *        tabs: [{
       *            id: "home", text: "HOME", groups: [{
       *                 text: "New", alignType: ej.Ribbon.alignType.rows,type:"custom",contentID:"btn"
       *            }]
       *         }]
       *   });
       * });	           
       * $("#Ribbon").ejRibbon("isEnable","HOME"); 
       * &lt;/script&gt;  
       * @memberof ejRibbon
       * @instance
       */
        isEnable: function (text) {
            var isEnable = true, tabIndex = this._tabText.filter(function (index) { return $(this).text() === text; }).parent().index();
            for (var i = 0; i < this.model.disabledItemIndex.length; i++) {
                if (this.model.disabledItemIndex[i] == tabIndex)
                    isEnable = false;
            }
            return isEnable;
        },
        _renderTab: function () {
            var ul = ej.buildTag("ul"), li, aTag, appTab = this.model.applicationTab.Type, tabs,contextualTabs,contextualTabSet, i, j;
            aTag = ej.buildTag("a", "", {}, { href: "#" + this._id + "_" + appTab.replace(/\s/g, '') });
            li = ej.buildTag("li.e-apptab", aTag);
            ul.append(li);

            for (i = 0; i < this.model.tabs.length; i++) {
                tabs = this.model.tabs[i];
                aTag = ej.buildTag("a", tabs.text, {}, { href: "#" + this._id + "_" + tabs.id });
                li = ej.buildTag("li", aTag).addClass("e-tab");
                ul.append(li);
            }
            if (this.model.contextualTabs.tabs) {
                for (i = 0; i < this.model.contextualTabs.tabs.length; i++) {
                    contextualTabs = this.model.contextualTabs.tabs[i];
                    aTag = ej.buildTag("a", contextualTabs.text, {}, { href: "#" + this._id + "_" + contextualTabs.id });
                    li = ej.buildTag("li", aTag).addClass("e-contextualtab");
                    ul.append(li);
                }
            }
            if (this.model.contextualTabs.tabSet) {
                for (i = 0; i < this.model.contextualTabs.tabSet.length; i++) {
                    for (j = 0; j < this.model.contextualTabs.tabSet[i].tabs.length; j++) {
                        contextualTabSet = this.model.contextualTabs.tabSet[i].tabs[j];
                        aTag = ej.buildTag("a", contextualTabSet.text, {}, { href: "#" + this._id + "_" + contextualTabSet.id });
                        li = ej.buildTag("li", aTag).addClass("e-contextualtabset");
                        ul.append(li);
                    }
                }
            }
            this.element.append(ul);
            this.element.ejTab({ width:this.model.width,disabledItemIndex: this.model.disabledItemIndex, enabledItemIndex: this.model.enabledItemIndex, itemActive: $.proxy(this._onTabSelect, this), beforeActive: $.proxy(this._onTabClick, this), itemAdd: $.proxy(this._onTabAdd, this), itemRemove: $.proxy(this._onTabRemove, this), beforeItemRemove: $.proxy(this._beforeTabRemove, this), create: $.proxy(this._create, this), selectedItemIndex: this.model.selectedItemIndex });
        },
        _groupingControls: function () {
            var tabLen = this.model.tabs.length, contextualTab, tabs, groupDiv, innerDiv, innerDivCustom, innerDivChild, contDiv, centerAlign = false, groupexpander, captionArea,contentbottom,model = {},tabcont;
            var m = 0, split, button, dropdown, separatorDiv, lastControl, t, k, l, i, j;
			if (this.model.contextualTabs.tabs) {
			for (var t = 0; t < this.model.contextualTabs.tabs.length; t++) {
				this.model.tabs[tabLen] = this.model.contextualTabs.tabs[t];
				++tabLen;
			  }
			}
			if (this.model.contextualTabs.tabSet) {
			for (var k = 0; k < this.model.contextualTabs.tabSet.length; k++) {
				contextualTab = this.model.contextualTabs.tabSet[k];
				for (var l = 0; l < contextualTab.tabs.length; l++) {
					this.model.tabs[tabLen] = contextualTab.tabs[l];
				++tabLen;
				}
			  }
			}
            for (i = 0; i < this.model.tabs.length; i++) {
                tabs = this.model.tabs[i];
                for (j = 0; j < tabs.groups.length; j++) {
                    contDiv = ej.buildTag("div#" + this._id + "_" + tabs.id + "_" + tabs.groups[j].text.replace(/\s/g, '') + "_content");
                    if (tabs.groups[j].type === "custom") {
                        if (!tabs.groups[j].contentID)
                            innerDivCustom = ej.buildTag("div#" + this._id + "_" + tabs.id + "_" + tabs.groups[j].text.replace(/\s/g, '') + "_" + tabs.groups[j].type + "_" + j, tabs.groups[j].content).addClass("e-innerdiv");
                        else {
                            innerDivCustom = ej.buildTag("div#" + this._id + "_" + tabs.id + "_" + tabs.groups[j].text.replace(/\s/g, '') + "_" + tabs.groups[j].type + "_" + j).addClass("e-innerdiv");
                            $("#" + tabs.groups[j].contentID).appendTo(innerDivCustom);
                        }
                        contDiv.append(innerDivCustom);
                    }
                    else {
                        if (tabs.groups[j].content) {
                            for (var l = 0; l < tabs.groups[j].content.length; l++) {
                                var subGroup = tabs.groups[j].content[l].groups;
                                var defaults = tabs.groups[j].content[l].defaults;
                                if (tabs.groups[j].content.length == 1)
                                    centerAlign = true;
                                if (tabs.groups[j].alignType === ej.Ribbon.alignType.columns)
                                    innerDiv = ej.buildTag("div#" + this._id + "_" + tabs.id + "_" + tabs.groups[j].text.replace(/\s/g, '') + "_" + m).addClass("e-innerdiv");
                                else if (tabs.groups[j].alignType === ej.Ribbon.alignType.rows) {
                                    innerDiv = ej.buildTag("div#" + this._id + "_" + tabs.id + "_" + tabs.groups[j].text.replace(/\s/g, '') + "_" + m).addClass("e-innerdivrow");
                                    ++m;
                                }
                                for (var m = 0; m < subGroup.length; m++) {
                                    if (ej.isNullOrUndefined(subGroup[m].type) && !ej.isNullOrUndefined(defaults))
                                        subGroup[m].type = defaults.type;
                                    if (ej.isNullOrUndefined(subGroup[m].isBig) && !ej.isNullOrUndefined(defaults))
                                        subGroup[m].isBig = defaults.isBig;
                                    if (ej.isNullOrUndefined(subGroup[m].height) && !ej.isNullOrUndefined(defaults))
                                        subGroup[m].height = defaults.height;
                                    if (ej.isNullOrUndefined(subGroup[m].width) && !ej.isNullOrUndefined(defaults))
                                        subGroup[m].width = defaults.width;
                                    if (subGroup[m].type === ej.Ribbon.type.splitButton) {
                                        split = subGroup[m];
                                        this._splitButtonControl = ej.buildTag("button#" + this._id + "_" + split.id, split.text).addClass(split.cssClass);
                                        if (tabs.groups[j].alignType === ej.Ribbon.alignType.columns)
                                            innerDivChild = ej.buildTag("div#" + this._id + "_" + tabs.id + "_" + tabs.groups[j].text.replace(/\s/g, '') + "_" + split.id, this._splitButtonControl).addClass("e-controlpadding").attr("title", split.toolTip);
                                        else if (tabs.groups[j].alignType === ej.Ribbon.alignType.rows)
                                            innerDivChild = ej.buildTag("div#" + this._id + "_" + tabs.id + "_" + tabs.groups[j].text.replace(/\s/g, '') + "_" + split.id, this._splitButtonControl).addClass("e-innerdivchild e-controlpadding").attr("title", split.toolTip);
                                         model = {}
                                        if (split.buttonSettings)
                                            model = split.buttonSettings;
                                         model.height = ej.isNullOrUndefined(model.height)?  subGroup[m].height :model.height;
                                        model.width = ej.isNullOrUndefined(model.width)? subGroup[m].width:model.width;
                                        this._splitButtonControl.ejSplitButton(model);
                                        (split.isBig) ? innerDivChild.children().removeAttr("style").addClass("e-big") : innerDivChild.children().removeAttr("style").addClass("e-small");
                                        innerDiv.append(innerDivChild);
                                        if (subGroup[m].enableSeparator) {
                                            if (tabs.groups[j].alignType === ej.Ribbon.alignType.rows) {
                                                separatorDiv = ej.buildTag("div#" + this._id + "_" + "separator" + "_" + split.id).addClass("e-separatordivrow");
												(!ej.isNullOrUndefined(defaults.height))?separatorDiv.height(defaults.height):(split.isBig) ? separatorDiv.addClass("e-big") : separatorDiv.addClass("e-small");
                                                innerDiv.append(separatorDiv);
                                            }
                                        }
                                    }
                                    else if (subGroup[m].type === ej.Ribbon.type.button) {
                                        button = subGroup[m];
                                        this._buttonControl = ej.buildTag("button#" + this._id + "_" + button.id, button.text).addClass(button.cssClass);
                                        if (tabs.groups[j].alignType === ej.Ribbon.alignType.columns)
                                            innerDivChild = ej.buildTag("div#" + this._id + "_" + tabs.id + "_" + tabs.groups[j].text.replace(/\s/g, '') + "_" + button.id, this._buttonControl).addClass("e-controlpadding").attr("title", button.toolTip);
                                        else if (tabs.groups[j].alignType === ej.Ribbon.alignType.rows)
                                            innerDivChild = ej.buildTag("div#" + this._id + "_" + tabs.id + "_" + tabs.groups[j].text.replace(/\s/g, '') + "_" + button.id, this._buttonControl).addClass("e-innerdivchild e-controlpadding").attr("title", button.toolTip);
                                         model = {}
                                        if (button.buttonSettings)
                                            model = button.buttonSettings;
                                         model.height = ej.isNullOrUndefined(model.height)?  subGroup[m].height :model.height;
                                        model.width = ej.isNullOrUndefined(model.width)? subGroup[m].width:model.width;
                                        this._buttonControl.ejButton(model);
                                        (button.isBig) ? innerDivChild.children().addClass("e-big") : innerDivChild.children().addClass("e-small");
                                        innerDiv.append(innerDivChild);
                                        if (subGroup[m].enableSeparator) {
                                            if (tabs.groups[j].alignType === ej.Ribbon.alignType.rows) {
                                                separatorDiv = ej.buildTag("div#" + this._id + "_" + "separator" + "_" + button.id).addClass("e-separatordivrow");
												(!ej.isNullOrUndefined(defaults.height))?separatorDiv.height(defaults.height):(button.isBig) ? separatorDiv.addClass("e-big") : separatorDiv.addClass("e-small");
                                                innerDiv.append(separatorDiv);
                                            }
                                        }
                                    }
                                    else if (subGroup[m].type === ej.Ribbon.type.dropDownList) {
                                        dropdown = subGroup[m];
                                        this._dropdownControl = ej.buildTag("input#" + this._id + "_" + dropdown.id, { type: "text" }).addClass(dropdown.cssClass);
                                        var innerDivChild;
                                        if (tabs.groups[j].alignType === ej.Ribbon.alignType.columns && dropdown.dropdownSettings)
                                            innerDivChild = ej.buildTag("div#" + this._id + "_" + tabs.id + "_" + tabs.groups[j].text.replace(/\s/g, '') + "_" + dropdown.id, this._dropdownControl).append($("#" + dropdown.dropdownSettings.targetID)).addClass("e-controlpadding").attr("title", dropdown.toolTip);
                                        else if (tabs.groups[j].alignType === ej.Ribbon.alignType.rows && dropdown.dropdownSettings)
                                            innerDivChild = ej.buildTag("div#" + this._id + "_" + tabs.id + "_" + tabs.groups[j].text.replace(/\s/g, '') + "_" + dropdown.id, this._dropdownControl).append($("#" + dropdown.dropdownSettings.targetID)).addClass("e-innerdivchild e-controlpadding").attr("title", dropdown.toolTip);
                                         model = {};
                                        if (dropdown.dropdownSettings)
                                            model = dropdown.dropdownSettings;
                                          model.height = ej.isNullOrUndefined(model.height)?  subGroup[m].height :model.height;
                                        model.width = ej.isNullOrUndefined(model.width)? subGroup[m].width:model.width;
                                        this._dropdownControl.ejDropDownList(model);
                                        innerDiv.append(innerDivChild);
                                        if (subGroup[m].enableSeparator) {
                                            if (tabs.groups[j].alignType === ej.Ribbon.alignType.rows) {
                                                separatorDiv = ej.buildTag("div#" + this._id + "_" + "separator" + "_" + dropdown.id).addClass("e-separatordivrow");
												(!ej.isNullOrUndefined(defaults.height))?separatorDiv.height(defaults.height):separatorDiv.addClass("e-small");
                                                innerDiv.append(separatorDiv);
                                            }
                                        }
                                    }
                                    contDiv.append(innerDiv);
                                }
                            }
                        }
                    }
                    contentbottom = ej.buildTag("div.e-contentbottom");
                    captionArea = ej.buildTag("div.e-captionarea", this.model.tabs[i].groups[j].text);
                    contentbottom.append(captionArea);
                    if (this.model.tabs[i].groups[j].enableGroupExpander) {
                        groupexpander = ej.buildTag("div#" + this._id + "_" + this.model.tabs[i].groups[j].text + "_e-groupexpander.e-icon e-expander e-groupexpander");
                        contentbottom.append(groupexpander);
                    }
                    groupDiv = ej.buildTag("div#" + this._id + "_" + tabs.id + "_" + tabs.groups[j].text.replace(/\s/g, '')).addClass("e-groupdiv").click($.proxy(this._onGroupClick, this));
                    if (centerAlign) {
                        groupDiv.addClass("e-centeralign");
                        centerAlign == false;
                    }
                    tabs.groups[j].width && groupDiv.width(tabs.groups[j].width);
                    groupDiv.append(contDiv);
                    groupDiv.append(contentbottom);
                    tabcont = this.element.find("#" + this._id + "_" + tabs.id);
					tabcont.append(groupDiv).addClass("e-parentdiv");
                }
            }
			this._setTabContentHeight();
        },

        _tabExpandCollapse: function (e) {
            var target, args, action;
            target = e.target.tagName == "div" ? e.target : e.target.parentNode;
            action = target.className == "e-expanded" ? "collapse" : "expand";
            args = { target: $(target), type: e.type, action: action, cancel: false };
            this._trigger("onToggleButtonClick", args);
            if (args.cancel)
                return;
            target.className == "e-expanded" ? this._collapse() : this._expand();
        },

        _wireEvents: function () {
		    this._on(this.element, "keydown", this._OnKeyDown);
            this._on(this.element.find("#" + this._id + "_togglebutton"), ej.eventType.click, this._tabExpandCollapse);
            this._on(this.element.find(".e-groupexpander"), ej.eventType.click, this._onGroupExpandClick);
        }
    });
    ej.Ribbon.alignType = {
        rows: "rows",
        columns: "columns"
    };
    ej.Ribbon.type = {
        button: "button",
        splitButton: "splitbutton",
        dropDownList: "dropdownlist"
    };
    ej.Ribbon.Locale = {};

    ej.Ribbon.Locale["en-US"] = {
        ExpandCollapse: "Expand Collapse"
    };
})(jQuery, Syncfusion);;
/**
* @fileOverview Plugin to style the Html ReportViewer elements
* @copyright Copyright Syncfusion Inc. 2001 - 2013. All rights reserved.
*  Use of this code is subject to the terms of our license.
*  A copy of the current license can be obtained at any time by e-mailing
*  licensing@syncfusion.com. Any infringement will be prosecuted under
*  applicable laws. 
* @version 12.1 
* @author &lt;a href="mailto:licensing@syncfusion.com"&gt;Syncfusion Inc&lt;/a&gt;
*/

(function ($, ej, undefined) {

   /**
   * @namespace ej
        * @classdesc Custom Design for Html ReportViewer control.
        * @class ejReportViewer
        * @requires jQuery
        * @requires jquery.easing.1.3.js
        * @requires jquery.globalize.js
        * @requires jquery.easing.1.3.js
        * @requires jsrender.js
        * @requires ej.core.js
        * @requires ej.treeview.js
        * @requires ej.dropdownlist.js
        * @requires ej.checkbox.js
        * @requires ej.waitingpopup.js
        * @requires ej.toolbar.js
        * @requires ej.radiobutton.js
        * @requires ej.splitter.js
        * @requires ej.button.js
        * @requires ej.chart.js
        * @requires ej.datepicker.js
        * @requires ej.map.js
        * @requires ej.touch.js
        * @requires ej.data.js
        * @requires ej.circulargauge.js
        * @requires ej.lineargauge.js   
        * @requires ej.reportviewer.js

         * @example
         * &lt;div id="reportviewer"&gt;ReportViewer&lt;/div&gt; <br> 
         * &lt;script&gt;
         * // Create ReportViewer
         * $('#reportviewer').ejReportViewer(); 	
         * &lt;/script&gt;
    */

    ej.widget("ejReportViewer", "ej.ReportViewer", {

        //Root Css Class
        _rootCSS: "e-reportviewer",
        // widget element will be automatically set in this
        element: null,
        // user defined model will be automatically set in this
        model: null,
        validTags: ["div"],

        //#region Defaults
        defaults: {
            /**		
            * Specifies the report Web API service url.
            * @default ""
            * @type {string}
            * @example
            * &lt;div id="reportviewer"&gt;&lt;/div&gt; 
            * &lt;script&gt;
            * 	$("#reportviewer").ejReportViewer({ reportServiceUrl:  "../api/RDLReport" });            
            * &lt;/script&gt;
			* @memberof ejReportViewer
			* @instance
            */
            reportServiceUrl: "",
            /**		
            * Gets or sets the path of the report file.
            * @default ""
            * @type {string}
            * @example
            * &lt;div id="reportviewer"&gt;&lt;/div&gt; 
            * &lt;script&gt;
            * 	$("#reportviewer").ejReportViewer({ reportPath: "~/App_Data/Sample.rdl" });            
            * &lt;/script&gt;
			* @memberof ejReportViewer
			* @instance
            */
            reportPath: "",
            /**		
            * Gets or sets the reports server url.
			* @default ""
            * @type {string}
            * @example 
            * &lt;div id="reportviewer"&gt;&lt;/div&gt; 
            * &lt;script&gt;
            * 	$("#reportviewer").ejReportViewer({ reportServerUrl:  "http://172.16.7.164/ReportServer/SampleReport.rdl" });            
            * &lt;/script&gt;
			* @memberof ejReportViewer
			* @instance
            */
            reportServerUrl: "",
            /**		
            * Gets or sets the list of data sources for the RDLC report.
            * @default []
            * @type {array}
            * @name ejReportViewer#dataSources
            * @example 
            * &lt;div id="reportviewer"&gt;&lt;/div&gt; 
            * &lt;script&gt;
            * 	$("#reportviewer").ejReportViewer({ dataSources: [{name:"Menu Items",
            *                         values:[{ OrderId: "21D60", FoodName: "Burger", Price: 20, Category: "Veg" },
            *                                 { OrderId: "21D61", FoodName: "Pizza", Price: 25, Category: "Non-Veg" },
            *                                 { OrderId: "21D63", FoodName: "Sandwiches", Price: 30, Category: "Non-Veg" },
            *                                 { OrderId: "21D65", FoodName: "Chicken Drum Sticks", Price: 23, Category: "Non-Veg" },
            *                                 { OrderId: "21D64", FoodName: "Fulka", Price: 15, Category: "Veg" }]}]
            *                                 });
            * &lt;/script&gt;
            */
            dataSources: [],
            /**		
            * Gets or sets the list of parameters associated with the report.
            * @default []
            * @type {array}
            * @name ejReportViewer#parameters
            * @example 
            * &lt;div id="reportviewer"&gt;&lt;/div&gt; 
            * &lt;script&gt;
            * 	$("#reportviewer").ejReportViewer({ parameters: [{
            *                                 name:"Vehicle",
            *                                 labels:["Motor Bikes"],
            *                                 prompt:"Please select the color",
            *                                 values:["Red","Green","Blue","Yellow","Black"],
            *                                 nullable:false
            *                                  }]});
            * &lt;/script&gt;
            */
            parameters: [],

           /**		
              * Specifies the export settings. 
              * @type {object}
              * @memberof ejReportViewer
              * @instance
            */							
            exportSettings: {
                /**		
                 * Specifies the export formats.
                 * @alias ejReportViewer#exportSettings->exportOptions
                 * @default ej.ReportViewer.ExportOptions.All
                 * @type {enum}
                 * @example 
                 * &lt;div id="reportviewer"&gt;&lt;/div&gt; 
                 * &lt;script&gt;        
                 *	$("#reportviewer").ejReportViewer(
                 *             {
                 *                exportSettings:{ exportOptions: ej.ReportViewer.ExportOptions.Html | ej.ReportViewer.ExportOptions.Pdf }
                 *             });            
                 * &lt;/script&gt;
                 * @memberof ejReportViewer
                 * @instance
               */
                exportOptions: 1 | 2 | 4 | 8,
                /**		
                 * Specifies the excel export format.
                 * @alias ejReportViewer#exportSettings->excelFormat
                 * @default ej.ReportViewer.ExcelFormats.Excel97to2003
                 * @type {enum}
                 * @example 
                 * &lt;div id="reportviewer"&gt;&lt;/div&gt; 
                 * &lt;script&gt;        
                 *	$("#reportviewer").ejReportViewer(
                 *             {
                 *                exportSettings:{ excelFormat: ej.ReportViewer.ExcelFormats.Excel97to2003}
                 *             });            
                 * &lt;/script&gt;
                 * @memberof ejReportViewer
                 * @instance
               */
                excelFormat: "excel97to2003",
                 /**		
                 * Specifies the word export format.
                 * @alias ejReportViewer#exportSettings->wordFormat
                 * @default ej.ReportViewer.WordFormats.Doc
                 * @type {enum}
                 * @example 
                 * &lt;div id="reportviewer"&gt;&lt;/div&gt; 
                 * &lt;script&gt;        
                 *	$("#reportviewer").ejReportViewer(
                 *             {
                 *                exportSettings:{ wordFormat: ej.ReportViewer.WordFormats.Doc}
                 *             });            
                 * &lt;/script&gt;
                 * @memberof ejReportViewer
                 * @instance
               */
                wordFormat: "doc"
            },

            /**		
              * Specifies the toolbar settings. 
              * @type {object}
              * @memberof ejReportViewer
              * @instance
            */							
            toolbarSettings: {
                /**		
                  * Specifies the toolbar items.
				  * @alias ejReportViewer#toolbarSettings->items
                  * @default ej.ReportViewer.ToolbarItems.All
                  * @type {enum}
                  * @example 
                  * &lt;div id="reportviewer"&gt;&lt;/div&gt; 
                  * &lt;script&gt;        
                  *	$("#reportviewer").ejReportViewer(
				  *             {
				  *                toolbarSettings:{ exportOptions: ej.ReportViewer.ToolbarItems.All }
				  *             });
                  * &lt;/script&gt;
				  * @memberof ejReportViewer
			      * @instance
                */
                items: 1 | 2 | 4 | 8 | 16 | 32 | 64 | 128,
                /**		
                  * Shows or hides the toolbar.
				  * @alias ejReportViewer#toolbarSettings->showToolbar
                  * @default  true
                  * @type {boolean}
                  * @example 
                  * &lt;div id="reportviewer"&gt;&lt;/div&gt; 
                  * &lt;script&gt;		
                  *	$("#reportviewer").ejReportViewer(
                  *             {
                  *                toolbarSettings:{showToolbar: true}
                  *             });		
                  * &lt;/script&gt;
				  * @memberof ejReportViewer
			      * @instance
                */
                showToolbar: true,
		        /**		
                  * Specifies the toolbar template ID..
				  * @alias ejReportViewer#toolbarSettings->templateId
                  * @default  null
                  * @type {string}
                  * @example 
                  * &lt;div id="reportviewer"&gt;&lt;/div&gt; 
                  * &lt;script&gt;		
                  *	$("#reportviewer").ejReportViewer(
                  *             {
                  *                toolbarSettings:{templateId: ""}
                  *             });		
                  * &lt;/script&gt;
				  * @memberof ejReportViewer
			      * @instance
                */
                templateId: "",
		        /**		
                  * Fires when user click on toolbar item in the toolbar.
				  * @alias ejReportViewer#toolbarSettings->click
                  * @default  null
                  * @type {string}
                  * @example 
                  * &lt;div id="reportviewer"&gt;&lt;/div&gt; 
                  * &lt;script&gt;		
                  *	$("#reportviewer").ejReportViewer(
                  *             {
                  *                toolbarSettings:{click: "onToolbarClick"}
                  *             });		
                  * &lt;/script&gt;
				  * @memberof ejReportViewer
			      * @instance
                */
                click: "",

		         /**		
                  * Shows or hides the tooltip of toolbar items.
	              * @alias ejReportViewer#toolbarSettings->showTooltip
                  * @default  true
                  * @type {boolean}
                  * @example 
                  * &lt;div id="reportviewer"&gt;&lt;/div&gt; 
                  * &lt;script&gt;		
                  *	$("#reportviewer").ejReportViewer(
                  *             {
                  *                toolbarSettings:showTooltip: true}
                  *             });		
                  * &lt;/script&gt;
				  * @memberof ejReportViewer
			      * @instance
                */
                showTooltip: true
            },
			
			/**		
            * Specifies the locale for report viewer.
            * @default  "en-US"
            * @type {string}
            * @example 
            * &lt;div id="reportviewer"&gt;&lt;/div&gt; 
            * &lt;script&gt;		
            * 	$("#reportviewer").ejReportViewer(
            *                 {
            *                    locale: "it-IT"
            *                 });		
            * &lt;/script&gt;
		    * @memberof ejReportViewer
		    * @instance
            */
            locale: "en-US",
			
            /**		
            * Enables and disables the print mode.
            * @default  false
            * @type {boolean}
            * @example 
            * &lt;div id="reportviewer"&gt;&lt;/div&gt; 
            * &lt;script&gt;		
            * 	$("#reportviewer").ejReportViewer(
            *                 {
            *                    printMode:true
            *                 });		
            * &lt;/script&gt;
		    * @memberof ejReportViewer
		    * @instance
            */
            printMode: false,
            /**		
            * Specifies the processing mode of the report.
            * @default ej.ReportViewer.ProcessingMode.Remote
            * @type {enum}
            * @example 
            * &lt;div id="reportviewer"&gt;&lt;/div&gt; 
            * &lt;script&gt;        
            * 	$("#reportviewer").ejReportViewer({ processingMode: ej.ReportViewer.ProcessingMode.Remote });            
            * &lt;/script&gt;
		    * @memberof ejReportViewer
		    * @instance
            */
            processingMode: "remote",
            /**		
            * Gets or sets the zoom factor for report viewer.
            * @default 1
            * @type {number}
            * @example 
            * &lt;div id="reportviewer"&gt;&lt;/div&gt; 
            * &lt;script&gt;        
            * 	$("#reportviewer").ejReportViewer({ zoomFactor:  2 });            
            * &lt;/script&gt;
		    * @memberof ejReportViewer
		    * @instance
            */
            zoomFactor: 1,            
            /**		
            * Fires when the report is loaded.If you want to perform any operation after the successful loading of report, you can make use of the reportLoaded event. 
			* @event
			* @name ejReportViewer#reportLoaded		
			* @param {Object} argument event parameters from reportviewer     
			* @param {boolean} argument.cancel true if the event should be canceled; otherwise, false.
			* @param {object}  argument.model returns the report model
			* @param {string}  argument.type returns the name of the event
            * @example 
            * &lt;div id="reportviewer"&gt;&lt;/div&gt; 
            * &lt;script&gt;        
            * 	$("#reportviewer").ejReportViewer({
		    *  	  reportLoaded: function(args)
		    *             {
            *        		 // Write a code block to perform any action when the report is loaded successfully.
		    *             }
		    *     });             
            * &lt;/script&gt;
		    * @memberof ejReportViewer
		    * @instance
            */
            reportLoaded: null,
            /**		
            * Fires before report rendering is completed.If you want to perform any operation before the rendering of report,you can make use of the renderingBegin event. 
			* @event
			* @name ejReportViewer#renderingBegin		
			* @param {Object} argument Event parameters from reportviewer     
			* @param {boolean}  argument.cancel true if the event should be canceled; otherwise, false.
			* @param {object}  argument.model returns the report model
			* @param {string}  argument.type returns the name of the event
            * @example 
            * &lt;div id="reportviewer"&gt;&lt;/div&gt; 
            * &lt;script&gt;  
            * //rendering begin event for report.		  
            * 	$("#reportviewer").ejReportViewer({
		    *        renderingBegin:function(args)
		    *             {
            *        		 // Write a code block to perform any operation before rendering.
		    *             }
		    *     });                 
            * &lt;/script&gt;
		    * @memberof ejReportViewer
		    * @instance
            */
            renderingBegin: null,
            /**		
		    * Fires after report rendering completed.If you want to perform any operation after the rendering of report,you can make use of this renderingComplete event. 
			* @event
			* @name ejReportViewer#renderingComplete		
			* @param {Object} argument Event parameters from reportviewer     
			* @param {boolean}  argument.cancel true if the event should be canceled; otherwise, false.
			* @param {object}  argument.model returns the report model
			* @param {string}  argument.type returns the name of the event
            * @example 
            * &lt;div id="reportviewer"&gt;&lt;/div&gt; 
            * &lt;script&gt;        
		    * //rendering complete event for reportviewer control.
            * 	$("#reportviewer").ejReportViewer({ 
		    *       renderingComplete:function(args)
		    *             {
            *        		 // Write a code block to perform any operation after rendering completed.
		    *             }
		    *     });            
            * &lt;/script&gt;
		    * @memberof ejReportViewer
		    * @instance
            */
            renderingComplete: null,
            /**		
            * Fires when any error occurred while rendering the report.If you want to perform any operation when an error occurs in the report, you can make use of the reportError event.
			* @event
			* @name ejReportViewer#reportError		
			* @param {Object} argument Event parameters from reportviewer     
			* @param {boolean}  argument.cancel true if the event should be canceled; otherwise, false.
			* @param {string}  argument.error returns the error details
			* @param {object}  argument.model returns the report model
			* @param {string}  argument.type returns the name of the event         
            * @example 
            * &lt;div id="reportviewer"&gt;&lt;/div&gt; 
            * &lt;script&gt;        
            * 	$("#reportviewer").ejReportViewer({
		    *		  reportError: function (args) {
            *       // Write a code block to perform any operation when report error occurs.
            *       }
            *     });                         
            * &lt;/script&gt;
		    * @memberof ejReportViewer
		    * @instance
            */
            reportError: null,
            /**		
            * Fires during drill through action done in report.If you want to perform any operation when a drill through action is performed, you can make use of the drillThrough event.
			* @event
			* @name ejReportViewer#drillThrough		
			* @param {Object} argument Event parameters from reportviewer     
			* @param {boolean}  argument.cancel true if the event should be canceled; otherwise, false.
			* @param {Object} argument.actionInfo returns the actionInfo's parameters bookmarkLink,hyperLink,reportName,parameters.
			* @param {object}  argument.model returns the report model
			* @param {string}  argument.type returns the name of the event         
            * @example 
            * &lt;div id="reportviewer"&gt;&lt;/div&gt; 
            * &lt;script&gt;        
            * 	$("#reportviewer").ejReportViewer({ 
		    *          drillThrough: function (args) {
            *       // Write a code block to perform any operation when drill through action occurs in report.
            *       }
            *     });                   
            * &lt;/script&gt;
		    * @memberof ejReportViewer
		    * @instance
            */
            drillThrough: null,
            /**		
            * Fires when the report viewer is destroyed successfully.If you want to perform any operation after destroying the reportviewer control,you can make use of the destroy event.
			* @event
			* @name ejReportViewer#destroy
			* @param {Object} argument Event parameters from reportviewer     
			* @param {boolean}  argument.cancel true if the event should be canceled; otherwise, false.
			* @param {object}  argument.model returns the report model
			* @param {string}  argument.type returns the name of the event         
            * @example 
            * &lt;div id="reportviewer"&gt;&lt;/div&gt; 
            * &lt;script&gt;        
            * 	$("#reportviewer").ejReportViewer({ 
		    *      destroy:  function (args) {
            *       // Write a code block to perform any operation after destroy of reportviewer.
            *       }
            *     });           
            * &lt;/script&gt;
		    * @memberof ejReportViewer
		    * @instance
            */
            destroy: null,
			
			/**
            * Gets or sets the name of the data source.
            * @name ejReportViewer#dataSources->name
            * @default ""
            * @type {string}
            * @example
            * &lt;div id="reportviewer"&gt;&lt;/div&gt; 
            * &lt;script&gt;
            * 	$("#reportviewer").ejReportViewer({ dataSources: [name:"Menu Items",
            *                         values:[{ OrderId: "21D60", FoodName: "Burger", Price: 20, Category: "Veg" },
            *                                 { OrderId: "21D61", FoodName: "Pizza", Price: 25, Category: "Non-Veg" },
            *                                 { OrderId: "21D63", FoodName: "Sandwiches", Price: 30, Category: "Non-Veg" },
            *                                 { OrderId: "21D65", FoodName: "Chicken Drum Sticks", Price: 23, Category: "Non-Veg" },
            *                                 { OrderId: "21D64", FoodName: "Fulka", Price: 15, Category: "Veg" }]]
            *                                 });
            * &lt;/script&gt;
            */

            /**
            * Gets or sets the values of data source.
            * @name ejReportViewer#dataSources->values
            * @default null
            * @type {object}
            * @example
            * &lt;div id="reportviewer"&gt;&lt;/div&gt; 
            * &lt;script&gt;
            * 	$("#reportviewer").ejReportViewer({ dataSources: [name:"Menu Items",
			*                         values:[{ OrderId: "21D60", FoodName: "Burger", Price: 20, Category: "Veg" },
            *                                 { OrderId: "21D61", FoodName: "Pizza", Price: 25, Category: "Non-Veg" },
            *                                 { OrderId: "21D63", FoodName: "Sandwiches", Price: 30, Category: "Non-Veg" },
            *                                 { OrderId: "21D65", FoodName: "Chicken Drum Sticks", Price: 23, Category: "Non-Veg" },
            *                                 { OrderId: "21D64", FoodName: "Fulka", Price: 15, Category: "Veg" }]]
			*                                 });
            * &lt;/script&gt;
            */
            /**
            * Gets or sets the name of the parameter.
            * @name ejReportViewer#parameters->name
            * @default ""
            * @type {string}
            * @example
            * &lt;div id="reportviewer"&gt;&lt;/div&gt; 
            * &lt;script&gt;
            * 	$("#reportviewer").ejReportViewer({ parameters: [{
            *                                 name:"Vehicle",
            *                                 labels:["Motor Bikes"],
            *                                 prompt:"Please select the color",
            *                                 values:["Red","Green","Blue","Yellow","Black"],
            *                                 nullable:false
            *                                  }]});
            * &lt;/script&gt;
            */

            /**
            * Gets or sets the parameter labels.
            * @name ejReportViewer#parameters->labels
            * @default null
            * @type {array}
            * @example
            * &lt;div id="reportviewer"&gt;&lt;/div&gt; 
            * &lt;script&gt;
            * 	$("#reportviewer").ejReportViewer({ parameters: [{
            *                                 name:"Vehicle",
            *                                 labels:["Motor Bikes"],
            *                                 prompt:"Please select the color",
            *                                 values:["Red","Green","Blue","Yellow","Black"],
            *                                 nullable:false
            *                                  }]});
            * &lt;/script&gt;
            */

            /**
            * Gets or sets the prompt message associated with the specified parameter.
            * @name ejReportViewer#parameters->prompt
            * @default ""
            * @type {string}
            * @example
            * &lt;div id="reportviewer"&gt;&lt;/div&gt; 
            * &lt;script&gt;
            * 	$("#reportviewer").ejReportViewer({ parameters: [{
            *                                 name:"Vehicle",
            *                                 labels:["Motor Bikes"],
            *                                 prompt:"Please select the Color",
            *                                 values:["Red","Green","Blue","Yellow","Black"],
            *                                 nullable:false
            *                                  }]});
            * &lt;/script&gt;
            */

            /**
            * Gets or sets whether the parameter allows nullable value or not.
            * @name ejReportViewer#parameters->nullable
            * @default false
            * @type {boolean}
            * @example
            * &lt;div id="reportviewer"&gt;&lt;/div&gt; 
            * &lt;script&gt;
            * 	$("#reportviewer").ejReportViewer({ parameters: [{
            *                                 name:"Vehicle",
            *                                 labels:["Motor Bikes"],
            *                                 prompt:"Please select the color",
            *                                 values:["Red","Green","Blue","Yellow","Black"],
            *                                 nullable:false
            *                                  }]});
            * &lt;/script&gt;
            */

            /**
            * Gets or sets the parameter values.
            * @name ejReportViewer#parameters->values
            * @default null
            * @type {array}
            * @example
            * &lt;div id="reportviewer"&gt;&lt;/div&gt; 
            * &lt;script&gt;
            * 	$("#reportviewer").ejReportViewer({ parameters: [{
            *                                 name:"Vehicle",
            *                                 labels:["Motor Bikes"],
            *                                 prompt:"Please select the color",
            *                                 values:["Red","Green","Blue","Yellow","Black"],
            *                                 nullable:false
            *                                  }]});
            * &lt;/script&gt;
            */
        },

        dataTypes: {
            dataSources: "array",
            parameters: "array",
            toolbarSettings: {
                showToolbar: "boolean"
            }
        },
        //#endregion

        //#region Local Members
        _dataSources: null,
        _svg: true,
        _authenticationToken: null,
        _refresh: false,
        _isToolbarClick: false,
        _printMode: false,
        _pageModel: null,
        _currentPage: 1,
        _pageLayoutPage: 1,
        _reportParameters: null,
        _reportDataSources: null,
        _reporDataSets: null,
        _zoomLevel: 2,
        _preZoomVal: 1,
        _actionUrl: null,
        _isDocumentMap: false,
        _isPageDocMap: false,
        _pageDocMapFlag: false,
        _childReportAuthentication: null,
        _parents: [],
        _parentPageXY: null,
        _isDevice: false,
        _zoomVal: 1,
        _originX: 0,
        _originY: 0,
        _reportAction: {
            reportLoad: 'ReportLoad',
            getDataSourceCredential: 'GetDataSourceCredential',
            validateDSCredential: 'ValidateDSCredential',
            updateDSCredential: 'UpdateDSCredential',
            getParameters: 'GetParameters',
            setParameters: 'SetParameters',
            updateParameters: 'UpdateParameters',
            updateDataSource: 'UpdateDataSource',
            getPageModel: 'GetPageModel',
            getPrintModel: 'GetPrintModel',
            drillDown: 'DrillDown',
            clearCache: 'ClearCache',
            documentMap: 'DocumentMap',
            drillThrough: 'DrillThrough'
        },
        //#endregion

        //#region Initialization
        _setModel: function (options) {
            var reload = false;
            var update = false;
            for (var prop in options) {
                switch (prop) {
                    case "reportPath":
                    case "reportServerUrl":
                    case "reportServiceUrl":
                        reload = true;
                        break;
                    case "dataSources":
                    case "parameters":
                    case "processingMode":
                        update = true;
                        break;
                    case "zoomFactor":
                        this._zoomContainer(options[prop], false);
                        break;
                    case "printMode":
                        this._updatePreviewLayout(options[prop]);
                        break;
                    case "toolbarSettings":
                        this._updateToolbarmodel();
                        break;
                    case "locale":
                        this._setCultureInfo();
                        break;
                }
            }
            if (update && !reload) {
                this.doAjaxPost("POST", this._actionUrl, JSON.stringify({ 'reportAction': this._reportAction.getDataSourceCredential, 'dataSources': this.model.dataSources, 'parameters': this.model.parameters }), "_getDataSourceCredential");
            }

            if (reload) {
                this.doAjaxPost("POST", this._actionUrl, JSON.stringify({ 'reportAction': this._reportAction.clearCache }), '_reportReload');
            }
        },

        _destroy: function () {
            $('#' + this._id + '_exportIframe').remove();
            $('#' + this._id + '_toolbar_exportListTip').remove();
            $(this.element).find('.e-reportviewer-viewer').remove();
            $('#' + this._id + '_pageInfoPopup').remove();
            this.doAjaxPost("POST", this._actionUrl, JSON.stringify({ 'reportAction': this._reportAction.clearCache }), '_clearCurrentServerCache');
        },

        _init: function () {
            this._svg = (window.SVGSVGElement) ? true : false;
            this._actionUrl = this.model.reportServiceUrl + "/PostReportAction";
            this._authenticationToken = this._getAuthenticationToken();
            this._renderViewer();
            this._initViewer();
            this._on($(window), "resize", this._viewerResize);
            if (this.model.reportServiceUrl) {
                this.doAjaxPost("POST", this._actionUrl, JSON.stringify({
                    'reportAction': this._reportAction.reportLoad, 'controlId': this._id, 'reportPath': this.model.reportPath,
                    'reportServerUrl': this.model.reportServerUrl, 'processingMode': this.model.processingMode
                }), "_reportLoaded");
            }
        },

        _initViewer: function () {
            this._wireEvents();
            this._initToolbar();
            this._currentPage = 1;
            this._pageLayoutPage = this._currentPage;
            if (!this._refresh) {
                this._preZoomVal = 1;
                this._zoomLevel = 2;
            } 
            this._isDocumentMap = false;
            this._printMode = this.model.printMode;
            this._isPageDocMap = false;
            this._pageDocMapFlag = false;
            var tbSettings = this.model.toolbarSettings;
            if (!tbSettings.templateId) {
                this._showToolbar(tbSettings.showToolbar);
                this._showPrintButton(tbSettings.items & ej.ReportViewer.ToolbarItems.Print);
                this._showRefreshButton(tbSettings.items & ej.ReportViewer.ToolbarItems.Refresh);
                this._showZoomControl(tbSettings.items & ej.ReportViewer.ToolbarItems.Zoom);
                this._showExportControls(tbSettings.items & ej.ReportViewer.ToolbarItems.Export);
                this._showExportList();
                this._showPageNavigationControls(tbSettings.items & ej.ReportViewer.ToolbarItems.PageNavigation);
                this._showParameterBlock(tbSettings.items & ej.ReportViewer.ToolbarItems.Parameters);
                this._showPreviewButton(tbSettings.items & ej.ReportViewer.ToolbarItems.PrintLayout);
                this._showDrillThrough(this._parents.length > 0);
                this._showDocumentMap(false);
            }
            this._showViewerBlock(false);
            this._setContainerSize();
            this._showloadingIndicator(true);
        },
        //#endregion

        //-------------------- Render the ReportViewer Ctrl [start] -------------------------//
        _renderPageInfoPopup: function () {
            var pagePopup = $('#' + this._id + '_pageInfoPopup');
            if (!(pagePopup.length > 0)) {
                pagePopup = ej.buildTag("div.e-reportviewer-pagepopupinfo", "", { 'WHITE-SPACE': 'nowrap' }, { id: this._id + '_pageInfoPopup' });
                var pageNoDiv = ej.buildTag("input.e-reportviewer-pagenumber e-reportviewer-elementalignments ejinputtext", "", {}, { 'type': 'text', 'value': this._currentPage, id: this._id + '_popupPageNo' });
                pageNoDiv.css('height', '25.5px');
                pageNoDiv.css('width', '25px');
                pageNoDiv.css('background-color', 'black');
                pageNoDiv.css('color', 'white');
                pageNoDiv.css('border-color', 'black');

                var pageSeparateSpan = ej.buildTag("span.e-reportviewer-popuptotalpage", "", {}, {});
                pageSeparateSpan.append(' / ' + this._pageModel.TotalPages);
                pagePopup.append(pageNoDiv);
                pagePopup.append(pageSeparateSpan);
                $('body').append(pagePopup);
                this._on($('#' + this._id + '_popupPageNo'), "click", this._popupClick);
                this._on($('#' + this._id + '_popupPageNo'), "keypress", this._onkeyPress);
            }

            if (pagePopup.css('display') == 'block') {
                pagePopup.stop().fadeIn();
                pagePopup.stop().fadeOut(3000);
                pagePopup.hide(3000);
            }
            else {
                pagePopup.show();
                pagePopup.stop().fadeIn();
                pagePopup.stop().fadeOut(3000);
                pagePopup.hide(3000);
            }

            var viewerContainer = $('#' + this._id + '_viewerContainer');
            var popupX = (viewerContainer.offset().left + (viewerContainer.width() / 2)) - (pagePopup.width() / 2);
            var popupY = (viewerContainer.offset().top + (viewerContainer.height() / 2)) - (pagePopup.height() / 2);
            pagePopup.css({ 'left': popupX, 'top': popupY });
            this._updatePageNo();
        },
		
		_renderToolTip: function () {
            var $divTooltip = ej.buildTag("div.e-reportviewer-tbdiv e-reportviewer-tooltip", "", { 'display': 'none' }, { 'id': this._id + '_rptTooltip' });
            var $tooltipHeader = ej.buildTag("span.e-reportviewer-headerspan", "", { 'display': 'block' }, { 'id': this._id + '_rptTooltip_Header' });
            var $tooltipContent = ej.buildTag("span.e-reportviewer-contentspan", "", { 'display': 'block' }, { 'id': this._id + '_rptTooltip_Content' });
            $divTooltip.append($tooltipHeader);
            $divTooltip.append($tooltipContent);
            $('body').append($divTooltip);
        },

        _renderToolBar: function (targetTag) {
            if (!this.model.toolbarSettings.templateId) {
                var div = ej.buildTag("div.e-reportviewer-toolbarcontainer .e-reportviewer-viewer", "", { 'width': '100%' }, { 'id': this._id + '_toolbarContainer' });
                targetTag.append(div);

                if (this._isDevice) {
                    var $ultoolbaritems = ej.buildTag("ul.e-reportviewer-toolbarul", "", {});
                    this._appendToolbarItems($ultoolbaritems, 'export');
                    this._appendToolbarItems($ultoolbaritems, 'gotoparent');
                    this._appendToolbarItems($ultoolbaritems, 'refresh');
                    this._appendToolbarItems($ultoolbaritems, 'zoomin');
                    this._appendToolbarItems($ultoolbaritems, 'zoomout');
                    this._appendToolbarItems($ultoolbaritems, 'documentmap');
                    this._appendToolbarItems($ultoolbaritems, 'parameter');
                    div.append($ultoolbaritems);
                } else {
                    var $ulprintExport = ej.buildTag("ul.e-reportviewer-toolbarul", "", {});
                    this._appendToolbarItems($ulprintExport, 'print');
                    this._appendToolbarItems($ulprintExport, 'export');
                    this._appendToolbarItems($ulprintExport, 'preview');
                    div.append($ulprintExport);

                    var $ulnavigate = ej.buildTag("ul.e-reportviewer-toolbarul", "", {});
                    this._appendToolbarItems($ulnavigate, 'gotofirst');
                    this._appendToolbarItems($ulnavigate, 'gotoprevious');
                    this._appendToolbarItems($ulnavigate, 'gotopage');
                    this._appendToolbarItems($ulnavigate, 'gotonext');
                    this._appendToolbarItems($ulnavigate, 'gotolast');
                    this._appendToolbarItems($ulnavigate, 'gotoparent');
                    div.append($ulnavigate);

                    var $ulzoom = ej.buildTag("ul.e-reportviewer-toolbarul", "", {});
                    this._appendToolbarItems($ulzoom, 'zoomin');
                    this._appendToolbarItems($ulzoom, 'zoomout');
                    this._appendToolbarItems($ulzoom, 'zoom');
                    div.append($ulzoom);

                    var $ulro = ej.buildTag("ul.e-reportviewer-toolbarul", "", {});
                    this._appendToolbarItems($ulro, 'refresh');
                    this._appendToolbarItems($ulro, 'documentmap');
                    this._appendToolbarItems($ulro, 'parameter');
                    div.append($ulro);
                    $('#' + this._id + '_toolbar_zoomSelection').ejDropDownList({ height: "27px", width: "75px", change: this._zoomValChange, selectedItem: 2 });
                }
                div.ejToolbar({ enableSeparator: !this._isDevice, click: $.proxy(this._toolbarClick, this) });
            } else {
                var templateDiv = $('#' + this.model.toolbarSettings.templateId);
                targetTag.append(templateDiv);
                templateDiv.ejToolbar({ enableSeparator: true, height: templateDiv.height(), click: this.model.toolbarSettings.click });
                templateDiv.css('display', 'block');
            }
        },

        _appendToolbarItems: function (litag, eletype) {
            var $divouter;
            switch (eletype) {
                case 'print':
                    $divouter = ej.buildTag("li.e-reportviewer-toolbarli", "", {}, {});
                    var $spanprint = ej.buildTag("span.e-reportviewer-icon e-reportviewer-print", "", {}, { 'id': this._id + '_toolbar_Print' });
                    $divouter.append($spanprint);
                    break;
                case 'export':
                    $divouter = ej.buildTag("li.e-reportviewer-toolbarli", "", {}, {});
                    var $spanexport = ej.buildTag("span.e-reportviewer-icon e-reportviewer-export", "", {}, { 'id': this._id + '_ejtb_export' });
                    $divouter.append($spanexport);
                    break;
                case 'gotofirst':
                    $divouter = ej.buildTag("li.e-reportviewer-toolbarli", "", {}, {});
                    var $spangotofirst = ej.buildTag("span.e-reportviewer-icon e-reportviewer-gotofirst", "", {});
                    $divouter.append($spangotofirst);
                    break;
                case 'gotolast':
                    $divouter = ej.buildTag("li.e-reportviewer-toolbarli", "", {}, {});
                    var $spangotolast = ej.buildTag("span.e-reportviewer-icon e-reportviewer-gotolast", "", {});
                    $divouter.append($spangotolast);
                    break;
                case 'gotonext':
                    $divouter = ej.buildTag("li.e-reportviewer-toolbarli", "", {}, {});
                    var $spangotonext = ej.buildTag("span.e-reportviewer-icon e-reportviewer-gotonext", "", {});
                    $divouter.append($spangotonext);
                    break;
                case 'gotoprevious':
                    $divouter = ej.buildTag("li.e-reportviewer-toolbarli", "", {}, {});
                    var $spangotoprevious = ej.buildTag("span.e-reportviewer-icon e-reportviewer-gotoprevious", "", {});
                    $divouter.append($spangotoprevious);
                    break;
                case 'gotoparent':
                    $divouter = ej.buildTag("li.e-reportviewer-toolbarli", "", {}, {});
                    var $spangotoparentreport = ej.buildTag("span.e-reportviewer-icon e-reportviewer-gotoparent", "", {});
                    $divouter.append($spangotoparentreport);
                    break;
                case 'gotopage':
                    $divouter = ej.buildTag("div.e-reportviewer-tbpage", "", { 'display': 'block' });
                    var $inputpageno = ej.buildTag("input.e-reportviewer-pagenumber e-reportviewer-elementalignments ejinputtext", "", {}, { 'type': 'text', 'value': '0', 'id': this._id + '_txtpageNo' });
                    var $spanpageno = ej.buildTag("span.e-reportviewer-labelpageno", "", {}, {});
                    $divouter.append($inputpageno);
                    $divouter.append($spanpageno);
                    break;
                case 'zoomin':
                    $divouter = ej.buildTag("li.e-reportviewer-toolbarli", "", {}, {});
                    var $spanzoomin = ej.buildTag("span.e-reportviewer-icon e-reportviewer-zoomin", "", {});
                    $divouter.append($spanzoomin);
                    break;
                case 'zoomout':
                    $divouter = ej.buildTag("li.e-reportviewer-toolbarli", "", {}, {});
                    var $spanzoomout = ej.buildTag("span.e-reportviewer-icon e-reportviewer-zoomout", "", {});
                    $divouter.append($spanzoomout);
                    break;
                case 'zoom':
                    $divouter = ej.buildTag("div.e-reportviewer-ejdropdownlist", "", {}, {});
                    var $sltzoom = ej.buildTag("select.e-reportviewer-tbdiv e-reportviewer-zoomlist", "", {}, { 'id': this._id + '_toolbar_zoomSelection' });
                    $sltzoom.append("<option>50%</option>");
                    $sltzoom.append("<option>75%</option>");
                    $sltzoom.append("<option Selected>100%</option>");
                    $sltzoom.append("<option>125%</option>");
                    $sltzoom.append("<option>150%</option>");
                    $sltzoom.append("<option>200%</option>");
                    $sltzoom.append("<option>400%</option>");
                    $divouter.append($sltzoom);
                    break;
                case 'preview':
                    $divouter = ej.buildTag("li.e-reportviewer-toolbarli", "", {}, { 'id': this._id + '_ejtb_preview' });
                    var $spanpreview = ej.buildTag("span.e-reportviewer-icon e-reportviewer-preview", "", {}, {});
                    $divouter.append($spanpreview);
                    break;
                case 'refresh':
                    $divouter = ej.buildTag("li.e-reportviewer-toolbarli", "", {}, {});
                    var $spanrefresh = ej.buildTag("span.e-reportviewer-icon e-reportviewer-refresh", "", {}, { 'id': this._id + '_toolbar_refresh' });
                    $divouter.append($spanrefresh);
                    break;
                case 'documentmap':
                    $divouter = ej.buildTag("li.e-reportviewer-toolbarli", "", {}, { 'id': this._id + '_ejtb_documentmap' });
                    var $spandocumentMap = ej.buildTag("span.e-reportviewer-icon e-reportviewer-documentmap", "", {});
                    $divouter.append($spandocumentMap);
                    break;
                case 'parameter':
                    $divouter = ej.buildTag("li.e-reportviewer-toolbarli", "", {}, { 'id': this._id + '_ejtb_parameter' });
                    var $spanparameter = ej.buildTag("span.e-reportviewer-icon e-reportviewer-parameter", "", {});
                    $divouter.append($spanparameter);
                    break;
                case 'pdf':
                    $divouter = ej.buildTag("li.e-reportviewer-popupli", "", {}, { 'id': this._id + '_pdf' });
                    $divouter.append('PDF');
                    break;
                case 'xls':
                    $divouter = ej.buildTag("li.e-reportviewer-popupli", "", {}, { 'id': this._id + '_xls' });
                    $divouter.append('Excel');
                    break;
                case 'word':
                    $divouter = ej.buildTag("li.e-reportviewer-popupli", "", {}, { 'id': this._id + '_word' });
                    $divouter.append('Word');
                    break;
                case 'html':
                    $divouter = ej.buildTag("li.e-reportviewer-popupli", "", {}, { 'id': this._id + '_html' });
                    $divouter.append('Html');
                    break;
            }
            litag.append($divouter);
        },

        _renderToolTipExport: function () {
            var $tipExport = ej.buildTag("div.e-reportviewer-tbdiv e-reportviewer-icon e-reportviewer-exporttip", "", { 'display': 'none' }, { 'id': this._id + '_toolbar_exportListTip' });
            this._appendToolbarItems($tipExport, 'pdf');
            this._appendToolbarItems($tipExport, 'xls');
            this._appendToolbarItems($tipExport, 'word');
            this._appendToolbarItems($tipExport, 'html');
            return $tipExport;
        },

        _renderViewerBlockinDevice: function (targetTag) {
            var div = ej.buildTag("div.e-reportviewer-viewer e-reportviewer-viewerblock e-reportviewer-blockstyle", "", {}, { 'id': this._id + '_viewBlockContainer' });
            targetTag.append(div);

            var $innerConetnt = ej.buildTag("div.e-reportviewer-viewerblockcellcontent", "", { 'margin': '1px', 'padding': '1px', 'width': '99%', 'height': '99%', 'display': 'inline-table' });
            var $blockContent = ej.buildTag("div.e-reportviewer-viewerblockcontent", "", { 'width': '100%', 'height': 'auto' });
            var $buttonConetnt = ej.buildTag("div.e-reportviewer-viewreport", "", { 'width': '100%', 'height': '30px', 'border-left': '0px', 'padding': '10px 0' });
            var $button = ej.buildTag("input.e-reportviewer-viewreportbutton e-btn", "", {}, { 'type': 'button', 'value': ej.ReportViewer.localization[this.model.locale] ? ej.ReportViewer.localization[this.model.locale]["viewButton"] : ej.ReportViewer.localization["en-us"]["viewButton"], 'id': this._id + '_viewReportClick' });
            $buttonConetnt.append($button);
            $innerConetnt.append($blockContent);
            $innerConetnt.append($buttonConetnt);

            $button.ejButton({ showRoundedCorner: true, height: 23 });

            div.append($innerConetnt);
            return div;
        },

        _renderViewerBlockinWeb: function (targetTag) {
            var div = ej.buildTag("div.e-reportviewer-viewer e-reportviewer-viewerblock", "", {}, { 'id': this._id + '_viewBlockContainer' });
            targetTag.append(div);

            var $innerConetnt = ej.buildTag("table.e-reportviewer-viewerblockcellcontent", "", { 'margin': '1px', 'padding': '5px 5px 10px' });
            var $tr = ej.buildTag("tr", "", { 'width': '100%' });
            var $tdleft = ej.buildTag("td.e-reportviewer-viewerblockcontent", "", {});
            var $tdright = ej.buildTag("td.e-reportviewer-viewreport", "", {});
            var $button = ej.buildTag("input.e-reportviewer-viewreportbutton e-btn", "", {}, { 'type': 'button', 'value': ej.ReportViewer.localization[this.model.locale] ? ej.ReportViewer.localization[this.model.locale]["viewButton"] : ej.ReportViewer.localization["en-US"]["viewButton"], 'id': this._id + '_viewReportClick' });
            $tdright.append($button);
            $tr.append($tdleft);
            $tr.append($tdright);
            $innerConetnt.append($tr);

            $button.ejButton({ showRoundedCorner: true, height: 23 });

            div.append($innerConetnt);
            return div;
        },

        _renderViewerContainer: function (targetTag) {
            var div = ej.buildTag("div", "", {}, { 'id': this._id + '_reportviewerContainer' });
            targetTag.append(div);

            var $reportviewContainer = ej.buildTag("div.e-reportviewer-viewer e-reportviewer-scrollcontainer e-reportviewer-viewercontainer", "", { 'height': '100%', 'width': '100%', 'font-size': '8pt' }, { 'id': this._id + '_viewerContainer' });

            var $loadingindicator = ej.buildTag("div", "", { 'margin': '0px', 'height': '99.8%', 'width': '100%' }, { 'id': this._id + '_loadingIndicator' });
            var $loadingemptybackview = ej.buildTag("div", "", { 'margin': '0px', 'height': '99.8%', 'width': '100%', 'background-color': 'rgba(164, 183, 216, 0.18)', 'display': 'block' }, { 'id': this._id + '_loadingIndicatorBackView' });

            var $pageviewOuterContainer = ej.buildTag("div.e-reportviewer-pageviewcontainer", "", {}, { 'id': this._id + '_pageviewOuterContainer' });
            var $pageviewOuterline = ej.buildTag("div.e-reportviewer-pageouterline", "", {}, { 'id': this._id + '_pageviewOuterline' });
            var $pageviewContainer = ej.buildTag("div.e-reportviewer-pageview", "", { 'background-color': '#FFFFFF' }, { 'id': this._id + '_pageviewContainer' });

            var $pageviewheaderContainer = ej.buildTag("div", "", { 'position': 'relative', 'background-color': 'transparent' }, { 'id': this._id + '_pageviewheaderContainer' });
            var $pageviewbodyContainer = ej.buildTag("div", "", { 'position': 'relative', 'background-color': 'transparent' }, { 'id': this._id + '_pageviewbodyContainer' });
            var $pageviewfooterContainer = ej.buildTag("div", "", { 'position': 'relative', 'background-color': 'transparent' }, { 'id': this._id + '_pageviewfooterContainer' });

            var $pageviewheaderborder = ej.buildTag("div.pageHeaderBorder", "", { 'position': 'relative', 'background-color': '#FFFFFF' });
            var $pageviewbodyborder = ej.buildTag("div.pageBodyBorder", "", { 'position': 'relative', 'background-color': '#FFFFFF' });
            var $pageviewfooterborder = ej.buildTag("div.pageFooterBorder", "", { 'position': 'relative', 'background-color': '#FFFFFF' });

            $pageviewheaderborder.append($pageviewheaderContainer);
            $pageviewbodyborder.append($pageviewbodyContainer);
            $pageviewfooterborder.append($pageviewfooterContainer);

            $pageviewContainer.append($pageviewheaderborder);
            $pageviewContainer.append($pageviewbodyborder);
            $pageviewContainer.append($pageviewfooterborder);

            $loadingindicator.append($loadingemptybackview);
            $reportviewContainer.append($loadingindicator);
            $pageviewOuterline.append($pageviewContainer);
            $pageviewOuterContainer.append($pageviewOuterline);
            $reportviewContainer.append($pageviewOuterContainer);

            if (this._isDevice) {
                this._renderViewerBlockinDevice($reportviewContainer);
            } else {
                var $documentmapcontainer = ej.buildTag("div.e-reportviewer-documentmapcontainer", "", {}, { 'id': this._id + '_documentmapContainer' });
                div.append($documentmapcontainer);
            }

            div.append($reportviewContainer);

            return div;
        },

        _renderViewer: function () {
            var height = this.element.height();
            var width = this.element.width();
            if (height === 0 && this.element[0].parentElement.clientHeight != 0) {
                this.element.height(this.element[0].parentElement.clientHeight);
            }
            if (width === 0 && this.element[0].parentElement.clientWidth != 0) {
                this.element.width(this.element[0].parentElement.clientWidth);
            }

            //control root div tag.
            var viewer = ej.buildTag("div.e-reportviewer-viewer", "", {});
            this.element.append(viewer);
            this._isDevice = this._isMobileDevice();
            this._renderToolBar(viewer);
            if (!this._isDevice) {
                this._renderViewerBlockinWeb(viewer);
                this._renderToolTip();
            }
            this._renderViewerContainer(viewer);
        },
        //-------------------- Render the ReportViewer Ctrl [End] -------------------------//

        //-------------------- Render Credential Block and Actions [Start] -------------------------//
        _renderCredentialBlock: function (datasources) {
            var table = ej.buildTag("table", "", {}, {});
            for (var index = 0; index < datasources.length; index++) {
                var datasource = datasources[index];
                var $headertr = ej.buildTag("tr", "", {}, {});
                var $headertd = ej.buildTag("td", "", {}, { 'colspan': !this._isDevice ? '5' : '1' });
                $headertd.html(!this._isDevice ? datasource.Promt : 'Specify the ' + datasource.Name + ':');
                $headertr.append($headertd);

                if (!this._isDevice) {
                    var $row = ej.buildTag("tr", "", {}, {});
                    this._appendCredentialItems($row, 'namelbl', datasource);
                    this._appendCredentialItems($row, 'nametxt', datasource);
                    $row.append(ej.buildTag("td.e-reportviewer-viewerblockcellcontent", "", { 'width': '7px' }, {}));
                    this._appendCredentialItems($row, 'passwdlbl', datasource);
                    this._appendCredentialItems($row, 'passwdtxt', datasource);

                    table.append($headertr);
                    table.append('<tr height="6px"/>');
                    table.append($row);
                    table.append('<tr height="6px"/>');
                } else {
                    table.append($headertr);
                    table.append('<tr height="4px"/>');

                    var $row1 = ej.buildTag("tr", "", {}, {});
                    this._appendCredentialItems($row1, 'namelbl', datasource);
                    var $row2 = ej.buildTag("tr", "", {}, {});
                    this._appendCredentialItems($row2, 'nametxt', datasource);
                    var $row3 = ej.buildTag("tr", "", {}, {});
                    this._appendCredentialItems($row3, 'passwdlbl', datasource);
                    var $row4 = ej.buildTag("tr", "", {}, {});
                    this._appendCredentialItems($row4, 'passwdtxt', datasource);

                    table.append($row1);
                    table.append($row2);
                    table.append($row3);
                    table.append($row4);
                    table.append('<tr height="6px"/>');
                }
            }
            var viewerBlock = $('#' + this._id + '_viewBlockContainer').find(".e-reportviewer-viewerblockcontent");
            viewerBlock.append(table);
            this._on($('#' + this._id + '_viewReportClick'), "click", { ds: datasources }, this._viewReportCredentialClick);
        },

        _appendCredentialItems: function (targetTag, eleType, datasource) {
            var $tbcell;
            switch (eleType) {
                case 'namelbl':
                    $tbcell = ej.buildTag("td.e-reportviewer-viewerblockcellcontent", "", {}, {});
                    $tbcell.append('Login Name&nbsp;');
                    break;
                case 'nametxt':
                    $tbcell = ej.buildTag("td.e-reportviewer-viewerblockcellcontent", "", {}, {});
                    var $loginname = ej.buildTag("input.e-reportviewer-textbox", "", {}, { 'type': 'text', 'autocomplete': 'off', 'id': datasource.ControlId + "_loginName" });
                    $tbcell.append($loginname);
                    break;
                case 'passwdlbl':
                    $tbcell = ej.buildTag("td.e-reportviewer-viewerblockcellcontent", "", {}, {});
                    $tbcell.append('Password&nbsp;');
                    break;
                case 'passwdtxt':
                    $tbcell = ej.buildTag("td.e-reportviewer-viewerblockcellcontent", "", {}, {});
                    var $password = ej.buildTag("input.e-reportviewer-textbox", "", {}, { 'type': 'password', 'autocomplete': 'off', 'id': datasource.ControlId + "_password" });
                    $tbcell.append($password);
                    break;
            }
            targetTag.append($tbcell);
        },

        _viewReportCredentialClick: function (event) {
            var datasources = event.data.ds;
            var jsdatasource = [];
            for (var index = 0; index < datasources.length; index++) {
                var datasource = datasources[index];
                var username = $('#' + datasource.ControlId + '_loginName').val();
                var password = $('#' + datasource.ControlId + '_password').val();
                jsdatasource.push({ 'Name': datasource.Name, 'UserName': username, 'Password': password });
            }
            this._dataSources = jsdatasource;
            this.doAjaxPost("POST", this._actionUrl, JSON.stringify({ 'reportAction': this._reportAction.validateDSCredential, 'datasoures': jsdatasource }), "_validateDSCredential");
        },
        //-------------------- Render Credential Block and Actions [End] -------------------------//

        //-------------------- Render Parameter Block and Actions [Start] -------------------------//
        _renderParameterBlock: function (parameters) {
            var table = ej.buildTag("table", "", {}, { 'isViewClick': false });
            if (this._isDevice) {
                table.css('width','100%');
            }
            if (parameters != null && parameters.length > 0) {
                var viewerBlock = $('#' + this._id + '_viewBlockContainer').find(".e-reportviewer-viewerblockcontent");
                viewerBlock.html(table);
                for (var index = 0; index < parameters.length; index++) {
                    var parameter = parameters[index];

                    var tablerow;
                    var $contentDiv;
                    var $tdCell01;
                    var $tdCell02;
                    if (!this._isDevice) {
                        if (index % 2 == 0) {
                            table.append('<tr height="10px"/>');
                            tablerow = ej.buildTag("tr", "", {}, {});
                            table.append(tablerow);
                        } else {
                            var $tdmiddle = ej.buildTag("td", "", { 'width': '15px' }, {});
                            tablerow.append($tdmiddle);
                        }
                        $tdCell01 = ej.buildTag("td.e-reportviewer-viewerblockcellcontent", "", {}, {});
                        $tdCell01.append(parameter.Promt + '&nbsp;&nbsp;');
                        tablerow.append($tdCell01);
                        $tdCell02 = ej.buildTag("td.e-reportviewer-viewerblockcellcontent", "", { 'min-width': '130px', 'max-width': '185px' }, {});
                        tablerow.append($tdCell02);

                        $contentDiv = ej.buildTag("div", "", { 'width': '100%', 'display': 'flex', 'white-space': 'nowrap' }, { 'id': parameter.ControlId + '_div' });
                        $tdCell02.append($contentDiv);

                    } else {
                        table.append('<tr height="10px"/>');
                        tablerow = ej.buildTag("tr", "", {}, {});
                        table.append(tablerow);
                        $tdCell01 = ej.buildTag("td.e-reportviewer-viewerblockcellcontent", "", {}, {});
                        $tdCell01.append(parameter.Promt + '&nbsp;&nbsp;');
                        tablerow.append($tdCell01);

                        tablerow = ej.buildTag("tr", "", {}, {});
                        table.append(tablerow);
                        $tdCell02 = ej.buildTag("td.e-reportviewer-viewerblockcellcontent", "", { 'min-width': '130px', 'max-width': '185px' }, {});
                        tablerow.append($tdCell02);

                        $contentDiv = ej.buildTag("div", "", { 'width': '100%', 'display': 'flex', 'white-space': 'nowrap' }, { 'id': parameter.ControlId + '_div' });
                        $tdCell02.append($contentDiv);
                    }
                    this._renderParameterElements(parameter, $contentDiv);
                }
            }
            table.append('<tr height="10px"/>');
            this._on($('#' + this._id + '_viewReportClick'), "click", { params: parameters }, this._viewReportParamsClick);
        },

        _renderParameterElements: function (parameter, $contentDiv) {
            var indexddl;
            var $parameterSpan;
            var $parameterNull;
            var $optionText;
            if (parameter.IsMultiValue) {
                var $paramMultiValuedropdown = ej.buildTag("select", "", {}, { 'id': parameter.ControlId, 'name': parameter.Name });
                var itemindex = [];
                for (indexddl = 0; indexddl < parameter.AvailableLabels.length; indexddl++) {
                    $optionText = $('<option value="' + parameter.AvailableValues[indexddl] + '">' + parameter.AvailableLabels[indexddl] + '</option>');
                    if (jQuery.inArray(parameter.AvailableValues[indexddl], parameter.DefaultValuesfields) != -1) {
                        itemindex.push(indexddl);
                    }
                    $paramMultiValuedropdown.append($optionText);
                }
                $contentDiv.append($paramMultiValuedropdown);
                $paramMultiValuedropdown.ejDropDownList({ width: '95%', height: '26px', showCheckbox: true, selectedItems: itemindex, watermarkText: 'Select Option', change: parameter.IsDependent ? this._paramsChangeEvent : '' });
            } else if (parameter.DataType == "DateTime") {
                var $parameterdatepicker = ej.buildTag("input.e-reportviewer-textbox", "", {}, { 'type': 'text', 'id': parameter.ControlId, 'maxlength': '256', 'name': parameter.Name });
                $contentDiv.append($parameterdatepicker);
                $parameterdatepicker.ejDatePicker({ width: parameter.IsNullable ? "75%" : "95%", height: '26px', value: parameter.Label != null && parameter.Label.length > 0 ? new Date(parameter.Label) : null, change: parameter.IsDependent ? this._paramsChangeEvent : '' });
                if (parameter.IsNullable) {
                    $parameterSpan = ej.buildTag("span", "", { 'WHITE-SPACE': 'nowrap', 'display': 'block' }, {});
                    $parameterNull = ej.buildTag("input", "", {}, { 'type': 'checkbox', 'name': 'chkDateTime', 'id': parameter.ControlId + '~chk', 'value': this._id });
                    $parameterSpan.append($parameterNull);
                    $parameterSpan.append('&nbsp;<span class=e-reportviewer-label>Null</span>');
                    $contentDiv.append('&nbsp;');
                    $contentDiv.append($parameterSpan);
                    if (parameter.Label != null && parameter.Label.length > 0) {
                        $parameterNull.ejCheckBox({ "change": this._paramNullChange });
                    } else {
                        $parameterNull.ejCheckBox({ "change": this._paramNullChange, checked: true });
                        this._paramElementDisable(parameter.ControlId, 'chkDateTime', true);
                    }
                }
            } else if (parameter.AvailableValues != null) {
                var $parameterdropdown = ej.buildTag("select", "", {}, { 'id': parameter.ControlId, 'name': parameter.Name, 'data': 'select' });
                var selectindex = 9999;
                for (indexddl = 0; indexddl < parameter.AvailableLabels.length; indexddl++) {
                    if (parameter.AvailableLabels[indexddl] != 'Select a value') {
                        $optionText = $('<option value="' + parameter.AvailableValues[indexddl] + '">' + parameter.AvailableLabels[indexddl] + '</option>');
                        if (parameter.AvailableLabels[indexddl] == parameter.Label) {
                            selectindex = indexddl;
                        }
                        $parameterdropdown.append($optionText);
                    }
                }
                $contentDiv.append($parameterdropdown);
                $parameterdropdown.ejDropDownList({ width: '95%', height: '26px', selectedItemIndex: selectindex, watermarkText: 'Select a Value', change: parameter.IsDependent ? this._paramsChangeEvent : '' });
            } else if (parameter.DataType == "Boolean") {
                var $parametertable = ej.buildTag("table", "", {}, {});
                var $parametertr = ej.buildTag("tr", "", {}, {});
                $parametertable.append($parametertr);

                var propStat = false;
                if (parameter.Label != null && parameter.Label.length > 0) {
                    if (parameter.Label.toLowerCase() == "true") {
                        propStat = true;
                    }
                }

                var $parametertd01 = ej.buildTag("td", "", {}, {});
                var $parameterradio01 = ej.buildTag("input", "", {}, { 'type': 'radio', 'name': parameter.Name, 'id': parameter.ControlId + '_01' });
                $parametertd01.append($parameterradio01);
                $parametertd01.append("&nbsp;<span>True</span>");
                $parameterradio01.ejRadioButton({ checked: propStat });
                $parametertr.append($parametertd01);

                $parametertr.append('<td width=10px/>');

                var $parametertd02 = ej.buildTag("td", "", {}, {});
                var $parameterradio02 = ej.buildTag("input", "", {}, { 'type': 'radio', 'name': parameter.Name, 'id': parameter.ControlId + '_02' });
                $parametertd02.append($parameterradio02);
                $parametertd02.append("&nbsp;<span>False</span>");
                $parameterradio02.ejRadioButton({ checked: !propStat });
                $parametertr.append($parametertd02);

                $contentDiv.append($parametertable);

                if (parameter.IsNullable) {
                    $parameterSpan = ej.buildTag("span", "", { 'WHITE-SPACE': 'nowrap', 'display': 'block' }, {});
                    $parameterNull = ej.buildTag("input", "", {}, { 'type': 'checkbox', 'name': 'chkRadio', 'id': parameter.ControlId + '~chk', 'value': this._id });
                    $parameterSpan.append($parameterNull);
                    $parameterSpan.append('&nbsp;<span class=e-reportviewer-label>Null</span>');
                    $contentDiv.append('&nbsp;');
                    $contentDiv.append($parameterSpan);
                    $parameterNull.ejCheckBox({ "change": this._paramNullChange });
                    if (parameter.Label != null && parameter.Label.length > 0) {
                        $parameterNull.ejCheckBox({ "change": this._paramNullChange });
                    } else {
                        $parameterNull.ejCheckBox({ "change": this._paramNullChange, checked: true });
                        this._paramElementDisable(parameter.ControlId, 'chkRadio', true);
                    }
                }
            } else {
                var $parametertxtbx = ej.buildTag("input.e-reportviewer-textbox", "", {}, { 'type': 'text', 'autocomplete': 'off', 'id': parameter.ControlId, 'maxlength': '256', 'name': parameter.Name, 'value': parameter.Label });
                $contentDiv.append($parametertxtbx);
                if (parameter.IsNullable) {
                    $parametertxtbx.css('width', '75%');
                    $parameterSpan = ej.buildTag("span", "", { 'WHITE-SPACE': 'nowrap', 'display': 'block' }, {});
                    $parameterNull = ej.buildTag("input", "", {}, { 'type': 'checkbox', 'name': 'chkTextbox', 'id': parameter.ControlId + '~chk', 'value': this._id });
                    $parameterSpan.append($parameterNull);
                    $parameterSpan.append('&nbsp;<span class=e-reportviewer-label>Null</span>');
                    $contentDiv.append('&nbsp;');
                    $contentDiv.append($parameterSpan);
                    $parameterNull.ejCheckBox({ "change": this._paramNullChange });
                    if (parameter.Label != null && parameter.Label.length > 0) {
                        $parameterNull.ejCheckBox({ "change": this._paramNullChange });
                    } else {
                        $parameterNull.ejCheckBox({ "change": this._paramNullChange, checked: true });
                        this._paramElementDisable(parameter.ControlId, 'chkTextbox', true);
                    }
                } else {
                    $parametertxtbx.css('width', '96%');
                }

                if (parameter.IsDependent) {
                    this._on($parametertxtbx, "change", this._paramsChangeEvent);
                }
            }
        },

        _updateParamElements: function (parameters) {
            for (var index = 0; index < parameters.length; index++) {
                var parameter = parameters[index];
                var targetTag = $('#' + parameter.ControlId + '_div');
                if (parameter.IsMultiValue || parameter.AvailableValues != null || parameter.DataType == "Boolean") {
                    targetTag.empty();
                    this._renderParameterElements(parameter, targetTag);
                } else if (parameter.DataType == "DateTime") {
                    var ejDatePick = $('#' + parameter.ControlId).data("ejDatePicker");
                    ejDatePick.setModel({ value: parameter.Label });
                } else {
                    $('#' + parameter.ControlId).val(parameter.Label);
                }
            }
        },

        _paramsChangeEvent: function (event) {
            var ejViewerId = this._id.substr(0, (this._id.lastIndexOf('Param') - 1));
            var proxy = $('#' + ejViewerId).data('ejReportViewer');
            var updateParam = {
                Name: '',
                Labels: [],
                Values: []
            };
            var targetTag = $('#' + this._id);
            updateParam.Name = targetTag.attr('name');
            if (this.pluginName == 'ejDropDownList') {
                if (this.model.showCheckbox) {
                    for (var optIndex = 0; optIndex < this.model.selectedItems.length; optIndex++) {
                        updateParam.Labels.push(this.selectOptionItems[this.model.selectedItems[optIndex]].text);
                        updateParam.Values.push(this.selectOptionItems[this.model.selectedItems[optIndex]].value);
                    }
                } else {
                    updateParam.Labels.push(this.selectOptionItems[this.model.selectedItemIndex].text);
                    updateParam.Values.push(this.selectOptionItems[this.model.selectedItemIndex].value);
                }
            } else if (this.pluginName == 'ejDatePicker') {
                var dateTime = targetTag.data("ejDatePicker").getValue();
                updateParam.Labels.push(dateTime);
                updateParam.Values.push(dateTime);
            } else if (this.pluginName == 'ejRadioButton') {
                var isCheck = data("ejRadioButton").model.checked;
                updateParam.Labels.push(isCheck);
                updateParam.Values.push(isCheck);
            } else {
                updateParam.Labels.push(targetTag.val());
                updateParam.Values.push(targetTag.val());
            }
            proxy.doAjaxPost("POST", proxy._actionUrl, JSON.stringify({ 'reportAction': proxy._reportAction.updateParameters, 'updateParam': updateParam }), "_updateParameters");
        },

        _viewReportParamsClick: function (event) {
            var parameters = event.data.params;
            var reportParams = this._getParameterJson(parameters);
            this._refresh = true;
            $('#' + this._id + '_viewBlockContainer .e-reportviewer-viewerblockcontent table:first').attr('isviewclick', 'true');
            this.doAjaxPost("POST", this._actionUrl, JSON.stringify({ 'reportAction': this._reportAction.setParameters, 'parameters': reportParams }), "_setParameters");
        },

        _getParameterJson: function (parameters) {
            var reportParams = [];
            for (var index = 0; index < parameters.length; index++) {
                var parameter = parameters[index];
                var values = [];
                var labels = [];
                var nullable = false;
                var isNull;
                if (parameter.IsMultiValue) {
                    var ddl = $('#' + parameter.ControlId).data("ejDropDownList");
                    var selectedIndex = ddl.model.selectedItems;
                    for (var optIndex = 0; optIndex < selectedIndex.length; optIndex++) {
                        labels.push(ddl.selectOptionItems[selectedIndex[optIndex]].text);
                        values.push(ddl.selectOptionItems[selectedIndex[optIndex]].value);
                    }
                } else if (parameter.DataType == "DateTime") {
                    isNull = $('#' + parameter.ControlId + '_div').find('.e-chkbox-wrap').attr('aria-checked');
                    if (isNull == 'true') {
                        nullable = true;
                    } else {
                        var dateTime = $('#' + parameter.ControlId).data("ejDatePicker").getValue();
                        labels.push(dateTime);
                        values.push(dateTime);
                    }
                } else if (parameter.AvailableValues != null) {
                    var ddlVal = $('#' + parameter.ControlId).data("ejDropDownList").getSelectedValue();
                    var ddltxt = $('#' + parameter.ControlId).data("ejDropDownList").getSelectedItem();
                    labels.push(ddltxt);
                    values.push(ddlVal);
                } else if (parameter.DataType == "Boolean") {
                    isNull = $('#' + parameter.ControlId + '_div').find('.e-chkbox-wrap').attr('aria-checked');
                    if (isNull == 'true') {
                        nullable = true;
                    } else {
                        var radio1 = $('#' + parameter.ControlId + '_01').data("ejRadioButton").model.checked;
                        labels.push(radio1);
                        values.push(radio1);
                    }
                } else {
                    isNull = $('#' + parameter.ControlId + '_div').find('.e-chkbox-wrap').attr('aria-checked');
                    if (isNull == 'true') {
                        nullable = true;
                    } else {
                        var val = $('#' + parameter.ControlId).val();
                        labels.push(val);
                        values.push(val);
                    }
                }

                var parameterValue = { Name: parameter.Name, Values: values, Labels: labels, Nullable: nullable };
                reportParams.push(parameterValue);
            }
            return reportParams;
        },

        _paramNullChange: function (event) {
            var id = event.model.id.split('~');
            var ejReport = $('#' + event.model.value).data("ejReportViewer");
            ejReport._paramElementDisable(id[0], event.model.name, event.isChecked);
        },

        _paramElementDisable: function (elementId, elementType, isEnable) {
            if (elementType == 'chkDateTime') {
                var dateTime = $('#' + elementId).data("ejDatePicker");
                if (isEnable) {
                    dateTime.disable();
                } else {
                    dateTime.enable();
                }
            } else if (elementType == 'chkTextbox') {
                if (isEnable) {
                    $('#' + elementId).attr('disabled', 'disabled');
                    $('#' + elementId).css('opacity', '0.45');
                } else {
                    $('#' + elementId).removeAttr('disabled');
                    $('#' + elementId).css('opacity', '1');
                }
            } else if (elementType == 'chkRadio') {
                var radio1 = $('#' + elementId + '_01').data("ejRadioButton");
                var radio2 = $('#' + elementId + '_02').data("ejRadioButton");
                if (isEnable) {
                    radio1.disable();
                    radio2.disable();
                } else {
                    radio1.enable();
                    radio2.enable();
                }
            }
        },

        _selectparamToolItem: function (enable) {
            if (!this.model.toolbarSettings.templateId) {
                var _ejToolbar = $('#' + this._id + '_toolbarContainer').data("ejToolbar");
                if (enable) {
                    _ejToolbar.selectItemByID(this._id + '_ejtb_parameter');
                } else {
                    _ejToolbar.deselectItemByID(this._id + '_ejtb_parameter');
                }
            }
        },

        _showParameterBlock: function (enable) {
            if (enable) {
                $('#' + this._id + '_viewBlockContainer .e-reportviewer-viewerblockcellcontent').css("display", "block");
                $('#' + this._id + '_toolbarContainer').find('.e-reportviewer-parameter').parent().css("display", "block");
            } else {
                $('#' + this._id + '_viewBlockContainer .e-reportviewer-viewerblockcellcontent').css("display", "none");
                $('#' + this._id + '_toolbarContainer').find('.e-reportviewer-parameter').parent().css("display", "none");
            }

            if (this._isDevice) {
                this._showViewerBlock(enable);
                this._showViewerPage(!enable);
            }
        },

        _toggleParameterBlock: function () {
            this._selectparamToolItem(!($('#' + this._id + '_viewBlockContainer .e-reportviewer-viewerblockcellcontent').css("display") == 'block'));
            $('#' + this._id + '_viewBlockContainer .e-reportviewer-viewerblockcellcontent').toggle();
            this._setContainerSize();

            if (this._isDevice) {
                var isEnable = $('#' + this._id + '_viewBlockContainer .e-reportviewer-viewerblockcellcontent').css("display") == "block";
                this._showViewerBlock(isEnable);
                this._showViewerPage(!isEnable);
            }
        },
        //-------------------- Render Parameter Block and Actions [End] -------------------------//

        //-------------------- Render Exception Block and Actions [Start] -------------------------//
        _renderExceptionBlock: function (errmsg) {
            this._resetExceptionBlock();
            $('#' + this._id + '_viewBlockContainer .e-reportviewer-viewerblockcellcontent').css("display", "none");
            var table = ej.buildTag("table.errmsg", "", {}, {});
            var $row = ej.buildTag("tr", "", {}, {});
            var $cell = ej.buildTag("td.e-reportviewer-viewerblockcellcontent", "", {}, {});
            $row.append($cell);
            $cell.append("<h5>Report Viewer encountered some issues loading this report. Please click <a id=" + this._id + '_viewBlockContainer_errmsg' + " href=javascript:void(0); >here</a>  to see the error details</h5>");
            table.append($row);
            $('#' + this._id + '_viewBlockContainer').append(table);
            this._on($('#' + this._id + '_viewBlockContainer_errmsg'), "click", { err: errmsg }, this._errorPopupblock);
        },

        _errorPopupblock: function (event) {
            alert(event.data.err);
        },

        _resetExceptionBlock: function () {
            $('#' + this._id + '_viewBlockContainer table.errmsg').remove();
        },

        _renderExcpetion: function (exception) {
            this._showloadingIndicator(false);
            //$('#' + this._id + '_viewerContainer').html('Exception:' + exception);
            this._renderExceptionBlock(exception);
            this._showViewerBlock(true);
        },
        //-------------------- Render Exception Block and Actions [End] -------------------------//

        //-------------------- Apply Page Style and Actions [Start] -------------------------//
        _setContainerSize: function () {
            var ctrlheight = $('#' + this._id).height();
            var viewerblockHeight = 0;
            if (!this._isDevice) {
                var viewerContainer = $('#' + this._id + '_viewBlockContainer');
                if (viewerContainer.css('display').toLowerCase() == 'block') {
                    viewerblockHeight = $('#' + this._id + '_viewBlockContainer').height();
                }
            }

            var toolbarHeight = this.model.toolbarSettings.templateId ? $('#' + this.model.toolbarSettings.templateId).height() : $('#' + this._id + '_toolbarContainer').height();

            var calcHeight = ctrlheight - viewerblockHeight - toolbarHeight - 4;
            $('#' + this._id + '_viewerContainer').css({ "height": calcHeight + "px" });
        },

        _setPageSize: function (pageHeight, pageWidth, headerHeight, footerHeight) {
            var totalpageHeight = pageHeight + headerHeight + footerHeight + 3;

            $('#' + this._id + '_pageviewContainer').css({ "width": pageWidth + "px", "height": totalpageHeight + "px" });
            $('#' + this._id + '_pageviewheaderContainer').css({ "width": pageWidth + "px", "height": headerHeight + "px" });
            $('#' + this._id + '_pageviewfooterContainer').css({ "width": pageWidth + "px", "height": footerHeight + "px" });
            $('#' + this._id + '_pageviewbodyContainer').css({ "height": pageHeight + "px", "width": pageWidth + "px" });
            var scalePageSize = $('#' + this._id + '_pageviewContainer')[0].getBoundingClientRect();

            $('#' + this._id + '_pageviewOuterline').css({ "width": scalePageSize.width + "px", "height": scalePageSize.height + "px" });
        },

        _renderPageModels: function (pageData) {
            $('#' + this._id + '_pageviewheaderContainer').empty();
            $('#' + this._id + '_pageviewfooterContainer').empty();
            $('#' + this._id + '_pageviewbodyContainer').empty();
            if (pageData.HeaderStyleModel) {
                this._applyPageStyle($('#' + this._id + '_pageviewContainer .pageHeaderBorder'), pageData.HeaderStyleModel, false);
            }

            if (pageData.FooterStyleModel) {
                this._applyPageStyle($('#' + this._id + '_pageviewContainer .pageFooterBorder'), pageData.FooterStyleModel, false);
            }

            if (pageData.BodyStyleModel) {
                this._applyPageStyle($('#' + this._id + '_pageviewContainer .pageBodyBorder'), pageData.BodyStyleModel, false);
            }

            if (pageData.HeaderModel && pageData.HeaderModel.length > 0) {
                this._renderPageControls(pageData.HeaderModel, $('#' + this._id + '_pageviewheaderContainer'), false, null);
            }

            if (pageData.FooterModel && pageData.FooterModel.length > 0) {
                this._renderPageControls(pageData.FooterModel, $('#' + this._id + '_pageviewfooterContainer'), false, null);
            }

            if (pageData.PageModel && pageData.PageModel.length > 0) {
                this._renderPageControls(pageData.PageModel, $('#' + this._id + '_pageviewbodyContainer'), false, null);
            }

            if ($('.txtToggle_' + this._id).length > 0) {
                this._drillAction();
            }

            if ($('.drillAction_' + this._id).length > 0) {
                this._drillThroughAction();
            }

            if (this._isDocumentMap && !this._printMode) {
                this._documentMapAction();
                this._isDocumentMap = false;
                this._pageDocMapFlag = true;
            }

            if (this._parentPageXY) {
                $('#' + this._id + '_viewerContainer').animate({ scrollTop: this._parentPageXY.y, scrollLeft: this._parentPageXY.x }, 1000);
                this._parentPageXY = null;
            }

            if (this._isDevice) {
                this._renderPageInfoPopup();
            }

            if (this.model.printMode) {
                $('#' + this._id + '_pageviewContainer').css({ 'padding-left': this._pageModel.MarginLeft, 'padding-top': this._pageModel.MarginTop, 'padding-right': this._pageModel.MarginRight, 'padding-bottom': this._pageModel.MarginBottom });
            } else {
                $('#' + this._id + '_pageviewContainer').css('padding','0px');
            }
        },

        _applyPageStyle: function (page, styleModel, printMode) {
            var resourceName = printMode ? 'PrintImage' : 'GetResource';
            page.css("background-color", styleModel.BackgroundColor);
            if (styleModel.BackgroundSrc) {
                page.css("background-image", "url(" + this.model.reportServiceUrl + '/' + resourceName + '/?key=' + styleModel.BackgroundSrc + '&resourcetype=sfimg&isPrint=' + printMode + ")");
            }
            if (styleModel.Border) {
                this._applyBorderStyle(styleModel.Border, page);
            }
        },

        _renderPageControls: function (pageModelItems, renderArea, printMode, printPageId) {
            var $reportItemsContainer = ej.buildTag("div", "", {}, {});
            renderArea.html($reportItemsContainer);
            for (var index = 0; index < pageModelItems.length; index++) {
                var reportItem = pageModelItems[index];
                if (reportItem.ItemType == "TextBoxModel") {
                    var textboxControl = this._renderTextBoxControl(reportItem, false, null, printMode, printPageId);
                    $reportItemsContainer.append(textboxControl);
                }
                else if (reportItem.ItemType == "LineModel") {
                    var lineControl = this._renderLineControl(reportItem, null, printMode, printPageId);
                    $reportItemsContainer.append(lineControl);
                }
                else if (reportItem.ItemType == "ImageModel") {
                    this._renderImageControl(reportItem, $reportItemsContainer, null, printMode, printPageId);
                }
                else if (reportItem.ItemType == "TablixModel") {
                    this._renderTablixControl(reportItem, $reportItemsContainer, null, printMode, printPageId);
                }
                else if (reportItem.ItemType == "GaugeModel") {
                    this._renderGaugeControl(reportItem, $reportItemsContainer, null, printMode, printPageId);
                }
                else if (reportItem.ItemType == "RectangleModel") {
                    this._renderRectangleControl(reportItem, false, $reportItemsContainer, null, printMode, printPageId);
                }
                else if (reportItem.ItemType == "ChartModel") {
                    this._renderChartControl(reportItem, $reportItemsContainer, null, printMode, printPageId);
                }
                else if (reportItem.ItemType == "MapModel") {
                    this._renderMapControl(reportItem, $reportItemsContainer, null, printMode, printPageId);
                }
                else if (reportItem.ItemType == "SubReportModel") {
                    this._renderSubReportControl(reportItem, false, $reportItemsContainer, null, printMode, printPageId);
                }
            }
        },
        //-------------------- Apply Page Style and Actions [End] -------------------------//

        //-------------------- Render the Page ReportItems [start] -------------------------//
        _controlKeyGenerator: function (currentKey, parentId, printMode, printPageId) {
            var objId = !ej.isNullOrUndefined(parentId) ? '^' + parentId : '';
            if (printMode) {
                objId = objId.length == 0 ? '^' + printPageId : objId + '_' + printPageId;
            }
            return currentKey + objId;
        },

        _applyBorderStyle: function (border, tag) {
            if (border.Default) {
                tag.css("border-width", border.Default.Thickness + 'px');
                tag.css("border-color", border.Default.BorderBrush);
                tag.css("border-style", border.Default.BorderStyle);
            }

            if (border.LeftBorder) {
                tag.css("border-left-width", border.LeftBorder.Thickness + 'px');
                tag.css("border-left-color", border.LeftBorder.BorderBrush);
                tag.css("border-left-style", border.LeftBorder.BorderStyle);
            }

            if (border.RightBorder) {
                tag.css("border-right-width", border.RightBorder.Thickness + 'px');
                tag.css("border-right-color", border.RightBorder.BorderBrush);
                tag.css("border-right-style", border.RightBorder.BorderStyle);
            }

            if (border.TopBorder) {
                tag.css("border-top-width", border.TopBorder.Thickness + 'px');
                tag.css("border-top-color", border.TopBorder.BorderBrush);
                tag.css("border-top-style", border.TopBorder.BorderStyle);
            }

            if (border.BottomBorder) {
                tag.css("border-bottom-width", border.BottomBorder.Thickness + 'px');
                tag.css("border-bottom-color", border.BottomBorder.BorderBrush);
                tag.css("border-bottom-style", border.BottomBorder.BorderStyle);
            }
        },

        _applyTextStyle: function (textboxModel, tag) {
            if (textboxModel.Padding.Top) {
                tag.css("padding-top", textboxModel.Padding.Top);
            }

            if (textboxModel.Padding.Right) {
                tag.css("padding-right", textboxModel.Padding.Right);
            }

            if (textboxModel.Padding.Bottom) {
                tag.css("padding-bottom", textboxModel.Padding.Bottom);
            }

            if (textboxModel.Padding.Left) {
                tag.css("padding-left", textboxModel.Padding.Left);
            }

            if (textboxModel.TextBoxBackgroundColor) {
                tag.css("backgroundColor", textboxModel.TextBoxBackgroundColor);
            }

            this._applyBorderStyle(textboxModel.Border, tag);

            var tagName = tag.get(0).tagName;
            if (tagName.toLowerCase() == 'td') {
                var padwidthadjust = parseInt(tag.css("padding-left"), 10) + parseInt(tag.css("padding-right"), 10);
                var padheightadjust = parseInt(tag.css("padding-top"), 10) + parseInt(tag.css("padding-bottom"), 10);

                var borderwidthadjust = parseInt(tag.css("border-left-width"), 10) + parseInt(tag.css("border-right-width"), 10);
                var borderheightadjust = parseInt(tag.css("border-top-width"), 10) + parseInt(tag.css("border-bottom-width"), 10);

                var adjusWidth = (parseInt(tag.css('width'), 10) - (padwidthadjust + borderwidthadjust));
                var adjusHeight = (parseInt(tag.css('height'), 10) - (padheightadjust + borderheightadjust));

                //tag.css('width', adjusWidth + 'px');
                //tag.css('height', adjusHeight + 'px');
            }

        },


        //-------------------- Render the Tablix ReportItem [start] -------------------------//
        _renderTablixControl: function (tablixmodel, renderarea, parentId, printMode, printPageId) {
            var proxy = this;
            var ctrlId = proxy._controlKeyGenerator(proxy._id + '_' + tablixmodel.Name, parentId, printMode, printPageId);
            var tablixDiv = ej.buildTag("div", "", {}, { 'id': ctrlId });
            renderarea.append(tablixDiv);
            tablixDiv.css("position", "absolute");
            tablixDiv.css("top", tablixmodel.Top);
            tablixDiv.css("left", tablixmodel.Left);
            tablixDiv.css("width", tablixmodel.Width);
            tablixDiv.css("height", tablixmodel.Height);

            var tablixtable = ej.buildTag("table", "", { 'border-collapse': 'collapse', 'font-size': '8pt' }, { 'cellspacing': '0', 'cellpadding': '0' });
            tablixDiv.append(tablixtable);

            if (tablixmodel.CellModels && tablixmodel.CellModels.length > 0) {
                var emptyRow = ej.buildTag("tr", "", { 'padding': '0px', 'margin': '0px', 'height': '0px' }, {});
                tablixtable.append(emptyRow);
                var colindex;
                for (colindex = 0; colindex < tablixmodel.CellModels[0].length; colindex++) {
                    var emptyCol = ej.buildTag("td", "", { 'padding': '0px', 'margin': '0px', 'width': tablixmodel.ColWidths[colindex] + 'px' }, { 'rowSpan': '1', 'colSpan': '1' });
                    emptyRow.append(emptyCol);
                }

                for (var rowindex = 0; rowindex < tablixmodel.CellModels.length; rowindex++) {
                    var tableRow = ej.buildTag("tr", "", { 'padding': '0px', 'margin': '0px', 'text-align': 'left', 'height': tablixmodel.RowHeights[rowindex] + 'px' }, { 'vAlign': 'top' });
                    tablixtable.append(tableRow);
                    var row = tablixmodel.CellModels[rowindex];
                    for (colindex = 0; colindex < row.length; colindex++) {
                        var tableCol = ej.buildTag("td", "", { 'padding': '0px', 'margin': '0px', 'text-align': 'left' }, { 'rowSpan': '1', 'colSpan': '1' });
                        tableRow.append(tableCol);
                        var cell = row[colindex].ItemModel;
                        var border = row[colindex].Border;
                        if (border) {
                            proxy._applyBorderStyle(border, tableCol);
                        }

                        if (cell) {
                            if (cell.ItemType == "TextBoxModel") {
                                var textboxControl = proxy._renderTextBoxControl(cell, true, ctrlId + 'tablixRow' + rowindex + 'xCol' + colindex, printMode, printPageId);
                                //textboxControl = textboxControl.find("div");
                                proxy._applyTextStyle(cell, tableCol);
                                tableCol.append(textboxControl);
                            } else if (cell.ItemType == "LineModel") {
                                var lineControl = proxy._renderLineControl(cell, ctrlId + 'tablixRow' + rowindex + 'xCol' + colindex, printMode, printPageId);
                                lineControl.css("position", "relative");
                                tableCol.append(lineControl);
                            } else if (cell.ItemType == "ImageModel") {
                                proxy._renderImageControl(cell, tableCol, ctrlId + 'tablixRow' + rowindex + 'xCol' + colindex, printMode, printPageId);
                            } else if (cell.ItemType == "TablixModel") {
                                proxy._renderTablixControl(cell, tableCol, ctrlId + 'tablixRow' + rowindex + 'xCol' + colindex, printMode, printPageId);
                            } else if (cell.ItemType == "GaugeModel") {
                                proxy._renderGaugeControl(cell, tableCol, ctrlId + 'tablixRow' + rowindex + 'xCol' + colindex, printMode, printPageId);
                            } else if (cell.ItemType == "RectangleModel") {
                                proxy._renderRectangleControl(cell, true, tableCol, ctrlId + 'tablixRow' + rowindex + 'xCol' + colindex, printMode, printPageId);
                            } else if (cell.ItemType == "ChartModel") {
                                this._renderChartControl(cell, tableCol, ctrlId + 'tablixRow' + rowindex + 'xCol' + colindex, printMode, printPageId);
                            } else if (cell.ItemType == "MapModel") {
                                this._renderMapControl(cell, tableCol, ctrlId + 'tablixRow' + rowindex + 'xCol' + colindex, printMode, printPageId);
                            } else if (cell.ItemType == "SubReportModel") {
                                proxy._renderSubReportControl(cell, true, tableCol, ctrlId + 'tablixRow' + rowindex + 'xCol' + colindex, printMode, printPageId);
                            }
                        } else {
                            tableCol.css('display', 'none');
                        }
                    }
                }
                this._applyTablixCellSpans(tablixtable, tablixmodel.CoveredRanges, tablixmodel.SpanCells);
            }
        },

        _applyTablixCellSpans: function (tablix, coveredRanges, spanCells) {
            if (coveredRanges && coveredRanges.length > 0) {
                for (var i = 0; i < coveredRanges.length; i++) {
                    var coverRange = coveredRanges[i];
                    if (coverRange.Top == coverRange.Bottom && coverRange.Left == coverRange.Right) {
                        this._applySpanStyles(spanCells[coverRange.CellKey], $(tablix.children().children()[coverRange.Top].children[coverRange.Left - 1]));
                    } else if (coverRange.Top <= coverRange.Bottom && coverRange.Left <= coverRange.Right) {
                        var j;
                        if (coverRange.CellKey != '' && coverRange.CellKey.length > 1) {
                            this._applySpanStyles(spanCells[coverRange.CellKey], $(tablix.children().children()[coverRange.Top].children[coverRange.Left - 1]));
                        }

                        for (j = coverRange.Top; j < (coverRange.Top + (coverRange.Bottom - coverRange.Top)) ; j++) {
                            $(tablix.children().children()[j + 1].children[coverRange.Left - 1]).css("display", "none");
                        }

                        for (j = coverRange.Left; j < (coverRange.Left + (coverRange.Right - coverRange.Left)) ; j++) {
                            $(tablix.children().children()[coverRange.Top].children[j]).css("display", "none");
                        }

                        $(tablix.children().children()[coverRange.Top].children[coverRange.Left - 1]).attr("colspan", (coverRange.Right - coverRange.Left) + 1);
                        $(tablix.children().children()[coverRange.Top].children[coverRange.Left - 1]).attr("rowspan", (coverRange.Bottom - coverRange.Top) + 1);
                    }
                }
            }
        },

        _applySpanStyles: function (cellInfo, tag) {
            var cellModel = cellInfo.ItemModel;
            var cellborder = cellInfo.Border;
            if (cellModel.ItemType == "TextBoxModel") {
                if (tag.css('display') == 'none') {
                    var textboxCtrl = this._renderTextBoxControl(cellModel, true);
                    textboxCtrl = textboxCtrl.find("div");
                    this._applyTextStyle(cellModel, tag);
                    tag.append(textboxCtrl);
                    tag.css('display', '');
                }

                if (cellborder) {
                    this._applyBorderStyle(cellborder, tag);
                }
            }
        },
        //-------------------- Render the Tablix ReportItem [end] -------------------------//

        _renderTextBoxControl: function (textboxModel, isTablixCell, parentId, printMode, printPageId) {
            var ctrlId = this._controlKeyGenerator(this._id + '_' + textboxModel.Name, parentId, printMode, printPageId);

            var textboxDiv = ej.buildTag("div", "", {}, { 'id': ctrlId, title: textboxModel.ToolTip });

            if (!isTablixCell) {
                textboxDiv.css("position", "absolute");
                textboxDiv.css("top", textboxModel.Top);
                textboxDiv.css("left", textboxModel.Left);
                textboxDiv.css("width", textboxModel.Width);
                textboxDiv.css("height", textboxModel.Height);
                this._applyTextStyle(textboxModel, textboxDiv);
            }

            if (textboxModel.ToggleInfo) {
                var cssToggleName = textboxModel.IsToggle ? 'e-reportviewer-expandtoggle' : 'e-reportviewer-collapsetoggle';
                var toogleiconDiv = ej.buildTag('div.' + cssToggleName + ' txtToggle_' + this._id, '', { 'display': 'block', 'width': '14px', 'height': '14px', 'padding-right': '1px','margin-Top': '1px'  }, {});
                toogleiconDiv.data('drillObj', textboxModel.ToggleInfo);
                textboxDiv.append(toogleiconDiv);
                textboxDiv.css('display', 'flex');
            }

            if (textboxModel.ActionInfo) {
                textboxDiv.data('actionObj', textboxModel.ActionInfo);
                textboxDiv.addClass('drillAction_' + this._id + ' e-reportviewer-drillhover');
            }

            for (var indexi = 0; indexi < textboxModel.Paragraphval.length; indexi++) {
                var $textboxParaDiv = ej.buildTag("div", "", {});
                var paragraph = textboxModel.Paragraphval[indexi];
                for (var indexj = 0; indexj < paragraph.Runs.length; indexj++) {
                    var run = paragraph.Runs[indexj];
                    var $textboxSpanDiv = ej.buildTag("span", "", {});

                    if (run.ActionInfo) {
                        $textboxSpanDiv.data('actionObj', run.ActionInfo);
                        $textboxSpanDiv.addClass('drillAction_' + this._id + ' e-reportviewer-drillhover');
                    }

                    if (run.Style.Font.FontFamily) {
                        $textboxSpanDiv.css("font-family", run.Style.Font.FontFamily);
                    }

                    if (run.Style.Font.FontSize) {
                        $textboxSpanDiv.css("font-size", run.Style.Font.FontSize);
                    }

                    if (run.Style.Font.FontStyle) {
                        $textboxSpanDiv.css("font-style", run.Style.Font.FontStyle);
                    }

                    if (run.Style.Font.FontWeight) {
                        $textboxSpanDiv.css("font-weight", run.Style.Font.FontWeight);
                    }

                    if (run.Style.TextColor) {
                        $textboxSpanDiv.css("color", run.Style.TextColor);
                    } else {
                        $textboxSpanDiv.css("color", "black");
                    }

                    if (run.Style.TextDecoration) {
                        $textboxSpanDiv.css("text-decoration", run.Style.TextDecoration);
                    }

                    if (run.RunText) {
                        $textboxSpanDiv.append(run.RunText);
                    }
                    $textboxParaDiv.append($textboxSpanDiv);
                }

                if (paragraph.TextAlignment) {
                    $textboxParaDiv.css("text-align", paragraph.TextAlignment);
                }

                if (paragraph.LeftIndent) {
                    $textboxParaDiv.css("padding-left", paragraph.LeftIndent);
                }

                if (paragraph.RightIndent) {
                    $textboxParaDiv.css("padding-right", paragraph.RightIndent);
                }

                if (paragraph.SpaceBefore) {
                    $textboxParaDiv.css("padding-top", paragraph.SpaceBefore);
                }

                if (paragraph.SpaceAfter) {
                    $textboxParaDiv.css("padding-bottom", paragraph.SpaceAfter);
                }

                if (paragraph.ListStyle) {
                    $textboxParaDiv.css("list-style-type", paragraph.ListStyle);
                }

                if (paragraph.HangingIndent) {
                    $textboxParaDiv.css("text-indent", paragraph.HangingIndent);
                }

                textboxDiv.append($textboxParaDiv);
            }
            return textboxDiv;
        },

        //-------------------- Render the Chart ReportItem [start] -------------------------//
        _renderChartControl: function (chartModel, parentObj, parentId, printMode, printPageId) {
            if (chartModel.ChartAreas) {
                var chartAreaHeight = ((chartModel.Height) / (chartModel.ChartAreas.length));
                var ctrlId = this._controlKeyGenerator(this._id + '_' + chartModel.Name, parentId, printMode, printPageId);
                var chartDiv = ej.buildTag("div", "", {}, { 'id': ctrlId, 'title': chartModel.ToolTip });
                parentObj.append(chartDiv);
                chartDiv.css("position", "absolute");

                if (chartModel.ChartStyle) {
                    if (chartModel.ChartStyle.Border) {
                        this._applyBorderStyle(chartModel.ChartStyle.Border, chartDiv);
                    }

                    if (chartModel.ChartStyle.FillStyle) {
                        chartDiv.css('background-color', chartModel.ChartStyle.FillStyle.BackgroundColor);
                    }
                }

                if (chartModel.Top) {
                    chartDiv.css("top", chartModel.Top);
                }

                if (chartModel.Left) {
                    chartDiv.css("left", chartModel.Left);
                }

                if (chartModel.Width) {
                    chartDiv.css("width", chartModel.Width);
                }

                if (chartModel.Height) {
                    chartDiv.css("height", chartModel.Height);
                }

                for (var areaIndex = 0; areaIndex < chartModel.ChartAreas.length; areaIndex++) {
                    var chartArea = chartModel.ChartAreas[areaIndex];
                    var chartChildDiv = null;
                    if (printMode) {
                        chartChildDiv = ej.buildTag("div", "", {}, { 'id': this._id + '_' + chartModel.Name + '_' + chartArea.Name + '_' + printPageId, 'title': chartModel.ToolTip });
                    } else {
                        chartChildDiv = ej.buildTag("div", "", {}, { 'id': this._id + '_' + chartModel.Name + '_' + chartArea.Name, 'title': chartModel.ToolTip });
                    }
                    chartDiv.append(chartChildDiv);
                    chartChildDiv.css("height", chartAreaHeight);
                    chartChildDiv.css("margin", "1px");
                    chartChildDiv.ejChart();
                    var ejChartCtrl = chartChildDiv.data("ejChart");

                    var ejchartSeries = ejChartCtrl.model.series[0];
                    ejChartCtrl.model.series.pop(ejchartSeries);

                    if (chartArea.Style) {
                        if (chartArea.Style.FillStyle && chartArea.Style.FillStyle.BackgroundColor != "#00ffffff") {
                            ejChartCtrl.model.chartArea.background = chartArea.Style.FillStyle.BackgroundColor;
                        }
                        if (chartArea.Style.Border && chartArea.Style.Border.Default) {
                            if (chartArea.Style.Border.Default.BorderBrush) {
                                ejChartCtrl.model.chartArea.border.color = chartArea.Style.Border.Default.BorderBrush;
                            }
                            if (chartArea.Style.Border.Default.BorderStyle == "Default" || chartArea.Style.Border.Default.BorderStyle == "None") {
                                ejChartCtrl.model.chartArea.border.width = 0;
                                ejChartCtrl.model.chartArea.border.opacity = 0;
                            }
                            else {
                                ejChartCtrl.model.chartArea.border.width = chartArea.Style.Border.Default.Thickness;
                            }
                        }
                        else {
                            ejChartCtrl.model.chartArea.border.opacity = 0;
                        }
                    }

                    if (areaIndex == 0 && chartModel.ChartTiles.length > 0) {
                        var charttile = chartModel.ChartTiles[areaIndex];
                        if (charttile.Caption && charttile.Visible) {
                            ejChartCtrl.model.title.text = charttile.Caption;
                            if (charttile.Style) {
                                if (charttile.Style.Color) {
                                    ejChartCtrl.model.title.font.color = charttile.Style.Color;
                                }

                                if (charttile.Style.Font) {
                                    if (charttile.Style.Font.FontFamily) {
                                        ejChartCtrl.model.title.font.fontFamily = charttile.Style.Font.FontFamily;
                                    }

                                    if (charttile.Style.Font.FontStyle) {
                                        ejChartCtrl.model.title.font.fontStyle = charttile.Style.Font.FontStyle;
                                    }

                                    if (charttile.Style.Font.FontWeight) {
                                        ejChartCtrl.model.title.font.fontWeight = charttile.Style.Font.FontWeight;
                                    }

                                    if (charttile.Style.Font.FontSize != 0) {
                                        ejChartCtrl.model.title.font.size = charttile.Style.Font.FontSize + 'px';
                                    }
                                }
                            }
                        }
                    }

                    for (var seriesIndex = 0; seriesIndex < chartArea.ChartSeries.length; seriesIndex++) {
                        var chartseries = chartArea.ChartSeries[seriesIndex];
                        var seriesObj = $.extend(true, {}, ejchartSeries);
                        var seriesPoints = seriesObj.points[0];
                        seriesObj.points = [];
                        seriesObj.type = chartseries.Type;
                        seriesObj.name = (chartseries.Legend && chartseries.Legend.LegendText) ? chartseries.Legend.LegendText : chartseries.Name;

                        if (chartseries.Type == "explodedpie") {
                            seriesObj.type = "pie";
                            seriesObj.explodeAll = true;
                            seriesObj.explodeOffset = 8;
                        }

                        else if (chartseries.Type == "explodeddoughnut") {
                            seriesObj.type = "doughnut";
                            seriesObj.explodeAll = true;
                            seriesObj.explodeOffset = 10;
                        }
                       
                        if (chartseries.Type == "stackingcolumn") {
                            seriesObj.isStacking = true;
                        } else {
                            seriesObj.isStacking = false;
                        }

                        var actionInfos = [];
                        for (var pointIndex = 0; pointIndex < chartseries.PointValues.length; pointIndex++) {
                            var pointObj = $.extend(true, {}, seriesPoints);
                            var point = chartseries.PointValues[pointIndex];
                            pointObj.YValues = [];

                            pointObj.X = pointObj.x = point.X;

                            if (point.Y) {
                                pointObj.Y = pointObj.y = point.Y;
                                pointObj.YValues.push(pointObj.Y);
                            }

                            if (point.Low) {
                                pointObj.YValues.push(point.Low);
                            }

                            if (point.High) {
                                pointObj.YValues.push(point.High);
                            }

                            if (point.Start) {
                                pointObj.YValues.push(point.Start);
                            }

                            if (point.End) {
                                pointObj.YValues.push(point.End);
                            }

                            if (chartseries.Type == "bubble") {
                                pointObj.size = 0.5;
                            }

                            if (point.ChartDataLabel && !point.ChartDataLabel.Visible) {
                                pointObj.text = "";
                            }
                            else {
                                if (point.ChartDataLabel && point.ChartDataLabel.Format) {
                                    pointObj.text = point.ChartDataLabel.Format
                                }
                            }

                            if (point.ActionInfo) {
                                actionInfos.push(point.ActionInfo);
                            }

                            if (chartModel.ColorPalette == "Custom") {
                                for (var paletteIndex = 0; paletteIndex < chartModel.CustomPaletteColors.length; paletteIndex++) {
                                    ejChartCtrl.model.colors[paletteIndex] = chartModel.CustomPaletteColors[paletteIndex];
                                }
                            }
                            else {
                                if (!chartseries.Type == "doughnut" || !chartseries.Type == "pie") {
                                    ejChartCtrl.model.colors = [];
                                }
                                else if (point.Style && point.Style.Color) {
                                    ejChartCtrl.model.colors[pointIndex] = point.Style.Color;
                                }
                            }

                            if (point.Style && point.Style.Color) {
                                seriesObj.fill = point.Style.Color;
                            }

                            seriesObj.points.push(pointObj);

                        }

                        if (actionInfos.length > 0) {
                            ejChartCtrl.model.pointRegionClick = this._drillThroughClick;
                            ejChartCtrl.ejreportid = this._id;
                            chartChildDiv.data(ejChartCtrl._id + 'actionObj_Series' + seriesIndex, actionInfos);
                            ejChartCtrl.model.pointRegionMouseMove = function chartSeriesMouseHover(event) {
                                if (ej.isNullOrUndefined($('#' + this._id).data(this._id + 'actionObj_Series' + event.data.region.SeriesIndex))) {
                                    return false;
                                }
                                $('#' + this._id + '_svg_Series' + event.data.region.SeriesIndex + '_Point' + event.data.region.Region.PointIndex).css("cursor", "pointer");
                            };
                        }

                        seriesObj.visibility = chartseries.Visibility ? 'visible' : 'hide';
                        seriesObj.tooltip.format = " x: #point.x#<br/>y: #point.y#";
                        seriesObj.tooltip.visible = true;
                        seriesObj.enableAnimation = false;

                        if (chartModel.CustomPaletteColors && (chartModel.ColorPalette) && chartModel.ColorPalette == "Custom") {
                            seriesObj.fill = chartModel.CustomPaletteColors[seriesIndex];
                        }

                        if (chartseries.Style && chartseries.Style.Border && chartseries.Style.Border.Default) {
                            if (chartseries.Style.Border.Default.BorderBrush) {
                                seriesObj.border.color = chartseries.Style.Border.Default.BorderBrush;
                            }

                            if (chartseries.Style.Border.Default.Thickness != 0) {
                                seriesObj.border.width = chartseries.Style.Border.Default.Thickness;
                            }

                            if (chartseries.Style.Border.Default.BorderStyle == "Dashed") {
                                seriesObj.border.dashArray = "2,4";
                            }
                            else if (chartseries.Style.Border.Default.BorderStyle == "Dotted") {
                                seriesObj.border.dashArray = "2,2";
                            }
                        }

                        if (chartseries.DataPointsStyle && chartseries.DataPointsStyle.length > 0) {
                            if (chartseries.Type == "scatter") {
                                var datapointStyle = chartseries.DataPointsStyle[0];
                                
                                if (datapointStyle.ChartMarker && datapointStyle.ChartMarker.MarkerType) {
                                    seriesObj.marker.shape = datapointStyle.ChartMarker.MarkerType;
                                } else {
                                    seriesObj.marker.shape = "Circle";
                                }
                                if (datapointStyle.ChartMarker.Color) {
                                    seriesObj.marker.fill = datapointStyle.ChartMarker.Color;
                                }

                                if (datapointStyle.ChartMarker.BorderColor) {
                                    seriesObj.marker.border.color = datapointStyle.ChartMarker.BorderColor;
                                }

                                if (datapointStyle.ChartMarker.Borderwidth) {
                                    seriesObj.marker.border.width = datapointStyle.ChartMarker.Borderwidth == 0 ? 2 : datapointStyle.ChartMarker.Borderwidth;
                                }

                                if (datapointStyle.ChartMarker.Size) {
                                    seriesObj.marker.size.height = seriesObj.marker.size.width = parseInt((datapointStyle.ChartMarker.Size == 0 || datapointStyle.ChartMarker.Size < 6)) ? 6 : (datapointStyle.ChartMarker.Size);
                                } else {
                                    seriesObj.marker.size.height = seriesObj.marker.size.width = 10;
                                }
                                seriesObj.marker.visible = true;
                            } else {
                                var datapointStyle = chartseries.DataPointsStyle[0];
                                if (datapointStyle.ChartMarker) {
                                    if (datapointStyle.ChartMarker.MarkerType) {
                                        seriesObj.marker.shape = datapointStyle.ChartMarker.MarkerType;
                                    }

                                    if (datapointStyle.ChartMarker.Color) {
                                        seriesObj.marker.fill = datapointStyle.ChartMarker.Color;
                                    }

                                    if (datapointStyle.ChartMarker.BorderColor) {
                                        seriesObj.marker.border.color = datapointStyle.ChartMarker.BorderColor;
                                    }

                                    if (datapointStyle.ChartMarker.Borderwidth) {
                                        seriesObj.marker.border.width = datapointStyle.ChartMarker.Borderwidth == 0 ? 2 : datapointStyle.ChartMarker.Borderwidth;
                                    }

                                    if (datapointStyle.ChartMarker.Size) {
                                        seriesObj.marker.size.height = seriesObj.marker.size.width = parseInt((datapointStyle.ChartMarker.Size == 0 || datapointStyle.ChartMarker.Size < 6)) ? 6 : (datapointStyle.ChartMarker.Size);
                                    }
                                }

                                if (chartseries.Visibility && datapointStyle.ChartMarker && datapointStyle.ChartMarker.MarkerType) {
                                    seriesObj.marker.visible = true;
                                }
                                else {
                                    seriesObj.marker.visible = false;
                                }

                                if (chartseries.PointValues.length > 0 && chartseries.PointValues[0].ChartDataLabel) {
                                    seriesObj.marker.dataLabel.visible = false;
                                    var pointDatalbl = chartseries.PointValues[0].ChartDataLabel;
                                    seriesObj.marker.dataLabel.visible = pointDatalbl.Visible;
                                    if (pointDatalbl.UsExvalueAsLabel && pointDatalbl.Visible) {
                                        seriesObj.marker.dataLabel.connectorLine.type = "line";
                                        seriesObj.marker.dataLabel.connectorLine.width = 0.5;
                                        seriesObj.marker.dataLabel.textPosition = "top";
                                        if (pointDatalbl.TextColor) {
                                            seriesObj.marker.dataLabel.font.color = pointDatalbl.TextColor;
                                        }
                                    }
                                }
                            }
                        }
                        ejChartCtrl.model.series.push(seriesObj);
                    }

                    if (chartModel.ChartLegends && chartModel.ChartLegends.length > 0) {
                        var chartLegend = chartModel.ChartLegends[0];
                        ejChartCtrl.model.legend.visible = chartLegend.Visibilty;

                        if (chartLegend.Alignment) {
                            ejChartCtrl.model.legend.alignment = chartLegend.Alignment;
                        }

                        if (chartLegend.Position) {
                            ejChartCtrl.model.legend.position = chartLegend.Position;
                        }

                        if (chartLegend.Style) {
                            if (chartLegend.Style.Color) {
                                ejChartCtrl.model.legend.font.color = chartLegend.Style.Color;
                            }

                            if (chartLegend.Style.Font) {
                                if (chartLegend.Style.Font.FontFamily) {
                                    ejChartCtrl.model.legend.font.fontFamily = chartLegend.Style.Font.FontFamily;
                                }

                                if (chartLegend.Style.Font.FontStyle) {
                                    ejChartCtrl.model.legend.font.fontStyle = chartLegend.Style.Font.FontStyle;
                                }

                                if (chartLegend.Style.Font.FontWeight) {
                                    ejChartCtrl.model.legend.font.fontWeight = chartLegend.Style.Font.FontWeight;
                                }

                                if (chartLegend.Style.Font.FontSize != 0) {
                                    ejChartCtrl.model.legend.font.size = chartLegend.Style.Font.FontSize + 'px';
                                }
                            }

                            if (chartLegend.Style.Border && chartLegend.Style.Border.Default) {
                                if (chartLegend.Style.Border.Default.BorderBrush) {
                                    ejChartCtrl.model.legend.border.color = chartLegend.Style.Border.Default.BorderStyle == "Default" ? "transparent" : chartLegend.Style.Border.Default.BorderBrush;
                                }

                                if (chartLegend.Style.Border.Default.Thickness != 0) {
                                    ejChartCtrl.model.legend.border.width = chartLegend.Style.Border.Default.Thickness;
                                }
                            }
                        }
                    }
                    else {
                        ejChartCtrl.model.legend.visible = false;
                    }

                    if (ejChartCtrl.model.series[0].type != 'polar' && ejChartCtrl.model.series[0].type != 'radar') {
                        //X AXIS
                        for (var chartAreaXaxisIndex = 0; chartAreaXaxisIndex < chartArea.ChartAreaXAxis.length; chartAreaXaxisIndex++) {
                            var chartXAxis = chartArea.ChartAreaXAxis[chartAreaXaxisIndex];
                            if (chartXAxis.Name.toString().toLowerCase() == "primary") {
                                this._applyAxis(chartXAxis, ejChartCtrl.model.primaryXAxis, 'XAxis', 'primary');
                            }
                            else if (chartXAxis.Name.toString().toLowerCase() == "secondary") {
                                if (!ej.isNullOrUndefined(ejChartCtrl.model.axes)) {
                                    var xaxesObj = $.extend(true, {}, ejChartCtrl.model.axes[0]);
                                    this._applyAxis(chartXAxis, xaxesObj, 'XAxis', 'secondary');
                                    xaxesObj.opposedPosition = true;
                                    xaxesObj.orientation = 'Horizontal';
                                    ejChartCtrl.model.axes.push(xaxesObj);
                                }
                            }
                        }

                        //Y AXIS
                        for (var chartAreaYaxisIndex = 0; chartAreaYaxisIndex < chartArea.ChartAreaYAxis.length; chartAreaYaxisIndex++) {
                            var chartYAxis = chartArea.ChartAreaYAxis[chartAreaYaxisIndex];
                            if (chartYAxis.Name.toString().toLowerCase() == "primary") {
                                this._applyAxis(chartYAxis, ejChartCtrl.model.primaryYAxis, 'YAxis', 'primary');
                            }
                            else if (chartYAxis.Name.toString().toLowerCase() == "secondary") {
                                if (!ej.isNullOrUndefined(ejChartCtrl.model.axes)) {
                                    var yaxesObj = $.extend(true, {}, ejChartCtrl.model.axes[0]);
                                    yaxesObj.opposedPosition = true;
                                    yaxesObj.orientation = "vertical";
                                    this._applyAxis(chartYAxis, yaxesObj, 'YAxis', 'secondary');
                                    ejChartCtrl.model.axes.push(yaxesObj);
                                }
                            }
                        }
                    }

                    ejChartCtrl.redraw();
                }
            }
        },

        _applyAxis: function(axisModelProp, axisName, axisSpecific, axistype) {
            if (axisModelProp.ChartAxisTitle.Caption) {
                axisName.title.text = axisModelProp.ChartAxisTitle.Caption;
            }

            if (axisModelProp.ChartAxisTitle.TextColor) {
                axisName.title.font.color = axisModelProp.ChartAxisTitle.TextColor;
            }

            if (axisModelProp.ChartAxisTitle.Font) {
                if (axisModelProp.ChartAxisTitle.Font.FontSize) {
                    axisName.title.font.size = axisModelProp.ChartAxisTitle.Font.FontSize;
                }

                if (axisModelProp.ChartAxisTitle.Font.FontFamily) {
                    axisName.title.font.fontFamily = axisModelProp.ChartAxisTitle.Font.FontFamily;
                }

                if (axisModelProp.ChartAxisTitle.Font.FontStyle) {
                    axisName.title.font.fontStyle = axisModelProp.ChartAxisTitle.Font.FontStyle;
                }
            }

            if ((axisModelProp.Maximum && axisModelProp.Maximum != "NaN" && axisModelProp.Maximum != 0) && (axisModelProp.Minimum && axisModelProp.Minimum != "NaN" && axisModelProp.Minimum != 0)) {
                axisName.range.max = axisModelProp.Maximum;
                axisName.range.min = axisModelProp.Minimum;
            }
            if (axisModelProp.Interval != "NaN") {
                //axisName.range.interval = axisModelProp.Interval;
            }

            if (axisModelProp.IntervalType) {
                //axisName.intervalType = axisModelProp.IntervalType;
            }

            if (axisModelProp.ChartMajorGridLines) {
                this._applyGridLines(axisModelProp.ChartMajorGridLines, axisName.majorGridLines);
            }

            if (axisModelProp.ChartMinorGridLines) {
                this._applyGridLines(axisModelProp.ChartMinorGridLines, axisName.minorGridLines);
            }

            if (axisModelProp.ChartMajorTickMarks) {
                this._applyTickMarks(axisModelProp.ChartMajorTickMarks, axisName.majorTickLines);
            }

            if (axisModelProp.ChartMinorTickMarks) {
                this._applyTickMarks(axisModelProp.ChartMinorTickMarks, axisName.minorTickLines);
            }

            if (axisModelProp.LabelsAutoFitEnabled){
                if (axisModelProp.PrExventWordWrap){
                    if (axisModelProp.PrExventLabelOffset) {
                        if (axisModelProp.AllowLabelRotation) {
                            axisName.labelIntersectAction = axisModelProp.AllowLabelRotation;
                        }
                        else {
                            axisName.labelIntersectAction = 'multiplerows';
                        }
                    }
                }
                else {
                    axisName.labelIntersectAction = 'wrap';
                }
            }
            else {
                    axisName.labelRotation = axisModelProp.Angle;
            }

            if (axisModelProp.LineColor) {
                axisName.axisLine.color = axisModelProp.LineColor;
            }
            if (axisModelProp.LineWidth) {
                axisName.axisLine.width = axisModelProp.LineWidth;
            }

            if (axisModelProp.HideEndLabels) {
                axisName.edgeLabelPlacement = 'hide';
            }

            if (axisModelProp.VisibleStatus == "True" || (axistype == 'primary' && axisModelProp.VisibleStatus == 'Auto')) {
                axisName.visible = true;
                axisName.axisLine.visible = true;
            } else {
                axisName.visible = false;
                axisName.axisLine.visible = false;
            }
            axisName.rangePadding = 'normal';
        },

        _applyGridLines: function (applySrc, applyTarget) {
            applyTarget.visible = applySrc.Enabled;
            if (applySrc.Enabled) {
                if (applySrc.Style && applySrc.Style.Default) {
                    if (applySrc.Style.Default.BorderBrush) {
                        applyTarget.color = applySrc.Style.Default.BorderBrush;
                    }

                    applyTarget.width = applySrc.Style.Default.BorderStyle == "None" ? 0 : applySrc.Style.Default.Thickness;

                    if (applySrc.Style.Default.BorderStyle == "Dashed") {
                        applyTarget.dashArray = "2,4";
                    }
                    else if (applySrc.Style.Default.BorderStyle == "Dotted") {
                        applyTarget.dashArray = "2,2";
                    }
                }
            }
        },

        _applyTickMarks: function (applySrc, applyTarget) {
            applyTarget.visible = applySrc.Enabled;

            if (applySrc.Style && applySrc.Style.Default) {
                if (applySrc.Style.Default.BorderBrush) {
                    applyTarget.color = applySrc.Style.Default.BorderBrush;
                }

                if (applySrc.Style.Default.Thickness) {
                    applyTarget.width = applySrc.Style.Default.Thickness;
                }
            }

            if (applySrc.Length) {
                applyTarget.size = applySrc.Length;
            }
        },
        //-------------------- Render the Chart ReportItem [end] -------------------------//

        _renderImageControl: function (imgModel, parnetObj, parentId, printMode, printPageId) {
            var ctrlId = this._controlKeyGenerator(this._id + '_' + imgModel.Name, parentId, printMode, printPageId);
            var imageDiv = ej.buildTag("div", "", {}, { id: ctrlId, title: imgModel.ToolTip });
            parnetObj.append(imageDiv);
            imageDiv.css("position", "absolute");
            imageDiv.css("top", imgModel.Top);
            imageDiv.css("left", imgModel.Left);
            imageDiv.css("width", imgModel.Height);
            imageDiv.css("height", imgModel.Width);

            imageDiv.css("padding-left", imgModel.PaddingLeft);
            imageDiv.css("padding-right", imgModel.PaddingRight);
            imageDiv.css("padding-top", imgModel.PaddingTop);
            imageDiv.css("padding-bottom", imgModel.PaddingBottom);

			if (imgModel.IsTablixChild) {
                imageDiv.css("position", "relative");
            }

            if (imgModel.ActionInfo) {
                imageDiv.data('actionObj', imgModel.ActionInfo);
                imageDiv.addClass('drillAction_' + this._id + ' e-reportviewer-drillhover');
            }

            if (imgModel.Border) {
                this._applyBorderStyle(imgModel.Border, imageDiv);
            }

            var resourceName = printMode ? 'PrintImage' : 'GetResource';
			var imgwidth = imgModel.IsTablixChild ? imgModel.Width : '100%';
            var imgheight = imgModel.IsTablixChild ? imgModel.Height : '100%';
            var imagetag = ej.buildTag("img", "", { 'width': imgwidth, 'height': imgheight }, { 'src': this.model.reportServiceUrl + '/' + resourceName + '/?key=' + imgModel.ImageUrl + '&resourcetype=sfimg&isPrint=' + printMode });
            imageDiv.append(imagetag);
        },

        _renderRectangleControl: function (rectModel, isTablixCell, parentObj, parentId, printMode, printPageId) {
            var ctrlId = this._controlKeyGenerator(this._id + '_' + rectModel.Name, parentId, printMode, printPageId);
            var rectDiv = ej.buildTag("div", "", {}, { 'id': ctrlId, title: rectModel.ToolTip });
            parentObj.append(rectDiv);
            rectDiv.css('position', isTablixCell ? 'relative' : 'absolute');
            rectDiv.css('top', rectModel.Top);
            rectDiv.css('left', rectModel.Left);
            rectDiv.css('width', rectModel.Width);
            rectDiv.css('height', rectModel.Height);
            rectDiv.css('background-color', rectModel.BackgroundColor);

            if (rectModel.Border && !isTablixCell) {
                this._applyBorderStyle(rectModel.Border, rectDiv);
            }

            if (rectModel.IsTablixChild && rectModel.ReportModels && rectModel.ReportModels.length > 0) {
                for (var i = 0; i < rectModel.ReportModels.length; i++) {
                    var innerItem = rectModel.ReportModels[i];
                    if (innerItem) {
                        if (innerItem.ItemType == "TextBoxModel") {
                            var textboxControl = this._renderTextBoxControl(innerItem, false, ctrlId + '_' + innerItem.Name, printMode, printPageId);
                            rectDiv.append(textboxControl);
                        }
                        else if (innerItem.ItemType == "LineModel") {
                            var lineControl = this._renderLineControl(innerItem, ctrlId + '_' + innerItem.Name, printMode, printPageId);
                            rectDiv.append(lineControl);
                        }
                        else if (innerItem.ItemType == "ImageModel") {
                            this._renderImageControl(innerItem, rectDiv, ctrlId + '_' + innerItem.Name, printMode, printPageId);
                        }
                        else if (innerItem.ItemType == "TablixModel") {
                            this._renderTablixControl(innerItem, rectDiv, ctrlId + '_' + innerItem.Name, printPageId);
                        }
                        else if (innerItem.ItemType == "GaugeModel") {
                            this._renderGaugeControl(innerItem, rectDiv, ctrlId + '_' + innerItem.Name, printMode, printPageId);
                        }
                        else if (innerItem.ItemType == "RectangleModel") {
                            this._renderRectangleControl(innerItem, false, rectDiv, ctrlId + + '_' + innerItem.Name, printMode, printPageId);
                        }
                        else if (innerItem.ItemType == "ChartModel") {
                            this._renderChartControl(innerItem, rectDiv, ctrlId + '_' + innerItem.Name, printMode, printPageId);
                        }
                        else if (innerItem.ItemType == "MapModel") {
                            this._renderMapControl(innerItem, rectDiv, ctrlId + '_' + innerItem.Name, printMode, printPageId);
                        }
                        else if (innerItem.ItemType == "SubReportModel") {
                            this._renderSubReportControl(innerItem, false, rectDiv, ctrlId + + '_' + innerItem.Name, printMode, printPageId);
                        }
                    }
                }
            }
        },

        _renderMapControl: function (mapModel, parentObj, parentId, printMode, printPageId) {
            var mapContainer = mapModel.MapProperties;
            var ctrlId = this._controlKeyGenerator(this._id + '_' + mapModel.Name, parentId, printMode, printPageId);
            var mapDiv = ej.buildTag("div", "", {}, { 'id': ctrlId });
            var titleContainerHeight = 0;
            parentObj.append(mapDiv);

            mapDiv.css("position", "absolute");
            mapDiv.css("top", mapModel.Top);
            mapDiv.css("left", mapModel.Left);
            mapDiv.css("height", mapModel.Height);
            mapDiv.css("width", mapModel.Width);
            if (mapContainer.Background && mapContainer.Background.BackgroundColor) {
                mapDiv.css("background-color", mapContainer.Background.BackgroundColor);
            }

            if (mapContainer.ActionInfo) {
                mapDiv.data('actionObj', mapContainer.ActionInfo);
                mapDiv.addClass('drillAction_' + this._id + ' e-reportviewer-drillhover');
            }

            if (mapContainer.Border) {
                this._applyBorderStyle(mapContainer.Border, mapDiv);
            }
            var title = mapContainer.MapTitles;
            if (title && title.length > 0) {

                for (var i = 0; i < title.length; i++) {
                    var titleDiv = ej.buildTag("div", "", {}, { 'id': ctrlId + '_' + title[i].Name });
                    titleDiv.css("position", "relative");
                    titleDiv.css("left", "1px");
                    titleDiv.css("height", title[i].Style.Font.FontSize - (3 * title[i].Border.Thickness));
                    titleDiv.css("width", mapModel.Width - (3 * title[i].Border.Thickness));
                    titleDiv.css("text-align", "center");
                    titleDiv.css("background-color", title[i].BackGround.BackgroundColor);
                    titleDiv.css("border-color", title[i].Border.BorderBrush);
                    titleDiv.css("border-width", title[i].Border.Thickness);
                    titleDiv.css("border-style", title[i].Border.BorderStyle);

                    var span = ej.buildTag("span", "", {}, { 'id': ctrlId + '_' + title[i].Name + '_' + i });
                    span.css("font-family", title[i].Style.Font.FontFamily);
                    span.css("font-size", (title[i].Style.Font.FontSize / 1.33));
                    span.css("font-style", title[i].Style.Font.FontStyle);
                    span.css("font-weight", title[i].Style.Font.FontWeight);
                    span.append(title[i].Text);
                    titleDiv.append(span);
                    mapDiv.append(titleDiv);
                }
            }

            if (mapContainer.MapViewport) {
                var viewPort = mapContainer.MapViewport;
                var shapDiv = ej.buildTag("div", "", {}, { 'id': ctrlId + '_' + mapModel.Name });
                mapDiv.append(shapDiv);
                shapDiv.css("left", "1px");
				shapDiv.css("top", "1px");

                if (title && title.length > 0) {
                    for (var titleIndex = 0; titleIndex < title.length; titleIndex++) {
                        titleContainerHeight = titleContainerHeight + title[titleIndex].Style.Font.FontSize;
                    }
                    shapDiv.css("height", ((mapModel.Height - titleContainerHeight) - (3 * viewPort.Border.Thickness)));
                }
                else {
                    shapDiv.css("height", (mapModel.Height - (3 * viewPort.Border.Thickness)));
                }

                shapDiv.css("width", mapModel.Width - (3 * viewPort.Border.Thickness));
                shapDiv.css("background-color", viewPort.BackGround.BackgroundColor);
                shapDiv.css("border-color", viewPort.Border.BorderBrush);
                shapDiv.css("border-width", viewPort.Border.Thickness);
                shapDiv.css("border-style", viewPort.Border.BorderStyle);

                shapDiv.ejMap({ layers: [{ layerType: "geometry" }] });
                var map = shapDiv.data("ejMap");
                map.model.zoomSettings.minValue = 1;
                map.model.zoomSettings.level = 1.5;
                map.model.zoomSettings.enableZoom = false;

                map.model.centerPosition = [viewPort.MapView.CenterX, viewPort.MapView.CenterY];

                if (mapContainer.MapPolygonLayers && mapContainer.MapPolygonLayers.length > 0) {
                    for (var polygonIndex = 0; polygonIndex < mapContainer.MapPolygonLayers.length; polygonIndex++) {
                        var polygon = mapContainer.MapPolygonLayers[polygonIndex];
                        var layer = $.extend(true, {}, map.model.layers[0]);
                        layer.layerType = "geometry";
                        layer.enableSelection = false;
                        layer.enableMouseHover = false;
                        layer.shapeData = polygon.MapFields;
                        if (polygon.MapBindingFieldPairs) {
                            layer.shapePropertyPath = polygon.MapBindingFieldPairs.FieldName;
                        }
                        layer.shapeDataPath = 'Value';
                        layer.dataSource = mapModel.MapModelData;

                        if (mapModel.MapModelData && mapModel.MapModelData.length > 0 && mapModel.MapModelData[0].ShapeActionInfo) {
                            map.ejreportid = this._id;
                            map.model.shapeSelected = this._drillThroughClick;
                            layer.enableSelection = true;
                            layer.enableMouseHover = true;
                            map.model.mouseover = function (event) {
                                if (event.originalEvent.data && event.originalEvent.data.ShapeActionInfo) {
                                    $(event.originalEvent.shape).css('cursor', 'pointer');
                                }
                            };
                        }

                        if (polygon.ShapeSettings) {
                            var shapesettings = $.extend(true, {}, layer.shapeSettings);
                            shapesettings.colorValuePath = polygon.ShapeSettings.ShapeColorValuePath;
                            shapesettings.valuePath = polygon.ShapeSettings.ShapeValuePath;
                            if (polygon.ShapeColorMappings && polygon.ShapeColorMappings.length > 1) {
                                shapesettings.colorMappings = { equalColorMapping: polygon.ShapeColorMappings };
                            }
                            else {
                                shapesettings.autoFill = true;
                                shapesettings.colorPalette = "palette1";
                            }
                            layer.shapeSettings = shapesettings;
                        }
                        if (polygon.MapCenterPointRules && polygon.MapCenterPointRules.length > 0 && polygon.MapCenterPointRules.MapSizeRule) {
                            var bubblesettings = $.extend(true, {}, layer.bubbleSettings);
                            bubblesettings.maxValue = polygon.BubbleSettings.MaxSize;
                            bubblesettings.minValue = polygon.BubbleSettings.MinSize;
                            bubblesettings.colorValuePath = polygon.BubbleSettings.ColorValuePath == null ? "Value" : polygon.BubbleSettings.ColorValuePath;
                            bubblesettings.valuePath = polygon.BubbleSettings.ValuePath;
                            if (polygon.BubbleColorMappings && polygon.BubbleColorMappings.length > 1) {
                                bubblesettings.colorMappings = { equalColorMapping: polygon.BubbleColorMappings };
                            }
                            layer.bubbleSettings = bubblesettings;
                        }

                        if (polygon.MapPolygonTemplate) {
                            var labelsettings = $.extend(true, {}, layer.labelSettings);
                            labelsettings.showLabels = true;
                            if (polygon.ShapeSettings) {
                                labelsettings.labelPath = polygon.ShapeSettings.LabelPath;
                            }
                            labelsettings.enableSmartLabel = false;
                            layer.labelSettings = labelsettings;
                        }

                        if (mapContainer.MapLegends && mapContainer.MapLegends.length > 0) {
                            var legendsettings = $.extend(true, {}, layer.legendSettings);
                            legendsettings.showLegend = true;
                            layer.legendSettings = legendsettings;
                        }
                        map.model.layers[polygonIndex] = layer;
                    }
                }
                map.refresh();
				shapDiv.css("position", "relative");
            }
        },

        //-------------------- Render the Gauge ReportItem [start] -------------------------//
        _renderGaugeControl: function (gaugeModel, parentObj, parentId, printMode, printPageId) {
            var gaugePanel = gaugeModel.GaugePanel;
            var ctrlId = this._controlKeyGenerator(this._id + '_' + gaugeModel.Name, parentId, printMode, printPageId);
            var gaugeDiv = ej.buildTag("div", "", {}, { 'id': ctrlId });
            parentObj.append(gaugeDiv);
            gaugeDiv.css("position", "absolute");
            gaugeDiv.css("top", gaugeModel.Top);
            gaugeDiv.css("left", gaugeModel.Left);
            var containerHeight = gaugeModel.Height;
            var containerWidth = gaugeModel.Width;
            var panelOrientation = 'Horizontal';
            var itemCount = 0;

            if (gaugePanel.RadialGauges && gaugePanel.RadialGauges.length > 0) {
                itemCount = itemCount + gaugePanel.RadialGauges.length;
            }

            if (gaugePanel.LinearGauges && gaugePanel.LinearGauges.length > 0) {
                itemCount = itemCount + gaugePanel.LinearGauges.length;
            }

            if (gaugePanel.Indicator && gaugePanel.Indicator.length > 0) {
                itemCount = itemCount + gaugePanel.Indicator.length;
            }

            if (itemCount != 0) {
                if (containerHeight > containerWidth) {
                    containerHeight = containerHeight / itemCount;
                    panelOrientation = 'Vertical';
                } else {
                    containerWidth = containerWidth / itemCount;
                }
            }

            if (gaugePanel.GaugeLabel) {
                this._renderGaugeLabels(gaugePanel.GaugeLabel, gaugeDiv, gaugeModel.Name, parentId, printMode, printPageId);
            }

            if (gaugePanel.RadialGauges && gaugePanel.RadialGauges.length > 0) {
                this._renderRadialGauge(gaugePanel.RadialGauges, gaugeDiv, gaugeModel.Name, panelOrientation, containerWidth, containerHeight, parentId, printMode, printPageId);
            }

            if (gaugePanel.LinearGauges && gaugePanel.LinearGauges.length > 0) {
                this._renderLinearGauge(gaugePanel.LinearGauges, gaugeDiv, gaugeModel.Name, panelOrientation, containerWidth, containerHeight, parentId, printMode, printPageId);
            }

            if (gaugePanel.Indicator && gaugePanel.Indicator.length > 0) {
                this._renderIndicator(gaugePanel.Indicator, gaugeDiv, gaugeModel.Name, containerWidth, containerHeight, parentId, printMode, printPageId);
            }

            gaugeDiv.css({ 'width': gaugeModel.Width, 'height': gaugeModel.Height, 'background-color': gaugePanel.BackgroundColor });

            if (gaugePanel.Border) {
                this._applyBorderStyle(gaugePanel.Border, gaugeDiv);
            }
        },

        _renderIndicator: function (indicators, gaugeDiv, gaugeName, containerWidth, containerHeight, parentId, printMode, printPageId) {
            var svgNs = "http://www.w3.org/2000/svg";
            for (var guageIndex = 0; guageIndex < indicators.length; guageIndex++) {
                var svgelement = document.createElementNS(svgNs, "svg");
                var groupElement = document.createElementNS(svgNs, "g");
                var scaleXy = (containerWidth < containerHeight) ? containerWidth : containerHeight;
                var indicator = indicators[guageIndex];
                var indicatorDiv = ej.buildTag("div", "", { 'width': containerWidth, 'height': containerHeight, 'top': '0px', 'left': '0px' }, { 'id': this._controlKeyGenerator(this._id + '_' + gaugeName + '_' + indicator.Name, parentId, printMode, printPageId), title: indicator.ToolTip });
                var indicatorinnerDiv = ej.buildTag("div", "", { 'width': '20px', 'height': '20px', 'position': 'relative', 'display': 'block' });
                //svgelement.setAttributeNS(null, 'width', 'auto');
                //svgelement.setAttributeNS(null, 'height', 'auto');
                svgelement.setAttributeNS(null, 'style', 'display: block;margin:auto');
                for (var state = 0; state < indicator.IndicatorState.length; state++) {
                    var style = indicator.IndicatorState[state].IndicatorStyle;
                    var startValue = indicator.IndicatorState[state].StartValue.Value;
                    var endValue = indicator.IndicatorState[state].EndValue.Value;
                    if (indicator.IndicatorData.Value >= startValue && indicator.IndicatorData.Value <= endValue) {
                        var fillColor = indicator.IndicatorState[state].FillColor;
                        var svgAttribute = 'display: block;';
                        var innerDivWidth = 17;
                        var innerDivHeight = 17;
                        switch (style) {
                            case "ArrowDown":
                                createElement(groupElement, "path", { fill: fillColor, stroke: "lightgray", d: "M7.60501098632813,16.3990020751953L14.06298828125,10.0289993286133 10.3890075683594,10.0289993286133 10.3890075683594,2.32599639892578 4.61099243164063,2.32599639892578 4.61099243164063,10.0289993286133 1.14801025390625,10.0289993286133 7.60501098632813,16.3990020751953z" });
                                innerDivWidth = 16;
                                innerDivHeight = 19;
                                svgAttribute = svgAttribute + 'margin-left:.5px;';
                                break;
                            case "ArrowDownIncline":
                                createElement(groupElement, "path", { fill: fillColor, stroke: "lightgray", d: "M13.5499877929688,13.8919982910156L13.6119995117188,4.822998046875 11.0130004882813,7.42100524902344 5.5670166015625,1.9739990234375 1.48101806640625,6.05900573730469 6.92901611328125,11.5059967041016 4.47900390625,13.9550018310547 13.5499877929688,13.8919982910156z" });
                                innerDivWidth = innerDivHeight = 16;
                                svgAttribute = svgAttribute + 'margin-left:.5px;';
                                break;
                            case "ArrowSide":
                                createElement(groupElement, "path", { fill: fillColor, stroke: "lightgray", d: "M15.1960144042969,8.43499755859375L8.8280029296875,1.97899627685547 8.8280029296875,5.65299987792969 1.125,5.65299987792969 1.125,11.4300003051758 8.8280029296875,11.4300003051758 8.8280029296875,14.8919982910156 15.1960144042969,8.43499755859375z" });
                                svgAttribute = svgAttribute + 'margin-left:.5px;';
                                break;
                            case "ArrowUp":
                                createElement(groupElement, "path", { fill: fillColor, stroke: "lightgray", d: "M7.27099609375,1.47299957275391L0.81500244140625,7.84200286865234 4.48800659179688,7.84200286865234 4.48800659179688,15.5439987182617 10.2659912109375,15.5439987182617 10.2659912109375,7.84200286865234 13.7279968261719,7.84200286865234 7.27099609375,1.47299957275391z" });
                                innerDivWidth = 16;
                                svgAttribute = svgAttribute + 'margin-left:.5px;';
                                break;
                            case "ArrowUpIncline":
                                createElement(groupElement, "path", { fill: fillColor, stroke: "lightgray", d: "M13.0830078125,2.04299926757813L4.0159912109375,1.98199462890625 6.61300659179688,4.58000183105469 1.16598510742188,10.0269927978516 5.25100708007813,14.1119995117188 10.697998046875,8.66499328613281 13.14599609375,11.1130065917969 13.0830078125,2.04299926757813z" });
                                innerDivWidth = 15;
                                innerDivHeight = 16;
                                svgAttribute = svgAttribute + 'margin-left:.5px;';
                                break;
                            case "BoxesAllFilled":
                                createElement(groupElement, "path", { fill: fillColor, stroke: "lightgray", d: "M2.031005859375,7.5L8.031005859375,7.5 8.031005859375,1.5 2.031005859375,1.5 2.031005859375,7.5z" });
                                createElement(groupElement, "path", { fill: fillColor, stroke: "lightgray", d: "M8.96798706054688,7.5L14.9679870605469,7.5 14.9679870605469,1.5 8.96798706054688,1.5 8.96798706054688,7.5z" });
                                createElement(groupElement, "path", { fill: fillColor, stroke: "lightgray", d: "M2.031005859375,14.43701171875L8.031005859375,14.43701171875 8.031005859375,8.43798828125 2.031005859375,8.43798828125 2.031005859375,14.43701171875z" });
                                createElement(groupElement, "path", { fill: fillColor, stroke: "lightgray", d: "M8.96798706054688,14.43701171875L14.9679870605469,14.43701171875 14.9679870605469,8.43798828125 8.96798706054688,8.43798828125 8.96798706054688,14.43701171875z" });
                                svgAttribute = svgAttribute + 'margin-top:2.5px;';
                                break;
                            case "BoxesNoneFilled":
                                createElement(groupElement, "path", { fill: "none", stroke: "lightgray", d: "M2.031005859375,7.5L8.031005859375,7.5 8.031005859375,1.5 2.031005859375,1.5 2.031005859375,7.5z" });
                                createElement(groupElement, "path", { fill: "none", stroke: "lightgray", d: "M8.96798706054688,7.5L14.9679870605469,7.5 14.9679870605469,1.5 8.96798706054688,1.5 8.96798706054688,7.5z" });
                                createElement(groupElement, "path", { fill: "none", stroke: "lightgray", d: "M2.031005859375,14.43701171875L8.031005859375,14.43701171875 8.031005859375,8.43798828125 2.031005859375,8.43798828125 2.031005859375,14.43701171875z" });
                                createElement(groupElement, "path", { fill: "none", stroke: "lightgray", d: "M8.96798706054688,14.43701171875L14.9679870605469,14.43701171875 14.9679870605469,8.43798828125 8.96798706054688,8.43798828125 8.96798706054688,14.43701171875z" });
                                innerDivHeight = 16;
                                break;
                            case "BoxesOneFilled":
                                createElement(groupElement, "path", { fill: "none", stroke: "lightgray", d: "M2.031005859375,7.5L8.031005859375,7.5 8.031005859375,1.5 2.031005859375,1.5 2.031005859375,7.5z" });
                                createElement(groupElement, "path", { fill: "none", stroke: "lightgray", d: "M8.96798706054688,7.5L14.9679870605469,7.5 14.9679870605469,1.5 8.96798706054688,1.5 8.96798706054688,7.5z" });
                                createElement(groupElement, "path", { fill: fillColor, stroke: "lightgray", d: "M2.031005859375,14.43701171875L8.031005859375,14.43701171875 8.031005859375,8.43798828125 2.031005859375,8.43798828125 2.031005859375,14.43701171875z" });
                                createElement(groupElement, "path", { fill: "none", stroke: "lightgray", d: "M8.96798706054688,14.43701171875L14.9679870605469,14.43701171875 14.9679870605469,8.43798828125 8.96798706054688,8.43798828125 8.96798706054688,14.43701171875z" });
                                innerDivHeight = 16;
                                break;
                            case "BoxesThreeFilled":
                                createElement(groupElement, "path", { fill: fillColor, stroke: "lightgray", d: "M2.031005859375,7.5L8.031005859375,7.5 8.031005859375,1.5 2.031005859375,1.5 2.031005859375,7.5z" });
                                createElement(groupElement, "path", { fill: "none", stroke: "lightgray", d: "M8.96798706054688,7.5L14.9679870605469,7.5 14.9679870605469,1.5 8.96798706054688,1.5 8.96798706054688,7.5z" });
                                createElement(groupElement, "path", { fill: fillColor, stroke: "lightgray", d: "M2.031005859375,14.43701171875L8.031005859375,14.43701171875 8.031005859375,8.43798828125 2.031005859375,8.43798828125 2.031005859375,14.43701171875z" });
                                createElement(groupElement, "path", { fill: fillColor, stroke: "lightgray", d: "M8.96798706054688,14.43701171875L14.9679870605469,14.43701171875 14.9679870605469,8.43798828125 8.96798706054688,8.43798828125 8.96798706054688,14.43701171875z" });
                                innerDivHeight = 16;
                                break;
                            case "BoxesTwoFilled":
                                createElement(groupElement, "path", { fill: "none", stroke: "lightgray", d: "M2.031005859375,7.5L8.031005859375,7.5 8.031005859375,1.5 2.031005859375,1.5 2.031005859375,7.5z" });
                                createElement(groupElement, "path", { fill: "none", stroke: "lightgray", d: "M8.96798706054688,7.5L14.9679870605469,7.5 14.9679870605469,1.5 8.96798706054688,1.5 8.96798706054688,7.5z" });
                                createElement(groupElement, "path", { fill: fillColor, stroke: "lightgray", d: "M2.031005859375,14.43701171875L8.031005859375,14.43701171875 8.031005859375,8.43798828125 2.031005859375,8.43798828125 2.031005859375,14.43701171875z" });
                                createElement(groupElement, "path", { fill: fillColor, stroke: "lightgray", d: "M8.96798706054688,14.43701171875L14.9679870605469,14.43701171875 14.9679870605469,8.43798828125 8.96798706054688,8.43798828125 8.96798706054688,14.43701171875z" });
                                innerDivHeight = 16;
                                break;
                            case "Circle":
                                createElement(groupElement, "path", { fill: fillColor, stroke: "lightgray", d: "M15.3310852050781,8.3330078125L14.7622108459473,11.150749206543 13.2108383178711,13.4517440795898 10.9098434448242,15.003116607666 8.09210205078125,15.5719909667969 5.27377843856812,15.003116607666 2.97248458862305,13.4517440795898 1.42100191116333,11.150749206543 0.85211181640625,8.3330078125 1.42100191116333,5.51524877548218 2.97248458862305,3.2142448425293 5.27377843856812,1.66286897659302 8.09210205078125,1.093994140625 10.9098434448242,1.66286897659302 13.2108383178711,3.2142448425293 14.7622108459473,5.51524877548218 15.3310852050781,8.3330078125" });
                                innerDivWidth = innerDivHeight = 17;
                                svgAttribute = svgAttribute + 'margin-left:0.5px;';
                                break;
                            case "Flag":
                                createElement(groupElement, "path", { fill: fillColor, stroke: "lightgray", d: "M1.53518676757813,1.97268676757813L1.71956861019135,1.87765264511108 2.22404956817627,1.65059661865234 3.90132141113281,1.14869689941406 5.98302745819092,1.16354370117188 6.99305105209351,1.58242082595825 7.88519287109375,2.39169311523438 9.55474090576172,4.09454298019409 11.1795692443848,5.14020156860352 12.5349636077881,5.63535928726196 13.3962097167969,5.68670654296875 13.5080919265747,5.78957939147949 13.3250951766968,6.16205215454102 12.1980743408203,7.28895568847656 10.2623682022095,8.2137336730957 9.06851482391357,8.33354949951172 7.76519775390625,8.08270263671875 5.64992141723633,7.59151554107666 4.50045776367188,7.73744964599609 4.02486801147461,8.11925792694092 3.93121337890625,8.335693359375 1.53518676757813,1.97268676757813z" });
                                innerDivWidth = 16;
                                innerDivHeight = 15;
                                break;
                            case "QuartersAllFilled":
                                createElement(groupElement, "path", { fill: fillColor, stroke: "lightgray", d: "M15.3310852050781,8.3330078125L14.7622108459473,11.150749206543 13.2108383178711,13.4517440795898 10.9098434448242,15.003116607666 8.09210205078125,15.5719909667969 5.27377843856812,15.003116607666 2.97248458862305,13.4517440795898 1.42100191116333,11.150749206543 0.85211181640625,8.3330078125 1.42100191116333,5.51524877548218 2.97248458862305,3.2142448425293 5.27377843856812,1.66286897659302 8.09210205078125,1.093994140625 10.9098434448242,1.66286897659302 13.2108383178711,3.2142448425293 14.7622108459473,5.51524877548218 15.3310852050781,8.3330078125" });
                                svgAttribute = svgAttribute + 'margin-left:0.5px;';
                                break;
                            case "QuartersNoneFilled":
                                createElement(groupElement, "path", { fill: "none", stroke: "lightgray", d: "M15.3310852050781,8.3330078125L14.7622108459473,11.150749206543 13.2108383178711,13.4517440795898 10.9098434448242,15.003116607666 8.09210205078125,15.5719909667969 5.27377843856812,15.003116607666 2.97248458862305,13.4517440795898 1.42100191116333,11.150749206543 0.85211181640625,8.3330078125 1.42100191116333,5.51524877548218 2.97248458862305,3.2142448425293 5.27377843856812,1.66286897659302 8.09210205078125,1.093994140625 10.9098434448242,1.66286897659302 13.2108383178711,3.2142448425293 14.7622108459473,5.51524877548218 15.3310852050781,8.3330078125" });
                                svgAttribute = svgAttribute + 'margin-left:0.5px;';
                                break;
                            case "QuartersOneFilled":
                                createElement(groupElement, "path", { fill: "none", stroke: "lightgray", d: "M1.2958984375,8.71868896484375L1.864914894104,11.5364475250244 3.41652679443359,13.8374519348145 5.71757793426514,15.3888282775879 8.534912109375,15.9577026367188 11.353235244751,15.3888282775879 13.6545295715332,13.8374519348145 15.2060127258301,11.5364475250244 15.77490234375,8.71868896484375 15.2060127258301,5.90094757080078 13.6545295715332,3.59995269775391 11.353235244751,2.04858016967773 8.534912109375,1.47970581054688 5.71757793426514,2.04858016967773 3.41652679443359,3.59995269775391 1.864914894104,5.90094757080078 1.2958984375,8.71868896484375" });
                                createElement(groupElement, "path", { fill: fillColor, d: "M8.57321166992188,1.86428833007813L11.3244533538818,2.38370847702026 13.4967041015625,3.80480575561523 14.9227085113525,5.96914529800415 15.4352111816406,8.71829223632813 15.5302124023438,8.7222900390625 8.57321166992188,8.7222900390625 8.57321166992188,1.86428833007813z" });
                                svgAttribute = svgAttribute + 'margin-top:0.5px;';
                                break;
                            case "QuartersThreeFilled":
                                createElement(groupElement, "path", { fill: fillColor, stroke: "lightgray", d: "M16.033203125,9.3330078125L15.4643287658691,12.150749206543 13.9129524230957,14.4517440795898 11.6119480133057,16.003116607666 8.794189453125,16.5719909667969 5.97586584091187,16.003116607666 3.6745719909668,14.4517440795898 2.12308931350708,12.150749206543 1.55419921875,9.3330078125 2.12308931350708,6.51524877548218 3.6745719909668,4.2142448425293 5.97586584091187,2.66286897659302 8.794189453125,2.093994140625 11.6119480133057,2.66286897659302 13.9129524230957,4.2142448425293 15.4643287658691,6.51524877548218 16.033203125,9.3330078125" });
                                createElement(groupElement, "path", { fill: "white", d: "M8.75729370117188,2.4794921875L6.00546979904175,2.99834585189819 3.83292007446289,4.41925430297852 2.40680551528931,6.58378267288208 1.894287109375,9.33349609375 1.79928588867188,9.33648681640625 8.75729370117188,9.33648681640625 8.75729370117188,2.4794921875z" });
                                svgAttribute = svgAttribute + 'margin-top:0.5px;';
                                break;
                            case "QuartersTwoFilled":
                                createElement(groupElement, "path", { fill: fillColor, stroke: "lightgray", d: "M15.1328125,8.3330078125L14.563796043396,11.150749206543 13.0121841430664,13.4517440795898 10.7111330032349,15.003116607666 7.893798828125,15.5719909667969 5.07547521591187,15.003116607666 2.7741813659668,13.4517440795898 1.22269868850708,11.150749206543 0.65380859375,8.3330078125 1.22269868850708,5.51524877548218 2.7741813659668,3.2142448425293 5.07547521591187,1.66286897659302 7.893798828125,1.093994140625 10.7111330032349,1.66286897659302 13.0121841430664,3.2142448425293 14.563796043396,5.51524877548218 15.1328125,8.3330078125" });
                                createElement(groupElement, "path", { fill: "white", d: "M7.81930541992188,1.45999145507813L5.0907883644104,1.99271726608276 2.95392227172852,3.42375564575195 1.56049299240112,5.59241724014282 1.06228637695313,8.3380126953125 1.57205629348755,11.0836029052734 2.99092483520508,13.252254486084 5.1532301902771,14.6832828521729 7.893310546875,15.2160034179688 7.81930541992188,1.45999145507813z" });
                                svgAttribute = svgAttribute + 'margin-top:0.5px;margin-left:0.5px;';
                                break;
                            case "SignalMeterFourFilled":
                                createElement(groupElement, "path", { fill: fillColor, stroke: "lightgray", d: "M3.44400024414063,17.3070068359375L1.53201293945313,17.3070068359375 1.53201293945313,11.5700073242188 3.44400024414063,11.5700073242188 3.44400024414063,17.3070068359375z" });
                                createElement(groupElement, "path", { fill: fillColor, stroke: "lightgray", d: "M7.14999389648438,17.3070068359375L5.23699951171875,17.3070068359375 5.23699951171875,9.65701293945313 7.14999389648438,9.65701293945313 7.14999389648438,17.3070068359375z" });
                                createElement(groupElement, "path", { fill: fillColor, stroke: "lightgray", d: "M10.8550109863281,17.3070068359375L8.9429931640625,17.3070068359375 8.9429931640625,5.83200073242188 10.8550109863281,5.83200073242188 10.8550109863281,17.3070068359375z" });
                                createElement(groupElement, "path", { fill: fillColor, stroke: "lightgray", d: "M14.56201171875,17.3070068359375L12.6489868164063,17.3070068359375 12.6489868164063,2.00698852539063 14.56201171875,2.00698852539063 14.56201171875,17.3070068359375z" });
                                innerDivHeight = 19;
                                svgAttribute = svgAttribute + 'margin-top:-10px;;margin-left:0.5px;';
                                break;
                            case "SignalMeterNoneFilled":
                                createElement(groupElement, "path", { fill: "none", stroke: "lightgray", d: "M3.44400024414063,17.3070068359375L1.53201293945313,17.3070068359375 1.53201293945313,11.5700073242188 3.44400024414063,11.5700073242188 3.44400024414063,17.3070068359375z" });
                                createElement(groupElement, "path", { fill: "none", stroke: "lightgray", d: "M7.14999389648438,17.3070068359375L5.23699951171875,17.3070068359375 5.23699951171875,9.65701293945313 7.14999389648438,9.65701293945313 7.14999389648438,17.3070068359375z" });
                                createElement(groupElement, "path", { fill: "none", stroke: "lightgray", d: "M10.8550109863281,17.3070068359375L8.9429931640625,17.3070068359375 8.9429931640625,5.83200073242188 10.8550109863281,5.83200073242188 10.8550109863281,17.3070068359375z" });
                                createElement(groupElement, "path", { fill: "none", stroke: "lightgray", d: "M14.56201171875,17.3070068359375L12.6489868164063,17.3070068359375 12.6489868164063,2.00698852539063 14.56201171875,2.00698852539063 14.56201171875,17.3070068359375z" });
                                innerDivHeight = 19;
                                svgAttribute = svgAttribute + 'margin-top:-10px;;margin-left:0.5px;';
                                break;
                            case "SignalMeterThreeFilled":
                                createElement(groupElement, "path", { fill: fillColor, stroke: "lightgray", d: "M3.44400024414063,17.3070068359375L1.53201293945313,17.3070068359375 1.53201293945313,11.5700073242188 3.44400024414063,11.5700073242188 3.44400024414063,17.3070068359375z" });
                                createElement(groupElement, "path", { fill: fillColor, stroke: "lightgray", d: "M7.14999389648438,17.3070068359375L5.23699951171875,17.3070068359375 5.23699951171875,9.65701293945313 7.14999389648438,9.65701293945313 7.14999389648438,17.3070068359375z" });
                                createElement(groupElement, "path", { fill: fillColor, stroke: "lightgray", d: "M10.8550109863281,17.3070068359375L8.9429931640625,17.3070068359375 8.9429931640625,5.83200073242188 10.8550109863281,5.83200073242188 10.8550109863281,17.3070068359375z" });
                                createElement(groupElement, "path", { fill: "none", stroke: "lightgray", d: "M14.56201171875,17.3070068359375L12.6489868164063,17.3070068359375 12.6489868164063,2.00698852539063 14.56201171875,2.00698852539063 14.56201171875,17.3070068359375z" });
                                innerDivHeight = 19;
                                svgAttribute = svgAttribute + 'margin-top:-10px;;margin-left:0.5px;';
                                break;
                            case "SignalMeterOneFilled":
                                createElement(groupElement, "path", { fill: fillColor, stroke: "lightgray", d: "M3.44400024414063,17.3070068359375L1.53201293945313,17.3070068359375 1.53201293945313,11.5700073242188 3.44400024414063,11.5700073242188 3.44400024414063,17.3070068359375z" });
                                createElement(groupElement, "path", { fill: "none", stroke: "lightgray", d: "M7.14999389648438,17.3070068359375L5.23699951171875,17.3070068359375 5.23699951171875,9.65701293945313 7.14999389648438,9.65701293945313 7.14999389648438,17.3070068359375z" });
                                createElement(groupElement, "path", { fill: "none", stroke: "lightgray", d: "M10.8550109863281,17.3070068359375L8.9429931640625,17.3070068359375 8.9429931640625,5.83200073242188 10.8550109863281,5.83200073242188 10.8550109863281,17.3070068359375z" });
                                createElement(groupElement, "path", { fill: "none", stroke: "lightgray", d: "M14.56201171875,17.3070068359375L12.6489868164063,17.3070068359375 12.6489868164063,2.00698852539063 14.56201171875,2.00698852539063 14.56201171875,17.3070068359375z" });
                                innerDivHeight = 19;
                                svgAttribute = svgAttribute + 'margin-top:-10px;;margin-left:0.5px;';
                                break;
                            case "SignalMeterTwoFilled":
                                createElement(groupElement, "path", { fill: fillColor, stroke: "lightgray", d: "M3.44400024414063,17.3070068359375L1.53201293945313,17.3070068359375 1.53201293945313,11.5700073242188 3.44400024414063,11.5700073242188 3.44400024414063,17.3070068359375z" });
                                createElement(groupElement, "path", { fill: fillColor, stroke: "lightgray", d: "M7.14999389648438,17.3070068359375L5.23699951171875,17.3070068359375 5.23699951171875,9.65701293945313 7.14999389648438,9.65701293945313 7.14999389648438,17.3070068359375z" });
                                createElement(groupElement, "path", { fill: "none", stroke: "lightgray", d: "M10.8550109863281,17.3070068359375L8.9429931640625,17.3070068359375 8.9429931640625,5.83200073242188 10.8550109863281,5.83200073242188 10.8550109863281,17.3070068359375z" });
                                createElement(groupElement, "path", { fill: "none", stroke: "lightgray", d: "M14.56201171875,17.3070068359375L12.6489868164063,17.3070068359375 12.6489868164063,2.00698852539063 14.56201171875,2.00698852539063 14.56201171875,17.3070068359375z" });
                                innerDivHeight = 19;
                                svgAttribute = svgAttribute + 'margin-top:-10px;;margin-left:0.5px;';
                                break;
                            case "StarQuartersAllFilled":
                                createElement(groupElement, "path", { fill: fillColor, stroke: "lightgray", d: "M6.41500854492188,6.37100219726563L1.56698608398438,7.07400512695313 5.07501220703125,10.4939880371094 4.24700927734375,15.3210144042969 8.58200073242188,13.0409851074219 12.9169921875,15.3210144042969 12.0899963378906,10.4939880371094 15.5979919433594,7.07400512695313 10.75,6.37100219726563 8.58200073242188,1.97698974609375 6.41500854492188,6.37100219726563z" });
                                break;
                            case "StarQuartersNoneFilled":
                                createElement(groupElement, "path", { fill: "none", stroke: "lightgray", d: "M6.41500854492188,6.37100219726563L1.56698608398438,7.07400512695313 5.07501220703125,10.4939880371094 4.24700927734375,15.3210144042969 8.58200073242188,13.0409851074219 12.9169921875,15.3210144042969 12.0899963378906,10.4939880371094 15.5979919433594,7.07400512695313 10.75,6.37100219726563 8.58200073242188,1.97698974609375 6.41500854492188,6.37100219726563z" });
                                break;
                            case "StarQuartersOneFilled":
                                createElement(groupElement, "path", { fill: "none", stroke: "lightgray", d: "M8.64109802246094,1.97750854492188L10.80810546875,6.37149047851563 15.6560974121094,7.07449340820313 12.1490936279297,10.4945068359375 12.9761047363281,15.3204956054688 8.64109802246094,13.0404968261719 4.30610656738281,15.3204956054688 5.13409423828125,10.4945068359375 1.6260986328125,7.07449340820313 6.47309875488281,6.37149047851563 8.64109802246094,1.97750854492188z" });
                                createElement(groupElement, "path", { fill: fillColor, stroke: "none", d: "M6.22300720214844,6.37100219726563L1.50100708007813,7.07400512695313 5.07099914550781,10.4939880371094 4.06300354003906,15.531005859375 7.08399963378906,13.864013671875 7.072998046875,4.968994140625 6.22300720214844,6.37100219726563z" });
                                break;
                            case "StarQuartersThreeFilled":
                                createElement(groupElement, "path", { fill: "none", stroke: "lightgray", d: "M8.60209655761719,1.97750854492188L10.7691040039063,6.37149047851563 15.6170959472656,7.07449340820313 12.110107421875,10.4945068359375 12.9371032714844,15.3204956054688 8.60209655761719,13.0404968261719 4.26710510253906,15.3204956054688 5.0950927734375,10.4945068359375 1.58709716796875,7.07449340820313 6.43409729003906,6.37149047851563 8.60209655761719,1.97750854492188z" });
                                createElement(groupElement, "path", { fill: fillColor, stroke: "none", d: "M6.40199279785156,6.37100219726563L1.58700561523438,7.07400512695313 5.11000061035156,10.4939880371094 4.28999328613281,15.3210144042969 8.49000549316406,13.0989990234375 10.6970062255859,14.27099609375 10.6970062255859,6.2239990234375 8.60200500488281,1.97698974609375 6.40199279785156,6.37100219726563z" });
                                break;
                            case "StarQuartersTwoFilled":
                                createElement(groupElement, "path", { fill: "none", stroke: "lightgray", d: "M8.17819213867188,1.97750854492188L10.34521484375,6.37149047851563 15.1932067871094,7.07449340820313 11.6861877441406,10.4945068359375 12.5132141113281,15.3204956054688 8.17819213867188,13.0404968261719 3.84320068359375,15.3204956054688 4.67120361328125,10.4945068359375 1.1632080078125,7.07449340820313 6.01019287109375,6.37149047851563 8.17819213867188,1.97750854492188z" });
                                createElement(groupElement, "path", { fill: fillColor, stroke: "none", d: "M6.010009765625,6.37100219726563L1.16299438476563,7.07400512695313 4.67098999023438,10.4939880371094 3.84298706054688,15.3210144042969 7.83499145507813,13.2219848632813 7.90399169921875,2.53201293945313 6.010009765625,6.37100219726563z" });
                                break;
                            case "ThreeSignsCircle":
                                createElement(groupElement, "path", { fill: fillColor, stroke: "lightgray", d: "M15.3310852050781,8.3330078125L14.7622108459473,11.150749206543 13.2108383178711,13.4517440795898 10.9098434448242,15.003116607666 8.09210205078125,15.5719909667969 5.27377843856812,15.003116607666 2.97248458862305,13.4517440795898 1.42100191116333,11.150749206543 0.85211181640625,8.3330078125 1.42100191116333,5.51524877548218 2.97248458862305,3.2142448425293 5.27377843856812,1.66286897659302 8.09210205078125,1.093994140625 10.9098434448242,1.66286897659302 13.2108383178711,3.2142448425293 14.7622108459473,5.51524877548218 15.3310852050781,8.3330078125" });
                                svgAttribute = svgAttribute + 'margin-top:1px;margin-left:0.5px;';
                                break;
                            case "ThreeSignsDiamond":
                                createElement(groupElement, "path", { fill: fillColor, stroke: "lightgray", d: "M1.08401489257813,8.01901245117188L8.10400390625,15.1589965820313 15.2929992675781,8.01901245117188 8.10400390625,0.878997802734375 1.08401489257813,8.01901245117188z" });
                                innerDivWidth = 16;
                                svgAttribute = svgAttribute + 'margin-top:3px;margin-left:1px;';
                                break;
                            case "ThreeSignsTriangle":
                                createElement(groupElement, "path", { fill: fillColor, stroke: "lightgray", d: "M0.83099365234375,14.2340087890625L15.7950134277344,14.2340087890625 8.31298828125,1.27499389648438 0.83099365234375,14.2340087890625z" });
                                svgAttribute = svgAttribute + 'margin-top:1px;';
                                break;
                            case "ThreeSymbolCheck":
                                createElement(groupElement, "path", { fill: fillColor, stroke: "lightgray", d: "M15.3310852050781,8.3330078125L14.7622108459473,11.150749206543 13.2108383178711,13.4517440795898 10.9098434448242,15.003116607666 8.09210205078125,15.5719909667969 5.27377843856812,15.003116607666 2.97248458862305,13.4517440795898 1.42100191116333,11.150749206543 0.85211181640625,8.3330078125 1.42100191116333,5.51524877548218 2.97248458862305,3.2142448425293 5.27377843856812,1.66286897659302 8.09210205078125,1.093994140625 10.9098434448242,1.66286897659302 13.2108383178711,3.2142448425293 14.7622108459473,5.51524877548218 15.3310852050781,8.3330078125" });
                                createElement(groupElement, "path", { fill: "white", stroke: "lightgray", d: "M12.15869140625,4.38958740234375L11.3836975097656,3.86260986328125 10.7337036132813,3.98660278320313 6.94171142578125,9.57958984375 5.19769287109375,7.83660888671875 4.53570556640625,7.83660888671875 3.87469482421875,8.49859619140625 3.87469482421875,9.16061401367188 6.5546875,11.8416137695313 7.1536865234375,12.1086120605469 7.71771240234375,11.7735900878906 12.28369140625,5.03860473632813 12.15869140625,4.38958740234375" });
                                svgAttribute = svgAttribute + 'margin-top:1px;margin-left:0.5px;';
                                break;
                            case "ThreeSymbolCross":
                                createElement(groupElement, "path", { fill: fillColor, stroke: "lightgray", d: "M1.10299682617188,7.94601440429688L1.66799592971802,10.7435569763184 3.20874404907227,13.0283813476563 5.49386739730835,14.569019317627 8.2919921875,15.1340026855469 8.2919921875,15.1340026855469 11.0901165008545,14.569019317627 13.3752403259277,13.0283813476563 14.9159889221191,10.7435569763184 15.4809875488281,7.94601440429688 15.4809875488281,7.94601440429688 14.9159889221191,5.14773225784302 13.3752403259277,2.86226272583008 11.0901165008545,1.32116842269897 8.2919921875,0.756011962890625 8.2919921875,0.756011962890625 5.49386739730835,1.32116842269897 3.20874404907227,2.86226272583008 1.66799592971802,5.14773225784302 1.10299682617188,7.94601440429688" });
                                createElement(groupElement, "path", { fill: "white", stroke: "lightgray", d: "M10.0479125976563,7.94430541992188L11.9789123535156,6.0133056640625 11.9789123535156,5.31130981445313 10.9259033203125,4.25729370117188 10.222900390625,4.25729370117188 8.2908935546875,6.18930053710938 6.35989379882813,4.25729370117188 5.65689086914063,4.25729370117188 4.60488891601563,5.31130981445313 4.60488891601563,6.0133056640625 6.534912109375,7.94430541992188 4.60488891601563,9.87728881835938 4.60488891601563,10.5783081054688 5.65689086914063,11.6322937011719 6.35989379882813,11.6322937011719 8.2908935546875,9.70028686523438 10.2239074707031,11.6322937011719 10.9259033203125,11.6322937011719 11.9789123535156,10.5783081054688 11.9789123535156,9.87728881835938 10.0479125976563,7.94430541992188z" });
                                svgAttribute = svgAttribute + 'margin-top:3px;margin-left:1px;';
                                break;
                            case "ThreeSymbolExclamation":
                                createElement(groupElement, "path", { fill: fillColor, stroke: "lightgray", d: "M0.983001708984375,7.94601440429688L1.54785919189453,10.7435569763184 3.08837127685547,13.0283813476563 5.37344741821289,14.569019317627 8.1719970703125,15.1340026855469 8.1719970703125,15.1340026855469 10.9699649810791,14.569019317627 13.2547416687012,13.0283813476563 14.7951431274414,10.7435569763184 15.3599853515625,7.94601440429688 15.3599853515625,7.94601440429688 14.7951431274414,5.14773225784302 13.2547416687012,2.86226272583008 10.9699649810791,1.32116842269897 8.1719970703125,0.756011962890625 8.1719970703125,0.756011962890625 5.37344741821289,1.32116842269897 3.08837127685547,2.86226272583008 1.54785919189453,5.14773225784302 0.983001708984375,7.94601440429688" });
                                createElement(groupElement, "path", { fill: "white", stroke: "lightgray", d: "M7.59280395507813,9.60159301757813L8.71978759765625,9.60159301757813 9.00180053710938,9.60159301757813 9.12579345703125,2.8916015625 7.143798828125,2.8916015625 7.31280517578125,9.60159301757813 7.59280395507813,9.60159301757813z" });
                                createElement(groupElement, "path", { fill: "white", stroke: "lightgray", d: "M8.96728515625,10.69140625L8.17428588867188,10.3753967285156 7.37130737304688,10.6924133300781 7.03228759765625,11.4764099121094 7.37530517578125,12.244384765625 8.17428588867188,12.54541015625 8.96328735351563,12.244384765625 9.310302734375,11.4764099121094 8.96728515625,10.69140625" });
                                svgAttribute = svgAttribute + 'margin-top:3px;margin-left:1px;';
                                break;
                            case "ThreeSymbolUnCircledCheck":
                                createElement(groupElement, "path", { fill: fillColor, stroke: "lightgray", d: "M13.8515930175781,2.2255859375L12.6495971679688,1.402587890625 12.1062202453613,1.29134368896484 11.6416015625,1.59561157226563 5.755615234375,10.32958984375 3.04959106445313,7.60760498046875 2.53897094726563,7.39609527587891 2.02459716796875,7.60760498046875 0.997589111328125,8.64260864257813 0.785346984863281,9.15935516357422 0.997589111328125,9.6776123046875 5.1575927734375,13.8635864257813 6.0845947265625,14.2785949707031 6.96060180664063,13.755615234375 14.0445861816406,3.2406005859375 14.1565895080566,2.69297027587891 13.8515930175781,2.2255859375" });
                                innerDivWidth = innerDivHeight = 16;
                                svgAttribute = svgAttribute + 'margin-top:3px;margin-left:.5px;';
                                break;
                            case "ThreeSymbolUnCircledCross":
                                createElement(groupElement, "path", { fill: fillColor, stroke: "lightgray", d: "M9.36907958984375,6.86331176757813L12.423095703125,3.71530151367188 12.6540832519531,3.14280700683594 12.423095703125,2.5703125 10.7581176757813,0.851287841796875 10.2026062011719,0.613548278808594 9.6470947265625,0.851287841796875 6.59613037109375,4.00131225585938 3.5421142578125,0.851287841796875 2.98759841918945,0.613548278808594 2.43310546875,0.851287841796875 0.768096923828125,2.5703125 0.537109375,3.14205169677734 0.768096923828125,3.71530151367188 3.82012939453125,6.86331176757813 0.768096923828125,10.0133056640625 0.537109375,10.5845413208008 0.768096923828125,11.1572875976563 2.43310546875,12.8763122558594 2.98723220825195,13.1140518188477 3.5421142578125,12.8763122558594 6.59613037109375,9.72628784179688 9.6480712890625,12.8763122558594 10.2027282714844,13.1140518188477 10.7581176757813,12.8763122558594 12.423095703125,11.1572875976563 12.6540832519531,10.5845413208008 12.423095703125,10.0133056640625 9.36907958984375,6.86331176757813z" });
                                innerDivWidth = innerDivHeight = 16;
                                svgAttribute = svgAttribute + 'margin-top:5px;margin-left:1.5px;';
                                break;
                            case "ThreeSymbolUnCircledExclamation":
                                createElement(groupElement, "path", { fill: "none", stroke: "none", d: "M0.983001708984375,7.94601440429688L1.54785919189453,10.7435569763184 3.08837127685547,13.0283813476563 5.37344741821289,14.569019317627 8.1719970703125,15.1340026855469 8.1719970703125,15.1340026855469 10.9699649810791,14.569019317627 13.2547416687012,13.0283813476563 14.7951431274414,10.7435569763184 15.3599853515625,7.94601440429688 15.3599853515625,7.94601440429688 14.7951431274414,5.14773225784302 13.2547416687012,2.86226272583008 10.9699649810791,1.32116842269897 8.1719970703125,0.756011962890625 8.1719970703125,0.756011962890625 5.37344741821289,1.32116842269897 3.08837127685547,2.86226272583008 1.54785919189453,5.14773225784302 0.983001708984375,7.94601440429688" });
                                createElement(groupElement, "path", { fill: fillColor, stroke: "none", d: "M7.59280395507813,9.60159301757813L8.71978759765625,9.60159301757813 9.00180053710938,9.60159301757813 9.12579345703125,2.8916015625 7.143798828125,2.8916015625 7.31280517578125,9.60159301757813 7.59280395507813,9.60159301757813z" });
                                createElement(groupElement, "path", { fill: fillColor, stroke: "none", d: "M8.96728515625,10.69140625L8.17428588867188,10.3753967285156 7.37130737304688,10.6924133300781 7.03228759765625,11.4764099121094 7.37530517578125,12.244384765625 8.17428588867188,12.54541015625 8.96328735351563,12.244384765625 9.310302734375,11.4764099121094 8.96728515625,10.69140625" });
                                svgAttribute = svgAttribute + 'margin-top:1px;margin-left:1px;';                               
                                break;
                            case "TrafficLight":
                                createElement(groupElement, "path", { fill: "black", stroke: "lightgray", d: "M3.83200073242188,1.46800231933594L2.76542282104492,1.68336224555969 1.89412689208984,2.27062797546387 1.3065185546875,3.14157843589783 1.09100341796875,4.20799255371094 1.09100341796875,4.20799255371094 1.09100341796875,13.3419952392578 1.3065185546875,14.4091548919678 1.89412689208984,15.2807502746582 2.76542282104492,15.8684692382813 3.83200073242188,16.0839996337891 3.83200073242188,16.0839996337891 12.9670104980469,16.0839996337891 14.0340003967285,15.8684692382813 14.9052505493164,15.2807502746582 15.4926300048828,14.4091548919678 15.7080078125,13.3419952392578 15.7080078125,13.3419952392578 15.7080078125,4.20799255371094 15.4926300048828,3.14157843589783 14.9052505493164,2.27062797546387 14.0340003967285,1.68336224555969 12.9670104980469,1.46800231933594 12.9670104980469,1.46800231933594 3.83200073242188,1.46800231933594z" });
                                createElement(groupElement, "path", { fill: fillColor, d: "M3.78421020507813,8.77540588378906L4.14698934555054,6.97894716262817 5.13620376586914,5.51202392578125 6.60316896438599,4.52304029464722 8.39920043945313,4.160400390625 10.1958141326904,4.52304029464722 11.6630783081055,5.51202392578125 12.6524028778076,6.97894716262817 13.0151977539063,8.77540588378906 12.6524028778076,10.5718555450439 11.6630783081055,12.0387744903564 10.1958141326904,13.027756690979 8.39920043945313,13.3903961181641 6.60316896438599,13.027756690979 5.13620376586914,12.0387744903564 4.14698934555054,10.5718555450439 3.78421020507813,8.77540588378906" });
                                innerDivWidth = innerDivHeight = 18;
                                svgAttribute = svgAttribute + 'margin-top:3px;margin-left:.5px;';
                                break;
                            case "TrafficLightUnrimmed":
                                createElement(groupElement, "path", { fill: fillColor, stroke: "lightgray", d: "M15.3310852050781,8.3330078125L14.7622108459473,11.150749206543 13.2108383178711,13.4517440795898 10.9098434448242,15.003116607666 8.09210205078125,15.5719909667969 5.27377843856812,15.003116607666 2.97248458862305,13.4517440795898 1.42100191116333,11.150749206543 0.85211181640625,8.3330078125 1.42100191116333,5.51524877548218 2.97248458862305,3.2142448425293 5.27377843856812,1.66286897659302 8.09210205078125,1.093994140625 10.9098434448242,1.66286897659302 13.2108383178711,3.2142448425293 14.7622108459473,5.51524877548218 15.3310852050781,8.3330078125" });
                                svgAttribute = svgAttribute + 'margin-top:3px;margin-left:.5px;';
                                break;
                            case "TriangleDash":
                                createElement(groupElement, "path", { fill: fillColor, stroke: "lightgray", d: "M13.9800109863281,7.56300354003906L0.501007080078125,7.56300354003906 0.501007080078125,4.67400360107422 13.9800109863281,4.67400360107422 13.9800109863281,7.56300354003906z" });
                                innerDivWidth = 15;
                                innerDivHeight = 9;
                                break;
                            case "TriangleDown":
                                createElement(groupElement, "path", { fill: fillColor, stroke: "lightgray", d: "M14.06201171875,0.788002014160156L7.60501098632813,7.15699768066406 1.14801025390625,0.788002014160156 14.06201171875,0.788002014160156z" });
                                innerDivWidth = 15;
                                innerDivHeight = 9;
                                svgAttribute = svgAttribute + 'margin-top:3px;margin-left:.5px;';
                                break;
                            case "TriangleUp":
                                createElement(groupElement, "path", { fill: fillColor, stroke: "lightgray", d: "M0.89599609375,7.30400085449219L7.35198974609375,0.93499755859375 13.8089904785156,7.30400085449219 0.89599609375,7.30400085449219z" });
                                innerDivWidth = 15;
                                innerDivHeight = 9;
                                svgAttribute = svgAttribute + 'margin-top:3px;margin-left:.5px;';
                                break;
                        }
                        indicatorinnerDiv.css({ 'width': innerDivWidth + 'px', 'height': innerDivHeight + 'px' });
                        svgelement.setAttributeNS(null, 'style', svgAttribute);
                        svgelement.appendChild(groupElement);
                    }
                }
                indicatorinnerDiv.append(svgelement);
                indicatorDiv.append(indicatorinnerDiv);
                gaugeDiv.append(indicatorDiv);

                var innerScaleWidth = indicatorinnerDiv.width();
                var innerScaleHeight = indicatorinnerDiv.height();
                var indidenomintaorLen = innerScaleWidth < innerScaleHeight ? innerScaleWidth : innerScaleHeight;
                var browserInfo = ej.browserInfo();
                scaleXy = scaleXy / indidenomintaorLen;

                if (browserInfo.name == 'msie') {
                    indicatorinnerDiv.css('transform', 'scale(' + scaleXy + ',' + scaleXy + ')');
                } else if (browserInfo.name == 'opera') {
                    indicatorinnerDiv.css('-o-transform', 'scale(' + scaleXy + ',' + scaleXy + ')');
                } else if (browserInfo.name == 'mozilla') {
                    indicatorinnerDiv.css('-ms-transform', 'scale(' + scaleXy + ',' + scaleXy + ')');
                } else if (browserInfo.name == 'webkit' || browserInfo.name == 'chrome') {
                    indicatorinnerDiv.css('-webkit-transform', 'scale(' + scaleXy + ',' + scaleXy + ')');
                }

                if (innerScaleWidth < innerScaleHeight) {
                    indicatorinnerDiv.css({ 'left': '50%', 'margin-left': -(innerScaleWidth / 2), 'top': '50%', 'margin-top': -(innerScaleWidth / 2) });
                } else {
                    indicatorinnerDiv.css({ 'left': '50%', 'margin-left': -(innerScaleHeight / 2), 'top': '50%', 'margin-top': -(innerScaleHeight / 2) });
                }
            }

            function createElement(root, name, attrs) {
                var element = document.createElementNS(svgNs, name);
                for (var attr in attrs) {
                    if (attrs.hasOwnProperty(attr)) {
                        element.setAttribute(attr, attrs[attr]);
                    }
                }
                root.appendChild(element);
            }
        },

        _renderGaugeLabels: function (gaugeLabels, gaugeDiv, gaugeName, parentId, printMode, printPageId) {
            var guageIndex;
            for (guageIndex = 0; guageIndex < gaugeLabels.length; guageIndex++) {
                var labelModel = gaugeLabels[guageIndex];
                var labelGaugeDiv = ej.buildTag("div", "", {}, { 'id': this._controlKeyGenerator(this._id + '_' + gaugeName + '_' + labelModel.Name, parentId, printMode, printPageId), title: labelModel.ToolTip });
                labelGaugeDiv.css({ "position": "absolute", "top": labelModel.Top, "left": labelModel.Left, "width": labelModel.length + 10, "height": labelModel.Height + 10, "background-color": labelModel.BackgroundColor, "border-color": labelModel.Style.BorderColor, "border-style": labelModel.Style.BorderStyle, "border-width": labelModel.Style.BorderWidth });
                var span = document.createElement("span");
                span.innerHTML = labelModel.Text;
                span.style.fontFamily = labelModel.Font.FontFamily;
                span.style.fontSize = labelModel.Font.FontSize;
                span.style.fontStyle = labelModel.Font.FontStyle;
                span.style.fontWeight = labelModel.Font.FontWeight;
                span.style.textAlign = labelModel.TextAlign;
                span.style.color = labelModel.TextColor;
                span.style.textDecoration = labelModel.TextDecoration;
                labelGaugeDiv.append(span);
                gaugeDiv.append(labelGaugeDiv);
            }
        },

        _renderRadialGauge: function (radialGauges, gaugeDiv, gaugeName, panelOrientation, containerWidth, containerHeight, parentId, printMode, printPageId) {
            for (var guageIndex = 0; guageIndex < radialGauges.length; guageIndex++) {
                var radialGauge = radialGauges[guageIndex];
                var radialGaugeDiv = ej.buildTag("div", "", {}, { 'id': this._controlKeyGenerator(this._id + '_' + gaugeName + '_' + radialGauge.Name, parentId, printMode, printPageId) });
                gaugeDiv.append(radialGaugeDiv);
                radialGaugeDiv.ejCircularGauge();

                var circular = radialGaugeDiv.data("ejCircularGauge");
                var radialscaleobj = circular.model.scales[0];
                circular.model.scales.pop(circular.model.scales[0]);

                var radialframe = radialGauge.GaugeFrame;
                var radialheight = (containerWidth < containerHeight) ? (containerWidth / 2) - radialframe.FrameWidth : (containerHeight / 2) - radialframe.FrameWidth;
                circular.model.radius = radialheight;
                circular.model.width = containerWidth;
                circular.model.height = containerHeight;
                circular.model.backgroundColor = radialGauge.BackgroundColor;
                circular.model.enableAnimation = false;

                if (radialGauge.ScaleProperties && radialGauge.ScaleProperties.length > 0) {
                    var radialscaleLen = radialGauge.ScaleProperties.length;
                    for (var radialscaleIndex = 0; radialscaleIndex < radialscaleLen; radialscaleIndex++) {
                        var radialScale = radialGauge.ScaleProperties[radialscaleIndex];
                        var radialscale = $.extend(true, {}, radialscaleobj);
                        radialscale.radius = (radialheight * ((radialscaleLen - radialscaleIndex) / radialscaleLen)) - radialframe.FrameWidth;
                        radialscale.startAngle = radialScale.StartAngle + 90;
                        radialscale.sweepAngle = radialScale.SweepAngle;
                        radialscale.showRanges = true;
                        radialscale.showScaleBar = true;
                        radialscale.maximum = radialScale.Maximum;
                        radialscale.minimum = radialScale.Minimum;
                        radialscale.border.color = radialScale.BorderColor;
                        radialscale.border.width = radialScale.BorderWidth;
                        radialscale.showPointers = true;
                        radialscale.showTicks = true;
                        radialscale.showLabels = true;
                        radialscale.showIndicators = true;
                        radialscale.backgroundColor = radialScale.BackgroundColor;

                        if (radialScale.RadialPointer && radialScale.RadialPointer.length > 0) {
                            var radialpointerobj = radialscale.pointers[0];
                            radialscale.pointers.pop(radialscale.pointers[0]);
                            var radialpointerLen = radialScale.RadialPointer.length;
                            for (var radialpointerIndex = 0; radialpointerIndex < radialpointerLen; radialpointerIndex++) {
                                var radialpointer = $.extend(true, {}, radialpointerobj);
                                var scalePointer = radialScale.RadialPointer[radialpointerIndex];
                                radialpointer.value = scalePointer.Value;
                                radialpointer.length = radialheight * ((radialscaleLen - radialscaleIndex) / radialscaleLen) - 30;
                                radialpointer.backgroundColor = scalePointer.BackgroundColor;
                                radialpointer.type = scalePointer.Type == "Needle" ? "needle" : "marker";
                                radialpointer.showBackNeedle = false;
                                radialpointer.width = scalePointer.Width;
                                radialpointer.border.width = scalePointer.BorderWidth;
                                radialpointer.border.color = scalePointer.BorderColor;

                                switch (scalePointer.NeedleStyle) {
                                    case "Rectangular":
                                        radialpointer.needleType = "rectangle";
                                        break;
                                    case "Arrow":
                                        radialpointer.needleType = "arrow";
                                        break;
                                    default:
                                        radialpointer.needleType = "triangle";
                                        break;
                                }

                                if (scalePointer.MarkerStyle == "wedge" || scalePointer.MarkerStyle == "triangle" || scalePointer.MarkerStyle == "circle" || scalePointer.MarkerStyle == "diamond" || scalePointer.MarkerStyle == "pentagon" || scalePointer.MarkerStyle == "trapezoid") {
                                    radialpointer.markerType = scalePointer.MarkerStyle.toLowerCase();
                                } else {
                                    radialpointer.markerType = "rectangle";
                                }

                                radialpointer.distanceFromScale = scalePointer.distanceFromScale;
                                radialscale.pointerCap.backgroundColor = scalePointer.CapBackGroundColor;
                                radialscale.pointerCap.radius = scalePointer.CapRadius / 3;
                                radialscale.pointers.push(radialpointer);
                            }
                        } else {
                            radialscale.pointers = [];
                        }

                        if (radialScale.Range && radialScale.Range.length > 0) {
                            var radialrangeobj = radialscale.ranges[0];
                            radialscale.ranges.pop(radialscale.ranges[0]);
                            var raddialrangeLen = radialScale.Range.length;
                            for (var raddialrangeIndex = 0; raddialrangeIndex < raddialrangeLen; raddialrangeIndex++) {
                                var radialrange = $.extend(true, {}, radialrangeobj);
                                var radialscaleRange = radialScale.Range[raddialrangeIndex];
                                radialrange.distanceFromScale = radialscaleRange.DistanceFromScale;
                                radialrange.backgroundColor = radialscaleRange.BackgroundColor;
                                if (radialscaleRange.EndValue == 0 && radialscaleRange.StartValue == 0) {
                                    radialrange.startValue = 70;
                                    radialrange.endValue = 100;
                                } else {
                                    radialrange.startValue = radialscaleRange.StartValue;
                                    radialrange.endValue = radialscaleRange.EndValue;
                                }
                                radialrange.startWidth = radialscaleRange.StartWidth;
                                radialrange.endWidth = radialscaleRange.EndWidth;
                                radialrange.border.color = radialscaleRange.BorderColor;
                                radialrange.border.width = radialscaleRange.BorderWidth;
                                radialscale.ranges.push(radialrange);
                            }
                        } else {
                            radialscale.ranges = [];
                        }

                        var radialtick = radialScale.TickMark;
                        var radialmajortickobj = radialscale.ticks[0];
                        var radialminortickobj = radialscale.ticks[1];
                        radialscale.ticks.pop(radialscale.ticks[0]);
                        radialscale.ticks.pop(radialscale.ticks[1]);

                        if (!radialtick.MajorTickHide) {
                            radialmajortickobj.type = "major";
                            radialmajortickobj.width = radialtick.MajorTickWidth + 1;
                            radialmajortickobj.distanceFromScale = radialtick.MajorTickDistanceFromScale;
                            radialmajortickobj.height = radialtick.MajorTickLength - 2;
                            radialmajortickobj.color = radialtick.MajorTickColor == null ? "black" : radialtick.MajorTickColor;
                            radialmajortickobj.placement = "near";
                            radialscale.ticks.push(radialmajortickobj);
                        }

                        if (!radialtick.MinorTickHide) {
                            radialminortickobj.type = "minor";
                            radialminortickobj.width = radialtick.MinorTickWidth;
                            radialminortickobj.distanceFromScale = radialtick.MinorTickDistanceFromScale;
                            radialminortickobj.height = radialtick.MinorTickLength - 1;
                            radialminortickobj.color = radialtick.MinorTickColor == null ? "black" : radialtick.MinorTickColor;
                            radialminortickobj.placement = "near";
                            radialscale.ticks.push(radialminortickobj);
                        }

                        var radiallabelValue = radialScale.Label;
                        var radialscalelabel = radialscale.labels[0];
                        radialscale.labels.pop(radialscale.labels[0]);
                        radialscalelabel.font.fontFamily = radiallabelValue.FontFamily;
                        radialscalelabel.font.size = radiallabelValue.FontSize;
                        radialscalelabel.font.fontStyle = radiallabelValue.FontStyle;
                        radialscalelabel.distanceFromScale = radiallabelValue.DistanceFromScale;
                        radialscalelabel.color = radiallabelValue.Color;
                        radialscale.labels.push(radialscalelabel);
                        radialscale.indicators = null;
                        circular.model.scales.push(radialscale);
                    }
                }
                circular.refresh();
                circular.contextEl.beginPath();
                circular.contextEl.arc((containerWidth / 2), (containerHeight / 2), radialheight, 0, 2 * Math.PI, false);
                circular.contextEl.fillStyle = "transparent";
                circular.contextEl.fill();
                circular.contextEl.lineWidth = radialframe.FrameWidth;
                circular.contextEl.strokeStyle = radialframe.FrameColor;
                circular.contextEl.stroke();
                circular.contextEl.closePath();

                if (printMode) {
                    radialGaugeDiv.append('<img src=' + circular.contextEl.canvas.toDataURL("image/png") + ' width=100% height=100%/>');
                    $(circular.contextEl.canvas).remove();
                }
                radialGaugeDiv.css({ 'width': containerWidth, 'height': containerHeight, 'float': panelOrientation == 'Horizontal' ? 'left' : 'top' });
            }
        },

        _renderLinearGauge: function (linearGauges, gaugeDiv, gaugeName, panelOrientation, containerWidth, containerHeight, parentId, printMode, printPageId) {
            for (var guageIndex = 0; guageIndex < linearGauges.length; guageIndex++) {
                var linearGauge = linearGauges[guageIndex];

                if (guageIndex % 2 != 0) {
                    var emptyDiv = ej.buildTag("div", "", { 'width': panelOrientation == 'Horizontal' ? '3px' : containerWidth - (frame.FrameWidth * 2), 'float': panelOrientation == 'Horizontal' ? 'left' : 'top', 'height': panelOrientation == 'Horizontal' ? containerHeight - (frame.FrameWidth * 2) : '3px' }, {});
                    gaugeDiv.append(emptyDiv);
                }

                var linearGaugeDiv = ej.buildTag("div", "", {}, { 'id': this._controlKeyGenerator(this._id + '_' + gaugeName + '_' + linearGauge.Name, parentId, printMode, printPageId) });
                gaugeDiv.append(linearGaugeDiv);
                linearGaugeDiv.ejLinearGauge();
                var linear = linearGaugeDiv.data("ejLinearGauge");

                var scaleobj = linear.model.scales[0];
                linear.model.scales.pop(linear.model.scales[0]);
                var frame = linearGauge.GaugeFrame;
                linear.model.width = containerWidth - (frame.FrameWidth * 2);
                linear.model.height = containerHeight - (frame.FrameWidth * 2);

                linear.model.enableAnimation = false;
                linear.model.backgroundColor = linearGauge.BackgroundColor;
                linear.model.orientation = linearGauge.Orientation;

                if (linearGauge.Orientation == "Horizontal") {
                    linear.model.orientation = linearGauge.Orientation;
                } else if (linearGauge.Orientation == "Vertical") {
                    linear.model.orientation = linearGauge.Orientation;
                } else {
                    if (containerWidth > containerHeight) {
                        linear.model.orientation = "Horizontal";
                    } else {
                        linear.model.orientation = "Vertical";
                    }
                }

                if (linearGauge.ScaleProperties && linearGauge.ScaleProperties.length > 0) {
                    var scaleLen = linearGauge.ScaleProperties.length;

                    for (var scaleIndex = 0; scaleIndex < scaleLen; scaleIndex++) {
                        var linearScale = linearGauge.ScaleProperties[scaleIndex];
                        var scale = $.extend(true, {}, scaleobj);
                        scale.maximum = linearScale.Maximum;
                        scale.minimum = linearScale.Minimum;
                        scale.showRanges = true;
                        scale.length = linear.model.orientation == "Horizontal" ? (containerWidth / 100) * 85 : (containerHeight / 100) * 85; //scalebarlength
                        scale.border.color = linearScale.BorderColor;
                        scale.border.width = linearScale.BorderWidth;
                        scale.showLabels = true;
                        scale.showRanges = true;
                        scale.showIndicators = true;
                        scale.showCustomLabels = true;
                        scale.showTicks = true;
                        scale.backgroundColor = linearScale.BackgroundColor;
                        scale.showBarPointers = false;
                        scale.showMarkerPointers = true;
                        scale.width = linear.model.orientation == "Vertical" ? (linear.model.width / 4) : (linear.model.height / 4);						

                        if (linearScale.LinearPointer && linearScale.LinearPointer.length > 0) {
                            var pointerobj = $.extend(true, {}, scale.markerPointers[0]);
                            scale.markerPointers.pop(scale.markerPointers[0]);
                            var pointerLen = linearScale.LinearPointer.length;

                            for (var pointerIndex = 0; pointerIndex < pointerLen; pointerIndex++) {
                                var pointer = $.extend(true, {}, pointerobj);
                                var scaleMarkerPointer = linearScale.LinearPointer[pointerIndex];
                                pointer.value = scaleMarkerPointer.MarkerValue;
                                pointer.length = scaleMarkerPointer.MarkerLength;
                                pointer.backgroundColor = scaleMarkerPointer.MarkerBackgroundColor;
                                pointer.width = scaleMarkerPointer.MarkerWidth;
                                pointer.border.width = scaleMarkerPointer.MarkerBorderWidth;
                                pointer.border.color = scaleMarkerPointer.MarkerBorderColor;
                                pointer.type = scaleMarkerPointer.MarkerStyle.toLowerCase();
                                pointer.distanceFromScale = scaleMarkerPointer.MarkerDistanceFromScale;
                                scale.markerPointers.push(pointer);
                            }
                        } else {
                            scale.markerPointers = [];
                        }

                        if (linearScale.Range && linearScale.Range.length > 0) {
                            var rangeobj = $.extend(true, {}, scale.ranges[0]);
                            scale.ranges.pop(scale.ranges[0]);
                            var rangeLen = linearScale.Range.length;

                            for (var rangeIndex = 0; rangeIndex < rangeLen; rangeIndex++) {
                                var range = $.extend(true, {}, rangeobj);
                                var scaleRange = linearScale.Range[rangeIndex];

                                range.distanceFromScale = scaleRange.DistanceFromScale;
                                range.backgroundColor = scaleRange.BackgroundColor;
                                range.startValue = scaleRange.StartValue;
                                range.startWidth = scaleRange.StartWidth;
                                range.endValue = scaleRange.EndValue;
                                range.endWidth = scaleRange.EndWidth;
                                range.border.color = scaleRange.BorderColor;
                                range.border.width = scaleRange.BorderWidth;
                                scale.ranges.push(range);
                            }
                        } else {
                            scale.ranges = [];
                        }

                        var tick = linearScale.TickMark;
                        var majortickobj = scale.ticks[0];
                        var minortickobj = scale.ticks[1];
                        scale.ticks.pop(scale.ticks[0]);
                        scale.ticks.pop(scale.ticks[1]);
                        majortickobj.type = "majorInterval";
                        majortickobj.width = tick.MajorTickWidth;
                        majortickobj.height = tick.MajorTickLength / 2;
                        majortickobj.color = tick.MajorTickColor == null ? "black" : tick.MajorTickColor;
                        scale.ticks.push(majortickobj);
                        minortickobj.type = "minorInterval";
                        minortickobj.width = tick.MinorTickWidth;
                        minortickobj.height = tick.MinorTickLength * 2;
                        minortickobj.color = tick.MinorTickColor == null ? "black" : tick.MinorTickColor;
                        scale.ticks.push(minortickobj);

                        var labelValue = linearScale.Label;
                        var scalelabel = scale.labels[0];
                        scale.labels.pop(scale.labels[0]);
                        scalelabel.font.fontFamily = labelValue.LinearLabelFontFamily;
                        scalelabel.font.size = labelValue.LinearLabelFontSize;
                        scalelabel.font.fontStyle = labelValue.LinearLabelFontStyle;
                        scalelabel.distanceFromScale = labelValue.LinearLabelDistanceFromScale;
                        scalelabel.textColor = labelValue.TextColor;
                        scale.labels.push(scalelabel);

                        scale.indicators = null;

                        linear.model.scales.push(scale);
                    }
                    linear.refresh();

                    if (printMode) {
                        linearGaugeDiv.append('<img src=' + linear.contextEl.canvas.toDataURL("image/png") + ' width=100% height=100%/>');
                        $(linear.contextEl.canvas).remove();
                    }
                    linearGaugeDiv.css({ 'width': panelOrientation == 'Horizontal' ? (containerWidth - (frame.FrameWidth * 2) - 3) : containerWidth - (frame.FrameWidth * 2), 'height': panelOrientation == 'Horizontal' ? containerHeight - (frame.FrameWidth * 2) : ((containerHeight - (frame.FrameWidth * 2)) - 3), 'float': panelOrientation == 'Horizontal' ? 'left' : 'top', 'border-color': frame.FrameColor, 'border-width': frame.FrameWidth, 'border-style': 'solid', 'background-color': linearGauge.BackgroundColor });
                }
            }
        },
        //-------------------- Render the Gauge ReportItem [start] -------------------------//

        _renderLineControl: function (lineModel, parentId, printMode, printPageId) {
            var svgNs = "http://www.w3.org/2000/svg";
            var ctrlId = this._controlKeyGenerator(this._id + '_' + lineModel.Name, parentId, printMode, printPageId);
            var linDiv = ej.buildTag("div", "", {}, { 'id': ctrlId });
            linDiv.css("position", "absolute");
            linDiv.css("top", lineModel.Top);
            linDiv.css("left", lineModel.Left);
            linDiv.css("width", lineModel.Width == 0 ? "11.00064" : (lineModel.Width));
            linDiv.css("height", lineModel.Height == 0 ? "16.00128" : (lineModel.Height));

            if (this._svg) {
                var svgelement = document.createElementNS(svgNs, "svg");
                svgelement.setAttributeNS(null, 'width', lineModel.Width == 0 ? "11.00064" : (lineModel.Width));
                svgelement.setAttributeNS(null, 'height', lineModel.Height == 0 ? "16.00128" : (lineModel.Height));

                var svgtagelement = document.createElementNS(svgNs, "line");
                svgtagelement.setAttributeNS(null, "x1", lineModel.X1 == 0 ? lineModel.LineWidth : (lineModel.X1 - lineModel.LineWidth));
                svgtagelement.setAttributeNS(null, "y1", lineModel.Y1 == 0 ? lineModel.LineWidth : (lineModel.Y1 - lineModel.LineWidth));
                svgtagelement.setAttributeNS(null, "x2", lineModel.X2 == 0 ? lineModel.LineWidth : (lineModel.X2 - lineModel.LineWidth));
                svgtagelement.setAttributeNS(null, "y2", lineModel.Y2 == 0 ? lineModel.LineWidth : (lineModel.Y2 - lineModel.LineWidth));
                svgtagelement.setAttributeNS(null, "stroke", lineModel.LineColor);
                svgtagelement.setAttributeNS(null, "stroke-width", lineModel.LineWidth);
                svgtagelement.setAttributeNS(null, "shape-rendering", "crispEdges");
                if (lineModel.LineStyle == "Dashed") {
                    svgtagelement.setAttributeNS(null, "stroke-dasharray", "" + (lineModel.LineWidth * 2) + "," + (lineModel.LineWidth * 2) + "");
                } else if (lineModel.LineStyle == "Dotted") {
                    svgtagelement.setAttributeNS(null, "stroke-dasharray", lineModel.LineWidth);
                }
                svgelement.appendChild(svgtagelement);
                linDiv.append(svgelement);
            } else {
                document.namespaces.add('v', 'urn:schemas-microsoft-com:vml');
                document.createStyleSheet().cssText = 'v\\:fill, v\\:path, v\\:polyline, v\\:line, v\\:rect,v\\:shape,v\\:oval, v\\:stroke' + '{ behavior:url(#default#VML); display: inline-block; } ';

                var vmlObject = document.createElement("<v:line id='line' from='" + lineModel.X1 + ',' + lineModel.Y1 + "' to='" + lineModel.X2 + ',' + lineModel.Y2 + "' strokecolor='" + lineModel.LineColor + "' strokeweight='" + lineModel.LineWidth + "pt'>" + "<v:stroke dashstyle='dot'></v:stroke>" + "</v:line>");
                var str = document.createElement("v:stroke");
                if (lineModel.LineStyle == "Dotted") {
                    str.setAttribute("dashstyle", "dot");
                } else if (lineModel.LineStyle == "Dashed") {
                    str.setAttribute("dashstyle", "dash");
                } else {
                    str.setAttribute("dashstyle", "solid");
                }
                str.outerHTML = str.outerHTML;
                vmlObject.appendChild(str);

                vmlObject.style.position = "absolute";
                vmlObject.style.width = lineModel.Width;
                vmlObject.style.height = lineModel.Height;
                vmlObject.style.visibility = "visible";

                linDiv.append(vmlObject);
            }
            return linDiv;
        },

        _renderSubReportControl: function (subreportModel, isTablixCell, parentObj, parentId, printMode, printPageId) {
            var ctrlId = this._controlKeyGenerator(this._id + '_' + subreportModel.Name, parentId, printMode, printPageId);
            var subReportDiv = ej.buildTag("div", "", {}, { 'id': ctrlId, title: subreportModel.ToolTip });

            parentObj.append(subReportDiv);
            subReportDiv.css('position', isTablixCell ? 'relative' : 'absolute');
            subReportDiv.css('top', subreportModel.Top);
            subReportDiv.css('left', subreportModel.Left);
            subReportDiv.css('width', subreportModel.Width);
            subReportDiv.css('height', subreportModel.Height);
            subReportDiv.css('background-color', subreportModel.BackgroundColor);

            if (subreportModel.Border && !isTablixCell) {
                this._applyBorderStyle(subreportModel.Border, subReportDiv);
            }

            if (subreportModel.ReportItemModels && subreportModel.ReportItemModels.length > 0) {
                for (var i = 0; i < subreportModel.ReportItemModels.length; i++) {
                    var innerItem = subreportModel.ReportItemModels[i];
                    if (innerItem) {
                        if (innerItem.ItemType == "TextBoxModel") {
                            var textboxControl = this._renderTextBoxControl(innerItem, false, ctrlId + '_' + innerItem.Name, printMode, printPageId);
                            subReportDiv.append(textboxControl);
                        } else if (innerItem.ItemType == "LineModel") {
                            var lineControl = this._renderLineControl(innerItem, ctrlId + '_' + innerItem.Name, printMode, printPageId);
                            subReportDiv.append(lineControl);
                        } else if (innerItem.ItemType == "ImageModel") {
                            this._renderImageControl(innerItem, subReportDiv, ctrlId + '_' + innerItem.Name, printMode, printPageId);
                        } else if (innerItem.ItemType == "TablixModel") {
                            this._renderTablixControl(innerItem, subReportDiv, ctrlId + '_' + innerItem.Name, printPageId);
                        } else if (innerItem.ItemType == "GaugeModel") {
                            this._renderGaugeControl(innerItem, subReportDiv, ctrlId + '_' + innerItem.Name, printMode, printPageId);
                        } else if (innerItem.ItemType == "RectangleModel") {
                            this._renderRectangleControl(innerItem, false, subReportDiv, ctrlId + +'_' + innerItem.Name, printMode, printPageId);
                        } else if (innerItem.ItemType == "ChartModel") {
                            this._renderChartControl(innerItem, subReportDiv, ctrlId + '_' + innerItem.Name, printMode, printPageId);
                        } else if (innerItem.ItemType == "MapModel") {
                            this._renderMapControl(innerItem, subReportDiv, ctrlId + '_' + innerItem.Name, printMode, printPageId);
                        } else if (innerItem.ItemType == "SubReportModel") {
                            this._renderSubReportControl(innerItem, false, subReportDiv, ctrlId + +'_' + innerItem.Name, printMode, printPageId);
                        }
                    }
                }
            }
        },
        //-------------------- Render the Page ReportItems[End] -------------------------//

        //-------------------- Ajax Web API Call back methods[start] -------------------------//
        doAjaxPost: function (type, url, jsondata, onSuccess) {
            var proxy = this;
            var inVokemethod = onSuccess;
            $.ajax({
                type: type,
                url: url,
                crossDomain: true,
                contentType: 'application/json; charset=utf-8',
                dataType: 'json',
                data: jsondata,
                beforeSend: function (req) {
                    if (inVokemethod == "_getPageModel" || inVokemethod == "_getPreviewModel") {
                        if (!proxy._isToolbarClick) {
                            proxy._showloadingIndicator(true);
                        } else {
                            proxy._showNavigationIndicator(true);
                        }
                    }
                    req.setRequestHeader('Authorization', proxy._authenticationToken);
                },
                success: function (data) {
                    if (data && typeof (data.Data) != "undefined") {
                        data = data.Data;
                    }
                    if (typeof (data) == "string") {
                        if (data.indexOf("Sf_Exception") != -1) {
                            proxy._renderExcpetion(inVokemethod + ":" + data);
                            return;
                        }
                    }
                    proxy[inVokemethod](data);
                },
                error: function (msg, textStatus, errorThrown) {
                    if (inVokemethod != "_clearCurrentServerCache") {
                        if (msg.readyState == 0) {
                            return;
                        }
                        alert('Exception' + msg.responseText);
                    }
                }
            });
        },

        _reportLoaded: function (reportresult) {
            if (typeof (reportresult) == "object" && reportresult.isReportLoad) {
                if (reportresult) {
                    this._reportParameters = reportresult.parameters;
                    this._reportDataSources = reportresult.dataSources;
                    this._reporDataSets = reportresult.dataSets;
                    this._onReportLoaded();
                    this.doAjaxPost("POST", this._actionUrl, JSON.stringify({ 'reportAction': this._reportAction.getDataSourceCredential, 'dataSources': this.model.dataSources, 'parameters': this.model.parameters }), "_getDataSourceCredential");
                }
                else {
                    this._renderExcpetion("Could not load the report");
                }
            } else {
                this._renderExcpetion("Report Loaded:" + reportresult);
            }
        },

        _getDataSourceCredential: function (jsondata) {
            if (ej.isNullOrUndefined(jsondata) || jsondata == "Sf_legacy") {
                this.doAjaxPost("POST", this._actionUrl, JSON.stringify({ 'reportAction': this._reportAction.updateDataSource }), "_updateDataSource");
            } else {
                this._renderCredentialBlock(jsondata);
                this._showViewerBlock(true);
            }
            this._showloadingIndicator(false);
            this._setContainerSize();
            this._showloadingIndicator(true);
        },

        _validateDSCredential: function (jsondata) {
            if (jsondata == "Sf_legacy") {
                this._showloadingIndicator(false);
                this._showViewerBlock(false);
                this._setContainerSize();
                this._showloadingIndicator(true);
                this.doAjaxPost("POST", this._actionUrl, JSON.stringify({ 'reportAction': this._reportAction.updateDSCredential, 'datasoures': this._dataSources }), "_updateDSCredential");
                this._dataSources = null;
            }
            else {
                alert(jsondata);
            }
        },

        _updateDSCredential: function (jsondata) {
            this.doAjaxPost("POST", this._actionUrl, JSON.stringify({ 'reportAction': this._reportAction.updateDataSource }), "_updateDataSource");
        },

        _getParameters: function (jsondata) {
            if (jsondata != "Sf_legacy") {
                this._renderParameterBlock(jsondata);
                this._showViewerBlock(true);
                this._showloadingIndicator(false);
                this._setContainerSize();
                var reportParams = this._getParameterJson(jsondata);
                this.doAjaxPost("POST", this._actionUrl, JSON.stringify({ 'reportAction': this._reportAction.setParameters, 'parameters': reportParams }), "_setParameters");
            }
            else {
                this._showParameterBlock(false);
                this.doAjaxPost("POST", this._actionUrl, JSON.stringify({ 'reportAction': this._reportAction.getPageModel, 'refresh': this._refresh, 'pageindex': this._currentPage, 'pageInit': this._isToolbarClick, 'isPrint': this._printMode }), !this._printMode ? "_getPageModel" : "_getPreviewModel");
            }
        },

        _setParameters: function (jsondata) {
            var isClick = $('#' + this._id + '_viewBlockContainer .e-reportviewer-viewerblockcontent table:first').attr('isviewclick');
            if (typeof (jsondata) == "boolean") {
                if (jsondata == false) {
                    this._showloadingIndicator(true);
                    this._setContainerSize();
                    if (this._isDevice) {
                        this._showViewerBlock(false);
                        this._showViewerPage(true);
                        this._selectparamToolItem(false);
                    }
                    this._currentPage = 1;
                    this.doAjaxPost("POST", this._actionUrl, JSON.stringify({ 'reportAction': this._reportAction.getPageModel, 'refresh': this._refresh, 'pageindex': this._currentPage, 'pageInit': this._isToolbarClick, 'isPrint': this._printMode }), !this._printMode ? "_getPageModel" : "_getPreviewModel");
                } else {
                    if (isClick == 'true') {
                        this._showloadingIndicator(false);
                        alert('Please select the parameter values.');
                    }
                }
            }
        },

        _updateParameters: function (jsondata) {
            if (jsondata != "Sf_legacy") {
                this._updateParamElements(jsondata);
            }
        },

        _updateDataSource: function (jsondata) {
            this.doAjaxPost("POST", this._actionUrl, JSON.stringify({ 'reportAction': this._reportAction.getParameters }), "_getParameters");
        },

        _getPageModel: function (jsondata) {
            var jsData = jsondata;
            this._showNavigationIndicator(false);
            this._showloadingIndicator(false);
            if (jsData) {
                if (jsData === 'IsDrillDown') {
                    return;
                }
                if (jsData.isDocumentMap) {
                    this._isDocumentMap = this._isPageDocMap = jsData.isDocumentMap;
                }
                if (jsData.errmsg && jsData.errmsg.length > 0) {
                    var isError = this._onReportError({ errmsg: jsData.errmsg });
                    if (!isError) {
                        if (this._isDevice) {
                            alert(jsData.errmsg);
                        } else {
                            this._renderExceptionBlock(jsData.errmsg);
                            this._showViewerBlock(true);
                        }
                    }
                } else {
                    this._resetExceptionBlock();
                }
                if (jsData.pgmodel && jsData.pgmodel.PageData) {
                    this._pageModel = jsData.pgmodel;
                    if (!this._isToolbarClick) {
                        this._enableToolbarItems();
                    }
                    if (this._refresh) {
                        this._onRenderingBegin();
                    }
                    this._gotoPage(this._currentPage);
                    this._pageLayoutPage = this._currentPage;
                    if (this._refresh) {
                        this._onRenderingComplete();
                    }
                    this._refresh = false;
                }
            }
        },

        _getPreviewModel: function (jsonData) {
            this._showNavigationIndicator(false);
            this._showloadingIndicator(false);
            if (jsonData) {
                if (jsonData.isDocumentMap) {
                    this._isDocumentMap = jsonData.isDocumentMap;
                    this._isPageDocMap = this._isDocumentMap;
                    this._pageDocMapFlag = false;
                }
                if (jsonData.pgmodel && jsonData.pgmodel.PageData) {
                    this._pageModel = jsonData.pgmodel;
                    if (!this._isToolbarClick) {
                        this._selectPreviewToolItem(true);
                        this._enableToolbarItems();
                    }
                    this._gotoPage(this._currentPage);
                    this._refresh = false;
                }
            }
        },

        _getPrintModel: function (jsonData) {
            if (jsonData && jsonData.PageData && jsonData.PageData.length > 0) {
                this._printReport(jsonData);
            }
        },

        _documentMapModel: function (jsondata) {
            if (jsondata && jsondata.indexOf("<ul id=treeViewID>") != -1) {
                this._showDocumentMap(this.model.toolbarSettings.items & ej.ReportViewer.ToolbarItems.DocumentMap);
                this._containerSplit(jsondata);
            }
        },

        _drillThroughModel: function (jsondata) {
            var pageXy = { x: $('#' + this._id + '_viewerContainer').scrollLeft(), y: $('#' + this._id + '_viewerContainer').scrollTop() };
            var reportProperties = this._cloneReportViewerProperties();
            this._parents.push({ pagePos: pageXy, parentPro: reportProperties });
            this._setInitialization();
            this._authenticationToken = reportProperties._childReportAuthentication;
            this._unwireEvents();
            this._initViewer();
            this._reportLoaded(jsondata);
        },

        _gotoParentReportModel: function (jsondata) {
            this._setParentReportViewerProperties();
            var currentpgno = this._currentPage;
            this._unwireEvents();
            this._initViewer();
            this._currentPage = currentpgno;
            this.doAjaxPost("POST", this._actionUrl, JSON.stringify({ 'reportAction': this._reportAction.getParameters }), "_getParameters");
        },

        _clearCurrentServerCache: function (jsonData) {
        },
        //-------------------- Ajax Web API Call back methods[end] -------------------------//

        //-------------------- Print Actions[Start] -------------------------//
        _print: function () {
            this.doAjaxPost("POST", this._actionUrl, JSON.stringify({ 'reportAction': this._reportAction.getPrintModel }), "_getPrintModel");
        },

        _printReport: function (printModel) {
            var browserInfo = ej.browserInfo();
            var reportModel = printModel;

            var pageHeight = reportModel.PaperHeight;
            var pageWidth = reportModel.PaperWidth;
            var headerTop = reportModel.MarginTop;
            var contentLeft = reportModel.MarginLeft;
            var isIe9 = (browserInfo.name == 'msie' && (parseInt(browserInfo.version) <= 9.0));
            var printWind = window.open('');

            if (printWind != null) {
                printWind.document.write('<html>');
                printWind.document.write('<style>h1 {display:block;height:0px;page-break-after:always;}@page { size:' + pageHeight + 'px ' + pageWidth + 'px;}@media print{ #header, #footer { display:none } }</style>');
                printWind.document.write('<BODY onload="window.print()" style="width:' + pageWidth + 'px">');

                for (var i = 0; i < reportModel.TotalPages; i++) {
                    var pagdata = reportModel.PageData[i];
                    printWind.document.write('<div style="position:relative;height:' + pageHeight + 'px;width:' + pageWidth + 'px">');
                    var printContent = this._renderPrintContainer(pagdata, headerTop, contentLeft, i);
                    printWind.document.write(printContent);
                    printWind.document.write('</div>');

                    if (!isIe9) {
                        printWind.document.write('<h1></h1>');
                    }
                    else if (isIe9 && (i < reportModel.TotalPages)) {
                        printWind.document.write('<h1></h1>');
                    }
                }

                printWind.document.write('</body></html>');
                printWind.document.close();
            }
        },

        _renderPrintContainer: function (pagedata, top, left, index) {
            var oDiv = document.createElement("div");
            document.body.appendChild(oDiv);
            $(oDiv).css({'overflow': 'auto', 'width':'1px', 'height': '1px' });
            //$(oDiv).hide();

            var $printviewheaderContainer = ej.buildTag("div", "", { 'position': 'relative', 'background-color': 'transparent', 'width': pagedata.Width + 'px', 'height': pagedata.HeaderHeight + 'px' }, {});
            var $printviewheaderborder = ej.buildTag("div.pageHeaderBorder", "", { 'position': 'relative', 'background-color': '#FFFFFF', 'left': left, 'top': top });
            $printviewheaderborder.append($printviewheaderContainer);
            $(oDiv).append($printviewheaderborder);

            var $printviewbodyContainer = ej.buildTag("div", "", { 'position': 'relative', 'background-color': 'transparent', 'width': pagedata.Width + 'px', 'height': pagedata.Height + 'px' }, {});
            var $printviewbodyborder = ej.buildTag("div.pageBodyBorder", "", { 'position': 'relative', 'background-color': '#FFFFFF', 'left': left, 'top': top });
            $printviewbodyborder.append($printviewbodyContainer);
            $(oDiv).append($printviewbodyborder);

            var $printviewfooterContainer = ej.buildTag("div", "", { 'position': 'relative', 'background-color': 'transparent', 'width': pagedata.Width + 'px', 'height': pagedata.FooterHeight + 'px' }, {});
            var $printviewfooterborder = ej.buildTag("div.pageFooterBorder", "", { 'position': 'relative', 'background-color': '#FFFFFF', 'left': left, 'top': top });
            $printviewfooterborder.append($printviewfooterContainer);
            $(oDiv).append($printviewfooterborder);

            if (pagedata.HeaderStyleModel) {
                this._applyPageStyle($printviewheaderborder, pagedata.HeaderStyleModel, true);
            }

            if (pagedata.FooterStyleModel) {
                this._applyPageStyle($printviewfooterborder, pagedata.FooterStyleModel, true);
            }

            if (pagedata.BodyStyleModel) {
                this._applyPageStyle($printviewbodyborder, pagedata.BodyStyleModel, true);
            }

            if (pagedata.HeaderModel && pagedata.HeaderModel.length > 0) {
                this._renderPageControls(pagedata.HeaderModel, $printviewheaderContainer, true, 'Print' + index);
            }

            if (pagedata.FooterModel && pagedata.FooterModel.length > 0) {
                this._renderPageControls(pagedata.FooterModel, $printviewfooterContainer, true, 'Print' + index);
            }

            if (pagedata.PageModel && pagedata.PageModel.length > 0) {
                this._renderPageControls(pagedata.PageModel, $printviewbodyContainer, true, 'Print' + index);
            }

            var htmlele = oDiv.innerHTML.replace(/PrintImage/g, 'GetResource');
            document.body.removeChild(oDiv);
            return htmlele;
        },

        _showPrintButton: function (enable) {
            if (enable) {
                $('#' + this._id + '_toolbar_Print').parent().css("display", "block");
            }
            else {
                $('#' + this._id + '_toolbar_Print').parent().css("display", "none");
            }
        },
        //-------------------- Print Actions[end] -------------------------//

        //-------------------- Export Actions[Start] -------------------------//    
        _exportMenuClick: function (event) {
            var exportType;
            var exportTxt = $(event.target).text();
            if (exportTxt) {
                if (exportTxt.indexOf("PDF") != -1) {
                    exportType = 'PDF';
                } else if (exportTxt.indexOf("Word") != -1) {
                    exportType = 'Word_' + this.model.exportSettings.wordFormat;
                } else if (exportTxt.indexOf("Excel") != -1) {
                    exportType = 'Excel_' + this.model.exportSettings.excelFormat;
                } else if (exportTxt.indexOf("Html") != -1) {
                    exportType = 'Html';
                }
                this._exportReport(exportType);
            }
        },

        _exportReport: function (exportType) {
            if (exportType && exportType.length > 0) {
                if ($('#' + this._id + '_exportIframe').length > 0) {
                    $('#' + this._id + '_exportIframe').attr('src', this.model.reportServiceUrl + '/GetResource?key=' + this._authenticationToken + '&resourcetype=' + exportType + '&isPrint=' + false);
                } else {
                    $('body').append('<iframe style="display:none" id=' + this._id + '_exportIframe' + ' width="0" height="0" frameborder="0" src="' + this.model.reportServiceUrl + '/GetResource?key=' + this._authenticationToken + '&resourcetype=' + exportType + '&isPrint=' + false + '"></iframe>');
                }
            }
            $('#' + this._id + '_toolbar_exportListTip').hide();
        },

        _showExportList: function () {
            if (this.model.exportSettings.exportOptions & ej.ReportViewer.ExportOptions.Html) {
                $('#' + this._id + '_html').css('display', 'block');
            } else {
                $('#' + this._id + '_html').css('display', 'none');
            }

            if (this.model.exportSettings.exportOptions & ej.ReportViewer.ExportOptions.Word) {
                $('#' + this._id + '_word').css('display', 'block');
            } else {
                $('#' + this._id + '_word').css('display', 'none');
            }

            if (this.model.exportSettings.exportOptions & ej.ReportViewer.ExportOptions.Excel) {
                $('#' + this._id + '_xls').css('display', 'block');
            } else {
                $('#' + this._id + '_xls').css('display', 'none');
            }

            if (this.model.exportSettings.exportOptions & ej.ReportViewer.ExportOptions.Pdf) {
                $('#' + this._id + '_pdf').css('display', 'block');
            } else {
                $('#' + this._id + '_pdf').css('display', 'none');
            }
        },

        _showExportControls: function (enable) {
            if (enable) {
                $('#' + this._id + '_toolbarContainer .e-reportviewer-export').parent().css("display", "block");
            }
            else {
                $('#' + this._id + '_toolbarContainer .e-reportviewer-export').parent().css("display", "none");
            }
        },

        _showTooltip: function () {
            var exportTooltip = $('#' + this._id + '_toolbar_exportListTip');
            if (!(exportTooltip.length > 0)) {
                exportTooltip = this._renderToolTipExport();
                $('body').append(exportTooltip);
                this._showExportList();
                this._on($('#' + this._id + '_toolbar_exportListTip .e-reportviewer-popupli'), "click", this._exportMenuClick);
            }

            if (exportTooltip.css('display') == 'block') {
                exportTooltip.css('display', 'none');
            } else {
                exportTooltip.css('display', 'block');
            }

            var exportTagPos = $('#' + this._id + '_toolbarContainer .e-reportviewer-export')[0].getBoundingClientRect();
            exportTooltip.css({ 'top': (exportTagPos.top + exportTagPos.height) - 3, 'left': (exportTagPos.left - 5) });
        },

        //-------------------- Export Actions[End] -------------------------//

        //-------------------- Page Navigation Actions[Start] -------------------------//
        _updatePageNavigation: function (pageNo, totalPage) {
            if (pageNo > 1 && pageNo < totalPage) {
                $('#' + this._id + '_toolbarContainer .e-reportviewer-gotofirst').removeClass('e-reportviewer-disabled');
                $('#' + this._id + '_toolbarContainer .e-reportviewer-gotoprevious').removeClass('e-reportviewer-disabled');
                $('#' + this._id + '_toolbarContainer .e-reportviewer-gotonext').removeClass('e-reportviewer-disabled');
                $('#' + this._id + '_toolbarContainer .e-reportviewer-gotolast').removeClass('e-reportviewer-disabled');
            } else if (pageNo == 1 && pageNo < totalPage) {
                $('#' + this._id + '_toolbarContainer .e-reportviewer-gotofirst').addClass('e-reportviewer-disabled');
                $('#' + this._id + '_toolbarContainer .e-reportviewer-gotoprevious').addClass('e-reportviewer-disabled');
                $('#' + this._id + '_toolbarContainer .e-reportviewer-gotonext').removeClass('e-reportviewer-disabled');
                $('#' + this._id + '_toolbarContainer .e-reportviewer-gotolast').removeClass('e-reportviewer-disabled');
            } else if (pageNo == totalPage && totalPage != 1) {
                $('#' + this._id + '_toolbarContainer .e-reportviewer-gotofirst').removeClass('e-reportviewer-disabled');
                $('#' + this._id + '_toolbarContainer .e-reportviewer-gotoprevious').removeClass('e-reportviewer-disabled');
                $('#' + this._id + '_toolbarContainer .e-reportviewer-gotonext').addClass('e-reportviewer-disabled');
                $('#' + this._id + '_toolbarContainer .e-reportviewer-gotolast').addClass('e-reportviewer-disabled');
            } else {
                $('#' + this._id + '_toolbarContainer .e-reportviewer-gotofirst').addClass('e-reportviewer-disabled');
                $('#' + this._id + '_toolbarContainer .e-reportviewer-gotoprevious').addClass('e-reportviewer-disabled');
                $('#' + this._id + '_toolbarContainer .e-reportviewer-gotonext').addClass('e-reportviewer-disabled');
                $('#' + this._id + '_toolbarContainer .e-reportviewer-gotolast').addClass('e-reportviewer-disabled');
            }
        },

        _gotoFirstPage: function () {
            if (this._pageModel && this._pageModel.TotalPages) {
                if (this._pageModel.TotalPages > 0) {
                    this._currentPage = 1;
                    this._refresh = false;
                    this.doAjaxPost("POST", this._actionUrl, JSON.stringify({ 'reportAction': this._reportAction.getPageModel, 'refresh': this._refresh, 'pageindex': this._currentPage, 'pageInit': this._isToolbarClick, 'isPrint': this._printMode }), !this._printMode ? "_getPageModel" : "_getPreviewModel");
                }
            }
        },

        _gotoLastPage: function () {
            if (this._pageModel && this._pageModel.TotalPages) {
                if (this._pageModel.TotalPages > 0) {
                    this._currentPage = this._pageModel.TotalPages;
                    this._refresh = false;
                    this.doAjaxPost("POST", this._actionUrl, JSON.stringify({ 'reportAction': this._reportAction.getPageModel, 'refresh': this._refresh, 'pageindex': this._currentPage, 'pageInit': this._isToolbarClick, 'isPrint': this._printMode }), !this._printMode ? "_getPageModel" : "_getPreviewModel");
                }
            }
        },

        _gotoNextPage: function () {
            if (this._pageModel && this._pageModel.TotalPages) {
                if (this._currentPage < this._pageModel.TotalPages) {
                    this._currentPage = this._currentPage + 1;
                    this._refresh = false;
                    this.doAjaxPost("POST", this._actionUrl, JSON.stringify({ 'reportAction': this._reportAction.getPageModel, 'refresh': this._refresh, 'pageindex': this._currentPage, 'pageInit': this._isToolbarClick, 'isPrint': this._printMode }), !this._printMode ? "_getPageModel" : "_getPreviewModel");
                }
            }
        },

        _gotoPreviousPage: function () {
            if (this._pageModel && this._pageModel.TotalPages) {
                if (this._currentPage > 1) {
                    this._currentPage = this._currentPage - 1;
                    this._refresh = false;
                    this.doAjaxPost("POST", this._actionUrl, JSON.stringify({ 'reportAction': this._reportAction.getPageModel, 'refresh': this._refresh, 'pageindex': this._currentPage, 'pageInit': this._isToolbarClick, 'isPrint': this._printMode }), !this._printMode ? "_getPageModel" : "_getPreviewModel");
                }
            }
        },

        _gotoPageNo: function (pageNo) {
            if (this._pageModel && this._pageModel.TotalPages) {
                var pagenumber = parseInt(pageNo);
                if (pagenumber >= 1 && pagenumber <= this._pageModel.TotalPages && this._currentPage != pagenumber) {
                    this._currentPage = pagenumber;
                    this._refresh = false;
                    this.doAjaxPost("POST", this._actionUrl, JSON.stringify({ 'reportAction': this._reportAction.getPageModel, 'refresh': this._refresh, 'pageindex': this._currentPage, 'pageInit': this._isToolbarClick, 'isPrint': this._printMode }), !this._printMode ? "_getPageModel" : "_getPreviewModel");
                }
            }
        },

        _gotoPage: function (pageNo) {
            if (this._pageModel && this._pageModel.PageData && this._pageModel.PageData.length > 0) {
                var pagenumber = parseInt(pageNo);
                this._updatePageNavigation(pagenumber, this._pageModel.TotalPages);
                if (this._pageModel.TotalPages > 0 && this._pageModel.TotalPages) {
                    $('#' + this._id + '_toolbarContainer span.e-reportviewer-labelpageno').html(" / " + this._pageModel.TotalPages);
                    $('#' + this._id + '_txtpageNo').val(pagenumber);
                }
                var pageData = this._pageModel.PageData[0];
                this._renderPageModels(pageData);
                this._setPageSize(pageData.Height, pageData.Width, pageData.HeaderHeight, pageData.FooterHeight);
            }
        },

        _onkeyPress: function (event) {
            if (event.keyCode == 13) {
                try {
                    var val = parseInt($(event.currentTarget).val());
                    this._gotoPageNo(val);
                } catch (err) {
                }
            }
        },

        _showPageNavigationControls: function (enable) {
            if (enable) {
                $('#' + this._id + '_txtpageNo').parents(".e-reportviewer-toolbarul").css("display", "block");
            } else {
                $('#' + this._id + '_txtpageNo').parents(".e-reportviewer-toolbarul").css("display", "none");
            }
        },
        //-------------------- Page Navigation Actions[End] -------------------------//

        //-------------------- Page Zoom[end] -------------------------//
        _zoomValChange: function (sender) {
            var ejViewer = $(this.itemsContainer).closest(".e-reportviewer");
            var ejViewerInstance = ejViewer.ejReportViewer("instance");
            var zoomVal = parseInt(sender.value) / 100;
            ejViewerInstance._zoomLevel = sender.itemId;
            ejViewerInstance._zoomContainer(zoomVal, false);
        },

        _zoomIn: function () {
            if (this._zoomLevel >= 6) {
                this._zoomLevel = 6;
            } else {
                this._zoomLevel++;
            }
            var zoomddl = $('#' + this._id + '_toolbar_zoomSelection').data('ejDropDownList');
            zoomddl._selectItemByIndex(this._zoomLevel);
        },

        _zoomOut: function () {
            if (this._zoomLevel <= 0) {
                this._zoomLevel = 0;
            } else {
                this._zoomLevel--;
            }
            var zoomddl = $('#' + this._id + '_toolbar_zoomSelection').data('ejDropDownList');
            zoomddl._selectItemByIndex(this._zoomLevel);
        },

        _zoomInContainer: function (isFactor, scaleFactor) {
            var zoomFactor = scaleFactor;
            var pageViewerContainer = $('#' + this._id + '_viewerContainer');
            var pagecontainer = $('#' + this._id + '_pageviewContainer');
            var pageViewline = $('#' + this._id + '_pageviewOuterline');
            if (isFactor) {
                this._zoomVal = this._zoomVal + zoomFactor;
            }
            var w = pagecontainer.width();
            var h = pagecontainer.height();
            if (this.model.printMode) {
                w = w + (this._pageModel.MarginLeft + this._pageModel.MarginRight);
                h = h + (this._pageModel.MarginTop + this._pageModel.MarginBottom);
            }

            this._applyPageTransform(pagecontainer, "scale(" + this._zoomVal + "," + this._zoomVal + ")");
            pageViewline.css({ "width": w * this._zoomVal, "height": h * this._zoomVal });

            if (!this.model.printMode) {
                if (pageViewerContainer.width() < pageViewline.outerWidth()) {
                    pageViewerContainer.scrollLeft(this._originX + (((w + 10) * zoomFactor) / 2));
                }

                if (pageViewerContainer.height() < pageViewline.outerHeight()) {
                    pageViewerContainer.scrollTop(this._originY + (((h + 10) * zoomFactor) / 2));
                }
            }
        },

        _fitToPage: function (fitType) {
            var pageViewerContainer = $('#' + this._id + '_viewerContainer');
            var pagecontainer = $('#' + this._id + '_pageviewContainer');
            var pageViewline = $('#' + this._id + '_pageviewOuterline');
            var w = pageViewerContainer.width();
            var h = pageViewerContainer.height();
            var pageW = pageViewline.width();
            var pageH = pageViewline.height();
            var scaleX = 1;
            var scaleY = 1;

            if (fitType == "fitToPage") {
                scaleX = (w - 40) / pageW * this._preZoomVal;
                scaleY = (h - 40) / pageH * this._preZoomVal;
                var scaleXY = scaleX > scaleY ? scaleY : scaleX;
                this._applyPageTransform(pagecontainer, "scale(" + scaleXY + "," + scaleXY + ")");
                pageViewline.css({ "width": w * scaleXY, "height": h * scaleXY });
            } else if (fitType == "fitToHeight") {
                if (h < pageH) {
                    scaleY = (h - 40) / pageH * this._preZoomVal;
                    this._applyPageTransform(pagecontainer, "scaleY(" + scaleY + ")");
                    pageViewline.css({ "height": h * scaleY, "width": pageW / this._preZoomVal });
                }
            } else if (fitType == "fitToWidth") {
                if (w < pageW) {
                    scaleX = (w - 40) / pageW * this._preZoomVal;
                    this._applyPageTransform(pagecontainer, "scaleX(" + scaleX + ")");
                    pageViewline.css({ "width": w * scaleX, "height": pageH / this._preZoomVal });
                }
            } 
        },

        _zoomOutContainer: function (isFactor, scaleFactor) {
            var zoomFactor = scaleFactor;
            var pageViewerContainer = $('#' + this._id + '_viewerContainer');
            var pagecontainer = $('#' + this._id + '_pageviewContainer');
            var pageViewline = $('#' + this._id + '_pageviewOuterline');
            if (isFactor) {
                this._zoomVal = this._zoomVal - zoomFactor;
            }
            var w = pagecontainer.width();
            var h = pagecontainer.height();
            this._applyPageTransform(pagecontainer, "scale(" + this._zoomVal + "," + this._zoomVal + ")");
            pageViewline.css({ "width": w * this._zoomVal, "height": h * this._zoomVal });

            if (pageViewerContainer.width() < pageViewline.outerWidth()) {
                pageViewerContainer.scrollLeft(this._originX - (((w + 10) * zoomFactor) / 2));
            }

            if (pageViewerContainer.height() < pageViewline.outerHeight()) {
                pageViewerContainer.scrollTop(this._originY - (((h + 10) * zoomFactor) / 2));
            }
        },

        _applyPageTransform: function (targetTag, transform) {
            var browserInfo = ej.browserInfo();
            if (browserInfo.name == 'msie') {
                targetTag.css({ '-ms-transform': transform, '-ms-transform-origin': '0 0' });
                this._setZoomScrollerIE();
            } else if (browserInfo.name == 'opera') {
                targetTag.css({ '-o-transform': transform, '-o-transform-origin': '0 0' });
            }
            else if (browserInfo.name == 'mozilla') {
                if (browserInfo.version == '11.0') {
                    targetTag.css({ 'transform': transform, 'transform-origin': '0 0' });
                } else {
                    targetTag.css({ '-moz-transform': transform, '-moz-transform-origin': '0 0' });
                }
            } else if (browserInfo.name == 'webkit' || browserInfo.name == 'chrome') {
                targetTag.css({ '-webkit-transform': transform, '-webkit-transform-origin': '0 0' });
            }
        },

        _zoomContainer: function (zoomfactor, isMode) {
            this._zoomVal = zoomfactor;
            var scalefactor = Math.abs(this._preZoomVal - zoomfactor);
            if (scalefactor == 0) return false;
            if (this._preZoomVal > zoomfactor) {
                this._zoomOutContainer(isMode, scalefactor);
            } else {
                this._zoomInContainer(isMode, scalefactor);
            }
            this._preZoomVal = zoomfactor;
        },

        _setZoomScrollerIE: function (zoomfact) {
            var browserInfo = ej.browserInfo();
            if (browserInfo.name == 'msie') {
                if (this._pageModel && this._pageModel.PageData && this._pageModel.PageData.length > 0) {
                    var scrollContainer = $('#' + this._id + '_viewerContainer');
                    var pagdata = this._pageModel.PageData[0];
                    var pageHeight = pagdata.Height;
                    var pageWidth = pagdata.Width;

                    var headerHeight = pagdata.HeaderHeight;
                    var footerHeight = pagdata.FooterHeight;

                    var totalpageHeight = pageHeight + headerHeight + footerHeight;

                    var height = scrollContainer.height();
                    var width = scrollContainer.width();

                    totalpageHeight = totalpageHeight * zoomfact;
                    pageWidth = pageWidth * zoomfact;

                    scrollContainer.css({ "overflow": "hidden" });
                    scrollContainer.css({ "overflow-x": "Auto" });
                    scrollContainer.css({ "overflow-y": "Auto" });

                    if (totalpageHeight < height && pageWidth < width) {
                        scrollContainer.css({ "overflow-x": "hidden" });
                        scrollContainer.css({ "overflow-y": "hidden" });
                    } else if (pageWidth < width) {
                        scrollContainer.css({ "overflow-x": "hidden" });
                    } else if (totalpageHeight < height) {
                        scrollContainer.css({ "overflow-y": "hidden" });
                    }
                }
            }
        },

        _showZoomControl: function (enable) {
            if (enable) {
                $('#' + this._id + '_toolbar_zoomSelection').parents(".e-reportviewer-toolbarul").css("display", "block");
            }
            else {
                $('#' + this._id + '_toolbar_zoomSelection').parents(".e-reportviewer-toolbarul").css("display", "none");
            }
        },
        //-------------------- Page Zoom[end] -------------------------//

        //-------------------- Refresh Report[Start] -------------------------//
        _refreshReport: function () {
            this._refresh = true;
            this._unwireEvents();
            this._initViewer();
            this.doAjaxPost("POST", this._actionUrl, JSON.stringify({ 'reportAction': this._reportAction.getDataSourceCredential, 'dataSourceCredentials': this.model.dataSourceCredentials, 'dataSources': this.model.dataSources, 'parameters': this.model.parameters }), "_getDataSourceCredential");
        },

        _showRefreshButton: function (enable) {
            if (enable) {
                $('#' + this._id + '_toolbar_refresh').parent().css("display", "block");
            }
            else {
                $('#' + this._id + '_toolbar_refresh').parent().css("display", "none");
            }
        },
        //-------------------- Refresh Report[End] -------------------------//

        //-------------------- Toolbar Actions[Start] -------------------------//
        _wireEvents: function () {
            if (!this.model.toolbarSettings.templateId) {
                this._on($('#' + this._id + '_txtpageNo'), "keypress", this._onkeyPress);
                this._on($('#' + this._id + '_zoomSelection'), "change", this._zoomValChange);
            }
            
            this._on($('#' + this._id + '_viewerContainer'), "scroll", this._scrollPage);
            this._on($('#' + this._id + '_pageviewOuterContainer'), "click", this._viewerClick);
            //this._on($('#' + this._id + '_viewerContainer'), "pinchin pinchout swipeleft swiperight", this._pinchAndSwipe);
            this._on($(document), "click", this._onReporClick);


            if (this._isDevice) {
                this._on($('#' + this._id + '_viewerContainer'), "scroll", this._scrollPage);
                this._on($('#' + this._id + '_viewerContainer'), "scrollstop", this._containerScrollStop);
            }else {
                this._on($('#' + this._id + '_toolbarContainer li,#' + this._id + '_toolbarContainer .e-reportviewer-ejdropdownlist,#' + this._id + '_toolbarContainer .e-reportviewer-tbpage'), "mouseover", this._showIconToolTip);
                this._on($('#' + this._id + '_toolbarContainer li,#' + this._id + '_toolbarContainer .e-reportviewer-ejdropdownlist,#' + this._id + '_toolbarContainer .e-reportviewer-tbpage'), "mouseout", this._hideIconToolTip);
            }
        },

        _unwireEvents: function () {
            if (!this.model.toolbarSettings.templateId) {
                this._off($('#' + this._id + '_txtpageNo'), "keypress", this._onkeyPress);
                this._off($('#' + this._id + '_zoomSelection'), "change", this._zoomValChange);
            }

            this._off($('#' + this._id + '_viewerContainer'), "scroll", this._scrollPage);
            this._off($('#' + this._id + '_pageviewOuterContainer'), "click", this._viewerClick);
            //this._off($('#' + this._id + '_viewerContainer'), "pinchin pinchout swipeleft swiperight", this._pinchAndSwipe);
            this._off($(document), "click", this._onReporClick);

            if (this._isDevice) {
                this._off($('#' + this._id + '_viewerContainer'), "scroll", this._scrollPage);
                this._off($('#' + this._id + '_viewerContainer'), "scrollstop", this._containerScrollStop);
            }else {
                this._off($('#' + this._id + '_toolbarContainer li,#' + this._id + '_toolbarContainer .e-reportviewer-ejdropdownlist,#' + this._id + '_toolbarContainer .e-reportviewer-tbpage'), "mouseover", this._showIconToolTip);
                this._off($('#' + this._id + '_toolbarContainer li,#' + this._id + '_toolbarContainer .e-reportviewer-ejdropdownlist,#' + this._id + '_toolbarContainer .e-reportviewer-tbpage'), "mouseout", this._hideIconToolTip);
            }
        },

        _onReporClick: function (event) {
            if (!($(event.target).hasClass('e-reportviewer-export') || $($( event.target ).children()).hasClass('e-reportviewer-export'))) {
                $('#' + this._id + '_toolbar_exportListTip').css('display', 'none');
            } else {
                if ($('#' + this._id + '_toolbar_exportListTip').css('display') != "block") {
                    $('#' + this._id + '_toolbar_exportListTip').css('display', 'none');
                }
            }
        },

        _toolbarClick: function (event) {
            var targetItem = event.target
            var clickedItem = event.target;
            if (this._isToolbarClick) {
                if ($(clickedItem).hasClass('e-reportviewer-toolbarli')) {
                    clickedItem = $(clickedItem).find('span');
                }
                if ($(clickedItem).hasClass("e-reportviewer-print")) {
                    this._print();
                }
                else if ($(clickedItem).hasClass("e-reportviewer-export")) {
                    this._showTooltip();
                }
                else if ($(clickedItem).hasClass("e-reportviewer-gotofirst") && !($(clickedItem).hasClass("e-reportviewer-disabled"))) {
                    this._gotoFirstPage();
                }
                else if ($(clickedItem).hasClass("e-reportviewer-gotolast") && !($(clickedItem).hasClass("e-reportviewer-disabled"))) {
                    this._gotoLastPage();
                }
                else if ($(clickedItem).hasClass("e-reportviewer-gotonext") && !($(clickedItem).hasClass("e-reportviewer-disabled"))) {
                    this._gotoNextPage();
                }
                else if ($(clickedItem).hasClass("e-reportviewer-gotoprevious") && !($(clickedItem).hasClass("e-reportviewer-disabled"))) {
                    this._gotoPreviousPage();
                }
                else if ($(clickedItem).hasClass("e-reportviewer-gotoparent")) {
                    this._gotoParentReport();
                }
                else if ($(clickedItem).hasClass("e-reportviewer-documentmap") && !this._printMode) {
                    this._documentMapClick();
                }
                else if ($(clickedItem).hasClass("e-reportviewer-zoomin")) {
                    this._zoomIn();
                }
                else if ($(clickedItem).hasClass("e-reportviewer-zoomout")) {
                    this._zoomOut();
                }
                else if ($(clickedItem).hasClass("e-reportviewer-preview")) {
                    this._printlayout();
                }
                else if ($(clickedItem).hasClass("e-reportviewer-refresh")) {
                    this._refreshReport();
                }
                else if ($(clickedItem).hasClass("e-reportviewer-parameter")) {
                    this._toggleParameterBlock();
                }
            }
        },

        _showToolbar: function (enableToolbar) {
            if (enableToolbar) {
                $('#' + this._id + '_toolbarContainer').css("display", "block");
            }
            else {
                $('#' + this._id + '_toolbarContainer').css("display", "none");
            }
        },

        _initToolbar: function () {
            this._isToolbarClick = false;

            if (!this.model.toolbarSettings.templateId) {
                if (!this._isDevice) {
                    var zoomddl = $('#' + this._id + '_toolbar_zoomSelection').data('ejDropDownList');
                    if (this._refresh) {
                        zoomddl._selectItemByIndex(zoomddl.model.selectedItemIndex);
                    } else {
                        zoomddl._selectItemByIndex(2);
                    }
                    zoomddl.disable();

                    $('#' + this._id + '_txtpageNo').attr('disabled', 'disabled');

                    $('#' + this._id + '_toolbarContainer .e-reportviewer-gotofirst').addClass('e-reportviewer-disabled');
                    $('#' + this._id + '_toolbarContainer .e-reportviewer-gotoprevious').addClass('e-reportviewer-disabled');
                    $('#' + this._id + '_toolbarContainer .e-reportviewer-gotonext').addClass('e-reportviewer-disabled');
                    $('#' + this._id + '_toolbarContainer .e-reportviewer-gotolast').addClass('e-reportviewer-disabled');
                }
            }
        },

        _enableToolbarItems: function () {
            this._isToolbarClick = true;
            if (!this.model.toolbarSettings.templateId) {
                if (!this._isDevice) {
                    $('#' + this._id + '_txtpageNo').removeAttr('disabled');
                    $('#' + this._id + '_toolbar_zoomSelection').data('ejDropDownList').enable();
                }
            }
        },
        //-------------------- Toolbar Actions[End] -------------------------//
		
		//-------------------- Tooltip Localization Actions[Start]------------------------//
        _showIconToolTip: function (event) {
            if ($('#' + this._id + '_rptTooltip').css('display') == 'none') {
                this._showTooltipContent(event);
            }
        },

        _hideIconToolTip: function () {
            $('#' + this._id + '_rptTooltip').css('display', 'none');
        },

        _showTooltipContent: function (event) {
            var currentToolbarItem = event.target;
            if (this.model.toolbarSettings.showTooltip) {
                var TagPos;
                var toolTipText;
                if ($(currentToolbarItem).hasClass('e-reportviewer-toolbarli')) {
                    currentToolbarItem = $(currentToolbarItem).find('span');
                }

                if ($(currentToolbarItem).hasClass("e-reportviewer-print")) {
                    TagPos = $('#' + this._id + '_toolbarContainer .e-reportviewer-print')[0].getBoundingClientRect();
                    toolTipText = this._getTootipText('print');
                }
                else if ($(currentToolbarItem).hasClass("e-reportviewer-export")) {
                    TagPos = $('#' + this._id + '_toolbarContainer .e-reportviewer-export')[0].getBoundingClientRect();
                    toolTipText = this._getTootipText('export');
                }
                else if ($(currentToolbarItem).hasClass("e-reportviewer-gotofirst")) {
                    TagPos = $('#' + this._id + '_toolbarContainer .e-reportviewer-gotofirst')[0].getBoundingClientRect();
                    toolTipText = this._getTootipText('first');
                }
                else if ($(currentToolbarItem).hasClass("e-reportviewer-gotolast")) {
                    TagPos = $('#' + this._id + '_toolbarContainer .e-reportviewer-gotolast')[0].getBoundingClientRect();
                    toolTipText = this._getTootipText('last');
                }
                else if ($(currentToolbarItem).hasClass("e-reportviewer-gotonext")) {
                    TagPos = $('#' + this._id + '_toolbarContainer .e-reportviewer-gotonext')[0].getBoundingClientRect();
                    toolTipText = this._getTootipText('next');
                }
                else if ($(currentToolbarItem).hasClass("e-reportviewer-gotoprevious")) {
                    TagPos = $('#' + this._id + '_toolbarContainer .e-reportviewer-gotoprevious')[0].getBoundingClientRect();
                    toolTipText = this._getTootipText('previous');
                }
                else if ($(currentToolbarItem).hasClass("e-reportviewer-documentmap")) {
                    TagPos = $('#' + this._id + '_toolbarContainer .e-reportviewer-documentmap')[0].getBoundingClientRect();
                    toolTipText = this._getTootipText('documentmap');
                }
                else if ($(currentToolbarItem).hasClass("e-reportviewer-zoomin")) {
                    TagPos = $('#' + this._id + '_toolbarContainer .e-reportviewer-zoomin')[0].getBoundingClientRect();
                    toolTipText = this._getTootipText('zoomin');
                }
                else if ($(currentToolbarItem).hasClass("e-reportviewer-zoomout")) {
                    TagPos = $('#' + this._id + '_toolbarContainer .e-reportviewer-zoomout')[0].getBoundingClientRect();
                    toolTipText = this._getTootipText('zoomout');
                }
                else if ($(currentToolbarItem).hasClass("e-reportviewer-preview")) {
                    TagPos = $('#' + this._id + '_toolbarContainer .e-reportviewer-preview')[0].getBoundingClientRect();
                    toolTipText = this._getTootipText('preview');
                }
                else if ($(currentToolbarItem).hasClass("e-reportviewer-refresh")) {
                    TagPos = $('#' + this._id + '_toolbarContainer .e-reportviewer-refresh')[0].getBoundingClientRect();
                    toolTipText = this._getTootipText('refresh');
                }
                else if ($(currentToolbarItem).hasClass("e-reportviewer-parameter")) {
                    TagPos = $('#' + this._id + '_toolbarContainer .e-reportviewer-parameter')[0].getBoundingClientRect();
                    toolTipText = this._getTootipText('parameter');
                }
                else if ($(currentToolbarItem).hasClass("e-reportviewer-gotoparent")) {
                    TagPos = $('#' + this._id + '_toolbarContainer .e-reportviewer-gotoparent')[0].getBoundingClientRect();
                    toolTipText = this._getTootipText('back');
                }
                else if ($(currentToolbarItem).hasClass("e-reportviewer-tbpage") || $(currentToolbarItem).hasClass("e-reportviewer-pagenumber") || $(currentToolbarItem).hasClass("e-reportviewer-labelpageno")) {
                    TagPos = $('#' + this._id + '_toolbarContainer .e-reportviewer-tbpage')[0].getBoundingClientRect();
                    toolTipText = this._getTootipText('pageIndex');
                }
                else if ($(currentToolbarItem).hasClass("e-reportviewer-ejdropdownlist") || $(currentToolbarItem).hasClass("e-dropdownlist") || $(currentToolbarItem).hasClass("e-select") || $(currentToolbarItem).hasClass("e-down-arrow")) {
                    TagPos = $('#' + this._id + '_toolbarContainer .e-reportviewer-ejdropdownlist')[0].getBoundingClientRect();
                    toolTipText = this._getTootipText('zoom');
                }

                if (toolTipText) {
                    $('#' + this._id + '_rptTooltip_Header').html(toolTipText.header);
                    $('#' + this._id + '_rptTooltip_Content').html(toolTipText.content);
                    $('#' + this._id + '_rptTooltip').css({ 'top': (TagPos.top + TagPos.height) + 5, 'left': (TagPos.left + (TagPos.width / 2)), 'display': 'block' ,'position':'fixed'});
                }
            }
        },

        _getTootipText: function (tipType) {
            var localeObj = ej.ReportViewer.localization[this.model.locale] ? ej.ReportViewer.localization[this.model.locale] : ej.ReportViewer.localization["en-us"];
            var headerTxt = '';
            var contentTxt = '';
            switch (tipType) {
                case 'print':
                    headerTxt = localeObj['toolbar']['print']['headerText'];
                    contentTxt = localeObj['toolbar']['print']['contentText'];
                    break;
                case 'export':
                    headerTxt = localeObj['toolbar']['exportformat']['headerText'];
                    contentTxt = localeObj['toolbar']['exportformat']['contentText'];
                    break;
                case 'first':
                    headerTxt = localeObj['toolbar']['first']['headerText'];
                    contentTxt = localeObj['toolbar']['first']['contentText'];
                    break;
                case 'previous':
                    headerTxt = localeObj['toolbar']['previous']['headerText'];
                    contentTxt = localeObj['toolbar']['previous']['contentText'];
                    break;
                case 'next':
                    headerTxt = localeObj['toolbar']['next']['headerText'];
                    contentTxt = localeObj['toolbar']['next']['contentText'];
                    break;
                case 'last':
                    headerTxt = localeObj['toolbar']['last']['headerText'];
                    contentTxt = localeObj['toolbar']['last']['contentText'];
                    break;
                case 'zoomin':
                    headerTxt = localeObj['toolbar']['zoomIn']['headerText'];
                    contentTxt = localeObj['toolbar']['zoomIn']['contentText'];
                    break;
                case 'zoomout':
                    headerTxt = localeObj['toolbar']['zoomOut']['headerText'];
                    contentTxt = localeObj['toolbar']['zoomOut']['contentText'];
                    break;
                case 'preview':
                    headerTxt = localeObj['toolbar']['printLayout']['headerText'];
                    contentTxt = localeObj['toolbar']['printLayout']['contentText'];
                    break;
                case 'documentmap':
                    headerTxt = localeObj['toolbar']['documentMap']['headerText'];
                    contentTxt = localeObj['toolbar']['documentMap']['contentText'];
                    break;
                case 'refresh':
                    headerTxt = localeObj['toolbar']['refresh']['headerText'];
                    contentTxt = localeObj['toolbar']['refresh']['contentText'];
                    break;
                case 'parameter':
                    headerTxt = localeObj['toolbar']['parameter']['headerText'];
                    contentTxt = localeObj['toolbar']['parameter']['contentText'];
                    break;
                case 'back':
                    headerTxt = localeObj['toolbar']['back']['headerText'];
                    contentTxt = localeObj['toolbar']['back']['contentText'];
                    break;
                case 'pageIndex':
                    headerTxt = localeObj['toolbar']['pageIndex']['headerText'];
                    contentTxt = localeObj['toolbar']['pageIndex']['contentText'];
                    break;
                case 'zoom':
                    headerTxt = localeObj['toolbar']['zoom']['headerText'];
                    contentTxt = localeObj['toolbar']['zoom']['contentText'];
                    break;
            }
            return { header: headerTxt, content: contentTxt };
        },
        //-------------------- Tooltip Localization Actions[End]------------------------//

        //-------------------- DrillDown Actions[Start] -------------------------//
        _drillAction: function () {
            if ($._data($('.txtToggle_' + this._id).get(0), 'events') == undefined) {
                this._on($('.txtToggle_' + this._id), "click", this._drillClick);
            }
        },

        _drillClick: function (event) {
            var drillInfo = $(event.currentTarget).data('drillObj');
            if (drillInfo) {
                this.doAjaxPost("POST", this._actionUrl, JSON.stringify({ 'reportAction': this._reportAction.drillDown, 'pageindex': this._currentPage, 'toggleInfo': drillInfo, 'refresh': false, 'pageInit': this._isToolbarClick, 'isPrint': this._printMode }), !this._printMode ? "_getPageModel" : "_getPreviewModel");
            }
        },
        //-------------------- DrillDown Actions[End] -------------------------//

        //-------------------- Document Map Actions[Start]----------------------//
        _showDocumentMap: function (enable) {
            if (enable) {
                $('#' + this._id + '_toolbarContainer').find('.e-reportviewer-documentmap').parent().css("display", "block");
            }
            else {
                $('#' + this._id + '_toolbarContainer').find('.e-reportviewer-documentmap').parent().css("display", "none");
            }
        },

        _appendDocumentMapDiv: function () {
            if (this._isDevice) {
                var documentMapDiv = $('#' + this._id + '_documentmapContainer');
                if (documentMapDiv.length == 0) {
                    documentMapDiv = ej.buildTag("div.e-reportviewer-documentmappopup", "", {}, { id: this._id + '_documentmapContainer' });
                    $('body').append(documentMapDiv);
                }
            }
        },

        _selectDocMapToolItem: function (enable) {
            if (!this.model.toolbarSettings.templateId) {
                var _ejToolbar = $('#' + this._id + '_toolbarContainer').data("ejToolbar");
                if (enable) {
                    _ejToolbar.selectItemByID(this._id + '_ejtb_documentmap');
                } else {
                    _ejToolbar.deselectItemByID(this._id + '_ejtb_documentmap');
                }
            }
        },

        _documentMapClick: function () {
            if (!this._isDevice) {
                var ejSplitterCtrl = $('#' + this._id + '_reportviewerContainer').data('ejSplitter');
                if ($(ejSplitterCtrl.panes[0]).hasClass('collapsed')) {
                    this._pageDocMapFlag = true;
                    ejSplitterCtrl.expand(0);
                    this._selectDocMapToolItem(true);
                } else {
                    this._pageDocMapFlag = false;
                    ejSplitterCtrl.collapse(0);
                    this._selectDocMapToolItem(false);
                }
            } else {
                var documentMapDiv = $('#' + this._id + '_documentmapContainer');
                if (documentMapDiv.css('display') == 'block') {
                    this._pageDocMapFlag = true;
                    documentMapDiv.css('display', 'none');
                    this._selectDocMapToolItem(false);
                } else {
                    this._pageDocMapFlag = false;
                    documentMapDiv.css('display', 'block');
                    this._selectDocMapToolItem(true);
                }
            }
        },

        _showDocumentMapContainer: function (enable) {
            if (!this._isDevice) {
                var ejSplitterCtrl = $('#' + this._id + '_reportviewerContainer').data('ejSplitter');
                if (enable) {
                    ejSplitterCtrl.expand(0);
                } else {
                    ejSplitterCtrl.collapse(0);
                }
            } else {
                if (enable) {
                    $('#' + this._id + '_documentmapContainer').css('display', 'block');
                } else {
                    $('#' + this._id + '_documentmapContainer').css('display', 'none');
                }
            }
        },

        _documentMapAction: function () {
            this.doAjaxPost("POST", this._actionUrl, JSON.stringify({ 'reportAction': this._reportAction.documentMap }), "_documentMapModel");
        },

        _containerSplit: function (jsondocmap) {
            var viewerContainer = $('#' + this._id + '_viewerContainer');
            this._appendDocumentMapDiv();
            jsondocmap = jsondocmap.replace("treeViewID", this._id + '_ejtreeView');
            $('#' + this._id + '_documentmapContainer').css("height", "100%");
            $('#' + this._id + '_documentmapContainer').html(jsondocmap);
            $('#' + this._id + '_ejtreeView').css("display", "none");
            $('#' + this._id + '_ejtreeView').ejTreeView({ height: viewerContainer.height(), nodeSelect: this._pageNavigate });

            if (this._isDevice) {
                $('#' + this._id + '_documentmapContainer').css({ 'position': 'absolute', 'left': viewerContainer.offset().left, 'top': viewerContainer.offset().top, 'height': viewerContainer.height(), 'width': viewerContainer.width() / 6 });
            } else {
                $('#' + this._id + '_reportviewerContainer').ejSplitter({ height: viewerContainer.height() + 1, orientation: ej.Orientation.Horizontal, properties: [{ paneSize: 140 }, {}] });
            }
            $('#' + this._id + '_ejtreeView').css("display", "block");
            this._selectDocMapToolItem(true);
        },

        _pageNavigate: function (args) {
            try {
                var jsondata = JSON.parse($(args.currentElement).attr('id'));
                var ejviewerCtrlId = this._id.replace('_ejtreeView', '');
                var ejReportViewer = $('#' + ejviewerCtrlId).data('ejReportViewer');
                if (ejReportViewer._isDevice) {
                    $('#' + ejviewerCtrlId + '_documentmapContainer').css('display', 'none');
                    this._selectDocMapToolItem(false);
                }
                ejReportViewer._gotoPageNo(jsondata.PageNo);
                $('#' + ejviewerCtrlId + '_viewerContainer').animate({ scrollTop: jsondata.TopPos - 12, scrollLeft: jsondata.LeftPos - 12 }, "600");
            } catch (err) {
            }
        },
        //-------------------- Document Map Actions[End]------------------------//

        //-------------------- DrillThrough Actions[Start]----------------------//
        _showDrillThrough: function (enable) {
            if (enable) {
                $('#' + this._id + '_toolbarContainer').find('.e-reportviewer-gotoparent').parent().css("display", "block");
            }
            else {
                $('#' + this._id + '_toolbarContainer').find('.e-reportviewer-gotoparent').parent().css("display", "none");
            }
        },

        _drillThroughAction: function () {
            if ($._data($('.drillAction_' + this._id).get(0), 'events') == undefined) {
                this._on($('.drillAction_' + this._id), "click", this._drillThroughClick);
            }
        },

        _drillThroughClick: function (event) {
            var drillActionInfo = null;
            var proxy = null;
            if (this.pluginName == 'ejChart') {
                proxy = $('#' + this.ejreportid).data('ejReportViewer');
                var chartActionInfo = $('#' + this._id).data(this._id + 'actionObj_Series' + event.data.region.SeriesIndex);
                if (ej.isNullOrUndefined(chartActionInfo)) {
                    return false;
                }
                drillActionInfo = chartActionInfo[event.data.region.Region.PointIndex];
            } else if (this.pluginName == 'ejMap') {
                proxy = $('#' + this.ejreportid).data('ejReportViewer');
				if (event.originalEvent[0] == null || ej.isNullOrUndefined(event.originalEvent[0].data)) {
                    return;
                }
                drillActionInfo = event.originalEvent[0].data.ShapeActionInfo;
            } else {
                proxy = this;
                drillActionInfo = $(event.currentTarget).data('actionObj');
            }

            if (drillActionInfo) {
                var actionInfo = { actionInfo: $.extend(true, {}, drillActionInfo) };
                var isDrill = proxy._onDrillThrough(actionInfo);
                if (!isDrill) {
                    if (drillActionInfo.Hyperlink) {
                        window.open(drillActionInfo.Hyperlink);
                    } else if (drillActionInfo.ReportName) {
                        proxy._childReportAuthentication = proxy._getAuthenticationToken();
                        proxy.doAjaxPost("POST", proxy._actionUrl, JSON.stringify({ 'reportAction': proxy._reportAction.drillThrough, 'reportName': drillActionInfo.ReportName, 'parameters': drillActionInfo.Parameters, 'authKey': proxy._childReportAuthentication }), "_drillThroughModel");
                    }
                }
            }
        },

        _gotoParentReport: function () {
            this.doAjaxPost("POST", this._actionUrl, JSON.stringify({ 'reportAction': this._reportAction.clearCache }), '_gotoParentReportModel');
        },

        _setInitialization: function () {
            this._dataSources = null;
            this._authenticationToken = null;
            this._childReportAuthentication = null;
            this._refresh = false;
            this._isToolbarClick = false;
            this._pageModel = null;
            this._currentPage = 1;
            this._reportParameters = null;
            this._reportDataSources = null;
            this._reporDataSets = null;
            this._zoomLevel = 2;
            this._isDocumentMap = false;
        },

        _cloneReportViewerProperties: function () {
            var reportProperties = {};
            reportProperties.model = $.extend(true, {}, this.model);
            reportProperties._dataSources = $.extend(true, {}, this._dataSources);
            reportProperties._authenticationToken = this._authenticationToken;
            reportProperties._childReportAuthentication = this._childReportAuthentication;
            reportProperties._refresh = this._refresh;
            reportProperties._isToolbarClick = this._isToolbarClick;
            reportProperties._pageModel = $.extend(true, {}, this._pageModel);
            reportProperties._currentPage = this._currentPage;
            reportProperties._reportParameters = $.extend(true, {}, this._reportParameters);
            reportProperties._reportDataSources = $.extend(true, {}, this._reportDataSources);
            reportProperties._reporDataSets = $.extend(true, {}, this._reporDataSets);
            reportProperties._zoomLevel = this._zoomLevel;
            reportProperties._preZoomVal = this._preZoomVal;
            reportProperties._isDevice = this._isDevice;
            reportProperties._originX = this._originX;
            reportProperties._originY = this._originY;
            reportProperties._printMode = this._printMode;
            reportProperties._isDocumentMap = this._isDocumentMap;
            return reportProperties;
        },

        _setParentReportViewerProperties: function () {
            var parentObj = this._parents[this._parents.length - 1];
            var parentProperties = parentObj.parentPro;
            this.model = parentProperties.model;
            this._dataSources = parentProperties._dataSources;
            this._authenticationToken = parentProperties._authenticationToken;
            this._childReportauthentication = parentProperties._childReportAuthentication;
            this._refresh = parentProperties._refresh;
            this._isToolbarClick = parentProperties._isToolbarClick;
            this._pageModel = parentProperties._pageModel;
            this._currentPage = parentProperties._currentPage;
            this._reportParameters = parentProperties._reportParameters;
            this._reportDataSources = parentProperties._reportDataSources;
            this._reporDataSets = parentProperties._reporDataSets;
            this._zoomLevel = parentProperties._zoomLevel;
            this._isDocumentMap = parentProperties._isDocumentMap;
            this._preZoomVal = parentProperties._preZoomVal;
            this._printMode = parentProperties._printMode;
            this._originX = parentProperties._originX;
            this._originY = parentProperties._originY;
            this._parentPageXY = parentObj.pagePos;
            this._parents.pop();
        },
        //-------------------- DrillThrough Actions[End]----------------------//

        //-------------------- Print Preview Layout[Start]----------------------//
        _printlayout: function () {
            this._updatePreviewLayout(!this._printMode);
        },

        _selectPreviewToolItem: function (enable) {
            if (!this.model.toolbarSettings.templateId) {
                var _ejToolbar = $('#' + this._id + '_toolbarContainer').data("ejToolbar");
                if (enable) {
                    _ejToolbar.selectItemByID(this._id + '_ejtb_preview');
                } else {
                    _ejToolbar.deselectItemByID(this._id + '_ejtb_preview');
                }
            }
        },

        _updatePreviewLayout: function (isPrintMode) {
            this._printMode = isPrintMode;
            if (this._printMode) {
                this._pageLayoutPage = this._currentPage;
            } else {
                this._currentPage = this._pageLayoutPage;
            }

            this._selectPreviewToolItem(this._printMode);

            if (this._isPageDocMap && !this._isDocumentMap) {
                this._showDocumentMapContainer(this._printMode ? false : this._pageDocMapFlag);
            }
            this.model.printMode = this._printMode;
            this.doAjaxPost("POST", this._actionUrl, JSON.stringify({ 'reportAction': this._reportAction.getPageModel, 'refresh': this._refresh, 'pageindex': this._currentPage, 'pageInit': this._isToolbarClick, 'isPrint': this._printMode }), !this._printMode ? "_getPageModel" : "_getPreviewModel");
        },

        _showPreviewButton: function (enable) {
            if (enable) {
                $('#' + this._id + '_toolbarContainer').find('.e-reportviewer-preview').parent().css("display", "block");
            }
            else {
                $('#' + this._id + '_toolbarContainer').find('.e-reportviewer-preview').parent().css("display", "none");
            }
        },
        //-------------------- Print Preview Layput[End]----------------------//

        //-------------------- SetCulture Layput[Start]----------------------//
        _setCultureInfo: function () {
            var culture = this.model.locale;
            $('#' + this._id + '_viewReportClick').val(ej.ReportViewer.localization[culture] ? ej.ReportViewer.localization[culture]["viewButton"] : ej.ReportViewer.localization["en-US"]["viewButton"]);
        },
        //-------------------- SetCulture Layput[End]----------------------//

        //-------------------- Common ReportViewer events & utils[start]----------------------//
        _itemfadeToggle: function (targetTag) {
            targetTag.fadeToggle();
        },

        _popupClick: function (event) {
            var pagePopup = $('#' + this._id + '_pageInfoPopup');
            pagePopup.fadeIn();
            pagePopup.finish();
        },

        _containerScrollStop: function (event) {
            var pagePopup = $('#' + this._id + '_pageInfoPopup');
            pagePopup.finish().fadeIn();
            pagePopup.finish().fadeOut(3000);
        },

        _viewerClick: function (event) {
            if (this._isDevice) {
                var pagePopup = $('#' + this._id + '_pageInfoPopup');
                pagePopup.finish().fadeIn(1000);
                pagePopup.finish().fadeOut(3000);
                pagePopup.hide(4000);
                $('#' + this._id + '_documentmapContainer').css('display', 'none');
            }
            $('#' + this._id + '_toolbar_exportListTip').css('display', 'none');
        },

        _pinchAndSwipe: function (event) {
            switch (event.type) {
                case "pinchin":
                    this._zoomOut();
                    break;
                case "pinchout":
                    this._zoomIn();
                    break;
                case "swipeleft":
                    this._gotoNextPage();
                    if (this._isDevice) {
                        this._updatePageNo();
                    }
                    break;
                case "swiperight":
                    this._gotoPreviousPage();
                    if (this._isDevice) {
                        this._updatePageNo();
                    }
                    break;
            }
        },

        _scrollPage: function (event) {
            var scrollContainer = $(event.target);
            var scrollY = this._orginY = scrollContainer.scrollTop();
            this._orginX = scrollContainer.scrollLeft();
            if (this._isDevice) {
                if (scrollY + scrollContainer.innerHeight() >= scrollContainer[0].scrollHeight) {
                    scrollContainer.scrollTop(2);
                    if (!this._pageModel || this._currentPage == this._pageModel.TotalPages) {
                        return;
                    }
                    this._gotoNextPage();
                    this._updatePageNo();
                } else if (scrollY == 0) {
                    scrollContainer.scrollTop(2);
                    if (!this._pageModel || this._currentPage == 1) {
                        return;
                    }
                    this._gotoPreviousPage();
                    this._updatePageNo();
                }
            }
        },

        _showViewerBlock: function (showblock) {
            if (showblock) {
                $('#' + this._id + '_viewBlockContainer').css("display", "block");
                var parameterBlock = $('#' + this._id + '_viewBlockContainer .e-reportviewer-viewerblockcellcontent').find('table');
                this._selectparamToolItem(parameterBlock.is('[isviewclick]'));
            } else {
                $('#' + this._id + '_viewBlockContainer').css("display", "none");
            }
        },

        _showNavigationIndicator: function (isShow) {
            $('#' + this._id + '_viewerContainer').ejWaitingPopup({ showOnInit: isShow });
            $('#' + this._id + '_viewerContainer_WaitingPopup').addClass('e-reportviewer-waitingpopup');
        },

        _showloadingIndicator: function (isShow) {
            if (isShow) {
                $('#' + this._id + '_loadingIndicator').css('display', 'block');
                $('#' + this._id + '_pageviewOuterContainer').css('display', 'none');
            }
            else {
                $('#' + this._id + '_loadingIndicator').css('display', 'none');
                $('#' + this._id + '_pageviewOuterContainer').css('display', 'block');
            }
            $('#' + this._id + '_loadingIndicator').ejWaitingPopup({ showOnInit: isShow });
            $('#' + this._id + '_loadingIndicator_WaitingPopup').addClass('e-reportviewer-waitingpopup');
        },

        _isMobileDevice: function () {
            //return (/mobile|tablet|android|kindle/i.test(navigator.userAgent.toLowerCase()));  
            return (/windows phone|android|iphone/i.test(navigator.userAgent.toLowerCase()));
        },

        _getAuthenticationToken: function () {
            function _guid() {
                function guid() {
                    return (((1 + Math.random()) * 0x10000) | 0).toString(16).substring(1);
                }
                return (guid() + guid() + "-" + guid() + "-" + guid() + "-" + guid() + "-" + guid() + guid() + guid()).toUpperCase();
            }
            return _guid() + "^" + this._id;
        },

        _updateToolbarmodel: function () {
            this._showToolbar(this.model.toolbarSettings.showToolbar);
            this._showPrintButton(this.model.toolbarSettings.items & ej.ReportViewer.ToolbarItems.Print);
            this._showRefreshButton(this.model.toolbarSettings.items & ej.ReportViewer.ToolbarItems.Refresh);
            this._showZoomControl(this.model.toolbarSettings.items & ej.ReportViewer.ToolbarItems.Zoom);
            this._showExportControls(this.model.toolbarSettings.items & ej.ReportViewer.ToolbarItems.Export);
            this._showExportList();
            this._showPageNavigationControls(this.model.toolbarSettings.items & ej.ReportViewer.ToolbarItems.PageNavigation);
            this._showParameterBlock(this.model.toolbarSettings.items & ej.ReportViewer.ToolbarItems.Parameters);
            this._showPreviewButton(model.toolbarSettings.items & ej.ReportViewer.ToolbarItems.PrintLayout);
        },

        _viewerResize: function (event) {
            var proxy = this;
            if (this.resizeTO) clearTimeout(this.resizeTO);
            this.resizeTO = setTimeout(function () {
                proxy.element.width(proxy.element[0].parentElement.clientWidth);
                if (proxy.element[0].parentElement.clientHeight != 0) {
                    proxy.element.height(proxy.element[0].parentElement.clientHeight);
                } else {
                    proxy.element.height(proxy.element.height());
                }
                proxy._setContainerSize();
            }, 200);
        },

        _showViewerPage: function (isEnable) {
            if (isEnable) {
                $('#' + this._id + '_pageviewOuterline').css('display', 'block');
            } else {
                $('#' + this._id + '_pageviewOuterline').css('display', 'none');
            }
        },

        _updatePageNo: function () {
            $('#' + this._id + '_popupPageNo').val(this._currentPage);
            $('#' + this._id + '_pageInfoPopup .e-reportviewer-popuptotalpage').html(' / ' + this._pageModel.TotalPages);
        },

        _reportReload: function () {
            this._actionUrl = this.model.reportServiceUrl + "/ReportingAction";
            this._refresh = true;
            this._unwireEvents();
            this._initViewer();
            this.doAjaxPost("POST", this._actionUrl, JSON.stringify({ 'reportAction': this._reportAction.reportLoad, 'controlId': this._id, 'reportPath': this.model.reportPath, 'reportServerUrl': this.model.reportServerUrl, 'processingMode': this.model.processingMode }), "_reportLoaded");
        },
        //-------------------- Common ReportViewer events & utils[End]----------------------//

        /*---------------------client side methods[start]----------------------------------------------------------*/
        /**
        * Get the available datasets name of the rdlc report.
        * @return jQuery
        * @example 
        * &lt;div id="reportviewer"&gt;ReportViewer&lt;/div&gt; <br> 
        * &lt;script&gt;
        * var reportviewerObj = $("#reportviewer").data("ejReportViewer");
        * reportviewerObj.getDataSetNames(); // To get the dataset names.
        * &lt;/script&gt;
        * @memberof ejReportViewer
        * @instance
        */
        getDataSetNames: function () {
            var reportdatasets = $.extend(true, {}, this._reporDataSets);
            return reportdatasets;
        },

        /**
        * Get the available parameters of the report.
        * @return jQuery
        * @example 
        * &lt;div id="reportviewer"&gt;ReportViewer&lt;/div&gt; <br> 
        * &lt;script&gt;
        * var reportviewerObj = $("#reportviewer").data("ejReportViewer");
        * reportviewerObj.getParameters(); // To get the parameters.
        * &lt;/script&gt;
        * @memberof ejReportViewer
        * @instance
        */
        getParameters: function () {
            var reportparameters = $.extend(true, {}, this._reportParameters);
            return reportparameters;
        },

        /**
        * Go to specific page index of the report.
        * @return jQuery
        * @example 
        * &lt;div id="reportviewer"&gt;ReportViewer&lt;/div&gt; <br> 
        * &lt;script&gt;
        * var reportviewerObj = $("#reportviewer").data("ejReportViewer");
        * reportviewerObj.gotoPageIndex(5); // To navigate the specific page
        * &lt;/script&gt;
        * @memberof ejReportViewer
        * @instance
        */
        gotoPageIndex: function (pageNo) {
            this._gotoPageNo(pageNo);
        },

        /**
        * Navigate to last page of the report.
        * @return jQuery
        * @example 
        * &lt;div id="reportviewer"&gt;ReportViewer&lt;/div&gt; <br> 
        * &lt;script&gt;
        * var reportviewerObj = $("#reportviewer").data("ejReportViewer");
        * reportviewerObj.gotoLastPage(); // Navigate to the last page
        * &lt;/script&gt;
        * @memberof ejReportViewer
        * @instance
        */
        gotoLastPage: function () {
            this._gotoLastPage();
        },

        /**
        * Navigate to first page of report.
        * @return jQuery
        * @example 
        * &lt;div id="reportviewer"&gt;ReportViewer&lt;/div&gt; <br> 
        * &lt;script&gt;
        * var reportviewerObj = $("#reportviewer").data("ejReportViewer");
        * reportviewerObj.gotoFirstPage(); // To navigate to first page
        * &lt;/script&gt;
        * @memberof ejReportViewer
        * @instance
        */
        gotoFirstPage: function () {
            this._gotoFirstPage();
        },

        /**
        * Navigate to next page from the current page.
        * @return jQuery
        * @example 
        * &lt;div id="reportviewer"&gt;ReportViewer&lt;/div&gt; <br> 
        * &lt;script&gt;
        * var reportviewerObj = $("#reportviewer").data("ejReportViewer");
        * reportviewerObj.gotoNextPage(); //To navigate to the next page
        * &lt;/script&gt;
        * @memberof ejReportViewer
        * @instance
        */
        gotoNextPage: function () {
            this._gotoNextPage();
        },

        /**
        * Navigate to previous page from the current page.
        * @return jQuery
        * @example 
        * &lt;div id="reportviewer"&gt;ReportViewer&lt;/div&gt; <br> 
        * &lt;script&gt;
        * var reportviewerObj = $("#reportviewer").data("ejReportViewer");
        * reportviewerObj.gotoPreviousPage(); // To navigate to the previous page
        * &lt;/script&gt;
        * @memberof ejReportViewer
        * @instance
        */
        gotoPreviousPage: function () {
            this._gotoPreviousPage();
        },

        /**
        * Print the report.
        * @return jQuery
        * @example 
        * &lt;div id="reportviewer"&gt;ReportViewer&lt;/div&gt; <br> 
        * &lt;script&gt;
        * var reportviewerObj = $("#reportviewer").data("ejReportViewer");
        * reportviewerObj.print(); //To perform print operation.
        * &lt;/script&gt;
        * @memberof ejReportViewer
        * @instance
        */
        print: function () {
            this._print();
        },
		
		/**
        * Apply print layout to the report.
        * @return jQuery
        * @example 
        * &lt;div id="reportviewer"&gt;ReportViewer&lt;/div&gt; <br> 
        * &lt;script&gt;
        * var reportviewerObj = $("#reportviewer").data("ejReportViewer");
        * reportviewerObj.printLayout(); //Changes between print layout and normal modes.
        * &lt;/script&gt;
        * @memberof ejReportViewer
        * @instance
        */
        printLayout: function () {
            this._printlayout();
        },
		
		/**
        * Export the report to the specified format.
        * @return jQuery
        * @example 
        * &lt;div id="reportviewer"&gt;ReportViewer&lt;/div&gt; <br> 
        * &lt;script&gt;
        * var reportviewerObj = $("#reportviewer").data("ejReportViewer");
        * reportviewerObj.exportReport(); //Exports the reports
        * &lt;/script&gt;
        * @memberof ejReportViewer
        * @instance
        */
        exportReport: function (format) {
            this._exportReport(format);
        },

        /**
        * Refresh the report.
        * @return jQuery
        * @example 
        * &lt;div id="reportviewer"&gt;ReportViewer&lt;/div&gt; <br> 
        * &lt;script&gt;
        * var reportviewerObj = $("#reportviewer").data("ejReportViewer");
        * reportviewerObj.refresh(); // To refresh the report.
        * &lt;/script&gt;
        * @memberof ejReportViewer
        * @instance
        */
        refresh: function () {
            this._refreshReport();
        },

        fitToPage: function(){
            this._fitToPage('fitToPage');
        },

        fitToPageWidth: function () {
            this._fitToPage('fitToWidth');
        },

        fitToPageHeight: function () {
            this._fitToPage('fitToHeight');
        },
        /*---------------------client side methods[end]----------------------------------------------------------*/

        /*---------------------client side Events[start]----------------------------------------------------------*/
        _onReportLoaded: function () {
            this._trigger("reportLoaded");
        },

        _onRenderingBegin: function () {
            this._trigger("renderingBegin");
        },

        _onRenderingComplete: function () {
            this._trigger("renderingComplete");
        },

        _onReportError: function (args) {
            return this._trigger("reportError", args);
        },

        _onDrillThrough: function (args) {
            return this._trigger("drillThrough", args);
        }
        /*---------------------client side Events[end]----------------------------------------------------------*/
    });

    /**
	* Enum for report export formats.	 
	* @enum {number}
	* @global 
	*/
    ej.ReportViewer.ExportOptions = {
        /**  Specifies Excel export format. */
        Excel: 1 << 0,
        /**  Specifies HTML export format. */
        Html: 1 << 1,
        /**  Specifies PDF export format. */
        Pdf: 1 << 2,
        /**  Specifies Word export format. */
        Word: 1 << 3,
        /**  Specifies all export formats(Word,Excel,PDF,HTML). */
        All: 1 << 0 | 1 << 1 | 1 << 2 | 1 << 3
    };

	/**
	* Enum for excel format.	 
	* @enum {string}
	* @global 
	*/
    ej.ReportViewer.ExcelFormats = {
        /**  Specifies Excel Version. */
        Excel97to2003: "excel97to2003" ,
        /**  Specifies Excel Version. */
        Excel2007: "excel2007",
        /**  Specifies Excel Version. */
        Excel2010: "excel2010",
        /**  Specifies Excel Version. */
        Excel2013: "excel2013"        
    };
	
	/**
	* Enum for word format.	 
	* @enum {string}
	* @global 
	*/
	ej.ReportViewer.WordFormats = {
        /**  Specifies Microsoft Word file format. */        
	    Doc: "doc",
		/**  Specifies Microsoft Word document template. */
	    Dot: "dot",
        /**  Specifies Microsoft Word 2007 file format. */
	    Docx: "docx",
		/**  Specifies Microsoft Word 2007 file format. */        
	    Word2007: "word2007",
        /**  Specifies Microsoft Word 2010 file format. */
	    Word2010: "word2010",
        /**  Specifies Microsoft Word 2013 file format. */
	    Word2013: "word2013",
       /**  Specifies Microsoft Word 2007 Template format. */
	    Word2007Dotx: "word2007dotx",
		/**  Specifies Microsoft Word 2010 Template format. */
	    Word2010Dotx: "word2010dotx",
        /**  Specifies Microsoft Word 2013 Template format. */
	    Word2013Dotx: "word2013dotx",
        /**  Specifies Microsoft Word 2007 macro enabled file format. */
	    Word2007Docm: "word2007docm",
        /**  Specifies Microsoft Word 2010 macro enabled file format. */        
	    Word2010Docm: "word2010docm",
        /**  Specifies Microsoft Word 2013macro enabled file format. */    
	    Word2013Docm: "word2013docm",
        /**  Specifies Microsoft Word 2007 macro enabled Template format. */    
	    Word2007Dotm: "word2007dotm",
        /**  Specifies Microsoft Word 2010 macro enabled Template format. */    
	    Word2010Dotm: "word2010dotm",
        /**  Specifies Microsoft Word 2010 macro enabled Template format. */    
	    Word2013Dotm: "word2013dotm",
        /**  Specifies Rtf format */  
	    Rtf: "rtf",
        /**  Specifies Text file format */         
	    Txt: "txt",
        /**  Specifies  E-book format */        
	    EPub: "epub",
        /**  Specifies Html format */        
	    Html: "html",
        /**  Specifies Xml format */        
	    Xml: "xml",
        /**  Specifies all format */        
	    Automatic: "automatic"
    };
	
    /**
	* Enum for report processing mode.	 
	* @enum {string}
	* @global 
	*/
    ej.ReportViewer.ProcessingMode = {
        /**  Specifies report processing mode as remote. */
        Remote: "remote",
        /**  Specifies report processing mode as local. */
        Local: "local"
    };

    /**
	* Enum for enable and disable toolbar items.	 
	* @enum {number}
	* @global 
	*/
    ej.ReportViewer.ToolbarItems = {
        /**  Enables export toolbar item. */
        Export: 1 << 0,
        /**  Enables zoom toolbar item. */
        Zoom: 1 << 1,
        /**  Enables page navigation toolbar item. */
        PageNavigation: 1 << 2,
        /**  Enables refresh toolbar item. */
        Refresh: 1 << 3,
        /**  Enables print toolbar item. */
        Print: 1 << 4,
        /** Enables document map toolbar item. */
        DocumentMap: 1 << 5,
        /**  Enables parameters toolbar item. */
        Parameters: 1 << 6,
        /**  Enables print layout toolbar item. */
        PrintLayout: 1 << 7,
        /**   Enables all the toolbar items. */
        All: 1 << 0 | 1 << 1 | 1 << 2 | 1 << 3 | 1 << 4 | 1 << 5 | 1 << 6 | 1 << 7
    };

    ej.ReportViewer.localization = {};

    ej.ReportViewer.localization["en-US"] = {
        toolbar: {
            print: {
                headerText: 'Print',
                contentText: 'Print the report.'
            },
            exportformat: {
                headerText: 'Export',
                contentText: 'Select the exported file format.'
            },
            first: {
                headerText: 'First',
                contentText: 'Go to the first page of the report.'
            },
            previous: {
                headerText: 'Previous',
                contentText: 'Go to the previous page of the report.'
            },
            next: {
                headerText: 'Next',
                contentText: 'Go to the next page of the report.'
            },
            last: {
                headerText: 'Last',
                contentText: 'Go to the last page of the report.'
            },
            documentMap: {
                headerText: 'Document Map',
                contentText: 'Show or hide the document map.'
            },
            parameter: {
                headerText: 'Parameter',
                contentText: 'Show or hide the parameters pane.'
            },
            zoomIn: {
                headerText: 'Zoom-In',
                contentText: 'Zoom in to the report.'
            },
            zoomOut: {
                headerText: 'Zoom-Out',
                contentText: 'Zoom out of the report.'
            },
            refresh: {
                headerText: 'Refresh',
                contentText: 'Refresh the report.'
            },
            printLayout: {
                headerText: 'Print Layout',
                contentText: 'Change between print layout and normal modes.'
            },
            pageIndex: {
                headerText: 'Page Number',
                contentText: 'Current page number to view.'
            },
            zoom: {
                headerText: 'Zoom',
                contentText: 'Zoom in or out on the report.'
            },
            back: {
                headerText: 'Back',
                contentText: 'Go back to the parent report.'
            }
        },
        viewButton: 'View Report'
    };
})(jQuery, Syncfusion);
;
(function ($, ej, undefined) {
    // ejGanttChart is the plugin name 
    // "ej.GanttChart" is "namespace.className" will hold functions and properties

    ej.widget("ejGanttChart", "ej.GanttChart", {
        // widget element will be automatically set in this
        element: null,

        // user defined model will be automatically set in this
        model: null,
        _rootCSS: "e-ganttchart",

        /* keyConfigurations for keyboard interaction in treegrid */
        keyConfigs: {
            focus: "e",

            //Insert
            insertRecord: "45",

            // delete
            deleteRecord: "46",

            //F2
            editRecord: "113",

            // Enter
            saveRequest: "13",

            //Esc   
            cancelRequest: "27",

            //Home,
            firstRowSelection: "36",

            //End,
            lastRowSelection: "35",

            //Up arrow
            upArrow: "38",

            //Down arrow
            downArrow: "40",

            //tab
            moveCellRight: "9",

            //shifttab
            moveCellLeft: "shift+9",

            //AltPlusDownArrow
            selectedRowExpand: "alt+40",

            //CtrlPlusDownArrow
            totalRowExpand: "ctrl+40",

            //AltPlusUpArrow
            selectedRowCollapse: "alt+38",

            //CtrlPlusUpArrow
            totalRowCollapse: "ctrl+38"
        },

        //default model
        defaults: {
            childPropertyName: "",
            taskIdMapping: "",
            taskNameMapping: "",
            startDateMapping: "",
            endDateMapping: "",
            childMapping: "",
            finishDateMapping: "",
            durationMapping: "",
            milestoneMapping: "",
            progressMapping: "",
            predecessorMapping: "",
            resourceInfoMapping: "",
            resourceNameMapping: "",
            resourceCollection: [], /*--------------Array containing the resource names */

            holidays: [
                {
                    day: null,
                    label: null,
                    background: null
                }
            ],

            stripLines: [
                {
                    day: null,
                    label: null,
                    lineStyle: "dotted",
                    lineColor: "#169dd7",
                    lineWidth: 2
                }
            ],

            enableTaskbarTooltip: true, /*------------showing tooltip on taskbar mouse over */
            enableTaskbarDragTooltip: true, /*--------showing taskbar while dragging taskbar */

            highlightWeekends: true, /*-------------Boolean value for highlighting weekends */

            scheduleStartDate: null, /*-------------start date of entire project */
            scheduleEndDate: null, /*---------------end date of entire project */

            enableProgressBarResizing: true, /*----------Boolean value for resizing progress bar */

            rowHeight: 30,
            progressbarHeight: 100,
            connectorlineWidth: 1,
            dateFormat: "MM/dd/yyyy",
            locale: "en-US",

            //background style
            taskbarBackground: "#DE8080", //"#5AC1E0",
            progressbarBackground: "#C44647", //"#169DD7",
            weekendBackground: '#F2F2F2',
            connectorLineBackground: "#383838",
            parentTaskbarBackground: "#383838",
            parentProgressbarBackground: "#1C1C1C",

            selectedItem: null,
            selectedRowIndex: -1,
            perDayWidth: null,

            perMonthWidth: null,//year-month schedule mode
            perWeekWidth: null,//month-week schedule mode
            perHourWidth:null,
            queryTaskbarInfo: null,

            taskbarTemplate: null,
            progressbarTemplate: null,
            parenttaskbarTemplate: null,
            parentprogressbarTemplate: null,

            //internal collections
            rows: [],
            ganttRecords: [],
            scheduleWeeks: [],

            scheduleYears: [],
            scheduleMonths: [], //new property for month schedule
            scheduleDays:[],
            projectStartDate: null, //new property instead of scheduleWeeks[0] and scheduleYears[0]
            projectEndDate: null,

            updatedGanttRecords: [],

            //tooltip templates
            tooltipTemplate: null,
            tooltipTemplateId:null,
            progressbarTooltipTemplateId: null,
            taskbarEditingTooltipTemplateId: null,

            taskbarEditingTooltipTemplate: null,
            progressbarTooltipTemplate: null,

            //boolean values
            showTaskNames: true,
            showProgressStatus: true,
            showResourceNames: true,
            enableRowHover: false,
            includeWeekend: false,
            allowZooming: false,
            allowGanttChartEditing: true,
            allowKeyboardNavigation: true,

            //baselines
            renderBaseline: false,
            baselineColor: '#fba41c ',

            /* editing API's */
            editSettings: {
                allowEditing: false,
                allowAdding: false,
                allowDeleting: false,
                editMode: "normal",
                dialogEditorTemplateId: null
            },

            scheduleHeaderSettings:
            {
                weekHeaderFormat: "MMM dd , yyyy",//for week - Day schedule mode
                dayHeaderFormat: "ddd",
                weekendBackground: '#F2F2F2',

                yearHeaderFormat: "yyyy",
                monthHeaderFormat: "MMM",

                hourHeaderFormat: "HH",

                scheduleHeaderType: "week"
            },
            workingTimeScale: "TimeScale8Hours",
            roundOffDayworkingTime: true,
            durationUnit: "day",

            localizedDays: null,
            localizedMonths: null,
            //localized string to be displayed in toolTip
            columnHeaderTexts: null,
            enablePredecessorEditing:true,
        },

        updateHighlightWeekends: function (boolValue) {
            var proxy = this;
            proxy.model.highlightWeekends = boolValue;

            if (proxy.model.highlightWeekends == true)
                proxy._renderWeekends();
            else
                proxy._$weekendContainer.remove();
        },

        updateWeekendBackground: function (bgcolor) {
            var proxy = this;
            proxy.model.weekendBackground = bgcolor;
            proxy._$weekendContainer.remove();
            proxy._renderWeekends();
        },

        showTooltip: function (boolValue) {
            var proxy = this;
            proxy.model.enableTaskbarTooltip = boolValue;
            proxy._createTaskbarTemplate();
            proxy._refresh();
        },

        showEditingTooltip: function (boolValue) {
            var proxy = this;
            proxy.model.enableTaskbarDragTooltip = boolValue;
            proxy._createTaskbarTemplate();
            proxy._refresh();
        },

        updateConnectorLineBackground: function (bgcolor) {
            var proxy = this;
            proxy.model.connectorLineBackground = bgcolor;
            proxy._createConnectorLineTemplate();
        },

        updateConnectorlineWidth: function (lineWidth) {
            var proxy = this;
            proxy.model.connectorlineWidth = parseInt(lineWidth);
            proxy._createConnectorLineTemplate();
        },


        updateTaskbarBackground: function (bgcolor) {
            var proxy = this;
            proxy.model.taskbarBackground = bgcolor;
            this._createTaskbarTemplate();
            this._refresh();
        },

        updateProgressbarBackground: function (bgcolor) {
            this.model.progressbarBackground = bgcolor;
            this._createTaskbarTemplate();
            this._refresh();
        },

        updateParentTaskbarBackground: function (bgcolor) {
            this.model.parentTaskbarBackground = bgcolor;
            this._createTaskbarTemplate();
            this._refresh();
        },

        updateParentProgressbarBackground: function (bgcolor) {
            this.model.parentProgressbarBackground = bgcolor;
            this._createTaskbarTemplate();
            this._refresh();
        },

        updateRenderTaskNames: function (bool) {
            var proxy = this;
            proxy.model.showTaskNames = bool;
            proxy._createTaskbarTemplate();
            proxy._refresh();
        },

        updateRendeProgressStatus: function (bool) {
            var proxy = this;
            proxy.model.showProgressStatus = bool;
            proxy._createTaskbarTemplate();
            proxy._refresh();
        },

        updateRenderResourceNames: function (bool) {
            var proxy = this;
            proxy.model.showResourceNames = bool;
            proxy._createTaskbarTemplate();
            proxy._refresh();
        },

        ganttChartEditing: function (boolValue) {
            var proxy = this;
            proxy.model.allowGanttChartEditing = boolValue;
        },

        resizeProgressbar: function (boolValue) {
            var proxy = this;
            proxy.model.enableProgressBarResizing = boolValue;

        },

        updateAllowKeyboardNavigation: function (bool) {
            this.model.allowKeyboardNavigation = bool;
        },

        updateProgressbarHeight: function (height) {
            var proxy = this;
            this.model.progressbarHeight = height;
            proxy._createTaskbarTemplate();
            proxy._refresh();
        },

        updateBaselineColor: function (color) {
            var proxy = this;
            this.model.baselineColor = color;
            proxy._createTaskbarTemplate();
            proxy._refresh();
        },

        updateRenderBaseline: function (bool) {
            var proxy = this;
            this.model.renderBaseline = bool;
            proxy._createTaskbarTemplate();
            proxy._refresh();
            proxy._createConnectorLineTemplate();
        },


        //returns the schedule year width
        _getYearWidth: function () {
            return (this.model.perMonthWidth * 12);
        },

        //returns the schedule month width
        _getMonthWidth: function () {
            var proxy = this,
                model = proxy.model,
                date, scheduleMonths = model.scheduleMonths,
                    monthWidth = 0;

            for (var i = 0; i < scheduleMonths.length; i++) {
                var startDate = scheduleMonths[i];

                if (typeof startDate === "object") {
                    date = new Date(startDate);
                }
                else {
                    date = Globalize.parseDate(startDate, "MM/dd/yyyy");
                }
                var monthLastDay = new Date(date.getFullYear(), date.getMonth() + 1, 0);

                var lastDate = monthLastDay.getDate();
                if (lastDate == 31)
                    monthWidth += (this.model.perDayWidth * 31);
                else if (lastDate == 30)
                    monthWidth += (this.model.perDayWidth * 30);
                else if (lastDate == 28)
                    monthWidth += (this.model.perDayWidth * 28);
            }
            return monthWidth;
        },



        //returns the schedule week width
        _getWeekWidth: function () {
            return (this.model.perDayWidth * 7);
        },

        _getNumberOftWeeks: function (startdate, enddate) {


            //var date1 = new Date(startdate), date2 = new Date(enddate);
            //// The number of milliseconds in one week
            //var ONE_WEEK = 1000 * 60 * 60 * 24 * 7;
            //// Convert both dates to milliseconds
            //var date1_ms = date1.getTime();
            //var date2_ms = date2.getTime();
            //// Calculate the difference in milliseconds
            //var difference_ms = Math.abs(date1_ms - date2_ms);
            //// Convert back to weeks and return hole weeks
            //return Math.floor(difference_ms / ONE_WEEK);
            var startDate = new Date(startdate),
                endDate = new Date(enddate);

            var totalWeeksCollection = [];
            do {
                totalWeeksCollection.push(new Date(startDate));
                startDate.setDate(startDate.getDate() + 7);
            } while (!(startDate >= endDate));

            return totalWeeksCollection;
        },

        //returns the ganttchart container height
        _getHeight: function () {
            return this._containerHeight;
        },

        //sets the height of the ganttchart container
        _setHeight: function (value) {
            this._containerHeight = value;
        },

        //returns the rows in the ganttchart
        getGanttChartRows: function () {
            return this._ganttChartRows;
        },

        //returns the gantt record collection
        getUpdatedRecords: function () {
            return this.model.ganttRecords;
        },

        //sets value for currentviewdata , ganttrecords and row collections
        setUpdatedRecords: function (currentViewData, updatedGanttRecords, rows,ids) {
            var proxy = this;
            proxy._currentViewData = currentViewData;
            proxy.model.updatedGanttRecords = updatedGanttRecords;
            proxy.model.ganttRecords = updatedGanttRecords;
            proxy.model.rows = rows;
            proxy.model._ids = ids;
        },

        //returns the string date type 
        _getFormatedDate: function (date, format) {
            return Globalize.format(date, format);
        },

        //returns the viewport height
        _getViewportHeight: function () {
            var proxy = this,
                headerdiv = proxy._$headerContainer,
                height;

            height = proxy.element.height() - headerdiv.height() -
                    parseFloat(headerdiv.css("border-bottom-width"));
            return height;
        },

        updateViewPortHeight:function(){
            var proxy = this;
            proxy._viewportHeight=proxy._getViewportHeight();
        },

        //constructor function
        _init: function () {

            var proxy = this, model = proxy.model,
            scheduleMode = model.scheduleHeaderSettings.scheduleHeaderType;

            proxy.element.addClass("e-ganttchart-core");
            proxy.element.attr("tabindex", "0");
            proxy._initPrivateProperties();
            proxy._initialize();
            proxy._renderGanttChart();
            proxy._$bodyContainer.append(proxy._$bodyContent);
            proxy._$ganttChartContainer.append(proxy._$bodyContainer);

            $(proxy._$bodyContainer).css({ "height": proxy._viewportHeight + "px" });
            if (proxy.model.enableVirtualization)
                $(proxy._$bodyContent).css({ "height": (model.rowHeight * model.ganttRecords.length) + "px" });

            if (scheduleMode == "week") {
                $(proxy._$bodyContent).css({ "width": (model.scheduleWeeks.length * proxy._scheduleWeekWidth) + "px" });
            }
            else if (scheduleMode == "year") {
                $(proxy._$bodyContent).css({ "width": (model.scheduleYears.length * proxy._scheduleYearWidth) + "px" });

            }
            else if (scheduleMode == "month") {
                $(proxy._$bodyContent).css({ "width": (proxy._scheduleMonthWidth) + "px" });
            }
            else if (scheduleMode == "day") {
                $(proxy._$bodyContent).css({ "width": (model.scheduleDays.length * (proxy.model.perHourWidth * 24)) + "px" });
            }

            proxy.createGridLinesTable();
            proxy.renderGanttRecords();
          //  Inner div for scroller
            var scrollerDiv = ej.buildTag('div');
            scrollerDiv.append(proxy._$bodyContent);
            proxy._$bodyContainer.append(scrollerDiv);
            proxy._$bodyContainer.ejScroller({
                width: proxy.element.width(),
                enableTouchScroll:false,
                height: proxy._getViewportHeight(), scroll: function (args) {

                    var arg = {},
                        scrollTop = args.scrollTop;

                    args.requestType = "scroll";
                    args.delta = scrollTop;
//                    proxy._completeAction(args);
                    if (scrollTop != undefined) {// && this.isVScroll()
                        proxy.onScrollHelper(scrollTop);
                    } else {
                        proxy._handleScroll(args);
                    }
                    
                }
            });
            proxy._$bodyContainer.ejScroller("model.keyConfigs", { up: "", down: "" });
            proxy._$bodyContainer.ejScroller("refresh");
            //check if Hscroller is render or not
            var isHorizontalScroll = proxy._$bodyContainer.ejScroller("isHScroll");
                if (!isHorizontalScroll) {
                    proxy._$bodyContainer.addClass("e-borderbox");
                }
                else {
                    proxy._$bodyContainer.removeClass("e-borderbox");
                }
            proxy._wireEvents();


        },

        //rendering gantt chart again after zoomingvalue changed
        reRenderGanttChart: function (daywidth, ganttRecords) {

            var proxy = this,
                   scheduleMode = model.scheduleHeaderSettings.scheduleHeaderType;

            proxy._wireEvents();
            proxy.model.perDayWidth = daywidth;

            // proxy.model.perMonthWidth = monthwidth;

            proxy.model.ganttRecords = ganttRecords;
            proxy._scheduleWeekWidth = proxy._getWeekWidth();
            proxy._scheduleYearWidth = proxy._getYearWidth();
            proxy._scheduleMonthWidth = proxy._getMonthWidth();

            $(proxy._$secondaryCanvas).empty();
            $(proxy._$headerContainer).empty();
            $(proxy._$scheduleDiv).empty();
            proxy._renderGanttChart();
            proxy._$bodyContainer.append(proxy._$bodyContent);
            proxy._$ganttChartContainer.append(proxy._$bodyContainer);

            if (scheduleMode == "week") {
                $(proxy._$bodyContent).css({ "width": (proxy.model.scheduleWeeks.length * proxy._scheduleWeekWidth) + "px" });
            }
            else if (scheduleMode == "year") {
                $(proxy._$bodyContent).css({ "width": (proxy.model.scheduleYears.length * proxy._scheduleYearWidth) + "px" });
            }
            else if (scheduleMode == "month") {
                $(proxy._$bodyContent).css({ "width": (proxy._scheduleMonthWidth) + "px" });
            } 
            else if (scheduleMode == "day") {
                $(proxy._$bodyContent).css({ "width": (proxy.model.scheduleDays.length * (proxy.model.perHourWidth * 24)) + "px" });
            }
        },

        //declaring private properties
        _initPrivateProperties: function () {
            var proxy = this;
            proxy._viewportHeight = 0,
                proxy._prevScrollTop = 0,
                proxy._scrollTop = 0,
                proxy._vScrollDir = 1,
                proxy._$ganttChartContainer = null,
                proxy._$headerContainer = null,
                proxy._$bodyContainer = null,
                proxy._$bodyContent = null,
                proxy._$dependencyViewContainer = null,
                proxy._$secondaryCanvas = null,
                proxy._$ganttViewTable = null,
                proxy._$ganttViewTablebody = null,
                proxy._weekdays = [],
                proxy._rowMargin = 0,
                proxy._renderedRange = null,
                proxy._visibleRange = null,
                proxy._vTop = null,
                proxy._vBottom = null,
                proxy._rTop = null,
                proxy._rBottom = null,
                proxy._vscrollDist = 0,
                proxy._containerHeight = 0,
                proxy._currentViewData = [],
                proxy._id = this.element.attr("id"),
                proxy._ganttChartRows = null,
                proxy._filteredRecords = [],
                proxy._sortedRecords = [],
                proxy._tempSortedRecords = [],
                proxy._scheduleHours = [],

                proxy._scheduleWeekWidth = proxy._getWeekWidth(),

                proxy._numOfWeeks = proxy._getNumberOftWeeks(this.model.projectStartDate, this.model.projectEndDate),
                proxy._scheduleYearWidth = proxy._getYearWidth(),
                proxy._scheduleMonthWidth = proxy._getMonthWidth(),
                proxy._totalWeekWidth = proxy._scheduleWeekWidth * proxy._numOfWeeks.length,
            proxy._balanceWidth = proxy._totalWeekWidth - proxy._scheduleMonthWidth;
            proxy._scheduleMonthWidth += proxy._balanceWidth;
            proxy._totalMonthWidth = 0;

            proxy._tempFilteredRecords = [],
            proxy._prevLeft = 0,
            proxy._allowDragging = false,
            proxy._leftResizing = false,
            proxy._rightResizing = false,
            proxy._progressResizing = false,
            proxy._editingContainer = null,
            proxy._allowExpandCollapse = true,
            proxy._leftResizer = null,
            proxy._taskbarItem = null,
            proxy._progressbarItem = null,
            proxy._progressResizer = null,
            proxy._rightResizer = null,
            proxy._tasknameContainer = null,
            proxy._mousePosX = 0,
            proxy._currMousePosX = 0,
            proxy._currMousePosY = 0,
            proxy._prevItem = {},
            proxy._mouseTarget = null,
            proxy._mouseDown = false,
            proxy._$scheduleDiv = null,
            proxy._leftResizerGripper = null,
            proxy._rightResizerGripper = null,
            proxy._progressHandle = null,
            proxy._progressHandleChild = null,
            proxy._mouseOverTooltip = document.getElementById(this.model.tooltipTemplate),
            proxy._progressBarTooltipID = document.getElementById(this.model.progressbarTooltipTemplateId),
            proxy._taskbarEditingTooltipID = document.getElementById(this.model.taskbarEditingTooltipTemplateId),
            proxy._ids = null,
            proxy._scrollBarHeight = 18,
            proxy._windowWidth = $(window).width(),
            proxy._windowHeight = $(window).height(),
            proxy._editingItem = null,
            proxy._editingTarget = null,
            proxy.tooltipState = null,
            proxy._visibleRecordsCount = 0;
            proxy._$gridLinesTablebody = null;

            //Properties for connector line mouse
            proxy._leftConnectorPoint = null;
            proxy._rightConnectorPoint = null;
            proxy._childLeftConnectorPoint = null;
            proxy._childRightConnectorPoint = null;
            proxy._editPredecessor = false;
            proxy._connectorPointX = null;
            proxy._connectorPointY = null;
            proxy._predecessorTooltip = null;
            proxy._totalCollapsedRecord = 0;
        },

        //Initialize ganttchart
        _initialize: function () {
            var proxy = this;
            proxy._$ganttChartContainer = ej.buildTag("div.e-ganttviewercontainer#ganttviewercontainer" + proxy._id, "", { width: "100%" }, {});
            proxy._$headerContainer = ej.buildTag("div.e-ganttviewerheaderContainer#ganttviewerheaderContainer" + proxy._id, "", {}, {});
            proxy._$bodyContainer = ej.buildTag("div.e-ganttviewerbodyContianer#ganttviewerbodyContianer" + proxy._id, "", {
                '-ms-touch-action': 'none',
                'touch-action': 'none',
            }, {});
            proxy._$bodyContent = ej.buildTag("div.e-ganttviewerbodyContent#ganttviewerbodyContent" + proxy._id, "", {
            }, {});

            proxy._$dependencyViewContainer = ej.buildTag("div.e-ganttviewerbodyContianer-dependencyViewContainer#dependencyViewContainer" + proxy._id,
                "", {
                    'z-index': '4',
                    'position': 'absolute'
                }, {});
            proxy._$secondaryCanvas = ej.buildTag("div.e-ganttviewerbodyContianer-secondaryCanvas", "", {
                'z-index': '3',
                'position': 'absolute'
            }, {});
            proxy._$weekendsContainer = ej.buildTag("div.e-ganttviewerbodyContianer-weekendsContainer", "", {
                'z-index': '1',
                'position': 'absolute'
            }, {});

            proxy._$ganttViewTable = ej.buildTag("table.e-ganttviewerbodyContianer-ganttViewTable#ganttViewTable" + proxy._id, "", {
                'z-index': '5',
                'position': 'absolute'
            }, { 'cellspacing': '0px' });
            proxy._$scheduleDiv = ej.buildTag("div.e-gantt-schedule-container#gantt-schedule" + proxy._id + "", "", {}, {});
        },

        _getScheduleLength: function (scheduleMode) {
            var proxy = this,
                model = proxy.model;

            switch (scheduleMode) {
                case "week":                    
                    return (model.scheduleWeeks.length * proxy._scheduleWeekWidth);
                    break;
                case "year":                    
                    return (model.scheduleYears.length * proxy._scheduleYearWidth);
                    break;
                case "month":
                    return (proxy._scheduleMonthWidth);
                    break;
                case "day":
                    return (model.scheduleDays.length * (proxy.model.perHourWidth * 24));
                    break;
                default:
                    return (model.scheduleWeeks.length * proxy._scheduleWeekWidth);

            }
        },

        //Rendering GanttChart
        _renderGanttChart: function () {
            var proxy = this, model = proxy.model,
                scheduleLength, $colsgroup, $column,
            scheduleMode = model.scheduleHeaderSettings.scheduleHeaderType;
            
        proxy._createSchedule(proxy._getDateType(model.scheduleStartDate),
            proxy._getDateType(model.scheduleEndDate));

            proxy._viewportHeight = proxy._getViewportHeight();
                       
            proxy._$ganttViewTable.css("width", proxy._getScheduleLength(scheduleMode) + 'px');
            
            $colsgroup = ej.buildTag("colgroup", "", {}, {});
            $column = $(document.createElement("col"));
            $column.css("width", proxy._getScheduleLength(scheduleMode) + 'px');
            
            $colsgroup.append($column);
            proxy._$ganttViewTable.append($colsgroup);

            proxy._createTooltipTemplate();
            proxy._createEditingTooltipTemplate();
            proxy._createProgressbarTooltipTemplate();

            proxy._createTaskbarTemplate();
            proxy._createConnectorLineTemplate();

            proxy._updateCurrentViewData();

            //Display weekends
            if (proxy.model.highlightWeekends == true
                && model.scheduleHeaderSettings.scheduleHeaderType == ej.Gantt.ScheduleHeaderType.Week)
                proxy._renderWeekends();

            //Display striplines
            if (proxy.model.stripLines != null) {
                this._renderStripLines(proxy.model.stripLines);
            }

            //Display Holidays
            if (proxy.model.holidays != null) {
                var holidayCount = proxy.model.holidays.length;
                for (var i = 0; i < holidayCount; i++) {
                    proxy._renderHoliday(proxy.model.holidays[i]);
                }
            }
        },

        //Rendering connectorlines from Gantt
        renderConnectorLines: function (connectorLinesCollection) {
            var proxy = this,
                connectorLineContainer = $.render[proxy._id + "ConnectorLineTemplate"](connectorLinesCollection);

            $(proxy._$dependencyViewContainer).empty();
            proxy._$dependencyViewContainer.append(connectorLineContainer);
            proxy._$bodyContent.append(proxy._$dependencyViewContainer);
        },

        //Empty the entire connectorline collection
        clearConnectorLines: function () {
            $(this._$dependencyViewContainer).empty();
        },

        //append single connectorline
        appendConnectorLine: function (connectorLineCollection) {
            var proxy = this,
            connectorLineContainer = $.render[proxy._id + "ConnectorLineTemplate"](connectorLineCollection);
            proxy._$dependencyViewContainer.append(connectorLineContainer);
        },
        //set Collapsed record Count

        setCollapsedRecordCount:function(count)
        {
            this._totalCollapsedRecord = count;
        },
        //get collapsed record count
        getCollapsedRecordCount:function()
        {
            return this._totalCollapsedRecord;
        },
        //remove a particular connector line
        removeConnectorline: function (lineId) {
            $("#ConnectorLine" + lineId).remove();
        },

        //Events
        _wireEvents: function () {
            var proxy = this,
                matched = jQuery.uaMatch(navigator.userAgent);

            //$(proxy._$bodyContainer).bind("scroll", $.proxy(proxy._handleScroll, proxy));

            proxy._on(proxy.element, "mouseenter", ".e-gantt-milestone-container,.e-gantt-childtaskbar,.e-gantt-childtaskbar-progress,.e-gantt-parenttaskbar-innerdiv,.e-gantt-parenttaskbar-progress", this._mouseHover);//.gantt-viewer-bodyContianer tr td
            proxy._on(proxy.element, "mouseleave", ".e-gantt-milestone-container,.e-gantt-childtaskbar,.e-gantt-childtaskbar-progress,.e-gantt-parenttaskbar-innerdiv,.e-gantt-parenttaskbar-progress", this._mouseLeave);//.gantt-viewer-bodyContianer tr td
            proxy._on(proxy.element, "click", proxy._cellClickHandler);

            proxy._on(proxy.element, "mouseenter", ".taskbarresizer-left", proxy._resizerleftOver);
            proxy._on(proxy.element, "mouseleave", ".taskbarresizer-left", proxy._resizerleftLeave);

            proxy._on(proxy.element, "mouseenter", ".taskbarresizer-right", proxy._resizerrightOver);
            proxy._on(proxy.element, "mouseleave", ".taskbarresizer-right", proxy._resizerrightLeave);

            proxy._on(proxy.element, "mouseenter", ".progressbarresizer-right", proxy._progressresizerOver);
            proxy._on(proxy.element, "mouseleave", ".progressbarresizer-right", proxy._progressresizerLeave);

            proxy._on(proxy.element, "mouseenter", ".e-connectorpoint-left", proxy._dragLeftOver);
            proxy._on(proxy.element, "mouseleave", ".e-connectorpoint-left", proxy._dragLeftLeave);

            proxy._on(proxy.element, "mouseenter", ".e-connectorpoint-right", proxy._dragRightOver);
            proxy._on(proxy.element, "mouseleave", ".e-connectorpoint-right", proxy._dragRightLeave);
          

            if (matched.browser.toLowerCase() != "msie") {
                proxy._on(proxy.element, "mousedown", ".e-tasklabel,.e-parentContainer,.e-childContainer,.e-gantt-milestone-container", proxy._mousedown);
            }
            else {
                proxy._on(proxy.element, "mousedown", ".e-tasklabel,.e-parentContainer,.e-childContainer,.e-gantt-milestone-container", proxy._mousedown);
            }

            $(proxy._$bodyContainer).bind("mouseup", $.proxy(proxy._handleMouseUp, proxy));
            $(document).bind("mouseup", $.proxy(proxy._tooltipMouseup, proxy));

            //if (matched.browser.toLowerCase() == "msie") {
            //    proxy._on(proxy.element, "MSPointerDown", ".e-tasklabel,.e-parentContainer,.e-childContainer,.e-gantt-milestone-container", proxy._mousedown);
            //    $(document.body).bind("MSPointerMove", function (event) {
            //        event.preventDefault();
            //    });
            //    $(document.body).bind("MSPointerMove", $.proxy(proxy.handleMouseMove, proxy));
            //    $(proxy._$bodyContainer).bind("MSPointerUp", $.proxy(proxy._handleMouseUp, proxy));
            //}


            if (matched.browser.toLowerCase() == "chrome") {
                proxy._on(proxy.element, "touchstart", ".e-tasklabel,.e-parentContainer,.e-childContainer,.e-gantt-milestone-container", proxy._mousedown);
                $(document.body).bind("touchmove", function (event) {
                    event.preventDefault();
                });
                $(document.body).bind("touchmove", $.proxy(proxy.handleMouseMove, proxy));

                proxy._on(proxy.element, "touchend", ".e-tasklabel,.e-parentContainer,.e-childContainer,.e-gantt-milestone-container", proxy._handleMouseUp);
                proxy._on(proxy.element, "touchleave", ".e-tasklabel,.e-parentContainer,.e-childContainer,.e-gantt-milestone-container", proxy._handleMouseUp);
                $(proxy._$bodyContainer).bind("touchend", $.proxy(proxy._handleMouseUp, proxy));
                $(proxy._$bodyContainer).bind("touchleave", $.proxy(proxy._handleMouseUp, proxy));
            }

            proxy._on(proxy.element, "mouseenter", ".e-stripline", proxy._stripLineMouseOver);
            proxy._on(proxy.element, "mouseleave", ".e-stripline", proxy._stripLineMouseLeave);
            proxy._enableEditingEvents();
        },

        //Event append for editDialogOpen while perform double click action
        _enableEditingEvents: function () {

            var proxy = this,
                model = proxy.model;

            if (model.editSettings.allowEditing) {
                if (model.editSettings.editMode === "normal") {
                    proxy._on(this.element, "dblclick", ".e-ganttviewerbodyContent", proxy._editdblClickHandler);
                }
            }
        },

        //Double click event handler for editing with editMode as normal
        _editdblClickHandler: function () {

            var proxy = this,
                args = {};

            if (proxy.model.editSettings.editMode === "normal") {
                args.requestType = ej.TreeGrid.Actions.BeginEdit;
                proxy._trigger("actionBegin", args);
            }

        },


        //Event to append RowHover effect
        _enableRowHover: function () {
            var proxy = this;
            if (proxy.model.enableRowHover)
                proxy._on(proxy.element, "mouseleave mouseenter",
                    "#ganttviewerbodyContent,.e-ganttviewerbodyContent tr,.e-ganttviewerbodyContent" + proxy._id,
                    proxy._rowHover);
            else
                proxy._on(proxy.element, "mouseleave mouseenter",
                    "#ganttviewerbodyContent,.e-ganttviewerbodyContent tr,.e-ganttviewerbodyContent" + proxy._id);
        },

        //MouseLeave event for striplines to remove tooltip
        _stripLineMouseLeave: function () {
            $(this._mouseOverTooltip).remove();
        },

        //mouseHover event for stripline to render tooltip
        _stripLineMouseOver: function (e) {

            e.target.style.cursor = "pointer";
            var proxy = this, model = proxy.model,
                $target = $(e.target),
                parentNode, match,
                tooltiptable,
                tooltipbody,
                posx = 0, posy = 0;

            if ($target.hasClass("e-striplinespan")) {
                parentNode = $target[0].parentNode;
            } else
                parentNode = $target[0];

            match = parentNode.id.match(/(\d+|[A-z]+)/g);

            if (e.type == "mouseenter" || e.type == "mousemove") {
                if (!e) e = window.event;
                if (e.originalEvent.pageX || e.originalEvent.pageY) {
                    posx = e.originalEvent.pageX;
                    posy = e.originalEvent.pageY;
                }
                else if (e.originalEvent.clientX || e.originalEvent.clientY) {
                    posx = e.originalEvent.clientX + document.body.scrollLeft
                        + document.documentElement.scrollLeft;
                    posy = e.originalEvent.clientY + document.body.scrollTop
                        + document.documentElement.scrollTop;
                }

                tooltiptable = ej.buildTag("table.e-tooltiptable", "", { "width": "auto" },
                    { 'cellspacing': '2px', 'cellpadding': '2px' });
                tooltipbody = ej.buildTag("tbody",
                    "<tr><td>" + model.stripLines[match[1]].day + "</td></tr><tr><td>" +
                        model.stripLines[match[1]].label + "</td></tr>", {}, {});

                tooltiptable.append(tooltipbody);
                if (model.tooltipTemplate == null) {
                    proxy._mouseOverTooltip = ej.buildTag("div.e-tooltipgantt#tooltipgantt" + proxy._id + "", tooltiptable,
                        { "top": (posy) + "px", "left": (posx + 15) + "px", 'position': 'absolute', 'z-index': '6' }, {});
                } else {
                    if (typeof (model.tooltipTemplate) != "object") {
                        var parser = $.parseHTML(model.tooltipTemplate);
                        proxy._mouseOverTooltip = parser[0];
                    }
                    else {
                        var markup = $(model.tooltipTemplate)[0].innerHTML;
                        parser = $.parseHTML(markup);
                        proxy._mouseOverTooltip = parser[1];
                    }

                    $(proxy._mouseOverTooltip).css({
                        "left": (posx + 15) + "px", "top": (posy) + "px",
                        "position": "absolute"
                    });
                    $(proxy._mouseOverTooltip).addClass("customTooltip");
                    proxy._mouseOverTooltip.innerHTML = "<table>" + tooltiptable[0].innerHTML + "</table>";
                }
                $(document.body).append(proxy._mouseOverTooltip);
            }
        },

        //Mouseup over tooltip 
        _tooltipMouseup: function () {
            var proxy = this;
            if (proxy._mouseTarget != null) {
                proxy._handleMouseUp(proxy._mouseTarget);
            }
        },

        //removes all the tooltip in ganttchart
        _clearTooltip: function () {
            var proxy = this;
            $(proxy._mouseOverTooltip).remove();
            $(proxy._taskbarEditingTooltipID).remove();
            $(proxy._progressBarTooltipID).remove();
            $('.e-editingtooltip').remove();
            $(".e-progressbartooltip").remove();
        },

        //mousewheel action for zooming ganttchart
        _mouseWheel: function (e) {
            if (this.model.allowZooming) {
                var proxy = this, args = {},
                    delta = null;

                e = e.originalEvent;
                e.preventDefault();

                if (e.wheelDelta)
                    delta = -e.wheelDelta / 120;
                else if (e.detail)
                    delta = e.detail / 3;

                if (!delta)
                    return;
                args.requestType = "scroll";
                args.delta = delta;
                proxy._enbleZooming(args);
            }
        },


        //mouseHover event on taskbar right helper element
        _resizerrightOver: function (e) {

            var proxy = this,
                $target = e.target,
                div = $target.parentNode;

            if (proxy.model.allowGanttChartEditing == true) {
                if (this._mouseDown == false)
                    e.target.style.cursor = "w-resize";
                proxy._setEditingElements(div);
            }
        },


        //mouseLeave event for taskbar right helper element
        _resizerrightLeave: function () {

            var proxy = this;
            if (proxy.model.allowGanttChartEditing == true) {
                proxy._removeEditingElements();
            }
        },


        //mouseHover event on taskbar left helper element
        _resizerleftOver: function (e) {

            var proxy = this,
                $target = e.target,
                div = $target.parentNode;
            if (proxy.model.allowGanttChartEditing == true) {
                if (this._mouseDown == false)
                    e.target.style.cursor = "e-resize";
                proxy._setEditingElements(div);
            }
        },


        //mouseLeave event for taskbar left helper element
        _resizerleftLeave: function () {
            var proxy = this;
            if (proxy.model.allowGanttChartEditing == true) {
                proxy._removeEditingElements();
            }
        },


        //mouseHover event on taskbar progressbar helper element
        _progressresizerOver: function (e) {
            var proxy = this,
                $target = e.target,
                div = $target.parentNode;
            if (proxy.model.allowGanttChartEditing == true) {
                if (proxy._mouseDown == false)
                    $target.style.cursor = "col-resize";
                proxy._setEditingElements(div);
            }
        },


        //mouseLeave event for progressbar helper element
        _progressresizerLeave: function () {
            var proxy = this;
            if (proxy.model.allowGanttChartEditing == true) {
                proxy._removeEditingElements();
            }
        },


        _secondMouseOverConnectorPoint: function (e, mouseUp)
        {
            var proxy = this, row, recordIndexr, item;

            if ((proxy._mouseDown && proxy._falseLine) || (mouseUp && proxy._falseLine)) {

                var $target = $(e.target),
                  $ganttGridRows = proxy.getGanttChartRows(),
                  row = $target.closest('tr'),
                  recordIndexr = $ganttGridRows.index(row),
                  item = proxy._currentViewData[recordIndexr];

                var fromItem = proxy._editingItem;
                var toItem = item, predecessor, fullPredecessor, currentTarget;

                if (proxy._prevItem.predecessorTarget === 'Start') {
                    predecessor = fromItem.taskId + 'S';
                } else if (proxy._prevItem.predecessorTarget === 'Finish') {
                    predecessor = fromItem.taskId + 'F';
                }

                if ($target.hasClass('e-connectorpoint-left')) {
                    predecessor += 'S';
                    currentTarget = "Start"
                }
                else if ($target.hasClass('e-connectorpoint-right')) {
                    predecessor += 'F';
                    currentTarget = "Finish"
                }

                //Append second item in predecessor tooltip

                $table = proxy._predecessorTooltip.find("table");
                $td = $table.find("tbody").find("tr:eq(1)").find("td:eq(2)").text(toItem.taskName);
                $td = $table.find("tbody").find("tr:eq(1)").find("td:eq(3)").text(currentTarget);
                var args = {};
                args.requestType = "validatePredecessor";
                args.fromItem = fromItem;
                args.toItem = toItem;

                if (args.toItem.item[proxy.model.predecessorMapping]) {
                    fullPredecessor = args.toItem.item[proxy.model.predecessorMapping] + "," + predecessor;
                }
                else {
                    fullPredecessor = predecessor;
                }
                args.predecessor = predecessor;
                args.data = {};
                args.data.taskID = toItem.taskId;
                args.predecessorString = [];
                args.predecessorString.push(fullPredecessor);
                var validation = proxy._completeAction(args);
                $td = $table.find("tbody").find("tr:eq(0)").find("td:eq(0)");
                if (validation) {
                   // $td.text("True");
                    $td.removeClass();
                    $td.addClass("e-predecessor-true");
                } else {
                  //  $td.text("False");
                    $td.removeClass();
                    $td.addClass("e-predecessor-false");
                }

                //Here also update on moueup//same code is required
                if (mouseUp) {
                    return args;
                }
                
            }
        },
        //mouse enter event for left-connectorpoint-point

        _dragLeftOver:function(e)
        {
            var proxy = this,
              $target = e.target,
              div = $target.parentNode;
            if (proxy.model.predecessorMapping && proxy.model.allowGanttChartEditing && proxy.model.enablePredecessorEditing) {
                if (this._mouseDown == false ) {
                    $target.style.cursor = "pointer";
                }
                proxy._setEditingElements(div);
            }
            if(!$(div).hasClass('e-parentContainer'))
                proxy._secondMouseOverConnectorPoint(e);

        },
        //mouse leave event for left-connectorpoint-point
        _dragLeftLeave: function (e) {

            var proxy = this;
            if (proxy.model.allowGanttChartEditing == true && proxy.model.enablePredecessorEditing) {
                proxy._removeEditingElements();
            }
        },

        //mouse enter event for right-connectorpoint-point

        _dragRightOver: function (e) {
            var proxy = this,
             $target = e.target,
             div = $target.parentNode;
            if (proxy.model.predecessorMapping && proxy.model.allowGanttChartEditing && proxy.model.enablePredecessorEditing) {
                if (this._mouseDown == false) {
                    $target.style.cursor = "pointer";
                }
                proxy._setEditingElements(div);
            }
            if (!$(div).hasClass('e-parentContainer'))
                proxy._secondMouseOverConnectorPoint(e);
        },
        //mouse leave event for right-connectorpoint-point
        _dragRightLeave: function (e) {
            var proxy = this;
            if (proxy.model.allowGanttChartEditing == true && proxy.model.enablePredecessorEditing) {
                proxy._removeEditingElements();
            }
        },




        //adding styles for taskbar helper elements
        _setEditingElements: function (div) {
            var proxy = this, progressHandler;

            if ($(div).hasClass("e-childContainer")) {
                proxy._leftResizerGripper = div.childNodes[1],//[0]
                proxy._rightResizerGripper = div.childNodes[5];//[4]
                progressHandler = div.childNodes[4];//[3]
                proxy._progressHandle = progressHandler.firstChild;
                proxy._progressHandleChild = progressHandler.childNodes[1];

                $(proxy._leftResizerGripper).addClass("gripper");
                $(proxy._rightResizerGripper).addClass("gripper");

                if (proxy.model.progressMapping) {
                    $(proxy._progressHandle).addClass("e-progresshandle");
                    $(proxy._progressHandleChild).addClass("e-progresshandleafter");
                }
            }


            if (proxy.model.enablePredecessorEditing) {
                if ($(div).hasClass("e-childContainer") && proxy.model.predecessorMapping && !proxy._falseLine) {

                    proxy._leftConnectorPoint = div.childNodes[0];
                    proxy._rightConnectorPoint = div.childNodes[6];
                    $(proxy._leftConnectorPoint).addClass("e-connectorpoint-hover");
                    $(proxy._rightConnectorPoint).addClass("e-connectorpoint-hover");
                }
                else if ($(div).hasClass("e-gantt-milestone-container") && proxy.model.predecessorMapping && !proxy._falseLine) {

                    proxy._leftConnectorPoint = div.childNodes[0];
                    proxy._rightConnectorPoint = div.childNodes[2];
                    $(proxy._leftConnectorPoint).addClass("e-connectorpoint-hover");
                    $(proxy._rightConnectorPoint).addClass("e-connectorpoint-hover");
                }
                else if ($(div).hasClass("e-childContainer") && proxy.model.predecessorMapping && proxy._falseLine) {
                    proxy._childLeftConnectorPoint = div.childNodes[0];
                    proxy._childRightConnectorPoint = div.childNodes[6];
                    $(proxy._childLeftConnectorPoint).addClass("e-connectorpoint-hover");
                    $(proxy._childRightConnectorPoint).addClass("e-connectorpoint-hover");
                }
                else if ($(div).hasClass("e-gantt-milestone-container") && proxy.model.predecessorMapping && proxy._falseLine) {
                    proxy._childLeftConnectorPoint = div.childNodes[0];
                    proxy._childRightConnectorPoint = div.childNodes[2];
                    $(proxy._childLeftConnectorPoint).addClass("e-connectorpoint-hover");
                    $(proxy._childRightConnectorPoint).addClass("e-connectorpoint-hover");
                }
                else if ($(div).hasClass("e-parentContainer") && proxy.model.predecessorMapping && !proxy._falseLine) {

                    proxy._leftConnectorPoint = div.childNodes[0];
                    proxy._rightConnectorPoint = div.childNodes[3];
                    $(proxy._leftConnectorPoint).addClass("e-connectorpoint-hover");
                    $(proxy._rightConnectorPoint).addClass("e-connectorpoint-hover");

                }
            }
        },


        //removing styles for taskbar helper elements
        _removeEditingElements: function () {
            var proxy = this;
            $(proxy._leftResizerGripper).removeClass("gripper");
            $(proxy._rightResizerGripper).removeClass("gripper");

            if (proxy.model.progressMapping) {
                $(proxy._progressHandle).removeClass("e-progresshandle");
                $(proxy._progressHandleChild).removeClass("e-progresshandleafter");
            }


            //Remove Connector points in chart
            if (proxy.model.enablePredecessorEditing) {
                if (proxy.model.predecessorMapping && proxy._mouseDown == false) {

                    $(proxy._leftConnectorPoint).removeClass('e-connectorpoint-hover');
                    $(proxy._rightConnectorPoint).removeClass('e-connectorpoint-hover');
                    proxy._leftConnectorPoint = null;
                    proxy._rightConnectorPoint = null;
                }

                if (proxy.model.predecessorMapping && proxy._leftConnectorPoint !== null && proxy._mouseDown == true || proxy._childLeftConnectorPoint) {
                    $(proxy._childLeftConnectorPoint).removeClass("e-connectorpoint-hover");
                    $(proxy._childRightConnectorPoint).removeClass("e-connectorpoint-hover");
                    //remove curret target predecessor in tooltip    
                    if (proxy._predecessorTooltip) {
                        $table = proxy._predecessorTooltip.find("table");
                        $table.find("tbody").find("tr:eq(1)").find("td:eq(2)").text("");
                        $table.find("tbody").find("tr:eq(1)").find("td:eq(3)").text("");
                        $td = $table.find("tbody").find("tr:eq(0)").find("td:eq(0)").removeClass();
                        //$td.text("False");
                        $td.addClass('e-predecessor-false');
                    }
                }
            }
        },


        refreshHeight:function(){

            var proxy = this;
            proxy._$bodyContainer.ejScroller("refresh");
        },

        //mousedown event for ganttchart
        _mousedown: function (e) {

            var proxy = this;
            proxy._clearTooltip();
            proxy._mouseDown = true;
            args = {};
            //proxy._clearContextMenu();
            var $target = $(e.target),
            $ganttGridRows = proxy.getGanttChartRows(),
            row = $target.closest('tr'),
            recordIndexr = $ganttGridRows.index(row),
            item = proxy._currentViewData[recordIndexr],
            div = $target[0].parentNode,
            matched = jQuery.uaMatch(navigator.userAgent);

            if (ej.isNullOrUndefined(item)) {
                return;
            }
            proxy._editingItem = item;
            proxy._editingTarget = $target;

            if ($(div).hasClass("e-gantt-childtaskbar-progress") ||
                $(div).hasClass("progressbarresizer-right")) {
                div = div.parentNode;
            }

            if ($(div).hasClass("e-childContainer") ||
                ($(div).hasClass("e-gantt-milestone")) || $(div).hasClass('e-parentContainer') || $(div).hasClass("e-gantt-milestone-container")) {

                $(row).addClass("e-editmode");
                proxy._mouseTarget = e;
            }

            if (proxy.model.allowGanttChartEditing && ($(div).hasClass("e-childContainer")
                || ($(div).hasClass("e-gantt-milestone")) || $(div).hasClass("e-gantt-milestone-container") || $(div).hasClass('e-parentContainer'))) { //mouse down on drag points in child and milestone container and parent container also included 

                if (e.target.style.cursor == "e-resize" && ($(div).hasClass("e-childContainer")
                    || ($(div).hasClass("e-gantt-milestone")))
                    || (e.type == "touchstart" && e.target.className == "taskbarresizer-left" && matched.browser.toLowerCase() == "chrome")) {
                    proxy._updateEditingType();
                    proxy._leftResizing = true;
                }

                if (e.target.style.cursor == "w-resize" && ($(div).hasClass("e-childContainer")
                    || ($(div).hasClass("e-gantt-milestone")))
                    || (e.type == "touchstart" && e.target.className == "taskbarresizer-right" && matched.browser.toLowerCase() == "chrome")) {
                    proxy._updateEditingType();
                    proxy._rightResizing = true;

                }
                if (e.target.style.cursor == "col-resize" && ($(div).hasClass("e-childContainer")
                    || ($(div).hasClass("e-gantt-milestone")))
                    || (e.type == "touchstart" && e.target.className == "progressbarresizer-right" && matched.browser.toLowerCase() == "chrome")) {
                    proxy._updateEditingType();
                    proxy._progressResizing = true;
                }
                if (e.target.style.cursor == "move" && ($(div).hasClass("e-childContainer")
                    || ($(div).hasClass("e-gantt-milestone")))) {
                    proxy._updateEditingType();
                    proxy._allowDragging = true;
                }
                if (e.target.style.cursor == "" && ($(div).hasClass("e-childContainer")
                  || ($(div).hasClass("e-gantt-milestone")))) {
                    proxy._updateEditingType();
                    proxy._allowDragging = true;
                }

                // ON drag points initiate the false line and edit type for predecessor

                if (proxy.model.enablePredecessorEditing) {
                    if (e.target.style.cursor == "pointer" && ($(div).hasClass("e-childContainer")
                     || ($(div).hasClass("e-gantt-milestone")) || $(div).hasClass("e-gantt-milestone-container") || $(div).hasClass('e-parentContainer'))) {
                        proxy._updateEditingType();
                        proxy._editPredecessor = true;
                        if ($target.hasClass('e-connectorpoint-left')) {
                            proxy._prevItem.predecessorTarget = 'Start';
                        }
                        else if ($target.hasClass('e-connectorpoint-right')) {
                            proxy._prevItem.predecessorTarget = 'Finish';
                        }
                    }
                }




                if (!e) e = window.event;
                if (e.originalEvent.pageX || e.originalEvent.pageY) {
                    proxy._currMousePosX = e.originalEvent.pageX;
                    proxy._currMousePosY = e.originalEvent.pageY;
                    proxy.dragPosX = e.originalEvent.pageX;
                }
                else if (e.originalEvent.clientX || e.originalEvent.clientY) {
                    proxy.dragPosX = e.originalEvent.clientX + document.body.scrollLeft
                        + document.documentElement.scrollLeft;
                }
                else if (e.originalEvent && e.originalEvent.changedTouches
                    && e.originalEvent.changedTouches.length > 0) {
                    proxy._currMousePosX = e.originalEvent.changedTouches[0].pageX;
                    proxy._currMousePosY = e.originalEvent.changedTouches[0].pageY;
                    proxy.dragPosX = e.originalEvent.changedTouches[0].pageX;
                }

                proxy._prevItem.left = item.left;
                proxy._prevItem.status = item.status;
                proxy._prevItem.duration = item.duration;
                proxy._prevItem.startDate = item.startDate;
                proxy._prevItem.endDate = item.endDate;
                proxy._prevItem.width = item.width;

                //For false line starting point
                if (proxy.model.enablePredecessorEditing) {
                    proxy._connectorPointX = proxy._currMousePosX - proxy._$bodyContent.offset().left;
                    proxy._connectorPointY = proxy._currMousePosY - proxy._$bodyContent.offset().top;
                }


                $(proxy._$bodyContainer).bind("mousemove", $.proxy(proxy.handleMouseMove, proxy));

                if (matched.browser.toLowerCase() == "chrome") {

                    proxy._on(proxy.element, "touchend", ".e-tasklabel,.e-parentContainer,.e-childContainer,.e-gantt-milestone-container", proxy._handleMouseUp);
                    proxy._on(proxy.element, "touchleave", ".e-tasklabel,.e-parentContainer,.e-childContainer,.e-gantt-milestone-container", proxy._handleMouseUp);
                    $(proxy._$bodyContainer).bind("touchend", $.proxy(proxy._handleMouseUp, proxy));
                    $(proxy._$bodyContainer).bind("touchleave", $.proxy(proxy._handleMouseUp, proxy));
                    $(document.body).bind("touchend", $.proxy(proxy._handleMouseUp, proxy));
                    $(document.body).bind("touchleave", $.proxy(proxy._handleMouseUp, proxy));
                }
            }

            if (($(div).hasClass("e-parentContainer")) && !$target.hasClass("e-connectorpoint-left") && !$target.hasClass("e-connectorpoint-right")) { // here check drag point class
                if (proxy._allowExpandCollapse && item && item.hasChildRecords) {
                        if (proxy.model.selectedRowIndex !== args.recordIndex) {
                            args.target = "ejGanttChart";
                            args.data = item;
                            proxy.model.selectedRowIndex = proxy.model.updatedGanttRecords.indexOf(args.data);
                            proxy.selectRows(proxy.model.selectedRowIndex);
                            args.recordIndex = proxy.model.selectedRowIndex;
                            proxy._trigger("rowSelected", args);
                            if (proxy.prevSelectedItem != null) {
                                proxy.prevSelectedItem.isSelected = false;
                                proxy.model.updatedGanttRecords[proxy.model.selectedRowIndex].isSelected = true;
                            }
                            proxy.prevSelectedItem = args.data;
                        }
                    proxy._expandCollapse(recordIndexr);
                }
            } else {
                proxy._appendTooltip($target, recordIndexr);
            }
        },


        handleMouseMove: function (e) {
            this._updateOnmouseMove(e);

        },


        //Handler for MouseUp event in ganttchart
        _handleMouseUp: function (e) {

            var proxy = this,
                args = {},
                $ganttGridRows = proxy.getGanttChartRows(),
                $target, row, recordIndexr = 0, item;

            
            //Calculate and validate predecessor and draw connectorline after second connector point connects
            if (($(e.target).hasClass('e-connectorpoint-left') || $(e.target).hasClass('e-connectorpoint-right')) && proxy._falseLine && !$(e.target).parent().hasClass('e-parentContainer')) {
                var newArgs = proxy._secondMouseOverConnectorPoint(e, true);
                if (newArgs.predecessorValidation) {
                    newArgs.requestType = "drawConnectorLine";
                    proxy._completeAction(newArgs);
                }
            }

            if (proxy._editingTarget != null && proxy._editingItem != null) {
                $target = proxy._editingTarget,
                row = $target.closest('tr'),
                recordIndexr = $ganttGridRows.index(row),
                item = proxy._editingItem;
                if ($(row).hasClass("e-editmode")) {
                    $(row).removeClass("e-editmode");
                    proxy._mouseTarget = null;
                }
            }


            proxy._editingTarget = null;
            proxy._editingItem = null;

            proxy._clearTooltip();
            proxy._$ganttChartContainer[0].style.cursor = "auto";

            proxy._allowExpandCollapse = true;

            if (item != null) {
                proxy.updateConnectorLineOffset(item);
            }

            if (item != null && (proxy._allowDragging || proxy._leftResizing || proxy._rightResizing || proxy._progressResizing)) {
                args.editingFields = { startdate: item.startDate, enddate: item.endDate, progress: item.status, duration: item.duration };
                args.data = item;
                args.previousData = proxy._prevItem;
                args.dragging = proxy._allowDragging;
                args.leftResizing = proxy._leftResizing;
                args.rightResizing = proxy._rightResizing;
                args.progressResizing = proxy._progressResizing;
                proxy._taskbarEdited(args);
            }

            if (args.cancel) {
                proxy._getPrevItem(item, recordIndexr);
            }

            $("#ganttviewerbodyContianer" + proxy._id).unbind("mousemove");
            proxy._updateEditingType();
            proxy._mouseDown = false;

            //Remove drag point and false lines on mouse up
            proxy._falseLine && proxy._falseLine.remove();
            proxy._falseLine = null;
            proxy._predecessorTooltip && proxy._predecessorTooltip.remove();
            proxy._predecessorTooltip = null;
            $(proxy._leftConnectorPoint).removeClass("e-connectorpoint-hover");
            $(proxy._rightConnectorPoint).removeClass("e-connectorpoint-hover");
            $(proxy._childRightConnectorPoint).removeClass("e-connectorpoint-hover");
            $(proxy._childLeftConnectorPoint).removeClass("e-connectorpoint-hover");
            proxy._leftConnectorPoint = null;
            proxy._rightConnectorPoint = null;
            proxy._childRightConnectorPoint = null;
            proxy._childLeftConnectorPoint = null;
        },

        getChildCount: function (record, count) {
            var currentRecord, proxy = this;
            if (!record.hasChildRecords)
                return 0;
            for (var i = 0; i < record.childRecords.length; i++) {
                currentRecord = record.childRecords[i];
                count++;
                if (currentRecord.hasChildRecords) {
                    count = proxy.getChildCount(currentRecord, count);
                }
            }
            return count;
        },

        //get the Expand status of GridRecord
        _getCollapsedParentItem: function (record) {
            var parentRecord = record.parentItem;
            if (parentRecord) {
                if (parentRecord.expanded === false) {
                    return parentRecord;
                } else {
                    return this._getCollapsedParentItem(parentRecord)
                }
            }
            else
            {
                return null;
            }
        },
        //keyPressed event for perform the keynavigate selection in gantt
        _keyPressed: function (action, target, arg, e) {

            var proxy = this,
                $ganttChartEle = proxy.element,
                ganttChartObject = $ganttChartEle.ejGanttChart("instance"),
                model = ganttChartObject.model;

            e.stopImmediatePropagation();
            e.preventDefault();

            if (!model.allowKeyboardNavigation)
                return true;

            switch (action) {

                case "downArrow":
                    if (!proxy._rowSelectingEventTrigger()) {
                        //get the last row index of the ganttchart control
                        var lastRowIndex = proxy.model.updatedGanttRecords.length - 1;

                        if (proxy.model.selectedRowIndex != lastRowIndex) {

                            if(!model.enableVirtualization && !model.selectedItem.expanded)
                            {
                                var child = proxy.getChildCount(model.selectedItem, 0);
                                var index=proxy.model.selectedRowIndex + child+1;
                                if (lastRowIndex >= index)
                                    proxy.selectRows(proxy.model.selectedRowIndex + child + 1);
                            }
                            else
                            {
                                proxy.selectRows(proxy.model.selectedRowIndex + 1);
                            }
                            model.selectedItem.isSelected = true;
                            var recordIndex = model.selectedItem &&
                                proxy.model.updatedGanttRecords.indexOf(model.selectedItem),
                                rowTop = lastRowIndex !== -1 && recordIndex * model.rowHeight,
                                scrollTop,
                                containerWidth,
                                scrollerExsist = proxy._$bodyContainer.children(".e-content").length;
                                if (scrollerExsist)
                                    scrollTop = proxy._$bodyContainer.children(".e-content").scrollTop();
                                else
                                    scrollTop = proxy._$bodyContainer.scrollTop();

                                containerWidth = scrollTop + proxy._viewportHeight - model.rowHeight-proxy._scrollBarHeight + (proxy.getCollapsedRecordCount() * model.rowHeight);
                            if (containerWidth < rowTop) {
                                proxy._$bodyContainer.ejScroller("scrollY", scrollTop + model.rowHeight, true);
                            }

                            if (model.enableVirtualization) {
                                proxy.element[0].focus();
                                proxy.selectRows(proxy.model.selectedRowIndex);
                            }
                            proxy._rowSelectedEventTrigger(proxy.model.selectedRowIndex); //for diasable completeaction
                        }
                    }
                    break;

                    //select the previous row by press down arrow key
                case "upArrow":
                    if (!proxy._rowSelectingEventTrigger()) {
                        if (proxy.model.selectedRowIndex !== 0) {
                            var previousItem = proxy.model.updatedGanttRecords[proxy.model.selectedRowIndex - 1],
                                parentItem,
                                rowClassName,length,
                                scrollTop,
                                containerWidth,
                                scrollerExsist = proxy._$bodyContainer.children(".e-content").length;
                            if (!model.enableVirtualization && previousItem.parentItem)
                            {
                                parentItem = proxy._getCollapsedParentItem(previousItem);
                                if (parentItem)
                                {
                                    length = proxy.getChildCount(parentItem, 0);
                                    proxy.selectRows(proxy.model.selectedRowIndex - length - 1);
                                }
                                else
                                {
                                    proxy.selectRows(proxy.model.selectedRowIndex - 1);
                                }
                            }
                            else
                            {
                                proxy.selectRows(proxy.model.selectedRowIndex - 1);
                            }
                            recordIndex = proxy.model.updatedGanttRecords.indexOf(model.selectedItem);
                            rowTop = recordIndex !== -1 && (recordIndex * model.rowHeight) - (proxy.getCollapsedRecordCount() * model.rowHeight);
                            if (scrollerExsist)
                                scrollTop = proxy._$bodyContainer.children(".e-content").scrollTop();
                            else
                                scrollTop = proxy._$bodyContainer.scrollTop();
                            if (scrollTop) {
                                if (scrollTop > rowTop) {
                                    if (rowTop >= 0)
                                        proxy._$bodyContainer.ejScroller("scrollY", rowTop, true);
                                    else
                                        proxy._$bodyContainer.ejScroller("scrollY", 0, true);
                                    if (model.enableVirtualization) {
                                        proxy.selectRows(recordIndex);
                                        proxy.element[0].focus();
                                    }
                                }
                            }
                            proxy._rowSelectedEventTrigger(proxy.model.selectedRowIndex);
                        }
                    }

                    break;

                    //select the first row while press the Home key
                case "firstRowSelection":
                    if (!proxy._rowSelectingEventTrigger()) {
                        proxy.selectRows(0);
                        recordIndex = model.updatedGanttRecords.indexOf(model.selectedItem);
                        rowTop = recordIndex !== -1 && recordIndex * model.rowHeight;
                        var currentPos = proxy._$ganttViewTable.height();
                        if (currentPos > (proxy._viewportHeight - proxy._scrollBarHeight)) {
                            args = {
                                requestType: "scroll",
                                delta: rowTop
                            };
                            proxy._$bodyContainer.ejScroller("scrollY", rowTop, true);
                        }
                            if (model.enableVirtualization) {
                                proxy.element[0].focus();
                                proxy.selectRows(0);
                            }
                        proxy._rowSelectedEventTrigger(0);
                    }
                    break;

                case "lastRowSelection":

                    if (!proxy._rowSelectedEventTrigger()) {
                        lastRowIndex = model.updatedGanttRecords.length - 1;
                        var previousItem = model.updatedGanttRecords[lastRowIndex];
                        if (!model.enableVirtualization && previousItem.parentItem) {
                            parentItem = proxy._getCollapsedParentItem(previousItem);
                            if (parentItem) {
                                length = proxy.getChildCount(parentItem, 0);
                                proxy.selectRows(lastRowIndex - length);
                            }
                            else {
                                proxy.selectRows(lastRowIndex);
                            }
                        }
                        else {
                            proxy.selectRows(lastRowIndex);
                        }
                        recordIndex = model.selectedItem && model.updatedGanttRecords.indexOf(model.selectedItem);
                        rowTop = lastRowIndex !== -1 && recordIndex * model.rowHeight;

                        if (rowTop + proxy.model.rowHeight >= proxy._viewportHeight) {

                            scrollTop = rowTop + proxy.model.rowHeight - (proxy._viewportHeight - proxy._scrollBarHeight) - (proxy.getCollapsedRecordCount() * model.rowHeight);
                            if (scrollTop >= 0) {
                                args = {
                                    requestType: "scroll",
                                    delta: scrollTop
                                };
                                proxy._$bodyContainer.ejScroller("scrollY", scrollTop, true);
                            }
                            if (model.enableVirtualization) {
                                proxy.element[0].focus();
                                proxy.selectRows(recordIndex);
                            }
                        }
                        proxy._rowSelectedEventTrigger(recordIndex);
                    }

                    break;
            }

            return true;
        },

        _rowSelectingEventTrigger: function () {
            var proxy = this,
                model = proxy.model,
                args = {},
                selectedItem,
                $gridRows;

            if (model.selectedRowIndex !== -1) {

                $gridRows = proxy.getGanttChartRows();
                selectedItem = proxy.model.updatedGanttRecords[model.selectedRowIndex];
                model.selectedItem = selectedItem;
                args = {
                    rowElement: $gridRows.eq(model.selectedRowIndex),
                    data: selectedItem,
                    recordIndex: model.selectedRowIndex
                };

            }

            return proxy._trigger('rowSelecting', args);
        },

        _rowSelectedEventTrigger: function (index) {
            var proxy = this,
                args,
                $gridRows = proxy.getGanttChartRows();
            args = {
                rowElement: $gridRows.eq(index),
                data: proxy.model.selectedItem,
                target: "ejGanttChart",
                recordIndex: index
            };
            proxy._trigger("rowSelected", args);
        },

        _updateEditingType: function () {
            var proxy = this;
            proxy._leftResizing = false;
            proxy._allowDragging = false;
            proxy._rightResizing = false;
            proxy._progressResizing = false;
            proxy._editPredecessor = false;
        },


        //update offset values for predecessor
        updateConnectorLineOffset: function (item) {

            var parentGanttRecord,
                model = this.model,
                predecessor = item.predecessor,
                predecessorValue = [],
                parentStartDate,
                parentEndDate,
                weekendOffset, childGanttRecord, dayOffset;

            if (predecessor) {
                for (var i = 0; i < predecessor.length; i++) {
                    if ((predecessor[i].to === item['taskId'].toString())) {

                        parentGanttRecord = model.rows[model._ids.indexOf(predecessor[i].from)];
                        childGanttRecord = model.rows[model._ids.indexOf(predecessor[i].to)];
                        parentStartDate = new Date(parentGanttRecord.startDate);
                        parentEndDate = new Date(parentGanttRecord.endDate);

                        if (predecessor[i].predecessorsType == "FS") {

                            if (childGanttRecord.left < (parentGanttRecord.left + parentGanttRecord.width)) {

                                dayOffset = ((parentGanttRecord.left + parentGanttRecord.width)
                                    - childGanttRecord.left) / this.model.perDayWidth;

                                dayOffset = childGanttRecord.isMilestone == true ? (dayOffset - 1) : dayOffset;
                                predecessor[i].offset = Math.round(-dayOffset);
                            }
                            else {
                                dayOffset = (childGanttRecord.left - (parentGanttRecord.left + parentGanttRecord.width))
                                    / this.model.perDayWidth;

                                if (dayOffset >= 0)
                                    predecessor[i].offset = Math.round(dayOffset);
                            }

                            if (model.includeWeekend === false) {
                                if (predecessor[i].offset > 0) {
                                    var weekEndDays = this._calculateWeekendDays(parentEndDate, new Date(childGanttRecord.startDate))
                                    predecessor[i].offset -= weekEndDays;
                                } else if (predecessor[i].offset < 0) {
                                    var weekEndDays = this._calculateWeekendDays(new Date(childGanttRecord.startDate), parentEndDate)
                                    predecessor[i].offset += weekEndDays;

                                }
                            }

                        }
                        if (predecessor[i].predecessorsType == "SS") {
                            if (childGanttRecord.left < parentGanttRecord.left) {
                                dayOffset = ((parentGanttRecord.left) - childGanttRecord.left) / this.model.perDayWidth;
                                predecessor[i].offset = Math.round(-dayOffset);
                            }
                            else if (childGanttRecord.left > parentGanttRecord.left) {
                                dayOffset = (childGanttRecord.left - (parentGanttRecord.left)) / this.model.perDayWidth;
                                predecessor[i].offset = Math.round(dayOffset);
                            }

                            if (model.includeWeekend === false) {
                                if (predecessor[i].offset > 0) {
                                    var weekEndDays = this._calculateWeekendDays(parentStartDate, new Date(childGanttRecord.startDate))
                                    predecessor[i].offset -= weekEndDays;
                                } else if (predecessor[i].offset < 0) {
                                    var weekEndDays = this._calculateWeekendDays(new Date(childGanttRecord.startDate), parentStartDate)
                                    predecessor[i].offset += weekEndDays;
                                }
                            }

                        }
                        if (predecessor[i].predecessorsType == "SF") {

                            if (parentStartDate.getDay() == 5) {
                                weekendOffset = 2 * this.model.perDayWidth;
                            }
                            if ((childGanttRecord.left + childGanttRecord.width) < parentGanttRecord.left) {

                                dayOffset = ((parentGanttRecord.left) - (childGanttRecord.left + childGanttRecord.width))
                                    / this.model.perDayWidth;
                                if (weekendOffset && this.model.includeWeekend == false)
                                    dayOffset = dayOffset - weekendOffset;
                                if (dayOffset > 0)
                                    predecessor[i].offset = Math.round(-dayOffset);
                            } else if ((childGanttRecord.left + childGanttRecord.width) > parentGanttRecord.left) {

                                dayOffset = ((childGanttRecord.left + childGanttRecord.width) -
                                    (parentGanttRecord.left)) / this.model.perDayWidth;
                                if (weekendOffset && this.model.includeWeekend == false)
                                    dayOffset = dayOffset - weekendOffset;
                                if (dayOffset > 0)
                                    predecessor[i].offset = Math.round(dayOffset);
                            }
                            if (model.includeWeekend === false) {
                                if (predecessor[i].offset > 0) {
                                    var weekEndDays = this._calculateWeekendDays(parentStartDate, new Date(childGanttRecord.startDate))
                                    predecessor[i].offset -= weekEndDays;
                                } else if (predecessor[i].offset < 0) {
                                    var weekEndDays = this._calculateWeekendDays(new Date(childGanttRecord.startDate), parentStartDate)
                                    predecessor[i].offset += weekEndDays;
                                }
                            }
                        }

                        if (predecessor[i].predecessorsType == "FF") {

                            if ((childGanttRecord.left + childGanttRecord.width) < (parentGanttRecord.left + parentGanttRecord.width)) {
                                dayOffset = ((parentGanttRecord.left + parentGanttRecord.width) -
                                    (childGanttRecord.left + childGanttRecord.width)) / this.model.perDayWidth;
                                predecessor[i].offset = Math.round(-dayOffset);
                            }
                            else if ((childGanttRecord.left + childGanttRecord.width) > (parentGanttRecord.left + parentGanttRecord.width)) {
                                dayOffset = ((childGanttRecord.left + childGanttRecord.width) -
                                    (parentGanttRecord.left + parentGanttRecord.width)) / this.model.perDayWidth;
                                predecessor[i].offset = Math.round(dayOffset);
                            }
                            if (model.includeWeekend === false) {
                                if (predecessor[i].offset > 0) {
                                    var weekEndDays = this._calculateWeekendDays(parentEndDate,new Date(childGanttRecord.endDate))
                                    predecessor[i].offset -= weekEndDays;
                                } else if (predecessor[i].offset < 0) {
                                    var weekEndDays = this._calculateWeekendDays(new Date(childGanttRecord.endDate), parentEndDate)
                                    predecessor[i].offset += weekEndDays;
                                }
                            }
                        } 
                        

                        if (predecessor[i].offset < 0) {
                            predecessorValue.push(predecessor[i].from + "" + predecessor[i].predecessorsType + ""
                                + predecessor[i].offset + "d");
                        }
                        else if (predecessor[i].offset >= 1) {
                            predecessorValue.push(predecessor[i].from + "" + predecessor[i].predecessorsType + "+"
                                + predecessor[i].offset + "d");
                        }
                        else {
                            predecessorValue.push(predecessor[i].from + "" + predecessor[i].predecessorsType);
                        }
                    }
                }
                if (predecessorValue.length > 0) {
                    item.item[model.predecessorMapping] = predecessorValue.join(',');
                    //this.refreshRow(itemIndex, false);
                }
            }
        },
        _calculateWeekendDays: function (fromDate, toDate)
        {
                var weekendDayCount = 0;

                while(fromDate < toDate){
                    fromDate.setDate(fromDate.getDate() + 1);
                    if(fromDate.getDay() === 0 || fromDate.getDay() == 6){
                        ++weekendDayCount ;
                    }
                }
                return weekendDayCount ;
        },

        //Returns the previous item properties
        _getPrevItem: function (item, index) {
            var proxy = this;
            item.left = proxy._prevItem.left;
            item.width = proxy._prevItem.width;
            item.startDate = proxy._prevItem.startDate;
            item.endDate = proxy._prevItem.endDate;
            item.duration = proxy._prevItem.duration;
            item.status = proxy._prevItem.status;
            item.progressWidth = this._getProgressWidth(item.width, item.status);
            this.refreshRow(index);
        },

        //handler for left resizing taskbar item
        _enableLeftResizing: function (e, item) {

            var proxy = this;
            e.target.style.cursor = "e-resize";
            $(proxy._leftResizer).addClass("gripper");
            $(proxy._rightResizer).addClass("gripper");

            if (proxy.dragPosX > proxy._mousePosX) {
                proxy._mousePosX = proxy.dragPosX - proxy._mousePosX;
                if (item.left > 0) {
                    if ((item.left + 30) < (item.left + item.width)) {
                        item.left -= proxy._mousePosX;
                    }
                    item.width += proxy._mousePosX;
                }
            }
            else {
                proxy._mousePosX -= proxy.dragPosX;
                if ((item.left + 30) < (item.left + item.width)) {
                    item.left += proxy._mousePosX;
                }
                item.width -= proxy._mousePosX;
            }
            if (e.originalEvent.pageX == 0) {
                this.dragPosX = e.originalEvent.targetTouches[0].pageX;
            }
            else {
                this.dragPosX = e.originalEvent.pageX;
            }
            if (model.scheduleHeaderSettings.scheduleHeaderType == ej.Gantt.ScheduleHeaderType.Day)
                item.width = item.width < proxy.model.perHourWidth ? proxy.model.perHourWidth : item.width;
            else
                item.width = item.width < proxy.model.perDayWidth ? proxy.model.perDayWidth : item.width;
            item.progressWidth = proxy._getProgressWidth(item.width, item.status);

            proxy._setItemPosition(item);

            if (proxy._taskbarItem && proxy._taskbarItem.childNodes.length > 0) {
                proxy._taskbarItem.removeChild(proxy._taskbarItem.lastChild);
            }
        },

        //handler for progressbar resizing
        _enableProgressResizing: function (e, item) {
            var proxy = this,
                diff,
                radius;
            e.target.style.cursor = "col-resize";
            $(proxy._leftResizer).addClass("gripper");
            $(proxy._rightResizer).addClass("gripper");

            if (proxy._progressResizer) {
                proxy._progressHandle = proxy._progressResizer.firstChild;
                proxy._progressHandleChild = proxy._progressResizer.childNodes[1];
            }

            if (proxy.dragPosX > proxy._mousePosX) {
                proxy._mousePosX = proxy.dragPosX - proxy._mousePosX;
                if (item.left > 0) {
                    item.progressWidth -= proxy._mousePosX;
                }
            }
            else {
                proxy._mousePosX -= proxy.dragPosX;
                item.progressWidth += proxy._mousePosX;
            }
            if (e.originalEvent.pageX == 0) {
                this.dragPosX = e.originalEvent.targetTouches[0].pageX;
            }
            else {
                this.dragPosX = e.originalEvent.pageX;
            }
            item.progressWidth = item.progressWidth > item.width ? item.width : item.progressWidth;
            item.progressWidth = item.progressWidth < 0 ? 0 : item.progressWidth;

            diff = item.width - item.progressWidth;

            if (diff <= 4) {
                radius = 4 - diff;
            }
            else
                radius = 0;

            $(proxy._progressbarItem).css({
                "width": (item.progressWidth) + "px",
                "left": (item.left) + "px",
                "border-top-right-radius": radius + "px",
                "border-bottom-right-radius": radius + "px"
            });
            $(proxy._progressResizer).css({ "left": (item.left + item.progressWidth - 10) + "px" });

            $(proxy._progressHandle).addClass("e-progresshandle");
            $(proxy._progressHandleChild).addClass("e-progresshandleafter");
        },


        //handler for right resizing taskbar
        _enableRightResizing: function (e, item) {

            var proxy = this;
            e.target.style.cursor = "w-resize";

            $(proxy._leftResizer).addClass("gripper");
            $(proxy._rightResizer).addClass("gripper");

            if (proxy.dragPosX > proxy._mousePosX) {
                proxy._mousePosX = proxy.dragPosX - proxy._mousePosX;
                if (item.left > 0) {
                    item.width -= proxy._mousePosX;
                }
            } else {
                proxy._mousePosX -= proxy.dragPosX;
                item.width += proxy._mousePosX;
            }
            if (e.originalEvent.pageX == 0) {
                this.dragPosX = e.originalEvent.targetTouches[0].pageX;
            }
            else {
                this.dragPosX = e.originalEvent.pageX;
            }

            if (model.scheduleHeaderSettings.scheduleHeaderType == ej.Gantt.ScheduleHeaderType.Day)
                item.width = item.width < proxy.model.perHourWidth ? proxy.model.perHourWidth : item.width;
            else
                item.width = item.width < proxy.model.perDayWidth ? proxy.model.perDayWidth : item.width;

            item.progressWidth = proxy._getProgressWidth(item.width, item.status);

            $(proxy._taskbarItem).css({ "left": (item.left) + "px", "width": (item.width) + "px" });
            $(proxy._progressbarItem).css({ "width": (item.progressWidth) + "px", "left": (item.left) + "px" });
            $(proxy._rightResizer).css({ "left": (item.left + item.width - 12) + "px" });
            $(proxy._progressResizer).css({ "left": (item.left + item.progressWidth - 10) + "px" });
            $(proxy._tasknameContainer).css({ "left": (item.left) + "px", "width": (item.width) + "px" });

            $(proxy._leftConnectorPoint).css({ "left": (item.left - 10) + "px" });
            $(proxy._rightConnectorPoint).css({ "left": (item.left + item.width) + "px" });

            if (proxy._taskbarItem && proxy._taskbarItem.childNodes.length > 0) {
                proxy._taskbarItem.removeChild(proxy._taskbarItem.lastChild);
            }
        },


        //Handler for dragging taskbar
        _enableDragging: function (e, item) {
            var proxy = this;
            e.target.style.cursor = "move";

            if ($(e.target.parentNode).hasClass("e-parentContainer")) {
                proxy._allowExpandCollapse = false;
            }

            if (this.dragPosX > proxy._mousePosX) {
                proxy._mousePosX = proxy.dragPosX - proxy._mousePosX;
                if (item.left > 0) {
                    item.left -= proxy._mousePosX;
                }
            } else {
                proxy._mousePosX -= proxy.dragPosX;
                item.left += proxy._mousePosX;
            }

            if (e.originalEvent.pageX == 0) {
                this.dragPosX = e.originalEvent.targetTouches[0].pageX;
            }
            else {
                this.dragPosX = e.originalEvent.pageX;
            }
            item.left = item.left < 0 ? 0 : item.left;

            $(proxy._progressbarItem).css({ "left": (item.left) + "px" });
            $(proxy._taskbarItem).css({ "left": (item.left) + "px" });
            $(proxy._rightResizer).css({ "left": (item.left + item.width - 12) + "px" });
            $(proxy._leftResizer).css({ "left": (item.left + 2) + "px" });
            $(proxy._progressResizer).css({ "left": (item.left + item.progressWidth - 10) + "px" });
            $(proxy._tasknameContainer).css({
                "left": (item.left) + "px",
                "width": (item.width) + "px"
            });

            $(proxy._leftConnectorPoint).css({ "left": (item.left-10) + "px" });
            $(proxy._rightConnectorPoint).css({ "left": (item.left + item.width) + "px" });

            if (proxy._taskbarItem && proxy._taskbarItem.childNodes.length > 0) {
                proxy._taskbarItem.removeChild(proxy._taskbarItem.lastChild);
            }
        },


        //Append tooltip while editing taskbar
        _appendTooltip: function (target, index) {

            var proxy = this, tooltipElement, tooltiptable, tooltipbody,
                $target = $(target),
                item = proxy._currentViewData[index],
                columnHeaderTexts = proxy.model.columnHeaderTexts;

            if (item != null) {
                if ((proxy._currMousePosX + 150) > proxy._windowWidth) {
                    proxy._currMousePosX -= (proxy._currMousePosX + 150) - proxy._windowWidth;
                }

                var div = $target[0];

                if ($(div).hasClass("e-parentContainer") || $(div).hasClass("e-childContainer")) {
                    proxy._editingContainer = div;
                    proxy._leftResizer = proxy._editingContainer.childNodes[0];
                    proxy._taskbarItem = proxy._editingContainer.childNodes[1];
                    proxy._progressbarItem = proxy._editingContainer.childNodes[2];
                    proxy._progressResizer = proxy._editingContainer.childNodes[3];
                    proxy._rightResizer = proxy._editingContainer.childNodes[4];
                }

                //Don't need to display the drag tooltip in Parent Taskbar
                if (this.model.enableTaskbarDragTooltip && (!$target.hasClass('e-parentContainer'))) {

                    if (this._progressResizing == true) {

                        proxy._$ganttChartContainer[0].style.cursor = "col-resize";

                        tooltiptable = ej.buildTag("table.e-tooltiptable", "", {}, { 'cellspacing': '2px', 'cellpadding': '2px' });
                        tooltipbody = ej.buildTag("tbody", "", {}, {});
                        tooltipbody[0].innerHTML = "<tr><td>" + columnHeaderTexts["status"] + " : " + item.status + "%</td></tr>";
                        tooltiptable.append(tooltipbody);
                        if (proxy.model.progressbarTooltipTemplate) {
                            proxy._progressBarTooltipID = ej.buildTag("div.e-progressbartooltip#progressbartooltip" + proxy._id + "", "",
                              {
                                  "top": (proxy._currMousePosY - 95) + "px",
                                  "left": (proxy._currMousePosX) + "px",
                                  'position': 'absolute',
                                  'z-index': '6'
                              }, {});
                            //proxy.tooltipState = "editingTemplateID";
                            var tooltipbody = ej.buildTag("tbody", $.render[proxy._id + "progressbarTooltipTemplate"](item), {}, {});
                            proxy._progressBarTooltipID[0].innerHTML = "<table>" + tooltipbody[0].innerHTML + "</table>";
                        }
                        else if (proxy.model.progressbarTooltipTemplateId) {
                            proxy._progressBarTooltipID = ej.buildTag("div.e-progressbartooltip#progressbartooltip" + proxy._id + "", "",
                                    {
                                        "top": (proxy._currMousePosY - 95) + "px",
                                        "left": (proxy._currMousePosX) + "px",
                                        'position': 'absolute',
                                        'z-index': '6'
                                    }, {});

                            //proxy.tooltipState = "editingTemplate";
                            var tooltipbody = ej.buildTag("tbody", $.render[proxy._id + "progressbarTooltipTemplate"](item), {}, {});
                            proxy._progressBarTooltipID[0].innerHTML = "<table>" + tooltipbody[0].innerHTML + "</table>";
                        }
                        else {
                            proxy._progressBarTooltipID = ej.buildTag("div.e-progressbartooltip#progressbartooltip" + proxy._id + "",
                                tooltiptable,
                                {
                                    "top": (proxy._currMousePosY - 65) + "px",
                                    "left": proxy._currMousePosX + "px",
                                    'position': 'absolute',
                                    'z-index': '6'
                                }, { 'onselectstart': 'return false' });
                        }
                        if (proxy._progressBarTooltipID) {
                            $(document.body).append(proxy._progressBarTooltipID);
                        }
                    }
                    // for apend tooltip for connector line
                    else if (this._editPredecessor === true && proxy.model.enablePredecessorEditing) {
                        if (proxy._predecessorTooltip === null) {
                            proxy._predecessorTooltip = ej.buildTag("div.e-tooltipganttpredecessor#predecessortooltip" + proxy._id + "",
                                      "",
                                      {
                                          "top": (proxy._currMousePosY + 10) + "px",
                                          "left": (proxy._currMousePosX + 10) + "px",
                                          'position': 'absolute',
                                          'z-index': '6'
                                      }, { 'onselectstart': 'return false' });
                            tooltiptable = ej.buildTag("table.e-tooltiptable", "", {}, { 'cellspacing': '2px', 'cellpadding': '2px' });
                            tooltipbody = ej.buildTag("tbody", "", {}, {});
                            tooltipbody[0].innerHTML = "<tr><td class='e-predecessor-false'></td><td >From:</td><td class='e-tooltiptaskname'>" + item.taskName + "</td><td>" + proxy._prevItem.predecessorTarget + "</td> </tr>" +
                                "<tr><td></td><td>To:</td><td class='e-tooltiptaskname'></td><td></td></tr>";
                            tooltiptable.append(tooltipbody);
                            proxy._predecessorTooltip.append(tooltiptable);
                            $(document.body).append(proxy._predecessorTooltip);

                            //   proxy._$bodyContent.append(proxy._predecessorTooltip);
                        }
                    }
                    else if (proxy._leftResizing == true || proxy._rightResizing == true || proxy._allowDragging == true)
                    {
                    if (proxy._leftResizing == true) {
                        proxy._$ganttChartContainer[0].style.cursor = "e-resize";

                        tooltipElement = {
                            ttipstartDate: item.startDate,
                            ttipduration: item.duration,
                        };
                    } else if (proxy._rightResizing == true) {

                        proxy._$ganttChartContainer[0].style.cursor = "w-resize";

                        tooltipElement = {
                            ttipendDate: item.endDate,
                            ttipduration: item.duration,
                        };
                    } else if (proxy._allowDragging == true) {
                        proxy._$ganttChartContainer[0].style.cursor = "move";

                        tooltipElement = {
                            ttipstartDate: item.startDate,
                            ttipendDate: item.endDate,
                        };
                    }


                    tooltiptable = ej.buildTag("table.e-tooltiptable", "", {}, { 'cellspacing': '2px', 'cellpadding': '2px' });

                    if (proxy.model.taskbarEditingTooltipTemplate) {
                        proxy._taskbarEditingTooltipID = ej.buildTag("div.e-editingtooltip#editingtooltip" + proxy._id + "", "",
                           {
                               "top": (proxy._currMousePosY - 95) + "px",
                               "left": (proxy._currMousePosX) + "px",
                               'position': 'absolute',
                               'z-index': '6'
                           }, {});
                        proxy.tooltipState = "editingTemplateID";
                        var tooltipbody = ej.buildTag("tbody", $.render[proxy._id + "editingTooltipTemplate"](item), {}, {});
                        proxy._taskbarEditingTooltipID[0].innerHTML = "<table>" + tooltipbody[0].innerHTML + "</table>";
                    }
                    else if (proxy.model.taskbarEditingTooltipTemplateId) {
                        proxy._taskbarEditingTooltipID = ej.buildTag("div.e-editingtooltip#editingtooltip" + proxy._id + "", "",
                                {
                                    "top": (proxy._currMousePosY - 95) + "px",
                                    "left": (proxy._currMousePosX) + "px",
                                    'position': 'absolute',
                                    'z-index': '6'
                                }, {});

                        proxy.tooltipState = "editingTemplate";
                        var tooltipbody = ej.buildTag("tbody", $.render[proxy._id + "editingTooltipTemplate"](item), {}, {});
                        proxy._taskbarEditingTooltipID[0].innerHTML = "<table>" + tooltipbody[0].innerHTML + "</table>";
                    }

                    else {
                        proxy.tooltipState = "";
                        if (tooltipElement) {
                            tooltipbody = ej.buildTag("tbody", $.render[proxy._id + "editingTooltipTemplate"](tooltipElement), {}, {});
                            tooltiptable.append(tooltipbody);
                        }

                        proxy._taskbarEditingTooltipID = ej.buildTag("div.e-editingtooltip#editingtooltip" + proxy._id + "", tooltiptable,
                            {
                                "top": (proxy._currMousePosY - 95) + "px",
                                "left": (proxy._currMousePosX) + "px",
                                'position': 'absolute',
                                'z-index': '6'
                            }, {});
                    }
                    if (proxy._taskbarEditingTooltipID) {
                        $(document.body).append(proxy._taskbarEditingTooltipID);
                    }
                }
                }
            }
        },

        _setItemPosition: function (item) {
            var proxy = this;
            $(proxy._leftResizer).css({ "left": (item.left + 2) + "px" });
            $(proxy._rightResizer).css({ "left": (item.left + item.width - 12) + "px" });
            $(proxy._taskbarItem).css({
                "left": (item.left) + "px",
                "width": (item.width) + "px"
            });
            $(proxy._progressbarItem).css({
                "left": (item.left) + "px",
                "width": (item.progressWidth) + "px"
            });
            $(proxy._progressResizer).css({ "left": (item.left + item.progressWidth - 10) + "px" });
            $(proxy._tasknameContainer).css({
                "left": (item.left) + "px",
                "width": (item.width) + "px"
            });

            $(proxy._leftConnectorPoint).css({ "left": (item.left - 10) + "px" });
            $(proxy._rightConnectorPoint).css({ "left": (item.left + item.width) + "px" });


        },

        //returns the custom template for tooltip
        _getTemplate: function (template) {
            if (typeof (template) != "object") {
                var parser = $.parseHTML(this.model.tooltipTemplate);
                return parser[0];
            } else {
                var markup = $(template)[0].innerHTML;
                parser = $.parseHTML(markup);
                return parser[1];
            }
        },


        //Handles the mousemove after mousedown event
        _updateOnmouseMove: function (e) {

            var proxy = this, args = {}, posy,
                $target = (proxy._editingTarget) ? proxy._editingTarget : null,
                $ganttGridRows = proxy.getGanttChartRows(),
                row = ($target) ? $target.closest('tr') : null,
                recordIndexr = $ganttGridRows.index(row),
                item = proxy._editingItem,
                div = ($target) ? $target[0].parentNode : null;

            if ($(div).hasClass("progressbarresizer-right") || $(div).hasClass("e-gantt-childtaskbar-progress")) {
                div = div.parentNode;
            }

            if (item != null) {

                args.editingFields = {
                    startdate: item.startDate,
                    enddate: item.endDate,
                    progress: item.status,
                    duration: item.duration
                };
                args.rowData = { rowData: item };

                proxy._taskbarEditing(args);

                if (!args.cancel) {

                    if (!e) e = window.event;
                    if (e.originalEvent.pageX || e.originalEvent.pageY) {
                        proxy._mousePosX = e.originalEvent.pageX;
                        posy = e.originalEvent.pageY;
                    } else if (e.originalEvent.clientX || e.originalEvent.clientY) {
                        proxy._mousePosX = e.originalEvent.clientX + document.body.scrollLeft
                            + document.documentElement.scrollLeft;
                        posy = e.originalEvent.clientY + document.body.scrollTop
                            + document.documentElement.scrollTop;
                    }
                    else if (e.originalEvent.touches[0].pageX || e.originalEvent.touches[0].pageY) {
                        proxy._mousePosX = e.originalEvent.targetTouches[0].pageX;
                        posy = e.originalEvent.targetTouches[0].pageY;
                    }

                    if ($target.hasClass("e-gantt-milestone")) {
                        if ($(row).hasClass("e-editmode")) {
                            this._editMilestone(e);
                        }
                    } else {

                        if ($(div).hasClass("e-childContainer")) {
                            proxy._editingContainer = div;
                            proxy._leftResizer = proxy._editingContainer.childNodes[1]; //[0]
                            proxy._taskbarItem = proxy._editingContainer.childNodes[2];//[1]
                            proxy._progressbarItem = proxy._editingContainer.childNodes[3];//[2]
                            proxy._progressResizer = proxy._editingContainer.childNodes[4];//[3]
                            proxy._rightResizer = proxy._editingContainer.childNodes[5];//[4]
                            proxy._tasknameContainer = proxy._editingContainer.childNodes[5];//task name contaner wrong
                            if (proxy.model.predecessorMapping && proxy.model.enablePredecessorEditing) {
                                proxy._leftConnectorPoint = div.childNodes[0];
                                proxy._rightConnectorPoint = div.childNodes[6];
                                $(proxy._leftConnectorPoint).addClass("e-connectorpoint-hover");
                                $(proxy._rightConnectorPoint).addClass("e-connectorpoint-hover");
                            }

                        }

                        if ($(row).hasClass("e-editmode") &&
                            (($(div).hasClass("e-parentContainer")) || $(div).hasClass("e-gantt-milestone-container") || $(div).hasClass("e-childContainer"))) {

                            if (proxy._leftResizing == true) {
                                proxy._enableLeftResizing(e, item);
                                proxy._updateEditedItem(item, "leftResizing");

                            } else if (proxy._progressResizing == true
                                && proxy.model.enableProgressBarResizing
                                && proxy.model.progressMapping) {
                                proxy._enableProgressResizing(e, item);
                                proxy._updateEditedItem(item, "progressResizing");

                            } else if (proxy._rightResizing == true) {
                                proxy._enableRightResizing(e, item);
                                proxy._updateEditedItem(item, "rightResizing");

                            } else if (proxy._allowDragging == true) {
                                proxy._enableDragging(e, item);
                                proxy._updateEditedItem(item, "dragging");
                            }

                            //for predecessor , draw false line move the tooltip
                            else if (proxy._editPredecessor == true) {
                                proxy._drawFalseLine(e, item);
                            }
                        }
                    }
                    proxy._clearTooltip();
                    proxy._appendTooltip($target, recordIndexr);
                }
            }
        },


        //Handler for milestone editing
        _editMilestone: function (e) {

            var proxy = this,
                $target = proxy._editingTarget,
                div = $target[0].parentNode,
                $ganttGridRows = proxy.getGanttChartRows(),
                row = $target.closest('tr'),
                recordIndexr = $ganttGridRows.index(row),
                item = proxy._editingItem;

            if (item != null) {

                if (!e) e = window.event;
                if (e.originalEvent.pageX || e.originalEvent.pageY) {
                    proxy._mousePosX = e.originalEvent.pageX;
                } else if (e.originalEvent.clientX || e.originalEvent.clientY) {
                    proxy._mousePosX = e.originalEvent.clientX + document.body.scrollLeft
                        + document.documentElement.scrollLeft;
                }
                else if (e.originalEvent.touches[0].pageX || e.originalEvent.touches[0].pageY) {
                    proxy._mousePosX = e.originalEvent.targetTouches[0].pageX;
                    posy = e.originalEvent.targetTouches[0].pageY;
                }

                e.target.style.cursor = "move";
                if (proxy.dragPosX > proxy._mousePosX) {
                    proxy._mousePosX = proxy.dragPosX - proxy._mousePosX;
                    if (item.left > 0) {
                        item.left -= proxy._mousePosX;
                    }
                } else {
                    proxy._mousePosX -= proxy.dragPosX;
                    item.left += proxy._mousePosX;
                }
                if (e.originalEvent.pageX == 0) {
                    this.dragPosX = e.originalEvent.targetTouches[0].pageX;
                }
                else {
                    this.dragPosX = e.originalEvent.pageX;
                }
                item.left = item.left < 0 ? 0 : item.left;

                $(div).css({ "left": (item.left) + "px" });

                $(proxy._leftConnectorPoint).css({ "left": (item.left - 10) + "px" });
                $(proxy._rightConnectorPoint).css({ "left": (item.left + 24) + "px" });

                proxy._updateEditedItem(item, "dragging");
                proxy._appendTooltip($target, recordIndexr);
            }
        },


        //Updates the edited item's values
        _updateEditedItem: function (ganttRecord, reason) {

            var proxy = this,
                model = proxy.model,
                projectStartDate = ganttRecord._getFormatedDate(model.projectStartDate, model.dateFormat),
                projectStartDate = ganttRecord._getDateFromFormat(projectStartDate, model.dateFormat),
                projectStartDateHours = ganttRecord._getDateFromFormat(ganttRecord.startDate, model.dateFormat).getHours(),
                projectStartDateMinutes = ganttRecord._getDateFromFormat(ganttRecord.startDate, model.dateFormat).getMinutes(),

                remainDays, day, offset, rem, startDate, endDate;

            switch (reason) {

                case 'dragging':
                    remainDays = ganttRecord.left % model.perDayWidth,
                    day = (ganttRecord.left - remainDays) / model.perDayWidth,
                    offset = 0;
                    if (remainDays > 0 && (model.workingTimeScale != ej.Gantt.workingTimeScale.TimeScale24Hours
                        && model.workingTimeScale != ej.Gantt.workingTimeScale.TimeScale24Hours)) {
                        offset = remainDays >= model.perDayWidth / 2 ? 1 : 0;
                    }
                    if (model.scheduleHeaderSettings.scheduleHeaderType == ej.Gantt.ScheduleHeaderType.Day ||
                        (model.scheduleHeaderSettings.scheduleHeaderType == ej.Gantt.ScheduleHeaderType.Week && model.dateFormat.indexOf("hh") != -1)) {
                        var remainingHours = Math.round(remainDays / (model.perDayWidth / 24));//hour calculation
                        projectStartDate.setDate(projectStartDate.getDate() + day + offset);
                        if (model.workingTimeScale == ej.Gantt.workingTimeScale.TimeScale24Hours)
                            projectStartDate.setHours(remainingHours);
                        else
                            projectStartDate.setHours(8);
                    }
                    else if (model.dateFormat.indexOf("hh:") &&
                       model.workingTimeScale == ej.Gantt.workingTimeScale.TimeScale8Hours) {
                        var remainingHours = Math.round(remainDays / (model.perDayWidth / 9));
                        projectStartDate.setHours(8 + remainingHours);
                        projectStartDate.setDate(projectStartDate.getDate() + day + offset);
                    }

                    ganttRecord.startDate = ganttRecord._calculateFormatedDate(projectStartDate, model.dateFormat);
                    ganttRecord._calculateEndDate(model.includeWeekend, model.holidays, model, model.perDayWidth);
                   
                    //update on dataSource
                     ganttRecord.item[model.startDateMapping] = ganttRecord.startDate;
                     ganttRecord.item[model.endDateMapping] = ganttRecord.endDate;
                    break;

                case 'leftResizing':
                    remainDays = ganttRecord.left % model.perDayWidth,
                    day = (ganttRecord.left - remainDays) / model.perDayWidth,
                    offset = 0;
                    if (remainDays > 0 && (model.workingTimeScale != ej.Gantt.workingTimeScale.TimeScale24Hours
                        && model.workingTimeScale != ej.Gantt.workingTimeScale.TimeScale24Hours)) {// 
                        offset = remainDays >= model.perDayWidth / 2 ? 1 : 0;
                    }

                    var remainingHours = Math.round(remainDays / (model.perDayWidth / 24));//hour calculation

                    projectStartDate.setDate(projectStartDate.getDate() + day + offset);

                    if (model.scheduleHeaderSettings.scheduleHeaderType == ej.Gantt.ScheduleHeaderType.Day ||
                        (model.scheduleHeaderSettings.scheduleHeaderType == ej.Gantt.ScheduleHeaderType.Week && model.dateFormat.indexOf("hh") != -1)) {
                        if (model.workingTimeScale == ej.Gantt.workingTimeScale.TimeScale24Hours)
                            projectStartDate.setHours(remainingHours);
                        else
                            projectStartDate.setHours(8);
                    }
                    else if (model.dateFormat.indexOf("hh:") &&
                        model.workingTimeScale == ej.Gantt.workingTimeScale.TimeScale8Hours) {
                        var remainingHours = Math.round(remainDays / (model.perDayWidth / 9));
                        projectStartDate.setHours(8 + remainingHours);
                    }


                    //if (projectStartDate < new Date(ganttRecord.endDate)) {
                    ganttRecord.startDate = ganttRecord._calculateFormatedDate(projectStartDate, model.dateFormat);
                 

                    ganttRecord._calculateDuration(projectStartDate, new Date(ganttRecord.endDate),
                        model.includeWeekend, model.holidays, false,model);
                    //}
                    if (Globalize.parseDate(ganttRecord.startDate, this.model.dateFormat).getTime() == Globalize.parseDate(ganttRecord.endDate, this.model.dateFormat).getTime()
                        && ej.isNullOrUndefined(ganttRecord.isMilestone) && ganttRecord.isMilestone == false && ganttRecord.duration == 0) {
                        ganttRecord.duration = 1;
                    }

                    //update on dataSource
                    ganttRecord.item[model.startDateMapping] = ganttRecord.startDate;
                    if(model.durationMapping)
                        ganttRecord.item[model.durationMapping] = ganttRecord.duration;

                    break;

                case 'rightResizing':
                    day = ganttRecord.width / model.perDayWidth,
                    rem = ganttRecord.width % model.perDayWidth,
                    startDate = new Date(ganttRecord.startDate);
                    offset = 0;

                    if (rem > 0 && (model.workingTimeScale != "TimeScale24Hours"
                      && model.workingTimeScale != "TimeScale24Hours")) {
                        offset = rem >= (model.perDayWidth / 2) ? 1 : 0;
                    }

                    var totalLength = ganttRecord.left + ganttRecord.width;
                    var remainingLength = totalLength % model.perDayWidth;

                    var remainingHours = Math.round(remainingLength / (model.perDayWidth / 24));//hour calculation


                    if ((model.scheduleHeaderSettings.scheduleHeaderType == ej.Gantt.ScheduleHeaderType.Day
                        && model.dateFormat.indexOf("hh") == -1)) {

                        var startDateHour = (ganttRecord.left % model.perDayWidth) / model.perHourWidth;
                        var remainingHourDiff = Math.round(remainingLength % (model.perHourWidth));

                        endDate = day < 1 ? ganttRecord._dateUpdate(startDate, 0, (remainingHourDiff)) :
                            ganttRecord._dateUpdate(startDate, Math.ceil(day - 1 + offset), remainingHourDiff);

                        startDate.setHours(startDateHour);
                    }

                    else if ((model.scheduleHeaderSettings.scheduleHeaderType == ej.Gantt.ScheduleHeaderType.Week
                    && model.dateFormat.indexOf("hh") != -1)) {
                        if (model.durationUnit == ej.Gantt.DurationUnit.Hour) {
                            var startDateHour = Math.round((ganttRecord.left % model.perDayWidth) / (model.perDayWidth / 24));
                            var remainingHourDiff = Math.round(totalLength / (model.perDayWidth / 24));
                            remainingHourDiff -= startDateHour;
                            endDate = ganttRecord._dateUpdate(startDate, 0, remainingHourDiff);
                        }
                        else {
                            var dayCount = (day - 1) + offset;
                            dayCount = Math.round(dayCount);
                            endDate = ganttRecord._dateUpdate(startDate, dayCount, 0);
                        }

                    }


                    else if ((model.scheduleHeaderSettings.scheduleHeaderType == ej.Gantt.ScheduleHeaderType.Day
                        && model.dateFormat.indexOf("hh") != -1)) {

                        var startDateHour = (ganttRecord.left % model.perDayWidth) / model.perHourWidth;
                        var remainingHourDiff = Math.round(totalLength / (model.perHourWidth));
                        remainingHourDiff -= startDateHour;

                        endDate = ganttRecord._dateUpdate(startDate, 0, (remainingHourDiff));

                        startDate.setHours(startDateHour);
                    }

                    else
                        endDate = ganttRecord._dateUpdate(startDate, day - 1 + offset, 0);
                    
                    if (model.scheduleHeaderSettings.scheduleHeaderType == ej.Gantt.ScheduleHeaderType.Day ||
                        (model.scheduleHeaderSettings.scheduleHeaderType == ej.Gantt.ScheduleHeaderType.Week && model.dateFormat.indexOf("hh") != -1)) {
                        if (model.workingTimeScale != ej.Gantt.workingTimeScale.TimeScale24Hours) {
                            if (model.durationUnit == ej.Gantt.DurationUnit.Hour) {
                                endDate.setHours(Math.round(remainingHours));
                            }
                            else
                                endDate.setHours(17);
                        }
                    }
                    else if (model.dateFormat.indexOf("hh:") &&
                       model.workingTimeScale == ej.Gantt.workingTimeScale.TimeScale8Hours) {
                        var remainingHours = Math.round(remainingLength / (model.perDayWidth / 9));
                        endDate.setHours(Math.round(8 + remainingHours));
                    }
                    ganttRecord.endDate = ganttRecord._calculateFormatedDate(endDate, model.dateFormat);

                    if (model.holidays && model.holidays.length) {
                        var holidayIndex = 0,
                            holidayslength = model.holidays.length,
                            holiday;
                        for (holidayIndex = 0; holidayIndex < holidayslength; holidayIndex++) {
                            holiday = new Date(model.holidays[holidayIndex]);
                            if (endDate === holiday) {
                                endDate.setDate(endDate.getDate() - 1);
                            }
                        }
                    }
                    if (this.model.includeWeekend == false) {
                        if (endDate.getDay() == 0)
                            endDate.setDate(endDate.getDate() - 2);
                        else if (endDate.getDay() == 6)
                            endDate.setDate(endDate.getDate() - 1);
                    }
                    ganttRecord._calculateDuration(startDate, endDate, model.includeWeekend, model.holidays, false, model);

                    //update on datasource
                    if(model.endDateMapping)
                        ganttRecord.item[model.endDateMapping] = ganttRecord.endDate;
                    if (model.durationMapping)
                        ganttRecord.item[model.durationMapping] = ganttRecord.duration;


                    break;

                case 'progressResizing':
                    ganttRecord.status = proxy._getProgressPercent(ganttRecord.width, ganttRecord.progressWidth);
                    if (model.progressMapping)
                        ganttRecord.item[model.progressMapping] = ganttRecord.status;
                    break;
            }
        },


        //false line implementation 
        _drawFalseLine: function (e, item) {

            var proxy = this,

              x1 = proxy._connectorPointX;
              y1 = proxy._connectorPointY;

              x2 = e.pageX - proxy._$bodyContent.offset().left,
              y2 = e.pageY - proxy._$bodyContent.offset().top;

            //var sy=proxy._$bodyContainer.ejScroller("option", "scrollTop");
            //var sx = proxy._$bodyContainer.ejScroller("option", "scrollLeft");

            var length = Math.sqrt((x1 - x2) * (x1 - x2) + (y1 - y2) * (y1 - y2));
            var angle = Math.atan2(y2 - y1, x2 - x1) * 180 / Math.PI;
            var transform = 'rotate(' + angle + 'deg)';

            proxy._falseLine && proxy._falseLine.remove();

                proxy._falseLine = ej.buildTag("div.e-gantt-falseLine#ganttfalseline" + proxy._id, "", {}, {});
                proxy._falseLine.css({
                    'transform-origin': '0% 100%',
                    'right':'auto',
                    'position': 'absolute',
                    'transform': transform,
                    '-ms-transform':transform, /* IE 9 */
                    '-moz-transform':transform, /* Firefox */
                    '-webkit-transform':transform, /* Safari and Chrome */
                    '-o-transform':transform, /* Opera */
                    'border-top-width': '2px',
                    'border-top-style': 'dashed',
                    'z-index': '5'
                }).width(length - 3).offset({ left: x1, top: y1 });
                proxy._$bodyContent.append(proxy._falseLine);

            //Change the position of predecesspr tooltip
                if (((e.pageX + proxy._predecessorTooltip.width() + 30) > proxy._windowWidth)) {
                    var pos =e.pageX - proxy._predecessorTooltip.width() - 10;
                    proxy._predecessorTooltip.css('left', ((pos) + 'px'));
                }
                else {
                    proxy._predecessorTooltip.css('left', (e.pageX+10) + 'px');
                }
                if (((e.pageY + proxy._predecessorTooltip.height() + 30) > proxy._windowHeight)) {
                    var pos = e.pageY - (proxy._predecessorTooltip.height()*2);
                    proxy._predecessorTooltip.css('top', ((pos) + 'px'));
                } else {
                    proxy._predecessorTooltip.css('top', (e.pageY + 10) + 'px');
                }
               
               
        },

        _mouseOverTooltip: function () {

            $(proxy._mouseOverTooltip).remove();

            if (this._mouseDown == false) {
                $(proxy._leftResizerGripper).removeClass("gripper");
                $(proxy._rightResizerGripper).removeClass("gripper");

                if (proxy.model.progressMapping) {
                    $(proxy._progressHandle).removeClass("e-progresshandle");
                    $(proxy._progressHandleChild).removeClass("e-progresshandleafter");
                }
            }
        },

        _mouseLeave: function () {

            var proxy = this;
            $(".e-tooltipgantt").remove();
            $(".progressbarresizer-right").css({ "background-image": "none" });
            $(proxy._mouseOverTooltip).remove();

            if (this._mouseDown == false) {
                $(proxy._leftResizerGripper).removeClass("gripper");
                $(proxy._rightResizerGripper).removeClass("gripper");

                if (proxy.model.progressMapping) {
                    $(proxy._progressHandle).removeClass("e-progresshandle");
                    $(proxy._progressHandleChild).removeClass("e-progresshandleafter");
                }

                if (proxy.model.predecessorMapping && proxy.model.enablePredecessorEditing) {

                    $(proxy._leftConnectorPoint).removeClass('e-connectorpoint-hover');
                    $(proxy._rightConnectorPoint).removeClass('e-connectorpoint-hover');
                    proxy._leftConnectorPoint = null;
                    proxy._rightConnectorPoint = null;
                }
            }
        },

        _mouseHover: function (e) {

            var $target = $(e.target), proxy = this,
                posx = 0,
                posy = 0,
                $ganttChartRows = proxy.getGanttChartRows(),
                row = $target.closest('tr'),
                div = $target[0].parentNode,
                progressHandler,
                recordIndexr = $ganttChartRows.index(row),
                item = proxy._currentViewData[recordIndexr],
                tooltipElement, tooltiptable, tooltipbody;

            if (proxy._rightResizing == false && proxy._leftResizing == false &&
                proxy._progressResizing == false && proxy._mouseDown == false) {

                if ($target.hasClass('e-gantt-parenttaskbar-innerdiv')
                    || $target.hasClass('e-gantt-parenttaskbar-progress')) {
                }
                else if (proxy.model.allowGanttChartEditing) {
                    e.target.style.cursor = "move";
                }


                if (e.type == "mouseenter" || e.type == "mousemove") {

                    if (!e) e = window.event;
                    if (e.originalEvent.pageX || e.originalEvent.pageY) {
                        posx = e.originalEvent.pageX;
                        posy = e.originalEvent.pageY;
                    }
                    else if (e.originalEvent.clientX || e.originalEvent.clientY) {
                        posx = e.originalEvent.clientX + document.body.scrollLeft
                            + document.documentElement.scrollLeft;
                        posy = e.originalEvent.clientY + document.body.scrollTop
                            + document.documentElement.scrollTop;
                    }

                    if ((posx + 200) > proxy._windowWidth) {
                        posx -= (posx + 200) - proxy._windowWidth;
                    }

                    if ((posy + 150) > proxy._windowHeight) {
                        posy -= (posy + 150) - proxy._windowHeight;
                    }

                    if ($(div).hasClass("e-childContainer") && proxy.model.allowGanttChartEditing) {
                        proxy._leftResizerGripper = div.childNodes[1],//[0]  // New div is added for drag points so other elemnts are changed
                        proxy._rightResizerGripper = div.childNodes[5];//[4]
                        progressHandler = div.childNodes[4];//[3]
                        proxy._progressHandle = progressHandler.firstChild;
                        proxy._progressHandleChild = progressHandler.childNodes[1];//[1]

                        $(proxy._leftResizerGripper).addClass("gripper");
                        $(proxy._rightResizerGripper).addClass("gripper");

                        if (proxy.model.progressMapping) {
                            $(proxy._progressHandle).addClass("e-progresshandle");
                            $(proxy._progressHandleChild).addClass("e-progresshandleafter");
                        }
                    }

                    //Before Mouse Down  , Apply style for drag points in child cotainer and miles stone container
                    if ($(div).hasClass("e-childContainer") && proxy.model.predecessorMapping  && proxy.model.allowGanttChartEditing && proxy.model.enablePredecessorEditing) {

                        proxy._leftConnectorPoint = div.childNodes[0];
                        proxy._rightConnectorPoint = div.childNodes[6];
                        $(proxy._leftConnectorPoint).addClass("e-connectorpoint-hover");
                        $(proxy._rightConnectorPoint).addClass("e-connectorpoint-hover");
                    }
                    else if ($(div).hasClass("e-gantt-milestone") && proxy.model.predecessorMapping && proxy.model.allowGanttChartEditing && proxy.model.enablePredecessorEditing) {
                        var milesStoneContainer = div.parentNode;
                        proxy._leftConnectorPoint = milesStoneContainer.childNodes[0];
                        proxy._rightConnectorPoint = milesStoneContainer.childNodes[2];
                        $(proxy._leftConnectorPoint).addClass("e-connectorpoint-hover");
                        $(proxy._rightConnectorPoint).addClass("e-connectorpoint-hover");
                    }


                    if (item && proxy.model.enableTaskbarTooltip) {
                        tooltipElement = {
                            ttipstartDate: item.startDate,
                            ttipendDate: item.endDate,
                            ttipduration: item.duration,
                            ttipprogress: item.status,
                            ttiptaskname: item.taskName,
                        };

                        tooltiptable = ej.buildTag("table.e-tooltiptable", "", {}, { 'cellspacing': '5' });
                      

                        if (proxy.model.tooltipTemplate == null && proxy.model.tooltipTemplateId == null) {
                            proxy.tooltipState = "";
                            tooltipbody = ej.buildTag("tbody", $.render[proxy._id + "tooltipTemplate"](tooltipElement), {}, {});
                            tooltiptable.append(tooltipbody);
                            proxy._mouseOverTooltip = ej.buildTag("div.e-tooltipgantt#tooltipgantt" + proxy._id + "", tooltiptable,
                                { "top": (posy + 10) + "px", "left": (posx + 15) + "px", 'position': 'absolute', 'z-index': '6', 'border-radius': '5px' }, {});

                            proxy._mouseOverTooltip.innerHTML = "<table>" + tooltiptable[0].innerHTML + "</table>";
                            $(document.body).append(proxy._mouseOverTooltip);
                        }
                        else if (proxy.model.tooltipTemplate) {
                            proxy._mouseOverTooltip = ej.buildTag("div.e-tooltipgantt#tooltipgantt" + proxy._id + "","",
                                { "top": (posy + 10) + "px", "left": (posx + 15) + "px", 'position': 'absolute', 'z-index': '6', 'border-radius': '5px' }, {});
                            $(document.body).append(proxy._mouseOverTooltip);

                            proxy.tooltipState = "Template";
                            var tooltipbody = ej.buildTag("tbody", $.render[proxy._id + "tooltipTemplate"](item), {}, {});
                            proxy._mouseOverTooltip[0].innerHTML = "<table>" + tooltipbody[0].innerHTML + "</table>";
                        }
                        else if (proxy.model.tooltipTemplateId) {
                            proxy._mouseOverTooltip = ej.buildTag("div.e-tooltipgantt#tooltipgantt" + proxy._id + "", "",
                                { "top": (posy + 10) + "px", "left": (posx + 15) + "px", 'position': 'absolute', 'z-index': '6', 'border-radius': '5px' }, {});
                            $(document.body).append(proxy._mouseOverTooltip);

                            proxy.tooltipState = "TemplateID";
                            var tooltipbody = ej.buildTag("tbody", $.render[proxy._id + "tooltipTemplate"](item), {}, {});
                            proxy._mouseOverTooltip[0].innerHTML = "<table>" + tooltipbody[0].innerHTML + "</table>";
                        }

                        $(proxy._mouseOverTooltip).css({
                            "left": (posx + 15) + "px",
                            "top": (posy) + "px",
                            "position": "absolute"
                        });
                        $(proxy._mouseOverTooltip).addClass("customTooltip");
                    }
                }
            }
        },



        _createEditingTooltipTemplate: function () {

            var proxy = this,
                td,
                columnHeaderTexts = proxy.model.columnHeaderTexts,
                helpers = {
                    _getStartDate: proxy._tooltipStartDate,
                    _getEndDate: proxy._tooltipEndDate,
                    _getDuration: proxy._tooltipDuration,
                    _getTaskName: proxy._tooltipTaskName,
                    _getDurationUnit: $.proxy(proxy._getEditDurationUnits, proxy),
                    _getProgress: proxy._tooltipProgress
                },
            editingTemplateId = proxy.model.taskbarEditingTooltipTemplateId,
            editingTemplate = proxy.model.taskbarEditingTooltipTemplate;

            $.views.helpers(helpers);

            
            if (editingTemplateId) {
                var parentTr = $("#" + editingTemplateId)[0].innerHTML;
            }
            else if (editingTemplate) {
                var parentTr = editingTemplate;
            }
            else {
                var parentTr = "<tr class='e-tooltip_rowcell'>";

                td = "{{if ~_getTaskName()}}" +
                    "<td class='e-tooltiptaskname' style='width:40px;text-align:left;' colspan='3';>{{:ttiptaskname}}</td>{{/if}}" +
                    "</tr><tr>" +
                    "{{if ~_getStartDate()}}" +
                    "<td class='e-tooltiptd-alignleft' style='width:30px;white-space:nowrap;'>" + columnHeaderTexts["startDate"] + "</td>" +
                    "<td class='e-tooltiptd-aligncenter' style='width:10px;font-weight:bold;'>:</td>" +
                    "<td class='e-tooltiptd-alignright' style='width:30px;white-space:nowrap;'>{{:ttipstartDate}}</td> {{/if}}" +
                    "</tr><tr>" +
                    "{{if ~_getEndDate()}}" +
                    "<td class='e-tooltiptd-alignleft' style='width:30px;white-space:nowrap;'>" + columnHeaderTexts["endDate"] + "</td>" +
                    "<td class='e-tooltiptd-aligncenter' style='width:10px;font-weight:bold;'>:</td>" +
                    "<td class='e-tooltiptd-alignright' style='width:30px;white-space:nowrap;'>{{:ttipendDate}}</td>{{/if}}" +
                    "</tr><tr>" +
                    "{{if ~_getDuration()}}" +
                    "<td class='e-tooltiptd-alignleft' style='width:30px;white-space:nowrap;'>" + columnHeaderTexts["duration"] + "</td>" +
                    "<td class='e-tooltiptd-aligncenter' style='width:10px;font-weight:bold;'>:</td>" +
                    "<td class='e-tooltiptd-alignright' style='width:30px;white-space:nowrap;'>{{:ttipduration}}  {{: ~_getDurationUnit(#data)}}</td>{{/if}}" +
                    "</tr><tr>" +
                    "{{if ~_getProgress()}}" +
                    "<td class='e-tooltiptd-alignleft' style='width:30px;white-space:nowrap;'>" + columnHeaderTexts["status"] + "</td>" +
                    "<td class='e-tooltiptd-aligncenter' style='width:10px;font-weight:bold;'>:</td>" +
                    "<td class='e-tooltiptd-alignright' style='width:30px;white-space:nowrap;'>{{:ttipprogress}}%</td>{{/if}}";

                parentTr += td;
                parentTr += "</tr>";
            }

            var templates = {};
            templates[proxy._id + "editingTooltipTemplate"] = parentTr;
            $.templates(templates);
        },

        _getEditDurationUnits: function (data) {
            var proxy = this;
            if (proxy.model.durationUnit == ej.Gantt.DurationUnit.Hour)
                return "hours";
            else
                return "days";

        },

        _createProgressbarTooltipTemplate: function () {
            var proxy = this,
                  templateId = proxy.model.progressbarTooltipTemplateId,
            template = proxy.model.progressbarTooltipTemplate;

            if (templateId) {
                var parentTr = $("#" + proxy.model.progressbarTooltipTemplateId)[0].innerHTML;
            }
            else if (proxy.model.progressbarTooltipTemplate) {
                var parentTr = proxy.model.progressbarTooltipTemplate;
            }
            var templates = {};
            templates[proxy._id + "progressbarTooltipTemplate"] = parentTr;
            $.templates(templates);
        },


        //Template to create tooltip
        _createTooltipTemplate: function () {

            var proxy = this,
                td,
                columnHeaderTexts = proxy.model.columnHeaderTexts,
                helpers = {
                    _getStartDate: proxy._tooltipStartDate,
                    _getEndDate: proxy._tooltipEndDate,
                    _getDuration: proxy._tooltipDuration,
                    _getTaskName: proxy._tooltipTaskName,
                    _getProgress: proxy._tooltipProgress,
                    _getDurationUnit: $.proxy(proxy._getDurationUnits, proxy),
                },
                templateId = proxy.model.tooltipTemplateId,
            template = proxy.model.tooltipTemplate;

            $.views.helpers(helpers);

            if (templateId ) {
                var parentTr = $("#" + proxy.model.tooltipTemplateId)[0].innerHTML;
            }
            else if (proxy.model.tooltipTemplate ) {
                var parentTr = proxy.model.tooltipTemplate;
            }
           
            else {
                var parentTr = "<tr class='e-tooltip_rowcell'>";

                td = "{{if ~_getTaskName()}}" +
                    "<td class='e-tooltiptaskname' style='width:40px;text-align:left;' colspan='3';>{{:ttiptaskname}}</td>{{/if}}" +
                    "</tr><tr>" +
                    "{{if ~_getStartDate()}}" +
                    "<td class='e-tooltiptd-alignleft' style='width:30px;white-space:nowrap;'>" + columnHeaderTexts["startDate"] + "</td>" +
                    "<td class='e-tooltiptd-aligncenter' style='width:10px;font-weight:bold;'>:</td>" +
                    "<td class='e-tooltiptd-alignright' style='width:30px;white-space:nowrap;'>{{:ttipstartDate}}</td> {{/if}}" +
                    "</tr><tr>" +
                    "{{if ~_getEndDate()}}" +
                    "<td class='e-tooltiptd-alignleft' style='width:30px;white-space:nowrap;'>" + columnHeaderTexts["endDate"] + "</td>" +
                    "<td class='e-tooltiptd-aligncenter' style='width:10px;font-weight:bold;'>:</td>" +
                    "<td class='e-tooltiptd-alignright' style='width:30px;white-space:nowrap;'>{{:ttipendDate}}</td>{{/if}}" +
                    "</tr><tr>" +
                    "{{if ~_getDuration()}}" +
                    "<td class='e-tooltiptd-alignleft' style='width:30px;white-space:nowrap;'>" + columnHeaderTexts["duration"] + "</td>" +
                    "<td class='e-tooltiptd-aligncenter' style='width:10px;font-weight:bold;'>:</td>" +
                    "<td class='e-tooltiptd-alignright' style='width:30px;white-space:nowrap;'>{{:ttipduration}} {{: ~_getDurationUnit(#data)}}</td>{{/if}}" +
                    "</tr><tr>" +
                    "{{if ~_getProgress()}}" +
                    "<td class='e-tooltiptd-alignleft' style='width:30px;white-space:nowrap;'>" + columnHeaderTexts["status"] + "</td>" +
                    "<td class='e-tooltiptd-aligncenter' style='width:10px;font-weight:bold;'>:</td>" +
                    "<td class='e-tooltiptd-alignright' style='width:30px;white-space:nowrap;'>{{:ttipprogress}}%</td>{{/if}}";

                parentTr += td;
                parentTr += "</tr>";
            }

            var templates = {};
            templates[proxy._id + "tooltipTemplate"] = parentTr;
            $.templates(templates);
        },

        _tooltipTaskName: function () {
            return this.data.ttiptaskname;
        },

        _tooltipProgress: function () {
            return this.data.ttipprogress;
        },

        _tooltipStartDate: function () {
            return this.data.ttipstartDate;
        },

        _tooltipEndDate: function () {
            return this.data.ttipendDate;
        },

        _tooltipDuration: function () {
            return this.data.ttipduration;
        },

        _getDurationUnits: function (data) {
            var proxy = this;
            if (proxy.model.durationUnit == ej.Gantt.DurationUnit.Hour)
                return "hours";
            else
                return "days";

        },

        _cellClickHandler: function (e) {
            var $target = $(e.target),
                $row = $target.closest('tr'),
                proxy = this,
                args = {},
                selectedItem,
                index,
                $ganttGridRows = proxy.getGanttChartRows();

            //proxy._clearContextMenu();
            if (proxy.model.selectedRowIndex !== -1) {

                selectedItem = proxy.model.updatedGanttRecords && proxy.model.updatedGanttRecords[proxy.model.selectedRowIndex];
                args = {
                    rowElement: $row,
                    recordIndex: proxy.model.selectedRowIndex
                };
            }

            if (proxy._trigger('rowSelecting', args))  //check the cancel arguements in rowSelecting event
                return;

            index = $ganttGridRows.index($target.parent());

            if (index == -1) {
                index = $ganttGridRows.index($row);
            }

            if (index !== -1) {
                
                selectedItem = proxy._currentViewData[index];
                proxy.model.selectedRowIndex = proxy.model.updatedGanttRecords.indexOf(selectedItem);
                proxy.selectRows(proxy.model.selectedRowIndex);
                args.target = "ejGanttChart";
                
                args.data = selectedItem;
                args.recordIndex = proxy.model.selectedRowIndex;
                proxy._trigger("rowSelected", args);

                if (proxy.prevSelectedItem != null) {
                    proxy.prevSelectedItem.isSelected = false;
                    proxy.model.updatedGanttRecords[proxy.model.selectedRowIndex].isSelected = true;
                }

                proxy.prevSelectedItem = selectedItem;

                proxy.element[0].focus();
            }

        },

        getRowByIndex: function (from, to) {
            try {
                var proxy = this;

                var gridRows = proxy.getGanttChartRows(),
                    $gridRows = $(gridRows),
                    $row = $();

                if (proxy.model.enableVirtualization) {
                    var recordstart = proxy.model.updatedGanttRecords[from];
                    from = proxy._currentViewData.indexOf(recordstart);
                }

                if (ej.isNullOrUndefined(to)) {
                    if (proxy.model.enableVirtualization) {
                        var recordend = proxy.model.updatedGanttRecords[to];
                        to = proxy._currentViewData.indexOf(recordend);
                    }
                    return $(gridRows[from]);
                } else {
                    return $($gridRows.slice(from, to));
                }
            } catch (e) {
                return $();
            }
        },

        selectRows: function (index) {
            var proxy = this,
                $ganttGridRows = proxy.getGanttChartRows(),
                selectedItem;
            if (proxy.model.selectedItem)
            {
                proxy.model.selectedItem.isSelected = false;
            }
            $ganttGridRows.removeClass("e-gantt-mouseclick");
            var $row = proxy.getRowByIndex(index);
            $row.addClass("e-gantt-mouseclick");
            proxy.model.selectedRowIndex = index;
            selectedItem = proxy.model.updatedGanttRecords[index];
            proxy.model.selectedItem = selectedItem;
        },

        updateSelectedItem: function (recordIndex) {
            var proxy = this,
                $ganttGridRows = $(proxy.getGanttChartRows());

            $ganttGridRows.removeClass("e-gantt-mouseclick");
            proxy.model.ganttRecords[recordIndex].isSelected = true;
            proxy.model.selectedItem = proxy.model.ganttRecords[recordIndex];
            this._refresh();
            proxy._rowSelectedEventTrigger(recordIndex);

        },


        ganttrowHover: function (index) {
            var proxy = this;
            if (!proxy.model.enableRowHover)
                return;
            var $gridRows =proxy.getGanttChartRows();
            $gridRows.removeClass("e-gantt_mousehover").addClass("e-gantt_mouseleave");;
            $($gridRows[index]).addClass("e-gantt_mousehover").removeClass("e-gantt_mouseleave");
        },

        _rowHover: function (e) {
            var proxy = this,
                $target = $(e.target),
                $gridRows,
                args = {},
                index,
                row;
            if (!proxy.model.enableRowHover)
                return false;
            $gridRows = $(proxy.getGanttChartRows());
            index = $gridRows.index($target.parent());
            if (index == -1) {
                row = $target.closest('tr');
                index = $gridRows.index(row);
            }
            args.index = index;

            if (e.type == "mouseenter") {
                args.reason = "mouseenter";
                proxy.addRowHover(index);
            } else {
                args.reason = "mouseleave";
                proxy.removeRowHover();
            }
            proxy._trigger("rowHover", args);
            return false;
        },

        addRowHover: function (index) {
            var $gridRows = this.getGanttChartRows();
            if (index != null)
                index != -1 && $($gridRows[index]).addClass("e-enablerowhover");
        },

        removeRowHover: function () {
            var $gridRows = this.getGanttChartRows();
            $gridRows.removeClass("e-enablerowhover");
        },

        updateHeight: function (height) {
            var proxy = this,
                model = proxy.model;
            if (model.stripLines) {
                $(".e-stripline,.e-stripLines").css({ height: height + "px" });
            }

            if (model.holidays) {
                $(".e-holidays,.e-holiday").css({ height: height + "px" });
            }
            $(".e-secondary_canvas").css({ height: height + "px" });
            $(".e-weekends").css({ height: height + "px" });

            if (height < proxy._viewportHeight) {
                if (height <= model.rowHeight) {
                    $(".e-holiday-label").css({ top:-height-30});
                }
                else
                    $(".e-holiday-label").css({ top: height / 2 });
            }
            else
                $(".e-holiday-label").css({ top: proxy._viewportHeight / 2 });

            proxy._$bodyContainer.ejScroller("refresh");

            if (model.enableVirtualization) {
               
               //proxy.onScrollHelper(proxy._renderedRange.top * model.rowHeight);
            }
        },


        //method to trigger actioncomplete event
        _completeAction: function (args) {
            var proxy = this;
            args.vscrollExsist = proxy._$bodyContainer.ejScroller("isVScroll");
            proxy._trigger("actionComplete", args);
            if (args.requestType === "validatePredecessor") {
                return args.predecessorValidation;
            }
        },


        //method to trigger taskbarediting event
        //to apply changes during taskbar editing
        _taskbarEditing: function (args) {
            var proxy = this;
            proxy._trigger("taskbarEditing", args);
        },


        //method to trigger taskbaredited event
        //to apply changes after taskbar edited
        _taskbarEdited: function (args) {
            var proxy = this;
            proxy._trigger("taskbarEdited", args);
        },


        //method to trigger querytaskbarinfo 
        //to apply changes at load time
        _queryTaskbarinfo: function (args) {
            this._trigger("queryTaskbarInfo", args);
            this._updateTaskbarItems(args);
        },


        //method to trigger refresh row
        _refreshRow: function (args) {
            var proxy = this;
            proxy._trigger("refreshRow", args);
        },

        _enbleZooming: function (args) {
            var proxy = this;
            proxy._trigger("zooming", args);
        },


        //Updates taskbar items styles when chart loads
        _updateTaskbarItems: function (args) {

            var $ganttChartRows = this.getGanttChartRows(),
                index = this.model.rows.indexOf(args.data),
                taskbarContainer = $ganttChartRows[index], taskElement;

            if (taskbarContainer) {
                if (taskbarContainer.childNodes.length > 1)
                    taskElement = taskbarContainer.childNodes[1].firstChild;
                else
                    taskElement = taskbarContainer.childNodes[0].firstChild;

                if ($(taskElement).hasClass("e-parentContainer")) {

                    var parentTaskbar = taskElement.firstChild,
                        parentProgressbar = taskElement.childNodes[1],
                        parentTaskbarStyle = parentTaskbar.style,
                        parentProgressbarStyle = parentProgressbar.style;

                    if (parentTaskbarStyle.backgroundColor != args.parentTaskbarBackground) {
                        parentTaskbarStyle.backgroundColor = args.parentTaskbarBackground;
                    }
                    if (parentProgressbarStyle.backgroundColor != args.parentProgressbarBackground) {
                        parentProgressbarStyle.backgroundColor = args.parentProgressbarBackground;
                    }

                } else if ($(taskElement).hasClass("e-childContainer")) {

                    var taskbar = taskElement.childNodes[1],
                        progressbar = taskElement.childNodes[2],
                        taskbarStyle = taskbar.style,
                        progressbarStyle = progressbar.style;

                    if (taskbarStyle.backgroundColor != args.taskbarBackground) {
                        taskbarStyle.backgroundColor = args.taskbarBackground;
                    }
                    if (progressbarStyle.backgroundColor != args.progressbarBackground) {
                        progressbarStyle.backgroundColor = args.progressbarBackground;
                    }
                }
            }
        },

        //to perform expand collapse operation from ganttchart
        _expandCollapse: function (recordIndex) {

            var proxy = this,
                args = {},
                data = proxy._currentViewData[recordIndex];

            args.data = data;
            args.recordIndex = proxy.model.updatedGanttRecords && proxy.model.updatedGanttRecords.indexOf(data);
            args.source = "ganttChart";
            args.expanded = !data.expanded;
            if (data) {
                if (!args.expanded) {
                    proxy._trigger("collapsing", args);
                }
                else {
                    proxy._trigger("expanding", args);
                }
            }
        },


        //get the Expand status of GridRecord
        _getExpandStatus: function (record) {
            var parentRecord = record.parentItem;
            if (parentRecord != null) {
                if (parentRecord.expanded === false)
                    return false;
                else if (parentRecord.parentItem) {
                    if (parentRecord.parentItem.expanded === false)
                        return false;
                    else
                        return this._getExpandStatus(parentRecord.parentItem);
                }
                else
                    return true;
            } else
                return true;
        },



        //method to scroll ganttchart from gridtree scroll event       

        onScrollHelper: function (scrollTop) {

            var proxy = this;
			if (!proxy._$bodyContainer.ejScroller("isVScroll"))
			{
                if(scrollTop!=0)
                proxy._$bodyContainer.ejScroller("scrollY", 0, true);
                return;
            }
            proxy._mouseLeave();

            var div = $("#ganttviewerbodyContianer" + this._id);
            
            $(".e-ganttviewerbodyContent").scrollTop(scrollTop);

            //Afetr addd new div in body container ,tregrid wheel event is not updated in chart part.
            $(".e-ganttviewerbodyContent").parent(".e-content").scrollTop(scrollTop);

            var h = parseInt(proxy.model.updatedGanttRecords.length * 30) - (proxy._viewportHeight - 18);
            

            if (scrollTop > h) {
                scrollTop = h;
            }

            if (scrollTop < 0) {
                scrollTop = 0;
            }

            var args = { scrollTop: scrollTop };

            proxy._handleScroll(args);
            
            //var div = $("#ganttviewerbodyContianer" + this._id);
            //div.scrollTop(scrollTop);
            
        },



        //Handler for scroll event from ganttchart
        _handleScroll: function (args) {

            var proxy = this,
                args =args|| {},
                headerdiv = proxy._$headerContainer,
                contentdiv = proxy._$bodyContainer,
                left = proxy._prevLeft;

            
            if (args.scrollTop !== undefined) {
                proxy._scrollTop = args.scrollTop;
            }

            if (args.scrollLeft !== undefined) {
                left = args.scrollLeft;
            }

            var diff = (left - proxy._prevLeft),
                vDiff = proxy._scrollTop - proxy._prevScrollTop;

            if (diff) {
                headerdiv.scrollLeft(left);
            }
                        else {
                            headerdiv.scrollLeft(proxy._prevLeft);
                        }

            if (proxy.model.enableVirtualization) {
                proxy._vscrollDist = Math.abs(proxy._scrollTop - proxy._prevScrollTop);
                if (proxy._vscrollDist) {
                    proxy._vScrollDir = proxy._prevScrollTop <= proxy._scrollTop ? 1 : -1;
                        proxy._updateCurrentViewData();

                    proxy.renderGanttRecords();
                    proxy._prevScrollTop = proxy._scrollTop;
                }
            }
            args.requestType = "scroll";
            args.delta = proxy._scrollTop;
            proxy._prevLeft = left;
            proxy._prevScrollTop = proxy._scrollTop;
            if (diff === 0 && vDiff !== 0) {
                proxy._completeAction(args);
            }
        },


        //helper method to refresh the ganttchart content from gantt
        refreshHelper: function (currentviewdata, records,collapseRecordsCount) {

            var proxy = this;
            proxy._currentViewData = currentviewdata;
            proxy.model.updatedGanttRecords = records;
            if (records) {
                proxy._refresh();
                proxy._setHeight(records.length * proxy.model.rowHeight);
                if (proxy.model.enableVirtualization) {
                    $("#ganttviewerbodyContent" + proxy._id + "").css({
                        "height": proxy._containerHeight + "px"
                    });
                    proxy.updateHeight(proxy._containerHeight);
                }
                else
                {
                    var tempHeight =0;
                    if (collapseRecordsCount)
                        tempHeight = (records.length - collapseRecordsCount) * proxy.model.rowHeight;
                    else
                        tempHeight = records.length * proxy.model.rowHeight;

                    proxy.updateHeight(tempHeight);
                }
           
                var isVScroll = proxy._$bodyContainer.ejScroller("isVScroll");
                var isHScroll = proxy._$bodyContainer.ejScroller("isHScroll");
                if (isVScroll || isHScroll)
                    proxy._$bodyContainer.ejScroller("refresh");
            }
        },

        getVisibleRecordsCount:function()
        {
            return this._visibleRecordsCount;
        },

        reRenderGanttGridLinesTable: function (length) {
            var proxy = this;
            //   var $gridLinesTablebody = $("#gridLinesTablebody" + proxy._id + "");
            //if (proxy.model.enableVirtualization) {
            //    proxy._$gridLinesTablebody.append(proxy._createGridLinesTableRow(length));
            //} else {
            proxy._visibleRecordsCount = (proxy._visibleRecordsCount + length < 0) ?
                proxy._visibleRecordsCount : proxy._visibleRecordsCount + length;

            if (proxy._visibleRecordsCount > 0) {
                proxy._$gridLinesTablebody.empty().append(proxy._createGridLinesTableRow(proxy._visibleRecordsCount));
                //this is called after update height method so scroller is refreshed
                proxy._$bodyContainer.ejScroller("refresh");
            }
            //}
        },

        //to refresh the content of the ganttchart
        _refresh: function () {

            var proxy = this, model = proxy.model,
                data = $.render[proxy._id + "_CustomTemplate2"](proxy._currentViewData);

            proxy._$ganttViewTablebody = $("#ganttViewTablebody" + proxy._id + "");

            proxy._$ganttViewTablebody.empty().append(data);

            //var $gridLinesTablebody = $("#gridLinesTablebody" + proxy._id + "");
            
            if (model.enableVirtualization) {
                proxy._visibleRecordsCount = proxy._currentViewData.length;
                proxy._$gridLinesTablebody.empty().append(proxy._createGridLinesTableRow(proxy._currentViewData.length));
            }
            else {
                if (proxy._visibleRecordsCount == 0 || proxy._visibleRecordsCount > proxy._currentViewData.length)
                    proxy._visibleRecordsCount = proxy._currentViewData.length;
                proxy._$gridLinesTablebody.empty().append(proxy._createGridLinesTableRow(proxy._visibleRecordsCount));
            }


            if (proxy._$ganttViewTablebody[0] != null) {
                this._ganttChartRows = $(proxy._$ganttViewTablebody[0].childNodes);
            }
            else {
                proxy._$ganttViewTablebody = ej.buildTag("tbody.e-gantt-viewer-ganttViewTablebody#ganttViewTablebody" + proxy._id + "",
                    data, {}, {});
                proxy._ganttChartRows = $(proxy._$ganttViewTablebody[0].childNodes);
            }
            proxy._eventBindings();
        },


        //to rerender a particular ganttchart row
        refreshRow: function (index) {

            var proxy = this,
                $tr = $(proxy.getGanttChartRows()[index]),
                selectedItem = proxy._currentViewData[index];

            if (index != -1 && selectedItem) {
                $tr.replaceWith($.render[proxy._id + "_CustomTemplate2"](selectedItem));
                proxy._setGanttChartRows($(proxy._$ganttViewTablebody[0].childNodes));
            }
        },

        //REFRESH THE SCROLLER FOR TREEGRIDCONTENT
        refreshScroller: function (panSize) {

            var proxy = this;
            //proxy._$gridContent.css({'width':panSize+"px"});
            proxy._$bodyContainer.removeClass("borderbox");
            var scrollTop = proxy._$bodyContainer.ejScroller("option", "scrollTop");
            proxy._$bodyContainer.ejScroller("option", { "width": panSize });
            proxy._$bodyContainer.ejScroller("refresh");
            var isHorizontalScroll = proxy._$bodyContainer.ejScroller("isHScroll");
            if (!isHorizontalScroll) {
                proxy._$bodyContainer.addClass("borderbox");
            }
            else {
                proxy._$bodyContainer.removeClass("borderbox");
            }
            var isVerticalScroll = proxy._$bodyContainer.ejScroller("isVScroll");
            if(isVerticalScroll){
                proxy._$bodyContainer.ejScroller("scrollY", scrollTop, true);
                }

        },

        renderRecord: function (args,collapseCount) {
            var proxy = this, model = proxy.model;
            addedchartRow = $.render[this._id + "_CustomTemplate2"](args._cAddedRecord);
            if (proxy._ganttChartRows.length != 0) {
                var insertIndex = proxy._currentViewData.indexOf(model.selectedItem);
                if (args.position) {
                    if (args.position == "Above") {
                        proxy.getGanttChartRows().eq(insertIndex-1).before(addedchartRow);//cuurent view data is updated before it is renderd
                    } else if (args.position == "Below") {
                        proxy.getGanttChartRows().eq(insertIndex).after(addedchartRow);
                    }
                }
                else {
                    if( insertIndex== -1) {
                        proxy.getGanttChartRows().eq(0).before(addedchartRow);
                    } else {
                        proxy.getGanttChartRows().eq(insertIndex).after(addedchartRow);
                    }
                }
                proxy._setGanttChartRows($(proxy._$ganttViewTablebody[0].childNodes));
            }

            if (proxy._ganttChartRows.length == 0) {
                proxy._updateCurrentViewData();
                proxy._refresh();
                proxy._setGanttChartRows($(proxy._$ganttViewTablebody[0].childNodes));
            }
            if (model.enableVirtualization === false)
            {
                proxy._setHeight((proxy.model.ganttRecords.length - collapseCount) * proxy.model.rowHeight);

            } else
            {
                proxy._setHeight(proxy.model.ganttRecords.length * proxy.model.rowHeight);
            }
            proxy.updateHeight(proxy._containerHeight);
            proxy._$bodyContainer.ejScroller("refresh");
           // proxy._updateCurrentViewData();
            //model.updatedGanttRecords.unshift(args._cAddedRecord);
           // proxy._refresh();
            //var _$ganttChartRows = proxy._$ganttViewTablebody[0].childNodes;
        },

        _setGanttChartRows: function (value) {
            this._ganttChartRows = value;
        },

       
        //delete gantt chart rows without rerendering of whole page
        deleteChartRows: function (startIndex, count) {
            var proxy = this;
            var $chartRows = proxy.getGanttChartRows();
            for (var i = startIndex; i < startIndex + count; i++) {
                $chartRows.splice(i, 1);
            }
        },


        //helper method to bind querytaskbarinfo events to ganttchart rows
        _eventBindings: function () {

            var $ganttChartRows = $(this.getGanttChartRows()),
                rowsLength = $ganttChartRows.length,
                taskbarContainer,
                args = {}, progressbar, taskbar,
                parentProgressbar, parentTaskbar, taskElement;

            if (rowsLength != 0) {

                for (var row = 0; row < rowsLength; row++) {

                    taskbarContainer = $ganttChartRows[row];

                    if ($(taskbarContainer).hasClass("e-ganttrowcell")) {

                        if (taskbarContainer.childNodes.length > 1)
                            taskElement = taskbarContainer.childNodes[1].firstChild;
                        else
                            taskElement = taskbarContainer.childNodes[0].firstChild;

                        if ($(taskElement).hasClass("e-parentContainer")) {

                            parentTaskbar = taskElement.firstChild;
                            parentProgressbar = taskElement.childNodes[1];
                        } else if ($(taskElement).hasClass("e-childContainer")) {

                            taskbar = taskElement.childNodes[1];
                            progressbar = taskElement.childNodes[2];
                        }

                        args.data = this.model.ganttRecords[row];
                        args.taskbar = taskbarContainer;

                        args.taskbarBackground = taskbar != null ?
                            taskbar.style.backgroundColor : null;

                        args.progressbarBackground = progressbar != null ?
                            progressbar.style.backgroundColor : null;

                        args.parentTaskbarBackground = parentTaskbar != null ?
                            parentTaskbar.style.backgroundColor : null;

                        args.parentProgressbarBackground = parentProgressbar != null ?
                            parentProgressbar.style.backgroundColor : null;
                        this._queryTaskbarinfo(args);
                    }
                }
            }
        },


        //to update the currentviewdata of ganttchart
        _updateCurrentViewData: function () {
            var proxy = this, margin = 0;
            if (!this.model.enableVirtualization) {
                this._currentViewData = this.model.updatedGanttRecords;
            }
            else {
                this._getRenderedRowRange();

                //console.log("Gantt Chart Rendered Range Top: " + proxy._renderedRange.top + "Bottom :" + proxy._renderedRange.bottom);

                this._currentViewData = this.model.updatedGanttRecords.slice(proxy._renderedRange.top,
                    proxy._renderedRange.bottom);
                if (proxy._vscrollDist !== 0) {
                    margin = proxy._scrollTop;
                    margin -= proxy._rowMargin;
                }
                $("#ganttViewTable" + proxy._id + "").css({ "top": margin + "px" });

                $("#ganttgridLinesTable" + proxy._id + "").css({ "top": margin + "px" });

                proxy._vTop = proxy._visibleRange.top;
                proxy._vBottom = proxy._visibleRange.bottom;
                proxy._rTop = proxy._renderedRange.top;
                proxy._rBottom = proxy._renderedRange.bottom;
            }

        },

       
        //to render ganttchart content
        renderGanttRecords: function () {

            var proxy = this;
            proxy._refresh();
            proxy._$ganttViewTable.append(proxy._$ganttViewTablebody);
            proxy._$bodyContent.append(proxy._$ganttViewTable);
        },

        _getRenderedRowRange: function () {

            var proxy = this;
            proxy._getVisibleRowRange();
            var topIndex = proxy._visibleRange.top;
            var bottomIndex = proxy._visibleRange.bottom;
            proxy._renderedRange = { top: topIndex, bottom: bottomIndex };
            return proxy._renderedRange;
        },

        _getVisibleRowRange: function () {

            var proxy = this,
                top = proxy._scrollTop / proxy.model.rowHeight,
                coeff = top - Math.floor(top),
                topIndex,
                bottomIndex, length;

            //console.log("GanttChart ScrollTop :" + proxy._scrollTop);

            proxy._rowMargin = coeff * proxy.model.rowHeight;
            topIndex = Math.floor(top);
            length = proxy.model.ganttRecords.length;
            bottomIndex = proxy._getrowposition(proxy._scrollTop + proxy._viewportHeight);
            topIndex = Math.max(0, topIndex);
            bottomIndex = Math.min(length, bottomIndex);
            proxy._visibleRange = { top: topIndex, bottom: bottomIndex };
        },

        _getrowposition: function (y) {
            return Math.ceil((y) / this.model.rowHeight);
        },


        //returns the date object
        _getDateType: function (date) {

            if (date != null) {
                if (typeof (date) == "object") {
                    return date;
                } else {
                    var re = new RegExp("\\-", "g");
                    var re1 = new RegExp("\\.", "g");
                    date = date.replace(re, "/");
                    date = date.replace(re1, "/");

                    return new Date(date);
                }
            }
            return null;
        },

        _createScheduleTopDayTemplate: function () {
            var proxy = this,
                model = proxy.model,
                dayFormat = model.scheduleHeaderSettings.dayHeaderFormat,
                helpers = {
                    renderDay: $.proxy(proxy._getDaySplit, proxy),
                };
            $.views.helpers(helpers);

            var parentTr = "<th class='e-schedule-week-headercell' style='box-sizing:border-box;-moz-box-sizing:border-box;'>";

            td = "<div class='e-schedule-week-headercell-content'  style='width:"
                + ((proxy.model.perHourWidth*24)-1) + "px;margin:auto;white-space:nowrap;'>"
                + "{{:~renderDay('" + dayFormat + "',#data)}}" + "</div>" +               
            "</div>"

            parentTr += td;
            parentTr += "</th>";

            var templates = {};
            templates[proxy._id + "ScheduleTopDayTemplate"] = parentTr;
            $.templates(templates);
        },

        //helper methood for day schedule mode

        _getDaySplit: function (dateformat,data) {
            var date = new Date(data),
               proxy = this,
               model = proxy.model,
               formatedDate;

            formatedDate = Globalize.format(date, dateformat, model.locale);
            return formatedDate;
        },


        // Month Week Schedule Mode

        _createScheduleTopMonthTemplate: function () {

            var proxy = this, td, i = 0,
                model = proxy.model,
                monthFormat = model.scheduleHeaderSettings.monthHeaderFormat,
                helpers = {
                    renderMonth: $.proxy(proxy._getMonthsplit, proxy),
                    setMonthWidth: $.proxy(proxy._setMonthWidth, proxy),
                    isLastMonth: $.proxy(proxy._checkLastMonthWidth, proxy),
                };

            $.views.helpers(helpers);
            var parentTr = "<th class='e-schedule-week-headercell' style='box-sizing:border-box;-moz-box-sizing:border-box;width:"
                + "{{:~setMonthWidth(#data)}}" + "px;'>";

            td = "<div class='e-schedule-week-headercell-content'  style='margin:auto;white-space:nowrap;'>"
                + "{{:~renderMonth('" + monthFormat + "',#data)}}" + "</div>"
                 +"{{if ~isLastMonth(#data)}}"
                 + "<th class='e-schedule-week-headercell' style='box-sizing:border-box;-moz-box-sizing:border-box;width:"+ proxy._balanceWidth + "px;'>"
                    + "<div class='e-schedule-week-headercell-content' style='margin:auto;white-space:nowrap;'>"
                +"</div></th>{{/if}}"
            
                parentTr += td;
                parentTr += "</th>";
                //if (proxy._balanceWidth) {
                //    parentTr += tdx;
                //    parentTr += "</th>";
                //}

            var templates = {};
            templates[proxy._id + "ScheduleTopMonthTemplate"] = parentTr;
            $.templates(templates);
        },

        //helper method for calculating unique month width
        _setMonthWidth: function (data) {
            var proxy = this;
            var date;
            if (typeof data === "object") {
                date = new Date(data);
            }
            else {
                date = Globalize.parseDate(data, "MM/dd/yyyy");
            }

            var monthLastDay = new Date(date.getFullYear(), date.getMonth() + 1, 0);

            var lastDate = monthLastDay.getDate();
            if (lastDate == 31)
                return (this.model.perDayWidth * 31);
            else if (lastDate == 30)
                return (this.model.perDayWidth * 30);
            else if (lastDate == 28)
                return (this.model.perDayWidth * 28);
            else if (lastDate == 29)
                return (this.model.perDayWidth * 29);
        },

        _checkLastMonthWidth: function (data) {
            var proxy = this;
            if (data == proxy.model.scheduleMonths[proxy.model.scheduleMonths.length - 1] && proxy._balanceWidth>0) {
                return true;
            }
            else
            {
                return false;
            }
        },


        //helper method for _createScheduleMonthTemplate
        _getMonthsplit: function (monthFormat, data) {

            var date = new Date(data),
                proxy = this,
                model = proxy.model,
                formatedDate;

            formatedDate = Globalize.format(date, monthFormat, model.locale);
            return formatedDate;

        },


        _createScheduleYearTemplate: function () {

            var proxy = this, td,
                model = proxy.model,
                yearFormat = model.scheduleHeaderSettings.yearHeaderFormat,
                helpers = {
                    renderYear: $.proxy(proxy._getYearsplit, proxy)
                };

            $.views.helpers(helpers);
            var parentTr = "<th class='e-schedule-week-headercell' style='box-sizing:border-box;-moz-box-sizing:border-box;'>";
            
            td = "<div class='e-schedule-week-headercell-content'  style='width:"
                + (proxy._scheduleYearWidth - 1) + "px;margin:auto;white-space:nowrap;'>"
                + "{{:~renderYear('" + yearFormat + "',#data)}}" + "</div>";

            parentTr += td;
            parentTr += "</th>";

            var templates = {};
            templates[proxy._id + "ScheduleYearTemplate"] = parentTr;
            $.templates(templates);
        },

        //helper method for _createScheduleYearTemplate
        _getYearsplit: function (yearFormat, data) {

            var date = new Date(data),
                proxy = this,
                model = proxy.model,
                formatedDate;

            formatedDate = Globalize.format(date, yearFormat, model.locale);
            return formatedDate;

        },

        
        //Template to render schedule week view
        _createScheduleWeekTemplate: function () {

            var proxy = this, td,
                model = proxy.model,
                dateFormat = model.scheduleHeaderSettings.weekHeaderFormat,
                helpers = {
                    renderWeek: $.proxy(proxy._getWeeksplit, proxy)
                };

            $.views.helpers(helpers);
            var parentTr = "<th class='e-schedule-week-headercell' style='box-sizing:border-box;-moz-box-sizing:border-box;'>";

            td = "<div class='e-schedule-week-headercell-content'  style='width:"
                + (proxy._scheduleWeekWidth - 1) + "px;margin:auto;white-space:nowrap;'>"
                + "{{:~renderWeek('" + dateFormat + "',#data)}}" + "</div>";

            parentTr += td;
            parentTr += "</th>";

            var templates = {};
            templates[proxy._id + "ScheduleWeekTemplate"] = parentTr;
            $.templates(templates);
        },

        //helper method for _createScheduleWeekTemplate
        _getWeeksplit: function (dateFormat, data) {

            var date = new Date(data),
                proxy = this,
                model = proxy.model,
                formatedDate;
            //var values = dateFormat.split(' '),
            //    scheduleUnits = values[0].match(/(\d+|[A-z]+)/g),
            //    month = date.getMonth(),
            //    day = date.getDay() == 1 ? proxy._days("mon") : "",
            //    weekTemplate = [];

            //switch (month) {
            //    case 0:
            //        month = proxy._getLocalizedLabels("jan");
            //        break;
            //    case 1:
            //        month = proxy._getLocalizedLabels("feb");
            //        break;
            //    case 2:
            //        month = proxy._getLocalizedLabels("mar");
            //        break;
            //    case 3:
            //        month = proxy._getLocalizedLabels("apr");
            //        break;
            //    case 4:
            //        month = proxy._getLocalizedLabels("may");
            //        break;
            //    case 5:
            //        month = proxy._getLocalizedLabels("jun");
            //        break;
            //    case 6:
            //        month = proxy._getLocalizedLabels("jul");
            //        break;
            //    case 7:
            //        month = proxy._getLocalizedLabels("aug");
            //        break;
            //    case 8:
            //        month = proxy._getLocalizedLabels("sep");
            //        break;
            //    case 9:
            //        month = proxy._getLocalizedLabels("oct");
            //        break;
            //    case 10:
            //        month = proxy._getLocalizedLabels("nov");
            //        break;
            //    case 11:
            //        month = proxy._getLocalizedLabels("dec");
            //        break;
            //}

            //for (var i = 0; i < scheduleUnits.length; i++) {
            //    switch (scheduleUnits[i]) {//('0' + currentDate.getHours()).slice(-2)
            //        case "d":
            //        case "D":
            //            weekTemplate.push(('0' + date.getDate()).slice(-2));
            //            break;
            //        case "dd":
            //        case "DD":
            //            weekTemplate.push(date.getDate());
            //            break;
            //        case "ddd":
            //        case "DDD":
            //            weekTemplate.push(day.substring(0, 3));
            //            break;
            //        case "dddd":
            //        case "DDDD":
            //            weekTemplate.push(day);
            //            break;
            //        case "M":
            //        case "m":
            //            weekTemplate.push(('0' + date.getMonth() + 1).slice(-2));
            //            break;
            //        case "MM":
            //        case "mm":
            //            weekTemplate.push(date.getMonth() + 1);
            //            break;
            //        case "MMM":
            //        case "mmm":
            //            weekTemplate.push(month.substring(0, 3));
            //            break;
            //        case "MMMM":
            //        case "mmmm":
            //            weekTemplate.push(month);
            //            break;
            //        case "YY":
            //        case "yy":
            //            weekTemplate.push(('0' + date.getFullYear()).slice(-2));
            //            break;
            //        case "YYYY":
            //        case "yyyy":
            //            weekTemplate.push(date.getFullYear());
            //            break;
            //    }
            //}
            formatedDate = Globalize.format(date, dateFormat, model.locale);

            return formatedDate.toLowerCase().replace(/(-(\w)|(^| )(\w)|:(\w)|,(\w)|\/(\w)|\.(\w))/g, function (x) {
                return x.toUpperCase();
            });
        },

        _createScheduleHourTemplate: function () {
            var proxy = this, model = proxy.model,td,
                hourHeaderFormat = model.scheduleHeaderSettings.hourHeaderFormat,
                helpers = {
                    renderHour:$.proxy(proxy._getHourTemplate,proxy),
                };
            $.views.helpers(helpers);
            var parentTr = "<th  class='e-schedule-day-headercell' " +
                "style='-moz-box-sizing:border-box;box-sizing:border-box;'>";
            td = "<div class='e-schedule-day-headercell-content' style='width:"
                + (proxy.model.perHourWidth - 1) + "px;position:static;box-sizing:border-box;overflow:hidden;text-overflow:clip'>" +
                "{{:~renderHour('" + hourHeaderFormat + "',#data)}}</div>";

            parentTr += td;

            parentTr += "</th>";
            var templates = {};
            templates[proxy._id + "scheduleHourTemplate"] = parentTr;
            $.templates(templates);
        },

        _getHourTemplate: function (format,data) {
            var date = new Date(data),
              proxy = this,
              model = proxy.model,
              formatedDate;

            formatedDate = Globalize.format(date, format, model.locale);
            return formatedDate;
        },


        //Template to render schedule week view in month-week schedule mode
        _createScheduleBottomWeekTemplate: function () {

            var proxy = this, td, model = proxy.model,
                weekFormat = model.scheduleHeaderSettings.weekHeaderFormat,
                helpers = {
                    renderWeek: $.proxy(proxy._getMonthWeekTemplate, proxy)
                };

            $.views.helpers(helpers);

            var parentTr = "<th  class='e-schedule-day-headercell' " +
                "style='width:"+ (proxy.model.perWeekWidth) + "px;-moz-box-sizing:border-box;box-sizing:border-box;'>";
            td = "<div class='e-schedule-day-headercell-content' style='position:static;box-sizing:border-box;overflow:hidden;text-overflow:clip;padding-left:2px;'>" +
                "{{:~renderWeek('" + weekFormat + "',#data)}}</div>";

            parentTr += td;

            parentTr += "</th>";
            var templates = {};
            templates[proxy._id + "ScheduleBottomWeekTemplate"] = parentTr;
            $.templates(templates);
        },

        _getMonthWeekTemplate: function (weekFormat, data) {

            var date = new Date(data),
                proxy = this, model = proxy.model;

            //date.setDate(date.getDate() + (weekLength * 7));

            switch (weekFormat) {
                case "d":
                case "dd":
                    //return (('0' + date.getDate()).slice(-2));
                case "ddd":

                default: return Globalize.format(date, weekFormat, model.locale);
            }
        },




        //Template to render schedule day view
        _createScheduleMonthTemplate: function () {

            var proxy = this, td, model = proxy.model,
                monthFormat = model.scheduleHeaderSettings.monthHeaderFormat,
                helpers = {
                    renderMonth: $.proxy(proxy._getMonthTemplate, proxy)
                };

            $.views.helpers(helpers);

            var parentTr = "<th  class='e-schedule-day-headercell' " +
                "style='-moz-box-sizing:border-box;box-sizing:border-box;'>";
            td = "<div class='e-schedule-day-headercell-content' style='width:"
                + (proxy.model.perMonthWidth - 1) + "px;position:static;box-sizing:border-box;overflow:hidden;text-overflow:clip'>" +
                "{{:~renderMonth('" + monthFormat + "',0,#data)}}</div>" +

                "<th  class='e-schedule-day-headercell' " +
                "style='-moz-box-sizing:border-box;box-sizing:border-box;'>" +
                "<div class='e-schedule-day-headercell-content' style='width:"
                + (proxy.model.perMonthWidth - 1) + "px;position:static;box-sizing:border-box;overflow:hidden;text-overflow:clip'>" +
                "{{:~renderMonth('" + monthFormat + "',1,#data)}}</div>" +

                "<th  class='e-schedule-day-headercell' " +
                "style='-moz-box-sizing:border-box;box-sizing:border-box;'>" +
                "<div class='e-schedule-day-headercell-content' style='width:"
                + (proxy.model.perMonthWidth - 1) + "px;position:static;box-sizing:border-box;overflow:hidden;text-overflow:clip'>" +
                "{{:~renderMonth('" + monthFormat + "',2,#data)}}</div>" +

                "<th  class='e-schedule-day-headercell' " +
                "style='-moz-box-sizing:border-box;box-sizing:border-box;'>" +
                "<div class='e-schedule-day-headercell-content' style='width:"
                + (proxy.model.perMonthWidth - 1) + "px;position:static;box-sizing:border-box;overflow:hidden;text-overflow:clip'>" +
                "{{:~renderMonth('" + monthFormat + "',3,#data)}}</div>" +

                "<th  class='e-schedule-day-headercell' " +
                "style='-moz-box-sizing:border-box;box-sizing:border-box;'>" +
                "<div class='e-schedule-day-headercell-content' style='width:"
                + (proxy.model.perMonthWidth - 1) + "px;position:static;box-sizing:border-box;overflow:hidden;text-overflow:clip'>" +
                "{{:~renderMonth('" + monthFormat + "',4,#data)}}</div>" +

                "<th  class='e-schedule-day-headercell' " +
                "style='-moz-box-sizing:border-box;box-sizing:border-box;'>" +
                "<div class='e-schedule-day-headercell-content' style='width:"
                + (proxy.model.perMonthWidth - 1) + "px;position:static;box-sizing:border-box;overflow:hidden;text-overflow:clip'>" +
                "{{:~renderMonth('" + monthFormat + "',5,#data)}}</div>" +

                "<th  class='e-schedule-day-headercell' " +
                "style='-moz-box-sizing:border-box;box-sizing:border-box;'>" +
                "<div class='e-schedule-day-headercell-content' style='width:"
                + (proxy.model.perMonthWidth - 1) + "px;position:static;box-sizing:border-box;overflow:hidden;text-overflow:clip'>" +
                "{{:~renderMonth('" + monthFormat + "',6,#data)}}</div>" +

                "<th  class='e-schedule-day-headercell' " +
                "style='-moz-box-sizing:border-box;box-sizing:border-box;'>" +
                "<div class='e-schedule-day-headercell-content' style='width:"
                + (proxy.model.perMonthWidth - 1) + "px;position:static;box-sizing:border-box;overflow:hidden;text-overflow:clip'>" +
                "{{:~renderMonth('" + monthFormat + "',7,#data)}}</div>" +

                "<th  class='e-schedule-day-headercell' " +
                "style='-moz-box-sizing:border-box;box-sizing:border-box;'>" +
                "<div class='e-schedule-day-headercell-content' style='width:"
                + (proxy.model.perMonthWidth - 1) + "px;position:static;box-sizing:border-box;overflow:hidden;text-overflow:clip'>" +
                "{{:~renderMonth('" + monthFormat + "',8,#data)}}</div>" +

                "<th  class='e-schedule-day-headercell' " +
                "style='-moz-box-sizing:border-box;box-sizing:border-box;'>" +
                "<div class='e-schedule-day-headercell-content' style='width:"
                + (proxy.model.perMonthWidth - 1) + "px;position:static;box-sizing:border-box;overflow:hidden;text-overflow:clip'>" +
                "{{:~renderMonth('" + monthFormat + "',9,#data)}}</div>" +

                "<th  class='e-schedule-day-headercell' " +
                "style='-moz-box-sizing:border-box;box-sizing:border-box;'>" +
                "<div class='e-schedule-day-headercell-content' style='width:"
                + (proxy.model.perMonthWidth - 1) + "px;position:static;box-sizing:border-box;overflow:hidden;text-overflow:clip'>" +
                "{{:~renderMonth('" + monthFormat + "',10,#data)}}</div>" +

                "<th  class='e-schedule-day-headercell' " +
                "style='-moz-box-sizing:border-box;box-sizing:border-box;'>" +
                "<div class='e-schedule-day-headercell-content' style='width:"
                + (proxy.model.perMonthWidth - 1) + "px;position:static;box-sizing:border-box;overflow:hidden;text-overflow:clip'>" +
                "{{:~renderMonth('" + monthFormat + "',11,#data)}}</div>";

            
            parentTr += td;
            parentTr += "</th>";
            var templates = {};
            templates[proxy._id + "ScheduleMonthTemplate"] = parentTr;
            $.templates(templates);
        },

        _getMonthTemplate: function (monthFormat, monthLength, data) {

            var date = new Date(data),
                proxy = this, model = proxy.model;
            date.setMonth(date.getMonth() + monthLength);

            switch (monthFormat) {
                case "M":
                case "m":
                    return (('0' + date.getDate()).slice(-2));
                case "MM":
                case "mm":
                    return date.getDate();
                case "MMM":
                case "mmm":
                case "MMMM":
                case "mmmm":
                    return Globalize.format(date, monthFormat, model.locale);
                default: return Globalize.format(date, monthFormat, model.locale);
            }
        },




        //Template to render schedule day view
        _createScheduleDayTemplate: function () {

            var proxy = this, td, model = proxy.model,
                dayFormat = model.scheduleHeaderSettings.dayHeaderFormat,
                helpers = {
                    renderDay: $.proxy(proxy._getDayTemplate, proxy)
                };

            $.views.helpers(helpers);

            var parentTr = "<th  class='e-schedule-day-headercell  e-headercell-weekend' " +
                "style='-moz-box-sizing:border-box;box-sizing:border-box;" +
                "background-color:" + this.model.scheduleHeaderSettings.weekendBackground + "'>";
            td = "<div class='e-schedule-day-headercell-content' style='width:"
                + (proxy.model.perDayWidth - 1) + "px;position:static;box-sizing:border-box;overflow:hidden;text-overflow:clip'>" +
                "{{:~renderDay('" + dayFormat + "',0,#data)}}</div>" +

                "<th  class='e-schedule-day-headercell' " +
                "style='-moz-box-sizing:border-box;box-sizing:border-box;'>" +
                "<div class='e-schedule-day-headercell-content' style='width:"
                + (proxy.model.perDayWidth - 1) + "px;position:static;box-sizing:border-box;overflow:hidden;text-overflow:clip'>" +
                "{{:~renderDay('" + dayFormat + "',1,#data)}}</div>" +

                "<th  class='e-schedule-day-headercell' " +
                "style='-moz-box-sizing:border-box;box-sizing:border-box;'>" +
                "<div class='e-schedule-day-headercell-content' style='width:"
                + (proxy.model.perDayWidth - 1) + "px;position:static;box-sizing:border-box;overflow:hidden;text-overflow:clip'>" +
                "{{:~renderDay('" + dayFormat + "',2,#data)}}</div>" +

                "<th  class='e-schedule-day-headercell' " +
                "style='-moz-box-sizing:border-box;box-sizing:border-box;'>" +
                "<div class='e-schedule-day-headercell-content' style='width:"
                + (proxy.model.perDayWidth - 1) + "px;position:static;box-sizing:border-box;overflow:hidden;text-overflow:clip'>" +
                "{{:~renderDay('" + dayFormat + "',3,#data)}}</div>" +

                "<th  class='e-schedule-day-headercell'" +
                " style='-moz-box-sizing:border-box;box-sizing:border-box;'>" +
                "<div class='e-schedule-day-headercell-content' style='width:"
                + (proxy.model.perDayWidth - 1) + "px;position:static;box-sizing:border-box;overflow:hidden;text-overflow:clip'>" +
                "{{:~renderDay('" + dayFormat + "',4,#data)}}</div>" +

                "<th  class='e-schedule-day-headercell'" +
                " style='-moz-box-sizing:border-box;box-sizing:border-box;'>" +
                "<div class='e-schedule-day-headercell-content' style='width:"
                + (proxy.model.perDayWidth - 1) + "px;position:static;box-sizing:border-box;overflow:hidden;text-overflow:clip'>" +
                "{{:~renderDay('" + dayFormat + "',5,#data)}}</div>" +

                "<th  class='e-schedule-day-headercell  e-headercell-weekend' " +
                "style='-moz-box-sizing:border-box;box-sizing:border-box;" +
                "background-color:" + this.model.scheduleHeaderSettings.weekendBackground + "'>" +
                "<div class='e-schedule-day-headercell-content' style='width:"
                + (proxy.model.perDayWidth - 1) + "px;position:static;box-sizing:border-box;overflow:hidden;text-overflow:clip'>" +
                "{{:~renderDay('" + dayFormat + "',6,#data)}}</div>";

            parentTr += td;
            parentTr += "</th>";
            var templates = {};
            templates[proxy._id + "_CustomTemplate1"] = parentTr;
            $.templates(templates);
        },

        _getDayTemplate: function (dayFormat, dayLength, data) {

            var date = new Date(data),
                proxy = this, model = proxy.model;
            date.setDate(date.getDate() + dayLength);

            switch (dayFormat) {
                case "D":
                case "d":
                    return (('0' + date.getDate()).slice(-2));
                case "DD":
                case "dd":
                    return date.getDate();
                case "DDD":
                case "ddd":
                case "DDDD":
                case "dddd":
                    return Globalize.format(date, dayFormat, model.locale)[0].toUpperCase();
                default: return Globalize.format(date, dayFormat, model.locale)[0].toUpperCase();
            }
        },


        //Method to render schedule header
        _createSchedule: function (startdate, enddate) {

            var proxy = this,
                model=proxy.model,
                scheduleMode = model.scheduleHeaderSettings.scheduleHeaderType,
                $colsgroup, $column, $thead,
                $tr, $td, $div, $table;

            startdate = startdate == null ? new Date() : startdate;
            enddate = enddate == null ? new Date(null) : enddate;

            proxy._createScheduleWeekTemplate();
            proxy._createScheduleYearTemplate();
            proxy._createScheduleTopMonthTemplate();//template for MONTH week schedule mode
            proxy._createScheduleTopDayTemplate();

            $table = ej.buildTag("table.e-schedule-headerrow-week", "",
                { 'display': 'block' },
                { 'cell-spacing': '0' }
            );

            $colsgroup = ej.buildTag("colgroup", "", {}, {});
            $column = $(document.createElement("col"));
            $column.css("width", proxy._getScheduleLength(scheduleMode) + 'px');
            
            $colsgroup.append($column);

            $thead = ej.buildTag("thead.e-ejganttschedule", "",
                { 'display': 'block', 'border-collapse': 'collapse' }, {});

            if (scheduleMode == "week")
                $tr = ej.buildTag("tr", $.render[this._id + "ScheduleWeekTemplate"](proxy.model.scheduleWeeks), {}, {});
            else if (scheduleMode == "year")
                $tr = ej.buildTag("tr", $.render[this._id + "ScheduleYearTemplate"](proxy.model.scheduleYears), {}, {});
            else if (scheduleMode == "month")
                $tr = ej.buildTag("tr", $.render[this._id + "ScheduleTopMonthTemplate"](proxy.model.scheduleMonths), {}, {});
            else if (scheduleMode == "day")
                $tr = ej.buildTag("tr", $.render[this._id + "ScheduleTopDayTemplate"](proxy.model.scheduleDays), {}, {});

            $td = ej.buildTag("th.schedule", "", {}, {});
            $div = ej.buildTag("div.schedule", "", { 'width': 20 + 'px' }, {});
            $td.append($div);
            $tr.append($td);

            $thead.append($tr);
            $table.append($colsgroup);
            $table.append($thead);
            proxy._$scheduleDiv.append($table);

            $tr = null;

            proxy._createScheduleDayTemplate();
            proxy._createScheduleMonthTemplate();//for year - month schedule mode
            proxy._createScheduleBottomWeekTemplate(); //for month - week schedule mode

            proxy._createScheduleHourTemplate();

            $table = ej.buildTag("table.e-schedule-headerrow-day", "",
                {
                    'display': 'block',
                    'border-collapse': 'collapse'
                }, { 'cell-spacing': '0' });
            $thead = ej.buildTag("thead.e-ejganttschedule", "",
                { 'display': 'block', 'border-collapse': 'collapse' }, {});
            
            if (scheduleMode == "week")
                $tr = ej.buildTag("tr", $.render[this._id + "_CustomTemplate1"](proxy.model.scheduleWeeks), {}, {});
            if (scheduleMode == "year")
                $tr = ej.buildTag("tr", $.render[this._id + "ScheduleMonthTemplate"](proxy.model.scheduleYears), {}, {});
            if (scheduleMode == "month")
                $tr = ej.buildTag("tr", $.render[this._id + "ScheduleBottomWeekTemplate"](proxy._numOfWeeks), {}, {});
            if (scheduleMode == "day")
                $tr = ej.buildTag("tr", $.render[this._id + "scheduleHourTemplate"](proxy._getScheduleHours()), {}, {});

            $td = ej.buildTag("th.schedule", "", {}, {});
            $div = ej.buildTag("div.schedule", "", { 'width': 20 + 'px' }, {});
            $td.append($div);
            $tr.append($td);

            $thead.append($tr);
            $table.append($colsgroup);
            $table.append($thead);
            proxy._$scheduleDiv.append($table);

            proxy._$headerContainer.append(proxy._$scheduleDiv);
            proxy._$ganttChartContainer.append(proxy._$headerContainer);
            $("#" + proxy._id).append(proxy._$ganttChartContainer);
        },

        _getScheduleHours: function () {
            var proxy = this, model = proxy.model,
                startHour = new Date(proxy.model.projectStartDate),
                endHour = new Date(model.projectEndDate);
            endHour.setHours(24);
            do {
                proxy._scheduleHours.push(new Date(startHour));
                startHour.setHours(startHour.getHours() + 1);
            } while (!(startHour.getTime()>= endHour.getTime()));

            return proxy._scheduleHours;
        },

        createGridLinesTable: function () {
            var proxy=this,
                model = proxy.model,
                $gridLinesTable,
                gridLineTable,
                scheduleMode = model.scheduleHeaderSettings.scheduleHeaderType,
                scheduleLength,
                tableRows;

            $gridLinesTable = ej.buildTag("table.e-ganttgridLinesTable#ganttgridLinesTable" + proxy._id, "", {
                'z-index': '2',
                'position': 'absolute',
            }, { 'cellspacing': '0px' });
            var $colsgroup = ej.buildTag("colgroup", "", {}, {});
            var $column = $(document.createElement("col"));

            switch (scheduleMode) {
                case "week":
                    scheduleLength = model.scheduleWeeks.length,
                    $gridLinesTable.css("width",(scheduleLength * proxy._scheduleWeekWidth) + 'px');
                    $column.css("width", (scheduleLength * this._scheduleWeekWidth) + 'px');
                    break;

                case "year":
                    scheduleLength = model.scheduleYears.length,                    
                    $gridLinesTable.css("width", (scheduleLength * proxy._scheduleYearWidth) + 'px');
                    $column.css("width", (scheduleLength * this._scheduleYearWidth) + 'px');

                    break;
                case "month":
                    scheduleLength = model.scheduleMonths.length,
                    $gridLinesTable.css("width", (proxy._scheduleMonthWidth) + 'px');
                    $column.css("width", (this._scheduleMonthWidth) + 'px');
                    break;
                case "day":
                    scheduleLength = model.scheduleDays.length,
                    $gridLinesTable.css("width", (scheduleLength * (proxy.model.perHourWidth*24)) + 'px');
                    $column.css("width", (scheduleLength * (proxy.model.perHourWidth * 24)) + 'px');
                    break;
                default:
                    break;

            }

            $colsgroup.append($column);
            $gridLinesTable.append($colsgroup);

            tableRows = proxy._createGridLinesTableRow(model.ganttRecords.length);
            
            var $gridLinesTablebody = ej.buildTag("tbody.e-gantt-viewer-gridLinesTablebody#gridLinesTablebody" + proxy._id + "",
                tableRows, {}, {});
            proxy._$gridLinesTablebody = $gridLinesTablebody;
            $gridLinesTable.append(proxy._$gridLinesTablebody);
            proxy._$bodyContent.append($gridLinesTable);
        },


        _createGridLinesTableRow: function (length) {
            var proxy = this,
                tableRows="";

            for (var i = 0; i < length; i++) {
                tableRows += "<tr style='display:table-row;height:" + this.model.rowHeight + "px'><td class='e-ganttgridlines'></td></tr>"
            }
            return tableRows;
        },


        //Template to render taskbar and milestone items
        _createTaskbarTemplate: function () {

            var proxy = this,
                model = proxy.model,
                tdWidth,
                scheduleMode = model.scheduleHeaderSettings.scheduleHeaderType,
                tr, baselineHeight = 4, taskbarHeight,
                shrinkedTaskbarHeight = 0,
                baselineColor = model.baselineColor,
                helpers = {
                    _getType: proxy._getTypedata,
                    _expander: proxy._addRecordExpandCollapse,
                    _milestoneMapping: proxy._isMilestone,
                    _resources: $.proxy(proxy._getResourceInfo, proxy),
                    _rowClassName: ej.TreeGrid._getrowClassName,//proxy._getrowClassName,
                    _expandStatus: $.proxy(proxy._getExpandStatusRecord, proxy),
                    _borderRadius: proxy._getBorderRadius,
                };

            if (scheduleMode == "week") {
                tdWidth = proxy.model.scheduleWeeks.length * proxy._scheduleWeekWidth;
            }
            else if (scheduleMode == "year") {
                tdWidth = proxy.model.scheduleYears.length * proxy._scheduleYearWidth;
            }
            else if (scheduleMode == "month") {
                tdWidth = proxy._scheduleMonthWidth;
            }
            else if (scheduleMode == "day") {
                tdWidth = proxy.model.scheduleDays.length * (proxy.model.perHourWidth*24);
            }

            if (model.renderBaseline) {
                //taskbarHeight = model.rowHeight - 10;
                taskbarHeight = model.rowHeight - baselineHeight - 5;
                shrinkedTaskbarHeight = 0;
            } else {
                taskbarHeight = model.rowHeight - 10;
            }

            $.views.helpers(helpers);

            if (!model.progressMapping) {


                this.model.showProgressStatus = false;
            }

            if (!model.resourceInfoMapping) {
                model.showResourceNames = false;
            }

            var parentTr = "<tr class='{{:~_rowClassName()}}  {{if isSelected}}e-ganttrowcell {{:~_expander()}} e-gantt-mouseclick{{else}}e-ganttrowcell {{:~_expander()}}{{/if}}' id='chartrowcell"
                + proxy._id + "' style='display:{{:~_expandStatus(#data)}};'>";
            var radius = "{{:~_borderRadius()}}";

            tr = "{{if ~_getType()}}" +

                //Parent taskbar Item

                "{{if " + this.model.showTaskNames + "}}<td class='e-chartcell'>" +
                "<div class='e-tasknameContainer'  unselectable='on' style='left:-{{:width}}px;width:{{:left+width}}px;'>" +
                "<span class='e-taskbarname'  style='margin-top:" + ((this.model.rowHeight) / 2) + "px;margin-right:30px;" +
                "font-weight:bold'>{{:taskName}}</span></div></td>{{/if}}" +

                "<td  class='e-chartcell' vertical-align='middle' style='width:" + tdWidth + "px;'>" +

               "<div class='e-parentContainer' unselectable='on' style='height:29px;z-index:3;'>" +

                "<div class='e-connectorpoint-left' style='left:{{:left-10}}px;width:10px;margin-top:10px;height:" + (taskbarHeight / 2) + "px;'></div>" +

                "<div class='e-gantt-parenttaskbar-innerdiv {{:~_expander()}}' style='left:{{:left}}px;margin-top:5px;width:{{:width}}px;height:"
                + (taskbarHeight) + "px;background-color:" + this.model.parentTaskbarBackground + " ;'></div>" +
                "<div  class='e-gantt-parenttaskbar-progress {{:~_expander()}}  progressbar' id='progressbar' " +
                "style='margin-top:5px;border-style:{{if progressWidth}}solid{{else}}none{{/if}}; left:{{:left}}px;width:{{:progressWidth}}px;height:" + (taskbarHeight)
                + "px;position:absolute;background-color:" + this.model.parentProgressbarBackground + ";z-index:3;" +
                "border-top-right-radius:" + radius + "px;border-bottom-right-radius:" + radius + "px;'></div>" +

                  "<div class='e-connectorpoint-right' style='left:{{:left+width}}px;width:10px;margin-top:10px;height:" + (taskbarHeight / 2) + "px;'></div>" +

             "</div>" +

                "{{if " + this.model.renderBaseline + " }}" +
                "<div  class='e-baselinebar' style='margin-top:-4px;left:{{:baselineLeft}}px;" +
                "width:{{:baselineWidth}}px;height:" + baselineHeight + "px;background-color: " + baselineColor + ";" +
                "position:absolute;z-index:3;box-sizing:border-box;-moz-box-sizing:border-box;'></div>{{/if}}" +

                "</td>" +

               "{{if " + this.model.showResourceNames + "}}<td class='e-chartcell' >" +
                "<div class='e-resourceinfo'  unselectable='on' style='margin-top:5px;left:{{:left+width+30}}px;" +
                "margin:0;margin-left:5px;margin-top:-9px;position:absolute;background-color:transparent;font-weight:bold'>" +
                "<span class='e-resourcename'  style='margin-top:" + ((this.model.rowHeight) / 2) + "px;margin-right:30px;outline:none;'>"
                + "{{:~_resources(#data)}}" + "</span></div></td>{{/if}}" +


               //Milestone Item

                "{{else ~_milestoneMapping()}}" +

                "{{if " + this.model.showTaskNames + "}}<td class='e-chartcell'>" +
                "<div class='e-tasknameContainer'  unselectable='on' style='left:-{{:width+150}}px;width:{{:left+width+150}}px;'>" +
                "<span class='e-taskbarname'  style='margin-top:" + ((this.model.rowHeight) / 2) + "px;margin-right:30px;'>" +
                "{{:taskName}}</span></div></td>{{/if}}" +

               "<td class='e-chartcell' style='width:" + tdWidth + "px;'>" +
                "<div class='e-gantt-milestone-container' style=height:29px;>" +
                "<div class='e-connectorpoint-left' style='left:{{:left-10}}px;width:10px;margin-top:10px;height:" + (taskbarHeight / 2) + "px;'></div>" +
                "<div class='e-gantt-milestone' style='position:absolute;left:{{:left}}px;'>" +
                "<div class='e-gantt-milestone milestone-top'  style='top:" + (3) + "px;'></div>" +
                "<div class='e-gantt-milestone milestone-bottom'  style='top:" + (15) + "px;'></div>" +
                "</div>"+
                 "<div class='e-connectorpoint-right' style='left:{{:left+24}}px;width:10px;margin-top:10px;height:" + (taskbarHeight / 2) + "px;'></div>" +
                "</div>" +

                "{{if " + this.model.renderBaseline + " }}" +
                "<div class='e-baseline-gantt-milestone-container' >" +
                "<div class='e-baseline-gantt-milestone' style='position:absolute;left:{{:baselineLeft}}px;z-index:2;'>" +
                "<div class='e-baseline-gantt-milestone baseline-milestone-top'  " +
                "style=' border-bottom-color: " + baselineColor + ";z-index:2;top:" + (-26) + "px;'></div>" +
                "<div class='e-baseline-gantt-milestone baseline-milestone-bottom'  " +
                "style=' border-top-color: " + baselineColor + ";z-index:2;top:" + (-14) + "px;'></div>" +
                "</div></div>{{/if}}" +

                "</td>" +

               "{{if " + this.model.showResourceNames + "}}<td class='e-chartcell' >" +
                "<div class='e-resourceinfo'  unselectable='on' style='margin-top:5px;left:{{:left+56}}px;margin:0;margin-left:5px;margin-top:-9px;" +
                "position:absolute;background-color:transparent;'><span class='e-resourcename'  style='margin-top:"
                + ((this.model.rowHeight) / 2) + "px;margin-right:30px;'>" + "{{:~_resources(#data)}}" + "</span></div></td>{{/if}}" +

                //Child Taskbar Item

                "{{else}}" +

                //left label
               "{{if " + this.model.showTaskNames + "}}<td class='e-chartcell'>" +
                "<div class='e-tasknameContainer'  unselectable='on' style='left:-{{:width}}px;width:{{:left+width}}px;'>" +
                "<span class='e-taskbarname'  style='margin-top:" + ((this.model.rowHeight) / 2) + "px;margin-right:30px;'>" +
                "{{:taskName}}</span></div></td>{{/if}}" +

                //taskbar
               "<td class='e-chartcell' style='width:" + tdWidth + "px;'>" +

               

               "<div  class='e-childContainer'  unselectable='on' style='height:29px;'>" +

               "<div class='e-connectorpoint-left' style='left:{{:left-10}}px;width:10px;margin-top:10px;height:" + (taskbarHeight/2) + "px;'></div>"+

              "<div class='taskbarresizer-left e-icon' style='left:{{:left+2}}px;width:10px;margin-top:5px;height:" + (taskbarHeight) + "px;'></div>" +

               "<div  class='e-gantt-childtaskbar' style='left:{{:left}}px;width:{{:width}}px;height:"
                + (taskbarHeight) + "px;background-color:" + this.model.taskbarBackground + ";margin-top:5px;'></div>" +

               "<div  class='e-gantt-childtaskbar-progress  progressbar' id='progressbar' style='border-style:{{if progressWidth}}solid{{else}}none{{/if}}; left:{{:left}}px;" +
                "width:{{:progressWidth}}px;text-align:right;line-height:" + (taskbarHeight) + "px;margin-top:5px;" +
                //"margin-top:" + (((taskbarHeight) - (this.model.progressbarHeight * ((taskbarHeight) / 100))) / 2) + "px;" +
                "height:" + (this.model.progressbarHeight * ((taskbarHeight) / 100)) + "px;" +
                "position:absolute;background-color:" + this.model.progressbarBackground + ";" +
                "border-top-right-radius:" + radius + "px;border-bottom-right-radius:" + radius + "px;'>" +
                "{{if " + this.model.showProgressStatus + "}}<span class='e-tasklabel' style=''>{{:status}}%</span>{{/if}}</div>" +

               "<div class='progressbarresizer-right' style='left:{{:left+progressWidth-10}}px;width:20px;margin-top:5px;" +
                "height:" + (taskbarHeight) + "px;position:absolute;z-index:4;'>" +
               "<div class='progressbarresizer-right' style='top:" + (22 - (shrinkedTaskbarHeight / 2)) + "px;margin-top:5px;position:absolute;'><div class='e-progresshandler-element'></div></div>" +//-" + (shrinkedTaskbarHeight/2) + "
               "<div class='progressbarresizer-right' style='top:" + (17 - (shrinkedTaskbarHeight / 2)) + "px;margin-top:5px;position:absolute;'><div class='e-progresshandlerafter-element'></div></div></div>" +

               "<div class='taskbarresizer-right e-icon' style='left:{{:left+width-12}}px;width:10px;" +
                "height:" + (taskbarHeight) + "px;position:absolute;z-index:2;margin-top:5px;'></div>" +
               
                "<div class='e-connectorpoint-right' style='left:{{:left+width}}px;width:10px;margin-top:10px;height:" + (taskbarHeight / 2) + "px;'></div>" +
                //"<div class='e-connectorpoint-right' style='left:{{:left-20}}px;width:20px;margin-top:10px;background-color:black;height:" + (taskbarHeight) + "px;'></div>"+
                 "</div>" +

                "{{if " + this.model.renderBaseline + " }}" +
                "<div  class='e-baselinebar' style='margin-top:-4px;left:{{:baselineLeft}}px;" +
                "width:{{:baselineWidth}}px;height:" + baselineHeight + "px;background-color: " + baselineColor + ";" +
                "position:absolute;z-index:3;box-sizing:border-box;-moz-box-sizing:border-box;'></div>{{/if}}" +

                "</td>" +

                //right label
               "{{if " + this.model.showResourceNames + "}}<td class='e-chartcell' >" +
                "<div class='e-resourceinfo'  unselectable='on' style='left:{{:left+width+30}}px;margin:0;margin-left:5px;margin-top:-9px;" +
                "position:absolute;text-align:left;background-color:transparent;'><span class='e-resourcename'  style='margin-top:" + ((this.model.rowHeight) / 2) + "px;margin-right:30px;'>" +
                "{{:~_resources(#data)}}" + "</span></div></td>{{/if}}{{/if}}";

            parentTr += tr;
            parentTr += "</tr>";

            var templates = {};
            templates[this._id + "_CustomTemplate2"] = parentTr;
            $.templates(templates);

        },

        _getResourceInfo: function (data) {
            var resource = null;
            if (data.resourceInfo != null) {
                var resourcelength = data.resourceInfo.length;
                if (resourcelength > 0) {
                    for (var i = 0; i < resourcelength; i++) {
                        if (resource == null)
                            resource = data.resourceInfo[i][this.model.resourceNameMapping];
                        else
                            resource += " , " + data.resourceInfo[i][this.model.resourceNameMapping];
                    }
                    return resource;
                } else
                    return data.resourceInfo[0][this.model.resourceNameMapping];
            }
            return String.empty;
        },

        _getrowClassName: function () {
            var rowClass = "gridrowtaskId",
                proxy = this;
            if (proxy.data.parentItem)
                rowClass += proxy.data.parentItem.taskId.toString();
            rowClass += "level";
            rowClass += proxy.data.level.toString();
            return rowClass;
        },

        _getExpandStatusRecord: function (data) {
            var proxy = this;
            if (proxy._getExpandStatus(data)) {
                return 'table-row';
            }
            return 'none';
        },


        _isMilestone: function () {
            if (this.data.isMilestone)
                return true;
            else
                return false;
        },

        _addRecordExpandCollapse: function () {
            if (this.data.expanded)
                return 'e-chartexpand';
            else if (!this.data.expanded && this.data.hasChildRecords) {
                return 'e-chartcollapse';
            }
            return false;
        },

        _getTypedata: function () {
            return this.data.hasChildRecords;
        },


        _getBorderRadius: function () {

            var proxy = this,
                diff = proxy.data.width - proxy.data.progressWidth;

            if (diff <= 4) {
                return 4 - diff;
            }
            else
                return 0;

        },


        _createConnectorLineTemplate: function () {
            var proxy = this,
                lineStroke = this.model.connectorlineWidth,
                shrinkedTaskbarHeight = 0,
                deltaTop, taskbarMidpoint,
                lineColor = this.model.connectorLineBackground,
                helpers = {
                    _getWidthVal: proxy._getWidth,
                    _getTop: proxy._getPredecessorTop,
                    _getPosition: proxy._getParentPosition,
                    _setContainerWidthSSType2: proxy._getContainerWidthSSType2,
                    _setInnerChildWidthSSType2: proxy._getInnerChildWidthSSType2,
                    _setInnerElementLeftSSType2: proxy._getInnerElementLeftSSType2,
                    _setInnerElementWidthSSType2: proxy._getInnerElementWidthSSType2,
                    _isMilestone: proxy._isTaskMilestone,
                    _isMilestoneParent: proxy._isMilestoneParent,
                };

            if (proxy.model.renderBaseline) {
                deltaTop = shrinkedTaskbarHeight / 2;
                taskbarMidpoint = ((this.model.rowHeight) / 2) - deltaTop;
            } else {
                taskbarMidpoint = ((this.model.rowHeight) / 2);
            }

            $.views.helpers(helpers);
            var connectorContainer = "<div  id='ConnectorLine{{:ConnectorLineId}}' style='background-color=black'>";

            var div = "{{if ~_getPosition()=='FSType1'}}"//FS
            + "<div style='left:{{:ParentLeft+ParentWidth}}px;top:{{:ParentTop+" + taskbarMidpoint + "}}px;" +
                "width:{{:(ChildLeft - (ParentLeft + ParentWidth + 20))}}px;height:{{:Height}}px;position:absolute'>"
            + "<div class='e-line' style='left:{{if milestoneParent}}-7px{{else}}0px{{/if}};width:{{if milestoneParent}}{{:(ChildLeft-(ParentLeft + ParentWidth + 10))+" + lineStroke + "-2}}px{{else}}{{:(ChildLeft-(ParentLeft + ParentWidth + 10))+" + lineStroke + "-10}}px{{/if}};" +
                "height:" + lineStroke + "px;background-color:" + lineColor + ";position:relative'></div>"
            + "<div class='e-line' style='left:{{:(ChildLeft-(ParentLeft + ParentWidth + 10))+" + lineStroke + "-10}}px;" +
                "width:" + lineStroke + "px;height:{{:Height-" + lineStroke + "}}px;background-color:" + lineColor + ";position:relative'></div>"
            + "<div class='e-line' style='left:{{:(ChildLeft-(ParentLeft + ParentWidth + 10))+" + lineStroke + "-10}}px;width:10px;" +
                "height:" + lineStroke + "px;background-color:" + lineColor + ";position:relative'></div>"
            + "<div class='connectorline-rightarrow' style='left:{{:(ChildLeft - (ParentLeft + ParentWidth + 10))}}px;" +
                "border-left: 10px solid " + lineColor + ";top:{{:-6-" + lineStroke + "}}px;" +
                "border-bottom-width:{{:5+" + lineStroke + "}}px;border-top-width:{{:5+" + lineStroke + "}}px;width:0;height:0;position:relative'></div>"
            + "</div>"

            + "{{else  ~_getPosition()=='FSType2'}}"//FS
            + "<div style='left:{{:ParentLeft}}px;top:{{:ParentTop+" + taskbarMidpoint + "}}px;" +
                "width:{{:~_getWidthVal()}}px;height:{{:Height}}px;position:absolute'>"
            + "<div class='e-line' style='left:{{if milestoneParent}}{{:ParentWidth-7}}px{{else}}{{:ParentWidth}}px{{/if}};width:{{if milestoneParent}}17px{{else}}10px{{/if}};height:" + lineStroke + "px;background-color:" + lineColor + ";position:relative'></div>"
            + "<div class='e-line' style='left:{{:ParentWidth+10-" + lineStroke + "}}px;width:" + lineStroke + "px;" +
                "height:{{:Height-14-" + lineStroke + "}}px;background-color:" + lineColor + ";" +
                "position:relative'></div><div class='e-line' style='left:{{:(ParentWidth - (((ParentLeft + ParentWidth) - ChildLeft) + 20))}}px;" +
                "width:{{:(((ParentLeft + ParentWidth) - ChildLeft) + 30 )}}px;height:" + lineStroke + "px;" +
                "background-color:" + lineColor + ";position:relative'></div>"
            + "<div class='e-line' style='left:{{:(ParentWidth - (((ParentLeft + ParentWidth) - ChildLeft) + 20))}}px;width:" + lineStroke + "px;" +
                "height:{{:14-" + lineStroke + "}}px;background-color:" + lineColor + ";position:relative'></div>"
            + "<div class='e-line' style='left:{{:(ParentWidth - (((ParentLeft + ParentWidth) - ChildLeft) + 20))}}px;width:10px;" +
                "height:" + lineStroke + "px;background-color:" + lineColor + ";position:relative'></div>"
            + "<div class='connectorline-rightarrow' style='left:{{:(ParentWidth - (((ParentLeft + ParentWidth) - ChildLeft) + 10))}}px;" +
                "border-left: 10px solid " + lineColor + ";border-bottom-width:{{:5+" + lineStroke + "}}px;" +
                "border-top-width:{{:5+" + lineStroke + "}}px;top:{{:-6-" + lineStroke + "}}px;width:0;height:0;position:relative'></div>"
            + "</div>"

            + "{{else ~_getPosition()=='FSType3'}}"
            + "<div style='left:{{:ChildLeft-20}}px;top:{{:ChildTop+" + taskbarMidpoint + "}}px;" +
                "width:{{:ParentWidth+ChildWidth}}px;height:{{:Height}}px;position:absolute'>"
            + "<div class='connectorline-rightarrow' style='left:10px;border-left: 10px solid " + lineColor + ";" +
                "border-bottom-width:{{:5+" + lineStroke + "}}px;border-top-width:{{:5+" + lineStroke + "}}px;" +
                "top:{{:-6-" + lineStroke + "}}px;width:0;height:0;position:relative'></div>"
            + "<div class='e-line' style='width:10px;height:" + lineStroke + "px;background-color:" + lineColor + ";" +
                "position:relative;top:-13px;'></div>"
            + "<div class='e-line' style='width:" + lineStroke + "px;height:{{:Height-14-" + lineStroke + "}}px;" +
                "background-color:" + lineColor + ";position:relative;top:-13px;'></div>"
            + "<div class='e-line' style='width:{{:(((ParentLeft + ParentWidth) - ChildLeft) + 30)}}px;height:" + lineStroke + "px;" +
                "background-color:" + lineColor + ";position:relative;top:-13px;'></div>"
            + "<div class='e-line' style='left:{{:(((ParentLeft + ParentWidth) - ChildLeft) + 30)}}px;width:" + lineStroke + "px;" +
                "height:{{:12-" + lineStroke + "}}px;background-color:" + lineColor + ";position:relative;" +
                "top:-13px;'></div>"
            + "<div class='e-line' style='left:{{:(((ParentLeft + ParentWidth) - ChildLeft) + 20)}}px;width:10px;height:" + lineStroke + "px;" +
                "background-color:" + lineColor + ";position:relative;top:-13px;'></div>"
            + "</div>"

            + "{{else ~_getPosition()=='FSType4'}}"
            + "<div style='left:{{:ParentLeft+ParentWidth}}px;top:{{:ChildTop+" + taskbarMidpoint + "}}px;" +
                "width:{{:(ChildLeft - (ParentLeft + ParentWidth + 20))}}px;height:{{:Height}}px;position:absolute'>"
            + "<div class='connectorline-rightarrow' style='left:{{:(ChildLeft - (ParentLeft + ParentWidth + 10))}}px;" +
                "border-left: 10px solid " + lineColor + ";top:{{:-6-" + lineStroke + "}}px;" +
                "border-bottom-width:{{:5+" + lineStroke + "}}px;border-top-width:{{:5+" + lineStroke + "}}px;width:0;height:0;position:relative'></div>"
            + "<div class='e-line' style='left:{{:(ChildLeft - (ParentLeft + ParentWidth) - 20)}}px;top:-13px;width:10px;" +
                "height:" + lineStroke + "px;background-color:" + lineColor + ";position:relative'></div>"
            + "<div class='e-line' style='top:-13px;left:{{:(ChildLeft - (ParentLeft + ParentWidth) - 20)}}px;width:" + lineStroke + "px;" +
                "height:{{:Height-" + lineStroke + "}}px;background-color:" + lineColor + ";position:relative'></div>"
            + "<div class='e-line' style='{{if ~_isMilestoneParent()}}left:-6px;{{/if}}top:-13px;width:{{if ~_isMilestoneParent()}}{{:(ChildLeft-(ParentLeft + ParentWidth+20 )+ 6)+" + lineStroke + "}}px;{{else}}{{:(ChildLeft-(ParentLeft + ParentWidth + 20))+" + lineStroke + "}}px;{{/if}}" +
                "height:" + lineStroke + "px;background-color:" + lineColor + ";position:relative'></div>"
            + "</div>"

            + "{{else ~_getPosition()=='SSType4'}}"
            + "<div  style='left:{{:ParentLeft-10}}px;top:{{:ChildTop+" + taskbarMidpoint + "}}px;" +
                "width:{{:ChildLeft-ParentLeft}}px;height:{{:Height}}px;position:absolute'>"
            + "<div class='connectorline-rightarrow' style='left:{{:ChildLeft-ParentLeft}}px;border-left: 10px solid " + lineColor + ";" +
                "top:{{:-6-" + lineStroke + "}}px;border-bottom-width:{{:5+" + lineStroke + "}}px;" +
                "border-top-width:{{:5+" + lineStroke + "}}px;width:0;height:0;position:relative'></div>"
            + "<div class='e-line' style='top:-13px;width:{{:ChildLeft-ParentLeft}}px;height:" + lineStroke + "px;" +
                "background-color:" + lineColor + ";position:relative'></div>"
            + "<div class='e-line' style='top:-13px;width:" + lineStroke + "px;height:{{:Height-" + lineStroke + "}}px;" +
                "background-color:" + lineColor + ";position:relative'></div>"
            + "<div class='e-line' style='top:-13px;width:10px;height:" + lineStroke + "px;background-color:" + lineColor + ";position:relative'></div>"
            + "</div>"

            + "{{else ~_getPosition()=='SSType3'}}"
            + "<div  style='left:{{:ChildLeft-20}}px;top:{{:ChildTop+" + taskbarMidpoint + "}}px;" +
                "width:{{:ParentLeft-ChildLeft}}px;height:{{:Height}}px;position:absolute'>"
            + "<div class='connectorline-rightarrow' style='left:10px;border-left: 10px solid " + lineColor + ";" +
                "top:{{:-6-" + lineStroke + "}}px;border-bottom-width:{{:5+" + lineStroke + "}}px;" +
                "border-top-width:{{:5+" + lineStroke + "}}px;width:0;height:0;position:relative'></div>"
            + "<div class='e-line' style='top:-13px;width:10px;height:" + lineStroke + "px;background-color:" + lineColor + ";position:relative'></div>"
            + "<div class='e-line' style='top:-13px;width:" + lineStroke + "px;height:{{:Height-" + lineStroke + "}}px;" +
                "background-color:" + lineColor + ";position:relative'></div>"
            + "<div class='e-line' style='top:-13px;width:{{:ParentLeft-ChildLeft+20}}px;height:" + lineStroke + "px;" +
                "background-color:" + lineColor + ";position:relative'></div>"
            + "</div>"

            + "{{else ~_getPosition()=='SSType2'}}"
            + "<div  style='left:{{:~_setInnerElementLeftSSType2()}}px;top:{{:ParentTop+" + taskbarMidpoint + "}}px;" +
                "width:{{:~_setContainerWidthSSType2()}}px;height:{{:Height}}px;position:absolute'>"
            + "<div class='e-line' style='width:{{:~_setInnerChildWidthSSType2()}}px;height:" + lineStroke + "px;" +
                "background-color:" + lineColor + ";position:relative'></div>"
            + "<div class='e-line' style='width:" + lineStroke + "px;height:{{:Height-" + lineStroke + "}}px;" +
                "background-color:" + lineColor + ";position:relative'></div>"
            + "<div class='e-line' style='width:{{:~_setInnerElementWidthSSType2()}}px;height:" + lineStroke + "px;" +
                "background-color:" + lineColor + ";position:relative'></div>"
            + "<div class='connectorline-rightarrow' style='left:{{:~_setInnerElementWidthSSType2()}}px;" +
                "border-left: 10px solid " + lineColor + ";top:{{:-6-" + lineStroke + "}}px;" +
                "border-bottom-width:{{:5+" + lineStroke + "}}px;border-top-width:{{:5+" + lineStroke + "}}px;width:0;" +
                "height:0;position:relative'></div>"
            + "</div>"

            + "{{else ~_getPosition()=='SSType1'}}"
            + "<div  style='left:{{:ChildLeft-20}}px;top:{{:ParentTop+" + taskbarMidpoint + "}}px;" +
                "width:{{:ParentLeft-ChildLeft}}px;height:{{:Height}}px;position:absolute'>"
            + "<div class='e-line' style='width:{{:ParentLeft-ChildLeft+20}}px;height:" + lineStroke + "px;" +
                "background-color:" + lineColor + ";position:relative'></div>"
            + "<div class='e-line' style='width:" + lineStroke + "px;height:{{:Height-" + lineStroke + "}}px;" +
                "background-color:" + lineColor + ";position:relative'></div>"
            + "<div class='e-line' style='width:10px;height:" + lineStroke + "px;background-color:" + lineColor + ";" +
                "position:relative'></div>"
            + "<div class='connectorline-rightarrow' style='left:10px;border-left: 10px solid " + lineColor + ";" +
                "top:{{:-6-" + lineStroke + "}}px;border-bottom-width:{{:5+" + lineStroke + "}}px;" +
                "border-top-width:{{:5+" + lineStroke + "}}px;width:0;height:0;position:relative'></div>"
            + "</div>"

            + "{{else ~_getPosition()=='FFType1'}}"
            + "<div class='FFtype1' style='left:{{:ChildLeft+ChildWidth}}px;top:{{:ParentTop+" + taskbarMidpoint + "}}px;" +
                "width:{{:((ChildLeft + ChildWidth)-(ParentLeft + ParentWidth))+30}}px;height:{{:Height}}px;position:absolute'>"
            + "<div class='e-line' style='left:{{if ~_isMilestoneParent()}}{{:((ParentLeft + ParentWidth) - (ChildLeft + ChildWidth))-10}}px;{{else}}{{:((ParentLeft + ParentWidth) - (ChildLeft + ChildWidth))}}px;{{/if}}" +
                "width:{{if ~_isMilestoneParent()}}30px;{{else}}20px;{{/if}}" +
                "height:" + lineStroke + "px;background-color:" + lineColor + ";position:relative'></div>"
            + "<div class='e-line' style='left:{{if ~_isMilestoneParent()}}{{:((ParentLeft + ParentWidth) - (ChildLeft + ChildWidth))+20}}px;{{else}}{{:((ParentLeft + ParentWidth) - (ChildLeft + ChildWidth))+20}}px;{{/if}}width:" + lineStroke + "px;" +
                "height:{{:Height-" + lineStroke + "}}px;background-color:" + lineColor + ";" +
                "position:relative'></div>"
            + "<div class='e-line' style='left:{{if ~_isMilestone()}}4px;{{else}}10px;{{/if}}width:{{if ~_isMilestone()}}{{:((ParentLeft + ParentWidth) - (ChildLeft + ChildWidth))+16}}px;{{else}}{{:((ParentLeft + ParentWidth) - (ChildLeft + ChildWidth))+10}}px;{{/if}}" +
                "height:" + lineStroke + "px;background-color:" + lineColor + ";position:relative'></div>"
            + "<div class='connectorline-leftarrow' style='{{if ~_isMilestone()}}left:-6px;{{/if}}border-right: 10px solid " + lineColor + ";" +
                "top:{{:-6-" + lineStroke + "}}px;border-bottom-width:{{:5+" + lineStroke + "}}px;" +
                "border-top-width:{{:5+" + lineStroke + "}}px;width:0;height:0;position:relative'></div>"
            + "</div>"

            + "{{else ~_getPosition()=='FFType2'}}"
            + "<div style='left:{{:ParentLeft+ParentWidth}}px;top:{{:ParentTop+" + taskbarMidpoint + "}}px;" +
                "width:{{:((ChildLeft +ChildWidth)-(ParentLeft + ParentWidth))+20}}px;height:{{:Height}}px;position:absolute'>"
            + "<div class='e-line' style='{{if ~_isMilestoneParent()}}left:-6px;{{/if}}width:{{if ~_isMilestoneParent()}}{{:((ChildLeft +ChildWidth)-(ParentLeft + ParentWidth))+26}}px;{{else}}{{:((ChildLeft +ChildWidth)-(ParentLeft + ParentWidth))+20}}px;{{/if}}height:" + lineStroke + "px;" +
                "background-color:" + lineColor + ";position:relative'></div>"
            + "<div class='e-line' style='left:{{:((ChildLeft +ChildWidth)-(ParentLeft + ParentWidth))+20}}px;width:" + lineStroke + "px;" +
                "height:{{:Height-" + lineStroke + "}}px;background-color:" + lineColor + ";" +
                "position:relative'></div>"
            + "<div class='e-line' style='left:{{if ~_isMilestone()}}{{:((ChildLeft +ChildWidth)-(ParentLeft + ParentWidth))+4}}px;{{else}}{{:((ChildLeft +ChildWidth)-(ParentLeft + ParentWidth))+10}}px;{{/if}}" +
                "width:{{if ~_isMilestone()}}16px;{{else}}10px;{{/if}}" +
                "height:" + lineStroke + "px;background-color:" + lineColor + ";position:relative'></div>"
            + "<div class='connectorline-leftarrow' style='left:{{if ~_isMilestone()}}{{:((ChildLeft +ChildWidth)-(ParentLeft + ParentWidth))-6}}px;{{else}}{{:((ChildLeft +ChildWidth)-(ParentLeft + ParentWidth))}}px;{{/if}}" +
                "border-right: 10px solid " + lineColor + ";top:{{:-6-" + lineStroke + "}}px;" +
                "border-bottom-width:{{:5+" + lineStroke + "}}px;border-top-width:{{:5+" + lineStroke + "}}px;width:0;height:0;position:relative'></div>"
            + "</div>"

            + "{{else ~_getPosition()=='FFType3'}}"
            + "<div  style='left:{{:ChildLeft+ChildWidth}}px;top:{{:ChildTop+" + taskbarMidpoint + "}}px;" +
                "width:{{:((ParentLeft + ParentWidth) - (ChildLeft + ChildWidth))+20}}px;height:{{:Height}}px;position:absolute'>"
            + "<div class='connectorline-leftarrow' style='top:-12.5px;border-right: 10px solid " + lineColor + ";" +
                "top:{{:-6-" + lineStroke + "}}px;border-bottom-width:{{:5+" + lineStroke + "}}px;" +
                "border-top-width:{{:5+" + lineStroke + "}}px;width:0;height:0;position:relative'></div>"
            + "<div class='e-line' style='width:{{:((ParentLeft + ParentWidth) - (ChildLeft + ChildWidth))+20}}px;top:-12.5px;" +
                "height:" + lineStroke + "px;background-color:" + lineColor + ";position:relative'></div>"
            + "<div class='e-line' style='left:{{:((ParentLeft + ParentWidth) - (ChildLeft + ChildWidth))+20}}px;top:-12.5px;" +
                "width:" + lineStroke + "px;height:{{:Height-" + lineStroke + "}}px;background-color:" + lineColor + ";position:relative'></div>"
            + "<div class='e-line' style='left:{{:((ParentLeft + ParentWidth) - (ChildLeft + ChildWidth))}}px;top:-12.5px;width:20px;" +
                "height:" + lineStroke + "px;background-color:" + lineColor + ";position:relative'></div>"
            + "</div>"

            + "{{else ~_getPosition()=='FFType4'}}"
            + "<div  style='left:{{:ParentLeft+ParentWidth}}px;top:{{:ChildTop+" + taskbarMidpoint + "}}px;" +
                "width:{{:((ChildLeft +ChildWidth)-(ParentLeft + ParentWidth))+20}}px;height:{{:Height}}px;position:absolute'>"
            + "<div  class='connectorline-leftarrow' style='left:{{:((ChildLeft +ChildWidth)-(ParentLeft + ParentWidth))}}px;" +
                "border-right: 10px solid " + lineColor + ";top:{{:-6-" + lineStroke + "}}px;" +
                "border-bottom-width:{{:5+" + lineStroke + "}}px;border-top-width:{{:5+" + lineStroke + "}}px;width:0;height:0;" +
                "position:relative'></div>"
            + "<div class='e-line' style='left:{{:((ChildLeft +ChildWidth)-(ParentLeft + ParentWidth))+10}}px;top:-13px;width:10px;" +
                "height:" + lineStroke + "px;background-color:" + lineColor + ";position:relative'></div>"
            + "<div class='e-line' style='left:{{:((ChildLeft +ChildWidth)-(ParentLeft + ParentWidth))+20}}px;top:-13px;width:" + lineStroke + "px;" +
                "height:{{:Height-" + lineStroke + "}}px;background-color:" + lineColor + ";position:relative'></div>"
            + "<div class='e-line' style='width:{{:((ChildLeft +ChildWidth)-(ParentLeft + ParentWidth))+20}}px;top:-13px;height:" + lineStroke + "px;" +
                "background-color:" + lineColor + ";position:relative'></div>"
            + "</div>"

            + "{{else  ~_getPosition()=='SFType4'}}"//SF
            + "<div style='left:{{:ParentLeft-10}}px;top:{{:ChildTop+" + taskbarMidpoint + "}}px;width:{{:((ChildLeft + ChildWidth) - (ParentLeft ))+20}}px;" +
                "height:{{:Height}}px;position:absolute'>"
            + "<div class='connectorline-leftarrow' style='left:{{:((ChildLeft + ChildWidth) - (ParentLeft ))+10}}px;" +
                "top:-13px;border-right: 10px solid " + lineColor + ";top:{{:-6-" + lineStroke + "}}px;" +
                "border-bottom-width:{{:5+" + lineStroke + "}}px;border-top-width:{{:5+" + lineStroke + "}}px;width:0;height:0;" +
                "position:relative'></div>"
            + "<div class='e-line' style='left:{{:((ChildLeft + ChildWidth) - (ParentLeft ))+20}}px;top:-13px;width:10px;" +
                "height:" + lineStroke + "px;background-color:" + lineColor + ";position:relative'></div>"
            + "<div class='e-line' style='left:{{:((ChildLeft + ChildWidth) - (ParentLeft ))+30}}px;top:-13px;width:" + lineStroke + "px;" +
                "height:{{:Height-15-" + lineStroke + "}}px;background-color:" + lineColor + ";" +
                "position:relative'></div>"
            + "<div class='e-line' style='top:-13px;width:{{:((ChildLeft + ChildWidth) - (ParentLeft ))+30}}px;height:" + lineStroke + "px;" +
                "background-color:" + lineColor + ";position:relative'></div>"
            + "<div class='e-line' style='top:-13px;width:" + lineStroke + "px;height:{{:12-" + lineStroke + "}}px;background-color:" + lineColor + ";position:relative'></div>"
            + "<div class='e-line' style='top:-13px;width:10px;height:" + lineStroke + "px;background-color:" + lineColor + ";position:relative'></div>"
            + "</div>"

            + "{{else ~_getPosition()=='SFType3'}}"//SF
            + "<div style='left:{{:ChildLeft+ChildWidth}}px;top:{{:ChildTop+" + taskbarMidpoint + "}}px;" +
                "width:{{:((ParentLeft)-(ChildLeft + ChildWidth))}}px;height:{{:Height}}px;position:absolute'>"
            + "<div class='connectorline-leftarrow' style='top:-13px;border-right: 10px solid " + lineColor + ";" +
                "top:{{:-6-" + lineStroke + "}}px;border-bottom-width:{{:5+" + lineStroke + "}}px;" +
                "border-top-width:{{:5+" + lineStroke + "}}px;width:0;height:0;position:relative'></div>"
            + "<div class='e-line' style='left:10px;top:-13px;width:10px;height:" + lineStroke + "px;" +
                "background-color:" + lineColor + ";position:relative'></div>"
            + "<div class='e-line' style='left:20px;top:-13px;width:" + lineStroke + "px;height:{{:Height-" + lineStroke + "}}px;" +
                "background-color:" + lineColor + ";position:relative'></div>"
            + "<div class='e-line' style='left:20px;top:-13px;width:{{:(ChildLeft-(ParentLeft + ParentWidth + 20))+" + lineStroke + "}}px;" +
                "height:" + lineStroke + "px;background-color:" + lineColor + ";position:relative'></div>"
            + "</div>"

            + "{{else  ~_getPosition()=='SFType1'}}"//SF
            + "<div style='left:{{:ParentLeft-10}}px;top:{{:ParentTop+" + taskbarMidpoint + "}}px;" +
                "width:{{:ChildLeft+ChildWidth+20}}px;height:{{:Height}}px;position:absolute'>"
            + "<div class='e-line' style='width:10px;height:" + lineStroke + "px;background-color:" + lineColor + ";" +
                "position:relative'></div>"
            + "<div class='e-line' style='width:" + lineStroke + "px;height:{{:Height-15-" + lineStroke + "}}px;" +
                "background-color:" + lineColor + ";position:relative'></div>"
            + "<div class='e-line' style='width:{{:((ChildLeft + ChildWidth) - (ParentLeft ))+30}}px;height:" + lineStroke + "px;" +
                "background-color:" + lineColor + ";position:relative'></div>"
            + "<div class='e-line' style='left:{{:((ChildLeft + ChildWidth) - (ParentLeft ))+30}}px;width:" + lineStroke + "px;" +
                "height:{{:12-" + lineStroke + "}}px;background-color:" + lineColor + ";position:relative'></div>"
            + "<div class='e-line' style='left:{{:((ChildLeft + ChildWidth) - (ParentLeft ))+20}}px;width:10px;height:" + lineStroke + "px;" +
                "background-color:" + lineColor + ";position:relative'></div>"
            + "<div class='connectorline-leftarrow' style='left:{{:((ChildLeft + ChildWidth) - (ParentLeft ))+10}}px;" +
                "border-right: 10px solid " + lineColor + ";top:{{:-6-" + lineStroke + "}}px;" +
                "border-bottom-width:{{:5+" + lineStroke + "}}px;border-top-width:{{:5+" + lineStroke + "}}px;" +
                "width:0;height:0;position:relative'></div>"
            + "</div>"

            + "{{else ~_getPosition()=='SFType2'}}"//SF
            + "<div  style='left:{{:ChildLeft+ChildWidth}}px;top:{{:ParentTop+" + taskbarMidpoint + "}}px;" +
                "width:{{:((ParentLeft) - (ChildLeft + ChildWidth))}}px;height:{{:Height}}px;position:absolute'>"
            + "<div class='e-line' style='left:{{:((ParentLeft) - (ChildLeft + ChildWidth))-10}}px;width:10px;height:" + lineStroke + "px;" +
                "background-color:" + lineColor + ";position:relative'></div>"
            + "<div class='e-line' style='left:{{:((ParentLeft) - (ChildLeft + ChildWidth))-10}}px;width:" + lineStroke + "px;" +
                "height:{{:Height-" + lineStroke + "}}px;background-color:" + lineColor + ";" +
                "position:relative'></div>"
            + "<div class='e-line' style='left:10px;width:{{:((ParentLeft) - (ChildLeft + ChildWidth))-20}}px;height:" + lineStroke + "px;" +
                "background-color:" + lineColor + ";position:relative'></div>"
            + "<div class='connectorline-leftarrow' style='border-right: 10px solid " + lineColor + ";" +
                "top:{{:-6-" + lineStroke + "}}px;border-bottom-width:{{:5+" + lineStroke + "}}px;border-top-width:{{:5+" + lineStroke + "}}px;width:0;height:0;position:relative'></div>"
            + "</div>"

            + "{{/if}}";

            connectorContainer += div;
            connectorContainer += "</div>";

            var templates = {};
            templates[this._id + "ConnectorLineTemplate"] = connectorContainer;
            $.templates(templates);
        },

        _isTaskMilestone: function () {
            if (this.data.milestoneChild)
                return true;
            else
                return false;
            
        },

        _isMilestoneParent: function () {
            if (this.data.milestoneParent)
                return true;
            else
                return false;
        },

        _getInnerChildWidthSSType2: function () {
            if ((this.data.ParentLeft + this.data.ParentWidth) < this.data.ChildLeft) {
                return 10;
            }
            if (this.data.ParentLeft == this.data.ChildLeft) {
                return 20;
            }
            return (this.data.ChildLeft - this.data.ParentLeft);
        },

        _getInnerElementLeftSSType2: function () {
            if (this.data.ParentLeft == this.data.ChildLeft) {
                return (this.data.ParentLeft - 20);
            }
            return (this.data.ParentLeft - 10);
        },

        _getInnerElementWidthSSType2: function () {
            if (this.data.ParentLeft == this.data.ChildLeft) {
                return 10;
            }
            return (this.data.ChildLeft - this.data.ParentLeft);
        },


        _getContainerWidthSSType2: function () {
            if (this.data.ParentLeft == this.data.ChildLeft) {
                return 20;
            }
            return (this.data.ChildLeft - this.data.ParentLeft);
        },

        _getPredecessorTop: function () {
            if (this.data.ParentTop > this.data.ChildTop)
                return true;
            else
                return false;
        },

        _getParentPosition: function () {
            if (this.data.ParentTop < this.data.ChildTop) {

                if (this.data.Type == "FF") {
                    if (this.data.ChildLeft >= this.data.ParentLeft ||
                        ((this.data.ChildLeft + this.data.ChildWidth) >= this.data.ParentLeft) && this.data.milestoneParent) {
                        return "FFType2";
                    }
                    else {
                        return "FFType1";
                    }
                }

                else if ((this.data.ParentLeft < this.data.ChildLeft) && (this.data.ChildLeft > (this.data.ParentLeft + this.data.ParentWidth + 25))) {
                    if (this.data.Type == "FS") return "FSType1";
                    if (this.data.Type == "SF") return "SFType1";
                    else if (this.data.Type == "SS") return "SSType2";
                    else if (this.data.Type == "FF") return "FFType2";
                }
                else if ((this.data.ParentLeft < this.data.ChildLeft && (this.data.ChildLeft < (this.data.ParentLeft + this.data.ParentWidth)))
                    || (this.data.ParentLeft == this.data.ChildLeft || this.data.ParentLeft > this.data.ChildLeft)) {

                    if (this.data.ParentLeft > (this.data.ChildLeft + this.data.ChildWidth + 25)) {
                        if (this.data.Type == "SF") return "SFType2";
                    }

                    if (this.data.ParentLeft > this.data.ChildLeft) {
                        if (this.data.Type == "SS") return "SSType1";
                        if (this.data.Type == "SF") return "SFType1";
                        if (this.data.Type == "FF") return "FFType1";
                    }
                    else if ((this.data.ChildLeft + this.data.ChildWidth) > (this.data.ParentLeft + this.data.ParentWidth)) {
                        if (this.data.Type == "FF") return "FFType2";
                    }
                    if (this.data.Type == "FS") return "FSType2";
                    else if (this.data.Type == "SS") return "SSType2";
                    else if (this.data.Type == "FF") return "FFType1";
                    else if (this.data.Type == "SF") return "SFType1";
                }
                else if ((this.data.ParentLeft + 25) < this.data.ChildLeft) {

                    if (this.data.Type == "FS") return "FSType2";
                    else if (this.data.Type == "FF") return "FFType2";

                }

            }
            else if (this.data.ParentTop > this.data.ChildTop) {
                if ((this.data.ParentLeft < this.data.ChildLeft) && (this.data.ChildLeft > (this.data.ParentLeft + this.data.ParentWidth))) {
                    if (this.data.Type == "FS") return "FSType4";
                    if (this.data.ParentLeft < this.data.ChildLeft || ((this.data.ChildLeft + this.data.ChildWidth) > (this.data.ParentLeft + this.data.ParentWidth))) {
                        if (this.data.Type == "SS") return "SSType4";
                        if (this.data.Type == "FF") return "FFType4";
                    }
                    else if ((this.data.ChildLeft + this.data.ChildWidth) > (this.data.ParentLeft + this.data.ParentWidth)) {
                        if (this.data.Type == "FF") return "FFType4";
                    }
                }
                else if ((this.data.ParentLeft < this.data.ChildLeft && (this.data.ChildLeft < (this.data.ParentLeft + this.data.ParentWidth)))
                    || (this.data.ParentLeft == this.data.ChildLeft || this.data.ParentLeft > this.data.ChildLeft)) {
                    if ((this.data.ChildLeft + this.data.ChildWidth) <= (this.data.ParentLeft + this.data.ParentWidth)) {
                        if (this.data.Type == "FF") return "FFType3";
                        if (this.data.Type == "SF") return "SFType3";
                    }
                    else if ((this.data.ChildLeft + this.data.ChildWidth) > (this.data.ParentLeft + this.data.ParentWidth)) {
                        if (this.data.Type == "FF") return "FFType4";
                        if (this.data.Type == "SF") return "SFType4";
                    }
                    if (this.data.Type == "FS") return "FSType3";
                    else if (this.data.Type == "SS") return "SSType3";
                }
                else if (this.data.ParentLeft < this.data.ChildLeft) {

                    if (this.data.Type == "FS") return "FSType3";

                }
            }
            return false;
        },

        _getWidth: function () {
            return ((this.data.ChildLeft + this.data.ChildWidth) < (this.data.ParentLeft + this.data.ParentWidth)) ?
                this.data.ParentWidth : (this.data.ChildWidth + this.data.ParentWidth);
        },


        //renders Weekends in chart
        _renderWeekends: function () {
            var proxy = this,
                left = 0,
                width = (proxy.model.perDayWidth) + 1,
                height = ((proxy.model.ganttRecords.length) * this.model.rowHeight),
                count = proxy.model.scheduleWeeks.length,
                $sunday, $saturday,
                $weekendContainer = ej.buildTag("div.e-secondary_canvas", "", { 'height': height + 'px' }, {});

            for (var i = 0; i < count; i++) {
                $sunday = ej.buildTag("div.e-weekends", "", {
                    'left': left + 'px',
                    'width': width + 'px',
                    'height': height + 'px',
                    'background-color': this.model.weekendBackground
                }, {});
                $saturday = ej.buildTag("div.e-weekends", "", {
                    'left': (left + (proxy.model.perDayWidth * 6)) + 'px',
                    'width': width + 'px',
                    'height': height + 'px',
                    'background-color': this.model.weekendBackground
                }, {});
                $weekendContainer.append($sunday);
                $weekendContainer.append($saturday);
                left = left + this._scheduleWeekWidth;
            }

            proxy._$weekendsContainer.append($weekendContainer);
            proxy._$bodyContent.append(proxy._$weekendsContainer);
        },



        //Renders StripLines for given striplines collection object
        _renderStripLines: function (stripLines) {

            var proxy = this,
                height = ((proxy.model.ganttRecords.length) * this.model.rowHeight),
                stripLineCount = stripLines.length, day, left,
                projectStartDate = proxy.model.projectStartDate,
                dayWidth = this.model.perDayWidth,
                stripLineContainer = ej.buildTag("div.e-stripLines", "", { 'height': height + 'px' }, {});

            for (var i = 0; i < stripLineCount; i++) {

                day = this._getDateType(stripLines[i].day);
                left = ((day - this._getDateType(projectStartDate)) / 86400000) * dayWidth;
                stripLines[i].lineWidth = stripLines[i].lineWidth > 30 ? 30 : stripLines[i].lineWidth;

                var stripline = ej.buildTag("div.e-stripline#stripline" + [i] + "", "", {
                    'left': left + 'px',
                    'width': '1px',
                    'height': height + 'px',
                    'border-left-width': stripLines[i].lineWidth + 'px',
                    'border-left-color': stripLines[i].lineColor,
                    'border-left-style': stripLines[i].lineStyle
                }, {});

                var innerspan = ej.buildTag("span.e-striplinespan", stripLines[i].label, {
                    'background-color': stripLines[i].lineColor,
                }, {});

                if (stripLines[i].day && stripLines[i].day.length > 0) {
                    if (stripLines[i].label && stripLines[i].label.length > 0) {
                        stripline.append(innerspan);
                    }
                    stripLineContainer.append(stripline);
                }
            }

            proxy._$secondaryCanvas.append(stripLineContainer);
            proxy._$bodyContent.append(proxy._$secondaryCanvas);
        },


        //returns progress width for the taskbar
        _getProgressWidth: function (parentwidth, percent) {
            return (parentwidth * percent) / 100;
        },


        //returns progress percentage of the taskbar
        _getProgressPercent: function (parentwidth, progresswidth) {
            return Math.ceil(((progresswidth / parentwidth) * 100).toFixed(2));
        },


        //renders holidays for holiday collection object
        _renderHoliday: function (holiday) {

            var proxy = this, scheduleHeaderType = proxy.model.scheduleHeaderSettings.scheduleHeaderType,
                date = this._getDateType(holiday.day),
                weekCount = proxy.model.scheduleWeeks.length,
                monthcount = proxy.model.scheduleMonths.length,
                yearcount = proxy.model.scheduleYears.length,
                daycount=proxy.model.scheduleDays.length,
                prevDate = 0,
                left = 0, textLength, text, $holidayContainer, $holidaySpan,
                ganttRecords = proxy.model.ganttRecords.length, height;
            if (scheduleHeaderType == "week") {
                for (var i = 0; i < weekCount; i++) {
                    var givendate = (date - this._getDateType(proxy.model.scheduleWeeks[i])) / 86400000;
                    if (givendate < 0) {
                        left = prevDate * this.model.perDayWidth;
                        //prevDate = 0;
                        break;
                    }
                    if (prevDate == 0)
                        prevDate = givendate;
                }
            }
            else if (scheduleHeaderType == "month") {
                for (var i = 0; i < monthcount; i++) {                  
                    var givenmonth = (date - proxy.model.scheduleMonths[i]) / 86400000;                   
                    if (givenmonth < 0) {
                        left = prevDate * this.model.perDayWidth;
                        //prevDate = 0;
                        break;
                    }
                    if (prevDate == 0)
                        prevDate = givenmonth;
                }
            }
            else if (scheduleHeaderType == "year") {
                for (var i = 0; i < yearcount; i++) {
                    var givenyear = (date - proxy.model.scheduleYears[i]) / 86400000;
                    if (givenyear < 0) {
                        left = prevDate * this.model.perDayWidth; 
                        //prevDate = 0;
                        break;
                    }
                    if (prevDate == 0)
                        prevDate = givenyear+22;
                }
            }
                else if (scheduleHeaderType == "day") {
                    for (var i = 0; i < daycount; i++) {
                        var givenday = (date - this._getDateType(proxy.model.scheduleDays[i])) / 86400000;
                        if (givenday < 0) {
                            left = prevDate * this.model.perDayWidth;
                            //prevDate = 0;
                            break;
                        }
                        if (prevDate == 0)
                            prevDate = givenday;
                    }
            }
            height = (ganttRecords * this.model.rowHeight);
            textLength = holiday.label != null ? holiday.label.length : 0;
            if (holiday.background == null)
                holiday.background = "white";
            if (proxy.model.scheduleHeaderSettings.scheduleHeaderType != "day") {
                text = ej.buildTag("div.e-holiday-label", holiday.label, {
                    'top': (proxy._getHolidayLabelHeight(textLength)) + 'px',
                }, {});
            }
            else
            {
                text = ej.buildTag("div.e-holiday-label", holiday.label, {
                    'bottom': (proxy._getHolidayLabelHeight(textLength)) + 'px',
                }, {});
            }
            $holidayContainer = ej.buildTag("div.e-secondary-canvas-holiday");
            $holidaySpan = ej.buildTag("div.e-holidays", "", {
                'left': left + 'px',
                'width': proxy.model.perDayWidth + 'px',
                'height': height + 'px',
                'background-color': holiday.background
            }, {});

            if (holiday.label != null)
                $holidaySpan.append(text);
            $holidayContainer.append($holidaySpan);

            proxy._$secondaryCanvas.append($holidayContainer);
            proxy._$bodyContent.append(proxy._$secondaryCanvas);
        },

        _getHolidayLabelHeight: function (textlength) {
            var proxy = this, model = proxy.model,
                containerHeight = proxy._getHeight(),
                totalHeight = model.ganttRecords.length * proxy.model.rowHeight;
            if (containerHeight > 0)
                return (containerHeight / 2);
            else if (proxy._viewportHeight < totalHeight)
                return (proxy._viewportHeight / 2);
            else
                return totalHeight / 2;

        },

        // all events bound using this._on will be unbind automatically
        _destroy: function () {

            var proxy = this;
            proxy.element.empty().removeClass("e-ganttchart-core e-ganttchart" + proxy.model.cssClass);

        },
    });

    ej.GanttChart.contextMenuItem = {
        taskDetails: "Task Details....",
        add: "Add new task",
        indent: "Indent",
        outdent: "Outdent",
        Delete: "Delete",
    };

    jQuery.uaMatch = function (ua) {
        ua = ua.toLowerCase();

        var match = /(chrome)[ \/]([\w.]+)/.exec(ua) ||
            /(webkit)[ \/]([\w.]+)/.exec(ua) ||
            /(opera)(?:.*version|)[ \/]([\w.]+)/.exec(ua) ||
            /(msie) ([\w.]+)/.exec(ua) ||
            ua.indexOf("compatible") < 0 && /(mozilla)(?:.*? rv:([\w.]+)|)/.exec(ua) ||
            [];

        return {
            browser: match[1] || "",
            version: match[2] || "0"
        };
    };

})(jQuery, Syncfusion);;
/**
* @fileOverview Plugin to style the Html AutoComplete elements
* @copyright Copyright Syncfusion Inc. 2001 - 2013. All rights reserved.
*  Use of this code is subject to the terms of our license.
*  A copy of the current license can be obtained at any time by e-mailing
*  licensing@syncfusion.com. Any infringement will be prosecuted under
*  applicable laws. 
* @version 12.1 
* @author <a href="mailto:licensing@syncfusion.com">Syncfusion Inc</a>
*/

(function ($, ej, undefined) {

    ej.widget("ejRadialMenuBase", "ej.RadialMenuBase", {
       
        _initialization: function () {
            this._svgLink = "http://www.w3.org/2000/svg";
            this._menuItems = $(this.element.children()).clone();
            $(this.element.children()).remove();
            this._menuCount = $(this._menuItems.children()).length;
            this._startXY = (this.model.width / 2);
        },
        /**
        * To create SVG element	
        * @private
        */
        _createSVGElement: function () {
            var svgObj = document.createElementNS(this._svgLink, 'svg');
            if (this.model.width != null) {
                svgObj.setAttribute("width", this.model.width);
                svgObj.setAttribute("height", this.model.width);
            }
            return svgObj;
        },
        /**
        * To create group element	
        * @private
        */
        _createGroupElement: function () {
            var group = document.createElementNS(this._svgLink, "g");
            return group;
        },
        /**
        * To create Image element	
        * @private
        */
        _createImageElement: function () {
            var image = document.createElementNS(this._svgLink, "image");
            return image;
        },
        /**
        * To create Text element	
        * @private
        */
        _createTextElement: function () {
            var text = document.createElementNS(this._svgLink, "text");
            return text;
        },
        /**
        * To create path element 	
        * @private
        */
        _createPathElement: function () {
            var path = document.createElementNS(this._svgLink, "path");
            return path;
        },
        /**
        * To create polygon element	
        * @private
        */
        _createPolygonElement: function () {
            var polygon = document.createElementNS(this._svgLink, "polygon");
            return polygon;
        },
        /**
        * To create circle element	
        * @private
        */
        _createCircleElement: function () {
            var circle = document.createElementNS(this._svgLink, "circle");
            return circle;
        },
        _calculateRadialArc: function (menuCount, size, mRadius, Arcdradius, polyRadius) {
            var itemCurrentPos = 0,
                startAngle = 2 * Math.PI * itemCurrentPos,
                endAngle = 2 * Math.PI * (itemCurrentPos + (10 / (menuCount * 10))),
                radius = 0.5 * size * Math.min(this.model.width, this.model.width),
                menuItemRadius = mRadius * size * Math.min(this.model.width, this.model.width),
                dradius = Arcdradius * radius,
                polygonRadius = polyRadius * radius,
                degPoint = 360 / menuCount,
                degree = degPoint / 2;
            var x1 = [], y1 = [], x2 = [], y2 = [], midx = [], midy = [], dStartX = [], dStartY = [], dEndX = [], dEndY = [], dmidx = [], dmidy = [], deg = [], longArc = [];
            for (var i = 0; i < menuCount; i++) {
                var pathStartAngle = -.5 * Math.PI;
                startAngle = startAngle + pathStartAngle;
                endAngle = endAngle + pathStartAngle - 0.000001;
                longArc[i] = endAngle - startAngle < Math.PI ? 0 : 1;
                var midAngle = (startAngle + endAngle) / 2;
                x1[i] = this._startXY + radius * Math.cos(startAngle);
                y1[i] = this._startXY + radius * Math.sin(startAngle);
                x2[i] = this._startXY + radius * Math.cos(endAngle);
                y2[i] = this._startXY + radius * Math.sin(endAngle);
                midx[i] = this._startXY + menuItemRadius * Math.cos(midAngle);
                midy[i] = this._startXY + menuItemRadius * Math.sin(midAngle);
                dStartX[i] = this._startXY + dradius * Math.cos(startAngle);
                dStartY[i] = this._startXY + dradius * Math.sin(startAngle);
                dEndX[i] = this._startXY + dradius * Math.cos(endAngle);
                dEndY[i] = this._startXY + dradius * Math.sin(endAngle);
                dmidx[i] = this._startXY + polygonRadius * Math.cos(midAngle);
                dmidy[i] = this._startXY + polygonRadius * Math.sin(midAngle);
                deg[i] = degree;
                itemCurrentPos += 10 / (menuCount * 10);
                startAngle = 2 * Math.PI * itemCurrentPos;
                endAngle = 2 * Math.PI * (itemCurrentPos + (10 / (menuCount * 10)));
                degree += degPoint;
            }
            return ({ x1: x1, y1: y1, x2: x2, y2: y2, midx: midx, midy: midy, dStartX: dStartX, dStartY: dStartY, dEndX: dEndX, dEndY: dEndY, dmidx: dmidx, dmidy: dmidy, deg: deg, longArc: longArc, radius: radius, dradius: dradius });
        },
        _renderRadial: function () {
            this._radialSVG = this._createSVGElement();
            this._radialChildSVG = this._createSVGElement();
            this._radial = ej.buildTag("div." + this._prefix + "radial " + this._prefix + "abs " + this.model.imageClass + "").css({ "left": this._startXY - 22 + "px", "top": this._startXY - 22 + "px" });
            this._radialContainer = ej.buildTag("div." + this._prefix + "rel").css({ "height": this.model.width, "width": this.model.width });
            this.element.append(this._radialContainer.append(this._radial)).addClass("" + this._prefix + "radialmenu " + this._prefix + "overlow " + this._prefix + "user-select");
            this._renderRadialMenu();
        },
        _renderRadialMenu: function () {
            this._childArcGroup = this._createGroupElement();
            this._childArcGroup.setAttribute("id", "childarcgroup");
            this._polygonGroup = this._createGroupElement();
            this._polygonGroup.setAttribute("id", "polygongroup");
            this._polygonGroup.setAttribute("transform", "translate(-10,-10)");
            this._radialArcGroup = this._createGroupElement();
            this._menuItemsGroup = this._createGroupElement();
            this._menuItemsGroup.setAttribute("id", "menuitemsgroup");
            this._menuItemsGroup.setAttribute("transform", "translate(-10,-10)");
            this._menuItemsArcPaths = this._createGroupElement();
            this._menuItemsArcPaths.setAttribute("id", "menuitemsarcpaths");
            this._menuItemsPaths = this._createGroupElement();
            this._radialSVGDiv = ej.buildTag("div#radialsvgdiv." + this._prefix + "abs").css("height", this.model.width);
            this._radialCircleSVGDiv = ej.buildTag("div#radialcirclesvgdiv." + this._prefix + "abs " + this._prefix + "displaynone");
            this._radialChildSVGDiv = ej.buildTag("div#radialchildsvgdiv." + this._prefix + "abs " + this._prefix + "displaynone");
            var pointsVal = this._calculateRadialArc(this._menuCount, 1, 0.28, 0.8, 0.95);
            var pointsArcVal = this._calculateRadialArc(this._menuCount, 0.79, null, 0.97, null);
            var pointsItemVal = this._calculateRadialArc(this._menuCount, 0.8, null, 0.3, null);
            var direction, arcDirection, itemDirection;
            for (var i = 0; i < this._menuCount; i++) {
                direction = "M" + " " + pointsVal.x1[i] + " " + pointsVal.y1[i] + " " + "A" + " " + pointsVal.radius + " " + pointsVal.radius + " " + "0" + " " + pointsVal.longArc[i] + " " + "1" + " " + pointsVal.x2[i] + " " + pointsVal.y2[i] + " " + "L" + " " + pointsVal.dEndX[i] + " " + pointsVal.dEndY[i] + " " + "A" + " " + pointsVal.dradius + " " + pointsVal.dradius + " " + "1" + " " + pointsVal.longArc[i] + " " + "0" + " " + pointsVal.dStartX[i] + " " + pointsVal.dStartY[i] + " " + "z";
                arcDirection = "M" + " " + pointsArcVal.x1[i] + " " + pointsArcVal.y1[i] + " " + "A" + " " + pointsArcVal.radius + " " + pointsArcVal.radius + " " + "0" + " " + pointsArcVal.longArc[i] + " " + "1" + " " + pointsArcVal.x2[i] + " " + pointsArcVal.y2[i] + " " + "L" + " " + pointsArcVal.dEndX[i] + " " + pointsArcVal.dEndY[i] + " " + "A" + " " + pointsArcVal.dradius + " " + pointsArcVal.dradius + " " + "1" + " " + pointsArcVal.longArc[i] + " " + "0" + " " + pointsArcVal.dStartX[i] + " " + pointsArcVal.dStartY[i] + " " + "z";
                itemDirection = "M" + " " + pointsItemVal.x1[i] + " " + pointsItemVal.y1[i] + " " + "A" + " " + pointsItemVal.radius + " " + pointsItemVal.radius + " " + "0" + " " + pointsItemVal.longArc[i] + " " + "1" + " " + pointsItemVal.x2[i] + " " + pointsItemVal.y2[i] + " " + "L" + " " + pointsItemVal.dEndX[i] + " " + pointsItemVal.dEndY[i] + " " + "A" + " " + pointsItemVal.dradius + " " + pointsItemVal.dradius + " " + "1" + " " + pointsItemVal.longArc[i] + " " + "0" + " " + pointsItemVal.dStartX[i] + " " + pointsItemVal.dStartY[i] + " " + "z";
                if ($($(this._menuItems.children()[i]).find("ul")).length != 0) {
                    var childPath = this._createPathElement();
                    childPath.setAttribute("class", "" + this._prefix + "childdefault");
                    childPath.setAttribute("index", i);
                    childPath.setAttribute("d", direction);
                    var transform = pointsVal.deg[i] > 270 ? "translate(" + (pointsVal.dmidx[i] + 5) + "," + (pointsVal.dmidy[i] + 15) + ")" + "rotate(" + pointsVal.deg[i] + ")" : "translate(" + (pointsVal.dmidx[i] + 10) + "," + (pointsVal.dmidy[i] + 10) + ")" + "rotate(" + pointsVal.deg[i] + ")";
                    var polygon = this._createPolygonElement();
                    polygon.setAttribute("points", "10,10 0,10 5,5");
                    polygon.setAttribute("index", i);
                    polygon.setAttribute("fill", "#FFFFFF");
                    polygon.setAttribute("transform", transform);
                    $(this._childArcGroup).append(childPath);
                    $(this._polygonGroup).append(polygon);
                }
                var path = this._createPathElement();
                path.setAttribute("class", "" + this._prefix + "default");
                path.setAttribute("index", i);
                path.setAttribute("d", direction);
                var arcPath = this._createPathElement();
                arcPath.setAttribute("class", "" + this._prefix + "bgcolor");
                arcPath.setAttribute("index", i);
                arcPath.setAttribute("d", arcDirection);
                $(this._menuItemsArcPaths).append(arcPath);
                var itemsPath = this._createPathElement();
                itemsPath.setAttribute("class", "" + this._prefix + "bgcolor");
                itemsPath.setAttribute("index", i);
                itemsPath.setAttribute("d", itemDirection);
                $(this._menuItemsPaths).append(itemsPath);
                $(this._radialArcGroup).append(path);
                var imgTextGroup = this._createGroupElement();
                imgTextGroup.setAttribute("cursor", "context-menu");
                imgTextGroup.setAttribute("index", i);
                var img = this._createImageElement(),
                    imgPath = $(this._menuItems.children("li")[i]).attr("data-ej-imagepath"),
                    imgName = $(this._menuItems.children("li")[i]).attr("data-ej-imagename"),
                    imgURL = $(this._menuItems.children("li")[i]).attr("data-ej-imageurl"),
                    imgLoc = imgURL ? imgURL : (this.model.renderMode.toLowerCase() != "flat" ? imgPath + "/" + this.model.renderMode.toLowerCase() + "/" + this.model.theme.toLowerCase() + "/" + imgName : imgPath + "/" + this.model.renderMode.toLowerCase() + "/" + imgName);
                img.setAttribute("width", 20);
                img.setAttribute("height", 20);
                img.setAttributeNS('http://www.w3.org/1999/xlink', 'href', imgLoc);
                img.setAttribute("x", pointsVal.midx[i]);
                img.setAttribute("y", pointsVal.midy[i] - 10);
                $(imgTextGroup).append(img);
                var text = this._createTextElement();
                text.textContent = $(this._menuItems.children("li")[i]).attr("data-ej-text");
                text.setAttribute("x", pointsVal.midx[i] + 10);
                text.setAttribute("y", (pointsVal.midy[i] + 25));
                text.setAttribute("style", "text-anchor: middle;font-size: 12px;font-family: Segoe UI;");
                text.setAttribute("class", "" + this._prefix + "textcolor");
                $(imgTextGroup).append(text);
                $(this._menuItemsGroup).append(imgTextGroup);
            }
            var circle = this._createCircleElement();
            circle.setAttribute("cx", this._startXY);
            circle.setAttribute("cy", this._startXY);
            circle.setAttribute("r", this._startXY / 3.5);
            circle.setAttribute("class", "" + this._prefix + "bgcolor");
            $(this._radialSVG).append(this._radialArcGroup);
            this._menuItemsArcGroup = $(this._radialArcGroup).clone();
            $(this._radialChildSVG).append(this._menuItemsPaths).append(this._menuItemsGroup).append(this._menuItemsArcGroup).append(this._childArcGroup).append(this._polygonGroup).append(this._menuItemsArcPaths).append(circle);
            this._radialContainer.append(this._radialSVGDiv.append($(this._radialCircleSVGDiv).append(this._radialSVG)).append($(this._radialChildSVGDiv).append(this._radialChildSVG)));
            this._arcEnterLeaveHandler = $.proxy(this._arcEnterLeaveHandler, this);
            this._arcClickHandler = $.proxy(this._arcClickHandler, this);
            this._menuItemHandler = $.proxy(this._menuItemHandler, this);
            this._menuItemArcClickHandler = $.proxy(this._menuItemArcClickHandler, this);
            this._menuItemTouchStartHandler = $.proxy(this._menuItemStartHandler, this);
            this._menuItemArcTouchStartHandler = $.proxy(this._menuItemArcStartHandler, this);
            ej.listenEvents([$(this._childArcGroup).children(), $(this._childArcGroup).children(), $(this._childArcGroup).children()], ["mouseenter", "mouseleave", ej.tapEvent()], [this._arcEnterLeaveHandler, this._arcEnterLeaveHandler, this._arcClickHandler], false);
            ej.listenEvents([$(this._polygonGroup).children(), $(this._polygonGroup).children()], ["mouseenter", ej.tapEvent()], [this._arcEnterLeaveHandler, this._arcClickHandler], false);
            ej.listenEvents([$(this._menuItemsGroup).children(), $(this._menuItemsGroup).children(), $(this._menuItemsGroup).children()], ["mouseenter", ej.tapEvent(), ej.startEvent()], [this._menuItemHandler, this._menuItemHandler, this._menuItemTouchStartHandler], false);
            ej.listenEvents([$(this._menuItemsPaths).children(), $(this._menuItemsPaths).children(), $(this._menuItemsPaths).children(), $(this._menuItemsArcGroup).children(), $(this._menuItemsArcGroup).children()], ["mouseenter", "mouseleave", ej.tapEvent(), ej.tapEvent(), ej.startEvent()], [this._menuItemHandler, this._menuItemHandler, this._menuItemHandler, this._menuItemArcClickHandler, this._menuItemArcTouchStartHandler], false);
        },
        _arcClickHandler: function (e) {
            this._menuChildSVG = this._createSVGElement();
            this._radialChildSVGDiv.removeClass("" + this._prefix + "radialshow").addClass("" + this._prefix + "scalehide");
            this._radialChildAnimate();
            this._radial.removeClass(this.model.imageClass).addClass("" + this._prefix + "backarrow " + this.model.backImageClass + "");
            this._index = parseInt($(e.target).attr("index"));
            var child = $(this._menuItems.children()[this._index]).children();
            var childcount = $(child.children()).length;
            this._childItemsGroup = this._createGroupElement();
            this._childItemsGroup.setAttribute("id", "childitemsgroup");
            this._childItemsGroup.setAttribute("transform", "translate(-10,-10)");
            this._childItemArcGroup = this._createGroupElement();
            this._childItemsArcPaths = this._createGroupElement();
            this._childItemsArcPaths.setAttribute("id", "childitemsarcpaths");
            this._childItemsPaths = this._createGroupElement();
            var direction, arcDirection, itemDirection;
            var pointsVal = this._calculateRadialArc(childcount, 1, 0.28, 0.8, null);
            var pointsArcVal = this._calculateRadialArc(childcount, 0.79, null, 0.97, null);
            var pointsItemVal = this._calculateRadialArc(childcount, 0.8, null, 0.3, null);
            for (var i = 0; i < childcount; i++) {
                direction = "M" + " " + pointsVal.x1[i] + " " + pointsVal.y1[i] + " " + "A" + " " + pointsVal.radius + " " + pointsVal.radius + " " + "0" + " " + pointsVal.longArc[i] + " " + "1" + " " + pointsVal.x2[i] + " " + pointsVal.y2[i] + " " + "L" + " " + pointsVal.dEndX[i] + " " + pointsVal.dEndY[i] + " " + "A" + " " + pointsVal.dradius + " " + pointsVal.dradius + " " + "1" + " " + pointsVal.longArc[i] + " " + "0" + " " + pointsVal.dStartX[i] + " " + pointsVal.dStartY[i] + " " + "z";
                arcDirection = "M" + " " + pointsArcVal.x1[i] + " " + pointsArcVal.y1[i] + " " + "A" + " " + pointsArcVal.radius + " " + pointsArcVal.radius + " " + "0" + " " + pointsArcVal.longArc[i] + " " + "1" + " " + pointsArcVal.x2[i] + " " + pointsArcVal.y2[i] + " " + "L" + " " + pointsArcVal.dEndX[i] + " " + pointsArcVal.dEndY[i] + " " + "A" + " " + pointsArcVal.dradius + " " + pointsArcVal.dradius + " " + "1" + " " + pointsArcVal.longArc[i] + " " + "0" + " " + pointsArcVal.dStartX[i] + " " + pointsArcVal.dStartY[i] + " " + "z";
                itemDirection = "M" + " " + pointsItemVal.x1[i] + " " + pointsItemVal.y1[i] + " " + "A" + " " + pointsItemVal.radius + " " + pointsItemVal.radius + " " + "0" + " " + pointsItemVal.longArc[i] + " " + "1" + " " + pointsItemVal.x2[i] + " " + pointsItemVal.y2[i] + " " + "L" + " " + pointsItemVal.dEndX[i] + " " + pointsItemVal.dEndY[i] + " " + "A" + " " + pointsItemVal.dradius + " " + pointsItemVal.dradius + " " + "1" + " " + pointsItemVal.longArc[i] + " " + "0" + " " + pointsItemVal.dStartX[i] + " " + pointsItemVal.dStartY[i] + " " + "z";
                var path = this._createPathElement();
                path.setAttribute("class", "" + this._prefix + "default");
                path.setAttribute("index", i);
                path.setAttribute("d", direction);
                $(this._childItemArcGroup).append(path);
                var arcPath = this._createPathElement();
                arcPath.setAttribute("class", "" + this._prefix + "bgcolor");
                arcPath.setAttribute("index", i);
                arcPath.setAttribute("d", arcDirection);
                $(this._childItemsArcPaths).append(arcPath);
                var itemsPath = this._createPathElement();
                itemsPath.setAttribute("class", "" + this._prefix + "bgcolor");
                itemsPath.setAttribute("index", i);
                itemsPath.setAttribute("d", itemDirection);
                $(this._childItemsPaths).append(itemsPath);
                var imgTextGroup = this._createGroupElement();
                imgTextGroup.setAttribute("cursor", "context-menu");
                imgTextGroup.setAttribute("index", i);
                var img = this._createImageElement(),
                    imgPath = $(child.children("li")[i]).attr("data-ej-imagepath"),
                    imgName = $(child.children("li")[i]).attr("data-ej-imagename"),
                    imgURL = $(child.children()[i]).attr("data-ej-imageurl"),
                  imgLoc = imgURL ? imgURL : (this.model.renderMode.toLowerCase() != "flat" ? imgPath + "/" + this.model.renderMode.toLowerCase() + "/" + this.model.theme.toLowerCase() + "/" + imgName : imgPath + "/" + this.model.renderMode.toLowerCase() + "/" + imgName);
                img.setAttribute("width", 20);
                img.setAttribute("height", 20);
                img.setAttributeNS('http://www.w3.org/1999/xlink', 'href', imgLoc);
                img.setAttribute("x", pointsVal.midx[i]);
                img.setAttribute("y", pointsVal.midy[i] - 10);
                $(imgTextGroup).append(img);
                var text = this._createTextElement();
                text.textContent = $(child.children()[i]).attr("data-ej-text");
                text.setAttribute("x", pointsVal.midx[i] + 10);
                text.setAttribute("y", (pointsVal.midy[i] + 25));
                text.setAttribute("style", "text-anchor: middle;font-size: 12px;font-family: Segoe UI;");
                text.setAttribute("class", "" + this._prefix + "textcolor");
                $(imgTextGroup).append(text);
                $(this._childItemsGroup).append(imgTextGroup);
            }
            var circle = this._createCircleElement();
            circle.setAttribute("cx", this._startXY);
            circle.setAttribute("cy", this._startXY);
            circle.setAttribute("r", this._startXY / 3.5);
            circle.setAttribute("class", "" + this._prefix + "bgcolor");
            this._menuChildSVGDiv = ej.buildTag("div#menuchildsvgdiv." + this._prefix + "abs " + this._prefix + "scaleshow");
            this._radialSVGDiv.append($(this._menuChildSVGDiv).append($(this._menuChildSVG).append(this._childItemsPaths).append(this._childItemsGroup).append(this._childItemArcGroup).append(this._childItemsArcPaths).append(circle)));
            this._childItemArcClickHandler = $.proxy(this._childItemArcClickHandler, this);
            this._childItemHandler = $.proxy(this._childItemHandler, this);
            this._childItemArcTouchStartHandler = $.proxy(this._childItemArcStartHandler, this);
            this._childItemTouchStartHandler = $.proxy(this._childItemStartHandler, this);
            ej.listenEvents([$(this._childItemArcGroup).children(), $(this._childItemArcGroup).children(), $(this._childItemsGroup).children(), $(this._childItemsGroup).children(), $(this._childItemsGroup).children(), $(this._childItemsPaths).children(), $(this._childItemsPaths).children(), $(this._childItemsPaths).children()], [ej.startEvent(), ej.tapEvent(), ej.startEvent(), ej.tapEvent(), "mouseenter", ej.tapEvent(), "mouseleave", "mouseenter"], [this._childItemArcTouchStartHandler, this._childItemArcClickHandler, this._childItemTouchStartHandler, this._childItemHandler, this._childItemHandler, this._childItemHandler, this._childItemHandler, this._childItemHandler], false);
        },
        _arcEnterLeaveHandler: function (e) {
            var index = parseInt($(e.target).attr("index"));
            if (e.type == "mouseenter") {
                if (e.currentTarget.tagName == "path")
                    $(e.target).attr("class", "" + this._prefix + "active");
                else
                    $($('#childarcgroup path[index=' + index + ']')).attr("class", "" + this._prefix + "active");
            }
            else {
                if (e.currentTarget.tagName == "path")
                    $(e.target).attr("class", "" + this._prefix + "childdefault");
                else
                    $($('#childarcgroup path[index=' + index + ']')).attr("class", "" + this._prefix + "childdefault");
            }
        },
        _menuItemHandler: function (e) {
            var index = parseInt($(e.currentTarget).attr("index"));
            if (e.type == "mouseenter")
                $($('#menuitemsarcpaths path[index=' + index + ']')).attr("class", "" + this._prefix + "childdefault");
            else if (e.type == "mouseleave")
                $($('#menuitemsarcpaths path[index=' + index + ']')).attr("class", "" + this._prefix + "bgcolor");
            else {
                var text = e.currentTarget.tagName == "path" ? $($($(this._menuItemsGroup).children()[index]).children()).text() : $(e.currentTarget).text();
                if (this.model[this._endEvent])
                    this._trigger(this._endEvent, { text: text, index: index, childIndex: null });
                if (this.model.select)
                    this._trigger("select", { text: text, index: index, childIndex: null });
            }
        },
        _childItemHandler: function (e) {
            var index = parseInt($(e.currentTarget).attr("index"));
            if (e.type == "mouseenter")
                $($('#childitemsarcpaths path[index=' + index + ']')).attr("class", "" + this._prefix + "childdefault");
            else if (e.type == "mouseleave")
                $($('#childitemsarcpaths path[index=' + index + ']')).attr("class", "" + this._prefix + "bgcolor");
            else {
                var text = e.currentTarget.tagName == "path" ? $($($(this._childItemsGroup).children()[index]).children()).text() : $(e.currentTarget).text();
                if (this.model[this._endEvent])
                    this._trigger(this._endEvent, { text: text, index: this._index, childIndex: index });
                if (this.model.select)
                    this._trigger("select", { text: text, index: this._index, childIndex: index });
            }
        },
        _childItemArcClickHandler: function (e) {
            var index = parseInt($(e.target).attr("index"));
            var text = $($($(this._childItemsGroup).children()[index]).children("text")).text();
            if (this.model[this._endEvent])
                this._trigger(this._endEvent, { text: text, index: this._index, childIndex: index });
            if (this.model.select)
                this._trigger("select", { text: text, index: this._index, childIndex: index });
        },
        _menuItemArcClickHandler: function (e) {
            var index = parseInt($(e.target).attr("index"));
            var text = $($($(this._menuItemsGroup).children()[index]).children("text")).text();
            if (this.model[this._endEvent])
                this._trigger(this._endEvent, { text: text, index: index, childIndex: null });
            if (this.model.select)
                this._trigger("select", { text: text, index: index, childIndex: null });
        },
        _menuItemStartHandler: function (e) {
            var index = parseInt($(e.currentTarget).attr("index"));
            var text = e.currentTarget.tagName == "path" ? $($($(this._menuItemsGroup).children()[index]).children()).text() : $(e.currentTarget).text();
            if (this.model[this._startEvent])
                this._trigger(this._startEvent, { text: text, index: index, childIndex: null });
        },
        _childItemStartHandler: function (e) {
            var index = parseInt($(e.currentTarget).attr("index"));
            var text = e.currentTarget.tagName == "path" ? $($($(this._childItemsGroup).children()[index]).children()).text() : $(e.currentTarget).text();
            if (this.model[this._startEvent])
                this._trigger(this._startEvent, { text: text, index: this._index, childIndex: index });

        },
        _childItemArcStartHandler: function (e) {
            var index = parseInt($(e.target).attr("index"));
            var text = $($($(this._childItemsGroup).children()[index]).children("text")).text();
            if (this.model[this._startEvent])
                this._trigger(this._startEvent, { text: text, index: this._index, childIndex: index });
        },
        _menuItemArcStartHandler: function (e) {
            var index = parseInt($(e.target).attr("index"));
            var text = $($($(this._menuItemsGroup).children()[index]).children("text")).text();
            if (this.model[this._startEvent])
                this._trigger(this._startEvent, { text: text, index: index, childIndex: null });
        },
        _setModel: function (options) {
            this._refresh();
        },
        _refresh: function () {
            this._destroy();
            this._init();
        },
        _destroy: function () {
            this.element.removeAttr("class style").children().remove();
            this.element.append(this._menuItems);
        }
    });

})(jQuery, Syncfusion);;
(function ($, ej, undefined) {
    /**
    * @namespace ej
    * @class ejRadialMenu
    * @requires jQuery
    * @requires ej.core
    * @requires ej.unobtrusive
    * @requires ej.data
    * @requires ej.touch
    * @classdesc Custom Design for Html radialmenu control.
    * @example
    * // Create radialmenu in obtrusive way
    * &lt;script&gt; 
    * $(function(){
    * $("#defaultradialmenu").ejRadialMenu({"width":300}); 
    * });
    * &lt;/script&gt;
    * &lt;div &gt;
    * &lt;br /&gt;
    * &lt;p&gt;
    * Syncfusion is the enterprise technology partner of choice for Windows development
    * &lt;/p&gt;
    * &lt;/div&gt;  
    * &lt;div id="defaultradialmenu"&gt;
    * &lt;ul&gt;
    * &lt;li data-ej-imageurl="../themes/sample/radialmenu/social.png"
    * data-ej-text="social"&gt;&lt;/li&gt;
    * &lt;li data-ej-imageurl="../themes/sample/radialmenu/music.png" 
    * data-ej-text="music"&gt;&lt;/li&gt;
    * &lt;li data-ej-imageurl="../themes/sample/radialmenu/direction.png" 
    * data-ej-text="direction"&gt;&lt;/li&gt;
    * &lt;li data-ej-imageurl="../themes/sample/radialmenu/message.png" 
    * data-ej-text="message"&gt;&lt;/li&gt;
    * &lt;li data-ej-imageurl="../themes/sample/radialmenu/browser.png" 
    * data-ej-text="browser"&gt;&lt;/li&gt;
    * &lt;/ul&gt;
    * &lt;/div&gt;
    */
    ej.widget("ejRadialMenu", "ej.RadialMenu", {
        _rootCSS: "e-radialmenu",
        // widget element will be automatically set in this
        element: null,
        // user defined model will be automatically set in this
        model: null,
        _setFirst: true,
        // default model
        defaults: {
            /**		
            * Specifies the width of radial menu
            * @default null
            * @type {int}
            * @example 
            * // Set Radialmenu width on initialization. 
            * //To set width API  
            * &lt;div &gt;
            * &lt;br /&gt;
            * &lt;p&gt;
            * Syncfusion is the enterprise technology partner of choice for Windows development
            * &lt;/p&gt;
            * &lt;/div&gt;  
            * &lt;div id="defaultradialmenu"&gt;
            * &lt;ul&gt;
            * &lt;li data-ej-imageurl="../themes/sample/radialmenu/social.png"
            * data-ej-text="social"&gt;&lt;/li&gt;
            * &lt;li data-ej-imageurl="../themes/sample/radialmenu/music.png" 
            * data-ej-text="music"&gt;&lt;/li&gt;
            * &lt;li data-ej-imageurl="../themes/sample/radialmenu/direction.png" 
            * data-ej-text="direction"&gt;&lt;/li&gt;
            * &lt;li data-ej-imageurl="../themes/sample/radialmenu/message.png" 
            * data-ej-text="message"&gt;&lt;/li&gt;
            * &lt;li data-ej-imageurl="../themes/sample/radialmenu/browser.png" 
            * data-ej-text="browser"&gt;&lt;/li&gt;
            * &lt;/ul&gt;
            * &lt;/div&gt;
            * &lt;script&gt;
            * $(function () {
            * $("#defaultradialmenu").ejRadialMenu({ "width":300});	
            * });
            * &lt;/script&gt;
            * @example 
            * //Get or set the Radialmenu Image, after initialization:
            * &lt;script&gt;
            * // Gets the Image API.		
            *  $("#radialmenu").ejRadialMenu ("option", "width");			
            * // Sets the Width API
            * $("#radialmenu").ejRadialMenu ("option", "width", 300);            
            * &lt;/script&gt;
            * @memberof ejRadialMenu
            * @instance
            */
            width: null,
            /**		
            * Renders the Image for the items using Class.
            * @default null
            * @type {string}
            * @example 
            * // Set Radialmenu Image on initialization. 
            * //To set image API  
            * &lt;div &gt;
            * &lt;br /&gt;
            * &lt;p&gt;
            * Syncfusion is the enterprise technology partner of choice for Windows development
            * &lt;/p&gt;
            * &lt;/div&gt;  
            * &lt;div id="defaultradialmenu"&gt;
            * &lt;ul&gt;
            * &lt;li data-ej-imageurl="../themes/sample/radialmenu/social.png"
            * data-ej-text="social"&gt;&lt;/li&gt;
            * &lt;li data-ej-imageurl="../themes/sample/radialmenu/music.png" 
            * data-ej-text="music"&gt;&lt;/li&gt;
            * &lt;li data-ej-imageurl="../themes/sample/radialmenu/direction.png" 
            * data-ej-text="direction"&gt;&lt;/li&gt;
            * &lt;li data-ej-imageurl="../themes/sample/radialmenu/message.png" 
            * data-ej-text="message"&gt;&lt;/li&gt;
            * &lt;li data-ej-imageurl="../themes/sample/radialmenu/browser.png" 
            * data-ej-text="browser"&gt;&lt;/li&gt;
            * &lt;/ul&gt;
            * &lt;/div&gt;
            * &lt;script&gt;
            * $(function () {
            * $("#defaultradialmenu").ejRadialMenu({ "imageClass":"e-m-radialimage","width":300});	
            * });
            * &lt;/script&gt;
            * @example 
            * //Get or set the Radialmenu Image, after initialization:
            * &lt;script&gt;
            * // Gets the Image API.		
            *  $("#radialmenu").ejRadialMenu ("option", "imageClass");			
            * // Sets the Width API
            * $("#radialmenu").ejRadialMenu ("option", "imageClass", "e-m-radialimage");            
            * &lt;/script&gt;
            * @memberof ejRadialMenu
            * @instance
            */
            imageClass: null,
            /**		
            * Renders the back button Image for the items.
            * @default null
            * @type {string}
            * @example 
            * // Set Radialmenu back button Image on initialization. 
            * //To set back button image API 
            * &lt;div &gt;
            * &lt;br /&gt;
            * &lt;p&gt;
            * Syncfusion is the enterprise technology partner of choice for Windows development
            * &lt;/p&gt;
            * &lt;/div&gt;  
            * &lt;div id="defaultradialmenu"&gt;
            * &lt;ul&gt;
            * &lt;li data-ej-imageurl="../themes/sample/radialmenu/social.png"
            * data-ej-text="social"&gt;&lt;/li&gt;
            * &lt;li data-ej-imageurl="../themes/sample/radialmenu/music.png" 
            * data-ej-text="music"&gt;&lt;/li&gt;
            * &lt;li data-ej-imageurl="../themes/sample/radialmenu/direction.png" 
            * data-ej-text="direction"&gt;&lt;/li&gt;
            * &lt;li data-ej-imageurl="../themes/sample/radialmenu/message.png" 
            * data-ej-text="message"&gt;&lt;/li&gt;
            * &lt;li data-ej-imageurl="../themes/sample/radialmenu/browser.png" 
            * data-ej-text="browser"&gt;&lt;/li&gt;
            * &lt;/ul&gt;
            * &lt;/div&gt;
            * &lt;script&gt;
            * $(function () {
            * $("#defaultradialmenu").ejRadialMenu({ "backImageClass":"e-m-backimage","width":300});	
            * });
            * &lt;/script&gt;
            * @example 
            * //Get or set the Radialmenu back button Image, after initialization:
            * &lt;script&gt;
            * // Gets the back button Image API.		
            * $("#defaultradialmenu").ejRadialMenu ("option", "backImageClass");			
            * // Sets the Width API
            * $("#defaultradialmenu").ejRadialMenu ("option", "backImageClass", "e-m-backimage");            
            * &lt;/script&gt;
            * @memberof ejRadialMenu
            * @instance
            */
            backImageClass: null,
            /**		
            * Renders the back button Image for the items.
            * @default null
            * @type {string}
            * @example 
            * // Set Radialmenu back button Image on initialization. 
            * //To set back button image API 
            * &lt;div &gt;
            * &lt;br /&gt;
            * &lt;p&gt;
            * Syncfusion is the enterprise technology partner of choice for Windows development
            * &lt;/p&gt;
            * &lt;/div&gt;  
            * &lt;div id="defaultradialmenu"&gt;
            * &lt;ul&gt;
            * &lt;li data-ej-imageurl="../themes/sample/radialmenu/social.png"
            * data-ej-text="social"&gt;&lt;/li&gt;
            * &lt;li data-ej-imageurl="../themes/sample/radialmenu/music.png" 
            * data-ej-text="music"&gt;&lt;/li&gt;
            * &lt;li data-ej-imageurl="../themes/sample/radialmenu/direction.png" 
            * data-ej-text="direction"&gt;&lt;/li&gt;
            * &lt;li data-ej-imageurl="../themes/sample/radialmenu/message.png" 
            * data-ej-text="message"&gt;&lt;/li&gt;
            * &lt;li data-ej-imageurl="../themes/sample/radialmenu/browser.png" 
            * data-ej-text="browser"&gt;&lt;/li&gt;
            * &lt;/ul&gt;
            * &lt;/div&gt;
            * &lt;script&gt;
            * $(function () {
            * $("#defaultradialmenu").ejRadialMenu({ "targetElementId":"button","width":300});	
            * });
            * &lt;/script&gt;
            * @example 
            * //Get or set the Radialmenu back button Image, after initialization:
            * &lt;script&gt;
            * // Gets the back button Image API.		
            * $("#defaultradialmenu").ejRadialMenu ("option", "targetElementId");			
            * // Sets the Width API
            * $("#defaultradialmenu").ejRadialMenu ("option", "targetElementId", "button");            
            * &lt;/script&gt;
            * @memberof ejRadialMenu
            * @instance
            */
            targetElementId: null,
            /**    
            * Event triggers when the mouse down happens.
            * @event
            * @name ejRadialMenu#mouseDown 	
            * @param {Object} argument Event parameters from Radialmenu     
            * @param {boolean} argument.cancel if the event should be canceled; otherwise, false.
            * @param {Object} argument.model returns the Radialmenu model
            * @param {string} argument.type returns the name of the event
            * @param {object} argument.item returns the item of element
            * @param {String} argument.itemName returns the name of item
            * @example 
            * //MouseDown event for Radialmenu
            * &lt;div &gt;
            * &lt;br /&gt;
            * &lt;p&gt;
            * Syncfusion is the enterprise technology partner of choice for Windows development
            * &lt;/p&gt;
            * &lt;/div&gt;  
            * &lt;div id="defaultradialmenu"&gt;
            * &lt;ul&gt;
            * &lt;li data-ej-imageurl="../themes/sample/radialmenu/social.png"
            * data-ej-text="social"&gt;&lt;/li&gt;
            * &lt;li data-ej-imageurl="../themes/sample/radialmenu/music.png" 
            * data-ej-text="music"&gt;&lt;/li&gt;
            * &lt;li data-ej-imageurl="../themes/sample/radialmenu/direction.png" 
            * data-ej-text="direction"&gt;&lt;/li&gt;
            * &lt;li data-ej-imageurl="../themes/sample/radialmenu/message.png" 
            * data-ej-text="message"&gt;&lt;/li&gt;
            * &lt;li data-ej-imageurl="../themes/sample/radialmenu/browser.png" 
            * data-ej-text="browser"&gt;&lt;/li&gt;
            * &lt;/ul&gt;
            * &lt;/div&gt;
            * &lt;script&gt;
            * $("#defaultradialmenu").ejRadialMenu({
            *   mousedown: function (args) { //handle the event
            * }
            * });         
            * &lt;/script&gt;
            * @memberof ejRadialMenu
            * @instance
            */
            mouseDown: null,
            /**    
            * Event triggers when the mouse up happens.
            * @event
            * @name ejRadialMenu#mouseUp 	
            * @param {Object} argument Event parameters from Radialmenu     
            * @param {boolean} argument.cancel if the event should be canceled; otherwise, false.
            * @param {Object} argument.model returns the Radialmenu model
            * @param {string} argument.type returns the name of the event
            * @param {object} argument.item returns the item of element
            * @param {String} argument.itemName returns the name of item
            * @example 
            * //MouseUp event for Radialmenu
            * &lt;div &gt;
            * &lt;br /&gt;
            * &lt;p&gt;
            * Syncfusion is the enterprise technology partner of choice for Windows development
            * &lt;/p&gt;
            * &lt;/div&gt;  
            * &lt;div id="defaultradialmenu"&gt;
            * &lt;ul&gt;
            * &lt;li data-ej-imageurl="../themes/sample/radialmenu/social.png"
            * data-ej-text="social"&gt;&lt;/li&gt;
            * &lt;li data-ej-imageurl="../themes/sample/radialmenu/music.png" 
            * data-ej-text="music"&gt;&lt;/li&gt;
            * &lt;li data-ej-imageurl="../themes/sample/radialmenu/direction.png" 
            * data-ej-text="direction"&gt;&lt;/li&gt;
            * &lt;li data-ej-imageurl="../themes/sample/radialmenu/message.png" 
            * data-ej-text="message"&gt;&lt;/li&gt;
            * &lt;li data-ej-imageurl="../themes/sample/radialmenu/browser.png" 
            * data-ej-text="browser"&gt;&lt;/li&gt;
            * &lt;/ul&gt;
            * &lt;/div&gt;
            * &lt;script&gt;
            * $("#defaultradialmenu").ejRadialMenu({
            *   mouseup: function (args) { //handle the event
            * }
            * });         
            * &lt;/script&gt;
            * @memberof ejRadialMenu
            * @instance
            */
            mouseUp: null,
            /**    
            * Event triggers when we select an item.
            * @event
            * @name ejRadialMenu#select 	
            * @param {Object} argument Event parameters from Radialmenu     
            * @param {boolean} argument.cancel if the event should be canceled; otherwise, false.
            * @param {Object} argument.model returns the Radialmenu model
            * @param {string} argument.type returns the name of the event
            * @param {object} argument.item returns the item of element
            * @param {String} argument.itemName returns the name of item
            * @example 
            * //select event for Radialmenu
            * &lt;div &gt;
            * &lt;br /&gt;
            * &lt;p&gt;
            * Syncfusion is the enterprise technology partner of choice for Windows development
            * &lt;/p&gt;
            * &lt;/div&gt;  
            * &lt;div id="defaultradialmenu"&gt;
            * &lt;ul&gt;
            * &lt;li data-ej-imageurl="../themes/sample/radialmenu/social.png"
            * data-ej-text="social"&gt;&lt;/li&gt;
            * &lt;li data-ej-imageurl="../themes/sample/radialmenu/music.png" 
            * data-ej-text="music"&gt;&lt;/li&gt;
            * &lt;li data-ej-imageurl="../themes/sample/radialmenu/direction.png" 
            * data-ej-text="direction"&gt;&lt;/li&gt;
            * &lt;li data-ej-imageurl="../themes/sample/radialmenu/message.png" 
            * data-ej-text="message"&gt;&lt;/li&gt;
            * &lt;li data-ej-imageurl="../themes/sample/radialmenu/browser.png" 
            * data-ej-text="browser"&gt;&lt;/li&gt;
            * &lt;/ul&gt;
            * &lt;/div&gt;
            * &lt;script&gt;
            * $("#defaultradialmenu").ejRadialMenu({
            *   select: function (args) { //handle the event
            * }
            * });         
            * &lt;/script&gt;
            * @memberof ejRadialMenu
            * @instance
            */
            select: null
        },
        dataTypes: {

        },
        _init: function () {
            this._initialization();
            this._prefix = "e-";
            this._startEvent = "mouseDown";
            this._endEvent = "mouseUp";
            this._renderRadial();
            this._wireEvents();
        },
        _wireEvents: function () {
            this._radialClickHandler = $.proxy(this._radialClickHandler, this);
            this._documentClickHandler = $.proxy(this._documentClickHandler, this);
            ej.listenEvents([this._radial, $(document)], [ej.tapEvent(), ej.tapEvent()], [this._radialClickHandler, this._documentClickHandler], false);
        },
        _radialClickHandler: function (e) {
            if (this._radial.hasClass("e-backarrow")) {
                this._menuChildSVGDiv.removeClass("e-scaleshow").addClass("e-scalehide");
                this._radial.removeClass("e-backarrow " + this.model.backImageClass + "").addClass(this.model.imageClass);
                setTimeout(function () {
                    $("#menuchildsvgdiv.e-abs").remove();
                }, 150);
                setTimeout(function () {
                    $("#radialchildsvgdiv.e-abs").removeClass("e-scalehide e-displaynone").addClass("e-scaleshow");
                }, 200);
            }
            else {
                if (this._radialCircleSVGDiv.hasClass("e-displaynone")) {
                    this._radialCircleSVGDiv.removeClass("e-displaynone e-radialhide").addClass("e-radialshow");
                    this._radialChildSVGDiv.removeClass("e-displaynone e-radialhide").addClass("e-radialshow");
                }
                else {
                    this._radialCircleSVGDiv.removeClass("e-radialshow").addClass("e-radialhide");
                    this._radialChildSVGDiv.removeClass("e-radialshow e-scaleshow").addClass("e-radialhide");
                    setTimeout(function () {
                        $("#radialcirclesvgdiv.e-abs").addClass("e-displaynone");
                        $("#radialchildsvgdiv.e-abs").addClass("e-displaynone");
                    }, 160);
                }
            }
        },
        _radialChildAnimate: function () {
            setTimeout(function () {
                $("#radialchildsvgdiv.e-abs").removeClass("e-scalehide e-scaleshow").addClass("e-displaynone");
            }, 150);
        },
        _documentClickHandler: function (e) {
            if ($($(e.target).closest("div.e-radialmenu")).length == 0 && $($(e.target).closest("div#" + this.model.targetElementId + "")).length == 0) {
                this.menuHide();
                setTimeout(function () {
                    $(".e-radialmenu.e-js").addClass("e-displaynone");
                }, 200);
            }
            else if ($("#radialcirclesvgdiv.e-abs").hasClass("e-displaynone"))
                this.show();
            else if ($($(e.target).closest("div.e-radialmenu")).length == 0)
                this.menuHide();
        },
        // public methods
        /**
        * To Show the redialmenu 
        * @return jQuery
        * @example 
        * &lt;div &gt;
        * &lt;br /&gt;
        * &lt;p&gt;
        * Syncfusion is the enterprise technology partner of choice for Windows development
        * &lt;/p&gt;
        * &lt;/div&gt;  
        * &lt;div id="defaultradialmenu"&gt;
        * &lt;ul&gt;
        * &lt;li data-ej-imageurl="../themes/sample/radialmenu/social.png"
        * data-ej-text="social"&gt;&lt;/li&gt;
        * &lt;li data-ej-imageurl="../themes/sample/radialmenu/music.png" 
        * data-ej-text="music"&gt;&lt;/li&gt;
        * &lt;li data-ej-imageurl="../themes/sample/radialmenu/direction.png" 
        * data-ej-text="direction"&gt;&lt;/li&gt;
        * &lt;li data-ej-imageurl="../themes/sample/radialmenu/message.png" 
        * data-ej-text="message"&gt;&lt;/li&gt;
        * &lt;li data-ej-imageurl="../themes/sample/radialmenu/browser.png" 
        * data-ej-text="browser"&gt;&lt;/li&gt;
        * &lt;/ul&gt;
        * &lt;/div&gt;
        * @example 
        * //Get or set the Radialmenu renderMode, after initialization:
        * &lt;script&gt;
        *  $("#defaultradialmenu").ejRadialMenu ("show");
        * &lt;/script&gt;
        *@memberof ejRadialMenu
        * @instance
        */
        show: function () {
            this.element.removeClass("e-displaynone");
        },
        /**
        * To hide the redialmenu 
        * @return jQuery
        * @example 
        * &lt;div &gt;
        * &lt;br /&gt;
        * &lt;p&gt;
        * Syncfusion is the enterprise technology partner of choice for Windows development
        * &lt;/p&gt;
        * &lt;/div&gt;  
        * &lt;div id="defaultradialmenu"&gt;
        * &lt;ul&gt;
        * &lt;li data-ej-imageurl="../themes/sample/radialmenu/social.png"
        * data-ej-text="social"&gt;&lt;/li&gt;
        * &lt;li data-ej-imageurl="../themes/sample/radialmenu/music.png" 
        * data-ej-text="music"&gt;&lt;/li&gt;
        * &lt;li data-ej-imageurl="../themes/sample/radialmenu/direction.png" 
        * data-ej-text="direction"&gt;&lt;/li&gt;
        * &lt;li data-ej-imageurl="../themes/sample/radialmenu/message.png" 
        * data-ej-text="message"&gt;&lt;/li&gt;
        * &lt;li data-ej-imageurl="../themes/sample/radialmenu/browser.png" 
        * data-ej-text="browser"&gt;&lt;/li&gt;
        * &lt;/ul&gt;
        * &lt;/div&gt;
        * @example 
        * //Get or set the Radialmenu renderMode, after initialization:
        * &lt;script&gt;
        *  $("#defaultradialmenu").ejRadialMenu ("hide");
        * &lt;/script&gt;
        *@memberof ejRadialMenu
        * @instance
        */
        hide: function () {
            this.element.addClass("e-displaynone");
        },
        /**
        * To hide the redialmenu but not the icon
        * @return jQuery
        * @example 
        * &lt;div &gt;
        * &lt;br /&gt;
        * &lt;p&gt;
        * Syncfusion is the enterprise technology partner of choice for Windows development
        * &lt;/p&gt;
        * &lt;/div&gt;  
        * &lt;div id="defaultradialmenu"&gt;
        * &lt;ul&gt;
        * &lt;li data-ej-imageurl="../themes/sample/radialmenu/social.png"
        * data-ej-text="social"&gt;&lt;/li&gt;
        * &lt;li data-ej-imageurl="../themes/sample/radialmenu/music.png" 
        * data-ej-text="music"&gt;&lt;/li&gt;
        * &lt;li data-ej-imageurl="../themes/sample/radialmenu/direction.png" 
        * data-ej-text="direction"&gt;&lt;/li&gt;
        * &lt;li data-ej-imageurl="../themes/sample/radialmenu/message.png" 
        * data-ej-text="message"&gt;&lt;/li&gt;
        * &lt;li data-ej-imageurl="../themes/sample/radialmenu/browser.png" 
        * data-ej-text="browser"&gt;&lt;/li&gt;
        * &lt;/ul&gt;
        * &lt;/div&gt;
        * @example 
        * //Get or set the Radialmenu renderMode, after initialization:
        * &lt;script&gt;
        *  $("#defaultradialmenu").ejRadialMenu ("menuHide");
        * &lt;/script&gt;
        *@memberof ejRadialMenu
        * @instance
        */
        menuHide: function () {
            if ($("#menuchildsvgdiv.e-abs").length != 0) {
                this._radialCircleSVGDiv.removeClass("e-radialshow e-radialhide e-scaleshow e-scalehide").addClass("e-displaynone");
                this._radialChildSVGDiv.removeClass("e-radialshow e-radialhide e-scaleshow e-scalehide").addClass("e-displaynone");
                this._radial.removeClass("e-backarrow " + this.model.backImageClass + "").addClass(this.model.imageClass);
                this._menuChildSVGDiv.removeClass("e-scaleshow").addClass("e-radialhide");
                setTimeout(function () {
                    $("#menuchildsvgdiv.e-abs").removeClass("e-radialhide").remove();
                }, 150);
            }
            else {
                this._radialCircleSVGDiv.removeClass("e-radialshow").addClass("e-radialhide");
                this._radialChildSVGDiv.removeClass("e-radialshow e-scaleshow").addClass("e-radialhide");
                setTimeout(function () {
                    $("#radialcirclesvgdiv.e-abs").addClass("e-displaynone").removeClass("e-radialshow e-radialhide e-scaleshow e-scalehide");
                    $("#radialchildsvgdiv.e-abs").addClass("e-displaynone").removeClass("e-radialshow e-radialhide e-scaleshow e-scalehide");
                }, 160);
            }
        }
    });
    $.extend(true, ej.RadialMenu.prototype, ej.RadialMenuBase.prototype);
})(jQuery, Syncfusion);;

ej.Ej3DRender = function () {

   
};

(function ($) {
    
    ej.Ej3DRender.Polygons = [];

    
    ej.Ej3DRender.prototype = {
        matrix3D: function () {
            
        },

        vector3D: function () {
             
        },
       
        BSPTreeBuilder: function () {

        },

        Graphics3D: function () {

        },

        polygon3D: function () {

        }
    }

    var Ej3DRender = new ej.Ej3DRender();
    
    Ej3DRender.vector3D.prototype = {
        //Members
        x: "",
        y: "",
        z: "",
        Epsilon:0.00001,
        //all coordinates indicate 0
        //  empty: new Vector3D(0, 0, 0),

        //isValid: !isNaN(this.x) && !isNaN(this.y) && !isNaN(this.z),
        isValid: function (point) {
            return !isNaN(point.x) && !isNaN(point.y) && !isNaN(point.z)
        },
        //Constructor
        vector3D: function (points, vz) {
            this.x = points.x;
            this.y = points.y;
            this.z = vz;
        },

        vector3D: function (vx, vy, vz) {
            this.x = vx;
            this.y = vy;
            this.z = vz;
            return {x:vx, y:vy, z:vz}
        },

        //operations
        vector3DMinus: function (v1, v2) {
            return this.vector3D(v1.x - v2.x, v1.y - v2.y, v1.z - v2.z);
        },

        vector3DPlus: function (v1, v2) {
            return this.vector3D(v1.x + v2.x, v1.y + v2.y, v1.z + v2.z);
        },

        vector3DMultiply: function (v1, v2) {
            var _x = v1.y * v2.z - v2.y * v1.z;
            var _y = v1.z * v2.x - v2.z * v1.x;
            var _z = v1.x * v2.y - v2.x * v1.y;

            return this.vector3D(_x, _y, _z);
        },

        vector3dAND: function (v1, v2) {
            return (v1.x * v2.x + v1.y * v2.y + v1.z * v2.z);
        },

        vector3DStarMultiply: function (v1, val) {
            var _x = v1.x * val;
            var _y = v1.y * val;
            var _z = v1.z * val;

            return this.vector3D(_x, _y, _z);
        },

        //Methods
        getLength: function (vector) {
            var sqt = this.vector3dAND(vector, vector);
            return Math.sqrt(sqt);
        },

        //getLength: function (vector) {
        //    var sqt = this.vector3dAND(vector, vector);
        //    return Math.sqrt(sqt);
        //},

        normalize: function () {
            var l = this.getLength();
            x /= l;
            y /= l;
            z /= l;
        },
        getNormal:function(v1, v2, v3)
        {
            var v4 = this.vector3DMinus(v1, v2);
            var v5 = this.vector3DMinus(v3, v2);
            var n = this.vector3DMultiply(v4, v5);
            var l = this.getLength(n);

           if (l < this.Epsilon)
           {
              l = 0;
            }
           return { x: n.x/ l, y: n.y / l, z: n.z / l }
        }
    }

   Ej3DRender.matrix3D.prototype = {
       //Memebr
        MATRIX_SIZE: 4,
       //Constructor
        matrix3D: function (size) {
            var mData = [];
           for (var i = 0; i < size; i++) {
               mData[i] = this.createArray(size);
           }
           return mData;
        },

       //method
        isAffine: function (mData) {

            return (mData[0][3] == 0) && (mData[1][3] == 0)
                    && (mData[2][3] == 0) && (mData[3][3] == 1);
        },
       createArray: function (initialSize) {
           var a = [];
           for (index = 0; index < initialSize; ++index) {
               a[index] = 0;
           }
           return a;
       },

       getIdentity: function () {

           var mData=this.matrix3D(this.MATRIX_SIZE);

           for (var i = 0; i < this.MATRIX_SIZE; i++) {
               mData[i][i] = 1.0;
           }
          
           return mData;
       },

      getInvertal:function(matrix)
      {
         var m = this.getIdentity();

         for (var i = 0; i < this.MATRIX_SIZE; i++)
         {
           for (var j = 0; j < this.MATRIX_SIZE; j++)
           {
               m[i][j] = this.getMinor(matrix, i, j);
            }
         }

         m = this.transposed(m);
         m = this.getMatrixMultiple((1 / this.getDeterminant(matrix)) , m);

      return m;
      },
      getMatrixMultiple: function (factor, matrix) {
          for (var i = 0; i < matrix.length; i++) {
              for (var j = 0; j < matrix[i].length; j++) {
                  matrix[i][j] = matrix[i][j] * factor;
              }
          }
          return matrix;
      },
      getMatrixVectorMutiple:function(m1, point)
      {
            var x = m1[0][0] * point.x + m1[1][0] * point.y + m1[2][0] * point.z + m1[3][0];
            var y = m1[0][1] * point.x + m1[1][1] * point.y + m1[2][1] * point.z + m1[3][1];
            var z = m1[0][2] * point.x + m1[1][2] * point.y + m1[2][2] * point.z + m1[3][2];

      if (!this.isAffine(m1))
      {
         var c = 1 / (m1[0][3] * point.x + m1[1][3] * point.y + m1[2][3] * point.z + m1[3][3]);
         x *= c;
         y *= c;
         z *= c;
      }

       return {x:x, y:y, z:z};
      },
      getMatrixVectorAnd:function(m1, v1)
      {
            var x = m1[0][0] * v1.x + m1[1][0] * v1.y + m1[2][0] * v1.z;
            var y = m1[0][1] * v1.x + m1[1][1] * v1.y + m1[2][1] * v1.z;
            var z = m1[0][2] * v1.x + m1[1][2] * v1.y + m1[2][2] * v1.z;

            return Ej3DRender.vector3D.prototype.vector3D(x, y, z);
         },
      getMatrixAdd:function(m1, m2) 
      {
          var m = this.matrix3D(4);

       for (var i = 0; i < 4; i++)
        {
          for (var j = 0; j < 4; j++)
          {
             m[i][j] = m1[i][j] + m2[i][j];
          }
        }

       return m;
     },
      getMatrixMultiplication:function(m1, m2)
      {
          var res = this.getIdentity();

         for (var i = 0; i < this.MATRIX_SIZE; i++)
         {
             for (var j = 0; j < this.MATRIX_SIZE; j++)
             {
             var v = 0;

             for (var k = 0; k < this.MATRIX_SIZE; k++)
             {
                 v += m1[k][j] * m2[i][k];
                 var t = v;
             }

             res[i][j] = v;
          }
       }

       return res;
      },
      getMatrixEqual:function(m1,m2)
       {
            var res = true;

           for (var i = 0; i < m1.length; i++)
          {
              for (var j = 0; j < m1.length; j++)
              {
                 if (m1[i][j] != m2[i][j])
                 {
                     res = false;
                 }
             }
         }

      return res;
      },
      getMatrixNotEqual:function(m1, m2)
      {
            var res = true; 

         for (var i = 0; i < m1.length; i++)
        {
             for (var j = 0; j < m1.length; j++)
          {
             if (m1[i][j] != m2[i][j])
             {
                res = false;
             }
          }
       }

        return !res;
    },


      getMinor:function(dd, columnIndex, rowIndex)
      {
          return (((columnIndex + rowIndex) % 2 == 0) ? 1 : -1) * this.getDeterminant(this.getMMtr(dd, columnIndex, rowIndex));

      },

     getMMtr:function(dd, columnIndex, rowIndex)
     {
        var count = dd.length - 1;
        var d = this.createArray(count);

        for (var i = 0; i < count; i++)
        {
            var m = (i >= columnIndex) ? i + 1 : i;
            d[i] = this.createArray(count);

            for (var j = 0; j < count; j++)
            {
                var n = (j >= rowIndex) ? j + 1 : j;

                d[i][j] = dd[m][n];
            }
        }
         return d;
     },
    getDeterminant:function(dd)
    {
        var count = dd.length;
        var res = 0;

        if (count < 2)
        {
            res = dd[0][0];
        }
        else
        {
            var k = 1;

            for (var i = 0; i < count; i++)
            {
                var dm = this.getMMtr(dd, i, 0);

                res += k * dd[i][0] * this.getDeterminant(dm);
                k = (k > 0) ? -1 : 1;
            }
        }

        return res;
    },


       transform: function (x, y, z) {
           var res = this.getIdentity();

           res[3][0] = x;
           res[3][1] = y;
           res[3][2] = z;

           return res;
       },


       turn: function (angle) {
           var res = this.getIdentity();

           res[0][0] = Math.cos(angle);
           res[2][0] = -Math.sin(angle);
           res[0][2] = Math.sin(angle);
           res[2][2] = Math.cos(angle);

           return res;
       },

       tilt: function (angle) {
           var res = this.getIdentity();
          
           res[1][1] = Math.cos(angle);
           res[2][1] = Math.sin(angle);
           res[1][2] = -(Math.sin(angle));
           res[2][2] = Math.cos(angle);
          
           return res;          
       },
       transposed:function(matrix3D)
       {
           var m = this.getIdentity();

           for (var i = 0; i < this.MATRIX_SIZE; i++)
            {
                for (var j = 0; j < this.MATRIX_SIZE; j++)
                {
                    m[i][j] = matrix3D[j][i];
                }
           }
          return m;
       },
      shear:function(xy, xz, yx, yz, zx, zy)
        {
          var res = this.getIdentity();

           res[1, 0] = xy;
           res[2, 0] = xz;
           res[0, 1] = yx;
           res[2, 1] = yz;
           res[0, 2] = zx;
           res[1, 2] = zy;

           return res;
        }


   }
   Ej3DRender.Graphics3D.prototype = {
      addVisual:function(polygon)
         {
          if ((polygon == null) || (polygonobj.test(polygon)))
            {
                return -1;
            }
         polygon.Graphics3D = this;
         return bsptreeobj.add(polygon);
      },

       getVisualCount:function()
           {
           return ej.Ej3DRender.Polygons.length();
       },
       remove:function(polygon)
        {
           ej.Ej3DRender.Polygons.Remove(polygon);
       },
 
     clearVisual:function()
       {
         ej.Ej3DRender.Polygons=null;
       },

      getVisual:function()
          {
                return ej.Ej3DRender.Polygons;
         },
      prepareView:function(perspectiveAngle, depth, rotation, tilt, size)
      {
          if (arguments.length==0) {
              bsptreeobj.build();
          }
          else {
              if (ej.Ej3DRender.transform == null)
                  ej.Ej3DRender.transform = ej.EjSvgRender.chartTransform3D.transform3D(size);
              else
                  ej.Ej3DRender.transform.mViewport = size;
              ej.Ej3DRender.transform.Rotation = rotation;
              ej.Ej3DRender.transform.Tilt = tilt;
              ej.Ej3DRender.transform.Depth = depth;
              ej.Ej3DRender.transform.PerspectiveAngle = perspectiveAngle;
              ej.EjSvgRender.chartTransform3D.transform(ej.Ej3DRender.transform);
              ej.Ej3DRender.tree = bsptreeobj.build();
             
          } 
      },
      view: function (panel, sender, rotation, tilt, size, perspectiveAngle, depth)
      {
          var MaxValue = 32767;
          if (arguments.length == 2) {
              if (panel == null) return;
              var eye = vector.vector3D(0, 0, MaxValue);
              this.drawBspNode3D(ej.Ej3DRender.tree, eye, panel, sender);
          }
          else {
              if (panel == null) return;
            
              if (ej.Ej3DRender.transform == null)
                  ej.Ej3DRender.transform = ej.EjSvgRender.chartTransform3D.transform3D(size);
              else
                  ej.Ej3DRender.transform.mViewport = size;
              ej.Ej3DRender.transform.Rotation = rotation;
              ej.Ej3DRender.transform.Tilt = tilt;
              ej.Ej3DRender.transform.Depth = depth;
              ej.Ej3DRender.transform.PerspectiveAngle = perspectiveAngle;
              ej.EjSvgRender.chartTransform3D.transform(ej.Ej3DRender.transform);
              var eye = vector.vector3D(0, 0, MaxValue);
              this.drawBspNode3D(ej.Ej3DRender.tree, eye, panel, sender);
          }
      },

      draw3DElement: function (tr, sender) {
          if (tr.Plane.element){
              if (tr.Plane.element.tag == "text" || tr.Plane.element.tag == "dataLabel")
                  polygonobj.drawText(tr.Plane, sender);
              else if (tr.Plane.element.tag == "polyline")
                  polygonobj.drawPolyLine(tr.Plane, sender);
              else
                  polygonobj.drawLine(tr.Plane, sender);
            }
              
          else
            polygonobj.draw(tr.Plane, sender);          
      },

    drawBspNode3D:function(tr, eye, panel, sender)
    {
        if (tr == null || ej.Ej3DRender.transform == null) return;
     while (true)
     {
         var r = vector.vector3dAND(polygonobj.getNormal(ej.EjSvgRender.chartTransform3D.result(ej.Ej3DRender.transform), tr.Plane.VectorPoints), eye);
        if (r > tr.Plane.D)
        {
            if (tr.Front != null)
            {
                this.drawBspNode3D(tr.Front, eye, panel, sender);
            }

            this.draw3DElement(tr, sender);

            if (tr.Back != null)
            {
                tr = tr.Back;
                continue;
            }
        }
        else
        {
            if (tr.Back != null)
            {
                this.drawBspNode3D(tr.Back, eye, panel, sender);
            }

            this.draw3DElement(tr, sender);

            if (tr.Front != null)
            {
                tr = tr.Front;
                continue;
            }
        }
        break;
    }

    }

 }


   Ej3DRender.BSPTreeBuilder.prototype = {
       //Members
       EPSILON: 0.0005,
       Polygon:[],
       
       //Methods
       add:function(poly)
       {
           ej.Ej3DRender.Polygons.push(poly);
           return ej.Ej3DRender.Polygons.length - 1;
       },

       remove:function(index)
       {
           ej.Ej3DRender.Polygons.splice(index,1);
       },

       clear:function()
       {
           ej.Ej3DRender.Polygons=[];
       },

       count:function()
       {
           return ej.Ej3DRender.Polygons.length;
       },

       getNext:function(i, count)
        {
            if (i >= count)
            {
             return i - count;
            }
            if (i < 0)
              {
                  return i + count;
              }

             return i;
       },
       getNodeCount:function(el)
        {
            return (el == null) ? 0 : 1 + this.getNodeCount(el.Back) + this.getNodeCount(el.Front);
        },
       vector3DIndexClassification:function(point, ind, res){
           return{
                 index:ind,
                 result: res,
                 vector:point,

                isCuttingBackPoint:false,
                cuttingBackPairIndex:null,
                alreadyCuttedBack:false,

                isCuttingFrontPoint:false,
                cuttingFrontPairIndex:null,
                alreadyCuttedFront:false
         }
       },
       classifyPoint: function(pt, pln)
         {
            var res = "OnPlane";
            var sv = -pln.D - Ej3DRender.vector3D.prototype.vector3dAND(pt , pln.normal);

           if (sv > this.EPSILON)
            {
              res ="OnBack";
            }
          else if (sv < -this.EPSILON)
            {
              res = "OnFront";
            }

            return res;
        },
      
     classifyPolygon:function(pln, plg)
        {
            var res = "Unknown";
            var points = plg.Points;

           if (points == null)
               return res;
               var onBack = 0;
               var onFront = 0;
               var onPlane = 0;
               var normal = pln.normal;// root node normailized value perpendicular direction
               var d = pln.D; // constant of the plan or depth

          for (var i = 0, len = points.length; i < len; i++)
           {
              var r = -d - Ej3DRender.vector3D.prototype.vector3dAND(points[i], normal); // Comparision of Plane point depth with the other nodes
 
              if (r > this.EPSILON)
                {
                  onBack++;
                }
              else if (r < -this.EPSILON)
                {
                   onFront++;
                }
             else
               {
                  onPlane++;
               }

             if ((onBack > 0) && (onFront > 0))
               {
                 break;
               }
          }
         if (onPlane == points.length)
            {
               res = "OnPlane";
            }
          else if (onFront + onPlane == points.length)
           {
              res = "ToRight";
           }
          else if (onBack + onPlane == points.length)
           {
             res = "ToLeft";
           }
          else
          {
                res = "Unknown";
          }
      return res;
     },
   splitPolygon:function(poly, part)
     {
            var backP = [];
            var frontP = [];

    // this code looks for points which lie on the part plane and divide polygon into two parts
    if (poly.Points != null)
    {
        var polyPoints = [];
        var backPartPoints = [];
        var frontPartPoints = [];

        var outpts;
        var inpts;

        var count = poly.Points.length;
        for (var i = 0; i < count;i++)
        {
            var ptB = poly.Points[i];
            var ptC = poly.Points[this.getNext(i + 1, count)];
            var sideB = this.classifyPoint(ptB, part);
            var sideC = this.classifyPoint(ptC, part);

            var vwiwcB = this.vector3DIndexClassification(ptB, polyPoints.length, sideB);
            polyPoints.push(vwiwcB);

            if ((sideB != sideC) && (sideB != "OnPlane") &&
                (sideC !="OnPlane"))
            {
                var v = vector.vector3DMinus(ptB, ptC);
                var dir = vector.vector3DMinus(vector.vector3DStarMultiply(part.normal, (-part.D)), ptC);

                var sv = vector.vector3dAND(dir, part.normal);
                var sect = sv / vector.vector3dAND(part.normal, v);
                var ptP = vector.vector3DPlus(ptC, vector.vector3DStarMultiply(v, sect));
                var vwiwc = this.vector3DIndexClassification(ptP, polyPoints.length, "OnPlane");

                polyPoints.push(vwiwc);
                backPartPoints.push(vwiwc);
                frontPartPoints.push(vwiwc);
            }
            else
                if (sideB == "OnPlane")
                {
                    var ptA = poly.Points[this.getNext(i - 1, count)];
                    var sideA = this.classifyPoint(ptA, part);
                    if ((sideA == sideC)) continue;
                    if ((sideA != "OnPlane") && (sideC != "OnPlane"))
                    {
                        backPartPoints.push(vwiwcB);
                        frontPartPoints.push(vwiwcB);
                    }
                    else
                        if (sideA =="OnPlane")
                        {
                            switch (sideC)
                            {
                                case "OnBack":
                                    backPartPoints.push(vwiwcB);
                                    break;
                                case "OnFront":
                                    frontPartPoints.push(vwiwcB);
                                    break;
                            }
                        }
                        else
                            if (sideC =="OnPlane")
                            {
                                switch (sideA)
                                {
                                    case "OnBack":
                                        backPartPoints.push(vwiwcB);
                                        break;
                                    case "OnFront":
                                        frontPartPoints.push(vwiwcB);
                                        break;
                                }
                            }
                }
        }

        if ((frontPartPoints.length != 0) || (backPartPoints.length != 0))
        {
            for (var i = 0; i < backPartPoints.length - 1; i += 2)
            {
                var vwiwc1 = backPartPoints[i];
                var vwiwc2 = backPartPoints[i + 1];
                vwiwc1.CuttingBackPoint = true;
                vwiwc2.CuttingBackPoint = true;
                vwiwc1.CuttingBackPairIndex = vwiwc2.index;
                vwiwc2.CuttingBackPairIndex = vwiwc1.index;
            }
            for (var i = 0; i < frontPartPoints.length - 1; i += 2)
            {
                var vwiwc1 = frontPartPoints[i];
                var vwiwc2 = frontPartPoints[i + 1];
                vwiwc1.CuttingFrontPoint = true;
                vwiwc2.CuttingFrontPoint = true;
                vwiwc1.CuttingFrontPairIndex = vwiwc2.index;
                vwiwc2.CuttingFrontPairIndex = vwiwc1.index;
            }


            for (var i = 0; i < backPartPoints.length - 1; i++)
            {
                var vwiwc1 = backPartPoints[i];
                if (vwiwc1.alreadyCuttedBack) continue;
                var outpts=this.cutOutBackPolygon(polyPoints, vwiwc1);

                if (outpts.length > 2) {
                    var polygon1 = polygonobj.polygon3D(outpts, poly);
                    backP.push($.extend({}, polygon1));
                }
            }

            for (var i = 0; i < frontPartPoints.length - 1; i++)
            {
                var vwiwc2 = frontPartPoints[i];
                if (vwiwc2.alreadyCuttedFront) continue;
                inpts=this.cutOutFrontPolygon(polyPoints, vwiwc2);
                if (inpts.length > 2) {
                    var polygon2 = polygonobj.polygon3D(inpts, poly);
                    frontP.push($.extend({}, polygon2));
                }
            }
        } 
    }
     else
     {
        backP.push(poly);
        frontP.push(poly); 
     }

    return{BackP:backP, FrontP:frontP}
   },
  cutOutFrontPolygon:function(polyPoints, vwiwc)
   {
      points = [];

    var curVW = vwiwc;

    while (true)
    {
        curVW.alreadyCuttedFront = true;
        points.push(curVW.vector);

        var curVWPair = polyPoints[curVW.CuttingFrontPairIndex];

        if (curVW.CuttingFrontPoint)
        {
            if (!curVWPair.alreadyCuttedFront)
            {
                curVW = curVWPair;
            }
            else
            {
                var curVWPrev = polyPoints[this.getNext(curVW.index - 1, polyPoints.length)];
                var curVWNext = polyPoints[this.getNext(curVW.index + 1, polyPoints.length)];

                if ((curVWPrev.result =="OnFront") && !curVWPrev.alreadyCuttedFront)
                {
                    curVW = curVWPrev;
                }
                else
                    if ((curVWNext.result == "OnFront") && !curVWNext.alreadyCuttedFront)
                    {
                        curVW = curVWNext;
                    }
                    else
                    {
                        return points;
                    }
            }
        }
        else
        {
            var curPrev = polyPoints[this.getNext(curVW.index - 1, polyPoints.length)];
            var curNext = polyPoints[this.getNext(curVW.index + 1, polyPoints.length)];

            if ((curPrev.result != "OnBack") && !curPrev.alreadyCuttedFront)
            {
                curVW = curPrev;
            }
            else
                if ((curNext.result != "OnBack") && !curNext.alreadyCuttedFront)
                {
                    curVW = curNext;
                }
                else
                {
                    return points;
                }
        }
    }
    return points;
 },

   cutOutBackPolygon:function(polyPoints, vwiwc)
   {
       var points = [];
       var curVW = vwiwc;

    while (true)
    {
        curVW.alreadyCuttedBack = true;
        points.push(curVW.vector);

        var curVWPair = polyPoints[curVW.CuttingBackPairIndex];

        if (curVW.CuttingBackPoint)
        {
            if (!curVWPair.alreadyCuttedBack)
            {
                curVW = curVWPair;
            }
            else
            {
                var curVWPrev = polyPoints[this.getNext(curVW.index - 1, polyPoints.length)];
                var curVWNext = polyPoints[this.getNext(curVW.index + 1, polyPoints.length)];

                if ((curVWPrev.result == "OnBack") && !curVWPrev.alreadyCuttedBack)
                {
                    curVW = curVWPrev;
                }
                else
                    if ((curVWNext.result == "OnBack") && !curVWNext.alreadyCuttedBack)
                    {
                        curVW = curVWNext;
                    }
                    else
                    {
                        return points;
                    }
            }
        }
        else
        {
            var curVWPrev = polyPoints[this.getNext(curVW.index - 1, polyPoints.length)];
            var curVWNext = polyPoints[this.getNext(curVW.index + 1, polyPoints.length)];

            if ((curVWPrev.result != "OnFront") && !curVWPrev.alreadyCuttedBack)
            {
                curVW = curVWPrev;
            }
            else
                if ((curVWNext.result != "OnFront") && !curVWNext.alreadyCuttedBack)
                {
                    curVW = curVWNext;
                }
                else
                {
                    return points;
                }
        }
    }
     return points
  },
     build:function()
       {
           if(!arguments[0]){
             return (this.build(ej.Ej3DRender.Polygons));               
           }
           else{
               var arlist=arguments[0];
               if (arlist.length < 1) return null;
               var bspNode = {Back:null, Front:null, Plane:null}
               var plane = arlist[0];
               bspNode.Plane = plane;
               var arleft = [];
               var arright = [];

               for (var i = 1, len = arlist.length; i < len; i++)
               {
                   var pln = arlist[i];

                   if (pln == plane) continue;
                   var r = this.classifyPolygon(plane, pln);

                   switch (r)
                   {
                       case "OnPlane":
                       case "ToRight":
                           arright.push(pln);
                           break;

                       case "ToLeft":
                           arleft.push(pln);
                           break;

                       case "Unknown":
                           //if (pln is Line3D || pln is UIElement3D)
                           if(pln.element && (pln.element.tag == "line" || pln.element.tag == "text"))
                           {
                               arleft.push(pln);
                           }
                         else
                         {
                            
                               var result = this.splitPolygon(pln, plane);                              
                               for (var k = 0; k < result.BackP.length; k++) {
                                   result.BackP[k].Name = result.BackP[k].Name + "back";
                                   arleft.push(result.BackP[k]);
                               }
                               for (var j = 0; j < result.FrontP.length; j++) {
                                   result.FrontP[j].Name = result.FrontP[j].Name + "front";
                                   arright.push(result.FrontP[j]);
                               }
                        }
                   break;
               }
           }

           if (arleft.length > 0)
           {
               bspNode.Back = this.build(arleft);
           }

           if (arright.length > 0)
           {
               bspNode.Front = this.build(arright);
           }

           return bspNode;
         }
    }

   }

   Ej3DRender.polygon3D.prototype = {
       // Member    
       epsilon: 0.00001,
       normal: { x: 0, y: 0, z: 0 },
       a: (this.normal) ? this.normal.x : 0,
       b: (this.normal) ? this.normal.y : 0,
       c: (this.normal) ? this.normal.z : 0,
       vector: new (new ej.Ej3DRender()).vector3D(),

       // Constructor
       polygon3D: function (points, tag, index, stroke, strokeThickness, opacity, fill, name) {

           if (arguments.length == 3) {
               this.calcNormal(arguments[0], arguments[1], arguments[2])
           }
           else if (arguments.length == 2) {
               points = arguments[0];
               this.calcNormal(points[0], points[1], points[2]);
               this.vectorPoints = points;
               this.calcNormal(this.vectorPoints);
               var polygon = arguments[1];
               polygon.Normal = this.normal;
               polygon.normal = this.normal;
               polygon.Points = points;
               polygon.VectorPoints = this.vectorPoints;
               polygon.IsSplitted = true;
               polygon.d = this.d;
               polygon.D = this.d;
               return polygon;
                
           }
           else {
               this.calcNormal(points[0], points[1], points[2]);
               this.vectorPoints = points;
               this.calcNormal(this.vectorPoints);
               var element = { Tag: 'path', Parent: arguments[arguments.length - 1] }
               var polygon= {
                   Normal: this.normal,
                   normal: this.normal,
                   Points: points,
                   VectorPoints: this.vectorPoints,
                   Index: index,
                   Tag: tag,
                   Name: (name) ? name : null,
                   Stroke: stroke,
                   StrokeThickness: strokeThickness,
                   Opacity:opacity,
                   Fill: fill,
                   d: this.d,
                   D: this.d
               }
               if (arguments.length != 1) 
                   polygon.Element = element;

               return polygon;
               
           }
       },

       //Methods
       createLine: function (line, x1, y1, x2, y2, depth) {
           var strokeThickness = line.width;
           var vectorColl = [];
           vectorColl[0] = vector.vector3D(x1, y1, depth);
           vectorColl[1] = vector.vector3D(x1 + strokeThickness, y2 + strokeThickness, depth);
           vectorColl[2] = vector.vector3D(x2, y2, depth);
           return this.line3D(line, vectorColl);
       },
       createPolyline: function (points, element) {
           if (points.length == 2) {
               var prePoint = points[1];
               points.push(vector.vector3D(prePoint.x, prePoint.y, prePoint.z));
           }
            return this.polyLine3D(element, points);
       },
       polyLine3D: function (element, points) {
           var plane = this.polygon3D(points);
           plane.element = element;
           return plane;
       },

       line3D:function(element, points)
       {
           var plane = this.polygon3D(points);
           plane.element = element;
           return plane;
       },
      text3D:function(element,points)
      {
          var plane = this.polygon3D(points);
          plane.element = element;
          return plane;
      },

       createBox: function (v1, v2, tag, index, graphics3D, stroke, fill, strokeThickness, opacity, inverse, name, parent) {
           var res = [];

           var p1 = new Array(vector.vector3D(v1.x, v1.y, v1.z),
                vector.vector3D(v2.x, v1.y, v1.z),
                vector.vector3D(v2.x, v2.y, v1.z),
                vector.vector3D(v1.x, v2.y, v1.z))

           var p2 = new Array(
               vector.vector3D(v1.x, v1.y, v2.z),
               vector.vector3D(v2.x, v1.y, v2.z),
               vector.vector3D(v2.x, v2.y, v2.z),
               vector.vector3D(v1.x, v2.y, v2.z)
            )

           var p3 = new Array(

               vector.vector3D(v1.x, v1.y, v2.z),
               vector.vector3D(v2.x, v1.y, v2.z),
               vector.vector3D(v2.x, v1.y, v1.z),
               vector.vector3D(v1.x, v1.y, v1.z)
          )

           var p4 = new Array(
               vector.vector3D(v1.x, v2.y, v2.z),
               vector.vector3D(v2.x, v2.y, v2.z),
               vector.vector3D(v2.x, v2.y, v1.z),
               vector.vector3D(v1.x, v2.y, v1.z)
           )

           var p5 = new Array(
               vector.vector3D(v1.x, v1.y, v1.z),
               vector.vector3D(v1.x, v1.y, v2.z),
               vector.vector3D(v1.x, v2.y, v2.z),
               vector.vector3D(v1.x, v2.y, v1.z)
           )

           var p6 = new Array(
               vector.vector3D(v2.x, v1.y, v1.z),
               vector.vector3D(v2.x, v1.y, v2.z),
               vector.vector3D(v2.x, v2.y, v2.z),
               vector.vector3D(v2.x, v2.y, v1.z)
           )
           if (arguments[10]) {

               res[0] = this.polygon3D(p1, tag, index, stroke, strokeThickness, opacity, fill, "_0_"+ name, parent);
               res[1] = this.polygon3D(p2, tag, index, stroke, strokeThickness, opacity, fill, "_1_" + name, parent);
               res[2] = this.polygon3D(p3, tag, index, stroke, strokeThickness, opacity, fill, "_2_" + name, parent);
               res[3] = this.polygon3D(p4, tag, index, stroke, strokeThickness, opacity, fill, "_3_" + name, parent);
               res[4] = this.polygon3D(p5, tag, index, stroke, strokeThickness, opacity, fill, "_4_" + name, parent);
               res[5] = this.polygon3D(p6, tag, index, stroke, strokeThickness, opacity, fill, "_5_" + name, parent);
           }
           else {
               var parent= arguments[arguments.length-1];
               res[0] = this.polygon3D(p1, tag, index, stroke, strokeThickness, opacity, fill, "_0_" + index, parent);
               res[1] = this.polygon3D(p2, tag, index, stroke, strokeThickness, opacity, fill, "_1_" + index, parent);
               res[2] = this.polygon3D(p3, tag, index, stroke, strokeThickness, opacity, fill, "_2_" + index, parent);
               res[3] = this.polygon3D(p4, tag, index, stroke, strokeThickness, opacity, fill, "_3_" + index, parent);
               res[4] = this.polygon3D(p5, tag, index, stroke, strokeThickness, opacity, fill, "_4_" + index, parent);
               res[5] = this.polygon3D(p6, tag, index, stroke, strokeThickness, opacity, fill, "_5_" + index, parent);
           }

           if (inverse) {
               graphics.addVisual(res[0]);
               graphics.addVisual(res[1]);
               graphics.addVisual(res[2]);
               graphics.addVisual(res[3]);
               graphics.addVisual(res[4]);
               graphics.addVisual(res[5]);
           }
           else {
               graphics.addVisual(res[5]);
               graphics.addVisual(res[4]);
               graphics.addVisual(res[0]);
               graphics.addVisual(res[1]);
               graphics.addVisual(res[2]);
               graphics.addVisual(res[3]);
              
           }
           return res;
       },

       updateBox: function (plan, v1, v2, sender) {
           if (plan.Length < 6) return;

           this.update(new Array(vector.vector3D(v1.x, v1.y, v1.z),
                                        vector.vector3D(v2.x, v1.y, v1.z),
                                        vector.vector3D(v2.x, v2.y, v1.z),
                                        vector.vector3D(v1.x, v2.y, v1.z)), plan[0], sender);

           this.update(new Array(vector.vector3D(v1.x, v1.y, v2.z),
                                       vector.vector3D(v2.x, v1.y, v2.z),
                                       vector.vector3D(v2.x, v2.y, v2.z),
                                       vector.vector3D(v1.x, v2.y, v2.z)), plan[1], sender);

           this.update(new Array(vector.vector3D(v1.x, v1.y, v2.z),
                                       vector.vector3D(v2.x, v1.y, v2.z),
                                       vector.vector3D(v2.x, v1.y, v1.z),
                                       vector.vector3D(v1.x, v1.y, v1.z)), plan[2], sender);

           this.update(new Array(vector.vector3D(v1.x, v2.y, v2.z),
                                       vector.vector3D(v2.x, v2.y, v2.z),
                                       vector.vector3D(v2.x, v2.y, v1.z),
                                       vector.vector3D(v1.x, v2.y, v1.z)), plan[3], sender);

           this.update(new Array(vector.vector3D(v1.x, v1.y, v1.z),
                                       vector.vector3D(v1.x, v1.y, v2.z),
                                       vector.vector3D(v1.x, v2.y, v2.z),
                                       vector.vector3D(v1.x, v2.y, v1.z)), plan[4], sender);

           this.update(new Array(vector.vector3D(v2.x, v1.y, v1.z),
                                       vector.vector3D(v2.x, v1.y, v2.z),
                                       vector.vector3D(v2.x, v2.y, v2.z),
                                       vector.vector3D(v2.x, v2.y, v1.z)), plan[5], sender);
       },


       calcNormal: function () {
           if (arguments.length >= 3) {
               // Relative information of the points
               var v1 = arguments[0];
               var v2 = arguments[1];
               var v3 = arguments[2];
               var v4 = vector.vector3DMinus(v1, v2);
               var v5 = vector.vector3DMinus(v3, v2);
               var n = vector.vector3DMultiply(v4, v5);

               var l = vector.getLength(n);//Get length of the vector

               if (l < this.epsilon) {
                   l = 1;
               }

               this.normal= vector.vector3D(n.x / l, n.y / l, n.z / l); //Calculate normalization of the vector
               this.d = -(this.normal.x * v1.x + this.normal.y * v1.y + this.normal.z * v1.z);// Normalized values * 1st coordinates Coordinates - Depth of the plan

               if (arguments[3]) {
                   arguments[3].normal = this.normal;
                   arguments.d = this.d;
               }
           }
           else {
               var Points = arguments[0];
               this.calcNormal(Points[0], Points[1], Points[2], arguments[1]);

               for (var i = 3; (i < Points.length) && (this.test()) ; i++) {
                   this.calcNormal(Points[i], Points[0], Points[i / 2]);
               }
           }
       },

       test: function () {
           return !vector.isValid(this.normal);
       },

       transform: function (matrix, plan) {
           if (plan.Points != null) {
               for (var i = 0; i < plan.Points.length; i++) {

                   plan.VectorPoints[i] = plan.Points[i] = matrixobj.getMatrixVectorMutiple(matrix, plan.Points[i]);
               }

               this.calcNormal(plan.VectorPoints, plan);
           }
           else {
               var v = matrix * (plan.normal * -plan.d);
               plan.normal = matrixobj.getMatrixVectorAnd(matrix, plan.normal);
               vector.normalize(normal);
               plan.d = -(plan.normal & v);
           }
       },

       getPoint: function (x, y, normal, d) {
           if (typeof x == double) {
               var z = -(normal.x * x + normal.y * y + d) / normal.z;

               return vector.vector3D(x, y, z);
           }
           else {
               var position = x;
               var ray = y;
               var dir = vector.vector3DMinus(vector.vector3DStarMultiply(normal, (-d)), position);

               var sv = vector.vector3dAND(dir, normal);
               var sect = sv / vector.vector3dAND(normal, ray);

               return vector.vector3DPlus(position, (vector.vector3DStarMultiply(ray * sect)));
           }
       },

       getNormal: function (transform, VectorPoints) {
           var norm;

           if (VectorPoints != null) {
               norm = vector.getNormal(matrixobj.getMatrixVectorMutiple(transform, VectorPoints[0]),
                  matrixobj.getMatrixVectorMutiple(transform, VectorPoints[1]), matrixobj.getMatrixVectorMutiple(transform, VectorPoints[2]))


               for (var i = 3; (i < VectorPoints.length) && !vector.isValid(norm) && VectorPoints[i / 2]; i++) {
                   var v1 = matrixobj.getMatrixVectorMutiple(transform, VectorPoints[i]);
                   var v2 = matrixobj.getMatrixVectorMutiple(transform, VectorPoints[0]);
                   var v3 = matrixobj.getMatrixVectorMutiple(transform, VectorPoints[i / 2]);

                   norm = vector.getNormal(v1, v2, v3);
               }
           }
           else {
               norm = matrixobj.getMatrixVectorAnd(transform, normal);
               vector.normalize();
           }

           return norm;

       },

     createTextElement:function(position,  element, xLen, yLen)
      {
        
       var vectorColl = [];
       var x = position.x;
       var y = position.y;
 
       var desiredWidth = element.Width;
       var desiredHeight = element.Height;

          vectorColl[0] = this.vector.vector3D(x, y, position.z);
          vectorColl[1] = this.vector.vector3D(x + desiredWidth, y + desiredHeight + yLen, position.z);
          vectorColl[2] = this.vector.vector3D(x + desiredWidth + xLen, y + desiredHeight + yLen, position.z);
         return this.text3D(element, vectorColl);
     },

     redraw: function () { },

     drawPolyLine: function (panel, sender) {
         var transform = ej.Ej3DRender.transform;
         var pathDirection = ej.EjSvgRender.utils._getStringBuilder();
         var StartPoint = ej.EjSvgRender.chartTransform3D.toScreen(panel.VectorPoints[0], transform);
         pathDirection.append("M" + " " + (StartPoint.x) + " " + (StartPoint.y) + " ");
         for(var i=0; i< panel.VectorPoints.length; i++)
         {
            lineSegment = ej.EjSvgRender.chartTransform3D.toScreen(panel.VectorPoints[i], transform);
            pathDirection.append("L" + " " + (lineSegment.x) + " " + (lineSegment.y) + " ");
                     
         }        
          
         direction = pathDirection.toString();

        
         var optionsLine = {
             'id': panel.element.id,
             'stroke-dasharray': panel.element.dashArray,
             'stroke-width': panel.element.width,
             'stroke': panel.element.stroke,
             'd': direction
         };

         sender.svgRenderer.drawPath(optionsLine, panel.element.child);
     },

       drawLine: function (panel, sender) {
           
           var transform = ej.Ej3DRender.transform;
           if (transform == null) return;
           var actual3DPosition1 = ej.EjSvgRender.chartTransform3D.toScreen(panel.VectorPoints[0], transform);
           var actual3DPosition2 = ej.EjSvgRender.chartTransform3D.toScreen(panel.VectorPoints[2], transform);

           var optionsLine = {
               'id': panel.element.id,
               x1: actual3DPosition1.x ,
               y1: actual3DPosition1.y,
               x2: actual3DPosition2.x,
               y2:actual3DPosition2.y,
               'stroke-dasharray': panel.element.dashArray,
               'stroke-width': panel.element.width,
               'stroke': panel.element.stroke,
               'opacity': panel.element.opacity
           };

           sender.svgRenderer.drawLine(optionsLine, panel.element.child);
 

       },
       drawText: function (panel, sender) {
           
           var element = panel.element;
           var transform =ej.Ej3DRender.transform;
           if (transform == null) return;
           var actual3DPosition = ej.EjSvgRender.chartTransform3D.toScreen(panel.VectorPoints[0], transform);
           var x = actual3DPosition.x;
           var y = actual3DPosition.y;
 
           if (element.tag == "text") {
               var options = {
                   'id': element.id,
                   'x': x,
                   'y': y,
                   'fill': element.font.color,
                   'font-size': element.font.size,
                   'font-family': element.font.fontFamily,
                   'font-style': element.font.fontStyle,
                   'font-weight': element.font.fontWeight,
                   'opacity': element.font.opacity,
                   'text-anchor': element.TextAnchor,            
                   "cursor":"default",
                   'transform': (element.Transform) ? 'rotate(' + -90 + ',' + (x) + ',' + y + ')' : null
               };           
              
               sender.svgRenderer.drawText(options, panel.element.Label.Text, panel.element.child, panel.element.Label);			 
           }
           else {
               var series = element.series;           
 
               var labelFormat = (series.yAxis && series.yAxis.labelFormat) ? series.yAxis.labelFormat : "";
               var pointText = (element.point.text) ? element.point.text : element.point.y + labelFormat.substring(labelFormat.indexOf('}') + 1);
               var textOffset = ej.EjSvgRender.utils._measureText(pointText, null, series.marker.dataLabel.font);
               var margin = series.marker.dataLabel.margin;
               var width = textOffset.width + margin.left + margin.right;
               var height = textOffset.height + margin.top + margin.bottom;
               var location = { X: actual3DPosition.x, Y: actual3DPosition.y };

                  
                      if (series.marker.dataLabel.shape)
                          var elementShape = series.marker.dataLabel.shape;
                      else if (series.marker.dataLabel.shape)
                          elementShape = marker.dataLabel.shape;
                      else
                          elementShape = "None";

                      $.each(sender.model.symbolShape, function (name) {
                          if (elementShape.toLowerCase() == name.toLowerCase())
                              symbolName = name;
                      });

                   var xXalue = location.X - (margin.left) / 2 + (margin.right) / 2;
                   var yValue = location.Y - (margin.top) / 2 - (height / margin.top) + (margin.bottom) / 2;

                   var seriesIndex = $.inArray(series, sender.model.series);
                   ej.EjSeriesRender.prototype.dataLabelSymbol(seriesIndex, series, element.pointIndex, xXalue, yValue, width, height, symbolName, sender);
            
           }

       },

       draw: function (panel,sender) {            
           if (panel.VectorPoints == null || panel.VectorPoints.length <= 0) return;
           var transform = ej.Ej3DRender.transform;
           var pathDirection = ej.EjSvgRender.utils._getStringBuilder();
           var color = panel.Fill;
           var figure={Segments:null, StartPoint:null};
           if (transform != null)
           {
               figure.StartPoint = ej.EjSvgRender.chartTransform3D.toScreen(panel.VectorPoints[0], transform);
               pathDirection.append("M" + " " + (figure.StartPoint.x) + " " + (figure.StartPoint.y) + " ");
               for(var i=0; i< panel.VectorPoints.length; i++)
                {
                       lineSegment = ej.EjSvgRender.chartTransform3D.toScreen(panel.VectorPoints[i], transform);
                       pathDirection.append("L" + " " + (lineSegment.x) + " " + (lineSegment.y) + " ");
                     
                }
           }
          
           direction = pathDirection.toString();
          
    
            var lightCoefZ =(2 * (Math.abs(vector.vector3dAND(panel.normal, vector.vector3D(0, 0, 1))) - 1));
            var lightCoefY =(2 * (Math.abs(vector.vector3dAND(panel.normal, vector.vector3D(0, 1, 0))) - 1));
            var lightCoefX =(2 * (Math.abs(vector.vector3dAND(panel.normal, vector.vector3D(1, 0, 0))) - 1));
           if (lightCoefZ == lightCoefX)
           {               
                   color = this.applyZLight(color, sender);  
           }
          else if (((lightCoefY == lightCoefZ) || (lightCoefZ != 0 && lightCoefY < lightCoefZ)))
           {
            color= this.applyXLight(color, sender);
           }
          else if (lightCoefZ < 0)
          {
           color = this.applyZLight(color, sender);
          }
          else
          {
            color = color;
          }
          if (sender.model.AreaType == "none")
               panel.StrokeThickness = 0;
           var optionsMinorGrid = {
               'id': sender.svgObject.id +"_"+ panel.Name,
               'fill': color,
               'stroke': panel.Stroke,
               'stroke-width': panel.StrokeThickness,
               'opacity': panel.Opacity,
               'd': direction
           };
           sender.svgRenderer.drawPath(optionsMinorGrid, panel.Element.Parent);

       },

       applyXLight: function (color, sender)
       { 
           var RGB = sender.svgRenderer.hexToRGB(color);
           RGB.R = (parseInt)(RGB.R * 0.7);
           RGB.G = (parseInt)(RGB.G * 0.7);
           RGB.B = (parseInt)(RGB.B * 0.7);
           return sender.svgRenderer.hexFromRGB(RGB);
       },

       applyZLight:function(color, sender)
         {
           var RGB = sender.svgRenderer.hexToRGB(color);
           RGB.R = (parseInt)(RGB.R * 0.9);
           RGB.G = (parseInt)(RGB.G * 0.9);
           RGB.B = (parseInt)(RGB.B * 0.9);
           return sender.svgRenderer.hexFromRGB(RGB);
        },

       update: function (updatedVectors, panel, sender) {
           if (panel.VectorPoints == null || panel.VectorPoints.length <= 0) return;
           panel.VectorPoints = updatedVectors;
           var transform = ej.Ej3DRender.transform;
           var pathDirection = ej.EjSvgRender.utils._getStringBuilder();
           var color = panel.Fill;
           var figure = { Segments: null, StartPoint: null };
           if (transform != null) {
               figure.StartPoint = ej.EjSvgRender.chartTransform3D.toScreen(panel.VectorPoints[0], transform);
               pathDirection.append("M" + " " + (figure.StartPoint.x) + " " + (figure.StartPoint.y) + " ");
               for (var i = 0; i < panel.VectorPoints.length; i++) {
                   lineSegment = ej.EjSvgRender.chartTransform3D.toScreen(panel.VectorPoints[i], transform);
                   pathDirection.append("L" + " " + (lineSegment.x) + " " + (lineSegment.y) + " ");

               }
           }

           direction = pathDirection.toString();    

           if ($(sender.chartObj.chart3D).find("#" + sender.chartObj.svgObject.id + "_" + panel.Name).length > 0) {
               var element = $(sender.chartObj.chart3D).find("#" + sender.chartObj.svgObject.id + "_" + panel.Name)[0];
               sender.chartObj.svgRenderer._setAttr($(element), { 'd': direction });
              
           }
         
       }


   }
   var Ej3DRender = new ej.Ej3DRender();
   var vector = new Ej3DRender.vector3D();
   var matrixobj = new Ej3DRender.matrix3D();
   var bsptreeobj = new Ej3DRender.BSPTreeBuilder();
   var polygonobj = new Ej3DRender.polygon3D();
   var graphics = new Ej3DRender.Graphics3D();

  
})(jQuery);;
ej.Ej3DAxisRenderer = function () {
};

ej.Ej3DSeriesRender = function () {
};


ej.Ej3DChart = function () {
};

ej.series3DTypes = {};

(function ($) {

    ej.Ej3DAxisRenderer.prototype = {

        _drawAxes: function (index, axis) {
            if (axis.majorGridLines.visible)
                this._drawGridLines3D(axis);
            if (axis.majorTickLines.visible)
                this._renderTicks3D(axis, axis.majorTickLines.size, axis.majorTickLines.width);

            //if(axis.minorTickLines.visible)
            //this._renderTicks3D(axis, axis.minorTickLines.size, axis.minorTickLines.width);
            if (axis.visible) {
                this._drawAxisLabel(axis);
                this._drawAxisTitle(axis);
            }
        },

        _drawAxisTitle: function (axis) {
            if (axis.orientation.toLowerCase() == "horizontal") {
                var xtitleLocation = (this.model.elementSpacing) + axis._LableMaxWidth.height;
                var titlesize = (ej.EjSvgRender.utils._measureText(axis.title.text, (axis.width), axis.title.font).height / 2);
                var commonEventArgs = $.extend({}, ej.EjSvgRender.commonChartEventArgs);
                commonEventArgs.data = { Text: axis.title.text, location: { x: (axis.width) / 2, y: (xtitleLocation + titlesize + axis.majorTickLines.size + axis.axisLine.width) }, axes: axis };
                this._trigger("axesTitleRendering", commonEventArgs);

                var x1 = commonEventArgs.data.location.x + axis.x;
                var y1 = (axis.opposedPosition) ? (axis.y - commonEventArgs.data.location.y) : (commonEventArgs.data.location.y + axis.y);

                element = { Width: titlesize.width, Height: titlesize.height, Label: commonEventArgs.data, TextAnchor: 'middle', tag: 'text', font: axis.title.font, id: this.svgObject.id + "axisTitle", child: this.chart3D };

                this.graphics.addVisual(this.polygon.createTextElement(this.vector.vector3D(x1, y1, 0), element, 10, 10));
            }
            else {
                var titleSize = ej.EjSvgRender.utils._measureText(axis.title.text, (axis.height), axis.title.font);
                var x1 = (axis.opposedPosition) ? axis.x + ((3 * this.model.elementSpacing) + axis._LableMaxWidth.width + axis.majorTickLines.size + axis.axisLine.width) : axis.x - ((3 * this.model.elementSpacing) + axis._LableMaxWidth.width + axis.majorTickLines.size + axis.axisLine.width);

                var commonEventArgs = $.extend({}, ej.EjSvgRender.commonChartEventArgs);
                commonEventArgs.data = { Text: axis.title.text, location: { x: titleSize.width / 2, y: 0 }, axes: axis };
                this._trigger("axesTitleRendering", commonEventArgs);

                var y1 = commonEventArgs.data.location.y + (axis.y + axis.height) + (((axis.height) / 2) * -1);

                var labelRotation = (axis.opposedPosition) ? 90 : -90;

                element = { Width: titleSize.width, Height: titleSize.height, Transform:true , Label: commonEventArgs.data, TextAnchor: 'middle', tag: 'text', font: axis.title.font, id: this.svgObject.id + '_YAxisTitle', child: this.chart3D };

                this.graphics.addVisual(this.polygon.createTextElement(this.vector.vector3D(x1, y1, 0), element, 10, 10));
            }
        },

        _drawAxisLabel: function (axis) {
            var labelsCount = axis.visibleLabels.length;
           
            for (var i = 0; i < labelsCount; i++) {
                var x1 = 0, y1 = 0, x2 = 0, y2 = 0;
                var textAnchor;
                
                var textSize = ej.EjSvgRender.utils._measureText(axis.visibleLabels[i].Text, this.model.m_AreaBounds.Width, axis.font);
                var value = ej.EjSvgRender.utils._valueToCoefficient(axis, axis.visibleLabels[i].Value);
                value = isNaN(value) ? 0 : value;

                if (axis.orientation.toLowerCase() == "horizontal") {
                    x1 = x2 = Math.round(axis.plotOffset + this.model.m_AreaBounds.X + (this.model.m_AreaBounds.Width * value));
                    y1 = (axis.opposedPosition) ? (this.model.m_AreaBounds.Y - this.model.wallSize - (axis.majorTickLines.size) - (this.model.elementSpacing/2)) : this.model.m_AreaBounds.Y + this.model.m_AreaBounds.Height + this.model.wallSize + (axis.majorTickLines.size) + this.model.elementSpacing;
                    textAnchor="middle"
                }
                else {
                    y1 = y2 = Math.round(axis.plotOffset + this.model.m_AreaBounds.Y + (textSize.height/4) + (this.model.m_AreaBounds.Height * (1 - value)));
                    x1 = (axis.opposedPosition) ? (this.model.m_AreaBounds.X + this.model.m_AreaBounds.Width + this.model.wallSize + axis.majorTickLines.size + this.model.elementSpacing / 2) : (this.model.m_AreaBounds.X - this.model.wallSize - axis.majorTickLines.size - this.model.elementSpacing / 2);
                    textAnchor = (axis.opposedPosition) ?"start" : "end"
                }
 
             
                element = { Width: textSize.width, Height: textSize.height, Label: axis.visibleLabels[i], TextAnchor: textAnchor, tag: 'text', font: axis.font, id: this.svgObject.id + axis.orientation + i, child: this.chart3D };

                this.graphics.addVisual(this.polygon.createTextElement(this.vector.vector3D(x1, y1, 0), element, 10, 10));
            }

        },

        _renderTicks3D: function (axis, size, width) {
            var labelsCount = axis.visibleLabels.length;
            var parent;
            var gEle;

            for (var i = 0; i < labelsCount; i++)
            {
                var x1 = 0, x2 = 0, y1=0, y2=0;
            
       
                var value =ej.EjSvgRender.utils._valueToCoefficient(axis, axis.visibleLabels[i].Value);
                value = isNaN(value) ? 0 : value;
            
                if (axis.orientation.toLowerCase() == "horizontal")
                {
                    x1 = x2 = Math.round(axis.plotOffset + (this.model.m_AreaBounds.Width * value));
                }
                else
                {
                    y1 = y2 = Math.round(axis.plotOffset + ((this.model.m_AreaBounds.Height) * (1 - value)));
                }

                var position = this._calculatePosition3D(axis, axis.tickLinesPosition, size, width, x1, y1, x2, y2);


                var line = { width: axis.majorTickLines.width, stroke: axis.majorTickLines.color, child: this.chart3D,  tag: 'line' };

                line.id = axis.name + "_majorTickLines_" + i;

                this.graphics.addVisual(this.polygon.createLine(line, position.X1, position.Y1, position.X2, position.Y2, 0));

                if (axis.minorGridLines.visible && axis.minorTicksPerInterval > 0 && i < labelsCount - 1) {
                    minorTicks = axis.visibleRange.interval / (axis.minorTicksPerInterval + 1);
                    for (var k = 0; k < axis.minorTicksPerInterval; k++) {
                        value = ej.EjSvgRender.utils._valueToCoefficient(axis, axis.visibleLabels[i].Value + (minorTicks * (k + 1)));

                        value = isNaN(value) ? 0 : value;

                        if (axis.orientation.toLowerCase() == "horizontal") {
                            x1 = x2 = Math.round(axis.plotOffset + (this.model.m_AreaBounds.Width * value));
                        }
                        else {
                            y1 = y2 = Math.round(axis.plotOffset + ((this.model.m_AreaBounds.Height) * (1 - value)));
                        }

                        var position = this._calculatePosition3D(axis, axis.tickLinesPosition, size, width, x1, y1, x2, y2);


                        var line = { width: axis.minorTickLines.width, stroke: axis.minorTickLines.color, child: this.chart3D, tag: 'line' };

                        line.id = axis.name + "_minorTickLines_" + i + k;

                        this.graphics.addVisual(this.polygon.createLine(line, position.X1, position.Y1, position.X2, position.Y2, 0));
                    }
                }
            }
        },

        _calculatePosition3D: function (axis, ticksPosition, tickSize, width, x1, y1, x2, y2) {
            var orientation = axis.orientation;
            var isOpposed = axis.opposedPosition;
            ticksPosition = "outside";
            if (axis.orientation.toLowerCase() == "horizontal") {
                switch (ticksPosition) {
                    case "inside":
                        y1 = isOpposed ? width : 0;
                        y2 = isOpposed ? y1 + tickSize : tickSize;
                        break;
                    case "outside":
                        y1 = isOpposed ? 0 : axis.axisLine.width;
                        y2 = isOpposed ? tickSize : y1 + tickSize;
                        break;
                }
                var areabounds = this.model.m_AreaBounds;
                var screenPositionTop = ((isOpposed) ? this.model.m_AreaBounds.Y - this.model.wallSize - (tickSize) : this.model.m_AreaBounds.Y + this.model.m_AreaBounds.Height + this.model.wallSize - (tickSize / 2));
                var screenPositionLeft = axis.x;
                y1 += screenPositionTop;
                y2 += screenPositionTop;

                x1 += screenPositionLeft;
                x2 += screenPositionLeft;
            }
            else {
                switch (ticksPosition) {
                    case "inside":
                        x1 = isOpposed ? 0 : axis.axisLine.width;
                        x2 = isOpposed ? tickSize : x1 + tickSize;
                        break;
                    case "outside":
                        x1 = isOpposed ? axis.axisLine.width : 0;
                        x2 = isOpposed ? x1 + tickSize : tickSize;
                        break;
                }
                var screenPositionLeft = (isOpposed) ? this.model.m_AreaBounds.X + this.model.m_AreaBounds.Width + this.model.wallSize  : (this.model.m_AreaBounds.X - this.model.wallSize - tickSize);
                var screenPositionTop = this.model.m_AreaBounds.Y;
                x1 += screenPositionLeft;
                x2 += screenPositionLeft;

                y1 += screenPositionTop;
                y2 += screenPositionTop;
            }
            return { X1: x1, Y1: y1, X2: x2, Y2: y2 };
        },

    

        _drawGridLines3D:function(axis)
        {
          
            if (axis == null)
                return;
        
            var labelsCount = axis.visibleLabels.length;
            var minorTicks;
            var orientation = axis.orientation;
            var x1, x2, y1, y2;
            if (orientation.toLowerCase() == "horizontal")
            {
                 
                var i;
                for (i = 0; i < labelsCount; i++)
                {                
                    var value = ej.EjSvgRender.utils._valueToCoefficient(axis, axis.visibleLabels[i].Value);
                    value = isNaN(value) ? 0 : value;
                    x2 = x1 = (Math.round(this.model.m_AreaBounds.Width * value) + this.model.m_AreaBounds.X);
                    y1 = this.model.m_AreaBounds.Y;
                    y2 = this.model.m_AreaBounds.Y + this.model.m_AreaBounds.Height;
              
                
                    var depth = this.model.depth > 2 ? this.model.depth - 2 : 1;
                    var bottom = axis.opposedPosition ? this.model.m_AreaBounds.Y : this.model.m_AreaBounds.Height + this.model.m_AreaBounds.Y;

                    var line = { opacity: axis.majorGridLines.opacity, width: axis.majorGridLines.width, stroke: axis.majorGridLines.color, child: this.chart3D,  tag: 'line' };
                    line.id = this.svgObject.id + axis.name + "_gridlines_" + i;
                    this.graphics.addVisual(this.polygon.createLine(line, x1, y1, x2, y2, depth));
                    var parallelLine = $.extend({}, line);
                    parallelLine.id = this.svgObject.id + axis.name + "_parallelGridlines_" + i;
                  
                    var line3D =   this.polygon.createLine(parallelLine, x2, 0, x2, -depth, bottom);
                    //To fold the gridline alone the wall(bottom)
                    this.polygon.transform(this.matrixobj.tilt((parseFloat)(Math.PI / 2)), line3D);
                        
                    this.graphics.addVisual(line3D);
                    
                    if (axis.minorGridLines.visible && axis.minorTicksPerInterval > 0 && i<labelsCount-1) {
                        minorTicks = axis.visibleRange.interval / (axis.minorTicksPerInterval + 1);
                        for (var k = 0; k < axis.minorTicksPerInterval; k++) {
                            value = ej.EjSvgRender.utils._valueToCoefficient(axis, axis.visibleLabels[i].Value + (minorTicks * (k+1)));
                            value = isNaN(value) ? 0 : value;
                            x2 = x1 = (Math.round(this.model.m_AreaBounds.Width * value) + this.model.m_AreaBounds.X);
                            y1 = this.model.m_AreaBounds.Y;
                            y2 = this.model.m_AreaBounds.Y + this.model.m_AreaBounds.Height;

                            var line = { opacity: axis.minorGridLines.opacity, width: axis.minorGridLines.width, stroke: axis.minorGridLines.color, child: this.chart3D, tag: 'line' };
                            line.id = this.svgObject.id + axis.name + "_minorgridlines_" + i + k;
                            this.graphics.addVisual(this.polygon.createLine(line, x1, y1, x2, y2, depth));
                            var parallelLine = $.extend({}, line);
                            parallelLine.id = this.svgObject.id + axis.name + "_parallelMinorGridlines_" + i + k;

                            var line3D = this.polygon.createLine(parallelLine, x2, 0, x2, -depth, bottom);
                            //To fold the gridline alone the wall(bottom)
                            this.polygon.transform(this.matrixobj.tilt((parseFloat)(Math.PI / 2)), line3D);

                            this.graphics.addVisual(line3D);
                        }
                    }
                
                    index++;
                }
            }
            else
            {
                var i;
       
              
                for (i = 0; i < labelsCount; i++)
                {
                    //if (i < linesCount)
                    //{
                    var line;
                    var value = ej.EjSvgRender.utils._valueToCoefficient(axis, axis.visibleLabels[i].Value);
                    value = isNaN(value) ? 0 : value;
                    x1 = this.model.m_AreaBounds.X;
                    y1 = Math.round((axis.height) * (1 - value)) + 0.5;
                    y1 += axis.y;
                    x2 = x1 + this.model.m_AreaBounds.Width;
                    y2 = y1;

                    var depth = this.model.depth > 2 ? this.model.depth - 2 : 1;

                    var line = { opacity: axis.majorGridLines.opacity, width: axis.majorGridLines.width, stroke: axis.majorGridLines.color, axisName: axis.name, child: this.chart3D, tag: 'line' };
                 
                    line.id = this.svgObject.id + axis.name + "_gridlines_" + i;

                    this.graphics.addVisual(this.polygon.createLine(line, x1, y1, x2, y2, depth));

                    var depthD = axis.opposedPosition ? this.model.m_AreaBounds.Width + this.model.m_AreaBounds.X + 1 : this.model.m_AreaBounds.X;

                    var sideLine = $.extend({}, line);
                    sideLine.id = this.svgObject.id + axis.name + "_parallelGridlines_" + i;
              
                    var line3D = this.polygon.createLine(sideLine, -depth, y2, 0, y2, depthD);
                    //To fold the gridline alone the wall(right of vertical)
                    this.polygon.transform(this.matrixobj.turn((parseFloat)(-Math.PI / 2)), line3D);
                    this.graphics.addVisual(line3D);

                    if (axis.minorGridLines.visible && axis.minorTicksPerInterval > 0 && i < labelsCount - 1) {
                        minorTicks = axis.visibleRange.interval / (axis.minorTicksPerInterval + 1);
                        for (var k = 0; k < axis.minorTicksPerInterval; k++) {
                            var value = ej.EjSvgRender.utils._valueToCoefficient(axis, axis.visibleLabels[i].Value + (minorTicks * (k + 1)));
                            value = isNaN(value) ? 0 : value;
                            x1 = this.model.m_AreaBounds.X;
                            y1 = Math.round((axis.height) * (1 - value)) + 0.5;
                            y1 += axis.y;
                            x2 = x1 + this.model.m_AreaBounds.Width;
                            y2 = y1;
                            var line = { opacity: axis.minorGridLines.opacity, width: axis.minorGridLines.width, stroke: axis.minorGridLines.color, axisName: axis.name, child: this.chart3D, tag: 'line' };

                            line.id = this.svgObject.id + axis.name + "_minorgridlines_" + i + k;

                            this.graphics.addVisual(this.polygon.createLine(line, x1, y1, x2, y2, depth));                          

                            var sideLine = $.extend({}, line);
                            sideLine.id = this.svgObject.id + axis.name + "_parallelMinorGridlines_" + i + k;

                            var line3D = this.polygon.createLine(sideLine, -depth, y2, 0, y2, depthD);
                            //To fold the gridline alone the wall(right of vertical)
                            this.polygon.transform(this.matrixobj.turn((parseFloat)(-Math.PI / 2)), line3D);
                            this.graphics.addVisual(line3D);
                        }
                    }

                    ////}
                    index++;
                }

            }
        }

    },

    

    ej.Ej3DSeriesRender.prototype={
        _getSegmentDepth:function(series)
        {
            var actualDepth = this.chartObj.model.depth;
            var start, end;

            if (this.chartObj.model.sideBySideSeriesPlacement)
            {
                var space = actualDepth / 4;
                start = space;
                end = space * 3;
            }
            else
            {
                var index = series.position-1;
                var count = series.all;
                var space = actualDepth / ((count * 2) + count + 1);
                start = space + (space * index * 3);
                end = start + space * 2;
            }
            return {Start:start, End:end, Delta:end-start};
        },

        createSegment: function (center, start, end, height, r, i, y, insideRadius, pointindex) {
         
            return {
                StartValue : start,
                EndValue : end,
                depth : height,
                radius : r,      
                index : i,
                YData : y,
                Center : center,
                inSideRadius : insideRadius,
                ActualEndValue : end,
                ActualStartValue: start,
                pointIndex: pointindex
            }

            return segment;

        },

        calculateSize:function(sender, series){
            var legend = sender.chartObj.model.legend;
   
            var legXSpace = 0;
            var legYSpace = 0;
            if (legend.visible && legend.position.toLowerCase() != "custom") {
                if (legend.position.toLowerCase() == "right" || legend.position.toLowerCase() == "left")
                    legXSpace = ((legend.position.toLowerCase() == "right") ? sender.chartObj.model.margin.right : sender.chartObj.model.margin.left) + sender.chartObj.model.LegendBounds.Width;
                else
                    legYSpace = ((legend.position.toLowerCase() == "top") ? sender.chartObj.model.margin.top : sender.chartObj.model.margin.bottom) + sender.chartObj.model.LegendBounds.Height;

            }

            var yOffset = ((sender.chartObj.model.title.text) ? sender.chartObj.model._titleLocation.Y : 0) + legYSpace;

            series.actualWidth = $(sender.chartObj.svgObject).width() - legXSpace;
            series.actualHeight = $(sender.chartObj.svgObject).height() - yOffset;
        },

        createPoints: function (series, sender) {
            series.segments = [];
            this.calculateSize(sender, series);
            var all = 0;
            var visiblepoints = this._pieVisiblePoints(series);
            for (j = 0; j < visiblepoints.length; j++) {
            all += visiblepoints[j].YValues[0];
            }
            var coef = 360 / all;
            var count = series.points.length;
            var radius =(series.radius)? series.radius : (series.pieCoefficient * Math.min(series.actualWidth, series.actualHeight)) / 2;

            var InsideRadius = (series.InsideRadius) ? series.InsideRadius : 0;
            var YValues = sender.chartObj._getYValues(series);

            var pieHeight = sender.chartObj.model.depth;
            if (series.explodeIndex >= 0 && series.explodeIndex < count || series.explodeAll) {
                radius -= series.explodeOffset;
            }
            var arcStartAngle = 0, arcEndAngle = 0;
            var current = 0;
            var segindex = 0;
            for (var i = 0; i < count; i++) {
                if (series.points[i].visible) {                    
                    var pointindex = $.inArray(series.points[i], series.points);
                var val = Math.abs(YValues[i]);
                arcEndAngle = Math.abs(val) * ((Math.PI * 2) / all);
                var rect = {};
                rect.x = 0;
                rect.y = 0;

                if (val != 0) {
                    if (series.explodeIndex == i || series.explodeAll) {
                        var offset = {X:0, Y:0};
                        offset.X = Math.cos(2 * Math.PI * (current + val / 2) / all),
                        offset.Y = Math.sin(2 * Math.PI * (current + val / 2) / all);
                        rect.x = 0.01 * radius * offset.X * series.explodeOffset;
                        rect.y = 0.01 * radius * offset.Y * series.explodeOffset;
                    }

                    var center = sender.chartObj.vector.vector3D(rect.x + series.actualWidth / 2, rect.y + series.actualHeight / 2, 0);
                    {
                            var segment = this.createSegment(center, (parseFloat)(coef * current), (parseFloat)(coef * val), pieHeight, radius, i, val, InsideRadius, pointindex, series);
                        series.segments.push(segment);
                    }
                }

                if (series.marker.dataLabel.visible)
                    this._addPieDataLabel(segindex, YValues[i], arcStartAngle, arcStartAngle + arcEndAngle, i, radius, this._isChartRotated(sender) ? sender.chartObj.model.depth + 5 : 0, center, series.points[i]);

                segindex++;
                arcStartAngle += arcEndAngle;
                current += val;

                }
            }
            return series.segments;
        },

        _addPieDataLabel: function (x, y, startAngle, endAngle, i, radius, startDepth, center, point) {
            var angle = (startAngle + endAngle) / 2;
            point.symbolLocation = { x: 0, y: 0, radius: 0, angle: 0, StartDepth: 0 };
            point.symbolLocation.x = x;
            point.symbolLocation.y = y;
            point.symbolLocation.radius = radius;
            point.symbolLocation.angle = angle;
            point.symbolLocation.center = center;
            point.startDepth = startDepth;
        },

        _isChartRotated:function(sender)
           {
            var actualTiltView = Math.abs(sender.chartObj.model.tilt % 360);
            var actualRotateView = Math.abs(sender.chartObj.model.rotattion % 360);
            if ((actualTiltView > 90 && actualTiltView < 270) ^ (actualRotateView > 90 && actualRotateView < 270))
              {
                return true;
              }
                  return false;
          },

        createSector: function (seg, sender, style) {
            var count = (parseInt)(Math.ceil(seg.ActualEndValue / 6));
            var DtoR = Math.PI / 180;
            var inc = 0;
            var Points = [];
            var depth = sender.chartObj.model.depth;
            if (count < 1) return null;
            var res = [];
            var f = seg.ActualEndValue / count;

            var oPts = [];
            var iPts = [];

            for (var i = 0; i < count + 1; i++) {
                var ox = (parseFloat)(seg.Center.x + seg.radius * Math.cos((seg.ActualStartValue + i * f) * DtoR));
                var oy = (parseFloat)(seg.Center.y + seg.radius * Math.sin((seg.ActualStartValue + i * f) * DtoR));

                oPts[i] = { X: ox, Y: oy };

                var ix = (parseFloat)(seg.Center.x + seg.inSideRadius * Math.cos((seg.ActualStartValue + i * f) * DtoR));
                var iy = (parseFloat)(seg.Center.y + seg.inSideRadius * Math.sin((seg.ActualStartValue + i * f) * DtoR));

                iPts[i] = { X: ix, Y: iy };
                Points.push({ X: ox, Y: oy });
            }

            var oPlgs = [];

            for (var i = 0; i < count; i++) {
                vts = new Array(sender.chartObj.vector.vector3D(oPts[i].X, oPts[i].Y, 0),
                       sender.chartObj.vector.vector3D(oPts[i].X, oPts[i].Y, depth),
                       sender.chartObj.vector.vector3D(oPts[i + 1].X, oPts[i + 1].Y, depth),
                       sender.chartObj.vector.vector3D(oPts[i + 1].X, oPts[i + 1].Y, 0))


                oPlgs[i] = sender.chartObj.polygon.polygon3D(vts, this, seg.index, style.borderColor, style.borderWidth, style.opacity, style.interior, inc.toString()  + "Region" + "_Series_" + 0 + "path" + seg.index, sender.chartObj.chart3D);
                inc++;
            }

            res[1] = oPlgs;

            if (seg.inSideRadius > 0) {
                var iPlgs = [];

                for (var i = 0; i < count; i++) {
                    vts = new Array(sender.chartObj.vector.vector3D(iPts[i].X, iPts[i].Y, 0),
                      sender.chartObj.vector.vector3D(iPts[i].X, iPts[i].Y, depth),
                      sender.chartObj.vector.vector3D(iPts[i + 1].X, iPts[i + 1].Y, depth),
                      sender.chartObj.vector.vector3D(iPts[i + 1].X, iPts[i + 1].Y, 0))


                    iPlgs[i] = sender.chartObj.polygon.polygon3D(vts, this, seg.index, style.borderColor, style.borderWidth, style.opacity, style.interior, inc.toString() + "Region" + "_Series_" + 0 + "path" + seg.index, sender.chartObj.chart3D);
                    inc++;
                }

                res[3] = iPlgs;
            }

            var tVtxs = [];
            var bVtxs = [];

            for (var i = 0; i < count + 1; i++) {
                tVtxs.push(sender.chartObj.vector.vector3D(oPts[i].X, oPts[i].Y, 0));
                bVtxs.push(sender.chartObj.vector.vector3D(oPts[i].X, oPts[i].Y, depth));
            }

            if (seg.inSideRadius > 0) {
                for (var i = count; i > -1; i--) {
                    tVtxs.push(sender.chartObj.vector.vector3D(iPts[i].X, iPts[i].Y, 0));
                    bVtxs.push(sender.chartObj.vector.vector3D(iPts[i].X, iPts[i].Y, depth));
                }
            }
            else {
                tVtxs.push(seg.Center);
                bVtxs.push(sender.chartObj.vector.vector3D(seg.Center.x, seg.Center.y, depth));
            }
            res[0] = [];
            res[0].push(sender.chartObj.polygon.polygon3D(tVtxs, this, seg.index, style.borderColor, style.borderWidth, style.opacity, style.interior, inc.toString() + "Region" + "_Series_" + 0 + "path" + seg.index, sender.chartObj.chart3D));
            inc++;
            res[0].push(sender.chartObj.polygon.polygon3D(bVtxs, this, seg.index, style.borderColor, style.borderWidth, style.opacity, style.interior, inc.toString() + "Region" + "_Series_" + 0 + "path" + seg.index, sender.chartObj.chart3D));
            inc++;



            if (seg.inSideRadius > 0) {
                var rvts = new Array(

                      sender.chartObj.vector.vector3D(oPts[0].X, oPts[0].Y, 0),
                      sender.chartObj.vector.vector3D(oPts[0].X, oPts[0].Y, depth),
                      sender.chartObj.vector.vector3D(iPts[0].X, iPts[0].Y, depth),
                      sender.chartObj.vector.vector3D(iPts[0].X, iPts[0].Y, 0)
                 )

                var lvts = new Array
                 (
                     sender.chartObj.vector.vector3D(oPts[count].X, oPts[count].Y, 0),
                     sender.chartObj.vector.vector3D(oPts[count].X, oPts[count].Y, depth),
                     sender.chartObj.vector.vector3D(iPts[count].X, iPts[count].Y, depth),
                     sender.chartObj.vector.vector3D(iPts[count].X, iPts[count].Y, 0)
                 )
                res[2] = [];

                res[2].push(sender.chartObj.polygon.polygon3D(rvts, this, seg.index, style.borderColor, style.borderWidth, style.opacity, style.interior, inc.toString() + "Region" + "_Series_" + 0 + "path" + seg.index, sender.chartObj.chart3D));
                inc++;
                res[2].push(sender.chartObj.polygon.polygon3D(lvts, this, seg.index, style.borderColor, style.borderWidth, style.opacity, style.interior, inc.toString() + "Region" + "_Series_" + 0 + "path" + seg.index, sender.chartObj.chart3D));
                inc++;             

            }
            else {
                var rvts = new Array(
                     sender.chartObj.vector.vector3D(oPts[0].X, oPts[0].Y, 0),
                     sender.chartObj.vector.vector3D(oPts[0].X, oPts[0].Y, depth),
                     sender.chartObj.vector.vector3D(seg.Center.x, seg.Center.y, depth),
                     sender.chartObj.vector.vector3D(seg.Center.x, seg.Center.y, 0)
                )

                var lvts = new Array(
                     sender.chartObj.vector.vector3D(oPts[count].X, oPts[count].Y, 0),
                     sender.chartObj.vector.vector3D(oPts[count].X, oPts[count].Y, depth),
                     sender.chartObj.vector.vector3D(seg.Center.x, seg.Center.y, depth),
                     sender.chartObj.vector.vector3D(seg.Center.x, seg.Center.y, 0)
                )
                res[2] = [];
                res[2].push(sender.chartObj.polygon.polygon3D(rvts, this, seg.index, style.borderColor, style.borderWidth, style.opacity, style.interior, inc.toString() + "Region" + "_Series_" + 0 + "path" + seg.index, sender.chartObj.chart3D));
                inc++;
                res[2].push(sender.chartObj.polygon.polygon3D(lvts, this, seg.index, style.borderColor, style.borderWidth, style.opacity, style.interior, inc.toString() + "Region" + "_Series_" + 0 + "path" + seg.index, sender.chartObj.chart3D));
                inc++;


            }


            return res;
        },

        createPolygons: function (series, sender, returnPoly) {
            var segments;
            if (!series.segments || !returnPoly)
                segments = this.createPoints(series, sender);
            else
                segments = series.segments
            var poligons = [];
            var seg;
            var style;

            for (var i = 0; i < segments.length; i++) {
                seg = segments[i];
                var pointindex = segments[i].pointIndex;
                style = sender.chartObj.setStyle(sender, series, 0, pointindex);
                if (typeof (sender.chartObj.model.pointColors[pointindex]) == "object")
                    style.interior = sender.chartObj.model.pointColors[pointindex][0].color;
                var plgs = this.createSector(seg, sender, style);
                if (plgs != null)
                    for (var ai = 0; ai < plgs.length; ai++) {
                        if (!poligons[ai])
                            poligons[ai] = [];
                        if (plgs[ai] != null) {
                            for (var pi = 0; pi < plgs[ai].length; pi++) {
                                poligons[ai].push(plgs[ai][pi]);

                            }
                        }
                    }


            }
            if (returnPoly) {
                return poligons;               
            }

            for (var ai = 0; ai < poligons.length; ai++) {
                for (var k = 0; k < poligons[ai].length; k++) {
                    sender.chartObj.graphics.addVisual(poligons[ai][k]);
                }
            }
        },

        draw3DDataLabel: function (series, pointIndex, point, sender) {


            if (this.chartObj.model.AreaType == "none") {
                location = { x: 0, y: 0 };
                var pointText = (point.text) ? point.text : point.y;

                var commonEventArgs = $.extend({}, ej.EjSvgRender.commonChartEventArgs);
                commonEventArgs.data = { Text: pointText, location: { x: pointX, y: pointY }, series: series, pointIndex: pointIndex };
                this.chartObj._trigger("displayTextRendering", commonEventArgs);

                var size = ej.EjSvgRender.utils._measureText(pointText, null, series.marker.dataLabel.font);

                if (series.type.toLowerCase() == "pie") {
                    var width = Math.min(series.actualWidth, series.actualHeight) / 2;
                    var center = point.symbolLocation.center;

                    if (series != null && series.labelPosition.toLowerCase() != "inside") {
                        radius = point.symbolLocation.radius + ((series.marker.dataLabel.connectorLine.height) ? (series.marker.dataLabel.connectorLine.height) :0);
                    }
                    else if (series != null) {
                        radius = point.symbolLocation.radius / 2;
                    }
                   
                    pointX = location.X = center.x + radius * Math.cos(point.symbolLocation.angle);
                    pointY = location.Y = center.y + radius * Math.sin(point.symbolLocation.angle);

                }
                else if (series.type.toLowerCase() == "doughnut")
                {
                   
                    var width = Math.min(series.actualWidth, series.actualHeight) / 2;
                    var center = point.symbolLocation.center;
                   
                    var dradius = point.symbolLocation.radius + ((series.marker.dataLabel.connectorLine.height) ? (series.marker.dataLabel.connectorLine.height) :0);
                   
                    if (series != null &&series.labelPosition != "inside")
                    {
                        radius = point.symbolLocation.radius + ((series.marker.dataLabel.connectorLine.height) ? (series.marker.dataLabel.connectorLine.height) : 0);
                    }
                    else if (series != null )
                    {
                        radius = dradius + (point.symbolLocation.radius-dradius) / 2;
                    }
                    pointX=location.X = center.x + radius * Math.cos(point.symbolLocation.angle);
                    pointY=location.Y = center.y + radius * Math.sin(point.symbolLocation.angle);
                }
            }
            else {
                var labelFormat = series.yAxis.labelFormat ? series.yAxis.labelFormat : "";
                var seriesIndex = $.inArray(series, this.chartObj.model.series);
                var pointText = (point.text) ? point.text : point.y + labelFormat.substring(labelFormat.indexOf('}') + 1);
                var pointHeight = 0;
                var commonEventArgs = $.extend({}, ej.EjSvgRender.commonChartEventArgs);
                commonEventArgs.data = { Text: pointText, location: { x: pointX, y: pointY }, series: series, pointIndex: pointIndex };
                this.chartObj._trigger("displayTextRendering", commonEventArgs);

                var size = ej.EjSvgRender.utils._measureText(pointText, null, series.marker.dataLabel.font);

                var location = ej.EjSvgRender.utils.Transform3DToVisible(series, point.symbolLocation.x, point.symbolLocation.y, this.chartObj);
                var connectorHeight = (series.marker.dataLabel.connectorLine.height) ? series.marker.dataLabel.connectorLine.height : 0;
                var angle = (6.28 * (1 - (90 / 360.0)));
                var pointY = location.Y;
                var pointX = location.X;
                if (!this.chartObj.model.requireInvertedAxes) {
                    if (point.y > 0)
                        pointY = location.Y + (Math.sin((angle)) * connectorHeight);
                    else
                        pointY = location.Y + (Math.sin((-angle)) * connectorHeight);
                }
                else {
                    if (point.x > 0)
                        pointX = location.X - (Math.sin((angle)) * connectorHeight);
                    else
                        pointX = location.X + (Math.sin((angle)) * connectorHeight);
                      pointHeight = - size.height / 3;
                }
            }

         
          
            var depthInfo = this._getSegmentDepth(series);

            

            if (series.marker.dataLabel.shape.toLowerCase() != "none") {
              
                var element = { tag: 'dataLabel', series: series, point: point, pointIndex: pointIndex, id: this.chartObj.svgObject.id + seriesIndex + '_DataLabel' + pointIndex, child: this.chartObj.chart3D };

                this.chartObj.graphics.addVisual(this.chartObj.polygon.createTextElement(this.chartObj.vector.vector3D(pointX, pointY, (point.symbolLocation.z) ? point.symbolLocation.z : 0), element, 0, -size.height));
            }

          
            var element = { Width: size.width, Height: size.height, Label: commonEventArgs.data, TextAnchor: "middle", tag: 'text', font: series.marker.dataLabel.font, id: this.chartObj.svgObject.id + '_SeriesText' + pointIndex + seriesIndex, child: this.chartObj.chart3D };

            if (this.chartObj.model.AreaType != "none") {
                if (series.marker.dataLabel.connectorLine.height && series.marker.dataLabel.connectorLine.height > 0) {
                    var drawPoints = [];
                    drawPoints.push({ x: location.X, y: location.Y + pointHeight, z: point.symbolLocation.z });

                    drawPoints.push({ x: pointX, y: pointY + pointHeight, z: point.symbolLocation.z });
                    this.drawLineSegment(drawPoints, pointIndex, series);
                }
                this.chartObj.graphics.addVisual(this.chartObj.polygon.createTextElement(this.chartObj.vector.vector3D(pointX, pointY, point.symbolLocation.z), element, 0, -size.height));
            }
            else {
                if (series.labelPosition.toLowerCase() == "outside")
                    this.updateConnectorLine(point, pointIndex, series, location, sender)
                var finalSize = { x: series.actualWidth, y: series.actualHeight };
                this.chartObj.graphics.addVisual(this.chartObj.polygon.createTextElement(this.chartObj.vector.vector3D(location.X, location.Y, -1), element, 0, -size.height));
            }        
        },
        updateConnectorLine: function (point,pointIndex, series, location) {
            var drawPoints = [];
            var symbolLocation = point.symbolLocation;
            var x = symbolLocation.center.x + Math.cos(symbolLocation.angle) * symbolLocation.radius;
            var y = symbolLocation.center.y + Math.sin(symbolLocation.angle) * symbolLocation.radius;
            drawPoints.push({ x: x, y: y });
            var height = (series.marker.dataLabel.connectorLine.height) ? series.marker.dataLabel.connectorLine.height : 0;
            var labelRadiusFromOrigin = symbolLocation.radius + height;
            x = symbolLocation.center.x + (Math.cos((symbolLocation.angle)) * (labelRadiusFromOrigin));
            y = symbolLocation.center.y + (Math.sin((symbolLocation.angle)) * (labelRadiusFromOrigin));
           
            drawPoints.push({ x: x, y: y });
            //var hipen = height / 5;
            //x += (x > symbolLocation.center.x) ? hipen : -hipen;
            //drawPoints.push({ x: x, y: y });
            if (series.marker.dataLabel.connectorLine.type =="bezier")
                drawPoints = this.getBezierApproximation(drawPoints, 256, series);
            this.drawLineSegment(drawPoints, pointIndex, series)
        },
        /// <returns></returns>
        getBezierApproximation: function (controlPoints, outputSegmentCount, series)
        {
            var points = [];
             for (var i = 0; i <= outputSegmentCount; i++)
             {
               var t =i / outputSegmentCount;
               points.push(this.GetBezierPoint(t, controlPoints, 0, controlPoints.length, series));
             }
            return points;
         },
        drawLineSegment: function (drawpoints, pointIndex, series) {
            var vectorPoints = [];
            for (var i = 0; i < drawpoints.length; i++) {
                vectorPoints.push(this.chartObj.vector.vector3D(drawpoints[i].x, drawpoints[i].y, drawpoints[i].z ? drawpoints[i].z:0));
            }
            var seriesIndex=$.inArray(series, this.chartObj.model._visibleSeries);
            var color = (this.chartObj.model.AreaType=="none") ? this.chartObj.model.pointColors[pointIndex] : this.chartObj.model.seriesColors[seriesIndex];
            var stroke = (series.marker.dataLabel.connectorLine.stroke)?series.marker.dataLabel.connectorLine.stroke:color;
            var line = { width: series.marker.dataLabel.connectorLine.width, stroke: stroke, child: this.chartObj.chart3D, tag: 'polyline', id:this.chartObj.svgObject.id + "_" + seriesIndex + "_" + pointIndex };
          
            this.chartObj.graphics.addVisual(this.chartObj.polygon.createPolyline(vectorPoints, line));
        }

    },

     ej.ej3DDoughnutSeries = {

         draw: function (series, sender) {
             var MARGINS_RATIO = 0.03;
             sender._pieVisiblePoints(series);
              sender.calculateSize(sender, series);
              series.radius = (((1 - MARGINS_RATIO) * Math.min(series.actualWidth / 2, series.actualHeight / 2))  * 0.8);
              series.InsideRadius = (series.radius * series.doughnutCoefficient);
              sender.createPolygons(series, sender);
         },
         doAnimation: function (series, sender) {
             var seriesRendering = this;
             series.count = 0;
             $.each(series.segments, function (pointIndex, point) {

                 var radius = point.radius;
                 var insideRadius = point.inSideRadius;

                 series.animate = true;
                 $(sender.chartObj.element).each(function () { point.radius = 0, point.inSideRadius=0 }).animate(
                     { Radius: radius, InsideRadius: insideRadius },

                     {
                         duration: 1000, queue: false, step: function (now, fx) {


                             if (fx.prop.toString() === "Radius") {
                                 point.radius = now;
                             }
                             else {
                                 point.inSideRadius = now;
                             }


                             var poligons = sender.createPolygons(series, sender, true);

                             for (var ai = 0; ai < poligons.length; ai++) {
                                 for (var k = 0; k < poligons[ai].length; k++) {                                     
                                     sender.chartObj.polygon.update(poligons[ai][k].VectorPoints, poligons[ai][k], sender)
                                 }
                             }
                            
                         },
                         complete: function () {
                             sender.chartObj.model.AnimationComplete = true;
                             var commonEventArgs = $.extend({}, ej.EjSvgRender.commonChartEventArgs);
                             commonEventArgs.data = { series: series };
                             sender.chartObj._trigger("animationComplete", commonEventArgs);

                         }
                     });
             });
         }
     },

      ej.series3DTypes.doughnut = ej.ej3DDoughnutSeries;

    ej.ej3DPieSeries = {

        draw: function (series, sender) {
            sender._pieVisiblePoints(series);
            series.InsideRadius = null;
            series.radius = null;
            sender.createPolygons(series, sender);
        },
        doAnimation: function (series, sender) {
            var seriesRendering = this;
            series.count = 0;           
            $.each(series.segments, function (pointIndex, point) {
               
                var radius = point.radius;
              
                series.animate = true;
                $(sender.chartObj.element).each(function () { point.radius = 0  }).animate(
                    {  Radius: radius },

                    {
                        duration: 800, queue: false, step: function (now, fx) {

                            
                            if (fx.prop.toString() === "Radius") {
                                point.radius = now;
                            }
                           
                                                    
                            var poligons = sender.createPolygons(series, sender, true);

                            for (var ai = 0; ai < poligons.length; ai++) {
                                for (var k = 0; k < poligons[ai].length; k++) {
                                    //sender.chartObj.graphics.addVisual(poligons[ai][k]);
                                    sender.chartObj.polygon.update(poligons[ai][k].VectorPoints, poligons[ai][k], sender)
                                }
                            }
                            //sender.update()
                            //  var size = { Width: $(sender.chartObj.svgObject).width(), Height: $(sender.chartObj.svgObject).height() };
                            //  sender.chartObj.graphics.prepareView(sender.chartObj.model.perspectiveAngle, sender.chartObj.model.depth, sender.chartObj.model.rotation, sender.chartObj.model.tilt, size);
                            //  sender.chartObj.graphics.view(sender.chartObj.svgObject, sender.chartObj);
                           

                        },
                        complete: function () {                         
                         sender.chartObj.model.AnimationComplete = true;
                         var commonEventArgs = $.extend({}, ej.EjSvgRender.commonChartEventArgs);
                         commonEventArgs.data = { series: series };
                         sender.chartObj._trigger("animationComplete", commonEventArgs);
                  
                      }
                    });
            });
        }

    }

    ej.series3DTypes.pie = ej.ej3DPieSeries;

    ej.ej3DStackingColumn = {

        draw: function (series, sender) {
            var visiblePoints = sender._isVisiblePoints(series);
            this.createSegments(series, sender);
            for (var i = 0; i < visiblePoints.length; i++) {
                visiblePoints[i].plans = null;
                ej.ej3DColumnSeries.update(series, visiblePoints[i], i, sender)
            }
        },

        createSegments: function (series, sender) {
            var xValues = sender.chartObj._getXValues(series);
          
            var median;
            if (xValues == null) return;
            
            var sbsInfo = sender.getSideBySideInfo(series);
            var depthInfo = sender._getSegmentDepth(series);           
            median = sbsInfo.Delta / 2;
            var visiblePoints = sender._isVisiblePoints(series);
            for (var i = 0; i < visiblePoints.length; i++) {

                var x1 = xValues[i] + sbsInfo.Start;
                var x2 = xValues[i] + sbsInfo.End;
                var y2 = series.stackedValue.StartValues[i];
                var y1 = series.stackedValue.EndValues[i];

                ej.ej3DColumnSeries._setData(x1, y1, x2, y2, depthInfo.Start, depthInfo.End, sender, visiblePoints[i]);

                if (!series.marker.dataLabel.visible) continue;
                visiblePoints[i].symbolLocation = { x: 0, y: 0, z: 0 };
                switch (series.marker.dataLabel.textPosition) {
                    case "top":
                        visiblePoints[i].symbolLocation.x = x1 + median;
                        visiblePoints[i].symbolLocation.y = y1;
                        visiblePoints[i].symbolLocation.z = depthInfo.Start + (depthInfo.Delta / 2);
                        break;
                    case "bottom":
                        visiblePoints[i].symbolLocation.x = x1 + median;
                        visiblePoints[i].symbolLocation.y = y2;
                        visiblePoints[i].symbolLocation.z = depthInfo.Start + (depthInfo.End - depthInfo.Start) / 2;
                        break;
                    default:
                        visiblePoints[i].symbolLocation.x = x1 + median;
                        visiblePoints[i].symbolLocation.y = y1 + (y2 - y1) / 2;
                        visiblePoints[i].symbolLocation.z = depthInfo.Start;
                        break;
                }

            }

        },

        doAnimation: function (series, sender) {            

                var seriesRender = this;

                $.each(series.points, function (pointIndex, point) {

                    var topValue = point.Top;
                    var bottomValue = point.Bottom;

                    $(point).each(function () { point.Top = 0, point.Bottom = 0}).animate(
                    { Top: topValue, Bottom: bottomValue },

                    {
                        duration: 1200, queue: false, step: function (now, fx) {

                            if (fx.prop.toString() === "Top") {
                                point.Top = now;
                            }
                            if (fx.prop.toString() === "Bottom") {
                                point.Bottom = now;
                            }

                            ej.ej3DColumnSeries.update(series, point, pointIndex, sender);

                        },
                        
                         complete: function () {                  
                       
                                sender.chartObj.model.AnimationComplete = true;
                                var commonEventArgs = $.extend({}, ej.EjSvgRender.commonChartEventArgs);
                                commonEventArgs.data = { series: series };
                                sender.chartObj._trigger("animationComplete", commonEventArgs);
                  
                            }

                    });

                });

           }

        
        
    },

     ej.series3DTypes.stackingcolumn = ej.ej3DStackingColumn;

     ej.series3DTypes.stackingbar = ej.ej3DStackingColumn;

   
    ej.ej3DColumnSeries = {
        draw: function (series, sender) {
            var visiblePoints = sender._isVisiblePoints(series);
            this.createSegments(series, sender);
            for (var i = 0; i < visiblePoints.length; i++) {
                visiblePoints[i].plans = null;
                this.update(series, visiblePoints[i], i, sender)
            }
        },

        update: function (series, point, pointIndex, sender) {
            var plans;
            var valueType = series.xAxis._valueType.toLowerCase();
            var seriesIndex = $.inArray(series, sender.chartObj.model._visibleSeries);    
            var xBase = (valueType == "logarithmic") ? null : 1;
            var xIsLogarithmic = (valueType == "logarithmic") ? true :false;
            var left = xIsLogarithmic ? Math.log(point.Left, xBase) : point.Left;
            var right = xIsLogarithmic ? Math.log(point.Right, xBase) : point.Right;
            var bottom = series.yAxis.visibleRange.min;
            var top = series.yAxis.visibleRange.max;
            var xStart = series.xAxis.visibleRange.min;
            var xEnd = series.xAxis.visibleRange.max;
            if ((!(left >= xStart) || !(left <= xEnd)) && (!(right >= xStart) || !(right <= xEnd))) return;
           
            var topValue;
            if(point.Top<0)
                topValue = point.Top > bottom ? point.Top : bottom;
            else
                topValue = point.Top < top ? point.Top : top;
            var tlpoint = ej.EjSvgRender.utils.Transform3DToVisible(series, point.Left > xStart ? point.Left : xStart, topValue, sender.chartObj);
            var rbpoint = ej.EjSvgRender.utils.Transform3DToVisible(series, xEnd > point.Right ? point.Right : xEnd, bottom > point.Bottom ? bottom : point.Bottom, sender.chartObj);
            
            var tlfVector = sender.chartObj.vector.vector3D(Math.min(tlpoint.X, rbpoint.X), Math.min(tlpoint.Y, rbpoint.Y), point.StartDepth);
            var brbVector = sender.chartObj.vector.vector3D(Math.max(tlpoint.X, rbpoint.X), Math.max(tlpoint.Y, rbpoint.Y), point.EndDepth);
           
            var styleOptions = sender.chartObj.setStyle(sender, series, seriesIndex, pointIndex);
            if (typeof (sender.chartObj.model.seriesColors[seriesIndex]) == "object")
                styleOptions.interior = sender.chartObj.model.seriesColors[seriesIndex][0].color;
            var name = "Region" + "_Series_" + seriesIndex + "_Point_" + pointIndex

            if (!point.plans)
                point.plans = sender.chartObj.polygon.createBox(tlfVector, brbVector, this, pointIndex, null,
                    styleOptions.borderColor, styleOptions.interior, styleOptions.borderWidth, styleOptions.opacity, sender.chartObj.model.requireInvertedAxes, name, sender.chartObj.chart3D);
            else
                sender.chartObj.polygon.updateBox(point.plans, tlfVector, brbVector, sender);
        },

        _setData:function()
        {  
            var values=arguments;
            point = arguments[arguments.length - 1];
            sender = arguments[arguments.length - 2];
        
            point.Left = values[0];
            point.Bottom = values[3];
            point.Top = values[1];
            point.Right = values[2];
            point.StartDepth = values[4];
            point.EndDepth = values[5];
            point.XRange = sender.getDoubleRange(point.Left, point.Right);
            if (!isNaN(point.Top) && !isNaN(point.Bottom))
                point.YRange = sender.getDoubleRange(point.Top, point.Bottom);
        },

        createSegments:function(series, sender)
        {
            var xValues = sender.chartObj._getXValues(series);
            var YValues = sender.chartObj._getYValues(series);
            var median;
            if (xValues == null) return;
      
            var sbsInfo = sender.getSideBySideInfo(series);
            var depthInfo = sender._getSegmentDepth(series);
         
            median = sbsInfo.Delta / 2;
            var visiblePoints = sender._isVisiblePoints(series);
            for (var i = 0; i < visiblePoints.length; i++)
            {
                 
                var x1 = xValues[i] + sbsInfo.Start;
                var x2 = xValues[i] + sbsInfo.End;
                var y1 = YValues[i];
                var y2 = 0;
        
                this._setData(x1, y1, x2, y2, depthInfo.Start, depthInfo.End, sender, visiblePoints[i]);

                XData = xValues[i];
                YData = YValues[i];
                //Item = ActualData[i];
              
                if (!series.marker.dataLabel.visible) continue;
                visiblePoints[i].symbolLocation = {x:0, y:0, z:0};
                switch (series.marker.dataLabel.textPosition) {
                    case "top":
                        visiblePoints[i].symbolLocation.x = x1 + median;
                        visiblePoints[i].symbolLocation.y = y1;
                        visiblePoints[i].symbolLocation.z = depthInfo.Start + (depthInfo.Delta / 2);
                        break;
                    case "bottom":
                        visiblePoints[i].symbolLocation.x = x1 + median;
                        visiblePoints[i].symbolLocation.y = y2;
                        visiblePoints[i].symbolLocation.z = depthInfo.Start + (depthInfo.End - depthInfo.Start) / 2;
                        break;
                    default:
                        visiblePoints[i].symbolLocation.x = x1 + median;
                        visiblePoints[i].symbolLocation.y = Math.abs(y2 - y1) / 2;
                        visiblePoints[i].symbolLocation.z = depthInfo.Start;
                        break;
                }
        
            }

    
        },

        doAnimation: function (series, sender) {

            var seriesRender = this;

            $.each(series.points, function (pointIndex, point) {

                var topValue = point.Top;
                $(point).each(function () { point.Top = 0 }).animate(
                { Top: topValue },

                {
                    duration: 1200, queue: false, step: function (now, fx) {

                        if (fx.prop.toString() === "Top") {
                            point.Top = now;
                        }                      

                        seriesRender.update(series, point, pointIndex, sender);

                    },
                    
                     complete: function () {                        
                            sender.chartObj.model.AnimationComplete = true;
                            var commonEventArgs = $.extend({}, ej.EjSvgRender.commonChartEventArgs);
                            commonEventArgs.data = { series: series };
                            sender.chartObj._trigger("animationComplete", commonEventArgs);
                  
                        }

                });

            })

        }
     
    }

    ej.series3DTypes.column = ej.ej3DColumnSeries;

    //bar series
    ej.series3DTypes.bar = ej.ej3DColumnSeries;

    ej.Ej3DChart.prototype = {
        renderSeries: function (sender, series) {            

            var series;
            ej.Ej3DSeriesRender.prototype.chartObj = sender;
         
            $.extend(ej.Ej3DSeriesRender.prototype, ej.EjSeriesRender.prototype);
            
            var seriesRendering = new ej.Ej3DSeriesRender();
            var commonEventArgs = $.extend({}, ej.EjSvgRender.commonChartEventArgs);
            commonEventArgs.data = {"series":series};
            seriesRendering.chartObj._trigger("seriesRendering", commonEventArgs);
                var type = series.type.toLowerCase();
                var options = ej.series3DTypes[type];
                 options.draw(series, seriesRendering);
                if (series.marker.dataLabel.visible) {
                    $.each(series.points, function (pointIndex, point) {
                        if (ej.util.isNullOrUndefined(point.marker) || (point.marker.dataLabel && point.marker.dataLabel.visible))
                            seriesRendering.draw3DDataLabel(series, pointIndex, point);
                    });
                }


               
 
        },
        update3DWall: function (sender) {
            if (sender.model.AreaType == "cartesianaxes") {
                $.extend(ej.Ej3DRender.prototype, this);
                this.updateBackWall(sender);
                var axis;
                for (var i = 0; i < sender.model.axes.length; i++) {
                    axis = sender.model.axes[i];
                    if (axis.orientation.toLowerCase() == "vertical") {
                        if (!axis.opposedPosition)
                            this.updateLeftWall(sender);
                        else
                            this.updateRightWall(sender);
                    }
                    else {
                        if (!axis.opposedPosition)
                            this.updateBottomWall(sender);
                        else
                            this.updateTopWall(sender);
                    }
                }
            }                  
        },

      updateTopWall:function(sender)
        {
        
            var offset = 0;
            var  areaBounds= sender.model.m_AreaBounds;
            if (sender.model.wallSize < areaBounds.Y)
                offset = areaBounds.Y - sender.model.wallSize;
           else
                offset = -(sender.model.wallSize - areaBounds.Y);
            var tlfVector = sender.vector.vector3D(areaBounds.X + areaBounds.Width, -sender.model.depth, areaBounds.Y - 0.1);
            var brbVector = sender.vector.vector3D(areaBounds.X, - 0.1, offset);
            topSideWallPlans = sender.polygon.createBox(tlfVector, brbVector, this, 0, "Graphics3D", "#e2e1e1", "#e2e1e1", 0, 0.15, false, "TopWallBrush", sender.chart3D);
  
          for (var i = 0; i < topSideWallPlans.length; i++)
              sender.polygon.transform(sender.matrixobj.tilt((parseFloat)(Math.PI / 2)), topSideWallPlans[i]);
    
                  
    },

       updateRightWall:function(sender)
        {
           var rightRect = { left: -(sender.model.depth), top: sender.model.m_AreaBounds.Y, bottom: sender.model.m_AreaBounds.Height+sender.model.m_AreaBounds.Y, right: 0 };
           var tlfVector = sender.vector.vector3D(rightRect.left, rightRect.top, sender.model.m_AreaBounds.X+sender.model.m_AreaBounds.Width + 1.5);
           var brbVector = sender.vector.vector3D(rightRect.right, rightRect.bottom, sender.model.m_AreaBounds.X + sender.model.m_AreaBounds.Width + sender.model.wallSize);
           rightSideWallPlans = sender.polygon.createBox(tlfVector, brbVector, sender, 0, "Graphics3D", "#e2e1e1", "#e2e1e1", 0, 0.15, false, "RightWallBrush", sender.chart3D);
           for (var i = 0; i < rightSideWallPlans.length; i++)
             sender.polygon.transform(sender.matrixobj.turn((parseFloat)(-Math.PI / 2)), rightSideWallPlans[i]);           
        },
        updateBackWall: function (sender) {
            var areaBounds = sender.model.m_AreaBounds;           
            var tlfVector = sender.vector.vector3D(areaBounds.X, areaBounds.Y, sender.model.depth == 0 ? 1.5 : sender.model.depth + sender.model.wallSize);
            var brbVector = sender.vector.vector3D((areaBounds.X + areaBounds.Width), areaBounds.Y + areaBounds.Height, sender.model.depth == 0 ? 1.5 : sender.model.depth);

            sender.polygon.createBox(tlfVector, brbVector, sender, 0, "Graphics3D", "#e2e1e1", "#e2e1e1", 0, 0.15, false, "BackWallBrush", sender.chart3D);
        },
        updateLeftWall: function (sender) {
            leftRect = { left: -(sender.model.depth), top: sender.model.m_AreaBounds.Y, bottom: sender.model.m_AreaBounds.Height+sender.model.m_AreaBounds.Y, right: 0 };
            offset = sender.model.m_AreaBounds.X;
            var tlfVector = sender.vector.vector3D(leftRect.left, leftRect.top, offset - 0.1);
            var brbVector = sender.vector.vector3D(leftRect.right, leftRect.bottom, offset - sender.model.wallSize);

            leftSideWallPlans = sender.polygon.createBox(tlfVector, brbVector, this, 0, "Graphics3D", "#e2e1e1", "#e2e1e1", 0, 0.15, false, "LeftWallBrush", sender.chart3D);
            for (var i = 0; i < leftSideWallPlans.length; i++)
                sender.polygon.transform(sender.matrixobj.turn((parseFloat)(-Math.PI / 2)), leftSideWallPlans[i]);
        },
        updateBottomWall: function (sender) {
            var areaBounds = sender.model.m_AreaBounds;
            var tlfVector = sender.vector.vector3D((areaBounds.X + areaBounds.Width), -(sender.model.depth), sender.model.wallSize + areaBounds.Y + areaBounds.Height);
            var brbVector = sender.vector.vector3D(areaBounds.X, -0.1, areaBounds.Y+areaBounds.Height + 1);

            bottomSideWallPlans = sender.polygon.createBox(brbVector, tlfVector, this, 0, "Graphics3D", "#D3D3D3", "#D3D3D3", 0, 0.15, false, "BottomWallBrush", sender.chart3D);
            for (var i = 0; i < bottomSideWallPlans.length; i++)
                sender.polygon.transform(sender.matrixobj.tilt((parseFloat)(Math.PI / 2)), bottomSideWallPlans[i]);
        }
    }

})(jQuery);
ej.EjVmlRender = function (element) {
   var doc = document;
    this.vmlNamespace = "urn:schemas-microsoft-com:vml";
    this.vmStyle = "#default#VML";
    doc.namespaces.add('v', 'urn:schemas-microsoft-com:vml');
    doc.createStyleSheet().cssText =
  'v\\:fill, v\\:path, v\\:polyline, v\\:line, v\\:rect,v\\:shape,v\\:oval, v\\:stroke' +
  '{ behavior:url(#default#VML); display: inline-block; } ';
    this._rootId = jQuery(element).attr("id");
    this.svgObj = doc.createElement("div");
    this.svgObj.style.position = 'relative';
    this.svgObj.setAttribute('id', this._rootId + '_vml');
    this.isdocmode8 = doc.documentMode === 8;
    this.changeOptions = {'id':'id',
        'fill': 'fillcolor', 'stroke': 'strokecolor', 'stroke-width': 'strokeweight', 'd': 'path', 'font-size': 'font-size', 'font-family': 'font-family', 'font-style': 'font-style',
        'font-weight': 'font-weight', 'points':'points'

        };
};
(function($) {
ej.EjVmlRender.prototype = {
  
    drawPath: function (options, element) {
        var darray= options.d.split(" ");
        if($.inArray('A', darray) != -1)
        {
            var w = parseFloat(darray[4]);
            var h = parseFloat(darray[5]);
            options.d=this.drawArc(w, h, options);
        }
        if ($("#" + options.id).length > 0) {
            options.d = this.changePathValue(options.d);
            this.applyVMLStyle($("#" + options.id), options);
        }
        else {
            options.d = this.changePathValue(options.d);
            var shape = document.createElement("v:shape");
            var $shapeEle = $(shape);
            this.applyVMLStyle($shapeEle, options);
            $shapeEle.appendTo(element);
        }
    },
    createLegendSvg:function(element)
    {
        var doc = document;
        this.vmlNamespace = "urn:schemas-microsoft-com:vml";
        this.vmStyle = "#default#VML";
        doc.namespaces.add('v', 'urn:schemas-microsoft-com:vml');
        doc.createStyleSheet().cssText =
      'v\\:fill, v\\:path, v\\:polyline, v\\:line, v\\:rect,v\\:shape,v\\:oval, v\\:stroke' +
      '{ behavior:url(#default#VML); display: inline-block; } ';
        this._rootId = jQuery(element).attr("id");
        this.legendsvgObj = doc.createElement("div");
        this.legendsvgObj.style.position = 'relative';
        this.legendsvgObj.setAttribute('id',"legend_"+ this._rootId + '_vml');
        this.isdocmode8 = doc.documentMode === 8;
        this.changeOptions = {
            'id': 'id',
            'fill': 'fillcolor', 'stroke': 'strokecolor', 'stroke-width': 'strokeweight', 'd': 'path', 'font-size': 'font-size', 'font-family': 'font-family', 'font-style': 'font-style',
            'font-weight': 'font-weight', 'points': 'points'

        };
        return this.legendsvgObj;
    },

    drawPolyline: function (options, element) {
        var darray = options.points.split(" ");
        
        if ($("#" + options.id).length > 0) {
            options.points = this.changePathValue(options.points);
            this.applyVMLStyle($("#" + options.id), options);
        }
        else {
            options.points = this.changePathValue(options.points);
            var shape = document.createElement("v:polyline");
            var $shapeEle = $(shape);
            this.applyVMLStyle($shapeEle, options);
            $shapeEle.appendTo(element);
        }
    },
    setFillAttribute: function (element, options) {
        var fill = document.createElement("v:fill");
        var opacity = options["fill-opacity"] ? options["fill-opacity"] : options["opacity"];
        opacity = (opacity) ?opacity: (options["opacity"]) ?  options["opacity"]:1;
        var fillcolor = options["fill"];
        if (fillcolor != "none" && !ej.util.isNullOrUndefined(fillcolor)) {
          
            if (!(fillcolor).match('gradient'))
                $(element).attr("fillcolor", fillcolor);
            else {
                var colorName = ((fillcolor).replace('gradient', ""));
                fill.setAttribute('type', "gradient");
                fill.setAttribute("colors", colorName);
            }
        }
        else 
            opacity = 0.1;
        
        
        fill.setAttribute('opacity', opacity);
        $(fill).appendTo(element);
    },
    setStrokeAttribute: function (element, options) {
        var stroke = options['stroke'];
        if (stroke == "transparent") 
            options['opacity'] = 0.1;
        
         
        var opacity = options["opacity"];
        if (opacity) {
            var fill = document.createElement("v:stroke");

           

            fill.setAttribute('opacity', opacity);
            $(fill).appendTo(element);
        }
    },
    changePathValue: function (options) {
        if (!options["d"]) {
            var pathArray = options.split(" ");
            for (var i = 0; i < pathArray.length; i++) {
                var matches = pathArray[i].match(/\d+/g);
                if (matches != null) {
                    pathArray[i] = Math.round(parseFloat(pathArray[i]));
                }
            }
            options = pathArray.join(' ');
            return options;
        }
    },


    drawArc: function ( w, h, options) {
    var x = 0, y = 0;
    var startAngle = options.start-1.571,
    endAngle =options.end-1.571,
    endAngle = (Number(Math.PI.toFixed(2)) == Number((endAngle + startAngle).toFixed(2))) ? endAngle - 0.01 : endAngle,
    radius = options.r || w || h,
    cosStart = Math.cos(startAngle),
    sinStart = Math.sin(startAngle),
    cosEnd = Math.cos(endAngle),
    sinEnd = Math.sin(endAngle),
    innerRadius = options.innerR,
    circleCorrection = 0.08 / radius, 
    innerCorrection = (innerRadius && 0.1 / innerRadius) || 0,
    path;

    if (endAngle - startAngle === 0) {
            return '';

    } else if (2 * Math.PI - endAngle + startAngle < circleCorrection) {
            cosEnd = -circleCorrection;
    } else if (endAngle - startAngle < innerCorrection) {
            cosEnd = Math.cos(startAngle + innerCorrection);
        }

        path = [
    'wa', // To draw clockwise arc
    x - radius, 
    y - radius, 
    x + radius, 
    y + radius, 
    x + radius * cosStart, 
    y + radius * sinStart, 
    x + radius * cosEnd, 
    y + radius * sinEnd  
        ];

        if (options.open && !innerRadius) {
            path.push(
        'e',
        M,
        x, 
        y
    );
        }

        path.push(
            // To draw anti clockwise arc
    'at', 
    x - innerRadius, 
    y - innerRadius, 
    x + innerRadius, 
    y + innerRadius, 
    x + innerRadius * cosEnd, 
    y + innerRadius * sinEnd, 
    x + innerRadius * cosStart, 
    y + innerRadius * sinStart, 
    'x', 
    'e' 
);
        path=path.join(" ");
        return path;

  },
    changeVMLStyle: function ($element, options) {
        var chart = this;
		var strokeColor = options.stroke;
        $element.css("width", options.width).css("visibility", options.visibility).css("height", options.height).css("position", "absolute").css('left', options.x).css('top', options.y);
        $.each(options, function (index, val) {
            if (chart.changeOptions[index] != null || chart.changeOptions[index] != undefined) {
                if (index == 'd' || index=='points') {
                    val = chart.changePathValue(options[index]).toLowerCase();
                }
                if (val == 'transparent') {
                    options[index] = val = 'white';
                     if(index=="fill" && (!options.hasOwnProperty('opacity') ||  !options['opacity'])) 
                       options['opacity'] = 0.1;                  
                }
                if ($element[0].tagName === "SPAN") {
                    if (index == "fill") $element.css('color', val);
                    else {
                        if (index == "id") $element.attr(chart.changeOptions[index], val);
                        else
                            $element.css(chart.changeOptions[index], val);
                    }
                } else {
                   
                    if (chart.isdocmode8) { // IE8 setAttribute bug
                        $element.get(0)[chart.changeOptions[index]] = val;
                    } else {
                        $element.get(0).setAttribute(chart.changeOptions[index], val);
                    }      
                }
            }
        });
        if (options.hasOwnProperty('fill-opacity') || options.hasOwnProperty('opacity') || options.hasOwnProperty('fill'))
            if (!($element[0].tagName === "SPAN")) {
                chart.setFillAttribute($element, options);
               if (options.hasOwnProperty('stroke')) {
                    options.stroke = strokeColor;
                    chart.setStrokeAttribute($element, options);
                }
            }
    },
    applyVMLStyle: function ($element, options) {
        
        if ($element[0].tagName !== "SPAN") {
            if (ej.util.isNullOrUndefined(options.width)) options.width = "1000px";
            if (ej.util.isNullOrUndefined(options.height)) options.height = "1000px";
        }
        if (ej.util.isNullOrUndefined(options.x)) options.x = "0px";
        if (ej.util.isNullOrUndefined(options.y)) options.y = "0px";

        this.changeVMLStyle($element, options);
    },
    drawLine: function (options, element) {
        if ($("#" + options.id).length > 0) {
            var from = (options.x1) + ',' + (options.y1);
            var to = (options.x2) + ',' + (options.y2);
            $(options).attr("from", from).attr("to", to);
            this.applyVMLStyle($("#" + options.id), options);
        }
        else {
            var line = document.createElement("v:line");
            var $lineEle = $(line);
            var from = (options.x1) + ',' + (options.y1);
            var to = (options.x2) + ',' + (options.y2);
            $lineEle.attr("from", from).attr("to", to);
            this.applyVMLStyle($lineEle, options);
            $lineEle.appendTo(element);
        }
     },
    //drawPolygon: function (options, element) {
     
    //},
     drawCircle: function (options, element) {
         if ($("#" + options.id).length > 0) {
             options.x = options.cx - options.r;
             options.y = options.cy - options.r;
             options.width = options.height = (2 * options.r);
             this.applyVMLStyle($("#" + options.id), options);
         }
         else {
             var circle = document.createElement("v:oval");
             var $circleEle = $(circle);
             options.x = options.cx - options.r;
             options.y = options.cy - options.r;
             options.width = options.height = (2 * options.r);
             this.applyVMLStyle($circleEle, options);
             if ($("#" + options.id).length == 0)
             $circleEle.appendTo(element);
         }
     },
    
   drawEllipse: function (options, element) {
       if ($("#" + options.id).length > 0) {
           options.x = options.cx - options.rx;
           options.y = options.cy - options.ry;
           options.width = (2 * options.rx);
           options.height = (2 * options.ry);
           this.applyVMLStyle($("#" + options.id), options);
       }
       else {
           var ellipse = document.createElement("v:oval");
           var $ellipseEle = $(ellipse);
           options.x = options.cx - options.rx;
           options.y = options.cy - options.ry;
           options.width = (2 * options.rx);
           options.height = (2 * options.ry);
           this.applyVMLStyle($ellipseEle, options);
           $ellipseEle.appendTo(element);
       }
    },
    drawRect: function (options, element) {
        if ($("#" + options.id).length > 0) {
            this.applyVMLStyle($("#" + options.id), options);
        } else {
            var rect = document.createElement("v:rect");
            var $rectEle = $(rect);
            this.applyVMLStyle($rectEle, options);
            $rectEle.appendTo(element);
        }
    },
    _getAttrVal: function (ele, val, option) {
          val = (val == "x") ? "left" : val;
        val = (val == "y") ? "top" : val;
        var value1 = $(ele).attr(val);
        var value = value1 ? value1 : ($(ele).css(val));
        
        if (value != null)
            return value;
        else
            return option;
    },
    _setAttr: function (element, attribute) { 
        if (attribute.transform) {
		  var attrTrans=attribute.transform;
            var trans = attrTrans.slice(attrTrans.indexOf('(') + 1, attrTrans.indexOf(')')).split(',');
            attribute.x = trans[0];
            attribute.y = trans[1];
        }
        var $ele = $(element);
        if($ele.length>0)
            this.changeVMLStyle($ele, attribute);
    },
    createGradientElement: function (name, colors) {
        var colorName;
       
        if (Object.prototype.toString.call(colors) == '[object Array]') {
            colorName = "gradient" + " ";
            var appendString = "";
            for (var j = 0, i=colors.length-1 ; i>=0, j < colors.length ; j++,i--){
                colorName += appendString + colors[j]["colorStop"] + " " + colors[i]["color"];
                appendString = ",";
            }
        }
        else {
            colorName = colors;
        }
        return colorName; 
    },

    //drawGradient: function (options, gradientEle, element) {

      
    //},
    setSpanAttr: function (options, label, element) {
	    var fontSize = (options["font-size"] > 0) ? (options["font-size"] + "px") : options["font-size"];
        var font = { size: fontSize, fontStyle: options["font-style"], fontFamily: options["font-weight"] };
        var bounds = ej.EjSvgRender.utils._measureText(label, null, font);
        
            $(element).css("white-space", "nowrap");
        
        if (options["text-anchor"] === "middle")
            options.x = Math.round(options.x - bounds.width / 2);
        else if (options["text-anchor"] === "end")
            options.x = Math.round(options.x - bounds.width);
        options.y = Math.round(options.y -( font.size.replace("px","")));
        this.applyVMLStyle(element, options);
        
       
    },
    drawText: function (options, label, groupEle,font) {
        var textDoc = document;
        var $Ele = $("#" + options.id);
        if ($Ele.length > 0) {
            this._textAttrReplace(options, label, font, $Ele);
        } else {
            var span = textDoc.createElement("span");
            var $spanEle = $(span);
            if (jQuery.type(label) == "array") {
                this.setSpanAttr(options, label, $spanEle);
                for (var i = 0; i < label.length; i++) {
                    var textspan = textDoc.createElement("span");
                    $(textspan).html(label[i]);                
                    $(textspan).appendTo(span);
                    var brtag = textDoc.createElement("br");
                    $(brtag).appendTo(span);
                    
                }
            } else {
                this.setSpanAttr(options, label, $spanEle);
                $spanEle.html(label);
            }         
            $spanEle.appendTo(groupEle);          
        }
    },
    //text element attributes values are replaced 
    _textAttrReplace: function (options, label, font, $Ele) {
        if (jQuery.type(label) == "array") {
            var elements = $("#" + options.id).children("span");
            this.setSpanAttr(options, label, $Ele);
            if (elements.length > 0 && elements.length == label.length) {
                for (var i = 0; i < elements.length; i++) {
                    var element = elements[i];
                    $(element).text(label[i]);
                }
            }
            else {
                $("#" + options.id).remove();
                this.drawText(options, label, this.gTransToolEle, font);

            }
        } else {
            this.setSpanAttr(options, label, $Ele);
            $Ele.text(label);
        }

    },

   drawImage: function (options, element) {
        var imgElement = document.createElement("img");
        var $imgElement = $(imgElement);
          $imgElement.css({ 'position': 'absolute' });
          $.each(options, function (index, val) {
              if (index == "href") index = "src";
                $imgElement.attr(index, val);
        });
        $imgElement.appendTo(element);
       
    },

    createDefs: function () {
        
    },

    //createClipPath: function (options) {
        
    //},
    //createForeignObject: function (options) {
      
    //},
    createGroup: function (options) {
        var group = document.createElement("div");
        if (!ej.util.isNullOrUndefined(options.transform)) {
            var trans = options.transform.slice(options.transform.indexOf('(') + 1, options.transform.indexOf(')')).split(',');
            $(group).css({ 'position': 'absolute', 'left': trans[0] + 'px', 'top': trans[1] + 'px' });
        }
		 if (options.cursor) 
            $(group).css({ 'cursor': options.cursor });
        
        $.each(options, function(index, val) {
            if (index !== 'transform' && index !="clip-path")
                $(group).attr(index, val);
        });
      
        return group;
    },

    createText: function (options, label) {
        var text = document.createElement("span");
        $(text).attr(options);
        if (!ej.util.isNullOrUndefined(label))
            text.textContent = label;
        return text;
        
        
    },

    //createPath: function (options) {
       
    //},

    //createCircle: function (options) {
       
    //},

    //createLine: function (options) {
       
    //},
   
   
    //Draw clip path for each series to avoid series overlap in multiple axes zooming
    drawAxesBoundsClipPath: function (gSeriesGroupEle, options, invertedAxis) {
        var clipOptions;
        var element = $(gSeriesGroupEle);
        element.removeAttr("clip-path");
        var trans = this._getTransform(options.xAxis, options.yAxis, invertedAxis);
        clipOptions = {
            'id': gSeriesGroupEle.id + '_ClipRect',
            'x': 0,
            'y': 0,
            'width': (trans.width),
            'height': (trans.height),
            'fill': 'white',
            'stroke-width': 1,
            'stroke': 'transparent'
        };
        this.drawClipPath(clipOptions, gSeriesGroupEle);
        element.attr('clip-path', 'url(#' + clipOptions.id + ')');

    },
   
    getAttrVal: function (ele, val, option) {
        var value = $(ele).attr(val);
        if (value != null)
            return value;
        else
            return option;
    },
   
    round: function (value, div, up) {

        return div * (up ? Math.ceil(value / div) : Math.floor(value / div));
    },
    
   
   
    
    hexFromRGB: function (color) {
        var r = color.R;
        var g = color.G;
        var b = color.B;
        var hex = [r.toString(16), g.toString(16), b.toString(16)];
        $.each(hex, function (nr, val) { if (val.length === 1) { hex[nr] = "0" + val; } });
        return "#" + hex.join("").toUpperCase();
    },
    hexToRGB: function (hex) {
        var result = /^#?([a-f\d]{2})([a-f\d]{2})([a-f\d]{2})$/i.exec(hex);
        return result ? {
            R: parseInt(result[1], 16),
            G: parseInt(result[2], 16),
            B: parseInt(result[3], 16)
        } : null;
    },
    createDelegate: function (context, handler) {
        return function (e) {
            handler.apply(context, [e, this]);
        };
    },

    drawClipPath: function (options, element) {
        if (options.id.indexOf("ChartAreaClipRect") == -1) {
            var left = Math.round(options.x)+'px';
            var top = Math.round(options.y)+'px';
            var right = Math.round(options.width)+'px';
            var bottom = Math.round(options.height) + 'px';
            var cssClip = 'rect(' + top + ' ' + right + ' ' + bottom + ' ' +left + ')';
            $(element).css("position", "absolute").css('clip',cssClip);
        }
    },
   
    append: function (childEle, parentEle) {
        $(childEle).appendTo(parentEle);
    }
};
})(jQuery);;
ej.EjCanvasRender = function (element) {
    this.svgObj = document.createElement('canvas');
    this._rootId = jQuery(element).attr("id");
    this.svgObj.setAttribute('id', this._rootId + '_canvas');
    this.ctx = this.svgObj.getContext("2d");
    
};
(function ($) {
    ej.EjCanvasRender.prototype = {

        drawPath: function (options, element, canvasTranslate) {

            var path = options.d;
            var dataSplit = path.split(" ");
            if (!options.lgndCtx) {
                this.ctx.save();
                this.ctx.beginPath();
                if (canvasTranslate) this.ctx.translate(canvasTranslate[0], canvasTranslate[1]);
                this.ctx.globalAlpha = options.opacity ? options.opacity : options["fill-opacity"];
                var flag = true;
                this.ctx.lineWidth = options["stroke-width"];
				var dashArray = options["stroke-dasharray"] ? options["stroke-dasharray"].split(",") : false;
                if (dashArray) this.ctx.setLineDash(dashArray);
                this.ctx.strokeStyle = options.stroke;
                for (var i = 0; i < dataSplit.length - 1; i = i + 3) {
                    var x1 = parseFloat(dataSplit[i + 1]);
                    var y1 = parseFloat(dataSplit[i + 2]);
                    switch (dataSplit[i]) {
                        case "M":
                            if (!options.canvasInnerR)
                                this.ctx.moveTo(x1, y1);
                            break;
                        case "L":
                            if (!options.canvasInnerR)
                                this.ctx.lineTo(x1, y1);
                            break;
                        case "C":
                            this.ctx.bezierCurveTo(x1, y1, parseFloat(dataSplit[i + 3]), parseFloat(dataSplit[i + 4]), parseFloat(dataSplit[i + 5]), parseFloat(dataSplit[i + 6]));
                            i = i + 4;
                            break;
                        case "A":
                            if (!options.canvasInnerR) {
                                this.ctx.moveTo(options.x, options.y);
                                this.ctx.arc(options.x, options.y, options.radius, options.start, options.end, options.counterClockWise);
                                this.ctx.lineTo(options.x, options.y);
                            }
                            else if (flag) {
                                this.ctx.arc(options.x, options.y, options.radius - (options.canvasInnerR - options.canvasInnerR / 4), options.start, options.end, options.counterClockWise);
                                this.ctx.lineWidth = (options.radius - options.canvasInnerR);
                                this.ctx.strokeStyle = options.fill;
                                flag = false; 
                            }
                            i = i + 5;
                            break;
                    }
                }
                if (options.fill != "none" && options.fill != undefined && !options.canvasInnerR) {
                    this.ctx.fillStyle = options.fill;
                    this.ctx.fill();
                }
                this.ctx.stroke();
                this.ctx.restore();
            } else {
                this.lgndCtx.save();
                this.lgndCtx.beginPath();
                if (canvasTranslate) this.lgndCtx.translate(canvasTranslate[0], canvasTranslate[1]);
                this.lgndCtx.globalAlpha = options.opacity ? options.opacity : options["fill-opacity"];

                this.lgndCtx.lineWidth = options["stroke-width"];
                this.lgndCtx.strokeStyle = options.stroke;
                for (var i = 0; i < dataSplit.length - 1; i = i + 3) {
                    var x1 = parseFloat(dataSplit[i + 1]);
                    var y1 = parseFloat(dataSplit[i + 2]);
                    switch (dataSplit[i]) {
                        case "M":
                            this.lgndCtx.moveTo(x1, y1);
                            break;
                        case "L":
                            this.lgndCtx.lineTo(x1, y1);
                            break;
                        case "C":
                            this.lgndCtx.bezierCurveTo(x1, y1, parseFloat(dataSplit[i + 3]), parseFloat(dataSplit[i + 4]), parseFloat(dataSplit[i + 5]), parseFloat(dataSplit[i + 6]));
                            i = i + 4;
                            break;
                        case "A":
                            this.lgndCtx.moveTo(options.x, options.y);
                            this.lgndCtx.arc(options.x, options.y, options.radius, options.start, options.end, false);
                            this.lgndCtx.lineTo(options.x, options.y);
                            i = i + 5;
                            break;
							case "a":
                            this.lgndCtx.beginPath();
                            var centerX = parseFloat(dataSplit[i - 2]) + 5;
                            var centerY = dataSplit[i - 1];
                            this.lgndCtx.arc(centerX, centerY, 5, 0, 2 * Math.PI, false);
                            i = i + 5;
                            break;
                        case "Z":
                            this.lgndCtx.fillStyle = options.fill;
                            this.lgndCtx.fill();
                            i = i - 2;
                            break;
                    }
                }
                if (options.fill != "none" && options.fill != undefined) {
                    this.lgndCtx.fillStyle = options.fill;
                    this.lgndCtx.fill();
                }
                this.lgndCtx.stroke();
                this.lgndCtx.restore();
            }
        },
        
       
        drawRect: function (options, element) {
            this.ctx.save();
            this.ctx.beginPath();
            this.ctx.globalAlpha = options.opacity;
            this.ctx.lineWidth = options["stroke-width"];
            this.ctx.strokeStyle = options.stroke;
            this.ctx.rect(options.x, options.y, options.width, options.height);
            if (options.fill == "none")
                options.fill = "transparent";
            this.ctx.fillStyle = options.fill;
            this.ctx.fillRect(options.x, options.y, options.width, options.height);
            this.ctx.stroke();
            this.ctx.restore();
        },
        createGradientElement: function (name, colors) {
            var colorName;
            var ctx = this.ctx;
            if (Object.prototype.toString.call(colors) == '[object Array]') {
                var my_gradient = ctx.createLinearGradient(0, 0, 0, this.svgObj.height);
             
                for (var i = 0; i <= colors.length - 1 ; i++) {
                    var color = colors[i].color;
                    var colorStop = (colors[i].colorStop).slice(0, -1);
                    colorStop = parseInt(colorStop) / 100;
                    my_gradient.addColorStop(colorStop, color);
                    
                }
                colorName = my_gradient;
            }
            else {
                colorName = colors;
            }
            return colorName;
        },
        drawLine: function (options) {
            this.ctx.save();
            this.ctx.beginPath();
			this.ctx.lineWidth = options["stroke-width"];
            this.ctx.strokeStyle = options.stroke;
            this.ctx.moveTo(options.x1, options.y1);
            this.ctx.lineTo(options.x2, options.y2);
            this.ctx.stroke();
            this.ctx.restore();
        },
        drawText: function (options, label) {
            if (!ej.util.isNullOrUndefined(options["font-weight"]) && options["font-weight"].toLowerCase() == "regular")
                options["font-weight"] = "normal";
            var font = (options["font-style"].toLowerCase() + " " + options["font-weight"] + " " + options["font-size"] + " " + options["font-family"]);
            // text-anchor
            var anchor = options["text-anchor"];
            if (anchor == "middle")
                anchor = "center";
            if (!options.lgndCtx) {
                this.ctx.save();
                this.ctx.fillStyle = options.fill;                
                this.ctx.font = font;                
                this.ctx.textAlign = anchor;
                // dominant-baseline
                //if (options["dominant-baseline"] == "middle")
                    //this.ctx.textBaseline = "center";
                if (!options.labelRotation) {
                    this.ctx.fillText(label, options.x, options.y);
                } else {
                    var txtlngth = 0;
                    if (options.labelRotation == 90 && options.id.indexOf("XLabel") != -1)
                        txtlngth = this.ctx.measureText(label).width;
                    
                    this.ctx.translate(options.x + (txtlngth/2), options.y);
                    this.ctx.rotate(options.labelRotation * Math.PI / 180);
                    this.ctx.fillText(label, 0, 0);
                }
                this.ctx.restore();
            } else {
                this.lgndCtx.save();
                this.lgndCtx.fillStyle = options.fill;
                this.lgndCtx.font = font;
                this.lgndCtx.textAlign = anchor;

                // dominant-baseline
                if (options["dominant-baseline"] == "middle")
                    this.lgndCtx.textBaseline = "center";
                if (!options.labelRotation) {
                    this.lgndCtx.fillText(label, options.x, options.y);
                } else {
                    this.lgndCtx.translate(options.x, options.y);
                    this.lgndCtx.rotate(options.labelRotation * Math.PI / 180);
                    this.lgndCtx.fillText(label, 0, 0);
                }
                this.lgndCtx.restore();
            }
        },
        drawZoomRect: function(options, chartObj) {

            var chartPos = document.getElementById(this.svgObj.id).getClientRects()[0];
            var zoomRect = $("<div id=" + options.id + "></div>");
            var areaBounds = chartObj.model.m_AreaBounds;
			
			// for drawing zoom area within the chartArea
            if (options.x < chartObj.canvasX || options.x + chartPos.left + options.width > chartPos.left + chartPos.width -20) {
                width = this.prevWidth;
                x = this.prevX;
            } else {
                width = this.prevWidth = options.width;
                x = this.prevX = options.x + chartPos.left;
            }
            
            if (options.y < chartObj.canvasY || options.y + chartPos.top + options.height  > chartPos.top + areaBounds.Y + areaBounds.Height) {
                height = this.prevHeight;
                y = this.prevY;
            } else {
                height = this.prevHeight = options.height;
                y = this.prevY = options.y + chartPos.top;
            }
            
            $(zoomRect).css({
                    "width": width,
                    "height": height,
                    "top": y + $(document).scrollTop(),
                    "left": x + $(document).scrollLeft(),
                    "background-color": options.fill,
                    "border-style": "solid",
                    "position": "absolute",
                    "border-color": options.stroke,
                    "border-width": options["stroke-width"]
                });
                document.getElementById(this._rootId).appendChild(zoomRect[0]);
        },
        zoomButton: function (options) {
            var chartPos = document.getElementById(this.svgObj.id).getClientRects()[0];
            if ($("#" + options.zoomId).length <= 0) {
                var zoomButton = $("<div id=" + options.zoomId + "></div>");
                $(zoomButton).css({
                    "width": options.width,
                    "height": options.height,
                    "top": options.top + chartPos.top + $(document).scrollTop(),
                    "left": options.left + chartPos.left - 20 + $(document).scrollLeft(),
                    "background-color": options.fill,
                    "border-style": "solid",
                    "position": "absolute",
                    "border-radius": "0.4em",
                    "border-color": options.fill
                });
                var chartPos = document.getElementById(this._rootId + "_canvas").getClientRects()[0];
                var svgLink = "http://www.w3.org/2000/svg";
                var svgShape = document.createElementNS(svgLink, "svg");
                svgShape.setAttribute('id', options.zoomId);
                svgShape.setAttribute('width', 25);
                svgShape.setAttribute('height', 25);
               
                var attrOptions = {
                    'id': 'zoomPath',
                    'fill': '#FFFFFF',
                    'transform': 'translate(-3,-3)'
                };
                attrOptions.d = "M26.101,22.893l-6.605-6.174c1.414-2.781,0.897-6.267-1.496-8.504c-2.901-2.711-7.448-2.56-10.161,0.341    c-2.712,2.9-2.56,7.45,0.341,10.163c2.426,2.266,6,2.523,8.694,0.853l6.579,6.151L26.101,22.893z M10.186,16.571    c-1.715-1.604-1.805-4.293-0.203-6.009c1.605-1.715,4.295-1.805,6.009-0.201c1.715,1.603,1.805,4.293,0.202,6.007    C14.59,18.084,11.901,18.175,10.186,16.571";

                var path = document.createElementNS(svgLink, "path");
                $(path).attr(attrOptions).appendTo(svgShape);
                zoomButton[0].appendChild(svgShape);
                document.getElementById(this._rootId).appendChild(zoomButton[0]);
            } else {
                var zoomButton = document.getElementById(options.zoomId);
                $(zoomButton).css({
                    "top": options.top + chartPos.top + $(document).scrollTop(),
                    "left": options.left + chartPos.left - 20 + $(document).scrollLeft(),
                });
            }
        },
        panButton: function (options) {
            var chartPos = document.getElementById(this.svgObj.id).getClientRects()[0];
            if ($("#" + options.panId).length <= 0) {
                var panButton = $("<div id=" + options.panId + "></div>");
                $(panButton).css({
                    "width": options.width,
                    "height": options.height,
                    "top": options.top + chartPos.top + $(document).scrollTop(),
                    "left": options.left + chartPos.left - 10 + $(document).scrollLeft(),
                    "background-color": options.fill,
                    "border-style": "solid",
                    "position": "absolute",
                    "border-radius": "0.4em",
                    "border-color": options.fill,
                });
                var chartPos = document.getElementById(this._rootId + "_canvas").getClientRects()[0];
                var svgLink = "http://www.w3.org/2000/svg";
                var svgShape = document.createElementNS(svgLink, "svg");
                svgShape.setAttribute('id', options.panId);
                svgShape.setAttribute('width', 25);
                svgShape.setAttribute('height', 25);

                var attrOptions = {
                    'id': 'panPath',
                    'fill': '#FFFFFF',
                    'transform': 'translate(-3,-3)'
                };
                attrOptions.points = "26.105,16 21.053,12.211 21.053,14.737 17.263,14.737 17.263,10.947 19.834,10.947 16.044,5.895 12.255,10.947 14.737,10.947 14.737,14.737 10.947,14.737 10.947,12.211 5.895,16 10.947,19.789 10.947,17.263 14.737,17.263 14.737,21.053 12.255,21.053 16.044,26.105 19.834,21.053 17.263,21.053 17.263,17.263 21.053,17.263 21.053,19.789";

                var path = document.createElementNS(svgLink, "polygon");
                $(path).attr(attrOptions).appendTo(svgShape);
                panButton[0].appendChild(svgShape);
                document.getElementById(this._rootId).appendChild(panButton[0]);     
            } else {
                var panButton = document.getElementById(options.panId);
                $(panButton).css({
                    "top": options.top + chartPos.top + $(document).scrollTop(),
                    "left": options.left + chartPos.left - 10 + $(document).scrollLeft(),
                });
            }
        },
        resetZoom: function (options) {
            var chartPos = document.getElementById(this.svgObj.id).getClientRects()[0];
            if ($("#" + options.resetZoomId).length <= 0) {
                var resetZoom = $("<div id=" + options.resetZoomId + "></div>");
                $(resetZoom).css({
                    "width": options.width,
                    "height": options.height,
                    "top": options.top + chartPos.top + $(document).scrollTop(),
                    "left": options.left + chartPos.left + $(document).scrollLeft(),
                    "background-color": options.fill,
                    "border-style": "solid",
                    "border-radius": "0.4em",
                    "position": "absolute",
                    "border-color": options.fill,
                });
                var chartPos = document.getElementById(this._rootId + "_canvas").getClientRects()[0];
                var svgLink = "http://www.w3.org/2000/svg";
                var svgShape = document.createElementNS(svgLink, "svg");
                svgShape.setAttribute('id', options.resetZoomId);
                svgShape.setAttribute('width', 25);
                svgShape.setAttribute('height', 25);

                var points1 = {
                    'id': 'p1',
                    'fill': '#FFFFFF',
                    'transform': 'translate(-3,-3)'
                };
                points1.points = "11.895,18.398 8.061,22.23 5.796,19.967 5.796,26.283 12.112,26.283 9.848,24.018 13.682,20.186";

                var path1 = document.createElementNS(svgLink, "polygon");
                $(path1).attr(points1).appendTo(svgShape);

                var points2 = {
                    'id': 'p2',
                    'fill': '#FFFFFF',
                    'transform': 'translate(-3,-3)'
                };
                points2.points = "19.691,6.072 21.955,8.337 18.121,12.172 19.908,13.959 23.742,10.123 26.007,12.389 26.007,6.072";

                var path2 = document.createElementNS(svgLink, "polygon");
                $(path2).attr(points2).appendTo(svgShape);

                var points3 = {
                    'id': 'p3',
                    'fill': '#FFFFFF',
                    'transform': 'translate(-3,-3)'
                };
                points3.points = "11.895,13.958 13.682,12.172 9.848,8.337 12.112,6.072 5.796,6.072 5.796,12.389 8.061,10.123";

                var path3 = document.createElementNS(svgLink, "polygon");
                $(path3).attr(points3).appendTo(svgShape);

                var points4 = {
                    'id': 'p4',
                    'fill': '#FFFFFF',
                    'transform': 'translate(-3,-3)'
                };
                points4.points = "19.908,18.396 18.121,20.186 21.955,24.018 19.691,26.283 26.007,26.283 26.007,19.967 23.741,22.23";

                var path4 = document.createElementNS(svgLink, "polygon");
                $(path4).attr(points4).appendTo(svgShape);

                resetZoom[0].appendChild(svgShape);
                document.getElementById(this._rootId).appendChild(resetZoom[0]);
            } else {
                var resetZoom = document.getElementById(options.resetZoomId);
                $(resetZoom).css({
                    "top": options.top + chartPos.top + $(document).scrollTop(),
                    "left": options.left + chartPos.left + $(document).scrollLeft(),
                });
            }
        },
        drawToolTipRect: function (options, evt) {
            var tooltipdivRect = $("<div id=" + this.svgObj.id + "_TrackToolTip></div>");
            var padding = 5;
            $(tooltipdivRect).css({
                "width": options.width + 2 + padding,
                "height": options.height + padding,
                "top": options.y,
                "left": options.x,
                "background-color": options.fill,
                "border-style": "solid",
                "position": "absolute",
                "border-color": options.stroke,
                "border-width": options["stroke-width"],
                "opacity": options["fill-opacity"]
            });
            document.getElementById(this._rootId).appendChild(tooltipdivRect[0]);
        },
        drawToolTipText: function (options, text) {
            var tooltipText = $("#" + this.svgObj.id + "_TrackToolTip");
            var chartPos = document.getElementById(this._rootId + "_canvas").getClientRects()[0];
            $(tooltipText).css({
                "color": options.fill,
                "font-family": options["font-family"],
                "font-size": options["font-size"],
                "font-weight": options["font-weight"],
                "font-style": options["font-style"],
                "left": options.x + chartPos.left + $(document).scrollLeft(),
                "top": options.y + chartPos.top + $(document).scrollTop() - 10,
                "align-self": "baseline",
                "text-align": "center" 
            });
            $(tooltipText).html(text);
        },
        drawCrosshairLine: function (options, evt) {
            var line = $("<div id=" + options.id + " style='position: absolute; z-index: 13000;'></div>");
            $(line).css({
                "width": options.width,
                "height": options.height,
                "left": options.left + $(document).scrollLeft(),
                "top": options.top + $(document).scrollTop(),
                "border-style": "solid",
                "opacity": 0.5,
                "visibility": "visible",
                "border-color": options.stroke,
                "border-width": options["stroke-width"],
                "z-index": 100
            });
            document.getElementById(this._rootId).appendChild(line[0]);
        },
        drawCrosshairlabel: function (rectOptions, textOptions, text) {
            var chartPos = document.getElementById(this.svgObj.id).getClientRects()[0];
            if ($("#" + rectOptions.id).length > 0) {
                var label = document.getElementById(rectOptions.id);                
                $(label).css({
                    "width": rectOptions.width - 5,
                    "height": rectOptions.height - 5,
                    "left": textOptions.x + chartPos.left + $(document).scrollLeft() - 5,
                    "top": textOptions.y + chartPos.top + $(document).scrollTop() - 15,
                    "visibility": "visible"
                });
                document.getElementById(rectOptions.id).textContent = text;
            } else {
                var label = $("<div id=" + rectOptions.id + " style='position: absolute; z-index: 13000;'></div>");
                $(label).css({
                    "color": textOptions.fill,
                    "font-family": textOptions["font-family"],
                    "font-size": textOptions["font-size"],
                    "font-weight": textOptions["font-weight"],
                    "font-style": textOptions["font-style"],
                    "background-color": rectOptions.fill,
                    "border-style": "solid",
                    "border-color": rectOptions.stroke,
                    "border-width": rectOptions["stroke-width"],
                    "opacity": rectOptions["fill-opacity"],
                    "text-align" : "center"
                });
                
                document.getElementById(this._rootId).appendChild(label[0]);
            }
        },
        drawTrackToolTip: function (rectOptions, textOptions, text) {
            var chartPos = document.getElementById(this.svgObj.id).getClientRects()[0];
            if ($("#" + rectOptions.id).length > 0) {
                var label = document.getElementById(rectOptions.id);               
                $(label).css({
                    "left": textOptions.x + chartPos.left + $(document).scrollLeft() - 5,
                    "top": textOptions.y + chartPos.top + $(document).scrollTop() - 15,
                    "visibility": "visible"
                });
                document.getElementById(rectOptions.id).textContent = text;
            } else {
                var label = $("<div id=" + rectOptions.id + " style='position: absolute; z-index: 13000;'></div>");
                $(label).css({
                    "color": textOptions.fill,
                    "font-family": textOptions["font-family"],
                    "font-size": textOptions["font-size"],
                    "font-weight": textOptions["font-weight"],
                    "font-style": textOptions["font-style"],
                    "width": rectOptions.width,
                    "height": rectOptions.height - 5,
                    "background-color": rectOptions.fill,
                    "border-style": "solid",
                    "border-color": rectOptions.stroke,
                    "border-width": rectOptions["stroke-width"],
                    "opacity": rectOptions["fill-opacity"]
                });
                document.getElementById(this._rootId).appendChild(label[0]);
            }
        },
        trackSymbol: function (options, evt, symbolName, trackSymbol, chartObj) {
            var seriesIndex = options.SeriesIndex;
            var pointIndex = options.PointIndex;
            if (chartObj.model.series[seriesIndex].type.toLowerCase() == "bubble") {
                var point = chartObj.model.series[seriesIndex].points[pointIndex];
                var width = point.radius * 2;
                var height = width;
                options.Style.BorderColor = options.Style.Color;
            }
            
            if (chartObj.model.series[seriesIndex].type.toLowerCase() == "scatter" && ej.util.isNullOrUndefined(trackSymbol)) {
                var circleOptions = {
                    "cx": evt.startX -2,
                    "cy": evt.startY -2,
                    "r": options.ShapeSize.width/2,
                    "fill" : options.Style.Color,
                    "opacity": options.Style.Opacity,
                    "stroke" : options.Style.BorderColor
                };
                this.drawCircle(circleOptions, null);
            }
            
        else {
                if ($("#" + "canvas_trackSymbol").length > 0)
                    var parentDiv = $("#" + "canvas_trackSymbol");
                else
                    parentDiv = $("<div id=" + "canvas_trackSymbol" + "></div>");

                var chartPos = document.getElementById(this._rootId+'_canvas').getClientRects()[0];
                var diff = 2;
               
                var symbol = $("<div id=" + options.ID + "></div>");
                var left = (evt.startX + chartPos.left + $(document).scrollLeft() - options.ShapeSize.width + diff);
                var top = (evt.startY + chartPos.top + $(document).scrollTop() - options.ShapeSize.height + diff);
                
                    $(symbol).css({
                       
                        "left":  width ? left - width / 3 : left ,
                        "top":  height ? top - height / 3 : top ,
                        "position": "absolute",
                        "visibility": "visible",                        
                        "opacity": options.Style.Opacity,
                        "z-index" : 200
                    });
                    var style = options.ShapeSize;
                    options.symbol = symbol;
                    var shapeAttr = {};
                    switch (symbolName.toLowerCase()) {
                        case "circle":
                            $(symbol).css({
                                "width": width ? width : options.ShapeSize.width,
                                "height": height ? height : options.ShapeSize.height,
								"left": width ? left - width / 3 - diff : (options.ShapeSize.width > 10 ? (left + (options.ShapeSize.width - 10) / 2) : left),
                                "top":  height ? top - height / 3 - diff : (options.ShapeSize.height > 10 ? (top + (options.ShapeSize.height - 10) / 2) : top),
                                "border-style": "solid",
                                "background-color": options.Style.Color,
                                "border-color": options.Style.BorderColor,
                                "border-width": options.Style.BorderWidth,
                                "border-radius": 200,
                            });
                            break;
                        case "rectangle":
                            $(symbol).css({
                                "width": options.ShapeSize.width,
                                "height": options.ShapeSize.height,
                                "border-style": "solid",
                                "background-color": options.Style.Color,
                                "border-color": options.Style.BorderColor,
                                "border-width": options.Style.BorderWidth,
                            });
                            break;
                        case "diamond":
                            $(symbol).css({

                                "width": options.ShapeSize.width / 2,
                                "height": options.ShapeSize.height / 2,
                                "left": (evt.startX + chartPos.left + $(document).scrollLeft() - options.ShapeSize.width - 15),
                                "top": (evt.startY + chartPos.top + $(document).scrollTop() - options.ShapeSize.width / 2),
                                "border-style": "solid",
                                "background-color": options.Style.Color,
                                "border-color": options.Style.BorderColor,
                                "margin": "3px 0 0 30px",
                                "transform": "rotate(-45deg)",
                                "transform-origin": "0 100%",
                            });
                            break;
                        case "triangle":
                                                     
                            var direction = "M" + " " + (evt.startX + (-style.width / 2)) + " " + (evt.startY + (style.height / 2)) + " " + "L" + " " + (evt.startX) + " " + (evt.startY + (-style.height / 2))
                                + " " + "L" + " " + (evt.startX + (style.width / 2)) + " " + (evt.startY + (style.height / 2)) + " " + "L" + " " + (evt.startX + (-style.width / 2)) + " " + (evt.startY + (style.height / 2));
                            shapeAttr.direction = direction;
                            this.drawSvgShape(options, shapeAttr);
                           
                            break;
                        case "invertedtriangle":
                                                     
                            var direction = "M" + " " + (evt.startX + (-style.width / 2)) + " " + (evt.startY + (style.height / 2)) + " " + "L" + " " + (evt.startX) + " " +
                                (evt.startY + (-style.height / 2)) + " " + "L" + " " + (evt.startX + (style.width / 2)) + " " + (evt.startY + (style.height / 2)) + " " + "L" + " " +
                                (evt.startX + (-style.width / 2)) + " " + (evt.startY + (style.height / 2));
                            var x = evt.startX;
                            var y = evt.startY;
                            var rotate = 'rotate(180,' + x + ',' + y + ')';

                            shapeAttr.direction = direction;
                            shapeAttr.rotate = rotate;
                            this.drawSvgShape(options, shapeAttr);
                            
                            break;
                       
                        case "cross":
                           
                            var direction = "M" + " " + (evt.startX + (-style.width / 2)) + " " + (evt.startY) + " " + "L" + " " + (evt.startX + (style.width / 2))
                                + " " + (evt.startY) + " " + "M" + " " + (evt.startX) + " " + (evt.startY + (style.height / 2)) + " " + "L" + " " + (evt.startX) + " " +
                                (evt.startY + (-style.height / 2));

                            shapeAttr.direction = direction;
                            this.drawSvgShape(options, shapeAttr);

                            break;
                        case "star":
                           
                            var direction = "M" + " " + (evt.startX + (style.width / 3)) + " " + (evt.startY + (-style.height / 2)) + " " + "L" + " " + (evt.startX + (-style.width / 2)) + " " +
                                (evt.startY + (style.height / 6)) + " " + "L" + " " + (evt.startX + (style.width / 2)) + " " + (evt.startY + (style.height / 6)) + " " + "L" + " " + (evt.startX + (-style.width / 3)) + " " +
                                (evt.startY + (-style.height / 2)) + " " + "L" + " " + (evt.startX) + " " + (evt.startY + (style.height / 2)) + " " + "L" + " " + (evt.startX + (style.width / 3)) + " " + (evt.startY + (-style.height / 2));

                            shapeAttr.direction = direction;
                            this.drawSvgShape(options, shapeAttr);

                            break;
                        case "hexagon":
                           
                            var direction = "M" + " " + (evt.startX + (-style.width / 2)) + " " + (evt.startY) + " " + "L" + " " + (evt.startX + (-style.width / 4)) + " " +
                                (evt.startY + (-style.height / 2)) + " " + "L" + " " + (evt.startX + (style.width / 4)) + " " + (evt.startY + (-style.height / 2)) + " " + "L" + " " +
                                (evt.startX + (style.width / 2)) + " " + (evt.startY) + " " + "L" + " " + (evt.startX + (style.width / 4)) + " " + (evt.startY + (style.height / 2)) + " " + "L" + " " +
                                (evt.startX + (-style.width / 4)) + " " + (evt.startY + (style.height / 2)) + " " + "L" + " " + (evt.startX + (-style.width / 2)) + " " + (evt.startY);

                            shapeAttr.direction = direction;
                            this.drawSvgShape(options, shapeAttr);

                        case "wedge":
                           
                            var direction = "M" + " " + (evt.startX + (-style.width / 2)) + " " + (evt.startY) + " " + "L" + " " + (evt.startX + style.width) + " " +
                                (evt.startY + (-style.height / 2)) + " " + "L" + " " + (evt.startX + (3 * (style.width / 4))) + " " + (evt.startY) + " " + "L" + " " +
                                (evt.startX + (style.width)) + " " + (evt.startY + (style.height / 2)) + " " + "L" + " " + (evt.startX) + " " + (evt.startY);

                            shapeAttr.direction = direction;
                            this.drawSvgShape(options, shapeAttr);

                            break;
                        case "trapezoid":
                                                       
                            var direction = "M" + " " + (evt.startX + (-style.width / 2)) + " " + (evt.startY) + " " + "L" + " " + (evt.startX + (-style.width / 2)) + " " +
                                (evt.startY + (-style.height / 4)) + " " + "L" + " " + (evt.startX + (-style.width / 2) + (style.width)) + " " + (evt.startY + (-style.height / 2)) + " " + "L" + " " +
                                (evt.startX + (-style.width / 2) + (style.width)) + " " + (evt.startY + (style.height / 2)) + " " + "L" + " " + (evt.startX + (-style.width / 2)) + " " + (evt.startY + (style.height / 4)) +
                                " " + "L" + " " + (evt.startX + (-style.width / 2)) + " " + (evt.startY);

                            shapeAttr.direction = direction;
                            this.drawSvgShape(options, shapeAttr);

                            break;
                        case "uparrow":
                           
                            var direction = "M" + " " + ((evt.startX) + style.width) + " " + (evt.startY - (style.height / 2) + (style.height / 4)) + " " + "L" + " " + ((evt.startX) + style.width) + " " + (evt.startY - (style.height / 2) + (-style.height / 4)) + " " + "L" + " " + ((evt.startX) + (style.width / 2)) + " " + (evt.startY - (style.height / 2) + (-style.height / 4)) + " " + "L" + " " + ((evt.startX) + (style.width / 2)) + " " + (evt.startY - (style.height / 2) + (-style.height / 2)) + " " + "L" + " " + ((evt.startX)) + " " + (evt.startY - (style.height / 2)) + " " + "L" + " " + ((evt.startX) + (style.width / 2)) + " " + (evt.startY - (style.height / 2) + (style.height / 2)) + " " + "L" + " " + ((evt.startX) + (style.width / 2)) + " " + (evt.startY - (style.height / 2) + (style.height / 4)) + " " + "L" + " " + ((evt.startX) + style.width) + " " + (evt.startY - (style.height / 2) + (style.height / 4));
                            var x = ((evt.startX));
                            var y = (evt.startY - (style.height / 2));
                            
                            var rotate = 'rotate(90,' + x + ',' + y + ')';
                            shapeAttr.direction = direction;
                            shapeAttr.rotate = rotate;
                            this.drawSvgShape(options, shapeAttr);

                            break;
                        case "downarrow":
                           
                            var direction = "M" + " " + ((evt.startX) + style.width) + " " + (evt.startY + (style.height / 2) + (style.height / 4)) + " " + "L" + " " + ((evt.startX) + style.width) + " " +
                                (evt.startY + (style.height / 2) + (-style.height / 4)) + " " + "L" + " " + ((evt.startX) + (style.width / 2)) + " " + (evt.startY + (style.height / 2) + (-style.height / 4))
                                + " " + "L" + " " + ((evt.startX) + (style.width / 2)) + " " + (evt.startY + (style.height / 2) + (-style.height / 2)) + " " + "L" + " " + ((evt.startX)) + " " + (evt.startY + (style.height / 2))
                                + " " + "L" + " " + ((evt.startX) + (style.width / 2)) + " " + (evt.startY + (style.height / 2) + (style.height / 2)) + " " + "L" + " " + ((evt.startX) + (style.width / 2)) + " " +
                                (evt.startY + (style.height / 2) + (style.height / 4)) + " " + "L" + " " + ((evt.startX) + style.width) + " " + (evt.startY + (style.height / 2) + (style.height / 4));
                            var x = ((evt.startX));
                            var y = (evt.startY + (style.height / 2));
                            var rotate = 'rotate(-90,' + x + ',' + y + ')';

                            shapeAttr.direction = direction;
                            shapeAttr.rotate = rotate;
                            this.drawSvgShape(options, shapeAttr);

                            break;
                        case "leftarrow":
                           
                            var direction = "M" + " " + ((evt.startX - (style.width / 2)) + style.width) + " " + (evt.startY + (style.height / 4)) + " " + "L" + " " + ((evt.startX - (style.width / 2)) + style.width) + " " + (evt.startY + (-style.height / 4)) + " " + "L" + " " +
                                ((evt.startX - (style.width / 2)) + (style.width / 2)) + " " + (evt.startY + (-style.height / 4)) + " " + "L" + " " + ((evt.startX - (style.width / 2)) + (style.width / 2)) + " " + (evt.startY + (-style.height / 2)) + " " + "L" + " " + ((evt.startX -
                                (style.width / 2))) + " " + (evt.startY) + " " + "L" + " " + ((evt.startX - (style.width / 2)) + (style.width / 2)) + " " + (evt.startY + (style.height / 2)) + " " + "L" + " " + ((evt.startX - (style.width / 2)) + (style.width / 2)) + " " +
                                (evt.startY + (style.height / 4)) + " " + "L" + " " + ((evt.startX - (style.width / 2)) + style.width) + " " + (evt.startY + (style.height / 4));

                            shapeAttr.direction = direction;
                            this.drawSvgShape(options, shapeAttr);

                            break;
                        case "rightarrow":

                            var direction = "M" + " " + ((evt.startX + (style.width / 2)) + style.width) + " " + (evt.startY + (style.height / 4)) + " " + "L" + " " +
                                ((evt.startX + (style.width / 2)) + style.width) + " " + (evt.startY + (-style.height / 4)) + " " + "L" + " " + ((evt.startX + (style.width / 2)) + (style.width / 2)) + " " +
                                (evt.startY + (-style.height / 4)) + " " + "L" + " " + ((evt.startX + (style.width / 2)) + (style.width / 2)) + " " + (evt.startY + (-style.height / 2)) + " " + "L" + " " +
                                ((evt.startX + (style.width / 2))) + " " + (evt.startY) + " " + "L" + " " + ((evt.startX + (style.width / 2)) + (style.width / 2)) + " " + (evt.startY + (style.height / 2)) + " " + "L" + " " +
                                ((evt.startX + (style.width / 2)) + (style.width / 2)) + " " + (evt.startY + (style.height / 4)) + " " + "L" + " " + ((evt.startX + (style.width / 2)) + style.width) + " " + (evt.startY + (style.height / 4));
                            var x = ((evt.startX + (style.width / 2)));
                            var y = evt.startY;
                            var rotate = 'rotate(180,' + x + ',' + y + ')';

                            shapeAttr.direction = direction;
                            shapeAttr.rotate = rotate;
                            this.drawSvgShape(options, shapeAttr);

                            break;
                        case "pentagon":
                           
                            var eq = 72;
                            var radius = Math.sqrt(style.height * style.height + style.width * style.width) / 2;
                            var sb = ej.EjSvgRender.utils._getStringBuilder();
                            for (var i = 0; i <= 5; i++) {
                                var deg = i * eq;
                                var rad = (Math.PI / 180) * deg;
                                var x1 = radius * Math.cos(rad);
                                var y1 = radius * Math.sin(rad);
                                if (i == 0)
                                    sb.append("M" + " " + (evt.startX + x1) + " " + (evt.startY + y1) + " ");
                                else
                                    sb.append("L" + " " + (evt.startX + x1) + " " + (evt.startY + y1) + " ");
                            }
                            shapeAttr.direction = direction;
                            this.drawSvgShape(options, shapeAttr);

                            break;
                        case "ellipse":
                            $(symbol).css({
                                "width": options.ShapeSize.width,
                                "height": options.ShapeSize.height / 2,
                                "border-style": "solid",
                                "top": (evt.startY + chartPos.top + $(document).scrollTop() - options.ShapeSize.height + 4),
                                "background-color": options.Style.Color,
                                "border-color": options.Style.BorderColor,
                                "border-width": options.Style.BorderWidth,
                                "-moz-border-radius": "100px / 50px",
                                "-webkit-border-radius": "100px / 50px",
                                "border-radius": "100px/50px",
                            });
                            break;
                        case "horizline":
                            $(symbol).css({
                                "width": options.ShapeSize.width,
                                "height": 0,
                                "border-style": "solid",
                                "top": (evt.startY + chartPos.top + $(document).scrollTop() - 2),
                                "border-color": options.Style.BorderColor,
                                "border-width": '2px',
                            });
                            break;
                        case "vertline":
                            $(symbol).css({
                                "width": 0,
                                "height": options.ShapeSize.height,
                                "border-style": "solid",
                                "left": (evt.startX + chartPos.left + $(document).scrollLeft() - 2),
                                "border-color": options.Style.BorderColor,
                                "border-width": '2px',
                            });
                            break;
                        default:
                            $(symbol).css({
                                "width": options.ShapeSize.width,
                                "height": options.ShapeSize.height,
                                "border-style": "solid",
                                "background-color": options.Style.Color,
                                "border-color": options.Style.BorderColor,
                                "border-width": options.Style.BorderWidth,
                                "border-radius": "50px",
                            });
                            break;
                    }
                    parentDiv[0].appendChild(symbol[0]);
                document.getElementById(this._rootId).appendChild(parentDiv[0]);
            }
        },
        //draw SVG Marker shape on mouse over the points
        drawSvgShape: function (options, attr) {
            var chartPos = document.getElementById(this._rootId+"_canvas").getClientRects()[0];
            var svgLink = "http://www.w3.org/2000/svg";
            var svgShape = document.createElementNS(svgLink, "svg");
            svgShape.setAttribute('id', 'mySVG');
            svgShape.setAttribute('width', chartPos.width);
            svgShape.setAttribute('height', chartPos.height);
            $(options.symbol).css({
                "left":  $(document).scrollLeft() + chartPos.left + 'px',
                "top": $(document).scrollTop() + chartPos.top + 'px',
            });
            var attrOptions = {
                'id': 'svgPath',
                'fill-opacity': options.Style.Opacity,
                'stroke-width': options.Style.BorderWidth,
                'fill': options.Style.Color,
                'transform' : attr.rotate,
                'stroke': options.Style.BorderColor,
            };
            attrOptions.d = attr.direction;

            var path = document.createElementNS(svgLink, "path");
            $(path).attr(attrOptions).appendTo(svgShape);
            options.symbol[0].appendChild(svgShape);
        },
        drawCircle: function (options, element) {
            if (!options.lgndCtx) {
                this.ctx.save();
                this.ctx.beginPath();
                this.ctx.arc(options.cx, options.cy, options.r, 0, 2 * Math.PI);
                this.ctx.fillStyle = options.fill;
                this.ctx.globalAlpha = options.opacity;
                this.ctx.fill();
                this.ctx.lineWidth = options["stroke-width"];
                this.ctx.strokeStyle = options.stroke;
                this.ctx.stroke();
                this.ctx.restore();
            } else {
                this.lgndCtx.save();
                this.lgndCtx.beginPath();
                this.lgndCtx.arc(options.cx, options.cy, options.r, 0, 2 * Math.PI);
                this.lgndCtx.fillStyle = options.fill;
                this.lgndCtx.globalAlpha = options.opacity;
                this.lgndCtx.fill();
                this.lgndCtx.lineWidth = options["stroke-width"];
                this.lgndCtx.strokeStyle = options.stroke;
                this.lgndCtx.stroke();
                this.lgndCtx.restore();
            }
        },    
        
        drawPolyline: function (options, element) {
            
            this.ctx.save();
            this.ctx.beginPath();
            var points = options.points.split(" ");
            for (var i = 0; i < points.length - 1; i++) {
                var point = points[i].split(",");
                var x = point[0];
                var y = point[1];
                if (i == 0) 
                    this.ctx.moveTo(x,y);
                 else
                    this.ctx.lineTo(x,y);
            }
            this.ctx.lineWidth = 0.5;
            this.ctx.stroke();
            this.ctx.restore();            
        },
        drawPolygon: function (options, element) {
           
        },
        setFillAttribute: function (element, options) {
           
        },
        setStrokeAttribute: function (element, options) {
            
        },
        changePathValue: function (options) {
            
        },


        drawArc: function (w, h, options) {
            var x = 0, y = 0;
  
        },
        changeVMLStyle: function($element, options) {
           
        },
        applyVMLStyle: function ($element, options) {

        },
       
        drawEllipse: function (options, element) {

            var circumference = Math.max(options.rx, options.ry);
            var scaleX = options.rx / circumference;
            var scaleY = options.ry / circumference;
            if (!options.lgndCtx) {
                this.ctx.save();
                this.ctx.beginPath();
                this.ctx.translate(options.rx, options.ry);
                this.ctx.scale(scaleX, scaleY);
                this.ctx.arc(options.cx - options.rx, options.cy * 2 - options.rx, 7.2, 0, 2 * Math.PI, false);
                this.ctx.fillStyle = options.fill;
                this.ctx.fill();
                this.ctx.lineWidth = options["stroke-width"];
                this.ctx.strokeStyle = options.stroke;
                this.ctx.stroke();
                this.ctx.restore();
            } else {
                this.lgndCtx.save();
                this.lgndCtx.beginPath();
                this.lgndCtx.translate(options.rx, options.ry);
                this.lgndCtx.scale(scaleX, scaleY);
                this.lgndCtx.arc(options.cx - options.rx, options.cy * 2 - options.rx, 7.2, 0, 2 * Math.PI, false);
                this.lgndCtx.fillStyle = options.fill;
                this.lgndCtx.fill();
                this.lgndCtx.lineWidth = options["stroke-width"];
                this.lgndCtx.strokeStyle = options.stroke;
                this.lgndCtx.stroke();
                this.lgndCtx.restore();
            }
        },
      
        _getAttrVal: function (ele, val, option) {
           
        },
        _setAttr: function (element, attribute) {
          
        },
       
        setSpanAttr: function (options, label, element) {
           
        },
       
        //text element attributes values are replaced 
        _textAttrReplace: function (options, label, font, $Ele) {
           
        },

        drawImage: function (options, element) {
             this.ctx.save();
             var imageObj = new Image();
             imageObj.src = options.href;
             this.ctx.drawImage(imageObj, options.x, options.y, options.width, options.height);
             this.ctx.restore();
        },

        createDefs: function () {

        },

        createGroup: function (options) {
           
        },

        createText: function (options, label) {
            var text = document.createElement("text");
            $(text).css({"transform": options.rotateAngle,
                "position": "absolute"}); 
            if (!ej.util.isNullOrUndefined(label))
                text.textContent = label;
            return text;
        },


        //Draw clip path for each series to avoid series overlap in multiple axes zooming
        drawAxesBoundsClipPath: function (gSeriesGroupEle, options, invertedAxis) {
            var clipOptions = options;
           
        },

        getAttrVal: function (ele, val, option) {
       
        },

        hexFromRGB: function (color) {
          
        },

        createDelegate: function (context, handler) {
           
        },

        drawClipPath: function (options, element) {
            
        },

        append: function (childEle, parentEle) {
            $(childEle).appendTo(parentEle);
			},
			
		createLegendCanvas: function (element) {            
            this.legendsvgObj = document.createElement('canvas');
            this._rootId = jQuery(element).attr("id");
            this.legendsvgObj.setAttribute('id', "legend_" + this._rootId + '_canvas');
            this.lgndCtx = this.legendsvgObj.getContext("2d");
            return this.legendsvgObj;
        }
		
    };
})(jQuery);


;
ej.EjSvgRender = function (element) {

    this.svgLink = "http://www.w3.org/2000/svg";
    this.svgObj = document.createElementNS(this.svgLink, "svg");
    this._rootId = jQuery(element).attr("id");
    this.svgObj.setAttribute('id', this._rootId + '_svg');

};
(function ($) {
ej.EjSvgRender.prototype = {
    drawPath: function (options, element) {
        if ($("#" + options.id).length > 0) { 
            $("#" + options.id).attr(options);
        }
        else {
            var path = document.createElementNS(this.svgLink, "path");
            $(path).attr(options).appendTo(element);
        }

    },
    createLegendSvg:function(element)
    {
        this.svgLink = "http://www.w3.org/2000/svg";
        this.legendsvgObj = document.createElementNS(this.svgLink, "svg");
        this._rootId = jQuery(element).attr("id");
        this.legendsvgObj.setAttribute('id', "legend_" + this._rootId + '_svg');
        return this.legendsvgObj;
    },
    drawPolyline: function (options, element) {
        if ($("#" + options.id).length > 0) {
            $("#" + options.id).attr(options);
        }
        else {
            var polyline = document.createElementNS(this.svgLink, "polyline");
            $(polyline).attr(options).appendTo(element);
        }

    },

    drawLine: function (options, element) {
        if ($("#" + options.id).length > 0) {
            $("#" + options.id).attr(options);
        }
        else {
            var path = document.createElementNS(this.svgLink, "line");
            $(path).attr(options);
            $(path).appendTo(element);
        }
    },
    drawPolygon: function (options, element) {
        if ($("#" + options.id).length > 0) {
            $("#" + options.id).attr(options);
        }
        else {
            var polygon = document.createElementNS(this.svgLink, "polygon");
            $(polygon).attr(options);
            $(polygon).appendTo(element);
        }
    },
    drawCircle: function (options, element) {
        if ($("#" + options.id).length > 0) {
            $("#" + options.id).attr(options);
        }
        else {
            var circle = document.createElementNS(this.svgLink, "circle");
            $(circle).attr(options).appendTo(element);
        }
    },
    drawEllipse: function (options, element) {
        if ($("#" + options.id).length > 0) {
            $("#" + options.id).attr(options);
        }
        else {
            var ellipse = document.createElementNS(this.svgLink, "ellipse");
            $(ellipse).attr(options).appendTo(element);
        }
    },
    
    drawRect: function (options, element) {
            if ($("#" + options.id).length > 0) {
                $("#" + options.id).attr(options);
            }
            else {
                var rect = document.createElementNS(this.svgLink, "rect");
                $(rect).attr(options).appendTo(element);
            }    
    },

    createGradientElement: function (name, colors, x1, y1, x2, y2, element) {
        var colorName;
        if (Object.prototype.toString.call(colors) == '[object Array]') {
            var options = {
                'id': this.svgObj.id + '_' + name + 'Gradient',
                'x1': x1 + '%',
                'y1': y1 + '%',
                'x2': x2 + '%',
                'y2': y2 + '%'
            };
            var cName = '#' + this.svgObj.id + '_' + name + 'Gradient';
                this.drawGradient(options, colors, element);
            colorName = 'url(#' + this.svgObj.id + '_' + name + 'Gradient)';
        }
        else {
            colorName = colors;
        }
        return colorName;
    },

    drawGradient: function (options, gradientEle, element) {

        var defs = this.createDefs();
        var linerGradient = document.createElementNS(this.svgLink, "linearGradient");
        $(linerGradient).attr(options);
        for (var i = 0; i < gradientEle.length; i++) {
            var stop = document.createElementNS(this.svgLink, "stop");
            $(stop).attr({
                'offset': gradientEle[i].colorStop,
                'stop-color': gradientEle[i].color,
                'stop-opacity': 1
            });
            $(stop).appendTo(linerGradient);
        }

        $(linerGradient).appendTo(defs);
        $(defs).appendTo(element);
    },

    drawText: function (options, label, groupEle,font) {
        
        if ($("#" + options.id).length > 0) 
            this._textAttrReplace(options, label,font);           
       else{
            var text = document.createElementNS(this.svgLink, "text");
            var $text = $(text);


            if (jQuery.type(label) == "array") {
                var j = 0;
                $text.attr(options);
                for (var i = 0; i < label.length; i++) {
                    var textspan = document.createElementNS(this.svgLink, "tspan");
                    textspan.textContent = label[i];
                    $(textspan).attr({ "x": options.x, "dy": j });
                    $(textspan).appendTo(text);
                    var bounds = ej.EjSvgRender.utils._measureText(label[i],null,font);
                    j = bounds.height+2;
                }
            } else {
                text.textContent = label;
                $text.attr(options);
            }
            $text.appendTo(groupEle);

 
        }
    },
    //text element attributes values are replaced 
    _textAttrReplace: function (options, label, font) {
        $("#" + options.id).attr(options);

        if (jQuery.type(label) == "array") {
            var elements = $("#" + options.id).children("tspan");
            var j = 0;
            if (elements.length > 0 && elements.length == label.length) {
                for (var i = 0; i < elements.length; i++) {
                    var element = elements[i];
                    $(element).attr({ "x": options.x, "dy": j });
                    element.textContent = label[i];
                    var bounds = ej.EjSvgRender.utils._measureText(label[i], null, font);
                    j = bounds.height + 2;
                }
            }
            else {
                $("#" + options.id).remove();
                this.drawText(options, label, this.gTransToolEle,font);

            }
        } else {
            $("#" + options.id).text(label);
        }

    },

    drawImage: function (options, element) {

        var img = document.createElementNS(this.svgLink, 'image');
        img.setAttributeNS(null, 'height', options.height);
        img.setAttributeNS(null, 'width', options.width);
        img.setAttributeNS('http://www.w3.org/1999/xlink', 'href', options.href);
        img.setAttributeNS(null, 'x', options.x);
        img.setAttributeNS(null, 'y', options.y);
        img.setAttributeNS(null, 'id', options.id);
        img.setAttributeNS(null, 'visibility', options.visibility);
        if (!ej.util.isNullOrUndefined(options.clippath) || !ej.util.isNullOrUndefined(options.preserveAspectRatio)) {
            img.setAttributeNS(null, 'clip-path', options.clippath);
            img.setAttributeNS(null, "preserveAspectRatio", options.preserveAspectRatio);
        }
        $(img).appendTo(element);

    },

    createDefs: function () {
        var defs = document.createElementNS(this.svgLink, "defs");
        return defs;
    },

    createClipPath: function (options) {
        var clipPath = document.createElementNS(this.svgLink, "clipPath");
        $(clipPath).attr(options);
        return clipPath;
    },
    createForeignObject: function (options) {
        var object = document.createElementNS(this.svgLink, "foreignObject");
        $(object).attr(options);
        return object;
    },
    createGroup: function (options) {
        var group = document.createElementNS(this.svgLink, "g");
        $(group).attr(options);
        return group;
    },

    createText: function (options, label) {
        var text = document.createElementNS(this.svgLink, "text");
        $(text).attr(options);
        if (!ej.util.isNullOrUndefined(label))
            text.textContent = label;
        return text;
    },

    createPath: function (options) {
        var path = document.createElementNS(this.svgLink, "path");
        $(path).attr(options);
        return path;
    },

    createCircle: function (options) {
        var circle = document.createElementNS(this.svgLink, "circle");
        $(circle).attr(options);
        return circle;
    },

    createLine: function (options) {
        var line = document.createElementNS(this.svgLink, "line");
        $(line).attr(options);
        return line;
    },

   
    _getAttrVal: function (ele, val, option) {
        var value = $(ele).attr(val);
        if (value != null)
            return value;
        else
            return option;
    },
 
    
    hexFromRGB: function (color) {
        var r = color.R;
        var g = color.G;
        var b = color.B;
        var hex = [r.toString(16), g.toString(16), b.toString(16)];
        $.each(hex, function (nr, val) { if (val.length === 1) { hex[nr] = "0" + val; } });
        return "#" + (hex.join("").toUpperCase());
    },

    hexToRGB:function(hex) {
    var result = /^#?([a-f\d]{2})([a-f\d]{2})([a-f\d]{2})$/i.exec(hex);
    return result ? {
        R: parseInt(result[1], 16),
        G: parseInt(result[2], 16),
        B: parseInt(result[3], 16)
    } : null;
  },


    createDelegate: function (context, handler) {
        return function (e) {
            handler.apply(context, [e, this]);
        };
    },

    drawClipPath: function (options, element) {

        var defs = this.createDefs();
        var clipPath = this.createClipPath({ 'id': options.id });
        this.drawRect(options, clipPath);

        this.append(clipPath, defs);

        this.append(defs, element);

    },
   
   append: function (childEle, parentEle) {
        $(childEle).appendTo(parentEle);
   },
   _setAttr: function (element, attribute) {
       $(element).attr(attribute);
   }
};

ej.EjSvgRender.commonChartEventArgs = {
    cancel: false,
    data: null
};
ej.EjSvgRender.utils = {

   

    _decimalPlaces: function (num) {
        var match = ('' + num).match(/(?:\.(\d+))?(?:[eE]([+-]?\d+))?$/);
        if (!match) { return 0; }
        return Math.max(
       0,
        // Number of digits right of decimal point.
       (match[1] ? match[1].length : 0)
        // Adjust for scientific notation.
       - (match[2] ? +match[2] : 0));
    },
    _getLabelContent: function (pos, axis) {
        switch (axis._categoryValueType) {
            case 'number':
                var customFormat = (ej.util.isNullOrUndefined(axis.labelFormat)) ? null : axis.labelFormat.match('{value}');
                return (ej.util.isNullOrUndefined(axis.labelFormat)) ? axis.labels[Math.floor(pos)] : (customFormat != null) ? axis.labelFormat.replace('{value}', axis.labels[Math.floor(pos)]) : (Globalize.format(axis.labels[Math.floor(pos)], axis.labelFormat));
            case 'date':
                return (Globalize.format(new Date(axis.labels[Math.floor(pos)]), ((ej.util.isNullOrUndefined(axis.labelFormat)) ? "dd/MM/yyyy" : axis.labelFormat)));
            case 'string':
                return axis.labels[Math.floor(pos)];
            default:
                return "";
        }
    },
    
    //Calculation for label size in template
    _getSeriesTemplateSize: function (point, pointIndex, series, isLeft, sender) {
        var point;
        var pointIndex;
        var areaType = sender.model.AreaType;
         
        var chartContainer = sender._id;
        var seriesIndex = $.inArray(series, sender.model._visibleSeries);

            if ($('#template_group_' + chartContainer).length != 0)
                var templateContainer = $('#template_group_' + chartContainer);
            else
                templateContainer = $("<div></div>").attr('id', "template_group_" + chartContainer);

            templateContainer.css('position', 'relative').css('z-index', 1000);
            var cloneNode = $("#" + series.marker.dataLabel.template).clone();
            $(cloneNode).attr("id", series.marker.dataLabel.template + '_' + seriesIndex + '_' + pointIndex + '_' + chartContainer);
            var $cloneNode = $(cloneNode);
            $cloneNode.css("position", "absolute");
           
            point.count = 1;
            var data = { series: series, point: point };
            $cloneNode.html($cloneNode.html().parseTemplate(data));

            var display = (areaType == "cartesianaxes" || !series.enableAnimation || (series.type.toLowerCase() == "pyramid" || series.type.toLowerCase() == "funnel")) ? "block" : "none";
            $cloneNode.css("display", display).appendTo($(templateContainer));
            $(templateContainer).appendTo('#' + chartContainer);
            point.size = { height: $cloneNode.height(), width: $cloneNode.width() };
         
            if (isLeft) {
                if (ej.util.isNullOrUndefined(series.LeftLabelMaxHeight) || series.LeftLabelMaxHeight < point.size.height) {
                    series.LeftLabelMaxHeight = point.size.height;
                }
                if (ej.util.isNullOrUndefined(series.LeftLabelMaxWidth) || series.LeftLabelMaxWidth < point.size.width) {
                    series.LeftLabelMaxWidth = point.size.width;
                }
            }
            else {
                if (ej.util.isNullOrUndefined(series.RightLabelMaxHeight) || series.RightLabelMaxHeight < point.size.height) {
                    series.RightLabelMaxHeight = point.size.height;
                }
                if (ej.util.isNullOrUndefined(series.RightLabelMaxWidth) || series.RightLabelMaxWidth < point.size.width) {
                    series.RightLabelMaxWidth = point.size.width;
                }
            }
        
    },
    getMinPointsDelta: function (axis, chartObj, start) {
        var chart = chartObj;
        var m_minPointsDelta = Number.MAX_VALUE;
        $.each(chart.model._visibleSeries, function (index, series) {
            if (series.visibility.toLowerCase() == 'visible' && axis.name == series.xAxisName) {
                var xValues = ej.DataManager(series.points, ej.Query().sortBy("X")).executeLocal();
                if (xValues.length == 1) {
                    var minValue = (start) ? start : series.xAxis.visibleRange.min;
                    var delta = xValues[0].X - minValue;
                    if (delta != 0)
                        m_minPointsDelta = Math.min(m_minPointsDelta, delta);
                }
                else {
                    $.each(xValues, function (pointIndex, point) {
                        if (pointIndex > 0) {
                            var deltaValue = point.X - xValues[pointIndex - 1].X;
                            if (deltaValue != 0) {
                                m_minPointsDelta = Math.min(m_minPointsDelta, deltaValue);
                            }
                        }
                    });
                }
            }
        });

        if (m_minPointsDelta == Number.MAX_VALUE) {
            m_minPointsDelta = 1;
        }
        return m_minPointsDelta;
    },
    //Calculation for label size
    _getSeriesMaxLabel: function (series) {
        var maxTxtDim = { width: 0, height: 0 };
        var width = $(this.svgObject).width();
        var labels=[];
        if (series.labels.length > 0) {
            for (var j = 0; j < series.labels.length; j++) {
                var dim = this._measureText(series.visibleLabels[j], width, series.marker.dataLabel.font);
                if (maxTxtDim.width < dim.width) {
                    maxTxtDim.width = dim.width;
                }
                if (maxTxtDim.height < dim.height) {
                    maxTxtDim.height = dim.height;
                }
            }
            series.LabelMaxHeight = maxTxtDim.height;
            series.LabelMaxWidth = maxTxtDim.width;
        }
        else {
            labels.push(series.rightsidePoints);
            labels.push(series.leftsidePoints);

            for (var k = 0; k < labels.length; k++) {
                for (var j = 0; j < labels[k].length; j++) {
                    var text = (labels[k][j].text) ? labels[k][j].text : labels[k][j].y;
                    var dim = this._measureText(text, width, series.marker.dataLabel.font);
                    if (maxTxtDim.width < dim.width) {
                        maxTxtDim.width = dim.width;
                    }
                    if (maxTxtDim.height < dim.height) {
                        maxTxtDim.height = dim.height;
                    }
                }
                if (k == 0) {
                    series.RightLabelMaxHeight = maxTxtDim.height;
                    series.RightLabelMaxWidth = maxTxtDim.width;
                }
                else {
                    series.LeftLabelMaxHeight = maxTxtDim.height;
                    series.LeftLabelMaxWidth = maxTxtDim.width;
                }
             }
        }
       

    },
    _getMaxLabelWidth: function (axis, sender) {
        var maxTxtDim = { width: 0, height: 0 };
        var rotateLabel = '';
        var currentRow = 1;
        var vmlrendering = sender.svgRenderer.vmlNamespace;
        var range = axis.visibleRange;
        var intersectAction = axis.labelIntersectAction ? axis.labelIntersectAction.toLowerCase() : "";
        var labelPlacement = axis.labelPlacement;
        var opposedPosition = axis.opposedPosition;
        var roundingPlaces = axis.roundingPlaces;
        var labelRotation = axis.labelRotation;
        var orientation = axis.orientation;
        var maxHeight = 0;
        var prevLabels = [];
		
        if (axis.visible) {
		    var visibleLabels = axis.visibleLabels;
            for (var j = 0; j < axis.visibleLabels.length; j++) {
			    axis.visibleLabels[j].y = 0;
                var dim = this._measureText(axis.visibleLabels[j].Text, $(this.svgObject).width(), axis.font, axis.labelRotation);
                if (maxTxtDim.width < dim.width) {
                    maxTxtDim.width = dim.width;
                    rotateLabel = axis.visibleLabels[j].Text;
                }
                if (maxTxtDim.height < dim.height)
                    maxTxtDim.height = dim.height;
            }
			
			 // initialize rows to 1
                maxTxtDim.rows = 1;
                maxTxtDim.maxWidth = maxTxtDim.width;
                maxTxtDim.maxHeight = maxTxtDim.height;
				
            if (!vmlrendering && labelRotation) {
                    maxHeight = this.rotatedLabel(axis, sender, labelRotation, rotateLabel);
                    if (axis.labelIntersectAction && axis.labelIntersectAction.toLowerCase() == 'multiplerows')
                        maxHeight = maxTxtDim.height + maxHeight;
                    maxTxtDim.height = maxHeight;
                }

                if (axis.labelIntersectAction) {
                    if (!vmlrendering) {
                        if (intersectAction == 'rotate90')
                            maxTxtDim.height = this.rotatedLabel(axis, sender, 90, rotateLabel);
                        else if (intersectAction == 'rotate45')
                            maxTxtDim.height = this.rotatedLabel(axis, sender, 45, rotateLabel);
                    }
                    if (intersectAction == 'wrap') {
                        // pointX calculation to find the gap between the ticks
                        labelPlacement = (!(labelPlacement)) ? ej.datavisualization.Chart.LabelPlacement.BetweenTicks : labelPlacement;
                        var ticksbwtLabel = ((axis.labels.length > 1) && (labelPlacement.toLowerCase() == "betweenticks")) ? -0.5 : 0;
                        var tempInterval = ((axis.labels.length > 1) && (ticksbwtLabel < 0)) ? 1 + ticksbwtLabel : visibleLabels[1].Value;
                        tempInterval = (!(roundingPlaces)) ? parseFloat(tempInterval.toFixed((ej.EjSvgRender.utils._decimalPlaces(range.interval) == 0 ?
                            1 : ej.EjSvgRender.utils._decimalPlaces(range.interval)))) : parseFloat(tempInterval.toFixed(roundingPlaces));
                        var pointX = Math.ceil(((tempInterval - range.min) / (range.max - range.min)) * (axis.length));
                        var row = Math.round(maxTxtDim.width / pointX);
                        maxTxtDim.rows = labelRotation ? row - currentRow : row + currentRow;
                        maxTxtDim.height = (maxTxtDim.height + (row) * maxTxtDim.height);
                    }
                    else if (intersectAction == 'multiplerows') {
                        var spaceValue = sender._getLegendSpace();
                        var chartBorderWidth = sender.model.border.width;
                      
                        var verticalaxis = sender.model.axes[1];
                        var realWidth = $(sender.svgObject).width() - sender.model.margin.left - sender.model.margin.right;
                        var axisTitleHeight = (axis.title.text == "" || !(axis.visible)) ? 0 : (this._measureText(axis.title.text, realWidth, axis.title.font).height + (2 * sender.model.elementSpacing));
                        var vAxesWidth = sender.model.elementSpacing + axisTitleHeight + axis.majorTickLines.size + axis.axisLine.width;
                        var yLabels = sender._getYValues(sender.model._visibleSeries[0]);
                        var largest = Math.max.apply(Math, yLabels);
                        var vaxis = sender.model.axes[1];
                        var dim = this._measureText(largest, $(this.svgObject).width(), vaxis.font, vaxis.labelRotation);
 
                        var rightSpacing = spaceValue.rightLegendWidth + vAxesWidth + dim.width + sender.model.margin.right + sender.model.margin.left + (2 * chartBorderWidth);
                        var boundsWidth = $(sender.svgObject).width() - (rightSpacing);
                        if (orientation.toLowerCase() == 'horizontal') {
                            var addedMaxHeight = 0;
                            //loop to get current label
                            for (j = 0; j < visibleLabels.length; j++) {
                                //declaration
                                var currentLabel = visibleLabels[j];
                                var flag = false;
                                maxHeight = Math.max(maxHeight, addedMaxHeight)
                                addedMaxHeight = currentLabel.y = maxTxtDim.height;
                                var text = this._measureText(currentLabel.Text, $(this.svgObject).width(), axis.font);
                                var textWidth = text.width;
                                var textHeight = text.height;
                                var currentPoint = Math.abs(Math.floor(((currentLabel.Value - range.min) / (range.delta)) * (boundsWidth)));
                                // loop to get previous labels                         
                                for (var k = 0; k < prevLabels.length && !flag; k++) {
                                    for (var l = 0; prevLabels[k] && l < prevLabels[k].length; l++) {
                                        var prevLabel = prevLabels[k][l];
                                        var prevPoint = Math.abs(Math.floor(((prevLabel.Value - range.min) / (range.delta)) * (boundsWidth)));
                                        var preTextWidth = this._measureText(prevLabel.Text, $(this.svgObject).width(), axis.font).width;
                                        var value = prevPoint + preTextWidth / 2;
                                        if (value > currentPoint - textWidth / 2 && !ej.util.isNullOrUndefined(currentLabel.y)) {
                                            addedMaxHeight = currentLabel.y + textHeight;
                                            currentLabel.y += textHeight;
                                            if (k + 1 == prevLabels.length)
                                            flag = true;
                                        }
                                        else {
                                            if (l + 1 == prevLabels[k].length) {
                                                flag = true;
                                                break;
                                            }
                                        }
                                    }
                                  
                                 }
                                currentLabel.y = addedMaxHeight;
                                row = (addedMaxHeight / textHeight) - 1;
                                if( prevLabels[row]==undefined)
                                prevLabels[row] = [];
                                prevLabels[row].push(currentLabel);
                               
                            }
                            maxHeight = (prevLabels.length * textHeight);
                         //   maxHeight = opposedPosition ? maxHeight + textHeight : maxHeight;
                            maxTxtDim.height = maxTxtDim.height > maxHeight ? maxTxtDim.height : maxHeight;
                            //calculating rows
                            if (opposedPosition)
                                maxTxtDim.rows = Math.round((maxHeight + textHeight) / textHeight);
                            else
                                maxTxtDim.rows = Math.round((maxHeight + textHeight) / textHeight);
                            maxTxtDim.rows = labelRotation ? maxTxtDim.rows - 1 : maxTxtDim.rows + 1;
                            if (maxTxtDim.rows < 1) maxTxtDim.rows = 1;
                        }
                        if (orientation.toLowerCase() == 'vertical') {
                            var addedMaxWidth = 0;
                            //loop to get current label
                            for (j = 0; j < axis.visibleLabels.length; j++) {
                                currentLabel = axis.visibleLabels[j];
                                text = this._measureText(currentLabel.Text, $(this.svgObject).width(), axis.font);
                                textWidth = text.width;
                                textHeight = text.height;
                                currentPoint = Math.abs(Math.floor(((currentLabel.Value - axis.visibleRange.min) / (axis.visibleRange.delta)) * (axis.length)));
                                for (i = 0; i < j; i++) {
                                    // loop to get previous labels
                                    prevLabel = axis.visibleLabels[i];
                                    prevPoint = Math.abs(Math.floor(((prevLabel.Value - axis.visibleRange.min) / (axis.visibleRange.delta)) * (axis.length)));
                                    var prevTextHeight = this._measureText(prevLabel.Text, $(this.svgObject).width(), axis.font).height;
                                    value = prevPoint + prevTextHeight / 2;
                                    if (value > currentPoint - textHeight / 2 && axis.visibleLabels[i].y == currentLabel.y) {
                                        addedMaxWidth = currentLabel.y + textWidth;
                                        currentLabel.y = addedMaxWidth;
                                    }
                                }
                               var maxheight = addedMaxWidth;
                            }
                            // label width + multiple rows width + gap between the rows
                            maxTxtDim.width = maxTxtDim.width + maxheight +5;
                        }
                    }
            }

        }
        return maxTxtDim;
    },
	
        rotatedLabel: function (axis, sender, value, rotatedLabel) {
            // to get height of rotated labels
            var rotatedOptions = {
                'font-size': axis.font.size,
                'transform': 'rotate(' + value + ',0,0)',
                'font-family': axis.font.fontFamily,
                'font-style': axis.font.fontStyle,
                'rotateAngle' : 'rotate('+ value +'deg)',
                'text-anchor': 'middle'
            };
            var text = sender.svgRenderer.createText(rotatedOptions, rotatedLabel);
            var height = Math.ceil((this._measureBounds(text, sender).height));
            return height;
        },
		
    _getTransform: function (xAxis, yAxis, invertedAxis) {
        var x , y , width, height;
        if (invertedAxis) {
            x = yAxis.x;
            y = xAxis.y;
            width = yAxis.width;
            height = xAxis.height;
        } else {
            {
                x = xAxis.x;
                y = yAxis.y;
                width = xAxis.width;
                height = yAxis.height;
            }
        }
        return { x: x, y: y, width: width, height: height };
    },
    _measureText: function (text, maxwidth, font) {
        var element = $(document).find("#measureTex");
        if (element.length==0) {
            var textObj = document.createElement('text');
            $(textObj).attr({ 'id': 'measureTex'});
            document.body.appendChild(textObj);
        }
        else
        {
            var textObj = element[0];
        }
       
        var style=null, size=null, family=null;
        textObj.innerHTML = text;
        if (font != undefined && font.size == undefined) {
            var fontarray = font;
            fontarray=fontarray.split(" ");
            style = fontarray[0];
            size = fontarray[1];
            family = fontarray[2];
        }
       
        if (font != null) {
            textObj.style.fontSize = (font.size > 0) ? (font.size + "px") : font.size? font.size : size ;
            textObj.style.fontStyle = font.fontStyle? font.fontStyle : style ;
            textObj.style.fontFamily = font.fontFamily ? font.fontFamily : family;
        }
        textObj.style.backgroundColor = 'white';
        textObj.style.position = 'absolute';
        textObj.style.top = -100;
        textObj.style.left = 0;
		textObj.style.visibility = 'hidden';
        if (maxwidth)
            textObj.style.maxwidth = maxwidth + "px";
         
    
        var bounds = { width: textObj.offsetWidth, height: textObj.offsetHeight };
        
        return bounds;
    },
    _measureBounds: function (element, sender) {
        if (sender.model.enableCanvasRendering) 
             $(sender.element).append(element);
         else {
            sender.svgRenderer.append(element, sender.svgObject);
            sender.svgRenderer.append(sender.svgObject, sender.element);
        }
        var box = element.getBoundingClientRect();
        var bounds = { left: box.left, right: box.right, top: box.top, bottom: box.bottom, width: (box.right - box.left), height: (box.bottom - box.top) };
        $(element).remove();
        return bounds;
    },
    //Draw clip path for each series to avoid series overlap in multiple axes zooming
    _drawAxesBoundsClipPath: function (gSeriesGroupEle, options, sender) {
        var clipOptions;
        var element = $(gSeriesGroupEle);
        var trans = this._getTransform(options.xAxis, options.yAxis, sender.model.requireInvertedAxes);
        var width = (sender.model.AreaType == "polaraxes") ? $(sender.svgObject).width() : trans.width;
        var height = (sender.model.AreaType == "polaraxes") ? $(sender.svgObject).height() : trans.height;

        clipOptions = {
            'id': gSeriesGroupEle ? gSeriesGroupEle.id + '_ClipRect': '',
            'x': 0,
            'y': 0,
            'width': width,
            'height': height,
            'fill': 'white',
            'stroke-width': 1,
            'stroke': 'transparent'
        };
        
        sender.svgRenderer.drawClipPath(clipOptions, gSeriesGroupEle);
        element.attr('clip-path', 'url(#' + clipOptions.id + ')');

    },
    _getStringBuilder: function () {

        var data = [];
        var counter = 0;

        return {
            // adds string s to the stringbuilder

            append: function (s) {
                data[counter++] = s;
                return this;
            },

            // removes j elements starting at i, or 1 if j is omitted

            remove: function (i, j) {
                data.splice(i, j || 1);
                return this;
            },

            // inserts string s at i

            insert: function (i, s) {
                data.splice(i, 0, s);
                return this;
            },

            // builds the string

            toString: function (s) { return data.join(s || ""); }
        };


    },
    _addRegion: function (chart, bounds, series, point, pointIndex) {
	    var type = series.type;
        var seriesIndex = $.inArray(series, chart.model._visibleSeries);
        if (seriesIndex >= 0) {
	        var regionItem = { SeriesIndex: seriesIndex, Region: { PointIndex: pointIndex, Bounds: bounds }, type: type };
	        chart.model.chartRegions.push(regionItem);
	    }
    },

    AddRegion: function (chart, bounds, isStripLine) {
       
        if (isStripLine) {
            var regionItem = { isStripLine: isStripLine, Region: { Bounds: bounds } };
            chart.model.chartRegions.push(regionItem);
        }
    },


    _getSvgXY: function (x, y, series, sender) {
        var svgX, svgY;
        if (!(sender.model.requireInvertedAxes)) {
              svgX = x + series.xAxis.x;
              svgY = y + series.yAxis.y;
        } else {
              svgX = x + series.yAxis.x;
              svgY = y + series.xAxis.y;
        }
        return { X: svgX, Y: svgY };
    },
    _getPoint: function (point, series) {
        var x = point.X;
        var y = point.YValues[0];

        var xvalue = (series.xAxis._valueType == "logarithmic") ? ej.EjSvgRender.utils._logBase((x == 0 ? 1 : x), series.xAxis.logBase) : x;
        var yvalue = (series.yAxis._valueType == "logarithmic") ? ej.EjSvgRender.utils._logBase((y == 0 ? 1 : y), series.xAxis.logBase) : y;
      
        xvalue =this._getPointXY(xvalue, series.xAxis.visibleRange, series.xAxis.isInversed) * (series.xAxis.width);
        yvalue = (1 - this._getPointXY(yvalue, series.yAxis.visibleRange, series.yAxis.isInversed)) * (series.yAxis.height);

        return  { X: xvalue, Y: yvalue };
    },
    _getPointXY: function (value, Range, isInversed) {
         
        var result = 0;
        result = (value - Range.min) / (Range.delta);
     
        return (isInversed) ? (1 - result) : result;   
        
    },
    
    _dateTimeLabelFormat: function (intervalType) {
        var format = 'd';
        if (intervalType.toLowerCase() == 'years')
            format = 'MMM, yyyy';
        else if (intervalType.toLowerCase() == 'months')
            format = 'dd, MMM';
        else if (intervalType.toLowerCase() == 'days')
            format = 'MM/dd/yyyy';
        else if (intervalType.toLowerCase() == 'hours')
            format = 'dd, hh:mm';
        else if (intervalType.toLowerCase() == 'seconds' || intervalType.toLowerCase() == 'minutes')
            format = 'hh:mm:ss';
        else if (intervalType.toLowerCase() == 'milliseconds')
            format = 'hh:mm:ss:tt';
        return format;
    },
    _getFontString: function (fontObj) {
        if (fontObj == null)
            fontObj = {};
        if (!fontObj.FontFamily)
            fontObj.FontFamily = "Arial";
        if (!fontObj.FontStyle)
            fontObj.FontStyle = "Normal";
        if (!fontObj.Size)
            fontObj.Size = "12px";

        return fontObj.FontStyle + " " + fontObj.Size + " " + fontObj.FontFamily;
    },

    _valueToVector: function (axis, value) {
        return this._coefficientToVector(this._valueToPolarCoefficient(axis, value));
    },

    TransformToVisible: function (currentseries, x, y, sender) {
        x = (currentseries.xAxis._valueType == "logarithmic") && x > 0 ? Math.log(x, currentseries.xAxis.logBase) : x;
        y = (currentseries.xAxis._valueType == "logarithmic") && y > 0 ? Math.log(y, currentseries.yAxis.logBase) : y;
        var radius = sender.model.Radius * this._valueToCoefficient(currentseries.yAxis, y);
        //var radius = this.chartObj.model.Radius * ej.EjSvgRender.utils._valueToPolarCoefficient(currentseries.yAxis, y);
        var point = this._valueToVector(currentseries.xAxis, x);
        return { X: sender.model.centerX + radius * point.X, Y: sender.model.centerY + radius * point.Y };
    },

    Transform3DToVisible:function(currentseries, x, y, sender)
    {
      if (currentseries.xAxis != null && currentseries.yAxis != null)
       {
          var valueType = currentseries.xAxis._valueType.toLowerCase();
          var xIsLogarithmic = (valueType == "logarithmic") ? true :false;
          x = x = xIsLogarithmic && x > 0 ? math.log(x, xlogarithmicBase) : x;
          y = y;

       if (sender.model.requireInvertedAxes)
        {
           var left = sender.model.m_AreaBounds.X;
           var top = sender.model.m_AreaBounds.Y;
            var pointX = left + currentseries.yAxis.width * ej.EjSvgRender.utils._valueToCoefficient(currentseries.yAxis, y, sender);
            var pointY = top + currentseries.xAxis.height * (1 - ej.EjSvgRender.utils._valueToCoefficient(currentseries.xAxis, x, sender))
           return { X: pointX, Y: pointY }          
        }
        else
        {
            var left = currentseries.xAxis.x;
            var top = currentseries.yAxis.y;
            var x = left + Math.round(currentseries.xAxis.width * ej.EjSvgRender.utils._valueToCoefficient(currentseries.xAxis, x, sender));
            var y= top + Math.round(currentseries.yAxis.height * (1 - ej.EjSvgRender.utils._valueToCoefficient(currentseries.yAxis,y, sender)));
            return {X:x, Y:y}
        }
    }
                
    return new Point(0, 0);
  },

    _valueToPolarCoefficient:function(axis, value)
         {
            var start = axis.visibleRange.min;
            var delta = axis.visibleRange.delta;

            result = (value - start) / delta;
            result *= 1 - 1 / (axis.visibleLabels.length);

            return axis.isInversed ?  result : 1 - result;
        },

    _coefficientToVector: function (coefficient)
       {
            var angle = Math.PI * (1.5 - 2 * coefficient);

            return  {X:Math.cos(angle), Y: Math.sin(angle)};
      },


    _valueToCoefficient: function (axis, value, sender) {
        if (sender && sender.model.AreaType=='polaraxes') {
            var yvalue = value;
        }
        else
        var yvalue = (axis._valueType.toLowerCase() == "logarithmic") ?
          ej.EjSvgRender.utils._logBase((value == 0 ? 1 : value), axis.logBase) : value;

        yvalue = (yvalue - axis.visibleRange.min) / (axis.visibleRange.delta);
        
        return (axis.isInversed) ? 1 - yvalue : yvalue;
    },
    _getBoundingClientRect: function (element, sender,series,invertedAxes) {
        var box = element.getBoundingClientRect();
        var position = $("#" + (sender.svgObject.id))[0].getBoundingClientRect();
        var xSeries, ySeries;
        if (invertedAxes) {
            xSeries = this._getTransform(series.xAxis, series.yAxis, true).x;
            ySeries = this._getTransform(series.xAxis, series.yAxis, true).y;
        } else {
            xSeries = this._getTransform(series.xAxis, series.yAxis, false).x;
            ySeries = this._getTransform(series.xAxis, series.yAxis, false).y;
        }
        var x = box.left - (xSeries + position.left);
        var y = box.top - (ySeries + position.top);
        return { x: x, y: y, width: (box.right - box.left), height: (box.bottom - box.top) };
    },
    _minMax: function (value, min, max) {
        return value > max ? max : (value < min ? min : value);
    },
    _inside: function (value, range) {
        if (value === "")
            return false;
        return (value <= range.max) && (value >= range.min);
    },
    _logBase: function (val, base) {
        return Math.log(val) / Math.log(base);
    },
    _correctRect: function (x1, y1, x2, y2) {
        return { X: Math.min(x1, x2), Y: Math.min(y1, y2), Width: Math.abs(x2 - x1), Height: Math.abs(y2 - y1) };
    },
    _getValuebyPoint: function (x, y, series) {


      
        var xValue = (series.xAxis.isInversed) ? (1 - (x / series.xAxis.width)) : (x / series.xAxis.width);
        var yValue = (series.yAxis.isInversed) ? (1 - (y / series.yAxis.height)) : (y / series.yAxis.height);

        xValue = xValue * (series.xAxis.visibleRange.delta) + series.xAxis.visibleRange.min;

        yValue = yValue * (series.yAxis.visibleRange.delta) + series.yAxis.visibleRange.min;

        xValue = (series.xAxis._valueType == "logarithmic") ? Math.pow(series.xAxis.logBase, xValue) : xValue;

        yValue = (series.yAxis._valueType == "logarithmic") ? Math.pow(series.yAxis.logBase, yValue) : yValue;

        return { PointX: xValue, PointY: yValue };

    }
   
};
ej.EjSvgRender.chartTransform3D = {
      ToRadial: Math.PI / 180,

      transform3D: function (size) {
          if (!this.vector){
          this.vector=new (new ej.Ej3DRender()).vector3D();
          this.matrixobj = new (new ej.Ej3DRender()).matrix3D();
          this.bsptreeobj = new (new ej.Ej3DRender()).BSPTreeBuilder();
          this.polygon = new (new ej.Ej3DRender()).polygon3D();
          }
        return {
            mViewport: size,
            Rotation: 0,
            Tilt: 0,
            Depth: 0,
            PerspectiveAngle: 0,
            needUpdate:true,
            centeredMatrix: this.matrixobj.getIdentity(),
            Perspective: this.matrixobj.getIdentity(),
            resultMatrix: this.matrixobj.getIdentity(),
            viewMatrix: this.matrixobj.getIdentity(),
            Depth: 0           
          };
     },

    transform: function (trans) {
        this.setCenter(this.vector.vector3D(trans.mViewport.Width / 2, trans.mViewport.Height / 2, trans.Depth / 2), trans);
        this.setViewMatrix(this.matrixobj.transform(0, 0, trans.Depth), trans)
        this.setViewMatrix(this.matrixobj.getMatrixMultiplication(trans.viewMatrix, this.matrixobj.turn(-this.ToRadial * trans.Rotation)), trans);
        this.setViewMatrix(this.matrixobj.getMatrixMultiplication(trans.viewMatrix, this.matrixobj.tilt(-this.ToRadial * trans.Tilt)), trans);
        this.updatePerspective(trans.PerspectiveAngle, trans);
        trans.needUpdate = true;
        
    },

   updatePerspective:function(angle, trans)
    {
       var width = (((trans.mViewport.Width + trans.mViewport.Height) * Math.tan(this.degreeToRadianConverter((180 - Math.abs(angle % 181)) / 2.0))) + (trans.Depth * 2) / 2);
       trans.Perspective[0][0] = width;
       trans.Perspective[1][1] = width;
       trans.Perspective[2][3] = 1;
       trans.Perspective[3][3] = width;
     
   },

   degreeToRadianConverter:function(degree)
   {
           return degree * Math.PI / 180;
   },
   toScreen:function(vector3D, trans)
      {
       vector3D = this.matrixobj.getMatrixVectorMutiple(this.result(trans), vector3D);
       return { x: vector3D.x, y: vector3D.y };
   },

   setViewMatrix: function (value, trans) {
       if (trans.viewMatrix == value) return;
       trans.viewMatrix = value;
       trans.needUpdate = true;
      
   },

   setCenteredMatrix: function (value, trans) {
       if (trans.viewMatrix == value) return;
       trans.centeredMatrix = value;
       trans.needUpdate = true;
       
   },

   result:function(trans) 
     {
       if (!trans.needUpdate) return trans.resultMatrix;
       trans.resultMatrix = this.matrixobj.getMatrixMultiplication(this.matrixobj.getInvertal(trans.centeredMatrix),
                       trans.Perspective);
       trans.resultMatrix = this.matrixobj.getMatrixMultiplication(trans.resultMatrix, trans.viewMatrix);
       trans.resultMatrix = this.matrixobj.getMatrixMultiplication(trans.resultMatrix, trans.centeredMatrix);
       trans.needUpdate = false;

       return trans.resultMatrix;
    },


   setCenter:function(center, trans)
    {
        trans.centeredMatrix = this.matrixobj.transform(-center.x, -center.y, -center.z);
        trans.needUpdate = true;
   },
   toPlane:function(point, plane, trans)
     {
     var vec1 = this.vector.vector3D(point.x, point.y, 0);
     var vec2 = this.vector.vector3DPlus(vec1, vector.vector3D(0, 0, 1));

     vec1 = this.vector.vector3DMultiply(trans.centeredMatrix , vec1);
     vec2 = this.vector.vector3DMultiply(trans.centeredMatrix , vec2);

     vec1 = this.vector.vector3DMultiply(this.matrixobj.getInvertal(trans.Perspective) , vec1);
     vec2 =  this.vector.vector3DMultiply(this.matrixobj.getInvertal(trans.Perspective) , vec2);

     vec1 = this.polygon.getPoint(vec1, this.vector.vector3DMinus(vec2 - vec1));

     vec1 = this.vector.vector3DMultiply(this.matrixobj.getInvertal(trans.viewMatrix), vec1);
     vec1 =  this.vector.vector3DMultiply(this.matrixobj.getInvertal(trans.centeredMatrix), vec1);

     return vec1;
    }

};
ej.EjSvgRender.chartSymbol =
{
    _drawSeriesType: function (location, symbolStyle, sender) {
        switch (sender.model.series[symbolStyle.SeriesIndex].type.toLowerCase()) {
            case ej.datavisualization.Chart.Type.Line:
                this._drawLine(location, symbolStyle, sender, sender.gLegendItemEle);
                break;
            case ej.datavisualization.Chart.Type.StepLine:
                this._drawStepLine(location, symbolStyle, sender, sender.gLegendItemEle);
                break;
            case ej.datavisualization.Chart.Type.StackingArea:
            case ej.datavisualization.Chart.Type.Area:
                this._drawArea(location, symbolStyle, sender, sender.gLegendItemEle);
                break;
            case ej.datavisualization.Chart.Type.StepArea:
                this._drawStepArea(location, symbolStyle, sender, sender.gLegendItemEle);
                break;
            case ej.datavisualization.Chart.Type.Bar:
            case ej.datavisualization.Chart.Type.StackingBar:
                this._drawBar(location, symbolStyle, sender, sender.gLegendItemEle);
                break;
            case ej.datavisualization.Chart.Type.Pie:
                this._drawPie(location, symbolStyle, sender, sender.gLegendItemEle);
                break;
            case ej.datavisualization.Chart.Type.Doughnut:
                this._drawDoughnut(location, symbolStyle, sender, sender.gLegendItemEle);
                break;
            case ej.datavisualization.Chart.Type.Hilo:
                this._drawHilo(location, symbolStyle, sender, sender.gLegendItemEle);
                break;
            case ej.datavisualization.Chart.Type.HiloOpenClose:
                this._drawHiloOpenClose(location, symbolStyle, sender, sender.gLegendItemEle);
                break;
            case ej.datavisualization.Chart.Type.Candle:
                this._drawCandle(location, symbolStyle, sender, sender.gLegendItemEle);
                break;
            case ej.datavisualization.Chart.Type.Pyramid:
                this._drawPyramid(location, symbolStyle, sender, sender.gLegendItemEle);                
                break;
            case ej.datavisualization.Chart.Type.Funnel:
                this._drawFunnel(location, symbolStyle, sender, sender.gLegendItemEle);
                break;
            case ej.datavisualization.Chart.Type.Spline:
                this._drawSpline(location, symbolStyle, sender, sender.gLegendItemEle);
                break;
            case ej.datavisualization.Chart.Type.SplineArea:
                this._drawSplineArea(location, symbolStyle, sender, sender.gLegendItemEle);
                break;
            case ej.datavisualization.Chart.Type.RangeColumn:
                this._drawRangeColumn(location, symbolStyle, sender, sender.gLegendItemEle);
                break;
            case ej.datavisualization.Chart.Type.Bubble:
            case ej.datavisualization.Chart.Type.Scatter:
                this._drawCircle(location, symbolStyle, sender, sender.gLegendItemEle);
                break;
            case ej.datavisualization.Chart.Type.Column:
            case ej.datavisualization.Chart.Type.StackingColumn:
                this._drawColumn(location, symbolStyle, sender, sender.gLegendItemEle);
                break;
            default:
                this._drawRectangle(location, symbolStyle, sender, sender.gLegendItemEle);
                break;
        }

    },

    _drawCircle: function (location, symbolStyle, sender, element) {
        var svgRender = sender.svgRenderer;
        var svgObj = sender.svgObject;
        var style = symbolStyle.ShapeSize;
        var radius = Math.sqrt(style.height * style.height + style.width * style.width) / 2;
        			
        var options = {
            'id': symbolStyle.ID, 'cx': location.startX, 'cy': location.startY, 'r': radius,
            'fill': symbolStyle.Style.Color, 'stroke-width': symbolStyle.Style.BorderWidth, 'stroke': symbolStyle.Style.BorderColor, 'opacity': symbolStyle.Style.Opacity, 'visibility': symbolStyle.Style.Visibility, 'lgndCtx': symbolStyle.context
        };

        svgRender.drawCircle(options, element);
    },

    _drawLeftArrow: function (location, symbolStyle, sender, element) {
        var svgRender = sender.svgRenderer;
        var svgObj = sender.svgObject;
        var style = symbolStyle.ShapeSize;
        var path;
        path = "M" + " " + ((location.startX - (style.width / 2)) + style.width) + " " + (location.startY + (style.height / 4)) + " " + "L" + " " + ((location.startX - (style.width / 2)) + style.width) + " " + (location.startY + (-style.height / 4)) + " " + "L" + " " + ((location.startX - (style.width / 2)) + (style.width / 2)) + " " + (location.startY + (-style.height / 4)) + " " + "L" + " " + ((location.startX - (style.width / 2)) + (style.width / 2)) + " " + (location.startY + (-style.height / 2)) + " " + "L" + " " + ((location.startX - (style.width / 2))) + " " + (location.startY) + " " + "L" + " " + ((location.startX - (style.width / 2)) + (style.width / 2)) + " " + (location.startY + (style.height / 2)) + " " + "L" + " " + ((location.startX - (style.width / 2)) + (style.width / 2)) + " " + (location.startY + (style.height / 4)) + " " + "L" + " " + ((location.startX - (style.width / 2)) + style.width) + " " + (location.startY + (style.height / 4));
        var options = {
            'id': symbolStyle.ID,
            'fill': symbolStyle.Style.Color,
            'stroke-width': symbolStyle.Style.BorderWidth,
            'stroke': symbolStyle.Style.BorderColor,
            'opacity': symbolStyle.Style.Opacity,
            'visibility':symbolStyle.Style.Visibility,
            'lgndCtx': symbolStyle.context,
            'd': path
        };

        svgRender.drawPath(options, element);

    },


    _drawRightArrow: function (location, symbolStyle, sender, element) {
        var svgRender = sender.svgRenderer;
        var svgObj = sender.svgObject;
        var style = symbolStyle.ShapeSize;
        var path;
        path = "M" + " " + ((location.startX + (style.width / 2)) + style.width) + " " + (location.startY + (style.height / 4)) + " " + "L" + " " + ((location.startX + (style.width / 2)) + style.width) + " " + (location.startY + (-style.height / 4)) + " " + "L" + " " + ((location.startX + (style.width / 2)) + (style.width / 2)) + " " + (location.startY + (-style.height / 4)) + " " + "L" + " " + ((location.startX + (style.width / 2)) + (style.width / 2)) + " " + (location.startY + (-style.height / 2)) + " " + "L" + " " + ((location.startX + (style.width / 2))) + " " + (location.startY) + " " + "L" + " " + ((location.startX + (style.width / 2)) + (style.width / 2)) + " " + (location.startY + (style.height / 2)) + " " + "L" + " " + ((location.startX + (style.width / 2)) + (style.width / 2)) + " " + (location.startY + (style.height / 4)) + " " + "L" + " " + ((location.startX + (style.width / 2)) + style.width) + " " + (location.startY + (style.height / 4));
        var x = ((location.startX + (style.width / 2)));
        var y = location.startY;
        var options = {
            'id': symbolStyle.ID,
            'fill': symbolStyle.Style.Color,
            'stroke-width': symbolStyle.Style.BorderWidth,
            'stroke': symbolStyle.Style.BorderColor,
            'transform': 'rotate(180,' + x + ',' + y + ')',
            'opacity': symbolStyle.Style.Opacity,
            'visibility':symbolStyle.Style.Visibility,
            'lgndCtx': symbolStyle.context,
            'd': path
        };


        svgRender.drawPath(options, element);


    },


    _drawUpArrow: function (location, symbolStyle, sender, element) {
        var svgRender = sender.svgRenderer;
        var svgObj = sender.svgObject;
        var style = symbolStyle.ShapeSize;
        var path;
        path = "M" + " " + ((location.startX) + style.width) + " " + (location.startY - (style.height / 2) + (style.height / 4)) + " " + "L" + " " + ((location.startX) + style.width) + " " + (location.startY - (style.height / 2) + (-style.height / 4)) + " " + "L" + " " + ((location.startX) + (style.width / 2)) + " " + (location.startY - (style.height / 2) + (-style.height / 4)) + " " + "L" + " " + ((location.startX) + (style.width / 2)) + " " + (location.startY - (style.height / 2) + (-style.height / 2)) + " " + "L" + " " + ((location.startX)) + " " + (location.startY - (style.height / 2)) + " " + "L" + " " + ((location.startX) + (style.width / 2)) + " " + (location.startY - (style.height / 2) + (style.height / 2)) + " " + "L" + " " + ((location.startX) + (style.width / 2)) + " " + (location.startY - (style.height / 2) + (style.height / 4)) + " " + "L" + " " + ((location.startX) + style.width) + " " + (location.startY - (style.height / 2) + (style.height / 4));
        var x = ((location.startX));
        var y = (location.startY - (style.height / 2));
        var options = {
            'id': symbolStyle.ID,
            'fill': symbolStyle.Style.Color,
            'stroke-width': symbolStyle.Style.BorderWidth,
            'stroke': symbolStyle.Style.BorderColor,
            'transform': 'rotate(90,' + x + ',' + y + ')',
            'opacity': symbolStyle.Style.Opacity,
            'visibility': symbolStyle.Style.Visibility,
            'lgndCtx': symbolStyle.context,
            'd': path
        };


        svgRender.drawPath(options, element);


    },

    _drawDownArrow: function (location, symbolStyle, sender, element) {
        var svgRender = sender.svgRenderer;
        var svgObj = sender.svgObject;
        var style = symbolStyle.ShapeSize;
        var path;
        path = "M" + " " + ((location.startX) + style.width) + " " + (location.startY + (style.height / 2) + (style.height / 4)) + " " + "L" + " " + ((location.startX) + style.width) + " " + (location.startY + (style.height / 2) + (-style.height / 4)) + " " + "L" + " " + ((location.startX) + (style.width / 2)) + " " + (location.startY + (style.height / 2) + (-style.height / 4)) + " " + "L" + " " + ((location.startX) + (style.width / 2)) + " " + (location.startY + (style.height / 2) + (-style.height / 2)) + " " + "L" + " " + ((location.startX)) + " " + (location.startY + (style.height / 2)) + " " + "L" + " " + ((location.startX) + (style.width / 2)) + " " + (location.startY + (style.height / 2) + (style.height / 2)) + " " + "L" + " " + ((location.startX) + (style.width / 2)) + " " + (location.startY + (style.height / 2) + (style.height / 4)) + " " + "L" + " " + ((location.startX) + style.width) + " " + (location.startY + (style.height / 2) + (style.height / 4));


        var x = ((location.startX));
        var y = (location.startY + (style.height / 2));
        var options = {
            'id': symbolStyle.ID,
            'fill': symbolStyle.Style.Color,
            'stroke-width': symbolStyle.Style.BorderWidth,
            'stroke': symbolStyle.Style.BorderColor,
            'transform': 'rotate(-90,' + x + ',' + y + ')',
            'opacity': symbolStyle.Style.Opacity,
            'visibility': symbolStyle.Style.Visibility,
            'lgndCtx': symbolStyle.context,
            'd': path
        };


        svgRender.drawPath(options, element);

    },

    _drawCross: function (location, symbolStyle, sender, element) {
        var svgRender = sender.svgRenderer;
        var svgObj = sender.svgObject;
        var style = symbolStyle.ShapeSize;

        var path;
        path = "M" + " " + (location.startX + (-style.width / 2)) + " " + (location.startY) + " " + "L" + " "+ (location.startX + (style.width / 2)) + " " + (location.startY) + " " +
        "M" + " " + (location.startX) + " " + (location.startY + (style.height / 2)) + " " + "L" + " " + (location.startX) + " " + (location.startY + (-style.height / 2));
        var options = {
            'id': symbolStyle.ID,
            'opacity': symbolStyle.Style.Opacity,
            'stroke-width': symbolStyle.Style.BorderWidth,
            'stroke': symbolStyle.Style.Color,
            'visibility': symbolStyle.Style.Visibility,
            'lgndCtx': symbolStyle.context,
            'd': path
        };

        svgRender.drawPath(options, element);

    },

    _drawHorizLine: function (location, symbolStyle, sender, element) {
        var svgRender = sender.svgRenderer;
        var svgObj = sender.svgObject;
        var style = symbolStyle.ShapeSize;

        var path;
        path = "M" + " " + (location.startX + (-style.width / 2)) + " " + (location.startY) + " " + "L" + " " + (location.startX + (style.width / 2)) + " " + (location.startY);
        var options = {
            'id': symbolStyle.ID,
            'opacity': symbolStyle.Style.Opacity,
            'stroke-width': symbolStyle.Style.BorderWidth,
            'stroke': symbolStyle.Style.Color,
            'visibility': symbolStyle.Style.Visibility,
            'lgndCtx': symbolStyle.context,
            'd': path
        };

        svgRender.drawPath(options, element);


    },
    _drawVertLine: function (location, symbolStyle, sender, element) {
        var svgRender = sender.svgRenderer;
        var svgObj = sender.svgObject;
        var style = symbolStyle.ShapeSize;
        var path;
        path = "M" + " " + (location.startX) + " " + (location.startY + (style.height / 2)) + " " + "L" + " " + (location.startX) + " " + (location.startY + (-style.height / 2));
        var options = {
            'id': symbolStyle.ID,
            'opacity': symbolStyle.Style.Opacity,
            'stroke-width': symbolStyle.Style.BorderWidth,
            'stroke': symbolStyle.Style.Color,
            'visibility': symbolStyle.Style.Visibility,
            'lgndCtx': symbolStyle.context,
            'd': path
        };


        svgRender.drawPath(options, element);

    },

    _drawTriangle: function (location, symbolStyle, sender, element) {
        var svgRender = sender.svgRenderer;
        var svgObj = sender.svgObject;
        var style = symbolStyle.ShapeSize;
        var path;
        path = "M" + " " + (location.startX + (-style.width / 2)) + " " + (location.startY + (style.height / 2)) + " " + "L" + " " + (location.startX) + " " + (location.startY + (-style.height / 2)) + " " + "L" + " " + (location.startX + (style.width / 2)) + " " + (location.startY + (style.height / 2)) + " " + "L" + " " + (location.startX + (-style.width / 2)) + " " + (location.startY + (style.height / 2));
        var options = {
            'id': symbolStyle.ID,
            'fill': symbolStyle.Style.Color,
            'stroke-width': symbolStyle.Style.BorderWidth,
            'stroke': symbolStyle.Style.BorderColor,
            'opacity': symbolStyle.Style.Opacity,
            'visibility': symbolStyle.Style.Visibility,
            'lgndCtx': symbolStyle.context,
            'd': path
        };

        svgRender.drawPath(options, element);

    },

    _drawInvertedTriangle: function (location, symbolStyle, sender, element) {
        var svgRender = sender.svgRenderer;
        var svgObj = sender.svgObject;
        var style = symbolStyle.ShapeSize;

        var path;
        path = "M" + " " + (location.startX + (-style.width / 2)) + " " + (location.startY + (style.height / 2)) + " " + "L" + " " + (location.startX) + " " + (location.startY + (-style.height / 2)) + " " + "L" + " " + (location.startX + (style.width / 2)) + " " + (location.startY + (style.height / 2)) + " " + "L" + " " + (location.startX + (-style.width / 2)) + " " + (location.startY + (style.height / 2));
        var x = location.startX;
        var y = location.startY;
        var options = {
            'id': symbolStyle.ID,
            'fill': symbolStyle.Style.Color,
            'stroke-width': symbolStyle.Style.BorderWidth,
            'stroke': symbolStyle.Style.BorderColor,
            'transform': 'rotate(180,' + x + ',' + y + ')',
            'opacity': symbolStyle.Style.Opacity,
            'visibility': symbolStyle.Style.Visibility,
            'lgndCtx': symbolStyle.context,
            'd': path
        };

        svgRender.drawPath(options, element);


    },

    _drawHexagon: function (location, symbolStyle, sender, element) {
        var svgRender = sender.svgRenderer;
        var svgObj = sender.svgObject;
        var style = symbolStyle.ShapeSize;

        var path;
        path = "M" + " " + (location.startX + (-style.width / 2)) + " " + (location.startY) + " " + "L" + " " + (location.startX + (-style.width / 4)) + " " + (location.startY + (-style.height / 2)) + " " + "L" + " " + (location.startX + (style.width / 4)) + " " + (location.startY + (-style.height / 2)) + " " + "L" + " " + (location.startX + (style.width / 2)) + " " + (location.startY) + " " + "L" + " " + (location.startX + (style.width / 4)) + " " + (location.startY + (style.height / 2)) + " " + "L" + " " + (location.startX + (-style.width / 4)) + " " + (location.startY + (style.height / 2)) + " " + "L" + " " + (location.startX + (-style.width / 2)) + " " + (location.startY);
        var options = {
            'id': symbolStyle.ID,
            'fill': symbolStyle.Style.Color,
            'stroke-width': symbolStyle.Style.BorderWidth,
            'stroke': symbolStyle.Style.BorderColor,
            'opacity': symbolStyle.Style.Opacity,
            'visibility': symbolStyle.Style.Visibility,
            'lgndCtx': symbolStyle.context,
            'd': path
        };

        svgRender.drawPath(options, element);

    },

    _drawWedge: function (location, symbolStyle, sender, element) {
        var svgRender = sender.svgRenderer;
        var svgObj = sender.svgObject;
        var style = symbolStyle.ShapeSize;

        var path;
        path = "M" + " " + (location.startX + (-style.width / 2)) + " " + (location.startY) + " " + "L" + " " + (location.startX + style.width) + " " + (location.startY + (-style.height / 2)) + " " + "L" + " " + (location.startX + (3 * (style.width / 4))) + " " + (location.startY) + " " + "L" + " " + (location.startX + (style.width)) + " " + (location.startY + (style.height / 2)) + " " + "L" + " " + (location.startX) + " " + (location.startY);
        var options = {
            'id': symbolStyle.ID,
            'fill': symbolStyle.Style.Color,
            'stroke-width': symbolStyle.Style.BorderWidth,
            'stroke': symbolStyle.Style.BorderColor,
            'opacity': symbolStyle.Style.Opacity,
            'visibility': symbolStyle.Style.Visibility,
            'lgndCtx': symbolStyle.context,
            'd': path
        };

        svgRender.drawPath(options, element);

    },

    _drawPentagon: function (location, symbolStyle, sender, element) {
        var svgRender = sender.svgRenderer;
        var svgObj = sender.svgObject;
        var style = symbolStyle.ShapeSize;

        var eq = 72;
        var radius = Math.sqrt(style.height * style.height + style.width * style.width) / 2;
        var sb = ej.EjSvgRender.utils._getStringBuilder();
        for (var i = 0; i <= 5; i++) {
            var deg = i * eq;
            var rad = (Math.PI / 180) * deg;
            var x1 = radius * Math.cos(rad);
            var y1 = radius * Math.sin(rad);
            if (i == 0)
                sb.append("M" + " " + (location.startX + x1) + " " + (location.startY + y1) + " ");

            else
                sb.append("L" + " " + (location.startX + x1) + " " + (location.startY + y1) + " ");


        }
        var path = sb.toString();
        var options = {
            'id': symbolStyle.ID,
            'fill': symbolStyle.Style.Color,
            'stroke-width': symbolStyle.Style.BorderWidth,
            'stroke': symbolStyle.Style.BorderColor,
            'opacity': symbolStyle.Style.Opacity,
            'visibility': symbolStyle.Style.Visibility,
            'lgndCtx': symbolStyle.context,
            'd': path
        };

        svgRender.drawPath(options, element);


    },

    _drawStar: function (location, symbolStyle, sender, element) {
        var svgRender = sender.svgRenderer;
        var style = symbolStyle.ShapeSize;
        var svgObj = sender.svgObject;

        var path;
        path = "M" + " " + (location.startX + (style.width / 3)) + " " + (location.startY + (-style.height / 2)) + " " + "L" + " " + (location.startX + (-style.width / 2)) + " " + (location.startY + (style.height / 6)) + " " + "L" + " " + (location.startX + (style.width / 2)) + " " + (location.startY + (style.height / 6)) + " " + "L" + " " + (location.startX + (-style.width / 3)) + " " + (location.startY + (-style.height / 2)) + " " + "L" + " " + (location.startX) + " " + (location.startY + (style.height / 2)) + " " + "L" + " " + (location.startX + (style.width / 3)) + " " + (location.startY + (-style.height / 2));
        var options = {
            'id': symbolStyle.ID,
            'fill': symbolStyle.Style.Color,
            'stroke-width': symbolStyle.Style.BorderWidth,
            'stroke': symbolStyle.Style.BorderColor,
            'opacity': symbolStyle.Style.Opacity,
            'visibility': symbolStyle.Style.Visibility,
            'lgndCtx': symbolStyle.context,
            'd': path
        };

        svgRender.drawPath(options, element);


    },

    _drawRectangle: function (location, symbolStyle, sender, element) {
        var svgRender = sender.svgRenderer;
        var svgObj = sender.svgObject;
        var style = symbolStyle.ShapeSize;

        var path;
        path = "M" + " " + (location.startX + (-style.width / 2)) + " " + (location.startY + (-style.height / 2)) + " " + "L" + " " + (location.startX + (style.width / 2)) + " " + (location.startY + (-style.height / 2)) + " " + "L" + " " + (location.startX + (style.width / 2)) + " " + (location.startY + (style.height / 2)) + " " + "L" + " " + (location.startX + (-style.width / 2)) + " " + (location.startY + (style.height / 2)) + " " + "L" + " " + (location.startX + (-style.width / 2)) + " " + (location.startY + (-style.height / 2))+"Z";
        var options = {
            'id': symbolStyle.ID,
            'fill': symbolStyle.Style.Color,
            'stroke-width': symbolStyle.Style.BorderWidth,
            'stroke': symbolStyle.Style.BorderColor,
            'opacity': symbolStyle.Style.Opacity,
            'visibility': symbolStyle.Style.Visibility,
            'lgndCtx': symbolStyle.context,
            'd': path
        };

        svgRender.drawPath(options, element);
    },

    _drawTrapezoid: function (location, symbolStyle, sender, element) {
        var svgRender = sender.svgRenderer;
        var svgObj = sender.svgObject;
        var style = symbolStyle.ShapeSize;

        var path;
        path = "M" + " " + (location.startX + (-style.width / 2)) + " " + (location.startY) + " " + "L" + " " + (location.startX + (-style.width / 2)) + " " + (location.startY + (-style.height / 4)) + " " + "L" + " " + (location.startX + (-style.width / 2) + (style.width)) + " " + (location.startY + (-style.height / 2)) + " " + "L" + " " + (location.startX + (-style.width / 2) + (style.width)) + " " + (location.startY + (style.height / 2)) + " " + "L" + " " + (location.startX + (-style.width / 2)) + " " + (location.startY + (style.height / 4)) + " " + "L" + " " + (location.startX + (-style.width / 2)) + " " + (location.startY);
        var options = {
            'id': symbolStyle.ID,
            'fill': symbolStyle.Style.Color,
            'stroke-width': symbolStyle.Style.BorderWidth,
            'stroke': symbolStyle.Style.BorderColor,
            'opacity': symbolStyle.Style.Opacity,
            'visibility': symbolStyle.Style.Visibility,
            'lgndCtx': symbolStyle.context,
            'd': path
        };

        svgRender.drawPath(options, element);

    },

    _drawDiamond: function (location, symbolStyle, sender, element) {
        var svgRender = sender.svgRenderer;
        var svgObj = sender.svgObject;
        var style = symbolStyle.ShapeSize;

        var path;
        path = "M" + " " + (location.startX + (-style.width / 2)) + " " + (location.startY) + " " + "L" + " " + (location.startX) + " " + (location.startY + (-style.height / 2)) + " " + "L" + " " + (location.startX + (style.width / 2)) + " " + (location.startY) + " " + "L" + " " + (location.startX) + " " + (location.startY + (style.height / 2)) + " " + "L" + " " + (location.startX + (-style.width / 2)) + " " + (location.startY);

        var options = {
            'id': symbolStyle.ID,
            'fill': symbolStyle.Style.Color,
            'stroke-width': symbolStyle.Style.BorderWidth,
            'stroke': symbolStyle.Style.BorderColor,
            'opacity': symbolStyle.Style.Opacity,
            'visibility': symbolStyle.Style.Visibility,
            'lgndCtx': symbolStyle.context,
            'd': path
        };

        svgRender.drawPath(options, element);


    },
    _drawEllipse: function (location, symbolStyle, sender, element) {
        var svgRender = sender.svgRenderer;
        var svgObj = sender.svgObject;
        var style = symbolStyle.ShapeSize;

        var x = location.startX;
        var y = location.startY;
        var options = {
            'id': symbolStyle.ID,
            'fill': symbolStyle.Style.Color,
            'stroke-width': symbolStyle.Style.BorderWidth,
            'stroke': symbolStyle.Style.BorderColor,
            'opacity': symbolStyle.Style.Opacity,
            'visibility': symbolStyle.Style.Visibility,
            'lgndCtx': symbolStyle.context,
            'cx': x,
            'cy': y,
            'rx': style.width,
            'ry': style.height / 2
        };


        svgRender.drawEllipse(options, element);


    },
    _drawImage: function (location, symbolStyle, sender, element) {
        var svgRender = sender.svgRenderer;
        var svgObj = sender.svgObject;
        var style = symbolStyle.ShapeSize;
        var x = location.startX + (-style.width / 2);
        var y = location.startY + (-style.width / 2);
        var width = style.width;
        var height = style.height;
        var options = { 'id': svgObj.id + '_image' + symbolStyle.PointIndex, 'height': height, 'width': width, 'href': symbolStyle.Imageurl,
            'x': x, 'y': y, 'visibility': 'visible', 'lgndCtx': symbolStyle.context
        };

        svgRender.drawImage(options, element);


    },
    _drawLine: function (location, symbolStyle, sender, element) {
        var svgRender = sender.svgRenderer;
        var svgObj = sender.svgObject;
        var style = symbolStyle.ShapeSize;
        var elementspace = symbolStyle.ElementSpace;
        var path;        
       
        path = "M" + " " + (location.startX + (-style.width / 2) + (-elementspace / 4)) + " " + (location.startY + (style.height / 10)) + " " + "L" + " " + (location.startX - Math.floor(style.width / 3)) + " " + (location.startY + (style.height / 10)) + " " + "M" + " " + (location.startX + Math.floor(style.width / 3)) + " " + (location.startY + (style.height / 10)) + " " + "L" + " " + (location.startX + (style.width/2) + (elementspace / 4)) + " " + (location.startY + (style.height / 10));
        var options = {
            'id': symbolStyle.ID,
            'fill': symbolStyle.Style.Color,
            'stroke-width': symbolStyle.Style.BorderWidth*2,
            'stroke': symbolStyle.Style.Color,
            'opacity': symbolStyle.Style.Opacity,
            'visibility': symbolStyle.Style.Visibility,
            'd': path,
            'lgndCtx': true
        };
        var circlepath = "M" + " " + (location.startX - Math.floor(style.width / 3)) + " " + (location.startY + (style.height / 10)) + " " + "a " + (Math.floor(style.width / 3)) + " " + (Math.floor(style.width / 3)) + " " + 0 + " " + 1 + " " + 0 + " " + 2 * (Math.floor(style.width / 3)) + " " + 0 + " " + "a" + (Math.floor(style.width / 3)) + " " + (Math.floor(style.width / 3)) + " " + 0 + " " + 1 + " " + 0 + " " + (-2 * (Math.floor(style.width / 3))) + " " + 0;
        var circleoptions = {
            'id': symbolStyle.ID,
            'fill': "transparent",
            'stroke-width': symbolStyle.Style.BorderWidth*2,
            'stroke': symbolStyle.Style.Color,
            'opacity': symbolStyle.Style.Opacity,
            'visibility': symbolStyle.Style.Visibility,
            'd': circlepath,
            'lgndCtx': true
        };

        svgRender.drawPath(options, element);
        svgRender.drawPath(circleoptions, element);

    },
    _drawColumn: function (location, symbolStyle, sender, element) {
        var svgRender = sender.svgRenderer;
        var svgObj = sender.svgObject;
        var style = symbolStyle.ShapeSize;
        var elementspace = symbolStyle.ElementSpace;
        var path;       
         path = "M" + " " + (location.startX - 3 * (style.width / 5)) + " " + (location.startY - (style.height / 5)) + " " + "L" + " " + (location.startX + 3 * (-style.width / 10)) + " " + (location.startY - (style.height / 5)) + " " + "L" + " " + (location.startX + 3 * (-style.width / 10)) + " " + (location.startY + (style.height / 2)) + " " + "L" + " " + (location.startX - 3 * (style.height / 5)) + " " + (location.startY + (style.height / 2)) + " " + "Z" + " " + "M" + " " + (location.startX + (-style.width / 10) - (style.width / 20)) + " " + (location.startY - (style.height / 4) - (elementspace / 2)) + " " + "L" + " " + (location.startX + (style.width / 10) + (style.width / 20)) + " " + (location.startY - (style.height / 4) - (elementspace / 2)) + " " + "L" + " " + (location.startX + (style.width / 10) + (style.width / 20)) + " " + (location.startY + (style.height / 2)) + " " + "L" + " " + (location.startX + (-style.width / 10) - (style.width / 20)) + " " + (location.startY + (style.height / 2)) + " " + "Z" + " " + "M" + " " + (location.startX + 3 * (style.width / 10)) + " " + (location.startY) + " " + "L" + " " + (location.startX + 3 * (style.width / 5)) + " " + (location.startY) + " " + "L" + " " + (location.startX + 3 * (style.width / 5)) + " " + (location.startY + (style.height / 2)) + " " + "L" + " " + (location.startX + 3 * (style.width / 10)) + " " + (location.startY + (style.height / 2)) + " " + "Z";
      
         var options = {
            'id': symbolStyle.ID,
            'fill': symbolStyle.Style.Color,
            'stroke-width': symbolStyle.Style.BorderWidth,
            'stroke': symbolStyle.Style.BorderColor,
            'opacity': symbolStyle.Style.Opacity,
            'visibility': symbolStyle.Style.Visibility,
            'd': path,
             'lgndCtx': true
        };

        svgRender.drawPath(options, element);

    },
    _drawRangeColumn: function (location, symbolStyle, sender, element) {
        var svgRender = sender.svgRenderer;
        var svgObj = sender.svgObject;
        var style = symbolStyle.ShapeSize;
        var elementspace = symbolStyle.ElementSpace;
        var path;       
        path = "M" + " " + (location.startX + (-style.width / 5)) + " " + (location.startY + (style.height / 2) + (elementspace / 4)) + " " + "L" + " " + (location.startX + (-style.width / 5)) + " " + (location.startY - (style.height / 2) - (elementspace / 4)) + " " + "L" + " " + (location.startX + (style.width / 2)) + " " + (location.startY + (-style.height / 2) + (-elementspace / 4)) + " " + "L" + " " + (location.startX + (style.width / 2)) + " " + (location.startY + (style.height / 2) + (elementspace / 4)) + " " + "Z";
       
        var options = {
            'id': symbolStyle.ID,
            'fill': symbolStyle.Style.Color,
            'stroke-width': symbolStyle.Style.BorderWidth,
            'stroke': symbolStyle.Style.BorderColor,
            'opacity': symbolStyle.Style.Opacity,
            'visibility': symbolStyle.Style.Visibility,
            'd': path,
            'lgndCtx': true
        };

        svgRender.drawPath(options, element);

    },

    _drawBar: function (location, symbolStyle, sender, element) {
        var svgRender = sender.svgRenderer;
        var svgObj = sender.svgObject;
        var style = symbolStyle.ShapeSize;
        var elementspace = symbolStyle.ElementSpace;
        var path;        
       
        path = "M" + " " + (location.startX + (-style.width / 2) + (-elementspace / 4)) + " " + (location.startY - 3 * (style.height / 5)) + " " + "L" + " " + (location.startX + 3 * (style.width / 10)) + " " + (location.startY - 3 * (style.height / 5)) + " " + "L" + " " + (location.startX + 3 * (style.width / 10)) + " " + (location.startY - 3 * (style.height / 10)) + " " + "L" + " " + (location.startX - (style.width / 2) + (-elementspace / 4)) + " " + (location.startY - 3 * (style.height / 10)) + " " + "Z" + " " + "M" + " " + (location.startX + (-style.width / 2) + (-elementspace / 4)) + " " + (location.startY - (style.height / 5) + (elementspace / 20)) + " " + "L" + " " + (location.startX + (style.width / 2) + (elementspace / 4)) + " " + (location.startY - (style.height / 5) + (elementspace / 20)) + " " + "L" + " " + (location.startX + (style.width / 2) + (elementspace / 4)) + " " + (location.startY + (style.height / 10) + (elementspace / 20)) + " " + "L" + " " + (location.startX - (style.width / 2) + (-elementspace / 4)) + " " + (location.startY + (style.height / 10) + (elementspace / 20)) + " " + "Z" + " " + "M" + " " + (location.startX - (style.width / 2) + (-elementspace / 4)) + " " + (location.startY + (style.height/5) + (elementspace / 10)) + " " + "L" + " " + (location.startX + (-style.width / 4)) + " " + (location.startY + (style.height/5) + (elementspace / 10)) + " " + "L" + " " + (location.startX + (-style.width / 4)) + " " + (location.startY + (style.height / 2)+(elementspace/10)) + " " + "L" + " " + (location.startX - (style.width / 2) + (-elementspace / 4)) + " " + (location.startY + (style.height / 2)+(elementspace/10)) + " " + "Z";
        var options = {
            'id': symbolStyle.ID,
            'fill': symbolStyle.Style.Color,
            'stroke-width': symbolStyle.Style.BorderWidth,
            'stroke': symbolStyle.Style.BorderColor,
            'opacity': symbolStyle.Style.Opacity,
            'visibility': symbolStyle.Style.Visibility,
            'd': path,
            'lgndCtx': true
        };

        svgRender.drawPath(options, element);

    },
   
    _drawStepLine: function (location, symbolStyle, sender, element) {
        var svgRender = sender.svgRenderer;
        var svgObj = sender.svgObject;
        var style = symbolStyle.ShapeSize;
        var elementspace = symbolStyle.ElementSpace;
        var path;       
       
        path = "M" + " " + (location.startX + (-style.width / 2) - (elementspace/4)) + " " + (location.startY + (style.height / 2)) + " " + "L" + " " + (location.startX + (-style.width / 2) + (style.width / 10)) + " " + (location.startY + (style.height / 2)) + " " + "L" + " " + (location.startX + (-style.width / 2) + (style.width / 10)) + " " + (location.startY) + " " + "L" + " " + (location.startX + (-style.width / 10)) + " " + (location.startY) + " " + "L" + " " + (location.startX + (-style.width / 10)) + " " + (location.startY + (style.height / 2)) + " " + "L" + " " + (location.startX + (style.width / 5)) + " " + (location.startY + (style.height / 2)) + " " + "L" + " " + (location.startX + (style.width / 5)) + " " + (location.startY + (-style.height / 2)) + " " + "L" + " " + (location.startX + (style.width / 2)) + " " + (location.startY + (-style.height / 2)) + "L" + " " + (location.startX + (style.width / 2)) + " " + (location.startY + (style.height / 2)) + " " + "L" + "" + (location.startX + (style.width/2) + (elementspace/4)) + " " + (location.startY + (style.height / 2));
        var options = {
            'id': symbolStyle.ID,
            'fill': "transparent",
            'stroke-width': symbolStyle.Style.BorderWidth,
            'stroke': symbolStyle.Style.Color,
            'opacity': symbolStyle.Style.Opacity,
            'visibility': symbolStyle.Style.Visibility,
            'd': path,
            'lgndCtx': true
        };

        svgRender.drawPath(options, element);

    },
    _drawSpline: function (location, symbolStyle, sender, element) {
        var svgRender = sender.svgRenderer;
        var svgObj = sender.svgObject;
        var style = symbolStyle.ShapeSize;
        var path;        
       
        path = "M" + " " + (location.startX - (style.width / 2)) + " " + (location.startY + (style.height / 5)) + " " + "q" + " " + (style.width/4) + " " + (-style.height) + " " + (style.width/2) + " " + 0 + " " + "M" + " " + location.startX + " " + (location.startY + (style.height / 5)) + " " + "q" + " " + (style.width/4) + " " + (style.height/2) + " " + " " + (style.width/2) + " " + (-style.height/2);
        var options = {
            'id': symbolStyle.ID,
             'fill': "transparent",
            'stroke-width': symbolStyle.Style.BorderWidth*2,
            'stroke': symbolStyle.Style.Color,
            'opacity': symbolStyle.Style.Opacity,
            'visibility': symbolStyle.Style.Visibility,
            'd': path,
            'lgndCtx': true
        };

        svgRender.drawPath(options, element);

    },
    _drawSplineArea: function (location, symbolStyle, sender, element) {
        var svgRender = sender.svgRenderer;
        var svgObj = sender.svgObject;
        var style = symbolStyle.ShapeSize;
        var path;       
       
        path = "M" + " " + (location.startX - (style.width / 2)) + " " + (location.startY + (style.height / 5)) + " " + "q" + " " + (style.width / 4) + " " + (-style.height) + " " + (style.width / 2) + " " + 0 + " " + "Z" + " " + "M" + " " + location.startX + " " + (location.startY + (style.height / 5)) + " " + "q" + " " + (style.width / 4) + " " + (style.height) + " " + " " + (style.width / 2) + " " + 0 + " " + "Z";
        var options = {
            'id': symbolStyle.ID,
            'fill': symbolStyle.Style.Color,
            'stroke-width': symbolStyle.Style.BorderWidth,
            'stroke': symbolStyle.Style.Color,
            'opacity': symbolStyle.Style.Opacity,
            'visibility': symbolStyle.Style.Visibility,
            'd': path,
            'lgndCtx': true
        };

        svgRender.drawPath(options, element);

    },
    _drawArea: function (location, symbolStyle, sender, element) {
        var svgRender = sender.svgRenderer;
        var svgObj = sender.svgObject;
        var style = symbolStyle.ShapeSize;
        var elementspace = symbolStyle.ElementSpace;
        var path;
        path = "M" + " " + (location.startX - (style.width / 2) - (elementspace / 4)) + " " + (location.startY + (style.height / 2)) + " " + "L" + " " + (location.startX + (-style.width / 4) + (-elementspace / 8)) + " " + (location.startY - (style.height / 2)) + " " + "L" + " " + (location.startX) + " " + (location.startY + (style.height / 4)) + " " + "L" + " " + (location.startX + (style.width / 4) + (elementspace / 8)) + " " + (location.startY + (-style.height / 2) + (style.height / 4)) + "L" + " " + (location.startX + (style.height / 2) + (elementspace / 4)) + " " + (location.startY + (style.height / 2)) + " " + "Z";
      
        var options = {
            'id': symbolStyle.ID,
            'fill': symbolStyle.Style.Color,
            'stroke-width': symbolStyle.Style.BorderWidth,
            'stroke': symbolStyle.Style.BorderColor,
            'opacity': symbolStyle.Style.Opacity,
            'visibility': symbolStyle.Style.Visibility,
            'd': path,
            'lgndCtx': true
        };

        svgRender.drawPath(options, element);

    },
    _drawStepArea: function (location, symbolStyle, sender, element) {
        var svgRender = sender.svgRenderer;
        var svgObj = sender.svgObject;
        var style = symbolStyle.ShapeSize;
        var elementspace = symbolStyle.ElementSpace;
        var path;

        path = "M" + " " + (location.startX + (-style.width / 2) + (-elementspace / 4)) + " " + (location.startY + (style.height / 2)) + " " + "L" + " " + (location.startX + (-style.width / 2) + (-elementspace / 4)) + " " + (location.startY + (-style.height / 2)) + " " + "L" + " " + (location.startX - (style.width / 4)) + " " + (location.startY - (style.height / 2)) + " " + "L" + " " + (location.startX - (style.width / 4)) + " " + (location.startY - (style.height / 4)) + " " + "L" + " " + (location.startX + (style.width / 4)) + " " + (location.startY - (style.height / 4)) + " " + "L" + " " + (location.startX + (style.width / 4)) + " " + (location.startX) + " " + "L" + " " + (location.startX + (style.width / 2) + (elementspace / 4)) + " " + (location.startY) + " " + "L" + " " + (location.startX + (style.width / 2) + (elementspace / 4)) + " " + (location.startY + (style.height / 2)) + " " + "Z";       
        var options = {
            'id': symbolStyle.ID,
            'fill': symbolStyle.Style.Color,
            'stroke-width': symbolStyle.Style.BorderWidth,
            'stroke': symbolStyle.Style.BorderColor,
            'opacity': symbolStyle.Style.Opacity,
            'visibility': symbolStyle.Style.Visibility,
            'd': path,
            'lgndCtx': true
        };

        svgRender.drawPath(options, element);



    },
    _drawPyramid: function (location, symbolStyle, sender, element) {
        var svgRender = sender.svgRenderer;
        var svgObj = sender.svgObject;
        var style = symbolStyle.ShapeSize;
        var elementspace = symbolStyle.ElementSpace;
        var path;        
       path = "M" + " " + (location.startX + (-style.width / 2) + (-elementspace / 4)) + " " + (location.startY + (style.height / 2)+(elementspace/8)) + " " + "L" + " " + (location.startX) + " " + (location.startY + (-style.height / 2) + (-elementspace / 8)) + " " + "L" + " " + (location.startX + (style.width / 2) + (elementspace / 4)) + " " + (location.startY + (style.height / 2)+(elementspace/8)) + " " + "Z";
      
        var options = {
            'id': symbolStyle.ID,
            'fill': symbolStyle.Style.Color,
            'stroke-width': symbolStyle.Style.BorderWidth,
            'stroke': symbolStyle.Style.BorderColor,
            'opacity': symbolStyle.Style.Opacity,
            'visibility': symbolStyle.Style.Visibility,
            'd': path,
            'lgndCtx': true
        };

        svgRender.drawPath(options, element);



    },
    _drawFunnel: function (location, symbolStyle, sender, element) {
        var svgRender = sender.svgRenderer;
        var svgObj = sender.svgObject;
        var style = symbolStyle.ShapeSize;
        var elementspace = symbolStyle.ElementSpace;
        var path;
        path = "M" + " " + (location.startX + (-style.width / 2) + (-elementspace / 5)) + " " + (location.startY + (-style.height / 2) + (-elementspace / 4)) + " " + "L" + " " + (location.startX + (-style.width / 5)) + " " + (location.startY + (style.height / 4)) + " " + "L" + " " + (location.startX + (-style.width / 5)) + " " + (location.startY + (style.height / 2) + (elementspace / 4)) + " " + "L" + " " + (location.startX + (style.width / 5)) + " " + (location.startY + (style.height / 2) + (elementspace / 4)) + " " + "L" + " " + (location.startX + (style.width / 5)) + " " + (location.startY + (style.height / 4)) + " " + "L" + " " + (location.startX + (style.width / 2) + (elementspace / 5)) + " " + (location.startY + (-style.height / 2) + (-elementspace / 4)) + " " + "Z";
      
        var options = {
            'id': symbolStyle.ID,
            'fill': symbolStyle.Style.Color,
            'stroke-width': symbolStyle.Style.BorderWidth,
            'stroke': symbolStyle.Style.BorderColor,
            'opacity': symbolStyle.Style.Opacity,
            'visibility': symbolStyle.Style.Visibility,
            'd': path,
            'lgndCtx': true
        };

        svgRender.drawPath(options, element);



    },
    _drawCandle: function (location, symbolStyle, sender, element) {
        var svgRender = sender.svgRenderer;
        var svgObj = sender.svgObject;
        var style = symbolStyle.ShapeSize;
        var elementspace = symbolStyle.ElementSpace;
        var path;       
       
        path = "M" + " " + (location.startX + (-style.width / 2)) + " " + (location.startY + (style.height / 4) + (elementspace / 8)) + " " + "L" + " " + (location.startX + (style.width / 2)) + " " + (location.startY + (style.height / 4) + (elementspace / 8)) + " " + "L" + " " + (location.startX + (style.width / 2)) + " " + (location.startY + (-style.height / 2) + (elementspace / 8)) + " " + "L" + " " + (location.startX + (-style.width / 2)) + " " + (location.startY + (-style.height / 2) + (elementspace / 8)) + " " + "Z" + " " + "M" + " " + (location.startX) + " " + (location.startY + (-style.height / 2) + (elementspace / 8)) + " " + "L" + " " + (location.startX) + " " + (location.startY + (-style.height / 2) + (-elementspace / 4)) + " " + "M" + " " + (location.startX) + " " + (location.startY + (style.height / 4) + (elementspace / 8)) + " " + "L" + " " + (location.startX) + " " + (location.startY + (style.height / 4) + (elementspace / 2));
        var options = {
            'id': symbolStyle.ID,
            'fill': symbolStyle.Style.Color,
            'stroke-width': symbolStyle.Style.BorderWidth,
            'stroke': symbolStyle.Style.Color,
            'opacity': symbolStyle.Style.Opacity,
            'visibility': symbolStyle.Style.Visibility,
            'd': path,
            'lgndCtx': true
        };

        svgRender.drawPath(options, element);



    },
    _drawHilo: function (location, symbolStyle, sender, element) {
        var svgRender = sender.svgRenderer;
        var svgObj = sender.svgObject;
        var style = symbolStyle.ShapeSize;
        var elementspace = symbolStyle.ElementSpace;
        var path;
        path = "M" + " " + (location.startX + (style.width / 2)) + " " + (location.startY + (style.height / 2)+(elementspace/4)) + " " + "L" + " " + (location.startX + (style.width / 2)) + " " + (location.startY + (-style.height / 2)+(-elementspace/4));        
        var options = {
            'id': symbolStyle.ID,
            'fill': symbolStyle.Style.Color,
            'stroke-width': symbolStyle.Style.BorderWidth,
            'stroke': symbolStyle.Style.Color,
            'opacity': symbolStyle.Style.Opacity,
            'visibility': symbolStyle.Style.Visibility,
            'd': path,
            'lgndCtx': true
        };

        svgRender.drawPath(options, element);



    },
    _drawHiloOpenClose: function (location, symbolStyle, sender, element) {
        var svgRender = sender.svgRenderer;
        var svgObj = sender.svgObject;
        var style = symbolStyle.ShapeSize;
        var cartesian = polarToCartesian(18, 12, 4.0710678118654755, 270);
        var elementspace = symbolStyle.ElementSpace;
        var path;        
        
        path = "M" + " " + (location.startX) + " " + (location.startY - 3 * (style.height / 10)) + " " + "L" + " " + (location.startX - (style.width / 2) - (elementspace / 4)) + " " + (location.startY - 3 * (style.height / 10)) + " " + "M" + " " + (location.startX) + " " + (location.startY - (style.height / 2) - (elementspace / 4)) + " " + "L" + " " + (location.startX) + " " + (location.startY + (style.height / 2) + (elementspace / 4)) + " " + "M" + " " + (location.startX) + " " + (location.startY + 3 * (style.height / 10)) + " " + "L" + " " + (location.startX + (style.width / 2) + (elementspace / 4)) + " " + (location.startY + 3 * (style.height / 10));
        var options = {
            'id': symbolStyle.ID,
            'fill': "transparent",
            'stroke-width': symbolStyle.Style.BorderWidth,
            'stroke': symbolStyle.Style.Color,
            'opacity': symbolStyle.Style.Opacity,
            'visibility': symbolStyle.Style.Visibility,
            'd': path,
            'lgndCtx': true
        };

        svgRender.drawPath(options, element);



    },

    _drawDoughnut: function (location, symbolStyle, sender, element) {
        var svgRender = sender.svgRenderer;
        var svgObj = sender.svgObject;
        var style = symbolStyle.ShapeSize;
        var elementspace = symbolStyle.ElementSpace;
        var radius = Math.sqrt(style.height * style.width) / 2;
        radius = radius + (elementspace / 5);
        var cartesianlarge = polarToCartesian(location.startX, location.startY, radius, 270);
        var cartesiansmall = polarToCartesian(location.startX + (style.width / 10), location.startY, radius, 270);
        var elementspace = symbolStyle.ElementSpace;
        path = "M" + " " + location.startX + " " + location.startY + " " + "L" + " " + (location.startX + radius) + " " + (location.startY) + " " + "A" + " " + (radius) + " " + (radius) + " " + 0 + " " + 1 + " " + 1 + " " + cartesianlarge[0] + " " + cartesianlarge[1] + " " + "Z" + " " + "M" + " " + (location.startX + (style.width / 10)) + " " + (location.startY - (style.height / 10)) + " " + "L" + (location.startX + (radius)) + " " + (location.startY - style.height / 10) + " " + "A" + " " + (radius) + " " + (radius) + " " + 0 + " " + 0 + " " + 0 + " " + cartesiansmall[0] + " " + cartesiansmall[1] + " " + "Z";

        var options = {
            'id': symbolStyle.ID,
            'fill': symbolStyle.Style.Color,
            'stroke-width': symbolStyle.Style.BorderWidth,
            'stroke': symbolStyle.Style.BorderColor,
            'opacity': symbolStyle.Style.Opacity,
            'visibility': symbolStyle.Style.Visibility,
            'd': path,
            'lgndCtx': true
        };
        var circle = {
            'id': symbolStyle.ID, 'cx': location.startX, 'cy': location.startY, 'r': (radius/2),'lgndCtx': true,
            'fill': "white", 'stroke-width': symbolStyle.Style.BorderWidth, 'stroke': symbolStyle.Style.BorderColor, 'opacity': symbolStyle.Style.Opacity, 'visibility': symbolStyle.Style.Visibility
        };

        svgRender.drawPath(options, element);
        svgRender.drawCircle(circle, element);
    },

    _drawPie: function (location, symbolStyle, sender, element) {
        var svgRender = sender.svgRenderer;
        var svgObj = sender.svgObject;
        var style = symbolStyle.ShapeSize;
        var elementspace = symbolStyle.ElementSpace;
        var radius = Math.sqrt(style.height * style.width) / 2;
        radius = radius + (elementspace / 5);
        var cartesianlarge = polarToCartesian(location.startX, location.startY, radius, 270);
        var cartesiansmall = polarToCartesian(location.startX + (style.width / 10), location.startY, radius, 270);
        var elementspace = symbolStyle.ElementSpace;
        path = "M" + " " + location.startX + " " + location.startY + " " + "L" + " " + (location.startX + radius) + " " + (location.startY) + " " + "A" + " " + (radius) + " " + (radius) + " " + 0 + " " + 1 + " " + 1 + " " + cartesianlarge[0] + " " + cartesianlarge[1] + " " + "Z" + " " + "M" + " " + (location.startX + (style.width / 10)) + " " + (location.startY - (style.height / 10)) + " " + "L" + (location.startX + (radius)) + " " + (location.startY - style.height / 10) + " " + "A" + " " + (radius) + " " + (radius) + " " + 0 + " " + 0 + " " + 0 + " " + cartesiansmall[0] + " " + cartesiansmall[1] + " " + "Z";

        var options = {
            'id': symbolStyle.ID,
            'fill': symbolStyle.Style.Color,
            'stroke-width': symbolStyle.Style.BorderWidth,
            'stroke': symbolStyle.Style.BorderColor,
            'opacity': symbolStyle.Style.Opacity,
            'visibility': symbolStyle.Style.Visibility,
            'd': path,
            'lgndCtx': true
        };

        svgRender.drawPath(options, element);
    }

};
function polarToCartesian(startX, startY, radius, angleInDegrees) {
    var angleInRadians = angleInDegrees * Math.PI / 180.0;
    var x = startX + radius * Math.cos(angleInRadians);
    var y = startY + radius * Math.sin(angleInRadians);
    return [x, y];
}
ej.EjSvgRender.seriesPalette = {
    defaultMetro: ["#E94649", "#F6B53F", "#6FAAB0", "#C4C24A", "#FB954F", "#005277", "#8BC652", "#69D2E7", "#E27F2D", "#6A4B82"],
    defaultGradient:
        {
            borderColors: ["#F34649", "#F6D321", "#6EB9B0", "#CBC26A", "#FBAF4F", "#E2CDB1", "#FFC0B7", "#68E1E6", "#E1A62D", "#9C6EBF"],
            seriesColors: [[{ color: "#F34649", colorStop: "0%" }, { color: "#B74143", colorStop: "100%"}],
     [{ color: "#F6D321", colorStop: "0%" }, { color: "#F6AE26", colorStop: "100%"}],
     [{ color: "#6EB9B0", colorStop: "0%" }, { color: "#3F77BD", colorStop: "100%"}],
     [{ color: "#CBC26A", colorStop: "0%" }, { color: "#9AAD21", colorStop: "100%"}],
     [{ color: "#FBAF4F", colorStop: "0%" }, { color: "#F07542", colorStop: "100%"}],
     [{ color: "#E2CDB1", colorStop: "0%" }, { color: "#AAA089", colorStop: "100%"}],
     [{ color: "#8BC652", colorStop: "0%" }, { color: "#6F9E41", colorStop: "100%"}],
     [{ color: "#68E1E6", colorStop: "0%" }, { color: "#3D9CBE", colorStop: "100%"}],
     [{ color: "#E1A62D", colorStop: "0%" }, { color: "#B66824", colorStop: "100%"}],
     [{ color: "#9C6EBF", colorStop: "0%"}], [{ color: "#593F6D", colorStop: "100%"}]]
        },
    blueMetro: ["#005378", "#006691", "#007EB5", "#0D97D4", "#00AEFF", "#14B9FF", "#54CCFF", "#87DBFF", "#ADE5FF", "#C5EDFF"],
    blueGradient:
     {
         seriesColors: [[{ color: "#005277", colorStop: "0%" }, { color: "#00304F", colorStop: "100%"}],
            [{ color: "#006590", colorStop: "0%" }, { color: "#004068", colorStop: "100%"}],
            [{ color: "#007DB4", colorStop: "0%" }, { color: "#00558B", colorStop: "100%"}],
            [{ color: "#0D97D4", colorStop: "0%" }, { color: "#057FC7", colorStop: "100%"}],
            [{ color: "#00ADFE", colorStop: "0%" }, { color: "#008BE9", colorStop: "100%"}],
            [{ color: "#14B8FE", colorStop: "0%" }, { color: "#0798EB", colorStop: "100%"}],
            [{ color: "#53CBFF", colorStop: "0%" }, { color: "#35AFEB", colorStop: "100%"}],
            [{ color: "#86DAFF", colorStop: "0%" }, { color: "#64C0EC", colorStop: "100%"}],
            [{ color: "#ACE5FF", colorStop: "0%" }, { color: "#8DCEED", colorStop: "100%"}],
            [{ color: "#C4ECFF", colorStop: "0%"}], [{ color: "#A3D1E6", colorStop: "100%"}]],
         borderColors: ["#005277", "#006590", "#007DB4", "#0D97D4", "#00ADFE", "#14B8FE", "#53CBFF", "#86DAFF", "#ACE5FF", "#C4ECFF"]
     },
    greenMetro: ["#496612", "#597B15", "#709A1B", "#87B62A", "#9AD926", "#A6DC37", "#BCE654", "#C8E780", "#D5EFA5", "#E2F3BE"],
    greenGradient:
     {

         seriesColors: [[{ color: "#5C7F16", colorStop: "0%" }, { color: "#384C08", colorStop: "100%"}],
            [{ color: "#6A9319", colorStop: "0%" }, { color: "#486009", colorStop: "100%"}],
            [{ color: "#739D1C", colorStop: "0%" }, { color: "#57760B", colorStop: "100%"}],
            [{ color: "#90B546", colorStop: "0%" }, { color: "#6E9215", colorStop: "100%"}],
            [{ color: "#9AD826", colorStop: "0%" }, { color: "#75A010", colorStop: "100%"}],
            [{ color: "#A5DB36", colorStop: "0%" }, { color: "#8EB91D", colorStop: "100%"}],
            [{ color: "#BBE554", colorStop: "0%" }, { color: "#A4C849", colorStop: "100%"}],
            [{ color: "#C8E780", colorStop: "0%" }, { color: "#B4D072", colorStop: "100%"}],
            [{ color: "#D4EEA5", colorStop: "0%" }, { color: "#BFD593", colorStop: "100%"}],
            [{ color: "#E1F2BD", colorStop: "0%"}], [{ color: "#C8D7A8", colorStop: "100%"}]],
         borderColors: ["#5C7F16", "#6A9319", "#739D1C", "#90B546", "#9AD826", "#A5DB36", "#BBE554", "#C8E780", "#D4EEA5", "#E1F2BD"]
     },

    sandleMetro: ["#6C450C", "#82520D", "#A36812", "#C07F1F", "#E69719", "#E89A2B", "#EEB564", "#F3CB93", "#F7DEB4", "#F9E6CA"],
    sandleGradient:
     {


         seriesColors: [[{ color: "#7F602F", colorStop: "0%" }, { color: "#512D04", colorStop: "100%"}],
            [{ color: "#986827", colorStop: "0%" }, { color: "#673803", colorStop: "100%"}],
            [{ color: "#A16C1F", colorStop: "0%" }, { color: "#8A4B05", colorStop: "100%"}],
            [{ color: "#BF812A", colorStop: "0%" }, { color: "#AD630D", colorStop: "100%"}],
            [{ color: "#E49519", colorStop: "0%" }, { color: "#B86607", colorStop: "100%"}],
            [{ color: "#E7992A", colorStop: "0%" }, { color: "#D7780D", colorStop: "100%"}],
            [{ color: "#EDB463", colorStop: "0%" }, { color: "#D98F31", colorStop: "100%"}],
            [{ color: "#F2CA92", colorStop: "0%" }, { color: "#DAAC6F", colorStop: "100%"}],
            [{ color: "#F6DDB3", colorStop: "0%" }, { color: "#DABE8F", colorStop: "100%"}],
            [{ color: "#F8E5C9", colorStop: "0%"}], [{ color: "#DDBE92", colorStop: "100%"}]],
         borderColors: ["#7F602F", "#986827", "#A16C1F", "#BF812A", "#E49519", "#E7992A", "#EDB463", "#F2CA92", "#F6DDB3", "#F8E5C9"]

     }
};

ej.EjSvgRender.themes = {
    flatlight:
    {
        background: 'transparent',
        legend:
        {
            font: { color: "#282828"},
            title:
            {
               font: { color: "#282828"}
             }
        },
        title:
        {
            font: { color: '#565656'},
            subTitle:
                {
                    font: { color: '#565656'}
                }
        },
        primaryXAxis:
        {
            majorGridLines:
            {
                color: "#DFDFDF"
            },
            majorTickLines:
            {
                color: "#8E8E8E"
            },
            minorGridLines:
            {
                color: "#DFDFDF"
            },
            minorTickLines:
            {
                color: "#8E8E8E"
            },
            axisLine: { color: '#8E8E8E' },
            font: { color: '#282828' },
            title:
            {
                font: { color: '#282828' }
            },
            crosshairLabel: { rx: 0, ry: 0, border: { color: '#3D3D3D', width: 1 }, fill: '#3D3D3D', font: { fontFamily: 'Segoe UI', fontStyle: 'Normal', size: '13px', fontWeight: 'regular', opacity: 1, color: '#DBDBDB' } }
        },
        secondaryX:
        {
            majorGridLines:
            {
                color: "#DFDFDF"
            },
            majorTickLines:
            {
                color: "#8E8E8E"
            },
            minorGridLines:
            {
                color: "#DFDFDF"
            },
            minorTickLines:
            {
                color: "#8E8E8E"
            },
            axisLine: { color: '#8E8E8E' },
            font: { color: '#282828' },
            title:
            {
                font: { color: '#282828' }
            },
            crosshairLabel: { rx: 0, ry: 0, border: { color: '#3D3D3D' ,width: 1 }, fill: '#3D3D3D', font: { fontFamily: 'Segoe UI', fontStyle: 'Normal', size: '13px', fontWeight: 'regular', opacity: 1, color: '#DBDBDB'} }
        },

        primaryYAxis:
        {
            majorGridLines:
            {
                color: "#DFDFDF"
            },
            majorTickLines:
            {
                color: "#8E8E8E"
            },
            minorGridLines:
            {
                color: "#DFDFDF"
            },
            minorTickLines:
            {
                color: "#8E8E8E"
            },
            axisLine: { color: '#8E8E8E' },
            font: { color: '#282828' },
            title:
            {
                font: { color: '#282828' }
            },
            crosshairLabel: { rx: 0, ry: 0, border: { color: '#3D3D3D', borderWidth: 1 }, fill: '#3D3D3D', font: { fontFamily: 'Segoe UI', fontStyle: 'Normal', size: '13px', fontWeight: 'regular', opacity: 1, color: '#DBDBDB' } }
        },
        secondaryY:
        {
            majorGridLines:
            {
                color: "#DFDFDF"
            },
            majorTickLines:
            {
                color: "#8E8E8E"
            },
            minorGridLines:
            {
                color: "#DFDFDF"
            },
            minorTickLines:
            {
                color: "#8E8E8E"
            },
            axisLine: { color: '#8E8E8E' },
            font: { color: '#282828' },
            title:
            {
                font: { color: '#282828' }
            },
            crosshairLabel: { rx: 0, ry: 0, border: { color: '#3D3D3D', width: 1 }, fill: '#3D3D3D', font: { fontFamily: 'Segoe UI', fontStyle: 'Normal', size: '13px', fontWeight: 'regular', opacity: 1, color: '#DBDBDB' } }
        },
        crosshair:
        {
            line:
            {
                width: 1,
                color: 'Black'
            }
        },
        
        seriesBorderDefaultColors: ej.EjSvgRender.seriesPalette.defaultMetro,
        colors: ej.EjSvgRender.seriesPalette.defaultMetro

    },

    flatdark:
        {
            background: '#111111',

            legend:
            {
                font: { color: "#C9C9C9"},
                title:
                {
                  font: { color: "#C9C9C9"}
                }
            },
            title:
            {
                font: { color: '#C9C9C9'},
                subTitle:
                {
                    font: { color: '#C9C9C9'}
                }
            },
            primaryXAxis:
             {
                 majorGridLines:
                 {
                     color: "#333333"
                 },
                 majorTickLines:
                  {
                      color: "#AAAAAA"
                  },
                 minorGridLines:
                  {
                      color: "#333333"
                  },
                 minorTickLines:
                  {
                      color: "#AAAAAA"
                  },
                 axisLine: { color: '#AAAAAA' },
                 font: { color: '#C9C9C9' },
                 title:
                 {

                     font: { color: '#C9C9C9' }
                 },
                 crosshairLabel: { rx: 0, ry: 0, fill: '#B5B5B5', border: { color: '#B5B5B5', width: 1 }, font: { fontFamily: 'Segoe UI', fontStyle: 'Normal', size: '13px', fontWeight: 'regular', opacity: 1, color: '#444444' } }
             },
            secondaryX:
             {
                 majorGridLines:
                 {
                     color: "#333333"
                 }, 
                 majorTickLines:
                  {
                      color: "#AAAAAA"
                  },
                 minorGridLines:
                  {
                      color: "#333333"
                  },
                 minorTickLines:
                  {
                      color: "#AAAAAA"
                  },
                 axisLine: { color: '#AAAAAA' },
                 font: { color: '#C9C9C9' },
                 title:
                 {

                     font: { color: '#C9C9C9' }
                 },
                 crosshairLabel: {   rx: 0, ry: 0, fill: '#B5B5B5', border:{color: '#B5B5B5', width: 1 }, font: { fontFamily: 'Segoe UI', fontStyle: 'Normal', size: '13px', fontWeight: 'regular', opacity: 1, color: '#444444' } }
             },
            primaryYAxis:
             {
                 majorGridLines:
                 {
                     color: "#333333"
                 },
                 majorTickLines:
                  {
                      color: "#AAAAAA"
                  },
                 minorGridLines:
                  {
                      color: "#333333"
                  },
                 minorTickLines:
                  {
                      color: "#AAAAAA"
                  },
                 axisLine: { color: '#AAAAAA' },
                 font: { color: '#C9C9C9' },
                 title:
                 {

                     font: { color: '#C9C9C9' }
                 },
                 crosshairLabel: { rx: 0, ry: 0, fill: '#B5B5B5', border:{color: '#B5B5B5', width: 1 }, font: { fontFamily: 'Segoe UI', fontStyle: 'Normal', size: '13px', fontWeight: 'regular', opacity: 1, color: '#444444' } }
             },
            secondaryY:
             {
                 majorGridLines:
                 {
                     color: "#333333"
                 },
                 majorTickLines:
                  {
                      color: "#AAAAAA"
                  },
                 minorGridLines:
                  {
                      color: "#333333"
                  },
                 minorTickLines:
                  {
                      color: "#AAAAAA"
                  },
                 axisLine: { color: '#AAAAAA' },
                 font: { color: '#C9C9C9' },
                 title:
                 {

                     font: { color: '#C9C9C9' }
                 },
                 crosshairLabel: { rx: 0, ry: 0, fill: '#B5B5B5', border: { color: '#B5B5B5', width: 1 }, font: { fontFamily: 'Segoe UI', fontStyle: 'Normal', size: '13px', fontWeight: 'regular', opacity: 1, color: '#444444'} }
             },
            crosshair:
             {
                 line:
                 {
                     width: 1,
                     color: 'White'
                 }
             },
            
            seriesBorderDefaultColors: ej.EjSvgRender.seriesPalette.defaultMetro,
            colors: ej.EjSvgRender.seriesPalette.defaultMetro

        },


    gradientlight:
        {
            background: 'transparent',
            legend:
            {
                font: { color: "#282828"},
                title:
                {
                    font: { color: "#282828"}
                }
            },
            title:
            {
                font: { color: '#565656'},
                subTitle:
                {
                    font: { color: '#565656'}
                }
            },
            primaryXAxis:
             {
                 majorGridLines:
                 {
                     color: "#DFDFDF"
                 },
                 majorTickLines:
                  {
                      color: "#8E8E8E"
                  },
                 minorGridLines:
                  {
                      color: "#DFDFDF"
                  },
                 minorTickLines:
                  {
                      color: "#8E8E8E"
                  },
                 axisLine: { color: '#8E8E8E' },
                 font: { color: '#282828' },
                 title:
                 {

                     font: { color: '#282828' }
                 },
                 crosshairLabel: {rx: 3, ry: 3, fill: '#3D3D3D',border:{color: '#3D3D3D',  width: 2 }, font: { fontFamily: 'Segoe UI', fontStyle: 'Normal', size: '13px', fontWeight: 'regular', opacity: 1, color: '#DBDBDB'} }
             },
            secondaryX:
             {
                 majorGridLines:
                 {
                     color: "#DFDFDF"
                 },
                 majorTickLines:
                  {
                      color: "#8E8E8E"
                  },
                 minorGridLines:
                  {
                      color: "#DFDFDF"
                  },
                 minorTickLines:
                  {
                      color: "#8E8E8E"
                  },
                 axisLine: { color: '#8E8E8E' },
                 font: { color: '#282828' },
                 title:
                 {

                     font: { color: '#282828' }
                 },
                 crosshairLabel: {rx: 3, ry: 3, fill: '#3D3D3D',border:{color: '#3D3D3D',  width: 2 }, font: { fontFamily: 'Segoe UI', fontStyle: 'Normal', size: '13px', fontWeight: 'regular', opacity: 1, color: '#DBDBDB'} }
             },
            primaryYAxis:
             {
                 majorGridLines:
                 {
                     color: "#DFDFDF"
                 },
                 majorTickLines:
                  {
                      color: "#8E8E8E"
                  },
                 minorGridLines:
                  {
                      color: "#DFDFDF"
                  },
                 minorTickLines:
                  {
                      color: "#8E8E8E"
                  },
                 axisLine: { color: '#8E8E8E' },
                 font: { color: '#282828' },
                 title:
                 {

                     font: { color: '#282828' }
                 },
                 crosshairLabel: {rx: 3, ry: 3, fill: '#3D3D3D',border:{color: '#3D3D3D',  width: 2 }, font: { fontFamily: 'Segoe UI', fontStyle: 'Normal', size: '13px', fontWeight: 'regular', opacity: 1, color: '#DBDBDB'} }
             },
            secondaryY:
             {
                 majorGridLines:
                 {
                     color: "#DFDFDF"
                 },
                 majorTickLines:
                  {
                      color: "#8E8E8E"
                  },
                 minorGridLines:
                  {
                      color: "#DFDFDF"
                  },
                 minorTickLines:
                  {
                      color: "#8E8E8E"
                  },
                 axisLine: { color: '#8E8E8E' },
                 font: { color: '#282828' },
                 title:
                 {

                     font: { color: '#282828' }
                 },
                 crosshairLabel: {rx: 3, ry: 3, fill: '#3D3D3D',border:{color: '#3D3D3D',  width: 2 }, font: { fontFamily: 'Segoe UI', fontStyle: 'Normal', size: '13px', fontWeight: 'regular', opacity: 1, color: '#DBDBDB'} }
             },
            crosshair:
             {
                 line:
                 {
                     width: 1,
                     color: 'Black'
                 }
             },
            
            seriesBorderDefaultColors: ej.EjSvgRender.seriesPalette.defaultGradient.borderColors,
            colors: ej.EjSvgRender.seriesPalette.defaultGradient.seriesColors

        },

    gradientdark:
        {
            background: '#111111',
            legend:
            {
                font: { color: "#C9C9C9"},
                title:
               {
                   font: { color: "#C9C9C9"}
               }
            },
            title:
            {
                font: { color: '#C9C9C9'},
                subTitle:
                {
                    font: { color: '#C9C9C9'}
                }
            },
            primaryXAxis:
             {
                 majorGridLines:
                 {
                     color: "#333333"
                 },
                 majorTickLines:
                  {
                      color: "#AAAAAA"
                  },
                 minorGridLines:
                  {
                      color: "#333333"
                  },
                 minorTickLines:
                  {
                      color: "#AAAAAA"
                  },
                 axisLine: { color: '#AAAAAA' },
                 font: { color: '#C9C9C9' },
                 title:
                 {

                     font: { color: '#C9C9C9' }
                 },
                 crosshairLabel: {rx: 3, ry: 3, fill: '#B5B5B5',border:{color: '#B5B5B5',  width: 2 }, font: { fontFamily: 'Segoe UI', fontStyle: 'Normal', size: '13px', fontWeight: 'regular', opacity: 1, color: '#444444'} }
             },
            secondaryX:
             {
                 majorGridLines:
                 {
                     color: "#333333"
                 },
                 majorTickLines:
                  {
                      color: "#AAAAAA"
                  },
                 minorGridLines:
                  {
                      color: "#333333"
                  },
                 minorTickLines:
                  {
                      color: "#AAAAAA"
                  },
                 axisLine: { color: '#AAAAAA' },
                 font: { color: '#C9C9C9' },
                 title:
                 {

                     font: { color: '#C9C9C9' }
                 },
                 crosshairLabel: {rx: 3, ry: 3, fill: '#B5B5B5',border:{color: '#B5B5B5',  width: 2 }, font: { fontFamily: 'Segoe UI', fontStyle: 'Normal', size: '13px', fontWeight: 'regular', opacity: 1, color: '#444444'} }
             },
            primaryYAxis:
             {
                 majorGridLines:
                 {
                     color: "#333333"
                 },
                 majorTickLines:
                  {
                      color: "#AAAAAA"
                  },
                 minorGridLines:
                  {
                      color: "#333333"
                  },
                 minorTickLines:
                  {
                      color: "#AAAAAA"
                  },
                 axisLine: { color: '#AAAAAA' },
                 font: { color: '#C9C9C9' },
                 title:
                 {

                     font: { color: '#C9C9C9' }
                 },
                 crosshairLabel: {rx: 3, ry: 3, fill: '#B5B5B5',border:{color: '#B5B5B5',  width: 2 }, font: { fontFamily: 'Segoe UI', fontStyle: 'Normal', size: '13px', fontWeight: 'regular', opacity: 1, color: '#444444'} }
             },
            secondaryY:
             {
                 majorGridLines:
                 {
                     color: "#333333"
                 },
                 majorTickLines:
                  {
                      color: "#AAAAAA"
                  },
                 minorGridLines:
                  {
                      color: "#333333"
                  },
                 minorTickLines:
                  {
                      color: "#AAAAAA"
                  },
                 axisLine: { color: '#AAAAAA' },
                 font: { color: '#C9C9C9' },
                 title:
                 {

                     font: { color: '#C9C9C9' }
                 },
                 crosshairLabel: {rx: 3, ry: 3, fill: '#B5B5B5',border:{color: '#B5B5B5',  width: 2 }, font: { fontFamily: 'Segoe UI', fontStyle: 'Normal', size: '13px', fontWeight: 'regular', opacity: 1, color: '#444444'} }
             },
            crosshair:
             {
                 line:
                 {
                     width: 1,
                     color: 'White'
                 }
             },
             
            seriesBorderDefaultColors: ej.EjSvgRender.seriesPalette.defaultGradient.borderColors,
            colors: ej.EjSvgRender.seriesPalette.defaultGradient.seriesColors

        }
};
$.extend(ej.EjSvgRender.themes, {

    "azure":
    {

        seriesBorderDefaultColors: ej.EjSvgRender.seriesPalette.blueMetro,
        colors: ej.EjSvgRender.seriesPalette.blueMetro
    },

    "azuredark":
    {
        seriesBorderDefaultColors: ej.EjSvgRender.seriesPalette.blueMetro,
        colors: ej.EjSvgRender.seriesPalette.blueMetro
    },
    "gradient-azure":
    {
        seriesBorderDefaultColors: ej.EjSvgRender.seriesPalette.blueGradient.borderColors,
        colors: ej.EjSvgRender.seriesPalette.blueGradient.seriesColors
    },

    "gradient-azuredark":
    {
        seriesBorderDefaultColors: ej.EjSvgRender.seriesPalette.blueGradient.borderColors,
        colors: ej.EjSvgRender.seriesPalette.blueGradient.seriesColors
    },

    "lime":
    {
        seriesBorderDefaultColors: ej.EjSvgRender.seriesPalette.greenMetro,
        colors: ej.EjSvgRender.seriesPalette.greenMetro
    },

    "limedark":
    {
        seriesBorderDefaultColors: ej.EjSvgRender.seriesPalette.greenMetro,
        colors: ej.EjSvgRender.seriesPalette.greenMetro
    },
    "gradient-lime":
    {
        seriesBorderDefaultColors: ej.EjSvgRender.seriesPalette.greenGradient.borderColors,
        colors: ej.EjSvgRender.seriesPalette.greenGradient.seriesColors
    },

    "gradient-limedark":
    {
        seriesBorderDefaultColors: ej.EjSvgRender.seriesPalette.greenGradient.borderColors,
        colors: ej.EjSvgRender.seriesPalette.greenGradient.seriesColors
    },
    "saffron":
    {
        seriesBorderDefaultColors: ej.EjSvgRender.seriesPalette.sandleMetro,
        colors: ej.EjSvgRender.seriesPalette.sandleMetro
    },

    "saffrondark":
    {
        seriesBorderDefaultColors: ej.EjSvgRender.seriesPalette.sandleMetro,
        colors: ej.EjSvgRender.seriesPalette.sandleMetro
    },
    "gradient-saffron":
{
    seriesBorderDefaultColors: ej.EjSvgRender.seriesPalette.sandleGradient.borderColors,
    colors: ej.EjSvgRender.seriesPalette.sandleGradient.seriesColors
},

    "gradient-saffrondark":
{
    seriesBorderDefaultColors: ej.EjSvgRender.seriesPalette.sandleGradient.borderColors,
    colors: ej.EjSvgRender.seriesPalette.sandleGradient.seriesColors
}
});
})(jQuery);;
ej.axisTypes = {};

ej.EjAxisRenderer = function () {
};
var _sideBySeriesPadding = function (chartObj, start, end, axis) {
    var visibleSeries = ej.DataManager(chartObj.model._visibleSeries, ej.Query().sortBy("_zOrder")).executeLocal();
    var isHorizontal = false;
    var isVertical = false;
    var data, startPadding, endPadding;
    for (var i = 0; i < visibleSeries.length; i++) {
        var type = visibleSeries[i].type.toLowerCase();
        if (axis.orientation.toLowerCase()=="horizontal" && (type == "column" || type == "rangecolumn" || type == "stackingcolumn" || type == "candle" || type == "hilo" || type == "hiloopenclose")) {
            isHorizontal = true;
            break;
        } else
            if (axis.orientation.toLowerCase() == "vertical" && (type == "bar" || type == "stackingbar")) {
                isVertical = true;
                break;
        }
    }
    if ((isHorizontal && axis.orientation.toLowerCase() == 'horizontal') || (isVertical && axis.orientation.toLowerCase() == 'vertical')) {
        axis.m_minPointsDelta = undefined;
        var padding = ej.EjSvgRender.utils.getMinPointsDelta(axis, chartObj, start) * 0.5;
        start = start - padding;
        end = end + padding;
		axis.padding = padding;
    }
    
    return data = { startPadding: start, endPadding: end };
};
ej.EjStripline = function (chartobj) {
    this.chart = chartobj;
};
(function ($) {
    ej.EjAxisRenderer.prototype = {

        //Calculate min/max value for series
        _seriesMinMax: function (chartObj, axis, seriesCollection, type) {
            if ((seriesCollection.length > 0 && seriesCollection[0].points.length > 0) || this.chartObj.model.indicators[0].points.length>0) {

                var maxX, minY, minX, maxY;
                var maxMinXValue = { maxX: null, minX: null };
                var maxMinYValue = { maxY: null, minY: null };

                for (var i = 0; i < seriesCollection.length; i++) {
                    if (seriesCollection[i].visibility.toLowerCase() === 'visible') {
                        seriesCollection[i]._visiblePoints = [];
                        for (var j = 0; j < seriesCollection[i].points.length; j++) {
                            var point = seriesCollection[i].points[j];
                            if (!ej.util.isNullOrUndefined(point.x) && !ej.util.isNullOrUndefined(point.y) || point.isEmpty == false)
                                seriesCollection[i]._visiblePoints.push(point);
                        }
                    }
                }

                for (var i = 0; i < seriesCollection.length; i++) {
                   
                        if (seriesCollection[i]._visiblePoints.length > 0) {
                            if (seriesCollection[i].visibility.toLowerCase() === 'visible' && seriesCollection[i].points.length > 0) {
                                minX = maxX = seriesCollection[i]._visiblePoints[0].X;
                                maxY = minY = seriesCollection[i]._visiblePoints[0].YValues[0];
                                for (var j = 0; j < seriesCollection[i]._visiblePoints.length; j++) {
                                    var currentPoint = seriesCollection[i]._visiblePoints[j];
                                    if (minX > currentPoint.X)
                                        minX = currentPoint.X;
                                    if (maxX < currentPoint.X)
                                        maxX = currentPoint.X;
                                    if (seriesCollection[i]._hiloTypes) {
                                        for (var y = 0; y < 2; y++) {
                                            if (minY > currentPoint.YValues[y])
                                                minY = currentPoint.YValues[y];
                                            if (maxY < currentPoint.YValues[y])
                                                maxY = currentPoint.YValues[y];
                                        }
                                    } else {
                                        if (minY > currentPoint.YValues[0])
                                            minY = currentPoint.YValues[0];
                                        if (maxY < currentPoint.YValues[0])
                                            maxY = currentPoint.YValues[0];
                                    }
                                }
                                seriesCollection[i].minX = minX;
                                seriesCollection[i].maxX = maxX;
                                seriesCollection[i].minY = minY;
                                seriesCollection[i].maxY = maxY;


                            }
                        }

                    if (ej.util.isNullOrUndefined(maxMinXValue.minX) && seriesCollection[i]._visiblePoints.length > 0) {
                        maxMinXValue.maxX = seriesCollection[i].maxX;
                        maxMinXValue.minX = seriesCollection[i].minX;
                        maxMinYValue.maxY = seriesCollection[i].maxY;
                        maxMinYValue.minY = seriesCollection[i].minY;
                    }

                    if (maxMinXValue.minX > seriesCollection[i].minX) {
                        maxMinXValue.minX = seriesCollection[i].minX;
                    }
                    if (maxMinXValue.maxX < seriesCollection[i].maxX) {
                        maxMinXValue.maxX = seriesCollection[i].maxX;
                    }
                    if (maxMinYValue.minY > seriesCollection[i].minY) {
                        maxMinYValue.minY = seriesCollection[i].minY;
                    }
                    if (maxMinYValue.maxY < seriesCollection[i].maxY) {
                        maxMinYValue.maxY = seriesCollection[i].maxY;
                    }

                }
                // Find min/max for stacked series
                if (this.chartObj.model.stackedValue[axis.name]) {
                    var value = this.chartObj.model.stackedValue[axis.name];
                    if (maxMinYValue.minY > value.min)
                        maxMinYValue.minY = value.min;
                    if (maxMinYValue.maxY < value.max)
                        maxMinYValue.maxY = value.max;
                }

                // Find min/max for indicator series

                if (this.chartObj.model.indicatorRange && this.chartObj.model.indicatorRange[axis.name]) {
                    value = this.chartObj.model.indicatorRange[axis.name];
                    if (axis.orientation.toLowerCase() == "horizontal") {
                        if (maxMinXValue.minX > value.min || maxMinXValue.minX==null)
                            maxMinXValue.minX = value.min;
                        if (maxMinXValue.maxX < value.max || maxMinXValue.maxX==null)
                            maxMinXValue.maxX = value.max;
                    } else {
                        if (maxMinYValue.minY > value.min || maxMinYValue.minY==null)
                            maxMinYValue.minY = value.min;
                        if (maxMinYValue.maxY < value.max || maxMinYValue.maxY==null)
                            maxMinYValue.maxY = value.max;
                    }
                }


                if (maxMinXValue.maxX == maxMinXValue.minX)
                    maxMinXValue.maxX += 1;
               
			    if (maxMinYValue.maxY == maxMinYValue.minY) {                   // max == min
                    if (maxMinYValue.maxY < 0)                                  // value less than 0
                        maxMinYValue.maxY = 0;
                    else if (maxMinYValue.maxY <= 1 && maxMinYValue.maxY > 0)   // value between 0 and 1
                        maxMinYValue.minY = 0;
                    else {                                                      // value greater than 1
                        maxMinYValue.minY = 0;
                        maxMinYValue.maxY += 1;
                    }
                }
               
                var deltaY = maxMinYValue.maxY - maxMinYValue.minY;
                if (type == "double") {
                        var data = _sideBySeriesPadding(chartObj, maxMinXValue.minX, maxMinXValue.maxX, axis);
                        maxMinXValue.minX = data.startPadding;
                        maxMinXValue.maxX = data.endPadding;
                        deltaX = maxMinXValue.maxX - maxMinXValue.minX;                   
                }


                //Set range for each axis
                if (axis.orientation.toLowerCase() == "horizontal") {
                    axis.range = (!(chartObj.model.requireInvertedAxes)) ?
                     (type == "double") ? this._getDoubleAutoRange(axis, maxMinXValue.minX, maxMinXValue.maxX, deltaX)
                         : this._getLogAutoRange(axis, maxMinXValue.minX, maxMinXValue.maxX, deltaX)
                    : (type == "double") ? this._getDoubleAutoRange(axis, maxMinYValue.minY, maxMinYValue.maxY, deltaY)
                        : this._getLogAutoRange(axis, maxMinYValue.minY, maxMinYValue.maxY, deltaY);
                }
                if (axis.orientation.toLowerCase() == "vertical") {
                    axis.range = (!(chartObj.model.requireInvertedAxes)) ?
                         (type == "double") ? this._getDoubleAutoRange(axis, maxMinYValue.minY, maxMinYValue.maxY, deltaY)
                             : this._getLogAutoRange(axis, maxMinYValue.minY, maxMinYValue.maxY, deltaY)
                       : (type == "double") ? this._getDoubleAutoRange(axis, maxMinXValue.minX, maxMinXValue.maxX, deltaX)
                           : this._getLogAutoRange(axis, maxMinXValue.minX, maxMinXValue.maxX, deltaX);
                }
            } else {
                if (!axis.setRange)
                    axis.range = { min: 0, max: 5, interval: 1, Delta: 4 };
            }
        },

    AlignRangeStart: function (sDate, intervalSize, intervalType) {
        var sResult = new Date(sDate);
        if (intervalType.toLowerCase() == "days") {
            var day = Math.floor(Math.floor((sDate.getDate()) / intervalSize) * intervalSize);
            sResult = new Date(sDate.getFullYear(), sDate.getMonth(), day, 0, 0, 0);
        } else if (intervalType.toLowerCase() == "hours") {
            var hour = Math.floor(Math.floor((sDate.getHours()) / intervalSize) * intervalSize);
            sResult = new Date(sDate.getFullYear(), sDate.getMonth(), sDate.getDate(), hour, 0, 0);
        } else if (intervalType.toLowerCase() == "milliseconds") {
            var milliseconds = Math.floor(Math.floor((sDate.getMilliseconds()) / intervalSize) * intervalSize);
            sResult = new Date(sDate.getFullYear(), sDate.getMonth(), sDate.getDate(), sDate.getHours(), sDate.getMinutes(), sDate.getSeconds(), milliseconds);
        } else if (intervalType.toLowerCase() == "seconds") {
            var seconds = Math.floor(Math.floor((sDate.getSeconds()) / intervalSize) * intervalSize);
            sResult = new Date(sDate.getFullYear(), sDate.getMonth(), sDate.getDate(), sDate.getHours(), sDate.getMinutes(), seconds, 0);
        } else if (intervalType.toLowerCase() == "minutes") {
            var minutes = Math.floor(Math.floor((sDate.getMinutes()) / intervalSize) * intervalSize);
            sResult = new Date(sDate.getFullYear(), sDate.getMonth(), sDate.getDate(), sDate.getHours(), minutes, 0, 0);
        } else if (intervalType.toLowerCase() == "months") {
            var month = Math.floor(Math.floor((sDate.getMonth()) / intervalSize) * intervalSize);
            sResult = new Date(sDate.getFullYear(), month, sDate.getDate(), 0, 0, 0);
        } else if (intervalType.toLowerCase() == "years") {
            var year = Math.floor(Math.floor(sDate.getFullYear() / intervalSize) * intervalSize);
            sResult = new Date(year, sDate.getMonth(), sDate.getDate(), 0, 0, 0);
        }
        return sResult;
    },
    _increaseDateTimeInterval: function (date, interval, intervalType) {
        var result = new Date(date);
        if (intervalType.toLowerCase() == "days") {
            result.setDate(date.getDate() + interval);
        } else if (intervalType.toLowerCase() == "hours") {
            result.setHours(date.getHours() + interval);
        } else if (intervalType.toLowerCase() == "milliseconds") {
            result.setMilliseconds(date.getMilliseconds() + interval);
        } else if (intervalType.toLowerCase() == "seconds") {
            result.setSeconds(date.getSeconds() + interval);
        } else if (intervalType.toLowerCase() == "minutes") {
            result.setMinutes(date.getMinutes() + interval);
        } else if (intervalType.toLowerCase() == "months") {
            result.setMonth(date.getMonth() + interval);
        } else if (intervalType.toLowerCase() == "years") {
            result.setYear(date.getFullYear() + interval);
        }

        return result;
    },

    _calculatePadding: function (axis) {

        var start = (typeof axis.range.min == "string" && !isNaN(Date.parse(axis.range.min))) ? Date.parse(axis.range.min) : (axis.range.min).getTime();
        var end = (typeof axis.range.max == "string" && !isNaN(Date.parse(axis.range.max))) ? Date.parse(axis.range.max) : (axis.range.max).getTime();
        var interval = ((this._increaseDateTimeInterval(new Date(start), axis.range.interval, axis.intervalType)).getTime()) - start;
        if (!this.chartObj.zoomed && !axis.setRange) {
            start = new Date(start);
            end = new Date(end);
            var intervalType = axis.intervalType.toLowerCase();
            if (axis.rangePadding.toLowerCase() == 'none') {
                start =  start.getTime();
                end = end.getTime();
            } else if (axis.rangePadding.toLowerCase() == 'additional' || axis.rangePadding.toLowerCase() == 'round') {
                switch (intervalType) {
                    case 'years':
                        var startYear = (start.getFullYear() / axis.range.interval) * axis.range.interval;
                        var endYear = end.getFullYear() + (start.getFullYear() - startYear);
                        if (startYear <= 0) {
                            startYear = 1;
                        }
                        if (endYear <= 0) {
                            endYear = 1;
                        }
                        if (axis.rangePadding.toLowerCase() == 'additional') {
                            start = (new Date(startYear - axis.range.interval, 1, 1, 0, 0, 0)).getTime();
                            end = (new Date(endYear + axis.range.interval, 1, 1, 0, 0, 0)).getTime();
                        } else {
                            start = (new Date(startYear, 1, 1, 0, 0, 0)).getTime();
                            end = (new Date(endYear, 1, 1, 0, 0, 0)).getTime();
                        }
                        break;
                    case 'months':
                        var month = (start.getMonth() / axis.range.interval) * axis.range.interval;
                        if (month <= 0) {
                            month = 1;
                        }
                        var endMonth = end.getMonth() + (start.getMonth() - month);
                        if (endMonth <= 0) {
                            endMonth = 1;
                        }
                        if (axis.rangePadding.toLowerCase() == 'round') {
                            start = (new Date(start.getFullYear(), month, 1, 0, 0, 0)).getTime();
                            end = (new Date(end.getFullYear(), endMonth, endMonth == 2 ? 28 : 30, 0, 0, 0)).getTime();
                        } else {
                            start = (new Date(start.getFullYear(), month + (-axis.range.interval), 1, 0, 0, 0)).getTime();
                            end = (new Date(end.getFullYear(), endMonth + (axis.range.interval), endMonth == 2 ? 28 : 30, 0, 0, 0)).getTime();
                        }
                        break;
                    case 'days':
                        var day = (start.getDate() / axis.range.interval) * axis.range.interval;
                        if (day <= 0) {
                            day = 1;
                        }
                        var endDay = end.getDate() + (start.getDate() - day);
                        if (endDay <= 0) {
                            endDay = 1;
                        }
                        if (axis.rangePadding.toLowerCase() == 'round') {
                            start = (new Date(start.getFullYear(), start.getMonth(), day, 0, 0, 0)).getTime();
                            end = (new Date(end.getFullYear(), end.getMonth(), endDay, 0, 0, 0)).getTime();
                        } else {
                            start = (new Date(start.getFullYear(), start.getMonth(), day + (-axis.range.interval), 0, 0, 0)).getTime();
                            end = (new Date(end.getFullYear(), end.getMonth(), endDay + (axis.range.interval), 0, 0, 0)).getTime();
                        }
                        break;
                    case 'hours':
                        var hour = (start.getHours() / axis.range.interval) * axis.range.interval;
                        var endHour = end.getHours() + (start.getHours() - hour);
                        if (axis.rangePadding.toLowerCase() == 'round') {
                            start = (new Date(start.getFullYear(), start.getMonth(), start.getDay(), hour, 0, 0)).getTime();
                            end = (new Date(end.getFullYear(), end.getMonth(), end.getDay(), endHour, 0, 0)).getTime();
                        } else {
                            start = (new Date(start.getFullYear(), start.getMonth(), start.getDay(), hour + (-axis.range.interval), 0, 0)).getTime();
                            end = (new Date(end.getFullYear(), end.getMonth(), end.getDay(), endHour + (axis.range.interval), 0, 0)).getTime();
                        }
                        break;
                    case 'minutes':
                        var minute = (start.getMinutes() / axis.range.interval) * axis.range.interval;
                        var endMinute = end.getMinutes() + (start.getMinutes() - minute);
                        if (axis.rangePadding.toLowerCase() == 'round') {
                            start = (new Date(start.getFullYear(), start.getMonth(), start.getDay(), start.getHour(), minute, 0)).getTime();
                            end = (new Date(end.getFullYear(), end.getMonth(), end.getDay(), end.getHour(), endMinute, 0)).getTime();
                        } else {
                            start = (new Date(start.getFullYear(), start.getMonth(), start.getDay(), start.getHours(), minute + (-axis.range.interval), 0)).getTime();
                            end = (new Date(end.getFullYear(), end.getMonth(), end.getDay(), end.getHours(), minute + (axis.range.interval), 0)).getTime();
                        }
                        break;
                    case 'seconds':
                        var second = (start.getSeconds() / axis.range.interval) * axis.range.interval;
                        var endSecond = end.getSeconds() + (start.getSeconds() - second);
                        if (axis.rangePadding.toLowerCase() == 'round') {
                            start = (new Date(start.getFullYear(), start.getMonth(), start.getDay(), start.getHours(), start.getMinutes(), second, 0)).getTime();
                            end = (new Date(end.getFullYear(), end.getMonth(), end.getDay(), end.getHour(), end.getMinutes(), endSecond, 0)).getTime();
                        } else {
                            start = (new Date(start.getFullYear(), start.getMonth(), start.getDay(), start.getHours(), start.getMinutes(), second + (-axis.range.interval), 0)).getTime();
                            end = (new Date(end.getFullYear(), end.getMonth(), end.getDay(), end.getHours(), end.getMinutes(), endSecond + (axis.range.interval), 0)).getTime();
                        }
                        break;
                    case 'milliseconds':
                        var milliSecond = (start.getMilliseconds() / axis.range.interval) * axis.range.interval;
                        var endMilliSecond = end.getMilliseconds() + (start.getMilliseconds() - milliSecond);
                        if (axis.rangePadding.toLowerCase() == 'round') {
                            start = (new Date(start.getFullYear(), start.getMonth(), start.getDay(), start.getHours(), start.getMinutes(), start.getSeconds(), milliSecond)).getTime();
                            end = (new Date(end.getFullYear(), end.getMonth(), end.getDay(), end.getHours(), end.getMinutes(), end.getSeconds(), endMilliSecond)).getTime();
                        } else {
                            start = (new Date(start.getFullYear(), start.getMonth(), start.getDay(), start.getHours(), start.getMinutes(), start.getSeconds(), milliSecond + (-axis.range.interval))).getTime();
                            end = (new Date(end.getFullYear(), end.getMonth(), end.getDay(), end.getHours(), end.getMinutes(), end.getSeconds(), endMilliSecond + (axis.range.interval))).getTime();
                        }
                        break;
                }

            }

        }
        axis.actualRange.interval = interval;
        axis.actualRange.min = start;
        axis.actualRange.max = end;
        axis._intervalType = axis.intervalType;
        axis._interval = axis.range.interval;
        axis.actualRange.delta = (axis.actualRange.max - axis.actualRange.min);
        axis.range.min = new Date(start);
        axis.range.max = new Date(end);
        this._calculateVisibleRange(axis);

    },
    _calculateVisibleRange: function (axis) {
        axis.visibleRange = $.extend(true, {}, axis.actualRange);
        axis._intervalType = axis.intervalType;
        if (axis.zoomFactor < 1 || axis.zoomPosition > 0) {

            var baseRange = axis.actualRange;
            if (axis.isInversed) {
                var start = axis.actualRange.max - axis.zoomPosition * axis.actualRange.delta;
                var end = start - axis.zoomFactor * axis.actualRange.delta;
            }
            else {
                var start = axis.actualRange.min + axis.zoomPosition * axis.actualRange.delta;
                var end = start + axis.zoomFactor * axis.actualRange.delta;
            }

            if (start < baseRange.min) {
                end = end + (baseRange.min - start);
                start = baseRange.min;
            }

            if (end > baseRange.max) {
                start = start - (end - baseRange.max);
                end = baseRange.max;
            }

            var startDate = new Date(start);
            var endDate = new Date(end);
            //Use below code to find min,max and interval for visible range
            var intervalX = this.calculateDateTimeNiceInterval(axis, startDate, endDate);
            axis.visibleRange.min = Math.min(startDate.getTime(), endDate.getTime());
            axis.visibleRange.max = Math.max(startDate.getTime(),endDate.getTime());
            var interval = ((this._increaseDateTimeInterval(startDate, intervalX.interval, intervalX.intervalType)).getTime()) - axis.visibleRange.min;
            axis._intervalType = intervalX.intervalType;
            axis.visibleRange.interval = interval;
            axis.visibleRange.delta = Math.abs(axis.visibleRange.max - axis.visibleRange.min);
            axis._interval = intervalX.interval;
        }

    },
    _calculateAxisLabels: function (axis) {
        var position;
        if (this.chartObj.zoomed || axis.zoomed || !axis.setRange)
            position = (this.AlignRangeStart(new Date(axis.visibleRange.min), axis._interval, axis._intervalType)).getTime();

        else
            position = axis.visibleRange.min;
        while (position <= axis.visibleRange.max) {
            if (position >= axis.visibleRange.min && position <= axis.visibleRange.max) {

                axis.visibleLabels.push({ Value: position, Text: (Globalize.format(new Date(position), ((!(axis.labelFormat)) ? ej.EjSvgRender.utils._dateTimeLabelFormat(axis._intervalType) : axis.labelFormat))) });
            }
            position = (this._increaseDateTimeInterval(new Date(position), axis._interval, axis._intervalType)).getTime();
            var commonAxesEventArgs = $.extend({}, ej.EjSvgRender.commonChartEventArgs);
            commonAxesEventArgs.data = { label: axis.visibleLabels[axis.visibleLabels.length - 1], axis: axis };
            this.chartObj._trigger("axesLabelRendering", commonAxesEventArgs);
            axis.visibleLabels[axis.visibleLabels.length - 1] = commonAxesEventArgs.data.label;
        }
        axis._LableMaxWidth = ej.EjSvgRender.utils._getMaxLabelWidth(axis, this.chartObj);
    },
    calculateDateTimeNiceInterval: function (axis, startDate, endDate) { 
        var oneDay = 24 * 60 * 60 * 1000;
        //var axisInterval ;
        var totalDays = (Math.abs((startDate.getTime() - endDate.getTime()) / (oneDay)));
        var interval = this.calculateNumericNiceInterval(totalDays / 365, axis);

        if (interval >= 1) {
            return { interval: interval, intervalType: "Years" };
        }

        interval = this.calculateNumericNiceInterval(totalDays / 30, axis);

        if (interval >= 1) {
            return { interval: interval, intervalType: "Months" };
        }

        interval = this.calculateNumericNiceInterval(totalDays, axis);

        if (interval >= 1) {


            return { interval: interval, intervalType: "Days" };
        }

        var totalHours = totalDays * 24;

        interval = this.calculateNumericNiceInterval(totalHours, axis);

        if (interval >= 1) {

            return { interval: interval, intervalType: "Hours" };
        }

        var totalMinutes = totalDays * 24 * 60;

        interval = this.calculateNumericNiceInterval(totalMinutes, axis);

        if (interval >= 1) {

            return { interval: interval, intervalType: "Minutes" };
        }

        var totalSeconds = totalDays * 24 * 60 * 60;

        interval = this.calculateNumericNiceInterval(totalSeconds, axis);

        if (interval >= 1) {

            return { interval: interval, intervalType: "Seconds" };
        }

        var totalMilliseconds = totalDays * 24 * 60 * 60 * 1000;

        interval = this.calculateNumericNiceInterval(totalMilliseconds, axis);

        if (interval >= 1) {

            return { interval: interval, intervalType: "Milliseconds" };
        }

    },
    calculateNumericNiceInterval: function (delta, axis) {
        var desiredIntervalsCount = this.GetActualDesiredIntervalsCount(axis, axis.length);
        var niceInterval = delta / desiredIntervalsCount;
        var minInterval = Math.pow(10, Math.floor(ej.EjSvgRender.utils._logBase(niceInterval, 10)));
        var intervalDivs = [10, 5, 2, 1];

        for (var i = 0; i < intervalDivs.length; i++) {
            var currentInterval = minInterval * intervalDivs[i];
            if (desiredIntervalsCount < (delta / currentInterval)) {
                return niceInterval;
            }

            niceInterval = currentInterval;
        }

        return niceInterval;
    },
    GetActualDesiredIntervalsCount: function (axis, size) {
        if (ej.util.isNullOrUndefined(axis.desiredIntervals)) {
            var desiredIntervalsCount = (axis.orientation.toLowerCase() == "horizontal" ? 0.533 : 1) * axis.maximumLabels;
            desiredIntervalsCount = Math.max((size * (desiredIntervalsCount / 100)), 1);
            axis.desiredIntervals = desiredIntervalsCount;
            return desiredIntervalsCount;
        } else {
            return axis.desiredIntervals;
        }
    },
    _calculateRange: function (axis, seriesCollection) {
        if (!this.chartObj.zoomed) {
            var defaultDate = new Date();
            if (seriesCollection.length > 0 && seriesCollection[0].points.length > 0 || this.chartObj.model.indicators[0].points.length > 0) {
                var maxX, minY;
                var maxY, minX;
                var maxMinXValue = { maxX: null, minX: null };
                var maxMinYValue = { maxY: null, minY: null };
                for (var i = 0; i < seriesCollection.length; i++) {
                    if (seriesCollection[i].points.length > 0) {
                        if (seriesCollection[i].visibility.toLowerCase() === 'visible' && seriesCollection[i].points.length > 0) {
                            minX = maxX = seriesCollection[i].points[0].X;
                            maxY = minY = seriesCollection[i].points[0].YValues[0];
                            for (var j = 0; j < seriesCollection[i].points.length; j++) {
                                if (minX > seriesCollection[i].points[j].X)
                                    minX = seriesCollection[i].points[j].X;
                                if (maxX < seriesCollection[i].points[j].X)
                                    maxX = seriesCollection[i].points[j].X;
                                if (minY > seriesCollection[i].points[j].YValues[0])
                                    minY = seriesCollection[i].points[j].YValues[0];
                                if (maxY < seriesCollection[i].points[j].YValues[0])
                                    maxY = seriesCollection[i].points[j].YValues[0];
                            }
                            seriesCollection[i].minX = minX;
                            seriesCollection[i].maxX = maxX;
                            seriesCollection[i].minY = minY;
                            seriesCollection[i].maxY = maxY;
                        }
                    }
                    
                    if ((maxMinXValue.minX = (maxMinXValue.minX == null  && typeof seriesCollection[i].minX != "number") ? seriesCollection[i].minX : maxMinXValue.minX) > seriesCollection[i].minX) {
                        maxMinXValue.minX = (typeof seriesCollection[i].minX != "number") ? seriesCollection[i].minX : maxMinXValue.minX;
                        }
                    if ((maxMinXValue.maxX = (maxMinXValue.maxX == null && typeof seriesCollection[i].maxX != "number") ? seriesCollection[i].maxX : maxMinXValue.maxX) < seriesCollection[i].maxX) {
                        maxMinXValue.maxX = (typeof seriesCollection[i].maxX != "number") ? seriesCollection[i].maxX : maxMinXValue.maxX;
                        }
                        if (maxMinYValue.minY > seriesCollection[i].minY) {
                            maxMinYValue.minY = seriesCollection[i].minY;
                        }
                        if (maxMinYValue.maxY < seriesCollection[i].maxY) {
                            maxMinYValue.maxY = seriesCollection[i].maxY;
                        }
               
                }
                maxMinXValue.minX = (maxMinXValue.minX == null) ? defaultDate : maxMinXValue.minX;
                maxMinXValue.maxX = (maxMinXValue.maxX == null) ? new Date(new Date().setMonth(defaultDate.getMonth() + 5)) : maxMinXValue.maxX;
                if (maxMinXValue.minX == maxMinXValue.maxX) {
                    maxMinXValue.minX = new Date(Date.parse(maxMinXValue.minX) - 2592000000);
                    maxMinXValue.maxX = new Date(Date.parse(maxMinXValue.maxX) + 2592000000);
                }

                var data = _sideBySeriesPadding(this.chartObj, Date.parse(maxMinXValue.minX), Date.parse(maxMinXValue.maxX), axis);
                maxMinXValue.minX = new Date(data.startPadding);
                maxMinXValue.maxX = new Date(data.endPadding);

                // Find min/max for indicator series

                if (this.chartObj.model.indicatorRange && this.chartObj.model.indicatorRange[axis.name]) {
                    var value = this.chartObj.model.indicatorRange[axis.name];
                    if (maxMinXValue.minX > new Date(value.min))
                        maxMinXValue.minX = new Date(value.min);
                    if (maxMinXValue.maxX < new Date(value.max))
                        maxMinXValue.maxX = new Date(value.max);
                }

                var intervalX = this.calculateDateTimeNiceInterval(axis, maxMinXValue.minX, maxMinXValue.maxX);                
            
                    if ((!axis.setRange)) {
                        axis.range = { min: maxMinXValue.minX, max: maxMinXValue.maxX, interval: intervalX.interval };
                        axis.intervalType = intervalX.intervalType;

                    } else {
                        if (ej.util.isNullOrUndefined(axis.actual_Range)) {
                            if (ej.util.isNullOrUndefined(axis.range.interval))
                                axis.range.interval = intervalX.interval;
                            if (!axis.range.max)
                                axis.range.max = maxMinXValue.maxX;
                            if (!axis.range.min)
                                axis.range.min = maxMinXValue.minX;
                            if (!axis.intervalType)
                                axis.intervalType = intervalX.intervalType;
                            if (typeof axis.range.min == "string" && axis.range.min.indexOf("/Date(") != -1)
                                axis.range.min = new Date(parseInt(axis.range.min.substr(6)));
                            if (typeof axis.range.max == "string" && axis.range.max.indexOf("/Date(") != -1)
                                axis.range.max = new Date(parseInt(axis.range.max.substr(6)));
                            axis.actual_Range = $.extend(true, {}, axis.range);
                            axis.interval_Type = axis.intervalType;
                            axis.setRange = true;
                        }
                        axis.range = { min: axis.actual_Range.min, max: axis.actual_Range.max, interval: axis.actual_Range.interval };
                        axis.intervalType = axis.interval_Type;
                    }
            } else {
                if (!axis.setRange) {
                    axis.range = { min: defaultDate, max: new Date(new Date().setMonth(defaultDate.getMonth() + 5)), interval: 1 };
                    axis.intervalType = 'Months';
                }

            }
        }
    },

    _calculateRanges: function (chartObj, axis, seriesCollection) {

        this.chartObj = chartObj;

        this._calculateRange(axis, seriesCollection);

        this._calculatePadding(axis);

        this._calculateAxisLabels(axis);
    }
};

function ejExtendClass(parent, members) {
    var object = function () {
    };
    object.prototype = new parent();
    $.extend(object.prototype, members);
    return object;
}

//DateTime calculation
var ejDateTimeValue = ejExtendClass(ej.EjAxisRenderer);
ej.axisTypes.datetime = ejDateTimeValue;

var ejDoubleValue = ejExtendClass(ej.EjAxisRenderer, {
    
  
    _calculateRange: function (chartObj, axis, seriesCollection) {
        if (!chartObj.zoomed) {
            this._seriesMinMax(chartObj,axis, seriesCollection, "double");
        }
    },
    
    _getDoubleAutoRange: function (axis, min, max, delta) {
        var interval = this.calculateNumericNiceInterval(delta, axis);
        if (!axis.setRange) {
            return axis.range = { min: min, max: max, interval: interval, Delta: delta };

        } else {
            if (ej.util.isNullOrUndefined(axis.range.interval))
                      axis.range.interval = interval;
            if (ej.util.isNullOrUndefined(axis.range.max))
                       axis.range.max = max;          
            if (ej.util.isNullOrUndefined(axis.range.min))
                       axis.range.min = min;
            if (ej.util.isNullOrUndefined(axis.actual_Range)) {
                axis.actual_Range = $.extend(true, {}, axis.range);
                axis.setRange = true;
            }
            return axis.range = { min: axis.actual_Range.min, max: axis.actual_Range.max, interval: axis.actual_Range.interval, delta: (axis.actual_Range.max - axis.actual_Range.min) };
        }
    },
    _calculatePadding: function (chartObj, axis, baseRange) {
        var start = baseRange.min;
        var end = baseRange.max;
        var rangePadding = axis.rangePadding.toLowerCase();
        var interval= baseRange.interval;
        if ((!axis.setRange) && (!chartObj.zoomed)) {
               
            if (rangePadding == 'normal') {
                var minimum = 0, remaining;
                if (start < 0) {
                    start = 0;
                    minimum = baseRange.min + (baseRange.min / 20);

                    remaining =  interval + (minimum %  interval);

                    if ((0.365 *  interval) >= remaining) {
                        minimum -= interval;
                    }

                    if (minimum %  interval < 0) {
                        minimum = (minimum -  interval) - (minimum % interval);
                    }
                } else {
                    minimum = start < ((5.0 / 6.0) * end)
                        ? 0
                        : (start - (end - start) / 2);
                    if (minimum %  interval > 0) {
                        minimum -= (minimum %  interval);
                    }
                }
                var maximum = (end + (end - start) / 20);

                remaining = interval - (maximum % interval);

                if ((0.365 *  interval) >= remaining) {
                    maximum +=  interval;
                }

                if (maximum %  interval > 0) {
                    maximum = (maximum +  interval) - (maximum % interval);
                }
                 if (minimum <= 0) {
                    interval = this.calculateNumericNiceInterval(maximum - minimum, axis);
                    maximum = Math.ceil(maximum / interval) * interval;
                }
                start = (!(axis.roundingPlaces)) ? parseFloat(minimum.toFixed((ej.EjSvgRender.utils._decimalPlaces(interval) == 0 ? 1 : ej.EjSvgRender.utils._decimalPlaces(interval)))) : parseFloat(minimum.toFixed(axis.roundingPlaces));
                end = (!(axis.roundingPlaces)) ? parseFloat(maximum.toFixed((ej.EjSvgRender.utils._decimalPlaces(interval) == 0 ? 1 : ej.EjSvgRender.utils._decimalPlaces(interval)))) : parseFloat(maximum.toFixed(axis.roundingPlaces));

            } else if (rangePadding == 'additional' || rangePadding == 'round') {
                start = Math.floor((start / interval) * interval);
                end = Math.ceil((end / interval) * interval);
                if (rangePadding == 'additional') {
                    start -= interval;
                    end +=  interval;
                }
            }


        }
        axis.actualRange.min = start;
        axis.actualRange.max = end;
        axis.actualRange.interval = interval;
        axis.range.min = start;
        axis.range.max = end;
        axis.range.interval =  interval;
        axis.actualRange.delta = end - start;
        this._calculateVisibleRange(axis);
    },
    _calculateVisibleRange: function (axis) {
        axis.visibleRange = $.extend(true, {}, axis.actualRange);
        if (axis.zoomFactor < 1 || axis.zoomPosition > 0) {
            var baseRange = axis.actualRange;

            if (axis.isInversed) {
                var start=axis.actualRange.max - axis.zoomPosition * axis.actualRange.delta;
                var end = start - axis.zoomFactor * axis.actualRange.delta;
            }
            else {
                var start = axis.actualRange.min + axis.zoomPosition * axis.actualRange.delta;
                var end = start + axis.zoomFactor * axis.actualRange.delta;
            }

            if (start < baseRange.min) {
                end = end + (baseRange.min - start);
                start = baseRange.min;
            }

            if (end > baseRange.max) {
                start = start - (end - baseRange.max);
                end = baseRange.max;
            }
            var delta =Math.abs(end-start);
            axis.visibleRange.interval = (this.calculateNumericNiceInterval(delta, axis));
            var factor = (ej.util.isNullOrUndefined(axis.roundingPlaces)) ? (ej.EjSvgRender.utils._decimalPlaces(axis.visibleRange.interval) == 0 ? 1 : ej.EjSvgRender.utils._decimalPlaces(axis.visibleRange.interval)) : axis.roundingPlaces;
            axis.visibleRange.interval = parseFloat(axis.visibleRange.interval.toFixed(factor));
            axis.visibleRange.min = Math.min(start, end);
            axis.visibleRange.max = Math.max(start, end);
            axis.visibleRange.delta = delta;
            axis.zoomed = (this.chartObj.zoomed === null || this.chartObj.zoomed === undefined) ? true : this.chartObj.zoomed;
        }

    },
    _calculateAxisLabels: function (chartObj, currentAxis) {

        var tempInterval;
		if (chartObj.zoomed || currentAxis.zoomed || currentAxis.padding)
            tempInterval = currentAxis.visibleRange.min - (currentAxis.visibleRange.min % currentAxis.visibleRange.interval);
		else
            tempInterval = currentAxis.visibleRange.min;
        for (; tempInterval <= currentAxis.visibleRange.max; tempInterval += currentAxis.visibleRange.interval) {
            tempInterval = (!(currentAxis.roundingPlaces)) ? parseFloat(tempInterval.toFixed((ej.EjSvgRender.utils._decimalPlaces(currentAxis.visibleRange.interval) == 0 ? 1 : ej.EjSvgRender.utils._decimalPlaces(currentAxis.visibleRange.interval)))) : parseFloat(tempInterval.toFixed(currentAxis.roundingPlaces));

            if (ej.EjSvgRender.utils._inside(tempInterval, currentAxis.visibleRange)) {


                var customFormat = (!(currentAxis.labelFormat)) ? null : currentAxis.labelFormat.match('{value}');
                var labelText = (!(currentAxis.labelFormat)) ? tempInterval : (customFormat != null) ? currentAxis.labelFormat.replace('{value}', tempInterval) : (Globalize.format(tempInterval, currentAxis.labelFormat));
                currentAxis.visibleLabels.push({ Value: tempInterval, Text: labelText });
                // customize label by event
                var commonAxesEventArgs = $.extend({}, ej.EjSvgRender.commonChartEventArgs);
                commonAxesEventArgs.data = { label: currentAxis.visibleLabels[currentAxis.visibleLabels.length - 1], axis: currentAxis };
                chartObj._trigger("axesLabelRendering", commonAxesEventArgs);
                currentAxis.visibleLabels[currentAxis.visibleLabels.length - 1] = commonAxesEventArgs.data.label;
            }
        }

        currentAxis._LableMaxWidth = ej.EjSvgRender.utils._getMaxLabelWidth(currentAxis, this.chartObj);

    },

    _calculateRanges: function (chartObj, axis, seriesCollection) {
        this.chartObj = chartObj;

        this._calculateRange(chartObj, axis, seriesCollection);


        this._calculatePadding(chartObj, axis, axis.range);


        this._calculateAxisLabels(chartObj, axis);

    }
});
ej.axisTypes.double = ejDoubleValue;
//log axis
var ejLogValue = ejExtendClass(ej.EjAxisRenderer, {
    
    // Get value for log min and max
   _getRoundValue:function (x,div,up)
    {
        return (up ? Math.ceil(x / div) : Math.floor(x / div)) * div;
   },
   
   // Caculate log nice interval
  calculateLogNiceInterval: function (delta, axis) {
       var desiredIntervalsCount = this.GetActualDesiredIntervalsCount(axis, axis.length);
       var niceInterval = delta;
       var minInterval = Math.pow(10, Math.floor(ej.EjSvgRender.utils._logBase(niceInterval, 10)));
       var intervalDivs = [10, 5, 2, 1];

       for (var i = 0; i < intervalDivs.length; i++) {
           var currentInterval = minInterval * intervalDivs[i];
           if (desiredIntervalsCount < (delta / currentInterval)) {
               return niceInterval;
           }

           niceInterval = currentInterval;
       }

       return niceInterval;
  },
   
    // Get log value for auto range
    _getLogAutoRange: function (axis, min, max, delta) {
        
        var logStart = ej.EjSvgRender.utils._logBase(min, axis.logBase);
        logStart = $.isNumeric(logStart) ? logStart : min;
        var logEnd = ej.EjSvgRender.utils._logBase(max, axis.logBase);
        logEnd = $.isNumeric(logEnd) ? logEnd : max;
     
        var mulS = this._getRoundValue(logStart, 1, false);
        var mulE = this._getRoundValue(logEnd, 1, true);
        delta = mulE - mulS;
        
        var interval = this.calculateLogNiceInterval(delta, axis);
        if (!axis.setRange) {
            return axis.range = { min: mulS, max: mulE, interval: interval, Delta: delta };

        } else {
            if (ej.util.isNullOrUndefined(axis.range.interval))
                axis.range.interval = interval;
            if (ej.util.isNullOrUndefined(axis.range.max))
                axis.range.max = mulE;
            if (ej.util.isNullOrUndefined(axis.range.min))
                axis.range.min = mulS;
            if (ej.util.isNullOrUndefined(axis.actual_Range)) {
                axis.actual_Range = $.extend(true, {}, axis.range);
                axis.setRange = true;
            }
            return axis.range = {
                min: axis.actual_Range.min,
                max: axis.actual_Range.max,
                interval: axis.actual_Range.interval,
                delta: (axis.actual_Range.max - axis.actual_Range.min)
            };
        }
    },
   
      //Calcualte auto log range from series
        _calculateLogRange: function (chartObj, axis, seriesCollection) {
            if (!chartObj.zoomed) {
                this._seriesMinMax(chartObj,axis, seriesCollection, "logarithmic");
            }
        },
   
       //Assign base range values to actual range
        _calculateLogPadding: function (chartObj, axis, baseRange) {
            // No padding support for log axis
            var start = baseRange.min;
            var end = baseRange.max;
            var interval = baseRange.interval;
            axis.actualRange.min = start;
            axis.actualRange.max = end;
            axis.actualRange.interval = interval;
            axis.range.min = start;
            axis.range.max = end;
            axis.range.interval = interval;
            axis.actualRange.delta = end - start;
            this._calculateVisibleRange(axis);
        },
   
        // Calculate visible range for zooming
        _calculateVisibleRange: function (axis) {
            axis.visibleRange = $.extend(true, {}, axis.actualRange);
            if (axis.zoomFactor < 1 || axis.zoomPosition > 0) {
                var baseRange = axis.actualRange;
                if (axis.isInversed) {
                    var start = axis.actualRange.max - axis.zoomPosition * axis.actualRange.delta;
                    var end = start - axis.zoomFactor * axis.actualRange.delta;
                }
                else {
                    var start = axis.actualRange.min + axis.zoomPosition * axis.actualRange.delta;
                    var end = start + axis.zoomFactor * axis.actualRange.delta;
                }

                if (start < baseRange.min) {
                    end = end + (baseRange.min - start);
                    start = baseRange.min;
                }

                if (end > baseRange.max) {
                    start = start - (end - baseRange.max);
                    end = baseRange.max;
                }
                var delta = Math.abs(end - start);
                axis.visibleRange.interval = (this.calculateLogNiceInterval(delta, axis));
                var factor = (ej.util.isNullOrUndefined(axis.roundingPlaces)) ?
                    (ej.EjSvgRender.utils._decimalPlaces(axis.visibleRange.interval) == 0 ? 1
                    : ej.EjSvgRender.utils._decimalPlaces(axis.visibleRange.interval)) : axis.roundingPlaces;
                axis.visibleRange.interval = parseFloat(axis.visibleRange.interval.toFixed(factor));
                axis.visibleRange.min = Math.min(start, end);
                axis.visibleRange.max = Math.max(start, end);
                axis.visibleRange.delta = delta;
                axis.zoomed = (this.chartObj.zoomed === null || this.chartObj.zoomed === undefined) ? true : this.chartObj.zoomed;
            }

        },
   
        //Method implementation for Generate Labels in ChartAxis
        _calculateAxisLabels: function (chartObj, currentAxis) {

            var tempInterval;
            if (chartObj.zoomed || currentAxis.zoomed)
                tempInterval = currentAxis.visibleRange.min - (currentAxis.visibleRange.min % currentAxis.visibleRange.interval);
            else
                tempInterval = currentAxis.visibleRange.min;
            for (; tempInterval <= currentAxis.visibleRange.max; tempInterval += currentAxis.visibleRange.interval) {
                

                if (ej.EjSvgRender.utils._inside(tempInterval, currentAxis.visibleRange)) {      
                    var customFormat = (!(currentAxis.labelFormat)) ? null : currentAxis.labelFormat.match('{value}');
                    var tempIntervaltext = Math.pow(currentAxis.logBase, tempInterval);
                    tempIntervaltext = (!(currentAxis.roundingPlaces)) ?
                        parseFloat(tempIntervaltext.toFixed((ej.EjSvgRender.utils._decimalPlaces(currentAxis.visibleRange.interval) == 0 ? 1
                            : ej.EjSvgRender.utils._decimalPlaces(currentAxis.visibleRange.interval))))
                            : parseFloat(tempIntervaltext.toFixed(currentAxis.roundingPlaces));
                    var labelText = (!(currentAxis.labelFormat)) ? tempIntervaltext : (customFormat != null) ?
                             currentAxis.labelFormat.replace('{value}', tempIntervaltext) :
                              (Globalize.format(tempIntervaltext, currentAxis.labelFormat));
                    currentAxis.visibleLabels.push({ Value:tempInterval, Text: labelText });
                    // customize label by event
                    var commonAxesEventArgs = $.extend({}, ej.EjSvgRender.commonChartEventArgs);
                    commonAxesEventArgs.data = { label: currentAxis.visibleLabels[currentAxis.visibleLabels.length - 1], axis: currentAxis };
                    chartObj._trigger("axesLabelRendering", commonAxesEventArgs);
                    currentAxis.visibleLabels[currentAxis.visibleLabels.length - 1] = commonAxesEventArgs.data.label;
                }
            }

            currentAxis._LableMaxWidth = ej.EjSvgRender.utils._getMaxLabelWidth(currentAxis, this.chartObj);

        },


        //Category axis calculation
        _calculateRanges: function (chartObj, axis, seriesCollection) {
            this.chartObj = chartObj;
            this._calculateLogRange(chartObj,axis, seriesCollection);
            this._calculateLogPadding(chartObj,axis,axis.range);
            this._calculateAxisLabels(chartObj,axis);
        }

        
    });
    ej.axisTypes.logarithmic = ejLogValue;
    
//Category Axes calculation
    var ejCategoryValue = ejExtendClass(ej.EjAxisRenderer, {
        //axis range calculation and assign indexed value to x value of points
        _calculateCategoryRange: function (axis, seriesCollection) {
            var legendCollapsed = (this.chartObj.model.legendCollapsed == undefined) ? !this.chartObj.zoomed : this.chartObj.model.legendCollapsed;
            if (!this.chartObj.zoomed || legendCollapsed) {
            var maxXValue = { maxX: null, minX: 0 };
            var labels = [];
            var pointX;
            if (seriesCollection.length > 0 && seriesCollection[0].points.length > 0  || this.chartObj.model.indicators[0].points.length>0) {
                for (var m = 0; m < seriesCollection.length; m++) {
                    for (var n = 0; n < seriesCollection[m].points.length; n++) {
                        if (seriesCollection[m]._xAxisValueType == "date")
                            pointX = Globalize.format(new Date(seriesCollection[m].points[n].x), (ej.util.isNullOrUndefined(axis.labelFormat) ? "dd/MM/yyyy" : axis.labelFormat));
                        else
                            pointX = seriesCollection[m].points[n].x;
                        var index = $.inArray(pointX, labels);
                        if (index < 0)
                            labels.push(pointX);
                    }
                }
                for (var l = 0; l < seriesCollection.length; l++) {
                    seriesCollection[l]._pointCollection = [];
                    for (var k = 0; k < seriesCollection[l].points.length; k++) {
                        seriesCollection[l]._pointCollection.push(seriesCollection[l].points[k].x);
                        if (seriesCollection[l]._xAxisValueType == "date")
                            pointX = Globalize.format(new Date(seriesCollection[l].points[k].x), (ej.util.isNullOrUndefined(axis.labelFormat) ? "dd/MM/yyyy" : axis.labelFormat));
                        else
                            pointX = seriesCollection[l].points[k].x;
                        seriesCollection[l].points[k].X = $.inArray(pointX, labels);
                       
                    }
                    seriesCollection[l].minX = 0;
                    seriesCollection[l].maxX = (seriesCollection[l]._xAxisValueType == "date") ? seriesCollection[l]._pointCollection.length - 1 : labels.length - 1;
                    // To find maximum x value of axis from series collection
                    if (l == 0) {
                        maxXValue.maxX = seriesCollection[l].maxX;
                        axis.labels = (seriesCollection[l]._xAxisValueType == "date") ? seriesCollection[l]._pointCollection : labels;
                        axis._categoryValueType = seriesCollection[l]._xAxisValueType;
                    }
                    if (maxXValue.maxX < (seriesCollection[l].maxX)) {
                        maxXValue.maxX = seriesCollection[l].maxX;
                        axis.labels = [];
                        axis.labels = labels;
                    }
                }
                if (this.chartObj.model.indicatorRange && this.chartObj.model.indicatorRange[axis.name]) {
                          var value = this.chartObj.model.indicatorRange[axis.name];
                          if (maxXValue.minX > value.min || maxXValue.minX == null)
                              maxXValue.minX = value.min;
                          if (maxXValue.maxX < value.max || maxXValue.maxX == null)
                              maxXValue.maxX = value.max;
                }
                var deltaX = maxXValue.maxX - maxXValue.minX;
                var intervalX = this._calculateActualInterval(deltaX, axis);
                axis.range = { min: maxXValue.minX, max: maxXValue.maxX, interval: intervalX, Delta: deltaX };
            }
            else {
                if(axis.labels.length>0)
                    axis.range = { min: 0, max: axis.labels.length - 1, interval: 1, Delta: axis.labels.length - 1 };
            }       
            }

        },

    //Calculates actual interval
    _calculateActualInterval: function (delta, axis) {
        if (axis.categoryInterval == null)
            return Math.max(1, Math.floor(delta / this.GetActualDesiredIntervalsCount(axis, axis.length)));
        else
            return axis.categoryInterval;
    },

    //Apply padding based on labelPlacement
    _applyRangePadding: function (axis) {
        if (!this.chartObj.zoomed) {
            // ticks based on labelplacement and polaraxes
            var ticks = 0;
            if(this.chartObj.model.AreaType !== 'polaraxes')
            ticks = ((axis.labelPlacement === null || axis.labelPlacement === undefined)) ? -0.5 : (axis.labelPlacement.toLowerCase() == "betweenticks") ? (-0.5) : 0;
			 if (ticks < 0) {
                axis.range.min = axis.range.min + ticks;
                axis.range.max = axis.range.max - ticks;
                axis.range.delta = axis.range.max - axis.range.min;
                axis.actualRange = axis.range;
            } else {
                var data = _sideBySeriesPadding(this.chartObj, axis.range.min, axis.range.max, axis);
                axis.range.min = data.startPadding;
                axis.range.max = data.endPadding;
                axis.range.delta = axis.range.max - axis.range.min;
                axis.actualRange = axis.range;
			}
        }
        this._calculateVisibleRange(axis);
    },

    //Calculates the visible range  
    _calculateVisibleRange: function (axis) {
        axis.visibleRange = $.extend(true, {}, axis.actualRange);
        if (axis.zoomFactor < 1 || axis.zoomPosition > 0) {
            var baseRange = axis.actualRange;
            if (axis.isInversed) {
                var start = axis.actualRange.max - axis.zoomPosition * axis.actualRange.delta;
                var end = start - axis.zoomFactor * axis.actualRange.delta;
            }
            else {
                var start = axis.actualRange.min + (axis.zoomPosition * axis.actualRange.delta);
                var end = start + axis.zoomFactor * axis.actualRange.delta;
            }

            if (start < baseRange.min) {
                end = end + (baseRange.min - start);
                start = baseRange.min;
            }

            if (end > baseRange.max) {
                start = start - (end - baseRange.max);
                end = baseRange.max;
            }
            var delta = Math.abs(end - start);
            axis.visibleRange.interval = this._calculateActualInterval(delta, axis);
            axis.visibleRange.min = Math.min(start,end);
            axis.visibleRange.max = Math.max(start, end);
            axis.visibleRange.delta = delta;
            axis.zoomed = (this.chartObj.zoomed === null || this.chartObj.zoomed === undefined) ? true : this.chartObj.zoomed;
        }

    },

    //Generate category label for axis
    _calculateAxisLabels: function (axis) {
        var interval = axis.visibleRange.interval;
        var position = axis.visibleRange.min - (axis.visibleRange.min % axis.range.interval);
        for (; position <= axis.visibleRange.max; position += interval) {
            if (ej.EjSvgRender.utils._inside(position, axis.visibleRange)) {
                var pos = Math.round(position);
                axis.visibleLabels.push({ Value: pos, Text: ej.EjSvgRender.utils._getLabelContent(pos, axis) });

                // customize label by event
                var commonAxesEventArgs = $.extend({}, ej.EjSvgRender.commonChartEventArgs);
                commonAxesEventArgs.data = { label: axis.visibleLabels[axis.visibleLabels.length - 1], axis: axis };
                this.chartObj._trigger("axesLabelRendering", commonAxesEventArgs);
                axis.visibleLabels[axis.visibleLabels.length - 1] = commonAxesEventArgs.data.label;
            }
        }
        axis._LableMaxWidth = ej.EjSvgRender.utils._getMaxLabelWidth(axis, this.chartObj);
    },

    //Category axis calculation
    _calculateRanges: function (chartObj, axis, seriesCollection) {
        this.chartObj = chartObj;

        this._calculateCategoryRange(axis, seriesCollection);
        this._applyRangePadding(axis);

        this._calculateAxisLabels(axis);
    }
});
ej.axisTypes.category = ejCategoryValue;

ej.EjAxisRenderer.prototype = {
    _drawAxes: function (axisIndex, axis) {

        if (this.model.AreaType == "cartesianaxes")
        {
            if (axis.orientation.toLowerCase() == "horizontal")
            {
                this._drawXAxisGridLine(axisIndex, axis);
                if (axis.visible)
                {
                    this.model.xAxisLabelRegion = [];
                    if (axis.labelPosition!='inside')
                    {
                        this._drawXAxisLabels(axisIndex, axis);
                    }
                    this._drawXTitle(axisIndex, axis);
                }
            }

            if (axis.orientation.toLowerCase() == "vertical") {
                this._drawYAxisGridLine(axisIndex, axis);
                if (axis.visible) {
                    if (axis.labelPosition != 'inside')
                    {
                        this._drawYAxisLabels(axisIndex, axis);
                    }
                    this._drawYTitle(axisIndex, axis);
                }
            }
        }
        else {
            if (axis.orientation.toLowerCase() == "horizontal") {
                this._drawPolarGridLine(axis);
                this._drawPolarLabels(axis);
                 
            }
             else 
                this._drawPolarCircle(axis);           
         }
    },
    _getSharpPath:function (width) {
        var value = ((width % 2) == 0) ? 0 : 0.5;
        return value;
    },
    _drawAxisLine: function (axis) {

        // Yaxis MajorGridlines, Ticklines and Labels.
        var gEle = this.svgRenderer.createGroup({ 'id': this.svgObject.id + '_YAxisLine' });
        var sbYMajorTick = ej.EjSvgRender.utils._getStringBuilder();
        var x1 = this.model.centerX;
        var y1 = this.model.centerY;
        var x2 = this.model.centerX;
        var y2 = this.model.centerY - this.model.Radius;
        var axisLine = "M" + " " + x1 + " " + y1 + " " + "L" + " " + x2 + " " + y2;
        if (axis.axisLine.visible) {
            var options = {
                'id': this.svgObject.id + '_YAxisLines',
                'fill': 'none',
                'stroke-width': axis.axisLine.width,
                'stroke': axis.axisLine.color,
                'opacity': axis.axisLine.opacity,
                'stroke-dasharray': axis.axisLine.dashArray,
                'd': axisLine
            };
            this.svgRenderer.drawPath(options, gEle);
        }
        //Drawing Major Grid Lines 
        $(gEle).appendTo(this.gPolarAxisEle);

        if (axis.majorTickLines.visible) {
            gEle = this.svgRenderer.createGroup({ 'id': this.svgObject.id + '_YAxisMajorTicks' });
                for (var j = 0; j < axis.visibleLabels.length; j++) {
                    var label = axis.visibleLabels[j];
                    var radius = this.model.Radius * ej.EjSvgRender.utils._valueToCoefficient(axis, label.Value, this);
                    var line = {};
                    line.X1 = this.model.centerX;
                    line.Y1 = this.model.centerY - radius;
                    line.X2 = line.X1 - axis.majorTickLines.size;
                    line.Y2 = line.Y1;
                    sbYMajorTick.append("M" + " " + line.X1 + " " + line.Y1 + " " + "L" + " " + line.X2 + " " + line.Y2 + " ");
                }
               
                  yMajorTickDir = sbYMajorTick.toString();
                    var options = {
                        'id': this.svgObject.id + '_YAxisMajorTicks',
                        'fill': 'none',
                        'stroke-width': axis.majorTickLines.width,
                        'stroke': axis.majorTickLines.color,
                        'd': yMajorTickDir
                    };
            //Drawing Major Tickline Lines
                    this.svgRenderer.drawPath(options, gEle);
                    $(gEle).appendTo(this.gPolarAxisEle);
                
        }

        var gEle = this.svgRenderer.createGroup({ 'id': this.svgObject.id + '_YAxisLabels' });
        var font = axis.font;
        for (var j = 0; j < axis.visibleLabels.length; j++) {
            var radius = this.model.Radius * ej.EjSvgRender.utils._valueToCoefficient(axis, axis.visibleLabels[j].Value, this);
            var labelText = axis.visibleLabels[j].Text;

            var size = ej.EjSvgRender.utils._measureText(labelText, null, axis.font);
           
            var X = this.model.centerX - axis.majorTickLines.size- (this.model.elementSpacing/3);
            var Y = this.model.centerY - radius + (size.height/5);
            
           
            var options = {
                'id': this.svgObject.id + '_' + axis.name + '_YLabel_' + j,
                'x': X,
                'y': Y,
                'fill': font.color,
                'font-size': font.size,
                'font-family': font.fontFamily,
                'font-style': font.fontStyle,
                'font-weight': font.fontWeight,
                'opacity': font.opacity,
                'text-anchor': 'end'
            };

            this.svgRenderer.drawText(options, labelText, gEle);

        }
        //Drawing YAxis Labels
        $(gEle).appendTo(this.gPolarAxisEle);
       

    },
    _drawPolarLabels: function (axis) {
        var gEle = this.svgRenderer.createGroup({ 'id': this.svgObject.id + '_XAxisLabels'  });
        var aroundRadius = this.model.Radius + axis.majorTickLines.size;
        var font = axis.font;
        for (var j = 0; j < axis.visibleLabels.length; j++) {
            var coef = ej.EjSvgRender.utils._valueToPolarCoefficient(axis, axis.visibleLabels[j].Value);

            var vector = ej.EjSvgRender.utils._valueToVector(axis, axis.visibleLabels[j].Value);
            var labelText = axis.visibleLabels[j].Text;
            var x = this.model.centerX + aroundRadius * vector.X;
            var y = this.model.centerY + aroundRadius * vector.Y;
            var textAnchor = (x < this.model.centerX) ? 'end' : ((x > this.model.centerX) ? 'start' : 'middle');

            var size= ej.EjSvgRender.utils._measureText(labelText, this.model.m_AreaBounds.Height, axis.font);
            // calculation for positioning the label along the arc
            if (coef == 0.25)
            {
                x -= this.model.elementSpacing / 2;
                y += (size.height) / 4;
            }
           else if (coef == 0.5)
            {
               y += (size.height) ;
            }
           else if (coef == 0.75)
           {
               x += this.model.elementSpacing / 2;
               y += (size.height) / 4;
           }
           else if (coef == 1 || coef==0) {
               y -= (size.height) / 2;
           }
               
           else if (0.25 < coef && coef < 0.5) {
               x -= this.model.elementSpacing / 2;
               y += (size.height) / 2;
           }
           else if (0.5 < coef && coef <= 0.75) {
               x += this.model.elementSpacing / 2;
               y += (size.height) / 2;
           }
           else if (0 < coef && coef < 0.25)
             {
                   x -= this.model.elementSpacing / 2;
           }
           else
               x += this.model.elementSpacing / 2;


            var options = {
                'id': this.svgObject.id + '_' + axis.name + '_XLabel_' + j,
                'x': x,
                'y': y,
                'fill': font.color,
                'font-size': font.size,
                'font-family': font.fontFamily,
                'font-style': font.fontStyle,
                'font-weight': font.fontWeight,
                'opacity': font.opacity,
                'text-anchor': textAnchor
            };

            this.svgRenderer.drawText(options, labelText, gEle);

        }
        $(gEle).appendTo(this.gXaxisEle);

    },
    _drawPolarCircle: function (axis) {
        if (axis.majorGridLines.visible) {
            var sbYMajorGrid = ej.EjSvgRender.utils._getStringBuilder();
            var gEle = this.svgRenderer.createGroup({ 'id': this.svgObject.id + '_YAxisGrid' });
            if (this.model.isPolar) {
                for (var j = 0; j < axis.visibleLabels.length; j++) {
                    var label = axis.visibleLabels[j];
                    var radius = this.model.Radius * ej.EjSvgRender.utils._valueToCoefficient(axis, label.Value, this);

                    var options = {
                        'id': this.svgObject.id + '_YAxisGridLines_' + j,
                        'cx': this.model.centerX,
                        'cy': this.model.centerY,
                        'r': radius,
                        'fill': 'transparent',
                        'stroke-width': axis.majorGridLines.width,
                        'stroke': axis.majorGridLines.color,
                         'opacity': (this.vmlRendering) ? 0.3: axis.majorGridLines.opacity
                    };

                    //Drawing Major Grid Lines for polar
                    this.svgRenderer.drawCircle(options, gEle);
                   
                }
            }
            else {
                for (var j = 0; j < axis.visibleLabels.length; j++) {
                    
                        var label = axis.visibleLabels[j];
                        var radius = this.model.Radius * ej.EjSvgRender.utils._valueToCoefficient(axis, label.Value, this);
                        // calculation for spider web
                        for (var i = 0; i < this.model.axes[0].visibleLabels.length; i++) {
                            var xAxis = this.model.axes[0];
                            var vector = ej.EjSvgRender.utils._valueToVector(xAxis, xAxis.visibleLabels[i].Value);
                            var vector2;
                            if ((i + 1) < xAxis.visibleLabels.length) {
                                vector2 = ej.EjSvgRender.utils._valueToVector(xAxis, xAxis.visibleLabels[i + 1].Value);
                            }
                            else {
                                vector2 = ej.EjSvgRender.utils._valueToVector(xAxis, xAxis.visibleLabels[0].Value);
                            }
                            var connectPoint = { X: this.model.centerX + radius * vector.X, Y: this.model.centerY + radius * vector.Y };
                            var endPoint = { X: this.model.centerX + radius * vector2.X, Y: this.model.centerY + radius * vector2.Y };
                            sbYMajorGrid.append("M" + " " + connectPoint.X + " " + connectPoint.Y + " " + "L" + " " + endPoint.X + " " + endPoint.Y + " ");
                        }
                }
                if (axis.majorGridLines.visible) {
                    yMajorGridDir = sbYMajorGrid.toString();
                    var options = {
                        'id': this.svgObject.id + '_YAxisGridLines',
                        'fill': 'none',
                        'stroke-width': axis.majorGridLines.width,
                        'stroke': axis.majorGridLines.color,
                        'opacity': axis.majorGridLines.opacity,
                        'stroke-dasharray': axis.majorGridLines.dashArray,
                        'd': yMajorGridDir
                    };
                    //Drawing Major Grid Lines for radar
                    this.svgRenderer.drawPath(options, gEle);
                }
            }
        }
        $(gEle).appendTo(this.gYaxisEle);

    },
    _drawPolarGridLine: function (axis) {

        var legXSpace = 0;
        var legYSpace = 0;

        var sbXMajorGrid = ej.EjSvgRender.utils._getStringBuilder();
        var sbXMajorTick = ej.EjSvgRender.utils._getStringBuilder();

        var legend = this.model.legend;
        if (legend.visible && legend.position.toLowerCase() != "custom") {
            if (legend.position.toLowerCase() == "right" || legend.position.toLowerCase() == "left")
                legXSpace = ((legend.position.toLowerCase() == "right") ? this.model.margin.right : this.model.margin.left)  + this.model.LegendViewerBounds.Width;
            else
                legYSpace = ((legend.position.toLowerCase() == "top") ? this.model.margin.top : this.model.margin.bottom)  + this.model.LegendViewerBounds.Height;

        }
        // calculating the radius of the chart with avaible size
        var yOffset = ((this.model.title.text) ? this.model._titleLocation.Y : 0) + legYSpace;

        var actualWidth = $(this.svgObject).width() - legXSpace;
        var actualHeight = $(this.svgObject).height() - yOffset;

        this.model.centerX = actualWidth * 0.5 + ((legend.position.toLowerCase() === "left") ? legXSpace : 0);
        this.model.centerY = actualHeight * 0.5 + ((legend.position.toLowerCase() === "top") ? yOffset : ((this.model.title.text) ? (this.model._titleLocation.Y) : 0));

        var gEle = this.svgRenderer.createGroup({ 'id': this.svgObject.id + '_XAxisGrid' });
        this.model.finalSize = { width: actualWidth, height: actualHeight };

        this.model.Radius = Math.min(this.model.finalSize.width, this.model.finalSize.height) / 2- (2*this.model.elementSpacing)-axis.majorTickLines.size-axis._LableMaxWidth.maxHeight;
       
        var pos = 0;
        var isRadar = false;
        
        if (axis.majorGridLines.visible)
        {
            
            for (var j = 0; j < axis.visibleLabels.length; j++)
            {

            var vector = ej.EjSvgRender.utils._valueToVector(axis, axis.visibleLabels[j].Value);
            var line = {};
            line.X1 = this.model.centerX;

            line.Y1 = this.model.centerY;
            line.X2 = this.model.centerX + this.model.Radius * vector.X;
            line.Y2 = this.model.centerY + this.model.Radius * vector.Y;
            sbXMajorGrid.append("M" + " " + line.X1 + " " + line.Y1 + " " + "L" + " " + line.X2 + " " + line.Y2 + " ");
            }

        }
        if (axis.majorTickLines.visible) {
            var radius=this.model.Radius;
            for (var j = 0; j < axis.visibleLabels.length; j++)
            {
            var vector = ej.EjSvgRender.utils._valueToVector(axis, axis.visibleLabels[j].Value);
            var line = {};
            line.X1 = this.model.centerX + radius * vector.X;
            line.Y1 =this.model.centerY + radius * vector.Y;
            line.X2 = line.X1 + axis.majorTickLines.size * vector.X;
            line.Y2 = line.Y1 + axis.majorTickLines.size * vector.Y;
            sbXMajorTick.append("M" + " " + line.X1 + " " + line.Y1 + " " + "L" + " " + line.X2 + " " + line.Y2 + " ");
           }
        }
    

   
        if (axis.majorGridLines.visible) {
            xMajorGridDir = sbXMajorGrid.toString();
            var options = {
                'id': this.svgObject.id + '_XAxisGridLines',
                'fill': 'none',
                'stroke-width': axis.majorGridLines.width,
                'stroke': axis.majorGridLines.color,
                'opacity': axis.majorGridLines.opacity,
                'stroke-dasharray': axis.majorGridLines.dashArray,
                'd': xMajorGridDir
            };
            //Drawing XAxis Major Grid Lines
            this.svgRenderer.drawPath(options, gEle);
            $(gEle).appendTo(this.gXaxisEle);
        }

            if (axis.majorTickLines.visible) {
                xMajorTickDir = sbXMajorTick.toString();
                var options = {
                    'id': this.svgObject.id + '_XAxisMajorTicks',
                    'fill': 'none',
                    'stroke-width': axis.majorTickLines.width,
                    'stroke': axis.majorTickLines.color,
                    'd': xMajorTickDir
                };
                //Drawing XAxis Major Ticks Lines
                this.svgRenderer.drawPath(options, gEle);
                $(gEle).appendTo(this.gXaxisEle);
            }       
    },
    _drawXAxisTickLine:function(axisIndex,xAxis,gEle)
    {
        var xMajorTicksDir, xMinorTicksDir;
        var sbXMinorTicks = ej.EjSvgRender.utils._getStringBuilder();       
        var sbXMajorTicks = ej.EjSvgRender.utils._getStringBuilder();       
        var minorPointX;
        var x1 = Math.floor(xAxis.x);
        var x2 = Math.floor(xAxis.x + xAxis.width);

        if (axisIndex != 0 && ((xAxis.labelPosition == 'inside' && xAxis.tickLinesPosition == 'inside') || (xAxis.labelPosition == 'inside' && xAxis.tickLinesPosition != 'inside')) && (!(xAxis.y == (this.model.m_AreaBounds.Height + this.model.m_AreaBounds.Y) || (xAxis.y == this.model.m_AreaBounds.Y))))
        {
            var y1 = (!xAxis.opposedPosition) ? Math.floor(xAxis.y + xAxis._LableMaxWidth.height) : Math.floor(xAxis.y - xAxis._LableMaxWidth.height);
            var y2 = (!xAxis.opposedPosition) ? Math.floor(xAxis.y + xAxis._LableMaxWidth.height) : Math.floor(xAxis.y - xAxis._LableMaxWidth.height);
        }
        else if (axisIndex != 0 && (xAxis.labelPosition != 'inside' && xAxis.tickLinesPosition == 'inside') && (!(xAxis.y == (this.model.m_AreaBounds.Height + this.model.m_AreaBounds.Y) || (xAxis.y == this.model.m_AreaBounds.Y)))) {
            var y1 = (!xAxis.opposedPosition) ? Math.floor(xAxis.y + xAxis.majorTickLines.size) : Math.floor(xAxis.y - xAxis.majorTickLines.size);
            var y2 = (!xAxis.opposedPosition) ? Math.floor(xAxis.y + xAxis.majorTickLines.size) : Math.floor(xAxis.y - xAxis.majorTickLines.size);
        }

        else
        {
            var y1 = Math.floor(xAxis.y);
            var y2 = Math.floor(xAxis.y);
        }
        if (!gEle)
        var gTickEle = this.svgRenderer.createGroup({ 'id': this.svgObject.id + '_XAxisTicklines' + '_' + axisIndex });
        xAxis.labelPlacement = (!(xAxis.labelPlacement)) ? ej.datavisualization.Chart.LabelPlacement.BetweenTicks : xAxis.labelPlacement;
        var ticksbwtLabel = ((xAxis.labels.length > 1) && (xAxis.labelPlacement.toLowerCase() == "betweenticks")) ? -0.5 : 0;
        var collectionLength = xAxis.visibleLabels.length;
        //For between ticks and gridlines
        if ((xAxis._valueType.toLowerCase() == ej.datavisualization.Chart.ValueType.Category) && ticksbwtLabel < 0) {
            collectionLength = xAxis.labels.length + 1;
        }
        for (var i = 0; i < collectionLength; i++) {

            var tempInterval = ((xAxis.labels.length > 1) && (ticksbwtLabel < 0)) ? i + ticksbwtLabel : xAxis.visibleLabels[i].Value;

            tempInterval = (!(xAxis.roundingPlaces)) ? parseFloat(tempInterval.toFixed((ej.EjSvgRender.utils._decimalPlaces(xAxis.visibleRange.interval) == 0 ? 1 : ej.EjSvgRender.utils._decimalPlaces(xAxis.visibleRange.interval)))) : parseFloat(tempInterval.toFixed(xAxis.roundingPlaces));

            if (ej.EjSvgRender.utils._inside(tempInterval, xAxis.visibleRange)) {

                if ((xAxis.hidePartialLabels) && ((i == 0) || (i == (collectionLength - 1))))
                    continue;
                if (xAxis.minorGridLines.visible || xAxis.minorTickLines.visible) {
                    var ticksVal = this._getSharpPath(xAxis.minorTickLines.width);
                    var linesVal = this._getSharpPath(xAxis.minorGridLines.width);
                    var yTickPosition;
                    if (xAxis._valueType == "logarithmic") {
                        minorTicks = tempInterval;
                        var logmax = xAxis.visibleRange.max;
                        var logmin = xAxis.visibleRange.min;

                        var logtickstart = Math.pow(xAxis.logBase, minorTicks - xAxis.visibleRange.interval);
                        var logtickend = Math.pow(xAxis.logBase, minorTicks);
                        var logtickInterval = (logtickend - logtickstart) / (xAxis.minorTicksPerInterval + 1);
                        var logtickPos = logtickstart + logtickInterval;
                        minorTicks = ej.EjSvgRender.utils._logBase(logtickPos, xAxis.logBase);
                        for (var j = 0; j < xAxis.minorTicksPerInterval; j++) {


                            if (minorTicks < logmax && minorTicks > logmin) {

                                minorPointX = Math.ceil(((minorTicks - logmin) / (logmax - logmin)) * (xAxis.width));

                                //Calculate Minor Ticks
                                if ((xAxis.minorTickLines) && xAxis.minorTickLines.visible) {
                                    var xTickPosition = Math.floor(minorPointX + x1);
                                    if ((xAxis.tickLinesPosition != 'inside' && xAxis.labelPosition != 'inside') || (xAxis.tickLinesPosition != 'inside' && xAxis.labelPosition == 'inside'))
                                        yTickPosition = (xAxis.opposedPosition) ? Math.floor(y1 - xAxis.minorTickLines.size) : Math.floor(xAxis.minorTickLines.size + y1);
                                    else
                                        yTickPosition = (xAxis.opposedPosition) ? Math.floor(y1 + xAxis.minorTickLines.size) : y1 - Math.floor(xAxis.minorTickLines.size);
                                    sbXMinorTicks.append("M" + " " + (xTickPosition + ticksVal) + " " + (yTickPosition + ticksVal) + " " + "L" + " " + (xTickPosition + ticksVal) + " " + (y1 + ticksVal) + " ");
                                }
                            }
                        }
                    }
                    else {
                        for (var j = 0, minorTicks = tempInterval ; j < xAxis.minorTicksPerInterval; j++) {
                            minorTicks += xAxis.visibleRange.interval / (xAxis.minorTicksPerInterval + 1);
                            if (minorTicks < xAxis.visibleRange.max && minorTicks > xAxis.visibleRange.min) {
                                minorPointX = Math.ceil(((minorTicks - xAxis.visibleRange.min) / (xAxis.visibleRange.max - xAxis.visibleRange.min)) * (xAxis.width));

                                //Calculate Minor Ticks
                                if ((xAxis.minorTickLines) && xAxis.minorTickLines.visible) {
                                    if ((xAxis.tickLinesPosition != 'inside' && xAxis.labelPosition != 'inside') || (xAxis.tickLinesPosition != 'inside' && xAxis.labelPosition == 'inside'))
                                        yTickPosition = (xAxis.opposedPosition) ? Math.floor(y1 - xAxis.minorTickLines.size) : Math.floor(xAxis.minorTickLines.size + y1);
                                    else
                                        yTickPosition = (xAxis.opposedPosition) ? Math.floor(y1 + xAxis.minorTickLines.size) : Math.floor(y1 - xAxis.minorTickLines.size);
                                    var points = Math.floor(minorPointX + x1);
                                    sbXMinorTicks.append("M" + " " + (points + ticksVal) + " " + (yTickPosition + ticksVal) + " " + "L" + " " + (points + ticksVal) + " " + (y1 + ticksVal) + " ");
                                }
                            }
                        }
                    }

                }

                //calculate major ticks
                if (xAxis.majorTickLines.visible || xAxis.majorGridLines.visible) {
                    var mtVal = this._getSharpPath(xAxis.majorTickLines.width);
                    var mlVal = this._getSharpPath(xAxis.majorGridLines.width);
                    //Calculate Major Ticks and Grid lines
                    var pointX = this.pointX = Math.ceil(ej.EjSvgRender.utils._getPointXY(tempInterval, xAxis.visibleRange, xAxis.isInversed) * (xAxis.width));
                    //Calculate Major Ticks
                    if (xAxis.majorTickLines.visible) {
                     
                        var lineStart = xAxis.majorTickLines.size + xAxis.axisLine.width / 2;
                        if ((xAxis.tickLinesPosition == 'inside' && xAxis.labelPosition == 'inside') || (xAxis.tickLinesPosition == 'inside' && xAxis.labelPosition != 'inside'))
                            var yPosition = (xAxis.opposedPosition) ? y1 + lineStart : y1 - lineStart;
                        else
                            var yPosition = (xAxis.opposedPosition) ? y1 - lineStart : lineStart + y1;
                        sbXMajorTicks.append("M" + " " + (Math.floor(pointX + x1) + mtVal) + " " + (Math.floor(yPosition) + mtVal) + " " + "L" + " " + (Math.floor(pointX + x1) + mtVal) + " " + (Math.floor(y1 + ((xAxis.opposedPosition) ? (-xAxis.axisLine.width / 2) : (xAxis.axisLine.width / 2))) + mtVal) + " ");
                    }
                }
            }
        }
        //draw minor ticks
        if (xAxis.minorTickLines.visible && xAxis.visible && xAxis.minorTicksPerInterval > 0) {
            xMinorTicksDir = sbXMinorTicks.toString();
            var optionsMinorTicks = {
                'id': this.svgObject.id + '_XAxisMinorTicks_' + axisIndex,
                'fill': 'none',
                'stroke-width': xAxis.minorTickLines.width,
                'stroke': xAxis.minorTickLines.color,
                'd': xMinorTicksDir
            };
            if(!gEle)
                this.svgRenderer.drawPath(optionsMinorTicks, gTickEle);
            else
                this.svgRenderer.drawPath(optionsMinorTicks, gEle);

        }
        if (xAxis.majorTickLines.visible && xAxis.visible) {
            xMajorTicksDir = sbXMajorTicks.toString();
            var optionsMajorTicks = {
                'id': this.svgObject.id + '_XAxisMajorTicks_' + axisIndex,
                'fill': 'none',
                'stroke-width': xAxis.majorTickLines.width,
                'stroke': xAxis.majorTickLines.color,
                'd': xMajorTicksDir
            };
            if (!gEle) {
                
                this.svgRenderer.drawPath(optionsMajorTicks, gTickEle);
            }
            else
                this.svgRenderer.drawPath(optionsMajorTicks, gEle);
        }
        if(xAxis.tickLinesPosition=='inside')
            $(gTickEle).appendTo(this.gXaxisTickEle);
        else
            $(gEle).appendTo(this.gXaxisEle);

    },
    _drawXAxisGridLine: function (axisIndex, xAxis) {
         var xMajorGridDir, xMinorGridDir;        
        var sbXMinorGrid = ej.EjSvgRender.utils._getStringBuilder();        
        var sbXMajorGrid = ej.EjSvgRender.utils._getStringBuilder();
        var minorPointX;
        var xPoint = [];

        var x1 = Math.floor(xAxis.x);
        var x2 = Math.floor(xAxis.x + xAxis.width);
        
        if (axisIndex != 0 && (xAxis.labelPosition == 'inside' && xAxis.tickLinesPosition == 'inside') && (!(xAxis.y == (this.model.m_AreaBounds.Height + this.model.m_AreaBounds.Y) || (xAxis.y == this.model.m_AreaBounds.Y))))
        {
            var y1 = (!xAxis.opposedPosition) ? Math.floor(xAxis.y + xAxis._LableMaxWidth.height) : Math.floor(xAxis.y - xAxis._LableMaxWidth.height);
            var y2 = (!xAxis.opposedPosition) ? Math.floor(xAxis.y + xAxis._LableMaxWidth.height) : Math.floor(xAxis.y - xAxis._LableMaxWidth.height);
        }
        else if((xAxis.labelPosition == 'inside' && xAxis.tickLinesPosition != 'inside') && (!(xAxis.y == (this.model.m_AreaBounds.Height + this.model.m_AreaBounds.Y) || (xAxis.y == this.model.m_AreaBounds.Y))))
        {
            var y1 = (!xAxis.opposedPosition) ? Math.floor(xAxis.y + xAxis._LableMaxWidth.height) : Math.floor(xAxis.y - xAxis._LableMaxWidth.height);
            var y2 = (!xAxis.opposedPosition) ? Math.floor(xAxis.y + xAxis._LableMaxWidth.height) : Math.floor(xAxis.y - xAxis._LableMaxWidth.height);
        }
        else if (axisIndex != 0 && (xAxis.labelPosition != 'inside' && xAxis.tickLinesPosition == 'inside') && (!(xAxis.y == (this.model.m_AreaBounds.Height + this.model.m_AreaBounds.Y) || (xAxis.y == this.model.m_AreaBounds.Y)))) {
            var y1 = (!xAxis.opposedPosition) ? Math.floor(xAxis.y + xAxis.majorTickLines.size) : Math.floor(xAxis.y - xAxis.majorTickLines.size);
            var y2 = (!xAxis.opposedPosition) ? Math.floor(xAxis.y + xAxis.majorTickLines.size) : Math.floor(xAxis.y - xAxis.majorTickLines.size);
        }
        else
        {
            var y1 = Math.floor(xAxis.y);
            var y2 = Math.floor(xAxis.y);
        }
       
        var gEle = this.svgRenderer.createGroup({ 'id': this.svgObject.id + '_XAxisGrid' + '_' + axisIndex });
        var gAgb = this.svgRenderer.createGroup({ 'id': this.svgObject.id + '_XAxisAlternateGridBand' + '_' + axisIndex });

        if (xAxis.axisLine.visible) {
            var offset = xAxis.axisLine.offset;
            var val=this._getSharpPath(xAxis.axisLine.width);
            var optionsLine = {
                'id': this.svgObject.id + '_XAxisLine',
                x1: (x1 - xAxis.plotOffset)+ val + offset ,
                y1: y1+val,
                x2: (x2 + xAxis.plotOffset) -offset+val,
                y2: y2+val,
                'stroke-dasharray': xAxis.axisLine.dashArray,
                'stroke-width': xAxis.axisLine.width,
                'stroke': xAxis.axisLine.color,
                'opacity': (this.vmlRendering) ? 0.1 : xAxis.axisLine.opacity
            };

            this.svgRenderer.drawLine(optionsLine, gEle);
        }
        if (xAxis.tickLinesPosition != 'inside') {
            this._drawXAxisTickLine(axisIndex, xAxis, gEle);
        }
        xAxis.labelPlacement = (!(xAxis.labelPlacement)) ? ej.datavisualization.Chart.LabelPlacement.BetweenTicks : xAxis.labelPlacement;
        var ticksbwtLabel = ((xAxis.labels.length > 1) && (xAxis.labelPlacement.toLowerCase() == "betweenticks")) ? -0.5 : 0;
        var collectionLength = xAxis.visibleLabels.length;
        //For between ticks and gridlines
        if ((xAxis._valueType.toLowerCase() == ej.datavisualization.Chart.ValueType.Category) && ticksbwtLabel < 0) {
            collectionLength = xAxis.labels.length + 1;
        }
        for (var i = 0; i < collectionLength; i++) {

           var tempInterval = ((xAxis.labels.length > 1) && (ticksbwtLabel < 0)) ? i + ticksbwtLabel : xAxis.visibleLabels[i].Value;

            tempInterval = (!(xAxis.roundingPlaces)) ? parseFloat(tempInterval.toFixed((ej.EjSvgRender.utils._decimalPlaces(xAxis.visibleRange.interval) == 0 ? 1 : ej.EjSvgRender.utils._decimalPlaces(xAxis.visibleRange.interval)))) : parseFloat(tempInterval.toFixed(xAxis.roundingPlaces));

            if (ej.EjSvgRender.utils._inside(tempInterval, xAxis.visibleRange)) {

                if ((xAxis.hidePartialLabels) && ((i == 0) || (i == (collectionLength - 1))))
                    continue;
            
                //Calculate Minor Ticks and Grid lines
                if (xAxis.minorGridLines.visible || xAxis.minorTickLines.visible) {
                    var ticksVal = this._getSharpPath(xAxis.minorTickLines.width);
                    var linesVal = this._getSharpPath(xAxis.minorGridLines.width);
                    var yTickPosition;
                    if (xAxis._valueType == "logarithmic"){
                    minorTicks = tempInterval;
                    var logmax = xAxis.visibleRange.max;
                    var logmin = xAxis.visibleRange.min;

                    var logtickstart = Math.pow(xAxis.logBase, minorTicks - xAxis.visibleRange.interval);
                    var logtickend = Math.pow(xAxis.logBase, minorTicks);
                    var logtickInterval = (logtickend - logtickstart) / (xAxis.minorTicksPerInterval + 1);
                    var logtickPos = logtickstart + logtickInterval;
                    minorTicks = ej.EjSvgRender.utils._logBase(logtickPos, xAxis.logBase);
                    for (var j = 0; j < xAxis.minorTicksPerInterval; j++) {


                        if (minorTicks < logmax && minorTicks > logmin) {

                            minorPointX = Math.ceil(((minorTicks - logmin) / (logmax - logmin)) * (xAxis.width));

                           

                            //Calculate Minor Gridlines
                            if ((xAxis.minorGridLines) && xAxis.minorGridLines.visible) {
                                var tickPosition = Math.floor(minorPointX + x1) ;
                                sbXMinorGrid.append("M" + " " + (tickPosition + linesVal) + " " + (y1 + linesVal) + " " + "L" + " " + (tickPosition + linesVal) + " " + (Math.floor(this.model.m_AreaBounds.Y) + linesVal) + " ");
                            }

                        }
                        logtickPos += logtickInterval;
                        minorTicks = ej.EjSvgRender.utils._logBase(logtickPos, xAxis.logBase);
                    }
                    } else {
                        for (var j = 0, minorTicks = tempInterval ; j < xAxis.minorTicksPerInterval; j++) {
                            minorTicks += xAxis.visibleRange.interval / (xAxis.minorTicksPerInterval + 1);
                            if (minorTicks < xAxis.visibleRange.max && minorTicks > xAxis.visibleRange.min) {
                                minorPointX = Math.ceil(((minorTicks - xAxis.visibleRange.min) / (xAxis.visibleRange.max - xAxis.visibleRange.min)) * (xAxis.width));


                                //Calculate Minor Gridlines
                                if ((xAxis.minorGridLines) && xAxis.minorGridLines.visible)
                                    sbXMinorGrid.append("M" + " " + (Math.floor(minorPointX + x1) + linesVal) + " " + (y1 + linesVal) + " " + "L" + " " + (Math.floor(minorPointX + x1) + linesVal) + " " + (Math.floor(this.model.m_AreaBounds.Y) + linesVal) + " ");
                            }
                        }
                    }
            }

                if (xAxis.majorTickLines.visible || xAxis.majorGridLines.visible) {
                    var mtVal = this._getSharpPath(xAxis.majorTickLines.width);
                    var mlVal = this._getSharpPath(xAxis.majorGridLines.width);
                    var pointX = this.pointX = Math.ceil(ej.EjSvgRender.utils._getPointXY(tempInterval, xAxis.visibleRange, xAxis.isInversed) * (xAxis.width));
                    var mX1 = (Math.floor(pointX + x1));
                    if (xAxis.zoomed)
                        xPoint[0] = xAxis.x;
                    if (i == 0) {
                        if (mX1 == Math.floor(this.model.m_AreaBounds.X)) {
                            xPoint.push(mX1 + mlVal);
                            continue;
                            
                        }
                    }
                    if ((i == (collectionLength - 1)) && (this.model.chartArea.border.color != "transparent" && this.model.chartArea.border.width > 0)) {
                         if (mX1 == Math.floor(this.model.m_AreaBounds.X + this.model.m_AreaBounds.Width))
                             continue;
                         if (xAxis.zoomed)
                             xPoint[collectionLength] = mX1 + mlVal;
                       
                    }
                     
                    //Calculate Major Gridlines
                    if (xAxis.majorGridLines.visible)
                        sbXMajorGrid.append("M" + " " + (mX1 + mlVal) + " " + (Math.floor(this.model.m_AreaBounds.Y + this.model.m_AreaBounds.Height) + mlVal) + " " + "L" + " " + (mX1 + mlVal) + " " + (Math.floor(this.model.m_AreaBounds.Y) + mlVal) + " ");
                    if (i == collectionLength - 1) 
                        continue;
                    xPoint.push((mX1 + mlVal));
                }

            }

        }

       
        if (xAxis.alternateGridBand.odd.fill != "transparent" || xAxis.alternateGridBand.even.fill != "transparent") {
            var height = this.model.m_AreaBounds.Height;
            
            for (var i = 1; i < xPoint.length; i += 2) {
                var width = xPoint[2] - xPoint[1];
                if (i == xPoint.length - 1 && xAxis.zoomed) {
                    var zoomWidth = this.model.m_AreaBounds.X + this.model.m_AreaBounds.Width - xPoint[xPoint.length - 1] - xAxis.plotOffset;
                    width = zoomWidth;
                }
                var optionsAlternateodd = {
                    'id': this.svgObject.id + '_XAxisAlternateodd' + [i],
                    'x': xPoint[i],
                    'y': Math.floor(this.model.m_AreaBounds.Y),
                    'width': width,
                    'height': height,
                    'fill': xAxis.alternateGridBand.odd.fill,
                    'opacity': xAxis.alternateGridBand.odd.opacity,
                    'stroke': "transparent",
                    'clip-path': 'url(#' + this.svgObject.id + '_ChartAreaClipRect)'

                };
                this.svgRenderer.drawRect(optionsAlternateodd, gAgb);
            }

            for (var i = 0; i < xPoint.length; i += 2) {
                var width = xPoint[2] - xPoint[1];
                if (i == 0 && xAxis.zoomed) {
                    var zoomWidth = xPoint[1] - xAxis.x;
                    width = zoomWidth;
                }
                if (i == xPoint.length - 1 && xAxis.zoomed) {
                    var zoomWidth = this.model.m_AreaBounds.X + this.model.m_AreaBounds.Width - xPoint[xPoint.length - 1]-xAxis.plotOffset;
                    width = zoomWidth;
                }
                var optionsAlternateeven = {
                    'id': this.svgObject.id + '_XAxisAlternateeven' + [i],
                    'x': xPoint[i],
                    'y': Math.floor(this.model.m_AreaBounds.Y),
                    'width': width,
                    'height': height,
                    'fill': xAxis.alternateGridBand.even.fill,
                    'opacity': xAxis.alternateGridBand.even.opacity,
                    'clip-path': 'url(#' + this.svgObject.id + '_ChartAreaClipRect)',
                    'stroke': "transparent"
                };
                this.svgRenderer.drawRect(optionsAlternateeven, gAgb);
            }
            $(gAgb).appendTo(gEle);
        }

        //Drawing Minor Ticks
        if (xAxis.minorTickLines.visible && xAxis.visible && xAxis.minorTicksPerInterval>0) {
            xMinorTicksDir = sbXMinorTicks.toString();
            var optionsMinorTicks = {
                'id': this.svgObject.id + '_XAxisMinorTicks',
                'fill': 'none',
                'stroke-width': xAxis.minorTickLines.width,
                'stroke': xAxis.minorTickLines.color,
                'd': xMinorTicksDir
            };

            this.svgRenderer.drawPath(optionsMinorTicks, gEle);

        }
        if (xAxis.minorGridLines.visible && xAxis.minorTicksPerInterval > 0) {
            xMinorGridDir = sbXMinorGrid.toString();
            //Drawing Minor Grid Lines
            var optionsMinorGrid = {
                'id': this.svgObject.id + '_XAxisMinorGridLines',
                'fill': 'none',
                'stroke-width': xAxis.minorGridLines.width,
                'stroke': xAxis.minorGridLines.color,
                'stroke-dasharray': xAxis.minorGridLines.dashArray,
                'd': xMinorGridDir
            };
            this.svgRenderer.drawPath(optionsMinorGrid, gEle);
        }       
        xMajorGridDir = sbXMajorGrid.toString();
        if (xAxis.majorGridLines.visible) {
            var options = {
                'id': this.svgObject.id + '_XAxisMajorGridLines',
                'fill': 'none',
                'stroke-width': xAxis.majorGridLines.width,
                'stroke': xAxis.majorGridLines.color,
                'opacity': xAxis.majorGridLines.opacity,
                'stroke-dasharray': xAxis.majorGridLines.dashArray,
                'd': xMajorGridDir
            };

            //Drawing Major Grid Lines
            if(options.d != '')
            this.svgRenderer.drawPath(options, gEle);

        }
        $(gEle).appendTo(this.gXaxisEle);      
         
        
    },

    _drawYAxisTickLine:function(axisIndex,yAxis,gEle)
    {
        var yMajorTicksDir = "", yMinorTicksDir = "";
        var pointY = 0;
        if (axisIndex != 1 && ((yAxis.labelPosition == 'inside' && yAxis.tickLinesPosition == 'inside') || (yAxis.labelPosition == 'inside' && yAxis.tickLinesPosition != 'inside')) && (!(yAxis.x == (this.model.m_AreaBounds.Width + this.model.m_AreaBounds.X) || (yAxis.x == (this.model.m_AreaBounds.X))))) {
            var x1 = (!yAxis.opposedPosition) ? Math.floor(yAxis.x - yAxis._LableMaxWidth.width - this.model.elementSpacing) : Math.floor(yAxis.x + yAxis._LableMaxWidth.width + this.model.elementSpacing);
            var x2 = (!yAxis.opposedPosition) ? Math.floor(yAxis.x - yAxis._LableMaxWidth.width - this.model.elementSpacing) : Math.floor(yAxis.x + yAxis._LableMaxWidth.width + this.model.elementSpacing);
        }
        else if (axisIndex != 1 && (yAxis.labelPosition != 'inside' && yAxis.tickLinesPosition == 'inside') && (!(yAxis.x == (this.model.m_AreaBounds.Width + this.model.m_AreaBounds.X) || (yAxis.x == (this.model.m_AreaBounds.X))))) {
            var x1 = (!yAxis.opposedPosition) ? Math.floor(yAxis.x - yAxis._LableMaxWidth.width - this.model.elementSpacing - yAxis.majorTickLines.size) : Math.floor(yAxis.x+yAxis.majorTickLines.size);
            var x2 = (!yAxis.opposedPosition) ? Math.floor(yAxis.x - yAxis._LableMaxWidth.width - this.model.elementSpacing - yAxis.majorTickLines.size) : Math.floor(yAxis.x + yAxis.majorTickLines.size);
        }
        else            
        {
            var x1 = Math.floor(yAxis.x);
            var x2 = Math.floor(yAxis.x);
        }   

        var y1 = Math.floor(yAxis.y + yAxis.height);
        var y2 = Math.floor(yAxis.y);       
    

        var sbYMinorTicks = ej.EjSvgRender.utils._getStringBuilder();   
        var sbYMajorTicks = ej.EjSvgRender.utils._getStringBuilder();      
        var gTickEle = this.svgRenderer.createGroup({ 'id': this.svgObject.id + '_YAxisTicklines' + '_' + axisIndex });
      
        for (var i = 0; i < yAxis.visibleLabels.length; i++)
        {
            var tempInterval = yAxis.visibleLabels[i].Value;
            tempInterval = (!(yAxis.roundingPlaces)) ? parseFloat(tempInterval.toFixed((ej.EjSvgRender.utils._decimalPlaces(yAxis.visibleRange.interval) == 0 ? 1 : ej.EjSvgRender.utils._decimalPlaces(yAxis.visibleRange.interval)))) : parseFloat(tempInterval.toFixed(yAxis.roundingPlaces));

            if (ej.EjSvgRender.utils._inside(tempInterval, yAxis.visibleRange))
            {
 
                if ((yAxis.hidePartialLabels) && ((i == 0) || (i == (yAxis.visibleLabels.length - 1))))
                    continue;

                // Calculate Minor Ticks and Grid lines      
                if (yAxis.minorGridLines.visible || yAxis.minorTickLines.visible) {
                    var minorPointY;
                    var tickVal = this._getSharpPath(yAxis.minorTickLines.width);
                    var lineVal = this._getSharpPath(yAxis.minorGridLines.width);
                    if (yAxis._valueType == "logarithmic") {
                        minorTicks = tempInterval;
                        var logmax = yAxis.visibleRange.max;
                        var logmin = yAxis.visibleRange.min;

                        var logtickstart = Math.pow(yAxis.logBase, minorTicks - yAxis.visibleRange.interval);
                        var logtickend = Math.pow(yAxis.logBase, minorTicks);
                        var logtickInterval = (logtickend - logtickstart) / (yAxis.minorTicksPerInterval + 1);
                        var logtickPos = logtickstart + logtickInterval;
                        minorTicks = ej.EjSvgRender.utils._logBase(logtickPos, yAxis.logBase);
                        for (var j = 0; j < yAxis.minorTicksPerInterval; j++) {


                            if (minorTicks < logmax && minorTicks > logmin) {

                                minorPointY = Math.ceil(((minorTicks - logmin) / (logmax - logmin)) * (yAxis.height));

                                //Calculate Minor Ticks
                                if ((yAxis.minorTickLines) && yAxis.minorTickLines.visible) {

                                    if (yAxis.minorTickLines.visible) {
                                        if ((yAxis.tickLinesPosition != 'inside' && yAxis.labelPosition != 'inside') || (yAxis.tickLinesPosition != 'inside' && yAxis.labelPosition == 'inside'))
                                            ytickPosition = (yAxis.opposedPosition) ? yAxis.minorTickLines.size : -yAxis.minorTickLines.size;
                                        else
                                            var ytickPosition = (yAxis.opposedPosition) ? -yAxis.minorTickLines.size : yAxis.minorTickLines.size;
                                        sbYMinorTicks.append("M" + " " + (Math.floor(ytickPosition + x1) + tickVal) + " " + (Math.floor((minorPointY * -1) + y1) + tickVal) + " " + "L" + " " + (x1 + tickVal) + " " + (Math.floor((minorPointY * -1) + y1) + tickVal) + " ");
                                    }
                                }                              
                            }
                            logtickPos += logtickInterval;
                            minorTicks = ej.EjSvgRender.utils._logBase(logtickPos, yAxis.logBase);
                        }
                    }
                    else {
                        for (var j = 0, minorTicks = tempInterval; j < yAxis.minorTicksPerInterval; j++) {

                            minorTicks += yAxis.visibleRange.interval / (yAxis.minorTicksPerInterval + 1);
                            if (minorTicks < yAxis.visibleRange.max && minorTicks > yAxis.visibleRange.min) {
                                minorPointY = ((minorTicks - yAxis.visibleRange.min) / (yAxis.visibleRange.delta)) * (yAxis.height);
                                // Calculate Minor Ticks   
                                if (yAxis.minorTickLines.visible) {
                                    if ((yAxis.tickLinesPosition != 'inside' && yAxis.labelPosition != 'inside') || (yAxis.tickLinesPosition != 'inside' && yAxis.labelPosition == 'inside'))
                                        var ytickPosition = (yAxis.opposedPosition) ? yAxis.minorTickLines.size : -yAxis.minorTickLines.size;
                                    else
                                        var ytickPosition = (yAxis.opposedPosition) ? -yAxis.minorTickLines.size : yAxis.minorTickLines.size;
                                    sbYMinorTicks.append("M" + " " + (Math.floor(ytickPosition + x1) + tickVal) + " " + (Math.floor((minorPointY * -1) + y1) + tickVal) + " " + "L" + " " + (x1 + tickVal) + " " + (Math.floor((minorPointY * -1) + y1) + tickVal) + " ");
                                }
                               
                            }
                        }
                    }
                }
                // Calculate Major Ticks and Grid lines
                if (yAxis.majorTickLines.visible || yAxis.majorGridLines.visible) {
                    var mtVal = this._getSharpPath(yAxis.minorTickLines.width);
                    var mlVal = this._getSharpPath(yAxis.majorGridLines.width);
                    // Calculate Major Ticks
                    if (yAxis.majorTickLines.visible)
                    {
                        pointY = ej.EjSvgRender.utils._getPointXY(tempInterval, yAxis.visibleRange, yAxis.isInversed) * (yAxis.height);
                        if ((yAxis.tickLinesPosition != 'inside' && yAxis.labelPosition != 'inside') || (yAxis.tickLinesPosition != 'inside' && yAxis.labelPosition == 'inside'))
                            var xPosition = (yAxis.opposedPosition) ? yAxis.majorTickLines.size + yAxis.axisLine.width / 2 : -(yAxis.majorTickLines.size + yAxis.axisLine.width / 2);
                        else if ((yAxis.tickLinesPosition == 'inside' && yAxis.labelPosition != 'inside') || (yAxis.tickLinesPosition == 'inside' && yAxis.labelPosition == 'inside'))
                            var xPosition = (yAxis.opposedPosition) ? -(yAxis.majorTickLines.size + yAxis.axisLine.width / 2) : (yAxis.majorTickLines.size + yAxis.axisLine.width / 2);

                        sbYMajorTicks.append("M" + " " + (Math.floor(xPosition + x1) + mtVal) + " " + (Math.floor((pointY * -1) + y1) + mtVal) + " " + "L" + " " + (Math.floor(x1 + ((yAxis.opposedPosition) ? (yAxis.axisLine.width / 2) : (yAxis.axisLine.width / 2))) + mtVal) + " " + (Math.floor((pointY * -1) + y1) + mtVal) + " ");

                    }


                   
                }
            }
        }

        //Drawning Minor Ticks
        if (yAxis.minorTickLines.visible && yAxis.visible && yAxis.minorTicksPerInterval > 0) {
            yMinorTicksDir = sbYMinorTicks.toString();
            var optionsMinotTick = {
                'id': this.svgObject.id + '_YAxisMinorTicks_'+axisIndex,
                'fill': 'none',
                'stroke-width': yAxis.minorTickLines.width,
                'stroke': yAxis.minorTickLines.color,
                'd': yMinorTicksDir
            };
            if(yAxis.tickLinesPosition=='inside')
                this.svgRenderer.drawPath(optionsMinotTick, gTickEle);
            else
                this.svgRenderer.drawPath(optionsMinotTick, gEle);
        }
       

        if (yAxis.majorTickLines.visible && yAxis.visible) {
            yMajorTicksDir = sbYMajorTicks.toString();
            //Drawning Major Ticks
            var optionsMajorTick = {
                'id': this.svgObject.id + '_YAxisMajorTicks_'+axisIndex,
                'fill': 'none',
                'stroke-width': yAxis.majorTickLines.width,
                'stroke': yAxis.majorTickLines.color,
                'd': yMajorTicksDir
            };
            if (yAxis.tickLinesPosition == 'inside')
                this.svgRenderer.drawPath(optionsMajorTick, gTickEle);
            else
                this.svgRenderer.drawPath(optionsMajorTick, gEle);
        }       
       if(!gEle)
           $(gTickEle).appendTo(this.gYaxisTickEle);
        else
           $(gEle).appendTo(this.gYaxisEle);

    },

    _drawYAxisGridLine: function (axisIndex, yAxis) {

        var yMajorGridDir = "", yMajorTicksDir = "", yMinorTicksDir = "", yMinorGridDir = "";
        var pointY = 0;
        
        var y1 = Math.floor(yAxis.y + yAxis.height);
        var y2 = Math.floor(yAxis.y);
        var yPoint = [];
        if (axisIndex != 1 && ((yAxis.labelPosition == 'inside' && yAxis.tickLinesPosition == 'inside') || (yAxis.labelPosition == 'inside' && yAxis.tickLinesPosition != 'inside')) && (!(yAxis.x == (this.model.m_AreaBounds.Width + this.model.m_AreaBounds.X) || (yAxis.x == (this.model.m_AreaBounds.X))))) {
            var x1 = (!yAxis.opposedPosition) ? Math.floor(yAxis.x - yAxis._LableMaxWidth.width - this.model.elementSpacing) : Math.floor(yAxis.x + yAxis._LableMaxWidth.width + this.model.elementSpacing);
            var x2 = (!yAxis.opposedPosition) ? Math.floor(yAxis.x - yAxis._LableMaxWidth.width - this.model.elementSpacing) : Math.floor(yAxis.x + yAxis._LableMaxWidth.width + this.model.elementSpacing);
        }
    else if(axisIndex != 1 && (yAxis.labelPosition != 'inside' && yAxis.tickLinesPosition == 'inside') && (!(yAxis.x == (this.model.m_AreaBounds.Width + this.model.m_AreaBounds.X) || (yAxis.x == (this.model.m_AreaBounds.X))))) 
        {
            var x1 = (!yAxis.opposedPosition) ? Math.floor(yAxis.x - yAxis.majorTickLines.size) : Math.floor(yAxis.x + yAxis.majorTickLines.size);
            var x2 = (!yAxis.opposedPosition) ? Math.floor(yAxis.x - yAxis.majorTickLines.size) : Math.floor(yAxis.x + yAxis.majorTickLines.size);
        }
        else {
            var x1 = Math.floor(yAxis.x);
            var x2 = Math.floor(yAxis.x);
        }             
        var sbYMinorGrid = ej.EjSvgRender.utils._getStringBuilder();        
        var sbYMajorGrid = ej.EjSvgRender.utils._getStringBuilder();
        var gEle = this.svgRenderer.createGroup({ 'id': this.svgObject.id + '_YAxisGrid' + '_' + axisIndex });
        var gAgb = this.svgRenderer.createGroup({ 'id': this.svgObject.id + '_YAxisAlternateGridBand' + '_' + axisIndex });

        if (yAxis.axisLine.visible) {
            var offset = yAxis.axisLine.offset;
            var val = this._getSharpPath(yAxis.axisLine.width);
            
            var optionsLine = {
                'id': this.svgObject.id + yAxis.name + '_YAxisLine',
                x1: x1+val,
                y1: (y2 - yAxis.plotOffset) + offset+val,
                x2: x2+val,
                y2: (y1 + yAxis.plotOffset) - offset+val,
                'stroke-width': yAxis.axisLine.width,
                'stroke-dasharray': yAxis.axisLine.dashArray,
                'stroke': yAxis.axisLine.color,
                'opacity': (this.vmlRendering) ? 0.1 :yAxis.axisLine.opacity
            };
            this.svgRenderer.drawLine(optionsLine, gEle);
        }
     
        if (yAxis.axisBottomLine) {
            if (yAxis.axisBottomLine.visible) {
                var optionsBottomLine = {
                    'id': this.svgObject.id + yAxis.name  + '_YAxisBottomLine',
                    x1: this.model.m_AreaBounds.X,
                    y1: y1,
                    x2: this.model.m_AreaBounds.X + this.model.m_AreaBounds.Width,
                    y2: y1,
                    'stroke-width': yAxis.axisBottomLine.width,
                    'stroke': yAxis.axisBottomLine.color
                };
                this.svgRenderer.drawLine(optionsBottomLine, gEle);
            }
        }
        if (yAxis.tickLinesPosition != 'inside') {
            this._drawYAxisTickLine(axisIndex, yAxis, gEle);
        }

        for (var i = 0; i < yAxis.visibleLabels.length; i++) {
            var tempInterval = yAxis.visibleLabels[i].Value;
            tempInterval = (!(yAxis.roundingPlaces)) ? parseFloat(tempInterval.toFixed((ej.EjSvgRender.utils._decimalPlaces(yAxis.visibleRange.interval) == 0 ? 1 : ej.EjSvgRender.utils._decimalPlaces(yAxis.visibleRange.interval)))) : parseFloat(tempInterval.toFixed(yAxis.roundingPlaces));

            if (ej.EjSvgRender.utils._inside(tempInterval, yAxis.visibleRange)) {

                if ((yAxis.hidePartialLabels) && ((i == 0) || (i == (yAxis.visibleLabels.length - 1))))
                    continue;

                // Calculate Minor Ticks and Grid lines      
                if (yAxis.minorGridLines.visible || yAxis.minorTickLines.visible) {
                    var minorPointY;
                    var tickVal = this._getSharpPath(yAxis.minorTickLines.width);
                    var lineVal = this._getSharpPath(yAxis.minorGridLines.width);
                    if (yAxis._valueType == "logarithmic") {
                        minorTicks = tempInterval;
                        var logmax = yAxis.visibleRange.max;
                        var logmin = yAxis.visibleRange.min;

                        var logtickstart = Math.pow(yAxis.logBase, minorTicks - yAxis.visibleRange.interval);
                        var logtickend = Math.pow(yAxis.logBase, minorTicks);
                        var logtickInterval = (logtickend - logtickstart) / (yAxis.minorTicksPerInterval + 1);
                        var logtickPos = logtickstart + logtickInterval;
                        minorTicks = ej.EjSvgRender.utils._logBase(logtickPos, yAxis.logBase);
                        for (var j = 0; j < yAxis.minorTicksPerInterval; j++) {


                            if (minorTicks < logmax && minorTicks > logmin) {

                                minorPointY = Math.ceil(((minorTicks - logmin) / (logmax - logmin)) * (yAxis.height));

                            
                              
                                //Calculate Minor Gridlines
                                if (yAxis.minorGridLines.visible)
                                    sbYMinorGrid.append("M" + " " + (Math.floor(this.model.m_AreaBounds.X) + lineVal) + " " + (Math.floor((minorPointY * -1) + y1) + lineVal) + " " + "L" + " " + (Math.floor(this.model.m_AreaBounds.Width + this.model.m_AreaBounds.X) + lineVal) + " " + (Math.floor((minorPointY * -1) + y1) + lineVal) + " ");

                            }
                            logtickPos += logtickInterval;
                            minorTicks = ej.EjSvgRender.utils._logBase(logtickPos,yAxis.logBase);
                        }
                    } else {
                        for (var j = 0, minorTicks = tempInterval; j < yAxis.minorTicksPerInterval; j++) {

                            minorTicks += yAxis.visibleRange.interval / (yAxis.minorTicksPerInterval + 1);
                            if (minorTicks < yAxis.visibleRange.max && minorTicks > yAxis.visibleRange.min) {
                                  minorPointY = ((minorTicks - yAxis.visibleRange.min) / (yAxis.visibleRange.delta)) * (yAxis.height);
                              
                                // Calculate Minor Grid lines   
                                if (yAxis.minorGridLines.visible)
                                    sbYMinorGrid.append("M" + " " + (Math.floor(this.model.m_AreaBounds.X) + lineVal) + " " + (Math.floor((minorPointY * -1) + y1) + lineVal) + " " + "L" + " " + (Math.floor(this.model.m_AreaBounds.Width + this.model.m_AreaBounds.X) + lineVal) + " " + (Math.floor((minorPointY * -1) + y1) + lineVal) + " ");
                            }
                        }
                    }
                } 
                // Calculate Grid lines
                if (yAxis.majorTickLines.visible || yAxis.majorGridLines.visible) {
                    var mtVal = this._getSharpPath(yAxis.minorTickLines.width);
                    var mlVal = this._getSharpPath(yAxis.majorGridLines.width);
                   
                    // Calculate Major Grid lines
                    if (yAxis.majorGridLines.visible) {
                        pointY = ej.EjSvgRender.utils._getPointXY(tempInterval, yAxis.visibleRange, yAxis.isInversed) * (yAxis.height);
                        var mX1 = (Math.floor(this.model.m_AreaBounds.X) + mlVal);
                        var mY1 = (Math.floor((pointY * -1) + y1) );
                        var mX2 = (Math.floor(this.model.m_AreaBounds.Width + this.model.m_AreaBounds.X) + mlVal);
                        if (i == 0) {
                            if (mY1==Math.floor(this.model.m_AreaBounds.Y + this.model.m_AreaBounds.Height))
                                continue;
                            if (yAxis.zoomed)
                                yPoint[0] = mY1 + mlVal;
                            
                        }
                        if ((i == (yAxis.visibleLabels.length - 1)) && (this.model.chartArea.border.color != "transparent" && this.model.chartArea.border.width > 0)) {
                           
                            if (mY1 == (Math.floor(this.model.m_AreaBounds.Y))) {
                                yPoint.push(mY1 + mlVal);
                                continue;
                            }

                        }
                       
                        

                        sbYMajorGrid.append("M" + " " + mX1 + " " + (mY1 + mlVal) + " " + "L" + " " + mX2 + " " + (mY1 + mlVal) + " ");
                        if (i == 0) 
                            continue;
                        
                        yPoint.push(mY1 + mlVal);
                        
                    }
                }
            }
        }
        
        if (yAxis.alternateGridBand.odd.fill != "transparent" || yAxis.alternateGridBand.even.fill != "transparent") {
           
            var width = this.model.m_AreaBounds.Width;
            if (yAxis.zoomed) {
                yPoint[yPoint.length] = yAxis.y;
                
            }
            for (var i = 1; i < yPoint.length; i += 2) {
                var height = yPoint[0] - yPoint[1];
                if (i == yPoint.length - 1 && yAxis.zoomed) {
                    var zoomHeight = yPoint[yPoint.length - 2] - yAxis.y;
                    height = zoomHeight;
                }
                var yoptionsAlternateodd = {
                    'id': this.svgObject.id + '_YAxisAlternateodd' + [i],
                    'x': Math.floor(this.model.m_AreaBounds.X),
                    'y': yPoint[i],
                    'width': width,
                    'height': height,
                    'fill': yAxis.alternateGridBand.odd.fill,
                    'opacity': yAxis.alternateGridBand.odd.opacity,
                    'stroke' : "transparent",
                    'clip-path': 'url(#' + this.svgObject.id + '_ChartAreaClipRect)'
                };
                this.svgRenderer.drawRect(yoptionsAlternateodd, gAgb);
            }
            
            for (var i = 0; i < yPoint.length; i += 2) {
                var height = yPoint[0] - yPoint[1];
                if (i == yPoint.length - 1 && yAxis.zoomed) {
                    var zoomHeight = yPoint[yPoint.length - 2] - yAxis.y;
                    height = zoomHeight;
                }
                if (i == 0 && yAxis.zoomed) {
                    var zoomHeight = this.model.m_AreaBounds.Y + this.model.m_AreaBounds.Height - yPoint[0] - yAxis.plotOffset;
                    height = zoomHeight;
                }
                var yoptionsAlternateeven = {
                    'id': this.svgObject.id + '_YAxisAlternateeven' + [i],
                    'x': Math.floor(this.model.m_AreaBounds.X),
                    'y': yPoint[i],
                    'width': width,
                    'height': height,
                    'fill': yAxis.alternateGridBand.even.fill,
                    'opacity': yAxis.alternateGridBand.even.opacity,
                    'stroke': "transparent",
                    'clip-path': 'url(#' + this.svgObject.id + '_ChartAreaClipRect)'
                };
                this.svgRenderer.drawRect(yoptionsAlternateeven, gAgb);
            }
            $(gAgb).appendTo(gEle);
        }

     
        if (yAxis.minorGridLines.visible && yAxis.minorTicksPerInterval > 0) {
            //Drawning Minor Grid Lines
            yMinorGridDir = sbYMinorGrid.toString();
            var optionsMinorGrid = {
                'id': this.svgObject.id + '_YAxisMinorGridLines',
                'fill': 'none',
                'stroke-width': yAxis.minorGridLines.width,
                'stroke': yAxis.minorGridLines.color,
                'stroke-dasharray': yAxis.minorGridLines.dashArray,
                'd': yMinorGridDir
            };
            this.svgRenderer.drawPath(optionsMinorGrid, gEle);

        }

        //Drawing Major Grid Lines
        yMajorGridDir = sbYMajorGrid.toString();
        if (yAxis.majorGridLines.visible) {
            var options = {
                'id': this.svgObject.id + '_YAxisMajorGridLines',
                'fill': 'none',
                'stroke-width': yAxis.majorGridLines.width,
                'opacity': yAxis.majorGridLines.opacity,
                'stroke-dasharray': yAxis.majorGridLines.dashArray,
                'stroke': yAxis.majorGridLines.color,
                'd': yMajorGridDir
            };

            if(options.d != '')
                this.svgRenderer.drawPath(options, gEle);

        }
        $(gEle).appendTo(this.gYaxisEle);
    },
	
    _drawXAxisLabels: function (axisIndex, xAxis) {
        var gTickEle = this.svgRenderer.createGroup({ 'id': this.svgObject.id + '_XAxisLabels' + '_' + axisIndex, 'cursor': 'default' });
        var gEle = this.svgRenderer.createGroup({ 'id': this.svgObject.id + '_XAxisLabels' + '_' + axisIndex, 'cursor': 'default' });
        var prevLabels = [];

        if (axisIndex != 0 && ((xAxis.labelPosition == 'inside' && xAxis.tickLinesPosition == 'inside')||(xAxis.labelPosition == 'inside' && xAxis.tickLinesPosition != 'inside')) && (!(xAxis.y == (this.model.m_AreaBounds.Height + this.model.m_AreaBounds.Y) || (xAxis.y == this.model.m_AreaBounds.Y)))) {

            xAxis.y = (!xAxis.opposedPosition) ? Math.floor(xAxis.y + xAxis._LableMaxWidth.height) : Math.floor(xAxis.y - xAxis._LableMaxWidth.height);
        }
        else if (axisIndex != 0 && (xAxis.labelPosition != 'inside' && xAxis.tickLinesPosition == 'inside') && (!(xAxis.y == (this.model.m_AreaBounds.Height + this.model.m_AreaBounds.Y) || (xAxis.y == this.model.m_AreaBounds.Y)))) {
            xAxis.y = (!xAxis.opposedPosition) ? Math.floor(xAxis.y+xAxis.majorTickLines.size) : Math.floor(xAxis.y-xAxis.majorTickLines.size);
        }
        else
            xAxis.y = Math.floor(xAxis.y);  
       
        // loop to draw labels
        for (var i = 0; i < xAxis.visibleLabels.length; i++) {
            var label = xAxis.visibleLabels[i];
			var labelText = label.displayText = label.Text;
            if (ej.EjSvgRender.utils._inside(label.Value, xAxis.visibleRange)) {
                //Declaration
                var textanchor = null;
                var textcollection = [];
                    var gap = xAxis.width / xAxis.visibleLabels.length;       //space between ticks
                    var range = xAxis.visibleRange;
                    var areaBoundWidth = this.model.m_AreaBounds.Width;
                    var pointX = Math.abs(Math.floor(ej.EjSvgRender.utils._getPointXY(label.Value, range, xAxis.isInversed) * (xAxis.width)));
                    var textSize = ej.EjSvgRender.utils._measureText(labelText, areaBoundWidth, xAxis.font);
                    var textWidth = textSize.width;
                    var textHeight = textSize.height;
                    var labelMaxWidth = xAxis._LableMaxWidth;
                    var labelMaxHeight = labelMaxWidth.height;
                    var labelMaxRow = labelMaxWidth.rows;
                    var vmlrendering = this.svgRenderer.vmlNamespace;
                    var opposedPosition = xAxis.opposedPosition;
                    var lineWidth = xAxis.axisLine.width;
                    var font = xAxis.font;
                    var labelRotation = xAxis.labelRotation;            
                    var count = 0;
                    var flag = false;
                    this.edgeLabel = false; // flag to find the supprot for edge labels
                    var anchor = textWidth / 2;
                    var chartAreaWidth = this.model.m_AreaBounds.Width;
                    var chartAreaX = this.model.m_AreaBounds.X;
					
                if (!ej.util.isNullOrUndefined(label.Text)) {
                    var x = pointX + xAxis.x - anchor;
                    if ((xAxis.labelPosition == "inside" && xAxis.tickLinesPosition == 'inside') || ((xAxis.labelPosition == 'inside' && xAxis.tickLinesPosition != 'inside') && xAxis.labelIntersectAction == 'multiplerows'))
                    {                        
                        y = (opposedPosition) ? (xAxis.y + xAxis.majorTickLines.size + textHeight / 2 + (lineWidth / 2)
                       + (labelMaxHeight / (2 * labelMaxRow)) - 5) : (xAxis.y - xAxis.majorTickLines.size - (lineWidth / 2)
                      - (labelMaxHeight / (2 * labelMaxRow)) + 5);
                    }                        
                    else if (xAxis.labelPosition == 'inside'&& xAxis.tickLinesPosition!='inside')
                    {                        
                          y = (opposedPosition) ? (xAxis.y + textHeight / 2 + (lineWidth / 2)
                       + (labelMaxHeight / (2 * labelMaxRow)) - 5) : (xAxis.y - (lineWidth / 2)
                      - (labelMaxHeight / (2 * labelMaxRow)) + 5);                    
                    }
                    else if ((xAxis.labelPosition != 'inside' && xAxis.tickLinesPosition != 'inside') || ((xAxis.labelPosition != 'inside' && xAxis.tickLinesPosition == 'inside') && xAxis.labelIntersectAction == 'multiplerows')) {
                        var y = (opposedPosition) ? (xAxis.y - xAxis.majorTickLines.size + textHeight / 2 - (lineWidth / 2)
                            - (labelMaxHeight / (2 * labelMaxRow)) - 5) : (xAxis.y + xAxis.majorTickLines.size + (lineWidth / 2)
                            + (labelMaxHeight / (2 * labelMaxRow)) + 5);
                    }
                    else if (xAxis.labelPosition != 'inside' && xAxis.tickLinesPosition == 'inside')
                    {                      
                            var y = (opposedPosition) ? (xAxis.y + textHeight / 2 - (lineWidth / 2)
                             - (labelMaxHeight / (2 * labelMaxRow)) - 5) : (xAxis.y + (lineWidth / 2)
                             + (labelMaxHeight / (2 * labelMaxRow)) + 5);                       
                    }
                   
                    
                    if (xAxis.labelPosition == 'inside' && xAxis.tickLinesPosition != 'inside') {
                        if (xAxis.labelRotation == 90 || xAxis.labelIntersectAction == 'rotate90')
                            y = (opposedPosition) ? y  : y - xAxis.majorTickLines.size;
                    }
                    else if (xAxis.labelPosition != 'inside' && xAxis.tickLinesPosition == 'inside') {
                        if (xAxis.labelRotation == 90 || xAxis.labelIntersectAction == 'rotate90')
                            y = (opposedPosition) ? y - xAxis.majorTickLines.size : y - xAxis.majorTickLines.size;
                    }
                    else if (xAxis.labelPosition == 'inside' && xAxis.tickLinesPosition == 'inside') {
                        if (xAxis.labelRotation == 90 || xAxis.labelIntersectAction == 'rotate90')
                            y = (opposedPosition) ? y : y - xAxis.majorTickLines.size;
                    }

                        var options = {
                            'id': this.svgObject.id + '_' + xAxis.name + '_XLabel_' + i,
                            'x': x,
                            'y': y,
                            'fill': font.color,
                            'font-size': font.size,
                            'font-family': font.fontFamily,
                            'font-style': font.fontStyle,
                            'font-weight': font.fontWeight,
                            'opacity': font.opacity
                        };
              
                    // To set position of edge labels
                        if (xAxis.edgeLabelPlacement) {
                            var position = xAxis.edgeLabelPlacement.toLowerCase();
                            var lastLabel = xAxis.visibleLabels.length - 1;
                            switch (position) {
                                case "none":
                                    break;
                                case "shift":
                                    {
                                        if (i == 0) {  // Shift first label
                                            if (options.x < xAxis.x) {
                                                var diff = xAxis.x - options.x;
                                                options.x = x = options.x + diff;
                                                this.diff = diff;
                                                this.edgeLabel = true;
                                            }
                                        } else if (i == lastLabel)  // Shift last label
                                            if (options.x + textSize.width > xAxis.x + xAxis.width) {
                                                diff = (options.x + textSize.width) - (xAxis.x + xAxis.width);
                                                options.x = x = options.x - diff;
                                                this.diff = diff;
                                                this.edgeLabel = true;
                                            }
                                        break;
                                    }
                                  case "hide": // to hide the edge labels
                                    if ((i == 0 && options.x < xAxis.x) || (i == lastLabel && options.x + textWidth >  chartAreaWidth + chartAreaX)) {
                                        labelText = "";
                                    }
                            }
                        }
                    
                    //To perform rotation 
                        if (!vmlrendering && labelRotation && labelRotation != 0)
                            this.labelRotation(xAxis, x + anchor , y, options, label, labelRotation);

                        if (xAxis.labelIntersectAction) {
                            this.diff = ej.util.isNullOrUndefined(this.diff) ? 0 : this.diff;
                            //To perform label intersect action
                            var intersectAction = xAxis.labelIntersectAction.toLowerCase();
                            if (intersectAction == 'none') { // addded enum - none to labelIntersect action
                               
                                this.svgRenderer.drawText(options, labelText, gTickEle);
                                if(xAxis.labelPosition=='inside')
                                    $(gTickEle).appendTo(this.gXaxisTickEle);
                                else
                                    $(gTickEle).appendTo(this.gXaxisEle);
                                continue;
                            } 
                            else if (intersectAction == 'trim') {
                            if (textWidth > gap && labelText != "") { // check textwidth is greater than gap
                            for (var t = 1; t < labelText.toString().length; t++) {
                                labelText = label.displayText.toString().substring(0, t) + '... ';
                                    textWidth = ej.EjSvgRender.utils._measureText(labelText, areaBoundWidth, font).width;
                                    if (textWidth >= gap) {
                                        labelText = labelText.toString().substring(0, t - 1) + '... ';
                                        label.displayText = labelText;
                                        break;
                                    }
                                }
                                label.displayText = labelText; // trimed text is assigned to displayText
                            }
                            if (this.edgeLabel) // for edge label
                                x = options.x = (i == lastLabel) ? x + anchor : x;
                            else
                                x = options.x = x + anchor - textWidth / 2;

                            }else if (intersectAction == 'wrap') {                               
                                
                                var line = 0;
                                var textcoll=this.rowscalculation(labelText.toString(), areaBoundWidth,xAxis,gap,options);
                                var count = textcoll.length;
                                if (xAxis.labelPosition != 'inside')
                                    options.y = opposedPosition ? options.y + textHeight : options.y - textHeight;
                                else
                                    options.y = opposedPosition ? options.y - textHeight : options.y - textHeight * (count);
                                for (k = 0; k < textcoll.length; k++)
                                {
                                    if (xAxis.labelPosition != 'inside')
                                    {
                                        options.y = xAxis.opposedPosition ? options.y - (textHeight) : options.y + (textHeight);
                                        this.svgRenderer.drawText(options, textcoll[k], gTickEle);
                                    }
                                    else
                                    {                                        
                                        options.y = xAxis.opposedPosition ? options.y + (textHeight) : options.y + (textHeight);
                                        this.svgRenderer.drawText(options, textcoll[k], gTickEle);
                                        count -= 1;
                                    }

                                }
                                
                                continue;
                            } else if (labelRotation != 0 && !vmlrendering && intersectAction == 'rotate90')
                                this.labelRotation(xAxis, x + anchor, y, options, label, 90, i);
                            else if (labelRotation != 0 && !vmlrendering && intersectAction == 'rotate45')
                                this.labelRotation(xAxis, x + anchor, y, options, label, 45, i);
                            else if (intersectAction == 'hide') {
                                for (var j = 0; j < i; j++) {
                                    // loop to get previous label
                                    var preLabel = xAxis.visibleLabels[j];
                                    var temp = 0;
                                    opposedPosition = xAxis.opposedPosition;
                                    areaBoundWidth = this.model.m_AreaBounds.Width;
                                   
                                    var prePoint = Math.abs(Math.floor(ej.EjSvgRender.utils._getPointXY(preLabel.Value, range, xAxis.isInversed) * (xAxis.width)));
                                    var preTextWidth = ej.EjSvgRender.utils._measureText(preLabel.displayText, areaBoundWidth, xAxis.font).width;
                                    var value = prePoint + ((xAxis.isInversed) ? -preTextWidth / 2 : preTextWidth / 2);
                                    value = (j == 0) ? value + this.diff : value;
                                    if (intersectAction == 'hide' && ((xAxis.isInversed) ? (value < pointX + textWidth / 2) : (value > pointX - textWidth / 2)))
                                        label.displayText = labelText = '';
                                }
                            }
                            else if(intersectAction == 'multiplerows'){ // to perform multipleRows on intersect
                                for (var k = 0; k < prevLabels.length && !flag; k++) {
                                    for (var l = 0; prevLabels[k] && l < prevLabels[k].length; l++) {
                                        var preLabel = prevLabels[k][l];
                                        var temp = 0;
                                        opposedPosition = xAxis.opposedPosition;
                                        areaBoundWidth = this.model.m_AreaBounds.Width;

                                        var prePoint = Math.abs(Math.floor(ej.EjSvgRender.utils._getPointXY(preLabel.Value, range, xAxis.isInversed) * (xAxis.width)));
                                        var preTextWidth = ej.EjSvgRender.utils._measureText(preLabel.displayText, areaBoundWidth, xAxis.font).width;
                                        var value = prePoint + ((xAxis.isInversed) ? -preTextWidth / 2 : preTextWidth / 2);
                                        if (this.edgeLabel || prevLabels.length==1) {
                                            value = value + this.diff;
                                        }
                                        if ((xAxis.isInversed) ? (value < pointX + textWidth / 2) : (value > pointX - textWidth / 2)) {
                                            if (opposedPosition) {
                                                if (xAxis.labelPosition != 'inside')
                                                    temp = options.y - textHeight;
                                                else
                                                    temp = options.y + textHeight;
                                                options.y = temp;
                                                if (k + 1 == prevLabels.length)
                                                    flag = true;
                                                count++;
                                                break;
                                            } else {
                                                if (xAxis.labelPosition != 'inside')
                                                    temp = options.y + textHeight;
                                                else
                                                    temp = options.y - textHeight;
                                                options.y = temp;
                                                if (k + 1 == prevLabels.length)
                                                    flag = true;
                                                count++;
                                                break;
                                              
                                            }
                                        }
                                        else {
                                            if (l + 1 == prevLabels[k].length) {
                                                flag = true;
                                                break;
                                            }
                                        }                           
                                    }
                                }
                                if (temp == undefined) {
                                    if (prevLabels[0] == undefined)
                                        prevLabels[0] = [];
                                    prevLabels[0].push(label);
                                }
                                else {                                  
                                    if (prevLabels[count] == undefined)
                                        prevLabels[count] = [];
                                    prevLabels[count].push(label);
                                }
                         }         
                        }
                    if ((!this.edgeLabel) && !this.zoomed && !xAxis.zoomed && !(labelRotation) && (((pointX + xAxis.x) + (textWidth/2)) > ($(this.svgObject).width()))) {
                        $(options).attr('x', ($(this.svgObject).width() - 2));
                        $(options).attr('text-anchor', 'end');
                    }
					xAxis.visibleLabels[i].y = options.y;
                    this.svgRenderer.drawText(options, labelText, gTickEle);
					 //create region to chart axis labels
                    var labelSize = ej.EjSvgRender.utils._measureText(labelText, null, font);
                    var bounds = { X: options.x, Y: options.y, Width: labelSize.width, Height: labelSize.width };
                    var xAxisLabelRegion = { Bounds: bounds, TrimText: labelText, LabelText: label.Text };
                    this.model.xAxisLabelRegion.push(xAxisLabelRegion)
                }
            }
        }
        if(xAxis.labelPosition=='inside')
            $(gTickEle).appendTo(this.gXaxisTickEle);
        else
            $(gTickEle).appendTo(this.gXaxisEle);

    },
	
    rowscalculation: function (labelText, areaBoundWidth,xAxis,gap,options)
    {
        var textSize = ej.EjSvgRender.utils._measureText(labelText, areaBoundWidth, xAxis.font);
        var textWidth = textSize.width;
        var anchor = textWidth / 2;
        var text = labelText.toString();
        var textLength = text.length;
        var textHeight = textSize.height;
        var textcollection = [];
        i = 0;
        var line = 0;
        
        if (this.edgeLabel) gap = (i == 0) ? gap - this.diff : gap;
        if (textWidth > gap) {
            options.x = options.x + anchor;
            var line = 0;
            if (this.edgeLabel) options.x = (i == 0) ? options.x - this.diff : options.x + this.diff;
            //options.x = options.x + anchor;
            for (var w = 1; w <= text.length; w++) {
                labelText = text.substring(0, w);
                textWidth = ej.EjSvgRender.utils._measureText(labelText, areaBoundWidth, xAxis.font).width;
                if (textWidth > gap) {
                    line = line + 1; // To find the no of rows splitted
                    count = +1;                    
                    labelText = text.substring(0, w - 1);
                    textcollection[i] = labelText;
                    text = text.slice(w - 1, textLength);                    
                    var newTextWidth = ej.EjSvgRender.utils._measureText(labelText, areaBoundWidth, xAxis.font).width;
                    if (line == 1) // Check for first label
                        options.x = options.x - newTextWidth / 2;
                    i++;
                    w = 0;
                }                
            }           
        }
        textcollection[i] = labelText;
        return textcollection;
    },

	labelRotation: function (axis, x, y, options, label, degree, i) {
	        // To rotate axis labels
	        var labelText = label.Text;
            var angle = (degree > 360) ? degree - 360 : (degree < -360) ? degree + 360 : degree;
            var rotate = 'rotate(' + angle + ',' + (x) + ',' + y + ')';
            $(options).attr('transform', rotate);
			$(options).attr('labelRotation', angle);
            var textElement = this.svgRenderer.createText(options, labelText);
            var diffHeight = Math.ceil(ej.EjSvgRender.utils._measureBounds(textElement, this).height - ej.EjSvgRender.utils._measureText(labelText, null, axis.font).height);
            diffHeight = axis._LableMaxWidth.height - diffHeight - ej.EjSvgRender.utils._measureText(labelText, null, axis.font).height;
            
        if(axis.labelPosition!='inside')
            var yLocation = (axis.opposedPosition) ? (diffHeight / 2) : (-diffHeight / 2);
        else
            var yLocation = (axis.opposedPosition) ? (-diffHeight / 2) : (diffHeight / 2);

        rotate = 'rotate(' + angle + ',' + (x) + ',' + (y + yLocation) + ')';
        if (this.edgeLabel && degree == 90)               
            y = (i == 0) ? options.y + this.diff : options.y - this.diff;
       
	        $(options).attr({
                'transform' :rotate,
                'y': (y + yLocation)
            });
        },
		
    _drawYAxisLabels: function (axisIndex, yAxis) {
        // method to draw y axis labels
        //if(yAxis.labelPosition=='inside')
        var gEle = this.svgRenderer.createGroup({ 'id': this.svgObject.id + '_YAxisLabels' + '_' + axisIndex });        
        if (axisIndex != 1 && ((yAxis.labelPosition == 'inside' && yAxis.tickLinesPosition == 'inside') || (yAxis.labelPosition == 'inside' && yAxis.tickLinesPosition != 'inside'))&& !((yAxis.x == (this.model.m_AreaBounds.Width + this.model.m_AreaBounds.X)) || (yAxis.x == (this.model.m_AreaBounds.X))))
        {
            yAxis.x = (!yAxis.opposedPosition) ? Math.floor(yAxis.x - yAxis._LableMaxWidth.width - this.model.elementSpacing) : Math.floor(yAxis.x + yAxis._LableMaxWidth.width + this.model.elementSpacing);
        }
        else if (axisIndex != 1 && (!(yAxis.x == (this.model.m_AreaBounds.Width + this.model.m_AreaBounds.X)) || (yAxis.x == (this.model.m_AreaBounds.X))) && (yAxis.labelPosition!='inside' && yAxis.tickLinesPosition=='inside'))
        {
            yAxis.x = (!yAxis.opposedPosition) ? Math.floor(yAxis.x - yAxis.majorTickLines.size) : Math.floor(yAxis.x + yAxis.majorTickLines.size);
        }      
        else
        {
            yAxis.x = Math.floor(yAxis.x);
        }   
      
        for (var i = 0; i < yAxis.visibleLabels.length; i++) {
            var label = yAxis.visibleLabels[i];
			var visibleRange = yAxis.visibleRange;
			if (ej.EjSvgRender.utils._inside(label.Value, visibleRange)) {
			    var pointY = Math.abs(ej.EjSvgRender.utils._getPointXY(label.Value, visibleRange, yAxis.isInversed) * (yAxis.height));
			    var text = ej.EjSvgRender.utils._measureText(label.Text, this.model.m_AreaBounds.Height, yAxis.font);
			    var textHeight = text.height;
			    var opposedPosition = yAxis.opposedPosition;
			    var font = yAxis.font;
			    var range = yAxis.visibleRange;
			    var lineWidth = yAxis.axisLine.width;
			    var labelMaxWidth = yAxis._LableMaxWidth;
			    var labelMaxHeight = labelMaxWidth.height;
			    var labelMaxRow = labelMaxWidth.rows;

			    if(yAxis.labelPosition=='inside' && yAxis.tickLinesPosition=='inside')
			    {
			        var x = (opposedPosition) ? (yAxis.x - (yAxis.majorTickLines.size) - (lineWidth / 2) - 5)
               : (yAxis.x + (lineWidth / 2) + 5 + yAxis.majorTickLines.size);
			        var textanchor = (opposedPosition) ? 'end' : 'start';			       
			    }
			    else if (yAxis.labelPosition == 'inside' && yAxis.tickLinesPosition != 'inside')
			    {			        
			        var x = (opposedPosition) ? (yAxis.x - (lineWidth / 2)  - 5)
                : (yAxis.x + (lineWidth / 2) + 5);
			        var textanchor = (opposedPosition) ? 'end' : 'start';			       
			    }
			    else if (yAxis.tickLinesPosition == 'inside' && yAxis.labelPosition != 'inside')
			    {

			        var x = (opposedPosition) ? ((lineWidth / 2) + yAxis.x + 5)
                        : (yAxis.x - (lineWidth / 2) - 5);
			        var textanchor = (opposedPosition) ? 'start' : 'end';
			        
			    }
			    else
			    {
			        var x = (opposedPosition) ? ((yAxis.majorTickLines.size) + (lineWidth / 2) + yAxis.x + 5)
                        : (yAxis.x - (lineWidth / 2) - 5 - yAxis.majorTickLines.size);
			        var textanchor = (opposedPosition) ? 'start' : 'end';			    
			}                       

                    var y = ((pointY * -1) + (yAxis.y + yAxis.height) + textHeight/4);
                    var options = {
                        'id': this.svgObject.id + '_YLabel_' + axisIndex +'_'+ i,
                        'x': x,
                        'y': y,
                        'fill': font.color,
                        'font-size': font.size,
                        'font-family': font.fontFamily,
                        'font-style': font.fontStyle,
                        'font-weight': font.fontWeight,
                        'opacity': font.opacity,
                        'text-anchor': textanchor
                    };
					
                // To set position of edge labels in yAxis
                    if (yAxis.edgeLabelPlacement) {
                        var position = yAxis.edgeLabelPlacement.toLowerCase();
                        var lastLabel = yAxis.visibleLabels.length - 1;

                        switch (position) {
                            case "none":
                                break;
                            case "shift":
                                {
                                    if (i == 0) {  // Shift first label
                                        if (options.y > yAxis.y + yAxis.height) {
                                            var diff = options.y - (yAxis.y + yAxis.height);
                                            options.y = y = options.y - diff;
                                            this.diff = diff;
                                            this.edgeLabel = true;
                                        }
                                    } else if (i == lastLabel) // Shift last label
                                        if (options.y - textHeight/2 < yAxis.y) {
                                            diff = yAxis.y - (options.y - textHeight / 2);
                                            options.y = y = options.y + diff;
                                            this.diff = diff;
                                            this.edgeLabel = true;
                                        }
                                    break;
                                }
                            case "hide": //to hide the edge labels
                                if ((i == 0 && options.y > yAxis.y + yAxis.height) || (i == lastLabel && options.y - textHeight / 2 < this.model.m_AreaBounds.Y)) {
                                    label.Text = "";
                                }
                        }
                    }

				 //Label intersection
                    if (yAxis.labelIntersectAction) {
                        if (intersectAction == 'none') {
                            this.svgRenderer.drawText(options, label.Text, gEle);
                            //if(yAxis.labelPosition=='inside')
                            //    $(gEle).appendTo(this.gYaxisTickEle);
                            //else
                            //    $(gEle).appendTo(this.gYaxisEle);
                            continue;
                        }
                        var intersectAction = yAxis.labelIntersectAction.toLowerCase();
                        for (var j = 0; j < i; j++) {
                            var prevLabel = yAxis.visibleLabels[j];
                            var temp = 0;
                            var prevPoint = Math.abs((ej.EjSvgRender.utils._getPointXY(prevLabel.Value, range, yAxis.isInversed) * (yAxis.height)));
                                var prevTextHeight = ej.EjSvgRender.utils._measureText(prevLabel.Text, this.model.m_AreaBounds.Height, font).height;
                                var value = ((prevPoint * -1) + (yAxis.y + yAxis.height) + prevTextHeight/4);
                                value = value + ((yAxis.isInversed) ? prevTextHeight / 2 : -prevTextHeight / 2);
                            if (this.edgeLabel)
                                value = (j == 0) ? value - prevTextHeight / 4 : value;
                                if (intersectAction == 'hide' && prevLabel.Text.toString()!="" && ((yAxis.isInversed) ? value > options.y - textHeight / 4 : value < options.y + textHeight / 4))
                                    label.Text = '';
                                else if (intersectAction == 'multiplerows' && ((yAxis.isInversed) ? value > options.y - textHeight / 4 : value < options.y + textHeight / 4) && prevLabel.x == options.x) {
                                    if (opposedPosition) {
                                        if(yAxis.labelPosition !="inside")
                                            temp = prevLabel.x + yAxis._LableMaxWidth.maxWidth + 5;  // gap between the rows
                                        else
                                            temp = prevLabel.x - yAxis._LableMaxWidth.maxWidth - 5;                                            
                                        options.x = temp;
                                    }
                                    else {
                                        if (yAxis.labelPosition != "inside")
                                            temp = prevLabel.x - yAxis._LableMaxWidth.maxWidth - 5; // gap between the rows
                                        else
                                            temp = prevLabel.x + yAxis._LableMaxWidth.maxWidth + 5;
                                        options.x = temp;
                                     }
                                }
                            }
                        }

                    yAxis.visibleLabels[i].x = options.x;	
                this.svgRenderer.drawText(options, label.Text, gEle);
            }
        }
        if(yAxis.labelPosition=='inside')
            $(gEle).appendTo(this.gYaxisTickEle);
        else
            $(gEle).appendTo(this.gYaxisEle);
    },
	
    _drawXTitle: function (axisIndex, axis) {

        if (axis.title.text != "" && axis.title.text) {
            var gEle = this.svgRenderer.createGroup({ 'id': this.svgObject.id + '_XAxisTitle' + '_' + axisIndex });
            var xtitleLocation = (this.model.elementSpacing) + axis._LableMaxWidth.height;
            var titlesize = (ej.EjSvgRender.utils._measureText(axis.title.text, (axis.width), axis.title.font).height / 2);
            var commonEventArgs = $.extend({}, ej.EjSvgRender.commonChartEventArgs);

            var y = this.model.elementSpacing + axis._LableMaxWidth.height + titlesize + axis.majorTickLines.size + axis.axisLine.width;

            if ((axis.tickLinesPosition == 'inside' && axis.labelPosition == 'inside') && (axisIndex == 0 || (axis.y == (this.model.m_AreaBounds.Height + this.model.m_AreaBounds.Y)) || (axis.y == (this.model.m_AreaBounds.Y))))
                var y =this.model.elementSpacing + titlesize + axis.axisLine.width;
            else if ((axisIndex == 0 || (axis.y == (this.model.m_AreaBounds.Width + this.model.m_AreaBounds.X)) || (axis.y == (this.model.m_AreaBounds.X))) && (axis.tickLinesPosition != 'inside' && axis.labelPosition == 'inside'))
                var y = this.model.elementSpacing + titlesize +axis.majorTickLines.size+ axis.axisLine.width;
            else if ((axisIndex == 0 || (axis.y == (this.model.m_AreaBounds.Width + this.model.m_AreaBounds.X)) || (axis.y == (this.model.m_AreaBounds.X))) && (axis.tickLinesPosition == 'inside' && axis.labelPosition != 'inside'))
                var y = this.model.elementSpacing + axis._LableMaxWidth.height+titlesize + axis.axisLine.width;
            else if ((axis.tickLinesPosition != 'inside' && axis.labelPosition != 'inside') || (axisIndex != 1))
                var y = this.model.elementSpacing + axis._LableMaxWidth.height + titlesize + axis.majorTickLines.size + axis.axisLine.width;

            commonEventArgs.data = { title: axis.title.text, location: { x: (axis.width) / 2, y: y }, axes: axis };
            this._trigger("axesTitleRendering", commonEventArgs);
            var options = {
                'id': this.svgObject.id + '_XAxisTitle' + '_' + axisIndex,
                'x': commonEventArgs.data.location.x + axis.x,
                'y': (axis.opposedPosition) ? (axis.y - commonEventArgs.data.location.y) : (commonEventArgs.data.location.y + axis.y ),
                'fill': axis.title.font.color,
                'font-size': axis.title.font.size,
                'font-family': axis.title.font.fontFamily,
                'font-style': axis.title.font.fontStyle,
                'font-weight': axis.title.font.fontWeight,
                'opacity': axis.title.font.opacity,
                'text-anchor': 'middle',
                'dominant-baseline': 'middle'
            };
            if (this.model.enableCanvasRendering) {
                var matched = jQuery.uaMatch(navigator.userAgent);
                var isIE11 = !!navigator.userAgent.match(/Trident\/7\./);
                var browser = matched.browser.toLowerCase();
                 if (browser != "msie" && !isIE11)
                     options.y = !axis.opposedPosition ? options.y + (titlesize / 2) + (browser == "mozilla" ? 1 : -2) : options.y + (titlesize / 2) + 1; // text dominant baseline for canvas
            }
            if ($(this.svgObject).find("#" + this.svgObject.id + '_XAxisTitle' + '_' + axisIndex).length > 0)
                $(this.svgObject).find("#" + this.svgObject.id + '_XAxisTitle' + '_' + axisIndex).attr(options);
            else
                this.svgRenderer.drawText(options, commonEventArgs.data.title, gEle);
            $(gEle).appendTo(this.gXaxisEle);
        }

    },
    _drawYTitle: function (axisIndex, yAxis) {

        if (yAxis.title.text != "" && yAxis.title.text) {
            var gEle = this.svgRenderer.createGroup({ 'id': this.svgObject.id + '_YAxisTitle' + '_' + axisIndex });
            var titleSize = ej.EjSvgRender.utils._measureText(yAxis.title.text, (yAxis.height), yAxis.title.font);
            if ((yAxis.tickLinesPosition == 'inside' && yAxis.labelPosition == 'inside') && (axisIndex == 1 || (yAxis.x == (this.model.m_AreaBounds.Width + this.model.m_AreaBounds.X)) || (yAxis.x == (this.model.m_AreaBounds.X))))
                var x = (yAxis.opposedPosition) ? yAxis.x + ((3 * this.model.elementSpacing) + yAxis.axisLine.width-5) : yAxis.x - ((3 * this.model.elementSpacing) + yAxis.axisLine.width);
            else if ((axisIndex == 1 || (yAxis.x == (this.model.m_AreaBounds.Width + this.model.m_AreaBounds.X)) || (yAxis.x == (this.model.m_AreaBounds.X))) && (yAxis.tickLinesPosition != 'inside' && yAxis.labelPosition == 'inside'))
                var x = (yAxis.opposedPosition) ? yAxis.x + ((3 * this.model.elementSpacing) + yAxis.majorTickLines.size + yAxis.axisLine.width) : yAxis.x - ((3 * this.model.elementSpacing) + yAxis.majorTickLines.size + yAxis.axisLine.width);
            else if ((axisIndex == 1 || (yAxis.x == (this.model.m_AreaBounds.Width + this.model.m_AreaBounds.X)) || (yAxis.x == (this.model.m_AreaBounds.X))) && (yAxis.tickLinesPosition == 'inside' && yAxis.labelPosition != 'inside'))
                var x = (yAxis.opposedPosition) ? yAxis.x + ((3 * this.model.elementSpacing) + yAxis._LableMaxWidth.width + yAxis.axisLine.width) : yAxis.x - ((3 * this.model.elementSpacing) + yAxis._LableMaxWidth.width + yAxis.axisLine.width);
            else if ((yAxis.tickLinesPosition != 'inside' && yAxis.labelPosition != 'inside')||(axisIndex!=1))
                var x = (yAxis.opposedPosition) ? yAxis.x + ((3 * this.model.elementSpacing) + yAxis._LableMaxWidth.width + yAxis.majorTickLines.size + yAxis.axisLine.width) : yAxis.x - ((3 * this.model.elementSpacing) + yAxis._LableMaxWidth.width + yAxis.majorTickLines.size + yAxis.axisLine.width);
          
            var commonEventArgs = $.extend({}, ej.EjSvgRender.commonChartEventArgs);
            commonEventArgs.data = { title: yAxis.title.text, location: { x: titleSize.width / 2, y: 0 }, axes: yAxis };
            this._trigger("axesTitleRendering", commonEventArgs);

            var y = commonEventArgs.data.location.y + (yAxis.y + yAxis.height) + (((yAxis.height) / 2) * -1);

            var labelRotation = (yAxis.opposedPosition) ? 90 : -90;

            var options = {
                'id': this.svgObject.id + '_YAxisTitle' + '_' + axisIndex,
                'x': x,
                'y': y,
                'fill': yAxis.title.font.color,
				'labelRotation': labelRotation,
                'transform': 'rotate(' + labelRotation + ',' + (x) + ',' + y + ')',
                'font-size': yAxis.title.font.size,
                'font-family': yAxis.title.font.fontFamily,
                'font-style': yAxis.title.font.fontStyle,
                'font-weight': yAxis.title.font.fontWeight,
                'opacity': yAxis.title.font.opacity,
                'text-anchor': 'middle',
                'dominant-baseline': 'middle'
            };
             if (this.model.enableCanvasRendering) {
                var matched = jQuery.uaMatch(navigator.userAgent);
                var isIE11 = !!navigator.userAgent.match(/Trident\/7\./);
                var browser = matched.browser.toLowerCase();
                 if (browser != "msie" && !isIE11)
                    options.x = !yAxis.opposedPosition ? options.x + (titleSize.height / 4) + (browser == "mozilla" ? 1 : -2) : options.x - (titleSize.height / 4) + 1; // text dominant baseline for canvas
            }
            if ($(this.svgObject).find("#" + this.svgObject.id + '_YAxisTitle' + '_' + axisIndex).length > 0)
                $(this.svgObject).find("#" + this.svgObject.id + '_YAxisTitle' + '_' + axisIndex).attr(options);
            else
                this.svgRenderer.drawText(options, commonEventArgs.data.title, gEle);
            $(gEle).appendTo(this.gYaxisEle);
        }

    }
},
ej.EjStripline.prototype = {
    _drawStripline: function (axis, stripLine) {

        var x = 0, height = 0, angle = 0;
        var y = 0, pointYstart = 0, yHeight = 0;
        var width = 0, widthValue = 0, textX = 0, textY = 0;
        if (stripLine.visible) {
            var striplineid = (stripLine.zIndex == 'over') ? this.gStriplineOver : this.gStriplineBehind;
            striplineid = ej.util.isNullOrUndefined(striplineid) ? "" : striplineid;
            var isVisible = true;
            if (axis.orientation.toLowerCase() == "horizontal") {
                var pointstart = 0;

                if (stripLine.startFromAxis) {

                    var point = (ej.util.isNullOrUndefined(stripLine.offset)) ? axis.visibleRange.min : axis.visibleRange.min + stripLine.offset;
                    pointstart = (((point - axis.visibleRange.min) / (axis.visibleRange.delta)) * (axis.width)) + axis.x; //($(this.svgObject).width() - this.model.m_Spacing.Left - this.model.m_Spacing.Right);

                    widthValue = (((point + (ej.util.isNullOrUndefined(stripLine.width) ? 0 : stripLine.width) - axis.visibleRange.min) / (axis.visibleRange.delta)) * (axis.width));

                } else {
                    if (ej.util.isNullOrUndefined(stripLine.start) && (ej.util.isNullOrUndefined(stripLine.end) || ej.util.isNullOrUndefined(stripLine.width))) {
                        isVisible = false;
                    } else {
                        stripLine.start = (jQuery.type(stripLine.start) == "date") ? (stripLine.start).getTime() : stripLine.start;
                        if (!ej.util.isNullOrUndefined(stripLine.end) && (jQuery.type(stripLine.end) == "date")) {
                            stripLine.end = (stripLine.end).getTime();
                        }
                        pointstart = (ej.EjSvgRender.utils._getPointXY(stripLine.start, axis.visibleRange, axis.isInversed) * (axis.width)) + axis.x; //($(this.svgObject).width() - this.model.m_Spacing.Left - this.model.m_Spacing.Right);
                        var pointend = (ej.EjSvgRender.utils._getPointXY(stripLine.end, axis.visibleRange, axis.isInversed) * (axis.width)) + axis.x; //($(this.svgObject).width() - this.model.m_Spacing.Left - this.model.m_Spacing.Right);
                        var widthMax = (ej.EjSvgRender.utils._getPointXY((stripLine.start + (ej.util.isNullOrUndefined(stripLine.width) ? 0 : stripLine.width)), axis.visibleRange, axis.isInversed) * (axis.width)) + axis.x;

                        widthValue = (axis.isInversed ? (pointend < widthMax) : (pointend > widthMax)) ? Math.abs(pointstart - pointend) : Math.abs(pointstart - widthMax);
                        
                    }
                }
                width = widthValue;
                height = this.chart.model.m_AreaBounds.Height;
                x = (axis.isInversed) ? (pointstart - widthValue) : pointstart;
                y = this.chart.model.m_AreaBounds.Y;
                angle = -90;
                var textWidth = ej.EjSvgRender.utils._measureText(stripLine.text, null, stripLine.font).height;
                if (stripLine.textAlignment == 'middletop') {
                    textX = x+textWidth;
                    textY = Math.abs(y + height / 2);
                } else if (stripLine.textAlignment == 'middlecenter') {
                    textX = (Math.floor(x + width / 2) + (textWidth / 3));
                    textY = Math.abs(y + height / 2);
                } else if (stripLine.textAlignment == 'middlebottom') {
                    textX = x + width - (textWidth);
                    textY = Math.abs(y + height / 2);
                }


            } else {
                if (stripLine.startFromAxis) {
                    var pointY = (ej.util.isNullOrUndefined(stripLine.offset)) ? axis.visibleRange.min : axis.visibleRange.min + stripLine.offset;
                    pointYstart = (axis.y + axis.height) - (((pointY - axis.visibleRange.min) / (axis.visibleRange.delta)) * (axis.height));
                    widthValue = (((pointY + (ej.util.isNullOrUndefined(stripLine.width) ? 0 : stripLine.width) - axis.visibleRange.min) / (axis.visibleRange.delta)) * (axis.height));
                    yHeight = widthValue;
                } else {

                    if (ej.util.isNullOrUndefined(stripLine.start) && (ej.util.isNullOrUndefined(stripLine.end) || ej.util.isNullOrUndefined(stripLine.width))) {
                        isVisible = false;
                    } else {
                        stripLine.start = (jQuery.type(stripLine.start) == "date") ? (stripLine.start).getTime() : stripLine.start;
                        if (!ej.util.isNullOrUndefined(stripLine.end) && (jQuery.type(stripLine.end) == "date")) {
                            stripLine.end = (stripLine.end).getTime();
                        }
                        pointYstart = (axis.y + axis.height) - (ej.EjSvgRender.utils._getPointXY(stripLine.start, axis.visibleRange, axis.isInversed) * (axis.height));
                        var pointYend = (axis.y + axis.height) - (ej.EjSvgRender.utils._getPointXY((ej.util.isNullOrUndefined(stripLine.end) ? 0 : stripLine.end), axis.visibleRange, axis.isInversed) * (axis.height));
                        var widthYValue = (axis.y + axis.height) - (ej.EjSvgRender.utils._getPointXY((stripLine.start + (ej.util.isNullOrUndefined(stripLine.width) ? 0 : stripLine.width)), axis.visibleRange, axis.isInversed) * (axis.height));
                        yHeight = (axis.isInversed ? (pointYend > widthYValue) : (pointYend < widthYValue)) ? Math.abs(pointYstart - pointYend) : Math.abs(pointYstart - widthYValue);
                    }
                }
                height = yHeight;
                x = this.chart.model.m_AreaBounds.X;
                width = this.chart.model.m_AreaBounds.Width;
                y = pointYstart + ((axis.isInversed) ? 0 : -yHeight);
                var textHeight = ej.EjSvgRender.utils._measureText(stripLine.text, width, stripLine.font).height;

                if (stripLine.textAlignment == 'middletop') {
                    textX = x + width / 2;
                    textY = y + textHeight;
                } else if (stripLine.textAlignment == 'middlecenter') {
                    textX = x + width / 2;
                    textY = (Math.abs(y + (height / 2)) + (textHeight / 3));
                } else if (stripLine.textAlignment == 'middlebottom') {
                    textX = x + width / 2;
                    textY = (y + height) - textHeight;

                }


            }
            if (isVisible) {
                if (!ej.util.isNullOrUndefined(stripLine.imageUrl)) {
                    var imgOptions = {
                        'height': height,
                        'width': width,
                        'href': stripLine.imageUrl,
                        'x': x,
                        'y': y,
                        'id': striplineid.id + '_backImage',
                        'visibility': 'visible',
                        'preserveAspectRatio': 'none'
                    };

                    this.svgRenderer.drawImage(imgOptions, striplineid);
                } else {
                    var options = {
                        'id': striplineid.id + '_striplineRect',
                        'x': x,
                        'y': y,
                        'width': width,
                        'height': height,
                        'fill': stripLine.color,
                        'opacity': stripLine.opacity,
                        'stroke-width': stripLine.borderWidth,
                        'stroke':(stripLine.borderWidth==0)?"transparent": stripLine.borderColor
                    };
                    var textOptions = {
                        'id': striplineid.id + '_striplineRectText',
                        'x': textX,
                        'y': textY,
                        'fill': stripLine.font.color,
                        'transform': 'rotate(' + angle + ',' + (textX) + ',' + textY + ')',
                        'labelRotation': angle,
                        'font-size': stripLine.font.size,
                        'font-family': stripLine.font.fontFamily,
                        'font-style': stripLine.font.fontStyle,
                        'font-weight': stripLine.font.fontWeight,
                        'text-anchor': 'middle'
                    };

                    var bounds = { X: x, Y: y, Width: width, Height: height };
                    var isStripline = stripLine.zIndex == "over" ? true : false;
                    ej.EjSvgRender.utils.AddRegion(this.chart, bounds, isStripline)

                    this.svgRenderer.drawRect(options, striplineid);
                    this.svgRenderer.drawText(textOptions, stripLine.text, striplineid);
                }
            }

        }
    }
};
})(jQuery);
ej.EjLegendRender = function (chartobj) {

    this.chartObj = chartobj;
};


(function ($) {
ej.EjLegendRender.prototype =
{
    
  
    drawLegendTitle: function () {
        //draw legend title
        var legendTitle = this.chartObj.model.legend.title;
         var legendBoundsWidth = this.chartObj.model.LegendBounds.Width;
         var size = ej.EjSvgRender.utils._measureText(legendTitle.text, null, this.chartObj.model.legend.title.font); 
        var startX, startY;
        startX = this.chartObj.model.elementSpacing/2;
        startY = this.chartObj.model.elementSpacing+ (size.height) / 2;
        if (this.chartObj.model.legend.title.textAlignment.toLowerCase() == "near")
        {
            startX = this.chartObj.model.elementSpacing/2;
        }
        else if (this.chartObj.model.legend.title.textAlignment.toLowerCase() == "far")
        {
            var startX = legendBoundsWidth -(size.width) - this.chartObj.model.elementSpacing / 2;
           
        }
        else {
            var startX = legendBoundsWidth / 2 - (size.width) / 2;
        }

        if (startX < 0) {
            startX = 0;
            this.chartObj.model.LegendBounds.Width =(size. width);
        }
       
        
        if (this.chartObj.model.LegendBounds.Width < size.width + startX)
        {
            this.chartObj.model.LegendBounds.Width = size.width+startX;
        }
        
        var fontLegendtitle = this.chartObj.model.legend.title.font;
        var options = {
            'id': this.chartObj.svgObject.id + '_LegendTitleText',
            'x': startX,
            'y': startY,
            'fill': fontLegendtitle.color,
            'font-size': fontLegendtitle.size,
            'font-style': fontLegendtitle.fontStyle,
            'font-family': fontLegendtitle.fontFamily,
            'font-weight': fontLegendtitle.fontWeight,
            'text-anchor': 'start',
            'lgndCtx': true
        };

        this.chartObj.svgRenderer.drawText(options, legendTitle.text, this.chartObj.gLegendEle);

    },
    
    drawLegendItem: function (legendItem, x, y) {
        var shapeSize = this.chartObj.model.legend.itemStyle;
        var elementSpace = this.chartObj.model.elementSpacing;
        var location = { startX: x + shapeSize.width / 2, startY: (y + shapeSize.height / 2) };
        
        var series = this.chartObj.model.series;
       
        
        var style = { ShapeSize:shapeSize,ElementSpace:elementSpace, Style: legendItem.LegendStyle, Gradient: legendItem.Gradient, SeriesIndex: legendItem.SeriesIndex, context: true };
        var symbolName;
        for (name in this.chartObj.model.symbolShape) {
            if (legendItem.Shape.toLowerCase() == name.toLowerCase()) {
                if (name.toLowerCase() == "seriestype")
                    symbolName = "SeriesType";
                else
                    symbolName = name;
                break;
            }     
        }
            if (symbolName == "SeriesType") {
                if (series[legendItem.SeriesIndex].type.toLowerCase() == "radar" || series[legendItem.SeriesIndex].type.toLowerCase() == "polar") {
                    var symbol = series[legendItem.SeriesIndex].drawType;
                    symbolName = this.capitalizeFirstString(symbol);
                }
                if (series[legendItem.SeriesIndex].type.toLowerCase() == "scatter") {
                    var symbol = series[legendItem.SeriesIndex].marker.shape;
                    symbolName = this.capitalizeFirstString(symbol);
                }
            }
            if (symbolName == "None") {
                if (series[legendItem.SeriesIndex].type.toLowerCase() == "pie" || series[legendItem.SeriesIndex].type.toLowerCase() == "doughnut")
                    symbolName = "Circle";
                else
                symbolName = "Rectangle";
            }
       
        var index = (ej.util.isNullOrUndefined(legendItem.PointIndex)) ? legendItem.SeriesIndex : legendItem.PointIndex;
        this.chartObj.gLegendItemEle = this.chartObj.svgRenderer.createGroup({ 'id': this.chartObj.svgObject.id + '_Legend' + index, 'cursor': 'pointer' });

        var commonEventArgs = $.extend({}, ej.EjSvgRender.commonChartEventArgs);
        commonEventArgs.data = { legendItem: legendItem, location: location, legend: true, model: this.chartObj.model, svgRenderer: this.chartObj.svgRenderer, svgObject: this.chartObj.svgObject, symbolShape: symbolName, style: style, gLegendItemEle: this.chartObj.gLegendItemEle };

        this.chartObj._trigger("legendItemRendering", commonEventArgs);
        
        
        ej.EjSvgRender.chartSymbol["_draw" + commonEventArgs.data.symbolShape](commonEventArgs.data.location, commonEventArgs.data.style, commonEventArgs.data, this.chartObj.gLegendItemEle);
        var color; 
        if (legendItem.LegendStyle.Color && legendItem.LegendStyle.Color.toLowerCase() !== 'gray' && this.chartObj.model.enableCanvasRendering) // for canvas gradient color
            color = this.chartObj.model.legend.font.color;
        else if (legendItem.LegendStyle.Color && legendItem.LegendStyle.Color.toLowerCase() !== 'gray') // for svg
            color = this.chartObj.model.legend.font.color;
        else
            color = 'gray';
    
        var fontLegend = this.chartObj.model.legend.font;
        var itemInfo = commonEventArgs.data.legendItem;
        var heightText = ej.EjSvgRender.utils._measureText(itemInfo.Text, this.chartObj.model.LegendBounds.Width, fontLegend).height;
        if (this.chartObj.model.legend.shape == "seriestype" || this.chartObj.model.legend.shape == "seriesType")
            x += this.chartObj.model.legend.itemPadding/2;
        var pointindex = legendItem.PointIndex;
        var options = {
            'id': this.chartObj.svgObject.id + '_LegendItemText'+ pointindex,
            'x': shapeSize.width + x + this.chartObj.model.legend.itemPadding/2,
            'y': commonEventArgs.data.location.startY,
            'fill': color,
            'font-size': fontLegend.size,
            'font-style': fontLegend.fontStyle,
            'font-family': fontLegend.fontFamily,
            'font-weight': fontLegend.fontWeight,
            'text-anchor': 'start',
            'lgndCtx': true
        };
        var matched = jQuery.uaMatch(navigator.userAgent);
		var isIE11 = !!navigator.userAgent.match(/Trident\/7\./);
       // if (matched.browser.toLowerCase() == "msie" || isIE11) 
            options.y = (commonEventArgs.data.location.startY +(heightText / 4)); // for dominant-baseline
        //else 
        //    this.chartObj.svgRenderer._setAttr($(options), { 'dominant-baseline': 'middle' });
			
		this.chartObj.svgRenderer.drawText(options,commonEventArgs.data.legendItem.Text,this.chartObj.gLegendItemEle);
		this.chartObj.svgRenderer.append(this.chartObj.gLegendItemEle, this.chartObj.gLegendEle);
		var legendbound = { X: this.chartObj.model.LegendBounds.X + this.chartObj.model.legend.border.width, Y: this.chartObj.model.LegendBounds.Y + this.chartObj.model.legend.border.width };
        var itembound = { X: (x), Y: (y) };
        var bounds = { LegendBound: legendbound, ItemBound: itembound };
        var legendRegion = { LegendItem: legendItem, Location: location, SymbolShape: symbolName, Style: style, Bounds: bounds };
        this.chartObj.model.legendRegion.push(legendRegion);
    },
    
    capitalizeFirstString:function(string){
        return string.charAt(0).toUpperCase() + string.slice(1);
    },

    drawLegend: function () {

        var modelsubTitleHeight = (this.chartObj.model.title.subTitle.text == "") ? 0 : (ej.EjSvgRender.utils._measureText(this.chartObj.model.title.subTitle.text, $(this.svgObject).width() - this.chartObj.model.margin.left - this.chartObj.model.margin.right, this.chartObj.model.title.subTitle.font).height + this.chartObj.model.elementSpacing);
        
        if (!ej.util.isNullOrUndefined(this.chartObj.model.legend) && this.chartObj.model.legend.visible) {
            // draw legend.
           var legendTitle = this.chartObj.model.legend.title;
           var height = ej.EjSvgRender.utils._measureText(legendTitle.text, null, this.chartObj.model.legend.title.font).height;
            if (this.chartObj.model.legend.position.toLowerCase() == this.chartObj.model.legendPosition.bottom) {
                this.chartObj.model.LegendBounds.Y = $(this.chartObj.svgObject).height() - (this.chartObj.model.elementSpacing* 1.5)- this.chartObj.model.LegendViewerBounds.Height-height;

                this.chartObj.model.LegendBounds.X = ($(this.chartObj.svgObject).width() / 2) - (this.chartObj.model.LegendViewerBounds.Width / 2);

            }
            else if (this.chartObj.model.legend.position.toLowerCase() == this.chartObj.model.legendPosition.top) {
                this.chartObj.model.LegendBounds.Y = ((this.chartObj.model._titleLocation) ? this.chartObj.model._titleLocation.Y : 0) + modelsubTitleHeight + this.chartObj.model.elementSpacing;
                this.chartObj.model.LegendBounds.X = ($(this.chartObj.svgObject).width() / 2) - (this.chartObj.model.LegendViewerBounds.Width / 2);
            }

            else if (this.chartObj.model.legend.position.toLowerCase() == this.chartObj.model.legendPosition.right) {
                this.chartObj.model.LegendBounds.Y = ($(this.chartObj.svgObject).height()) / 2 - (this.chartObj.model.LegendViewerBounds.Height / 2);
                this.chartObj.model.LegendBounds.X = $(this.chartObj.svgObject).width() - this.chartObj.model.elementSpacing - this.chartObj.model.LegendViewerBounds.Width;
            }

            else if (this.chartObj.model.legend.position.toLowerCase() == this.chartObj.model.legendPosition.left) {
                this.chartObj.model.LegendBounds.Y = ($(this.chartObj.svgObject).height()) / 2 - (this.chartObj.model.LegendViewerBounds.Height / 2);
                this.chartObj.model.LegendBounds.X = this.chartObj.model.elementSpacing;
            }
            else {
                this.chartObj.model.LegendBounds.Y = (this.chartObj.model.legend.location.y < $(this.chartObj.svgObject).height()) ? this.chartObj.model.legend.location.y : 0;
                this.chartObj.model.LegendBounds.X = (this.chartObj.model.legend.location.x < $(this.chartObj.svgObject).width()) ? this.chartObj.model.legend.location.x : 0;
            }
            if (this.chartObj.model.legend.position.toLowerCase() == 'left' || this.chartObj.model.legend.position.toLowerCase() == 'right') {
                if (this.chartObj.model.legend.alignment.toLowerCase() == 'center') {
                    this.chartObj.model.LegendBounds.Y = ($(this.chartObj.svgObject).height()) / 2 - (this.chartObj.model.LegendViewerBounds.Height / 2);
                } else if (this.chartObj.model.legend.alignment.toLowerCase() == 'near') {
                    this.chartObj.model.LegendBounds.Y = this.chartObj.model.m_Spacing.Top + this.chartObj.model.elementSpacing + (this.chartObj.model.legend.itemPadding / 2);
                } else {
                    this.chartObj.model.LegendBounds.Y = $(this.chartObj.svgObject).height() - this.chartObj.model.m_Spacing.Bottom - this.chartObj.model.LegendViewerBounds.Height - this.chartObj.model.elementSpacing - 5;
                }
                //adjust the legend viewer bound height based on legend alignment
                if (this.chartObj.model.AreaType == "cartesianaxes") {
                    if (this.chartObj.model.LegendBounds.Y < this.chartObj.model.m_AreaBounds.Y || this.chartObj.model.m_AreaBounds.Y + this.chartObj.model.LegendViewerBounds.Height >= this.chartObj.model.m_AreaBounds.Height) {
                        var legendHeight = 0;
                        for (var k = 0; k < this.chartObj.model.legendCollection.length; k++) {
                            legendHeight += this.chartObj.model.legendCollection[k].Bounds.Height;
                            if (legendHeight > this.chartObj.model.m_AreaBounds.Height) {
                                legendHeight -= this.chartObj.model.legendCollection[k].Bounds.Height; break;
                            }
                            this.chartObj.model.LegendViewerBounds.Height = legendHeight;
                            this.chartObj.model.LegendBounds.Y = (this.chartObj.model.m_AreaBounds.Height / 2) - (this.chartObj.model.LegendViewerBounds.Height / 2)
                           + this.chartObj.model.m_AreaBounds.Y;
                        }
                    }
                }
                else {
                    if (this.chartObj.model.LegendBounds.Y < 0 || this.chartObj.model.LegendBounds.Y + this.chartObj.model.LegendViewerBounds.Height >= $(this.chartObj.svgObject).height()) {
                        var legendHeight = 0;
                        for (var k = 0; k < this.chartObj.model.legendCollection.length; k++) {
                            legendHeight += this.chartObj.model.legendCollection[k].Bounds.Height;
                            if (legendHeight > $(this.chartObj.svgObject).height()) {
                                legendHeight -= this.chartObj.model.legendCollection[k].Bounds.Height;
                                break;
                            }
                        }
                        this.chartObj.model.LegendViewerBounds.Height = legendHeight;
                        this.chartObj.model.LegendBounds.Y = ($(this.chartObj.svgObject).height() / 2) - (this.chartObj.model.LegendViewerBounds.Height / 2);
                    }
                }
            }
            else {
                if (this.chartObj.model.legend.position.toLowerCase() != 'custom') {
                    if (this.chartObj.model.legend.alignment.toLowerCase() == 'center') {
                        if (this.chartObj.model.AreaType == "cartesianaxes") 
                            this.chartObj.model.LegendBounds.X = (this.chartObj.model.m_AreaBounds.Width / 2) - (this.chartObj.model.LegendViewerBounds.Width / 2)
                            + this.chartObj.model.m_AreaBounds.X;
                        else
                            this.chartObj.model.LegendBounds.X = ($(this.chartObj.svgObject).width() / 2) - (this.chartObj.model.LegendViewerBounds.Width / 2);
                    } else if (this.chartObj.model.legend.alignment.toLowerCase() == 'near') {
                        this.chartObj.model.LegendBounds.X = this.chartObj.model.m_Spacing.Left + this.chartObj.model.elementSpacing + (this.chartObj.model.legend.itemPadding / 2);
                    } else {
                        this.chartObj.model.LegendBounds.X = $(this.chartObj.svgObject).width() - this.chartObj.model.m_Spacing.Right - this.chartObj.model.LegendViewerBounds.Width - this.chartObj.model.elementSpacing - 5;
                    }
                    //adjust the legend viewer bound width based on legend alignment
                    if (this.chartObj.model.AreaType == "cartesianaxes")
                    {
                        if (this.chartObj.model.LegendBounds.X < this.chartObj.model.m_AreaBounds.X || this.chartObj.model.m_AreaBounds.X + this.chartObj.model.LegendViewerBounds.Width >= this.chartObj.model.m_AreaBounds.Width) {
                            var legendWidth = 0;
                            for (var k = 0; k < this.chartObj.model.legendCollection.length; k++) {
                                legendWidth += this.chartObj.model.legendCollection[k].Bounds.Width;
                                if (legendWidth > this.chartObj.model.m_AreaBounds.Width) {
                                    legendWidth -= this.chartObj.model.legendCollection[k].Bounds.Width; break;
                                }
                            }
                            this.chartObj.model.LegendViewerBounds.Width = legendWidth;
                            this.chartObj.model.LegendBounds.X = (this.chartObj.model.m_AreaBounds.Width / 2) - (this.chartObj.model.LegendViewerBounds.Width / 2)
                           + this.chartObj.model.m_AreaBounds.X;
                        }
                    }
                    else
                    {
                        if(this.chartObj.model.LegendBounds.X<0||this.chartObj.model.LegendBounds.X+this.chartObj.model.LegendViewerBounds.Width>=$(this.chartObj.svgObject).width())
                        {
                            var legendWidth = 0;
                            for (var k = 0; k < this.chartObj.model.legendCollection.length; k++) {
                                legendWidth += this.chartObj.model.legendCollection[k].Bounds.Width;
                                if (legendWidth > $(this.chartObj.svgObject).width()) {
                                    legendWidth -= this.chartObj.model.legendCollection[k].Bounds.Width;
                                    break;
                                }
                            }
                            this.chartObj.model.LegendViewerBounds.Width = legendWidth;
                            this.chartObj.model.LegendBounds.X = ($(this.chartObj.svgObject).width() / 2) - (this.chartObj.model.LegendViewerBounds.Width / 2);
                            
                        }
                    }
                }
                else
                {
                    if (this.chartObj.model.LegendBounds.X + this.chartObj.model.LegendViewerBounds.Width > $(this.chartObj.svgObject).width())
                        this.chartObj.model.LegendViewerBounds.Width = $(this.chartObj.svgObject).width() - this.chartObj.model.LegendBounds.X;
                    if (this.chartObj.model.LegendBounds.Y + this.chartObj.model.LegendViewerBounds.Height > $(this.chartObj.svgObject).height())
                        this.chartObj.model.LegendViewerBounds.Height = $(this.chartObj.svgObject).height() - this.chartObj.model.LegendBounds.Y;
                }
            }
			//Set height and width to legend Canvas element
			var legendTitle = this.chartObj.model.legend.title;
			var legendHeight = ej.EjSvgRender.utils._measureText(legendTitle.text, null, legendTitle.font).height;
            this.chartObj.legendSvgContainer.height = this.chartObj.model.LegendBounds.Height + legendHeight;
            this.chartObj.legendSvgContainer.width = this.chartObj.model.LegendBounds.Width;
            if (legendTitle.text != "" && legendTitle.text)
               this.drawLegendTitle();
            var legnTx = this.chartObj.model.LegendBounds.X;

            var legnTy = this.chartObj.model.LegendBounds.Y;
            var currentX, currentY;
            var startX, startY;
            currentX = startX = this.chartObj.model.legend.itemPadding;
            currentY = startY = this.chartObj.model.legend.itemPadding;
            if (this.chartObj.model.legend.title.text != "" && this.chartObj.model.legend.title.text != null && (this.chartObj.model.legend.position.toLowerCase() != "custom")) {
                currentY = startY = this.chartObj.model.legend.itemPadding + height;
            }               
            
            for (var k = 0; k < this.chartObj.model.legendCollection.length; k++) {
                var position = this.chartObj.model.legend.position.toLowerCase();
                if (position == this.chartObj.model.legendPosition.top || position == this.chartObj.model.legendPosition.bottom || position == this.chartObj.model.legendPosition.custom || this.chartObj.model.legend.rowCount || this.chartObj.model.legend.columnCount) {
                    if ((currentX + this.chartObj.model.legendCollection[k].Bounds.Width) > this.chartObj.model.LegendBounds.Width + startX) {
                        currentX = this.chartObj.model.legend.itemPadding;
                        currentY += this.chartObj.model.legendCollection[k].Bounds.Height;
                    }
                    this.drawLegendItem(this.chartObj.model.legendCollection[k], currentX, (currentY));
                    currentX += this.chartObj.model.legendCollection[k].Bounds.Width;
                }
                else {
                    if (((currentY + this.chartObj.model.legendCollection[k].Bounds.Height) > this.chartObj.model.LegendBounds.Height + startY)) {
                        currentY = this.chartObj.model.legend.itemPadding;
                        currentX = (this.chartObj.model.legendCollection[k].Bounds.Width) + (this.chartObj.model.legend.itemPadding);

                    }
                    this.drawLegendItem(this.chartObj.model.legendCollection[k], (currentX), (currentY));
                    currentY += this.chartObj.model.legendCollection[k].Bounds.Height;
                }
            }  
            $(this.chartObj.legendContainer).removeAttr("style");
            $(this.chartObj.legendContainer).show();

       $(this.chartObj.legendContainer).css({ "position": "absolute", "left": legnTx, "top": legnTy, "width": this.chartObj.model.LegendViewerBounds.Width, "height": this.chartObj.model.LegendViewerBounds.Height+height, "border-width": this.chartObj.model.legend.border.width, "border-color": this.chartObj.model.legend.border.color, "border-style": "solid" });
            if (this.chartObj.model.LegendBounds.Height  > this.chartObj.model.LegendViewerBounds.Height) {
                $(this.chartObj.legendContainer).css({ "overflow-y": "auto" });
            }
            if (this.chartObj.model.LegendBounds.Width  > this.chartObj.model.LegendViewerBounds.Width) {
                $(this.chartObj.legendContainer).css({ "overflow-x": "auto" });
            }
            if (this.chartObj.model.LegendBounds.Width > this.chartObj.model.LegendViewerBounds.Width && this.chartObj.model.LegendBounds.Height > this.chartObj.model.LegendViewerBounds.Height)
            {
                $(this.chartObj.legendContainer).css({ "overflow":"auto"});
            }
            $(this.chartObj.legendSvgContainer).css({ "height": this.chartObj.model.LegendBounds.Height+height, "width": this.chartObj.model.LegendBounds.Width });
            this.chartObj.svgRenderer.append(this.chartObj.gLegendEle, this.chartObj.legendSvgContainer);   
            if (this.chartObj.scrolltop != undefined)
                $(this.chartObj.legendContainer).scrollTop(this.chartObj.scrolltop);
            if (this.chartObj.scrollleft != undefined)
                $(this.chartObj.legendContainer).scrollLeft(this.chartObj.scrollleft);
           
        }
        else
            $(this.chartObj.legendContainer).hide();
    }
};
})(jQuery);;

ej.seriesTypes = {};

ej.EjSeriesRender = function () {

};
ej.indicatorTypes = {};

ej.EjIndicatorRender = function () {

};
(function ($) {

    ej.EjIndicatorRender.prototype = {

        draw: function (indicator, sender) {

            this.chartObj = sender;
            var translate = null;

            if (this.chartObj.model.AreaType == "cartesianaxes") {
                var transX = indicator.xAxis.x;
                var transY = indicator.yAxis.y;
                translate = 'translate(' + transX + ',' + transY + ')';
            }

            var indicatorIndex = $.inArray(indicator, this.chartObj.model.indicators);
            var indicatorOptions = { 'id': this.chartObj.svgObject.id + '_indicatorGroup' + '_' + indicatorIndex, 'transform': translate };
            this.gIndicatorGroupEle = this.chartObj.svgRenderer.createGroup(indicatorOptions);

            
            if (indicator.points.length > 0) {
                for (var i = 0; i < indicator.segment.length; i++) {
                    var segment = indicator.segment[i];
                    var style = { fill: segment.fill, width: segment.width, opacity: indicator.opacity };
                        segment.isIndicator = true;
                        segment.xAxis = indicator.xAxis;
                        segment.yAxis = indicator.yAxis;
                         if (segment.type == "line")
                            this.calculateDirection(segment, style);
                        else
                            this.drawColumn(segment, style, sender);
                    }
                }

        },

        _drawLinePath: function (indicator, style, lDirection) {
            
          
          
            var indicatorIndex = $.inArray(indicator, this.chartObj.model.indicator);
            if (lDirection != "") {
                var options = {
                    'id': this.chartObj.svgObject.id + "_" + indicatorIndex + "_" + indicator.name,
                    'fill': 'none',
                    'stroke-dasharray': indicator.dashArray,
                    'stroke-width': style.width,
                    'stroke': style.fill,
                    'stroke-linecap': indicator.lineCap,
                    'stroke-linejoin': indicator.lineJoin,
                    'opacity': style.opacity,
                    'd': lDirection
                };

                this.chartObj.svgRenderer.drawPath(options, this.gIndicatorGroupEle);
            }

            this.chartObj.svgRenderer.append(this.gIndicatorGroupEle, this.chartObj.gIndicatorEle);
        },

        calculateDirection: function (currentseries, style) {
		    var canvasX = 0;
            var canvasY = 0;
            if (this.chartObj.model.enableCanvasRendering) {
                canvasX = currentseries.xAxis.x;
                canvasY = currentseries.yAxis.y;
            }
            currentseries.points = ej.DataManager(currentseries.points, ej.Query().sortBy("X")).executeLocal();

            var visiblePoints = ej.EjSeriesRender.prototype._isVisiblePoints(currentseries);
            var lDirection;
            var sb = ej.EjSvgRender.utils._getStringBuilder();

            var firstPoint = null;
            var secondPoint;
            var firstIndex = -1;
            for (var i = 0; i < visiblePoints.length; i++) {
                secondPoint = visiblePoints[i];
                if (secondPoint.visible) {
                    if (firstPoint != null) {
                        if (visiblePoints.length > firstIndex + 1) {
                            var nextpoint = visiblePoints[firstIndex + 1];
                            var point1 = ej.EjSvgRender.utils._getPoint(firstPoint, currentseries);
                            var point2 = ej.EjSvgRender.utils._getPoint(nextpoint, currentseries);
                            
                            sb.append("M" + " " + (point1.X + canvasX) + " " + ((point1.Y + canvasY)) + " " + "L" + " " + (point2.X + canvasX) + " " + ((point2.Y + canvasY)) + " ");

                        }

                    }
                    firstPoint = secondPoint;
                    firstIndex = i;

                }
                else {
                    firstPoint = null;
                }

            }

            lDirection = sb.toString();

            this._drawLinePath(currentseries, style, lDirection);
        },

         drawColumn: function (option, style, chart) {
            this.chartObj = chart;
            var series = option;
            var seriesIndex = $.inArray(series, this.chartObj.model.indicators);
            var origin = Math.max(option.yAxis.visibleRange.min, 0);

            var sidebysideinfo = {Start: -0.35, End: 0.35};//ej.EjSeriesRender.prototype.getSideBySideInfo(series, this);
            var visiblePoints = ej.EjSeriesRender.prototype._isVisiblePoints(series);

            var cSer = this;
            for (var i = 0; i < visiblePoints.length; i++) {
                var pointIndex = i;
                var point = visiblePoints[i];

                var y1 = point.YValues[0];
                var y2 = origin;
                if (point.visible) {
                    //calculate sides
                    var data = ej.EjSeriesRender.prototype.calculateSides(point, sidebysideinfo);
                    var x1 = data.x1;
                    var x2 = data.x2;
                    var rect = ej.EjSeriesRender.prototype.getRectangle(x1, y1, x2, y2, series);

                    if (rect.X < 0) {
                        rect.Width = rect.Width + rect.X;
                        rect.X = 0;
                    }

                    rect.Width = ((rect.X + rect.Width) > cSer.chartObj.model.m_AreaBounds.Width) ? cSer.chartObj.model.m_AreaBounds.Width - rect.X : rect.Width;
                    //drawing part
                    var xr = Math.min(0, rect.Width);
                    var yr = Math.min(0, rect.Height);

                    var bounds;
                    if ((xr == 0 || yr == 0) && rect.Width > 0) {
                        options = {
                            'id': this.chartObj.svgObject.id + "_" + seriesIndex + "_" + series.name,
                            'x': rect.X + cSer.chartObj.canvasX,
                            'y': rect.Y + cSer.chartObj.canvasY,
                            'width': rect.Width,
                            'height': rect.Height,
                            'fill': style.fill,
                            'stroke-width': series.border.width,
                            'plot': y1 < 0 ? "negative" : "positive",
                            'opacity': series.histogram.opacity,
                            'stroke': series.border.color
                        };

                        this.chartObj.svgRenderer.drawRect(options, this.gIndicatorGroupEle);

                        var svgXy = ej.EjSvgRender.utils._getSvgXY(rect.X, (rect.Y), series, cSer.chartObj);
                        bounds = { X: svgXy.X, Y: svgXy.Y, Width: rect.Width, Height: rect.Height };
                        ej.EjSvgRender.utils._addRegion(cSer.chartObj, bounds, series, point, pointIndex);
                    }
                }
            }

            this.chartObj.svgRenderer.append(this.gIndicatorGroupEle, this.chartObj.gIndicatorEle);
        },
		
		calculateSegment: function (indicator, sender) {
            this.chartObj = sender;
              indicator.segment = [];
              indicator.xRange = {};
              indicator.yRange = {};

              //Upperband 
              var upperSeries = $.extend(true, {}, indicator);
              upperSeries.name = "upperseries";
              upperSeries.fill = indicator.upperLine.fill;
              upperSeries.width = indicator.upperLine.width;
              upperSeries.type = "line";

              for (var k = 0; k < indicator.points.length; k++) {
                  indicator.points[k].YValues.length = 1;
                  indicator.points[k].YValues[0] = indicator.points[k].y = 70;
              }
              
              upperSeries.points = $.extend(true, [], indicator.points);
              var xValues = sender._getXValues(upperSeries);
              var yValues = sender._getYValues(upperSeries);
              
              indicator.xRange.min = Math.min.apply(Math, xValues);
              indicator.xRange.max = Math.max.apply(Math, xValues);
              indicator.yRange.min = Math.min.apply(Math, yValues);
              indicator.yRange.max = Math.max.apply(Math, yValues);

              indicator.segment.push(upperSeries);

              //Lowerband
              var lowerSeries = $.extend(true, {}, indicator);
              lowerSeries.name = "lowerSeries";
              lowerSeries.width = indicator.lowerLine.width;
              lowerSeries.fill = indicator.lowerLine.fill;
              lowerSeries.type = "line";
           

              for (var n = 0; n < indicator.points.length; n++) {
                  indicator.points[n].YValues[0] = indicator.points[n].y = 30;
              }

               lowerSeries.points = $.extend(true, [], indicator.points);

              yValues = sender._getYValues(lowerSeries);

              indicator.yRange.min = Math.min(Math.min.apply(Math, yValues), indicator.yRange.min);
              indicator.yRange.max = Math.max(Math.max.apply(Math, yValues), indicator.yRange.max);

              indicator.segment.push(lowerSeries);
              
              // signalLine
              var trendSeries = $.extend(true, {}, indicator); 
              trendSeries.name = "signalSeries";
              trendSeries.width = indicator.width;
              trendSeries.fill = indicator.fill;
              trendSeries.type = "line";
              
              var c = 0, c1 = 0;
              var pmf = 0;
              var nmf = 0;
              var len = indicator.period;
              c1 = indicator.points[0].close;

              for (var i = 1; i <=len; ++i) {
                  c = indicator.points[i].close;
                  if (c > c1)
                      pmf += c - c1;
                  else if (c < c1)
                      nmf += c1 - c;
                  c1 = c;
                  indicator.points[i-1].YValues[0] = indicator.points[i-1].y = null;
              }
              pmf = pmf / len;
              nmf = nmf / len;
              
              indicator.points[len].YValues[0] = indicator.points[len].y = 100 - 100 / (1 + pmf / nmf);

              for (var j = len + 1; j < indicator.points.length; j++) {
                  c = indicator.points[j].close;
                  if (c > c1) {
                      pmf = (pmf * (len - 1) + (c - c1)) / len;
                      nmf = (nmf * (len - 1)) / len;
                  }
                  else if (c < c1) {
                      nmf = (nmf * (len - 1) + (c1 - c)) / len;
                      pmf = (pmf * (len - 1)) / len;
                  }
                  c1 = c;
                  indicator.points[j].YValues[0] = indicator.points[j].y = 100 - (100 / (1 + pmf / nmf));
              }
              trendSeries.points = $.extend(true, [], indicator.points);
              var trendPoints = [];
              for (var i = len, j = 0; i < trendSeries.points.length; i++, j++)
                  trendPoints[j] = trendSeries.points[i].y;

              indicator.yRange.min = Math.min(Math.min.apply(Math, trendPoints), indicator.yRange.min);
              indicator.yRange.max = Math.max(Math.max.apply(Math, trendPoints), indicator.yRange.max);

              indicator.segment.push(trendSeries);
              this.calculateIndicatorRange(sender, indicator);
           
        },

        calculateIndicatorRange: function(sender, indicator){
            //Indicator X-Axis Calculation
            if (sender.model.indicatorRange[indicator.xAxisName] == undefined) {
                sender.model.indicatorRange[indicator.xAxisName] = {};
                sender.model.indicatorRange[indicator.xAxisName] = { min: indicator.xRange.min, max: indicator.xRange.max };
            } else {
                if (sender.model.indicatorRange[indicator.xAxisName].min > indicator.xRange.min)
                    sender.model.indicatorRange[indicator.xAxisName].min = indicator.xRange.min;
                if (sender.model.indicatorRange[indicator.xAxisName].max < indicator.xRange.max)
                    sender.model.indicatorRange[indicator.xAxisName].max = indicator.xRange.max;
            }
            //Indicator Y-Axis Calculation
            if (sender.model.indicatorRange[indicator.yAxisName] === undefined) {
                sender.model.indicatorRange[indicator.yAxisName] = {};
                sender.model.indicatorRange[indicator.yAxisName] = { min: indicator.yRange.min, max: indicator.yRange.max };
            } else {
                if (sender.model.indicatorRange[indicator.yAxisName].min > indicator.yRange.min)
                    sender.model.indicatorRange[indicator.yAxisName].min = indicator.yRange.min;
                if (sender.model.indicatorRange[indicator.yAxisName].max < indicator.yRange.max)
                    sender.model.indicatorRange[indicator.yAxisName].max = indicator.yRange.max;
            }
        },

        doAnimation: function () {
            var chartObj = this.chartObj;
            var clipRect = $(chartObj.gIndicatorEle).find("#" + this.gIndicatorGroupEle.id + "_ClipRect").children();
            $(clipRect).animate(
                { width: parseFloat(chartObj.svgRenderer._getAttrVal($(chartObj.gIndicatorEle).find("#" + this.gIndicatorGroupEle.id + "_ClipRect").children(), "width")) },
                {
                    duration: 2000,

                    step: function (now) {
                        chartObj.svgRenderer._setAttr($(clipRect), { "width": now });
                    }
                });
        }

    },
    ej.ejRSI = ejExtendClass(ej.EjIndicatorRender);

    ej.indicatorTypes.rsi = ej.ejRSI;

     ej.ejMACD = ejExtendClass(ej.EjIndicatorRender, {


        calculateSegment: function (indicator, sender) {
            this.chartObj = sender;
            indicator.segment = [];
            indicator.xRange = {};
            indicator.yRange = {};

            var pointsLength = indicator.points.length;
            var longPeriod = indicator.longPeriod;
            var shortPeriod = indicator.shortPeriod;
            var trigger = indicator.trigger;
            var diff = longPeriod-shortPeriod;
            var shortEMA = [];
            var longEMA = [];
            var macdPoints = [];
            var signalEMA = [];
            var histogramPoints = [];
            shortEMA = this.calculateEMA(indicator.points, shortPeriod, true);
            longEMA = this.calculateEMA(indicator.points, longPeriod, true);

            //Calculte MACD Line points: MACD Line: (shortPeriod EMA - longPeriod EMA)  
            for (i = 0; i < longEMA.length; i++)
                macdPoints.push((shortEMA[i + diff] - longEMA[i]));

            //MACD Line 
            var macdSeries = $.extend(true, {}, indicator);
            macdSeries.name = "macdSeries";
            macdSeries.fill = indicator.macdLine.fill;
            macdSeries.width = indicator.macdLine.width;
            macdSeries.type = "line";
            macdSeries.xAxis = indicator.xAxis;
            macdSeries.yAxis = indicator.yAxis;

            var l = longPeriod - 1;
            for (var t = 0, i=0; t < pointsLength; t++) {
                if (t < l)
                    indicator.points[t].YValues[0] = indicator.points[t].y = null;
                else{
                    indicator.points[t].YValues[0] = indicator.points[t].y = macdPoints[i];
                    i += 1;
                }
            }
            macdSeries.points = $.extend(true, [], indicator.points);
            //////////////////////////////////////////////

            //Signal Line: Trigger value(9-day default) EMA of MACD Line 
            signalEMA = this.calculateEMA(macdPoints, trigger, false);
            var trendSeries = $.extend(true, {}, indicator);
            trendSeries.name = "signalSeries";
            trendSeries.fill = indicator.fill;
            trendSeries.width = indicator.width;
            trendSeries.type = "line";
            trendSeries.xAxis = indicator.xAxis;
            trendSeries.yAxis = indicator.yAxis;

            var l = longPeriod + trigger - 1;
            for (var t = 0, i = 0; t < pointsLength; t++) {
                if (t < l)
                    indicator.points[t].YValues[0] = indicator.points[t].y = null;
                else {
                    indicator.points[t].YValues[0] = indicator.points[t].y = signalEMA[i];
                    i += 1;
                }
            }
            trendSeries.points = $.extend(true, [], indicator.points);
            /////////////////////////////////////////

            //MACD Histogram: MACD Line - Signal Line
            var histogramSeries = $.extend(true, {}, indicator);
            histogramSeries.border = {};
            histogramSeries.name = "histogramSeries";
            histogramSeries.fill = indicator.histogram.fill;
            histogramSeries.opacity = indicator.histogram.opacity;
            histogramSeries.fill = indicator.histogram.fill;
            histogramSeries.border = indicator.histogram.border;
            histogramSeries.type = "column";
            histogramSeries.xAxis = indicator.xAxis;
            histogramSeries.yAxis = indicator.yAxis;

            for (var j = 0, k = l; k < pointsLength; j++, k++)
                histogramPoints.push((macdPoints[j + (trigger-1)]) - (signalEMA[j]));
            for (var j = 0, k = l; k < pointsLength; j++, k++) {
                indicator.points[k].YValues[0] = indicator.points[k].y = histogramPoints[j];
            }
            histogramSeries.points = $.extend(true, [], indicator.points);
            switch (indicator.macdType) {
                case "line":
                    {
                        //Push MACD Line 
                        xValues = sender._getXValues(macdSeries);
                        indicator.xRange.min = Math.min.apply(Math, xValues);
                        indicator.xRange.max = Math.max.apply(Math, xValues);
                        indicator.yRange.min = Math.min.apply(Math, macdPoints);
                        indicator.yRange.max = Math.max.apply(Math, macdPoints);
                        indicator.segment.push(macdSeries);

                        //Push Signal Line
                        var min = Math.min.apply(Math, signalEMA), max = Math.max.apply(Math, signalEMA);
                        indicator.yRange.min = indicator.yRange.min < min ? indicator.yRange.min : min;
                        indicator.yRange.max = indicator.yRange.max > max ? indicator.yRange.max : max;
                        indicator.segment.push(trendSeries);
                    }
                    break;
                case "histogram":
                    {
                        //Push histogram Series
                        var min1 = Math.min.apply(Math, histogramPoints), max1 = Math.max.apply(Math, histogramPoints);
                        indicator.yRange.min = indicator.yRange.min < min1 ? indicator.yRange.min : min1;
                        indicator.yRange.max = indicator.yRange.max > max1 ? indicator.yRange.max : max1;
                        indicator.segment.push(histogramSeries);
                    }
                    break;
                case "both":
                    {
                        //Push histogram Series
                        var min1 = Math.min.apply(Math, histogramPoints), max1 = Math.max.apply(Math, histogramPoints);
                        indicator.yRange.min = indicator.yRange.min < min1 ? indicator.yRange.min : min1;
                        indicator.yRange.max = indicator.yRange.max > max1 ? indicator.yRange.max : max1;
                        indicator.segment.push(histogramSeries);

                        //Push MACD Line 
                        xValues = sender._getXValues(macdSeries);
                        indicator.xRange.min = Math.min.apply(Math, xValues);
                        indicator.xRange.max = Math.max.apply(Math, xValues);
                        indicator.yRange.min = Math.min.apply(Math, macdPoints);
                        indicator.yRange.max = Math.max.apply(Math, macdPoints);
                        indicator.segment.push(macdSeries);

                        //Push Signal Line
                        var min = Math.min.apply(Math, signalEMA), max = Math.max.apply(Math, signalEMA);
                        indicator.yRange.min = indicator.yRange.min < min ? indicator.yRange.min : min;
                        indicator.yRange.max = indicator.yRange.max > max ? indicator.yRange.max : max;
                        indicator.segment.push(trendSeries);
                    }
                    break;
            }
            this.calculateIndicatorRange(sender, indicator);
        },
        calculateEMA: function (points, period, flag) {
            var sma = 0;
            var initialEMA = 0;
            var pointsLength = points.length;
            var getPoints = [];
            var getEma = [];
            var emaPercent = (2 / (period + 1));
            if (flag) { // calculating EMA for Indicators Points
                for (var i = 0; i < period; i++) {
                    getPoints.push(points[i].close);
                    sma = sma + getPoints[i];
                }

                initialEMA = (sma / period);
                getEma.push(initialEMA);
                var emaAvg = initialEMA;
                for (j = period; j < pointsLength; j++) {
                    emaAvg = (points[j].close - emaAvg) * emaPercent + emaAvg;
                    getEma.push(emaAvg);
                }
            } else { //Calculating EMA for Signal Points
                for (var i = 0; i < period; i++) {
                    getPoints.push(points[i]);
                    sma = sma + getPoints[i];
                }

                initialEMA = (sma / period);
                getEma.push(initialEMA);
                var emaAvg = initialEMA;
                for (j = period; j < pointsLength; j++) {
                    emaAvg = (points[j] - emaAvg) * emaPercent + emaAvg;
                    getEma.push(emaAvg);
                }
            }
            return getEma;
        }

    });

    ej.indicatorTypes.macd = ej.ejMACD;
    ej.EjSeriesRender.prototype = {

    setLineSeriesStyle: function (currentSeries) {

        var seriesIndex = $.inArray(currentSeries, this.chartObj.model._visibleSeries);
        var serColor = this.chartObj.model.seriesColors[seriesIndex];
       
        
        var seriesInterior = this.chartObj.svgRenderer.createGradientElement(this.getSeriesName(currentSeries) + seriesIndex, serColor, 0, 0, 0, $(this.chartObj.svgObject).height(), this.chartObj.gSeriesEle);
        var seriesBorder = this.chartObj.model.seriesBorderColors[seriesIndex];
        return { SeriesInterior: seriesInterior, SeriesBorder: seriesBorder };

    },
    getSeriesName: function(series) {
        var serName;
        if (series.name)
            serName = series.name.replace(/\s/g, '');
        else
            serName = "series";
        
        return serName;
    },
    
    drawAreaPath: function (series, style, direction, canvasTranslate) {
        var seriesIndex = $.inArray(series, this.chartObj.model.series);
        var translate = null;
        if (this.chartObj.model.AreaType == "cartesianaxes") {
            var transX = series.xAxis.x;
            var transY = series.yAxis.y;
            translate = 'translate(' + transX + ',' + transY + ')'
        }
        var serOptions = { 'id': this.chartObj.svgObject.id + '_SeriesGroup' + '_' + seriesIndex, 'transform': translate};

        this.gSeriesGroupEle = this.chartObj.svgRenderer.createGroup(serOptions);
        if (direction != "") {
            var options = {
                'id': this.chartObj.svgObject.id + "_" + seriesIndex,
                'fill': style.SeriesInterior,
                'fill-opacity': series.opacity,
                'stroke-width': series.border.width,
                'stroke': series.border.color,
                'stroke-linecap': series.lineCap,
                'stroke-linejoin': series.lineJoin,
                'd': direction
            };
            this.chartObj.svgRenderer.drawPath(options, this.gSeriesGroupEle, canvasTranslate);
        } 
    },

    setAreaSeriesStyle: function (series) {

        var seriesIndex = $.inArray(series, this.chartObj.model._visibleSeries);
        var maxY = Math.max.apply(0,$.map(series.points, function(v) {return v.YValues[0];
        }));
        var minY = Math.min.apply(0, $.map(series.points, function(v) {
            return v.YValues[0];
        }));
        var minX = Math.min.apply(0, $.map(series.points, function (v) {
            return v.X;
        }));

        var point1Xy = ej.EjSvgRender.utils._getPointXY(minX, minY, series);
        var point2Xy = ej.EjSvgRender.utils._getPointXY(minX, maxY, series);
        var serColor = this.chartObj.model.seriesColors[seriesIndex];

        var seriesInterior = this.chartObj.svgRenderer.createGradientElement(this.getSeriesName(series) + seriesIndex, serColor, point1Xy.X, point1Xy.Y, point2Xy.X, point2Xy.Y, this.chartObj.gSeriesEle);

        return { SeriesInterior: seriesInterior };

    },





    naturalSpline: function (points) {
        var count = points.length;
        //constant
        var yIndex = 0, a = 6;
        var ySpline = [], ySplineDuplicate = [];

        // assigning the first and last value as zero
        ySpline[0] = ySplineDuplicate[0] = 0;
        ySpline[points.length - 1] = 0;

        for (var i = 1; i < count - 1; i++) {
            var d1 = points[i].X - points[i - 1].X;
            var d2 = points[i + 1].X - points[i - 1].X;
            var d3 = points[i + 1].X - points[i].X;
            var dy1 = points[i + 1].YValues[yIndex] - points[i].YValues[yIndex];
            var dy2 = points[i].YValues[yIndex] - points[i - 1].YValues[yIndex];

            if (d1 == 0 || d2 == 0 || d3 == 0) {
                ySpline[i] = 0;
                ySplineDuplicate[i] = 0;
            }
            else {
                var p = 1 / (d1 * ySpline[i - 1] + 2 * d2);

                ySpline[i] = -p * d3;
                ySplineDuplicate[i] = p * (a * (dy1 / d3 - dy2 / d1) - d1 * ySplineDuplicate[i - 1]);
            }
        }

        for (var k = count - 2; k >= 0; k--)
            ySpline[k] = ySpline[k] * ySpline[k + 1] + ySplineDuplicate[k];


        return ySpline;

    },
    getBezierControlPoints: function (point1, point2, ySpline1, ySpline2, yIndex) {
        var controlPoint1;
        var controlPoint2;
        var one3 = 1 / 3.0;
        var deltaX2 = (point2.X - point1.X);

        deltaX2 = deltaX2 * deltaX2;

        var dx1 = 2 * point1.X + point2.X;
        var dx2 = point1.X + 2 * point2.X;

        var dy1 = 2 * point1.YValues[yIndex] + point2.YValues[yIndex];
        var dy2 = point1.YValues[yIndex] + 2 * point2.YValues[yIndex];

        var y1 = one3 * (dy1 - one3 * deltaX2 * (ySpline1 + 0.5 * ySpline2));
        var y2 = one3 * (dy2 - one3 * deltaX2 * (0.5 * ySpline1 + ySpline2));

        controlPoint1 = { X: dx1 * one3, YValues: [y1] };
        controlPoint2 = { X: dx2 * one3, YValues: [y2] };

        return { controlPoint1: controlPoint1, controlPoint2: controlPoint2 };


    },

    improveChartPerformance: function(chartSeries) {
        var tempPoints = [];
       
        var xDelta = chartSeries.xAxis.visibleRange.max - chartSeries.xAxis.visibleRange.min;
        var yDelta = chartSeries.yAxis.visibleRange.max - chartSeries.yAxis.visibleRange.min;
       
        var xTolerance = Math.abs((xDelta * 1) / this.chartObj.model.m_AreaBounds.Width);
        var yTolerance = Math.abs((yDelta * 1) / this.chartObj.model.m_AreaBounds.Height);

        var prevXValue = (chartSeries.points[0] && chartSeries.points[0].X > xTolerance) ? 0 : xTolerance;
        var prevYValue = (chartSeries.points[0] && chartSeries.points[0].y > yTolerance) ? 0 : yTolerance;

        var xVal = 0;
        for (var k = 0; k < chartSeries.points.length; k++) {
            var isEmpty = (chartSeries.points[k].isEmpty === null || chartSeries.points[k].isEmpty === undefined) ? false : chartSeries.points[k].isEmpty;
            chartSeries.points[k].visible = (chartSeries.points[k].visible === null || chartSeries.points[k].visible === undefined) ? true : chartSeries.points[k].visible;
            if (chartSeries.points[k].X === null || chartSeries.points[k].X === undefined) {
                chartSeries.points[k].isEmpty = true;
                chartSeries.points[k].visible = false;
                chartSeries.points[k].X = chartSeries.xAxis.visibleRange.min;
            } else
                chartSeries.points[k].isEmpty = isEmpty;
            if ((chartSeries.points[k].YValues[0] == null) || (chartSeries.points[k].YValues[0] == 'undefined')) {
                chartSeries.points[k].isEmpty = true;
                chartSeries.points[k].visible = false;
                chartSeries.points[k].YValues[0] = chartSeries.yAxis.visibleRange.min;
            } else {
                chartSeries.points[k].isEmpty = chartSeries.points[k].isEmpty;
                chartSeries.points[k].visible = (chartSeries.points[k].isEmpty) ? !chartSeries.points[k].isEmpty : chartSeries.points[k].visible;
            }

            var xVal = chartSeries.points[k].X;
            var yVal = chartSeries.points[k].YValues[0];
            if (Math.abs(prevXValue - xVal) >= xTolerance || Math.abs(prevYValue - yVal) >= yTolerance) {

                tempPoints.push(chartSeries.points[k]);
                prevXValue = xVal;
                prevYValue = yVal;
            }
        }
        return tempPoints;

    },
    _isVisiblePoints: function(chartSeries) {
        for (var k = 0; k < chartSeries.points.length; k++) {
            var isEmpty = (ej.util.isNullOrUndefined(chartSeries.points[k].isEmpty)) ? false : chartSeries.points[k].isEmpty;
            chartSeries.points[k].visible = (ej.util.isNullOrUndefined(chartSeries.points[k].visible)) ? true : chartSeries.points[k].visible;
            if (ej.util.isNullOrUndefined(chartSeries.points[k].X)) {
                chartSeries.points[k].isEmpty = true;
                chartSeries.points[k].visible = false;
                chartSeries.points[k].X = chartSeries.xAxis.visibleRange.min;
            } else
                chartSeries.points[k].isEmpty = isEmpty;
            if ((chartSeries.points[k].YValues[0]==null) || (chartSeries.points[k].YValues[0]=='undefined')) {
                chartSeries.points[k].isEmpty = true;
                chartSeries.points[k].visible = false;
                if (ej.util.isNullOrUndefined(chartSeries.points[k].YValues))
                    chartSeries.points[k].YValues = [];
                chartSeries.points[k].YValues[0] = chartSeries.yAxis.visibleRange.min;
            } else {
                chartSeries.points[k].isEmpty = chartSeries.points[k].isEmpty;
                chartSeries.points[k].visible = (chartSeries.points[k].isEmpty) ? !chartSeries.points[k].isEmpty : chartSeries.points[k].visible;
            }
            if (jQuery.type(chartSeries.points[k].X) == "date")
                chartSeries.points[k].X = (chartSeries.points[k].X).getTime();

        }
        return chartSeries.points;
    },
   
    _calculatePolarAxesSegment: function (currentseries) {

        var points = currentseries.visiblePoints;
        var segment = ej.EjSvgRender.utils._getStringBuilder();
        var SeriesIndex=$.inArray(currentseries, this.chartObj.model._visibleSeries)
        var size;
        var interval;
        var centerX = this.chartObj.model.centerX;
        var centerY = this.chartObj.model.centerY;
        // calculating side by side info.
        var sideInfo = this._getSeriesPosition(currentseries);

        size = $.extend(true, {}, currentseries.marker.size, size);
        if (size.width <= 10) {
            size.width = size.height = (1.5 * this.chartObj.model.elementSpacing);
        }
        if (currentseries.drawType.toLowerCase() == 'column' && !currentseries.regionAdded) {
           
            var innerRadius = 0;
            var serOptions = { 'id': this.chartObj.svgObject.id + '_SeriesGroup' + '_' + SeriesIndex };
            this.gSeriesGroupEle = this.chartObj.svgRenderer.createGroup(serOptions);
            var interval = currentseries.xAxis.visibleRange.interval;
            var ticks = (currentseries.xAxis.labelPlacement && currentseries.xAxis.labelPlacement.toLowerCase() == "onticks" && this.chartObj.model.isPolar) ? interval / 2 : 0;
            var inversed = (currentseries.xAxis.isInversed) ? 0 : 1;
            interval = interval / sideInfo.all;
            var nextPoint = interval;
            // calculation for start and end angle for each positions.
            for (var l = 0; l < currentseries.visiblePoints.length; l++) {
                var itemCurrentXPos=temp= (currentseries.visiblePoints[l].X - currentseries.xAxis.visibleRange.min) + ((interval) * sideInfo.pos - ticks);
                itemCurrentXPos = (itemCurrentXPos) / this.chartObj.model.sumofYValues;
                point = currentseries.visiblePoints[l];
               
                var nextPoint = (currentseries.xAxis.visibleRange.interval) / sideInfo.all;
                point.startAngle = 2 * Math.PI * itemCurrentXPos;
                point.endAngle = 2 * Math.PI * (itemCurrentXPos + ((nextPoint) / this.chartObj.model.sumofYValues));
            }
        }
        for (var i = 0; i < points.length; i++) {
            var point1;
            var point2;

            // drawType 'Line' calculation
            if (currentseries.drawType.toLowerCase() == 'line') {
                point1 = ej.EjSvgRender.utils.TransformToVisible(currentseries, points[i].X, points[i].y, this.chartObj);

                if (i < points.length - 1) {
                    point2 = ej.EjSvgRender.utils.TransformToVisible(currentseries, points[i + 1].X, points[i + 1].y, this.chartObj);
                    segment.append("M" + " " + (point1.X) + " " + (point1.Y) + " " + "L" + " " + (point2.X) + " " + (point2.Y) + " ");
                }
                    // when isClosed is true
                else {
                    if (currentseries.isClosed) {
                        point2 = ej.EjSvgRender.utils.TransformToVisible(currentseries, points[0].X, points[0].y, this.chartObj);
                        segment.append("M" + " " + (point1.X) + " " + (point1.Y) + " " + "L" + " " + (point2.X) + " " + (point2.Y) + " ");
                    }
                }            
            }

            else if (currentseries.drawType.toLowerCase() == 'column') {
                var interval = currentseries.xAxis.visibleRange.interval;
                var inversed= (currentseries.xAxis.isInversed) ? 0 : 1;
                var interval = interval / sideInfo.all;
                // radius for each point based on the y values.
                var radius = this.chartObj.model.Radius * ej.EjSvgRender.utils._valueToCoefficient(currentseries.yAxis, points[i].y);
                
                var chartStartAngle = -.5 * Math.PI;
                var startAngle = points[i].startAngle + chartStartAngle;
                var endAngle = points[i].endAngle + chartStartAngle - 0.000001;

                var x1,y1, x2, y2;
           
                // calculating arc when the Stacking mode is enabled
               
                if (currentseries.isStacking) {
                    radius = this.chartObj.model.Radius * ej.EjSvgRender.utils._valueToCoefficient(currentseries.yAxis, currentseries.stackedValue.EndValues[i]);

                    x1 = centerX + radius * Math.cos(startAngle);
                    y1 = centerY + radius * Math.sin(startAngle);

                    x2 = centerX + radius * Math.cos(endAngle);
                    y2 = centerY + radius * Math.sin(endAngle);

                    innerRadius = this.chartObj.model.Radius * ej.EjSvgRender.utils._valueToCoefficient(currentseries.yAxis, currentseries.stackedValue.StartValues[i]);

                    var dStartX = centerX + innerRadius * Math.cos(startAngle);
                    var dStartY = centerY + innerRadius * Math.sin(startAngle);

                    var dEndX = centerX + innerRadius * Math.cos(endAngle);
                    var dEndY = centerY + innerRadius * Math.sin(endAngle);
                     if (this.chartObj.model.isPolar) 
                            direction = "M" + " " + x1 + " " + y1 + " " + "A" + " " + radius + " " + radius + " " + "0" + " " + "0" + " " + inversed + " " + x2 + " " + y2 + " " + "L" + " " + dEndX + " " + dEndY + " " + "A" + " " + innerRadius + " " + innerRadius + " " + "1" + " " + "0" + " " + "0" + " " + dStartX + " " + dStartY + " " + "z";
                       else
                            direction = "M" + " " + x1 + " " + y1 + " " + "L" + " " + x2 + " " + y2 + " " + "L" + " " + dEndX + " " + dEndY + " " + "L" + " " + dStartX + " " + dStartY + " " + "z";
                    }
                        // calculating arc for normal mode.
                    else {

                        x1 = centerX + radius * Math.cos(startAngle);
                        y1 = centerY + radius * Math.sin(startAngle);

                        x2 = centerX + radius * Math.cos(endAngle);
                        y2 = centerY + radius * Math.sin(endAngle);
                      if (this.chartObj.model.isPolar)
                            direction = "M" + " " + x1 + " " + y1 + " " + "A" + " " + radius + " " + radius + " " + "0" + " " + "0" + " " + inversed + " " + x2 + " " + y2 + " " + "L" + " " + centerX + " " + centerY + " " + "z";
                     else
                          direction = "M" + " " + x1 + " " + y1 + " " + "L" + " " + x2 + " " + y2 + " " + "L" + " " + centerX + " " + centerY + " " + "z";
                    }                   
        
                   // gradient creation
                var seriesInterior = this.chartObj.svgRenderer.createGradientElement(this.getSeriesName(currentseries) + "_" + SeriesIndex + i, this.chartObj.model.seriesColors[SeriesIndex], 0, 0, 0, 100, this.chartObj.gSeriesEle);
                options = {
                     'id': this.chartObj.svgObject.id + "_" + this.getSeriesName(currentseries) + SeriesIndex + "_path" + i,
                     'fill': seriesInterior,
                    'stroke': currentseries.border.color,
                    'stroke-width': currentseries.border.width,
                    'seriesIndex': SeriesIndex,
                    'pointIndex': i,
                    'd': direction,
                    'start': points[i].startAngle - 1.57,
                    'end': points[i].endAngle - 1.57,
                    'radius': radius,
                    'innerR': innerRadius
                };
                if (this.chartObj.model.isPolar) {
                    options.x = centerX;
                    options.y = centerY;
					bounds = { PointIndex: i, StartAngle: points[i].startAngle, EndAngle: points[i].endAngle, CenterX: centerX, CenterY: centerY, Radius: radius, DRadius: innerRadius };
                }
                point = currentseries.visiblePoints[i];
                if ((!ej.util.isNullOrUndefined(currentseries.marker) && currentseries.marker.visible) || (!ej.util.isNullOrUndefined(currentseries.marker) && currentseries.marker.dataLabel.visible))
                    point.symbolLocation = { X: ((x1 + x2) / 2), Y: ((y1 + y2) / 2) };
                            
                this.chartObj.svgRenderer.drawPath(options, this.gSeriesGroupEle);

            }

                // drawType 'Area' calculation
            else {
                point1 = ej.EjSvgRender.utils.TransformToVisible(currentseries, points[i].X, points[i].y, this.chartObj);
                if (i == 0) {
                    segment.append("M" + " " + (point1.X) + " " + (point1.Y) + " ");
                }
                    // when isClosed is enabled
                else if (i == points.length - 1) {
                    if(!currentseries.isClosed){
                        segment.append("L" + " " + (point1.X) + " " + (point1.Y) + " " + "L" + " " + (centerX) + " " + (centerY)  );
                    }
                    else
                        segment.append("L" + " " + (point1.X) + " " + (point1.Y) + " " + "z");
                }
                    
                else {
                    segment.append("L" + " " + (point1.X) + " " + (point1.Y) + " ");
                }
            }
            if (!currentseries.regionAdded && currentseries.drawType.toLowerCase() != 'column') {
                var bounds = { X: point1.X - size.width / 2, Y: point1.Y - size.height / 2, Width: size.width, Height: size.height };
                ej.EjSvgRender.utils._addRegion(this.chartObj, bounds, currentseries, points[i], i);
            }
			if (!currentseries.regionAdded && currentseries.drawType.toLowerCase() == 'column' && this.chartObj.model.enableCanvasRendering)
                ej.EjSvgRender.utils._addRegion(this.chartObj, bounds, currentseries, null, null); 
        }

        if (!currentseries.regionAdded && currentseries.drawType.toLowerCase() == 'column' && !this.chartObj.model.enableCanvasRendering)
            ej.EjSvgRender.utils._addRegion(this.chartObj, bounds, currentseries, null, null);
        return segment.toString();
    },
    _pieVisiblePoints: function (currentseries) {
        var visiblePoints = [];
        var visibleLabels = [];
        var point;
        for (var i = 0; i < currentseries.points.length; i++) {
            point = currentseries.points[i];
            if (ej.util.isNullOrUndefined(point.visible))
                point.visible = true;
            if (point.y < 0) {
                point.YValues[0] = Math.abs(point.YValues[0])
            }
            if (point.visible == true && !ej.util.isNullOrUndefined(point.y)) {
                visiblePoints.push(point);
                visibleLabels.push(point.text ? point.text : point.y);
            }
        }
        currentseries.visiblePoints = visiblePoints;
        currentseries.visibleLabels = visibleLabels;
        return visiblePoints;
    },
    getMinMaxValue: function (point1, point2, degree) {
        var minX = Math.min(point1.x, point2.x);
        var minY = Math.min(point1.y, point2.y);
        var maxX = Math.max(point1.x, point2.x);
        var maxY = Math.max(point1.y, point2.y);
        var pointValue;
        switch (degree) {
            case 0:
            case 360:
                pointValue = maxY;
                break;
            case 90:
                pointValue = minX;
                break;
            case 180:
                pointValue = minY;
                break;
            case 270:
                pointValue = maxX;
                break;
        }
        return pointValue;
    },
    pieDoughnutCenter:function(currentseries)
    {
        var oldCenterX = this.chartObj.model.centerX;
        var oldCenterY = this.chartObj.model.centerY;
        var startAngle = currentseries.startAngle;
        var endAngle = currentseries.endAngle;
        var regions = [-630,-540,-450,-360,-270,-180,-90,0, 90, 180, 270,360,450,540,630];
        var region = [];
        
        if (currentseries.startAngle < currentseries.endAngle) {
            for (i = 0; i < regions.length;i++) {
                if (regions[i] > startAngle && regions[i] < endAngle)
                    region[region.length] = (regions[i] % 360) < 0 ? (regions[i] % 360) + 360 : (regions[i] % 360);
            }
            }
        else {
            for (i = 0; i < regions.length; i++) {
                if (regions[i] < startAngle && regions[i] > endAngle)
                    region[region.length] = (regions[i] % 360) < 0 ? (regions[i] % 360) + 360 : (regions[i] % 360);
            }
        }
        var radius = this.chartObj.model.Radius;
        var startRadian = 2 * Math.PI * (startAngle - 90) / 360;
        var endRadian = 2 * Math.PI * (endAngle - 90) / 360;
        var centerPoint = { x: this.chartObj.model.centerX, y: this.chartObj.model.centerY };
        var startPoint = { x: this.chartObj.model.centerX + radius * Math.cos(startRadian), y: this.chartObj.model.centerY + radius * Math.sin(startRadian) }
        var endPoint = { x: this.chartObj.model.centerX + radius * Math.cos(endRadian), y: this.chartObj.model.centerY + radius * Math.sin(endRadian) }
        switch (region.length) {
            case 0:
                var longX = Math.abs(centerPoint.x - startPoint.x) > Math.abs(centerPoint.x - endPoint.x) ? startPoint.x : endPoint.x;
                var longY = Math.abs(centerPoint.y - startPoint.y) > Math.abs(centerPoint.y - endPoint.y) ? startPoint.y : endPoint.y;
                var midPoint = { x: (centerPoint.x + longX) / 2, y: (centerPoint.y + longY) / 2 }
                this.chartObj.model.centerX = this.chartObj.model.centerX + (this.chartObj.model.centerX - midPoint.x);
                this.chartObj.model.centerY = this.chartObj.model.centerY + (this.chartObj.model.centerY - midPoint.y);
                break;
            case 1:
                var maxRadian = 2 * Math.PI * (region[0] - 90) / 360;
                var maxPoint = { x: this.chartObj.model.centerX + radius * Math.cos(maxRadian), y: this.chartObj.model.centerY + radius * Math.sin(maxRadian) }
                switch (region[0]) {
                    case 0:
                    case 360:
                        point1 = { x: startPoint.x, y: maxPoint.y };
                        point2 = { x: endPoint.x, y: centerPoint.y };
                        break;
                    case 90:
                        point1 = { x: centerPoint.x, y: endPoint.y };
                        point2 = { x: maxPoint.x, y: startPoint.y };
                        break;
                    case 180:
                        point1 = { x: endPoint.x, y: centerPoint.y };
                        point2 = { x: startPoint.x, y: maxPoint.y };
                        break;
                    case 270:
                        point1 = { x: maxPoint.x, y: startPoint.y };
                        point2 = { x: centerPoint.x, y: endPoint.y };
                        break;
                }
                var midPoint = { x: (point1.x + point2.x) / 2, y: (point1.y + point2.y) / 2 }
                this.chartObj.model.centerX = this.chartObj.model.centerX + ((this.chartObj.model.centerX - midPoint.x) >= radius ? 0 : (this.chartObj.model.centerX - midPoint.x));
                this.chartObj.model.centerY = this.chartObj.model.centerY + ((this.chartObj.model.centerY - midPoint.y) >= radius ? 0 : (this.chartObj.model.centerY - midPoint.y));
                break;
            case 2:
                var minRadian = 2 * Math.PI * (region[0] - 90) / 360;
                var maxRadian = 2 * Math.PI * (region[1] - 90) / 360;
                var maxPoint = { x: this.chartObj.model.centerX + radius * Math.cos(maxRadian), y: this.chartObj.model.centerY + radius * Math.sin(maxRadian) }
                var minPoint = { x: this.chartObj.model.centerX + radius * Math.cos(minRadian), y: this.chartObj.model.centerY + radius * Math.sin(minRadian) }
                var point1;
                var point2
                if (region[0] == 90 && region[1] == 180 || region[0] == 270 && region[1] == 0)
                    point1 = { x: minPoint.x, y: maxPoint.y }
                else 
                    point1 = { x: maxPoint.x, y: minPoint.y }
                if (region[0] == 90 || region[0] == 270)
                    point2 = { x: this.getMinMaxValue(startPoint, endPoint, region[0]), y: this.getMinMaxValue(startPoint, endPoint, region[1]) }
                else
                    point2 = { x: this.getMinMaxValue(startPoint, endPoint, region[1]), y: this.getMinMaxValue(startPoint, endPoint, region[0]) }
                var midPoint = { x: Math.abs(point1.x - point2.x) / 2 >= radius ? 0 : (point1.x + point2.x) / 2, y: Math.abs(point1.y - point2.y) / 2 >= radius ? 0 : (point1.y + point2.y) / 2 }
                this.chartObj.model.centerX = this.chartObj.model.centerX + (midPoint.x == 0 ? 0 : (this.chartObj.model.centerX - midPoint.x) >= radius ? 0 : (this.chartObj.model.centerX - midPoint.x));
                this.chartObj.model.centerY = this.chartObj.model.centerY + (midPoint.y == 0 ? 0 : (this.chartObj.model.centerY - midPoint.y) >= radius ? 0 : (this.chartObj.model.centerY - midPoint.y));
                break;
        }
        for (var i = 0; i < currentseries.points.length; i++) {
            this.chartObj.model.startX[i] =this.chartObj.model.startX[i]-( oldCenterX - this.chartObj.model.centerX);
            this.chartObj.model.startY[i] =this.chartObj.model.startY[i]-( oldCenterY - this.chartObj.model.centerY);
        }
    },
    calculatingSliceAngle: function (currentseries) {
        var visiblePoints = this._pieVisiblePoints(currentseries);
        var legXSpace = 0;
        var legYSpace = 0;
        this.chartObj.model.arcData = [];
        currentseries.rightsidePoints = [];
        currentseries.leftsidePoints = [];
        currentseries.labels = [];
        this.chartObj.model.chartRegions = [];
        this.chartObj.model.sumofYValues = 0;
        this.chartObj.model.midPoint = 0;
        this.chartObj.model.centerX = 0, this.chartObj.model.centerY = 0;
        this.chartObj.model.startX = [], this.chartObj.model.startY = [];
        var point;
        this.chartObj.model.Radius = 0;
        var itemCurrentXPos = 0;

        for (j = 0; j < visiblePoints.length; j++) {
            this.chartObj.model.sumofYValues += visiblePoints[j].YValues[0];
        }
        var totalDegree;
        if (currentseries.endAngle != null)
            totalDegree = (currentseries.endAngle - currentseries.startAngle);
        else
            totalDegree = 360;
        totalDegree = (totalDegree != 360 && totalDegree != -360) ? totalDegree % 360 : totalDegree;
        if (currentseries.startAngle) {
            currentseries.startAngle = currentseries.startAngle % 360;
            currentseries.endAngle = currentseries.startAngle + totalDegree;
            itemCurrentXPos = (currentseries.startAngle) ? ((this.chartObj.model.sumofYValues / 360) * currentseries.startAngle) : 0;
            itemCurrentXPos = itemCurrentXPos / this.chartObj.model.sumofYValues;
            this.chartObj.model.itemCurrentXPos = itemCurrentXPos;
        }
        else
            currentseries.endAngle =totalDegree;
        var legend = this.chartObj.model.legend;
        if (legend.visible && legend.position.toLowerCase() != "custom") {
            if (legend.position.toLowerCase() == "right" || legend.position.toLowerCase() == "left")
                legXSpace = ((legend.position.toLowerCase() == "right") ? this.chartObj.model.margin.right : this.chartObj.model.margin.left) + this.chartObj.model.LegendViewerBounds.Width;
            else
                legYSpace = ((legend.position.toLowerCase() == "top") ? this.chartObj.model.margin.top : this.chartObj.model.margin.bottom) + this.chartObj.model.LegendViewerBounds.Height;

        }

        var yOffset = ((this.chartObj.model.title.text) ? this.chartObj.model._titleLocation.Y : 0) + legYSpace;

        var actualWidth = $(this.chartObj.svgObject).width() - legXSpace;
        var actualHeight = $(this.chartObj.svgObject).height() - yOffset;

        this.chartObj.model.centerX = actualWidth * 0.5 + ((legend.position.toLowerCase() === "left") ? legXSpace : 0);
        this.chartObj.model.centerY = actualHeight * 0.5 + ((legend.position.toLowerCase() === "top") ? yOffset : ((this.chartObj.model.title.text) ? (this.chartObj.model._titleLocation.Y) : 0));
        var endAngle;
        totalDegree = currentseries.endAngle - currentseries.startAngle;
        var pieFactor = totalDegree / 180;
        for (var l = 0; l < currentseries.visiblePoints.length; l++) {
            point = currentseries.visiblePoints[l];
            if (point.visible) {
                var pointIndex = l;
                if (l == 0)
                    point.startAngle = 2 * Math.PI * itemCurrentXPos;
                else
                    point.startAngle = endAngle;
                    endAngle = point.endAngle = pieFactor*Math.PI * (point.YValues[0] / this.chartObj.model.sumofYValues) + point.startAngle;
            point.midAngle = (point.endAngle + point.startAngle) / 2;
            point.pointIndex = pointIndex;
            var chartStartingAngle = -.5 * Math.PI;
            
            point.radian = (point.midAngle) % (2 * Math.PI)
            
            if (currentseries.labelPosition.toLowerCase() == "outsideextended") {

                if (point.radian < Math.PI)
                    currentseries.rightsidePoints.push(point);
                else
                   currentseries.leftsidePoints.push(point);
            }
            itemCurrentXPos += point.YValues[0] / this.chartObj.model.sumofYValues;

            var midAngle = point.midAngle + chartStartingAngle;

            if ((pointIndex == currentseries.explodeIndex || currentseries.explodeAll) && !this.chartObj.vmlRendering) {

                this.chartObj.model.startX[pointIndex] = this.chartObj.model.centerX + Math.cos(midAngle) * currentseries.explodeOffset;
                this.chartObj.model.startY[pointIndex] = this.chartObj.model.centerY + Math.sin(midAngle) * currentseries.explodeOffset;
            }
            else {
                this.chartObj.model.startX[pointIndex] = this.chartObj.model.centerX;
                this.chartObj.model.startY[pointIndex] = this.chartObj.model.centerY;
            }
        }
          
        }

        if (currentseries.labelPosition.toLowerCase() == "outsideextended") {
            currentseries.rightsidePoints = ej.DataManager(currentseries.rightsidePoints, ej.Query().sortBy("radian")).executeLocal();
            currentseries.leftsidePoints = ej.DataManager(currentseries.leftsidePoints, ej.Query().sortBy("radian")).executeLocal();
        }
        if (currentseries.marker.dataLabel.template) {
            if (currentseries.labelPosition.toLowerCase() == "outsideextended") {
                for (var i = 0; i < currentseries.rightsidePoints.length; i++) {
                    ej.EjSvgRender.utils._getSeriesTemplateSize(currentseries.rightsidePoints[i], currentseries.rightsidePoints[i].pointIndex, currentseries, false, this.chartObj);
                }
                for (var k = 0; k < currentseries.leftsidePoints.length; k++) {
                    ej.EjSvgRender.utils._getSeriesTemplateSize(currentseries.leftsidePoints[k], currentseries.leftsidePoints[k].pointIndex, currentseries, true, this.chartObj);
                }

            }
            else {
                for (var j = 0; j < currentseries.visiblePoints.length; j++) {
                    ej.EjSvgRender.utils._getSeriesTemplateSize(currentseries.visiblePoints[j], currentseries.visiblePoints[j].pointIndex, currentseries, true, this.chartObj);
                }
            }
            
        }
        else {
            ej.EjSvgRender.utils._getSeriesMaxLabel(currentseries);
        }

      
        currentseries.finalSize= { width: actualWidth, height: actualHeight };
        return currentseries.finalSize;
    },
	 _drawHiloPath: function (series, style, interior, direction, i, pointbounds) {

	     var seriesIndex = $.inArray(series, this.chartObj.model._visibleSeries);

        var options = {
            'id': this.chartObj.svgObject.id + "_" + seriesIndex + "_" + i,
            'fill':"none" ,
            'stroke-dasharray': series.dashArray,
            'stroke-width': style.borderWidth,
            'stroke': interior,
            'stroke-linecap': series.lineCap,
            'stroke-linejoin': series.lineJoin,
            'opacity': series.opacity,
            'd': direction
        };
        this.chartObj.svgRenderer.drawPath(options, this.gSeriesGroupEle);
        var valWidth, valHeight;
        x = (Math.min(pointbounds.point2.X, pointbounds.point1.X) + series.xAxis.x) - (style.borderWidth);
	    y = (Math.min(pointbounds.point2.Y , pointbounds.point1.Y) + series.yAxis.y) ;
        if (pointbounds.point1.Y != pointbounds.point2.Y) {
            valWidth = style.borderWidth + (style.borderWidth / 2);
              valHeight = Math.abs(pointbounds.point1.Y - pointbounds.point2.Y);
        }
	     else {
            valWidth = Math.abs(pointbounds.point2.X - pointbounds.point1.X);
            valHeight = style.borderWidth + (style.borderWidth / 2); 
        }
        var bounds = { X: x, Y: y, Width: valWidth, Height: valHeight };
        ej.EjSvgRender.utils._addRegion(this.chartObj, bounds, series, null, i);
      
	 },
	 isIntersec: function (other, currBounds) {
	     if (currBounds.Bottom < other.Top || currBounds.Right < other.Left
               || currBounds.Top > other.Bottom || currBounds.Left > other.Right) {
	         return false;
	     }
	      return true;
	 },
	 IntersectWith: function (sender, point, currBounds) {
	     var chartObj = this;
	     var intersect = false;
	     var bounds;
	     if (point.radian > Math.PI && (sender.model.bounds["left"])) {
	         bounds = (sender.model.bounds["left"]);
	         if (bounds.length > 0) {
	             for (var i = point.index + 1; i < bounds.length; i++) {
	                 var other = bounds[i];
	                 if (other && this.isIntersec(other, currBounds)) {
	                     return true;
	                 }
	             }
	         }

	     }
	     else {
	         bounds = (sender.model.bounds["right"]) ? sender.model.bounds["right"] : sender.model.bounds;
	         var chartObj = this;
   	         if (bounds.length > 0) {
   	               for (var i = point.index - 1; i >= 0; i--) {
   	                 var other = bounds[i];
   	                 if (other && this.isIntersec(other, currBounds)) {
   	                     return true;
   	                 }
   	             }
   	         }
	     }
	     return intersect;
	     
	 },
   
   
	 _calculateArcData: function (sender, pointIndex, point, series) {
	     var visiblePoints = series.visiblePoints;
	     var chartObj = sender;
	     var chartStartAngle = -.5 * Math.PI;
	     var currBounds;
	     var startAngle = point.startAngle + chartStartAngle;
	     var endAngle = point.endAngle + chartStartAngle - 0.000001;
	     var totalDegree = series.endAngle - series.startAngle;
	     var longArc = endAngle - startAngle < Math.PI ? 0 : 1;
	     var midAngle = (startAngle + endAngle) / 2;
	     var direction;
	     var startX;
	     var startY
	     var clockWise=totalDegree > 0?1:0;
	     var longArc = clockWise ? endAngle - startAngle < Math.PI ? 0 : 1 : endAngle - startAngle > -1 * Math.PI ? 0 : 1;
	    
	     if ((pointIndex == series.explodeIndex || series.explodeAll) && !chartObj.vmlRendering) {
           
	         startX = chartObj.model.centerX + Math.cos(midAngle) * series.explodeOffset;
	         startY = chartObj.model.centerY + Math.sin(midAngle) * series.explodeOffset;
	     }
	     else {
	         startX = chartObj.model.centerX;
	         startY = chartObj.model.centerY;
	     }


	     var x1 = startX + chartObj.model.Radius * Math.cos(startAngle);
	     var y1 = startY + chartObj.model.Radius * Math.sin(startAngle);

	     var x2 = startX + chartObj.model.Radius * Math.cos(endAngle);
	     var y2 = startY + chartObj.model.Radius * Math.sin(endAngle);
	    
	     if (series.type.toLowerCase() == "doughnut") {
	         var dStartX = startX + chartObj.model.dRadius * Math.cos(startAngle);
	         var dStartY = startY + chartObj.model.dRadius * Math.sin(startAngle);

	         var dEndX = startX + chartObj.model.dRadius * Math.cos(endAngle);
	         var dEndY = startY + chartObj.model.dRadius * Math.sin(endAngle);
	         var dClockWise = clockWise ? 0 : 1;
	         direction = "M" + " " + x1 + " " + y1 + " " + "A" + " " + chartObj.model.Radius + " " + chartObj.model.Radius + " " + "0" + " " + longArc + " " + clockWise + " " + x2 + " " + y2 + " " + "L" + " " + dEndX + " " + dEndY + " " + "A" + " " + chartObj.model.dRadius + " " + chartObj.model.dRadius + " " + "1" + " " + longArc + " " + dClockWise + " " + dStartX + " " + dStartY + " " + "z";
	     }

	     else{
	         direction = "M" + " " + x1 + " " + y1 + " " + "A" + " " + chartObj.model.Radius + " " + chartObj.model.Radius + " " + "0" + " " + longArc + " " + clockWise + " " + x2 + " " + y2 + " " + "L" + " " + startX + " " + startY + " " + "z";
	     }

        // display label for pie and doughnut chart
	     

        return { Direction: direction };

    },

    getXCordinate: function (x, radius, angle) {
        var x1 = x + radius * (Math.cos(angle));
        return x1;
    },

    getYCordinate: function (y, radius, angle) {
        var y1 = y + radius * (Math.sin(angle));
        return y1;
    },
 getDoubleRange: function (start, end) {
        var mstart;
        var mend;
        if (start > end) {
            mstart = end;
            mend = start;
        }
        else {
            mstart = start;
            mend = end;
        }

        var mdelta = mend - mstart;
        var mmedian = (mstart + mend) / 2;
        var misEmpty = isNaN(mstart) || isNaN(mend);

        return { Start: mstart, End: mend, Delta: mdelta, Median: mmedian, IsEmpty: misEmpty };

    },
    isDependentSeries: function (series) {
        var type = series.type.toLowerCase();
        if (!this.chartObj.model.requireInvertedAxes)
            return (type == ej.datavisualization.Chart.Type.Candle || type == ej.datavisualization.Chart.Type.Column || type == ej.datavisualization.Chart.Type.Hilo || type == ej.datavisualization.Chart.Type.HiloOpenClose || type == ej.datavisualization.Chart.Type.StackingColumn || type == ej.datavisualization.Chart.Type.RangeColumn) ? true : false;
        else
            return (type == ej.datavisualization.Chart.Type.Bar || type == ej.datavisualization.Chart.Type.StackingBar) ? true : false;
    },

    getPointXYOrgin: function (x, y, series) {
        var xvalue = ej.EjSvgRender.utils._valueToCoefficient(series.xAxis, x) * (series.xAxis.width);
        var yvalue = (1 - ej.EjSvgRender.utils._valueToCoefficient(series.yAxis, y)) * (series.yAxis.height);
        return { X: xvalue, Y: yvalue };
    },
    getRectangle: function (x1, y1, x2, y2, series) {
        var pt1 = this.getPointXYOrgin(x1, y1, series);
        var pt2 = this.getPointXYOrgin(x2, y2, series);
        return ej.EjSvgRender.utils._correctRect(pt1.X, pt1.Y, pt2.X, pt2.Y);
    },
    calculateSides: function (point, sidebysideinfo) {
        var x1 = point.X + sidebysideinfo.Start;
        var x2 = point.X + sidebysideinfo.End;
        return { x1: x1, x2: x2 };
    },
    _getSeriesPosition: function (currentSeries) {
     // calculation for position of series in radar and polar.
        if (ej.util.isNullOrUndefined(currentSeries.position) || this.chartObj.model.legendCollapsed) {
            var stacked;
            var stackingposition
            var all=0;
            var seriesCollection=[];
            for (var i = 0; i < this.chartObj.model._visibleSeries.length; i++) {
                var series=this.chartObj.model._visibleSeries[i];
                if (series.visibility.toLowerCase() === 'visible' && series.drawType.toLowerCase() == 'column') {
                    seriesCollection.push(series);
                }
            }
            for (var j = 0; j < seriesCollection.length; j++) {
                var series = seriesCollection[j];
                    if (series.isStacking) {
                        if (!stacked) {
                            stackingposition = all;
                            all++;
                            stacked = true;
                        }
                        series.position = stackingposition;
                    }
                    else {
                        series.position = all;
                        all++;
                      }
            }
            for (var k = 0; k < seriesCollection.length; k++) 
                seriesCollection[k].all = all;
            
        }
        return { all: currentSeries.all, pos: currentSeries.position };
    },
     getSideBySidePositions: function (currentSeries) {

         var chart = this;
         var stackingGroup = [];
         var model = chart.chartObj.model;
         
         if (ej.util.isNullOrUndefined(currentSeries.position) || model.legendCollapsed) {
             var hAxes = (model.requireInvertedAxes) ? model.vAxes : model.hAxes;
             var vAxes = (model.requireInvertedAxes) ? model.hAxes : model.vAxes;

             $.each(hAxes, function (index, hAxis) {
                 var all = 0;
                 var seriesCollection = [];
                 for (j = 0; j < model._visibleSeries.length; j++) {
                     var series = model._visibleSeries[j];
                     if (series.visibility.toLowerCase() === 'visible') {
                         if (chart.isDependentSeries(series)) {
                             if (hAxis.name == series.xAxisName) {
                                 seriesCollection.push(series);
                             }
                         }
                     }
                 }
                 var vAxeseries = [];
                 for (var j = 0; j < seriesCollection.length; j++) {
                     var index = (model.requireInvertedAxes) ? seriesCollection[j].yAxis.columnIndex : seriesCollection[j].yAxis.rowIndex;
                     if (!vAxeseries[index])
                         vAxeseries[index] = [];
                     vAxeseries[index].push(seriesCollection[j]);
                 }

                 for (var i = 0; i < vAxeseries.length; i++) {
                     if (!ej.util.isNullOrUndefined(vAxeseries[i])) {
                         vAxeseries[i].all = 0;
                         for (var j = 0; j < vAxeseries[i].length; j++) {
                             series = vAxeseries[i][j];
                             if (series.type.toLowerCase().indexOf("stacking") != -1) {
                                 if (series.stackingGroup) {
                                     if (ej.util.isNullOrUndefined(stackingGroup[series.stackingGroup])) {
                                         vAxeseries[i].all++;
                                         series.position = vAxeseries[i].all;
                                         stackingGroup[series.stackingGroup] = vAxeseries[i].all;
                                     }
                                     else
                                         series.position = stackingGroup[series.stackingGroup];
                                 }
                                 else {
                                     if (ej.util.isNullOrUndefined(series.yAxis.position) || model.legendCollapsed) {
                                         vAxeseries[i].all++;
                                         series.position = vAxeseries[i].all;
                                         series.yAxis.position = vAxeseries[i].all;
										 model.legendCollapsed=false;
                                     }
                                     else
                                         series.position = series.yAxis.position;
                                 }
                             }
                             else {
                                 vAxeseries[i].all++;
                                 series.position = vAxeseries[i].all;
                             }
                         }
                     }
                 }

                 for (var k = 0; k < seriesCollection.length; k++) {
                     var index = (seriesCollection[k].type.toLowerCase().indexOf("bar") != -1) ? seriesCollection[k].yAxis.columnIndex : seriesCollection[k].yAxis.rowIndex;
                     if (seriesCollection[k].type.toLowerCase().indexOf("stacking") != -1)
                         seriesCollection[k].all = vAxeseries[index].all;
                     else
                         seriesCollection[k].all = vAxeseries[index].length;
                 }

             });

         }
         return { all: currentSeries.all, pos: currentSeries.position };
     },
	
    getSideBySideInfo: function (series) {
        var pos;
        var all;

        var data = this.getSideBySidePositions(series);
        pos = data.pos;
        all = data.all;

        var seriesSpacing = 0;
        var pointSpacing = 0.3 - 0;
        if (series.xAxis.m_minPointsDelta == undefined)
            series.xAxis.m_minPointsDelta = ej.EjSvgRender.utils.getMinPointsDelta(series.xAxis, this.chartObj);
        var width = series.xAxis.m_minPointsDelta * (1 - pointSpacing);
        if (!this.chartObj.model.sideBySideSeriesPlacement && this.chartObj.model.enable3D)
            return this.getDoubleRange(-width / 2, width / 2);
        var div = 1 / all;
        var loc = 0.5 - div * (pos - 1);

        var range = this.getDoubleRange(-loc, -loc + div);

        // multiplying with width  and scaling
        if (!range.IsEmpty) {
            range = this.getDoubleRange(range.Start * width, range.End * width);
            var radius = 0.5 * (1 - seriesSpacing) * range.Delta;
            var median = range.Median;
            range = this.getDoubleRange(median - radius, median + radius);
        }

        return range;

    },



    draw: function (chart, options) {

        this.chartObj = chart;
        
        //DrawLineGraph
        var lDirection;
        var sb = ej.EjSvgRender.utils._getStringBuilder();
        var currentseries = options;
        var style = this.setLineSeriesStyle(currentseries);
        if (currentseries.sorting)
            currentseries.points = ej.DataManager(currentseries.points, ej.Query().sortBy("X")).executeLocal();
        var visiblePoints = this.improveChartPerformance(currentseries);

        var firstPoint = null;
        var secondPoint;
        var firstIndex = -1;
        for (var i = 0; i < visiblePoints.length; i++) {
            secondPoint = visiblePoints[i];
            if (secondPoint.visible) {
                if (firstPoint != null) {
                    if (visiblePoints.length > firstIndex + 1) {
                        var nextpoint = visiblePoints[firstIndex + 1];
                        var point1 = ej.EjSvgRender.utils._getPoint(firstPoint,currentseries);
                        var point2 = ej.EjSvgRender.utils._getPoint(nextpoint,currentseries);
                        var chartObj = this.chartObj;
                        sb.append("M" + " " + (point1.X + chartObj.canvasX) + " " + ((point1.Y + chartObj.canvasY)) + " " + "L" + " " + (point2.X + chartObj.canvasX) + " " + ((point2.Y + chartObj.canvasY)) + " ");

                    }

                }
                firstPoint = secondPoint;
                firstIndex = i;

            }
            else {
                firstPoint = null;
            }

        }

        lDirection = sb.toString();

        this._drawLinePath(currentseries, style, lDirection);
        
        this.chartObj.svgRenderer.append(this.gSeriesGroupEle,this.chartObj.gSeriesEle);
        

    },
    _doLineSymbol: function(element, sec, val, series, invertedAxes) {
        var beginTime = parseInt(val * sec);
        var chartObj=this.chartObj;
        var box = ej.EjSvgRender.utils._getBoundingClientRect(element, chartObj, series, invertedAxes);
        var centerX = box.x + (box.width / 2);
        var centerY = box.y + (box.height / 2);
        var scaleVal;
        var $ele = $(element);
        var rotate = chartObj.svgRenderer._getAttrVal($ele, "transform");
        rotate = !ej.util.isNullOrUndefined(rotate) ? rotate : " " ;
        chartObj.svgRenderer._setAttr($ele, { "transform": "scale(0)" });
        $ele.delay(beginTime).animate(
            {
                scale: 1
            },
            {
                duration: 200,

                step: function(now) {
                    scaleVal = now;
                    chartObj.svgRenderer._setAttr($ele, { "transform": "translate(" + centerX + " " + centerY + ") scale(" + scaleVal + ") translate(" + (-centerX) + " " + (-centerY) + ")" + rotate });

                }
            }
        );
    },
    animateSymbol: function(element,delayInterval,series,invertedAxes) {
        var chartObj=this.chartObj;
        var box = ej.EjSvgRender.utils._getBoundingClientRect(element, chartObj, series, invertedAxes);
        var centerX = box.x + (box.width / 2);
        var centerY = box.y + (box.height / 2);
        var scaleVal;
        var $ele = $(element);
        var rotate = chartObj.svgRenderer._getAttrVal($ele, "transform");
        rotate = !ej.util.isNullOrUndefined(rotate) ? rotate : " ";
        chartObj.svgRenderer._setAttr($ele,{"transform": "scale(0)"});
        $ele.delay(delayInterval).each(function () {this.scale = 0.5; }).animate(
            {
                scale: 1
            },
            {
                duration: 500,

                step: function(now) {
                    scaleVal = now;
                    chartObj.svgRenderer._setAttr($ele, { "transform": "translate(" + centerX + " " + centerY + ") scale(" + scaleVal + ") translate(" + (-centerX) + " " + (-centerY) + ")" + (rotate) });
                }
            }
        );
    },
    animateStackingRect: function (element, series, invertedAxes) {
        var chartObj = this.chartObj;

        var boxY = (1 - ej.EjSvgRender.utils._valueToCoefficient(series.yAxis, 0)) * (series.yAxis.height);
        var box = ej.EjSvgRender.utils._getBoundingClientRect(element, chartObj, series, invertedAxes);

        var centerX, centerY;
        var $element = $(element);
        if (chartObj.svgRenderer._getAttrVal($element, "plot")) {
            centerX = box.x;
            centerY = boxY;
        }
        var scaleVal;
        $element.animate(
            {
                scale: 1
            },
            {
                duration: 1000,

                step: function (now) {
                    scaleVal = now;
                    chartObj.svgRenderer._setAttr($element, { "transform": "translate(" + centerX + " " + centerY + ") scale(1," + scaleVal + ") translate(" + (-centerX) + " " + (-centerY) + ")" });
                }
            });
    },
    animateRect: function (element, series) {
        var chartObj = this.chartObj;
        var box = ej.EjSvgRender.utils._getBoundingClientRect(element, chartObj, series, false);
        var centerX, centerY;
        var $element = $(element);
        if (chartObj.svgRenderer._getAttrVal($element,"plot")) {
            if (chartObj.svgRenderer._getAttrVal($element,"plot") === "negative") {
                centerX = box.x;
                centerY = box.y;
            } else {
                centerX = (box.x +box.width);
                centerY = (box.y + box.height);
            }
        } else {    
            centerX = (box.x + (box.width / 2));
            centerY = box.y + (box.height / 2);
        }
        var scaleVal;
        $element.animate(
            {
                scale: 1
            },
            {
                duration: 1000,

                step: function(now) {
                    scaleVal = now;
                    chartObj.svgRenderer._setAttr($element, { "transform": "translate(" + centerX + " " + centerY + ") scale(1," + scaleVal + ") translate(" + (-centerX) + " " + (-centerY) + ")" });
                }
            });

    },
    animateBarRect: function (element,series,invertedAxes) {
        var chartObj = this.chartObj;
        var box = ej.EjSvgRender.utils._getBoundingClientRect(element, chartObj, series, invertedAxes);
        var centerX, centerY;
        var $element= $(element);
        if (chartObj.svgRenderer._getAttrVal($element,"plot") === "negative") {
            centerX = box.x + box.width;
            centerY = box.y + box.height;
        } else {
            
            centerX = box.x;
            centerY = box.y;
        }
        var scaleVal;
        $element.animate(
            {
                scale: 1
            },
            {
                duration: 1000,

                step: function (now) {
                    scaleVal = now; 
                    chartObj.svgRenderer._setAttr($element, {"transform": "translate(" + centerX + " " + centerY + ") scale(" + scaleVal + ",1) translate(" + (-centerX) + " " + (-centerY) + ")"});
                }
            });

    },
    animateStackingBar: function (element, series, invertedAxes) {
        var chartObj = this.chartObj;

        var boxX = (ej.EjSvgRender.utils._valueToCoefficient(series.yAxis, 0)) * (series.yAxis.width);
        var box = ej.EjSvgRender.utils._getBoundingClientRect(element, chartObj, series, invertedAxes);

        var centerX, centerY;
        var $element = $(element);
       
            centerX = boxX;
            centerY = box.y;
        
        var scaleVal;
        $element.animate(
            {
                scale: 1
            },
            {
                duration: 1000,

                step: function (now) {
                    scaleVal = now;
                    chartObj.svgRenderer._setAttr($element, { "transform": "translate(" + centerX + " " + centerY + ") scale(" + scaleVal + ",1) translate(" + (-centerX) + " " + (-centerY) + ")" });
                }
            });
    },
    _doLineAnimation: function (series) {
        var chartObj=this.chartObj;
        var clipRect = $(chartObj.gSeriesEle).find("#" + this.gSeriesGroupEle.id + "_ClipRect").children();
        $(clipRect).animate(
            { width: parseFloat(chartObj.svgRenderer._getAttrVal($(chartObj.gSeriesEle).find("#" + this.gSeriesGroupEle.id + "_ClipRect").children(),"width")) },
            {
                duration: 2000,

                step: function (now) {
                    chartObj.svgRenderer._setAttr($(clipRect), { "width": now });
                }
            });
        var elements = $(chartObj.gSymbolGroupEle).children().not("defs");
        var secondsPerPoint = 2000 / elements.length;
        for (var i = 0; i < elements.length; i++) {
            var element = elements[i];
            this._doLineSymbol(element, secondsPerPoint, i, series, this.chartObj.model.requireInvertedAxes);
        }        
        

    },
    doAnimation: function (series) {

        this._doLineAnimation(series);
    },
    
    _drawLinePath: function (series, style, direction) {

        var seriesIndex = $.inArray(series, this.chartObj.model._visibleSeries);
        var translate = null;
        if (this.chartObj.model.AreaType == "cartesianaxes") {
            var transX = series.xAxis.x;
            var transY = series.yAxis.y;
            translate = 'translate(' + transX + ',' + transY + ')'
        }
        
        var serOptions = { 'id': this.chartObj.svgObject.id + '_SeriesGroup' + '_' + seriesIndex, 'transform': translate };
        this.gSeriesGroupEle = this.chartObj.svgRenderer.createGroup(serOptions);
        if (direction != "") {
            var options = {
                'id': this.chartObj.svgObject.id + "_" + seriesIndex,
                'fill': 'none',
                'stroke-dasharray': series.dashArray,
                'stroke-width': series.width,
                'stroke': style.SeriesInterior,
                'stroke-linecap': series.lineCap,
                'stroke-linejoin': series.lineJoin,
                'opacity': series.opacity,
                'd': direction
            };

            this.chartObj.svgRenderer.drawPath(options, this.gSeriesGroupEle);
        }


    },
   
   
    
    

    drawDataLabel: function (series, index, point, x, y,chartRegionIndex) {
	       // method for data label symbols - cartesianaxes
                var labelFormat = series.yAxis.labelFormat?series.yAxis.labelFormat:"";
                var pointText = (point.text) ? point.text : point.y + labelFormat.substring(labelFormat.indexOf('}') + 1);
                var seriesIndex = $.inArray(series, this.chartObj.model._visibleSeries);
                var textAlign, textBaseline;
				var marker = point.marker? point.marker : series.marker;
			    marker = $.extend(true, {}, series.marker, marker);
                var dataLabelFont = marker.dataLabel.font;
                var size, fontStyle, fontFamily;
				var type = series.type.toLowerCase();
                size = ej.util.isNullOrUndefined(dataLabelFont.size) ? "11px" : dataLabelFont.size;
                fontStyle = ej.util.isNullOrUndefined(dataLabelFont.fontStyle) ? "Normal" : dataLabelFont.fontStyle;
                fontFamily = ej.util.isNullOrUndefined(dataLabelFont.fontFamily) ? "Segoe UI" : dataLabelFont.fontFamily;
                    var labelfont = { size: size, fontStyle: fontStyle, fontFamily: fontFamily };
                    var textOffset = ej.EjSvgRender.utils._measureText(pointText, null, labelfont);
                   
                    var position = this.textPosition(series,seriesIndex, textOffset, type, x, y,chartRegionIndex);

                    var commonEventArgs = $.extend({}, ej.EjSvgRender.commonChartEventArgs);
                    commonEventArgs.data = { text: pointText, location: { x: position.x + this.chartObj.canvasX, y: position.y + this.chartObj.canvasY }, seriesIndex: seriesIndex, pointIndex: index};
                    this.chartObj._trigger("displayTextRendering", commonEventArgs);
                    
                    var dataLabelOffset = marker.dataLabel.offset;
					if (series.marker.dataLabel.offset) {
                        if (series.marker.dataLabel.textPosition == 'bottom')
                            commonEventArgs.data.location.y = commonEventArgs.data.location.y + dataLabelOffset;
                        else
                            commonEventArgs.data.location.y = commonEventArgs.data.location.y - dataLabelOffset;
                    }
					
                    if (!commonEventArgs.cancel)
                    {
                          var options ={
                            'id': this.chartObj.svgObject.id + '_SeriesText' + index + seriesIndex,
                            'x': commonEventArgs.data.location.x,
                            'y': commonEventArgs.data.location.y,
                            'fill': dataLabelFont.color,
                            'font-size': dataLabelFont.size,
                            'font-family': dataLabelFont.fontFamily,
                            'font-style': dataLabelFont.fontStyle,
                            'font-weight': dataLabelFont.fontWeight,
                            'text-anchor': 'middle'
                          };
					   var margin = marker.dataLabel.margin;
                       var width = textOffset.width + margin.left + margin.right;
                       var height = textOffset.height + margin.top + margin.bottom;
                       var location = { X: commonEventArgs.data.location.x, Y: commonEventArgs.data.location.y };

                        if (ej.util.isNullOrUndefined(series.marker.dataLabel.template)) {
                            if (marker.dataLabel.shape)
                                var element = marker.dataLabel.shape;
                            else if (series.marker.dataLabel.shape)
                                element=  marker.dataLabel.shape;
                            else 
                                element = "None";

                            $.each(this.chartObj.model.symbolShape, function (name) {
                                if (element.toLowerCase() == name.toLowerCase())
                                    symbolName = name;
                            });
							
							var xXalue = commonEventArgs.data.location.x - (margin.left) / 2 + (margin.right) / 2;
							var yValue = commonEventArgs.data.location.y - (margin.top) / 2 - (height/margin.top) + (margin.bottom) / 2;
                            //To perform rotation for datalabels
							var degree = marker.dataLabel.angle;
							var angle = (degree > 360) ? degree - 360 : (degree < -360) ? degree + 360 : degree;
							var rotate = 'rotate(' + angle + ',' + (xXalue) + ',' + (yValue) + ')';
							$(options).attr('transform', rotate);
							$(options).attr('labelRotation', degree);//for canvas label rotation
							$(options).attr({
							    'transform': rotate

							});
                            this.dataLabelSymbol(seriesIndex, series, index, xXalue, yValue, width, height, symbolName, this.chartObj);
                            this.chartObj.svgRenderer.drawText(options, commonEventArgs.data.text, this.chartObj.gSeriesTextEle);
						}

                        else {
                            location.X -= this.chartObj.canvasX;
                            location.Y -= this.chartObj.canvasY;
                            this.drawLabelTemplate(series, point, index, location, this.chartObj);
                        } 
                            
                }
    },

    //smart labels for pie and doughnut

    updateSmartLabelPosition: function (currentseries, point, pointIndex, bounds, midAngle, startPoint, size, sender) {
        var renderingPoints = [];
        var chartObj = sender;
        var seriesType = new ej.seriesTypes[currentseries.type.toLowerCase()]();
        var connectorDirection = "";

        var connectorX = bounds.connector.connectorX;
        var connectorY = bounds.connector.connectorY;
        var midX = bounds.midPoint.midX;
        var midY = bounds.midPoint.midY;
        var dMidX = bounds.doughnutMidPoint.dMidX;
        var dMidY = bounds.doughnutMidPoint.dMidY;
        var startX = startPoint.startX;
        var startY = startPoint.startY;
        var textGap = (currentseries.marker.dataLabel.template) ? 0 : 4;

        var outerX;
        var textOffset = currentseries.marker.dataLabel.connectorLine.height;
        var radius = chartObj.model.Radius;

        var isIntersectedLabel;
        var angle = midAngle;
         
        //calculation for inside labels
        if (currentseries.labelPosition.toLowerCase() == 'inside') {
            positionX = (midX + startX) / 2;
            positionY = (midY + startY) / 2;

            positionX = (midX + positionX) / 2;
            positionY = (midY + positionY) / 2;

            if (point.isIntersected && chartObj.model.bounds[pointIndex]) {
                currBounds = chartObj.model.bounds[pointIndex];
            }
            else {
                if (midX > startX)
                    currBounds = { X: positionX, Y: positionY, Left: positionX, Top: positionY, Right: positionX + size.width, Bottom: positionY + size.height };
                else
                    currBounds = { X: positionX, Y: positionY, Left: positionX - size.width, Top: positionY, Right: positionX, Bottom: positionY + size.height };
             }

            do {
                isIntersectedLabel = false;
                if (!this.IntersectWith(chartObj, point, currBounds)) continue;
                angle += 0.01;
                point.isIntersected = true;
                currBounds.Angle = angle;
                if (midX > startX) {
                    currBounds.X = currBounds.Left = seriesType.getXCordinate(startX, (radius) + textOffset, angle);
                    currBounds.Right = currBounds.X + size.width;
                }
                else {
                    currBounds.X = currBounds.Right = seriesType.getXCordinate(startX, (radius) + textOffset, angle);
                    currBounds.Left = currBounds.Right - size.width;
                }
                currBounds.Y = currBounds.Top = seriesType.getYCordinate(startY, (radius) + textOffset, angle);
                currBounds.Bottom = currBounds.Y + size.height;
                isIntersectedLabel = true;
            } while (isIntersectedLabel)
            chartObj.model.bounds[pointIndex] = currBounds;

            if (point.isIntersected && chartObj.model.bounds[pointIndex].Angle) {

                positionX = seriesType.getXCordinate(startX, (radius) + textOffset, chartObj.model.bounds[pointIndex].Angle);
                positionY = seriesType.getYCordinate(startY, (radius) + textOffset, chartObj.model.bounds[pointIndex].Angle);

                var connectorMidX = seriesType.getXCordinate(startX, (radius) + textOffset / 3, midAngle);
                var connectorMidY = seriesType.getYCordinate(startY, (radius) + textOffset / 3, midAngle);
                connectorDirection = "M" + " " + midX + " " + midY + " " + "L" + " " + connectorMidX + " " + connectorMidY + " " + "M" + " " + connectorMidX + " " + connectorMidY + " " + "L" + " " + positionX + " " + positionY;

            }
        }

       //calculation for outside labels
        else if (currentseries.labelPosition.toLowerCase() == 'outside') {
           
            if (midX > startX) 
                currBounds = { X: connectorX, Y: connectorY, Left: connectorX, Top: connectorY, Right: connectorX + size.width, Bottom: connectorY + size.height };
           else
               currBounds = { X: connectorX , Y: connectorY, Left: connectorX - size.width, Top: connectorY, Right: connectorX, Bottom: connectorY + size.height };
            
            do {
                isIntersectedLabel = false;
                if (!this.IntersectWith(chartObj, point, currBounds)) continue;
                angle += 0.01;
                point.isIntersected = true;
                if (midX > startX) {
                    currBounds.X = currBounds.Left = seriesType.getXCordinate(startX, (radius) + textOffset, angle);
                    currBounds.Right = currBounds.X + size.width;
                }
                else {
                    currBounds.X = currBounds.Right = seriesType.getXCordinate(startX, (radius) + textOffset, angle);
                   currBounds.Left = currBounds.Right - size.width;
                }

                currBounds.Y = currBounds.Top = seriesType.getYCordinate(startY, (radius) + textOffset, angle);              
                currBounds.Bottom = currBounds.Y + size.height;
                isIntersectedLabel = true;
            } while (isIntersectedLabel)

            chartObj.model.bounds[pointIndex] = currBounds;

            var connectorMidX = seriesType.getXCordinate(startX, (radius) + textOffset / 3, midAngle);
            var connectorMidY = seriesType.getYCordinate(startY, (radius) + textOffset / 3, midAngle);

            renderingPoints.push({ X: midX, Y: midY });
            renderingPoints.push({ X: connectorMidX, Y: connectorMidY });
            renderingPoints.push({ X: currBounds.X, Y: currBounds.Y });
            positionX = currBounds.X;
            positionY = currBounds.Y;
            if (textOffset > 0) {
                if (midX < startX) {
                    outerX = currBounds.X - chartObj.model.elementSpacing;
                    positionX = outerX - textGap;
                }
                else {
                    outerX = currBounds.X + chartObj.model.elementSpacing;
                    positionX = outerX + textGap;
                }
                renderingPoints.push({ X: outerX, Y: currBounds.Y });
            }
            if (currentseries.marker.dataLabel.connectorLine.type.toLowerCase() == 'bezier' && !chartObj.vmlRendering) {
                seriesType.drawBezierSegment(renderingPoints, currentseries, pointIndex, chartObj);
            }
            else {
                connectorDirection = "M" + " " + midX + " " + midY + " " + "L" + " " + connectorMidX + " " + connectorMidY + " "+ "M" + " " + connectorMidX + " " + connectorMidY + " " + "L" + " " + currBounds.X + " " + currBounds.Y + " " + "M" + " " + currBounds.X + " " + currBounds.Y +  " " + "L" + " " + outerX + " " + currBounds.Y;
              }
                      
        }


         //calculation for outsideExtended labels
        else {
            var labelX = midX;
            var labelY = midY;

            midX = midX + (Math.cos((midAngle)) * (-(radius / 10)));
            midY = midY + (Math.sin((midAngle)) * (-(radius / 10)));

            var connectorHeight = radius / 4;

            renderingPoints.push({ X: midX, Y: midY });
            midX = midX + (Math.cos((midAngle)) * connectorHeight);
            midY = midY + (Math.sin((midAngle)) * connectorHeight);

            renderingPoints.push({ X: midX, Y: midY });

            var legendWidth = (chartObj.model.legend.visible && chartObj.model.legend.position.toLowerCase() == "left") ? (chartObj.model.margin.left + chartObj.model.LegendViewerBounds.Width) : 0;
            var centerX = chartObj.model.centerX;
            var labelEdge;

            if (midX < startX) {

                midX = ((centerX - radius - (connectorHeight * 2) - currentseries.LeftLabelMaxWidth > 0)
                                 ? centerX - radius - (connectorHeight * 2) + legendWidth
                                 : (currentseries.LeftLabelMaxWidth > labelX) ? labelX : currentseries.LeftLabelMaxWidth + legendWidth) - size.width / 2;

                labelEdge = renderingPoints[1].X - currentseries.LeftLabelMaxWidth;

                var height = (centerX - radius - (connectorHeight * 2) - currentseries.LeftLabelMaxWidth > 0) ? connectorHeight
                             : (currentseries.LeftLabelMaxWidth > labelX) ? 0
                             : (labelEdge > 3 * (connectorHeight / 2)) ? connectorHeight : labelEdge / 2;

                connectorLineEdge = +size.width / 2;

            }
            else {
                var legendRight = (chartObj.model.legend.visible && chartObj.model.legend.position.toLowerCase() == "right") ? (chartObj.model.margin.right + chartObj.model.LegendBounds.Width) : 0;
                midX = ((centerX + radius + (connectorHeight * 2) + currentseries.RightLabelMaxWidth < $(chartObj.svgObject).width() - legendRight)
               ? centerX + radius + (connectorHeight * 2) - legendWidth
               : ((centerX + radius + currentseries.RightLabelMaxWidth) > $(chartObj.svgObject).width()) ? labelX : $(chartObj.svgObject).width() - currentseries.RightLabelMaxWidth - legendRight) + size.width / 2;


                labelEdge = ($(chartObj.svgObject).width() - currentseries.RightLabelMaxWidth) - renderingPoints[1].X;

                var height = (centerX + radius + (connectorHeight * 2) + currentseries.RightLabelMaxWidth < $(chartObj.svgObject).width()) ? connectorHeight
                            : ((centerX + radius + currentseries.RightLabelMaxWidth) > $(chartObj.svgObject).width()) ? 0
                            : (labelEdge > 3 * (connectorHeight / 2)) ? connectorHeight : labelEdge / 2;
                connectorLineEdge = -size.width / 2;
            }

            var distanceFromOrigin = (Math.sqrt(Math.pow(labelX - midX, 2) + Math.pow(labelY - midY, 2))) / 10;

            var isLeft = (midX < startX) ? true : false;
            midX = isLeft ? midX + distanceFromOrigin : midX - distanceFromOrigin;
            var index = isLeft ? 1 : -1;

            var bounds = isLeft ? chartObj.model.bounds["left"] : chartObj.model.bounds["right"];
            var currBounds = { X: midX, Y: midY, Left: midX, Top: midY, Right: midX + size.width, Bottom: midY + size.height, index: pointIndex };
            if (this.IntersectWith(chartObj, point, currBounds)) {
                renderingPoints.push(midX < startX
                       ? { X: midX + height + connectorLineEdge, Y: midY }
                       : { X: midX - height + connectorLineEdge, Y: midY });
                midY = bounds[point.index + index].Bottom + 2;

                currBounds.Y = midY;
                currBounds.Top = midY;
                currBounds.Bottom = midY + size.height;

            }

            renderingPoints.push({ X: midX + connectorLineEdge, Y: midY });

            if (!isLeft) {
                chartObj.model.bounds["right"][point.index] = (currBounds);
                if (renderingPoints[renderingPoints.length - 1].X < renderingPoints[renderingPoints.length - 2].X)
                    renderingPoints[renderingPoints.length - 2].X = renderingPoints[renderingPoints.length - 1].X;
               
            }
            else {
                chartObj.model.bounds["left"][point.index] = (currBounds);
                if (renderingPoints[renderingPoints.length - 1].X > renderingPoints[renderingPoints.length - 2].X)
                    renderingPoints[renderingPoints.length - 2].X = renderingPoints[renderingPoints.length - 1].X;
            }


            connectorDirection = ej.EjSvgRender.utils._getStringBuilder();
            if (currentseries.marker.dataLabel.connectorLine.type.toLowerCase() == 'bezier' && !chartObj.vmlRendering) {
                seriesType.drawBezierSegment(renderingPoints, currentseries, pointIndex, chartObj);
            }
            else {
                for (k = 0; k < renderingPoints.length; k++) {
                    if (k == renderingPoints.length - 1)
                        connectorDirection.append(" ");
                    else
                        connectorDirection.append("M" + " " + (renderingPoints[k].X) + " " + (renderingPoints[k].Y) + " " + "L" + " " + (renderingPoints[k + 1].X) + " " + (renderingPoints[k + 1].Y) + " ");
                }
                connectorDirection = connectorDirection.toString();
            }
            positionX = renderingPoints[renderingPoints.length - 1].X + ((midX < startX) ? -textGap : textGap);
            positionY = renderingPoints[renderingPoints.length - 1].Y;
        }
        return { positionX: positionX, positionY: positionY, connectorDirection: connectorDirection };
    },

    updateLabelPosition: function (currentseries, point, pointIndex, bounds, midAngle, startPoint, size, sender) {

        var renderingPoints=[];
        var chartObj=sender; 
        var seriesType = new ej.seriesTypes[currentseries.type]();
        var connectorDirection = "";

        var connectorX = bounds.connector.connectorX;
        var connectorY = bounds.connector.connectorY;
        var midX = bounds.midPoint.midX;
        var midY = bounds.midPoint.midY;
        var dMidX = bounds.doughnutMidPoint.dMidX;
        var dMidY = bounds.doughnutMidPoint.dMidY;
        var startX = startPoint.startX;
        var startY = startPoint.startY;
        var textGap = (currentseries.marker.dataLabel.template) ? 0 :4;
       
        var outerX;
        var textOffset=currentseries.marker.dataLabel.connectorLine.height;
        var radius = chartObj.model.Radius;

        //calculation for outside labels

        renderingPoints.push({ X: midX, Y: midY });
        renderingPoints.push({ X: connectorX, Y: connectorY });
        positionX = connectorX;
        positionY = connectorY;

        if (currentseries.labelPosition.toLowerCase() == 'outside') {
            if (textOffset > 0) {
                if (connectorX < startX) {
                    outerX = connectorX - chartObj.model.elementSpacing;
                    positionX = outerX - textGap;
                }
                else {
                    outerX = connectorX + chartObj.model.elementSpacing;
                    positionX = outerX + textGap;
                }
                renderingPoints.push({ X: outerX, Y: connectorY });
            } 
           
            if (currentseries.marker.dataLabel.connectorLine.type.toLowerCase() == 'bezier' && !chartObj.vmlRendering) {
                seriesType.drawBezierSegment(renderingPoints, currentseries, pointIndex, chartObj);
            }
            else {
                connectorDirection = "M" + " " + midX + " " + midY + " " + "L" + " " + connectorX + " " + connectorY + " " + "M" + " " + connectorX + " " + connectorY + "L" + " " + outerX + " " + connectorY;
            }
          
        }


            //calculation for outsideextended labels
        else if (currentseries.labelPosition.toLowerCase() == 'outsideextended') {

            labelX=midX;
            labelY=midY;

            midX = midX + (Math.cos((midAngle)) * (-(radius / 10)));
            midY = midY + (Math.sin((midAngle)) * (-(radius / 10)));


            var renderingPoints = [];

            var connectorHeight = radius / 4;

            renderingPoints.push({ X: midX, Y: midY });
            connectorX = midX + (Math.cos((midAngle)) * connectorHeight);
            connectorY = midY + (Math.sin((midAngle)) * connectorHeight);

            renderingPoints.push({ X: connectorX, Y: connectorY });

            var legendWidth = (chartObj.model.legend.visible && (chartObj.model.legend.position.toLowerCase() == "left" || chartObj.model.legend.position.toLowerCase() == "right")) ? (chartObj.model.elementSpacing) : 0;

            if (midX < startX) {
                connectorX = (chartObj.model.centerX - radius - (connectorHeight * 2) - currentseries.LeftLabelMaxWidth > 0)
                                     ? chartObj.model.centerX - radius - (connectorHeight * 2) + legendWidth
                                     : (currentseries.LeftLabelMaxWidth > labelX) ? labelX : currentseries.LeftLabelMaxWidth;
            }
            else {
                connectorX = (chartObj.model.centerX + radius + (connectorHeight * 2) + currentseries.RightLabelMaxWidth < $(chartObj.svgObject).width())
                     ? chartObj.model.centerX + radius + (connectorHeight * 2) - legendWidth
                     : ((chartObj.model.centerX + radius + currentseries.RightLabelMaxWidth) > $(chartObj.svgObject).width()) ? labelX : $(chartObj.svgObject).width() - currentseries.RightLabelMaxWidth;
                      
            }

            renderingPoints.push({ X: connectorX, Y: connectorY });

            connectorDirection = ej.EjSvgRender.utils._getStringBuilder();
            if (currentseries.marker.dataLabel.connectorLine.type.toLowerCase() == 'bezier' && !chartObj.vmlRendering) {
                seriesType.drawBezierSegment(renderingPoints, currentseries, pointIndex, chartObj);
            }
            else {
                for (k = 0; k < renderingPoints.length; k++) {
                    if (k == renderingPoints.length - 1)
                        connectorDirection.append(" ");
                    else
                        connectorDirection.append("M" + " " + (renderingPoints[k].X) + " " + (renderingPoints[k].Y) + " " + "L" + " " + (renderingPoints[k + 1].X) + " " + (renderingPoints[k + 1].Y) + " ");
                }
                connectorDirection = connectorDirection.toString();
            }
            positionX = renderingPoints[renderingPoints.length - 1].X + ((midX < startX) ? -textGap : textGap);;
            positionY = renderingPoints[renderingPoints.length - 1].Y;
        }

       //calculation for inside labels

        else {
            if (currentseries.type.toLowerCase() == "doughnut") {
                positionX = (midX + dMidX) / 2;
                positionY = (midY + dMidY) / 2;
            }
            else {
                positionX = (midX + startX) / 2;
                positionY = (midY + startY) / 2;

                positionX = (midX + positionX) / 2;
                positionY = (midY + positionY) / 2;
            }
        }

        return { positionX: positionX, positionY: positionY, connectorDirection: connectorDirection };
    },

	drawDataLabelAcc: function (sender, currentseries, pointIndex, point) {
	    // method for data label symbols - accumulation series
	    if (point.visible && currentseries.marker.dataLabel.visible) {
	        var chartObj = sender;
	        var seriesType = new ej.seriesTypes[currentseries.type.toLowerCase()]();
	        var outerX;
	        var connectorDirection;
	        var positionX = 0;
	        var positionY = 0;
	        var pointColor;
	        var numberToFixed = ej.util.isNullOrUndefined(chartObj.model.roundingPlaces) ? 2 : chartObj.model.roundingPlaces;

	        var _labelPosition = currentseries.labelPosition.toLowerCase();


	        var text = (Math.round(point.YValues[0]) == point.YValues[0]) ? point.YValues[0] : point.YValues[0].toFixed(numberToFixed);
	        var labelfont = { size: currentseries.marker.dataLabel.font.size, fontStyle: currentseries.marker.dataLabel.font.fontStyle, fontFamily: currentseries.marker.dataLabel.font.fontFamily };
	        var textsize = ej.EjSvgRender.utils._measureText((ej.util.isNullOrUndefined(point.text) ? text : point.text), null, labelfont);

	        var margin = currentseries.marker.dataLabel.margin;

	        var textWidth = textsize.width + margin.left + margin.right;
	        var textHeight = textsize.height + margin.top + margin.bottom;

	        if (currentseries.type.toLowerCase() == "pyramid" || currentseries.type.toLowerCase() == "funnel") {
	            var measureTitle = (chartObj.model.title.text) ? ej.EjSvgRender.utils._measureText(chartObj.model.title.text, $(this.svgObject).width() - chartObj.model.margin.left - chartObj.model.margin.right, chartObj.model.title.font) : 0;
	            var pyrX = chartObj.model.elementSpacing + chartObj.model.margin.left +
                           (chartObj.model.legend.position.toLowerCase() === "left") ?
                           (chartObj.model.LegendViewerBounds.Width) : 0;
	            var pyrY = ((chartObj.model.legend.position.toLowerCase() === "top")
                            ? (chartObj.model.LegendViewerBounds.Height) : 0) +
                            ((chartObj.model.title.text) ? (chartObj.model._titleLocation.Y + measureTitle.height) : (chartObj.model.margin.top + chartObj.model.elementSpacing));
	            textWidth = textWidth + 10;
	            var Location = { X: point.xLocation + pyrX, Y: point.yLocation + pyrY };
	            positionX = point.xLocation;
	            positionY = point.yLocation;
	            var marker = point.marker ? point.marker : currentseries.marker;
	            marker = $.extend(true, {}, currentseries.marker, marker);
	            connectorDirection = point.connectorLine;

	        }

	        if (currentseries.type.toLowerCase() != "pyramid" && currentseries.type.toLowerCase() != "funnel") {
	            var textOffset;
	            var radius = chartObj.model.Radius;

	            if (ej.util.isNullOrUndefined(currentseries.marker.dataLabel.connectorLine.height)) {
	                text = ej.util.isNullOrUndefined(point.text) ? point.y : point.text;
	                currentseries.marker.dataLabel.connectorLine.height = ej.EjSvgRender.utils._measureText(text, null, currentseries.marker.dataLabel.font).height;
	            }

	            textOffset = currentseries.marker.dataLabel.connectorLine.height;

	            var chartStartingAngle = -.5 * Math.PI; // to move the arc to 90 degree from the start.
	            var renderingPoints = [];

	            var startX, startY;

	            var midAngle = point.midAngle + chartStartingAngle;
	            if ((pointIndex == currentseries.explodeIndex || currentseries.explodeAll) && !chartObj.vmlRendering) {

	                startX = chartObj.model.centerX + Math.cos(midAngle) * currentseries.explodeOffset;
	                startY = chartObj.model.centerY + Math.sin(midAngle) * currentseries.explodeOffset;
	            }
	            else {
	                startX = chartObj.model.centerX;
	                startY = chartObj.model.centerY;
	            }
	            var midX = seriesType.getXCordinate(startX, (radius), midAngle);
	            var midY = seriesType.getYCordinate(startY, (radius), midAngle);

	            var dMidX = seriesType.getXCordinate(startX, (chartObj.model.dRadius), midAngle);
	            var dMidY = seriesType.getYCordinate(startY, (chartObj.model.dRadius), midAngle);

	            var connectorX = this.getXCordinate(startX, (radius) + textOffset, midAngle);
	            var connectorY = this.getYCordinate(startY, (radius) + textOffset, midAngle);

	            if (currentseries.marker.dataLabel.template) {
	                size = point.size;
	            }
	            else {
	                var marker = point.marker ? point.marker : currentseries.marker;
	                marker = $.extend(true, {}, currentseries.marker, marker);
	                var shape = marker.dataLabel.shape;
	                size = ej.EjSvgRender.utils._measureText(currentseries.visibleLabels[pointIndex], $(chartObj.svgObject).width(), currentseries.marker.dataLabel.font);
	                if (shape.toLowerCase() != "none") {
	                    var margin = currentseries.marker.dataLabel.margin;
	                    size.height = size.height + margin.top + margin.bottom;
	                    size.width = size.width + margin.left + margin.right;
	                }
	            }

	            var midPoint = { midX: midX, midY: midY };
	            var doughnutMidPoint = { dMidX: dMidX, dMidY: dMidY };
	            var connector = { connectorX: connectorX, connectorY: connectorY };
	            var bounds = { midPoint: midPoint, doughnutMidPoint: doughnutMidPoint, connector: connector };
	            var startPoint = { startX: startX, startY: startY };

	            if (!currentseries.enableSmartLabels || textWidth > chartObj.model.m_AreaBounds.Width) {
	                position = seriesType.updateLabelPosition(currentseries, point, pointIndex, bounds, midAngle, startPoint, size, sender);
	                positionX = position.positionX;
	                positionY = position.positionY;
	                connectorDirection = position.connectorDirection;
	            }
	                // smart labels

	            else {

	                position = seriesType.updateSmartLabelPosition(currentseries, point, pointIndex, bounds, midAngle, startPoint, size, sender);
	                positionX = position.positionX;
	                positionY = position.positionY;
	                connectorDirection = position.connectorDirection;
	            }



	        }
	        var textAnchor = ((_labelPosition != 'inside' && _labelPosition != 'insidenooverlap') || ((currentseries.type.toLowerCase() == "pyramid" || currentseries.type.toLowerCase() == "funnel") && _labelPosition == 'outside')) ? ((positionX < chartObj.model.centerX) ? 'end' : 'start') : 'middle';
        var width = (_labelPosition == 'inside' || _labelPosition == 'insidenooverlap') ? 0 : (positionX < chartObj.model.centerX) ? -textsize.width : textsize.width;
	        var pyrX = 0; var pyrY = 0;
			if ((this.chartObj && this.chartObj.model.enableCanvasRendering) && (currentseries.type.toLowerCase() == "pyramid" || currentseries.type.toLowerCase() == "funnel")) {
	            pyrX = ((this.chartObj.model.legend.position.toLowerCase() === "left") ? (this.chartObj.model.LegendBounds.Width) : 0) + this.chartObj.model.elementSpacing + this.chartObj.model.margin.left;
	            pyrY = ((this.chartObj.model.legend.position.toLowerCase() === "top") ? (this.chartObj.model.LegendBounds.Height) : 0) + ((this.chartObj.model.title.text) ? (this.chartObj.model._titleLocation.Y + measureTitle.height) : (this.chartObj.model.margin.top + this.chartObj.model.elementSpacing));
	        }
        if (!currentseries.marker.dataLabel.template) {
		       $("#" + chartObj.svgObject.id + '_PointText' + pointIndex).remove();
            var textOptions = {
                'id': chartObj.svgObject.id + '_PointText'  + pointIndex,
                'x': positionX + pyrX,
                'y': positionY + textsize.height / 4 + pyrY,
                'fill': currentseries.marker.dataLabel.font.color,
	                'font-size': currentseries.marker.dataLabel.font.size,
	                'font-family': currentseries.marker.dataLabel.font.fontFamily,
	                'font-style': currentseries.marker.dataLabel.font.fontStyle,
	                'font-weight': currentseries.marker.dataLabel.font.fontWeight,
	                'opacity': currentseries.marker.dataLabel.font.opacity,
	                'text-anchor': textAnchor
	            };
	            chartObj.svgRenderer.drawText(textOptions, ((ej.util.isNullOrUndefined(point.text) ? text : point.text)), chartObj.gSeriesTextEle);
	        }

	        if (currentseries.marker.dataLabel.template) {
	            Location = ej.util.isNullOrUndefined(Location) ? { X: positionX, Y: positionY } : Location;
	            var pointindex = point.pointIndex;
	            seriesType.drawLabelTemplate(currentseries, point, pointIndex, Location, sender);
	        }

	        else {
	            $.each(chartObj.model.symbolShape, function (name) {
	                var marker = point.marker ? point.marker : currentseries.marker;
	                marker = $.extend(true, {}, currentseries.marker, marker);
	                var shape = marker.dataLabel.shape;
	                if (shape.toLowerCase() == name.toLowerCase())
	                    symbolName = name;
	            });

	            positionX = pyrX + positionX - (margin.left) / 2 + (margin.right) / 2 + width / 2;
	            positionY = pyrY + positionY - (margin.top) / 2 + (margin.bottom) / 2;

	            seriesType.dataLabelSymbol(0, currentseries, pointIndex, positionX, positionY, textWidth, textHeight, symbolName, chartObj);
	            chartObj.svgRenderer.drawText(textOptions, ((ej.util.isNullOrUndefined(point.text) ? text : point.text)), chartObj.gSeriesTextEle);

	        }
	        pointColor = jQuery.type(chartObj.model.pointColors[pointIndex]) == "array" ? chartObj.model.pointColors[pointIndex][0].color : chartObj.model.pointColors[pointIndex];
	        var translate = [];
	        translate[0] = pyrX;
	        translate[1] = pyrY;
	        if ((currentseries.marker.dataLabel.connectorLine.type.toLowerCase() != 'bezier' || chartObj.vmlRendering || (currentseries.type.toLowerCase() == "pyramid" || currentseries.type.toLowerCase() == "funnel")) && (connectorDirection != '')) {
	            if (connectorDirection){
	                var connectorOptions = {
	                    'id': chartObj.svgObject.id +  "_connectorLine" + pointIndex,
	                    'stroke': (currentseries.marker.dataLabel.connectorLine.color) ? currentseries.marker.dataLabel.connectorLine.color : pointColor,
	                    'stroke-width': currentseries.marker.dataLabel.connectorLine.width,
	                    'd': connectorDirection
	                };
	            chartObj.svgRenderer.drawPath(connectorOptions, chartObj.gSeriesConnectorEle, translate);
	            }
	        }
	    }
        
	},
	drawBezierSegment: function (controlpoints, currentseries, pointIndex, sender) {
	    var polyLine = ej.EjSvgRender.utils._getStringBuilder();
	    var seriesType = new ej.seriesTypes[currentseries.type.toLowerCase()]();
	    for (var i = 0; i <= 16; i++) {
	        var t = i / 16;
	        var points = seriesType.GetBezierPoint(t, controlpoints, 0, controlpoints.length, currentseries);
	        polyLine.append(points.X + "," + points.Y + " ");
	    }
	    var connectorOptions = {
	        'id': sender.svgObject.id + "_bezierLine" + pointIndex,
	        'stroke': (currentseries.marker.dataLabel.connectorLine.color) ? currentseries.marker.dataLabel.connectorLine.color : sender.model.pointColors[pointIndex],
	        'stroke-width': currentseries.marker.dataLabel.connectorLine.width,
            'fill':'none',
	        'points': polyLine.toString()
	    };
	    sender.svgRenderer.drawPolyline(connectorOptions, sender.gSeriesConnectorEle);
	},
	GetBezierPoint:function(t, controlPoints, index, count, currentseries)
	{
	    var seriesType = new ej.seriesTypes[currentseries.type.toLowerCase()]();
	    if (count == 1)
	        return controlPoints[index];
	    var p0 = seriesType.GetBezierPoint(t, controlPoints, index, count - 1, currentseries);
	    var p1 = seriesType.GetBezierPoint(t, controlPoints, index + 1, count - 1, currentseries);
	    var x = (p0.X) ? p0.X : p0.x;
	    var y = (p0.Y) ? p0.Y : p0.y;
	    var x1 = (p1.X) ? p1.X : p1.x;
	    var y1 = (p1.Y) ? p1.Y : p1.y;
	    var X = (1 - t) * x+ t * x1;
	    var Y = (1 - t) * y + t * y1;
	    if (p0.x)
	        return { x: X, y: Y }
	    else
	        return { X: X, Y: Y };;
	    


       // return new Point((1 - t) * p0.X + t * p1.X, (1 - t) * p0.Y + t * p1.Y);
    },
	
    drawLabelTemplate: function (series, point, pointIndex, location, sender) {
        // method for data label template
        if(sender)
        this.chartObj = sender;
	    var seriesIndex = $.inArray(series, this.chartObj.model._visibleSeries);
        var areaType = this.chartObj.model.AreaType;
        var currentSeries = this.chartObj.model.series[seriesIndex];
		var xPosition = 0, yPosition = 0;
        var chartContainer = this.chartObj._id;
		var marker = point.marker? point.marker : series.marker;
	    marker = $.extend(true, {}, series.marker, marker);
        var style = { 'interior': series.marker.dataLabel.fill, 'opacity': series.marker.dataLabel.opacity, 'borderColor': series.marker.dataLabel.border.color, 'borderWidth': series.marker.dataLabel.border.width };
        var color, width, height;
		var type = this.chartObj.model.chartRegions[pointIndex] ? this.chartObj.model.chartRegions[pointIndex].type : "";
        if (areaType !="none") {

            if ($('#template_group_' + chartContainer).length != 0)
                var templateContainer = $('#template_group_' + chartContainer);
            else
                templateContainer = $("<div></div>").attr('id', "template_group_" + chartContainer);

            templateContainer.css('position', 'relative').css('z-index', 1000);
			if ($("#" + series.marker.dataLabel.template).length == 0) // To check the specified div is in DOM
                return;
            else
            var cloneNode = $("#" + series.marker.dataLabel.template).clone();
            $(cloneNode).attr("id", series.marker.dataLabel.template + '_' + seriesIndex + '_' + pointIndex + '_' + chartContainer);
            var $cloneNode = $(cloneNode);
            $cloneNode.css("position", "absolute");

            if (series.xAxis._valueType.toLowerCase() == "datetime")
                point.x = (Globalize.format(new Date(point.X), ((ej.util.isNullOrUndefined(series.xAxis.labelFormat)) ? ej.EjSvgRender.utils._dateTimeLabelFormat(series.xAxis.intervalType) : series.xAxis.labelFormat)));
            if (series.xAxis._valueType.toLowerCase() == "category")
                point.x = ej.EjSvgRender.utils._getLabelContent(pointIndex, series.xAxis);

            point.count = 1;
            var data = { series: series, point: point };
            $cloneNode.html($cloneNode.html().parseTemplate(data));

            var display = (areaType == "cartesianaxes" || !series.enableAnimation || (series.type.toLowerCase() == "pyramid" || series.type.toLowerCase() == "funnel")) ? "block" : "none";
            $cloneNode.css("display", display).appendTo($(templateContainer));
            $(templateContainer).appendTo('#' + chartContainer);
            var areaBound = this.chartObj.model.m_AreaBounds;
            if (areaType == "cartesianaxes" && !this.chartObj.model.enable3D) {
            xPosition = areaBound.X;
            yPosition = currentSeries.yAxis.y;
            }
            width = $cloneNode.width() / 2;
            height = $cloneNode.height();
            if (ej.util.isNullOrUndefined(style) && style.interior)
                color = style.interior;
            else
                color= this.chartObj.model.seriesColors[seriesIndex];
        }
        else {
            if (ej.util.isNullOrUndefined(style) && style.interior)
                color = style.interior;
            else
                color = this.chartObj.model.pointColors[point.index];
           width = this.chartObj.model.centerX > location.X ? point.size.width : 0;
            $cloneNode = $('#template_group_' + chartContainer).find("#" + series.marker.dataLabel.template + '_' + seriesIndex + '_' + pointIndex + '_' + chartContainer);
            height = point.size.height / 2;
        }
        
        if (currentSeries.type == 'hilo' || currentSeries.type == 'hiloopenclose' || currentSeries.type == 'candle')
            color = currentSeries.points[pointIndex].interior;
            color = jQuery.type(color) == "array" ? color[0].color : color;
			
       if (type == 'bar' || type == 'stackingbar') {
            var left = location.X + xPosition - width;
            var top = location.Y + areaBound.Y -height;
        } else {
        var left =location.X + xPosition - width;
        var top = yPosition + location.Y - height;
		}
		
       $cloneNode.css("left", left).css("top", top).css("background-color", color).css("display", "block");
        if (areaType == "cartesianaxes") {
            if ((left > xPosition + areaBound.Width || top > yPosition + areaBound.Height) || (this.chartObj.zoomed && (left < xPosition || top < yPosition || left > xPosition + areaBound.Width || top > yPosition + areaBound.Height)))
                $cloneNode.css("display", "none");
        }
    },
	
	animateLabelTemplate : function(options) {
	// method to animate data label template
        var seriesIndex = $.inArray(options, this.chartObj.model._visibleSeries);
        var length = options.points.length;
        var secondsPerPoint = 1000;
        
        for (var i = 0; i < length; i++) {            
            var ele = options.marker.dataLabel.template + '_' + seriesIndex + '_' + i + '_' + this.chartObj._id;
            ele = $('#' + ele);
            ele.css("display", "none");
            ele.delay(secondsPerPoint).fadeIn(300);
			}
    },
	
	textPosition: function (series, seriesIndex, textOffset, type, x, y,chartRegionIndex) {
        var horizontalTextAlignment = series.marker.dataLabel.horizontalTextAlignment;
        var verticalTextAlignment = series.marker.dataLabel.verticalTextAlignment;
        var visiblePointLength = series.points.length;
        var seriesPointIndex = chartRegionIndex;
	    var lineHeight = 10;
	    if (seriesIndex > 0)
	        seriesPointIndex = this.chartObj.model.chartRegions.length - visiblePointLength + chartRegionIndex;
       if (type == 'column' || type == 'stackingcolumn') {
           var pointHeight = this.chartObj.model.chartRegions[seriesPointIndex].Region.Bounds.Height;
            if (series.marker.dataLabel.textPosition.toLowerCase() == "bottom")
                y += pointHeight;
            else if (series.marker.dataLabel.textPosition.toLowerCase() == "middle")
                y += pointHeight / 2 + (textOffset.height) / 4;
            else
                y += (textOffset.height) / 4;

            if (horizontalTextAlignment && horizontalTextAlignment.toLowerCase() == "far")
                x += textOffset.width + lineHeight;
            else if (horizontalTextAlignment && horizontalTextAlignment.toLowerCase() == "near")
                x -= ((textOffset.width) + lineHeight);

            if (verticalTextAlignment && verticalTextAlignment.toLowerCase() == "near")
                y += textOffset.height / 3;
            else if (verticalTextAlignment && verticalTextAlignment.toLowerCase() == "far")
                y -= textOffset.height / 3;
        }

        else if (type == 'bar' || type == 'stackingbar') {
            if (seriesIndex > 0)
                seriesPointIndex = this.chartObj.model.chartRegions.length - visiblePointLength + chartRegionIndex;
            var pointWidth = this.chartObj.model.chartRegions[seriesPointIndex].Region.Bounds.Width;
			y = y+ textOffset.height / 4;
            if (series.marker.dataLabel.textPosition.toLowerCase() == "bottom")
                x -= pointWidth;
            else if (series.marker.dataLabel.textPosition.toLowerCase() == "middle")
                x -= pointWidth / 2 + textOffset.width / 2;
            else
                x = x - textOffset.width / 4;

            if (horizontalTextAlignment && horizontalTextAlignment.toLowerCase() == "far")
                x += textOffset.width - textOffset.width/4 ;
            else if (horizontalTextAlignment && horizontalTextAlignment.toLowerCase() == "near")
                x -= textOffset.width / 4;
            else // for center
                x += textOffset.width / 4;
				
            if (verticalTextAlignment && verticalTextAlignment.toLowerCase() == "near")
                y += textOffset.height;
            else if (verticalTextAlignment && verticalTextAlignment.toLowerCase() == "far")
                y -= textOffset.height;
        }
        else if (type == "polar" || type == "radar") {
                y += textOffset.height / 4;
            if (series.marker.dataLabel.textPosition.toLowerCase() == "bottom")
                y += (textOffset.height);
            else if (series.marker.dataLabel.textPosition.toLowerCase() == "top")
                y -=  (textOffset.height);

            if (horizontalTextAlignment && horizontalTextAlignment.toLowerCase() == "far")
                x += textOffset.width;
            else if (horizontalTextAlignment && horizontalTextAlignment.toLowerCase() == "near")
                x -= (textOffset.width);

            if (verticalTextAlignment && verticalTextAlignment.toLowerCase() == "near")
                y += (textOffset.height)/2;
            else if (verticalTextAlignment && verticalTextAlignment.toLowerCase() == "far")
                y -= ((textOffset.height)/2);
        }
        else {
            if (series.marker.dataLabel.textPosition.toLowerCase() == "bottom")
                y += (2 * textOffset.height + lineHeight);
            else if (series.marker.dataLabel.textPosition.toLowerCase() == "top")
                y -= (lineHeight + (2 * textOffset.height) / 2);

            if (horizontalTextAlignment && horizontalTextAlignment.toLowerCase() == "far")
                x += textOffset.width + lineHeight;
            else if (horizontalTextAlignment && horizontalTextAlignment.toLowerCase() == "near")
                x -= ((textOffset.width) + lineHeight);

            if (verticalTextAlignment && verticalTextAlignment.toLowerCase() == "near")
                y += (textOffset.height + lineHeight);
            else if (verticalTextAlignment && verticalTextAlignment.toLowerCase() == "far")
                y -= (lineHeight + (textOffset.height) / 2);
        }
        var location = { x: x, y: y };
        return location;
    },
	
    drawSymbolImage: function (series, pointIndex) {
                var point = series.points[pointIndex];
                if (point.visible) {

                if (point.marker) {
                    if (!(point.marker.visible || series.marker.visible) || point.marker.shape != this.chartObj.model.symbolShape.image)
                        return;
                }

                
                var element = (point.marker) ? point : series;
                element = $.extend(true, {}, series.marker, element);
                if (element.shape.toLowerCase() == "image") {
                    var img = new Image();
                    img.src = element.imageUrl;
                    element.image = img;
					}
			}
        
    },

    changeCrossHairSymbol: function(element, track, ptIndex, serIndex, series) {
        if (!ej.util.isNullOrUndefined(track)) {
            var trackMarker = {};
            element = $.extend(true, {}, series.marker, element);
            trackMarker = $.extend(true, {}, this.chartObj.model.crosshair.marker, trackMarker);

            if (ej.util.isNullOrUndefined(trackMarker.shape)) {
                trackMarker.shape = element.shape;
            }
            
            if ((!this.chartObj.model.crosshair.visible || ej.util.isNullOrUndefined(trackMarker.fill)) && series.type.toLowerCase() != "bubble" && series.type.toLowerCase() != "scatter") {
                if(trackMarker.shape.toLowerCase()=="cross" ||trackMarker.shape.toLowerCase()=="horizline"||trackMarker.shape.toLowerCase()=="vertline" )   
                    trackMarker.fill = jQuery.type(this.chartObj.model.seriesColors[serIndex]) == "array" ? this.chartObj.model.seriesColors[serIndex][0].color : this.chartObj.model.seriesColors[serIndex];         
                else
                    trackMarker.fill = element.border.color;
            }
            if (series.type.toLowerCase() == "bubble" && !this.chartObj.model.enableCanvasRendering) {
                var box = $(this.chartObj.gSeriesEle).find("#" + this.chartObj.svgObject.id + '_circlesymbol_' + ptIndex + "_" + serIndex)[0].getBoundingClientRect();
                trackMarker.size.width = (box.right - box.left) / 2 + 15;
                trackMarker.size.height = (box.bottom - box.top) / 2 + 15;
            } else {
                if (trackMarker.size.width <= element.size.width)
                    trackMarker.size.width = element.size.width + 2;
                if (trackMarker.size.height <= element.size.height)
                    trackMarker.size.height = element.size.height + 2;
               
                if (ej.util.isNullOrUndefined(trackMarker.border.color) || !this.chartObj.model.crosshair.visible){              
                 if (!ej.util.isNullOrUndefined(element.fill))
                    trackMarker.border.color = jQuery.type(element.fill) == "array" ? element.fill._gradientStop[0].color : element.fill;
                else
                  trackMarker.border.color = jQuery.type(this.chartObj.model.seriesColors[serIndex]) == "array" ? this.chartObj.model.seriesColors[serIndex][0].color : this.chartObj.model.seriesColors[serIndex];
                }
            }
        }
        series._trackMarker = trackMarker;
        return trackMarker;


    },
	
    dataLabelSymbol: function (seriesIndex, series, pointIndex, x, y, width, height, symbolName, sender) {
        var chartObj = sender;
        if (symbolName == 'None')
            return;
            var point = series.points[pointIndex];
            var location = { startX: x, startY: y };
            var symbolEle = chartObj.gSymbolGroupEle;
            var element = point.marker ? point.marker : series.marker;
            var gradientName = "symbol";
            var style = { 'interior': series.marker.dataLabel.fill, 'opacity': series.marker.dataLabel.opacity, 'borderColor': series.marker.dataLabel.border.color, 'borderWidth': series.marker.dataLabel.border.width };
            var options = {
                'style': style,
                'gradientName': gradientName,
                'symbolEle': symbolEle,
                'width': width,
                'height': height,
                'point': point,
                'location': location,
                'seriesIndex': seriesIndex,
                'pointIndex': pointIndex,
                'element': element,
                'symbolName': symbolName,
                'id': chartObj.svgObject.id + '_dataLabel_' + seriesIndex + "_" + pointIndex
            };
            this.drawSymbolStyle(options, chartObj);
    },

    drawSymbolStyle: function (options, sender) {
        var chartObj = sender;
        var borderColor, borderWidth, opacity;
        var style = options.style, gradientName = options.gradientName, symbolEle = options.symbolEle, width = options.width;
        var height = options.height, point = options.point, location = options.location, seriesIndex = options.seriesIndex, visibility=options.visibility;
        var pointIndex = options.pointIndex, element = options.element, symbolName = options.symbolName;
        var trackSymbol = options.trackSymbol;
        if (style != null) {
            if (style.borderColor != "")
                borderColor = style.borderColor ? style.borderColor: "transparent";

            opacity = style.opacity;
            if (style.borderWidth)
                borderWidth = style.borderWidth;
        }
        var colors = null;
        if (style.interior) {
            colors = ej.util.isNullOrUndefined(style.interior._gradientStop) ? style.interior : style.interior._gradientStop;
        }
        else {       
            if (point.fill)
                colors = ej.util.isNullOrUndefined(point.fill._gradientStop) ? point.fill : point.fill._gradientStop;
            else
                colors = (chartObj.model.AreaType!="none") ? chartObj.model.seriesColors[seriesIndex] : chartObj.model.pointColors[pointIndex];
        }
       chartObj.symbolColorName = chartObj.svgRenderer.createGradientElement(gradientName + seriesIndex, colors, 0, 0, 0, $(chartObj.svgObject).height(), symbolEle);

        var symbolStyle = { ShapeSize: {width:width, height:height}, Style: { BorderColor: borderColor, BorderWidth: borderWidth, Opacity: opacity, Visibility:visibility, Color: chartObj.symbolColorName}, PointIndex: pointIndex, SeriesIndex: seriesIndex, Imageurl: element.imageUrl, Image: element.image, ID:options.id };

        var commonEventArgs = $.extend({}, ej.EjSvgRender.commonChartEventArgs);
        commonEventArgs.data = { location: location, style: symbolStyle };
        chartObj._trigger("symbolRendering", commonEventArgs);
        if (!commonEventArgs.Cancel) {
            if (chartObj.model.enableCanvasRendering){
				var type=chartObj.model.series[seriesIndex].type.toLowerCase();
				//Added for Marker location while setting plotoffset value to axis
                location.startY = location.startY + (chartObj.model.series[seriesIndex].yAxis.y ? (chartObj.model.series[seriesIndex].yAxis.y - chartObj.canvasY) : 0);
                commonEventArgs.data.location.startY = location.startY;
                if (ej.util.isNullOrUndefined(trackSymbol)) { // to draw marker/datalabel
				    if (type != "pyramid" && type != "funnel") {
					    chartObj.svgRenderer.ctx.save();
                        chartObj.svgRenderer.ctx.rect(chartObj.model.m_AreaBounds.X, chartObj.model.m_AreaBounds.Y, chartObj.model.m_AreaBounds.Width, chartObj.model.m_AreaBounds.Height);
                        chartObj.svgRenderer.ctx.clip();
						ej.EjSvgRender.chartSymbol["_draw" + symbolName](commonEventArgs.data.location, commonEventArgs.data.style, chartObj, symbolEle);
                        chartObj.svgRenderer.ctx.restore();
				    }else  ej.EjSvgRender.chartSymbol["_draw" + symbolName](commonEventArgs.data.location, commonEventArgs.data.style, chartObj, symbolEle);
                    
                    if (chartObj.model.series[seriesIndex].type.toLowerCase() == "scatter") { // storing chartregion for scatter
                        var series = chartObj.model.series[seriesIndex];
                        width = symbolStyle.ShapeSize.width;
                        height = symbolStyle.ShapeSize.height;
                        series.points[pointIndex].cx = location.startX - width;
                        series.points[pointIndex].cy = location.startY - height;
                        series.points[pointIndex].width = width;
                        series.points[pointIndex].height = height;
                    }
                }
                else  // to draw track symbol
                chartObj.svgRenderer.trackSymbol(commonEventArgs.data.style, commonEventArgs.data.location, symbolName, trackSymbol, chartObj);
            }
            else
                ej.EjSvgRender.chartSymbol["_draw" + symbolName](commonEventArgs.data.location, commonEventArgs.data.style, chartObj, (symbolEle)?symbolEle : sender.chart3D);
        }

        if (!sender.chart3D && chartObj.model.AreaType == "none" && $(symbolEle).children().not("defs").length > chartObj.model._visibleSeries[seriesIndex].visiblePoints.length) {
            $(symbolEle.childNodes[options.pointIndex]).replaceWith($(symbolEle.childNodes[$(symbolEle.childNodes).length - 1]));
        }

    },
	
    drawSymbol: function (seriesIndex,series, pointIndex, x, y,chart,trackSymbol) {

        if (ej.util.isNullOrUndefined(this.chartObj)) this.chartObj = chart;


        var point = series.points[pointIndex];
       

             var symbolName = "None";
             var tracker = this.changeCrossHairSymbol(point.marker ? point.marker : series.marker, trackSymbol, pointIndex, seriesIndex, series);
             var element = tracker ? tracker : $.extend(true, {}, series.marker, point.marker);

             $.each(this.chartObj.model.symbolShape, function(name) {
                 if (element.shape.toLowerCase() == name.toLowerCase())
                     symbolName = name;
             });
                if (symbolName == "None")
                    return;
                var location = this.chartObj.model.AreaType == "cartesianaxes" ? { startX: x + this.chartObj.canvasX, startY: y + this.chartObj.canvasY } : { startX: x, startY: y };
                var style = {'interior' : element.fill, 'opacity': element.opacity, 'borderColor': element.border.color, 'borderWidth' :element.border.width };

                var symbolEle = (trackSymbol) ? this.chartObj.gTrackerEle : this.chartObj.gSymbolGroupEle;
                var id = (trackSymbol) ? (this.chartObj.svgObject.id + '_trackSymbol_' + seriesIndex + "_" + pointIndex) : (this.chartObj.svgObject.id + '_symbol_' + seriesIndex + "_" + pointIndex);
                var gradientName = trackSymbol ? "TrackSymbol" : "symbol";
                var width = element.size.width;
                var height = element.size.height;
                var options = {
                    'style': style,
                    'gradientName': gradientName,
                    'symbolEle': symbolEle,
                    'width': width,
                    'height': height,
                    'point': point,
                    'location': location,
                    'seriesIndex': seriesIndex,
                    'pointIndex': pointIndex,
                    'element': element,
                    'visibility': (point.visible) ? 'visible' : 'hidden',
                    'symbolName': symbolName,
                    'trackSymbol' : trackSymbol,
                    'id': id
                };
                this.drawSymbolStyle(options, this.chartObj);
        
    },

    chartAreaType: "cartesianAxes",
    requireInvertedAxes: false,
    stackingSeries: false,
    hiloTypes:false
    
};


function ejExtendClass(parent, members) {
    var object = function () { };
    object.prototype = new parent();
    $.extend(object.prototype, members);
    return object;
}

ej.ejLineSeries = ejExtendClass(ej.EjSeriesRender);

ej.seriesTypes.line = ej.ejLineSeries;


ej.ejStepLineSeries = ejExtendClass(ej.EjSeriesRender, {

    draw: function (chart, options) {
        this.chartObj = chart;
        var lDirection;
        var sb = ej.EjSvgRender.utils._getStringBuilder();
        var currentseries = options;
        var style = this.setLineSeriesStyle(currentseries);
        if (currentseries.sorting)
            currentseries.points = ej.DataManager(currentseries.points, ej.Query().sortBy("X")).executeLocal();
        var visiblePoints = this.improveChartPerformance(currentseries);

        var firstPoint = null;
        var secondPoint;
        var firstIndex = -1;
        var xOffset = 0;
        if (currentseries.xAxis._valueType.toLowerCase() == "category" && currentseries.xAxis.labelPlacement.toLowerCase() != "onticks")
            xOffset = 0.5;
        for (var i = 0; i <= visiblePoints.length; i++) {
            //secondPoint = visiblePoints[i];
            if (i < visiblePoints.length && visiblePoints[i].visible) {
                    secondPoint = { X: visiblePoints[i].X - xOffset, YValues: [visiblePoints[i].y] };
                    point1 = ej.EjSvgRender.utils._getPoint(secondPoint, currentseries);
                if (firstPoint != null) {
                    if (visiblePoints.length > firstIndex + 1) {
                        var step = {X:secondPoint.X , YValues:[firstPoint.YValues[0]]};
                        var stepPoint = ej.EjSvgRender.utils._getPoint(step, currentseries);
                        var chartObj = this.chartObj;
                        sb.append("M" + " " + (point1.X + chartObj.canvasX) + " " + ((point1.Y + chartObj.canvasY)) + " " + "L" + " " + (stepPoint.X + chartObj.canvasX) + " " + (stepPoint.Y + chartObj.canvasY) + " " + "L" + " " + (point2.X + chartObj.canvasX) + " " + (point2.Y + chartObj.canvasY) + " ");
                    }
                }
                    firstPoint = secondPoint;
                    point2 = point1;  
            }
            else {
                if (i == visiblePoints.length && xOffset > 0) {
                    secondPoint = { X: visiblePoints[i-1].X + xOffset, YValues: [visiblePoints[i-1].y] };
                    var point1 = ej.EjSvgRender.utils._getPoint(secondPoint, currentseries);
                    sb.append("M" + " " + (point1.X + chartObj.canvasX) + " " + ((point1.Y + chartObj.canvasY)) + " " + "L" + " " + (point2.X + chartObj.canvasX) + " " + (point2.Y + chartObj.canvasY) + " ");
                }
                else
                    firstPoint = null;
            }
        }
        lDirection = sb.toString();

        this._drawLinePath(currentseries, style, lDirection);

        this.chartObj.svgRenderer.append(this.gSeriesGroupEle, this.chartObj.gSeriesEle);

      
       
       
    },
    doAnimation: function (series) {

        this._doLineAnimation(series);
    }

});

ej.ejStepAreaSeries = ejExtendClass(ej.EjSeriesRender, {

    draw: function (chart, options) {
        this.chartObj = chart;
        var aDirection;
        var sb = ej.EjSvgRender.utils._getStringBuilder();
        var currentseries = options;
        var style = this.setAreaSeriesStyle(currentseries);
        if (currentseries.sorting)
            currentseries.points = ej.DataManager(currentseries.points, ej.Query().sortBy("X")).executeLocal();
        var visiblePoints = this.improveChartPerformance(currentseries);
        var xOffset = 0;
        var firstPoint = null;
        var secondPoint;
        if (currentseries.xAxis._valueType.toLowerCase() == "category" && currentseries.xAxis.labelPlacement.toLowerCase() != "onticks")
            xOffset = 0.5;
        var origin = Math.max(options.yAxis.visibleRange.min, 0);
        var startPoint = null;
        var start = true;
        var chartObj = this.chartObj;
        for (var i = 0; i <= visiblePoints.length; i++) {

            if (i < visiblePoints.length) {
                if (visiblePoints[i].visible) {

                    secondPoint = { X: visiblePoints[i].X - xOffset, YValues: [visiblePoints[i].y] };
                    point1 = ej.EjSvgRender.utils._getPoint(secondPoint, currentseries);
                    if (!startPoint) {
                        startPoint = { X: visiblePoints[i].X - xOffset, YValues: [origin] }
                        var startLoc = ej.EjSvgRender.utils._getPoint(startPoint, currentseries);
                        sb.append("M" + " " + (startLoc.X + chartObj.canvasX) + " " + (startLoc.Y + chartObj.canvasY));
                    }
                    if (firstPoint != null) {
                        var step = { X: secondPoint.X, YValues: [firstPoint.YValues[0]] };
                        var stepPoint = ej.EjSvgRender.utils._getPoint(step, currentseries);
                        if (start) {
                            sb.append(" " + "L" + " " + (point2.X + chartObj.canvasX) + " " + (point2.Y + chartObj.canvasY) + " ");
                            start = false;
                        }
                        sb.append("L" + " " + (stepPoint.X + chartObj.canvasX) + " " + ((stepPoint.Y + chartObj.canvasY)) + " " + "L" + " " + (point1.X + chartObj.canvasX) + " " + (point1.Y + chartObj.canvasY) + " ");
                        if ((xOffset == 0 && i == visiblePoints.length - 1) || (i < (visiblePoints.length - 1) && !visiblePoints[i + 1].visible)) {
                            if (xOffset > 0) {
                                secondPoint = { X: visiblePoints[i].X + xOffset, YValues: [visiblePoints[i].y] };
                                var point1 = ej.EjSvgRender.utils._getPoint(secondPoint, currentseries);
                                sb.append("L" + " " + (point1.X + chartObj.canvasX) + " " + (point1.Y + chartObj.canvasY) + " " + "L" + " " + (point1.X + chartObj.canvasX) + " " + (startLoc.Y + chartObj.canvasY) + " ");
                            }
                            else
                                sb.append("L" + " " + point1.X + chartObj.canvasX + " " + (startLoc.Y + chartObj.canvasY) + " ");
                            startPoint = null;
                            firstPoint = null;
                            start = true;
                        }
                    }
                    if (startPoint) {
                        point2 = point1;
                        firstPoint = secondPoint;
                    }
                }
                else {
                    startPoint = null;
                    firstPoint = null;
                    start = true;
                }

              }
             else {
                    if  (visiblePoints[i-1].visible && xOffset > 0) {
                                secondPoint = { X: visiblePoints[i - 1].X + xOffset, YValues: [visiblePoints[i - 1].y] };
                                var point1 = ej.EjSvgRender.utils._getPoint(secondPoint, currentseries);
                                sb.append("L" + " " + (point1.X + chartObj.canvasX) + " " + (point1.Y + chartObj.canvasY) + " " + "L" + " " + (point1.X + chartObj.canvasX) + " " + (startLoc.Y + chartObj.canvasY) + " ");
                       }                           
                }
            
         }
        aDirection = sb.toString();

        this.drawAreaPath(currentseries, style, aDirection);

        this.chartObj.svgRenderer.append(this.gSeriesGroupEle, this.chartObj.gSeriesEle);

       
     
    },

    doAnimation: function (series) {

        this._doLineAnimation(series);
    }

});
ej.seriesTypes.steparea = ej.ejStepAreaSeries;


ej.seriesTypes.stepline = ej.ejStepLineSeries;


 ej.ejColumnSeries = ejExtendClass(ej.EjSeriesRender, {
   
  
      
     draw: function (chart,options)
      { 
         this.chartObj=chart;
         var series = options;
         var seriesIndex = $.inArray(series, this.chartObj.model._visibleSeries);    
         var origin = Math.max(options.yAxis.visibleRange.min, 0);
        
         var sidebysideinfo = this.getSideBySideInfo(series);    
         var visiblePoints = this._isVisiblePoints(series);
              
         var transX = series.xAxis.x;
         var transY = series.yAxis.y;
         
         var serOptions = { 'id': this.chartObj.svgObject.id + '_SeriesGroup' + '_' + seriesIndex, 'transform': 'translate(' + transX + ',' + transY + ')' };

         this.gSeriesGroupEle = this.chartObj.svgRenderer.createGroup(serOptions);
         
         var cSer = this;
         for (var i = 0; i < visiblePoints.length; i++) {
             var pointIndex=i;
             var point=visiblePoints[i];
           
             var y1 = point.YValues[0];
             var y2 = origin;
             if (point.visible) {
             //calculate sides
             var data = cSer.calculateSides(point, sidebysideinfo);
             var x1 = data.x1;
             var x2 = data.x2;
             
			 var styleOptions= this.chartObj.setStyle(cSer, series, seriesIndex, pointIndex);

			 var rect = cSer.getRectangle(x1, y1, x2, y2, series);

             if (rect.X < 0) {
                 rect.Width = rect.Width + rect.X;
                 rect.X = 0;
             }
			 
             rect.Width = ((rect.X + rect.Width) > cSer.chartObj.model.m_AreaBounds.Width) ? cSer.chartObj.model.m_AreaBounds.Width - rect.X : rect.Width;
             //drawing part
             var xr = Math.min(0, rect.Width);
             var yr = Math.min(0, rect.Height);

             var  bounds;        
             if ((xr == 0 || yr == 0) && rect.Width>0) {                      
                         options = {
                             'id': cSer.chartObj.svgObject.id + '_Series' + seriesIndex+ '_Point' + pointIndex,
                             'x': rect.X + cSer.chartObj.canvasX,
                             'y': rect.Y + cSer.chartObj.canvasY,
                             'width': rect.Width,
                             'height': rect.Height,
                             'fill': styleOptions.interior,
                             'stroke-width': styleOptions.borderWidth,
                             'plot': y1 < 0 ? "negative" : "positive",
                              'opacity': styleOptions.opacity,
                             'stroke': styleOptions.borderColor
                         };

                         cSer.chartObj.svgRenderer.drawRect(options, cSer.gSeriesGroupEle);

                         var svgXy = ej.EjSvgRender.utils._getSvgXY(rect.X, (rect.Y), series, cSer.chartObj);
                        bounds = { X: svgXy.X, Y: svgXy.Y, Width: rect.Width, Height: rect.Height };
                        ej.EjSvgRender.utils._addRegion(cSer.chartObj, bounds, series, point, pointIndex);
                 }
                 if ((!ej.util.isNullOrUndefined(series.marker) && series.marker.visible) ||(!ej.util.isNullOrUndefined(series.marker) && series.marker.dataLabel.visible)) {
                     point.symbolLocation = { X: rect.X + (rect.Width / 2), Y: ((y1 < 0 && !series.yAxis.isInversed) || (series.yAxis.isInversed && y1 > 0)) ? ((rect.Y) + (rect.Height)) : (rect.Y) };
                 }
             }
         }

         this.chartObj.svgRenderer.append(this.gSeriesGroupEle, this.chartObj.gSeriesEle);

        
     },
   
     doAnimation: function (series) {
       var elements = $(this.chartObj.gSeriesEle).find("#" + this.gSeriesGroupEle.id).children("rect");
       for (var i = 0; i < elements.length; i++) {
           var element = elements[i];
           this.animateRect(element, series);
       }
        var symbolElements = $(this.chartObj.gSymbolGroupEle).children().not("defs");
        for (var j = 0; j < symbolElements.length; j++) {
            var symbol = symbolElements[j];
            this._doLineSymbol(symbol, 500, 2, series, this.chartObj.model.requireInvertedAxes);
        }   
   }

 });
 ej.seriesTypes.column = ej.ejColumnSeries;

 ej.ejStackingColumnSeries = ejExtendClass(ej.EjSeriesRender, {

     draw: function (chart, options) {
         this.chartObj = chart;
         var series = options;
         var seriesIndex = $.inArray(series, this.chartObj.model._visibleSeries);
         //var origin = Math.max(options.yAxis.visibleRange.min, 0);
        
         var sidebysideinfo = this.getSideBySideInfo(series);
         var visiblePoints = this._isVisiblePoints(series);

         var transX = series.xAxis.x;
         var transY = series.yAxis.y;

         var serOptions = { 'id': this.chartObj.svgObject.id + '_SeriesGroup' + '_' + seriesIndex, 'transform': 'translate(' + transX + ',' + transY + ')' };

         this.gSeriesGroupEle = this.chartObj.svgRenderer.createGroup(serOptions);
         var cSer = this;
         for (var i = 0; i < visiblePoints.length; i++) {
             var pointIndex = i;
             var point = visiblePoints[i];

             var y2 = series.stackedValue.StartValues[i];
             var y1= series.stackedValue.EndValues[i];
             if (point.visible) {
                 //calculate sides
                 var data = cSer.calculateSides(point, sidebysideinfo);
                 var x1 = data.x1;
                 var x2 = data.x2;
                 
				 var styleOptions= this.chartObj.setStyle(cSer, series, seriesIndex, pointIndex);

                 var rect = cSer.getRectangle(x1, y1, x2, y2, series);
                 if (rect.X < 0) {
                     rect.Width = rect.Width + rect.X;
                     rect.X = 0;
                 }

                 rect.Width = ((rect.X + rect.Width) > cSer.chartObj.model.m_AreaBounds.Width) ? cSer.chartObj.model.m_AreaBounds.Width - rect.X : rect.Width;
                 //drawing part
                 var xr = Math.min(0, rect.Width);
                 var yr = Math.min(0, rect.Height);

                 var bounds;
                 if (xr == 0 || yr == 0) {
                     options = {
                         'id': cSer.chartObj.svgObject.id + '_Series' + seriesIndex + '_Point' + pointIndex,
                         'x': rect.X + cSer.chartObj.canvasX,
                         'y': rect.Y + cSer.chartObj.canvasY,
                         'width': rect.Width,
                         'height': rect.Height,
                         'fill': styleOptions.interior,
                         'stroke-width': styleOptions.borderWidth,
                         'plot': y1 < 0 ? "negative" : "positive",
                         'opacity': styleOptions.opacity,
                         'stroke': styleOptions.borderColor
                     };

                     cSer.chartObj.svgRenderer.drawRect(options, cSer.gSeriesGroupEle);

                     var svgXy = ej.EjSvgRender.utils._getSvgXY(rect.X, (rect.Y), series, cSer.chartObj);
                     bounds = { X: svgXy.X, Y: svgXy.Y, Width: rect.Width, Height: rect.Height };
                     ej.EjSvgRender.utils._addRegion(cSer.chartObj, bounds, series, point, pointIndex);
                 }
                 if ((!ej.util.isNullOrUndefined(series.marker) && series.marker.visible) || (!ej.util.isNullOrUndefined(series.marker) && series.marker.dataLabel.visible)) {
                     point.symbolLocation = { X: rect.X + (rect.Width / 2), Y: ((y1 < 0 && !series.yAxis.isInversed) || (series.yAxis.isInversed && y1 > 0)) ? ((rect.Y) + (rect.Height)) : (rect.Y) };
                 }
             }
         }

         this.chartObj.svgRenderer.append(this.gSeriesGroupEle, this.chartObj.gSeriesEle);

        

     },
     doAnimation: function (series) {
         var elements = $(this.chartObj.gSeriesEle).find("#" + this.gSeriesGroupEle.id).children("rect");
         for (var i = 0; i < elements.length; i++) {
             var element = elements[i];
             this.animateStackingRect(element, series, this.chartObj.model.requireInvertedAxes);
         }
         var symbolElements = $(this.chartObj.gSymbolGroupEle).children().not("defs");
         for (var j = 0; j < symbolElements.length; j++) {
             var symbol = symbolElements[j];
             this._doLineSymbol(symbol, 500, 2, series, this.chartObj.model.requireInvertedAxes);
         }
     },
     
     stackingSeries: true

 });
 ej.seriesTypes.stackingcolumn = ej.ejStackingColumnSeries;
 
 ej.seriesTypes.rangecolumn = ej.ejRangeColumnSeries;

 ej.ejRangeColumnSeries = ejExtendClass(ej.EjSeriesRender, {
   
     draw: function (chart, options) {
         this.chartObj = chart;
         var series = options;
         var seriesIndex = $.inArray(series, this.chartObj.model._visibleSeries);
      
         var sidebysideinfo = this.getSideBySideInfo(series);
         var visiblePoints = this._isVisiblePoints(series);

         var transX = series.xAxis.x;
         var transY = series.yAxis.y;

         var serOptions = { 'id': this.chartObj.svgObject.id + '_SeriesGroup' + '_' + seriesIndex, 'transform': 'translate(' + transX + ',' + transY + ')' };

         this.gSeriesGroupEle = this.chartObj.svgRenderer.createGroup(serOptions);
         var cSer = this;
         for (var i = 0; i < visiblePoints.length; i++) {
             var pointIndex = i;
             var point = visiblePoints[i];

             var y1 = point.YValues[0];
             var y2 = point.YValues[1];
             if (point.visible) {
                 //calculate sides
                 var data = cSer.calculateSides(point, sidebysideinfo);
                 var x1 = data.x1;
                 var x2 = data.x2;
                  
				 var styleOptions = this.chartObj.setStyle(cSer, series, seriesIndex, pointIndex);
				  
                 var rect = cSer.getRectangle(x1, y1, x2, y2, series);
                 if (rect.X < 0) {
                     rect.Width = rect.Width + rect.X;
                     rect.X = 0;
                 }
                 var width=cSer.chartObj.model.m_AreaBounds.Width;
                 rect.Width = ((rect.X + rect.Width) > width) ? width - rect.X : rect.Width;
                 //drawing part
                 var xr = Math.min(0, rect.Width);
                 var yr = Math.min(0, rect.Height);

                 var bounds;
                 if (xr == 0 || yr == 0) {
                     options = {
                         'id': cSer.chartObj.svgObject.id + '_series' + seriesIndex + '_Point' + pointIndex,
                         'x': rect.X + cSer.chartObj.canvasX,
                         'y': rect.Y + cSer.chartObj.canvasY,
                         'width': rect.Width,
                         'height': rect.Height,
                         'fill': styleOptions.interior,
                         'stroke-width': styleOptions.borderWidth,
                         'opacity': styleOptions.opacity,
                         'stroke': styleOptions.borderColor
                     };

                     cSer.chartObj.svgRenderer.drawRect(options, cSer.gSeriesGroupEle);

                     //Add region for each rect
                     var svgXy = ej.EjSvgRender.utils._getSvgXY(rect.X, (rect.Y), series, cSer.chartObj);
                     bounds = { X: svgXy.X, Y: svgXy.Y, Width: rect.Width, Height: rect.Height };
                     ej.EjSvgRender.utils._addRegion(cSer.chartObj, bounds, series, point, pointIndex);
                 }
                 //Add rect location details for symbol
                 if ((!ej.util.isNullOrUndefined(series.marker) && series.marker.visible) || (!ej.util.isNullOrUndefined(series.marker) && series.marker.dataLabel.visible)) {
                     point.symbolLocation = { X: rect.X + (rect.Width / 2), Y: ((y1 < 0 && !series.yAxis.isInversed) || (series.yAxis.isInversed && y1 > 0)) ? ((rect.Y) + (rect.Height)) : (rect.Y) };
                 }
             }
         }

         this.chartObj.svgRenderer.append(this.gSeriesGroupEle, this.chartObj.gSeriesEle);

      

     },

     doAnimation: function (series) {
         var elements = $(this.chartObj.gSeriesEle).find("#" + this.gSeriesGroupEle.id).children("rect");
         for (var i = 0; i < elements.length; i++) {
             var element = elements[i];
             this.animateRect(element,series);
         }
         var symbolElements = $(this.chartObj.gSymbolGroupEle).children().not("defs");
         for (var j = 0; j < symbolElements.length; j++) {
             var symbol = symbolElements[j];
             this._doLineSymbol(symbol, 500, 2, series, this.chartObj.model.requireInvertedAxes);
         }
     },
     hiloTypes: true

 });
 ej.seriesTypes.rangecolumn = ej.ejRangeColumnSeries;

 ej.ejStackingBarSeries = ejExtendClass(ej.EjSeriesRender, {
     getPointXYOrgin: function (x, y,series) {
         var xvalue = (1 - ej.EjSvgRender.utils._valueToCoefficient(series.xAxis, x)) * (series.xAxis.height);
         var yvalue = (ej.EjSvgRender.utils._valueToCoefficient(series.yAxis, y)) * (series.yAxis.width);
         return { X: yvalue, Y: xvalue };
     },
     getRectangle: function (x1, y1, x2, y2, series) {
         var pt1 = this.getPointXYOrgin(x1, y1,series);
         var pt2 = this.getPointXYOrgin(x2, y2,series);
         return ej.EjSvgRender.utils._correctRect(pt1.X, pt1.Y, pt2.X, pt2.Y);
     },
     calculateSides: function (point, sidebysideinfo) {
         var x1 = point.X + sidebysideinfo.Start;
         var x2 = point.X + sidebysideinfo.End;
         return { x1: x1, x2: x2 };
     },
     draw: function (chart, options) {
         this.chartObj = chart;
         var series = options;
         var seriesIndex = $.inArray(series, this.chartObj.model._visibleSeries);
         //var origin = Math.max(options.yAxis.visibleRange.min, 0);
         var sidebysideinfo = this.getSideBySideInfo(series);
         var visiblePoints = this._isVisiblePoints(series);
         var trans = ej.EjSvgRender.utils._getTransform(series.xAxis, series.yAxis, this.chartObj.model.requireInvertedAxes);

         var serOptions = { 'id': this.chartObj.svgObject.id + '_SeriesGroup' + '_' + seriesIndex, 'transform': 'translate(' + trans.x + ',' + trans.y + ')' };

         this.gSeriesGroupEle = this.chartObj.svgRenderer.createGroup(serOptions);
         var cSer = this;
         for (var i = 0; i < visiblePoints.length; i++) {
             var pointIndex = i;
             var point = visiblePoints[i];
             var y1 = series.stackedValue.EndValues[i];
             var y2 =series.stackedValue.StartValues[i];
             if (point.visible) {
                 //calculate sides
                 var data = cSer.calculateSides(point, sidebysideinfo);
                 var x1 = data.x1;
                 var x2 = data.x2;
                 
				 var styleOptions = this.chartObj.setStyle(cSer, series, seriesIndex, pointIndex);

                 var rect = cSer.getRectangle(x1, y1, x2, y2, series);

                 var xr = Math.min(0, rect.Width);
                 var yr = Math.min(0, rect.Height);

                 if (xr == 0 || yr == 0) {
                     var bounds;
                     options = {
                         'id': cSer.chartObj.svgObject.id + '_Series' + seriesIndex + '_Point' + pointIndex,
                         'x': rect.X + cSer.chartObj.canvasX,
                         'y': rect.Y + cSer.chartObj.canvasY,
                         'width': rect.Width,
                         'height': rect.Height,
                         'fill': styleOptions.interior,
                         'stroke-width': styleOptions.borderWidth,
						 'opacity': styleOptions.opacity,
                         'stroke': styleOptions.borderColor
                     };

                     cSer.chartObj.svgRenderer.drawRect(options, cSer.gSeriesGroupEle);

                     var svgXy = ej.EjSvgRender.utils._getSvgXY((rect.X), (rect.Y), series, cSer.chartObj);
                     bounds = { X: svgXy.X, Y: svgXy.Y, Width: rect.Width, Height: rect.Height };

                     ej.EjSvgRender.utils._addRegion(cSer.chartObj, bounds, series, point, pointIndex);
                 }

                 if ((!ej.util.isNullOrUndefined(series.marker) && series.marker.visible) || (!ej.util.isNullOrUndefined(series.marker) && series.marker.dataLabel.visible)) {
                     point.symbolLocation = { X: ((y1 < 0 && !series.yAxis.isInversed) || (series.yAxis.isInversed && y1 > 0)) ? (rect.X) : (rect.X + rect.Width), Y: ((rect.Y) + (rect.Height / 2)) };
                 }
             }
         }

         this.chartObj.svgRenderer.append(this.gSeriesGroupEle, this.chartObj.gSeriesEle);
 

     },
     doAnimation: function (series) {
         var elements = $(this.chartObj.gSeriesEle).find("#" + this.gSeriesGroupEle.id).children("rect");
         for (var i = 0; i < elements.length; i++) {
             var element = elements[i];
             this.animateStackingBar(element, series, this.chartObj.model.requireInvertedAxes);
         }
         var symbolElements = $(this.chartObj.gSymbolGroupEle).children().not("defs");
         for (var j = 0; j < symbolElements.length; j++) {
             var symbol = symbolElements[j];
             this._doLineSymbol(symbol, 500, 2, series, this.chartObj.model.requireInvertedAxes);
         }
     },

     stackingSeries: true,
     requireInvertedAxes: true

 });
 ej.seriesTypes.stackingbar = ej.ejStackingBarSeries;

 ej.ejBarSeries = ejExtendClass(ej.EjSeriesRender, {
     getPointXYOrgin: function (x, y, orginX, orginY, series) {
         var xvalue = (1-ej.EjSvgRender.utils._valueToCoefficient(series.xAxis, x)) * (series.xAxis.height);
         var yvalue = (ej.EjSvgRender.utils._valueToCoefficient(series.yAxis, y)) * (series.yAxis.width);
         return { X: yvalue, Y: xvalue };
     },
     getRectangle: function (x1, y1, x2, y2, series) {
         var orginX = series.xAxis.visibleRange.min;
         var orginY = Math.max(series.yAxis.visibleRange.min, 0);
         var pt1 = this.getPointXYOrgin(x1, y1, orginX, orginY, series);
         var pt2 = this.getPointXYOrgin(x2, y2, orginX, orginY, series);
         return ej.EjSvgRender.utils._correctRect(pt1.X, pt1.Y, pt2.X, pt2.Y);
     },
     calculateSides: function (point, sidebysideinfo) {
         var x1 = point.X + sidebysideinfo.Start;
         var x2 = point.X + sidebysideinfo.End;
         return { x1: x1, x2: x2 };
     },
     draw: function (chart, options) {
         this.chartObj = chart;
         var series = options;
         var seriesIndex = $.inArray(series, this.chartObj.model._visibleSeries);
         var origin = Math.max(options.yAxis.visibleRange.min, 0);       
         var style = $.extend(true, {}, this.chartObj.model.seriesStyle, series.style);
         var sidebysideinfo = this.getSideBySideInfo(series);
         var visiblePoints = this._isVisiblePoints(series);
         var colors;
         var trans = ej.EjSvgRender.utils._getTransform(series.xAxis, series.yAxis, this.chartObj.model.requireInvertedAxes);
         
         var serOptions = { 'id': this.chartObj.svgObject.id + '_SeriesGroup' + '_' + seriesIndex, 'transform': 'translate(' + trans.x + ',' + trans.y + ')' };

         this.gSeriesGroupEle = this.chartObj.svgRenderer.createGroup(serOptions);
         var cSer = this;
         for (var i = 0; i < visiblePoints.length; i++) {
             var pointIndex = i;
             var point = visiblePoints[i];           
             var y1 = point.YValues[0];
             var y2 = origin;
             if (point.visible) {
                 //calculate sides
                 var data = cSer.calculateSides(point, sidebysideinfo);
                 var x1 = data.x1;
                 var x2 = data.x2;
                 
				 var styleOptions = this.chartObj.setStyle(cSer, series, seriesIndex, pointIndex);

                 var rect = cSer.getRectangle(x1, y1, x2, y2, series);
                
                 var xr = Math.min(0, rect.Width);
                 var yr = Math.min(0, rect.Height);

                if (xr == 0 || yr == 0) {
                     var bounds;          
                         options = {
                             'id': cSer.chartObj.svgObject.id + '_Series' + seriesIndex+ '_Point' + pointIndex,
                             'x': rect.X + cSer.chartObj.canvasX,
                             'y': rect.Y + cSer.chartObj.canvasY,
                             'width': rect.Width,
                             'height': rect.Height,
                             'fill': styleOptions.interior,
                             'stroke-width': styleOptions.borderWidth,
                             'plot': y1 < 0 ? "negative" : "positive",
                             'opacity': styleOptions.opacity,
                             'stroke': styleOptions.borderColor
                         };

                         cSer.chartObj.svgRenderer.drawRect(options, cSer.gSeriesGroupEle);

                         var svgXy = ej.EjSvgRender.utils._getSvgXY((rect.X), (rect.Y), series, cSer.chartObj);
                        bounds = { X: svgXy.X, Y: svgXy.Y, Width: rect.Width, Height: rect.Height };
                     
                        ej.EjSvgRender.utils._addRegion(cSer.chartObj, bounds, series, point, pointIndex);
                 }


                 if ((!ej.util.isNullOrUndefined(series.marker) && series.marker.visible) || (!ej.util.isNullOrUndefined(series.marker) && series.marker.dataLabel.visible)) {
                     point.symbolLocation = { X: ((y1 < 0 && !series.yAxis.isInversed) || (series.yAxis.isInversed && y1 > 0)) ? (rect.X) : (rect.X + rect.Width), Y: ((rect.Y) + (rect.Height / 2)) };
                 }
             }
         }

         this.chartObj.svgRenderer.append(this.gSeriesGroupEle, this.chartObj.gSeriesEle);

        
     },
     doAnimation: function(series) {
         var elements = $(this.chartObj.gSeriesEle).find("#" + this.gSeriesGroupEle.id).children("rect");
         for (var i = 0; i < elements.length; i++) {
             var element = elements[i];
             this.animateBarRect(element, series, this.chartObj.model.requireInvertedAxes);
         }
         var symbolElements = $(this.chartObj.gSymbolGroupEle).children().not("defs");
         for (var j = 0; j < symbolElements.length; j++) {
             var symbol = symbolElements[j];
             this._doLineSymbol(symbol, 500, 2, series, this.chartObj.model.requireInvertedAxes);
         }   
     },
     requireInvertedAxes: true
 });
 ej.seriesTypes.bar = ej.ejBarSeries;


 ej.ejStackingAreaSeries = ejExtendClass(ej.EjSeriesRender, {

     draw: function (chart, options) {
         this.chartObj = chart;
         var aDirection;
         var areasb = ej.EjSvgRender.utils._getStringBuilder();
         var currentSeries = options;
         var style = this.setAreaSeriesStyle(currentSeries);

         currentSeries.points = ej.DataManager(currentSeries.points, ej.Query().sortBy("X")).executeLocal();



         var visiblePoints = this._isVisiblePoints(currentSeries);
     
         var origin = Math.max(options.yAxis.visibleRange.min, currentSeries.stackedValue.StartValues[0]);
         var canvasX = this.chartObj.canvasX;
         var canvasY = this.chartObj.canvasY;

         var startPoint = { X: visiblePoints[0].X, YValues: [origin] };
         var startLoc = ej.EjSvgRender.utils._getPoint(startPoint, currentSeries);
         areasb.append("M" + " " + (startLoc.X + canvasX) + " " + (startLoc.Y + canvasY) + " ");
         //var index = $.inArray(currentSeries, this.chartObj.model._visibleSeries);
        
         for (j = 0; j < currentSeries.stackedValue.EndValues.length; j++) {
             if (visiblePoints[j].visible) {
                    if (visiblePoints.length > j + 1) {
                        currentPoint = { X: visiblePoints[j].X, YValues: [currentSeries.stackedValue.EndValues[j]] };
                     if (visiblePoints[j + 1].visible)
                         nextpoint = { X: visiblePoints[j + 1].X, YValues: [currentSeries.stackedValue.EndValues[j + 1]] };
                     else
                         nextpoint = { X: visiblePoints[j].X, YValues: [origin] };

                       point1 = ej.EjSvgRender.utils._getPoint(currentPoint, currentSeries);
                       point2 = ej.EjSvgRender.utils._getPoint(nextpoint, currentSeries);

                     if (j > 0 && !visiblePoints[j - 1].visible) {
                         previouspoint = ej.EjSvgRender.utils._getPoint({ X: visiblePoints[j - 1].X, YValues: [origin] }, currentSeries);
                         areasb.append("L" + " " + (previouspoint.X + canvasX) + " " + ((previouspoint.Y + canvasY)) + " " + " " + "L" + " " + (point1.X + canvasX) + " " + ((previouspoint.Y + canvasY)) + " ");
                     }

                     areasb.append("L" + " " + (point1.X + canvasX) + " " + ((point1.Y + canvasY)) + " " + "L" + " " + (point2.X + canvasX) + " " + ((point2.Y + canvasY)) + " ");

                 }
             }
         }
                
        
          
         for (var j = currentSeries.stackedValue.StartValues.length - 1; j > 0; j--) {
                 if (visiblePoints[j].visible) {

                     var currentPoint = { X: visiblePoints[j].X, YValues: [currentSeries.stackedValue.StartValues[j]] };
                     var nextpoint = { X: visiblePoints[j - 1].X, YValues: [currentSeries.stackedValue.StartValues[j - 1]] };

                     var point1 = ej.EjSvgRender.utils._getPoint(currentPoint, currentSeries);
                     var point2 = ej.EjSvgRender.utils._getPoint(nextpoint, currentSeries);
                 if (this.chartObj.model.enableCanvasRendering)
                     areasb.append("L" + " " + (point1.X + canvasX) + " " + (point1.Y + canvasY) + " " + "L" + " " + (point2.X + canvasX) + " " + (point2.Y + canvasY) + " ");
                 else
                     areasb.append((point1.X) + " " + ((point1.Y)) + " " + (point2.X) + " " + ((point2.Y)) + " ");
                     }
                 }
                

         aDirection = areasb.toString();

         this.drawAreaPath(currentSeries, style, aDirection);

         this.chartObj.svgRenderer.append(this.gSeriesGroupEle, this.chartObj.gSeriesEle);

          


     },

     doAnimation: function (series) {

         this._doLineAnimation(series);
     },
     stackingSeries:true

 });
 ej.seriesTypes.stackingarea = ej.ejStackingAreaSeries;

 ej.ejAreaSeries = ejExtendClass(ej.EjSeriesRender, {

  
     draw: function (chart,options) {
         this.chartObj = chart;
         var aDirection;
         var areasb = ej.EjSvgRender.utils._getStringBuilder();
         var currentSeries = options;
         var style = this.setAreaSeriesStyle(currentSeries);
         var chartObj = this.chartObj;
         currentSeries.points = ej.DataManager(currentSeries.points, ej.Query().sortBy("X")).executeLocal();
      
         var visiblePoints = this._isVisiblePoints(currentSeries);
         var origin =  Math.max(options.yAxis.visibleRange.min, 0);
         
         var startPoint;
         var translate = [];
         translate[0] = chartObj.canvasX;
         translate[1] = chartObj.canvasY;
        
         for (var i = 0; i < visiblePoints.length; i++) {
             if (visiblePoints[i].visible) {
                 if (visiblePoints.length > i + 1) {
                     if (!startPoint) {
                         startPoint = { X: visiblePoints[i].X, YValues: [origin] }
                         var point1 = ej.EjSvgRender.utils._getPoint(startPoint, currentSeries);
                         areasb.append("M" + " " + (point1.X) + " " + ((point1.Y)));
                     }


                     var point1 = ej.EjSvgRender.utils._getPoint(visiblePoints[i], currentSeries);

                     areasb.append(" " + "L" + " " + (point1.X) + " " + ((point1.Y)));

                     if (!visiblePoints[i + 1].visible) {
                         point = { X: visiblePoints[i].X, YValues: [origin] }
                         var point2 = ej.EjSvgRender.utils._getPoint(point, currentSeries);
                         var point3 = ej.EjSvgRender.utils._getPoint(startPoint, currentSeries);
                         areasb.append(" " + "L" + " " + (point2.X) + " " + ((point2.Y)) + " " + "L" + " " + (point3.X) + " " + ((point3.Y)));
                         startPoint = null;
                     }
                 }
                 else {
                     if (visiblePoints[i - 1] && visiblePoints[i - 1].visible) {
                         var point1 = ej.EjSvgRender.utils._getPoint(visiblePoints[i], currentSeries);
                         areasb.append(" " + "L" + " " + (point1.X) + " " + ((point1.Y)));
                     }
                 }
             }         
         }

         var endPoint = { X: visiblePoints[visiblePoints.length - 1].X, YValues: [origin] };
         var endLoc = ej.EjSvgRender.utils._getPoint(endPoint, currentSeries);
         if (visiblePoints.length > 1)
             areasb.append(" " + "L" + " " + (endLoc.X) + " " + ((endLoc.Y)) + " ");
         
         aDirection = areasb.toString();

         this.drawAreaPath(currentSeries, style, aDirection, translate);

         this.chartObj.svgRenderer.append(this.gSeriesGroupEle, this.chartObj.gSeriesEle);

     
       

     },

       doAnimation: function(series) {

           this._doLineAnimation(series);
       }

 });

 ej.seriesTypes.area = ej.ejAreaSeries;

 ej.ejRangeAreaSeries = ejExtendClass(ej.EjSeriesRender, {


     draw: function (chart, options) {
         this.chartObj = chart;
         var aDirection;
         var areasb = ej.EjSvgRender.utils._getStringBuilder();
         var currentSeries = options;
         var style = this.setAreaSeriesStyle(currentSeries);

         currentSeries.points = ej.DataManager(currentSeries.points, ej.Query().sortBy("X")).executeLocal();

         var visiblePoints = this._isVisiblePoints(currentSeries);        
         var translate = [];
         translate[0] = this.chartObj.canvasX;
         translate[1] = this.chartObj.canvasY;
         var startPoint;

         for (  var i = 0; i <visiblePoints.length; i++) {
             if (visiblePoints[i].visible) {
                 if (visiblePoints.length>i+1)
                     {
                     if (!startPoint) {
                         var origin = visiblePoints[i].low;
                         startPoint = { X: visiblePoints[i].X, YValues: [origin] }
                         var point1 = ej.EjSvgRender.utils._getPoint(startPoint, currentSeries);
                         areasb.append("M" + " " + (point1.X) + " " + ((point1.Y)));
                     }
                 var point1 = ej.EjSvgRender.utils._getPoint(visiblePoints[i], currentSeries);
                 areasb.append(" " + "L" + " " + (point1.X) + " " + ((point1.Y)));

                 if(!visiblePoints[i+1].visible)
                 {
                     for (var j = i; j >=0; j--)
                     {
                         if (visiblePoints[j].visible) {
                             origin = visiblePoints[j].low;
                             var point = { X: visiblePoints[j].X, YValues: [origin] };
                             var point2 = ej.EjSvgRender.utils._getPoint(point, currentSeries);
                             areasb.append(" " + "L" + " " + (point2.X) + " " + ((point2.Y)));
                         }
                         else
                             break;
                     }
                     startPoint = null;
                 }

                 }
                 else
                 {
                     origin = visiblePoints[i].high;
                     var endPoint = { X: visiblePoints[visiblePoints.length - 1].X, YValues: [origin] };
                     var endLoc = ej.EjSvgRender.utils._getPoint(endPoint, currentSeries);
                     if (visiblePoints.length > 1)
                         areasb.append(" " + "L" + " " + (endLoc.X) + " " + ((endLoc.Y)));
                 }
             }
         }
         for (var i = visiblePoints.length; i > 0; i--)
         {
             if (visiblePoints[i - 1].visible)
             {
                 origin = visiblePoints[i-1].low;
                 var point = { X: visiblePoints[i - 1].X, YValues: [origin] };
                 var point2 = ej.EjSvgRender.utils._getPoint(point, currentSeries);
                 areasb.append(" " + "L" + " " + (point2.X) + " " + ((point2.Y)));
             }
             else if (visiblePoints.length>i)
             {
                 var point2 = ej.EjSvgRender.utils._getPoint(visiblePoints[i], currentSeries);
                 areasb.append(" " + "L" + " " + (point2.X) + " " + ((point2.Y)));
                 break;
             }                       
         }     
         aDirection = areasb.toString();
         this.drawAreaPath(currentSeries, style, aDirection, translate);
         this.chartObj.svgRenderer.append(this.gSeriesGroupEle, this.chartObj.gSeriesEle);
     },

     doAnimation: function (series) {

         this._doLineAnimation(series);
     },
     hiloTypes:true

 });

 ej.seriesTypes.rangearea = ej.ejRangeAreaSeries;

 ej.ejSplineSeries = ejExtendClass(ej.EjSeriesRender, {
    
     draw: function (chart,options) {
         this.chartObj = chart;
         var series = options;
         var spDirection = "";
         var splinesb = ej.EjSvgRender.utils._getStringBuilder();
         var style = this.setLineSeriesStyle(series);
         var yIndex = 0;
         var visiblePoints = this._isVisiblePoints(series);
         var ySpline = this.naturalSpline(visiblePoints);
         var firstPoint = null;
         var secondPoint = null;
         var firstIndex = -1;
		 //Removed spline sorting behavior based on the 'X' points.
         //series.points = ej.DataManager(series.points, ej.Query().sortBy("X")).executeLocal();
         
         for (var i = 0; i < visiblePoints.length; i++) {
             var pointIndex = i;
             secondPoint = visiblePoints[i];             
             if (secondPoint.visible) {
                 if (firstPoint != null) {
                     var controlPoint1 = null;
                     var controlPoint2 = null;
                     var data = this.getBezierControlPoints(firstPoint, secondPoint, ySpline[firstIndex], ySpline[pointIndex], yIndex);
                     controlPoint1 = data["controlPoint1"];
                     controlPoint2 = data["controlPoint2"];
                     var pt1 = ej.EjSvgRender.utils._getPoint(firstPoint, series);
                     var pt2 = ej.EjSvgRender.utils._getPoint(secondPoint, series);
                     var bpt1 = ej.EjSvgRender.utils._getPoint(controlPoint1, series);
                     var bpt2 = ej.EjSvgRender.utils._getPoint(controlPoint2, series);
                     var chartObj = this.chartObj;
                     splinesb.append("M" + " " + (pt1.X + chartObj.canvasX) + " " + (pt1.Y + chartObj.canvasY) + " " + "C" + " " + (bpt1.X + chartObj.canvasX) + " " + (bpt1.Y + chartObj.canvasY) + " " + (bpt2.X + chartObj.canvasX) + " " + (bpt2.Y + chartObj.canvasY) + " " + (pt2.X + chartObj.canvasX) + " " + (pt2.Y + chartObj.canvasY) + " ");

                 }
                 firstPoint = secondPoint;
                 firstIndex = pointIndex;
             } else {
                 firstPoint = null;
             }
         }
         
        spDirection = splinesb.toString();
        this._drawLinePath(series, style, spDirection);

         this.chartObj.svgRenderer.append(this.gSeriesGroupEle, this.chartObj.gSeriesEle);
 
       

     },

     doAnimation: function (series) {

          this._doLineAnimation(series);
       }

 });

 ej.seriesTypes.spline = ej.ejSplineSeries;

 ej.ejSplineAreaSeries = ejExtendClass(ej.EjSeriesRender, {


     draw: function (chart, options) {
         this.chartObj = chart;
         var series = options;
         var spDirection = "";
         var splinesb = ej.EjSvgRender.utils._getStringBuilder();
         var style = this.setAreaSeriesStyle(series);
         var yIndex = 0;
         //Removed splineArea sorting behavior based on the 'X' points.
         series.points = ej.DataManager(series.points, ej.Query().sortBy("X")).executeLocal();
         var visiblePoints = this._isVisiblePoints(series);
         var ySpline = this.naturalSpline(visiblePoints);
         var firstPoint = null;
         var secondPoint = null;
         var firstIndex = -1;
		
         var origin = Math.max(options.yAxis.visibleRange.min, 0);
         var startPoint = null;
         var start = true;
         var chartObj = this.chartObj;
         var translate = [];
         translate[0] = chartObj.canvasX;
         translate[1] = chartObj.canvasY;
         var count = 0;
         for (var i = 0; i < visiblePoints.length; i++) {
             var pointIndex = i;
             secondPoint = visiblePoints[i];
             if (secondPoint.visible) {
                 if (!startPoint) {
                     startPoint = { X: visiblePoints[i].X, YValues: [origin] };
                     var startLoc = ej.EjSvgRender.utils._getPoint(startPoint, series);
                    }
                 if (firstPoint != null) {
                     var controlPoint1 = null;
                     var controlPoint2 = null;
                     var data = this.getBezierControlPoints(firstPoint, secondPoint, ySpline[firstIndex], ySpline[pointIndex], yIndex);
                     controlPoint1 = data["controlPoint1"];
                     controlPoint2 = data["controlPoint2"];
                     var pt1 = ej.EjSvgRender.utils._getPoint(firstPoint, series);
                     var pt2 = ej.EjSvgRender.utils._getPoint(secondPoint, series);
                     var bpt1 = ej.EjSvgRender.utils._getPoint(controlPoint1, series);
                     var bpt2 = ej.EjSvgRender.utils._getPoint(controlPoint2, series);
                     if (start) {
                         splinesb.append("M" + " " + (pt1.X) + " " + (startLoc.Y) + " " + "L" + " " + (pt1.X) + " " + ((pt1.Y)) + " ");
                         start = false;
                     }
                     splinesb.append("C" + " " + (bpt1.X) + " " + (bpt1.Y) + " " + (bpt2.X) + " " + (bpt2.Y) + " " + (pt2.X) + " " + (pt2.Y) + " ");
                     if (pointIndex == visiblePoints.length - 1 || (pointIndex < visiblePoints.length - 1 && !visiblePoints[i + 1].visible)) {
                         splinesb.append("L" + " " + (pt2.X) + " " + (startLoc.Y));
                         startPoint = null;
                         start = true;
                     }
                 }
                 firstPoint = secondPoint;
                 firstIndex = pointIndex;
             } else {
                 firstPoint = null;
             }
         }

         spDirection = splinesb.toString();

         this.drawAreaPath(series, style, spDirection, translate);

         this.chartObj.svgRenderer.append(this.gSeriesGroupEle, this.chartObj.gSeriesEle);

    
        

     },
     doAnimation: function (series) {

         this._doLineAnimation(series);
     }

 });

 ej.seriesTypes.splinearea = ej.ejSplineAreaSeries;



 ej.ejScatterSeries = ejExtendClass(ej.EjSeriesRender, {

        
    
     draw: function (chart,options) {
         this.chartObj = chart;
         var series = options;
         this._isVisiblePoints(series);
         if (!series.marker.visible)
             series.marker.visible = true;

     },

       doAnimation: function(series) {

           var elements = $(this.chartObj.gSymbolGroupEle).children().not("defs");
           for (var i = 0; i < elements.length; i++) {
               var ubound = 20;
               var lbound = 0;
               var randomValue = Math.floor(Math.random() * (ubound - lbound) + lbound);
               var delayInterval = parseInt(randomValue * 50);
               this.animateSymbol(elements[i], delayInterval, series, this.chartObj.model.requireInvertedAxes);
           }


       }

 });

 ej.seriesTypes.scatter = ej.ejScatterSeries;

 
 ej.ejBubbleSeries = ejExtendClass(ej.EjSeriesRender, {

     createBubbleGroup: function(series) {

         var seriesIndex = $.inArray(series, this.chartObj.model.series);
         var transX = series.xAxis.x;

         var transY = series.yAxis.y;
         var serOptions = { 'id': this.chartObj.svgObject.id + '_SeriesGroup' + '_' + seriesIndex, 'transform': 'translate(' + transX + ',' + transY + ')' };

         this.gSeriesGroupEle = this.chartObj.svgRenderer.createGroup(serOptions);

     },
     draw: function (chart,options) {
         this.chartObj = chart;
         var series = options;    
         series.points = ej.DataManager(series.points, ej.Query().sortBy("X")).executeLocal();
         var seriesIndex = $.inArray(series, this.chartObj.model._visibleSeries); 
         var visiblePoints = this._isVisiblePoints(series);
         var sizeValueIndex = 1;
         var minRadius = (options.bubbleOptions && options.bubbleOptions.minRadius) ? options.bubbleOptions.minRadius :10;
         var maxradius = (options.bubbleOptions && options.bubbleOptions.maxRadius) ? options.bubbleOptions.maxRadius :30;
         var radius = maxradius - minRadius;
         var segmentRadius;
         var maximumSize = Math.max.apply(0,$.map(visiblePoints,function (v) { return v.YValues[sizeValueIndex]; }));
         this.createBubbleGroup(series);
         for (var i = 0; i < visiblePoints.length; i++) {
             var point = visiblePoints[i];
             var pointIndex= i;
             if (point.visible) {
                 segmentRadius = minRadius + radius * Math.abs(point.YValues[sizeValueIndex] / maximumSize);
                 var location = ej.EjSvgRender.utils._getPoint(point, series);
                 
				 var styleOptions = this.chartObj.setStyle(this, series, seriesIndex, pointIndex);
				 
                 var bubbleOptions = {
                     'id': this.chartObj.svgObject.id + '_circlesymbol_' + i + '_' + seriesIndex,
                     'cx': location.X + this.chartObj.canvasX,
                     'cy': (location.Y) + this.chartObj.canvasY,
                     'r': segmentRadius,
                     'fill': styleOptions.interior,
                     'stroke-width': styleOptions.borderWidth,
                     'opacity': styleOptions.opacity,
                     'stroke': styleOptions.borderColor
                 };

                 point.radius = segmentRadius; // radius is stored to draw tracksymbol for bubble in canvas
                 
                 this.chartObj.svgRenderer.drawCircle(bubbleOptions, this.gSeriesGroupEle);
				 
			 

                 var cx = location.X;
                 var cy = location.Y;
                 var r = segmentRadius;


                 var valwidth, x, y, valheight;
                 x = ((cx - r) + series.xAxis.x);
                 y = ((cy - r) + series.yAxis.y);

                 valheight = 2 * r;
                 valwidth = 2 * r;

                 var bounds = { X: x, Y: y, Width: valwidth, Height: valheight };
                 ej.EjSvgRender.utils._addRegion(this.chartObj, bounds, series, null, i);
             }
         }
         this.chartObj.svgRenderer.append(this.gSeriesGroupEle, this.chartObj.gSeriesEle);

         
         
     },
    
     
     doAnimation: function(series) {
         var elements = $(this.gSeriesGroupEle).children().not("defs");
         
         for (var i = 0; i < elements.length; i++) {
             var ubound = 20;
             var lbound = 0;
             var randomValue = Math.floor(Math.random() * (ubound - lbound) + lbound);
             var delayInterval = parseInt(randomValue * 50);
             this.animateSymbol(elements[i], delayInterval, series, this.chartObj.model.requireInvertedAxes);
         }
     }
 });

ej.seriesTypes.bubble = ej.ejBubbleSeries;
 ej.ejhiloSeries = ejExtendClass(ej.EjSeriesRender, {
     draw: function (chart, options) {

         this.chartObj = chart;
         
         //DrawHiloGraph      
        
         var currentseries = options;
          currentseries.points = ej.DataManager(currentseries.points, ej.Query().sortBy("X")).executeLocal();
         var visiblePoints = this._isVisiblePoints(currentseries);
         var sidebysideinfo = this.getSideBySideInfo(currentseries);
         var point = null;
         
         var seriesIndex = $.inArray(currentseries, this.chartObj.model._visibleSeries);
         var transX = currentseries.xAxis.x;
         var transY = currentseries.yAxis.y;
         var serOptions = { 'id': this.chartObj.svgObject.id + '_SeriesGroup' + '_' + seriesIndex, 'transform': 'translate(' + transX + ',' + transY + ')' };
         this.gSeriesGroupEle = this.chartObj.svgRenderer.createGroup(serOptions);
       

         for (var i = 0; i < visiblePoints.length; i++) {
             point = visiblePoints[i];
			 var pointIndex= i;
             if (point.visible) {
                
                 var lDirection = "";
                    if (ej.util.isNullOrUndefined(point.YValues[0]) || ej.util.isNullOrUndefined(point.YValues[1]))
                     continue;
					 
                    var styleOptions = this.chartObj.setStyle(this, currentseries, seriesIndex, pointIndex);

                    var pathInterior = this.chartObj.setHiloStyle(currentseries, pointIndex, seriesIndex);
              
				 
                 var lowvalue = {}, highvalue = {};
                 if (point.YValues[0] < point.YValues[1]) {
                     lowvalue = { X: point.X + sidebysideinfo.Median, YValues: [point.YValues[0]] };
                     highvalue = { X: point.X + sidebysideinfo.Median, YValues: [point.YValues[1]] };
                 } else {
                     lowvalue = { X: point.X + sidebysideinfo.Median, YValues: [point.YValues[1]] };
                     highvalue = { X: point.X + sidebysideinfo.Median, YValues: [point.YValues[0]] };
                 }
                 var point1 = ej.EjSvgRender.utils._getPoint(lowvalue   , currentseries);
                 var point2 = ej.EjSvgRender.utils._getPoint(highvalue, currentseries);
                 lDirection = "M" + " " + (point1.X + this.chartObj.canvasX) + " " + ((point1.Y + this.chartObj.canvasY)) + " " + "L" + " " + (point2.X + this.chartObj.canvasX) + " " + ((point2.Y + this.chartObj.canvasY)) + " ";
                 var pointbounds = { point1:point1, point2:point2 };
                 this._drawHiloPath(currentseries, styleOptions, pathInterior, lDirection, i, pointbounds);
             }
         }
        
         this.chartObj.svgRenderer.append(this.gSeriesGroupEle, this.chartObj.gSeriesEle);
       
         

      
     },
     
     doAnimation: function (series) {

         var elements = $(this.gSeriesGroupEle).children().not("defs");
        var secondsPerPoint = 2000 / elements.length;
        for (var i = 0; i < elements.length; i++) {
            var element = elements[i];
            this._doLineSymbol(element, secondsPerPoint, i, series, this.chartObj.model.requireInvertedAxes);
        }     
     },
     hiloTypes: true
 });

 ej.seriesTypes.hilo = ej.ejhiloSeries;
 
 //Draw HiloOpenClose
 ej.ejhiloopencloseSeries = ejExtendClass(ej.EjSeriesRender, {
     draw: function (chart, options) {

         this.chartObj = chart;
         
         //DrawHiloGraph      

         var currentseries = options;
         currentseries.points = ej.DataManager(currentseries.points, ej.Query().sortBy("X")).executeLocal();
         var visiblePoints = this._isVisiblePoints(currentseries);
         var sidebysideinfo = this.getSideBySideInfo(currentseries);
         var point = null;

         var seriesIndex = $.inArray(currentseries, this.chartObj.model._visibleSeries);
         var transX = currentseries.xAxis.x;
         var transY = currentseries.yAxis.y;
         var serOptions = { 'id': this.chartObj.svgObject.id + '_SeriesGroup' + '_' + seriesIndex, 'transform': 'translate(' + transX + ',' + transY + ')' };
         this.gSeriesGroupEle = this.chartObj.svgRenderer.createGroup(serOptions);
         
         var interior;

     


         for (var i = 0; i < visiblePoints.length; i++) {
		    var pointIndex = i;
             point = visiblePoints[i];
             if (point.visible) {

                 var lDirection = "";
                 var drawOpenWing = (ej.util.isNullOrUndefined(currentseries.drawMode) ? true : (currentseries.drawMode.toLowerCase() == "both" || currentseries.drawMode.toLowerCase() == "open") ? true : false);
                 var drawCloseWing = (ej.util.isNullOrUndefined(currentseries.drawMode) ? true : (currentseries.drawMode.toLowerCase() == "both" || currentseries.drawMode.toLowerCase() == "close") ? true : false);

                
                 var lowvalue = {}, highvalue = {}, openvalue = {}, closevalue = {};
                 if (ej.util.isNullOrUndefined(point.YValues[0]) || ej.util.isNullOrUndefined(point.YValues[1]) || ej.util.isNullOrUndefined(point.YValues[2]) || ej.util.isNullOrUndefined(point.YValues[3]))
                     continue;
                  
                    
                 if (point.YValues[0] < point.YValues[1]) {
                     lowvalue = { X: point.X + sidebysideinfo.Median, YValues: [point.YValues[0]] };
                     highvalue = { X: point.X + sidebysideinfo.Median, YValues: [point.YValues[1]] };
                     openvalue = { X: point.X, YValues: [point.YValues[2]] };
                     closevalue = { X: point.X, YValues: [point.YValues[3]] };
                     
                 } else {
                     lowvalue = { X: point.X + sidebysideinfo.Median, YValues: [point.YValues[1]] };
                     highvalue = { X: point.X + sidebysideinfo.Median, YValues: [point.YValues[0]] };
                     openvalue = { X: point.X, YValues: [point.YValues[2]] };
                     closevalue = { X: point.X, YValues: [point.YValues[3]] };
                    
                 }
                 if (point.YValues[2] < point.YValues[3]) 
                     interior = (ej.util.isNullOrUndefined(currentseries.bearFillColor)) ? ((currentseries.fill) ? currentseries.fill : "#339933") : currentseries.bearFillColor;
                 
                 else 
                     interior = (ej.util.isNullOrUndefined(currentseries.bullFillColor)) ? ((currentseries.fill) ? currentseries.fill : "#E51400") : currentseries.bullFillColor;
                    
				 
                 var styleOptions = this.chartObj.setStyle(this, currentseries, seriesIndex, pointIndex, interior);

                 var pathInterior = this.chartObj.setHiloStyle(currentseries, pointIndex,seriesIndex, interior);
                 

				 if (point.fill) {
                     interior = jQuery.type(point.interior) == "array" ? point.fill[0] : point.fill;
                 }
                 else {
                      point.fill = interior;
                 }
				 var canvasX = this.chartObj.canvasX;
				 var canvasY = this.chartObj.canvasY;
                 
                 //Draw open points
                 if (drawOpenWing) {

                     var startOpenValue = { X: openvalue.X + sidebysideinfo.Median, YValues: openvalue.YValues };
                     var pto1 = ej.EjSvgRender.utils._getPoint(startOpenValue, currentseries);
                     var endOpenValue = { X: openvalue.X + sidebysideinfo.Start, YValues: openvalue.YValues };
                     var pto2 = ej.EjSvgRender.utils._getPoint(endOpenValue, currentseries);
                      lDirection = "M" + " " + (pto1.X + canvasX) + " " + ((pto1.Y + canvasY)) + " " + "L" + " " + (pto2.X + canvasX) + " " + ((pto2.Y + canvasY)) + " ";
                     var openbounds = { point1: pto1, point2: pto2 };
                     this._drawHiloPath(currentseries, styleOptions,pathInterior,  lDirection, i, openbounds);
                     lDirection = "";

                 }
                 //Draw close points
                 if (drawCloseWing) {
                     var startCloseValue = { X: closevalue.X + sidebysideinfo.Median, YValues: closevalue.YValues };
                     var ptc1 = ej.EjSvgRender.utils._getPoint(startCloseValue, currentseries);
                     var endCloseValue = { X: closevalue.X + sidebysideinfo.End, YValues: closevalue.YValues };
                     var ptc2 = ej.EjSvgRender.utils._getPoint(endCloseValue, currentseries);
                     lDirection = "M" + " " + (ptc1.X + canvasX) + " " + ((ptc1.Y + canvasY)) + " " + "L" + " " + (ptc2.X + canvasX) + " " + ((ptc2.Y + canvasY)) + " ";
                     var pointbounds = { point1: ptc1, point2: ptc2 };
                     this._drawHiloPath(currentseries, styleOptions, pathInterior, lDirection, i, pointbounds);
                     lDirection = "";

                 }
                 // Draw High low points
                 var point1 = ej.EjSvgRender.utils._getPoint(lowvalue, currentseries);
                 var point2 = ej.EjSvgRender.utils._getPoint(highvalue, currentseries);
                 lDirection = "M" + " " + (point1.X + canvasX) + " " + ((point1.Y + canvasY)) + " " + "L" + " " + (point2.X + canvasX) + " " + ((point2.Y + canvasY)) + " ";
                 var bounds = { point1: point1, point2: point2 };
                 this._drawHiloPath(currentseries, styleOptions, pathInterior, lDirection, i, bounds);
             }

         }
         this.chartObj.svgRenderer.append(this.gSeriesGroupEle, this.chartObj.gSeriesEle);
     
     },
      
     doAnimation: function (series) {

        var elements = $(this.gSeriesGroupEle).children().not("defs");
        var secondsPerPoint = 2000 / elements.length;
        for (var i = 0; i < elements.length; i++) {
            var element = elements[i];
            this._doLineSymbol(element, secondsPerPoint, i, series, this.chartObj.model.requireInvertedAxes);
        }
     },
     hiloTypes: true

 });

 ej.seriesTypes.hiloopenclose = ej.ejhiloopencloseSeries;
 
 //Draw candle
ej.ejCandleSeries = ejExtendClass(ej.EjSeriesRender, {
     draw: function (chart, options) {

         this.chartObj = chart;
         
         //DrawCandleGraph      

         var currentseries = options;
         currentseries.points = ej.DataManager(currentseries.points, ej.Query().sortBy("X")).executeLocal();
         var visiblePoints = this._isVisiblePoints(currentseries);
         var sidebysideinfo = this.getSideBySideInfo(currentseries);
         var point = null;

         var seriesIndex = $.inArray(currentseries, this.chartObj.model._visibleSeries);
         var transX = currentseries.xAxis.x;
         var transY = currentseries.yAxis.y;
         var serOptions = { 'id': this.chartObj.svgObject.id + '_SeriesGroup' + '_' + seriesIndex, 'transform': 'translate(' + transX + ',' + transY + ')' };
         this.gSeriesGroupEle = this.chartObj.svgRenderer.createGroup(serOptions);
         var interior;

         for (var i = 0; i < visiblePoints.length; i++) {
             point = visiblePoints[i];
			  var pointIndex= i;
             if (point.visible) {

                 var lDirection = "";
               
                 var lowvalue = {}, highvalue = {}, openvalue = {}, closevalue = {};
                 if (ej.util.isNullOrUndefined(point.YValues[0]) || ej.util.isNullOrUndefined(point.YValues[1]) || ej.util.isNullOrUndefined(point.YValues[2]) ||ej.util.isNullOrUndefined(point.YValues[3]))
                     continue;
                
                 if (point.YValues[0] < point.YValues[1]) {
                     lowvalue = { X: point.X + sidebysideinfo.Median, YValues: [point.YValues[0]] };
                     highvalue = { X: point.X + sidebysideinfo.Median, YValues: [point.YValues[1]] };
                     openvalue = { X: point.X, YValues: [point.YValues[2]] };
                     closevalue = { X: point.X, YValues: [point.YValues[3]] };
                     
                 } else {
                     lowvalue = { X: point.X + sidebysideinfo.Median, YValues: [point.YValues[1]] };
                     highvalue = { X: point.X + sidebysideinfo.Median, YValues: [point.YValues[0]] };
                     openvalue = { X: point.X, YValues: [point.YValues[2]] };
                     closevalue = { X: point.X, YValues: [point.YValues[3]] };
                     
                 }      
                 if (point.YValues[2] < point.YValues[3]) {
                     
                     openvalue = { X: point.X, YValues: [point.YValues[2]] };
                     closevalue = { X: point.X, YValues: [point.YValues[3]] };
                     interior = (ej.util.isNullOrUndefined(currentseries.bearFillColor)) ? "#339933" : currentseries.bearFillColor;
                     
                 } else {
                    
                     openvalue = { X: point.X, YValues: [point.YValues[3]] };
                     closevalue = { X: point.X, YValues: [point.YValues[2]] };
                     interior = (ej.util.isNullOrUndefined(currentseries.bullFillColor)) ? "#E51400" : currentseries.bullFillColor;
                    
                 }
                 
                 var styleOptions = this.chartObj.setStyle(this, currentseries, seriesIndex, pointIndex, interior);

                 var pathInterior = this.chartObj.setHiloStyle(currentseries, pointIndex,seriesIndex, interior);
				 
				 if (point.style && point.style.interior) {
                     interior = jQuery.type(point.style.interior) == "array" ? point.style.interior[0] : point.style.interior;
                 }
               
                 else {
                     point.fill = pathInterior;
                 }

          //Draw High low points
                 var point1 = ej.EjSvgRender.utils._getPoint(lowvalue, currentseries);
                 var point2 = ej.EjSvgRender.utils._getPoint(highvalue, currentseries);
                 var canvasX = this.chartObj.canvasX;
                 var canvasY = this.chartObj.canvasY;
                 lDirection = "M" + " " + (point1.X + canvasX) + " " + ((point1.Y + canvasY)) + " " + "L" + " " + (point2.X + canvasX) + " " + ((point2.Y + canvasY)) + " ";
                 var pointbounds = { point1: point1, point2: point2 };
                 this._drawHiloPath(currentseries, styleOptions, pathInterior, lDirection, i, pointbounds);
                 //Draw open and close points
              
                     var startRect = { X: closevalue.X + sidebysideinfo.Start, YValues: closevalue.YValues };
                     var orginRect = ej.EjSvgRender.utils._getPoint(startRect, currentseries);

                     var rectSize = { X: openvalue.X + sidebysideinfo.End, YValues: openvalue.YValues };
                     var sizeRect = ej.EjSvgRender.utils._getPoint(rectSize, currentseries);
                     var rect = ej.EjSvgRender.utils._correctRect(orginRect.X, orginRect.Y, sizeRect.X, sizeRect.Y);
                  
                     var candleOptions = {
                        'id': this.chartObj.svgObject.id + '_Series' + seriesIndex+ '_Point' + i,
                        'x': rect.X + this.chartObj.canvasX,
                        'y': rect.Y + this.chartObj.canvasY,
                        'width': rect.Width,
                        'height': rect.Height,
                        'fill': styleOptions.interior,
                        'stroke-width': styleOptions.borderWidth,
                         'stroke': styleOptions.borderColor
                     };

                    this.chartObj.svgRenderer.drawRect(candleOptions, this.gSeriesGroupEle);

                    var svgXy = ej.EjSvgRender.utils._getSvgXY(rect.X, rect.Y, currentseries, this.chartObj);
                    var bounds = { X: svgXy.X, Y: svgXy.Y, Width: rect.Width, Height: rect.Height };

                    ej.EjSvgRender.utils._addRegion(this.chartObj, bounds, currentseries, point, i);
                                    
             }
             
         }
         this.chartObj.svgRenderer.append(this.gSeriesGroupEle, this.chartObj.gSeriesEle);
        

     },
     
     doAnimation: function (series) {
      var elements = $(this.gSeriesGroupEle).children().not("defs");
        var secondsPerPoint = 2000 / elements.length;
        for (var i = 0; i < elements.length; i++) {
            var element = elements[i];
            this._doLineSymbol(element, secondsPerPoint, i,series, this.chartObj.model.requireInvertedAxes);
        }
        
     },
     hiloTypes: true
 });
 ej.seriesTypes.candle = ej.ejCandleSeries;

 ej.ejPieSeries = ejExtendClass(ej.EjSeriesRender, {

     draw: function (chart, options) {

         this.chartObj = chart;
         var currentseries = options;
         var internalRegion = [];

         this.chartObj.model.bounds = [];
        
         var size = this.calculatingSliceAngle(currentseries);                  

         this.chartObj.model.Radius = 0.5 * currentseries.pieCoefficient * Math.min(size.width, size.height);
        var min= Math.min(size.width, size.height);
        if (min < 0)
        {
            return min;
        }
        var totalDegree=Math.abs(currentseries.endAngle-currentseries.startAngle);
        if (totalDegree < 270) 
            this.pieDoughnutCenter(currentseries);
         var text;
         var seriesIndex = $.inArray(currentseries, this.chartObj.model._visibleSeries);
         var serOptions = { 'id': this.chartObj.svgObject.id + '_SeriesGroup' + '_' + seriesIndex };
         this.gSeriesGroupEle = this.chartObj.svgRenderer.createGroup(serOptions);

         var visibility = (currentseries.enableAnimation) ? 'hidden' : 'visible';

         if (currentseries.marker.dataLabel.visible) {
             var txtOptions = { 'id': this.chartObj.svgObject.id + '_textGroup' + '_' + seriesIndex, 'visibility': visibility };
             this.chartObj.gSeriesTextEle = this.chartObj.svgRenderer.createGroup(txtOptions);
             
             var symbolOptions = { 'id': this.chartObj.svgObject.id + '_symbolGroup' + '_' + seriesIndex, 'visibility': visibility };
             this.chartObj.gSymbolGroupEle = this.chartObj.svgRenderer.createGroup(symbolOptions);

             var connectorOptions = { 'id': this.chartObj.svgObject.id + '_connectorGroup' + '_' + seriesIndex, 'visibility': visibility };
             this.chartObj.gSeriesConnectorEle = this.chartObj.svgRenderer.createGroup(connectorOptions);
             
         }
                         
         for (var j = 0; j < currentseries.points.length; j++) {
             pointIndex = j;
             var point = currentseries.points[j];
             if (point.visible) {
                 var result = this._calculateArcData(this.chartObj, pointIndex, point, currentseries);
                 var sliceXY = result.Direction.split(" "); 
             var styleOptions = this.chartObj.setStyle(this, currentseries, 0,  pointIndex);
			 if (currentseries.startAngle < currentseries.endAngle) // for canvas rendering with modified start and end angle
                     var counterClockWise = false;
                 else
                     counterClockWise = true;
             options = {
                 'id': this.chartObj.svgObject.id + '_Series' + seriesIndex + "path" + pointIndex,
                 'fill': styleOptions.interior,
                 'stroke-width': styleOptions.borderWidth,
                 'stroke': styleOptions.borderColor,
                     'stroke-linecap': currentseries.lineCap,
                     'stroke-linejoin': currentseries.lineJoin,
                     'd': result.Direction,
                     'opacity': styleOptions.opacity,
                     'radius':this.chartObj.model.Radius,
                     'start': point.startAngle - 1.57,
                     'end': point.endAngle - 1.57,
                     'pointIndex': pointIndex,
					 'counterClockWise': counterClockWise,
                     'innerR': 0,
                     'x': sliceXY[12],
                     'y': sliceXY[13]
                 };
                 this.chartObj.svgRenderer.drawPath(options, this.gSeriesGroupEle);

                 region = { PointIndex: pointIndex, StartAngle: point.startAngle, EndAngle: point.endAngle, StartX: this.chartObj.model.startX[pointIndex], StartY: this.chartObj.model.startY[pointIndex] };
                 internalRegion.push(region);

             }
         }

         var seriesData = { Radius: this.chartObj.model.Radius, CenterX: this.chartObj.model.centerX, CenterY: this.chartObj.model.centerY };
         var region = { Series: currentseries, SeriesData: seriesData, Region: internalRegion, SeriesIndex: $.inArray(currentseries, this.chartObj.model._visibleSeries) };
         this.chartObj.model.chartRegions.push(region);

         this.chartObj.svgRenderer.append(this.gSeriesGroupEle, this.chartObj.gSeriesEle);
     },
     doAnimation: function (series) {
         var chartObj = this.chartObj;
         var seriesRender = this;
         var angle = (series.points[0]) ? series.points[0].startAngle : 0;
         $.each(series.visiblePoints, function (pointIndex, point) {
             var startAngle = point.startAngle;
             var endAngle = point.endAngle;
             chartObj.model._radius = chartObj.model.Radius;
             
             point.startAngle = 0;
             point.endAngle =point.startAngle;
             chartObj.model.Radius = 0;
             var piePath = $(chartObj.gSeriesEle).find("#" + seriesRender.gSeriesGroupEle.id)[0].childNodes[pointIndex];
             $(piePath).each(function () { this.StartAngle = angle, this.EndAngle = angle; }).animate(
                 { StartAngle: startAngle, EndAngle: endAngle, pieRadius: chartObj.model._radius },

                 {
                     duration: 1200, queue: false, step: function (now, fx) {

                         if (fx.prop.toString() === "StartAngle") {
                             point.startAngle = now;
                             point.endAngle = point.startAngle;
                         }
                         else if (fx.prop.toString()=="pieRadius") {
                             chartObj.model.Radius = now;
                         } 
                         else { 
                             point.endAngle = now;
                         }
                         var result = seriesRender._calculateArcData(chartObj, pointIndex, point, series);
                         chartObj.svgRenderer._setAttr($(piePath), { "d": result.Direction });
                         chartObj.model.Radius = chartObj.model._radius;                     
                     },
                     complete: function () {
                         if (pointIndex == series.visiblePoints.length - 1) {
                             if (series.marker.dataLabel.visible) {
                                 chartObj.svgRenderer._setAttr($(chartObj.gSeriesEle).find("#" + chartObj.gSeriesTextEle.id), { "visibility": "visible" });
                                 if (chartObj.gSymbolGroupEle)
                                     chartObj.svgRenderer._setAttr($(chartObj.gSeriesEle).find("#" + chartObj.gSymbolGroupEle.id), { "visibility": "visible" });
                                 if (series.marker.dataLabel.template) {
                                     var elements = $(chartObj.element[0].childNodes[0].childNodes);
                                     for (var i = 0; i < elements.length; i++)
                                         $(elements[i]).css('display', 'block');
                                 }
                                 if (chartObj.gSeriesConnectorEle)
                                     chartObj.svgRenderer._setAttr($(chartObj.gSeriesEle).find("#" + chartObj.gSeriesConnectorEle.id), { "visibility": "visible" });
                             }
                             chartObj.model.AnimationComplete = true;
                             var commonEventArgs = $.extend({}, ej.EjSvgRender.commonChartEventArgs);
                             commonEventArgs.data = { series: series };
                             chartObj._trigger("animationComplete", commonEventArgs);
                         }
                     }
                 });
           });
     },
     chartAreaType : "None"

 });

 ej.seriesTypes.pie = ej.ejPieSeries;

 ej.ejDoughnutSeries = ejExtendClass(ej.EjSeriesRender, {
     draw: function (chart, options) {
         this.chartObj = chart;
         var currentseries = options;
         var internalRegion = [];

         this.chartObj.model.bounds = [];

         var size = this.calculatingSliceAngle(currentseries);
        
          this.chartObj.model.Radius = 0.5 * currentseries.doughnutSize * Math.min(size.width, size.height);
         var min = Math.min(size.width, size.height);
         if (min < 0) {
             return min;
         }
         var totalDegree = Math.abs(currentseries.endAngle - currentseries.startAngle);
         if (totalDegree < 270)//for centering the chart
            this.pieDoughnutCenter(currentseries);
         this.chartObj.model.dRadius = currentseries.doughnutCoefficient * this.chartObj.model.Radius;


         var numberToFixed = ej.util.isNullOrUndefined(this.chartObj.model.roundingPlaces) ? 2 : this.chartObj.model.roundingPlaces;

         var seriesIndex = $.inArray(currentseries, this.chartObj.model._visibleSeries);
         var serOptions = { 'id': this.chartObj.svgObject.id + '_SeriesGroup' + '_' + seriesIndex };
         this.gSeriesGroupEle = this.chartObj.svgRenderer.createGroup(serOptions);

         var visibility = (currentseries.enableAnimation) ? 'hidden' : 'visible';

         if (currentseries.marker.dataLabel.visible) {
             var txtOptions = { 'id': this.chartObj.svgObject.id + '_textGroup' + '_' + seriesIndex, 'visibility': visibility };
             this.chartObj.gSeriesTextEle = this.chartObj.svgRenderer.createGroup(txtOptions);

             var symbolOptions = { 'id': this.chartObj.svgObject.id + '_symbolGroup' + '_' + seriesIndex, 'visibility': visibility };
             this.chartObj.gSymbolGroupEle = this.chartObj.svgRenderer.createGroup(symbolOptions);

             var connectorOptions = { 'id': this.chartObj.svgObject.id + '_connectorGroup' + '_' + seriesIndex, 'visibility': visibility };
             this.chartObj.gSeriesConnectorEle = this.chartObj.svgRenderer.createGroup(connectorOptions);

         }
                
         for (var j = 0; j < currentseries.points.length; j++) {
             var point = currentseries.points[j];
             var pointIndex = j;
             if (point.visible) {
                 var explodeX = 0;
                 var explodeY = 0;
                 var result = this._calculateArcData(this.chartObj, pointIndex, point, currentseries);
				 //for canvas slice explode
                 if ((pointIndex == currentseries.explodeIndex || currentseries.explodeAll) && !this.chartObj.vmlRendering) {
                     var chartStartAngle = -.5 * Math.PI; 
                     var startAngle = point.startAngle + chartStartAngle;
                     var endAngle = point.endAngle + chartStartAngle - 0.000001;
                     var midAngle = (startAngle + endAngle) / 2;
                     explodeX = this.chartObj.model.centerX + Math.cos(midAngle) * currentseries.explodeOffset;
                     explodeY = this.chartObj.model.centerY + Math.sin(midAngle) * currentseries.explodeOffset;
                 }

             var styleOptions = this.chartObj.setStyle(this, currentseries, 0, pointIndex);

                 if (currentseries.startAngle < currentseries.endAngle) // for canvas rendering with modified start and end angle
                     var counterClockWise = false;
                 else
                     counterClockWise = true;
                 
             options = {
                 'id': this.chartObj.svgObject.id + '_Series' + seriesIndex + "path" + pointIndex,
                 'fill': styleOptions.interior,
                 'stroke-width': styleOptions.borderWidth,
                 'stroke': styleOptions.borderColor,
                     'stroke-linecap': currentseries.lineCap,
                     'opacity': styleOptions.opacity,
                     'stroke-linejoin': currentseries.lineJoin,
                     'd': result.Direction,
                     'start': point.startAngle - 1.57,
                     'end': point.endAngle - 1.57,
                     'pointIndex': pointIndex,
                     'radius': this.chartObj.model.Radius,
                     'innerR': this.chartObj.model.dRadius,
                     'canvasInnerR': this.chartObj.model.dRadius,
                     'counterClockWise': counterClockWise,
                     'x': explodeX !=0 ? explodeX : this.chartObj.model.centerX,
                     'y': explodeY !=0 ? explodeY : this.chartObj.model.centerY
                 };
                 this.chartObj.svgRenderer.drawPath(options, this.gSeriesGroupEle);


                 region = { PointIndex: j, StartAngle: point.startAngle, EndAngle: point.endAngle, StartX: this.chartObj.model.startX[pointIndex], StartY: this.chartObj.model.startY[pointIndex] };
                 internalRegion.push(region);

             }
         }

         var seriesData = { Radius: this.chartObj.model.Radius, DRadius: this.chartObj.model.dRadius, CenterX: this.chartObj.model.centerX, CenterY: this.chartObj.model.centerY };
         var region = { Series: currentseries, SeriesData: seriesData, Region: internalRegion, SeriesIndex: $.inArray(currentseries, this.chartObj.model._visibleSeries) };
         this.chartObj.model.chartRegions.push(region);

         this.chartObj.svgRenderer.append(this.gSeriesGroupEle, this.chartObj.gSeriesEle);

     },

    
     doAnimation: function (series) {
         var chartObj = this.chartObj;
         var seriesRender = this;
         var angle = (series.points[0]) ? series.points[0].startAngle : 0;
         $.each(series.visiblePoints, function (pointIndex, point) {
             var startAngle = point.startAngle;
             var endAngle = point.endAngle;

             point.startAngle = 0;
             point.endAngle = point.startAngle;           
         var doughPath = $(chartObj.gSeriesEle).find("#" + seriesRender.gSeriesGroupEle.id)[0].childNodes[pointIndex];
             $(doughPath).each(function () { this.StartAngle = angle, this.EndAngle = angle; }).animate(

             { StartAngle: startAngle, EndAngle: endAngle },

             {
             duration: 1200, queue: false, step: function (now, fx) {
                 if (fx.prop.toString() === "StartAngle") {
                     point.startAngle = now;
                     point.endAngle = point.startAngle;
                 }
                 else                     
                     point.endAngle = now;
                       
                 var result = seriesRender._calculateArcData(chartObj, pointIndex, point, series);
                 chartObj.svgRenderer._setAttr($(doughPath), { "d": result.Direction });
             },
             complete: function () {
                 if (pointIndex == series.visiblePoints.length - 1) {
                     if (series.marker.dataLabel.visible) {
                         chartObj.svgRenderer._setAttr($(chartObj.gSeriesEle).find("#" + chartObj.gSeriesTextEle.id), { "visibility": "visible" });
                         if (chartObj.gSymbolGroupEle)
                             chartObj.svgRenderer._setAttr($(chartObj.gSeriesEle).find("#" + chartObj.gSymbolGroupEle.id), { "visibility": "visible" });
                         if (series.marker.dataLabel.template) {
                             var elements = $(chartObj.element[0].childNodes[0].childNodes);
                             for (var i = 0; i < elements.length; i++)
                                 $(elements[i]).css('display', 'block');
                         }
                         if (chartObj.gSeriesConnectorEle)
                             chartObj.svgRenderer._setAttr($(chartObj.gSeriesEle).find("#" + chartObj.gSeriesConnectorEle.id), { "visibility": "visible" });
                     }
                     chartObj.model.AnimationComplete = true;
                     var commonEventArgs = $.extend({}, ej.EjSvgRender.commonChartEventArgs);
                     commonEventArgs.data = { series: series };
                     chartObj._trigger("animationComplete", commonEventArgs);
                 }
             }
         });             
         });
     },

     chartAreaType:"None"
 });

 ej.seriesTypes.doughnut = ej.ejDoughnutSeries;

 ej.ejPyramidSeries = ejExtendClass(ej.EjSeriesRender, {
     draw: function (chart, options) {

         this.chartObj = chart;
         var point;
         var seriesRender = this;
         var currentseries = options;
         var visiblePoints = this._pieVisiblePoints(currentseries);
         var internalRegion = [];
         var translate = [];
         this.chartObj.model.chartRegions = [];
         this.chartObj.model.sumofYValues = 0;

         //var explodeValue = 0;
         //var itemCurrentXPos = 0;
         var textSize = 0;
         var text;

         var measureTitle = (this.chartObj.model.title.text) ? ej.EjSvgRender.utils._measureText(this.chartObj.model.title.text, $(this.svgObject).width() - this.chartObj.model.margin.left - this.chartObj.model.margin.right, this.chartObj.model.title.font) : 0;

         var xOffset = (this.chartObj.model.margin.left + this.chartObj.model.elementSpacing + this.chartObj.model.margin.right + this.chartObj.model.elementSpacing + ((this.chartObj.model.legend.position.toLowerCase() === "right" || this.chartObj.model.legend.position.toLowerCase() === "left") ? this.chartObj.model.LegendViewerBounds.Width : 0));
         var yOffset = (((this.chartObj.model.title.text) ? (this.chartObj.model._titleLocation.Y + measureTitle.height) : this.chartObj.model.elementSpacing) + this.chartObj.model.margin.top + this.chartObj.model.elementSpacing + this.chartObj.model.margin.bottom + this.chartObj.model.elementSpacing + ((this.chartObj.model.legend.position.toLowerCase() === "top" || this.chartObj.model.legend.position.toLowerCase() === "bottom") ? this.chartObj.model.LegendViewerBounds.Height : 0));

         if (currentseries.labelPosition.toLowerCase() === 'outside') {
             for (var i = 0; i < visiblePoints.length; i++) {
                 var textWidth = ej.EjSvgRender.utils._measureText((ej.util.isNullOrUndefined(visiblePoints[i].text) ? visiblePoints[i].y : visiblePoints[i].text), null, currentseries.marker.dataLabel.font).width;
                 if (textSize < textWidth)
                     textSize = textWidth;
             }
           this.chartObj.model.textSize = textSize;
         }

         this.chartObj.model.actualWidth = $(chart.svgObject).width() - xOffset - textSize;
         this.chartObj.model.actualHeight = $(chart.svgObject).height() - yOffset;
         var pyrX = this.chartObj.pyrX = ((this.chartObj.model.legend.position.toLowerCase() === "left") ? (this.chartObj.model.LegendViewerBounds.Width) : 0) + this.chartObj.model.elementSpacing + this.chartObj.model.margin.left;
         var pyrY = this.chartObj.pyrY = ((this.chartObj.model.legend.position.toLowerCase() === "top") ? (this.chartObj.model.LegendViewerBounds.Height) : 0) + ((this.chartObj.model.title.text) ? (this.chartObj.model._titleLocation.Y + measureTitle.height) : (this.chartObj.model.margin.top + this.chartObj.model.elementSpacing));
         translate[0] = pyrX;
         translate[1] = pyrY;
         for (var j = 0; j < visiblePoints.length; j++) {
             this.chartObj.model.sumofYValues += visiblePoints[j].YValues[0];
         }


         var numberToFixed = ej.util.isNullOrUndefined(this.chartObj.model.roundingPlaces) ? 2 : this.chartObj.model.roundingPlaces;

         var seriesIndex = $.inArray(currentseries,this.chartObj.model._visibleSeries);
         var serOptions = { 'id': this.chartObj.svgObject.id + '_SeriesGroup' + '_' + seriesIndex, 'transform': 'translate(' + pyrX + ',' + pyrY + ')' };
         this.gSeriesGroupEle = this.chartObj.svgRenderer.createGroup(serOptions);

         if (currentseries.marker.dataLabel.visible) {
             var txtOptions = { 'id': this.chartObj.svgObject.id + '_textGroup' + '_' + seriesIndex, 'transform': 'translate(' + pyrX + ',' + pyrY + ')' };
             this.chartObj.gSeriesTextEle =  this.chartObj.svgRenderer.createGroup(txtOptions);
             if (currentseries.labelPosition.toLowerCase() === 'outside') {

                 var symbolOptions = { 'id': this.chartObj.svgObject.id + '_symbolGroup' + '_' + seriesIndex, 'transform': 'translate(' + pyrX + ',' + pyrY + ')' };
                 this.chartObj.gSymbolGroupEle = this.chartObj.svgRenderer.createGroup(symbolOptions);

                 var connectorOptions = { 'id': this.chartObj.svgObject.id + '_connectorGroup' + '_' + seriesIndex, 'transform': 'translate(' + pyrX + ',' + pyrY + ')' };
                 this.chartObj.gSeriesConnectorEle =  this.chartObj.svgRenderer.createGroup(connectorOptions);
             }
         }

         if (currentseries.pyramidMode.toLowerCase() == "linear")
             this.calculateLinearSegments(currentseries);
         else
             this.calculateSurfaceSegments(currentseries);

         for (j = 0; j < currentseries.points.length; j++) {
             var pointIndex = j;
             point = currentseries.points[j];
             if (point.visible) {
                 var result = this._getPyramidData(currentseries, pointIndex);

             var styleOptions = this.chartObj.setStyle(this, currentseries, 0,  pointIndex);
			 
             options = {
                 'id': this.chartObj.svgObject.id + '_Series' + "path" + pointIndex,
                 'fill': styleOptions.interior,
                 'stroke-width': styleOptions.borderWidth,
                 'stroke': styleOptions.borderColor,
                     'stroke-linecap': currentseries.lineCap,
                     'stroke-linejoin': currentseries.lineJoin,
                     'opacity': styleOptions.opacity,
                     'pointIndex': pointIndex,
                     'd': result.Direction
                 };
                 this.chartObj.svgRenderer.drawPath(options, this.gSeriesGroupEle, translate);

                 if (currentseries.marker.dataLabel.template) {
                     ej.EjSvgRender.utils._getSeriesTemplateSize(point, pointIndex, currentseries, true, this.chartObj);
                 }

                 point.xLocation = result.PositionX;
                 point.yLocation = result.PositionY;
                 point.connectorLine = result.Connector;

                 region = { PointIndex: pointIndex,Line1:result.Line1,Line2:result.Line2,Line3:result.Line3,Line4:result.Line4 };
                 internalRegion.push(region);
             }
         }
        var region = { Series: currentseries, Region: internalRegion, SeriesIndex: $.inArray(currentseries, this.chartObj.model._visibleSeries) };
         this.chartObj.model.chartRegions.push(region);

         if (this.gSeriesGroupEle) {
             $(this.gSeriesGroupEle.childNodes[pointIndex]).bind('mousemove', function (evt) {
                 evt.target = seriesRender.gSeriesGroupEle.childNodes[pointIndex];
                 seriesRender.chartObj.chartInteractiveBehavior(evt);
             });

             this.chartObj.svgRenderer.append(this.gSeriesGroupEle, this.chartObj.gSeriesEle);
         }

        

     },



     doAnimation: function (series) {
      
     },
     _getPyramidData: function (series, index) {
        
         var model = this.chartObj.model;
         var offset = ((series.explodeIndex == index) || (series.explodeAll) ? series.explodeOffset : 0);

         var top = model.pyramidData[index].CurrY;
         var bottom = model.pyramidData[index].CurrY + model.pyramidData[index].Height;
         var topRadius = 0.5 * (1 - model.pyramidData[index].CurrY);
         var bottomRadius = 0.5 * (1 - bottom);

         var line1 = { x: offset + topRadius * model.actualWidth, y: top * model.actualHeight};
         var line2 = { x: offset + (1 - topRadius) * model.actualWidth, y: top * model.actualHeight};
         var line3 = { x: offset + (1 - bottomRadius) * model.actualWidth, y: bottom * model.actualHeight};
         var line4 = { x: offset + bottomRadius * model.actualWidth, y: bottom * model.actualHeight};

         var direction = "M" + " " + (line1.x) + " " + (line1.y) + " " + "L" + " " + (line2.x) + " " + (line2.y) + " " + "L" + " " + (line3.x) + " " + (line3.y) + " " + "L" + " " + (line4.x) + " " + (line4.y) + " " + "z";

         if (series.marker.dataLabel.visible) {
             var positionX;
             var positionY;
           
             if (series.labelPosition.toLowerCase() === 'outside') {

                 positionX = (line2.x + line3.x) / 2;
                 positionY = (line2.y + line3.y) / 2;

                 var pyrX = ((this.chartObj.model.legend.position.toLowerCase() == "left" || this.chartObj.model.legend.position.toLowerCase() == "right") ? (this.chartObj.model.LegendViewerBounds.Width) : 0) + this.chartObj.model.elementSpacing + this.chartObj.model.margin.left;

                 if (ej.util.isNullOrUndefined(series.marker.dataLabel.connectorLine.height)) {
                     var textOffset = $(this.chartObj.svgObject).width() - (pyrX + positionX + this.chartObj.model.textSize + this.chartObj.model.elementSpacing);
                 }
                 else
                     textOffset = series.marker.dataLabel.connectorLine.height;
                                                
                 var connectorX = this.getXCordinate(positionX, textOffset, 0);
                 var connectorY = this.getYCordinate(positionY, textOffset, 0);
                
                 var connectorDirection = "M" + " " + positionX + " " + positionY + " " + "L" + " " + connectorX + " " + connectorY;

                 positionX = connectorX;
                 positionY = connectorY ;

                 
             }
             else {
                 positionX = (line1.x + line2.x) / 2;
                 positionY = (line3.y + line1.y) / 2;
             }
         }
         return { Direction: direction, PositionX: positionX, PositionY: positionY, Connector: connectorDirection, Line1: line1, Line2: line2,Line3: line3,Line4: line4 };
     },

     calculateLinearSegments: function (series) {
         var currY = 0;
         var model = this.chartObj.model;
         model.pyramidData = [];
         if (series.gapRatio > 1)
         {
             series.gapRatio = 0;
         }
         var coef = 1 / (model.sumofYValues * (1 + series.gapRatio / (1 - series.gapRatio)));
         var count = series.points.length;
         for (var i = 0; i < series.points.length; i++) {
             if (series.points[i].visible) {
                 var height = coef * series.points[i].YValues[0];
                 model.pyramidData[i] = { CurrY: currY, Height: height };
                 currY += (series.gapRatio / (count - 1)) + height;
             }
         }
     },

     calculateSurfaceSegments: function (series) {
         var count = series.points.length;
         var model = this.chartObj.model;
         model.pyramidData = [];
         var currY = 0;
         var y = [];
         var height = [];
         var gapHeight = series.gapRatio / (count - 1);
         var preSum = this.getSurfaceHeight(0, model.sumofYValues);
         for (i = 0; i < count; i++) {
             if (series.points[i].visible) {
                 y[i] = currY;
                 height[i] = this.getSurfaceHeight(currY, Math.abs(series.points[i].YValues[0]));
                 currY += height[i] + gapHeight * preSum;
             }
         }
         var coef = 1 / (currY - gapHeight * preSum);
         for (var i = 0; i < count; i++) {
             if (series.points[i].visible) {
                 var currHeight = coef * y[i];
                 model.pyramidData[i] = { CurrY: currHeight, Height: coef * height[i] };
             }
         }
     },

     getSurfaceHeight: function (y, surface) {
         var result = this.solveQuadraticEquation(1, 2 * y, -surface);
         if (result) {
             return result;
         }
         return 0;
     },
     solveQuadraticEquation: function (a, b, c) {
         var root1;
         var root2;

         if (a != 0) {
             var d = b * b - 4 * a * c;

             if (d >= 0) {
                 var sd = Math.sqrt(d);

                 root1 = (-b - sd) / (2 * a);
                 root2 = (-b + sd) / (2 * a);
                 return Math.max(root1, root2);

             }
         }
         else if (b != 0) {
             root1 = -c / b;
             root2 = -c / b;
             return Math.max(root1, root2);
         }

         return false;
     },

     chartAreaType: "None"
 });

 ej.seriesTypes.pyramid = ej.ejPyramidSeries;
   
 ej.ejFunnelSeries = ejExtendClass(ej.EjSeriesRender, {
     draw: function (chart, options) {        
         this.chartObj = chart;
         var point;
         var seriesRender = this;
         var currentseries = options;
         var visiblePoints = this._pieVisiblePoints(currentseries);
         var internalRegion = [];

         this.chartObj.model.chartRegions = [];
         this.chartObj.model.sumofYValues = 0;
         var translate = [];
         var textSize = 0;
         var text;

         var measureTitle = (this.chartObj.model.title.text) ? ej.EjSvgRender.utils._measureText(this.chartObj.model.title.text, $(this.svgObject).width() - this.chartObj.model.margin.left - this.chartObj.model.margin.right, this.chartObj.model.title.font) : 0;

         var xOffset = (this.chartObj.model.margin.left + this.chartObj.model.elementSpacing + this.chartObj.model.margin.right + this.chartObj.model.elementSpacing + ((this.chartObj.model.legend.position.toLowerCase() === "right" || this.chartObj.model.legend.position.toLowerCase() === "left") ? this.chartObj.model.LegendViewerBounds.Width : 0));
         var yOffset = (((this.chartObj.model.title.text) ? (this.chartObj.model._titleLocation.Y + measureTitle.height) : this.chartObj.model.elementSpacing) + this.chartObj.model.margin.top + this.chartObj.model.elementSpacing + this.chartObj.model.margin.bottom + this.chartObj.model.elementSpacing + ((this.chartObj.model.legend.position.toLowerCase() === "top" || this.chartObj.model.legend.position.toLowerCase() === "bottom") ? this.chartObj.model.LegendViewerBounds.Height : 0));

         if (currentseries.labelPosition.toLowerCase() === 'outside') {
             for (var i = 0; i < visiblePoints.length; i++) {
                 var textWidth = ej.EjSvgRender.utils._measureText((ej.util.isNullOrUndefined(visiblePoints[i].text) ? visiblePoints[i].y : visiblePoints[i].text), null, currentseries.marker.dataLabel.font).width;
                 if (textSize < textWidth)
                     textSize = textWidth;
             }
             this.chartObj.model.textSize = textSize;
         }

         this.chartObj.model.actualWidth = $(chart.svgObject).width() - xOffset - textSize;
         this.chartObj.model.actualHeight = $(chart.svgObject).height() - yOffset;
         var pyrX = this.chartObj.pyrX = ((this.chartObj.model.legend.position.toLowerCase() === "left") ? (this.chartObj.model.LegendViewerBounds.Width) : 0) + this.chartObj.model.elementSpacing + this.chartObj.model.margin.left;
         var pyrY = this.chartObj.pyrY = ((this.chartObj.model.legend.position.toLowerCase() === "top") ? (this.chartObj.model.LegendViewerBounds.Height) : 0) + ((this.chartObj.model.title.text) ? (this.chartObj.model._titleLocation.Y + measureTitle.height) : (this.chartObj.model.margin.top + this.chartObj.model.elementSpacing));
         translate[0] = pyrX;
         translate[1] = pyrY;
         for (var j = 0; j < visiblePoints.length; j++) {
             this.chartObj.model.sumofYValues += visiblePoints[j].YValues[0];
         }


         var numberToFixed = ej.util.isNullOrUndefined(this.chartObj.model.roundingPlaces) ? 2 : this.chartObj.model.roundingPlaces;

         var seriesIndex = $.inArray(currentseries, this.chartObj.model._visibleSeries);
         var serOptions = { 'id': this.chartObj.svgObject.id + '_SeriesGroup' + '_' + seriesIndex, 'transform': 'translate(' + pyrX + ',' + pyrY + ')' };
         this.gSeriesGroupEle = this.chartObj.svgRenderer.createGroup(serOptions);

         if (currentseries.marker.dataLabel.visible) {
             var txtOptions = { 'id': this.chartObj.svgObject.id + '_textGroup' + '_' + seriesIndex, 'transform': 'translate(' + pyrX + ',' + pyrY + ')' };
             this.chartObj.gSeriesTextEle = this.chartObj.svgRenderer.createGroup(txtOptions);
             if (currentseries.labelPosition.toLowerCase() === 'outside') {

                 var symbolOptions = { 'id': this.chartObj.svgObject.id + '_symbolGroup' + '_' + seriesIndex, 'transform': 'translate(' + pyrX + ',' + pyrY + ')' };
                 this.chartObj.gSymbolGroupEle = this.chartObj.svgRenderer.createGroup(symbolOptions);

                 var connectorOptions = { 'id': this.chartObj.svgObject.id + '_connectorGroup' + '_' + seriesIndex, 'transform': 'translate(' + pyrX + ',' + pyrY + ')' };
                 this.chartObj.gSeriesConnectorEle = this.chartObj.svgRenderer.createGroup(connectorOptions);
             }
         }
         var currY = 0; 
         var model = this.chartObj.model;
         model.funnelData = [];
         if (currentseries.gapRatio > 1) {
             currentseries.gapRatio = 0;
         }
         var coef = 1 / (model.sumofYValues * (1 + currentseries.gapRatio / (1 - currentseries.gapRatio)));
         var count = currentseries.points.length;
         var spacing = currentseries.gapRatio / count;
         for (var i = count - 1; i >= 0; i--) {
             if (currentseries.points[i].visible) {
                 var height = coef * currentseries.points[i].YValues[0];
                 model.funnelData[i] = { CurrY: currY, Height: height };
                 currY += height + spacing;
             }
         }
         for (j = 0; j <count  ; j++) {
             var pointIndex = j;
             point = currentseries.points[j];
             if (point.visible) {
                 var result = this._getFunnelData(currentseries, pointIndex,chart);

                 var styleOptions = this.chartObj.setStyle(this, currentseries, 0, pointIndex);

                 options = {
                     'id': this.chartObj.svgObject.id + '_Series' + "path" + pointIndex,
                     'fill': styleOptions.interior,
                     'stroke-width': styleOptions.borderWidth,
                     'stroke': styleOptions.borderColor,
                     'stroke-linecap': currentseries.lineCap,
                     'stroke-linejoin': currentseries.lineJoin,
                     'opacity': styleOptions.opacity,
                     'pointIndex': pointIndex,
                     'd': result.Direction
                 };
                 this.chartObj.svgRenderer.drawPath(options, this.gSeriesGroupEle, translate);

                 if (currentseries.marker.dataLabel.template) {
                     ej.EjSvgRender.utils._getSeriesTemplateSize(point, pointIndex, currentseries, true, this.chartObj);
                 }

                 point.xLocation = result.PositionX;
                 point.yLocation = result.PositionY;
                 point.connectorLine = result.Connector;

                 region = { PointIndex: pointIndex, Line1: result.Line1, Line2: result.Line2, Line3: result.Line3, Line4: result.Line4 };
                 internalRegion.push(region);
             }
         }
         var region = { Series: currentseries, Region: internalRegion, SeriesIndex: $.inArray(currentseries, this.chartObj.model._visibleSeries) };
         this.chartObj.model.chartRegions.push(region);

         if(this.gSeriesGroupEle)
         $(this.gSeriesGroupEle.childNodes[pointIndex]).bind('mousemove', function (evt) {
             evt.target = seriesRender.gSeriesGroupEle.childNodes[pointIndex];
             seriesRender.chartObj.chartInteractiveBehavior(evt);
         });

         this.chartObj.svgRenderer.append(this.gSeriesGroupEle, this.chartObj.gSeriesEle);


     },



     doAnimation: function (series) {

     },
     _getFunnelData: function (series, index, chart) {

         var model = this.chartObj.model;
         var offset = ((series.explodeIndex == index) || (series.explodeAll) ? series.explodeOffset : 0);
         var topRadius, minRadius, bottomRadius, endTop, endMin, endBottom, top, bottomY, bottom;
         var line1, line2, line3, line4, line5, line6;
         var lineWidth;
         var minimumHeight;
         if (series.funnelHeight.indexOf("%") != -1)
             minimumHeight = model.actualHeight * (parseInt(series.funnelHeight) / 100);
         else
             minimumHeight = parseInt(series.funnelHeight);
         var minimumWidth;
         if (series.funnelWidth.indexOf("%") != -1)
             minimumWidth = model.actualWidth * (parseInt(series.funnelWidth) / 100);
         else
             minimumWidth = parseInt(series.funnelWidth);

         top =  model.funnelData[index].CurrY * model.actualHeight;
         bottom = top + model.funnelData[index].Height*model.actualHeight;
         lineWidth = minimumWidth + (model.actualWidth - minimumWidth) * ((model.actualHeight - minimumHeight - top) / (model.actualHeight - minimumHeight));
         topRadius = (model.actualWidth / 2) - lineWidth / 2;
         endTop = topRadius + lineWidth;
         if (bottom > model.actualHeight - minimumHeight || model.actualHeight == minimumHeight)
             lineWidth= minimumWidth;
         else
             lineWidth= minimumWidth + (model.actualWidth - minimumWidth) * ((model.actualHeight - minimumHeight - bottom) / (model.actualHeight - minimumHeight));
         bottomRadius = (model.actualWidth / 2) - (lineWidth / 2);
         endBottom = bottomRadius + lineWidth;
         if (top >= model.actualHeight - minimumHeight) {
             topRadius = bottomRadius = minRadius = (model.actualWidth / 2) - minimumWidth / 2;
             endTop = endBottom = endMin = (model.actualWidth / 2) + minimumWidth / 2;

         } else if (bottom > (model.actualHeight - minimumHeight)) {
             minRadius = bottomRadius = (model.actualWidth / 2) - lineWidth / 2;
             endMin = endBottom = minRadius + lineWidth;

             bottomY = model.actualHeight - minimumHeight;
         }

         line1 = { x: offset + topRadius, y: top };
         line2 = { x: offset + endTop, y: top };
         line4 = { x: offset + endBottom, y: bottom };
         line5 = { x: offset + bottomRadius, y: bottom };
         line3 = { x: offset + endBottom, y: bottom };
         line6 = { x: offset + bottomRadius, y: bottom };
         if (bottomY) {
             line3 = { x: offset + endMin, y: bottomY };
             line6 = { x: offset + minRadius, y: bottomY };
         }
         var direction = "M" + " " + (line1.x) + " " + (line1.y) + " " + "L" + " " + (line2.x) + " " + (line2.y) + " " + "L" + " " + (line3.x) + " " + (line3.y) + " " + "L" + " " + (line4.x) + " " + (line4.y) + " " + "L" + " " + (line5.x) + " " + (line5.y) + " " + "L" + " " + (line6.x) + " " + (line6.y) + " " + "z";
         if (series.marker.dataLabel.visible) {
             var positionX;
             var positionY;

             if (series.labelPosition.toLowerCase() === 'outside') {

                 positionX = (line1.x + line2.x) / 2;
                 positionY = (line1.y + line4.y) / 2;

                 var pyrX = ((this.chartObj.model.legend.position.toLowerCase() == "left" || this.chartObj.model.legend.position.toLowerCase() == "right") ? (this.chartObj.model.LegendViewerBounds.Width) : 0) + this.chartObj.model.elementSpacing + this.chartObj.model.margin.left;

                 if (ej.util.isNullOrUndefined(series.marker.dataLabel.connectorLine.height)) {
                     var textOffset = $(this.chartObj.svgObject).width() - (pyrX + positionX + this.chartObj.model.textSize + this.chartObj.model.elementSpacing);
                 }
                 else
                     textOffset = series.marker.dataLabel.connectorLine.height;

                 var connectorX = this.getXCordinate(positionX, textOffset, 0);
                 var connectorY = this.getYCordinate(positionY, textOffset, 0);

                 var connectorDirection = "M" + " " + positionX + " " + positionY + " " + "L" + " " + connectorX + " " + connectorY;

                 positionX = connectorX;
                 positionY = connectorY;


             }
             else {
                 positionX = (line1.x + line2.x) / 2;
                 positionY = (line1.y + line4.y) / 2;
             }
         }
         return { Direction: direction, PositionX: positionX, PositionY: positionY, Connector: connectorDirection, Line1: line1, Line2: line2, Line3: line3, Line4: line4 };
     },

     
     chartAreaType: "None"
 });

 ej.seriesTypes.funnel = ej.ejFunnelSeries;
    /*Funnel series*/

 ej.ejPolarSeries = ejExtendClass(ej.EjSeriesRender, {

     draw: function (chart, options) {

         this.chartObj = chart;
         var point;
         var seriesRender = this;
         var currentseries = options;
         var visiblePoints = this._pieVisiblePoints(currentseries);
         var internalRegion = [];
        

         var style = this.setLineSeriesStyle(currentseries);
        
         if (currentseries.drawType.toLowerCase() == "column") {
             this.chartObj.model.sumofYValues = 0;
             var count = currentseries.xAxis.visibleRange.interval;
             var min =  currentseries.xAxis.visibleRange.min
             do{
                 this.chartObj.model.sumofYValues += currentseries.xAxis.visibleRange.interval;
                 min += currentseries.xAxis.visibleRange.interval;
             } while (min <= currentseries.xAxis.visibleRange.max)
           //  this.chartObj.model.sumofYValues = visiblePoints.length;
         }

         var direction = this._calculatePolarAxesSegment(currentseries);

         if (currentseries.drawType.toLowerCase() == "area")
             this.drawAreaPath(currentseries, style, direction);
         else if (currentseries.drawType.toLowerCase() == "line")
             this._drawLinePath(currentseries, style, direction);
         
         
         
         this.chartObj.svgRenderer.append(this.gSeriesGroupEle, this.chartObj.gSeriesEle);

       
     },
     doAnimation: function (series) {
             var chartObj = this.chartObj;
             series.regionAdded = true;
             chartObj.model._radius = chartObj.model.Radius;
             chartObj.model.Radius = 0;
             var seriesRender = this;
             var seriesIndex = $.inArray(series, chartObj.model._visibleSeries);
             var Path = $(chartObj.gSeriesEle).find("#" + seriesRender.gSeriesGroupEle.id)[0].childNodes[0];
             $(Path).animate(
                 {  Radius: chartObj.model._radius },

                 {
                     duration: 1200, queue: false, step: function (now, fx) {  
                          
                       chartObj.model.Radius = now;
                                                 
                       var direction = seriesRender._calculatePolarAxesSegment(series);
                       if(series.drawType.toLowerCase()!='column')
                       chartObj.svgRenderer._setAttr($(Path), { "d": direction });
                       chartObj.model.Radius = chartObj.model._radius;
                     },
                     complete: function () {
                         series.regionAdded = false;
                         if (series.marker.dataLabel.visible)
                             chartObj.svgRenderer._setAttr($(chartObj.gSeriesEle).find("#" + chartObj.svgObject.id + '_TextGroup' + '_' + seriesIndex), { "visibility": "visible" });
                         if (series.marker.visible)
                             chartObj.svgRenderer._setAttr($(chartObj.gSeriesEle).find("#" + chartObj.svgObject.id + '_symbolGroup' + '_' + seriesIndex), { "visibility": "visible" });

                             chartObj.model.AnimationComplete = true;
                             var commonEventArgs = $.extend({}, ej.EjSvgRender.commonChartEventArgs);
                             commonEventArgs.data = { series: series };
                             chartObj._trigger("animationComplete", commonEventArgs);
                         }
                     
                 });       
     },
     chartAreaType: "PolarAxes"

 });

 ej.seriesTypes.polar = ej.ejPolarSeries;
 ej.ejRadarSeries = ejExtendClass(ej.EjSeriesRender, {

     draw: function (chart, options) {

         this.chartObj = chart;
         var point;
         var seriesRender = this;
         var currentseries = options;
         var visiblePoints = this._pieVisiblePoints(currentseries);
         var internalRegion = [];

         var style = this.setLineSeriesStyle(currentseries);

         if (currentseries.drawType.toLowerCase() == "column") {
             this.chartObj.model.sumofYValues = 0;
             var count = currentseries.xAxis.visibleRange.interval;
             var min = currentseries.xAxis.visibleRange.min
             do {
                 this.chartObj.model.sumofYValues += currentseries.xAxis.visibleRange.interval;
                 min += currentseries.xAxis.visibleRange.interval;
             } while (min <= currentseries.xAxis.visibleRange.max)
             //  this.chartObj.model.sumofYValues = visiblePoints.length;
         }

         var direction = this._calculatePolarAxesSegment(currentseries);

          if (currentseries.drawType.toLowerCase() == "area")
             this.drawAreaPath(currentseries, style, direction);
         else if (currentseries.drawType.toLowerCase() == "line")
             this._drawLinePath(currentseries, style, direction);

         this.chartObj.svgRenderer.append(this.gSeriesGroupEle, this.chartObj.gSeriesEle);

         

     },
     doAnimation: function (series) {
         var chartObj = this.chartObj;
         series.regionAdded = true;
         chartObj.model._radius = chartObj.model.Radius;
         chartObj.model.Radius = 0;
         var seriesRender = this;
         var seriesIndex = $.inArray(series, chartObj.model._visibleSeries);
         var Path = $(chartObj.gSeriesEle).find("#" + seriesRender.gSeriesGroupEle.id)[0].childNodes[0];
         $(Path).animate(
             { Radius: chartObj.model._radius },

             {
                 duration: 1200, queue: false, step: function (now, fx) {

                     chartObj.model.Radius = now;

                     var direction = seriesRender._calculatePolarAxesSegment(series);
                     if (series.drawType.toLowerCase() != 'column')
                         chartObj.svgRenderer._setAttr($(Path), { "d": direction });
                     chartObj.model.Radius = chartObj.model._radius;
                 },
                 complete: function () {
                     series.regionAdded = false;
                     if (series.marker.dataLabel.visible)
                         chartObj.svgRenderer._setAttr($(chartObj.gSeriesEle).find("#" + chartObj.svgObject.id + '_TextGroup' + '_' + seriesIndex), { "visibility": "visible" });
                     if (series.marker.visible)
                         chartObj.svgRenderer._setAttr($(chartObj.gSeriesEle).find("#" + chartObj.svgObject.id + '_symbolGroup' + '_' + seriesIndex), { "visibility": "visible" });

                     chartObj.model.AnimationComplete = true;
                     var commonEventArgs = $.extend({}, ej.EjSvgRender.commonChartEventArgs);
                     commonEventArgs.data = { series: series };
                     chartObj._trigger("animationComplete", commonEventArgs);
                 }

             });
     },
     chartAreaType: "PolarAxes"

 });

 ej.seriesTypes.radar = ej.ejRadarSeries;
})(jQuery);;
ej.ejChart = {};
(function($) {
String.prototype.parseTemplate = function () {
    var str = this;
    $.each(arguments[0], function (dataIndex, dataValue) {
        if (dataValue.count > 0) {
            $.each(dataValue, function (datachildIndex, datachildValue) {
                str = str.replace(new RegExp('\\#' + dataIndex + '.' + datachildIndex + '\\#', 'gm'), datachildValue);
            });
        } else {
            str = str.replace(new RegExp('\\#' + dataIndex + '\\#', 'gm'), dataValue);
        }

    });
    return str;
};


var Gradient = function (colors) {
    this._gradientStop = [];
    for (var j = 0; j < colors.length; j++) {
        this._gradientStop.push(colors[j]);
    }
};



 ej.ejChart = {

    _renderSfChart: function () {

        var svgSupport = (window.SVGSVGElement) ? true : false;
        if (svgSupport) {
            if (this.model.enableCanvasRendering) {
                this.svgRenderer = new ej.EjCanvasRender(this.element);
                this.legendSvgContainer = this.svgRenderer.createLegendCanvas(this.element); //creating new CANVAS element for legend
                this.legendContainer = $("<div></div>").attr('id', "legend_" + this._id).css("height", "0px");
                this.chartContainer = $("<div></div>").css("position", "relative").attr('id', "chartContainer_" + this._id);
                this.svgRenderer.append(this.legendSvgContainer, this.legendContainer);
                $(this.legendContainer).appendTo(this.chartContainer);
                $(this.chartContainer).appendTo(this.element);
                this.svgRenderer.svgObj.height = ej.util.isNullOrUndefined(this.model.size.height) ? "450" : this.model.size.height;
                this.svgRenderer.svgObj.width = ej.util.isNullOrUndefined(this.model.size.width) ? $("#" + this._id).width() : this.model.size.width;
            } else {
                this.svgRenderer = new ej.EjSvgRender(this.element);
                this.legendSvgContainer = this.svgRenderer.createLegendSvg(this.element); //creating new SVG element for legend
                this.legendContainer = $("<div></div>").attr('id', "legend_" + this._id).css("height","0px");
                this.chartContainer = $("<div></div>").css("position", "relative").attr('id', "chartContainer_" + this._id);
                this.svgRenderer.append(this.legendSvgContainer, this.legendContainer);
                $(this.legendContainer).appendTo(this.chartContainer);
                $(this.chartContainer).appendTo(this.element);
				this.legendContainer.find("svg").attr("class", "e-designerhide");
            }

                this._createChart();
                $(this.svgObject).appendTo(this.element);
        }
        else {
            this.model.enableCanvasRendering = false;
            this.svgRenderer = new ej.EjVmlRender(this.element);
            this.legendSvgContainer = this.svgRenderer.createLegendSvg(this.element); //creating new div element for legend
            this.legendContainer = $("<div></div>").attr('id', "legend_" + this._id).css("height", "0px");
            this.chartContainer = $("<div></div>").css("position", "relative").attr('id', "chartContainer_" + this._id);
            this.svgRenderer.append(this.legendSvgContainer, this.legendContainer);
            $(this.legendContainer).appendTo(this.chartContainer);
            $(this.chartContainer).appendTo(this.element);
            this._createChart();
            $(this.svgObject).appendTo(this.chartContainer);
        }
        return true;
    },

 setSvgSize: function (sender) {
            var chartObj = sender;
            var containerHeight = $(chartObj.element).height();
            var height = 450; //set default height if size not specified to chart or container
            var width = 600;
            var containerWidth = $(chartObj.element).width();
            var legendcontainerHeight = $(chartObj.chartContainer).height();
            if (chartObj.model.size.width) {
                var chartWidth = chartObj.model.size.width;
                if (chartWidth.indexOf("%") != -1)
                    width = (containerWidth / 100) * parseInt(chartWidth);
                else
                    width = parseInt(chartWidth);
            }
            else if (containerWidth > 0)
                width = containerWidth;

            $(chartObj.svgObject).width(width);

            if (chartObj.model.size.height) {
                var chartHeight = chartObj.model.size.height;
                if (chartHeight.indexOf("%") != -1)    
                    height = (!this.vmlRendering) ? (containerHeight / 100) * parseInt(chartHeight) : height;
                else
                    height = parseInt(chartHeight);
            }
            else if (containerHeight > 0)
                height = containerHeight;
            else
                $(chartObj.svgObject).css("display", "block"); //Added css to chart due to 5px difference for container and svg (http://jsfiddle.net/Me5Zd/)       
            $(chartObj.svgObject).height(height);
},
    setModelProperties: function () {

        //set size for the svgObject
        this.setSvgSize(this);

        this.svgWidth = $(this.svgObject).width();
        this.svgHeight = $(this.svgObject).height();

        this.chartCross = {
            visible: (this.chartCross)?(this.model.crosshair.visible||this.chartCross.visible):this.model.crosshair.visible,
            mArea: this.model.crosshair.type
        };

        // extend series with commonSeriesOptions
        this.serAnimation = [];
        if (this.model.series) {
            for (var j = 0; j < this.model.series.length; j++) {
                this.model.series[j] = ej.copyObject({}, this.model.commonSeriesOptions, this.model.series[j]);
                this.serAnimation.push((this.model.series[j].enableAnimation === null || this.model.series[j].enableAnimation === undefined) ? this.model.commonSeriesOptions.enableAnimation : this.model.series[j].enableAnimation);
            }
        }
        if (this.model.indicators) {
            for (var k = 0; k < this.model.indicators.length; k++) {
                this.model.indicators[k] = ej.copyObject({}, this.model.indicatorDefaults, this.model.indicators[k]);
            }
        }

    },

    _createChart: function () {
        this.svgObject = this.svgRenderer.svgObj;

        var commonlaodEventArgs = $.extend({}, ej.EjSvgRender.commonChartEventArgs);
        this._trigger("load", commonlaodEventArgs);
        this.setTheme(ej.EjSvgRender.themes, this.model.theme);
        var model = this.model;
  
        if (model.series[0] && model.series[0]._isdesigntime) {
            model.series[0] = {};
            model.series[0].points = [];
            var randomPoints = [];
            for (var i = 1; i <= 10; i++) {
                randomPoints.push({ x: i, y: (Math.floor(Math.random() * (10 - 40)) + 40) });

            }
            model.series[0].points = randomPoints;
        }
		this.vmlRendering = (this.svgRenderer.vmlNamespace) ? true : false;
        if ($(this.element).is(":visible") || $(this.element).css("visibility") != "hidden" || $(this.element).css('display') != 'none') {
            this.bindEvents();
            this.bindTo();
        }
    },
    setTheme: function (jsonObj, themeName) {
        var name = themeName.toLowerCase();
        if ((name.indexOf("azure") >= 0 || name.indexOf("lime") >= 0 || name.indexOf("saffron") >= 0) && (name.indexOf("gradient") == -1) && (name.indexOf("dark") == -1))
            jsonObj[name] = $.extend(true, {}, jsonObj["flatlight"], jsonObj[name]);
        else if ((name.indexOf("azuredark") >= 0 || name.indexOf("limedark") >= 0 || name.indexOf("saffrondark")) >= 0 && (name.indexOf("gradient") == -1))
            jsonObj[name] = $.extend(true, {}, jsonObj["flatdark"], jsonObj[name]);
        else if ((name.indexOf("azure") >= 0 || name.indexOf("lime") >= 0 || name.indexOf("saffron") >= 0) && (name.indexOf("gradient") >= 0) && (name.indexOf("dark") == -1))
            jsonObj[name] = $.extend(true, {}, jsonObj["gradientlight"], jsonObj[name]);
        else if ((name.indexOf("azuredark") >= 0 || name.indexOf("limedark") >= 0 || name.indexOf("saffrondark") >= 0) && (name.indexOf("gradient") >= 0))
            jsonObj[name] = $.extend(true, {}, jsonObj["gradientdark"], jsonObj[name]);
        else if (!(jsonObj[name])) name = "flatlight";

        if (this.model._themeChanged)
            this.model = ej.copyObject({}, this.model, jsonObj[name]);
        else
            this.model = ej.copyObject({}, jsonObj[name], this.model);


    },
    _drawBackInterior: function () {
        if (this.model.backInterior && this.model.backInterior._gradientStop) {
            var options = {
                'id': this.svgObject.id + '_backGradient',
                'x1': '0%',
                'y1': '0%',
                'x2': '0%',
                'y2': $(this.svgObject).height() + '%'
            };
            this.svgRenderer.drawGradient(options, this.model.backInterior._gradientStop);
        }
    },
    


    _drawBackImage: function () {
        var options = {
            'height': $(this.svgObject).height(),
            'width': $(this.svgObject).width(),
            'href': this.model.backGroundImageUrl,
            'x': 0,
            'y': 0,
            'id': this.svgObject.id + '_backImage',
            'visibility': 'visible',
            'preserveAspectRatio': 'none'
        };

        this.svgRenderer.drawImage(options, this.svgObject);

    },
    _getDataType: function (val) {
        var type = typeof (val);
        if (type === "number" || type === "string")
            return type;
        else if (jQuery.type(val) == "date")
            return "date";
        else
            return false;

    },
    addedXYValues: function (excludeDataUpdate) {
        excludeDataUpdate = (excludeDataUpdate) ? excludeDataUpdate : false;
        var type = this.model.series[0].type.toLowerCase();
        if (ej.seriesTypes[type].prototype.chartAreaType == "cartesianAxes") {
            this.model.AreaType = "cartesianaxes";
        }
        else if (ej.seriesTypes[type].prototype.chartAreaType == "None")
            this.model.AreaType = "none";
        else {
            if (type == 'polar')
                this.model.isPolar = true;
            else
                this.model.isPolar = false;

            this.model.AreaType = "polaraxes";
        }
        this.model.requireInvertedAxes = ej.seriesTypes[type].prototype.requireInvertedAxes;
        if (!excludeDataUpdate) {
            for (var i = 0; i < this.model.series.length; i++) {
                var setXValueType = false, isDatetime = false, pointX, currentSeries;

                if (!this.model.series[i].dataSource) {
                    this.model.series[i] = $.extend(true, {}, this.model.commonSeriesOptions, this.model.series[i]);
                    currentSeries = this.model.series[i];
                    if (this.model.primaryXAxis.valueType == "datetime")
                        isDatetime = true;
                    for (k = 0; k < currentSeries.points.length; k++) {
                        currentSeries.points[k].YValues = [];
                        if ((parseFloat(currentSeries.points[k].x)) && (typeof (currentSeries.points[k].x == "string")) && (this.model.primaryXAxis.valueType != "category") && (this.model.AreaType != "none")) {
                            if (!isDatetime) {
                                currentSeries.points[k].X = parseFloat(currentSeries.points[k].x);
                                currentSeries.points[k].x = parseFloat(currentSeries.points[k].x);
                            }
                            else {
                                currentSeries.points[k].X = new Date(currentSeries.points[k].x);
                                currentSeries.points[k].x = new Date(currentSeries.points[k].x);
                                }
                        }
                        else
                            currentSeries.points[k].X = currentSeries.points[k].x;
                        if (!setXValueType) {
                            pointX = currentSeries.points[k].x;
                            if (!ej.util.isNullOrUndefined(pointX)) {
                                currentSeries._xAxisValueType = this._getDataType(pointX);
                                setXValueType = true;
                            }
                        }
                        if (!(currentSeries._hiloTypes)) {
                            currentSeries.points[k].YValues[0] = currentSeries.points[k].y;

                            if (currentSeries.points[k].size)
                                currentSeries.points[k].YValues[1] = currentSeries.points[k].size;

                        }
                        else {
                            if (currentSeries.points[k].high) {
                                currentSeries.points[k].YValues[0] = currentSeries.points[k].high;
                                currentSeries.points[k].y = currentSeries.points[k].high;
                            }
                            if (currentSeries.points[k].low)
                                currentSeries.points[k].YValues[1] = currentSeries.points[k].low;
                            if (currentSeries.points[k].open)
                                currentSeries.points[k].YValues[2] = currentSeries.points[k].open;
                            if (currentSeries.points[k].close)
                                currentSeries.points[k].YValues[3] = currentSeries.points[k].close;
                        }
                    }
                }
                else {
                    currentSeries = this.model.series[i];
                    for (var j = 0, length=currentSeries.points.length; j <length; j++) {
                        pointX = currentSeries.points[j].X= currentSeries.points[j].x;                  
                        if (ej.util.isNullOrUndefined(currentSeries._xAxisValueType)) {
                           if(!ej.util.isNullOrUndefined(pointX))currentSeries._xAxisValueType = this._getDataType(pointX);
                          }
                    }
                }
            }
           this._setInternalValues();
         }
    },
    _setInternalValues: function () {
        
        this.model._visibleSeries = [];
        for (var k = 0; k < this.model.series.length; k++) {
            var series = this.model.series[k];
            if (this.model.AreaType == "cartesianaxes" && ej.seriesTypes[series.type.toLowerCase()].prototype.chartAreaType == "cartesianAxes") {
                series._zOrder = (series.zOrder) ? series.zOrder : 0;
                this.model._visibleSeries.push(series);
            }
            else if (this.model.AreaType == "none" && ej.seriesTypes[series.type.toLowerCase()].prototype.chartAreaType == "None") {
                series._zOrder = (series.zOrder) ? series.zOrder : 0;
                this.model._visibleSeries.push(series);
            }
            else if (this.model.AreaType == "polaraxes" && ej.seriesTypes[series.type.toLowerCase()].prototype.chartAreaType == "PolarAxes") {
                series._zOrder = (series.zOrder) ? series.zOrder : 0;
                this.model._visibleSeries.push(series);
            }

            if (series.yAxisName || series.xAxisName) {
                if (this.model.axes) {
                    for (var t = 0; t < this.model.axes.length; t++) {
                        if (series.yAxisName && (series.yAxisName == this.model.axes[t].name)) {
                            this.model.axes[t].orientation = (this.model.axes[t].orientation) ? this.model.axes[t].orientation : (this.model.requireInvertedAxes) ? "horizontal" : "vertical";
                        }
                        if (series.xAxisName && (series.xAxisName == this.model.axes[t].name)) {
                            this.model.axes[t].orientation = (this.model.axes[t].orientation) ? this.model.axes[t].orientation : (this.model.requireInvertedAxes) ? "vertical" : "horizontal";
                        }
                    }
                }
            }
        }
    },
    setHiloStyle: function(series, pointIndex, seriesIndex, interiorColor){
        var point = series.points[pointIndex];
                  
        if (point.fill)
              interior = point.fill;
         else
              interior = series.fill ? series.fill : interiorColor;
				
            // setting default series color		
            var colors = interior ? interior : this.model.seriesColors[seriesIndex];
          
            // applying gradient color		  
         
            seriesInterior = jQuery.type(colors) == "array" ? colors[0].color : colors;

            return seriesInterior;
                       
    },
	setStyle: function (chart, series, seriesIndex, pointIndex, interiorColor) {
    
        var point = series.points[pointIndex];
       // var pointStyle = point.style ? point.style : " ";
        var interior, borderColor, borderWidth, opacity;
		
        // setting interior
        if (chart.chartObj.model.AreaType == "cartesianaxes") {
            if (point.fill)
                interior = point.fill;
            else
                interior = series.fill ? series.fill : interiorColor;
				
		// setting default series color		
            var colors = interior ? interior : this.model.seriesColors[seriesIndex];
          
        // applying gradient color		  
			if (chart.chartObj.model.requireInvertedAxes) 
                var seriesInterior = this.svgRenderer.createGradientElement(chart.getSeriesName(series) + "_" + seriesIndex + pointIndex, colors, 0, 0, 100, 0, chart.gSeriesGroupEle);
            else                                       
                seriesInterior = this.svgRenderer.createGradientElement(chart.getSeriesName(series) + "_" + seriesIndex + pointIndex, colors, 0, 0, 0, 100, chart.gSeriesGroupEle);
         } 
		 
		// interior for pie type series
        else {
            interior = point.fill ? point.fill : this.model.pointColors[pointIndex];
            var pointInterior = this.svgRenderer.createGradientElement(chart.getSeriesName(series) + pointIndex, interior, 0, 0, 0, $(chart.svgObject).height(), chart.chartObj.gSeriesEle);
        }
        
        // setting borderColor
            borderColor =((point.border) && point.border.color)? point.border.color : series.border.color;
            borderWidth =((point.border) && point.border.width) ? point.border.width : series.border.width;
            opacity = point.opacity ? point.opacity : series.opacity;
            

        var options = {
            'interior': seriesInterior ? seriesInterior: pointInterior,
            'borderColor': borderColor,
            'borderWidth': borderWidth,
            'opacity': opacity
        };
        return options;
    },
	
    colorNameToHex: function (colour) {
        var color = colour;
        var colours = {
            "aliceblue": "#f0f8ff", "antiquewhite": "#faebd7", "aqua": "#00ffff", "aquamarine": "#7fffd4", "azure": "#f0ffff",
            "beige": "#f5f5dc", "bisque": "#ffe4c4", "black": "#000000", "blanchedalmond": "#ffebcd", "blue": "#0000ff", "blueviolet": "#8a2be2", "brown": "#a52a2a", "burlywood": "#deb887",
            "cadetblue": "#5f9ea0", "chartreuse": "#7fff00", "chocolate": "#d2691e", "coral": "#ff7f50", "cornflowerblue": "#6495ed", "cornsilk": "#fff8dc", "crimson": "#dc143c", "cyan": "#00ffff",
            "darkblue": "#00008b", "darkcyan": "#008b8b", "darkgoldenrod": "#b8860b", "darkgray": "#a9a9a9", "darkgreen": "#006400", "darkkhaki": "#bdb76b", "darkmagenta": "#8b008b", "darkolivegreen": "#556b2f",
            "darkorange": "#ff8c00", "darkorchid": "#9932cc", "darkred": "#8b0000", "darksalmon": "#e9967a", "darkseagreen": "#8fbc8f", "darkslateblue": "#483d8b", "darkslategray": "#2f4f4f", "darkturquoise": "#00ced1",
            "darkviolet": "#9400d3", "deeppink": "#ff1493", "deepskyblue": "#00bfff", "dimgray": "#696969", "dodgerblue": "#1e90ff",
            "firebrick": "#b22222", "floralwhite": "#fffaf0", "forestgreen": "#228b22", "fuchsia": "#ff00ff",
            "gainsboro": "#dcdcdc", "ghostwhite": "#f8f8ff", "gold": "#ffd700", "goldenrod": "#daa520", "gray": "#808080", "green": "#008000", "greenyellow": "#adff2f",
            "honeydew": "#f0fff0", "hotpink": "#ff69b4",
            "indianred ": "#cd5c5c", "indigo ": "#4b0082", "ivory": "#fffff0", "khaki": "#f0e68c",
            "lavender": "#e6e6fa", "lavenderblush": "#fff0f5", "lawngreen": "#7cfc00", "lemonchiffon": "#fffacd", "lightblue": "#add8e6", "lightcoral": "#f08080", "lightcyan": "#e0ffff", "lightgoldenrodyellow": "#fafad2",
            "lightgrey": "#d3d3d3", "lightgreen": "#90ee90", "lightpink": "#ffb6c1", "lightsalmon": "#ffa07a", "lightseagreen": "#20b2aa", "lightskyblue": "#87cefa", "lightslategray": "#778899", "lightsteelblue": "#b0c4de",
            "lightyellow": "#ffffe0", "lime": "#00ff00", "limegreen": "#32cd32", "linen": "#faf0e6",
            "magenta": "#ff00ff", "maroon": "#800000", "mediumaquamarine": "#66cdaa", "mediumblue": "#0000cd", "mediumorchid": "#ba55d3", "mediumpurple": "#9370d8", "mediumseagreen": "#3cb371", "mediumslateblue": "#7b68ee",
            "mediumspringgreen": "#00fa9a", "mediumturquoise": "#48d1cc", "mediumvioletred": "#c71585", "midnightblue": "#191970", "mintcream": "#f5fffa", "mistyrose": "#ffe4e1", "moccasin": "#ffe4b5",
            "navajowhite": "#ffdead", "navy": "#000080",
            "oldlace": "#fdf5e6", "olive": "#808000", "olivedrab": "#6b8e23", "orange": "#ffa500", "orangered": "#ff4500", "orchid": "#da70d6",
            "palegoldenrod": "#eee8aa", "palegreen": "#98fb98", "paleturquoise": "#afeeee", "palevioletred": "#d87093", "papayawhip": "#ffefd5", "peachpuff": "#ffdab9", "peru": "#cd853f", "pink": "#ffc0cb", "plum": "#dda0dd", "powderblue": "#b0e0e6", "purple": "#800080",
            "red": "#ff0000", "rosybrown": "#bc8f8f", "royalblue": "#4169e1",
            "saddlebrown": "#8b4513", "salmon": "#fa8072", "sandybrown": "#f4a460", "seagreen": "#2e8b57", "seashell": "#fff5ee", "sienna": "#a0522d", "silver": "#c0c0c0", "skyblue": "#87ceeb", "slateblue": "#6a5acd", "slategray": "#708090", "snow": "#fffafa", "springgreen": "#00ff7f", "steelblue": "#4682b4",
            "tan": "#d2b48c", "teal": "#008080", "thistle": "#d8bfd8", "tomato": "#ff6347", "turquoise": "#40e0d0",
            "violet": "#ee82ee",
            "wheat": "#f5deb3", "white": "#ffffff", "whitesmoke": "#f5f5f5",
            "yellow": "#ffff00", "yellowgreen": "#9acd32"
        };

        if (Object.prototype.toString.call(color) == '[object Array]')
            return color;

        if (typeof colours[color.toLowerCase()] != 'undefined')
            return colours[color.toLowerCase()];

        return color;
    },
	
	// set culture for chart
     _setCulture: function (culture) {
        this.culture = Globalize.culture(culture);
    },
	
	
     draw: function (excludeDataUpdate) {

        this._drawBackInterior();
        this.model.chartRegions = [];
        this.model.stackedValue = {};
        this._setCulture(this.model.locale);

		this.addedXYValues(excludeDataUpdate);
        

        var commonpreRenderEventArgs = $.extend({}, ej.EjSvgRender.commonChartEventArgs);
        commonpreRenderEventArgs.data = {};
        this._trigger("preRender", commonpreRenderEventArgs);

        this.gLegendEle = this.svgRenderer.createGroup({ 'id': this.svgObject.id + '_Legend' });

        this.gTitleEle = this.svgRenderer.createGroup({ 'id': this.svgObject.id + '_ChartTitle' });
        this.gSubTitleEle = this.svgRenderer.createGroup({ 'id': this.svgObject.id + '_ChartsubTitle' });
        this._createAxisLabelAndRange();

        this._initializeSeriesColors();

        for (i = 0; i < this.model.series.length; i++) {
            if (!ej.util.isNullOrUndefined(this.model.series[i].style)) {
                if (!ej.util.isNullOrUndefined(this.model.series[i].style.interior)) {
                    var interior = this.model.series[i].style.interior;
                    interior = this.colorNameToHex(interior);
                    this.model.series[i].style.interior = interior;
                }
            }
        }

        this._calculateLegendBounds();

        this.legendRender = new ej.EjLegendRender(this);

        var commonLegendEventArgs = $.extend({}, ej.EjSvgRender.commonChartEventArgs);
        commonLegendEventArgs.data = { legendBounds: this.model.LegendBounds };
        this._trigger("legendBoundsCalculate", commonLegendEventArgs);

         // Assigning indicator points
        for (var j = 0; j < this.model.indicators.length; j++) {
            var indicator = this.model.indicators[j];
            indicator.xAxisName = (!(indicator.xAxisName)) ? this.model.axes[0].name : indicator.xAxisName;
            indicator.yAxisName = (!(indicator.yAxisName)) ? this.model.axes[1].name : indicator.yAxisName;
            indicator.isIndicator = true;
            if (indicator.dataSource) {
                indicator._hiloTypes = true;
                this._processJsonData(indicator.dataSource, indicator);
            } else if (indicator.points.length > 0) {
                for (var i = 0; i < indicator.points.length; i++) {
                    indicator.points[i].X = indicator.points[i].x;
                    indicator.points[i].YValues = [];
                    indicator.points[i].YValues[0] = indicator.points[i].close;
                }
            }
            else if (indicator.seriesName != "") {
                for (var index = 0; index < this.model._visibleSeries.length; index++) {
                    var series = this.model._visibleSeries[index];
                    if (indicator.seriesName == series.name)
                        indicator.points = $.extend(true, [], series.points);
                }
            }
        }

         // calculate category points
        for (k = 0; k < this.model.axes.length; k++) {
            if (this.model.axes[k].valueType && this.model.axes[k].valueType.toLowerCase() == "category") {
                for (i = 0; i < this.model.indicators.length; i++) {
                    indicator = this.model.indicators[i];
                    if (this.model.axes[k].name == indicator.xAxisName) {
                        for (j = 0; j < indicator.points.length; j++)
                            indicator.points[j].X = j;
                    }
                }
            }
        }

         // calculate ranges for indicator
        this.model.indicatorRange = [];
        for (index = 0; index < this.model.indicators.length; index++) {
            indicator = this.model.indicators[index];
            if (indicator.points.length > 0) {
                type = indicator.type.toLowerCase();
              options = new ej.indicatorTypes[type]();
              options.calculateSegment(indicator, this);
         }
     }


         for (var m = 0; m < this.model._visibleSeries.length; m++) {
            var options = this.model._visibleSeries[m];
            for (i = 0; i < options.points.length; i++) {
                if (typeof options.points[i].X == "string" && options.points[i].X.indexOf("/Date(") != -1)
                    options.points[i].X = new Date(parseInt(options.points[i].X.substr(6)));
            }
        }
        this.model.hAxes = [];
        this.model.vAxes = [];
        var chartobj = this;
        var isStriplineOver = false, isStriplineBehind = false;
        this._arrangeAxis();
        for (var k = 0; k < this.model.axes.length; k++) {
            var axis = this.model.axes[k];
            var seriesCollection = [];
            for (var slCount = 0; slCount < axis.stripLine.length && axis.stripLine[slCount].visible; slCount++) {
                axis.stripLine[slCount] = $.extend(true, {}, this.model.stripLineDefault, axis.stripLine[slCount]);
                if (axis.stripLine[slCount].zIndex.toLowerCase() == 'over') {
                    isStriplineOver = true;
                } else {
                    isStriplineBehind = true;
                }
            }
            var axisOrientation = axis.orientation.toLowerCase();
            var zOrder = this.model._visibleSeries.length - 1;
            for (j = 0; j < this.model._visibleSeries.length; ++j) {
                zOrder -= 1;
                if (this.model._visibleSeries[j].visibility.toLowerCase() == 'visible') {
                    if ((this.model.enableCanvasRendering && this.model._visibleSeries[j].type == "polar" && this.model._visibleSeries[j].isStacking && this.model._visibleSeries[j].drawType == "column"))
                       this.model._visibleSeries[j]._zOrder = zOrder;                    
                    var series = this.model._visibleSeries[j];
                    series.xAxisName = (!(series.xAxisName)) ? chartobj.model.axes[0].name : series.xAxisName;
                    series.yAxisName = (!(series.yAxisName)) ? chartobj.model.axes[1].name : series.yAxisName;
                    var xAxisName = series.xAxisName.toLowerCase();
                    var yAxisName = series.yAxisName.toLowerCase();
                    var name = axis.name.toLowerCase();
                    if (xAxisName || yAxisName) {
                        if (xAxisName == name || yAxisName == name) {
                            if (axisOrientation == "horizontal" && axis.valueType && axis.valueType.toString() == "datetime") {
                                for (var m = 0; m < series.points.length; m++) {
                                    if (typeof series.points[m].X == "string" && !isNaN(Date.parse(series.points[m].X)))
                                        series.points[m].X = new Date(Date.parse(series.points[m].X));
                                }
                            }
                            else {
                                if (series._xAxisValueType == "string" && !(axis.valueType))
                                    axis._valueType = (axisOrientation == 'horizontal' && !this.model.requireInvertedAxes) ? 'category' : (axisOrientation == 'vertical' && this.model.requireInvertedAxes) ? 'category' : 'double';

                                if (series._xAxisValueType == "date" && !(axis.valueType))
                                    axis._valueType = (axisOrientation == 'horizontal' && !this.model.requireInvertedAxes) ? "datetime" : (axisOrientation == 'vertical' && this.model.requireInvertedAxes) ? 'datetime' : 'double';
                            }
                            seriesCollection.push(series);
                        }
                    }
                }
            }
            if (axisOrientation == ((!this.model.requireInvertedAxes) ? "vertical" : "horizontal")) {
                for (var i = 0; i < seriesCollection.length; i++) {
                    var type = seriesCollection[i].type.toLowerCase();
                    if (ej.seriesTypes[type].prototype.stackingSeries || (this.model.AreaType == 'polaraxes' && seriesCollection[i].drawType.toLowerCase() == 'column' && seriesCollection[i].isStacking)) {
                        // calculation for stacking values
                        this._calculateStackingValues(seriesCollection, axis);
                        break;
                    }
                }
            }
            axis._valueType = (!(axis._valueType)) ? (!(axis.valueType) ? "double" : axis.valueType) : axis._valueType;
            var axisRange = new ej.axisTypes[axis._valueType.toLowerCase()]();
            axisRange._calculateRanges(chartobj, axis, seriesCollection);
            var commonAxisEventArgs = $.extend({}, ej.EjSvgRender.commonChartEventArgs);
            commonAxisEventArgs.data = { range: axis.range };
            this._trigger("axesRangeCalculate", commonAxisEventArgs);
        }

        var commonAxesInitEventArgs = $.extend({}, ej.EjSvgRender.commonChartEventArgs);
        commonAxesInitEventArgs.data = { axes: this.model.axes };
        this._trigger("axesLabelsInitialize", commonAxesInitEventArgs);

        this._calculateAreaBounds();

        if (this.model.enable3D) {

            var Ej3DRender = new ej.Ej3DRender();
            this.vector = new Ej3DRender.vector3D();
            this.matrixobj = new Ej3DRender.matrix3D();
            this.bsptreeobj = new Ej3DRender.BSPTreeBuilder();
            this.polygon = new Ej3DRender.polygon3D();
            this.graphics = new Ej3DRender.Graphics3D();

            this.chart3D = this.svgRenderer.createGroup({ 'id': this.svgObject.id + '_Chart3D' });
            ej.Ej3DRender.Polygons = [];
        }


        if (this.model.AreaType != "none") {        

            var commonAreaEventArgs = $.extend({}, ej.EjSvgRender.commonChartEventArgs);
            commonAreaEventArgs.data = { areaBounds: this.model.m_AreaBounds };
            this._trigger("chartAreaBoundsCalculate", commonAreaEventArgs);

            this._drawChartAreaRect();

            if (this.model.title.text != "" && this.model.title.text)
                this._drawTitle();

            this._calculateAxisSize();

            this.gXaxisEle = this.svgRenderer.createGroup({ 'id': this.svgObject.id + '_XAxis' });

            this.gYaxisEle = this.svgRenderer.createGroup({ 'id': this.svgObject.id + '_YAxis' });
     
            this.gPolarAxisEle = this.svgRenderer.createGroup( { 'id': this.svgObject.id + '_AxisLine' });
            if (this.model.enable3D) {

                $.extend(ej.Ej3DAxisRenderer.prototype, this);
                this.chartaxis = new ej.Ej3DAxisRenderer(this);
            }
            else {
                $.extend(ej.EjAxisRenderer.prototype, this);
                this.chartaxis = new ej.EjAxisRenderer(this);

                this.svgRenderer.append(this.gXaxisEle, this.svgObject);

                this.svgRenderer.append(this.gYaxisEle, this.svgObject);
            }

            var chartaxis = this.chartaxis;
            for (var l = 0; l < this.model.axes.length; l++) {
                chartaxis._drawAxes(l, this.model.axes[l]);
            }
          
            if (this.model.enable3D) {
                ej.Ej3DChart.prototype.update3DWall(this);
            }
          

            this.svgRenderer.append(this.gXaxisEle, this.svgObject);

            this.svgRenderer.append(this.gYaxisEle, this.svgObject);

            var x = (this.model.AreaType == 'polaraxes') ? 0: this.model.m_AreaBounds.X;
            var y = (this.model.AreaType == 'polaraxes') ? 0 : this.model.m_AreaBounds.Y;

            var width = (this.model.AreaType == 'polaraxes') ? $(this.svgObject).width() : this.model.m_AreaBounds.Width;
            var height = (this.model.AreaType == 'polaraxes') ? $(this.svgObject).height() : this.model.m_AreaBounds.Height;

            var clipRectOptions = {
                'id': this.svgObject.id + '_ChartAreaClipRect',
                'x': x,
                'y': y,
                'width': width,
                'height': height,
                'fill': 'white',
                'stroke-width': 1,
                'stroke': 'Gray'
            };

            this.svgRenderer.drawClipPath(clipRectOptions, this.svgObject);
			
			this._setZoomProperties();
        }

        else {
            this._drawChartAreaRect();

            clipRectOptions = {
                'id': this.svgObject.id + '_ChartAreaClipRect',
                'x': 0,
                'y': 0,
                'width': $(this.svgObject).width(),
                'height': $(this.svgObject).height(),
                'fill': 'white',
                'stroke-width': 1,
                'stroke': 'Gray'
            };

            this.svgRenderer.drawClipPath(clipRectOptions, this.svgObject);

            if (this.model.title.text != "" && this.model.title.text)
                this._drawTitle();
        }

        if (isStriplineBehind && this.model.AreaType == "cartesianaxes") {
            this.gStriplineBehind = this.svgRenderer.createGroup({ 'id': this.svgObject.id + '_StriplineBehind', 'clip-path': 'url(#' + this.svgObject.id + '_ChartAreaClipRect)' });
            this._renderStripline('behind');
        }

        if (this.model.initSeriesRender)
            this.seriesRender();

        if (this.model.initSeriesRender)
            this.indicatorRender();

        if (this.model.AreaType == "polaraxes") {
            this.chartaxis._drawAxisLine(this.model.axes[1]);
        }

        this.gXaxisTickEle = this.svgRenderer.createGroup({ 'id': this.svgObject.id + '_XAxisTick' });

        this.gYaxisTickEle = this.svgRenderer.createGroup({ 'id': this.svgObject.id + '_YAxisTick' });

        $.extend(ej.EjAxisRenderer.prototype, this);
        this.chartaxis = new ej.EjAxisRenderer(this);
        var chartaxis = this.chartaxis;
      
        for (var l = 0; l < this.model.axes.length; l++)
        {            
           
            if (this.model.axes[l].tickLinesPosition == 'inside' || this.model.axes[l].labelPosition=='inside') {
                if (this.model.axes[l].orientation.toLowerCase() == "horizontal")
                {
                    
                    chartaxis._drawXAxisTickLine(l, this.model.axes[l]);
                    chartaxis._drawXAxisLabels(l, this.model.axes[l]);
                }
                else if (this.model.axes[l].orientation.toLowerCase() == "vertical")
                {
                    chartaxis._drawYAxisTickLine(l, this.model.axes[l]);
                    chartaxis._drawYAxisLabels(l, this.model.axes[l]);
                }
            }                     
        }         
       
                this.svgRenderer.append(this.gXaxisTickEle, this.svgObject);
                this.svgRenderer.append(this.gYaxisTickEle, this.svgObject);
       

        if (isStriplineOver && this.model.AreaType == "cartesianaxes") {
            this.gStriplineOver = this.svgRenderer.createGroup({ 'id': this.svgObject.id + '_StriplineOver', 'clip-path': 'url(#' + this.svgObject.id + '_ChartAreaClipRect)' });
            this._renderStripline('over');
        }

         if (!this.resetZooming && (this.zooming || this.panning))
             this._enableZoomingButtons();
         this.resetZooming = false;
        var commonLoadedEventArgs = $.extend({}, ej.EjSvgRender.commonChartEventArgs);
        commonLoadedEventArgs.data = { model: this.model };
        this._trigger("loaded", commonLoadedEventArgs);


    },
	
	// set value to private variable for two way binding
	_setZoomProperties:function(){
	    for (var l = 0; l < this.model.axes.length; l++) {
			  var axis=this.model.axes[l];
               
		var _zf = axis.zoomFactor.toFixed(3);
        var _zp = axis.zoomPosition.toFixed(3);
        if (axis.name == this.model.primaryXAxis.name && (_zf!= this._xZoomFactor() ||_zp!= this._xZoomPosition() )) {
            this._xZoomFactor(axis.zoomFactor.toFixed(3));
            this._xZoomPosition(axis.zoomPosition.toFixed(3));
        }
        if (axis.name == this.model.primaryYAxis.name && (_zf != this._yZoomFactor() || _zp != this._yZoomPosition())) {
            this._yZoomFactor(axis.zoomFactor.toFixed(3));
            this._yZoomPosition(axis.zoomPosition.toFixed(3));
        }
				
        }
	},

     indicatorRender: function() {
         if ($(this.svgObject).find("#" + this.svgObject.id + "_IndicatorCollection").length > 0) {
             $(this.svgObject).find("#" + this.svgObject.id + "_IndicatorCollection").remove();
         }

         var indOptions = { 'id': this.svgObject.id + '_IndicatorCollection', 'clip-path': 'url(#' + this.svgObject.id + '_ChartAreaClipRect)' };

         this.gIndicatorEle = this.svgRenderer.createGroup(indOptions);

         for (var i = 0; i < this.model.indicators.length && this.model.indicators[i].points.length>0; i++) {
             var indicator = this.model.indicators[i];
             var options;
             for (var j = 0; j < this.model.axes.length; j++) {
                 var axis = this.model.axes[j];

                 if (axis.name && axis.orientation.toLowerCase() == 'horizontal' && axis.name.toLowerCase() == indicator.xAxisName.toLowerCase()) {
                     indicator.xAxis = axis;
                 } else if (axis.name && axis.orientation.toLowerCase() == 'vertical' && axis.name.toLowerCase() == indicator.yAxisName.toLowerCase()) {
                     indicator.yAxis = axis;
                 }
             }
             var type = indicator.type.toLowerCase();
             options = new ej.indicatorTypes[type]();
             //Clip the canvas Chart indicator series out of chartArea Bounds
             if (this.model.enableCanvasRendering) {
                 this.svgRenderer.ctx.save();
                 this.svgRenderer.ctx.rect(this.model.m_AreaBounds.X, this.model.m_AreaBounds.Y, this.model.m_AreaBounds.Width, this.model.m_AreaBounds.Height);
                 this.svgRenderer.ctx.clip();
                 options.draw(indicator, this);
                 this.svgRenderer.ctx.restore();
             } else
                 options.draw(indicator, this);
             var element = options.gIndicatorGroupEle;
             if (element) ej.EjSvgRender.utils._drawAxesBoundsClipPath(element, indicator, this);

             if (!this.vmlRendering && !this.model.enableCanvasRendering) {
                 if (indicator.enableAnimation) {
                     this.svgRenderer.append(this.gIndicatorEle, this.svgObject);
                     options.doAnimation(indicator);
                     this.model.AnimationComplete = true;
                 }
             }
         }
         
         this.svgRenderer.append(this.gIndicatorEle, this.svgObject);


     },
    seriesRender: function () {
        var chartObj = this;
        if ($(this.svgObject).find("#" + this.svgObject.id + "_SeriesCollection").length > 0) {
            $(this.svgObject).find("#" + this.svgObject.id + "_SeriesCollection").remove();
        }
        var serOptions = { 'id': this.svgObject.id + '_SeriesCollection', 'clip-path': 'url(#' + this.svgObject.id + '_ChartAreaClipRect)' };

        this.gSeriesEle = this.svgRenderer.createGroup(serOptions);
        var visibleSeries = ej.DataManager(this.model._visibleSeries, ej.Query().sortBy("_zOrder")).executeLocal();
        for (var i = 0; i < visibleSeries.length; i++) {
            if (visibleSeries[i].visibility == "visible") {
            if (!ej.util.isNullOrUndefined(visibleSeries[i].xAxis)) {
                visibleSeries[i].xAxis = null;
                visibleSeries[i].yAxis = null;
            }
            for (var j = 0; j < this.model.axes.length; j++) {
                var axis = this.model.axes[j];
                var index = i;
                if (axis.name && axis.orientation.toLowerCase() == 'horizontal' && axis.name.toLowerCase() == visibleSeries[i].xAxisName.toLowerCase()) {
                    visibleSeries[i].xAxis = axis;
                }
                else if (axis.name && axis.orientation.toLowerCase() == 'vertical' && axis.name.toLowerCase() == visibleSeries[i].yAxisName.toLowerCase()) {
                    visibleSeries[i].yAxis = axis;
                }
                if (this.model.requireInvertedAxes) {
                    if (axis.name && axis.orientation.toLowerCase() == 'vertical' && axis.name.toLowerCase() == visibleSeries[i].xAxisName.toLowerCase()) {
                        visibleSeries[i].xAxis = axis;
                    }
                    else if (axis.name && axis.orientation.toLowerCase() == 'horizontal' && axis.name.toLowerCase() == visibleSeries[i].yAxisName.toLowerCase()) {
                        visibleSeries[i].yAxis = axis;
                    }
                }

            }
        }
        }
        for (var m = 0; m < visibleSeries.length; m++) {
            var index = m;
            var seriesOptions = visibleSeries[m];
            if (this.model.AreaType == "none" && seriesOptions.visibility.toLowerCase() === 'visible' && !this.model.enable3D) {
                if (index === 0) {

                    var type = seriesOptions.type.toLowerCase();
                    var seriesIndex = $.inArray(visibleSeries[0], this.model._visibleSeries);
                    this.model._visibleSeries[seriesIndex] = visibleSeries[0] = $.extend(true, seriesOptions, visibleSeries[0]);
                    var commonEventArgs = $.extend({}, ej.EjSvgRender.commonChartEventArgs);
                    commonEventArgs.data = { series: seriesOptions };
                    this._trigger("seriesRendering", commonEventArgs);
                    var seriesType = new ej.seriesTypes[type.toLowerCase()]();
                    var minhightwidth =seriesType.draw(chartObj, seriesOptions);
                    
                    if (!minhightwidth) {
                            if (seriesOptions.enableAnimation && !this.vmlRendering && !this.model.enableCanvasRendering) {
                            seriesType.doAnimation(seriesOptions);
                            }
                            this.drawAccDisplayText(seriesType, seriesOptions);
                    }
                }
            }
            else {			 
                if (seriesOptions.visibility.toLowerCase() === 'visible' && !this.model.enable3D)
                    this.renderSeries(seriesOptions);
                else{
				   if (!this.model.enable3D)
				     this.svgRenderer.append(this.svgObject, this.element);
				   else{
					 if (((this.model.AreaType == "none" && index == 0) || this.model.AreaType == "cartesianaxes") && seriesOptions.visibility.toLowerCase() === 'visible')
                       ej.Ej3DChart.prototype.renderSeries(this, seriesOptions);
					 }
			      }
			  }
        }
		if (this.model.enable3D) {
          this.svgRenderer.append(this.chart3D, this.svgObject);
           
          var size = { Width: $(this.svgObject).width(), Height: $(this.svgObject).height() };
          this.graphics.prepareView(this.model.perspectiveAngle, this.model.depth, this.model.rotation, this.model.tilt, size);
          this.graphics.view(this.svgObject, this);

          //for (var i = 0; i < visibleSeries.length; i++) {
          //    if (visibleSeries[i].visibility.toLowerCase() == "visible" && visibleSeries[i].enableAnimation && !(this.model.animated)) {
          //        var seriesRendering = new ej.Ej3DSeriesRender();
          //        var type = visibleSeries[i].type.toLowerCase();
          //        var options = ej.series3DTypes[type];
          //        if (this.model.AreaType == "none" && i > 0)
          //            break;
          //        options.doAnimation(visibleSeries[i], seriesRendering);
          //    }

          //}

        }
        if (this.svgRenderer.vmlNamespace && (this.model.AreaType != "none"))
            this.svgRenderer.append(this.gSeriesEle, this.svgObject);
        if (this.model.AreaType == 'polaraxes')
            this.svgRenderer.append(this.gPolarAxisEle, this.svgObject);
        if ($(this.svgObject).find("#" + this.svgObject.id + "_StriplineOver").length > 0) {
            var detachEle = $(this.svgObject).find("#" + this.svgObject.id + "_StriplineOver").detach();
            detachEle.appendTo(this.svgObject);
        }
		
		this.legendRender.drawLegend();
		
        chartObj.model.initSeriesRender = true;
    },
    _renderStripline: function (zIndex) {
        $.extend(ej.EjStripline.prototype, this);
        this.chartstripline = new ej.EjStripline(this);
        var stripline = this.chartstripline;
        for (var sl = 0; sl < this.model.axes.length; sl++) {
            var slAxis = this.model.axes[sl];
            for (var slCount = 0; slCount < slAxis.stripLine.length && slAxis.stripLine[slCount].visible; slCount++) {
                if (slAxis.stripLine[slCount].zIndex.toLowerCase() == zIndex)
                    stripline._drawStripline(slAxis, slAxis.stripLine[slCount]);
            }
        }
        if (zIndex == 'over') {

            this.svgRenderer.append(this.gStriplineOver, this.svgObject);
        } else {
            this.svgRenderer.append(this.gStriplineBehind, this.svgObject);
        }
    },

    renderSeries: function (options) {

        var chart = this;
        var axis;
        var index;
        var seriesIndex = $.inArray(options, this.model._visibleSeries);

        for (var i = 0; i < options.points.length; i++) {
            if (typeof options.points[i].x == "string" && options.points[i].x.indexOf("/Date(") != -1)
                options.points[i].x = new Date(parseInt(options.points[i].x.substr(6)));
        }
        options.xAxis = (options.xAxis === null || options.xAxis === undefined) ? this.model.axes[0] : options.xAxis;
        options.yAxis = (options.yAxis === null || options.yAxis === undefined) ? this.model.axes[1] : options.yAxis;
        var type = options.type.toLowerCase();
        var symbolOptions;
        var translate = null;
        var visibility;

        if (this.model.AreaType == "cartesianaxes") {
            var trans = ej.EjSvgRender.utils._getTransform(options.xAxis, options.yAxis, this.model.requireInvertedAxes);
            var translate = 'translate(' + trans.x + ',' + trans.y + ')';
            visibility = 'visible';
        }
        else {
            visibility = (options.enableAnimation) ? 'hidden' : 'visible'
        }
        var txtOptions = { 'id': this.svgObject.id + '_TextGroup' + '_' + seriesIndex, 'transform': translate, 'visibility': visibility };
        if (type != "scatter")
            symbolOptions = { 'id': this.svgObject.id + '_symbolGroup' + '_' + seriesIndex, 'transform': translate, 'visibility': visibility };
        else
            symbolOptions = { 'id': this.svgObject.id + '_SeriesGroup' + '_' + seriesIndex, 'transform': translate };
        this.gSymbolGroupEle = this.svgRenderer.createGroup(symbolOptions);
        this.gSeriesTextEle = this.svgRenderer.createGroup(txtOptions);

        var commonEventArgs = $.extend({}, ej.EjSvgRender.commonChartEventArgs);
        commonEventArgs.data = { series: options };
        this._trigger("seriesRendering", commonEventArgs);

        var series = new ej.seriesTypes[type]();
        if (this.model.enableCanvasRendering) {
            if (options.type.toLowerCase() != "bar" && options.type.toLowerCase() != "stackingbar") {
                this.canvasX = options.xAxis.x;
                this.canvasY = options.yAxis.y;
            }
            this.svgRenderer.ctx.save();
            this.svgRenderer.ctx.rect(this.model.m_AreaBounds.X, this.model.m_AreaBounds.Y, this.model.m_AreaBounds.Width, this.model.m_AreaBounds.Height);
            this.svgRenderer.ctx.clip();
            series.draw(this, options);
            this.svgRenderer.ctx.restore();
        }
        else
		{
		    this.canvasX = this.canvasY = 0;
            series.draw(this, options);
	    }
        
        //Add axes bounds clip path for series
        var element = series.gSeriesGroupEle;
        if (element) ej.EjSvgRender.utils._drawAxesBoundsClipPath(element, options, this);

        // Draw Symbols

        if (options.marker.visible) {
            if (!(type == "candle" || type == "hilo" || type == "hiloopenclose")) {
                var chartObj = this;
                $.each(options.points, function (pointIndex, point) {
				//this condition provide for better Scatter performance
                    if(series.chartObj.model.series[seriesIndex].marker.shape=='image')
                    series.drawSymbolImage(options, pointIndex);

                    var point1 = (chartObj.model.AreaType != "polaraxes")
                    ? ((point.symbolLocation) ? point.symbolLocation : ej.EjSvgRender.utils._getPoint(point, options))
                    : ((point.symbolLocation) ? point.symbolLocation : ej.EjSvgRender.utils.TransformToVisible(options, point.X, point.y, chartObj))

                    if (chartObj.model.AreaType == "polaraxes") {
                        var labelFormat = options.yAxis.labelFormat ? options.yAxis.labelFormat : "";
                        var pointText = (point.text) ? point.text : point.y + labelFormat.substring(labelFormat.indexOf('}') + 1);
                        var textOffset = ej.EjSvgRender.utils._measureText(pointText, null, options.marker.dataLabel.font);
                     }
                    if ((ej.util.isNullOrUndefined(point.marker) || ej.util.isNullOrUndefined(point.marker.visible) || (point.marker && point.marker.visible)) && (point.visible))
                    series.drawSymbol(seriesIndex, options, pointIndex, point1.X, point1.Y);
                });
                if (type != "scatter") {
                    this.svgRenderer.append(this.gSymbolGroupEle, this.gSeriesEle);
                    //Add axes bounds clip path for marker
                    if (options.marker.visible) {
                        element = this.gSymbolGroupEle;
                       ej.EjSvgRender.utils._drawAxesBoundsClipPath(element, options, this);
                    }
                    //   Add axes bounds clip path for marker text
                    if (options.marker.dataLabel.visible) {
                        element = this.gSeriesTextEle;
                        ej.EjSvgRender.utils._drawAxesBoundsClipPath(element, options, this);
                    }
                } else {
                    element = this.gSymbolGroupEle;
                    ej.EjSvgRender.utils._drawAxesBoundsClipPath(element, options, this);
                    this.svgRenderer.append(this.gSymbolGroupEle, this.gSeriesEle);

                    if (options.tooltip.visible || type == "scatter") {
                        if (!this.model.enableCanvasRendering)
                            var elements = $(this.gSymbolGroupEle).children().not("defs");
                        else
                            elements = this.model.series[seriesIndex].points;
                        if (this.vmlRendering) {
                            this.cloneSeriesEle = $(this.gSeriesEle).clone();
                            this.cloneobj = $(this.element).clone();
                            this.svgclone = $(this.svgObject).clone();
                            $(document.body).append(this.cloneobj);
                            this.svgRenderer.append(this.cloneSeriesEle, this.svgclone);
                            this.svgRenderer.append(this.svgclone, this.cloneobj);
                        }
                        else {
                            this.svgRenderer.append(this.gSeriesEle, this.svgObject);
                            this.svgRenderer.append(this.svgObject, this.element);
                        }


                        for (var j = 0; j < elements.length; j++) {
                            var elementNode = (this.vmlRendering) ? ($("#" + (elements[j].id))[0]) : elements[j];
                           
                            if (this.model.enableCanvasRendering) {
                                x = elementNode.cx;
                                y = elementNode.cy;
                                valwidth = elementNode.width;
                                valheight = elementNode.height;
                            } else {
                                var bound = ej.EjSvgRender.utils._getBoundingClientRect(elementNode, this, options, false);

                                var valwidth, x, y, valheight;
                                x = (bound.x + options.xAxis.x);
                                y = (bound.y + options.yAxis.y);

                                valheight = bound.height;
                                valwidth = bound.width;

                            }

                            var bounds = { X: x, Y: y, Width: valwidth, Height: valheight };
                            ej.EjSvgRender.utils._addRegion(this, bounds, options, null, j);
                        }
                        if (this.vmlRendering) {
                            $(this.cloneobj).remove();
                            $(this.svgclone).remove();
                        }
                    }
                }
            }

        }

        //Draw Series Text

        this.drawDisplayText(series, options);

     if (this.model.enableCanvasRendering) 
            this.svgRenderer.append(this.svgObject, this.element);
    else if (!this.vmlRendering) {
            this.svgRenderer.append(this.gSeriesEle, this.svgObject);
            this.svgRenderer.append(this.svgObject, this.element);
            if (options.enableAnimation) {
			var templateContainer = $('#template_group_' + this._id); 
                var dataLabel=options.marker.dataLabel;				
               if (templateContainer.children().length != 0 && (dataLabel.visible && dataLabel.template))	 	
                   series.animateLabelTemplate(options);
                series.doAnimation(options);
                this.model.AnimationComplete = true;
                var commonEventArgs = $.extend({}, ej.EjSvgRender.commonChartEventArgs);
                commonEventArgs.data = {series : options};
                this._trigger("animationComplete", commonEventArgs);
            }
        }

    },
	
	drawDisplayText : function (series, options){
	    // method to draw text 
	    if (options.marker.dataLabel.visible) {
	        var chartObj = this;
	        var count = -1;
	        $.each(options.points, function (pointIndex, point) {
	            var point1 = (chartObj.model.AreaType != "polaraxes")
	                ? ((point.symbolLocation) ? point.symbolLocation : ej.EjSvgRender.utils._getPoint(point, options))
	                : (ej.EjSvgRender.utils.TransformToVisible(options, point.X, point.y, chartObj));
	            if ((ej.util.isNullOrUndefined(point.marker) || (point.marker.dataLabel && point.marker.dataLabel.visible)) && point.visible){
                    count++;
	                if (chartObj.zoomed) {
	                    var seriesIndex = $.inArray(options, chartObj.model._visibleSeries);
	                    for (var i = 0; i < chartObj.model.chartRegions.length; i++) {
	                        if (seriesIndex == chartObj.model.chartRegions[i].SeriesIndex && pointIndex == chartObj.model.chartRegions[i].Region.PointIndex) {
	                            if (chartObj.model.enableCanvasRendering) {
	                                chartObj.svgRenderer.ctx.save();
	                                chartObj.svgRenderer.ctx.rect(chartObj.model.m_AreaBounds.X, chartObj.model.m_AreaBounds.Y, chartObj.model.m_AreaBounds.Width, chartObj.model.m_AreaBounds.Height);
	                                chartObj.svgRenderer.ctx.clip();
	                                series.drawDataLabel(options, pointIndex, point, point1.X, point1.Y, i);
	                                chartObj.svgRenderer.ctx.restore();
	                            } else
	                                series.drawDataLabel(options, pointIndex, point, point1.X, point1.Y, i);
	                        }
	                    }
	                }
	                else {
	                    if (chartObj.model.enableCanvasRendering) {
	                        chartObj.svgRenderer.ctx.save();
	                        chartObj.svgRenderer.ctx.rect(chartObj.model.m_AreaBounds.X, chartObj.model.m_AreaBounds.Y, chartObj.model.m_AreaBounds.Width, chartObj.model.m_AreaBounds.Height);
	                        chartObj.svgRenderer.ctx.clip();
	                        series.drawDataLabel(options, pointIndex, point, point1.X, point1.Y, count);
	                        chartObj.svgRenderer.ctx.restore();
	                    } else
	                        series.drawDataLabel(options, pointIndex, point, point1.X, point1.Y, count);
	                }
	            }
	        });

	        this.svgRenderer.append(this.gSymbolGroupEle, this.gSeriesEle);
	        this.svgRenderer.append(this.gSeriesTextEle, this.gSeriesEle);
	    }
	},

	drawAccDisplayText: function (series, options) {
	    // method to draw text 
	    var chartObj = this;
	    if (options.marker.dataLabel.visible) {
	        if ((options.type.toLowerCase() != "pyramid" && options.type.toLowerCase() != "funnel") && (options.leftsidePoints.length > 0 || options.rightsidePoints.length > 0)) {
	            chartObj.model.bounds["left"] = [];
	            chartObj.model.bounds["right"] = [];

	            for (var i = 0; i < options.rightsidePoints.length; i++) {
	                options.rightsidePoints[i].index = i;
	                series.drawDataLabelAcc(chartObj, options, options.rightsidePoints[i].pointIndex, options.rightsidePoints[i])
	            }
	            for (var j = options.leftsidePoints.length - 1; j >= 0; j--) {
	                options.leftsidePoints[j].index = j;
	                series.drawDataLabelAcc(chartObj, options, options.leftsidePoints[j].pointIndex, options.leftsidePoints[j])
	            }
	        }
	        else {
	            for (var i = 0; i < options.points.length; i++) {
	                options.points[i].index = i;
	                series.drawDataLabelAcc(chartObj, options, i, options.points[i])
	            }
	        }
	    }

	    this.svgRenderer.append(this.gSeriesConnectorEle,  this.gSeriesEle);

	    this.svgRenderer.append(this.gSymbolGroupEle,  this.gSeriesEle);

	    this.svgRenderer.append(this.gSeriesTextEle,  this.gSeriesEle);

	    this.svgRenderer.append(this.gSeriesEle,  this.svgObject);

	    if (!this.vmlRendering) 
	    this.svgRenderer.append(this.svgObject, this.element);
	},

    _processOData: function (series) {
        var chart = this;
        var queryPromise = series.dataSource.executeQuery(series.query);
        queryPromise.done(function (e) {
            chart._processJsonData(e.result, series);
            series.visibility = 'visible';
            $(chart.svgObject).empty();
            $(chart.legendSvgContainer).empty();
            if (chart.model.enableCanvasRendering) {
                chart.svgRenderer.ctx.clearRect(0, 0, chart.svgObject.width, chart.svgObject.height);
                $("#canvas_trackSymbol").remove();
            }

            var redrawChart=true;
            for (var i = 0; i < chart.model.series.length; i++)
                if (chart.model.series[i].dataSource)
                {
                    if (chart.model.series[i].visibility == 'hidden') {
                        redrawChart = false;
                        break;
                    }
                }
            if (redrawChart) {
                chart.draw();
            }
        });
        queryPromise.fail(function (e) {
            series.visibility = 'visible';
            $(chart.svgObject).empty();
            $(chart.legendSvgContainer).empty();
            var redrawChart=true;
            for (var i = 0; i < chart.model.series.length; i++)
                if (chart.model.series[i].dataSource)
                {
                    if (chart.model.series[i].visibility == 'hidden') {
                        redrawChart = false;
                        break;
                    }
                }
            if (redrawChart) {
                chart.draw();
            }
        });
    },
    _processJsonData: function (jsonObj, series) {
        if (series.points.length > 0)
           series.points.length = 0;
        var xdata = [];
        var dataSource = series.dataSource;
        var xName = series.xName;
        var yNames = series.yName;

        for (var j = 0; j < jsonObj.length; j++) {
            if (typeof jsonObj[j][xName] == "string" && jsonObj[j][xName].indexOf("/Date(") != -1) {
                jsonObj[j][xName] = new Date(parseInt(jsonObj[j][xName].substr(6)));
                xdata.push((jsonObj[j][xName]) ? jsonObj[j][xName] : j);
            }
            else
                xdata.push((jsonObj[j][xName]) ? jsonObj[j][xName] : j);
        }
        var yValues = [];
        var index = 0;
        if (!yNames) {
            if (series._hiloTypes) {
                var yVal = [];
                //For high
                var high = series.high;
                for (var s = 0; s < jsonObj.length; s++) {
                    yVal.push(jsonObj[s][high] === null || jsonObj[s][high] === undefined ? jsonObj[s][high] : parseFloat(jsonObj[s][high]));

                }
                yValues[index] = yVal;
                index = index + 1;
                //For low
                yVal = [];
                var low = series.low;
                for (var l = 0; l < jsonObj.length; l++) {
                    yVal.push(jsonObj[l][low] === null || jsonObj[l][low] === undefined ? jsonObj[l][low] : parseFloat(jsonObj[l][low]));
                }
                yValues[index] = yVal;
                index = index + 1;
                if (series.type.toLowerCase() !== 'hilo') {
                    //For open
                    yVal = [];
                    var open = series.open;
                    for (var h = 0; h < jsonObj.length; h++) {
                        yVal.push(jsonObj[h][open] === null || jsonObj[h][open] === undefined ? jsonObj[h][open] : parseFloat(jsonObj[h][open]));
                    }
                    yValues[index] = yVal;
                    index = index + 1;

                    //For close
                    yVal = [];
                    var close = series.close;
                    for (var mI = 0; mI < jsonObj.length; mI++) {
                        yVal.push((jsonObj[mI][close] === null || jsonObj[mI][close] === undefined) ? jsonObj[mI][close] : parseFloat(jsonObj[mI][close]));
                    }
                    yValues[index] = yVal;
                }
            }
        }
        if (yNames) {
           
                var yNVal = [];
                for (var yI = 0; yI < jsonObj.length; yI++) {
                    yNVal.push((jsonObj[yI][yNames]) ? parseFloat(jsonObj[yI][yNames]) : jsonObj[yI][yNames]);
                }
                yValues[index] = yNVal;
                index = index + 1;
            

            if (series.type.toLowerCase() == 'bubble') {
                yVal = [];
                var size = series.size;
                for (var b = 0; b < jsonObj.length; b++) {
                    yVal.push(jsonObj[b][size] === null || jsonObj[b][size] === undefined ? jsonObj[b][size] : parseFloat(jsonObj[b][size]));
                }
                yValues[index] = yVal;
            }
        }


        for (var m = 0; m < xdata.length; m++) {
            series.points[m] = { x: xdata[m], X: xdata[m], y: yValues[0][m], YValues: [], visible: true };
            if (series.type.toLowerCase() == 'bubble') {
                series.points[m].YValues.push(yValues[0][m]);
                series.points[m].size = series.points[m].YValues[1] = yValues[1][m];
            }
            else if (series._hiloTypes) {
                series.points[m].high = series.points[m].YValues[0] = yValues[0][m];
                series.points[m].low = series.points[m].YValues[1] = yValues[1][m];
                if (series.type.toLowerCase() != 'hilo' && series.type.toLowerCase() != 'rangecolumn') {
                    series.points[m].open = series.points[m].YValues[2] = yValues[2][m];
                    series.points[m].close = series.points[m].YValues[3] = yValues[3][m];
                }
            }
            else {
                series.points[m].YValues.push(yValues[0][m]);
            }
        }
    },
    bindTo: function (excludeDataUpdate) {
        if (this.model.series) {
            var series = this.model.series;
            this.setModelProperties();
            if (!this.zoomed && !this.panning) {
                for (var i = 0; i < series.length; i++) {
                    if (series[i].visibility == null || series[i].visibility == undefined) {
                        series[i].visibility = 'visible';
                        if (series[i].type) series[i]._hiloTypes = ej.seriesTypes[series[i].type.toLowerCase()].prototype.hiloTypes;
                    }
                    if (!(series[i].points))
                        series[i].points = [];
                    if (series[i].dataSource) {
                        if (series[i].dataSource instanceof ej.DataManager) {
                            series[i].visibility = 'hidden';
                            this._processOData(series[i]);
                        } else if (series[i].dataSource.length > 0) {

                            this._processJsonData(series[i].dataSource, series[i]);

                        }
                    }
                }
            }
            if($(this.svgObject).width()>0)
            this.draw(excludeDataUpdate);
        }



    },

    chartResize: function () {
        var chart = this;
        var $svgObj = $(chart.svgObject);
        if (this.model.enableCanvasRendering) {
            $("#canvas_trackSymbol").remove();
            var ctx = chart.svgObject.getContext("2d");
            ctx.clearRect(0, 0, this.svgWidth, this.svgHeight);
            this.svgRenderer.svgObj.height = ej.util.isNullOrUndefined(this.model.size.height) ? "450" : this.model.size.height;
            this.svgRenderer.svgObj.width = $("#" + this._id).width();

        }
        if (this.resizeTO) clearTimeout(this.resizeTO);
        this.resizeTO = setTimeout(function () {
            //width calculation
            if (chart.model.size.width && chart.model.size.width.indexOf("%") != -1) {
                var chartWidth = chart.model.size.width;
                var containerWidth = $(chart.element).width();
                var width = (containerWidth / 100) * parseInt(chartWidth);
                $svgObj.width(width);
            }
            else
                $svgObj.width($(chart.element).width());
            //height calculation
            if (chart.model.size.height && chart.model.size.height.indexOf("%") != -1) {
                var chartHeight = chart.model.size.height;
                var containerHeight= $(chart.element).height();
                var height = (containerHeight / 100) * parseInt(chartHeight);
                $svgObj.height(height);
            }
            else
                $svgObj.height($(chart.element).height());
            if ($(chart.svgObject).width() > 0) {
                $(chart.svgObject).empty();
	        $(chart.legendSvgContainer).empty();
                $('#template_group_' + chart._id).remove();
                chart.disableAnimation();
                for (var i = 0; chart.model.axes && i < chart.model.axes.length; i++)
                    chart.model.axes[i].desiredIntervals = null;
                for (var j = 0; j < chart.model.series.length; j++)
                    chart.model.series[j].regionAdded = false;
                chart.draw();
                if (chart.zoomed)
                    chart._enableZoomingButtons();
                chart.enableAnimation();
            }
        }, 500);

    },
    isDevice: function () {
        return (/mobile|tablet|android|kindle/i.test(navigator.userAgent.toLowerCase()));
        // comment above line temporary. Due to the below code event wont bind for tablet device
        //  return (/mobile|android|kindle/i.test(navigator.userAgent.toLowerCase()));
    },
    bindEvents: function () {
	    var matched = jQuery.uaMatch(navigator.userAgent);
		
	    this._on($(this.svgObject), "touchstart", this.chartMouseDown);
        this._on(this.element, "touchmove", this.chartMouseMove);
        this._on($(document), "touchend", this.chartMouseUp);
		this._on(this.element, 'touchleave', this.chartMouseLeave);
		
		//this._on($(this.svgObject), "mousedown", this.chartMouseDown);
		this._on($(this.element), "mousedown", this.chartMouseDown);
        this._on(this.element, 'mouseleave', this.chartMouseLeave);
        this._on($(document), "mouseup", this.chartMouseUp);
        this._on($(this.element), "click", this.chartMouseClick);
        var isIE11 = !!navigator.userAgent.match(/Trident\/7\./);
        if (this.isDevice() && !isIE11 && !this.vmlRendering) {
            if (window.navigator.msPointerEnabled) {
                this._on(this.element, 'MSPointerMove', this.chartMouseMove);
                this._on(this.element, 'MSPointerDown', this.chartMouseDown);
			    this._on(this.element, 'MSPointerOut', this.chartMouseLeave);
                this._on($(document), "MSPointerUp", this.chartMouseUp);
			    this._on($(this.svgObject), 'click', this.chartMouseClick);
                this._on($(this.legendSvgContainer), 'click', this.chartMouseClick);
                if (this.model.zooming.enable || this.model.crosshair.visible)
				$(this.svgObject).css('-ms-touch-action', 'none');
            }
            else {
                this._on(this.element, 'touchmove', this.chartTouchMove);
                this._on($(this.svgObject), 'touchstart', this.chartTouchClick);
                    this._on($(this.legendSvgContainer), 'touchstart', this.chartTouchClick);
            }
        }
        else {
            this._on(this.element, 'mousemove', this.chartMouseMove);
            this._on($(this.svgObject), 'click', this.chartMouseClick);
           this._on($(this.legendSvgContainer), 'click', this.chartMouseClick);
        }
        if (this.model.canResize) {
            if (!this.isDevice() || window.navigator.msPointerEnabled)
                this._on($(window), "resize", this.chartResize);
            else
                this._on($(window), "orientationchange", this.chartResize);
        }
        
        var eventName = matched.browser.toLowerCase() == "mozilla" ? "DOMMouseScroll" : "mousewheel";
        this._on(this.element, eventName, this.chartMouseWheel);

    },
    calZoomFactors: function (cumulativeScale, origin, currentZoomFactor, currentZoomPos) {
        var calcZoomFactorVal, calcZoomPosition;
        if (cumulativeScale == 1) {
            calcZoomFactorVal = 1;
            calcZoomPosition = 0;
        } else {
            calcZoomFactorVal = ej.EjSvgRender.utils._minMax(1 / cumulativeScale, 0, 1);
            calcZoomPosition = currentZoomPos + ((currentZoomFactor - calcZoomFactorVal) * origin);
        }
        return { zoomMFactor: calcZoomFactorVal, zoomMPosition: calcZoomPosition };
    },
    doMouseWheelZoom: function (cumulativeScale, origin, axis) {
        if (cumulativeScale >= 1) {
            var calZoomVal = this.calZoomFactors(cumulativeScale, origin, axis.zoomFactor, axis.zoomPosition);
            //calcZoomPos = Math.Round(calcZoomPos, 2);
            if (axis.zoomPosition != calZoomVal.zoomMPosition) {
                this.zoomed = true;
                axis.zoomPosition = calZoomVal.zoomMPosition;
                axis.zoomFactor = (calZoomVal.zoomMPosition + calZoomVal.zoomMFactor) > 1 ? (1 - calZoomVal.zoomMPosition) : calZoomVal.zoomMFactor;
                return true;
            }
        }

        return false;

    },
    enableAnimation: function () {
        for (var m = 0; m < this.model.series.length; m++) {
            var series = this.model.series[m];
            series.enableAnimation = this.serAnimation[m];
        }
    },
    disableAnimation: function () {
        var chart = this;
        for (var m = 0; m < chart.model.series.length; m++) 
            chart.model.series[m].enableAnimation = false;
        for (var k = 0; k < chart.model.indicators.length; k++) 
            chart.model.indicators[k].enableAnimation = false;
    },
    _removeTrackBall: function () {
        var id = "#" + this.svgObject.id;
        $("#" + this.svgObject.id + "_trackSymbol").remove();
		 $(document).find("#measureTex").remove();
		 if (this.chartCross.visible) {
		     // Visibility is set to hidden
		     var element = $(this.svgObject).find(id + "_AxisCrossToolTip");
        this.svgRenderer._setAttr($(element), { "visibility": 'hidden' });
		     element = $(this.svgObject).find(id + "_CrosshairVertical");
        this.svgRenderer._setAttr($(element), { "d": 'M 0 0' });
		     element = $(this.svgObject).find(id + "_CrosshairHorizontal");
        this.svgRenderer._setAttr($(element), { "d": 'M 0 0' });
		     
        element = $(this.svgObject).find(id + "_TrackAxisToolTip");
        this.svgRenderer._setAttr($(element), { "visibility": 'hidden'});
        element = $(this.svgObject).find(id + "_Tracker");
        this.svgRenderer._setAttr($(element), { "d": 'M 0 0'});
        element = $(this.svgObject).find('[id*="_TrackToolTip"]');
        this.svgRenderer._setAttr($(element), { "visibility": 'hidden' });
        element = $(this.svgObject).find('[id*="_TrackToolTip"]').children();
        this.svgRenderer._setAttr($(element), { "visibility": 'hidden' });
        element = $(this.svgObject).find('[id*="_trackSymbol_"]');
        this.svgRenderer._setAttr($(element), { "visibility": 'hidden' });
        $(id + "_CrosshairVertical").css("visibility", "hidden");
        $(id + "_CrosshairHorizontal").css("visibility", "hidden");
        $("#" + this._id).find('[id*="canvas_AxisToolTipRect"]').css("visibility", "hidden");
        $("#" + this._id).find('[id*="canvas_ToolTipRect"]').css("visibility", "hidden");
		$("#" + this._id).find('[id*="canvas_Tracker"]').remove();
        $("#" + this._id).find('[id*="canvas_trackSymbol"]').css("visibility", "hidden");
		}
    },
    enableTrackBall: function () {
        this.model.crosshair.visible = this.chartCross.visible;
        this.model.crosshair.type = this.chartCross.mArea;
    },
    disableTrackBall: function () {

        this.model.crosshair.visible = false;
        
        this._removeTrackBall();
    },
    _enableZoomingButtons: function () {
        if ($("#" + this._id + '_ResetZoom').length == 0) {
            this.resetZoomButton();
            this.panButton();
            this.zoomButton();
        }
    },
    chartMouseWheel: function (e) {
        if (this.model.zooming.enableMouseWheel && this.model.AreaType == "cartesianaxes" && !this.model.enable3D) {
            var chart = this;       
            var chartZoomed;
            this.zoomed = false;
            var canUpdate = false;
            var id = "#" + chart.svgObject.id;
            var matched = jQuery.uaMatch(navigator.userAgent);
            var direction = matched.browser.toLowerCase() == "mozilla" ? (-(e.originalEvent.detail) / 3 > 0 ? 1 : -1) : ((e.originalEvent.wheelDelta / 120) > 0 ? 1 : -1);
            for (var i = 0; i < this.model.axes.length; i++) {
                var axis = this.model.axes[i];
                if ((axis.orientation.toLowerCase() == "vertical" && (chart.model.zooming.type.toLowerCase() == "y" || chart.model.zooming.type.trim().toLowerCase() == "x,y")) || (axis.orientation.toLowerCase() == "horizontal" && (chart.model.zooming.type.toLowerCase() == "x" || chart.model.zooming.type.trim().toLowerCase() == "x,y"))) {

                    var currentScale = Math.max(1 / ej.EjSvgRender.utils._minMax(axis.zoomFactor, 0, 1), 1);
                    var cumulativeScale = Math.max(currentScale + (0.25 * direction), 1);
                    canUpdate = canUpdate | chart.doMouseWheelZoom(cumulativeScale, 0.5, axis);
                }
                if (axis.zoomFactor != 1 && axis.zoomPosition != 0)
                    chartZoomed = true;
            }

            this.disableAnimation();

            this.disableTrackBall();
            if (canUpdate) {
                this.model.legendCollapsed = false;
                this.zoomed = true;
                chart.redraw(true);
              }
            
            this._enableZoomingButtons();
            this.mouseWheelCoords = { x: e.originalEvent.pageX, y: e.originalEvent.pageY };
            if (!chartZoomed) {
                $(chart.svgObject).find(id + '_ResetZoom,' + id + '_PanBtn,' + id + '_ZoomBtn').remove();
                this.enableAnimation();
                this.panning = false;
            }
			var matched = jQuery.uaMatch(navigator.userAgent);
            if (e.preventDefault && matched.browser == 'chrome' && this.model.zooming.enable)
                e.preventDefault();
            e.returnValue = false;
        }

        $(".tooltipDiv").remove();
    },
    getLegendData: function (evt) {
        var mouseMoveCords = this.calMousePosition(evt);
        var scrolltop = $("#legend_" + this._id).scrollTop();
        var scrollleft = $("#legend_" + this._id).scrollLeft();
        this.scrolltop = scrolltop;
        this.scrollleft = scrollleft;
        this.mousemoveX = mouseMoveCords.X+scrollleft;
        this.mousemoveY = mouseMoveCords.Y+scrolltop;

        {
            var currentX = this.mousemoveX;
            var currentY = this.mousemoveY;
            var chartId = this._id;
            var region;
            if ((!ej.util.isNullOrUndefined(evt.target.id) && this.svgRenderer._getAttrVal($(evt.target).parents(':eq(1)'), 'id') == this.svgObject.id + '_Legend') ||(this.model.enableCanvasRendering)) {
                $.each(this.model.legendRegion, function (index, regionItem) {

                    if ((currentX >= regionItem.Bounds.LegendBound.X + regionItem.Bounds.ItemBound.X) && (currentX <= regionItem.Bounds.LegendBound.X + regionItem.Bounds.ItemBound.X + regionItem.LegendItem.Bounds.Width)) {
                        if ((currentY >= regionItem.Bounds.LegendBound.Y + regionItem.Bounds.ItemBound.Y - (regionItem.LegendItem.Bounds.Height / 4)) && (currentY <= regionItem.Bounds.LegendBound.Y + regionItem.Bounds.ItemBound.Y + regionItem.LegendItem.Bounds.Height / 2)) {
                           region = regionItem;
                           $("#legend_" +chartId + "_canvas").css("cursor", "pointer");
                        }else
                            $("#legend_" + chartId + "_canvas").css("cursor", "default");
                    } 
                });
                if (!ej.util.isNullOrUndefined(region)) {
                    return { legendItem: region, series: this.model._visibleSeries[region.LegendItem.SeriesIndex] };
                }
            }

        }
        return false;
    },

    get3DSeriesPoint: function (evt) {
        var currentX = this.mousemoveX;
        var currentY = this.mousemoveY;
        var x = (this.model.AreaType == 'polaraxes') ? 0 : this.model.m_AreaBounds.X;
        var y = (this.model.AreaType == 'polaraxes') ? 0 : this.model.m_AreaBounds.Y;
        var width = (this.model.AreaType == 'polaraxes') ? $(this.svgObject).width() : this.model.m_AreaBounds.Width;
        var height = (this.model.AreaType == 'polaraxes') ? $(this.svgObject).height() : this.model.m_AreaBounds.Height;

        if (currentX > x && currentX < (x + width) && currentY > y && currentY < (y + height)) {
            var nodeName = $(evt.target)[0].nodeName;
            if ((nodeName == "path" || nodeName == "shape") && $(evt.target)[0].id.indexOf("Region")>1) {
                index = $(evt.target)[0].id.match(/(\d+)/g);
                pointIndex = parseInt(index[index.length - 1]);
                seriesIndex = parseInt(index[index.length - 2]);
                var regionValue = {};
                regionValue.Region = {};
                regionValue.Region.PointIndex = pointIndex;
                regionValue.SeriesIndex = seriesIndex;
                var pointData = {};
                pointData.pointIndex = pointIndex;
                region = { region: regionValue, pointData: pointData, location: { x: currentX, y: currentY} };
                return region;
            }
        }
    },

    GetSeriesPoint: function (evt) {
        var mouseX;
        var mouseY;
        var currentX = this.mousemoveX;
        var currentY = this.mousemoveY;
        var region;
        var chartObj = this;
        if (chartObj.model.enable3D) {
            return this.get3DSeriesPoint(evt);           
        }
        var x = (this.model.AreaType == 'polaraxes') ? 0 : this.model.m_AreaBounds.X;
        var y = (this.model.AreaType == 'polaraxes') ? 0 : this.model.m_AreaBounds.Y;
        var width = (this.model.AreaType == 'polaraxes') ? $(this.svgObject).width() : this.model.m_AreaBounds.Width;
        var height = (this.model.AreaType == 'polaraxes') ? $(this.svgObject).height() : this.model.m_AreaBounds.Height;

        if ((this.model.AreaType == 'cartesianaxes' || this.model.AreaType == 'polaraxes')) {
          
            if (currentX > x && currentX < (x + width) && currentY > y && currentY < (y + height)) {
                $.each(this.model.chartRegions, function (index, regionItem) {
                    if (!ej.util.isNullOrUndefined(regionItem.SeriesIndex)) {
                    var currentSer = chartObj.model._visibleSeries[regionItem.SeriesIndex];
                    if (currentSer.drawType.toLowerCase() == 'column' && evt.target) {
                        if (chartObj.model.enableCanvasRendering && currentSer.type.toLowerCase()=="polar") {

                            var bounds = regionItem.Region.Bounds;
                            var chartStartAngle = -.5 * Math.PI;
                            var innerRadius = bounds.DRadius;
                            var fromCenterX;
                            var fromCenterY;
                            fromCenterX = (currentX) - (bounds.CenterX);
                            fromCenterY = (currentY) - (bounds.CenterY);
                            var series = chartObj.model._visibleSeries[regionItem.SeriesIndex];
                            var startAngle = series.startAngle;
                            var endAngle = series.endAngle;

                            var arcAngle = (startAngle) ? 2 * Math.PI * (chartObj.model.itemCurrentXPos < 0 ? 1 + chartObj.model.itemCurrentXPos : chartObj.model.itemCurrentXPos) : 0;
                            var clickAngle = (Math.atan2(fromCenterY, fromCenterX) - chartStartAngle - arcAngle) % (2 * Math.PI);
                            if (clickAngle < 0 && bounds.PointIndex != 0) clickAngle = 2 * Math.PI + clickAngle;

                            var pointStartAngle = parseFloat(bounds.StartAngle.toFixed(14));
                            var pointEndAngle = parseFloat(bounds.EndAngle.toFixed(14));
                            pointStartAngle -= arcAngle;
                            pointEndAngle -= arcAngle;

                            if (clickAngle >= pointStartAngle && clickAngle <= pointEndAngle) {
                                index = bounds.PointIndex;
                                distanceFromCenter = Math.sqrt(Math.pow(Math.abs(fromCenterX), 2) + Math.pow(Math.abs(fromCenterY), 2));
                                if (distanceFromCenter <= bounds.Radius && distanceFromCenter > innerRadius) {
                                    var pointRegion = { Region: { PointIndex: index }, SeriesIndex: regionItem.SeriesIndex, type: regionItem.type };
                                    region = pointRegion;
                                }
                            }
                        }
                        else {
                        var nodeName = $(evt.target)[0].nodeName;
                        if ((nodeName == "path" || nodeName == "shape") && $($(evt.target).parent())[0].id == chartObj.svgObject.id + '_SeriesGroup' + '_' + regionItem.SeriesIndex) {
                                index = parseInt(chartObj.svgRenderer._getAttrVal($(evt.target)[0], "pointIndex"));
                            regionItem = { Region: { PointIndex: index }, SeriesIndex: regionItem.SeriesIndex, type:regionItem.type };
                            region = regionItem;
                            return false;
                        }
                    }
                    }
                    else {
                        if ((((currentX >= regionItem.Region.Bounds.X) && (currentX <= (regionItem.Region.Bounds.X + (regionItem.Region.Bounds.Width))))) && ((currentY > regionItem.Region.Bounds.Y) && (currentY < (regionItem.Region.Bounds.Y) + (regionItem.Region.Bounds.Height)))) {
                            region = regionItem;
                            return false;
                        }
                    }
                    }
                });
            }
            if (region) {
                return { region: region, location: { x: currentX, y: currentY} };
            }
        }

        else {
            var chartObj = this;
            var distanceFromCenter;
            var regionValue;
            if (chartObj.model.chartRegions.length > 0 && (chartObj.model.chartRegions[0].Series.type.toLowerCase() == "pyramid"||chartObj.model.chartRegions[0].Series.type.toLowerCase() == "funnel")) {
                var nodeName = $(evt.target)[0].nodeName;
                if ((nodeName == "path" || nodeName == "shape") && $($(evt.target).parent())[0].id == chartObj.svgObject.id + '_SeriesGroup' + '_' + chartObj.model.chartRegions[0].SeriesIndex) {
                    regionValue = chartObj.model.chartRegions[0];
                    index = $(evt.target)[0].id.match(/(\d+)/g);
                    index = parseInt(index[index.length - 1]);
                    regionValue.Region.PointIndex = index;
                    var pointData = {};
                    pointData.pointIndex = index;
                    region = { region: regionValue, pointData: pointData, location: { x: currentX, y: currentY} };
                    return region;
                }
                else {                    
                    regionValue = chartObj.model.chartRegions[0];
                    var pyrX = this.pyrX;
                    var pyrY = this.pyrY;
                        var pointData = {};                       
                        for (var j = 0; j < regionValue.Region.length; j++) {
                            line1 = regionValue.Region[j].Line1;
                            line2 = regionValue.Region[j].Line2;
                            line3 = regionValue.Region[j].Line3;
                            line4 = regionValue.Region[j].Line4;
                            var polygon = [
                                            { x: line1.x + pyrX, y: line1.y + pyrY },
                                            { x: line2.x + pyrX, y: line2.y + pyrY },
                                            { x: line3.x + pyrX, y: line3.y + pyrY },
                                            { x: line4.x + pyrX, y: line4.y + pyrY },
                            ];

                            var currentPoint = { x: currentX, y: currentY };
                            var point = this.isPointInPolygon(polygon, currentPoint);
                            if (point) {
                                pointData.pointIndex = regionValue.Region[j].PointIndex;
                                regionValue.Region.PointIndex = pointData.pointIndex;
                                region = { region: regionValue, pointData: pointData, location: { x: currentX, y: currentY } };                                                                
                                return region;
                            }

                        }
                    }                
                }
            
            else {
                // ReSharper disable DuplicatingLocalDeclaration
                $.each(chartObj.model.chartRegions, function (regionIndex, regionValue) {
                    // ReSharper restore DuplicatingLocalDeclaration
                    var chartStartAngle = -.5 * Math.PI;
                    var seriesData = regionValue.SeriesData; var regionData = regionValue.Region;
                    var innerRadius = (chartObj.model._visibleSeries[regionValue.SeriesIndex].type.toLowerCase() == 'doughnut') ? seriesData.DRadius : 0;
                    var fromCenterX;
                    var fromCenterY;
                    fromCenterX = (currentX) - (seriesData.CenterX);
                    fromCenterY = (currentY) - (seriesData.CenterY);
                    var series = chartObj.model._visibleSeries[regionValue.SeriesIndex];
                    var startAngle = series.startAngle;
                    var endAngle = series.endAngle;
                    startAngle = startAngle < 0 ? startAngle + 360 : startAngle;
                    endAngle = endAngle < 0 ? endAngle + 360 : endAngle;
                    var totalDegree = (series.endAngle - series.startAngle);
                    //Anticlockwise pie and doughnut
                    if (totalDegree < 0) {
                        endAngle = endAngle / 360;
                        var arcAngle = (endAngle) ? 2 * Math.PI * (endAngle < 0 ? 1 + endAngle : endAngle) : 0;
                        var clickAngle = (Math.atan2(fromCenterY, fromCenterX) - chartStartAngle - arcAngle) % (2 * Math.PI);;
                        if (clickAngle < 0) clickAngle = 2 * Math.PI + clickAngle;

                        pointData = [];
                        for (var i = 0; i < regionData.length; i++) {
                            var pointStartAngle = parseFloat(regionData[i].StartAngle.toFixed(14));
                            var pointEndAngle = parseFloat(regionData[i].EndAngle.toFixed(14));
                            pointStartAngle = pointStartAngle < 0 ? 2 * Math.PI + pointStartAngle : pointStartAngle;
                            pointEndAngle = pointEndAngle < 0 ? 2 * Math.PI + pointEndAngle : pointEndAngle;
                            pointStartAngle -= arcAngle;
                            pointEndAngle -= arcAngle;
                            pointStartAngle = parseFloat(pointStartAngle.toFixed(14));
                            pointEndAngle = parseFloat(pointEndAngle.toFixed(14));
                            if (series.startAngle >= 0 && series.endAngle <= 0) {
                                pointStartAngle = pointStartAngle <= 0 ? 2 * Math.PI + pointStartAngle : pointStartAngle;
                                pointEndAngle = pointEndAngle < 0 ? 2 * Math.PI + pointEndAngle : pointEndAngle;
                            }
                            if (clickAngle <= pointStartAngle && clickAngle >= pointEndAngle) {
                                pointData.push(regionData[i]);
                                break;
                            }
                        }
                    }  //Clockwise pie and doughnut
                    else {
                        var arcAngle = (startAngle) ? 2 * Math.PI * (chartObj.model.itemCurrentXPos < 0 ? 1 + chartObj.model.itemCurrentXPos : chartObj.model.itemCurrentXPos) : 0;
                        var clickAngle = (Math.atan2(fromCenterY, fromCenterX) - chartStartAngle - arcAngle)%(2*Math.PI);
                        if (clickAngle < 0) clickAngle = 2 * Math.PI + clickAngle;
                        pointData = [];
                        for (var i = 0; i < regionData.length; i++) {
                            var pointStartAngle = parseFloat(regionData[i].StartAngle.toFixed(14));
                            var pointEndAngle = parseFloat(regionData[i].EndAngle.toFixed(14));
                            pointStartAngle = pointStartAngle < 0 ? 2 * Math.PI + pointStartAngle : pointStartAngle;
                            pointEndAngle = pointEndAngle < 0 ? 2 * Math.PI + pointEndAngle : pointEndAngle;
                            pointStartAngle -= arcAngle;
                            pointEndAngle -= arcAngle;
                            pointStartAngle = parseFloat(pointStartAngle.toFixed(14));
                            pointEndAngle = parseFloat(pointEndAngle.toFixed(14));
                            if (series.startAngle <= 0 && (series.endAngle >= 0||series.endAngle==null)) {
                            pointStartAngle = pointStartAngle < 0 ? 2 * Math.PI + pointStartAngle : pointStartAngle;
                            pointEndAngle = pointEndAngle <= 0 ? 2 * Math.PI + pointEndAngle : pointEndAngle;
                            }
                            if (clickAngle >= pointStartAngle && clickAngle <= pointEndAngle) {
                                pointData.push(regionData[i]);
                                break;
                            }
                        }
                    }
                    if (pointData.length > 0) {
                        var isExploded = (series.explodeAll || (series.explodeIndex == pointData[0].PointIndex));
                        if (isExploded) {

                            var fromCenterX1 = currentX - pointData[0].StartX;
                            var fromCenterY1 = currentY - pointData[0].StartY;

                            clickAngle = Math.atan2(fromCenterY1, fromCenterX1) - chartStartAngle;
                            if (clickAngle < 0) clickAngle = 2 * Math.PI + clickAngle;
                            pointData = [];
                            for (var j = 0; j < regionData.length; j++) {
                                if (clickAngle >= regionData[j].StartAngle && clickAngle <= regionData[j].EndAngle)
                                    pointData.push(regionData[j]);
                            }
                            if (pointData.length > 0) {

                                distanceFromCenter = Math.sqrt(Math.pow(Math.abs(fromCenterX1), 2) + Math.pow(Math.abs(fromCenterY1), 2));
                                if (distanceFromCenter <= seriesData.Radius && distanceFromCenter > innerRadius) {
                                    regionData.PointIndex = pointData[0].PointIndex;
                                    region = { region: regionValue, pointData: pointData, location: { x: currentX, y: currentY} };
                                }
                            }
                        }
                        else {
                            distanceFromCenter = Math.sqrt(Math.pow(Math.abs(fromCenterX), 2) + Math.pow(Math.abs(fromCenterY), 2));
                            if (distanceFromCenter <= seriesData.Radius && distanceFromCenter > innerRadius) {
                                regionData.PointIndex = pointData[0].PointIndex;
                                region = { region: regionValue, pointData: pointData, location: { x: currentX, y: currentY} };
                            }
                        }
                    }

                });

                return region;
            }
        }
    },
    isPointInPolygon: function (polygon, point) {
        for (var p = false, i = -1, l = polygon.length, j = l - 1; ++i < l; j = i)
            ((polygon[i].y <= point.y && point.y < polygon[j].y) || (polygon[j].y <= point.y && point.y < polygon[i].y))
            && (point.x < (polygon[j].x - polygon[i].x) * (point.y - polygon[i].y) / (polygon[j].y - polygon[i].y) + polygon[i].x)
            && (p = !p);
        return p;
    },
    mousePosition: function (evt) {
         if (!ej.util.isNullOrUndefined(evt.pageX) && evt.pageX>0)
            return { x: evt.pageX, y: evt.pageY };
        else if (!ej.util.isNullOrUndefined(evt.originalEvent.pageX) && evt.originalEvent.pageX>0)
            return { x: evt.originalEvent.pageX, y: evt.originalEvent.pageY };
         else if (evt.originalEvent.changedTouches != undefined)
         {
             if (!ej.util.isNullOrUndefined(evt.originalEvent.changedTouches[0].pageX) && evt.originalEvent.changedTouches[0].pageX > 0)
                 return { x: evt.originalEvent.changedTouches[0].pageX, y: evt.originalEvent.changedTouches[0].pageY };
         }
         else
             return { x: 0, y: 0 };
         
    },


    calMousePosition: function (e) {
        var matched = jQuery.uaMatch(navigator.userAgent);
        var mouseposition = this.mousePosition(e);
        e.pageX = mouseposition.x;
        e.pageY = mouseposition.y;
        var mouseX, mouseY;
        var browser = matched.browser.toLowerCase();
        if (browser == "webkit" || browser == "mozilla") {
            mouseX = (e.pageX) - $(this.svgObject).parent().offset().left;
            mouseY = (e.pageY) - $(this.svgObject).parent().offset().top;
        } else {
            mouseX = (e.pageX) - $(this.svgObject).offset().left;
            mouseY = (e.pageY) - $(this.svgObject).offset().top;
        }

        return { X: mouseX, Y: mouseY };

    },

    cancelEvent: function (e) {
        e.stopPropagation();
        var matched = jQuery.uaMatch(navigator.userAgent);
        if(matched.browser == 'chrome' && (this.model.zooming.enable||this.model.crosshair.visible) ){
            e.preventDefault();
        }
        if (this.model.enable3D) {
            e.preventDefault();
            e.stopPropagation();
        }
        e.returnValue = false;
        e.cancelBubble = true;
        return false;
    },
    resetZoom: function () {
        var chartobj = this;
        var id = "#" + chartobj._id;
        if (chartobj.zoomed) {
            $.each(this.model.axes, function (index, axis) {
                axis.zoomFactor = 1;
                axis.zoomPosition = 0;
            });
			$("#"+chartobj._id + "_canvas").css({ "cursor": "default" });
			$("#" + chartobj._id + "_ResetZoom").remove();
			$("#" + chartobj._id + "_PanBtn").remove();
			$("#" + chartobj._id + "_ZoomBtn").remove();
			    
            $(id + "_ResetZoom").remove();
            $(id + "_PanBtn").remove();
            $(id + "_ZoomBtn").remove();
            chartobj.zoomed = false;
            chartobj.zooming = false;
            chartobj.panning = false;
            chartobj.resetZooming = true;
            chartobj.redraw();
            chartobj.enableAnimation();
            chartobj.enableTrackBall();
            chartobj.svgRenderer._setAttr($(chartobj.svgObject).find(id + "_XAxis," + id + "_ChartArea," + id + "_YAxis," + id + "_SeriesCollection"), { "cursor": "default" });
        }
    },
    startZoom: function () {
        var chartObj = this;
        var id = "#" + chartObj._id;
        var iconColor = this.model.theme.toLowerCase().indexOf("dark") >= 0 ? "#000000" : " #FFFFFF";
        chartObj.svgRenderer._setAttr($(id + '_ZoomBtn'), { 'selected': 'true' });
        chartObj.svgRenderer._setAttr($(id + '_ZoomPath'), { "fill": "#14B9FF" });
        chartObj.svgRenderer._setAttr($(id + '_ZoomIcon'), { "fill": '#FFFFFF' });
        chartObj.svgRenderer._setAttr($(id + '_PanPath'), { "fill": '#B7B7B7' });
        chartObj.svgRenderer._setAttr($(id + '_panIcon'), { "fill": iconColor });
        $(id + "_ZoomBtn").css({ "border-color": "#14B9FF", "background-color": "#14B9FF" });
        $(id + "_PanBtn").css({ "border-color": "#B7B7B7", "background-color": "#B7B7B7" });
		$("#"+chartObj._id + "_canvas").css({ "cursor": "default" });
        this._cursorToDefault();
        chartObj.panning = false;
        chartObj.enableTrackBall();
    },
    startPan: function () {
        var chartObj = this;
        var id = "#" + chartObj._id;
        var iconColor = this.model.theme.toLowerCase().indexOf("dark") >= 0 ? "#000000" : " #FFFFFF";
        chartObj.svgRenderer._setAttr($(id + '_ZoomBtn'), { 'selected': 'false' });
        chartObj.svgRenderer._setAttr($(id + '_PanBtn'), { 'selected': 'true' });
        chartObj.svgRenderer._setAttr($(id + '_ZoomPath'), { "fill": "#B7B7B7" });
        chartObj.svgRenderer._setAttr($(id + '_ZoomIcon'), { "fill": iconColor });
        chartObj.svgRenderer._setAttr($(id + '_PanPath'), { "fill": '#14B9FF' });
        chartObj.svgRenderer._setAttr($(id + '_panIcon'), { "fill": '#FFFFFF' });
        $(id + "_ZoomBtn").css({ "border-color": "#B7B7B7", "background-color": "#B7B7B7" });
        $(id + "_PanBtn").css({ "border-color": "#14B9FF", "background-color": "#14B9FF" });
		$("#"+chartObj._id + "_canvas").css({ "cursor": "pointer" });
        this._cursorToPointer();
        chartObj.panning = true;
        chartObj.disableTrackBall();

    },
    zoomButton: function () {
        var padding = 20;
        var transX = this.model.m_AreaBounds.X + (this.model.m_AreaBounds.Width - (3 * 32) - padding); //32 path bouding box for pan,reset, zoom button 
        var transY = (this.model.m_AreaBounds.Y + this.model.m_AreaBounds.Height) + (this.model.elementSpacing - (this.model.m_AreaBounds.Height));
        var iconColor = this.model.theme.toLowerCase().indexOf("dark") >= 0 ? "#000000" : " #FFFFFF";
        var gEle = this.svgRenderer.createGroup({ 'id': this._id + '_ZoomBtn', 'transform': 'translate(' + transX + ',' + transY + ')', 'selected': 'true' });

        var pathOptions = {
            'id': this._id + '_ZoomPath',
            'd': "M32,27c0,2.75-2.25,5-5,5H5c-2.75,0-5-2.25-5-5V5c0-2.75,2.25-5,5-5h22c2.75,0,5,2.25,5,5V27z",
            'fill': this.panning ? '#b7b7b7' : '#14B9FF',
            'zoomId': this._id + '_ZoomBtn',
            'width': 27,
            'height': 27,
            'left': transX + 20,
            'top': transY
        };
        if (this.model.enableCanvasRendering) {
            this.svgRenderer.zoomButton(pathOptions);
            this._on($("#" + this._id + '_ZoomBtn'), "touchstart click", this.startZoom);
        } else {
            this.svgRenderer.drawPath(pathOptions, gEle);

            var options = {
                'id': this._id + '_ZoomIcon',
                'd': "M26.101,22.893l-6.605-6.174c1.414-2.781,0.897-6.267-1.496-8.504c-2.901-2.711-7.448-2.56-10.161,0.341    c-2.712,2.9-2.56,7.45,0.341,10.163c2.426,2.266,6,2.523,8.694,0.853l6.579,6.151L26.101,22.893z M10.186,16.571    c-1.715-1.604-1.805-4.293-0.203-6.009c1.605-1.715,4.295-1.805,6.009-0.201c1.715,1.603,1.805,4.293,0.202,6.007    C14.59,18.084,11.901,18.175,10.186,16.571",
                'fill': this.panning ? iconColor : '#FFFFFF'
            };
            this.svgRenderer.drawPath(options, gEle);
            this.svgRenderer.append(gEle, this.svgObject);

            this._on($(gEle), "touchstart click", this.startZoom);
        }
    },
    panButton: function () {

        var padding = 15;
        var transX = this.model.m_AreaBounds.X + (this.model.m_AreaBounds.Width - (2 * 32) - padding); //32 path bounding box for pan and resetzoom button
        var transY = (this.model.m_AreaBounds.Y + this.model.m_AreaBounds.Height) + (this.model.elementSpacing - (this.model.m_AreaBounds.Height));
        var iconColor = this.model.theme.toLowerCase().indexOf("dark") >= 0 ? "#000000" : " #FFFFFF";
        var gEle = this.svgRenderer.createGroup({ 'id': this._id + '_PanBtn', 'transform': 'translate(' + transX + ',' + transY + ')', 'selected': 'false' });

        var pathOptions = {
            'id': this._id + '_PanPath',
            'd': "M 32 27 c 0 2.75 -2.25 5 -5 5 H 5 c -2.75 0 -5 -2.25 -5 -5 V 5 c 0 -2.75 2.25 -5 5 -5 h 22 c 2.75 0 5 2.25 5 5 V 27 Z",
            'fill': !this.panning ? '#b7b7b7' : '#14B9FF',
            'panId': this._id + '_PanBtn',
            'width': 27,
            'height': 27,
            'left': transX + 10,
            'top': transY
        };
        if (this.model.enableCanvasRendering) {
            this.svgRenderer.panButton(pathOptions);
            this._on($("#" + this._id + '_PanBtn'), "touchstart click", this.startPan);
        } else {
            this.svgRenderer.drawPath(pathOptions, gEle);

            var options = {
                'id': this._id + '_panIcon',
                'points': "26.105,16 21.053,12.211 21.053,14.737 17.263,14.737 17.263,10.947 19.834,10.947 16.044,5.895 12.255,10.947 14.737,10.947 14.737,14.737 10.947,14.737 10.947,12.211 5.895,16 10.947,19.789 10.947,17.263 14.737,17.263 14.737,21.053 12.255,21.053 16.044,26.105 19.834,21.053 17.263,21.053 17.263,17.263 21.053,17.263 21.053,19.789",
                'fill': !this.panning ? iconColor : '#FFFFFF'
            };
            this.svgRenderer.drawPolygon(options, gEle);
            this.svgRenderer.append(gEle, this.svgObject);

            this._on($(gEle), "touchstart click", this.startPan);

        }
    },
    resetZoomButton: function () {

        var padding = 10;
        var transX = this.model.m_AreaBounds.X + (this.model.m_AreaBounds.Width - (32 + padding)); //32 path bouding box
        var transY = (this.model.m_AreaBounds.Y + this.model.m_AreaBounds.Height) + (this.model.elementSpacing - (this.model.m_AreaBounds.Height));
        var gEle = this.svgRenderer.createGroup({ 'id': this._id + '_ResetZoom', 'transform': 'translate(' + transX + ',' + transY + ')', 'selected': 'false' });
        var iconColor = this.model.theme.toLowerCase().indexOf("dark") >= 0 ? "#000000" : " #FFFFFF";
        var options = {
            'id': this._id + '_ResetZoomPath',
            'd': "M 32 27 c 0 2.75 -2.25 5 -5 5 H 5 c -2.75 0 -5 -2.25 -5 -5 V 5 c 0 -2.75 2.25 -5 5 -5 h 22 c 2.75 0 5 2.25 5 5 V 27 Z",
            'fill': '#b7b7b7',
            'resetZoomId': this._id + '_ResetZoom',
            'width': 27,
            'height': 27,
            'left': transX,
            'top': transY
        };
        if (this.model.enableCanvasRendering) {
            this.svgRenderer.resetZoom(options);
            this._on($("#" +this._id + '_ResetZoom'), "touchstart click", this.resetZoom);
        } else {
           
            this.svgRenderer.drawPath(options, gEle);

            var ptOptions = {
                'id': this._id + '_ptIcon',
                'points': "11.895,18.398 8.061,22.23 5.796,19.967 5.796,26.283 12.112,26.283 9.848,24.018 13.682,20.186",
                'fill': iconColor
            };

            this.svgRenderer.drawPolygon(ptOptions, gEle);
            var pt1Options = {
                'id': this._id + '_ptIcon1',
                'points': "19.691,6.072 21.955,8.337 18.121,12.172 19.908,13.959 23.742,10.123 26.007,12.389 26.007,6.072",
                'fill': iconColor
            };

            this.svgRenderer.drawPolygon(pt1Options, gEle);

            var pt2Options = {
                'id': this._id + '_ptIcon2',
                'points': "11.895,13.958 13.682,12.172 9.848,8.337 12.112,6.072 5.796,6.072 5.796,12.389 8.061,10.123",
                'fill': iconColor
            };

            this.svgRenderer.drawPolygon(pt2Options, gEle);

            var pt3Options = {
                'id': this._id + '_ptIcon3',
                'points': "19.908,18.396 18.121,20.186 21.955,24.018 19.691,26.283 26.007,26.283 26.007,19.967 23.741,22.23",
                'fill': iconColor
            };

            this.svgRenderer.drawPolygon(pt3Options, gEle);

            this.svgRenderer.append(gEle, this.svgObject);

            this._on($(gEle), "touchstart click", this.resetZoom);
        }

    },

    doZoom: function (zoomRect, zoomRectWidth, zoomRectHeight) {
        var chart = this;
        if (chart.model.enableCanvasRendering) {
            var zoomArea = $("#" + this.svgObject.id + "_ZoomArea")[0].getClientRects()[0];
            var chartArea = $("#" + this.svgObject.id)[0].getClientRects()[0];
            var zoomX = zoomArea.left - chartArea.left - $(document).scrollLeft(); 
            var zoomY = zoomArea.top - chartArea.top - $(document).scrollTop(); 
        } else {
            var zoomX = parseFloat(chart.svgRenderer._getAttrVal($(zoomRect), "x"));
            var zoomY = parseFloat(chart.svgRenderer._getAttrVal($(zoomRect), "y"));
        }
        $("#" + this.svgObject.id + "_ZoomArea").remove();
        this.drag = false;
        this.zoomed = true;
        for (var k = 0; k < this.model.axes.length; k++) {
            var axis = this.model.axes[k];
            var previousZoomFactor = axis.zoomFactor;
            var previousZoomPosition = axis.zoomPosition;
            if (axis.orientation.toLowerCase() == "horizontal") {
                axis.zoomFactor = chart.model.zooming.type.toLowerCase() != "y"
                    ? previousZoomFactor * (zoomRectWidth / (chart.model.m_AreaBounds.Width)) : 1;
                axis.zoomPosition = chart.model.zooming.type.toLowerCase() != "y"
                    ? previousZoomPosition + Math.abs((zoomX - chart.model.m_AreaBounds.X) / (chart.model.m_AreaBounds.Width)) * previousZoomFactor : 0;
                if (parseFloat(axis.zoomFactor.toFixed(3)) <= 0.001) {
                    axis.zoomFactor = previousZoomFactor;
                    axis.zoomPosition = previousZoomPosition;

                }
            } else {
                axis.zoomFactor = chart.model.zooming.type.toLowerCase() != "x"
                    ? previousZoomFactor * zoomRectHeight / chart.model.m_AreaBounds.Height : 1;
                axis.zoomPosition = chart.model.zooming.type.toLowerCase() != "x"
                    ? previousZoomPosition + (1 - Math.abs((zoomRectHeight + (zoomY - chart.model.m_AreaBounds.Y)) / (chart.model.m_AreaBounds.Height))) * previousZoomFactor : 0;
                if (parseFloat(axis.zoomFactor.toFixed(3)) <= 0.001) {
                    axis.zoomFactor = previousZoomFactor;
                    axis.zoomPosition = previousZoomPosition;

                }
            }
        }

        this.disableAnimation();
        this.model.legendCollapsed = false;
        this.redraw(true);

        this._enableZoomingButtons();
    },
    _cursorToDefault: function () {
        var id = "#" + this.svgObject.id;
        this.svgRenderer._setAttr($(this.svgObject).find(id + "_XAxis," + id + "_ChartArea," + id + "_YAxis," + id + "_SeriesCollection," + id + "_StriplineBehind," + id + '_StriplineOver'), { "cursor": "default" });
    },
    _cursorToPointer: function () {
        var id = "#" + this.svgObject.id;
        this.svgRenderer._setAttr($(this.svgObject).find(id + "_XAxis," + id + "_ChartArea," + id + "_YAxis," + id + "_SeriesCollection," + id + "_StriplineBehind," + id + '_StriplineOver'), { "cursor": "pointer" });
    },
    chartMouseUp: function (evt) {
	   // this.cancelEvent(evt);
        var chart = this;
        if (this.model.zooming.enable && this.model.AreaType == "cartesianaxes") {
            var zoomRect = $("#" + this.svgObject.id + "_ZoomArea");
            if (zoomRect[0]) {
                if (this.model.enableCanvasRendering) {
                    var zoomArea = zoomRect[0].getClientRects()[0];
                    var zoomRectWidth = zoomArea.width;
                    var zoomRectHeight = zoomArea.height;
                } else {
                    var $zoomRect = $(zoomRect);
                    var zoomRectWidth = parseFloat(this.svgRenderer._getAttrVal($zoomRect, "width"));
                    var zoomRectHeight = parseFloat(this.svgRenderer._getAttrVal($zoomRect, "height"));
                }
            }
            if (zoomRectWidth > 0 && zoomRectHeight > 0) {
                chart.doZoom(zoomRect, zoomRectWidth, zoomRectHeight);
            }
            else
                this.drag = false;
            if (!chart.panning)
                this.enableTrackBall();
        }
       // Panning is done on mouse up for other devices and when deferredZoom is set to true
        var id = this.svgObject.id;
        var parentId = ej.util.isNullOrUndefined(evt.target.parentNode) ? "" : evt.target.parentNode.id;

        //condition to find the buttons
        if (parentId == id + "_ResetZoom" || parentId == id + '_ZoomBtn' || evt.target.id == id + "_ResetZoom" || evt.target.id == id + '_ZoomBtn')
            this.panning = false;

        //Panning for other devices
        if ((this.isDevice()  || this.model.zooming.enableDeferredZoom) && this.panning && this.doPan && this.model.AreaType == "cartesianaxes") {
            var oDelta;
            oDelta = {
                'x': this.oPreviousCoords.x - this.currentPageX,
                'y': this.oPreviousCoords.y - this.currentPageY
            };

            this.oPreviousCoords = {
                'x': this.currentPageX,
                'y': this.currentPageY
            };
            $.each(this.model.axes, function (index, axis) {
                var currentScale = Math.max(1 / ej.EjSvgRender.utils._minMax(axis.zoomFactor, 0, 1), 1);
                chart.translate(axis, (oDelta.x), (oDelta.y), currentScale);
            });
            this.model.legendCollapsed = false;
            this.redraw();
            this._cursorToPointer();
            this._enableZoomingButtons();

        }
        
        if (this.doPan)
            this.doPan = false;
        this.rotateActivate = false;
    },

    chartMouseDown: function (e) {
	     this.cancelEvent(e);
        if (!this.svgRenderer.vmlNamespace) {
            this.oPreviousCoords = {};
            if (this.svgRenderer._getAttrVal($(e.target).parent(), "id") != this.svgObject.id + "_ResetZoom" && this.svgRenderer._getAttrVal($(e.target).parent(), "id") != this.svgObject.id + "_PanBtn" && this.model.zooming.enable && !(this.panning) && this.model.AreaType == "cartesianaxes") {
                var mouseDownCords = this.calMousePosition(e);
                this.mouseDownX = mouseDownCords.X;
                this.mouseDownY = mouseDownCords.Y;
                if (this.mouseDownX >= this.model.m_AreaBounds.X && this.mouseDownX < (this.model.m_AreaBounds.X + this.model.m_AreaBounds.Width) && this.mouseDownY < (this.model.m_AreaBounds.Y + this.model.m_AreaBounds.Height) && this.mouseDownY >= this.model.m_AreaBounds.Y) {
                    this.drag = true;
                    this.disableTrackBall();
                }


            }
            else if (this.panning) {
                var mousePanCords = this.calMousePosition(e);
                this.mousePanX = mousePanCords.X;
                this.mousePanY = mousePanCords.Y;
                if (this.mousePanX >= this.model.m_AreaBounds.X && this.mousePanX < (this.model.m_AreaBounds.X + this.model.m_AreaBounds.Width) && this.mousePanY < (this.model.m_AreaBounds.Y + this.model.m_AreaBounds.Height) && this.mousePanY >= this.model.m_AreaBounds.Y) {
                    this.doPan = true;
                    this.oPreviousCoords = {
                        'x': e.pageX,
                        'y': e.pageY
                    };
                }
            }


        }
        if (this.model.enable3D && this.model.enableRotation)
            this.rotateActivate = true;
    },
    chartMouseLeave: function (evt) {
        this.cancelEvent(evt);
        // Changes the visibility to hidden
        $("#" + this.svgObject.id).find("#" + this.svgObject.id + "_TrackToolTip").attr('visibility','hidden');
        if (!($(".tooltipDiv").hasClass(this.svgObject.id)))
            $(".tooltipDiv").remove();
        this._removeTrackBall();
        
    },
    chartTouchClick: function (evt) {
	    this.cancelEvent(evt);
         evt = evt.originalEvent.touches[0];
        this._doClick(evt);
    },
    _doClick: function (evt) {
        var seriesIndex;
        var chartSeries;
        for (var i = 0; i < this.model._visibleSeries.length; i++) {
            seriesIndex = i;
            chartSeries = this.model._visibleSeries[i];
            if (this.model.AreaType == "cartesianaxes" && chartSeries.visibility.toLowerCase() == 'visible' && chartSeries.type.toLowerCase() !== "scatter" && chartSeries.type.toLowerCase() !== "bubble" && chartSeries.type.toLowerCase() !== "column" && chartSeries.type.toLowerCase() !== "bar" && chartSeries.type.toLowerCase() !== "stackingbar" && chartSeries.type.toLowerCase() !== "stackingcolumn" && !chartSeries._hiloTypes) {
                var serY = [];
                var serX = [];
                var location = null;
                var mouseMoveCords = this.calMousePosition(evt);
                this.mousemoveX = mouseMoveCords.X;
                this.mousemoveY = mouseMoveCords.Y;
                var closestXyPoint = this.getClosesPointXY(serX, serY, chartSeries, this.mousemoveX, this.mousemoveY);
                if (!ej.util.isNullOrUndefined(closestXyPoint.point)) {
                    var commonPointEventArgs = $.extend({}, ej.EjSvgRender.commonChartEventArgs);
                    commonPointEventArgs.data = { location: {x: this.mousemoveX, y: this.mousemoveY}, region: { SeriesIndex: i ,Region: { PointIndex: closestXyPoint.index } } };
                    this._trigger("pointRegionClick", commonPointEventArgs);
                }
            }
        }
        var data = this.GetSeriesPoint(evt);
        if (data) {        
           var commonEventArgs = $.extend({}, ej.EjSvgRender.commonChartEventArgs);
           commonEventArgs.data = data;
           this._trigger("pointRegionClick", commonEventArgs); 
        }
        if (this.model.AreaType == "none" && this.model.enable3D && $(evt.target)[0].nodeName == "path") {
            index = $(evt.target)[0].id.match(/(\d+)/g);
            index = parseInt(index[index.length - 1]);
            var currentexplodeindex = this.model._visibleSeries[0].explodeIndex;
            if (currentexplodeindex == index)
                this.model._visibleSeries[0].explodeIndex = null;           
            else
                this.model._visibleSeries[0].explodeIndex = index;                                                                 
            $(this.svgObject).empty();
            
            this.draw();			 
        }
        var legenddata = this.getLegendData(evt);
        if (legenddata) {
            $('#template_group_' + this._id).remove();
            var commonEventArgsCommonEventArgs = $.extend({}, ej.EjSvgRender.commonChartEventArgs);
            commonEventArgsCommonEventArgs.data = legenddata;
            this._trigger("legendItemClick", commonEventArgsCommonEventArgs);
            if (this.model.AreaType != "none") {
                legenddata.series.visibility = legenddata.series.visibility.toLowerCase() === 'visible' ? 'hidden' : 'visible';
                this.model.series[legenddata.legendItem.LegendItem.SeriesIndex].visibility=legenddata.series.visibility
            }
            else {
                var point = legenddata.series.points[legenddata.legendItem.LegendItem.PointIndex].visible;
                legenddata.series.points[legenddata.legendItem.LegendItem.PointIndex].visible = point == true ? false : true;
            }
            if(this.model.enableCanvasRendering){
            /** Canvas Chart image is cleared when click the legend item to show/hide the series**/
            var chartRect = document.getElementById(this._id).getClientRects()[0];
            this.svgRenderer.ctx.clearRect(0, 0, chartRect.width, chartRect.height);
            //Canvas series marker element removed
        }
            $("#" + this._id).find('[id*="canvas_symbol"]').remove();
           
                $(this.svgObject).empty();
            	$(this.legendSvgContainer).empty();
                for (var j = 0; j < this.model.series.length; j++)
                    this.model.series[j].regionAdded = false;
                this.model.legendCollapsed = true;                
                this.disableAnimation();
                this.draw();
                this.enableAnimation();
                if (this.model.AreaType != "none" && this.zoomed)
                    this._enableZoomingButtons();
            
        }
    },
    chartMouseClick: function (evt) {
	
	this.cancelEvent(evt);
        if (window.navigator.msPointerEnabled) {
            evt = evt.originalEvent;
        }
        this._doClick(evt);
    },

    drawTrackerSymbol: function (series, seriesIndex, ptIndex, tracker, point, id) {

        var type = series.type.toLowerCase();
        var chartSeriesObj = new ej.seriesTypes[type]();

        var trans = ej.EjSvgRender.utils._getTransform(series.xAxis, series.yAxis, this.model.requireInvertedAxes);
        if (this.model.AreaType != "none") {
            if (this.model.AreaType == 'cartesianaxes' && ((point.X + trans.x) <= (trans.x + trans.width)) && ((point.X + trans.x) >= trans.x || point.X == 0) &&
                 (Math.abs(point.Y + trans.y) >= (trans.y) || point.Y == 0)) {
                if (this.model.crosshair.visible || type == "bubble" || type == "scatter") {
                    if ($(this.svgObject).find("#" + this.svgObject.id + '_TrackSymbolClipRect' + '_' + seriesIndex).length == 0 && !this.vmlRendering) {
                        var clipRectOptions = {
                            'id': this.svgObject.id + '_TrackSymbolClipRect' + '_' + seriesIndex,
                            'x': 0,
                            'y': 0,
                            'width': series.xAxis.width,
                            'height': series.yAxis.height,
                            'fill': 'white',
                            'stroke-width': 1,
                            'stroke': 'Gray'
                    };

                    this.svgRenderer.drawClipPath(clipRectOptions, this.svgObject);
                    }
                    $('#' + this.svgObject.id + '_TrackSymbol' + '_' + seriesIndex).remove();
                    var trackId = "canvas_trackSymbol_" +seriesIndex;
                    $("#" + this._id).find('[id*=' +trackId + ']').remove();
                    var transSymbolOptions = { 'clip-path': 'url(#' + this.svgObject.id + '_TrackSymbolClipRect' + '_' + seriesIndex +')', 'id': this.svgObject.id + '_TrackSymbol' + '_' + seriesIndex, 'transform': 'translate(' + trans.x + ',' + trans.y + ')' };
                    this.gTrackerEle = this.svgRenderer.createGroup(transSymbolOptions);
                }
            }
            else {
                var transSymbolOptions = { 'id': this.svgObject.id + '_TrackSymbol' + '_' + seriesIndex };
                this.gTrackerEle = this.svgRenderer.createGroup(transSymbolOptions);
            }

            if (this.model.crosshair.marker.visible && !series.isIndicator)
                chartSeriesObj.drawSymbol(seriesIndex, series, ptIndex, (point.X), (point.Y), this, tracker);
            this.svgRenderer.append(this.gTrackerEle, this.svgObject);
        }
            
        
    },
    getClosesPointXY: function (serX, serY, series, x, y) {
        var closestPoint;
        var valAxis = ej.EjSvgRender.utils._getTransform(series.xAxis, series.yAxis, this.model.requireInvertedAxes);
        var mousePoint = ej.EjSvgRender.utils._getValuebyPoint(Math.abs(x - valAxis.x), Math.abs(y - (valAxis.y + valAxis.height)), series);
        var pointX = mousePoint.PointX;
        var pointY = mousePoint.PointY;
        var ptIndex;
        var chartPoint;
        var pointIndex;
        if ((y <= (valAxis.y + valAxis.height) && valAxis.y <= y) && (valAxis.x <= x && x <= (valAxis.x + valAxis.width))) {
            for (var i = 0; i < series.points.length; i++) {
                chartPoint = series.points[i];
                pointIndex = i;
                serX.push(chartPoint.X);
                serY.push(chartPoint.YValues[0]);
            }
            var closestX = this.getClosest(serX, pointX);
            var closestY = this.getClosest(serY, pointY);
            if (!ej.util.isNullOrUndefined(closestX) && !ej.util.isNullOrUndefined(closestY)) {
                for (var i = 0; i < series.points.length; i++) {
                    chartPoint = series.points[i];
                    pointIndex = i;
                    // To get the location of the point
                    var loc = ej.EjSvgRender.utils._getPoint(chartPoint, series);
                    var boundsY = this.model.m_AreaBounds.Y;
                    // px difference between the point and mouse position
                     marker = $.extend(true, {}, series.marker, chartPoint.marker);
                     var markerSize = (marker.size) ? (marker.size.height + marker.border.width) * 3 : 0;
                     var width = series.width;
                    var diff = markerSize > width? markerSize : width;
                    if (chartPoint.X == closestX && chartPoint.YValues[0] == closestY && loc.Y + boundsY - diff < y && y < loc.Y + boundsY + diff) {
                        closestPoint = chartPoint;
                        ptIndex = pointIndex;

                    }
                }

            }
        }
        return { point: closestPoint, index: ptIndex };
    },
    getClosestPointX: function (serX, series, x, y) {
        var closestPoint = [];
        var valAxis = ej.EjSvgRender.utils._getTransform(series.xAxis, series.yAxis, this.model.requireInvertedAxes);
        var mousePoint = ej.EjSvgRender.utils._getValuebyPoint(Math.abs(x - valAxis.x), Math.abs(y - (valAxis.y + valAxis.height)), series);
        var pointX = mousePoint.PointX;
        var ptIndex = [];
        $.each(series.points, function (pointIndex, chartPoint) {
            serX.push(chartPoint.X);
        });
        var closest = this.getClosest(serX, pointX);
        if (!ej.util.isNullOrUndefined(closest)) {
            $.each(series.points, function (pointIndex, cPoint) {
                if (cPoint.X == closest) {
                    closestPoint.push(cPoint);
                    ptIndex.push(pointIndex);

                }
            });
        }
        return { point: closestPoint, index: ptIndex };
    },
    getClosest: function (obj, val) {
        var closest = null;

        // Work out min and max
        var min = Math.min.apply(null, obj);
        var max = Math.max.apply(null, obj);

        // Only calculate closest if point is within array
        if (val >= min && val <= max + 1) {
            var i;
            for (i = 0; i < obj.length; i++) {
                if (closest == null || Math.abs(obj[i] - val) < Math.abs(closest - val)) {
                    closest = obj[i];
                }
            }
        }
        return closest;
    },
    createTooltip: function (region, evt, series) {
        if(!series)
          series = this.model._visibleSeries[region.SeriesIndex];
         var seriesPoint = series.points[region.Region.PointIndex];
        var point = $.extend(true, {}, seriesPoint);
		var tooltipMargin=10;
        if (point.visible) {
            var tooltip =null;
            var commonEventArgs = $.extend({}, ej.EjSvgRender.commonChartEventArgs);
            commonEventArgs.data = { currentText: tooltip, seriesIndex: region.SeriesIndex, pointIndex: region.Region.PointIndex, series: series };
            this._trigger("toolTipInitialize", commonEventArgs);
            var tooltipdiv = $('.tooltipDiv');
            if ($('.tooltipDiv').length == 0) {
                this.tooltipFirst = true;
                tooltipdiv = $("<div class='tooltipDiv' style='position: absolute; z-index: 13000; display: block;'></div>");
                $(document.body).append(tooltipdiv);
                $('.tooltipDiv').addClass(this.svgObject.id);
            } else {
                $(".tooltipDiv").css("display", "block");
                this.tooltipFirst = false;
            }
            if (this.model.AreaType == "cartesianaxes") {
                if (series.xAxis._valueType.toLowerCase() == "datetime")
                    point.x = (Globalize.format(new Date(point.X), ((ej.util.isNullOrUndefined(series.xAxis.labelFormat)) ? ej.EjSvgRender.utils._dateTimeLabelFormat(series.xAxis.intervalType) : series.xAxis.labelFormat)));
                if (series.xAxis._valueType.toLowerCase() == "category")
                    point.x = ej.EjSvgRender.utils._getLabelContent(region.Region.PointIndex, series.xAxis);
            }
            if (series.tooltip.template != null) {
                var cloneNode = $("#" + series.tooltip.template).clone();
                $('.tooltipDiv')[0].innerHTML = "";
                $(cloneNode).css("display", "block").appendTo(tooltipdiv);
                series.count = 1;
                point.count = 1;
                var seriesColor = this.getSeriesColor(point, region.SeriesIndex, series);
				if (seriesColor)
                    $(tooltipdiv).css("background-color", jQuery.type(seriesColor) == "array" ? seriesColor[0].color : seriesColor);
				else
                   $(tooltipdiv).css("background-color", this.model.pointColors[region.Region.PointIndex]);
				var data = { series: series, point: point };
				if (!ej.util.isNullOrUndefined(commonEventArgs.data.currentText))
				    $(tooltipdiv).html(commonEventArgs.data.currentText);
                else
                $(tooltipdiv).html($(tooltipdiv).html().parseTemplate(data));
            } else {
                $(tooltipdiv).html(commonEventArgs.data.ToolTip);
            }
            var xPos = evt.pageX  + tooltipMargin;
            var yPos = evt.pageY  + tooltipMargin;
            xPos = ((xPos + $(tooltipdiv).width()) < $(window).width()) ? (xPos) : (xPos - $(tooltipdiv).width());
            yPos = ((yPos + $(tooltipdiv).height()) < $(window).height()) ? (yPos) : (yPos - $(tooltipdiv).height());

            if (xPos === undefined || xPos === null)
                xPos = evt.pageX  + tooltipMargin;
            if (yPos === undefined || yPos === null)
                yPos = evt.pageY  + tooltipMargin;
            $(tooltipdiv).css("left", xPos);
            $(tooltipdiv).css("top", yPos);
            var tooltipOptions = series.tooltip;
            if (tooltipOptions.enableAnimation) {
                $(tooltipdiv).css({
                    'transition-property': 'left,top',
                    '-moz-transition-property': 'left,top', /* Firefox 4 */
                    '-webkit-transition-property': 'left,top', /* Safari and Chrome */
                    '-o-transition-property': 'left,top',
                    'transition-duration': tooltipOptions.duration,
                    '-moz-transition-duration': tooltipOptions.duration, /* Firefox 4 */

                    '-webkit-transition-duration': this.tooltipFirst ? '0s' : tooltipOptions.duration, /* Safari and Chrome */
                    '-o-transition-duration': tooltipOptions.duration /* Opera */
                });
            }
        }
    },

    translate: function (axis, translateX, translateY, currentScale) {
        var offset = axis.orientation.toLowerCase() == "horizontal"
                                              ? translateX / this.model.m_AreaBounds.Width / currentScale
                                              : translateY / this.model.m_AreaBounds.Height / currentScale;

        axis.zoomPosition = axis.orientation.toLowerCase() == "horizontal"
            ? ej.EjSvgRender.utils._minMax(axis.zoomPosition + offset, 0, (1 - axis.zoomFactor))
            : ej.EjSvgRender.utils._minMax(axis.zoomPosition - offset, 0, (1 - axis.zoomFactor));
    },
    tooltip: function (chart, evt) {
        var tooltipShowing = false;
        var chartSeries;
        var seriesIndex;
        var data = this.GetSeriesPoint(evt);
        $("#" + "canvas" + "_trackSymbol").remove();
		if (ej.util.isNullOrUndefined(data)) // To hide the tooltip when it is moved on chartArea other than series
            $("#" + chart.svgObject.id + "_TrackToolTip").remove();
		if (!this.model.crosshair.visible && this.model.AreaType == "cartesianaxes") {
		    var indicators = this.model.indicators;
		    var seriesCollection = this.model._visibleSeries;
		    for (var j = 0; j < indicators.length && indicators[j].segment; j++) {
		        seriesCollection = seriesCollection.concat(indicators[j].segment);
		    }

		    for (var i = 0; i < seriesCollection.length; i++) {
		        chartSeries = seriesCollection[i];
                seriesIndex = i;
				
				var type = chartSeries.type.toLowerCase(); // to remove track symbol on mouse move out of the point
				if ((ej.util.isNullOrUndefined(data)) && (type == "bubble" || type == "scatter"))
				    $('#' + chart.svgObject.id + '_TrackSymbol' + '_' + seriesIndex).remove();
				
                if (chartSeries.visibility.toLowerCase() == 'visible' && chartSeries.type.toLowerCase() !== "scatter" && chartSeries.type.toLowerCase() !== "bubble" && chartSeries.type.toLowerCase() !== "column" && chartSeries.type.toLowerCase() !== "bar" && chartSeries.type.toLowerCase() !== "stackingbar" && chartSeries.type.toLowerCase() !== "stackingcolumn" && !chartSeries._hiloTypes) {
                    $("#" + chart.svgObject.id + "_TrackToolTipTemplate_" + seriesIndex).remove();
                    if (!($(".tooltipDiv").hasClass(chart.svgObject.id)))
                        $(".tooltipDiv").remove();
                    $("#" + chart.svgObject.id).find("#" + chart.svgObject.id + "_TrackSymbol" + '_' + seriesIndex).remove();
                    $('#' + chart.svgObject.id + '_TrackSymbol' + '_' + seriesIndex).remove();
                    var serY = [];
                    var serX = [];
                    location = null;
                    var closestXyPoint = chart.getClosesPointXY(serX, serY, chartSeries, chart.mousemoveX, chart.mousemoveY);
                    if (closestXyPoint.point)
                        location = ej.EjSvgRender.utils._getPoint(closestXyPoint.point, chartSeries);
                    var pointData = this.model.prevPoint;
                    if (pointData && pointData.seriesIndex == i && !closestXyPoint.point) {
                        var pointLocation = ej.EjSvgRender.utils._getPoint(pointData.point, pointData.series);
                        chart.drawTrackerSymbol(pointData.series, pointData.seriesIndex, pointData.pointIndex, null, pointLocation);
                        this.model.prevPoint = null;
                    }
                    
                    if (!chartSeries.isIndicator && chartSeries.marker.visible && closestXyPoint.point) {
                        if (ej.util.isNullOrUndefined(closestXyPoint.point.marker) || ej.util.isNullOrUndefined(closestXyPoint.point.marker.visible) || (closestXyPoint.point.marker && closestXyPoint.point.marker.visible))
                        chart.drawTrackerSymbol(chartSeries, seriesIndex, closestXyPoint.index, false, location);

                        if (pointData && closestXyPoint.point != pointData.point){
                            var prevLocation = ej.EjSvgRender.utils._getPoint(pointData.point, pointData.series);
                            chart.drawTrackerSymbol(pointData.series, pointData.seriesIndex, pointData.pointIndex, null, prevLocation, id);
                        }
                        if (ej.util.isNullOrUndefined(closestXyPoint.point.marker) || ej.util.isNullOrUndefined(closestXyPoint.point.marker.visible) || (closestXyPoint.point.marker && closestXyPoint.point.marker.visible))
                        this.model.prevPoint = { point: closestXyPoint.point, pointIndex: closestXyPoint.index, series: chartSeries, seriesIndex: seriesIndex };
                    }


                    if (chartSeries.tooltip.visible && closestXyPoint.point && (!data || (i > data.region.SeriesIndex && !(chartSeries._zOrder < this.model._visibleSeries[data.region.SeriesIndex]._zOrder)))) {
                        trans = ej.EjSvgRender.utils._getTransform(chartSeries.xAxis, chartSeries.yAxis, chart.model.requireInvertedAxes);
                        if ((location.X + trans.x) <= (trans.x + trans.width) && ((location.X + trans.x) >= trans.x || location.X == 0) && (Math.abs(location.Y - (trans.y + trans.height)) <= (trans.y + trans.height))
                                     && (Math.abs(location.Y - (trans.y + trans.height)) >= trans.y || location.Y == 0)) {
                            if ($(chart.svgObject).find("#" + chart.svgObject.id + "_TrackToolTip").length == 0) {
                                var transOptions = { 'id': chart.svgObject.id + '_TrackToolTip', 'transform': 'translate(' + trans.x + ',' + trans.y + ')' };
                                chart.gTransToolEle = chart.svgRenderer.createGroup(transOptions);
                            }
                            chart.svgRenderer._setAttr($(chart.svgObject).find("#" + chart.svgObject.id + "_TrackToolTip"), { 'transform': 'translate(' + trans.x + ',' + trans.y + ')', 'visibility': 'visible' });
                            if (ej.util.isNullOrUndefined(chartSeries.tooltip.template)) {
                                tooltipShowing = true;
                                $(".tooltipDiv").remove();
                                if (this.model.enableCanvasRendering)
                                    chart.displayCanvasTooltip(location, closestXyPoint.point, chartSeries, closestXyPoint.index);
                                else
                                    chart.displayShowTooltip(location, closestXyPoint.point, chartSeries, closestXyPoint.index);
                            }
                            else {
                                tooltipShowing = true;
                                $("#" + chart.svgObject.id).find("#" + chart.svgObject.id + "_TrackToolTip").attr('visibility','hidden');
                                var region = { SeriesIndex: seriesIndex, Region: { PointIndex: closestXyPoint.index } };
                                chart.createTooltip(region, evt, chartSeries);
                            }

                            //Comment the below code for VML -The line series tooltip color not displayed due to attach and detach the element to DOM
                              var detachEle = $(chart.svgObject).find("#" + chart.svgObject.id + "_TrackToolTip").detach();
                             detachEle.appendTo(chart.svgObject);

                            var commonPointEventArgs = $.extend({}, ej.EjSvgRender.commonChartEventArgs);
                             commonPointEventArgs.data = { location: {x: this.mousemoveX, y: this.mousemoveX}, region: { SeriesIndex: i ,Region: { PointIndex: closestXyPoint.index } } };
                            chart._trigger("pointRegionMouseMove", commonPointEventArgs);
                        }
                    }
                }
            }
        }
	   else {
            $("#" + this.svgObject.id).find("#" + this.svgObject.id + "_TrackToolTip").remove();
        }
       
        if (!($(".tooltipDiv").hasClass(this.svgObject.id)))
            $(".tooltipDiv").remove();
        if (data && ej.util.isNullOrUndefined(data.region.isStripLine)) {
            if (this.model.AreaType == "none" && this.model._visibleSeries[data.region.SeriesIndex].explode) {
                if (this.model._visibleSeries[data.region.SeriesIndex].enableAnimation) {
                    if (this.model.AnimationComplete)
                        this.pieExplosion(data, evt);
                }
                else
                    this.pieExplosion(data, evt);
            }
            series = this.model._visibleSeries[data.region.SeriesIndex];
            var seriesPoint = this.model._visibleSeries[data.region.SeriesIndex].points[data.region.Region.PointIndex];

            if (seriesPoint && (series.type.toLowerCase() == "bubble" || series.type.toLowerCase() == "scatter" || this.model.AreaType=='polaraxes')) {
                for (var i = 0; i < this.model._visibleSeries.length; i++) {
                    $("#" + this.svgObject.id).find("#" + this.svgObject.id + "_TrackSymbol" + '_' + i).remove();
                    var trackId = "canvas_trackSymbol_" + i;
                    $("#" + this._id).find('[id*=' + trackId + ']').remove();
                }
                var pointlocation = (this.model.AreaType != "polaraxes")
                    ? ej.EjSvgRender.utils._getPoint(seriesPoint, series)
                    : (ej.EjSvgRender.utils.TransformToVisible(series, seriesPoint.X, seriesPoint.y, this))
               
                if (this.model.AreaType == "cartesianaxes")
                    this.drawTrackerSymbol(series, data.region.SeriesIndex, data.region.Region.PointIndex, true, pointlocation);
                else {
                    if(series.marker.visible && series.drawType!="column")
                    this.drawTrackerSymbol(series, data.region.SeriesIndex, data.region.Region.PointIndex, true, pointlocation);
                }
            }

            if (series.tooltip.visible && !tooltipShowing) {

                if (series.visibility.toLowerCase() == 'visible') {
                    if (ej.util.isNullOrUndefined(series.tooltip.template)) {
                        var location;
                        if (this.model.AreaType == "cartesianaxes") {

                            var trans = ej.EjSvgRender.utils._getTransform(series.xAxis, series.yAxis, this.model.requireInvertedAxes);
                            if ($(this.svgObject).find("#" + this.svgObject.id + "_TrackToolTip").length == 0) {
                                var transToolOptions = { 'id': this.svgObject.id + '_TrackToolTip', 'transform': 'translate(' + trans.x + ',' + trans.y + ')' };
                                this.gTransToolEle = this.svgRenderer.createGroup(transToolOptions);
                            }
                            this.svgRenderer._setAttr($(this.svgObject).find("#" + this.svgObject.id + "_TrackToolTip"), { 'transform': 'translate(' + trans.x + ',' + trans.y + ')' });
                            $('#' + this.svgObject.id + "_TrackToolTip").attr("visibility", "visible");
                            location = { X: (data.location.x - trans.x), Y: Math.abs(data.location.y - (this.model.enableCanvasRendering ? this.model.m_AreaBounds.Y: trans.y)) };
                        } else {
                            if ($(this.svgObject).find("#" + this.svgObject.id + "_TrackToolTip").length == 0) {
                                var transToolTipOptions = { 'id': this.svgObject.id + '_TrackToolTip' };
                                this.gTransToolEle = this.svgRenderer.createGroup(transToolTipOptions);
                            }
                            location = { X: (data.location.x), Y: Math.abs(data.location.y) };
                        }


                        $(".tooltipDiv").remove();
                        if (this.model.enableCanvasRendering)
                            this.displayCanvasTooltip(location, seriesPoint, series, data.region.Region.PointIndex);
                        else
                            this.displayShowTooltip(location, seriesPoint, series, data.region.Region.PointIndex);
                            
                        //Comment the below code for VML -The line series tooltip color not displayed due to attach and detach the element to DOM
                        // if (series.type.toLowerCase() == "bubble" || series.type.toLowerCase() == "scatter") {
                        //     var detachEle = $(this.svgObject).find("#" + this.svgObject.id + "_TrackToolTip").detach();
                        //    detachEle.appendTo(this.svgObject);
                        // }
                    } else {
                        $("#" + chart.svgObject.id).find("#" + chart.svgObject.id + "_TrackToolTip").attr('visibility', 'hidden');
                        this.createTooltip(data.region, evt);
                    }
                }
            }
            var commonEventArgs = $.extend({}, ej.EjSvgRender.commonChartEventArgs);
            commonEventArgs.data = data;
            this._trigger("pointRegionMouseMove", commonEventArgs);
        }
        else {
            if (!ej.util.isNullOrUndefined(this.model.explodeValue)) {
                var type = this.model.series[0].type;
                var series = this.model._visibleSeries[0];
                var seriesType = new ej.seriesTypes[type]();
                var targetElement;
                var id;
                var _labelPosition = series.labelPosition.toLowerCase();
                var result = seriesType._calculateArcData(this, this.model.explodeValue, series.points[this.model.explodeValue], series);
                if (this.model.enableCanvasRendering) {
                    this.model.series[0].explodeIndex = null;
                    var chartRect = document.getElementById(this.svgObject.id).getClientRects()[0];
                    this.svgRenderer.ctx.clearRect(chartRect.left, chartRect.top, chartRect.width, chartRect.height);
                    $("#" + this._id).ejChart("redraw");
                }
                //var explodeId = this.model.series[0].name + "group" + index;
                id = this.svgObject.id + '_SeriesGroup' + '_0';
                targetElement = $(this.gSeriesEle).children('#' + id)[0];
                var elements = $(targetElement).children();
                for (var i = 0; i < elements.length; i++) {
                    var element = elements[i];
                    var index = this.svgRenderer._getAttrVal($(element)[0], "pointIndex");
                    if (parseInt(index) == this.model.explodeValue) {
                        this.svgRenderer._setAttr($(element), { 'd': result.Direction });
                        break;
                    }
                }
               
                if (this.model._visibleSeries[0].marker.dataLabel.visible) {
                    seriesType.drawDataLabelAcc(this, series, this.model.explodeValue, series.points[this.model.explodeValue]);
                }
                this.model.explodeValue = null;
            }
        }


    },

    chartCrossHair: function (chart, mouseLocation) {
        if (this.model.crosshair.visible && this.model.crosshair.type.toLowerCase()=="trackball") {

            var point;
            var seriesIndex;
            var series;
            var indicators = this.model.indicators;
            var seriesCollection = this.model._visibleSeries;
            for (var j = 0; j < indicators.length && indicators[j].segment; j++) {
                seriesCollection = seriesCollection.concat(indicators[j].segment);
            }

            for (var i = 0; i < seriesCollection.length; i++) {
                seriesIndex = i;
                series = seriesCollection[i];
                if (series.visibility.toLowerCase() == 'visible') {
                    var serX = [];
                    var closestPoint;

                    closestPoint = chart.getClosestPointX(serX, series, chart.mousemoveX, chart.mousemoveY);

                    if (closestPoint.point) {
                        for (var t = 0; t < closestPoint.point.length; t++) {
                            if (closestPoint.point[t].visible) {
                                point = ej.EjSvgRender.utils._getPoint(closestPoint.point[t], series);
                                var chartPos = document.getElementById(this.svgRenderer.svgObj.id).getClientRects()[0];
                                var dir = "M" + " " + (point.X + series.xAxis.x) + " " + (chart.model.m_AreaBounds.Y) + " " + "L" + " " + (point.X + series.xAxis.x) + " " + (chart.model.m_AreaBounds.Y + chart.model.m_AreaBounds.Height);

                                var pathOptions = {
                                    'id': chart.svgObject.id + "_Tracker",
                                    'fill': 'none',
                                    'width': 0,
                                    'height': this.model.m_AreaBounds.Height,
                                    'top': this.model.m_AreaBounds.Y + chartPos.top,
                                    'left': this.model.m_AreaBounds.X + chartPos.left,
                                    'stroke-width': chart.model.crosshair.line.width,
                                    'clip-path': 'url(#' + chart.svgObject.id + '_ChartAreaClipRect)',
                                    'stroke': chart.model.crosshair.line.color,
                                    'd': dir
                                };
                                if ($("#" + chart.svgObject.id + "_Tracker").length == 0) {
                                    if (!chart.model.enableCanvasRendering)
                                        chart.svgRenderer.drawPath(pathOptions, chart.svgObject);
                                    else
                                        chart.svgRenderer.drawCrosshairLine(pathOptions, chart.svgObject);
                                }
                                else {
                                    chart.svgRenderer._setAttr($(chart.svgObject).find("#" + chart.svgObject.id + "_Tracker"), { "d": dir });
                                    $("#" + chart.svgObject.id + "_Tracker").css("left", point.X + series.xAxis.x + chartPos.left + $(document).scrollLeft());
                                    $("#" + this._id).find('[id*="canvas_Tracker"]').css("visibility", "visible");
                                }

                                var crossHairTrans = ej.EjSvgRender.utils._getTransform(series.xAxis, series.yAxis, chart.model.requireInvertedAxes);
                                var tToolOptions = { 'transform': 'translate(' + crossHairTrans.x + ',' + crossHairTrans.y + ')', 'id': chart.svgObject.id + '_TrackToolTip' + '_' + seriesIndex };

                                chart.gTransToolEle = chart.svgRenderer.createGroup(tToolOptions);

                                var prePoint = this.model.prePoint;

                                //To check the current closestPoint and previous point is same
                                if (ej.util.isNullOrUndefined(prePoint) || prePoint.pointIndex[0] != closestPoint.index[0] || !$('#' + chart.svgObject.id + "_trackSymbol_" + seriesIndex + '_' + closestPoint.index)) {

                                    //Remove the track elements only once on point change
                                    if (ej.util.isNullOrUndefined(initialPoint)) {
                                        if (this.model.crosshair.tooltipTemplate != null) {
                                            $(document).find('[id*="_TrackToolTipTemplate_"]').attr("visibility", "hidden");
                                        }
                                        if (chart.gTransToolEle && chart.gTrackerEle) {
                                            $(this.svgObject).find('[id*="_TrackSymbol_"],[id*="_TrackToolTip"]').attr("visibility", "hidden");
                                            $("#" + this._id).find('[id*="canvas_trackSymbol"]').css("visibility", "hidden");
                                        }
                                        var initialPoint = closestPoint;
                                    }


                                    chart.drawTrackerSymbol(series, seriesIndex, closestPoint.index[t], true, point);
                                     
                                    // Condition to find the crosshair is specified with tooltip template
                                    if (!chart.model.crosshair.tooltipTemplate)
                                        chart.displayTooltip(point, closestPoint.point[t], series, closestPoint.index[t]);
                                    else {
                                        var tX, tY;
                                        chart.displayTemplateTooltip(tX, tY, point, closestPoint.point[t], series, seriesIndex, closestPoint.index[t]);
                                    }
                                } else {
                                    // If both are same points, then viibility is set to visible
                                    var element = $(this.svgObject).find('[id*="_TrackSymbol_"],[id*="_TrackToolTip"]');
                                    this.svgRenderer._setAttr($(element), { "visibility": 'visible' });
                                    $("#" + this._id).find('[id*="canvas_trackSymbol"]').css("visibility", "visible");
                                    element = $(this.svgObject).find('[id*="_TrackToolTip"]').children();
                                    this.svgRenderer._setAttr($(element), { "visibility": 'visible' });
                                    
                                    element = $(this.svgObject).find('[id*="_trackSymbol_"]');
                                    this.svgRenderer._setAttr($(element), { "visibility": 'visible' });

                                    $(chart.svgObject).find('[id*="_TrackAxisToolTip"]').attr("visibility", "visible");
                                    
                                    // Sets the tooltip template visibility to visible
                                    if (this.model.crosshair.tooltipTemplate != null) {
                                        element = $(document).find('[id*="_TrackToolTipTemplate_"]');
                                        this.svgRenderer._setAttr($(element), { "visibility": 'visible' });
                                    }
                                }
                            } else {
                                $('#' + this.svgObject.id + '_TrackToolTip_' + seriesIndex).remove();
                                $('#' + this.svgObject.id + '_TrackSymbol_' + seriesIndex).remove();
                            }
                        }
                    } else {
                        $(chart.svgObject).find('[id*="_Tracker"]').remove();
                    }
                }
            }
           // To get the previous point values
            if (this.model.closestPoint != null)
            {
                this.model.prePoint = { point: closestPoint.point, pointIndex: closestPoint.index, series: series, seriesIndex: seriesIndex };
            }
            if (!chart.gTrackAxisEle) {
                var gTrackAxisOptions = { 'id': chart.svgObject.id + '_TrackAxisToolTip', 'visibility': 'visible' };
                chart.gTrackAxisEle = chart.svgRenderer.createGroup(gTrackAxisOptions);
                chart.svgRenderer.append(chart.gTrackAxisEle, chart.svgObject);
            }
           
            if (point) {
                $.each(chart.model.axes, function (axisIndex, axis) {
                    $(chart.svgObject).find('[id*="_TrackAxisToolTip"]').attr("visibility", "visible");
                    var areaX = chart.model.m_AreaBounds.X;

                    if (axis.orientation.toLowerCase() == "horizontal" && axis.crosshairLabel.visible) {
                        if ((point.X + areaX) <= (axis.x + axis.width) && axis.x <= (point.X + areaX)) {
                            var valueType = axis._valueType.toLowerCase();
                            var xVal = ((point.X / (axis.width)) * (axis.visibleRange.delta) + axis.visibleRange.min).toFixed(2).replace(new RegExp("\\.0{" + 2 + "}"), "");
                            if (valueType == "logarithmic") xVal = Math.pow(axis.logBase, xVal);
                            if (valueType == "datetime") xVal = (Globalize.format(new Date(Math.floor(xVal)), ((ej.util.isNullOrUndefined(axis.labelFormat)) ? ej.EjSvgRender.utils._dateTimeLabelFormat(axis.intervalType) : axis.labelFormat)));
                            if (valueType == "category") xVal = ej.EjSvgRender.utils._getLabelContent(Math.floor(xVal), axis);
                            var pointLocation = { X: (point.X + areaX), Y: (axis.opposedPosition) ? (axis.y + axis.height - axis.majorTickLines.size) : (axis.y + axis.height + axis.majorTickLines.size + 10) };
                            chart.displayAxisTooltip(pointLocation, xVal, axis, axisIndex, mouseLocation);
                        }
                    }

                });
            }


        }
    },
    chartAreaCrossHair: function (chart, mouseLocation) {
        var round = ((ej.util.isNullOrUndefined(this.model.primaryYAxis.roundingPlaces)) ? 2 : this.model.primaryYAxis.roundingPlaces);
        if (this.model.crosshair.visible && this.model.crosshair.type.toLowerCase() == "crosshair") {
            var d = "M" + " " + (this.mousemoveX) + " " + (this.model.m_AreaBounds.Y) + " " + "L" + " " + (this.mousemoveX) + " " + (this.model.m_AreaBounds.Y + this.model.m_AreaBounds.Height);
            var dhor = "M" + " " + (this.model.m_AreaBounds.X) + " " + (this.mousemoveY) + " " + "L" + " " + (this.model.m_AreaBounds.X + this.model.m_AreaBounds.Width) + " " + (this.mousemoveY);
            
           
            var chartPos = document.getElementById(this.svgRenderer.svgObj.id).getClientRects()[0];
            if ($("#" + chart.svgObject.id + "_CrosshairVertical").length == 0) {
                var options = {
                    'id': chart.svgObject.id + "_CrosshairVertical",
                    'fill': 'none',
                    'width': 0,
                    'height': this.model.m_AreaBounds.Height,
                    'top': this.model.m_AreaBounds.Y + chartPos.top,
                    'left': this.model.m_AreaBounds.X + chartPos.left,
                    'stroke-width': chart.model.crosshair.line.width,
                    'stroke': chart.model.crosshair.line.color,
                    'clip-path': 'url(#' + this.svgObject.id + '_ChartAreaClipRect)',
                    'd': d
                };
                if (chart.model.enableCanvasRendering)
                    chart.svgRenderer.drawCrosshairLine(options, chart.svgObject);
                else
                    chart.svgRenderer.drawPath(options, chart.svgObject);                    
            } else {
                chart.svgRenderer._setAttr($(chart.svgObject).find("#" + chart.svgObject.id + "_CrosshairVertical"), { "d": d });
                $("#" + chart.svgObject.id + "_CrosshairVertical").css("left", this.mousemoveX + chartPos.left + $(document).scrollLeft());
                $("#" + chart.svgObject.id + "_CrosshairVertical").css("visibility", "visible");
            }
            
            if ($("#" + chart.svgObject.id + "_CrosshairHorizontal").length == 0) {
                var optionsHor = {
                    'id': chart.svgObject.id + "_CrosshairHorizontal",
                    'fill': 'none',
                    'width': this.model.m_AreaBounds.Width,
                    'height': 0,
                    'top': this.model.m_AreaBounds.Y + chartPos.top,
                    'left': this.model.m_AreaBounds.X + chartPos.left,
                    'stroke-width': chart.model.crosshair.line.width,
                    'stroke': chart.model.crosshair.line.color,
                    'clip-path': 'url(#' + this.svgObject.id + '_ChartAreaClipRect)',
                    'd': dhor
                };
                if (chart.model.enableCanvasRendering)
                    chart.svgRenderer.drawCrosshairLine(optionsHor, chart.svgObject);
                else
                    chart.svgRenderer.drawPath(optionsHor, chart.svgObject);
            } else {
                chart.svgRenderer._setAttr($(chart.svgObject).find("#" + chart.svgObject.id + "_CrosshairHorizontal"), { "d": dhor });
                $("#" + chart.svgObject.id + "_CrosshairHorizontal").css("top", this.mousemoveY + chartPos.top + $(document).scrollTop());
                $("#" + chart.svgObject.id + "_CrosshairHorizontal").css("left", this.model.m_AreaBounds.X + chartPos.left + $(document).scrollLeft());
                $("#" + chart.svgObject.id + "_CrosshairHorizontal").css("visibility", "visible");
            }
            if ( $('#' + chart.svgObject.id + '_AxisCrossToolTip').length ==0) {
                var gAxisCrossOptions = { 'id': chart.svgObject.id + '_AxisCrossToolTip', 'visibility': 'visible' };
                chart.gTrackAxisEle = chart.svgRenderer.createGroup(gAxisCrossOptions);
                chart.svgRenderer.append(chart.gTrackAxisEle, chart.svgObject);
            }
            else {
                this.svgRenderer._setAttr($(chart.gTrackAxisEle), { "visibility": 'visible' });
            }

            $.each(chart.model.axes, function (axisIndex, axis) {
                var valueType = axis._valueType.toLowerCase();
                if (axis.orientation.toLowerCase() == "horizontal" && axis.crosshairLabel.visible) {
                    if ((chart.mousemoveX) <= (axis.x + axis.width) && axis.x <= (chart.mousemoveX)) {
                        var labelplacement = (axis._valueType == "category" && axis.labelPlacement.toLowerCase()=='betweenticks') ? 0.5 : 0;
                        var xVal = (((chart.mousemoveX - axis.x) / (axis.width)) * (axis.visibleRange.delta) + axis.visibleRange.min + labelplacement).toFixed(2).replace(new RegExp("\\.0{" + 2 + "}"), "");
                        if (valueType == "logarithmic") xVal = Math.pow(axis.logBase, xVal).toFixed(2).replace(new RegExp("\\.0{" + 2 + "}"), "");
                        if (valueType == "datetime") xVal = (Globalize.format(new Date(Math.floor(xVal)), ((ej.util.isNullOrUndefined(axis.labelFormat)) ? ej.EjSvgRender.utils._dateTimeLabelFormat(axis.intervalType) : axis.labelFormat)));
                        if (valueType == "category") xVal = ej.EjSvgRender.utils._getLabelContent(xVal, axis);
                        var xPointLocation = { X: (chart.mousemoveX), Y: (axis.y) };
                        chart.displayAxisTooltip(xPointLocation, xVal, axis, axisIndex, mouseLocation);
                    }
                } else if (axis.orientation.toLowerCase() == "vertical" && axis.crosshairLabel.visible) {
                    if ((chart.mousemoveY) <= (axis.y + axis.height) && axis.y <= (chart.mousemoveY)) {
                        var yVal = ((Math.abs(1 - (chart.mousemoveY - (axis.y)) / (axis.height)) * (axis.visibleRange.delta) + axis.visibleRange.min).toFixed(round).replace(new RegExp("\\.0{" + round + "}"), ""));
                        if (valueType == "logarithmic") yVal = Math.pow(axis.logBase, yVal).toFixed(2).replace(new RegExp("\\.0{" + 2 + "}"), "");
                        if (valueType == "datetime") yVal = (Globalize.format(new Date(Math.floor(yVal)), ((ej.util.isNullOrUndefined(axis.labelFormat)) ? ej.EjSvgRender.utils._dateTimeLabelFormat(axis.intervalType) : axis.labelFormat)));
                        if (valueType == "category") yVal = ej.EjSvgRender.utils._getLabelContent(yVal, axis);
                        var ypointLocation = { X: axis.x, Y: (chart.mousemoveY) };
                        chart.displayAxisTooltip(ypointLocation, yVal, axis, axisIndex, mouseLocation);
                    }
                }
            });

        }
    },
    chartTouchMove: function (evt) {
	   this.cancelEvent(evt);
        evt = evt.originalEvent.touches[0];
        this.chartInteractiveBehavior(evt);

    },

    chartInteractiveBehavior: function (evt) {
        var mouseMoveCords = this.calMousePosition(evt);
        this.mousemoveX = mouseMoveCords.X;
        this.mousemoveY = mouseMoveCords.Y;

        var id = "#" + this.svgObject.id;
        if (this.mouseWheelCoords) {
            var changeX = Math.abs(this.mouseWheelCoords.x - evt.pageX);
            var changeY = Math.abs(this.mouseWheelCoords.y - evt.pageY);
            if ((changeX > 0 || changeY > 0) && !(this.panning)) {
                this.enableTrackBall();
            } else {
                this.disableTrackBall();
            }
        }

        var chart = this;

        if (chart.model.AreaType == "cartesianaxes" && this.mousemoveX >= this.model.m_AreaBounds.X && this.mousemoveX < (this.model.m_AreaBounds.X + this.model.m_AreaBounds.Width) && this.mousemoveY < (this.model.m_AreaBounds.Y + this.model.m_AreaBounds.Height) && this.mousemoveY >= this.model.m_AreaBounds.Y && this.model.initSeriesRender) {
            var mouseLocation = { x: chart.mousemoveX, y: chart.mousemoveY };
            var targetId = chart.svgRenderer._getAttrVal($(evt.target).parent(), "id");
            if ((targetId != undefined) && (targetId == chart.svgRenderer._getAttrVal($(this.svgObject).find(id + '_ZoomBtn'), "id") || targetId == chart.svgRenderer._getAttrVal($(this.svgObject).find(id + '_ResetZoom'), "id") || targetId == chart.svgRenderer._getAttrVal($(this.svgObject).find(id + '_PanBtn'), "id"))) {
                this.disableTrackBall();
            }
            else {
                this.enableTrackBall();
            }

            //show tooltip and track ball
			if(!this.model.crosshair.visible)
            this.tooltip(chart, evt);

            //crossHair of chart
            this.chartCrossHair(chart, mouseLocation);

            //crossHair for chart area
            this.chartAreaCrossHair(chart, mouseLocation);

        } else {
            if (chart.model.AreaType == "none" || chart.model.AreaType=='polaraxes')
                this.tooltip(chart, evt);
            else {
                $("#" + chart.svgObject.id).find("#" + chart.svgObject.id + "_TrackToolTip").attr('visibility','hidden');
                $("#" + chart.svgObject.id + "_TrackToolTip").remove();
                $(".tooltipDiv").css("display","none");
                this._removeTrackBall();
            }
        }
    },

    chartMouseMove: function (evt) {
	 
	   this.cancelEvent(evt);
	   
	     if (this.rotateActivate && this.oPreviousCoords) {
            var difY = (this.oPreviousCoords.Y - this.mousemoveY);
            var difX = (this.oPreviousCoords.X - this.mousemoveX);          
            if (difX||difY) {
                this.model.tilt -= difY;
                this.model.rotation += difX;     
                $(this.chart3D).empty();              
                var size = { Width: $(this.svgObject).width(), Height: $(this.svgObject).height() };
                this.graphics.view(this.svgObject, this, this.model.rotation, this.model.tilt, size, this.model.perspectiveAngle, this.model.depth);
                this.svgRenderer.append(this.chart3D, this.svgObject);
                this.svgRenderer.append(this.gTransToolEle, this.svgObject);
            }
            this.oPreviousCoords.Y = this.mousemoveY;
            this.oPreviousCoords.X = this.mousemoveX;
        }
		 
        var chart = this;
        this.calMousePosition(evt)
		$('#tooltip').remove();
        if (!this.panning) {
            if (!this.drag)
                this.chartInteractiveBehavior(evt);
            var id = "#" + this.svgObject.id;
            if (this.model.zooming.enable && this.model.AreaType == "cartesianaxes" && !this.model.enable3D) {
                if (this.drag) {
                    this.disableTrackBall();
                    $('#' + this.svgObject.id + '_ZoomArea').remove();
                    var width, height, x, y;
                    if (this.model.zooming.type.toLowerCase() == "x") {
                        width = Math.abs(this.mousemoveX - this.mouseDownX);
                        height = this.model.m_AreaBounds.Height;
                        x = this.mousemoveX > this.mouseDownX ? this.mouseDownX : this.mousemoveX;
                        y = this.model.m_AreaBounds.Y;
                    } else if (this.model.zooming.type.toLowerCase() == "y") {

                        width = this.model.m_AreaBounds.Width;
                        height = Math.abs(this.mousemoveY - this.mouseDownY);
                        x = this.model.m_AreaBounds.X;
                        y = this.mousemoveY > this.mouseDownY ? this.mouseDownY : this.mousemoveY;

                    } else {
                        width = Math.abs(this.mousemoveX - this.mouseDownX);
                        height = Math.abs(this.mousemoveY - this.mouseDownY);
                        x = this.mousemoveX > this.mouseDownX ? this.mouseDownX : this.mousemoveX;
                        y = this.mousemoveY > this.mouseDownY ? this.mouseDownY : this.mousemoveY;

                    }

                    $(this.svgObject).css({
                        '-moz-user-select': '-moz-none',
                        '-khtml-user-select': 'none',
                        '-webkit-user-select': 'none',
                        '-ms-user-select': 'none',
                        'user-select': 'none'
                    });

                    this.zooming = true;

                    var rectOptions = {
                        'id': this.svgObject.id + '_ZoomArea',
                        'x': x,
                        'y': y,
                        'width': width,
                        'height': height,
                        'fill': 'rgba(69,114,167,0.25)',
                        'stroke-width': 1,
                        'stroke': 'rgba(69,114,167,0.25)',
                        'clip-path': 'url(#' + this.svgObject.id + '_ChartAreaClipRect)'
                    };
                    if (this.model.enableCanvasRendering)
                        this.svgRenderer.drawZoomRect(rectOptions, this);
                    else
                        this.svgRenderer.drawRect(rectOptions, this.svgObject);
                }

            }
        }
		var deferredZoom = this.model.zooming.enableDeferredZoom;
        // Panning
        if (this.panning && this.doPan && this.model.AreaType == "cartesianaxes") {
            this.currentPageX = evt.pageX;
            this.currentPageY = evt.pageY;

            if (!this.isDevice() && !deferredZoom) {
                var oDelta;
                oDelta = {
                    'x': this.oPreviousCoords.x - evt.pageX,
                    'y': this.oPreviousCoords.y - evt.pageY
                };

                this.oPreviousCoords = {
                    'x': evt.pageX,
                    'y': evt.pageY
                };
                $.each(this.model.axes, function(index, axis) {
                    var currentScale = Math.max(1 / ej.EjSvgRender.utils._minMax(axis.zoomFactor, 0, 1), 1);
                    chart.translate(axis, (oDelta.x), (oDelta.y), currentScale);
                });

                this.redraw(true);
                this._cursorToPointer();
                this._enableZoomingButtons();
            }
            // Translate the series for other devices
            else if (this.isDevice() || deferredZoom) {
                for (var k = 0; k < this.model._visibleSeries.length; k++) {
                    var series = this.model.series[k];
                    var transform = ej.EjSvgRender.utils._getTransform(series.xAxis, series.yAxis, this.model.requireInvertedAxes);
                    var transX = (evt.pageX - this.oPreviousCoords.x) + transform.x;
                    var transY = (evt.pageY - this.oPreviousCoords.y) + transform.y;
                    var translate = 'translate(' + transX + ',' + transY + ')';
                    var svgObjectId = this.svgObject.id;
                    $("#" + svgObjectId + "_SeriesGroup_" + k).attr("transform", translate);
                    $("#" + svgObjectId + "_symbolGroup_" + k).attr("transform", translate);
                    $("#" + svgObjectId + "_TextGroup_" + k).attr("transform", translate);
                    
                }
                for (var index = 0; index < this.model.indicators.length && this.model.indicators[index].points.length>0; index++) {
                    var indicator = this.model.indicators[index];
                    transform = ej.EjSvgRender.utils._getTransform(indicator.xAxis, indicator.yAxis, this.model.requireInvertedAxes);
                    transX = (evt.pageX - this.oPreviousCoords.x) + transform.x;
                    transY = (evt.pageY - this.oPreviousCoords.y) + transform.y;
                    translate = 'translate(' + transX + ',' + transY + ')';
                    $("#" + this.svgObject.id + "_indicatorGroup_" + index).attr("transform", translate);
                }
            }
        }
		
		//Declaration
            var targetid = evt.target.id;
            var id = this._id;
            var parentId = ej.util.isNullOrUndefined(evt.target.parentNode) ? "" : evt.target.parentNode.id;
			
	//condition to find the buttons and display tooltip on hover
       if ( parentId ==  id + "_ResetZoom")
           createBtnTooltip("Reset");
        else if (parentId == id + '_PanBtn')
            createBtnTooltip("Pan");
        else if (parentId == id + '_ZoomBtn')
            createBtnTooltip("Zoom");

	// method to create tooltip for zooming and panning buttons
        function createBtnTooltip(text) {
            if ($("#tooltip").length <= 0) {
                var tooltipdiv = $("<div id = 'tooltip'></div>");
                $(tooltipdiv).html("&nbsp" + text + "&nbsp");
                $(document.body).append(tooltipdiv);
                // adding css prop to the div
                $(tooltipdiv).css({
                    "left": evt.pageX + 10,
                    "top": evt.pageY + 10,
                    "display": "block",
                    "position": "absolute",
                    "z-index": "13000",
                    "cursor": "default",
                    "font-family": "Segoe UI",
                    "color": "#707070",
                    "font-size": "12px",
                    "background-color": "#FFFFFF",
                    "border": "1px solid #707070"
                });
            } else {
                $("#tooltip").css({
                    "left": evt.pageX + 10,
                    "top": evt.pageY + 10
                });
            }

        }
           var svgId = this.svgObject.id;
            var hAxes = this.model.hAxes;

            if (targetid.indexOf("_XLabel_") >= 0) {
                // Loop to find the axis
                for (var j = 0; j < hAxes.length; j++) {
                    if (targetid.indexOf(svgId + '_' + hAxes[j].name + '_XLabel_') >= 0)
                        break;
                }
                // Loop to find the labels in the axis
                for (var i = 0; i < hAxes[j].visibleLabels.length && this.model.AreaType=='cartesianaxes'; i++) {
                    if (targetid == svgId + '_' + hAxes[j].name + '_XLabel_' + i) {
                        var ele = hAxes[j].visibleLabels[i];
                        var tooltipdiv = $("<div id = 'tooltip'></div>");
                        var str = ele.Text == ele.displayText ? "" : ele.Text;
                        if (str == "") break;
                        $(tooltipdiv).html("&nbsp" + str + "&nbsp");
                        $(document.body).append(tooltipdiv);
                        // adding css prop to the div
                        $(tooltipdiv).css({
                            "left": evt.pageX + 10,
                            "top": evt.pageY + 10,
                            "display": "block",
                            "position": "absolute",
                            "z-index": "13000",
                            "cursor": "default",
                            "font-family": "Segoe UI",
                            "color": "#707070",
                            "font-size": "12px",
							"background-color": "#FFFFFF",
                            "border": "1px solid #707070"
                        });
                    }
                }
            }

        if ($(this.svgObject).find(id + '_ResetZoom,' + id + '_PanBtn,' + id + '_ZoomBtn').length > 0) {
            $(this.svgObject).find(id + '_ResetZoom,' + id + '_PanBtn,' + id + '_ZoomBtn').appendTo(this.svgObject);
        }

        var legenddata = this.getLegendData(evt);
        if (legenddata) {
            var commonLegendMoveEventArgs = $.extend({}, ej.EjSvgRender.commonChartEventArgs);
            commonLegendMoveEventArgs.data = legenddata;
            this._trigger("legendItemMouseMove", commonLegendMoveEventArgs);
        }
        if (this.model.enableCanvasRendering) {

            //condition to find the buttons and display tooltip on hover for Canvas
            switch(targetid){
                case id + "_ResetZoom":
                    createBtnTooltip("Reset");
                    break;
                case id + '_PanBtn':
                    createBtnTooltip("Pan");
                    break;
                case id + '_ZoomBtn':
                    createBtnTooltip("Zoom");
                    break;
            }
            this.canvasAxisLabelTooltip(evt);
        }
    },
  
    canvasAxisLabelTooltip: function(evt){
        var mouseMoveCords = this.calMousePosition(evt);
        this.mousemoveX = mouseMoveCords.X;
        this.mousemoveY = mouseMoveCords.Y;
        var currentX = this.mousemoveX;
        var currentY = this.mousemoveY;
          
        if (this.model.xAxisLabelRegion) {
            $.each(this.model.xAxisLabelRegion, function (index, regionItem) {
                var axisLabelRegion = regionItem;

                if ((currentX >= regionItem.Bounds.X) && (currentX <= regionItem.Bounds.X + regionItem.Bounds.Width)) {
                    if ((currentY >= regionItem.Bounds.Y - (regionItem.Bounds.Height / 3)) && (currentY <= regionItem.Bounds.Y)) {

                        var tooltipdiv = $("<div id = 'tooltip'></div>");
                        if (regionItem.TrimText != regionItem.LabelText) {
                            $(tooltipdiv).html("&nbsp" + regionItem.LabelText + "&nbsp");
                            $(document.body).append(tooltipdiv);
                            // adding css prop to the div
                            $(tooltipdiv).css({
                                "left": evt.pageX + 10,
                                "top": evt.pageY + 10,
                                "display": "block",
                                "position": "absolute",
                                "z-index": "13000",
                                "cursor": "default",
                                "font-family": "Segoe UI",
                                "color": "#707070",
                                "font-size": "12px",
                                "background-color": "#FFFFFF",
                                "border": "1px solid #707070"
                            });
                        }
                    }
                }
            });
        }
    },
    pieExplosion: function (data) {
        var region = data;
        var id;
        var targetElement;
        var symbolName;
        var series = region.region.Series;
        var seriesType = new ej.seriesTypes[series.type]();
        var explodePoint = series.explodeIndex;
        var _labelPosition = series.labelPosition.toLowerCase();
        if (explodePoint != region.pointData[0].PointIndex && !region.region.Series.explodeAll) {
            series.explodeIndex = region.pointData[0].PointIndex;

            if (!ej.util.isNullOrUndefined(this.model.explodeValue)) {
                var result = seriesType._calculateArcData(this, this.model.explodeValue, series.points[this.model.explodeValue], region.region.Series);
                id = this.svgObject.id + '_SeriesGroup' + '_' + region.region.SeriesIndex;
                
                targetElement = $(this.gSeriesEle).children('#' + id)[0];
                var elements = $(targetElement).children();
                for (var i = 0; i < elements.length; i++) {
                    var element = elements[i];
                    var index = this.svgRenderer._getAttrVal($(element)[0], "pointIndex");
                    if (parseInt(index) == this.model.explodeValue) {
                        this.svgRenderer._setAttr($(element), { 'd': result.Direction });
                        break;
                    }
                }


                if (series.marker.dataLabel.visible) {
                    seriesType.drawDataLabelAcc(this, series, this.model.explodeValue, series.points[this.model.explodeValue])
                }
            }

            result = seriesType._calculateArcData(this, region.pointData[0].PointIndex, this.model.chartRegions[0].Series.points[region.pointData[0].PointIndex], region.region.Series);
            id = this.svgObject.id + '_SeriesGroup' + '_' + region.region.SeriesIndex;
            targetElement = $(this.gSeriesEle).children('#' + id)[0];
            var elements = $(targetElement).children();
            for (var i = 0; i < elements.length; i++) {
                var element = elements[i];
                var index = this.svgRenderer._getAttrVal($(element)[0], "pointIndex");
                if (parseInt(index) == region.pointData[0].PointIndex) {
                    this.svgRenderer._setAttr($(element), { 'd': result.Direction });
                    break;
                }
            }
            if (this.model.enableCanvasRendering) {
                var chartRect = document.getElementById(this.svgObject.id).getClientRects()[0];
                this.svgRenderer.ctx.clearRect(chartRect.left, chartRect.top, chartRect.width, chartRect.height);
                $("#" + this._id).ejChart("redraw");
            }
            if (series.marker.dataLabel.visible) {
                seriesType.drawDataLabelAcc(this, series, region.pointData[0].PointIndex, series.points[region.pointData[0].PointIndex])
            }
            this.model.explodeValue = region.pointData[0].PointIndex;

        }
        region.region.Series.explodeIndex = explodePoint;
    },
    displayTemplateTooltip: function (x, y, location, seriesPoint, series, serIndex, ptIndex) {
        var point = $.extend(true, {}, seriesPoint);
        point.x = point.X;
        if (series.xAxis._valueType.toLowerCase() == "datetime")
            point.x = (Globalize.format(new Date(point.X), ((ej.util.isNullOrUndefined(series.xAxis.labelFormat)) ? ej.EjSvgRender.utils._dateTimeLabelFormat(series.xAxis.intervalType) : series.xAxis.labelFormat)));
        if (series.xAxis._valueType.toLowerCase() == "category") point.x = ej.EjSvgRender.utils._getLabelContent(ptIndex, series.xAxis);
        // x and y is set to 0, if undefined
        x = ej.util.isNullOrUndefined(x) ? 0 : x;
        y = ej.util.isNullOrUndefined(y) ? 0 : y;
        var xLoc = location.X + x + 15;
        var yLoc = (location.Y) + y + 15;
        var tooltipdiv = $("<div style='position: absolute; z-index: 13000; display: block;'></div>");
        $(tooltipdiv).addClass("TrackToolTipTemplate");
        this.svgRenderer._setAttr($(tooltipdiv), { "id": this.svgObject.id + "_TrackToolTipTemplate_" + serIndex });
        if (this.model.crosshair.tooltipTemplate != null) {
            var cloneNode = $("#" + this.model.crosshair.tooltipTemplate).clone();
            $(cloneNode).css("display", "block").appendTo(tooltipdiv);
            series.count = 1;
            point.count = 1;
            var seriesColor = this.getSeriesColor(point, serIndex);
            $(tooltipdiv).css("background-color", jQuery.type(seriesColor) == "array" ? seriesColor[0].color : seriesColor);
            var data = { series: series, point: point };
            $(tooltipdiv).html($(tooltipdiv).html().parseTemplate(data));
        }
        $(document.body).append(tooltipdiv);
        $(tooltipdiv).css("left", xLoc + $(tooltipdiv).width());
        $(tooltipdiv).css("top", yLoc + $(tooltipdiv).height());

    },

    displayAxisTooltip: function (location, text, axis, index, mouseLoc) {
        var maxTickSize = 0;
        for (var i = 0; i < this.model.axes.length; i++) {
            if (this.model.axes[i].majorTickLines.size > maxTickSize) {
                maxTickSize = this.model.axes[i].majorTickLines.size;
            }
        }
        var maxLocation = (axis.opposedPosition) ? ((maxTickSize + 10) + (axis.x + axis.width)) : (axis.x + axis.width);
        if (location.X >= 0 && location.X <= maxLocation) {
            var x = (location.X);
            var y = location.Y;
            var padding = 5;
            var commonTrackTextArgs = $.extend({}, ej.EjSvgRender.commonChartEventArgs);
            commonTrackTextArgs.data = { axisIndex: index, chartAxis: axis, currentTrackText: text, location: mouseLoc };
            this._trigger("trackAxisToolTip", commonTrackTextArgs);

            var trackAxisText = commonTrackTextArgs.data.currentTrackText;

            var textOffset = ej.EjSvgRender.utils._measureText(trackAxisText, null, axis.crosshairLabel.font);
            if (axis.orientation.toLowerCase() == 'horizontal') {
                x = x - textOffset.width / 2;
                if (axis.opposedPosition == true) {
				  y = axis.Location.Y1 - textOffset.height/2 + padding - maxTickSize;  
                }

                if (axis.opposedPosition == false) {
				    y = axis.Location.Y1 + textOffset.height - padding + maxTickSize;
                }
            }
            if (!(axis.opposedPosition) && (x + textOffset.width) > (this.model.m_AreaBounds.Width + this.model.m_AreaBounds.X)) {
                x = ((this.model.m_AreaBounds.Width + this.model.m_AreaBounds.X) - textOffset.width) + padding + this.model.elementSpacing;
            }
            if (axis.orientation.toLowerCase() == 'vertical') {
			     y = location.Y + textOffset.height / 4;
                if (axis.opposedPosition == true) {
                     x = axis.x + 2 + padding - 2 ;
                }
                if (axis.opposedPosition == false) {
                    x = axis.x - textOffset.width - maxTickSize;
                }
            }

            var textAxisOptions = {
                'id': this.svgObject.id + '_AxisToolTipText' + '_' + index,
                'x': x,
                'y': y,
                'fill': axis.crosshairLabel.font.color,
                'font-size': axis.crosshairLabel.font.size,
                'font-family': axis.crosshairLabel.font.fontFamily,
                'font-style': axis.crosshairLabel.font.fontStyle,
                'font-weight': axis.crosshairLabel.font.fontWeight,
                'text-anchor': 'start',
                'opacity': axis.crosshairLabel.font.opacity
              
            };

            var fontSize = ej.EjSvgRender.utils._measureText(commonTrackTextArgs.data.currentTrackText, this.model.m_AreaBounds.Width, axis.crosshairLabel.font);
            var crosshairLabel = axis.crosshairLabel;
             var toolAxisRectOptions = {
                'id': this.svgObject.id + '_AxisToolTipRect' + '_' + index,
                'x': (x - padding),
                'y': (y - (fontSize.height)),
                'width': (fontSize.width) + (2* padding),
                'height': (2*fontSize.height)-(2*padding),
                'rx': crosshairLabel.rx,
                'ry': crosshairLabel.ry,
                'fill': crosshairLabel.fill,
                'stroke-width': crosshairLabel.border.width,
                'stroke': crosshairLabel.border.color
            };
             if (!this.model.enableCanvasRendering) {
             this.svgRenderer.drawRect(toolAxisRectOptions, this.gTrackAxisEle);
             this.svgRenderer.drawText(textAxisOptions, commonTrackTextArgs.data.currentTrackText, this.gTrackAxisEle);
             }else
                 this.svgRenderer.drawCrosshairlabel(toolAxisRectOptions, textAxisOptions, commonTrackTextArgs.data.currentTrackText);
        }
    },
    displayTooltip: function (location, point, series, pointIndex) {
        var seriesIndex = $.inArray(series, this.model._visibleSeries);
        if ((location.X + this.model.m_AreaBounds.X > this.model.m_AreaBounds.X || location.X == 0) && ((location.X + this.model.m_AreaBounds.X) <= (this.model.m_AreaBounds.X + this.model.m_AreaBounds.Width)) &&
     (location.Y + this.model.m_AreaBounds.Y) >= this.model.m_AreaBounds.Y && (location.Y + this.model.m_AreaBounds.Y) < (this.model.m_AreaBounds.Y + this.model.m_AreaBounds.Height)) {
            var padding = 5;
            var x = location.X + (!series.isIndicator ? ((ej.util.isNullOrUndefined(series._trackMarker)) ? series.marker.size.width : (series._trackMarker.size.width)) :0) + padding + 1;
            var y = (location.Y);
            var format = series.tooltip.format;
            var numberToFixed = ((ej.util.isNullOrUndefined(series.yAxis.roundingPlaces)) ? 2 : series.yAxis.roundingPlaces);
            var trackTooltipText = this.getTooltipFormat(point, series, seriesIndex, pointIndex, format, numberToFixed);
            var commonTrackTextArgs = $.extend({}, ej.EjSvgRender.commonChartEventArgs);
            commonTrackTextArgs.data = { serIndex: seriesIndex, pointIndex: pointIndex, currentText: trackTooltipText };
            var tooltip = series.tooltip;
            this._trigger("trackToolTip", commonTrackTextArgs);
            var font = tooltip.font ? $.extend(false, series.font, {}, tooltip.font) : series.font;
            var textOffset = ej.EjSvgRender.utils._measureText(commonTrackTextArgs.data.currentText, series.xAxis.width, font);
            var seriesColor = this.getSeriesColor(point, seriesIndex, series);
            var toolTipOptions = this.getTooltipOptions(seriesColor, series);
            var rectBorderColor = toolTipOptions.rectBColor;
            var rectFillColor = toolTipOptions.rectFColor;
            var textColor = toolTipOptions.rectTextColor;
            var rX = toolTipOptions.rectX;
            var rY = toolTipOptions.rectY;
            var rectOptions = series.tooltip;
            var rectWidth = textOffset.width + (2 * padding);
            var rectHeight = textOffset.height + (2 * padding);

            if (x + rectWidth - padding >= this.model.m_AreaBounds.Width)
                x = x - rectWidth - ((!series.isIndicator)?((ej.util.isNullOrUndefined(series._trackMarker)) ? series.marker.size.width : (series._trackMarker.size.width)):0) - (2 * padding) - 2;
            var toolRectOptions = {
                'id': this.svgObject.id + '_ToolTipRect' + '_' + seriesIndex + '_' + series.name,
                'x': x - padding + this.canvasX,
                'y': y - textOffset.height / 2 - padding + this.canvasY,
                'width': rectWidth,
                'height': rectHeight,
                'fill': rectOptions.fill ? rectOptions.fill : rectFillColor,
                'stroke-width': rectOptions.border.width ? rectOptions.border.width : 1,
                'stroke': rectOptions.border.color ? rectOptions.border.color : rectBorderColor,
                'fill-opacity': rectOptions.opacity ? rectOptions.opacity : 0.85,
                'rx': rectOptions.rx ? rectOptions.rx : rX,
                'ry': rectOptions.ry ? rectOptions.ry : rY
            };
            $('#' + this.svgObject.id + '_ToolTipRect' + '_' + seriesIndex + '_' + series.name).attr("visibility", "visible");
            //this.svgRenderer.drawRect(toolRectOptions, this.gTransToolEle);
            var text = commonTrackTextArgs.data.currentText;
            var fontText = text;
            if (commonTrackTextArgs.data.currentText.indexOf("<br/>") >= 0) {
                text = commonTrackTextArgs.data.currentText.split("<br/>");
                fontText = text[0];
            }
            var fontSize = ej.EjSvgRender.utils._measureText(text, series.xAxis.width, font).height;
            var textOptions = {
                'id': this.svgObject.id + '_ToolTipText' + '_' + seriesIndex + '_' + series.name,
                'x': x + this.canvasX,
                'y': y - textOffset.height / 2 - padding + fontSize + this.canvasY,
                'fill': (tooltip.font && tooltip.font.color) ? font.color : textColor,
                'font-size': font.size,
                'font-family': font.fontFamily,
                'font-style': font.fontStyle,
                'font-weight': font.fontWeight,
                'text-anchor': 'start'
            };
            $('#' + this.svgObject.id + '_ToolTipText' + '_' + seriesIndex + '_' + series.name).attr("visibility", "visible");
          
            if (this.model.enableCanvasRendering)
                this.svgRenderer.drawTrackToolTip(toolRectOptions, textOptions, text);
            else {
                this.svgRenderer.drawRect(toolRectOptions, this.gTransToolEle);
                this.svgRenderer.drawText(textOptions, text, this.gTransToolEle, font);
            }
            if ($('#' + this.svgObject.id + "_TrackToolTip_" + seriesIndex).length == 0)
                this.svgRenderer.append(this.gTransToolEle, this.svgObject);
        } else {
            $('#' + this.svgObject.id + '_TrackToolTip_' + seriesIndex).remove();
        }

    },
    getTooltipOptions: function (seriesColor, series) {
        var rectBorderColor, rectFillColor, rX, rY, textColor;
        if (this.model.theme.indexOf("gradient") >= 0) {
            rectBorderColor = jQuery.type(seriesColor) == "array" ? seriesColor[0].color : seriesColor;
            rectFillColor = 'white';
            textColor = "#333333";
            rX = 5;
            rY = 5;

        } else {
            var type = series.type.toLowerCase();
            if (type != "column" && type != "bar" && type != "rangecolumn" && type != "stackingcolumn" && type != "stackingbar" && this.model.AreaType != "none") {
                rectFillColor = jQuery.type(seriesColor) == "array" ? seriesColor[0].color : seriesColor;
                rectBorderColor = 'transparent';
                textColor = "white";
            } else {
                rectBorderColor = "#333333";
                rectFillColor = 'white';
                textColor = "#333333";
            }
            rX = 0;
            rY = 0;
        }
        return { rectBColor: rectBorderColor, rectFColor: rectFillColor, rectTextColor: textColor, rectX: rX, rectY: rY };
    },
    getSeriesColor: function (point, seriesIndex, series) {
        var seriesColor;
        if (series && !series.isIndicator) {
            if (point.fill) {
                if (!point.fill._gradientStop)
                    seriesColor = point.fill;
                else {
                    seriesColor = point.fill._gradientStop[0].color;
                }
            } else {
                seriesColor = this.model.seriesColors[seriesIndex];
            }
        } else {
            seriesColor = series.fill;
        }
        return seriesColor;
    },
    getTooltipFormat: function (seriesPoint, series, serIndex, ptIndex, format, numberToFixed) {
        var point = $.extend(true, {}, seriesPoint);
        series.count = 1;
        point.count = 1;
        if (this.model.AreaType == "cartesianaxes") {
            if (series.xAxis._valueType.toLowerCase() == "datetime")
                point.x = (Globalize.format(new Date(point.X), ((ej.util.isNullOrUndefined(series.xAxis.labelFormat)) ? ej.EjSvgRender.utils._dateTimeLabelFormat(series.xAxis.intervalType) : series.xAxis.labelFormat)));
            if (series.xAxis._valueType.toLowerCase() == "category")
                point.x = ej.EjSvgRender.utils._getLabelContent(point.X, series.xAxis);
        }
        else {
            if (series._xAxisValueType == "date")
                point.x = (Globalize.format(new Date(point.X), (ej.EjSvgRender.utils._dateTimeLabelFormat('days'))));
        }
        point.y = (ej.EjSvgRender.utils._decimalPlaces(point.y) == 0) ? point.y : point.y.toFixed(numberToFixed);
        var data = { series: series, point: point };
        format = format.parseTemplate(data);
        return format;
    },
    displayCanvasTooltip: function (location, point, series, pointIndex) {
        $("#" + this.svgObject.id + "_TrackToolTip").remove();
        if (point.visible) {
            var seriesIndex = $.inArray(series, this.model._visibleSeries),
            seriesColor,
            textOffset,
            padding = 7,
            x,
            y,
            trackTooltipText,
            format = series.tooltip.format;

            //Set tooltip position ,text and color
            switch (this.model.AreaType) {

                case "cartesianaxes":
                    var numberToFixed = (series.yAxis.roundingPlaces === null || series.yAxis.roundingPlaces === undefined) ? 2 : series.yAxis.roundingPlaces;
                    x = location.X + this.canvasX + ((ej.util.isNullOrUndefined(series._trackMarker)) ? series.marker.size.width : (series._trackMarker.size.width)) + padding + 1;
                    y = (location.Y) + this.canvasY;
                    seriesColor = this.getSeriesColor(point, seriesIndex);
                    break;

                case "polaraxes":
                    var numberToFixed = (series.yAxis.roundingPlaces === null || series.yAxis.roundingPlaces === undefined) ? 2 : series.yAxis.roundingPlaces;
                    x = location.X + ((ej.util.isNullOrUndefined(series._trackMarker)) ? series.marker.size.width : (series._trackMarker.size.width)) + padding + 1;
                    y = location.Y;
                    seriesColor = this.getSeriesColor(point, seriesIndex);
                    break;

                case "none":
                    x = location.X + series.marker.size.width + padding;
                    y = (location.Y);
                    numberToFixed = ej.util.isNullOrUndefined(this.model.roundingPlaces) ? 2 : chartObj.model.roundingPlaces;
                    seriesColor = this.model.pointColors[pointIndex];
                    break;
            }
            trackTooltipText = this.getTooltipFormat(point, series, seriesIndex, pointIndex, format, numberToFixed);;
            textOffset = ej.EjSvgRender.utils._measureText(trackTooltipText);
            var toolTipOptions = this.getTooltipOptions(seriesColor, series);
            var rectBorderColor = toolTipOptions.rectBColor;
            var rectFillColor = toolTipOptions.rectFColor;
            var textColor = toolTipOptions.rectTextColor;
            var rX = toolTipOptions.rectX;
            var rY = toolTipOptions.rectY;
            var rectOptions = series.tooltip;
            //draw tooltip rectangle
            var toolRectOptions = {
                'id': this.svgObject.id + '_ToolTipRect',
                'x': x,
                'y': y,
                'width': textOffset.width,
                'height': textOffset.height,
                'fill': (rectOptions.fill) ? rectOptions.fill : rectFillColor,
                'stroke-width': (rectOptions.border.width) ? rectOptions.border.width : 1,
                'stroke': (rectOptions.border.color) ? rectOptions.border.color : rectBorderColor,
                'fill-opacity': rectOptions.opacity,
                'rx': (rectOptions.rx) ? rectOptions.rx : rX,
                'ry': (rectOptions.ry) ? rectOptions.ry : rY
            };
                this.svgRenderer.drawToolTipRect(toolRectOptions, this.gTransToolEle);

            //event for tooltip text
            var commonTrackTextArgs = $.extend({}, ej.EjSvgRender.commonChartEventArgs);
            commonTrackTextArgs.data = { seriesIndex: seriesIndex, pointIndex: pointIndex, currentText: trackTooltipText };
            this._trigger("toolTipInitialize", commonTrackTextArgs);

            //draw tooltip text
            var fontSize = 0;
            var tooltip = series.tooltip;
            var font = (tooltip.font) ? $.extend(false, series.font, {}, tooltip.font) : series.font;
            var textOptions = {
                'id': this.svgObject.id + '_ToolTipText',
                'x': x,
                'y': y,
                'fill': (tooltip.font && tooltip.font.color) ? font.color : textColor,
                'font-size': font.size,
                'font-family': font.fontFamily,
                'font-style': font.fontStyle,
                'font-weight': font.fontWeight,
                'text-anchor': 'start'
            };

            // Add string in text array for tooltip format
            var text = commonTrackTextArgs.data.currentText;
            //if (commonTrackTextArgs.data.currentText.indexOf("<br/>") >= 0) {
            //    text = commonTrackTextArgs.data.currentText.split("<br/>");
            //}
            fontSize = (this.model.AreaType == "cartesianaxes") ? ej.EjSvgRender.utils._measureText(text, series.xAxis.width, font).height : ej.EjSvgRender.utils._measureText(text, null, font).height;
            var maxWidth = (this.model.AreaType == "cartesianaxes") ? (this.model.requireInvertedAxes) ? series.yAxis.width : series.xAxis.width : $(this.svgObject).width();
                this.svgRenderer.drawToolTipText(textOptions, text);

               //Adjust x-position to display tooltip within chart area 
                var box = $("#" + this.svgObject.id + "_TrackToolTip")[0].getBoundingClientRect();
                if ((x + (box.right - box.left)) >= maxWidth + (10 * padding)) {
                    var chartPos = document.getElementById(this.svgObject.id).getClientRects()[0];
                    var diff = x - (((box.right - box.left) + ((ej.util.isNullOrUndefined(series._trackMarker)) ? ((series.marker.visible) ? series.marker.size.width : 0) : series._trackMarker.size.width)) + (2 * padding));
                $("#" + this.svgObject.id + "_TrackToolTip").css("left", diff + chartPos.left + $(document).scrollLeft());
                var elements = $("#" + this.svgObject.id + "_TrackToolTip");
                    }
        }
    },
    displayShowTooltip: function (location, point, series, pointIndex) {
        if (point.visible) {
            var seriesIndex = (!series.isIndicator) ? $.inArray(series, this.model._visibleSeries) : 0,
            seriesColor,
            textOffset,
            padding = 7,
            x,
            y,
            trackTooltipText,
            format = series.tooltip.format;

            //Set tooltip position ,text and color
            if (this.model.AreaType == "cartesianaxes" || this.model.AreaType=='polaraxes') {
                var numberToFixed = (series.yAxis.roundingPlaces === null || series.yAxis.roundingPlaces === undefined) ? 2 : series.yAxis.roundingPlaces;
                x = location.X + ((!series.isIndicator) ? ((ej.util.isNullOrUndefined(series._trackMarker)) ? series.marker.size.width : (series._trackMarker.size.width)) : 0) + padding + 1;
                y = (location.Y);
                trackTooltipText = this.getTooltipFormat(point, series, seriesIndex, pointIndex, format, numberToFixed);
                seriesColor = this.getSeriesColor(point, seriesIndex, series);
                textOffset = ej.EjSvgRender.utils._measureText(trackTooltipText);
            } else {
                x = location.X + series.marker.size.width + padding;
                y = (location.Y);
                numberToFixed = ej.util.isNullOrUndefined(this.model.roundingPlaces) ? 2 : chartObj.model.roundingPlaces;
                trackTooltipText = this.getTooltipFormat(point, series, seriesIndex, pointIndex, format, numberToFixed); ;
                textOffset = ej.EjSvgRender.utils._measureText(trackTooltipText);
                seriesColor = this.model.pointColors[pointIndex];
            }
            var toolTipOptions = this.getTooltipOptions(seriesColor, series);
            var rectBorderColor = toolTipOptions.rectBColor;
            var rectFillColor = toolTipOptions.rectFColor;
            var textColor = toolTipOptions.rectTextColor;
            var rX = toolTipOptions.rectX;
            var rY = toolTipOptions.rectY;
            var rectOptions = series.tooltip;
            //draw tooltip rectangle
            var toolRectOptions = {
                'id': this.svgObject.id + '_ToolTipRect',
                'x': x,
                'y': y,
                'width': textOffset.width,
                'height': textOffset.height,
                'fill': (rectOptions.fill) ? rectOptions.fill : rectFillColor,
                'stroke-width': (rectOptions.border.width) ? rectOptions.border.width : 1,
                'stroke': (rectOptions.border.color) ? rectOptions.border.color : rectBorderColor,
                'fill-opacity': rectOptions.opacity,
                'rx': (rectOptions.rx) ? rectOptions.rx : rX,
                'ry': (rectOptions.ry) ? rectOptions.ry : rY
            };
            this.svgRenderer.drawRect(toolRectOptions, this.gTransToolEle);
            
            //event for tooltip text
            var commonTrackTextArgs = $.extend({}, ej.EjSvgRender.commonChartEventArgs);
            commonTrackTextArgs.data = { seriesIndex: seriesIndex, pointIndex: pointIndex, currentText: trackTooltipText };
            this._trigger("toolTipInitialize", commonTrackTextArgs);

            //draw tooltip text
            var fontSize = 0;
            var tooltip = series.tooltip;
            var font = (tooltip.font) ? $.extend(false, series.font, {}, tooltip.font) : series.font;
            var textOptions = {
                'id': this.svgObject.id + '_ToolTipText',
                'x': x,
                'y': y,
                'fill': (tooltip.font && tooltip.font.color) ? font.color : textColor,
                'font-size': font.size,
                'font-family': font.fontFamily,
                'font-style': font.fontStyle,
                'font-weight': font.fontWeight,
                'text-anchor': 'start'
            };

            // Add string in text array for tooltip format
            var text = commonTrackTextArgs.data.currentText;
            if (commonTrackTextArgs.data.currentText.indexOf("<br/>") >= 0) {
                text = commonTrackTextArgs.data.currentText.split("<br/>");
            }
            fontSize = (this.model.AreaType == "cartesianaxes") ? ej.EjSvgRender.utils._measureText(text, series.xAxis.width, font).height : ej.EjSvgRender.utils._measureText(text, null, font).height;
            var maxWidth = (this.model.AreaType == "cartesianaxes") ? (this.model.requireInvertedAxes) ? series.yAxis.width : series.xAxis.width : $(this.svgObject).width()-(this.model.legend.position.toLowerCase()=="right" ?(this.model.LegendViewerBounds.Width + 2*this.model.elementSpacing):0);
            var maxHeight = (this.model.AreaType == "cartesianaxes") ? (this.model.requireInvertedAxes) ? series.xAxis.height : series.yAxis.height : $(this.svgObject).height() -(this.model.legend.position.toLowerCase()=="bottom" ?(this.model.LegendViewerBounds.Height + this.model.elementSpacing):0);
                this.svgRenderer.drawText(textOptions, text, this.gTransToolEle, font);
            if ($(this.svgObject).find("#" + this.svgObject.id +"_TrackToolTip").length == 0)
                this.svgRenderer.append(this.gTransToolEle, this.svgObject);
            //Adjust x-position to display tooltip within chart area 
            var box = $("#" + textOptions.id)[0].getBoundingClientRect();
            if ((x + (box.right - box.left)) >= maxWidth) {
                var diff = x - ((box.right - box.left) + ((!series.isIndicator) ? ((ej.util.isNullOrUndefined(series._trackMarker)) ? ((series.marker.visible) ? series.marker.size.width : 0) : series._trackMarker.size.width) : 0) + (4 * padding));
                this.svgRenderer._setAttr("#" + textOptions.id, { 'x': diff });
                var elements = $("#" + textOptions.id).children("tspan");
                for (var i = 0; i < elements.length; i++) {
                    var element = elements[i];
                    this.svgRenderer._setAttr(element, { 'x': diff });
                }
                x = diff;
            }
            if ((y + (box.bottom - box.top)) >= maxHeight) {
                var diffY = y - (((box.bottom - box.top) + ((!series.isIndicator)?((ej.util.isNullOrUndefined(series._trackMarker)) ? ((series.marker.visible) ? series.marker.size.height : 0) : series._trackMarker.size.height) :0))) + (4 * padding);
                var diffTextY = this.vmlRendering ? (diffY - (font.size.replace("px", ""))) : diffY;
                this.svgRenderer._setAttr("#" + textOptions.id, { 'y': diffTextY });
                y = diffY;
            }
            //adjust rectangle size based on text length  
            this.svgRenderer._setAttr("#" + toolRectOptions.id, { 'x': (x - padding), 'y': ((y - (fontSize / 2)) - padding), 'width': (box.right - box.left) + 2 * padding, 'height': (box.bottom - box.top) + padding });
        }
    },
    _initializeSeriesColors: function () {
        var chartObj = this;
        var modelColor = (chartObj.model.palette) ? chartObj.model.palette : chartObj.model.colors;
        var count=modelColor.length
        for (var i = 0; i < this.model._visibleSeries.length; i++) {
            var series = this.model._visibleSeries[i];           
            if (series.visibility.toLowerCase() === 'visible' && chartObj.model.AreaType != "none") {
                if (!ej.util.isNullOrUndefined(series.fill)) {
                    chartObj.model.seriesColors[i] = ej.util.isNullOrUndefined(series.fill._gradientStop) ? series.fill : series.fill._gradientStop;
                } else {
                    chartObj.model.seriesColors[i] = modelColor[i % count];
                }
                if (!ej.util.isNullOrUndefined(series.border.color)) {
                    chartObj.model.seriesBorderColors[i] = series.border.color;
                } else {
                    chartObj.model.seriesBorderColors[i] = chartObj.model.seriesBorderDefaultColors[i % 10];
                }
            } else {
                if (series.visibility.toLowerCase() === 'visible') {
                    for (var j = 0; j < series.points.length; j++) {
                        var point = series.points[j];         
                        if (!ej.util.isNullOrUndefined(point.fill)) {
                            chartObj.model.pointColors[j] = ej.util.isNullOrUndefined(point.fill._gradientStop) ? point.fill : point.fill._gradientStop;
                            } else {
                            chartObj.model.pointColors[j] = modelColor[j % count];
                            }
                        if (!ej.util.isNullOrUndefined(point.border) && !ej.util.isNullOrUndefined(point.border.color)) {
                            chartObj.model.pointBorderColors[j] = point.border.color;
                            } else {
                                chartObj.model.pointBorderColors[j] = chartObj.model.seriesBorderDefaultColors[i % 10];
                            }      

                    }
                }

            }

        }
    },



    _createAxisLabelAndRange: function () {
        var unsift = false;
        if (this.model.requireInvertedAxes) {
            this.model.primaryYAxis.orientation = "horizontal";
            this.model.primaryXAxis.orientation = "vertical";
            this.model.primaryYAxis.labelPlacement = (!(this.model.primaryYAxis.labelPlacement)) ? ej.datavisualization.Chart.LabelPlacement.OnTicks : this.model.primaryYAxis.labelPlacement;
        }
        else {
            this.model.primaryYAxis.orientation = "vertical";
            this.model.primaryXAxis.orientation = "horizontal";
        }
        this.model.axes = (!this.model.axes) ? [] : this.model.axes;
        for (var i = 0; i < this.model.axes.length; i++) {
            var axis = this.model.axes[i];
            axis.orientation = (!axis.orientation) ? "vertical" : (axis.name == this.model.primaryXAxis.name) ? this.model.primaryXAxis.orientation : (axis.name == this.model.primaryYAxis.name) ? this.model.primaryYAxis.orientation : axis.orientation;
            if (axis.orientation.toLowerCase() == "horizontal")
                this.model.axes[i] = $.extend(true, {}, this.model.secondaryX, axis);
            else
                this.model.axes[i] = $.extend(true, {}, this.model.secondaryY, axis);
        }


        this.model.primaryYAxis.name = "SecondaryAxis";
        this.model.primaryXAxis.name = "PrimaryAxis";

        for (var j = 0; j < this.model.axes.length; j++) {
            var axis = this.model.axes[j];
            if (axis.name === this.model.primaryXAxis.name || axis.name === this.model.primaryYAxis.name)
                unsift = true;
        }

        if (!unsift) {
            this.model.axes.unshift(this.model.primaryYAxis);
            this.model.axes.unshift(this.model.primaryXAxis);
        }
        for (var k = 0; k < this.model.axes.length; k++) {
            var axis = this.model.axes[k];

            axis.visibleLabels = [];
            axis.setRange = (!(axis.range)) ? false : (ej.util.isNullOrUndefined(axis.setRange)) ? true : axis.setRange;
            axis.setRange = ((axis.actual_Range) || (axis.setRange != false)) ? true : false;
            axis.actualRange = (!(axis.actualRange)) ? (axis.range == null) ? {} : $.extend(true, {}, axis.range) : axis.actualRange;
            axis.visibleRange = (!(axis.visibleRange)) ? (axis.range == null) ? {} : axis.range : axis.visibleRange;
            axis.name = !(axis.name) ? k.toString() : axis.name;
        }
    },






    _drawTitle: function () {
        var measureTitle = ej.EjSvgRender.utils._measureText(this.model.title.text, $(this.svgObject).width() - this.model.margin.left - this.model.margin.right, this.model.title.font);
        var modelTitleHeight = (this.model.title.text == "") ? 0 : (measureTitle.height + this.model.elementSpacing);


        // Drawing the chart title
        var legendSpace = (this.model.legend.position.toLowerCase() == "left") ? this.model.LegendViewerBounds.Width / 2 : ((this.model.legend.position.toLowerCase() == "right" ? (-this.model.LegendViewerBounds.Width / 2) : 0))
        var x = (this.model.AreaType != "cartesianaxes") ?  legendSpace : this.model.m_AreaBounds.X;
        if (this.model.title.text != "" && this.model.title.text != null) {
            var titleLocation = this.model.margin.top + (modelTitleHeight / 2) + (this.model.elementSpacing);
            var locX = ((this.model.AreaType != "cartesianaxes") ? ($(this.svgObject).width() / 2) : (this.model.m_AreaBounds.Width / 2)) - measureTitle.width / 2;
           
            if (this.model.title.textAlignment.toLowerCase() == "near") {
                locX = 0;
            }
            else if (this.model.title.textAlignment.toLowerCase() == "far") {
                locX = $(this.svgObject).width() - (this.model.margin.right + this.model.elementSpacing + this.model.m_AreaBounds.X) - measureTitle.width;
            }


            var commonEventArgs = $.extend({}, ej.EjSvgRender.commonChartEventArgs);
            commonEventArgs.data = { title: this.model.title.text, location: { x: locX, y: titleLocation }, size: measureTitle };
            this._trigger("titleRendering", commonEventArgs);
            this.model._titleLocation = { X: commonEventArgs.data.location.x + x, Y: commonEventArgs.data.location.y };
            var options = {
                'id': this.svgObject.id + '_ChartTitle',
                'x': commonEventArgs.data.location.x + x,
                'y': commonEventArgs.data.location.y,
                'fill': this.model.title.font.color,
                'font-size': this.model.title.font.size,
                'font-family': this.model.title.font.fontFamily,
                'font-style': this.model.title.font.fontStyle,
                'font-weight': this.model.title.font.fontWeight,
                'text-anchor': 'start'
            };

            this.svgRenderer.drawText(options, commonEventArgs.data.title, this.gTitleEle);

            this.svgRenderer.append(this.gTitleEle, this.svgObject);
     if (this.model.title.subTitle.text != "" && this.model.title.subTitle.text)
       this._drawSubTitle();
        }
    },



    _getLegendSize: function (series) {

        var font = this.model.legend.font;
        var textSize = ej.EjSvgRender.utils._measureText(series.name, null, font);
        var symbolSize = this.model.legend.itemStyle;
        var padding = this.model.legend.itemPadding;


        var width = padding + symbolSize.width + padding + textSize.width;
        var height = Math.max(symbolSize.height + padding, textSize.height + padding);

        return { Width: width, Height: height };

    },

    _getYValues: function (series) {
        var values = [];
        for (var i = 0; i < series.points.length; i++)
            values.push(series.points[i].y);
        return values;
    },
    _getXValues: function (series) {
        var values = [];
        for (var i = 0; i < series.points.length; i++)
            values.push(series.points[i].X);
        return values;
    },




    _drawSubTitle: function () {


        var measureTitle = ej.EjSvgRender.utils._measureText(this.model.title.text, $(this.svgObject).width() - this.model.margin.left - this.model.margin.right, this.model.title.font);
        var measuresubTitle = ej.EjSvgRender.utils._measureText(this.model.title.subTitle.text, $(this.svgObject).width() - this.model.margin.left - this.model.margin.right, this.model.title.subTitle.font);

        var textanchor = "middle";

        var modelsubTitleHeight = (this.model.title.subTitle.text == "") ? 0 : (measuresubTitle.height + this.model.elementSpacing);

        // Drawing the chart subtitle

        if (this.model.title.subTitle.text != "" && this.model.title.subTitle.text != null) {
            var subTitleLocation =  (modelsubTitleHeight) / 2 + this.model.elementSpacing + this.model._titleLocation.Y ;

            var locX = this.model._titleLocation.X + (measureTitle.width / 2);

            if (this.model.title.subTitle.textAlignment.toLowerCase() == "near") {

                locX = this.model._titleLocation.X;
                textanchor = "start";
            }
            else if (this.model.title.subTitle.textAlignment.toLowerCase() == "far") {

                locX = this.model._titleLocation.X + measureTitle.width;

                textanchor = "end";
            }
            var commonEventArgs = $.extend({}, ej.EjSvgRender.commonChartEventArgs);
            commonEventArgs.data = { subTitle: this.model.title.subTitle.text, location: { x: locX, y: subTitleLocation }, size: measuresubTitle };
            this._trigger("subTitleRendering", commonEventArgs);
            this.model._subTitleLocation = { X: commonEventArgs.data.location.x, Y: commonEventArgs.data.location.y };
            var options = {
                'id': this.svgObject.id + '_ChartsubTitle',

                'x': commonEventArgs.data.location.x,
                'y': commonEventArgs.data.location.y,
                'fill': this.model.title.subTitle.font.color,
                'font-size': this.model.title.subTitle.font.size,
                'font-family': this.model.title.subTitle.font.fontFamily,
                'font-style': this.model.title.subTitle.font.fontStyle,
                'font-weight': this.model.title.subTitle.font.fontWeight,
                'text-anchor': textanchor
            };

            this.svgRenderer.drawText(options, commonEventArgs.data.subTitle, this.gSubTitleEle);

            this.svgRenderer.append(this.gSubTitleEle, this.gTitleEle);
        }
    },


    _calculateStackingValues: function (seriesCollection, axis) {

        this.model.stackedValue[axis.name] = [];
        this.model.stackedValue[axis.name].min = 0;
        this.model.stackedValue[axis.name].max = 0;
        var stackAxes = (!this.model.requireInvertedAxes) ? this.model.hAxes : this.model.vAxes;

        for (var k = 0; k < stackAxes.length; k++) {
            j = 0;
            var lastPosValue = [];
            var lastNegValue = [];
            for (var i = 0; i < seriesCollection.length; i++) {
                if (seriesCollection[i].xAxisName == stackAxes[k].name) {
                    if (seriesCollection[i].type.toLowerCase().indexOf("stacking") != -1 || seriesCollection[i].isStacking) {
                        var values = {};
                        values.StartValues = [];
                        values.EndValues = [];
                        if (seriesCollection[i].stackingGroup && seriesCollection[i].type.toLowerCase() != "stackingarea") {
                            if (ej.util.isNullOrUndefined(lastPosValue[seriesCollection[i].stackingGroup])) {
                                lastPosValue[seriesCollection[i].stackingGroup] = [];
                                lastNegValue[seriesCollection[i].stackingGroup] = [];
                            }
                        }

                        else {
                            seriesCollection[i].stackingGroup = "";
                            if (ej.util.isNullOrUndefined(lastPosValue[seriesCollection[i].stackingGroup])) {
                                lastPosValue[seriesCollection[i].stackingGroup] = [];
                                lastNegValue[seriesCollection[i].stackingGroup] = [];
                            }
                        }
                        series = seriesCollection[i];
                        series.points = ej.DataManager(series.points, ej.Query().sortBy("X")).executeLocal();
                        var yValues = this._getYValues(series);
                        for (var j = 0; j < series.points.length; j++) {
                            var lastValue = 0;
                            var currentValue = yValues[j];
                            if (ej.util.isNullOrUndefined(lastPosValue[seriesCollection[i].stackingGroup][series.points[j].X])) {
                                lastPosValue[seriesCollection[i].stackingGroup][series.points[j].X]=0;
                            }
                            if (ej.util.isNullOrUndefined(lastNegValue[seriesCollection[i].stackingGroup][series.points[j].X])) {
                                lastNegValue[seriesCollection[i].stackingGroup][series.points[j].X] = 0;
                            }
                            if (values.StartValues.length <= j) {
                                values.StartValues.push(0);
                                values.EndValues.push(0);
                            }
                            if (currentValue > 0) {
                                lastValue = lastPosValue[seriesCollection[i].stackingGroup][series.points[j].X];
                                lastPosValue[seriesCollection[i].stackingGroup][series.points[j].X] += currentValue;
                            }
                            else {
                                lastValue = lastNegValue[seriesCollection[i].stackingGroup][series.points[j].X];
                                lastNegValue[seriesCollection[i].stackingGroup][series.points[j].X] += currentValue;
                            }

                            values.StartValues[j] = lastValue;
                            values.EndValues[j] = currentValue + lastValue;
                            series.points[j].YValues = [values.EndValues[j]];
                        }
                        values.stackedSeries = false;
                        this.model.stackedValue[axis.name].push(values);
                        if (this.model.stackedValue[axis.name].min > Math.min.apply(0, values.StartValues))
                            this.model.stackedValue[axis.name].min = Math.min.apply(0, values.StartValues);
                        if (this.model.stackedValue[axis.name].max < Math.max.apply(0, values.EndValues))
                            this.model.stackedValue[axis.name].max = Math.max.apply(0, values.EndValues);
                        series.stackedValue = values;
                    }
                }
            }
        }
    },
    _legendItemBounds: function (itemCount, legendItemWidth, legendItemHeight) {
        var legend = this.model.legend;
        var legnedHeightIncr, column, legendWidth = 0, legendHeight = 0;
        if (ej.util.isNullOrUndefined(legend.columnCount) && legend.rowCount) {
            legnedHeightIncr = legend.rowCount;
            column = Math.ceil(itemCount / legnedHeightIncr);
            legendWidth = legendItemWidth * column;
            legendHeight = legendItemHeight * legnedHeightIncr;
        }
        else if (ej.util.isNullOrUndefined(legend.rowCount) && legend.columnCount) {
            legnedHeightIncr = Math.ceil(itemCount / legend.columnCount);
            legendWidth = legendItemWidth * this.model.legend.columnCount;
            legendHeight = legendItemHeight * legnedHeightIncr;
        }
        else if ((legend.rowCount) && (legend.columnCount)) {
            if (legend.columnCount < legend.rowCount) {
                legnedHeightIncr = legend.rowCount;
                column = Math.ceil(itemCount / legnedHeightIncr);
                legendWidth = legendItemWidth * column;
            }
            else {
                legnedHeightIncr = Math.ceil(itemCount / legend.columnCount);
                legendWidth = legendItemWidth * legend.columnCount;
            }
            legendHeight = legendItemHeight * legnedHeightIncr;
        }
        legendHeight += this.model.elementSpacing;
        return { LegendWidth: legendWidth, LegendHeight: legendHeight };
    },

    _calculateLegendBounds: function () {
        var chartObj = this;
        this.model.legendCollection = [];
        this.model.legendRegion = [];
        var width = 0, height = 0, legendItemWidth = 0, legendItemHeight = 0, legendWidth = 0, legendHeight = 0, legnedHeightIncr = 1;
        if (this.model._visibleSeries && this.model.legend.visible) {

            //calculating legend Position
            var legendsize;
            var style;
            var opacity;
            var colors = null;
            var colorGradName;
            if (this.model.AreaType == "none") {
                series = this.model._visibleSeries[0];
                var drawSeries = new ej.seriesTypes[series.type.toLowerCase()]();
                drawSeries._pieVisiblePoints(series);
                var visiblePoints = series.points;
                var point;
                var pointindex;
                var tempSeries;
                var legendFont;
                var legendItem;
                for (j = 0; j < visiblePoints.length; j++) {
                    point = visiblePoints[j];
                    pointindex = j;
                    tempSeries = { name: point.X, Font: series.font };
                    legendsize = chartObj._getLegendSize(tempSeries);
                    legendItemWidth = Math.max(legendsize.Width, legendItemWidth);
                    legendItemHeight = Math.max(legendsize.Height, legendItemHeight);

                }
                if (chartObj.model.legend.columnCount || chartObj.model.legend.rowCount) {
                    legendBounds = this._legendItemBounds(visiblePoints.length, legendItemWidth, legendItemHeight);
                    legendWidth = legendBounds.LegendWidth;
                    legendHeight = legendBounds.LegendHeight;
                }

                for (var k = 0; k < visiblePoints.length; k++) {
                    point = visiblePoints[k];
                    pointindex = k;
                    tempSeries = { name: point.X, Font: series.font };
                    legendsize = chartObj._getLegendSize(tempSeries);
                    if (!(chartObj.model.legend.rowCount) && !(chartObj.model.legend.columnCount)) {
                        if ((chartObj.model.legend.position.toLowerCase() == chartObj.model.legendPosition.top || chartObj.model.legend.position.toLowerCase() == chartObj.model.legendPosition.bottom || chartObj.model.legend.position.toLowerCase() == chartObj.model.legendPosition.custom)) {
                            width += legendsize.Width;
                            if (width > $(chartObj.svgObject).width()) {
                                width -= legendsize.Width;
                                legendWidth = Math.max(legendWidth, width);
                                width = legendsize.Width;
                                legnedHeightIncr++;
                                legendHeight = legendItemHeight * legnedHeightIncr;
                                legendHeight += this.model.elementSpacing;
                            }
                            else
                                legendWidth = Math.max(legendWidth, width);

                            height = Math.max(height, legendItemHeight);
                        }
                        else {
                            height += legendItemHeight;
                            legendWidth = Math.max(width, legendItemWidth);
                        }
                    }
                
                    var legend = (chartObj.model.legend);
                    opacity = legend.opacity;


                    if (legend.fill) {
                        colors = !(legend.fill._gradientStop) ? legend.fill : legend.fill._gradientStop;
                        }
                        else {
                            colors = chartObj.model.pointColors[pointindex];
                        }

                    if (point.visible && series.visibility.toLowerCase()=="visible")
                        colorGradName = chartObj.svgRenderer.createGradientElement('legend' + pointindex, colors, 0, 0, 0, $(chartObj.svgObject).height(), chartObj.gLegendEle);
                    else
                        colorGradName = 'gray';



                        if (series.font)

                            legendFont = series.font;

                        else
                            legendFont = chartObj.model.legend.font;

                        var bounds = (chartObj.model.legend.rowCount || chartObj.model.legend.columnCount) ? { Width: legendItemWidth, Height: legendItemHeight} : legendsize;
                        legendItem = { Text: point.X, Shape: chartObj.model.legend.shape, LegendStyle: { BorderColor: chartObj.model.legend.itemStyle.border.color, BorderWidth: chartObj.model.legend.itemStyle.border.width, Opacity: opacity, Color: colorGradName, Font: legendFont }, Bounds: bounds, SeriesIndex: 0, PointIndex: pointindex };
                        chartObj.model.legendCollection.push(legendItem);

                    
                }
            }
            else {
                var series;
                for (var i = 0; i < this.model._visibleSeries.length; i++) {
                    series = this.model._visibleSeries[i];
                    legendsize = chartObj._getLegendSize(series);
                    legendItemWidth = Math.max(legendsize.Width, legendItemWidth);
                    legendItemHeight = Math.max(legendsize.Height, legendItemHeight);
                }
                if (chartObj.model.legend.columnCount || chartObj.model.legend.rowCount) {
                    var legendBounds = this._legendItemBounds(this.model._visibleSeries.length, legendItemWidth, legendItemHeight);
                    legendWidth = legendBounds.LegendWidth;
                    legendHeight = legendBounds.LegendHeight;
                }
                for (var j = 0; j < this.model._visibleSeries.length; j++) {
                    index = j;
                    series = this.model._visibleSeries[j];
                    legendsize = chartObj._getLegendSize(series);
                    if (!chartObj.model.legend.rowCount && !chartObj.model.legend.columnCount) {
                        if (chartObj.model.legend.position.toLowerCase() == chartObj.model.legendPosition.top || chartObj.model.legend.position.toLowerCase() == chartObj.model.legendPosition.bottom || chartObj.model.legend.position.toLowerCase() == chartObj.model.legendPosition.custom) {
                            width += legendsize.Width;
                            if (width > $(chartObj.svgObject).width()) {
                                width -= legendsize.Width;
                                legendWidth = Math.max(legendWidth, width);
                                width = legendsize.Width;
                                legnedHeightIncr++;
                                legendHeight = legendItemHeight * legnedHeightIncr;
                                legendHeight += this.model.elementSpacing;
                            }
                            height = Math.max(height, legendsize.Height);
                        }


                        else {

                            height += legendItemHeight;
                            width = Math.max(width, legendItemWidth);
                        }
                    }
                     
                    var legend = (chartObj.model.legend);
                        opacity = legend.opacity;
                    

                    //                    // calculate gradient. 

                        if (legend.fill) {
                            colors = !(legend.fill._gradientStop) ? legend.fill : legend.fill._gradientStop;
                    }
                    else {
                        colors = chartObj.model.seriesColors[index];
                    }

                    if (series.visibility.toLowerCase() === 'visible')
                        colorGradName = chartObj.svgRenderer.createGradientElement('legend' + index, colors, 0, 0, 0, $(chartObj.svgObject).height(), chartObj.gLegendEle);
                    else
                        colorGradName = 'gray';

                    if (series.font)

                        legendFont = series.font;

                    else
                        legendFont = chartObj.model.legend.font;

                    bounds = (chartObj.model.legend.rowCount || chartObj.model.legend.columnCount) ? { Width: legendItemWidth, Height: legendItemHeight} : legendsize;
                    legendItem = { Text: (series.name) ? series.name : "series" + i, Shape: chartObj.model.legend.shape, LegendStyle: { BorderColor: chartObj.model.legend.itemStyle.border.color, BorderWidth: chartObj.model.legend.itemStyle.border.width, Opacity: opacity, Color: colorGradName, Font: legendFont }, Bounds: bounds, SeriesIndex: index };
                    chartObj.model.legendCollection.push(legendItem);
                }
            }
            if (this.model.legend.position.toLowerCase() === this.model.legendPosition.top || this.model.legend.position.toLowerCase() === this.model.legendPosition.bottom || this.model.legend.position.toLowerCase() === this.model.legendPosition.custom) {
                width += chartObj.model.legend.itemPadding;
                height += chartObj.model.legend.itemPadding;
                this.model.LegendBounds = { Width: Math.max(legendWidth, width), Height: Math.max(legendHeight, height), Rows: legnedHeightIncr };
            }
            else {
                width += chartObj.model.legend.itemPadding;
                height += chartObj.model.legend.itemPadding;
                this.model.LegendBounds = { Width: Math.max(legendWidth, width), Height: Math.max(legendHeight, height), Columns: legnedHeightIncr };
            }
            //Calculating legend viewer bounds or calculate user specified legend bounds
            var legendviewerHeight = 0, legendviewerWidth = 0, legendHeightTemp, legendWidthTemp;
            this.model.LegendViewerBounds = { Width: "0", Height: "0" };
            var position = this.model.legend.position.toLowerCase();
            if (this.model.legend.size.height == "" || this.model.legend.size.height==null) {
                if (position == this.model.legendPosition.left || position == this.model.legendPosition.right || position == this.model.legendPosition.custom)
                    legendHeightTemp = "100%";
                else
                    legendHeightTemp = "20%";
            }
            else
            {
                legendHeightTemp = this.model.legend.size.height;
            }
            if (legendHeightTemp.indexOf("%") != -1)
                legendHeightTemp = Math.abs(($(this.svgObject).height() / 100) * parseInt(legendHeightTemp));

            else
                legendHeightTemp = parseInt(legendHeightTemp);
            if (this.model.legend.size.width == "" || this.model.legend.size.width == null) {
                if (position == this.model.legendPosition.top || position == this.model.legendPosition.bottom || position == this.model.legendPosition.custom)
                    legendWidthTemp = "100%";
                else
                    legendWidthTemp = "20%";
            }
            else
            {
                legendWidthTemp = this.model.legend.size.width;
            }
            if (legendWidthTemp.indexOf("%") != -1)
                legendWidthTemp = Math.abs(($(this.svgObject).width() / 100) * parseInt(legendWidthTemp));

            else
                legendWidthTemp = parseInt(legendWidthTemp);
            if (this.model.legend.size.height == "" || this.model.legend.size.height == null)
                    this.model.LegendViewerBounds.Height = Math.min(legendHeightTemp, this.model.LegendBounds.Height);
                else
                    this.model.LegendViewerBounds.Height = legendHeightTemp;
            if (this.model.legend.size.width == "" || this.model.legend.size.width == null)
                    this.model.LegendViewerBounds.Width = Math.min(legendWidthTemp, this.model.LegendBounds.Width);
                else
                    this.model.LegendViewerBounds.Width = legendWidthTemp;
        }
        else {
            this.model.LegendViewerBounds = { Width: 0, Height: 0 };
        }

    },



    GetPointXYOrgin: function (x, y, orginX, orginY) {

        var xvalue = ((x - orginX) / (this.model.axes.PrimaryXaxis.visibleRange.max - orginX)) * (this.model.m_AreaBounds.Width);
        var yvalue = ((y - orginY) / (this.model.axes.PrimaryYaxis.visibleRange.max - orginY)) * (this.model.m_AreaBounds.Height - this.GetPointXY(this.model.axes.PrimaryXaxis.visibleRange.min, Math.max(this.model.axes.PrimaryYaxis.visibleRange.min, 0)).Y);

        return { X: xvalue, Y: yvalue };
    },






  


    _drawChartAreaRect: function () {

        var fillColor = !(this.model.background && this.model.background._gradientStop) ? this.model.background : 'url(#' + this.svgObject.id + '_backGradient)';

        if (this.model.backGroundImageUrl)
            fillColor = 'transparent';
        
        var chartBorder = this.model.border;
        
            var options = {
                'id': this.svgObject.id + '_SvgRect',
                'x': (chartBorder.width > 0) ? chartBorder.width / 2 : 0,
                'y': (chartBorder.width > 0) ? chartBorder.width / 2 : 0,
                'width': $(this.svgObject).width() - (2 * chartBorder.width),
                'height': $(this.svgObject).height() - (2 * chartBorder.width),
                'fill': fillColor,
                'opacity': chartBorder.opacity,
                'stroke-width': chartBorder.width,
                'stroke': chartBorder.color
            };
            this.svgRenderer.drawRect(options, this.svgObject);
        


        if (this.model.backGroundImageUrl)
            this._drawBackImage();

        if (this.model.AreaType == 'cartesianaxes' && !this.model.enable3D) {
            var borderOptions = {
                'id': this.svgObject.id + '_ChartArea',
                'x': this.model.m_AreaBounds.X,
                'y': this.model.m_AreaBounds.Y,
                'width': this.model.m_AreaBounds.Width,
                'height': this.model.m_AreaBounds.Height,
                'fill': this.model.chartArea.background,
                'stroke-width': this.model.chartArea.border.width,
                'opacity': this.model.chartArea.border.opacity,
                'stroke': this.model.chartArea.border.color
            };
            this.svgRenderer.drawRect(borderOptions, this.svgObject);


        }

    },
    axesIndexCount: function (axis,index) {
        var vRowcount = [];
        for (var k = 0; k < axis.length; k++) {
            var currentAxis = axis[k];
            if (currentAxis.orientation.toLowerCase() == "vertical") {
                if (currentAxis.rowIndex == index)
                    vRowcount.push(currentAxis);
            }
            else {
                if (currentAxis.columnIndex == index)
                    vRowcount.push(currentAxis);
            }
        }
        return vRowcount ;
    },
    axesCount: function (axis) {
        var vRowcount = [], indexValue=[], definitionsLength=0;
        var chartModel = this.model;
        $.each(axis, function (index, currentAxis) {
            if (currentAxis.orientation.toLowerCase() == "vertical") {
                definitionsLength =(chartModel.rowDefinitions)?chartModel.rowDefinitions.length:0;
                if (index == 0) vRowcount.push({ axis: currentAxis, index: currentAxis.rowIndex });
                else if (currentAxis.rowIndex != vRowcount[vRowcount.length - 1].axis.rowIndex)
                    vRowcount.push({ axis: currentAxis, index: currentAxis.rowIndex});
            }
            else {
                definitionsLength =(chartModel.columnDefinitions)? chartModel.columnDefinitions.length:0;
                if (index == 0) vRowcount.push({ axis: currentAxis, index: currentAxis.columnIndex });
                else if (currentAxis.columnIndex != vRowcount[vRowcount.length - 1].axis.columnIndex)
                    vRowcount.push({ axis: currentAxis, index: currentAxis.columnIndex });
            }
        });
        var length = vRowcount.length;
        if (definitionsLength > length) {
            length = definitionsLength;
            for (var l = 0; l < definitionsLength; l++)
                indexValue.push(l);
        } else {
            for (var i = 0; i < length; i++)
                indexValue.push(vRowcount[i].index);
        }
        
        return { length: length, indexValue: indexValue };

    },
   
     //Calculate position for each vertical axis
    _calRowSize: function () {
        var start = 0;
        var yaxisOrign = [];
        var totalRealLength = 0;
        var orginY = this.model.m_AreaBounds.Y + this.model.m_AreaBounds.Height;
        //Caluculate size for rowdefinitions values
        if (this.model._rowDefinitions[0].rowDefinitions != null) {
            for (var i = 0, rowMax = this.model._rowDefinitions.length; i < rowMax; i++) {
                var rowDef = this.model._rowDefinitions[i].rowDefinitions;
                
                var realLength = 0;
                if (rowDef.unit.toLowerCase() == "percentage")
                    realLength = Math.floor((this.model.m_AreaBounds.Height) * (rowDef.rowHeight / 100));
                else
                    realLength = rowDef.rowHeight;
                if (i == 0) yaxisOrign.push(orginY);
                   
                      yaxisOrign.push( (orginY - realLength - totalRealLength ));
                    totalRealLength += realLength;
                

            }
            
        }
        //Calculate size based on chart area size
        else {
            var height = Math.floor((this.model.m_AreaBounds.Height) / this.model._rowDefinitions.length);

            for (start = (orginY) ; start >= this.model.m_AreaBounds.Y; start = start - height) {
                yaxisOrign.push(start);
            }
        }
        return yaxisOrign;
    },
    
     //Calculate position for each horizontal axis
    _calColumnSize: function () {
        var xaxisOrign = [];
        var orginX = this.model.m_AreaBounds.X;
        var areaWidth = this.model.m_AreaBounds.Width;
        var totalRealLength = 0;
        var columnDef = this.model._columnDefinitions;
        //Caluculate size for columndefinitions values
        if (columnDef[0].columnDefinitions != null) {
            for (var i = 0, max = columnDef.length; i < max; i++) {
                var rowDef = columnDef[i].columnDefinitions;

                var realLength;
                if (rowDef.unit.toLowerCase() == "percentage")
                    realLength = Math.floor((areaWidth) * (rowDef.columnWidth / 100));
                else
                    realLength = rowDef.columnWidth;
                if (i == 0) xaxisOrign.push(orginX);

                xaxisOrign.push((orginX + realLength + totalRealLength));
                totalRealLength += realLength;
            }
        }
        //Caluculate size based on chart area size
        else {
            var width = Math.floor((areaWidth) / columnDef.length);
            for (var start = orginX, maxStart = orginX + areaWidth; start <= maxStart; start = start + width) {
                xaxisOrign.push(start);

            }
        }
        return xaxisOrign;
    },
    
     //Calculate position and size for each axis
    _calculateAxisSize: function () {

        //For Horizontal
        var isInitAxes = true, isOppInitAxes = true;
        var xaxisOrign = this._calColumnSize();
        var columnDefinition = this.model._columnDefinitions;
        for (var i = 0, maxLength = columnDefinition.length; i < maxLength; i++) {
            var x1 = xaxisOrign[i];
            var x2 = xaxisOrign[i + 1];
            var nearIndexVal = 0;
            var farIndexVal = 0;;
            var isColFirstAxes = true, isColOppFirstAxes = true;
            var oppY = this.model.m_AreaBounds.Y;
            var norY = oppY + this.model.m_AreaBounds.Height;
            for (var j = 0, maxCol = columnDefinition[i].axis.length; j < maxCol; j++) {
                var currentColAxis = columnDefinition[i].axis[j];
                var realY = 0, axisColumnSpace=0;
                if (!currentColAxis.opposedPosition) {
                    if (!(isColFirstAxes && (isInitAxes || nearIndexVal==0))) {
                        axisColumnSpace = columnDefinition.nearSizes[nearIndexVal];
                        realY = norY = (norY + axisColumnSpace);
                        nearIndexVal++;
                    }
                    else {
                        realY = norY;
                        isColFirstAxes = false;
                    }

                } else {
                    if (!(isColOppFirstAxes && (isOppInitAxes || farIndexVal==0))) {
                        axisColumnSpace = columnDefinition.farSizes[farIndexVal];
                        realY = oppY = (oppY - axisColumnSpace);
                        farIndexVal++;
                    }
                    else {
                        realY = oppY;
                        isColOppFirstAxes = false;
                    }
                    }
                if (currentColAxis.columnSpan) {
                    //Skip the axis size calculation for duplicate span axis
                    if (!(columnDefinition[i].index == currentColAxis.columnIndex))
                        continue;
                    var axisSize = (maxLength >= (i + currentColAxis.columnSpan)) ? i + currentColAxis.columnSpan : i+1;
                    x2 = xaxisOrign[axisSize];
                    if (!currentColAxis.opposedPosition && nearIndexVal==0)isInitAxes = isColFirstAxes;
                    if (currentColAxis.opposedPosition && farIndexVal == 0) isOppInitAxes = isColOppFirstAxes;
                } else {
                    x2 = xaxisOrign[i + 1];
                    isInitAxes = true;
                    isOppInitAxes = true;
                }
                
               currentColAxis.Location = { };
                currentColAxis.Location.X1 = x1 + currentColAxis.plotOffset;
                currentColAxis.Location.Y1 = realY;
                currentColAxis.Location.X2 = x2 - currentColAxis.plotOffset;
                currentColAxis.Location.Y2 = realY;
                currentColAxis.x = currentColAxis.Location.X1;
                currentColAxis.y = currentColAxis.Location.Y1;
                currentColAxis.width = (currentColAxis.Location.X2 - currentColAxis.Location.X1);
                currentColAxis.height = currentColAxis.Location.Y2 - currentColAxis.Location.Y1;

            }
        }
        
        //For Vertical
        isInitAxes = true, isOppInitAxes = true;
        var yaxisOrign = this._calRowSize();
        var rowDefinition = this.model._rowDefinitions;
        for (var i = 0, rowLength = rowDefinition.length; i < rowLength; i++) {
            var y1 = yaxisOrign[i];
            var y2 = yaxisOrign[i + 1];
            var nearIndex = 0;
            var farIndex = 0;
            var isFirstAxes = true, isOppFirstAxes = true;
           var x = this.model.m_AreaBounds.X;
           var norX = x, oppX = x + this.model.m_AreaBounds.Width;
           for (var j = 0, maxRow = rowDefinition[i].axis.length; j < maxRow; j++) {
                var axisSpace = 0;
                var realX = 0;
               
                var currentAxis = rowDefinition[i].axis[j];
               
               if (!currentAxis.opposedPosition) {
                   if (!(isFirstAxes && (isInitAxes || nearIndex==0))) {
                        axisSpace = rowDefinition.nearSizes[nearIndex];
                        realX = norX = (norX - axisSpace);
                        nearIndex++;
                    } else {
                        realX = x;
                        isFirstAxes = false;
                    }

                } else {
                   if (!(isOppFirstAxes && (isOppInitAxes|| isOppInitAxes==0))) {
                        axisSpace = rowDefinition.farSizes[farIndex];
                        realX = oppX = (oppX + axisSpace);
                        farIndex++;
                    } else {
                        realX = oppX;
                        isOppFirstAxes = false;
                    }
                }
                if (currentAxis.rowSpan) {
                    //Skip the axis size calculation for duplicate span axis
                    if (!(rowDefinition[i].index == currentAxis.rowIndex))
                        continue;
                    var axisRowSize = (rowLength >= (i + currentAxis.rowSpan)) ? i + currentAxis.rowSpan : i + 1;
                    y2 = yaxisOrign[axisRowSize];
                    if (!currentAxis.opposedPosition && nearIndex==0) isInitAxes = isFirstAxes;
                    if (currentAxis.opposedPosition && farIndex==0) isOppInitAxes = isOppFirstAxes;
                } else {
                    y2 = yaxisOrign[i + 1];
                    isInitAxes = true;
                    isOppInitAxes = true;
                }
                currentAxis.Location = { };
                currentAxis.Location.X1 = realX;
                currentAxis.Location.Y1 = y1 - currentAxis.plotOffset;
                currentAxis.Location.X2 = realX;
                currentAxis.Location.Y2 = y2 + currentAxis.plotOffset;
                currentAxis.x = currentAxis.Location.X1;
                currentAxis.y = currentAxis.Location.Y2;
                currentAxis.height = (currentAxis.Location.Y1 - currentAxis.Location.Y2);
                currentAxis.width = currentAxis.axisLine.width;

            }
        }
    },
    _arrangeAxis: function () {
        var chartobj = this;
        var axis;
        var customRow;
        //Generate columnIndex/RowIndex value if it is not specify in sample  
        for (var i = 0; i < this.model.axes.length; i++) {
            axis = this.model.axes[i];
            if (axis.orientation.toLowerCase() == "horizontal") {
                chartobj.model.hAxes.push(axis);
                axis.columnIndex = (ej.util.isNullOrUndefined(axis.columnIndex)) ? 0 : (axis.columnIndex);
                chartobj.model.hAxes[chartobj.model.hAxes.length - 1].columnIndex = axis.columnIndex;
            } else if (axis.orientation.toLowerCase() == "vertical") {

                chartobj.model.vAxes.push(axis);
                axis.rowIndex = (ej.util.isNullOrUndefined(axis.rowIndex)) ? 0 : (axis.rowIndex);
                chartobj.model.vAxes[chartobj.model.vAxes.length - 1].rowIndex = axis.rowIndex;
                customRow = (!ej.util.isNullOrUndefined(chartobj.model.rowDefinitions)) ? chartobj.model.rowDefinitions[axis.rowIndex] : null;
                if (customRow) {
                    axis.axisBottomLine = {};
                    axis.axisBottomLine.visible = true;
                    axis.axisBottomLine.lineWidth = customRow.lineWidth;
                    axis.axisBottomLine.color = customRow.lineColor;
                }
            }
        }

        this.model.hAxes = ej.DataManager(this.model.hAxes, ej.Query().sortBy("columnIndex")).executeLocal();

        this.model.vAxes = ej.DataManager(this.model.vAxes, ej.Query().sortBy("rowIndex")).executeLocal();
        
        //column axis information push into _rowDefinitions
        var rowCount = this.axesCount(chartobj.model.vAxes);
         chartobj.model._rowDefinitions = [];
        if (chartobj.model.vAxes.length>0) {
            for (var j = 0; j < rowCount.length; j++) {
                var rowCollection = chartobj.model.rowDefinitions;
                var rowDefinitions = ((rowCollection)) ? (rowCollection[j] ? rowCollection[j] : rowCollection[0]) : null;
                chartobj.model._rowDefinitions.push({axis: this.axesIndexCount(chartobj.model.vAxes, rowCount.indexValue[j])
                                , index: rowCount.indexValue[j], rowDefinitions: rowDefinitions });
            }
        }

        var rowDefinition = chartobj.model._rowDefinitions;
        //Using filter to get spanning row axis collection
        var spanningRow = this._axisFilter(rowDefinition, "rowSpan");
        //Arrange the entire axis for row span
        this._SpanningAxes(rowDefinition, spanningRow, true);

        //column axis information push into _columnDefinitions
        var columnCount = this.axesCount(chartobj.model.hAxes);
        chartobj.model._columnDefinitions = [];
        if (chartobj.model.hAxes.length > 0) {
            for (var k = 0; k < columnCount.length; k++) {
                var columnCollection = chartobj.model.columnDefinitions;
                var columnDefinitions = (columnCollection) ? (columnCollection[k] ? columnCollection[k] : columnCollection[0]) : null;
                chartobj.model._columnDefinitions.push({
                    axis: this.axesIndexCount(chartobj.model.hAxes, columnCount.indexValue[k])
                    , index: columnCount.indexValue[k], columnDefinitions: columnDefinitions });
            }
        }
        
        var columnDefinition = chartobj.model._columnDefinitions;
        //Using filter to get spanning column axis collection
        var spanningColumn = this._axisFilter(columnDefinition, "columnSpan");
        //Arrange the entire axis for column span
        this._SpanningAxes(columnDefinition, spanningColumn, false);
       
        this._axisSize();
        
        
    },
    
     //Arrange the entire axis for spanning
    _SpanningAxes: function (definition,spanningCollection, isRow) {
        for (var spanIndex = 0, spanMax = spanningCollection.length; spanIndex < spanMax; spanIndex++) {
            var currentAxis = spanningCollection[spanIndex];
             if (currentAxis == null) break;
             var span = isRow ? currentAxis.rowSpan : currentAxis.columnSpan;
             var index = isRow ? currentAxis.rowIndex : currentAxis.columnIndex;
             var axisIndex = definition[index].axis.indexOf(currentAxis);
             for (var k = 1, m = index + 1; k < span && m < definition.length; k++, m++) {
                 if (definition[m].axis.length >= axisIndex) {
                     //Insert duplicate span row to appropriate rowdefinition
                     definition[m].axis.splice(axisIndex, 0, currentAxis);
                 }
             }
         }
     },
    
     //Filter span axis from axes
    _axisFilter: function (definition, fieldName) {
        var axisResult = [];
        definition.filter(function (axes) {
             //Execute query to get spanning axis from axis array
              var axisCollection = ej.DataManager(axes["axis"], ej.Query().where(fieldName, ">", 1)).executeLocal();
              for (var i = 0, max = axisCollection.length; i < max; i++)
                  axisResult.push(axisCollection[i]);
        });
        return axisResult;
    },
    
    _axisSize: function () {
        //The below calcultion to get approximate length of axis
         var spaceValue = this._getLegendSpace();
         var hSpace = $(this.svgObject).width() - this.model.margin.left - this.model.margin.right - (this.model.elementSpacing * 2) - spaceValue.leftLegendWidth - spaceValue.rightLegendWidth;
        var hWidth = Math.floor(hSpace / this.axesCount(this.model.hAxes).length);

        var vSpace = $(this.svgObject).height() - this.model.margin.top - this.model.margin.bottom - (this.model.elementSpacing * 2) - spaceValue.topLegendHeight - spaceValue.bottomLegendHeight - spaceValue.modelTitleHeight-spaceValue.modelsubTitleHeight;
        var vWidth = Math.floor(vSpace / this.axesCount(this.model.vAxes).length);

       

        for (var j = 0; j < this.model.axes.length; j++) {
            axis = this.model.axes[j];
            if (axis.orientation.toLowerCase() == "horizontal") {
                if (this.model.AreaType != "polaraxes") {
                    var customColumn = (!ej.util.isNullOrUndefined(this.model.columnDefinitions)) ? this.model.columnDefinitions[axis.columnIndex] : null;
                    var columnLength = hWidth;
                    if (customColumn) {
                        if (customColumn.unit.toLowerCase() == "percentage")
                            columnLength = Math.floor((hWidth) * (customColumn.columnWidth / 100));
                        else
                            columnLength = customColumn.columnWidth;

                    }
                    axis.length = axis.columnSpan ? axis.columnSpan * columnLength : columnLength;
                }
                else {
                    if (hSpace > vSpace*2)
                        axis.length = hSpace - vSpace;
                    else
                        axis.length = hSpace;
                }
            } else if (axis.orientation.toLowerCase() == "vertical") {
                if (this.model.AreaType != "polaraxes") {
                    var customRow = (!ej.util.isNullOrUndefined(this.model.rowDefinitions)) ? this.model.rowDefinitions[axis.rowIndex] : null;
                    var realLength = vWidth;
                    if (customRow) {
                        if (customRow.unit.toLowerCase() == "percentage")
                            realLength = Math.floor((vWidth) * (customRow.rowHeight / 100));
                        else
                            realLength = customRow.rowHeight;

                    }
                    axis.length = axis.rowSpan ? (axis.rowSpan * realLength) : realLength;
                }
                else {
                    axis.length = vSpace / 2;
                }
            }
        }
    },
   _axisMeasure: function (axis, realWidth, orientation,j) {
        var axisTitleHeight =0; 
        var labelSize =0; 
		if(orientation == "vertical")
		{
		 labelSize= axis._LableMaxWidth.width;
		 axisTitleHeight =(axis.title.text == "" || !(axis.visible)) ? 0 : (ej.EjSvgRender.utils._measureText(axis.title.text, realWidth, axis.title.font).height + (2 * this.model.elementSpacing));
		}
		else
		{
		labelSize=(axis._LableMaxWidth.height);
	    axisTitleHeight =(axis.title.text == "" || !(axis.visible)) ? 0 : (ej.EjSvgRender.utils._measureText(axis.title.text, realWidth, axis.title.font).height + (this.model.elementSpacing));
		}
		var width;		
		if ((j == 0) && (axis.tickLinesPosition == 'inside' && axis.labelPosition == 'inside'))
		    axis.AxisMaxWidth = axisTitleHeight + axis.axisLine.width + (this.model.elementSpacing);
		else if ((j == 0) && (axis.tickLinesPosition == 'inside' && axis.labelPosition != 'inside'))
		    axis.AxisMaxWidth = axisTitleHeight + axis.axisLine.width + (this.model.elementSpacing) + labelSize;
		else if ((j == 0) && (axis.tickLinesPosition != 'inside' && axis.labelPosition == 'inside'))
		    axis.AxisMaxWidth = axisTitleHeight + axis.axisLine.width + (this.model.elementSpacing) + axis.majorTickLines.size;
		else
		    axis.AxisMaxWidth = axisTitleHeight + axis.axisLine.width + this.model.elementSpacing + axis.majorTickLines.size + labelSize;		
        return axis.AxisMaxWidth;
    },
     
    _arraySum: function (val) {
        var total = 0;
        for (var i = 0; i < val.length; i++) {
            total +=  parseFloat(val[i]) || 0;
        }
        return total;
    },
    _calSpace: function (realWidth, definitions,orientation) {
        var nearSizes = [];
        var farSizes = [];
        for (var i = 0; i < definitions.length; i++) {
            var nearIndex = 0, farIndex = 0;
            for (var j = 0; j < definitions[i].axis.length; j++) {
                var currentaxis = definitions[i].axis[j];
                var measureValue = this._axisMeasure(currentaxis, realWidth, orientation,j);
                if (!currentaxis.opposedPosition) {
                    if (nearSizes.length <= nearIndex) {
                        nearSizes.push(measureValue);
                    } else if (nearSizes[nearIndex] < (measureValue)) {
                        nearSizes[nearIndex] = measureValue;
                    }
                    nearIndex++;
                }
                else {
                    if (farSizes.length <= farIndex) {
                        farSizes.push(measureValue);
                    } else if (farSizes[farIndex] < (measureValue)) {
                        farSizes[farIndex] = measureValue;
                    }
                    farIndex++;
                }

            }
        }
        for (var i = 0; i < definitions.length; i++) {
            var nearIndex = 0;
            var labelsize = 0;
            for (var j = 0; j < definitions[i].axis.length; j++) {
                var currentoppaxis = definitions[i].axis[j];
                if (!currentoppaxis.opposedPosition) {
                    if (j != 0) {
                        if (currentoppaxis.orientation == "Vertical") {
                            labelSize = currentoppaxis._LableMaxWidth.width;
                            
                        }
                        else {
                            labelSize = currentoppaxis._LableMaxWidth.height;                            
                        }

                        if (currentoppaxis.labelPosition == 'inside' && currentoppaxis.tickLinesPosition == 'inside') {
                            nearSizes[nearIndex] -= (labelSize + currentoppaxis.majorTickLines.size);
                            break;
                        }
                        else if (currentoppaxis.labelPosition != 'inside' && currentoppaxis.tickLinesPosition == 'inside') {
                            nearSizes[nearIndex] -= currentoppaxis.majorTickLines.size;
                            break;
                        }
                        else if (currentoppaxis.labelPosition == 'inside' && currentoppaxis.tickLinesPosition != 'inside') {
                            nearSizes[nearIndex] -= labelSize;
                            break;
                        }
                        else {
                            nearSizes[nearIndex] = nearSizes[nearIndex];
                            break;
                        }
                    }
                    else
                        break;
                }

            }
            break;
        }
        for (var i = 0; i < definitions.length; i++) {
            var farIndex = 0;
            for (var j = 0; j < definitions[i].axis.length; j++) {                
                var currentoppaxis = definitions[i].axis[j];                
                if (currentoppaxis.opposedPosition) {
                    if (j != 0) {
                        if (currentoppaxis.orientation == "Vertical") {
                            labelSize = currentoppaxis._LableMaxWidth.width;

                        }
                        else {
                            labelSize = currentoppaxis._LableMaxWidth.height;
                        }
                        if (currentoppaxis.labelPosition == 'inside' && currentoppaxis.tickLinesPosition == 'inside') {
                            farSizes[farIndex] -= (labelSize + currentoppaxis.majorTickLines.size+5);
                            break;
                        }
                        else if (currentoppaxis.labelPosition != 'inside' && currentoppaxis.tickLinesPosition == 'inside') {
                            farSizes[farIndex] -= currentoppaxis.majorTickLines.size;
                            break;
                        }
                        else if (currentoppaxis.labelPosition == 'inside' && currentoppaxis.tickLinesPosition != 'inside') {
                            farSizes[farIndex] -= labelSize;
                            break;
                        }
                        else {
                            farSizes[farIndex] = farSizes[farIndex];
                            break;
                        }
                    }
                    else
                        break;
                }              
                
             }
            break;
        }
        return { nearSizes: nearSizes, farSizes: farSizes };
    },
    
    _getLegendSpace:function () {
        var position = this.model.legend.position.toLowerCase();
        var legendPosition = this.model.legendPosition;
       var ltheight = 0;
        var space = (this.model.LegendViewerBounds.Width + (this.model.legend.itemPadding / 2) + this.model.elementSpacing);
        var leftLegendWidth = position == legendPosition.left ? space : 0;
        var rightLegendWidth = position == legendPosition.right ? space : 0;

        if (this.model.legend.title.text) {
            ltheight = ej.EjSvgRender.utils._measureText(this.model.legend.title, null, this.model.legend.title.font).height;          
        }
        var topLegendHeight = position == legendPosition.top ? (this.model.LegendViewerBounds.Height + ltheight) : 0;
        var bottomLegendHeight = position == legendPosition.bottom ? (this.model.LegendViewerBounds.Height + ltheight) : 0;
        var modelTitleHeight = (this.model.title.text == "") ? 0 : (ej.EjSvgRender.utils._measureText(this.model.title.text, $(this.svgObject).width() - this.model.margin.left - this.model.margin.right, this.model.title.font).height + this.model.elementSpacing);
    
         var modelsubTitleHeight = (this.model.title.subTitle.text == "") ? 0 : (ej.EjSvgRender.utils._measureText(this.model.title.subTitle.text, $(this.svgObject).width() - this.model.margin.left - this.model.margin.right, this.model.title.subTitle.font).height + this.model.elementSpacing) ;
       
        return {
            leftLegendWidth: leftLegendWidth, rightLegendWidth: rightLegendWidth, topLegendHeight: topLegendHeight, bottomLegendHeight: bottomLegendHeight, modelTitleHeight: modelTitleHeight, modelsubTitleHeight: modelsubTitleHeight};


    },
    _calculateAreaBounds: function () {
        var chartobj = this;
        var realWidth = $(this.svgObject).width() - this.model.margin.left - this.model.margin.right;
        // Calculate area bounds X and width
        var chartBorderWidth = chartobj.model.border.width;

        //Calcultion for multiple axes(vertical)
        var sizes = this._calSpace(realWidth, chartobj.model._rowDefinitions,"vertical");
        chartobj.model._rowDefinitions.nearSizes = sizes.nearSizes;
        chartobj.model._rowDefinitions.farSizes = sizes.farSizes;
        var vAxesWidth = this._arraySum(sizes.nearSizes);
        var vAxesOppWidth = this._arraySum(sizes.farSizes);

        var spaceValue=this._getLegendSpace();
       
        var x = vAxesWidth + spaceValue.leftLegendWidth + chartobj.model.margin.left + chartBorderWidth;     
        var rightSpacing = vAxesOppWidth + spaceValue.rightLegendWidth + chartobj.model.margin.right + chartobj.model.margin.left + (2 * chartBorderWidth);
        var width = $(this.svgObject).width();
        var boundsWidth =  width - (x + rightSpacing);
        if (boundsWidth < 0) {
            $(this.svgObject).width(width - boundsWidth + 1);
            this._calculateAreaBounds();
        }
        else {
        // Calculate area bounds Y and Height   
        
        //Calcultion for multiple axes(Horizontal)
        var columnSizes = this._calSpace(realWidth, chartobj.model._columnDefinitions,"horizontal");
        chartobj.model._columnDefinitions.nearSizes = columnSizes.nearSizes;
            chartobj.model._columnDefinitions.farSizes = columnSizes.farSizes;
            var hAxesWidth = this._arraySum(columnSizes.nearSizes);
            var hAxesOppWidth = this._arraySum(columnSizes.farSizes);
         
        var y = hAxesOppWidth  + this.model.margin.top+ (this.model.elementSpacing) + spaceValue.modelTitleHeight + (spaceValue.modelsubTitleHeight)  + spaceValue.topLegendHeight + chartBorderWidth;
            var bottomSpacing = hAxesWidth + this.model.margin.bottom + spaceValue.bottomLegendHeight + (2 * chartBorderWidth);
            var boundsHeight = Math.abs($(this.svgObject).height() - (y + bottomSpacing));

            this.model.m_AreaBounds = { X: x, Y: y, Width: boundsWidth, Height: boundsHeight };
            this.model.m_Spacing = { Left: x, Top: y, Right: rightSpacing, Bottom: bottomSpacing };

            if (this.model.enableCanvasRendering) {  // for adding bounds in canvas rendering
                this.canvasX = x;
                this.canvasY = y;
                this.canvasWidth = boundsWidth;
                this.canvasHeight = boundsHeight;
            } else 
                this.canvasX = this.canvasY = this.canvasHeight = this.canvasWidth = 0;
        }
      }
};

jQuery.uaMatch = function (ua) {
    ua = ua.toLowerCase();

    var match = /(chrome)[ \/]([\w.]+)/.exec(ua) ||
        /(webkit)[ \/]([\w.]+)/.exec(ua) ||
        /(opera)(?:.*version|)[ \/]([\w.]+)/.exec(ua) ||
        /(msie) ([\w.]+)/.exec(ua) ||
        ua.indexOf("compatible") < 0 && /(mozilla)(?:.*? rv:([\w.]+)|)/.exec(ua) ||
        [];

    return {
        browser: match[1] || "",
        version: match[2] || "0"
    };
};


 
})(jQuery);;
/**
* @fileOverview Plugin to style the Html Chart elements
* @copyright Copyright Syncfusion Inc. 2001 - 2013. All rights reserved.
*  Use of this code is subject to the terms of our license.
*  A copy of the current license can be obtained at any time by e-mailing
*  licensing@syncfusion.com. Any infringement will be prosecuted under
*  applicable laws. 
* @version 12.1 
* @author <a href="mailto:licensing@syncfusion.com">Syncfusion Inc</a>
*/

(function ($, ej) {
    /**
     * @class ejChart
     * @requires jQuery.js
     * @requires ej.core.js
     * @requires ej.data.js
     * @requires ej.chart.js
     * @requires jQuery.globalize.js
     * @classdesc The chart can be easily configured to the DOM element, such as div. you can create a grid with a highly customizable look and feel.
     * @example 
     * &lt;div id="container"&gt;&lt;/div&gt; <br> 
     * &lt;script&gt;
     * // Create Chart
     * $('#container').ejChart(); 	
     * &lt;/script&gt;
     */

    ej.widget("ejChart", "ej.datavisualization.Chart", /** @lends ejChart# */{

        // widget element will be automatically set in this
        element: null,

        // user defined model will be automatically set in this
        model: null,

        validTags: ["div"],

        // default model
        defaults: /** @lends ejChart# */ {
            /**		
			* Options for customizing the color, opacity and width of the border of chart.
			* @type {object}
			*/
            border: /** @lends ejChart# */ {
                /**		
			* Specifies the border color of the chart.
            * @default null
			* @type {string}
			* @example  				
			* &lt;div id="container"&gt;&lt;/div&gt; 
            * &lt;script&gt;
            * $("#container").ejChart({
            *    border: { color: "green" }                      
            * });
            * &lt;/script&gt;                
			* @alias ejChart#border->color
			*/
                color: 'transparent',
                /**		
			* Specifies the border width of the chart.
			* @default 0
			* @type {number}		 
			* @example 
			* &lt;div id="container"&gt;&lt;/div&gt; 
            * &lt;script&gt;
            * $("#container").ejChart({
            *    border: { width: 2 }                      
            * });
            * &lt;/script&gt; 
		    * @alias ejChart#border->width
			*/
                width: 0,
                /**		
			* Specifies the border opacity of the chart
			* @default 0.3
			* @type {number}			
			* @example 
			* &lt;div id="container"&gt;&lt;/div&gt; 
            * &lt;script&gt;
            * $("#container").ejChart({
            *    border: { opacity: 0.5 }                      
            * });
            * &lt;/script&gt; 
			 * @alias ejChart#border->opacity
			*/
                opacity: 0.3
            },
            /**		
			* Specifies options for configuring the border and background for chart plotting area.
			* @type {object}
			*/
            chartArea: /** @lends ejChart# */
            {
                /**		
			* Options for customizing the color, opacity and width of the border of chart plotting area.
			* @type {object}
			 * @alias ejChart#chartArea->border
			*/
                border: {
                    /**		
          * Specifies the border color of chart plotting area.
          * @default Gray
          * @type {string}				
          * @example 
          * &lt;div id="container"&gt;&lt;/div&gt; 
          * &lt;script&gt;
          * $("#container").ejChart({
          *    chartArea: { border: { color :"green"} }                      
          * });
          * &lt;/script&gt; 
          * @alias ejChart#chartArea->border->color
          */
                    color: 'Gray',
                    /**		
          * Specifies the border width of chart plotting area.
          * @default 0.5
          * @type {number}
          * @example 
          * &lt;div id="container"&gt;&lt;/div&gt; 
          * &lt;script&gt;
          * $("#container").ejChart({
          *     chartArea: { border: { width : 0.2} }                      
          * });
          * &lt;/script&gt; 
          * @alias ejChart#chartArea->border->width
          */
                    width: 0.5,
                    /**		
          * Specifies the border opacity of chart plotting area.
          * @default 0.3
          * @type {number}	
          * @example 
          * &lt;div id="container"&gt;&lt;/div&gt; 
          * &lt;script&gt;
          * $("#container").ejChart({
          *     chartArea: { border: { opacity : 0.5} }                      
          * });
          * &lt;/script&gt; 
          * @alias ejChart#chartArea->border->opacity
          */
                    opacity: 0.3
                },
                /**		
           * Specifies the background for chart plotting area.
           * @default transparent
           * @type {string}				
           * @example 
           * &lt;div id="container"&gt;&lt;/div&gt; 
           * &lt;script&gt;
           * $("#container").ejChart({
           *     chartArea: { background : "white" }                      
           * });
           * &lt;/script&gt; 
           * @alias ejChart#chartArea->background
           */
                background: 'transparent'

            },
            /**		
			* This is a horizontal axis which contains options to configure axis. This is the primary x axis for all the series in series array. If you need to override x axis for particular series, you can create an axis object by providing unique name using name property and add it to axes array. Then, assign the name to the series’s xAxisName property to link both axis and series.
			* @type {object}
			*/
            primaryXAxis: /** @lends ejChart# */
     {
         /**		
* Specifies the position of the axis labels.
* @default "outside"
* @type {enum}				
* @example 
* &lt;div id="container"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#container").ejChart({
      * primaryXAxis: { labelPosition : "inside" }                       
      * });
      * &lt;/script&gt; 
 * @alias ejChart#primaryXAxis->labelPosition
*/
         labelPosition: 'outside',
		  /**		
* Specifies the position of the axis tick lines.
* @default "outside"
* @type {enum}				
* @example 
* &lt;div id="container"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#container").ejChart({
      * primaryXAxis: { tickLinesPosition : "inside" }                       
      * });
      * &lt;/script&gt; 
 * @alias ejChart#primaryXAxis->tickLinesPosition
*/
         tickLinesPosition: 'outside',
     /**		
    * This is a horizontal axis alternate grid band which contains options to highlight alternate grid bands.
    * @type {object}
  * @alias ejChart#primaryXAxis->alternateGridBand
  */
 alternateGridBand:
            {
/**		
* Specifies the Odd Alternative Grid Band.
* @type {object}
  * @alias ejChart#primaryXAxis->alternateGridBand->odd
  */
  odd: {
/**		
* Specifies the fill color of Odd grid bands
* @default transparent
* @type {string}				     
* @example 
* &lt;div id="container"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#container").ejChart({
      * primaryXAxis: { alternateGridBand: { odd :{ fill : "red" } } }                      
      * });
      * &lt;/script&gt; 
 * @alias ejChart#primaryXAxis->alternateGridBand->odd->fill
*/
                    fill: "transparent",
                    /**		
* Specifies the opacity of the Odd grid band.
* @default 1
* @type {number}				
* @example 
* &lt;div id="container"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#container").ejChart({
      * primaryXAxis: { alternateGridBand: { odd :{ opacity : 0.5 } } }                      
      * });
      * &lt;/script&gt; 
 * @alias ejChart#primaryXAxis->alternateGridBand->odd->opacity
*/
                    opacity: 1
                },
/**		
* Specifies the Even Alternative Grid Band.
* @type {object}
 * @alias ejChart#primaryXAxis->alternateGridBand->even
  */
                even: {
                    /**		
* Specifies the fill color of Even grid bands
* @default transparent
* @type {string}				     
* @example 
* &lt;div id="container"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#container").ejChart({
      * primaryXAxis: { alternateGridBand: { even :{ fill : "green" } } }                    
      * });
      * &lt;/script&gt; 
 * @alias ejChart#primaryXAxis->alternateGridBand->even->fill
*/

                    fill: "transparent",
                    /**		
* Specifies the opacity of the Even grid band.
* @default 1
* @type {number}				
* @example 
* &lt;div id="container"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#container").ejChart({
      * primaryXAxis: { alternateGridBand: { even :{ opacity : 0.5 } } }                   
      * });
      * &lt;/script&gt; 
 * @alias ejChart#primaryXAxis->alternateGridBand->even->opacity
*/
                    opacity: 1
                }
            },
			 /**		
     * Options for configuring major grid lines color, width, dash arrays etc., 
     * @type {object}
     * @alias ejChart#primaryXAxis->majorGridLines
     */
         majorGridLines:
{
    /**		
* Specifies the width of the major grid lines.
* @default 1
* @type {number}			
* @example 
* &lt;div id="container"&gt;&lt;/div&gt; 
 * &lt;script&gt;
 * $("#container").ejChart({
 *     primaryXAxis: { majorGridLines: { width : 0.5} }                      
 * });
 * &lt;/script&gt; 
 * @alias ejChart#primaryXAxis->majorGridLines->width
*/
    width: 1,
    /**		
* Controls the pattern of dashes and gaps used to stroke the major grid lines.
* @default null
* @type {string}				
* @example 
* &lt;div id="container"&gt;&lt;/div&gt; 
 * &lt;script&gt;
 * $("#container").ejChart({
 *     primaryXAxis: { majorGridLines: { dashArray : "2,3"} }                      
 * });
 * &lt;/script&gt; 
 * @alias ejChart#primaryXAxis->majorGridLines->dashArray
*/
    dashArray: "",
    /**		
* Specifies the visibility of the major grid lines.
* @default true
* @type {boolean}			
* @example 
* &lt;div id="container"&gt;&lt;/div&gt; 
* &lt;script&gt;
* $("#container").ejChart({
* primaryXAxis: { majorGridLines: { visible: false } }                      
* });
* &lt;/script&gt; 
* @alias ejChart#primaryXAxis->majorGridLines->visible
*/
    visible: true,
    /**		
* Specifies the opacity of the major grid lines.
* @default 1
* @type {number}				
* @example 
* &lt;div id="container"&gt;&lt;/div&gt; 
* &lt;script&gt;
* $("#container").ejChart({
* primaryXAxis: { majorGridLines: { opacity: 0.5 } }                      
* });
* &lt;/script&gt; 
* @alias ejChart#primaryXAxis->majorGridLines->opacity
*/
    opacity: 1

},
         /**		
     * Options for configuring major tick lines color, width, dash arrays etc.,
     * @type {object}
 * @alias ejChart#primaryXAxis->majorTickLines
     */
         majorTickLines:
 {
     /**		
* Specifies the width of the major tick lines.
* @default 1
* @type {number}				
* @example 
* &lt;div id="container"&gt;&lt;/div&gt; 
* &lt;script&gt;
* $("#container").ejChart({
* primaryXAxis: { majorTickLines: { width: 2 } }                      
* });
* &lt;/script&gt; 
 * @alias ejChart#primaryXAxis->majorTickLines->width
*/
     width: 1,
     /**		
* Length of the major tick lines.
* @default 5
* @type {number}			
* @example 
* &lt;div id="container"&gt;&lt;/div&gt; 
* &lt;script&gt;
* $("#container").ejChart({
* primaryXAxis: { majorTickLines: { size: 2 } }                      
* });
* &lt;/script&gt; 
* @alias ejChart#primaryXAxis->majorTickLines->size
*/
     size: 5,
     /**		
* Specifies the visibility of the major tick lines.
* @default true
* @type {boolean}				
* @example 
* &lt;div id="container"&gt;&lt;/div&gt; 
* &lt;script&gt;
* $("#container").ejChart({
* primaryXAxis: { majorTickLines: { visible: false } }                      
* });
* &lt;/script&gt;  
 * @alias ejChart#primaryXAxis->majorTickLines->visible
*/
     visible: true
 },
         /**		
     * Options for configuring minor grid lines color, width, dash arrays etc., 
     * @type {object}
      * @alias ejChart#primaryXAxis->minorGridLines
     */
         minorGridLines:
 {
     /**		
* Specifies the width of the minorGridLines.
* @default 1
* @type {number}				
* @example 
* &lt;div id="container"&gt;&lt;/div&gt; 
* &lt;script&gt;
* $("#container").ejChart({
* primaryXAxis: { minorGridLines: { width: 2 } }                      
* });
* &lt;/script&gt;  
* @alias ejChart#primaryXAxis->minorGridLines->width
*/
     width: 1,
     /**		
* Controls the pattern of dashes and gaps used to stroke the minor grid lines.
* @default null
* @type {string}			
* @example 
* &lt;div id="container"&gt;&lt;/div&gt; 
* &lt;script&gt;
* $("#container").ejChart({
* primaryXAxis: { minorGridLines: { dashArray: "2,3" } }                      
* });
* &lt;/script&gt;
* @alias ejChart#primaryXAxis->minorGridLines->dashArray
*/
     dashArray: "",
     /**		
* Specifies the visibility of the minorGridLines. The user can modify the visibility of the minorGridLines in primaryXAxis.
* @default false
* @type {boolean}				
* @example 
* &lt;div id="container"&gt;&lt;/div&gt; 
* &lt;script&gt;
* $("#container").ejChart({
* primaryXAxis: { minorGridLines: { visible: true } }                      
* });
* &lt;/script&gt; 
* @alias ejChart#primaryXAxis->minorGridLines->visible
*/
     visible: true

 },
         /**		
     * Options for configuring minor tick lines color, width, dash arrays etc,.
     * @type {object}
     * @alias ejChart#primaryXAxis->minorTickLines
     */
         minorTickLines:
{
    /**		
* Specifies the width of the minor tick lines.
* @default 1
* @type {number}			
* @example 
* &lt;div id="container"&gt;&lt;/div&gt; 
* &lt;script&gt;
* $("#container").ejChart({
* primaryXAxis: { minorTickLines: { width: 2 } }                      
* });
* &lt;/script&gt; 
* @alias ejChart#primaryXAxis->minorTickLines->width
*/
    width: 1,
    /**		
* Length of the minor tick lines.
* @default 5
* @type {number}				
* @example 
* &lt;div id="container"&gt;&lt;/div&gt; 
* &lt;script&gt;
* $("#container").ejChart({
* primaryXAxis: { minorTickLines: { size: 2 } }                      
* });
* &lt;/script&gt; 
* @alias ejChart#primaryXAxis->minorTickLines->size
*/
    size: 5,
    /**		
* Specifies the visibility of the minor tick lines.
* @default false
* @type {boolean}				
* @example 
* &lt;div id="container"&gt;&lt;/div&gt; 
* &lt;script&gt;
* $("#container").ejChart({
* primaryXAxis: { minorTickLines: { visible: true } }                      
* });
* &lt;/script&gt; 
* @alias ejChart#primaryXAxis->minorTickLines->visible
*/
    visible: true
},
         /**		
      * Specifies the number of minor ticks to render per major tick/interval.
      * @default null
      * @type {number}			
      * @example 
      * &lt;div id="container"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#container").ejChart({
      * primaryXAxis: { minorTicksPerInterval: 5 }                      
      * });
      * &lt;/script&gt; 
      * @alias ejChart#primaryXAxis->minorTicksPerInterval
      */
         minorTicksPerInterval: null,
         /**		
     * Index of the column object in columnDefinitions array to which this axis is associated with.
     * @default null
     * @type {number}			
     * @example 
     * &lt;div id="container"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#container").ejChart({
      * primaryXAxis: { columnIndex: 2 }                      
      * });
      * &lt;/script&gt; 
     * @alias ejChart#primaryXAxis->columnIndex
     */
         columnIndex: null,
         /**		
      * An axis can be spanned along multiple columns or plotting areas. This property specifies the number of columns/plotting areas this axis should be spanned.
      * @default null
      * @type {number}			
      * @example 
      * &lt;div id="container"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#container").ejChart({
      * primaryXAxis: { columnSpan: 2 }                      
      * });
      * &lt;/script&gt; 
      * @alias ejChart#primaryXAxis->columnSpan
      */
         columnSpan: null,
         /**		
      * Rotates the axis labels by the specified angle. Angle must be given in degrees.
      * @default null
      * @type {number}			
      * @example 
      * &lt;div id="container"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#container").ejChart({
      * primaryXAxis: { labelRoatation: 90 }                      
      * });
      * &lt;/script&gt; 
      * @alias ejChart#primaryXAxis->labelRotation
      */
         labelRotation: null,
         /**		
     * You can plot any type of data like date time, double, string etc., This property determines the type of data that this axis will handle. See {@link ValueType}
     * @default null
     * @type {enum}			
     * @example 
      * &lt;div id="container"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#container").ejChart({
      * primaryXAxis: { valueType: "double" }                      
      * });
      * &lt;/script&gt;  
     * @alias ejChart#primaryXAxis->valueType
     */
         valueType: null,
         /**		
       * Unique name of the axis.
       * @default null
       * @type {string}			
       * @example 
       * &lt;div id="container"&gt;&lt;/div&gt; 
       * &lt;script&gt;
       * $("#container").ejChart({
       * primaryXAxis: { name: "xAxis" }                      
       * });
       * &lt;/script&gt;  
       * @alias ejChart#primaryXAxis->name
       */
         name: null,
         /**		
      * Provides the format for axis labels.
      * @default null
      * @type {string}				
      * @example 
       * &lt;div id="container"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#container").ejChart({
      * primaryXAxis: { labelFormat: "{value}%" }                      
      * });
      * &lt;/script&gt; 
      * @alias ejChart#primaryXAxis->labelFormat
      */
         labelFormat: null,
         /**		
     * Specifies the desired number of intervals for the range. With this setting, you can request axis to calculate nice numbers such that they result in the number of intervals approximately equal to your desired interval.
     * @default null
     * @type {number}				
     * @example 
      * &lt;div id="container"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#container").ejChart({
      * primaryXAxis: { desiredIntervals: 5 }                      
      * });
      * &lt;/script&gt; 
     * @alias ejChart#primaryXAxis->desiredIntervals
     */
         desiredIntervals: null,
         /**		
      * Specifies whether the type of the interval calculated is in days, months, hours, minutes etc., This is applicable only when valueType is ‘datetime’. See {@link IntervalType}
      * @default null
      * @type {enum}			
      * @example 
       * &lt;div id="container"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#container").ejChart({
      * primaryXAxis: { intervalType: "days" }                      
      * });
      * &lt;/script&gt; 
       * @alias ejChart#primaryXAxis->intervalType
      */
         intervalType: null,
         /**		
      * Rounds the axis labels to the specified number of decimals.
      * @default null
      * @type {number}				
      * @example 
       * &lt;div id="container"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#container").ejChart({
      * primaryXAxis: { roundingPlaces: 3 }                      
      * });
      * &lt;/script&gt;  
        * @alias ejChart#primaryXAxis->roundingPlaces
      */
         roundingPlaces: null,
         /**		
      * Specifies the logarithmic base value. This is applicable only when valueType is logarithmic.
      * @default 10
      * @type {number}			
      * @example 
      * &lt;div id="container"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#container").ejChart({
      * primaryXAxis: { logBase: 5 }                      
      * });
      * &lt;/script&gt; 
       * @alias ejChart#primaryXAxis->logBase
      */
         logBase: 10,
         /**		
      * Horizontal/vertical padding for the plotting area based on the orientation of the axis.
      * @default 10
      * @type {number}			
      * @example 
      * &lt;div id="container"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#container").ejChart({
      * primaryXAxis: { plotOffset: 0 }                      
      * });
      * &lt;/script&gt; 
      * @alias ejChart#primaryXAxis->plotOffset
      */
         plotOffset: 0,
         labels: [],
         /**		
             * Options to configure stripLine’s color, text, border etc.,
             * @name ejChart#primaryXAxis->stripLine 
             * @default [ ]
             * @type {array}
             */
         stripLine: [
                {
                    /**		
     * Specifies the visibility of the stripLine.
     * @default false
     * @type {boolean}			
     * @example 
     * &lt;div id="container"&gt;&lt;/div&gt; 
     * &lt;script&gt;
     * $("#container").ejChart({
     * primaryXAxis: { stripLine:[{ visible : true }]}                      
     * });
     * &lt;/script&gt; 
     * @alias ejChart#primaryXAxis->stripLine->visible
     */
                    visible: false,
                    /**		
   * Indicates whether to render the stripLine from the minimum/start value of the axis.
   * @default false
   * @type {boolean}			
   * @example 
   * &lt;div id="container"&gt;&lt;/div&gt; 
   * &lt;script&gt;
   * $("#container").ejChart({
   * primaryXAxis: { stripLine:[{ startFromAxis : true }]}                      
   * });
   * &lt;/script&gt; 
   * @alias ejChart#primaryXAxis->stripLine->startFromAxis
   */
                    startFromAxis: false,
                    /**		
   * Text to be displayed inside the stripLine.
   * @default "stripLine"
   * @type {string}			
   * @example 
   * &lt;div id="container"&gt;&lt;/div&gt; 
   * &lt;script&gt;
   * $("#container").ejChart({
   * primaryXAxis: { stripLine:[{ text : "Empty Point" }]}                      
   * });
   * &lt;/script&gt; 
   * @alias ejChart#primaryXAxis->stripLine->text
   */
                    text: "stripLine",
                    /**		
   * Alignment of the text displayed in stripLine. See {@link TextAlignment}
   * @default "middlecenter"
   * @type {enum}			
   * @example 
   * &lt;div id="container"&gt;&lt;/div&gt; 
   * &lt;script&gt;
   * $("#container").ejChart({
   * primaryXAxis: { stripLine:[{ textAlignment : "middletop" }]}                      
   * });
   * &lt;/script&gt; 
   * @alias ejChart#primaryXAxis->stripLine->textAlignment
   */
                    textAlignment: "middlecenter",
                    /**		
    * Specifies the font of stripLine in primaryXAxis. These are font properties for stripLine text
    * @type {object}
    * @alias ejChart#primaryXAxis->stripLine->font
    */
                    font: {
                        /**		
  * Specifies the fontFamily of stripLine in primaryXAxis. Stripline text render with this fontFamily
  * @default "middlecenter"
  * @type {enum}			
  * @example 
  * &lt;div id="container"&gt;&lt;/div&gt; 
  * &lt;script&gt;
  * $("#container").ejChart({
  * primaryXAxis: { stripLine:[{ font : { fontFamily : "Algerian"} }]}                      
  * });
  * &lt;/script&gt; 
  * @alias ejChart#primaryXAxis->stripLine->font->fontFamily
  */
                        fontFamily: 'Segoe UI',
                        /**		
 * Specifies the fontStyle of stripLine in primaryXAxis. Stripline text render with this fontStyle. See {@link FontStyle}
 * @default "Normal"
 * @type {enum}			
 * @example 
 * &lt;div id="container"&gt;&lt;/div&gt; 
 * &lt;script&gt;
 * $("#container").ejChart({
 * primaryXAxis: { stripLine:[{ font : { fontStyle: "Bold"} }]}                      
 * });
 * &lt;/script&gt; 
 * @alias ejChart#primaryXAxis->stripLine->font->fontStyle
 */
                        fontStyle: 'Normal',
                        /**		
 * Specifies the size of stripLine in primaryXAxis. Stripline text render with this size
 * @default "12px"
 * @type {string}			
 * @example 
 * &lt;div id="container"&gt;&lt;/div&gt; 
 * &lt;script&gt;
 * $("#container").ejChart({
 * primaryXAxis: { stripLine:[{ font : { size: "15px"} }]}                      
 * });
 * &lt;/script&gt; 
 * @alias ejChart#primaryXAxis->stripLine->font->size
 */
                        size: '12px',
                        /**		
 * Specifies the fontWeight of stripLine in primaryXAxis. Stripline text render with this fontWeight
 * @default "regular"
 * @type {string}			
 * @example 
 * &lt;div id="container"&gt;&lt;/div&gt; 
 * &lt;script&gt;
 * $("#container").ejChart({
 * primaryXAxis: { stripLine:[{ font : { fontWeight: "lighter"} }]}                      
 * });
 * &lt;/script&gt; 
 * @alias ejChart#primaryXAxis->stripLine->font->fontWeight
 */
                        fontWeight: 'Regular',
                        /**		
* Specifies the color of stripLine in primaryXAxis. Stripline text render with this color
* @default "black"
* @type {string}			
* @example 
* &lt;div id="container"&gt;&lt;/div&gt; 
* &lt;script&gt;
* $("#container").ejChart({
* primaryXAxis: { stripLine:[{ font : { color: "green"} }]}                      
* });
* &lt;/script&gt; 
* @alias ejChart#primaryXAxis->stripLine->font->color
*/
                        color: 'black',
                        /**		
* Specifies the opacity of stripLine in primaryXAxis. Stripline text render with this opacity
* @default 1
* @type {number}			
* @example 
* &lt;div id="container"&gt;&lt;/div&gt; 
* &lt;script&gt;
* $("#container").ejChart({
* primaryXAxis: { stripLine:[{ font : { opacity: 0.5} }]}                      
* });
* &lt;/script&gt; 
* @alias ejChart#primaryXAxis->stripLine->font->opacity
*/
                        opacity: 1
                    },
                    /**		
* Specifies the start value of the stripLine.
* @default null
* @type {number}			
* @example 
* &lt;div id="container"&gt;&lt;/div&gt; 
* &lt;script&gt;
* $("#container").ejChart({
* primaryXAxis: { stripLine:[{ start: 2 }]}                      
* });
* &lt;/script&gt; 
* @alias ejChart#primaryXAxis->stripLine->start
*/
                    start: null,
                    /**		
* Specifies the end value of the stripLine.
* @default null
* @type {number}			
* @example 
* &lt;div id="container"&gt;&lt;/div&gt; 
* &lt;script&gt;
* $("#container").ejChart({
* primaryXAxis: { stripLine:[{ end: 5 }]}                      
* });
* &lt;/script&gt; 
* @alias ejChart#primaryXAxis->stripLine->end
*/
                    end: null,
                    /**		
* Background color of the stripLine.
* @default "gray"
* @type {string}			
* @example 
* &lt;div id="container"&gt;&lt;/div&gt; 
* &lt;script&gt;
* $("#container").ejChart({
* primaryXAxis: { stripLine:[{ color: "green" }]}                      
* });
* &lt;/script&gt; 
* @alias ejChart#primaryXAxis->stripLine->color
*/
                    color: 'gray',
                    /**		
* Border color of the stripLine.
* @default "gray"
* @type {string}			
* @example 
* &lt;div id="container"&gt;&lt;/div&gt; 
* &lt;script&gt;
* $("#container").ejChart({
* primaryXAxis: { stripLine:[{ borderColor: "green" }]}                      
* });
* &lt;/script&gt; 
* @alias ejChart#primaryXAxis->stripLine->borderColor
*/
                    borderColor: 'black',
                    /**		
* Specifies the z order position of the stripLine.
* @default "over"
* @type {enum}			
* @example 
* &lt;div id="container"&gt;&lt;/div&gt; 
* &lt;script&gt;
* $("#container").ejChart({
* primaryXAxis: { stripLine:[{ zIndex: "behind" }]}                      
* });
* &lt;/script&gt; 
* @alias ejChart#primaryXAxis->stripLine->zIndex
*/
                    zIndex: 'over'
                }
         ],
         /**		
    * Contains property to customize the title in chart.
    * @type {object}
      * @alias ejChart#primaryXAxis->title
    */
         title:
 {

     /**		
   * The value of the title of chart can be specified using this property.
   * @default ""
   * @type {string}				
   * @example 
    * &lt;div id="container"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#container").ejChart({
      * primaryXAxis: { title: { text: "Year" } }                      
      * });
      * &lt;/script&gt; 
     * @alias ejChart#primaryXAxis->title->text
   */
     text: "",
     /**		
* Specifies the font. These font properties are customization are applied for title text. 
* @type {object}
 * @alias ejChart#primaryXAxis->title->font
*/
     font:
{
    /**		
  * Specifies the title fontfamily of primaryXAxis. This fontFamily is applied to the title text of primaryXAxis.
  * @default "Segoe UI"
  * @type {string}			 
  * @example 
   * &lt;div id="container"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#container").ejChart({
      * primaryXAxis: { title: { font : { fontFamily : "Algerain"} } }                      
      * });
      * &lt;/script&gt;  
  * @alias ejChart#primaryXAxis->title->font->fontFamily
  */
    fontFamily: 'Segoe UI',
    /**		
    * Specifies the title fontStyle of primaryXAxis. This fontStyle is applied to the title text of primaryXAxis.
    * @default ej.datavisualization.Chart.FontStyle.Normal
    * @type {enum}				 
    * @example 
    * &lt;div id="container"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#container").ejChart({
      * primaryXAxis: { title: { font : { fontStyle : "Italic"} } }                      
      * });
      * &lt;/script&gt;  
    * @alias ejChart#primaryXAxis->title->font->fontStyle
    */
    fontStyle: 'Normal',
    /**		
 * Specifies the title size of primaryXAxis. This font size is applied to the title text of primaryXAxis.
 * @default "16px"
 * @type {string}				
 * @example 
 * &lt;div id="container"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#container").ejChart({
      * primaryXAxis: { title: { font : { size : "14px"} } }                      
      * });
      * &lt;/script&gt;   
  * @alias ejChart#primaryXAxis->title->font->size
 */
    size: '16px',
    /**		
 * Specifies the title opacity of primaryXAxis. This opacity value is applied to the title text of primaryXAxis.
 * @default 1
 * @type {number}				
 * @example 
* &lt;div id="container"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#container").ejChart({
      * primaryXAxis: { title: { font : { opacity : 0.8} } }                      
      * });
      * &lt;/script&gt;  
  * @alias ejChart#primaryXAxis->title->font->opacity
 */
    opacity: 1,
    /**		
 * Specifies the title fontWeight of primaryXAxis. This fontWeight is applied to the title text of primaryXAxis. See {@link FontWeight}
 * @default ej.datavisualization.Chart.FontWeight.Regular
 * @type {enum}				 
 * @example 
* &lt;div id="container"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#container").ejChart({
      * primaryXAxis: { title: { font : { fontWeight : "lighter"} } }                      
      * });
      * &lt;/script&gt;  
   * @alias ejChart#primaryXAxis->title->font->fontWeight
 */
    fontWeight: 'regular'
}
 },

         /**		
      * If the range is not given explicitly, range will be calculated automatically. You can customize the automatic range calculation using rangePadding. See {@link RangePadding}
      * @default ej.datavisualization.Chart.rangePaddding.None
      * @type {enum}				
      * @example 
      * &lt;div id="container"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#container").ejChart({
      * primaryXAxis: { rangePadding : "normal" }                      
      * });
      * &lt;/script&gt;  
       * @alias ejChart#primaryXAxis->rangePadding
      */
         rangePadding: 'None',
         additionalPadding: [1, 1],
         orientation: 'Horizontal',
         /**		
      * Specifies the maximum number of labels to be displayed in 100 pixels.
      * @default 3
      * @type {number}				
      * @example 
      * &lt;div id="container"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#container").ejChart({
      * primaryXAxis: { maximumLabels : 5 }                      
      * });
      * &lt;/script&gt;  
      * @alias ejChart#primaryXAxis->maximumLabels
      */
         maximumLabels: 3,
         /**		
     * Specifies a value that indicates whether to position the axis opposite to its default position.
     * @default false
     * @type {boolean}			
     * @example 
     * &lt;div id="container"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#container").ejChart({
      * primaryXAxis: { opposedPosition : true }                      
      * });
      * &lt;/script&gt;  
      * @alias ejChart#primaryXAxis->opposedPosition
     */
         opposedPosition: false,
         /**		
       * Specifies the options to configure axis line.
       * @type {object}
       * @alias ejChart#primaryXAxis->axisLine
       */
         axisLine:
  {
      /**		
     * Specifies the visibility of axis line.
     * @default true
     * @type {boolean}			
     * @example 
     * &lt;div id="container"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#container").ejChart({
      * primaryXAxis: { axisLine : { visible : false } }                      
      * });
      * &lt;/script&gt;  
     * @alias ejChart#primaryXAxis->axisLine->visible
     */
      visible: true,
      /**		
     * Specifies the width of axisLine in primaryXAxis.
     * @default 1
     * @type {number}			
     * @example 
     * &lt;div id="container"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#container").ejChart({
      * primaryXAxis: { axisLine : { width : 2 } }                      
      * });
      * &lt;/script&gt;  
     * @alias ejChart#primaryXAxis->axisLine->width
     */
      width: 1,
      /**		
 * Controls the pattern of dashes and gaps used to stroke the axis line.
 * @default null
 * @type {string}			
 * @example 
 * &lt;div id="container"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#container").ejChart({
      * primaryXAxis: { axisLine : { dashArray : "2,3" } }                      
      * });
      * &lt;/script&gt;  
  * @alias ejChart#primaryXAxis->axisLine->dashArray
 */
      dashArray: "",
      /**		
* Specifies the horizontal/vertical padding of axis line. Normally, it is used along with plotOffset to pad the plot area.
* @default null
* @type {number}			
* @example 
* &lt;div id="container"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#container").ejChart({
      * primaryXAxis: { axisLine : { offset : 5 } }                      
      * });
      * &lt;/script&gt;  
 * @alias ejChart#primaryXAxis->axisLine->offset
*/
      offset: 0
  },
         /**
         * Specifies the action to take when the axis labels are overlapping with each other. See {@link LabelIntersectAction}
         * @default ej.datavisualization.Chart.LabelIntersectAction.None
         * @type {enum}		
         * @example 		 
         * &lt;div id="container"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#container").ejChart({
      * primaryXAxis: { labelIntersectAction : "multipleRows" }                      
      * });
      * &lt;/script&gt;  
         * @alias ejChart#primaryXAxis->labelIntersectAction
         */
         labelIntersectAction: "none",

         /**		
      * Determines how to position labels at the edge of the axis. See {@link EdgeLabelPlacement} for the available options.
     * @default ej.datavisualization.Chart.EdgeLabelPlacement.None
      * @type {enum}			 
      * @example 
       * &lt;div id="container"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#container").ejChart({
      * primaryXAxis: { edgeLabelPlacement : "shift" }                      
      * });
      * &lt;/script&gt;  
       * @alias ejChart#primaryXAxis->edgeLabelPlacement
     */
         edgeLabelPlacement: "none",

         /**		
    * Specifies the font of primaryXAxis.
    * @type {object}
    * @alias ejChart#primaryXAxis->font

    */
         font:
  {
      /**		
      * Specifies the fontfamily of primaryXAxis. The axis labels render with this specified fontFamily.
      * @default "Segoe UI"
      * @type {string}				
      * @example 
       * &lt;div id="container"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#container").ejChart({
      * primaryXAxis: { font : { fontFamily : "Algerian"} }                      
      * });
      * &lt;/script&gt;   
       * @alias ejChart#primaryXAxis->font->fontFamily
      */
      fontFamily: 'Segoe UI',
      /**		
     * Specifies the fontStyle of primaryXAxis. The axis labels render with this specified fontStyle. See {@link FontStyle}
     * @default ej.datavisualization.Chart.FontStyle.Normal
     * @type {enum}				
     * @example 
     * &lt;div id="container"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#container").ejChart({
      * primaryXAxis: { font : { fontStyle : "Italic"} }                      
      * });
      * &lt;/script&gt;  
     * @alias ejChart#primaryXAxis->font->fontStyle
     */
      fontStyle: 'Normal',
      /**		
      * Specifies the title size of primaryXAxis. The axis labels render with this specified size.
      * @default "13px"
      * @type {string}			
      * @example 
       * &lt;div id="container"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#container").ejChart({
      * primaryXAxis: { font : { size : "12px"} }                      
      * });
      * &lt;/script&gt;  
      * @alias ejChart#primaryXAxis->font->size
      */
      size: '13px',
      /**		
      * Specifies the fontWeight of primaryXAxis. The axis labels render with this specified fontWeight.See {@link FontWeight}
      * @default ej.datavisualization.Chart.FontWeight.Regular
      * @type {enum}				
      * @example 
       * &lt;div id="container"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#container").ejChart({
      * primaryXAxis: { font : { fontWeight : "lighter"} }                      
      * });
      * &lt;/script&gt;  
      * @alias ejChart#primaryXAxis->font->fontWeight
      */
      fontWeight: 'regular',
      /**		
     * Specifies the title opacity of primaryXAxis. The axis labels render with this specified opacity.
     * @default 1
     * @type {number}			
     * @example 
     * &lt;div id="container"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#container").ejChart({
      * primaryXAxis: { font : { opacity : 0.5} }                      
      * });
      * &lt;/script&gt;  
     * @alias ejChart#primaryXAxis->font->opacity
     */
      opacity: 1
  },
         /**		
      * Specifies the visibility of the axis.
      * @default true
      * @type {boolean}				
      * @example 
       * &lt;div id="container"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#container").ejChart({
      * primaryXAxis: { visible: false }                      
      * });
      * &lt;/script&gt;  
      * @alias ejChart#primaryXAxis->visible
      */
         visible: true,
         /**		
    * Options to configure the visibility of the crosshair label. 
    * @type {object}
     * @alias ejChart#primaryXAxis->crosshairLabel
    */
         crosshairLabel:
   {
       /**		
      * Specifies the visibility of crosshair label. 
      * @default false
      * @type {boolean}			
      * @example 
       * &lt;div id="container"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#container").ejChart({
      * primaryXAxis: { crosshairLabel : { visible : true} }                      
      * });
      * &lt;/script&gt;  
       * @alias ejChart#primaryXAxis->crosshairLabel->visible
      */
       visible: false
   },
         /**		
       * This property determines the factor by which the axis is scaled. Value must be specified between 0 and 1
       * @default 1
       * @type {number}			
       * @example 
        * &lt;div id="container"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#container").ejChart({
      * primaryXAxis: { zoomFactor : 0.5 }                      
      * });
      * &lt;/script&gt;  
       * @alias ejChart#primaryXAxis->zoomFactor
       */
         zoomFactor: 1,
         /**		
      * This property determines the starting position of the zoomed axis. Value must be specified between 0 and 1.
      * @default 0
      * @type {number}				
      * @example 
       * &lt;div id="container"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#container").ejChart({
      * primaryXAxis: { zoomPosition :0.5 }                      
      * });
      * &lt;/script&gt;  
     * @alias ejChart#primaryXAxis->zoomPosition
      */
         zoomPosition: 0
     },

            /**		
           * This is a vertical axis which contains options to configure axis. This is the primary y axis for all the series in series array. If you need to override y axis for particular series, you can create an axis object by providing unique name using name property and add it to axes array. Then, assign the name to the series’s yAxisName property to link both axis and series.
           * @type {object}
           */
            primaryYAxis: /** @lends ejChart# */
      {
          /**		
* Specifies the position of the axis labels.
* @default "outside"
* @type {enum}				
* @example 
* &lt;div id="container"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#container").ejChart({
      * primaryYAxis: { labelPosition : "inside" }                       
      * });
      * &lt;/script&gt; 
 * @alias ejChart#primaryYAxis->labelPosition
*/
          labelPosition: 'outside',
		   /**		
* Specifies the position of the axis tick lines.
* @default "outside"
* @type {enum}				
* @example 
* &lt;div id="container"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#container").ejChart({
      * primaryYAxis: { tickLinesPosition : "inside" }                       
      * });
      * &lt;/script&gt; 
 * @alias ejChart#primaryYAxis->tickLinesPosition
*/
          tickLinesPosition: 'outside',
          /**		
         * This is a Vertical axis alternate grid band which contains options to highlight alternate grid bands.
         * @type {object}
		 * @alias ejChart#primaryYAxis->alternateGridBand
         */
          alternateGridBand:
                     {
                         /**		
                         * Specifies the Odd Alternative Grid Band.
                         * @type {object}
						 * @alias ejChart#primaryYAxis->alternateGridBand->odd
                         */
                         odd: {
                             /**		
                             * Specifies the fill color of Odd grid bands
                             * @default transparent
                             * @type {string}	
                             * @example 
                    * &lt;div id="container"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#container").ejChart({
      * primaryYAxis: { alternateGridBand: { odd : { fill :"red" }  } }                      
      * });
      * &lt;/script&gt; 
                           * @alias ejChart#primaryYAxis->alternateGridBand->odd->fill							 
                             */
                             fill: "transparent",
                             /**		
         * Specifies the opacity of the Odd grid band.
         * @default 1
         * @type {number}				
          * @example 
                    * &lt;div id="container"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#container").ejChart({
    * primaryYAxis: { alternateGridBand: { odd : { opacity :0.5 }  } }                       
      * });
      * &lt;/script&gt; 
                           * @alias ejChart#primaryYAxis->alternateGridBand->odd->opacity							 
                             */
                             opacity: 1
                         },
                         /**		
                         * Specifies the Even Alternative Grid Band.
                         * @type {object}
						  * @alias ejChart#primaryYAxis->alternateGridBand->even
                         */
                         even: {
                             /**		
         * Specifies the fill color of Even grid bands
         * @default transparent
         * @type {string}
         * @example 
* &lt;div id="container"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#container").ejChart({
      * primaryYAxis: { alternateGridBand: { even : {fill : "red" } } }                      
      * });
      * &lt;/script&gt; 
 * @alias ejChart#primaryYAxis->alternateGridBand->even->fill
*/		 

                             fill: "transparent",
                             /**		
         * Specifies the opacity of the Even grid band.
         * @default 1
         * @type {number}				
         * @example 
* &lt;div id="container"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#container").ejChart({
      * primaryYAxis: { alternateGridBand: { even : {opacity : 0.5 } } }                      
      * });
      * &lt;/script&gt; 
 * @alias ejChart#primaryYAxis->alternateGridBand->even->opacity
*/		
                             opacity: 1
                         }
                     },
					  /**		
    * Options for configuring major grid lines color, width, dash arrays etc., 
    * @type {object}
     * @alias ejChart#primaryYAxis->majorGridLines
    */
          majorGridLines:
     {
         /**		
* Specifies the width of the major grid lines.
* @default 1
* @type {number}			
* @example 
* &lt;div id="container"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#container").ejChart({
      * primaryYAxis: { majorGridLines : {width : 2} }                      
      * });
      * &lt;/script&gt;   
* @alias ejChart#primaryYAxis->majorGridLines->width
*/
         width: 1,
         /**		
* Controls the pattern of dashes and gaps used to stroke the major grid lines.
* @default null
* @type {string}			
* @example 
* &lt;div id="container"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#container").ejChart({
      * primaryYAxis: { majorGridLines : {dashArray : "2,3"} }                      
      * });
      * &lt;/script&gt;   
* @alias ejChart#primaryYAxis->majorGridLines->dashArray
*/
         dashArray: "",
         /**		
* Specifies the visibility of the major grid lines.
* @default true
* @type {boolean}			
* @example 
* &lt;div id="container"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#container").ejChart({
      * primaryYAxis: { majorGridLines : {visible : false} }                      
      * });
      * &lt;/script&gt;   
* @alias ejChart#primaryYAxis->majorGridLines->visible
*/
         visible: true,
         /**		
* Specifies the opacity of the major grid lines.
* @default 1
* @type {number}			
* @example 
* &lt;div id="container"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#container").ejChart({
      * primaryYAxis: { majorGridLines : {opacity : 0.5} }                      
      * });
      * &lt;/script&gt;  
* @alias ejChart#primaryYAxis->majorGridLines->opacity
*/
         opacity: 1

     },
          /**		
* Options for configuring major tick lines color, width, dash arrays etc.,
* @type {object}
* @alias ejChart#primaryYAxis->majorTickLines
*/
          majorTickLines:
     {
         /**		
* Specifies the width of the major tick lines.
* @default 1
* @type {number}			
* @example 
* &lt;div id="container"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#container").ejChart({
      * primaryYAxis: { majorTickLines : {width : 2} }                      
      * });
      * &lt;/script&gt;  
* @alias ejChart#primaryYAxis->majorTickLines->width
*/
         width: 1,
         /**		
* Length of the major tick lines.
* @default 5
* @type {number}				
* @example 
* &lt;div id="container"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#container").ejChart({
      * primaryYAxis: { majorTickLines : {size : 2} }                      
      * });
      * &lt;/script&gt;  
* @alias ejChart#primaryYAxis->majorTickLines->size
*/
         size: 5,
         /**		
* Specifies the visibility of the major tick lines.
* @default true
* @type {boolean}				
* @example 
* &lt;div id="container"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#container").ejChart({
      * primaryYAxis: { majorTickLines : {visible : false} }                      
      * });
      * &lt;/script&gt;  
* @alias ejChart#primaryYAxis->majorTickLines->visible
*/
         visible: true
     },
          /**		
     * Options for configuring minor grid lines color, width, dash arrays etc.,
     * @type {object}
  * @alias ejChart#primaryYAxis->minorGridLines
     */
          minorGridLines:
     {
         /**		
* Specifies the width of the minor grid lines.
* @default 1
* @type {number}			
* @example 
* &lt;div id="container"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#container").ejChart({
      * primaryYAxis: { minorGridLines : {width : 2} }                      
      * });
      * &lt;/script&gt;   
* @alias ejChart#primaryYAxis->minorGridLines->width
*/
         width: 1,
         /**		
* Controls the pattern of dashes and gaps used to stroke the minor grid lines.
* @default null
* @type {string}		
* @example 
* &lt;div id="container"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#container").ejChart({
      * primaryYAxis: { minorGridLines : {dashArray : "2,3"} }                      
      * });
      * &lt;/script&gt;   
* @alias ejChart#primaryYAxis->minorGridLines->dashArray
*/
         dashArray: "",
         /**		
* Specifies the visibility of the minorGridLines. The user can modify the visibility of the minorGridLines in primaryYAxis.
* @default false
* @type {boolean}				
* @example 
* &lt;div id="container"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#container").ejChart({
      * primaryYAxis: { minorGridLines : {visible : true} }                      
      * });
      * &lt;/script&gt;  
* @alias ejChart#primaryYAxis->minorGridLines->visible
*/
         visible: true

     },
          /**		
* Options for configuring minor tick lines color, width, dash arrays etc.,
* @type {object}
* @alias ejChart#primaryYAxis->minorTickLines
*/
          minorTickLines:
     {
         /**		
* Specifies the width of the minor tick lines.
* @default 1
* @type {number}			
* @example 
* &lt;div id="container"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#container").ejChart({
      * primaryYAxis: { minorTickLines : {width : 2} }                      
      * });
      * &lt;/script&gt;  
* @alias ejChart#primaryYAxis->minorTickLines->width
*/
         width: 1,
         /**		
* Length of the minor tick lines.
* @default 5
* @type {number}				
* @example 
* &lt;div id="container"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#container").ejChart({
      * primaryYAxis: { minorTickLines : {size : 2} }                      
      * });
      * &lt;/script&gt;  
* @alias ejChart#primaryYAxis->minorTickLines->size
*/
         size: 5,
         /**		
* Specifies the visibility of the minor tick lines.
* @default false
* @type {boolean}				
* @example 
* &lt;div id="container"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#container").ejChart({
      * primaryYAxis: { minorTickLines : {visible : true} }                      
      * });
      * &lt;/script&gt;  
* @alias ejChart#primaryYAxis->minorTickLines->visible
*/
         visible: true
     },
          /**		
* Specifies the number of minor ticks to render per major tick/interval.
* @default null
* @type {number}			
* @example 
 * &lt;div id="container"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#container").ejChart({
      * primaryYAxis: { minorTicksPerInterval: 3 }                      
      * });
      * &lt;/script&gt; 
* @alias ejChart#primaryYAxis->minorTicksPerInterval
*/
          minorTicksPerInterval: null,
          /**		
* Index of the row object in rowDefinitions array to which this axis is associated with.
* @default null
* @type {number}				
* @example 
* &lt;div id="container"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#container").ejChart({
      * primaryYAxis: { rowIndex: 1 }                      
      * });
      * &lt;/script&gt; 
* @alias ejChart#primaryYAxis->rowIndex
*/
          rowIndex: null,
          /**		
 * An axis can be spanned along multiple rows or plotting areas. This property specifies the number of rows/plotting areas this axis should be spanned.
 * @default null
 * @type {number}				
 * @example 
* &lt;div id="container"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#container").ejChart({
      * primaryYAxis: { rowSpan: 2 }                      
      * });
      * &lt;/script&gt; 
* @alias ejChart#primaryYAxis->rowSpan
 */
          rowSpan: null,
          /**		
* You can plot any type of data like date time, double, string etc., This property determines the type of data that this axis will handle.
* @default null
* @type {enum}			
* @example 
* &lt;div id="container"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#container").ejChart({
      * primaryYAxis: { valueType: "double" }                      
      * });
      * &lt;/script&gt;  
* @alias ejChart#primaryYAxis->valueType
*/
          valueType: null,
          /**		
  * Unique name of the axis.
  * @default null
  * @type {string}			
  * @example 
  * &lt;div id="container"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#container").ejChart({
      * primaryYAxis: { name: "yAxis" }                      
      * });
      * &lt;/script&gt; 
* @alias ejChart#primaryYAxis->name
  */
          name: null,
          /**		
 * Provides the format for axis labels.
 * @default null
 * @type {string}				
 * @example 
* &lt;div id="container"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#container").ejChart({
      * primaryYAxis: { labelFormat: "{value}F" }                      
      * });
      * &lt;/script&gt;  
 * @alias ejChart#primaryYAxis->labelFormat
 */
          labelFormat: null,
          /**		
* Specifies the desired number of intervals for the range. With this setting, you can request axis to calculate nice numbers such that they result in the number of intervals approximately equal to your desired interval.
* @default null
* @type {number}				
* @example 
* &lt;div id="container"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#container").ejChart({
      * primaryYAxis: { desiredIntervals: 5 }                      
      * });
      * &lt;/script&gt; 
 * @alias ejChart#primaryYAxis->desiredIntervals
*/
          desiredIntervals: null,
          /**		
 * Specifies whether the type of the interval calculated is in days, months, hours, minutes etc., This is applicable only when valueType is ‘datetime’. See {@link IntervalType}
 * @default null
 * @type {enum}			
 * @example 
* &lt;div id="container"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#container").ejChart({
      * primaryYAxis: { intervalType: "days" }                      
      * });
      * &lt;/script&gt;  
  * @alias ejChart#primaryYAxis->intervalType
 */
          intervalType: null,
          /**		
 * Rounds the axis labels to the specified number of decimals.
 * @default null
 * @type {number}			
 * @example 
 * &lt;div id="container"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#container").ejChart({
      * primaryYAxis: { roundingPlaces: 2 }                      
      * });
      * &lt;/script&gt; 
  * @alias ejChart#primaryYAxis->roundingPlaces
 */
          roundingPlaces: null,
          labels: [],
          /**		
* Options to configure the title font and alignment of the axis title.
* @type {object}
* @alias ejChart#primaryYAxis->title
*/
          title:
    {
        /**		
* Text to be displayed as a title for axis.
* @default ""
* @type {string}				
* @example 
* &lt;div id="container"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#container").ejChart({
      * primaryYAxis: { title :{ text: "yAxis" } }                      
      * });
      * &lt;/script&gt; 
* @alias ejChart#primaryYAxis->title->text
*/
        text: "",
        /**		
* Specifies the font. These font properties are customization are applied for title text.
* @type {object}
* @alias ejChart#primaryYAxis->title->font
*/
        font:
     {
         /**		
* Specifies the title fontfamily of primaryYAxis. This fontFamily is applied to the title text of primaryYAxis.
* @default "Segoe UI"
* @type {string}				
* @example 
* &lt;div id="container"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#container").ejChart({
      * primaryYAxis: { title :{ font :{ fontFamily: "Algerian" } } }                      
      * });
      * &lt;/script&gt; 
* @alias ejChart#primaryYAxis->title->font->fontFamily
*/
         fontFamily: 'Segoe UI',
         /**		
* Specifies the title fontStyle of primaryYAxis. This fontStyle is applied to the title text of primaryYAxis. See {@link FontStyle}
* @default ej.datavisualization.Chart.FontStyle.Normal
* @type {enum}			 
* @example 
* &lt;div id="container"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#container").ejChart({
      * primaryYAxis: { title :{ font :{ fontStyle : "Italic" } } }                      
      * });
      * &lt;/script&gt;  
* @alias ejChart#primaryYAxis->title->font->fontStyle
*/
         fontStyle: 'Normal',
         /**		
* Specifies the title opacity of primaryYAxis. This font size is applied to the title text of primaryYAxis.
* @default 1
* @type {number}			 
* @example 
* &lt;div id="container"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#container").ejChart({
      * primaryYAxis: { title :{ font :{ opacity: 0.5 } } }                      
      * });
      * &lt;/script&gt;  
* @alias ejChart#primaryYAxis->title->font->opacity
*/
         opacity: 1,
         /**		
* Specifies the title size of primaryYAxis. This opacity value is applied to the title text of primaryYAxis.
* @default "16px"
* @type {string}			 
* @example 
* &lt;div id="container"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#container").ejChart({
      * primaryYAxis: { title :{ font :{ size: "12px" } } }                      
      * });
      * &lt;/script&gt;  
* @alias ejChart#primaryYAxis->title->font->size
*/
         size: '16px',
         /**		
* Specifies the title fontWeight of primaryYAxis. This fontWeight is applied to the title text of primaryYAxis. See {@link FontWeight}
* @default ej.datavisualization.Chart.FontWeight.Regular
* @type {enum}				
* @example 
* &lt;div id="container"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#container").ejChart({
      * primaryYAxis: { title :{ font :{ fontWeight: "normal" } } }                      
      * });
      * &lt;/script&gt;  
* @alias ejChart#primaryYAxis->title->font->fontWeight
*/
         fontWeight: 'regular'
     }
    },
          /**		
* If the range is not given explicitly, range will be calculated automatically. You can customize the automatic range calculation using rangePadding. See {@link RangePadding}
* @default ej.datavisualization.Chart.RangePadding.None
* @type {enum}			
* @example 
* &lt;div id="container"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#container").ejChart({
      * primaryYAxis: { rangePadding : "none" }                      
      * });
      * &lt;/script&gt;   
* @alias ejChart#primaryYAxis->rangePadding
*/
          rangePadding: 'Normal',

          /**		
            * Options to configure stripLine’s color, text, border etc.,
            * @name ejChart#primaryYAxis->stripLine 
            * @default [ ]
            * @type {array}
            */
          stripLine: [
               {
                   /**		
     * Specifies the visibility of the stripLine.
     * @default false
     * @type {boolean}			
     * @example 
     * &lt;div id="container"&gt;&lt;/div&gt; 
     * &lt;script&gt;
     * $("#container").ejChart({
     * primaryYAxis: { stripLine:[{ visible : true }]}                      
     * });
     * &lt;/script&gt; 
     * @alias ejChart#primaryYAxis->stripLine->visible
     */
                   visible: false,
                   /**		
   * Indicates whether to render the stripLine from the minimum/start value of the axis.
   * @default false
   * @type {boolean}			
   * @example 
   * &lt;div id="container"&gt;&lt;/div&gt; 
   * &lt;script&gt;
   * $("#container").ejChart({
   * primaryYAxis: { stripLine:[{ startFromAxis : true }]}                      
   * });
   * &lt;/script&gt; 
   * @alias ejChart#primaryYAxis->stripLine->startFromAxis
   */
                   startFromAxis: false,
                   /**		
  * Text to be displayed inside the stripLine.
  * @default "stripLine"
  * @type {string}			
  * @example 
  * &lt;div id="container"&gt;&lt;/div&gt; 
  * &lt;script&gt;
  * $("#container").ejChart({
  * primaryYAxis: { stripLine:[{ text : "Empty Point" }]}                      
  * });
  * &lt;/script&gt; 
  * @alias ejChart#primaryYAxis->stripLine->text
  */
                   text: "stripLine",
                   /**		
  * Alignment of the text displayed in stripLine. See {@link TextAlignment}
  * @default "middlecenter"
  * @type {enum}			
  * @example 
  * &lt;div id="container"&gt;&lt;/div&gt; 
  * &lt;script&gt;
  * $("#container").ejChart({
  * primaryYAxis: { stripLine:[{ textAlignment : "middletop" }]}                      
  * });
  * &lt;/script&gt; 
  * @alias ejChart#primaryYAxis->stripLine->textAlignment
  */
                   textAlignment: "middlecenter",
                   /**		
    * Specifies the font of stripLine in primaryYAxis. These are font properties for stripLine text
    * @type {object}
    * @alias ejChart#primaryYAxis->stripLine->font
    */
                   font: {
                       /**		
 * Specifies the fontFamily of stripLine in primaryYAxis. Stripline text render with this fontFamily
 * @default "middlecenter"
 * @type {enum}			
 * @example 
 * &lt;div id="container"&gt;&lt;/div&gt; 
 * &lt;script&gt;
 * $("#container").ejChart({
 * primaryYAxis: { stripLine:[{ font : { fontFamily : "Algerian"} }]}                      
 * });
 * &lt;/script&gt; 
 * @alias ejChart#primaryYAxis->stripLine->font->fontFamily
 */
                       fontFamily: 'Segoe UI',
                       /**		
* Specifies the fontWeight of stripLine in primaryYAxis. Stripline text render with this fontWeight
* @default "regular"
* @type {string}			
* @example 
* &lt;div id="container"&gt;&lt;/div&gt; 
* &lt;script&gt;
* $("#container").ejChart({
* primaryYAxis: { stripLine:[{ font : { fontWeight: "lighter"} }]}                      
* });
* &lt;/script&gt; 
* @alias ejChart#primaryYAxis->stripLine->font->fontWeight
*/
                       fontWeight: 'Regular',
                       /**		
* Specifies the fontStyle of stripLine in primaryYAxis. Stripline text render with this fontStyle. See {@link FontStyle}
* @default "Normal"
* @type {enum}			
* @example 
* &lt;div id="container"&gt;&lt;/div&gt; 
* &lt;script&gt;
* $("#container").ejChart({
* primaryYAxis: { stripLine:[{ font : { fontStyle: "Bold"} }]}                      
* });
* &lt;/script&gt; 
* @alias ejChart#primaryYAxis->stripLine->font->fontStyle
*/
                       fontStyle: 'Normal',
                       /**		
* Specifies the size of stripLine in primaryYAxis. Stripline text render with this size
* @default "12px"
* @type {string}			
* @example 
* &lt;div id="container"&gt;&lt;/div&gt; 
* &lt;script&gt;
* $("#container").ejChart({
* primaryYAxis: { stripLine:[{ font : { size: "15px"} }]}                      
* });
* &lt;/script&gt; 
* @alias ejChart#primaryYAxis->stripLine->font->size
*/
                       size: '12px',
                       /**		
* Specifies the color of stripLine in primaryYAxis. Stripline text render with this color
* @default "black"
* @type {string}			
* @example 
* &lt;div id="container"&gt;&lt;/div&gt; 
* &lt;script&gt;
* $("#container").ejChart({
* primaryYAxis: { stripLine:[{ font : { color: "green"} }]}                      
* });
* &lt;/script&gt; 
* @alias ejChart#primaryYAxis->stripLine->font->color
*/
                       color: 'black',
                       /**		
* Specifies the opacity of stripLine in primaryYAxis. Stripline text render with this opacity
* @default 1
* @type {number}			
* @example 
* &lt;div id="container"&gt;&lt;/div&gt; 
* &lt;script&gt;
* $("#container").ejChart({
* primaryYAxis: { stripLine:[{ font : { opacity: 0.5} }]}                      
* });
* &lt;/script&gt; 
* @alias ejChart#primaryYAxis->stripLine->font->opacity
*/
                       opacity: 1
                   },
                   /**		
* Specifies the start value of the stripLine.
* @default null
* @type {number}			
* @example 
* &lt;div id="container"&gt;&lt;/div&gt; 
* &lt;script&gt;
* $("#container").ejChart({
* primaryYAxis: { stripLine:[{ start: 2 }]}                      
* });
* &lt;/script&gt; 
* @alias ejChart#primaryYAxis->stripLine->start
*/
                   start: null,
                   /**		
* Specifies the end value of the stripLine.
* @default null
* @type {number}			
* @example 
* &lt;div id="container"&gt;&lt;/div&gt; 
* &lt;script&gt;
* $("#container").ejChart({
* primaryYAxis: { stripLine:[{ end: 5 }]}                      
* });
* &lt;/script&gt; 
* @alias ejChart#primaryYAxis->stripLine->end
*/
                   end: null,
                   /**		
* Background color of the stripLine.
* @default "gray"
* @type {string}			
* @example 
* &lt;div id="container"&gt;&lt;/div&gt; 
* &lt;script&gt;
* $("#container").ejChart({
* primaryYAxis: { stripLine:[{ color: "green" }]}                      
* });
* &lt;/script&gt; 
* @alias ejChart#primaryYAxis->stripLine->color
*/
                   color: 'gray',
                   /**		
* Border color of the stripLine.
* @default "gray"
* @type {string}			
* @example 
* &lt;div id="container"&gt;&lt;/div&gt; 
* &lt;script&gt;
* $("#container").ejChart({
* primaryYAxis: { stripLine:[{ borderColor: "green" }]}                      
* });
* &lt;/script&gt; 
* @alias ejChart#primaryYAxis->stripLine->borderColor
*/
                   borderColor: 'black',
                   /**		
* Specifies the z order position of the stripLine.
* @default "over"
* @type {enum}			
* @example 
* &lt;div id="container"&gt;&lt;/div&gt; 
* &lt;script&gt;
* $("#container").ejChart({
* primaryYAxis: { stripLine:[{ zIndex: "behind" }]}                      
* });
* &lt;/script&gt; 
* @alias ejChart#primaryYAxis->stripLine->zIndex
*/
                   zIndex: 'over'
               }
          ],
          /**		
 * Specifies the logarithmic base value. This is applicable only when valueType is logarithmic.
 * @default 10
 * @type {number}				
 * @example 
* &lt;div id="container"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#container").ejChart({
      * primaryYAxis: { logBase: 5 }                      
      * });
      * &lt;/script&gt;   
 * @alias ejChart#primaryYAxis->logBase
 */
          logBase: 10,
          /**		
 * Horizontal/vertical padding for the plotting area based on the orientation of the axis.
 * @default 10
 * @type {number}			 
 * @example 
 * &lt;div id="container"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#container").ejChart({
      * primaryYAxis: { plotOffset: 5 }                      
      * });
      * &lt;/script&gt;   
  * @alias ejChart#primaryYAxis->plotOffset
 */
          plotOffset: 0,
          orientation: 'Vertical',
          /**		
* Specifies the maximum number of labels to be displayed in 100 pixels.
* @default 3
* @type {number}			 
* @example 
* &lt;div id="container"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#container").ejChart({
      * primaryYAxis: { maximumLabels: 5 }                      
      * });
      * &lt;/script&gt;   
 * @alias ejChart#primaryYAxis->maximumLabels
*/
          maximumLabels: 3,
          /**
          * Specifies the action to take when the axis labels are overlapping with each other. See {@link LabelIntersectAction}
     * @default ej.datavisualization.Chart.LabelIntersectAction.None
      * @type {enum}			 
      * @example 
      * &lt;div id="container"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#container").ejChart({
      * primaryYAxis: { labelIntersectAction: "multipleRows" }                      
      * });
      * &lt;/script&gt;    
       * @alias ejChart#primaryYAxis->labelIntersectAction
      */
          labelIntersectAction: "none",
          /**		
    * Determines how to position labels at the edge of the axis. See {@link EdgeLabelPlacement} for the available options.
   * @default ej.datavisualization.Chart.EdgeLabelPlacement.None
    * @type {enum}			 
    * @example 
    * &lt;div id="container"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#container").ejChart({
      * primaryYAxis: { edgeLabelPlacement: "shift" }                      
      * });
      * &lt;/script&gt; 
     * @alias ejChart#primaryYAxis->edgeLabelPlacement
    */
          edgeLabelPlacement: "none",

          /**		
* Options to configure the visibility of the crosshair label.
* @type {object}
* @alias ejChart#primaryYAxis->crosshairLabel
*/
          crosshairLabel:
   {
       /**		
* Specifies the visibility of crosshairLabel. This is the property to change the visibility of the axis labels. 
* @default false
* @type {boolean}			 
* @example 
* &lt;div id="container"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#container").ejChart({
      * primaryYAxis: { crosshairLabel: { visible : true } }                      
      * });
      * &lt;/script&gt; 
 * @alias ejChart#primaryYAxis->crosshairLabel->visible
*/
       visible: false
   },
          /**		
 * Specifies the visibility of the axis. 
 * @default true
 * @type {boolean}			
 * @example 
 * &lt;div id="container"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#container").ejChart({
      * primaryYAxis: { visible : false }                      
      * });
      * &lt;/script&gt;  
  * @alias ejChart#primaryYAxis->visible
 */
          visible: true,
          /**		
* Specifies a value that indicates whether to position the axis opposite to its default position. 
* @default false
* @type {boolean}			
* @example 
* &lt;div id="container"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#container").ejChart({
      * primaryYAxis: { opposedPosition : true }                      
      * });
      * &lt;/script&gt; 
 * @alias ejChart#primaryYAxis->opposedPosition
*/
          opposedPosition: false,
          /**		
* Specifies the font of primaryYAxis.  These font properties are applied for the axis labels. 
* @type {object}
* @alias ejChart#primaryYAxis->font
*/
          font:
   {
       /**		
* Specifies the fontfamily of primaryYAxis. The axis labels render with this specified fontFamily.
* @default "Segoe UI"
* @type {string}				
* @example 
* &lt;div id="container"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#container").ejChart({
      * primaryYAxis: { font: { fontFamily : "Algerian" } }                      
      * });
      * &lt;/script&gt; 
 * @alias ejChart#primaryYAxis->font->fontFamily
*/
       fontFamily: 'Segoe UI',
       /**		
* Specifies the fontStyle of primaryYAxis. The axis labels render with this specified fontStyle.  See {@link FontStyle}
* @default ej.datavisualization.Chart.FontStyle.Normal
* @type {enum}				
* @example 
* &lt;div id="container"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#container").ejChart({
      * primaryYAxis: { font: { fontStyle : "italic" } }                      
      * });
      * &lt;/script&gt; 
 * @alias ejChart#primaryYAxis->font->fontStyle
*/
       fontStyle: 'Normal',
       /**		
 * Specifies the title size of primaryYAxis. The axis labels render with this specified size.
 * @default "13px"
 * @type {string}			
 * @example 
 * &lt;div id="container"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#container").ejChart({
      * primaryYAxis: { font: { size : "12px" } }                      
      * });
      * &lt;/script&gt; 
  * @alias ejChart#primaryYAxis->font->size
 */
       size: '13px',
       /**		
* Specifies the title opacity of primaryYAxis. The axis labels render with this specified opacity.
* @default 1
* @type {number}		 
* @example 
* &lt;div id="container"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#container").ejChart({
      * primaryYAxis: { font: { opacity : 0.5 } }                      
      * });
      * &lt;/script&gt; 
* @alias ejChart#primaryYAxis->font->opacity
*/
       opacity: 1,
       /**		
* Specifies the fontWeight of primaryYAxis. The axis labels render with this specified fontWeight.  {@link FontWeight}
* @default ej.datavisualization.Chart.FontWeight.Regular
* @type {enum}			
* @example 
* &lt;div id="container"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#container").ejChart({
      * primaryYAxis: { font: { fontWeight : "normal" } }                      
      * });
      * &lt;/script&gt;  
 * @alias ejChart#primaryYAxis->font->fontWeight
*/
       fontWeight: 'regular'
   },
          /**		
  * Specifies the options to configure axis line.
  * @type {object}
  * @alias ejChart#primaryYAxis->axisLine
  */
          axisLine:
   {
       /**		
* Specifies the visibility of axis line.
* @default true
* @type {boolean}				
* @example 
* &lt;div id="container"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#container").ejChart({
      * primaryYAxis: { axisLine: { visible : false } }                      
      * });
      * &lt;/script&gt; 
 * @alias ejChart#primaryYAxis->axisLine->visible
*/
       visible: true,
       /**		
* Specifies the width of axisLine in primaryYAxis. The user can change the width of the axisLine.
* @default 1
* @type {number}				 
* @example 
* &lt;div id="container"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#container").ejChart({
      * primaryYAxis: { axisLine: { width : 2 } }                      
      * });
      * &lt;/script&gt; 
* @alias ejChart#primaryYAxis->axisLine->width
*/
       width: 1,
       /**		
* Controls the pattern of dashes and gaps used to stroke the axis line.
* @default null
* @type {string}			 
* @example 
* &lt;div id="container"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#container").ejChart({
      * primaryYAxis: { crosshairLabel: { axisLine :{ dashArray : "2,3" } } }                      
      * });
      * &lt;/script&gt; 
* @alias ejChart#primaryYAxis->axisLine->dashArray
*/
       dashArray: "",
       /**		
* Specifies the horizontal/vertical padding of axis line. Normally, it is used along with plotOffset to pad the plot area.
* @default null
* @type {number}			
* @example 
* &lt;div id="container"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#container").ejChart({
      * primaryYAxis: { axisLine: { offset : 5 } }                      
      * });
      * &lt;/script&gt; 
* @alias ejChart#primaryYAxis->axisLine->offset
*/
       offset: 0
   },

          /**		
  * This property determines the factor by which the axis is scaled. Value must be specified between 0 and 1. 
  * @default 1
  * @type {number}		
  * @example 
 * &lt;div id="container"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#container").ejChart({
      * primaryYAxis: { zoomFactor : 0.5 }                      
      * });
      * &lt;/script&gt; 
   * @alias ejChart#primaryYAxis->zoomFactor
  */
          zoomFactor: 1,
          /**		
 * This property determines the starting position of the zoomed axis. Value must be specified between 0 and 1.
 * @default 0
 * @type {number}			 
 * @example 
* &lt;div id="container"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#container").ejChart({
      * primaryYAxis: { zoomPosition : 0.5 }                      
      * });
      * &lt;/script&gt;  
 * @alias ejChart#primaryYAxis->zoomPosition
 */
          zoomPosition: 0
      },

           
            secondaryX: 
             {
                 tickLinesPosition: "outside",
                 labelPosition:"outside",
                
                 majorGridLines:
        {
           
            width: 1,
           
            dashArray: "",
           
            visible: true
        },
                 alternateGridBand:
           {
               odd: {
                   fill: "transparent",
                   opacity: 1
               },
               even: {
                   fill: "transparent",
                   opacity: 1
               }
           },
                
                 majorTickLines:
         {
           
             width: 1,
            
             size: 5,
           
             visible: true
         },
                
                 minorGridLines:
         {
            
             width: 1,
            
             dashArray: "",
            
             visible: true

         },
                
                 minorTickLines:
        {
           
            width: 1,
           
            size: 5,
          
            visible: true
        },
               
                 minorTicksPerInterval: null,
               
                 columnIndex: null,
              
                 columnSpan: null,
                
                 labelRotation: null,
                
                 valueType: null,
               
                 name: null,
              
                 labelFormat: null,
               
                 desiredIntervals: null,
                 
                 intervalType: null,
               
                 roundingPlaces: null,
              
                 logBase: 10,
              
                 plotOffset: 0,
                 labels: [],
                 stripLine: [
              {
                  visible: false,
                  startFromAxis: false,
                  text: "stripLine",
                  textAlignment: "middlecenter",
                  font: { fontFamily: 'Segoe UI', fontStyle: 'Normal', fontWeight: 'regular', size: '12px', color: 'black', opacity: 1 },
                  start: null,
                  end: null,
                  color: 'gray',
                  borderColor: 'black',
                  zIndex: 'over'
              }
                 ],
             
                 title:
         {

           
             text: "",
            
             font:
        {
            
            fontFamily: 'Segoe UI',
           
            fontStyle: 'Normal',
        
            size: '16px',
         
            opacity: 1,
         
            fontWeight: 'regular'
        }

        },
               
                 rangePadding: 'None',
                 additionalPadding: [1, 1],
              
                 orientation: 'Horizontal',
             
                 maximumLabels: 3,
               
                 opposedPosition: false,
               
                 axisLine:
          {
              
              visible: true,
             
              width: 1,
           
              dashArray: "",
             
              offset: 0
          },
                
                 labelIntersectAction: "none",
                
                 edgeLabelPlacement: "none",
                
                 font:
          {
             
              fontFamily: 'Segoe UI',
            
              fontStyle: 'Normal',
             
              size: '13px',
              
              fontWeight: 'regular',
             
              opacity: 1
          },
               
                 visible: true,
                
                 crosshairLabel:
           {
              
               visible: false
           },
              
                 zoomFactor: 1,
              
                 zoomPosition: 0
             },
          
            secondaryY: 
                {
                    tickLinesPosition: "outside",
                    labelPosition: "outside",
                  
                    majorGridLines:
       {
          
           width: 1,
          
           dashArray: "",
          
           visible: true

       },
                    alternateGridBand:
           {
               odd: {
                   fill: "transparent",
                   opacity: 1
               },
               even: {
                   fill: "transparent",
                   opacity: 1
               }
           },
                 
                    majorTickLines:
                        {
                       
                            width: 1,
                      
                            size: 5,
                        
                            visible: true
                        },
                   
                    minorGridLines:
                        {
                       
                            width: 1,
                        
                            dashArray: "",
                         
                            visible: true

                        },
                   
                    minorTickLines:
                        {
                         
                            width: 1,
                          
                            size: 5,
                          
                            visible: true
                        },
                  
                    minorTicksPerInterval: null,
                 
                    rowIndex: null,
                  
                    rowSpan: null,
                   
                    valueType: null,
                   
                    name: null,
                   
                    labelFormat: null,
                   
                    desiredIntervals: null,
                 
                    intervalType: null,
                  
                    roundingPlaces: null,
                    labels: [],
                  
                    title:
                    {
                    
                        text: "",
                       
                        font:
                        {
                         
                           
                            fontStyle: 'Normal',
                          
                            opacity: 1,
                           
                            size: '16px',
                           
                            fontWeight: 'regular'
                        }
                    },
                 
                    rangePadding: 'Normal',

                    stripLine: [
                         {
                             visible: false,
                             startFromAxis: false,
                             text: "stripLine",
                             textAlignment: "middlecenter",
                             font: { fontFamily: 'Segoe UI', fontStyle: 'Normal', fontWeight: 'regular', size: '12px', color: 'black', opacity: 1 },
                             start: null,
                             end: null,
                             color: 'gray',
                             borderColor: 'black',
                             zIndex: 'over'
                        }
                    ],
                   
                    logBase: 10,
                   
                    plotOffset: 0,
                   
                    orientation: 'Vertical',
                   
                    maximumLabels: 3,
                 
                    labelIntersectAction: "none",
                   
                    edgeLabelPlacement: "none",
                   
                    crosshairLabel:
                    {
                      
                        visible: false
                    },
                  
                    visible: true,
                  
                    opposedPosition: false,
                   
                    font:
                    {
                        fontFamily: 'Segoe UI',
                       
                        fontStyle: 'Normal',
                       
                        size: '13px',
                        
                        opacity: 1,
                       
                        fontWeight: 'regular'
                    },
                   
                    axisLine:
                        {
                           
                            visible: true,
                         
                            width: 1,
                          
                            dashArray: "",
                          
                            offset: 0
                        },

                   
                    zoomFactor: 1,
                   
                    zoomPosition: 0
                },
            indicatorDefaults: {
                points: [],
                tooltip: { visible: false, format: "#point.x# : #point.y#", fill: null, border: { width: 1, color: null }, duration: '500ms', enableAnimation: true, opacity: 0.95 },
                seriesName: "",
                font: { fontFamily: 'Segoe UI', fontStyle: 'Normal', fontWeight: 'Regular', size: '12px', color: '#707070', opacity: 1 },
                type: "SMA",
                period: 14,
                lowerLine: { fill: "#008000", width: 1 },
                upperLine: { fill: "#ff0000", width: 1 },
				macdLine: { fill: "#ff9933", width: 1 },
				histogram: { fill: "#ccccff", opacity: 1, border: { color: "#9999ff", width: 1 } },
                fill: "#00008B",
				width: 1,
                xAxisName: "",
                yAxisName: "",
                visibility: "visible",
                macdType: "line",
                shortPeriod: 12,
                longPeriod: 26,
				trigger: 9,
                enableAnimation:false
            },

            stripLineDefault: { 
                
                visible: false,
                
                startFromAxis: false,
               
                text: "stripLine",
               
                textAlignment: 'middlecenter',
                
                font:
                {
                   
                    fontFamily: 'Segoe UI',
                  
                    fontStyle: 'Normal',
                   
                    fontWeight: 'Regular',
                   
                    size: '12px',
                   
                    color: 'black',
                    
                    opacity: 1
                },
               
                start: null,
               
                end: null,
               
                color: 'gray',
                
                borderColor: 'black',
               
                zIndex: 'over',
               
                borderWidth: 1
            },
            /**		
         * You can split chart into multiple plotting areas horizontally and vertically. In order to split the chart vertically, you need to specify number of row objects in rowDefintions array. Each object in the rowDefnitions represent a plotting area in chart.
         * @type {array}			 
         * @example 
         * &lt;div id="container"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#container").ejChart({
      * primaryXAxis :{ rowDefinitions :[{unit : "percentage"}] }                      
      * });
      * &lt;/script&gt;  
         */
            rowDefinitions: null,
            /**		
        * You can split chart into multiple plotting areas horizontally and vertically. In order to split the chart horizontally, you need to specify number of column objects in columDefintions array. Each object in the columnDefnitions represent a plotting area in chart.
        * @type {array}		 
        * @example 
       * &lt;div id="container"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#container").ejChart({
      * primaryXAxis :{ columnDefinitions :[{unit : "percentage"}] }                      
      * });
      * &lt;/script&gt; 
        */
            columnDefinitions: null,
            /**		
           * Specifies the chart title. This title property denotes the title of the chart. 
           * @type {object}
           */
            title: /** @lends ejChart# */
				{ 
				    /**		
       * Specifies the text of the chart title. The text which is to be render as chart title is set here.
       * @default ""
       * @type {string}		 
       * @example 
       * &lt;div id="container"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#container").ejChart({
      * title : { text : "Power Production"}                     
      * });
      * &lt;/script&gt; 
       * @alias ejChart#title->text
       */
				    text: "",
				    /**		
     * This property is used to change the alignment of the title horizontally. See {@link TextAlignment}
     * @default ej.datavisualization.Chart.TextAlignment.Center
     * @type {enum}			
     * @example 
     * &lt;div id="container"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#container").ejChart({
      * title : { textAlignment : "near"}                     
      * });
      * &lt;/script&gt; 
     * @alias ejChart#title->textAlignment
     */
				    textAlignment: "center",
				    /**		
          * Specifies the chart title. These are the font properties involved in chart title customization.
          * @type {object}
           * @alias ejChart#title->font
          */
				    font: {
				        /**		
 * Specifies the title fontfamily of primaryXAxis. Chart title render with this specified fontFamily.
 * @default "Segoe UI"
 * @type {string}				 
 * @example 
  * &lt;div id="container"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#container").ejChart({
      * title : { font : { fontFamily : "Algerian" } }                     
      * });
      * &lt;/script&gt; 
  * @alias ejChart#title->font->fontFamily
 */
				        fontFamily: 'Segoe UI',
				        /**		
    * Specifies the title fontStyle of primaryXAxis. Chart title render with this specified fontStyle. See {@link FontStyle}
    * @default ej.datavisualization.Chart.FontStyle.Normal
    * @type {enum}			 
    * @example 
      * &lt;div id="container"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#container").ejChart({
      * title : { font : { fontStyle : "italic" } }                     
      * });
      * &lt;/script&gt; 
     * @alias ejChart#title->font->fontStyle
    */
				        fontStyle: 'Normal',
				        /**		
 * Specifies the title size. Chart title render with this specified font size.
 * @default "20px"
 * @type {string}				
 * @example 
 * &lt;div id="container"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#container").ejChart({
      * title : { font : { size : "22px" } }                     
      * });
      * &lt;/script&gt; 
  * @alias ejChart#title->font->size
 */
				        size: '20px',
				        /**		
* Specifies the title opacity. Chart title render with this specified font opacity.
* @default 0.5
* @type {number}			 
* @example 
 * &lt;div id="container"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#container").ejChart({
      * title : { font : { opacity : 0.8 } }                     
      * });
      * &lt;/script&gt; 
 * @alias ejChart#title->font->opacity
*/
				        opacity: 0.5,
				        /**		
 * Specifies the title fontWeight. Chart title render with this specified fontWeight.
 * @default ej.datavisualization.Chart.FontWeight.Regular
 * @type {enum}			 
 * @example 
 * &lt;div id="container"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#container").ejChart({
      * title : { font : { fontWeight : "lighter" } }                     
      * });
      * &lt;/script&gt; 
 * @alias ejChart#title->font->fontWeight
 */
				        fontWeight: 'regular'
				    },
/**		
  * Specifies the options to customize the chart subtitle.
  * @type {object}
  * @alias ejChart#title->subTitle
  */
				
  subTitle:
         {
		 /**		
* Specifies the text to be displayed as subtitle.
* @default ""
* @type {string}				
* @example 
* &lt;div id="container"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#container").ejChart({
      * title: { subTitle: { text : "Performace chart" } }                      
      * });
      * &lt;/script&gt; 
 * @alias ejChart#title->subTitle->text
*/
             text: "",
			 /**		
* Specifies the text alignment for subtitle text.
* @default "far"
* @type {enum}				
* @example 
* &lt;div id="container"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#container").ejChart({
      * title: { subTitle: { textAlignment : "near" } }                      
      * });
      * &lt;/script&gt; 
 * @alias ejChart#title->subTitle->textAlignment
*/
             textAlignment: 'far',
			 /**		
  * Specifies the font options for customizing the subtitle.
  * @type {object}
  * @alias ejChart#title->subTitle->font
  */
             font:
             {
 /**		
* Specifies the font family of subtitle.
* @default "Segoe UI"
* @type {string}				
* @example 
* &lt;div id="container"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#container").ejChart({
      * title: {subTitle : {font :{ fontFamily : "Algerian" } } }                      
      * });
      * &lt;/script&gt; 
 * @alias ejChart#title->subTitle->font->fontFamily
*/
             fontFamily: 'Segoe UI',
/**		
* Specifies the font style of subtitle.
* @default "Normal"
* @type {enum}				
* @example 
* &lt;div id="container"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#container").ejChart({
      * title: { subTitle : {font :{ fontStyle : "Normal" } } }                     
      * });
      * &lt;/script&gt; 
 * @alias ejChart#title->subTitle->font->fontStyle
*/
             fontStyle: 'Normal',   
/**		
* Specifies the font size of subtitle.
* @default "12px"
* @type {enum}				
* @example 
* &lt;div id="container"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#container").ejChart({
      * title: { subTitle : {font :{ size : "14px" } } }                     
      * });
      * &lt;/script&gt; 
 * @alias ejChart#title->subTitle->font->size
*/			 
             size: '12px',
			 /**		
* Specifies the font opacity of subtitle.
* @default 1
* @type {double}				
* @example 
* &lt;div id="container"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#container").ejChart({
      * title: { subTitle : {font :{ opacity : 0.5 } } }                     
      * });
      * &lt;/script&gt; 
 * @alias ejChart#title->subTitle->font->opacity
*/	
             opacity: 1,
			 /**		
* Specifies the font weight of subtitle.
* @default "regular"
* @type {enum}				
* @example 
* &lt;div id="container"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#container").ejChart({
      * title: { subTitle : {font :{ fontWeight : "regular" } } }                     
      * });
      * &lt;/script&gt; 
 * @alias ejChart#title->subTitle->font->fontWeight
*/	
             fontWeight: 'regular'
              }
           }
				    
	},  				



				

            lineCap: { butt: 'butt', round: 'round', square: 'square' },
            lineJoin: { round: 'round', bevel: 'bevel', miter: 'miter' },
            legendAlignment: { near: 'near', center: 'center', far: 'far' },
            legendPosition: { top: 'top', bottom: 'bottom', right: 'right', left: 'left', custom: 'custom' },
           
            enableAnimation: true,
            /**		
          * Contains all the properties to customize legend.
          * @type {object}
          */
            legend: /** @lends ejChart# */
            {
			/**		
  * Specifies the title of the legend.
  * @type {object}
  * @alias ejChart#legend->title
  */
                title:
                      {
					  /**		
* Specifies the text to be displayed as legend title.
* @default ""
* @type {string}				
* @example 
* &lt;div id="container"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#container").ejChart({
      * legend: { title: { text : "Countries" } }                      
      * });
      * &lt;/script&gt; 
 * @alias ejChart#legend->title->text
*/
                          text: "",
						  /**		
* Specifies the text alignment of legend title.
* @default "center"
* @type {enum}				
* @example 
* &lt;div id="container"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#container").ejChart({
      * legend: { title: { textAlignment : "near" } }                      
      * });
      * &lt;/script&gt; 
 * @alias ejChart#legend->title->textAlignment
*/
                          textAlignment: 'center',
						  /**		
  * Specifies the font options to customize the legend title.
  * @type {object}
  * @alias ejChart#legend->title->font
  */
                          font:
                              {
							  /**		
* Specifies the size of legend title text.
* @default "12px"
* @type {string}				
* @example 
* &lt;div id="container"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#container").ejChart({
      * legend: { title: { font :{size: "14px" } } }                      
      * });
      * &lt;/script&gt; 
 * @alias ejChart#legend->title->font->size
*/
                                  size: '12px',
								  /**		
* Specifies the font family of legend title text.
* @default "Segoe UI"
* @type {string}				
* @example 
* &lt;div id="container"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#container").ejChart({
      * legend: { title: { font :{fontFamily: "Algerian" } } }                      
      * });
      * &lt;/script&gt; 
 * @alias ejChart#legend->title->font->fontFamily
*/
                                  fontFamily: "Segoe UI",
								   /**		
* Specifies the font style of legend title text.
* @default "normal"
* @type {enum}				
* @example 
* &lt;div id="container"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#container").ejChart({
      * legend: { title: { font :{fontStyle: "normal" } } }                      
      * });
      * &lt;/script&gt; 
 * @alias ejChart#legend->title->font->fontStyle
*/
                                  fontStyle: 'normal',
								  /**		
* Specifies the font weight of legend title text.
* @default "normal"
* @type {enum}				
* @example 
* &lt;div id="container"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#container").ejChart({
      * legend: { title: { font :{fontWeight: "normal" } } }                      
      * });
      * &lt;/script&gt; 
 * @alias ejChart#legend->title->font->fontWeight
*/
								  fontWeight : 'normal'
                              }
                      },
              

                /**		
          * Options for customizing the color and width of the border of legend.
          * @type {object}
           * @alias ejChart#legend->border
          */
                border:
                {
                    /**		
* Specifies the border color of legend.
* @default "transparent"
* @type {string}			
* @example 
* &lt;div id="container"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#container").ejChart({
      * legend : {border :{ color :"green"}}                     
      * });
      * &lt;/script&gt;   
* @alias ejChart#legend->border->color
*/
                    color: 'transparent',

                    /**		
* Specifies the border width of legend.
* @default 1
* @type {number}				
* @example 
* &lt;div id="container"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#container").ejChart({
      * legend :{ border :{width :2}}                     
      * });
      * &lt;/script&gt;  
 * @alias ejChart#legend->border->width
*/
                    width: 1
                },
                /**		
 * Toggles the legend visibility.
 * @default true
 * @type {boolean}			 
 * @example 
* &lt;div id="container"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#container").ejChart({
      * legend :{visible : false}                     
      * });
      * &lt;/script&gt;   
  * @alias ejChart#legend->visible
 */
                visible: true,
                /**		
* This is used to specify the gap/padding between the legend items.
* @default 10
* @type {number}			
* @example 
* &lt;div id="container"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#container").ejChart({
      * legend :{itemPadding : 5}                     
      * });
      * &lt;/script&gt;  
 * @alias ejChart#legend->itemPadding
*/
                itemPadding: 10,
                /**		
 * This property specifies the shape of the legend items. See {@link Shape}
 * @default ej.datavisualization.Chart.Shape.None
 * @type {enum}				 
 * @example 
 * &lt;div id="container"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#container").ejChart({
      * legend :{ shape : "circle" }                      
      * });
      * &lt;/script&gt;  
  * @alias ejChart#legend->shape
 */
                shape: 'None',
                /**		
 * This property is used to change the alignment of the legend horizontally.. See {@link Alignment}
 * @default ej.datavisualization.Chart.Alignment.Center
 * @type {enum}				 
 * @example 
 * &lt;div id="container"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#container").ejChart({
      * legend :{alignment : "far"}                    
      * });
      * &lt;/script&gt;   
  * @alias ejChart#legend->alignment
 */
                alignment: 'Center',
                /**		
* This property is used to change the alignment of the data label vertically. See {@link Position}
* @default ej.datavisualization.Chart.Position.Bottom
* @type {enum}				 
* @example 
* &lt;div id="container"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#container").ejChart({
      * legend :{ position : "top"}                    
      * });
      * &lt;/script&gt;   
* @alias ejChart#legend->position
*/
                position: 'Bottom',
                /**		
          * Contains property to customize the position of the legend
          * @type {object}
          * @alias ejChart#legend->location
          */
                location:
                {
                    /**		
 * This property specifies the x position for rendering legend.
 * @default 0
 * @type {number}				 
 * @example 
* &lt;div id="container"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#container").ejChart({
      * legend :{location :{x :20}}                    
      * });
      * &lt;/script&gt;   
 * @alias ejChart#legend->location->x
 */
                    x: 0,
                    /**		
* This property specifies the y position for rendering legend.
* @default 0
* @type {number}				 
* @example 
* &lt;div id="container"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#container").ejChart({
      * legend :{location : {y : 100}}                  
      * });
      * &lt;/script&gt;  
* @alias ejChart#legend->location->y
*/
                    y: 0
                },
                /**		
          * Contains property to customize the legend item style.
          * @type {object}
           * @alias ejChart#legend->itemStyle
          */
                itemStyle:
                {
                    /**		
* Specifies the legend item height.
* @default 10
* @type {number}			
* @example 
* &lt;div id="container"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#container").ejChart({
      * legend :{ itemStyle :{height : 20}}                    
      * });
      * &lt;/script&gt;  
* @alias ejChart#legend->itemStyle->height
*/
                    height: 10,
                    /**		
 * Specifies the legend item width.
 * @default 10
 * @type {number}				 
 * @example 
 * &lt;div id="container"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#container").ejChart({
      * legend :{ itemStyle :{width : 15}}                    
      * });
      * &lt;/script&gt; 
      * @alias ejChart#legend->itemStyle->width
 */
                    width: 10,
                    /**		
         * Options for customizing the color and width of the border of legend.
          * @type {object}
           * @alias ejChart#legend->itemStyle->border
          */
                    border:
                    {
                        /**		
 * Specifies the border color of legend.
 * @default "transparent"
 * @type {string}				 
 * @example 
* &lt;div id="container"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#container").ejChart({
      * legend :{ itemStyle :{border : { color : "green' }}}                    
      * });
      * &lt;/script&gt; 
 * @alias ejChart#legend->itemStyle->border->color
 */
                        color: 'transparent',
                        /**		
 * Specifies the border width of legend.
 * @default 1
 * @type {number}				 
 * @example 
 * &lt;div id="container"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#container").ejChart({
      * legend :{ itemStyle :{border :{ width : 2 }}}                    
      * });
      * &lt;/script&gt;  
 * @alias ejChart#legend->itemStyle->border->width
 */
                        width: 1
                    }
                },
                /**		
 * This property specifies the number of rows for the legend items to arrange.
 * @default null
 * @type {number}			
 * @example 
 * &lt;div id="container"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#container").ejChart({
      * legend :{ rowCount :2}                    
      * });
      * &lt;/script&gt; 
 * @alias ejChart#legend->rowCount
 */
                rowCount: null,
                /**		
 * This property specifies the number of columns for the legend items to arrange.
 * @default null
 * @type {number}			 
 * @example 
 * &lt;div id="container"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#container").ejChart({
      * legend :{ columnCount : 2}                    
      * });
      * &lt;/script&gt; 
 * @alias ejChart#legend->columnCount
 */
                columnCount: null,
                /**		
       * Specifies the opacity of legend
       * @default 1
       * @type {number}				 
       * @example 
       * &lt;div id="container"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#container").ejChart({
      * legend :{ opacity : 0.5}                    
      * });
      * &lt;/script&gt; 
       * @alias ejChart#legend->opacity
       */
                opacity: 1,
                /**		
      * Specifies the fill color of legend
      * @default null
      * @type {string}				 
      * @example 
      * &lt;div id="container"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#container").ejChart({
      * legend :{ fill : "green"}                    
      * });
      * &lt;/script&gt; 
     * @alias ejChart#legend->fill
      */
                fill: null,
                /**		
          * Specifies the legend font. These font properties are used to customize the legend item text.
          * @type {object}
           * @alias ejChart#legend->font
          */
                font:
                {
                    /**		
    * Specifies the legend fontfamily color. Legend item text render with this specified font family.
    * @default "Segoe UI"
    * @type {string}				
    * @example 
    * &lt;div id="container"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#container").ejChart({
      * legend :{ font :{fontFamily : "algerian"}}                    
      * });
      * &lt;/script&gt;  
   * @alias ejChart#legend->font->fontFamily
    */
                    fontFamily: 'Segoe UI',
                    /**		
    * Specifies the legend font style. Legend item text render with this specified font style. See {@link FontStyle}
    * @default ej.datavisualization.Chart.FontStyle.Normal
    * @type {enum}			
    * @example 
   * &lt;div id="container"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#container").ejChart({
      * legend :{ font :{fontStyle : "italic"}}                    
      * });
      * &lt;/script&gt; 
    * @alias ejChart#legend->font->fontStyle
    */
                    fontStyle: 'Normal',
                    /**		
    * Specifies the legend font weight.  Legend item text render with this specified font weight.{@link FontWeight}
    * @default ej.datavisualization.Chart.FontWeight.Regular
    * @type {enum}				 
    * @example 
    * &lt;div id="container"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#container").ejChart({
      * legend :{ font :{fontWeight : "lighter"}}                    
      * });
      * &lt;/script&gt;  
    * @alias ejChart#legend->font->fontWeight
    */
                    fontWeight: 'Regular',
                    /**		
     * Specifies the legend font size.  Legend item text render with this specified size.
     * @default "12px"
     * @type {string}			 
     * @example 
     * &lt;div id="container"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#container").ejChart({
      * legend :{ font :{size : "14px"}}                    
      * });
      * &lt;/script&gt; 
     * @alias ejChart#legend->font->size
     */
                    size: '12px'
                },
                /**		
* Contains property to customize the size of legend.
* @type {object}
* @alias ejChart#legend->size
                  */
                size:
                    {
                        /**
* Specifies the legend height.
* @default null
* @type {number}			
* @example 
* &lt;div id="container"&gt;&lt;/div&gt; 
      * &lt;script&gt;
    * $("#container").ejChart({
    * legend :{ size :{height : 20%}}                    
    * });
    * &lt;/script&gt;  
* @alias ejChart#legend->size->height
*/
                        height: null,
                        /**
* Specifies the legend width.
* @default null
* @type {number}			
* @example 
* &lt;div id="container"&gt;&lt;/div&gt; 
      * &lt;script&gt;
        * $("#container").ejChart({
        * legend :{ size :{width : 20%}}                    
        * });
        * &lt;/script&gt;  
    * @alias ejChart#legend->size->width
    */
                        width:null
                    }
            },
            
            indicators: /** @lends ejChart# */[
              {
                  points: [],
                  tooltip: { visible: false, format: "#point.x# : #point.y#", fill: null, border: { width: 1, color: null }, duration: '500ms', enableAnimation: true, opacity: 0.95 },
                  seriesName: "", type: "SMA", period: 14,
                  lowerLine: { fill: "#008000", width: 1 },
                  upperLine: { fill: "#ff0000", width: 1 },
				  macdLine: { fill: "#ff9933", width: 1 },
				  histogram: { fill: "#ccccff", opacity: 1, border: { color: "#9999ff", width: 1 } },
                  fill: "#00008B",
                  font: { fontFamily: 'Segoe UI', fontStyle: 'Normal', fontWeight: 'Regular', size: '12px', color: '#707070', opacity: 1 },
                  width: 1,
                  xAxisName: "",
                  yAxisName: "",
                  macdType: "line",
                  shortPeriod: 12,
                  longPeriod: 26,
				  trigger: 9,
                  visibility: "visible",
                  enableAnimation:false
              }
            ],
            /**		
         * Specifies the properties used for customizing the series.
         * @type {array}
         */
            series: /** @lends ejChart# */[
                {
                    /**		
     * Specifies the type of series to be rendered in chart. see {@link Type}
     * @default "line"
     * @type {enum}			 
     * @example 
     * &lt;div id="container"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#container").ejChart({
      * series : [{type : "column" }]                   
      * });
      * &lt;/script&gt; 
     * @alias ejChart#series->type
     */
                    type: 'column',
                    /**		
    * Specifies the animation of the series.  ejChart supports animation for the series. This can be enable/disable by this property
    * @default true
    * @type {boolean}			 
    * @example 
    * &lt;div id="container"&gt;&lt;/div&gt; 
     * &lt;script&gt;
     * $("#container").ejChart({
     * series : [{enableAnimation : false }]                   
     * });
     * &lt;/script&gt; 
    * @alias ejChart#series->enableAnimation
    */
                    enableAnimation: false,
                    /**		
    * Used in Polar and Radar series to specify whether the line/area series has to be rendered in a closed state by joining the start and end point of the series.
    * @default true
    * @type {boolean}			 
    * @example 
    * &lt;div id="container"&gt;&lt;/div&gt; 
     * &lt;script&gt;
     * $("#container").ejChart({
     * series : [{isClosed : false }]                   
     * });
     * &lt;/script&gt; 
    * @alias ejChart#series->isClosed
    */
                    isClosed: true,
                    /**		
    * Used in Polar and Radar series to specify whether column type has to be rendered the stacked value of chart.
    * @default true
    * @type {boolean}			 
    * @example 
    * &lt;div id="container"&gt;&lt;/div&gt; 
     * &lt;script&gt;
     * $("#container").ejChart({
     * series : [{isStacking : false }]                   
     * });
     * &lt;/script&gt; 
    * @alias ejChart#series->isStacking
    */
                    isStacking: true,
                    /**		
    * Specifies the type of series that can be drawn in a Radar or Polar series. It supports line, column and area type. See {@link DrawType}
    * @default true
    * @type {boolean}			 
    * @example 
    * &lt;div id="container"&gt;&lt;/div&gt; 
     * &lt;script&gt;
     * $("#container").ejChart({
     * series : [{drawType : false }]                   
     * });
     * &lt;/script&gt; 
    * @alias ejChart#series->drawType
    */
                    drawType: 'line',
                    /**		
    * Sets the radius of the hole on the doughnut series with respect to plot area. Value ranges from 0 to 1.
    * @default 0.4
    * @type {number}			 
    * @example 
    * &lt;div id="container"&gt;&lt;/div&gt; 
     * &lt;script&gt;
     * $("#container").ejChart({
     * series : [{doughnutCoefficient : 0.5 }]                   
     * });
     * &lt;/script&gt; 
    * @alias ejChart#series->doughnutCoefficient
    */
                    doughnutCoefficient: 0.4,
                    /**		
   * Represents the distance the slice has to be moved out when it is exploded.
   * @default 25
   * @type {number}			 
   * @example 
   * &lt;div id="container"&gt;&lt;/div&gt; 
    * &lt;script&gt;
    * $("#container").ejChart({
    * series : [{explodeOffset : 20 }]                   
    * });
    * &lt;/script&gt; 
   * @alias ejChart#series->explodeOffset
   */
                    explodeOffset: 25,
                    /**		
  * Sets the rendering mode of the pyramid as linear or surface.
  * @default "linear"
  * @type {enum}			 
  * @example 
  * &lt;div id="container"&gt;&lt;/div&gt; 
   * &lt;script&gt;
   * $("#container").ejChart({
   * series : [{pyramidMode : "linear" }]                   
   * });
   * &lt;/script&gt; 
  * @alias ejChart#series->pyramidMode
  */
                    pyramidMode: 'linear',
/**		
  * Sets the funnel width.
  * @default "11.6%"(11.6% of actualwidth)
  * @type {string}
  * @example 
  * &lt;div id="container"&gt;&lt;/div&gt; 
   * &lt;script&gt;
   * $("#container").ejChart({
   * series : [{funnelWidth : '40%' }]                   
   * });
   * &lt;/script&gt; 
   * @alias ejChart#series->funnelWidth
  */
                    funnelWidth: '11.6%',
                    /**		
  * Sets the funnel height.
  * @default "32.7%"(32.7% of actualheight)
  * @type {string}
  * @example 
  * &lt;div id="container"&gt;&lt;/div&gt; 
   * &lt;script&gt;
   * $("#container").ejChart({
   * series : [{funnelHeight : '40%' }]                   
   * });
   * &lt;/script&gt; 
   * @alias ejChart#series->funnelHeight
  */
                    funnelHeight:'32.7%',
                    /**		
  * Specifies the position of the axis labels to render labels either inside or outside of plot area. See {@link LabelPosition}
  * @default "inside"
  * @type {enum}			 
  * @example 
  * &lt;div id="container"&gt;&lt;/div&gt; 
   * &lt;script&gt;
   * $("#container").ejChart({
   * series : [{labelPosition : "outside" }]                   
   * });
   * &lt;/script&gt; 
  * @alias ejChart#series->labelPosition
  */
                    labelPosition: 'inside',
                    /**		
 * Specifies the gap between each slice of a pyramid series in pixels.
 * @default 0
 * @type {number}			 
 * @example 
 * &lt;div id="container"&gt;&lt;/div&gt; 
  * &lt;script&gt;
  * $("#container").ejChart({
  * series : [{gapRatio : 2 }]                   
  * });
  * &lt;/script&gt; 
 * @alias ejChart#series->gapRatio
 */
                    gapRatio: 0,
                    /**		
* Specifies the points of the series.  This is to specify the point values for the series
* @default [ ] 
* @type {array}			 
* @example 
* &lt;div id="container"&gt;&lt;/div&gt; 
 * &lt;script&gt;
 * $("#container").ejChart({
 * series : [{points : [{x : 2000, y : 15}] }]                   
 * });
 * &lt;/script&gt; 
* @alias ejChart#series->points
*/
                    points: null,
                    /**		
* Sets the radius of the pie series with respect to plot area. Value ranges from 0 to 1.
* @default 0.8
* @type {number}			 
* @example 
* &lt;div id="container"&gt;&lt;/div&gt; 
 * &lt;script&gt;
 * $("#container").ejChart({
 * series : [{pieCoefficient : 0.6 }]                   
 * });
 * &lt;/script&gt; 
* @alias ejChart#series->pieCoefficient
*/
                    pieCoefficient: 0.8,
                    /**		
* Sets the radius of the doughnut series with respect to plot area. Value ranges from 0 to 1. 
* @default 0.8
* @type {number}			 
* @example 
* &lt;div id="container"&gt;&lt;/div&gt; 
* &lt;script&gt;
* $("#container").ejChart({
* series : [{doughnutSize : 0.6 }]                   
* });
* &lt;/script&gt; 
* @alias ejChart#series->doughnutSize
*/
                    doughnutSize: 0.8,
                    /**		
* Specifies the dataSource for series. 
* @default null	
* @type {object}
* @example 
* &lt;div id="container"&gt;&lt;/div&gt; 
* &lt;script&gt;
* $("#container").ejChart({
* series : [{dataSource: data.open }]                   
* });
* &lt;/script&gt; 
* @alias ejChart#series->dataSource
*/
                    dataSource: null,
                    /**		
* Specifies the bullFillColor for series.  This is used in hilo series types
* @default null
* @type {string}			 
* @example 
* &lt;div id="container"&gt;&lt;/div&gt; 
* &lt;script&gt;
* $("#container").ejChart({
* series : [{bullFillColor: "green" }]                   
* });
* &lt;/script&gt; 
* @alias ejChart#series->bullFillColor
*/
                    bullFillColor: null,
                    /**		
* Specifies the bearFillColor for series.  This is used in hilo series types
* @default null
* @type {string}			 
* @example 
* &lt;div id="container"&gt;&lt;/div&gt; 
* &lt;script&gt;
* $("#container").ejChart({
* series : [{bearFillColor: "blue" }]                   
* });
* &lt;/script&gt; 
* @alias ejChart#series->bearFillColor
*/
                    bearFillColor: null,
                    /**		
* Specifies the query for dataSource in series.  This is used to take the query from dataSource
* @default null
* @type {object}			 
* @example 
* &lt;div id="container"&gt;&lt;/div&gt; 
* &lt;script&gt;
* var query =  ej.Query().from("Orders").take(10);
* $("#container").ejChart({
* series : [{query: query }]                   
* });
* &lt;/script&gt; 
* @alias ejChart#series->query
*/
                    query: null,
                    /**		
* Specifies the xName for dataSource in series.  This is used to take the x values from dataSource
* @default null	
* @type {string}
* @example 
* &lt;div id="container"&gt;&lt;/div&gt; 
* &lt;script&gt;
* $("#container").ejChart({
* series : [{xName: "XValue" }]                   
* });
* &lt;/script&gt; 
* @alias ejChart#series->xName
*/
                    xName: '',
                    /**		
* Specifies the yName for dataSource in series.  This is used to take the y values from dataSource
* @default null
* @type {string}			 
* @example 
* &lt;div id="container"&gt;&lt;/div&gt; 
* &lt;script&gt;
* $("#container").ejChart({
* series : [{yName: "YValue" }]                   
* });
* &lt;/script&gt; 
* @alias ejChart#series->yName
*/
                    yName: '',
                    /**		
* Specifies the visibility for series.  This is used to change the visibility of series
* @default "visible"
* @type {string}			 
* @example 
* &lt;div id="container"&gt;&lt;/div&gt; 
* &lt;script&gt;
* $("#container").ejChart({
* series : [{visibility: "hidden" }]                   
* });
* &lt;/script&gt; 
* @alias ejChart#series->visibility
*/
                    visibility: "visible",
                    /**		
* Specifies the starting angle for rendering pie / doughnut series.
* @default null
* @type {number}			 
* @example 
* &lt;div id="container"&gt;&lt;/div&gt; 
* &lt;script&gt;
* $("#container").ejChart({
* series : [{startAngle: 140 }]                   
* });
* &lt;/script&gt; 
* @alias ejChart#series->startAngle
*/
                    startAngle: null,
                    /**		
* Specifies the ending angle for rendering pie / doughnut series.
* @default null
* @type {number}			 
* @example 
* &lt;div id="container"&gt;&lt;/div&gt; 
* &lt;script&gt;
        * $("#container").ejChart({
        * series : [{endAngle: 270 }]                   
        * });
        * &lt;/script&gt; 
        * @alias ejChart#series->endAngle
        */
                    endAngle:null,
                    /**		
* Specifies the XAxisName for series.  This is used to specify the name of the xAxis
* @default null
* @type {string}			 
* @example 
* &lt;div id="container"&gt;&lt;/div&gt; 
* &lt;script&gt;
* $("#container").ejChart({
* series : [{xAxisName: "xAxis" }]                   
* });
* &lt;/script&gt; 
* @alias ejChart#series->xAxisName
*/
                    xAxisName: null,
                    /**		
* Specifies the yAxisName for series.  This is used to specify the name of the yAxis
* @default null
* @type {string}			 
* @example 
* &lt;div id="container"&gt;&lt;/div&gt; 
* &lt;script&gt;
* $("#container").ejChart({
* series : [{yAxisName: "yAxis" }]                   
* });
* &lt;/script&gt; 
* @alias ejChart#series->yAxisName
*/
                    yAxisName: null,
                    /**		
* Explodes all the pie slices. (Also works for doughnut series)es
* @default null
* @type {boolean}			 
* @example 
* &lt;div id="container"&gt;&lt;/div&gt; 
* &lt;script&gt;
* $("#container").ejChart({
* series : [{explodeAll: true }]                   
* });
* &lt;/script&gt; 
* @alias ejChart#series->explodeAll
*/
                    explodeAll: null,
                    /**		
* Index of a slice to be exploded from the whole pie. (Also works for doughnut series)
* @default null
* @type {number}			 
* @example 
* &lt;div id="container"&gt;&lt;/div&gt; 
* &lt;script&gt;
* $("#container").ejChart({
* series : [{explodeIndex : 2 }]                   
* });
* &lt;/script&gt; 
* @alias ejChart#series->explodeIndex
*/
                    explodeIndex: null,  
                    /**		
* Enabling this property can avoid pie series data label overlap each other.
* @default null
* @type {number}			 
* @example 
* &lt;div id="container"&gt;&lt;/div&gt; 
* &lt;script&gt;
* $("#container").ejChart({
* series : [{enableSmartLabels : false }]                   
* });
* &lt;/script&gt; 
* @alias ejChart#series->enableSmartLabels
*/
                    enableSmartLabels: true,
                    /**		
      * Contains all the properties to customize tooltip
      * @type {object}
       * @alias ejChart#series->tooltip
      */
                    tooltip: {
                        /**		
* tooltip visibility can be toggled using this property
* @default false
* @type {boolean}			 
* @example 
* &lt;div id="container"&gt;&lt;/div&gt; 
* &lt;script&gt;
* $("#container").ejChart({
* series : [{tooltip : {visible : true} }]                   
* });
* &lt;/script&gt; 
* @alias ejChart#series->tooltip->visible
*/
                        visible: false,
                        /**		
* Formatting the values of point on a tooltip can be customized using this property.
* @default "#point.x# : #point.y#"
* @type {string}			 
* @example 
* &lt;div id="container"&gt;&lt;/div&gt; 
* &lt;script&gt;
* $("#container").ejChart({
* series : [{tooltip : {format : "#point.x# : #point.y#"} }]                   
* });
* &lt;/script&gt; 
* @alias ejChart#series->tooltip->format
*/
                        format: "#point.x# : #point.y#",
                        /**		
* Specifies the fill color of the tooltip.
* @default null
* @type {string}			 
* @example 
* &lt;div id="container"&gt;&lt;/div&gt; 
* &lt;script&gt;
* $("#container").ejChart({
* series : [{tooltip : {fill : "green"} }]                   
* });
* &lt;/script&gt; 
* @alias ejChart#series->tooltip->fill
*/
                        fill: null,
                        /**		
      * Options for customizing the color, opacity and width of the border of tooltip.
      * @type {object}
       * @alias ejChart#series->tooltip->border
      */
                        border: {
                            /**		
* Specifies the width of tooltip.
* @default 1
* @type {number}			 
* @example 
* &lt;div id="container"&gt;&lt;/div&gt; 
* &lt;script&gt;
* $("#container").ejChart({
* series : [{tooltip : {border : { width :2} }]                   
* });
* &lt;/script&gt; 
* @alias ejChart#series->tooltip->border->width
*/
                            width: 1,
                            /**		
* Specifies the color of tooltip.
* @default null
* @type {string}			 
* @example 
* &lt;div id="container"&gt;&lt;/div&gt; 
* &lt;script&gt;
* $("#container").ejChart({
* series : [{tooltip : {border : { color :"green"} }]                   
* });
* &lt;/script&gt; 
* @alias ejChart#series->tooltip->border->color
*/
                            color: null
                        },
                        /**		
* Enabling this property animates the tooltip on moving the cursor from one point to the other.
* @default true
* @type {boolean}			 
* @example 
* &lt;div id="container"&gt;&lt;/div&gt; 
* &lt;script&gt;
* $("#container").ejChart({
* series : [{tooltip : { enableAnimation: false }]                   
* });
* &lt;/script&gt; 
* @alias ejChart#series->tooltip->enableAnimation
*/
                        enableAnimation: true,
                        /**		
* Specifies the duration of the tooltip to stay awake.
* @default "500ms"
* @type {string}			 
* @example 
* &lt;div id="container"&gt;&lt;/div&gt; 
* &lt;script&gt;
* $("#container").ejChart({
* series : [{tooltip : { duration: "300ms" }]                   
* });
* &lt;/script&gt; 
* @alias ejChart#series->tooltip->duration
*/
                        duration: '500ms',
                        /**		
* Specifies the opacity of the tooltip. Values ranges from 0 to 1.
* @default 0.95
* @type {number}			 
* @example 
* &lt;div id="container"&gt;&lt;/div&gt; 
* &lt;script&gt;
* $("#container").ejChart({
* series : [{tooltip : { opacity: 0.5 }]                   
* });
* &lt;/script&gt; 
* @alias ejChart#series->tooltip->opacity
*/
                        opacity: 0.95
                    },
                    /**		
         * Specifies the fill color of series.
         * @default null
         * @type {string}			 
         * @example 
        * &lt;div id="container"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#container").ejChart({
      * series :[{fill : "green"}]                 
      * });
      * &lt;/script&gt;
        * @alias ejChart#series->fill
         */
                    fill: null,
                    /**		
       * Specifies the opacity of series.  Series render with this specified opacity.
       * @default 1
       * @type {number}				 
       * @example 
       * &lt;div id="container"&gt;&lt;/div&gt; 
     * &lt;script&gt;
     * $("#container").ejChart({
     * series :[{opacity : 0.5}]                  
     * });
     * &lt;/script&gt; 
        * @alias ejChart#series->opacity
       */
                    opacity: 1,
                    /**		
      * Specifies the style to draw the end point of a line like butt, round, square. See {@link LineCap}
      * @default ej.datavisualization.Chart.LineCap.Butt
      * @type {enum}				 
      * @example 
      * &lt;div id="container"&gt;&lt;/div&gt; 
     * &lt;script&gt;
     * $("#container").ejChart({
     * series :[{lineCap : "butt"}]                  
     * });
     * &lt;/script&gt;
       * @alias ejChart#series->lineCap
      */
                    lineCap: 'butt',
                    /**		
     * Specifies the shape to use at the intersection of two lines drawn.  See {@link LineJoin}
     * @default ej.datavisualization.Chart.LineJoin.Round
     * @type {enum}				 
     * @example 
     * &lt;div id="container"&gt;&lt;/div&gt; 
     * &lt;script&gt;
     * $("#container").ejChart({
     * series :[{lineCap : "round"}]                  
     * });
     * &lt;/script&gt; 
      * @alias ejChart#series->lineJoin
     */
                    lineJoin: 'round',
                    /**		
     * Controls the pattern of dashes and gaps used to stroke the line series.
     * @default ""
     * @type {string}				 
     * @example 
    * &lt;div id="container"&gt;&lt;/div&gt; 
     * &lt;script&gt;
     * $("#container").ejChart({
     * series :[{dashArray : "2,3"}]                  
     * });
     * &lt;/script&gt; 
      * @alias ejChart#series->dashArray
     */
                    dashArray: "",
                    /**		
         * Specifies the border of the series. These are the properties involved in customizing the series border.
         * @type {object}
           * @alias ejChart#series->border
         */
                    border: {
                        /**		
   * Specifies the border width of series.Series in chart render with this specified value as border width
   * @default 1
   * @type {number}			 
   * @example 
   * &lt;div id="container"&gt;&lt;/div&gt; 
     * &lt;script&gt;
     * $("#container").ejChart({
     * series :[{border :{ width : 2 } }]                  
     * });
     * &lt;/script&gt;
    * @alias ejChart#series->border->width
   */
                        width: 1,
                        /**		
  * Specifies the border color of series. Series in chart render with this specified color as border color
  * @default "transparent"
  * @type {string}			 
  * @example 
 * &lt;div id="container"&gt;&lt;/div&gt; 
     * &lt;script&gt;
     * $("#container").ejChart({
     * series :[{border :{ color : "green" } }]                  
     * });
     * &lt;/script&gt;
  * @alias ejChart#series->border->color
  */
                        color: 'transparent'
                    },
                    width: 2,
                    /**		
       * Contains the customizing properties to add marker symbols to the points plotted in the chart.
       * @type {object}
         * @alias ejChart#series->marker
       */
                    marker: {
                        /**		
 * Specifies the shape of the marker. See {@link Shape}
 * @default ej.datavisualization.Chart.Shape.Circle
 * @type {enum}				
 * @example 
* &lt;div id="container"&gt;&lt;/div&gt; 
     * &lt;script&gt;
     * $("#container").ejChart({
     * series :[{marker :{ shape: "rectangle"}]                  
     * });
     * &lt;/script&gt;
  * @alias ejChart#series->marker->shape
 */
                        shape: 'circle',
                        /**		
        * Specifies the size of the marker.
        * @type {object}
         * @alias ejChart#series->marker->size
        */
                        size: {
                            /**		
 * Specifies the width of the marker.
 * @default 6
 * @type {number}				 
 * @example 
 * &lt;div id="container"&gt;&lt;/div&gt; 
     * &lt;script&gt;
     * $("#container").ejChart({
     * series :[{marker :{ size :{ width : 2 } } }]                  
     * });
     * &lt;/script&gt;
 * @alias ejChart#series->marker->size->width
 */
                            width: 6,
                            /**		
* Specifies the height of the marker. 
* @default 6
* @type {number}				 
* @example 
* &lt;div id="container"&gt;&lt;/div&gt; 
     * &lt;script&gt;
     * $("#container").ejChart({
     * series :[{marker :{size :{height : 5}}}]                 
     * });
     * &lt;/script&gt;
* @alias ejChart#series->marker->size->height
*/
                            height: 6
                        },
                        /**		
* Toggles the visibility of the marker.
* @default false
* @type {boolean}			
* @example 
* &lt;div id="container"&gt;&lt;/div&gt; 
     * &lt;script&gt;
     * $("#container").ejChart({
     * series :[{marker :{ visible : true}}]                  
     * });
     * &lt;/script&gt; 
* @alias ejChart#series->marker->visible
*/
                        visible: false,
                        /**		
* Specifies the opacity of the marker. Values ranges from 0 to 1.
* @default 1
* @type {number}			
* @example 
* &lt;div id="container"&gt;&lt;/div&gt; 
     * &lt;script&gt;
     * $("#container").ejChart({
     * series :[{marker :{ opacity : 0.5 }}]                  
     * });
     * &lt;/script&gt;
* @alias ejChart#series->marker->opacity
*/
                        opacity: 1,
                        /**		
* Specifies the fill color of the marker.
* @default null
* @type {string}				 
* @example 
* &lt;div id="container"&gt;&lt;/div&gt; 
     * &lt;script&gt;
     * $("#container").ejChart({
     * series :[{marker : { fill : "green" } }]                  
     * });
     * &lt;/script&gt;
* @alias ejChart#series->marker->fill
*/
                        fill: null,
                        /**		
      * Options for customizing the color, opacity and width of the border of marker.
      * @type {object}
       * @alias ejChart#series->marker->border
      */
                        border: {
                            /**		
                        * Specifies the border color of marker.
                        * @default "white"
                        * @type {string}				 
                        * @example 
                        * &lt;div id="container"&gt;&lt;/div&gt; 
                        * &lt;script&gt;
                        * $("#container").ejChart({
                        * series :[{color : "green"}]                  
                        * });
                        * &lt;/script&gt; 
                        * @alias ejChart#series->marker->border->color
                        */
                            color: 'white',
                            /**		
* Specifies the width of the marker
* @default 3
* @type {number}				
* @example 
* &lt;div id="container"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#container").ejChart({
      * series :[{marker :{border :{width : 2}}}]                  
      * });
      * &lt;/script&gt;
* @alias ejChart#series->marker->border->width
*/
                            width: 3
                        },
                        /**		
       * Contains the customizing properties to add data labels of the points plotted in the chart.
       * @type {object}
        * @alias ejChart#series->marker->dataLabel
       */
                        dataLabel: {
                            /**		
* Specifies the visibility of dataLabel in the marker. The user can change the visibility of dataLabel by applying this property.
* @default false
* @type {boolean}			 
* @example 
* &lt;div id="container"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#container").ejChart({
      * series :[{marker :{dataLabel :{visible : true}}]                  
      * });
      * &lt;/script&gt; 
* @alias ejChart#series->marker->dataLabel->visible
*/
                            visible: false,
                            /**		
* Specifies the rotation angle of dataLabel in the marker. The user can change the rotation angle of dataLabel by applying this property.
* @default null
* @type {number}			 
* @example 
* &lt;div id="container"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#container").ejChart({
      * series :[{marker :{dataLabel :{angle : 90}}]                  
      * });
      * &lt;/script&gt; 
* @alias ejChart#series->marker->dataLabel->angle
*/
                            angle:0,
                            /**		
* This property is used to change the alignment of the data label vertically.
* @default ej.datavisualization.Chart.verticalTextAlignment.Center
* @type {enum}				 
* @example 
* &lt;div id="container"&gt;&lt;/div&gt; 
     * &lt;script&gt;
     * $("#container").ejChart({
     * series :[{marker :{dataLabel :{verticalTextAlignment : "far"}}}]                  
     * });
     * &lt;/script&gt;
* @alias ejChart#series->marker->dataLabel->verticalTextAlignment
*/
                            verticalTextAlignment: 'center',
                            /**		
* This property is used to change the alignment of the data label horizontally.
* @default ej.datavisualization.Chart.horizontalTextAlignment.Center
* @type {enum}			 
* @example 
* &lt;div id="container"&gt;&lt;/div&gt; 
    * &lt;script&gt;
    * $("#container").ejChart({
    * series :[{marker :{dataLabel :{horizontalTextAlignment : "far"}}}]                  
    * });
    * &lt;/script&gt;
* @alias ejChart#series->marker->dataLabel->horizontalTextAlignment
*/
                            horizontalTextAlignment: 'center',
                            /**		
* This property is used to change the position of the text in data label. See {@link TextPosition}
* @default ej.datavisualization.Chart.textPosition.Top
* @type {enum}			
* @example 
* &lt;div id="container"&gt;&lt;/div&gt; 
     * &lt;script&gt;
     * $("#container").ejChart({
     * series :[{marker :{dataLabel :{textPosition : "bottom"}}}]                 
     * });
     * &lt;/script&gt;
* @alias ejChart#series->marker->dataLabel->textPosition
*/
                            textPosition: 'top',
                            /**		
* Specifies the rendering shape of the marker. See {@link Shape}
* @default ej.datavisualization.Chart.Shape.None
* @type {enum}				
* @example 
* &lt;div id="container"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#container").ejChart({
      * series :[{marker :{dataLabel :{shape : "circle"}}}]                 
      * });
      * &lt;/script&gt;
* @alias ejChart#series->marker->dataLabel->shape
*/
                            shape: 'none',
                            /**		
* Specifies the opacity of dataLabel.
* @default 1
* @type {number}				 
* @example 
* &lt;div id="container"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#container").ejChart({
      * series :[{marker :{dataLabel :{opacity : 0.5}}}]                 
      * });
      * &lt;/script&gt;
* @alias ejChart#series->marker->dataLabel->opacity
*/
                            opacity: 1,
                            /**		
         * Specifies the fill color of dataLabel.
         * @default null
         * @type {string}				 
         * @example 
         * &lt;div id="container"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#container").ejChart({
      * series :[{marker :{dataLabel :{fill : "green"}}}]                 
      * });
      * &lt;/script&gt; 
         * @alias ejChart#series->marker->dataLabel->fill
         */
                            fill: null,
                            /**		
 * Options for customizing the color, opacity and width of the border of datalabel.
 * @type {object}
  * @alias ejChart#series->marker->dataLabel->border
 */
                            border: {
                                /**		
         * Specifies the border color of datalabel.
         * @default null
         * @type {string}				 
         * @example 
         * &lt;div id="container"&gt;&lt;/div&gt; 
     * &lt;script&gt;
     * $("#container").ejChart({
     * series :[{marker :{dataLabel :{border : {color : "green"}}}}]                 
     * });
     * &lt;/script&gt;
         * @alias ejChart#series->marker->dataLabel->border->color
         */
                                color: 'white',
                                /**		
         * Specifies the border width of dataLabel.
         * @default 0.1
         * @type {number}				 
         * @example 
         * &lt;div id="container"&gt;&lt;/div&gt; 
     * &lt;script&gt;
     * $("#container").ejChart({
     * series :[{marker :{dataLabel :{border :{ width :2 }}}}]                 
     * });
     * &lt;/script&gt; 
        * @alias ejChart#series->marker->dataLabel->border->width
         */
                                width: 0.1
                            },
                            /**		
     * Contains the options to customize the line connecting the point and the data label.
     * @type {object}
     * @alias ejChart#series->marker->dataLabel->connectorLine
     */
                            connectorLine: {
                                /**		
            * Specifies the width of the connector line.
            * @default 0.5
            * @type {number}				 
            * @example 
            * &lt;div id="container"&gt;&lt;/div&gt; 
     * &lt;script&gt;
     * $("#container").ejChart({
     * series :[{marker :{dataLabel :{connectorLine :{ width : 2 }}}}]                 
     * });
     * &lt;/script&gt; 
            * @alias ejChart#series->marker->dataLabel->connectorLine->width
            */
                                width: 0.5,
                                /**		
            * Specifies whether to connect the point and data label using bezier curve or straight line. See {@link Type}
            * @default ej.datavisualization.Chart.Type.Line
            * @type {enum}			 
            * @example 
           * &lt;div id="container"&gt;&lt;/div&gt; 
     * &lt;script&gt;
     * $("#container").ejChart({
     * series :[{marker :{dataLabel :{connectorLine :{ type : "spline" }}}}]                 
     * });
     * &lt;/script&gt;
            * @alias ejChart#series->marker->dataLabel->connectorLine->type
            */
                                type: 'line'
                            },
                            /**		
     * Specifies the font of the dataLabel. These are the properties involved in customizing the data label font.
     * @type {object}
      * @alias ejChart#series->marker->dataLabel->font 
     */
                            font: {
                                /**		
           * Specifies the fontFamily of dataLabel. Text in the data label render with the specified font family.
           * @default "Segoe UI"
           * @type {string}				 
           * @example 
          * &lt;div id="container"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#container").ejChart({
      * series :[{marker :{dataLabel :{ font :{fontFamily : "algerian"}}}}]                 
      * });
      * &lt;/script&gt; 
          * @alias ejChart#series->marker->dataLabel->font->fontFamily
           */
                                fontFamily: 'Segoe UI',
                                /**		
        * Specifies the fontStyle of dataLabel. Text in the data label render with the specified font style. See {@link FontStyle}
        * @default ej.datavisualization.Chart.FontStyle.Normal
        * @type {enum}			 
        * @example 
        * &lt;div id="container"&gt;&lt;/div&gt; 
    * &lt;script&gt;
    * $("#container").ejChart({
    * series :[{marker :{dataLabel :{font :{ fontStyle : "italic" }}}}]                 
    * });
    * &lt;/script&gt; 
        * @alias ejChart#series->marker->dataLabel->font->fontStyle
        */
                                fontStyle: 'Normal',
                                /**		
         * Specifies the fontWeight of dataLabel. Text in the data label render with the specified font weight. See {@link FontWeight}
         * @default ej.datavisualization.Chart.FontWeight.Regular
         * @type {enum}				 
         * @example 
         * &lt;div id="container"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#container").ejChart({
      * series :[{marker :{dataLabel :{font : { fontWeight : "lighter" }}}}]                 
      * });
      * &lt;/script&gt;
         * @alias ejChart#series->marker->dataLabel->font->fontWeight
         */
                                fontWeight: 'Regular',
                                /**		
        * Specifies the font size of dataLabel. Text in the data label render with the specified font size.
        * @default "12px"
        * @type {string}			 
        * @example 
        * &lt;div id="container"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#container").ejChart({
      * series :[{marker :{dataLabel :{font : { size : "14px" }}}}]                 
      * });
      * &lt;/script&gt; 
        * @alias ejChart#series->marker->dataLabel->font->size
        */
                                size: '12px',
                                /**		
       * Specifies the color of the text in dataLabel.  Text in the data label render with the specified font color.
       * @default "#707070"
       * @type {string}			 
       * @example 
      * &lt;div id="container"&gt;&lt;/div&gt; 
     * &lt;script&gt;
     * $("#container").ejChart({
     * series :[{marker :{dataLabel :{font :{ color : "green" }}}}]                 
     * });
     * &lt;/script&gt;; 
       * @alias ejChart#series->marker->dataLabel->font->color
       */
                                color: '#707070',
                                /**		
      * Specifies the opacity of the text in dataLabel. Text in the data label render with the specified opacity.
      * @default 1
      * @type {number}				 
      * @example 
      * &lt;div id="container"&gt;&lt;/div&gt; 
     * &lt;script&gt;
     * $("#container").ejChart({
     * series :[{marker :{dataLabel :{font :{ opacity : 0.5 }}}}]                 
     * });
     * &lt;/script&gt;
      * @alias ejChart#series->marker->dataLabel->font->opacity
      */
                                opacity: 1
                            },
                            /**		
     * Specifies the margin for the data label.
     * @type {object}
      * @alias ejChart#series->marker->dataLabel->margin
     */
                            margin: {
                                /**		
     * Specifies the left margin value in dataLabel. By specifying this property, we can adjust the text in data label to left.
     * @default 5
     * @type {number}				 
     * @example 
     * &lt;div id="container"&gt;&lt;/div&gt; 
     * &lt;script&gt;
     * $("#container").ejChart({
     * series :[{marker :{dataLabel :{margin :{ left : 10}}}}]                 
     * });
     * &lt;/script&gt; 
     * @alias ejChart#series->marker->dataLabel->margin->left
     */
                                left: 5,
                                /**		
     * Specifies the top margin value in dataLabel. By specifying this property, we can adjust the text in data label to top.
     * @default 5
     * @type {number}				 
     * @example 
     * &lt;div id="container"&gt;&lt;/div&gt; 
     * &lt;script&gt;
     * $("#container").ejChart({
     * series :[{marker :{dataLabel :{margin :{ top :10 } }}}]                 
     * });
     * &lt;/script&gt;
     * @alias ejChart#series->marker->dataLabel->margin->top
     */
                                top: 5,
                                /**		
      * Specifies the bottom margin value in dataLabel. By specifying this property, we can adjust the text in data label to bottom.
      * @default 5
      * @type {number}				
      * @example 
      * &lt;div id="container"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#container").ejChart({
      * series :[{marker :{dataLabel :{margin :{ bottom :10 }}}}]                 
      * });
      * &lt;/script&gt;
      * @alias ejChart#series->marker->dataLabel->margin->bottom
      */
                                bottom: 5,
                                /**		
     * Specifies the right margin value in dataLabel. By specifying this property, we can adjust the text in data label to right.
     * @default 5
     * @type {number}			
     * @example 
      * &lt;div id="container"&gt;&lt;/div&gt; 
     * &lt;script&gt;
     * $("#container").ejChart({
     * series :[{marker :{dataLabel :{margin :{ right :10 }}}}]                 
     * });
     * &lt;/script&gt;
     * @alias ejChart#series->marker->dataLabel->margin->right
     */
                                right: 5
                            }

                        }
                    },
                    /**		
      * Specifies the font that is common to all series. These are the properties involved in customizing the font of the series in chart. 
      * @type {object}
       * @alias ejChart#series->font
      */
                    font: {
                        /**		
     * Specifies the fontFamily, that is common to all series. Text in the series, render with the specified font family.
     * @default "Segoe UI"
     * @type {string}				 
     * @example 
      * &lt;div id="container"&gt;&lt;/div&gt; 
     * &lt;script&gt;
     * $("#container").ejChart({
     * series :[{ font : { fontFamily : "Algerian"}}]                 
     * });
     * &lt;/script&gt; 
     * @alias ejChart#series->font->fontFamily
     */
                        fontFamily: 'Segoe UI',
                        /**		
    * Specifies the fontStyle, that is common to all series. Text in the series, render with the specified font style.
    * @default ej.datavisualization.Chart.FontStyle.Normal
    * @type {enum}		
    * @example 
     * &lt;div id="container"&gt;&lt;/div&gt; 
    * &lt;script&gt;
    * $("#container").ejChart({
    * series : [{font :{fontStyle : "italic"}} ]                
    * });
    * &lt;/script&gt;
   * @alias ejChart#series->font->fontStyle
    */
                        fontStyle: 'Normal',
                        /**		
    * Specifies the fontWeight, that is common to all series. Text in the series, render with the specified font weight.
    * @default ej.datavisualization.Chart.FontWeight.Regular
    * @type {enum}				 
    * @example 
    * &lt;div id="container"&gt;&lt;/div&gt; 
     * &lt;script&gt;
     * $("#container").ejChart({
     * series :[{font :{fontWeight : "lighter"}}]                 
     * });
     * &lt;/script&gt;
    * @alias ejChart#series->font->fontWeight
    */
                        fontWeight: 'Regular',
                        /**		
    * Specifies the font size, that is common to all series. Text in the series, render with the specified font size.
    * @default "12px"
    * @type {string}				 
    * @example 
    * &lt;div id="container"&gt;&lt;/div&gt; 
     * &lt;script&gt;
     * $("#container").ejChart({
     * series :[{font :{size : "14px"}}]                 
     * });
     * &lt;/script&gt; 
    * @alias ejChart#series->font->size
    */
                        size: '12px',
                        /**		
   * Specifies the color, that is common to all series. Text in the series, render with the specified font color.
   * @default "#707070"
   * @type {string}				 
   * @example 
    * &lt;div id="container"&gt;&lt;/div&gt; 
     * &lt;script&gt;
     * $("#container").ejChart({
     * series :[{font :{color : "green"}}]                 
     * });
     * &lt;/script&gt;
   * @alias ejChart#series->font->color
   */
                        color: '#707070',
                        /**		
  * Specifies the opacity, that is common to all series. Text in the series, render with the specified font opacity.
  * @default 1
  * @type {number}
  * @example 
  * &lt;div id="container"&gt;&lt;/div&gt; 
    * &lt;script&gt;
    * $("#container").ejChart({
    * series :[{font :{opacity : 0.5}}]                 
    * });
    * &lt;/script&gt;
  * @alias ejChart#series->font->opacity
  */
                        opacity: 1
                    },
                    _isdesigntime:true

                }
            ],
            /**		
         * Options for configuring the properties commonly for all the series. You can also override the options for specific series by setting the options directly in each series object in series array.
         * @type {object}
         */
            commonSeriesOptions: /** @lends ejChart# */ {
                /**		
    * Specifies the type of series to be rendered in chart. See {@link Type}
    * @default ej.datavisualization.Chart.Type.Line
    * @type {enum}			 
    * @example 
    * &lt;div id="container"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#container").ejChart({
      * commonSeriesOptions :{ type : "spline"}                  
      * });
      * &lt;/script&gt; 
    * @alias ejChart#commonSeriesOptions->type
    */
                type:'column',
                /**		
   * Specifies the series animation common to all series. By specifying this, we can enable/disable the animation of the series.
   * @default true
   * @type {boolean}				 
   * @example 
   * &lt;div id="container"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#container").ejChart({
      * commonSeriesOptions :{ enableAnimation : false}                  
      * });
      * &lt;/script&gt; 
   * @alias ejChart#commonSeriesOptions->enableAnimation
   */
                enableAnimation: false,
                /**		
* Used in Polar and Radar series to specify whether the line/area series has to be rendered in a closed state by joining the start and end point of the series.
* @default true
* @type {boolean}				 
* @example 
* &lt;div id="container"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#container").ejChart({
      * commonSeriesOptions :{ isClosed : false}                  
      * });
      * &lt;/script&gt; 
* @alias ejChart#commonSeriesOptions->isClosed
*/
                isClosed: true,
                /**		
* Used in Polar and Radar series to specify whether column type has to be rendered the stacked value of chart.
* @default false
* @type {boolean}				 
* @example 
* &lt;div id="container"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#container").ejChart({
      * commonSeriesOptions :{ isStacking : "true"}                  
      * });
      * &lt;/script&gt; 
* @alias ejChart#commonSeriesOptions->isStacking
*/
                isStacking: false,
                /**		
* Specifies the type of series that can be drawn in a Radar or Polar series. It supports line, column and area type.  See {@link DrawType}
* @default ej.datavisualization.Chart.DrawType.Line
* @type {enum}				 
* @example 
* &lt;div id="container"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#container").ejChart({
      * commonSeriesOptions :{ drawType : "area"}                  
      * });
      * &lt;/script&gt; 
* @alias ejChart#commonSeriesOptions->drawType
*/
                drawType: 'line',
                /**		
  * Sets the radius of the hole on the doughnut series with respect to plot area. Value ranges from 0 to 1.
  * @default 0.4
  * @type {number}				 
  * @example 
 * &lt;div id="container"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#container").ejChart({
      * commonSeriesOptions :{ doughnutCoefficient : 0.5}                  
      * });
      * &lt;/script&gt; 
  * @alias ejChart#commonSeriesOptions->doughnutCoefficient
  */
                doughnutCoefficient: 0.4,
                /**		
   * Represents the distance the slice has to be moved out when it is exploded.
   * @default 0.4
   * @type {number}				
   * @example 
  * &lt;div id="container"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#container").ejChart({
      * commonSeriesOptions :{ explodeOffset : 20}                  
      * });
      * &lt;/script&gt; 
  * @alias ejChart#commonSeriesOptions->explodeOffset
   */
                explodeOffset: 25,
                /**		
 * Sets the rendering mode of the pyramid as linear or surface. See {@link PyramidMode}
 * @default ej.datavisualization.Chart.PyramidMode.Linear
 * @type {enum}				 
 * @example 
 * &lt;div id="container"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#container").ejChart({
      * commonSeriesOptions :{ pyramidMode : "linear"}                  
      * });
      * &lt;/script&gt; 
 * @alias ejChart#commonSeriesOptions->pyramidMode
 */
                pyramidMode: 'linear',
/**		
  * Sets the funnel width.
  * @default "11.6%"(11.6% of actualwidth)
  * @example 
  * &lt;div id="container"&gt;&lt;/div&gt; 
   * &lt;script&gt;
   * $("#container").ejChart({
   * series : [{funnelWidth : '40%' }]                   
   * });
   * &lt;/script&gt; 
   * @alias ejChart#commonSeriesOptions->funnelWidth
  */
                funnelWidth: '11.6%',
/**		
  * Sets the funnel height.
  * @default "32.7%"(32.7% of actualheight)
  * @example 
  * &lt;div id="container"&gt;&lt;/div&gt; 
   * &lt;script&gt;
   * $("#container").ejChart({
   * series : [{funnelHeight : '40%' }]                   
   * });
   * &lt;/script&gt; 
   * @alias ejChart#commonSeriesOptions->funnelHeight
  */
                funnelHeight: '32.7%',
                /**		
 * Specifies the position of the axis labels to render labels either inside or outside of plot area. See {@link LabelPosition}
 * @default ej.datavisualization.Chart.LabelPosition.Inside
 * @type {enum}			 
 * @example 
 * &lt;div id="container"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#container").ejChart({
      * commonSeriesOptions :{ labelPosition : "outside"}                  
      * });
      * &lt;/script&gt;  
 * @alias ejChart#commonSeriesOptions->labelPosition
 */
                labelPosition: 'inside',
                /**		
* Specifies the gap between each slice of a pyramid series in pixels.
* @default 0
* @type {number}			 
* @example 
* &lt;div id="container"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#container").ejChart({
      * commonSeriesOptions :{ gapRatio : 5}                  
      * });
      * &lt;/script&gt;  
* @alias ejChart#commonSeriesOptions->gapRatio
*/
                gapRatio: 0,
                /**		
* Sets the radius of the pie series with respect to plot area. Value ranges from 0 to 1.
* @default 0
* @type {number}			
* @example 
* &lt;div id="container"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#container").ejChart({
      * commonSeriesOptions :{ pieCoefficient : 1}                  
      * });
      * &lt;/script&gt;  
* @alias ejChart#commonSeriesOptions->pieCoefficient
*/
                pieCoefficient: 0.8,
                /**		
* Sets the radius of the doughnut series with respect to plot area. Value ranges from 0 to 1.
* @default 0.8
* @type {number}			 
* @example 
* &lt;div id="container"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#container").ejChart({
      * commonSeriesOptions :{ doughnutSize : 0.9}                  
      * });
      * &lt;/script&gt; 
* @alias ejChart#commonSeriesOptions->doughnutSize
*/
                doughnutSize: 0.8,
                /**		
* Specifies the dataSource for series. 
* @default null	
* @type {object}
* @example 
* &lt;div id="container"&gt;&lt;/div&gt; 
* &lt;script&gt;
* $("#container").ejChart({
* commonSeriesOptions : {dataSource: data.open }                   
* });
* &lt;/script&gt; 
* @alias ejChart#commonSeriesOptions->dataSource
*/
                dataSource: null,
                /**		
* Specifies the xName for dataSource in series.  This is used to take the x values from dataSource
* @default null	
* @type {string}
* @example 
* &lt;div id="container"&gt;&lt;/div&gt; 
* &lt;script&gt;
* $("#container").ejChart({
* commonSeriesOptions : {xName: "XValue" }                   
* });
* &lt;/script&gt; 
* @alias ejChart#commonSeriesOptions->xName
*/
                xName: '',
                /**		
* Specifies the yName for dataSource in series.  This is used to take the y values from dataSource
* @default null	
* @type {string}
* @example 
* &lt;div id="container"&gt;&lt;/div&gt; 
* &lt;script&gt;
* $("#container").ejChart({
* commonSeriesOptions :{yName: "XValue" }               
* });
* &lt;/script&gt; 
* @alias ejChart#commonSeriesOptions->yName
*/
				yName:'', 
                /**		
  * Specifies the starting angle for rendering pie / doughnut series.
  * @default null
  * @type {number}			 
  * @example 
  * &lt;div id="container"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#container").ejChart({
      * commonSeriesOptions :{ startAngle : 150}                  
      * });
      * &lt;/script&gt; ; 
  * @alias ejChart#commonSeriesOptions->startAngle
  */
                startAngle: null,
                /**		
* Specifies the ending angle for rendering pie / doughnut series.
* @default null
* @type {number}			 
* @example 
* &lt;div id="container"&gt;&lt;/div&gt; 
* &lt;script&gt;
* $("#container").ejChart({
* commonSeriesOptions :{ endAngle : 270}                  
* });
* &lt;/script&gt; ; 
* @alias ejChart#commonSeriesOptions->endAngle
*/
                endAngle:null,
                /**		
* Specifies the name of the xAxis.
* @default null
* @type {string}			
* @example 
* &lt;div id="container"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#container").ejChart({
      * commonSeriesOptions :{ xAxisName : "xAxis"}                  
      * });
      * &lt;/script&gt; 
* @alias ejChart#commonSeriesOptions->xAxisName
*/
                xAxisName: null,
                /**		
* Specifies the name of the yAxis. 
* @default null
* @type {string}			 
* @example 
* &lt;div id="container"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#container").ejChart({
      * commonSeriesOptions :{ yAxisName : "yAxis"}                  
      * });
      * &lt;/script&gt; 
* @alias ejChart#commonSeriesOptions->yAxisName
*/
                yAxisName: null,
                /**		
* Explodes all the pie slices. (Also works for doughnut series)
* @default false
* @type {boolean}			 
* @example 
* &lt;div id="container"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#container").ejChart({
      * commonSeriesOptions :{ explodeAll : true}                  
      * });
      * &lt;/script&gt; 
* @alias ejChart#commonSeriesOptions->explodeAll
*/
                explodeAll: null,
                /**		
* Index of a slice to be exploded from the whole pie. (Also works for doughnut series)
* @default null
* @type {number}			 
* @example 
* &lt;div id="container"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#container").ejChart({
      * commonSeriesOptions :{ explodeIndex : 2}                  
      * });
      * &lt;/script&gt; 
* @alias ejChart#commonSeriesOptions->explodeIndex
*/
                explodeIndex: null,
                /**		
* Enabling this property can avoid pie series data label overlap each other.
* @default true
* @type {boolean}				 
* @example 
* &lt;div id="container"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#container").ejChart({
      * commonSeriesOptions :{ enableSmartLabels : false}                  
      * });
      * &lt;/script&gt;  
* @alias ejChart#commonSeriesOptions->enableSmartLabels
*/
                enableSmartLabels: true,
                /**		
         * Contains all the properties to customize tooltip
         * @type {object}
         * @alias ejChart#commonSeriesOptions->tooltip
         */
                tooltip: {
                    /**		
* tooltip visibility can be toggled using this property
* @default false
* @type {boolean}				 
* @example 
* &lt;div id="container"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#container").ejChart({
      * commonSeriesOptions :{ tooltip :{visible : true} }                  
      * });
      * &lt;/script&gt; 
* @alias ejChart#commonSeriesOptions->tooltip->visible
*/
                    visible: false,
                    /**		
* Formatting the values of point on a tooltip can be customized using this property.
* @default "#point.x# : #point.y#"
* @type {string}			 
* @example 
* &lt;div id="container"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#container").ejChart({
      * commonSeriesOptions :{ format : "#point.x# : #point.y#%"}                  
      * });
      * &lt;/script&gt;  
* @alias ejChart#commonSeriesOptions->tooltip->format
*/
                    format: "#point.x# : #point.y#",
                    /**		
* Specifies the fill color of the tooltip.
* @default null
* @type {string}			 
* @example 
* &lt;div id="container"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#container").ejChart({
      * commonSeriesOptions :{tooltip :{fill : "green"}}                  
      * });
      * &lt;/script&gt;  
* @alias ejChart#commonSeriesOptions->tooltip->fill
*/
                    fill: null,
                    /**		
         * Options for customizing the color, opacity and width of the border of tooltip.
         * @type {object}
          * @alias ejChart#commonSeriesOptions->tooltip->border
         */
                    border: {
                        /**		
* Specifies the border color of tooltip.
* @default null
* @type {string}				 
* @example 
* &lt;div id="container"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#container").ejChart({
      * commonSeriesOptions :{tooltip :{border:{ color : "green" }}}                  
      * });
      * &lt;/script&gt;  
* @alias ejChart#commonSeriesOptions->tooltip->border->color
*/
                        color: null,
                        /**		
* Specifies the border width of tooltip.
* @default 1
* @type {number}			 
* @example 
* &lt;div id="container"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#container").ejChart({
      * commonSeriesOptions :{tooltip :{border :{ width : 2}}}                  
      * });
      * &lt;/script&gt;   
* @alias ejChart#commonSeriesOptions->tooltip->border->width
*/
                        width: 1
                    },
                    /**		
 * Enabling this property animates the tooltip on moving the cursor from one point to the other.
 * @default true
 * @type {boolean}			 
 * @example 
 * &lt;div id="container"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#container").ejChart({
      * commonSeriesOptions :{tooltip :{enableAnimation : false}}                  
      * });
      * &lt;/script&gt;  
 * @alias ejChart#commonSeriesOptions->tooltip->enableAnimation
 */
                    enableAnimation: true,
                    /**		
 * Specifies the duration of the tooltip to stay awake.
 * @default "500ms"
 * @type {string}				 
 * @example 
 * &lt;div id="container"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#container").ejChart({
      * commonSeriesOptions :{tooltip :{duration : "300ms"}}                  
      * });
      * &lt;/script&gt;   
 * @alias ejChart#commonSeriesOptions->tooltip->duration
 */
                    duration: '500ms',
                    /**		
 * Specifies the opacity of the tooltip. Values ranges from 0 to 1.
 * @default 0.5
 * @type {number}			
 * @example 
* &lt;div id="container"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#container").ejChart({
      * commonSeriesOptions :{tooltip :{opacity : 0.5}}                  
      * });
      * &lt;/script&gt;  
 * @alias ejChart#commonSeriesOptions->tooltip->opacity
 */
                    opacity: 0.95
                },
                /**		
         * Specifies the fill color of series.
         * @default null
         * @type {string}			 
         * @example 
        * &lt;div id="container"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#container").ejChart({
      * commonSeriesOptions :{fill : "green"}                  
      * });
      * &lt;/script&gt;
        * @alias ejChart#commonSeriesOptions->fill
         */
                fill: null,
                /**		
        * Specifies the opacity of series. The value ranges form 0 to 1.
        * @default 1
        * @type {number}				 
        * @example 
        * &lt;div id="container"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#container").ejChart({
      * commonSeriesOptions :{opacity : 0.5}                  
      * });
      * &lt;/script&gt; 
         * @alias ejChart#commonSeriesOptions->opacity
        */
                opacity: 1,
                /**		
       * Specifies the style to draw the end point of a line like butt, round, square. See {@link LineCap}
       * @default ej.datavisualization.Chart.LineCap.Butt
       * @type {enum}				 
       * @example 
       * &lt;div id="container"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#container").ejChart({
      * commonSeriesOptions :{lineCap : "butt"}                  
      * });
      * &lt;/script&gt;
        * @alias ejChart#commonSeriesOptions->lineCap
       */
                lineCap: 'butt',
                /**		
      * Specifies the shape to use at the intersection of two lines drawn.  See {@link LineJoin}
      * @default ej.datavisualization.Chart.LineJoin.Round
      * @type {enum}				 
      * @example 
      * &lt;div id="container"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#container").ejChart({
      * commonSeriesOptions :{lineCap : "round"}                  
      * });
      * &lt;/script&gt; 
       * @alias ejChart#commonSeriesOptions->lineJoin
      */
                lineJoin: 'round',
                /**		
      * Controls the pattern of dashes and gaps used to stroke the line series.
      * @default ""
      * @type {string}				 
      * @example 
     * &lt;div id="container"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#container").ejChart({
      * commonSeriesOptions :{dashArray : "2,3"}                  
      * });
      * &lt;/script&gt; 
       * @alias ejChart#commonSeriesOptions->dashArray
      */
                dashArray: "",
                /**		
         * Options for customizing the color, opacity and width of the border of series.
         * @type {object}
           * @alias ejChart#commonSeriesOptions->border
         */
                border: {
                    /**		
   * Specifies the border color of series.
   * @default "transparent"
   * @type {string}			 
   * @example 
  * &lt;div id="container"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#container").ejChart({
      * commonSeriesOptions :{border :{ color : "green" } }                  
      * });
      * &lt;/script&gt;
   * @alias ejChart#commonSeriesOptions->border->color
   */
                    color: 'transparent',
                    /**		
    * Specifies the border width of series.
    * @default 1
    * @type {number}			 
    * @example 
    * &lt;div id="container"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#container").ejChart({
      * commonSeriesOptions :{border :{ width : 2 } }                  
      * });
      * &lt;/script&gt;
     * @alias ejChart#commonSeriesOptions->border->width
    */
                    width: 1
                },
                width: 2,
                /**		
         * Contains the customizing properties to add marker symbols to the points plotted in the chart 
         * @type {object}
           * @alias ejChart#commonSeriesOptions->marker
         */
                marker:
                {
                    /**		
  * Specifies the shape of the marker. See {@link Shape}
  * @default ej.datavisualization.Chart.Shape.Circle
  * @type {enum}				
  * @example 
 * &lt;div id="container"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#container").ejChart({
      * commonSeriesOptions :{marker :{ shape: "rectangle"}                  
      * });
      * &lt;/script&gt;
   * @alias ejChart#commonSeriesOptions->marker->shape
  */
                    shape: 'circle',
                    /**		
         * Specifies the size of the marker.
         * @type {object}
          * @alias ejChart#commonSeriesOptions->marker->size
         */
                    size: {
                        /**		
  * Specifies the width of the marker.
  * @default 6
  * @type {number}				 
  * @example 
  * &lt;div id="container"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#container").ejChart({
      * commonSeriesOptions :{marker :{ size :{ width : 2 } } }                  
      * });
      * &lt;/script&gt;
  * @alias ejChart#commonSeriesOptions->marker->size->width
  */
                        width: 6,
                        /**		
* Specifies the height of the marker.
* @default 6
* @type {number}				 
* @example 
* &lt;div id="container"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#container").ejChart({
      * commonSeriesOptions :{marker :{size :{height : 5}}}                  
      * });
      * &lt;/script&gt;
* @alias ejChart#commonSeriesOptions->marker->size->height
*/
                        height: 6
                    },
                    /**		
* Toggles the visibility of the marker.
* @default false
* @type {boolean}			
* @example 
* &lt;div id="container"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#container").ejChart({
      * commonSeriesOptions :{marker :{ visible : true}}                  
      * });
      * &lt;/script&gt; 
* @alias ejChart#commonSeriesOptions->marker->visible
*/
                    visible: false,
                    /**		
* Specifies the opacity of the marker. Values ranges from 0 to 1.
* @default 1
* @type {number}			
* @example 
* &lt;div id="container"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#container").ejChart({
      * commonSeriesOptions :{marker :{ opacity : 0.5 }}                  
      * });
      * &lt;/script&gt;
* @alias ejChart#commonSeriesOptions->marker->opacity
*/
                    opacity: 1,
                    /**		
* Specifies the fill color of the marker.
* @default null
* @type {string}				 
* @example 
* &lt;div id="container"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#container").ejChart({
      * commonSeriesOptions :{marker : { fill : "green" } }                  
      * });
      * &lt;/script&gt;
* @alias ejChart#commonSeriesOptions->marker->fill
*/
                    fill: null,
                    /**		
       * Options for customizing the color, opacity and width of the border of marker.
       * @type {object}
        * @alias ejChart#commonSeriesOptions->marker->border
       */
                    border: {
                        /**		
                        * Specifies the border color of marker.
                        * @default "white"
                        * @type {string}				 
                        * @example 
                        * &lt;div id="container"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#container").ejChart({
      * commonSeriesOptions :{color : "green"}                  
      * });
      * &lt;/script&gt; 
                        * @alias ejChart#commonSeriesOptions->marker->border->color
                        */
                        color: 'white',
                        /**		
* Specifies the border width of marker.
* @default 3
* @type {number}				
* @example 
* &lt;div id="container"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#container").ejChart({
      * commonSeriesOptions :{marker :{border :{width : 2}}}                  
      * });
      * &lt;/script&gt;
* @alias ejChart#commonSeriesOptions->marker->border->width
*/
                        width: 3
                    },
                    /**		
       * Contains the customizing properties to add data labels of the points plotted in the chart.
       * @type {object}
        * @alias ejChart#commonSeriesOptions->marker->dataLabel
       */
                    dataLabel:
                    {
                        /**		
* Toggles the visibility of dataLabel in the marker.
* @default false
* @type {boolean}			 
* @example 
* &lt;div id="container"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#container").ejChart({
      * commonSeriesOptions :{marker :{dataLabel :{visible : true}}                  
      * });
      * &lt;/script&gt; 
* @alias ejChart#commonSeriesOptions->marker->dataLabel->visible
*/
                        visible: false,
                        /**		
 * rotates dataLabel in the marker.
* @default null
* @type {number}			 
* @example 
* &lt;div id="container"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#container").ejChart({
      * commonSeriesOptions :{marker :{dataLabel :{angle : 90}}                  
      * });
      * &lt;/script&gt; 
* @alias ejChart#commonSeriesOptions->marker->dataLabel->angle
*/
                        angle:0,
                        /**
* This property is used to change the alignment of the data label vertically.
* @default ej.datavisualization.Chart.verticalTextAlignment.Center
* @type {enum}				 
* @example 
* &lt;div id="container"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#container").ejChart({
      * commonSeriesOptions :{marker :{dataLabel :{verticalTextAlignment : "far"}}}                  
      * });
      * &lt;/script&gt;
* @alias ejChart#commonSeriesOptions->marker->dataLabel->verticalTextAlignment
*/
                        verticalTextAlignment: 'center',
                        /**		
* This property is used to change the alignment of the data label horizontally.
* @default ej.datavisualization.Chart.horizontalTextAlignment.Center
* @type {enum}			 
* @example 
* &lt;div id="container"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#container").ejChart({
      * commonSeriesOptions :{marker :{dataLabel :{horizontalTextAlignment : "far"}}}                  
      * });
      * &lt;/script&gt;
* @alias ejChart#commonSeriesOptions->marker->dataLabel->horizontalTextAlignment
*/
                        horizontalTextAlignment: 'center',
                        /**		
* This property is used to change the position of the text in data label. See {@link TextPosition}
* @default ej.datavisualization.Chart.textPosition.Top
* @type {enum}			
* @example 
* &lt;div id="container"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#container").ejChart({
      * commonSeriesOptions :{marker :{dataLabel :{textPosition : "bottom"}}}                 
      * });
      * &lt;/script&gt;
* @alias ejChart#commonSeriesOptions->marker->dataLabel->textPosition
*/
                        textPosition: 'top',
                        /**		
* Specifies the rendering shape of the marker. See {@link Shape}
* @default ej.datavisualization.Chart.Shape.None
* @type {enum}				
* @example 
* &lt;div id="container"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#container").ejChart({
      * commonSeriesOptions :{marker :{dataLabel :{shape : "circle"}}}                 
      * });
      * &lt;/script&gt;
* @alias ejChart#commonSeriesOptions->marker->dataLabel->shape
*/
                        shape: 'none',

                        /**		
* Specifies the opacity of dataLabel.
* @default 1
* @type {number}				 
* @example 
* &lt;div id="container"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#container").ejChart({
      * commonSeriesOptions :{marker :{dataLabel :{opacity : 0.5}}}                 
      * });
      * &lt;/script&gt;
* @alias ejChart#commonSeriesOptions->marker->dataLabel->opacity
*/
                        opacity: 1,
                        /**		
         * Specifies the fill color of datalabel.
         * @default null
         * @type {string}				 
         * @example 
         * &lt;div id="container"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#container").ejChart({
      * commonSeriesOptions :{marker :{dataLabel :{fill : "green"}}}                 
      * });
      * &lt;/script&gt; 
         * @alias ejChart#commonSeriesOptions->marker->dataLabel->fill
         */
                        fill: null,
                        /**		
 * Options for customizing the color, opacity and width of the border of datalabel.
 * @type {object}
  * @alias ejChart#commonSeriesOptions->marker->dataLabel->border
 */
                        border: {
                            /**		
          * Specifies the border color of datalabel.
          * @default null
          * @type {string}				 
          * @example 
          * &lt;div id="container"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#container").ejChart({
      * commonSeriesOptions :{marker :{dataLabel :{border : {color : "green"}}}}                 
      * });
      * &lt;/script&gt;
          * @alias ejChart#commonSeriesOptions->marker->dataLabel->border->color
          */
                            color: 'white',
                            /**		
          * Specifies the border width of datalabel.
          * @default 0.1
          * @type {number}				 
          * @example 
          * &lt;div id="container"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#container").ejChart({
      * commonSeriesOptions :{marker :{dataLabel :{border :{ width :2 }}}}                 
      * });
      * &lt;/script&gt; 
         * @alias ejChart#commonSeriesOptions->marker->dataLabel->border->width
          */
                            width: 0.1
                        },
                        /**		
      * Contains the options to customize the line connecting the point and the data label.
      * @type {object}
      * @alias ejChart#commonSeriesOptions->marker->dataLabel->connectorLine
      */
                        connectorLine:
                        {
                            /**		
             * Specifies the width of the connector line.
             * @default 0.5
             * @type {number}				 
             * @example 
             * &lt;div id="container"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#container").ejChart({
      * commonSeriesOptions :{marker :{dataLabel :{connectorLine :{ width : 2 }}}}                 
      * });
      * &lt;/script&gt; 
             * @alias ejChart#commonSeriesOptions->marker->dataLabel->connectorLine->width
             */
                            width: 0.5,
                            /**		
             * Specifies whether to connect the point and data label using bezier curve or straight line. See {@link Type}
             * @default ej.datavisualization.Chart.Type.Line
             * @type {enum}			 
             * @example 
            * &lt;div id="container"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#container").ejChart({
      * commonSeriesOptions :{marker :{dataLabel :{connectorLine :{ type : "spline" }}}}                 
      * });
      * &lt;/script&gt;
             * @alias ejChart#commonSeriesOptions->marker->dataLabel->connectorLine->type
             */
                            type: 'line'
                        },
                        /**		
      * Specifies the font of the dataLabel. These are the properties involved in customizing the data label font.
      * @type {object}
       * @alias ejChart#commonSeriesOptions->marker->dataLabel->font 
      */
                        font:
                        {
                            /**		
           * Specifies the fontFamily of dataLabel. Text in the data label render with the specified font family.
           * @default "Segoe UI"
           * @type {string}				 
           * @example 
          * &lt;div id="container"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#container").ejChart({
      * commonSeriesOptions :{marker :{dataLabel :{ font :{fontFamily : "algerian"}}}}                 
      * });
      * &lt;/script&gt; 
          * @alias ejChart#commonSeriesOptions->marker->dataLabel->font->fontFamily
           */
                            fontFamily: 'Segoe UI',
                            /**		
          * Specifies the fontStyle of dataLabel. Text in the data label render with the specified font style. See {@link FontStyle}
          * @default ej.datavisualization.Chart.FontStyle.Normal
          * @type {enum}			 
          * @example 
          * &lt;div id="container"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#container").ejChart({
      * commonSeriesOptions :{marker :{dataLabel :{font :{ fontStyle : "italic" }}}}                 
      * });
      * &lt;/script&gt; 
          * @alias ejChart#commonSeriesOptions->marker->dataLabel->font->fontStyle
          */
                            fontStyle: 'Normal',
                            /**		
         * Specifies the fontWeight of dataLabel. Text in the data label render with the specified font weight. See {@link FontWeight}
         * @default ej.datavisualization.Chart.FontWeight.Regular
         * @type {enum}				 
         * @example 
         * &lt;div id="container"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#container").ejChart({
      * commonSeriesOptions :{marker :{dataLabel :{font : { fontWeight : "lighter" }}}}                 
      * });
      * &lt;/script&gt;
         * @alias ejChart#commonSeriesOptions->marker->dataLabel->font->fontWeight
         */
                            fontWeight: 'Regular',
                            /**		
        * Specifies the font size of dataLabel. Text in the data label render with the specified font size.
        * @default "12px"
        * @type {string}			 
        * @example 
        * &lt;div id="container"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#container").ejChart({
      * commonSeriesOptions :{marker :{dataLabel :{font : { size : "14px" }}}}                 
      * });
      * &lt;/script&gt; 
        * @alias ejChart#commonSeriesOptions->marker->dataLabel->font->size
        */
                            size: '12px',
                            /**		
        * Specifies the color of the text in dataLabel.  Text in the data label render with the specified font color.
        * @default "#707070"
        * @type {string}			 
        * @example 
       * &lt;div id="container"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#container").ejChart({
      * commonSeriesOptions :{marker :{dataLabel :{font :{ color : "green" }}}}                 
      * });
      * &lt;/script&gt;; 
        * @alias ejChart#commonSeriesOptions->marker->dataLabel->font->color
        */
                            color: '#707070',
                            /**		
       * Specifies the opacity of the text in dataLabel. Text in the data label render with the specified opacity.
       * @default 1
       * @type {number}				 
       * @example 
       * &lt;div id="container"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#container").ejChart({
      * commonSeriesOptions :{marker :{dataLabel :{font :{ opacity : 0.5 }}}}                 
      * });
      * &lt;/script&gt;
       * @alias ejChart#commonSeriesOptions->marker->dataLabel->font->opacity
       */
                            opacity: 1
                        },
                        /**		
      * Specifies the margin for the data label.
      * @type {object}
       * @alias ejChart#commonSeriesOptions->marker->dataLabel->margin
      */
                        margin:
                        {
                            /**		
      * Specifies the left margin value for dataLabel. By specifying this property, we can adjust the text in data label to left.
      * @default 5
      * @type {number}				 
      * @example 
      * &lt;div id="container"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#container").ejChart({
      * commonSeriesOptions :{marker :{dataLabel :{margin :{ left : 10}}}}                 
      * });
      * &lt;/script&gt; 
      * @alias ejChart#commonSeriesOptions->marker->dataLabel->margin->left
      */
                            left: 5,
                            /**		
      * Specifies the top margin value for dataLabel. By specifying this property, we can adjust the text in data label to top.
      * @default 5
      * @type {number}				 
      * @example 
      * &lt;div id="container"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#container").ejChart({
      * commonSeriesOptions :{marker :{dataLabel :{margin :{ top :10 } }}}                 
      * });
      * &lt;/script&gt;
      * @alias ejChart#commonSeriesOptions->marker->dataLabel->margin->top
      */
                            top: 5,
                            /**		
      * Specifies the bottom margin value for dataLabel. By specifying this property, we can adjust the text in data label to bottom.
      * @default 5
      * @type {number}				
      * @example 
      * &lt;div id="container"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#container").ejChart({
      * commonSeriesOptions :{marker :{dataLabel :{margin :{ bottom :10 }}}}                 
      * });
      * &lt;/script&gt;
      * @alias ejChart#commonSeriesOptions->marker->dataLabel->margin->bottom
      */
                            bottom: 5,
                            /**		
      * Specifies the right margin value for dataLabel. By specifying this property, we can adjust the text in data label to right.
      * @default 5
      * @type {number}			
      * @example 
       * &lt;div id="container"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#container").ejChart({
      * commonSeriesOptions :{marker :{dataLabel :{margin :{ right :10 }}}}                 
      * });
      * &lt;/script&gt;
      * @alias ejChart#commonSeriesOptions->marker->dataLabel->margin->right
      */
                            right: 5
                        }
                    }
                },
                /**		
      * Specifies the font that is common to all series. These are the properties involved in customizing the font of the series in chart. 
      * @type {object}
       * @alias ejChart#commonSeriesOptions->font
      */
                font: {
                    /**		
      * Specifies the fontFamily, that is common to all series. Text in the series, render with the specified font family.
      * @default "Segoe UI"
      * @type {string}				 
      * @example 
       * &lt;div id="container"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#container").ejChart({
      * commonSeriesOptions :{ font : { fontFamily : "Algerian"}}                 
      * });
      * &lt;/script&gt; 
      * @alias ejChart#commonSeriesOptions->font->fontFamily
      */
                    fontFamily: 'Segoe UI',
                    /**		
      * Specifies the fontStyle, that is common to all series. Text in the series, render with the specified font style.
      * @default ej.datavisualization.Chart.FontStyle.Normal
      * @type {enum}		
      * @example 
       * &lt;div id="container"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#container").ejChart({
      * commonSeriesOptions : {font :{fontStyle : "italic"}}                 
      * });
      * &lt;/script&gt;
     * @alias ejChart#commonSeriesOptions->font->fontStyle
      */
                    fontStyle: 'Normal',
                    /**		
     * Specifies the fontWeight, that is common to all series. Text in the series, render with the specified font weight.
     * @default ej.datavisualization.Chart.FontWeight.Regular
     * @type {enum}				 
     * @example 
     * &lt;div id="container"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#container").ejChart({
      * commonSeriesOptions :{font :{fontWeight : "lighter"}}                 
      * });
      * &lt;/script&gt;
     * @alias ejChart#commonSeriesOptions->font->fontWeight
     */
                    fontWeight: 'Regular',
                    /**		
     * Specifies the font size, that is common to all series. Text in the series, render with the specified font size.
     * @default "12px"
     * @type {string}				 
     * @example 
     * &lt;div id="container"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#container").ejChart({
      * commonSeriesOptions :{font :{size : "14px"}}                 
      * });
      * &lt;/script&gt; 
     * @alias ejChart#commonSeriesOptions->font->size
     */
                    size: '12px',
                    /**		
    * Specifies the color, that is common to all series. Text in the series, render with the specified font color.
    * @default "#707070"
    * @type {string}				 
    * @example 
     * &lt;div id="container"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#container").ejChart({
      * commonSeriesOptions :{font :{color : "green"}}                 
      * });
      * &lt;/script&gt;
    * @alias ejChart#commonSeriesOptions->font->color
    */
                    color: '#707070',
                    /**		
    * Specifies the opacity, that is common to all series. Text in the series, render with the specified font opacity.
    * @default 1
    * @type {number}
	* @example 
    * &lt;div id="container"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#container").ejChart({
      * commonSeriesOptions :{font :{opacity : 0.5}}                 
      * });
      * &lt;/script&gt;
    * @alias ejChart#commonSeriesOptions->font->opacity
    */
                    opacity: 1
                }
            },
            /**		
     * Contains the properties to customize crosshair or trackball.
     * @type {object}
     */
            crosshair: /** @lends ejChart# */
            {
                /**		
     * Contains property to customize the marker in crosshair.
     * @type {object}
      * @alias ejChart#crosshair->marker
     */
                marker:
                {
                    /**		
    * Toggles the visibility of the marker in crosshair.
    * @default true
    * @type {boolean}			
    * @example 
     * &lt;div id="container"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#container").ejChart({
      * crosshair :{marker :{visible :false}}              
      * });
      * &lt;/script&gt;
     * @alias ejChart#crosshair->marker->visible
    */
                    visible: true,
                    /**		
     * Contains property to customize the size of marker.
     * @type {object}
       * @alias ejChart#crosshair->marker->size
     */
                    size:
                    {
                        /**		
   * Specifies the width of the marker in crosshair.
   * @default 10
   * @type {number}			 
   * @example 
    * &lt;div id="container"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#container").ejChart({
      * crosshair :{marker :{size : {width :15}}}              
      * });
      * &lt;/script&gt;
    * @alias ejChart#crosshair->marker->size->width
   */
                        width: 10,
                        /**		
 * Specifies the height of the marker in crosshair.
 * @default 10
 * @type {number}				 
 * @example 
  * &lt;div id="container"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#container").ejChart({
      * crosshair :{marker :{size :{ height :15 }}}              
      * });
      * &lt;/script&gt; 
 * @alias ejChart#crosshair->marker->size->height
 */
                        height: 10
                    },
                    /**		
 * Specifies the opacity of the marker in crosshair.
 * @default true
 * @type {boolean}			 
 * @example 
  * &lt;div id="container"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#container").ejChart({
      * crosshair :{marker :{opacity :2}}              
      * });
      * &lt;/script&gt; 
  * @alias ejChart#crosshair->marker->opacity
 */
                    opacity: 1,
                    /**		
     * Options for customizing the width of the border of marker.
     * @type {object}
       * @alias ejChart#crosshair->marker->border
     */
                    border:
                    {
                        /**		
      * Specifies the border width of the marker in crosshair.
      * @default 3
      * @type {number}			 
      * @example 
       * &lt;div id="container"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#container").ejChart({
      * crosshair :{marker :{border :{ width :2 }}}              
      * });
      * &lt;/script&gt;
       * @alias ejChart#crosshair->marker->border->width
      */
                        width: 3
                    }
                },
                /**		
      * Toggles the visibility of the crosshair / trackball in the plot area
      * @default false
      * @type {boolean}				
      * @example 
       * &lt;div id="container"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#container").ejChart({
      * crosshair :{visible :true}              
      * });
      * &lt;/script&gt;
       * @alias ejChart#crosshair->visible
      */
                visible: false,
                /**		
      * Specifies the type of the crosshair. It may be trackball or crosshair.
      * @default ej.datavisualization.Chart.CrosshairType.Crosshair
      * @type {enum}				 
      * @example 
       * &lt;div id="container"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#container").ejChart({
      * crosshair :{type : "trackball"}              
      * });
      * &lt;/script&gt; 
       * @alias ejChart#crosshair->type
      */
                type: "crosshair"
            },


            pointStyle:
            {
                lineCap: 'butt', lineJoin: 'round', opacity: 1, interior: null, borderColor: null, borderWidth: 1
            },
            textStyle:
            {
                marker: { textAlignment: 'center', textPosition: 'top' },
                font: { fontFamily: 'Segoe UI', fontStyle: 'Normal', fontWeight: 'Regular', size: '12px', color: '#707070', opacity: 1 }
            },
            symbolShape: {
                None: 0,
                LeftArrow: 1,
                RightArrow: 2,
                Circle: 3,
                Cross: 4,
                HorizLine: 5,
                VertLine: 6,
                Diamond: 7,
                Rectangle: 8,
                Triangle: 9,
                InvertedTriangle: 10,
                Hexagon: 11,
                Pentagon: 12,
                Star: 13,
                Ellipse: 14,
                Wedge: 15,
                Trapezoid: 16,
                UpArrow: 17,
                DownArrow: 18,
                Image: 19,
                SeriesType: 20

            },
            initSeriesRender: true,
            /**		
     * By specifying this property the user can change the theme of the chart. See {@link Theme}
     * @default ej.datavisualization.Chart.Theme.Flatlight
     * @type {enum}				 
     * @example 
     * &lt;div id="container"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#container").ejChart({
      * theme : "flatdark"            
      * });
      * &lt;/script&gt;
     */
            theme: "flatlight",
            /**		
    * Sets a value whether to make the chart responsive on resize.
    * @default false
    * @type {boolean}			 
    * @example 
     * &lt;div id="container"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#container").ejChart({
      * canResize : true             
      * });
      * &lt;/script&gt;
    */
            canResize: false,
			    /**		
    * To enable the 3D view of chart control.
    * @default false
    * @type {boolean}			 
    * @example 
     * &lt;div id="container"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#container").ejChart({
      * enable3D : true             
      * });
      * &lt;/script&gt;
    */
            enable3D: false,
			 /**		
    * Enables the 3D chart to perform rotation.
    * @default false
    * @type {boolean}			 
    * @example 
     * &lt;div id="container"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#container").ejChart({
      * enableRotation : true             
      * });
      * &lt;/script&gt;
    */
            enableRotation: false,
			 /**		
    * The sideBySideSeriesPlacement property defines the appearance of the different set of data on 3D chart
    * @default false
    * @type {boolean}			 
    * @example 
     * &lt;div id="container"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#container").ejChart({
      * sideBySideSeriesPlacement : true             
      * });
      * &lt;/script&gt;
    */
            sideBySideSeriesPlacement: false,
			 /**		
    * Enables the 3D chart to perform rotation.
    * @default 90
    * @type {number}			 
    * @example 
     * &lt;div id="container"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#container").ejChart({
      * perspectiveAngle : 60             
      * });
      * &lt;/script&gt;
    */
            perspectiveAngle: 90,   
/**		
     * The rotation property is used to spin the 3D chart.
    * @default 0
    * @type {number}			 
    * @example 
     * &lt;div id="container"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#container").ejChart({
      * rotation : 45             
      * });
      * &lt;/script&gt;
    */			
            rotation: 0,
			/**		
    * The tilt property defines the angle of the slope of 3D chart.
    * @default 0
    * @type {number}			 
    * @example 
     * &lt;div id="container"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#container").ejChart({
      * tilt : 5             
      * });
      * &lt;/script&gt;
    */		
            tilt: 0,
			/**		
    * In 3D, Cartesian axes lines are represented as wall. The property wallSize defines the width of the wall.
    * @default 2
    * @type {number}			 
    * @example 
     * &lt;div id="container"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#container").ejChart({
      * wallSize : 5             
      * });
      * &lt;/script&gt;
    */		
            wallSize: 2,
			/**		
    * The depth property defines the depth of the 3D chart from front view of the series to wall.
    * @default 100
    * @type {number}			 
    * @example 
     * &lt;div id="container"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#container").ejChart({
      * depth : 100            
      * });
      * &lt;/script&gt;
    */		
            depth:100,


            /**		
    * By enabling this property, the ejChart will render in Html5 canvas.
	* This canvas rendering supports all other functionalities except 3D and animation.
    * @default false
    * @type {boolean}			 
    * @example 
     * &lt;div id="container"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#container").ejChart({
      * enableCanvasRendering : true             
      * });
      * &lt;/script&gt;
    */
            enableCanvasRendering: false,
            /**		
     * Contains property to customize zooming in chart.
     * @type {object}
     */
            zooming: /** @lends ejChart# */
            {
                /**		
   * Enabling this property will allow zooming for chart plot area.
   * @default false
   * @type {boolean}				 
   * @example 
     * &lt;div id="container"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#container").ejChart({
      * zooming :{enable :true}          
      * });
      * &lt;/script&gt;
    * @alias ejChart#zooming->enable
   */
                enable: false,
                /**		
   * Specifies whether zooming has to be vertical or horizontal.
   * @default 'x,y'
   * @type {string}			 
   * @example 
   * &lt;div id="container"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#container").ejChart({
      * zooming :{type : "y"}            
      * });
      * &lt;/script&gt; 
   * @alias ejChart#zooming->type
   */
                type: 'x,y',
                /**		
   * Enabling this property will allow user to zoom plot are on rolling mouse wheel.
   * @default false
   * @type {boolean}			 
   * @example 
    * &lt;div id="container"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#container").ejChart({
      * zooming:{enableMouseWheel : true}            
      * });
      * &lt;/script&gt; 
   * @alias ejChart#zooming->enableMouseWheel
   */
                enableMouseWheel: false,
                /**		
 * Enabling this property will allow user to pan the zoomed chart on mouse up.
 * @default false
 * @type {boolean}			 
 * @example 
  * &lt;div id="container"&gt;&lt;/div&gt; 
    * &lt;script&gt;
    * $("#container").ejChart({
    * zooming:{enableDeferredZoom : true}            
    * });
    * &lt;/script&gt; 
    * @alias ejChart#zooming->enableDeferredZoom
 */
          enableDeferredZoom : false
            },
            type: 'x,y',
            /**		
* This property is to specify the localization of chart.
* @default "en-US"
* @type {string}				 
* @example 
 * &lt;div id="container"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#container").ejChart({
      * locale : "en-US"            
      * });
      * &lt;/script&gt;
*/
            locale: "en-US",

            xZoomFactor: 1,

            yZoomFactor: 1,

            xZoomPosition: 0,

            yZoomPosition: 0,
            /**     
              * Fires on load of chart.
              * @event
              * @name ejChart#load	
              * @param {Object} argument.Data parameters from chart     
			 * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
			 * @param {object}  argument.model returns the chart model
			 * @param {string}  argument.type returns the name of the event
              * @example 
              * //load event for chart
              * $("#container").ejChart({
              *    load: function (args) {}
              * });
              */
            load: "",
            /**     
              * Fires on rendering the axis labels.
              * @event
              * @name ejChart#axesLabelRendering
              * @param {Object} argument.Axis parameters from chart   
              * @param {Object} argument.Label parameters from chart for processing of labels
			  * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
			  * @param {object}  argument.model returns the chart model
			  * @param {string}  argument.type returns the name of the event
              * @example 
              * //axesLabelRendering event for chart
              * $("#container").ejChart({
              *    axesLabelRendering: function (args) {}
              * });
              
              */
            axesLabelRendering: "",
            /**     
             * Fires on axes range calculation.
             * @event
             * @name ejChart#axesRangeCalculate	
             * @param {Object} argument.Data parameters from chart     
			 * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
			 * @param {object}  argument.model returns the chart model
			 * @param {string}  argument.type returns the name of the event
             * @example 
             * //axesRangeCalculate event for chart
             * $("#container").ejChart({
             *    axesRangeCalculate: function (args) {}
             * });
             
             */
            axesRangeCalculate: "",
            /**     
             * Fires on rendering the axes title.
             * @event
             * @name ejChart#axesTitleRendering	
             * @param {Object} argument.Data parameters from chart     
			 * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
			 * @param {object}  argument.model returns the chart model
			 * @param {string}  argument.type returns the name of the event
             * @example 
             * //axesTitleRendering event for chart
             * $("#container").ejChart({
             *    axesTitleRendering: function (args) {}
             * });
             
             */
            axesTitleRendering: "",
            /**     
             * Fires on chart area bounds calculation.
             * @event
             * @name ejChart#chartAreaBoundsCalculate	
             * @param {Object} argument.Data parameters from chart     
			 * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
			 * @param {object}  argument.model returns the chart model
			 * @param {string}  argument.type returns the name of the event
             * @example 
             * //chartAreaBoundsCalculate event for chart
             * $("#container").ejChart({
             *    chartAreaBoundsCalculate: function (args) {}
             * });
             
             */
            chartAreaBoundsCalculate: "",
            /**     
             * Fires on rendering of legend item.
             * @event
             * @name ejChart#legendItemRendering
             * @param {Object} argument.Data parameters from chart     
			 * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
			 * @param {object}  argument.model returns the chart model
			 * @param {string}  argument.type returns the name of the event
             * @example 
             * //legendItemRendering event for chart
             * $("#container").ejChart({
             *    legendItemRendering: function (args) {}
             * });
             
             */
            legendItemRendering: "",
            /**     
             * Fires on calculating the legend bounds
             * @event
             * @name ejChart#legendBoundsCalculate
             * @param {Object} argument.Data parameters from chart     
			 * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
			 * @param {object}  argument.model returns the chart model
			 * @param {string}  argument.type returns the name of the event
             * @example 
             * //legendBoundsCalculate event for chart
             * $("#container").ejChart({
             *    legendBoundsCalculate: function (args) {}
             * });
             
             */
            legendBoundsCalculate: "",
            /**     
             * Fires on pre render.
             * @event
             * @name ejChart#preRender	
             * @param {Object} argument.Data parameters from chart     
			 * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
			 * @param {object}  argument.model returns the chart model
			 * @param {string}  argument.type returns the name of the event
             * @example 
             * //preRender event for chart
             * $("#container").ejChart({
             *    preRender: function (args) {}
             * });
             
             */
            preRender: "",
            /**     
             * Fires on rendering the series.
             * @event
             * @name ejChart#seriesRendering
             * @param {Object} argument.Data parameters from chart     
			 * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
			 * @param {object}  argument.model returns the chart model
			 * @param {string}  argument.type returns the name of the event
             * @example 
             * //seriesRendering event for chart
             * $("#container").ejChart({
             *    seriesRendering: function (args) {}
             * });
             
             */
            seriesRendering: "",
            /**     
             * Fires on rendering the symbols.
             * @event
             * @name ejChart#symbolRendering
             * @param {Object} argument.Data parameters from chart     
			 * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
			 * @param {object}  argument.model returns the chart model
			 * @param {string}  argument.type returns the name of the event
             * @example 
             * //symbolRendering event for chart
             * $("#container").ejChart({
             *    symbolRendering: function (args) {}
             * });
             
             */
            symbolRendering: "",
            /**     
             * Fires on rendering the title.
             * @event
             * @name ejChart#titleRendering	
             * @param {Object} argument.Data parameters from chart     
			 * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
			 * @param {object}  argument.model returns the chart model
			 * @param {string}  argument.type returns the name of the event
             * @example 
             * //titleRendering event for chart
             * $("#container").ejChart({
             *    titleRendering: function (args) {}
             * });
             
             */
            titleRendering: "",
           subtitleRendering: "",
           
              
             
            /**     
             * Fires on initializing the axis labels.
             * @event
             * @name ejChart#axesLabelsInitialize
             * @param {Object} argument.Data parameters from chart     
			 * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
			 * @param {object}  argument.model returns the chart model
			 * @param {string}  argument.type returns the name of the event
             * @example 
             * //axesLabelsInitialize event for chart
             * $("#container").ejChart({
             *    axesLabelsInitialize: function (args) {}
             * });
             
             */
            axesLabelsInitialize: "",
            /**     
             * Fires on click of point region.
             * @event
             * @name ejChart#pointRegionClick	
             * @param {Object} argument.Data parameters from chart     
			 * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
			 * @param {object}  argument.model returns the chart model
			 * @param {string}  argument.type returns the name of the event
             * @example 
             * //pointRegionClick event for chart
             * $("#container").ejChart({
             *    pointRegionClick: function (args) {}
             * });
             
             */
            pointRegionClick: "",
            /**     
             * Fires on mousemove over the point region.
             * @event
             * @name ejChart#pointRegionMouseMove	
             * @param {Object} argument.Data parameters from chart     
			 * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
			 * @param {object}  argument.model returns the chart model
			 * @param {string}  argument.type returns the name of the event
             * @example 
             * //pointRegionMouseMove event for chart
             * $("#container").ejChart({
             *    pointRegionMouseMove: function (args) {}
             * });
             
             */
            pointRegionMouseMove: "",
            /**     
             * Fires on click of legend item
             * @event
             * @name ejChart#legendItemClick	
             * @param {Object} argument.Data parameters from chart     
			 * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
			 * @param {object}  argument.model returns the chart model
			 * @param {string}  argument.type returns the name of the event
             * @example 
             * //legendItemClick event for chart
             * $("#container").ejChart({
             *    legendItemClick: function (args) {}
             * });
             
             */
            legendItemClick: "",
            /**     
             * Fires on mouse move over legend item.
             * @event
             * @name ejChart#legendItemMouseMove	
             * @param {Object} argument.Data parameters from chart     
			 * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
			 * @param {object}  argument.model returns the chart model
			 * @param {string}  argument.type returns the name of the event
             * @example 
             * //legendItemMouseMove event for chart
             * $("#container").ejChart({
             *    legendItemMouseMove: function (args) {}
             * });
             
             */
            legendItemMouseMove: "",
            /**     
             * Fires on rendering the display text.
             * @event
             * @name ejChart#displayTextRendering		
             * @param {Object} argument.Data parameters from chart     
			 * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
			 * @param {object}  argument.model returns the chart model
			 * @param {string}  argument.type returns the name of the event
             * @example 
             * //displayTextRendering event for chart
             * $("#container").ejChart({
             *    displayTextRendering: function (args) {}
             * });
             
             */
            displayTextRendering: "",
            /**     
             * Fires on initialize the tooltip.
             * @event
             * @name ejChart#toolTipInitialize	
             * @param {Object} argument.Data parameters from chart     
			 * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
			 * @param {object}  argument.model returns the chart model
			 * @param {string}  argument.type returns the name of the event
             * @example 
             * //toolTipInitialize event for chart
             * $("#container").ejChart({
             *    toolTipInitialize: function (args) {}
             * });
             
             */
            toolTipInitialize: "",
            /**     
             * Fires on track the axis tooltip.
             * @event
             * @name ejChart#trackAxisToolTip	
             * @param {Object} argument.Data parameters from chart     
			 * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
			 * @param {object}  argument.model returns the chart model
			 * @param {string}  argument.type returns the name of the event
             * @example 
             * //trackAxisToolTip event for chart
             * $("#container").ejChart({
             *    trackAxisToolTip: function (args) {}
             * });
             
             */
            trackAxisToolTip: "",
            /**     
             * Fires on tracking the tooltip.
             * @event
             * @name ejChart#trackToolTip	
             * @param {Object} argument.Data parameters from chart     
			 * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
			 * @param {object}  argument.model returns the chart model
			 * @param {string}  argument.type returns the name of the event
             * @example 
             * //trackToolTip event for chart
             * $("#container").ejChart({
             *    trackToolTip: function (args) {}
             * });
             
             */
            trackToolTip: "",
            /**     
             * Fires on completion of animation.
             * @event
             * @name ejChart#animationComplete
             * @param {Object} argument.Data parameters from chart     
			 * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
			 * @param {object}  argument.model returns the chart model
			 * @param {string}  argument.type returns the name of the event
             * @example 
             * //animationComplete event for chart
             * $("#container").ejChart({
             *    animationComplete: function (args) {}
             * });
             
             */
            animationComplete: "",
            /**     
             * Fires when chart is destroyed completely.
             * @event
             * @name ejChart#destroy
             * @example 
             * //animationComplete event for chart
             * $("#container").ejChart("destroy");
             
             */
            destroy: "",
            /**     
            * Fires after chart is created.
            * @event
            * @name ejChart#create		
            * @example 
            * //create event for chart
            * $("#container").ejChart("create");
            
            */
            create: "",

            margin: { left: 10, right: 10, top: 10, bottom: 10 },
            size: {},
            elementSpacing: 10,

            seriesColors: [],
            seriesBorderColors: [],
            dateStart: 25568.791666666668,
            pointColors: [],
            pointBorderColors: []
        },
        /**
          * Specify the data types for default properties 
          * @private
          */
        dataTypes: {
            commonSeriesOptions:{dataSource:"data"},
            axes: "array",
            series: "array",
            seriesColors: "array",
            palette: "array",
            rowDefinitions: "array",
            columnDefinitions: "array",
            seriesBorderColors: "array",
            pointColors: "array",
            pointBorderColors: "array",
            initSeriesRender: "boolean",
            theme: "enum",
            canResize: "boolean",
            elementSpacing: "number",
            primaryXAxis: {
                labels: "array",
                stripLine: "array",
                orientation: "enum",
                rangePadding: "enum",
                tickLinesPosition: "enum",
                labelPosition:"enum",
                opposedPosition: "boolean",
                zoomFactor: "number",
                zoomPosition: "number"
            },
            primaryYAxis: {
                labels: "array",
                stripLine: "array",
                orientation: "enum",
                rangePadding: "enum",

                opposedPosition: "boolean",
                zoomFactor: "number",
                zoomPosition: "number"
            },
            legend: {

                shape: "enum",
                alignment: "enum",
                position: "enum",
                itemPadding: "number"
            },
            zooming: {
                enable: "boolean",
                type: "string",
                enableMouseWheel: "boolean"
            },
            size: {
                width: "string",
                height: "string"
            }

        },

        observables: ["xZoomFactor", "yZoomFactor", "xZoomPosition", "yZoomPosition"],
        _xZoomFactor: ej.util.valueFunction("xZoomFactor"),
        _yZoomFactor: ej.util.valueFunction("yZoomFactor"),
        _xZoomPosition: ej.util.valueFunction("xZoomPosition"),
        _yZoomPosition: ej.util.valueFunction("yZoomPosition"),

        /**
        * Create the chart widget
        * @private
        */
        _init: function () {

            this._renderSfChart();

        },
        _destroy: function () {
            $(this.element).removeClass("e-chart e-js").find("#" + this.svgObject.id).remove();
        },
        /**
        * redraw the chart widget
		* @return jQuery
		* @example 
		* &lt;div id="container"&gt;&lt;/div&gt; <br> 
		* &lt;script&gt;
		* // Redraw Chart
		* var chartObj = $("#container").data("ejChart");
		* chartObj.redraw(); // redraw the chart
		* &lt;/script&gt;
		* @example 
		* &lt;div id="container"&gt;&lt;/div&gt; <br> 
		* &lt;script&gt;
		* $("#container").ejChart("redraw");	
		* &lt;/script&gt;
		
         */
        redraw: function (excludeDataUpdate) {
            for (var i = 0; i < this.model.axes.length; i++){
                this.model.axes[i].desiredIntervals = null;
                this.model.axes[i]._valueType = null;
                this.model.axes[i].position = null;
            } 
            $('#template_group_' + this._id).remove();
			for (var i = 0; i < this.model.series.length; i++) {
                this.model.series[i].position = null;
            }
			$('#template_group_' + this._id).remove();
            if(this.model.enable3D)
            ej.Ej3DRender.transform = null;
            
			
            if (this.model.enableCanvasRendering) {
                if (this.model.zooming.enable && this.svgRenderer.ctx)
                    this.svgRenderer.ctx.clearRect(0, 0, ej.util.isNullOrUndefined(this.model.size.width) ? $("#" + this._id).width() : this.model.size.width, ej.util.isNullOrUndefined(this.model.size.height) ? "450" : this.model.size.height);
                else {
                    $(this.svgObject).remove();
					var width=this.svgWidth;
                    $("#canvas_trackSymbol").remove();
                    this.svgRenderer = new ej.EjCanvasRender(this.element); // to create canvas container
                    this.svgRenderer.svgObj.height = ej.util.isNullOrUndefined(this.model.size.height) ? "450" : this.model.size.height;
                    this.svgRenderer.svgObj.width = ej.util.isNullOrUndefined(this.model.size.width) ? width : this.model.size.width;
                    this.svgObject = this.svgRenderer.svgObj;
                    var ctx = this.svgRenderer.svgObj.getContext("2d");

                    $(this.legendSvgContainer).remove();
                    $(this.legendContainer).remove();
                    this.legendContainer = $("<div></div>").attr('id', "legend_" + this._id).css("height", "0px");
                    this.legendSvgContainer = this.svgRenderer.createLegendCanvas(this.element); //creating new CANVAS element for legend
                    this.svgRenderer.append(this.legendSvgContainer, this.legendContainer);
                    $(this.legendContainer).appendTo(this.chartContainer);
                }
            }
            else if ((this.svgObject.id).indexOf("_canvas") != -1) {
                $(this.svgObject).remove();
                this.svgRenderer = new ej.EjSvgRender(this.element);  // to create svg container
                this.svgObject = this.svgRenderer.svgObj;
                this.canvasX = this.canvasY = 0;

                $(this.legendSvgContainer).remove();
				$(this.legendContainer).remove();
                this.legendContainer = $("<div></div>").attr('id', "legend_" + this._id).css("height", "0px");
                this.legendSvgContainer = this.svgRenderer.createLegendSvg(this.element); //creating new SVG element for legend
                this.svgRenderer.append(this.legendSvgContainer, this.legendContainer);
				$(this.legendContainer).appendTo(this.chartContainer);
            } else {
                $(this.svgObject).empty();              // using existing svg container			   
                $(this.legendSvgContainer).empty();
            }
			
            this.bindTo(excludeDataUpdate);
        },
		
		 /**
        * export the chart widget
		* @return jQuery
		* @example 
		* &lt;div id="container"&gt;&lt;/div&gt; <br> 
		* &lt;script&gt;
		* // Export Chart
		* var canvas = $("#container").ejChart("exportChart"); 
        * var dt = canvas.toDataURL();
        * this.href = dt;
		* &lt;/script&gt;
         */
		exportChart: function (x)
        {         
             var Id = this._id;
            if ( $("#Export_"+Id).length <= 0) {

            var Id = this._id;
            var chart = document.getElementById(Id + '_canvas');
            var legend = document.getElementById('legend_' + Id + '_canvas');

            var exportChart = document.createElement('canvas');            
            exportChart.setAttribute('id', 'Export_'+Id);
            exportChart.setAttribute('width', this.svgWidth);
            exportChart.setAttribute('height', this.svgHeight);

            //$("body").append(exportChart);
            var exprtCtx = exportChart.getContext("2d");
            
            var legendPos = this.model.LegendBounds;
            exprtCtx.drawImage(chart, 0, 0);
			
			if(legendPos)
            exprtCtx.drawImage(legend, legendPos.X, legendPos.Y);
            //$("#Export_" +Id).css('display', 'none');
          }
            return exportChart;
        },
		
        /**
                 * To configure the properties at runtime using SetModel		
                 * @private
                 */
        _setModel: function (options) {
            for (var prop in options) {
                this.disableAnimation();
                switch (prop) {
                    case "theme":
                        this.model._themeChanged = true;
                        this.model.theme = options[prop];
                        this.setTheme(ej.EjSvgRender.themes, this.model.theme);
                        $.extend(true, this.model.axes[0], ej.EjSvgRender.themes[this.model.theme].primaryXAxis);
                        $.extend(true, this.model.axes[1], ej.EjSvgRender.themes[this.model.theme].primaryYAxis);
                        for (var k = 2; k < this.model.axes.length; k++) {
                            if (this.model.axes[k].orientation.toLowerCase() == "horizontal")
                                $.extend(true, this.model.axes[k], ej.EjSvgRender.themes[this.model.theme].secondaryX);
                            else
                                $.extend(true, this.model.axes[k], ej.EjSvgRender.themes[this.model.theme].secondaryY);
                        }
                        break;
                    case "commonSeriesOptions":
                        for (var i = 0; i < this.model.series.length; i++)
                            $.extend(true, this.model.series[i], {}, options[prop]);
                        break;
                    case "series":
                        $.extend(true, this.model.series, {}, options[prop]);
                        break;
                    case "legend":
                        $.extend(true, this.model.legend, {}, options[prop]);
                        break;
                    case "axes":
                        $.extend(true, this.model.axes, {}, options[prop]);
                        break;
                    case "primaryXAxis":
                        this.model.axes[0].setRange = (options[prop].range) ? true : false;
                        this.model.axes[0].actual_Range = (this.model.axes[0].setRange) ? null : this.model.axes[0].actual_Range;
                        $.extend(true, this.model.axes[0], {}, options[prop]);
                        break;
                    case "primaryYAxis":
                        this.model.axes[1].setRange = (options[prop].range) ? true : false;
                        this.model.axes[1].actual_Range = (this.model.axes[1].setRange) ? null : this.model.axes[1].actual_Range;
                        $.extend(true, this.model.axes[1], {}, options[prop]);
                        break;
                    case "xZoomFactor":
                        this.model.axes[0].zoomFactor = this._xZoomFactor();
                        break;
                    case "yZoomFactor":
                        this.model.axes[1].zoomFactor = this._yZoomFactor();
                        break;
                    case "xZoomPosition":
                        this.model.axes[0].zoomPosition = this._xZoomPosition();
                        break;
                    case "yZoomPosition":
                        this.model.axes[1].zoomPosition = this._yZoomPosition();
                        break;
                    case "drilldown":
                        this.model.series = [];
                        this.model.explodeValue = null;
                        $.extend(true, this.model, {}, options[prop]);
                        break;
                    case "datasource":
                        for (var i = 0; i < this.model.series.length; i++) {
                            this.model.series[i].dataSource = null;
                            this.model.series[i].points = null;
                        }
                        $.extend(true, this.model, {}, options[prop]);
                        break;

                    default:
                        $.extend(true, this.model, {}, options[prop]);
                }
            }

            if (this.model.enableCanvasRendering) {
                $(this.svgObject).remove();
                $("#canvas_trackSymbol").remove();
                this.svgRenderer = new ej.EjCanvasRender(this.element); // to create canvas container
                this.svgRenderer.svgObj.height = ej.util.isNullOrUndefined(this.model.size.height) ? "450" : this.model.size.height;
                this.svgRenderer.svgObj.width = ej.util.isNullOrUndefined(this.model.size.width) ? $("#" + this._id).width() : this.model.size.width;
                this.svgObject = this.svgRenderer.svgObj;

                $(this.legendSvgContainer).remove();
                this.legendSvgContainer = this.svgRenderer.createLegendCanvas(this.element); //creating new CANVAS element for legend
                this.svgRenderer.append(this.legendSvgContainer, this.legendContainer);
            }
            else if ((this.svgObject.id).indexOf("_canvas") != -1) {
                $(this.svgObject).remove();
                this.svgRenderer = new ej.EjSvgRender(this.element);  // to create svg container
                this.svgObject = this.svgRenderer.svgObj;
                this.canvasX = this.canvasY = 0;

                $(this.legendSvgContainer).remove();
                this.legendSvgContainer = this.svgRenderer.createLegendSvg(this.element); //creating new SVG element for legend
                this.svgRenderer.append(this.legendSvgContainer, this.legendContainer);
            }
            else {
                $(this.svgObject).empty();              // using existing svg container
                $(this.legendSvgContainer).empty();
            }
            
            this.bindTo();
        }
    });

    /**
	 * Enum for chart crosshairType.	 
	 * @enum {string}
	 * @global 
	 */
    ej.datavisualization.Chart.CrosshairType = {
        /**  set crosshair for chart. */
        Crosshair: 'crosshair',
        /**  set trackBall for chart. */
        TrackBall: 'trackBall'
    };
    /**
	 * Enum for chart valueType.	 
	 * @enum {string}
	 * @global 
	 */
    ej.datavisualization.Chart.ValueType = {
        /**  set valueType for chart to double. */
        Double: 'double',
        /**  set valueType for chart to dateTime. */
        DateTime: 'datetime',
        /**  set valueType for chart to category. */
        Category: 'category',
        /**  set valueType for chart to logarithmic. */
        Logarithmic: 'logarithmic'
    };
    /**
	 * Enum for chart series type.	 
	 * @enum {string}
	 * @global 
	 */
    ej.datavisualization.Chart.Type = {
        /**  sets the seriesType of chart to line. */
        Line: 'line',
        /**  sets the seriesType of chart to spline. */
        Spline: 'spline',
        /**  sets the seriesType of chart to column. */
        Column: 'column',
        /**  sets the seriesType of chart to doughnut. */
        Doughnut: 'doughnut',
        /**  sets the seriesType of chart to area. */
        Area: 'area',
        /**  sets the seriesType of chart to splinearea. */
        SplineArea: 'splinearea',
        /**  sets the seriesType of chart to stepline. */
        StepLine: 'stepline',
        /**  sets the seriesType of chart to steparea. */
        StepArea: 'steparea',
        /**  sets the seriesType of chart to pie. */
        Pie: 'pie',
        /**  sets the seriesType of chart to hilo. */
        Hilo: 'hilo',
        /**  sets the seriesType of chart to hiloopenclose. */
        HiloOpenClose: 'hiloopenclose',
        /**  sets the seriesType of chart to candle. */
        Candle: 'candle',
        /**  sets the seriesType of chart to bubble. */
        Bubble: 'bubble',
        /**  sets the seriesType of chart to scatter. */
        Scatter: 'scatter',
        /**  sets the seriesType of chart to bar. */
        Bar: 'bar',
        /**  sets the seriesType of chart to stackingarea. */
        StackingArea: 'stackingarea',
        /**  sets the seriesType of chart to rangecolumn. */
        RangeColumn: 'rangecolumn',
        /**  sets the seriesType of chart to stackingcolumn. */
        StackingColumn: 'stackingcolumn',
        /**  sets the seriesType of chart to stackingbar. */
        StackingBar: 'stackingbar',
        /**  sets the seriesType of chart to pyramid. */
        Pyramid: 'pyramid',
        /**  sets the seriesType of chart to funnel. */
        Funnel: 'funnel',
        /**  sets the seriesType of chart to polar. */
        Polar: 'polar',
        /**  sets the seriesType of chart to radar. */
        Radar: 'radar',
        /**  sets the seriesType of chart to rangeArea. */
        RangeArea: 'rangearea'

    };
    /**
	 * Enum for chart labelPlacement.	 
	 * @enum {string}
	 * @global 
	 */
    ej.datavisualization.Chart.LabelPlacement = {
        /**  specifies the placement of labels between the ticks. */
        BetweenTicks: "betweenTicks",
        /**  specifies the placement of labels on the ticks. */
        OnTicks: "onTicks"
    };
    /**
    * Enum for chart labelIntersectAction.	 
    * @enum {string}
    * @global 
    */
    ej.datavisualization.Chart.LabelIntersectAction = {
        /**  No action is made on intersection of labels. */
        None: "none",
        /**  Rotate the labels to 90 degree on intersect. */
        Rotate90: "rotate90",
        /**  Rotate the labels to 45 degree on intersect. */
        Rotate45: "rotate45",
        /**  Wrap the label to multiple lines on intersect. */
        Wrap: "wrap",
        /**  Trim the text and display whole text on mouse over. */
        Trim: "trim",
        /**  Hide the label which intersect. */
        Hide: "hide",
        /**  Labels are placed on another row on intersect. */
        MultipleRows: "multipleRows"
    };
    /**
 * Enum for chart edgeLabelPlacement.	 
 * @enum {string}
 * @global 
 */
    ej.datavisualization.Chart.EdgeLabelPlacement = {
        /**  Places the edge labels in the same position. */
        None: "none",
        /**  Shift the edge labels inside the chart area. */
        Shift: "shift",
        /**  Hide the edge labels which exceeds the chart area boundary. */
        Hide: "hide"
    };
    /**
	 * Enum for chart themes.	 
	 * @enum {string}
	 * @global 
	 */
    ej.datavisualization.Chart.Theme = {
        /**  Render chart in azure theme. */
        Azure: 'azure',
        /**  Render chart in flatlight theme. */
        FlatLight: 'flatlight',
        /**  Render chart in azuredark theme. */
        Azuredark: 'azuredark',
        /**  Render chart in lime theme. */
        Lime: 'lime',
        /**  Render chart in limedark theme. */
        LimeDark: 'limedark',
        /**  Render chart in saffron theme. */
        Saffron: 'saffron',
        /**  Render chart in saffrondark theme. */
        SaffronDark: 'saffrondark',
        /**  Render chart in gradientlight theme. */
        GradientLight: 'gradientlight',
        /**  Render chart in gradientdark theme. */
        GradientDark: 'gradientdark'
    };
    /**
	 * Enum for chart style of font.	 
	 * @enum {string}
	 * @global 
	 */
    ej.datavisualization.Chart.FontStyle = {
        /**  Render normal font. */
        Normal: 'normal',
        /**  Render bold font. */
        Bold: 'bold',
        /**  Render italic font. */
        Italic: 'italic'
    };
    /**
	 * Enum for weight of font.	 
	 * @enum {string}
	 * @global 
	 */
    ej.datavisualization.Chart.FontWeight = {
        /**  Render regular font. */
        Regular: 'regular',
        /**  Render lighter font. */
        Lighter: 'lighter'
    };
    /**
	 * Enum for chart orientation.	 
	 * @enum {string}
	 * @global 
	 */
    ej.datavisualization.Chart.Orientation = {
        /**  Sets chart orientation to horizontal. */
        Horizontal: 'horizontal',
        /**  Sets chart orientation to vertical. */
        Vertical: 'vertical'
    };
    /**
	 * Enum for chart interval type.	 
	 * @enum {string}
	 * @global 
	 */
    ej.datavisualization.Chart.IntervalType = {
        /**  Sets chart interval type to days. */
        Days: 'days',
        /**  Sets chart interval type to hours. */
        Hours: 'hours',
        /**  Sets chart interval type to seconds. */
        Seconds: 'seconds',
        /**  Sets chart interval type to milliseconds. */
        Milliseconds: 'milliseconds',
        /**  Sets chart interval type to minutes. */
        Minutes: 'minutes',
        /**  Sets chart interval type to months. */
        Months: 'months',
        /**  Sets chart interval type to years. */
        Years: 'years'
    };
    /**
	 * Enum for range padding.	 
	 * @enum {string}
	 * @global 
	 */
    ej.datavisualization.Chart.RangePadding = {
        /**  Sets chart range padding as additional. */
        Additional: 'additional',
        /**  Sets chart range padding as normal. */
        Normal: 'normal',
        /**  Sets chart range padding as none. */
        None: 'none',
        /**  Sets chart range padding as round. */
        Round: 'round'
    };
    /**
	 * Enum for text alignment.	 
	 * @enum {string}
	 * @global 
	 */
    ej.datavisualization.Chart.TextAlignment = {
        /**  Support for changing text alignment to middleTop. */
        MiddleTop: 'middletop',
        /**  Support for changing text alignment to middleCenter. */
        MiddleCenter: 'middlecenter',
        /**  Support for changing text alignment to middleBottom. */
        MiddleBottom: 'middlebottom'
    };
    /**
	 * Enum for label position.	 
	 * @enum {string}
	 * @global 
	 */
    ej.datavisualization.Chart.LabelPosition = {
        /**  Support for changing the labelPosition to inside. */
        Inside: 'inside',
        /**  Support for changing the labelPosition to outside. */
        Outside: 'outside'
    };
    /**
	 * Enum for zIndex.	 
	 * @enum {string}
	 * @global 
	 */
    ej.datavisualization.Chart.ZIndex = {
        /**  Support for changing the zIndex to over. */
        Over: 'over',
        /**  Support for changing the labelPosition to behind. */
        Behind: 'behind'
    };
    /**
	 * Enum for chart unit.	 
	 * @enum {string}
	 * @global 
	 */
    ej.datavisualization.Chart.Unit = {
        /**  Specifies the unit in percentage. */
        percentage: 'percentage',
        /**  Specifies the unit in pixel. */
        pixel: 'pixel'
    };
    /**
	 * Enum for chart pyramid mode.	 
	 * @enum {string}
	 * @global 
	 */
    ej.datavisualization.Chart.PyramidMode = {
        /**  Specifies the mode of the pyramid as linear. */
        Linear: 'linear',
        /**  Specifies the mode of the pyramid as surface. */
        Surface: 'Surface'
    };
    /**
    * Enum for chart drawType mode.	 
    * @enum {string}
    * @global 
    */
    ej.datavisualization.Chart.DrawType = {
        /**  Specifies the drawType as line. */
        Line: 'line',
        /**  Specifies the drawType as column. */
        Column: 'column',
        /**  Specifies the drawType as area. */
        Area: 'area'
    };
    /**
	 * Enum for chart shape.	 
	 * @enum {string}
	 * @global 
	 */
    ej.datavisualization.Chart.Shape = {
        /**  Sets the shape to none. */
        None: 'none',
        /**  Sets the shape to leftarrow. */
        LeftArrow: 'leftarrow',
        /**  Sets the shape to rightarrow. */
        RightArrow: 'rightarrow',
        /**  Sets the shape to circle. */
        Circle: 'circle',
        /**  Sets the shape to cross. */
        Cross: 'cross',
        /**  Sets the shape to horizline. */
        HorizLine: 'horizline',
        /**  Sets the shape to vertline. */
        VertLine: 'vertLine',
        /**  Sets the shape to diamond. */
        Diamond: 'diamond',
        /**  Sets the shape to rectangle. */
        Rectangle: 'rectangle',
        /**  Sets the shape to triangle. */
        Triangle: 'triangle',
        /**  Sets the shape to invertedtriangle. */
        InvertedTriangle: 'invertedtriangle',
        /**  Sets the shape to hexagon. */
        Hexagon: 'hexagon',
        /**  Sets the shape to pentagon. */
        Pentagon: 'pentagon',
        /**  Sets the shape to star. */
        Star: 'star',
        /**  Sets the shape to ellipse. */
        Ellipse: 'ellipse',
        /**  Sets the shape to wedge. */
        Wedge: 'wedge',
        /**  Sets the shape to trapezoid. */
        Trapezoid: 'trapezoid',
        /**  Sets the shape to uparrow. */
        UpArrow: 'uparrow',
        /**  Sets the shape to downarrow. */
        DownArrow: 'downarrow',
        /**  Sets the shape to image. */
        Image: 'image'
    };
    /**
	 * Enum for hilopenclose drawmode.	 
	 * @enum {string}
	 * @global 
	 */
    ej.datavisualization.Chart.DrawMode = {
        /**  Specifies the draw mode as both. */
        Both: 'both',
        /**  Specifies the draw mode as open. */
        Open: 'open',
        /**  Specifies the draw mode as close */
        Close: 'close'
    };
    /**
	 * Enum for linecap.	 
	 * @enum {string}
	 * @global 
	 */
    ej.datavisualization.Chart.LineCap = {
        /**  Sets the linecap as butt. */
        Butt: 'butt',
        /**  Sets the linecap as round. */
        Round: 'round',
        /**  Sets the linecap as square. */
        Square: 'square'
    };
    /**
	 * Enum for line join.	 
	 * @enum {string}
	 * @global 
	 */
    ej.datavisualization.Chart.LineJoin = {
        /**  Sets the lineJoin to round. */
        Round: 'round',
        /**  Sets the lineJoin to bevel. */
        Bevel: 'bevel',
        /**  Sets the lineJoin to miter. */
        Miter: 'miter'
    };
    /**
	 * Enum for position.	 
	 * @enum {string}
	 * @global 
	 */
    ej.datavisualization.Chart.Position = {
        /**  Sets the position to top. */
        Top: 'top',
        /**  Sets the position to middle. */
        Middle: 'middle',
        /**  Sets the position to bottom. */
        Bottom: 'bottom'
    };
    /**
	 * Enum for alignment.	 12
	 * @enum {string}
	 * @global 
	 */
    ej.datavisualization.Chart.Alignment = {
        /**  Sets the lineJoin to center. */
        Center: 'center',
        /**  Sets the lineJoin to near. */
        Near: 'near',
        /**  Sets the lineJoin to far. */
        Far: 'far'
    };
    $.extend(ej.datavisualization.Chart.prototype, ej.ejChart);
})(jQuery, Syncfusion);;
/**
* @fileOverview Plugin for Circular Gauge
* @copyright Copyright Syncfusion Inc. 2001 - 2014. All rights reserved.
*  Use of this code is subject to the terms of our license.
*  A copy of the current license can be obtained at any time by e-mailing
*  licensing@syncfusion.com. Any infringement will be prosecuted under
*  applicable laws. 
* @version 11.2 
* @author <a href="mailto:licensing@syncfusion.com">Syncfusion Inc</a>
*/

(function ($, ej, undefined) {

    /**
    * @namespace ej
    * @classdesc The Circular gauge  can be easily configured to the DOM element, such as div. you can create a circular gauge with a highly customizable look and feel.
    * @class ejCircularGauge
    * @param {object} options - For seting the Circular gauge
    * @requires jQuery
    * @requires ej.common.all
    * @requires excanvas.js
    * @example 
    * &lt;div id="CoreCircularGauge"&gt;
    * &lt;/div&gt; <br> 
    * &lt;script&gt;
    * $(function () {
        $("#CoreCircularGauge").ejCircularGauge({
        });
    });
    * &lt;/script&gt;
    * @remarks
    * excanvas.js need to add for IE8 
    */

    ej.widget({ "ejCircularGauge": "ej.datavisualization.CircularGauge" /** @lends ejCircularGauge# */ }, {
        _rootCSS: "e-circulargauge",
        validTags: ["div", "span"],
        // default model
        defaults: /** @lends ejCircularGauge# */{
            /**		
			* Specifies the value of circular gauge.
			* @default 0
            * @alias ejCircularGauge#value
			* @type {number}
			* @example 
            * &lt;div id="CoreCircularGauge"&gt;
            * &lt;/div&gt; <br> 
            * &lt;script&gt;
			* $("#CoreCircularGauge").ejCircularGauge({ value: 30 });
            * &lt;/script&gt;
			* @memberof ejCircularGauge
			* @instance
			*/
            value: 0,
            /**		
			* Specifies the minimum value of circular gauge.
			* @default 0
            * @alias ejCircularGauge#minimum
			* @type {number}
			* @example 
            * &lt;div id="CoreCircularGauge"&gt;
            * &lt;/div&gt; <br> 
            * &lt;script&gt;
			* $("#CoreCircularGauge").ejCircularGauge({ minimum: 10 });
            * &lt;/script&gt;
			* @memberof ejCircularGauge
			* @instance
			*/
            minimum: 0,
            /**		
			* Specifies the maximum value of circular gauge.
			* @default 100
            * @alias ejCircularGauge#maximum
			* @type {number}
			* @example 
            * &lt;div id="CoreCircularGauge"&gt;
            * &lt;/div&gt; <br> 
            * &lt;script&gt;
			* $("#CoreCircularGauge").ejCircularGauge({ maximum: 120 });
            * &lt;/script&gt;
			* @memberof ejCircularGauge
			* @instance
			*/
            maximum: 100,
            /**		
			* Specifies the radius of circular gauge.
			* @default 180
            * @alias ejCircularGauge#radius
			* @type {number}
			* @example 
            * &lt;div id="CoreCircularGauge"&gt;
            * &lt;/div&gt; <br> 
            * &lt;script&gt;
			* $("#CoreCircularGauge").ejCircularGauge({ radius: 100 });
            * &lt;/script&gt;
			* @memberof ejCircularGauge
			* @instance
			*/
            radius: 180,
            /**		
			* Specifies the width of circular gauge.
			* @default 360
            * @alias ejCircularGauge#width
			* @type {number}
			* @example 
            * &lt;div id="CoreCircularGauge"&gt;
            * &lt;/div&gt; <br> 
            * &lt;script&gt;
			* $("#CoreCircularGauge").ejCircularGauge({ width: 400 });
            * &lt;/script&gt;
			* @memberof ejCircularGauge
			* @instance
			*/
            width: 360, // Width Should be 2*radius
            /**		
			* Specifies the height of circular gauge.
            * @alias ejCircularGauge#height
			* @default 360
			* @type {number}
			* @example 
            * &lt;div id="CoreCircularGauge"&gt;
            * &lt;/div&gt; <br> 
            * &lt;script&gt;
			* $("#CoreCircularGauge").ejCircularGauge({ height: 400 });
            * &lt;/script&gt;
			* @memberof ejCircularGauge
			* @instance
			*/
            height: 360, //Height Should be 2*radious
            /**		
			* Specify the frame of circular gauge
			* @default Object
            * @alias ejCircularGauge#frame
            * @type {object}
			* @example 
            * &lt;div id="CoreCircularGauge"&gt;
            * &lt;/div&gt; <br> 
            * &lt;script&gt;		
			* $("#CoreCircularGauge").ejCircularGauge({ frame:{frameType:ej.datavisualization.CircularGauge.Frame.FullCircle, backgroundImageUrl:"", halfCircleFrameStartAngle:180, halfCircleFrameEndAngle:360} });
            * &lt;/script&gt;
			* @memberof ejCircularGauge
			* @instance
			*/
            frame: {
                /**		
			    * Specifies the frameType of circular gauge. See {@link Frame}
			    * @default ej.datavisualization.CircularGauge.Frame.FullCircle
                * @alias ejCircularGauge#frame->frameType
			    * @type {enum}
			    * @example 
                * &lt;div id="CoreCircularGauge"&gt;
                * &lt;/div&gt; <br> 
                * &lt;script&gt;
			    * $("#CoreCircularGauge").ejCircularGauge({  frame:{frameType : "halfcircle"} });
                * &lt;/script&gt;
			    * @memberof ejCircularGauge
			    * @instance
			    */
                frameType: "fullcircle", // ["fullcircle", "halfcircle"],
                /**		
			    * Specify the url of the frame background image for circular gauge
			    * @default null
                * @alias ejCircularGauge#frame->backgroundImageUrl
			    * @type {string}
			    * @example 
                * &lt;div id="CoreCircularGauge"&gt;
                * &lt;/div&gt; <br> 
                * &lt;script&gt;			
			    * 	$("#CoreCircularGauge").ejCircularGauge({  frame:{backgroundImageUrl : "Sun.jpg" }});
                * &lt;/script&gt;
			    * @memberof ejCircularGauge
			    * @instance
			    */
                backgroundImageUrl: null,
                /**		
			    * Specifies the start angle for the half circular frame.
			    * @default 180
                * @alias ejCircularGauge#frame->halfCircleFrameStartAngle
			    * @type {number}
			    * @example  
                * &lt;div id="CoreCircularGauge"&gt;
                * &lt;/div&gt; <br> 
                * &lt;script&gt;
			    * $("#CoreCircularGauge").ejCircularGauge({ frame:{frameType : "halfcircle",halfCircleFrameStartAngle: 0} });
                * &lt;/script&gt;
			    * @memberof ejCircularGauge
			    * @instance
			    */
                halfCircleFrameStartAngle: 180,
                /**		
                * Specifies the end angle for the half circular frame.
                * @default 360
                * @alias ejCircularGauge#frame->halfCircleFrameEndAngle
                * @type {number}
                * @example  
                * &lt;div id="CoreCircularGauge"&gt;
                * &lt;/div&gt; <br> 
                * &lt;script&gt;
                * $("#CoreCircularGauge").ejCircularGauge({ frame:{frameType : "halfCircle",halfCircleFrameEndAngle: 270}});
                * &lt;/script&gt;
                * @memberof ejCircularGauge
                * @instance
                */
                halfCircleFrameEndAngle: 360
            },
            /**		
			* Specifies the backgroundcolor of circular gauge.
			* @default null
            * @alias ejCircularGauge#backgroundColor
			* @type {string}
			* @example 			
            * &lt;div id="CoreCircularGauge"&gt;
            * &lt;/div&gt; <br> 
            * &lt;script&gt;
			* $("#CoreCircularGauge").ejCircularGauge({  backgroundColor : "#F234F4" });
            * &lt;/script&gt;
			* @memberof ejCircularGauge
			* @instance
			*/
            backgroundColor: null,
            /**		
			* Specifies the interiorGradient of circular gauge.
			* @default null
            * @alias ejCircularGauge#interiorGradient
			* @type {object}
			* @example 			
            * &lt;div id="CoreCircularGauge"&gt;
            * &lt;/div&gt; <br> 
            * &lt;script&gt;
			* $("#CoreCircularGauge").ejCircularGauge({ interiorGradient: { colorInfo:[{colorStop : 0, color:"#FFFFFF"},{colorStop : 1, color:"#AAAAAA"}] } }); 
            * &lt;/script&gt;
			* @memberof ejCircularGauge
			* @instance
			*/
            interiorGradient: null, //{colorInfo:[{colorStop : 0, color:"Green"},{colorStop : 1, color:"Yellow"}] },
            /**		
			* Specify readonly value of circular gauge
			* @default true
            * @alias ejCircularGauge#readOnly
			* @type {boolean}
			* @example 			
            * &lt;div id="CoreCircularGauge"&gt;
            * &lt;/div&gt; <br> 
            * &lt;script&gt;
			* $("#CoreCircularGauge").ejCircularGauge({  readOnly : false });
            * &lt;/script&gt;
			* @memberof ejCircularGauge
			* @instance
			*/
            readOnly: true,
            /**		
			* Specify animate value of circular gauge
			* @default true
            * @alias ejCircularGauge#enableAnimation
			* @type {boolean}
			* @example 
            * &lt;div id="CoreCircularGauge"&gt;
            * &lt;/div&gt; <br> 
            * &lt;script&gt 			
			* $("#CoreCircularGauge").ejCircularGauge({ enableAnimation: true,scales: [{ pointers: [{ value: 50 }] }] });
            * &lt;/script&gt;
			* @memberof ejCircularGauge
			* @instance
			*/
            enableAnimation: true,
            /**		
			* Specifies animationSpeed of circular gauge
			* @default 500
            * @alias ejCircularGauge#animationSpeed
			* @type {number}
			* @example 
            * &lt;div id="CoreCircularGauge"&gt;
            * &lt;/div&gt; <br> 
            * &lt;script&gt;
			* 	$("#CoreCircularGauge").ejCircularGauge({ animationSpeed: 500,scales: [{ pointers: [{ value: 50 }] }] });
            * &lt;/script&gt;
			 * @memberof ejCircularGauge
			* @instance
			*/
            animationSpeed: 500,
            /**		
			* Specify the theme of circular gauge.
			* @default "flatlight"
            * @alias ejCircularGauge#theme
			* @type {string}
			* @example 
            * &lt;div id="CoreCircularGauge"&gt;
            * &lt;/div&gt; <br> 
            * &lt;script&gt;
			* 	$("#CoreCircularGauge").ejCircularGauge({  theme : "flatlight" });
            * &lt;/script&gt;
			 * @memberof ejCircularGauge
			* @instance
			*/
            theme: "flatlight",
            /**		
			* Specify isRadialGradient value of circular gauge
			* @default false
            * @alias ejCircularGauge#isRadialGradient
			* @type {boolean}
			* @example 
            * &lt;div id="CoreCircularGauge"&gt;
            * &lt;/div&gt; <br> 
            * &lt;script&gt;		
			* 	$("#CoreCircularGauge").ejCircularGauge({  isRadialGradient : true });
            * &lt;/script&gt;
			 * @memberof ejCircularGauge
			* @instance
			*/
            isRadialGradient: false,
            /**		
			* Specify enableResize value of circular gauge
			* @default false
            * @alias ejCircularGauge#enableResize
			* @type {boolean}
			* @example 
            * &lt;div id="CoreCircularGauge"&gt;
            * &lt;/div&gt; <br> 
            * &lt;script&gt;			
			* 	$("#CoreCircularGauge").ejCircularGauge({  enableResize : true });
            * &lt;/script&gt;
			 * @memberof ejCircularGauge
			* @instance
			*/
            enableResize: false,
            //Events

             /**     
			 * Triggers while the ticks are being drawn on the gauge.
			 * @event
			 * @name ejCircularGauge#drawTicks		
             * @param {Object} args.object returns the object of the gauge.
             * @param {boolean} args.cancel returns the cancel option value
             * @param {Object} args.context returns the context element
             * @param {Object} args.position returns the startX and startY of the ticks
             * @param {Object} args.model returns the gauge model
             * @param {Object} args.scaleElement returns the options of the scale element.
             * @param {number} args.scaleIndex returns the scaleIndex to which the tick belongs.
             * @param {string} args.style returns the ticks style
             * @param {Object} args.tick returns the tick object of the gauge.
             * @param {number} args.tick.angle returns the angle of the tick.
             * @param {Object} args.tick.element returns the current tick element.
             * @param {number} args.tick.index returns the index of the tick.
             * @param {number} args.pointerValue returns the label value of the tick.
             * @param {string} args.type returns the name of the event
			 * @example 
             * &lt;div id="CoreCircularGauge"&gt;
             * &lt;/div&gt; <br> 
             * &lt;script&gt;
             * $("#CoreCircularGauge").ejCircularGauge({
             *    drawTicks: function (args) {}
             * });
            * &lt;/script&gt;
			 * @memberof ejCircularGauge
			 * @instance
			 */
            drawTicks: null,
            /**     
			 * Triggers while the labels are being drawn on the gauge.
			 * @event
			 * @name ejCircularGauge#drawLabels		
             * @param {Object} args.object returns the object of the gauge.
             * @param {boolean} args.cancel returns the cancel option value
             * @param {Object} args.context returns the context element
             * @param {Object} args.position returns the startX and startY of the labels
             * @param {Object} args.model returns the gauge model
             * @param {Object} args.scaleElement returns the options of the scale element.
             * @param {number} args.scaleIndex returns the scaleIndex to which the label belongs.
             * @param {string} args.style returns the label style
             * @param {Object} args.label returns the label object of the gauge.
             * @param {number} args.label.angle returns the angle of the labels.
             * @param {Object} args.label.element returns the current label element.
             * @param {number} args.label.index returns the index of the label.
             * @param {number} args.pointerValue returns the value of the label.
             * @param {string} args.type returns the name of the event
			 * @example 
             * &lt;div id="CoreCircularGauge"&gt;
             * &lt;/div&gt; <br> 
             * &lt;script&gt;
             * $("#CoreCircularGauge").ejCircularGauge({
             *    drawLabels: function (args) {}
             * });
            * &lt;/script&gt;
			 * @memberof ejCircularGauge
			 * @instance
			 */
            drawLabels: null,
            /**     
			 * Triggers while the pointers are being drawn on the gauge.
			 * @event
			 * @name ejCircularGauge#drawPointers		
             * @param {Object} args.object returns the object of the gauge.
             * @param {boolean} args.cancel returns the cancel option value
             * @param {Object} args.context returns the context element
             * @param {Object} args.position returns the startX and startY of the pointer
             * @param {Object} args.model returns the gauge model
             * @param {Object} args.scaleElement returns the options of the scale element.
             * @param {number} args.scaleIndex returns the scaleIndex to which the pointer belongs.
             * @param {string} args.style returns the pointer style
             * @param {Object} args.pointer returns the pointer object of the gauge.
             * @param {number} args.pointer.angle returns the angle of the pointer.
             * @param {Object} args.pointer.element returns the current pointer element.
             * @param {number} args.pointer.index returns the index of the pointer.
             * @param {number} args.pointer.value returns the value of the pointer.
             * @param {string} args.type returns the name of the event
			 * @example 
             * &lt;div id="CoreCircularGauge"&gt;
             * &lt;/div&gt; <br> 
             * &lt;script&gt;
             * $("#CoreCircularGauge").ejCircularGauge({
             *    drawPointers: function (args) {}
             * });
            * &lt;/script&gt;
			 * @memberof ejCircularGauge
			 * @instance
			 */
            drawPointers: null,
            /**     
			 * Triggers when the ranges begin to be getting drawn on the gauge.
			 * @event
			 * @name ejCircularGauge#drawRange		
             * @param {Object} args.object returns the object of the gauge.
             * @param {boolean} args.cancel returns the cancel option value
             * @param {Object} args.context returns the context element
             * @param {Object} args.position returns the startX and startY of the range
             * @param {Object} args.model returns the gauge model
             * @param {Object} args.scaleElement returns the options of the scale element.
             * @param {number} args.scaleIndex returns the scaleIndex to which the range belongs.
             * @param {string} args.style returns the range style
             * @param {Object} args.rangeElement returns the current range element.
             * @param {number} args.rangeIndex returns the index of the range.
             * @param {string} args.type returns the name of the event
			 * @example 
             * &lt;div id="CoreCircularGauge"&gt;
             * &lt;/div&gt; <br> 
             * &lt;script&gt;
             * $("#CoreCircularGauge").ejCircularGauge({
             *    drawRange: function (args) {}
             * });
            * &lt;/script&gt;
			 * @memberof ejCircularGauge
			 * @instance
			 */
            drawRange: null,
            /**     
			 * Triggers while the custom labels are being drawn on the gauge.
			 * @event
			 * @name ejCircularGauge#drawCustomLabel		
             * @param {Object} args.object returns the object of the gauge.
             * @param {boolean} args.cancel returns the cancel option value
             * @param {Object} args.context returns the context element
             * @param {Object} args.position returns the startX and startY of the custom label
             * @param {Object} args.model returns the gauge model
             * @param {Object} args.scaleElement returns the options of the scale element.
             * @param {number} args.scaleIndex returns the scaleIndex to which the custom label belongs.
             * @param {string} args.style returns the custom label style
             * @param {Object} args.customLabelElement returns the current custom label element.
             * @param {number} args.customLabelIndex returns the index of the custom label.
             * @param {string} args.type returns the name of the event
			 * @example 
             * &lt;div id="CoreCircularGauge"&gt;
             * &lt;/div&gt; <br> 
             * &lt;script&gt;
             * $("#CoreCircularGauge").ejCircularGauge({
             *    drawCustomLabel: function (args) {}
             * });
            * &lt;/script&gt;
			 * @memberof ejCircularGauge
			 * @instance
			 */
            drawCustomLabel: null,
            /**     
			 * Triggers while the indicators are being started to drawn on the gauge.
			 * @event
			 * @name ejCircularGauge#drawIndicators		
             * @param {Object} args.object returns the object of the gauge.
             * @param {boolean} args.cancel returns the cancel option value
             * @param {Object} args.context returns the context element
             * @param {Object} args.position returns the startX and startY of the indicator
             * @param {Object} args.model returns the gauge model
             * @param {Object} args.scaleElement returns the options of the scale element.
             * @param {number} args.scaleIndex returns the scaleIndex to which the indicator belongs.
             * @param {string} args.style returns the indicator style
             * @param {Object} args.indicatorElement returns the current indicator element.
             * @param {number} args.indicatorIndex returns the index of the indicator.
             * @param {string} args.type returns the name of the event
			 * @example 
             * &lt;div id="CoreCircularGauge"&gt;
             * &lt;/div&gt; <br> 
             * &lt;script&gt;
             * $("#CoreCircularGauge").ejCircularGauge({
             *    drawIndicators: function (args) {}
             * });
            * &lt;/script&gt;
			 * @memberof ejCircularGauge
			 * @instance
			 */
            drawIndicators: null,
            /**     
			 * Triggers while the pointer cap is being drawn on the gauge.
			 * @event
			 * @name ejCircularGauge#drawPointerCap		
             * @param {Object} args.object returns the object of the gauge.
             * @param {boolean} args.cancel returns the cancel option value
             * @param {Object} args.context returns the context element
             * @param {Object} args.scaleElement returns the options of the scale element.
             * @param {Object} args.position returns the startX and startY of the pointer cap.
             * @param {Object} args.model returns the gauge model
             * @param {string} args.style returns the pointer cap style
             * @param {string} args.type returns the name of the event
			 * @example 
             * &lt;div id="CoreCircularGauge"&gt;
             * &lt;/div&gt; <br> 
             * &lt;script&gt;
             * $("#CoreCircularGauge").ejCircularGauge({
             *    drawPointerCap: function (args) {}
             * });
            * &lt;/script&gt;
			 * @memberof ejCircularGauge
			 * @instance
			 */
            drawPointerCap: null,
            /**     
			 * Triggers while the gauge start to Load.
			 * @event
			 * @name ejCircularGauge#load		
             * @param {Object} args.object returns the object of the gauge.
			 * @param {cancel } args.cancel returns the cancel option value
 			 * @param {Model} args.Model returns the gauge model
 			 * @param {scaleElement} args.scaleElement returns the entire scale element.
			 * @param {context} args.context returns the context element
			 * @param {type} args.type returns the name of the event
			 * @example 
	         * &lt;div id="CircularGauge1"&gt;&lt;/div&gt; <br> 
	         * &lt;script&gt;
             * $("#CircularGauge1").ejCircularGauge({
             *    load: function (args) {}
             * });
	         * &lt;/script&gt;	
			 * @memberof ejCircularGauge
			 * @instance
			 */
            load: null,
            /**     
			 * Triggers when the rendering of the gauge is completed.
			 * @event
			 * @name ejCircularGauge#renderComplete		
             * @param {Object} args.object returns the object of the gauge.
             * @param {boolean} args.cancel returns the cancel option value
             * @param {Object} args.context returns the context element
             * @param {Object} args.scaleElement returns the entire scale element.
             * @param {Object} args.model returns the gauge model
             * @param {string} args.type returns the name of the event
			 * @example 
             * &lt;div id="CoreCircularGauge"&gt;
             * &lt;/div&gt; <br> 
             * &lt;script&gt;
             * $("#CoreCircularGauge").ejCircularGauge({
             *    renderComplete: function (args) {}
             * });
            * &lt;/script&gt;
			 * @memberof ejCircularGauge
			 * @instance
			 */
            renderComplete: null,
            /**     
			 * Triggers when the left mouse button is clicked.
			 * @event
			 * @name ejCircularGauge#mouseClick		
             * @param {Object} args.object returns the object of the gauge.
             * @param {boolean} args.cancel returns the cancel option value
             * @param {Object} args.model returns the gauge model
             * @param {Object} args.type returns the name of the event
             * @param {Object} args.scaleElement returns the scale element.
             * @param {number} args.scaleIndex returns the scaleIndex to which the pointer belongs.
             * @param {Object} args.context returns the context element
             * @param {Object} args.pointer returns the pointer object
             * @param {number} args.pointer.index returns the pointer Index
             * @param {Object} args.pointer.element returns the pointer element.
             * @param {number} args.pointer.value returns the value of the pointer.
             * @param {number} args.pointer.angle returns the angle of the pointer.
             * @param {string} args.style returns the pointer style
             * @param {Object} args.position returns the startX and startY of the pointer.
			 * @example 
             * &lt;div id="CoreCircularGauge"&gt;
             * &lt;/div&gt; <br> 
             * &lt;script&gt;
             * $("#CoreCircularGauge").ejCircularGauge({
             *    mouseClick: function (args) {}
             * });
            * &lt;/script&gt;
			 * @memberof ejCircularGauge
			 * @instance
			 */
            mouseClick: null,
            /**     
			 * Triggers when clicking and dragging the mouse pointer over the gauge pointer.
			 * @event
			 * @name ejCircularGauge#mouseClickMove		
             * @param {Object} args.object returns the object of the gauge.
             * @param {boolean} args.cancel returns the cancel option value
             * @param {Object} args.model returns the gauge model
             * @param {Object} args.type returns the name of the event
             * @param {Object} args.scaleElement returns the scale element.
             * @param {number} args.scaleIndex returns the scaleIndex to which the pointer belongs.
             * @param {Object} args.context returns the context element
             * @param {Object} args.pointer returns the pointer object
             * @param {number} args.pointer.index returns the pointer Index
             * @param {Object} args.pointer.element returns the pointer element.
             * @param {number} args.pointer.value returns the value of the pointer.
             * @param {number} args.pointer.angle returns the angle of the pointer.
             * @param {string} args.style returns the pointer style
             * @param {Object} args.position returns the startX and startY of the pointer.
			 * @example 
             * &lt;div id="CoreCircularGauge"&gt;
             * &lt;/div&gt; <br> 
             * &lt;script&gt;
             * $("#CoreCircularGauge").ejCircularGauge({
             *    mouseClickMove: function (args) {}
             * });
            * &lt;/script&gt;
			 * @memberof ejCircularGauge
			 * @instance
			 */
            mouseClickMove: null,
            /**     
			 * Triggers when the mouse click is released.
			 * @event
			 * @name ejCircularGauge#mouseClickUp		
             * @param {Object} args.object returns the object of the gauge.
             * @param {boolean} args.cancel returns the cancel option value
             * @param {Object} args.model returns the gauge model
             * @param {Object} args.type returns the name of the event
             * @param {Object} args.scaleElement returns the scale element.
             * @param {number} args.scaleIndex returns the scaleIndex to which the pointer belongs.
             * @param {Object} args.context returns the context element
             * @param {Object} args.pointer returns the pointer object
             * @param {number} args.pointer.index returns the pointer Index
             * @param {Object} args.pointer.element returns the pointer element.
             * @param {number} args.pointer.value returns the value of the pointer.
             * @param {number} args.pointer.angle returns the angle of the pointer.
             * @param {string} args.style returns the pointer style
             * @param {Object} args.position returns the startX and startY of the pointer.
			 * @example 
             * &lt;div id="CoreCircularGauge"&gt;
             * &lt;/div&gt; <br> 
             * &lt;script&gt;
             * $("#CoreCircularGauge").ejCircularGauge({
             *    mouseClickUp: function (args) {}
             * });
            * &lt;/script&gt;
			 * @memberof ejCircularGauge
			 * @instance
			 */
            mouseClickUp: null,

            //Scale Collection
            /**		
			* Specify the pointers, ticks, labels, indicators, ranges  of circular gauge
			* @default null
			* @type {object}
			* @example 
            * &lt;div id="CoreCircularGauge"&gt;
            * &lt;/div&gt; <br> 
            * &lt;script&gt;			
			* $("#CoreCircularGauge").ejCircularGauge({ scales: [{showScaleBar: true, size: 6, border:{width: 1.5} }] });
            * &lt;/script&gt;
			* @memberof ejCircularGauge
			* @instance
			*/
            scales: null
        },
        // * @memberof ejCircularGauge

        _defaultScaleValues: function () {
            return {
                /**		
			    * Specify scaleBar size of circular gauge
			    * @default 6
                * @alias ejCircularGauge#scales->size
			    * @type {number}
			    * @example 
                * &lt;div id="CoreCircularGauge"&gt;
                * &lt;/div&gt; <br> 
                * &lt;script&gt;			
			    * $("#CoreCircularGauge").ejCircularGauge({ scales: [{ showScaleBar: true, size: 6 }] });
                * &lt;/script&gt;
			    * @memberof ejCircularGauge
			    * @instance
			    */
                size: 6,
                /**		
			    * Specify pointer cap of circular gauge
			    * @default Object
                * @alias ejCircularGauge#scales->pointerCap
                * @type {object}
			    * @example  
                * &lt;div id="CoreCircularGauge"&gt;
                * &lt;/div&gt; <br> 
                * &lt;script&gt;			
			    * $("#CoreCircularGauge").ejCircularGauge({ scales: [{ pointerCap:{radius: 7, borderWidth:3, interiorGradient:null, borderColor:null } }] });
                * &lt;/script&gt;
			    * @memberof ejCircularGauge
			    * @instance
			    */
                pointerCap: {
                    /**		
			        * Specify pointerCap Radius value of circular gauge
			        * @default 7
                    * @alias ejCircularGauge#scales->pointerCap->radius
			        * @type {number}
			        * @example 
                    * &lt;div id="CoreCircularGauge"&gt;
                    * &lt;/div&gt; <br> 
                    * &lt;script&gt; 			
			        * 	$("#CoreCircularGauge").ejCircularGauge({ scales: [{ pointerCap:{radius: 10} }] });
                    * &lt;/script&gt;
			        * @memberof ejCircularGauge
			        * @instance
			        */
                    radius: 7,
                    /**		
			        * Specify pointerCap borderWidth value of circular gauge
			        * @default 3
                    * @alias ejCircularGauge#scales->pointerCap->borderWidth
			        * @type {number}
			        * @example  	
                    * &lt;div id="CoreCircularGauge"&gt;
                    * &lt;/div&gt; <br> 
                    * &lt;script&gt;		
			        * 	$("#CoreCircularGauge").ejCircularGauge({ scales: [{ pointerCap:{borderWidth: 8 } }] });
                    * &lt;/script&gt;
			        * @memberof ejCircularGauge
			        * @instance
			        */
                    borderWidth: 3,
                    /**		
			        * Specify cap interiorGradient value of circular gauge
			        * @default null
                    * @alias ejCircularGauge#scales->pointerCap->interiorGradient
			        * @type {Object}
			        * @example 		
                    * &lt;div id="CoreCircularGauge"&gt;
                    * &lt;/div&gt; <br> 
                    * &lt;script&gt;	
			        * 	$("#CoreCircularGauge").ejCircularGauge({ scales: [{ pointerCap:{interiorGradient: {colorInfo:[{colorStop : 0, color:"#FFFFFF"},{colorStop : 1, color:"#AAAAAA"}] }}}] });
                    * &lt;/script&gt;
			        * @memberof ejCircularGauge
			        * @instance
			        */
                    interiorGradient: null, //{colorInfo:[{colorStop : 0, color:"#FFFFFF"},{colorStop : 1, color:"#AAAAAA"}] }
                    /**		
			        * Specify cap borderColor of circular gauge
			        * @default null
                    * @alias ejCircularGauge#scales->pointerCap->borderColor
			        * @type {string}
			        * @example 
                    * &lt;div id="CoreCircularGauge"&gt;
                    * &lt;/div&gt; <br> 
                    * &lt;script&gt;			
			        * 	$("#CoreCircularGauge").ejCircularGauge({ scales: [{ pointerCap: {borderColor: "Brown" }}] });
                    * &lt;/script&gt;
			        * @memberof ejCircularGauge
			        * @instance
			        */
                    borderColor: null,
                    /**		
			        * Specify cap backgroundColor of circular gauge
			        * @default null
                    * @alias ejCircularGauge#scales->pointerCap->backgroundColor
			        * @type {string}
			        * @example 
                    * &lt;div id="CoreCircularGauge"&gt;
                    * &lt;/div&gt; <br> 
                    * &lt;script&gt; 			
		    	    * 	$("#CoreCircularGauge").ejCircularGauge({ scales: [{ pointerCap: {backgroundColor: "Green"} }] });
                    * &lt;/script&gt;
    			    * @memberof ejCircularGauge
			        * @instance
			        */
                    backgroundColor:null
                },
                /**		
			    * Specify showScaleBar of circular gauge
			    * @default false
                * @alias ejCircularGauge#scales->showScaleBar
			    * @type {boolean}
			    * @example 
                * &lt;div id="CoreCircularGauge"&gt;
                * &lt;/div&gt; <br> 
                * &lt;script&gt;			
			    * 	$("#CoreCircularGauge").ejCircularGauge({ scales: [{ showScaleBar: true }] });
                * &lt;/script&gt;
			    * @memberof ejCircularGauge
			    * @instance
			    */
                showScaleBar: false,
                /**		
			    * Specify sweepAngle of circular gauge
			    * @default 310
                * @alias ejCircularGauge#scales->sweepAngle
			    * @type {number}
			    * @example 
                * &lt;div id="CoreCircularGauge"&gt;
                * &lt;/div&gt; <br> 
                * &lt;script&gt;			
			    * 	$("#CoreCircularGauge").ejCircularGauge({ scales: [{ sweepAngle: 200 }] });
                * &lt;/script&gt;
			    * @memberof ejCircularGauge
			    * @instance
			    */
                sweepAngle: 310,
                /**		
			    * Specify scale radius of circular gauge
			    * @default 170
                * @alias ejCircularGauge#scales->radius
			    * @type {number}
			    * @example 
                * &lt;div id="CoreCircularGauge"&gt;
                * &lt;/div&gt; <br> 
                * &lt;script&gt;			
			    * 	$("#CoreCircularGauge").ejCircularGauge({ scales: [{ showScaleBar: true, radius: 100 }] });
                * &lt;/script&gt;
			    * @memberof ejCircularGauge
			    * @instance
			    */
                radius: 170,
                /**		
			    * Specify startAngle of circular gauge
			    * @default 115
                * @alias ejCircularGauge#scales->startAngle
			    * @type {number}
			    * @example 
                * &lt;div id="CoreCircularGauge"&gt;
                * &lt;/div&gt; <br> 
                * &lt;script&gt;		
			    * 	$("#CoreCircularGauge").ejCircularGauge({ scales: [{ startAngle: 90 }] });
                * &lt;/script&gt;
			    * @memberof ejCircularGauge
			    * @instance
			    */
                startAngle: 115,
                /**		
			    * Specify majorIntervalValue of circular gauge
			    * @default 10
                * @alias ejCircularGauge#scales->majorIntervalValue
			    * @type {number}
			    * @example 
                * &lt;div id="CoreCircularGauge"&gt;
                * &lt;/div&gt; <br> 
                * &lt;script&gt;			
			    * 	$("#CoreCircularGauge").ejCircularGauge({ scales: [{ majorIntervalValue: 5 }] });
                * &lt;/script&gt;
			    * @memberof ejCircularGauge
			    * @instance
			    */
                majorIntervalValue: 10,
                /**		
			    * Specify minorIntervalValue of circular gauge
			    * @default 2
                * @alias ejCircularGauge#scales->minorIntervalValue
			    * @type {number}
			    * @example 
                * &lt;div id="CoreCircularGauge"&gt;
                * &lt;/div&gt; <br> 
                * &lt;script&gt;		
			    * 	$("#CoreCircularGauge").ejCircularGauge({ scales: [{ minorIntervalValue: 1 }] });
                * &lt;/script&gt;
			    * @memberof ejCircularGauge
			    * @instance
			    */
                minorIntervalValue: 2,
                /**		
			    * Specify maximum scale value of circular gauge
			    * @default null
                * @alias ejCircularGauge#scales->maximum
			    * @type {number}
			    * @example 
                * &lt;div id="CoreCircularGauge"&gt;
                * &lt;/div&gt; <br> 
                * &lt;script&gt;		
			    * 	$("#CoreCircularGauge").ejCircularGauge({ scales: [{ maximum: 200 }] });
                * &lt;/script&gt;
			    * @memberof ejCircularGauge
			    * @instance
			    */
                maximum: null,
                /**		
			    * Specify minimum scale value of circular gauge
			    * @default null
                * @alias ejCircularGauge#scales->minimum
			    * @type {number}
			    * @example 
                * &lt;div id="CoreCircularGauge"&gt;
                * &lt;/div&gt; <br> 
                * &lt;script&gt;		
			    * 	$("#CoreCircularGauge").ejCircularGauge({ scales: [{ minimum: 20 }] });
                * &lt;/script&gt;
			    * @memberof ejCircularGauge
			    * @instance
			    */
                minimum: null,
                /**		
			    * Specify border for scales of circular gauge
			    * @default Object
                * @alias ejCircularGauge#scales->border
                * @type {object}
			    * @example  
                * &lt;div id="CoreCircularGauge"&gt;
                * &lt;/div&gt; <br> 
                * &lt;script&gt;			
			    * $("#CoreCircularGauge").ejCircularGauge({ scales: [{ border:{color:null, width:1.5 }}] });
                * &lt;/script&gt;
			    * @memberof ejCircularGauge
			    * @instance
			    */
                border: {
                    /**		
			        * Specify border color for scales of circular gauge
			        * @default null
                    * @alias ejCircularGauge#scales->border->color
			        * @type {string}
			        * @example 
                    * &lt;div id="CoreCircularGauge"&gt;
                    * &lt;/div&gt; <br> 
                    * &lt;script&gt;			
			        * 	$("#CoreCircularGauge").ejCircularGauge({ scales: [{showScaleBar: true,  border:[{color: "#1BA1E2" }]}] });
                    * &lt;/script&gt;
			        * @memberof ejCircularGauge
			        * @instance
			        */
                    color: null,
                    /**		
			        * Specify border width of circular gauge
			        * @default 1.5
                    * @alias ejCircularGauge#scales->border->width
	    		    * @type {number}
    			    * @example 
                    * &lt;div id="CoreCircularGauge"&gt;
                    * &lt;/div&gt; <br> 
                    * &lt;script&gt; 			
			        * 	$("#CoreCircularGauge").ejCircularGauge({ scales: [{ showScaleBar: true, border:{width: 1.5} }] });
                    * &lt;/script&gt;
			        * @memberof ejCircularGauge
			        * @instance
			        */
                    width:1.5
                },
                /**		
			    * Specify backgroundColor for the scale of circular gauge
			    * @default null
                * @alias ejCircularGauge#scales->backgroundColor
			    * @type {string}
			    * @example 
                * &lt;div id="CoreCircularGauge"&gt;
                * &lt;/div&gt; <br> 
                * &lt;script&gt;		
			    * 	$("#CoreCircularGauge").ejCircularGauge({ scales: [{ showScaleBar: true, backgroundColor: "#1BA1E2" }] });
                * &lt;/script&gt;
			     * @memberof ejCircularGauge
			    * @instance
			    */
                backgroundColor: null,
                /**		
			    * Specify scale direction of circular gauge. See {@link Directions}
			    * @default ej.datavisualization.CircularGauge.Directions.Clockwise
                * @alias ejCircularGauge#scales->direction
			    * @type {enum}
			    * @example 
                * &lt;div id="CoreCircularGauge"&gt;
                * &lt;/div&gt; <br> 
                * &lt;script&gt;	
			    * 	$("#CoreCircularGauge").ejCircularGauge({ scales: [{ direction: "counterclockwise" }] });
                * &lt;/script&gt;
			     * @memberof ejCircularGauge
			    * @instance
			    */
                direction: "clockwise", //["clockwise", "counterclockwise"]
                /**		
			    * Specify showPointers of circular gauge
			    * @default true
                * @alias ejCircularGauge#scales->showPointers
			    * @type {boolean}
			    * @example 
                * &lt;div id="CoreCircularGauge"&gt;
                * &lt;/div&gt; <br> 
                * &lt;script&gt;	
			    * 	$("#CoreCircularGauge").ejCircularGauge({ scales: [{ showPointers: true }] });
                * &lt;/script&gt;
			     * @memberof ejCircularGauge
			    * @instance
			    */
                showPointers: true,
                /**		
			    * Specify showRanges of circular gauge
			    * @default false
                * @alias ejCircularGauge#scales->showRanges
			    * @type {boolean}
			    * @example 
                * &lt;div id="CoreCircularGauge"&gt;
                * &lt;/div&gt; <br> 
                * &lt;script&gt;	
			    * 	$("#CoreCircularGauge").ejCircularGauge({ scales: [{ showRanges: false }] });
                * &lt;/script&gt;
			     * @memberof ejCircularGauge
			    * @instance
			    */
                showRanges: false,
                /**		
			    * Specify showTicks of circular gauge
			    * @default true
                * @alias ejCircularGauge#scales->showTicks
			    * @type {boolean}
			    * @example 
                * &lt;div id="CoreCircularGauge"&gt;
                * &lt;/div&gt; <br> 
                * &lt;script&gt;	
			    * 	$("#CoreCircularGauge").ejCircularGauge({ scales: [{ showTicks: true }] });
                * &lt;/script&gt;
			     * @memberof ejCircularGauge
			    * @instance
			    */
                showTicks: true,
                /**		
			    * Specify showLabels of circular gauge
			    * @default true
                * @alias ejCircularGauge#scales->showLabels
			    * @type {boolean}
			    * @example 
                * &lt;div id="CoreCircularGauge"&gt;
                * &lt;/div&gt; <br> 
                * &lt;script&gt;	
			    * 	$("#CoreCircularGauge").ejCircularGauge({ scales: [{ showLabels: true }] });
                * &lt;/script&gt;
			     * @memberof ejCircularGauge
			    * @instance
			    */
                showLabels: true,
                /**		
			    * Specify showIndicators of circular gauge
			    * @default false
                * @alias ejCircularGauge#scales->showIndicators
			    * @type {boolean}
			    * @example 
                * &lt;div id="CoreCircularGauge"&gt;
                * &lt;/div&gt; <br> 
                * &lt;script&gt;			
			    * 	$("#CoreCircularGauge").ejCircularGauge({ scales: [{ showIndicators: false }] });
                * &lt;/script&gt;
			     * @memberof ejCircularGauge
			    * @instance
			    */
                showIndicators: false,
				/**		
			    * Specify opacity value of circular gauge
			    * @default 1
                * @alias ejCircularGauge#scales->opacity
			    * @type {number}
			    * @example 	
                * &lt;div id="CoreCircularGauge"&gt;
                * &lt;/div&gt; <br> 
                * &lt;script&gt;
			    * 	$("#CoreCircularGauge").ejCircularGauge({ scales: [{showScaleBar: true, opacity:0.5 }] });
                * &lt;/script&gt;
			     * @memberof ejCircularGauge
			    * @instance
			    */
                opacity:1,
				/**		
			    * Specify shadowOffset value of circular gauge
			    * @default 0
                * @alias ejCircularGauge#scales->shadowOffset
			    * @type {number}
			    * @example  
                * &lt;div id="CoreCircularGauge"&gt;
                * &lt;/div&gt; <br> 
                * &lt;script&gt;			
			    * 	$("#CoreCircularGauge").ejCircularGauge({ scales: [{ shadowOffset: 1}] });
                * &lt;/script&gt;
			     * @memberof ejCircularGauge
			    * @instance
			    */
				shadowOffset:0,
                // Pointers Collection
                /**		
			    * Specify pointers value of circular gauge
			    * @default Array
                * @alias ejCircularGauge#scales->pointers
			    * @type {Array}
			    * @example 	
                * &lt;div id="CoreCircularGauge"&gt;
                * &lt;/div&gt; <br> 
                * &lt;script&gt;	
			    * 	$("#CoreCircularGauge").ejCircularGauge({ scales: [{ pointers: [{ distanceFromScale: 0, showBackNeedle: false }] }] });
                * &lt;/script&gt;
			        * @memberof ejCircularGauge
			    * @instance
			    */
                pointers: [{
                    /**		
			        * Specify distanceFromScale value for pointers of circular gauge
			        * @default 0
                    * @alias ejCircularGauge#scales->pointers->distanceFromScale
			        * @type {number}
			        * @example 
                    * &lt;div id="CoreCircularGauge"&gt;
                    * &lt;/div&gt; <br> 
                    * &lt;script&gt;			
			        * 	$("#CoreCircularGauge").ejCircularGauge({ scales: [{ pointers: [{ distanceFromScale: 10 }] }] });
                    * &lt;/script&gt;
			        * @instance
			        */
                    distanceFromScale: 0,
                    /**		
			        * Specify showBackNeedle value of circular gauge
			        * @default false
                    * @alias ejCircularGauge#scales->pointers->showBackNeedle
			        * @type {boolean}
			        * @example 
                    * &lt;div id="CoreCircularGauge"&gt;
                    * &lt;/div&gt; <br> 
                    * &lt;script&gt;				
			        * $("#CoreCircularGauge").ejCircularGauge({ scales: [{ pointers: [{ showBackNeedle: true, backNeedleLength: 10 }] }] });
                    * &lt;/script&gt;
			        * @memberof ejCircularGauge
			        * @instance
			        */
                    showBackNeedle: false,
                    /**		
			        * Specify backNeedleLength of circular gauge
			        * @default 10
                    * @alias ejCircularGauge#scales->pointers->backNeedleLength
			        * @type {number}
			        * @example 
                    * &lt;div id="CoreCircularGauge"&gt;
                    * &lt;/div&gt; <br> 
                    * &lt;script&gt;			
			        * $("#CoreCircularGauge").ejCircularGauge({ scales: [{ pointers: [{ showBackNeedle: true, backNeedleLength: 10 }] }] });
                    * &lt;/script&gt;
			        * @memberof ejCircularGauge
			        * @instance
			        */
                    backNeedleLength: 10,
                    /**		
			        * Specify pointer length of circular gauge
			        * @default 150
                    * @alias ejCircularGauge#scales->pointers->length
			        * @type {number}
			        * @example 
                    * &lt;div id="CoreCircularGauge"&gt;
                    * &lt;/div&gt; <br> 
                    * &lt;script&gt;		
			        * 	$("#CoreCircularGauge").ejCircularGauge({ scales: [{ pointers: [{ length: 50 }] }] });
                    * &lt;/script&gt;
			         * @memberof ejCircularGauge
			        * @instance
			        */
                    length: 150,
                    /**		
			        * Specify pointer Placement value of circular gauge. See {@link PointerPlacement}
			        * @default ej.datavisualization.CircularGauge.PointerPlacement.Near
                    * @alias ejCircularGauge#scales->pointers->placement
			        * @type {enum}
			        * @example 
                    * &lt;div id="CoreCircularGauge"&gt;
                    * &lt;/div&gt; <br> 
                    * &lt;script&gt;	 			
			        * 	$("#CoreCircularGauge").ejCircularGauge({ scales: [{ pointers: [{ placement: "far" }] }] });
                    * &lt;/script&gt;
			         * @memberof ejCircularGauge
			        * @instance
			        */
                    placement: "near", //[near,far,center],
                    /**		
			        * Specify pointer width of circular gauge
			        * @default 7
                    * @alias ejCircularGauge#scales->pointers->width
                    * @type {number}
			        * @example 
                    * &lt;div id="CoreCircularGauge"&gt;
                    * &lt;/div&gt; <br> 
                    * &lt;script&gt;			
			        * 	$("#CoreCircularGauge").ejCircularGauge({ scales: [{ pointers: [{ width: 7 }] }] });
                    * &lt;/script&gt;
			         * @memberof ejCircularGauge
			        * @instance
			        */
                    width: 7,
                    /**		
			        * Specify opacity value for pointer of circular gauge
			        * @default 1
                    * @alias ejCircularGauge#scales->pointers->opacity
                    * @type {number}
			        * @example 
                    * &lt;div id="CoreCircularGauge"&gt;
                    * &lt;/div&gt; <br> 
                    * &lt;script&gt;			
			        * 	$("#CoreCircularGauge").ejCircularGauge({ scales: [{ pointers: [{ opacity: 0.3 }] }] });
                    * &lt;/script&gt;
			         * @memberof ejCircularGauge
			        * @instance
			        */
                    opacity: 1,
                    
                    /**		
			        * Specify value of the pointer of circular gauge
			        * @default null
                    * @alias ejCircularGauge#scales->pointers->value
                    * @type {number}
			        * @example 
                    * &lt;div id="CoreCircularGauge"&gt;
                    * &lt;/div&gt; <br> 
                    * &lt;script&gt;				
			        * $("#CoreCircularGauge").ejCircularGauge({ scales: [{ pointers: [{ value: 50 }] }] });
                    * &lt;/script&gt;
			        * @memberof ejCircularGauge
			        * @instance
			        */
                    value: null,
                    /**		
			        * Specify the border for pointers of circular gauge
			        * @default Object
                    * @alias ejCircularGauge#scales->pointers->border
                    * @type {object}
			        * @example  
                    * &lt;div id="CoreCircularGauge"&gt;
                    * &lt;/div&gt; <br> 
                    * &lt;script&gt;			
			        * $("#CoreCircularGauge").ejCircularGauge({ scales: [{ pointers:[{border:{color:null, width:1.5 }}]}] });
                    * &lt;/script&gt;
		    	    * @memberof ejCircularGauge
	    		    * @instance
    			    */
                    border: {
                        /**		
			            * Specify border color for pointer of circular gauge
			            * @default null
                        * @alias ejCircularGauge#scales->pointers->border->color
                        * @type {string}
			            * @example 
                        * &lt;div id="CoreCircularGauge"&gt;
                        * &lt;/div&gt; <br> 
                        * &lt;script&gt;	 			
			            * 	$("#CoreCircularGauge").ejCircularGauge({ scales: [{ pointers: [{ border:{color: "#1A1A1A"} }] }] });
                        * &lt;/script&gt;
			            * @memberof ejCircularGauge
			            * @instance
			            */
                        color: null,
                        /**		
			            * Specify border width for pointers of circular gauge
			            * @default 1.5
                        * @alias ejCircularGauge#scales->pointers->border->width
                        * @type {number}
			            * @example 
                        * &lt;div id="CoreCircularGauge"&gt;
                        * &lt;/div&gt; <br> 
                        * &lt;script&gt;			
			            * 	$("#CoreCircularGauge").ejCircularGauge({ scales: [{ pointers: [{ border:{width: 1.5 }}] }] });
                        * &lt;/script&gt;
			            * @memberof ejCircularGauge
    			        * @instance
			            */
                        width: 1.5
                    },
                    /**		
			        * Specify backgroundColor for the pointer of circular gauge
			        * @default null
                    * @alias ejCircularGauge#scales->pointers->backgroundColor
                    * @type {string}
			        * @example 
                    * &lt;div id="CoreCircularGauge"&gt;
                    * &lt;/div&gt; <br> 
                    * &lt;script&gt;				
			        * 	$("#CoreCircularGauge").ejCircularGauge({ scales: [{ pointers: [{ backgroundColor: "#1A1A1A" }] }] });
                    * &lt;/script&gt;
			         * @memberof ejCircularGauge
			        * @instance
			        */
                    backgroundColor: null,
                    /**		
			        * Specify pointer gradients of circular gauge
			        * @default null
                    * @alias ejCircularGauge#scales->pointers->gradients
                    * @type {Object}
			        * @example 
                    * &lt;div id="CoreCircularGauge"&gt;
                    * &lt;/div&gt; <br> 
                    * &lt;script&gt;			
			        * 	$("#CoreCircularGauge").ejCircularGauge({ scales: [{ pointers: [{ gradients: {colorInfo:[{colorStop : 0, color:"#FFFFFF"},{colorStop : 1, color:"#AAAAAA"}] }}] }] });
                    * &lt;/script&gt;
			         * @memberof ejCircularGauge
			        * @instance
			        */
                    gradients: null, //{colorInfo:[{colorStop : 0, color:"#FFFFFF"},{colorStop : 1, color:"#AAAAAA"}] }
                    /**		
			        * Specify pointer type value of circular gauge. See {@link PointerType}
			        * @default ej.datavisualization.CircularGauge.PointerType.Needle
                    * @alias ejCircularGauge#scales->pointers->type
			        * @type {enum}
			        * @example 
                    * &lt;div id="CoreCircularGauge"&gt;
                    * &lt;/div&gt; <br> 
                    * &lt;script&gt;			
			        * 	$("#CoreCircularGauge").ejCircularGauge({ scales: [{ pointers: [{ type: "marker" }] }] });
                    * &lt;/script&gt;
			         * @memberof ejCircularGauge
			        * @instance
			        */
                    type: "needle", //["needle", "marker"]
                    /**		
			        * Specify needle Style value of circular gauge. See {@link NeedleType}
			        * @default ej.datavisualization.CircularGauge.NeedleType.Triangle
                    * @alias ejCircularGauge#scales->pointers->needleType
			        * @type {enum}
			        * @example 
                    * &lt;div id="CoreCircularGauge"&gt;
                    * &lt;/div&gt; <br> 
                    * &lt;script&gt;				
			        * 	$("#CoreCircularGauge").ejCircularGauge({ scales: [{ pointers: [{ needleType: "triangle" }] }] });
                    * &lt;/script&gt;
			         * @memberof ejCircularGauge
			        * @instance
			        */
                    needleType: "triangle", //["triangle", "rectangle", "trapezoid", "arrow"],
                    /**		
			        * Specify marker Style value of circular gauge. See {@link MarkerType}
			        * @default ej.datavisualization.CircularGauge.MarkerType.Rectangle
                    * @alias ejCircularGauge#scales->pointers->markerType
			        * @type {enum}
			        * @example 
                    * &lt;div id="CoreCircularGauge"&gt;
                    * &lt;/div&gt; <br> 
                    * &lt;script&gt;			
			        * 	$("#CoreCircularGauge").ejCircularGauge({ scales: [{ pointers: [{ type: "marker", markerType: "rectangle" }] }] });
                    * &lt;/script&gt;
			         * @memberof ejCircularGauge
			        * @instance
			        */
                    markerType: "rectangle" //["rectangle", "triangle", "ellipse", "diamond", "pentagon", "circle", "slider", "pointer", "wedge", "trapezoid", "roundedrectangle"],
                }],
                /**		
			    * Specify ranges value of circular gauge
			    * @default Array
                * @alias ejCircularGauge#scales->ranges
                * @type {Array}
			    * @example 
                * &lt;div id="CoreCircularGauge"&gt;
                * &lt;/div&gt; <br> 
                * &lt;script&gt;				
			    * 	$("#CoreCircularGauge").ejCircularGauge({ scales: [{showRanges:true [{ ranges: [{ distanceFromScale: 25, size: 5}] }] }]});
                * &lt;/script&gt;
			    * @memberof ejCircularGauge
			    * @instance
			    */
                ranges: [{
                    /**		
			        * Specify distanceFromScale value for ranges of circular gauge
			        * @default 25
                    * @alias ejCircularGauge#scales->ranges->distanceFromScale
                    * @type {number}
			        * @example 
                    * &lt;div id="CoreCircularGauge"&gt;
                    * &lt;/div&gt; <br> 
                    * &lt;script&gt;				
			        * $("#CoreCircularGauge").ejCircularGauge({scales: [{showRanges: true,ranges: [{distanceFromScale: -30,startValue: 0,endValue: 70}]}]});
                    * &lt;/script&gt;
			        * @memberof ejCircularGauge
			        * @instance
                    * @remarks
                    * for reflecting the customization in Ranges, showRanges must be true.
			        */
                    distanceFromScale: 25,
                    /**		
			        * Specify size of the range value of circular gauge
			        * @default 5
                    * @alias ejCircularGauge#scales->ranges->size
                    * @type {number}
			        * @example 
                    * &lt;div id="CoreCircularGauge"&gt;
                    * &lt;/div&gt; <br> 
                    * &lt;script&gt; 			
			        * 	$("#CoreCircularGauge").ejCircularGauge({scales: [{showRanges: true,ranges: [{distanceFromScale: -30,startValue: 0,endValue: 70,size:5}]}]});
                    * &lt;/script&gt;
			         * @memberof ejCircularGauge
			        * @instance
                    * @remarks
                    * for reflecting the customization in Ranges, showRanges must be true.
			        */
                    size: 5,
                    /**		
			        * Specify placement of circular gauge. See {@link RangePlacement}
			        * @default ej.datavisualization.CircularGauge.RangePlacement.Near
                    * @alias ejCircularGauge#scales->ranges->placement
                    * @type {enum}
			        * @example 
                    * &lt;div id="CoreCircularGauge"&gt;
                    * &lt;/div&gt; <br> 
                    * &lt;script&gt;	
			        * 	$("#CoreCircularGauge").ejCircularGauge({scales: [{showRanges: true,ranges: [{distanceFromScale: -30,startValue: 0,endValue: 70,placement: "center"}]}]});
                    * &lt;/script&gt;
			         * @memberof ejCircularGauge
			        * @instance
                    * @remarks
                    * for reflecting the customization in Ranges, showRanges must be true.
			        */
                    placement: "near", //["near", "far", "center"]
                    /**		
			        * Specify startValue for ranges of circular gauge
			        * @default null
                    * @alias ejCircularGauge#scales->ranges->startValue
                    * @type {number}
			        * @example 
                    * &lt;div id="CoreCircularGauge"&gt;
                    * &lt;/div&gt; <br> 
                    * &lt;script&gt;			
			        * 	$("#CoreCircularGauge").ejCircularGauge({scales: [{showRanges: true,ranges: [{distanceFromScale: -30,startValue: 0,endValue: 70}]}]});
                    * &lt;/script&gt;
			         * @memberof ejCircularGauge
			        * @instance
                    * @remarks
                    * for reflecting the customization in Ranges, showRanges must be true.
			        */
                    startValue: null,
                    /**		
			        * Specify endValue for ranges of circular gauge
			        * @default null
                    * @alias ejCircularGauge#scales->ranges->endValue
                    * @type {number}
			        * @example 
                    * &lt;div id="CoreCircularGauge"&gt;
                    * &lt;/div&gt; <br> 
                    * &lt;script&gt;		
			        * 	$("#CoreCircularGauge").ejCircularGauge({ scales: [{showRanges:true, ranges: [{ startValue: 10, endValue: 100,distanceFromScale: -25 }] }]});
                    * &lt;/script&gt;
			         * @memberof ejCircularGauge
			        * @instance
                    * @remarks
                    * for reflecting the customization in Ranges, showRanges must be true.
			        */
                    endValue: null,
                    /**		
			        * Specify startWidth of circular gauge
			        * @default [Array.number] scale.ranges.startWidth = 10
                    * @alias ejCircularGauge#scales->ranges->startWidth
                    * @type {number}
			        * @example 
                    * &lt;div id="CoreCircularGauge"&gt;
                    * &lt;/div&gt; <br> 
                    * &lt;script&gt;		
			        * $("#CoreCircularGauge").ejCircularGauge({ scales: [{showRanges:true , ranges: [{ startValue: 10, endValue: 100,startWidth: 10,distanceFromScale: -25 }] }]});
                    * &lt;/script&gt;
			        * @memberof ejCircularGauge
			        * @instance
                    * @remarks
                    * for reflecting the customization in Ranges, showRanges must be true.
			        */
                    startWidth: 10,
                    /**		
			        * Specify endWidth for ranges of circular gauge
			        * @default 10
                    * @alias ejCircularGauge#scales->ranges->endWidth
                    * @type {number}
			        * @example 
                    * &lt;div id="CoreCircularGauge"&gt;
                    * &lt;/div&gt; <br> 
                    * &lt;script&gt;			
			        * 	$("#CoreCircularGauge").ejCircularGauge({scales: [{showRanges: true,ranges: [{distanceFromScale: -30,startValue: 0,endValue: 70}]}]});
                    * &lt;/script&gt;
			         * @memberof ejCircularGauge
			        * @instance
                    * @remarks
                    * for reflecting the customization in Ranges, showRanges must be true.
			        */
                    endWidth: 10,
                    /**		
			        * Specify range gradients of circular gauge
			        * @default null
                    * @alias ejCircularGauge#scales->ranges->gradients
                    * @type {object}
			        * @example 
                    * &lt;div id="CoreCircularGauge"&gt;
                    * &lt;/div&gt; <br> 
                    * &lt;script&gt;			
			        * 	$("#CoreCircularGauge").ejCircularGauge({ scales: [{showRanges:true , ranges: [{startValue: 10, endValue: 100, gradients: { colorInfo:[{ colorStop : 0, color:"#FFFFFF"},{colorStop : 1, color:"#AAAAAA"}] }}] }]});
                    * &lt;/script&gt;
			         * @memberof ejCircularGauge
			        * @instance
                    * @remarks
                    * for reflecting the customization in Ranges, showRanges must be true.
			        */
                    gradients: null, //{colorInfo:[{colorStop : 0, color:"#FFFFFF"},{colorStop : 1, color:"#AAAAAA"}] }
                    
                    /**		
			        * Specify opacity value for ranges of circular gauge
			        * @default null
                    * @alias ejCircularGauge#scales->ranges->opacity
                    * @type {number}
			        * @example 
                    * &lt;div id="CoreCircularGauge"&gt;
                    * &lt;/div&gt; <br> 
                    * &lt;script&gt;			
			        * 	$("#CoreCircularGauge").ejCircularGauge({ scales: [{showRanges:true , ranges: [{ startValue: 10, endValue: 100,startWidth: 10,endWidth: 10,distanceFromScale: -25,opacity: 0.5 }] }] });
                    * &lt;/script&gt;
			         * @memberof ejCircularGauge
			        * @instance
                    * @remarks
                    * for reflecting the customization in Ranges, showRanges must be true.
			        */
                    opacity: null,
                    /**		
			        * Specify backgroundColor for the ranges of circular gauge
			        * @default "#32b3c6"
                    * @alias ejCircularGauge#scales->ranges->backgroundColor
                    * @type {string}
			        * @example 
                    * &lt;div id="CoreCircularGauge"&gt;
                    * &lt;/div&gt; <br> 
                    * &lt;script&gt;			
			        * 	$("#CoreCircularGauge").ejCircularGauge({ scales: [{showRanges:true , ranges: [{ startValue: 10, endValue: 100,startWidth: 10,endWidth: 10,backgroundColor: "Red" }]  }]});
                    * &lt;/script&gt;
			         * @memberof ejCircularGauge
			        * @instance
                    * @remarks
                    * for reflecting the customization in Ranges, showRanges must be true.
			        */
                    backgroundColor: "#32b3c6",
                    /**		
			        * Specify border for ranges of circular gauge
			        * @default Object
                    * @alias ejCircularGauge#scales->ranges->border
                    * @type {object}
			        * @example  
                    * &lt;div id="CoreCircularGauge"&gt;
                    * &lt;/div&gt; <br> 
                    * &lt;script&gt;			
			        * $("#CoreCircularGauge").ejCircularGauge({ scales: [{ ranges:[{border:{color:null, width:1.5 }}]}] });
                    * &lt;/script&gt;
			        * @memberof ejCircularGauge
			        * @instance
			        */
                    border: {
                        /**		
			            * Specify border color for ranges of circular gauge
			            * @default "#32b3c6"
                        * @alias ejCircularGauge#scales->ranges->border->color
                        * @type {string}
			            * @example 
                        * &lt;div id="CoreCircularGauge"&gt;
                        * &lt;/div&gt; <br> 
                        * &lt;script&gt;			
			            * 	$("#CoreCircularGauge").ejCircularGauge({ scales: [{showRanges:true , ranges: [{ startValue: 10, endValue: 100,startWidth: 10,endWidth: 10,border:{color: "#32b3c6"} }] }] });
                        * &lt;/script&gt;
			            * @memberof ejCircularGauge
			            * @instance
                        * @remarks
                        * for reflecting the customization in Ranges, showRanges must be true.
			            */
                        color: "#32b3c6",
                        /**		
    			        * Specify border width for ranges of circular gauge
	    		        * @default 1.5
                        * @alias ejCircularGauge#scales->ranges->border->width
                        * @type {number}
			            * @example 
                        * &lt;div id="CoreCircularGauge"&gt;
                        * &lt;/div&gt; <br> 
                        * &lt;script&gt; 			
			            * 	$("#CoreCircularGauge").ejCircularGauge({ scales: [{showRanges:true , ranges: [{ startValue: 10, endValue: 100,startWidth: 10,endWidth: 10,distanceFromScale: -25,border:{width: 1.5} }] }] });
                        * &lt;/script&gt;
			            * @memberof ejCircularGauge
			            * @instance
                        * @remarks
                        * for reflecting the customization in Ranges, showRanges must be true.
			            */
                        width:1.5
                    }
                }],
                /**		
			    * Specify ticks of circular gauge
			    * @default Array
                * @alias ejCircularGauge#scales->ticks
                * @type {Array}
			    * @example 
                * &lt;div id="CoreCircularGauge"&gt;
                * &lt;/div&gt; <br> 
                * &lt;script&gt;		
			    * $("#CoreCircularGauge").ejCircularGauge({ scales: [{ ticks: [{ angle: 10, distanceFromScale: 10 }] }] });
                    * &lt;/script&gt;
			    * @memberof ejCircularGauge
			    * @instance
			    */
                ticks: [{
                    /**		
			        * Specify the angle for the ticks of circular gauge
			        * @default 0
                    * @alias ejCircularGauge#scales->ticks->angle
                    * @type {number}
			        * @example 
                    * &lt;div id="CoreCircularGauge"&gt;
                    * &lt;/div&gt; <br> 
                    * &lt;script&gt;			
			        * 	$("#CoreCircularGauge").ejCircularGauge({ scales: [{ ticks: [{ angle: 10 }] }] });
                    * &lt;/script&gt;
			         * @memberof ejCircularGauge
			        * @instance
			        */
                    angle: 0,
                    /**		
			        * Specify distanceFromScale value for ticks of circular gauge
			        * @default 0
                    * @alias ejCircularGauge#scales->ticks->distanceFromScale
                    * @type {number}
			        * @example 
                    * &lt;div id="CoreCircularGauge"&gt;
                    * &lt;/div&gt; <br> 
                    * &lt;script&gt;			
			        * 	$("#CoreCircularGauge").ejCircularGauge({ scales: [{ ticks: [{ distanceFromScale: 10 }] }] });
                    * &lt;/script&gt;
			         * @memberof ejCircularGauge
			        * @instance
			        */
                    distanceFromScale: 0,
                    /**		
			        * Specify tick color of circular gauge
			        * @default null
                    * @alias ejCircularGauge#scales->ticks->color
                    * @type {string}
			        * @example 
                    * &lt;div id="CoreCircularGauge"&gt;
                    * &lt;/div&gt; <br> 
                    * &lt;script&gt;
			        * 	$("#CoreCircularGauge").ejCircularGauge({ scales: [{ ticks: [{ color: "#777777" }] }] });
                    * &lt;/script&gt;
			         * @memberof ejCircularGauge
			        * @instance
			        */
                    color:null,
                    /**		
			        * Specify tick Style of circular gauge. See {@link TickType}
			        * @default  ej.datavisualization.CircularGauge.TickPlacement.Major
                    * @alias ejCircularGauge#scales->ticks->type
                    * @type {enum}
			        * @example 
                    * &lt;div id="CoreCircularGauge"&gt;
                    * &lt;/div&gt; <br> 
                    * &lt;script&gt;
			        * 	$("#CoreCircularGauge").ejCircularGauge({ scales: [{ ticks: [{ type: "major" }] }] });
                    * &lt;/script&gt;
			         * @memberof ejCircularGauge
			        * @instance
			        */
                    type: "major", //["major", "minor",]
                    /**		
			        * Specify tick placement of circular gauge. See {@link TickPlacement}
			        * @default ej.datavisualization.CircularGauge.TickPlacement.Near
                    * @alias ejCircularGauge#scales->ticks->placement
                    * @type {enum}
			        * @example 
                    * &lt;div id="CoreCircularGauge"&gt;
                    * &lt;/div&gt; <br> 
                    * &lt;script&gt;	
			        * 	$("#CoreCircularGauge").ejCircularGauge({ scales: [{ ticks: [{ placement: "near" }] }] });
                    * &lt;/script&gt;
			         * @memberof ejCircularGauge
			        * @instance
			        */
                    placement: "near", //[near,far,center],
                    /**		
			        * Specify tick height of circular gauge
			        * @default 16
                    * @alias ejCircularGauge#scales->ticks->height
                    * @type {number}
			        * @example 
                    * &lt;div id="CoreCircularGauge"&gt;
                    * &lt;/div&gt; <br> 
                    * &lt;script&gt;	
			        * 	$("#CoreCircularGauge").ejCircularGauge({ scales: [{ ticks: [{ height: 16 }] }] });
                    * &lt;/script&gt;
			         * @memberof ejCircularGauge
			        * @instance
			        */
                    height: 16,
                    /**		
			        * Specify tick width of circular gauge
			        * @default 3
                    * @alias ejCircularGauge#scales->ticks->width
                    * @type {number}
			        * @example 
                    * &lt;div id="CoreCircularGauge"&gt;
                    * &lt;/div&gt; <br> 
                    * &lt;script&gt;		
			        * 	$("#CoreCircularGauge").ejCircularGauge({ scales: [{ ticks: [{ width: 3 }] }] });
                    * &lt;/script&gt;
			         * @memberof ejCircularGauge
			        * @instance
			        */
                    width: 3
                }, {
                    angle: 0,
                    distanceFromScale: 0,
                    color:null,
                    type: "minor", //["major", "minor",]
                    placement: "near", //[near,far,center]
                    height: 7,
                    width: 1
                }],
                /**		
			    * Specify the text values displayed in a meaningful manner alongside the ticks of circular gauge
			    * @default Array
                * @alias ejCircularGauge#scales->labels
                * @type {Array}
			    * @example 
                * &lt;div id="CoreCircularGauge"&gt;
                * &lt;/div&gt; <br> 
                * &lt;script&gt;		
			    * $("#CoreCircularGauge").ejCircularGauge({ scales: [{ labels: [{ angle: 10, opacity: 0.4 }] }] });
                * &lt;/script&gt;
			    * @memberof ejCircularGauge
			    * @instance
			    */
                labels: [{
                    /**		
			        * Specify the angle for the labels of circular gauge
			        * @default 0
                    * @alias ejCircularGauge#scales->labels->angle
                    * @type {number}
			        * @example 
                    * &lt;div id="CoreCircularGauge"&gt;
                    * &lt;/div&gt; <br> 
                    * &lt;script&gt;			
			        * $("#CoreCircularGauge").ejCircularGauge({ scales: [{ labels: [{ angle: 10 }] }] });
                    * &lt;/script&gt;
			        * @memberof ejCircularGauge
			        * @instance
			        */
                    angle: 0,
                    /**		
			        * Specify labels autoAngle value of circular gauge
			        * @default false
                    * @alias ejCircularGauge#scales->labels->autoAngle
			        * @type {boolean}
			        * @example 
                    * &lt;div id="CoreCircularGauge"&gt;
                    * &lt;/div&gt; <br> 
                    * &lt;script&gt;		
			        * 	$("#CoreCircularGauge").ejCircularGauge({ scales: [{ labels:[{autoAngle: true}] }] });
                    * &lt;/script&gt;
			        * @memberof ejCircularGauge
			        * @instance
			        */
                    autoAngle:false,
                    /**		
			        * Specify opacity value for labels of circular gauge
			        * @default null
                    * @alias ejCircularGauge#scales->labels->opacity
                    * @type {number}
			        * @example 
                    * &lt;div id="CoreCircularGauge"&gt;
                    * &lt;/div&gt; <br> 
                    * &lt;script&gt;			
			        * 	$("#CoreCircularGauge").ejCircularGauge({ scales: [{ labels: [{ opacity: 0.4 }] }] });
                    * &lt;/script&gt;
			         * @memberof ejCircularGauge
			        * @instance
			        */
                    opacity: null,
                    /**		
			        * Specify font for labels of circular gauge
			        * @default Object
                    * @alias ejCircularGauge#scales->labels->font
                    * @type {object}
			        * @example  
                    * &lt;div id="CoreCircularGauge"&gt;
                    * &lt;/div&gt; <br> 
                    * &lt;script&gt;			
			        * 	$("#CoreCircularGauge").ejCircularGauge({ scales: [{ labels: [{ font: { size: "12px", fontFamily: "Segou", fontStyle: "Bold" } }] }] });
                    * &lt;/script&gt;
			         * @memberof ejCircularGauge
			        * @instance
			        */
                    font: {
                   /**		
                   * Specify font for labels of circular gauge
                   * @default "11px"
                   * @alias ejCircularGauge#scales->labels->font->size
                   * @type {string}
                   * @example 
                   * &lt;div id="CoreCircularGauge"&gt;
                   * &lt;/div&gt; <br> 
                   * &lt;script&gt;				
                   * 	$("#CoreCircularGauge").ejCircularGauge({ scales: [{ labels: [{ font: { size: "12px" } }] }] });
                   * &lt;/script&gt;
                   * @memberof ejCircularGauge
                   * @instance
                   */
                        size: "11px",
                  /**		
                  * Specify font for labels of circular gauge
                  * @default "Arial"
                  * @alias ejCircularGauge#scales->labels->font->fontFamily
                  * @type {String}
                  * @example 
                  * &lt;div id="CoreCircularGauge"&gt;
                  * &lt;/div&gt; <br> 
                  * &lt;script&gt;				
                  * $("#CoreCircularGauge").ejCircularGauge({ scales: [{ labels: [{ font: { fontFamily: "Arial" } }] }] });
                  * &lt;/script&gt;
                  * @memberof ejCircularGauge
                  * @instance
                  */
                        fontFamily: "Arial",
                  /**		
                  * Specify font for labels of circular gauge
                  * @default "Bold"
                  * @alias ejCircularGauge#scales->labels->font->fontStyle
                  * @type {string}
                  * @example 
                  * &lt;div id="CoreCircularGauge"&gt;
                  * &lt;/div&gt; <br> 
                  * &lt;script&gt;				
                  * $("#CoreCircularGauge").ejCircularGauge({ scales: [{ labels: [{ font: { fontStyle: "Bold" } }] }] });
                  * &lt;/script&gt;
                  * @memberof ejCircularGauge
                  * @instance
                  */
                        fontStyle: "Bold"
                    },
                    /**		
			        * Specify label color of circular gauge
			        * @default null
                    * @alias ejCircularGauge#scales->labels->color
                    * @type {string}
			        * @example 
                    * &lt;div id="CoreCircularGauge"&gt;
                    * &lt;/div&gt; <br> 
                    * &lt;script&gt;			
			        * 	$("#CoreCircularGauge").ejCircularGauge({ scales: [{ labels: [{ color: "Red" }] }] });
                    * &lt;/script&gt;
			        * @memberof ejCircularGauge
			        * @instance
			        */
                    color: null,
                    /**		
			        * Specify distanceFromScale value for labels of circular gauge
			        * @default 0
                    * @alias ejCircularGauge#scales->labels->distanceFromScale
                    * @type {number}
			        * @example 
                    * &lt;div id="CoreCircularGauge"&gt;
                    * &lt;/div&gt; <br> 
                    * &lt;script&gt;	 			
			        * 	$("#CoreCircularGauge").ejCircularGauge({ scales: [{ labels: [{ distanceFromScales: 10 }] }] });
                    * &lt;/script&gt;
			        * @memberof ejCircularGauge
			        * @instance
			        */
                    distanceFromScale: 0,
                    /**		
			        * Specify includeFirstValue of circular gauge
			        * @default true
                    * @alias ejCircularGauge#scales->labels->includeFirstValue
                    * @type {boolean}
			        * @example 
                    * &lt;div id="CoreCircularGauge"&gt;
                    * &lt;/div&gt; <br> 
                    * &lt;script&gt;				
			        * $("#CoreCircularGauge").ejCircularGauge({ scales: [{ labels: [{ includeFirstValue: false }] }] });
                    * &lt;/script&gt;
			        * @memberof ejCircularGauge
			        * @instance
			        */
                    includeFirstValue: true,
                    /**		
			        * Specify label placement of circular gauge. See {@link LabelPlacement}
			        * @default ej.datavisualization.CircularGauge.LabelPlacement.Near
                    * @alias ejCircularGauge#scales->labels->placement
                    * @type {enum}
			        * @example 
                    * &lt;div id="CoreCircularGauge"&gt;
                    * &lt;/div&gt; <br> 
                    * &lt;script&gt;				
			        * 	$("#CoreCircularGauge").ejCircularGauge({ scales: [{ labels: [{ placement: "near" }] }] });
                    * &lt;/script&gt;
			        * @memberof ejCircularGauge
			        * @instance
			        */
                    placement: "near", //[near,far,center],
                    /**		
                    * Specify label Style of circular gauge.	See {@link LabelType}
                    * @default ej.datavisualization.CircularGauge.LabelType.Major
                    * @alias ejCircularGauge#scales->labels->type
                    * @type {enum}
                    * @example 
                    * &lt;div id="CoreCircularGauge"&gt;
                    * &lt;/div&gt; <br> 
                    * &lt;script&gt;			
                    * 	$("#CoreCircularGauge").ejCircularGauge({ scales: [{ labels: [{ type: "major" }] }] });
                  * &lt;/script&gt;
                    * @memberof ejCircularGauge
                    * @instance
                    */
                    type: "major",//["major", "minor",]
                    /**		
                    * Specify unitText of circular gauge
                    * @default ""
                    * @alias ejCircularGauge#scales->labels->unitText
                    * @type {string}
                    * @example 
                    * &lt;div id="CoreCircularGauge"&gt;
                    * &lt;/div&gt; <br> 
                    * &lt;script&gt;			
                    * 	$("#CoreCircularGauge").ejCircularGauge({ scales: [{ labels: [{ unitText: "kmph" }] }] });
                  * &lt;/script&gt;
                    * @memberof ejCircularGauge
                    * @instance
                    */
                    unitText: "",
                    /**		
                    * Specify unitTextPosition of circular gauge.	See {@link UnitTextPosition}
                    * @default ej.datavisualization.CircularGauge.LabelType.Back
                    * @alias ejCircularGauge#scales->labels->unitTextPosition
                    * @type {enum}
                    * @example 
                    * &lt;div id="CoreCircularGauge"&gt;
                    * &lt;/div&gt; <br> 
                    * &lt;script&gt;	 			
                    * 	$("#CoreCircularGauge").ejCircularGauge({ scales: [{ labels: [{ unitText: "kmph",unitTextPosition: "front" }] }] });
                  * &lt;/script&gt;
                    * @memberof ejCircularGauge
                    * @instance
                    */
                    unitTextPosition: "back",//["front","back"]

                }],
                /**		
                * Specify representating state of circular gauge
                * @default Array
                * @alias ejCircularGauge#scales->indicators
                * @type {Array}
                * @example 
                * &lt;div id="CoreCircularGauge"&gt;
                * &lt;/div&gt; <br> 
                * &lt;script&gt;				
                * 	$("#CoreCircularGauge").ejCircularGauge({ scales: [{showIndicators:true,indicators: [{
                *   height: 30,width: 10,type: "circle",value: 0,position: { x: 185, y: 300 },
                *   stateRanges: [{ endValue: 70, startValue: 0, backgroundColor: "#5DF243", borderColor: "#5DF243", text: "", textColor: "#870505" },
                *   { endValue: 200, startValue: 70, backgroundColor: "#145608", borderColor: "#145608", text: "", textColor: "#870505" }]}]}]});
                * &lt;/script&gt;
                * @memberof ejCircularGauge
                * @instance
                * @remarks
                * for reflecting the customization in Indicators, showIndicators must be true.
                */
                indicators: [{
                    /**		
                    * Specify indicator height of circular gauge
                    * @default 15
                    * @alias ejCircularGauge#scales->indicators->height
                    * @type {number}
                    * @example 
                    * &lt;div id="CoreCircularGauge"&gt;
                    * &lt;/div&gt; <br> 
                    * &lt;script&gt;				
                    * 	$("#CoreCircularGauge").ejCircularGauge({ scales: [{showIndicators:true,indicators: [{
                    *   height: 30,type: "circle",value: 0,position: { x: 185, y: 300 },
                    *   stateRanges: [{ endValue: 70, startValue: 0, backgroundColor: "#5DF243" },
                    *   { endValue: 200, startValue: 70, backgroundColor: "#145608"}]}]}]});
                    * &lt;/script&gt;
                    * @memberof ejCircularGauge
                    * @instance
                    * @remarks
                    * for reflecting the customization in Indicators, showIndicators must be true.
                    */
                    height: 15,
                    /**		
                    * Specify indicator width of circular gauge
                    * @default 15
                    * @alias ejCircularGauge#scales->indicators->width
                    * @type {number}
                    * @example 
                    * &lt;div id="CoreCircularGauge"&gt;
                    * &lt;/div&gt; <br> 
                    * &lt;script&gt;				
                    * 	$("#CoreCircularGauge").ejCircularGauge({ scales: [{showIndicators:true,indicators: [{
                    *   width: 30,type: "circle",value: 0,position: { x: 185, y: 300 },
                    *   stateRanges: [{ endValue: 70, startValue: 0, backgroundColor: "#5DF243" },
                    *   { endValue: 200, startValue: 70, backgroundColor: "#145608"}]}]}]});
                    * &lt;/script&gt;
                    * @memberof ejCircularGauge
                    * @instance
                    * @remarks
                    * for reflecting the customization in Indicators, showIndicators must be true.
                    */
                    width: 15,
                    /**		
                    * Specify indicator style of circular gauge.	See {@link IndicatorType}
                    * @default ej.datavisualization.CircularGauge.IndicatorType.Circle
                    * @alias ejCircularGauge#scales->indicators->type
                    * @type {enum}
                    * @example 
                    * &lt;div id="CoreCircularGauge"&gt;
                    * &lt;/div&gt; <br> 
                    * &lt;script&gt;				
                    * 	$("#CoreCircularGauge").ejCircularGauge({ scales: [{showIndicators:true,indicators: [{
                    *   width: 30,type: "circle",value: 0,position: { x: 185, y: 300 },
                    *   stateRanges: [{ endValue: 70, startValue: 0, backgroundColor: "#5DF243" },
                    *   { endValue: 200, startValue: 70, backgroundColor: "#145608"}]}]}]});
                    * &lt;/script&gt;
                    * @memberof ejCircularGauge
                    * @instance
                    * @remarks
                    * for reflecting the customization in Indicators, showIndicators must be true.
                    */
                    type: "circle", //["rectangle", "circle","roundedrectangle", "text"]
                    /**		
                    * Specify indicator value of circular gauge
                    * @default 0
                    * @alias ejCircularGauge#scales->indicators->value
                    * @type {number}
                    * @example 
                    * &lt;div id="CoreCircularGauge"&gt;
                    * &lt;/div&gt; <br> 
                    * &lt;script&gt;				
                    * 	$("#CoreCircularGauge").ejCircularGauge({ scales: [{showIndicators:true,indicators: [{
                    *   width: 30,type: "circle",value: 0,position: { x: 185, y: 300 },
                    *   stateRanges: [{ endValue: 70, startValue: 0, backgroundColor: "#5DF243" },
                    *   { endValue: 200, startValue: 70, backgroundColor: "#145608"}]}]}]});
                    * &lt;/script&gt;
                    * @memberof ejCircularGauge
                    * @instance
                    * @remarks
                    * for reflecting the customization in Indicators, showIndicators must be true.
                    */
                    value: 0,
                    /**		
                    * Specify imageUrl of circular gauge
                    * @default null
                    * @alias ejCircularGauge#scales->indicators->imageUrl
                    * @type {string}
                    * @example 
                    * &lt;div id="CoreCircularGauge"&gt;
                    * &lt;/div&gt; <br> 
                    * &lt;script&gt;			
                    * 	$("#CoreCircularGauge").ejCircularGauge({ scales: [{showIndicators:true,indicators: [{
                    *   width: 30,type: "image",value: 0,imageUrl:"Sun.jpeg",position: { x: 185, y: 300 },
                    *   stateRanges: [{ endValue: 70, startValue: 0, backgroundColor: "#5DF243" },
                    *   { endValue: 200, startValue: 70, backgroundColor: "#145608"}]}]}]});
                    * &lt;/script&gt;
                    * @memberof ejCircularGauge
                    * @instance
                    * @remarks
                    * for reflecting the customization in Indicators, showIndicators must be true.
                    */
                    imageUrl: null,
                    /**		
                    * Specify position of circular gauge
                    * @default Object
                    * @alias ejCircularGauge#scales->indicators->position
                    * @type {object}
                    * @example 
                    * &lt;div id="CoreCircularGauge"&gt;
                    * &lt;/div&gt; <br> 
                    * &lt;script&gt;				
                    * 	$("#CoreCircularGauge").ejCircularGauge({ scales: [{showIndicators:true,indicators: [{
                    *   width: 30,type: "circle",value: 0,position: { x: 185, y: 150 },
                    *   stateRanges: [{ endValue: 70, startValue: 0, backgroundColor: "#5DF243" },
                    *   { endValue: 200, startValue: 70, backgroundColor: "#145608"}]}]}]});
                    * &lt;/script&gt;
                    * @memberof ejCircularGauge
                    * @instance
                    * @remarks
                    * for reflecting the customization in Indicators, showIndicators must be true.
                    */
                    position: {
                        /**		
                        * Specify x-axis of position of circular gauge
                        * @default 0
                        * @alias ejCircularGauge#scales->indicators->position->x
                        * @type {number}
                        * @example 
                        * &lt;div id="CoreCircularGauge"&gt;
                        * &lt;/div&gt; <br> 
                        * &lt;script&gt;	 			
                        * 	$("#CoreCircularGauge").ejCircularGauge({ scales: [{showIndicators:true,indicators: [{
                        *   width: 30,type: "circle",value: 0,position: { x: 185,y:0 },
                        *   stateRanges: [{ endValue: 70, startValue: 0, backgroundColor: "#5DF243" },
                        *   { endValue: 200, startValue: 70, backgroundColor: "#145608"}]}]}]});
                        * &lt;/script&gt;
                        * @memberof ejCircularGauge
                        * @instance
                        * @remarks
                        * for reflecting the customization in Indicators, showIndicators must be true.
                        */
                        x: 0,
                        /**		
                        * Specify y-axis of position of circular gauge
                        * @default 0
                        * @alias ejCircularGauge#scales->indicators->position->y
                        * @type {number}
                        * @example 
                        * &lt;div id="CoreCircularGauge"&gt;
                        * &lt;/div&gt; <br> 
                        * &lt;script&gt;				
                        * 	$("#CoreCircularGauge").ejCircularGauge({ scales: [{showIndicators:true,indicators: [{
                        *   width: 30,type: "circle",value: 0,position: { x:0,y: 185 },
                        *   stateRanges: [{ endValue: 70, startValue: 0, backgroundColor: "#5DF243" },
                        *   { endValue: 200, startValue: 70, backgroundColor: "#145608"}]}]}]});
                        * &lt;/script&gt;
                        * @memberof ejCircularGauge
                        * @instance
                        * @remarks
                        * for reflecting the customization in Indicators, showIndicators must be true.
                        */
                        y: 0
                    },
                    /**		
                    * Specify the various states of circular gauge
                    * @default Array
                    * @alias ejCircularGauge#scales->indicators->stateRanges
                    * @type {Array}
                    * @example 
                    * &lt;div id="CoreCircularGauge"&gt;
                    * &lt;/div&gt; <br> 
                    * &lt;script&gt;			
                    * 	$("#CoreCircularGauge").ejCircularGauge({ scales: [{showIndicators:true,indicators: [{
                    *   width: 30,type: "circle",value: 0,position: { x: 185, y:300 },
                    *   stateRanges: [{ endValue: 70, startValue: 0, backgroundColor: "#5DF243" },
                    *   { endValue: 200, startValue: 70, backgroundColor: "#145608"}]}]}]});
                    * &lt;/script&gt;
                    * @memberof ejCircularGauge
                    * @instance
                        * @remarks
                        * for reflecting the customization in Indicators, showIndicators must be true.
                    */
                    stateRanges: [{
                        /**		
                        * Specify end value for each specified state of circular gauge
                        * @default 0
                        * @alias ejCircularGauge#scales->indicators->stateRanges->endValue
                        * @type {number}
                        * @example 
                        * &lt;div id="CoreCircularGauge"&gt;
                        * &lt;/div&gt; <br> 
                        * &lt;script&gt;		
                        * 	$("#CoreCircularGauge").ejCircularGauge({ scales: [{showIndicators:true,indicators: [{
                        *   width: 30,type: "circle",value: 0,position: { x: 185, y:300 },
                        *   stateRanges: [{ endValue: 70, startValue: 0, backgroundColor: "#5DF243" },
                        *   { endValue: 200, startValue: 70, backgroundColor: "#145608"}]}]}]});
                        * &lt;/script&gt;
                        * @memberof ejCircularGauge
                        * @instance
                        * @remarks
                        * for reflecting the customization in Indicators, showIndicators must be true.
                        */
                        endValue: 0,
                        /**		
                        * Specify start value for each specified state of circular gauge
                        * @default 0
                        * @alias ejCircularGauge#scales->indicators->stateRanges->startValue
                        * @type {number}
                        * @example 
                        * &lt;div id="CoreCircularGauge"&gt;
                        * &lt;/div&gt; <br> 
                        * &lt;script&gt;	 			
                        * 	$("#CoreCircularGauge").ejCircularGauge({ scales: [{showIndicators:true,indicators: [{
                        *   width: 30,type: "circle",value: 0,position: { x: 185, y:300 },
                        *   stateRanges: [{ endValue: 70, startValue: 0, backgroundColor: "#5DF243" },
                        *   { endValue: 200, startValue: 70, backgroundColor: "#145608"}]}]}]});
                        * &lt;/script&gt;
                        * @memberof ejCircularGauge
                        * @instance
                        * @remarks
                        * for reflecting the customization in Indicators, showIndicators must be true.
                        */
                        startValue: 0,
                        /**		
                        * Specify backgroundColor for indicator of circular gauge
                        * @default null
                        * @alias ejCircularGauge#scales->indicators->stateRanges->backgroundColor
                        * @type {string}
                        * @example 
                        * &lt;div id="CoreCircularGauge"&gt;
                        * &lt;/div&gt; <br> 
                        * &lt;script&gt;		
                        * 	$("#CoreCircularGauge").ejCircularGauge({ scales: [{showIndicators:true,indicators: [{
                        *   width: 30,type: "circle",value: 0,position: { x: 185, y:300 },
                        *   stateRanges: [{ endValue: 70, startValue: 0, backgroundColor: "Red" },
                        *   { endValue: 200, startValue: 70, backgroundColor: "Yellow"}]}]}]});
                        * &lt;/script&gt;
                        * @memberof ejCircularGauge
                        * @instance
                        * @remarks
                        * for reflecting the customization in Indicators, showIndicators must be true.
                        */
                        backgroundColor: null,
                        /**		
                        * Specify borderColor for indicator of circular gauge
                        * @default null
                        * @alias ejCircularGauge#scales->indicators->stateRanges->borderColor
                        * @type {string}
                        * @example 
                        * &lt;div id="CoreCircularGauge"&gt;
                        * &lt;/div&gt; <br> 
                        * &lt;script&gt;	
                        * 	$("#CoreCircularGauge").ejCircularGauge({ scales: [{showIndicators:true,indicators: [{
                        *   width: 30,type: "circle",value: 0,position: { x: 185, y:300 },
                        *   stateRanges: [{ endValue: 70, startValue: 0, backgroundColor: "#5DF243",borderColor:"Red" },
                        *   { endValue: 200, startValue: 70, backgroundColor: "#145608", borderColor:"yellow"}]}]}]});
                        * &lt;/script&gt;
                        * @memberof ejCircularGauge
                        * @instance
                        * @remarks
                        * for reflecting the customization in Indicators, showIndicators must be true.
                        */
                        borderColor: null,
                        /**		
                        * Specify value of the text as the indicator when the indicator style is set with the value "text" of circular gauge
                        * @default ""
                        * @alias ejCircularGauge#scales->indicators->stateRanges->text
                        * @type {string}
                        * @example 
                        * &lt;div id="CoreCircularGauge"&gt;
                        * &lt;/div&gt; <br> 
                        * &lt;script&gt;	
                        * $("#CoreCircularGauge").ejCircularGauge({
                        * scales: [{showIndicators: true, indicators: [{
                        * width: 30, type: "text", value: 0, position: { x: 185, y: 300 },
                        * stateRanges: [{ endValue: 70, startValue: 0, text: "staterange1" },
                        * { endValue: 200, startValue: 70, text: "staterange1" }]}]}]});
                        * &lt;/script&gt;
                        * @memberof ejCircularGauge
                        * @instance
                        * @remarks
                        * for reflecting the customization in Indicators, showIndicators must be true.
                        */
                        text: "",
                        /**		
                        * Specify value of the textColor as the indicator when the indicator style is set with the value "text" of circular gauge
                        * @default null
                        * @alias ejCircularGauge#scales->indicators->stateRanges->textColor
                        * @type {string}
                        * @example 
                        * &lt;div id="CoreCircularGauge"&gt;
                        * &lt;/div&gt; <br> 
                        * &lt;script&gt;			
                        * $("#CoreCircularGauge").ejCircularGauge({
                        * scales: [{showIndicators: true, indicators: [{
                        * width: 30, type: "text", value: 0, position: { x: 185, y: 300 },
                        * stateRanges: [{ endValue: 70, startValue: 0, text: "staterange1", textColor: "Yellow" },
                        * { endValue: 200, startValue: 70, text: "staterange1", textColor: "Yellow" }]}]}]});
                        * &lt;/script&gt;
                        * @memberof ejCircularGauge
                        * @instance
                        * @remarks
                        * for reflecting the customization in Indicators, showIndicators must be true.
                        */
                        textColor: null,
                        /**		
                        * Specify value of the font as the indicator when the indicator style is set with the value "text" of circular gauge
                        * @default null
                        * @alias ejCircularGauge#scales->indicators->stateRanges->font
                        * @type {object}
                        * @example 
                        * &lt;div id="CoreCircularGauge"&gt;
                        * &lt;/div&gt; <br> 
                        * &lt;script&gt;		
                        * $("#CoreCircularGauge").ejCircularGauge({
                        * scales: [{showIndicators: true, indicators: [{
                        * width: 30, type: "text", value: 0, position: { x: 185, y: 300 },
                        * stateRanges: [{ endValue: 70, startValue: 0, text: "staterange1" },
                        * { endValue: 200, startValue: 70, text: "staterange1", font: { size: "11px", fontFamily: "Arial", fontStyle: "Bold" } }]}]}]});
                        * &lt;/script&gt;
                        * @memberof ejCircularGauge
                        * @instance
                        * @remarks
                        * for reflecting the customization in Indicators, showIndicators must be true.
                        */
                        font: null
                    }]
                }],

                //subGaugeHeight,subGaugeWidth,position:{X:5,Y:10},controlID
                /**		
                * Specify subGauge of circular gauge
                * @default Array
                * @alias ejCircularGauge#scales->subGauges
                * @type {Array}
                * @example 
                * &lt;div id="CoreCircularGauge"&gt;
                * &lt;/div&gt; <br> 
                * &lt;div id="subGauge1"&gt;
                * &lt;/div&gt; <br> 
                * &lt;script&gt;	
                *$("#subGauge1").ejCircularGauge({backgroundColor: "#f5b43f",scales: [{radius: 150}]});
                *$("#CoreCircularGauge").ejCircularGauge({height: 500,width: 500,scales: [{radius: 250,subGauges: [{controlID: "subGauge1",height: 200,width: 200,position: { x: 200, y: 150 }}]}]});
                * &lt;/script&gt;
                * @memberof ejCircularGauge
                * @instance
                */
                subGauges: [{
                    /**		
                    * Specify subGauge Height of circular gauge
                    * @default 150
                    * @alias ejCircularGauge#scales->subGauges->height
                    * @type {number}
                    * @example 
                    * &lt;div id="CoreCircularGauge"&gt;
                    * &lt;/div&gt; <br> 
                    * &lt;div id="subGauge1"&gt;
                    * &lt;/div&gt; <br> 
                    * &lt;script&gt;	
                    * $("#subGauge1").ejCircularGauge({backgroundColor: "#f5b43f",scales: [{radius: 150}]});
                    * $("#CoreCircularGauge").ejCircularGauge({height: 500,width: 500,scales: [{radius: 250,subGauges: [{controlID: "subGauge1",height: 400,width: 200,position: { x: 200, y: 100 }}]}]});
                    * &lt;/script&gt;
                    * @memberof ejCircularGauge
                    * @instance
                    */
                    height: 150,
                    /**		
                    * Specify subGauge Width of circular gauge
                    * @default 150
                    * @alias ejCircularGauge#scales->subGauges->width
                    * @type {number}
                    * @example 
                    * &lt;div id="CoreCircularGauge"&gt;
                    * &lt;/div&gt; <br> 
                    * &lt;div id="subGauge1"&gt;
                    * &lt;/div&gt; <br> 
                    * &lt;script&gt;	
                    *$("#subGauge1").ejCircularGauge({backgroundColor: "#f5b43f",scales: [{radius: 150}]});
                    *$("#CoreCircularGauge").ejCircularGauge({height: 500,width: 500,scales: [{radius: 250,subGauges: [{controlID: "subGauge1",height: 200,width: 300,position: { x: 200, y: 150 }}]}]});
                    * &lt;/script&gt;
                    * @memberof ejCircularGauge
                    * @instance
                    */
                    width: 150,
                    /**		
                    * Specify position for sub-gauge of circular gauge
                    * @default Object
                    * @alias ejCircularGauge#scales->subGauges->position
                    * @type {object}
                    * @example 
                    * &lt;div id="CoreCircularGauge"&gt;
                    * &lt;/div&gt; <br> 
                    * &lt;div id="subGauge1"&gt;
                    * &lt;/div&gt; <br> 
                    * &lt;script&gt;	
                    *$("#subGauge1").ejCircularGauge({backgroundColor: "#f5b43f",scales: [{radius: 150}]});
                    *$("#CoreCircularGauge").ejCircularGauge({height: 500,width: 500,scales: [{radius: 250,subGauges: [{controlID: "subGauge1",height: 200,width: 200,position: { x: 200, y: 150 }}]}]});
                    * &lt;/script&gt;
                    * @memberof ejCircularGauge
                    * @instance
                    */
                    position: {
                        /**		
                        * Specify x-axis position for sub-gauge of circular gauge
                        * @default 0
                        * @alias ejCircularGauge#scales->subGauges->position->x
                        * @type {number}
                        * @example 
                        * &lt;div id="CoreCircularGauge"&gt;
                        * &lt;/div&gt; <br> 
                        * &lt;div id="subGauge1"&gt;
                        * &lt;/div&gt; <br> 
                        * &lt;script&gt;	
                        *$("#subGauge1").ejCircularGauge({backgroundColor: "#f5b43f",scales: [{radius: 150}]});
                        *$("#CoreCircularGauge").ejCircularGauge({height: 500,width: 500,scales: [{radius: 250,subGauges: [{controlID: "subGauge1",height: 200,width: 200,position: { x: 200, y: 0 }}]}]});
                        * &lt;/script&gt;
                        * @memberof ejCircularGauge
                        * @instance
                        */
                        x: 0,
                        /**		
                        * Specify y-axis position for sub-gauge of circular gauge
                        * @default 0
                        * @alias ejCircularGauge#scales->subGauges->position->y
                        * @type {number}
                        * @example 
                        * &lt;div id="CoreCircularGauge"&gt;
                        * &lt;/div&gt; <br> 
                        * &lt;div id="subGauge1"&gt;
                        * &lt;/div&gt; <br> 
                        * &lt;script&gt;	
                        *$("#subGauge1").ejCircularGauge({backgroundColor: "#f5b43f",scales: [{radius: 150}]});
                        *$("#CoreCircularGauge").ejCircularGauge({height: 500,width: 500,scales: [{radius: 250,subGauges: [{controlID: "subGauge1",height: 200,width: 200,position: { x: 0, y: 150 }}]}]});
                        * &lt;/script&gt;
                        * @memberof ejCircularGauge
                        * @instance
                        */
                        y: 0
                    }
                }],
                //textAngle ,value:"CustomText", position:{X:5,Y:10}, color, font: { size: "11px", fontFamily: "Arial", fontStyle: "Bold" },
                customLabels: []
            };
        },
        dataTypes: {
            scales: "data"
        },
        observables: ["value", "minimum", "maximum"],
        value: ej.util.valueFunction("value"),
        minimum: ej.util.valueFunction("minimum"),
        maximum: ej.util.valueFunction("maximum"),

        // set model function
		/**
         * To configure the properties at runtime using SetModel		
		 * @private
         */	 
        _setModel: function (options) {
            var option;
            for (option in options) {
                switch (option) {
                    case "height": this.model.height = options[option]; break;
                    case "width": this.model.width = options[option]; break;
                    case "radius": this.model.radius = options[option]; break;
                    case "frame": $.extend(this.model.frame, options[option]);
                    case "backgroundColor": this.model.backgroundColor = options[option]; break;
                    case "interiorGradient": this.model.interiorGradient = options[option]; break;
                    case "readOnly": this.model.readOnly = options[option]; break;
                    case "theme": this.model.theme = options[option]; break;
                    case "isRadialGradient": this.model.isRadialGradient = options[option]; break;
                    case "value":
                        (this.value() == "") && this.value(0);
                        for (var i = 0; this.model.scales[i] != null; i++) {
                            for (j = 0; this.model.scales[i].pointers[j] != null; j++) {
                                this.model.scales[i].pointers[j].value = parseFloat(this.value());
                            }
                        }
                        break;
                    case "minimum":
                        (this.minimum() == "") && this.minimum(0);
                        for (var i = 0; this.model.scales[i] != null; i++) {
                            this.model.scales[i].minimum = parseInt(this.minimum());
                        }
                        break;
                    case "maximum":
                        (this.maximum() == "") && this.maximum(100);
                        for (var i = 0; this.model.scales[i] != null; i++) {
                            this.model.scales[i].maximum = parseInt(this.maximum());
                        }
                        break;
                    case "scales":
                        $.extend(true, this.model.scales, options[option]);
                        break;
                }
            }
            this._init();
        },

        /**
           * destroy the circular gauge widget.
           * all events bound using this._on will be unbind automatically and bring the control to pre-init state.
           * @alias destroy
           * @return jQuery
           * @example 
           * &lt;div id="CoreCircularGauge"&gt;&lt;/div&gt; <br> 
           * &lt;script&gt;
           * $("#CoreCircularGauge").ejCircularGauge();
           * var circulargaugeObj = $("#CoreCircularGauge").data("ejCircularGauge");
           * circulargaugeObj.destroy();
           * &lt;/script&gt;
           * @memberof ejCircularGauge
           * @instance
        */
        _destroy: function () {
            this._unWireEvents();
            this.element.empty().removeClass("e-widget");
        },


        // constructor function
		/**
         * Constructor to initialize the gauge	
		 * @private
         */	 
        _init: function () {
            //this._isHalfCircle = false;
            this._scalesInitialize();
            this._trigger("load");
            this._setTheme();
            this._initialize();
            if (!this.model.readOnly)
                this._wireEvents();
            this._onWindowResize();
        },
		/**
         * Invoke when window size changes 
		 * @private
         */	
        _onWindowResize: function () {
            if (this.model.enableResize) {
                proxy = this;
                if (!this.isDevice())
                    this._on($(window), "resize", proxy.resizeCanvas);
                else
                    this._on($(window), "orientationchange", proxy.resizeCanvas);
            }
        },
		/**
         * To check whether it is a device or not
		 * @private
         */	
        isDevice: function () {
            return (/mobile|tablet|android|kindle/i.test(navigator.userAgent.toLowerCase()));
        },
		/**
         * To initialize the gauge object
		 * @private
         */	
        _initialize: function () {
            this._initObject(this);
            this._drawOuterLayer();
            this._drawScales();
            if (this.model.renderComplete)
                this._onRenderComplete();
        },
		/**
         * To initialize the scales
		 * @private
         */	
        _scalesInitialize: function () {
            var proxy = this;
            if (this.model.scales != null) {
                $.each(this.model.scales, function (index, element) {
                    element = proxy._checkArrayObject(element, index);
                    var obj = proxy._defaultScaleValues();
                    $.extend(obj, element);
                    $.extend(element, obj);
                });
            }
            else {
                this.model.scales = [this._defaultScaleValues()];
            }
        },
		/**
         * To check the each array element to load
		 * @private
         */	
        _checkArrayObject: function (element, initialName) {
            var proxy = this;
            $.each(element, function (name, innerElement) {
                if (innerElement instanceof Array) {
                    proxy._checkArrayObject(innerElement, name);
                }
                else if (innerElement != null && typeof innerElement == "object") {
                    var allObjects = proxy._defaultScaleValues();
                    proxy._LoadIndividualDefaultValues(innerElement, allObjects, (typeof name === "number") ? initialName : name);
                }
            });
            return element;
        },
		/**
         * To Load individual default values
		 * @private
         */	
        _LoadIndividualDefaultValues: function (obj, allObjects, name) {
            var defaultObj = null;
            var proxy = this;
            $.each(allObjects, function (n, element) {
                if (name == n) {
                    defaultObj = element;
                    return;
                }
            });
            if (defaultObj instanceof Array) defaultObj = defaultObj[0];

            $.each(obj, function (objName, ele) {
                if (ele instanceof Array) {
                    proxy._checkArrayObject(ele, name);
                }
                else if (ele != null && typeof ele == "object") {
                    proxy._LoadIndividualDefaultValues(ele, ele, (typeof objName === "number") ? name : objName);
                }
            });

            $.extend(defaultObj, obj);
            $.extend(obj, defaultObj);
            return obj;
        },
		/**
         * To initialize the gauge object
		 * @private
         */	
        _initObject: function (element) {
            this.element.addClass("e-widget");
            element.GaugeEl = this.element;
            for (var i = 0; this.model.scales[i] != null; i++) {
                if (this.model.scales[i].minimum == null)
                    this.model.scales[i].minimum = this.minimum();
                if (this.model.scales[i].maximum == null)
                    this.model.scales[i].maximum = this.maximum();
                for (j = 0; this.model.scales[i].pointers[j] != null; j++) {
                    if (this.model.scales[i].pointers[j].value == null)
                        this.model.scales[i].pointers[j].value = this.value();
                }
            }
            if (element.canvasEl)
                element.canvasEl.remove();
            else
                element.canvasEl = $("<canvas></canvas>");
            element.GaugeEl.append(element.canvasEl);
            element.canvasEl.attr("role", "presentation");
            initialDivWidth = element.GaugeEl.width();
            initialDivHeight = element.GaugeEl.height();
            element.canvasEl[0].setAttribute("width", element.model.width);
            element.canvasEl[0].setAttribute("height", element.model.height);
            element.centerX = element.canvasEl[0].width / 2;
            element.centerY = element.canvasEl[0].height / 2;
            if (this._isHalfCircle && this._isHalfCircle()) {
                if ((element.model.frame.halfCircleFrameEndAngle - element.model.frame.halfCircleFrameStartAngle) >= 180) {
                    if ((element.model.frame.halfCircleFrameStartAngle == 180) && (element.model.frame.halfCircleFrameEndAngle == 360)) {
                        element.centerY = element.canvasEl[0].height;
                    }
                    else if ((element.model.frame.halfCircleFrameStartAngle == 0) && (element.model.frame.halfCircleFrameEndAngle == 180)) {
                        element.centerY = 0;
                    }
                    else if ((element.model.frame.halfCircleFrameStartAngle == 90) && (element.model.frame.halfCircleFrameEndAngle == 270)) {
                        element.centerX = element.canvasEl[0].width;
                    }
                    else if ((element.model.frame.halfCircleFrameStartAngle == 270) && (element.model.frame.halfCircleFrameEndAngle == 90)) {
                        element.centerX = 0;
                    }
                }
                else if ((element.model.frame.halfCircleFrameEndAngle - element.model.frame.halfCircleFrameStartAngle) <= 90) {
                    if ((element.model.frame.halfCircleFrameStartAngle >= 0) && (element.model.frame.halfCircleFrameStartAngle <= 180)) {
                        element.centerX = 0;
                        element.centerY = 0;
                    }
                    if ((element.model.frame.halfCircleFrameStartAngle == 90)) {
                        element.centerX = element.canvasEl[0].width;
                    }
                    if (element.model.frame.halfCircleFrameStartAngle == 0 || element.model.frame.halfCircleFrameStartAngle == 270) {
                        element.centerX = 0;
                    }
                    if ((element.model.frame.halfCircleFrameStartAngle == 180) && (element.model.frame.halfCircleFrameEndAngle == 270)) {
                        element.centerX = element.canvasEl[0].width;
                        element.centerY = element.canvasEl[0].height / 2;
                    }
                }
            }
            var elem = element.canvasEl[0];
            if (typeof window.G_vmlCanvasManager != "undefined") {
                elem = window.G_vmlCanvasManager.initElement(elem);
            }
            if (!elem || !elem.getContext) {
                return;
            }
            element.contextEl = element.canvasEl[0].getContext("2d");
        },
		/**
         * Wiring the events to Circular gauge control		
		 * @private
         */		
        _wireEvents: function () {
            this.startEv = 'mousedown';
            this.moveEv = 'mousemove';
            this.endEv = 'mouseup';
            this.onMouseMoveHandler = $.proxy(this._onMouseMove, this);
            this.onMouseUpHandler = $.proxy(this._onMouseUp, this);
            this.onMouseDownHandler = $.proxy(this._onMouseDown, this);
            $(this.canvasEl).bind(this.startEv, this.onMouseDownHandler);
        },
		/**
         * To unWiring the events to Circular gauge control		
		 * @private
         */	
        _unWireEvents: function () {
            $(this.canvasEl).unbind(this.startEv, this.onMouseDownHandler);
        },
		/**
         * Section For handle the Mouse Down event
		 * @private
         */	
        _onMouseDown: function (e) {
            this._blockDefaultActions(e);
            var startPoint = { "x": this.centerX, "y": this.centerY };
            var endPoint = { "x": e.pageX - $(this.canvasEl).offset().left, "y": e.pageY - $(this.canvasEl).offset().top };
            var indegree = 180 * this._getCirucumferenceAngle(startPoint, endPoint) / Math.PI;
            var self = this;
            $.each(this.model.scales, function (index, element) {
                self.scaleIndex = index;
                if (element.pointers != null) {
                    self.pointerEl = element.pointers;
                    $.each(element.pointers, function (pointerIndex, pointerElement) {
                        if (self._isHalfCircle())
                            element.showBackNeedle = false;
                        angle = self._getAngle(pointerElement.value);
                        if (angle > 360)
                            angle = angle - 360;
                        var greatervalue = angle + pointerElement.width;
                        var lesservalue = angle - pointerElement.width;
                        radius = Math.sqrt((endPoint.x - startPoint.x) * (endPoint.x - startPoint.x) + (endPoint.y - startPoint.y) * (endPoint.y - startPoint.y));
                        if (self._isBetween(lesservalue, greatervalue, indegree) && radius <= pointerElement.length) {
                                self._onMouseClick(angle, element.value);
                            self.activeElement = pointerElement;
                            $(document).bind(self.moveEv, self.onMouseMoveHandler);
                            $(document).bind(self.endEv, self.onMouseUpHandler);
                        }
                    });
                }
            });
        },
		/**
         * Section For handle the Mouse Up event
		 * @private
         */	
        _onMouseUp: function (e) {
            $(document).unbind(self.moveEv, self.onMouseMoveHandler);
            $(document).unbind(self.endEv, self.onMouseUpHandler);
            this._onMouseClickUp(this._getAngle(this.activeElement.value), this.activeElement.value);
        },
		/**
         * Section For handle the Mouse Move event
		 * @private
         */	
        _onMouseMove: function (e) {
            this._blockDefaultActions(e);
            var startPoint = { "x": this.centerX, "y": this.centerY };
            var endPoint = { "x": e.pageX - $(this.canvasEl).offset().left, "y": e.pageY - $(this.canvasEl).offset().top };
            var indegree = 180 * this._getCirucumferenceAngle(startPoint, endPoint) / Math.PI;
            if (indegree < this.scaleEl[this.scaleIndex].startAngle && !this._isHalfCircle())
                indegree = indegree + 360;
            //if ((this._isHalfCircle() && this._isBetween(this.scaleEl[this.scaleIndex].startAngle, this.scaleEl[this.scaleIndex].sweepAngle, indegree)) || !this._isHalfCircle())
            //    this.activeElement.value = this._getValue(indegree);
                this._onMouseClickMove(this._getAngle(this.activeElement.value), this.activeElement.value);
            if (this._getValue(indegree) <= this.scaleEl[this.scaleIndex].maximum) {
                    if (this.scaleEl[this.scaleIndex].direction == "clockwise")
                        this.activeElement.value = this._getValue(indegree);
                    else
                        this.activeElement.value = this.scaleEl[this.scaleIndex].maximum - this._getValue(indegree);
            }
            if (this.contextEl.putImageData)
                this._reDrawPointer();
            else
                this._init();
        },

		/**
         * To check whether gauge is half circle or not
		 * @private
         */	
        _isHalfCircle: function () {
            if (this.model.frame.frameType == "halfcircle")
                return true;
        },
		/**
         * To get the Y position of semi circular gauge
		 * @private
         */	
        _getHalfCircleYPosition: function () {
            return this._getYCordinate(this.centerY, 0, (this.model.frame.halfCircleFrameStartAngle + this.model.frame.halfCircleFrameEndAngle) / 2);
        },
		/**
         * To get the X position of semi circular gauge
		 * @private
         */	
        _getHalfCircleXPosition: function () {
            return this._getXCordinate(this.centerX, 0, (this.model.frame.halfCircleFrameStartAngle + this.model.frame.halfCircleFrameEndAngle) / 2);
        },
		/**
         * To get the X Co-ordinate of semi circular gauge
		 * @private
         */	
        _getXCordinate: function (x, radius, angle) {
            var x1 = x + radius * (Math.cos(Math.PI * (angle / 180)));
            return x1;
        },
		/**
         * To get the Y Co-ordinate of semi circular gauge
		 * @private
         */	
        _getYCordinate: function (y, radius, angle) {
            var y1 = y + radius * (Math.sin(Math.PI * (angle / 180)));
            return y1;
        },
		/**
         * To get the angle circular gauge
		 * @private
         */	
        _getAngle: function (value) {
            var tempValue, angle;
            if (value >= this.scaleEl[this.scaleIndex].minimum && value <= this.scaleEl[this.scaleIndex].maximum)
                tempValue = this.scaleEl[this.scaleIndex].direction == "clockwise" ? value - this.scaleEl[this.scaleIndex].minimum : this.scaleEl[this.scaleIndex].maximum - value;
            else {
                if (this.scaleEl[this.scaleIndex].direction == "clockwise") {
                    if (value <= this.scaleEl[this.scaleIndex].minimum)
                        tempValue = this.scaleEl[this.scaleIndex].minimum;
                    else
                        tempValue = this.scaleEl[this.scaleIndex].maximum;
                }
                else {
                    if (value <= this.scaleEl[this.scaleIndex].minimum)
                        tempValue = this.scaleEl[this.scaleIndex].maximum;
                    else
                        tempValue = this.scaleEl[this.scaleIndex].minimum;
                }
            }
            angle = (tempValue * ((this.scaleEl[this.scaleIndex].sweepAngle) / (this.scaleEl[this.scaleIndex].maximum - this.scaleEl[this.scaleIndex].minimum))) + (this.scaleEl[this.scaleIndex].startAngle);
            return angle;
        },
		/**
         * To Handle the Decimal value manupulation
		 * @private
         */	
        _subtractDecimal: function (value, interval) {
            var strValue = value.toString();
            var strInterval = interval.toString();
            var vDecimal;
            var iDecimal;
            if (strValue.indexOf('.') > -1)
                vDecimal = strValue.length - strValue.indexOf('.') - 1;
            else
                vDecimal = 0;
            if (strInterval.indexOf('.') > -1)
                iDecimal = strInterval.length - strInterval.indexOf('.') - 1;
            else
                iDecimal = 0;
            var decimal = vDecimal > iDecimal ? vDecimal : iDecimal;
            var correctValue = (value * Math.pow(10, decimal) - interval * Math.pow(10, decimal)) / Math.pow(10, decimal);
            return correctValue;
        },
		/**
         * To get the circumference angle for the circular gauge
		 * @private
         */	
        _getCirucumferenceAngle: function (startPoint, endPoint) {
            if (endPoint.x > startPoint.x) {
                if (endPoint.y > startPoint.y)
                    return this._tangent(startPoint, endPoint); //quadrant2
                else {
                    if (endPoint.y == startPoint.y)
                        return 0;
                    else
                        return 2 * Math.PI + this._tangent(startPoint, endPoint); //quadrant1
                }
            }
            else {
                if (endPoint.x == startPoint.x) {
                    if (endPoint.y == startPoint.y)
                        return 0;   //0 degree
                    else {
                        if (endPoint.y > startPoint.y)
                            return Math.PI / 2;  // 90 degree
                        else
                            return 1.5 * Math.PI; // 270 degree
                    }
                }
                else {
                    if (endPoint.y == startPoint.y)
                        return Math.PI; // 180 degree
                    else {
                        if (endPoint.y > startPoint.y)
                            return Math.PI + this._tangent(startPoint, endPoint);  // quadrant3
                        else
                            return Math.PI + this._tangent(startPoint, endPoint); //quadrant4
                    }
                }
            }
        },
		/**
         * To Return the Tangent value in radians
		 * @private
         */	
        _tangent: function (startPoint, endPoint) {
            var distance = (endPoint.y - startPoint.y) / (endPoint.x - startPoint.x);
            var inradians = Math.atan(distance);
            //indegrees=180*inradians/Math.PI
            return inradians;
        },
		/**
         * To get the Image for Indicator
		 * @private
         */	
        _getIndicatorImage: function () {
            if (this.pointerImage)
                return this.pointerImage;
            else
                this._getPointerImage();
        },
		/**
         * To get the Image for Pointer
		 * @private
         */	
        _getPointerImage: function () {
            if (this.customLabelImage)
                return this.customLabelImage;
            else
                return this._getCustomLabelImage();
        },
		/**
         * To get the Image for SubGauge
		 * @private
         */	
        _getSubGaugeImage: function () {
            if (this.rangeImage)
                return this.rangeImage;
            else
                return this._getRangeImage();
        },
		/**
         * To get the Image for CustomLabel
		 * @private
         */	
        _getCustomLabelImage: function () {
            if (this.subGaugeImage)
                return this.subGaugeImage;
            else
                return this._getSubGaugeImage();
        },
		/**
         * To get the Image for Range
		 * @private
         */	
        _getRangeImage: function () {
            if (this.labelImage)
                return this.labelImage;
            else
                return this._getLabelImage();
        },
		/**
         * To get the Image for Label
		 * @private
         */	
        _getLabelImage: function () {
            if (this.tickImage)
                return this.tickImage;
            else
                return this._getTickImage();
        },
		/**
         * To get the Image for Tick
		 * @private
         */	
        _getTickImage: function () {
            if (this.scaleImage)
                return this.scaleImage;
            else
                return this.outerImage;
        },
		/**
         * To get the value from angle given
		 * @private
         */	
        _getValue: function (angle) {
            return ((angle - this.scaleEl[this.scaleIndex].startAngle) / this.scaleEl[this.scaleIndex].sweepAngle) * (this.scaleEl[this.scaleIndex].maximum - this.scaleEl[this.scaleIndex].minimum);
        },
		/**
         * To draw scales
		 * @private
         */	
        _drawScales: function () {
            var self = this;
            this.scaleEl = this.model.scales;
            $.each(this.model.scales, function (index, element) {
                self.scaleIndex = index;
                if (element.showScaleBar)
                    self._setScaleCordinates(element);
            });
            this._setTicks();
            this._setLabels();
            this._setRanges();
            this._subGauge();
            this._setCustomLabel();
            this._setPointers();
            $.each(this.model.scales, function (index, element) {
                if (element.showPointers) {
                    if (element.pointers.length > 1)
                        self.model.enableAnimation = false;
                }
            });
            if (!this.contextEl.putImageData)
                this.model.enableAnimation = false;
            if (this.model.animationSpeed != null && this.model.animationSpeed > 0) {
                var delay = this.model.animationSpeed / 25;
                if (delay >= 0) {
                    if (this.model.enableAnimation) {
                        this._onAnimate(delay);
                    }
                }
            }
            this._setIndicators();
        },
		/**
         * To set the value for ticks coordinates
		 * @private
         */	
        _setTicks: function () {
            var self = this;
            $.each(this.model.scales, function (index, element) {
                if (element.showTicks) {
                    self.scaleIndex = index;
                    if (element.ticks != null) {
                        self.tickEl = element.ticks;
                        $.each(element.ticks, function (tickIndex, tickElement) {
                            self.tickIndex = tickIndex;
                            self._setTicksCordinates(tickIndex, tickElement);
                        });
                    }
                }
            });
        },
		/**
         * To set the Label coordnates
		 * @private
         */	
        _setLabels: function () {
            var self = this;
            $.each(this.model.scales, function (index, element) {
                if (element.showLabels) {
                    self.scaleIndex = index;
                    if (element.labels != null) {
                        self.labelEl = element.labels;
                        $.each(element.labels, function (labelIndex, labelElement) {
                            self.labelIndex = labelIndex;
                            self._setLabelCoridnates(labelIndex, labelElement);
                        });
                    }
                }
            });
        },
		/**
         * To set the Indicators coordnates
		 * @private
         */	
        _setIndicators: function () {
            var self = this;
            $.each(this.model.scales, function (index, element) {
                if (element.showIndicators) {
                    self.scaleIndex = index;
                    if (element.indicators != null) {
                        self.indicatorEl = element.indicators;
                        $.each(element.indicators, function (indIndex, indElement) {
                            self.indicatorIndex = indIndex;
                            self._drawIndicator(indIndex, indElement);
                        });
                    }
                }
            });
        },
		/**
         * To set the Pointers coordnates
		 * @private
         */	
        _setPointers: function () {
            var self = this;
            $.each(this.model.scales, function (index, element) {
                if (element.showPointers) {
                    self.scaleIndex = index;
                    if (element.pointers != null) {
                        self.pointerEl = element.pointers;
                        $.each(element.pointers, function (pointerIndex, pointerElement) {
                            if (self._isHalfCircle())
                                element.showBackNeedle = false;
                            self.pointerIndex = pointerIndex;
                            self._drawPointers(pointerIndex, pointerElement);
                            //  setInterval( function(){ self._drawPointers(pointerIndex,pointerElement);}, 30 ); 
                        });
                    }
                }

            });
        },
		/**
         * To handle values while Animating
		 * @private
         */	
        _onAnimate: function (delay) {
            var self = this, timer;
            var currentValue = self.model.scales[0].minimum;
            var pointerValue = self.model.scales[0].pointers[0].value;
            timer = setInterval(function pointerchan() {
                if (pointerValue > currentValue) {
                    currentValue = currentValue + ((self.model.scales[0].maximum-self.model.scales[0].minimum)/100);
                    pointerValue > currentValue ? self.setPointerValue(0, 0, currentValue) : self.setPointerValue(0, 0, pointerValue);
                }
                else if (currentValue == pointerValue) {
                    window.clearInterval(timer);
                }
            }, delay);
        },
		/**
         * To set the Ranges coordnates
		 * @private
         */	
        _setRanges: function () {
            var self = this;
            $.each(this.model.scales, function (index, element) {
                if (element.showRanges) {
                    self.scaleIndex = index;
                    if (element.ranges != null) {
                        self.rangeEl = element.ranges;
                        $.each(element.ranges, function (rangeIndex, rangeElement) {
                            self.rangeIndex = rangeIndex;
                            self._setRangeCordinates(rangeIndex, rangeElement);
                        });
                    }
                }
            });
        },
		/**
         * To set the CustomLabel coordnates
		 * @private
         */	
        _setCustomLabel: function () {
            var self = this;
            $.each(this.model.scales, function (index, element) {
                self.scaleIndex = index;
                if (element.customLabels != null) {
                    self.customLabelEl = element.customLabels;
                    $.each(element.customLabels, function (labelIndex, labelElement) {
                        self.customLabelIndex = index;
                        self._setCustomLabelCordinates(labelIndex, labelElement);
                    });
                }
            });
        },
		/**
         * To Invoke the subgauge elements
		 * @private
         */	
        _subGauge: function () {
            var self = this;
            $.each(this.model.scales, function (index, element) {
                self.scaleIndex = index;
                if (element.subGauges != null) {
                    self.subGaugeEl = element.subGauges;
                    $.each(element.subGauges, function (subGaugeIndex, subGaugeElement) {
                        self.subGaugeIndex = index;
                        self._setSubGauge(subGaugeIndex, subGaugeElement);
                    });
                }
            });
        },
		/**
         * To set the Scale coordnates
		 * @private
         */	
        _setScaleCordinates: function (element) {
            this.location = {
                "centerX": this._isHalfCircle() ? this._getHalfCircleXPosition() : this.centerX, "centerY": this._isHalfCircle() ? this._getHalfCircleYPosition() : this.centerY,
                "startAngle": element.startAngle, "endAngle": (element.startAngle + element.sweepAngle),
                "startRadius": this.scaleEl[this.scaleIndex].radius - this.scaleEl[this.scaleIndex].size / 2, "endRadius": this.scaleEl[this.scaleIndex].radius + this.scaleEl[this.scaleIndex].size / 2
            };
            this.style = {
                "radius": element.radius - element.size / 2,
                "strokeStyle": (element.border.color == "transparent") ? "rgba(0,0,0,0)" : this._getColor(element, element.border.color), // this._getColor(element, element.borderColor),
                "lineWidth": element.border.width,
                "size": element.size,
                "isFill": true,
                "opacity": isNaN(element.opacity) ? 1 : element.opacity,
                "isStroke": true,
                "shadowOffset": element.shadowOffset,
                "fillStyle": (element.backgroundColor == "transparent") ? "rgba(0,0,0,0)" : this._getColor(element, element.backgroundColor), //this._getColor(element, element.backgroundColor),
                "counterClockwise": element.direction == "clockwise" ? false : true
            };
            if (element.maximum < element.minimum) {
                var tempValue = element.maximum;
                element.maximum = element.minimum;
                element.minimum=tempValue;
            }
            this.maximum(element.maximum);
            this.minimum(element.minimum);
            this._drawScaleBar(this.location, this.style);
            if (this.contextEl.getImageData)
                this.scaleImage = this.contextEl.getImageData(0, 0, this.model.width, this.model.height);
        },
		/**
         * To draw Outer layer for the circuar gauge
		 * @private
         */	
        _drawOuterLayer: function () {
            var interiorGradient, startPoint, endPoint, hcFrameInnerAngle;
            if (this._isHalfCircle()) {
                startPoint = { "x": this.centerX, "y": this.centerY };
                endPoint = { "x": this.centerX + this.model.radius, "y": this.centerY / 2 };
                hcFrameInnerAngle = this._getCirucumferenceAngle(startPoint, endPoint);
            }
            if (!this.model.isRadialGradient) {
                interiorGradient = this.contextEl.createLinearGradient(0, 0, 0, this.canvasEl[0].height);
            }
            else {
                interiorGradient = this.contextEl.createRadialGradient(this.canvasEl[0].width / 2, this.canvasEl[0].height / 2, 0, this.canvasEl[0].width / 2, this.canvasEl[0].height / 2, this.model.radius);
            }
            if (!ej.isNullOrUndefined(this.model.interiorGradient))
                this._setGradientColor(this, interiorGradient, this.model.interiorGradient.colorInfo);

            this.frameOuterLocation = { "centerX": this.centerX, "hcCenterX": this.centerX, "hcCenterY": this.centerY, "centerY": this.centerY, "startAngle": this._isHalfCircle() ? Math.PI * ((this.model.frame.halfCircleFrameStartAngle) / 180) : 0, "endAngle": this._isHalfCircle() ? Math.PI * ((this.model.frame.halfCircleFrameEndAngle) / 180) : 2 * Math.PI };
            if (this._isHalfCircle())
                this.frameInnerLocation = { "centerX": this.centerX, "hcCenterX": this._getXCordinate(this.centerX, 0, (this.model.frame.halfCircleFrameStartAngle + this.model.frame.halfCircleFrameEndAngle) / 2), "hcCenterY": this._getYCordinate(this.centerX, 0, (this.model.frame.halfCircleFrameStartAngle + this.model.frame.halfCircleFrameEndAngle) / 2), "centerY": this.centerY, "startAngle": Math.PI * ((this.model.frame.halfCircleFrameStartAngle) / 180) + hcFrameInnerAngle, "endAngle": Math.PI * ((this.model.frame.halfCircleFrameEndAngle) / 180) - hcFrameInnerAngle };
            this.frameInnerStyle = {
                "radius": this.model.radius,
                "isStroke": false, "isFill": true,
                "fillStyle": (this.model.backgroundColor == "transparent") ? "rgba(0,0,0,0)" : this._getColor(null, this.model.backgroundColor),
                "counterClockwise": false
            };
            if (this.model.frame.frameType == "fullcircle") {
                this._drawCircleFrame(this.frameOuterLocation, this.frameInnerStyle);
            }
            else if (this.model.frame.frameType == "halfcircle") {
                this._drawHalfCircle(this.frameInnerLocation, this.frameInnerStyle);
            }
            if (this.contextEl.getImageData)
                this.outerImage = this.contextEl.getImageData(0, 0, this.model.width, this.model.height);
        },
		/**
         * To set Ticks Cordinates for the circuar gauge
		 * @private
         */	
        _setTicksCordinates: function (index, element) {
            var startX, startY, lineHeight, minimumMeet, interval, radius, angle;
            if (element.type == "major") {
                interval = this.scaleEl[this.scaleIndex].majorIntervalValue;
                this.majorTickHeight = element.height;
            }
            else {
                interval = this.scaleEl[this.scaleIndex].minorIntervalValue;
            }
            if (element.placement == "far")
                radius = this.scaleEl[this.scaleIndex].radius + this.scaleEl[this.scaleIndex].size / 2 + this.scaleEl[this.scaleIndex].border.width / 2 + element.distanceFromScale;
            if (element.placement == "center")
                radius = this.scaleEl[this.scaleIndex].radius - element.height / 2 - this.scaleEl[this.scaleIndex].border.width / 2 - element.distanceFromScale;
            if (element.placement == "near")
                radius = this.scaleEl[this.scaleIndex].radius - this.scaleEl[this.scaleIndex].size / 2 - this.scaleEl[this.scaleIndex].border.width / 2 - element.distanceFromScale;
            lineHeight = element.placement == "near" ? -element.height : element.height;
            for (var value = this.scaleEl[this.scaleIndex].maximum; value >= this.scaleEl[this.scaleIndex].minimum; value -= interval) {
                if (interval == this.scaleEl[this.scaleIndex].minorIntervalValue && value % this.scaleEl[this.scaleIndex].majorIntervalValue != 0 || interval == this.scaleEl[this.scaleIndex].majorIntervalValue) {
                    if (value == this.scaleEl[this.scaleIndex].minimum)
                        minimumMeet = true;
                    angle = this._getAngle(value);
                    angle = angle > 360 ? angle - 360 : angle;
                    startX = this._getXCordinate(this._isHalfCircle() ? this._getHalfCircleXPosition() : this.centerX, radius, angle);
                    startY = this._getYCordinate(this._isHalfCircle() ? this._getHalfCircleYPosition() : this.centerY, radius, angle);
                    this.location = { "startX": startX, "startY": startY };
                    this.style = { "angle": element.angle + angle, "isStroke": true, "isFill": false, "lineHeight": lineHeight, "lineWidth": element.width, "strokeStyle": (element.color == "transparent") ? "rgba(0,0,0,0)" : this._getColor(element, element.color) };
                    if (this.model.drawTicks)
                        this._onDrawTicks(angle, value);
                    this._drawTickMark(this.location, this.style);
                }
            }
            if (!minimumMeet) {
                angle = this._getAngle(this.scaleEl[this.scaleIndex].minimum);
                angle = angle > 360 ? angle - 360 : angle;
                startX = this._getXCordinate(this._isHalfCircle() ? this._getHalfCircleXPosition() : this.centerX, radius, angle);
                startY = this._getYCordinate(this._isHalfCircle() ? this._getHalfCircleYPosition() : this.centerY, radius, angle);
                this.location = { "startX": startX, "startY": startY };
                this.style = { "angle": element.angle + angle, "isStroke": true, "isFill": false, "lineHeight": lineHeight, "lineWidth": element.width, "strokeStyle": (element.color == "transparent") ? "rgba(0,0,0,0)" : this._getColor(element, element.color) };
                //if (this.model.drawTicks)
                //    this._onDrawTicks(angle, value);
                //this._drawTickMark(this.location, this.style);
            }

            if (this.contextEl.getImageData)
                this.tickImage = this.contextEl.getImageData(0, 0, this.model.width, this.model.height);
        },
		/**
         * To set Label Cordinates for the circuar gauge
		 * @private
         */	
        _setLabelCoridnates: function (index, element) {
            var minimumMeet;
            var startX, startY, lineHeight, incrementAngle, interval, radius;
            interval = element.type == "major" ? this.scaleEl[this.scaleIndex].majorIntervalValue : this.scaleEl[this.scaleIndex].minorIntervalValue;
            incrementAngle = element.type == "major" ? (this.majorIntervalAngle = this.scaleEl[this.scaleIndex].sweepAngle / ((this.scaleEl[this.scaleIndex].maximum - this.scaleEl[this.scaleIndex].minimum) / interval)) : this.scaleEl[this.scaleIndex].sweepAngle / ((this.scaleEl[this.scaleIndex].maximum - this.scaleEl[this.scaleIndex].minimum) / interval);
            if (element.placement == "far")
                radius = this.scaleEl[this.scaleIndex].radius + this.scaleEl[this.scaleIndex].size / 2 + this.majorTickHeight + element.distanceFromScale;
            if (element.placement == "center")
                radius = this.scaleEl[this.scaleIndex].radius - 10 - element.distanceFromScale;
            if (element.placement == "near")
                radius = this.scaleEl[this.scaleIndex].radius - this.scaleEl[this.scaleIndex].size / 2 - 10 - this.majorTickHeight - element.distanceFromScale;
            for (var value = this.scaleEl[this.scaleIndex].maximum; value >= this.scaleEl[this.scaleIndex].minimum; value = this._subtractDecimal(value, interval)) {
                if (value == this.scaleEl[this.scaleIndex].minimum)
                    minimumMeet = true;
                if (interval == this.scaleEl[this.scaleIndex].minorIntervalValue && value % this.scaleEl[this.scaleIndex].majorIntervalValue != 0 || interval == this.scaleEl[this.scaleIndex].majorIntervalValue) {
                    if (value == this.scaleEl[this.scaleIndex].minimum && !element.includeFirstValue) {
                        continue;
                    }
                    angle = this._getAngle(value);
                    angle = angle > 360 ? angle - 360 : angle;
                    startX = this._getXCordinate(this._isHalfCircle() ? this._getHalfCircleXPosition() : this.centerX, radius, angle);
                    startY = this._getYCordinate(this._isHalfCircle() ? this._getHalfCircleYPosition() : this.centerY, radius, angle);
                    this.location = { "startX": startX, "startY": startY };
                    this.style = { "angle": this.scaleEl[this.scaleIndex].labels[this.labelIndex].autoAngle ? angle + element.angle : element.angle, "isStroke": false, "isFill": true, "textValue": value, "opacity": !(element.opacity) ? 1 : element.opacity, "font": this._getFontString(element, element.font), "fillStyle": (element.color == "transparent") ? "rgba(0,0,0,0)" : this._getColor(element, element.color) };
                    if (this.model.drawLabels)
                        this._onDrawLabels(this.scaleEl[this.scaleIndex].labels[this.labelIndex].autoAngle ? angle + element.angle : element.angle, value);
                    this._drawLabel(this.location, this.style, false);
                }
            }
            if (!minimumMeet) {
                angle = this._getAngle(this.scaleEl[this.scaleIndex].minimum);
                angle = angle > 360 ? angle - 360 : angle;
                startX = this._getXCordinate(this._isHalfCircle() ? this._getHalfCircleXPosition() : this.centerX, radius, angle);
                startY = this._getYCordinate(this._isHalfCircle() ? this._getHalfCircleYPosition() : this.centerY, radius, angle);
                this.location = { "startX": startX, "startY": startY };
                this.style = { "angle": this.scaleEl[this.scaleIndex].labels[this.labelIndex].autoAngle ? angle + element.angle : element.angle, "isStroke": false, "isFill": true, "textValue": this.scaleEl[this.scaleIndex].minimum, "opacity": !(element.opacity) ? 1 : element.opacity, "font": this._getFontString(element, element.font), "lineHeight": lineHeight, "lineWidth": element.width, "fillStyle": (element.color == "transparent") ? "rgba(0,0,0,0)" : this._getColor(element, element.color) };
                if (this.model.drawLabels)
                    this._onDrawLabels(angle, value);
                this._drawLabel(this.location, this.style, false);
                angle = this._getAngle(value);
                angle = angle > 360 ? angle - 360 : angle;
            }
            if (this.contextEl.getImageData)
                this.labelImage = this.contextEl.getImageData(0, 0, this.model.width, this.model.height);
        },
		/**
         * To set Range Cordinates for the circuar gauge
		 * @private
         */	
        _setRangeCordinates: function (index, element) {
                var startX, startY, endX, endY, startAngle, radius, endAngle, startRadius, endRadius, grd, controlAngle;
                controlAngle = this._getAngle((element.startValue + element.endValue) / 2);
                startAngle = this._getAngle(element.startValue);
                endAngle = this._getAngle(element.endValue);
                radius = this.scaleEl[this.scaleIndex].radius - element.distanceFromScale - this.scaleEl[this.scaleIndex].size / 2 - element.size - this.majorTickHeight;
                startRadius = this.scaleEl[this.scaleIndex].radius - element.distanceFromScale - this.scaleEl[this.scaleIndex].size / 2 - this.majorTickHeight;
                endRadius = this.scaleEl[this.scaleIndex].radius - element.distanceFromScale - element.size - this.scaleEl[this.scaleIndex].size / 2 - this.majorTickHeight;
                startX = this._getXCordinate(this._isHalfCircle() ? this._getHalfCircleXPosition() : this.centerX, radius, startAngle);
                startY = this._getYCordinate(this._isHalfCircle() ? this._getHalfCircleYPosition() : this.centerY, radius, startAngle);
                endX = this._getXCordinate(this._isHalfCircle() ? this._getHalfCircleXPosition() : this.centerX, radius, endAngle);
                endY = this._getYCordinate(this._isHalfCircle() ? this._getHalfCircleYPosition() : this.centerY, radius, endAngle);
                startAngle = (180 * this._getCirucumferenceAngle({ "x": this._isHalfCircle() ? this._getHalfCircleXPosition() : this.centerX, "y": this._isHalfCircle() ? this._getHalfCircleYPosition() : this.centerY }, { "x": startX, "y": startY })) / Math.PI;
                endAngle = (180 * this._getCirucumferenceAngle({ "x": this._isHalfCircle() ? this._getHalfCircleXPosition() : this.centerX, "y": this._isHalfCircle() ? this._getHalfCircleYPosition() : this.centerY }, { "x": endX, "y": endY })) / Math.PI;

                if (!ej.isNullOrUndefined(element.gradients)) {
                    grd = this.contextEl.createRadialGradient(this.centerX, this.centerY, endRadius, this.centerX, this.centerY, startRadius);
                    this._setGradientColor(this, grd, element.gradients.colorInfo);
                } else
                    grd = (element.backgroundColor == "transparent") ? "rgba(0,0,0,0)" : this._getColor(element, element.backgroundColor); // this._getColor(element, element.backgroundColor);
                this.location = { "startX": startX, "startY": startY, "endX": endX, "endY": endY, "startAngle": startAngle, "endAngle": endAngle };
                this.style = {
                    "startWidth": element.startWidth,
                    "isFill": true,
                    "fillStyle": grd,
                    "strokeStyle": (element.border.color == "transparent") ? "rgba(0,0,0,0)" : element.border.color, // element.borderColor,
                    "opacity": isNaN(element.opacity) ? 1 : element.opacity,
                    "counterClockwise": this.scaleEl[this.scaleIndex].direction == "clockwise" ? false : true,
                    "startRadius": startRadius, "endRadius": endRadius, "endWidth": element.endWidth, "lineWidth": element.border.width, "isStroke": true
                };
                if (this.model.drawRange)
                    this._onDrawRange(this.location, this.style);
                this._drawRange(this.location, this.style);
            if (this.contextEl.getImageData)
                this.rangeImage = this.contextEl.getImageData(0, 0, this.model.width, this.model.height);
        },
		/**
         * To set the subGauges
		 * @private
         */	
        _setSubGauge: function (index, element) {
            var subGaugeDiv = $('div[id=' + element.controlID + ']');
            if (subGaugeDiv.length > 0 && subGaugeDiv.find('canvas').length) {
                var sourceCanvas = subGaugeDiv.find('canvas')[0].getContext('2d');
                this.contextEl.drawImage(sourceCanvas.canvas, element.position.x, element.position.y, element.width, element.height);
                subGaugeDiv.css('display', 'none');
                if (this.contextEl.getImageData)
                    this.subGaugeImage = this.contextEl.getImageData(0, 0, this.model.width, this.model.height);
            }
        },
		/**
         * To set Custom Label Cordinates for the circuar gauge
		 * @private
         */	
        _setCustomLabelCordinates: function (index, element) {
            element.color = element.color ? (element.color) : "#282828";
            this.location = element.position ? ({ "startX": element.position.x, "startY": element.position.y }) : { "startX": 0, "startY": 0 };
            this.style = { "angle": element.textAngle, "textValue": element.value, "fillStyle": (element.color == "transparent") ? "rgba(0,0,0,0)" : this._getColor(element, element.color), "font": this._getFontString(element, element.font) };
            if (this.model.drawCustomLabel)
                this._onDrawCustomLabel(this.location, this.style);
            this._drawLabel(this.location, this.style, true);
            if (this.contextEl.getImageData)
                this.customLabelImage = this.contextEl.getImageData(0, 0, this.model.width, this.model.height);
        },
		/**
         * To Draw Indicators
		 * @private
         */	
        _drawIndicator: function (index, element) {
            var self = this, isInStateRange = false;
            this.location = { "centerX": element.position.x - element.width / 2, "textLocation": element.position, "centerY": element.position.y - element.height / 2, "startAngle": 0, "endAngle": 2 * Math.PI };
            this.style = {
                "radius": (element.height + element.width) / 2,
                "strokeStyle": "#2BA104",
                "cornerRadius": element.type == "roundedrectangle" ? 2 : 0,
                "height": element.height,
                "width": element.width,
                "lineWidth": element.indicatorBorderWidth,
                "fillStyle": "#2BA104",
                "isStroke": true,
                "isFill": true,
                "indicatorText": element.indicatorText,
                "textColor": null,
                "font": null,
                "counterClockwise": false
            };
            if (this.model.drawIndicators)
                this._onDrawIndicators(this.style, this.location);
            if (element.type == ej.datavisualization.CircularGauge.IndicatorType.Image) {
                var image = new Image();
                image.onload = function () {
                    self.contextEl.drawImage(this, element.position.x, element.position.y);
                };
                image.src = element.imageUrl;
            }
            else if (element.stateRanges != null) {
                $.each(element.stateRanges, function (index, srEl) {//srEl = StateRange Element
                    if (self.pointerEl[self.pointerIndex].value >= srEl.startValue && self.pointerEl[self.pointerIndex].value <= srEl.endValue) {
                        isInStateRange = true;
                        if (!ej.isNullOrUndefined(srEl.text) && srEl.text.length > 0) {
                            self.style.indicatorText = srEl.text;
                            self.style.textColor = (srEl.textColor == "transparent") ? "rgba(0,0,0,0)" : srEl.textColor; // srEl.textColor;
                            self.style.font = element.font;
                        }
                        self.style.strokeStyle = (srEl.borderColor == "transparent") ? "rgba(0,0,0,0)" : srEl.borderColor;
                        self.style.fillStyle = (srEl.backgroundColor == "transparent") ? "rgba(0,0,0,0)" : srEl.backgroundColor;
                        self._drawIndicatorFrame(element.type, self.location, self.style);
                    }
                });
            }
            if (!isInStateRange && element.type != ej.datavisualization.CircularGauge.IndicatorType.Image) {
                self._drawIndicatorFrame(element.type, self.location, self.style);
            }
            if (this.contextEl.getImageData && element.type != ej.datavisualization.CircularGauge.IndicatorType.Image)
                this.indicatorImage = this.contextEl.getImageData(0, 0, this.model.width, this.model.height);
        },

        _drawIndicatorFrame: function(type, location,style) {
            switch (type) {
                case "circle":
                    this._drawCircleFrame(location, style);
                    break;
                case "roundedrectangle":
                case "rectangle":
                    this._drawRectangleFrame(location, style);
                    break;
                case "text":
                    this._drawText(location, style);
                    break
            }
        },
		/**
         * To Draw Scale Bar
		 * @private
         */	
        _drawScaleBar: function (location, style) {
            this.contextEl.shadowColor = (style.strokeStyle == "transparent") ? "rgba(0,0,0,0)" : style.strokeStyle;
            if (style.shadowOffset)
                this.contextEl.shadowBlur = style.shadowOffset;
            this._contextOpenPath(style, this);
            if ((location.endAngle - location.startAngle) == 0)
                this.contextEl.arc(location.centerX, location.centerY, location.startRadius, Math.PI * (0 / 180), Math.PI * (0 / 180), false);
            else if ((location.endAngle - location.startAngle) % 360 == 0) {
                this.contextEl.arc(location.centerX, location.centerY, location.startRadius, Math.PI * (0 / 180), Math.PI * (360 / 180), false);
                this.contextEl.arc(location.centerX, location.centerY, location.endRadius, Math.PI * (360 / 180), Math.PI * (0 / 180), true);
            }
            else {
                this.contextEl.arc(location.centerX, location.centerY, location.startRadius, Math.PI * (location.startAngle / 180), Math.PI * (location.endAngle / 180), false);
                this.contextEl.arc(location.centerX, location.centerY, location.endRadius, Math.PI * (location.endAngle / 180), Math.PI * (location.startAngle / 180), true);
            }
            this._contextClosePath(style, this);
        },
		/**
         * To Draw TickMark
		 * @private
         */	
        _drawTickMark: function (location, style) {
            this._contextOpenPath(style, this);
            this.contextEl.translate(location.startX, location.startY);
            this.contextEl.moveTo(0, 0);
            this.contextEl.rotate((Math.PI * (style.angle / 180)));
            this.contextEl.lineTo(style.lineHeight, 0);
            this._contextClosePath(style, this);
        },
		/**
         * To Draw Label
		 * @private
         */	
        _drawLabel: function (location, style, isCustomLabel) {
            this._contextOpenPath(style, this);
            this.contextEl.textAlign = "center";
            this.contextEl.textBaseline = "middle";
            this.contextEl.font = style.font;
            this.contextEl.translate(location.startX, location.startY);
            this.contextEl.moveTo(0, 0);
            this.scaleEl[this.scaleIndex].labels[this.labelIndex].autoAngle ? this.contextEl.rotate((Math.PI * ((style.angle - 270) / 180))) : this.contextEl.rotate((Math.PI * (style.angle / 180)));
            if (!ej.isNullOrUndefined(isCustomLabel) && !isCustomLabel) {
                var textPostion = this.model.scales[this.scaleIndex].labels[this.labelIndex].unitTextPosition;
                if (!ej.isNullOrUndefined(textPostion) && textPostion.toString() == "back")
                    style.textValue = ((style.textValue % 1 != 0) ? +style.textValue.toFixed(3) : style.textValue) + this.model.scales[this.scaleIndex].labels[this.labelIndex].unitText;
                else if (!ej.isNullOrUndefined(textPostion) && textPostion.toString() == "front")
                    style.textValue = this.model.scales[this.scaleIndex].labels[this.labelIndex].unitText + style.textValue;
            }
            this.contextEl.fillText(style.textValue, 0, 0);
            this._contextClosePath(style, this);
        },
		/**
         * To Draw Circle Frame
		 * @private
         */	
        _drawCircleFrame: function (location, style) {
            this._contextOpenPath(style, this);
            this.contextEl.arc(location.centerX, location.centerY, style.radius, location.startAngle, location.endAngle, style.counterClockwise);
            this._contextClosePath(style, this);
            if (style.indicatorText)
                this._drawText(location, style);
        },
		/**
         * To Draw Half Circle
		 * @private
         */	
        _drawHalfCircle: function (location, style) {
            this._contextOpenPath(style, this);
            this.contextEl.lineJoin = "round";
            this.contextEl.arc(location.centerX, location.centerY, style.radius, location.startAngle, location.endAngle, false);
            this.contextEl.lineTo(location.hcCenterX, location.hcCenterY);
            this._contextClosePath(style, this);
        },
		/**
         * To Draw Rectangular Frame
		 * @private
         */	
        _drawRectangleFrame: function (location, style) {
            this._contextOpenPath(style, this);
            this.contextEl.translate(location.centerX, location.centerY - style.height / 2);
            this.contextEl.moveTo(style.cornerRadius, 0);
            this.contextEl.lineTo(style.width - style.cornerRadius, 0);
            this.contextEl.quadraticCurveTo(style.width, 0, style.width, style.cornerRadius);
            this.contextEl.lineTo(style.width, style.height - style.cornerRadius);
            this.contextEl.quadraticCurveTo(style.width, style.height, style.width - style.cornerRadius, style.height);
            this.contextEl.lineTo(style.cornerRadius, style.height);
            this.contextEl.quadraticCurveTo(0, style.height, 0, style.height - style.cornerRadius);
            this.contextEl.lineTo(0, style.cornerRadius);
            this.contextEl.quadraticCurveTo(0, 0, style.cornerRadius, 0);
            this._contextClosePath(style, this);
            if (style.indicatorText)
                this._drawText(location, style);
        },
		/**
         * To Draw Text for Indicators
		 * @private
         */	
        _drawText: function (location, style) {
            this.contextEl.beginPath();
            this.contextEl.textAlign = "center";
            this.contextEl.fillStyle = (style.textColor == "transparent") ? "rgba(0,0,0,0)" : style.textColor;
            this.contextEl.font = style.font;
            this.contextEl.fillText(style.indicatorText, location.textLocation.x, location.textLocation.y);
            this.contextEl.closePath();
        },
		/**
         * To Draw Range
		 * @private
         */	
        _drawRange: function (location, style) {
            this._contextOpenPath(style, this);
            this.contextEl.arc(this._isHalfCircle() ? this._getHalfCircleXPosition() : this.centerX, this._isHalfCircle() ? this._getHalfCircleYPosition() : this.centerY, style.startRadius, Math.PI * (location.startAngle / 180), Math.PI * (location.endAngle / 180), style.counterClockwise);
            this.contextEl.arc(this._isHalfCircle() ? this._getHalfCircleXPosition() : this.centerX, this._isHalfCircle() ? this._getHalfCircleYPosition() : this.centerY, style.endRadius, Math.PI * (location.endAngle / 180), Math.PI * (location.startAngle / 180), !style.counterClockwise);
            this._contextClosePath(style, this);
        },
		/**
         * To Draw Text for Pointers
		 * @private
         */	
        _drawPointers: function (index, element) {
            element.value = element.value > this.scaleEl[this.scaleIndex].maximum ? this.scaleEl[this.scaleIndex].maximum : element.value;
            element.value = element.value < this.scaleEl[this.scaleIndex].minimum ? this.scaleEl[this.scaleIndex].minimum : element.value;
            var startX, startY, angle, height, grd;
            angle = this._getAngle(element.value);
            angle = angle > 360 ? angle - 360 : angle;
            if (element.type == "needle") {
                if (element.placement == "far")
                    height = element.length + this.scaleEl[this.scaleIndex].size / 2;
                if (element.placement == "center")
                    height = element.length;
                if (element.placement == "near")
                    height = element.length - this.scaleEl[this.scaleIndex].size / 2;
            }
            else {
                height = element.length;
                if (element.placement == "far") {
                    startX = this._getXCordinate(this._isHalfCircle() ? this._getHalfCircleXPosition() : this.centerX, this.scaleEl[this.scaleIndex].radius + this.scaleEl[this.scaleIndex].size / 2 + element.distanceFromScale, angle);
                    startY = this._getYCordinate(this._isHalfCircle() ? this._getHalfCircleYPosition() : this.centerX, this.scaleEl[this.scaleIndex].radius + this.scaleEl[this.scaleIndex].size / 2 + element.distanceFromScale, angle);
                }
                if (element.placement == "center") {
                    startX = this._getXCordinate(this._isHalfCircle() ? this._getHalfCircleXPosition() : this.centerX, this.scaleEl[this.scaleIndex].radius - this.scaleEl[this.scaleIndex].size / 2 - element.distanceFromScale, angle);
                    startY = this._getYCordinate(this._isHalfCircle() ? this._getHalfCircleYPosition() : this.centerX, this.scaleEl[this.scaleIndex].radius - this.scaleEl[this.scaleIndex].size / 2 - element.distanceFromScale, angle);
                }
                if (element.placement == "near") {
                    startX = this._getXCordinate(this._isHalfCircle() ? this._getHalfCircleXPosition() : this.centerX, this.scaleEl[this.scaleIndex].radius - this.scaleEl[this.scaleIndex].size / 2 - element.distanceFromScale - this.majorTickHeight - 15, angle);
                    startY = this._getYCordinate(this._isHalfCircle() ? this._getHalfCircleYPosition() : this.centerX, this.scaleEl[this.scaleIndex].radius - this.scaleEl[this.scaleIndex].size / 2 - element.distanceFromScale - this.majorTickHeight - 15, angle);
                }
            }
            if (height > this.model.radius)
                height = this.model.radius;

            this.location = { "startX": element.type == "needle" ? this._isHalfCircle() ? this._getHalfCircleXPosition() : this.centerX : startX, "startY": element.type == "needle" ? this._isHalfCircle() ? this._getHalfCircleYPosition() : this.centerY : startY };
            if (!ej.isNullOrUndefined(element.gradients)) {
                grd = this.contextEl.createLinearGradient(0, 0, 0, element.width);
                this._setGradientColor(this, grd, element.gradients.colorInfo);
            } else
                grd = (element.backgroundColor == "transparent") ? "rgba(0,0,0,0)" : this._getColor(element, element.backgroundColor);
            this.style = {
                "width": element.width,
                "isFill": true,
                "isStroke": true,
                "radius": 0,
                "showBackNeedle": element.showBackNeedle,
                "backNeedleLength": element.backNeedleLength,
                "angle": element.type == "needle" ? angle : element.placement == "far" ? angle : angle + 180,
                "height": height, "lineWidth": element.border.width, "opacity": isNaN(element.opacity) ? 1 : element.opacity,
                "strokeStyle": (element.border.color == "transparent") ? "rgba(0,0,0,0)" : this._getColor(element, element.border.color),
                "fillStyle": grd
            };
            if (this.model.drawPointers)
                this._onDrawPointers(angle, element.value);
            if (element.type == "needle") {
                this._drawNeedlePointer(this.location, this.style, element);
                this._setPointerCap(element);
            }
            else {
                if (element.markerType == "roundedrectangle")
                    this.style.radius = 5;
                this._drawMarkerType(element.markerType, this.location, this.style);
            }
            if (this.contextEl.getImageData)
                this.pointerImage = this.contextEl.getImageData(0, 0, this.model.width, this.model.height);
            this.value(element.value);
        },

        _drawMarkerType: function (type, location, style) {
            switch (type) {
                case "rectangle":
                    this._drawRectangle(location,style,this);
                    break;
                case "triangle":
                    this._drawTriangle(location,style,this);
                    break;
                case "ellipse":
                    this._drawEllipse(location,style,this);
                    break;
                case "diamond":
                    this._drawDiamond(location,style,this);
                    break;
                case "pentagon":
                    this._drawPentagon(location,style,this);
                    break;
                case "circle":
                    this._drawCircle(location,style,this);
                    break;
                case "slider":
                    this._drawSlider(location,style,this);
                    break;
                case "pointer":
                    this._drawPointer(location,style,this);
                    break;
                case "wedge":
                    this._drawWedge(location,style,this);
                    break;
                case "trapezoid":
                    this._drawTrapezoid(location,style,this);
                    break;
                case "roundedrectangle":
                    this._drawRoundedRectangle(location,style,this);
                    break;
            }
        },

		/**
         * To Draw Needle Pointer
		 * @private
         */	
        _drawNeedlePointer: function (location, style, element) {
            this._contextOpenPath(style, this);
            this.contextEl.translate(location.startX, location.startY);
            this.contextEl.rotate(Math.PI * ((style.angle) / 180));
            this.contextEl.moveTo(0, -style.width / 2);
            if (this.pointerEl[this.pointerIndex].needleType == "triangle") {
                this.contextEl.lineTo(style.height, 0);
            }
            else if (this.pointerEl[this.pointerIndex].needleType == "rectangle") {
                this.contextEl.lineTo(style.height, -style.width / 2);
                this.contextEl.lineTo(style.height, style.width / 2);
            }
            else if (this.pointerEl[this.pointerIndex].needleType == "trapezoid") {
                this.contextEl.lineTo(style.height, -style.width / 4);
                this.contextEl.lineTo(style.height, style.width / 4);
            }
            else if (this.pointerEl[this.pointerIndex].needleType == "arrow") {
                this.contextEl.lineTo(style.height - style.height / 4, -style.width / 6);
                this.contextEl.lineTo(style.height - style.height / 4, -style.width / 2);
                this.contextEl.lineTo(style.height, 0);
                this.contextEl.lineTo(style.height - style.height / 4, style.width / 2);
                this.contextEl.lineTo(style.height - style.height / 4, style.width / 6);
            }
            this.contextEl.lineTo(0, style.width / 2);
            if (style.showBackNeedle) {
                this.contextEl.lineTo(-(style.backNeedleLength + this.scaleEl[this.scaleIndex].pointerCap.radius / 2), style.width / 2);
                this.contextEl.lineTo(-(style.backNeedleLength + this.scaleEl[this.scaleIndex].pointerCap.radius / 2), -style.width / 2);
            }
            this.canvasEl.attr("aria-label", this.model.scales[this.scaleIndex].pointers[this.pointerIndex].value);
            this._contextClosePath(style, this);
        },
		/**
         * To set the pointer cap position
		 * @private
         */	
        _setPointerCap: function (element) {
            var grd;
            //var gradients = [{ "ColorStop": 0.19, "Color": "#F2DFDF" }, { "ColorStop": 0.71, "Color": "#FFFFFF" }, { "ColorStop": 0.77, "Color": "#836B33" }];
            grd = this.contextEl.createRadialGradient(this._isHalfCircle() ? this._getHalfCircleXPosition() : this.centerX, this._isHalfCircle() ? this._getHalfCircleYPosition() : this.canvasEl[0].height / 2, 0, this._isHalfCircle() ? this._getHalfCircleXPosition() : this.centerX, this._isHalfCircle() ? this._getHalfCircleYPosition() : this.canvasEl[0].height / 2, this.scaleEl[this.scaleIndex].pointerCap.radius);
            if (!ej.isNullOrUndefined(this.scaleEl[this.scaleIndex].pointerCap.interiorGradient))
                this._setGradientColor(this, grd, this.scaleEl[this.scaleIndex].pointerCap.interiorGradient.colorInfo);
            else
                grd = (this.scaleEl[this.scaleIndex].pointerCap.backgroundColor == "transparent") ? "rgba(0,0,0,0)" : this._getColor(this, this._getColor(element, this.scaleEl[this.scaleIndex].pointerCap.backgroundColor)); // this._getColor(this, this._getColor(element, this.scaleEl[this.scaleIndex].pointerCap.backgroundColor));
            this.location = { "centerX": this._isHalfCircle() ? this._getHalfCircleXPosition() : this.centerX, "centerY": this._isHalfCircle() ? this._getHalfCircleYPosition() : this.centerY, "startAngle": 0, "endAngle": 2 * Math.PI };
            this.style = { "isStroke": true, "isFill": true, "strokeStyle": (this.scaleEl[this.scaleIndex].pointerCap.borderColor == "transparent") ? "rgba(0,0,0,0)" : this._getColor(this, this._getColor(element, this.scaleEl[this.scaleIndex].pointerCap.borderColor)), "radius": this.scaleEl[this.scaleIndex].pointerCap.radius, "lineWidth": this.scaleEl[this.scaleIndex].pointerCap.borderWidth, "fillStyle": grd };
            if (this.model.drawPointerCap)
                this._onDrawPointerCap();
            this._drawCircleFrame(this.location, this.style);
        },
		/**
         * To Redraw the sclae
		 * @private
         */	
        _reDrawScale: function () {
            this.contextEl.putImageData(this.outerImage, 0, 0);
            this._drawScales();

        },
		/**
         * To Redraw the Pointers
		 * @private
         */	
        _reDrawPointer: function () {
            this.contextEl.putImageData(this._getPointerImage(), 0, 0);
            this._setPointers();
            this._setIndicators();
        },
		/**
         * To redraw the Custom Label
		 * @private
         */	
        _reDrawCustomLabel: function () {
            this.contextEl.putImageData(this._getCustomLabelImage(), 0, 0);
            this._setCustomLabel();
            this._setPointers();
            this._setIndicators();
        },
		/**
         * To Redraw the Ranges
		 * @private
         */	
        _reDrawRange: function () {
            this.contextEl.putImageData(this._getRangeImage(), 0, 0);
            this._setRanges();
            this._subGauge();
            this._setCustomLabel();
            this._setPointers();
            this._setIndicators();
        },
		/**
         * To Redraw the Label
		 * @private
         */	
        _reDrawLabel: function () {
            this.contextEl.putImageData(this._getLabelImage(), 0, 0);
            this._setLabels();
            this._setRanges();
            this._subGauge();
            this._setCustomLabel();
            this._setPointers();
            this._setIndicators();
        },
		/**
         * To Redraw the TickMark
		 * @private
         */	
        _reDrawTickMark: function () {
            this.contextEl.putImageData(this._getTickImage(), 0, 0);
            this._setTicks();
            this._setLabels();
            this._setRanges();
            this._subGauge();
            this._setCustomLabel();
            this._setPointers();
            this._setIndicators();
        },
		/**
         * To Redraw the SubGauge
		 * @private
         */	
        _reDrawSubGauge: function () {
            this.contextEl.putImageData(this._getSubGaugeImage(), 0, 0);
            this._subGauge();
            this._setCustomLabel();
            this._setPointers();
            this._setIndicators();
        },

        // ************************************* Client side Methods ***********************************
		/**
         * To check whether the device is andriod or not
		 * @private
         */	
        isAndroid: function () {
            return (/android/i.test(navigator.userAgent.toLowerCase()));
        },
        /**
          * Switching the redraw option for the gauge
          * @return jQuery
          * @param {string} argument.value redraw value for the gauge
          * @example 
          * &lt;div id="CoreCircularGauge"&gt;&lt;/div&gt; <br>  
          * &lt;script&gt;
          * $("#CoreCircularGauge").ejCircularGauge();
          * var circulargaugeObj = $("#CoreCircularGauge").data("ejCircularGauge");
          * circulargaugeObj.redraw("scale");
          * &lt;/script&gt;
          *@memberof ejCircularGauge
          * @instance
        */
        redraw: function (value) {
            switch (value) {
                case "scale":
                    this._reDrawScale(); break;
                case "pointer":
                    this._reDrawPointer(); break;
                case "range":
                    this._reDrawRange(); break;
                case "label":
                    this._reDrawLabel(); break;
                case "tickMark":
                    this._reDrawTickMark();
                    break;
                case "subGauges":
                    this._reDrawSubGauge();
                    break;
                case "CustomLabel":
                    this._reDrawCustomLabel();
                    break;
            }
        },
		/**
         * To get Indicator Image
		 * @private
         */	
        _getIndicatorImage: function () {
            if (this.pointerImage)
                return this.pointerImage;
            else
                this._getPointerImage();
        },
		/**
         * To get Pointer Image
		 * @private
         */	
        _getPointerImage: function () {
            if (this.customLabelImage)
                return this.customLabelImage;
            else
                return this._getCustomLabelImage();
        },
		/**
         * To get sub gauge Image
		 * @private
         */	
        _getSubGaugeImage: function () {
            if (this.rangeImage)
                return this.rangeImage;
            else
                return this._getRangeImage();
        },
		/**
         * To get Custom Label Image
		 * @private
         */	
        _getCustomLabelImage: function () {
            if (this.subGaugeImage)
                return this.subGaugeImage;
            else
                return this._getSubGaugeImage();
        },
		/**
         * To get Range Image
		 * @private
         */	
        _getRangeImage: function () {
            if (this.labelImage)
                return this.labelImage;
            else
                return this._getLabelImage();
        },
		/**
         * To get Label Image
		 * @private
         */	
        _getLabelImage: function () {
            if (this.tickImage)
                return this.tickImage;
            else
                return this._getTickImage();
        },
		/**
         * To get Tick Image
		 * @private
         */	
        _getTickImage: function () {
            if (this.scaleImage)
                return this.scaleImage;
            else
                return this.outerImage;
        },
        /**
        * To get IndicatorValue
          * @return number
          * @param {number} argument.scaleIndex scaleIndex value for the Gauge 
          * @param {number} argument.indicatorIndex indicatorIndex value for the Gauge
          * @example 
          * &lt;div id="CoreCircularGauge"&gt;&lt;/div&gt; <br>  
          * &lt;script&gt;
          * $("#CoreCircularGauge").ejCircularGauge({ scales: [{showIndicators:true,indicators: [{
          *   height: 30,width: 10,type: "circle",value: 0,position: { x: 185, y: 300 },
          *   stateRanges: [{ endValue: 70, startValue: 0, backgroundColor: "#5DF243", borderColor: "#5DF243", text: "", textColor: "#870505" },
          *   { endValue: 200, startValue: 70, backgroundColor: "#145608", borderColor: "#145608", text: "", textColor: "#870505" }]}]}]});
          * var circulargaugeObj = $("#CoreCircularGauge").data("ejCircularGauge");
          * circulargaugeObj.getIndicatorValue(0, 0);
          * &lt;/script&gt;
          *@memberof ejCircularGauge
          * @instance
        */
        getIndicatorValue: function (scaleIndex, indicatorIndex) {
            if (scaleIndex < this.model.scales.length && indicatorIndex < this.model.scales[scaleIndex].indicators.length)
                return this.scaleEl[scaleIndex].indicators[indicatorIndex].value;
        },
        /**
          * To set IndicatorValue 
          * @return jQuery
          * @param {number} argument.scaleIndex scaleIndex value for the gauge
          * @param {number} argument.indicatorIndex indicatorIndex value for the gauge
          * @param {number} argument.value value for the gauge
          * @example 
          * &lt;div id="CoreCircularGauge"&gt;&lt;/div&gt; <br>  
          * &lt;script&gt;
          * $("#CoreCircularGauge").ejCircularGauge({ scales: [{showIndicators:true,indicators: [{
          *   height: 30,width: 10,type: "circle",value: 0,position: { x: 185, y: 300 },
          *   stateRanges: [{ endValue: 70, startValue: 0, backgroundColor: "#5DF243", borderColor: "#5DF243", text: "", textColor: "#870505" },
          *   { endValue: 200, startValue: 70, backgroundColor: "#145608", borderColor: "#145608", text: "", textColor: "#870505" }]}]}]});
          * var circulargaugeObj = $("#CoreCircularGauge").data("ejCircularGauge");
          * circulargaugeObj.setIndicatorValue(0, 0, 10);
          * &lt;/script&gt;
          *@memberof ejCircularGauge
          * @instance
        */
        setIndicatorValue: function (scaleIndex, indicatorIndex, value) {
            if (scaleIndex < this.model.scales.length && indicatorIndex < this.model.scales[scaleIndex].indicators.length && value != null) {
                this.scaleEl[scaleIndex].indicators[indicatorIndex].value = value;
                if (this.contextEl.putImageData && !this.isAndroid())
                    this._reDrawPointer();
                else
                    this._initialize();
            }
        },
        /**
        * To get PointerValue
          * @return number
          * @param {number} argument.scaleIndex scaleIndex value for the Gauge 
          * @param {number} argument.pointerIndex pointerIndex value for the Gauge
          * @example 
          * &lt;div id="CoreCircularGauge"&gt;&lt;/div&gt; <br>  
          * &lt;script&gt;
          * $("#CoreCircularGauge").ejCircularGauge();
          * var circulargaugeObj = $("#CoreCircularGauge").data("ejCircularGauge");
          * circulargaugeObj.getPointerValue(0, 0);
          * &lt;/script&gt;
          *@memberof ejCircularGauge
          * @instance
        */
        getPointerValue: function (scaleIndex, pointerIndex) {
            if (scaleIndex < this.model.scales.length && pointerIndex < this.model.scales[scaleIndex].pointers.length)
                return this.scaleEl[scaleIndex].pointers[pointerIndex].value;
        },
        /**
          * To set MarkerDistanceFromScale 
          * @return jQuery
          * @param {number} argument.scaleIndex scaleIndex value for the gauge
          * @param {number} argument.pointerIndex pointerIndex value for the gauge
          * @param {number} argument.value value for the gauge
          * @example 
          * &lt;div id="CoreCircularGauge"&gt;&lt;/div&gt; <br>  
          * &lt;script&gt;
          * $("#CoreCircularGauge").ejCircularGauge();
          * var circulargaugeObj = $("#CoreCircularGauge").data("ejCircularGauge");
          * circulargaugeObj.setMarkerDistanceFromScale(0, 0, 10);
          * &lt;/script&gt;
          * @memberof ejCircularGauge
          * @instance
        */
        setMarkerDistanceFromScale: function (scaleIndex, pointerIndex, value) {
            if (scaleIndex < this.model.scales.length && pointerIndex < this.model.scales[scaleIndex].pointers.length && value != null) {
                this.scaleEl[scaleIndex].pointers[pointerIndex].distanceFromScale = value;
                if (this.contextEl.putImageData && !this.isAndroid())
                    this._reDrawPointer();
                else
                    this._initialize();
            }
        },
        /**
        * To get MarkerDistanceFromScale
          * @return number
          * @param {number} argument.scaleIndex scaleIndex value for the Gauge 
          * @param {number} argument.pointerIndex pointerIndex value for the Gauge
          * @example 
          * &lt;div id="CoreCircularGauge"&gt;&lt;/div&gt; <br>  
          * &lt;script&gt;
          * $("#CoreCircularGauge").ejCircularGauge();
          * var circulargaugeObj = $("#CoreCircularGauge").data("ejCircularGauge");
          * circulargaugeObj.getMarkerDistanceFromScale(0, 0);
          * &lt;/script&gt;
          *@memberof ejCircularGauge
          * @instance
        */
        getMarkerDistanceFromScale: function (scaleIndex, pointerIndex) {
            if (scaleIndex < this.model.scales.length && pointerIndex < this.model.scales[scaleIndex].pointers.length)
                return this.scaleEl[scaleIndex].pointers[pointerIndex].distanceFromScale;
        },
        /**
          * To set PointerLength 
          * @return jQuery
          * @param {number} argument.scaleIndex scaleIndex value for the gauge
          * @param {number} argument.pointerIndex pointerIndex value for the gauge
          * @param {number} argument.value value for the gauge
          * @example 
          * &lt;div id="CoreCircularGauge"&gt;&lt;/div&gt; <br>  
          * &lt;script&gt;
          * $("#CoreCircularGauge").ejCircularGauge();
          * var circulargaugeObj = $("#CoreCircularGauge").data("ejCircularGauge");
          * circulargaugeObj.setPointerLength(0, 0, 90);
          * &lt;/script&gt;
          *@memberof ejCircularGauge
          * @instance
        */
        setPointerLength: function (scaleIndex, pointerIndex, value) {
            if (scaleIndex < this.model.scales.length && pointerIndex < this.model.scales[scaleIndex].pointers.length && value != null) {
                this.scaleEl[scaleIndex].pointers[pointerIndex].length = value;
                if (this.contextEl.putImageData && !this.isAndroid())
                    this._reDrawPointer();
                else
                    this._initialize();
            }
        },
        /**
        * To get PointerLength
          * @return number
          * @param {number} argument.scaleIndex scaleIndex value for the Gauge 
          * @param {number} argument.pointerIndex pointerIndex value for the Gauge
          * @example 
          * &lt;div id="CoreCircularGauge"&gt;&lt;/div&gt; <br>  
          * &lt;script&gt;
          * $("#CoreCircularGauge").ejCircularGauge();
          * var circulargaugeObj = $("#CoreCircularGauge").data("ejCircularGauge");
          * circulargaugeObj.getPointerLength(0, 0);
          * &lt;/script&gt;
          *@memberof ejCircularGauge
          * @instance
        */
        getPointerLength: function (scaleIndex, pointerIndex) {
            if (scaleIndex < this.model.scales.length && pointerIndex < this.model.scales[scaleIndex].pointers.length)
                return this.scaleEl[scaleIndex].pointers[pointerIndex].length;
        },
        /**
          * To set PointerWidth 
          * @return jQuery
          * @param {number} argument.scaleIndex scaleIndex value for the gauge
          * @param {number} argument.pointerIndex pointerIndex value for the gauge
          * @param {number} argument.value value for the gauge
          * @example 
          * &lt;div id="CoreCircularGauge"&gt;&lt;/div&gt; <br>  
          * &lt;script&gt;
          * $("#CoreCircularGauge").ejCircularGauge();
          * var circulargaugeObj = $("#CoreCircularGauge").data("ejCircularGauge");
          * circulargaugeObj.setPointerWidth(0, 0, 10);
          * &lt;/script&gt;
          * @memberof ejCircularGauge
          * @instance
        */
        setPointerWidth: function (scaleIndex, pointerIndex, value) {
            if (scaleIndex < this.model.scales.length && pointerIndex < this.model.scales[scaleIndex].pointers.length && value != null) {
                this.scaleEl[scaleIndex].pointers[pointerIndex].width = value;
                if (this.contextEl.putImageData && !this.isAndroid())
                    this._reDrawPointer();
                else this._initialize();
            }
        },
        /**
        * To get PointerWidth
          * @return number
          * @param {number} argument.scaleIndex scaleIndex value for the Gauge 
          * @param {number} argument.pointerIndex pointerIndex value for the Gauge
          * @example 
          * &lt;div id="CoreCircularGauge"&gt;&lt;/div&gt; <br>  
          * &lt;script&gt;
          * $("#CoreCircularGauge").ejCircularGauge();
          * var circulargaugeObj = $("#CoreCircularGauge").data("ejCircularGauge");
          * circulargaugeObj.getPointerWidth(0, 0);
          * &lt;/script&gt;
          *@memberof ejCircularGauge
          * @instance
        */
        getPointerWidth: function (scaleIndex, pointerIndex) {
            if (scaleIndex < this.model.scales.length && pointerIndex < this.model.scales[scaleIndex].pointers.length)
                return this.scaleEl[scaleIndex].pointers[pointerIndex].width;
        },

        /**
          * To set BackNeedleLength 
          * @return jQuery
          * @param {number} argument.scaleIndex scaleIndex value for the gauge 
          * @param {number} argument.pointerIndex pointerIndex value for the gauge
          * @param {number} argument.value value for the gauge
          * @example 
          * &lt;div id="CoreCircularGauge"&gt;&lt;/div&gt; <br>  
          * &lt;script&gt;
          * $("#CoreCircularGauge").ejCircularGauge({ scales: [{ pointers: [{ showBackNeedle: true }] }] });
          * var circulargaugeObj = $("#CoreCircularGauge").data("ejCircularGauge");
          * circulargaugeObj.setBackNeedleLength(0, 0, 10);
          * &lt;/script&gt;
          *@memberof ejCircularGauge
          * @instance
        */
        setBackNeedleLength: function (scaleIndex, pointerIndex, value) {
            if (scaleIndex < this.model.scales.length && pointerIndex < this.model.scales[scaleIndex].pointers.length && value != null) {
                this.scaleEl[scaleIndex].pointers[pointerIndex].backNeedleLength = value;
                if (this.contextEl.putImageData && !this.isAndroid())
                    this._reDrawPointer();
                else this._initialize();
            }
        },
        /**
        * To get BackNeedleLength
          * @return number
          * @param {number} argument.scaleIndex scaleIndex value for the Gauge 
          * @param {number} argument.pointerIndex pointerIndex value for the Gauge
          * @example 
          * &lt;div id="CoreCircularGauge"&gt;&lt;/div&gt; <br>  
          * &lt;script&gt;
          * $("#CoreCircularGauge").ejCircularGauge({ scales: [{ pointers: [{ showBackNeedle: true }] }] });
          * var circulargaugeObj = $("#CoreCircularGauge").data("ejCircularGauge");
          * circulargaugeObj.getBackNeedleLength(0, 0);
          * &lt;/script&gt;
          *@memberof ejCircularGauge
          * @instance
        */
        getBackNeedleLength: function (scaleIndex, pointerIndex) {
            if (scaleIndex < this.model.scales.length && pointerIndex < this.model.scales[scaleIndex].pointers.length)
                return this.scaleEl[scaleIndex].pointers[pointerIndex].backNeedleLength;
        },
        /**
          * To set NeedleStyle 
          * @return jQuery
          * @param {number} argument.scaleIndex scaleIndex value for the gauge
          * @param {number} argument.pointerIndex pointerIndex value for the gauge
          * @param {number} argument.value value for the gauge
          * @example 
          * &lt;div id="CoreCircularGauge"&gt;&lt;/div&gt; <br>  
          * &lt;script&gt;
          * $("#CoreCircularGauge").ejCircularGauge();
          * var circulargaugeObj = $("#CoreCircularGauge").data("ejCircularGauge");
          * circulargaugeObj.setNeedleStyle(0, 0, "arrow");
          * &lt;/script&gt;
          *@memberof ejCircularGauge
          * @instance
        */
        setNeedleStyle: function (scaleIndex, pointerIndex, value) {
            if (scaleIndex < this.model.scales.length && pointerIndex < this.model.scales[scaleIndex].pointers.length && value != null) {
                this.scaleEl[scaleIndex].pointers[pointerIndex].needleType = value;
                if (this.contextEl.putImageData && !this.isAndroid())
                    this._reDrawPointer();
                else this._initialize();
            }
        },
        /**
        * To get NeedleStyle
          * @return number
          * @param {number} argument.scaleIndex scaleIndex value for the Gauge 
          * @param {number} argument.pointerIndex pointerIndex value for the Gauge
          * @example 
          * &lt;div id="CoreCircularGauge"&gt;&lt;/div&gt; <br>  
          * &lt;script&gt;
          * $("#CoreCircularGauge").ejCircularGauge();
          * var circulargaugeObj = $("#CoreCircularGauge").data("ejCircularGauge");
          * circulargaugeObj.getNeedleStyle(0, 0);
          * &lt;/script&gt;
          *@memberof ejCircularGauge
          * @instance
        */
        getNeedleStyle: function (scaleIndex, pointerIndex) {
            if (scaleIndex < this.model.scales.length && pointerIndex < this.model.scales[scaleIndex].pointers.length)
                return this.scaleEl[scaleIndex].pointers[pointerIndex].needleType;
        },
        /**
          * To set PointerPlacement 
          * @return jQuery
          * @param {number} argument.scaleIndex scaleIndex value for the gauge
          * @param {number} argument.pointerIndex pointerIndex value for the gauge
          * @param {number} argument.value value for the gauge
          * @example 
          * &lt;div id="CoreCircularGauge"&gt;&lt;/div&gt; <br>  
          * &lt;script&gt;
          * $("#CoreCircularGauge").ejCircularGauge();
          * var circulargaugeObj = $("#CoreCircularGauge").data("ejCircularGauge");
          * circulargaugeObj.setPointerPlacement(0, 0,"near");
          * &lt;/script&gt;
          *@memberof ejCircularGauge
          * @instance
        */
        setPointerPlacement: function (scaleIndex, pointerIndex, value) {
            if (scaleIndex < this.model.scales.length && pointerIndex < this.model.scales[scaleIndex].pointers.length && value != null) {
                this.scaleEl[scaleIndex].pointers[pointerIndex].pointerPlacement = value;
                if (this.contextEl.putImageData && !this.isAndroid())
                    this._reDrawPointer();
                else this._initialize();
            }
        },
		/**
        * To get PointerPlacement
          * @return number
          * @param {number} argument.scaleIndex scaleIndex value for the Gauge 
          * @param {number} argument.pointerIndex pointerIndex value for the Gauge
          * @example 
          * &lt;div id="CoreCircularGauge"&gt;&lt;/div&gt; <br>  
          * &lt;script&gt;
          * $("#CoreCircularGauge").ejCircularGauge();
          * var circulargaugeObj = $("#CoreCircularGauge").data("ejCircularGauge");
          * circulargaugeObj.getPointerPlacement(0, 0);
          * &lt;/script&gt;
          *@memberof ejCircularGauge
          * @instance
        */
        getPointerPlacement: function (scaleIndex, pointerIndex) {
            if (scaleIndex < this.model.scales.length && pointerIndex < this.model.scales[scaleIndex].pointers.length)
                return this.scaleEl[scaleIndex].pointers[pointerIndex].pointerPlacement;
        },
        /**
          * To set PointerNeedleType 
          * @return jQuery
          * @param {number} argument.scaleIndex scaleIndex value for the gauge
          * @param {number} argument.pointerIndex pointerIndex value for the gauge
          * @param {number} argument.value value for the gauge
          * @example 
          * &lt;div id="CoreCircularGauge"&gt;&lt;/div&gt; <br>  
          * &lt;script&gt;
          * $("#CoreCircularGauge").ejCircularGauge();
          * var circulargaugeObj = $("#CoreCircularGauge").data("ejCircularGauge");
          * circulargaugeObj.setPointerNeedleType(0, 0, "triangle");
          * &lt;/script&gt;
          *@memberof ejCircularGauge
          * @instance
        */
        setPointerNeedleType: function (scaleIndex, pointerIndex, value) {
            if (scaleIndex < this.model.scales.length && pointerIndex < this.model.scales[scaleIndex].pointers.length && value != null) {
                this.scaleEl[scaleIndex].pointers[pointerIndex].pointerNeedleType = value;
                if (this.contextEl.putImageData && !this.isAndroid())
                    this._reDrawCustomLabel();
                else this._initialize();
            }
        },
        /**
        * To get PointerNeedleType
          * @return number
          * @param {number} argument.scaleIndex scaleIndex value for the Gauge 
          * @param {number} argument.pointerIndex pointerIndex value for the Gauge
          * @example 
          * &lt;div id="CoreCircularGauge"&gt;&lt;/div&gt; <br>  
          * &lt;script&gt;
          * $("#CoreCircularGauge").ejCircularGauge();
          * var circulargaugeObj = $("#CoreCircularGauge").data("ejCircularGauge");
          * circulargaugeObj.getPointerNeedleType(0, 0);
          * &lt;/script&gt;
          *@memberof ejCircularGauge
          * @instance
        */
        getPointerNeedleType: function (scaleIndex, pointerIndex) {
            if (scaleIndex < this.model.scales.length && pointerIndex < this.model.scales[scaleIndex].pointers.length)
                return this.scaleEl[scaleIndex].pointers[pointerIndex].pointerNeedleType;
        },
        /**
          * To set MarkerStyle 
          * @return jQuery
          * @param {number} argument.scaleIndex scaleIndex value for the gauge
          * @param {number} argument.pointerIndex pointerIndex value for the gauge
          * @param {number} argument.value value for the gauge
          * @example 
          * &lt;div id="CoreCircularGauge"&gt;&lt;/div&gt; <br>  
          * &lt;script&gt;
          * $("#CoreCircularGauge").ejCircularGauge();
          * var circulargaugeObj = $("#CoreCircularGauge").data("ejCircularGauge");
          * circulargaugeObj.setMarkerStyle(0, 0, "rectangle");
          * &lt;/script&gt;
          *@memberof ejCircularGauge
          * @instance
        */
        setMarkerStyle: function (scaleIndex, pointerIndex, value) {
            if (scaleIndex < this.model.scales.length && pointerIndex < this.model.scales[scaleIndex].pointers.length && value != null) {
                var Exist = false;
                if (value == "roundedrectangle" || !ej.isNullOrUndefined(ej.datavisualization.CircularGauge.MarkerType[value.replace(/\w\S*/g, function (txt) { return txt.charAt(0).toUpperCase() + txt.substr(1).toLowerCase(); })])) {
                    this.scaleEl[scaleIndex].pointers[pointerIndex].markerType = value;
                    if (this.contextEl.putImageData && !this.isAndroid())
                        this._reDrawCustomLabel();
                    else this._initialize();
                }
            }
        },
        /**
        * To get MarkerStyle
          * @return number
          * @param {number} argument.scaleIndex scaleIndex value for the Gauge 
          * @param {number} argument.pointerIndex pointerIndex value for the Gauge
          * @example 
          * &lt;div id="CoreCircularGauge"&gt;&lt;/div&gt; <br>  
          * &lt;script&gt;
          * $("#CoreCircularGauge").ejCircularGauge();
          * var circulargaugeObj = $("#CoreCircularGauge").data("ejCircularGauge");
          * circulargaugeObj.getMarkerStyle(0, 0);
          * &lt;/script&gt;
          *@memberof ejCircularGauge
          * @instance
        */
        getMarkerStyle: function (scaleIndex, pointerIndex) {
            if (scaleIndex < this.model.scales.length && pointerIndex < this.model.scales[scaleIndex].pointers.length)
                return this.scaleEl[scaleIndex].pointers[pointerIndex].markerType;
        },
        /**
          * To set CustomLabelValue 
          * @return jQuery
          * @param {number} argument.scaleIndex scaleIndex value for the gauge
          * @param {number} argument.customLabelIndex customLabelIndex value for the gauge
          * @param {number} argument.value value for the gauge
          * @example 
          * &lt;div id="CoreCircularGauge"&gt;&lt;/div&gt; <br>  
          * &lt;script&gt;
          * $("#CoreCircularGauge").ejCircularGauge({scales: [{customLabels:[{value:"MyLabel",position:{x:180,y:300},color:"#fc0606",font:{size: "20px", fontFamily: "Arial", fontStyle: "Bold" }}]}]});
          * var circulargaugeObj = $("#CoreCircularGauge").data("ejCircularGauge");
          * circulargaugeObj.setCustomLabelValue(0, 0, "CircularGauge");
          * &lt;/script&gt;
          *@memberof ejCircularGauge
          * @instance
        */
        setCustomLabelValue: function (scaleIndex, customLabelIndex, value) {
            if (scaleIndex < this.model.scales.length && customLabelIndex < this.model.scales[scaleIndex].customLabels.length && value != null) {
                this.scaleEl[scaleIndex].customLabels[customLabelIndex].value = value;
                if (this.contextEl.putImageData && !this.isAndroid())
                    this._reDrawCustomLabel();
                else
                    this._initialize();
            }
        },
        /**
        * To get CustomLabelValue
          * @return number
          * @param {number} argument.scaleIndex scaleIndex value for the Gauge 
          * @param {number} argument.customLabelIndex customLabelIndex value for the Gauge
          * @example 
          * &lt;div id="CoreCircularGauge"&gt;&lt;/div&gt; <br>  
          * &lt;script&gt;
          * $("#CoreCircularGauge").ejCircularGauge({scales: [{customLabels:[{textAngle:30,value:"MyLabel",position:{x:250,y:300},color:"#fc0606",font:{size: "20px", fontFamily: "Arial", fontStyle: "Bold" }}]}]});
          * var circulargaugeObj = $("#CoreCircularGauge").data("ejCircularGauge");
          * circulargaugeObj.getCustomLabelValue(0, 0);
          * &lt;/script&gt;
          *@memberof ejCircularGauge
          * @instance
        */
        getCustomLabelValue: function (scaleIndex, customLabelIndex) {
            if (scaleIndex < this.model.scales.length && customLabelIndex < this.model.scales[scaleIndex].customLabels.length)
                return this.scaleEl[scaleIndex].customLabels[customLabelIndex].value;
        },

        /**
          * To set SubGaugeLocation 
          * @return jQuery
          * @param {number} argument.scaleIndex scaleIndex value for the gauge
          * @param {number} argument.GaugeIndex GaugeIndex value for the gauge
          * @param {number} argument.value value for the gauge
          * @example 
          * &lt;div id="CoreCircularGauge"&gt;&lt;/div&gt; <br> 
          * &lt;div id="subGauge1"&gt;
          * &lt;/div&gt; <br>
          * &lt;script&gt;
          * $("#subGauge1").ejCircularGauge({backgroundColor: "#f5b43f",scales: [{radius: 150}]});
          * $("#CoreCircularGauge").ejCircularGauge({height: 500,width: 500,scales: [{radius: 250,subGauges: [{controlID: "subGauge1",height: 400,width: 200,position: { x: 200, y: 150 }}]}]});
          * var circulargaugeObj = $("#CoreCircularGauge").data("ejCircularGauge");
          * circulargaugeObj.setSubGaugeLocation(0, 0, {x:50,y:100});
          * &lt;/script&gt;
          *@memberof ejCircularGauge
          * @instance
        */
        setSubGaugeLocation: function (scaleIndex, GaugeIndex, value) {
            if (scaleIndex < this.model.scales.length && GaugeIndex < this.model.scales[scaleIndex].subGauges.length && value != null) {
                this.scaleEl[scaleIndex].subGauges[GaugeIndex].position.x = value.x;
                this.scaleEl[scaleIndex].subGauges[GaugeIndex].position.y = value.y;
                if (this.contextEl.putImageData && !this.isAndroid())
                    this._reDrawSubGauge();
                else
                    this._initialize();
            }
        },
        /**
        * To get SubGaugeLocation
          * @return number
          * @param {number} argument.scaleIndex scaleIndex value for the Gauge 
          * @param {number} argument.GaugeIndex GaugeIndex value for the Gauge
          * @example 
          * &lt;div id="CoreCircularGauge"&gt;&lt;/div&gt; <br>  
          * &lt;div id="subGauge1"&gt;
          * &lt;/div&gt; <br> 
          * &lt;script&gt;
          * $("#subGauge1").ejCircularGauge({backgroundColor: "#f5b43f",scales: [{radius: 150}]});
          * $("#CoreCircularGauge").ejCircularGauge({height: 500,width: 500,scales: [{radius: 250,subGauges: [{controlID: "subGauge1",height: 400,width: 200,position: { x: 200, y: 150 }}]}]});
          * circulargaugeObj.getSubGaugeLocation(0, 0);
          * &lt;/script&gt;
          * @memberof ejCircularGauge
          * @instance
        */
        getSubGaugeLocation: function (scaleIndex, GaugeIndex) {
            if (scaleIndex < this.model.scales.length && GaugeIndex < this.model.scales[scaleIndex].subGauges.length)
                return this.scaleEl[scaleIndex].subGauges[GaugeIndex].Location;
        },
        /**
          * To set CustomLabelAngle 
          * @return jQuery
          * @param {number} argument.scaleIndex scaleIndex value for the gauge
          * @param {number} argument.customLabelIndex customLabelIndex value for the gauge
          * @param {number} argument.value value for the gauge
          * @example 
          * &lt;div id="CoreCircularGauge"&gt;&lt;/div&gt; <br>  
          * &lt;script&gt;
          * $("#CoreCircularGauge").ejCircularGauge({scales: [{customLabels:[{value:"MyLabel",position:{x:250,y:300},color:"#fc0606",font: { size: "20px", fontFamily: "Arial", fontStyle: "Bold" }}]}]});
          * var circulargaugeObj = $("#CoreCircularGauge").data("ejCircularGauge");
          * circulargaugeObj.setCustomLabelAngle(0, 0, 10);
          * &lt;/script&gt;
          *@memberof ejCircularGauge
          * @instance
        */
        setCustomLabelAngle: function (scaleIndex, customLabelIndex, value) {
            if (scaleIndex < this.model.scales.length && customLabelIndex < this.model.scales[scaleIndex].customLabels.length && value != null) {
                this.scaleEl[scaleIndex].customLabels[customLabelIndex].textAngle = value;
                if (this.contextEl.putImageData && !this.isAndroid())
                    this._reDrawCustomLabel();
                else
                    this._initialize();
            }
        },
        /**
        * To get CustomLabelAngle
          * @return number
          * @param {number} argument.scaleIndex scaleIndex value for the Gauge 
          * @param {number} argument.customLabelIndex customLabelIndex value for the Gauge
          * @example 
          * &lt;div id="CoreCircularGauge"&gt;&lt;/div&gt; <br> 
          * &lt;script&gt;
          * $("#CoreCircularGauge").ejCircularGauge({scales: [{customLabels:[{textAngle:30,value:"MyLabel",position:{x:250,y:300},color:"#fc0606",font:{size: "20px", fontFamily: "Arial", fontStyle: "Bold" }}]}]});
          * var circulargaugeObj = $("#CoreCircularGauge").data("ejCircularGauge");
          * circulargaugeObj.getCustomLabelAngle(0, 0);
          * &lt;/script&gt;
          *@memberof ejCircularGauge
          * @instance
        */
        getCustomLabelAngle: function (scaleIndex, customLabelIndex) {
            if (scaleIndex < this.model.scales.length && customLabelIndex < this.model.scales[scaleIndex].customLabels.length)
                return this.scaleEl[scaleIndex].customLabels[customLabelIndex].textAngle;
        },
        /**
          * To set RangeStartValue 
          * @return jQuery
          * @param {number} argument.scaleIndex scaleIndex value for the gauge
          * @param {number} argument.rangeIndex rangeIndex value for the gauge
          * @param {number} argument.value value for the gauge
          * @example 
          * &lt;div id="CoreCircularGauge"&gt;&lt;/div&gt; <br>  
          * &lt;script&gt;
          * $("#CoreCircularGauge").ejCircularGauge({scales: [{showRanges: true,ranges: [{distanceFromScale: -30,startValue: 0,endValue: 70}]}]});
          * var circulargaugeObj = $("#CoreCircularGauge").data("ejCircularGauge");
          * circulargaugeObj.setRangeStartValue(0, 0, 10);
          * &lt;/script&gt;
          *@memberof ejCircularGauge
          * @instance
        */
        setRangeStartValue: function (scaleIndex, rangeIndex, value) {
            if (scaleIndex < this.model.scales.length && rangeIndex < this.model.scales[scaleIndex].ranges.length && value != null) {
                this.scaleEl[scaleIndex].ranges[rangeIndex].startValue = value;
                if (this.contextEl.putImageData && !this.isAndroid())
                    this._reDrawRange();
                else
                    this._initialize();
            }
        },
        /**
        * To get RangeStartValue
          * @return number
          * @param {number} argument.scaleIndex scaleIndex value for the Gauge 
          * @param {number} argument.rangeIndex rangeIndex value for the Gauge
          * @example 
          * &lt;div id="CoreCircularGauge"&gt;&lt;/div&gt; <br>  
          * &lt;script&gt;
          * $("#CoreCircularGauge").ejCircularGauge({scales: [{showRanges: true,ranges: [{distanceFromScale: -30,startValue: 0,endValue: 70}]}]});
          * var circulargaugeObj = $("#CoreCircularGauge").data("ejCircularGauge");
          * circulargaugeObj.getRangeStartValue(0, 0);
          * &lt;/script&gt;
          *@memberof ejCircularGauge
          * @instance
        */
        getRangeStartValue: function (scaleIndex, rangeIndex) {
            if (scaleIndex < this.model.scales.length && rangeIndex < this.model.scales[scaleIndex].ranges.length)
                return this.scaleEl[scaleIndex].ranges[rangeIndex].startValue;
        },
        /**
          * To set RangeEndValue 
          * @return jQuery
          * @param {number} argument.scaleIndex scaleIndex value for the gauge
          * @param {number} argument.rangeIndex rangeIndex value for the gauge
          * @param {number} argument.value value for the gauge
          * @example 
          * &lt;div id="CoreCircularGauge"&gt;&lt;/div&gt; <br>  
          * &lt;script&gt;
          * $("#CoreCircularGauge").ejCircularGauge({scales: [{showRanges: true,ranges: [{distanceFromScale: -30,startValue: 0,endValue: 70}]}]});
          * var circulargaugeObj = $("#CoreCircularGauge").data("ejCircularGauge");
          * circulargaugeObj.setRangeEndValue(0, 0, 70);
          * &lt;/script&gt;
          *@memberof ejCircularGauge
          * @instance
        */
        setRangeEndValue: function (scaleIndex, rangeIndex, value) {
            if (scaleIndex < this.model.scales.length && rangeIndex < this.model.scales[scaleIndex].ranges.length && value != null) {
                this.scaleEl[scaleIndex].ranges[rangeIndex].endValue = value;
                if (this.contextEl.putImageData && !this.isAndroid())
                    this._reDrawRange();
                else
                    this._initialize();
            }
        },
        /**
        * To get RangeEndValue
          * @return number
          * @param {number} argument.scaleIndex scaleIndex value for the Gauge 
          * @param {number} argument.rangeIndex rangeIndex value for the Gauge
          * @example 
          * &lt;div id="CoreCircularGauge"&gt;&lt;/div&gt; <br>  
          * &lt;script&gt;
          * $("#CoreCircularGauge").ejCircularGauge({scales: [{showRanges: true,ranges: [{distanceFromScale: -30,startValue: 0,endValue: 70}]}]});
          * var circulargaugeObj = $("#CoreCircularGauge").data("ejCircularGauge");
          * circulargaugeObj.getRangeEndValue(0, 0);
          * &lt;/script&gt;
          *@memberof ejCircularGauge
          * @instance
        */
        getRangeEndValue: function (scaleIndex, rangeIndex) {
            if (scaleIndex < this.model.scales.length && rangeIndex < this.model.scales[scaleIndex].ranges.length)
                return this.scaleEl[scaleIndex].ranges[rangeIndex].endValue;
        },
        /**
          * To set RangeSize 
          * @return jQuery
          * @param {number} argument.scaleIndex scaleIndex value for the gauge
          * @param {number} argument.rangeIndex rangeIndex value for the gauge
          * @param {number} argument.value value for the gauge
          * @example 
          * &lt;div id="CoreCircularGauge"&gt;&lt;/div&gt; <br>  
          * &lt;script&gt;
          * $("#CoreCircularGauge").ejCircularGauge({scales: [{showRanges: true,ranges: [{distanceFromScale: -30,startValue: 0,endValue: 70}]}]});
          * var circulargaugeObj = $("#CoreCircularGauge").data("ejCircularGauge");
          * circulargaugeObj.setRangeSize(0, 0, 10);
          * &lt;/script&gt;
          *@memberof ejCircularGauge
          * @instance
        */
        setRangeSize: function (scaleIndex, rangeIndex, value) {
            if (scaleIndex < this.model.scales.length && rangeIndex < this.model.scales[scaleIndex].ranges.length && value != null) {
                this.scaleEl[scaleIndex].ranges[rangeIndex].size = value;
                if (this.contextEl.putImageData && !this.isAndroid())
                    this._reDrawRange();
                else
                    this._initialize();
            }
        },
        /**
        * To get RangeSize
          * @return number
          * @param {number} argument.scaleIndex scaleIndex value for the Gauge 
          * @param {number} argument.rangeIndex rangeIndex value for the Gauge
          * @example 
          * &lt;div id="CoreCircularGauge"&gt;&lt;/div&gt; <br>  
          * &lt;script&gt;
          * $("#CoreCircularGauge").ejCircularGauge({scales: [{showRanges: true,ranges: [{distanceFromScale: -30,startValue: 0,endValue: 70}]}]});
          * var circulargaugeObj = $("#CoreCircularGauge").data("ejCircularGauge");
          * circulargaugeObj.getRangeSize(0, 0);
          * &lt;/script&gt;
          *@memberof ejCircularGauge
          * @instance
        */
        getRangeSize: function (scaleIndex, rangeIndex) {
            if (scaleIndex < this.model.scales.length && rangeIndex < this.model.scales[scaleIndex].ranges.length)
                return this.scaleEl[scaleIndex].ranges[rangeIndex].size;
        },
        /**
          * To set RangeDistanceFromScale 
          * @return jQuery
          * @param {number} argument.scaleIndex scaleIndex value for the gauge
          * @param {number} argument.rangeIndex rangeIndex value for the gauge
          * @param {number} argument.value value for the gauge
          * @example 
          * &lt;div id="CoreCircularGauge"&gt;&lt;/div&gt; <br>  
          * &lt;script&gt;
          * $("#CoreCircularGauge").ejCircularGauge({scales: [{showRanges: true,ranges: [{distanceFromScale: -30,startValue: 0,endValue: 70}]}]});
          * var circulargaugeObj = $("#CoreCircularGauge").data("ejCircularGauge");
          * circulargaugeObj.setRangeDistanceFromScale(0, 0, 10);
          * &lt;/script&gt;
          *@memberof ejCircularGauge
          * @instance
        */
        setRangeDistanceFromScale: function (scaleIndex, rangeIndex, value) {
            if (scaleIndex < this.model.scales.length && rangeIndex < this.model.scales[scaleIndex].ranges.length && value != null) {
                this.scaleEl[scaleIndex].ranges[rangeIndex].distanceFromScale = value;
                if (this.contextEl.putImageData && !this.isAndroid())
                    this._reDrawRange();
                else
                    this._initialize();
            }
        },
        /**
        * To get RangeDistanceFromScale
          * @return number
          * @param {number} argument.scaleIndex scaleIndex value for the Gauge 
          * @param {number} argument.rangeIndex rangeIndex value for the Gauge
          * @example 
          * &lt;div id="CoreCircularGauge"&gt;&lt;/div&gt; <br>  
          * &lt;script&gt;
          * $("#CoreCircularGauge").ejCircularGauge({scales: [{showRanges: true,ranges: [{distanceFromScale: -30,startValue: 0,endValue: 70}]}]});
          * var circulargaugeObj = $("#CoreCircularGauge").data("ejCircularGauge");
          * circulargaugeObj.getRangeDistanceFromScale(0, 0);
          * &lt;/script&gt;
          *@memberof ejCircularGauge
          * @instance
        */
        getRangeDistanceFromScale: function (scaleIndex, rangeIndex) {
            if (scaleIndex < this.model.scales.length && rangeIndex < this.model.scales[scaleIndex].ranges.length)
                return this.scaleEl[scaleIndex].ranges[rangeIndex].distanceFromScale;
        },
        /**
          * To set RangePosition 
          * @return jQuery
          * @param {number} argument.scaleIndex scaleIndex value for the gauge
          * @param {number} argument.rangeIndex rangeIndex value for the gauge
          * @param {number} argument.value value for the gauge
          * @example 
          * &lt;div id="CoreCircularGauge"&gt;&lt;/div&gt; <br>  
          * &lt;script&gt;
          * $("#CoreCircularGauge").ejCircularGauge({scales: [{showRanges: true,ranges: [{distanceFromScale: -30,startValue: 0,endValue: 70}]}]});
          * var circulargaugeObj = $("#CoreCircularGauge").data("ejCircularGauge");
          * circulargaugeObj.setRangePosition(0, 0, "far");
          * &lt;/script&gt;
          *@memberof ejCircularGauge
          * @instance
        */
        setRangePosition: function (scaleIndex, rangeIndex, value) {
            if (scaleIndex < this.model.scales.length && rangeIndex < this.model.scales[scaleIndex].ranges.length && value != null) {
                this.scaleEl[scaleIndex].ranges[rangeIndex].placement = value;
                if (this.contextEl.putImageData && !this.isAndroid())
                    this._reDrawRange();
                else
                    this._initialize();
            }
        },
        /**
        * To get RangePosition
          * @return number
          * @param {number} argument.scaleIndex scaleIndex value for the Gauge 
          * @param {number} argument.rangeIndex rangeIndex value for the Gauge
          * @example 
          * &lt;div id="CoreCircularGauge"&gt;&lt;/div&gt; <br>  
          * &lt;script&gt;
          * $("#CoreCircularGauge").ejCircularGauge({scales: [{showRanges: true,ranges: [{distanceFromScale: -30,startValue: 0,endValue: 70}]}]});
          * var circulargaugeObj = $("#CoreCircularGauge").data("ejCircularGauge");
          * circulargaugeObj.getRangePosition(0, 0);
          * &lt;/script&gt;
          *@memberof ejCircularGauge
          * @instance
        */
        getRangePosition: function (scaleIndex, rangeIndex) {
            if (scaleIndex < this.model.scales.length && rangeIndex < this.model.scales[scaleIndex].ranges.length)
                return this.scaleEl[scaleIndex].ranges[rangeIndex].placement;
        },
        /**
          * To set RangeBorderWidth 
          * @return jQuery
          * @param {number} argument.scaleIndex scaleIndex value for the gauge
          * @param {number} argument.rangeIndex rangeIndex value for the gauge
          * @param {number} argument.value value for the gauge
          * @example 
          * &lt;div id="CoreCircularGauge"&gt;&lt;/div&gt; <br>  
          * &lt;script&gt;
          * $("#CoreCircularGauge").ejCircularGauge({scales: [{showRanges: true,ranges: [{distanceFromScale: -30,startValue: 0,endValue: 70}]}]});
          * var circulargaugeObj = $("#CoreCircularGauge").data("ejCircularGauge");
          * circulargaugeObj.setRangeBorderWidth(0, 0, 5);
          * &lt;/script&gt;
          *@memberof ejCircularGauge
          * @instance
        */
        setRangeBorderWidth: function (scaleIndex, rangeIndex, value) {
            if (scaleIndex < this.model.scales.length && rangeIndex < this.model.scales[scaleIndex].ranges.length && value != null) {
                this.scaleEl[scaleIndex].ranges[rangeIndex].border.width = value;
                if (this.contextEl.putImageData && !this.isAndroid())
                    this._reDrawRange();
                else
                    this._initialize();
            }
        },
        /**
        * To get RangeBorderWidth
          * @return number
          * @param {number} argument.scaleIndex scaleIndex value for the Gauge 
          * @param {number} argument.rangeIndex rangeIndex value for the Gauge
          * @example 
          * &lt;div id="CoreCircularGauge"&gt;&lt;/div&gt; <br>  
          * &lt;script&gt;
          * $("#CoreCircularGauge").ejCircularGauge({scales: [{showRanges: true,ranges: [{distanceFromScale: -30,startValue: 0,endValue: 70}]}]});
          * var circulargaugeObj = $("#CoreCircularGauge").data("ejCircularGauge");
          * circulargaugeObj.getRangeBorderWidth(0, 0);
          * &lt;/script&gt;
          *@memberof ejCircularGauge
          * @instance
        */
        getRangeBorderWidth: function (scaleIndex, rangeIndex) {
            if (scaleIndex < this.model.scales.length && rangeIndex < this.model.scales[scaleIndex].ranges.length)
                return this.scaleEl[scaleIndex].ranges[rangeIndex].border.width;
        },
        /**
          * To set PointerValue 
          * @return jQuery
          * @param {number} argument.scaleIndex scaleIndex value for the gauge
          * @param {number} argument.pointerIndex pointerIndex value for the gauge
          * @param {number} argument.value value for the gauge
          * @example 
          * &lt;div id="CoreCircularGauge"&gt;&lt;/div&gt; <br>  
          * &lt;script&gt;
          * $("#CoreCircularGauge").ejCircularGauge();
          * var circulargaugeObj = $("#CoreCircularGauge").data("ejCircularGauge");
          * circulargaugeObj.setPointerValue(0, 0, 10);
          * &lt;/script&gt;
          *@memberof ejCircularGauge
          * @instance
        */
        setPointerValue: function (scaleIndex, pointerIndex, value) {
            if (scaleIndex < this.model.scales.length && pointerIndex < this.model.scales[scaleIndex].pointers.length && value != null) {
            if(value >= this.scaleEl[scaleIndex].minimum && value <= this.scaleEl[scaleIndex].maximum)
                this.scaleEl[scaleIndex].pointers[pointerIndex].value = value;
                if (this.contextEl.putImageData && !this.isAndroid())
                    this._reDrawPointer();
                else
                    this._initialize();
            }
        },
        /**
          * To set LabelAngle 
          * @return jQuery
          * @param {number} argument.scaleIndex scaleIndex value for the gauge
          * @param {number} argument.labelIndex labelIndex value for the gauge
          * @param {number} argument.angle angle value for the gauge
          * @example 
          * &lt;div id="CoreCircularGauge"&gt;&lt;/div&gt; <br>  
          * &lt;script&gt;
          * $("#CoreCircularGauge").ejCircularGauge();
          * var circulargaugeObj = $("#CoreCircularGauge").data("ejCircularGauge");
          * circulargaugeObj.setLabelAngle(0, 0, 10);
          * &lt;/script&gt;
          *@memberof ejCircularGauge
          * @instance
        */
        setLabelAngle: function (scaleIndex, labelIndex, angle) {
            if (scaleIndex < this.model.scales.length && labelIndex < this.model.scales[scaleIndex].labels.length && angle != null) {
                this.scaleEl[scaleIndex].labels[labelIndex].angle = angle;
                if (this.contextEl.putImageData && !this.isAndroid())
                    this._reDrawLabel();
                else this._initialize();
            }
        },
        /**
        * To get LabelAngle
          * @return number
          * @param {number} argument.scaleIndex scaleIndex value for the Gauge 
          * @param {number} argument.labelIndex labelIndex value for the Gauge
          * @example 
          * &lt;div id="CoreCircularGauge"&gt;&lt;/div&gt; <br>  
          * &lt;script&gt;
          * $("#CoreCircularGauge").ejCircularGauge();
          * var circulargaugeObj = $("#CoreCircularGauge").data("ejCircularGauge");
          * circulargaugeObj.getLabelAngle(0, 0);
          * &lt;/script&gt;
          *@memberof ejCircularGauge
          * @instance
        */
        getLabelAngle: function (scaleIndex, labelIndex) {
            if (scaleIndex < this.model.scales.length && labelIndex < this.model.scales[scaleIndex].labels.length)
                return this.scaleEl[scaleIndex].labels[labelIndex].angle;
        },
        /**
          * To set LabelDistanceFromScale 
          * @return jQuery
          * @param {number} argument.scaleIndex scaleIndex value for the gauge
          * @param {number} argument.labelIndex labelIndex value for the gauge
          * @param {number} argument.value value for the gauge
          * @example 
          * &lt;div id="CoreCircularGauge"&gt;&lt;/div&gt; <br>  
          * &lt;script&gt;
          * $("#CoreCircularGauge").ejCircularGauge();
          * var circulargaugeObj = $("#CoreCircularGauge").data("ejCircularGauge");
          * circulargaugeObj.setLabelDistanceFromScale(0, 0, 10);
          * &lt;/script&gt;
          *@memberof ejCircularGauge
          * @instance
        */
        setLabelDistanceFromScale: function (scaleIndex, labelIndex, value) {
            if (scaleIndex < this.model.scales.length && labelIndex < this.model.scales[scaleIndex].labels.length && value != null) {
                this.scaleEl[scaleIndex].labels[labelIndex].distanceFromScale = value;
                if (this.contextEl.putImageData && !this.isAndroid())
                    this._reDrawLabel();
                else this._initialize();
            }
        },
        /**
        * To get LabelDistanceFromScale
          * @return number
          * @param {number} argument.scaleIndex scaleIndex value for the Gauge 
          * @param {number} argument.labelIndex labelIndex value for the Gauge
          * @example 
          * &lt;div id="CoreCircularGauge"&gt;&lt;/div&gt; <br>  
          * &lt;script&gt;
          * $("#CoreCircularGauge").ejCircularGauge();
          * var circulargaugeObj = $("#CoreCircularGauge").data("ejCircularGauge");
          * circulargaugeObj.getLabelDistanceFromScale(0, 0);
          * &lt;/script&gt;
          *@memberof ejCircularGauge
          * @instance
        */
        getLabelDistanceFromScale: function (scaleIndex, labelIndex) {
            if (scaleIndex < this.model.scales.length && labelIndex < this.model.scales[scaleIndex].labels.length)
                return this.scaleEl[scaleIndex].labels[labelIndex].distanceFromScale;
        },
        /**
          * To set LabelStyle 
          * @return jQuery
          * @param {number} argument.scaleIndex scaleIndex value for the gauge
          * @param {number} argument.labelIndex labelIndex value for the gauge
          * @param {number} argument.value value for the gauge
          * @example 
          * &lt;div id="CoreCircularGauge"&gt;&lt;/div&gt; <br>  
          * &lt;script&gt;
          * $("#CoreCircularGauge").ejCircularGauge();
          * var circulargaugeObj = $("#CoreCircularGauge").data("ejCircularGauge");
          * circulargaugeObj.setLabelStyle(0, 0, "major");
          * &lt;/script&gt;
          *@memberof ejCircularGauge
          * @instance
        */
        setLabelStyle: function (scaleIndex, labelIndex, value) {
            if (scaleIndex < this.model.scales.length && labelIndex < this.model.scales[scaleIndex].labels.length && value != null) {
                this.scaleEl[scaleIndex].labels[labelIndex].type = value;
                if (this.contextEl.putImageData && !this.isAndroid())
                    this._reDrawLabel();
                else this._initialize();
            }
        },
        /**
        * To get LabelStyle
          * @return number
          * @param {number} argument.scaleIndex scaleIndex value for the Gauge 
          * @param {number} argument.labelIndex labelIndex value for the Gauge
          * @example 
          * &lt;div id="CoreCircularGauge"&gt;&lt;/div&gt; <br>  
          * &lt;script&gt;
          * $("#CoreCircularGauge").ejCircularGauge();
          * var circulargaugeObj = $("#CoreCircularGauge").data("ejCircularGauge");
          * circulargaugeObj.getLabelStyle(0, 0);
          * &lt;/script&gt;
          *@memberof ejCircularGauge
          * @instance
        */
        getLabelStyle: function (scaleIndex, labelIndex) {
            if (scaleIndex < this.model.scales.length && labelIndex < this.model.scales[scaleIndex].labels.length)
                return this.scaleEl[scaleIndex].labels[labelIndex].type;
        },
        /**
          * To set LabelPlacement 
          * @return jQuery
          * @param {number} argument.scaleIndex scaleIndex value for the gauge
          * @param {number} argument.labelIndex labelIndex value for the gauge
          * @param {number} argument.value value for the gauge
          * @example 
          * &lt;div id="CoreCircularGauge"&gt;&lt;/div&gt; <br>  
          * &lt;script&gt;
          * $("#CoreCircularGauge").ejCircularGauge();
          * var circulargaugeObj = $("#CoreCircularGauge").data("ejCircularGauge");
          * circulargaugeObj.setLabelPlacement(0, 0, "far");
          * &lt;/script&gt;
          *@memberof ejCircularGauge
          * @instance
        */
        setLabelPlacement: function (scaleIndex, labelIndex, value) {
            if (scaleIndex < this.model.scales.length && labelIndex < this.model.scales[scaleIndex].labels.length && value != null) {
                this.scaleEl[scaleIndex].labels[labelIndex].placement = value;
                if (this.contextEl.putImageData && !this.isAndroid())
                    this._reDrawLabel();
                else this._initialize();
            }
        },
        /**
        * To get LabelPlacement
          * @return number
          * @param {number} argument.scaleIndex scaleIndex value for the Gauge 
          * @param {number} argument.labelIndex labelIndex value for the Gauge
          * @example 
          * &lt;div id="CoreCircularGauge"&gt;&lt;/div&gt; <br>  
          * &lt;script&gt;
          * $("#CoreCircularGauge").ejCircularGauge();
          * var circulargaugeObj = $("#CoreCircularGauge").data("ejCircularGauge");
          * circulargaugeObj.getLabelPlacement(0, 0);
          * &lt;/script&gt;
          *@memberof ejCircularGauge
          * @instance
        */
        getLabelPlacement: function (scaleIndex, labelIndex) {
            if (scaleIndex < this.model.scales.length && labelIndex < this.model.scales[scaleIndex].labels.length)
                return this.scaleEl[scaleIndex].labels[labelIndex].placement;
        },
        /**
          * To set TickAngle 
          * @return jQuery
          * @param {number} argument.scaleIndex scaleIndex value for the gauge
          * @param {number} argument.tickIndex tickIndex value for the gauge
          * @param {number} argument.value value for the gauge
          * @example 
          * &lt;div id="CoreCircularGauge"&gt;&lt;/div&gt; <br>  
          * &lt;script&gt;
          * $("#CoreCircularGauge").ejCircularGauge();
          * var circulargaugeObj = $("#CoreCircularGauge").data("ejCircularGauge");
          * circulargaugeObj.setTickAngle(0, 0, 10);
          * &lt;/script&gt;
          *@memberof ejCircularGauge
          * @instance
        */
        setTickAngle: function (scaleIndex, tickIndex, angle) {
            if (scaleIndex < this.model.scales.length && tickIndex < this.model.scales[scaleIndex].ticks.length && angle != null) {
                this.scaleEl[scaleIndex].ticks[tickIndex].angle = angle;
                if (this.contextEl.putImageData && !this.isAndroid())
                    this._reDrawTickMark();
                else
                    this._initialize();
            }
        },
        /**
        * To get TickAngle
          * @return number
          * @param {number} argument.scaleIndex scaleIndex value for the Gauge 
          * @param {number} argument.tickIndex tickIndex value for the Gauge
          * @example 
          * &lt;div id="CoreCircularGauge"&gt;&lt;/div&gt; <br>  
          * &lt;script&gt;
          * $("#CoreCircularGauge").ejCircularGauge();
          * var circulargaugeObj = $("#CoreCircularGauge").data("ejCircularGauge");
          * circulargaugeObj.getTickAngle(0, 0);
          * &lt;/script&gt;
          *@memberof ejCircularGauge
          * @instance
        */
        getTickAngle: function (scaleIndex, tickIndex) {
            if (scaleIndex < this.model.scales.length && tickIndex < this.model.scales[scaleIndex].ticks.length) {
                return this.scaleEl[scaleIndex].ticks[tickIndex].angle;
            }
        },
        /**
          * To set TickStyle 
          * @return jQuery
          * @param {number} argument.scaleIndex scaleIndex value for the gauge
          * @param {number} argument.tickIndex tickIndex value for the gauge
          * @param {number} argument.value value for the gauge
          * @example 
          * &lt;div id="CoreCircularGauge"&gt;&lt;/div&gt; <br>  
          * &lt;script&gt;
          * $("#CoreCircularGauge").ejCircularGauge();
          * var circulargaugeObj = $("#CoreCircularGauge").data("ejCircularGauge");
          * circulargaugeObj.setTickStyle(0, 0, "minor");
          * &lt;/script&gt;
          *@memberof ejCircularGauge
          * @instance
        */
        setTickStyle: function (scaleIndex, tickIndex, value) {
            if (scaleIndex < this.model.scales.length && tickIndex < this.model.scales[scaleIndex].ticks.length && value != null) {
                this.scaleEl[scaleIndex].ticks[tickIndex].type = value;
                if (this.contextEl.putImageData && !this.isAndroid())
                    this._reDrawTickMark();
                else this._initialize();
            }
        },
        /**
        * To get TickStyle
          * @return number
          * @param {number} argument.scaleIndex scaleIndex value for the Gauge 
          * @param {number} argument.tickIndex tickIndex value for the Gauge
          * @example 
          * &lt;div id="CoreCircularGauge"&gt;&lt;/div&gt; <br>  
          * &lt;script&gt;
          * $("#CoreCircularGauge").ejCircularGauge();
          * var circulargaugeObj = $("#CoreCircularGauge").data("ejCircularGauge");
          * circulargaugeObj.getTickStyle(0, 0);
          * &lt;/script&gt;
          *@memberof ejCircularGauge
          * @instance
        */
        getTickStyle: function (scaleIndex, tickIndex) {
            if (scaleIndex < this.model.scales.length && tickIndex < this.model.scales[scaleIndex].ticks.length)
                return this.scaleEl[scaleIndex].ticks[tickIndex].type;
        },
        /**
          * To set TickPlacement 
          * @return jQuery
          * @param {number} argument.scaleIndex scaleIndex value for the gauge
          * @param {number} argument.tickIndex tickIndex value for the gauge
          * @param {number} argument.value value for the gauge
          * @example 
          * &lt;div id="CoreCircularGauge"&gt;&lt;/div&gt; <br>  
          * &lt;script&gt; 
          * $("#CoreCircularGauge").ejCircularGauge();
          * var circulargaugeObj = $("#CoreCircularGauge").data("ejCircularGauge");
          * circulargaugeObj.setTickPlacement(0, 0, "near");
          * &lt;/script&gt;
          *@memberof ejCircularGauge
          * @instance
        */
        setTickPlacement: function (scaleIndex, tickIndex, value) {
            if (scaleIndex < this.model.scales.length && tickIndex < this.model.scales[scaleIndex].ticks.length && value != null) {
                this.scaleEl[scaleIndex].ticks[tickIndex].tickPlacement = value;
                if (this.contextEl.putImageData && !this.isAndroid())
                    this._reDrawTickMark();
                else this._initialize();
            }
        },
        /**
        * To get TickPlacement
          * @return number
          * @param {number} argument.scaleIndex scaleIndex value for the Gauge 
          * @param {number} argument.tickIndex tickIndex value for the Gauge
          * @example 
          * &lt;div id="CoreCircularGauge"&gt;&lt;/div&gt; <br>  
          * &lt;script&gt;
          * $("#CoreCircularGauge").ejCircularGauge();
          * var circulargaugeObj = $("#CoreCircularGauge").data("ejCircularGauge");
          * circulargaugeObj.getTickPlacement(0, 0);
          * &lt;/script&gt;
          *@memberof ejCircularGauge
          * @instance
        */
        getTickPlacement: function (scaleIndex, tickIndex) {
            if (scaleIndex < this.model.scales.length && tickIndex < this.model.scales[scaleIndex].ticks.length)
                return this.scaleEl[scaleIndex].ticks[tickIndex].tickPlacement;
        },
        /**
          * To set TickWidth 
          * @return jQuery
          * @param {number} argument.scaleIndex scaleIndex value for the gauge
          * @param {number} argument.tickIndex tickIndex value for the gauge
          * @param {number} argument.value value for the gauge
          * @example 
          * &lt;div id="CoreCircularGauge"&gt;&lt;/div&gt; <br>  
          * &lt;script&gt;
          * $("#CoreCircularGauge").ejCircularGauge();
          * var circulargaugeObj = $("#CoreCircularGauge").data("ejCircularGauge");
          * circulargaugeObj.setTickWidth(0, 0, 5);
          * &lt;/script&gt;
          *@memberof ejCircularGauge
          * @instance
        */
        setTickWidth: function (scaleIndex, tickIndex, value) {
            if (scaleIndex < this.model.scales.length && tickIndex < this.model.scales[scaleIndex].ticks.length && value != null) {
                this.scaleEl[scaleIndex].ticks[tickIndex].width = value;
                if (this.contextEl.putImageData && !this.isAndroid())
                    this._reDrawTickMark();
                else this._initialize();
            }
        },
        /**
        * To get TickWidth
          * @return number
          * @param {number} argument.scaleIndex scaleIndex value for the Gauge 
          * @param {number} argument.tickIndex tickIndex value for the Gauge
          * @example 
          * &lt;div id="CoreCircularGauge"&gt;&lt;/div&gt; <br>  
          * &lt;script&gt;
          * $("#CoreCircularGauge").ejCircularGauge();
          * var circulargaugeObj = $("#CoreCircularGauge").data("ejCircularGauge");
          * circulargaugeObj.getTickWidth(0, 0);
          * &lt;/script&gt;
          *@memberof ejCircularGauge
          * @instance
        */
        getTickWidth: function (scaleIndex, tickIndex) {
            if (scaleIndex < this.model.scales.length && tickIndex < this.model.scales[scaleIndex].ticks.length)
                return this.scaleEl[scaleIndex].ticks[tickIndex].width;
        },
        /**
          * To set TickHeight 
          * @return jQuery
          * @param {number} argument.scaleIndex scaleIndex value for the gauge
          * @param {number} argument.tickIndex tickIndex value for the gauge
          * @param {number} argument.value value for the gauge
          * @example 
          * &lt;div id="CoreCircularGauge"&gt;&lt;/div&gt; <br>  
          * &lt;script&gt;
          * $("#CoreCircularGauge").ejCircularGauge();
          * var circulargaugeObj = $("#CoreCircularGauge").data("ejCircularGauge");
          * circulargaugeObj.setTickHeight(0, 0, 10);
          * &lt;/script&gt;
          *@memberof ejCircularGauge
          * @instance
        */
        setTickHeight: function (scaleIndex, tickIndex, value) {
            if (scaleIndex < this.model.scales.length && tickIndex < this.model.scales[scaleIndex].ticks.length && value != null) {
                this.scaleEl[scaleIndex].ticks[tickIndex].height = value;
                if (this.contextEl.putImageData && !this.isAndroid())
                    this._reDrawTickMark();
                else this._initialize();
            }
        },
        /**
        * To get TickHeight
          * @return number
          * @param {number} argument.scaleIndex scaleIndex value for the Gauge 
          * @param {number} argument.labelIndex labelIndex value for the Gauge
          * @example 
          * &lt;div id="CoreCircularGauge"&gt;&lt;/div&gt; <br>  
          * &lt;script&gt;
          * $("#CoreCircularGauge").ejCircularGauge();
          * var circulargaugeObj = $("#CoreCircularGauge").data("ejCircularGauge");
          * circulargaugeObj.getTickHeight(0, 0);
          * &lt;/script&gt;
          *@memberof ejCircularGauge
          * @instance
        */
        getTickHeight: function (scaleIndex, tickIndex) {
            if (scaleIndex < this.model.scales.length && tickIndex < this.model.scales[scaleIndex].ticks.length)
                return this.scaleEl[scaleIndex].ticks[tickIndex].height;
        },
        /**
          * To set TickDistanceFromScale 
          * @return jQuery
          * @param {number} argument.scaleIndex scaleIndex value for the gauge
          * @param {number} argument.tickIndex tickIndex value for the gauge
          * @param {number} argument.value value for the gauge
          * @example 
          * &lt;div id="CoreCircularGauge"&gt;&lt;/div&gt; <br>  
          * &lt;script&gt;
          * $("#CoreCircularGauge").ejCircularGauge();
          * var circulargaugeObj = $("#CoreCircularGauge").data("ejCircularGauge");
          * circulargaugeObj.setTickDistanceFromScale(0, 0, 15);
          * &lt;/script&gt;
          *@memberof ejCircularGauge
          * @instance
        */
        setTickDistanceFromScale: function (scaleIndex, tickIndex, value) {
            if (scaleIndex < this.model.scales.length && tickIndex < this.model.scales[scaleIndex].ticks.length && value != null) {
                this.scaleEl[scaleIndex].ticks[tickIndex].distanceFromScale = value;
                if (this.contextEl.putImageData && !this.isAndroid())
                    this._reDrawTickMark();
                else this._initialize();
            }
        },
        /**
        * To get TickDistanceFromScale
          * @return number
          * @param {number} argument.scaleIndex scaleIndex value for the Gauge 
          * @param {number} argument.tickIndex tickIndex value for the Gauge
          * @example 
          * &lt;div id="CoreCircularGauge"&gt;&lt;/div&gt; <br>  
          * &lt;script&gt;
          * $("#CoreCircularGauge").ejCircularGauge();
          * var circulargaugeObj = $("#CoreCircularGauge").data("ejCircularGauge");
          * circulargaugeObj.getTickDistanceFromScale(0, 0);
          * &lt;/script&gt;
          *@memberof ejCircularGauge
          * @instance
        */
        getTickDistanceFromScale: function (scaleIndex, tickIndex) {
            if (scaleIndex < this.model.scales.length && tickIndex < this.model.scales[scaleIndex].ticks.length)
                return this.scaleEl[scaleIndex].ticks[tickIndex].distanceFromScale;
        },
        /**
          * To set StartAngle 
          * @return jQuery
          * @param {number} argument.scaleIndex scaleIndex value for the gauge
          * @param {number} argument.value value for the gauge
          * @example 
          * &lt;div id="CoreCircularGauge"&gt;&lt;/div&gt; <br>  
          * &lt;script&gt;
          * $("#CoreCircularGauge").ejCircularGauge();
          * var circulargaugeObj = $("#CoreCircularGauge").data("ejCircularGauge");
          * circulargaugeObj.setStartAngle(0, 10);
          * &lt;/script&gt;
          *@memberof ejCircularGauge
          * @instance
        */
        setStartAngle: function (scaleIndex, angle) {
            if (scaleIndex < this.model.scales.length && angle != null) {
                this.scaleEl[scaleIndex].startAngle = angle;
                if (this.contextEl.putImageData && !this.isAndroid())
                    this._reDrawScale();
                else this._initialize();
            }
        },
        /**
        * To get StartAngle
          * @return number
          * @param {number} argument.scaleIndex scaleIndex value for the Gauge
          * @example 
          * &lt;div id="CoreCircularGauge"&gt;&lt;/div&gt; <br>  
          * &lt;script&gt;
          * $("#CoreCircularGauge").ejCircularGauge();
          * var circulargaugeObj = $("#CoreCircularGauge").data("ejCircularGauge");
          * circulargaugeObj.getStartAngle(0);
          * &lt;/script&gt;
          *@memberof ejCircularGauge
          * @instance
        */
        getStartAngle: function (scaleIndex) {
            if (scaleIndex < this.model.scales.length) {
                return this.scaleEl[scaleIndex].startAngle;
            }
        },
        /**
          * To set SweepAngle 
          * @return jQuery
          * @param {number} argument.scaleIndex scaleIndex value for the gauge
          * @param {number} argument.value value for the gauge
          * @example 
          * &lt;div id="CoreCircularGauge"&gt;&lt;/div&gt; <br>  
          * &lt;script&gt;
          * $("#CoreCircularGauge").ejCircularGauge();
          * var circulargaugeObj = $("#CoreCircularGauge").data("ejCircularGauge");
          * circulargaugeObj.setSweepAngle(0, 220);
          * &lt;/script&gt;
          *@memberof ejCircularGauge
          * @instance
        */
        setSweepAngle: function (scaleIndex, angle) {
            if (scaleIndex < this.model.scales.length && angle != null) {
                this.scaleEl[scaleIndex].sweepAngle = angle;
                if (this.contextEl.putImageData && !this.isAndroid())
                    this._reDrawScale();
                else this._initialize();
            }
        },
        /**
        * To get SweepAngle
          * @return number
          * @param {number} argument.scaleIndex scaleIndex value for the Gauge
          * @example 
          * &lt;div id="CoreCircularGauge"&gt;&lt;/div&gt; <br>  
          * &lt;script&gt;
          * $("#CoreCircularGauge").ejCircularGauge();
          * var circulargaugeObj = $("#CoreCircularGauge").data("ejCircularGauge");
          * circulargaugeObj.getSweepAngle(0);
          * &lt;/script&gt;
          *@memberof ejCircularGauge
          * @instance
        */
        getSweepAngle: function (scaleIndex) {
            if (scaleIndex < this.model.scales.length) {
                return this.scaleEl[scaleIndex].sweepAngle;
            }
        },
        /**
          * To set MinimumValue 
          * @return jQuery
          * @param {number} argument.scaleIndex scaleIndex value for the gauge
          * @param {number} argument.value value for the gauge
          * @example 
          * &lt;div id="CoreCircularGauge"&gt;&lt;/div&gt; <br>  
          * &lt;script&gt;
          * $("#CoreCircularGauge").ejCircularGauge();
          * var circulargaugeObj = $("#CoreCircularGauge").data("ejCircularGauge");
          * circulargaugeObj.setMinimumValue(0, 10);
          * &lt;/script&gt;
          *@memberof ejCircularGauge
          * @instance
        */
        setMinimumValue: function (scaleIndex, value) {
            if (scaleIndex < this.model.scales.length && value != null) {
                if(value < this.scaleEl[scaleIndex].maximum)
                this.scaleEl[scaleIndex].minimum = value;
                if (this.contextEl.putImageData && !this.isAndroid())
                    this._reDrawScale();
                else this._initialize();
            }
        },
        /**
        * To get MinimumValue
          * @return number
          * @param {number} argument.scaleIndex scaleIndex value for the Gauge
          * @example 
          * &lt;div id="CoreCircularGauge"&gt;&lt;/div&gt; <br>  
          * &lt;script&gt;
          * $("#CoreCircularGauge").ejCircularGauge();
          * var circulargaugeObj = $("#CoreCircularGauge").data("ejCircularGauge");
          * circulargaugeObj.getMinimumValue(0);
          * &lt;/script&gt;
          *@memberof ejCircularGauge
          * @instance
        */
        getMinimumValue: function (scaleIndex) {
            if (scaleIndex < this.model.scales.length)
                return this.scaleEl[scaleIndex].minimum;
        },
        /**
          * To set MaximumValue 
          * @return jQuery
          * @param {number} argument.scaleIndex scaleIndex value for the gauge
          * @param {number} argument.value value for the gauge
          * @example 
          * &lt;div id="CoreCircularGauge"&gt;&lt;/div&gt; <br>  
          * &lt;script&gt;
          * $("#CoreCircularGauge").ejCircularGauge();
          * var circulargaugeObj = $("#CoreCircularGauge").data("ejCircularGauge");
          * circulargaugeObj.setMaximumValue(0, 130);
          * &lt;/script&gt;
          *@memberof ejCircularGauge
          * @instance
        */
        setMaximumValue: function (scaleIndex, value) {
            if (scaleIndex < this.model.scales.length && value != null) {
                if(value > this.scaleEl[scaleIndex].minimum)
                this.scaleEl[scaleIndex].maximum = value;
                if (this.contextEl.putImageData && !this.isAndroid())
                    this._reDrawScale();
                else this._initialize();
            }
        },
        /**
        * To get MaximumValue
          * @return number
          * @param {number} argument.scaleIndex scaleIndex value for the Gauge
          * @example 
          * &lt;div id="CoreCircularGauge"&gt;&lt;/div&gt; <br>  
          * &lt;script&gt;
          * $("#CoreCircularGauge").ejCircularGauge();
          * var circulargaugeObj = $("#CoreCircularGauge").data("ejCircularGauge");
          * circulargaugeObj.getMaximumValue(0);
          * &lt;/script&gt;
          *@memberof ejCircularGauge
          * @instance
        */
        getMaximumValue: function (scaleIndex) {
            if (scaleIndex < this.model.scales.length)
                return this.scaleEl[scaleIndex].maximum;
        },
        /**
          * To set ScaleBarSize 
          * @return jQuery
          * @param {number} argument.scaleIndex scaleIndex value for the gauge
          * @param {number} argument.value value for the gauge
          * @example 
          * &lt;div id="CoreCircularGauge"&gt;&lt;/div&gt; <br>  
          * &lt;script&gt;
          * $("#CoreCircularGauge").ejCircularGauge();
          * var circulargaugeObj = $("#CoreCircularGauge").data("ejCircularGauge");
          * circulargaugeObj.setScaleBarSize(0, 160);
          * &lt;/script&gt;
          *@memberof ejCircularGauge
          * @instance
        */
        setScaleBarSize: function (scaleIndex, value) {
            if (scaleIndex < this.model.scales.length && value != null) {
                this.scaleEl[scaleIndex].size = value;
                if (this.contextEl.putImageData && !this.isAndroid())
                    this._reDrawScale();
                else this._initialize();
            }
        },
        /**
        * To get ScaleBarSize
          * @return number
          * @param {number} argument.scaleIndex scaleIndex value for the Gauge
          * @example 
          * &lt;div id="CoreCircularGauge"&gt;&lt;/div&gt; <br>  
          * &lt;script&gt;
          * $("#CoreCircularGauge").ejCircularGauge();
          * var circulargaugeObj = $("#CoreCircularGauge").data("ejCircularGauge");
          * circulargaugeObj.getScaleBarSize(0);
          * &lt;/script&gt;
          *@memberof ejCircularGauge
          * @instance
        */
        getScaleBarSize: function (scaleIndex) {
            if (scaleIndex < this.model.scales.length)
                return this.scaleEl[scaleIndex].size;
        },
        /**
          * To set ScaleRadius 
          * @return jQuery
          * @param {number} argument.scaleIndex scaleIndex value for the gauge
          * @param {number} argument.value value for the gauge
          * @example 
          * &lt;div id="CoreCircularGauge"&gt;&lt;/div&gt; <br>  
          * &lt;script&gt;
          * $("#CoreCircularGauge").ejCircularGauge();
          * var circulargaugeObj = $("#CoreCircularGauge").data("ejCircularGauge");
          * circulargaugeObj.setScaleRadius(0, 140);
          * &lt;/script&gt;
          *@memberof ejCircularGauge
          * @instance
        */
        setScaleRadius: function (scaleIndex, value) {
            if (scaleIndex < this.model.scales.length && value != null) {
                this.scaleEl[scaleIndex].radius = value;
                if (this.contextEl.putImageData && !this.isAndroid())
                    this._reDrawScale();
                else this._initialize();
            }
        },
        /**
        * To get ScaleRadius
          * @return number
          * @param {number} argument.scaleIndex scaleIndex value for the Gauge
          * @example 
          * &lt;div id="CoreCircularGauge"&gt;&lt;/div&gt; <br>  
          * &lt;script&gt;
          * $("#CoreCircularGauge").ejCircularGauge();
          * var circulargaugeObj = $("#CoreCircularGauge").data("ejCircularGauge");
          * circulargaugeObj.getScaleRadius(0);
          * &lt;/script&gt;
          *@memberof ejCircularGauge
          * @instance
        */
        getScaleRadius: function (scaleIndex) {
            if (scaleIndex < this.model.scales.length)
                return this.scaleEl[scaleIndex].radius;
        },
        /**
          * To set MajorIntervalValue 
          * @return jQuery
          * @param {number} argument.scaleIndex scaleIndex value for the gauge
          * @param {number} argument.value value for the gauge
          * @example 
          * &lt;div id="CoreCircularGauge"&gt;&lt;/div&gt; <br>  
          * &lt;script&gt;
          * $("#CoreCircularGauge").ejCircularGauge();
          * var circulargaugeObj = $("#CoreCircularGauge").data("ejCircularGauge");
          * circulargaugeObj.setMajorIntervalValue(0, 10);
          * &lt;/script&gt;
          *@memberof ejCircularGauge
          * @instance
        */
        setMajorIntervalValue: function (scaleIndex, value) {
            if (scaleIndex < this.model.scales.length && value != null) {
                this.scaleEl[scaleIndex].majorIntervalValue = value;
                if (this.contextEl.putImageData && !this.isAndroid())
                    this._reDrawScale();
                else this._initialize();
            }
        },
        /**
        * To get MajorIntervalValue
          * @return number
          * @param {number} argument.scaleIndex scaleIndex value for the Gauge
          * @example 
          * &lt;div id="CoreCircularGauge"&gt;&lt;/div&gt; <br>  
          * &lt;script&gt;
          * $("#CoreCircularGauge").ejCircularGauge();
          * var circulargaugeObj = $("#CoreCircularGauge").data("ejCircularGauge");
          * circulargaugeObj.getMajorIntervalValue(0);
          * &lt;/script&gt;
          *@memberof ejCircularGauge
          * @instance
        */
        getMajorIntervalValue: function (scaleIndex) {
            if (scaleIndex < this.model.scales.length)
                return this.scaleEl[scaleIndex].majorIntervalValue;
        },
        /**
          * To set MinorIntervalValue 
          * @return jQuery
          * @param {number} argument.scaleIndex scaleIndex value for the gauge
          * @param {number} argument.value value for the gauge
          * @example 
          * &lt;div id="CoreCircularGauge"&gt;&lt;/div&gt; <br>  
          * &lt;script&gt;
          * $("#CoreCircularGauge").ejCircularGauge();
          * var circulargaugeObj = $("#CoreCircularGauge").data("ejCircularGauge");
          * circulargaugeObj.setMinorIntervalValue(0, 2);
          * &lt;/script&gt;
          *@memberof ejCircularGauge
          * @instance
        */
        setMinorIntervalValue: function (scaleIndex, value) {
            if (scaleIndex < this.model.scales.length && value != null) {
                this.scaleEl[scaleIndex].minorIntervalValue = value;
                if (this.contextEl.putImageData && !this.isAndroid())
                    this._reDrawScale();
                else this._initialize();
            }
        },
        /**
        * To get MinorIntervalValue
          * @return number
          * @param {number} argument.scaleIndex scaleIndex value for the Gauge
          * @example 
          * &lt;div id="CoreCircularGauge"&gt;&lt;/div&gt; <br>  
          * &lt;script&gt;
          * $("#CoreCircularGauge").ejCircularGauge();
          * var circulargaugeObj = $("#CoreCircularGauge").data("ejCircularGauge");
          * circulargaugeObj.getMinorIntervalValue(0);
          * &lt;/script&gt;
          * @memberof ejCircularGauge
          * @instance
        */
        getMinorIntervalValue: function (scaleIndex) {
            if (scaleIndex < this.model.scales.length)
                return this.scaleEl[scaleIndex].minorIntervalValue;
        },
        /**
          * To set PointerCapRadius 
          * @return jQuery
          * @param {number} argument.scaleIndex scaleIndex value for the gauge
          * @param {number} argument.value value for the gauge
          * @example 
          * &lt;div id="CoreCircularGauge"&gt;&lt;/div&gt; <br>  
          * &lt;script&gt;
          * $("#CoreCircularGauge").ejCircularGauge();
          * var circulargaugeObj = $("#CoreCircularGauge").data("ejCircularGauge");
          * circulargaugeObj.setPointerCapRadius(0, 10);
          * &lt;/script&gt;
          *@memberof ejCircularGauge
          * @instance
        */
        setPointerCapRadius: function (scaleIndex, value) {
            if (scaleIndex < this.model.scales.length && value != null) {
                this.scaleEl[scaleIndex].pointerCap.radius = value;
                if (this.contextEl.putImageData && !this.isAndroid())
                    this._reDrawScale();
                else this._initialize();
            }
        },
        /**
        * To get PointerCapRadius
          * @return number
          * @param {number} argument.scaleIndex scaleIndex value for the Gauge
          * @example 
          * &lt;div id="CoreCircularGauge"&gt;&lt;/div&gt; <br>  
          * &lt;script&gt;
          * $("#CoreCircularGauge").ejCircularGauge();
          * var circulargaugeObj = $("#CoreCircularGauge").data("ejCircularGauge");
          * circulargaugeObj.getPointerCapRadius(0);
          * &lt;/script&gt;
          * @memberof ejCircularGauge
          * @instance
        */
        getPointerCapRadius: function (scaleIndex) {
            if (scaleIndex < this.model.scales.length)
                return this.scaleEl[scaleIndex].pointerCap.radius;
        },
        /**
          * To set ScaleBorderWidth 
          * @return jQuery
          * @param {number} argument.scaleIndex scaleIndex value for the gauge
          * @param {number} argument.value value for the gauge
          * @example 
          * &lt;div id="CoreCircularGauge"&gt;&lt;/div&gt; <br>  
          * &lt;script&gt;
          * $("#CoreCircularGauge").ejCircularGauge({ scales: [{showScaleBar: true, size: 6, border:{width: 1.5} }] });
          * var circulargaugeObj = $("#CoreCircularGauge").data("ejCircularGauge");
          * circulargaugeObj.setScaleBorderWidth(0, 3);
          * &lt;/script&gt;
          * @memberof ejCircularGauge
          * @instance
        */
        setScaleBorderWidth: function (scaleIndex, value) {
            if (scaleIndex < this.model.scales.length && value != null) {
                this.scaleEl[scaleIndex].border.width = value;
                if (this.contextEl.putImageData && !this.isAndroid())
                    this._reDrawScale();
                else this._initialize();
            }
        },
        /**
        * To get ScaleBorderWidth
          * @return number
          * @param {number} argument.scaleIndex scaleIndex value for the Gauge
          * @example 
          * &lt;div id="CoreCircularGauge"&gt;&lt;/div&gt; <br>  
          * &lt;script&gt;
          * $("#CoreCircularGauge").ejCircularGauge({ scales: [{showScaleBar: true, size: 6, border:{Width: 1.5} }] });
          * var circulargaugeObj = $("#CoreCircularGauge").data("ejCircularGauge");
          * circulargaugeObj.getScaleBorderWidth(0);
          * &lt;/script&gt;
          * @memberof ejCircularGauge
          * @instance
        */
        getScaleBorderWidth: function (scaleIndex) {
            if (scaleIndex < this.model.scales.length)
                return this.scaleEl[scaleIndex].border.width;
        },
        /**
          * To set PointerCapBorderWidth 
          * @return jQuery
          * @param {number} argument.scaleIndex scaleIndex value for the gauge
          * @param {number} argument.value value for the gauge
          * @example 
          * &lt;div id="CoreCircularGauge"&gt;&lt;/div&gt; <br>  
          * &lt;script&gt;
          * $("#CoreCircularGauge").ejCircularGauge();
          * var circulargaugeObj = $("#CoreCircularGauge").data("ejCircularGauge");
          * circulargaugeObj.setPointerCapBorderWidth(0, 5);
          * &lt;/script&gt;
          *@memberof ejCircularGauge
          * @instance
        */
        setPointerCapBorderWidth: function (scaleIndex, value) {
            if (scaleIndex < this.model.scales.length && value != null) {
                this.scaleEl[scaleIndex].capBorderWidth = value;
                if (this.contextEl.putImageData && !this.isAndroid())
                    this._reDrawScale();
                else this._initialize();
            }
        },
        /**
        * To get PointerCapBorderWidth
          * @return number
          * @param {number} argument.scaleIndex scaleIndex value for the Gauge
          * @example 
          * &lt;div id="CoreCircularGauge"&gt;&lt;/div&gt; <br>  
          * &lt;script&gt;
          * $("#CoreCircularGauge").ejCircularGauge();
          * var circulargaugeObj = $("#CoreCircularGauge").data("ejCircularGauge");
          * circulargaugeObj.getPointerCapBorderWidth(0);
          * &lt;/script&gt;
          * @memberof ejCircularGauge
          * @instance
        */
        getPointerCapBorderWidth: function (scaleIndex) {
            if (scaleIndex < this.model.scales.length)
                return this.scaleEl[scaleIndex].capBorderWidth;
        },
        /**
          * To set ScaleDirection 
          * @return jQuery
          * @param {number} argument.scaleIndex scaleIndex value for the gauge
          * @param {number} argument.value value for the gauge
          * @example 
          * &lt;div id="CoreCircularGauge"&gt;&lt;/div&gt; <br>  
          * &lt;script&gt;
          * $("#CoreCircularGauge").ejCircularGauge();
          * var circulargaugeObj = $("#CoreCircularGauge").data("ejCircularGauge");
          * circulargaugeObj.setScaleDirection(0, "clockwise");
          * &lt;/script&gt;
          *@memberof ejCircularGauge
          * @instance
        */
        setScaleDirection: function (scaleIndex, value) {
            if (scaleIndex < this.model.scales.length && value != null) {
                this.scaleEl[scaleIndex].direction = value;
                if (this.contextEl.putImageData && !this.isAndroid())
                    this._reDrawScale();
                else this._initialize();
            }
        },
        /**
        * To get ScaleDirection
          * @return number
          * @param {number} argument.scaleIndex scaleIndex value for the Gauge
          * @example 
          * &lt;div id="CoreCircularGauge"&gt;&lt;/div&gt; <br>  
          * &lt;script&gt;
          * $("#CoreCircularGauge").ejCircularGauge();
          * var circulargaugeObj = $("#CoreCircularGauge").data("ejCircularGauge");
          * circulargaugeObj.getScaleDirection(0);
          * &lt;/script&gt;
          *@memberof ejCircularGauge
          * @instance
        */
        getScaleDirection: function (scaleIndex) {
            if (scaleIndex < this.model.scales.length)
                return this.scaleEl[scaleIndex].direction;
        },
        /**
          * To set includeFirstValue 
          * @return jQuery
          * @param {number} argument.scaleIndex scaleIndex value for the gauge
          * @param {number} argument.labelIndex labelIndex value for the gauge
          * @param {number} argument.value value for the gauge
          * @example 
          * &lt;div id="CoreCircularGauge"&gt;&lt;/div&gt; <br>  
          * &lt;script&gt;
          * $("#CoreCircularGauge").ejCircularGauge();
          * var circulargaugeObj = $("#CoreCircularGauge").data("ejCircularGauge");
          * circulargaugeObj.includeFirstValue(0, 0, false);
          * &lt;/script&gt;
          *@memberof ejCircularGauge
          * @instance
        */
        includeFirstValue: function (scaleIndex, labelIndex, value) {
            if (scaleIndex < this.model.scales.length && labelIndex < this.model.scales[scaleIndex].labels.length && value != null) {
                this.scaleEl[scaleIndex].labels[labelIndex].includeFirstValue = value;
                if (this.contextEl.putImageData)
                    this._reDrawLabel();
                else this.initialize();
            }
        },
		/**
         * To Redraw scale
		 * @private
         */
        _reDrawScale: function () {
            if (this.model.frame.backgroundImageUrl)
                this.initialize();
            else {
                this.contextEl.putImageData(this.outerImage, 0, 0);
                this._drawScales();

            }
        },
		/**
         * To Redraw Pointer
		 * @private
         */
        _reDrawPointer: function () {
            if (this.model.frame.backgroundImageUrl)
                this.initialize();
            else {
                this.contextEl.putImageData(this._getPointerImage(), 0, 0);
                this._setPointers();
                this._setIndicators();
            }
        },
		/**
         * To Redraw Custom Label
		 * @private
         */
        _reDrawCustomLabel: function () {
            if (this.model.frame.backgroundImageUrl)
                this.initialize();
            else {
                this.contextEl.putImageData(this._getCustomLabelImage(), 0, 0);
                this._setCustomLabel();
                this._setPointers();
                this._setIndicators();

            }
        },
		/**
         * To Redraw Range
		 * @private
         */
        _reDrawRange: function () {
            if (this.model.frame.backgroundImageUrl)
                this._initialize();
            else {
                this.contextEl.putImageData(this._getRangeImage(), 0, 0);
                this._setRanges();
                this._subGauge();
                this._setCustomLabel();
                this._setPointers();
                this._setIndicators();

            }
        },
		/**
         * To Redraw Label
		 * @private
         */
        _reDrawLabel: function () {
            if (this.model.frame.backgroundImageUrl)
                this._initialize();
            else {
                this.contextEl.putImageData(this._getLabelImage(), 0, 0);
                this._setLabels();
                this._setRanges();
                this._subGauge();
                this._setCustomLabel();
                this._setPointers();
                this._setIndicators();

            }
        },
		/**
         * To Redraw TickMark
		 * @private
         */
        _reDrawTickMark: function () {
            if (this.model.frame.backgroundImageUrl)
                this._initialize();
            else {
                this.contextEl.putImageData(this._getTickImage(), 0, 0);
                this._setTicks();
                this._setLabels();
                this._setRanges();
                this._subGauge();
                this._setCustomLabel();
                this._setPointers();
                this._setIndicators();

            }
        },
		/**
         * To Redraw SubGauge
		 * @private
         */
        _reDrawSubGauge: function () {
            if (this.model.frame.backgroundImageUrl)
                this._initialize();
            else {
                this.contextEl.putImageData(this._getSubGaugeImage(), 0, 0);
                this._subGauge();
                this._setCustomLabel();
                this._setPointers();
                this._setIndicators();

            }
        },
		/**
         * To refresh SubGauge
		 * @private
         */
        refreshSubGauge: function () {
            if (this.contextEl.putImageData)
                this._reDrawSubGauge();
            else
                this._initialize();
        },
		/**
         * To refresh the Entire gauge
		 * @private
         */
        refresh: function () {
            this._initialize();
        },
        /**
        * To export Image
          * @return jQuery
          * @param {string} argument.fileName fileName for the Image
          * @param {string} argument.fileType fileType for the Image
          * @example 
          * &lt;div id="CoreCircularGauge"&gt;&lt;/div&gt; <br> 
          * &lt;script&gt;
          * $("#CoreCircularGauge").ejCircularGauge();
          * var circulargaugeObj = $("#CoreCircularGauge").data("ejCircularGauge");
          * circulargaugeObj.exportImage("myImage","jpeg");
          * &lt;/script&gt;
          *@memberof ejCircularGauge
          * @instance
        */
        exportImage: function (fileName, fileType) {
            /// <summary>This function save the rendered canvas image</summary>
            /// <param name="fileName" type="String">fileName to which the image has been saved</param>
            /// <param name="fileType" type="String">fileType to which the image has been saved</param>
            var image = this.canvasEl[0].toDataURL();
            image = image.replace(/^data:[a-z]*;,/, '');
            var image1 = image.split(',');
            var byteString = atob(image1[1]);
            var buffer = new ArrayBuffer(byteString.length);
            var intArray = new Uint8Array(buffer);
            for (var i = 0; i < byteString.length; i++) {
                intArray[i] = byteString.charCodeAt(i);
            }
            blob = new Blob([buffer], { type: "image/png" });
            if (ej.browserInfo().name === "msie" && parseFloat(ej.browserInfo().version) > 9)
                window.navigator.msSaveOrOpenBlob(blob, fileName + '.' + fileType);
            else {
                var pom = document.createElement('a');
                var url = URL.createObjectURL(blob);
                pom.href = url;
                pom.setAttribute('download', fileName + '.' + fileType);
                if (document.createEvent) {
                    e = document.createEvent("MouseEvents");
                    e.initMouseEvent("click", true, true, window, 0, 0, 0, 0, 0, false, false, false, false, 0, null);
                    pom.dispatchEvent(e);
                }
                else if (pom.fireEvent) {
                    pom.fireEvent("onclick");
                }
            }
        },
		/**
         * To Resize the gauge while resizing the window
		 * @private
         */
        resizeCanvas: function () {
            if (proxy.element.width() != initialDivWidth || proxy.element.height() != initialDivHeight) {
                var canvasWidth = proxy.element.width() / initialDivWidth;
                var canvasHeight = proxy.element.height() / initialDivHeight;
                var ratio;
                if (canvasWidth != 1 && canvasHeight != 1)
                    ratio = canvasWidth;
                else
                    ratio = canvasHeight != 1 ? canvasHeight : canvasWidth;
                proxy.model.width *= ratio;
                for (var i = 0; proxy.model.scales[i] != null; i++) {
                    proxy.model.scales[i].radius *= ratio;
                    for (var j = 0; proxy.model.scales[i].pointers[j] != null; j++) {
                        proxy.model.scales[i].pointers[j].length *= ratio;
                        proxy.model.scales[i].pointers[j].pointerCap.radius *= ratio;
                        proxy.model.scales[i].pointers[j].width *= ratio;
                        proxy.model.scales[i].pointers[j].backNeedleLength *= ratio;
                    }
                }
                proxy.refresh();
                initialDivWidth = proxy.element.width();
                initialDivHeight = proxy.element.height();
            }
        },

        // ********************************* ClientSide Events ************************************
		/**
         * Section For handle the DrawTicks event
		 * @private
         */
        _onDrawTicks: function (tickAngle, pointerValue) {
            var tick = { index: this.tickIndex, element: this.tickEl[this.tickIndex], angle: parseInt(tickAngle) }
            var data = { object: this, scaleElement: this.model.scales[this.scaleIndex], scaleIndex: this.scaleIndex, context: this.contextEl, pointerValue: pointerValue, style: this.style, position: this.location,tick:tick };
            this._trigger("drawTicks", data);
        },
		/**
         * Section For handle the DrawLabels event
		 * @private
         */
        _onDrawLabels: function (labelAngle, pointerValue) {
            var label = { index: this.labelIndex, element: this.labelEl[this.labelIndex], angle: parseInt(labelAngle) }
            var data = { object: this, scaleElement: this.model.scales[this.scaleIndex], scaleIndex: this.scaleIndex, context: this.contextEl, pointerValue: pointerValue, style: this.style, position: this.location,label:label };
            this._trigger("drawLabels", data);
        },
		/**
         * Section For handle the DrawPointers event
		 * @private
         */
        _onDrawPointers: function (pointerAngle, pointerValue) {
            var pointer = { index: this.pointerIndex, element: this.pointerEl[this.pointerIndex], angle: parseInt(pointerAngle), pointerValue: pointerValue, }
            var data = { object: this, scaleElement: this.model.scales[this.scaleIndex], scaleIndex: this.scaleIndex, context: this.contextEl, style: this.style, position: this.location, pointer: pointer };
            this._trigger("drawPointers", data);
        },
		/**
         * Section For handle the DrawRange event
		 * @private
         */
        _onDrawRange: function () {
            var data = { object: this, scaleElement: this.model.scales[this.scaleIndex], scaleIndex: this.scaleIndex, rangeIndex: this.rangeIndex, rangeElement: this.rangeEl[this.rangeEl], context: this.contextEl, style: this.style, position: this.location };
            this._trigger("drawRange", data);
        },
		/**
         * Section For handle the DrawCustomLabel event
		 * @private
         */
        _onDrawCustomLabel: function () {
            var data = { object: this, scaleElement: this.model.scales[this.scaleIndex], scaleIndex: this.scaleIndex, customLabelIndex: this.customLabelIndex, customLabelElement: this.customLabelEl[this.customLabelIndex], context: this.contextEl, style: this.style, position: this.location };
            this._trigger("drawCustomLabel", data);
        },
		/**
         * Section For handle the DrawIndicators event
		 * @private
         */
        _onDrawIndicators: function () {
            var data = { object: this, scaleElement: this.model.scales[this.scaleIndex], scaleIndex: this.scaleIndex, indicatorIndex: this.indicatorIndex, indicatorEl: this.indicatorEl[this.indicatorIndex], context: this.contextEl, style: this.style, position: this.location };
            this._trigger("drawIndicators", data);
        },
		/**
         * Section For handle the DrawPointerCap event
		 * @private
         */
        _onDrawPointerCap: function () {
            var data = { object: this, scaleElement: this.model.scales, position: this.location, style: this.style, context: this.contextEl };
            this._trigger("drawPointerCap", data);
        },
		/**
         * Section For handle the RenderComplete event
		 * @private
         */
        _onRenderComplete: function () {
            var data = { object: this, scaleElement: this.model.scales, context: this.contextEl };
            this._trigger("renderComplete", data);
        },
		/**
         * Section For handle the MouseClick event
		 * @private
         */
        _onMouseClick: function (pointerAngle, pointerValue) {
            var pointer = { index: this.pointerIndex, element: this.pointerEl[this.pointerIndex], value: pointerValue, angle: parseInt(pointerAngle) }
            var data = { object: this, scaleElement: this.model.scales[this.scaleIndex], scaleIndex: this.scaleIndex, context: this.contextEl, style: this.style, position: this.position };
            this._trigger("mouseClick", data);
        },
		/**
         * Section For handle the MouseClickMove event
		 * @private
         */
        _onMouseClickMove: function (pointerAngle, pointerValue) {
            var pointer = { index: this.pointerIndex, element: this.pointerEl[this.pointerIndex], value: pointerValue, angle: parseInt(pointerAngle) }
            var data = { object: this, scaleElement: this.model.scales[this.scaleIndex], scaleIndex: this.scaleIndex, context: this.contextEl, style: this.style, position: this.location };
            this._trigger("mouseClickMove", data);
        },
		/**
         * Section For handle the MouseClickUp event
		 * @private
         */
        _onMouseClickUp: function (pointerAngle, pointerValue) {
            var pointer = { index: this.pointerIndex, element: this.pointerEl[this.pointerIndex], value: pointerValue, angle: parseInt(pointerAngle) }
            var data = { object: this, scaleElement: this.model.scales[this.scaleIndex], scaleIndex: this.scaleIndex, context: this.contextEl, style: this.style, position: this.location };
            this._trigger("mouseClickUp", data);
        },

        //************************* Common Functions **********************************
		/**
         * To set the theme
		 * @private
         */
        _setTheme: function () {
            selectedTheme = ej.datavisualization.CircularGauge.Themes[this.model.theme];
            this._setThemeColors(selectedTheme);
        },
		/**
         * To set the theme color
		 * @private
         */
        _setThemeColors: function (selectedTheme) {
            var result = [], jsonObj = ej.datavisualization.CircularGauge.Themes;
            for (var name in jsonObj) {
                result.push(name);
            }
            for (var th = 0; th < result.length; th++) {
                this.model.backgroundColor = ((!this.model.backgroundColor || this.model.backgroundColor == jsonObj[result[th]].backgroundColor) ? selectedTheme.backgroundColor : this.model.backgroundColor);
                for (var i = 0; i < this.model.scales.length; i++) {
                    this.model.scales[i].backgroundColor = (!this.model.scales[i].backgroundColor || this.model.scales[i].backgroundColor == jsonObj[result[th]].scales.backgroundColor) ? selectedTheme.scales.backgroundColor : this.model.scales[i].backgroundColor;
                    this.model.scales[i].border.color = (!this.model.scales[i].border.color || this.model.scales[i].border.color == jsonObj[result[th]].scales.border.color) ? selectedTheme.scales.border.color : this.model.scales[i].border.color;
                    for (var j = 0; j < this.model.scales[i].pointers.length; j++) {
                        this.model.scales[i].pointers[j].backgroundColor = (!this.model.scales[i].pointers[j].backgroundColor || this.model.scales[i].pointers[j].backgroundColor == jsonObj[result[th]].scales.pointers.backgroundColor) ? selectedTheme.scales.pointers.backgroundColor : this.model.scales[i].pointers[j].backgroundColor;
                        this.model.scales[i].pointers[j].border.color = (!this.model.scales[i].pointers[j].border.color || this.model.scales[i].pointers[j].border.color == jsonObj[result[th]].scales.pointers.border.color) ? selectedTheme.scales.pointers.border.color : this.model.scales[i].pointers[j].border.color;
                        this.model.scales[i].pointerCap.backgroundColor = (!this.model.scales[i].pointerCap.backgroundColor || this.model.scales[i].pointerCap.backgroundColor == jsonObj[result[th]].scales.pointerCap.backgroundColor) ? selectedTheme.scales.pointerCap.backgroundColor : this.model.scales[i].pointerCap.backgroundColor;
                        this.model.scales[i].pointerCap.borderColor = (!this.model.scales[i].pointerCap.borderColor || this.model.scales[i].pointerCap.borderColor == jsonObj[result[th]].scales.pointerCap.borderColor) ? selectedTheme.scales.pointerCap.borderColor : this.model.scales[i].pointerCap.borderColor;
                    }
                    for (var k = 0; k < this.model.scales[i].ticks.length; k++) {
                        this.model.scales[i].ticks[k].color = (!this.model.scales[i].ticks[k].color || this.model.scales[i].ticks[k].color == jsonObj[result[th]].scales.ticks.color) ? selectedTheme.scales.ticks.color : this.model.scales[i].ticks[k].color;
                    }
                    for (var l = 0; l < this.model.scales[i].labels.length; l++) {
                        this.model.scales[i].labels[l].color = (!this.model.scales[i].labels[l].color || this.model.scales[i].labels[l].color == jsonObj[result[th]].scales.labels.color) ? selectedTheme.scales.labels.color : this.model.scales[i].labels[l].color;
                    }
                }
            }
        },
		/**
         * To get the FontString
		 * @private
         */
        _getFontString: function (element, font) {
            return (font ? (font.fontStyle ? font.fontStyle : "") + " " + ((font.size == null) ? "11px" : font.size) + " " + font.fontFamily : "");
        },
		/**
         * To draw Triangle
		 * @private
         */
        _drawTriangle: function (location, style, element) {
            style = this._setPointerDimension(style, element);
            this._contextOpenPath(style, element);
            element.contextEl.translate(location.startX, location.startY);
            this._setContextRotation(style, element);
            element.contextEl.moveTo(0, 0);
            element.contextEl.lineTo(style.width, -style.height / 2);
            element.contextEl.lineTo(style.width, style.height / 2);
            this._contextClosePath(style, element);
        },
		/**
         * To draw Pointer
		 * @private
         */
        _drawPointer: function (location, style, element) {
            style = this._setPointerDimension(style, element);
            this._contextOpenPath(style, element);
            element.contextEl.translate(location.startX, location.startY);
            this._setContextRotation(style, element);
            element.contextEl.moveTo(style.width, style.height / 4);
            element.contextEl.lineTo(style.width, -style.height / 4);
            element.contextEl.lineTo(style.width / 2, -style.height / 4);
            element.contextEl.lineTo(style.width / 2, -style.height / 2);
            element.contextEl.lineTo(0, 0);
            element.contextEl.lineTo(style.width / 2, style.height / 2);
            element.contextEl.lineTo(style.width / 2, style.height / 4);
            this._contextClosePath(style, element);
        },
		/**
         * To draw Wedge
		 * @private
         */
        _drawWedge: function (location, style, element) {
            style = this._setPointerDimension(style, element);
            this._contextOpenPath(style, element);
            element.contextEl.translate(location.startX, location.startY);
            this._setContextRotation(style, element);
            element.contextEl.moveTo(0, 0);
            element.contextEl.lineTo(style.width, -style.height / 2);
            element.contextEl.lineTo(3 * style.width / 4, 0);
            element.contextEl.lineTo(style.width, style.height / 2);
            this._contextClosePath(style, element);
        },
		/**
         * To draw Slider
		 * @private
         */
        _drawSlider: function (location, style, element) {
            style = this._setPointerDimension(style, element);
            this._contextOpenPath(style, element);
            element.contextEl.translate(location.startX, location.startY);
            this._setContextRotation(style, element);
            element.contextEl.moveTo(0, 0);
            element.contextEl.lineTo(style.width / 4, -style.height / 2);
            element.contextEl.lineTo(style.width, -style.height / 2);
            element.contextEl.lineTo(style.width, style.height / 2);
            element.contextEl.lineTo(style.width / 4, style.height / 2);
            this._contextClosePath(style, element);
        },
		/**
         * To draw Star
		 * @private
         */
        _drawStar: function (location, style, element) {
            this._contextOpenPath(style, element);
            if (element.model.Orientation == "Horizontal" && element.markerPlacement == "near") {
                element.contextEl.moveTo(location.startX + style.width - (style.width / 6), location.startY); // 1
                element.contextEl.lineTo(location.startX, location.startY + style.height - style.height / 3);
                element.contextEl.lineTo(location.startX + style.width, location.startY + style.height - style.height / 3);
                element.contextEl.lineTo(location.startX + style.width / 6, location.startY);
                element.contextEl.lineTo(location.startX + style.width / 2, location.startY + style.height);
            }
            else {
                element.contextEl.moveTo(location.startX + (style.width / 6), location.startY + style.height); // 1
                element.contextEl.lineTo(location.startX + style.width, location.startY + (style.height / 3)); // 2
                element.contextEl.lineTo(location.startX, location.startY + (style.height / 3)); // 3
                element.contextEl.lineTo(location.startX + style.width - (style.width / 6), location.startY + style.height); // 4
                element.contextEl.lineTo(location.startX + style.width / 2, location.startY); // 5
            }
            this._contextClosePath(style, element);
        },
		/**
         * To draw Pentagon
		 * @private
         */
        _drawPentagon: function (location, style, element) {
            style = this._setPointerDimension(style, element);
            this._contextOpenPath(style, element);
            element.contextEl.translate(location.startX, location.startY);
            this._setContextRotation(style, element);
            element.contextEl.moveTo(0, 0);
            element.contextEl.lineTo(style.width / 3, -style.height / 2);
            element.contextEl.lineTo(style.width, -style.height / 4);
            element.contextEl.lineTo(style.width, style.height / 4);
            element.contextEl.lineTo(style.width / 3, style.height / 2);
            this._contextClosePath(style, element);
        },
		/**
         * To draw Diamond
		 * @private
         */
        _drawDiamond: function (location, style, element) {
            style = this._setPointerDimension(style, element);
            this._contextOpenPath(style, element);
            element.contextEl.translate(location.startX, location.startY);
            this._setContextRotation(style, element);
            element.contextEl.moveTo(0, 0);
            element.contextEl.lineTo(style.width / 2, -style.height / 2);
            element.contextEl.lineTo(style.width, 0);
            element.contextEl.lineTo(style.width / 2, style.height / 2);
            element.contextEl.lineTo(0, 0);
            this._contextClosePath(style, element);
        },
		/**
         * To draw Circle
		 * @private
         */
        _drawCircle: function (location, style, element) {
            var radius = Math.sqrt(style.height * style.height + style.width * style.width) / 2;
            style = this._setPointerDimension(style, element);
            this._contextOpenPath(style, element);
            element.contextEl.translate(location.startX, location.startY);
            this._setContextRotation(style, element);
            element.contextEl.arc(radius / 2, 0, radius / 2, 0, Math.PI * 2, true);
            this._contextClosePath(style, element);
        },
		/**
         * To draw Trapezoid
		 * @private
         */
        _drawTrapezoid: function (location, style, element) {
            style = this._setPointerDimension(style, element);
            this._contextOpenPath(style, element);
            element.contextEl.translate(location.startX, location.startY);
            this._setContextRotation(style, element);
            element.contextEl.moveTo(0, 0);
            element.contextEl.lineTo(0, -style.height / 4);
            element.contextEl.lineTo(style.width, -style.height / 2);
            element.contextEl.lineTo(style.width, style.height / 2);
            element.contextEl.lineTo(0, style.height / 4);
            this._contextClosePath(style, element);
        },
		/**
         * To draw Rectangle
		 * @private
         */
        _drawRectangle: function (location, style, element) {
            style = this._setPointerDimension(style, element);
            this._contextOpenPath(style, element);
            element.contextEl.translate(location.startX, location.startY);
            this._setContextRotation(style, element);
            element.contextEl.moveTo(0, 0);
            element.contextEl.lineTo(0, -style.height / 2);
            element.contextEl.lineTo(style.width, -style.height / 2);
            element.contextEl.lineTo(style.width, style.height / 2);
            element.contextEl.lineTo(0, style.height / 2);
            this._contextClosePath(style, element);
        },
		/**
         * To draw Rounded Rectangle
		 * @private
         */
        _drawRoundedRectangle: function (location, style, element) {
            style = this._setPointerDimension(style, element);
            this._contextOpenPath(style, element);
            element.contextEl.translate(location.startX, location.startY - style.height / 2);
            this._setContextRotation(style, element);
            element.contextEl.moveTo(style.radius, 0);
            element.contextEl.lineTo(style.width - style.radius, 0);
            element.contextEl.quadraticCurveTo(style.width, 0, style.width, style.radius);
            element.contextEl.lineTo(style.width, style.height - style.radius);
            element.contextEl.quadraticCurveTo(style.width, style.height, style.width - style.radius, style.height);
            element.contextEl.lineTo(style.radius, style.height);
            element.contextEl.quadraticCurveTo(0, style.height, 0, style.height - style.radius);
            element.contextEl.lineTo(0, style.radius);
            element.contextEl.quadraticCurveTo(0, 0, style.radius, 0);
            this._contextClosePath(style, element);
        },
		/**
         * To draw Custom Image
		 * @private
         */
        _drawCustomImage: function (element, imageUrl) {
            var image = new Image();
            var self = this;
            $(image).load(function () {
                element.contextEl.drawImage(this, 0, 0, element.model.Width, element.model.Height);
                if (element.model.scales != null)
                    element._drawScales();
                if (element.model.items != null)
                    element._renderItems();
            }).attr('src', imageUrl);
        },
		/**
         * To draw Ellipse
		 * @private
         */
        _drawEllipse: function (location, style, element) {
            var radius = Math.sqrt(style.height * style.height + style.width * style.width) / 2;
            style = this._setPointerDimension(style, element);
            this._contextOpenPath(style, element);
            element.contextEl.translate(location.startX, location.startY);
            this._setContextRotation(style, element);
            element.contextEl.scale(2, 1);
            element.contextEl.arc(radius / 2, 0, radius / 2, 0, Math.PI * 2, true);
            this._contextClosePath(style, element);
        },
		/**
         * To set Pointer Dimension
		 * @private
         */
        _setPointerDimension: function (style, element) {
            if (!ej.isNullOrUndefined(element.model.Orientation)) {
                if (element.model.Orientation == "Horizontal") {
                    var tempWidth = style.width;
                    var tempHeight = style.height;
                    style.height = tempWidth;
                    style.width = tempHeight;
                }
            }
            return style;
        },
		/**
         * To set Context Rotation
		 * @private
         */
        _setContextRotation: function (style, element) {
            element.contextEl.rotate(Math.PI * (style.angle / 180));
        },
		/**
         * To Manupulate the Context open path.
		 * @private
         */
        _contextOpenPath: function (style, element) {
            element.contextEl.save();
            element.contextEl.beginPath();
            if (style.strokeStyle)
                element.contextEl.strokeStyle = style.strokeStyle;
            if (style.opacity != undefined)
                element.contextEl.globalAlpha = style.opacity;
            if (style.lineWidth)
                element.contextEl.lineWidth = style.lineWidth;
            if (style.fillStyle)
                element.contextEl.fillStyle = style.fillStyle;
        },
		/**
         * To Manupulate the Context Close path.
		 * @private
         */
        _contextClosePath: function (style, element) {
            element.contextEl.closePath();
            if (style.isFill)
                element.contextEl.fill();
            if (style.isStroke)
                element.contextEl.stroke();
            element.contextEl.restore();
        },
		/**
         * To block default actions.
		 * @private
         */
        _blockDefaultActions: function (e) {
            e.cancelBubble = true;
            e.returnValue = false;
            if (e.preventDefault) e.preventDefault();
            if (e.stopPropagation) e.stopPropagation();
        },
		/**
         * To check whether the given value is between the first and last value
		 * @private
         */
        _isBetween: function (first, last, number) {
            return (first < last ? number >= first && number <= last : number >= last && number <= first);
        },
		/**
         * To get the color
		 * @private
         */
        _getColor: function (element, option) {
            if (typeof (option) === "string") {
                return option;
            }
            else {
                return ("rgba(" + option.r + ", " + option.g + "," + option.b + ", " + option.a / 255 + ")");
            }
        },
		/**
         * To set the GradientColor
		 * @private
         */
        _setGradientColor: function (element, gradients, options) {
            var self = element;
            if (options.Name || typeof (options) === "string") {
                gradients.addColorStop(0, this._getColor(element, options));
                gradients.addColorStop(1, this._getColor(element, options));
            }
            else
                $.each(options, function (index, colorElement) {
                    gradients.addColorStop(colorElement.colorStop != NaN ? colorElement.colorStop : 0, typeof (colorElement.color) === "string" ? colorElement.color : self._getColor(element, colorElement.color));
                });
        }
    });
    /**
	 * Enum for gauge Frame	 
	 * @enum {string}
	 * @global 
	 */
    ej.datavisualization.CircularGauge.Frame = {
        /**  To set the CircularGauge Frame as fullcircle.*/
        FullCircle: "fullcircle",
        /**  To set the CircularGauge Frame as halfcircle.*/
        HalfCircle: "halfcircle"
    };
    /**
	 * Enum for gauge Directions	 
	 * @enum {string}
	 * @global 
	 */
    ej.datavisualization.CircularGauge.Directions = {
        /**  To set the Directions clockwise. */
        Clockwise: "clockwise",
        /**  To set the Directions counterclockwise. */
        CounterClockwise: "counterclockwise"
    };
    /**
	 * Enum for gauge PointerPlacement	 
	 * @enum {string}
	 * @global 
	 */
    ej.datavisualization.CircularGauge.PointerPlacement = {
        /**  To set the PointerPlacement near. */
        Near: "near",
        /**  To set the PointerPlacement far. */
        Far: "far",
        /**  To set the PointerPlacement center. */
        Center: "center"
    };
    /**
	 * Enum for gauge PointerType	 
	 * @enum {string}
	 * @global 
	 */
    ej.datavisualization.CircularGauge.PointerType = {
        /**  To set the PointerType needle. */
        Needle: "needle",
        /**  To set the PointerType marker. */
        Marker: "marker"
    };
    /**
	 * Enum for gauge NeedleType	 
	 * @enum {string}
	 * @global 
	 */
    ej.datavisualization.CircularGauge.NeedleType = {
        /**  To set the NeedleType triangle. */
        Triangle: "triangle",
        /**  To set the NeedleType rectangle. */
        Rectangle: "rectangle",
        /**  To set the NeedleType trapezoid. */
        Trapezoid: "trapezoid",
        /**  To set the NeedleType arrow. */
        Arrow: "arrow"
    };
    /**
	 * Enum for gauge MarkerType	 
	 * @enum {string}
	 * @global 
	 */
    ej.datavisualization.CircularGauge.MarkerType = {
        /**  To set the MarkerType rectangle. */
        Rectangle: "rectangle",
        /**  To set the MarkerType triangle. */
        Triangle: "triangle",
        /**  To set the MarkerType ellipse. */
        Ellipse: "ellipse",
        /**  To set the MarkerType diamond. */
        Diamond: "diamond",
        /**  To set the MarkerType pentagon. */
        Pentagon: "pentagon",
        /**  To set the MarkerType circle. */
        Circle: "circle",
        /**  To set the MarkerType slider. */
        Slider: "slider",
        /**  To set the MarkerType pointer. */
        Pointer: "pointer",
        /**  To set the MarkerType wedge. */
        Wedge: "wedge",
        /**  To set the MarkerType trapezoid. */
        Trapezoid: "trapezoid",
        /**  To set the MarkerType roundedrectangle. */
        RoundedRectangle: "roundedrectangle"
    };
    /**
	 * Enum for gauge RangePlacement	 
	 * @enum {string}
	 * @global 
	 */
    ej.datavisualization.CircularGauge.RangePlacement = {
        /**  To set the RangePlacement near. */
        Near: "near",
        /**  To set the RangePlacement far. */
        Far: "far",
        /**  To set the RangePlacement center. */
        Center: "center"
    };
    /**
	 * Enum for gauge TickType	 
	 * @enum {string}
	 * @global 
	 */
    ej.datavisualization.CircularGauge.TickType = {
        /**  To set the TickType major. */
        Major: "major",
        /**  To set the TickType minor. */
        Minor: "minor"
    };
    /**
	 * Enum for gauge TickPlacement	 
	 * @enum {string}
	 * @global 
	 */
    ej.datavisualization.CircularGauge.TickPlacement = {
        /**  To set the TickPlacement near. */
        Near: "near",
        /**  To set the TickPlacement far. */
        Far: "far",
        /**  To set the TickPlacement center. */
        Center: "center"
    };
    /**
	 * Enum for gauge LabelPlacement	 
	 * @enum {string}
	 * @global 
	 */
    ej.datavisualization.CircularGauge.LabelPlacement = {
        /**  To set the LabelPlacement near. */
        Near: "near",
        /**  To set the LabelPlacement far. */
        Far: "far",
        /**  To set the LabelPlacement center. */
        Center: "center"
    };
    /**
	 * Enum for gauge LabelType	 
	 * @enum {string}
	 * @global 
	 */
    ej.datavisualization.CircularGauge.LabelType = {
        /**  To set the LabelType major. */
        Major: "major",
        /**  To set the LabelType minor. */
        Minor: "minor"
    };
    /**
	 * Enum for gauge UnitTextPlacement	 
	 * @enum {string}
	 * @global 
	 */
    ej.datavisualization.CircularGauge.UnitTextPlacement = {
        /**  To set the UnitTextPlacement back. */
        Back: "back",
        /**  To set the UnitTextPlacement front. */
        Front: "front"
    };
    /**
	 * Enum for gauge IndicatorType	 
	 * @enum {string}
	 * @global 
	 */
    ej.datavisualization.CircularGauge.IndicatorType = {
        /**  To set the IndicatorType rectangle. */
        Rectangle: "rectangle",
        /**  To set the IndicatorType circle. */
        Circle: "circle",
        /**  To set the IndicatorType roundedrectangle. */
        RoundedRectangle: "roundedrectangle",
        /**  To set the IndicatorType Text. */
        Text: "text",
        /**  To set the IndicatorType Text. */
        Image:"image"
    };
	/**
	 * Enum for gauge Themes	 
	 * @enum {string}
	 * @global 
	 */
    ej.datavisualization.CircularGauge.Themes = {
	/**  To set the themes flatlight. */
        flatlight: {
            backgroundColor: "#FFFFFF",
            scales: {
                pointerCap: {
                    borderColor: "#424242",
                    backgroundColor: "#424242"
                },
                backgroundColor: "#777777",
                border: {
                    color: "#777777"
                },
                pointers: {
                    backgroundColor: "#424242",
                    border: {
                        color:"#424242"
                    },
                },
                ticks: {
                    color: "#777777"
                },
                labels: {
                    color: "#282828"
                }
            }
        },
	/**  To set the themes flatdark. */
        flatdark: {
            backgroundColor: "#1F1F1F",
            scales: {
                pointerCap: {
                    borderColor: "#686868",
                    backgroundColor: "#686868"
                },
                backgroundColor: "#7a7a7a",
                border: {
                    color:"#7a7a7a"
                },
                pointers: {
                    backgroundColor: "#686868",
                    border: {
                        color:"#686868"
                    },
                },
                ticks: {
                    color: "#7a7a7a"
                },
                labels: {
                    color: "#d3d3d3"
                }
            }
        }
    };

})(jQuery, Syncfusion);;
/**
* @fileOverview Plugin to style the Html Button elements
* @copyright Copyright Syncfusion Inc. 2001 - 2013. All rights reserved.
*  Use of this code is subject to the terms of our license.
*  A copy of the current license can be obtained at any time by e-mailing
*  licensing@syncfusion.com. Any infringement will be prosecuted under
*  applicable laws. 
* @version 12.1 
* @author <a href="mailto:licensing@syncfusion.com">Syncfusion Inc</a>
*/

(function ($, ej, undefined) {
	/**
    * @namespace ej
    * @classdesc The Linera gauge  can be easily configured to the DOM element, such as div. you can create a linear gauge with a highly customizable look and feel.
	* @class ejLinearGauge
    * @param {object} options - For seting the Linear gauge  
	* @requires jQuery
	* @requires ej.common.all
    * @requires excanvas.js
	* @example 
	* &lt;div id="LinearGauge1"&gt;&lt;/div&gt; <br> 
	* &lt;script&gt;
    * $('#LinearGauge1').ejLinearGauge(); 	
	* &lt;/script&gt;
    * @remarks
    * excanvas.js need to add for IE8 
	*/
    // ejLinearGauge is the plugin name 
    // "ej.datavisualization.LinearGauge" is "namespace.className" will hold functions and properties

    ej.widget({ "ejLinearGauge": "ej.datavisualization.LinearGauge" /** @lends ejLinearGauge# */}, {
        // widget element will be automatically set in this
        element: null,
        _rootCSS: "e-lineargauge",
        _animationFlag: true,
        // user defined model will be automatically set in this
        model: null,
        validTags: ["div", "span"],

        // default model
        defaults:/** @lends ejLinearGauge# */{
			/**		
			* Specifies the value of the Gauge.
			* @default 0
            * @alias ejLinearGauge#value
			* @type {number}
			* @example 
	        * &lt;div id="LinearGauge1"&gt;&lt;/div&gt; <br> 
	        * &lt;script&gt;
			* $("#LinearGauge1").ejLinearGauge({  size: 5.5});	
	        * &lt;/script&gt;				 
			* @memberof ejLinearGauge
			* @instance
			*/	
            value: 0,
			/**		
			* Specifies the minimum value of Linear gauge.
			* @default 0
            * @alias ejLinearGauge#minimum
			* @type {number}
			* @example 
	        * &lt;div id="LinearGauge1"&gt;&lt;/div&gt; <br> 
	        * &lt;script&gt; 
			* 	$("#LinearGauge1").ejLinearGauge({ minimum: 10 });  
	        * &lt;/script&gt;	  
			* @memberof ejLinearGauge
			* @instance
			*/
            minimum: 0,
			/**		
			* Specifies the maximum value of Linear gauge.
			* @default 100
            * @alias ejLinearGauge#maximum
			* @type {number}
			* @example 
	        * &lt;div id="LinearGauge1"&gt;&lt;/div&gt; <br> 
	        * &lt;script&gt;
			* 	$("#LinearGauge1").ejLinearGauge({ maximum: 110 }); 
	        * &lt;/script&gt;	   
			* @memberof ejLinearGauge
			* @instance
			*/
            maximum: 100,
			/**		
			* Specifies the width of Linear gauge.
			* @default 150
            * @alias ejLinearGauge#width
			* @type {number}
			* @example 
	        * &lt;div id="LinearGauge1"&gt;&lt;/div&gt; <br> 
	        * &lt;script&gt;
			* 	$("#LinearGauge1").ejLinearGauge({ width: 360 });  
	        * &lt;/script&gt;	 
			* @memberof ejLinearGauge
			* @instance
			*/
            width: 150,
			/**		
			* Specifies the height of Linear gauge.
			* @default 400
            * @alias ejLinearGauge#height
			* @type {number}
			* @example 
	        * &lt;div id="LinearGauge1"&gt;&lt;/div&gt; <br> 
	        * &lt;script&gt; 
			* 	$("#LinearGauge1").ejLinearGauge({ height: 360 }); 
	        * &lt;/script&gt;	  
			* @memberof ejLinearGauge
			* @instance
			*/
            height: 400,
			/**		
			* Specifies the theme for Linear gauge. See {@link LinearGauge.Themes}
			* @default ej.datavisualization.LinearGauge.flatlight
            * @alias ejLinearGauge#theme
			* @type {enum}
			* @example 
	        * &lt;div id="LinearGauge1"&gt;&lt;/div&gt; <br> 
	        * &lt;script&gt;
			* 	$("#LinearGauge1").ejLinearGauge({ theme: ej.datavisualization.LinearGauge.flatdark });
	        * &lt;/script&gt;	  
			* @memberof ejLinearGauge
			* @instance
			*/
            theme: "flatlight",
			/**		
			* Specifies the orientation for Linear gauge.
			* @default "Vertical"
            * @alias ejLinearGauge#orientation
			* @type {string}
			* @example 
	        * &lt;div id="LinearGauge1"&gt;&lt;/div&gt; <br> 
	        * &lt;script&gt;
			* 	$("#LinearGauge1").ejLinearGauge({ orientation: "Vertical" });   
	        * &lt;/script&gt;	
			* @memberof ejLinearGauge
			* @instance
			*/
            orientation: "Vertical",
			/**		
			* Specifies the pointerGradient1 for Linear gauge.
			* @default null
            * @alias ejLinearGauge#pointerGradient1
			* @type {Object}
			* @example 
	        * &lt;div id="LinearGauge1"&gt;&lt;/div&gt; <br> 
	        * &lt;script&gt; 
			* 	$("#LinearGauge1").ejLinearGauge({ pointerGradient1: { colorInfo:[{ colorStop : 0, color:"#FFFFFF"},{colorStop : 1, color:"#AAAAAA"}] } });  
	        * &lt;/script&gt;	 
			* @memberof ejLinearGauge
			* @instance
			*/
            pointerGradient1: null,
			/**		
			* Specifies the pointerGradient2 for Linear gauge.
			* @default null
            * @alias ejLinearGauge#pointerGradient2
			* @type {object}
			* @example 
	        * &lt;div id="LinearGauge1"&gt;&lt;/div&gt; <br> 
	        * &lt;script&gt;
			* 	$("#LinearGauge1").ejLinearGauge({ pointerGradient2: { colorInfo:[{ colorStop : 0, color:"#FFFFFF"},{colorStop : 1, color:"#AAAAAA"}] } });  
	        * &lt;/script&gt;	 
			* @memberof ejLinearGauge
			* @instance
			*/
            pointerGradient2: null,
			/**		
			* Specifies the backgroundColor for Linear gauge.
			* @default null
            * @alias ejLinearGauge#backgroundColor
			* @type {string}
			* @example 
	        * &lt;div id="LinearGauge1"&gt;&lt;/div&gt; <br> 
	        * &lt;script&gt;
			* 	$("#LinearGauge1").ejLinearGauge({ backgroundColor: "Red" });   
	        * &lt;/script&gt;	
			* @instance
			*/
            backgroundColor: null,
			/**		
			* Specifies the borderColor for Linear gauge.
			* @default null
            * @alias ejLinearGauge#borderColor
			* @type {string}
			* @example 
	        * &lt;div id="LinearGauge1"&gt;&lt;/div&gt; <br> 
	        * &lt;script&gt;
			* 	$("#LinearGauge1").ejLinearGauge({ borderColor: "Red" });   
	        * &lt;/script&gt;	
			* @memberof ejLinearGauge
			* @instance
			*/
            borderColor: null,
			/**		
			* Specifies the labelColor for Linear gauge.
			* @default null
            * @alias ejLinearGauge#labelColor
			* @type {string}
			* @example 
	        * &lt;div id="LinearGauge1"&gt;&lt;/div&gt; <br> 
	        * &lt;script&gt;
			* 	$("#LinearGauge1").ejLinearGauge({ labelColor: "Red" });   
	        * &lt;/script&gt;	
			* @memberof ejLinearGauge
			* @instance
			*/
            labelColor: null,
			/**		
			* Specifies the tick Color for Linear gauge.
			* @default null
            * @alias ejLinearGauge#tickColor
			* @type {string}
			* @example 
	        * &lt;div id="LinearGauge1"&gt;&lt;/div&gt; <br> 
	        * &lt;script&gt;
			* 	$("#LinearGauge1").ejLinearGauge({ tickColor: "Red" });   
	        * &lt;/script&gt;	
			* @memberof ejLinearGauge
			* @instance
			*/
            tickColor: null,
			/**		
			* Specifies the read only state.
			* @default true
            * @alias ejLinearGauge#readOnly
			* @type {boolean}
			* @example 
	        * &lt;div id="LinearGauge1"&gt;&lt;/div&gt; <br> 
	        * &lt;script&gt;
			* 	$("#LinearGauge1").ejLinearGauge({ readOnly: false });   
	        * &lt;/script&gt;	
			* @memberof ejLinearGauge
			* @instance
			*/
            readOnly: true,
			/**		
			* Specifies the can resize state.
			* @default false
            * @alias ejLinearGauge#enableResize
			* @type {boolean}
			* @example 
	        * &lt;div id="LinearGauge1"&gt;&lt;/div&gt; <br> 
	        * &lt;script&gt;
			* 	$("#LinearGauge1").ejLinearGauge({ enableResize: true });   
	        * &lt;/script&gt;	
			* @memberof ejLinearGauge
			* @instance
			*/
            enableResize: false,
            /**		
		    * Specify frame of linear gauge
		    * @default Object
            * @alias ejLinearGauge#frame
            * @type {object}
            * @example  
            * &lt;div id="LinearGauge1"&gt;
            * &lt;/div&gt; <br> 
            * &lt;script&gt;			
    	    * $("#LinearGauge1").ejLinearGauge({ frame: { backgroundImageUrl:null, outerWidth:12, innerWidth:8 } });
            * &lt;/script&gt;
		    * @memberof ejLinearGauge
		    * @instance
		    */
			frame: {
			    /**		
			    * Specifies the frame backgroundimageurl of linear gauge
			    * @default null
                * @alias ejLinearGauge#frame->backgroundImageUrl
			    * @type {string}
			    * @example 
	            * &lt;div id="LinearGauge1"&gt;&lt;/div&gt; <br> 
	            * &lt;script&gt;  
			    * 	$("#LinearGauge1").ejLinearGauge({ frame:{backgroundImageUrl: "bg.png"} });  
	            * &lt;/script&gt;	
			    * @memberof ejLinearGauge
			    * @instance
			    */
			    backgroundImageUrl: null,
			    /**		
			    * Specifies the frame OuterWidth
			    * @default 12
                * @alias ejLinearGauge#frame->outerWidth
			    * @type {number}
			    * @example 
	            * &lt;div id="LinearGauge1"&gt;&lt;/div&gt; <br> 
	            * &lt;script&gt;
			    * 	$("#LinearGauge1").ejLinearGauge({ frame:{outerWidth: 13 }});   
	            * &lt;/script&gt;	
			    * @instance
			    */
			    outerWidth: 12,
			    /**		
			    * Specifies the frame InnerWidth
			    * @default 8
                * @alias ejLinearGauge#frame->innerWidth
			    * @type {number}
			    * @example 
	            * &lt;div id="LinearGauge1"&gt;&lt;/div&gt; <br> 
	            * &lt;script&gt;
			    * 	$("#LinearGauge1").ejLinearGauge({ frame:{innerWidth: 9}});   
	            * &lt;/script&gt;	
			    * @memberof ejLinearGauge
			    * @instance
			    */
                innerWidth:8
			},
			/**		
			* Specifies the scales
			* @default Object
            * @alias ejLinearGauge#scales
			* @type {Object}
			* @example 
	        * &lt;div id="LinearGauge1"&gt;&lt;/div&gt; <br> 
	        * &lt;script&gt;
			* 	$("#LinearGauge1").ejLinearGauge({ scales: [{}]});   
	        * &lt;/script&gt;	
			* @memberof ejLinearGauge
			* @instance
			*/
            scales: null,
			/**		
			* Specifies the animate state
			* @default true
            * @alias ejLinearGauge#enableAnimation
			* @type {boolean}
			* @example 
	        * &lt;div id="LinearGauge1"&gt;&lt;/div&gt; <br> 
	        * &lt;script&gt;
			* 	$("#LinearGauge1").ejLinearGauge({ enableAnimation: true, value:50});  
	        * &lt;/script&gt;	
			* @memberof ejLinearGauge
			* @instance
			*/
            enableAnimation: true,
            /**		
           * Specifies the animate state for marker pointer
           * @default true
           * @alias ejLinearGauge#enableMarkerPointerAnimation
           * @type {boolean}
           * @example 
           * &lt;div id="LinearGauge1"&gt;&lt;/div&gt; <br> 
           * &lt;script&gt;
           * 	$("#LinearGauge1").ejLinearGauge({ enableMarkerPointerAnimation: true, value:50});  
           * &lt;/script&gt;	
           * @memberof ejLinearGauge
           * @instance
           */
            enableMarkerPointerAnimation: true,
			/**		
			* Specifies the animationSpeed
			* @default 500
            * @alias ejLinearGauge#animationSpeed
			* @type {number}
			* @example 
	        * &lt;div id="LinearGauge1"&gt;&lt;/div&gt; <br> 
	        * &lt;script&gt;
			* 	$("#LinearGauge1").ejLinearGauge({ animationSpeed: 1000, value:50});
	        * &lt;/script&gt;	   
			* @memberof ejLinearGauge
			* @instance
			*/
            animationSpeed: 500,
			/**     
			 * Triggers while the ticks are being drawn on the gauge.
			 * @event
			 * @name ejLinearGauge#drawTicks		
             * @param {Object} args.object  returns the object of the gauge.
			 * @param {boolean } args.cancel returns the cancel option value
			 * @param {object} args.context returns the context element
			 * @param {oject} args.position returns the startX and startY of the ticks
 			 * @param {oject} args.Model returns the gauge model
 			 * @param {object} args.scaleElement returns the options of the scale element.
 			 * @param {number} args.scaleIndex returns the scaleIndex to which the tick belongs.
   			 * @param {string} args.style returns the ticks style
             * @param {oject} args.tick returns the ticks style
			 * @param {number} args.tick.angle returns the angle of the tick.
			 * @param {object} args.tick.element returns the current tick element.
			 * @param {number} args.tick.index returns the index of the tick.
			 * @param {number} args.tick.value returns the tick value of the tick.
			 * @param {type} args.type returns the name of the event
			 * @example 
	         * &lt;div id="LinearGauge1"&gt;&lt;/div&gt; <br> 
	         * &lt;script&gt;
             * $("#LinearGauge1").ejLinearGauge({
             *    drawTicks: function (args) {}
             * });
	         * &lt;/script&gt;	
			 * @memberof ejLinearGauge
			 * @instance
			 */
            drawTicks: null,
			/**     
			 * Triggers while the label are being drawn on the gauge.
			 * @event
			 * @name ejLinearGauge#drawLabels		
             * @param {Object} args.object  returns the object of the gauge.
			 * @param {boolean } args.cancel returns the cancel option value
			 * @param {object} args.context returns the context element
			 * @param {object} args.position returns the startX and startY of the label
 			 * @param {object} args.Model returns the gauge model
 			 * @param {object} args.scaleElement returns the options of the scale element.
 			 * @param {number} args.scaleIndex returns the scaleIndex to which the label belongs.
   			 * @param {string} args.style returns the label style
             * @param {object} args.label returns the label style
			 * @param {number} args.label.angle returns the angle of the label.
			 * @param {oject} args.label.element returns the current label element.
			 * @param {number} args.label.index returns the index of the label.
			 * @param {number} args.label.value returns the label value of the label.
			 * @param {type} args.type returns the name of the event
			 * @example 
	         * &lt;div id="LinearGauge1"&gt;&lt;/div&gt; <br> 
	         * &lt;script&gt;
             * $("#LinearGauge1").ejLinearGauge({
             *    drawLabels: function (args) {}
             * });
	         * &lt;/script&gt;	
			 * @memberof ejLinearGauge
			 * @instance
			 */
            drawLabels: null,
			/**     
			 * Triggers while the bar pointer are being drawn on the gauge.
			 * @event
			 * @name ejLinearGauge#drawBarPointers		
             * @param {Object} args.object returns the object of the gauge.
			 * @param {boolean}  args.cancel returns the cancel option value
			 * @param {object} args.context returns the context element
			 * @param {object} args.position returns the startX and startY of the pointer
 			 * @param {object} args.Model returns the gauge model
 			 * @param {object} args.scaleElement returns the options of the scale element.
 			 * @param {number} args.scaleIndex returns the scaleIndex to which the pointer belongs.
   			 * @param {string} args.style returns the pointer style
			 * @param {object} args.barElement returns the current Bar pointer element.
			 * @param {number} args.barPointerIndex returns the index of the bar pointer.
			 * @param {number} args.PointerValue returns the value of the bar pointer.
			 * @param {type} args.type returns the name of the event
			 * @example 
	         * &lt;div id="LinearGauge1"&gt;&lt;/div&gt; <br> 
	         * &lt;script&gt;
             * $("#LinearGauge1").ejLinearGauge({
             *    drawBarPointers: function (args) {}
             * });
	         * &lt;/script&gt;	
			 * @memberof ejLinearGauge
			 * @instance
			 */
            drawBarPointers: null,
			/**     
			 * Triggers while the marker are being drawn on the gauge.
			 * @event
			 * @name ejLinearGauge#drawMarkerPointers		
             * @param {Object} args.object returns the object of the gauge.
			 * @param {boolean}  args.cancel returns the cancel option value
			 * @param {object} args.context returns the context element
			 * @param {object} args.position returns the startX and startY of the pointer
 			 * @param {object} args.Model returns the gauge model
 			 * @param {object} args.scaleElement returns the options of the scale element.
 			 * @param {number} args.scaleIndex returns the scaleIndex to which the pointer belongs.
   			 * @param {string} args.style returns the ticks style
			 * @param {object} args.markerElement returns the current marker pointer element.
			 * @param {number} args.markerPointerIndex returns the index of the marker pointer.
			 * @param {number} args.pointerValue returns the value of the marker pointer.
			 * @param {number} args.pointerAngle returns the angle of the marker pointer.
			 * @param {type} args.type returns the name of the event
			 * @example 
	         * &lt;div id="LinearGauge1"&gt;&lt;/div&gt; <br> 
	         * &lt;script&gt;
             * $("#LinearGauge1").ejLinearGauge({
             *    drawMarkerPointers: function (args) {}
             * });
	         * &lt;/script&gt;	
			 * @memberof ejLinearGauge
			 * @instance
			 */
            drawMarkerPointers: null,
			/**     
			 * Triggers while the range are being drawn on the gauge.
			 * @event
			 * @name ejLinearGauge#drawRange		
             * @param {Object} args.object  returns the object of the gauge.
			 * @param {bolean } args.cancel returns the cancel option value
			 * @param {object} args.context returns the context element
			 * @param {object} args.position returns the startX and startY of the range
 			 * @param {object} args.Model returns the gauge model
 			 * @param {object} args.scaleElement returns the options of the scale element.
 			 * @param {number} args.scaleIndex returns the scaleIndex to which the pointer belongs.
   			 * @param {string} args.style returns the range style
			 * @param {object} args.rangeElement returns the current range element.
			 * @param {number} args.rangeIndex returns the index of the range.
			 * @param {type} args.type returns the name of the event
			 * @example 
	         * &lt;div id="LinearGauge1"&gt;&lt;/div&gt; <br> 
	         * &lt;script&gt;
             * $("#LinearGauge1").ejLinearGauge({
             *    drawRange: function (args) {}
             * });
	         * &lt;/script&gt;	
			 * @memberof ejLinearGauge
			 * @instance
			 */
            drawRange: null,
			/**     
			 * Triggers while the customLabel are being drawn on the gauge.
			 * @event
			 * @name ejLinearGauge#drawCustomLabel		
             * @param {Object} args.object returns the object of the gauge.
			 * @param {boolean } args.cancel returns the cancel option value
			 * @param {object} args.context returns the context element
			 * @param {object} args.position returns the startX and startY of the customLabel
 			 * @param {object} args.Model returns the gauge model
 			 * @param {object} args.scaleElement returns the options of the scale element.
 			 * @param {number} args.scaleIndex returns the scaleIndex to which the pointer belongs.
   			 * @param {object} args.style returns the customLabel style
			 * @param {object} args.customLabelElement returns the current customLabel element.
			 * @param {number} args.customLabelIndex returns the index of the customLabel.
			 * @param {type} args.type returns the name of the event
			 * @example 
	         * &lt;div id="LinearGauge1"&gt;&lt;/div&gt; <br> 
	         * &lt;script&gt;
             * $("#LinearGauge1").ejLinearGauge({
             *    drawCustomLabel: function (args) {}
             * });
	         * &lt;/script&gt;	
			 * @memberof ejLinearGauge
			 * @instance
			 */
            drawCustomLabel: null,
			/**     
			 * Triggers while the Indicator are being drawn on the gauge.
			 * @event
			 * @name ejLinearGauge#drawIndicators		
             * @param {Object} args.object returns the object of the gauge.
			 * @param {boolean } args.cancel returns the cancel option value
			 * @param {object} args.context returns the context element
			 * @param {object} args.position returns the startX and startY of the Indicator
 			 * @param {object} args.Model returns the gauge model
 			 * @param {object} args.scaleElement returns the options of the scale element.
 			 * @param {numer} args.scaleIndex returns the scaleIndex to which the pointer belongs.
   			 * @param {string} args.style returns the Indicator style
			 * @param {object} args.IndicatorElement  returns the current Indicator element.
			 * @param {number} args.IndicatorIndex returns the index of the Indicator.
			 * @param {type} args.type returns the name of the event
			 * @example 
	         * &lt;div id="LinearGauge1"&gt;&lt;/div&gt; <br> 
	         * &lt;script&gt;
             * $("#LinearGauge1").ejLinearGauge({
             *    drawIndicators: function (args) {}
             * });
	         * &lt;/script&gt;	
			 * @memberof ejLinearGauge
			 * @instance
			 */
            drawIndicators: null,
			/**     
			 * Triggers while the gauge start to Load.
			 * @event
			 * @name ejLinearGauge#load		
             * @param {Object} args.object returns the object of the gauge.
			 * @param {boolean } args.cancel returns the cancel option value
 			 * @param {object} args.Model returns the gauge model
 			 * @param {object} args.scaleElement returns the entire scale element.
			 * @param {object} args.context returns the context element
			 * @param {type} args.type returns the name of the event
			 * @example 
	         * &lt;div id="LinearGauge1"&gt;&lt;/div&gt; <br> 
	         * &lt;script&gt;
             * $("#LinearGauge1").ejLinearGauge({
             *    load: function (args) {}
             * });
	         * &lt;/script&gt;	
			 * @memberof ejLinearGauge
			 * @instance
			 */
            load: null,
			/**     
			 * Triggers when the gauge is initialized.
			 * @event
			 * @name ejLinearGauge#init		
             * @param {Object} args.object returns the object of the gauge.
			 * @param {boolean } args.cancel returns the cancel option value
 			 * @param {object} args.Model returns the gauge model
 			 * @param {object} args.scaleElement returns the entire scale element.
			 * @param {object} args.context returns the context element
			 * @param {type} args.type eturns the name of the event
			 * @example 
	         * &lt;div id="LinearGauge1"&gt;&lt;/div&gt; <br> 
	         * &lt;script&gt;
             * $("#LinearGauge1").ejLinearGauge({
             *    init: function (args) {}
             * });
	         * &lt;/script&gt;	
			 * @memberof ejLinearGauge
			 * @instance
			 */
            init: null,
			/**     
			 * Triggers while the rendering of the gauge completed.
			 * @event
			 * @name ejLinearGauge#renderComplete		
             * @param {Object} args.object returns the object of the gauge.
			 * @param {boolean } args.cancel returns the cancel option value
 			 * @param {Object} args.Model returns the gauge model
 			 * @param {Object} args.scaleElement returns the entire scale element.
			 * @param {Object} args.context returns the context element
			 * @param {type} args.type returns the name of the event
			 * @example 
	         * &lt;div id="LinearGauge1"&gt;&lt;/div&gt; <br> 
	         * &lt;script&gt;
             * $("#LinearGauge1").ejLinearGauge({
             *    renderComplete: function (args) {}
             * });
	         * &lt;/script&gt;	
			 * @memberof ejLinearGauge
			 * @instance
			 */
            renderComplete: null,
            /**     
			 * Triggers when the left mouse button is clicked.
			 * @event
			 * @name ejLinearGauge#mouseClick		
             * @param {Object} args.object returns the object of the gauge.
             * @param {boolean} args.cancel returns the cancel option value
             * @param {Object} args.model returns the gauge model
             * @param {Object} args.type returns the name of the event
             * @param {Object} args.scaleElement returns the scale element.
             * @param {number} args.scaleIndex returns the scaleIndex to which the pointer belongs.
             * @param {Object} args.context returns the context element* @param {Object} args.markerpointer returns the context element
             * @param {number} args.markerpointer.index returns the pointer Index
             * @param {Object} args.markerpointer.element returns the pointer element.
             * @param {number} args.markerpointer.value returns the value of the pointer.
             * @param {string} args.style returns the pointer style
             * @param {Object} args.position returns the startX and startY of the pointer.
			 * @example 
             * &lt;div id="LinearGauge1"&gt;
             * &lt;/div&gt; <br> 
             * &lt;script&gt;
             * $("#LinearGauge1").ejLinearGauge({
             *    mouseClick: function (args) {}
             * });
            * &lt;/script&gt;
			 * @memberof ejLinearGauge
			 * @instance
			 */
            mouseClick: null,
            /**     
			 * Triggers when clicking and dragging the mouse pointer over the gauge pointer.
			 * @event
			 * @name ejLinearGauge#mouseClickMove		
             * @param {Object} args.object returns the object of the gauge.
             * @param {boolean} args.cancel returns the cancel option value
             * @param {Object} args.model returns the gauge model
             * @param {Object} args.type returns the name of the event
             * @param {Object} args.scaleElement returns the scale element.
             * @param {number} args.scaleIndex returns the scaleIndex to which the pointer belongs.
             * @param {Object} args.context returns the context element
             * @param {Object} args.markerpointer returns the context element
             * @param {number} args.markerpointer.index returns the pointer Index
             * @param {Object} args.markerpointer.element returns the pointer element.
             * @param {number} args.markerpointer.value returns the value of the pointer.
             * @param {string} args.style returns the pointer style
             * @param {Object} args.position returns the startX and startY of the pointer.
			 * @example 
             * &lt;div id="LinearGauge1"&gt;
             * &lt;/div&gt; <br> 
             * &lt;script&gt;
             * $("#LinearGauge1").ejLinearGauge({
             *    mouseClickMove: function (args) {}
             * });
            * &lt;/script&gt;
			 * @memberof ejLinearGauge
			 * @instance
			 */
            mouseClickMove: null,
            /**     
			 * Triggers when the mouse click is released.
			 * @event
			 * @name ejLinearGauge#mouseClickUp		
             * @param {Object} args.object returns the object of the gauge.
             * @param {boolean} args.cancel returns the cancel option value
             * @param {Object} args.model returns the gauge model
             * @param {Object} args.type returns the name of the event
             * @param {Object} args.scaleElement returns the scale element.
             * @param {number} args.scaleIndex returns the scaleIndex to which the pointer belongs.
             * @param {Object} args.context returns the context element* @param {Object} args.markerpointer returns the context element
             * @param {number} args.markerpointer.index returns the pointer Index
             * @param {Object} args.markerpointer.element returns the pointer element.
             * @param {number} args.markerpointer.value returns the value of the pointer.
             * @param {string} args.style returns the pointer style
             * @param {Object} args.position returns the startX and startY of the pointer.
			 * @example 
             * &lt;div id="LinearGauge1"&gt;
             * &lt;/div&gt; <br> 
             * &lt;script&gt;
             * $("#LinearGauge1").ejLinearGauge({
             *    mouseClickUp: function (args) {}
             * });
            * &lt;/script&gt;
			 * @memberof ejLinearGauge
			 * @instance
			 */
            mouseClickUp: null,
        },
				/**		
				* Specifies the scales.
				* @default object
                * @alias ejLinearGauge#scales
				* @type {object}
				* @example 
	            * &lt;div id="LinearGauge1"&gt;&lt;/div&gt; <br> 
	            * &lt;script&gt;
				* 	$("#LinearGauge1").ejLinearGauge({  scales:[{minimum:5}]});
	            * &lt;/script&gt;				
				* @memberof ejLinearGauge
				* @instance
				*/	
        _defaultScaleValues: function () {
            return {
				/**		
				* Specifies the minimum of the Scale.
				* @default null
                * @alias ejLinearGauge#scales->minimum
				* @type {number}
				* @example 
	            * &lt;div id="LinearGauge1"&gt;&lt;/div&gt; <br> 
	            * &lt;script&gt;
				* 	$("#LinearGauge1").ejLinearGauge({  scales:[{minimum:10}]});	
	            * &lt;/script&gt;						
				* @memberof ejLinearGauge
				* @instance
				*/	
                minimum: null,
				/**		
				* Specifies the maximum of the Scale.
				* @default null
                * @alias ejLinearGauge#scales->maximum
				* @type {number}
				* @example 
	            * &lt;div id="LinearGauge1"&gt;&lt;/div&gt; <br> 
	            * &lt;script&gt;
				* 	$("#LinearGauge1").ejLinearGauge({  scales:[{maximum:110}]});	
	            * &lt;/script&gt;					
				* @memberof ejLinearGauge
				* @instance
				*/	
                maximum: null,
				/**		
				* Specifies the majorIntervalValue of the Scale.
				* @default 10
                * @alias ejLinearGauge#scales->majorIntervalValue
				* @type {number}
				* @example 
	            * &lt;div id="LinearGauge1"&gt;&lt;/div&gt; <br> 
	            * &lt;script&gt; 
				* 	$("#LinearGauge1").ejLinearGauge({  scales:[{majorIntervalValue:10}]});
	            * &lt;/script&gt;						
				* @memberof ejLinearGauge
				* @instance
				*/	
                majorIntervalValue: 10,
				/**		
				* Specifies the minorIntervalValue of the Scale.
				* @default 2
                * @alias ejLinearGauge#scales->minorIntervalValue
				* @type {number}
				* @example 
	            * &lt;div id="LinearGauge1"&gt;&lt;/div&gt; <br> 
	            * &lt;script&gt;
				* 	$("#LinearGauge1").ejLinearGauge({  scales:[{minorIntervalValue:1}]});	
	            * &lt;/script&gt;						
				* @memberof ejLinearGauge
				* @instance
				*/	
                minorIntervalValue: 2,
				/**		
				* Specifies the scale Direction of the Scale. See {@link Directions}
				* @default ej.datavisualization.LinearGauge.Directions.CounterClockwise
                * @alias ejLinearGauge#scales->direction
				* @type {enum}
				* @example 
	            * &lt;div id="LinearGauge1"&gt;&lt;/div&gt; <br> 
	            * &lt;script&gt;
				* 	$("#LinearGauge1").ejLinearGauge({  scales:[{direction:ej.datavisualization.LinearGauge.Directions.Clockwise}]});	
	            * &lt;/script&gt;					
				* @memberof ejLinearGauge
				* @instance
				*/	
                direction: "counterclockwise",
				/**		
				* Specifies the backgroundColor of the Scale.
				* @default null
                * @alias ejLinearGauge#scales->backgroundColor
				* @type {string}
				* @example 
	            * &lt;div id="LinearGauge1"&gt;&lt;/div&gt; <br> 
	            * &lt;script&gt;
				* 	$("#LinearGauge1").ejLinearGauge({  scales:[{backgroundColor:"red"}]});	
	            * &lt;/script&gt;				
				* @memberof ejLinearGauge
				* @instance
				*/	
                backgroundColor: null,
				/**		
				* Specifies the border of the Scale.
				* @default Object
                * @alias ejLinearGauge#scales->border
				* @type {object}
				* @example 
	            * &lt;div id="LinearGauge1"&gt;&lt;/div&gt; <br> 
	            * &lt;script&gt;
				* 	$("#LinearGauge1").ejLinearGauge({  scales:[{border:{color:"red", width:1.5}}]});	
	            * &lt;/script&gt;					
				* @memberof ejLinearGauge
				* @instance
				*/	
                border: {
                    /**		
				    * Specifies the border color of the Scale.
				    * @default null
                    * @alias ejLinearGauge#scales->border->color
				    * @type {string}
				    * @example 
	                * &lt;div id="LinearGauge1"&gt;&lt;/div&gt; <br> 
	                * &lt;script&gt;
				    * 	$("#LinearGauge1").ejLinearGauge({  scales:[{border:{color:"red"}}]});	
	                * &lt;/script&gt;					
				    * @memberof ejLinearGauge
				    * @instance
				    */
                    color: null,
                    /**
				    * Specifies the border width of the Scale.
				    * @default 1.5
                    * @alias ejLinearGauge#scales->border->width
				    * @type {number}
				    * @example 
	                * &lt;div id="LinearGauge1"&gt;&lt;/div&gt; <br> 
	                * &lt;script&gt;
				    * 	$("#LinearGauge1").ejLinearGauge({  scales:[{border:{width:2.5}}]});	
	                * &lt;/script&gt;					
				    * @memberof ejLinearGauge
				    * @instance
				    */
                    width:1.5
                },
				/**		
				* Specifies the opacity of the Scale.
				* @default NaN
                * @alias ejLinearGauge#scales->opacity
				* @type {number}
				* @example 
	            * &lt;div id="LinearGauge1"&gt;&lt;/div&gt; <br> 
	            * &lt;script&gt;
				* 	$("#LinearGauge1").ejLinearGauge({  scales:[{opacity:0.2}]});	
	            * &lt;/script&gt;						
				* @memberof ejLinearGauge
				* @instance
				*/	
                opacity: NaN,
				/**		
				* Specifies the scaleBar width.
				* @default 30
                * @alias ejLinearGauge#scales->width
				* @type {number}
				* @example 
	            * &lt;div id="LinearGauge1"&gt;&lt;/div&gt; <br> 
	            * &lt;script&gt;
				* 	$("#LinearGauge1").ejLinearGauge({  scales:[{width:25}]});		
	            * &lt;/script&gt;					
				* @memberof ejLinearGauge
				* @instance
				*/	
                width: 30,
				/**		
				* Specifies the shadowOffset.
				* @default 0
                * @alias ejLinearGauge#scales->shadowOffset
				* @type {number}
				* @example 
	            * &lt;div id="LinearGauge1"&gt;&lt;/div&gt; <br> 
	            * &lt;script&gt;
				* 	$("#LinearGauge1").ejLinearGauge({  scales:[{shadowOffset:1}]});		
	            * &lt;/script&gt;				
				* @memberof ejLinearGauge
				* @instance
				*/	
                shadowOffset: 0,
				/**		
				* Specifies the scaleBar Length.
				* @default 290
                * @alias ejLinearGauge#scales->length
				* @type {number}
				* @example 
	            * &lt;div id="LinearGauge1"&gt;&lt;/div&gt; <br> 
	            * &lt;script&gt;
				* 	$("#LinearGauge1").ejLinearGauge({  scales:[{length:300}]});
	            * &lt;/script&gt;						
				* @memberof ejLinearGauge
				* @instance
				*/	
                length: 290,
				/**		
				* Specifies the scaleBar type .See {@link ScaleType}
				* @default ej.datavisualization.LinearGauge.ScaleType.Rectangle
                * @alias ejLinearGauge#scales->type
				* @type {enum}
				* @example 
	            * &lt;div id="LinearGauge1"&gt;&lt;/div&gt; <br> 
	            * &lt;script&gt;
				* 	$("#LinearGauge1").ejLinearGauge({  scales:[{type:ej.datavisualization.LinearGauge.ScaleType.Rectangle}]});	
	            * &lt;/script&gt;				
				* @memberof ejLinearGauge
				* @instance
				*/	
                type: "rectangle",
				/**		
				* Specifies the position
				* @default Object
                * @alias ejLinearGauge#scales->position
				* @type {object}
				* @example 
	            * &lt;div id="LinearGauge1"&gt;&lt;/div&gt; <br> 
	            * &lt;script&gt;
				* 	$("#LinearGauge1").ejLinearGauge({  scales:[{position:{x:30, y:30}}]});		
	            * &lt;/script&gt;					
				* @instance
				*/	
                position: { 
				/**		
				* Specifies the Horizontal position
				* @default 50
                * @alias ejLinearGauge#scales->position->x
				* @type {number}
				* @example 
	            * &lt;div id="LinearGauge1"&gt;&lt;/div&gt; <br> 
	            * &lt;script&gt;
				* 	$("#LinearGauge1").ejLinearGauge({  scales:[{position:{x:30}}]});		
	            * &lt;/script&gt;				
				* @memberof ejLinearGauge
				* @instance
				*/	
				x: 50, 
				/**		
				* Specifies the vertical position
				* @default 50
                * @alias ejLinearGauge#scales->position->y
				* @type {number}
				* @example 
	            * &lt;div id="LinearGauge1"&gt;&lt;/div&gt; <br> 
	            * &lt;script&gt;
				* 	$("#LinearGauge1").ejLinearGauge({  scales:[{position:{y:30}}]});		
	            * &lt;/script&gt;					
				* @memberof ejLinearGauge
				* @instance
				*/	
				y: 50 },
				/**		
				* Specifies the showRanges state.
				* @default false
                * @alias ejLinearGauge#scales->showRanges
				* @type {boolean}
				* @example 
	            * &lt;div id="LinearGauge1"&gt;&lt;/div&gt; <br> 
	            * &lt;script&gt;
				* 	$("#LinearGauge1").ejLinearGauge({  scales:[{showRanges:false}]});	
	            * &lt;/script&gt;					
				* @memberof ejLinearGauge
				* @instance
				*/	
                showRanges: false,
				/**		
				* Specifies the showIndicators state.
				* @default false
                * @alias ejLinearGauge#scales->showIndicators
				* @type {boolean}
				* @example 
	            * &lt;div id="LinearGauge1"&gt;&lt;/div&gt; <br> 
	            * &lt;script&gt;
				* 	$("#LinearGauge1").ejLinearGauge({  scales:[{showIndicators:true}]});	
	            * &lt;/script&gt;						
				* @memberof ejLinearGauge
				* @instance
				*/	
                showIndicators: false,
				/**		
				* Specifies the showCustomLabels state.
				* @default false
                * @alias ejLinearGauge#scales->showCustomLabels
				* @type {boolean}
				* @example 
	            * &lt;div id="LinearGauge1"&gt;&lt;/div&gt; <br> 
	            * &lt;script&gt; 
				* 	$("#LinearGauge1").ejLinearGauge({  scales:[{showCustomLabels:true}]});	
	            * &lt;/script&gt;					
				* @memberof ejLinearGauge
				* @instance
				*/	
                showCustomLabels: false,
				/**		
				* Specifies the showLabels state.
				* @default true
                * @alias ejLinearGauge#scales->showLabels
				* @type {boolean}
				* @example 
	            * &lt;div id="LinearGauge1"&gt;&lt;/div&gt; <br> 
	            * &lt;script&gt;
				* 	$("#LinearGauge1").ejLinearGauge({  scales:[{showLabels:false}]});		
	            * &lt;/script&gt;				
				* @memberof ejLinearGauge
				* @instance
				*/	
                showLabels: true,
				/**		
				* Specifies the showTicks state.
				* @default true
                * @alias ejLinearGauge#scales->showTicks
				* @type {boolean}
				* @example 
	            * &lt;div id="LinearGauge1"&gt;&lt;/div&gt; <br> 
	            * &lt;script&gt;
				* 	$("#LinearGauge1").ejLinearGauge({  scales:[{showTicks:false}]});			
	            * &lt;/script&gt;				
				* @memberof ejLinearGauge
				* @instance
				*/	
                showTicks: true,
				/**		
				* Specifies the showBarPointers state.
				* @default true
                * @alias ejLinearGauge#scales->showBarPointers
				* @type {boolean}
				* @example 
	            * &lt;div id="LinearGauge1"&gt;&lt;/div&gt; <br> 
	            * &lt;script&gt; 
				* 	$("#LinearGauge1").ejLinearGauge({  scales:[{showBarPointers:false}]});		
	            * &lt;/script&gt;				
				* @memberof ejLinearGauge
				* @instance
				*/	
                showBarPointers: true,
				/**		
				* Specifies the showMarkerPointers state.
				* @default true
                * @alias ejLinearGauge#scales->showMarkerPointers
				* @type {boolean}
				* @example 
	            * &lt;div id="LinearGauge1"&gt;&lt;/div&gt; <br> 
	            * &lt;script&gt; 
				* 	$("#LinearGauge1").ejLinearGauge({  scales:[{showMarkerPointers:false}]});	
	            * &lt;/script&gt;				
				* @memberof ejLinearGauge
				* @instance
				*/	
                showMarkerPointers: true,
				/**		
				* Specifies the ticks in the scale.
				* @default Array
                * @alias ejLinearGauge#scales->ticks
				* @type {Array}
				* @example 
	            * &lt;div id="LinearGauge1"&gt;&lt;/div&gt; <br> 
	            * &lt;script&gt;
				* 	$("#LinearGauge1").ejLinearGauge({  scales:[{ticks:[{angle:30}]}]});		
	            * &lt;/script&gt;					
				* @memberof ejLinearGauge
				* @instance
				*/	
                ticks: [{
                    /**		
					* Specifies the DistanceFromScale in the tick.
					* @default Object
                    * @alias ejLinearGauge#scales->ticks->distanceFromScale
					* @type {object}
					* @example 
    	            * &lt;div id="LinearGauge1"&gt;&lt;/div&gt; <br> 
	                * &lt;script&gt;
					* 	$("#LinearGauge1").ejLinearGauge({  scales:[{ticks:[{distanceFromScale:{x:10, y:10}}]}]});	
	                * &lt;/script&gt;				
					* @memberof ejLinearGauge
					* @instance
					*/                    
                    distanceFromScale: {
                        /**		
					    * Specifies the xDistanceFromScale in the tick.
					    * @default 0
                        * @alias ejLinearGauge#scales->ticks->distanceFromScale->x
					    * @type {number}
					    * @example 
    	                * &lt;div id="LinearGauge1"&gt;&lt;/div&gt; <br> 
	                    * &lt;script&gt;
					    * 	$("#LinearGauge1").ejLinearGauge({  scales:[{ticks:[{distanceFromScale:{x:10}}]}]});	
	                    * &lt;/script&gt;				
					    * @memberof ejLinearGauge
					    * @instance
					    */
                        x: 0,
                        /**		
					    * Specifies the yDistanceFromScale in the tick.
					    * @default 0
                        * @alias ejLinearGauge#scales->ticks->distanceFromScale->y
					    * @type {number}
					    * @example 
	                    * &lt;div id="LinearGauge1"&gt;&lt;/div&gt; <br> 
	                    * &lt;script&gt;  
					    * 	$("#LinearGauge1").ejLinearGauge({  scales:[{ticks:[{distanceFromScale:{y:10}}]}]});		
	                    * &lt;/script&gt;			
					    * @memberof ejLinearGauge
					    * @instance
					    */
                        y:0
                    },
					/**		
					* Specifies the angle in the tick.
					* @default 0
                    * @alias ejLinearGauge#scales->ticks->angle
					* @type {number}
					* @example 
	                * &lt;div id="LinearGauge1"&gt;&lt;/div&gt; <br> 
	                * &lt;script&gt;
					* 	$("#LinearGauge1").ejLinearGauge({  scales:[{ticks:[{angle:20}]}]});				
	                * &lt;/script&gt;			 
					* @memberof ejLinearGauge
					* @instance
					*/	
                    angle: 0,
					/**		
					* Specifies the tick Color in the tick.
					* @default null
                    * @alias ejLinearGauge#scales->ticks->color
					* @type {string}
					* @example 
	                * &lt;div id="LinearGauge1"&gt;&lt;/div&gt; <br> 
	                * &lt;script&gt;
					* 	$("#LinearGauge1").ejLinearGauge({  scales:[{ticks:[{color:"Blue"}]}]});	
	                * &lt;/script&gt;				
					* @memberof ejLinearGauge
					* @instance
					*/	
                    color: null,
					/**		
					* Specifies the tick Style in the tick. See {@link TickType}
					* @default ej.datavisualization.LinearGauge.TickType.MajorInterval
                    * @alias ejLinearGauge#scales->ticks->type
					* @type {enum}
					* @example 
	                * &lt;div id="LinearGauge1"&gt;&lt;/div&gt; <br> 
	                * &lt;script&gt;
					* 	$("#LinearGauge1").ejLinearGauge({  scales:[{ticks:[{type:ej.datavisualization.LinearGauge.TickType.MajorInterval}]}]});		
	                * &lt;/script&gt;		
					* @memberof ejLinearGauge
					* @instance
					*/	
                    type: "majorinterval",
					/**		
					* Specifies the tick Placement in the tick. See {@link TickPlacement}
					* @default ej.datavisualization.LinearGauge.TickPlacement.Near
                    * @alias ejLinearGauge#scales->ticks->placement
					* @type {enum}
					* @example 
	                * &lt;div id="LinearGauge1"&gt;&lt;/div&gt; <br> 
	                * &lt;script&gt; 
					* 	$("#LinearGauge1").ejLinearGauge({  scales:[{ticks:[{placement:ej.datavisualization.LinearGauge.TickPlacement.Far}]}]});		
	                * &lt;/script&gt;				
					* @memberof ejLinearGauge
					* @instance
					*/	
                    placement: "near",
					/**		
					* Specifies the opacity in the tick.
					* @default 0
                    * @alias ejLinearGauge#scales->ticks->opacity
					* @type {number}
					* @example 
	                * &lt;div id="LinearGauge1"&gt;&lt;/div&gt; <br> 
	                * &lt;script&gt;
					* 	$("#LinearGauge1").ejLinearGauge({  scales:[{ticks:[{opacity:0.5}]}]});					 	
	                * &lt;/script&gt;	
					* @memberof ejLinearGauge
					* @instance
					*/	
                    opacity: 0,
					/**		
					* Specifies the tick Height in the tick.
					* @default 10
                    * @alias ejLinearGauge#scales->ticks->height
					* @type {number}
					* @example 
	                * &lt;div id="LinearGauge1"&gt;&lt;/div&gt; <br> 
	                * &lt;script&gt;
					* 	$("#LinearGauge1").ejLinearGauge({  scales:[{ticks:[{height:8}]}]});		
	                * &lt;/script&gt;	
					* @memberof ejLinearGauge
					* @instance
					*/	
                    height: 10,
					/**		
					* Specifies the tick Width in the tick.
					* @default 3
                    * @alias ejLinearGauge#scales->ticks->width
					* @type {number}
					* @example 
	                * &lt;div id="LinearGauge1"&gt;&lt;/div&gt; <br> 
	                * &lt;script&gt;
					* 	$("#LinearGauge1").ejLinearGauge({  scales:[{ticks:[{width:4}]}]});			
	                * &lt;/script&gt;			
					* @memberof ejLinearGauge
					* @instance
					*/	
                    width: 3
                },
                {              
                    distanceFromScale: {
                        x: 0,
                        y:0
                    },
                    angle: 0,
                    color: null,
                    type: "minorinterval",
                    placement: "near",
                    opacity: 0,
                    height: 5,
                    width: 2
                }],
                /**		
					* Specifies the ranges in the tick.
					* @default Array
                    * @alias ejLinearGauge#scales->ranges
					* @type {Array}
					* @example 
	                * &lt;div id="LinearGauge1"&gt;&lt;/div&gt; <br> 
	                * &lt;script&gt;
					* 	$("#LinearGauge1").ejLinearGauge({  scales:[{ranges:[{endWidth:4}]}]});		
	                * &lt;/script&gt;					
					* @memberof ejLinearGauge
					* @instance
					*/
                ranges: [{
                    /**		
					* Specifies the endWidth in the ranges.
					* @default 10
                    * @alias ejLinearGauge#scales->ranges->endWidth
					* @type {number}
					* @example 
	                * &lt;div id="LinearGauge1"&gt;&lt;/div&gt; <br> 
	                * &lt;script&gt;
					* $("#LinearGauge1").ejLinearGauge({ scales: [{ showBarPointers: false, width: 3, length: 310, showRanges: true,ranges: [{ startWidth: 10, endWidth: 20, endValue: 60, startValue: 0, backgroundColor: "#E94649" }] }] });	
	                * &lt;/script&gt;					 
					* @memberof ejLinearGauge
					* @instance
                    * @remarks
                    * for reflecting the customization in range, showRanges must be true.
					*/
                    endWidth: 10,
                    /**		
					* Specifies the range Position in the ranges. See {@link RangePlacement}
					* @default ej.datavisualization.LinearGauge.RangePlacement.Center
                    * @alias ejLinearGauge#scales->ranges->placement
					* @type {enum}
					* @example 
	                * &lt;div id="LinearGauge1"&gt;&lt;/div&gt; <br> 
	                * &lt;script&gt;
					* 	$("#LinearGauge1").ejLinearGauge({ scales: [{ showBarPointers: false, width: 3, length: 310, showRanges: true,ranges: [{ startWidth: 10, endWidth: 10, endValue: 60, startValue: 0, backgroundColor: "#E94649",placement: "center" }] }] });
	                * &lt;/script&gt;	
					* @memberof ejLinearGauge
					* @instance
                    * @remarks
                    * for reflecting the customization in range, showRanges must be true.
					*/
                    placement: "center",
					/**		
					* Specifies the startWidth in the ranges.
					* @default 10
                    * @alias ejLinearGauge#scales->ranges->startWidth
					* @type {number}
					* @example 
	                * &lt;div id="LinearGauge1"&gt;&lt;/div&gt; <br> 
	                * &lt;script&gt;
					* $("#LinearGauge1").ejLinearGauge({ scales: [{ showBarPointers: false, width: 3, length: 310, showRanges: true,ranges: [{ startWidth: 20, endWidth: 10, endValue: 60, startValue: 0, backgroundColor: "#E94649" }] }] });		
	                * &lt;/script&gt;					 
					* @memberof ejLinearGauge
					* @instance
                    * @remarks
                    * for reflecting the customization in range, showRanges must be true.
					*/
                    startWidth: 10,
					/**		
					* Specifies the distanceFromScale in the ranges.
					* @default 0
                    * @alias ejLinearGauge#scales->ranges->distanceFromScale
					* @type {number}
					* @example 
	                * &lt;div id="LinearGauge1"&gt;&lt;/div&gt; <br> 
	                * &lt;script&gt;
					* $("#LinearGauge1").ejLinearGauge({ scales: [{ showBarPointers: false, width: 3, length: 310, showRanges: true,ranges: [{ startWidth: 10, endWidth: 10, endValue: 60, startValue: 0, backgroundColor: "#E94649",distanceFromScale: 10 }] }] });	
	                * &lt;/script&gt;				
					* @memberof ejLinearGauge
					* @instance
                    * @remarks
                    * for reflecting the customization in range, showRanges must be true.
					*/
                    distanceFromScale: 0,
					/**		
					* Specifies the endValue in the ranges.
					* @default 60
                    * @alias ejLinearGauge#scales->ranges->endValue
					* @type {number}
					* @example 
	                * &lt;div id="LinearGauge1"&gt;&lt;/div&gt; <br> 
	                * &lt;script&gt;
					* $("#LinearGauge1").ejLinearGauge({ scales: [{ showBarPointers: false, width: 3, length: 310, showRanges: true,ranges: [{ startWidth: 10, endWidth: 10, endValue: 60, startValue: 0, backgroundColor: "#E94649" }] }] });					
	                * &lt;/script&gt;		
					* @memberof ejLinearGauge
					* @instance
                    * @remarks
                    * for reflecting the customization in range, showRanges must be true.
					*/
                    endValue: 60,
					/**		
					* Specifies the startValue in the ranges.
					* @default 20
                    * @alias ejLinearGauge#scales->ranges->startValue
					* @type {number}
					* @example 
	                * &lt;div id="LinearGauge1"&gt;&lt;/div&gt; <br> 
	                * &lt;script&gt;
					* $("#LinearGauge1").ejLinearGauge({ scales: [{ showBarPointers: false, width: 3, length: 310, showRanges: true,ranges: [{ startWidth: 10, endWidth: 10, endValue: 60, startValue: 10, backgroundColor: "#E94649" }] }] });			
	                * &lt;/script&gt;				
					* @memberof ejLinearGauge
					* @instance
                    * @remarks
                    * for reflecting the customization in range, showRanges must be true.
					*/
                    startValue: 20,
					/**		
					* Specifies the range Gradient in the ranges.
					* @default null
                    * @alias ejLinearGauge#scales->ranges->gradients
					* @type {object}
					* @example 
	                * &lt;div id="LinearGauge1"&gt;&lt;/div&gt; <br> 
	                * &lt;script&gt; 
					* $("#LinearGauge1").ejLinearGauge({ scales: [{ showBarPointers: false, width: 3, length: 310, showRanges: true,ranges: [{ startWidth: 10, endWidth: 10, endValue: 60, startValue: 0, backgroundColor: "#E94649",gradients:{ colorInfo:[{colorStop : 0, color:"#FFFFFF"},{colorStop : 1, color:"#AAAAAA"}] } }] }] });	
	                * &lt;/script&gt;					
					* @memberof ejLinearGauge
					* @instance
                    * @remarks
                    * for reflecting the customization in range, showRanges must be true.
					*/
                    gradients: null,
					/**		
					* Specifies the backgroundColor in the ranges.
					* @default null
                    * @alias ejLinearGauge#scales->ranges->backgroundColor
					* @type {string}
					* @example 
	                * &lt;div id="LinearGauge1"&gt;&lt;/div&gt; <br> 
	                * &lt;script&gt; 
					* $("#LinearGauge1").ejLinearGauge({ scales: [{ showBarPointers: false, width: 3, length: 310, showRanges: true,ranges: [{ startWidth: 10, endWidth: 10, endValue: 60, startValue: 0, backgroundColor: "Green" }] }] });			
	                * &lt;/script&gt;					
					* @memberof ejLinearGauge
					* @instance
                    * @remarks
                    * for reflecting the customization in range, showRanges must be true.
					*/
                    backgroundColor: null,
					/**		
					* Specifies the border in the ranges.
					* @default Object
                    * @alias ejLinearGauge#scales->ranges->border
					* @type {Object}
					* @example 
	                * &lt;div id="LinearGauge1"&gt;&lt;/div&gt; <br> 
	                * &lt;script&gt;
					* $("#LinearGauge1").ejLinearGauge({ scales: [{ showBarPointers: false, width: 3, length: 310, showRanges: true,ranges: [{ startWidth: 10, endWidth: 10, endValue: 60, startValue: 0, backgroundColor: "#E94649", border:{color: "Green", width:1.5} }] }] });		
	                * &lt;/script&gt;				
					* @memberof ejLinearGauge
					* @instance
                    * @remarks
                    * for reflecting the customization in range, showRanges must be true.
					*/
                    border: {
                        /**		
					    * Specifies the border color in the ranges.
					    * @default null
                        * @alias ejLinearGauge#scales->ranges->border->color
					    * @type {string}
					    * @example 
	                    * &lt;div id="LinearGauge1"&gt;&lt;/div&gt; <br> 
	                    * &lt;script&gt;
					    * $("#LinearGauge1").ejLinearGauge({ scales: [{ showBarPointers: false, width: 3, length: 310, showRanges: true,ranges: [{ startWidth: 10, endWidth: 10, endValue: 60, startValue: 0, backgroundColor: "#E94649", border:{color: "Green"} }] }] });		
	                    * &lt;/script&gt;				
					    * @memberof ejLinearGauge
    					* @instance
                        * @remarks
                        * for reflecting the customization in range, showRanges must be true.
					    */
                        color: null,
                        /**		
					    * Specifies the border width in the ranges.
					    * @default 1.5
                        * @alias ejLinearGauge#scales->ranges->border->width
					    * @type {number}
					    * @example 
	                    * &lt;div id="LinearGauge1"&gt;&lt;/div&gt; <br> 
	                    * &lt;script&gt;
					    * $("#LinearGauge1").ejLinearGauge({ scales: [{ showBarPointers: false, width: 3, length: 310, showRanges: true,ranges: [{ startWidth: 10, endWidth: 10, endValue: 60, startValue: 0, backgroundColor: "#E94649", border:{width:1.5} }] }] });		
	                    * &lt;/script&gt;				
					    * @memberof ejLinearGauge
					    * @instance
                        * @remarks
                        * for reflecting the customization in range, showRanges must be true.
					    */
                        width:1.5
                    },
					/**		
					* Specifies the opacity in the ranges.
					* @default null
                    * @alias ejLinearGauge#scales->ranges->opacity
					* @type {number}
					* @example 
	                * &lt;div id="LinearGauge1"&gt;&lt;/div&gt; <br> 
	                * &lt;script&gt;
					* 	$("#LinearGauge1").ejLinearGauge({ scales: [{ showBarPointers: false, width: 3, length: 310, showRanges: true,ranges: [{ startWidth: 10, endWidth: 10, endValue: 60, startValue: 0, backgroundColor: "#E94649",opacity: 0.3 }] }] });				
	                * &lt;/script&gt;			
					* @memberof ejLinearGauge
					* @instance
                    * @remarks
                    * for reflecting the customization in range, showRanges must be true.
					*/
                    opacity: null
                }],
					/**		
					* Specifies the labels.
					* @default Array
                    * @alias ejLinearGauge#scales->labels
					* @type {Array}
					* @example 
	                * &lt;div id="LinearGauge1"&gt;&lt;/div&gt; <br> 
	                * &lt;script&gt;
					* 	$("#LinearGauge1").ejLinearGauge({  scales:[{labels:[{distanceFromScale:{y:1}}]}]});			
	                * &lt;/script&gt;	
					* @memberof ejLinearGauge
					* @instance
					*/
                labels: [{
                    /**		
					* Specifies the DistanceFromScale of labels.
					* @default object
                    * @alias ejLinearGauge#scales->labels->distanceFromScale
					* @type {object}
					* @example 
	                * &lt;div id="LinearGauge1"&gt;&lt;/div&gt; <br> 
	                * &lt;script&gt;
					* 	$("#LinearGauge1").ejLinearGauge({  scales:[{labels:[{distanceFromScale:{x:10, y:10}}]}]});		
	                * &lt;/script&gt;				
					* @memberof ejLinearGauge
					* @instance
					*/
                    distanceFromScale: {
                        /**		
					    * Specifies the xDistanceFromScale of labels.
					    * @default -10
                        * @alias ejLinearGauge#scales->labels->distanceFromScale->x
					    * @type {number}
					    * @example 
	                    * &lt;div id="LinearGauge1"&gt;&lt;/div&gt; <br> 
	                    * &lt;script&gt;
					    * 	$("#LinearGauge1").ejLinearGauge({  scales:[{labels:[{distanceFromScale:{x:10}}]}]});		
	                    * &lt;/script&gt;				
					    * @memberof ejLinearGauge
					    * @instance
					    */  
                        x: -10,
                        /**		
					    * Specifies the yDistanceFromScale of labels.
					    * @default 0
                        * @alias ejLinearGauge#scales->labels->distanceFromScale->y
					    * @type {number}
					    * @example 
	                    * &lt;div id="LinearGauge1"&gt;&lt;/div&gt; <br> 
	                    * &lt;script&gt;  
					    * 	$("#LinearGauge1").ejLinearGauge({  scales:[{labels:[{distanceFromScale:{y:20}}]}]});		
	                    * &lt;/script&gt;	
					    * @memberof ejLinearGauge
					    * @instance
					    */  
                        y:0
                    },
                    /**		
					* Specifies the angle of labels.
					* @default 0
                    * @alias ejLinearGauge#scales->labels->angle
					* @type {number}
					* @example 
	                * &lt;div id="LinearGauge1"&gt;&lt;/div&gt; <br> 
	                * &lt;script&gt;
					* 	$("#LinearGauge1").ejLinearGauge({  scales:[{labels:[{angle:30}]}]});				
	                * &lt;/script&gt;	
					* @memberof ejLinearGauge
					* @instance
					*/
                    angle: 0,
					/**		
					* Specifies the font of labels.
					* @default Object
                    * @alias ejLinearGauge#scales->labels->font
					* @type {object}
					* @example 
	                * &lt;div id="LinearGauge1"&gt;&lt;/div&gt; <br> 
	                * &lt;script&gt;
					* 	$("#LinearGauge1").ejLinearGauge({  scales:[{labels:[{font:{size: "11px"}}]}]});		
	                * &lt;/script&gt;		
					* @memberof ejLinearGauge
					* @instance
					*/
                    font: { 
					/**		
					* Specifies the size of font.
					* @default "11px"
                    * @alias ejLinearGauge#scales->labels->font->size
					* @type {string}
					* @example 
	                * &lt;div id="LinearGauge1"&gt;&lt;/div&gt; <br> 
	                * &lt;script&gt;
					* 	$("#LinearGauge1").ejLinearGauge({  scales:[{labels:[{font:{size: "18px"}}]}]});		
	                * &lt;/script&gt;			
					* @memberof ejLinearGauge
					* @instance
					*/
					size: "11px", 
					/**		
					* Specifies the fontFamily of font.
					* @default "Arial"
                    * @alias ejLinearGauge#scales->labels->font->fontFamily
					* @type {string}
					* @example 
	                * &lt;div id="LinearGauge1"&gt;&lt;/div&gt; <br> 
	                * &lt;script&gt;
					* 	$("#LinearGauge1").ejLinearGauge({  scales:[{labels:[{font:{fontFamily: "Segoe UI"}}]}]});		
	                * &lt;/script&gt;				
					* @memberof ejLinearGauge
					* @instance
					*/
					fontFamily: "Arial", 
					/**		
					* Specifies the fontStyle of font.See {@link FontStyle}
					* @default ej.datavisualization.LinearGauge.FontStyle.Bold
                    * @alias ejLinearGauge#scales->labels->font->fontStyle
					* @type {enum}
					* @example 
	                * &lt;div id="LinearGauge1"&gt;&lt;/div&gt; <br> 
	                * &lt;script&gt;
					* 	$("#LinearGauge1").ejLinearGauge({  scales:[{labels:[{font:{fontStyle: ej.datavisualization.LinearGauge.FontStyle.Normal}}]}]});		
	                * &lt;/script&gt;			
					* @memberof ejLinearGauge
					* @instance
					*/
					fontStyle: "bold" },
					/**		
					* Specifies the textColor of font.
					* @default null
                    * @alias ejLinearGauge#scales->labels->textColor
					* @type {string}
					* @example 
	                * &lt;div id="LinearGauge1"&gt;&lt;/div&gt; <br> 
	                * &lt;script&gt; 
					* $("#LinearGauge1").ejLinearGauge({  scales:[{labels:[{textColor: "green"}]}]}); 	
	                * &lt;/script&gt;	
					* @memberof ejLinearGauge
					* @instance
					*/
                    textColor: null,
					/**		
					* Specifies the opacity of label.
					* @default 0
                    * @alias ejLinearGauge#scales->labels->opacity
					* @type {number}
					* @example 
	                * &lt;div id="LinearGauge1"&gt;&lt;/div&gt; <br> 
	                * &lt;script&gt;
					* 	$("#LinearGauge1").ejLinearGauge({  scales:[{labels:[{opacity: 0.5}]}]});		
	                * &lt;/script&gt;				
					* @memberof ejLinearGauge
					* @instance
					*/
                    opacity: 0,
					/**		
					* Specifies the label Style of label. See {@link LabelType}
					* @default ej.datavisualization.LinearGauge.LabelType.Major
                    * @alias ejLinearGauge#scales->labels->type
					* @type {enum}
					* @example 
	                * &lt;div id="LinearGauge1"&gt;&lt;/div&gt; <br> 
	                * &lt;script&gt;
					* 	$("#LinearGauge1").ejLinearGauge({  scales:[{labels:[{type: ej.datavisualization.LinearGauge.LabelType.Major}]}]});	
	                * &lt;/script&gt;				
					* @memberof ejLinearGauge
					* @instance
					*/
                    type: "major",
					/**		
					* Specifies the label Placement of label. See {@link LabelPlacement}
					* @default ej.datavisualization.LinearGauge.LabelPlacement.Near
                    * @alias ejLinearGauge#scales->labels->placement
					* @type {enum}
					* @example 
	                * &lt;div id="LinearGauge1"&gt;&lt;/div&gt; <br> 
	                * &lt;script&gt; 
					* 	$("#LinearGauge1").ejLinearGauge({  scales:[{labels:[{placement: ej.datavisualization.LinearGauge.LabelPlacement.Far}]}]});	
	                * &lt;/script&gt;				
					* @memberof ejLinearGauge
					* @instance
					*/
                    placement: "near",
					/**		
					* need to includeFirstValue.
					* @default true
                    * @alias ejLinearGauge#scales->labels->includeFirstValue
					* @type {boolean}
					* @example 
	                * &lt;div id="LinearGauge1"&gt;&lt;/div&gt; <br> 
	                * &lt;script&gt;  
					* 	$("#LinearGauge1").ejLinearGauge({  scales:[{labels:[{includeFirstValue: false}]}]});		
	                * &lt;/script&gt;			
					* @memberof ejLinearGauge
					* @instance
					*/
                    includeFirstValue: true,
					/**		
					* Specifies the unitText of label.  
					* @default ""
                    * @alias ejLinearGauge#scales->labels->unitText
					* @type {string}
					* @example 
	                * &lt;div id="LinearGauge1"&gt;&lt;/div&gt; <br> 
	                * &lt;script&gt;
					* 	$("#LinearGauge1").ejLinearGauge({  scales:[{labels:[{unitText: "F"}]}]});	
	                * &lt;/script&gt;					
					* @instance
					*/
					unitText: "",
					/**		
					* Specifies the unitText Position of label.See {@link UnitTextPlacement}
					* @default ej.datavisualization.LinearGauge.UnitTextPlacement.Back
                    * @alias ejLinearGauge#scales->labels->unitTextPlacement
					* @type {enum}
					* @example 
	                * &lt;div id="LinearGauge1"&gt;&lt;/div&gt; <br> 
	                * &lt;script&gt;
					* 	$("#LinearGauge1").ejLinearGauge({  scales:[{labels:[{unitText: "F",unitTextPlacement: "front"}]}]});				
	                * &lt;/script&gt;		
					* @memberof ejLinearGauge
					* @instance
					*/
					unitTextPlacement: "back"//["front","back"]
                }],
				    /**		
					* Specifies the markerPointers
					* @default Array
                    * @alias ejLinearGauge#scales->markerPointers
					* @type {Array}
					* @example 
	                * &lt;div id="LinearGauge1"&gt;&lt;/div&gt; <br> 
	                * &lt;script&gt;
					* 	$("#LinearGauge1").ejLinearGauge({  scales:[{markerPointers:[{type: "triangle"}]}]});	
	                * &lt;/script&gt;				
					* @memberof ejLinearGauge
					* @instance
					*/
                markerPointers: [{
					/**		
					* Specifies the marker Style of marker pointerSee {@link MarkerType}
					* @default ej.datavisualization.LinearGauge.MarkerType.Triangle
                    * @alias ejLinearGauge#scales->markerPointers->type
					* @type {enum}
					* @example 
	                * &lt;div id="LinearGauge1"&gt;&lt;/div&gt; <br> 
	                * &lt;script&gt;
					* 	$("#LinearGauge1").ejLinearGauge({  scales:[{markerPointers:[{type: ej.datavisualization.LinearGauge.MarkerType.Rectangle}]}]});		
	                * &lt;/script&gt;			
					* @memberof ejLinearGauge
					* @instance
					*/
                    type: "triangle",
					/**		
					* Specifies the pointer Length of marker pointer
					* @default 30
                    * @alias ejLinearGauge#scales->markerPointers->length
					* @type {number}
					* @example 
	                * &lt;div id="LinearGauge1"&gt;&lt;/div&gt; <br> 
	                * &lt;script&gt;
					* 	$("#LinearGauge1").ejLinearGauge({  scales:[{markerPointers:[{length: 25}]}]});		
	                * &lt;/script&gt;			
					* @memberof ejLinearGauge
					* @instance
					*/
                    length: 30,
					/**		
					* Specifies the pointer Placement of marker pointer  See {@link PointerPlacement}
					* @default ej.datavisualization.LinearGauge.PointerPlacement.Far
                    * @alias ejLinearGauge#scales->markerPointers->placement
					* @type {enum}
					* @example 
	                * &lt;div id="LinearGauge1"&gt;&lt;/div&gt; <br> 
	                * &lt;script&gt;
					* 	$("#LinearGauge1").ejLinearGauge({  scales:[{markerPointers:[{placement: ej.datavisualization.LinearGauge.PointerPlacement.Near}]}]});		
	                * &lt;/script&gt;			
					* @memberof ejLinearGauge
					* @instance
					*/
                    placement: "far",
					/**		
					* Specifies the pointer Gradient of marker pointer
					* @default Object
                    * @alias ejLinearGauge#scales->markerPointers->gradients
					* @type {object}
					* @example 
	                * &lt;div id="LinearGauge1"&gt;&lt;/div&gt; <br> 
	                * &lt;script&gt; 
					* 	$("#LinearGauge1").ejLinearGauge({  scales:[{markerPointers:[{gradients: { colorInfo:[{ colorStop : 0, color:"#FFFFFF"},{colorStop : 1, color:"#AAAAAA"}] }}]}]});		
	                * &lt;/script&gt;			
					* @memberof ejLinearGauge
					* @instance
					*/
                    gradients: null,
					/**		
					* Specifies the distanceFromScale of marker pointer
					* @default 0
                    * @alias ejLinearGauge#scales->markerPointers->distanceFromScale
					* @type {number}
					* @example 
	                * &lt;div id="LinearGauge1"&gt;&lt;/div&gt; <br> 
	                * &lt;script&gt; 
					* 	$("#LinearGauge1").ejLinearGauge({  scales:[{markerPointers:[{distanceFromScale: 2}]}]});		
	                * &lt;/script&gt;			
					* @memberof ejLinearGauge
					* @instance
					*/
                    distanceFromScale: 0,
					/**		
					* Specifies the pointer Width of marker pointer
					* @default 30
                    * @alias ejLinearGauge#scales->markerPointers->width
					* @type {number}
					* @example 
	                * &lt;div id="LinearGauge1"&gt;&lt;/div&gt; <br> 
	                * &lt;script&gt;
					* 	$("#LinearGauge1").ejLinearGauge({  scales:[{markerPointers:[{width: 25}]}]});		
	                * &lt;/script&gt;				
					* @memberof ejLinearGauge
					* @instance
					*/
                    width: 30,
					/**		
					* Specifies the value of marker pointer
					* @default null
                    * @alias ejLinearGauge#scales->markerPointers->value
					* @type {number}
					* @example 
	                * &lt;div id="LinearGauge1"&gt;&lt;/div&gt; <br> 
	                * &lt;script&gt;
					* 	$("#LinearGauge1").ejLinearGauge({  scales:[{markerPointers:[{value: 25}]}]});		
	                * &lt;/script&gt;				
					* @memberof ejLinearGauge
					* @instance
					*/
                    value: null,
					/**		
					* Specifies the backgroundColor of marker pointer
					* @default null
                    * @alias ejLinearGauge#scales->markerPointers->backgroundColor
					* @type {string}
					* @example 
	                * &lt;div id="LinearGauge1"&gt;&lt;/div&gt; <br> 
	                * &lt;script&gt; 
					* 	$("#LinearGauge1").ejLinearGauge({  scales:[{markerPointers:[{backgroundColor: "blue"}]}]});		
	                * &lt;/script&gt;	
					* @memberof ejLinearGauge
					* @instance
					*/
                    backgroundColor: null,
					/**		
					* Specifies the border of marker pointer
					* @default Object
                    * @alias ejLinearGauge#scales->markerPointers->border
					* @type {object}
					* @example 
	                * &lt;div id="LinearGauge1"&gt;&lt;/div&gt; <br> 
	                * &lt;script&gt;
					* 	$("#LinearGauge1").ejLinearGauge({  scales:[{markerPointers:[{border:{color: "blue", width: 1.5}}]}]});		
	                * &lt;/script&gt;		
					* @memberof ejLinearGauge
					* @instance
					*/
                    border: {
                        /**		
					    * Specifies the border color of marker pointer
					    * @default null
                        * @alias ejLinearGauge#scales->markerPointers->border->color
					    * @type {string}
					    * @example 
    	                * &lt;div id="LinearGauge1"&gt;&lt;/div&gt; <br> 
	                    * &lt;script&gt;
					    * 	$("#LinearGauge1").ejLinearGauge({  scales:[{markerPointers:[{border:{color: "blue", width: 1.5}}]}]});		
    	                * &lt;/script&gt;		
					    * @memberof ejLinearGauge
					    * @instance
					    */
                        color: null,
                        /**		
					    * Specifies the border of marker pointer
					    * @default number
                        * @alias ejLinearGauge#scales->markerPointers->border->width
    					* @type {number}
					    * @example 
	                    * &lt;div id="LinearGauge1"&gt;&lt;/div&gt; <br> 
	                    * &lt;script&gt;
    					* 	$("#LinearGauge1").ejLinearGauge({  scales:[{markerPointers:[{border:{color: "blue", width: 1.5}}]}]});		
	                    * &lt;/script&gt;		
					    * @memberof ejLinearGauge
					    * @instance
					    */
                        width:1.5
                    },
					/**		
					* Specifies the opacity of marker pointer
					* @default 1
                    * @alias ejLinearGauge#scales->markerPointers->opacity
					* @type {number}
					* @example 
	                * &lt;div id="LinearGauge1"&gt;&lt;/div&gt; <br> 
	                * &lt;script&gt;
					* 	$("#LinearGauge1").ejLinearGauge({  scales:[{markerPointers:[{opacity: 0.5}]}]});		
	                * &lt;/script&gt;			
					* @memberof ejLinearGauge
					* @instance
					*/
                    opacity: 1
                }],
					/**		
					* Specifies the scaleBar Gradient of bar pointer
					* @default Array
                    * @alias ejLinearGauge#scales->barPointers
					* @type {Array}
					* @example 
	                * &lt;div id="LinearGauge1"&gt;&lt;/div&gt; <br> 
	                * &lt;script&gt;
					* 	$("#LinearGauge1").ejLinearGauge({  scales:[{barPointers:[{}]}]});				
	                * &lt;/script&gt;			 
					* @memberof ejLinearGauge
					* @instance
					*/
                barPointers: [{
				    /**		
					* Specifies the scaleBar Gradient of bar pointer
					* @default null
                    * @alias ejLinearGauge#scales->barPointers->gradients
					* @type {object}
					* @example 
	                * &lt;div id="LinearGauge1"&gt;&lt;/div&gt; <br> 
	                * &lt;script&gt;
					* 	$("#LinearGauge1").ejLinearGauge({  scales:[{barPointers:[{value:50, gradients: { colorInfo:[{ colorStop : 0, color:"#FFFFFF"},{colorStop : 1, color:"#AAAAAA"}] }}]}]});		
	                * &lt;/script&gt;	
					* @memberof ejLinearGauge
					* @instance
					*/
                    gradients: null,
					/**		
					* Specifies the distanceFromScale of bar pointer
					* @default 0
                    * @alias ejLinearGauge#scales->barPointers->distanceFromScale
					* @type {number}
					* @example 
	                * &lt;div id="LinearGauge1"&gt;&lt;/div&gt; <br> 
	                * &lt;script&gt;
					* 	$("#LinearGauge1").ejLinearGauge({  scales:[{barPointers:[{value:50, distanceFromScale: 20}]}]});		
	                * &lt;/script&gt;				
					* @memberof ejLinearGauge
					* @instance
					*/
                    distanceFromScale: 0,
					/**		
					* Specifies the pointer Width of bar pointer
					* @default width=30
                    * @alias ejLinearGauge#scales->barPointers->width
					* @type {number}
					* @example 
	                * &lt;div id="LinearGauge1"&gt;&lt;/div&gt; <br> 
	                * &lt;script&gt;
					* 	$("#LinearGauge1").ejLinearGauge({  scales:[{barPointers:[{value:50, width: 25}]}]});		
	                * &lt;/script&gt;			
					* @memberof ejLinearGauge
					* @instance
					*/
                    width: 30,
					/**		
					* Specifies the value of bar pointer
					* @default null
                    * @alias ejLinearGauge#scales->barPointers->value
					* @type {number}
					* @example 
	                * &lt;div id="LinearGauge1"&gt;&lt;/div&gt; <br> 
	                * &lt;script&gt;
					* 	$("#LinearGauge1").ejLinearGauge({  scales:[{barPointers:[{value: 100}]}]});		
	                * &lt;/script&gt;				
					* @memberof ejLinearGauge
					* @instance
					*/
                    value: null,
					/**		
					* Specifies the backgroundColor of bar pointer
					* @default null
                    * @alias ejLinearGauge#scales->barPointers->backgroundColor
					* @type {string}
					* @example 
	                * &lt;div id="LinearGauge1"&gt;&lt;/div&gt; <br> 
	                * &lt;script&gt; 
					* 	$("#LinearGauge1").ejLinearGauge({  scales:[{barPointers:[{value:50, backgroundColor: "red"}]}]});		
	                * &lt;/script&gt;			
					* @memberof ejLinearGauge
					* @instance
					*/
                    backgroundColor: null,
                    /**		
					* Specifies the border of bar pointer
					* @default Object
                    * @alias ejLinearGauge#scales->barPointers->border
					* @type {object}
					* @example 
	                * &lt;div id="LinearGauge1"&gt;&lt;/div&gt; <br> 
	                * &lt;script&gt;
					* 	$("#LinearGauge1").ejLinearGauge({  scales:[{barPointers:[{value:50, border:{color: "red", width:1.5}}]}]});	
	                * &lt;/script&gt;					
					* @memberof ejLinearGauge
					* @instance
					*/
                    border: {
                        /**		
					    * Specifies the border Color of bar pointer
					    * @default null
                        * @alias ejLinearGauge#scales->barPointers->border->color
					    * @type {string}
					    * @example 
	                    * &lt;div id="LinearGauge1"&gt;&lt;/div&gt; <br> 
	                    * &lt;script&gt;
					    * 	$("#LinearGauge1").ejLinearGauge({  scales:[{barPointers:[{value:50, border:{color: "red"}}]}]});	
	                    * &lt;/script&gt;					
					    * @memberof ejLinearGauge
					    * @instance
					    */
                        color: null,
                        /**		
					    * Specifies the border Width of bar pointer
					    * @default 1.5
                        * @alias ejLinearGauge#scales->barPointers->border->width
					    * @type {number}
					    * @example 
	                    * &lt;div id="LinearGauge1"&gt;&lt;/div&gt; <br> 
	                    * &lt;script&gt;
					    * 	$("#LinearGauge1").ejLinearGauge({  scales:[{barPointers:[{value:50, border:{width: 2}}]}]});		
	                    * &lt;/script&gt;					
					    * @memberof ejLinearGauge
					    * @instance
					    */
                        width:1.5
                    },
					/**		
					* Specifies the opacity of bar pointer
					* @default 1
                    * @alias ejLinearGauge#scales->barPointers->opacity
					* @type {number}
					* @example 
	                * &lt;div id="LinearGauge1"&gt;&lt;/div&gt; <br> 
	                * &lt;script&gt;
					* 	$("#LinearGauge1").ejLinearGauge({  scales:[{barPointers:[{value:50, opacity: 0.5}]}]}); 	
	                * &lt;/script&gt;	
					* @memberof ejLinearGauge
					* @instance
					*/
                    opacity: 1
                }],
					/**		
					* Specifies the indicator
					* @default Array
                    * @alias ejLinearGauge#scales->indicators
					* @type {Array}
					* @example 
	                * &lt;div id="LinearGauge1"&gt;&lt;/div&gt; <br> 
	                * &lt;script&gt;
					* 	$("#LinearGauge1").ejLinearGauge({scales: [{showBarPointers: false, showIndicators: true, length: 310, indicators: [{
                    *    font: {size: "11px",fontFamily: "Arial",fontStyle: "bold"},height: 30,
                    *    type: "rectangle",width: 30,position: {x: 60,y: 70},textLocation: {x: 0,y: 0},
                    *    stateRanges: [{endValue: 60,startValue: 50,backgroundColor: "Red",borderColor: "Green",textColor: null}],
                    *    value: 0, backgroundColor: "Red", border:{color: "Red", width: 1.5}, opacity: NaN}]}]});		
	                * &lt;/script&gt;			
					* @memberof ejLinearGauge
                    * @remarks
                    * for reflecting the customization in Indicators, showIndicators must be true.
					* @instance
					*/	
                indicators: [{
					/**		
					* Specifies the font of bar indicators
					* @default Object
                    * @alias ejLinearGauge#scales->indicators->font
					* @type {object}
					* @example 
	                * &lt;div id="LinearGauge1"&gt;&lt;/div&gt; <br> 
	                * &lt;script&gt;
                    *$("#LinearGauge1").ejLinearGauge({
                    *   value: 50, scales: [{showBarPointers: true, showIndicators: true, length: 310, indicators: [{
                    *   font: { size: "11px", fontFamily: "Arial", fontStyle: "bold" }, height: 30,
                    *   type: "text", width: 30, position: { x: 60, y: 100 }, textLocation: { x: 50, y: 100 },
                    *   stateRanges: [{ endValue: 60, startValue: 50, textColor: "Green", text: "Linear" }],opacity: 0.5}]}]});
	                * &lt;/script&gt;				
					* @memberof ejLinearGauge
                    * @remarks
                    * for reflecting the customization in Indicator, showIndicators must be true.
					* @instance
					*/
                    font: { 
					/**		
					* Specifies the size of font in bar indicators
					* @default "11px"
                    * @alias ejLinearGauge#scales->indicators->font->size
					* @type {string}
					* @example 
	                * &lt;div id="LinearGauge1"&gt;&lt;/div&gt; <br> 
	                * &lt;script&gt;
                    *$("#LinearGauge1").ejLinearGauge({
                    *   value: 50, scales: [{showBarPointers: true, showIndicators: true, length: 310, indicators: [{
                    *   font: { size: "20px", fontFamily: "Arial", fontStyle: "bold" }, height: 30,
                    *   type: "text", width: 30, position: { x: 60, y: 100 }, textLocation: { x: 50, y: 100 },
                    *   stateRanges: [{ endValue: 60, startValue: 50, textColor: "Green", text: "Linear" }],opacity: 0.5}]}]});	
	                * &lt;/script&gt;				
					* @memberof ejLinearGauge
					* @instance
                    * @remarks
                    * for reflecting the customization in Indicator, showIndicators must be true.
					*/
					size: "11px", 
					/**		
					* Specifies the fontFamily of font in bar indicators
					* @default "Arial"
                    * @alias ejLinearGauge#scales->indicators->font->fontFamily
					* @type {string}
					* @example 
	                * &lt;div id="LinearGauge1"&gt;&lt;/div&gt; <br> 
	                * &lt;script&gt; 	
                    *$("#LinearGauge1").ejLinearGauge({
                    *   value: 50, scales: [{showBarPointers: true, showIndicators: true, length: 310, indicators: [{
                    *   font: { size: "11px", fontFamily: "Segoe UI", fontStyle: "bold" }, height: 30,
                    *   type: "text", width: 30, position: { x: 60, y: 100 }, textLocation: { x: 50, y: 100 },
                    *   stateRanges: [{ endValue: 60, startValue: 50, textColor: "Green", text: "Linear" }],opacity: 0.5}]}]});			
	                * &lt;/script&gt;	
					* @memberof ejLinearGauge
					* @instance
                    * @remarks
                    * for reflecting the customization in Indicator, showIndicators must be true.
					*/
					fontFamily: "Arial",
					/**		
					* Specifies the fontStyle of font in bar indicators.  See {@link FontStyle}
					* @default ej.datavisualization.LinearGauge.FontStyle.Bold
                    * @alias ejLinearGauge#scales->indicators->font->fontStyle
					* @type {enum}
					* @example 
	                * &lt;div id="LinearGauge1"&gt;&lt;/div&gt; <br> 
	                * &lt;script&gt;
                    *$("#LinearGauge1").ejLinearGauge({
                    *   value: 50, scales: [{showBarPointers: true, showIndicators: true, length: 310, indicators: [{
                    *   font: { size: "11px", fontFamily: "Arial", fontStyle: "Normal" }, height: 30,
                    *   type: "text", width: 30, position: { x: 60, y: 100 }, textLocation: { x: 50, y: 100 },
                    *   stateRanges: [{ endValue: 60, startValue: 50, textColor: "Green", text: "Linear" }],opacity: 0.5}]}]});	
	                * &lt;/script&gt;			
					* @memberof ejLinearGauge
					* @instance
                    * @remarks
                    * for reflecting the customization in Indicator, showIndicators must be true.
					*/					
					fontStyle: "bold" },
					/**		
					* Specifies the indicator Height of bar indicators
					* @default 30
                    * @alias ejLinearGauge#scales->indicators->height
					* @type {number}
					* @example 
	                * &lt;div id="LinearGauge1"&gt;&lt;/div&gt; <br> 
	                * &lt;script&gt; 
                    * $("#LinearGauge1").ejLinearGauge({
                    *    value: 50, scales: [{showBarPointers: true, showIndicators: true, length: 310, indicators: [{
                    *    type: "rectangle",height: 50,
                    *    position: { x: 0,y: 0},
                    *    stateRanges: [{endValue: 60,startValue: 40,backgroundColor: "Green",borderColor: "Red",}],
                    *    border:{width: 1.5}}]}]});	
	                * &lt;/script&gt;					 
					* @memberof ejLinearGauge
					* @instance
                    * @remarks
                    * for reflecting the customization in Indicator, showIndicators must be true.
					*/					
                    height: 30,
					/**		
					* Specifies the indicator Style of font in bar indicators
					* @default ej.datavisualization.LinearGauge.IndicatorType.Rectangle
                    * @alias ejLinearGauge#scales->indicators->type
					* @type {enum}
					* @example 
	                * &lt;div id="LinearGauge1"&gt;&lt;/div&gt; <br> 
	                * &lt;script&gt;
					* $("#LinearGauge1").ejLinearGauge({
                    *    value: 50, scales: [{showBarPointers: true, showIndicators: true, length: 310, indicators: [{
                    *    height: 30,type: "rectangle",width: 30,
                    *    position: { x: 0,y: 0},
                    *    stateRanges: [{endValue: 60,startValue: 40,backgroundColor: "Green",borderColor: "Red",}],
                    *    border:{width: 1.5}}]}]});
	                * &lt;/script&gt;	
					* @memberof ejLinearGauge
					* @instance
                    * @remarks
                    * for reflecting the customization in Indicator, showIndicators must be true.
					*/	
                    type: "rectangle",
					/**		
					* Specifies the indicator Width in bar indicators
					* @default 30
                    * @alias ejLinearGauge#scales->indicators->width
					* @type {number}
					* @example 
	                * &lt;div id="LinearGauge1"&gt;&lt;/div&gt; <br> 
	                * &lt;script&gt;
					* $("#LinearGauge1").ejLinearGauge({
                    *    value: 50, scales: [{showBarPointers: true, showIndicators: true, length: 310, indicators: [{
                    *    type: "rectangle",width: 50,
                    *    position: { x: 0,y: 0},
                    *    stateRanges: [{endValue: 60,startValue: 40,backgroundColor: "Green",borderColor: "Red",}],
                    *    border:{width: 1.5}}]}]});
	                * &lt;/script&gt;				
					* @memberof ejLinearGauge
					* @instance
                    * @remarks
                    * for reflecting the customization in Indicator, showIndicators must be true.
					*/	
                    width: 30,
						/**		
					* Specifies the position in bar indicators
					* @default Object
                    * @alias ejLinearGauge#scales->indicators->position
					* @type {object}
					* @example 
	                * &lt;div id="LinearGauge1"&gt;&lt;/div&gt; <br> 
	                * &lt;script&gt;
					* $("#LinearGauge1").ejLinearGauge({
                    *    value: 50, scales: [{showBarPointers: true, showIndicators: true, length: 310, indicators: [{
                    *    height: 30,type: "rectangle",width: 30,
                    *    position: { x: 0,y: 0},
                    *    stateRanges: [{endValue: 60,startValue: 40,backgroundColor: "Green",borderColor: "Red",}],
                    *    border:{width: 1.5}}]}]});		
	                * &lt;/script&gt;		
					* @memberof ejLinearGauge
					* @instance
                    * @remarks
                    * for reflecting the customization in Indicator, showIndicators must be true.
					*/	
                    position: { 
					/**		
					* Specifies the x position in bar indicators
					* @default 0
                    * @alias ejLinearGauge#scales->indicators->position->x
					* @type {number}
					* @example 
	                * &lt;div id="LinearGauge1"&gt;&lt;/div&gt; <br> 
	                * &lt;script&gt;
					* $("#LinearGauge1").ejLinearGauge({
                    *    value: 50, scales: [{showBarPointers: true, showIndicators: true, length: 310, indicators: [{
                    *    height: 30,type: "rectangle",width: 30,
                    *    position: { x: 20,y: 0},
                    *    stateRanges: [{endValue: 60,startValue: 40,backgroundColor: "Green",borderColor: "Red",}],
                    *    border:{width: 1.5}}]}]});		
	                * &lt;/script&gt;		
					* @memberof ejLinearGauge
					* @instance
                    * @remarks
                    * for reflecting the customization in Indicator, showIndicators must be true.
					*/	
					x: 0, 
					/**		
					* Specifies the y position in bar indicators
					* @default 0
                    * @alias ejLinearGauge#scales->indicators->position->y
					* @type {number}
					* @example 
	                * &lt;div id="LinearGauge1"&gt;&lt;/div&gt; <br> 
	                * &lt;script&gt;
					* $("#LinearGauge1").ejLinearGauge({
                    *    value: 50, scales: [{showBarPointers: true, showIndicators: true, length: 310, indicators: [{
                    *    height: 30,type: "rectangle",width: 30,
                    *    position: { x: 0,y: 100},
                    *    stateRanges: [{endValue: 60,startValue: 40,backgroundColor: "Green",borderColor: "Red",}],
                    *    border:{width: 1.5}}]}]});
	                * &lt;/script&gt;			
					* @memberof ejLinearGauge
					* @instance
                    * @remarks
                    * for reflecting the customization in Indicator, showIndicators must be true.
					*/	
					y: 0 },
					/**		
					* Specifies the textLocation in bar indicators
					* @default Object
                    * @alias ejLinearGauge#scales->indicators->textLocation
					* @type {object}
					* @example 
	                * &lt;div id="LinearGauge1"&gt;&lt;/div&gt; <br> 
	                * &lt;script&gt; 
                    * $("#LinearGauge1").ejLinearGauge({
                    *   value: 50, scales: [{showBarPointers: true, showIndicators: true, length: 310, indicators: [{
                    *   font: { size: "11px", fontFamily: "Arial", fontStyle: "Normal" }, height: 30,
                    *   type: "text", width: 30, position: { x: 60, y: 100 }, textLocation: { x: 50, y: 100 },
                    *   stateRanges: [{ endValue: 60, startValue: 50, textColor: "Green", text: "Linear" }],opacity: 0.5}]}]});	
	                * &lt;/script&gt;		
					* @memberof ejLinearGauge
					* @instance
                    * @remarks
                    * for reflecting the customization in Indicator, showIndicators must be true.
					*/	
                    textLocation: { 
					/**		
					* Specifies the textLocation position in bar indicators
					* @default 0
                    * @alias ejLinearGauge#scales->indicators->textLocation->x
					* @type {number}
					* @example 
	                * &lt;div id="LinearGauge1"&gt;&lt;/div&gt; <br> 
	                * &lt;script&gt;
					* $("#LinearGauge1").ejLinearGauge({
                    *   value: 50, scales: [{showBarPointers: true, showIndicators: true, length: 310, indicators: [{
                    *   font: { size: "11px", fontFamily: "Arial", fontStyle: "Normal" }, height: 30,
                    *   type: "text", width: 30, position: { x: 60, y: 100 }, textLocation: { x: 50, y: 0 },
                    *   stateRanges: [{ endValue: 60, startValue: 50, textColor: "Green", text: "Linear" }],opacity: 0.5}]}]});
	                * &lt;/script&gt;		
					* @memberof ejLinearGauge
					* @instance
                    * @remarks
                    * for reflecting the customization in Indicator, showIndicators must be true.
					*/	
					x: 0, 
					/**		
					* Specifies the Y position in bar indicators
					* @default 0
                    * @alias ejLinearGauge#scales->indicators->textLocation->y
					* @type {number}
					* @example 
	                * &lt;div id="LinearGauge1"&gt;&lt;/div&gt; <br> 
	                * &lt;script&gt;
					* $("#LinearGauge1").ejLinearGauge({
                    *   value: 50, scales: [{showBarPointers: true, showIndicators: true, length: 310, indicators: [{
                    *   font: { size: "11px", fontFamily: "Arial", fontStyle: "Normal" }, height: 30,
                    *   type: "text", width: 30, position: { x: 60, y: 100 }, textLocation: { x: 0, y: 100 },
                    *   stateRanges: [{ endValue: 60, startValue: 50, textColor: "Green", text: "Linear" }],opacity: 0.5}]}]});
	                * &lt;/script&gt;			
					* @memberof ejLinearGauge
					* @instance
                    * @remarks
                    * for reflecting the customization in Indicator, showIndicators must be true.
					*/	
					y: 0 },
					/**		
					* Specifies the opacity in bar indicators
					* @default Array
                    * @alias ejLinearGauge#scales->indicators->stateRanges
					* @type {Array}
					* @example 
	                * &lt;div id="LinearGauge1"&gt;&lt;/div&gt; <br> 
	                * &lt;script&gt; 
					* $("#LinearGauge1").ejLinearGauge({
                    *   value: 50, scales: [{showBarPointers: true, showIndicators: true, length: 310, indicators: [{
                    *   font: { size: "11px", fontFamily: "Arial", fontStyle: "Normal" }, height: 30,
                    *   type: "text", width: 30, position: { x: 60, y: 100 }, textLocation: { x: 0, y: 100 },
                    *   stateRanges: [{ endValue: 60, startValue: 50, textColor: "Green", text: "Linear" }],opacity: 0.5}]}]});	
	                * &lt;/script&gt;			
					* @memberof ejLinearGauge
					* @instance
                    * @remarks
                    * for reflecting the customization in Indicator, showIndicators must be true.
					*/
                    stateRanges: [{
					/**		
					* Specifies the endValue in bar indicators
					* @default 60
                    * @alias ejLinearGauge#scales->indicators->stateRanges->endValue
					* @type {number}
					* @example 
	                * &lt;div id="LinearGauge1"&gt;&lt;/div&gt; <br> 
	                * &lt;script&gt; 
					* $("#LinearGauge1").ejLinearGauge({
                    *   value: 50, scales: [{showBarPointers: true, showIndicators: true, length: 310, indicators: [{
                    *   font: { size: "11px", fontFamily: "Arial", fontStyle: "Normal" }, height: 30,
                    *   type: "text", width: 30, position: { x: 60, y: 100 }, textLocation: { x: 0, y: 100 },
                    *   stateRanges: [{ endValue: 90, startValue: 40, textColor: "Green", text: "Linear" }],opacity: 0.5}]}]});
	                * &lt;/script&gt;		
					* @memberof ejLinearGauge
					* @instance
                    * @remarks
                    * for reflecting the customization in Indicator, showIndicators must be true.
					*/
                    endValue: 60,
					/**		
					* Specifies the startValue in bar indicators
					* @default 50
                    * @alias ejLinearGauge#scales->indicators->stateRanges->startValue
					* @type {number}
					* @example 
	                * &lt;div id="LinearGauge1"&gt;&lt;/div&gt; <br> 
	                * &lt;script&gt; 
                    * $("#LinearGauge1").ejLinearGauge({
                    *   value: 50, scales: [{showBarPointers: true, showIndicators: true, length: 310, indicators: [{
                    *   font: { size: "11px", fontFamily: "Arial", fontStyle: "Normal" }, height: 30,
                    *   type: "text", width: 30, position: { x: 60, y: 100 }, textLocation: { x: 0, y: 100 },
                    *   stateRanges: [{ endValue: 90, startValue: 40, textColor: "Green", text: "Linear" }],opacity: 0.5}]}]});
	                * &lt;/script&gt;					
					* @memberof ejLinearGauge
					* @instance
                    * @remarks
                    * for reflecting the customization in Indicator, showIndicators must be true.
					*/
                    startValue: 50,
					/**		
					* Specifies the backgroundColor in bar indicators
					* @default null
                    * @alias ejLinearGauge#scales->indicators->stateRanges->backgroundColor
					* @type {string}
					* @example 
	                * &lt;div id="LinearGauge1"&gt;&lt;/div&gt; <br> 
	                * &lt;script&gt; 
					* $("#LinearGauge1").ejLinearGauge({
                    *    value: 50, scales: [{showBarPointers: true, showIndicators: true, length: 310, indicators: [{
                    *    height: 30,type: "rectangle",width: 30,
                    *    position: { x: 0,y: 0},
                    *    stateRanges: [{endValue: 60,startValue: 40,backgroundColor: "Red",borderColor: "Green"}],
                    *    border:{width: 1.5}}]}]});
	                * &lt;/script&gt;	
					* @memberof ejLinearGauge
					* @instance
                    * @remarks
                    * for reflecting the customization in Indicator, showIndicators must be true.
					*/
                    backgroundColor: null,
					/**		
					* Specifies the borderColor in bar indicators
					* @default null
                    * @alias ejLinearGauge#scales->indicators->stateRanges->borderColor
					* @type {string}
					* @example 
	                * &lt;div id="LinearGauge1"&gt;&lt;/div&gt; <br> 
	                * &lt;script&gt;
                    * $("#LinearGauge1").ejLinearGauge({
                    *    value: 50, scales: [{showBarPointers: true, showIndicators: true, length: 310, indicators: [{
                    *    height: 30,type: "rectangle",width: 30,
                    *    position: { x: 0,y: 0},
                    *    stateRanges: [{endValue: 60,startValue: 40,backgroundColor: "Green",borderColor: "Red"}],
                    *    border:{width: 1.5}}]}]});
	                * &lt;/script&gt;	
					* @memberof ejLinearGauge
					* @instance
                    * @remarks
                    * for reflecting the customization in Indicator, showIndicators must be true.
					*/
                    borderColor: null,
					/**		
					* Specifies the text in bar indicators
					* @default ""
                    * @alias ejLinearGauge#scales->indicators->stateRanges->text
					* @type {string}
					* @example 
	                * &lt;div id="LinearGauge1"&gt;&lt;/div&gt; <br> 
	                * &lt;script&gt; 
					* 	$("#LinearGauge1").ejLinearGauge({
                    *   value: 50, scales: [{showBarPointers: true, showIndicators: true, length: 310, indicators: [{
                    *   font: { size: "11px", fontFamily: "Arial", fontStyle: "Normal" }, height: 30,
                    *   type: "text", width: 30, position: { x: 60, y: 100 }, textLocation: { x: 20, y: 100 },
                    *   stateRanges: [{ endValue: 90, startValue: 40, textColor: "Green", text: "Linear Gauge" }],opacity: 0.5}]}]});	
	                * &lt;/script&gt;					
					* @memberof ejLinearGauge
					* @instance
                    * @remarks
                    * for reflecting the customization in Indicator, showIndicators must be true.
					*/
                    text: "",
					/**		
					* Specifies the textColor in bar indicators
					* @default null
                    * @alias ejLinearGauge#scales->indicators->stateRanges->textColor
					* @type {string}
					* @example 
	                * &lt;div id="LinearGauge1"&gt;&lt;/div&gt; <br> 
	                * &lt;script&gt;  
					* $("#LinearGauge1").ejLinearGauge({
                    *   value: 50, scales: [{showBarPointers: true, showIndicators: true, length: 310, indicators: [{
                    *   font: { size: "11px", fontFamily: "Arial", fontStyle: "Normal" }, height: 30,
                    *   type: "text", width: 30, position: { x: 60, y: 100 }, textLocation: { x: 0, y: 100 },
                    *   stateRanges: [{ endValue: 90, startValue: 40, textColor: "Red", text: "Linear" }],opacity: 0.5}]}]});	
	                * &lt;/script&gt;					
					* @memberof ejLinearGauge
					* @instance
                    * @remarks
                    * for reflecting the customization in Indicator, showIndicators must be true.
					*/
                        textColor: null
                    }],
					/**		
					* Specifies the value in bar indicators
					* @default 0
                    * @alias ejLinearGauge#scales->indicators->value
					* @type {number}
					* @example 
	                * &lt;div id="LinearGauge1"&gt;&lt;/div&gt; <br> 
	                * &lt;script&gt;
					* $("#LinearGauge1").ejLinearGauge({
                    *    value: 50, scales: [{showBarPointers: true, showIndicators: true, length: 310, indicators: [{
                    *    height: 30,type: "rectangle",width: 30,
                    *    position: { x: 0,y: 0},
                    *    stateRanges: [{endValue: 60,startValue: 40,backgroundColor: "Green",borderColor: "Red"}],
                    *    value:50}]}]});		
	                * &lt;/script&gt;			
					* @memberof ejLinearGauge
					* @instance
                    * @remarks
                    * for reflecting the customization in Indicator, showIndicators must be true.
					*/
                    value: 0,
					/**		
					* Specifies the backgroundColor in bar indicators
					* @default null
                    * @alias ejLinearGauge#scales->indicators->backgroundColor
					* @type {string}
					* @example 
	                * &lt;div id="LinearGauge1"&gt;&lt;/div&gt; <br> 
	                * &lt;script&gt;
                    * $("#LinearGauge1").ejLinearGauge({
                    *    value: 50, scales: [{showBarPointers: true, showIndicators: true, length: 310, indicators: [{
                    *    height: 30,type: "rectangle",width: 30,
                    *    position: { x: 0,y: 0},
                    *    stateRanges: [{endValue: 60,startValue: 40,backgroundColor: "Green",borderColor: "Red"}],
                    *    backgroundColor:"green"}]}]});	
	                * &lt;/script&gt;		
					* @memberof ejLinearGauge
					* @instance
                    * @remarks
                    * for reflecting the customization in Indicator, showIndicators must be true.
					*/
                    backgroundColor: null,
                    /**		
					* Specifies the border in bar indicators
					* @default Object
                    * @alias ejLinearGauge#scales->indicators->border
					* @type {number}
					* @example 
	                * &lt;div id="LinearGauge1"&gt;&lt;/div&gt; <br> 
	                * &lt;script&gt; 
                    * $("#LinearGauge1").ejLinearGauge({
                    *    value: 50, scales: [{showBarPointers: true, showIndicators: true, length: 310, indicators: [{
                    *    height: 30,type: "rectangle",width: 30,
                    *    position: { x: 0,y: 0},
                    *    stateRanges: [{endValue: 60,startValue: 40,backgroundColor: "Green",borderColor: "Red"}],
                    *    border:{color:"red", width: 5}}]}]});		
	                * &lt;/script&gt;			
		    		* @memberof ejLinearGauge
			    	* @instance
                    * @remarks
                    * for reflecting the customization in Indicator, showIndicators must be true.
					*/                    
                    border: {
                        /**		
					    * Specifies the border Color in bar indicators
					    * @default null
                        * @alias ejLinearGauge#scales->indicators->border->color
					    * @type {string}
					    * @example 
    	                * &lt;div id="LinearGauge1"&gt;&lt;/div&gt; <br> 
	                    * &lt;script&gt;
                        * $("#LinearGauge1").ejLinearGauge({
                        *    value: 50, scales: [{showBarPointers: true, showIndicators: true, length: 310, indicators: [{
                        *    height: 30,type: "rectangle",width: 30,
                        *    position: { x: 0,y: 0},
                        *    stateRanges: [{endValue: 60,startValue: 40,backgroundColor: "Green",borderColor: "Red"}],
                        *    border:{color: "Red"}}]}]});	
	                    * &lt;/script&gt;		
					    * @memberof ejLinearGauge
					    * @instance
                        * @remarks
                        * for reflecting the customization in Indicator, showIndicators must be true.
					    */
                        color: null,
                        /**		
					    * Specifies the border Width in bar indicators
					    * @default 1.5
                        * @alias ejLinearGauge#scales->indicators->border->width
					    * @type {number}
					    * @example 
	                    * &lt;div id="LinearGauge1"&gt;&lt;/div&gt; <br> 
	                    * &lt;script&gt; 
                        * $("#LinearGauge1").ejLinearGauge({
                        *    value: 50, scales: [{showBarPointers: true, showIndicators: true, length: 310, indicators: [{
                        *    height: 30,type: "rectangle",width: 30,
                        *    position: { x: 0,y: 0},
                        *    stateRanges: [{endValue: 60,startValue: 40,backgroundColor: "Green",borderColor: "Red"}],
                        *    border:{width: 5}}]}]});		
	                    * &lt;/script&gt;			
		    			* @memberof ejLinearGauge
			    		* @instance
                        * @remarks
                        * for reflecting the customization in Indicator, showIndicators must be true.
					    */
                        width:1.5
                    },
					/**		
					* Specifies the opacity in bar indicators
					* @default NaN
                    * @alias ejLinearGauge#scales->indicators->opacity
					* @type {number}
					* @example 
	                * &lt;div id="LinearGauge1"&gt;&lt;/div&gt; <br> 
	                * &lt;script&gt;
                    * $("#LinearGauge1").ejLinearGauge({ readonly:false, scales: [{showBarPointers: false, showIndicators: true, indicators: [{height: 30,
                    * type: "rectangle",position: {x: 80,y: 110},
                    * stateRanges: [{endValue: 80,startValue: 50,backgroundColor: "Red",borderColor: "Green",textColor: null}],
                    * value: 0, backgroundColor: "Red", border:{color: "Green"}, opacity:0.5}]}]});
	                * &lt;/script&gt;			
					* @memberof ejLinearGauge
					* @instance
                    * @remarks
                    * for reflecting the customization in Indicator, showIndicators must be true.
					*/
                    opacity: NaN
                }],
					/**		
					* Specifies the  customLabel
					* @default Array
                    * @alias ejLinearGauge#scales->customLabels
					* @type {Array}
					* @example 
	                * &lt;div id="LinearGauge1"&gt;&lt;/div&gt; <br> 
	                * &lt;script&gt;
					* 	$("#LinearGauge1").ejLinearGauge({  scales:[{showCustomLabels:true,customLabels: [{ value:"MyLabel", position:{x:49,y:100}, color:"#fc0606", font: { size: "20px",fontFamily: "Arial", fontStyle: "bold" }}]}]});
	                * &lt;/script&gt;					
					* @memberof ejLinearGauge
					* @instance
                    * @remarks
                    * for reflecting the customization in customLabel, showCustomLabels must be true.
					*/
                customLabels: [{
					/**		
					* Specifies the font in  customLabels
					* @default Object
                    * @alias ejLinearGauge#scales->customLabels->font
					* @type {object}
					* @example 
	                * &lt;div id="LinearGauge1"&gt;&lt;/div&gt; <br> 
	                * &lt;script&gt;
					* 	$("#LinearGauge1").ejLinearGauge({  scales:[{showCustomLabels:true,customLabels: [{ value:"MyLabel", position:{x:49,y:100}, color:"#fc0606", font: { size: "20px",fontFamily: "Arial", fontStyle: "bold" }}]}]});
	                * &lt;/script&gt;			
					* @memberof ejLinearGauge
					* @instance
                    * @remarks
                    * for reflecting the customization in customLabels, showCustomLabels must be true.
					*/
                    font: { 
					/**		
					* Specifies the font size in  customLabels
					* @default "11px"
                    * @alias ejLinearGauge#scales->customLabels->font->size
					* @type {string}
					* @example 
	                * &lt;div id="LinearGauge1"&gt;&lt;/div&gt; <br> 
	                * &lt;script&gt; 
					* 	$("#LinearGauge1").ejLinearGauge({  scales:[{showCustomLabels:true,customLabels: [{ value:"MyLabel", position:{x:49,y:100}, color:"#fc0606", font: { size: "20px",fontFamily: "Arial", fontStyle: "bold" }}]}]});
	                * &lt;/script&gt;				
					* @memberof ejLinearGauge
					* @instance
                    * @remarks
                    * for reflecting the customization in customLabels, showCustomLabels must be true.
					*/
					size: "11px", 
					/**		
					* Specifies the fontFamily in  customLabels
					* @default "Arial"
                    * @alias ejLinearGauge#scales->customLabels->font->fontFamily
					* @type {string}
					* @example 
	                * &lt;div id="LinearGauge1"&gt;&lt;/div&gt; <br> 
	                * &lt;script&gt; 
					* 	$("#LinearGauge1").ejLinearGauge({  scales:[{showCustomLabels:true,customLabels: [{ value:"MyLabel", position:{x:49,y:100}, color:"#fc0606", font: { size: "20px",fontFamily: "Arial", fontStyle: "bold" }}]}]});
	                * &lt;/script&gt;			
					* @memberof ejLinearGauge
					* @instance
                    * @remarks
                    * for reflecting the customization in customLabels, showCustomLabels must be true.
					*/
					fontFamily: "Arial", 
					/**		
					* Specifies the fontStyle in  customLabels. See {@link FontStyle}
					* @default ej.datavisualization.LinearGauge.FontStyle.Bold
                    * @alias ejLinearGauge#scales->customLabels->font->fontStyle
					* @type {enum}
					* @example 
	                * &lt;div id="LinearGauge1"&gt;&lt;/div&gt; <br> 
	                * &lt;script&gt;
					* 	$("#LinearGauge1").ejLinearGauge({  scales:[{showCustomLabels:true,customLabels: [{ value:"MyLabel", position:{x:49,y:100}, color:"#fc0606", font: { size: "20px",fontFamily: "Arial", fontStyle: "bold" }}]}]});
	                * &lt;/script&gt;	
					* @memberof ejLinearGauge
					* @instance
                    * @remarks
                    * for reflecting the customization in customLabels, showCustomLabels must be true.
					*/
					fontStyle: "bold" },
					/**		
					* Specifies the label Color in  customLabels
					* @default null
                    * @alias ejLinearGauge#scales->customLabels->color
					* @type {number}
					* @example 
	                * &lt;div id="LinearGauge1"&gt;&lt;/div&gt; <br> 
	                * &lt;script&gt;
					* 	$("#LinearGauge1").ejLinearGauge({  scales:[{showCustomLabels:true,customLabels: [{ value:"MyLabel", position:{x:49,y:100}, color:"yellow", font: { size: "20px",fontFamily: "Arial", fontStyle: "Bold" }}]}]});
	                * &lt;/script&gt;				
					* @instance
                    * @remarks
                    * for reflecting the customization in customLabels, showCustomLabels must be true.
					*/
                    color: null,
					/**		
					* Specifies the opacity in  customLabels
					* @default 0
                    * @alias ejLinearGauge#scales->customLabels->opacity
					* @type {string}
					* @example 
	                * &lt;div id="LinearGauge1"&gt;&lt;/div&gt; <br> 
	                * &lt;script&gt;
					* 	$("#LinearGauge1").ejLinearGauge({  scales:[{showCustomLabels:true,customLabels: [{ value:"MyLabel", position:{x:49,y:100}, color:"#fc0606",opacity:0.5, font: { size: "20px",fontFamily: "Arial", fontStyle: "bold" }}]}]});
	                * &lt;/script&gt;					
					* @memberof ejLinearGauge
					* @instance
                    * @remarks
                    * for reflecting the customization in customLabels, showCustomLabels must be true.
					*/
                    opacity: 0,
					/**		
					* Specifies the label Value in  customLabels
					* @default ""
                    * @alias ejLinearGauge#scales->customLabels->value
					* @type {string}
					* @example 
	                * &lt;div id="LinearGauge1"&gt;&lt;/div&gt; <br> 
	                * &lt;script&gt;
					* 	$("#LinearGauge1").ejLinearGauge({  scales:[{showCustomLabels:true,customLabels: [{ value:"LinearGauge", position:{x:49,y:100}, color:"#fc0606", font: { size: "20px",fontFamily: "Arial", fontStyle: "bold" }}]}]});
	                * &lt;/script&gt;			
					* @instance
                    * @remarks
                    * for reflecting the customization in customLabels, showCustomLabels must be true.
					*/
                    value: "",
					/**		
					* Specifies the textAngle in  customLabels
					* @default 0
                    * @alias ejLinearGauge#scales->customLabels->textAngle
					* @type {number}
					* @example 
	                * &lt;div id="LinearGauge1"&gt;&lt;/div&gt; <br> 
	                * &lt;script&gt;
					* 	$("#LinearGauge1").ejLinearGauge({  scales:[{showCustomLabels:true,customLabels: [{ textAngle: 20,value:"LinearGauge", position:{x:49,y:100}, color:"#fc0606", font: { size: "20px",fontFamily: "Arial", fontStyle: "bold" }}]}]});		
	                * &lt;/script&gt;				
					* @memberof ejLinearGauge
					* @instance
                    * @remarks
                    * for reflecting the customization in customLabels, showCustomLabels must be true.
					*/
                    textAngle: 0,
					/**		
					* Specifies the position in  customLabels
					* @default Object
                    * @alias ejLinearGauge#scales->customLabels->position
					* @type {object}
					* @example 
	                * &lt;div id="LinearGauge1"&gt;&lt;/div&gt; <br> 
	                * &lt;script&gt;
					* 	$("#LinearGauge1").ejLinearGauge({  scales:[{showCustomLabels:true,customLabels: [{ value:"LinearGauge", position:{x:49,y:100}, color:"#fc0606", font: { size: "20px",fontFamily: "Arial", fontStyle: "bold" }}]}]});		
	                * &lt;/script&gt;		
					* @memberof ejLinearGauge
					* @instance
                    * @remarks
                    * for reflecting the customization in customLabels, showCustomLabels must be true.
					*/
                    position: { 
					/**		
					* Specifies the position x in  customLabels
					* @default 0
                    * @alias ejLinearGauge#scales->customLabels->position->x
					* @type {number}
					* @example 
	                * &lt;div id="LinearGauge1"&gt;&lt;/div&gt; <br> 
	                * &lt;script&gt;
					* 	$("#LinearGauge1").ejLinearGauge({  scales:[{showCustomLabels:true,customLabels: [{ textAngle: 20,value:"LinearGauge", position:{x:10,y:50}, color:"#fc0606", font: { size: "20px",fontFamily: "Arial", fontStyle: "bold" }}]}]});		
	                * &lt;/script&gt;			
					* @memberof ejLinearGauge
					* @instance
                    * @remarks
                    * for reflecting the customization in customLabels, showCustomLabels must be true.
					*/
					x: 0, 
					/**		
					* Specifies the y in  customLabels
					* @default 0
                    * @alias ejLinearGauge#scales->customLabels->position->y
					* @type {number}
					* @example 
	                * &lt;div id="LinearGauge1"&gt;&lt;/div&gt; <br> 
	                * &lt;script&gt;
					* 	$("#LinearGauge1").ejLinearGauge({  scales:[{showCustomLabels:true,customLabels: [{ textAngle: 20,value:"LinearGauge", position:{x:49,y:10}, color:"#fc0606", font: { size: "20px",fontFamily: "Arial", fontStyle: "bold" }}]}]});		
	                * &lt;/script&gt;				
					* @memberof ejLinearGauge
					* @instance
                    * @remarks
                    * for reflecting the customization in customLabels, showCustomLabels must be true.
					*/
					y: 0 }
                }]
            };
        },

        dataTypes: {
            scales: "data"
        },
        observables: ["value", "minimum", "maximum"],
        value: ej.util.valueFunction("value"),
        minimum: ej.util.valueFunction("minimum"),
        maximum: ej.util.valueFunction("maximum"),

        // constructor function
		/**
         * To initiate the gauge		
		 * @private
         */	 
        _init: function () {
            this._initialize();
            this._trigger("load");
            this._setTheme();
            this._render();
            if (!this.model.readOnly)
                this.wireEvents();
			this._onWindowResize();
        },
		/**
         * Invoke when window size changes 
		 * @private
         */	
		_onWindowResize:function()
        {
            if (this.model.enableResize) {
                proxy = this;
                if (!this.isDevice())
                    this._on($(window), "resize", proxy.resizeCanvas);
                else
                    this._on($(window), "orientationchange", proxy.resizeCanvas);
            }
        },
		/**
         * To check whether it is a device or not
		 * @private
         */	
        isDevice: function () {
            return (/mobile|tablet|android|kindle/i.test(navigator.userAgent.toLowerCase()));
        },
		/**
         * To configure the properties at runtime using SetModel		
		 * @private
         */	 
        _setModel: function (options) {
            var option;
            for (option in options) {
                switch (option) {
                    case "theme": this.model.theme = options[option]; this._init(); break;
                    case "height": this.model.height = options[option]; break;
                    case "width": this.model.width = options[option]; break;
                    case "orientation": this.model.orientation = options[option]; break;
                    case "pointerGradient1": this.model.pointerGradient1 = options[option]; break;
                    case "pointerGradient2": this.model.pointerGradient2 = options[option]; break;
                    case "labelColor": this.model.labelColor = options[option]; break;
                    case "tick": $.extend(this.model.tick,options[option]); break;
                    case "backgroundColor": this.model.backgroundColor = options[option]; break;
                    case "borderColor": this.model.borderColor = options[option]; break;
                    case "frame": $.extend(this.model.frame, options[option]);
                    case "readOnly": this.model.readOnly = options[option]; break;
                    case "value":
                        (this.value() == "") && this.value(0);
                        for (var i = 0; this.model.scales[i] != null; i++) {
                            for (j = 0; this.model.scales[i].markerPointers[j] != null; j++) {
                                this.model.scales[i].markerPointers[j].value = parseFloat(this.value());
                            }
                        }
                        for (var i = 0; this.model.scales[i] != null; i++) {
                            for (j = 0; this.model.scales[i].barPointers[j] != null; j++) {
                                this.model.scales[i].barPointers[j].value = parseFloat(this.value());
                            }
                        }
                        break;
                    case "minimum":
                        (this.minimum() == "") && this.minimum(0);
                        for (var i = 0; this.model.scales[i] != null; i++) {
                            this.model.scales[i].minimum = parseInt(this.minimum());
                        }
                        break;
                    case "maximum":
                        (this.maximum() == "") && this.maximum(0);
                        for (var i = 0; this.model.scales[i] != null; i++) {
                            this.model.scales[i].maximum = parseInt(this.maximum());
                        }
                        break;
                    case "scales":
                        this.model.scales = options[option];
                        this._itemInitialize();
                        break;
                }
            }
            this._render();
        },
		/**
        * destroy the linear gauge
		* all events bound using this._on will be unbind automatically and bring the control to pre-init state.
		* @alias destroy
		* @return jQuery
		* @example 
		* &lt;div id="LinearGauge1"&gt;&lt;/div&gt; <br> 
		* &lt;script&gt;
        * $("#LinearGauge1").ejLinearGauge();
		* var linearGaugeobj = $("#LinearGauge1").data("ejLinearGauge");
		* linearGaugeobj.destroy();
		* &lt;/script&gt;
		* @memberof ejLinearGauge
		* @instance
         */	       	 
        _destroy: function () {
            this.activeElement = null;
            this.canvasEl = null;
            this.contextEl = null;
            this.unWireEvents();
            this.element.empty().removeClass("e-widget");
        },
		/**
         * To initialize the gauge object
		 * @private
         */	
        _initialize: function () {
            this.GaugeEl = this.element;
            this.scaleStartX = [];
            this.scaleStartY = [];
            this.isScaleModified = false;
            this.target = this.element[0];
            this._itemInitialize();
            this.Model = this.model;
        },
		/**
         * To render the gauge object
		 * @private
         */
        _render: function () {
            this.initialize();
        },
		/**
         * To Initialize the items
		 * @private
         */
        _itemInitialize: function () {
            var proxy = this;
            if (this.model.scales != null) {
                $.each(this.model.scales, function (index, element) {
                    element = proxy._checkArrayObject(element, index);
                    var obj = proxy._defaultScaleValues();
                    $.extend(obj, element);
                    $.extend(element, obj);
                });
            }
            else {
                this.model.scales = [this._defaultScaleValues()];
            }
        },
		/**
         * To check the each array element to load
		 * @private
         */	
        _checkArrayObject: function (element, initialName) {
            var proxy = this;
            $.each(element, function (name, innerElement) {
                if (innerElement instanceof Array) {
                    innerElement = proxy._checkArrayObject(innerElement, name);
                }
                else if (innerElement != null && typeof innerElement == "object") {
                    var allObjects = proxy._defaultScaleValues();
                    innerElement = proxy._LoadIndividualDefaultValues(innerElement, allObjects, (typeof name === "number") ? initialName : name);
                }
            });
            return element;
        },
		/**
         * To Load individual default values
		 * @private
         */	
        _LoadIndividualDefaultValues: function (obj, allObjects, name) {
            var defaultObj = null;
            var proxy = this;
            $.each(allObjects, function (n, element) {
                if (name == n) {
                    defaultObj = element;
                    return;
                }
            });
            if (defaultObj instanceof Array) defaultObj = defaultObj[0];

            $.each(obj, function (objName, Ele) {
                if (Ele instanceof Array) {
                    Ele = proxy._checkArrayObject(Ele, name);
                }
                else if (Ele != null && typeof Ele == "object") {
                    Ele = proxy._LoadIndividualDefaultValues(Ele, defaultObj, (typeof objName === "number") ? name : objName);
                }
            });

            $.extend(defaultObj, obj);
            $.extend(obj, defaultObj);
            return obj;
        },
		/**
         * To Initialize the gauge
		 * @private
         */
        initialize: function () {
            this._initObject(this);

            if (this.Model.frame.backgroundImageUrl)
                this._drawCustomImage(this, this.Model.frame.backgroundImageUrl);
            else {
                if (this.Model.scales != null)
                    this._drawScales();
            }
        },
		/**
         * To Initialize the object
		 * @private
         */
        _initObject: function (element) {
            this.element.addClass("e-widget");
            element.GaugeEl = element.element;
            for (var i = 0; this.model.scales[i] != null; i++) {
                if (this.model.scales[i].minimum == null)
                    this.model.scales[i].minimum = this.minimum();
                if (this.model.scales[i].maximum == null)
                    this.model.scales[i].maximum = this.maximum();
                for (var j = 0; this.model.scales[i].markerPointers[j] != null; j++) {
                    if (this.model.scales[i].markerPointers[j].value == null)
                        this.model.scales[i].markerPointers[j].value = this.value();
                }
                for (var j = 0; this.model.scales[i].barPointers[j] != null; j++) {
                    if (this.model.scales[i].barPointers[j].value == null)
                        this.model.scales[i].barPointers[j].value = this.value();
                }
            }
            if (element.canvasEl)
                element.canvasEl.remove();
            else
                element.canvasEl = $("<canvas></canvas>");
            element.GaugeEl.append(element.canvasEl);
			element.canvasEl.attr("role", "presentation");
            initialDivWidth = element.GaugeEl.width();
            initialDivHeight = element.GaugeEl.height();
            element.canvasEl[0].setAttribute("width", element.model.width);
            element.canvasEl[0].setAttribute("height", element.model.height);
            element.centerX = element.canvasEl[0].width / 2;
            element.centerY = element.canvasEl[0].height / 2;
            var elem = element.canvasEl[0];
            if (typeof window.G_vmlCanvasManager != "undefined") {
                elem = window.G_vmlCanvasManager.initElement(elem);
            }
            if (!elem || !elem.getContext) {
                return;
            }
            element.contextEl = element.canvasEl[0].getContext("2d");
        },
		/**
         * To draw circular frame
		 * @private
         */
        _drawFrameCircle: function (location, style, element) {
            this._contextOpenPath(style, element);
            element.contextEl.arc(location.startX, location.startY, style.circleRadius, 0, 2 * Math.PI, true);
            this._contextClosePath(style, element);
            if (style.indicatorText)
                element._drawText(location, style);
        },
		/**
         * To draw Rectangular frame
		 * @private
         */
        _drawFrameRectangle: function (location, style, element) {
            this._contextOpenPath(style, element);
            element.contextEl.moveTo(location.startX + style.radius, location.startY);
            element.contextEl.lineTo(location.startX + style.width - style.radius, location.startY);
            element.contextEl.lineTo(location.startX + style.width, location.startY + style.height - style.radius);
            element.contextEl.lineTo(location.startX + style.radius, location.startY + style.height);
            this._contextClosePath(style, element);
            if (style.indicatorText)
                element._drawText(location, style);
        },
		/**
         * To draw Thermometer frame
		 * @private
         */
        _drawFrameThermometer: function (location, style, element) {
            var radius = element.Model.orientation == "Vertical" ? Math.sqrt(style.width * style.width + style.width * style.width) / 2 : Math.sqrt(style.height * style.height + style.height * style.height) / 2; //radius of the bottom circle
            this._contextOpenPath(style, element);
            if (element.Model.orientation == "Vertical") {
                if (element.scaleEl[element.scaleIndex].direction == "counterclockwise") {
                    element.contextEl.arc(location.startX + Math.cos(Math.PI * (45 / 180)) * radius,
                                       location.startY + style.height - Math.sin(Math.PI * (45 / 180)) * radius,
                                       radius, Math.PI * (-45 / 180), Math.PI * (225 / 180), false);
                    element.contextEl.lineTo(location.startX, location.startY + style.calDistance + style.width / 2);
                    if (style.topRounded)
                        element.contextEl.arc(location.startX + style.width / 2, location.startY + style.width / 2, style.width / 2, -Math.PI, 0, false);
                    else
                        element.contextEl.lineTo(location.startX + style.width, location.startY + style.calDistance + style.width / 2);
                }
                else {
                    element.contextEl.arc(location.startX + Math.cos(Math.PI * (45 / 180)) * radius,
                                       location.startY + Math.sin(Math.PI * (45 / 180)) * radius,
                                       radius, Math.PI * (45 / 180), Math.PI * (-225 / 180), true);
                    element.contextEl.lineTo(location.startX, location.startY + style.height - style.width / 2);
                    if (style.topRounded)
                        element.contextEl.arc(location.startX + style.width / 2, location.startY + style.height - style.width / 2, style.width / 2, -Math.PI, 0, true);
                    else
                        element.contextEl.lineTo(location.startX + style.width, location.startY + style.height - style.width / 2);
                }
            }
            else {
                if (element.scaleEl[element.scaleIndex].direction == "counterclockwise") {
                    element.contextEl.arc(location.startX + style.width - radius / 4 - Math.cos(Math.PI * (45 / 180)) * radius,
                               location.startY + Math.sin(Math.PI * (45 / 180)) * radius,
                               radius, Math.PI * (135 / 180), Math.PI * (225 / 180), true);
                    element.contextEl.lineTo(location.startX + style.height / 2, location.startY);
                    if (style.topRounded)
                        element.contextEl.arc(location.startX + style.height / 2, location.startY + style.height / 2, style.height / 2, Math.PI * (270 / 180), Math.PI * (90 / 180), true);
                    else
                        element.contextEl.lineTo(location.startX + style.height / 2, location.startY + style.height);

                }
                else {
                    element.contextEl.arc(location.startX + radius / 4 + Math.cos(Math.PI * (45 / 180)) * radius,
                               location.startY + Math.sin(Math.PI * (45 / 180)) * radius,
                               radius, Math.PI * (45 / 180), Math.PI * (315 / 180), false);
                    element.contextEl.lineTo(location.startX + style.width - style.height / 2, location.startY);
                    if (style.topRounded)
                        element.contextEl.arc(location.startX + style.width - style.height / 2, location.startY + style.height / 2, style.height / 2, Math.PI * (270 / 180), Math.PI * (90 / 180), false);
                    else
                        element.contextEl.lineTo(location.startX + style.width - style.height / 2, location.startY + style.height);
                }
            }
            this._contextClosePath(style, element);
        },
		/**
         * To draw RoundedRectangle frame
		 * @private
         */
        _drawFrameRoundedRectangle: function (location, style, element) {
            this._contextOpenPath(style, element);
            element.contextEl.moveTo(location.startX + style.radius, location.startY);
            element.contextEl.lineTo(location.startX + style.width - style.radius, location.startY);
            element.contextEl.quadraticCurveTo(location.startX + style.width, location.startY, location.startX + style.width, location.startY + style.radius);
            element.contextEl.lineTo(location.startX + style.width, location.startY + style.height - style.radius);
            element.contextEl.quadraticCurveTo(location.startX + style.width, location.startY + style.height, location.startX + style.width - style.radius, location.startY + style.height);
            element.contextEl.lineTo(location.startX + style.radius, location.startY + style.height);
            element.contextEl.quadraticCurveTo(location.startX, location.startY + style.height, location.startX, location.startY + style.height - style.radius);
            element.contextEl.lineTo(location.startX, location.startY + style.radius);
            element.contextEl.quadraticCurveTo(location.startX, location.startY, location.startX + style.radius, location.startY);
            this._contextClosePath(style, element);
            if (style.indicatorText)
                this._drawText(location, style);
        },
		/**
         * To styling context open path
		 * @private
         */
        _contextOpenPath: function (style, element) {
            element.contextEl.save();
            element.contextEl.beginPath();
            if (style.strokeStyle)
                element.contextEl.strokeStyle = style.strokeStyle;
            if (style.opacity != undefined)
                element.contextEl.globalAlpha = style.opacity;
            if (style.lineWidth)
                element.contextEl.lineWidth = style.lineWidth;
            if (style.fillStyle)
                element.contextEl.fillStyle = style.fillStyle;
        },
		/**
         * To styling context Close path
		 * @private
         */
        _contextClosePath: function (style, element) {
            element.contextEl.closePath();
            if (style.isFill)
                element.contextEl.fill();
            if (style.isStroke)
                element.contextEl.stroke();
            element.contextEl.restore();
        },
		/**
         * To Draw scales
		 * @private
         */
        _drawScales: function () {
            var self = this;
            this.scaleEl = this.Model.scales;
            this.contextEl.save();
            this.contextEl.translate(this.Model.frame.outerWidth + this.Model.frame.innerWidth, this.Model.frame.outerWidth + this.Model.frame.innerWidth);
            $.each(this.Model.scales, function (index, element) {
                self.scaleIndex = index;
                self._setScaleCordinates(element, element.type);
            });
            this._setTicks();
            this._setLabels();
            this._setRange();
            this._setCustomLabel();
            this._flagPointer = false;
            this._tempOpacity=this.model.scales[0].barPointers[0].opacity;
            this._setBarPointers();
            this._setMarkerPointers();
            this._setIndicators();
            $.each(this.Model.scales, function (ind, elemt) {
                if (elemt.showBarPointers) {
                    if (elemt.barPointers.length > 1)
                        self.model.enableAnimation = false;
                }
                if (elemt.showMarkerPointers) {
                    if (elemt.markerPointers.length > 1)
                        self.model.enableAnimation = false;
                }
            });
            if (!this.contextEl.putImageData)
                this.model.enableAnimation = false;
            if (this.model.animationSpeed != null && this.model.animationSpeed > 0) {
                var delay = this.model.animationSpeed / 25;
                if (delay >= 0) {
                    if (this.model.enableAnimation && this._animationFlag) {
                        this._onAnimate(delay);
                    }
                }
            }
        },
		/**
         * To set ticks coordinates
		 * @private
         */
        _setTicks: function () {
            var self = this;
            $.each(this.Model.scales, function (index, element) {
                if (element.showTicks) {
                    self.scaleIndex = index;
                    if (element.ticks != null) {
                        self.tickEl = element.ticks;
                        $.each(element.ticks, function (index, element) {
                            self.tickIndex = index;
                            self._setTicksCordinates(element, index);
                        });
                    }
                }
            });
        },
		/**
         * To set Label coordinates
		 * @private
         */
        _setLabels: function () {
            var self = this;
            $.each(this.Model.scales, function (index, element) {
                if (element.showLabels) {
                    self.scaleIndex = index;
                    if (element.labels != null) {
                        self.labelEl = element.labels;
                        $.each(element.labels, function (index, element) {
                            self.labelIndex = index;
                            self._setLabelCordinates(element, index);
                        });
                    }
                }
            });
        },
		/**
         * To set Indicator coordinates
		 * @private
         */
        _setIndicators: function () {
            var self = this;
            $.each(this.Model.scales, function (index, element) {
                self.scaleIndex = index;
                if (element.indicators != null && element.showIndicators) {
                    self.indicatorEl = element.indicators;
                    $.each(element.indicators, function (index, element) {
                        self.indicatorIndex = index;
                        self._drawIndicator(index, element);
                    });
                }
            });
        },
		/**
         * To set Bar Pointer coordinates
		 * @private
         */
        _setBarPointers: function () {
            var self = this;
            $.each(this.Model.scales, function (index, element) {
                if (element.showBarPointers) {
                    self.scaleIndex = index;
                    if (element.barPointers != null) {
                        self.barPointerEl = element.barPointers;
                        $.each(element.barPointers, function (index, element) {
                            self.barPointerIndex = index;
                            element.opacity = (self.scaleIndex == 0 && self.barPointerIndex == 0 && self.model.enableAnimation == true && (self._flagPointer == false && self._animationFlag == true) && self.model.scales[0].type == "thermometer") ? 0 : self._tempOpacity;
                            self._drawScaleBarPointer(element, index);
                            self._flagPointer=true;
                        });
                    }
                }
            });
        },
		/**
         * To set Marker Pointer coordinates
		 * @private
         */
        _setMarkerPointers: function () {
            var self = this;
            $.each(this.Model.scales, function (index, element) {
                if (element.showMarkerPointers) {
                    self.scaleIndex = index;
                    if (element.markerPointers != null) {
                        self.markerPointerEl = element.markerPointers;
                        $.each(element.markerPointers, function (index, element) {
                            self.markerPointerIndex = index;
                            self._drawMarkerPointer(element, index);
							self.canvasEl.attr("aria-label", self.model.scales[self.scaleIndex].markerPointers[self.markerPointerIndex].value);
                        });
                    }
                }
            });
        },
		/**
         * To set delay value
		 * @private
         */
        _onAnimate: function (delay) {
            var self = this, timer, timer1;
            var currentValue = self.model.scales[0].minimum;
            var barPointerValue = self.model.scales[0].barPointers[0].value;
            var markerPointerValue = self.model.scales[0].markerPointers[0].value;
            timer = setInterval(function pointerchan() {
                if (barPointerValue > currentValue) {
                    currentValue = currentValue + ((self.model.scales[0].maximum-self.model.scales[0].minimum)/100);
                    if (self.scaleEl[0].type == "thermometer") {
                        self.model.scales[0].barPointers[0].value = (barPointerValue > currentValue) ? currentValue: barPointerValue;
                        if (self.contextEl.putImageData != "undefined") {
                            self._setBarPointers();
                        }
                        else {
                            barPointerValue > currentValue ? self.setBarPointerValue(0, 0, currentValue) : self.setBarPointerValue(0, 0, barPointerValue);
                        }
                    }
                    else
                        barPointerValue > currentValue ? self.setBarPointerValue(0, 0, currentValue) : self.setBarPointerValue(0, 0, barPointerValue);
                }
                else {
                    self._animationFlag = false;
                    self.setBarPointerValue(0, 0, barPointerValue);
                    window.clearInterval(timer);
                }
            }, delay);
            timer1 = setInterval(function pointerchan() {
            if(self.model.enableMarkerPointerAnimation){
                if (markerPointerValue > currentValue) {
                    currentValue = currentValue + ((self.model.scales[0].maximum-self.model.scales[0].minimum)/100);
                    if (self.scaleEl[0].type == "thermometer") {
                        self.model.scales[0].markerPointers[0].value = (markerPointerValue > currentValue) ? currentValue : markerPointerValue;
                        if (self.contextEl.putImageData != "undefined") {
                            self._setMarkerPointers();
                            }
                            else {
                                markerPointerValue > currentValue ? self.setPointerValue(0, 0, currentValue) : self.setPointerValue(0, 0, markerPointerValue);
                            }
                        }
                        else
                            markerPointerValue > currentValue ? self.setPointerValue(0, 0, currentValue) : self.setPointerValue(0, 0, markerPointerValue);
                    }
                    else {
                        self._animationFlag = false;
                        self.setPointerValue(0, 0, markerPointerValue);
                        window.clearInterval(timer1);
                }}
            }, delay);
        },
		/**
         * To set range coordinates
		 * @private
         */
        _setRange: function () {
            var self = this;
            $.each(this.Model.scales, function (index, element) {
                self.scaleIndex = index;
                if (element.ranges != null && element.showRanges) {
                    self.rangeEl = element.ranges;
                    $.each(element.ranges, function (index, element) {
                        self.rangeIndex = index;
                        self._drawRange(element);
                    });
                }
            });
        },
		/**
         * To set CustomLabel coordinates
		 * @private
         */
        _setCustomLabel: function () {
            var self = this;
            $.each(this.Model.scales, function (index, element) {
                self.scaleIndex = index;
                if (element.customLabels != null && element.showCustomLabels) {
                    self.customLabelEl = element.customLabels;
                    $.each(element.customLabels, function (index, element) {
                        self.customLabelIndex = index;
                        self._setCustomLabelCordinates(index, element);
                    });
                }
            });
        },
		/**
         * To set ScaleCordinates
		 * @private
         */
        _setScaleCordinates: function (element, scaleStyle) {
            var bottomRadius, location, style, radius;
            this.opacity = 1;
            this.bottomRadius = Math.sqrt(element.width * element.width + element.width * element.width) / 2;

            this.bounds = {
                height: this.canvasEl[0].height - 2 * (this.Model.frame.outerWidth + this.Model.frame.innerWidth),
                width: this.canvasEl[0].width - 2 * (this.Model.frame.outerWidth + this.Model.frame.innerWidth)
            };
            if (this.Model.orientation == "Vertical") {
                this.scaleStartX[this.scaleIndex] = (this.bounds.width - element.width) * ((element.position.x) / 100);
                this.scaleStartY[this.scaleIndex] = (this.bounds.height - element.length) * ((element.position.y) / 100);
            }
            else {
                this.scaleStartX[this.scaleIndex] = (this.bounds.width - element.length) * ((element.position.x) / 100);
                this.scaleStartY[this.scaleIndex] = (this.bounds.height - element.width) * ((element.position.y) / 100);
            }
            radius = scaleStyle == "roundedrectangle" ? 5 : 0;
            location = { "startX": this.scaleStartX[this.scaleIndex], "startY": this.scaleStartY[this.scaleIndex] };
            style = {
                "width": this.Model.orientation == "Vertical" ? element.width : element.length, "isStroke": true,
                "topRounded": true,
                "fillStyle": element.backgroundColor ? ((element.backgroundColor == "transparent") ? "rgba(0,0,0,0)" : this._getColor(element, element.backgroundColor)) : ((this.Model.backgroundColor == "transparent") ? "rgba(0,0,0,0)" : this._getColor(element, this.Model.backgroundColor)),
                "lineWidth": element.border.width,
                "radius": radius,
                "height": this.Model.orientation == "Vertical" ? element.length : element.width,
                "isStroke": true,
                "isFill": true,
                "strokeStyle": element.border.color ? ((element.border.color == "transparent") ? "rgba(0,0,0,0)" : this._getColor(element, element.border.color)) : ((this.Model.borderColor == "transparent") ? "rgba(0,0,0,0)" : this._getColor(element, this.Model.borderColor))
            };
	    if (element.maximum < element.minimum) {
                var tempValue = element.maximum;
                element.maximum = element.minimum;
                element.minimum=tempValue;
            }
            this.minimum(element.minimum);
            this.maximum(element.maximum);
            if (element.shadowOffset) {
                this.contextEl.shadowBlur = element.shadowOffset;
                this.contextEl.shadowColor = (style.fillStyle == "transparent") ? "rgba(0,0,0,0)" : style.fillStyle;
            }
            this._drawFrame(scaleStyle, location, style);
            if (this.scaleEl[this.scaleIndex].type == "thermometer" && !this.isScaleModified) {
                this._modifyWidth();
                this.isScaleModified = true;
            }
            if (this.contextEl.getImageData)
                this.scaleImage = this.contextEl.getImageData(0, 0, this.Model.width, this.Model.height);
        },
		/**
         * To set Ticks Cordinates
		 * @private
         */
        _setTicksCordinates: function (element, index) {
            var location, style, staticPosition, height, tempVal, lineChangePosition, interval;
            if (this.scaleEl[this.scaleIndex].majorIntervalValue > this.scaleEl[this.scaleIndex].minorIntervalValue) {
                interval = element.type == "majorinterval" ? this.scaleEl[this.scaleIndex].majorIntervalValue : this.scaleEl[this.scaleIndex].minorIntervalValue;
                if (element.placement == "near") {
                    staticPosition = this.Model.orientation == "Vertical" ? this.scaleStartX[this.scaleIndex] : this.scaleStartY[this.scaleIndex];
                }
                else if (element.placement == "far") {
                    staticPosition = this.Model.orientation == "Vertical" ? this.scaleStartX[this.scaleIndex] + this.scaleEl[this.scaleIndex].width : this.scaleStartY[this.scaleIndex] + this.scaleEl[this.scaleIndex].width;
                }
                else if (element.placement == "center") {
                    staticPosition = this.Model.orientation == "Vertical" ? this.scaleStartX[this.scaleIndex] + this.scaleEl[this.scaleIndex].width / 2 : this.scaleStartY[this.scaleIndex] + this.scaleEl[this.scaleIndex].width / 2;
                }
                height = element.placement == "near" ? -element.height : element.height;
                for (var value = this.scaleEl[this.scaleIndex].maximum; value >= this.scaleEl[this.scaleIndex].minimum; value -= interval) {
                    if (interval == this.scaleEl[this.scaleIndex].minorIntervalValue && value % this.scaleEl[this.scaleIndex].majorIntervalValue != 0 || interval == this.scaleEl[this.scaleIndex].majorIntervalValue) {
                        lineChangePosition = this._getClockwiseLinePosition(value);
                        this.location = {
                            "lineChangePosition": lineChangePosition + (this.Model.orientation == "Horizontal" ? element.distanceFromScale.x : (element.distanceFromScale.y)),
                            "lineStaticPosition": staticPosition + (this.Model.orientation == "Horizontal" ? element.distanceFromScale.y : (element.distanceFromScale.x))
                        };
                        this.style = {
                            "lineHeight": height,
                            "angle": this.Model.orientation == "Vertical" ? element.angle : element.angle + 270,
                            "tickShape": element.TickShape,
                            "strokeStyle": element.color ? ((element.color == "transparent") ? "rgba(0,0,0,0)" : this._getColor(element, element.color)) : ((this.Model.tickColor == "transparent") ? "rgba(0,0,0,0)" : this._getColor(element, this.Model.tickColor)),
                            "lineWidth": element.width
                        }
                        if (this.Model.drawTicks)
                            this._onDrawTicks(this.Model.orientation == "Vertical" ? element.angle : element.angle + 270, value);
                        this._drawTickMark(this.location, this.style);
                    }
                }

                if (this.contextEl.getImageData)
                    this.tickImage = this.contextEl.getImageData(0, 0, this.Model.width, this.Model.height);
            }
        },
		/**
         * To draw Ticks Mark
		 * @private
         */
        _drawTickMark: function (location, style) {
            var length = this.Model.orientation == "Vertical" ? style.lineHeight - location.lineStaticPosition : location.lineStaticPosition;
            this.contextEl.beginPath();
            this.contextEl.save();
            this.contextEl.lineWidth = style.lineWidth;
            this.contextEl.strokeStyle = style.strokeStyle;
            if (this.Model.orientation == "Vertical")
                this.contextEl.translate(location.lineStaticPosition, location.lineChangePosition);
            else
                this.contextEl.translate(location.lineChangePosition, location.lineStaticPosition);
            this.contextEl.moveTo(0, 0);
            if (this.scaleEl[this.scaleIndex].direction == "clockwise")
                this.contextEl.rotate((Math.PI * (style.angle / 180)));
            else
                this.contextEl.rotate(-(Math.PI * (style.angle / 180)));
            this.contextEl.lineTo(style.lineHeight, 0);
            this.contextEl.stroke();
            this.contextEl.restore();
            this.contextEl.closePath();
        },
		/**
         * To manupulate decimal values
		 * @private
         */
        _addDecimal: function (_value, _interval) {
            var value = _value.toString();
            var interval = _interval.toString();
            var vDecimal;
            var iDecimal;
            if (value.indexOf('.') > -1)
                vDecimal = value.length - value.indexOf('.') - 1;
            else
                vDecimal = 0;
            if (interval.indexOf('.') > -1)
                iDecimal = interval.length - interval.indexOf('.') - 1;
            else
                iDecimal = 0;
            var Decimal = vDecimal > iDecimal ? vDecimal : iDecimal
            var correctValue = (_value * Math.pow(10, Decimal) + _interval * Math.pow(10, Decimal)) / Math.pow(10, Decimal);
            return correctValue;
        },
		/**
         * To set label coordinates
		 * @private
         */
        _setLabelCordinates: function (element, index) {

            var location, style, xDistanceFromScale, minimumMeet, yDistanceFromScale, staticPosition, tempVal, height, lineChangePosition, textValue, labelLocation, labelStyle, interval;
            if (this.scaleEl[this.scaleIndex].majorIntervalValue > this.scaleEl[this.scaleIndex].minorIntervalValue) {

                if (this.Model.orientation == "Vertical") {
                    xDistanceFromScale = element.distanceFromScale.x;
                    yDistanceFromScale = element.distanceFromScale.y;
                }
                else {
                    xDistanceFromScale = element.distanceFromScale.y;
                    yDistanceFromScale = element.distanceFromScale.x;
                }
                interval = element.type == "major" ? this.scaleEl[this.scaleIndex].majorIntervalValue : this.scaleEl[this.scaleIndex].minorIntervalValue;
                if (element.placement == "near") {
                    staticPosition = this.Model.orientation == "Vertical" ? this.scaleStartX[this.scaleIndex] - this.scaleEl[this.scaleIndex].border.width / 2 : this.scaleStartY[this.scaleIndex] - this.scaleEl[this.scaleIndex].border.width - 5;
                    this.contextEl.textAlign = this.Model.orientation == "Vertical" ? "right" : "center";
                }
                else if (element.placement == "far") {
                    staticPosition = this.Model.orientation == "Vertical" ? this.scaleStartX[this.scaleIndex] + this.scaleEl[this.scaleIndex].width + this.scaleEl[this.scaleIndex].border.width / 2 : this.scaleStartY[this.scaleIndex] + this.scaleEl[this.scaleIndex].width + this.scaleEl[this.scaleIndex].border.width + 5;
                    this.contextEl.textAlign = this.Model.orientation == "Vertical" ? "left" : "center"
                }
                else {
                    this.contextEl.textAlign = "center";
                    staticPosition = this.Model.orientation == "Vertical" ? this.scaleStartX[this.scaleIndex] + this.scaleEl[this.scaleIndex].width / 2 + this.scaleEl[this.scaleIndex].border.width / 2 : this.scaleStartY[this.scaleIndex] + this.scaleEl[this.scaleIndex].width / 2 + this.scaleEl[this.scaleIndex].border.width / 2;
                }
                for (var value = this.scaleEl[this.scaleIndex].minimum; value <= this.scaleEl[this.scaleIndex].maximum; value = this._addDecimal(value, interval)) {
                    if (interval == this.scaleEl[this.scaleIndex].minorIntervalValue && value % this.scaleEl[this.scaleIndex].majorIntervalValue != 0 || interval == this.scaleEl[this.scaleIndex].majorIntervalValue) {
                        lineChangePosition = this.scaleEl[this.scaleIndex].direction == "counterclockwise" ? this._getCounterClockwiseLinePosition(value) : this._getClockwiseLinePosition(value);
                        this.labelValue = value;
                        this.location = {
                            "lineChangePosition": lineChangePosition + (yDistanceFromScale),
                            "lineStaticPosition": staticPosition + (xDistanceFromScale)
                        };
                        this.style = {
                            "angle": this.Model.orientation == "Vertical" ? element.angle : element.angle + 270,
                            "fillStyle": element.textColor ? ((element.textColor == "transparent") ? "rgba(0,0,0,0)" : this._getColor(element, element.textColor)) : ((this.Model.labelColor == "transparent") ? "rgba(0,0,0,0)" : this._getColor(element, this.Model.labelColor)), "opacity": isNaN(element.opacity) ? 1 : element.opacity,
                            "font": this._getFontString(this, element.font),
                            "textValue": this.labelValue
                        };

                        if (this.Model.drawLabels)
                            this._onDrawLabels(this.Model.orientation == "Vertical" ? element.angle : element.angle + 270);
                        this._drawLabel(this.location, this.style,false);
                    }
                }
            }

            if (this.contextEl.getImageData)
                this.labelImage = this.contextEl.getImageData(0, 0, this.Model.width, this.Model.height);

        },
		/**
         * To draw Label
		 * @private
         */
        _drawLabel: function (location, style, isCustomLabel) {
            this.contextEl.beginPath();
            this.contextEl.save();
            this.contextEl.textBaseline = "middle";
            this.contextEl.fillStyle = style.fillStyle;
            this.contextEl.font = style.font;
            if (style.opacity)
                this.contextEl.globalAlpha = style.opacity;
            if (this.Model.orientation == "Vertical")
                this.contextEl.translate(location.lineStaticPosition, location.lineChangePosition);
            else
                this.contextEl.translate(location.lineChangePosition, location.lineStaticPosition);
            this.contextEl.moveTo(0, 0);
            if (this.scaleEl[this.scaleIndex].direction == "clockwise")
                this.contextEl.rotate((Math.PI * (style.angle / 180)));
            else
                this.contextEl.rotate(-(Math.PI * (style.angle / 180)));
			if(!ej.isNullOrUndefined(isCustomLabel) && !isCustomLabel){
			    var textPostion = this.model.scales[this.scaleIndex].labels[this.labelIndex].unitTextPlacement;
            if (!ej.isNullOrUndefined(textPostion) && textPostion.toString() == "back")
                style.textValue = style.textValue + this.model.scales[this.scaleIndex].labels[this.labelIndex].unitText;
            else if (!ej.isNullOrUndefined(textPostion) && textPostion.toString() == "front")
                style.textValue = this.model.scales[this.scaleIndex].labels[this.labelIndex].unitText + style.textValue;
            }
            this.contextEl.fillText(style.textValue, 0, 0);
            this.contextEl.fill();
            this.contextEl.restore();

        },
		/**
         * To draw Scale bar pointer
		 * @private
         */
        _drawScaleBarPointer: function (element, index) {
            element.value = element.value > this.scaleEl[this.scaleIndex].maximum ? this.scaleEl[this.scaleIndex].maximum : element.value;
            element.value = element.value < this.scaleEl[this.scaleIndex].minimum ? this.scaleEl[this.scaleIndex].minimum : element.value;
            var grd, location, style, radius, height, width, startY, backgroundColor, startX, gradients;
            gradients = [{ "ColorStop": 0, "Color": (this.Model.pointerGradient1 == "transparent") ? "rgba(0,0,0,0)" : this.Model.pointerGradient1 }, { "ColorStop": 1, "Color": (this.Model.pointerGradient2 == "transparent") ? "rgba(0,0,0,0)" : this.Model.pointerGradient2}];
            radius = this.scaleEl[this.scaleIndex].type == "roundedrectangle" ? 5 : 0;
            if (this.scaleEl[this.scaleIndex].direction == "clockwise") {
                lineYPosition = this._getClockwiseLinePosition(element.value);
                if (this.scaleEl[this.scaleIndex].type == "thermometer" && this.isScaleModified) {
                    this._restoreWidth();
                    this.isModify = true;
                }
                if (this.Model.orientation == "Vertical") {
                    startX = this.scaleStartX[this.scaleIndex] + this.scaleEl[this.scaleIndex].width / 2 - element.width / 2 + element.distanceFromScale;
                    grd = this.contextEl.createLinearGradient(startX, this.scaleStartY[this.scaleIndex], startX + element.width, this.scaleStartY[this.scaleIndex]);
                }
                else {
                    startX = this.scaleStartX[this.scaleIndex];
                    grd = this.contextEl.createLinearGradient(startX, this.scaleStartY[this.scaleIndex] + this.scaleEl[this.scaleIndex].width / 2 - element.width / 2, startX, this.scaleStartY[this.scaleIndex] + this.scaleEl[this.scaleIndex].width / 2 + element.width / 2);
                }
            }
            else {
                lineYPosition = this._getCounterClockwiseLinePosition(element.value);
                if (this.scaleEl[this.scaleIndex].type == "thermometer" && this.isScaleModified) {
                    this._restoreWidth();
                    this.isModify = true;
                }
                if (this.Model.orientation == "Vertical") {
                    startX = this.scaleStartX[this.scaleIndex] + this.scaleEl[this.scaleIndex].width / 2 - element.width / 2 + element.distanceFromScale;
                    grd = this.contextEl.createLinearGradient(startX, this.scaleStartY[this.scaleIndex] + this.scaleEl[this.scaleIndex].length - lineYPosition, startX + element.width, this.scaleStartY[this.scaleIndex] + this.scaleEl[this.scaleIndex].length - lineYPosition);
                }
                else {
                    startX = this.scaleEl[this.scaleIndex].type == "thermometer" ? lineYPosition - this.scaleEl[this.scaleIndex].width / 2 - this.scaleEl[this.scaleIndex].border.width / 2 : lineYPosition - this.scaleEl[this.scaleIndex].border.width / 2;
                    grd = this.contextEl.createLinearGradient(startX, this.scaleStartY[this.scaleIndex] + this.scaleEl[this.scaleIndex].width / 2 - element.width / 2, startX, this.scaleStartY[this.scaleIndex] + this.scaleEl[this.scaleIndex].width / 2 + element.width / 2);
                }
            }
            if (element.backgroundColor)
                backgroundColor = ((element.backgroundColor == "transparent") ? "rgba(0,0,0,0)" : this._getColor(element, element.backgroundColor));
            else if (element.gradients)
                this._setGradientColor(this, grd, element.gradients.ColorInfo);
            else if (this.Model.ScaleInterior)
                this._setGradientColor(this, grd, this.Model.ScaleInterior.ColorInfo);
            else
                this._setGradientColor(this, grd, gradients);
            if (this.Model.orientation == "Vertical") {
                startY = this.scaleEl[this.scaleIndex].direction == "clockwise" ? this.scaleStartY[this.scaleIndex] + this.scaleEl[this.scaleIndex].border.width / 2 : lineYPosition;
                if (this.scaleEl[this.scaleIndex].direction == "counterclockwise" && this.scaleEl[this.scaleIndex].type == "thermometer")
                    startY = startY - this.scaleEl[this.scaleIndex].width / 2;
                height = this.scaleEl[this.scaleIndex].direction == "clockwise" ? lineYPosition - this.scaleStartY[this.scaleIndex] : this.scaleStartY[this.scaleIndex] + this.scaleEl[this.scaleIndex].length - lineYPosition - this.scaleEl[this.scaleIndex].border.width / 2;
                width = element.width;
            }
            else {
                startY = this.scaleStartY[this.scaleIndex] + this.scaleEl[this.scaleIndex].width / 2 - element.width / 2 + this.scaleEl[this.scaleIndex].border.width / 2 + element.distanceFromScale;
                height = element.width;
                width = this.scaleEl[this.scaleIndex].direction == "clockwise" ? lineYPosition - this.scaleStartX : this.scaleStartX[this.scaleIndex] + this.scaleEl[this.scaleIndex].length - lineYPosition;
            }
            this.location = { "startX": startX + this.scaleEl[this.scaleIndex].border.width / 2, "startY": startY };
            this.style = {
                "width": (this.scaleEl[this.scaleIndex].type == "thermometer" && this.Model.orientation == "Horizontal") ? width + height / 2 - this.scaleEl[this.scaleIndex].border.width / 2 : width,
                "lineWidth": element.border.width,
                "radius": radius,
                "topRounded": element.value == this.scaleEl[this.scaleIndex].maximum ? true : false,
                "isStroke": false,
                "isFill": true,
                "height": (this.scaleEl[this.scaleIndex].type == "thermometer" && this.Model.orientation == "Vertical") ? height + width / 2 : height,
                "strokeStyle": element.border.color == null ? ((this.Model.borderColor == "transparent") ? "rgba(0,0,0,0)" : this._getColor(element, this.Model.borderColor)) : ((element.border.color == "transparent") ? "rgba(0,0,0,0)" : this._getColor(element, element.border.color)),
                "fillStyle": element.backgroundColor ? ((element.backgroundColor == "transparent") ? "rgba(0,0,0,0)" : this._getColor(element, element.backgroundColor)) : ((grd == "transparent") ? "rgba(0,0,0,0)" : this._getColor(element, grd)),
                "opacity": isNaN(element.opacity) ? 0.4 : element.opacity,
                "calDistance": (this.scaleEl[this.scaleIndex].type == "thermometer" && this.Model.orientation == "Vertical") ? (this.scaleEl[this.scaleIndex].width - this.barPointerEl[this.barPointerIndex].width) / 2 : 0
            };
            this.value(element.value);
            if (this.Model.drawBarPointers)
                this._onDrawBarPointers(element.value);
            this._drawFrame(this.scaleEl[this.scaleIndex].type, this.location, this.style);
            if (this.contextEl.getImageData)
                this.barPointerImage = this.contextEl.getImageData(0, 0, this.Model.width, this.Model.height);
        },
		/**
         * To draw Marker Pointer
		 * @private
         */
        _drawMarkerPointer: function (element, index) {
            element.value = element.value > this.scaleEl[this.scaleIndex].maximum ? this.scaleEl[this.scaleIndex].maximum : element.value;
            element.value = element.value < this.scaleEl[this.scaleIndex].minimum ? this.scaleEl[this.scaleIndex].minimum : element.value;
            var location, style, startX, startY, radius, linePosition, angle, grd, backgroundColor;
            var gradients = [{ "ColorStop": 0, "Color": (this.Model.pointerGradient1 == "transparent") ? "rgba(0,0,0,0)" : this.Model.pointerGradient1 }, { "ColorStop": 1, "Color": (this.Model.pointerGradient2 == "transparent") ? "rgba(0,0,0,0)" : this.Model.pointerGradient2}];
            this.markerPlacement = element.placement;
            radius = Math.sqrt(element.width * element.width + element.length * element.length) / 2;
            if (this.scaleEl[this.scaleIndex].type == "thermometer" && this.isModify)
                this._modifyWidth();
            if (this.Model.orientation == "Vertical") {
                if (this.markerPlacement == "far") {
                    startX = this.scaleStartX[this.scaleIndex] + this.scaleEl[this.scaleIndex].width + this.scaleEl[this.scaleIndex].border.width / 2 + element.distanceFromScale;
                    angle = 0;
                }
                if (this.markerPlacement == "near") {
                    if (element.type == "star")
                        startX = this.scaleStartX[this.scaleIndex] + element.distanceFromScale - element.width;
                    else
                        startX = this.scaleStartX[this.scaleIndex] + element.distanceFromScale;
                    angle = 180;
                }
                if (this.markerPlacement == "center") {
                    if (element.type == "circle")
                        startX = this.scaleStartX[this.scaleIndex] + this.scaleEl[this.scaleIndex].width / 2 - element.width / 2 + element.distanceFromScale + (element.border.width);
                    else
                        startX = this.scaleStartX[this.scaleIndex] + this.scaleEl[this.scaleIndex].width / 2 - element.width / 2 + element.distanceFromScale;
                    angle = 0;
                }
            }
            else {
                if (this.markerPlacement == "far") {
                    startX = this.scaleStartY[this.scaleIndex] + this.scaleEl[this.scaleIndex].width + this.scaleEl[this.scaleIndex].border.width / 2 + element.distanceFromScale;
                    angle = 90;
                }
                if (this.markerPlacement == "near") {
                    if (element.type == "star")
                        startX = this.scaleStartY[this.scaleIndex] - this.scaleEl[this.scaleIndex].border.width / 2 + element.distanceFromScale - element.length;
                    else
                        startX = this.scaleStartY[this.scaleIndex] - this.scaleEl[this.scaleIndex].border.width / 2 + element.distanceFromScale;
                    angle = 270;
                }
                if (this.markerPlacement == "center") {
                    if (element.type == "circle")
                        startX = this.scaleStartY[this.scaleIndex] + this.scaleEl[this.scaleIndex].width / 2 - element.length / 2 + element.distanceFromScale + (element.border.width);
                    else
                        startX = this.scaleStartY[this.scaleIndex] + this.scaleEl[this.scaleIndex].width / 2 - element.length / 2 + element.distanceFromScale;
                    angle = 90;
                }
            }
            linePosition = this.scaleEl[this.scaleIndex].direction == "clockwise" ? this._getClockwiseLinePosition(element.value) : this._getCounterClockwiseLinePosition(element.value);
            if (element.type == "star") {
                if (this.Model.orientation == "Vertical") {
                    grd = this.contextEl.createLinearGradient(startX, this.scaleStartY[this.scaleIndex], startX + element.width, this.scaleStartY[this.scaleIndex]);
                    startY = linePosition - element.length / 3;
                }
                else {
                    grd = this.contextEl.createLinearGradient(linePosition, startX, linePosition, startX + element.length);
                    startY = linePosition - element.width / 2;
                }
            }
            else {
                grd = this.contextEl.createLinearGradient(0, 0, element.width, 0);
                startY = linePosition;
            }
            if (element.type == "roundedrectangle") {
                if (this.Model.orientation == "Vertical" && this.markerPlacement == "near")
                    startY += element.length;
                else if (this.Model.orientation == "Horizontal") {
                    if (this.markerPlacement == "near")
                        startY -= (element.width);
                    startX += element.width / 2;
                }
            }
            if (element.backgroundColor)
                backgroundColor = ((element.backgroundColor == "transparent") ? "rgba(0,0,0,0)" : this._getColor(element, element.backgroundColor));
            else if (element.gradients)
                this._setGradientColor(this, grd, element.gradients.ColorInfo);
            else if (this.Model.PointerInterior)
                this._setGradientColor(this, grd, this.Model.PointerInterior.ColorInfo);
            else
                this._setGradientColor(this, grd, gradients);

            this.location = { "startX": this.Model.orientation == "Vertical" ? startX : startY, "startY": this.Model.orientation == "Vertical" ? startY : startX };
            this.style = {
                "width": element.width,
                "radius": element.type == "rectangle" ? 0 : radius,
                "height": element.length,
                "lineWidth": element.border.width,
                "isFill": true,
                "isStroke": true,
                "angle": angle,
                "strokeStyle": ((element.border.color == "transparent") ? "rgba(0,0,0,0)" : this._getColor(element, element.border.color)),
                "markerPlacement": this.markerPlacement,
                "opacity": isNaN(element.opacity) ? 0.4 : element.opacity,
                "fillStyle": element.backgroundColor ? ((element.backgroundColor == "transparent") ? "rgba(0,0,0,0)" : this._getColor(element, element.backgroundColor)) : ((grd == "transparent") ? "rgba(0,0,0,0)" : this._getColor(element, grd))
            };
            this.value(element.value);
            if (this.Model.drawMarkerPointers)
                this._onDrawMarkerPointers(angle, element.value);
            if (element.type == "roundedrectangle")
                this.style.radius = 5;
            this._drawMarkerType(element.type, this.location, this.style);
            if (this.scaleEl[this.scaleIndex].type == "thermometer" && this.isModify) {
                this._restoreWidth();
                this.isScaleModified = false;
            }
            if (this.contextEl.getImageData)
                this.markerPointerImage = this.contextEl.getImageData(0, 0, this.Model.width, this.Model.height);
        },

        _drawMarkerType: function (type, location, style) {
            switch (type) {
                case "rectangle":
                    this._drawRectangle(location, style, this);
                    break;
                case "triangle":
                    this._drawTriangle(location, style, this);
                    break;
                case "ellipse":
                    this._drawEllipse(location, style, this);
                    break;
                case "diamond":
                    this._drawDiamond(location, style, this);
                    break;
                case "pentagon":
                    this._drawPentagon(location, style, this);
                    break;
                case "circle":
                    this._drawCircle(location, style, this);
                    break;
                case "slider":
                    this._drawSlider(location, style, this);
                    break;
                case "star":
                    this._drawStar(location, style, this);
                    break;
                case "pointer":
                    this._drawPointer(location, style, this);
                    break;
                case "wedge":
                    this._drawWedge(location, style, this);
                    break;
                case "trapezoid":
                    this._drawTrapezoid(location, style, this);
                    break;
                case "roundedrectangle":
                    this._drawRoundedRectangle(location, style, this);
                    break;
            }
        },

		/**
         * To draw Range
		 * @private
         */
        _drawRange: function (element) {
	    element.startValue = element.startValue < this.scaleEl[this.scaleIndex].minimum ? this.scaleEl[this.scaleIndex].minimum : element.startValue;
            element.endValue = element.endValue > this.scaleEl[this.scaleIndex].maximum ? this.scaleEl[this.scaleIndex].maximum:element.endValue;
            this.rangePosition = element.placement;
            var startLinePosition, endLinePosition, location, style, startX, startY, grd, backgroundColor;
            var gradients = [{ "ColorStop": 0, "Color": (this.Model.pointerGradient1 == "transparent") ? "rgba(0,0,0,0)" : this.Model.pointerGradient1 }, { "ColorStop": 1, "Color": (this.Model.pointerGradient2 == "transparent") ? "rgba(0,0,0,0)" : this.Model.pointerGradient2}];
            startLinePosition = (this.scaleEl[this.scaleIndex].direction == "clockwise") ? this._getClockwiseLinePosition(element.startValue) : this._getCounterClockwiseLinePosition(element.startValue);
            endLinePosition = (this.scaleEl[this.scaleIndex].direction == "clockwise") ? this._getClockwiseLinePosition(element.endValue) : this._getCounterClockwiseLinePosition(element.endValue);
            if (this.Model.orientation == "Vertical") {
                if (element.placement == "far")
                    startX = this.scaleStartX[this.scaleIndex] + element.distanceFromScale + this.scaleEl[this.scaleIndex].width + this.scaleEl[this.scaleIndex].border.width;
                if (element.placement == "near")
                    startX = this.scaleStartX[this.scaleIndex] + element.distanceFromScale;
                if (element.placement == "center")
                    startX = (element.startWidth > element.endWidth) ? this.scaleStartX[this.scaleIndex] + element.distanceFromScale + this.scaleEl[this.scaleIndex].width / 2 - element.startWidth / 2 : this.scaleStartX[this.scaleIndex] + element.distanceFromScale + this.scaleEl[this.scaleIndex].width / 2 - element.endWidth / 2;
                grd = this.contextEl.createLinearGradient(endLinePosition, endLinePosition, endLinePosition, startLinePosition);
                this.location = { "startX": startX, "startY": startLinePosition, "endY": endLinePosition };
            }
            else {
                if (element.placement == "far")
                    startY = this.scaleStartY[this.scaleIndex] + this.scaleEl[this.scaleIndex].width + element.distanceFromScale + this.scaleEl[this.scaleIndex].border.width;
                if (element.placement == "near")
                    startY = this.scaleStartY[this.scaleIndex] + element.distanceFromScale;
                if (element.placement == "center")
                    startY = (element.startWidth > element.endWidth) ? this.scaleStartY[this.scaleIndex] + element.distanceFromScale + this.scaleEl[this.scaleIndex].width / 2 - element.startWidth / 2 : this.scaleStartY[this.scaleIndex] + element.distanceFromScale + this.scaleEl[this.scaleIndex].width / 2 - element.endWidth / 2;
                grd = this.contextEl.createLinearGradient(endLinePosition, startY, startLinePosition, startY);
                this.location = { "startX": startLinePosition, "startY": startY, "endX": endLinePosition };
            }

            if (element.backgroundColor)
                backgroundColor = ((element.backgroundColor == "transparent") ? "rgba(0,0,0,0)" : this._getColor(element, element.backgroundColor));
            else if (element.gradients)
                this._setGradientColor(this, grd, element.gradients.ColorInfo);
            else if (this.Model.RangeInterior)
                this._setGradientColor(this, grd, this.Model.RangeInterior.ColorInfo);
            else
                this._setGradientColor(this, grd, gradients);

            this.style = {
                "startWidth": element.startWidth, "lineWidth": element.border.width,
                "isStroke": true, "isFill": true, "opacity": isNaN(element.opacity) ? 0.4 : element.opacity,
                "endWidth": element.endWidth,
                "fillStyle": element.backgroundColor ? ((backgroundColor == "transparent") ? "rgba(0,0,0,0)" : this._getColor(element, backgroundColor)) : ((grd == "transparent") ? "rgba(0,0,0,0)" : this._getColor(element, grd)),
                "strokeStyle": element.border.color ? ((element.border.color == "transparent") ? "rgba(0,0,0,0)" : this._getColor(element, element.border.color)) : ((this.Model.borderColor == "transparent") ? "rgba(0,0,0,0)" : this._getColor(element, this.Model.borderColor))
            };
            if (this.Model.drawRange)
                this._onDrawRange();
            this._drawRangeBar(this.location, this.style);
            if (this.contextEl.getImageData)
                this.rangeImage = this.contextEl.getImageData(0, 0, this.Model.width, this.Model.height);
        },
		/**
         * To draw Range bar
		 * @private
         */
        _drawRangeBar: function (location, style) {
            this._contextOpenPath(style, this);
            if (this.Model.orientation == "Vertical") {
                this.contextEl.moveTo(location.startX, location.startY);
                this.contextEl.lineTo(location.startX, location.endY);
                if (this.rangePosition == "near") {
                    this.contextEl.lineTo(location.startX - style.endWidth, location.endY);
                    this.contextEl.lineTo(location.startX - style.startWidth, location.startY);
                }
                else {
                    this.contextEl.lineTo(location.startX + style.endWidth, location.endY);
                    this.contextEl.lineTo(location.startX + style.startWidth, location.startY);
                }
            }
            else {
                this.contextEl.moveTo(location.startX, location.startY);
                this.contextEl.lineTo(location.endX, location.startY);
                if (this.rangePosition == "near") {
                    this.contextEl.lineTo(location.endX, location.startY - style.endWidth);
                    this.contextEl.lineTo(location.startX, location.startY - style.startWidth);
                }
                else {
                    this.contextEl.lineTo(location.endX, location.startY + style.endWidth);
                    this.contextEl.lineTo(location.startX, location.startY + style.startWidth);
                }
            }
            this._contextClosePath(style, this);
        },
		/**
         * To set Custom Label Cordinates
		 * @private
         */
        _setCustomLabelCordinates: function (index, element) {
            var location, style, startX, startY;
            this.contextEl.textAlign = "center";
            if (this.Model.orientation == "Vertical") {
                startX = (this.bounds.width) * ((element.position.x) / 100);
                startY = (this.bounds.height) * ((element.position.y) / 100);
            }
            else {
                startX = (this.bounds.width) * ((element.position.x) / 100);
                startY = (this.bounds.height) * ((element.position.y) / 100);
            }
            this.location = { "lineStaticPosition": this.Model.orientation == "Vertical" ? startX : startY, "lineChangePosition": this.Model.orientation == "Vertical" ? startY : startX }
            this.style = { "angle": element.textAngle, "textValue": element.value, "fillStyle": element.color ? ((element.color == "transparent") ? "rgba(0,0,0,0)" : this._getColor(element, element.color)) : ((this.Model.labelColor == "transparent") ? "rgba(0,0,0,0)" : this._getColor(element, this.Model.labelColor)), "font": element.font };
            if (this.Model.drawCustomLabel)
                this._onDrawCustomLabel();
            this._drawLabel(this.location, this.style,true);

            if (this.contextEl.getImageData)
                this.customLabelImage = this.contextEl.getImageData(0, 0, this.Model.width, this.Model.height);
        },
		/**
         * To draw Indicator
		 * @private
         */
        _drawIndicator: function (index, element) {
            var self = this, xlocation, ylocation, txtLocation, txtLocationX, txtLocationY, isInStateRange = false;
            xlocation = (this.bounds.width - 2 * element.width) * ((element.position.x) / 100);
            ylocation = (this.bounds.height - 2 * element.height) * ((element.position.y) / 100);
            txtLocation = { "x": (this.bounds.width) * ((element.textLocation.x) / 100), "y": (this.bounds.height) * ((element.textLocation.y) / 100) };
            self.location = { "startX": element.type == "circle" ? xlocation + element.width : xlocation, "textLocation": txtLocation, "startY": element.type == "circle" ? ylocation + element.height : ylocation, "startAngle": 0, "endAngle": 2 * Math.PI };
            self.style = {
                "radius": element.type == "roundedrectangle" ? 2 : 0,
                "strokeStyle": element.border.color ? ((element.border.color == "transparent") ? "rgba(0,0,0,0)" : this._getColor(element, element.border.color)) : this._getColor(element, "#FFFFFF"),
                "angle": 0,
                "circleRadius": (element.height + element.width) / 2,
                "height": element.height,
                "width": element.width,
                "lineWidth": element.border.width,
                "fillStyle": element.backgroundColor ? ((element.backgroundColor == "transparent") ? "rgba(0,0,0,0)" : this._getColor(element, element.backgroundColor)) : this._getColor(element, "#FFFFFF"),
                "isStroke": true,
                "isFill": true,
                "indicatorText": null,
                "textColor": null,
                "font": null,
                "counterClockwise": false
            };
            if (this.Model.drawIndicators)
                this._onDrawIndicators(self.style, self.location);
            if (element.stateRanges != null) {
                $.each(element.stateRanges, function (index, srEl) {//srEl = StateRange Element
                    if (self.markerPointerEl[self.markerPointerIndex].value >= srEl.startValue && self.markerPointerEl[self.markerPointerIndex].value <= srEl.endValue) {
                        isInStateRange = true;
                        if (!ej.isNullOrUndefined(srEl.text) && srEl.text.length > 0) {
                            self.style.indicatorText = srEl.text;
                            self.style.textColor = ((srEl.textColor == "transparent") ? "rgba(0,0,0,0)" : self._getColor(element, srEl.textColor));
                            self.style.font = self._getFontString(self, element.font);
                        }
                        if (element.type != "text") {
                            self.style.strokeStyle = ((srEl.borderColor == "transparent") ? "rgba(0,0,0,0)" : self._getColor(element, srEl.borderColor));
                            self.style.fillStyle = ((srEl.backgroundColor == "transparent") ? "rgba(0,0,0,0)" : self._getColor(element, srEl.backgroundColor));
                            self._drawFrame(element.type, self.location, self.style, self);
                        }
                        else if (element.type == "text")
                            self._drawText(self.location, self.style);
                    }
                });
                }
            if (!isInStateRange && element.type != "text")
                this._drawFrame(element.type, self.location, self.style, self);
            if (this.contextEl.getImageData)
                this.indicatorImage = this.contextEl.getImageData(0, 0, this.Model.width, this.Model.height);
        },

        _drawFrame: function (type, location, style) {
            switch (type) {
                case "circle":
                    this._drawFrameCircle(location, style, this);
                    break;
                case "rectangle":
                    this._drawFrameRectangle(location, style, this);
                    break;
                case "roundedrectangle":
                    this._drawFrameRoundedRectangle(location, style, this);
                    break;
                case "thermometer":
                    this._drawFrameThermometer(location,style,this) ;
                    break;
            }
        },
		/**
         * To draw the text in the gauge
		 * @private
         */
        _drawText: function (location, style) {
            this.contextEl.beginPath();
            this.contextEl.textAlign = "center";
            this.contextEl.fillStyle = ((style.textColor == "transparent") ? "rgba(0,0,0,0)" : style.textColor);
            this.contextEl.font = style.font;
            this.contextEl.fillText(style.indicatorText, location.textLocation.x, location.textLocation.y);
            this.contextEl.closePath();
        },
		/**
         * To draw the triangle
		 * @private
         */
        _drawTriangle: function (location, style, element) {
            style = this._setPointerDimension(style, element);
            this._contextOpenPath(style, element);
            element.contextEl.translate(location.startX, location.startY);
            this._setContextRotation(style, element);
            element.contextEl.moveTo(0, 0);
            element.contextEl.lineTo(style.width, -style.height / 2);
            element.contextEl.lineTo(style.width, style.height / 2);
            this._contextClosePath(style, element);
        },
		/**
         * To draw the Pointer
		 * @private
         */
        _drawPointer: function (location, style, element) {
            style = this._setPointerDimension(style, element);
            this._contextOpenPath(style, element);
            element.contextEl.translate(location.startX, location.startY);
            this._setContextRotation(style, element);
            element.contextEl.moveTo(style.width, style.height / 4);
            element.contextEl.lineTo(style.width, -style.height / 4);
            element.contextEl.lineTo(style.width / 2, -style.height / 4);
            element.contextEl.lineTo(style.width / 2, -style.height / 2);
            element.contextEl.lineTo(0, 0);
            element.contextEl.lineTo(style.width / 2, style.height / 2);
            element.contextEl.lineTo(style.width / 2, style.height / 4);
            this._contextClosePath(style, element);
        },
		/**
         * To draw the Wedge
		 * @private
         */
        _drawWedge: function (location, style, element) {
            style = this._setPointerDimension(style, element);
            this._contextOpenPath(style, element);
            element.contextEl.translate(location.startX, location.startY);
            this._setContextRotation(style, element);
            element.contextEl.moveTo(0, 0);
            element.contextEl.lineTo(style.width, -style.height / 2);
            element.contextEl.lineTo(3 * style.width / 4, 0);
            element.contextEl.lineTo(style.width, style.height / 2);
            this._contextClosePath(style, element);
        },
		/**
         * To draw the Slider
		 * @private
         */
        _drawSlider: function (location, style, element) {
            style = this._setPointerDimension(style, element);
            this._contextOpenPath(style, element);
            element.contextEl.translate(location.startX, location.startY);
            this._setContextRotation(style, element);
            element.contextEl.moveTo(0, 0);
            element.contextEl.lineTo(style.width / 4, -style.height / 2);
            element.contextEl.lineTo(style.width, -style.height / 2);
            element.contextEl.lineTo(style.width, style.height / 2);
            element.contextEl.lineTo(style.width / 4, style.height / 2);
            this._contextClosePath(style, element);
        },
		/**
         * To draw the Star
		 * @private
         */
        _drawStar: function (location, style, element) {
            this._contextOpenPath(style, element);
            if (element.Model.orientation == "Horizontal" && element.markerPlacement == "near") {
                element.contextEl.moveTo(location.startX + style.width - (style.width / 6), location.startY); // 1
                element.contextEl.lineTo(location.startX, location.startY + style.height - style.height / 3);
                element.contextEl.lineTo(location.startX + style.width, location.startY + style.height - style.height / 3);
                element.contextEl.lineTo(location.startX + style.width / 6, location.startY);
                element.contextEl.lineTo(location.startX + style.width / 2, location.startY + style.height);
            }
            else {
                element.contextEl.moveTo(location.startX + (style.width / 6), location.startY + style.height); // 1
                element.contextEl.lineTo(location.startX + style.width, location.startY + (style.height / 3)); // 2
                element.contextEl.lineTo(location.startX, location.startY + (style.height / 3)); // 3
                element.contextEl.lineTo(location.startX + style.width - (style.width / 6), location.startY + style.height); // 4
                element.contextEl.lineTo(location.startX + style.width / 2, location.startY); // 5
            }
            this._contextClosePath(style, element);
        },
		/**
         * To draw the Pentagon
		 * @private
         */
        _drawPentagon: function (location, style, element) {
            style = this._setPointerDimension(style, element);
            this._contextOpenPath(style, element);
            element.contextEl.translate(location.startX, location.startY);
            this._setContextRotation(style, element);
            element.contextEl.moveTo(0, 0);
            element.contextEl.lineTo(style.width / 3, -style.height / 2);
            element.contextEl.lineTo(style.width, -style.height / 4);
            element.contextEl.lineTo(style.width, style.height / 4);
            element.contextEl.lineTo(style.width / 3, style.height / 2);
            this._contextClosePath(style, element);
        },
		/**
         * To draw the Diamond
		 * @private
         */
        _drawDiamond: function (location, style, element) {
            style = this._setPointerDimension(style, element);
            this._contextOpenPath(style, element);
            element.contextEl.translate(location.startX, location.startY);
            this._setContextRotation(style, element);
            element.contextEl.moveTo(0, 0);
            element.contextEl.lineTo(style.width / 2, -style.height / 2);
            element.contextEl.lineTo(style.width, 0);
            element.contextEl.lineTo(style.width / 2, style.height / 2);
            element.contextEl.lineTo(0, 0);
            this._contextClosePath(style, element);
        },
		/**
         * To draw the Circle
		 * @private
         */
        _drawCircle: function (location, style, element) {
            var radius = Math.sqrt(style.height * style.height + style.width * style.width) / 2;
            style = this._setPointerDimension(style, element);
            this._contextOpenPath(style, element);
            element.contextEl.translate(location.startX, location.startY);
            this._setContextRotation(style, element);
            element.contextEl.arc(radius / 2, 0, radius / 2, 0, Math.PI * 2, true);
            this._contextClosePath(style, element);
        },
		/**
         * To draw the Trapezoid
		 * @private
         */
        _drawTrapezoid: function (location, style, element) {
            style = this._setPointerDimension(style, element);
            this._contextOpenPath(style, element);
            element.contextEl.translate(location.startX, location.startY);
            this._setContextRotation(style, element);
            element.contextEl.moveTo(0, 0);
            element.contextEl.lineTo(0, -style.height / 4);
            element.contextEl.lineTo(style.width, -style.height / 2);
            element.contextEl.lineTo(style.width, style.height / 2);
            element.contextEl.lineTo(0, style.height / 4);
            this._contextClosePath(style, element);
        },
		/**
         * To draw the Rectangle
		 * @private
         */
        _drawRectangle: function (location, style, element) {
            style = this._setPointerDimension(style, element);
            this._contextOpenPath(style, element);
            element.contextEl.translate(location.startX, location.startY);
            this._setContextRotation(style, element);
            element.contextEl.moveTo(0, 0);
            element.contextEl.lineTo(0, -style.height / 2);
            element.contextEl.lineTo(style.width, -style.height / 2);
            element.contextEl.lineTo(style.width, style.height / 2);
            element.contextEl.lineTo(0, style.height / 2);
            this._contextClosePath(style, element);
        },
		/**
         * To draw the Rounded Rectangle
		 * @private
         */
        _drawRoundedRectangle: function (location, style, element) {
            style = this._setPointerDimension(style, element);
            this._contextOpenPath(style, element);
            element.contextEl.translate(location.startX, location.startY - style.height / 2);
            this._setContextRotation(style, element);
            element.contextEl.moveTo(style.radius, 0);
            element.contextEl.lineTo(style.width - style.radius, 0);
            element.contextEl.quadraticCurveTo(style.width, 0, style.width, style.radius);
            element.contextEl.lineTo(style.width, style.height - style.radius);
            element.contextEl.quadraticCurveTo(style.width, style.height, style.width - style.radius, style.height);
            element.contextEl.lineTo(style.radius, style.height);
            element.contextEl.quadraticCurveTo(0, style.height, 0, style.height - style.radius);
            element.contextEl.lineTo(0, style.radius);
            element.contextEl.quadraticCurveTo(0, 0, style.radius, 0);
            this._contextClosePath(style, element);
        },
		/**
         * To draw the Custom image 
		 * @private
         */
        _drawCustomImage: function (element, imageUrl) {
            var image = new Image();
            var self = this;
            $(image).load(function () {
                element.contextEl.drawImage(this, 0, 0, element.Model.width, element.Model.height);
                if (element.Model.scales != null)
                    element._drawScales();
                if (element.Model.Items != null)
                    element._renderItems();
            }).attr('src', imageUrl);
        },
		/**
         * To draw the Ellipse
		 * @private
         */
        _drawEllipse: function (location, style, element) {
            var radius = Math.sqrt(style.height * style.height + style.width * style.width) / 2;
            style = this._setPointerDimension(style, element);
            this._contextOpenPath(style, element);
            element.contextEl.translate(location.startX, location.startY);
            this._setContextRotation(style, element);
            element.contextEl.scale(2, 1);
            element.contextEl.arc(radius / 2, 0, radius / 2, 0, Math.PI * 2, true);
            this._contextClosePath(style, element);
        },
        //Client-Side Methods
		/**
         * To get Indicator Image
		 * @private
         */	
        _getIndicatorImage: function () {
            if (this.pointerImage)
                return this.pointerImage;
            else
                this._getMarkerPointerImage();
        },
		/**
         * To get Bar Pointer Image
		 * @private
         */	
        _getBarPointerImage: function () {
            if (this.customLabelImage)
                return this.customLabelImage;
            else
                return this._getCustomLabelImage();
        },
		/**
         * To get Marker Pointer Image
		 * @private
         */	
        _getMarkerPointerImage: function () {
            if (this.barPointerImage)
                return this.barPointerImage;
            else
                return this._getCustomLabelImage();
        },
		/**
         * To get Custom Label Image
		 * @private
         */	
        _getCustomLabelImage: function () {
            if (this.rangeImage)
                return this.rangeImage;
            else
                return this._getRangeImage();
        },
		/**
         * To get Range Image
		 * @private
         */	
        _getRangeImage: function () {
            if (this.labelImage)
                return this.labelImage;
            else
                return this._getLabelImage();
        },
		/**
         * To get Label Image
		 * @private
         */	
        _getLabelImage: function () {

            if (this.tickImage)
                return this.tickImage;
            else
                return this._getTickImage();
        },
		/**
         * To get Tick Image
		 * @private
         */	
        _getTickImage: function () {
            if (this.scaleImage)
                return this.scaleImage;
            else
                return this.outerImage;
        },
		/**
        * To get the Indicator Value in number
		* @return number
		* @param {number} argument.scaleIndex scaleIndex value for the Gauge 
		* @param {number}  argument.indicatorIndex indicatorIndex value for the Gauge
		* @example 
		* &lt;div id="LinearGauge1"&gt;&lt;/div&gt; <br>  
		* &lt;script&gt;
        * $("#LinearGauge1").ejLinearGauge();
		* var LinearGaugeObj = $("#LinearGauge1").data("ejLinearGauge");
		* LinearGaugeObj.getIndicatorValue(0,0);
		* &lt;/script&gt;
		*@memberof ejLinearGauge
		* @instance
        */
        getIndicatorValue: function (scaleIndex, indicatorIndex) {
            if (scaleIndex < this.Model.scales.length && indicatorIndex < this.Model.scales[scaleIndex].indicators.length)
                return this.scaleEl[scaleIndex].indicators[indicatorIndex].value;
        },
		/**
        * To set IndicatorValue 
		* @return jQuery
		* @param {number} argument.scaleIndex scaleIndex value for the Gauge 
		* @param {number}  argument.indicatorIndex indicatorIndex value for the Gauge
		* @param {number} arguemnt.value for Gauge
		* @example 
		* &lt;div id="LinearGauge1"&gt;&lt;/div&gt; <br>  
		* &lt;script&gt;
        * $("#LinearGauge1").ejLinearGauge({
        * value: 50, scales: [{showBarPointers: true, showIndicators: true, length: 310, indicators: [{
        * height: 30,type: "Rectangle",width: 30,
        * position: { x: 0,y: 0},
        * stateRanges: [{endValue: 60,startValue: 40,backgroundColor: "Green",borderColor: "Red"}],
        * value:50}]}]});
		* var LinearGaugeObj = $("#LinearGauge1").data("ejLinearGauge");
		* LinearGaugeObj.setIndicatorValue(0,0,30);
		* &lt;/script&gt;
		*@memberof ejLinearGauge
		* @instance
        */
        setIndicatorValue: function (scaleIndex, indicatorIndex, value) {
            if (scaleIndex < this.Model.scales.length && indicatorIndex < this.Model.scales[scaleIndex].indicators.length && value != null) {
                this.scaleEl[scaleIndex].indicators[indicatorIndex].value = value;
                this.initialize();
            }
        },
		/**
        * To set PointerValue 
		* @return jQuery
		* @param {number} argument.scaleIndex scaleIndex value for the Gauge 
		* @param {number}  argument.pointerIndex pointerIndex value for the Gauge
		* @param {number} arguemnt.value Pointer value for Gauge
		* @example 
		* &lt;div id="LinearGauge1"&gt;&lt;/div&gt; <br>  
		* &lt;script&gt;
        * $("#LinearGauge1").ejLinearGauge();
		* var LinearGaugeObj = $("#LinearGauge1").data("ejLinearGauge");
		* LinearGaugeObj.setPointerValue(0,0,30);
		* &lt;/script&gt;
		*@memberof ejLinearGauge
		* @instance
        */
        setPointerValue: function (scaleIndex, pointerIndex, value) {
            if (scaleIndex < this.Model.scales.length && pointerIndex < this.Model.scales[scaleIndex].markerPointers.length && value != null) {
            if(value <= this.scaleEl[scaleIndex].maximum && value >= this.scaleEl[scaleIndex].minimum)
                this.scaleEl[scaleIndex].markerPointers[pointerIndex].value = value;
                if (this.contextEl.putImageData) {
                        this.contextEl.putImageData(this._getMarkerPointerImage(), 0, 0);
                        this._setMarkerPointers();
                        this._setIndicators();
                }
                else
                    this.initialize();
            }
        },
		/**
        * To get PointerValue in number
		* @return number
		* @param {number} argument.scaleIndex scaleIndex value for the Gauge 
		* @param {number}  argument.pointerIndex pointerIndex value for the Gauge
		* @example 
		* &lt;div id="LinearGauge1"&gt;&lt;/div&gt; <br>  
		* &lt;script&gt;
        * $("#LinearGauge1").ejLinearGauge();
		* var LinearGaugeObj = $("#LinearGauge1").data("ejLinearGauge");
		* LinearGaugeObj.getPointerValue(0,0);
		* &lt;/script&gt;
        * @memberof ejLinearGauge
		* @instance
        */
        getPointerValue: function (scaleIndex, pointerIndex) {
            if (scaleIndex < this.Model.scales.length && pointerIndex < this.Model.scales[scaleIndex].markerPointers.length) {
                return this.scaleEl[scaleIndex].markerPointers[pointerIndex].value;
            }
        },
		/**
        * To set PointerWidth 
		* @return jQuery
		* @param {number} argument.scaleIndex  scaleIndex value for the Gauge 
		* @param {number}  argument.pointerIndex pointerIndex value for the Gauge
		* @param {number} arguemnt.width Pointer width for Gauge
		* @example 
		* &lt;div id="LinearGauge1"&gt;&lt;/div&gt; <br>  
		* &lt;script&gt;
        * $("#LinearGauge1").ejLinearGauge();
		* var LinearGaugeObj = $("#LinearGauge1").data("ejLinearGauge");
		* LinearGaugeObj.setPointerWidth(0,0,30);
		* &lt;/script&gt;
		*@memberof ejLinearGauge
		* @instance
        */
        setPointerWidth: function (scaleIndex, pointerIndex, width) {
            if (scaleIndex < this.Model.scales.length && pointerIndex < this.Model.scales[scaleIndex].markerPointers.length && width != null) {
                this.scaleEl[scaleIndex].markerPointers[pointerIndex].width = width;
                if (this.contextEl.putImageData) {
                    if (this.scaleEl[this.scaleIndex].type == "thermometer")
                        this.initialize();
                    else {
                        this.contextEl.putImageData(this._getMarkerPointerImage(), 0, 0);
                        this._setMarkerPointers();
                    }
                }
                else
                    this.initialize();
            }
        },
		/**
        * To get PointerWidth in number
		* @return number
		* @param {number} argument.scaleIndex scaleIndex value for the Gauge 
		* @param {number}  argument.pointerIndex pointerIndex value for the Gauge
		* @example 
		* &lt;div id="LinearGauge1"&gt;&lt;/div&gt; <br>  
		* &lt;script&gt;
        * $("#LinearGauge1").ejLinearGauge();
		* var LinearGaugeObj = $("#LinearGauge1").data("ejLinearGauge");
		* LinearGaugeObj.getPointerWidth(0,0);
		* &lt;/script&gt;
		*@memberof ejLinearGauge
		* @instance
        */
        getPointerWidth: function (scaleIndex, pointerIndex) {
            if (scaleIndex < this.Model.scales.length && pointerIndex < this.Model.scales[scaleIndex].markerPointers.length) {
                return this.scaleEl[scaleIndex].markerPointers[pointerIndex].width;
            }
        },
		/**
        * To set PointerHeight 
		* @return jQuery
		* @param {number} argument.scaleIndex scaleIndex value for the Gauge 
		* @param {number}  argument.pointerIndex pointerIndex value for the Gauge
		* @param {number} arguemnt.height for Gauge
		* @example 
		* &lt;div id="LinearGauge1"&gt;&lt;/div&gt; <br>  
		* &lt;script&gt;
        * $("#LinearGauge1").ejLinearGauge();
		* var LinearGaugeObj = $("#LinearGauge1").data("ejLinearGauge");
		* LinearGaugeObj.setPointerHeight(0,0,30);
		* &lt;/script&gt;
		*@memberof ejLinearGauge
		* @instance
        */
        setPointerHeight: function (scaleIndex, pointerIndex, height) {
            if (scaleIndex < this.Model.scales.length && pointerIndex < this.Model.scales[scaleIndex].markerPointers.length && height != null) {
                this.scaleEl[scaleIndex].markerPointers[pointerIndex].length = height;
                if (this.contextEl.putImageData) {
                    if (this.scaleEl[this.scaleIndex].type == "thermometer")
                        this.initialize();
                    else {
                        this.contextEl.putImageData(this._getMarkerPointerImage(), 0, 0);
                        this._setMarkerPointers();
                    }
                }
                else
                    this.initialize();
            }
        },
		/**
        * To get PointerHeight in number
		* @return number
		* @param {number} argument.scaleIndex scaleIndex value for the Gauge 
		* @param {number}  argument.pointerIndex pointerIndex value for the Gauge
		* @example 
		* &lt;div id="LinearGauge1"&gt;&lt;/div&gt; <br>  
		* &lt;script&gt;
        * $("#LinearGauge1").ejLinearGauge();
		* var LinearGaugeObj = $("#LinearGauge1").data("ejLinearGauge");
		* LinearGaugeObj.getPointerHeight(0,0);
		* &lt;/script&gt;
		*@memberof ejLinearGauge
		* @instance
        */
        getPointerHeight: function (scaleIndex, pointerIndex) {
            if (scaleIndex < this.Model.scales.length && pointerIndex < this.Model.scales[scaleIndex].markerPointers.length)
                return this.scaleEl[scaleIndex].markerPointers[pointerIndex].length;
        },
		/**
         * To get color
		 * @private
         */
        _getColor: function (element, option) {
            if (typeof (option) === "string") {
                return option;
            }
            else {
                return ("rgba(" + option.r + ", " + option.g + "," + option.b + ", " + option.a / 255 + ")");
            }
        },
		/**
        * To set setPointerDistanceFromScale 
		* @return jQuery
		* @param {number} argument.scaleIndex scaleIndex value for the Gauge 
		* @param {number}  argument.pointerIndex pointerIndex value for the Gauge
		* @param {number} arguemnt.value for Gauge
		* @example 
		* &lt;div id="LinearGauge1"&gt;&lt;/div&gt; <br>  
		* &lt;script&gt;
        * $("#LinearGauge1").ejLinearGauge();
		* var LinearGaugeObj = $("#LinearGauge1").data("ejLinearGauge");
		* LinearGaugeObj.setPointerDistanceFromScale(0,0,30);
		* &lt;/script&gt;
		*@memberof ejLinearGauge
		* @instance
        */
        setPointerDistanceFromScale: function (scaleIndex, pointerIndex, value) {
            if (scaleIndex < this.Model.scales.length && pointerIndex < this.Model.scales[scaleIndex].markerPointers.length && value != null) {
                this.scaleEl[scaleIndex].markerPointers[pointerIndex].distanceFromScale = value;
                if (this.contextEl.putImageData) {
                    if (this.scaleEl[this.scaleIndex].type == "thermometer")
                        this.initialize();
                    else {
                        this.contextEl.putImageData(this._getMarkerPointerImage(), 0, 0);
                        this._setMarkerPointers();
                    }
                }
                else
                    this.initialize();
            }
        },
		/**
        * To get Pointer Distance From Scale in number
		* @return number
		* @param {number} argument.scaleIndex scaleIndex value for the Gauge 
		* @param {number}  argument.pointerIndex pointerIndex value for the Gauge
		* @example 
		* &lt;div id="LinearGauge1"&gt;&lt;/div&gt; <br>  
		* &lt;script&gt;
        * $("#LinearGauge1").ejLinearGauge();
		* var LinearGaugeObj = $("#LinearGauge1").data("ejLinearGauge");
		* LinearGaugeObj.getPointerDistanceFromScale(0,0);
		* &lt;/script&gt;
		*@memberof ejLinearGauge
		* @instance
        */
        getPointerDistanceFromScale: function (scaleIndex, pointerIndex) {
            if (scaleIndex < this.Model.scales.length && pointerIndex < this.Model.scales[scaleIndex].markerPointers.length)
                return this.scaleEl[scaleIndex].markerPointers[pointerIndex].distanceFromScale;
        },
		/**
        * To set setPointerPlacement 
		* @return jQuery
		* @param {number} argument.scaleIndex  scaleIndex value for the Gauge 
		* @param {number}  argument.pointerIndex pointerIndex value for the Gauge
		* @param {number} arguemnt.value  pointer placement for Gauge
		* @example 
		* &lt;div id="LinearGauge1"&gt;&lt;/div&gt; <br>  
		* &lt;script&gt;
        * $("#LinearGauge1").ejLinearGauge();
		* var LinearGaugeObj = $("#LinearGauge1").data("ejLinearGauge");
		* LinearGaugeObj.setPointerPlacement(0,0,"far");
		* &lt;/script&gt;
		*@memberof ejLinearGauge
		* @instance
        */
        setPointerPlacement: function (scaleIndex, pointerIndex, value) {
            if (scaleIndex < this.Model.scales.length && pointerIndex < this.Model.scales[scaleIndex].markerPointers.length && value != null) {
                this.scaleEl[scaleIndex].markerPointers[pointerIndex].placement = value;
                if (this.contextEl.putImageData) {
                    if (this.scaleEl[this.scaleIndex].type == "thermometer")
                        this.initialize();
                    else {
                        this.contextEl.putImageData(this._getMarkerPointerImage(), 0, 0);
                        this._setMarkerPointers();
                    }
                }
                else
                    this.initialize();
            }
        },
		/**
        * To get Pointer Placement in String
		* @return string
		* @param {number} argument.scaleIndex scaleIndex value for the Gauge 
		* @param {number}  argument.pointerIndex pointerIndex value for the Gauge
		* @example 
		* &lt;div id="LinearGauge1"&gt;&lt;/div&gt; <br>  
		* &lt;script&gt;
        * $("#LinearGauge1").ejLinearGauge();
		* var LinearGaugeObj = $("#LinearGauge1").data("ejLinearGauge");
		* LinearGaugeObj.getPointerPlacement(0,0);
		* &lt;/script&gt;
		*@memberof ejLinearGauge
		* @instance
        */
        getPointerPlacement: function (scaleIndex, pointerIndex) {
            if (scaleIndex < this.Model.scales.length && pointerIndex < this.Model.scales[scaleIndex].markerPointers.length)
                return this.scaleEl[scaleIndex].MarkerPointer[pointerIndex].placement;
        },
		/**
        * To set setMarkerStyle 
		* @return jQuery
		* @param {number} argument.scaleIndex scaleIndex value for the Gauge 
		* @param {number}  argument.pointerIndex pointerIndexvalue for the Gauge
		* @param {string} arguemnt.value marker Style for Gauge
		* @example 
		* &lt;div id="LinearGauge1"&gt;&lt;/div&gt; <br>  
		* &lt;script&gt;
        * $("#LinearGauge1").ejLinearGauge();
		* var LinearGaugeObj = $("#LinearGauge1").data("ejLinearGauge");
		* LinearGaugeObj.setMarkerStyle(0,0,"triangle");
		* &lt;/script&gt;
		*@memberof ejLinearGauge
		* @instance
        */
        setMarkerStyle: function (scaleIndex, pointerIndex, value) {
            if (scaleIndex < this.Model.scales.length && pointerIndex < this.Model.scales[scaleIndex].markerPointers.length && value != null) {
                this.scaleEl[scaleIndex].markerPointers[pointerIndex].type = value;
                if (this.contextEl.putImageData) {
                    if (this.scaleEl[this.scaleIndex].type == "thermometer")
                        this.initialize();
                    else {
                        this.contextEl.putImageData(this._getMarkerPointerImage(), 0, 0);
                        this._setMarkerPointers();
                    }
                }
                else
                    this.initialize();
            }
        },
		/**
        * To get MarkerStyle in number
		* @return number
		* @param {number} argument.scaleIndex vscaleIndex alue for the Gauge 
		* @param {number}  argument.pointerIndex pointerIndex value for the Gauge
		* @example 
		* &lt;div id="LinearGauge1"&gt;&lt;/div&gt; <br>  
		* &lt;script&gt;
        * $("#LinearGauge1").ejLinearGauge();
		* var LinearGaugeObj = $("#LinearGauge1").data("ejLinearGauge");
		* LinearGaugeObj.getMarkerStyle(0,0);
		* &lt;/script&gt;
		* @memberof ejLinearGauge
		* @instance
        */
        getMarkerStyle: function (scaleIndex, pointerIndex) {
            if (scaleIndex < this.Model.scales.length && pointerIndex < this.Model.scales[scaleIndex].markerPointers.length)
                return this.scaleEl[scaleIndex].markerPointers[pointerIndex].markerStyle;
        },
		/**
        * To set setBarPointerValue 
		* @return jQuery
		* @param {number} argument.scaleIndex scaleIndex value for the Gauge 
		* @param {number}  argument.pointerIndex pointerIndex value for the Gauge
		* @param {number} arguemnt.value Bar Pointer Value for Gauge
		* @example 
		* &lt;div id="LinearGauge1"&gt;&lt;/div&gt; <br>  
		* &lt;script&gt;
        * $("#LinearGauge1").ejLinearGauge();
		* var LinearGaugeObj = $("#LinearGauge1").data("ejLinearGauge");
		* LinearGaugeObj.setBarPointerValue(0,0,30);
		* &lt;/script&gt;
		*@memberof ejLinearGauge
		* @instance
        */
        setBarPointerValue: function (scaleIndex, pointerIndex, value) {
            if (scaleIndex < this.Model.scales.length && pointerIndex < this.Model.scales[scaleIndex].barPointers.length && value != null) {
            if(value <= this.scaleEl[scaleIndex].maximum && value >= this.scaleEl[scaleIndex].minimum)
                this.scaleEl[scaleIndex].barPointers[pointerIndex].value = value;
                if (this.contextEl.putImageData) {
                    if (this.scaleEl[this.scaleIndex].type == "thermometer")
                        this.initialize();
                    else
                        this._reDrawBarPointer();
                }
                else
                    this.initialize();
            }
        },
		/**
        * To get Bar Pointer Value in number
		* @return number
		* @param {number} argument.scaleIndex scaleIndex value for the Gauge 
		* @param {number}  argument.pointerIndex pointerIndex value for the Gauge
		* @example 
		* &lt;div id="LinearGauge1"&gt;&lt;/div&gt; <br>  
		* &lt;script&gt;
        * $("#LinearGauge1").ejLinearGauge({value:50});
		* var LinearGaugeObj = $("#LinearGauge1").data("ejLinearGauge");
		* LinearGaugeObj.getBarPointerValue(0,0);
		* &lt;/script&gt;
		*@memberof ejLinearGauge
		* @instance
        */
        getBarPointerValue: function (scaleIndex, pointerIndex) {
            if (scaleIndex < this.Model.scales.length && pointerIndex < this.Model.scales[scaleIndex].barPointers.length)
                return this.scaleEl[scaleIndex].barPointers[pointerIndex].value;
        },
		/**
        * To set setBarWidth 
		* @return jQuery
		* @param {number} argument.scaleIndex scaleIndexvalue for the Gauge 
		* @param {number}  argument.pointerIndex pointerIndex value for the Gauge
		* @param {number} arguemnt.value Bar Width for Gauge
		* @example 
		* &lt;div id="LinearGauge1"&gt;&lt;/div&gt; <br>  
		* &lt;script&gt;
        * $("#LinearGauge1").ejLinearGauge();
		* var LinearGaugeObj = $("#LinearGauge1").data("ejLinearGauge");
		* LinearGaugeObj.setBarWidth(0,0,30);
		* &lt;/script&gt;
		*@memberof ejLinearGauge
		* @instance
        */
        setBarWidth: function (scaleIndex, pointerIndex, value) {
            if (scaleIndex < this.Model.scales.length && pointerIndex < this.Model.scales[scaleIndex].barPointers.length && value != null) {
                this.scaleEl[scaleIndex].barPointers[pointerIndex].width = value;
                if (this.contextEl.putImageData) {
                    if (this.scaleEl[this.scaleIndex].type == "thermometer")
                        this.initialize();
                    else
                        this._reDrawBarPointer();
                }
                else
                    this.initialize();
            }
        },
		/**
        * To get Bar Width in number
		* @return number
		* @param {number} argument.scaleIndex scaleIndex value for the Gauge 
		* @param {number}  argument.pointerIndex pointerIndex value for the Gauge
		* @example 
		* &lt;div id="LinearGauge1"&gt;&lt;/div&gt; <br>  
		* &lt;script&gt;
        * $("#LinearGauge1").ejLinearGauge({value:50});
		* // get Bar Width in number
		* var LinearGaugeObj = $("#LinearGauge1").data("ejLinearGauge");
		* LinearGaugeObj.getBarWidth(0,0);
		* &lt;/script&gt;
		*@memberof ejLinearGauge
		* @instance
        */
        getBarWidth: function (scaleIndex, pointerIndex) {
            if (scaleIndex < this.Model.scales.length && pointerIndex < this.Model.scales[scaleIndex].barPointers.length)
                return this.scaleEl[scaleIndex].BarPointer[pointerIndex].width;
        },
		/**
        * To set setBarDistanceFromScale 
		* @return jQuery
		* @param {number} argument.scaleIndex scaleIndex,value for the Gauge 
		* @param {number}  argument.pointerIndex pointerIndex value for the Gauge
		* @param {number} arguemnt.value Bar DistanceFromScale value for Gauge
		* @example 
		* &lt;div id="LinearGauge1"&gt;&lt;/div&gt; <br>  
		* &lt;script&gt;
        * $("#LinearGauge1").ejLinearGauge();
		* var LinearGaugeObj = $("#LinearGauge1").data("ejLinearGauge");
		* LinearGaugeObj.setBarDistanceFromScale(0,0,30);
		* &lt;/script&gt;
		*@memberof ejLinearGauge
		* @instance
        */
        setBarDistanceFromScale: function (scaleIndex, pointerIndex, value) {
            if (scaleIndex < this.Model.scales.length && pointerIndex < this.Model.scales[scaleIndex].barPointers.length && value != null) {
                this.scaleEl[scaleIndex].barPointers[pointerIndex].distanceFromScale = value;
                if (this.contextEl.putImageData) {
                    if (this.scaleEl[this.scaleIndex].type == "thermometer")
                        this.initialize();
                    else
                        this._reDrawBarPointer();
                }
                else
                    this.initialize();
            }
        },
		/**
        * To get Bar Distance From Scale in number
		* @return number
		* @param {number} argument.scaleIndex scaleIndex value for the Gauge 
		* @param {number}  argument.pointerIndex pointerIndex value for the Gauge
		* @example 
		* &lt;div id="LinearGauge1"&gt;&lt;/div&gt; <br>  
		* &lt;script&gt;
        * $("#LinearGauge1").ejLinearGauge({value:50});
		* var LinearGaugeObj = $("#LinearGauge1").data("ejLinearGauge");
		* LinearGaugeObj.getBarDistanceFromScale(0,0);
		* &lt;/script&gt;
		*@memberof ejLinearGauge
		* @instance
        */
        getBarDistanceFromScale: function (scaleIndex, pointerIndex) {
            if (scaleIndex < this.Model.scales.length && pointerIndex < this.Model.scales[scaleIndex].barPointers.length)
                return this.scaleEl[scaleIndex].BarPointer[pointerIndex].distanceFromScale;
        },
		/**
        * To set setCustomLabelValue 
		* @return jQuery
		* @param {number} argument.scaleIndex scaleIndex value for the Gauge 
		* @param {number}  argument.customLabelIndex customLabelIndex value for the Gauge
		* @param {number} arguemnt.value CustomLabel value for Gauge
		* @example 
		* &lt;div id="LinearGauge1"&gt;&lt;/div&gt; <br>  
		* &lt;script&gt;
        * $("#LinearGauge1").ejLinearGauge({  scales:[{showCustomLabels:true,customLabels: [{ value:"MyLabel", position:{x:49,y:100}, color:"#fc0606", font: { size: "20px",fontFamily: "Arial", fontStyle: "bold" }}]}]});
		* var LinearGaugeObj = $("#LinearGauge1").data("ejLinearGauge");
		* LinearGaugeObj.setCustomLabelValue(0,0,"text");
		* &lt;/script&gt;
		*@memberof ejLinearGauge
		* @instance
        */
        setCustomLabelValue: function (scaleIndex, customLabelIndex, value) {
            if (scaleIndex < this.Model.scales.length && customLabelIndex < this.Model.scales[scaleIndex].customLabels.length && value != null) {
                this.scaleEl[scaleIndex].customLabels[customLabelIndex].value = value;
                if (this.contextEl.putImageData)
                    this._reDrawCustomLabel();
                else
                    this.initialize();
            }
        },
		/**
        * To get CustomLabel Value in string
		* @return string
		* @param {number} argument.scaleIndex scaleIndex value for the Gauge 
		* @param {number}  argument.customLabelIndex customLabelIndex value for the Gauge
		* @example 
		* &lt;div id="LinearGauge1"&gt;&lt;/div&gt; <br>  
		* &lt;script&gt;
        * $("#LinearGauge1").ejLinearGauge({  scales:[{showCustomLabels:true,customLabels: [{ value:"MyLabel", position:{x:49,y:100}, color:"#fc0606", font: { size: "20px",fontFamily: "Arial", fontStyle: "bold" }}]}]});
		* var LinearGaugeObj = $("#LinearGauge1").data("ejLinearGauge");
		* LinearGaugeObj.getCustomLabelValue(0,0);
		* &lt;/script&gt;
		*@memberof ejLinearGauge
		* @instance
        */
        getCustomLabelValue: function (scaleIndex, customLabelIndex) {
            if (scaleIndex < this.Model.scales.length && customLabelIndex < this.Model.scales[scaleIndex].customLabels.length)
                return this.scaleEl[scaleIndex].customLabels[customLabelIndex].value;
        },
		/**
        * To set setCustomLabelAngle 
		* @return jQuery
		* @param {number} argument.scaleIndex scaleIndex value for the Gauge 
		* @param {number}  argument.customLabelIndex customLabelIndex value for the Gauge
		* @param {number} arguemnt.value Custom Label Angle for Gauge
		* @example 
		* &lt;div id="LinearGauge1"&gt;&lt;/div&gt; <br>  
		* &lt;script&gt;
        * $("#LinearGauge1").ejLinearGauge({  scales:[{showCustomLabels:true,customLabels: [{ value:"MyLabel", position:{x:49,y:100}, color:"#fc0606", font: { size: "20px",fontFamily: "Arial", fontStyle: "bold" }}]}]});
		* var LinearGaugeObj = $("#LinearGauge1").data("ejLinearGauge");
		* LinearGaugeObj.setCustomLabelAngle(0,0,30);
		* &lt;/script&gt;
		*@memberof ejLinearGauge
		* @instance
        */
        setCustomLabelAngle: function (scaleIndex, customLabelIndex, value) {
            if (scaleIndex < this.Model.scales.length && customLabelIndex < this.Model.scales[scaleIndex].customLabels.length && value != null) {
                this.scaleEl[scaleIndex].customLabels[customLabelIndex].textAngle = value;
                if (this.contextEl.putImageData)
                    this._reDrawCustomLabel();
                else
                    this.initialize();
            }
        },
		/**
        * To get CustomLabel Angle in number
		* @return number
		* @param {number} argument.scaleIndex scaleIndex value for the Gauge
		* @param {number}  argument.customLabelIndex customLabelIndex value for the Gauge
		* @example 
		* &lt;div id="LinearGauge1"&gt;&lt;/div&gt; <br>  
		* &lt;script&gt;
        * $("#LinearGauge1").ejLinearGauge({  scales:[{showCustomLabels:true,customLabels: [{ value:"MyLabel", position:{x:49,y:100}, color:"#fc0606", font: { size: "20px",fontFamily: "Arial", fontStyle: "bold" }}]}]});
		* var LinearGaugeObj = $("#LinearGauge1").data("ejLinearGauge");
		* LinearGaugeObj.getCustomLabelAngle(0,0);
		* &lt;/script&gt;
		*@memberof ejLinearGauge
		* @instance
        */
        getCustomLabelAngle: function (scaleIndex, customLabelIndex) {
            if (scaleIndex < this.Model.scales.length && customLabelIndex < this.Model.scales[scaleIndex].customLabels.length)
                return this.scaleEl[scaleIndex].customLabels[customLabelIndex].textAngle;
        },
		/**
        * To set setRangeStartValue 
		* @return jQuery
		* @param {number} argument.scaleIndex scaleIndex value for the Gauge
		* @param {number}  argument.rangeIndex rangeIndex value for the Gauge
		* @param {number} arguemnt.value range start value for Gauge
		* @example 
		* &lt;div id="LinearGauge1"&gt;&lt;/div&gt; <br>  
		* &lt;script&gt;
        * $("#LinearGauge1").ejLinearGauge({ scales: [{ showBarPointers: false, width: 3, length: 310, showRanges: true,ranges: [{ startWidth: 10, endWidth: 20, endValue: 60, startValue: 0, backgroundColor: "#E94649" }] }] });
		* var LinearGaugeObj = $("#LinearGauge1").data("ejLinearGauge");
		* LinearGaugeObj.setRangeStartValue(0,0,20);
		* &lt;/script&gt;
		*@memberof ejLinearGauge
		* @instance
        */
        setRangeStartValue: function (scaleIndex, rangeIndex, value) {
            if (scaleIndex < this.Model.scales.length && rangeIndex < this.Model.scales[scaleIndex].ranges.length && value != null) {
                this.scaleEl[scaleIndex].ranges[rangeIndex].startValue = value;
                if (this.contextEl.putImageData)
                    this._reDrawRange();
                else
                    this.initialize();
            }
        },
		/**
        * To get Range Start Value in number
		* @return number
		* @param {number} argument.scaleIndex scaleIndex value for the Gauge
		* @param {number}  argument.rangeIndex rangeIndex value for the Gauge
		* @example 
		* &lt;div id="LinearGauge1"&gt;&lt;/div&gt; <br>  
		* &lt;script&gt;
        * $("#LinearGauge1").ejLinearGauge({ scales: [{ showBarPointers: false, width: 3, length: 310, showRanges: true,ranges: [{ startWidth: 10, endWidth: 20, endValue: 60, startValue: 0, backgroundColor: "#E94649" }] }] });
		* var LinearGaugeObj = $("#LinearGauge1").data("ejLinearGauge");
		* LinearGaugeObj.getRangeStartValue(0,0);
		* &lt;/script&gt;
		*@memberof ejLinearGauge
		* @instance
        */
        getRangeStartValue: function (scaleIndex, rangeIndex) {
            if (scaleIndex < this.Model.scales.length && rangeIndex < this.Model.scales[scaleIndex].ranges.length)
                return this.scaleEl[scaleIndex].ranges[rangeIndex].startValue;
        },
		/**
        * To set setRangeEndValue 
		* @return jQuery
		* @param {number} argument.scaleIndex scaleIndex value for the Gauge
		* @param {number}  argument.rangeIndex rangeIndex value for the Gauge
		* @param {number} arguemnt.value Range end value for Gauge
		* @example 
		* &lt;div id="LinearGauge1"&gt;&lt;/div&gt; <br>  
		* &lt;script&gt;
        * $("#LinearGauge1").ejLinearGauge({ scales: [{ showBarPointers: false, width: 3, length: 310, showRanges: true,ranges: [{ startWidth: 10, endWidth: 20, endValue: 60, startValue: 0, backgroundColor: "#E94649" }] }] });
		* var LinearGaugeObj = $("#LinearGauge1").data("ejLinearGauge");
		* LinearGaugeObj.setRangeEndValue(0,0,20);
		* &lt;/script&gt;
		*@memberof ejLinearGauge
		* @instance
        */
        setRangeEndValue: function (scaleIndex, rangeIndex, value) {
            if (scaleIndex < this.Model.scales.length && rangeIndex < this.Model.scales[scaleIndex].ranges.length && value != null) {
                this.scaleEl[scaleIndex].ranges[rangeIndex].endValue = value;
                if (this.contextEl.putImageData)
                    this._reDrawRange();
                else
                    this.initialize();
            }
        },
		/**
        * To get Range End Value in number
		* @return number
		* @param {number} argument.scaleIndex scaleIndex value for the Gauge
		* @param {number}  argument.rangeIndex rangeIndex value for the Gauge
		* @example 
		* &lt;div id="LinearGauge1"&gt;&lt;/div&gt; <br>  
		* &lt;script&gt;
        * $("#LinearGauge1").ejLinearGauge({ scales: [{ showBarPointers: false, width: 3, length: 310, showRanges: true,ranges: [{ startWidth: 10, endWidth: 20, endValue: 60, startValue: 0, backgroundColor: "#E94649" }] }] });
		* var LinearGaugeObj = $("#LinearGauge1").data("ejLinearGauge");
		* LinearGaugeObj.getRangeEndValue(0,0);
		* &lt;/script&gt;
		*@memberof ejLinearGauge
		* @instance
        */
        getRangeEndValue: function (scaleIndex, rangeIndex) {
            if (scaleIndex < this.Model.scales.length && rangeIndex < this.Model.scales[scaleIndex].ranges.length)
                return this.scaleEl[scaleIndex].ranges[rangeIndex].endValue;
        },
		/**
        * To set setRangeStartWidth 
		* @return jQuery
		* @param {number} argument.scaleIndex scaleIndex value for the Gauge
		* @param {number}  argument.rangeIndex rangeIndex value for the Gauge
		* @param {number} arguemnt.value Range Start Width for Gauge
		* @example 
		* &lt;div id="LinearGauge1"&gt;&lt;/div&gt; <br>  
		* &lt;script&gt;
        * $("#LinearGauge1").ejLinearGauge({ scales: [{ showBarPointers: false, width: 3, length: 310, showRanges: true,ranges: [{ startWidth: 10, endWidth: 20, endValue: 60, startValue: 0, backgroundColor: "#E94649" }] }] });
		* var LinearGaugeObj = $("#LinearGauge1").data("ejLinearGauge");
		* LinearGaugeObj.setRangeStartWidth(0,0,20);
		* &lt;/script&gt;
		*@memberof ejLinearGauge
		* @instance
        */
        setRangeStartWidth: function (scaleIndex, rangeIndex, value) {
            if (scaleIndex < this.Model.scales.length && rangeIndex < this.Model.scales[scaleIndex].ranges.length && value != null) {
                this.scaleEl[scaleIndex].ranges[rangeIndex].startWidth = value;
                if (this.contextEl.putImageData)
                    this._reDrawRange();
                else
                    this.initialize();
            }
        },
		/**
        * To get Range Start Width in number
		* @return number
		* @param {number} argument.scaleIndex scaleIndex value for the Gauge
		* @param {number}  argument.rangeIndex rangeIndex value for the Gauge
		* @example 
		* &lt;div id="LinearGauge1"&gt;&lt;/div&gt; <br>  
		* &lt;script&gt;
        * $("#LinearGauge1").ejLinearGauge({ scales: [{ showBarPointers: false, width: 3, length: 310, showRanges: true,ranges: [{ startWidth: 10, endWidth: 20, endValue: 60, startValue: 0, backgroundColor: "#E94649" }] }] });
		* var LinearGaugeObj = $("#LinearGauge1").data("ejLinearGauge");
		* LinearGaugeObj.getRangeStartWidth(0,0);
		* &lt;/script&gt;
		*@memberof ejLinearGauge
		* @instance
        */
        getRangeStartWidth: function (scaleIndex, rangeIndex) {
            if (scaleIndex < this.Model.scales.length && rangeIndex < this.Model.scales[scaleIndex].ranges.length)
                return this.scaleEl[scaleIndex].ranges[rangeIndex].startWidth;
        },
		/**
        * To set setRangeEndWidth 
		* @return jQuery
		* @param {number} argument.scaleIndex scaleIndex value for the Gauge
		* @param {number}  argument.rangeIndex rangeIndex value for the Gauge
		* @param {number} arguemnt.value Range End Width for Gauge
		* @example 
		* &lt;div id="LinearGauge1"&gt;&lt;/div&gt; <br>  
		* &lt;script&gt;
        * $("#LinearGauge1").ejLinearGauge({ scales: [{ showBarPointers: false, width: 3, length: 310, showRanges: true,ranges: [{ startWidth: 10, endWidth: 20, endValue: 60, startValue: 0, backgroundColor: "#E94649" }] }] });
		* var LinearGaugeObj = $("#LinearGauge1").data("ejLinearGauge");
		* LinearGaugeObj.setRangeEndWidth(0,0,20);
		* &lt;/script&gt;
		*@memberof ejLinearGauge
		* @instance
        */
        setRangeEndWidth: function (scaleIndex, rangeIndex, value) {
            if (scaleIndex < this.Model.scales.length && rangeIndex < this.Model.scales[scaleIndex].ranges.length && value != null) {
                this.scaleEl[scaleIndex].ranges[rangeIndex].endWidth = value;
                if (this.contextEl.putImageData)
                    this._reDrawRange();
                else
                    this.initialize();
            }
        },
		/**
        * To get Range End Width in number
		* @return number
		* @param {number} argument.scaleIndex scaleIndex value for the Gauge
		* @param {number}  argument.rangeIndex rangeIndex value for the Gauge
		* @example 
		* &lt;div id="LinearGauge1"&gt;&lt;/div&gt; <br>  
		* &lt;script&gt;
        * $("#LinearGauge1").ejLinearGauge({ scales: [{ showBarPointers: false, width: 3, length: 310, showRanges: true,ranges: [{ startWidth: 10, endWidth: 20, endValue: 60, startValue: 0, backgroundColor: "#E94649" }] }] });
		* var LinearGaugeObj = $("#LinearGauge1").data("ejLinearGauge");
		* LinearGaugeObj.getRangeEndWidth(0,0);
		* &lt;/script&gt;
		*@memberof ejLinearGauge
		* @instance
        */
        getRangeEndWidth: function (scaleIndex, rangeIndex) {
            if (scaleIndex < this.Model.scales.length && rangeIndex < this.Model.scales[scaleIndex].ranges.length)
                return this.scaleEl[scaleIndex].ranges[rangeIndex].endWidth;
        },
		/**
        * To set setRangeDistanceFromScale 
		* @return jQuery
		* @param {number} argument.scaleIndex scaleIndex value for the Gauge
		* @param {number}  argument.rangeIndex rangeIndex value for the Gauge
		* @param {number} arguemnt.value Range Distance FromScale for Gauge
		* @example 
		* &lt;div id="LinearGauge1"&gt;&lt;/div&gt; <br>  
		* &lt;script&gt;
        * $("#LinearGauge1").ejLinearGauge({ scales: [{ showBarPointers: false, width: 3, length: 310, showRanges: true,ranges: [{ startWidth: 10, endWidth: 20, endValue: 60, startValue: 0, backgroundColor: "#E94649" }] }] });
		* var LinearGaugeObj = $("#LinearGauge1").data("ejLinearGauge");
		* LinearGaugeObj.setRangeDistanceFromScale(0,0,20);
		* &lt;/script&gt;
		*@memberof ejLinearGauge
		* @instance
        */
        setRangeDistanceFromScale: function (scaleIndex, rangeIndex, value) {
            if (scaleIndex < this.Model.scales.length && rangeIndex < this.Model.scales[scaleIndex].ranges.length && value != null) {
                this.scaleEl[scaleIndex].ranges[rangeIndex].distanceFromScale = value;
                if (this.contextEl.putImageData)
                    this._reDrawRange();
                else
                    this.initialize();
            }
        },
		/**
        * To get Range Distance From Scale in number
		* @return number
		* @param {number} argument.scaleIndex scaleIndex value for the Gauge
		* @param {number}  argument.rangeIndex rangeIndex value for the Gauge
		* @example 
		* &lt;div id="LinearGauge1"&gt;&lt;/div&gt; <br>  
		* &lt;script&gt;
        * $("#LinearGauge1").ejLinearGauge({ scales: [{ showBarPointers: false, width: 3, length: 310, showRanges: true,ranges: [{ startWidth: 10, endWidth: 20, endValue: 60, startValue: 0, backgroundColor: "#E94649" }] }] });
		* var LinearGaugeObj = $("#LinearGauge1").data("ejLinearGauge");
		* LinearGaugeObj.getRangeDistanceFromScale(0,0);
		* &lt;/script&gt;
		*@memberof ejLinearGauge
		* @instance
        */
        getRangeDistanceFromScale: function (scaleIndex, rangeIndex) {
            if (scaleIndex < this.Model.scales.length && rangeIndex < this.Model.scales[scaleIndex].ranges.length)
                return this.scaleEl[scaleIndex].ranges[rangeIndex].distanceFromScale;
        },
		/**
        * To set setRangePosition 
		* @return jQuery
		* @param {number} argument.scaleIndex scaleIndex value for the Gauge
		* @param {number}  argument.rangeIndex rangeIndex value for the Gauge
		* @param {number} arguemnt.value Range Position for Gauge
		* @example 
		* &lt;div id="LinearGauge1"&gt;&lt;/div&gt; <br>  
		* &lt;script&gt;
        * $("#LinearGauge1").ejLinearGauge({ scales: [{ showBarPointers: false, width: 3, length: 310, showRanges: true,ranges: [{ startWidth: 10, endWidth: 20, endValue: 60, startValue: 0, backgroundColor: "#E94649" }] }] });
		* var LinearGaugeObj = $("#LinearGauge1").data("ejLinearGauge");
		* LinearGaugeObj.setRangePosition(0,0,20);
		* &lt;/script&gt;
		*@memberof ejLinearGauge
		* @instance
        */
        setRangePosition: function (scaleIndex, rangeIndex, value) {
            if (scaleIndex < this.Model.scales.length && rangeIndex < this.Model.scales[scaleIndex].ranges.length && value != null) {
                this.scaleEl[scaleIndex].ranges[rangeIndex].placement = value;
                if (this.contextEl.putImageData)
                    this._reDrawRange();
                else
                    this.initialize();
            }
        },
		/**
        * To get Range Position in number
		* @return number
		* @param {number} argument.scaleIndex scaleIndex value for the Gauge
		* @param {number}  argument.rangeIndex rangeIndex value for the Gauge
		* @example 
		* &lt;div id="LinearGauge1"&gt;&lt;/div&gt; <br>  
		* &lt;script&gt;
        * $("#LinearGauge1").ejLinearGauge({ scales: [{ showBarPointers: false, width: 3, length: 310, showRanges: true,ranges: [{ startWidth: 10, endWidth: 20, endValue: 60, startValue: 0, backgroundColor: "#E94649" }] }] });
		* var LinearGaugeObj = $("#LinearGauge1").data("ejLinearGauge");
		* LinearGaugeObj.getRangePosition(0,0);
		* &lt;/script&gt;
		*@memberof ejLinearGauge
		* @instance
        */
        getRangePosition: function (scaleIndex, rangeIndex) {
            if (scaleIndex < this.Model.scales.length && rangeIndex < this.Model.scales[scaleIndex].ranges.length)
                return this.scaleEl[scaleIndex].ranges[rangeIndex].placement;
        },
		/**
        * To set setRangeBorderWidth 
		* @return jQuery
		* @param {number} argument.scaleIndex scaleIndex value for the Gauge
		* @param {number}  argument.rangeIndex rangeIndex value for the Gauge
		* @param {number} arguemnt.value Range Border Width for Gauge
		* @example 
		* &lt;div id="LinearGauge1"&gt;&lt;/div&gt; <br>  
		* &lt;script&gt;
        * $("#LinearGauge1").ejLinearGauge({ scales: [{ showBarPointers: false, width: 3, length: 310, showRanges: true,ranges: [{ startWidth: 10, endWidth: 20, endValue: 60, startValue: 0, backgroundColor: "#E94649" }] }] });
		* var LinearGaugeObj = $("#LinearGauge1").data("ejLinearGauge");
		* LinearGaugeObj.setRangeBorderWidth(0,0,2);
		* &lt;/script&gt;
		*@memberof ejLinearGauge
		* @instance
        */
        setRangeBorderWidth: function (scaleIndex, rangeIndex, value) {
            if (scaleIndex < this.Model.scales.length && rangeIndex < this.Model.scales[scaleIndex].ranges.length && value != null) {
                this.scaleEl[scaleIndex].ranges[rangeIndex].border.width = value;
                if (this.contextEl.putImageData)
                    this._reDrawRange();
                else
                    this.initialize();
            }
        },
		/**
        * To get Range Border Width in number
		* @return number
		* @param {number} argument.scaleIndex scaleIndex value for the Gauge
		* @param {number}  argument.rangeIndex rangeIndex value for the Gauge
		* @example 
		* &lt;div id="LinearGauge1"&gt;&lt;/div&gt; <br>  
		* &lt;script&gt;
        * $("#LinearGauge1").ejLinearGauge({ scales: [{ showBarPointers: false, width: 3, length: 310, showRanges: true,ranges: [{ startWidth: 10, endWidth: 20, endValue: 60, startValue: 0, backgroundColor: "#E94649" }] }] });
		* var LinearGaugeObj = $("#LinearGauge1").data("ejLinearGauge");
		* LinearGaugeObj.getRangeBorderWidth(0,0);
		* &lt;/script&gt;
		*@memberof ejLinearGauge
		* @instance
        */
        getRangeBorderWidth: function (scaleIndex, rangeIndex) {
            if (scaleIndex < this.Model.scales.length && rangeIndex < this.Model.scales[scaleIndex].ranges.length)
                return this.scaleEl[scaleIndex].ranges[rangeIndex].border.width;
        },
		/**
        * To set setLabelAngle 
		* @return jQuery
		* @param {number} argument.scaleIndex scaleIndex value for the Gauge
		* @param {number}  argument.labelIndex value for the Gauge
		* @param {number} arguemnt.angle Label Angle for Gauge
		* @example 
		* &lt;div id="LinearGauge1"&gt;&lt;/div&gt; <br>  
		* &lt;script&gt;
        * $("#LinearGauge1").ejLinearGauge();
		* var LinearGaugeObj = $("#LinearGauge1").data("ejLinearGauge");
		* LinearGaugeObj.setLabelAngle(0,0,20);
		* &lt;/script&gt;
		*@memberof ejLinearGauge
		* @instance
        */
        setLabelAngle: function (scaleIndex, labelIndex, angle) {
            if (scaleIndex < this.Model.scales.length && labelIndex < this.Model.scales[scaleIndex].labels.length && angle != null) {
                this.scaleEl[scaleIndex].labels[labelIndex].angle = angle;
                if (this.contextEl.putImageData) this._reDrawLabel();
                else
                    this.initialize();
            }
        },
		/**
        * To get Label Angle in number
		* @return number
		* @param {number} argument.scaleIndex scaleIndex value for the Gauge
		* @param {number}  argument.labelIndex value for the Gauge
		* @example 
		* &lt;div id="LinearGauge1"&gt;&lt;/div&gt; <br>  
		* &lt;script&gt;
        * $("#LinearGauge1").ejLinearGauge();
		* var LinearGaugeObj = $("#LinearGauge1").data("ejLinearGauge");
		* LinearGaugeObj.getLabelAngle(0,0);
		* &lt;/script&gt;
		*@memberof ejLinearGauge
		* @instance
        */
        getLabelAngle: function (scaleIndex, labelIndex) {
            if (scaleIndex < this.Model.scales.length && labelIndex < this.Model.scales[scaleIndex].labels.length)
                return this.scaleEl[scaleIndex].labels[labelIndex].angle;
        },
		/**
        * To set setLabelStyle 
		* @return jQuery
		* @param {number} argument.scaleIndex scaleIndex value for the Gauge
		* @param {number}  argument.labelIndex value for the Gauge
		* @param {string} arguemnt.value Label Style for  Gauge
		* @example 
		* &lt;div id="LinearGauge1"&gt;&lt;/div&gt; <br>  
		* &lt;script&gt;
        * $("#LinearGauge1").ejLinearGauge();
		* var LinearGaugeObj = $("#LinearGauge1").data("ejLinearGauge");
		* LinearGaugeObj.setLabelStyle(0,0,"major");
		* &lt;/script&gt;
		*@memberof ejLinearGauge
		* @instance
        */
        setLabelStyle: function (scaleIndex, labelIndex, value) {
            if (scaleIndex < this.Model.scales.length && labelIndex < this.Model.scales[scaleIndex].barPointers.length && value != null) {
                this.scaleEl[scaleIndex].labels[labelIndex].type = $.inArray(value, this.Model.tickStyle);
                if (this.contextEl.putImageData) this._reDrawLabel();
                else
                    this.initialize();
            }
        },
		/**
        * To get LabelStyle in number
		* @return number
		* @param {number} argument.scaleIndex scaleIndex value for the Gauge
		* @param {number}  argument.labelIndex value for the Gauge
		* @example 
		* &lt;div id="LinearGauge1"&gt;&lt;/div&gt; <br>  
		* &lt;script&gt;
        * $("#LinearGauge1").ejLinearGauge();
		* var LinearGaugeObj = $("#LinearGauge1").data("ejLinearGauge");
		* LinearGaugeObj.getLabelStyle(0,0);
		* &lt;/script&gt;
		*@memberof ejLinearGauge
		* @instance
        */
        getLabelStyle: function (scaleIndex, labelIndex) {
            if (scaleIndex < this.Model.scales.length && pointerIndex < this.Model.scales[scaleIndex].labels.length)
                return this.scaleEl[scaleIndex].barPointers[labelIndex].type;
        },
		/**
        * To set setLabelPlacement 
		* @return jQuery
		* @param {number} argument.scaleIndex scaleIndex value for the Gauge
		* @param {number}  argument.labelIndex value for the Gauge
		* @param {number} arguemnt.value Label Placement for Gauge
		* @example 
		* &lt;div id="LinearGauge1"&gt;&lt;/div&gt; <br>  
		* &lt;script&gt;
        * $("#LinearGauge1").ejLinearGauge();
		* var LinearGaugeObj = $("#LinearGauge1").data("ejLinearGauge");
		* LinearGaugeObj.setLabelPlacement(0,0,"far");
		* &lt;/script&gt;
		*@memberof ejLinearGauge
		* @instance
        */
        setLabelPlacement: function (scaleIndex, labelIndex, value) {
            if (scaleIndex < this.Model.scales.length && labelIndex < this.Model.scales[scaleIndex].labels.length && value != null) {
                this.scaleEl[scaleIndex].labels[labelIndex].placement = $.inArray(value, this.Model.labelPlacement);
                if (this.contextEl.putImageData) this._reDrawLabel();
                else
                    this.initialize();
            }
        },
		/**
        * To get LabelPlacement in number
		* @return number
		* @param {number} argument.scaleIndex scaleIndex value for the Gauge
		* @param {number}  argument.labelIndex value for the Gauge
		* @example 
		* &lt;div id="LinearGauge1"&gt;&lt;/div&gt; <br>  
		* &lt;script&gt;
        * $("#LinearGauge1").ejLinearGauge();
		* var LinearGaugeObj = $("#LinearGauge1").data("ejLinearGauge");
		* LinearGaugeObj.getLabelPlacement(0,0);
		* &lt;/script&gt;
		*@memberof ejLinearGauge
		* @instance
        */
        getLabelPlacement: function (scaleIndex, labelIndex) {
            if (scaleIndex < this.Model.scales.length && labelIndex < this.Model.scales[scaleIndex].labels.length)
                return this.scaleEl[scaleIndex].barPointers[labelIndex].placement;
        },
		/**
        * To set setLabelXDistanceFromScale 
		* @return jQuery
		* @param {number} argument.scaleIndex scaleIndex value for the Gauge
		* @param {number}  argument.labelIndex value for the Gauge
		* @param {number} arguemnt.value Label XDistance From Scale for Gauge
		* @example 
		* &lt;div id="LinearGauge1"&gt;&lt;/div&gt; <br>  
		* &lt;script&gt;
        * $("#LinearGauge1").ejLinearGauge();
		* var LinearGaugeObj = $("#LinearGauge1").data("ejLinearGauge");
		* LinearGaugeObj.setLabelXDistanceFromScale(0,0,20);
		* &lt;/script&gt;
		*@memberof ejLinearGauge
		* @instance
        */
        setLabelXDistanceFromScale: function (scaleIndex, labelIndex, value) {
            if (scaleIndex < this.Model.scales.length && labelIndex < this.Model.scales[scaleIndex].labels.length && value != null) {
                this.scaleEl[scaleIndex].labels[labelIndex].distanceFromScale.x = value;
                if (this.contextEl.putImageData) this._reDrawLabel();
                else
                    this.initialize();
            }
        },
		/**
        * To get Label XDistance From Scale in number
		* @return number
		* @param {number} argument.scaleIndex scaleIndex value for the Gauge
		* @param {number}  argument.labelIndex value for the Gauge
		* @example 
		* &lt;div id="LinearGauge1"&gt;&lt;/div&gt; <br>  
		* &lt;script&gt;
        * $("#LinearGauge1").ejLinearGauge();
		* var LinearGaugeObj = $("#LinearGauge1").data("ejLinearGauge");
		* LinearGaugeObj.getLabelXDistanceFromScale(0,0);
		* &lt;/script&gt;
		*@memberof ejLinearGauge
		* @instance
        */
        getLabelXDistanceFromScale: function (scaleIndex, labelIndex) {
            if (scaleIndex < this.Model.scales.length && labelIndex < this.Model.scales[scaleIndex].labels.length)
                return this.scaleEl[scaleIndex].labels[labelIndex].distanceFromScale.x;
        },
		/**
        * To set setLabelYDistanceFromScale 
		* @return jQuery
		* @param {number} argument.scaleIndex scaleIndex value for the Gauge
		* @param {number}  argument.labelIndex value for the Gauge
		* @param {number} arguemnt.value Label YDistance From Scale for Gauge
		* @example 
		* &lt;div id="LinearGauge1"&gt;&lt;/div&gt; <br>  
		* &lt;script&gt;
        * $("#LinearGauge1").ejLinearGauge();
		* var LinearGaugeObj = $("#LinearGauge1").data("ejLinearGauge");
		* LinearGaugeObj.setLabelYDistanceFromScale(0,0,20);
		* &lt;/script&gt;
		*@memberof ejLinearGauge
		* @instance
        */
        setLabelYDistanceFromScale: function (scaleIndex, labelIndex, value) {
            if (scaleIndex < this.Model.scales.length && labelIndex < this.Model.scales[scaleIndex].labels.length && value != null) {
                this.scaleEl[scaleIndex].labels[labelIndex].distanceFromScale.y = value;
                if (this.contextEl.putImageData)
                    this._reDrawLabel();
                else
                    this.initialize();
            }
        },
		/**
        * To get PointerValue in number
		* @return number
		* @param {number} argument.scaleIndex scaleIndex value for the Gauge
		* @param {number}  argument.labelIndex value for the Gauge
		* @example 
		* &lt;div id="LinearGauge1"&gt;&lt;/div&gt; <br>  
		* &lt;script&gt;
        * $("#LinearGauge1").ejLinearGauge();
		* var LinearGaugeObj = $("#LinearGauge1").data("ejLinearGauge");
		* LinearGaugeObj.getLabelYDistanceFromScale(0,0);
		* &lt;/script&gt;
		*@memberof ejLinearGauge
		* @instance
        */
        getLabelYDistanceFromScale: function (scaleIndex, labelIndex) {
            if (scaleIndex < this.Model.scales.length && labelIndex < this.Model.scales[scaleIndex].labels.length)
                return this.scaleEl[scaleIndex].labels[labelIndex].distanceFromScale.y;
        },
		/**
        * To set setTickAngle 
		* @return jQuery
		* @param {number} argument.scaleIndex scaleIndex value for the Gauge
		* @param {number}  argument.TickIndex value for the Gauge
		* @param {number} arguemnt.angle Tick Angle for Gauge
		* @example 
		* &lt;div id="LinearGauge1"&gt;&lt;/div&gt; <br>  
		* &lt;script&gt;
        * $("#LinearGauge1").ejLinearGauge();
		* var LinearGaugeObj = $("#LinearGauge1").data("ejLinearGauge");
		* LinearGaugeObj.setTickAngle(0,0,20);
		* &lt;/script&gt;
		*@memberof ejLinearGauge
		* @instance
        */
        setTickAngle: function (scaleIndex, TickIndex, angle) {
            if (scaleIndex < this.Model.scales.length && TickIndex < this.Model.scales[scaleIndex].ticks.length && angle != null) {
                this.scaleEl[scaleIndex].ticks[TickIndex].angle = angle;
                if (this.contextEl.putImageData) {
                    this._reDrawTickMark();
                }
                else
                    this.initialize();
            }
        },
		/**
        * To get Tick Angle in number
		* @return number
		* @param {number} argument.scaleIndex scaleIndex value for the Gauge
		* @param {number}  argument.TickIndex value for the Gauge
		* @example 
		* &lt;div id="LinearGauge1"&gt;&lt;/div&gt; <br>  
		* &lt;script&gt;
        * $("#LinearGauge1").ejLinearGauge();
		* var LinearGaugeObj = $("#LinearGauge1").data("ejLinearGauge");
		* LinearGaugeObj.getTickAngle(0,0);
		* &lt;/script&gt;
		*@memberof ejLinearGauge
		* @instance
        */
        getTickAngle: function (scaleIndex, TickIndex) {
            if (scaleIndex < this.Model.scales.length && TickIndex < this.Model.scales[scaleIndex].ticks.length)
                return this.scaleEl[scaleIndex].ticks[TickIndex].angle;
        },
		/**
        * To set setTickWidth 
		* @return jQuery
		* @param {number} argument.scaleIndex scaleIndex value for the Gauge
		* @param {number}  argument.TickIndex value for the Gauge
		* @param {number} arguemnt.value Tick Width for Gauge
		* @example 
		* &lt;div id="LinearGauge1"&gt;&lt;/div&gt; <br>  
		* &lt;script&gt;
        * $("#LinearGauge1").ejLinearGauge();
		* var LinearGaugeObj = $("#LinearGauge1").data("ejLinearGauge");
		* LinearGaugeObj.setTickWidth(0,0,5);
		* &lt;/script&gt;
		*@memberof ejLinearGauge
		* @instance
        */
        setTickWidth: function (scaleIndex, TickIndex, value) {
            if (scaleIndex < this.Model.scales.length && TickIndex < this.Model.scales[scaleIndex].ticks.length && value != null) {
                this.scaleEl[scaleIndex].ticks[TickIndex].width = value;
                if (this.contextEl.putImageData) {
                    this._reDrawTickMark();
                }
                else
                    this.initialize();
            }
        },
		/**
        * To get Tick Width in number
		* @return number
		* @param {number} argument.scaleIndex scaleIndex value for the Gauge
		* @param {number}  argument.TickIndex value for the Gauge
		* @example 
		* &lt;div id="LinearGauge1"&gt;&lt;/div&gt; <br>  
		* &lt;script&gt;
        * $("#LinearGauge1").ejLinearGauge();
		* var LinearGaugeObj = $("#LinearGauge1").data("ejLinearGauge");
		* LinearGaugeObj.getTickWidth(0,0);
		* &lt;/script&gt;
		*@memberof ejLinearGauge
		* @instance
        */
        getTickWidth: function (scaleIndex, TickIndex) {
            if (scaleIndex < this.Model.scales.length && TickIndex < this.Model.scales[scaleIndex].ticks.length)
                return this.scaleEl[scaleIndex].labels[TickIndex].width;
        },
		/**
        * To set setTickHeight 
		* @return jQuery
		* @param {number} argument.scaleIndex scaleIndex value for the Gauge
		* @param {number}  argument.TickIndex value for the Gauge
		* @param {number} arguemnt.value Tick Height for Gauge
		* @example 
		* &lt;div id="LinearGauge1"&gt;&lt;/div&gt; <br>  
		* &lt;script&gt;
        * $("#LinearGauge1").ejLinearGauge();
		* var LinearGaugeObj = $("#LinearGauge1").data("ejLinearGauge");
		* LinearGaugeObj.setTickHeight(0,0,10);
		* &lt;/script&gt;
		*@memberof ejLinearGauge
		* @instance
        */
        setTickHeight: function (scaleIndex, TickIndex, value) {
            if (scaleIndex < this.Model.scales.length && TickIndex < this.Model.scales[scaleIndex].ticks.length && value != null) {
                this.scaleEl[scaleIndex].ticks[TickIndex].height = value;
                if (this.contextEl.putImageData) {
                    this._reDrawTickMark();
                }
                else
                    this.initialize();
            }
        },
		/**
        * To get Tick Height in number
		* @return number
		* @param {number} argument.scaleIndex scaleIndex value for the Gauge
		* @param {number}  argument.TickIndex value for the Gauge
		* @example 
		* &lt;div id="LinearGauge1"&gt;&lt;/div&gt; <br>  
		* &lt;script&gt;
        * $("#LinearGauge1").ejLinearGauge();
		* var LinearGaugeObj = $("#LinearGauge1").data("ejLinearGauge");
		* LinearGaugeObj.getTickHeight(0,0);
		* &lt;/script&gt;
		*@memberof ejLinearGauge
		* @instance
        */
        getTickHeight: function (scaleIndex, TickIndex) {
            if (scaleIndex < this.Model.scales.length && TickIndex < this.Model.scales[scaleIndex].ticks.length)
                return this.scaleEl[scaleIndex].labels[TickIndex].height;
        },
		/**
        * To set setTickStyle 
		* @return jQuery
		* @param {number} argument.scaleIndex scaleIndex value for the Gauge
		* @param {number}  argument.TickIndex value for the Gauge
		* @param {string} arguemnt.value Tick Stylefor Gauge
		* @example 
		* &lt;div id="LinearGauge1"&gt;&lt;/div&gt; <br>  
		* &lt;script&gt;
        * $("#LinearGauge1").ejLinearGauge();
		* var LinearGaugeObj = $("#LinearGauge1").data("ejLinearGauge");
		* LinearGaugeObj.setTickStyle(0,0,"major");
		* &lt;/script&gt;
		*@memberof ejLinearGauge
		* @instance
        */
        setTickStyle: function (scaleIndex, TickIndex, value) {
            if (scaleIndex < this.Model.scales.length && TickIndex < this.Model.scales[scaleIndex].ticks.length && value != null) {
                this.scaleEl[scaleIndex].ticks[TickIndex].type = $.inArray(value, this.Model.tickStyle);
                if (this.contextEl.putImageData) {
                    this._reDrawTickMark();
                }
                else
                    this.initialize();
            }
        },
		/**
        * To get Tick Style in string
		* @return string
		* @param {number} argument.scaleIndex scaleIndex value for the Gauge
		* @param {number}  argument.TickIndex value for the Gauge
		* @example 
		* &lt;div id="LinearGauge1"&gt;&lt;/div&gt; <br>  
		* &lt;script&gt;
        * $("#LinearGauge1").ejLinearGauge();
		* var LinearGaugeObj = $("#LinearGauge1").data("ejLinearGauge");
		* LinearGaugeObj.getTickStyle(0,0);
		* &lt;/script&gt;
		*@memberof ejLinearGauge
		* @instance
        */
        getTickStyle: function (scaleIndex, TickIndex) {
            if (scaleIndex < this.Model.scales.length && TickIndex < this.Model.scales[scaleIndex].ticks.length)
                return this.scaleEl[scaleIndex].barPointers[TickIndex].type;
        },
		/**
        * To set setTickPlacement 
		* @return jQuery
		* @param {number} argument.scaleIndex scaleIndex value for the Gauge
		* @param {number}  argument.TickIndex value for the Gauge
		* @param {number} arguemnt.value Tick Placement for Gauge
		* @example 
		* &lt;div id="LinearGauge1"&gt;&lt;/div&gt; <br>  
		* &lt;script&gt;
        * $("#LinearGauge1").ejLinearGauge();
		* var LinearGaugeObj = $("#LinearGauge1").data("ejLinearGauge");
		* LinearGaugeObj.setTickPlacement(0,0,"far");
		* &lt;/script&gt;
		*@memberof ejLinearGauge
		* @instance
        */
        setTickPlacement: function (scaleIndex, TickIndex, value) {
            if (scaleIndex < this.Model.scales.length && TickIndex < this.Model.scales[scaleIndex].ticks.length && value != null) {
                this.scaleEl[scaleIndex].ticks[TickIndex].placement = $.inArray(value, this.Model.tickPlacement);
                if (this.contextEl.putImageData) {
                    this._reDrawTickMark();
                }
                else
                    this.initialize();
            }
        },
		/**
        * To get getTickPlacement in number
		* @return number
		* @param {number} argument.scaleIndex scaleIndex value for the Gauge
		* @param {number}  argument.TickIndex value for the Gauge
		* @example 
		* &lt;div id="LinearGauge1"&gt;&lt;/div&gt; <br>  
		* &lt;script&gt;
        * $("#LinearGauge1").ejLinearGauge();
		* var LinearGaugeObj = $("#LinearGauge1").data("ejLinearGauge");
		* LinearGaugeObj.getTickPlacement(0,0);
		* &lt;/script&gt;
		*@memberof ejLinearGauge
		* @instance
        */
        getTickPlacement: function (scaleIndex, TickIndex) {
            if (scaleIndex < this.Model.scales.length && TickIndex < this.Model.scales[scaleIndex].ticks.length)
                return this.scaleEl[scaleIndex].barPointers[TickIndex].placement;
        },
		/**
        * To set setTickXDistanceFromScale 
		* @return jQuery
		* @param {number} argument.scaleIndex scaleIndex value for the Gauge
		* @param {number}  argument.TickIndex value for the Gauge
		* @param {number} arguemnt.value Tick XDistance From Scale for Gauge
		* @example 
		* &lt;div id="LinearGauge1"&gt;&lt;/div&gt; <br>  
		* &lt;script&gt;
        * $("#LinearGauge1").ejLinearGauge();
		* var LinearGaugeObj = $("#LinearGauge1").data("ejLinearGauge");
		* LinearGaugeObj.setTickXDistanceFromScale(0,0,20);
		* &lt;/script&gt;
		*@memberof ejLinearGauge
		* @instance
        */
        setTickXDistanceFromScale: function (scaleIndex, TickIndex, value) {
            if (scaleIndex < this.Model.scales.length && TickIndex < this.Model.scales[scaleIndex].ticks.length && value != null) {
                this.scaleEl[scaleIndex].ticks[TickIndex].distanceFromScale.x = value;
                if (this.contextEl.putImageData) {
                    this._reDrawTickMark();
                }
                else
                    this.initialize();
            }
        },
		/**
        * To get get Tick XDistance From Scale in number
		* @return number
		* @param {number} argument.scaleIndex scaleIndex value for the Gauge
		* @param {number}  argument.TickIndex value for the Gauge
		* @example 
		* &lt;div id="LinearGauge1"&gt;&lt;/div&gt; <br>  
		* &lt;script&gt;
        * $("#LinearGauge1").ejLinearGauge();
		* var LinearGaugeObj = $("#LinearGauge1").data("ejLinearGauge");
		* LinearGaugeObj.getTickXDistanceFromScale(0,0);
		* &lt;/script&gt;
		*@memberof ejLinearGauge
		* @instance
        */
        getTickXDistanceFromScale: function (scaleIndex, TickIndex) {
            if (scaleIndex < this.Model.scales.length && TickIndex < this.Model.scales[scaleIndex].ticks.length)
                return this.scaleEl[scaleIndex].ticks[TickIndex].distanceFromScale.x;
        },
		/**
        * To set setTickYDistanceFromScale 
		* @return jQuery
		* @param {number} argument.scaleIndex scaleIndex value for the Gauge
		* @param {number}  argument.TickIndex value for the Gauge
		* @param {number} arguemnt.value Tick YDistance From Scalefor Gauge
		* @example 
		* &lt;div id="LinearGauge1"&gt;&lt;/div&gt; <br>  
		* &lt;script&gt;
        * $("#LinearGauge1").ejLinearGauge();
		* var LinearGaugeObj = $("#LinearGauge1").data("ejLinearGauge");
		* LinearGaugeObj.setTickYDistanceFromScale(0,0,20);
		* &lt;/script&gt;
		*@memberof ejLinearGauge
		* @instance
        */
        setTickYDistanceFromScale: function (scaleIndex, TickIndex, value) {
            if (scaleIndex < this.Model.scales.length && TickIndex < this.Model.scales[scaleIndex].ticks.length && value != null) {
                this.scaleEl[scaleIndex].ticks[TickIndex].distanceFromScale.y = value;
                if (this.contextEl.putImageData) {
                    this._reDrawTickMark();
                }
                else
                    this.initialize();
            }
        },
		/**
        * To get Tick YDistance From Scale in number
		* @return number
		* @param {number} argument.scaleIndex scaleIndex value for the Gauge
		* @param {number}  argument.TickIndex value for the Gauge
		* @example 
		* &lt;div id="LinearGauge1"&gt;&lt;/div&gt; <br>  
		* &lt;script&gt;
        * $("#LinearGauge1").ejLinearGauge();
		* var LinearGaugeObj = $("#LinearGauge1").data("ejLinearGauge");
		* LinearGaugeObj.getTickYDistanceFromScale(0,0);
		* &lt;/script&gt;
		*@memberof ejLinearGauge
		* @instance
        */
        getTickYDistanceFromScale: function (scaleIndex, TickIndex) {
            if (scaleIndex < this.Model.scales.length && TickIndex < this.Model.scales[scaleIndex].ticks.length)
                return this.scaleEl[scaleIndex].ticks[TickIndex].distanceFromScale.y;
        },
		/**
        * To set setScaleLocation 
		* @return jQuery
		* @param {number} argument.scaleIndex scaleIndex value for the Gauge
		* @param {object} arguemnt.value Scale position for Gauge
		* @example 
		* &lt;div id="LinearGauge1"&gt;&lt;/div&gt; <br>  
		* &lt;script&gt;
        * $("#LinearGauge1").ejLinearGauge();
		* var LinearGaugeObj = $("#LinearGauge1").data("ejLinearGauge");
		* LinearGaugeObj.setScaleLocation(0,{x:20,y:20});
		* &lt;/script&gt;
		*@memberof ejLinearGauge
		* @instance
        */
        setScaleLocation: function (scaleIndex, value) {
            if (scaleIndex < this.Model.scales.length && value != null) {
                this.scaleEl[scaleIndex].position.x = value.x;
                this.scaleEl[scaleIndex].position.y = value.y;
                this.initialize();
            }
        },
		/**
        * To get Scale Location in object
		* @return object
		* @param {number} argument.scaleIndex scaleIndex value for the Gauge
		* @example 
		* &lt;div id="LinearGauge1"&gt;&lt;/div&gt; <br>  
		* &lt;script&gt;
        * $("#LinearGauge1").ejLinearGauge();
		* var LinearGaugeObj = $("#LinearGauge1").data("ejLinearGauge");
		* LinearGaugeObj.getScaleLocation(0);
		* &lt;/script&gt;
		*@memberof ejLinearGauge
		* @instance
        */
        getScaleLocation: function (scaleIndex) {
            if (scaleIndex < this.Model.scales.length)
                return { "x": this.scaleEl[scaleIndex].position.x, "y": this.scaleEl[scaleIndex].position.y }
        },
		/**
        * To set setMaximumValue 
		* @return jQuery
		* @param {number} argument.scaleIndex scaleIndex value for the Gauge
		* @param {number} arguemnt.value MaximumValue for Gauge
		* @example 
		* &lt;div id="LinearGauge1"&gt;&lt;/div&gt; <br>  
		* &lt;script&gt;
        * $("#LinearGauge1").ejLinearGauge();
		* var LinearGaugeObj = $("#LinearGauge1").data("ejLinearGauge");
		* LinearGaugeObj.setMaximumValue(0,20);
		* &lt;/script&gt;
		*@memberof ejLinearGauge
		* @instance
        */
        setMaximumValue: function (scaleIndex, value) {
            if (scaleIndex < this.Model.scales.length && value != null) {
            if(value > this.scaleEl[scaleIndex].minimum)
                this.scaleEl[scaleIndex].maximum = value;
                this.initialize();
            }
        },
		/**
        * To get Maximum Value in number
		* @return number
		* @param {number} argument.scaleIndex scaleIndex value for the Gauge
		* @example 
		* &lt;div id="LinearGauge1"&gt;&lt;/div&gt; <br>  
		* &lt;script&gt;
        * $("#LinearGauge1").ejLinearGauge();
		* var LinearGaugeObj = $("#LinearGauge1").data("ejLinearGauge");
		* LinearGaugeObj.getMaximumValue(0);
		* &lt;/script&gt;
		*@memberof ejLinearGauge
		* @instance
        */
        getMaximumValue: function (scaleIndex) {
            if (scaleIndex < this.Model.scales.length)
                return this.scaleEl[this.scaleIndex].maximum;
        },
		/**
        * To set setMinimumValue 
		* @return jQuery
		* @param {number} argument.scaleIndex scaleIndex value for the Gauge
		* @param {number} arguemnt.value MinimumValue for Gauge
		* @example 
		* &lt;div id="LinearGauge1"&gt;&lt;/div&gt; <br>  
		* &lt;script&gt;
        * $("#LinearGauge1").ejLinearGauge();
		* var LinearGaugeObj = $("#LinearGauge1").data("ejLinearGauge");
		* LinearGaugeObj.setMinimumValue(0,20);
		* &lt;/script&gt;
		*@memberof ejLinearGauge
		* @instance
        */
        setMinimumValue: function (scaleIndex, value) {
            if (scaleIndex < this.Model.scales.length && value != null) {
            if(value < this.scaleEl[scaleIndex].maximum)
                this.scaleEl[scaleIndex].minimum = value;
                this.initialize();
            }
        },
		/**
        * To get PointerValue in number
		* @return number
		* @param {number} argument.scaleIndex scaleIndex value for the Gauge
		* @param {number}  argument.pointerIndex value for the Gauge
		* @example 
		* &lt;div id="LinearGauge1"&gt;&lt;/div&gt; <br>  
		* &lt;script&gt;
        * $("#LinearGauge1").ejLinearGauge();
		* var LinearGaugeObj = $("#LinearGauge1").data("ejLinearGauge");
		* LinearGaugeObj.getMinimumValue(0,0);
		* &lt;/script&gt;
		*@memberof ejLinearGauge
		* @instance
        */
        getMinimumValue: function (scaleIndex) {
            if (scaleIndex < this.Model.scales.length)
                return this.scaleEl[this.scaleIndex].minimum;
        },
		/**
        * To set setScaleBarSize 
		* @return jQuery
		* @param {number} argument.scaleIndex scaleIndex value for the Gauge
		* @param {number} arguemnt.value ScaleBarSize for Gauge
		* @example 
		* &lt;div id="LinearGauge1"&gt;&lt;/div&gt; <br>  
		* &lt;script&gt;
        * $("#LinearGauge1").ejLinearGauge();
		* var LinearGaugeObj = $("#LinearGauge1").data("ejLinearGauge");
		* LinearGaugeObj.setScaleBarSize(0,20);
		* &lt;/script&gt;
		*@memberof ejLinearGauge
		* @instance
        */
        setScaleBarSize: function (scaleIndex, value) {
            if (scaleIndex < this.Model.scales.length && value != null) {
                this.scaleEl[scaleIndex].width = value;
                this.initialize();
            }
        },
		/**
        * To get Scale Bar Size in number
		* @return number
		* @param {number} argument.scaleIndex scaleIndex value for the Gauge
		* @param {number}  argument.pointerIndex value for the Gauge
		* @example 
		* &lt;div id="LinearGauge1"&gt;&lt;/div&gt; <br>  
		* &lt;script&gt;
        * $("#LinearGauge1").ejLinearGauge();
		* var LinearGaugeObj = $("#LinearGauge1").data("ejLinearGauge");
		* LinearGaugeObj.getScaleBarSize(0,0);
		* &lt;/script&gt;
		*@memberof ejLinearGauge
		* @instance
        */
        getScaleBarSize: function (scaleIndex) {
            if (scaleIndex < this.Model.scales.length)
                return this.scaleEl[scaleIndex].width;
        },
		/**
        * To set setScaleBarLength 
		* @return jQuery
		* @param {number} argument.scaleIndex scaleIndex value for the Gauge
		* @param {number} arguemnt.value Scale Bar Length for Gauge
		* @example 
		* &lt;div id="LinearGauge1"&gt;&lt;/div&gt; <br>  
		* &lt;script&gt;
        * $("#LinearGauge1").ejLinearGauge();
		* var LinearGaugeObj = $("#LinearGauge1").data("ejLinearGauge");
		* LinearGaugeObj.setScaleBarLength(0,150);
		* &lt;/script&gt;
		*@memberof ejLinearGauge
		* @instance
        */
        setScaleBarLength: function (scaleIndex, value) {
            if (scaleIndex < this.Model.scales.length && value != null) {
                this.scaleEl[scaleIndex].length = value;
                this.initialize();
            }
        },
		/**
        * To set setScaleStyle 
		* @return jQuery
		* @param {number} argument.scaleIndex scaleIndex value for the Gauge
		* @param {number} arguemnt.value for Gauge
		* @example 
		* &lt;div id="LinearGauge1"&gt;&lt;/div&gt; <br>  
		* &lt;script&gt;
        * $("#LinearGauge1").ejLinearGauge();
		* var LinearGaugeObj = $("#LinearGauge1").data("ejLinearGauge");
		* LinearGaugeObj.setScaleStyle(0,"thermometer");
		* &lt;/script&gt;
		*@memberof ejLinearGauge
		* @instance
        */
        setScaleStyle: function (scaleIndex, value) {
            if (scaleIndex < this.Model.scales.length && value != null) {
                this.scaleEl[scaleIndex].type = value;
                this.initialize();
            }
        },
		/**
        * To get Scale Style in string
		* @return string
		* @param {number} argument.scaleIndex scaleIndex value for the Gauge
		* @example 
		* &lt;div id="LinearGauge1"&gt;&lt;/div&gt; <br>  
		* &lt;script&gt;
        * $("#LinearGauge1").ejLinearGauge();
		* var LinearGaugeObj = $("#LinearGauge1").data("ejLinearGauge");
		* LinearGaugeObj.getScaleStyle(0);
		* &lt;/script&gt;
		*@memberof ejLinearGauge
		* @instance
        */
        getScaleStyle: function (scaleIndex) {
            if (scaleIndex < this.Model.scales.length)
                return this.scaleEl[scaleIndex].type;
        },
		/**
        * To get ScaleBarLength in number
		* @return number
		* @param {number} argument.scaleIndex scaleIndex value for the Gauge
		* @example 
		* &lt;div id="LinearGauge1"&gt;&lt;/div&gt; <br>  
		* &lt;script&gt;
        * $("#LinearGauge1").ejLinearGauge();
		* var LinearGaugeObj = $("#LinearGauge1").data("ejLinearGauge");
		* LinearGaugeObj.getScaleBarLength(0);
		* &lt;/script&gt;
		*@memberof ejLinearGauge
		* @instance
        */
        getScaleBarLength: function (scaleIndex) {
            if (scaleIndex < this.Model.scales.length)
                return this.scaleEl[scaleIndex].length;
        },
		/**
        * To set setScaleBorderWidth 
		* @return jQuery
		* @param {number} argument.scaleIndex scaleIndex value for the Gauge
		* @param {number} arguemnt.value Scale Border Width for Gauge
		* @example 
		* &lt;div id="LinearGauge1"&gt;&lt;/div&gt; <br>  
		* &lt;script&gt;
        * $("#LinearGauge1").ejLinearGauge();
		* var LinearGaugeObj = $("#LinearGauge1").data("ejLinearGauge");
		* LinearGaugeObj.setScaleBorderWidth(0,10);
		* &lt;/script&gt;
		*@memberof ejLinearGauge
		* @instance
        */
        setScaleBorderWidth: function (scaleIndex, value) {
            if (scaleIndex < this.Model.scales.length && value != null) {
                this.scaleEl[scaleIndex].border.width = value;
                this.initialize();
            }
        },
		/**
        * To set setScaleDirection 
		* @return jQuery
		* @param {number} argument.scaleIndex scaleIndex value for the Gauge
		* @param {number} arguemnt.value Scale Direction for Gauge
		* @example 
		* &lt;div id="LinearGauge1"&gt;&lt;/div&gt; <br>  
		* &lt;script&gt;
        * $("#LinearGauge1").ejLinearGauge();
		* var LinearGaugeObj = $("#LinearGauge1").data("ejLinearGauge");
		* LinearGaugeObj.setScaleDirection(0,"counterclockwise");
		* &lt;/script&gt;
		*@memberof ejLinearGauge
		* @instance
        */
        setScaleDirection: function (scaleIndex, value) {
            if (scaleIndex < this.Model.scales.length && value != null) {
                this.scaleEl[scaleIndex].direction = value;
                this.initialize();
            }
        },
		/**
        * To get Scale Direction in number
		* @return number
		* @param {number} argument.scaleIndex scaleIndex value for the Gauge
		* @example 
		* &lt;div id="LinearGauge1"&gt;&lt;/div&gt; <br>  
		* &lt;script&gt;
        * $("#LinearGauge1").ejLinearGauge();
		* var LinearGaugeObj = $("#LinearGauge1").data("ejLinearGauge");
		* LinearGaugeObj.getScaleDirection(0,0);
		* &lt;/script&gt;
		*@memberof ejLinearGauge
		* @instance
        */
        getScaleDirection: function (scaleIndex) {
            if (scaleIndex < this.Model.scales.length)
                return this.scaleEl[scaleIndex].direction;
        },
		/**
        * To get Scale Border Width in number
		* @return number
		* @param {number} argument.scaleIndex scaleIndex value for the Gauge
		* @example 
		* &lt;div id="LinearGauge1"&gt;&lt;/div&gt; <br>  
		* &lt;script&gt;
        * $("#LinearGauge1").ejLinearGauge();
		* var LinearGaugeObj = $("#LinearGauge1").data("ejLinearGauge");
		* LinearGaugeObj.getScaleBorderWidth(0);
		* &lt;/script&gt;
		*@memberof ejLinearGauge
		* @instance
        */
        getScaleBorderWidth: function (scaleIndex) {
            if (scaleIndex < this.Model.scales.length)
                return this.scaleEl[scaleIndex].border.width;
        },
		/**
        * To set setMajorIntervalValue 
		* @return jQuery
		* @param {number} argument.scaleIndex scaleIndex value for the Gauge
		* @param {number} arguemnt.value Major Interval Value for Gauge
		* @example 
		* &lt;div id="LinearGauge1"&gt;&lt;/div&gt; <br>  
		* &lt;script&gt;
        * $("#LinearGauge1").ejLinearGauge();
		* var LinearGaugeObj = $("#LinearGauge1").data("ejLinearGauge");
		* LinearGaugeObj.setMajorIntervalValue(0,5);
		* &lt;/script&gt;
		*@memberof ejLinearGauge
		* @instance
        */
        setMajorIntervalValue: function (scaleIndex, value) {
            if (scaleIndex < this.Model.scales.length && value != null) {
                this.scaleEl[scaleIndex].majorIntervalValue = value;
                this.initialize();
            }
        },
		/**
        * To get Major Interval Value in number
		* @return number
		* @param {number} argument.scaleIndex scaleIndex value for the Gauge
		* @example 
		* &lt;div id="LinearGauge1"&gt;&lt;/div&gt; <br>  
		* &lt;script&gt;
        * $("#LinearGauge1").ejLinearGauge();
		* var LinearGaugeObj = $("#LinearGauge1").data("ejLinearGauge");
		* LinearGaugeObj.getMajorIntervalValue(0);
		* &lt;/script&gt;
		*@memberof ejLinearGauge
		* @instance
        */
        getMajorIntervalValue: function (scaleIndex) {
            if (scaleIndex < this.Model.scales.length)
                return this.scaleEl[scaleIndex].majorIntervalValue;
        },
		/**
        * To set setMinorIntervalValue 
		* @return jQuery
		* @param {number} argument.scaleIndex scaleIndex value for the Gauge
		* @param {number} arguemnt.value Minor Interval Value for Gauge
		* @example 
		* &lt;div id="LinearGauge1"&gt;&lt;/div&gt; <br>  
		* &lt;script&gt;
        * $("#LinearGauge1").ejLinearGauge();
		* var LinearGaugeObj = $("#LinearGauge1").data("ejLinearGauge");
		* LinearGaugeObj.setMinorIntervalValue(0,2);
		* &lt;/script&gt;
		*@memberof ejLinearGauge
		* @instance
        */
        setMinorIntervalValue: function (scaleIndex, value) {
            if (scaleIndex < this.Model.scales.length && value != null) {
                this.scaleEl[scaleIndex].minorIntervalValue = value;
                this.initialize();
            }
        },
		/**
        * To get Minor Interval Value in number
		* @return number
		* @param {number} argument.scaleIndex scaleIndex value for the Gauge
		* @example 
		* &lt;div id="LinearGauge1"&gt;&lt;/div&gt; <br>  
		* &lt;script&gt;
        * $("#LinearGauge1").ejLinearGauge();
		* var LinearGaugeObj = $("#LinearGauge1").data("ejLinearGauge");
		* LinearGaugeObj.getMinorIntervalValue(0);
		* &lt;/script&gt;
		*@memberof ejLinearGauge
		* @instance
        */
        getMinorIntervalValue: function (scaleIndex) {
            if (scaleIndex < this.Model.scales.length)
                return this.scaleEl[scaleIndex].minorIntervalValue;
        },
		/**
         * To redraw the bar pointer
		 * @private
         */
        _reDrawBarPointer: function () {
            if (this.Model.frame.backgroundImageUrl)
                this.initialize();
            else {
                if (this.contextEl.putImageData != "undefined") {
                    this.contextEl.putImageData(this._getBarPointerImage(), 0, 0);
                    this._setBarPointers();
                    this._setMarkerPointers();
                }

            }
        },
		/**
         * To redraw the Marker pointer
		 * @private
         */
        _reDrawMarkerPointer: function () {
            if (this.Model.frame.backgroundImageUrl)
                this.initialize();
            else {
                if (this.contextEl.putImageData != "undefined") {
                    this.contextEl.putImageData(this._getMarkerPointerImage(), 0, 0);
                    this._setMarkerPointers();
                }

            }
        },
		/**
         * To redraw the custom label
		 * @private
         */
        _reDrawCustomLabel: function () {
            if (this.Model.frame.backgroundImageUrl)
                this.initialize();
            else {
                this.contextEl.putImageData(this._getCustomLabelImage(), 0, 0);
                this._setCustomLabel();
                this._setIndicators();
                this._setBarPointers();
                this._setMarkerPointers();

            }
        },
		/**
         * To redraw the Range
		 * @private
         */
        _reDrawRange: function () {
            if (this.Model.frame.backgroundImageUrl)
                this.initialize();
            else {
                this.contextEl.putImageData(this._getRangeImage(), 0, 0);
                this._setRange();
                this._setCustomLabel();
                this._setIndicators();
                this._setBarPointers();
                this._setMarkerPointers();

            }
        },
		/**
         * To redraw the Label
		 * @private
         */
        _reDrawLabel: function () {
            if (this.Model.frame.backgroundImageUrl)
                this.initialize();
            else {
                this.contextEl.putImageData(this._getLabelImage(), 0, 0);
                this._setLabels();
                this._setRange();
                this._setCustomLabel();
                this._setIndicators();
                this._setBarPointers();
                this._setMarkerPointers();

            }
        },
		/**
         * To redraw the TickMark
		 * @private
         */
        _reDrawTickMark: function () {
            if (this.Model.frame.backgroundImageUrl)
                this.initialize();
            else {
                this.contextEl.putImageData(this._getTickImage(), 0, 0);
                this._setTicks();
                this._setLabels();
                this._setRange();
                this._setCustomLabel();
                this._setIndicators();
                this._setBarPointers();
                this._setMarkerPointers();

            }
        },
		/**
         * To refresh the gauge
		 * @private
         */
        refresh: function () {
            this._init();
        },
		/**
        * To export Image
		* @return jQuery
		* @param {number} argument.fileName for the Image
		* @param {number}  argument.fileType for the Image
		* @example 
		* &lt;div id="LinearGauge1"&gt;&lt;/div&gt; <br>  
		* &lt;script&gt;
        * $("#LinearGauge1").ejLinearGauge();
		* var LinearGaugeObj = $("#LinearGauge1").data("ejLinearGauge");
		* LinearGaugeObj.exportImage("myImage","jpeg");
		* &lt;/script&gt;
		*@memberof ejLinearGauge
		* @instance
        */
		exportImage: function(fileName, fileType) {
			/// <summary>This function save the rendered canvas image</summary>
            /// <param name="fileName" type="String">fileName to which the image has been saved</param>
		    /// <param name="fileType" type="String">fileType to which the image has been saved</param>
		    var image = this.canvasEl[0].toDataURL();
		    image = image.replace(/^data:[a-z]*;,/, '');
		    var image1 = image.split(',');
		    var byteString = atob(image1[1]);
		    var buffer = new ArrayBuffer(byteString.length);
		    var intArray = new Uint8Array(buffer);
		    for (var i = 0; i < byteString.length; i++) {
		        intArray[i] = byteString.charCodeAt(i);
		    }
		    blob = new Blob([buffer], { type: "image/png" });
		    if (ej.browserInfo().name === "msie" && parseFloat(ej.browserInfo().version) > 9)
		        window.navigator.msSaveOrOpenBlob(blob, fileName + '.' + fileType);
		    else {
		        var pom = document.createElement('a');
		        var url = URL.createObjectURL(blob);
		        pom.href = url;
		        pom.setAttribute('download', fileName + '.' + fileType);
		        if (document.createEvent) {
		            e = document.createEvent("MouseEvents");
		            e.initMouseEvent("click", true, true, window, 0, 0, 0, 0, 0, false, false, false, false, 0, null);
		            pom.dispatchEvent(e);
		        }
		        else if (pom.fireEvent) {
		            pom.fireEvent("onclick");
		        }
		    }
        },
		/**
         * To resize canvas while resizing the window
		 * @private
         */
		resizeCanvas:function () {
            if (proxy.element.width() != initialDivWidth || proxy.element.height() != initialDivHeight) {
                var canvasWidth = proxy.element.width() / initialDivWidth;
                var canvasHeight = proxy.element.height() / initialDivHeight;
                proxy.model.width *=canvasWidth;
                proxy.model.height *= canvasHeight;
				for(var i=0;proxy.model.scales[i]!=null;i++){
                if (proxy.element.data("ejLinearGauge").model.orientation.toString() == "Vertical")
                    proxy.element.data("ejLinearGauge").model.scales[i].length *= canvasHeight;
                else
                    proxy.element.data("ejLinearGauge").model.scales[i].length *= canvasWidth;
				for(var j=0;proxy.model.scales[i].markerPointers[j]!=null;j++){
                proxy.model.scales[i].markerPointers[j].length *= canvasWidth;
                proxy.model.scales[i].markerPointers[j].width *= canvasWidth;
				}
				}
                proxy.refresh();
                initialDivWidth = proxy.element.width();
                initialDivHeight = proxy.element.height();
            }
        },
        //Client-Side Events
		/**
         * Section For handle the DrawTicks event
		 * @private
         */
		_onDrawTicks: function (tickAngle, tickValue) {
		    var tick = { index: this.tickIndex, element: this.tickEl[this.tickIndex], angle: parseInt(tickAngle), value: tickValue }
            var data = { Object: this, Model: this.Model, scaleElement: this.Model.scales[this.scaleIndex], scaleIndex: this.scaleIndex, context: this.contextEl, style: this.style, position: this.location };
            this._trigger("drawTicks", data);
        },
		/**
         * Section For handle the DrawLabels event
		 * @private
         */
		_onDrawLabels: function (labelAngle) {
		    var label = { index: this.labelIndex, element: this.labelEl[this.labelIndex], angle: parseInt(labelAngle), value: this.labelValue }
		    var data = { object: this, Model: this.Model, scaleElement: this.Model.scales[this.scaleIndex], scaleIndex: this.scaleIndex, context: this.contextEl, style: this.style, position: this.location, label: label };
		    this._trigger("drawLabels", data);
		},
		/**
         * Section For handle the DrawBarPointers event
		 * @private
         */
        _onDrawBarPointers: function (pointerValue) {
            var data = { object: this, Model: this.Model, scaleElement: this.Model.scales[this.scaleIndex], scaleIndex: this.scaleIndex, barPointerIndex: this.barPointerIndex, barElement: this.barPointerEl[this.barPointerIndex], context: this.contextEl, style: this.style, position: this.location, pointerValue: pointerValue };
            this._trigger("drawBarPointers", data);
        },
		/**
         * Section For handle the DrawMarkerPointers event
		 * @private
         */
        _onDrawMarkerPointers: function (pointerAngle, pointerValue) {
            var data = { object: this, Model: this.Model, scaleElement: this.Model.scales[this.scaleIndex], scaleIndex: this.scaleIndex, markerPointerIndex: this.markerPointerIndex, markerElement: this.markerPointerEl[this.markerPointerIndex], context: this.contextEl, style: this.style, position: this.location, pointerValue: pointerValue, pointerAngle: parseInt(pointerAngle) };
            this._trigger("drawMarkerPointers", data);
        },
		/**
         * Section For handle the DrawRange event
		 * @private
         */
        _onDrawRange: function () {
            var data = { object: this, Model: this.Model, scaleElement: this.Model.scales[this.scaleIndex], scaleIndex: this.scaleIndex, rangeIndex: this.rangeIndex, rangeElement: this.rangeEl[this.rangeIndex], context: this.contextEl, style: this.style, position: this.location };
            this._trigger("drawRange", data);
        },
		/**
         * Section For handle the DrawCustomLabel event
		 * @private
         */
        _onDrawCustomLabel: function () {
            var data = { object: this, Model: this.Model, scaleElement: this.Model.scales[this.scaleIndex], scaleIndex: this.scaleIndex, customLabelIndex: this.customLabelIndex, customLabelElement: this.customLabelEl[this.customLabelIndex], context: this.contextEl, style: this.style, position: this.location };
            this._trigger("drawCustomLabel", data);
        },
		/**
         * Section For handle the DrawIndicators event
		 * @private
         */
        _onDrawIndicators: function (location, style) {
            var data = { object: this, Model: this.Model, scaleElement: this.Model.scales[this.scaleIndex], scaleIndex: this.scaleIndex, indicatorIndex: this.indicatorIndex, indicatorEl: this.indicatorEl[this.indicatorIndex], context: this.contextEl, style: this.style, position: this.location };
            this._trigger("drawIndicators", data);
        },
		/**
         * Invoke while onload
		 * @private
         */
        onLoad: function () {
            var data = { object: this, Model: this.Model, scaleElement: this.Model.scales, context: this.contextEl };
            this._trigger("load", data);
        },
		/**
         * Invoke while initialize
		 * @private
         */
        _onInit: function () {
            var data = { object: this, Model: this.Model, scaleElement: this.Model.scales, context: this.contextEl };
            this._trigger("init", data);
        },
		/**
         * Invoke while render complete
		 * @private
         */
        _onRenderComplete: function () {
            var data = { object: this, Model: this.Model, scaleElement: this.Model.scales, context: this.contextEl };
            this._trigger("renderComplete", data);
        },
        /**
         * Section For handle the MouseClick event
		 * @private
         */
        _onMouseClick: function (pointerValue) {
            var markerpointer = { index: this.markerPointerIndex, element: this.markerPointerEl[this.markerPointerIndex], value: pointerValue }
            var data = { object: this, scaleElement: this.model.scales[this.scaleIndex], scaleIndex: this.scaleIndex, context: this.contextEl, style: this.style, position: this.location };
            this._trigger("mouseClick", data);
        },
        /**
         * Section For handle the MouseClickMove event
		 * @private
         */
        _onMouseClickMove: function (pointerValue) {
            var markerpointer = { index: this.markerPointerIndex, element: this.markerPointerEl[this.markerPointerIndex], value: pointerValue }
            var data = { object: this, scaleElement: this.model.scales[this.scaleIndex], scaleIndex: this.scaleIndex, context: this.contextEl, style: this.style, position: this.location };
            this._trigger("mouseClickMove", data);
        },
        /**
         * Section For handle the MouseClickUp event
		 * @private
         */
        _onMouseClickUp: function (pointerValue) {
            var markerpointer = { index: this.markerPointerIndex, element: this.markerPointerEl[this.markerPointerIndex], value: pointerValue }
            var data = { object: this, scaleElement: this.model.scales[this.scaleIndex], scaleIndex: this.scaleIndex, context: this.contextEl, style: this.style, position: this.location };
            this._trigger("mouseClickUp", data);
        },
        //_trigger: function (type, event, data) {
        //    var fn = this.Model[type];
        //    event = $.Event(event);
        //    event.type = type;
        //    $(this.GaugeEl).trigger(event, data);
        //    return !($.isFunction(fn) &&
        //        fn(this, data) === false ||
        //        event.isDefaultPrevented());
        //},

        //trigger: function (element, eventName, e) {
        //    e = $.extend(e || {}, new $.Event(eventName));
        //    e.stopPropagation();
        //    $(element).trigger(e);
        //    return e.isDefaultPrevented();
        //},
		/**
         * To restore width
		 * @private
         */
        _restoreWidth: function () {
            this.scaleEl[this.scaleIndex].length = this.scaleEl[this.scaleIndex].length + this.bottomRadius + this.scaleEl[this.scaleIndex].width;
            if (this.Model.orientation == "Vertical") {
                if (this.scaleEl[this.scaleIndex].direction == "clockwise")
                    this.scaleStartY[this.scaleIndex] = this.scaleStartY[this.scaleIndex] - this.bottomRadius - this.scaleEl[this.scaleIndex].width / 2;
                else
                    this.scaleStartY[this.scaleIndex] = this.scaleStartY[this.scaleIndex] - this.scaleEl[this.scaleIndex].width / 2;
            }
            else {
                if (this.scaleEl[this.scaleIndex].direction == "clockwise")
                    this.scaleStartX[this.scaleIndex] = this.scaleStartX[this.scaleIndex] - this.bottomRadius - this.scaleEl[this.scaleIndex].width / 2;
                else
                    this.scaleStartX[this.scaleIndex] = this.scaleStartX[this.scaleIndex] - this.scaleEl[this.scaleIndex].width / 2;
            }
        },
		/**
         * To modify width
		 * @private
         */
        _modifyWidth: function () {
            this.scaleEl[this.scaleIndex].length = this.scaleEl[this.scaleIndex].length - this.bottomRadius - this.scaleEl[this.scaleIndex].width;
            if (this.Model.orientation == "Vertical") {
                if (this.scaleEl[this.scaleIndex].direction == "clockwise")
                    this.scaleStartY[this.scaleIndex] = this.scaleStartY[this.scaleIndex] + this.bottomRadius + this.scaleEl[this.scaleIndex].width / 2;
                else
                    this.scaleStartY[this.scaleIndex] = this.scaleStartY[this.scaleIndex] + this.scaleEl[this.scaleIndex].width / 2;
            }
            else {
                if (this.scaleEl[this.scaleIndex].direction == "clockwise")
                    this.scaleStartX[this.scaleIndex] = this.scaleStartX[this.scaleIndex] + this.bottomRadius + this.scaleEl[this.scaleIndex].width / 2;
                else
                    this.scaleStartX[this.scaleIndex] = this.scaleStartX[this.scaleIndex] + this.scaleEl[this.scaleIndex].width / 2;
            }
        },
        // return y positions for vertical gauge
		/**
         * To get Clockwise LinePosition
		 * @private
         */
        _getClockwiseLinePosition: function (value) {
            var tempVal, linePosition;
            tempVal = (value - this.scaleEl[this.scaleIndex].minimum) / (this.scaleEl[this.scaleIndex].maximum - this.scaleEl[this.scaleIndex].minimum) * 100;
            linePosition = this.Model.orientation == "Vertical" ? this.scaleStartY[this.scaleIndex] + parseInt((tempVal * this.scaleEl[this.scaleIndex].length) / 100) : this.scaleStartX[this.scaleIndex] + parseInt((tempVal * this.scaleEl[this.scaleIndex].length) / 100);
            return linePosition;
        },
		/**
         * To get Counter Clockwise LinePosition
		 * @private
         */
        _getCounterClockwiseLinePosition: function (value) {
            var tempVal, linePosition;
            tempVal = this.scaleEl[this.scaleIndex].maximum - value + this.scaleEl[this.scaleIndex].minimum;
            tempVal = (tempVal - this.scaleEl[this.scaleIndex].minimum) / (this.scaleEl[this.scaleIndex].maximum - this.scaleEl[this.scaleIndex].minimum) * 100;
            linePosition = this.Model.orientation == "Vertical" ? this.scaleStartY[this.scaleIndex] + parseInt((tempVal * this.scaleEl[this.scaleIndex].length) / 100) : this.scaleStartX[this.scaleIndex] + parseInt((tempVal * this.scaleEl[this.scaleIndex].length) / 100);
            return linePosition;
        },
		/**
         * To get value
		 * @private
         */
        _getValue: function (position) {
            if (this.Model.orientation == "Vertical")
                tempVal = ((position.y - this.scaleStartY[this.scaleIndex]) / this.scaleEl[this.scaleIndex].length) * 100;
            else
                tempVal = (((position.x - this.scaleStartX[this.scaleIndex])) / this.scaleEl[this.scaleIndex].length) * 100;
            value = ((tempVal * (this.scaleEl[this.scaleIndex].maximum - this.scaleEl[this.scaleIndex].minimum)) + this.scaleEl[this.scaleIndex].minimum) / 100;
            if (this.scaleEl[this.scaleIndex].direction == "counterclockwise")
                value = this.scaleEl[this.scaleIndex].maximum + this.scaleEl[this.scaleIndex].minimum - value;
            return value;
        },
		/**
         * To get Pointer X Position
		 * @private
         */
        _getPointerXPosition: function (element) {
            if (this.Model.orientation == "Vertical") {
                if (this.markerPlacement == "far") {
                    startX = this.scaleStartX[this.scaleIndex] + this.scaleEl[this.scaleIndex].width + this.scaleEl[this.scaleIndex].border.width / 2 + element.distanceFromScale;
                    angle = 0;
                }
                if (this.markerPlacement == "near") {
                    startX = this.scaleStartX[this.scaleIndex] + element.distanceFromScale;
                    angle = 180;
                }
                if (this.markerPlacement == "center") {
                    if (element.type == "circle")
                        startX = this.scaleStartX[this.scaleIndex] + this.scaleEl[this.scaleIndex].width / 2 - radius + element.distanceFromScale;
                    else
                        startX = this.scaleStartX[this.scaleIndex] + this.scaleEl[this.scaleIndex].width / 2 - element.width / 2 + element.distanceFromScale;
                    angle = 0;
                }
            }
            else {
                if (this.markerPlacement == "far") {
                    startX = this.scaleStartY[this.scaleIndex] + this.scaleEl[this.scaleIndex].width + this.scaleEl[this.scaleIndex].border.width / 2 + element.distanceFromScale;
                    angle = 90;
                }
                if (this.markerPlacement == "near") {
                    startX = this.scaleStartY[this.scaleIndex] - this.scaleEl[this.scaleIndex].border.width / 2 + element.distanceFromScale;
                    angle = 270;
                }
                if (this.markerPlacement == "center") {
                    if (element.type == "circle")
                        startX = this.scaleStartY[this.scaleIndex] + this.scaleEl[this.scaleIndex].width / 2 - radius + element.distanceFromScale;
                    else
                        startX = this.scaleStartY[this.scaleIndex] + this.scaleEl[this.scaleIndex].width / 2 - element.length / 2 + element.distanceFromScale;
                    angle = 90;
                }
            }
            return { "startx": startX, "angle": angle };
        },
		/**
         * To get hexa decimal value from RGB combination
		 * @private
         */
        _hexFromRGB: function (r, g, b) {
            var hex = [r.toString(16), g.toString(16), b.toString(16)];
            $.each(hex, function (nr, val) { if (val.length === 1) { hex[nr] = "0" + val; } });
            return hex.join("").toUpperCase();
        },
		/**
         * To set GradientColor
		 * @private
         */
        _setGradientColor: function (element, gradients, options) {
            var self = element;
            if (options.Name || typeof (options) === "string") {
                gradients.addColorStop(0, options);
                gradients.addColorStop(1, options);
            }
            else
                $.each(options, function (index, colorElement) {
                    gradients.addColorStop(colorElement.ColorStop != NaN ? colorElement.ColorStop : 0, typeof (colorElement.Color) === "string" ? colorElement.Color : colorElement.Color);
                });
        },
		/**
         * To get Font String
		 * @private
         */
        _getFontString: function (element, font) {
            return font.fontStyle + " " + ((font.size == null) ? "11px" : font.size) + " " + font.fontFamily;
        },
		/**
         * To get  Pointer Dimension
		 * @private
         */
        _setPointerDimension: function (style, element) {
            if (element.Model.orientation) {
                if (element.Model.orientation == "Horizontal") {
                    var tempWidth = style.width;
                    var tempHeight = style.height;
                    style.height = tempWidth;
                    style.width = tempHeight;
                }
            }
            return style;
        },
		/**
         * To set  cntext Rotation
		 * @private
         */
        _setContextRotation: function (style, element) {
            element.contextEl.rotate(Math.PI * (style.angle / 180));
        },
		/**
         * To wire  the events
		 * @private
         */
        wireEvents: function () {
            this.startEv = 'mousedown';
            this.moveEv = 'mousemove';
            this.endEv = 'mouseup';
            //this.onMouseMoveHandler = $.proxy(this._onMouseMove, this);
            //this.onMouseUpHandler = $.proxy(this._onMouseUp, this);
            this.onMouseDownHandler = $.proxy(this._onMouseDown, this);
            this.element.bind(this.startEv, this.onMouseDownHandler);
        },
		/**
         * To Unwire  the events
		 * @private
         */
        unWireEvents: function () {
            this.element.unbind(this.startEv, this.onMouseDownHandler);
        },
		/**
         * To block Default Actions
		 * @private
         */
        _blockDefaultActions: function (e) {
            e.cancelBubble = true;
            e.returnValue = false;
            if (e.preventDefault) e.preventDefault();
            if (e.stopPropagation) e.stopPropagation();
        },
		/**
         * Section For handle the Mouse Down event
		 * @private
         */	
        _onMouseDown: function (e) {
            var startPoint, endPoint, value, position, greaterValue, lesserValue, startX, isOffset;
            this._blockDefaultActions(e);
            startPoint = { "x": this.centerX, "y": this.centerY };
            endPoint = { "x": e.pageX - $(this.canvasEl).offset().left - (this.Model.frame.outerWidth + this.Model.frame.innerWidth), "y": e.pageY - $(this.canvasEl).offset().top - (this.Model.frame.outerWidth + this.Model.frame.innerWidth) };
            offsetPoint = { "x": e.pageX - $(this.canvasEl).offset().left, "y": e.pageY - $(this.canvasEl).offset().top };
            value = this._getValue(endPoint);
            var self = this;
            $.each(this.Model.scales, function (index, element) {
                self.scaleIndex = index;
                if (element.markerPointers != null) {
                    self.markerPointerEl = element.markerPointers;
                    $.each(element.markerPointers, function (index, element) {
                        if (self.scaleEl[self.scaleIndex].direction == "clockwise")
                            position = self._getClockwiseLinePosition(element.value);
                        else
                            position = self._getCounterClockwiseLinePosition(element.value);
                        greaterValue = position + element.width;
                        lesserValue = position - element.width;
                        startX = self._getPointerXPosition(element).startx;
                        var isOffset = self._isBetween(((self.Model.orientation == "Horizontal" ? endPoint.y : endPoint.x) - element.width), ((self.Model.orientation == "Horizontal" ? endPoint.y : endPoint.x) + element.width), startX);
                        if ((self.Model.orientation == "Horizontal" ? self._isBetween(lesserValue, greaterValue, endPoint.x) : self._isBetween(lesserValue, greaterValue, endPoint.y)) && isOffset)
                            self.activeElement = element;
                        if (self.Model.scales[self.scaleIndex].barPointers[index] != null)
                            self.activeBarElement = self.Model.scales[self.scaleIndex].barPointers[index];
                        if (self.model.mouseClick)
                            self._onMouseClick(element.value);
                        self.onMouseMoveHandler = $.proxy(self._onMouseMove, self);
                        self.onMouseUpHandler = $.proxy(self._onMouseUp, self);
                        $(document).bind(self.moveEv, self.onMouseMoveHandler);
                        $(document).bind(self.endEv, self.onMouseUpHandler);
                    });
                }
            });
        },
		/**
         * To check whether the given value is in between first and Last arguemnet
		 * @private
         */	
        _isBetween: function (first, last, number) {
            return (first < last ? number >= first && number <= last : number >= last && number <= first);
        },
		/**
         * Section For handle the Mouse up event
		 * @private
         */	
        _onMouseUp: function () {
            this.mouseMove = false;
            $(document).unbind(self.moveEv, self.onMouseMoveHandler);
            $(document).unbind(self.endEv, self.onMouseUpHandler);
            if (this.model.mouseClickUp)
                this._onMouseClickUp(this.activeElement.value);
        },
		/**
         * Section For handle the Mouse move event
		 * @private
         */	
        _onMouseMove: function (e) {
            this._blockDefaultActions(e);
            startPoint = { "x": this.centerX, "y": this.centerY };
            endPoint = { "x": e.pageX - $(this.canvasEl).offset().left - (this.Model.frame.outerWidth + this.Model.frame.innerWidth), "y": e.pageY - $(this.canvasEl).offset().top - (this.Model.frame.outerWidth + this.Model.frame.innerWidth) };
            this.activeElement.value = this._getValue(endPoint);
            if (this.model.mouseClickMove)
                this._onMouseClickMove(this.activeElement.value);
            if (this.activeBarElement)
                this.activeBarElement.value = this._getValue(endPoint);
            if (this.contextEl.putImageData)
                this._reDrawBarPointer();
            else
                this._init();
        },

        //************************* Skin Functions **********************************//
		/**
         * To set theme
		 * @private
         */	
        _setTheme: function () {
            selectedTheme = ej.datavisualization.LinearGauge.Themes[this.model.theme];
            this._setThemeColors(selectedTheme);
        },
		/**
         * To set theme color
		 * @private
         */	
        _setThemeColors: function (selectedTheme) {
            var result = [], jsonObj = ej.datavisualization.LinearGauge.Themes;
            for (var name in jsonObj) {
                result.push(name);
            }
            for (var th = 0; th < result.length; th++) {
                this.model.backgroundColor = ((!this.model.backgroundColor || this.model.backgroundColor == jsonObj[result[th]].scales.backgroundColor) ? selectedTheme.scales.backgroundColor : this.model.backgroundColor);
                this.model.borderColor = ((!this.model.borderColor || this.model.borderColor == jsonObj[result[th]].scales.border.color) ? selectedTheme.scales.border.color : this.model.borderColor);
                this.model.labelColor = ((!this.model.labelColor || this.model.labelColor == jsonObj[result[th]].scales.labels.labelColor) ? selectedTheme.scales.labels.labelColor : this.model.labelColor);
                this.model.tickColor = ((!this.model.tickColor || this.model.tickColor == jsonObj[result[th]].scales.ticks.color) ? selectedTheme.scales.ticks.color : this.model.tickColor);

                for (var i = 0; i < this.model.scales.length; i++) {
                    for (var m = 0; m < this.model.scales[i].markerPointers.length; m++) {
                        this.model.scales[i].markerPointers[m].backgroundColor = (!this.model.scales[i].markerPointers[m].backgroundColor || this.model.scales[i].markerPointers[m].backgroundColor == jsonObj[result[th]].scales.markerPointers.backgroundColor) ? selectedTheme.scales.markerPointers.backgroundColor : this.model.scales[i].markerPointers[m].backgroundColor;
                        this.model.scales[i].markerPointers[m].border.color = (!this.model.scales[i].markerPointers[m].border.color || this.model.scales[i].markerPointers[m].border.color == jsonObj[result[th]].scales.markerPointers.border.color) ? selectedTheme.scales.markerPointers.border.color : this.model.scales[i].markerPointers[m].border.color;
                    }
                    for (var b = 0; b < this.model.scales[i].barPointers.length; b++) {
                        this.model.scales[i].barPointers[b].backgroundColor = (!this.model.scales[i].barPointers[b].backgroundColor || this.model.scales[i].barPointers[b].backgroundColor == jsonObj[result[th]].scales.barPointers.backgroundColor) ? selectedTheme.scales.barPointers.backgroundColor : this.model.scales[i].barPointers[b].backgroundColor;
                        this.model.scales[i].barPointers[b].border.color = (!this.model.scales[i].barPointers[b].border.color || this.model.scales[i].barPointers[b].border.color == jsonObj[result[th]].scales.barPointers.border.color) ? selectedTheme.scales.barPointers.border.color : this.model.scales[i].barPointers[b].border.color;
                    }
                }
            }
        }
    });
    /**
	 * Enum for gauge TickType	 
	 * @enum {string}
	 * @global 
	 */
    ej.datavisualization.LinearGauge.TickType = {
        /**  To set the TickType majorinterval. */
        MajorInterval: "majorinterval",
        /**  To set the TickType minorinterval. */
        MinorInterval: "minorinterval"
    };
    /**
	 * Enum for gauge LabelType	 
	 * @enum {string}
	 * @global 
	 */
    ej.datavisualization.LinearGauge.LabelType = {
        /**  To set the LabelType major. */
        Major: "major",
        /**  To set the LabelType minor. */
        Minor: "minor"
    };
    /**
	 * Enum for gauge FontStyle	 
	 * @enum {string}
	 * @global 
	 */
    ej.datavisualization.LinearGauge.FontStyle = {
        /**  To set the FontStyle bold. */
        Bold: "bold",
        /**  To set the FontStyle italic. */
        Italic: "italic",
        /**  To set the FontStyle regular. */
        Regular: "regular",
        /**  To set the FontStyle strikeout. */
        Strikeout: "strikeout",
        /**  To set the FontStyle underline. */
        Underline: "underline"
    };


    /**
	 * Enum for gauge PointerPlacement	 
	 * @enum {string}
	 * @global 
	 */
    ej.datavisualization.LinearGauge.PointerPlacement = {
        /**  To set the PointerPlacement near. */
        Near: "near",
        /**  To set the PointerPlacement far. */
        Far: "far",
        /**  To set the PointerPlacement center. */
        Center: "center"
    };
    /**
	 * Enum for gauge TickPlacement	 
	 * @enum {string}
	 * @global 
	 */
    ej.datavisualization.LinearGauge.TickPlacement = {
        /**  To set the TickPlacement near. */
        Near: "near",
        /**  To set the TickPlacement far. */
        Far: "far",
        /**  To set the TickPlacement center. */
        Center: "center"
    };
    /**
	 * Enum for gauge LabelPlacement	 
	 * @enum {string}
	 * @global 
	 */
    ej.datavisualization.LinearGauge.LabelPlacement = {
        /**  To set the LabelPlacement near. */
        Near: "near",
        /**  To set the LabelPlacement far. */
        Far: "far",
        /**  To set the LabelPlacement center. */
        Center: "center"
    };
    /**
	 * Enum for gauge RangePlacement	 
	 * @enum {string}
	 * @global 
	 */
    ej.datavisualization.LinearGauge.RangePlacement = {
        /**  To set the RangePlacement near. */
        Near: "near",
        /**  To set the RangePlacement far. */
        Far: "far",
        /**  To set the RangePlacement center. */
        Center: "center"
    };
    /**
	 * Enum for gauge UnitTextPlacement	 
	 * @enum {string}
	 * @global 
	 */

    ej.datavisualization.LinearGauge.UnitTextPlacement = {
        /**  To set the UnitTextPlacement front. */
        Front: "front",
        /**  To set the UnitTextPlacement back. */
        Back: "back"
    };
    /**
	 * Enum for gauge Directions	 
	 * @enum {string}
	 * @global 
	 */

    ej.datavisualization.LinearGauge.Directions = {
        /**  To set the Directions clockwise. */
        Clockwise: "clockwise",
        /**  To set the Directions counterclockwise. */
        CounterClockwise: "counterclockwise"
    };
    /**
	 * Enum for gauge ScaleType	 
	 * @enum {string}
	 * @global 
	 */
    ej.datavisualization.LinearGauge.ScaleType = {
        /**  To set the ScaleType rectangle. */
        Rectangle: "rectangle",
        /**  To set the ScaleType roundedrectangle. */
        RoundedRectangle: "roundedrectangle",
        /**  To set the ScaleType thermometer. */
        Thermometer: "thermometer"
    };
    /**
	 * Enum for gauge IndicatorType	 
	 * @enum {string}
	 * @global 
	 */
    ej.datavisualization.LinearGauge.IndicatorType = {
        /**  To set the IndicatorType rectangle. */
        Rectangle: "rectangle",
        /**  To set the IndicatorType circle. */
        Circle: "circle",
        /**  To set the IndicatorType roundedrectangle. */
        RoundedRectangle: "roundedrectangle",
        /**  To set the IndicatorType text. */
        Text: "text"
    };
    /**
	 * Enum for gauge MarkerType	 
	 * @enum {string}
	 * @global 
	 */
    ej.datavisualization.LinearGauge.MarkerType = {
        /**  To set the MarkerType rectangle. */
        Rectangle: "rectangle",
        /**  To set the MarkerType triangle. */
        Triangle: "triangle",
        /**  To set the MarkerType ellipse. */
        Ellipse: "ellipse",
        /**  To set the MarkerType diamond. */
        Diamond: "diamond",
        /**  To set the MarkerType pentagon. */
        Pentagon: "pentagon",
        /**  To set the MarkerType circle. */
        Circle: "circle",
        /**  To set the MarkerType star. */
        Star: "star",
        /**  To set the MarkerType slider. */
        Slider: "slider",
        /**  To set the MarkerType pointer. */
        Pointer: "pointer",
        /**  To set the MarkerType wedge. */
        Wedge: "wedge",
        /**  To set the MarkerType trapezoid. */
        Trapezoid: "trapezoid",
        /**  To set the MarkerType roundedrectangle. */
        RoundedRectangle: "roundedrectangle"
    };
	/**
	 * Enum for gauge Themes	 
	 * @enum {string}
	 * @global 
	 */
    ej.datavisualization.LinearGauge.Themes = {
	/**  To set the themes flatlight. */
        flatlight: {
            scales: {
                backgroundColor: "#FFFFFF",
                border: { color: "#1d1e1e" },
                barPointers: {
                    backgroundColor: "#8abc3b",
                    border: { color: "#8abc3b" }
                },
                markerPointers: {
                    backgroundColor: "#212121",
                    border: { color: "#212121" }
                },
                ticks: {
                    color: "#1d1e1e"
                },
                labels: {
                    labelColor: "#222222"
                }
            }
        },
		/**  To set the themes flatdark. */
        flatdark: {
            scales: {
                backgroundColor: "#808080",
                border: { color: "#808080" },
                barPointers: {
                    backgroundColor: "#8abc3b",
                    border: { color: "#8abc3b" }
                },
                markerPointers: {
                    backgroundColor: "#CCCCCC",
                    border: { color: "#CCCCCC" }
                },
                ticks: {
                    color: "#808080"
                },
                labels: {
                    labelColor: "#CCCCCC"
                }
            }
        }
    };

})(jQuery, Syncfusion);
;;
/**
* @fileOverview Plugin to style the Html Button elements
* @copyright Copyright Syncfusion Inc. 2001 - 2014. All rights reserved.
*  Use of this code is subject to the terms of our license.
*  A copy of the current license can be obtained at any time by e-mailing
*  licensing@syncfusion.com. Any infringement will be prosecuted under
*  applicable laws. 
* @version 12.1 
* @author <a href="mailto:licensing@syncfusion.com">Syncfusion Inc</a>
*/


(function ($, ej, undefined) {
    // Example plugin creation code
    // ejSample is the plugin name 
    // "ej.Sample" is "namespace.className" will hold functions and properties
    /**
    * @namespace ej
    * @classdesc The Digital gauge  can be easily configured to the DOM element, such as div. you can create a digital gauge with a highly customizable look and feel.
    * @class ejDigitalGauge
    * @param {object} options - For seting the Digital gauge    
    * @requires jQuery.js
    * @requires jquery.easing.js
    * @requires ej.common.all.js
    * @requires excanvas.js
    * @example 
    * &lt;div id="DigitalCore"&gt;&lt;/div&gt; <br> 
    * &lt;script&gt;
    * $('#DigitalCore').ejDigitalGauge();         
    * &lt;/script&gt;
    * @remarks
    * excanvas.js need to add for IE8 
    */



    ej.widget({ "ejDigitalGauge": "ej.datavisualization.DigitalGauge" }, {
        // widget element will be automatically set in this
        element: null,
        // user defined model will be automatically set in this
        model: null,
        validTags: ["div", "span"],
        _rootCSS: "e-digitalgauge",
        // default model
        defaults: /** @lends ejDigitalGauge# */{
            /**		
            * Specifies the segmentData for the DigitalGauge.
            */

            segmentData: {
                "0": [0, 1, 2, 3, 4, 5, 14, 15], "1": [1, 2], "2": [0, 14, 1, 6, 8, 4, 3, 15], "3": [0, 1, 2, 3, 6, 8, 14, 15], "4": [1, 2, 5, 6, 8], "5": [0, 2, 3, 5, 6, 8, 14, 15], "6": [0, 2, 3, 4, 5, 6, 8, 14, 15],
                "7": [0, 1, 2, 14], "8": [0, 1, 2, 3, 4, 5, 6, 8, 14, 15], "9": [0, 1, 2, 3, 5, 6, 8, 14, 15], "A": [0, 1, 2, 4, 5, 6, 8, 14], "B": [0, 1, 2, 3, 7, 9, 8, 14, 15], "C": [0, 3, 4, 5, 14, 15],
                "D": [0, 1, 2, 3, 7, 9, 14, 15], "E": [0, 3, 4, 5, 6, 8, 14, 15], "F": [0, 4, 5, 6, 8, 14], "G": [0, 2, 3, 4, 5, 8, 14, 15], "H": [1, 2, 4, 5, 6, 8], "I": [0, 3, 7, 9, 14, 15],
                "J": [1, 2, 3, 4, 15], "K": [4, 5, 6, 10, 11], "L": [3, 4, 5, 15], "M": [1, 2, 4, 5, 10, 13], "N": [1, 2, 4, 5, 11, 13], "O": [0, 1, 2, 3, 4, 5, 14, 15], "P": [0, 1, 4, 5, 6, 8, 14],
                "Q": [0, 1, 2, 3, 4, 5, 11, 14, 15], "R": [0, 1, 4, 5, 6, 8, 11, 14], "S": [0, 2, 3, 5, 6, 8, 14, 15], "T": [0, 7, 9, 14], "U": [1, 2, 3, 4, 5, 15], "V": [4, 5, 10, 12], "W": [1, 2, 4, 5, 11, 12],
                "X": [10, 11, 12, 13], "Y": [1, 5, 6, 7, 8], "Z": [0, 3, 10, 12, 14, 15]
            },
            /**		
            * Specifies the matrixSegmentData for the DigitalGauge.
            */
            matrixSegmentData: {
                "1": [0, 3, 0, 4, 1, 1, 1, 2, 1, 3, 1, 4, 2, 3, 2, 4, 3, 3, 3, 4, 4, 3, 4, 4, 5, 3, 5, 4, 6, 1, 6, 2, 6, 3, 6, 4, 6, 5, 6, 6],
                "2": [0, 1, 0, 2, 0, 3, 0, 4, 0, 5, 1, 5, 1, 6, 2, 5, 2, 6, 3, 4, 3, 5, 4, 3, 4, 2, 5, 2, 5, 1, 6, 1, 6, 2, 6, 3, 6, 4, 6, 5, 6, 6],
                "3": [0, 1, 0, 2, 0, 3, 0, 4, 0, 5, 1, 5, 1, 6, 2, 5, 2, 6, 3, 2, 3, 3, 3, 4, 3, 5, 3, 6, 4, 5, 4, 6, 5, 5, 5, 6, 6, 1, 6, 2, 6, 3, 6, 4, 6, 5],
                "4": [0, 3, 0, 4, 0, 5, 1, 2, 1, 3, 1, 4, 1, 5, 2, 1, 2, 2, 2, 4, 2, 5, 3, 0, 3, 1, 3, 4, 3, 5, 4, 0, 4, 1, 4, 2, 4, 3, 4, 4, 4, 5, 4, 6, 5, 4, 5, 5, 6, 4, 6, 5],
                "5": [0, 1, 0, 2, 0, 3, 0, 4, 0, 5, 0, 6, 1, 1, 1, 2, 2, 1, 2, 2, 3, 1, 3, 2, 3, 3, 3, 4, 3, 5, 3, 6, 4, 5, 4, 6, 5, 5, 5, 6, 6, 1, 6, 2, 6, 3, 6, 4, 6, 5, 6, 6],
                "6": [0, 3, 0, 4, 0, 5, 0, 6, 1, 2, 1, 3, 2, 1, 2, 2, 3, 1, 3, 2, 3, 3, 3, 4, 3, 5, 3, 6, 4, 1, 4, 2, 4, 3, 4, 6, 4, 7, 5, 2, 5, 3, 5, 6, 5, 7, 6, 3, 6, 4, 6, 5, 6, 6],
                "7": [0, 1, 0, 2, 0, 3, 0, 4, 0, 5, 0, 6, 0, 7, 1, 6, 1, 7, 2, 5, 2, 6, 3, 4, 3, 5, 4, 3, 4, 4, 5, 2, 5, 3, 6, 1, 6, 2],
                "8": [0, 2, 0, 3, 0, 4, 0, 5, 0, 6, 1, 1, 1, 2, 1, 6, 1, 7, 2, 1, 2, 2, 2, 6, 2, 7, 3, 2, 3, 3, 3, 4, 3, 5, 3, 6, 4, 1, 4, 2, 4, 6, 4, 7, 5, 1, 5, 2, 5, 6, 5, 7, 6, 2, 6, 3, 6, 4, 6, 5, 6, 6],
                "9": [0, 2, 0, 3, 0, 4, 0, 5, 1, 1, 1, 2, 1, 5, 1, 6, 2, 1, 2, 2, 2, 4, 2, 5, 2, 6, 3, 2, 3, 3, 3, 4, 3, 5, 3, 6, 4, 5, 4, 6, 5, 5, 5, 6, 6, 2, 6, 3, 6, 4, 6, 4, 6, 5],
                "0": [0, 2, 0, 3, 0, 4, 0, 5, 0, 6, 1, 1, 1, 2, 1, 6, 1, 7, 2, 1, 2, 2, 2, 6, 2, 7, 3, 1, 3, 2, 3, 6, 3, 7, 4, 1, 4, 2, 4, 6, 4, 7, 5, 1, 5, 2, 5, 6, 5, 7, 6, 2, 6, 3, 6, 4, 6, 5, 6, 6],
                "a": [0, 2, 0, 3, 0, 4, 0, 5, 0, 6, 1, 6, 1, 7, 2, 6, 2, 7, 3, 2, 3, 3, 3, 4, 3, 5, 3, 6, 3, 7, 4, 1, 4, 2, 4, 6, 4, 7, 5, 1, 5, 2, 5, 6, 5, 7, 6, 2, 6, 3, 6, 4, 6, 5, 6, 6, 6, 7],
                "b": [0, 1, 0, 2, 1, 1, 1, 2, 2, 1, 2, 2, 3, 1, 3, 2, 3, 3, 3, 4, 3, 5, 3, 6, 4, 1, 4, 2, 4, 3, 4, 6, 4, 7, 5, 1, 5, 2, 5, 6, 5, 7, 6, 1, 6, 2, 6, 3, 6, 4, 6, 5, 6, 6],
                "c": [1, 3, 1, 4, 1, 5, 1, 6, 2, 2, 2, 3, 3, 1, 3, 2, 4, 1, 4, 2, 5, 2, 5, 3, 6, 3, 6, 4, 6, 5, 6, 6],
                "d": [0, 6, 0, 7, 1, 6, 1, 7, 2, 6, 2, 7, 3, 2, 3, 3, 3, 4, 3, 5, 3, 6, 3, 7, 4, 1, 4, 2, 4, 5, 4, 6, 4, 7, 5, 1, 5, 2, 5, 6, 5, 7, 6, 2, 6, 3, 6, 4, 6, 5, 6, 6, 6, 7],
                "e": [0, 2, 0, 3, 0, 4, 0, 5, 0, 6, 1, 1, 1, 2, 1, 6, 1, 7, 2, 1, 2, 2, 2, 6, 2, 7, 3, 1, 3, 2, 3, 3, 3, 4, 3, 5, 3, 6, 4, 1, 4, 2, 5, 1, 5, 2, 6, 2, 6, 3, 6, 4, 6, 5, 6, 6, 6, 7],
                "f": [0, 4, 0, 5, 0, 6, 0, 7, 1, 3, 1, 4, 2, 3, 2, 4, 3, 1, 3, 2, 3, 3, 3, 4, 3, 5, 3, 6, 3, 7, 4, 3, 4, 4, 5, 3, 5, 4, 6, 3, 6, 4],
                "g": [0, 2, 0, 3, 0, 4, 0, 5, 0, 6, 0, 7, 1, 1, 1, 2, 1, 6, 1, 7, 2, 1, 2, 2, 2, 6, 2, 7, 3, 2, 3, 3, 3, 4, 3, 5, 3, 6, 3, 7, 4, 6, 4, 7, 5, 6, 5, 7, 6, 2, 6, 3, 6, 4, 6, 5, 6, 6],
                "h": [0, 1, 0, 2, 1, 1, 1, 2, 2, 1, 2, 2, 3, 1, 3, 2, 3, 3, 3, 4, 3, 5, 3, 6, 4, 1, 4, 2, 4, 3, 4, 6, 4, 7, 5, 1, 5, 2, 5, 6, 5, 7, 6, 1, 6, 2, 6, 6, 6, 7],
                "i": [0, 3, 0, 4, 2, 1, 2, 2, 2, 3, 2, 4, 3, 3, 3, 4, 4, 3, 4, 4, 5, 3, 5, 4, 6, 3, 6, 4],
                "j": [1, 5, 1, 6, 2, 5, 2, 6, 3, 5, 3, 6, 4, 1, 4, 2, 4, 5, 4, 6, 5, 1, 5, 2, 5, 5, 5, 6, 6, 2, 6, 3, 6, 4, 6, 5],
                "k": [0, 1, 0, 2, 1, 1, 1, 2, 1, 4, 1, 5, 2, 1, 2, 2, 2, 3, 2, 4, 3, 1, 3, 2, 3, 3, 4, 1, 4, 2, 4, 3, 4, 4, 4, 5, 5, 1, 5, 2, 5, 5, 5, 6, 6, 1, 6, 2, 6, 6, 6, 7],
                "l": [0, 2, 0, 3, 0, 4, 0, 5, 0, 6, 1, 5, 1, 6, 2, 5, 2, 6, 3, 5, 3, 6, 4, 5, 4, 6, 5, 5, 5, 6, 6, 5, 6, 6],
                "m": [0, 1, 0, 2, 0, 3, 0, 4, 0, 5, 0, 6, 1, 0, 1, 1, 1, 2, 1, 3, 1, 4, 1, 5, 1, 6, 1, 7, 2, 0, 2, 1, 2, 3, 2, 4, 2, 6, 2, 7, 3, 0, 3, 1, 3, 3, 3, 4, 3, 6, 3, 7, 4, 0, 4, 1, 4, 3, 4, 4, 4, 6, 4, 7, 5, 0, 5, 1, 5, 3, 5, 4, 5, 6, 5, 7, 6, 0, 6, 1, 6, 3, 6, 4, 6, 6, 6, 7],
                "n": [1, 1, 1, 2, 1, 3, 1, 4, 1, 5, 1, 6, 2, 0, 2, 1, 2, 2, 2, 6, 2, 7, 3, 0, 3, 1, 3, 6, 3, 7, 4, 0, 4, 1, 4, 6, 4, 7, 5, 0, 5, 1, 5, 6, 5, 7, 6, 0, 6, 1, 6, 6, 6, 7],
                "o": [1, 2, 1, 3, 1, 4, 1, 5, 2, 1, 2, 2, 2, 5, 2, 6, 3, 1, 3, 2, 3, 5, 3, 6, 4, 1, 4, 2, 4, 5, 4, 6, 5, 1, 5, 2, 5, 5, 5, 6, 6, 2, 6, 3, 6, 4, 6, 5],
                "p": [1, 1, 1, 2, 1, 3, 1, 4, 1, 5, 1, 6, 2, 1, 2, 2, 2, 3, 2, 6, 2, 7, 3, 1, 3, 2, 3, 6, 3, 7, 4, 1, 4, 2, 4, 3, 4, 4, 4, 5, 4, 6, 5, 1, 5, 2, 6, 1, 6, 2],
                "q": [0, 2, 0, 3, 0, 4, 0, 5, 0, 6, 0, 7, 1, 1, 1, 2, 1, 5, 1, 6, 1, 7, 2, 1, 2, 2, 2, 6, 2, 7, 3, 1, 3, 2, 3, 5, 3, 6, 3, 7, 4, 2, 4, 3, 4, 4, 4, 5, 4, 6, 4, 7, 5, 6, 5, 7, 6, 6, 6, 7],
                "r": [0, 1, 0, 3, 0, 4, 0, 5, 1, 1, 1, 2, 1, 3, 1, 4, 1, 5, 1, 6, 2, 1, 2, 2, 2, 6, 3, 1, 3, 2, 4, 1, 4, 2, 5, 1, 5, 2, 6, 1, 6, 2],
                "s": [1, 2, 1, 3, 1, 4, 1, 5, 2, 1, 2, 2, 3, 1, 3, 2, 4, 3, 4, 4, 5, 4, 5, 5, 6, 1, 6, 2, 6, 3, 6, 4],
                "t": [0, 3, 0, 4, 1, 3, 1, 4, 2, 1, 2, 2, 2, 3, 2, 4, 2, 5, 2, 6, 3, 3, 3, 4, 4, 3, 4, 4, 5, 3, 5, 4, 6, 4, 6, 5, 6, 6, 6, 7],
                "u": [1, 1, 1, 2, 1, 6, 1, 7, 2, 1, 2, 2, 2, 6, 2, 7, 3, 1, 3, 2, 3, 6, 3, 7, 4, 1, 4, 2, 4, 6, 4, 7, 5, 1, 5, 2, 5, 5, 5, 6, 5, 7, 6, 2, 6, 3, 6, 4, 6, 5, 6, 6, 6, 7],
                "v": [1, 1, 1, 2, 1, 6, 1, 7, 2, 2, 2, 3, 2, 5, 2, 6, 3, 2, 3, 3, 3, 5, 3, 6, 4, 3, 4, 4, 4, 5, 5, 4, 5, 5, 6, 4],
                "w": [0, 0, 0, 1, 0, 6, 0, 7, 1, 0, 1, 1, 1, 6, 1, 7, 2, 0, 2, 1, 2, 3, 2, 4, 2, 6, 2, 7, 3, 0, 3, 1, 3, 3, 3, 4, 3, 6, 3, 7, 4, 0, 4, 1, 4, 3, 4, 4, 4, 6, 4, 7, 5, 0, 5, 1, 5, 2, 5, 3, 5, 4, 5, 5, 5, 6, 5, 7, 6, 1, 6, 2, 6, 3, 6, 4, 6, 5, 6, 6],
                "x": [1, 1, 1, 2, 1, 6, 1, 7, 2, 2, 2, 3, 2, 5, 2, 6, 3, 3, 3, 4, 3, 5, 4, 3, 4, 4, 4, 5, 5, 2, 5, 3, 5, 5, 5, 6, 6, 1, 6, 2, 6, 6, 6, 7],
                "y": [1, 1, 1, 2, 1, 5, 1, 6, 2, 1, 2, 2, 2, 5, 2, 6, 3, 2, 3, 3, 3, 4, 3, 5, 4, 3, 4, 4, 5, 2, 5, 3, 6, 1, 6, 2],
                "z": [1, 2, 1, 3, 1, 4, 1, 5, 1, 6, 1, 7, 2, 6, 2, 7, 3, 5, 3, 6, 4, 4, 4, 5, 5, 3, 5, 4, 6, 2, 6, 3, 6, 4, 6, 5, 6, 6, 6, 7],
                "A": [0, 3, 0, 4, 1, 2, 1, 3, 1, 4, 1, 5, 2, 2, 2, 3, 2, 4, 2, 5, 3, 1, 3, 2, 3, 5, 3, 6, 4, 1, 4, 2, 4, 3, 4, 4, 4, 5, 4, 6, 5, 1, 5, 2, 5, 5, 5, 6, 6, 0, 6, 1, 6, 6, 6, 7],
                "B": [0, 1, 0, 2, 0, 3, 0, 4, 0, 5, 0, 6, 1, 1, 1, 2, 1, 3, 1, 6, 1, 7, 2, 1, 2, 2, 2, 6, 2, 7, 3, 1, 3, 2, 3, 3, 3, 4, 3, 5, 3, 6, 4, 1, 4, 2, 4, 3, 4, 6, 4, 7, 5, 1, 5, 2, 5, 6, 5, 7, 6, 1, 6, 2, 6, 3, 6, 4, 6, 5, 6, 6],
                "C": [0, 2, 0, 3, 0, 4, 0, 5, 0, 6, 1, 1, 1, 2, 2, 0, 2, 1, 3, 0, 3, 1, 4, 0, 4, 1, 5, 1, 5, 2, 6, 2, 6, 3, 6, 4, 6, 5, 6, 6],
                "D": [0, 1, 0, 2, 0, 3, 0, 4, 0, 5, 0, 6, 1, 1, 1, 2, 1, 6, 1, 7, 2, 1, 2, 2, 2, 6, 2, 7, 3, 1, 3, 2, 3, 6, 3, 7, 4, 1, 4, 2, 4, 6, 4, 7, 5, 1, 5, 2, 5, 6, 5, 7, 6, 1, 6, 2, 6, 3, 6, 4, 6, 5, 6, 6],
                "E": [0, 1, 0, 2, 0, 3, 0, 4, 0, 5, 0, 6, 1, 1, 1, 2, 2, 1, 2, 2, 3, 1, 3, 2, 3, 3, 3, 4, 3, 5, 3, 6, 4, 1, 4, 2, 5, 1, 5, 2, 6, 1, 6, 2, 6, 3, 6, 4, 6, 5, 6, 6],
                "F": [0, 1, 0, 2, 0, 3, 0, 4, 0, 5, 0, 6, 1, 1, 1, 2, 2, 1, 2, 2, 3, 1, 3, 2, 3, 3, 3, 4, 3, 5, 3, 6, 4, 1, 4, 2, 5, 1, 5, 2, 6, 1, 6, 2],
                "G": [0, 2, 0, 3, 0, 4, 0, 5, 0, 6, 1, 1, 1, 2, 2, 0, 2, 1, 3, 0, 3, 1, 3, 4, 3, 5, 3, 6, 4, 0, 4, 1, 4, 6, 5, 1, 5, 2, 5, 6, 6, 2, 6, 3, 6, 4, 6, 5, 6, 6],
                "H": [0, 1, 0, 2, 0, 6, 0, 7, 1, 1, 1, 2, 1, 6, 1, 7, 2, 1, 2, 2, 2, 6, 2, 7, 3, 1, 3, 2, 3, 3, 3, 4, 3, 5, 3, 6, 3, 7, 4, 1, 4, 2, 4, 6, 4, 7, 5, 1, 5, 2, 5, 6, 5, 7, 6, 1, 6, 2, 6, 6, 6, 7],
                "I": [0, 2, 0, 3, 0, 4, 0, 5, 0, 6, 1, 4, 2, 4, 3, 4, 4, 4, 5, 4, 6, 1, 6, 2, 6, 3, 6, 4, 6, 5, 6, 6, 6, 7],
                "J": [0, 2, 0, 3, 0, 4, 0, 5, 0, 6, 1, 5, 1, 6, 2, 5, 2, 6, 3, 5, 3, 6, 4, 1, 4, 2, 4, 5, 4, 6, 5, 1, 5, 2, 5, 5, 5, 6, 6, 2, 6, 3, 6, 4, 6, 5],
                "K": [0, 1, 0, 2, 0, 5, 0, 6, 1, 1, 1, 2, 1, 4, 1, 5, 2, 1, 2, 2, 2, 3, 2, 4, 3, 1, 3, 2, 3, 3, 4, 1, 4, 2, 4, 3, 4, 4, 5, 1, 5, 2, 5, 4, 5, 5, 6, 1, 6, 2, 6, 5, 6, 6],
                "L": [0, 1, 0, 2, 1, 1, 1, 2, 2, 1, 2, 2, 3, 1, 3, 2, 4, 1, 4, 2, 5, 1, 5, 2, 6, 1, 6, 2, 6, 3, 6, 4, 6, 5, 6, 6],
                "M": [0, 1, 0, 2, 0, 6, 0, 7, 1, 1, 1, 2, 1, 3, 1, 5, 1, 6, 1, 7, 2, 1, 2, 2, 2, 4, 2, 6, 2, 7, 3, 1, 3, 2, 3, 6, 3, 7, 4, 1, 4, 2, 4, 6, 4, 7, 5, 1, 5, 2, 5, 6, 5, 7, 6, 1, 6, 2, 6, 6, 6, 7],
                "N": [0, 1, 0, 2, 0, 6, 0, 7, 1, 1, 1, 2, 1, 3, 1, 6, 1, 7, 2, 1, 2, 2, 2, 4, 2, 6, 2, 7, 3, 1, 3, 2, 3, 5, 3, 6, 3, 7, 4, 1, 4, 2, 4, 6, 4, 7, 5, 1, 5, 2, 5, 6, 5, 7, 6, 1, 6, 2, 6, 6, 6, 7],
                "O": [0, 2, 0, 3, 0, 4, 0, 5, 1, 1, 1, 2, 1, 5, 1, 6, 2, 0, 2, 1, 2, 6, 2, 7, 3, 0, 3, 1, 3, 6, 3, 7, 4, 0, 4, 1, 4, 6, 4, 7, 5, 1, 5, 2, 5, 5, 5, 6, 6, 2, 6, 3, 6, 4, 6, 5],
                "P": [0, 1, 0, 2, 0, 3, 0, 4, 0, 5, 0, 6, 1, 1, 1, 2, 1, 3, 1, 6, 1, 7, 2, 1, 2, 2, 2, 6, 2, 7, 3, 1, 3, 2, 3, 3, 3, 4, 3, 5, 3, 6, 4, 1, 4, 2, 5, 1, 5, 2, 6, 1, 6, 2],
                "Q": [0, 2, 0, 3, 0, 4, 0, 5, 1, 1, 1, 2, 1, 5, 1, 6, 2, 0, 2, 1, 2, 6, 2, 7, 3, 0, 3, 1, 3, 6, 3, 7, 4, 0, 4, 1, 4, 4, 4, 6, 4, 7, 5, 1, 5, 2, 5, 5, 5, 6, 6, 2, 6, 3, 6, 4, 6, 5, 6, 6, 6, 7],
                "R": [0, 1, 0, 2, 0, 3, 0, 4, 0, 5, 0, 6, 1, 1, 1, 2, 1, 3, 1, 6, 1, 7, 2, 1, 2, 2, 2, 6, 2, 7, 3, 1, 3, 2, 3, 3, 3, 4, 3, 5, 3, 6, 4, 1, 4, 2, 4, 5, 5, 1, 5, 5, 6, 2, 6, 1, 6, 2, 6, 6],
                "S": [0, 2, 0, 3, 0, 4, 0, 5, 0, 6, 1, 1, 1, 2, 2, 1, 2, 2, 3, 2, 3, 3, 3, 4, 3, 5, 4, 5, 4, 6, 5, 5, 5, 6, 6, 1, 6, 2, 6, 3, 6, 4, 6, 5, 6, 6],
                "T": [0, 1, 0, 2, 0, 3, 0, 4, 0, 5, 0, 6, 0, 7, 1, 4, 2, 4, 3, 4, 4, 4, 5, 4],
                "U": [0, 1, 0, 2, 0, 6, 0, 7, 1, 1, 1, 2, 1, 6, 1, 7, 2, 1, 2, 2, 2, 6, 2, 7, 3, 1, 3, 2, 3, 6, 3, 7, 4, 1, 4, 2, 4, 6, 4, 7, 5, 1, 5, 2, 5, 6, 5, 7, 6, 2, 6, 3, 6, 4, 6, 5, 6, 6],
                "V": [0, 0, 0, 1, 0, 6, 0, 7, 1, 1, 1, 2, 1, 6, 1, 7, 2, 1, 2, 2, 2, 6, 2, 7, 3, 2, 3, 3, 3, 5, 3, 6, 4, 3, 4, 4, 4, 5, 4, 6, 5, 4, 5, 5, 6, 4],
                "W": [0, 1, 0, 2, 0, 6, 0, 7, 1, 1, 1, 2, 1, 6, 1, 7, 2, 1, 2, 2, 2, 6, 2, 7, 3, 1, 3, 2, 3, 6, 3, 7, 4, 1, 4, 2, 4, 4, 4, 6, 4, 7, 5, 1, 5, 2, 5, 3, 5, 5, 5, 6, 5, 7, 6, 1, 6, 2, 6, 6, 6, 7],
                "X": [0, 0, 0, 1, 0, 6, 0, 7, 1, 1, 1, 2, 1, 5, 1, 6, 2, 1, 2, 2, 2, 5, 2, 6, 3, 2, 3, 3, 3, 4, 3, 5, 4, 2, 4, 3, 4, 4, 4, 5, 5, 1, 5, 2, 5, 5, 5, 6, 6, 0, 6, 1, 6, 6, 6, 7],
                "Y": [0, 0, 0, 1, 0, 6, 0, 7, 1, 0, 1, 1, 1, 6, 1, 7, 2, 1, 2, 2, 2, 5, 2, 6, 3, 2, 3, 3, 3, 4, 3, 5, 4, 3, 4, 4, 5, 3, 5, 4, 6, 3, 6, 4],
                "Z": [0, 1, 0, 2, 0, 3, 0, 4, 0, 5, 0, 6, 0, 7, 1, 6, 1, 7, 2, 5, 2, 6, 3, 4, 3, 5, 4, 2, 4, 3, 5, 1, 5, 2, 6, 1, 6, 2, 6, 3, 6, 4, 6, 5, 6, 6, 6, 7],
                ",": [5, 3, 5, 4, 5, 5, 6, 4, 6, 5, 7, 3, 7, 4],
                ":": [1, 3, 1, 4, 1, 5, 2, 3, 2, 4, 2, 5, 4, 3, 4, 4, 4, 5, 5, 3, 5, 4, 5, 5],
                "%": [0, 6, 0, 7, 1, 1, 1, 2, 1, 5, 1, 6, 2, 1, 2, 2, 2, 4, 2, 5, 3, 3, 3, 4, 4, 2, 4, 3, 5, 1, 5, 2, 5, 4, 5, 5, 6, 0, 6, 1, 6, 4, 6, 5],
                "!": [0, 3, 0, 4, 0, 5, 1, 3, 1, 4, 1, 5, 2, 3, 2, 4, 2, 5, 3, 3, 3, 4, 3, 5, 4, 3, 4, 4, 4, 5, 5, 3, 5, 4, 5, 5, 7, 4],
                "(": [0, 2, 0, 3, 1, 1, 1, 2, 2, 1, 2, 2, 3, 1, 3, 2, 4, 1, 4, 2, 5, 1, 5, 2, 6, 1, 6, 2, 7, 2, 7, 3],
                ")": [0, 5, 0, 6, 1, 6, 1, 7, 2, 6, 2, 7, 3, 6, 3, 7, 4, 6, 4, 7, 5, 6, 5, 7, 6, 6, 6, 7, 7, 5, 7, 6],
                ".": [5, 3, 5, 4, 5, 5, 6, 3, 6, 4, 6, 5, 7, 3, 7, 4, 7, 5]
            },
            /**		
            * Specifies the frame of the Digital gauge.
            * @default
            * @alias ejDigitalGauge#frame
            * @type {object}
            * @example  
            * &lt;div id="DigitalCore"&gt;&lt;/div&gt; <br> 
            * &lt;script&gt;
            * $("#DigitalCore").ejDigitalGauge({ frame:{backgroundImageUrl: null, innerWidth:6, outerWidth:10}});
            * &lt;/script&gt;
            * @memberof ejDigitalGauge
			* @instance
            */
            frame: {
                /**		
                * Specifies the url of an image to be displayed as background of the Digital gauge.
                * @default null
                * @alias ejDigitalGauge#frame->backgroundImageUrl
                * @type {string}
                * @example  
                * &lt;div id="DigitalCore"&gt;&lt;/div&gt; <br> 
                * &lt;script&gt;
                * $("#DigitalCore").ejDigitalGauge({frame:{ backgroundImageUrl: "styles\images\Car.png" }});
                * &lt;/script&gt;
                * @memberof ejDigitalGauge
			    * @instance
                */
                backgroundImageUrl: null,
                /**		
                * Specifies the inner width for the frame, when the background image has been set for the Digital gauge..
                * @default 6
                * @alias ejDigitalGauge#frame->innerWidth
                * @type {number}
                * @example 
                * &lt;div id="DigitalCore"&gt;&lt;/div&gt; <br> 
                * &lt;script&gt;
                * $("#DigitalCore").ejDigitalGauge({frame:{ innerWidth: 30 }});
                * &lt;/script&gt;
                * @memberof ejDigitalGauge
			    * @instance
                */
                innerWidth: 6,
                /**		
                * Specifies the outer width of the frame, when the background image has been set for the Digital gauge.
                * @default 10
                * @alias ejDigitalGauge#frame->outerWidth
                * @type {number}
                * @example 
                * &lt;div id="DigitalCore"&gt;&lt;/div&gt; <br> 
                * &lt;script&gt;
                * $("#DigitalCore").ejDigitalGauge({frame: { outerWidth: 30 } });
                * &lt;/script&gt;
                * @memberof ejDigitalGauge
			    * @instance
                */
                outerWidth: 10,
            },
            /**		
            * Specifies the height of the DigitalGauge.
            * @default 150
            * @alias ejDigitalGauge#height
            * @type {number}
            * @example 
            * &lt;div id="DigitalCore"&gt;&lt;/div&gt; <br> 
            * &lt;script&gt;
            * $("#DigitalCore").ejDigitalGauge({ height: 60 }); 
            * &lt;/script&gt;
            * @memberof ejDigitalGauge
			* @instance
            */
            height: 150,
            /**		
            * Specifies the width for the Digital gauge.
            * @default 400
            * @alias ejDigitalGauge#width
            * @type {number}
            * @example 
            * &lt;div id="DigitalCore"&gt;&lt;/div&gt; <br> 
            * &lt;script&gt;
            * $("#DigitalCore").ejDigitalGauge({ width: 300 }); 
            * &lt;/script&gt;
            * @memberof ejDigitalGauge
			* @instance
            */
            width: 400,
            /**		
            * Specifies the resize option of the DigitalGauge.
            * @default false
            * @alias ejDigitalGauge#enableResize
            * @type {boolean}
            * @example 
            * &lt;div id="DigitalCore"&gt;&lt;/div&gt; <br> 
            * &lt;script&gt;
            * $("#DigitalCore").ejDigitalGauge({ enableResize: true }); 
            * &lt;/script&gt;
            * @memberof ejDigitalGauge
            * @instance
            */
            enableResize: false,
            /**		
            * Specifies the themes for the Digital gauge. See {@link Themes} 
            * @default flatlight
            * @alias ejDigitalGauge#themes
            * @type {string}
            * @example 
            * &lt;div id="DigitalCore"&gt;&lt;/div&gt; <br> 
            * &lt;script&gt;
            * $("#DigitalCore").ejDigitalGauge({ themes: "flatlight" });
            * &lt;/script&gt;
            * @memberof ejDigitalGauge
            * @instance
            */
            themes: "flatlight",
            /**		
            * Specifies the items for the DigitalGauge.
            * @default null
            * @alias ejDigitalGauge#items
            * @type {object}
            * @example 
            * &lt;div id="DigitalCore"&gt;&lt;/div&gt; <br> 
            * &lt;script&gt;
            * $("#DigitalCore").ejDigitalGauge({width: 500});
            * &lt;/script&gt;
            * @memberof ejDigitalGauge
            * @instance
            */
            items: null,
            /**     
            * Triggers when the gauge is initialized.
            * @event
            * @name ejDigitalGauge#init 	
            * @param {Object} args.object  returns the object of the gauge.
            * @param {boolean} args.cancel  returns the cancel option value
            * @param {Object} args.items   returns the all the options of the items.
            * @param {Object} args.context returns the context element
            * @param {Object} args.model   returns the gauge model
            * @param {String} args.type    returns the name of the event
            * @param {}
            * @example 
            * &lt;div id="DigitalCore"&gt;&lt;/div&gt; <br> 
            * &lt;script&gt;
            * $("#DigitalCore").ejDigitalGauge({
            *    init: function (args) {}
            * });      
            * &lt;/script&gt;
            * @memberof ejDigitalGauge
            * @instance
            */
            init: null,
            /**     
            * Triggers when the gauge is start to load.
            * @event
            * @name ejDigitalGauge#load 	
            * @param {Object} args.object  returns the object of the gauge.
            * @param {boolean} args.cancel  returns the cancel option value
            * @param {Object} args.items   returns the all the options of the items.
            * @param {Object} args.context returns the context element
            * @param {Object} args.model   returns the gauge model
            * @param {String} args.type    returns the name of the event
            * @example 
            * &lt;div id="DigitalCore"&gt;&lt;/div&gt; <br> 
            * &lt;script&gt;
            * $("#DigitalCore").ejDigitalGauge({
            *    load: function (args) {}
            * });      
            * &lt;/script&gt;
            * @memberof ejDigitalGauge
            * @instance
            */
            load: null,
            /**     
            * Triggers when the gauge render is completed.
            * @event
            * @name ejDigitalGauge#renderComplete 	
            * @param {Object} args.object  returns the object of the gauge.
            * @param {boolean} args.cancel  returns the cancel option value
            * @param {Object} args.items   returns the all the options of the items.
            * @param {Object} args.context returns the context element
            * @param {Object} args.model   returns the gauge model
            * @param {String} args.type    returns the name of the event
            * @example 
            * &lt;div id="DigitalCore"&gt;&lt;/div&gt; <br> 
            * &lt;script&gt;
            * $("#DigitalCore").ejDigitalGauge({
            *    renderComplete: function (args) {}
            * });       
            * &lt;/script&gt;
            * @memberof ejDigitalGauge
            * @instance
            */
            renderComplete: null,
            /**     
            * Triggers when the gauge item rendering.
            * @event
            * @name ejDigitalGauge#itemRendering 	
            * @param {Object} args.object  returns the object of the gauge.
            * @param {boolean} args.cancel  returns the cancel option value
            * @param {Object} args.items   returns the all the options of the items.
            * @param {Object} args.context returns the context element
            * @param {Object} args.model   returns the gauge model
            * @param {String} args.type    returns the name of the event
            * @example 
            * &lt;div id="DigitalCore"&gt;&lt;/div&gt; <br> 
            * &lt;script&gt;
            * $("#DigitalCore").ejDigitalGauge({
            *    itemRendering: function (args) {}
            * });      
            * &lt;/script&gt;
            * @memberof ejDigitalGauge
            * @instance
            */
            itemRendering: null,
            /**		
            * Specifies the value to the DigitalGauge.
            * @default text      
            * @alias ejDigitalGauge#value
            * @type {string}
            * @example 
            * &lt;div id="DigitalCore"&gt;&lt;/div&gt; <br> 
            * &lt;script&gt;
            * $("#DigitalCore").ejDigitalGauge({items: [{ value: "Welcome" }] });
            * &lt;/script&gt;
            * @memberof ejDigitalGauge
            * @instance
            */
            value: "text"
        },

        /**
        * Specify the data types for default properties 
        * @private
        */
        dataTypes: {
            segmentData: "data",
            matrixSegmentData: "data",
            items: "array"
        },

        /**
        * To configure set values of the Digital Gauge		
        * @private
        */
        _setValues: function () {
            this.gaugeEl = this.element;
            this.segmentCount = null;
            this.contextEl = null;
            this.style = null;
            this._value = null;
            this.location = null;
            this.canvasEl = null;
            this.segement16X = null;
            this.segment16Y = null;
            this.segmentHeight = null;
            this.segmentAngle = null;
            this.startX = 5;
            this.startY = 5;
            this.gradient = null;
            this.itemIndex = null;
            this.characterSpace = null;
            this.outerImage = null;
            this.radius = null;
            this.frameOuterLocation = null;
            this.frameInnerLocation = null;
            this.glassFrameLocation = null;
            this.glassFrameStyle = null;
            this.frameOuterStyle = null;
            this.character = null;
            this.frameInnerStyle = null;
            this._itemInitialize();
        },
        observables: ["value"],
        value: ej.util.valueFunction("value"),


        /**
        * To destroy the digital gauge 
        * @alias destroy
        * @return jQuery
        * @example 
        * &lt;div id="DigitalGauge1"&gt;Digital Gauge&lt;/div&gt; <br> 
        * &lt;script&gt;
        * $("#DigitalGauge1").ejDigitalGauge();
        * var gphObj = $("#DigitalGauge1").data("ejDigitalGauge");
        * gphObj.destroy();
        * &lt;/script&gt;
        *@memberof ejDigitalGauge
        * @instance
        */
        _destroy: function () {
            this.element.empty().removeClass("e-widget");
        },
        /**
        * To configure the properties at runtime using SetModel		
        * @private
        */
        _setModel: function (options) {
            var option;
            for (option in options) {
                switch (option) {
                    case "height":
                        this.model.height = options[option];
                        break;
                    case "width":
                        this.model.width = options[option];
                        break;
                    case "items":
                        this.model.items = options[option];
                        this._itemInitialize();
                        break;
                    case "frame":
                        $.extend(this.model.frame, options[prop]);
                    case "themes": this.model.themes = options[option]; break;
                    case "value":
                        for (var i = 0; this.model.items[i] != null; i++)
                            this.model.items[i].value = this.value();
                }
            }
            this.refresh();
        },
        /**
        * To initialize the Digital Gauge		
        * @private
        */
        _init: function () {
            this._setValues();
            this._trigger("load");
            this._setTheme();
            this._initialize();
            this._onWindowResize();
        },
        /**
        * To window resize for the Digital Gauge		
        * @private
        */
        _onWindowResize: function () {
            if (this.model.enableResize) {
                proxy = this;
                if (!this.isDevice())
                    this._on($(window), "resize", proxy.resizeCanvas);
                else
                    this._on($(window), "orientationchange", proxy.resizeCanvas);
            }
        },
        /**
        * To check the Device of the Digital Gauge rendered	
        * @private
        */
        isDevice: function () {
            return (/mobile|tablet|android|kindle/i.test(navigator.userAgent.toLowerCase()));
        },
        /**
        * To set theme for the Digital Gauge		
        * @private
        */
        _setTheme: function () {
            selectedThem = ej.datavisualization.DigitalGauge.Themes[this.model.themes];
            this._setThemeColors(selectedThem);
        },
        /**
        * To apply theme colors to the selected theme for the Digital Gauge		
        * @private
        */
        _setThemeColors: function (selectedThem) {
            var result = [], jsonObj = ej.datavisualization.DigitalGauge.Themes;
            for (var name in jsonObj) {
                result.push(name);
            }
            for (var th = 0; th < result.length; th++) {
                for (var i = 0; i < this.model.items.length; i++) {
                    this.model.items[i].segmentSettings.color = (ej.isNullOrUndefined(this.model.items[i].segmentSettings.color) || this.model.items[i].segmentSettings.color == jsonObj[result[th]].items.segmentSettings.color) ? selectedThem.items.segmentSettings.color : this.model.items[i].segmentSettings.color;
                    this.model.items[i].shadowColor = (!this.model.items[i].shadowColor || this.model.items[i].shadowColor == jsonObj[result[th]].items.shadowColor) ? selectedThem.items.shadowColor : this.model.items[i].shadowColor;
                    this.model.items[i].textColor = (!this.model.items[i].textColor || this.model.items[i].textColor == jsonObj[result[th]].items.textColor) ? selectedThem.items.textColor : this.model.items[i].textColor;
                }
            }
        },
        /**
        * To initialize the Digital Gauge		
        * @private
        */
        _initialize: function () {
            if (this.model.init)
                this._clientSideOnInit();
            this._initObject(this);
            if (this.model.load)
                this._clientSideOnLoad();
            if (this.model.frame.backgroundImageUrl != null)
                this._drawCustomImage(this, this.model.frame.backgroundImageUrl);
            else
                this._renderItems();
            if (this.model.renderComplete)
                this._clientSideOnRenderComplete();
        },
        /**
        * To initialize the items of Digital Gauge		
        * @private
        */
        _itemInitialize: function () {
            var proxy = this;
            if (this.model.items != null) {
                $.each(this.model.items, function (index, element) {
                    var obj = proxy._sendDefaultItem();
                    $.extend(true, obj, element);
                    $.extend(true, element, obj);
                });
            }
            else {
                this.model.items = [this._sendDefaultItem()];
            }
        },
        /**
        * To set the default items values for the Digital Gauge		
        * @private
        */
        _sendDefaultItem: function () {
            var defaultItems;
            return defaultItems = {
                /**		
                * Specifies the CharacterCount value for the DigitalGauge.
                * @name ejDigitalGauge#items->characterSettings
                * @default Object                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                     
                * @type {Object}
                * @example 
                * &lt;div id="DigitalCore"&gt;&lt;/div&gt; <br> 
                * &lt;script&gt;
                * $("#DigitalCore").ejDigitalGauge({items: [{ characterSettings: {count: 4} }] });
                * &lt;/script&gt;
                * @memberof ejDigitalGauge
                * @instance
                */
                characterSettings: {
                    /**		
                    * Specifies the CharacterCount value for the DigitalGauge.
                    * @name ejDigitalGauge#items->characterSettings->count
                    * @default 4                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                     
                    * @type {number}
                    * @example 
                    * &lt;div id="DigitalCore"&gt;&lt;/div&gt; <br> 
                    * &lt;script&gt;
                    * $("#DigitalCore").ejDigitalGauge({items: [{ characterSettings: {count: 4} }] });
                    * &lt;/script&gt;
                    * @memberof ejDigitalGauge
                    * @instance
                    */
                    count: 4,
                     /**		
                    * Specifies the CharacterCount value for the DigitalGauge.
                    * @name ejDigitalGauge#items->characterSettings->opacity
                    * @default 1                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                       
                    * @type {number}
                    * @example 
                    * &lt;div id="DigitalCore"&gt;&lt;/div&gt; <br> 
                    * &lt;script&gt;
                    * $("#DigitalCore").ejDigitalGauge({items: [{ characterSettings: {opacity: 1} }] });
                    * &lt;/script&gt;
                    * @memberof ejDigitalGauge
                    * @instance
                    */
                    opacity: 1,
                    /**		
                   * Specifies the value for spacing between the characters
                   * @name ejDigitalGauge#items->characterSettings->spacing
                   * @default 2                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                       
                   * @type {number}
                   * @example 
                   * &lt;div id="DigitalCore"&gt;&lt;/div&gt; <br> 
                   * &lt;script&gt;
                   * $("#DigitalCore").ejDigitalGauge({items: [{ characterSettings: {spacing: 3} }] });
                   * &lt;/script&gt; 
                   * @memberof ejDigitalGauge
                   * @instance
                   */
                    spacing: 2,
                    /**		
                    * Specifies the character type for the text to be displayed.  See {@link CharacterType}
                    * @name ejDigitalGauge#items->characterSettings->type
                    * @default ej.datavisualization.DigitalGauge.CharacterType.EightCrossEightDotMatrix                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                       
                    * @type {enum}
                    * @example 
                    * &lt;div id="DigitalCore"&gt;&lt;/div&gt; <br> 
                    * &lt;script&gt;
                    * $("#DigitalCore").ejDigitalGauge({items: [{characterSettings:{ type: "eightcrosseightdotmatrix" }}] });
                    * &lt;/script&gt; 
                    * @memberof ejDigitalGauge
                    * @instance
                    */
                    type:ej.datavisualization.DigitalGauge.CharacterType.EightCrossEightDotMatrix,
                },
                /**		
                * Enable/Disable the custom font to be applied to the text in the gauge. <br>
                * @name ejDigitalGauge#items->enableCustomFont
                * @default false                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                       
                * @type {boolean}
                * @example 
                * &lt;div id="DigitalCore"&gt;&lt;/div&gt; <br> 
                * &lt;script&gt;
                * $("#DigitalCore").ejDigitalGauge({items: [{ enableCustomFont: true }] });
                * &lt;/script&gt;
                * @memberof ejDigitalGauge
                * @instance
                */
                enableCustomFont: false,
                /**		
                * Set the length for the text segments. <br>
                * @name ejDigitalGauge#items->segmentSettings
                * @default Object                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                       
                * @type {Object}
                * @example 
                * &lt;div id="DigitalCore"&gt;&lt;/div&gt; <br> 
                * &lt;script&gt;
                * $("#DigitalCore").ejDigitalGauge({items: [{ segmentSettings: {length: 2} }] });
                * &lt;/script&gt;
                * @memberof ejDigitalGauge
                * @instance
                */
                segmentSettings: {
                    /**		
                    * Set the length for the text segments. <br>
                    * @name ejDigitalGauge#items->segmentSettings->color
                    * @default null                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                       
                    * @type {string}
                    * @example 
                    * &lt;div id="DigitalCore"&gt;&lt;/div&gt; <br> 
                    * &lt;script&gt;
                    * $("#DigitalCore").ejDigitalGauge({items: [{ segmentSettings: {color: "#FF1F2F"} }] });
                    * &lt;/script&gt;
                    * @memberof ejDigitalGauge
                    * @instance
                    */
                    color: null,
                    /**		
                    * Set the length for the text segments. <br>
                    * @name ejDigitalGauge#items->segmentSettings->gradient
                    * @default null                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                       
                    * @type {Object}
                    * @example 
                    * &lt;div id="DigitalCore"&gt;&lt;/div&gt; <br> 
                    * &lt;script&gt;
                    * $("#DigitalCore").ejDigitalGauge({items: [{ segmentSettings: {gradient: { colorInfo: [{ colorStop: 0, color: "Green" }, { colorStop: 1, color: "Yellow" }] } } }] });
                    * &lt;/script&gt;
                    * @memberof ejDigitalGauge
                    * @instance
                    */
                    gradient:null,
                     /**		
                    * Set the length for the text segments. <br>
                    * @name ejDigitalGauge#items->segmentSettings->length
                    * @default 2                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                       
                    * @type {number}
                    * @example 
                    * &lt;div id="DigitalCore"&gt;&lt;/div&gt; <br> 
                    * &lt;script&gt;
                    * $("#DigitalCore").ejDigitalGauge({items: [{ segmentSettings: {length: 2} }] });
                    * &lt;/script&gt;
                    * @memberof ejDigitalGauge
                    * @instance
                    */
                    length: 2,
                    /**		
                    * Set the length for the text segments. <br>
                    * @name ejDigitalGauge#items->segmentSettings->opacity
                    * @default 0                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                      
                    * @type {number}
                    * @example 
                    * &lt;div id="DigitalCore"&gt;&lt;/div&gt; <br> 
                    * &lt;script&gt;
                    * $("#DigitalCore").ejDigitalGauge({items: [{ segmentSettings: {opacity: 2} }] });
                    * &lt;/script&gt;
                    * @memberof ejDigitalGauge
                    * @instance
                    */
                    opacity: 0,
                    /**		
                    * Set the length for the text segments. <br>
                    * @name ejDigitalGauge#items->segmentSettings->spacing
                    * @default 1                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                      
                    * @type {number}
                    * @example 
                    * &lt;div id="DigitalCore"&gt;&lt;/div&gt; <br> 
                    * &lt;script&gt;
                    * $("#DigitalCore").ejDigitalGauge({items: [{ segmentSettings: {spacing: 1} }] });
                    * &lt;/script&gt;
                    * @memberof ejDigitalGauge
                    * @instance
                    */
                    spacing: 1,
                    /**		
                    * Set the length for the text segments. <br>
                    * @name ejDigitalGauge#items->segmentSettings->width
                    * @default 1                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                 
                    * @type {number}
                    * @example 
                    * &lt;div id="DigitalCore"&gt;&lt;/div&gt; <br> 
                    * &lt;script&gt;
                    * $("#DigitalCore").ejDigitalGauge({items: [{ segmentSettings: {width: 1} }] });
                    * &lt;/script&gt;
                    * @memberof ejDigitalGauge
                    * @instance
                    */
                    width: 1,
                },
                /**		
                * Set the value for enabling/disabling the blurring effect for the shadows of the text <br>
                * @name ejDigitalGauge#items->shadowBlur
                * @default 0                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                       
                * @type (number)
                * @example 
                * &lt;div id="DigitalCore"&gt;&lt;/div&gt; <br> 
                * &lt;script&gt;
                * $("#DigitalCore").ejDigitalGauge({items: [{ shadowBlur:  1 }] });
                * &lt;/script&gt;
                * @memberof ejDigitalGauge
                * @instance
                */
                shadowBlur: 0,
                /**		
                * Set the x offset value for the shadow of the text, indicating the location where it needs to be displayed. <br>
                * @name ejDigitalGauge#items->shadowOffsetX
                * @default 1                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                       
                * @type (number)
                * @example 
                * &lt;div id="DigitalCore"&gt;&lt;/div&gt; <br> 
                * &lt;script&gt;
                * $("#DigitalCore").ejDigitalGauge({items: [{ shadowOffsetX:  2 }] });
                * &lt;/script&gt;
                * @memberof ejDigitalGauge
                * @instance
                */
                shadowOffsetX: 1,
                /**		
                * Set the y offset value for the shadow of the text, indicating the location where it needs to be displayed. <br>
                * @name ejDigitalGauge#items->shadowOffsetY
                * @default 1                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                       
                * @type (number)
                * @example 
                * &lt;div id="DigitalCore"&gt;&lt;/div&gt; <br> 
                * &lt;script&gt;
                * $("#DigitalCore").ejDigitalGauge({items: [{ shadowOffsetY:  2 }] });
                * &lt;/script&gt;
                * @memberof ejDigitalGauge
                * @instance
                */
                shadowOffsetY: 1,
                /**		
                * Set the alignment of the text that is displayed within the gauge.See {@link TextAlign}       
                * @name ejDigitalGauge#items->textAlign		
                * @default "left"
                * @type {string}
                * @example 
                * &lt;div id="DigitalCore"&gt;&lt;/div&gt; <br> 
                * &lt;script&gt;
                * $("#DigitalCore").ejDigitalGauge({items: [{ textAlign:  "right" }] });
                * &lt;/script&gt; 
                * @memberof ejDigitalGauge
                * @instance
                */
                textAlign: "left",
                /**		
                * Set the specific font for the text, when the enableCustomFont is set to true <br>
                * @name ejDigitalGauge#items->font
                * @default Object                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                       
                * @type {Object}
                * @example 
                * &lt;div id="DigitalCore"&gt;&lt;/div&gt; <br> 
                * &lt;script&gt;
                * $("#DigitalCore").ejDigitalGauge({items: [{enableCustomFont: true ,font: { size: "12px",fontFamily: "Segou",fontStyle: "bold"}}]});
                * &lt;/script&gt;
                * @memberof ejDigitalGauge
                * @instance
                */
                font: /** @lends ejDigitalGauge# */{
                    /**		
                    * Set the font size value <br>
                    * @name ejDigitalGauge#items->font->size
                    * @default 11px                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                       
                    * @type {string}
                    * @example 
                    * &lt;div id="DigitalCore"&gt;&lt;/div&gt; <br> 
                    * &lt;script&gt;
                    * $("#DigitalCore").ejDigitalGauge({items: [{enableCustomFont: true,font: { size: "18px"}}]});
                    * &lt;/script&gt;
                    * @memberof ejDigitalGauge
                    * @instance
                    */
                    size: "11px",
                    /**		
                    * Set the font family value <br>
                    * @name ejDigitalGauge#items->font->fontFamily
                    * @default Arial                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                       
                    * @type {string}
                    * @example 
                    * &lt;div id="DigitalCore"&gt;&lt;/div&gt; <br> 
                    * &lt;script&gt;
                    * $("#DigitalCore").ejDigitalGauge({items: [{enableCustomFont: true ,font: { fontFamily: "Segou"}}]});
                    * &lt;/script&gt;
                    * @memberof ejDigitalGauge
                    * @instance
                    */
                    fontFamily: "Arial",
                    /**		
                    * Set the font style for the font. See {@link FontStyle}<br>
                    * @name ejDigitalGauge#items->font->fontStyle
                    * @default italic                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                       
                    * @type {enum}
                    * @example 
                    * &lt;div id="DigitalCore"&gt;&lt;/div&gt; <br> 
                    * &lt;script&gt;
                    * $("#DigitalCore").ejDigitalGauge({items: [{enableCustomFont: true ,font: { fontStyle: "bold" }}]});
                    * &lt;/script&gt;
                    * @memberof ejDigitalGauge
                    * @instance
                    */
                    fontStyle: "italic"
                },
                /**		
                * Set the location for the text, where it needs to be placed within the gauge. <br>
                * @name ejDigitalGauge#items->position
                * @default Object                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                     
                * @type {object}
                * @example 
                * &lt;div id="DigitalCore"&gt;&lt;/div&gt; <br> 
                * &lt;script&gt;
                * $("#DigitalCore").ejDigitalGauge({items: [{position: { x: 10, y: 20 } }]});
                * &lt;/script&gt;
                * @memberof ejDigitalGauge
                * @instance
                */
                position: {
                    /**		
                    * Set the horizontal location for the text, where it needs to be placed within the gauge. <br>
                    * @name ejDigitalGauge#items->position->x
                    * @default 0                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                      
                    * @type {number}
                    * @example 
                    * &lt;div id="DigitalCore"&gt;&lt;/div&gt; <br> 
                    * &lt;script&gt;
                    * $("#DigitalCore").ejDigitalGauge({items: [{position : { x: 10,y:0} }]});
                    * &lt;/script&gt;
                    * @memberof ejDigitalGauge
                    * @instance
                    */
                    x: 0,
                    /**		
                    * Set the vertical location for the text, where it needs to be placed within the gauge. <br>
                    * @name ejDigitalGauge#items->position->y
                    * @default 0                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                     
                    * @type {number}
                    * @example 
                    * &lt;div id="DigitalCore"&gt;&lt;/div&gt; <br> 
                    * &lt;script&gt;
                    * $("#DigitalCore").ejDigitalGauge({items: [{position: { x:0,y: 20 } }]});
                    * &lt;/script&gt;
                    * @memberof ejDigitalGauge
                    * @instance
                    */
                    y: 0
                },
                /**		
                * Specifies the color of the text shadow.
                * @name ejDigitalGauge#items->shadowColor
                * @default null
                * @type {string}
                * @example 
                * &lt;div id="DigitalCore"&gt;&lt;/div&gt; <br> 
                * &lt;script&gt;
                * $("#DigitalCore").ejDigitalGauge({ items: [{shadowColor: "#FF1F2F" }]});
                * &lt;/script&gt; 
                * @memberof ejDigitalGauge
                * @instance
                */
                shadowColor: null,
                /**		
                * Specifies the color of the text.
                * @name ejDigitalGauge#items->textColor
                * @default null
                * @type {string}
                * @example  
                * &lt;div id="DigitalCore"&gt;&lt;/div&gt; <br> 
                * &lt;script&gt;
                * $("#DigitalCore").ejDigitalGauge({ items: [{textColor: "#FF1F2F" }]});
                * &lt;/script&gt;  
                * @memberof ejDigitalGauge
                * @instance
                */
                textColor: null,
                /**		
                * Specifies the text value.
                * @name ejDigitalGauge#items->value
                * @default null
                * @type {string}
                * @example  
                * &lt;div id="DigitalCore"&gt;&lt;/div&gt; <br> 
                * &lt;script&gt;
                * $("#DigitalCore").ejDigitalGauge({ items: [{value: "Welcome" }]});
                * &lt;/script&gt;  
                * @memberof ejDigitalGauge
                * @instance
                */
                value: null
            };
        },
        /**
        * To init the object for the Digital Gauge		
        * @private
        */
        _initObject: function (element) {
            this.element.addClass("e-widget");
            element.gaugeEl = this.element;
            for (var i = 0; this.model.items[i] != null; i++) {
                if (this.model.items[i].value == null)
                    this.model.items[i].value = this.value();
            }
            if (element.canvasEl)
                element.canvasEl.remove();
            else
                element.canvasEl = $("<canvas></canvas>");
            element.gaugeEl.append(element.canvasEl);
            element.canvasEl.attr("role", "presentation");
            initialDivWidth = element.gaugeEl.width();
            initialDivHeight = element.gaugeEl.height();
            element.canvasEl[0].setAttribute("width", element.model.width);
            element.canvasEl[0].setAttribute("height", element.model.height);
            element.centerX = element.canvasEl[0].width / 2;
            element.centerY = element.canvasEl[0].height / 2;
            var elem = element.canvasEl[0];
            if (typeof window.G_vmlCanvasManager != "undefined") {
                elem = window.G_vmlCanvasManager.initElement(elem);
            }
            if (!elem || !elem.getContext) {
                return;
            }
            element.contextEl = element.canvasEl[0].getContext("2d");
        },
        /**
        * To draw the custom image for the Digital Gauge		
        * @private
        */
        _drawCustomImage: function (element, imageUrl) {
            var image = new Image();
            var self = this;
            $(image).load(function () {
                element.contextEl.drawImage(this, 0, 0, element.model.width, element.model.height);
                if (element.model.Scales != null)
                    element._drawScales();
                if (element.model.items != null)
                    element._renderItems();
            }).attr('src', imageUrl);
        },
        /**
        * To set the SegmentCount value to the Digital Gauge		
        * @private
        */
        _setSegmentCount: function (element) {
            switch (element) {
                case "sevensegment": this._SegmentCount = 7; break;
                case "fourteensegment": this._SegmentCount = 14; break;
                case "sixteensegment": this._SegmentCount = 16; break;
                case "eightcrosseightdotmatrix": this._SegmentCount = [8, 8]; break;
                case "eightcrosseightsquarematrix": this._SegmentCount = [8, 8]; break;
                default:
                    this._SegmentCount = 7;
            }

        },
        /**
        * To set the InnerPosition of the Digital Gauge		
        * @private
        */
        _setInnerPosition: function () {
            this.contextEl.save();
            this.contextEl.translate(this.model.frame.outerWidth + this.model.frame.innerWidth, this.model.frame.outerWidth + this.model.frame.innerWidth);
            this.bounds = {
                height: this.canvasEl[0].height - 2 * (this.model.frame.outerWidth + this.model.frame.innerWidth),
                width: this.canvasEl[0].width - 2 * (this.model.frame.outerWidth + this.model.frame.innerWidth)
            };
        },
        /**
        * To set the width to the Digital Gauge		
        * @private
        */
        _setWidth: function () {
            var characterCount = [];
            if (this.model.items != null) {
                var self = this;
                $.each(this.model.items, function (index, element) {
                    characterCount.push(element.characterSettings.count);
                });
            }
        },

        /**
        * Render the items for Digital Gauge		
        * @private
        */
        _renderItems: function () {
            if (this.model.items != null) {
                this._setInnerPosition();
                var self = this;
                $.each(this.model.items, function (index, element) {
                    self._setSegmentCount(element.characterSettings.type);
                    self.itemIndex = index;
                    self.canvasEl.attr("aria-label", element.value);
                    self._setShadow(index, element);
                    if (element.enableCustomFont)
                        self._setCustomFont(index, element);
                    else if (element.characterSettings.type.indexOf("matrix") != -1)
                        self._drawMatrixSegments(index, element);
                    else
                        self._drawSegments(index, element);
                });
            }
        },

        /**
        * To set the GradientColor to Digital Gauge		
        * @private
        */
        _setGradientColor: function (element, gradient, options) {
            var self = element;
            if (options.Name || typeof (options) === "string") {
                gradient.addColorStop(0, this._getColor(element, options));
                gradient.addColorStop(1, this._getColor(element, options));
            }
            else
                $.each(options, function (index, colorElement) {
                    gradient.addColorStop(colorElement.colorStop != NaN ? colorElement.colorStop : 0, typeof (colorElement.color) === "string" ? colorElement.color : this._getColor(element, colorElement.color));
                });
        },

        /**
        * To get the color of the Digital Gauge		
        * @private
        */
        _getColor: function (element, option) {
            if (typeof (option) === "string") {
                return option;
            }
            else {
                return ("rgba(" + option.R + ", " + option.G + "," + option.B + ", " + option.A / 255 + ")");
            }
        },

        /**
        * To draw the matrix segments to the Digital Gauge		
        * @private
        */
        _drawMatrixSegments: function (index, element) {
            var segmentXCollection = [], segmentCollection = [];
            if (element.value) {
                this._value = element.value.toString().split('');
                if (element.characterSettings.count < this._value.length)
                    element.characterSettings.count = this._value.length;
            }
            else
                this._value = "";
            this.radius = (element.characterSettings.type.indexOf("dot") != -1) ? (element.segmentSettings.length + element.segmentSettings.width) / 2 : element.segmentSettings.width / 2;
            var controlStartX = this.startX = (this.bounds.width - element.characterSettings.count * (this._SegmentCount[0] * (2 * this.radius) + element.characterSettings.spacing + this._SegmentCount[0] * element.segmentSettings.spacing)) * (element.position.x / 100);
            var controlStartY = this.startY = (this.bounds.height - (this._SegmentCount[1] * ((element.characterSettings.type.indexOf("dot") != -1) ? 2 * this.radius : element.segmentSettings.length) + this._SegmentCount[1] * element.segmentSettings.spacing)) * (element.position.y / 100);
            for (var character = 0; character < element.characterSettings.count; character++) {
                if (this._value) {
                    this.character = element.textAlign == "right" ? this._value[this._value.length - element.characterSettings.count + character] : this._value[character];
                    segmentCollection = this.model.matrixSegmentData[this.character];
                }
                if (character != 0) {
                    controlStartX = this.startX = this.startX + element.characterSettings.spacing + element.segmentSettings.spacing + (2 * this.radius);
                    this.startY = controlStartY;
                }
                for (var dotY = 0; dotY < this._SegmentCount[1]; dotY++) {
                    if (dotY != 0) {
                        this.startY = ((element.characterSettings.type.indexOf("dot") != -1) ? (2 * this.radius) : element.segmentSettings.length) + this.startY + element.segmentSettings.spacing;
                        this.startX = controlStartX;
                    }
                    if (segmentCollection) {
                        $.each(segmentCollection, function (index) {
                            if (index % 2 == 0) {
                                if (segmentCollection[index] > dotY)
                                    return false;
                                if (segmentCollection[index] == dotY)
                                    segmentXCollection.push(parseInt(segmentCollection[index + 1]));
                            }
                        });
                    }
                    for (var dotX = 0; dotX < this._SegmentCount[0]; dotX++) {
                        if (dotX != 0)
                            this.startX = this.startX + 2 * this.radius + element.segmentSettings.spacing;
                        if (element.characterSettings.type.indexOf("dot") != -1)
                            this.gradient = this.contextEl.createRadialGradient(0, 0, 0, 0, 0, this.radius);
                        else
                            this.gradient = this.contextEl.createLinearGradient(0, 0, element.segmentSettings.width, 0);
                        if (element.segmentSettings.gradient)
                            this._setGradientColor(this, this.gradient, element.segmentSettings.gradient.colorInfo);
                        this.location = { "startX": this.startX, "startY": this.startY };
                        this.style = {
                            "opacity": (segmentXCollection && ($.inArray(dotX, segmentXCollection)) != -1) ? element.characterSettings.opacity : element.segmentSettings.opacity,
                            "height": element.segmentSettings.length,
                            "width": element.segmentSettings.width,
                            "fillStyle": ((element.segmentSettings.color == "transparent") ? "rgba(0,0,0,0)" : this._getColor(this, element.segmentSettings.color)),
                            "skewX": element.SkewAngleX,
                            "skewY": element.SkewAngleX
                        };
                        if (this.model.itemRendering)
                            this._clientSideOnItemRendering(true, dotX, dotY);
                        if (element.characterSettings.type.indexOf("dot") != -1)
                            this._drawDot(this.location, this.style);
                        else
                            this._drawSquare(this.location, this.style);
                    }
                    segmentXCollection = [];
                }
            }
        },

        /**
        * Draw the segments of Digital Gauge		
        * @private
        */
        _drawSegments: function (index, element) {
            var segmentCollection = [], characterSpace, self = this;
            if (element.value) {
                this._value = element.value.toUpperCase().toString().split('');
                if (element.characterSettings.count < this._value.length)
                    element.characterSettings.count = this._value.length;
            }
            this.characterSpace = element.characterSettings.type == "sevensegment" ? 2 * element.segmentSettings.width : 4 * element.segmentSettings.width;
            this._renderSegmentCalculation(element);
            this.gradient = this.contextEl.createLinearGradient(0, 0, 0, element.segmentSettings.width);
            if (element.segmentSettings.color)
                this._setGradientColor(this, this.gradient, element.segmentSettings.color);
            else if (element.segmentSettings.gradient)
                this._setGradientColor(this, this.gradient, element.segmentSettings.gradient.colorInfo);
            //else
            //    this._setGradientColor(this, this.gradient, this.model.segmentColor);
            for (var character = 0; character < element.characterSettings.count; character++) {
                if (element.value)
                    segmentCollection = this.model.segmentData[element.textAlign == "right" ? this._value[this._value.length - element.characterSettings.count + character] : this._value[character]];
                for (var segment = 0; segment < this._SegmentCount; segment++) {
                    if (character != 0)
                        this.segment16X[segment] = this.segment16X[segment] + element.segmentSettings.length + this.characterSpace + element.characterSettings.spacing;
                    if (this._value)
                        this.character = element.textAlign == "right" ? this._value[this._value.length - element.characterSettings.count + character] : this._value[character]
                    this.location = { "startX": this.segment16X[segment], "startY": this.segment16Y[segment] };
                    this.style = {
                        "angle": this.angle[segment],
                        "fillStyle": this.gradient,
                        "isStroke": false,
                        "isFill": true,
                        "characterHeight": element.characterSettings.type == "sevensegment" ? element.segmentSettings.length : this.segmentHeight[segment],
                        "segmentWidth": element.segmentSettings.width,
                        "opacity": (segmentCollection && ($.inArray(segment, segmentCollection) != -1)) ? element.characterSettings.opacity : element.segmentSettings.opacity
                    };
                    if (this.model.itemRendering)
                        this._clientSideOnItemRendering(false, segment);
                    this._drawSegmentLayers(this.location, this.style);
                }
                this.value(element.value);
                segmentCollection = [];
            }
        },

        /**
        * To set the CustomFont style to Digital Gauge		
        * @private
        */
        _setCustomFont: function (index, element) {
            this.startX = (this.bounds.width - this._measureText(element.value, 0, this._getFontString(this, element.font)).width) * (element.position.x / 100);
            this.startY = (this.bounds.height - this._measureText(element.value, 0, this._getFontString(this, element.font)).height) * (element.position.y / 100);
            this.location = { "startX": this.startX, "startY": this.startY };
            this.style = { "font": this._getFontString(this, element.font), "text": element.value, "textColor": element.textColor ? ((element.textColor == "transparent") ? "rgba(0,0,0,0)" : this._getColor(this, element.textColor)) : ((element.segmentSettings.color == "transparent") ? "rgba(0,0,0,0)" : this._getColor(this, element.segmentSettings.color)) };
            if (this.model.itemRendering)
                this._clientSideOnItemRendering(false, segment);
            this._drawText(this.location, this.style);
        },

        /**
        * To get the Font String value of Digital Gauge		
        * @private
        */
        _getFontString: function (element, font) {
            return ((font.size == null) ? "11px" : font.size) + " " + font.fontFamily + " " + (font.fontStyle ? font.fontStyle : "");
        },

        /**
        * Rendering the SegmentCalculation of Digital Gauge		
        * @private
        */
        _renderSegmentCalculation: function (element) {
            var length = element.segmentSettings.length, width = element.segmentSettings.width;
            this.startX = (this.bounds.width - element.characterSettings.count * (length + this.characterSpace + element.characterSettings.spacing)) * (element.position.x / 100);
            this.startY = (this.bounds.height - 2 * length - width) * (element.position.y / 100);
            var tempLength = element.characterSettings.type == "sevensegment" ? length : length / 2;
            this.segment16X = [
                  this.startX + width / 2,
                  this.startX + length + 4 * width,
                  this.startX + length + 4 * width,
                  this.startX + width / 2,
                  this.startX,
                  this.startX,
                  this.startX + width / 2,
                  this.startX + tempLength + 2 * width,
                  this.startX + 2.5 * width + tempLength,
                  this.startX + tempLength + 2 * width,
                  this.startX + length + 2.5 * width,
                  this.startX + tempLength + 2.5 * width,
                  this.startX + tempLength + 1.5 * width,
                  this.startX + 1.5 * width,
                  this.startX + 5 * width / 2 + tempLength,
                  this.startX + 2.5 * width + tempLength
            ];
            this.segment16Y = [
                  this.startY,
                  this.startY,
                  this.startY + length + width,
                  this.startY + 2 * length + 2 * width,
                  this.startY + length + width,
                  this.startY,
                  this.startY + length + width,
                  this.startY + length + width,
                  this.startY + length + width,
                  this.startY,
                  this.startY + width,
                  this.startY + length + width,
                  this.startY + length + width,
                  this.startY + width,
                  this.startY,
                  this.startY + 2 * length + 2 * width
            ];
            this.segmentHeight = [
                  length / 2,
                  length,
                  length,
                  length / 2,
                  length,
                  length,
                  length / 2,
                  length,
                  length / 2,
                  length,
                  length,
                  length,
                  length,
                  length,
                  length / 2,
                  length / 2
            ];
            this.angle = [-90, 0, 0, -90, 0, 0, -90, 0, -90, 0, 27, -27, 27, -27, -90, -90];
            if (element.characterSettings.type == "sevensegment")
                this.segment16X[2] = this.segment16X[1] = this.startX + length + 2 * width;
            if (element.characterSettings.type == "fourteensegment")
                this.segmentHeight[3] = this.segmentHeight[0] = length + 2 * width;
        },

        /**
        * Drawing the Segment Layers of the Digital Gauge		
        * @private
        */
        _drawSegmentLayers: function (location, style) {
            this._contextOpenPath(style, this);
            this.contextEl.translate(location.startX, location.startY);
            this.contextEl.rotate(Math.PI * (style.angle / 180));
            this.contextEl.moveTo(0, 0);
            this.contextEl.lineTo(-style.segmentWidth, style.segmentWidth);
            this.contextEl.lineTo(-style.segmentWidth, style.characterHeight);
            this.contextEl.lineTo(0, style.characterHeight + style.segmentWidth);
            this.contextEl.lineTo(style.segmentWidth, style.characterHeight);
            this.contextEl.lineTo(style.segmentWidth, style.segmentWidth);
            this._contextClosePath(style, this);
        },

        /**
        * Drawing the Dot of the Digital Gauge		
        * @private
        */
        _drawDot: function (location, style) {
            this.contextEl.beginPath();
            this.contextEl.save();
            this.contextEl.translate(location.startX, location.startY);
            this.contextEl.fillStyle = style.fillStyle;
            this.contextEl.globalAlpha = style.opacity;
            this.contextEl.arc(0, 0, this.radius, 0, 2 * Math.PI, true);
            this.contextEl.fill();
            this.contextEl.closePath();
            this.contextEl.restore();
        },

        /**
        * To set the Shadow of the Digital Gauge		
        * @private
        */
        _setShadow: function (index, element) {
            this.contextEl.save();
            this.contextEl.shadowColor = ((element.shadowColor == "transparent") ? "rgba(0,0,0,0)" : this._getColor(this, element.shadowColor));
            this.contextEl.shadowOffsetY = element.shadowOffsetY;
            this.contextEl.shadowOffsetX = element.shadowOffsetX;
            this.contextEl.shadowBlur = element.shadowBlur;
        },

        /**
        * Drawing the Square to Digital Gauge		
        * @private
        */
        _drawSquare: function (location, style) {
            this.contextEl.beginPath();
            this.contextEl.save();
            this.contextEl.translate(location.startX, location.startY);
            // this.contextEl.setTransform(1, style.skewY, style.skewX, 1, location.startX, location.startY);
            this.contextEl.fillStyle = style.fillStyle;
            this.contextEl.globalAlpha = style.opacity;
            this.contextEl.rect(0, 0, style.width, style.height);
            this.contextEl.fill();
            this.contextEl.closePath();
            this.contextEl.restore();
        },

        /**
        * Drawing the Text value of Digital Gauge		
        * @private
        */

        _drawText: function (location, style) {
            this.contextEl.beginPath();
            this.contextEl.save();
            this.contextEl.font = style.font;
            this.contextEl.textBaseline = "hanging";
            this.contextEl.fillStyle = ((style.textColor == "transparent") ? "rgba(0,0,0,0)" : style.textColor);
            this.contextEl.fillText(style.text, location.startX, location.startY);
            this.contextEl.closePath();
            this.contextEl.restore();
        },

        /**
        * ClientSideMethod SetValue
        * Sets the value of an item to be displayed in the gauge.
        * @alias setValue 
        * @param {int} itemIndex  Index value of the digital gauge item
        * @param {string} value  Text value to be displayed in the gaugeS
        * @example 
        * &lt;div id="DigitalGauge1"&gt;&lt;/div&gt; <br> 
        * &lt;script&gt;
        * $("#DigitalGauge1").ejDigitalGauge();
        * var DigitalGaugeObj = $("#DigitalGauge1").data("ejDigitalGauge");
        * DigitalGaugeObj.setValue(0,"Welcome");
        * &lt;/script&gt;
        * @memberof ejDigitalGauge
        * @instance
        */

        setValue: function (itemIndex, value) {
            if (itemIndex < this.model.items.length && value != null) {
                this.model.items[itemIndex].value = value;
                this._initialize();
            }
        },

        /**
        * ClientSideMethod getValue
        * Gets the value of an item that is displayed on the gauge
        * @alias getValue       
        * @param {int}  itemIndex Index value of an item that displayed on the gauge      
        * @example 
        * &lt;div id="DigitalGauge1"&gt;&lt;/div&gt; <br> 
        * &lt;script&gt;
        * $("#DigitalGauge1").ejDigitalGauge();
        * var DigitalGaugeObj = $("#DigitalGauge1").data("ejDigitalGauge");
        * DigitalGaugeObj.getValue(0);	
        * &lt;/script&gt;
        * @memberof ejDigitalGauge
        * @instance
        */
        getValue: function (itemIndex) {
            return this.model.items[itemIndex].value;
        },

        /**
        * ClientSideMethod Set Position
        * Sets the location of an item to be displayed in the gauge
        * @alias setPosition       
        * @param {int} itemIndex  Index value of the digital gauge item
        * @param {object} value  Location value of the digital gauge
        * @example 
        * &lt;div id="DigitalGauge1"&gt;&lt;/div&gt; <br> 
        * &lt;script&gt;
        * $("#DigitalGauge1").ejDigitalGauge();
        * var DigitalGaugeObj = $("#DigitalGauge1").data("ejDigitalGauge");
        * DigitalGaugeObj.setPosition(0,{ x:50, y:40 });	
        * &lt;/script&gt;
        * @memberof ejDigitalGauge
        * @instance
        */

        setPosition: function (itemIndex, value) {
            if (itemIndex < this.model.items.length && value != null) {
                this.model.items[itemIndex].position.x = value.x;
                this.model.items[itemIndex].position.y = value.y;
                this._initialize();
            }
        },


        /**
        * Gets the location of an item that is displayed on the gauge.
        * @alias getPosition       
        * @param {int}  itemIndex Position value of an item that is displayed on the gauge.     
        * @example 
        * &lt;div id="DigitalGauge1"&gt;&lt;/div&gt; <br> 
        * &lt;script&gt;
        * $("#DigitalGauge1").ejDigitalGauge();
        * var DigitalGaugeObj = $("#DigitalGauge1").data("ejDigitalGauge");
        * DigitalGaugeObj.getPosition(0);	
        * &lt;/script&gt;
        * @memberof ejDigitalGauge
        * @instance
        */

        getPosition: function (itemIndex) {
            if (itemIndex < this.model.items.length)
                return { "x": this.model.items[itemIndex].position.x, "y": this.model.items[itemIndex].position.y };
        },

        /**
        * Refresh the digital gauge widget
        * @alias refresh
        * @example 
        * &lt;div id="DigitalGauge1"&gt;&lt;/div&gt; <br> 
        * &lt;script&gt;
        * $("#DigitalGauge1").ejDigitalGauge();
        * var GaugeObj = $("#DigitalGauge1").data("ejDigitalGauge");
        * GaugeObj.refresh();
        * &lt;/script&gt;     
        * @memberof ejDigitalGauge     
        * @instance      
        */
        refresh: function () {
            this._setTheme();
            this._initialize();
        },

        /**
        * To export Digital Gauge as Image
        * @alias exportImage
        * @param {string} fileName fileName for the Image
        * @param {string} fileType fileType for the Image
        * @example 
        * &lt;div id="DigitalGauge"&gt;DigitalGauge1&lt;/div&gt; <br> 
        * &lt;script&gt;
        * $("#DigitalGauge").ejDigitalGauge();
        * var DigitalGaugeObj = $("#DigitalGauge").data("ejDigitalGauge");
        * DigitalGaugeObj.exportImage("myImage","jpeg");
        * &lt;/script&gt;
        * @memberof ejDigitalGauge     
        * @instance
        */
        exportImage: function (fileName, fileType) {
            /// <summary>This function save the rendered canvas image</summary>
            /// <param name="fileName" type="String">fileName to which the image has been saved</param>
            /// <param name="fileType" type="String">fileType to which the image has been saved</param>
            var image = this.canvasEl[0].toDataURL();
            image = image.replace(/^data:[a-z]*;,/, '');
            var image1 = image.split(',');
            var byteString = atob(image1[1]);
            var buffer = new ArrayBuffer(byteString.length);
            var intArray = new Uint8Array(buffer);
            for (var i = 0; i < byteString.length; i++) {
                intArray[i] = byteString.charCodeAt(i);
            }
            blob = new Blob([buffer], { type: "image/png" });
            if (ej.browserInfo().name === "msie" && parseFloat(ej.browserInfo().version) > 9)
                window.navigator.msSaveOrOpenBlob(blob, fileName + '.' + fileType);
            else {
                var pom = document.createElement('a');
                var url = URL.createObjectURL(blob);
                pom.href = url;
                pom.setAttribute('download', fileName + '.' + fileType);
                if (document.createEvent) {
                    e = document.createEvent("MouseEvents");
                    e.initMouseEvent("click", true, true, window, 0, 0, 0, 0, 0, false, false, false, false, 0, null);
                    pom.dispatchEvent(e);
                }
                else if (pom.fireEvent) {
                    pom.fireEvent("onclick");
                }
            }
        },

        /**
        * ClientSideMethod ResizeCanvas		
        * @private
        */
        resizeCanvas: function () {
            if (proxy.element.width() != initialDivWidth || proxy.element.height() != initialDivHeight) {
                var canvasHeight = proxy.element.height() / initialDivHeight;
                var canvasWidth = proxy.element.width() / initialDivWidth;
                proxy.model.width *= canvasWidth;
                proxy.model.height *= canvasHeight;
                for (var i = 0; proxy.model.items[i] != null; i++) {
                    proxy.model.items[i].segmentSettings.width *= canvasWidth;
                    proxy.model.items[i].segmentSettings.spacing *= canvasWidth;
                }
                proxy.refresh();
                initialDivWidth = proxy.element.width();
                initialDivHeight = proxy.element.height();
            }
        },

        //ClientSide events
        /**
        * ClientSide events _clientSideOnLoad		
        * @private
        */
        _clientSideOnLoad: function () {
            var data = { object: this, items: this.model.items, context: this.contextEl };
            this._trigger("load", data);
        },

        /**
        * ClientSide events _clientSideOnItemRendering		
        * @private
        */
        _clientSideOnItemRendering: function (isMatrix, row, column) {
            var data;
            if (isMatrix)
                data = { object: this, items: this.model.items, character: this.character, context: this.contextEl, position: this.location, style: this.style, row: row, column: column };
            else
                data = { object: this, model: this.model, id: this.model.ClientId, items: this.model.items, character: this.character, context: this.contextEl, position: this.location, style: this.style, segment: row };
            this._trigger("itemRendering", data);
        },

        /**
        * ClientSide events _clientSideOnInit		
        * @private
        */
        _clientSideOnInit: function () {
            var data = { object: this, items: this.model.items, context: this.contextEl };
            this._trigger("init", data);
        },

        /**
        * ClientSide events _clientSideOnRenderComplete		
        * @private
        */
        _clientSideOnRenderComplete: function () {
            var data = { object: this, items: this.model.items, context: this.contextEl };
            this._trigger("renderComplete", data);
        },


        /*--------common js ----------*/
        /**
        * ClientSide events _contextOpenPath		
        * @private
        */
        _contextOpenPath: function (style, element) {
            element.contextEl.save();
            element.contextEl.beginPath();
            if (style.strokeStyle)
                element.contextEl.strokeStyle = style.strokeStyle;
            if (style.opacity != undefined)
                element.contextEl.globalAlpha = style.opacity;
            if (style.lineWidth)
                element.contextEl.lineWidth = style.lineWidth;
            if (style.fillStyle)
                element.contextEl.fillStyle = style.fillStyle;
        },

        /**
        * ClientSide events _contextClosePath		
        * @private
        */
        _contextClosePath: function (style, element) {
            element.contextEl.closePath();
            if (style.isFill)
                element.contextEl.fill();
            if (style.isStroke)
                element.contextEl.stroke();
            element.contextEl.restore();
        },

        /**
        * ClientSide events _measureText		
        * @private
        */
        _measureText: function (text, maxwidth, font) {
            var textObj = document.createElement('DIV');
            textObj.innerHTML = text;
            if (font != null)
                textObj.style.font = font;
            textObj.style.backgroundColor = 'white';
            textObj.style.position = 'absolute';
            textObj.style.top = -100;
            textObj.style.left = 0;
            if (maxwidth)
                textObj.style.maxwidth = maxwidth + "px";
            document.body.appendChild(textObj);
            var bounds = { width: textObj.offsetWidth, height: textObj.offsetHeight };
            $(textObj).remove();
            return bounds;
        }

    });

    /**
    * Enum for gauge CharacterType	 
    * @enum {string}
    * @global 
    */
    ej.datavisualization.DigitalGauge.CharacterType = {
        /**  To set the CharacterType sevensegment. */
        SevenSegment: "sevensegment",
        /**  To set the CharacterType fourteensegment. */
        FourteenSegment: "fourteensegment",
        /**  To set the CharacterType sixteensegment. */
        SixteenSegment: "sixteensegment",
        /**  To set the CharacterType eightcrosseightdotmatrix. */
        EightCrossEightDotMatrix: "eightcrosseightdotmatrix",
        /**  To set the CharacterType eightcrosseightsquarematrix. */
        EightCrossEightSquareMatrix: "eightcrosseightsquarematrix"
    };
    /**
    * Enum for gauge TextAlign	 
    * @enum {string}
    * @global 
    */
    ej.datavisualization.DigitalGauge.TextAlign = {
        /**  To set the TextAlign Left. */
        Left: "left",
        /**  To set the TextAlign right. */
        Right: "right"
    };

    /**
    * Enum for gauge FontStyle	 
    * @enum {string}
    * @global 
    */
    ej.datavisualization.DigitalGauge.FontStyle = {
        /**  To set the FontStyle normal. */
        Normal: "normal",
        /**  To set the FontStyle bold. */
        Bold: "bold",
        /**  To set the FontStyle italic. */
        Italic: "italic",
        /**  To set the FontStyle underline. */
        Underline: "underline",
        /**  To set the FontStyle strikeout. */
        Strikeout: "strikeout"
    };

    /**
    * Enum for gauge Themes	 
    * @enum {string}
    * @global 
    */
    ej.datavisualization.DigitalGauge.Themes = {
        /**  To set the themes flatlight. */
        flatlight: {
            items: {
                segmentSettings: {
                    color: "#232323",
                },
                shadowColor: "#232323",
                textColor: "#232323"
            }
        },
        /**  To set the themes flatdark. */
        flatdark: {
            items: {
                segmentSettings: {
                    color: "#b1b0b0",
                },
                shadowColor: "#b1b0b0",
                textColor: "#b1b0b0"
            }
        }
    };

})(jQuery, Syncfusion);;;
/**
* @fileOverview Plugin to style the Html Button elements
* @copyright Copyright Syncfusion Inc. 2001 - 2013. All rights reserved.s
*  Use of this code is subject to the terms of our license.
*  A copy of the current license can be obtained at any time by e-mailing
*  licensing@syncfusion.com. Any infringement will be prosecuted under
*  applicable laws. 
* @version 12.1 
* @author <a href="mailto:licensing@syncfusion.com">Syncfusion Inc</a>
*/

(function ($, ej, undefined) {
    /**
       * @class ejBulletGraph
       * @requires jQuery.js
       * @requires ej.core.js
       * @requires ej.data.js
       * @requires ej.bulletgraph.js
       * @classdesc Custom Design for Bullet graph.
       * @example 
       * &lt;div id="bulletGraph1"&gt;&lt;/div&gt; <br> 
       * &lt;script&gt;
       * // Create BulletGraph
       * $('#bulletGraph1').ejBulletGraph(); 	
       * &lt;/script&gt;
       */
    ej.widget("ejBulletGraph", "ej.datavisualization.BulletGraph", /** @lends ejBulletGraph# */ {
        // widget element will be automatically set in this
        element: null,

        // user defined model will be automatically set in this
        model: null,
        validTags: ["div"],
        // default model
        defaults:/** @lends ejBulletGraph# */ {
            /**		
			* Feature measure bar in bullet graph render till the specified value.
			* @default 0
            * @alias ejBulletGraph#value
			* @type {number}
			* @example 
			* &lt;div id="bulletGraph1"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#bulletGraph1").ejBulletGraph({
      * value : 1                    
      * });
      * &lt;/script&gt;  
			*/
            value: 0,
            /**		
			* Comparative measure bar in bullet graph render till the specified value.
			* @default 0
            * @alias ejBulletGraph#comparativeMeasureValue
			* @type {number}	
			* @example 
			* &lt;div id="bulletGraph1"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#bulletGraph1").ejBulletGraph({
      * comparativeMeasureValue : 1                    
      * });
      * &lt;/script&gt;  
			*/
            comparativeMeasureValue: 0,
            /**		
			* Specifies the width of the bullet graph.
			* @default 595
            * @alias ejBulletGraph#width
			* @type {number}
			* @example 
			* &lt;div id="bulletGraph1"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#bulletGraph1").ejBulletGraph({
      * width : 600                    
      * });
      * &lt;/script&gt;  
			*/
            width: 595,
            /**		
			* Specifies the height of the bullet graph.
			* @default 90
            * @alias ejBulletGraph#height
			* @type {number}
			* @example 
			* &lt;div id="bulletGraph1"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#bulletGraph1").ejBulletGraph({
      * height : 600                    
      * });
      * &lt;/script&gt;  
			*/
            height: 90,
            /**		
			* By specifying this property the user can change the theme of the bullet graph.
			* @default "flatlight"
            * @alias ejBulletGraph#theme
			* @type {string}	
			* @example 
            * &lt;div id="bulletGraph1"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#bulletGraph1").ejBulletGraph({
      * theme : "flatdark"                  
      * });
      * &lt;/script&gt;  
			*/
            theme: "flatlight",
            /**		
			* Bullet graph will render in the specified orientation.
			* @default "horizontal"
            * @alias ejBulletGraph#orientation
			* @type {enum}
			* @example 
			* &lt;div id="bulletGraph1"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#bulletGraph1").ejBulletGraph({
      * orientation : "vertical"                   
      * });
      * &lt;/script&gt; 
			*/
            orientation: "horizontal",
            /**		
			* Specifies the direction of flow in bullet graph. Neither it may be backward nor forward. See {@link FlowDirection}
			* @default "forward"
            * @alias ejBulletGraph#flowDirection
			* @type {enum} 
			* @example 
			* &lt;div id="bulletGraph1"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#bulletGraph1").ejBulletGraph({
      * flowDirection : "backward"                   
      * });
      * &lt;/script&gt; 
			*/
            flowDirection: "forward",
            /**		
			* Size of the qualitative range depends up on the specified value.
			* @default 32
            * @alias ejBulletGraph#qualitativeRangeSize
			* @type {number}
			* @example 
			* &lt;div id="bulletGraph1"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#bulletGraph1").ejBulletGraph({
      * qualitativeRangeSize : 35                  
      * });
      * &lt;/script&gt; 
			*/
            qualitativeRangeSize: 32,
            /**		
			* Length of the quantitative range depends up on the specified value.
			* @default 475
            * @alias ejBulletGraph#quantitativeScaleLength
			* @type {number} 
			* @example 
			* &lt;div id="bulletGraph1"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#bulletGraph1").ejBulletGraph({
      * quantitativeScaleLength :500                 
      * });
      * &lt;/script&gt; 
			*/
            quantitativeScaleLength: 475,
            /**		
           * Contains all the properties to customize tooltip.
           * @type {object}
           */
            tooltipSettings: /** @lends ejBulletGraph# */{
                /**		
			* Toggles the visibility of tooltip
			* @default true
            * @alias ejBulletGraph#tooltipSettings->visible
			* @type {boolean}
			* @example 
			* &lt;div id="bulletGraph1"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#bulletGraph1").ejBulletGraph({
      * tooltipSettings :{visible: false}
      * });
      * &lt;/script&gt; 
			*/
                visible: true,
                /**		
           * Specifies the ID of a div, which is to be displayed as tooltip.
           * @default null
           * @alias ejBulletGraph#tooltipSettings->template
           * @type {string}
           * @example 
           * &lt;div id="bulletGraph1"&gt;&lt;/div&gt; 
     * &lt;script&gt;
     * $("#bulletGraph1").ejBulletGraph({
     * tooltipSettings: { template : "tooltip"},
     * });
     * &lt;/script&gt; 
           */
                template: null
            },
            /**		
			* Contains all the properties to customize quantitative scale.
			* @type {object}
			*/
            quantitativeScaleSettings:  /** @lends ejBulletGraph# */{
                /**		
				* Contains property to customize the position of the quantitative scale
                * @alias ejBulletGraph#quantitativeScaleSettings->location
				* @type {object}				
				*/
                location: {
                    /**		
                    * This property specifies the x position for rendering quantitative scale.
                    * @default 10
                    * @alias ejBulletGraph#quantitativeScaleSettings->location->x
                    * @type {number}
                    * @example 
                    * &lt;div id="bulletGraph1"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#bulletGraph1").ejBulletGraph({
      * quantitativeScaleSettings : { location : { x : 15 } } 
      * });
      * &lt;/script&gt; 
                    */
                    x: 10,
                    /**		
                       * This property specifies the y position for rendering quantitative scale.
                       * @default 10
                       * @alias ejBulletGraph#quantitativeScaleSettings->location->y
                       * @type {number}
                       * @example 
                       * &lt;div id="bulletGraph1"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#bulletGraph1").ejBulletGraph({
      * quantitativeScaleSettings : { location : { y : 15 } }
      * });
      * &lt;/script&gt; 
                       */
                    y: 10
                },
                /**		
				* Specifies the minimum value of the Graph.
				* @default 0
                * @alias ejBulletGraph#quantitativeScaleSettings->minimum
				* @type {number}
				* @example 
				 * &lt;div id="bulletGraph1"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#bulletGraph1").ejBulletGraph({
      * quantitativeScaleSettings : { minimum : 1 }
      * });
      * &lt;/script&gt; 
				*/
                minimum: 0,
                /**		
				* Specifies the maximum value of the Graph.
				* @default 10
                * @alias ejBulletGraph#quantitativeScaleSettings->maximum
				* @type {number}
				* @example 
				 * &lt;div id="bulletGraph1"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#bulletGraph1").ejBulletGraph({
      * quantitativeScaleSettings : { maximum : 8 }                  
      * });
      * &lt;/script&gt; 
				*/
                maximum: 10,
                /**		
				* Specifies the interval for the Graph.
				* @default 1
                * @alias ejBulletGraph#quantitativeScaleSettings->interval
				* @type {number}
				* @example 
				 * &lt;div id="bulletGraph1"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#bulletGraph1").ejBulletGraph({
      * quantitativeScaleSettings : { interval : 2 }
      * });
      * &lt;/script&gt; 
				*/
                interval: 1,
                /**		
				* The specified number of minor ticks will be rendered per interval.
				* @default 4
                * @alias ejBulletGraph#quantitativeScaleSettings->minorTicksPerInterval
				* @type {number}
				* @example 
				 * &lt;div id="bulletGraph1"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#bulletGraph1").ejBulletGraph({
      * quantitativeScaleSettings : { minorTicksPerInterval : 5 }
      * });
      * &lt;/script&gt; 
				*/
                minorTicksPerInterval: 4,
                /**		
					* Contains property to customize the major tick lines.
                    * @alias ejBulletGraph#quantitativeScaleSettings->majorTickSettings
					* @type {object}
			*/
                majorTickSettings: {
                    /**		
                    * Specifies the size of the major ticks.
                    * @default 13
                    * @alias ejBulletGraph#quantitativeScaleSettings->majorTickSettings->size
                    * @type {number}
                    * @example 
                     * &lt;div id="bulletGraph1"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#bulletGraph1").ejBulletGraph({
      * quantitativeScaleSettings : { majorTickSettings : { size : 15 } }
      * });
      * &lt;/script&gt; 
                      */
                    size: 13,
                    /**		
                    * Specifies the stroke color of the major tick lines.
                    * @default null
                    * @alias ejBulletGraph#quantitativeScaleSettings->majorTickSettings->stroke
                    * @type {string}
                    * @example 
                      * &lt;div id="bulletGraph1"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#bulletGraph1").ejBulletGraph({
      * quantitativeScaleSettings : { majorTickSettings : { stroke : "red" } }                  
      * });
      * &lt;/script&gt; 
                      */
                    stroke: null,
                    /**		
                    * Specifies the width of the major tick lines.
                    * @default 2
                    * @alias ejBulletGraph#quantitativeScaleSettings->majorTickSettings->width
                    * @type {number}
                    * @example 
                      * &lt;div id="bulletGraph1"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#bulletGraph1").ejBulletGraph({
      * quantitativeScaleSettings : { majorTickSettings : { width : 1 } }                  
      * });
      * &lt;/script&gt; 
                      */
                    width: 2
                },
                /**		
					* Contains property to customize the minor ticks.
                    * @alias ejBulletGraph#quantitativeScaleSettings->minorTickSettings
					* @type {object}
					*/
                minorTickSettings: {
                    /**		
               * Specifies the size of minor ticks.
               * @default 7
               * @alias ejBulletGraph#quantitativeScaleSettings->minorTickSettings->size
               * @type {number}
               * @example 
                 * &lt;div id="bulletGraph1"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#bulletGraph1").ejBulletGraph({
      * quantitativeScaleSettings : { minorTickSettings : { size : 10} }                  
      * });
      * &lt;/script&gt; 
      */
					size: 7,
               /**     		
               * Specifies the stroke color of minor ticks in bullet graph.
               * @default null
               * @alias ejBulletGraph#quantitativeScaleSettings->minorTickSettings->stroke
               * @type {string}
               * @example 
                 * &lt;div id="bulletGraph1"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#bulletGraph1").ejBulletGraph({
      * quantitativeScaleSettings : { minorTickSettings : { stroke : "green" } }
      * });
      * &lt;/script&gt; 
                 */
                    stroke: null,
                    /**		
                    * Specifies the width of the minor ticks in bullet graph.
                    * @default 2
                    * @alias ejBulletGraph#quantitativeScaleSettings->minorTickSettings->width
                    * @type {number}
                    * @example 
                      * &lt;div id="bulletGraph1"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#bulletGraph1").ejBulletGraph({
      * quantitativeScaleSettings : { minorTickSettings : { width : 1 } }
      * });
      * &lt;/script&gt; 
                      */
                    width: 2
                },
                /**		
               * Specifies the position of the ticks to render either above or below the graph. See {@link LabelPosition} 
               * @default ej.datavisualization.BulletGraph.TickPosition.Below
               * @alias ejBulletGraph#quantitativeScaleSettings->tickPosition
               * @type {enum}
               * @example 
              * &lt;div id="bulletGraph1"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#bulletGraph1").ejBulletGraph({
      * quantitativeScaleSettings : { tickPosition : "above" }
      * });
      * &lt;/script&gt; 
               */
                tickPosition: "below",
                /**		
                * Contains property to customize the labels.
                * @alias ejBulletGraph#quantitativeScaleSettings->labelSettings
                * @type {object}
                */
                labelSettings: {
                    /**		
					* Specifies the prefix to be added with labels in bullet graph.
					* @default Empty string
                    * @alias ejBulletGraph#quantitativeScaleSettings->labelSettings->labelPrefix
					* @type {string}
					* @example 
					* &lt;div id="bulletGraph1"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#bulletGraph1").ejBulletGraph({
      * quantitativeScaleSettings : { labelSettings : { labelPrefix : "$" } }                  
      * });
      * &lt;/script&gt; 
					*/
                    labelPrefix: "",
                    /**		
					* Specifies the suffix to be added after labels in bullet graph.
					* @default Empty string
                    * @alias ejBulletGraph#quantitativeScaleSettings->labelSettings->labelSuffix
					* @type {string}
					* @example 
					* &lt;div id="bulletGraph1"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#bulletGraph1").ejBulletGraph({
      * quantitativeScaleSettings : { labelSettings : { labelSuffix : "K" } }                  
      * });
      * &lt;/script&gt; 
					*/
                    labelSuffix: "",
                    /**		
					* Specifies the stroke color of the labels in bullet graph.
					* @default null
                    * @alias ejBulletGraph#quantitativeScaleSettings->labelSettings->stroke
					* @type {string}
					* @example 
					* &lt;div id="bulletGraph1"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#bulletGraph1").ejBulletGraph({
      * quantitativeScaleSettings : { labelSettings : { stroke : "green" } }                  
      * });
      * &lt;/script&gt; 
					*/
                    stroke: null,
                    /**		
					* Specifies the Size of the labels.
					* @default 12
                    * @alias ejBulletGraph#quantitativeScaleSettings->labelSettings->size
					* @type {number}
					* @example 
					* &lt;div id="bulletGraph1"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#bulletGraph1").ejBulletGraph({
      * quantitativeScaleSettings : { labelSettings : { size : 10 } }                  
      * });
      * &lt;/script&gt; 
					*/
                    size: 12,
                    /**		
					* Specifies the horizontal/vertical padding of labels.
					* @default 20
                    * @alias ejBulletGraph#quantitativeScaleSettings->labelSettings->offset
					* @type {number}
					* @example 
					* &lt;div id="bulletGraph1"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#bulletGraph1").ejBulletGraph({
      * quantitativeScaleSettings : { labelSettings : { offset : 25 } }                  
      * });
      * &lt;/script&gt; 
					*/
                    offset: 20,
                    /**		
					* Contains property to customize the font of the labels in bullet graph.
                    * @alias ejBulletGraph#quantitativeScaleSettings->labelSettings->font
					* @type {object}				
					*/
                    font: {
                        /**		
                        * Specifies the fontFamily of labels in bullet graph. Labels render with this fontFamily.
                        * @default "Segoe UI"
                        * @alias ejBulletGraph#quantitativeScaleSettings->labelSettings->font->fontFamily
                        * @type {string}
                        * @example 
                        * &lt;div id="bulletGraph1"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#bulletGraph1").ejBulletGraph({
      * quantitativeScaleSettings : { labelSettings : { font :{ fontFamily : "Algerian" } } }                  
      * });
      * &lt;/script&gt; 
                        */
                        fontFamily: 'Segoe UI',
                        /**		
                        * Specifies the fontStyle of labels in bullet graph. Labels render with this fontStyle. See {@link FontStyle}
                        * @default "Normal"
                        * @alias ejBulletGraph#quantitativeScaleSettings->labelSettings->font->fontStyle
                        * @type {enum}
                        * @example 
                         * &lt;div id="bulletGraph1"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#bulletGraph1").ejBulletGraph({
      * quantitativeScaleSettings : { labelSettings : { font :{ fontStyle : "italic" } } }
      * });
      * &lt;/script&gt; 
                        */
                        fontStyle: 'Normal ',
                        /**		
                        * Specifies the fontWeight of labels in bullet graph. Labels render with this fontWeight. See {@link FontWeight}
                        * @default "regular"
                        * @alias ejBulletGraph#quantitativeScaleSettings->labelSettings->font->fontWeight
                        * @type {enum}
                        * @example 
                         * &lt;div id="bulletGraph1"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#bulletGraph1").ejBulletGraph({
      * quantitativeScaleSettings : { labelSettings : { font :{ fontWeight : "lighter" } } }
      * });
      * &lt;/script&gt; 
                        */
                        fontWeight: 'regular',
                        /**		
                        * Specifies the opacity of labels in bullet graph. Labels render with this opacity
                        * @default 1
                        * @alias ejBulletGraph#quantitativeScaleSettings->labelSettings->font->opacity
                        * @type {number}
                        * @example 
                         * &lt;div id="bulletGraph1"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#bulletGraph1").ejBulletGraph({
      * quantitativeScaleSettings : { labelSettings : { font :{ opacity : 0.5 } } }
      * });
      * &lt;/script&gt; 
                        */
                        opacity: 1
                    },
                    /**		
					* Specifies the position of the labels to render either above or below the graph. See {@link Position}
					* @default "below"
                    * @alias ejBulletGraph#quantitativeScaleSettings->labelSettings->position
					* @type {enum}
					* @example 
					 * &lt;div id="bulletGraph1"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#bulletGraph1").ejBulletGraph({
      * quantitativeScaleSettings : { labelSettings : { position : "above" } }                  
      * });
      * &lt;/script&gt; 
					*/
                    position: "below"
                },
                /**		
					* Contains property to customize the featured measure.
                    * @alias ejBulletGraph#quantitativeScaleSettings->featuredMeasureSettings
					* @type {object}
					*/
                featuredMeasureSettings: {
                    /**		
                    * Specifies the Stroke of the featured measure in bullet graph.
                    * @default null
                    * @alias ejBulletGraph#quantitativeScaleSettings->featuredMeasureSettings->stroke
                    * @type {number}
                    * @example 
                    * &lt;div id="bulletGraph1"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#bulletGraph1").ejBulletGraph({
      * quantitativeScaleSettings : { featuredMeasureSettings :{stroke : 2} }
      * });
      * &lt;/script&gt; 
                    */

                    stroke: null,
                    /**		
                    * Specifies the width of the featured measure in bullet graph.
                    * @default 2
                    * @alias ejBulletGraph#quantitativeScaleSettings->featuredMeasureSettings->width
                    * @type {number}
                    * @example 
                    * &lt;div id="ejBulletGraph"&gt;&lt;/div&gt; 
                      * &lt;script&gt;
                      *	$("#bulletGraph1").ejBulletGraph({
                      *    quantitativeScaleSettings: { featuredMeasureSettings: { width: 3 }},
                      *    });
                      * &lt;/script&gt;  
                      */
                    width: 6
                },
                /**		
					* Contains property to customize the comparative measure.
                    * @alias ejBulletGraph#quantitativeScaleSettings->comparativeMeasureSettings
					* @type {object}
					*/
                comparativeMeasureSettings: {
                    /**		
                    * Specifies the stroke of the comparative measure.
                    * @default null
                    * @alias ejBulletGraph#quantitativeScaleSettings->comparativeMeasureSettings->stroke
                    * @type {number}
                    * @example 
                     * &lt;div id="bulletGraph1"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#bulletGraph1").ejBulletGraph({
      * quantitativeScaleSettings : { comparativeMeasureSettings :{ stroke :2} }                  
      * });
      * &lt;/script&gt; 
      */
                    stroke: null,
                    /**		
                    * Specifies the width of the comparative measure.
                    * @default 5
                    * @alias ejBulletGraph#quantitativeScaleSettings->comparativeMeasureSettings->width
                    * @type {number}
                    * @example 
                     * &lt;div id="bulletGraph1"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#bulletGraph1").ejBulletGraph({
      *  quantitativeScaleSettings : { comparativeMeasureSettings :{ width :6} }                     
      * });
      * &lt;/script&gt; 
                      */
                    width: 5
                },
                /**		
				* Contains property to customize the featured measure.
                * @alias ejBulletGraph#quantitativeScaleSettings->featureMeasures
				* @type {array}
				*/
                featureMeasures: [{
                    /**		
                    * Feature measure render till the specified value.
                    * @default null
                    * @alias ejBulletGraph#quantitativeScaleSettings->featureMeasures->value
                    * @type {number}
                    * @example 
                   * &lt;div id="bulletGraph1"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#bulletGraph1").ejBulletGraph({
      * quantitativeScaleSettings : { featureMeasures :{ value :2} }                  
      * });
      * &lt;/script&gt;
                    */
                    value: null,
                    /**		
                    * Comparative measure render till the specified value.
                    * @default null
                     * @alias ejBulletGraph#quantitativeScaleSettings->featureMeasures->comparativeMeasureValue
                    * @type {number}
                    * @example 
                    * &lt;div id="bulletGraph1"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#bulletGraph1").ejBulletGraph({
      * quantitativeScaleSettings : { featureMeasures :{ comparativeMeasureValue :2} }                  
      * });
      * &lt;/script&gt;
                    */
                    comparativeMeasureValue: null,
                    /**		
                    * Specifies the category of feature measure.
                    * @default null
                    * @alias ejBulletGraph#quantitativeScaleSettings->featureMeasures->category
                    * @type {string}
                    * @example 
                   * &lt;div id="bulletGraph1"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#bulletGraph1").ejBulletGraph({
      * quantitativeScaleSettings : { featureMeasures :{ category :null} }                  
      * });
      * &lt;/script&gt;
                    */
                    category: null
                }]
            },
            /**		
            * Contains property to customize the fields.
            * @alias ejBulletGraph#quantitativeScaleSettings->fields
            * @type {object}
            */
            fields: {
                /**		
				* Specifies the dataSource for the bullet graph.
				* @default null
                * @alias ejBulletGraph#quantitativeScaleSettings->fields->dataSource
				* @type {object}
				* @example 
				* // To initialize with local Json data to the dataSource
                *$("#SampleBullet").ejBulletGraph({
                *fields: {
                *dataSource: localData,
                *category: “Category”,
                *featureMeasures: “FeatureValue”,
                *comparativeMeasure: “ComparativeValue”
                *}
                *});
                * // where local Json data assigned to the dataSource is as the below one
                *var localData = [{
                *                    FeatureValue: 8, ComparativeValue: 7.5,
                *                    Category: 1
                *                },
                *                {
                *                    FeatureValue: 9, ComparativeValue: 5,
                *                    Category: 2
                *                }];
                * // To initialize with remote data to the dataSource using DataManager
                *$("#SampleBullet").ejBulletGraph({
                *      fields: {
                *           dataSource: dataManager,
                *           query: queryString,
                *           category: "ProductID",
                *           featureMeasures: "UnitPrice",
                *           comparativeMeasure: "Quantity"
                *      }
                *});
                * // Data Manager creation
                *            var dataManager = ej.DataManager({
                *                url: "http://mvc.syncfusion.com/Services/Northwnd.svc/"
                *            });
                *
                * 
				* @example 
				* &lt;div id="bulletGraph1"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#bulletGraph1").ejBulletGraph({
      * fields : {datasource :{ej.DataManger :{url : "http://mvc.syncfusion.com/Services/Northwnd.svc/" }} }                 
      * });
      * &lt;/script&gt; 	
				*/
                dataSource: null,
                /**		
				* Specifies the query for fetching the values form data source to render the bullet graph.
				* @default null
                * @alias ejBulletGraph#quantitativeScaleSettings->fields->query
				* @type {string}
				*/
                query: null,
                /**		
				* Specifies the name of the table.
				* @default null
                * @alias ejBulletGraph#quantitativeScaleSettings->fields->tableName
				* @type {string}
				* @example 
                * &lt;div id="bulletGraph1"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#bulletGraph1").ejBulletGraph({
      * fields :{tableName : "Product"}                 
      * });
      * &lt;/script&gt;
				*/
                tableName: null,
                /**		
				* Specifies the category of the bullet graph.
				* @default null
                * @alias ejBulletGraph#quantitativeScaleSettings->fields->category
				* @type {string}
				* @example 
				* &lt;div id="bulletGraph1"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#bulletGraph1").ejBulletGraph({
      * fields :{category : "ProductId"}                  
      * });
      * &lt;/script&gt;
				*/
                category: null,
                /**		
				* Feature measure render based on the values in the specified field.
				* @default null
                * @alias ejBulletGraph#quantitativeScaleSettings->fields->featureMeasures
				* @type {string}
				* @example 
				* &lt;div id="bulletGraph1"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#bulletGraph1").ejBulletGraph({
      * fields :{featureMeasures : "UnitPrice"}                  
      * });
      * &lt;/script&gt;
				*/
                featureMeasures: null,
                /**		
				* Comparative measure render based on the values in the specified field.
				* @default null
                * @alias ejBulletGraph#quantitativeScaleSettings->fields->comparativeMeasure
				* @type {string}
				* @example 
				* &lt;div id="bulletGraph1"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#bulletGraph1").ejBulletGraph({
      * fields :{comparativeMeasure : "comparativeMeasureValue"}                  
      * });
      * &lt;/script&gt;
				*/
                comparativeMeasure: null
            },
            /**		
			* Toggles the animation of bullet graph.
			* @default true
			* @type {boolean}
			* @example 
			* &lt;div id="bulletGraph1"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#bulletGraph1").ejBulletGraph({
      * enableAnimation : false                 
      * });
      * &lt;/script&gt;
			*/
            enableAnimation: true,
            /**		
			*  Sets a value whether to make the bullet graph responsive on resize.
			* @default true
            * @alias ejBulletGraph#enableResizing
			* @type {boolean}
			* @example 
			* &lt;div id="bulletGraph1"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#bulletGraph1").ejBulletGraph({
      * enableResizing : false                  
      * });
      * &lt;/script&gt;
			*/
            enableResizing: true,
            /**		
			* Toggles the visibility of the range stroke color of the ticks.
			* @default false
            * @alias ejBulletGraph#applyRangeStrokeToTicks
			* @type {boolean}
			* @example 
			* &lt;div id="bulletGraph1"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#bulletGraph1").ejBulletGraph({
      * applyRangeStrokeToTicks : true               
      * });
      * &lt;/script&gt;
			*/
            applyRangeStrokeToTicks: false,
            /**		
			* Toggles the visibility of the range stroke color of the labels.
			* @default false
            * @alias ejBulletGraph#applyRangeStrokeToLabels
			* @type {boolean}
			* @example 
			* &lt;div id="bulletGraph1"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#bulletGraph1").ejBulletGraph({
      * applyRangeStrokeToLabels : true                
      * });
      * &lt;/script&gt;
			*/
            applyRangeStrokeToLabels: false,
            /**		
            * Contains property to customize the qualitative ranges.
            * @alias ejBulletGraph#qualitativeRanges
            * @type {array}
            */
            qualitativeRanges: [{
                /**		
				* Specifies the ending range to which the qualitative ranges will render.
				* @default 3
                * @alias ejBulletGraph#qualitativeRanges->rangeEnd
				* @type {number}
				* @example 
				* &lt;div id="bulletGraph1"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#bulletGraph1").ejBulletGraph({
      * qualitativeRanges :[{rangeEnd : 4.5}]                  
      * });
      * &lt;/script&gt;
				*/
                rangeEnd: 4.3,
                /**		
				* Specifies the stroke for the qualitative ranges.
				* @default null
                * @alias ejBulletGraph#qualitativeRanges->rangeStroke
				* @type {string}
				* @example 
				* &lt;div id="bulletGraph1"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#bulletGraph1").ejBulletGraph({
      * qualitativeRanges :[{rangeStroke : 5}]                 
      * });
      * &lt;/script&gt;
				*/
                rangeStroke: null,
                /**		
				* Specifies the opacity for the qualitative ranges.
				* @default 1
                * @alias ejBulletGraph#qualitativeRanges->rangeOpacity
				* @type {number}
				* @example 
				* &lt;div id="bulletGraph1"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#bulletGraph1").ejBulletGraph({
      * qualitativeRanges :[{rangeOpacity : 0.5}]                 
      * });
      * &lt;/script&gt;
				*/
                rangeOpacity: 1
            }, {
                /**		
				* Specifies the ending range to which the qualitative ranges will render.
				* @default 7.3
                * @alias ejBulletGraph#qualitativeRanges->rangeEnd
				* @type {number}
				* @example 
				* &lt;div id="bulletGraph1"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#bulletGraph1").ejBulletGraph({
      * qualitativeRanges :[{rangeEnd : 7}]                 
      * });
      * &lt;/script&gt;
				*/
                rangeEnd: 7.3,
                /**		
				* Specifies the stroke for the qualitative ranges.
				* @default null
                * @alias ejBulletGraph#qualitativeRanges->rangeStroke
				* @type {string}
				* @example 
				* &lt;div id="bulletGraph1"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#bulletGraph1").ejBulletGraph({
      * qualitativeRanges :[{rangeStroke : 5}]                 
      * });
      * &lt;/script&gt;
				*/
                rangeStroke: null,
                /**		
				* Specifies the opacity for the qualitative ranges.
				* @default 1
                * @alias ejBulletGraph#qualitativeRanges->rangeOpacity
				* @type {number}
				* @example 
				* &lt;div id="bulletGraph1"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#bulletGraph1").ejBulletGraph({
      * qualitativeRanges :[{rangeOpacity : 0.5}]                 
      * });
      * &lt;/script&gt;
				*/
                rangeOpacity: 1
            }, {
                /**		
				*  Specifies the ending range to which the qualitative ranges will render.
				* @default 10
                * @alias ejBulletGraph#qualitativeRanges->rangeEnd
				* @type {number}
				* @example 
				* &lt;div id="bulletGraph1"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#bulletGraph1").ejBulletGraph({
      * qualitativeRanges :[{rangeEnd : 5}]                 
      * });
      * &lt;/script&gt;
				*/
                rangeEnd: 10,
                /**		
				* Specifies the stroke for the qualitative ranges.
				* @default null
                * @alias ejBulletGraph#qualitativeRanges->rangeStroke
				* @type {string}
				* @example 
				* &lt;div id="bulletGraph1"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#bulletGraph1").ejBulletGraph({
      * qualitativeRanges :[{rangeStroke : 5}]                 
      * });
      * &lt;/script&gt;
				*/
                rangeStroke: null,
                /**		
				* Specifies the opacity for the qualitative ranges.
				* @default 1
                * @alias ejBulletGraph#qualitativeRanges->rangeOpacity
				* @type {number}
				* @example 
				* &lt;div id="bulletGraph1"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#bulletGraph1").ejBulletGraph({
      * qualitativeRanges :[{rangeOpacity : 0.5}]                 
      * });
      * &lt;/script&gt;
				*/
                rangeOpacity: 1
            }],
            /**		
            * Contains property to customize the caption in bullet graph.
            * @alias ejBulletGraph#captionSettings
            * @type {object}
            */
            captionSettings: {
                /**		
				* Specifies the angel in which the caption is rendered.
				* @default 0
                * @alias ejBulletGraph#captionSettings->textAngle
				* @type {number}
				* @example 
				* &lt;div id="bulletGraph1"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#bulletGraph1").ejBulletGraph({
      * captionSettings :{textAngle : 5}                 
      * });
      * &lt;/script&gt;
      */
                textAngle: 0,
                /**		
				* Contains property to customize the location.
                * @alias ejBulletGraph#captionSettings->location
				* @type {object}
				*/
                location: {
                    /**		
                    * Specifies the position in horizontal direction
                    * @default 17
                    * @alias ejBulletGraph#captionSettings->location->x
                    * @type {number}
                    * @example 
                    * &lt;div id="bulletGraph1"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#bulletGraph1").ejBulletGraph({
      * captionSettings :{location :{x :15}}            
      * });
      * &lt;/script&gt;
                    */
                    x: 17,
                    /**		
                    * Specifies the position in horizontal direction
                    * @default 30
                    * @alias ejBulletGraph#captionSettings->location->y
                    * @type {number}
                    * @example 
                   * &lt;div id="bulletGraph1"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#bulletGraph1").ejBulletGraph({
      * captionSettings :{location :{y :15}}
      * });
      * &lt;/script&gt;
                    */
                    y: 30
                },
                /**		
				* Specifies the text to be displayed on bullet graph.
				* @default ""
                * @alias ejBulletGraph#captionSettings->text
				* @type {string}
				* @example 
				* &lt;div id="bulletGraph1"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#bulletGraph1").ejBulletGraph({
      * captionSettings :{text : "Production"}
      * });
      * &lt;/script&gt;
				*/
                text: "",
                /**		
				* Contains property to customize the font of caption.
                * @alias ejBulletGraph#captionSettings->font
				* @type {object}
                */
                font: {
                    /**		
                    * Specifies the color of the text in caption.
                    * @default null
                    * @alias ejBulletGraph#captionSettings->font->color
                    * @type {string}
                    * @example 
                   * &lt;div id="bulletGraph1"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#bulletGraph1").ejBulletGraph({
      * captionSettings :{font :{color : "green"}}                  
      * });
      * &lt;/script&gt;
                    */
                    color: null,
                    /**		
                    * Specifies the fontFamily of caption. Caption text render with this fontFamily
                    * @default "Segoe UI"
                    * @alias ejBulletGraph#captionSettings->font->fontFamily
                    * @type {string}
                    * @example 
                    * &lt;div id="bulletGraph1"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#bulletGraph1").ejBulletGraph({
      * captionSettings :{font :{fontFamily : "algerian"}}
      * });
      * &lt;/script&gt;
                    */
                    fontFamily: 'Segoe UI',
                    /**		
                    * Specifies the fontStyle of caption. Caption text render with this fontStyle. See {@link FontStyle}
                    * @default "Normal"
                    * @alias ejBulletGraph#captionSettings->font->fontStyle
                    * @type {enum}
                    * @example 
                    * &lt;div id="bulletGraph1"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#bulletGraph1").ejBulletGraph({
      * captionSettings :{font :{fontStyle : "italic"}}                    
      * });
      * &lt;/script&gt;
                    */
                    fontStyle: 'Normal',
                    /**		
                    * Specifies the size of caption. Caption text render with this size
                    * @default "12px"
                    * @alias ejBulletGraph#captionSettings->font->size
                    * @type {string}
                    * @example 
                    * &lt;div id="bulletGraph1"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#bulletGraph1").ejBulletGraph({
      * captionSettings :{font :{size : "14px"}}                    
      * });
      * &lt;/script&gt;
                    */
                    size: '12px',
                    /**		
                    * Specifies the fontWeight of caption. Caption text render with this fontWeight. See {@link FontWeight}
                    * @default "regular"
                    * @alias ejBulletGraph#captionSettings->font->fontWeight
                    * @type {enum}
                    * @example 
                    * &lt;div id="bulletGraph1"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#bulletGraph1").ejBulletGraph({
      * captionSettings :{font :{fontWeight : "lighter"}}                    
      * });
      * &lt;/script&gt;
                    */
                    fontWeight: 'regular',
                    /**		
                    * Specifies the opacity of caption. Caption text render with this opacity.
                    * @default 1
                    * @alias ejBulletGraph#captionSettings->font->opacity
                    * @type {number}
                    * @example 
                    * &lt;div id="bulletGraph1"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#bulletGraph1").ejBulletGraph({
      * captionSettings :{font :{opacity : 0.5}}                    
      * });
      * &lt;/script&gt;
                    */
                    opacity: 1
                },
                /**		
				* Contains property to customize the subtitle.
                * @alias ejBulletGraph#captionSettings->subTitle
				* @type {object}
	            */
                subTitle: {
                    /**		
                    * Subtitle render in the specified angle.
                    * @default 0
                    * @alias ejBulletGraph#captionSettings->subTitle->textAngle
                    * @type {number}
                    * @example 
                    * &lt;div id="bulletGraph1"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#bulletGraph1").ejBulletGraph({
      * captionSettings :{ subTitle :{textAngle :10} }                    
      * });
      * &lt;/script&gt;
                    */
                    textAngle: 0,
                    /**		
                    * Specifies the text to be displayed as subtitle.
                    * @default ""
                    * @alias ejBulletGraph#captionSettings->subTitle->text
                    * @type {string}
                    * @example 
                   * &lt;div id="bulletGraph1"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#bulletGraph1").ejBulletGraph({
      * captionSettings :{ subTitle :{text : "Power Production"} }
      * });
      * &lt;/script&gt;e
                    */
                    text: "",
                    /**		
					* Contains property to customize the location of subtitle.
                    * @alias ejBulletGraph#captionSettings->subTitle->location
					* @type {object}
					*/
                    location: {
                        /**		
                        * Specifies the horizontal position of the subtitle.
                        * @default 10
                        * @alias ejBulletGraph#captionSettings->subTitle->location->x
                        * @type {number}
                        * @example 
                        * &lt;div id="bulletGraph1"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#bulletGraph1").ejBulletGraph({
      * captionSettings :{subTitle :{location : { x :12 }}}                    
      * });
      * &lt;/script&gt;
                        */
                        x: 10,
                        /**		
                        * Specifies the vertical position of the subtitle.
                        * @default 45
                        * @alias ejBulletGraph#captionSettings->subTitle->location->y
                        * @type {number}
                        * @example 
                        * &lt;div id="bulletGraph1"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#bulletGraph1").ejBulletGraph({
      * captionSettings :{subTitle :{location : { y :50 }}}                    
      * });
      * &lt;/script&gt;
      */
                        y: 45
                    },
                    /**		
					* Contains property to customize the font of subtitle.
                    * @alias ejBulletGraph#captionSettings->subTitle->font
					* @type {object}
                    */
                    font: {
                        /**		
                        * Specifies the color of the subtitle's text.
                        * @default null
                        * @alias ejBulletGraph#captionSettings->subTitle->font->color
                        * @type {string}
                        * @example 
                       * &lt;div id="bulletGraph1"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#bulletGraph1").ejBulletGraph({
      * captionSettings :{subTitle :{font : { color :"green" }}}                    
      * });
      * &lt;/script&gt;
      */
                        color: null,
                        /**		
                        * Specifies the fontFamily of subtitle. Subtitle text render with this fontFamily.
                        * @default "Segoe UI"
                        * @alias ejBulletGraph#captionSettings->subTitle->font->fontFamily
                        * @type {string}
                        * @example 
                         * &lt;div id="bulletGraph1"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#bulletGraph1").ejBulletGraph({
      * captionSettings :{subTitle :{font : { fontFamily :"Algerian" }}}                    
      * });
      * &lt;/script&gt;
      */
                        
                        fontFamily: 'Segoe UI',
                        /**		
                        * Specifies the fontStyle of subtitle. Subtitle text render with this fontStyle. See {@link FontStyle}
                        * @default "Normal"
                        * @alias ejBulletGraph#captionSettings->subTitle->font->fontStyle
                        * @type {enum}
                        * @example 
                         * &lt;div id="bulletGraph1"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#bulletGraph1").ejBulletGraph({
      * captionSettings :{subTitle :{font : { fontStyle :"italic" }}}                    
      * });
      * &lt;/script&gt;
      */
                        
                        fontStyle: 'Normal ',
                        /**		
                        * Specifies the size of subtitle. Subtitle text render with this size.
                        * @default "12px"
                        * @alias ejBulletGraph#captionSettings->subTitle->font->size
                        * @type {string}
                        * @example 
                        * &lt;div id="bulletGraph1"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#bulletGraph1").ejBulletGraph({
      * captionSettings :{subTitle :{font : { size :"14px" }}}                    
      * });
      * &lt;/script&gt;
      */
                        
                        size: '12px',
                        /**		
                        * Specifies the fontWeight of subtitle. Subtitle text render with this fontWeight. See {@link FontWeight}
                        * @default "regular"
                        * @alias ejBulletGraph#captionSettings->subTitle->font->fontWeight
                        * @type {enum}
                        * @example 
                         * &lt;div id="bulletGraph1"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#bulletGraph1").ejBulletGraph({
      * captionSettings :{subTitle :{font : { fontWeight :"lighter" }}}                    
      * });
      * &lt;/script&gt;
      */
                        fontWeight: 'regular',
                        /**		
                        * Specifies the opacity of subtitle. Subtitle text render with this opacity.
                        * @default 1
                        * @alias ejBulletGraph#captionSettings->subTitle->font->opacity
                        * @type {number}
                        * @example 
                         * &lt;div id="bulletGraph1"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#bulletGraph1").ejBulletGraph({
      * captionSettings :{subTitle :{font : { opacity :0.5 }}}                    
      * });
      * &lt;/script&gt;
      
                        */
                        opacity: 1
                    }
                },

                /**		
				* Contains property to customize the indicator.
                * @alias ejBulletGraph#captionSettings->indicator
				* @type {object}
	            */
                indicator: {
                    /**		
                    * Specifies whether indicator will be visible or not.
                    * @default false
                    * @alias ejBulletGraph#captionSettings->indicator->visibile
                    * @type {boolean}
                    * @example 
                    * &lt;div id="bulletGraph1"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#bulletGraph1").ejBulletGraph({
      * captionSettings :{ indicator :{visible : true} }                    
      * });
      * &lt;/script&gt;
                    */
                    visibile: false,
                    /**		
                    * indicator text render in the specified angle.
                    * @default 0
                    * @alias ejBulletGraph#captionSettings->indicator->textAngle
                    * @type {number}
                    * @example 
                    * &lt;div id="bulletGraph1"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#bulletGraph1").ejBulletGraph({
      * captionSettings :{ indicator :{textAngle :10} }                    
      * });
      * &lt;/script&gt;
                    */
                    textAngle: 0,
                    /**		
                    * Specifies the space between indicator symbol and text.
                    * @default 3
                    * @alias ejBulletGraph#captionSettings->indicator->textSpacing
                    * @type {number}
                    * @example 
                    * &lt;div id="bulletGraph1"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#bulletGraph1").ejBulletGraph({
      * captionSettings :{ indicator :{textSpacing :10} }                    
      * });
      * &lt;/script&gt;
                    */
                    textSpacing: 3,
                    /**		
                    * Specifies the text to be displayed as indicator text. By default difference between current value and target will be displayed
                    * @default ""
                    * @alias ejBulletGraph#captionSettings->indicator->text
                    * @type {string}
                    * @example 
                   * &lt;div id="bulletGraph1"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#bulletGraph1").ejBulletGraph({
      * captionSettings :{ indicator :{text : "Power Production"} }
      * });
      * &lt;/script&gt;e
                    */
                    text: "",                    
                    /**		
					* Contains property to customize the symbol of indicator.
                    * @alias ejBulletGraph#captionSettings->indicator->symbol
					* @type {object}
					*/
                    symbol: {
                        /**		
					    * Contains property to customize the border of indicator symbol.
                        * @alias ejBulletGraph#captionSettings->indicator->symbol->border
					    * @type {object}
					    */
                        border: {
                            /**		
                            * Specifies the border color of indicator symbol.
                            * @default null
                            * @alias ejBulletGraph#captionSettings->indicator->symbol->border->color
                            * @type {string}
                            * @example 
                            * &lt;div id="bulletGraph1"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#bulletGraph1").ejBulletGraph({
      * captionSettings :{indicator :{symbol : { border: { color :"green" } }}}                    
      * });
      * &lt;/script&gt;
                        */
                            color: null,
                            /**		
                            * Specifies the border width of indicator symbol.
                            * @default 1
                            * @alias ejBulletGraph#captionSettings->indicator->symbol->border->width
                            * @type {Number}
                            * @example 
                            * &lt;div id="bulletGraph1"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#bulletGraph1").ejBulletGraph({
      * captionSettings :{indicator :{symbol : { border: { width : 2 } }}}                    
      * });
      * &lt;/script&gt;
                        */
                            width: 1,
                        },
                        /**		
                        * Specifies the color of indicator symbol.
                        * @default null
                        * @alias ejBulletGraph#captionSettings->indicator->symbol->color
                        * @type {string}
                        * @example 
                        * &lt;div id="bulletGraph1"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#bulletGraph1").ejBulletGraph({
      * captionSettings :{indicator :{symbol : { color :"green" }}}                    
      * });
      * &lt;/script&gt;
                        */
                        color: null,
                        /**		
                        * Specifies the shape of indicator symbol.
                        * @default ""
                        * @alias ejBulletGraph#captionSettings->indicator->symbol->shape
                        * @type {string}
                        * @example 
                        * &lt;div id="bulletGraph1"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#bulletGraph1").ejBulletGraph({
      * captionSettings :{indicator :{symbol : { shape :"triangle" }}}                    
      * });
      * &lt;/script&gt;
      */
                        shape: "",
                        /**		
                        * Specifies the url of image that represents indicator symbol.
                        * @default ""
                        * @alias ejBulletGraph#captionSettings->indicator->symbol->imageURL
                        * @type {string}
                        * @example 
                        * &lt;div id="bulletGraph1"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#bulletGraph1").ejBulletGraph({
      * captionSettings :{indicator :{symbol : { imageURL :"../BulletIndicator.png" }}}                    
      * });
      * &lt;/script&gt;
      */
                        imageURL: "",
                        /**		
					    * Contains property to customize the size of indicator symbol.
                        * @alias ejBulletGraph#captionSettings->indicator->symbol->size
					    * @type {object}
					    */
                        size: {
                            /**		
                            * Specifies the width of indicator symbol.
                            * @default 10
                            * @alias ejBulletGraph#captionSettings->indicator->symbol->size->width
                            * @type {number}
                            * @example 
                            * &lt;div id="bulletGraph1"&gt;&lt;/div&gt; 
        * &lt;script&gt;
        * $("#bulletGraph1").ejBulletGraph({
        * captionSettings :{indicator :{symbol : { size : { width: 10 }}}                    
        * });
        * &lt;/script&gt;
        */
                            width: 10,
                            /**		
                            * Specifies the height of indicator symbol.
                            * @default 10
                            * @alias ejBulletGraph#captionSettings->indicator->symbol->size->height
                            * @type {number}
                            * @example 
                            * &lt;div id="bulletGraph1"&gt;&lt;/div&gt; 
        * &lt;script&gt;
        * $("#bulletGraph1").ejBulletGraph({
        * captionSettings :{indicator :{symbol : { size : { height: 10 } }}}                    
        * });
        * &lt;/script&gt;
        */
                            height: 10
                        },
                        /**		
                        * Specifies the opacity of indicator symbol.
                        * @default 1
                        * @alias ejBulletGraph#captionSettings->indicator->symbol->opacity
                        * @type {number}
                        * @example 
                         * &lt;div id="bulletGraph1"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#bulletGraph1").ejBulletGraph({
      * captionSettings :{indicator :{symbol : { opacity :0.5 }}}                    
      * });
      * &lt;/script&gt;
      
                        */
                        opacity: 1
                    },
                    /**		
					* Contains property to customize the location of indicator.
                    * @alias ejBulletGraph#captionSettings->indicator->location
					* @type {object}
					*/
                    location: {
                        /**		
                        * Specifies the horizontal position of the indicator.
                        * @default 10
                        * @alias ejBulletGraph#captionSettings->indicator->location->x
                        * @type {number}
                        * @example 
                        * &lt;div id="bulletGraph1"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#bulletGraph1").ejBulletGraph({
      * captionSettings :{indicator :{location : { x :12 }}}                    
      * });
      * &lt;/script&gt;
                        */
                        x: 10,
                        /**		
                        * Specifies the vertical position of the indicator.
                        * @default 60
                        * @alias ejBulletGraph#captionSettings->indicator->location->y
                        * @type {number}
                        * @example 
                        * &lt;div id="bulletGraph1"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#bulletGraph1").ejBulletGraph({
      * captionSettings :{indicator :{location : { y :60 }}}                    
      * });
      * &lt;/script&gt;
      */
                        y: 60
                    },
                    /**		
					* Contains property to customize the font of indicator.
                    * @alias ejBulletGraph#captionSettings->indicator->font
					* @type {object}
                    */
                    font: {
                        /**		
                        * Specifies the color of the indicator's text.
                        * @default null
                        * @alias ejBulletGraph#captionSettings->indicator->font->color
                        * @type {string}
                        * @example 
                       * &lt;div id="bulletGraph1"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#bulletGraph1").ejBulletGraph({
      * captionSettings :{indicator :{font : { color :"green" }}}                    
      * });
      * &lt;/script&gt;
      */
                        color: null,
                        /**		
                        * Specifies the fontFamily of indicator. Indicator text render with this fontFamily.
                        * @default "Segoe UI"
                        * @alias ejBulletGraph#captionSettings->indicator->font->fontFamily
                        * @type {string}
                        * @example 
                         * &lt;div id="bulletGraph1"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#bulletGraph1").ejBulletGraph({
      * captionSettings :{indicator :{font : { fontFamily :"Algerian" }}}                    
      * });
      * &lt;/script&gt;
      */

                        fontFamily: 'Segoe UI',
                        /**		
                        * Specifies the fontStyle of indicator. Indicator text render with this fontStyle. See {@link FontStyle}
                        * @default "Normal"
                        * @alias ejBulletGraph#captionSettings->indicator->font->fontStyle
                        * @type {enum}
                        * @example 
                         * &lt;div id="bulletGraph1"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#bulletGraph1").ejBulletGraph({
      * captionSettings :{indicator :{font : { fontStyle :"italic" }}}                    
      * });
      * &lt;/script&gt;
      */

                        fontStyle: 'Normal ',
                        /**		
                        * Specifies the size of indicator. Indicator text render with this size.
                        * @default "12px"
                        * @alias ejBulletGraph#captionSettings->indicator->font->size
                        * @type {string}
                        * @example 
                        * &lt;div id="bulletGraph1"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#bulletGraph1").ejBulletGraph({
      * captionSettings :{indicator :{font : { size :"14px" }}}                    
      * });
      * &lt;/script&gt;
      */

                        size: '12px',
                        /**		
                        * Specifies the fontWeight of indicator. Indicator text render with this fontWeight. See {@link FontWeight}
                        * @default "regular"
                        * @alias ejBulletGraph#captionSettings->indicator->font->fontWeight
                        * @type {enum}
                        * @example 
                         * &lt;div id="bulletGraph1"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#bulletGraph1").ejBulletGraph({
      * captionSettings :{indicator :{font : { fontWeight :"lighter" }}}                    
      * });
      * &lt;/script&gt;
      */
                        fontWeight: 'regular',
                        /**		
                        * Specifies the opacity of indicator text. Indicator text render with this Opacity.
                        * @default 1
                        * @alias ejBulletGraph#captionSettings->indicator->font->opacity
                        * @type {Number}
                        * @example 
                         * &lt;div id="bulletGraph1"&gt;&lt;/div&gt; 
      * &lt;script&gt;
      * $("#bulletGraph1").ejBulletGraph({
      * captionSettings :{indicator :{font : { opacity : 0.5 }}}                    
      * });
      * &lt;/script&gt;
      */
                        opacity: 1
                    }
                }
            },

            //Events
            /**     
			 * Fires on rendering of ticks.
			 * @event
			 * @name ejBulletGraph#drawTicks		
			 * @param{Object} args.Object returns the object of the bullet graph.
			 * @param{scaleElement}	args.scaleElement returns the options of the scale element.
			 * @param{tickElement} args.tickElement returns the current tick element.
			 * @param{tickType}	args.tickType returns the current tick element’s type.
			 * @example 
			 * //drawTicks event for bulletgraph
             * $("#bulletGraph1").ejBulletGraph({
             *    drawTicks: function (args) {}
             * });
			 */
            drawTicks: null,
            /**     
			 * Fires on rendering the labels.
			 * @event
			 * @name ejBulletGraph#drawLabels		
			 * @param {Object} args.Object returns the object of the bullet graph.
			 * @param {scaleElement} args.scaleElement returns the options of the scale element.
			 * @param {labelElement} args.tickElement returns the current label element.
			 * @param {labelType} args.labelType returns the label type.
			 * @example 
			 * //drawLabels event for bulletgraph
             * $("#bulletGraph1").ejBulletGraph({
             *    drawLabels: function (args) {}
             * });
			 */
            drawLabels: null,
            /**     
			 * Fires on rendering the caption of bullet graph.
			 * @event
			 * @name ejBulletGraph#drawCaption		
			 * @param {Object} args.Object returns the object of the bullet graph.
			 * @param {scaleElement} args.scaleElement returns the options of the scale element.
			 * @param {captionElement} args.captionElement returns the current captionSettings element.
			 * @param {captionType} args.captionType returns the type of the captionSettings.
			 * @example 
			 * //drawCaption event for bulletgraph
             * $("#bulletGraph1").ejBulletGraph({
             *    drawCaption: function (args) {}
             * });
			 */
            drawCaption: null,
            /**     
			 * Fires on rendering the indicator of bullet graph.
			 * @event
			 * @name ejBulletGraph#drawIndicator		
			 * @param {indicatorSettings} args.indicatorSettings returns an object to customize bullet graph indicator text and symbol before rendering it.
			 * @param {model} args.model returns the object of bullet graph.
			 * @param {type} args.type returns the type of event.
             * @param {cancel} args.cancel for cancelling the event.
			 * @example 
			 * //drawIndicator event for bulletgraph
             * $("#bulletGraph1").ejBulletGraph({
             *    drawIndicator: function (args) {}
             * });
			 */
            drawIndicator: null,
            /**     
			 * Fires on rendering the qualitative ranges.
			 * @event
			 * @name ejBulletGraph#drawQualitativeRanges		
			 * @param {Object} args.Object returns the object of the bullet graph.
			 * @param {scaleElement} args.scaleElement returns the options of the scale element.
			 * @param {captionElement} args.captionElement returns the current captionSettings element.
			 * @param {captionType} args.captionType returns the type of the captionSettings.
			 * @example 
			 * //drawQualitativeRanges event for bulletgraph
             * $("#bulletGraph1").ejBulletGraph({
             *    drawQualitativeRanges: function (args) {}
             * });
			 */
            drawQualitativeRanges: null,
            /**     
			 * Fires on rednering the feature measure bar.
			 * @event
			 * @name ejBulletGraph#drawFeatureMeasureBar		
			 * @param {Object} args.Object returns the object of the bullet graph.
			 * @param {scaleElement} args.scaleElement returns the options of the scale element.
			 * @param {currentElement} args.currentElement returns the options of feature measure element.
			 * @param {Value} args.Value returns the value of the feature measure bar.
			 * @example 
			 * //drawFeatureMeasureBar event for bulletgraph
             * $("#bulletGraph1").ejBulletGraph({
             *    drawFeatureMeasureBar: function (args) {}
             * });
			 */
            drawFeatureMeasureBar: null,
            /**     
			 * Fires on rendering the category.
			 * @event
			 * @name ejBulletGraph#drawCategory		
			 * @param {Object} args.Object returns the object of the bullet graph.
			 * @param {scaleElement} args.scaleElement returns the options of the scale element.
			 * @param {categoryElement} args.categoryElement returns the options of category element.
			 * @param {Value}  args.Value returns the text value of the category that is drawn.
			 * @example 
			 * //drawCategory event for bulletgraph
             * $("#bulletGraph1").ejBulletGraph({
             *    drawCategory: function (args) {}
             * });
			 */
            drawCategory: null,
            /**     
			 * Fires on rendering the comparative measure symbol.
			 * @event
			 * @name ejBulletGraph#drawComparativeMeasureSymbol		
			 * @param {Object}  args.Object returns the object of the bullet graph.
			 * @param {scaleElement} args.scaleElement returns the options of the scale element.
			 * @param {targetElement} args.targetElement returns the options of comparative measure element.
			 * @param {Value} args.Value returns the value of the comparative measure symbol.
			 * @example 
			 * //drawComparativeMeasureSymbol event for bulletgraph
             * $("#bulletGraph1").ejBulletGraph({
             *    drawComparativeMeasureSymbol: function (args) {}
             * });
			 */
            drawComparativeMeasureSymbol: null
        },

        observables: ["value", "comparativeMeasureValue"],
        value: ej.util.valueFunction("value"),
        comparativeMeasureValue: ej.util.valueFunction("comparativeMeasureValue"),
        /**
         * Specify the data types for default properties 
		 * @private
         */
        dataTypes: {
            quantitativeScaleSettings: {
                labelSettings: "data",
                featureMeasures: "array"
            },
            fields: { dataSource: "data", query: "data" },
            qualitativeRanges: "array",
            captionSettings: "data"
        },

        // constructor function
        /**
         * Create the bullet graph widget
		 * @private
         */
        _init: function () {
            this._renderBulletGraph();
        },
        /**
         * check whether SVG element
		 * @private
         */
        _isSVG: function () {
            if (window.SVGSVGElement)
                return true;
            else
                return false;
        },
        /**
         * rendering the bullet graph
		 * @private
         */
        _renderBulletGraph: function () {
            if (this._isSVG()) {
                this.svgRenderer = new ej.EjSvgRender(this.element);
                this.svgObject = this.svgRenderer.svgObj;
                this._trigger("load");
                for (var i = 0; this.model.quantitativeScaleSettings.featureMeasures[i] != null; i++) {
                    if (this.model.quantitativeScaleSettings.featureMeasures[i].value == null)
                        this.model.quantitativeScaleSettings.featureMeasures[i].value = this.value();
                    if (this.model.quantitativeScaleSettings.featureMeasures[i].comparativeMeasureValue == null)
                        this.model.quantitativeScaleSettings.featureMeasures[i].comparativeMeasureValue = this.comparativeMeasureValue();
                }
                this._setTheme(ej.datavisualization.BulletGraph.Themes, this.model.theme);
                this._renderBulletElements();
                this.bindEvents();
                if (this.model.enableAnimation)
                    this._animateMeasures();
            }
        },
        /**
         * checking the animate state
		 * @private
         */
        _animateMeasures: function () {
            this._doAnimation();
            this._doLineAnimation();
        },
        /**
         * rendering the bullet Element
		 * @private
         */
        _renderBulletElements: function () {
            this.svgObject.setAttribute('width', this.model.width);
            this.svgObject.setAttribute('height', this.model.height);
            this.svgObject.setAttribute('viewBox', "0 0 " + this.model.width + " " + this.model.height);
            this.svgObject.setAttribute('preserveAspectRatio', "xMinYMin");
            this.svgWidth = this.model.width;
            this.svgHeight = this.model.height;
            var captionGroup = this.svgRenderer.createGroup({ 'id': this.svgObject.id + '_captionGroup' });
            var scaleGroup = this.svgRenderer.createGroup({ 'id': this.svgObject.id + '_scaleGroup' });
            var bulletGroup = this.svgRenderer.createGroup({ 'id': this.svgObject.id + '_outerWrap' });

            this._initializeValues();
            var intervalValue = ((this.model.quantitativeScaleLength) / ((this._scale.maximum - this._scale.minimum) / this._scale.interval));

            //render captionSettings
            this._drawCaption(captionGroup);

            //render indicator
            this._drawIndicator(captionGroup);

            //render scale
            this._scaleGroup = scaleGroup;
            this._drawScale(captionGroup, scaleGroup);

            //draw ranges
            this._drawQualitativeRanges(scaleGroup);

            //render ticks
            this._drawMajorTicks(intervalValue, scaleGroup);
            this._drawMinorTicks(intervalValue, scaleGroup);

            //render labels
            this._drawLabels(intervalValue, scaleGroup);

            // data binding
            this._bindData();

            $(captionGroup).appendTo(bulletGroup);
            $(scaleGroup).appendTo(bulletGroup);

            this._changeOrientation(bulletGroup);

            $(bulletGroup).appendTo(this.svgObject);
            $(this.svgObject).appendTo(this.element);
        },
        /**
         * Initializing the control values
		 * @private
         */
        _initializeValues: function () {
            this._scale = this.model.quantitativeScaleSettings;
            this._labelPosition = this._scale.labelSettings.position;
            this._tickPosition = this._scale.tickPosition;
            this._flowDirection = this.model.flowDirection;
            this._orientation = this.model.orientation;
        },
        /**
         * checking the orientation
		 * @private
         */
        _changeOrientation: function (bulletGroup) {
            if (this._orientation == ej.datavisualization.BulletGraph.Orientation.Vertical) {
                bulletGroup.setAttribute("transform", "translate(" + (this.model.height - this.model.width) + ", " + (-(this.model.width + (2 * this._scale.labelSettings.offset) + this._scale.labelSettings.size)) + ") scale(1) rotate(-90 " + this.model.width + " " + this.model.height + ")");
            }
        },
        /**
         * To configure the properties at runtime using SetModel		
		 * @private
         */
        _setModel: function (options) {
            var isFlag = true;
            for (var key in options) {
                this.model.enableAnimation = false;
                switch (key) {
                    case "height":
                        this.model.height = options[key];
                        break;
                    case "width":
                        this.model.width = options[key];
                        break;
                    case "theme":
                        this.model.theme = options[key];
                        this._setTheme(ej.datavisualization.BulletGraph.Themes, this.model.theme);
                        break;
                    case "orientation":
                        this.model.orientation = options[key];
                        break;
                    case "flowDirection":
                        this.model.flowDirection = options[key];
                        break;
                    case "qualitativeRangeSize":
                        this.model.qualitativeRangeSize = options[key];
                        break;
                    case "quantitativeScaleLength":
                        this.model.quantitativeScaleLength = options[key];
                        break;
                    case "quantitativeScaleSettings":
                        $.extend(true, this.model.quantitativeScaleSettings, {}, options[key]);
                        break;
                    case "applyRangeStrokeToTicks":
                        this.model.applyRangeStrokeToTicks = options[key];
                        break;
                    case "applyRangeStrokeToLabels":
                        this.model.applyRangeStrokeToLabels = options[key];
                        break;
                    case "qualitativeRanges":
                        $.extend(true, this.model.qualitativeRanges, {}, options[key]);
                        break;
                    case "captionSettings":
                        $.extend(true, this.model.captionSettings, {}, options[key]);
                        break;
                    case "dataSource":
                        $.extend(true, this.model.fields, {}, options[key]);
                        break;
                    case "value":
                        for (var i = 0; this.model.quantitativeScaleSettings.featureMeasures[i] != null; i++) {
                            this.model.quantitativeScaleSettings.featureMeasures[i].value = parseFloat(this.value());
                        }
                        break;
                    case "comparativeMeasureValue":
                        for (var i = 0; this.model.quantitativeScaleSettings.featureMeasures[i] != null; i++) {
                            this.model.quantitativeScaleSettings.featureMeasures[i].comparativeMeasureValue = parseFloat(this.comparativeMeasureValue());
                        }
                        break;
                    case "enableAnimation":
                        this.model.enableAnimation = options[key];
                        if (this.model.enableAnimation) {
                            $(this.svgObject).empty();
                            this._renderBulletElements();
                            this._animateMeasures();
                        }
                        isFlag = false;
                        break;
                }
            }
            if (isFlag) {
                $(this.svgObject).empty();
                this._renderBulletElements();
            }
        },
        /**
         * To bind the datasource value		
		 * @private
         */
        _bindData: function () {

            if (!ej.isNullOrUndefined(this.model.fields) && this.model.fields["dataSource"] != null) {
                if (this.model.fields["dataSource"] instanceof ej.DataManager) {
                    this._initDataSource();
                }
                else {
                    this._dataCount = this.model.fields["dataSource"].length;
                    this._drawMeasures();
                }
            }
            else {      //this.model.quantitativeScaleSettings.featureMeasures != null
                this._dataCount = this._scale.featureMeasures.length;
                this._drawMeasures();
            }
        },
        /**
         * To draw measures value		
		 * @private
         */
        _drawMeasures: function () {
            this._drawFeatureMeasureBar();
            this._drawComparativeMeasureSymbol();
        },
        /**
         * To draw measures value		
		 * @private
         */
        _initDataSource: function () {
            var query = this._columnToSelect(this.model.fields);
            var proxy = this;
            var queryPromise = this.model.fields["dataSource"].executeQuery(query);
            queryPromise.done(function (e) {
                proxy.model.fields["dataSource"] = e.result;
                proxy._dataCount = e.result.length;
                proxy._drawFeatureMeasureBar();
                proxy._drawComparativeMeasureSymbol();
            });
        },
        /**
         * To select column name from the datasource		
		 * @private
         */
        _columnToSelect: function (mapper) {
            var column = [], queryManager = ej.Query();
            if (ej.isNullOrUndefined(mapper.query)) {
                for (var col in mapper) {
                    if (col !== "tableName" && col !== "query" && col !== "dataSource")
                        column.push(mapper[col]);
                }
                if (column.length > 0)
                    queryManager.select(column);
                if (!this.model.fields["dataSource"].dataSource.url.match(mapper.tableName + "$"))
                    !ej.isNullOrUndefined(mapper.tableName) && queryManager.from(mapper.tableName);
            }
            else
                queryManager = mapper.query;
            return queryManager;
        },
        /**
         * To draw the captionSettings for the graph	
		 * @private
         */        
        _drawCaption: function (captionGroup) {
            if (this.model.drawCaption) {
                var data = { font: this.model.captionSettings.font, location: this.model.captionSettings.location, subTitle: this.model.captionSettings.subTitle, text: this.model.captionSettings.text, textAngle: this.model.captionSettings.textAngle };
                this._trigger("drawCaption", data);
                this.model.captionSettings = $.extend(this.model.captionSettings, data);
            }
            this.svgRenderer.drawText(this._textOptions(this.model.captionSettings), this.model.captionSettings.text, captionGroup);
            if (this.model.captionSettings.subTitle.text != "") {                                    
                this.svgRenderer.drawText(this._textOptions(this.model.captionSettings.subTitle), this.model.captionSettings.subTitle.text, captionGroup);
            }
        },
        /**
         * To draw indicator for the graph	
		 * @private
         */
        _drawIndicator: function (captionGroup) {
            if (this.model.captionSettings.indicator.visible) {
                var indicatorSettings = this.model.captionSettings.indicator;
                var indicatorEventArgs = { indicatorSettings: this.model.captionSettings.indicator }
                if (this.model.drawIndicator) {
                    this._trigger("drawIndicator", indicatorEventArgs);
                    if (indicatorEventArgs.cancel == false)
                        indicatorSettings = $.extend({},indicatorEventArgs.indicatorSettings);
                }
                var textBounds = ej.EjSvgRender.utils._measureText(indicatorSettings.text, null, indicatorSettings.font);
                var symbolRect = {
                    'x': indicatorSettings.location.x + indicatorSettings.symbol.size.width / 2,
                    'y': indicatorSettings.location.y + +indicatorSettings.symbol.size.height / 2,
                    'width': indicatorSettings.symbol.size.width,
                    'height': indicatorSettings.symbol.size.height,
                };
                var symbolOptions = {
                    'id': captionGroup.id + '_indicatorSymbol',                    
                    'stroke': indicatorSettings.symbol.border.color,
                    'fill': indicatorSettings.symbol.color,
                    'stroke-width': indicatorSettings.symbol.border.width,
                    'opacity': indicatorSettings.symbol.opacity
                };
                var arrowPath = "M" + " " + ((symbolRect.x - (symbolRect.width / 2)) + symbolRect.width) + " " + (symbolRect.y + (symbolRect.height / 4)) + " "
                    + "L" + " " + ((symbolRect.x - (symbolRect.width / 2)) + symbolRect.width) + " " + (symbolRect.y + (-symbolRect.height / 4)) + " "
                    + "L" + " " + ((symbolRect.x - (symbolRect.width / 2)) + (symbolRect.width / 2)) + " " + (symbolRect.y + (-symbolRect.height / 4)) + " "
                    + "L" + " " + ((symbolRect.x - (symbolRect.width / 2)) + (symbolRect.width / 2)) + " " + (symbolRect.y + (-symbolRect.height / 2)) + " "
                    + "L" + " " + ((symbolRect.x - (symbolRect.width / 2))) + " " + (symbolRect.y) + " "
                    + "L" + " " + ((symbolRect.x - (symbolRect.width / 2)) + (symbolRect.width / 2)) + " " + (symbolRect.y + (symbolRect.height / 2)) + " "
                    + "L" + " " + ((symbolRect.x - (symbolRect.width / 2)) + (symbolRect.width / 2)) + " " + (symbolRect.y + (symbolRect.height / 4)) + " "
                    + "L" + " " + ((symbolRect.x - (symbolRect.width / 2)) + symbolRect.width) + " " + (symbolRect.y + (symbolRect.height / 4));

                switch (this.model.captionSettings.indicator.symbol.shape) {
                    case 'circle':
                        var radius = Math.min(symbolRect.height, symbolRect.width) / 2;
                        symbolOptions.cx = symbolRect.x;
                        symbolOptions.cy = symbolRect.y,
                        symbolOptions.r = radius,
                        this.svgRenderer.drawCircle(symbolOptions, captionGroup);
                        break;

                    case 'leftarrow':                        
                        symbolOptions.d = arrowPath;
                        this.svgRenderer.drawPath(symbolOptions, captionGroup);
                        break;
                    case 'rightarrow':
                        symbolOptions.d = arrowPath;
                        symbolOptions.transform = 'rotate(180,' + (symbolRect.x) + ',' + symbolRect.y + ')';
                        this.svgRenderer.drawPath(symbolOptions, captionGroup);
                        break;
                    case 'uparrow':
                        symbolOptions.d = arrowPath;
                        symbolOptions.transform = 'rotate(90,' + (symbolRect.x) + ',' + symbolRect.y + ')';
                        this.svgRenderer.drawPath(symbolOptions, captionGroup);
                        break;
                    case 'downarrow':
                        symbolOptions.d = arrowPath;
                        symbolOptions.transform = 'rotate(-90,' + (symbolRect.x) + ',' + symbolRect.y + ')';
                        this.svgRenderer.drawPath(symbolOptions, captionGroup);
                        break;
                    case 'cross':
                        var crossPath = "M" + " " + (symbolRect.x + (-symbolRect.width / 2)) + " " + (symbolRect.y) + " "
                            + "L" + " " + (symbolRect.x + (symbolRect.width / 2)) + " " + (symbolRect.y) + " " +
                            "M" + " " + (symbolRect.x) + " " + (symbolRect.y + (symbolRect.height / 2)) + " "
                            + "L" + " " + (symbolRect.x) + " " + (symbolRect.y + (-symbolRect.height / 2));
                        symbolOptions.d = crossPath;
                        symbolOptions.stroke = (symbolOptions.stroke) ? symbolOptions.stroke : symbolOptions.fill;
                        this.svgRenderer.drawPath(symbolOptions, captionGroup);
                        break;
                    case 'horizontalline':
                        var horizPath = "M" + " " + (symbolRect.x + (-symbolRect.width / 2)) + " " + (symbolRect.y) + " " + "L" + " " + (symbolRect.x + (symbolRect.width / 2)) + " " + (symbolRect.y);
                        symbolOptions.d = horizPath;
                        symbolOptions.stroke = (symbolOptions.stroke) ? symbolOptions.stroke : symbolOptions.fill;
                        this.svgRenderer.drawPath(symbolOptions, captionGroup);
                        break;
                    case 'verticalline':
                        var verPath = "M" + " " + (symbolRect.x) + " " + (symbolRect.y + (symbolRect.height / 2)) + " "
                            + "L" + " " + (symbolRect.x) + " " + (symbolRect.y + (-symbolRect.height / 2));
                        symbolOptions.d = verPath;
                        symbolOptions.stroke = (symbolOptions.stroke) ? symbolOptions.stroke : symbolOptions.fill;
                        this.svgRenderer.drawPath(symbolOptions, captionGroup);
                        break;
                    case 'triangle':
                        var triPath = "M" + " " + (symbolRect.x + (-symbolRect.width / 2)) + " " + (symbolRect.y + (symbolRect.height / 2)) + " "
                            + "L" + " " + (symbolRect.x) + " " + (symbolRect.y + (-symbolRect.height / 2)) + " "
                            + "L" + " " + (symbolRect.x + (symbolRect.width / 2)) + " " + (symbolRect.y + (symbolRect.height / 2)) + " "
                            + "L" + " " + (symbolRect.x + (-symbolRect.width / 2)) + " " + (symbolRect.y + (symbolRect.height / 2)) + " Z";
                        symbolOptions.d = triPath;
                        this.svgRenderer.drawPath(symbolOptions, captionGroup);
                        break;
                    case 'invertedtriangle':
                        var triPath = "M" + " " + (symbolRect.x + (-symbolRect.width / 2)) + " " + (symbolRect.y + (symbolRect.height / 2)) + " "
                            + "L" + " " + (symbolRect.x) + " " + (symbolRect.y + (-symbolRect.height / 2)) + " "
                            + "L" + " " + (symbolRect.x + (symbolRect.width / 2)) + " " + (symbolRect.y + (symbolRect.height / 2)) + " "
                            + "L" + " " + (symbolRect.x + (-symbolRect.width / 2)) + " " + (symbolRect.y + (symbolRect.height / 2)) + " Z";
                        symbolOptions.d = triPath;
                        symbolOptions.transform = 'rotate(180,' + (symbolRect.x) + ',' + symbolRect.y + ')';
                        this.svgRenderer.drawPath(symbolOptions, captionGroup);
                        break;
                    case 'hexagon':
                        var hexPath = "M" + " " + (symbolRect.x + (-symbolRect.width / 2)) + " " + (symbolRect.y) + " "
                            + "L" + " " + (symbolRect.x + (-symbolRect.width / 4)) + " " + (symbolRect.y + (-symbolRect.height / 2)) + " "
                            + "L" + " " + (symbolRect.x + (symbolRect.width / 4)) + " " + (symbolRect.y + (-symbolRect.height / 2)) + " "
                            + "L" + " " + (symbolRect.x + (symbolRect.width / 2)) + " " + (symbolRect.y) + " "
                            + "L" + " " + (symbolRect.x + (symbolRect.width / 4)) + " " + (symbolRect.y + (symbolRect.height / 2)) + " "
                            + "L" + " " + (symbolRect.x + (-symbolRect.width / 4)) + " " + (symbolRect.y + (symbolRect.height / 2)) + " "
                            + "L" + " " + (symbolRect.x + (-symbolRect.width / 2)) + " " + (symbolRect.y);
                        symbolOptions.d = hexPath;
                        this.svgRenderer.drawPath(symbolOptions, captionGroup);
                        break;
                    case 'wedge':
                        var wedgePath = "M" + " " + (symbolRect.x + (-symbolRect.width / 2)) + " " + (symbolRect.y) + " "
                            + "L" + " " + (symbolRect.x + symbolRect.width / 2) + " " + (symbolRect.y + (-symbolRect.height / 2)) + " "
                            + "L" + " " + (symbolRect.x + (symbolRect.width / 4)) + " " + (symbolRect.y) + " "
                            + "L" + " " + (symbolRect.x + (symbolRect.width / 2)) + " " + (symbolRect.y + (symbolRect.height / 2)) + " "
                            + "L" + " " + (symbolRect.x + (-symbolRect.width / 2)) + " " + (symbolRect.y);
                        symbolOptions.d = wedgePath;
                        this.svgRenderer.drawPath(symbolOptions, captionGroup);
                        break;
                    case 'pentagon':
                        var eq = 72;
                        var radius = Math.sqrt(symbolRect.height * symbolRect.height + symbolRect.width * symbolRect.width) / 2;
                        var sb = ej.EjSvgRender.utils._getStringBuilder();
                        for (var i = 0; i <= 5; i++) {
                            var deg = i * eq;
                            var rad = (Math.PI / 180) * deg;
                            var x1 = radius * Math.cos(rad);
                            var y1 = radius * Math.sin(rad);
                            if (i == 0)
                                sb.append("M" + " " + (symbolRect.x + x1) + " " + (symbolRect.y + y1) + " ");

                            else
                                sb.append("L" + " " + (symbolRect.x + x1) + " " + (symbolRect.y + y1) + " ");

                        }                        
                        symbolOptions.d = sb.toString();
                        this.svgRenderer.drawPath(symbolOptions, captionGroup);
                        break;
                    case 'star':
                        var starPath = "M" + " " + (symbolRect.x + (symbolRect.width / 3)) + " " + (symbolRect.y + (-symbolRect.height / 2)) + " "
                            + "L" + " " + (symbolRect.x + (-symbolRect.width / 2)) + " " + (symbolRect.y + (symbolRect.height / 6)) + " "
                            + "L" + " " + (symbolRect.x + (symbolRect.width / 2)) + " " + (symbolRect.y + (symbolRect.height / 6)) + " "
                            + "L" + " " + (symbolRect.x + (-symbolRect.width / 3)) + " " + (symbolRect.y + (-symbolRect.height / 2)) + " "
                            + "L" + " " + (symbolRect.x) + " " + (symbolRect.y + (symbolRect.height / 2)) + " "
                            + "L" + " " + (symbolRect.x + (symbolRect.width / 3)) + " " + (symbolRect.y + (-symbolRect.height / 2));
                        symbolOptions.d = starPath;
                        this.svgRenderer.drawPath(symbolOptions, captionGroup);
                        break;
                    case 'rectangle':
                        var rectPath = "M" + " " + (symbolRect.x + (-symbolRect.width / 2)) + " " + (symbolRect.y + (-symbolRect.height / 2)) + " "
                            + "L" + " " + (symbolRect.x + (symbolRect.width / 2)) + " " + (symbolRect.y + (-symbolRect.height / 2)) + " "
                            + "L" + " " + (symbolRect.x + (symbolRect.width / 2)) + " " + (symbolRect.y + (symbolRect.height / 2)) + " "
                            + "L" + " " + (symbolRect.x + (-symbolRect.width / 2)) + " " + (symbolRect.y + (symbolRect.height / 2)) + " "
                            + "L" + " " + (symbolRect.x + (-symbolRect.width / 2)) + " " + (symbolRect.y + (-symbolRect.height / 2));
                        symbolOptions.d = rectPath;
                        this.svgRenderer.drawPath(symbolOptions, captionGroup);
                        break;
                    case 'trapezoid':
                        var path = "M" + " " + (symbolRect.x + (-symbolRect.width / 2)) + " " + (symbolRect.y) + " "
                            + "L" + " " + (symbolRect.x + (-symbolRect.width / 2)) + " " + (symbolRect.y + (-symbolRect.height / 4)) + " "
                            + "L" + " " + (symbolRect.x + (-symbolRect.width / 2) + (symbolRect.width)) + " " + (symbolRect.y + (-symbolRect.height / 2)) + " "
                            + "L" + " " + (symbolRect.x + (-symbolRect.width / 2) + (symbolRect.width)) + " " + (symbolRect.y + (symbolRect.height / 2)) + " "
                            + "L" + " " + (symbolRect.x + (-symbolRect.width / 2)) + " " + (symbolRect.y + (symbolRect.height / 4)) + " "
                            + "L" + " " + (symbolRect.x + (-symbolRect.width / 2)) + " " + (symbolRect.y);
                        symbolOptions.d = path;
                        this.svgRenderer.drawPath(symbolOptions, captionGroup);
                        break;
                    case 'diamond':
                        var path = "M" + " " + (symbolRect.x + (-symbolRect.width / 2)) + " " + (symbolRect.y) + " "
                            + "L" + " " + (symbolRect.x) + " " + (symbolRect.y + (-symbolRect.height / 2)) + " "
                            + "L" + " " + (symbolRect.x + (symbolRect.width / 2)) + " " + (symbolRect.y) + " "
                            + "L" + " " + (symbolRect.x) + " " + (symbolRect.y + (symbolRect.height / 2)) + " "
                            + "L" + " " + (symbolRect.x + (-symbolRect.width / 2)) + " " + (symbolRect.y);
                        symbolOptions.d = path;
                        this.svgRenderer.drawPath(symbolOptions, captionGroup);
                        break;
                    case 'ellipse':                        
                        symbolOptions.cx = symbolRect.x;
                        symbolOptions.cy = symbolRect.y;
                        symbolOptions.rx = symbolRect.width/2;
                        symbolOptions.ry = symbolRect.height/2;
                        this.svgRenderer.drawEllipse(symbolOptions, captionGroup);
                        break;
                    case 'image':
                        symbolOptions.x = symbolRect.x - symbolRect.width / 2;
                        symbolOptions.y = symbolRect.y - symbolRect.height / 2;
                        symbolOptions.width = symbolRect.width;
                        symbolOptions.height = symbolRect.height;
                        symbolOptions.href = this.model.captionSettings.indicator.symbol.imageURL;
                        symbolOptions.visibility = 'visible';
                        this.svgRenderer.drawImage(symbolOptions, captionGroup);
                        break;
                    default:
                        break;
                }                
                    
                var textOptions = {
                    'x': indicatorSettings.location.x + symbolRect.width + textBounds.width + indicatorSettings.textSpacing,
                    'y': indicatorSettings.location.y + symbolRect.height / 2 + textBounds.height / 4,
                    'fill': indicatorSettings.font.color,
                    'font-size': indicatorSettings.font.size,
                    'font-family': indicatorSettings.font.fontFamily,
                    'font-style': indicatorSettings.font.fontStyle,
                    'font-weight': indicatorSettings.font.fontWeight,
                    'text-anchor': 'end',
                    'opacity': indicatorSettings.font.opacity,
                    'transform': 'rotate(' + indicatorSettings.textAngle + ',' + indicatorSettings.location.x + ',' + indicatorSettings.location.y + ')'
                };
                this.svgRenderer.drawText(textOptions, indicatorSettings.text, captionGroup);
            }
        },
        /**
         * To give the font option to the text	
		 * @private
         */
        _textOptions: function (text) {
            var y = (text.text === this.model.captionSettings.text) ? text.location.y : text.location.y - 15;
            var textOptions = {
                'x': text.location.x,
                'y': text.location.y,
                'fill': text.font.color,
                'font-size': text.font.size,
                'font-family': text.font.fontFamily,
                'font-style': text.font.fontStyle,
                'font-weight': text.font.fontWeight,
                'text-anchor': 'start',
                'opacity': text.font.opacity,
                'dominant-baseline': 'middle',
                'transform': 'rotate(' + text.textAngle + ',' + text.location.x + ',' + y + ')'
            };
            return textOptions;
        },
        /**
         * To draw the scales
		 * @private
         */
        _drawScale: function (captionGroup, scaleGroup) {
            var pointY, pointX;
            if (this._tickPosition == ej.datavisualization.BulletGraph.TickPosition.Below || this._tickPosition == ej.datavisualization.BulletGraph.TickPosition.Cross)
                pointY = (this._labelPosition == ej.datavisualization.BulletGraph.LabelPosition.Below) ? this._scale.location.y : this._scale.location.y + this._scale.labelSettings.offset + this._scale.labelSettings.size;
            else
                pointY = (this._labelPosition == ej.datavisualization.BulletGraph.LabelPosition.Below) ? this._scale.location.y + this._scale.majorTickSettings.size : this._scale.location.y + this._scale.labelSettings.offset + this._scale.labelSettings.size + this._scale.majorTickSettings.size;

            if (this._flowDirection == ej.datavisualization.BulletGraph.FlowDirection.Forward)
                pointX = this._scale.location.x;
            else
                pointX = (this._orientation == ej.datavisualization.BulletGraph.Orientation.Horizontal) ? this._scale.location.x - this._scale.labelSettings.offset : this._scale.location.x;

            var scaleOptions = {
                'id': this.svgObject.id + '_SvgScale',
                'x': pointX,
                'y': pointY,
                'width': this.model.quantitativeScaleLength,
                'height': this.model.qualitativeRangeSize,
                'fill': "transparent",
                'stroke-width': 0
            };
            this.svgRenderer.drawRect(scaleOptions, scaleGroup);
        },
        /**
         * To draw the major ticks
		 * @private
         */
        _drawMajorTicks: function (intervalValue, scaleGroup) {
            var majorPointX, y1, tickIndex = 0;
            if (this._flowDirection == ej.datavisualization.BulletGraph.FlowDirection.Forward)
                majorPointX = (this._orientation == ej.datavisualization.BulletGraph.Orientation.Horizontal) ? this._scale.location.x + (this._scale.majorTickSettings.width / 2) : this._scale.location.x + this.model.quantitativeScaleLength - this._scale.labelSettings.offset + (this._scale.majorTickSettings.width / 2);
            else
                majorPointX = (this._orientation == ej.datavisualization.BulletGraph.Orientation.Horizontal) ? this._scale.location.x + this.model.quantitativeScaleLength - this._scale.labelSettings.offset - (this._scale.majorTickSettings.width / 2) : this._scale.location.x + (this._scale.majorTickSettings.width / 2);
           
            if (this._tickPosition == ej.datavisualization.BulletGraph.TickPosition.Below)
                y1 = (this._labelPosition == ej.datavisualization.BulletGraph.LabelPosition.Below) ? this._scale.location.y + this.model.qualitativeRangeSize : this._scale.location.y + this.model.qualitativeRangeSize + this._scale.labelSettings.offset + this._scale.labelSettings.size;
            else if (this._tickPosition == ej.datavisualization.BulletGraph.TickPosition.Above)
                y1 = (this._labelPosition == ej.datavisualization.BulletGraph.LabelPosition.Below) ? this._scale.location.y : this._scale.location.y + this._scale.labelSettings.offset + this._scale.labelSettings.size;
            else
                y1 = (this._labelPosition == ej.datavisualization.BulletGraph.LabelPosition.Below) ? this._scale.location.y + ((this.model.qualitativeRangeSize) / 2) - (this._scale.majorTickSettings.size / 2) : this._scale.location.y + ((this.model.qualitativeRangeSize) / 2) - (this._scale.majorTickSettings.size / 2) + this._scale.labelSettings.offset + this._scale.labelSettings.size;

            for (var i = this._scale.minimum; i <= this._scale.maximum; i += this._scale.interval) {
                tickIndex++;
                if (i >= this._scale.maximum)
                    if (this._orientation == ej.datavisualization.BulletGraph.Orientation.Horizontal && this._flowDirection == ej.datavisualization.BulletGraph.FlowDirection.Backward)
                        majorPointX += this._scale.majorTickSettings.width;
                    else
                        majorPointX -= this._scale.majorTickSettings.width;
                if (this._orientation == ej.datavisualization.BulletGraph.Orientation.Vertical && this._flowDirection == ej.datavisualization.BulletGraph.FlowDirection.Forward)
                    majorPointX = (i >= this._scale.maximum) ? majorPointX + (this._scale.majorTickSettings.width) : (i == this._scale.minimum) ? majorPointX - (this._scale.majorTickSettings.width) : majorPointX;

                if (this.model.applyRangeStrokeToTicks)
                    strokeColor = this._bindingRangeStrokes(majorPointX - (this._scale.majorTickSettings.width / 2));

                var options = this._majorTickLines(this._scale, majorPointX, y1);

                if (this._orientation == ej.datavisualization.BulletGraph.Orientation.Vertical && this._flowDirection == ej.datavisualization.BulletGraph.FlowDirection.Forward && i == this._scale.minimum)
                    majorPointX = majorPointX + this._scale.majorTickSettings.width;
                
                if (this.model.drawTicks) {
                    data = { majorTickSettings: this._scale.majorTickSettings, minorTickSettings: this._scale.minorTickSettings, minorTicksPerInterval: this._scale.minorTicksPerInterval, maximum: this._scale.maximum, minimum: this._scale.minimum, interval: this._scale.interval };
                    this._trigger("drawTicks", data);                    
                    options = this._majorTickLines(data, majorPointX, y1);
                }
                var majorTicks = this.svgRenderer.createLine(options);
                majorPointX = ((this._flowDirection == ej.datavisualization.BulletGraph.FlowDirection.Backward && this._orientation == ej.datavisualization.BulletGraph.Orientation.Horizontal) || (this._flowDirection == ej.datavisualization.BulletGraph.FlowDirection.Forward && this._orientation == ej.datavisualization.BulletGraph.Orientation.Vertical)) ? majorPointX - intervalValue : majorPointX + intervalValue;
                scaleGroup.appendChild(majorTicks);
            }
        },
        _majorTickLines: function (data, majorPointX, y1) {
            var options = {
                'x1': majorPointX,
                'y1': y1,
                'x2': majorPointX,
                'y2': y1 + data.majorTickSettings.size,
                'stroke-width': data.majorTickSettings.width,
                'stroke': (this.model.applyRangeStrokeToTicks) ? strokeColor : data.majorTickSettings.stroke
            };
            return options;
        },
        /**
         * To bind the stroke in forward direction
		 * @private
         */
        _forwardStrokeBinding: function (majorPointX) {
            if (majorPointX >= this._scale.location.x && majorPointX <= (this._rangeCollection[0] + this._scale.location.x)) {
                return this.model.qualitativeRanges[0].rangeStroke;
            }
            for (var k = 0; k <= this._rangeCollection.length - 1; k++) {
                if (majorPointX >= (this._rangeCollection[k] + this._scale.location.x) && majorPointX <= (this._rangeCollection[k + 1] + this._scale.location.x))
                    return this.model.qualitativeRanges[k + 1].rangeStroke;
            }
        },
        /**
         * To bind the stroke in backward direction
		 * @private
         */
        _backwardStrokeBinding: function (majorPointX) {
            if (majorPointX >= this._rangeCollection[this._rangeCollection.length - 1]) {
                return this.model.qualitativeRanges[0].rangeStroke;
            }
            for (k = 0; k <= this._rangeCollection.length - 1; k++) {
                if (majorPointX >= this._rangeCollection[k] && majorPointX < this._rangeCollection[k + 1]) {
                    var index = $.inArray(this._rangeCollection[k], this._rangeCollection)
                    return this.model.qualitativeRanges[(this._rangeCollection.length - 1) - index].rangeStroke;
                }
            }
        },
        /**
         * To bind the range stroke
		 * @private
         */
        _bindingRangeStrokes: function (majorPointX) {
            if (this._rangeCollection.length == 1) {
                if (majorPointX >= this._scale.location.x && majorPointX <= (this._rangeCollection[0] + this._scale.location.x))
                    return this.model.qualitativeRanges[0].rangeStroke;
            }
            else {
                if ((this.model.orientation == ej.datavisualization.BulletGraph.Orientation.Horizontal && this._flowDirection == ej.datavisualization.BulletGraph.FlowDirection.Forward) || (this.model.orientation == ej.datavisualization.BulletGraph.Orientation.Vertical && this._flowDirection == ej.datavisualization.BulletGraph.FlowDirection.Backward)) {
                    return this._forwardStrokeBinding(majorPointX);
                }
                else {
                    return this._backwardStrokeBinding(majorPointX);
                }
            }
        },
        /**
         * To draw minor ticks
		 * @private
         */
        _drawMinorTicks: function (intervalValue, scaleGroup) {
            var majorPointX, minorPointX, y1, y2, x, tickIndex = 0;

            if (this._flowDirection == ej.datavisualization.BulletGraph.FlowDirection.Forward)
                majorPointX = (this._orientation == ej.datavisualization.BulletGraph.Orientation.Horizontal) ? this._scale.location.x : this._scale.location.x - this._scale.labelSettings.offset  + this.model.quantitativeScaleLength;
            else
                majorPointX = (this._orientation == ej.datavisualization.BulletGraph.Orientation.Horizontal) ? this._scale.location.x + this.model.quantitativeScaleLength - this._scale.labelSettings.offset - (this._scale.minorTickSettings.width / 2) : this._scale.location.x;

            if (this._tickPosition == ej.datavisualization.BulletGraph.TickPosition.Below) {
                y1 = (this._labelPosition == ej.datavisualization.BulletGraph.LabelPosition.Below) ? this._scale.location.y + this.model.qualitativeRangeSize : this._scale.location.y + this.model.qualitativeRangeSize + this._scale.labelSettings.offset + this._scale.labelSettings.size;
                y2 = y1 + this._scale.minorTickSettings.size;
            }
            else if (this._tickPosition == ej.datavisualization.BulletGraph.TickPosition.Above) {
                y1 = (this._labelPosition == ej.datavisualization.BulletGraph.LabelPosition.Below) ? this._scale.location.y + this._scale.minorTickSettings.size : this._scale.location.y + this._scale.labelSettings.offset + this._scale.labelSettings.size + this._scale.minorTickSettings.size;
                y2 = (this._labelPosition == ej.datavisualization.BulletGraph.LabelPosition.Below) ? this._scale.location.y + this._scale.majorTickSettings.size : this._scale.location.y + this._scale.labelSettings.offset + this._scale.labelSettings.size + this._scale.majorTickSettings.size;
            }
            else {
                y1 = (this._labelPosition == ej.datavisualization.BulletGraph.LabelPosition.Below) ? this._scale.location.y + ((this.model.qualitativeRangeSize) / 2) - (this._scale.minorTickSettings.size / 2) : this._scale.location.y + ((this.model.qualitativeRangeSize) / 2) - (this._scale.minorTickSettings.size / 2) + this._scale.labelSettings.offset + this._scale.labelSettings.size;
                y2 = y1 + this._scale.minorTickSettings.size;
            }
            
            var maxLimit = this._scale.maximum;
           
            for (var i = this._scale.minimum; i <= maxLimit; i += this._scale.interval) {
                minorPointX = intervalValue / this._scale.minorTicksPerInterval;

                for (var j = 1; j <= this._scale.minorTicksPerInterval; j++) {
                    tickIndex++;
                    x = ((this._orientation == ej.datavisualization.BulletGraph.Orientation.Vertical && this._flowDirection == ej.datavisualization.BulletGraph.FlowDirection.Forward) || (this._orientation == ej.datavisualization.BulletGraph.Orientation.Horizontal && this._flowDirection == ej.datavisualization.BulletGraph.FlowDirection.Backward))
                        ? majorPointX - minorPointX + (minorPointX / (this._scale.minorTicksPerInterval + 1))
                        : majorPointX + minorPointX - (minorPointX / (this._scale.minorTicksPerInterval + 1));

                    if (this._flowDirection == ej.datavisualization.BulletGraph.FlowDirection.Backward && this._orientation == ej.datavisualization.BulletGraph.Orientation.Horizontal) {
                        if (tickIndex >= ((this._scale.maximum - this._scale.minimum) * this._scale.minorTicksPerInterval))
                            x += (this._scale.minorTickSettings.width / 2);
                    }

                    if (this.model.applyRangeStrokeToTicks)
                        strokeColor = this._bindingRangeStrokes(x);
                   
                    var options = this._minorTickLines(this._scale, x, y1, y2);
                    if (this.model.drawTicks) {
                        data = { majorTickSettings: this._scale.majorTickSettings, minorTickSettings: this._scale.minorTickSettings, minorTicksPerInterval: this._scale.minorTicksPerInterval, maximum: this._scale.maximum, minimum: this._scale.minimum, interval: this._scale.interval };
                        this._trigger("drawTicks", data);
                        this._scale = $.extend(this._scale, data);
                        options = this._minorTickLines(data, x, y1, y2);
                    }
                   
                    var chkMax;
                    if (this._flowDirection == ej.datavisualization.BulletGraph.FlowDirection.Backward && this._orientation == ej.datavisualization.BulletGraph.Orientation.Horizontal) {
                        chkMax = (this.model.quantitativeScaleLength + this._scale.location.x - this._scale.labelSettings.offset + 1);                       
                        if (x <= chkMax && x >= this._scale.location.x - this._scale.labelSettings.offset + 1) {
                            var minorTicks = this.svgRenderer.createLine(options);
                            scaleGroup.appendChild(minorTicks);
                        }
                    }
                    else if (this._flowDirection == ej.datavisualization.BulletGraph.FlowDirection.Forward && this._orientation == ej.datavisualization.BulletGraph.Orientation.Vertical) {
                        chkMax = (this._scale.location.x - this._scale.labelSettings.offset + 1);
                        if (x >= chkMax) {
                            var minorTicks = this.svgRenderer.createLine(options);
                            scaleGroup.appendChild(minorTicks);
                        }
                    }
                    else {
                        chkMax = this.model.quantitativeScaleLength + this._scale.location.x + 1;
                        if (x <= chkMax) {
                            var minorTicks = this.svgRenderer.createLine(options);
                            scaleGroup.appendChild(minorTicks);
                        }
                    }
                    
                    minorPointX = (intervalValue / this._scale.minorTicksPerInterval) * (j + 1);
                }
                majorPointX = ((this._flowDirection == ej.datavisualization.BulletGraph.FlowDirection.Backward && this._orientation == ej.datavisualization.BulletGraph.Orientation.Horizontal) || (this._flowDirection == ej.datavisualization.BulletGraph.FlowDirection.Forward && this._orientation == ej.datavisualization.BulletGraph.Orientation.Vertical)) ? majorPointX - intervalValue : majorPointX + intervalValue;
            }
        },
        _minorTickLines: function (data, x, y1, y2) {
            var options = {
                'x1': x,
                'y1': y1,
                'x2': x,
                'y2': y2,
                'stroke-width': data.minorTickSettings.width,
                'stroke': (this.model.applyRangeStrokeToTicks) ? strokeColor : data.minorTickSettings.stroke
            };
            return options;
        },
        /**
         * To Draw labels
		 * @private
         */
        _drawLabels: function (intervalValue, scaleGroup) {
            var labelX, pointY;
            if (this._flowDirection == ej.datavisualization.BulletGraph.FlowDirection.Forward)
                labelX = (this._orientation == ej.datavisualization.BulletGraph.Orientation.Horizontal) ? this._scale.location.x : this._scale.location.x + this.model.quantitativeScaleLength - this._scale.labelSettings.offset - (this._scale.majorTickSettings.width / 2);
            else
                labelX = (this._orientation == ej.datavisualization.BulletGraph.Orientation.Horizontal) ? this._scale.location.x + this.model.quantitativeScaleLength - this._scale.labelSettings.offset - this._scale.majorTickSettings.width : this._scale.location.x - (this._scale.majorTickSettings.width / 2);
            
            if (this._tickPosition == ej.datavisualization.BulletGraph.TickPosition.Below || this._tickPosition == ej.datavisualization.BulletGraph.TickPosition.Cross)
                pointY = (this._labelPosition == ej.datavisualization.BulletGraph.LabelPosition.Below) ? this._scale.location.y + this.model.qualitativeRangeSize + this._scale.majorTickSettings.size + this._scale.labelSettings.offset : this._scale.location.y + this._scale.labelSettings.size;
            else
                pointY = (this._labelPosition == ej.datavisualization.BulletGraph.LabelPosition.Below) ? this._scale.location.y + this.model.qualitativeRangeSize + this._scale.labelSettings.offset + this._scale.majorTickSettings.size : this._scale.location.y + this._scale.labelSettings.size;

            for (var i = this._scale.minimum; i <= this._scale.maximum; i += this._scale.interval) {
                if (this._orientation == ej.datavisualization.BulletGraph.Orientation.Vertical)
                    transformText = 'rotate(' + 90 + ',' + (labelX - 4) + ',' + pointY + ')';
                else
                    transformText = 'rotate(' + 0 + ',' + labelX + ',' + pointY + ')';
                labelX = (i == this._scale.maximum) ? labelX - (this._scale.majorTickSettings.width) : labelX;
                if (this._orientation == ej.datavisualization.BulletGraph.Orientation.Vertical)
                    labelX = (this._flowDirection == ej.datavisualization.BulletGraph.FlowDirection.Forward) ? (i == this._scale.maximum || i == this._scale.minimum) ? labelX + (this._scale.majorTickSettings.width) : labelX : (i == this._scale.minimum) ? labelX + (this._scale.majorTickSettings.width / 2) : labelX;

                if (this.model.applyRangeStrokeToLabels)
                    strokeColor = this._bindingRangeStrokes(labelX);

                var labelOptions = this._labelOptions(this.model.quantitativeScaleSettings.labelSettings, labelX, pointY);

                if (this.model.drawLabels) {
                    var data = { font: this._scale.labelSettings.font, labelPrefix: this._scale.labelSettings.labelPrefix, labelSuffix: this._scale.labelSettings.labelSuffix, offset: this._scale.labelSettings.offset, size: this._scale.labelSettings.size, stroke: this._scale.labelSettings.stroke };
                    this._trigger("drawLabels", data);
                    labelOptions = this._labelOptions(data, labelX, pointY);
                }

                    if (this._flowDirection == ej.datavisualization.BulletGraph.FlowDirection.Forward)
                        labelX = (this._orientation == ej.datavisualization.BulletGraph.Orientation.Horizontal) ? labelX + intervalValue : labelX - intervalValue;
                    else
                        labelX = (this._orientation == ej.datavisualization.BulletGraph.Orientation.Horizontal) ? labelX - intervalValue : labelX + intervalValue;
                                                              
                this.svgRenderer.drawText(labelOptions, labelOptions.labelPrefix + i + labelOptions.labelSuffix, scaleGroup);
            }
        },

        _labelOptions: function (data, labelX, pointY) {
            var labelOptions = {
                'x': labelX,
                'y': pointY,
                'text-anchor': 'middle',
                'fill': (this.model.applyRangeStrokeToLabels) ? strokeColor : data.stroke,
                'font-size': data.size + "px",
                'font-family': data.font.fontFamily,
                'font-style': data.font.fontStyle,
                'font-weight': data.font.fontWeight,
                'opacity': data.font.opacity,
                'transform': transformText,
                'labelPrefix': data.labelPrefix,
                'labelSuffix': data.labelSuffix,
            };
            return labelOptions;
        },
        /**
         * To qualititative ranges
		 * @private
         */
        _drawQualitativeRanges: function (scaleGroup) {
            var pointX, pointY, width;
            this._rangeCollection = [];
            if (this._tickPosition == ej.datavisualization.BulletGraph.TickPosition.Below || this._tickPosition == ej.datavisualization.BulletGraph.TickPosition.Cross)
                pointY = (this._labelPosition == ej.datavisualization.BulletGraph.LabelPosition.Below) ? this._scale.location.y : this._scale.location.y + this._scale.labelSettings.offset + this._scale.labelSettings.size;
            else
                pointY = (this._labelPosition == ej.datavisualization.BulletGraph.LabelPosition.Below) ? this._scale.location.y + this._scale.majorTickSettings.size : this._scale.location.y + this._scale.labelSettings.offset + this._scale.labelSettings.size + this._scale.majorTickSettings.size;

            for (var i = this.model.qualitativeRanges.length - 1; i >= 0; i--) {
                var rangeEnd = this.model.qualitativeRanges[i].rangeEnd;
                rangeEnd = (rangeEnd > this._scale.maximum) ? this._scale.maximum : rangeEnd;
                if (this._flowDirection == ej.datavisualization.BulletGraph.FlowDirection.Forward) {
                    pointX = (this._orientation == ej.datavisualization.BulletGraph.Orientation.Horizontal) ? this._scale.location.x : this._scale.location.x - this._scale.labelSettings.offset + (this.model.quantitativeScaleLength - this.model.quantitativeScaleLength / ((this._scale.maximum - this._scale.minimum) / ((this._scale.maximum - this._scale.minimum) - (this._scale.maximum - rangeEnd))));
                    width = (this._orientation == ej.datavisualization.BulletGraph.Orientation.Horizontal) ? this.model.quantitativeScaleLength / ((this._scale.maximum - this._scale.minimum) / ((this._scale.maximum - this._scale.minimum) - (this._scale.maximum - rangeEnd))) : this.model.quantitativeScaleLength / ((this._scale.maximum - this._scale.minimum) / ((this._scale.maximum - this._scale.minimum) - (this._scale.maximum - rangeEnd)));
                }
                else {
                    pointX = (this._orientation == ej.datavisualization.BulletGraph.Orientation.Horizontal) ? this._scale.location.x - this._scale.labelSettings.offset + (this.model.quantitativeScaleLength - this.model.quantitativeScaleLength / ((this._scale.maximum - this._scale.minimum) / ((this._scale.maximum - this._scale.minimum) - (this._scale.maximum - rangeEnd)))) : this._scale.location.x;
                    width = (this._orientation == ej.datavisualization.BulletGraph.Orientation.Horizontal) ? this.model.quantitativeScaleLength / ((this._scale.maximum - this._scale.minimum) / ((this._scale.maximum - this._scale.minimum) - (this._scale.maximum - rangeEnd))) : this.model.quantitativeScaleLength / ((this._scale.maximum - this._scale.minimum) / ((this._scale.maximum - this._scale.minimum) - (this._scale.maximum - rangeEnd)));
                }
                var rangeOptions = {
                    'x': pointX,
                    'y': pointY,
                    'height': this.model.qualitativeRangeSize,
                    'width': (width > 0) ? (width < this.model.quantitativeScaleLength ? width : this.model.quantitativeScaleLength ) : 0,
                    'fill': this.model.qualitativeRanges[i].rangeStroke,
                    'opacity': this.model.qualitativeRanges[i].rangeOpacity
                };
                ((this.model.orientation == ej.datavisualization.BulletGraph.Orientation.Horizontal && this._flowDirection == ej.datavisualization.BulletGraph.FlowDirection.Forward) || (this.model.orientation == ej.datavisualization.BulletGraph.Orientation.Vertical && this._flowDirection == ej.datavisualization.BulletGraph.FlowDirection.Backward)) ? this._rangeCollection.push(width) : this._rangeCollection.push(pointX);
                if (this.model.drawQualitativeRanges)
                    this._onDrawQualitativeRanges(rangeOptions, this.model.qualitativeRanges[i].rangeEnd, i);
                this.svgRenderer.drawRect(rangeOptions, scaleGroup);
            }
            this._rangeCollection.sort(this._sortRangeCollection);
        },
        /**
         * To return the range value
		 * @private
         */
        _sortRangeCollection: function (a, b) {
            return (a - b);
        },
        /**
         * To calculate the bounds for Feature Measure Bar
		 * @private
         */
        _calculateFeatureMeasureBounds: function (value, categoryValue) {
            var min = this._scale.minimum;
            value = (value < min && min < 0) ? min : value;                
            if (value >= min) {
                var x, lx, width;
                var locX = this._scale.location.x;
                var scaleLength = this.model.quantitativeScaleLength;
                var diff = this._scale.maximum - this._scale.minimum;
                var valueDiff = this._scale.maximum - value;
                var orientation = this._flowDirection.toLowerCase() + this._orientation.toLowerCase();
                categoryValue = (categoryValue == null) ? "" : categoryValue;
                var font = this._scale.labelSettings.font;
                font.size = this._scale.labelSettings.size;
                var stringLength = ej.EjSvgRender.utils._measureText(categoryValue.toString(), null, font).width;

                switch (orientation) {
                    case 'forwardhorizontal':
                    case 'backwardvertical':                        
                            x = locX + ((min > 0) ? 0 : scaleLength / diff * Math.abs(min));
                            width = scaleLength / (diff / ( (min > 0) ? diff - valueDiff : value));                            
                            if (value < 0) {
                                width = Math.abs(width);
                                x -= width;
                            }
                            width = (x + width < locX + scaleLength) ? width : locX + scaleLength - x;
                            lx = locX - ((orientation == 'forwardhorizontal') ? this._scale.labelSettings.offset + stringLength / 2 : this._scale.labelSettings.offset);
                            break;
                    default:
                        x = locX - this._scale.labelSettings.offset + (scaleLength - scaleLength / (diff / (diff - valueDiff)));
                        width = (min > 0) ? scaleLength / (diff / (diff - valueDiff)) : scaleLength / (diff / (value));
                        if (value < 0) {
                            width = Math.abs(width);
                            x -= width;
                        }
                        if (x < locX - this._scale.labelSettings.offset) {
                            width = x + width - locX + this._scale.labelSettings.offset;
                            x = locX - this._scale.labelSettings.offset;
                        }                        
                        lx = locX + scaleLength + ((orientation == 'backwardhorizontal') ? stringLength / 2 : 0);
                        break;
                }
                return { pointX: x, Width: width, lPointX: lx };
            }
            return false;
        },
        /**
         * To draw Feature Measure Bar
		 * @private
         */
        _drawFeatureMeasureBar: function () {
            var j = 1, count, value, categoryValue;
            var pointY, lPointY;
            if (this._dataCount > 0) {
                for (var i = this._dataCount - 1; i >= 0; i--) {
                    value = (!ej.isNullOrUndefined(this.model.fields) && this.model.fields["dataSource"] != null) ? this.model.fields["dataSource"][i][this.model.fields.featureMeasures] : this._scale.featureMeasures[i].value;
                    categoryValue = (!ej.isNullOrUndefined(this.model.fields) && this.model.fields["dataSource"] != null) ? this.model.fields["dataSource"][i][this.model.fields.category] : this._scale.featureMeasures[i].category;

                    if (this._tickPosition == ej.datavisualization.BulletGraph.TickPosition.Below || this._tickPosition == ej.datavisualization.BulletGraph.TickPosition.Cross) {
                        pointY = (this._labelPosition == ej.datavisualization.BulletGraph.LabelPosition.Below) ? this._scale.location.y + (((this.model.qualitativeRangeSize / this._dataCount) * j) / 2) - (this._scale.featuredMeasureSettings.width / 2) : this._scale.location.y + (((this.model.qualitativeRangeSize / this._dataCount) * j) / 2) - (this._scale.featuredMeasureSettings.width / 2) + this._scale.labelSettings.offset + this._scale.labelSettings.size;
                        lPointY = (this._labelPosition == ej.datavisualization.BulletGraph.LabelPosition.Below) ? this._scale.location.y + (((this.model.qualitativeRangeSize / this._dataCount) * j) / 2) + (this._scale.featuredMeasureSettings.width / 2) - 1 : this._scale.location.y + (((this.model.qualitativeRangeSize / this._dataCount) * j) / 2) + (this._scale.featuredMeasureSettings.width / 2) - 1 + this._scale.labelSettings.offset + this._scale.labelSettings.size;
                    }
                    else {
                        pointY = (this._labelPosition == ej.datavisualization.BulletGraph.LabelPosition.Below) ? this._scale.location.y + (((this.model.qualitativeRangeSize / this._dataCount) * j) / 2) - (this._scale.featuredMeasureSettings.width / 2) + this._scale.majorTickSettings.size : this._scale.location.y + (((this.model.qualitativeRangeSize / this._dataCount) * j) / 2) - (this._scale.featuredMeasureSettings.width / 2) + this._scale.majorTickSettings.size + this._scale.labelSettings.offset + this._scale.labelSettings.size;
                        lPointY = (this._labelPosition == ej.datavisualization.BulletGraph.LabelPosition.Below) ? this._scale.location.y + (((this.model.qualitativeRangeSize / this._dataCount) * j) / 2) + (this._scale.featuredMeasureSettings.width / 2) - 1 + this._scale.majorTickSettings.size : this._scale.location.y + (((this.model.qualitativeRangeSize / this._dataCount) * j) / 2) + (this._scale.featuredMeasureSettings.width / 2) - 1 + this._scale.majorTickSettings.size + this._scale.labelSettings.offset + this._scale.labelSettings.size;
                    }
                    var bounds = this._calculateFeatureMeasureBounds(value, categoryValue);
                    if (bounds) {

                        var featureBarOptions = this._featureBar(this._scale, bounds.pointX, pointY, bounds.Width, i);
                        if (this.model.drawFeatureMeasureBar) {
                            var data = { featuredMeasureSettings: this._scale.featuredMeasureSettings };
                            this._trigger("drawFeatureMeasureBar", data);
                            featureBarOptions = this._featureBar(data, bounds.pointX, pointY, bounds.Width, i);
                        }
                        this.svgRenderer.drawRect(featureBarOptions, this._scaleGroup);

                        if (this._orientation == ej.datavisualization.BulletGraph.Orientation.Vertical)
                            transformText = 'rotate(' + 90 + ',' + bounds.lPointX + ',' + (lPointY - 4) + ')';
                        else
                            transformText = 'rotate(' + 0 + ',' + bounds.lPointX + ',' + lPointY + ')';


                        var categoryOptions = this._drawcategory(this.model.quantitativeScaleSettings.labelSettings, bounds.lPointX, lPointY);
                        if (this.model.drawCategory) {
                            var data = { size: this.model.quantitativeScaleSettings.labelSettings.size, stroke: this.model.quantitativeScaleSettings.labelSettings.stroke, font: this.model.quantitativeScaleSettings.labelSettings.font, categoryValue: categoryValue };
                            this._trigger("drawCategory", data);
                            categoryOptions = this._drawcategory(data, bounds.lPointX, lPointY);
                            categoryValue = data.categoryValue;
                        }
                        if (!ej.isNullOrUndefined(categoryValue))
                            this.svgRenderer.drawText(categoryOptions, categoryValue, this._scaleGroup);
                        j += 2;
                        this.value(value);
                    }
                }

            }
        },
        _drawcategory: function (category, lPointX, lPointY) {
            var categoryOptions = {
                'x': lPointX,
                'y': lPointY + (this._scale.featuredMeasureSettings.width / 2),
                'text-anchor': 'middle',
                'fill': category.stroke,
                'font-size': category.size + "px",
                'font-family': category.font.fontFamily,
                'font-style': category.font.fontStyle,
                'font-weight': category.font.fontWeight,
                'opacity': category.font.opacity,
                'transform': transformText
            };
            return categoryOptions;
        },
        _featureBar: function (data, pointX, pointY, width, i) {
            var featureBarOptions = {
                'class': this.svgObject.id + "_FeatureMeasure",
                'id': this.svgObject.id + "_FeatureMeasure_" + i,
                'x': pointX,
                'y': pointY,
                'height': data.featuredMeasureSettings.width,
                'width': width,
                'fill': data.featuredMeasureSettings.stroke,
                'onmouseover': "evt.target.setAttribute('opacity', '0.7')",
                'onmouseout': "evt.target.setAttribute('opacity', '1')"
            };
            return featureBarOptions;
        },
        /**
         * To draw Comparitive Measure symbol
		 * @private
         */
        _drawComparativeMeasureSymbol: function () {
            var j = 1, value;
            var y1, y2, x1;
            if (this._dataCount > 0) {
                for (var i = this._dataCount - 1; i >= 0; i--) {
                    value = (!ej.isNullOrUndefined(this.model.fields) && this.model.fields["dataSource"] != null) ? this.model.fields["dataSource"][i][this.model.fields.comparativeMeasure] : this._scale.featureMeasures[i].comparativeMeasureValue;
                    if (value >= this._scale.minimum && value <= this._scale.maximum) {
                        if (this._tickPosition == ej.datavisualization.BulletGraph.TickPosition.Below || this._tickPosition == ej.datavisualization.BulletGraph.TickPosition.Cross) {
                            y1 = (this._labelPosition == ej.datavisualization.BulletGraph.LabelPosition.Below) ? this._scale.location.y + (((this.model.qualitativeRangeSize / this._dataCount) * j) / 2) - (this._scale.featuredMeasureSettings.width / 2) - this._scale.featuredMeasureSettings.width : this._scale.location.y + (((this.model.qualitativeRangeSize / this._dataCount) * j) / 2) - (this._scale.featuredMeasureSettings.width / 2) - this._scale.featuredMeasureSettings.width + this._scale.labelSettings.offset + this._scale.labelSettings.size;
                            y2 = (this._labelPosition == ej.datavisualization.BulletGraph.LabelPosition.Below) ? this._scale.location.y + (((this.model.qualitativeRangeSize / this._dataCount) * j) / 2) - (this._scale.featuredMeasureSettings.width / 2) + (2 * this._scale.featuredMeasureSettings.width) : this._scale.location.y + (((this.model.qualitativeRangeSize / this._dataCount) * j) / 2) - (this._scale.featuredMeasureSettings.width / 2) + (2 * this._scale.featuredMeasureSettings.width) + this._scale.labelSettings.offset + this._scale.labelSettings.size;
                        }
                        else {
                            y1 = (this._labelPosition == ej.datavisualization.BulletGraph.LabelPosition.Below) ? this._scale.location.y + (((this.model.qualitativeRangeSize / this._dataCount) * j) / 2) - (this._scale.featuredMeasureSettings.width / 2) - this._scale.featuredMeasureSettings.width + this._scale.majorTickSettings.size : this._scale.location.y + (((this.model.qualitativeRangeSize / this._dataCount) * j) / 2) - (this._scale.featuredMeasureSettings.width / 2) - this._scale.featuredMeasureSettings.width + this._scale.majorTickSettings.size + this._scale.labelSettings.offset + this._scale.labelSettings.size;
                            y2 = (this._labelPosition == ej.datavisualization.BulletGraph.LabelPosition.Below) ? this._scale.location.y + (((this.model.qualitativeRangeSize / this._dataCount) * j) / 2) - (this._scale.featuredMeasureSettings.width / 2) + (2 * this._scale.featuredMeasureSettings.width) + this._scale.majorTickSettings.size : this._scale.location.y + (((this.model.qualitativeRangeSize / this._dataCount) * j) / 2) - (this._scale.featuredMeasureSettings.width / 2) + (2 * this._scale.featuredMeasureSettings.width) + this._scale.majorTickSettings.size + this._scale.labelSettings.offset + this._scale.labelSettings.size;
                        }

                        if (this._flowDirection == ej.datavisualization.BulletGraph.FlowDirection.Forward)
                            x1 = (this._orientation == ej.datavisualization.BulletGraph.Orientation.Horizontal) ? this._scale.location.x + (this.model.quantitativeScaleLength / ((this._scale.maximum - this._scale.minimum) / ((this._scale.maximum - this._scale.minimum) - (this._scale.maximum - value)))) : this._scale.location.x - this._scale.labelSettings.offset + (this.model.quantitativeScaleLength - (this.model.quantitativeScaleLength / ((this._scale.maximum - this._scale.minimum) / ((this._scale.maximum - this._scale.minimum) - (this._scale.maximum - value)))));
                        else
                            x1 = (this._orientation == ej.datavisualization.BulletGraph.Orientation.Horizontal) ? this._scale.location.x - this._scale.labelSettings.offset + (this.model.quantitativeScaleLength - (this.model.quantitativeScaleLength / ((this._scale.maximum - this._scale.minimum) / ((this._scale.maximum - this._scale.minimum) - (this._scale.maximum - value))))) : this._scale.location.x + (this.model.quantitativeScaleLength / ((this._scale.maximum - this._scale.minimum) / ((this._scale.maximum - this._scale.minimum) - (this._scale.maximum - value))));

                        var compareMeasureOptions = this._compareMeasure(this._scale, x1, y1, y2, i, value);
                        if (this.model.drawComparativeMeasureSymbol) {
                            var data = { comparativeMeasureSettings: this._scale.comparativeMeasureSettings };
                            this._trigger("drawComparativeMeasureSymbol", data);
                            compareMeasureOptions = this._compareMeasure(data, x1, y1, y2, i, value);
                        }
                       
                        this.svgRenderer.drawLine(compareMeasureOptions, this._scaleGroup);
                        j += 2;
                        this.comparativeMeasureValue(value);
                    }
                }
            }
        },
        _compareMeasure: function(data, x1, y1, y2, i, value){
            var compareMeasureOptions = {
                'class': this.svgObject.id + "_ComparativeMeasure",
                'id': this.svgObject.id + "_ComparativeMeasure_" + i,
                'x1': (value == this._scale.maximum) ? x1 - (data.comparativeMeasureSettings.width / 2) : (value == this._scale.minimum) ? x1 + (data.comparativeMeasureSettings.width / 2) : x1,
                'y1': y1,
                'x2': (value == this._scale.maximum) ? x1 - (data.comparativeMeasureSettings.width / 2) : (value == this._scale.minimum) ? x1 + (data.comparativeMeasureSettings.width / 2) : x1,
                'y2': y2,
                'stroke-width': data.comparativeMeasureSettings.width,
                'stroke': data.comparativeMeasureSettings.stroke
            };
            return compareMeasureOptions;
        },
        /**
         * To set the theme for the bullet graph
		 * @private
         */
        _setTheme: function (jsonObj, themeName) {
            var result = [];
            this._scale = this.model.quantitativeScaleSettings;
            for (var name in jsonObj) {
                result.push(name);
            }
            for (var i = 0; i < result.length; i++) {
                this._scale.majorTickSettings.stroke = ((!this._scale.majorTickSettings.stroke || this._scale.majorTickSettings.stroke == jsonObj[result[i]].quantitativeScaleSettings.majorTickSettings.stroke) ? jsonObj[themeName].quantitativeScaleSettings.majorTickSettings.stroke : this._scale.majorTickSettings.stroke);
                this._scale.minorTickSettings.stroke = ((!this._scale.minorTickSettings.stroke || this._scale.minorTickSettings.stroke == jsonObj[result[i]].quantitativeScaleSettings.minorTickSettings.stroke) ? jsonObj[themeName].quantitativeScaleSettings.minorTickSettings.stroke : this._scale.minorTickSettings.stroke);
                this._scale.labelSettings.stroke = ((!this._scale.labelSettings.stroke || this._scale.labelSettings.stroke == jsonObj[result[i]].quantitativeScaleSettings.labelSettings.stroke) ? jsonObj[themeName].quantitativeScaleSettings.labelSettings.stroke : this._scale.labelSettings.stroke);
                this._scale.featuredMeasureSettings.stroke = ((!this._scale.featuredMeasureSettings.stroke || this._scale.featuredMeasureSettings.stroke == jsonObj[result[i]].quantitativeScaleSettings.featuredMeasureSettings.stroke) ? jsonObj[themeName].quantitativeScaleSettings.featuredMeasureSettings.stroke : this._scale.featuredMeasureSettings.stroke);
                this._scale.comparativeMeasureSettings.stroke = ((!this._scale.comparativeMeasureSettings.stroke || this._scale.comparativeMeasureSettings.stroke == jsonObj[result[i]].quantitativeScaleSettings.comparativeMeasureSettings.stroke) ? jsonObj[themeName].quantitativeScaleSettings.comparativeMeasureSettings.stroke : this._scale.comparativeMeasureSettings.stroke);
                this.model.captionSettings.font.color = ((!this.model.captionSettings.font.color || this.model.captionSettings.font.color == jsonObj[result[i]].captionSettings.font.color) ? jsonObj[themeName].captionSettings.font.color : this.model.captionSettings.font.color);
                this.model.captionSettings.subTitle.font.color = ((!this.model.captionSettings.subTitle.font.color || this.model.captionSettings.subTitle.font.color == jsonObj[result[i]].captionSettings.subTitle.font.color) ? jsonObj[themeName].captionSettings.subTitle.font.color : this.model.captionSettings.subTitle.font.color);
                for (var j = 0; j < this.model.qualitativeRanges.length; j++)
                    this.model.qualitativeRanges[j].rangeStroke = ((!this.model.qualitativeRanges[j].rangeStroke) ? (ej.isNullOrUndefined(jsonObj[themeName].qualitativeRanges[j]) ? jsonObj[themeName].qualitativeRanges[0].rangeStroke : jsonObj[themeName].qualitativeRanges[j].rangeStroke) : this.model.qualitativeRanges[j].rangeStroke);
            }
            //            this.model = $.extend(true, this.model, jsonObj[themeName], this.model);
        },

        // ********************************* ClientSide Events ************************************
       
        _onDrawQualitativeRanges: function (rangeOptions, rangeEnd, rangeIndex) {
            var data = { object: this, scaleElement: this.model.quantitativeScaleSettings, rangeIndex: rangeIndex, rangeElement: rangeOptions, rangeEndValue: rangeEnd };
            this._trigger("drawQualitativeRanges", data);
        },
        
        /**
         * To animate the graph
		 * @private
         */
        _doAnimation: function () {
            var elements = $("." + this.svgObject.id + "_FeatureMeasure");
            for (var i = elements.length - 1; i >= 0; i--) {
                var element = elements[i];
                this._animateFeatureBar(element);
            }
        },
        /**
         * To animate the Line in the graph
		 * @private
         */
        _doLineAnimation: function () {
            var compareElements = $("." + this.svgObject.id + "_ComparativeMeasure");
            var secondsPerPoint = 2000 / compareElements.length;
            for (var i = compareElements.length - 1; i >= 0; i--) {
                var element = compareElements[i];
                $(element).attr("transform", "scale(0)");
                this._doLineSymbol(element, secondsPerPoint, i);
            }
        },
        /**
         * To animate the FeatureBar in the graph
		 * @private
         */
        _animateFeatureBar: function (element) {
            var box = element.getBBox();
            var left, top;
            if (((this._orientation == ej.datavisualization.BulletGraph.Orientation.Horizontal) && (this._flowDirection == ej.datavisualization.BulletGraph.FlowDirection.Backward)) || ((this._orientation == ej.datavisualization.BulletGraph.Orientation.Vertical) && (this._flowDirection == ej.datavisualization.BulletGraph.FlowDirection.Forward))) {
                left = box.x + box.width;
                top = box.y + box.height;
            }
            else {
                left = box.x;
                top = box.y;
            }
            $(element).animate(
            {
                scale: 1
            },
            {
                duration: 1000,

                step: function (now) {
                    scaleVal = now;
                    $(element).attr("transform", "translate(" + left + " " + top + ") scale(" + now + ",1) translate(" + (-left) + " " + (-top) + ")");
                }
            });
        },
        /**
         * To manupulate the line symbol
		 * @private
         */
        _doLineSymbol: function (element, sec, val) {
            var beginTime = parseInt(val * sec);
            var box = element.getBBox();
            var centerX = box.x + (box.width / 2);
            var centerY = box.y + (box.height / 2);
            $(element).delay(beginTime).animate(
                {
                    scale: 1
                },
                {
                    duration: 200,
                    step: function (now) {
                        $(element).attr("transform", "translate(" + centerX + " " + centerY + ") scale(" + now + ") translate(" + (-centerX) + " " + (-centerY) + ")");
                    }
                });
        },
        /**
         * Section For Binding event
		 * @private
         */
        bindEvents: function () {
            if (this.model.tooltipSettings.visible)
                this._on($(this.svgObject), "mousemove", this._bulletMouseMove);
            if (this.model.enableResizing) {
                if (!this.isDevice())
                    this._on($(window), "resize", this._bulletResize);
                else
                    this._on($(window), "orientationchange", this._bulletResize);
            }
        },
        /**
         * To check whether it is a device or not
		 * @private
         */
        isDevice: function () {
            // return (/mobile|tablet|android|kindle/i.test(navigator.userAgent.toLowerCase()));
            // comment above line temporary. Due to the below code event wont bind for tablet device
            return (/mobile|tablet|android|kindle/i.test(navigator.userAgent.toLowerCase()));
        },
        /**
         * To resize the bullet
		 * @private
         */
        _bulletResize: function () {
            var bullet = this;
            var $svgObj = $(bullet.svgObject);
            if (this.resizeTO) clearTimeout(this.resizeTO);
            this.resizeTO = setTimeout(function () {
                if (!bullet.isDevice()) {
                    if (bullet.svgWidth > $(this).width()) {
                        $svgObj.width($(this).width());
                    } else {
                        $svgObj.width(bullet.svgWidth);
                    }
                    if (bullet.svgHeight > $(this).height()) {
                        $svgObj.height($(this).height());
                    } else {
                        $svgObj.height(bullet.svgHeight);
                    }

                }
                else {
                    $svgObj.width($(bullet.element).width());
                    $svgObj.height($(bullet.element).height());
                }
                bullet.redraw();
            }, 500);
        },
        /**
         * Section For handle the Mouse move event
		 * @private
         */
        _bulletMouseMove: function (evt) {
            var currentVal, targetVal, categoryVal, measureId, targetId, targetClass, tooltipdiv = $('.tooltipDiv');
            targetId = $(evt.target).attr("id");
            targetClass = $(evt.target).attr("class");
            if (targetClass != "undefined" && (targetClass == this.svgObject.id + '_FeatureMeasure' || targetClass == this.svgObject.id + '_ComparativeMeasure')) {
                measureId = targetId.substring(targetId.lastIndexOf("_") + 1);
                currentVal = (!ej.isNullOrUndefined(this.model.fields) && this.model.fields["dataSource"] != null) ? this.model.fields["dataSource"][measureId][this.model.fields.featureMeasures] : this._scale.featureMeasures[measureId].value;
                targetVal = (!ej.isNullOrUndefined(this.model.fields) && this.model.fields["dataSource"] != null) ? this.model.fields["dataSource"][measureId][this.model.fields.comparativeMeasure] : this._scale.featureMeasures[measureId].comparativeMeasureValue;
                categoryVal = (!ej.isNullOrUndefined(this.model.fields) && this.model.fields["dataSource"] != null) ? (ej.isNullOrUndefined(this.model.fields["dataSource"][measureId][this.model.fields.category])) ? null : this.model.fields["dataSource"][measureId][this.model.fields.category] : (ej.isNullOrUndefined(this._scale.featureMeasures[measureId].category)) ? null : this._scale.featureMeasures[measureId].category;
                var data = { currentValue: currentVal, targetValue: targetVal, category: categoryVal };
                if ($('.tooltipDiv').length == 0) {
                    tooltipdiv = $("<div class='tooltipDiv' style='position: absolute; z-index: 105; display: block;'></div>");
                    $(document.body).append(tooltipdiv);
                    $('.tooltipDiv').addClass(this.svgObject.id);
                }
                if (this.model.tooltipSettings.template != "" && this.model.tooltipSettings.template != null) {
                    var cloneNode = $("#" + this.model.tooltipSettings.template).clone();
                    $(cloneNode).css("display", "block").appendTo(tooltipdiv);
                    $(tooltipdiv).html($(cloneNode).render(data));
                } else {
                    var category = (!ej.isNullOrUndefined(data.category)) ? ("<br/> Category : " + data.category) : "";
                    $(tooltipdiv).html("Current : " + data.currentValue + "<br/> Target : " + data.targetValue + category);
                }
                $(tooltipdiv).css("font-size", "12px");
                var xPos = evt.clientX + $(document).scrollLeft() + 10;
                var yPos = evt.clientY + $(document).scrollTop() + 10;
                xPos = ((xPos + $(tooltipdiv).width()) < $(window).width()) ? (xPos) : (xPos - $(tooltipdiv).width());
                yPos = ((yPos + $(tooltipdiv).height()) < $(window).height()) ? (yPos) : (yPos - $(tooltipdiv).height());

                if (xPos === undefined || xPos === null)
                    xPos = evt.clientX + $(document).scrollLeft() + 10;
                if (yPos === undefined || yPos === null)
                    yPos = evt.clientY + $(document).scrollTop() + 10;
                $(tooltipdiv).css("left", xPos);
                $(tooltipdiv).css("top", yPos);
                $(tooltipdiv).css({
                    '-webkit-border-radius': "5px 5px 5px 5px",
                    '-moz-border-radius': "5px 5px 5px 5px",
                    '-o-border-radius': "5px 5px 5px 5px",
                    'border-radius': "5px 5px 5px 5px",
                    'background-color': "White",
                    'border': "1px Solid Black",
                    'padding-bottom': "5px",
                    'padding-left': "5px",
                    'padding-right': "5px",
                    'padding-top': "5px"
                });
            }
            else if (targetClass != this.svgObject.id + '_FeatureMeasure' && targetClass != this.svgObject.id + '_ComparativeMeasure') {
                $(".tooltipDiv").remove();
            }
        },

        //-----Public methods----------//
        /**
        * To redraw the bulet graph  
		* @return jQuery
		* @example 
		* &lt;div id="bulletgraph1"&gt;Bullet Graph&lt;/div&gt; <br> 
		* &lt;script&gt;
		* // Create Bullet graph
		* var gphObj = $("#bulletGraph1").data("ejBulletGraph");
		* gphObj.redraw(); // redraw the graph
		* &lt;/script&gt;
		* @example 
		* &lt;div id="bulletGraph1"&gt;BulletGraph&lt;/div&gt; <br> 
		* &lt;script&gt;
		* // redraw the Bullet graph
		* $("#butbulletGraph1ton1").ejButton("redraw");	
		* &lt;/script&gt;
        */
        redraw: function () {
            $(this.svgObject).empty();
            this._renderBulletElements();
        },
        /**
        * To destroy the bullet graph  
		* @return jQuery
		* @example 
		* &lt;div id="bulletgraph1"&gt;Bullet Graph&lt;/div&gt; <br> 
		* &lt;script&gt;
		* // Destroy Bullet graph
		* var gphObj = $("#bulletGraph1").data("ejBulletGraph");
		* gphObj.destroy(); // destroy the graph
		* &lt;/script&gt;
		* @example 
		* &lt;div id="bulletGraph1"&gt;BulletGraph&lt;/div&gt; <br> 
		* &lt;script&gt;
		* // destroy the Bullet graph
		* $("#bulletGraph1").ejButton("destroy");	
		* &lt;/script&gt;
        */
        destroy: function () {
            $(this.element).removeClass("e-bulletgraph e-js").find("#" + this.svgObject.id).remove();
        },
        /**
        * To set the value for feature measure bar.
		* @return jQuery
		* @param {number} argument.Index value for the graph 
		* @param {number}  argument.Measure value for the graph
		* @example 
		* &lt;div id="bulletGraph1"&gt;Bulletgraph&lt;/div&gt; <br> 
		* &lt;script&gt;
		* // To set the value for feature measure bar.
		* var btnObj = $("#bulletGraph1").data("ejBulletGraph");
		* btnObj.setFeatureMeasureBarValue(1,8); // set the value
		* &lt;/script&gt;
		* @example 
		* &lt;div id="bulletGraph1"&gt;BulletGraph&lt;/div&gt; <br> 
		* &lt;script&gt;
		* // To set the value for feature measure bar.
		* $("#bulletGraph1").ejBulletGraph("setFeatureMeasureBarValue(1,8)");	
		* &lt;/script&gt;
        */
        setFeatureMeasureBarValue: function (index, measureValue) {
            this._scale = this.model.quantitativeScaleSettings;
            var length = (!ej.isNullOrUndefined(this.model.fields) && this.model.fields["dataSource"] != null) ? this.model.fields["dataSource"].length : this._scale.featureMeasures.length; //(this._scale.featureMeasures != null) ? this._scale.featureMeasures.length : this.model.fields["dataSource"].length;
            if (index < length && measureValue != null) {
                if (!ej.isNullOrUndefined(this.model.fields) && this.model.fields["dataSource"] != null)
                    this.model.fields["dataSource"][index][this.model.fields.featureMeasures] = measureValue;
                else
                    this._scale.featureMeasures[index].value = measureValue;
                this.redraw();
                if (this.model.enableAnimation)
                    this._doAnimation();
            }
        },
        /**
        * To set the value for comparative measure in bullet graph.
		* @return jQuery
		* @param {number} argument.Index value for the graph 
		* @param {number}  argument.Measure value for the graph
		* @example 
		* &lt;div id="bulletGraph1"&gt;BulletGraph&lt;/div&gt; <br> 
		* &lt;script&gt;
		* // set the value for comparative measure
		* var btnObj = $("#bulletGraph1").data("ejBulletGraph");
		* btnObj.setComparativeMeasureSymbol(1,7); // set the value
		* &lt;/script&gt;
		* @example 
		* &lt;div id="bulletGraph1"&gt;BulletGraph&lt;/div&gt; <br> 
		* &lt;script&gt;
		* // set the value for comparative measure
		* $("#bulletGraph1").ejBulletGraph("setComparativeMeasureSymbol(1,7)");	
		* &lt;/script&gt;
        */
        setComparativeMeasureSymbol: function (index, measureValue) {
            this._scale = this.model.quantitativeScaleSettings;
            var length = (!ej.isNullOrUndefined(this.model.fields) && this.model.fields["dataSource"] != null) ? this.model.fields["dataSource"].length : this._scale.featureMeasures.length; //(this._scale.featureMeasures != null) ? this._scale.featureMeasures.length : this.model.fields["dataSource"].length;
            if (index < length && measureValue != null) {
                if (!ej.isNullOrUndefined(this.model.fields) && this.model.fields["dataSource"] != null)
                    this.model.fields["dataSource"][index][this.model.fields.comparativeMeasure] = measureValue;
                else
                    this._scale.featureMeasures[index].comparativeMeasureValue = measureValue;
                this.redraw();
                if (this.model.enableAnimation)
                    this._doLineAnimation();
            }
        }

    });
    /**
	 * Enum for Bullet Graph Orientation	 
	 * @enum {string}
	 * @global 
	 */
    ej.datavisualization.BulletGraph.Orientation = {
        /**  support for orientation in horizontal direction. */
        Horizontal: "horizontal",
        /**  support for orientation in vertical direction. */
        Vertical: "vertical"
    };
    /**
	 * Enum for the shape of indicator symbol 
	 * @enum {string}
	 * @global 
	 */
    ej.datavisualization.BulletGraph.Shape = {
        /**  support for drawing circle as indicator symbol. */
        Circle: "circle",
        /**  support for drawing cross as indicator symbol. */
        Cross: "cross",
        /**  support for drawing diamond as indicator symbol. */
        Diamond: "diamond",
        /**  support for drawing down arrow as indicator symbol. */
        DownArrow: "downarrow",
        /**  support for drawing ellipse as indicator symbol. */
        Ellipse: "ellipse",
        /**  support for drawing horizontal line as indicator symbol. */
        HorizontalLine: "horizontalLine",
        /**  support for drawing image as indicator symbol. */
        Image: "image",
        /**  support for drawing inverted triangle as indicator symbol. */
        InvertedTriangle: "invertedtriangle",
        /**  support for drawing left arrow as indicator symbol. */
        LeftArrow: "leftarrow",
        /**  support for drawing pentagon as indicator symbol. */
        Pentagon: "pentagon",
        /**  support for drawing rectangle as indicator symbol. */
        Rectangle: "Rectangle",
        /**  support for drawing right arrow as indicator symbol. */
        RightArrow: "rightarrow",
        /**  support for drawing star as indicator symbol. */
        Star: "star",
        /**  support for drawing trapezoid as indicator symbol. */
        Trapezoid: "trapezoid",
        /**  support for drawing triangle as indicator symbol. */
        Triangle: "triangle",
        /**  support for drawing up arrow as indicator symbol. */
        UpArrow: "uparrow",
        /**  support for drawing vertical line as indicator symbol. */
        VerticalLine: "verticalline",
        /**  support for drawing wedge as indicator symbol. */
        Wedge: "wedge",
    };
    /**
	 * Enum for Bullet Graph Tick Position 
	 * @enum {string}
	 * @global 
	 */
    ej.datavisualization.BulletGraph.TickPosition = {
        /**  To set the Tick position below. */
        Below: "below",
        /**  To set the Tick position above. */
        Above: "above",
        /**  To set the Tick position cross. */
        Cross: "cross"
    };
    /**
	 * Enum for Bullet Graph Label Position 
	 * @enum {string}
	 * @global 
	 */
    ej.datavisualization.BulletGraph.LabelPosition = {
        /**  To set the label position below. */
        Below: "below",
        /**  To set the label position above. */
        Above: "above"
    };
    /**
	 * Enum for Bullet Graph Flow Direction 
	 * @enum {string}
	 * @global 
	 */
    ej.datavisualization.BulletGraph.FlowDirection = {
        /**  To set the flow direction forward. */
        Forward: "forward",
        /**  To set the flow direction backward. */
        Backward: "backward"
    };
    /**
	 * Enum for Font Style  
	 * @enum {string}
	 * @global 
	 */
    ej.datavisualization.BulletGraph.FontStyle = {
        /**  To set the fontStyle Normal. */
        Normal: "Normal",
        /**  To set the fontStyle Italic. */
        Italic: "Italic",
        /**  To set the fontStyle Oblique. */
        Oblique: "Oblique"
    };
    /**
	 * Enum for Font Weight 
	 * @enum {string}
	 * @global 
	 */
    ej.datavisualization.BulletGraph.FontWeight = {
        /**  To set the fontWeight Normal. */
        Normal: "Normal",
        /**  To set the fontWeight Bold. */
        Bold: "Bold",
        /**  To set the fontWeight Bolder. */
        Bolder: "Bolder",
        /**  To set the fontWeight Lighter. */
        Lighter: "Lighter"
    };
    /**
	 * Enum for Bullet Graph Theme3
	 * @enum {string}
	 * @global 
	 */
    ej.datavisualization.BulletGraph.Themes = {
        /**  To set the theme flatlight. */
        flatlight: {
            quantitativeScaleSettings: {
                majorTickSettings: { stroke: "#191919" },
                minorTickSettings: { stroke: "#191919" },
                labelSettings: {
                    stroke: "#191919"
                },
                featuredMeasureSettings: { stroke: "#191919" },
                comparativeMeasureSettings: { stroke: "#191919" }
            },
            qualitativeRanges: [{
                rangeStroke: "#ebebeb"
            }, {
                rangeStroke: "#d8d8d8"
            }, {
                rangeStroke: "#7f7f7f"
            }],
            captionSettings: {
                font: { color: "#191919" },
                subTitle: {
                    font: { color: "#191919" }
                }
            }
        },
        /**  To set the theme flatdark. */
        flatdark: {
            /**  To set the theme quantitativeScaleSettings. */
            quantitativeScaleSettings: {
                majorTickSettings: { stroke: "#ffffff" },
                minorTickSettings: { stroke: "#ffffff" },
                labelSettings: {
                    stroke: "#ffffff"
                },
                featuredMeasureSettings: { stroke: "#ffffff" },
                comparativeMeasureSettings: { stroke: "#ffffff" }
            },
            qualitativeRanges: [{
                rangeStroke: "#b3b3b3"
            }, {
                rangeStroke: "#999999"
            }, {
                rangeStroke: "#4d4d4d"
            }],
            captionSettings: {
                font: { color: "#ffffff" },
                subTitle: {
                    font: { color: "#ffffff" }
                }
            }
        }
    };

})(jQuery, Syncfusion);
;
/**
* @fileOverview Plugin to style the Html RangeNavigator elements
* @copyright Copyright Syncfusion Inc. 2001 - 2013. All rights reserved.
*  Use of this code is subject to the terms of our license.
*  A copy of the current license can be obtained at any time by e-mailing
*  licensing@syncfusion.com. Any infringement will be prosecuted under
*  applicable laws. 
* @version 12.1 
* @author <a href="mailto:licensing@syncfusion.com">Syncfusion Inc</a>
*/
(function ($, ej, undefined) {
    /**
         * @namespace ej
         * @classdesc The rangenavigator can be easily configured to the DOM element, such as div. you can create a rangenavigator with a highly customizable look and feel.
         * @class ejRangeNavigator
         * @param {object} options - settings for rangenavigator
         * @requires jQuery
         * @requires jquery.globalize.min.js
         * @requires ej.core.js
         * @requires ej.data.js
         * @requires ej.chart.js
         * @requires ej.rangenavigator.js
         * @class ejRangeNavigator
         * @param {object} options - settings for rangenavigator    
         * @example 
         * &lt;div id="container"&gt;&lt;/div&gt; <br> 
         * &lt;script&gt;
         * // Create RangeNavigator
         * $('#container').RangeNavigator(); 	
         * &lt;/script&gt;
         */


    ej.widget("ejRangeNavigator", "ej.datavisualization.RangeNavigator", /** @lends ejRangeNavigator# */ {

        ejChart: "",
		
        validTags: ["div"],
        // default model
        defaults: /** @lends ejRangeNavigator# */ {
            /**		
               * By specifying this property the user can change the theme of the range navigator.
                 @default null
               * @type {string}
               * @example 
               * &lt;div id="RangeNavigator"&gt;&lt;/div&gt; 
               * &lt;script&gt;
               *	$("#container").ejRangeNavigator({
               *    theme: "flatdark";
               *    });
               * &lt;/script&gt;  
               */
            theme: "",
            /**		
               * Padding specifies the gap between the container and the range navigator.
                 @default "0"
               * @type {string}					
               * @example 
              * &lt;div id="RangeNavigator"&gt;&lt;/div&gt; 
               * &lt;script&gt;
               *	$("#container").ejRangeNavigator({
               *    padding: "15";
               *    });
               * &lt;/script&gt;  
               */
            padding: "0",
            /**		
               * Sets a value whether to make the range navigator responsive on resize.
                 @default false
               * @type {boolean}					
               * @example 
              * &lt;div id="RangeNavigator"&gt;&lt;/div&gt; 
               * &lt;script&gt;
               *	$("#container").ejRangeNavigator({
               *    enableAutoResizing: true;
               *    });
               * &lt;/script&gt;  
               */
            enableAutoResizing: false,
            /**		
               * Toggles the placement of slider exactly on the place it left or on the nearest interval.
                 @default false
               * @type {boolean}				
               * @example 
               * &lt;div id="RangeNavigator"&gt;&lt;/div&gt; 
               * &lt;script&gt;
               *	$("#container").ejRangeNavigator({
               *    allowSnapping: true;
               *    });
               * &lt;/script&gt;  
               */
            allowSnapping: false,
            
            /**		
           * Contains property to customize the hight and width of range navigator.
           * @type {object}
           */
            sizeSettings: /** @lends ejRangeNavigator# */ {
                /**		
               * Specifies width of the range navigator.
               * @alias ejRangeNavigator#sizeSettings->width
                 @default null
               * @type {string}					
               * @example 
               * &lt;div id="RangeNavigator"&gt;&lt;/div&gt; 
               * &lt;script&gt;
               *	$("#container").ejRangeNavigator({
               *    sizeSettings: { width : "900" },
               *    });
               * &lt;/script&gt;  
               */
                width: "",
                /**		
               * Specifies height of the range navigator.
               * @alias ejRangeNavigator#sizeSettings->height
                 @default null
               * @type {string}
               * @example 				
              * &lt;div id="RangeNavigator"&gt;&lt;/div&gt; 
               * &lt;script&gt;
               *	$("#container").ejRangeNavigator({
               *    sizeSettings: { height : "130" },
               *    });
               * &lt;/script&gt;  
               */
                height: ""
            },
            _size: {
                
            },
            /**		
               * This property is to specify the localization of range navigator.
                 @default "en-US"
               * @type {string}				
               * @example 
               * &lt;div id="RangeNavigator"&gt;&lt;/div&gt; 
               * &lt;script&gt;
               *	$("#container").ejRangeNavigator({
               *    locale: "en-US",
               *    });
               * &lt;/script&gt;  
               */
            locale: "en-US",
            /**		
              * You can plot data of type date time or numeric. This property determines the type of data that this axis will handle. See {@link ValueType} 
                @default "datetime"
              * @type {enum}		
              * @example 
              * &lt;div id="RangeNavigator"&gt;&lt;/div&gt; 
               * &lt;script&gt;
               *	$("#container").ejRangeNavigator({
               *    valueType: "numeric",
               *    });
               * &lt;/script&gt;  
               */
            valueType: "datetime",
            /**		
           * Options for configuring minor grid lines, major grid lines, axis line of axis.
           * @type {object}
           */
            valueAxisSettings:/** @lends ejRangeNavigator# */  {
                /**		
              * If the range is not given explicitly, range will be calculated automatically. You can customize the automatic range calculation using rangePadding.
              * @alias ejRangeNavigator#valueAxisSettings->rangePadding
                @default "none"
              * @type {string}				
              * @example 
              * &lt;div id="RangeNavigator"&gt;&lt;/div&gt; 
              * &lt;script&gt;
              *	$("#container").ejRangeNavigator({
              *    valueAxisSettings:{rangePadding: "normal"},
              *    });
              * &lt;/script&gt;  
              */
                rangePadding: 'none',
                /**		
                * Options for customizing the axis line.
                * @type {object}
                * @alias ejRangeNavigator#valueAxisSettings->axisLine
                */
                axisLine:
                    {
                        /**		
                        * Toggles the visibility of axis line.
                        * @alias ejRangeNavigator#valueAxisSettings->axisLine->visible
                        *  @default "none"
                        * @type {string}					
                        * @example 
                        * &lt;div id="RangeNavigator"&gt;&lt;/div&gt; 
                        * &lt;script&gt;
                        *	$("#container").ejRangeNavigator({
                        *    valueAxisSettings:{axisLine: {visible: true}}
                        *    });
                        * &lt;/script&gt;  
                        */
                        visible: false
                    },
                /**		
           * Options for customizing the font of the axis.
           * @type {object}
           * @alias ejRangeNavigator#valueAxisSettings->font
           */
                font: {
                    /**		
             * Text in axis render with the specified size.
             * @alias ejRangeNavigator#valueAxisSettings->font->size
               @default "0px"
             * @type {string}					
             * @example 
             * &lt;div id="RangeNavigator"&gt;&lt;/div&gt; 
             * &lt;script&gt;
             *	$("#container").ejRangeNavigator({
             *    valueAxisSettings:{font: {size: '12px'}}
             *    });
             * &lt;/script&gt;  
             */
                    size: '0px',
                },
                /**		
          * Options for customizing the major tick lines in axis.
          * @type {object}
          * @alias ejRangeNavigator#valueAxisSettings->majorTickLines
          */
                majorTickLines:
                {
                    /**		
                    * Specifies width of the major tick lines.
                    * @alias ejRangeNavigator#valueAxisSettings->majorTickLines->width
                      @default 0
                    * @type {number}				
                    * @example 
                    * &lt;div id="RangeNavigator"&gt;&lt;/div&gt; 
                    * &lt;script&gt;
                    *	$("#container").ejRangeNavigator({
                    *    valueAxisSettings:{majorTickLines: {width: 3}}
                    *    });
                    * &lt;/script&gt;  
                    */
                    width: 0,
                    /**		
                    * Specifies the size of the majorTickLines in RangeNavigator
                    * @alias ejRangeNavigator#valueAxisSettings->majorTickLines->size
                    *  @default 0
                    * @type {number}					
                    * @example 
                    * &lt;div id="RangeNavigator"&gt;&lt;/div&gt; 
                    * &lt;script&gt;
                    *	$("#container").ejRangeNavigator({
                    *    valueAxisSettings:{majorTickLines: {size: 3}}
                    *    });
                    * &lt;/script&gt;  
                    */
                    size: 0,
                    /**		
            * Toggles the visibility of major tick lines.
            * @alias ejRangeNavigator#valueAxisSettings->majorTickLines->visible
            *  @default true
            * @type {boolean}				
            * @example 
            * &lt;div id="RangeNavigator"&gt;&lt;/div&gt; 
            * &lt;script&gt;
            *	$("#container").ejRangeNavigator({
            *    valueAxisSettings:{majorTickLines: {visible: false}}
            *    });
            * &lt;/script&gt;  
            */
                    visible: true
                },
                /**		
                * Options for customizing the major grid lines.
                * @type {object}
                * @alias ejRangeNavigator#valueAxisSettings->majorGridLines
                */
                majorGridLines:  {
                    /**		
                    * Toggles the visibility of major grid lines.
                    * @alias ejRangeNavigator#valueAxisSettings->majorGridLines->visible
                      @default false
                    * @type {boolean}				
                    * @example 
                    * &lt;div id="RangeNavigator"&gt;&lt;/div&gt; 
                    * &lt;script&gt;
                    *	$("#container").ejRangeNavigator({
                    *    valueAxisSettings:{majorGridLines: {visible: true}}
                    *    });
                    * &lt;/script&gt;  
                    */
                    visible: false
                },
                /**		
             * Toggles the visibility of axis in range navigator.
             * @alias ejRangeNavigator#valueAxisSettings->visible
             *  @default false
             * @type {boolean}				
             * @example 
             * &lt;div id="RangeNavigator"&gt;&lt;/div&gt; 
             * &lt;script&gt;
             *	$("#container").ejRangeNavigator({
             *    valueAxisSettings:{visble: true}
             *    });
             * &lt;/script&gt;  
             */
                visible: false
            },
            /**		
              * If the range is not given explicitly, range will be calculated automatically. You can customize the automatic range calculation using rangePadding. See {@link RangePadding}
               * @default "none"
              * @type {enum}			
              * @example 
              * &lt;div id="RangeNavigator"&gt;&lt;/div&gt; 
              * &lt;script&gt;
              *	$("#container").ejRangeNavigator({
              *   rangePadding: "normal",
              *    });
              * &lt;/script&gt;  
              */
            rangePadding: "none",
            /**		
         * Toggles the direction of rendering the range navigator control.
         *  @default false
         * @type {boolean}
         * @example 
              * &lt;div id="RangeNavigator"&gt;&lt;/div&gt; 
              * &lt;script&gt;
              *	$("#container").ejRangeNavigator({
              *   enableRTL: true,
              *    });
              * &lt;/script&gt;  
              */
            enableRTL: false,
            /**		
           * Specifies the data source for range navigator.
           * @type {object}
		   * @example
           *	$("#container").ejRangeNavigator({
           *    dataSource:"data1", 
           *     xName: "X", 
           *     yName: "Y"
           *    });
           */
            dataSource: "",
            /**		
       * Specifies the xName for dataSource.  This is used to take the x values from dataSource
       * @type {object}
       * @example
       * &lt;div id="RangeNavigator"&gt;&lt;/div&gt; 
       * &lt;script&gt;
       *	$("#container").ejRangeNavigator({
       *    dataSource:"data1", 
       *    xName: "X"
       *    });
       * &lt;/script&gt; 
       */
                xName: "x",
            /**		
  * Specifies the yName for dataSource.  This is used to take the y values from dataSource
  * @type {object}
  * @example
  * &lt;div id="RangeNavigator"&gt;&lt;/div&gt; 
  * &lt;script&gt;
  *	$("#container").ejRangeNavigator({
  *    dataSource:"data1", 
  *    yName: "Y"
  *    });
  * &lt;/script&gt; 
  */
                yName: "y",
            /**
           * Options for customizing the tooltip in range navigator.
           * @type {object}
           * @example
            *	$("#container").ejRangeNavigator({
            *    tooltipSettings: { visible: true, labelFormat: "MMM/dd/yyyy", tooltipDisplayMode: "always", tooltipDisplayMode: "always" },
            *    });
           */
            tooltipSettings: /** @lends ejRangeNavigator# */ {
                /**		
      * Toggles the visibility of tooltip.
      * @alias ejRangeNavigator#tooltipSettings->visible
      * @default true
      * @type {boolean}			
      * @example 
             * &lt;div id="RangeNavigator"&gt;&lt;/div&gt; 
             * &lt;script&gt;
             *	$("#container").ejRangeNavigator({
             *    tooltipSettings:{visible: true}
             *    });
             * &lt;/script&gt;  
             */
                visible: true,
                /**		
   * Specifies the format of text to be displayed in tooltip.
   * @alias ejRangeNavigator#tooltipSettings->labelFormat
   * @default "MM/dd/yyyy"
   * @type {string}				
   * @example 
             * &lt;div id="RangeNavigator"&gt;&lt;/div&gt; 
             * &lt;script&gt;
             *	$("#container").ejRangeNavigator({
             *    tooltipSettings: { labelFormat: "MM/dd/yyyy"}
             *    });
             * &lt;/script&gt;  
             */
                labelFormat: "MM/dd/yyyy",
                /**		
  * Specifies the mode of displaying the tooltip. Neither to display the tooltip always nor on demand.
  * @alias ejRangeNavigator#tooltipSettings->tooltipDisplayMode
  * @default null
  * @type {string}				
  * @example 
  * &lt;div id="RangeNavigator"&gt;&lt;/div&gt; 
              * &lt;script&gt;
              *	$("#container").ejRangeNavigator({
              *    tooltipSettings:{tooltipDisplayMode: "always"},
              *    });
              * &lt;/script&gt;  
              */
                tooltipDisplayMode: "always",
                /**		
  * Specifies the background color of tooltip.
  * @default "#303030"
  * @type {string}					
  * @example 
  * &lt;div id="RangeNavigator"&gt;&lt;/div&gt; 
              * &lt;script&gt;
              *	$("#container").ejRangeNavigator({
              *    tooltipSettings:{backgroundColor: "#303030"},
              *    });
              * &lt;/script&gt;  
              * @alias ejRangeNavigator#tooltipSettings->backgroundColor
              */
                backgroundColor: "#303030",
                    /**		
           * Options for customizing the font in tooltip.
           * @type {object}
           * @alias ejRangeNavigator#tooltipSettings->font
           */
                    font:
                    {
                        /**		
 * Specifies the color of text in tooltip. Tooltip text render in the specified color.
 * @default "#FFFFFF"
 * @type {string}				
 * @example 
  * &lt;div id="RangeNavigator"&gt;&lt;/div&gt; 
              * &lt;script&gt;
              *	$("#container").ejRangeNavigator({
              *    tooltipSettings:{font:{color : "FFFFFF"}},
              *    });
              * &lt;/script&gt;
  * @alias ejRangeNavigator#tooltipSettings->font->color
 */
                        color: '#FFFFFF',
                        /**		
* Specifies the font family of text in tooltip. Tooltip text render in the specified font family.
* @default "Segoe UI"
* @type {string}			
* @example 
  * &lt;div id="RangeNavigator"&gt;&lt;/div&gt; 
              * &lt;script&gt;
              *	$("#container").ejRangeNavigator({
                tooltipSettings:{font:{family:"Arial"}}
                }); 
              * &lt;/script&gt;
* @alias ejRangeNavigator#tooltipSettings->font->family
*/
                        family: 'Segoe UI',
                        /**		
* Specifies the font style of text in tooltip. Tooltip text render in the specified font style.
* @default ej.datavisualization.RangeNavigator.fontStyle.Normal
* @type {string}			
* @example 
* &lt;div id="RangeNavigator"&gt;&lt;/div&gt; 
              * &lt;script&gt;
              *	$("#container").ejRangeNavigator({
              *    tooltipSettings:{font:{fontStyle:"Normal"}}
              *    });
              * &lt;/script&gt;
* @alias ejRangeNavigator#tooltipSettings->font->fontStyle
*/
                        fontStyle: 'Normal',
                        /**		
* Specifies the size of text in tooltip. Tooltip text render in the specified size.
* @default "10px"
* @type {string}				
* @example 
  * &lt;div id="RangeNavigator"&gt;&lt;/div&gt; 
              * &lt;script&gt;
              *	$("#container").ejRangeNavigator({
              *    tooltipSettings:{font:{size:"12px"}},
              *    });
              * &lt;/script&gt;
* @alias ejRangeNavigator#tooltipSettings->font->size
*/
                        size: '10px',
                        /**		
* Specifies the opacity of text in tooltip. Tooltip text render in the specified opacity.
* @default 1
* @type {number}				
* @example 
  * &lt;div id="RangeNavigator"&gt;&lt;/div&gt; 
              * &lt;script&gt;
              *	$("#container").ejRangeNavigator({
              *   tooltipSettings:{ font:{opacity:0.5}}
              *    });
              * &lt;/script&gt;
* @alias ejRangeNavigator#tooltipSettings->font->opacity
*/opacity: 1,
                        /**		
* Specifies the weight of text in tooltip. Tooltip text render in the specified weight.  
 @default ej.datavisualization.RangeNavigator.weight.Regular
* @type {string}				
* @example 
* &lt;div id="RangeNavigator"&gt;&lt;/div&gt; 
              * &lt;script&gt;
              *	$("#container").ejRangeNavigator({
              *    tooltipSettings:{ font:{weight:"regular"}},
              *    });
              * &lt;/script&gt;
* @alias ejRangeNavigator#tooltipSettings->font->weight
*/
                        weight: 'regular'
                    }
            },
                /**		
* This property determines the starting position of the zoomed axis. Value must be specified between 0 and 1.
@default "0"
* @type {string}			
* @example 
* &lt;div id="RangeNavigator"&gt;&lt;/div&gt; 
              * &lt;script&gt;
              *	$("#container").ejRangeNavigator({
              *    zoomPosition:"0",
              *    });
              * &lt;/script&gt;  
* @alias ejRangeNavigator#zoomSettings->zoomPosition
*/
                zoomPosition: "0",
                /**		
* This property determines the factor by which the axis is scaled. Value must be specified between 0 and 1.
@default "1"
* @type {string}				
* @example 
* &lt;div id="RangeNavigator"&gt;&lt;/div&gt; 
              * &lt;script&gt;
              *	$("#container").ejRangeNavigator({
                         zoomFactor:"1"
              *    });
              * &lt;/script&gt;  
* @alias ejRangeNavigator#zoomSettings->zoomFactor
*/
                zoomFactor: "1",
            /**		
          * Options for customizing the start and end range values.
          * @type {object}
          */
            selectedRangeSettings: /** @lends ejRangeNavigator# */ {
                /**		
* Specifies the starting range of range navigator.
@default null
* @type {string}				
* @example 
  * &lt;div id="RangeNavigator"&gt;&lt;/div&gt; 
              * &lt;script&gt;
              *	$("#container").ejRangeNavigator({
              *   selectedRangeSettings:{start:"01/05/1992"},
              *    });
              * &lt;/script&gt;  
* @alias ejRangeNavigator#selectedRangeSettings->start
*/
                start: "",
                /**		
*  Specifies the ending range of range navigator.
@default null
* @type {string}			
* @example 
* &lt;div id="RangeNavigator"&gt;&lt;/div&gt; 
              * &lt;script&gt;
              *	$("#container").ejRangeNavigator({
              *    selectedRangeSettings:{end:"01/05/1993"},
              *    });
              * &lt;/script&gt;  
* @alias ejRangeNavigator#selectedRangeSettings->end
*/
                end: ""
            },
            /**		
               * selectedData is for getting the data when the "rangeChanged" event trigger from clientside.
               */
            selectedData: "",
            /**		
          * Options for customizing the starting and ending ranges.
          * @type {object}
          */
            rangeSettings: /** @lends ejRangeNavigator# */ {
                /**		
*  Specifies the starting range of range navigator.
@default null
* @type {string}			
* @example 
* &lt;div id="RangeNavigator"&gt;&lt;/div&gt; 
              * &lt;script&gt;
              *	$("#container").ejRangeNavigator({
              *    rangeSettings:{start:"01/05/1993"},
              *    });
* @alias ejRangeNavigator#rangeSettings->start
*/
                start: "",
                /**		
*  Specifies the ending range of range navigator.
@default null
* @type {string}				
* @example 
  * &lt;div id="RangeNavigator"&gt;&lt;/div&gt; 
              * &lt;script&gt;
              *	$("#container").ejRangeNavigator({
              *    rangeSettings:{end:"01/05/1993"}
              *    });
              * &lt;/script&gt;  
* @alias ejRangeNavigator#rangeSettings->end
*/
                end: ""
            },
            /**		
         * Toggles the redrawing of chart on moving the sliders. 
           @default true
         * @type {boolean}			
         * @example 
         * &lt;div id="RangeNavigator"&gt;&lt;/div&gt; 
              * &lt;script&gt;
              *	$("#container").ejRangeNavigator({
              *    enableDeferredUpdate:false,
              *    });
              * &lt;/script&gt;  
         */
            enableDeferredUpdate: true,

            series: "",

            seriesSettings: "",
            /**		
         * Options for customizing the labels colors, font, style, size, horizontalAlignment and opacity.
         * @type {object}
         */
            labelSettings:/** @lends ejRangeNavigator# */  {
                /**		
         * Options for customizing the style of labels in range navigator.
         * @type {object}
         * @alias ejRangeNavigator#labelSettings->style
         */
                style: {
                    /**		
        * Options for customizing the font of labels in range navigator.
        * @type {object}
        * @alias ejRangeNavigator#labelSettings->style->font
        */
                    font: {
                        /**		
* Specifies the label color. This color is applied to the labels in range navigator.
  @default "#FFFFFF"
* @type {string}				
* @example 
* &lt;div id="RangeNavigator"&gt;&lt;/div&gt; 
              * &lt;script&gt;
              *	$("#container").ejRangeNavigator({
              *    labelSettings:{style:{ font:{color:"#ff0000"}}},
              *    });
              * &lt;/script&gt;
* @alias ejRangeNavigator#labelSettings->style->font->color
*/
                        color: '#333333',
                        /**		
* Specifies the label font family. Labels render with the specified font family.
  @default "Segoe UI"
* @type {string}				
* @example 
* &lt;div id="RangeNavigator"&gt;&lt;/div&gt; 
              * &lt;script&gt;
              *	$("#container").ejRangeNavigator({
              *    labelSettings:{style:{ font:{family:"Arial"}}},
              *    });
              * &lt;/script&gt;
* @alias ejRangeNavigator#labelSettings->style->font->family
*/
                        family: 'Segoe UI',
                        /**		
  * Specifies the label font style. Labels render with the specified font style..See {@link FontStyle}
    @default "Normal"
  * @type {enum}				
  * @example 
  * &lt;div id="RangeNavigator"&gt;&lt;/div&gt; 
              * &lt;script&gt;
              *	$("#container").ejRangeNavigator({
              *    labelSettings:{style:{ font:{style:"Noraml"}}},
              *    });
              * &lt;/script&gt;
  * @alias ejRangeNavigator#labelSettings->style->font->style
  */
                        style: 'Normal',
                        /**		
*  Specifies the label font size. Labels render with the specified font size.
  @default "1px"
* @type {string}			
* @example 
* &lt;div id="RangeNavigator"&gt;&lt;/div&gt; 
              * &lt;script&gt;
              *	$("#container").ejRangeNavigator({
              *    labelSettings:{style:{ font:{size:"12px"}}},
              *    });
              * &lt;/script&gt;
* @alias ejRangeNavigator#labelSettings->style->font->size
*/
                        size: '10px',
                        /**		
                        * Specifies the label font opacity. Labels render with the specified font opacity.
                          @default 1
                        * @type {number}					
                        * @example 
                        * &lt;div id="RangeNavigator"&gt;&lt;/div&gt; 
                        * &lt;script&gt;
                        *	$("#container").ejRangeNavigator({
                        *    labelSettings:{style:{ font:{opacity:0.5}}}
                        *    });
                        * &lt;/script&gt;
                        * @alias ejRangeNavigator#labelSettings->style->font->opacity
                        */
                        opacity: 1,
                        /**		
* Specifies the label font weight. Labels render with the specified font weight. See {@link FontWeight}
  @default "regular"
* @type {enum}				
* @example 
  * &lt;div id="RangeNavigator"&gt;&lt;/div&gt; 
              * &lt;script&gt;
              *	$("#container").ejRangeNavigator({
              *    labelSettings:{style:{ font:{weight:"Regular"}}},
              *    });
              * &lt;/script&gt;
* @alias ejRangeNavigator#labelSettings->style->font->weight
*/
                        weight: 'regular'
                    },
                    /**		
* Specifies the horizontalAlignment of the label in RangeNavigator. See {@link HorizontalAlignment} 
@default "center"
* @type {enum}					
* @example 
* &lt;div id="RangeNavigator"&gt;&lt;/div&gt; 
              * &lt;script&gt;
              *	$("#container").ejRangeNavigator({
              *   labelSettings:{style:{ horizontalAlignment:"center"}},
              *    });
              * &lt;/script&gt;
* @alias ejRangeNavigator#labelSettings->style->horizontalAlignment
*/
                    horizontalAlignment: "center"
                },

                /**		
       * Options for customizing the higher level labels in range navigator.
       * @type {object}
        * @alias ejRangeNavigator#labelSettings->higherLevel
       */
                higherLevel: {
                    /**		
* Specifies the intervalType for higher level labels. See {@link IntervalType}
@default "years"
* @type {enum}				
* @example 
  * &lt;div id="RangeNavigator"&gt;&lt;/div&gt; 
              * &lt;script&gt;
              *	$("#container").ejRangeNavigator({
              *    labelSettings:{higherLevel:{ intervalType:"days"}},
              *    });
              * &lt;/script&gt;
* @alias ejRangeNavigator#labelSettings->higherLevel->intervalType
*/
                    intervalType: 'years',
                    /**		
      * Options for customizing the style of higher level labels.
      * @type {object}
                    * @alias ejRangeNavigator#labelSettings->higherLevel->style
                    */
                    style: {
                        /**		
    * Options for customizing the font properties.
    * @type {object}
                  * @alias ejRangeNavigator#labelSettings->higherLevel->style->font
                  */
                        font: {
                            /**		
*  Specifies the label font color. Labels render with the specified font color.
@default "black"
* @type {string}				
 * @example 
  * &lt;div id="RangeNavigator"&gt;&lt;/div&gt; 
              * &lt;script&gt;
              *	$("#container").ejRangeNavigator({
              *    labelSettings:{higherLevel:{ style:{font:{color: "red"}}}},
              *    });
              * &lt;/script&gt;  
* @alias ejRangeNavigator#labelSettings->higherLevel->style->font->color
*/
                            color: "black",
                            /**		
*  Specifies the label font family. Labels render with the specified font family.
@default "Segoe UI"
* @type {string}				
 * @example 
  * &lt;div id="RangeNavigator"&gt;&lt;/div&gt; 
              * &lt;script&gt;
              *	$("#container").ejRangeNavigator({
              *    labelSettings:{higherLevel:{ style:{font:{fontFamily: "Algerian"}}}},
              *    });
              * &lt;/script&gt;  
* @alias ejRangeNavigator#labelSettings->higherLevel->style->font->fontFamily
*/
                            fontFamily: "Segoe UI",
                            /**		
* Specifies the label font style. Labels render with the specified font style.
@default "Normal"
* @type {string}				
* @example 
 * &lt;div id="RangeNavigator"&gt;&lt;/div&gt; 
             * &lt;script&gt;
             *	$("#container").ejRangeNavigator({
             *    labelSettings:{higherLevel:{ style:{font:{fontStyle: "Italic"}}}},
             *    });
             * &lt;/script&gt;  
* @alias ejRangeNavigator#labelSettings->higherLevel->style->font->fontStyle
*/
                            fontStyle: "Normal",
                            /**		
* Specifies the label font size. Labels render with the specified font size.
@default "12px"
* @type {string}				
* @example 
* &lt;div id="RangeNavigator"&gt;&lt;/div&gt; 
            * &lt;script&gt;
            *	$("#container").ejRangeNavigator({
            *    labelSettings:{higherLevel:{ style:{font:{size: "14px"}}}},
            *    });
            * &lt;/script&gt;  
* @alias ejRangeNavigator#labelSettings->higherLevel->style->font->size
*/
                            size: "12px",
                            /**		
* Specifies the label opacity. Labels render with the specified opacity.
@default "12px"
* @type {string}				
* @example 
* &lt;div id="RangeNavigator"&gt;&lt;/div&gt; 
            * &lt;script&gt;
            *	$("#container").ejRangeNavigator({
            *    labelSettings:{higherLevel:{ style:{font:{opacity: "14px"}}}},
            *    });
            * &lt;/script&gt;  
* @alias ejRangeNavigator#labelSettings->higherLevel->style->font->opacity
*/
                            opacity: 1,
                            /**		
* Specifies the label font weight. Labels render with the specified font weight.
@default "regular"
* @type {string}				
* @example 
* &lt;div id="RangeNavigator"&gt;&lt;/div&gt; 
           * &lt;script&gt;
           *	$("#container").ejRangeNavigator({
           *    labelSettings:{higherLevel:{ style:{font:{fontWeight: "regular"}}}},
           *    });
           * &lt;/script&gt;  
* @alias ejRangeNavigator#labelSettings->higherLevel->style->font->fontWeight
*/
                            fontWeight: "regular"
                        },
                        /**		
* Specifies the horizontal text alignment of the text in label.
@default "center"
* @type {string}				
* @example 
* &lt;div id="RangeNavigator"&gt;&lt;/div&gt; 
           * &lt;script&gt;
           *	$("#container").ejRangeNavigator({
           *    labelSettings:{higherLevel:{ style:{horizontalAlignment: "left"}}},
           *    });
           * &lt;/script&gt;  
* @alias ejRangeNavigator#labelSettings->higherLevel->style->horizontalAlignment
*/
                        horizontalAlignment: "center"
                    },
                    /**		
       * Options for customizing the grid line colors, width, dashArray, border.
       * @type {object}
       * @alias ejRangeNavigator#labelSettings->higherLevel->gridLineStyle
       */
                    gridLineStyle: {
                        /**		
* Specifies the color of grid lines in higher level.
@default "#B5B5B5"
* @type {string}					
* @example 
* &lt;div id="RangeNavigator"&gt;&lt;/div&gt; 
              * &lt;script&gt;
              *	$("#container").ejRangeNavigator({
              *    labelSettings:{higherLevel:{ gridLineStyle :{color:"#ff0000"}}},
              *    });
* @alias ejRangeNavigator#labelSettings->higherLevel->gridLineStyle->color
*/
                        color: "#B5B5B5",
                        /**		
* Specifies the width of grid lines in higher level.
@default "#B5B5B5"
* @type {string}				
* @example 
  * &lt;div id="RangeNavigator"&gt;&lt;/div&gt; 
              * &lt;script&gt;
              *	$("#container").ejRangeNavigator({
              *    labelSettings:{higherLevel:{ gridLineStyle :{width:1}}},
              *    });
              * &lt;/script&gt; 
* @alias ejRangeNavigator#labelSettings->higherLevel->gridLineStyle->width
*/
                        width: 1,
                        /**		
* Specifies the dashArray of grid lines in higher level.
@default "20 5 0"
* @type {string}					
* @example 
  * &lt;div id="RangeNavigator"&gt;&lt;/div&gt; 
              * &lt;script&gt;
              *	$("#container").ejRangeNavigator({
              *    labelSettings:{higherLevel:{ gridLineStyle :{dashArray:"20 10 5"}}},
              *    });
              * &lt;/script&gt; 
* @alias ejRangeNavigator#labelSettings->higherLevel->gridLineStyle->dashArray
*/
                        dashArray: "20 5 0"
                    },

                    /**		
       * Options for customizing the border of grid lines in higher level.
       * @type {object}
        * @alias ejRangeNavigator#labelSettings->higherLevel->border
       */
                    border:
                    {
                        /**		
* Specifies the border color of grid lines.
@default "transparent"
* @type {string}					
 * @example 
  * &lt;div id="RangeNavigator"&gt;&lt;/div&gt; 
              * &lt;script&gt;
              *	$("#container").ejRangeNavigator({
              *   labelSettings:{higherLevel:{ border :{color:"#ff0000"}}},
              *    });
              * &lt;/script&gt;
* @alias ejRangeNavigator#labelSettings->higherLevel->border->color
*/
                        color: "transparent",
                        /**		
* Specifies the border width of grid lines.
@default "0.5"
* @type {string}					
 * @example 
  * &lt;div id="RangeNavigator"&gt;&lt;/div&gt; 
              * &lt;script&gt;
              *	$("#container").ejRangeNavigator({
              *    labelSettings:{higherLevel:{ border :{width:1}}},
              *    });
              * &lt;/script&gt;
* @alias ejRangeNavigator#labelSettings->higherLevel->border->width
*/
                        width: 0.5
                    },
                    /**		
* Specifies the fill color of higher level labels.
@default "transparent"
* @type {string}				
 * @example 
  * &lt;div id="RangeNavigator"&gt;&lt;/div&gt; 
              * &lt;script&gt;
              *	$("#container").ejRangeNavigator({
              *    labelSettings:{higherLevel:{ fill:"days"}},
              *    });
              * &lt;/script&gt;
* @alias ejRangeNavigator#labelSettings->higherLevel->fill
*/
                    fill: "transparent",
                    /**		
* Specifies the position of the labels in higher level.See {@link Position}
@default "top"
* @type {enum}					
 * @example 
  * &lt;div id="RangeNavigator"&gt;&lt;/div&gt; 
              * &lt;script&gt;
              *	$("#container").ejRangeNavigator({
              *    labelSettings:{higherLevel:{ position:"bottom"}},
              *    });
              * &lt;/script&gt;
* @alias ejRangeNavigator#labelSettings->higherLevel->position
*/
                    position: "top",
                    /**		
* Toggles the visibility of higher level labels.
@default true
* @type {boolean}					
 * @example 
  * &lt;div id="RangeNavigator"&gt;&lt;/div&gt; 
              * &lt;script&gt;
              *	$("#container").ejRangeNavigator({
              *    labelSettings:{higherLevel:{ visible:false}},
              *    });
              * &lt;/script&gt;
* @alias ejRangeNavigator#labelSettings->higherLevel->visible
*/
                    visible: true,
                    /**		
* Specifies the position of the labels to render either inside or outside of plot area. See {@link LabelPlacement}
@default "outside"
* @type {enum}				
 * @example 
  * &lt;div id="RangeNavigator"&gt;&lt;/div&gt; 
              * &lt;script&gt;
              *	$("#container").ejRangeNavigator({
              *    labelSettings:{higherLevel:{ labelPlacement:"inside"}},
              *    });
              * &lt;/script&gt;
* @alias ejRangeNavigator#labelSettings->higherLevel->labelPlacement
*/
                    labelPlacement: "outside"
                },
                /**		
      * Options for customizing the labels in lower level.
      * @type {object}
       * @alias ejRangeNavigator#labelSettings->lowerLevel
      */
                lowerLevel: {
                    /**		
* Specifies the intervalType of the labels in lower level.See {@link IntervalType}
@default "years"
* @type {enum}				
 * @example 
  * &lt;div id="RangeNavigator"&gt;&lt;/div&gt; 
              * &lt;script&gt;
              *	$("#container").ejRangeNavigator({
              *    labelSettings:{lowerLevel:{ intervalType:"days"}},
              *    });
              * &lt;/script&gt;  
* @alias ejRangeNavigator#labelSettings->lowerLevel->intervalTypes
*/
                    intervalType: 'quarters',
                     /**		
      * Options for customizing the style of labels.
      * @type {object}
                    * @alias ejRangeNavigator#labelSettings->lowerLevel->style
                    */
                    style: {
                        /**		
    * Options for customizing the font of labels.
    * @type {object}
                  * @alias ejRangeNavigator#labelSettings->lowerLevel->style->font
                  */
                        font: {
                            /**		
* Specifies the color of labels. Label text render in this specified color.
@default "black"
* @type {string}				
 * @example 
  * &lt;div id="RangeNavigator"&gt;&lt;/div&gt; 
              * &lt;script&gt;
              *	$("#container").ejRangeNavigator({
              *    labelSettings:{lowerLevel:{ style:{font:{color: "red"}}}},
              *    });
              * &lt;/script&gt;  
* @alias ejRangeNavigator#labelSettings->lowerLevel->style->font->color
*/
                            color: "black",
                            /**		
* Specifies the font family of labels. Label text render in this specified font family.
@default "Segoe UI"
* @type {string}				
 * @example 
  * &lt;div id="RangeNavigator"&gt;&lt;/div&gt; 
              * &lt;script&gt;
              *	$("#container").ejRangeNavigator({
              *    labelSettings:{lowerLevel:{ style:{font:{fontFamily: "Algerian"}}}},
              *    });
              * &lt;/script&gt;  
* @alias ejRangeNavigator#labelSettings->lowerLevel->style->font->fontFamily
*/
                            fontFamily: "Segoe UI",
                            /**		
* Specifies the font style of labels. Label text render in this specified font style.
@default "Normal"
* @type {string}				
* @example 
 * &lt;div id="RangeNavigator"&gt;&lt;/div&gt; 
             * &lt;script&gt;
             *	$("#container").ejRangeNavigator({
             *    labelSettings:{lowerLevel:{ style:{font:{fontStyle: "Italic"}}}},
             *    });
             * &lt;/script&gt;  
* @alias ejRangeNavigator#labelSettings->lowerLevel->style->font->fontStyle
*/
                            fontStyle: "Normal",
                            /**		
* Specifies the size of labels. Label text render in this specified size.
@default "12px"
* @type {string}				
* @example 
* &lt;div id="RangeNavigator"&gt;&lt;/div&gt; 
            * &lt;script&gt;
            *	$("#container").ejRangeNavigator({
            *    labelSettings:{lowerLevel:{ style:{font:{size: "14px"}}}},
            *    });
            * &lt;/script&gt;  
* @alias ejRangeNavigator#labelSettings->lowerLevel->style->font->size
*/
                            size: "12px",
                            /**		
* Specifies the opacity of labels. Label text render in this specified opacity.
@default "12px"
* @type {string}				
* @example 
* &lt;div id="RangeNavigator"&gt;&lt;/div&gt; 
            * &lt;script&gt;
            *	$("#container").ejRangeNavigator({
            *    labelSettings:{lowerLevel:{ style:{font:{opacity: "14px"}}}},
            *    });
            * &lt;/script&gt;  
* @alias ejRangeNavigator#labelSettings->lowerLevel->style->font->opacity
*/
                            opacity: 1,
                            /**		
* Specifies the font weight of labels. Label text render in this specified font weight.
@default "regular"
* @type {string}				
* @example 
* &lt;div id="RangeNavigator"&gt;&lt;/div&gt; 
           * &lt;script&gt;
           *	$("#container").ejRangeNavigator({
           *    labelSettings:{lowerLevel:{ style:{font:{fontWeight: "regular"}}}},
           *    });
           * &lt;/script&gt;  
* @alias ejRangeNavigator#labelSettings->lowerLevel->style->font->fontWeight
*/
                            fontWeight: "regular"
                        },
                        /**		
* Specifies the horizontal text alignment of the text in label.
@default "center"
* @type {string}				
* @example 
* &lt;div id="RangeNavigator"&gt;&lt;/div&gt; 
           * &lt;script&gt;
           *	$("#container").ejRangeNavigator({
           *    labelSettings:{lowerLevel:{ style:{horizontalAlignment: "left"}}},
           *    });
           * &lt;/script&gt;  
* @alias ejRangeNavigator#labelSettings->lowerLevel->style->horizontalAlignment
*/
                        horizontalAlignment : "center"
                    },
                    /**		
      * Options for customizing the grid lines in lower level.
      * @type {object}
       * @alias ejRangeNavigator#labelSettings->lowerLevel->gridLineStyle
      */
                    gridLineStyle:
                    {  /**		
* Specifies the color of grid lines in lower level.
@default "#B5B5B5"
* @type {string}					
* @example 
  * &lt;div id="RangeNavigator"&gt;&lt;/div&gt; 
              * &lt;script&gt;
              *	$("#container").ejRangeNavigator({
              *    labelSettings:{lowerLevel:{ gridLineStyle :{color:"#ff0000"}}},
              *    });
              * &lt;/script&gt; 
* @alias ejRangeNavigator#labelSettings->lowerLevel->gridLineStyle->color
*/
                        color: "#B5B5B5",
                        /**		
* Specifies the width of grid lines in lower level.
@default "#B5B5B5"
* @type {string}					
* @example 
  * &lt;div id="RangeNavigator"&gt;&lt;/div&gt; 
              * &lt;script&gt;
              *	$("#container").ejRangeNavigator({
              *    labelSettings:{lowerLevel:{ gridLineStyle :{width:1}}},
              *    });
              * &lt;/script&gt; 
* @alias ejRangeNavigator#labelSettings->lowerLevel->gridLineStyle->width
*/
                        width: 1,
                        /**		
* Specifies the dashArray of gridLines in lowerLevel.
@default "20 5 0"
* @type {string}					
* @example 
  * &lt;div id="RangeNavigator"&gt;&lt;/div&gt; 
              * &lt;script&gt;
              *	$("#container").ejRangeNavigator({
              *    lowerLevel:{ gridLineStyle :{dashArray:"20 10 5"}}},
              *    });
              * &lt;/script&gt; 
* @alias ejRangeNavigator#labelSettings->lowerLevel->gridLineStyle->dashArray
*/
                        dashArray: ""
                    },
                    /**		
      * Options for customizing the border of grid lines in lower level.
      * @type {object}
     * @alias ejRangeNavigator#labelSettings->lowerLevel->border
      */
                    border:
                    {
                        /**		
* Specifies the border color of grid lines.
@default "transparent"
* @type {string}					
* @example 
  * &lt;div id="RangeNavigator"&gt;&lt;/div&gt; 
              * &lt;script&gt;
              *	$("#container").ejRangeNavigator({
              *    labelSettings:{lowerLevel:{ border :{color:"#ff0000"}}},
              *    });
              * &lt;/script&gt;
* @alias ejRangeNavigator#labelSettings->lowerLevel->border->color
*/
                        color: "transparent",
                        /**		
* Specifies the border width of grid lines.
@default "0.5"
* @type {string}					
* @example 
  * &lt;div id="RangeNavigator"&gt;&lt;/div&gt; 
              * &lt;script&gt;
              *	$("#container").ejRangeNavigator({
              *    labelSettings:{lowerLevel:{ border :{width:1}}},
              *    });
              * &lt;/script&gt;
* @alias ejRangeNavigator#labelSettings->lowerLevel->border->width
*/
                        width: 0.5
                    },
                    /**		
* Specifies the fill color of labels in lower level.
@default "transparent"
* @type {string}				
* @example 
  * &lt;div id="RangeNavigator"&gt;&lt;/div&gt; 
              * &lt;script&gt;
              *	$("#container").ejRangeNavigator({
              *   labelSettings:{lowerLevel:{ fill:"days"}},
              *    });
              * &lt;/script&gt;
* @alias ejRangeNavigator#labelSettings->lowerLevel->fill
*/
                    fill: "transparent",
                    /**		
* Specifies the position of the labels in lower level.See {@link Position}
@default "bottom"
* @type {enum}					
* @example 
  * &lt;div id="RangeNavigator"&gt;&lt;/div&gt; 
              * &lt;script&gt;
              *	$("#container").ejRangeNavigator({
              *    labelSettings:{lowerLevel:{ position:"bottom"}},
              *    });
              * &lt;/script&gt;
* @alias ejRangeNavigator#labelSettings->lowerLevel->position
*/
                    position: "bottom",
                    /**		
* Toggles the visibility of labels in lower level.
@default true
* @type {boolean}					
* @example 
  * &lt;div id="RangeNavigator"&gt;&lt;/div&gt; 
              * &lt;script&gt;
              *	$("#container").ejRangeNavigator({
              *    labelSettings:{lowerLevel:{ visible:false}},
              *    });
              * &lt;/script&gt;
* @alias ejRangeNavigator#labelSettings->lowerLevel->visible
*/

                    visible: true,
                    /**		
* Specifies the position of the labels to render either inside or outside of plot area. See {@link LabelPlacement}
@default "outside"
* @type {enum}				
* @example 
  * &lt;div id="RangeNavigator"&gt;&lt;/div&gt; 
              * &lt;script&gt;
              *	$("#container").ejRangeNavigator({
              *    labelSettings:{lowerLevel:{ labelPlacement:"inside"}},
              *    });
              * &lt;/script&gt;
* @alias ejRangeNavigator#labelSettings->lowerLevel->labelPlacement
*/
                    labelPlacement: "outside"
                }
            },
            /**		
     * Options for customizing the range navigator.
     * @type {object}
     */
            navigatorStyleSettings:/** @lends ejRangeNavigator# */  {
                /**		
* Specifies the color of the selected region in range navigator.
@default "#EFEFEF"
* @type {string}					
* @example 
  * &lt;div id="RangeNavigator"&gt;&lt;/div&gt; 
              * &lt;script&gt;
              *	$("#container").ejRangeNavigator({
              *    navigatorStyleSettings:{ selectedRegionColor:"#ff0000"}},
              *    });
              * &lt;/script&gt;  
* @alias ejRangeNavigator#navigatorStyleSettings->selectedRegionColor
*/
                selectedRegionColor: "#EFEFEF",
                /**		
* Specifies the color of the unselected region in range navigator.
@default "#5EABDE"
* @type {string}				
* @example 
  * &lt;div id="RangeNavigator"&gt;&lt;/div&gt; 
              * &lt;script&gt;
              *	$("#container").ejRangeNavigator({
              *    navigatorStyleSettings:{ unselectedRegionColor:"#ff0000"},
              *    });
              * &lt;/script&gt;  
* @alias ejRangeNavigator#navigatorStyleSettings->unselectedRegionColor
*/
                unselectedRegionColor: "#5EABDE",
                /**		
* Specifies the color of the thumb in range navigator.
@default "#2382C3"
* @type {string}			
* @example 
  * &lt;div id="RangeNavigator"&gt;&lt;/div&gt; 
              * &lt;script&gt;
              *	$("#container").ejRangeNavigator({
              *    navigatorStyleSettings:{ thumbColor:"#ff0000"},
              *    });
              * &lt;/script&gt;  
* @alias ejRangeNavigator#navigatorStyleSettings->thumbColor
*/
                thumbColor: "#2382C3",
                /**		
 * Specifies the radius of the thumb in range navigator.
 @default 10
 * @type {number}				
* @example 
  * &lt;div id="RangeNavigator"&gt;&lt;/div&gt; 
              * &lt;script&gt;
              *	$("#container").ejRangeNavigator({
              *    navigatorStyleSettings:{ thumbRadius:15},
              *    });
              * &lt;/script&gt;  
 * @alias ejRangeNavigator#navigatorStyleSettings->thumbRadius
 */
                thumbRadius: 10,
                /**		
 * Specifies the stroke color of the thumb in range navigator.
 @default "#303030"
 * @type {string}				
 * @example 
  * &lt;div id="RangeNavigator"&gt;&lt;/div&gt; 
              * &lt;script&gt;
              *	$("#container").ejRangeNavigator({
              *    navigatorStyleSettings:{ thumbStroke:"#ff0000"},
              *    });
              * &lt;/script&gt;  
 * @alias ejRangeNavigator#navigatorStyleSettings->thumbStroke
 */
                thumbStroke: "#303030",
                /**		
 * Specifies the background color of RangeNavigator.
 @default "#dddddd"
 * @type {string}				
* @example 
  * &lt;div id="RangeNavigator"&gt;&lt;/div&gt; 
              * &lt;script&gt;
              *	$("#container").ejRangeNavigator({
              *    navigatorStyleSettings:{ background:"#ff0000"},
              *    });
              * &lt;/script&gt;  
* @alias ejRangeNavigator#navigatorStyleSettings->background
 */
                background: "#dddddd",
                /**		
    * Options for customizing the border color and width of range navigator.
    * @type {object}
    * @alias ejRangeNavigator#navigatorStyleSettings->border
    */
                border:
                {
                    /**		
* Specifies the border color of range navigator.
@default "transparent"
* @type {string}					
* @example 
  * &lt;div id="RangeNavigator"&gt;&lt;/div&gt; 
              * &lt;script&gt;
              *	$("#container").ejRangeNavigator({
              *    navigatorStyleSettings:{ border:{color:"#ff0000"}},
              *    });
              * &lt;/script&gt;  
* @alias ejRangeNavigator#navigatorStyleSettings->border->color
*/
                    color: "transparent",
                    /**		
* Specifies the border width of range navigator.
@default 0.5
* @type {number}

  * @example 
  * &lt;div id="RangeNavigator"&gt;&lt;/div&gt; 
              * &lt;script&gt;
              *	$("#container").ejRangeNavigator({
              *    navigatorStyleSettings:{ border:{width:1}},
              *    });
              * &lt;/script&gt;  
* @alias ejRangeNavigator#navigatorStyleSettings->border->width
*/
                    width: 0.5,
                    /**		
* Specifies the dash array of range navigator.
@default null
* @type {string}
  * @example 
  * &lt;div id="RangeNavigator"&gt;&lt;/div&gt; 
              * &lt;script&gt;
              *	$("#container").ejRangeNavigator({
              *    navigatorStyleSettings:{ border:{dashArray:"2,3"}},
              *    });
              * &lt;/script&gt;  
* @alias ejRangeNavigator#navigatorStyleSettings->border->dashArray
*/
                    dashArray: ""
                },
                /**		
 * Specifies the opacity of RangeNavigator.
 @default 1
 * @type {number}
  * @example 
  * &lt;div id="RangeNavigator"&gt;&lt;/div&gt; 
              * &lt;script&gt;
              *	$("#container").ejRangeNavigator({
              *    navigatorStyleSettings:{ opacity:0.5},
              *    });
              * &lt;/script&gt;  
 * @alias ejRangeNavigator#navigatorStyleSettings->opacity
 */
                opacity: 1,
                /**		
    * Options for customizing the major grid lines.
    * @type {object}
     * @alias ejRangeNavigator#navigatorStyleSettings->majorGridLineStyle
    */
                majorGridLineStyle:
                {
                    /**		
 * Specifies the color of major grid lines in RangeNavigator.
 @default "#B5B5B5"
 * @type {string}				
 * @example 
  * &lt;div id="RangeNavigator"&gt;&lt;/div&gt; 
              * &lt;script&gt;
              *	$("#container").ejRangeNavigator({
              *    navigatorStyleSettings:{ majorGridLineStyle:{color:"#ff0000"}},
              *    });
              * &lt;/script&gt; 
   * @alias ejRangeNavigator#navigatorStyleSettings->majorGridLineStyle->color
 */
                    color: "#B5B5B5",
                    /**		
* Toggles the visibility of major grid lines.
@default true
* @type {boolean}				
* @example 
  * &lt;div id="RangeNavigator"&gt;&lt;/div&gt; 
              * &lt;script&gt;
              *	$("#container").ejRangeNavigator({
              *    navigatorStyleSettings:{ majorGridLineStyle:{visible:false}},
              *    });
              * &lt;/script&gt; 
  * @alias ejRangeNavigator#navigatorStyleSettings->majorGridLineStyle->visible
*/
                    visible: true
                },
                /**		
    * Options for customizing the minor grid lines.
    * @type {object}
       * @alias ejRangeNavigator#navigatorStyleSettings->minorGridLineStyle
    */
                minorGridLineStyle:
                {
                    /**		
 * Specifies the color of minor grid lines in RangeNavigator.
 @default "#B5B5B5"
 * @type {string}				
 * @example 
 * //Gets or sets color of majorGridLines in RangeNavigator, after initialization:<br>
 *	//Gets the color of majorGridLines in RangeNavigator
 *	$("#container").ejRangeNavigator("option", "navigatorStyleSettings.majorGridLineStyle.color");<br>			
 *	//Sets the color of majorGridLines in RangeNavigator
 * @alias ejRangeNavigator#navigatorStyleSettings->minorGridLineStyle->color
 */
                    color: "#B5B5B5",
                    /**		
* Toggles the visibility of minor grid lines.
@default true
* @type {boolean}					
 * @example 
  * &lt;div id="RangeNavigator"&gt;&lt;/div&gt; 
              * &lt;script&gt;
              *	$("#container").ejRangeNavigator({
              *    navigatorStyleSettings:{ majorGridLineStyle:{visible:false}},
              *    });
              * &lt;/script&gt;  
* @alias ejRangeNavigator#navigatorStyleSettings->minorGridLineStyle->visible
*/
                    visible: true
                },
            },

            /**     
                         * Fires after range navigator is loaded.
                         * @event
                         * @name ejRangeNavigator#loaded	
                         * @param {Object} argument.Data parameters from RangeNavigator     
                         * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
                         * @param {object}  argument.model returns the RangeNavigator model
                         * @param {string}  argument.type returns the name of the event
                         * @example 
                         * //loaded event for chart
                         * $("#container").ejRangeNavigator({
                         *    loaded: function (args) {}
                         * });
                         */
            loaded: "",
            /**     
                        * Fires on load of range navigator.
                        * @event
                        * @name ejRangeNavigator#load	
                        * @param {Object} argument.Data parameters from RangeNavigator     
                        * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
                        * @param {object}  argument.model returns the RangeNavigator model
                        * @param {string}  argument.type returns the name of the event
                        * @example 
                        * //load event for chart
                        * $("#container").ejRangeNavigator({
                        *    load: function (args) {}
                        * });
                        */
            load: "",
            /**     
                       * Fires on changing the range of range navigator.
                       * @event
                       * @name ejRangeNavigator#rangeChanged	
                       * @param {Object} argument.Data parameters from RangeNavigator     
                       * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
                       * @param {object}  argument.model returns the RangeNavigator model
                       * @param {string}  argument.type returns the name of the event
                       * @example 
                       * //rangeChanged event for chart
                       * $("#container").ejRangeNavigator({
                       *    rangeChanged: function (args) {}
                       * });
                       */
            rangeChanged: ""

        },
        /**
          * Rangenavigator contains the default values. dataTypes property is used to set the default value to rangenavigator
          * @private
          */
        dataTypes: {
          //  theme:"string",
           // padding:"string",
            enableAutoResizing:"boolean",
            allowSnapping:"boolean",
          //  size:{
          //      width:"string",
          //      height:"string"
          //  },
            dataSource:{data:"data",
			query:"data"
			},
            tooltipSettings: {
                visible:"boolean",
                labelFormat:"string",
                tooltipDisplayMode:"string",
            },
            
            zoomPosition: "string",
            zoomFactor: "string",
           
            selectedData: "string",
            enableDeferredUpdate: "boolean",
            series: "array",
            //seriesSettings: "string",
            labelSettings: {
                higherLevel: {
                    intervalType: 'string',
                    position: "string",
                    visible: "boolean",
                    labelPlacement: "string"
                },
                lowerLevel: {
                    intervalType: 'string',
                    position: "string",
                    visible: "boolean",
                    labelPlacement:"string"
                }
            },
            navigatorStyleSettings: {
                thumbRadius:"number",
            },
            //	loaded: "string",
            //rangeChanged: "string" 
		 
        },
        observables: ["dataSource"],

        _themes: {
            flatlight: {
                tooltipSettings: {
                    backgroundColor: "#303030",
                    font: { color: '#FFFFFF', family: 'Segoe UI', style: 'Normal', size: '10px', opacity: 1 ,weight: 'regular' },
                },
                labelSettings: {
                    style: { font: { color: '#1E1E1E', family: 'Segoe UI', style: 'Normal', size: '13px', opacity: 1, weight: 'regular' } },
                    higherLevel: {                        
                        style: { font: { color: '#1E1E1E', family: 'Segoe UI', style: 'Normal', size: '13px', opacity: 1, weight: 'regular' }, horizontalAlignment: "left" },
                        border: { color: "transparent", width: 0},
                        fill: "transparent",
                        gridLineStyle: { color: "#B5B5B5", width:1, dashArray:"15 5 0" },
                    },
                    lowerLevel: {
                        style: { font: { color: '#1E1E1E', family: 'Segoe UI', style: 'Normal', size: '13px', opacity: 1, weight: 'regular' } },
                        border: { color: "transparent", width: 0 },
                        fill: "transparent",
                        gridLineStyle: { color: "transparent", width:0 },
                    },
                },
                navigatorStyleSettings: {
                    selectedRegionColor: "#e5e5e5",
                    unselectedRegionColor: "#5EABDE",
                    thumbColor: "#2382C3",
                    thumbRadius: 10,
                    thumbStroke: "#303030",
                    background: "#EFEFEF",
                    border: { color: "#606262", width: 0 },
                    opacity:1,
                    majorGridLineStyle: { color: "#8c8c8c", visible: true },
                    minorGridLineStyle: { color: "#8c8c8c", visible: true },
                },
            },

            azurelight: {
                tooltipSettings: {
                    backgroundColor: "#303030",
                    font: { color: '#FFFFFF', family: 'Segoe UI', style: 'Normal', size: '10px', opacity: 1, weight: 'regular' },
                },
                labelSettings: {
                    style: { font: { color: '#1E1E1E', family: 'Segoe UI', style: 'Normal', size: '13px', opacity: 1, weight: 'regular' } },
                    higherLevel: {
                        style: { font: { color: '#1E1E1E', family: 'Segoe UI', style: 'Normal', size: '13px', opacity: 1, weight: 'regular' }, horizontalAlignment: "left" },
                        border: { color: "transparent", width: 0 },
                        fill: "transparent",
                        gridLineStyle: { color: "#B5B5B5", width: 1,dashArray: "15 5 0" },
                    },
                    lowerLevel: {
                        style: { font: { color: '#1E1E1E', family: 'Segoe UI', style: 'Normal', size: '13px', opacity: 1, weight: 'regular' } },
                        border: { color: "transparent", width: 0 },
                        fill: "transparent",
                        gridLineStyle: { color: "transparent", width: 0 },
                    },
                },
                navigatorStyleSettings: {
                    selectedRegionColor: "#e5e5e5",
                    unselectedRegionColor: "#5EABDE",
                    thumbColor: "#2382C3",
                    thumbRadius: 10,
                    thumbStroke: "#303030",
                    background: "#EFEFEF",
                    border: { color: "#606262", width: 0 },
                    opacity: 1,
                    majorGridLineStyle: { color: "#8c8c8c", visible: true },
                    minorGridLineStyle: { color: "#8c8c8c", visible: true },
                },
            },

            limelight: {
                tooltipSettings: {
                    backgroundColor: "#303030",
                    font: { color: '#FFFFFF', family: 'Segoe UI', style: 'Normal', size: '10px', opacity: 1, weight: 'regular' },
                },
                labelSettings: {
                    style: { font: { color: '#1E1E1E', family: 'Segoe UI', style: 'Normal', size: '13px', opacity: 1, weight: 'regular' } },
                    higherLevel: {
                        style: { font: { color: '#1E1E1E', family: 'Segoe UI', style: 'Normal', size: '13px', opacity: 1, weight: 'regular' }, horizontalAlignment: "left" },
                        border: { color: "transparent", width: 0 },
                        fill: "transparent",
                        gridLineStyle: { color: "#B5B5B5", width: 1, dashArray: "15 5 0" },
                    },
                    lowerLevel: {
                        style: { font: { color: '#1E1E1E', family: 'Segoe UI', style: 'Normal', size: '13px', opacity: 1, weight: 'regular' } },
                        border: { color: "transparent", width: 0 },
                        fill: "transparent",
                        gridLineStyle: { color: "transparent", width: 0 },
                    },
                },
                navigatorStyleSettings: {
                    selectedRegionColor: "#e5e5e5",
                    unselectedRegionColor: "#A9CA44 ",
                    thumbColor: "#AECF49",
                    thumbRadius: 10,
                    thumbStroke: "#303030",
                    background: "#EFEFEF",
                    border: { color: "#606262", width: 0 },
                    opacity: 1,
                    majorGridLineStyle: { color: "#8c8c8c", visible: true },
                    minorGridLineStyle: { color: "#8c8c8c", visible: true },
                },
            },

            saffronlight: {
                tooltipSettings: {
                    backgroundColor: "#303030",
                    font: { color: '#FFFFFF', family: 'Segoe UI', style: 'Normal', size: '10px', opacity: 1, weight: 'regular' },
                },
                labelSettings: {
                    style: { font: { color: '#1E1E1E', family: 'Segoe UI', style: 'Normal', size: '13px', opacity: 1, weight: 'regular' } },
                    higherLevel: {
                        style: { font: { color: '#1E1E1E', family: 'Segoe UI', style: 'Normal', size: '13px', opacity: 1, weight: 'regular' }, horizontalAlignment: "left" },
                        border: { color: "transparent", width: 0 },
                        fill: "transparent",
                        gridLineStyle: { color: "#B5B5B5", width: 1, dashArray: "15 5 0" },
                    },
                    lowerLevel: {
                        style: { font: { color: '#1E1E1E', family: 'Segoe UI', style: 'Normal', size: '13px', opacity: 1, weight: 'regular' } },
                        border: { color: "transparent", width: 0 },
                        fill: "transparent",
                        gridLineStyle: { color: "transparent", width: 0 },
                    },
                },
                navigatorStyleSettings: {
                    selectedRegionColor: "#e5e5e5",
                    unselectedRegionColor: "#FAA113",
                    thumbColor: "#F9920B",
                    thumbRadius: 10,
                    thumbStroke: "#303030",
                    background: "#EFEFEF",
                    border: { color: "#606262", width: 0 },
                    opacity: 1,
                    majorGridLineStyle: { color: "#8c8c8c", visible: true },
                    minorGridLineStyle: { color: "#8c8c8c", visible: true },
                },
            },

            gradientlight: {
                tooltipSettings: {
                    backgroundColor: "#303030",
                    font: { color: '#FFFFFF', family: 'Segoe UI', style: 'Normal', size: '10px', opacity: 1, weight: 'regular' },
                },
                labelSettings: {
                    style: { font: { color: '#1E1E1E', family: 'Segoe UI', style: 'Normal', size: '13px', opacity: 1, weight: 'regular' } },
                    higherLevel: {
                        style: { font: { color: '#1E1E1E', family: 'Segoe UI', style: 'Normal', size: '13px', opacity: 1, weight: 'regular' }, horizontalAlignment: "left" },
                        border: { color: "transparent", width: 0 },
                        fill: "transparent",
                        gridLineStyle: { color: "#B5B5B5", width: 1, dashArray: "15 5 0" },
                    },
                    lowerLevel: {
                        style: { font: { color: '#1E1E1E', family: 'Segoe UI', style: 'Normal', size: '13px', opacity: 1, weight: 'regular' } },
                        border: { color: "transparent", width: 0 },
                        fill: "transparent",
                        gridLineStyle: { color: "transparent", width: 0 },
                    },
                },
                navigatorStyleSettings: {
                    selectedRegionColor: [{ color: "#bbbbbb", colorStop: "0%" }, { color: "#efefef", colorStop: "15%" }, { color: "#bbbbbb", colorStop: "85%" }, { color: "#efefef", colorStop: "100%" }],
                    unselectedRegionColor: [{ color: "#67C1DC", colorStop: "50%" }, { color: "#3D93AA", colorStop: "100%" }],
                    thumbColor: [{ color: "#67C1DC", colorStop: "50%" }, { color: "#3D93AA", colorStop: "100%" }],
                    thumbRadius: 10,
                    thumbStroke: "#303030",
                    background: "#EFEFEF",
                    border: { color: "#606262", width: 0 },
                    opacity: 1,
                    majorGridLineStyle: { color: "#8c8c8c", visible: true },
                    minorGridLineStyle: { color: "#8c8c8c", visible: true },
                },
            },

            gradientazure: {
                tooltipSettings: {
                    backgroundColor: "#303030",
                    font: { color: '#FFFFFF', family: 'Segoe UI', style: 'Normal', size: '10px', opacity: 1, weight: 'regular' },
                },
                labelSettings: {
                    style: { font: { color: '#1E1E1E', family: 'Segoe UI', style: 'Normal', size: '13px', opacity: 1, weight: 'regular' } },
                    higherLevel: {
                        style: { font: { color: '#1E1E1E', family: 'Segoe UI', style: 'Normal', size: '13px', opacity: 1, weight: 'regular' }, horizontalAlignment: "left" },
                        border: { color: "transparent", width: 0 },
                        fill: "transparent",
                        gridLineStyle: { color: "#B5B5B5", width: 1, dashArray: "15 5 0" },
                    },
                    lowerLevel: {
                        style: { font: { color: '#1E1E1E', family: 'Segoe UI', style: 'Normal', size: '13px', opacity: 1, weight: 'regular' } },
                        border: { color: "transparent", width: 0 },
                        fill: "transparent",
                        gridLineStyle: { color: "transparent", width: 0 },
                    },
                },
                navigatorStyleSettings: {
                    selectedRegionColor: [{ color: "#bbbbbb", colorStop: "0%" }, { color: "#efefef", colorStop: "15%" }, { color: "#bbbbbb", colorStop: "85%" }, { color: "#efefef", colorStop: "100%" }],
                    unselectedRegionColor: [{ color: "#67C1DC", colorStop: "50%" }, { color: "#3D93AA", colorStop: "100%" }],
                    thumbColor: [{ color: "#67C1DC", colorStop: "50%" }, { color: "#3D93AA", colorStop: "100%" }],
                    thumbRadius: 10,
                    thumbStroke: "#303030",
                    background: "#EFEFEF",
                    border: { color: "#606262", width: 0 },
                    opacity: 1,
                    majorGridLineStyle: { color: "#8c8c8c", visible: true },
                    minorGridLineStyle: { color: "#8c8c8c", visible: true },
                },
            },

            gradientlime: {
                tooltipSettings: {
                    backgroundColor: "#303030",
                    font: { color: '#FFFFFF', family: 'Segoe UI', style: 'Normal', size: '10px', opacity: 1, weight: 'regular' },
                },
                labelSettings: {
                    style: { font: { color: '#1E1E1E', family: 'Segoe UI',style: 'Normal', size: '13px', opacity: 1, weight: 'regular' } },
                    higherLevel: {
                        style: { font: { color: '#1E1E1E', family: 'Segoe UI', style: 'Normal', size: '13px', opacity: 1, weight: 'regular' }, horizontalAlignment: "left" },
                        border: { color: "transparent", width: 0 },
                        fill: "transparent",
                        gridLineStyle: { color: "#B5B5B5", width: 1, dashArray: "15 5 0" },
                    },
                    lowerLevel: {
                        style: { font: { color: '#1E1E1E', family: 'Segoe UI', style: 'Normal', size: '13px', opacity: 1, weight: 'regular' } },
                        border: { color: "transparent", width: 0 },
                        fill: "transparent",
                        gridLineStyle: { color: "transparent", width: 0 },
                    },
                },
                navigatorStyleSettings: {
                    selectedRegionColor: [{ color: "#bbbbbb", colorStop: "0%" }, { color: "#efefef", colorStop: "15%" }, { color: "#bbbbbb", colorStop: "85%" }, { color: "#efefef", colorStop: "100%" }],
                    unselectedRegionColor: [{ color: "#A5C14A", colorStop: "50%" }, { color: "#738B1F", colorStop: "100%" }],
                    thumbColor: [{ color: "#A5C14A", colorStop: "50%" }, { color: "#738B1F", colorStop: "100%" }],
                    thumbRadius: 10,
                    thumbStroke: "#303030",
                    background: "#EFEFEF",
                    border: { color: "#606262", width: 0 },
                    opacity: 1,
                    majorGridLineStyle: { color: "#8c8c8c", visible: true },
                    minorGridLineStyle: { color: "#8c8c8c", visible: true },
                },
            },

            gradientsaffron: {
                tooltipSettings: {
                    backgroundColor: "#303030",
                    font: { color: '#FFFFFF', family: 'Segoe UI', style: 'Normal', size: '10px', opacity: 1, weight: 'regular' },
                },
                labelSettings: {
                    style: { font: { color: '#1E1E1E', family: 'Segoe UI', style: 'Normal', size: '13px', opacity: 1, weight: 'regular' } },
                    higherLevel: {
                        style: { font: { color: '#1E1E1E', family: 'Segoe UI', style: 'Normal', size: '13px', opacity: 1, weight: 'regular' }, horizontalAlignment: "left" },
                        border: { color: "transparent", width: 0 },
                        fill: "transparent",
                        gridLineStyle: { color: "#B5B5B5", width: 1, dashArray: "15 5 0" },
                    },
                    lowerLevel: {
                        style: { font: { color: '#1E1E1E', family: 'Segoe UI', style: 'Normal', size: '13px', opacity: 1, weight: 'regular' } },
                        border: { color: "transparent", width: 0 },
                        fill: "transparent",
                        gridLineStyle: { color: "transparent", width: 0 },
                    },
                },
                navigatorStyleSettings: {
                    selectedRegionColor: [{ color: "#bbbbbb", colorStop: "0%" }, { color: "#efefef", colorStop: "15%" }, { color: "#bbbbbb", colorStop: "85%" }, { color: "#efefef", colorStop: "100%" }],
                    unselectedRegionColor: [{ color: "#FEB75B", colorStop: "50%" }, { color: "#ED7E16", colorStop: "100%" }],
                    thumbColor: [{ color: "#FEB75B", colorStop: "50%" }, { color: "#ED7E16", colorStop: "100%" }],
                    thumbRadius: 10,
                    thumbStroke: "#303030",
                    background: "#EFEFEF",
                    border: { color: "#606262", width: 0 },
                    opacity: 1,
                    majorGridLineStyle: { color: "#8c8c8c", visible: true },
                    minorGridLineStyle: { color: "#8c8c8c", visible: true },
                },
            },

            flatdark: {
                tooltipSettings: {
                    backgroundColor: "#FFFFFF",
                    font: { color: '#1E1E1E', family: 'Segoe UI', style: 'Normal', size: '10px', opacity: 1, weight: 'regular' },
                },
                labelSettings: {
                    style: { font: { color: '#FFFFFF', family: 'Segoe UI', style: 'Regular', size: '13px', opacity: 1, weight: 'regular' } },
                    higherLevel: {                        
                        style: { font: { color: '#FFFFFF', family: 'Segoe UI', style: 'Regular', size: '13px', opacity: 1, weight: 'regular' }, horizontalAlignment: "left" },
                        border: { color: "transparent", width: 0 },
                        fill: "transparent",
                        gridLineStyle: { color: "#FFFFFF", width: 1, dashArray: "15 5 0" },
                    },
                    lowerLevel: {
                        style: { font: { color: '#FFFFFF', family: 'Segoe UI', style: 'Regular', size: '13px', opacity: 1, weight: 'regular' } },
                        border: { color: "transparent", width: 0 },
                        fill: "transparent",
                        gridLineStyle: { color: "transparent", width: 0 },
                    },
                },
                navigatorStyleSettings: {
                    selectedRegionColor: "#353635",
                    unselectedRegionColor: "#5EABDE",
                    thumbColor: "#2382C3",
                    thumbRadius: 10,
                    thumbStroke: "#FFFFFF",
                    background: "#FFFFFF",
                    opacity: 0.08,
                    border: { color: "#FFFFFF", width:0 },
                    majorGridLineStyle: { color: "#FFFFFF", visible: true },
                    minorGridLineStyle: { color: "#FFFFFF", visible: true },
                },
            },

            azuredark: {
                tooltipSettings: {
                    backgroundColor: "#FFFFFF",
                    font: { color: '#1E1E1E', family: 'Segoe UI', style: 'Normal', size: '10px', opacity: 1, weight: 'regular' },
                },
                labelSettings: {
                    style: { font: { color: '#FFFFFF', family: 'Segoe UI', style: 'Regular', size: '13px', opacity: 1, weight: 'regular' } },
                    higherLevel: {
                        style: { font: { color: '#FFFFFF', family: 'Segoe UI', style: 'Regular', size: '13px', opacity: 1, weight: 'regular' }, horizontalAlignment: "left" },
                        border: { color: "transparent", width: 0 },
                        fill: "transparent",
                        gridLineStyle: { color: "#FFFFFF", width: 1, dashArray: "15 5 0" },
                    },
                    lowerLevel: {
                        style: { font: { color: '#FFFFFF', family: 'Segoe UI', style: 'Regular', size: '13px', opacity: 1, weight: 'regular' } },
                        border: { color: "transparent", width: 0 },
                        fill: "transparent",
                        gridLineStyle: { color: "transparent", width: 0 },
                    },
                },
                navigatorStyleSettings: {
                    selectedRegionColor: "#353635",
                    unselectedRegionColor: "#5EABDE",
                    thumbColor: "#2382C3",
                    thumbRadius: 10,
                    thumbStroke: "#FFFFFF",
                    background: "#FFFFFF",
                    opacity: 0.08,
                    border: { color: "#FFFFFF", width: 0 },
                    majorGridLineStyle: { color: "#FFFFFF", visible: true },
                    minorGridLineStyle: { color: "#FFFFFF", visible: true },
                },
            },

            limedark: {
                tooltipSettings: {
                    backgroundColor: "#FFFFFF",
                    font: { color: '#1E1E1E', family: 'Segoe UI', style: 'Normal', size: '10px', opacity: 1, weight: 'regular' },
                },
                labelSettings: {
                    style: { font: { color: '#FFFFFF', family: 'Segoe UI', style: 'Regular', size: '13px', opacity: 1, weight: 'regular' } },
                    higherLevel: {
                        style: { font: { color: '#FFFFFF', family: 'Segoe UI', style: 'Regular', size: '13px', opacity: 1, weight: 'regular' }, horizontalAlignment: "left" },
                        border: { color: "transparent", width: 0 },
                        fill: "transparent",
                        gridLineStyle: { color: "#FFFFFF", width: 1, dashArray: "15 5 0" },
                    },
                    lowerLevel: {
                        style: { font: { color: '#FFFFFF', family: 'Segoe UI', style: 'Regular', size: '13px', opacity: 1, weight: 'regular' } },
                        border: { color: "transparent", width: 0 },
                        fill: "transparent",
                        gridLineStyle: { color: "transparent", width: 0 },
                    },
                },
                navigatorStyleSettings: {
                    selectedRegionColor: "#353635",
                    unselectedRegionColor: "#A9CA44",
                    thumbColor: "#AECF49",
                    thumbRadius: 10,
                    thumbStroke: "#FFFFFF",
                    background: "#FFFFFF",
                    opacity: 0.08,
                    border: { color: "#FFFFFF", width: 0 },
                    majorGridLineStyle: { color: "#FFFFFF", visible: true },
                    minorGridLineStyle: { color: "#FFFFFF", visible: true },
                },
            },
            
            saffrondark: {
                tooltipSettings: {
                    backgroundColor: "#FFFFFF",
                    font: { color: '#1E1E1E', family: 'Segoe UI', style: 'Normal', size: '10px', opacity: 1, weight: 'regular' },
                },
                labelSettings: {
                    style: { font: { color: '#FFFFFF', family: 'Segoe UI',style: 'Regular', size: '13px', opacity: 1, weight: 'regular' } },
                    higherLevel: {
                        style: { font: { color: '#FFFFFF', family: 'Segoe UI', style: 'Regular', size: '13px', opacity: 1, weight: 'regular' }, horizontalAlignment: "left" },
                        border: { color: "transparent", width: 0 },
                        fill: "transparent",
                        gridLineStyle: { color: "#FFFFFF", width: 1, dashArray: "15 5 0" },
                    },
                    lowerLevel: {
                        style: { font: { color: '#FFFFFF', family: 'Segoe UI', style: 'Regular', size: '13px', opacity: 1, weight: 'regular' } },
                        border: { color: "transparent", width: 0 },
                        fill: "transparent",
                        gridLineStyle: { color: "transparent", width: 0 },
                    },
                },
                navigatorStyleSettings: {
                    selectedRegionColor: "#353635",
                    unselectedRegionColor: "#FAA113",
                    thumbColor: "#F9920B",
                    thumbRadius: 10,
                    thumbStroke: "#FFFFFF",
                    background: "#FFFFFF",
                    opacity: 0.08,
                    border: { color: "#FFFFFF", width: 0 },
                    majorGridLineStyle: { color: "#FFFFFF", visible: true },
                    minorGridLineStyle: { color: "#FFFFFF", visible: true },
                },
            },

            gradientdark: {
                tooltipSettings: {
                    backgroundColor: "#FFFFFF",
                    font: { color: '#1E1E1E', family: 'Segoe UI', style: 'Normal', size: '10px', opacity: 1, weight: 'regular' },
                },
                labelSettings: {
                    style: { font: { color: '#FFFFFF', family: 'Segoe UI', style: 'Regular', size: '13px', opacity: 1, weight: 'regular' } },
                    higherLevel: {
                        style: { font: { color: '#FFFFFF', family: 'Segoe UI',style: 'Regular', size: '13px', opacity: 1, weight: 'regular' }, horizontalAlignment: "left" },
                        border: { color: "transparent", width: 0 },
                        fill: "transparent",
                        gridLineStyle: { color: "#FFFFFF", width: 1, dashArray: "15 5 0" },
                    },
                    lowerLevel: {
                        style: { font: { color: '#FFFFFF', family: 'Segoe UI', style: 'Regular', size: '13px', opacity: 1, weight: 'regular' } },
                        border: { color: "transparent", width: 0 },
                        fill: "transparent",
                        gridLineStyle: { color: "transparent", width: 0 },
                    },
                },
                navigatorStyleSettings: {
                    selectedRegionColor: [{ color: "#0a0a0a", colorStop: "0%" }, { color: "#282828", colorStop: "15%" }, { color: "#282828", colorStop: "85%" }, { color: "#0a0a0a", colorStop: "100%" }],
                    unselectedRegionColor: [{ color: "#67C1DC", colorStop: "50%" }, { color: "#3D93AA", colorStop: "100%" }],
                    thumbColor: [{ color: "#67C1DC", colorStop: "50%" }, { color: "#3D93AA", colorStop: "100%" }],
                    thumbRadius: 10,
                    thumbStroke: "#FFFFFF",
                    background: "#FFFFFF",
                    opacity: 0.08,
                    border: { color: "#FFFFFF", width: 0 },
                    majorGridLineStyle: { color: "#FFFFFF", visible: true },
                    minorGridLineStyle: { color: "#FFFFFF", visible: true },
                },
            },

            gradientazuredark: {
                tooltipSettings: {
                    backgroundColor: "#FFFFFF",
                    font: { color: '#1E1E1E', family: 'Segoe UI', style: 'Normal', size: '10px', opacity: 1, weight: 'regular' },
                },
                labelSettings: {
                    style: { font: { color: '#FFFFFF', family: 'Segoe UI', style: 'Regular', size: '13px', opacity: 1, weight: 'regular' } },
                    higherLevel: {
                        style: { font: { color: '#FFFFFF', family: 'Segoe UI', style: 'Regular', size: '13px', opacity: 1, weight: 'regular' }, horizontalAlignment: "left" },
                        border: { color: "transparent", width: 0 },
                        fill: "transparent",
                        gridLineStyle: { color: "#FFFFFF", width: 1, dashArray: "15 5 0" },
                    },
                    lowerLevel: {
                        style: { font: { color: '#FFFFFF', family: 'Segoe UI', style: 'Regular', size: '13px', opacity: 1, weight: 'regular' } },
                        border: { color: "transparent", width: 0 },
                        fill: "transparent",
                        gridLineStyle: { color: "transparent", width: 0 },
                    },
                },
                navigatorStyleSettings: {
                    selectedRegionColor: [{ color: "#0a0a0a", colorStop: "0%" }, { color: "#282828", colorStop: "15%" }, { color: "#282828", colorStop: "85%" }, { color: "#0a0a0a", colorStop: "100%" }],
                    unselectedRegionColor: [{ color: "#67C1DC", colorStop: "50%" }, { color: "#3D93AA", colorStop: "100%" }],
                    thumbColor: [{ color: "#67C1DC", colorStop: "50%" }, { color: "#3D93AA", colorStop: "100%" }],
                    thumbRadius: 10,
                    thumbStroke: "#FFFFFF",
                    background: "#FFFFFF",
                    opacity: 0.08,
                    border: { color: "#FFFFFF", width: 0 },
                    majorGridLineStyle: { color: "#FFFFFF", visible: true },
                    minorGridLineStyle: { color: "#FFFFFF", visible: true },
                },
            },

            gradientlimedark: {
                tooltipSettings: {
                    backgroundColor: "#FFFFFF",
                    font: { color: '#1E1E1E', family: 'Segoe UI', style: 'Normal', size: '10px', opacity: 1, weight: 'regular' },
                },
                labelSettings: {
                    style: { font: { color: '#FFFFFF', family: 'Segoe UI', style: 'Regular', size: '13px', opacity: 1, weight: 'regular' } },
                    higherLevel: {
                        style: { font: { color: '#FFFFFF', family: 'Segoe UI', fontStyle: 'Regular', size: '13px', opacity: 1, weight: 'regular' }, horizontalAlignment: "left" },
                        border: { color: "transparent", width: 0 },
                        fill: "transparent",
                        gridLineStyle: { color: "#FFFFFF", width: 1, dashArray: "15 5 0" },
                    },
                    lowerLevel: {
                        style: { font: { color: '#FFFFFF', family: 'Segoe UI', style: 'Regular', size: '13px', opacity: 1,weight: 'regular' } },
                        border: { color: "transparent", width: 0 },
                        fill: "transparent",
                        gridLineStyle: { color: "transparent", width: 0 },
                    },
                },
                navigatorStyleSettings: {
                    selectedRegionColor: [{ color: "#0a0a0a", colorStop: "0%" }, { color: "#282828", colorStop: "15%" }, { color: "#282828", colorStop: "85%" }, { color: "#0a0a0a", colorStop: "100%" }],
                    unselectedRegionColor: [{ color: "#A5C14A", colorStop: "50%" }, { color: "#738B1F", colorStop: "100%" }],
                    thumbColor: [{ color: "#A5C14A", colorStop: "50%" }, { color: "#738B1F", colorStop: "100%" }],
                    thumbRadius: 10,
                    thumbStroke: "#FFFFFF",
                    background: "#FFFFFF",
                    opacity: 0.08,
                    border: { color: "#FFFFFF", width: 0 },
                    majorGridLineStyle: { color: "#FFFFFF", visible: true },
                    minorGridLineStyle: { color: "#FFFFFF", visible: true },
                },
            },

            gradientsaffrondark: {
                tooltipSettings: {
                    backgroundColor: "#FFFFFF",
                    font: { color: '#1E1E1E', family: 'Segoe UI',style: 'Normal', size: '10px', opacity: 1, weight: 'regular' },
                },
                labelSettings: {
                    style: { font: { color: '#FFFFFF', family: 'Segoe UI', style: 'Regular', size: '13px', opacity: 1, weight: 'regular' } },
                    higherLevel: {
                        style: { font: { color: '#FFFFFF', family: 'Segoe UI', style: 'Regular', size: '13px', opacity: 1, weight: 'regular' }, horizontalAlignment: "left" },
                        border: { color: "transparent", width: 0 },
                        fill: "transparent",
                        gridLineStyle: { color: "#FFFFFF", width: 1, dashArray: "15 5 0" },
                    },
                    lowerLevel: {
                        style: { font: { color: '#FFFFFF', family: 'Segoe UI', style: 'Regular', size: '13px', opacity: 1, weight: 'regular' } },
                        border: { color: "transparent", width: 0 },
                        fill: "transparent",
                        gridLineStyle: { color: "transparent", width: 0 },
                    },
                },
                navigatorStyleSettings: {
                    selectedRegionColor: [{ color: "#0a0a0a", colorStop: "0%" }, { color: "#282828", colorStop: "15%" }, { color: "#282828", colorStop: "85%" }, { color: "#0a0a0a", colorStop: "100%" }],
                    unselectedRegionColor: [{ color: "#FEB75B", colorStop: "50%" }, { color: "#ED7E16", colorStop: "100%" }],
                    thumbColor: [{ color: "#FEB75B", colorStop: "50%" }, { color: "#ED7E16", colorStop: "100%" }],
                    thumbRadius: 10,
                    thumbStroke: "#FFFFFF",
                    background: "#FFFFFF",
                    opacity: 0.08,
                    border: { color: "#FFFFFF", width: 0 },
                    majorGridLineStyle: { color: "#FFFFFF", visible: true },
                    minorGridLineStyle: { color: "#FFFFFF", visible: true },
                },
            },

        },
        /**
                * To configure the properties at runtime using SetModel		
                * @private
                */
        _setModel: function (options) {
            for (var prop in options) {
              //  this.disableAnimation();
                switch (prop) {                   
                    case "theme":
                        this.model._themeChanged = true;
                        this.model.theme = options[prop];
                        this._setTheme(this._themes, this.model.theme);
                    default:
                        $.extend(true, this.model, {}, options[prop]);
                }
            }
          //  $(this.svgDocument).empty();		  
            this.bindTo();
			this.renderNavigator(this);
			this._bindevents();
        },
        /**
        * destroy the RangeNavigator widget
		* @return jQuery
		* @example 
		* &lt;div id="container"&gt;&lt;/div&gt; <br> 
		* &lt;script&gt;
		* // Destroy RangeNavigator
		* var obj = $("#container").data("ejRangeNavigator");
		* obj.destroy(); // destroy the RangeNavigator
		* &lt;/script&gt;
		* @example 
		* &lt;div id="container"&gt;&lt;/div&gt; <br> 
		* &lt;script&gt;
		* $("#container").ejRangeNavigator("destroy");	
		* &lt;/script&gt;
         */
        _destroy: function () {
            this.element.empty().removeClass(this.model.cssClass);
        },


        _createSvg: function (element) {

            this.svgSupport = (window.SVGSVGElement) ? true : false;
            var bwr = jQuery.uaMatch(navigator.userAgent);
            this.brow = bwr.browser.toLowerCase();
            if (this.svgSupport) {
                //rendering object created
                this.renderer = new ej.EjSvgRender(this.element);
                //Creating svg document
                this.svgns = this.renderer.svgLink;
                this.svgDocument = this.renderer.svgObj;
            }
            else {
                //rendering object created
                this.renderer = new ej.EjVmlRender(this.element);
                //Creating VML document
                this.svgDocument = this.renderer.svgObj;
            }

        },

        _setTheme: function (jsonObj, themeName) {
            var name = themeName.toLowerCase();
            this.model = ej.copyObject({}, this.model, jsonObj[name]);
        },

        _setPositions: function () {

            var higherposition = this.model.labelSettings.higherLevel.position;
            var lowerposition = this.model.labelSettings.lowerLevel.position;
            var highervisible = this.model.labelSettings.higherLevel.visible;
            var lowervisible = this.model.labelSettings.lowerLevel.visible;
            var higherlabelplace = this.model.labelSettings.higherLevel.labelPlacement;
            var lowerlabelplace = this.model.labelSettings.lowerLevel.labelPlacement;
            //to initialize the positions of the sliders, labelbars and chart
            if (higherposition === "top" && lowerposition === "bottom") {
                this.centerPosition = this.minHighHeight + 1;
                this.centerHeight = this.newHeight - this.minHighHeight - this.minLowHeight;
                this.bottomPosition = this.minHighHeight + this.centerHeight + 1;                    
            }
            else if ((higherposition === "bottom" && lowerposition === "bottom")) {
                this.centerPosition = 0;
                this.centerHeight = this.newHeight - this.minHighHeight - this.minLowHeight;
                this.bottomPosition = this.minLowHeight + this.centerHeight + 1;

            }
            else if (higherposition === "bottom" && lowerposition === "top") {
                this.centerPosition = this.minLowHeight + 1;
                topPosition = 0;
                this.centerHeight = this.newHeight - this.minHighHeight - this.minLowHeight;
                this.bottomPosition = this.minLowHeight + this.centerHeight + 1;
            }
            else if (higherposition === "top" && lowerposition === "top") {
                this.centerPosition = this.minLowHeight + this.minHighHeight + 1;
                this.centerHeight = this.newHeight - this.minHighHeight - this.minLowHeight;
                this.bottomPosition = this.newHeight;
            }

            if (highervisible === false && lowervisible === false) {
                this.centerPosition = 0;
                this.centerHeight = this.newHeight;
                this.bottomPostion = this.newHeight;
            }
            else if (highervisible === false) {
                this.centerPosition = 0;
                this.centerHeight = this.newHeight - this.minLowHeight;
                this.bottomPosition = this.newHeight;
            }
            else if (lowervisible === false) {
                this.centerPosition = this.minHighHeight + 1;
                this.centerHeight = this.newHeight - this.minHighHeight;
                this.bottomPosition = this.minHighHeight + this.centerHeight + 1;
            }

            if (higherlabelplace === "outside" && lowerlabelplace === "outside") {
                this.sliderPosition = this.centerPosition;
                this.sliderHeight = this.centerHeight;
            }
            else if (higherlabelplace === "outside") {
                this.sliderPosition = this.centerPosition;
                this.sliderHeight = this.newHeight;
            }
            else if (lowerlabelplace === "outside") {
                this.sliderPosition = 0;
                this.sliderHeight = this.centerHeight + this.minHighHeight;
            }
            else {
                this.sliderPosition = 0;
                this.sliderHeight = this.newHeight;
            }
            
        },

        renderNavigator: function (evt) {
		    this.culture = Globalize.culture(this.model.locale);
            this._initializeVariables.call(this, this.model, this.model.series, this.model.seriesSettings);

            this._rootId = $(this.element).attr("id");
            this.leftPadding =parseFloat($("#"+this._rootId).css('padding-left'));

            //initializing minimum height and width for the control
            var fontsize = parseFloat(this.model.labelSettings.higherLevel.style.font.size);
            var type = this.model.valueType;
            if (type == "datetime") {
                this.minHighHeight = fontsize < 20 ?
                    20 :
                    fontsize + 2;
                fontsize = parseFloat(this.model.labelSettings.lowerLevel.style.font.size);
            }
            else
                this.minHighHeight = 0;
            this.minLowHeight = fontsize < 20 ?
                20 :
                fontsize + 2;
            if (type == "datetime") {
                var ht = ($(this.element).height() && this.newHeight==undefined) ? $(this.element).height() :this.newHeight==undefined?120:this.newHeight;
                this.minCenterHeight = parseFloat(ht) - parseFloat(this.minLowHeight) - parseFloat(this.minHighHeight);
            } else {
                var ht = ($(this.element).height()) && this.newHeight==undefined ? $(this.element).height() -this.minLowHeight : this.newHeight==undefined?100:this.newHeight;
                this.minCenterHeight = ht;
            }
            this.minHeight = this.minHighHeight + this.minCenterHeight + this.minLowHeight;
            this.padding = parseFloat(this.model.padding);
            
            
            //calculating the height and width of the control
            this.newWidth = this.model.sizeSettings.width ? this.model.sizeSettings.width : $(this.element).width();
            this.newHeight = this.model.sizeSettings.height ? this.model.sizeSettings.height : ($(this.element).height() && this.newHeight==undefined) > this.minHeight ? $(this.element).height() : this.minHeight;
            
            
            //initializing the higherlevel and lowerlevel collections
            this._higherTextNode = [];
            this._higherTotalValues = [];
            this._higherLineLeft = [];
            this._higherTextLeft = [];
            this._lowerTextNode = [];
            this._lowerTotalValues = [];
            this._lowerLineLeft = [];
            this._lowerTextLeft = [];

            //calculating positions
            this.centerPosition=0;
            this.bottomPosition = 0;
            this.centerHeight = 0;
            this.sliderPosition = 0;
            this.sliderHeight = 0;
            
            //set positions
            this._setPositions();

            //set initial width and height for the svg document
            $(this.svgDocument).empty();
            this.svgDocument.setAttribute("position", "relative");
            this.svgDocument.setAttribute("overflow", "visible");
            if (this.svgSupport) {
                this.svgDocument.setAttribute("height", this.newHeight);
                this.svgDocument.setAttribute("width", this.newWidth);
            }
            else {
                $(this.svgDocument).width(this.newWidth);
                $(this.svgDocument).height(this.newHeight);
            }
            this.model._size.width = this.newWidth;
            this.model._size.height = this.newHeight;
            var commonLoadedEventArgs = $.extend({}, ej.EjSvgRender.commonChartEventArgs);
            commonLoadedEventArgs.data = { model: this.model };
            this._trigger("loaded", commonLoadedEventArgs);
            if (this.model.theme != "")
                this._setTheme(this._themes, this.model.theme);
            //create gradient
            var uncolor = this.model.navigatorStyleSettings.unselectedRegionColor;
            this.unselectgrad = this.renderer.createGradientElement('unselected', uncolor, 150, 0, 150, 100, this.svgDocument);
            var navibackcolor = this.model.navigatorStyleSettings.background;
            this.naviback = this.renderer.createGradientElement('naviback', navibackcolor, 150, 0, 150, 100, this.svgDocument);
            var selcolor = this.model.navigatorStyleSettings.selectedRegionColor;
            this.selectedgrad = this.renderer.createGradientElement('selected', selcolor, 150, 0, 150, 100, this.svgDocument);
            var tbColor = this.model.navigatorStyleSettings.thumbColor;
            this.tbClr = this.renderer.createGradientElement('thumbClr', tbColor, 150, 0, 150, 100, this.svgDocument);
            
            var areaHeight;
            if (type == "numeric")
                areaHeight = this.newHeight - this.model.navigatorStyleSettings.border.width - this.minLowHeight;
            else
                areaHeight = this.newHeight - this.model.navigatorStyleSettings.border.width -this.minLowHeight -this.minHighHeight;
            //Creating border for navigator
            var naviBorderOptions = {
                'id':this._rootId+'_Border',
                'x':this.padding,
                'y': this.sliderPosition,
                'width': this.newWidth-this.padding*2,
                'height': this.sliderHeight,
                'fill': this.naviback,
                'stroke-width':this.model.navigatorStyleSettings.border.width,
                'stroke': this.model.navigatorStyleSettings.border.color,
                'opacity': this.model.navigatorStyleSettings.opacity,
                'stroke-dasharray': this.model.navigatorStyleSettings.border.dashArray,
                
            }

            this.renderer.drawRect(naviBorderOptions, this.svgDocument);

            //creating g tags to group higherlevel and lowerlevel elements(txt,line).
            this.higherLevel = this.renderer.createGroup({ 'id': this._rootId+'higherLevel' });
            this.lowerLevel = this.renderer.createGroup({ 'id': this._rootId + 'lowerLevel' });
            this.centerLevel = this.renderer.createGroup({ 'id': this._rootId + 'centerLelvel' });
            this.higherLevel.height = this.minHighHeight;
            this.lowerLevel.height = this.minLowHeight;

            //creating g tags to group tooltipSettings
            if (this.model.tooltipSettings.visible && this.svgSupport) {
                this.leftTooltip = this.renderer.createGroup({ 'id': this._rootId + 'leftTooltip' });
                this.rightTooltip = this.renderer.createGroup({ 'id': this._rootId + 'rightTooltip' });

                var leftToolOptions = {
                    'fill': this.model.tooltipSettings.backgroundColor,
                    'd': 'M 0 0 L 0 21 L 64 21 L 64 6 L 70 0 Z'
                };
                this.renderer.drawPath(leftToolOptions, this.leftTooltip);
                var tooltipoptions = this.model.tooltipSettings.font;
                

                var textOptions = {
                    'id': this._rootId + '_LeftToolText',
                    'x': 5,
                    'fill': tooltipoptions.color,
                    'font-size': tooltipoptions.size,
                    'font-family': tooltipoptions.family,
                    'font-style': tooltipoptions.style,
                    'font-weight': tooltipoptions.weight,
                    'text-anchor': 'start',
                    'opacity': tooltipoptions.opacity,
                    'dominant-baseline': 'middle',
                    
                    
                };
                this.renderer.drawText(textOptions, Globalize.format(new Date(1 / 1 / 2000), "MMMM, yyyy"), this.leftTooltip);
                this.leftTxt = this.leftTooltip.childNodes[1];
                var rightToolOptions = {
                    'fill': this.model.tooltipSettings.backgroundColor,
                    'd': 'M 0 0 L 70 0 L 70 21 L 6 21 L 6 6 Z',
                    'horizontal-alignment': 'stretch'
                };
                this.renderer.drawPath(rightToolOptions, this.rightTooltip);
                this.renderer.drawText(textOptions, Globalize.format(new Date(1 / 1 / 2000), "MMMM, yyyy"), this.rightTooltip);
                this.rightTxt = this.rightTooltip.childNodes[1];
                
            }
            //creating g tag for holding chart
            this.chartGView = this.renderer.createGroup({ 'id': this._rootId + 'charView' });
            
            this.renderer.append(this.chartGView, this.svgDocument);
            if(this.model.valueType=="datetime")
            this.renderer.append(this.higherLevel, this.svgDocument);
            this.renderer.append(this.lowerLevel, this.svgDocument);
            this.renderer.append(this.centerLevel, this.svgDocument);
            
            //creating g tag for sliders
            this.leftUnSelected=this.renderer.createGroup({'id':'unselectleft'});
            this.gLeftSlider = this.renderer.createGroup({ 'id': 'leftslider' });
            this.gRightSlider = this.renderer.createGroup({ 'id': 'rightslider' });
            this.gCenterSlider = this.renderer.createGroup({ 'id': 'centerslider' });
            this.rightUnSelected = this.renderer.createGroup({ 'id': 'unselectright' });

            var unLeftSliderOptions = {
                'width': parseFloat(this.newWidth - 4 - this.padding*2),
                'height': this.sliderHeight,
                'fill':this.unselectgrad,
                'opacity': '0.3',
                'transform': 'translate('+this.padding+',' + this.sliderPosition + ')'
            };
            this.renderer.drawRect(unLeftSliderOptions, this.leftUnSelected);
            this.leftUnArea = this.leftUnSelected.firstChild;
            this.renderer.append(this.leftUnSelected, this.svgDocument);

            var centerSliderOptions = {
                'width': parseFloat(this.newWidth - 4 - this.padding*2),
                'height': this.sliderHeight,
                'fill': this.selectedgrad,
                'opacity': '0',
                'transform': 'translate('+this.padding+','+this.sliderPosition+')'
            };

            this.renderer.drawRect(centerSliderOptions, this.gCenterSlider);
            this.centerSlider = this.gCenterSlider.firstChild;
            $(this.centerSlider).css('cursor', 'pointer');

            this.renderer.append(this.gCenterSlider, this.svgDocument);
            var leftSliderOptions = {
                'width': '2',
                'height': this.sliderHeight,
                'fill': this.model.navigatorStyleSettings.thumbStroke,
                'opacity': '1',
                'y':this.sliderPosition
               
                
            };
            var lcircleOptions = {
                'cx': this.padding,
                'cy': this.sliderPosition + (this.sliderHeight / 2),
                'r': this.model.navigatorStyleSettings.thumbRadius,
                'stroke-width': 2,
                'stroke': this.model.navigatorStyleSettings.thumbStroke,
                'fill': this.tbClr
            };

                
            this.renderer.drawRect(leftSliderOptions, this.gLeftSlider);
            this.renderer.drawCircle(lcircleOptions, this.gLeftSlider);
            this.leftCircle = this.gLeftSlider.lastChild;
            $(this.leftCircle).css('cursor', 'pointer');
            this.leftSlider = this.gLeftSlider.firstChild;
            $(this.leftSlider).css('cursor', 'w-resize');

            this.renderer.append(this.gLeftSlider, this.svgDocument);

			var width = parseFloat(this.newWidth - 4 - this.padding*2); // To check width on initialize the range navigator
			width = width < 0 ? 0 : width;
			
            var unRightSliderOptions = {
                'width': width,
                'height': this.sliderHeight,
                'fill': this.unselectgrad,
                'opacity': '0.3',
                'transform': 'translate('+this.padding+',' + this.sliderPosition + ')'
            };
            this.renderer.drawRect(unRightSliderOptions, this.rightUnSelected);
            this.rightUnArea = this.rightUnSelected.firstChild;
            this.renderer.append(this.rightUnSelected, this.svgDocument);
            var rightSliderOptions = {
                'width': '2',
                'height': this.sliderHeight,
                'fill': this.model.navigatorStyleSettings.thumbStroke,
                'opacity': '1',
                'transform': 'translate(' + parseFloat(this.newWidth - 4-this.padding) + ', ' + this.sliderPosition + ' )'
            };
            var rcircleOptions = {
                'cx': this.newWidth - 4 - this.padding,
                'cy': this.sliderPosition + (this.sliderHeight / 2),
                'r': this.model.navigatorStyleSettings.thumbRadius,
                'stroke-width': 2,
                'stroke': this.model.navigatorStyleSettings.thumbStroke,
                'fill': this.tbClr
            };
            this.renderer.drawRect(rightSliderOptions, this.gRightSlider);
            this.renderer.drawCircle(rcircleOptions, this.gRightSlider);
            this.rightCircle = this.gRightSlider.lastChild;
            $(this.rightCircle).css('cursor', 'pointer');
            this.rightSlider = this.gRightSlider.firstChild;
            $(this.rightSlider).css('cursor', 'w-resize');

            this.renderer.append(this.gRightSlider, this.svgDocument);

            
            this.renderer.append(this.leftTooltip, this.svgDocument);
            this.renderer.append(this.rightTooltip, this.svgDocument);

            this.element.append(this.svgDocument);
            if (this.svgSupport) {
                this.trueCoords = this.svgDocument.createSVGPoint();

                this.grabPoint = this.svgDocument.createSVGPoint();
            }

            this.backDrop = this.element;
            this.dragTarget = null;            
            
            this._renderChart.call(this, this.model);
            if (evt != undefined && evt.target != undefined)
                this.resize = true;
            
            if ((this.startDateTime != undefined && this.endDateTime != undefined) || (this.startValue != undefined && this.endValue != undefined)) {
                this.calculateInterval.call(this, this.model.labelSettings.higherLevel, this.model.labelSettings.lowerLevel);

                this.setSliders();
            } else {
                if (!this.svgSupport)
                    this.renderer._setAttr($(this.leftSlider), { 'transform': 'translate(' + this.padding + ',' + this.sliderPosition + ')' });
                else
                    this.renderer._setAttr($(this.leftSlider), { 'transform': 'translate(' + this.padding + ',' + 0 + ')' });

                this.leftTooltip.setAttribute('opacity', 0);
                this.rightTooltip.setAttribute('opacity', 0);
            }
            if (this.model.tooltipSettings.visible && this.svgSupport && this.model.tooltipSettings.tooltipDisplayMode == "ondemand") {
                this.rightTooltip.setAttribute('opacity', 0);
                this.leftTooltip.setAttribute('opacity', 0);
            }
            //this._trigger("loaded", this.model);
        },

        eachInterval:"",

        //Rendering chart
        _renderChart: function (model) {
            if (model.dataSource != "" && model.series == "" && model.seriesSettings == "") {
                $(this.chartGView).ejChart(
                {
				    locale : this.model.locale,
                    border: { width: 0, color: "transparent" },
                    margin: { left: 0, right: 0, top: 0, bottom: 0 },
                    elementSpacing: 0,
                    chartArea:
                    {
                        background: "transparent",
                        border: { width: 0 }
                    },
                    primaryXAxis: {
                        axisLine: { visible: false },
                        font: { size: '0px', },
                        majorTickLines:
                        {
                            lineWidth: 0,
                            size: 0,
                            visible: true
                        },
                        majorGridLines: { visible: false },
                        rangePadding: this.model.rangePadding,
                        visible: false,
                        isInversed: this.model.enableRTL

                    },
                    primaryYAxis: this.model.valueAxisSettings,
                 series: [{
                     dataSource: model.dataSource, xName: model.xName, yName: model.yName,
                     type: 'line',
                     width:1.5,
                     enableAnimation:false

                 },
                 ],
                 size: { height: parseFloat(this.sliderHeight).toString(), width: parseFloat(this.newWidth - (2 * this.padding)).toString(), padding: -10 },
                 legend: {
                     visible: false,
                     itemSize: { height: 0, width: 0, borderColor: 'transparent', borderWidth: 0 },
                 }
             });
            }
            else if (model.dataSource != "" && model.series != "" && model.series[0].dataSource == undefined || model.seriesSettings != "" && model.seriesSettings.dataSource == undefined && model.dataSource!="") {
                model.seriesSettings.dataSource = model.dataSource;
				model.seriesSettings.xName=model.xName;
				model.seriesSettings.yName=model.yName;
                $(this.chartGView).ejChart(
            {
			    locale : this.model.locale,
                border: { width: 0, color: "transparent" },
                margin: { left: 0, right: 0, top: 0, bottom: 0 },
                elementSpacing: 0,
                chartArea:
                {
                    background: "transparent",
                    border: { width: 0 }
                },
                primaryXAxis: {

                    axisLine: { visible: false },
                    font: { size: '0px', },
                    majorTickLines:
                    {
                        lineWidth: 0, size: 0, visible: true
                    },
                    majorGridLines: { visible: false },
                    rangePadding: this.model.rangePadding,
                    visible: false,
                    isInversed: this.model.enableRTL

                },
                primaryYAxis: this.model.valueAxisSettings,
                series: model.series,
                commonSeriesOptions: model.seriesSettings,
                size: { height: parseFloat(this.sliderHeight).toString(), width: parseFloat(this.newWidth - (2 * this.padding)).toString(), padding: -10 },
                legend: {
                    visible: false,
                    itemSize: { height: 0, width: 0, borderColor: 'transparent', borderWidth: 0 },
                }
            });
            }
            else if (model.series != "" && model.series[0].dataSource != undefined || model.seriesSettings != "" && model.seriesSettings.dataSource != undefined) {
                if (model.seriesSettings == "") {
                    model.seriesSettings = {
                        type: 'line'
                    };
                }
                $(this.chartGView).ejChart(
            {
			    locale : this.model.locale,
                border: { width: 0, color: "transparent" },
                margin: { left: 0, right: 0, top: 0, bottom: 0 },
                elementSpacing: 0,
                chartArea:
                {
                    background: "transparent",
                    border: { width: 0 }
                },
                primaryXAxis: {

                    axisLine: { visible: false },
                    font: { size: '0px', },
                    majorTickLines:
                    {
                        lineWidth: 0, size: 0, visible: true
                    },
                    majorGridLines: { visible: false },
                    rangePadding: this.model.rangePadding,
                    visible: false,
                    isInversed: this.model.enableRTL

                },
                primaryYAxis: this.model.valueAxisSettings,
                series: model.series,
                commonSeriesOptions: model.seriesSettings,
                size: { height: parseFloat(this.sliderHeight).toString(), width: parseFloat(this.newWidth - (2 * this.padding)).toString(), padding: -10 },
                legend: {
                    visible: false,
                    itemSize: { height: 0, width: 0, borderColor: 'transparent', borderWidth: 0 },
                }
            });
            }

            this.ejChart = $(this.chartGView).data("ejChart");
            this.renderer._setAttr($(this.chartGView), { 'transform': 'translate(' + this.padding + ',' + this.sliderPosition + ')','height': this.sliderHeight });
           
        },

        _init: function () {
            this._createSvg(this);
			var commonLoadedEventArgs = $.extend({}, ej.EjSvgRender.commonChartEventArgs);
            commonLoadedEventArgs.data = { model: this.model };
            this._trigger("load", commonLoadedEventArgs);
            this.renderNavigator(this);
            if (this.model.enableAutoResizing) {
                this._on($(window), "resize", this.renderNavigator);
            }
            this._bindevents();
        },

        isDevice: function () {
            // return (/mobile|tablet|android|kindle/i.test(navigator.userAgent.toLowerCase()));
            // comment above line temporary. Due to the below code event wont bind for tablet device
            return (/mobile|android|kindle/i.test(navigator.userAgent.toLowerCase()));
        },

        _bindevents: function () {
            var matched = jQuery.uaMatch(navigator.userAgent);
            var isIE11 = !!navigator.userAgent.match(/Trident\/7\./);
            if (window.navigator.msPointerEnabled && !isIE11) {
                this._on($(this.svgDocument), "MSPointerDown", this._grab);
                this._on($(this.svgDocument), "MSPointerMove", this._drag);
                this._on($(this.svgDocument), "MSPointerUp", this._drop);
                this._on($(window), "MSPointerUp", this.mouseup);
                $(this.svgDocument).css('-ms-touch-action', 'none');
            }
            else if (this.isDevice()) {
                this._on($(this.svgDocument), "touchstart", this._grab);
                this._on($(this.svgDocument), "touchmove", this._drag);
                this._on($(this.svgDocument), "touchend", this._drop);
            }
            else if (matched.browser.toLowerCase() == "chrome") {
                this._on($(this.svgDocument), "touchstart", this._grab);
                this._on($(this.svgDocument), "touchmove", this._drag);
                this._on($(this.svgDocument), "touchend", this._drop);
                this._on($(this.svgDocument), "mousedown", this._grab);
                this._on($(this.svgDocument), "mousemove", this._drag);
                this._on($(this.svgDocument), "mouseup", this._drop);
            }
            else {
                this._on($(this.svgDocument), "mousedown", this._grab);
                this._on($(this.svgDocument), "mousemove", this._drag);
                this._on($(this.svgDocument), "mouseup", this._drop);
            }
            if (isIE11)
                $(this.svgDocument).css('touch-action', 'none');

            this._on($(window), "mouseup", this.mouseup);
        },

        _initializeVariables: function (model, series, sSettings) {
            //intializing ismouseup flag
            this.ismouseup = false;
            var labelStyle = this.model.labelSettings.style;
            var lowLabelStyle = this.model.labelSettings.lowerLevel.style;
            var highLabelStyle = this.model.labelSettings.higherLevel.style;
            //checking labelstyle
            if (highLabelStyle == "")
                this.model.labelSettings.higherLevel.style = labelStyle;
            if (lowLabelStyle == "") {
                this.model.labelSettings.lowerLevel.style = labelStyle;
            }
            else if (lowLabelStyle != undefined && lowLabelStyle.horizontalAlignment == undefined) {
                this.model.labelSettings.lowerLevel.style.horizontalAlignment = "center";
            }
            if (this.model.valueType == "numeric" && this.model.tooltipSettings.labelFormat=="MM/dd/yyyy") {
                this.model.tooltipSettings.labelFormat = "d";
            }
            //checking for datasource and finding start and end values
            if (model.dataSource != "" && model.xName != "" && model.yName != "") {
                this._processJsonData(model.dataSource, this.model);
            }
            else if (series!="" && series != undefined && series[0].dataSource != undefined && series[0].xName != undefined && series[0].yName != undefined) {
                this._processJsonData(series[0].dataSource, series[0]);
            }
            else if (sSettings.dataSource != undefined && sSettings.xName != undefined && sSettings.yName != undefined) {
                this._processJsonData(sSettings.dataSource, sSettings);
            }
            else if (this.model.rangeSettings.start != "" && this.model.rangeSettings.end != "") {
                if (this.model.valueType == "datetime") {
                    this.startDateTime = new Date(this.model.rangeSettings.start);
                    this.endDateTime = new Date(this.model.rangeSettings.end);
                }
                else {
                    this.startValue = this.model.rangeSettings.start;
                    this.endValue = this.model.rangeSettings.end;
                }
            }
            // initializing start year and end year
            if (this.startDateTime != undefined && this.endDateTime != undefined) {
                if (this.model.valueType == "datetime" && this.model.rangePadding == "round") {
                    var start = new Date(this.startValue);
                    var end = new Date(this.endValue);
                    this.startDateTime = new Date(start.getFullYear(), start.getMonth(), start.getDate(), 0, 0, 0);
                    this.endDateTime = new Date(end.getFullYear(), end.getMonth(), end.getDate(), 23, 59, 59);
                }
                this.startYear = this.startDateTime.getFullYear();
                this.startMonth = this.startDateTime.getMonth(); 
                this.endYear = this.endDateTime.getFullYear();
                this.endMonth = this.endDateTime.getMonth();                
            }
        },

        setSliders: function () {
            var wholeSize = this.newWidth - this.padding * 2;
            var type = this.model.valueType;
            var selectedRangeStart = this.model.selectedRangeSettings.start;
            var selectedRangeEnd = this.model.selectedRangeSettings.end;
           // var leftSliderPosition, rightSliderPosition;
            if (type == "datetime")
                this.eachInterval = wholeSize / parseFloat(this.endDateTime.getTime() - this.startDateTime.getTime());
            else
                this.eachInterval = wholeSize / parseFloat(this.endValue - this.startValue);
            if (selectedRangeStart === "" && selectedRangeEnd === "") {
                if (type == "datetime") {
                    selectedRangeStart = this.startDateTime;
                    selectedRangeEnd = this.endDateTime;
                }
                else {
                    selectedRangeStart = this.startValue;
                    selectedRangeEnd = this.endValue;
                }

            }

            if (type == "datetime") {
                this.leftSliderPosition = (new Date(selectedRangeStart).getTime() - this.startDateTime.getTime()) * this.eachInterval + this.padding;
                this.rightSliderPosition = (new Date(selectedRangeEnd).getTime() - this.startDateTime.getTime()) * this.eachInterval + this.padding;
            }
            else {
                this.leftSliderPosition = (selectedRangeStart - this.startValue) * this.eachInterval + this.padding;
                this.rightSliderPosition = (selectedRangeEnd - this.startValue) * this.eachInterval + this.padding;
            }
            if (this.model.tooltipSettings.tooltipDisplayMode == "always")
                this.setopacity = 1;
            else
                this.setopacity = 0;
            this.setSliderPositions(this.leftSliderPosition, null, null);
            this.setSliderPositions(null, null, this.rightSliderPosition);
            this.setopacity = 1;
            if (this.model.enableDeferredUpdate) {
                this._calculateSelectedData();
                if (this.startDateTime !== new Date(selectedRangeStart) && this.endDateTime !== new Date(selectedRangeEnd) && !this.resize) {
                    this._trigger("rangeChanged", this.model);
                }
            }

        },

        centerSliderWidth: "",
        zoomp: 0,
        zoomf:1,

        setSliderPositions: function (leftValue, centerValue, rightValue) {
            var doubledate;
            var type = this.model.valueType;
            var cPosition;
            var labelPlacement = this.model.labelSettings.higherLevel.labelPlacement;
            cPosition = labelPlacement == "inside" ? 0 : this.centerPosition;
            var rtlValue;
            if (leftValue != null && leftValue >= this.padding) {
                this.leftSliderPosition = leftValue;
                rtlValue = this.model.enableRTL ? (this.newWidth - leftValue) : leftValue;
                if(type=="datetime")
                    doubledate = (rtlValue / this.eachInterval) + this.startDateTime.getTime() - (this.padding / this.eachInterval);
                else
                    doubledate = (rtlValue / this.eachInterval) + this.startValue - (this.padding / this.eachInterval);
                if (this.model.tooltipSettings.visible && this.svgSupport) {
                    this.leftTooltip.setAttribute('opacity', this.setopacity);
                    if(type=="datetime")
                        this.leftTxt.textContent = Globalize.format(new Date(doubledate), this.model.tooltipSettings.labelFormat);
                    else
                        this.leftTxt.textContent = Globalize.format(doubledate, this.model.tooltipSettings.labelFormat);
                    var txtwidth = this.leftTooltip.lastChild.getBBox().width + 15;
                    var txtheight = this.leftTooltip.lastChild.getBBox().height;
                    var txty = this.brow!="msie" ? (txtheight + 6) / 2 : (txtheight / 2) + 6;
                    this.leftTxt.setAttribute('y', txty);
                    this.leftd = "M 0 0 L 0 " + (txtheight + 6) + " L " + (txtwidth - 6) + " " + (txtheight + 6) + " L " + (txtwidth - 6) + " 6 L " + txtwidth + " 0 Z";
                    this.rightd = "M 0 0 L " + txtwidth + " 0 L " + txtwidth + " " + (txtheight + 6) + " L 6 " + (txtheight + 6) + " L 6 6 Z";

                    if (leftValue - txtwidth >= 0) {
                        this.leftTooltip.firstChild.setAttribute('d', this.leftd);
                        this.renderer._setAttr($(this.leftTooltip), { 'transform': 'translate(' + (leftValue - txtwidth) + ',' + cPosition + ')' });
                        this.leftTxt.setAttribute('x', 5);
                    }
                    else if (leftValue + txtwidth < this.rightTooltip.getCTM().e) {
                        this.renderer._setAttr($(this.leftTooltip), { 'transform': 'translate(' + (leftValue) + ',' + cPosition + ')' });
                        this.leftTooltip.firstChild.setAttribute('d', this.rightd);
                        this.leftTxt.setAttribute('x', 10);
                    }
                    else if (this.rightTooltip.getCTM().f < 30) {
                        this.renderer._setAttr($(this.leftTooltip), {'transform': 'translate(' + (leftValue) + ',' + (cPosition + 30) + ')'});
                        this.leftTooltip.firstChild.setAttribute('d', this.rightd);
                    }
                    else {
                        this.renderer._setAttr($(this.leftTooltip), {'transform': 'translate(' + (leftValue) + ',' + (cPosition) + ')'});
                        this.leftTooltip.firstChild.setAttribute('d', this.rightd);
                    }
                    if (leftValue > this.newWidth - this.rightTooltip.lastChild.getBBox().width - 15 && this.rightTooltip.getCTM().f < 30) {
                        this.renderer._setAttr($(this.leftTooltip), {'transform': 'translate(' + (leftValue - txtwidth) + ',' + (cPosition + 30) + ')'});
                    }
                    if (this.rightTooltip.getCTM().f >= 30 && leftValue < this.rightTooltip.getCTM().e) {
                        this.renderer._setAttr($(this.rightTooltip), { 'transform': 'translate(' + this.rightTooltip.getCTM().e + ',' + (cPosition) + ')'});
                    }
                }
                if (this.ismouseup == false) {
                    // this.renderer._setAttr($(this.leftUnArea), { 'width': leftValue - this.padding});
                    this.renderer._setAttr($(this.leftUnArea), { 'width': leftValue - this.padding, 'transform': 'translate(' + this.padding + ',' + this.sliderPosition + ')' });
                    if (!this.svgSupport)
                        this.renderer._setAttr($(this.leftSlider), { 'transform': 'translate(' + leftValue + ',' + this.sliderPosition + ')' });
                    else
                        this.renderer._setAttr($(this.leftSlider), { 'transform': 'translate(' + leftValue + ',' + 0 + ')' });
                    if (this.leftCircle != undefined) {
                        if(this.svgSupport)
                            this.renderer._setAttr($(this.leftCircle), { 'cx': leftValue });
                        else
                            this.renderer._setAttr($(this.leftCircle), { 'transform': 'translate(' + parseFloat(leftValue-10) + ',' + parseFloat(this.sliderPosition + (this.sliderHeight / 2)) + ')' }); //this.sliderPosition + (this.sliderHeight / 2)
                    }
                    this.renderer._setAttr($(this.centerSlider), {'transform': 'translate(' + parseFloat(leftValue) + ',' + this.sliderPosition + ')'});                   
                    var width = parseFloat(this.rightSlider.getBoundingClientRect().left) - parseFloat(this.leftSlider.getBoundingClientRect().left);
                    this.renderer._setAttr($(this.centerSlider), { 'width': width});
                }
                else {
                    this.ismouseup = false;
                    this.renderer._setAttr($(this.leftUnArea), { 'width': leftValue - this.padding, 'transform': 'translate(' + this.padding + ',' + this.sliderPosition + ')' });
                    if (!this.svgSupport)
                        this.renderer._setAttr($(this.leftSlider), { 'transform': 'translate(' + leftValue + ',' + this.sliderPosition + ')' });
                    else
                        this.renderer._setAttr($(this.leftSlider), { 'transform': 'translate(' + leftValue + ',' + 0 + ')' });
                    if (this.leftCircle != undefined) {
                        if (this.svgSupport)
                            this.renderer._setAttr($(this.leftCircle), { 'cx': leftValue });
                        else
                            this.renderer._setAttr($(this.leftCircle), { 'transform': 'translate(' + parseFloat(leftValue - 10) + ',' + parseFloat(this.sliderPosition + (this.sliderHeight / 2)) + ')' }); //this.sliderPosition + (this.sliderHeight / 2)
                    }
                    this.renderer._setAttr($(this.centerSlider), {'transform': 'translate(' + parseFloat(leftValue) + ',' + this.sliderPosition + ')'});
                    var width = parseFloat(this.rightSlider.getBoundingClientRect().left) - parseFloat(this.leftSlider.getBoundingClientRect().left);
                    this.renderer._setAttr($(this.centerSlider), { 'width': width});
                }
                this.model.zoomPosition = (leftValue-this.padding) / (this.newWidth-this.padding*2);
                this.model.zoomFactor = (this.rightSlider.getBoundingClientRect().left + parseFloat(this.renderer._getAttrVal($(this.rightSlider), 'width')) - this.leftSlider.getBoundingClientRect().left) / (this.newWidth - this.padding * 2);

                if (type == "datetime") 
                    this.model.selectedRangeSettings.start = new Date(doubledate);                                   
                else 
                    this.model.selectedRangeSettings.start = doubledate;

            }
            else if (rightValue != null) {
                if (rightValue > this.newWidth - this.padding + 1)
                    rightValue = this.newWidth - this.padding;
                this.rightSliderPosition = rightValue;
                var startDateTime = new Date(this.model.rangeSettings.start);
                rtlValue = this.model.enableRTL ? (this.newWidth - rightValue) : rightValue;
               if(type=="datetime")
                   doubledate = (rtlValue / this.eachInterval) + this.startDateTime.getTime() - (this.padding / this.eachInterval);
                else
                   doubledate = (rtlValue / this.eachInterval) + this.startValue - (this.padding / this.eachInterval);
                this.model.selectedRangeSettings.end = new Date(doubledate);

                if (this.model.tooltipSettings.visible && this.svgSupport) {
                    this.rightTooltip.setAttribute('opacity', this.setopacity);
                    if(type=="datetime")
                        this.rightTxt.textContent = Globalize.format(new Date(doubledate), this.model.tooltipSettings.labelFormat);
                    else
                        this.rightTxt.textContent = Globalize.format(doubledate, this.model.tooltipSettings.labelFormat);
                    var txtwidth = this.rightTooltip.lastChild.getBBox().width + 15;
                    var txtheight = this.rightTooltip.lastChild.getBBox().height;
                    var txty = this.brow != "msie" ? (txtheight + 6) / 2 : (txtheight / 2) + 6;
                    this.rightTxt.setAttribute('y', txty);
                    this.rightTxt.setAttribute('x', 10);
                    this.leftd = "M 0 0 L 0 " + (txtheight + 6) + " L " + (txtwidth - 6) + " " + (txtheight + 6) + " L " + (txtwidth - 6) + " 6 L " + txtwidth + " 0 Z";
                    this.rightd = "M 0 0 L " + txtwidth + " 0 L " + txtwidth + " " + (txtheight + 6) + " L 6 " + (txtheight + 6) + " L 6 6 Z";
                    if (rightValue + txtwidth <= this.newWidth) {
                        this.rightTooltip.firstChild.setAttribute('d', this.rightd);
                        this.renderer._setAttr($(this.rightTooltip), {'transform': 'translate(' + (rightValue) + ',' + cPosition + ')'});
                    }
                    else if (rightValue + txtwidth >= this.newWidth && rightValue - txtwidth > this.centerSlider.getCTM().e) {
                        this.renderer._setAttr($(this.rightTooltip), {'transform': 'translate(' + (rightValue - txtwidth) + ',' + cPosition + ')'});
                        this.rightTooltip.firstChild.setAttribute('d', this.leftd);
                        this.rightTxt.setAttribute('x', 5);
                    }
                    else if (this.leftTooltip.getCTM().f < 30) {
                        this.renderer._setAttr($(this.rightTooltip), { 'transform': 'translate(' + (rightValue - txtwidth) + ',' + (cPosition + 30) + ')'});
                        this.rightTooltip.firstChild.setAttribute('d', this.leftd);
                    }
                    else {
                        this.renderer._setAttr($(this.rightTooltip), { 'transform': 'translate(' + (rightValue - txtwidth) + ',' + cPosition + ')'});
                        this.rightTooltip.firstChild.setAttribute('d', this.leftd);
                    }
                    if (rightValue < this.leftTooltip.getCTM().e + txtwidth && this.leftTooltip.getCTM().f < 30) {
                        this.renderer._setAttr($(this.rightTooltip), { 'transform': 'translate(' + (rightValue) + ',' + (cPosition + 30) + ')'});
                    }
                    if (this.leftTooltip.getCTM().f >= 30 && rightValue > this.leftTooltip.getCTM().e + txtwidth && this.rightTooltip.getCTM().e > this.leftTooltip.getCTM().e + txtwidth) {
                        this.renderer._setAttr($(this.leftTooltip), { 'transform': 'translate(' + this.leftTooltip.getCTM().e + ',' + (cPosition) + ')' });
                        this.rightTxt.setAttribute('x', 5);
                    }
                }
                    var rightUnAreawidth =  this.newWidth - rightValue-this.padding-2; // To check width on slider change
					rightUnAreawidth = rightUnAreawidth < 0 ? 0 : rightUnAreawidth;
					
                this.renderer._setAttr($(this.rightUnArea), {'width': rightUnAreawidth, 'transform': 'translate(' + parseFloat(rightValue + 2) + ',' + this.sliderPosition + ')' });
                this.renderer._setAttr($(this.rightSlider), { 'transform': 'translate(' + rightValue + ',' + this.sliderPosition + ')'});
                if (this.rightCircle != undefined) {
                    if (this.svgSupport)
                        this.renderer._setAttr($(this.rightCircle), { 'cx': rightValue });
                    else
                        this.renderer._setAttr($(this.rightCircle), { 'transform': 'translate(' + parseFloat(rightValue-10) + ',' + parseFloat(this.sliderPosition + (this.sliderHeight / 2)) + ')' }); //this.sliderPosition + (this.sliderHeight / 2)
                }
                this.centerSliderWidth = parseFloat(this.rightSlider.getBoundingClientRect().left) - parseFloat(this.leftSlider.getBoundingClientRect().left);
                this.renderer._setAttr($(this.centerSlider), { 'width': this.centerSliderWidth});

              //  this.model.zoomFactor = (rightValue - this.leftSlider.getBoundingClientRect().left) / (this.newWidth - this.padding * 2);
                this.model.zoomFactor = (this.rightSlider.getBoundingClientRect().left + parseFloat(this.renderer._getAttrVal($(this.rightSlider), 'width')) - this.leftSlider.getBoundingClientRect().left) / (this.newWidth - this.padding * 2);
            }
            else if (centerValue != null) {
                var rectwidth = this.renderer._getAttrVal($(this.leftSlider),'width');
                var leftPosX = centerValue;
               
                var rightPosX = centerValue + parseFloat(this.renderer._getAttrVal($(this.centerSlider),'width'));
                var startDateTime = new Date(this.model.rangeSettings.start);
                rtlValue = this.model.enableRTL ? (this.newWidth - leftPosX) : leftPosX;
                if(type=="datetime")
                    doubledate = (rtlValue / this.eachInterval) + this.startDateTime.getTime() - (this.padding / this.eachInterval);
                else
                    doubledate = (rtlValue / this.eachInterval) + this.startValue - (this.padding / this.eachInterval);
                this.model.selectedRangeSettings.start = new Date(doubledate);
                if (leftPosX >= this.padding && rightPosX <= this.newWidth - this.padding) {
                    if(this.svgSupport)
                        this.renderer._setAttr($(this.leftSlider), { 'transform': 'translate(' + leftPosX + ',' + 0 + ')' });
                    else
                        this.renderer._setAttr($(this.leftSlider), { 'transform': 'translate(' + leftPosX + ',' + parseFloat(this.sliderPosition) + ')' });
                    this.leftSliderPosition = leftPosX;
                    if (this.leftCircle != undefined) {
                        if (this.svgSupport)
                            this.renderer._setAttr($(this.leftCircle), { 'cx': leftPosX });
                        else
                            this.renderer._setAttr($(this.leftCircle), { 'transform': 'translate(' + parseFloat(leftPosX - 10) + ',' + parseFloat(this.sliderPosition + (this.sliderHeight / 2)) + ')' });                        
                    }
                    if (this.model.tooltipSettings.visible && this.svgSupport) {
                        this.leftTooltip.setAttribute('opacity', this.setopacity);
                        if(type=="datetime")
                            this.leftTxt.textContent = Globalize.format(new Date(doubledate), this.model.tooltipSettings.labelFormat);
                        else
                            this.leftTxt.textContent = Globalize.format(doubledate, this.model.tooltipSettings.labelFormat);
                        var txtwidth = this.leftTooltip.lastChild.getBBox().width + 15;
                        var txtheight = this.leftTooltip.lastChild.getBBox().height;
                        var txty = this.brow != "msie" ? (txtheight + 6) / 2 : (txtheight / 2) + 6;
                        this.leftTxt.setAttribute('y', txty);
                        this.rightTxt.setAttribute('y', txty);
                        this.leftd = "M 0 0 L 0 " + (txtheight + 6) + " L " + (txtwidth - 6) + " " + (txtheight + 6) + " L " + (txtwidth - 6) + " 6 L " + txtwidth + " 0 Z";
                        this.rightd = "M 0 0 L " + txtwidth + " 0 L " + txtwidth + " " + (txtheight + 6) + " L 6 " + (txtheight + 6) + " L 6 6 Z";
                        if (leftPosX - txtwidth >= 0) {
                            this.leftTooltip.firstChild.setAttribute('d', this.leftd);
                            this.renderer._setAttr($(this.leftTooltip), { 'transform': 'translate(' + (leftPosX - txtwidth) + ',' + cPosition + ')' });
                            this.leftTxt.setAttribute('x', 5);
                        }
                        else {
                            this.renderer._setAttr($(this.leftTooltip), { 'transform': 'translate(' + (leftPosX) + ',' + cPosition + ')'});
                            this.leftTooltip.firstChild.setAttribute('d', this.rightd);
                            this.leftTxt.setAttribute('x', 10);
                            if (this.leftSliderPosition + txtwidth > this.rightSlider.getCTM().e) {
                                this.renderer._setAttr($(this.leftTooltip), { 'transform': 'translate(' + (leftPosX) + ',' + (cPosition + 30) + ')'});
                            }

                        }
                    }
                    rtlValue = this.model.enableRTL ? (this.newWidth - rightPosX) : rightPosX;
                    var doubledate = (rtlValue / this.eachInterval) + this.startDateTime.getTime() - (this.padding / this.eachInterval);
                    this.model.selectedRangeSettings.end = new Date(doubledate);
                    this.renderer._setAttr($(this.rightSlider), { 'transform': 'translate(' + rightPosX + ',' + this.sliderPosition + ')'});
                    this.rightSliderPosition = rightPosX;
                    if (this.rightCircle != undefined) {
                        if (this.svgSupport)
                            this.renderer._setAttr($(this.rightCircle), { 'cx': rightPosX });
                        else
                            this.renderer._setAttr($(this.rightCircle), { 'transform': 'translate(' + parseFloat(rightPosX - 10) + ',' + parseFloat(this.sliderPosition + (this.sliderHeight / 2)) + ')' });                        
                    }
                    if (this.model.tooltipSettings.visible && this.svgSupport) {
                        this.rightTooltip.setAttribute('opacity', this.setopacity);
                        if(type=="datetime")
                            this.rightTxt.textContent = Globalize.format(new Date(doubledate), this.model.tooltipSettings.labelFormat);
                        else
                            this.rightTxt.textContent = Globalize.format(doubledate, this.model.tooltipSettings.labelFormat);
                        var txtwidth = this.rightTooltip.lastChild.getBBox().width + 15;
                        var txtheight = this.rightTooltip.lastChild.getBBox().height;
                        this.leftd = "M 0 0 L 0 " + (txtheight + 6) + " L " + (txtwidth - 6) + " " + (txtheight + 6) + " L " + (txtwidth - 6) + " 6 L " + txtwidth + " 0 Z";
                        this.rightd = "M 0 0 L " + txtwidth + " 0 L " + txtwidth + " " + (txtheight + 6) + " L 6 " + (txtheight + 6) + " L 6 6 Z";
                        if (rightPosX + txtwidth <= this.newWidth) {
                            this.rightTooltip.firstChild.setAttribute('d', this.rightd);
                            this.renderer._setAttr($(this.rightTooltip), { 'transform': 'translate(' + (rightPosX) + ',' + cPosition + ')' });
                            this.rightTxt.setAttribute('x', 10);
                        }
                        else {
                            this.renderer._setAttr($(this.rightTooltip), { 'transform': 'translate(' + (rightPosX - txtwidth) + ',' + cPosition + ')'});
                            this.rightTooltip.firstChild.setAttribute('d', this.leftd);
                            this.rightTxt.setAttribute('x', 5);
                            if (this.rightTooltip.getBoundingClientRect().left < this.leftSlider.getBoundingClientRect().left) {
                                this.renderer._setAttr($(this.rightTooltip), { 'transform': 'translate(' + (rightPosX - txtwidth) + ',' + (cPosition + 30) + ')' });
                            }
                        }
                    }
                    this.renderer._setAttr($(this.leftUnArea), { 'width': leftPosX - this.padding,'transform': 'translate(' + this.padding + ',' + this.sliderPosition + ')'});
                    this.renderer._setAttr($(this.rightUnArea), { 'transform': 'translate(' + parseFloat(rightPosX + 2) + ',' + this.sliderPosition + ')','width': this.newWidth - rightPosX-this.padding-2});

                    this.model.zoomPosition = (this.leftSlider.getBoundingClientRect().left - this.padding) / (this.newWidth - this.padding * 2);

                    this.renderer._setAttr($(this.centerSlider), { 'transform': 'translate(' + centerValue + ',' + this.sliderPosition + ')' });
                    this.model.zoomFactor = (this.rightSlider.getBoundingClientRect().left + parseFloat(this.renderer._getAttrVal($(this.rightSlider), 'width')) - this.leftSlider.getBoundingClientRect().left) / (this.newWidth - this.padding * 2);
                }
               
            }

            if (!this.model.enableDeferredUpdate && this.leftSlider.getBoundingClientRect().left<this.rightSlider.getBoundingClientRect().left) {
                this._calculateSelectedData();
                if (this.zoomp != this.model.zoomPosition || this.zoomf != this.model.zoomFactor) {
                    this._trigger("rangeChanged", this.model);
                    this.zoomp = this.model.zoomPosition;
                    this.zoomf = this.model.zoomFactor;
                }
            }
            
        },

        _calculateSelectedData: function () {
            var j = [];
            var count = 0;
            if(this.model.dataSource!="")
                for (var i = 0; i < this.model.dataSource.length; i++) {
                    if (Date.parse(this.model.selectedRangeSettings.start) <= Date.parse(this.model.dataSource[i][this.model.xName]) && Date.parse(this.model.selectedRangeSettings.end) >= Date.parse(this.model.dataSource[i][this.model.xName])) {
                        j[count] = this.model.dataSource[i];
                        count++;
                    }
                }
            else if (this.model.series!="" && this.model.series[0].dataSource!="" && this.model.series[0].dataSource != "") {
                for (var i = 0; i < this.model.series[0].dataSource.length; i++) {
                    if (Date.parse(this.model.selectedRangeSettings.start) <= Date.parse(this.model.series[0].dataSource[i][this.model.series[0].xName]) && Date.parse(this.model.selectedRangeSettings.end) >= Date.parse(this.model.series[0].dataSource[i][this.model.series[0].xName])) {
                        j[count] = this.model.series[0].dataSource[i];
                        count++;
                    }
                }
            }
            this.model.selectedData = j;
        },

        calculateInterval: function (higherLevel, lowerLevel) {
            if (higherLevel.visible === true && this.model.valueType=="datetime") {
                if (higherLevel.intervalType === "years")
                    this.setYearInterval(this, "higher");
                else if (higherLevel.intervalType === "quarters")
                    this.setQuarterInterval(this, "higher", 0);
                else if (higherLevel.intervalType === "months")
                    this.setMonthInterval(this, "higher", 0);
                else if (higherLevel.intervalType == "weeks")
                    this.setWeekInterval(this, "higher", 0);
                else if (higherLevel.intervalType === "days")
                    this.setDayInterval(this, "higher",0);
                else if (higherLevel.intervalType === "hours")
                    this.setHourInterval(this, "higher",0);
            }
            if (lowerLevel.visible === true && this.model.valueType=="datetime") {

                if (lowerLevel.intervalType === "years")
                    this.setYearInterval(this, "lower");
                else if (lowerLevel.intervalType === "quarters")
                    this.setQuarterInterval(this, "lower", 0);
                else if (lowerLevel.intervalType === "months")
                    this.setMonthInterval(this, "lower", 0);
                else if(lowerLevel.intervalType=="weeks")
                    this.setWeekInterval(this, "lower", 0);
                else if (lowerLevel.intervalType === "days")
                    this.setDayInterval(this, "lower",0);
                else if (lowerLevel.intervalType === "hours")
                    this.setHourInterval(this, "lower",0);
            }

            if (lowerLevel.visible === true && this.model.valueType == "numeric") {
                this.setInterval(this);                
            }


        },

        setInterval: function (naviobj) {
            var currentValue, lastValue;
            var delta = this.endValue - this.startValue;
            var rangePadding = naviobj.model.rangePadding;
            var start = this.startValue;
            var numericInterval = this.calculateNumericInterval(naviobj, delta);
            if (rangePadding == "additional") {
                this.endValue = this.endValue + numericInterval;
                this.startValue = this.startValue - numericInterval;
                delta = this.endValue - this.startValue;
              //  numericInterval = this.calculateNumericInterval(naviobj, delta);
            } else if (rangePadding == "normal") {
                var minimum = 0, remaining;
                if (start < 0) {
                    start = 0;
                    minimum = this.startValue + (this.startValue / 20);

                    remaining = numericInterval + (minimum % numericInterval);

                    if ((0.365 * numericInterval) >= remaining) {
                        minimum -= numericInterval;
                    }

                    if (minimum % numericInterval < 0) {
                        minimum = (minimum - numericInterval) - (minimum % numericInterval);
                    }
                } else {
                    minimum = start < ((5.0 / 6.0) * this.endValue)
                        ? 0
                        : (start - (this.endValue - start) / 2);
                    if (minimum % numericInterval > 0) {
                        minimum -= (minimum % numericInterval);
                    }
                }
                var maximum = (this.endValue + (this.endValue - start) / 20);

                remaining = numericInterval - (maximum % numericInterval);

                if ((0.365 * numericInterval) >= remaining) {
                    maximum += numericInterval;
                }

                if (maximum % numericInterval > 0) {
                    maximum = (maximum + numericInterval) - (maximum % numericInterval);
                }
                numericInterval = this.calculateNumericInterval(naviobj, maximum - minimum);
                this.startValue = minimum;
                this.endValue = maximum;

            } else if (rangePadding == "round") {
                this.startValue = Math.floor((this.startValue / numericInterval) * numericInterval);
                this.endValue = Math.ceil((this.endValue / numericInterval) * numericInterval);
            }

            currentValue = naviobj.startValue;
            count = 0;
            while (currentValue <= naviobj.endValue) {
                this._insertNumericText(naviobj,currentValue);
                count++;
                currentValue = currentValue + numericInterval;
            }
            this.insertLabels(naviobj, "lower");

        },

        calculateNumericInterval: function (naviobj,delta) {
            var desiredIntervalsCount = this.GetDesiredIntervalsCount(naviobj.newWidth);
            var niceInterval = delta / desiredIntervalsCount;
            var minInterval = Math.pow(10, Math.floor(ej.EjSvgRender.utils._logBase(niceInterval, 10)));
            var intervalDivs = [10, 5, 2, 1];

            for (var i = 0; i < intervalDivs.length; i++) {
                var currentInterval = minInterval * intervalDivs[i];
                if (desiredIntervalsCount < (delta / currentInterval)) {
                    return niceInterval;
                }

                niceInterval = currentInterval;
            }

            return niceInterval;
        },

        GetDesiredIntervalsCount: function (size) {
           // if (ej.util.isNullOrUndefined(axis.desiredIntervals)) {
                var desiredIntervalsCount = 0.8* 2;
                desiredIntervalsCount = Math.max((size * (desiredIntervalsCount / 100)), 1);
                return desiredIntervalsCount;
            //} else {
            //    return axis.desiredIntervals;
          //  }
        },

        setHourInterval: function (naviobj, position,level) {

            naviobj._higherTextNode = [];
            naviobj._higherTotalValues = [];
            naviobj._lowerTextNode = [];
            naviobj._lowerTotalValues = [];
            if (position === "higher")
                $("#higherLevel").children().empty();
            else
                $("#lowerLevel").children().empty();

            Date.prototype.addHours = function (hours) {
                var dat = new Date(this.valueOf())
                if (dat.getMilliseconds() !== 0) {
                    dat.setMilliseconds(dat.getMilliseconds() + (1000 - dat.getMilliseconds()));
                }
                if (dat.getSeconds() !== 0) {
                    dat.setSeconds(dat.getSeconds() + (60 - dat.getSeconds()));
                }
                if (dat.getMinutes() !== 0) {                    
                    dat.setMinutes(dat.getMinutes() + (60 - dat.getMinutes()));
                }
                else
                    dat.setHours(dat.getHours() + hours);
                return dat;
            }

            var hourArray = [];
            var currentDate = naviobj.startDateTime;
            var count = 0;
            var txtBlockWidth = 0;
            var bwidth = 0;
            switch (level) {
                case 0:
                    while (currentDate <= naviobj.endDateTime) {
                        bwidth = this._insertText(naviobj, position, "hhtt", currentDate, count, bwidth, false);
                        txtBlockWidth += bwidth;
                        count++;
                        currentDate = currentDate.addHours(1);
                    }
                    if (txtBlockWidth >= naviobj.newWidth)
                        naviobj.setHourInterval(naviobj, position, 1);
                    else
                        this.insertLabels(naviobj, position);
                    break;
                case 1:
                    while (currentDate <= naviobj.endDateTime) {
                        bwidth = this._insertText(naviobj, position, "hht", currentDate, count, bwidth, false);
                        txtBlockWidth += bwidth;
                        count++;                        
                        currentDate = currentDate.addHours(1);
                    }
                    this.insertLabels(naviobj, position);
                    break;
            }
        },

        // Empty the labelbars
        _emptyLabelBars: function (naviobj, position) {
            naviobj._higherTextNode = [];
            naviobj._higherTotalValues = [];
            naviobj._lowerTextNode = [];
            naviobj._lowerTotalValues = [];
            if (position === "higher")
                $("#higherLevel").children().empty();
            else
                $("#lowerLevel").children().empty();
        },

        //to insert week text as there is no globalized format for week
        _insertWeekText: function (naviobj, position, textValue, currentDate, count, bwidth) {
            var hiLabelFont = naviobj.model.labelSettings.higherLevel.style.font;
            var loLabelFont = naviobj.model.labelSettings.lowerLevel.style.font;
            if (position === "higher") {
                if (count == 0)
                    naviobj._higherTotalValues[count] = 0;
                else
                    naviobj._higherTotalValues[count] = currentDate.getTime() - naviobj.startDateTime.getTime();
                var textOptions = {
                    'id': this._rootId + '_higLabelBarText' + '_' + count,
                    'x': 0,
                    'y': 0,
                    'text-anchor': 'start',
                    'dominant-baseline': 'middle',
                    "font-size": hiLabelFont.size,
                    "font-family": hiLabelFont.family,
                    "font-style": hiLabelFont.style,
                    "font-weight": hiLabelFont.weight,
                    "fill": hiLabelFont.color
                };
                naviobj.renderer.drawText(textOptions, textValue, naviobj.higherLevel);
                naviobj._higherTextNode[count] = naviobj.higherLevel.childNodes[count];
                var labelfont = { size: naviobj.model.labelSettings.higherLevel.style.font.size, fontStyle: naviobj.model.labelSettings.higherLevel.style.font.style, fontFamily: naviobj.model.labelSettings.higherLevel.style.font.family };
                if (this.svgSupport)
                    var textOffset = ej.EjSvgRender.utils._measureText(naviobj.higherLevel.childNodes[count].textContent, null, labelfont);
                else
                    var textOffset = ej.EjSvgRender.utils._measureText(naviobj.higherLevel.childNodes[count].innerHTML, null, labelfont);
                bwidth = Math.max(bwidth, textOffset.width);
            }
            else {
                if (count == 0)
                    naviobj._lowerTotalValues[count] = 0;
                else
                    naviobj._lowerTotalValues[count] = currentDate.getTime() - naviobj.startDateTime.getTime();
                var textOptions = {
                    'id': this._rootId + '_LabelBarText' + '_' + count,
                    'x': 0,
                    'y': 0,
                    'text-anchor': 'start',
                    'dominant-baseline': 'middle',
                    "font-size": loLabelFont.size,
                    "font-family": loLabelFont.family,
                    "font-style": loLabelFont.style,
                    "font-weight": loLabelFont.weight,
                    "fill": loLabelFont.color
                };

                naviobj.renderer.drawText(textOptions, textValue, naviobj.lowerLevel);
                naviobj._lowerTextNode[count] = naviobj.lowerLevel.childNodes[count];
                var labelfont = { size: naviobj.model.labelSettings.lowerLevel.style.font.size, fontStyle: naviobj.model.labelSettings.lowerLevel.style.font.style, fontFamily: naviobj.model.labelSettings.lowerLevel.style.font.family };
                if (this.svgSupport)
                    var textOffset = ej.EjSvgRender.utils._measureText(naviobj.lowerLevel.childNodes[count].textContent, null, labelfont);
                else
                    var textOffset = ej.EjSvgRender.utils._measureText(naviobj.lowerLevel.childNodes[count].innerHTML, null, labelfont);
                bwidth = Math.max(bwidth, textOffset.width);
            }

            return bwidth;
        },

        //To calculate Week Interval
        setWeekInterval: function (naviobj, position, level) {

            this._emptyLabelBars(naviobj, position);
            //get week number of a year
            Date.prototype.getWeek = function () {
                var date = new Date(this.getTime());
                date.setHours(0, 0, 0, 0);
                // Thursday in current week decides the year.
                date.setDate(date.getDate() + 3 - (date.getDay() + 6) % 7);
                // January 4 is always in week 1.
                var week1 = new Date(date.getFullYear(), 0, 4);
                // Adjust to Thursday in week 1 and count number of weeks from date to week1.
                return 1 + Math.round(((date.getTime() - week1.getTime()) / 86400000
                                      - 3 + (week1.getDay() + 6) % 7) / 7);
            }

            Date.prototype.dayOfWeek = function () {
                var days = ['Sunday', 'Monday', 'Tuesday', 'Wednesday', 'Thursday', 'Friday', 'Saturday'];
                return days[this.getDay()];
            }

            Date.prototype.addDays = function (noOfDays) {
                var dat = new Date(this.valueOf());
                dat = new Date(dat.getFullYear(), dat.getMonth(), dat.getDate() + noOfDays);
                return dat;
            }

            Date.prototype.subDays = function (noOfDays) {
                var dat = new Date(this.valueOf());
                dat = new Date(dat.getFullYear(), dat.getMonth(), dat.getDate() - noOfDays);
                return dat;
            }

            var currentDate = naviobj.startDateTime;
            var count = 0;
            var txtBlockWidth = 0;
            var bwidth = 0;
			var culture = this.model.locale;
			var locale = ej.datavisualization.RangeNavigator.locale[culture] ? ej.datavisualization.RangeNavigator.locale[culture] : ej.datavisualization.RangeNavigator.locale["en-US"];

			
			    while (currentDate.dayOfWeek() != "Monday") {
			        currentDate = currentDate.subDays(1);
			    }
            switch (level) {
                case 0:
                    while (currentDate <= naviobj.endDateTime) {
                        bwidth = this._insertWeekText(naviobj, position, locale.intervals['week'].longWeeks + " " + currentDate.getWeek() + Globalize.format(currentDate, " MMMM, yyyy"), currentDate, count, bwidth);
                        txtBlockWidth += bwidth;
                        count++;
                        currentDate = currentDate.addDays(7);
                    }
                    if (txtBlockWidth >= naviobj.newWidth)
                        naviobj.setWeekInterval(naviobj, position, 1);
                    else
                        this.insertLabels(naviobj, position);
                    break;
                case 1:
                    while (currentDate <= naviobj.endDateTime) {
                        bwidth = this._insertWeekText(naviobj, position, locale.intervals['week'].longWeeks + " " + currentDate.getWeek() + Globalize.format(currentDate, " MMM, yy"), currentDate, count, bwidth);
                        txtBlockWidth += bwidth;
                        count++;
                        currentDate = currentDate.addDays(7);
                    }
                    if (txtBlockWidth >= naviobj.newWidth)
                        naviobj.setWeekInterval(naviobj, position, 2);
                    else
                        this.insertLabels(naviobj, position);
                    break;
                case 2:
                    while (currentDate <= naviobj.endDateTime) {
                        bwidth = this._insertWeekText(naviobj, position, locale.intervals['week'].longWeeks + " " + currentDate.getWeek(), currentDate, count, bwidth);
                        txtBlockWidth += bwidth;
                        count++;
                        currentDate = currentDate.addDays(7);
                    }
                    if (txtBlockWidth >= naviobj.newWidth)
                        naviobj.setWeekInterval(naviobj, position, 3);
                    else
                        this.insertLabels(naviobj, position);
                    break;
                case 3:
                    while (currentDate <= naviobj.endDateTime) {
                        bwidth = this._insertWeekText(naviobj, position, locale.intervals['week'].shortWeeks + " " + currentDate.getWeek(), currentDate, count, bwidth);
                        txtBlockWidth += bwidth;
                        count++;
                        currentDate = currentDate.addDays(7);
                    }
                        this.insertLabels(naviobj, position);
                    break;
            }
        },
       
        setDayInterval: function (naviobj, position,level) {
            naviobj._higherTextNode = [];
            naviobj._higherTotalValues = [];
            naviobj._lowerTextNode = [];
            naviobj._lowerTotalValues = []; 
            if (position === "higher")
                $("#higherLevel").children().empty();
            else
                $("#lowerLevel").children().empty();

            Date.prototype.addDays = function () {
                var dat = new Date(this.valueOf());
                dat = new Date(dat.getFullYear(), dat.getMonth(), dat.getDate() + 1);
                return dat;
            }

            var hourArray = [];
            var currentDate = naviobj.startDateTime;
            var count = 0;
            var txtBlockWidth = 0;
            var bwidth = 0;
            var bBox;
            switch (level) {
                case 0:
                    while (currentDate <= naviobj.endDateTime) {
                        bwidth= this._insertText(naviobj, position, "dddd, MMMM d, yyyy", currentDate, count, bwidth, false);
                        txtBlockWidth += bwidth;
                        count++;                       
                        currentDate = currentDate.addDays();
                    }
                    if (txtBlockWidth >= naviobj.newWidth)
                        naviobj.setDayInterval(naviobj, position, 1);
                    else
                        this.insertLabels(naviobj, position);
                    break;
                case 1:
                    while (currentDate <= naviobj.endDateTime) {
                        bwidth = this._insertText(naviobj, position, "ddd, MMM d, yy", currentDate, count, bwidth, false);
                        txtBlockWidth += bwidth;
                        count++;
                        currentDate = currentDate.addDays();                        
                    }
                    if (txtBlockWidth >= naviobj.newWidth)
                        naviobj.setDayInterval(naviobj, position, 2);
                    else
                        this.insertLabels(naviobj, position);
                    break;
                case 2:
                    while (currentDate <= naviobj.endDateTime) {
                        bwidth = this._insertText(naviobj, position, "dddd, d", currentDate, count, bwidth, false);
                        txtBlockWidth += bwidth;
                        count++;
                        currentDate = currentDate.addDays();                        
                    }
                    if (txtBlockWidth >= naviobj.newWidth)
                        naviobj.setDayInterval(naviobj, position, 3);
                    else
                        this.insertLabels(naviobj, position);
                    break;
                case 3:
                    while (currentDate <= naviobj.endDateTime) {
                        bwidth = this._insertText(naviobj, position, "dd", currentDate, count, bwidth, false);
                        txtBlockWidth += bwidth;
                        count++;
                        currentDate = currentDate.addDays();                       
                    }
                    this.insertLabels(naviobj, position);
                    break;
                default:
                    break;

            }

        },

        setMonthInterval: function (naviobj, position, level) {
            naviobj._higherTextNode = [];
            naviobj._higherTotalValues = [];
            naviobj._lowerTextNode = [];
            naviobj._lowerTotalValues = [];
            if (position === "higher")
                $("#higherLevel").children().empty();
            else
                $("#lowerLevel").children().empty();           

            var currentDate = naviobj.startDateTime;            

            var count = 0;
            var date = new Date(naviobj.startDateTime);
            var txtBlockWidth = 0;
            var bwidth = 0;
            var bBox;

            switch (level) {
                case 0:
                    while (currentDate <= naviobj.endDateTime) {
                        bwidth = this._insertText(naviobj, position, "MMMM, yyyy", currentDate, count, bwidth,false);
                        txtBlockWidth += bwidth;
                        count++;
                        currentDate = currentDate.addMonths();
                    }
                    if (txtBlockWidth >= naviobj.newWidth)
                        naviobj.setMonthInterval(naviobj, position, 1);
                    else
                        this.insertLabels(naviobj, position);
                    break;
                case 1:
                    while (currentDate <= naviobj.endDateTime) {
                        bwidth = this._insertText(naviobj, position, "MMMM", currentDate, count, bwidth,false);
                        txtBlockWidth += bwidth;
                        count++;
                        currentDate = currentDate.addMonths();
                    }
                    if (txtBlockWidth >= naviobj.newWidth)
                        naviobj.setMonthInterval(naviobj, position, 2);
                    else
                        this.insertLabels(naviobj, position);
                    break;
                case 2:
                    while (currentDate <= naviobj.endDateTime) {
                        bwidth = this._insertText(naviobj, position, "MMM", currentDate, count, bwidth,false);
                        txtBlockWidth += bwidth;                        
                        count++;
                        currentDate = currentDate.addMonths();
                    }
                    if (txtBlockWidth >= naviobj.newWidth)
                        naviobj.setMonthInterval(naviobj, position, 3);
                    else
                        this.insertLabels(naviobj, position);
                    break;
                case 3:
                    while (currentDate <= naviobj.endDateTime) {
                        bwidth = this._insertText(naviobj, position, "MMM", currentDate, count, bwidth, true);
                        txtBlockWidth += bwidth;
                        count++;
                        currentDate = currentDate.addMonths();
                    }
                    this.insertLabels(naviobj, position);
                    break;
                default:
                    break;

            }

        },

        _quarter: function (naviobj, position, quarterValue, yearString, date, txtBlockWidth, count) {
            
            for (var i = naviobj.startYear; i <= naviobj.endYear; i++) {
                var stringDate = yearString != null ? Globalize.format(date, yearString) : "";
                if (i === naviobj.startYear) {
                    for (var j = naviobj.startMonth; j <= 11; j++) {
                        if (position == "higher") {
                            var textOptions = {
                                'id': this._rootId + '_higLabelBarText' + '_' + count,
                                'x': 0,
                                'y': 0,
                                'text-anchor': 'start',
                                'dominant-baseline': 'middle',
                                "font-size": naviobj.model.labelSettings.higherLevel.style.font.size,
                                "font-family": naviobj.model.labelSettings.higherLevel.style.font.family,
                                "font-style": naviobj.model.labelSettings.higherLevel.style.font.style,
                                "font-weight": naviobj.model.labelSettings.higherLevel.style.font.weight,
                                "fill":naviobj.model.labelSettings.higherLevel.style.font.color
                            };
                        }
                        else {
                            var textOptions = {
                                'id': this._rootId + '_LabelBarText' + '_' + count,
                                'x': 0,
                                'y': 0,
                                'text-anchor': 'start',
                                'dominant-baseline': 'middle',
                                "font-size": naviobj.model.labelSettings.lowerLevel.style.font.size,
                                "font-family": naviobj.model.labelSettings.lowerLevel.style.font.family,
                                "font-style": naviobj.model.labelSettings.lowerLevel.style.font.style,
                                "font-weight": naviobj.model.labelSettings.lowerLevel.style.font.weight,
                                "fill":naviobj.model.labelSettings.lowerLevel.style.font.color
                            };
                        }
                        if (j >= 0 && j <= 2) {
                            if(position=="higher")
                                naviobj.renderer.drawText(textOptions, quarterValue + " 1 " + stringDate, naviobj.higherLevel);
                            else
                                naviobj.renderer.drawText(textOptions, quarterValue + " 1 " + stringDate, naviobj.lowerLevel);
                            date = new Date(date.setMonth(3));
                            k = 2;
                        }
                        else if (j >= 3 && j <= 5) {
                            if (position == "higher")
                                naviobj.renderer.drawText(textOptions, quarterValue + " 2 " + stringDate, naviobj.higherLevel);
                            else
                                naviobj.renderer.drawText(textOptions, quarterValue + " 2 " + stringDate, naviobj.lowerLevel);
                            date = new Date(date.setMonth(6));
                            k = 5;

                        }
                        else if (j >= 6 && j <= 8) {
                            if (position == "higher")
                                naviobj.renderer.drawText(textOptions, quarterValue + " 3 " + stringDate, naviobj.higherLevel);
                            else
                                naviobj.renderer.drawText(textOptions, quarterValue + " 3 " + stringDate, naviobj.lowerLevel);
                            date = new Date(date.setMonth(9));
                            k = 8;

                        }
                        else if (j >= 9 && j <= 11) {
                            if (position == "higher")
                                naviobj.renderer.drawText(textOptions, quarterValue + " 4 " + stringDate, naviobj.higherLevel);
                            else
                                naviobj.renderer.drawText(textOptions, quarterValue + " 4 " + stringDate, naviobj.lowerLevel);
                            date = new Date(date.setMonth(12));
                            k = 11;

                        }


                        if (position === "lower") {
                            if (j === naviobj.startMonth)
                                naviobj._lowerTotalValues[count] = naviobj.startDateTime.getTime() - naviobj.startDateTime.getTime();
                            else
                                naviobj._lowerTotalValues[count] = new Date(naviobj.startYear + "/" + parseInt(j + 1) + "/1").getTime() - naviobj.startDateTime.getTime();
                            j = k;
                            naviobj._lowerTextNode[count] = naviobj.lowerLevel.childNodes[count];
                            var labelfont = { size: naviobj.model.labelSettings.lowerLevel.style.font.size, fontStyle: naviobj.model.labelSettings.lowerLevel.style.font.style, fontFamily: naviobj.model.labelSettings.lowerLevel.style.font.family };
                            var textOffset = ej.EjSvgRender.utils._measureText(naviobj.lowerLevel.childNodes[count].textContent, null, labelfont);
                            txtBlockWidth += textOffset.width;
                        }
                        else {
                            if (j === naviobj.startMonth)
                                naviobj._higherTotalValues[count] = naviobj.startDateTime.getTime() - naviobj.startDateTime.getTime();
                            else
                                naviobj._higherTotalValues[count] = new Date(naviobj.startYear + "/" + parseInt(j + 1) + "/1").getTime() - naviobj.startDateTime.getTime();
                            j = k;
                            naviobj._higherTextNode[count] = naviobj.higherLevel.childNodes[count];
                            var labelfont = { size: naviobj.model.labelSettings.higherLevel.style.font.size, fontStyle: naviobj.model.labelSettings.higherLevel.style.font.style, fontFamily: naviobj.model.labelSettings.higherLevel.style.font.family };
                            var textOffset = ej.EjSvgRender.utils._measureText(naviobj.higherLevel.childNodes[count].textContent, null, labelfont);
                            txtBlockWidth += textOffset.width;
                        }
                        count++;
                    }
                }
                else if (i === naviobj.endYear) {
                    for (var j = 0; j <= naviobj.endMonth; j++) {
                        if (position == "higher") {
                            var textOptions = {
                                'id': this._rootId + '_higLabelBarText' + '_' + count,
                                'x': 0,
                                'y': 0,
                                'text-anchor': 'start',
                                'dominant-baseline': 'middle',
                                "font-size": naviobj.model.labelSettings.higherLevel.style.font.size,
                                "font-family": naviobj.model.labelSettings.higherLevel.style.font.family,
                                "font-style": naviobj.model.labelSettings.higherLevel.style.font.style,
                                "font-weight": naviobj.model.labelSettings.higherLevel.style.font.weight
                            };
                        }
                        else {
                            var textOptions = {
                                'id': this._rootId + '_LabelBarText' + '_' + count,
                                'x': 0,
                                'y': 0,
                                'text-anchor': 'start',
                                'dominant-baseline': 'middle',
                                "font-size": naviobj.model.labelSettings.lowerLevel.style.font.size,
                                "font-family": naviobj.model.labelSettings.lowerLevel.style.font.family,
                                "font-style": naviobj.model.labelSettings.lowerLevel.style.font.style,
                                "font-weight": naviobj.model.labelSettings.lowerLevel.style.font.weight
                            };
                        }
                        if (j >= 0 && j <= 2) {
                            if (position == "higher")
                                naviobj.renderer.drawText(textOptions, quarterValue + " 1 " + stringDate, naviobj.higherLevel);
                            else
                                naviobj.renderer.drawText(textOptions, quarterValue + " 1 " + stringDate, naviobj.lowerLevel);
                            date = new Date(date.setMonth(3));
                            k = 2;
                        }
                        else if (j >= 3 && j <= 5) {
                            if (position == "higher")
                                naviobj.renderer.drawText(textOptions, quarterValue + " 2 " + stringDate, naviobj.higherLevel);
                            else
                                naviobj.renderer.drawText(textOptions, quarterValue + " 2 " + stringDate, naviobj.lowerLevel);
                            date = new Date(date.setMonth(6));
                            k = 5;

                        }
                        else if (j >= 6 && j <= 8) {
                            if (position == "higher")
                                naviobj.renderer.drawText(textOptions, quarterValue + " 3 " + stringDate, naviobj.higherLevel);
                            else
                                naviobj.renderer.drawText(textOptions, quarterValue + " 3 " + stringDate, naviobj.lowerLevel);
                            date = new Date(date.setMonth(9));
                            k = 8;

                        }
                        else if (j >= 9 && j <= 11) {
                            if (position == "higher")
                                naviobj.renderer.drawText(textOptions, quarterValue + " 4 " + stringDate, naviobj.higherLevel);
                            else
                                naviobj.renderer.drawText(textOptions, quarterValue + " 4 " + stringDate, naviobj.lowerLevel);
                            date = new Date(date.setMonth(12));
                            k = 11;

                        }


                        if (position === "lower") {
                            naviobj._lowerTotalValues[count] = new Date(naviobj.endYear + "/" + parseInt(j + 1) + "/1").getTime() - naviobj.startDateTime.getTime();                            
                            j = k;
                            naviobj._lowerTextNode[count] = naviobj.lowerLevel.childNodes[count];
                            var labelfont = { size: naviobj.model.labelSettings.lowerLevel.style.font.size, fontStyle: naviobj.model.labelSettings.lowerLevel.style.font.style, fontFamily: naviobj.model.labelSettings.lowerLevel.style.font.family };
                            var textOffset = ej.EjSvgRender.utils._measureText(naviobj.lowerLevel.childNodes[count].textContent, null, labelfont);
                            txtBlockWidth += textOffset.width;
                        }
                        else {
                            naviobj._higherTotalValues[count] = new Date(naviobj.endYear + "/" + parseInt(j + 1) + "/1").getTime() - naviobj.startDateTime.getTime();
                            j = k;
                            naviobj._higherTextNode[count] = naviobj.higherLevel.childNodes[count];
                            var labelfont = { size: naviobj.model.labelSettings.higherLevel.style.font.size, fontStyle: naviobj.model.labelSettings.higherLevel.style.font.style, fontFamily: naviobj.model.labelSettings.higherLevel.style.font.family };
                            var textOffset = ej.EjSvgRender.utils._measureText(naviobj.higherLevel.childNodes[count].textContent, null, labelfont);
                            txtBlockWidth += textOffset.width;
                        }
                        count++;
                    }
                }
                else {
                    for (var j = 0; j <= 11; j++) {
                        if (position == "higher") {
                            var textOptions = {
                                'id': this._rootId + '_higLabelBarText' + '_' + count,
                                'x': 0,
                                'y': 0,
                                'text-anchor': 'start',
                                'dominant-baseline': 'middle',
                                "font-size": naviobj.model.labelSettings.higherLevel.style.font.size,
                                "font-family": naviobj.model.labelSettings.higherLevel.style.font.family,
                                "font-style": naviobj.model.labelSettings.higherLevel.style.font.style,
                                "font-weight": naviobj.model.labelSettings.higherLevel.style.font.weight
                            };
                        }
                        else {
                            var textOptions = {
                                'id': this._rootId + '_LabelBarText' + '_' + count,
                                'x': 0,
                                'y': 0,
                                'text-anchor': 'start',
                                'dominant-baseline': 'middle',
                                "font-size": naviobj.model.labelSettings.lowerLevel.style.font.size,
                                "font-family": naviobj.model.labelSettings.lowerLevel.style.font.family,
                                "font-style": naviobj.model.labelSettings.lowerLevel.style.font.style,
                                "font-weight": naviobj.model.labelSettings.lowerLevel.style.font.weight
                            };
                        }
                        if (j >= 0 && j <= 2) {
                            if (position == "higher")
                                naviobj.renderer.drawText(textOptions, quarterValue + " 1 " + stringDate, naviobj.higherLevel);
                            else
                                naviobj.renderer.drawText(textOptions, quarterValue + " 1 " + stringDate, naviobj.lowerLevel);
                            date = new Date(date.setMonth(3));
                            k = 2;
                        }
                        else if (j >= 3 && j <= 5) {
                            if (position == "higher")
                                naviobj.renderer.drawText(textOptions, quarterValue + " 2 " + stringDate, naviobj.higherLevel);
                            else
                                naviobj.renderer.drawText(textOptions, quarterValue + " 2 " + stringDate, naviobj.lowerLevel);
                            date = new Date(date.setMonth(6));
                            k = 5;

                        }
                        else if (j >= 6 && j <= 8) {
                            if (position == "higher")
                                naviobj.renderer.drawText(textOptions, quarterValue + " 3 " + stringDate, naviobj.higherLevel);
                            else
                                naviobj.renderer.drawText(textOptions, quarterValue + " 3 " + stringDate, naviobj.lowerLevel);
                            date = new Date(date.setMonth(9));
                            k = 8;

                        }
                        else if (j >= 9 && j <= 11) {
                            if (position == "higher")
                                naviobj.renderer.drawText(textOptions, quarterValue + " 4 " + stringDate, naviobj.higherLevel);
                            else
                                naviobj.renderer.drawText(textOptions, quarterValue + " 4 " + stringDate, naviobj.lowerLevel);
                            date = new Date(date.setMonth(12));
                            k = 11;

                        }


                        if (position === "lower") {
                            naviobj._lowerTotalValues[count] = new Date(i + "/" + parseInt(j + 1) + "/1").getTime() - naviobj.startDateTime.getTime();
                            j = k;
                            naviobj._lowerTextNode[count] = naviobj.lowerLevel.childNodes[count];
                            var labelfont = { size: naviobj.model.labelSettings.lowerLevel.style.font.size, fontStyle: naviobj.model.labelSettings.lowerLevel.style.font.style, fontFamily: naviobj.model.labelSettings.lowerLevel.style.font.family };
                            var textOffset = ej.EjSvgRender.utils._measureText(naviobj.lowerLevel.childNodes[count].textContent, null, labelfont);
                            txtBlockWidth += textOffset.width;
                        }
                        else {
                            naviobj._higherTotalValues[count] = new Date(i + "/" + parseInt(j + 1) + "/1").getTime() - naviobj.startDateTime.getTime();
                            j = k;
                            naviobj._higherTextNode[count] = naviobj.higherLevel.childNodes[count];
                            var labelfont = { size: naviobj.model.labelSettings.higherLevel.style.font.size, fontStyle: naviobj.model.labelSettings.higherLevel.style.font.style, fontFamily: naviobj.model.labelSettings.higherLevel.style.font.family };
                            var textOffset = ej.EjSvgRender.utils._measureText(naviobj.higherLevel.childNodes[count].textContent, null, labelfont);
                            txtBlockWidth += textOffset.width;
                        }
                        count++;
                    }
                }

            }
            return txtBlockWidth;
        },

        setQuarterInterval: function (naviobj, position, level) {
            naviobj._higherTextNode = [];
            naviobj._higherTotalValues = [];
            naviobj._lowerTextNode = [];
            naviobj._lowerTotalValues = [];
            if (position === "higher")
                $("#higherLevel").children().empty();
            else
                $("#lowerLevel").children().empty();
            var date = new Date(naviobj.startDateTime);
            var count = 0;
            var k = 0;
            var txtBlockWidth = 0;
			var culture = this.model.locale;
			var locale = ej.datavisualization.RangeNavigator.locale[culture] ? ej.datavisualization.RangeNavigator.locale[culture]:
                                                                               ej.datavisualization.RangeNavigator.locale["en-US"];
            switch (level) {
                case 0:
                    txtBlockWidth=this._quarter(naviobj, position, locale.intervals['quarter'].longQuarters, "yyyy", date, txtBlockWidth, count);                    
                    if (txtBlockWidth >= naviobj.newWidth)
                        naviobj.setQuarterInterval(naviobj, position, 1);
                    else
                        this.insertLabels(naviobj, position);
                    break;
                case 1:
                    txtBlockWidth = this._quarter(naviobj, position, locale.intervals['quarter'].longQuarters, "yy", date, txtBlockWidth, count);
                    if (txtBlockWidth >= naviobj.newWidth)
                        naviobj.setQuarterInterval(naviobj, position, 2);
                    else
                        this.insertLabels(naviobj, position);
                    break;
                case 2:
                    txtBlockWidth = this._quarter(naviobj, position, locale.intervals['quarter'].shortQuarters, "yyyy", date, txtBlockWidth, count);
                    if (txtBlockWidth >= naviobj.newWidth)
                        naviobj.setQuarterInterval(naviobj, position, 3);
                    else
                        this.insertLabels(naviobj, position);
                    break;
                case 3:
                    txtBlockWidth = this._quarter(naviobj, position, locale.intervals['quarter'].shortQuarters, null, date, txtBlockWidth, count);
                    this.insertLabels(naviobj, position);
                    break;
                default:
                    break;
            }
        },

        _insertText: function (naviobj, position, txtValue, currentDate, count, bwidth, setsubstring) {

            Date.prototype.addYears = function (years) {
                var dat = new Date(this.valueOf())
                if (dat.getMilliseconds() !== 0 || dat.getSeconds() !== 0
                 || dat.getMinutes() !== 0 || dat.getHours() !== 0 || dat.getDate() !== 0
                 || dat.getMonth() !== 0) {
                    dat = new Date(dat.getFullYear() + 1, 0, 1);
                }
                else
                    dat.setYear(dat.getFullYear() + years);
                return dat;
            }

            Date.prototype.addMonths = function () {
                var dat = new Date(this.valueOf())
                dat = new Date(dat.getFullYear(), dat.getMonth() + 1, 1);
                return dat;
            }

            if (position === "higher") {
                naviobj._higherTotalValues[count] = currentDate.getTime() - naviobj.startDateTime.getTime();
                var textOptions = {
                    'id': this._rootId + '_higLabelBarText' + '_' + count,
                    'x': 0,
                    'y': 0,
                    'text-anchor': 'start',
                    'dominant-baseline': 'middle',
                    "font-size": naviobj.model.labelSettings.higherLevel.style.font.size,
                    "font-family": naviobj.model.labelSettings.higherLevel.style.font.family,
                    "font-style": naviobj.model.labelSettings.higherLevel.style.font.style,
                    "font-weight": naviobj.model.labelSettings.higherLevel.style.font.weight,
                    "fill":naviobj.model.labelSettings.higherLevel.style.font.color
                };
                if(setsubstring==false)
                    naviobj.renderer.drawText(textOptions, Globalize.format(currentDate, txtValue), naviobj.higherLevel);
                else
                    naviobj.renderer.drawText(textOptions, Globalize.format(currentDate, txtValue).toString().substring(0, 1), naviobj.higherLevel);
                naviobj._higherTextNode[count] = naviobj.higherLevel.childNodes[count];
                var labelfont = { size: naviobj.model.labelSettings.higherLevel.style.font.size, fontStyle: naviobj.model.labelSettings.higherLevel.style.font.style, fontFamily: naviobj.model.labelSettings.higherLevel.style.font.family };
                if(this.svgSupport)
                    var textOffset = ej.EjSvgRender.utils._measureText(naviobj.higherLevel.childNodes[count].textContent, null, labelfont);
                else
                    var textOffset = ej.EjSvgRender.utils._measureText(naviobj.higherLevel.childNodes[count].innerHTML, null, labelfont);
                bwidth = Math.max(bwidth, textOffset.width);
            }
            else {
                naviobj._lowerTotalValues[count] = currentDate.getTime() - naviobj.startDateTime.getTime();
                var textOptions = {
                    'id': this._rootId + '_LabelBarText' + '_' + count,
                    'x': 0,
                    'y': 0,
                    'text-anchor': 'start',
                    'dominant-baseline': 'middle',
                    "font-size": naviobj.model.labelSettings.lowerLevel.style.font.size,
                    "font-family": naviobj.model.labelSettings.lowerLevel.style.font.family,
                    "font-style": naviobj.model.labelSettings.lowerLevel.style.font.style,
                    "font-weight": naviobj.model.labelSettings.lowerLevel.style.font.weight,
                    "fill":naviobj.model.labelSettings.lowerLevel.style.font.color
                };
                if(setsubstring==false)
                    naviobj.renderer.drawText(textOptions, Globalize.format(currentDate, txtValue), naviobj.lowerLevel);
                else
                    naviobj.renderer.drawText(textOptions, Globalize.format(currentDate, txtValue).substring(0, 1), naviobj.lowerLevel);
                naviobj._lowerTextNode[count] = naviobj.lowerLevel.childNodes[count];
                var labelfont = { size: naviobj.model.labelSettings.lowerLevel.style.font.size, fontStyle: naviobj.model.labelSettings.lowerLevel.style.font.style, fontFamily: naviobj.model.labelSettings.lowerLevel.style.font.family };
                if(this.svgSupport)
                    var textOffset = ej.EjSvgRender.utils._measureText(naviobj.lowerLevel.childNodes[count].textContent, null, labelfont);
                else
                    var textOffset = ej.EjSvgRender.utils._measureText(naviobj.lowerLevel.childNodes[count].innerHTML, null, labelfont);
                bwidth = Math.max(bwidth, textOffset.width);
            }
            return bwidth;
        },

        _insertNumericText: function (naviobj,currentValue) {
            naviobj._lowerTotalValues[count] = currentValue - naviobj.startValue;
            var font = naviobj.model.labelSettings.style.font;
            var textOptions = {
                'id': this._rootId + '_LabelBarText' + '_' + count,
                'x': 0,
                'y': 0,
                'text-anchor': 'start',
                'dominant-baseline': 'middle',
                "font-size": font.size,
                "font-family": font.family,
                "font-style": font.style,
                "font-weight": font.weight,
                "fill": font.color
            };
                naviobj.renderer.drawText(textOptions, currentValue, naviobj.lowerLevel);
            naviobj._lowerTextNode[count] = naviobj.lowerLevel.childNodes[count];
            var labelfont = { size: naviobj.model.labelSettings.lowerLevel.style.font.size, fontStyle: naviobj.model.labelSettings.lowerLevel.style.font.style, fontFamily: naviobj.model.labelSettings.lowerLevel.style.font.family };
            var textOffset;
            if(this.svgSupport) {
                textOffset = ej.EjSvgRender.utils._measureText(naviobj.lowerLevel.childNodes[count].textContent, null, labelfont);
            } else {
                textOffset = ej.EjSvgRender.utils._measureText(naviobj.lowerLevel.childNodes[count].innerHTML, null, labelfont);
            } 
        },

        setYearInterval: function (naviobj, position) {
            var currentDate = naviobj.startDateTime;
            count=0;
            while (currentDate <= naviobj.endDateTime) {
                this._insertText(naviobj, position, "yyyy", currentDate,count,0,false);                   
                count++;
                currentDate = currentDate.addYears(1);
            }            
            this.insertLabels(naviobj, position);
        },

        _applyPadding:function(){
            var i;
            for (i = 0; i < this._higherTotalValues.length; i++) {
                this._higherTotalValues[i] += this.padding;
            }
            for (i = 0; i < this._lowerTotalValues.length; i++) {
                this._lowerTotalValues[i] += this.padding;
            }
        },

        insertLabels: function (naviobj, level) {
            // this._applyPadding();
            var matched = jQuery.uaMatch(navigator.userAgent);
            var lowgridColor = this.model.labelSettings.lowerLevel.gridLineStyle.color;
            this.lgClr = this.renderer.createGradientElement('lgColor', lowgridColor, 150, 0, 150, 100, this.svgDocument);
            var type = this.model.valueType;
            var wholeSize = parseFloat(this.renderer._getAttrVal($(naviobj.svgDocument), 'width'));
            if (type == "datetime")
                this.eachInterval = (wholeSize - this.padding * 2) / parseFloat(this.endDateTime.getTime() - this.startDateTime.getTime());
            else
                this.eachInterval = (wholeSize - this.padding * 2) / parseFloat(this.endValue - this.startValue);
            var count = 0;
            var labelcount = 0;
            var txtwidth = 0;
            var rectoptions;
            var lineleft;
            var newleft;
            var textOffset;
            var labelfont;
            var i;
            if (level === "higher" && naviobj.model.labelSettings.higherLevel.visible===true) {

                if (naviobj.model.labelSettings.higherLevel.position == "top" && naviobj.model.labelSettings.lowerLevel.position == "top") {
                    rectoptions = {
                        x: this.padding,
                        y: 1,
                        width: naviobj.newWidth-naviobj.padding*2,
                        height: naviobj.minHighHeight,
                        'stroke': naviobj.model.labelSettings.higherLevel.border.color,
                        'stroke-width': naviobj.model.labelSettings.higherLevel.border.width,
                        fill: 'transparent'

                    };
                    naviobj.renderer.drawRect(rectoptions, naviobj.higherLevel);
                }
                else if (naviobj.model.labelSettings.higherLevel.position == "top") {
                    rectoptions = {
                        x: this.padding,
                        y: 1,
                        width: naviobj.newWidth - naviobj.padding * 2,
                        height: naviobj.minHighHeight,
                        'stroke': naviobj.model.labelSettings.higherLevel.border.color,
                        'stroke-width': naviobj.model.labelSettings.higherLevel.border.width,
                        fill: 'transparent'

                    };
                    naviobj.renderer.drawRect(rectoptions, naviobj.higherLevel);
                }
                else {
                    rectoptions = {
                        x: this.padding,
                        y: naviobj.bottomPosition,
                        width: naviobj.newWidth - naviobj.padding * 2,
                        height: naviobj.minHighHeight,
                        'stroke': naviobj.model.labelSettings.higherLevel.border.color,
                        'stroke-width': naviobj.model.labelSettings.higherLevel.border.width,
                        fill: 'transparent'

                    };
                    naviobj.renderer.drawRect(rectoptions, naviobj.higherLevel);
                }

                naviobj._higherLineLeft = [];

                if (!naviobj.model.enableRTL) {
                    lineleft = (naviobj._higherTotalValues[count] * this.eachInterval) + this.padding;
                } else {
                    lineleft = naviobj.newWidth - ((naviobj._higherTotalValues[naviobj._higherTextNode.length - 1] * this.eachInterval) + this.padding);
                }
                var isrtl;
                if (naviobj.model.enableRTL) {
                    i = naviobj._higherTextNode.length - 1;
                    isrtl = true;
                }
                else {
                    i = 0;
                    isrtl = false;
                }
                var setLeft = 0;

                for (; isrtl ? i >= 0 : i < naviobj._higherTextNode.length; isrtl?i--:i++) {
                    naviobj._higherLineLeft[i] = lineleft;
                    //  var txt =  naviobj._higherTextNode[i].getBBox();
                    labelfont = { size: naviobj.model.labelSettings.higherLevel.style.font.size, fontStyle: naviobj.model.labelSettings.higherLevel.style.font.style, fontFamily: naviobj.model.labelSettings.higherLevel.style.font.family };
                    textOffset = ej.EjSvgRender.utils._measureText(naviobj.higherLevel.childNodes[count].textContent, null, labelfont);
                    txtwidth = textOffset.width;
                    newleft = naviobj._higherTotalValues[i + 1] === undefined
                        ? isrtl ? 0 : naviobj.newWidth :
                        isrtl?naviobj.newWidth-((naviobj._higherTotalValues[i + 1] * this.eachInterval + this.padding)):
                        (naviobj._higherTotalValues[i + 1] * this.eachInterval + this.padding);
                    if (naviobj.model.labelSettings.higherLevel.style.horizontalAlignment === "center") {
                        setleft = isrtl ? newleft - (newleft - lineleft) / 2 - txtwidth / 2: (newleft - lineleft) / 2 + lineleft - txtwidth / 2;
                    }
                    else if (naviobj.model.labelSettings.higherLevel.style.horizontalAlignment === "left") {
                        setleft = isrtl ? newleft + 10 : lineleft + 10;
                    }
                    else
                        setleft = isrtl ? lineleft - txtwidth - 2 : newleft - txtwidth - 2;
                    naviobj.renderer._setAttr($(naviobj._higherTextNode[i]), { 'x': setleft });
                    naviobj.renderer._setAttr($(naviobj._higherTextNode[i]), { 'fill': naviobj.model.labelSettings.higherLevel.style.font.color });
                    naviobj._higherTextLeft[i] = setleft;
                    
                    if (naviobj.model.labelSettings.higherLevel.position == "top" && naviobj.model.labelSettings.lowerLevel.position == "top") {
                        naviobj._higherTextNode[i].setAttribute("y", naviobj.minHighHeight - 5);
                        rectoptions = {
                            x1: lineleft,
                            y1: '0',
                            x2: lineleft,
                            y2: naviobj.centerPosition,
                            'stroke': naviobj.model.labelSettings.higherLevel.gridLineStyle.color,
                            'stroke-width': naviobj.model.labelSettings.higherLevel.gridLineStyle.width,
                            'stroke-dasharray': naviobj.model.labelSettings.higherLevel.gridLineStyle.dashArray,

                        };
                        if (i != 0)
                        naviobj.renderer.drawLine(rectoptions, naviobj.higherLevel);
                    }
                    else if (naviobj.model.labelSettings.higherLevel.position == "top") {
                        if (this.svgSupport) {
                            if(matched.browser.toLowerCase() == "msie")
                                naviobj._higherTextNode[i].setAttribute("y", naviobj.centerPosition - 5);
                            else
                                naviobj._higherTextNode[i].setAttribute("y", naviobj.centerPosition - 9);
                        }
                        else
                            naviobj.renderer._setAttr($(naviobj._higherTextNode[i]), { 'y': (naviobj.centerPosition / 2) - 5 });
                        rectoptions = {
                            x1: lineleft,
                            y1: '0',
                            x2: lineleft,
                            y2: naviobj.centerPosition,
                            'stroke': naviobj.model.labelSettings.higherLevel.gridLineStyle.color,
                            'stroke-width': naviobj.model.labelSettings.higherLevel.gridLineStyle.width,
                            'stroke-dasharray':naviobj.model.labelSettings.higherLevel.gridLineStyle.dashArray,

                        };
                        if (i != 0)
                        naviobj.renderer.drawLine(rectoptions, naviobj.higherLevel);
                    }
                    else {
                        naviobj._higherTextNode[i].setAttribute("y", naviobj.newHeight - 5);
                        rectoptions = {
                            x1: lineleft,
                            y1: this.bottomPosition,
                            x2: lineleft,
                            y2: naviobj.newHeight,
                            'stroke': naviobj.model.labelSettings.higherLevel.gridLineStyle.color,
                            'stroke-width': naviobj.model.labelSettings.higherLevel.gridLineStyle.width,
                            'stroke-dasharray': naviobj.model.labelSettings.higherLevel.gridLineStyle.dashArray,

                        };
                        if (i != 0)
                        naviobj.renderer.drawLine(rectoptions, naviobj.higherLevel);
                    }
                    if ((setleft < this.padding || naviobj.newWidth - lineleft - this.padding < txtwidth) && type == "datetime") {
                        if (isrtl) {
                            if (setleft < this.padding && lineleft - this.padding > txtwidth) {
                                if (naviobj.model.labelSettings.higherLevel.style.horizontalAlignment === "center")
                                    naviobj._higherTextNode[i].setAttribute("x", this.padding + (lineleft - this.padding) / 2 - txtwidth / 2);
                                else if (naviobj.model.labelSettings.higherLevel.style.horizontalAlignment === "left")
                                    naviobj._higherTextNode[i].setAttribute("x", this.padding + 10);
                                else
                                    naviobj._higherTextNode[i].setAttribute("x", lineleft - txtwidth - 2);
                            }
                            else if (naviobj.newWidth - lineleft < txtwidth && naviobj.newWidth - newleft - this.padding > txtwidth) {
                                if (naviobj.model.labelSettings.higherLevel.style.horizontalAlignment === "center")
                                    naviobj._higherTextNode[i].setAttribute("x", newleft + (naviobj.newWidth - newleft - this.padding) / 2 - txtwidth / 2);
                                else if (naviobj.model.labelSettings.higherLevel.style.horizontalAlignment === "left")
                                    naviobj._higherTextNode[i].setAttribute("x", newleft + 10);
                                else
                                    naviobj._higherTextNode[i].setAttribute("x", naviobj.newWidth - txtwidth - 2 - this.padding);
                            }
                            else
                                naviobj.higherLevel.removeChild(naviobj._higherTextNode[i]);
                        }
                        else {
                            naviobj.higherLevel.removeChild(naviobj._higherTextNode[i]);
                            continue;
                        }
                    }
                    else if (setleft + txtwidth > naviobj.newWidth - this.padding && type == "datetime") {
                        if (naviobj.model.labelSettings.higherLevel.style.horizontalAlignment === "center")
                            setleft = lineleft + (naviobj.newWidth - lineleft - this.padding) / 2 - txtwidth / 2;
                        else if (naviobj.model.labelSettings.higherLevel.style.horizontalAlignment === "left")
                            setleft = lineleft;
                        else
                            setleft = naviobj.newWidth - txtwidth * 3/2;
                        naviobj._higherTextNode[i].setAttribute("x", setleft);
                        continue;
                    }
                    if (!this.model.enableRTL) {
                        lineleft = naviobj._higherTotalValues[i + 1] * this.eachInterval + this.padding;
                    } else {
                        lineleft= naviobj.newWidth - (naviobj._higherTotalValues[i -1] * this.eachInterval + this.padding);
                    }
                    count++;
                }

                

            }
            else if (naviobj.model.labelSettings.lowerLevel.visible === true) {

                naviobj._lowerLineLeft = [];
                if (!naviobj.model.enableRTL) {
                    lineleft = (naviobj._lowerTotalValues[count] * this.eachInterval) + this.padding;
                } else {
                    lineleft = naviobj.newWidth - ((naviobj._lowerTotalValues[naviobj._lowerTextNode.length - 1] * this.eachInterval) + this.padding);
                }
                var hy = naviobj.model.labelSettings.higherLevel.visible ? naviobj.bottomPosition : naviobj.centerHeight;

                if (naviobj.model.labelSettings.lowerLevel.position === "bottom" && naviobj.model.labelSettings.higherLevel.position === "top") {
                    rectoptions = {
                        x: this.padding,
                        y: hy,
                        width: this.newWidth - naviobj.padding * 2,
                        height: naviobj.minLowHeight - 1,
                        'stroke': naviobj.model.labelSettings.lowerLevel.border.color,
                        'stroke-width': naviobj.model.labelSettings.lowerLevel.border.width,
                        fill: 'transparent'
                    };
                    if (type != "numeric")
                        naviobj.renderer.drawRect(rectoptions, naviobj.lowerLevel);
                }
                else if (naviobj.model.labelSettings.lowerLevel.position === "bottom" && naviobj.model.labelSettings.higherLevel.position === "bottom") {
                    rectoptions = {
                        x: this.padding,
                        y: naviobj.centerHeight,
                        width: this.newWidth - naviobj.padding * 2,
                        height: naviobj.minLowHeight - 1,
                        'stroke': naviobj.model.labelSettings.lowerLevel.border.color,
                        'stroke-width': naviobj.model.labelSettings.lowerLevel.border.width,
                        fill: 'transparent'
                    };
                    naviobj.renderer.drawRect(rectoptions, naviobj.lowerLevel);
                }
                else if (naviobj.model.labelSettings.lowerLevel.position === "top" && naviobj.model.labelSettings.higherLevel.position === "bottom") {
                    rectoptions = {
                        x: this.padding,
                        y: 0,
                        width: this.newWidth - naviobj.padding * 2,
                        height: naviobj.minLowHeight - 1,
                        'stroke': naviobj.model.labelSettings.lowerLevel.border.color,
                        'stroke-width': naviobj.model.labelSettings.lowerLevel.border.width,
                        fill: 'transparent'
                    };
                    naviobj.renderer.drawRect(rectoptions, naviobj.lowerLevel);
                }
                else if (naviobj.model.labelSettings.higherLevel.position === "top" && naviobj.model.labelSettings.lowerLevel.position === "top") {
                    rectoptions = {
                        x: this.padding,
                        y: naviobj.minHighHeight - 1,
                        width: this.newWidth - naviobj.padding * 2,
                        height: naviobj.minLowHeight,
                        'stroke': naviobj.model.labelSettings.lowerLevel.border.color,
                        'stroke-width': naviobj.model.labelSettings.lowerLevel.border.width,
                        fill: 'transparent'
                    };
                    naviobj.renderer.drawRect(rectoptions, naviobj.lowerLevel);
                }

                if (naviobj.model.enableRTL) {
                    i = naviobj._lowerTextNode.length - 1;
                    isrtl = true;
                }
                else {
                    i = 0;
                    isrtl = false;
                }


                for (; isrtl ? i >= 0 : i < naviobj._lowerTextNode.length; isrtl?i--:i++) {
                    naviobj._lowerLineLeft[i] = lineleft;
                    labelfont = { size: naviobj.model.labelSettings.lowerLevel.style.font.size, fontStyle: naviobj.model.labelSettings.lowerLevel.style.font.style, fontFamily: naviobj.model.labelSettings.lowerLevel.style.font.family };
                    textOffset = ej.EjSvgRender.utils._measureText(naviobj.lowerLevel.childNodes[count].textContent, null, labelfont); // var txt = naviobj._lowerTextNode[i].getBBox();
                    txtwidth = textOffset.width;// txt.width;
                    var setleft = 0;
                    newleft = naviobj._lowerTotalValues[i + 1] === undefined
                        ? isrtl ? 0 : naviobj.newWidth :
                        isrtl ? naviobj.newWidth - ((naviobj._lowerTotalValues[i + 1] * this.eachInterval + this.padding)) :
                        (naviobj._lowerTotalValues[i + 1] * this.eachInterval + this.padding);
                    if (naviobj.model.labelSettings.lowerLevel.style.horizontalAlignment === "center") {
                        setleft = isrtl ? (lineleft - newleft) / 2 + newleft - txtwidth / 2 : (newleft - lineleft) / 2 + lineleft - txtwidth / 2;
                    }
                    else if (naviobj.model.labelSettings.lowerLevel.style.horizontalAlignment === "left") {
                        setleft = isrtl? newleft + 2 : lineleft + 2;
                    }
                    else
                        setleft = isrtl ? lineleft - txtwidth - 2 : newleft - txtwidth - 2;
                    if (type == "numeric")
                        setleft = lineleft - txtwidth / 2;
                    naviobj.renderer._setAttr($(naviobj._lowerTextNode[i]), { 'x': setleft });
                    naviobj.renderer._setAttr($(naviobj._lowerTextNode[i]), { 'fill': naviobj.model.labelSettings.lowerLevel.style.font.color });

                    naviobj._lowerTextLeft[i] = setleft;

                    if (naviobj.model.labelSettings.lowerLevel.position === "bottom" && naviobj.model.labelSettings.higherLevel.position === "top") {
                        if (!this.svgSupport)
                            naviobj.renderer._setAttr($(naviobj._lowerTextNode[i]), { 'y': naviobj.newHeight - textOffset.height });
                        else {
                            if (matched.browser.toLowerCase() == "msie")
                                naviobj._lowerTextNode[i].setAttribute("y", naviobj.newHeight - 5);
                            else
                                naviobj._lowerTextNode[i].setAttribute("y", naviobj.newHeight - 9);

                        }
                        var ht = naviobj.model.labelSettings.higherLevel.visible ? parseFloat(naviobj.minHighHeight + naviobj.centerHeight) : naviobj.minHighHeight;
                        rectoptions = {
                            x1: lineleft,
                            y1: ht,
                            x2: lineleft,
                            y2: naviobj.newHeight,
                            'stroke': this.lgClr,
                            'stroke-width': naviobj.model.labelSettings.lowerLevel.gridLineStyle.width,
                            'stroke-dasharray': naviobj.model.labelSettings.lowerLevel.gridLineStyle.dashArray,
                        };
                        if (i != 0 && type != "numeric")
                            naviobj.renderer.drawLine(rectoptions, naviobj.lowerLevel);
                    }
                    else if (naviobj.model.labelSettings.lowerLevel.position === "bottom" && naviobj.model.labelSettings.higherLevel.position === "bottom") {
                        naviobj._lowerTextNode[i].setAttribute("y", naviobj.bottomPosition - 5);
                        rectoptions = {
                            x1: lineleft,
                            y1: naviobj.centerHeight,
                            x2: lineleft,
                            y2: naviobj.bottomPosition,
                            'stroke': naviobj.model.labelSettings.lowerLevel.gridLineStyle.color,
                            'stroke-width': naviobj.model.labelSettings.lowerLevel.gridLineStyle.width,
                            'stroke-dasharray':naviobj.model.labelSettings.lowerLevel.gridLineStyle.dashArray,
                        };
                        if (i != 0)
                            naviobj.renderer.drawLine(rectoptions, naviobj.lowerLevel);
                    }
                    else if (naviobj.model.labelSettings.lowerLevel.position === "top" && naviobj.model.labelSettings.higherLevel.position === "bottom") {
                        naviobj._lowerTextNode[i].setAttribute("y", naviobj.centerPosition - 5);
                        rectoptions = {
                            x1: lineleft,
                            y1: 0,
                            x2: lineleft,
                            y2: naviobj.centerPosition,
                            'stroke': naviobj.model.labelSettings.lowerLevel.gridLineStyle.color,
                            'stroke-width': naviobj.model.labelSettings.lowerLevel.gridLineStyle.width,
                            'stroke-dasharray': naviobj.model.labelSettings.lowerLevel.gridLineStyle.dashArray,
                        };
                        if (i != 0)
                            naviobj.renderer.drawLine(rectoptions, naviobj.lowerLevel);
                    }
                    else if (naviobj.model.labelSettings.higherLevel.position === "top" && naviobj.model.labelSettings.lowerLevel.position === "top") {
                        naviobj._lowerTextNode[i].setAttribute("y", naviobj.centerPosition - 5);
                        rectoptions = {
                            x1: lineleft,
                            y1: naviobj.minHighHeight,
                            x2: lineleft,
                            y2: naviobj.centerPosition,
                            'stroke': naviobj.model.labelSettings.lowerLevel.gridLineStyle.color,
                            'stroke-width': naviobj.model.labelSettings.lowerLevel.gridLineStyle.width,
                            'stroke-dasharray': naviobj.model.labelSettings.lowerLevel.gridLineStyle.dashArray,
                        };
                        if (i != 0)
                            naviobj.renderer.drawLine(rectoptions, naviobj.lowerLevel);
                    }
                    if ((setleft < this.padding || naviobj.newWidth - lineleft - this.padding < txtwidth) && type == "datetime") {
                        if (isrtl) {
                            if (setleft < this.padding && lineleft - this.padding > txtwidth) {
                                if (naviobj.model.labelSettings.lowerLevel.style.horizontalAlignment === "center")
                                    naviobj._lowerTextNode[i].setAttribute("x", this.padding + (lineleft - this.padding) / 2 - txtwidth / 2);
                                else if (naviobj.model.labelSettings.lowerLevel.style.horizontalAlignment === "left")
                                    naviobj._lowerTextNode[i].setAttribute("x", this.padding + 10);
                                else
                                    naviobj._lowerTextNode[i].setAttribute("x", lineleft - txtwidth - 2);
                            }
                            else if (naviobj.newWidth - lineleft < txtwidth && naviobj.newWidth -this.padding - newleft > txtwidth) {
                                if (naviobj.model.labelSettings.lowerLevel.style.horizontalAlignment === "center")
                                    naviobj._lowerTextNode[i].setAttribute("x", newleft + (naviobj.newWidth - newleft - this.padding) / 2 - txtwidth / 2);
                                else if (naviobj.model.labelSettings.lowerLevel.style.horizontalAlignment === "left")
                                    naviobj._lowerTextNode[i].setAttribute("x", newleft + 10);
                                else
                                    naviobj._lowerTextNode[i].setAttribute("x", naviobj.newWidth - txtwidth - 2 - this.padding);
                            }
                            else
                                naviobj.lowerLevel.removeChild(naviobj._lowerTextNode[i]);
                            count--;
                        }
                        else {
                            naviobj.lowerLevel.removeChild(naviobj._lowerTextNode[i]);
                            count--;
                        }
                    }
                    else if (setleft + txtwidth > naviobj.newWidth - this.padding && type == "datetime") {
                        if (naviobj.model.labelSettings.lowerLevel.style.horizontalAlignment === "center")
                            setleft = lineleft + (naviobj.newWidth - lineleft - this.padding) / 2 - txtwidth / 2;
                        else if (naviobj.model.labelSettings.lowerLevel.style.horizontalAlignment === "left")
                            setleft = lineleft;
                        else
                            setleft = naviobj.newWidth - txtwidth * 3 / 2;
                        naviobj._lowerTextNode[i].setAttribute("x", setleft);                        
                    }
                    if (!this.model.enableRTL)
                        lineleft = naviobj._lowerTotalValues[i + 1] * this.eachInterval + this.padding;
                    else
                        lineleft = naviobj.newWidth - (naviobj._lowerTotalValues[i - 1] * this.eachInterval + this.padding);
                    count++;
                }



            }
            naviobj._setGridlines(naviobj);
            $(this.higherLevel).css('cursor', 'pointer');
            $(this.lowerLevel).css('cursor', 'pointer');
            //this.higherLevel.setAttributeNS(null, 'transform', 'translate(' + 20 + ',' + 0 + ')');
            //this.lowerLevel.setAttributeNS(null, 'transform', 'translate(' + 20 + ',' + 0 + ')');

        },

        _setGridlines: function (naviobj) {

            if (naviobj.model.navigatorStyleSettings.minorGridLineStyle.visible === true) {

                for (var i = 1; i < naviobj._lowerTextNode.length; i++) {
                    if (naviobj.model.labelSettings.higherLevel.position == "top" && naviobj.model.labelSettings.lowerLevel.position == "top") {
                        var rectoptions = {
                            x1: naviobj._lowerLineLeft[i],
                            y1: naviobj.centerHeight + naviobj.minLowHeight+naviobj.minHighHeight,
                            x2: naviobj._lowerLineLeft[i],
                            y2: naviobj.centerPosition,
                            'stroke': naviobj.model.navigatorStyleSettings.minorGridLineStyle.color,
                            'stroke-width': '1'
                        };
                        naviobj.renderer.drawLine(rectoptions, naviobj.centerLevel);
                    }
                    else {
                        var rectoptions = {
                            x1: naviobj._lowerLineLeft[i],
                            y1: naviobj.centerPosition,
                            x2: naviobj._lowerLineLeft[i],
                            y2: naviobj.centerPosition+naviobj.centerHeight,
                            'stroke': naviobj.model.navigatorStyleSettings.minorGridLineStyle.color,
                            'stroke-width': '1'
                        };
                        naviobj.renderer.drawLine(rectoptions, naviobj.centerLevel);
                    }
                    
                }
            }
            else {
                for (var i = 0; i < naviobj._higherTextNode.length; i++) {
                    // var lineLeft = naviobj._lowerLineLeft[i];
                    var rectoptions = {
                        x1: naviobj._higherLineLeft[i],
                        y1: naviobj.centerHeight + naviobj.minLowHeight,
                        x2: naviobj._higherLineLeft[i],
                        y2: naviobj.centerPosition,
                        'stroke': naviobj.model.navigatorStyleSettings.majorGridLineStyle.color,
                        'stroke-width': '1'
                    };
                    naviobj.renderer.drawLine(rectoptions, naviobj.centerLevel);
                }
            }

        },

        bindTo: function () {
            if (!ej.util.isNullOrUndefined(this.model.dataSource)) {
                if (this.model.dataSource != null && this.model.dataSource.length > 0) {

                    this._processJsonData(this.model.dataSource, this.model);
                }
            }

        },

        _processJsonData: function (jsonObj, newmodel) {

            var xdata = [];
            var yValues = [];
            var index = 0;

            for (var j = 0; j < jsonObj.length; j++) {
                if (typeof jsonObj[j][newmodel.xName] == "string" && jsonObj[j][newmodel.xName].indexOf("/Date(") != -1)
                    jsonObj[j][newmodel.xName] = new Date(parseInt(jsonObj[j][newmodel.xName].substr(6)));
                if (jQuery.type(jsonObj[j][newmodel.xName]) != "string" && jQuery.type(jsonObj[j][newmodel.xName]) != "date")
                    xdata.push(parseFloat(!ej.util.isNullOrUndefined(jsonObj[j][newmodel.xName]) ? jsonObj[j][newmodel.xName] : j));
                else {
                    xdata.push(!ej.util.isNullOrUndefined(jsonObj[j][newmodel.xName]) ? jsonObj[j][newmodel.xName] : j);
                }
            }

            if (!ej.util.isNullOrUndefined(newmodel.yName)) {
                for (var k = 0; k < newmodel.yName.length; k++) {
                    var yNVal = [];
                    for (var yI = 0; yI < jsonObj.length; yI++) {
                        yNVal.push(jsonObj[yI][newmodel.yName[k]] === null || jsonObj[yI][newmodel.yName[k]] === undefined ? jsonObj[yI][newmodel.yName[k]] : parseFloat(jsonObj[yI][newmodel.yName[k]]));
                    }
                    yValues[k] = yNVal;
                    index = k + 1;
                }
            }

            this.endDateTime = new Date(Math.max.apply(null, xdata));
            this.startDateTime = new Date(Math.min.apply(null, xdata));
            this.endValue = Math.max.apply(null, xdata);
            this.startValue = Math.min.apply(null, xdata);
        },

        calTouchPosition: function (e) {
            var matched = jQuery.uaMatch(navigator.userAgent);
            var mouseX, mouseY;
            
            if (matched.browser.toLowerCase() == "mozilla" || matched.browser.toLowerCase() == "webkit") {
                var touch = e.originalEvent.touches[0];
                mouseX = (touch.pageX) - $(this.svgDocument).parent().offset().left;
                mouseY = (touch.pageY) - $(this.svgDocument).parent().offset().top;
                this.leftPadding = $(this.svgDocument).offset().left;
                this.grabPadding = 0;
            } else if (matched.browser.toLowerCase() == "msie") {

                mouseX = (e.originalEvent.pageX) - $(this.svgDocument).offset().left;
                mouseY = (e.originalEvent.pageY) - $(this.svgDocument).offset().top;
                this.leftPadding = $(this.svgDocument).offset().left;
            }
            else {
                var touch = e.originalEvent.touches[0];
                mouseX = (touch.pageX) - $(this.svgDocument).offset().left;
                mouseY = (touch.pageY) - $(this.svgDocument).offset().top;
                this.leftPadding = $(this.svgDocument).offset().left;
            }

            return { X: mouseX, Y: mouseY };

        },

        calMousePosition: function (e) {
            var matched = jQuery.uaMatch(navigator.userAgent);
            var mouseX, mouseY;
            if (matched.browser.toLowerCase() == "mozilla" || matched.browser.toLowerCase() == "webkit") {
                mouseX = (e.pageX) - $(this.svgDocument).parent().offset().left;
                mouseY = (e.pageY) - $(this.svgDocument).parent().offset().top;
                this.leftPadding = 0;//$(this.svgDocument).offset().left;
                this.grabPadding = 0;
            } else {
                mouseX = (e.pageX) - $(this.svgDocument).offset().left;
                mouseY = (e.pageY) - $(this.svgDocument).offset().top;
                this.leftPadding = $(this.svgDocument).offset().left;
            }

            return { X: mouseX, Y: mouseY };

        },

        calTouchPosition: function (e) {
            var matched = jQuery.uaMatch(navigator.userAgent);
            var mouseX, mouseY;

            if (matched.browser.toLowerCase() == "mozilla" || matched.browser.toLowerCase() == "webkit") {
                var touch = e.originalEvent.touches[0];
                mouseX = (touch.pageX) - $(this.svgDocument).parent().offset().left;
                mouseY = (touch.pageY) - $(this.svgDocument).parent().offset().top;
                this.leftPadding = $(this.svgDocument).offset().left;
                this.grabPadding = 0;
            } else if (matched.browser.toLowerCase() == "msie") {

                mouseX = (e.originalEvent.pageX) - $(this.svgDocument).offset().left;
                mouseY = (e.originalEvent.pageY) - $(this.svgDocument).offset().top;
                this.leftPadding = $(this.svgDocument).offset().left;
            }
            else {
                var touch = e.originalEvent.touches[0];
                mouseX = (touch.pageX) - $(this.svgDocument).offset().left;
                mouseY = (touch.pageY) - $(this.svgDocument).offset().top;
                this.leftPadding = $(this.svgDocument).offset().left;
            }

            return { X: mouseX, Y: mouseY };

        },

        //calMousePosition: function (e) {
        //    var matched = jQuery.uaMatch(navigator.userAgent);
        //    var mouseX, mouseY;
        //    if (matched.browser.toLowerCase() == "mozilla" || matched.browser.toLowerCase() == "webkit") {
        //        mouseX = (e.pageX) - $(this.svgDocument).parent().offset().left;
        //        mouseY = (e.pageY) - $(this.svgDocument).parent().offset().top;
        //        this.leftPadding = $(this.svgDocument).offset().left;
        //        this.grabPadding = 0;
        //    } else {
        //        mouseX = (e.pageX) - $(this.svgDocument).offset().left;
        //        mouseY = (e.pageY) - $(this.svgDocument).offset().top;
        //        this.leftPadding = $(this.svgDocument).offset().left;
        //    }

        //    return { X: mouseX, Y: mouseY };

        //},

        _grab: function (evt) {
            evt.preventDefault();
            var matched = jQuery.uaMatch(navigator.userAgent);
            var naviobj = this;// evt.data.Param1;
            var mousedownCords;
            if ((this.svgSupport && evt.originalEvent.toString() !== "[object TouchEvent]") && matched.browser.toLowerCase() != "msie") {
                mousedownCords = naviobj.calMousePosition(evt);
            } else if (!this.svgSupport) {
                mousedownCords = naviobj.calMousePosition(evt);
            } else {
                mousedownCords = naviobj.calTouchPosition(evt);
            } // naviobj.calMousePosition(evt);
            naviobj.mouseDownX = mousedownCords.X;
            naviobj.mouseDownY = mousedownCords.Y;
            // find out which element we moused down on
            
            var targetElement;
            if (naviobj.edge == true) {
                targetElement = naviobj.target;
            }
            else {
                targetElement = evt.target;
            }
            
            //$(targetElement).css('cursor', 'w-resize');
            // you cannot drag the background itself, so ignore any attempts to mouse down on it
            if (targetElement.parentNode.id === "leftslider" || targetElement.parentNode.id === "rightslider" || targetElement.parentNode.id === "centerslider" || targetElement.parentNode.id === this._rootId + "higherLevel" || targetElement.parentNode.id === this._rootId + "lowerLevel") {
                //set the item moused down on as the element to be dragged
                naviobj.dragTarget = targetElement;
                naviobj.grabbed = true;
                naviobj.edge = false;
                if (this.svgSupport) {
                        var transMatrix = naviobj.dragTarget.getCTM();
                    if (matched.browser.toLowerCase() == "mozilla" || matched.browser.toLowerCase() == "webkit") {
                        naviobj.grabPoint.x = naviobj.mouseDownX + this.grabPadding - Number(transMatrix.e);
                    }
                    else
                        naviobj.grabPoint.x = naviobj.mouseDownX + this.leftPadding - Number(transMatrix.e);
                    naviobj.grabPoint.y = naviobj.trueCoords.y - Number(transMatrix.f);
                }
                var txtPosition;
                var i;
                var flag = false;
                var isrtl = naviobj.model.enableRTL;
                if (isrtl) {
                    naviobj._higherLineLeft.sort(function (a, b) { return a - b });
                    naviobj._lowerLineLeft.sort(function (a, b) { return a - b });
                }
                if (naviobj.dragTarget.parentNode.id === this._rootId + "higherLevel" && naviobj.model.valueType == "datetime") {
                    naviobj.grabbed = false;
                    txtPosition = naviobj.mouseDownX;
                    for (i=0;i< naviobj._higherLineLeft.length;i++) {
                        if (i + 1 < naviobj._higherLineLeft.length && txtPosition > naviobj._higherLineLeft[i] && txtPosition < naviobj._higherLineLeft[i + 1]) {
                            naviobj.setSliderPositions(naviobj._higherLineLeft[i], null, null);
                            naviobj.setSliderPositions(null, null, naviobj._higherLineLeft[i + 1]);
                            flag = true;
                        }
                        else if (i + 1 === naviobj._higherLineLeft.length && txtPosition > naviobj._higherLineLeft[i]) {
                            naviobj.setSliderPositions(naviobj._higherLineLeft[i], null, null);
                            naviobj.setSliderPositions(null, null, naviobj.newWidth - naviobj.padding);
                            flag = true;
                        }
                    }
                    if (isrtl && !flag) {
                        naviobj.setSliderPositions(0 + naviobj.padding, null, null);
                        naviobj.setSliderPositions(null, null, naviobj._higherLineLeft[0]);
                        flag = false;
                    }
                }
                else if (naviobj.dragTarget.parentNode.id === this._rootId + "lowerLevel" && naviobj.model.valueType == "datetime") {
                    naviobj.grabbed = false;
                    txtPosition = naviobj.mouseDownX;
                    for (i = 0; i < naviobj._lowerLineLeft.length; i++) {
                        if (i + 1 < naviobj._lowerLineLeft.length && txtPosition > naviobj._lowerLineLeft[i] && txtPosition < naviobj._lowerLineLeft[i + 1]) {
                            naviobj.setSliderPositions(naviobj._lowerLineLeft[i], null, null);
                            naviobj.setSliderPositions(null, null, naviobj._lowerLineLeft[i + 1]);
                            flag = true;
                        }
                        else if (i + 1 === naviobj._lowerLineLeft.length && txtPosition > naviobj._lowerLineLeft[i]) {
                            naviobj.setSliderPositions(naviobj._lowerLineLeft[i], null, null);
                            naviobj.setSliderPositions(null, null, naviobj.newWidth - naviobj.padding);
                            flag = true;
                        }
                    }
                    if (isrtl && !flag) {
                        naviobj.setSliderPositions(0 + naviobj.padding, null, null);
                        naviobj.setSliderPositions(null, null, naviobj._lowerLineLeft[0]);
                        flag = false;
                    }
                }

            }
        },

        _drag: function (evt) {

            evt.preventDefault();
            var matched = jQuery.uaMatch(navigator.userAgent);

            var naviobj = this;//evt.data.Param1;
            if ((this.svgSupport && evt.originalEvent.toString() !== "[object TouchEvent]") && matched.browser.toLowerCase() != "msie") {
                var mousedownCords = naviobj.calMousePosition(evt);
            }
            else if (!this.svgSupport)
                var mousedownCords = naviobj.calMousePosition(evt);
            else
                var mousedownCords = naviobj.calTouchPosition(evt);//naviobj.calMousePosition(evt);
            naviobj.mouseDownX = mousedownCords.X;
            naviobj.mouseDownY = mousedownCords.Y;
            if(this.svgSupport)
            naviobj.GetTrueCoords(evt);
            // if we don't currently have an element in tow, don't do anything
            if (naviobj.dragTarget) {
                // account for the offset between the element's origin and the
                //    exact place we grabbed it... this way, the drag will look more natural
                if (!naviobj.edge && naviobj.dragTarget === naviobj.leftSlider || naviobj.dragTarget === naviobj.rightSlider || naviobj.dragTarget === naviobj.centerSlider) {
                    if (this.svgSupport) {
					if(matched.browser.toLowerCase() == "webkit"||matched.browser.toLowerCase() == "mozilla")
					{
                        var newX = naviobj.mouseDownX - naviobj.grabPoint.x;
                        var newY = naviobj.mouseDownY - naviobj.grabPoint.y;
						}
						else
						{
						var newX = naviobj.trueCoords.x - naviobj.grabPoint.x;
                        var newY = naviobj.trueCoords.y - naviobj.grabPoint.y;
						}
                    }
                    else {
                        var newX = naviobj.mouseDownX;
                        var newY = naviobj.mouseDownY;
                    }
                }
                else {
                    var newX = naviobj.mouseDownX;// naviobj.trueCoords.x - naviobj.leftPadding;
                    //  var newX = naviobj.mouseDownX;// naviobj.trueCoords.x - naviobj.leftPadding;
                    var newY = naviobj.mouseDownY;// naviobj.trueCoords.y;
                }
                naviobj.diff = naviobj.leftSlider.getBoundingClientRect().left - naviobj.rightSlider.getBoundingClientRect().left;
                naviobj.leftdiff = naviobj.rightSlider.getBoundingClientRect().left - this.padding;
                naviobj.rightdiff = naviobj.leftSlider.getBoundingClientRect().left - this.padding;
                if (naviobj.dragTarget.parentNode.id === "leftslider") {
                    naviobj.grabbed = false;
                    if (naviobj.diff >= -1 && naviobj.diff <= 1 && naviobj.switched == false && naviobj.rightdiff<naviobj.newWidth-(2*this.padding)) {
                        naviobj.target = naviobj.rightSlider;
                        naviobj.switched = true;
                        naviobj.edge = true;
                        naviobj._grab(evt);
                        naviobj._drag(evt);
                    }
                    else {
                        if (newX <= this.rightSliderPosition) {
                            naviobj.setSliderPositions(newX, null, null);
                            this.leftSliderPosition = newX;
                            this.leftset = false;
                        }
                        else {
                            naviobj.setSliderPositions(this.rightSliderPosition, null, null);
                            this.leftset = true;
                            this.leftSliderPosition = this.rightSliderPosition;
                        }
                        if (naviobj.diff >= 1 || naviobj.diff <= -1)
                            naviobj.switched = false;
                    }
                    naviobj.left = newX;
                }
                else if (naviobj.dragTarget.parentNode.id === "rightslider") {
                    naviobj.grabbed = false;
                    // if (((this.svgSupport) && naviobj.leftSlider.getCTM().e == naviobj.rightSlider.getCTM().e && newX < naviobj.newWidth-15 && naviobj.rightSlider.getCTM().e == (naviobj.newWidth - naviobj.padding)) || (naviobj.leftSlider.getBoundingClientRect().left == naviobj.rightSlider.getBoundingClientRect().left && newX < naviobj.newWidth - 15 && naviobj.rightSlider.getBoundingClientRect().left == (naviobj.newWidth - naviobj.padding))) {                                                                
                    if (naviobj.diff >= 0 && naviobj.diff <= 1 && naviobj.switched == false && naviobj.leftdiff >0) {
                        naviobj.target = naviobj.leftSlider;
                        naviobj.switched = true;
                        naviobj.edge = true;
                        naviobj._grab(evt);
                        naviobj._drag(evt);
                    }
                    else {
                        //if (newX >= naviobj.leftSlider.getCTM().e)
                        if (newX >= this.leftSliderPosition) { // naviobj.leftSlider.getBoundingClientRect().left - this.leftPadding)
                            naviobj.setSliderPositions(null, null, newX);
                            this.rightSliderPosition = newX;
                        } else {
                            naviobj.setSliderPositions(null, null, this.leftSliderPosition);
                            this.rightSliderPosition = this.leftSliderPosition;
                        }
                        //naviobj.setSliderPositions(null, null, naviobj.leftSlider.getCTM().e);
                        if (naviobj.diff >= 1 || naviobj.diff <= -1)
                            naviobj.switched = false;
                    }
                    naviobj.right = newX;
                }
                else if (naviobj.dragTarget.parentNode.id === "centerslider") {
                    naviobj.grabbed = false;
                    naviobj.setSliderPositions(null, parseFloat(newX), null);
                    if (newX > newX + this.renderer._getAttrVal($(naviobj.dragTarget), 'width'))
                        naviobj.setSliderPositions(null, newX, null);
                    naviobj.center = newX;
                } 
            }
        },


        _drop: function (evt) {
            // if we aren't currently dragging an element, don't do anything
            if (this.dragTarget) {
                // since the element currently being dragged has its pointer-events turned off,
                //    we are afforded the opportunity to find out the element it's being dropped on
                var targetElement = evt.target;

                var naviobj = this;
                // set the global variable to null, so nothing will be dragged until we
                //    grab the next element
                if (naviobj.model.allowSnapping && parseInt(naviobj.leftSlider.getBoundingClientRect().left+2) < parseInt( naviobj.rightSlider.getBoundingClientRect().left)) {
                    naviobj.ismouseup = true;
                    if (naviobj.dragTarget.parentNode.id === "leftslider")
                        var txtPosition = naviobj.left;
                    else if (naviobj.dragTarget.parentNode.id === "rightslider")
                        var txtPosition = naviobj.right;
                    else if (naviobj.dragTarget.parentNode.id === "centerslider")
                        var txtPosition = naviobj.center;
                    var pos = 0;

                    for (var i = 0; i < naviobj._lowerLineLeft.length; i++) {
                        if (i + 1 < naviobj._lowerLineLeft.length && txtPosition > naviobj._lowerLineLeft[i] && txtPosition < naviobj._lowerLineLeft[i + 1]) {
                            naviobj.snapValue = naviobj._lowerLineLeft[i];
                            pos = (naviobj._lowerLineLeft[i + 1] - naviobj.snapValue);
                            if (txtPosition >= (naviobj.snapValue + (pos / 2))) {
                                naviobj.snapValue = naviobj._lowerLineLeft[i + 1];
                                if (naviobj.dragTarget.parentNode.id === "leftslider")
                                    naviobj.setSliderPositions(naviobj._lowerLineLeft[i + 1], null, null);
                                else if (naviobj.dragTarget.parentNode.id === "rightslider")
                                    naviobj.setSliderPositions(null, null, naviobj._lowerLineLeft[i + 1]);
                                else if (naviobj.dragTarget.parentNode.id === "centerslider") {
                                    naviobj.setSliderPositions(null, null, naviobj._lowerLineLeft[i+1] + parseFloat(naviobj.centerSlider.getAttribute("width")));
                                    naviobj.setSliderPositions(naviobj._lowerLineLeft[i + 1], null, null);
                                }

                            }
                            else {
                                naviobj.snapValue = naviobj._lowerLineLeft[i];
                                if (naviobj.dragTarget.parentNode.id === "leftslider")
                                    naviobj.setSliderPositions(naviobj._lowerLineLeft[i], null, null);
                                else if (naviobj.dragTarget.parentNode.id === "rightslider")
                                    naviobj.setSliderPositions(null, null, naviobj._lowerLineLeft[i]);
                                else if (naviobj.dragTarget.parentNode.id === "centerslider") {
                                    naviobj.setSliderPositions(null, null, naviobj._lowerLineLeft[i] + parseFloat(naviobj.centerSlider.getAttribute("width")));
                                    naviobj.setSliderPositions(naviobj._lowerLineLeft[i], null, null);
                                }

                            }
                        }
                        else if (i + 1 === naviobj._lowerLineLeft.length && txtPosition > naviobj._lowerLineLeft[i]) {
                            naviobj.setSliderPositions(null, null,naviobj.newWidth-naviobj.padding);
                        }
                    }
                }
                this.dragTarget = null;
               
                if (this.leftTooltip != undefined && this.model.tooltipSettings.tooltipDisplayMode=="always") {
                    this.leftTooltip.setAttribute('opacity', 1);
                    this.rightTooltip.setAttribute('opacity', 1);
                }
                else if (this.leftTooltip != undefined && this.model.tooltipSettings.tooltipDisplayMode == "ondemand") {
                    this.leftTooltip.setAttribute('opacity', 0);
                    this.rightTooltip.setAttribute('opacity', 0);
                }
                if (this.model.enableDeferredUpdate) {
                    this._calculateSelectedData();
                    if (!naviobj.grabbed) {
                        if (this.zoomp != this.model.zoomPosition || this.zoomf != this.model.zoomFactor) {
                            this._callRangeChange(this);
                            this.zoomp = this.model.zoomPosition;
                            this.zoomf = this.model.zoomFactor;
                        }
                        
                    }
                }
            }
        },

        _callRangeChange: function (args) {
            args._trigger("rangeChanged", args.model);
        },

        mouseup: function (evt) {
            if (this.dragTarget != null) {
                this._drop(evt);
                if (this.dragTarget != null) {
                    this.dragTarget.setAttributeNS(null, 'pointer-events', 'all');
                    this.dragTarget = null;
                }
                if (this.model.enableDeferredUpdate) {
                    this._calculateSelectedData();
                    this._trigger("rangeChanged", this.model);
                  
                }
            }
        },

        GetTrueCoords: function (evt) {
            // find the current zoom level and pan setting, and adjust the reported
            //    mouse position accordingly
            var newScale = this.svgDocument.currentScale;
            var translation = this.svgDocument.currentTranslate;
            var matched = jQuery.uaMatch(navigator.userAgent);
            if (evt.originalEvent.toString() !== "[object TouchEvent]" && matched.browser.toLowerCase() != "msie") {
                this.trueCoords.x = (evt.clientX - translation.x) / newScale;
                this.trueCoords.y = (evt.clientY - translation.y) / newScale;
            }
            else if (matched.browser.toLowerCase() == "msie") {
                this.trueCoords.x = (evt.originalEvent.clientX - translation.x) / newScale;
                this.trueCoords.y = (evt.originalEvent.clientY - translation.y) / newScale;
            }
            else {
                this.trueCoords.x = (evt.originalEvent.touches[0].clientX - translation.x) / newScale;
                this.trueCoords.y = (evt.originalEvent.touches[0].clientY - translation.y) / newScale;
            }
        }
    });

	 ej.datavisualization.RangeNavigator.locale = {};
     ej.datavisualization.RangeNavigator.locale["en-US"] = {

        intervals: {
            quarter: {longQuarters: "Quarter", shortQuarters: "Q"},
            week: {longWeeks: "Week", shortWeeks: "W"}
        }
    };
})(jQuery, Syncfusion);

jQuery.uaMatch = function (ua) {
    ua = ua.toLowerCase();

    var match = /(chrome)[ \/]([\w.]+)/.exec(ua) ||
        /(webkit)[ \/]([\w.]+)/.exec(ua) ||
        /(opera)(?:.*version|)[ \/]([\w.]+)/.exec(ua) ||
        /(msie) ([\w.]+)/.exec(ua) ||
        ua.indexOf("compatible") < 0 && /(mozilla)(?:.*? rv:([\w.]+)|)/.exec(ua) ||
        [];

    return {
        browser: match[1] || "",
        version: match[2] || "0"
    };
};;
/**
* @fileOverview Plugin to style the Html Map elements
* @copyright Copyright Syncfusion Inc. 2001 - 2013. All rights reserved.
*  Use of this code is subject to the terms of our license.
*  A copy of the current license can be obtained at any time by e-mailing
*  licensing@syncfusion.com. Any infringement will be prosecuted under
*  applicable laws. 
* @version 12.1 
* @author &lt;a href="mailto:licensing@syncfusion.com"&gt;Syncfusion Inc&lt;/a&gt;
*/

(function ($, ej, undefined) {
    /**
    * @namespace ej
    * @class ejMap
    * @requires jQuery
    * @requires ej.datavisualization.Map
    * @requires jsrender
    * @requires properties
    * @classdesc The map can be easily configured to the DOM element, such as div and can be created with a highly customized look and feel.
    * @example 
    * &lt;div id="container"&gt; <br> 
    * &lt;script&gt;
    * // Create Map
    * $('#container').ejMap(); 	
    * &lt;/script&gt; <br>  
    * &lt;/div&gt;
    */
    
    ej.widget({ "ejMap": "ej.datavisualization.Map" }, {

        validTags: ["div"],
        _initPrivateProperties: function () {
            this._rootgroup = null;
            this._bubblegroup = null;           
            this._scale = 1;
            this._prevDistance = 0;
            this._tileTranslatePoint = { x: 0, y: 0 };
            this._translatePoint = { x: 0, y: 0 };
            this._prevPoint = null;
            this._Tiles = [];
            this._prevScale = 0;
            this._tileDiv = null;
            this._containerWidth = 500;
            this._containerHeight = 400;
            this._baseTranslatePoint = { x: 0, y: 0 };
            this._isDragging = false;           
            this._prevLevel = 1;
            this._browser = null;
            this._baseScale = 0;
            this._mapBounds = null;
            this._svgns = "http://www.w3.org/2000/svg";
            this._ispanning = false;
            this._dragStartX = 0;
            this._isNavigationPressed = false;
			this._iskeyboardKeysPressed = false;
            this._isPolygonSelected = false;
            this._dragStartY = 0;
            this._width = 500;
            this._height = 500;
            this._svgDocument = null;
            this._tooltipElement = null;
            this._templateDiv = null;
            this._mapContainer = null;
            this._isSVG = true;
            this._VMLPathFractionCount = 0;
            this._sliderControl = null; 
            this._isTileMap=false;
            this._isRendered = false;
            this._urlTemplate = null;
            this._polylineCount = 0;
            this._pointscount = 0;
            this._isPinching = false;
            this._groupSize = null;			
			this._groupBorder = { x: 0, y: 0 };
        },


        defaults: {
            /**		
			* Specifies the background color for map
            * @default "white"
			* @type {string}
			* @example 
            * //To set background API value during initialization 
            *   $("#container").ejMap({background:'white'});
            * @example 
			* //Get or set the background API, after initialization:<br>
			*	//Gets the background value 
            *   var property = $("#container").data("ejMap").model.background;<br>
            *	//Sets the background value 
			*	$("#container").data("ejMap").model.background="transparent";
			* @memberof ejMap
			* @instance
            */
            background: "white",


            zoomSettings:{
                /**		
                * Specifies the minimum zoomSettings level of the map
                * @default 1
                * @type {number}
                * @example 
                * //To set minValue API value during initialization 
                *   $("#container").ejMap({zoomSettings:{minValue:1}});
                * @example 
                * //Get or set the minValue API, after initialization:<br>
                *	//Gets the minValue value 
                * 	var property = $("#container").data("ejMap").model.zoomSettings.minValue;<br>
                *	//Sets the minValue value 
                *	$("#container").data("ejMap").model.zoomSettings.minValue= 1;
                * @memberof ejMap
                * @instance
                */
                minValue: 1,

                /**		
                * Specifies the maximum zoom level of the map
                * @default 100
                * @type {number}
                * @example 
                * //To set maxValue API value during initialization 
                *   $("#container").ejMap({zoomSettings:{maxValue:100}});
                * @example 
                * //Get or set the maxValue API, after initialization:<br>
                *	//Gets the maxValue value 
                * 	var property = $("#container").data("ejMap").model.zoomSettings.maxValue;<br>
                *	//Sets the maxValue value 
                *	$("#container").data("ejMap").model.zoomSettings.maxValue= 100;
                * @memberof ejMap
                * @instance
                */
                maxValue: 100,

                /**		
                * Specifies the zoom factor for map zoom value.
                * @default 1
                * @type {number}
                * @example 
                * //To set zoomFactor API value during initialization 
                *   $("#container").ejMap({zoomSettings:{factor:1}});
                * @example 
                * //Get or set the zoomFactor API, after initialization:<br>
                *	//Gets the zoomFactor value 
                * 	var property = $("#container").data("ejMap").model.zoomSettings.factor;<br>
                *	//Sets the zoomFactor value 
                *	$("#container").data("ejMap").model.zoomSettings.factor= 1;
                * @memberof ejMap
                * @instance
                */
                factor: 1,

                /**		
                * Specifies the zoom level value for which map to be zoomed
                * @default 1
                * @type {number}
                * @example 
                * //To set zoomLevel API value during initialization 
                *   $("#container").ejMap({zoomSettings:{level:1}});
                * @example 
                * //Get or set the zoomLevel API, after initialization:<br>
                *	//Gets the zoomLevel value 
                * 	var property = $("#container").data("ejMap").model.zoomSettings.level;<br>
                *	//Sets the zoomLevel value 
                *	$("#container").data("ejMap").model.zoomSettings.level= 1;
                * @memberof ejMap
                * @instance
                */
                level: 1,

                /**		
            * Enables or Disables the zoom on selecting the map shape
            * @default false
            * @type {boolean}
            * @example 
            * //To set enableZoomOnSelection API value during initialization 
            * 	$("#container").ejMap({zoomSettings:{enableZoomOnSelection:true}});	
            * @example 
            * //Get or set the enableZoomOnSelection API, after initialization:<br>
            *	//Gets the enableZoomOnSelection value 
            *	var property = $("#container").data("ejMap").model.zoomSettings.enableZoomOnSelection;<br>		
            *	//Sets the enableZoomOnSelection value 
            *	$("#container").data("ejMap").model.zoomSettings.enableZoomOnSelection=true }); 
             * @memberof ejMap
            * @instance
            */
             enableZoomOnSelection: false,

                /**		
            * Enables or Disables the zooming of map
            * @default true
            * @type {boolean}
            * @example 
            * //To set enableZoom API value during initialization 
            * 	$("#container").ejMap({enableZoom:true});	
            * @example 
            * //Get or set the enableZoom API, after initialization:<br>
            *	//Gets the enableZoom value 
            *	var property = $("#container").data("ejMap").model.zoomSettings.enableZoom;<br>		
            *	//Sets the enableZoom value 
            *	$("#container").data("ejMap").model.zoomSettings.enableZoom=true }); 
            * @memberof ejMap
            * @instance
            */
                enableZoom: true,

            },

            /**		
            * Specify the center position where map should be displayed
            * @default [0,0]
            * @type {object} 
            * @example  
            * // Set the centerPosition during initialization. 			
            * 	 $("#container").ejMap({centerPosition: [38.5000, -98]});
            * @example 
            * //Get or set the centerPosition after initialization:<br>
            *   //Gets the centerPosition from map.
            *   var property =$("#container").data("ejMap").model.centerPosition;
            *   //Sets the centerPosition to map.
            *   $("#container").data("ejMap").model.centerPosition = [38.5000, -98];
            * @memberof ejMap
            * @instance
            */
            centerPosition: [0, 0],

            /**		
			 * Determines whether map need to resize when container is resized 
			 * @default true
			 * @type {boolean}
			 * @example 
			 * //To set enableResize API value during initialization 
			 * 	$("#container").ejMap({enableResize:true});	
			 * @example 
			 * //Get or set the enableResize API, after initialization:<br>
			 *	//Gets the enableResize value 
			 *	var property = $("#container").data("ejMap").model.enableResize;<br>		
			 *	//Sets the enableResize value 
			 *	$("#container").data("ejMap").model.enableResize= true }); 
			  * @memberof ejMap
			 * @instance
			 */
            enableResize: true,

            /**		
			 * Enables or Disables the map animation
			 * @default false
			 * @type {boolean}
			 * @example 
			 * //To set enableAnimation API value during initialization 
			 * 	$("#container").ejMap({enableAnimation:true});	
			 * @example 
			 * //Get or set the enableAnimation API, after initialization:<br>
			 *	//Gets the enableAnimation value 
			 *	var property = $("#container").data("ejMap").model.enableAnimation;<br>		
			 *	//Sets the enableAnimation value 
			 *	$("#container").data("ejMap").model.enableAnimation=true }); 
			  * @memberof ejMap
			 * @instance
			 */
            enableAnimation: false,           

            /**		
			* Enables or Disables the navigation control for map to perform zooming and panning on map shapes.
            * @default null 
			* @type {object}
			* @example 
            * //To set navigationControl during initialization 
            *   $("#container").ejMap(navigationControl:{enableNavigation:true,orientation:'vertical',absolutePosition:{x:5,y:20}});
            * @example 
			* //Get or set the navigationControl, after initialization:<br>
			*	//Gets the navigationControl  
            * 	var property = $("#container").data("ejMap").model.navigationControl;<br>
            *	//Sets the navigationControl  
			*	$("#container").data("ejMap").model.navigationControl={enableNavigation:true,orientation:'vertical',absolutePosition:{x:5,y:20}}); 
			* @memberof ejMap
			* @instance
            */
            navigationControl: null,

            /**		
			 * Enables or Disables the animation for layer change in map
			 * @default false
			 * @type {boolean}
			 * @example 
			 * //To set enableLayerChangeAnimation API value during initialization 
			 * 	$("#container").ejMap({enableLayerChangeAnimation:true});	
			 * @example 
			 * //Get or set the enableLayerChangeAnimation API, after initialization:<br>
			 *	//Gets the enableLayerChangeAnimation value 
			 *	var property = $("#container").data("ejMap").model.enableLayerChangeAnimation;<br>		
			 *	//Sets the enableLayerChangeAnimation value 
			 *	$("#container").data("ejMap").model.enableLayerChangeAnimation=false }); 
			 * @memberof ejMap
			 * @instance
			 */
            enableLayerChangeAnimation: false,

             /**		
			 * Enables or Disables the map panning
			 * @default true
			 * @type {boolean}
			 * @example 
			 * //To set enablePan API value during initialization 
			 * 	$("#container").ejMap({enablePan:true});	
			 * @example 
			 * //Get or set the enablePan API, after initialization:<br>
			 *	//Gets the enablePan value 
			 *	var property = $("#container").data("ejMap").model.enablePan;<br>		
			 *	//Sets the enablePan value 
			 *	$("#container").data("ejMap").model.enablePan=true }); 
			 * @memberof ejMap
			 * @instance
			 */
            enablePan: true,

           
            /**		
			* Specifies the base map-index of the map to determine the shapelayer to be displayed
            * @default 0 
			* @type {number}
			* @example 
            * //To set baseMapIndex API value during initialization 
            *   $("#container").ejMap({baseMapIndex:0});
            * @example 
			* //Get or set the baseMapIndex API, after initialization:<br>
			*	//Gets the baseMapIndex value 
            * 	var property = $("#container").data("ejMap").model.baseMapIndex;<br>
            *	//Sets the baseMapIndex value 
			*	$("#container").data("ejMap").model.baseMapIndex= 0;
			* @memberof ejMap
			* @instance
            */
            baseMapIndex: 0,

            /**     
            * Triggered on selecting the map shapes.
            * @event
            * @name ejMap#shapeSelected	            
            * @param {object} originalEvent.data Returns selected shape object.
            * @example 
            * //shapeSelected event for map
            * $("#container").ejMap({
            *    shapeSelected: function (event) {}
            * });
            * @memberof ejMap
            * @instance
            */
            shapeSelected: "",

            /**     
            * Triggered on selecting the map markers.
            * @event
            * @name ejMap#markerSelected              
            * @param {object} originalEvent.data Returns marker object.
            * @example 
            * //markerSelected event for map
            * $("#container").ejMap({
            *    markerSelected: function (event) {}
            * });
            * @memberof ejMap
            * @instance
            */
            markerSelected: "",

            /**     
            * Triggered when map is zoomed-in.
            * @event
            * @name ejMap#zoomedIn	
            * @param {Object} originalEvent Event parameters from map               
            * @param {object} zoomLevel Returns zoom level value for which the map is zoomed.
            * @example 
            * //zoomedIn event for map
            * $("#container").ejMap({
            *    zoomedIn: function (event) {}
            * });
            * @memberof ejMap
            * @instance
            */
            zoomedIn: "",

            /**     
            * Triggers once map render completed.
            * @event
            * @name ejMap#onRenderComplete	
            * @param {Object} originalEvent Event parameters from map  
            * @example 
            * //onRenderComplete event for map
            * $("#container").ejMap({
            *    onRenderComplete: function () {}
            * });
            * @memberof ejMap
            * @instance
            */
            onRenderComplete: "",

            /**     
            * Triggers when map panning ends.
            * @event
            * @name ejMap#panned		
            * @param {Object} originalEvent Event parameters from map  
            * @example 
            * //panned event for map
            * $("#container").ejMap({
            *    panned: function (event) {}
            * });
            * @memberof ejMap
            * @instance
            */

            panned: "",
            /**     
            * Triggers when map is zoomed out.
            * @event
            * @name ejMap#zoomedOut	
            * @param {Object} originalEvent Event parameters from map               
            * @param {object} zoomLevel Returns zoom level value for which the map is zoomed.		
            * @example 
            * //zoomedOut event for map
            * $("#container").ejMap({
            *    zoomedOut: function () {}
            * });
            * @memberof ejMap
            * @instance
            */
            zoomedOut: "",

            /**     
            * Triggers while hovering the map shape.
            * @event
            * @name ejMap#mouseover 	              
            * @param {object} originalEvent.data Returns hovered map shape object.            
            * @example 
            * //mouseover  event for map
            * $("#container").ejMap({
            *    mouseover : function (event) {}
            * });
            * @memberof ejMap
            * @instance
            */
            mouseover : "",

            /**     
            * Triggers while leaving the hovered map shape
            * @event
            * @name ejMap#mouseleave 		            
            * @param {object} originalEvent.data Returns hovered map shape object. 
            * @example 
            * //mouseleave  event for map
            * $("#container").ejMap({
            *    mouseleave : function (event) {}
            * });
            * @memberof ejMap
            * @instance
            */
            mouseleave : "",

           
            // Layers Collection
            /**		
            * Hold the shapelayers to be displayed in map
            * @default []
            * @type {shapeLayer}              
            * @example  
            * // Set the layers during initialization. 			
            * 	$("#container").ejMap({layers: [{ layerType: "geometry", enableMouseHover: true, shapeSettings: { stroke: "black", fill: "#C3E6ED", highlightColor: "#63B7B7", selectionColor: "#207BB2", strokeThickness: "0.5" }, shapeData: Africa }]})
            * @example 
            * //Get or set the layer after initialization:<br>
            *   //Gets the layer from map.
            *   var layer =$("#container").data("ejMap").model.layers[layerIndex];
            *   //Sets the layer to map.
            *   $("#container").data("ejMap").model.layers[layerIndex]  = { layerType: "geometry", enableMouseHover: true, shapeSettings: { stroke: "black", fill: "#C3E6ED", highlightColor: "#63B7B7", selectionColor: "#207BB2", strokeThickness: "0.5" }, shapeData: Africa };
            * @memberof ejMap
            * @instance
            */
            layers: []
        },

        observables: ["dataSource", "baseMapIndex"],
        enableZoom: ej.util.valueFunction("zoomSettings.enableZoom"),
        enableZoomOnSelection: ej.util.valueFunction("zoomSettings.enableZoomOnSelection"),
        zoomLevel: ej.util.valueFunction("zoomSettings.level"),
        minZoom: ej.util.valueFunction("zoomSettings.minValue"),
        zoomFactor: ej.util.valueFunction("zoomSettings.factor"),
        maxZoom: ej.util.valueFunction("zoomSettings.maxValue"),
        baseMapIndex: ej.util.valueFunction("baseMapIndex"),

        /**
         * Specify the data types for default properties 
		 * @private
         */
        dataTypes: {
            layers: "array"
        },

        /**
        * Destroy the map widget
        * @private
        */

        _destroy: function () {
            /// <summary>This function is  used to destroy the Map Object</summary>
            this._unWireEvents();
            if (_isSVG) {
                $(this.element).removeClass("e-map e-js").find("#svgDocument").remove();
            }
            else {
                $(this.element).removeClass("e-map e-js").find("#rootGroup").remove();
            }
        },

        /**
          * To configure the properties at runtime using SetModel		
          * @private
          */
        _setModel: function (options) {
            for (var prop in options) {
                switch (prop) {
                    case "zoomSettings.level":
                        if (this.enableZoom()) { 
                            
                            this.zoom(options[prop]);

                        }
                        break;
                    case "baseMapIndex":
                        this.baseMapIndex(options[prop]);
                        var map = this;
                        if (this.model.enableAnimation) {
                            this.model.enableLayerChangeAnimation = true;
                            $(this._mapContainer).animate({ opacity: 0 }, 500, function () {
                                map.refresh();
                            });

                        } else {
                            this.refresh();
                        }

                        break;
                    case "minZoom":
                        this.minZoom(options[prop]);
                        if (this.zoomLevel() < this.minZoom()) {                           
                            this.zoom(this.minZoom());
                        }
                        break;
                    case "maxZoom":
                        this.maxZoom(options[prop]);
                        if (this.zoomLevel() > this.maxZoom()) {                            
                            this.zoom(this.maxZoom());
                        }
                        break;
                    case "background":
                        this.model.background = options[prop];
                        $(this._mapContainer).css('background-image', this.model.background);
                }
            }
        },


        /**
        * Create the map widget
        * @private
        */
        _init: function () {
            var proxy = this;
			this._navigationStyle = null;
            this._initPrivateProperties();
            $.each(this.model.layers, function (index) {
                proxy._layerInitialize(index);
            });
            this._mapContainer = this.element;
            $(this._mapContainer).css({ "overflow": "hidden" });
            var style = $('<style>.labelLegend {padding:3px;font-weight: normal;color:black;width:auto;height:20px;text-align:center;font-size: 14px;font-family:Segoe UI;}.labelLegend:hover{cursor:pointer;}</style>');
            $('html > head').append(style);
            this.refresh();
            this._trigger("onRenderComplete");
            this._isRendered = true;
            if (_isSVG) {
                $(this._svgDocument).pinchZoom(this._rootgroup, this);
            }
        },

        /**
      * Create layers for the map widget
      * @private
      */
        _layerInitialize: function (layerindex) {
            var proxy = this;
            if (this.model.layers[layerindex] != null) {
                $.each(this.model.layers, function (index, element) {
                    element = proxy._checkArrayObject(element, index);
                    var obj = new shapeLayer();
                    $.extend(obj, element);
                    $.extend(element, obj);
                    $.each(element.subLayers, function (subindex, subelement) {
                        subelement = proxy._checkArrayObject(subelement, subindex);
                        var subobj = new shapeLayer();
                        $.extend(subobj, subelement);
                        $.extend(subelement, subobj);
                    });
                });
            }
            else {

                this.layers[0] = new shapeLayer();

            }
        },

        /**
      * Merges the model values with default value
      * @private
      */
        _checkArrayObject: function (element, initialName) {
            var proxy = this;
            $.each(element, function (name, innerElement) {
                if (innerElement instanceof Array) {
                    proxy._checkArrayObject(innerElement, name);
                }
                else if (innerElement != null && typeof innerElement == "object") {
                    var allObjects = new shapeLayer();
                    proxy._loadIndividualDefaultValues(innerElement, allObjects, (typeof name === "number") ? initialName : name);
                }
            });
            return element;
        },

         /**
         * Loads individual default values
         * @private
         */
        _loadIndividualDefaultValues: function (obj, allObjects, name) {
            var defaultObj = null;
            var proxy = this;
            $.each(allObjects, function (n, element) {
                if (name == n) {
                    defaultObj = element;
                    return;
                }
            });
            if (defaultObj instanceof Array) defaultObj = defaultObj[0];

            $.each(obj, function (objName, ele) {
                if (ele instanceof Array) {
                    proxy._checkArrayObject(ele, name);
                }
                else if (ele != null && typeof ele == "object") {
                    proxy._loadIndividualDefaultValues(ele, ele, (typeof objName === "number") ? name : objName);
                }
            });

            $.extend(defaultObj, obj);
            $.extend(obj, defaultObj);
            return obj;
        },

        /**
        * Refresh the map with animation
        * @private
        */
        _refreshWithAnimation: function () {
            this.model.layers[this.baseMapIndex()]._setMapItemsPositionWithAnimation(this);
            for (var key = 0; key < this.model.layers[this.baseMapIndex()].subLayers.length; key++) {
                this.model.layers[this.baseMapIndex()].subLayers[key]._setMapItemsPositionWithAnimation(this);
            }
        },

        /**
        * Reduces the shape stoke while zooming
        * @private
        */
        _resizeShape: function () {
            this.model.layers[this.baseMapIndex()]._resizeShapes(this);
            for (var key = 0; key < this.model.layers[this.baseMapIndex()].subLayers.length; key++) {
                this.model.layers[this.baseMapIndex()].subLayers[key]._resizeShapes(this);
            }

        },

        /**
       * Refresh the shape   layers
       * @private
       */
        _refrshLayers: function () {
            this.model.layers[this.baseMapIndex()]._setMapItemsPosition(this);
            for (var key = 0; key < this.model.layers[this.baseMapIndex()].subLayers.length; key++) {
                this.model.layers[this.baseMapIndex()].subLayers[key]._setMapItemsPosition(this);
            }

        },

        /**
      * Pans the tile map
      * @private
      */
        _panTileMap: function (factorX, factorY, centerPosition) {
            
            var totalSize = Math.pow(2, this.zoomLevel()) * 256;
            this._tileTranslatePoint.x = (factorX / 2) - (totalSize / 2);
            this._tileTranslatePoint.y = (factorY / 2) - (totalSize / 2);
            var position = this._convertTileLatLongtoPoint(centerPosition[0],centerPosition[1]);
            this._tileTranslatePoint.x -= position.x - (factorX / 2);
            this._tileTranslatePoint.y -= position.y - (factorY / 2);

        },

        /**
     * Generates tile view map
     * @private
     */
        _generateTiles: function (zoomLevel) {
            var userLang = navigator.language || navigator.userLanguage;
            this.Tiles = [];
            var xcount, ycount;
            xcount = ycount = Math.pow(2, zoomLevel);
            var baseLayer = this.model.layers[this.baseMapIndex()];
            var endY = Math.min(ycount, ((-this._tileTranslatePoint.y + this._height) / 256) + 1);
            var endX = Math.min(xcount, ((-this._tileTranslatePoint.x + this._width) / 256) + 1);
            var startX = (-(this._tileTranslatePoint.x + 256) / 256);
            var startY = (-(this._tileTranslatePoint.y + 256) / 256);
            for (var i = Math.round(startX) ; i < Math.round(endX) ; i++) {
                for (var j = Math.round(startY) ; j < Math.round(endY) ; j++) {
                    var x = 256 * i + this._tileTranslatePoint.x;
                    var y = 256 * j + this._tileTranslatePoint.y;
                    if (x > -256 && x <= this._width && y > -256 && y < this._height) {
                        if (i >= 0 && j >= 0) {
                            var tile = new Tile(i, j);
                            tile.left = x;
                            tile.top = y;
                            if (baseLayer.layerType == ej.datavisualization.Map.LayerType.Bing) {
                                tile.src = this._getBingMap(tile, baseLayer.key, baseLayer.bingMapType, userLang);
                            } else {
                                tile.src = this._urlTemplate.replace("level", this.zoomLevel()).replace('tileX', tile.X).replace('tileY', tile.Y);
                            }
                            this.Tiles.push(tile);
                        }
                    }
                }
            }
            var proxTiles = $.extend(true, [], this.Tiles);
            for (var layerIndex = 0; layerIndex < this.model.layers[this.baseMapIndex()].subLayers.length;layerIndex++) {
                var layer = this.model.layers[this.baseMapIndex()].subLayers[layerIndex];
                if (layer.layerType == ej.datavisualization.Map.LayerType.OSM || layer.layerType == ej.datavisualization.Map.LayerType.Bing) {
                    for (var tileIndex = 0; tileIndex < proxTiles.length; tileIndex++) {
                        var baseTile = proxTiles[tileIndex];
                        var subtile = $.extend(true, {}, baseTile);
                        if (layer.layerType == ej.datavisualization.Map.LayerType.Bing) {
                            subtile.src = this._getBingMap(subtile, layer.key, layer.bingMapType, userLang);
                        } else {
                            subtile.src = layer.urlTemplate.replace("level", this.zoomLevel()).replace('tileX', baseTile.X).replace('tileY', baseTile.Y);
                        }
                        this.Tiles.push(subtile);
                    }

                }
            }
            this._arrangeTiles();
        },

        /**
        * Generates bing map
        * @private
        */
        _getBingMap:function(tile,key,type,lang) {
            var quadKey = "";
            for (var i = this.zoomLevel(); i > 0; i--)
            {
                var digit = 0;
                var mask = 1 << (i - 1);
                if ((tile.X & mask) != 0)
                {
                    digit++;
                }
                if ((tile.Y & mask) != 0)
                {
                    digit+=2;
                }
                quadKey = quadKey+""+digit;
            }
            var layerType;
            if (type == ej.datavisualization.Map.BingMapType.Aerial) {
                layerType = "A,G";
            }
            else if (type == ej.datavisualization.Map.BingMapType.AerialWithLabel) {
                layerType = "A,G,L";
            } else {
                layerType = "G,VE,BX,L,LA";
            }
            var imageUrl = "http://ak.dynamic.t2.tiles.virtualearth.net/comp/ch/" + quadKey + "?mkt=" + lang + "&ur=IN&it=" + layerType + "&shading=hill&og=45&n=z&Key=" + key;
            return imageUrl;
        },
        
       

        /**
        * Arrange map tiles
        * @private
        */
        _arrangeTiles: function (){        
            var pathTemplate = "<div><div style='position:absolute;left:{{:left}}px;top:{{:top}}px;height:{{:height}}px;width:{{:width}}px;'><img src={{:src}}></img></div></div>";
            var tmpl2 = $.templates(pathTemplate);
            var htmlString = tmpl2.render(this.Tiles);
            var first = this._mapContainer[0].children[0];            
            this._tileDiv.html(htmlString);            
        },

        /**
        * Generate shape path from data provided
        * @private
        */
        _generatePath: function () {
           
            var baseLayer = this.model.layers[this.baseMapIndex()];
            var pathCollection = '';
            this._polylineCount = 0;
            this._pointscount = 0;
            _isSVG = (window.SVGSVGElement) ? true : false;
            if (baseLayer.layerType == ej.datavisualization.Map.LayerType.Geometry) {
                if (baseLayer != undefined && baseLayer.shapeData != null) {
                    baseLayer._isBaseLayer = true;
                    this._isTileMap = false;
					this._scale = this.zoomLevel();
                    pathCollection = this._readShapeData(baseLayer);
                }
            }
            else {                
                this._isTileMap = true;
                this._scale =  Math.pow(2, this.zoomLevel() - 1);
            }
            for (var key = 0; key < baseLayer.subLayers.length; key++) {
                if (baseLayer.subLayers[key].layerType == ej.datavisualization.Map.LayerType.Geometry && baseLayer.subLayers[key].shapeData != null) {
                    baseLayer.subLayers[key]._isBaseLayer = false;
                    pathCollection += this._readShapeData(baseLayer.subLayers[key]);
                }
            }
            if (_isSVG) {
                var htmlstring = '<div style="position:relative;" id="tileDiv"/><svg xmlns="http://www.w3.org/2000/svg" style= "overflow:hidden;z-index:0;float:left;left:0,top:0"' +
                             'id="svgDocument"> <g id="rootGroup">' + pathCollection + '</g></svg>';
                this._mapContainer.html(htmlstring);               
                this._svgDocument = this.element.find("#svgDocument")[0];
                this._tileDiv = this.element.find("#tileDiv");
                $(this._svgDocument).css({ height: this._height, width: this._width });
            }
            else {
                document.createStyleSheet().addRule(".vml", "behavior:url(#default#VML);display:inline-block");
                document.namespaces.add('v', 'urn:schemas-microsoft-com:vml', "#default#VML");
                var vmlGroup = '<div id="tileDiv"/><v:group   id = "rootGroup" style="position:absolute; width:' + this._width + 'px;height:' + this._height + 'px;" coordorigin = "0 0" coordsize="' + this._width + ' ' + this._height + '">' + pathCollection + '</v:group>';
                this._mapContainer.html(vmlGroup);                
                this._tileDiv = this.element.find("#tileDiv");
                this._bubblegroup = this._createGroup(false,"bubbleGroup");               
                this._bubblegroup.style.position = "absolute";
                this.element.append(this._bubblegroup);
            }
            if (baseLayer.layerType == ej.datavisualization.Map.LayerType.OSM || baseLayer.layerType == ej.datavisualization.Map.LayerType.Bing) {
                this._urlTemplate = baseLayer.urlTemplate;
                this._panTileMap(this._width, this._height, this.model.centerPosition);
                this._generateTiles(this.zoomLevel());
				if(_isSVG)
				{
				  $(this._svgDocument).css("position","relative");
				}
            }
            if(_isSVG)
			{			
                $(this._mapContainer).css("position", "relative");
			}			
            this._rootgroup = this.element.find("#rootGroup")[0];
        },

        /**
       * Calculate the BBox from JSON input
       * @private
       */
        _calculateBBox: function (polygonDatas, polylineDatas,pointData)
        {
            var minLatitude = maxLatitude = minLongitude = maxLongitude = 0;
            var isEntered = false;
            if (polygonDatas != null && polygonDatas.length > 0) {
                for (var i = 0; i < polygonDatas.length; i++) {
                    var currentData = polygonDatas[i];
                    if (currentData.geometry != null) {
                        coords = currentData.geometry.coordinates;
                    } else {
                        coords = currentData.coordinates;
                    }
                    var currentDataLength = coords.length;
                    for (var j = 0; j < currentDataLength; j++) {
                        var subData = '';
                        if (currentDataLength > 1) {
                            subData = coords[j][0];
                            if (subData.length <= 2) {
                                subData = coords[j];
                            }
                        } else {
                            subData = coords[j];
                        }
                        if (subData.length > 2) {
                            for (var k = 0; k < subData.length; k++) {
                                if (!isEntered) {
                                    minLatitude = maxLatitude = subData[k][1];
                                    minLongitude = maxLongitude = subData[k][0];
                                    isEntered = true;

                                } else {
                                    minLatitude = Math.min(minLatitude, subData[k][1]);
                                    maxLatitude = Math.max(maxLatitude, subData[k][1]);
                                    minLongitude = Math.min(minLongitude, subData[k][0]);
                                    maxLongitude = Math.max(maxLongitude, subData[k][0]);
                                }
                            }
                        } else {
                            if (!isEntered) {
                                minLatitude = maxLatitude = subData[1];
                                minLongitude = maxLongitude = subData[0];
                                isEntered = true;
                            } else {
                                minLatitude = Math.min(minLatitude, subData[1]);
                                maxLatitude = Math.max(maxLatitude, subData[1]);
                                minLongitude = Math.min(minLongitude, subData[0]);
                                maxLongitude = Math.max(maxLongitude, subData[0]);
                            }
                        }
                    }
                }
            }
            if (pointData!=null && pointData.length > 0) {
                for (var i = 0; i < pointData.length; i++) {
                    var currentData = pointData[i];
                    subData = currentData.geometry.coordinates;
                    if (!isEntered) {
                        minLatitude = maxLatitude = subData[1]; // (φ)
                        minLongitude = maxLongitude = subData[0];
                        isEntered = true;
                    } else {
                        minLatitude = Math.min(minLatitude, subData[1]);
                        maxLatitude = Math.max(maxLatitude, subData[1]);
                        minLongitude = Math.min(minLongitude, subData[0]);
                        maxLongitude = Math.max(maxLongitude, subData[0]);
                    }
                }
            }
            if (polylineDatas!=null && polylineDatas.length > 0) {
                for (var i = 0; i < polylineDatas.length; i++) {
                    var coordinates = polylineDatas[i].geometry.coordinates;
                    for (var k = 0; k < coordinates.length; k++) {
                        if (!isEntered) {
                            minLatitude = maxLatitude = coordinates[k][1]; // (φ)
                            minLongitude = maxLongitude = coordinates[k][0];
                            isEntered = true;
                        } else {
                            minLatitude = Math.min(minLatitude, coordinates[k][1]);
                            maxLatitude = Math.max(maxLatitude, coordinates[k][1]);
                            minLongitude = Math.min(minLongitude, coordinates[k][0]);
                            maxLongitude = Math.max(maxLongitude, coordinates[k][0]);
                        }
                    }
                }
            }
            return [[minLongitude, minLatitude], [maxLongitude, maxLatitude]];        
        },
        
        _readShapeData: function (layer) {
            var map = this;
            if (layer.shapeData != null) {
                if (typeof layer.shapeData == 'string') {
                    var dataManger =new ej.DataManager(layer.shapeData);
                    var query = ej.Query().from(layer);
                    dataManger.executeQuery(query).done(function (e) {
                        if (e.result != null) {
                            return map._getPathCollection(e.result, layer);
                        }
                    });

                } else {
                 return this._getPathCollection(layer.shapeData, layer);
                }
                
            }
        },

        /**
        * Generates path collection from JSON input
        * @private
        */
        _getPathCollection:function(data,layer)
        {
            var shapePath = "";
            var type = "";
            var dataManager="";
            if (data.features != null) {
                dataManager = new ej.DataManager(data.features);
                type = "geometry.type";
                if (layer._isBaseLayer) {
                    this._mapBounds = data.bbox;
                }
            }
            else if (data.geometries != null) {
                dataManager = new ej.DataManager(data.geometries);
                type = "type";
                if (layer._isBaseLayer) {
                    if (data.bbox != null)
                        this._mapBounds = [[data.bbox[0], data.bbox[1]], [data.bbox[2], data.bbox[3]]];
                    }                
            }
			if(data.geometries!=null || data.features != null){
            var queryPolyLine = ej.Query().from(layer).where(type, ej.FilterOperators.equal, 'LineString');
            var queryPolygon = ej.Query().from(layer).where(type, ej.FilterOperators.equal, 'Polygon');
            var queryMulitiPolygon = ej.Query().from(layer).where(type, ej.FilterOperators.equal, 'MultiPolygon');
            var polylineDatas = dataManager.executeLocal(queryPolyLine);           
            var polygonDatas = dataManager.executeLocal(queryPolygon);
            var queryPoint = ej.Query().from(layer).where(type, ej.FilterOperators.equal, 'Point');
            var pointData = dataManager.executeLocal(queryPoint);
            var multiPolygonDatas = dataManager.executeLocal(queryMulitiPolygon);
            this._polylineCount = this._polylineCount + polylineDatas.length;
            this._pointscount = this._pointscount + pointData.length;
            $.merge(polygonDatas, multiPolygonDatas);
            if (this._mapBounds == null && layer._isBaseLayer) {
                this._mapBounds = this._calculateBBox(polygonDatas, polylineDatas, pointData);
            }
			layer._newBounds = [];                 
            var pointHtml = "";
            if (pointData.length > 0) {
                for (var i = 0; i < pointData.length; i++) {
                    var currentData = pointData[i];                   
                    var check = '';
                    subData = currentData.geometry.coordinates;
                    var data = [];
                    var data_clean = [];                   
                    data.push({
                        lat: parseFloat(subData[1]),
                        lng: parseFloat(subData[0])                        
                    });                    
                    var l = 0;

                    var point = data[0];
                    latitude = point.lat; // (φ)
                    longitude = point.lng;   // (λ)                   
                    var p = this._convertTileLatLongtoPointForShapes(latitude, longitude, this._mapBounds);
                    if (_isSVG) {
                        data_clean.push({ x: p.x, y: p.y, lat: point.lat, lng: point.lng });
                    }
                    else {
                        data_clean.push({ x: p.x - ((10 / this._scale)/2), y: p.y-((10 / this._scale) / 2), lat: point.lat, lng: point.lng });
                    }

                    if (_isSVG) {
                        pointHtml += '<circle cx="' + data_clean[0].x + '" cy="' + data_clean[0].y + '" r="' + 5 / this._scale + '" stroke="' + layer.shapeSettings.stroke + '" stroke-width="' + layer.shapeSettings.strokeThickness + '0" fill="' + layer.shapeSettings.fill + '" />';
                    }
                    else {
                        var pointHtmlString = '<v:oval display="block" fill="' + layer.shapeSettings.fill + '" style="position:absolute;top: ' + data_clean[0].y + 'px;left:' + data_clean[0].x + 'px;width:' + 10 / this._scale + 'px;height:' + 10 / this._scale + 'px;stroke-width:' + layer.shapeSettings.strokeThickness + 'px;"></v:oval>';
                        pointHtml += pointHtmlString;
                    }
                }

            }            
            if (polygonDatas.length > 0)
            {
                var minX, maxX, minY, maxY,flag=true;
                for (var i = 0; i < polygonDatas.length; i++) {
                    var currentData = polygonDatas[i];
                    var midCoordinate = 0;
                    var midCoordinateLength;
                    var coords;
                    if (currentData.geometry != null) {
                        coords = currentData.geometry.coordinates;
                    }
                    else {
                        coords = currentData.coordinates;
                    }                    
                    midCoordinateLength = coords[0][0].length;
                    var currentDataLength = coords.length;
                    if (currentDataLength > 1) {
                        var lst = coords;
                        for (var index = 0; index < lst.length; index++) {
                            if (midCoordinateLength < lst[index][0].length) {
                                midCoordinateLength = lst[index][0].length;
                                midCoordinate = index;
                            }
                        }
                    }
                    var path_code = '';
                    for (var j = 0; j < currentDataLength; j++) {
                        var subData = '';                       
                        if (currentDataLength > 1) {
                            subData = coords[j][0];
                            if (subData.length <=2) {
                                subData = coords[j];
                            }
                        }
                        else {
                            subData = coords[j];
                        }
                        var data = [];
                        var data_clean = [];
                        if (subData.length > 2) {
                            for (var k = 0; k < subData.length; k++) {
                                data.push({
                                    lat: parseFloat(subData[k][1]),
                                    lng: parseFloat(subData[k][0])
                                });
                            }
                        }
                        else {
                            data.push({
                                lat: parseFloat(subData[1]),
                                lng: parseFloat(subData[0])
                            });
                        }                       
                        for (var l = 0; l < data.length - 1|| (data.length==1 && l==0); l++) {
                            var point = data[l];
                            latitude = point.lat; // (φ)
                            longitude = point.lng;   // (λ                           
                            var boundingBox = this._mapBounds;
                            if (this._isTileMap||(longitude >= boundingBox[0][0] && longitude <= boundingBox[1][0] && latitude >= boundingBox[0][1] && latitude <= boundingBox[1][1])) {
                               
                                var p = this._convertTileLatLongtoPointForShapes(latitude, longitude, boundingBox);
                                if (flag) {
                                    minX = maxX = p.x;
                                    minY = maxY = p.y;
                                    flag = false;
                                }
                                else {
                                    minX = Math.min(minX, p.x);
                                    maxX = Math.max(maxX, p.x);
                                    minY = Math.min(minY, p.y);
                                    maxY = Math.max(maxY, p.y);
                                }                                
                                if (_isSVG) {
                                    data_clean.push({ x: p.x, y: p.y, lat: point.lat, lng: point.lng });
                                }
                                else {
                                    data_clean.push({ x: p.x, y: p.y, lat: point.lat, lng: point.lng });
                                }
                            }
                            
                        }
                        if (j == midCoordinate) {
                            layer._newBounds.push(this._findMidPointofPoylgon(data_clean));
                        }
                        
                        if (data_clean.length > 0) {
                            if (_isSVG) {
                                path_code += "M" + (data_clean[0].x) + "," + (data_clean[0].y);
                                for (m = 1; m < data_clean.length; m++) {
                                    path_code += "," + (data_clean[m].x) + "," + (data_clean[m].y);
                                }
                                path_code += "Z";
                            }
                            else {
                                path_code += "m" + parseInt(data_clean[0].x) + "," + parseInt(data_clean[0].y);
                                path_code += " l" + parseInt(data_clean[0].x) + "," + parseInt(data_clean[0].y);
                                for (m = 1; m < data_clean.length; m++) {
                                    path_code += "," + parseInt(data_clean[m].x) + "," + parseInt(data_clean[m].y);
                                }
                                path_code += " e";
                            }
                        }

                    }
                    if (path_code != "") {
                        if (_isSVG) {
                            shapePath += '<path class="mapShape" stroke=' + layer.shapeSettings.stroke +
                                        ' stroke-width=' + (layer.shapeSettings.strokeThickness / this._scale) +
                                         ' fill= ' + layer.shapeSettings.fill + ' d="' + path_code + '"  stroke-linejoin= round stroke-linecap= square></path>';
                        }
                        else {
                            shapePath += '<v:shape  style="width:' + this._width + 'px;height:' + this._height + 'px;"  coordsize="'
                                + this._width + ' ' + this._height + '" coordorigin="0 0" strokecolor=' + layer.shapeSettings.stroke +
                                           ' stroke-width=' + (layer.shapeSettings.strokeThickness / this._scale) + 'px"' +
                                            ' fillcolor= ' + layer.shapeSettings.fill + ' path="' + path_code + '"></v:shape>';
                        }
                    }
                    else {
                        polygonDatas.splice(i, 1);
                        i--;
                    }
                    
                }
                
                this._groupSize = { minX: minX, maxX: maxX, minY: minY, maxY: maxY };
                
            }
            
            if (polylineDatas.length > 0) {

                for (var i = 0; i < polylineDatas.length; i++) {
                    var coordinates = polylineDatas[i].geometry.coordinates;
                    var linedata = '';
                    for (var k = 0; k < coordinates.length; k++) {
                        var points = this._convertTileLatLongtoPointForShapes(coordinates[k][1], coordinates[k][0], this._mapBounds);
                        linedata += points.x + "," + points.y;
                        if (k != coordinates.length - 1)
                            linedata += ",";
                    }

                    if (_isSVG) {
                        shapePath += '<polyline stroke=' + layer.shapeSettings.stroke +
                                    ' stroke-width=' + (layer.shapeSettings.strokeThickness / this._scale) + ' fill="transparent" stroke-width="1" points="' + linedata + '" ></polyline>';

                    }
                    else {
                        shapePath += '<v:polyline coordsize="'+ this._width + ',' + this._height + '" coordorigin="0 0" strokecolor=' + layer.shapeSettings.stroke +
                                                               ' strokeweight=' + (layer.shapeSettings.strokeThickness / this._scale) + 'px"' +
                                                                ' fillcolor=' + layer.shapeSettings.fill + '  points="' + linedata + '"/>';


                    }
                }

            }           
            var lines = [].concat(polygonDatas, polylineDatas);
            layer._polygonData = $.merge(lines, pointData);
            return shapePath + pointHtml;
			}
			else{
			  layer._polygonData=[];
			    return null;
		   }
        },
        

        /**
         * Creates VML group
         * @private
         */
        _createGroup: function (isRoot,name) {
            
            var vmlGroup;
            vmlGroup = document.createElement('<v:group class='+name+'>');
            vmlGroup.style.width = this._width + 'px';
            vmlGroup.style.height = this._height + 'px';            
            vmlGroup.coordorigin = "0 0";
            vmlGroup.coordsize = this._width + ' ' + this._height;
            if (isRoot) {
                this._rootgroup = vmlGroup;
                vmlGroup.style.left = '20px';
                vmlGroup.style.top = '20px';
            }
            return vmlGroup;
        },

        /**
        * Generates palette colors for shape
        * @private
        */
        _generatePaletteColorsForShape: function (shape, shapelayer, palette, palettesettings) {

            if (palettesettings != null) {
                shapelayer.shapeSettings.highlightColor = palettesettings.highlightColor;
                shapelayer.shapeSettings.highlightStroke = palettesettings.highlightStroke;
                shapelayer.shapeSettings.highlightBorderWidth = palettesettings.highlightBorderWidth;
                shapelayer.shapeSettings.selectionColor = palettesettings.SelectionColor;
                shapelayer.shapeSettings.selectionStroke = palettesettings.SelectionStroke;
                shapelayer.shapeSettings.selectionStrokeWidth = palettesettings.SelectionStrokeWidth;
            }

            if (_isSVG) {
                shape.setAttribute("fill", palette[shapelayer._prevPaletteIndex]);
            }
            else {
                shape.fillcolor = palette[shapelayer._prevPaletteIndex];
            }
            shapelayer._prevPaletteIndex = shapelayer._prevPaletteIndex + 1;
            if (shapelayer._prevPaletteIndex > palette.length - 1)
                shapelayer._prevPaletteIndex = 0;
        },

        /**
        * Renders map layers
        * @private
        */
        _renderLayers: function (layer, pathCount) {
            layer._initializeLocalValues();          
            var proxyrootGroup = this._rootgroup;
            var proxySVG = this._svgDocument;
            var proxyControl = this;
            var shapelayer = layer;
            if (shapelayer.legendSettings != null && shapelayer.shapeSettings.colorMappings != null && (shapelayer.legendSettings.type == undefined || shapelayer.legendSettings.type == ej.datavisualization.Map.LegendType.Layers))
                 shapelayer._generateLegends(this);
            if (shapelayer.legendSettings != null && shapelayer.bubbleSettings.colorMappings != null && (shapelayer.legendSettings.type == ej.datavisualization.Map.LegendType.Bubbles))
				 shapelayer._generateBubbleLegends(this);
            if (shapelayer.shapeSettings.colorMappings != null && shapelayer.shapeSettings.colorMappings.rangeColorMapping != null) {

                shapelayer.shapeSettings.colorMappings.rangeColorMapping.sort(this._orderByNameAscending);
            }
            if (shapelayer.bubbleSettings.colorMappings != null && shapelayer.bubbleSettings.colorMappings.rangeColorMapping != null) {
                shapelayer.bubbleSettings.colorMappings.rangeColorMapping.sort(this._orderByNameAscending);
            }
            var minValue = 0;
            var maxValue = 0;
            var sum = 0;
            if (shapelayer.dataSource != null && shapelayer.bubbleSettings != null) {
                if (shapelayer.bubbleSettings.colorValuePath  == null) {
                    shapelayer.bubbleSettings.colorValuePath  = shapelayer.bubbleSettings.valuePath;
                }
                for (var i = 0; i < shapelayer.dataSource.length; i++) {
                    var bubbledata = parseFloat(this._reflection(shapelayer.dataSource[i], shapelayer.bubbleSettings.valuePath));
                    if (i != 0) {
                        if (bubbledata > maxValue) {
                            maxValue = bubbledata;
                        } else if (bubbledata < minValue) {
                            minValue = bubbledata;
                        }
                    } else {
                        minValue = maxValue = bubbledata;
                    }
                }
            }

            this._generateTooltipForLayers(shapelayer);
            if (shapelayer.shapeData != null) {
                var shapesource = [];
                for (var key in shapelayer.dataSource) {
                    var shapeitem = shapelayer.dataSource[key];
                    var shapeItemValue = proxyControl._reflection(shapeitem, shapelayer.shapeDataPath);
                    shapesource.push(shapeItemValue);
                }
                for (var dataIndex = 0; dataIndex < shapelayer._polygonData.length; dataIndex++) {
                    var ValueObject = shapelayer._polygonData[dataIndex].properties;
                    var DBFValue = proxyControl._reflection(ValueObject, shapelayer.shapePropertyPath);
                    var shape = this._rootgroup.childNodes[dataIndex + pathCount];
                    var obj = {};
                    var bounds = shapelayer._newBounds[dataIndex];

                    if (shapelayer.shapeSettings.autoFill) {
                        switch (shapelayer.shapeSettings.colorPalette) {
                            case ej.datavisualization.Map.ColorPalette.Palette1:
                                this._generatePaletteColorsForShape(shape, shapelayer, shapelayer.colorPalettes.Palette1, shapelayer._colorPaletteSettings.Palette1);
                                break;
                            case ej.datavisualization.Map.ColorPalette.Palette2:
                                this._generatePaletteColorsForShape(shape, shapelayer, shapelayer.colorPalettes.Palette2, shapelayer._colorPaletteSettings.Palette2);
                                break;
                            case ej.datavisualization.Map.ColorPalette.Palette3:
                                this._generatePaletteColorsForShape(shape, shapelayer, shapelayer.colorPalettes.Palette3, shapelayer._colorPaletteSettings.Palette3);
                                break;
                            case ej.datavisualization.Map.ColorPalette.CustomPalette:
                                this._generatePaletteColorsForShape(shape, shapelayer, shapelayer.shapeSettings.customPalette, null);
                                break;
                        }
                    }

                    if (shapesource.indexOf(DBFValue) != -1) {
                        var item = shapelayer.dataSource[shapesource.indexOf(DBFValue)];
                        var ItemValue = DBFValue;
                        if (item != null) {
                            if (shape != null) {                              
                               
                                var id = ItemValue;
                            }
                                layer._bounds.push(bounds);
                                if (shapelayer.showMapItems) {
                                    var itemTemplateDiv = $("<div class='mapItems' style='display:block;position:absolute;pointer-events: none;'></div>");
                                    itemTemplateDiv[0].className = ItemValue;
                                    proxyControl._templateDiv.append(itemTemplateDiv);
                                    $(itemTemplateDiv).css({ "left": bounds.x, "top": bounds.y });
                                    var htmlString;
                                    if (shapelayer.mapItemsTemplate == null) {
                                        htmlString= $(' <div><label>{{:' + shapelayer.shapeSettings.valuePath + '}}</label></div>').render(item);;
                                    }
                                    else {
                                        htmlString = $("#" + shapelayer.mapItemsTemplate).render(item);
                                    }
                                    $(itemTemplateDiv).html(htmlString);
                                    layer._mapItems.push(itemTemplateDiv);
                                }                                   
                                                             
                                if (shapelayer.shapeSettings.colorMappings != null) {
                                    if (shapelayer.shapeSettings.colorValuePath  == null) {
                                        shapelayer.shapeSettings.colorValuePath  = shapelayer.shapeSettings.valuePath;
                                    }
                                    var shapeValue = proxyControl._reflection(item, shapelayer.shapeSettings.colorValuePath );
                                    if (shapeValue != null && !shapelayer.shapeSettings.autoFill) {
                                        if (((shapelayer.legendSettings != null && shapelayer.legendSettings.mode == ej.datavisualization.Map.LegendMode.Interactive) || shapelayer.shapeSettings.enableGradient) && shapelayer.shapeSettings.colorMappings.rangeColorMapping != null) {
                                            obj["legendrect"] = shapelayer._updateLegendRange(shapeValue, shapelayer, shape);
                                            var shapeColor = shapelayer.shapeSettings.fill;
                                            if (obj["legendrect"] != undefined) {
                                                shapeColor = obj["legendrect"].color;
                                            }
                                            if (_isSVG) {
                                                shape.setAttribute("fill", shapeColor);
                                            }
                                            else {
                                                shape.fillcolor = shapeColor;
                                            }
                                        } else {
                                            shapelayer._fillColors(shapeValue, shapelayer.shapeSettings.colorMappings, shape);
                                        }
                                    }
                                }
                                else if (!shapelayer.shapeSettings.autoFill) {
                                    if (_isSVG) {
                                        shape.setAttribute("fill", shapelayer.shapeSettings.fill);
                                    }
                                    else {
                                        shape.fillcolor = shapelayer.shapeSettings.fill;
                                        shape.strokecolor = shapelayer.shapeSettings.stroke;
                                    }
                                }
                            
                                if (shapelayer.bubbleSettings != null && shapelayer.bubbleSettings.valuePath != null) {
                                    if (_isSVG) {
                                        var circle = document.createElementNS(proxyControl._svgns, "circle");
                                    }
                                    else {
                                        var bubbleID = "bubble_" + id;
                                        var bubbleHtmlString = '<v:oval class="mapBubble" id=' + bubbleID + ' display="block" style="position:absolute;top: ' + 0 + 'px;left:' + 0 + 'px;width:100px;height:100px;stroke-width:' + 0 + 'px;"><v:fill opacity="0.9"/></v:oval>';
                                        this._bubblegroup.innerHTML = this._bubblegroup.innerHTML + bubbleHtmlString;
                                        var circle =document.getElementById("bubble_"+id);
                                    }

                                    var bubbleValue = proxyControl._reflection(item, shapelayer.bubbleSettings.valuePath);
                                    var bubblecolorValue = proxyControl._reflection(item, shapelayer.bubbleSettings.colorValuePath );
                                    if (shapelayer.bubbleSettings.colorMappings != null) {
                                        shapelayer._fillColors(bubblecolorValue, shapelayer.bubbleSettings.colorMappings, circle);
                                    }
                                    else {
                                        if (_isSVG) {
                                            circle.setAttribute("fill", shapelayer.bubbleSettings.color);
                                        }
                                        else {
                                            circle.strokecolor = shapelayer.bubbleSettings.color;
                                            circle.fillcolor = shapelayer.bubbleSettings.color;
                                        }
                                    }
                                    var radius = proxyControl._getRatioOfBubble(shapelayer.bubbleSettings.minValue, shapelayer.bubbleSettings.maxValue, bubbleValue, minValue, maxValue);
                                    if (_isSVG) {
                                        $(circle).attr({
                                            "cx": bounds.x, "cy": bounds.y,
                                            "fill-opacity": "0.9", "r": radius,"class":"mapBubble",
                                        });
                                        if (proxyControl.model.enableAnimation) {
                                            $(circle).css("display", "none");
                                        }
                                    }
                                    else {
                                        $(circle).css({
                                            "height":+2 * radius+"px",
                                            "width": +2 * radius + "px"
                                        });
                                    }
                                    if (shapelayer.bubbleSettings.showTooltip == true) {
                                        var tooltipTemplateDiv = $("<div class='bubbleToolTip'  style='position:absolute;display:none;z-index:9999'/>");
                                        var template_ID = null;
                                        if (shapelayer.bubbleSettings.tooltipTemplate != null) {
                                            template_ID = shapelayer.bubbleSettings.tooltipTemplate;
                                        } else {
                                            $(tooltipTemplateDiv).append('<div class="bubbleToolTip"  style="margin-left:10px;margin-top:-25px;"><div class="defaultToolTip"><p style="margin-top:-4px"><label  style="color:rgb(27, 20, 20);font-size:14px;font-weight:normal;font-family:Segoe UI">' + bubbleValue + '</label></p></div></div>');
                                        }
                                        $(circle).mouseenter({ htmlobj: tooltipTemplateDiv, templateID: template_ID, itemsrc: item }, proxyControl._bubbleEnterFunction);
                                        $(circle).mousemove({ htmlobj: tooltipTemplateDiv }, proxyControl._bubbleOverFunction);
                                        $(circle).mouseleave({ htmlobj: tooltipTemplateDiv }, proxyControl._bubbleleaveFunction);
                                    } else {
                                        $(circle).css("pointer-events", "none");
                                    }
                                    if (_isSVG) {
                                        proxySVG.appendChild(circle);
                                        layer._bubbles.push(circle);
                                    }
                                    else {
                                        layer._bubbles.push(circle);
                                    }
                                }
                                obj["data"] = item;
                        }
                    }
                    if (layer.labelSettings != null && layer.labelSettings.showLabels && layer.labelSettings.labelPath != null) {
                        var labelValue = proxyControl._reflection(ValueObject, layer.labelSettings.labelPath);
                        if (labelValue != null) {
                            if (bounds == null) {
                               //bounds = (shape);
                            }
                            var labelElement = $("<div class='labelStyle' style='font-family:Segoe UI;font-size:14px;white-space: nowrap;position:absolute;pointer-events:stroke;'></div>");
                            labelElement[0].innerHTML = labelValue;
                            this._templateDiv.append(labelElement);
                            $(labelElement).css({ "left": bounds.x, "top": bounds.y });
                            proxyControl._on($(labelElement), ej.eventType.mouseUp, { Param1: null, Param2: layer, Param3: shape }, proxyControl._polyClickFunction);
                            proxyControl._on($(labelElement), ej.eventType.mouseLeave, { Param1: layer,map:this }, proxyControl._polyLeaveFunction);
                            layer._labelCollection.push(labelElement);
                            layer._labelBounds.push(bounds);
                            layer._labelData.push(labelValue);

                        }
                    }
                    obj["shape"] = shape;
                    obj["shapeData"] = layer._polygonData[dataIndex];
                    if (shape != null) {
                        var shapefill = null; 
                        if (_isSVG) {
                            shapefill = shape.getAttribute("fill");
                            if (shapefill == "undefined")
                                shapefill = layer.shapeSettings.fill;
                            shape.setAttribute("nodeValue", shapefill);
                        }
                        else {
                            shapefill = shape.fillcolor.value;
                            if (shapefill == "undefined")
                                shapefill = layer.shapeSettings.fill;
                            shape.style.behavior = shapefill;
                        }
                    }
					if (proxyControl.enableZoomOnSelection()) {
                        $(shape).hover(function() {
                            $(this).css('cursor', 'pointer');
                        }, function() {
                            $(this).css('cursor', 'auto');
                        });
					}
					layer._mapShapes.push(obj);
					$(shape).mouseenter({ Param1: shapelayer, Param2: obj, map: proxyControl }, proxyControl._polyEnterFunction);
                    this._on($(shape), ej.eventType.mouseUp, { Param1: obj, Param2: shapelayer, Param3: shape }, proxyControl._polyClickFunction);
                    this._on($(shape), ej.eventType.mouseMove, { Param1: shapelayer, Param2: obj }, proxyControl._polyMoveFunction);
                    this._on($(shape), ej.eventType.mouseLeave, { Param1: shapelayer,Param2: obj,map:this }, proxyControl._polyLeaveFunction);
                    this._on($(shape), ej.eventType.mouseUp, { Param1: obj, Param2: shapelayer, Param3: shape }, proxyControl._polyUpFunction);
                    
                }
            }
        },


        /**
        * Calls the Zooms in and out method of the map
        * @private
        */
        _zooming: function (delta, e) {
            var position = this._getCurrentPoint(e);
            if (delta >= 120) {
                this._zoomingIn(position.x, position.y, e);
            }
            else {
                this._zoomingOut(position.x, position.y, e);
            }
        },

        /**
        * Calculate the zoom in functionality of map
        * @private
        */
        _zoomingIn: function (x, y, event, isAnimate) {
            var map = this;
            if (map.enableZoom() && map.zoomLevel() >= map.minZoom() && map.zoomLevel() <= map.maxZoom()) {
                var scal = map._scale;
                var position = { x: x, y: y };
                map._prevScale = scal;
                if (!this._isTileMap) {
                    map._prevPoint = { x: map._translatePoint.x, y: map._translatePoint.y };
                    map._translatePoint.x -= (map._width / map._scale - map._width / (map._scale + map.zoomFactor())) / (map._width / x);
                    map._translatePoint.y -= (map._height / map._scale - map._height / (map._scale + map.zoomFactor())) / (map._height / y);
                    map._scale = scal + map.zoomFactor();
                    map.zoomLevel(map.zoomLevel() + 1);
                }
                else {
                    this._tileZoom(map.zoomLevel() - 1, map.zoomLevel(), position);
                    var prevLevel = map.zoomLevel();
                    map.zoomLevel(map.zoomLevel() + 1);
                    this._generateTiles(this.zoomLevel());
                    map._translatePoint.x = (map._tileTranslatePoint.x - (0.5 * Math.pow(2, prevLevel))) / (Math.pow(2, prevLevel));
                    map._translatePoint.y = (map._tileTranslatePoint.y - (0.5 * Math.pow(2, prevLevel))) / (Math.pow(2, prevLevel));
                    map._scale = (Math.pow(2, prevLevel));
                }

                if (isAnimate != undefined && isAnimate && !this._isTileMap) {
                    map._animate(600);
                }
                else {
                    map._trigger("zoomedIn", { originalEvent: event, zoomLevel: map.zoomLevel() });
                    map._applyTransform(map._scale, map._translatePoint);
                    map._refrshLayers();
                    map._resizeShape();
                }
            
                map._updateSliderValue();
            }
        },

        /**
        * Calculate the zoom out functionality of map
        * @private
        */
        _zoomingOut: function (x, y, event, isAnimate) {
            var map = this;
            if (this.enableZoom()) {               
                if (this.zoomLevel() > this.minZoom()) {
                    var scal = map._scale;
                    var position = { x: x, y: y };
                    map._prevScale = scal;
                    if (!this._isTileMap) {                       
                        map._prevPoint = { x: map._translatePoint.x, y: map._translatePoint.y };
                        map._translatePoint.x -= (map._width / map._scale - map._width / (map._scale - map.zoomFactor())) / (map._width / x);
                        map._translatePoint.y -= (map._height / map._scale - map._height / (map._scale - map.zoomFactor())) / (map._height / y);
                        map._scale = map._scale - map.zoomFactor();
                        map.zoomLevel(map.zoomLevel() - 1);
                    }
                    else {
                        this._tileZoom(map.zoomLevel() + 1, map.zoomLevel(), position);
                        var prevLevel = map.zoomLevel();
                        map.zoomLevel(map.zoomLevel() - 1);
                        this._generateTiles(this.zoomLevel());
                        map._translatePoint.x = (map._tileTranslatePoint.x - (0.5 * Math.pow(2, this.zoomLevel()-1))) / (Math.pow(2, this.zoomLevel()-1));
                        map._translatePoint.y = (map._tileTranslatePoint.y - (0.5 * Math.pow(2, this.zoomLevel()-1))) / (Math.pow(2, this.zoomLevel()-1));
                        map._scale = (Math.pow(2, this.zoomLevel() - 1));                        
                    }

                    if (isAnimate != undefined && isAnimate && !this._isTileMap) {
                        map._animate(600);
                    }

                    else {
                        map._applyTransform(map._scale, map._translatePoint);
                        map._refrshLayers();
                        map._resizeShape();
                    }                   

                    map._updateSliderValue();
                }
                else {                   
                    this.zoom(this.minZoom(),false);
                }
                map._trigger("zoomedOut", { originalEvent: event, zoomLevel: map.zoomLevel() });
            }
        },

        /**
        * Calculates the bubble size ratio
        * @private
        */
        _getRatioOfBubble: function (min, max, value, minValue, maxValue) {
            var percent = (100 / (maxValue - minValue)) * (value - minValue);
            var bubbleRadius = (((parseFloat(max) - parseFloat(min)) / 100) * percent) + parseFloat(min);
            return bubbleRadius;
        },

        /**
        * Gets property from object
        * @private
        */
        _reflection: function (Source, path) {
            var ShapeValues = Source;
            if (path != null && Source != null) {
                var parts = path.split(".");
                var i = 0;
                for (; i < parts.length; i++) {

                    if (Source != undefined) {
                        var hasProperty = ShapeValues.hasOwnProperty(parts[i]);
                        if (hasProperty) {
                            $.each(ShapeValues, function (prop, propval) {
                                if (prop == parts[i]) {
                                    ShapeValues = propval;
                                    return false;
                                }

                            });
                        }
                    }
                }
                return ShapeValues;
            }
            else {
                return null;
            }

        },
		
		/**
        * Clear Selected Map Shapes
        * @public
        */
		
		clearShapeSelection:function(){		
		 var index = this.baseMapIndex();
            this.model.layers[index]._clearShapeWidth(this._scale);
            for (var key = 0; key < this.model.layers[index].subLayers.length; key++) {
                (this.model.layers[index].subLayers[key])._clearShapeWidth(this._scale);
            }
		},

        /**
        * Generates markers
        * @public
        */
        refreshMarkers: function () {
            var index = this.baseMapIndex();
            this.model.layers[index]._generateMarkerForLayers(this);
            for (var key = 0; key < this.model.layers[index].subLayers.length; key++) {
                (this.model.layers[index].subLayers[key])._generateMarkerForLayers(this);
            }
        },

        /**
        * Generates tooltip for layers
        * @private
        */
        _generateTooltipForLayers: function (layer) {
            if (layer.showTooltip) {
                var tooltipTemplateDiv = $('.tooltipTemplateDiv');
                tooltipTemplateDiv = $("<div class='shapeToolTip' style='z-index:1000;display:none;position:absolute;pointer-events:none;'></div>");
                $(document.body).append(tooltipTemplateDiv);
                layer._tooltipElement = tooltipTemplateDiv;
                if (layer.tooltipTemplate == null) {
                    var path = layer.shapeSettings.valuePath;
                    if (path != null) {
                        layer.tooltipTemplate = 'defaultTooltip';                      
                        this._mapContainer.append($('<div  id="defaultTooltip" style="display:none;"><div style="margin-left:10px;margin-top:-25px;"><div class="defaultToolTip"><p style="margin-top:-4px"><label  style="color:rgb(27, 20, 20);font-size:14px;font-weight:normal;font-family:Segoe UI">{{:#data["' + path + '"]}}</label></p></div></div></div>'));
                    }
                }
            }            
        },

        /**
        * Sorts in ascending order
        * @private
        */
        _orderByNameAscending: function (a, b) {
            if (a.Range == b.Range) {
                return 0;
            } else if (a.Range > b.Range) {
                return 1;
            }
            return -1;
        },

        /**
        * Resizes the container
        * @private
        */
        _resizingContainer: function () {            
            this._scale = this._isTileMap ? Math.pow(2, this.zoomLevel()-1) : this.zoomLevel();            
            if (this._isTileMap) {
                this._translatePoint.x = this._tileTranslatePoint.x / this._scale;
                this._translatePoint.y = this._tileTranslatePoint.y / this._scale;
            }
            else {                     
                this._translatePoint.x = this._tileTranslatePoint.x / this._scale;
                this._translatePoint.y = this._tileTranslatePoint.y / this._scale;
            }           
            this._applyTransform(this._scale, this._translatePoint);
            this._baseScale = this._scale;
            this._baseTranslatePoint = { x: (this._translatePoint.x * 2), y: (this._translatePoint.y * 2) };
           
        },

        /**
       * Calculates margin with conditions
       * @private
       */
        _calculateMarginConditions: function () {
            var trans = this._calculateMinMax();
            if (this._translatePoint.y> trans.maxY) {
                this._translatePoint.y = trans.maxY;
            }
            else if (this._translatePoint.y < trans.minY) {
                this._translatePoint.y = trans.minY;
            }
            if (this._translatePoint.x > trans.maxX) {
                this._translatePoint.x = trans.maxX;
            }
            else if (this._translatePoint.x < trans.minX) {
                this._translatePoint.x = trans.minX;
            }
        },

        /**
       * Calculates minimum and maximum of map boundaries
       * @private
       */
        _calculateMinMax: function () {           
            var bounds= this._groupBorder;            
            var maximumTransX, maximumTransY, minimumTransX, minimumTransY;
            if (this._containerHeight * this._scale <= this._height) {
                maximumTransY = (this._height - this._containerHeight * this._scale) / (2 * this._scale) - bounds.y;
                minimumTransY = (this._height - this._containerHeight * this._scale) / (2 * this._scale)-bounds.y;
            } else {
                maximumTransY = 0;
                minimumTransY = ((this._height - this._containerHeight * this._scale) / this._scale)-bounds.y;;
            }
            if (this._containerWidth * this._scale <= this._width) {
                maximumTransX = (this._width - this._containerWidth * this._scale) / (2 * this._scale)- bounds.x;
                minimumTransX = (this._width - this._containerWidth * this._scale) / (2 * this._scale) - bounds.x;
            } else {
                maximumTransX = 0;
                minimumTransX = ((this._width - this._containerWidth * this._scale) / this._scale) - bounds.x;
            }
            return { minX: minimumTransX, maxX: maximumTransX, minY: minimumTransY, maxY: maximumTransY };
        },

      
        /**
          * Calculates mid point of polygon
          * @private
          */
        _findMidPointofPoylgon: function (points) {
            var min = 0;           
            var max = points.length;
            var initialPoint = points[0];
            var minX=initialPoint.x,maxX = initialPoint.x;
            var minY = initialPoint.y, maxY = initialPoint.y;
            var startX,startY,startX1,startY1,sum=0,xSum=0,ySum=0;
           
            for (var i = min; i <= max - 1; i++) {
                var prevPoint = points[i];
                startX = prevPoint.x;
                startY = prevPoint.y;
                if (i == max - 1) {
                    startX1 = initialPoint.x;
                    startY1 = initialPoint.y;
                }
                else {
                    var nextPoint = points[i + 1];
                    startX1 = nextPoint.x;
                    startY1 = nextPoint.y;
                }
                sum = sum + Math.abs(((startX * startY1)) - (startX1 * startY));
                xSum = xSum + Math.abs(((startX + startX1) * (((startX * startY1) - (startX1 * startY)))));
                ySum = ySum + Math.abs(((startY + startY1) * (((startX * startY1) - (startX1 * startY)))));

            }
            sum = 0.5 * sum;
            xSum = (1 / (4 * sum)) * xSum;
            ySum = (1 / (4 * sum)) * ySum;
            return { x: xSum, y: ySum };
           

            //for (var i = min; i < max; i++) {
            //    minX = Math.min(minX, points[i].x);
            //    maxX = Math.max(maxX, points[i].x);
            //    minY = Math.min(minY, points[i].y);
            //    maxY= Math.max(maxY, points[i].y);
            //}          
            //return {x:minX+((maxX-minX)/2),y:(minY+(maxY-minY)/2)};            
        },


        /**
          * Converts tiles' latitude and longitude point for shapes
          * @private
          */
        _convertTileLatLongtoPointForShapes: function (lat, lng, bounds,vmlBound) {
            var Size;
            if (bounds == null) { bounds=[[-180,-85],[180,85]] }
            if (this._isTileMap) {
                Size = Math.pow(2, 1) * 256;
            }
            else {
                var factor;
                if (this._height < this._width) {
                    var percent = ((bounds[1][1] - bounds[0][1])/170) * 100;
                    factor = this._height/((percent * this._height) / 100);
                }
                else {
                    var percent = ((bounds[1][0] - bounds[0][0]) / 360) * 100;
                    factor = this._width/((percent * this._width) / 100);
                }

                Size = Math.min(this._height, this._width)*factor;      
            }
           
            var x = (lng + 180) / 360;
            var sinLatitude = Math.sin(lat * Math.PI / 179.5);
            if (sinLatitude == -1)
                sinLatitude = -0.5;
            var y = 0.5 - Math.log((1 + sinLatitude) / (1 - sinLatitude)) / (4 * Math.PI);
            var pixelX = (x * Size + 0.5);
            var pixelY = (y * Size + 0.5);
            if (vmlBound!=null) {
                pixelX -= vmlBound.x;
                pixelY -= vmlBound.y;
            }
            return { x: pixelX, y: pixelY };
           
            
        },
        
        /**
          * Converts tiles' latitude and longitude point
          * @private
          */
        _convertTileLatLongtoPoint: function (lat, lng) {
            var Size = Math.pow(2, this.zoomLevel()) * 256;          
            var x = (lng + 180) / 360; 
            var sinLatitude = Math.sin(lat * Math.PI / 180);
            var y = 0.5 - Math.log((1 + sinLatitude) / (1 - sinLatitude)) / (4 * Math.PI);
            var pixelX = (x * Size + 0.5) + this._tileTranslatePoint.x;
            var pixelY = (y * Size + 0.5) + this._tileTranslatePoint.y;            
            return { x: pixelX, y: pixelY };
        },
       
        /**
         * Converts tiles' latitude and longitude point for Marker
         * @private
         */
        _convertLatLongtoPointforMarker: function (lat, lng) {
            var point = this._convertTileLatLongtoPointForShapes(lat, lng, this._mapBounds);
            return { x: (point.x + this._translatePoint.x) * this._scale, y: (point.y + this._translatePoint.y) * this._scale };

        },
     
        /**
        * Animate the map
        * @private
        */
        _animate: function (animateduration) {
            this._calculateMarginConditions();
            $(this._rootgroup).stop(true, false);
            if (this._sliderControl != null) {
                $(this._sliderControl).stop(true, false);
            }
            var map = this;
            var currentTranformation = { x: this._translatePoint.x, y: this._translatePoint .y};
            var startPoint = [this._prevPoint.x, this._prevPoint.y];
            var endPoint = [this._translatePoint.x, this._translatePoint.y];

            function slope(horizontal, vertical) {
                
                if (horizontal[0] == vertical[0]) {
                    return null;
                }
                return (vertical[1] - horizontal[1]) / (vertical[0] - horizontal[0]);
            }

            function intercept(point, slopeValue) {
                if (slopeValue === null) {
                    return point[0];
                }
                return point[1] - slopeValue * point[0];
            }

            var slopeFactor = slope(startPoint, endPoint);
            var slopeIntersection = intercept(startPoint, slopeFactor);
            var horizontalDifference= endPoint[0] - startPoint[0];
            var verticalDifference=endPoint[1] - startPoint[1];
            var scaleDifference= this._scale-this._prevScale;
            var currentScale = this._scale;
            this._updateSliderValue();
            $(this._rootgroup).animate(
                {
                    count: 10
                },
                {
                    duration: animateduration,
                    step: function(now, fx) {
                        var scaleX = map._prevScale + ((now * (scaleDifference / fx.end)));
                        var transX = startPoint[0] + (now * (horizontalDifference / fx.end)) / (scaleX / currentScale);
                        var transY;
                        if (slopeFactor == null) {
                            transY = startPoint[1] + (now * (verticalDifference / fx.end));
                        } else {
                            transY = ((slopeFactor * transX) + slopeIntersection);
                        }
                        if (_isSVG) {
                            map._rootgroup.setAttribute('transform', 'scale(' + scaleX + ') translate(' + transX + ', ' + transY + ')');
                        }
                        else {
                            map._rootgroup.coordorigin = -transX + ',' + -transY;
                            map._rootgroup.coordsize = (map._width / scaleX) + ',' + (map._height / scaleX);
                        }
                        map._translatePoint.x = transX;
                        map._translatePoint.y = transY;
                        map._scale = scaleX;
                        map._refrshLayers();
                        map._resizeShape();
                    },
                    complete: function() {
                        map._translatePoint.x = currentTranformation.x;
                        map._translatePoint.y = currentTranformation.y;
                        map._scale = currentScale;
                        this.count = 0;
                    }
                }
            );

        },

        /**
        * Applies transforms to rootgroup
        * @private
        */
        _applyTransform: function (scale, translatePoint) {
            this._scale = scale;
            this._translatePoint = translatePoint;
           if(!this._isTileMap)
            this._calculateMarginConditions();            
            if (_isSVG) {
                this._rootgroup.setAttribute('transform', 'scale(' + this._scale + ') translate(' + this._translatePoint.x + ', ' + this._translatePoint.y + ')');
            }
            else {
                this._rootgroup.coordorigin = (-this._translatePoint.x) + ','  + (-this._translatePoint.y);
                this._rootgroup.coordsize = (this._width / scale)  + ',' + (this._height / scale);
            }

        },

        /**
         * Section For handling the mouse move event
		 * @private
         */
        _mouseMove: function (event) {
            if (event.type == "touchmove" && event.originalEvent.touches.length > 1) {
                this._isPinching = true;
            }
            else {
                var map = this;
                if (map.ispanning && map.model.enablePan) {
                    event.preventDefault();
                    var position;
                    if (event.type == "mousemove")
                        position = { x: event.pageX, y: event.pageY };
                    else if (event.type == "touchmove")
                        position = { x: event.originalEvent.changedTouches[0].pageX, y: event.originalEvent.changedTouches[0].pageY };
                    else if (event.type == "MSPointerMove")
                        position = { x: event.originalEvent.pageX, y: event.originalEvent.pageY };
                    if (this._isTileMap) {
                        var curtileX = this._tileTranslatePoint.x - (this._dragStartX - position.x);
                        var curtileY = this._tileTranslatePoint.y - (this._dragStartY - position.y);
                        this._tileTranslatePoint.x = curtileX;
                        this._tileTranslatePoint.y = curtileY;                        
                        this._generateTiles(this.zoomLevel());
                    }                    
                    var curX = map._translatePoint.x - (map._dragStartX - position.x) / map._scale;
                    var curY = map._translatePoint.y - (map._dragStartY - position.y) / map._scale;
                    if (curX != map._translatePoint.x || curY != map._translatePoint.y) {
                        map._isDragging = true;
                    }
                    this._translatePoint.x = curX;
                    this._translatePoint.y = curY;
                    map._applyTransform(map._scale, map._translatePoint);
                    map._dragStartX = position.x;
                    map._dragStartY = position.y;
                    map._refrshLayers();
                }
            }
        },

        /**
         * Section For handling the mouse leave event
		 * @private
         */
        _svgMouseLeave: function (event) {
            var map = this;
            map.ispanning = false;
        },

        /**
        * Section For handling the mouse wheel event
        * @private
        */
        _mouseWheel: function (event) {
		    if(this.model.enableAnimation){
				$(this._rootgroup).stop(true, false);
			}
            var e = event.originalEvent;
            var original = event;
            if (!_isSVG) {
                e = event;
            }
            e.preventDefault ? e.preventDefault() : original.preventDefault();
	    	if(_isSVG)
	    	{
	    		this._zooming(event.originalEvent.wheelDelta, event.originalEvent);
	    	}
	    	else
	    	{
            	this._zooming(event.originalEvent.wheelDelta, e);
	    	}
        },

        /**
        * Section For handling the mouse button up event
        * @private
        */
        _mouseButtonUp: function (event) {

            var map = this;
            if (map.ispanning) {
                map._trigger("panned", { originalEvent: event });
            }
            map.ispanning = false;
        },

        /**
        * Section For handling the mouse up event
        * @private
        */
        _mouseUp: function (event) {
            this.ispanning = false;
            this._isDragging = false;
        },

        /**
        * Section For handling the mouse button down event
        * @private
        */
        _mouseButtonDown: function (event) {
            this._isPinching = false;
           if (event.target.className != "e-vhandle") {         
            this._isNavigationPressed = false;
            if(this.model.enableAnimation){
                $(this._rootgroup).stop(true, false);
                if (this._sliderControl != null) {
                    $(this._sliderControl).stop(true, false);
                }
			}
            if (event.type == "touchstart" && event.originalEvent.touches.length > 1)
            { }
            else {
                event.preventDefault();
                var position;
                if (event.type == "mousedown")
                    position = { x: event.pageX, y: event.pageY };
                else if (event.type == "touchstart")
                    position = { x: event.originalEvent.changedTouches[0].pageX, y: event.originalEvent.changedTouches[0].pageY };
                else if (event.type == "MSPointerDown")
                    position = { x: event.originalEvent.pageX, y: event.originalEvent.pageY };
                var map = this;
                map.ispanning = true;
                map._dragStartX = position.x;
                map._dragStartY = position.y;
            }
		}
        },

        /**
       * Section to get current point position
       * @private
       */
        _getCurrentPoint: function (event) {
            var map = this._mapContainer;
            var xPos = event.pageX - map[0].offsetLeft;
            var yPos = event.pageY - map[0].offsetTop;
            return { x: xPos, y: yPos };
        },

        /**
        * Section For handling the double click event
        * @private
        */
        _doubleClick: function (event) {            
            var map = this;
            if (!this._isNavigationPressed && event.target.className.toString().indexOf("e-icon1")==-1) {
                var position = this._getCurrentPoint(event);               
                if (map.enableZoom() && map.zoomLevel()+1 >= map.minZoom() && map.zoomLevel()+1 <= map.maxZoom()) {
                    var scal = map._scale;
                    map._prevScale = scal;
                    if (!this._isTileMap) {
                        map._prevPoint = { x: map._translatePoint.x, y: map._translatePoint.y };
                        map._translatePoint.x -= (map._width / map._scale - map._width / (map._scale + map.zoomFactor())) / (map._width / position.x);
                        map._translatePoint.y -= (map._height / map._scale - map._height / (map._scale + map.zoomFactor())) / (map._height / position.y);
                        map._scale = scal + map.zoomFactor();                       
                        map.zoomLevel(map.zoomLevel() + 1);
                    }
                    else {
                        this._tileZoom(map.zoomLevel() - 1, map.zoomLevel(), position);
                        var prevLevel = map.zoomLevel();
                        map.zoomLevel(map.zoomLevel() + 1);
                        this._generateTiles(this.zoomLevel());                       
                        map._translatePoint.x = (map._tileTranslatePoint.x - (0.5 * Math.pow(2, prevLevel))) / (Math.pow(2, prevLevel));
                        map._translatePoint.y = (map._tileTranslatePoint.y - (0.5 * Math.pow(2, prevLevel))) / (Math.pow(2, prevLevel));
                        map._scale = (Math.pow(2, prevLevel));                        
                    }
                    if (map.model.enableAnimation && !this._isTileMap) {
                        map._animate(600);
                    }
                    else {
                        map._applyTransform(map._scale, map._translatePoint);
                        map._refrshLayers();
                        map._resizeShape();
                    }                 
                    map._updateSliderValue();
                }
            }
        },

        /**
        * Section to zoom the tile
        * @private
        */
        _tileZoom: function (prevlevel,level, position) {
            
            var map = this;
            if (level > 0 && level < 20) {
                this._tileDiv.html("");
                var prevSize = Math.pow(2, prevlevel) * 256;
                if (position == undefined) {
                    position = { x: this._width / 2, y: this._height / 2 };
                }
                var totalSize = Math.pow(2, level) * 256;
                var percentX = ((position.x - this._tileTranslatePoint.x) / prevSize) * 100;
                var percentY = ((position.y - this._tileTranslatePoint.y) / prevSize) * 100;
                var pointX = (percentX * totalSize) / 100;
                var pointY = (percentY * totalSize) / 100;
                this._tileTranslatePoint.x = position.x - pointX;
                this._tileTranslatePoint.y = position.y - pointY;
                
            }
        },          
        
        /**
        * Section For handling the mouse enter event for bubble
        * @private
        */
        _bubbleEnterFunction: function (event) {
            if (event.data.templateID != null) {
                var tooltiphtmlString = $("#" + event.data.templateID).render(event.data.itemsrc);
                $(event.data.htmlobj).html(tooltiphtmlString);
            }
            $(document.body).append(event.data.htmlobj);
            return $(event.data.htmlobj).css("display", "block").css({ "left": (event.pageX + 8) + "px", "top": (event.pageY + 6) + "px" });
        },

        /**
        * Section For handling the mouse leave event for bubble
        * @private
        */
        _bubbleleaveFunction: function (event) {
            $(event.data.htmlobj).remove();
            return $(event.data.htmlobj).css("display", "none");
        },

        /**
        * Section For handling the mouse over event for bubble
        * @private
        */
        _bubbleOverFunction: function (event) {
            return $(event.data.htmlobj).css("display", "block").css({ "left": (event.pageX + 8) + "px", "top": (event.pageY + 6) + "px" });
        },

        /**
       * Section For handling the mouse over event for shape path
       * @private
       */
        _polyEnterFunction: function (event) {
            var layer = event.data.Param1;
            var shape = event.data.Param2["shape"];
            var map = event.data.map;
            if (map != null && layer.enableMouseHover && map._browser!='msie') {
                map._rootgroup.insertBefore(shape, map._rootgroup.childNodes[map._rootgroup.childNodes.length- (map._polylineCount+map._pointscount) - 1]);
            }
            var legend = jQuery.grep(layer._smartLabels, function (obj) { return obj.shape == shape });
            if (layer.enableMouseHover) {
                layer._clearShapeWidth(map._scale);
                if (layer._prevHoverdLegend != null && !layer._contains(layer._prevSelectedLegends, layer._prevHoverdLegend)) {
                    layer._prevHoverdLegend.css("background-color", _isSVG ? layer._prevHoveredShape.getAttribute("fill") : layer._prevHoveredShape.fillcolor.value);
                }
                if (_isSVG) {
                    shape.setAttribute("stroke-width", (layer.shapeSettings.highlightBorderWidth/map._scale));
                    shape.setAttribute('stroke', layer.shapeSettings.highlightStroke);
                } else {
                    shape.strokeweight = layer.shapeSettings.highlightBorderWidth;
                    shape.strokecolor = layer.shapeSettings.highlightStroke/map._scale;
                }

                if (layer.shapeSettings.highlightColor != null) {
                    if (layer.shapeSettings.highlightColor != "transparent" && !layer._contains(layer._prevSelectedShapes,shape)) {
                        if (_isSVG) {
							if(layer.shapeSettings.highlightColor!='none'){
								shape.setAttribute('fill', layer.shapeSettings.highlightColor);								
							}
							shape.setAttribute("stroke-width", (layer.shapeSettings.highlightBorderWidth/map._scale));
							shape.setAttribute('stroke', layer.shapeSettings.highlightStroke);
                        } else {
                            shape.fillcolor = layer.shapeSettings.highlightColor;
                        }
                    }
                    if (legend.length > 0 && !layer._contains(layer._prevSelectedLegends,legend[0].legend)) {
                        
                        legend[0].legend.css("background-color", layer.shapeSettings.highlightColor);
                        layer._prevHoverdLegend = legend[0].legend;
                    }

                    layer._prevHoveredShape = shape;
                }

                if (layer.legendSettings!= null && layer.legendSettings.mode == ej.datavisualization.Map.LegendMode.Interactive && layer.shapeSettings.colorMappings!=null && layer.shapeSettings.colorMappings.rangeColorMapping != null
                     && !layer.shapeSettings.autoFill && (layer.legendSettings.showLegend == undefined || layer.legendSettings.showLegend)) {
                    for (var i = 0; i < layer._mapShapes.length; i++) {
                        var mapshape = layer._mapShapes[i].shape;
                        if (mapshape == shape) {
                            var mappings = null;
                            if (layer.shapeSettings.colorMappings.rangeColorMapping != undefined) {
                                mappings = layer.shapeSettings.colorMappings.rangeColorMapping;
                            }
                            var _legendwidth = layer.legendSettings.width;
                            if (layer.legendSettings.width == undefined)
                                _legendwidth = 150;

                            var rectwidth = (_legendwidth / mappings.length) / 10;
                            if (layer._mapShapes[i].legendrect != null) {
                                var rectPosition = layer._mapShapes[i].legendrect.marginLeft;
                                $(layer._interactiveArrow).css({ "margin-left": rectPosition + Math.ceil(rectwidth) - layer._interactiveArrow.width() / 2, "display": "block" });
                                break;
                            }
                        }
                    }
                }
                map._trigger("mouseover", { originalEvent: event.data.Param2 });
            }
                var tooltipObject = event.data.Param2;
                if (tooltipObject.hasOwnProperty("data") && layer.showTooltip) {
                    var element = layer._tooltipElement;
                    var template = layer.tooltipTemplate;
                    if (element != null && template != null) {
                        $(element).css({ "left": event.pageX, "top": event.pageY, "display": "block" });
                        var htmlString = $("#" + template).render(tooltipObject["data"]);
                        $(element).html(htmlString);

                    }
                }
                
        },
        
        /**
       * updates shape rect
       * @private
       */
        _updateShapeRect:function(layer) {

            for (var index = 0; index < layer._mapShapes.length; index++) {
                var obj = layer._mapShapes[index];
                if (obj.left == null) {
                    obj["left"] = layer._mapShapes[index].shape.getBoundingClientRect().left;
                    obj["right"] = layer._mapShapes[index].shape.getBoundingClientRect().right;
                    obj["top"] = layer._mapShapes[index].shape.getBoundingClientRect().top;
                    obj["bottom"] = layer._mapShapes[index].shape.getBoundingClientRect().bottom;
                    layer._mapShapes[index] = obj;
                }
            }
        },
        

        /**
       * Section For handling the mouse up event for shape path
       * @private
       */
        _polyUpFunction: function (event) {
            var layer = event.data.Param2;
            var shape = event.data.Param3;
            if (this.enableZoomOnSelection() && !this._isDragging && !this._isTileMap && !this._isPinching) {
                    if (_isSVG) {
                        var bounds = shape.getBBox();
                        this._zoomOnSelection(bounds);
                    } else {                        
                            this._updateShapeRect(layer);

                        for (var index = 0; index < layer._mapShapes.length; index++) {
                            if (shape == layer._mapShapes[index].shape) {
                                var left = layer._mapShapes[index].left;
                                var top = layer._mapShapes[index].top;
                                var right = layer._mapShapes[index].right;
                                var bottom = layer._mapShapes[index].bottom;
                                var bound = { x: left, y: top, width: right - left, height: bottom - top };
                                this._zoomOnSelection(bound);
                                break;
                            }
                        }
                    }
                    this.ispanning = false;
                }
            
            //this._updateSelection(layer,shape,event.data.Param1);
        },

        /**
       * Section For handling the mouse click event for Marker
       * @private
       */
        _markerPressed: function (event) {
            this._trigger("markerSelected", { originalEvent: event.data });
        },

        /**
       * Section For handling the mouse click event for shape path
       * @private
       */
       _polyClickFunction: function (event) {

            var ctrlkey = event.ctrlKey;
            var layer = event.data.Param2;
            var map = this;
            var shape = event.data.Param3;
            var legend = jQuery.grep(layer._smartLabels, function (obj) { return obj.shape == shape });
            map._isPolygonSelected = true;

            if (layer.enableSelection) {

                if (map._browser != 'msie') {
                    map._rootgroup.insertBefore(shape, map._rootgroup.childNodes[map._rootgroup.childNodes.length - (map._polylineCount + map._pointscount) - 1]);
                }
                if (ctrlkey && layer.selectionMode == ej.datavisualization.Map.SelectionMode.Multiple) {
                    if ($.inArray(shape, layer._prevSelectedShapes) == -1) {
                        shape.setAttribute("fill", layer.shapeSettings.selectionColor);
                        shape.setAttribute("stroke", layer.shapeSettings.selectionStroke);
                        shape.setAttribute("stroke-width", (layer.shapeSettings.selectionStrokeWidth / map._scale));
                        layer._prevSelectedShapes.push(shape);
                        layer._prevSelectedShapeDatas.push(event.data.Param1);
                    }
                    else {
                        shape.setAttribute("fill", shape.getAttribute("nodeValue"));
                        shape.setAttribute("stroke", layer.shapeSettings.stroke);
                        shape.setAttribute("stroke-width", layer.shapeSettings.strokeThickness / this._scale);
                        var index = layer._prevSelectedShapes.indexOf(shape);
                        layer._prevSelectedShapes.splice(index, 1);
                        layer._prevSelectedShapeDatas.splice(index, 1);
                    }
                }

                else {
                    
                        for (var i = 0; i < layer._prevSelectedShapes.length; i++) {
                            layer._prevSelectedShapes[i].setAttribute("fill", layer._prevSelectedShapes[i].getAttribute("nodeValue"));
                            layer._prevSelectedShapes[i].setAttribute("stroke", layer.shapeSettings.stroke);
                            layer._prevSelectedShapes[i].setAttribute("stroke-width", layer.shapeSettings.strokeThickness / this._scale);
                        }
                        if (!this._isPinching && ($.inArray(shape, layer._prevSelectedShapes) == -1)) {
                            layer._prevSelectedShapes = [];
                            layer._prevSelectedShapeDatas = [];
                            shape.setAttribute("fill", layer.shapeSettings.selectionColor);
                            shape.setAttribute("stroke", layer.shapeSettings.selectionStroke);
                            shape.setAttribute("stroke-width", (layer.shapeSettings.selectionStrokeWidth / map._scale));
                            layer._prevSelectedShapes.push(shape);
                            layer._prevSelectedShapeDatas.push(event.data.Param1);
                        }
                        else {
                            layer._prevSelectedShapes=[];
                            layer._prevSelectedShapeDatas = [];
                        }
                        
                    

                    
                }
            }
            map._trigger("shapeSelected", { originalEvent: layer._prevSelectedShapeDatas });
        },

        /**
        * Updates previous selected shape
        * @private
        */
        _updateSelection: function (layer, shape, data) {
            if (layer.enableSelection) {
                if (layer.selectionMode != ej.datavisualization.Map.SelectionMode.Multiple) {
                    layer._prevSelectedShapes.pop();
                    layer._prevSelectedShapeDatas.pop();
                }
                layer._prevSelectedShapes.push(shape);
                layer._prevSelectedShapeDatas.push(data);
            }
        },

        /**
      * Zooms map on shape selection
      * @private
      */
	   _zoomOnSelection: function (bounds) {            
	       var layerScale;
	       if (!_isSVG) {
	           bounds.x = (bounds.x - (this._baseTranslatePoint.x/2)) - (bounds.width/2);
	           bounds.y = (bounds.y - (this._baseTranslatePoint.y/2)) - (bounds.height / 2);
	       }
	       var boundwidth = bounds.width;
	       var boundHeight =bounds.height;
	       if ((this._width - 100) / (this._height - 100) > boundwidth / boundHeight) {
	           layerScale = (this._height - 100) / boundHeight;
	       } else {
	           layerScale = (this._width - 100) / boundwidth;
	       }
	       this._prevScale = this._scale;
	       this._scale = layerScale;
	       this._prevPoint = { x: this._translatePoint.x, y: this._translatePoint.y };
	       this.zoomLevel(this._scale - this._baseScale + 1);
	       var level = this.zoomLevel();
	       if (!(level > this.minZoom() && level < this.maxZoom())) {
	           this.zoomLevel(level > (((this.maxZoom() - this.minZoom()) / 2) + this.minZoom()) ? this.maxZoom() : this.minZoom());
	           this._scale = this.zoomLevel() + this._baseScale - 1;
	       }
	       var leftMargin = (this._width / 2) - ((boundwidth * this._scale) / 2);
	       var leftPos = leftMargin / this._scale;
	       var topMargin = (this._height / 2) - ((boundHeight * this._scale) / 2);
	       var topPos = topMargin / this._scale;
	       this._translatePoint.x = -bounds.x + leftPos;
	       this._translatePoint.y = -bounds.y + topPos;
	       if (this.model.enableAnimation && !this._isTileMap) {
	           this._animate(1200);
	       }
	       else {
	           this._applyTransform(this._scale, this._translatePoint);
	           this._updateSliderValue();
	       }

	       //this._refrshLayers();
            
        },

        /**
      * Section For handling the mouse move event for shape path
      * @private
      */
        _polyMoveFunction: function (event) {
            var element = event.data.Param1._tooltipElement;
            var tooltipObject = event.data.Param2;
            if (element != null && event.data.Param1.showTooltip && tooltipObject.hasOwnProperty("data")) {
                $(element).css({ "display": "block", "left": event.pageX + 10, "top": event.pageY + 10});
            }
        },

        /**
      * Section For handling the mouse leave event for shape path
      * @private
      */
        _polyLeaveFunction: function (event) {
            var layer = event.data.Param1;
            var map = event.data.map;
            var element = layer._tooltipElement;            
            if (element != null) {
                $(element).css("display", "none");
            }
            if (layer.legendSettings!=null && layer.legendSettings.mode == ej.datavisualization.Map.LegendMode.Interactive && layer._interactiveArrow != null)
                $(layer._interactiveArrow).css("display", "none");
            layer._clearShapeWidth(map._scale);
            map._trigger("mouseleave", { originalEvent: event.data.Param2 });
        },

        /**
         * Triggering the needed events for map control		
		 * @private
         */
        _wireEvents: function () {
            var matched = jQuery.uaMatch(navigator.userAgent);
            var browser = matched.browser.toLowerCase();
            this._on($(this._mapContainer), ej.eventType.mouseDown, this._mouseButtonDown);
			$(document).keydown({ className: "home", map: this }, this._keyboardKeysPressed);
            this._on($(this._mapContainer), ej.eventType.mouseMove, this._mouseMove);
            this._on($(document), ej.eventType.mouseUp, this._mouseUp);
            var isIE11 = !!navigator.userAgent.match(/Trident\/7\./);
            if (isIE11)
                browser = "msie";
            if (window.navigator.msPointerEnabled) {
                this._on($(this._mapContainer), "MSPointerUp", this._mouseButtonUp);
                $(this._mapContainer).css('-ms-touch-action', 'none');
            }
            else {
                this._on($(this._mapContainer), "mouseup", this._mouseButtonUp);
            }
            var map = this;
            this._browser = browser;
            if (browser != "mozilla") {
                this._on($(this._mapContainer), "mousewheel", this._mouseWheel);
            }
            else {
                var elem = this._svgDocument;
                if (elem.addEventListener) {
                    // Mouse Scrolling event for firefox
                    elem.addEventListener("DOMMouseScroll", MouseWheel, false);
                }

            }
            function MouseWheel(event) {
			if(map.model.enableAnimation){
				$(map._rootgroup).stop(true, false);
			}
			  event.preventDefault(event);
                map._zooming(-40 * event.detail, event);
            }
            this._on($(this._mapContainer), "dblclick", this._doubleClick);
            if (this.model.enableResize) {
                if (!this._isDevice())
                    this._on($(window), "resize", this._mapResize);
                else
                    this._on($(window), "orientationchange", this._mapResize);
            }
        },

        /**
       * Section For handling the map resize event
       * @private
       */
        _mapResize: function (event) {
            event.preventDefault();
            event.stopPropagation();
            var map = this;
            if (this.resizeTO) clearTimeout(this.resizeTO);
            this.resizeTO = setTimeout(function () {
                map.refresh();
            }, 500);
        },

        /**
        * checks the device type
        * @private
        */
        _isDevice: function () {
            return (/mobile|tablet|android|kindle/i.test(navigator.userAgent.toLowerCase()));
        },

        /**
         * Unwiring the needed events for map control		
		 * @private
         */
        _unWireEvents: function () {
            var matched = jQuery.uaMatch(navigator.userAgent);
            var browser = matched.browser.toLowerCase();
            this._on($(document), ej.eventType.mouseUp, this._mouseUp);
            var isIE11 = !!navigator.userAgent.match(/Trident\/7\./);
            if (isIE11)
                browser = "msie";
            if (window.navigator.msPointerEnabled) {
                this._off($(this._mapContainer), "MSPointerDown", this._mouseButtonDown);
                this._off($(this._mapContainer), "MSPointerMove", this._mouseMove);
                this._off($(this._mapContainer), "MSPointerUp", this._mouseButtonUp);
                $(this._mapContainer).css('-ms-touch-action', 'none');
            }
            else if (browser == "webkit" || browser == "chrome" || browser == "mozilla") {
                this._off($(this._mapContainer), "mousedown", this._mouseButtonDown);
                this._off($(this._mapContainer), "mousemove", this._mouseMove);
                this._off($(this._mapContainer), "mouseup", this._mouseButtonUp);

            }
            else {
                this._off($(this._mapContainer), "mousedown", this._mouseButtonDown);
                this._off($(this._mapContainer), "mousemove", this._mouseMove);
                this._off($(this._mapContainer), "mouseup", this._mouseButtonUp);

            }
            this._off($(this._mapContainer), "dblclick", this._doubleClick);

        },

        /**     
        * Method for navigating to specific shape based on latitude, longitude and zoomlevel.
        * @return jQuery
        * @param {number} latitude Pass the latitude value for map
	    * @param {number} longitude Pass the longitude value for map
	    * @param {number} level Pass the zoom level for map
        * @example 
        * //navigateTo method for map
        * $("#container").ejMap({
        *    navigateTo: function (lat, long, level) {}
        * });
        * @memberof ejMap
        * @instance
        */
        navigateTo: function (latitude, longitude, level) {
		
		    level=parseFloat(level);
            if (level == undefined) level = this.zoomLevel();
            if (level > this.minZoom() && level < this.maxZoom()) {
                this.zoomLevel(level);
            }
            else {
                this.zoomLevel(level > (((this.maxZoom() - this.minZoom()) / 2) + this.minZoom()) ? this.maxZoom() : this.minZoom());

            }           
            var translatePoint = this._convertTileLatLongtoPointForShapes(latitude, longitude, this._mapBounds);
            this._prevPoint = { x: this._translatePoint.x, y: this._translatePoint.y };
            this._prevScale = this._scale;
            this._scale = this._baseScale + ((level - 1) * this.zoomFactor());           
            var leftPosition = ((this._containerWidth + this._baseTranslatePoint.x) / 2) / this._scale;
            var topPosition = ((this._containerHeight + this._baseTranslatePoint.y) / 2) / this._scale;
            this._translatePoint.x = -translatePoint.x+ leftPosition;
            this._translatePoint.y = -translatePoint.y+ topPosition;
            if (this.model.enableAnimation && !this._isTileMap ){
                this._animate(1200);
            }
            else {
                this._applyTransform(this._scale, this._translatePoint);
            }
            this._updateSliderValue();
            this._refrshLayers();
        },

        /**
        * Performs shape selection on map
        * @private
        */
        selectShape: function (obj, layer,isZoom) {
            if (obj != null) {
                if (layer == null) {
                    layer = this.model.layers[this.baseMapIndex()];
                }
                for (var i = 0; i < layer._mapShapes.length; i++) {
                    var data = layer._mapShapes[i].data;
                    var shape = layer._mapShapes[i].shape;
                    if (data != null && obj == this._reflection(data, layer.shapeSettings.valuePath)) {

                        if (layer._prevSelectedShapes.length != 0 && layer.selectionMode != ej.datavisualization.Map.SelectionMode.Multiple) {
                            for (var i = 0; i < layer._prevSelectedShapes.length; i++) {
                                if (_isSVG) {
                                    layer._prevSelectedShapes[i].setAttribute("fill", layer._prevSelectedShapes[i].getAttribute("nodeValue"));
                                    layer._prevSelectedShapes[i].setAttribute("stroke", layer.shapeSettings.stroke);
                                    layer._prevSelectedShapes[i].setAttribute("stroke-width", layer.shapeSettings.strokeThickness / this._scale);
                                } else {
                                    layer._prevSelectedShapes[i].fillcolor = layer._prevSelectedShapes[i].style.behavior;
                                    layer._prevSelectedShapes[i].strokecolor = layer.shapeSettings.stroke;
                                    layer._prevSelectedShapes[i].strokeweight = layer.shapeSettings.strokeThickness / this._scale;
                                }
                            }
                        }
                        if (layer.enableSelection) {
                            if (_isSVG) {
							    if(layer.shapeSettings.selectionColor!="none"){
                                    shape.setAttribute("fill", layer.shapeSettings.selectionColor);
							    }
                                shape.setAttribute("stroke", layer.shapeSettings.selectionStroke);
                                shape.setAttribute("stroke-width", (layer.shapeSettings.selectionStrokeWidth / this._scale));
                            } else {
							     if(layer.shapeSettings.selectionColor!="none"){
                                      shape.fillcolor = layer.shapeSettings.selectionColor;
								}
                                shape.strokecolor = layer.shapeSettings.selectionStroke;
                                shape.strokeweight = layer.shapeSettings.selectionStrokeWidth / this._scale;
                            }
                            this._updateSelection(layer, shape, obj);
                            this._trigger("shapeSelected", { originalEvent: layer._prevSelectedShapeDatas });
                        }
                        if (isZoom && this.enableZoomOnSelection()) {
                            this._zoomOnSelection(shape.getBBox());
                        }
                        i = layer._mapShapes.length;
                    }
                }
            }

        },

        /**
        * Gets intersected elements
        * @private
        */
        _getIntersectedElements: function (evt, shapes) {
            evt.width += 5;
            evt.height += 5;
            if (_isSVG && this._browser!='mozilla' && this._browser!='webkit') {
                var rpos = this._svgDocument.createSVGRect();
                rpos.x = evt.left;
                rpos.y = evt.top;
                rpos.width = evt.width;
                rpos.height = evt.height;
                return this._svgDocument.getIntersectionList(rpos, null);
            }
            else {
                var elements = []; 
				var parentSize=$(this._mapContainer).offset();
                var  parentLeft=parentSize.left;
                var parentTop= parentSize.top;			
                for (var i = 0; i < shapes.length; i++) {
                    var shape = shapes[i].shape;
                    var bounds = shape.getBoundingClientRect();					
					var leftPos= $(shape).offset().left- parentLeft;
					var topPos= $(shape).offset().top- parentTop;					
                    if (this._isIntersect(evt, { left: leftPos, top: topPos, height: (bounds.bottom  - bounds.top ), width: (bounds.right -bounds.left) })) {
                        elements.push(shape);
                        return elements;
                    }
                }
                return elements;
            }
                
                
        },

        /**
        * Checks intersected elements
        * @private
        */
        _isIntersect: function (rect1, rect2) {           
            
                if (rect1.left >= (rect2.left + rect2.width)
|| rect1.left + rect1.width <= rect2.left
|| rect1.top>= rect2.top + rect2.height
|| rect1.top + rect1.height <= rect2.top) {
                    return false;
                }
                return true;
            
        },

        /**     
        * Method to perform map panning
        * @return jQuery	
        * @param {string} direction Pass the direction in which map should be panned
        * @example 
        * //pan method for map
        * $("#container").ejMap({
        *    pan: function (direction) {}
        * });
        * @memberof ejMap
        * @instance
        */
        pan: function (direction) {
            var map = this;
            var Xdiff = 0;
            var Ydiff = 0;
            if (this.zoomLevel() != 1) {
                switch (direction) {

                    case 'right':
                        {
                            Xdiff = this._width / 7;
                            Ydiff = 0;
                            break;
                        }
                    case 'top':
                        {
                            Xdiff = 0;
                            Ydiff = -(this._height / 7);
                            break;
                        }n
                    case 'left':
                        {
                            Xdiff = -(this._width / 7);
                            Ydiff = 0;
                            break;
                        }
                    case 'bottom':
                        {
                            Xdiff = 0;
                            Ydiff = this._height / 7;
                            break;
                        }
                }
                if (map.model.enablePan) {
                    if (this._isTileMap) {
                        var curtileX = this._tileTranslatePoint.x - Xdiff / map._scale;
                        var curtileY = this._tileTranslatePoint.y - Ydiff / map._scale;
                        this._tileTranslatePoint.x = curtileX;
                        this._tileTranslatePoint.y = curtileY;
                        this._generateTiles(this.zoomLevel());
                    }                    
                    var curX = map._translatePoint.x - Xdiff / map._scale;
                    map._prevScale = map._scale;
                    var curY = map._translatePoint.y - Ydiff / map._scale;
                    map._prevPoint = { x: this._translatePoint.x, y: this._translatePoint.y };
                    this._translatePoint.x = curX;
                    this._translatePoint.y = curY;
                    if (map.model.enableAnimation && !this._isTileMap) {                       
                        map._animate(600);
                    }
                    else {
                        map._applyTransform(map._scale, map._translatePoint);
                    }
                    map._refrshLayers();
                }
            }
			
        },      

        /**     
        * Method to perform map zooming. 
        * @return jQuery
        * @param {number} level Pass the zoom level for map to be zoomed
        * @param {boolean} isAnimate Pass the boolean value to enable or disable animation while zooming
        * @example 
        * //zoom method for map
        * $("#container").ejMap({
        *    zoom: function (level,isAnimate) {}
        * })
        * @memberof ejMap
        * @instance
        */
        zoom: function (level,isAnimate) {
           var map = this;
            if (level <= this.maxZoom() && level >= this.minZoom()) {
                if (this._isTileMap) {
                    this._tileZoom(map.zoomLevel(), level, {x:this._width/2,y:this._height/2});
                    var prevLevel = map.zoomLevel();
                    map.zoomLevel(level);
                    this._generateTiles(this.zoomLevel());
                    map._translatePoint.x = (map._tileTranslatePoint.x - (0.5 * Math.pow(2, prevLevel))) / (Math.pow(2, prevLevel));
                    map._translatePoint.y = (map._tileTranslatePoint.y - (0.5 * Math.pow(2, prevLevel))) / (Math.pow(2, prevLevel));
                    map._scale = (Math.pow(2, prevLevel));					
                }
                else {                    
                    var fac = (map._baseScale + ((level - 1) * map.zoomFactor()));
                    map._prevPoint = { x: map._translatePoint.x, y: map._translatePoint.y };
                    map._prevScale = map._scale;
                    map._translatePoint.x -= (map._width / map._scale - map._width / fac) / 2;
                    map._translatePoint.y -= (map._height / map._scale - map._height / fac) / 2;
                    map._scale = fac;
                    map.zoomLevel(level);
                    if (map.model.enableAnimation && isAnimate) {
                        map._animate(600);
                    }                  
                }
				if(!map.model.enableAnimation || !isAnimate)
				{
				     map._applyTransform(map._scale, map._translatePoint);
                     map._refrshLayers();
                     map._resizeShape();       
				}
				 map._updateSliderValue();                  
                
            }
            else if (level <= this.minZoom()) {
                this.zoomLevel(this.minZoom());
                this.zoom(this.zoomLevel());
            }
            else if (level >= this.maxZoom()) {
                this.zoomLevel(this.maxZoom());
                this.zoom(this.zoomLevel());
            }
        },

        /**     
       * Method to reload the map.
       * @return jQuery
       * @example 
       * //refresh method for map
       * $("#container").ejMap({
       *    refresh: function () {}
       * });
       * @memberof ejMap
       * @instance
       */
        refresh: function () {

            this._unWireEvents();
            $(this._svgDocument).empty();
            if (this._svgDocument != null) {
                this._svgDocument = null;
            }
            $(this._mapContainer).empty();
            this._scale = 1;
            this._translatePoint = { x: 0, y: 0 };            
            this._height = this._mapContainer.height();
            this._width = this._mapContainer.width();
             if (this._height == 0) {
                 this._height = this._mapContainer[0].parentElement.clientHeight != 0 ? this._mapContainer[0].parentElement.clientHeight : $(document).height();
            }
            if (this._width == 0) {
                this._width = this._mapContainer[0].parentElement.clientWidth != 0 ? this._mapContainer[0].parentElement.clientWidth : $(document).width();
            }
            if (this.baseMapIndex() >= this.model.layers.length) {
                this.baseMapIndex(0);
            }
            this._generatePath();           
            if (this._groupSize != null) {
                this._containerHeight = this._groupSize.maxY - this._groupSize.minY;
                this._containerWidth = this._groupSize.maxX - this._groupSize.minX;
                this._groupBorder = { x: this._groupSize.minX, y: this._groupSize.minY };
            }
            this._resizingContainer();
            this._renderMapElements();
            if (this.model.enableLayerChangeAnimation) {
                $(this._mapContainer).animate({ opacity: 1 }, 500);
            }
            
        },
       

        /**
        * Updates slider value
        * @private
        */
        _updateSliderValue: function (isAnimate) {
            var slider = this._sliderControl;
            if (isAnimate == undefined) isAnimate = this.model.enableAnimation;
            if (slider != null) {
                var obj = slider.data("ejSlider");               
                obj.option("value", this.zoomLevel());
            }
        },

        /**
        * Creates div element
        * @private
        */
        _createDivElement: function (parent, child,base,classname) {
            var parentElement = parent;
            var childElement = child;           
            childElement.appendTo(parentElement);           
            parentElement.appendTo(base);            
            this._on(childElement,ej.eventType.mouseDown,{ className: classname, map: this }, this._navigationControlPressed);
            
        },

        /**
      * Section For handling mouse click event for navigation control
      * @private
      */
        _navigationControlPressed: function (event) {
            
            event.stopPropagation();
            this._isNavigationPressed = true;
            var map = event.data.map;
            var isAnimate = map.model.enableAnimation;            
            if (event.data.className == "zoomIn") {               
                map.zoom(map.zoomLevel() + 1, true);
                if (!isAnimate) {
                    map._updateSliderValue();
                }
            }
            else if (event.data.className == "zoomOut") {               
                map.zoom(map.zoomLevel() - 1, true);
                if (!isAnimate) {
                    map._updateSliderValue();
                }
            }
            else if (event.data.className == "panLeft") {
                map.pan("left");                
            }
            else if (event.data.className == "panRight") {
                map.pan("right");
            }
            else if (event.data.className == "panTop") {
                map.pan("top");
            }
            else if (event.data.className == "panBottom") {
                map.pan("bottom");
            }
            else if (event.data.className == "home") {                
                map.zoom(1,true);
                if (!isAnimate) {
                    map._updateSliderValue();
                }
            }            
           
        },
			
		 _keyboardKeysPressed: function (event) {
            this._iskeyboardKeysPressed = true;
            var map = event.data.map;
            var isAnimate = map.model.enableAnimation;            
            if (event.ctrlKey && event.keyCode==38) {    			
                map._zoomingIn(map._width / 2, map._height / 2, event, isAnimate);
            }
            else if (event.ctrlKey && event.keyCode==40) {    
                map._zoomingOut(map._width / 2, map._height / 2, event, isAnimate);
            }
            else if (event.keyCode==37) {
                map.pan("left");                
            }
            else if (event.keyCode==39) {
                map.pan("right");
            }
            else if (event.keyCode==38) {
                map.pan("top");
            }
            else if (event.keyCode==40) {
                map.pan("bottom");
            }
		},
		
		
		
        /**     
          * Method to reload the navigation control with updated values.
          * @return jQuery	
          * @param {object} navigation Pass the navigation control instance
          * @example 
          * //Refresh navigation control method for map
          * $("#container").ejMap({
          *    refreshNavigationControl: function (navigation) {}
          * });
          * @memberof ejMap
          * @instance
          */
        refreshNavigationControl: function (navigation) {
            
            var prevNav = this._mapContainer.find("#ejNavigation");
            if (prevNav.length > 0) {
                this._mapContainer[0].removeChild(prevNav[0]);
            }
            if (this.model.navigationControl != null && this.model.navigationControl.enableNavigation) {
                if (navigation == undefined) {
                    navigation = this.model.navigationControl;
                }
                var navigationOrientation = ej.Orientation.Vertical;
                var controlSize;
                var navigationHeight = 150;
                var navigationWidth = 12;
                controlSize = { width: 90, height: 350 };
                var navigationControl = $("<div id='ejNavigation' class='e-map-navigation e-orientation-vert'/>");
                var sliderDiv = $("<div style='height:150px;width:10px;margin-top:-197px;margin-left: 34px;' />");
                if (navigation.orientation == 'horizontal') {
                    navigationOrientation = ej.Orientation.Horizontal;
                    navigationHeight = 12;
                    navigationWidth = 150;
                    controlSize = { width: 350, height: 90 };
                    navigationControl = $("<div id='ejNavigation' class='e-map-navigation e-orientation-horz' />");
                    sliderDiv = $("<div style='height:10px;width:150px;margin-top:-18px;margin-left: 137px;' />");
                }
                var baseDiv = $("<div class='e-panContainer'/>");
                var isHor = navigationOrientation == 'horizontal' ? 'Horz' : 'Vert';
                var zoominOutDiv = $("<div/>");
                var slidercontrol = $("<div  />");
                baseDiv.appendTo(navigationControl);
                zoominOutDiv.appendTo(navigationControl);
                sliderDiv.appendTo(slidercontrol);
                slidercontrol.appendTo(navigationControl);
                this._sliderControl = sliderDiv;
                var navPos = { x: 0, y: 0 };
                if (navigation.dockPosition == null || navigation.dockPosition == ej.datavisualization.Map.DockPosition.None) {
                    navPos.x = (this._width * navigation.absolutePosition.x) / 100;
                    navPos.y = (this._height * navigation.absolutePosition.y) / 100;
                } else {
                    navPos = this._getDockPosition(navigation.dockPosition, controlSize);
                }
                navigationControl.css({ "margin-left": navPos.x + "px", "margin-top": navPos.y + "px" });
                var map = this;
				var changeZoom = this._height * 0.0025;
				if (this._browser != 'mozilla'){
				navigationStyle = $('<style> .e-map-navigation {width: 90px;height: 350px;position:absolute;z-index:2;zoom:'+changeZoom+';}</style>');
				}
				else{
				navigationStyle = $('<style> .e-map-navigation {width: 90px;height: 350px;position:absolute;z-index:2;-moz-transform: scale('+changeZoom+');}</style>');
				}
				if (this._browser!='mozilla' && this._browser!='chrome'){
				navigationStyle.remove();
				$('html > head').append(navigationStyle);
				}
				else{
				$('.e-map-navigation').removeAttr(navigationStyle);
				$('html > head').append(navigationStyle);
				}
                $(sliderDiv).ejSlider({
                    orientation: navigationOrientation,
                    sliderType: ej.SliderType.MinRange,
                    value: 1,
                    animationSpeed: 1200,
                    minValue: this.minZoom(),
                    showTooltip: true,
                    enableAnimation: false,
                    maxValue: this.maxZoom(),
                    incrementStep: this.zoomFactor(),
                    slide: onslide,
                    change: onchange,
                    height: navigationHeight,
                    width: navigationWidth
                });
                navigationControl.appendTo(this._mapContainer);
                this._createDivElement($("<div class='e-icon1 e-incrementButton  icon_margin1' />"), $("<div class='e-icon1 nav-inc-" + isHor + "  e-innerIncrement'/>"), zoominOutDiv, "zoomIn");
                this._createDivElement($("<div class='e-icon1 e-incrementButton icon_margin2'/>"), $("<div class='e-icon1 nav-dec-" + isHor + " e-innerDecrement'/>"), zoominOutDiv, "zoomOut");
                this._createDivElement($("<div class='e-icon1 e-radialTop'/>"), $("<div class='e-icon1 e-arrowUp'/>"), baseDiv, "panTop");
                this._createDivElement($("<div class='e-icon1 e-radialLeft'/>"), $("<div class='e-icon1 e-arrowLeft'/>"), baseDiv, "panLeft");
                this._createDivElement($("<div class='e-icon1 e-radialRight'/>"), $("<div class='e-icon1 e-arrowRight'/>"), baseDiv, "panRight");
                this._createDivElement($("<div class='e-icon1 e-radialBottom'/>"), $("<div class='e-icon1 e-arrowDown'/>"), baseDiv, "panBottom");
                var homeDiv = $("<div class='e-icon1 e-home-bg'><div class='e-icon1 e-map-home'></div>");
                homeDiv.appendTo(baseDiv);
                homeDiv.mousedown({ className: "home", map: this }, this._navigationControlPressed);               
            }
			 function onchange(args) {
                    if (map != null && map._isRendered && map.zoomLevel() != args.value) {
                        map.zoom(args.value, false);
                    }

                }

                function onslide(args) {
                    if (map != null && map._isRendered && map.zoomLevel() != args.value) {
                        map.zoom(args.value, false);
                    }
                }
        },

        /**
       * Gets dock position for legend and navigation control
       * @private
       */
        _getDockPosition: function (position,size) {
            var dockPosition = { x: 0, y: 0 };
            switch (position) {
                case ej.datavisualization.Map.DockPosition.TopLeft:                   
                    break;
                case ej.datavisualization.Map.DockPosition.TopCenter:
                    dockPosition.x = (this._width / 2) - (size.width / 2);
                    break;
                case ej.datavisualization.Map.DockPosition.TopRight:
                    dockPosition.x = this._width - size.width;
                    break;
                case ej.datavisualization.Map.DockPosition.CenterLeft:
                    dockPosition.y = (this._height / 2) - (size.height / 2);
                    break;
                case ej.datavisualization.Map.DockPosition.Center:
                    dockPosition.x = (this._width / 2) - (size.width / 2);
                    dockPosition.y = (this._height / 2) - (size.height / 2);
                    break;
                case ej.datavisualization.Map.DockPosition.CenterRight:
                    dockPosition.x = this._width - size.width;
                    dockPosition.y = (this._height / 2) - (size.height / 2);
                    break;
                case ej.datavisualization.Map.DockPosition.BottomLeft:
                    dockPosition.y = this._height - size.height;
                    break;
                case ej.datavisualization.Map.DockPosition.BottomCenter:
                    dockPosition.x = (this._width / 2) - (size.width / 2);
                    dockPosition.y = this._height - size.height;
                    break;
                case ej.datavisualization.Map.DockPosition.BottomRight:
                    dockPosition.x = this._width - size.width;
                    dockPosition.y = this._height - size.height;
                    break;

            }
            return dockPosition;
        },
        
        /**
       * Renders map elements
       * @private
       */
        _renderMapElements: function () {           
            this._templateDiv = $("<div class='TemplateDiv'/>");
            this._templateDiv.appendTo(this._mapContainer);
            this._templateDiv.css({
                'pointer-events': 'none', 'overflow': 'hidden', "position":"absolute",
                'z-index': '1', 'height': this._height, 'width': this._width
            });			
            this._wireEvents();
            this.refreshLayers();       
            this.refreshNavigationControl(this.model.navigationControl);
        },
		
		_renderMapLayers:function(){
		
		 $(this._templateDiv).empty();
            var prevPathCount = 0;
            var baseLayer = this.model.layers[this.baseMapIndex()];
            if (baseLayer.layerType == ej.datavisualization.Map.LayerType.Geometry) {
                this._renderLayers(baseLayer, prevPathCount);
                if (baseLayer.shapeData != null)
                    prevPathCount = baseLayer._polygonData.length;
            }
            else {
                baseLayer._initializeLocalValues();
                var contribution = $("<div class='map-contribution'>");
                contribution[0].innerHTML = baseLayer.contribution;
                this._mapContainer.append(contribution);
            }
            for (var key = 0; key < this.model.layers[this.baseMapIndex()].subLayers.length; key++) {
                var sublayer = this.model.layers[this.baseMapIndex()].subLayers[key];
                sublayer._isBaseLayer = false;
                if (sublayer.layerType == ej.datavisualization.Map.LayerType.Geometry && sublayer.shapeData!=null) {
                    this._renderLayers(sublayer, prevPathCount);
                    prevPathCount += sublayer._polygonData.length;
                }
                else {
                    sublayer._initializeLocalValues();
                }
            }
            this.refreshMarkers();
            if (_isSVG && this.model.enableAnimation && !this._isTileMap) {
                this._refreshWithAnimation();
            }
            else {
                this._refrshLayers();
            }

		},


        /**     
          * Method to reload the shapeLayers with updated values
          * @return jQuery	
          * @example 
          * //refresh layers method for map
          * $("#container").ejMap({
          *    refreshLayers: function () {}
          * });
          * @memberof ejMap
          * @instance
          */
        refreshLayers: function () {          
		   var baseLayer = this.model.layers[this.baseMapIndex()];
		   this._processOData(baseLayer,this);          
		   
        },
		
		_processOData:function(layer,map){		    
		    if(layer.dataSource!=null){
                      if (layer.dataSource instanceof ej.DataManager) {
				          var queryPromise = layer.dataSource.executeQuery(layer.query);
			             queryPromise.done(function (e) {
			             if(e.result!=null)
				         {
                            layer.dataSource=e.result;
						    map._renderMapLayers();
				         }
			           });        
				  }
				 else
				 {
				    map._renderMapLayers();
				 }
            }
			else
			{
			  map._renderMapLayers();
			}
		}
		
		

    });

 

    var Tile = function (x, y) {
        this.X = x;
        this.Y = y;
        this.left = 0;
        this.top = 0;       
        this.height = 256;
        this.width = 256;
        this.src = null;
    };
   
    /** 
    * Layer for holding the map shapes
    * @memberof ejMap
    * @member
    * @instance
    */
    var shapeLayer = function () {
        
        
            /** 
            * Enables or disables the shape selection
             * @default true
             * @type {boolean} 
             * @example  
             * // Set the enableSelection during initialization. 			
             * 	$("#container").ejMap({layers: [{ enableSelection:true }]})
             * @example 
             * //Get or set the enableSelection after initialization:<br>
             *   //Gets the enableSelection from map.
             *   var property =$("#container").data("ejMap").model.layers[layerIndex].enableSelection;
             *   //Sets the enableSelection to map.
             *   $("#container").data("ejMap").model.layers[layerIndex].enableSelection  = true;
             * @alias ejMap#shapeLayer->enableSelection
             * @instance
             */
        this.enableSelection = true;

        /** 
            * Specifies the selection mode of the map. Accepted selection mode values are Default and Multiple.
            * @default "default"
			* @type {string}
             * @example  
             * // Set the selection mode during initialization. 			
             * 	$("#container").ejMap({layers: [{ selectionMode:'default' }]})
             * @example 
             * //Get or set the selection mode after initialization:<br>
             *   //Gets the selection mode from map.
             *   var property =$("#container").data("ejMap").model.layers[layerIndex].selectionMode;
             *   //Sets the selection mode to map.
             *   $("#container").data("ejMap").model.layers[layerIndex].selectionMode  = 'default';
             * @alias ejMap#shapeLayer->selectionMode
             * @instance
             */
        this.selectionMode = "default";

        this.key = null;

        this.bingMapType = "aerial";

        /**
        * Enables or disables the shape mouse hover
            * @default false
            * @type {boolean} 
            * @example  
            * // Set the enableMouseHover during initialization. 			
            * 	$("#container").ejMap({layers: [{ enableMouseHover:false }]})
            * @example 
            * //Get or set the enableMouseHover after initialization:<br>
            *   //Gets the enableMouseHover from map.
            *   var property =$("#container").data("ejMap").model.layers[layerIndex].enableMouseHover;
            *   //Sets the enableMouseHover to map.
            *   $("#container").data("ejMap").model.layers[layerIndex].enableMouseHover  = false;
            * @alias ejMap#shapeLayer->enableMouseHover
            * @instance
            */
        this.enableMouseHover = false;

        /** 
        * Specifies the shape data for the shape  layer
        * @type {object}
        * @example
        * // Set the shapeData of layer during initialization. 			
        * 	$("#container").ejMap({layers: [{ layerType: "geometry", shapeData: mapShapeData }]})
        * @example 
        * //Get or set the data after initialization:<br>
        *   //Gets the data from map layer.
        *   var data =$("#container").data("ejMap").model.layers[layerIndex].shapeData;
        *   //Sets the data to map layer.
        *   $("#container").data("ejMap").model.layers[layerIndex].shapeData  = mapShapeData;
        * @alias ejMap#shapeLayer->shapeData
        * @instance
        */
        this.shapeData = null;

        /**		
            * Specify markers for shape  layer.
            * @default []
            * @type {marker} 
            * @example  
            * // Set the markers during initialization. 			
            * 	$("#container").ejMap({layers: [{markers:[{label : "chennai",latitude : 13.08 ,longitude : 80.27}]}]})
            * @example 
            * //Get or set the markers after initialization:<br>
            *   //Gets the markers from map.
            *   var marker =$("#container").data("ejMap").model.layers[layerIndex].markers[markerIndex];
            *   //Sets the marker to map.
            *   $("#container").data("ejMap").model.layers[layerIndex].markers[markerIndex]  = {label : "chennai",latitude : 13.08 ,longitude : 80.27};
            * @alias ejMap#shapeLayer->markers
            * @instance
            */
        this.markers = [];

        /** 
        * Specifies the datasource for the shape  layer
        * @type {object}
        * @example
        * // Set the dataSource of layer during initialization. 			
        * 	$("#container").ejMap({layers: [{ layerType: "geometry", dataSource: source,  shapeData: mapShapeData }]})
        * @example 
        * //Get or set the dataSource after initialization:<br>
        *   //Gets the dataSource from map layer.
        *   var dataSource =$("#container").data("ejMap").model.layers[layerIndex].dataSource;
        *   //Sets the dataSource to map layer.
        *   $("#container").data("ejMap").model.layers[layerIndex].dataSource  = source;
        * @alias ejMap#shapeLayer->dataSource
        * @instance
        */
        this.dataSource = null;

        /** 
        * Specifies the url template for the OSM type map.
            * @default 'http://a.tile.openstreetmap.org/level/tileX/tileY.png'
            * @type {string} 
            * @example  
            * // Set the urlTemplate during initialization. 			
            * 	$("#container").ejMap({layers: [{ urlTemplate:'http://a.tile.openstreetmap.org/level/tileX/tileY.png' }]})
            * @example 
            * //Get or set the urlTemplate after initialization:<br>
            *   //Gets the urlTemplate from map.
            *   var property =$("#container").data("ejMap").model.layers[layerIndex].urlTemplate;
            *   //Sets the urlTemplate to map.
            *   $("#container").data("ejMap").model.layers[layerIndex].urlTemplate  = 'http://a.tile.openstreetmap.org/level/tileX/tileY.png';
            * @alias ejMap#shapeLayer->urlTemplate
            * @instance
            */
        this.urlTemplate = 'http://a.tile.openstreetmap.org/level/tileX/tileY.png';

        /** 
        * Shows or hides the tooltip for shapes
            * @default false
            * @type {boolean} 
            * @example  
            * // Set the showTooltip during initialization. 			
            * 	$("#container").ejMap({layers: [{ showTooltip:false }]})
            * @example 
            * //Get or set the showTooltip after initialization:<br>
            *   //Gets the showTooltip from map.
            *   var property =$("#container").data("ejMap").model.layers[layerIndex].showTooltip;
            *   //Sets the showTooltip to map.
            *   $("#container").data("ejMap").model.layers[layerIndex].showTooltip  = false;
            * @alias ejMap#shapeLayer->showTooltip
            * @instance
            */
        this.showTooltip = false;

        /** 
        * Specifies the tooltip template for shapes.
        * @type {string}
        * @example
        * // Set the tooltipTemplate of layer during initialization. 			
        * 	$("#container").ejMap({layers: [{ layerType: "geometry", tooltipTemplate: "Template",  shapeData: mapShapeData }]})
        * @example 
        * //Get or set the tooltipTemplate after initialization:<br>
        *   //Gets the tooltipTemplate from map.
        *   var tooltipTemplate =$("#container").data("ejMap").model.layers[layerIndex].tooltipTemplate;
        *   //Sets the tooltipTemplate to map.
        *   $("#container").data("ejMap").model.layers[layerIndex].tooltipTemplate  ="Template";
        * @alias ejMap#shapeLayer->tooltipTemplate
        * @instance
        */
        this.tooltipTemplate = null;

        /**
        * Specifies the map items template for shapes.
        * @type {string}
        * @example
        * // Set the mapItemsTemplate of layer during initialization. 			
        * 	$("#container").ejMap({layers: [{ layerType: "geometry", mapItemsTemplate: "Template",  shapeData: mapShapeData }]})
        * @example 
        * //Get or set the mapItemsTemplate after initialization:<br>
        *   //Gets the mapItemsTemplate from map.
        *   var mapItemsTemplate =$("#container").data("ejMap").model.layers[layerIndex].mapItemsTemplate;
        *   //Sets the mapItemsTemplate to map.
        *   $("#container").data("ejMap").model.layers[layerIndex].mapItemsTemplate  = "Template";
        * @alias ejMap#shapeLayer->mapItemsTemplate
        * @instance
        */
        this.mapItemsTemplate = null;

        /** 
        * Enables or disables the animation
            * @default false
            * @type {boolean} 
            * @example  
            * // Set the enableAnimation during initialization. 			
            * 	$("#container").ejMap({layers: [{ enableAnimation:false }]})
            * @example 
            * //Get or set the enableAnimation after initialization:<br>
            *   //Gets the enableAnimation from map.
            *   var property =$("#container").data("ejMap").model.layers[layerIndex].enableAnimation;
            *   //Sets the enableAnimation to map.
            *   $("#container").data("ejMap").model.layers[layerIndex].enableAnimation  = false;
            * @alias ejMap#shapeLayer->enableAnimation
            * @instance
            */
        this.enableAnimation = false;

        /** 
        * Specifies the legend setting of map layer
        * @default null
        * @type {object}
        * @example
        * // Set the legendSettings of layer during initialization. 			
        * 	$("#container").ejMap({layers: [{ layerType: "geometry", legendSettings: {showLegend:true, showLabels:true, position: "bottomleft", PositionX: 3, PositionY:80, height:18, width:190, type:"Layers", mode:"interactive", title: "Number of Students", leftLabel:"0 M ", rightLabel:"1.4 M "},  shapeData: mapShapeData }]})
        * @example 
        * //Get or set the legendSettings after initialization:<br>
        *   //Gets the legendSettings from map.
        *   var legendSettings =$("#container").data("ejMap").model.layers[layerIndex].legendSettings;
        *   //Sets the legendSettings to map.
        *   $("#container").data("ejMap").model.layers[layerIndex].legendSettings  = {showLegend:true, showLabels:true, position: "bottomleft", PositionX: 3, PositionY:80, height:18, width:190, type:"Layers", mode:"interactive", title: "Number of Students", leftLabel:"0 M ", rightLabel:"1.4 M "};
        * @alias ejMap#shapeLayer->legendSettings
        * @instance
        */
        this.legendSettings = null;

        /** 
        * Specifies the label setting of map layer
        * @default null
        * @type {object}
        * @example
        * // Set the labelSettings of layer during initialization. 			
        * 	$("#container").ejMap({layers: [{ layerType: "geometry", labelSettings: { showLabels: true, labelPath: 'iso_3166_2',enableSmartLabel: true },  shapeData: mapShapeData }]})
        * @example 
        * //Get or set the labelSettings after initialization:<br>
        *   //Gets the labelSettings from map.
        *   var labelSettings =$("#container").data("ejMap").model.layers[layerIndex].labelSettings;
        *   //Sets the labelSettings to map.
        *   $("#container").data("ejMap").model.layers[layerIndex].labelSettings  =  { showLabels: true, labelPath: 'iso_3166_2',enableSmartLabel: true };
        * @alias ejMap#shapeLayer->labelSettings
        * @instance
        */
        this.labelSettings = null,

        /** 
        * Specifies the map marker template for map layer.
        * @default null
        * @type {string}
        * @example
        * // Set the markerTemplate of layer during initialization. 			
        * 	$("#container").ejMap({layers: [{ layerType: "geometry", markerTemplate: "Template",  shapeData: mapShapeData }]})
        * @example 
        * //Get or set the markerTemplate after initialization:<br>
        *   //Gets the markerTemplate from map.
        *   var markerTemplate =$("#container").data("ejMap").model.layers[layerIndex].markerTemplate;
        *   //Sets the markerTemplate to map.
        *   $("#container").data("ejMap").model.layers[layerIndex].markerTemplate  = "Template";
        * @alias ejMap#shapeLayer->markerTemplate
        * @instance
        */
        this.markerTemplate = null;       

        /** 
        * Shows or hides the map items.
            * @default false
            * @type {boolean} 
            * @example  
            * // Set the showMapItems during initialization. 			
            * 	$("#container").ejMap({layers: [{ showMapItems:false }]})
            * @example 
            * //Get or set the showMapItems after initialization:<br>
            *   //Gets the showMapItems from map.
            *   var property =$("#container").data("ejMap").model.layers[layerIndex].showMapItems;
            *   //Sets the showMapItems to map.
            *   $("#container").data("ejMap").model.layers[layerIndex].showMapItems  = false;
            * @alias ejMap#shapeLayer->showMapItems
            * @instance
            */
		this.showMapItems = false;

        /** 
        * Specifies the map type.
            * @default 'geometry'
            * @type {string} 
            * @example  
            * // Set the layerType during initialization. 			
            * 	$("#container").ejMap({layers: [{ layerType:'geometry' }]})
            * @example 
            * //Get or set the layerType after initialization:<br>
            *   //Gets the layerType from map.
            *   var property =$("#container").data("ejMap").model.layers[layerIndex].layerType;
            *   //Sets the layerType to map.
            *   $("#container").data("ejMap").model.layers[layerIndex].layerType  = 'geometry';
            * @alias ejMap#shapeLayer->layerType
            * @instance
            */
        this.layerType = 'geometry';
        /**
         * Specifies inbuilt color palette		
		 * @private
         */
        this._colorPaletteSettings = {
            'Palette1':
            {
                'highlightColor': '#F7CD1C',
                'highlightStroke': 'white',
                'highlightBorderWidth': '1',
                'SelectionColor': '#F96C0D',
                'SelectionStroke': 'white',
                'SelectionStrokeWidth': '2.5'
            },
            'Palette2':
            {
                'highlightColor': '#68A3EA',
                'highlightStroke': 'White',
                'highlightBorderWidth': '1',
                'SelectionColor': '#116EF4',
                'SelectionStroke': 'White',
                'SelectionStrokeWidth': '2.5'
            },
            'Palette3':
            {
                'highlightColor': '#CCCCCC',
                'highlightStroke': 'white',
                'highlightBorderWidth': '1',
                'SelectionColor': '#4D4D4D',
                'SelectionStroke': 'white',
                'SelectionStrokeWidth': '2.5'
            }
        };
        this.colorPalettes = {
            'Palette1': ['#4A3825', '#736F3D','#F2DABD','#BF9D7E','#7F6039','#7F715F','#70845D','#CC995C','#736F3D','#89541B'],
            'Palette2': ['#E51400', '#730202','#EF6535','#C63477','#BF004D','#F0A30B','#CE1B1B','#97993D','#D6BF38','#835A2C'],
            'Palette3': ['#A4C400', '#008B00','#1BA0E2','#0050EF','#AA00FF','#D90073','#E51400','#F96800','#E3C800','#A20026']
        };
        /**
         * Specifies previous palette index		
		 * @private
         */
        this._prevPaletteIndex = 0;
        /**
         * Specifies new map bounds	
		 * @private
         */
        this._newBounds = [];
        /** 
        * Specifies the sub shape  layers
            * @default []
            * @type {shapeLayer} 
            * @example  
            * // Set the subLayers during initialization. 			
            * 	$("#container").ejMap({layers: [{ layerType: "geometry", enableMouseHover: true, shapeSettings: { stroke: "black", fill: "#C3E6ED", highlightColor: "#63B7B7", selectionColor: "#207BB2", strokeThickness: "0.5" },subLayers: [{shapeDataPath: "pathName", shapePropertyPath: "tableFieldName", mapItemsTemplate: 'Template', showMapItems: true, enableMouseHover: true, dataSource: mapDataSource, shapeSettings: { fill: "#9FD0D3", strokeThickness: "0.2", stroke: "white", highlightColor: "#63B7B7", },  shapeData: mapShapeData }, shapeData: mapShapeData }]})
            * @example 
            * //Get or set the layer after initialization:<br>
            *   //Gets the layer from map.
            *   var layer =$("#container").data("ejMap").model.layers[layerIndex];
            *   //Sets the layer to map.
            *   $("#container").data("ejMap").model.layers[layerIndex]  = { layerType: "geometry", enableMouseHover: true, shapeSettings: { stroke: "black", fill: "#C3E6ED", highlightColor: "#63B7B7", selectionColor: "#207BB2", strokeThickness: "0.5" },subLayers: [{shapeDataPath: "pathName", shapePropertyPath: "tableFieldName", mapItemsTemplate: 'Template', showMapItems: true, enableMouseHover: true, dataSource: mapDataSource, shapeSettings: { fill: "#9FD0D3", strokeThickness: "0.2", stroke: "white", highlightColor: "#63B7B7", },  shapeData: mapSubShapeData }, shapeData: mapShapeData };
            * @alias ejMap#shapeLayer->subLayers
            * @instance
            */
        this.subLayers = [];
        
       
        /** 
        * Specifies the shape settings of map layer
        * @type {object}
        * @example
        * // Set the shapeSettings of layer during initialization. 			
        * 	$("#container").ejMap({layers: [{ layerType: "geometry", enableMouseHover: true, shapeSettings: { stroke: "black", fill: "#C3E6ED", highlightColor: "#63B7B7", selectionColor: "#207BB2", strokeThickness: "0.5" }, shapeData: Africa }]})
        * @example 
        * //Get or set the shapeSettings after initialization:<br>
        *   //Gets the shapeSettings from map.
        *   var shapeSettings =$("#container").data("ejMap").model.layers[layerIndex].shapeSettings;
        *   //Sets the shapeSettings to map.
        *   $("#container").data("ejMap").model.layers[layerIndex]  = { layerType: "geometry", enableMouseHover: true, shapeSettings: { stroke: "black", fill: "#C3E6ED", highlightColor: "#63B7B7", selectionColor: "#207BB2", strokeThickness: "0.5" }, shapeData: Africa };
        * @alias ejMap#shapeLayer->shapeSettings
        * @instance
        */
        this.shapeSettings = {
            /**		
			* Specifies the mouse hover color of the shape layer in map
            * @default "gray"
			* @type {string}
			* @example 
            * //To set highlightColor API value during initialization 
			*   $("#container").ejMap({layers:{shapeSettings: {highlightColor:'gray'}}});
            * @example 
			* //Get or set the highlightColor API, after initialization:<br>
			*	//Gets the highlightColor value 
            *   var shapeProperty =$("#container").data("ejMap").model.layers[layerIndex].shapeSettings.highlightColor;<br> 	
            *	//Sets the highlightColor value 
			*	 $("#container").data("ejMap").model.layers[0].shapeSettings.highlightColor='gray'; 
            * @alias ejMap#shapeLayer->shapeSettings->highlightColor
			* @instance
            */
            highlightColor: "gray",

            /**		
			* Specifies the mouse over width of the shape  layer in map
            * @default 1
			* @type {number}
			* @example 
            * //To set highlightBorderWidth API value during initialization 
			*   $("#container").ejMap({layers: {shapeSettings: {highlightBorderWidth:1}}});
            * @example 
			* //Get or set the highlightBorderWidth API, after initialization:<br>
			*	//Gets the highlightBorderWidth value 
            *   var shapeProperty =$("#container").data("ejMap").model.layers[layerIndex].shapeSettings.highlightBorderWidth;<br> 	
            *	//Sets the highlightBorderWidth value 
			*	 $("#container").data("ejMap").model.layers[0].shapeSettings.highlightBorderWidth=1; 
			* @alias ejMap#shapeLayer->shapeSettings->highlightBorderWidth
			* @instance
            */
            highlightBorderWidth: 1,

            /**		
			* Specifies the shape selection color of the shape layer in map
            * @default "gray"
			* @type {string}
			* @example 
            * //To set selectionColor API value during initialization 
			*   $("#container").ejMap({layers: {shapeSettings: {selectionColor:'gray'}}});
            * @example 
			* //Get or set the selectionColor API, after initialization:<br>
			*	//Gets the selectionColor value 
            *   var shapeProperty =$("#container").data("ejMap").model.layers[layerIndex].shapeSettings.selectionColor;<br> 	
            *	//Sets the selectionColor value 
			*	 $("#container").data("ejMap").model.layers[0].shapeSettings.selectionColor='gray'; 
			* @alias ejMap#shapeLayer->shapeSettings->selectionColor
			* @instance
            */
            selectionColor: "gray",

            /**		
			* Specifies the shape fill color of the shape layer in map
            * @default "#E5E5E5"
			* @type {string}
			* @example 
            * //To set fill API value during initialization 
			*   $("#container").ejMap({layers: {shapeSettings: {fill:'#E5E5E5'}}});
            * @example 
			* //Get or set the fill API, after initialization:<br>
			*	//Gets the fill value 
            *   var shapeProperty =$("#container").data("ejMap").model.layers[layerIndex].shapeSettings.fill;<br> 	
            *	//Sets the fill value 
			*	 $("#container").data("ejMap").model.layers[0].shapeSettings.fill='#E5E5E5'; 
			* @alias ejMap#shapeLayer->shapeSettings->fill
			* @instance
            */
            fill: "#E5E5E5",

            /**		
			* Specifies the shape stroke thickness value of the shape layer in map
            * @default "0.2"
			* @type {string}
			* @example 
            * //To set strokeThickness API value during initialization 
			*   $("#container").ejMap({layers: {shapeSettings: {strokeThickness:'0.2'}}});
            * @example 
			* //Get or set the strokeThickness API, after initialization:<br>
			*	//Gets the strokeThickness value 
            *   var shapeProperty =$("#container").data("ejMap").model.layers[layerIndex].shapeSettings.strokeThickness;<br> 	
            *	//Sets the strokeThickness value 
			*	 $("#container").data("ejMap").model.layers[0].shapeSettings.strokeThickness='0.2'; 
			* @alias ejMap#shapeLayer->shapeSettings->strokeThickness
			* @instance
            */
            strokeThickness: "0.2",

            /**		
			* Specifies the shape stroke color of the shape layer in map
            * @default "#C1C1C1"
			* @type {string}
			* @example 
            * //To set stroke API value during initialization 
			*   $("#container").ejMap({layers: {shapeSettings: {stroke:'#C1C1C1'}}});
            * @example 
			* //Get or set the stroke API, after initialization:<br>
			*	//Gets the stroke value 
            *   var shapeProperty =$("#container").data("ejMap").model.layers[layerIndex].shapeSettings.stroke;<br> 	
            *	//Sets the stroke value 
			*	 $("#container").data("ejMap").model.layers[0].shapeSettings.stroke='#C1C1C1'; 
			* @alias ejMap#shapeLayer->shapeSettings->stroke
			* @instance
            */
            stroke: "#C1C1C1",

            /**		
			* Specifies the shape color palette value of the shape  layer in map. Accepted colorPalette values are palette1, palette2, palette3 and custompalette.
            * @default "palette1"
			* @type {string}
			* @example 
            * //To set colorPalette API value during initialization 
			*   $("#container").ejMap({layers: {shapeSettings: {colorPalette:'palette1'}}});
            * @example 
			* //Get or set the colorPalette API, after initialization:<br>
			*	//Gets the colorPalette value 
            *   var shapeProperty =$("#container").data("ejMap").model.layers[layerIndex].shapeSettings.colorPalette;<br> 	
            *	//Sets the colorPalette value 
			*	 $("#container").data("ejMap").model.layers[0].shapeSettings.colorPalette='palette1'; 
			* @alias ejMap#shapeLayer->shapeSettings->colorPalette
			* @instance
            */
            colorPalette: 'palette1',

            /**		
			* Specifies the mouse over stroke color of the shape  layer in map
            * @default "#C1C1C1"
			* @type {string}
			* @example 
            * //To set highlightStroke API value during initialization 
			*   $("#container").ejMap({layers: {shapeSettings: {highlightStroke:'#C1C1C1'}}});
            * @example 
			* //Get or set the highlightStroke API, after initialization:<br>
			*	//Gets the highlightStroke value 
            *   var shapeProperty =$("#container").data("ejMap").model.layers[layerIndex].shapeSettings.highlightStroke;<br> 	
            *	//Sets the highlightStroke value 
			*	 $("#container").data("ejMap").model.layers[0].shapeSettings.highlightStroke='#C1C1C1'; 
			* @alias ejMap#shapeLayer->shapeSettings->highlightStroke
			* @instance
            */
            highlightStroke: "#C1C1C1",

            /**		
			* Specifies the shape selection stroke color of the shape  layer in map
            * @default "#C1C1C1"
			* @type {string}
			* @example 
            * //To set selectionStroke API value during initialization 
			*   $("#container").ejMap({layers: {shapeSettings: {selectionStroke:'#C1C1C1'}}});
            * @example 
			* //Get or set the selectionStroke API, after initialization:<br>
			*	//Gets the selectionStroke value 
            *   var shapeProperty =$("#container").data("ejMap").model.layers[layerIndex].shapeSettings.selectionStroke;<br> 	
            *	//Sets the selectionStroke value 
			*	 $("#container").data("ejMap").model.layers[0].shapeSettings.selectionStroke='#C1C1C1'; 
			* @alias ejMap#shapeLayer->shapeSettings->selectionStroke
			* @instance
            */
            selectionStroke: "#C1C1C1",

            /**		
			* Specifies the shape selection stroke width of the shape  layer in map
            * @default 1
			* @type {number}
			* @example 
            * //To set selectionStrokeWidth API value during initialization 
			*   $("#container").ejMap({layers: {shapeSettings: {selectionStrokeWidth:1}}});
            * @example 
			* //Get or set the selectionStrokeWidth API, after initialization:<br>
			*	//Gets the selectionStrokeWidth value 
            *   var shapeProperty =$("#container").data("ejMap").model.layers[layerIndex].shapeSettings.selectionStrokeWidth;<br> 	
            *	//Sets the selectionStrokeWidth value 
			*	 $("#container").data("ejMap").model.layers[0].shapeSettings.selectionStrokeWidth=1; 
			* @alias ejMap#shapeLayer->shapeSettings->selectionStrokeWidth
			* @instance
            */
            selectionStrokeWidth: 1,

            /**		
			* Specifies the shape color valuePath of the shape  layer in map
            * @default null
			* @type {string}
			* @example 
            * //To set colorValuePath  API value during initialization 
			*   $("#container").ejMap({layers: {shapeSettings: {colorValuePath :'sales'}}});
            * @example 
			* //Get or set the colorValuePath  API, after initialization:<br>
			*	//Gets the colorValuePath  value 
            *   var shapeProperty =$("#container").data("ejMap").model.layers[layerIndex].shapeSettings.colorValuePath ;<br> 	
            *	//Sets the colorValuePath  value 
			*	 $("#container").data("ejMap").model.layers[0].shapeSettings.colorValuePath ='sales'; 
			* @alias ejMap#shapeLayer->shapeSettings->colorValuePath 
			* @instance
            */
            colorValuePath : null,


            /**		
			* Specifies the shape valuePath of the shape  layer in map
            * @default null
			* @type {string}
			* @example 
            * //To set valuePath API value during initialization 
			*   $("#container").ejMap({layers: {shapeSettings: {valuePath:'name'}}});
            * @example 
			* //Get or set the valuePath API, after initialization:<br>
			*	//Gets the valuePath value 
            *   var shapeProperty =$("#container").data("ejMap").model.layers[layerIndex].shapeSettings.valuePath;<br> 	
            *	//Sets the valuePath value 
			*	 $("#container").data("ejMap").model.layers[0].shapeSettings.valuePath='name'; 
			* @alias ejMap#shapeLayer->shapeSettings->valuePath
			* @instance
            */
            valuePath: null,

            /**		
			* Enables or Disables the gradient colors for map shapes.
            * @default false
			* @type {boolean}
			* @example 
            * //To set enableGradient API value during initialization 
			*   $("#container").ejMap({layers: {shapeSettings: {enableGradient:false}}});
            * @example 
			* //Get or set the enableGradient API, after initialization:<br>
			*	//Gets the enableGradient value 
            *   var shapeProperty =$("#container").data("ejMap").model.layers[layerIndex].shapeSettings.enableGradient;<br> 	
            *	//Sets the enableGradient value 
			*	 $("#container").data("ejMap").model.layers[0].shapeSettings.enableGradient=false; 
			* @alias ejMap#shapeLayer->shapeSettings->enableGradient
			* @instance
            */
            enableGradient: false,
        
            /**		
			* Specifies the colorMappings of the shape  layer in map
            * @default null
			* @type {object}
			* @example 
            * //To set colorMappings API value during initialization 
			*   $("#container").ejMap({layers:{ shapeSettings: {colorMappings:{rangeColorMapping:{from: 0,to: 100000,gradientColors: ["#9CBF4E", "#B8CE7B"]}}}}});
            * @example 
			* //Get or set the colorMappings API, after initialization:<br>
			*	//Gets the colorMappings value 
            *   var shapeProperty =$("#container").data("ejMap").model.layers[layerIndex].shapeSettings.colorMappings;<br> 	
            *	//Sets the colorMappings value 
			*	$("#container").data("ejMap").model.layers[0].shapeSettings.colorMappings={rangeColorMapping:{from: 0,to: 100000,gradientColors: ["#9CBF4E", "#B8CE7B"]}}; 
			* @alias ejMap#shapeLayer->shapeSettings->colorMappings
			* @instance
            */
            colorMappings: null,

            /**		
			* Enables or Disables the auto fill colors for shape  layer in map. When this property value set to true, shapes will be filled with palette colors.
            * @default false
			* @type {boolean}
			* @example 
            * //To set autoFill API value during initialization 
			*   $("#container").ejMap({layers: {shapeSettings: {autoFill:false}}});
            * @example 
			* //Get or set the autoFill API, after initialization:<br>
			*	//Gets the autoFill value 
            *   var shapeProperty =$("#container").data("ejMap").model.layers[layerIndex].shapeSettings.autoFill;<br> 	
            *	//Sets the autoFill value 
			*	 $("#container").data("ejMap").model.layers[0].shapeSettings.autoFill=false; 
			* @alias ejMap#shapeLayer->shapeSettings->autoFill
			* @instance
            */
            autoFill: false
        };

        /** 
        * Specifies the bubble settings for map
        * @type {object}
        * @example 
        * // Set the bubbleSettings of layer during initialization. 			
        * 	$("#container").ejMap({layers: [{ layerType: "geometry", enableMouseHover: true, shapeSettings: { stroke: "black", fill: "#C3E6ED", highlightColor: "#63B7B7", selectionColor: "#207BB2", strokeThickness: "0.5" },bubbleSettings:{ valuePath: "valuePath", minValue: 20, maxValue: 30, color: "#379F64",}, shapeData: mapShapeData }]})
        * @example 
        * //Get or set the bubbleSettings after initialization:<br>
        *   //Gets the bubbleSettings from map.
        *   var bubbleSettings =$("#container").data("ejMap").model.layers[layerIndex].bubbleSettings;
        *   //Sets the bubbleSettings to map.
        *   $("#container").data("ejMap").model.layers[layerIndex].bubbleSettings  = { valuePath: "valuePath", minValue: 20, maxValue: 30, color: "#379F64"};
        * @instance
        * @alias ejMap#shapeLayer->bubbleSettings
        */
        this.bubbleSettings = {

            /**		
			* Specifies the minimum size value of bubbles for shape  layer in map
            * @default "10"
			* @type {string}
			* @example 
            * //To set minValue API value during initialization 
			*   $("#container").ejMap({layers: {bubbleSettings: {minValue:'10'}}});
            * @example 
			* //Get or set the minValue API, after initialization:<br>
			*	//Gets the minValue value 
            *   var bubbleProperty =$("#container").data("ejMap").model.layers[layerIndex].bubbleSettings.minValue;<br> 	
            *	//Sets the minValue value 
			*	$("#container").data("ejMap").model.layers[0].bubbleSettings.minValue='10'; 
			* @alias ejMap#shapeLayer->bubbleSettings->minValue
			* @instance
            */
            minValue: "10",

            /**		
			* Specifies the maximum size value of bubbles for shape  layer in map
            * @default "20"
			* @type {string}
			* @example 
            * //To set maxValue API value during initialization 
			*   $("#container").ejMap({layers: {bubbleSettings: {maxValue:'20'}}});
            * @example 
			* //Get or set the maxValue API, after initialization:<br>
			*	//Gets the maxValue value 
            *   var bubbleProperty =$("#container").data("ejMap").model.layers[layerIndex].bubbleSettings.maxValue;<br> 	
            *	//Sets the maxValue value 
			*	$("#container").data("ejMap").model.layers[0].bubbleSettings.maxValue='20'; 
			* @alias ejMap#shapeLayer->bubbleSettings->maxValue
			* @instance
            */
            maxValue: "20",

            /**		
			* Specifies the mouse hover color of the shape  layer in map
            * @default "gray"
			* @type {string}
			* @example 
            * //To set color API value during initialization 
			*   $("#container").ejMap({layers:{bubbleSettings: {color:'gray'}}});
            * @example 
			* //Get or set the color API, after initialization:<br>
			*	//Gets the color value 
            *   var bubbleProperty =$("#container").data("ejMap").model.layers[layerIndex].bubbleSettings.color;<br> 	
            *	//Sets the color value 
			*	$("#container").data("ejMap").model.layers[0].bubbleSettings.color='gray'; 
			* @alias ejMap#shapeLayer->bubbleSettings->color
			* @instance
            */
            color: "gray",

            /**		
			* Specifies the bubble color valuePath of the shape  layer in map
            * @default null
			* @type {string}
			* @example 
            * //To set colorValuePath  API value during initialization 
			*   $("#container").ejMap({layers: {shapeSettings: {colorValuePath :'sales'}}});
            * @example 
			* //Get or set the colorValuePath  API, after initialization:<br>
			*	//Gets the colorValuePath  value 
            *   var bubbleProperty =$("#container").data("ejMap").model.layers[layerIndex].bubbleSettings.colorValuePath ;<br> 	
            *	//Sets the colorValuePath  value 
			*	$("#container").data("ejMap").model.layers[0].bubbleSettings.colorValuePath ='sales'; 
			* @alias ejMap#shapeLayer->bubbleSettings->colorValuePath 
			* @instance
            */
            colorValuePath : null,

            /**		
			* Specifies the bubble valuePath of the shape  layer in map
            * @default null
			* @type {string}
			* @example 
            * //To set valuePath API value during initialization 
			*   $("#container").ejMap({layers: {shapeSettings: {valuePath:'name'}}});
            * @example 
			* //Get or set the valuePath API, after initialization:<br>
			*	//Gets the valuePath value 
            *   var bubbleProperty =$("#container").data("ejMap").model.layers[layerIndex].bubbleSettings.valuePath;<br> 	
            *	//Sets the valuePath value 
			*	$("#container").data("ejMap").model.layers[0].bubbleSettings.valuePath='name'; 
			* @alias ejMap#shapeLayer->bubbleSettings->valuePath
			* @instance
            */
            valuePath: null,

            /**		
			* Specifies the colorMappings of the shape  layer in map
            * @default null
			* @type {object}
			* @example 
            * //To set colorMappings API value during initialization 
			*   $("#container").ejMap({layers:{ bubbleSettings: {colorMappings:{rangeColorMapping:{from: 0,to: 100000,gradientColors: ["#9CBF4E", "#B8CE7B"]}}}}});
            * @example 
			* //Get or set the colorMappings API, after initialization:<br>
			*	//Gets the colorMappings value 
            *   var bubbleProperty =$("#container").data("ejMap").model.layers[layerIndex].bubbleSettings.colorMappings;<br> 	
            *	//Sets the colorMappings value 
			*	$("#container").data("ejMap").model.layers[0].bubbleSettings.colorMappings={rangeColorMapping:{from: 0,to: 100000,gradientColors: ["#9CBF4E", "#B8CE7B"]}}; 
			* @alias ejMap#shapeLayer->bubbleSettings->colorMappings
			* @instance
            */
            colorMappings: null,

            /**		
			* Specifies the tooltip visibility  status of the shape  layer in map
            * @default false
			* @type {boolean}
			* @example 
            * //To set showTooltip API value during initialization 
			*   $("#container").ejMap({layers: {bubbleSettings: {showTooltip:false}}});
            * @example 
			* //Get or set the showTooltip API, after initialization:<br>
			*	//Gets the showTooltip value 
            *   var bubbleProperty =$("#container").data("ejMap").model.layers[layerIndex].bubbleSettings.showTooltip;<br> 	
            *	//Sets the showTooltip value 
			*	$("#container").data("ejMap").model.layers[0].bubbleSettings.showTooltip=false; 
			* @alias ejMap#shapeLayer->bubbleSettings->showTooltip
			* @instance
            */
            showTooltip: false,

            /**		
			* Specifies the bubble tooltip template of the shape  layer in map
            * @default null
			* @type {string}
			* @example 
            * //To set tooltipTemplate API value during initialization 
			*   $("#container").ejMap({layers: {bubbleSettings: {tooltipTemplate:'template'}}});
            * @example 
			* //Get or set the tooltipTemplate API, after initialization:<br>
			*	//Gets the tooltipTemplate value 
            *   var bubbleProperty =$("#container").data("ejMap").model.layers[layerIndex].bubbleSettings.tooltipTemplate;<br> 	
            *	//Sets the tooltipTemplate value 
			*	$("#container").data("ejMap").model.layers[0].bubbleSettings.tooltipTemplate='template'; 
			* @alias ejMap#shapeLayer->bubbleSettings->tooltipTemplate
			* @instance
            */
            tooltipTemplate: null
        };
    };

    shapeLayer.prototype =
    {
        /**
         * Specify the data types for default properties 
		 * @private
         */
        dataTypes: {
            dataSource: "data",
            markers: "array",
            subLayers: "array",
            shapeSettings: {
                colorMappings: "array"
            },
            bubbleSettings: {
                colorMappings: "array"
            }

        },

        /**
       * Initializes local values of the shape  layer
       * @private
       */
        _initializeLocalValues:function()
        {
            this._svgns = "http://www.w3.org/2000/svg";
            this._bounds = [];
            this._prevSelectedShapes = [];
            this._prevSelectedShapeDatas = [];
            this._prevSelectedLegends = [];
            this._isBaseLayer = true;
            this._prevHoverdLegend = null;
            this._prevHoveredShape = null;
            this._labelCollection = [];
            this._scrollBar = null;
            this._mapShapes = [];
            this._bubbles = [];
            this._labelBounds = [];            
            this._bubbleCount = 0;
            this._mapItems = [];           
            this._mapMarkers = [];
            this._smartLabels = [];
            this._labelData = [];
            this._interactiveArrow = null;
            this._legendRects = [];

        },
        
        /**
       * Generates marker for layers
       * @private
       */
        _generateMarkerForLayers: function (map) {
            this._mapMarkers = [];
            var rootTop = map._rootgroup.getBoundingClientRect().top;
            var rootLeft = map._rootgroup.getBoundingClientRect().left;
            for (var key = 0; key < this.markers.length; key++) {
                var markerObeject = this.markers[key];               
                if (this.markerTemplate != null) {
                    var markerTemplateDiv = $('.markerTemplateDiv');
                    markerTemplateDiv = $("<div class='mapMarker' style='display:block;position:absolute;pointer-events: stroke;'></div>");                   
                    map._templateDiv.append(markerTemplateDiv);
                    var htmlString = $("#" + this.markerTemplate).render(markerObeject);
                    $(markerTemplateDiv).html(htmlString);
                    map._on($(markerTemplateDiv), ej.eventType.mouseDown, { marker: $(markerTemplateDiv), data: markerObeject }, map._markerPressed);
                    this._mapMarkers.push(markerTemplateDiv);
                }
                else {                    
                    var markerItem = $(' <div class="mapMarker" style="display:block;position:absolute;pointer-events: stroke;"><label>' + markerObeject.label + '</label></div>');// document.createElementNS(this._svgns, "text");
                    if (_isSVG) {
                        map._templateDiv.append(markerItem);
                    }
                    else {
                        markerItem.appendTo(map._templateDiv);
                    }
                    map._on($(markerItem), ej.eventType.mouseDown, { marker: $(markerItem), data: markerObeject }, map._markerPressed);
                    this._mapMarkers.push(markerItem);

                }
            }            
        },

        _contains : function(array,actualobj) {
            var length = array.length;
            if (length > 0) {
                while (length--) {
                    if (array[length] === actualobj) {
                        return true;
                    }
                }
            }
            return false;
        },

        /**
       * Clears shape width
       * @private
       */
        _clearShapeWidth: function (scale) {
            if (scale == null) scale = 1;
            for (var index = 0; index < this._mapShapes.length; index++) {
                var mapshape = this._mapShapes[index];
                if (!this._contains(this._prevSelectedShapes,mapshape.shape)) {
                    if (_isSVG) {
                        mapshape.shape.setAttribute("fill", mapshape.shape.getAttribute("nodeValue"));
                        mapshape.shape.setAttribute("stroke-width", (this.shapeSettings.strokeThickness / scale));
                        mapshape.shape.setAttribute("stroke", this.shapeSettings.stroke);
                    }
                    else {
                        mapshape.shape.fillcolor = mapshape.shape.style.behavior;
                        mapshape.shape.strokeweight = this.shapeSettings.strokeThickness;
                        mapshape.shape.strokecolor = this.shapeSettings.stroke;
                    }
                }
            }
            


        },
        
        /**
       * Converts RGB to Hex decimal values
        * @private
       */
        _rgbToHex: function(R, G, B) {
             return "#" + this._toHex(R) + this._toHex(G) + this._toHex(B);
        },
        
        /**
       * Converts to Hex decimal values
        * @private
       */
        _toHex: function (n) {
            var charstr = "0123456789ABCDEF";
            n = parseInt(n, 10);
            if (isNaN(n)) return "00";
            n = Math.max(0, Math.min(n, 255));
            return charstr.charAt((n - n % 16) / 16)
                + charstr.charAt(n % 16);
        },
        
        /**
       * Converts Hex decimal values to Red value
        * @private
       */
        _hexToR: function(h) {
             return parseInt((this._cropHex(h)).substring(0, 2), 16);
        },
        
        /**
       * Converts Hex decimal values to Green value
        * @private
       */
        _hexToG: function(h) {
             return parseInt((this._cropHex(h)).substring(2, 4), 16);
        },
        
        /**
      * Converts Hex decimal values to Blue value
       * @private
      */
        _hexToB: function(h) {
             return parseInt((this._cropHex(h)).substring(4, 6), 16);
        },

        /**
      * crops Hex value
       * @private
      */
        _cropHex: function(h) {
             return (h.charAt(0) == "#") ? h.substring(1, 7) : h;
        },
        
        /**
        * Gets range color values
        * @private
        */
        _getRangeColorValues: function(range, start, end) {
            var rangeColorValues = [];
            rangeColorValues.push(start);
            if (start > end) {
                var rangeValue = (start - end) / (range-1);
                for (var i = range; i > 2; i--) {
                    start = start - rangeValue;
                    rangeColorValues.push(start);
                }
            }
            else {
                var rangeValue = (end - start) / (range-1);
                for (var i = 2; i < range; i++) {
                    start = start + rangeValue;
                    rangeColorValues.push(start + rangeValue);
                }
            }
            rangeColorValues.push(end);
            return rangeColorValues;
        },
        
        /**
        * Generate gradient collection
        * @private
        */
        _generateGradientCollection: function(gradientColors) {
            var gradientCollection = [];
            var startRangeColor = gradientColors[0];
            var endRangeColor = gradientColors[gradientColors.length - 1];
            var startR = this._hexToR(startRangeColor);
            var startG = this._hexToG(startRangeColor);
            var startB = this._hexToB(startRangeColor);
            var endR = this._hexToR(endRangeColor);
            var endG = this._hexToG(endRangeColor);
            var endB = this._hexToB(endRangeColor);
            var range = 10;
            var rRange = this._getRangeColorValues(range, startR, endR);
            var gRange = this._getRangeColorValues(range, startG, endG);
            var bRange = this._getRangeColorValues(range, startB, endB);
            for (var i = 0; i < range; i++) {
                gradientCollection.push(this._rgbToHex(rRange[i], gRange[i], bRange[i]));
            }
            return gradientCollection;
        },
        
        /**
        * Creates labels
        * @private
        */
        _createLabel:function(content, xpos, ypos) {
            var label = $("<div></div>"); // document.createElementNS(this._svgns, "text");
            label[0].innerHTML = content;
            label.css({
                "font-family": "Segoe UI",
                "pointer-events": "none",
                "text-align": "center",
                "margin-left": xpos + "px",
                "margin-top": ypos + "px",
                "position": "absolute"
            });
            return label;
        },
        
        /**
        * Creates interactive arrow for interactive legend
        * @private
        */
        _createInteractiveArrow: function (xpos, ypos) {
            
            var interactiveElement = $("<div class='e-icon1 e-interactiveArrow'></div>");
            //temporary code
            interactiveElement[0].innerHTML = "&#9650";
            
            interactiveElement.css({
                "margin-left": xpos + "px",
                "margin-top": ypos + "px",
                "position": "absolute",
                //temporary code
                "color": "#444444"
            });
            return interactiveElement;
        },

        /**
        * Generates legend for maps
        * @private
        */
        _generateLegends: function (map) {
            if ((this.legendSettings.showLegend == undefined || this.legendSettings.showLegend) && !this.shapeSettings.autoFill) {
                var xpos = 10;
                var ypos = 10;
                var _legendheight = this.legendSettings.height;
                var _legendwidth = this.legendSettings.width;
                var legenddiv = $("<div/>");
                if (_isSVG) {
                    legenddiv.appendTo(map._templateDiv);
                }
                else {
                    map._templateDiv.append(legenddiv);
                }
                var totalwidth = 0;
                var totalheight = 0;
                var isRange = true;
                var mappings = null;
                if (this.shapeSettings.colorMappings.rangeColorMapping != undefined) {
                    mappings = this.shapeSettings.colorMappings.rangeColorMapping;
                } else if (this.shapeSettings.colorMappings.equalColorMapping != undefined) {
                    mappings = this.shapeSettings.colorMappings.equalColorMapping;
                    isRange = false;
                }

                if ((this.legendSettings.type == undefined || this.legendSettings.type == ej.datavisualization.Map.LegendType.Layers) && this.shapeSettings.colorMappings != null
                   && (this.legendSettings.mode == undefined || this.legendSettings.mode == ej.datavisualization.Map.LegendMode.Default || (this.legendSettings.mode == ej.datavisualization.Map.LegendMode.Interactive
                   && this.shapeSettings.colorMappings.equalColorMapping != null))) {
                    
                    if (this.legendSettings.height == undefined)
                        _legendheight = 20;
                    if (this.legendSettings.width == undefined)
                        _legendwidth = 20;

                    for (var key = 0; key < mappings.length; key++) {
                        var colorMapping = mappings[key];
                        if (!colorMapping.hideLegend) {
                            var rect = $("<div class='e-mapLegend'/>");
                            rect.css({
                                "height": _legendheight + "px",
                                "width": _legendwidth + "px",
                                "background-color": colorMapping.color,
                                "margin-left": xpos + "px",
                                "margin-top": ypos + "px",
                                "position": "absolute"
                            });

                            var textcon = this._createLabel(colorMapping.legendLabel, xpos + _legendwidth + 5, ypos);
                            
                            if (colorMapping.legendLabel != null) {
                                textcon[0].innerText= colorMapping.legendLabel;

                            } else {
                                colorMapping.legendLabel = textcon[0].innerText = isRange ? colorMapping.from : colorMapping.value;
                            }
                            if (_isSVG) {
                                rect.appendTo(legenddiv);
                                textcon.appendTo(legenddiv);
                             }
                             else {
                                 legenddiv.append(rect);
                                 legenddiv.append(textcon);
                            }
                           
                            if (totalwidth < (textcon[0].innerText.length * 10) + _legendwidth)
                                totalwidth = (textcon[0].innerText.length * 10) + _legendwidth;
                            var textHeight = textcon.height();
                            var rectHeight = rect.height();
                            if (textHeight > rectHeight) {
                                ypos += (textHeight + 5);
                            } else {
                                ypos += (rectHeight + 5);
                            }
                        }
                    }
                    totalheight = ypos;
                } else if ((this.legendSettings.type == undefined || this.legendSettings.type == ej.datavisualization.Map.LegendType.Layers) && this.legendSettings.mode == ej.datavisualization.Map.LegendMode.Interactive && this.shapeSettings.colorMappings != null) {
                    var textcon = '';
                    
                    if (this.legendSettings.height == undefined)
                        _legendheight = 18;
                    if (this.legendSettings.width == undefined)
                        _legendwidth = 150;

                    if (this.legendSettings.leftLabel == null)
                        this.legendSettings.leftLabel = '';
                    if (this.legendSettings.title != null) {
                        var newxpos = xpos;
                        if (!this.legendSettings.showLabels)
                            newxpos = xpos + this.legendSettings.leftLabel.length * 10;
                        var textcon = this._createLabel(this.legendSettings.title, newxpos, ypos);
                        
                        if (_isSVG)
                            textcon.appendTo(legenddiv);
                        else
                            legenddiv.append(textcon);
                        ypos += 25;
                    }

                    if (this.legendSettings.showLabels)
                        ypos += 25;
                    
                    if (this.legendSettings.leftLabel != null && !this.legendSettings.showLabels) {
                        var textcon = this._createLabel(this.legendSettings.leftLabel, xpos, ypos - 3);

                        if (_isSVG)
                            textcon.appendTo(legenddiv);
                        else
                            legenddiv.append(textcon);
                        xpos = xpos + this.legendSettings.leftLabel.length * 10;
                    }

                    var interactiveElement = this._createInteractiveArrow(xpos, ypos + _legendheight);
                    interactiveElement.appendTo(legenddiv);
                    this._interactiveArrow = interactiveElement;

                    var _legendgroup = null;
                    if (!_isSVG && this.shapeSettings.enableGradient) {
                        _legendgroup = map._createGroup(false, "legendGroup");
                        _legendgroup.style.left = 0 + 'px';
                        _legendgroup.style.top = 0 + 'px';
                        _legendgroup.style.position = "relative";
                        legenddiv.append(_legendgroup);
                    }

                    for (var key = 0; key < mappings.length; key++) {
                        var colorMapping = mappings[key];
                        if (!colorMapping.hideLegend) {
                            var gradientCollection = [];
                            if (this.shapeSettings.enableGradient) {
                                gradientCollection = this._generateGradientCollection(colorMapping.gradientColors);
                            }
                            
                            var obj = {};
                            if (this.shapeSettings.enableGradient) {
                                if (_isSVG) {
                                    var canvas = $("<canvas/>");
                                    var ctx = canvas[0].getContext("2d");
                                    var grd = ctx.createLinearGradient(0, 0, 300, 0);

                                    var temp = 0;
                                    for (var i = 0; i < 10; i++) {
                                        temp = temp + (1 / 10);
                                        if (i == 0) {
                                            grd.addColorStop(0, colorMapping.gradientColors[0]);
                                            grd.addColorStop(temp, gradientCollection[i]);
                                        } else if (i == gradientCollection.length - 1) {
                                            grd.addColorStop(temp - (1 / 10), gradientCollection[i]);
                                            grd.addColorStop(temp, colorMapping.gradientColors[1]);
                                        } else {
                                            grd.addColorStop(temp - (1 / 10), gradientCollection[i]);
                                            grd.addColorStop(temp, gradientCollection[i + 1]);
                                        }
                                    }
                                    ctx.fillStyle = grd;
                                    ctx.fillRect(0, 0, 300, 300);
                                    canvas.css({
                                        "height": _legendheight + "px",
                                        "width": (_legendwidth / mappings.length) + "px",
                                        "margin-left": xpos + "px",
                                        "margin-top": ypos + "px",
                                        "opacity":"0.9",
                                        "filter":"alpha(opacity=90)", /* For IE8 and earlier */
                                        "position": "absolute"
                                    });
                                    canvas.appendTo(legenddiv);
                                } else {
                                    var legendID = "legend" + key;
                                    var legendHtmlString = '<v:rect id=' + legendID + ' display="block" style="position:absolute;top: ' + (ypos - 2) + 'px;left:' + xpos + 'px;width:' + (_legendwidth / mappings.length) + 'px;height:' + _legendheight + 'px;"><v:fill opacity="0.9px" type="gradient" method="linear sigma" angle="270"/><v:stroke opacity="0px"/></v:rect>';
                                    _legendgroup.innerHTML = _legendgroup.innerHTML + legendHtmlString;
                                    var legendelement = document.getElementById(legendID);
                                    legendelement.fillcolor = colorMapping.gradientColors[0];
                                    legendelement.fill.color2 = colorMapping.gradientColors[1];
                                }
                            }
                            else {
                                var rect = $("<div/>");
                                rect.css({
                                    "height": _legendheight + "px",
                                    "width": (_legendwidth / mappings.length) + "px",
                                    "background-color": colorMapping.color,
                                    "margin-left": xpos + "px",
                                    "margin-top": ypos + "px",
                                    "opacity":"0.9",
                                    "filter": "alpha(opacity=90)",/* For IE8 and earlier */
                                    "position": "absolute"
                                });

                                if (_isSVG)
                                    rect.appendTo(legenddiv);
                                else
                                    legenddiv.append(rect);
                            }

                            for (var i = 0; i < 10; i++) {
                                obj = {};
                                obj["marginLeft"] = xpos;
                                this._legendRects.push(obj);
                                xpos = xpos + (_legendwidth / mappings.length) / 10;
                            }

                            if (this.legendSettings.showLabels) {
                                var labelxpos = xpos - (_legendwidth / mappings.length);
                                var labelypos = ypos - 25;
                                var startlabel = this._createLabel((colorMapping.from), labelxpos, labelypos);
                                labelxpos = xpos;
                                var endlabel = this._createLabel((colorMapping.to), labelxpos, labelypos);
                                if (colorMapping.legendLabel != undefined)
                                    endlabel = this._createLabel((colorMapping.legendLabel), labelxpos - (colorMapping.legendLabel.length * 10)/2, labelypos);
                                if (_isSVG) {
                                    if (colorMapping == mappings[0])
                                        startlabel.appendTo(legenddiv);
                                    endlabel.appendTo(legenddiv);
                                }
                                else {
                                    if (colorMapping == mappings[0])
                                        legenddiv.append(startlabel);
                                    legenddiv.append(endlabel);
                                }
                            }
                        }
                    }

                    if (this.legendSettings.rightLabel != null && !this.legendSettings.showLabels) {
                        var textcon = this._createLabel(this.legendSettings.rightLabel, xpos + 10, ypos - 3);
                        if (_isSVG)
                            textcon.appendTo(legenddiv);
                        else
                            legenddiv.append(textcon);
                        xpos = xpos + this.legendSettings.rightLabel.length * 10;
                    }

                    totalwidth = xpos+10;
                    totalheight = ypos + _legendheight+this._interactiveArrow.height()+10;
                }
                
                if (this.legendSettings.position == 'none') {
                    var posx = (map._width * this.legendSettings.PositionX) / 100;
                    var posy = (map._height * this.legendSettings.PositionY) / 100;
                    legenddiv.css({ "margin-left": posx + "px", "margin-top": posy + "px" });
                } else {
                    var position = (this.legendSettings.position == undefined) ? "topleft" : this.legendSettings.position;
                    var controlSize = { width: totalwidth, height: totalheight };
                    var navPos = map._getDockPosition(position, controlSize);
                    legenddiv.css({ "margin-left": navPos.x + "px", "margin-top": navPos.y + "px" });
                }
            }
        },
		
		
		/**
        * Generates bubbleLegend for maps
        * @private
        */
		_generateBubbleLegends: function (map) {
		    if (this.legendSettings.showLegend == undefined || this.legendSettings.showLegend) {
			    var xpos = 10;
                var ypos = 10;
                var _bubblelegendheight = this.legendSettings.height;
                var _bubblelegendwidth = this.legendSettings.width;
                var bubblelegenddiv = $("<div/>");
                if (_isSVG) {
                    bubblelegenddiv.appendTo(map._templateDiv);
                }
                else {
                    map._templateDiv.append(bubblelegenddiv);
                }
				var totalwidth = 0;
                var totalheight = 0;
                var isRange = true;
                var mappings = null;
                if (this.bubbleSettings.colorMappings.rangeColorMapping != undefined) {
                    mappings = this.bubbleSettings.colorMappings.rangeColorMapping;
                } else if (this.bubbleSettings.colorMappings.equalColorMapping != undefined) {
                    mappings = this.bubbleSettings.colorMappings.equalColorMapping;
                    isRange = false;
                }
                if ((this.legendSettings.type == ej.datavisualization.Map.LegendType.Bubbles) && this.bubbleSettings.colorMappings != null
                   && (this.legendSettings.mode == undefined || this.legendSettings.mode == ej.datavisualization.Map.LegendMode.Default || (this.legendSettings.mode == ej.datavisualization.Map.LegendMode.Interactive
                   && this.bubbleSettings.colorMappings.equalColorMapping != null))) {
                    
				    if (this.legendSettings.height == undefined)
                        _bubblelegendheight = 20;
				    if (this.legendSettings.width == undefined)
                        _bubblelegendwidth = 20;

                    for (var key = 0; key < mappings.length; key++) {
                        var colorMapping = mappings[key];
                        if (!colorMapping.hideLegend) {
                            var rect = $("<div class='e-mapBubbleLegend'/>");
                            rect.css({
                                "height": _bubblelegendheight + "px",
                                "width": _bubblelegendwidth + "px",
                                "background-color": colorMapping.color,
                                "margin-left": xpos + "px",
                                "margin-top": ypos + "px",
                                "position": "absolute"
                            });
							var textcon = this._createLabel(colorMapping.legendLabel, xpos + _bubblelegendwidth + 5, ypos);
							if (colorMapping.legendLabel != null) {
                                textcon[0].innerText= colorMapping.legendLabel;

                            } else {
                                colorMapping.legendLabel = textcon[0].innerText = isRange ? colorMapping.from : colorMapping.value;
                            }
							if (_isSVG) {
                                rect.appendTo(bubblelegenddiv);
                                textcon.appendTo(bubblelegenddiv);
                             }
                             else {
                                 bubblelegenddiv.append(rect);
                                 bubblelegenddiv.append(textcon);
							}
							if (totalwidth < (textcon[0].innerText.length * 10) + _bubblelegendwidth)
							    totalwidth = (textcon[0].innerText.length * 10) + _bubblelegendwidth;
                            var textHeight = textcon.height();
                            var rectHeight = rect.height();
                            if (textHeight > rectHeight) {
                                ypos += (textHeight + 5);
                            } else {
                                ypos += (rectHeight + 5);
                            }
						}
                    }
                    totalheight = ypos;
                }                
				if (this.legendSettings.position == 'none') {
				    var posx = (map._width * this.legendSettings.PositionX) / 100;
				    var posy = (map._height * this.legendSettings.PositionY) / 100;
                    bubblelegenddiv.css({ "margin-left": posx + "px", "margin-top": posy + "px" });
                } else {
				    var position = (this.legendSettings.position == undefined) ? "topleft" : this.legendSettings.position;
                    var controlSize = { width: totalwidth, height: totalheight };
                    var navPos = map._getDockPosition(position, controlSize);
                    bubblelegenddiv.css({ "margin-left": navPos.x + "px", "margin-top": navPos.y + "px" });
                }
            }
				
		},
		

        /**
        * Animates the bubble while rendering
        * @private
        */
        _animateBubble: function (element, delayInterval) {
            var radius = element.getAttribute("r");
            var scaleVal;
            var $ele = $(element);
            var layer = this;
            $ele.delay(delayInterval).each(function () { }).animate(
                {
                    r: radius
                },
                {
                    duration: 700,

                    step: function (now) {
                        scaleVal = now;
                        if (_isSVG) {
                            $ele.attr("style", "display:block;pointer-events:stroke;");
                            $ele[0].setAttribute("r", scaleVal);
                        }
                        else {
                        }
                    },
                    complete: function () {
                        layer._bubbleCount++;
                        if (layer._bubbleCount == layer._bubbles.length) {
                            layer._setMapElements();
                        }
                    }
                }

            );
        },

        /**
       * Sets map elements
       * @private
       */
        _setMapElements: function () {
            for (var key = 0; key < this._mapItems.length; key++) {
                var item = this._mapItems[key];                
                $(item).css({ "display": "block" });
            }
        },

        /**
      * Sets map items position with animation
      * @private
      */
        _setMapItemsPositionWithAnimation: function (map) {

            for (var key = 0; key < this._bubbles.length; key++) {
                var bubble = this._bubbles[key];
                var position = this._bounds[key];               
                var xpos = ((position.x + map._translatePoint.x) * map._scale);
                var ypos = ((position.y + map._translatePoint.y) * map._scale);
                var ubound = 20;
                var lbound = 0;
                var randomValue = Math.floor(Math.random() * (ubound - lbound) + lbound);
                var delayInterval = parseInt(randomValue * 50);
                if (_isSVG) {
                    $(bubble).attr({ "cx": xpos, "cy": ypos });
                }
                this._animateBubble(bubble, delayInterval);

            }
            for (var key = 0; key < this._mapMarkers.length; key++) {
                var item = this._mapMarkers[key];
                var marker;
                if (this.markers.length > 0) {
                    marker = this.markers[key];
                }                
                var position;
                if (map._isTileMap) {
                    position = map._convertTileLatLongtoPoint(marker.latitude, marker.longitude);
                }
                else {
                    position = map._convertLatLongtoPointforMarker(marker.latitude, marker.longitude);
                }

                if (_isSVG) {
                    var xpos = position.x;
                }
                else {
                    var xpos = ((position.x + map._transformX) * map._scale);
                }
                var ypos = position.y;
                $(item).css({ "display":"block","left": xpos, "top": ypos - 100 });
                $(item).delay(500).each(function () { }).animate({ "top": ypos }, 500);
               
            }
            for (var key = 0; key < this._mapItems.length; key++) {
                var item = this._mapItems[key];
                var position = this._bounds[key];
                var box = item[0].getBoundingClientRect();               
                var xpos = ((position.x + map._translatePoint.x) * map._scale) - (box.width / 2);
                var ypos = ((position.y + map._translatePoint.y) * map._scale) - (box.height / 4);
                if(this._bubbles.length > 0)
				    $(item).css({ "left": xpos, "top": ypos, "display": "none" });
				else
                   $(item).css({ "left": xpos, "top": ypos, "display": "block" });
               
            }
            for (var key = 0; key < this._labelCollection.length; key++) {
                var item = this._labelCollection[key];
                var position = this._labelBounds[key];
                $(item).css("display", "block");
                var box = item[0].getBoundingClientRect();
                var boxwidth =_isSVG?box.width: box.right - box.left;
                var boxheight=_isSVG?box.height: box.bottom - box.top;
                var xpos = ((position.x + map._translatePoint.x) * map._scale) - (boxwidth / 2);
                var ypos = ((position.y + map._translatePoint.y) * map._scale) - (boxheight / 2);
                if (item[0].className = "labelLegend") {                    
                    item[0].className = "labelStyle";                    
                }
                item[0].innerHTML = this._labelData[key];
                $(item).css({ "left": xpos, "top": ypos });

            }
            if (this.labelSettings!=null && this.labelSettings.showLabels) {
                this._validateSmartLabel(map);
            }
        },

        /**
     * Resizes Shapes of the shape  layer
     * @private
     */
        _resizeShapes: function (map) {
            var thickness = this.shapeSettings.strokeThickness / map._scale;
            for (var i = 0; i < this._mapShapes.length; i++) {
                var element = this._mapShapes[i].shape;
                if (_isSVG) {
                    if (element.localName == 'circle') {
                        element.setAttribute("r", 5 / map._scale);
                    } else {
                        if (this._contains(this._prevSelectedShapes,element)) {
                            element.setAttribute("stroke-width", this.shapeSettings.selectionStrokeWidth / map._scale);
                        } else {
                            element.setAttribute("stroke-width", thickness);
                        }
                    }
                } else {
                    if (element.nodeName == 'oval') {

                    } else {
                        if (this._contains(this._prevSelectedShapes,element)) {
                            element.strokeweight = this.shapeSettings.selectionStrokeWidth / map._scale;
                        } else {
                            element.strokeweight = thickness;
                        }
                    }
                }
            }
        },
        /**
         * Sets map items position
         * @private
         */
        _setMapItemsPosition: function(map) {

            for (var key = 0; key < this._mapItems.length; key++) {
                var item = this._mapItems[key];
                var position = this._bounds[key];
                var box = item[0].getBoundingClientRect();
                var boxwidth = _isSVG ? box.width : box.right - box.left;
                var boxheight = _isSVG ? box.height : box.bottom - box.top;
                var xpos = ((position.x + map._translatePoint.x) * map._scale) - (boxwidth / 2);
                var ypos = ((position.y + map._translatePoint.y) * map._scale) - (boxheight / 4);
                $(item).css({ "left": xpos, "top": ypos });               
            }
            for (var key = 0; key < this._bubbles.length; key++) {
                var bubble = this._bubbles[key];
                var position = this._bounds[key];
                if (_isSVG) {
                    var xpos = ((position.x + map._translatePoint.x) * map._scale);
                    var ypos = ((position.y + map._translatePoint.y) * map._scale);
                    $(bubble).attr({ "cx": xpos, "cy": ypos });
                }
                else {
                    bubble = document.getElementById(bubble.id);
                    var xpos = ((position.x + map._translatePoint.x) * map._scale);
                    var ypos = ((position.y + map._translatePoint.y) * map._scale);
                    var bubbleTop = ypos-(bubble.getBoundingClientRect().bottom - bubble.getBoundingClientRect().top)/2;
                    var bubbleLeft = xpos-(bubble.getBoundingClientRect().right - bubble.getBoundingClientRect().left)/2;
                    
                    $(bubble).css({
                        "left": bubbleLeft,
                        "top": bubbleTop
                    });
                }
            }

            for (var key = 0; key < this._mapMarkers.length; key++) {
                var item = this._mapMarkers[key];

                var marker;
                if (this.markers.length > 0) {
                    marker = this.markers[key];
                }               
                var position;
                if (map._isTileMap) {
                    position = map._convertTileLatLongtoPoint(marker.latitude, marker.longitude);
                }
                else {
                    position = map._convertLatLongtoPointforMarker(marker.latitude, marker.longitude);
                }            
                var xpos = position.x;               
                var ypos = position.y;            
                $(item).css({ "left": xpos, "top": ypos });                
            }
            for (var key = 0; key < this._labelCollection.length; key++) {
                var item = this._labelCollection[key];
                var position = this._labelBounds[key];
                $(item).css("display", "block");
                var box = item[0].getBoundingClientRect();
                var boxwidth = _isSVG ? box.width : box.right - box.left;
                var boxheight = _isSVG ? box.height : box.bottom - box.top;
                var xpos = ((position.x + map._translatePoint.x) * map._scale) - (boxwidth / 2);
                var ypos = ((position.y + map._translatePoint.y) * map._scale) - (boxheight / 2);
                if (item[0].className = "labelLegend") {                    
                    item[0].className = "labelStyle";
                    $(item[0]).css("background-color", "transparent");
                }
                item[0].innerHTML = this._labelData[key];
                $(item).css({ "left": xpos, "top": ypos});
               
            }
            if (this.labelSettings != null && this.labelSettings.showLabels) {
                this._validateSmartLabel(map);
            }
        },

        /**
         * Validates smart labels
         * @private
         */
        _validateSmartLabel: function (map) {
            this._smartLabels = [];
            var filledRects = [];
            if (this._labelCollection.length > 0) {
                for (var index = 0; index < this._mapShapes.length; index++) {
                    var shapemidPoint = this._labelBounds[index];
                    var midPoint = { x: (shapemidPoint.x + map._translatePoint.x) * map._scale, y: (shapemidPoint.y + map._translatePoint.y) * map._scale };
                    var labelElement = this._labelCollection[index];
                    if (midPoint.x > 0 && midPoint.x < map._width && midPoint.y > 0 && midPoint.y < map._height) {
                        var shape = this._mapShapes[index].shape;
                        var bounds = shape.getBoundingClientRect();                        
                        var flag = false;
                        var labelSize = labelElement[0].getBoundingClientRect();
                        var labelHeight = labelSize.height+2;
                        var labelWidth = labelSize.width+2;
                        if (!_isSVG) {
                            bounds = { width: (bounds.right*map._scale) - (bounds.left*map._scale), height: (bounds.bottom *map._scale)- (bounds.top*map._scale) };
                            labelHeight = labelSize.bottom - labelSize.top;
                            labelWidth = labelSize.right - labelSize.left;
                        }
                        var leftPosition = 0;
                        var topPosition = 0;
                        var factor = 20;
                        var xArray = [];
                        var yArray = [];
                        if (labelHeight > bounds.height || labelWidth > bounds.width) {
                            
                            if (this.labelSettings.enableSmartLabel) {
                                if (this.labelSettings.smartLabelSize == ej.datavisualization.Map.LabelSize.Fixed) {
                                    labelHeight = 25;
                                    labelWidth = 15 * this.labelSettings.labelLength;
                                }
                                else {                                    
                                    labelWidth *= 1.3;
                                    labelHeight = 25;
                                }
                                while (!flag) {
                                    if (factor > 400) {
                                        flag = true;
                                    }
                                    xArray = pushToArray(factor);
                                    yArray = xArray;
                                    for (var xIndex = 0; xIndex < xArray.length; xIndex++) {
                                        for (var yIndex = 0; yIndex < xArray.length; yIndex++) {
                                            leftPosition = yArray[yIndex];
                                            topPosition = xArray[xIndex];                                           
                                            if (midPoint.x + leftPosition + labelWidth > map._width && midPoint.x < map._width) {
                                                leftPosition -=(midPoint.x + leftPosition + labelWidth)- map._width;
                                            }
                                            if (midPoint.x + leftPosition <0) {
                                                leftPosition =0;
                                            }
                                            if (midPoint.y + topPosition  <0) {
                                                topPosition=0;
                                            }
                                            if (midPoint.y + topPosition + labelHeight > map._height && midPoint.y < map._height) {
                                                topPosition -= (midPoint.y + topPosition + labelHeight) - map._height;
                                            }
                                            var rect = { left: midPoint.x + leftPosition, top: midPoint.y + topPosition, height: labelHeight, width: labelWidth };
                                            if (isEmpty(rect)) {
                                                var ele = map._getIntersectedElements(rect, this._mapShapes);
                                                if (ele != null && ele.length == 0) {
                                                    flag = true;                                                   
                                                    xIndex = xArray.length;
                                                    yIndex = xArray.length;
                                                    labelElement[0].className = 'labelLegend';													
                                                    if (this.labelSettings.smartLabelSize == ej.datavisualization.Map.LabelSize.Fixed) {
                                                        labelElement[0].innerHTML = labelElement[0].innerHTML.substring(0, this.labelSettings.labelLength);
                                                    }
                                                    labelElement.mouseenter({ Param1: this, Param2: this._mapShapes[index],map:map }, map._polyEnterFunction);
                                                    labelElement.mousemove({ Param1: this, Param2: this._mapShapes[index] }, map._polyMoveFunction);
                                                    $(labelElement).css({ left: midPoint.x + leftPosition, top: midPoint.y + topPosition, "background-color": _isSVG ? shape.getAttribute("fill") : shape.fillcolor.value});
                                                    filledRects.push(rect);
                                                    var labelObj = {};
                                                    labelObj["shape"] = shape;
                                                    labelObj["legend"] = labelElement;
                                                    this._smartLabels.push(labelObj);
                                                }
                                            }
                                        }
                                    }
                                    factor += 10;
                                }
                            } else {
                                labelElement.css("display", "none");
                            }
                        }
                    } 
                }
            }

            function pushToArray(value) {
                var array = [];
                array.push(0);
                for (var j = 10; j <= value; j += 20) {
                    array.push(-j);
                    array.push(j);
                }

                return array;
            }

            function isEmpty(rect) {
                for (var i = 0; i < filledRects.length; i++) {
                    if (isIntersect(rect, filledRects[i]))
                        return false;
                }
                return true;
            }
           
            function isIntersect(rect1, rect2) {
                
                if (rect1.left-2 >= (rect2.left+rect2.width)
|| rect1.left-2+rect1.width <= rect2.left-2
|| rect1.top-2 >= rect2.top+rect2.height
|| rect1.top+rect1.height <= rect2.top-2) {
                    return false;
                }
                return true;
            }


        },
       
        /**
         * Fills colors for the shapes based on color mapping
         * @private
         */
        _fillColors: function (value, colorMapping, shape) {

            if (colorMapping.rangeColorMapping != null) {
               
                this._fillRangeColors(value, colorMapping.rangeColorMapping, shape);
            }
            else if (colorMapping.equalColorMapping != null) {
                this._fillEqualColors(value, colorMapping, shape);
            }
        },

        /**
         * Fills colors for the shapes based on equal color mapping
         * @private
         */
        _fillEqualColors: function (value, colorMapping, shape) {
            $.each(colorMapping.equalColorMapping, function (index, gValue) {
                if (gValue.value == value) {
                    if (_isSVG) {
                        shape.setAttribute("fill", gValue.color);
                    }
                    else {
                        shape.fillcolor = gValue.color;
                    }
                }
            });
        },
        /**
         * Updates legend range
         * @private
         */
        _updateLegendRange: function (value, shapelayer, shape) {
            var colormapping = shapelayer.shapeSettings.colorMappings.rangeColorMapping;
            for (var index = 0; index < colormapping.length; index++) {
                var gradientCollection = null;
                var mapping = colormapping[index];
                if (shapelayer.shapeSettings.enableGradient) {
                    gradientCollection = this._generateGradientCollection(mapping.gradientColors);
                }
                if (value >= mapping.from && value <= mapping.to) {
                    var minValue = index;
                    if (index != 0)
                        minValue = index * 10;
                    var minRange = mapping.from;
                    var maxRange = mapping.from + ((mapping.to - mapping.from) / 10);
                    for (var i = minValue; i < minValue + 10; i++) {
                        if (value >= minRange && value <= maxRange) {
                            var obj = {};
                            if (shapelayer._legendRects[i] != undefined)
                                obj = shapelayer._legendRects[i];
                            if (shapelayer.shapeSettings.enableGradient) {
                                if (index != 0) {
                                    obj["color"] = gradientCollection[i - index * 10];
                                } else {
                                    obj["color"] = gradientCollection[i];
                                }
                            } else
                                obj["color"] = mapping.color;
                            return obj;
                        }
                        minRange = maxRange;
                        maxRange = maxRange + ((mapping.to - mapping.from) / 10);
                    }
                }
            }
        },

        /**
        * Fills colors for the shapes based on range color mapping
        * @private
        */
        _fillRangeColors: function (value, colormapping, shape) {
            
            for (var index = 0; index < colormapping.length; index++) {
                var mapping = colormapping[index];
                if (value >= mapping.from && value <= mapping.to) {
                    var shapeOpacity = this._getColorRatio(0.7, 1, value, mapping.from, mapping.to);
                    if (_isSVG) {
                        shape.setAttribute("fill", mapping.color);
                    }
                    else {
                        shape.fillcolor = mapping.color;
                    }
                    $(shape).css({ "opacity": shapeOpacity });
                }
            }
        },

        /**
        * Gets color ratio for gradient color mapping
        * @private
        */
        _getColorRatio: function (min, max, value, minValue, maxValue) {
            var percent = (100 / (maxValue - minValue)) * (value - minValue);
            var colorCode = (((parseFloat(max) - parseFloat(min)) / 100) * percent) + parseFloat(min);
            return colorCode;
        }

    };

    $.fn.pinchZoom = function (rootgroup, map) {

        var stateOrigin,
                    stateTf,
                    root = this[0],
                    startTouches = [],

           getTouchEventPoint = function (evt) {

               var p = root.createSVGPoint(),
                   targetTouches = evt.targetTouches,
                   offsetX,
                   offsetY;

               if (targetTouches.length) {
                   var touch = targetTouches[0];
                   offsetX = touch.pageX;
                   offsetY = touch.pageY;
               }

               p.x = offsetX;
               p.y = offsetY;

               return p;
           };

        handleTouchStart = function (evt) {
            var g = rootgroup;
            stateTf = g.getCTM().inverse();
            stateOrigin = getTouchEventPoint(evt).matrixTransform(stateTf);
            var touches = evt.touches;
            if (touches.length >= 2) {
                for (var i = 0; i < touches.length; i++) {
                    var touch = touches[i]
                      , found = false;
                    for (var j = 0; j < startTouches.length; j++) {
                        var startTouch = startTouches[j];
                        if (touch.identifier === startTouch.identifier) {
                            found = true;
                            break;
                        }
                    }
                    if (!found) {
                        var touchCopy = $.extend({}, touch);
                        startTouches.push(touchCopy);
                    }
                }
            }

            evt.preventDefault();
        };
        getDistance = function (touch1, touch2) {
            var x = touch2.pageX - touch1.pageX,
                y = touch2.pageY - touch1.pageY;
            return Math.sqrt((x * x) + (y * y));
        };

        setCTM = function (element, matrix) {
		    var isZoomIn=null;
            var s = "matrix(" + matrix.a + "," + matrix.b + "," + matrix.c + "," + matrix.d + "," + matrix.e + "," + matrix.f + ")";
            if (matrix.a > map._baseScale && map.enableZoom() && map.model.enablePan) {
			if(matrix.a > map._scale)
			{
			     isZoomIn=true;
			}
			else if(matrix.a < map._scale)
			{
			isZoomIn=false;
			}
			    map._scale = matrix.a;
			    map._translatePoint.x = matrix.e / matrix.a;
			    map._translatePoint.y = matrix.f / matrix.a;
			    map._applyTransform(map._scale, map._translatePoint);
  			    map.zoomLevel(map._scale - map._baseScale + 1);
	    		if (isZoomIn != null && isZoomIn) {
				    map._trigger("zoomedIn", { originalEvent: null, zoomLevel: map.zoomLevel() });
				    map._updateSliderValue(false);
					map._resizeShape();
				}
				else if(isZoomIn!=null)
				{
				    map._trigger("zoomedOut", { originalEvent: null, zoomLevel: map.zoomLevel() });
				    map._updateSliderValue(false);
					map._resizeShape();
				}
	    		map._isDragging = true;
                map._refrshLayers();
            }

        };

        handleTouchEnd = function (evt) {
            var changedTouches = evt.changedTouches;
            for (var i = 0; i < changedTouches.length; i++) {
                var changedTouch = changedTouches[i];
                for (var j = 0; j < startTouches.length; j++) {
                    var startTouch = startTouches[j];
                    if (startTouch.identifier === changedTouch.identifier) {
                        var idx = startTouches.indexOf(startTouch);
                        startTouches.splice(idx, 1);
                    }
                }
            }
            evt.preventDefault();
        };

        updateTouch = function (dstTouch, srcTouch) {
            dstTouch.pageX = srcTouch.pageX;
            dstTouch.pageY = srcTouch.pageY;
        };

        handleTouchMove = function (evt) {
            evt.preventDefault();

            var g = rootgroup,
                touches = evt.touches,
                z,
                p,
                k;

            if (touches.length >= 2) {
                var touch0 = touches[0]
                  , touch1 = touches[1]
                  , startTouch0 = startTouches[0]
                  , startTouch1 = startTouches[1];
                z = getDistance(touch0, touch1) / getDistance(startTouch0, startTouch1);
                p = root.createSVGPoint();
                p.x = (touch0.pageX + touch1.pageX) / 2;
                p.y = (touch0.pageY + touch1.pageY) / 2;
                p = p.matrixTransform(g.getCTM().inverse());
                k = root.createSVGMatrix().translate(p.x, p.y).scale(z).translate(-p.x, -p.y);
                setCTM(g, g.getCTM().multiply(k));
                if (typeof stateTf === "undefined") {
                    stateTf = g.getCTM().inverse();
                }
                stateTf = stateTf.multiply(k.inverse());
                updateTouch(startTouch0, touch0);
                updateTouch(startTouch1, touch1);
            } else if (!startTouches.length) {
                p = getTouchEventPoint(evt).matrixTransform(stateTf);
                setCTM(g, stateTf.inverse().translate(p.x - stateOrigin.x, p.y - stateOrigin.y));
            }
        };
        this[0].addEventListener('touchstart', handleTouchStart, false);
        this[0].addEventListener('touchend', handleTouchEnd, false);
        this[0].addEventListener('touchmove', handleTouchMove, false);


    };

    jQuery.uaMatch = function (ua) {
        ua = ua.toLowerCase();

        var match = /(chrome)[ \/]([\w.]+)/.exec(ua) ||
            /(webkit)[ \/]([\w.]+)/.exec(ua) ||
            /(opera)(?:.*version|)[ \/]([\w.]+)/.exec(ua) ||
            /(msie) ([\w.]+)/.exec(ua) ||
            ua.indexOf("compatible") < 0 && /(mozilla)(?:.*? rv:([\w.]+)|)/.exec(ua) ||
            [];

        return {
            browser: match[1] || "",
            version: match[2] || "0"
        };
    };

     /**
     * Enum for Map LayerType	 
     * @enum {string}
     * @global 
     */
    ej.datavisualization.Map.LayerType = {
        /**  support for layer type geometry. */
        Geometry: "geometry",
        /**  support for layer type osm. */
        OSM: "osm",
        /**  support for layer type bing. */
        Bing: "bing"
    };

    /**
    * Enum for Map legend icons	 
    * @enum {string}
    * @global 
    */
    ej.datavisualization.Map.LegendIcons = {
        /**  support for legend icon rectangle. */
        Rectangle: "rectangle",
        /**  support for legend icon circle. */
        Circle: "circle"
    };

    /**
    * Enum for Map Label size	 
    * @enum {string}
    * @global 
    */
    ej.datavisualization.Map.LabelSize = {
        /**  support for label size fixed. */
        Fixed: "fixed",
        /**  support for label size default. */
        Default: "default"
    };

    /**
    * Enum for Map Legend mode	 
    * @enum {string}
    * @global 
    */
    ej.datavisualization.Map.LegendMode = {
        /**  support for legend mode default. */
        Default: "default",
        /**  support for legend mode interactive. */
        Interactive: "interactive"
		
    };

    /**
    * Enum for Map Bing map types	 
    * @enum {string}
    * @global 
    */
    ej.datavisualization.Map.BingMapType = {
        /**  support for bing map type aerial. */
        Aerial: "aerial",
        /**  support for bing map type aerialwithlabel. */
        AerialWithLabel: "aerialwithlabel",
        /**  support for bing map type road. */
        Road: "road",
    };

    /**
    * Enum for Map selection mode	 
    * @enum {string}
    * @global 
    */
    ej.datavisualization.Map.SelectionMode = {
        /**  support for selection mode default. */
        Default: "default",
        /**  support for selection mode multiple. */
        Multiple: "multiple"
    };

    /**
    * Enum for Map color palette
    * @enum {string}
    * @global 
    */
    ej.datavisualization.Map.ColorPalette = {
        /**  support for color palette palette1. */
        Palette1: "palette1",
        /**  support for color palette palette2. */
        Palette2: "palette2",
        /**  support for color palette palette3. */
        Palette3: "palette3",
        /**  support for color palette custompalette. */
        CustomPalette: "custompalette"
    };

    /**
    * Enum for Map Legend type
    * @enum {string}
    * @global 
    */
    ej.datavisualization.Map.LegendType = {
        /**  support for Legend type layers. */
        Layers: "layers",
        /**  support for Legend type bubbles. */
        Bubbles: "bubbles"
    }

    /**
    * Enum for Map Navigation Control and Legend dock position	 
    * @enum {string}
    * @global 
    */
    ej.datavisualization.Map.DockPosition = {
        /**  support for dock position none. */
        None: "none",
        /**  support for dock position topleft. */
        TopLeft: "topleft",
        /**  support for dock position topcenter. */
        TopCenter: "topcenter",
        /**  support for dock position topright. */
        TopRight: "topright",
        /**  support for dock position centerleft. */
        CenterLeft: "centerleft",
        /**  support for dock position center. */
        Center: "center",
        /**  support for dock position centerright. */
        CenterRight: "centerright",
        /**  support for dock position bottomleft. */
        BottomLeft: "bottomleft",
        /**  support for dock position bottomcenter. */
        BottomCenter: "bottomcenter",
        /**  support for dock position bottomright. */
        BottomRight: "bottomright"
    };
    
})(jQuery, Syncfusion);
;
/**
* @fileOverview Plugin to style the Html TreeMap elements
* @copyright Copyright Syncfusion Inc. 2001 - 2013. All rights reserved.
*  Use of this code is subject to the terms of our license.
*  A copy of the current license can be obtained at any time by e-mailing
*  licensing@syncfusion.com. Any infringement will be prosecuted under
*  applicable laws. 
* @version 12.1 
* @author &lt;a href="mailto:licensing@syncfusion.com"&gt;Syncfusion Inc&lt;/a&gt;
*/
(function ($, ej, undefined) {
    /**
    * @namespace ej
    * @class ejTreeMap
    * @requires jQuery
    * @requires ej.datavisualization.TreeMap
    * @requires jsrender
    * @requires properties
    * @classdesc The treemap can be easily configured to the DOM element, such as div and can be created with a highly customized look and feel.
    * @example 
    * &lt;div id="container"&gt; <br> 
    * &lt;script&gt;
    * // Create TreeMap
    * $('#container').ejTreeMap(); 	
    * &lt;/script&gt; <br>  
    * &lt;/div&gt;
    */
   
    ej.widget({ "ejTreeMap": "ej.datavisualization.TreeMap" }, {

        validTags: ["div"],

        defaults: {
            /**
            * Specifies the leaf settings of the treemap
            * @default {borderThickness: 1,borderBrush: "white",showLabels: false,labelPath: null,itemTemplate: null}
			* @type {object}
			* @example 
            * //To set datasource API value during initialization 
			*   $("#container").ejTreeMap({leafItemSettings:{ labelPath: "GameName" , showLabels: true, itemTemplate: temp}});
            * @example 
            * //Get or set the datasource API, after initialization:<br>
            *	//Gets the datasource value 
            *   var property =$("#container").data("ejTreeMap").model.leafItemSettings;<br> 
            *	//Sets the datasource value 
            *	$("#container").data("ejTreeMap").model.leafItemSettings= { labelPath: "GameName" , showLabels: true, itemTemplate: temp}; 
            * @memberof ejTreeMap
            * @instance
            */
            leafItemSettings: {
                /**
                * Specifies the border thickness of the leaf item.
                * @default null
			    * @type {object}
			    * @example 
                * //To set borderThickness API value during initialization 
			    *   $("#container").ejTreeMap({leafItemSettings:{ borderThickness: 1}});
                * @example 
                * //Get or set the borderThickness API, after initialization:<br>
                *	//Gets the borderThickness value 
                *   var property =$("#container").data("ejTreeMap").model.leafItemSettings.borderThickness;<br> 
                *	//Sets the borderThickness value 
                *	$("#container").data("ejTreeMap").model.leafItemSettings = { borderThickness: 1}; 
                * @alias ejTreeMap#leafItemSettings->borderThickness
                * @instance
                */
                borderThickness: 1,

                /**
                * Specifies the border bruch color of the leaf item.
                * @default "white"
			    * @type {string}
			    * @example 
                * //To set borderBrush API value during initialization 
			    *   $("#container").ejTreeMap({leafItemSettings:{ borderBrush: "white"}});
                * @example 
                * //Get or set the borderBrush API, after initialization:<br>
                *	//Gets the borderBrush value 
                *   var property =$("#container").data("ejTreeMap").model.leafItemSettings.borderBrush;<br> 
                *	//Sets the borderBrush value 
                *	$("#container").data("ejTreeMap").model.leafItemSettings = { borderBrush: "white"}; 
                * @alias ejTreeMap#leafItemSettings->borderBrush
                * @instance
                */
                borderBrush: "white",

                /**
                * Shows or hides the label of the leaf item.
                * @default "false"
			    * @type {boolean}
			    * @example 
                * //To set showLabels API value during initialization 
			    *   $("#container").ejTreeMap({leafItemSettings:{ showLabels: "false"}});
                * @example 
                * //Get or set the showLabels API, after initialization:<br>
                *	//Gets the showLabels value 
                *   var property =$("#container").data("ejTreeMap").model.leafItemSettings.showLabels;<br> 
                *	//Sets the showLabels value 
                *	$("#container").data("ejTreeMap").model.leafItemSettings = { showLabels: "false"}; 
                * @alias ejTreeMap#leafItemSettings->showLabels
                * @instance
                */
                showLabels: false,

                /**
                * Specifies the label path of the leaf item.
                * @default null
                * @type {string}
                * @example 
                * //To set labelPath API value during initialization 
                *   $("#container").ejTreeMap({leafItemSettings:{ labelPath: "GameName"}});
                * @example 
                * //Get or set the labelPath API, after initialization:<br>
                *	//Gets the labelPath value 
                *   var property =$("#container").data("ejTreeMap").model.leafItemSettings.labelPath;<br> 
                *	//Sets the labelPath value 
                *	$("#container").data("ejTreeMap").model.leafItemSettings = { labelPath: "GameName"}; 
                * @alias ejTreeMap#leafItemSettings->labelPath
                * @instance
                */
                labelPath: null,

                /**
                * Specifies the label template of the leaf item.
                * @default null
			    * @type {string}
			    * @example 
                * //To set itemTemplate API value during initialization 
			    *   $("#container").ejTreeMap({leafItemSettings:{ itemTemplate: "temp"}});
                * @example 
                * //Get or set the itemTemplate API, after initialization:<br>
                *	//Gets the itemTemplate value 
                *   var property =$("#container").data("ejTreeMap").model.leafItemSettings.itemTemplate;<br> 
                *	//Sets the itemTemplate value 
                *	$("#container").data("ejTreeMap").model.leafItemSettings = { itemTemplate: "temp"}; 
                * @alias ejTreeMap#leafItemSettings->itemTemplate
                * @instance
                */
                itemTemplate: null
            },
            /**		
			* Specifies the datasource of the treemap
            * @default null
			* @type {object}
			* @example 
            * //To set datasource API value during initialization 
			*   $("#container").ejTreeMap({datasource:medal_data});
            * @example 
			* //Get or set the datasource API, after initialization:<br>
			*	//Gets the datasource value 
            *   var property =$("#container").data("ejTreeMap").model.datasource;<br> 
            *	//Sets the datasource value 
			*	$("#container").data("ejTreeMap").model.datasource = medal_data; 
			* @memberof ejTreeMap
			* @instance
            */
            dataSource: null,

			/**		
			* Enable/Disable the drillDown for treemap
            * @default false
			* @type {boolean}
			* @example 
            * //To set enableDrillDown API value during initialization 
			*   $("#container").ejTreeMap({enableDrillDown:true});
            * @example 
			* //Get or set the enableDrillDown API, after initialization:<br>
			*	//Gets the enableDrillDown value 
            *   var property =$("#container").data("ejTreeMap").model.enableDrillDown;<br> 
            *	//Sets the enableDrillDown value 
			*	$("#container").data("ejTreeMap").model.enableDrillDown = true; 
			* @memberof ejTreeMap
			* @instance
            */
            enableDrillDown: false,

            /** 
            * specifies the drillDown header color
                 * @default '#0478c3'
                 * @type {string} 
                 * @example  
                 * // Set the drillDownHeaderColor during initialization. 			
                 * 	$("#container").ejTreeMap({drillDownHeaderColor:'#0478c3'});
                * @example 
                * //Get or set the drillDownHeaderColor API, after initialization:<br>
                *	//Gets the drillDownHeaderColor value 
                *   var property =$("#container").data("ejTreeMap").model.drillDownHeaderColor;<br> 
                *	//Sets the drillDownHeaderColor value 
                *	$("#container").data("ejTreeMap").model.drillDownHeaderColor = '#0478c3'; 
                * @memberof ejTreeMap
                * @instance
                 */
            drillDownHeaderColor: '#0478c3',

            /** 
            * specifies the drillDown selection color
                 * @default '#000000'
                 * @type {string} 
                 * @example  
                 * // Set the drillDownSelectionColor during initialization. 			
                 * 	$("#container").ejTreeMap({drillDownSelectionColor:'#e5e5e5'});
                * @example 
			    * //Get or set the drillDownSelectionColor API, after initialization:<br>
			    *	//Gets the drillDownSelectionColor value 
                *   var property =$("#container").data("ejTreeMap").model.drillDownSelectionColor;<br> 
                *	//Sets the drillDownSelectionColor value 
			    *	$("#container").data("ejTreeMap").model.drillDownSelectionColor = '#000000'; 
			    * @memberof ejTreeMap
			    * @instance
                 */
            drillDownSelectionColor: '#000000',

            /** 
            * Specifies the selection mode of the treemap. Accepted selection mode values are Default and Multiple.
            * @default "default"
			* @type {string}
             * @example  
             * // Set the selection mode during initialization. 			
             * 	$("#container").ejTreeMap({layers: [{ selectionMode:'default' }]})
             * @example 
             * //Get or set the groupSelection mode after initialization:<br>
             *   //Gets the selection mode from treemap.
             *   var property =$("#container").data("ejTreeMap").model.layers[layerIndex].groupSelectionMode;
             *   //Sets the selection mode to treemap.
             *   $("#container").data("ejTreeMap").model.layers[layerIndex].groupSelectionMode  = 'default';
             * @alias ejTreeMap#shapeLayer->groupSelectionMode
             * @instance
             */
            groupSelectionMode : "default",
			
            /**		
			* Specifies the color valuepath of the treemap
            * @default null
			* @type {string}
			* @example 
            * //To set colorValuePath API value during initialization 
			*   $("#container").ejTreeMap({colorValuePath:'GoldMedals'});
            * @example 
			* //Get or set the colorValuePath API, after initialization:<br>
			*	//Gets the colorValuePath value 
            *   var property =$("#container").data("ejTreeMap").model.colorValuePath;<br> 
            *	//Sets the colorValuePath value 
			*	$("#container").data("ejTreeMap").model.colorValuePath = 'GoldMedals'; 
			* @memberof ejTreeMap
			* @instance
            */
            colorValuePath: null,

            /**		
			* Specifies the weight valuepath of the treemap
            * @default null
			* @type {string}
			* @example 
            * //To set weightValuePath API value during initialization 
			*   $("#container").ejTreeMap({weightValuePath:'TotalMedals'});
            * @example 
			* //Get or set the weightValuePath API, after initialization:<br>
			*	//Gets the weightValuePath value 
            *   var property =$("#container").data("ejTreeMap").model.weightValuePath;<br> 
            *	//Sets the weightValuePath value 
			*	$("#container").data("ejTreeMap").model.weightValuePath = 'TotalMedals'; 
			* @memberof ejTreeMap
			* @instance
            */
            weightValuePath: null,


            treeMapItems: [],

            /**		
			* Specifies the legend visibility  status of the treemap
            * @default false
			* @type {boolean}
			* @example 
            * //To set showLegend API value during initialization 
			*   $("#container").ejTreeMap({showLegend:false});
            * @example 
			* //Get or set the showLegend API, after initialization:<br>
			*	//Gets the showLegend value 
            *   var property =$("#container").data("ejTreeMap").model.showLegend;<br> 
            *	//Sets the showLegend value 
			*	$("#container").data("ejTreeMap").model.showLegend = false; 
			* @memberof ejTreeMap
			* @instance
            */
            showLegend: false,

            /**		
			* Specifies the border brush color of the treemap
            * @default "white"
			* @type {string}
			* @example 
            * //To set borderBrush API value during initialization 
			*   $("#container").ejTreeMap( {borderBrush:'white'});
            * @example 
			* //Get or set the borderBrush API, after initialization:<br>
			*	//Gets the borderBrush value 
            *   var property =$("#container").data("ejTreeMap").model.borderBrush;<br> 
            *	//Sets the borderBrush value 
			*	$("#container").data("ejTreeMap").model.borderBrush = 'white'; 
			* @memberof ejTreeMap
			* @instance
            */
            borderBrush: "white",

            /**		
			* Specifies the border thickness of the treemap
            * @default 1
			* @type {number}
			* @example 
            * //To set borderThickness API value during initialization 
			*   $("#container").ejTreeMap({borderThickness:1});
            * @example 
			* //Get or set the borderThickness API, after initialization:<br>
			*	//Gets the borderThickness value 
            *   var property =$("#container").data("ejTreeMap").model.borderThickness;<br> 
            *	//Sets the borderThickness value 
			*	$("#container").data("ejTreeMap").model.borderThickness = 1; 
			* @memberof ejTreeMap
			* @instance
            */
            borderThickness: 1,

            /**		
			* Specifies whether treemap need to resize when container is resized
            * @default true
			* @type {boolean}
			* @example 
            * //To set enableResize API value during initialization 
			*   $("#container").ejTreeMap({enableResize:false});
            * @example 
			* //Get or set the enableResize API, after initialization:<br>
			*	//Gets the enableResize value 
            *   var property =$("#container").data("ejTreeMap").model.enableResize;<br> 
            *	//Sets the enableResize value 
			*	$("#container").data("ejTreeMap").model.enableResize = false; 
			* @memberof ejTreeMap
			* @instance
            */
            enableResize: true,

            /**		
			* Specifies the items layout mode of the treemap. Accepted itemsLayoutMode values are Squarified, SliceAndDiceHorizontal, SliceAndDiceVertical and SliceAndDiceAuto
            * @default "Squarified"
			* @type {string}
			* @example 
            * //To set itemsLayoutMode API value during initialization 
			*   $("#container").ejTreeMap({itemsLayoutMode:ej.datavisualization.TreeMap.ItemsLayoutMode.Squarified});
            * @example 
			* //Get or set the itemsLayoutMode API, after initialization:<br>
			*	//Gets the itemsLayoutMode value 
            *   var property =$("#container").data("ejTreeMap").model.itemsLayoutMode;<br> 
            *	//Sets the itemsLayoutMode value 
			*	$("#container").data("ejTreeMap").model.itemsLayoutMode = ej.datavisualization.TreeMap.ItemsLayoutMode.Squarified; 
			* @memberof ejTreeMap
			* @instance
            */
            itemsLayoutMode: "squarified",

            // Level Collection
            /**		
            * Specify levels of treemap for grouped visualization of datas
            * @default []
            * @type {treeMapLevel} 
            * @example  
            * // Set the levels during initialization. 		
            *   $("#container").ejTreeMap({	levels: [{ groupPath: "Continent", groupGap: 5, headerHeight: 30, headerTemplate: 'headertemplate' }]})
            * @example 
            * //Get or set the levels after initialization:<br>
            *   //Gets the levels from map.
            *   var levels =$("#container").data("ejTreeMap").model.levels[levelIndex];
            *   //Sets the levels to map.
            *   $("#container").data("ejTreeMap").model.levels[levelIndex]  = { groupPath: "Continent", groupGap: 5, headerHeight: 30, headerTemplate: 'headertemplate' };
            * @memberof ejTreeMap
            * @instance
            */
            levels: [],

            /**		
			* Specifies the legendSettings of the treemap
            * @default null
			* @type {object}
			* @example 
            * //To set legendSettings API value during initialization 
			*   $("#container").ejTreeMap( {legendSettings:{ template: null, iconHeight: 15, iconWidth: 25 }});
            * @example 
			* //Get or set the legendSettings API, after initialization:<br>
			*	//Gets the legendSettings value 
            *   var property =$("#container").data("ejTreeMap").model.legendSettings;<br> 
            *	//Sets the legendSettings value 
			*	$("#container").data("ejTreeMap").model.legendSettings = { template: null, iconHeight: 15, iconWidth: 25 }; 
			* @memberof ejTreeMap
			* @instance
            */
            legendSettings: { template: null, iconHeight: 15, iconWidth: 25 },

            /**		
			* Specifies the range color mapping of the treemap
            * @default []
			* @type {object}
			* @example 
            * //To set rangeColorMapping API value during initialization 
			*   $("#container").ejTreeMap( {rangeColorMapping:[{ color: "#77D8D8",legendLabel:"1% Growth", from: "0", to: "1" }]});
            * @example 
			* //Get or set the rangeColorMapping API, after initialization:<br>
			*	//Gets the rangeColorMapping value 
            *   var property =$("#container").data("ejTreeMap").model.rangeColorMapping;<br> 
            *	//Sets the rangeColorMapping value 
			*	$("#container").data("ejTreeMap").model.rangeColorMapping = [{ color: "#77D8D8",legendLabel:"1% Growth", from: "0", to: "1" }]; 
			* @memberof ejTreeMap
			* @instance
            */
            rangeColorMapping: [],
            /**		
			* Specifies the uniform color mapping of the treemap
            * @default { color: null }
			* @type {object}
			* @example 
            * //To set uniColorMapping API value during initialization 
			*   $("#container").ejTreeMap( {uniColorMapping{ color: null }});
            * @example 
			* //Get or set the uniColorMapping API, after initialization:<br>
			*	//Gets the uniColorMapping value 
            *   var property =$("#container").data("ejTreeMap").model.uniColorMapping;<br> 
            *	//Sets the uniColorMapping value 
			*	$("#container").data("ejTreeMap").model.uniColorMapping = { color: null }; 
			* @memberof ejTreeMap
			* @instance
            */
            uniColorMapping: { color: null },

            /**		
			* Specifies the desaturation color mapping of the treemap
            * @default { from: 0, to: 0, color: 'blue', rangeMinimum: 0, rangeMaximum: 0 }
			* @type {object}
			* @example 
            * //To set desaturationColorMapping API value during initialization 
			*   $("#container").ejTreeMap( {desaturationColorMapping{ from:1, to:0.5 ,color:"#41B8C4", rangeMinimum:2000,rangeMaximum:8000}});
            * @example 
			* //Get or set the desaturationColorMapping API, after initialization:<br>
			*	//Gets the desaturationColorMapping value 
            *   var property =$("#container").data("ejTreeMap").model.desaturationColorMapping;<br> 
            *	//Sets the desaturationColorMapping value 
			*	$("#container").data("ejTreeMap").model.desaturationColorMapping = { from:1, to:0.5 ,color:"#41B8C4", rangeMinimum:2000,rangeMaximum:8000}; 
			* @memberof ejTreeMap
			* @instance
            */
            desaturationColorMapping: { from:0, to:0,color:null, rangeMinimum:0,rangeMaximum:0},

            /**		
			* Specifies the palette color mapping of the treemap
            * @default null
			* @type {object}
			* @example 
            * //To set paletteColorMapping API value during initialization 
			*   $("#container").ejTreeMap( {paletteColorMapping{colors: ["red","green","blue", "yellow"]}});
            * @example 
			* //Get or set the paletteColorMapping API, after initialization:<br>
			*	//Gets the paletteColorMapping value 
            *   var property =$("#container").data("ejTreeMap").model.paletteColorMapping;<br> 
            *	//Sets the paletteColorMapping value 
			*	$("#container").data("ejTreeMap").model.paletteColorMapping = {colors: ["red","green","blue", "yellow"]}; 
			* @memberof ejTreeMap
			* @instance
            */
            paletteColorMapping: null,

            /**		
			* Specifies whether treemap item need to highlighted on selection
            * @default false
			* @type {boolean}
			* @example 
            * //To set highlightOnSelection API value during initialization 
			*   $("#container").ejTreeMap({highlightOnSelection:false});
            * @example 
			* //Get or set the highlightOnSelection API, after initialization:<br>
			*	//Gets the highlightOnSelection value 
            *   var property =$("#container").data("ejTreeMap").model.highlightOnSelection;<br> 
            *	//Sets the highlightOnSelection value 
			*	$("#container").data("ejTreeMap").model.highlightOnSelection = false; 
			* @memberof ejTreeMap
			* @instance
            */
            highlightOnSelection: false,
			
			 /**		
			* Specifies whether treemap item need to highlighted on selection
            * @default false
			* @type {boolean}
			* @example 
            * //To set highlightGroupOnSelection API value during initialization 
			*   $("#container").ejTreeMap({highlightGroupOnSelection:false});
            * @example 
			* //Get or set the highlightGroupOnSelection API, after initialization:<br>
			*	//Gets the highlightGroupOnSelection value 
            *   var property =$("#container").data("ejTreeMap").model.highlightGroupOnSelection;<br> 
            *	//Sets the highlightGroupOnSelection value 
			*	$("#container").data("ejTreeMap").model.highlightGroupOnSelection = false; 
			* @memberof ejTreeMap
			* @instance
            */
            highlightGroupOnSelection: false,

            /**		
			* Specifies whether treemap tooltip need to be visible
            * @default false
			* @type {boolean}
			* @example 
            * //To set showTooltip API value during initialization 
			*   $("#container").ejTreeMap({showTooltip:false});
            * @example 
			* //Get or set the showTooltip API, after initialization:<br>
			*	//Gets the showTooltip value 
            *   var property =$("#container").data("ejTreeMap").model.showTooltip;<br> 
            *	//Sets the showTooltip value 
			*	$("#container").data("ejTreeMap").model.showTooltip = false; 
			* @memberof ejTreeMap
			* @instance
            */
            showTooltip: false,

            /**		
			* Specifies the tooltip template of the treemap
            * @default null
			* @type {string}
			* @example 
            * //To set tooltipTemplate API value during initialization 
			*   $("#container").ejTreeMap({tooltipTemplate:'template'});
            * @example 
			* //Get or set the tooltipTemplate API, after initialization:<br>
			*	//Gets the tooltipTemplate value 
            *   var property =$("#container").data("ejTreeMap").model.tooltipTemplate;<br> 
            *	//Sets the tooltipTemplate value 
			*	$("#container").data("ejTreeMap").model.tooltipTemplate = 'template'; 
			* @memberof ejTreeMap
			* @instance
            */
            tooltipTemplate: null,

            /**		
			* Specifies the border thickness when treemap items is highlighted in the treemap
            * @default 5
			* @type {number}
			* @example 
            * //To set highlightBorderThickness API value during initialization 
			*   $("#container").ejTreeMap({highlightBorderThickness:5});
            * @example 
			* //Get or set the highlightBorderThickness API, after initialization:<br>
			*	//Gets the highlightBorderThickness value 
            *   var property =$("#container").data("ejTreeMap").model.highlightBorderThickness;<br> 
            *	//Sets the highlightBorderThickness value 
			*	$("#container").data("ejTreeMap").model.highlightBorderThickness = 5; 
			* @memberof ejTreeMap
			* @instance
            */
            highlightBorderThickness: 5,

            /**		
           * Specifies the border thickness when treemap items is highlighted in the treemap
           * @default 5
           * @type {number}
           * @example 
           * //To set highlightGroupBorderThickness API value during initialization 
           *   $("#container").ejTreeMap({highlightGroupBorderThickness:5});
           * @example 
           * //Get or set the highlightGroupBorderThickness API, after initialization:<br>
           *	//Gets the highlightGroupBorderThickness value 
           *   var property =$("#container").data("ejTreeMap").model.highlightGroupBorderThickness;<br> 
           *	//Sets the highlightGroupBorderThickness value 
           *	$("#container").data("ejTreeMap").model.highlightGroupBorderThickness = 5; 
           * @memberof ejTreeMap
           * @instance
           */
            highlightGroupBorderThickness: 4,

            /**		
			* Specifies the highlight border brush of treemap
            * @default "gray"
			* @type {string}
			* @example 
            * //To set highlightBorderBrush API value during initialization 
			*   $("#container").ejTreeMap({highlightBorderBrush:'gray'});
            * @example 
			* //Get or set the highlightBorderBrush API, after initialization:<br>
			*	//Gets the highlightBorderBrush value 
            *   var property =$("#container").data("ejTreeMap").model.highlightBorderBrush;<br> 
            *	//Sets the highlightBorderBrush value 
			*	$("#container").data("ejTreeMap").model.highlightBorderBrush = 'gray'; 
			* @memberof ejTreeMap
			* @instance
            */
            highlightBorderBrush: "gray",

            /**		
           * Specifies the highlight border brush of treemap
           * @default "gray"
           * @type {string}
           * @example 
           * //To set highlightGroupBorderBrush API value during initialization 
           *   $("#container").ejTreeMap({highlightGroupBorderBrush:'gray'});
           * @example 
           * //Get or set the highlightGroupBorderBrush API, after initialization:<br>
           *	//Gets the highlightGroupBorderBrush value 
           *   var property =$("#container").data("ejTreeMap").model.highlightGroupBorderBrush;<br> 
           *	//Sets the highlightGroupBorderBrush value 
           *	$("#container").data("ejTreeMap").model.highlightGroupBorderBrush = 'gray'; 
           * @memberof ejTreeMap
           * @instance
           */
            highlightGroupBorderBrush: "gray"

        },
        /**
         * Specify the data types for default properties 
		 * @private
         */
        dataTypes: {
            dataSource: "data",
            treeMapItems: "array",
            levels: "array",
            rangeColorMapping: "array",
            paletteColorMapping: "array"
        },
		
        observables: ["dataSource", "colorValuePath", "weightValuePath"],
        dataSource: ej.util.valueFunction("dataSource"),
        colorValuePath: ej.util.valueFunction("colorValuePath"),
        weightValuePath: ej.util.valueFunction("weightValuePath"),

        _initPrivateProperties: function () {
            this._svgDocument = null;
            this._templateDiv = null;
            this._drillHeaderDiv = null;
            this._drillHoverDiv = null;
            this._backgroundTile = null;
            this._height = 500;
            this._initDiv = null;
            this._width = 500;
            this._svgns = "http://www.w3.org/2000/svg";
            this._prevSelectedItems=[];
			this._prevSelectedHeaders = [];
            /**     
        * Triggers on treemap item selected. 
        * @event
        * @name ejTreeMap#treeMapItemSelected            
        * @param {object} originalEvent.data Returns selected treeMapItem object.
        * @example 
        * //treemap item selected event for treemap
        * $("#container").ejTreeMap({
        *    treeMapItemSelected: function () {}
        * });
        * @memberof ejTreeMap
        * @instance
        */
            this.treeMapItemSelected = null;
            this.treeMapGroupSelected = null;
            this._browser = null;
            this._toolTipElement = null;
            this._rootTreeMapItems = [];
            this._drillHeader = null;
            this._drilldownItem = null;
            this._drilldownItems = [];
            this._treeMapHeaders = [];
            this._treeMapLabels = [];
            this._labelTemplateElements = [];
            this._itemGroups = [];
			this._prevSelectedItemDatas=[];
            this._prevSelectedGroupDatas = [];

        },

        /**
          * To configure the properties at runtime using SetModel		
          * @private
          */
        _setModel: function (options) {
            for (var prop in options) {
                switch (prop) {
                    case "itemsLayoutMode":
                        this.model.itemsLayoutMode = options[prop];
                        this.refresh();
                        break;
                }
            }
        },

        /**
        * Create the treemap widget
        * @private
        */
        _init: function () {

            this._initPrivateProperties();
            var style = $('<style>.drillGroupRect:hover{ background-color:' + this.model.drillDownSelectionColor + ';}</style>');
            $('html > head').append(style);
            this._levelInitialize();

            this.refresh();           

        },

        /**
        * Destroy the map widget
        * @private
        */
        _destroy: function() {

        },
        
        /**
        * Create level for treemap
        * @private
        */
        _levelInitialize: function () {
            var proxy = this;
            if (this.model.levels != null) {
                $.each(this.model.levels, function (index, element) {                    
                    element = proxy._checkArrayObject(element, index);
                    var obj = new treeMapLevel();                    
                    $.extend(obj, element);
                    $.extend(element, obj);                    
                });
            }
            else {
                this.levels.treeMapLevel = new treeMapLevel();

            }
			
			$(document).click({Param1: this}, this._docClickFunction);
        },

        /**
       * Checks array object
       * @private
       */
        _checkArrayObject: function (element, initialName) {
            var proxy = this;
            $.each(element, function (name, innerElement) {
                if (innerElement instanceof Array) {
                    proxy._checkArrayObject(innerElement, name);
                }
                else if (innerElement != null && typeof innerElement == "object") {
                    var allObjects = new treeMapLevel();
                    proxy._loadIndividualDefaultValues(innerElement, allObjects, (typeof name === "number") ? initialName : name);
                }
            });
            return element;
        },

        /**
          * Loads individual default values
          * @private
          */
        _loadIndividualDefaultValues: function (obj, allObjects, name) {
            var defaultObj = null;
            var proxy = this;
            $.each(allObjects, function (n, element) {
                if (name == n) {
                    defaultObj = element;
                    return;
                }
            });
            if (defaultObj instanceof Array) defaultObj = defaultObj[0];

            $.each(obj, function (objName, ele) {
                if (ele instanceof Array) {
                    proxy._checkArrayObject(ele, name);
                }
                else if (ele != null && typeof ele == "object") {
                    proxy._loadIndividualDefaultValues(ele, ele, (typeof objName === "number") ? name : objName);
                }
            });

            $.extend(defaultObj, obj);
            $.extend(obj, defaultObj);
            return obj;
        },


        /**     
       * Method to reload treemap with updated values.
       * @return jQuery
       * @example 
       * //refresh method for treemap
       * $("#container").ejTreeMap({
       *    refresh: function () {}
       * });
       * @memberof ejTreeMap
       * @instance
       */
        refresh: function () {
            this._initPrivateProperties();
            $(this.element).empty();
            if (this._svgDocument != null) {
                $(this._svgDocument).empty();
            }
            this._height = this.element.height();
            this._width = this.element.width();
             if (this._height == 0) {
                 this._height = this.element[0].parentElement.clientHeight != 0 ? this.element[0].parentElement.clientHeight : $(document).height();
            }
            if (this._width == 0) {
                this._width = this.element[0].parentElement.clientWidth != 0 ? this.element[0].parentElement.clientWidth : $(document).width();
            }
		    var matched = jQuery.uaMatch(navigator.userAgent);
            var browser = matched.browser.toLowerCase();
            var isIE11 = !!navigator.userAgent.match(/Trident\/7\./);
            this._browser = browser;
			  $(this.element).css({ 'position': 'relative' });			
           
            if (this._height != 0 && this._width != 0) {                
                this._templateDiv = $("<div class='_templateDiv'></div>");
                this._templateDiv.css({
                    'pointer-events': 'none', 'overflow': 'hidden', 'float': 'left',
                    'z-index': '2', 'height': this._height, 'width': this._width,"position":"absolute","left":"0","top":"0"
                });
                if (this.model.enableDrillDown) {
                    this._drillHeaderDiv = $("<div class='_drillHeaderDiv'></div>");
                    this._drillHeaderDiv.css({
                        'overflow': 'hidden', 'float': 'left',
                        'z-index': '3', "position": "absolute", "left": "0", "top": "0"
                    });
                    this._drillHoverDiv = $("<div class='_drillHoverDiv'></div>");
                    this._drillHoverDiv.css({
                        'overflow': 'hidden', 'float': 'left',
                        'z-index': '3', 'height': this._height - 30, 'width': this._width, "position": "absolute", "left": "0", "top": "30px"
                    });
                }
                this._backgroundTile= $('<div id="backgroundTile" style="overflow:hidden;z-index:0;"></div>');
				if(this.model.dataSource!=null){
                      if (this.model.dataSource instanceof ej.DataManager) {
				          this._processOData(this);
				      }
				 else
				 {
				     this._renderTreeMap(this.dataSource());
				 }
                }
                if (this.model.enableResize) {
                    this._on($(window), "resize", this._treeMapResize);
                }
            }
        },
		
	   /**
      * Section For handling treemap redering
      * @private
      */
		_renderTreeMap:function(data)
		{
		    this._groupLevels(data);
            if (this.model.showLegend) {
                this._generateLegend();
            }
            this._generateToolTip();
            this._renderLabels();
		},
		
		 /**
         * Section For handling treemap redering
         * @private
         */
		_processOData: function (treemap) {  
			var treeMap= this;		 
			var queryPromise = treemap.model.dataSource.executeQuery(treemap.model.query);
			queryPromise.done(function (e) {
			    if(e.result!=null)
				{
                   treeMap._renderTreeMap(e.result);
				}
			});        
        },
    
        /**
      * Section For handling treemap resize
      * @private
      */
        _treeMapResize: function (event) {
            event.preventDefault();
            event.stopPropagation();
            var treemap = this;
            if (this.resizeTO) clearTimeout(this.resizeTO);
            this.resizeTO = setTimeout(function () {
                treemap.refresh();
            }, 500);
        },
       
        /**
          * Groups levels of treemap
          * @private
          */
        _groupLevels: function (data) {
           
            var subItems = null;
            if(this.model.enableDrillDown)
                this._drillDownHeader(this.model.levels[0].groupPath,false);
            var rootItems = this._getTopGroupitem(this.model.leafItemSettings.labelPath, data, 0);
            subItems = rootItems;
            if (this.model.levels.length == 0) {
                var levelItem = this.model.leafItemSettings;
                this._rootTreeMapItems = rootItems;
                this._getTopLevels(rootItems[0].innerItems, levelItem);
                this._generateTreeMapItems(rootItems[0].innerItems, "gray", this.colorValuePath());           
				
                this._generateLabels(rootItems[0].innerItems, null, levelItem.itemTemplate, levelItem.showLabels);                    
            }
            if(this.model.levels.length>0){
                for (var i = 0; i <= this.model.levels.length; i++) {
                    var levelItem = this.model.levels[i];
                    if (i != 0) {
                        var cloneItems = subItems;
                        var PrevItem = this.model.levels[i - 1];
                        subItems = [];
                        if (levelItem == null) levelItem = this.model.leafItemSettings;
                        this._generateSubItems(cloneItems, levelItem, subItems, PrevItem);
                        
                        this._rootTreeMapItems = subItems;                        
                        if (this.model.enableDrillDown)
                        {
                            if (i == 1) {
                                this._generateTreeMapItems(subItems, "gray", PrevItem.DisplayPath);                                
                                return false;                          
                            }
                        }
                        else if (i == this.model.levels.length) {
                            this._generateTreeMapItems(subItems, "gray", PrevItem.DisplayPath);
                            
                        }                                                  
						
                        this._generateLabels(subItems, null, levelItem.itemTemplate, levelItem.showLabels);
                    }
                    else {
                        var rootItems1 = this._getGroupitem(levelItem.groupPath, rootItems[0].Data, levelItem.headerHeight);
                        this._rootTreeMapItems = rootItems1;
                        this._getTopLevels(rootItems1, levelItem);
                        subItems = rootItems1;                        
                    }
                }
                
            }
        },


        _groupdrillDownLevels: function (data, actualindex) {       

            var subItems = null;
            if (actualindex == this.model.levels.length-1)
            {
                this._drillHoverDiv.css("pointer-events", "none");
            }
            else
            {
                this._drillHoverDiv.css("pointer-events", "auto");
            }
            if (this.model.levels.length > 0) {

                for (var i = 0; i < this.model.levels.length; i++) {                   
                    var levelItem = this.model.levels[i];
                    if (i != 0) {
                        var cloneItems = subItems;
                        var PrevItem = this.model.levels[i - 1];
                        subItems = [];
                        if (levelItem == null) levelItem = this.model.leafItemSettings;

                        this._generateSubItems(cloneItems, levelItem, subItems, PrevItem, true, (actualindex+1 != i));

                        this._rootTreeMapItems = subItems;
                        if (i == actualindex + 1 || i== this.model.levels.length-1) {
                            this._generateTreeMapItems(subItems, "gray", PrevItem.DisplayPath);
                        }
                        if (i == actualindex)
                            this._generateLabels(subItems, null, levelItem.itemTemplate, levelItem.showLabels);
                    }
                    else {

                        var rootItems1 = this._getGroupitem(levelItem.groupPath, data.Data, levelItem.headerHeight);
                        this._rootTreeMapItems = rootItems1;
                        this._getTopLevels(rootItems1, levelItem);
                        subItems = rootItems1;
                    }
                }

            }            
        },

        /**
          * Gets top levels of treemap
          * @private
          */
        _getTopLevels: function (rootItems, level) {
            var gap = (level.groupGap != null) ? level.groupGap : 0;
            var layout = this.model.itemsLayoutMode;
            var legendHeight = 0;
            var defaultspacing = 0;
            if (this.model.enableDrillDown) {
                defaultspacing = 30;
            }
                if (this.model.showLegend && this.model.legendSettings != null) {
                    legendHeight = this.model.legendSettings.iconHeight + 20;
                }
                if (layout == ej.datavisualization.TreeMap.ItemsLayoutMode.SliceAndDiceHorizontal) {
                    this._calculateSliceAndDiceItemSize(rootItems, 0, legendHeight, this._width, this._height, gap, defaultspacing, true, legendHeight);
                }
                else if (layout == ej.datavisualization.TreeMap.ItemsLayoutMode.SliceAndDiceVertical) {
                    this._calculateSliceAndDiceItemSize(rootItems, 0, legendHeight, this._width, this._height, gap, defaultspacing, false, legendHeight);
                }
                else if (layout == ej.datavisualization.TreeMap.ItemsLayoutMode.SliceAndDiceAuto) {
                    this._calculateSliceAndDiceItemSize(rootItems, 0, legendHeight, this._width, this._height, gap, defaultspacing, null, legendHeight);
                }
                else {
                    this._calculateSquarifiedItemSize(rootItems, 0, legendHeight, this._width, this._height, gap, defaultspacing, legendHeight);
                }
            
        },

      
        /**
         * Generates sub treemap items
         * @private
         */
        _generateSubItems: function (cloneItems, levelItem, subItems, PrevItem, disablelabels, disablebackground) {            
            var layout = this.model.itemsLayoutMode;
            var cloneItemsCount = cloneItems.length;
            var path = levelItem.groupPath;
            if (path == null) path = this.model.leafItemSettings.labelPath;
            if (path == null) path = this.weightValuePath();
            var gap = levelItem.groupGap != null ? levelItem.groupGap : 0;
            var headerHeight = ((PrevItem.showHeader || PrevItem.showHeader == null) && !this.model.enableDrillDown) ? PrevItem.headerHeight : 0;
            var totalItemsCount = 0;
            for (var j = 0; j < cloneItemsCount; j++) {
                var treeItems = this._getGroupitem(path, cloneItems[j].Data, levelItem.headerHeight);
                if (layout == ej.datavisualization.TreeMap.ItemsLayoutMode.SliceAndDiceHorizontal) {
                    this._calculateSliceAndDiceItemSize(treeItems, cloneItems[j].LeftPosition + (PrevItem.groupPadding), cloneItems[j].TopPosition + (PrevItem.groupPadding), cloneItems[j].ItemWidth - (PrevItem.groupBorderThickness + (2 * PrevItem.groupPadding)), cloneItems[j].ItemHeight - (PrevItem.groupBorderThickness + (2 * PrevItem.groupPadding)), gap, headerHeight, true, 0);
                }
                else if (layout == ej.datavisualization.TreeMap.ItemsLayoutMode.SliceAndDiceVertical) {
                    this._calculateSliceAndDiceItemSize(treeItems, cloneItems[j].LeftPosition + (PrevItem.groupPadding), cloneItems[j].TopPosition + (PrevItem.groupPadding), cloneItems[j].ItemWidth - (PrevItem.groupBorderThickness + (2 * PrevItem.groupPadding)), cloneItems[j].ItemHeight - (PrevItem.groupBorderThickness + (2 * PrevItem.groupPadding)), gap, headerHeight, false, 0);
                }
                else if (layout == ej.datavisualization.TreeMap.ItemsLayoutMode.SliceAndDiceAuto) {
                    this._calculateSliceAndDiceItemSize(treeItems, cloneItems[j].LeftPosition + (PrevItem.groupPadding), cloneItems[j].TopPosition + (PrevItem.groupPadding), cloneItems[j].ItemWidth - (PrevItem.groupBorderThickness + (2 * PrevItem.groupPadding)), cloneItems[j].ItemHeight - (PrevItem.groupBorderThickness + (2 * PrevItem.groupPadding)), gap, headerHeight, null, 0);
                }
                else {
                    this._calculateSquarifiedItemSize(treeItems, cloneItems[j].LeftPosition + (PrevItem.groupPadding), cloneItems[j].TopPosition + (PrevItem.groupPadding), cloneItems[j].ItemWidth - (PrevItem.groupBorderThickness + (2 * PrevItem.groupPadding)), cloneItems[j].ItemHeight - (PrevItem.groupBorderThickness + (2 * PrevItem.groupPadding)), gap, headerHeight, 0);
                }                
                var headerItem = this._rootTreeMapItems[j];                
                headerItem.headerHeight = PrevItem.headerHeight;
                headerItem.showHeader = PrevItem.showHeader;
                headerItem.headerWidth = cloneItems[j].ItemWidth;
                headerItem.headerTemplate = PrevItem.headerTemplate;
                headerItem.headerLeftPosition = cloneItems[j].LeftPosition;
                headerItem.headerTopPosition = cloneItems[j].TopPosition;
                if (PrevItem.showHeader || PrevItem.showHeader==null) {
						if (this.model.enableDrillDown)
                            this._createBackGround(headerItem, PrevItem, cloneItems[j], disablebackground);
                        else {
						    this._createBackGround(headerItem, PrevItem);
						    this._generateHeaders(headerItem, PrevItem);
						}
                }              
               
                cloneItems[j].ChildtreeMapItems = treeItems;
                cloneItems[j].GroupingLevel = this.model.levels.indexOf(PrevItem) + 1;
                for (var k = 0; k < treeItems.length; k++) {
                    subItems[totalItemsCount] = treeItems[k];
                    subItems[totalItemsCount].ParentHeader = headerItem.header;
                    subItems[totalItemsCount].ItemHeight -= PrevItem.groupPadding;
                    subItems[totalItemsCount].ItemWidth -= PrevItem.groupPadding;
                    subItems[totalItemsCount].LeftPosition += PrevItem.groupBorderThickness;
                    subItems[totalItemsCount].TopPosition += PrevItem.groupBorderThickness;
                    totalItemsCount++;
                }              
            }
            if (disablelabels == undefined) {
                this._generateLabels(cloneItems, PrevItem.labelTemplate, null, PrevItem.showLabels);
            }
        },        

        /**

        * create background for treemap items
        * @private
        */
        _createBackGround: function (header, level,parentitem, disablebackground) {            
            var rect = $("<div />");
            var height = (header.showHeader|| header.showHeader==null) ? header.ItemHeight - (2*level.groupBorderThickness) : 0;
            var top = (header.showHeader || header.showHeader == null) ? header.TopPosition : 0;
            rect.css({
                "height": height + "px",
                "width": header.ItemWidth - (2 * level.groupBorderThickness) + "px",
                "left": header.LeftPosition + "px",
                "top": top + "px",
                "border-style": "solid",
                "border-width": level.groupBorderThickness+"px",
                "border-color": level.groupBorderColor,
                "position": "absolute",
                "background-color": level.groupBackground
            });
            rect.appendTo(this._backgroundTile);
            var height = parseFloat(rect[0].style.top) - 30 + "px";
            if (this.model.enableDrillDown && !disablebackground) {
                var hoverrect = $("<div class='drillGroupRect'/>");
                hoverrect.css({
                    "height": rect[0].style.height,
                    "width": rect[0].style.width,
                    "left": rect[0].style.left,
                    "top": height,
                    "position": "absolute",
                    "opacity": 0.2,
                    "display": "block"
                });
                this._drillHoverDiv.append(hoverrect);
                $(hoverrect).mousedown({ treemap: this, level: level, param1: parentitem }, this._headerClickFunction);
                $(hoverrect).mouseleave({ treemap: this, param1: header.header }, this._mouseLeaveFunction);
                $(hoverrect).mousemove({ treemap: this, hoverItem: parentitem }, this._mouseRectHoverFunction);
            }
            $(rect).mousedown({treemap: this,level:level,param1:parentitem }, this._headerClickFunction);
            $(rect).mousemove({treemap: this,param1:header.header, hoverItem:parentitem }, this._mouseHoverFunction);
        },

        _mouseRectHoverFunction: function (event) {
          
            var treeMap = event.data.treemap;
            if (treeMap.model.showTooltip) {
                var element = treeMap._toolTipElement;
                var template = treeMap.model.tooltipTemplate;
                if (element != null && template != null) {
                    var g = $("#text");
                    g[0].innerHTML = event.pageX + 10;
                    $(element).css({ "left": event.pageX + 10, "top": event.pageY + 10, "display": "block" });
                    var htmlString = $("#" + template).render(event.data.hoverItem.Data[0]);
                    $(element).html(htmlString);
                    

                }
            }
        },
		
        _mouseHoverFunction: function (event) {
            var treeMap = event.data.treemap;
            if (treeMap.model.showTooltip) {
                var element = treeMap._toolTipElement;
                var template = treeMap.model.tooltipTemplate;
                if (element != null && template != null) {
                    $(element).css({ "left": event.pageX + 10, "top": event.pageY + 10, "display": "block" });
                    var htmlString = $("#" + template).render(event.data.hoverItem.Data[0]);
                    $(element).html(htmlString);
                   

                }
            }
        },

       
        _mouseLeaveFunction: function (event) {
            var treeMap = event.data.treemap;
            if (treeMap.model.showTooltip)
            {
                if (treeMap._toolTipElement != null) {
                    $(treeMap._toolTipElement).css("display", "none");
                }
            }
        },
		
        _headerClickFunction: function (event) {
            var level = event.data.level;
            var data = event.data.param1;
		    var ctrlkey = event.ctrlKey;
		    var treeMap = event.data.treemap;
		    if (treeMap.model.highlightGroupOnSelection) {
		        if (treeMap._browser != "msie") {
		            var lastItem = treeMap._backgroundTile[0].children[treeMap._backgroundTile[0].children.length - 0];
		            treeMap._backgroundTile[0].insertBefore(this, lastItem);
		        }
		        if (ctrlkey) {

		            if ($.inArray(this,treeMap._prevSelectedHeaders)==-1) {
		                $(this).css({ "border-width": treeMap.model.highlightGroupBorderThickness, "border-color": treeMap.model.highlightBorderBrush });
		                treeMap._prevSelectedHeaders.push(this);
		                treeMap._prevSelectedGroupDatas.push(event.data.Param1);
		            }
		            else {
                        
                        $(this).css({ "border-width": level.groupBorderThickness, "border-color": level.groupBorderColor });
                        var index = treeMap._prevSelectedHeaders.indexOf(this);
                        treeMap._prevSelectedHeaders.splice(index, 1);
                        treeMap._prevSelectedGroupDatas.splice(index, 1);
		            }
		        }
		        else {
		            for (var k = 0; k < treeMap._prevSelectedHeaders.length; k++) {
		               
		                $(treeMap._prevSelectedHeaders[k]).css({ "border-width": level.groupBorderThickness, "border-color": level.groupBorderColor });
		            }

		            if ($.inArray(this, treeMap._prevSelectedHeaders) == -1) {
		                $(this).css({ "border-width": treeMap.model.highlightGroupBorderThickness, "border-color": treeMap.model.highlightBorderBrush });
		                treeMap._prevSelectedHeaders = [];
		                treeMap._prevSelectedHeaders.push(this);
		            }
		            else {
		                treeMap._prevSelectedHeaders = [];
		            }
		            
		            treeMap._prevSelectedGroupDatas.push(event.data.Param1);
		        }
		    }

		    if (treeMap.model.enableDrillDown && treeMap.model.levels.length > 1 && ((treeMap._drilldownItem != null && treeMap._drilldownItem.header != data.header) || treeMap._drilldownItem == null)) {
                var header = "";
                for(var i=0;i< treeMap._drillHeaderDiv[0].children[0].children.length;i++)
		        {
                    if (treeMap._drillHeaderDiv[0].children[0].children[i].className == "header")
                        header = treeMap._drillHeaderDiv[0].children[0].children[i].innerHTML;
		        }
                treeMap._drillHeaderDiv[0].innerHTML = "";
                treeMap._templateDiv[0].innerHTML = "";
                treeMap._drillHoverDiv[0].innerHTML = "";
		        treeMap._drillDownHeader(header + "." + data.header,true);
		        treeMap._drilldownItem = data;
		        treeMap._drilldownItems.push(data);
		        treeMap._groupdrillDownLevels(data, treeMap._drilldownItems.length);
		        return false;
		       		        
		    }
		    treeMap._trigger("treeMapGroupSelected", { selectedGroups: treeMap._prevSelectedHeaders,orginalEvent:event });
			
		},

        /**
        * Generates labels for treemap items
        * @private
        */
        _generateLabels: function (Items,labelTemplate, itemTemplate, showLabels) {            
            for (var i = 0; i < Items.length; i++) {
                var item = Items[i];
                if(itemTemplate!=null) {
                    var OriginalNode = $("#" + itemTemplate);
                    var item_templateDiv = $("<div style='overflow:hidden;display:block;position:absolute;pointer-events: none;'></div>");
                    item_templateDiv[0].data = item;
                    var tmpl2 = $.templates(OriginalNode.html());
                    var htmlString = tmpl2.render(item);
                    $(item_templateDiv).html(htmlString);
                    $(item_templateDiv).css({
                        "left": item.LeftPosition + 10 +"px",
                        "top": item.TopPosition + 5 +"px"
                    });
                    this._templateDiv.append(item_templateDiv);
                    this._labelTemplateElements.push(item_templateDiv);
                }
				
				if (showLabels && labelTemplate == null) {                    

				    var textcon = $('<label style="font-size:14px;font-weight:normal;font-family:Segoe UI;">' + item.label + '</label>');
					if (this.model.enableDrillDown)
					{
						$(textcon).css({
							"position": "absolute","color":"black",
							"overflow": "hidden", "width": item.ItemWidth - 10 + "px", "left": item.LeftPosition + 10 + "px",
							"top": item.TopPosition + 5 + "px", "height": item.ItemHeight-10,"font-weight":"normal","pointer-events":"none"
						});
					}
					else
					{
					    $(textcon).css({
					        "position": "absolute", "color": "white",
					        "overflow": "hidden", "width": item.ItemWidth - 10 + "px", "left": item.LeftPosition + 10 + "px",
					        "top": item.TopPosition + 5 + "px", "height": item.ItemHeight - 10, "font-weight": "normal", "pointer-events": "none"
					    });
					}
                    this._treeMapLabels.push(textcon);
                    this._templateDiv.append(textcon);
                }            
                if(labelTemplate != null && showLabels) {
                    var OriginalNode = $("#" + labelTemplate);
                    var item_templateDiv = $("<div style='overflow:hidden;display:block;position:absolute;pointer-events: none;'></div>");
                    item_templateDiv[0].data = item;
                    var tmpl2 = $.templates(OriginalNode.html());
                    var htmlString = tmpl2.render(item);
                    $(item_templateDiv).html(htmlString);
                    $(item_templateDiv).css({
                        "left": item.LeftPosition + (item.ItemWidth / 2),
                        "top": item.TopPosition + (item.ItemHeight / 2)
                    });
                    this._templateDiv.append(item_templateDiv);
                    this._labelTemplateElements.push(item_templateDiv);
                }
            }
        },

        /**
        * Generates tooltip for treemap
        * @private
        */
        _generateToolTip: function () {

            if (this.model.showTooltip) {
                var tooltip = document.getElementsByClassName("tooltipelement");
                if (tooltip != null && tooltip.length == 0) {
                    var tooltip_templateDiv = $("<div class='tooltipelement' style='display:none;position:absolute;z-index:1000;pointer-events:none;'></div>");
                    $(document.body).append(tooltip_templateDiv);
                    this._toolTipElement = tooltip_templateDiv;
                }
                else
                {
                    this._toolTipElement = tooltip[0];
                }
                if (this.model.tooltipTemplate == null) {

                    var path;
                    if (this.model.leafItemSettings.labelPath!=null)
                        path = this.model.leafItemSettings.labelPath;
                    else 
                        path = this.weightValuePath();
                    if (path != null) {
                        this.model.tooltipTemplate = 'defaultTooltip';

                        this.element.append($('<div id="defaultTooltip" style="display:none;"><div style="margin-left:10px;margin-top:-25px;"><div class="defaultToolTip"><p style="margin-top:-4px"><label  style="color:rgb(27, 20, 20);font-size:14px;font-weight:normal;font-family:Segoe UI">{{:#data["' + path + '"]}}</label></p></div></div></div>'));
                    }
                }
            }
            

        },

        /**
        * Generates legend for treemap
        * @private
        */
        _generateLegend: function () {            
            if (this.model.rangeColorMapping != null) {
                var colormapping = this.model.rangeColorMapping;
                var xPos = 0;
                var yPos = 0;
                var legendSettings = this.model.legendSettings;
                var legendtype = legendSettings.template;
                for (var i = 0; i < colormapping.length; i++) {
                    var legendItem;

                    if (legendtype == "Ellipse") {
                        legendItem = this._getEllipseLegend(legendSettings, xPos, yPos);
                    }
                    else {
                        legendItem = this._getRectLegend(legendSettings, xPos, yPos);
                    }

                    $(legendItem).css("background-color", colormapping[i].color);
                    legendItem.appendTo(this._svgDocument);
                    xPos += legendSettings.iconWidth + 5;                   
                    var legendText = $("<div/>");
                    legendText.css({                        
                        "left": xPos + "px",
                        "top": yPos + "px",
                        "position": "absolute"
                    });                    
                    $(legendText).css({"font-size":"15px", "color":"gray"});
                    legendText[0].innerHTML= colormapping[i].legendLabel;
                    legendText.appendTo(this._svgDocument);
                    var bounds = legendText[0].getBoundingClientRect();
                    var boundwidth = window.SVGSVGElement ? bounds.width : bounds.right - bounds.left;
                    xPos += boundwidth + 18;

                }
            }
        },

        /**
        * Gets ellipse legend
        * @private
        */
        _getEllipseLegend: function (legengItem, xPos, yPos) {

            var rect = $("<div class='e-mapLegend'/>");
            rect.css({
                "height": this.model.legendSettings.iconHeight+ "px",
                "width": this.model.legendSettings.iconWidth + "px",
                "border-radius":this.model.legendSettings.iconHeight/2+"px",
                "left": xPos + "px",
                "top": yPos + "px",
                "position": "absolute"
            });
            return rect;

        },

        /**
        * Gets rect legend
        * @private
        */
        _getRectLegend: function (legengItem, xPos, yPos) {
            var rect = $("<div />");
            rect.css({
                "height": this.model.legendSettings.iconHeight+ "px",
                "width": this.model.legendSettings.iconWidth + "px",                
                "left": xPos + "px",
                "top": yPos + "px",
                "position": "absolute"
            });
            return rect;
        },

        /**
        * Gets position from parent
        * @private
        */
        _getPositionFromParent: function (element, parentelement) {
            var child = element.getBoundingClientRect();
            var parent = parentelement.getBoundingClientRect();
            var width = window.SVGSVGElement ? child.width: child.right-child.left;
            var height = window.SVGSVGElement ? child.height : child.bottom - child.top;            
            return { left: child.left - parent.left, top: child.top - parent.top, height: height, width: width };
        },

        /**
       * Renders labels
       * @private
       */
        _renderLabels: function () {

            for (var j = 0; j < this._labelTemplateElements.length; j++) {
                var label = this._labelTemplateElements[j];                
                var bounds = this._getPositionFromParent(label[0],this._templateDiv[0]);
                var item = label[0].data;
                var leftPos = $(label).css('left');
                var topPos = $(label).css('top');
                $(label).css({
                    "left": parseInt(leftPos) - (bounds.width / 2),
                    "top": parseInt(topPos) - (bounds.height / 2),
                    "pointer-events": "none"
                });
               

            }

        },

        /**
       * Generates headers
       * @private
       */
        _generateHeaders: function (headerValue,level) {           
            if (headerValue.headerTemplate == null) {
                var headerItem = $('<div style="display:block;position:absolute;pointer-events: stroke;overflow: hidden;"><label style="color:#262626;font-size:15px;font-weight:normal;font-family:Segoe UI;">' + headerValue.header + '</label></div>');
                $(headerItem).css({
                    "left": headerValue.headerLeftPosition+2,
                    "top": headerValue.headerTopPosition+2,
                    "width": headerValue.headerWidth - level.groupPadding,
                    "height": headerValue.headerHeight,
                    "margin-left": level.groupPadding                    
                });                
                this._templateDiv.append(headerItem);
            }
            else{
                var item_templateDiv = $("<div style='display:block;position:absolute;pointer-events: none;overflow: hidden;'></div>");
                this._templateDiv.append(item_templateDiv);
                $(item_templateDiv).css({
                    "left": headerValue.headerLeftPosition,
                    "width": headerValue.headerWidth, "top": headerValue.headerTopPosition,
                    "height": headerValue.headerHeight
                });
                var htmlString = $("#" + headerValue.headerTemplate).render(headerValue);
                $(item_templateDiv).html(htmlString);

            }
        },

        _drillDownHeader: function (header, enableNavigation) {
            var item_templateDiv = $("<div style='display:block;overflow: hidden;'><div class='header' style='margin-top:5px;height:30px;position:absolute;background-color: transparent'>" + header + "</div></div>");

            if (enableNavigation)
                item_templateDiv = $("<div style='display:block;overflow: hidden;'><svg style='margin-top:10px;width:15px;height:15px'><polyline points='8,0 8,10 0,5 8,0' style='fill:white;stroke:white'/></svg><div class='header' style='left:17px;top:0px;position:absolute;margin-top:6px;height:30px;position:absolute;background-color: transparent'  '>" + header + "</div></div>");

            $(item_templateDiv).css({
                "left": "0px",
                "width": this._width+"px", "background-color": this.model.drillDownHeaderColor, "top": 0,
                "height": "30px", "color":"white", "font-size": "15px","text-indent":"10px","cursor":"pointer"
            });           
            this._drillHeaderDiv.append(item_templateDiv);
            $(item_templateDiv).mousedown({ treemap: this }, this._drilldownfunction);
            
        },

        _drilldownfunction: function (event) {
            var treeMap = event.data.treemap;                       
            if (treeMap._drilldownItems.length > 0) {
                if (treeMap._drilldownItems.length == 1) {
                    treeMap.refresh();
                    treeMap._drilldownItems = [];
                }
                else {
                    var header = "";
                    for (var i = 0; i < treeMap._drillHeaderDiv[0].children[0].children.length; i++) {
                        if (treeMap._drillHeaderDiv[0].children[0].children[i].className == "header")
                            header = treeMap._drillHeaderDiv[0].children[0].children[i].innerHTML;
                    }
                    var headers = header.split(".");
                    treeMap._drillHeaderDiv[0].innerHTML = "";
                    var drillHeader = "";
                    for (i = 0; i < headers.length - 1; i++) {
                        drillHeader = drillHeader + headers[i];
                        if (i != headers.length - 2)
                            drillHeader = drillHeader + ".";
                    }

                    treeMap._drillDownHeader(drillHeader, true);
                    treeMap._drilldownItem = treeMap._drilldownItems[treeMap._drilldownItems.length - 2];
                    treeMap._drilldownItems.pop(treeMap._drilldownItems[treeMap._drilldownItems.length - 1]);
                    treeMap._templateDiv[0].innerHTML = "";
                    treeMap._groupdrillDownLevels(treeMap._drilldownItem, treeMap._drilldownItems.length);
                }
            }            
        },
       
        /**
          * Generates treemap items
          * @private
          */
        _generateTreeMapItems: function (Items, fill, path) {
            this._getTreeMapItemTemplate(Items);
            if (Items != null) {
                for (var i = 0; i < Items.length; i++) {
                    var item = Items[i];                    
                    var rect = this._svgDocument[0].childNodes[i];                    
                    var ColorValue = item._generateColorMappings(this.model);
                    if (ColorValue != null) {
                        $(rect).css("background-color", ColorValue);
                    }
                    else {
                        $(rect).css("background-color", "#e5e5e5");
                    }
                    if (item.Data != null && item.Data.length>0) {
                        item.Data = item.Data[0];
                    }
                    item.Rectangle = rect;
                    $(rect).mouseenter({ Param1: item, Param2: this }, this._rectMouseEnter);
                    $(rect).mousemove({ Param1: item, Param2: this}, this._rectMouseMove);
                    $(rect).mouseleave({ Param1: item, Param2: this }, this._rectMouseLeave);
                    $(rect).mouseup({ Param1: item, Param2: this }, this._rectMouseLeave);
                    $(rect).mousedown({ Param1: item.Data, treemap: this }, this._rectClickFunction);
                }
                if (this.model.rangeColorMapping.length == 0 && this.model.uniColorMapping.color == null && this.model.desaturationColorMapping.color != null) {
                    this._setDesaturationColor(Items, this.model.desaturationColorMapping);
                }
                else if (this.model.PaletteColorMapping != null) {
                    this._setPaletteColor(Items, this.model.PaletteColorMapping);
                }
            }
        },

        /**
          * Gets treemap item template
          * @private
          */
        _getTreeMapItemTemplate: function (Items) {            
            
            var rects ="";
            for (var i = 0; i < Items.length; i++) {
                var item = Items[i];
                rects += '<div style="border-style:solid;position:absolute;left:' + item.LeftPosition + 'px;top:' + item.TopPosition + 'px;height:' + item.ItemHeight + 'px;width:' + item.ItemWidth + 'px;border-width:' + this.model.leafItemSettings.borderThickness + 'px;border-color:' + this.model.leafItemSettings.borderBrush + '"></div>';
            }
            if (this._svgDocument != null && this.model.enableDrillDown) {
                this._svgDocument[0].innerHTML = "";
            }
            this._svgDocument = $('<div style= "overflow:hidden;' +
                              'z-index:1;"' +
                             'id="svgDocument">' + rects + '</div>');
            this._backgroundTile.appendTo(this.element);
            this._svgDocument.appendTo(this.element);           
            this._svgDocument.css({
                "height": this._height + "px", "width": this._width + "px", "overflow": "hidden",
                "z-index": 1, "position": "absolute", "left": "0", "top": "0"
            });
            $(this._backgroundTile).css({
                "height": this._height + "px", "width": this._width + "px", "overflow": "hidden",
                "z-index": 0, "position": "absolute", "left": "0", "top": "0"
            });
            this._templateDiv.appendTo(this.element);
            if (this.model.enableDrillDown) {
                this._drillHeaderDiv.appendTo(this.element);
                this._drillHoverDiv.appendTo(this.element);
            }
        },

        /**
         * Section For handling mouse click event for treemap item
         * @private
         */
        _rectClickFunction: function (event) {
            var ctrlkey = event.ctrlKey;
            var treeMap = event.data.treemap;

            if (treeMap.model.highlightOnSelection) {
                if (treeMap._browser != "msie") {
                    var lastItem = treeMap._svgDocument[0].children[treeMap._svgDocument[0].children.length - 1];
                    treeMap._svgDocument[0].insertBefore(this, lastItem);
                }
                if (ctrlkey) {
                    if ($.inArray(this,treeMap._prevSelectedItems)==-1) {
                        $(this).css({ "border-width": treeMap.model.highlightBorderThickness, "border-color": treeMap.model.highlightBorderBrush });
                        treeMap._prevSelectedItems.push(this);
                        treeMap._prevSelectedItemDatas.push(event.data.Param1);
                    }
                    else {
                        $(this).css({ "border-width": treeMap.model.leafItemSettings.borderThickness, "border-color": treeMap.model.leafItemSettings.borderBrush });
                        var index = treeMap._prevSelectedItems.indexOf(this);
                        treeMap._prevSelectedItems.splice(index, 1);
                        treeMap._prevSelectedItemDatas.splice(index,1);
                    }
                }
                else {
                    for (var k = 0; k < treeMap._prevSelectedItems.length; k++) {
                        $(treeMap._prevSelectedItems[k]).css({ "border-width": treeMap.model.leafItemSettings.borderThickness, "border-color": treeMap.model.leafItemSettings.borderBrush });
                    }

                    if ($.inArray(this, treeMap._prevSelectedItems)==-1) {
                        $(this).css({ "border-width": treeMap.model.highlightBorderThickness, "border-color": treeMap.model.highlightBorderBrush });
                        treeMap._prevSelectedItems = [];
                        treeMap._prevSelectedItems.push(this);
                    }
                    else {
                        treeMap._prevSelectedItems = [];
                    }                    
                    treeMap._prevSelectedItems.push(event.data.Param1);
                }
            }

            treeMap._trigger("treeMapItemSelected", {selectedItems:treeMap._prevSelectedItems,originalEvent:event});
        },
        /**
         * Section For handling mouse enter event for  treemap item
         * @private
         */
        _rectMouseEnter: function(event) {
            var tooltipObject = event.data.Param1.Data;
            if (tooltipObject != null) {
                var element = event.data.Param2._toolTipElement;
                var template = event.data.Param2.model.tooltipTemplate;
                if (element != null && template != null) {
                    $(element).css({ "left": event.pageX+10, "top": event.pageY+10, "display": "block" });
                    var htmlString = $("#" + template).render(tooltipObject);
                    $(element).html(htmlString);
                    
                }
            }
        },
        /**
         * Section For handling mouse move event for  treemap item
         * @private
         */
        _rectMouseMove: function (event) {
            var element = event.data.Param2._toolTipElement;
            var treeMap = event.data.Param2;
            if (treeMap.model.enableDrillDown && treeMap._browser != 'msie') {
                for (var i = 0; i < treeMap._drillHoverDiv[0].children.length; i++) {
                    var child = treeMap._drillHoverDiv[0].children[i];
                    if (child.className == event.data.Param1.ParentHeader) {
                        $(child).css({ "display": "block" });
                    }
                }
            }
                if (element != null) {
                    $(element).css({ "left": event.pageX+10, "top": event.pageY+10, "display": "block" });
                }
        },

        /**
        * Section For handling mouse leave event for  treemap item
        * @private
        */
        _rectMouseLeave: function (event) {            
            var element = event.data.Param2._toolTipElement;
            if (element != null) {
                $(element).css("display", "none");
            }
        },
		
		_docClickFunction: function(event) {
            var element = event.data.Param1._toolTipElement;
            if (element != null) {
                $(element).css("display", "none");
            }
        },

        /**
        * Sets desaturation color for treemap items
        * @private
        */
        _setDesaturationColor: function (Items, desaturationColorMapping) {
		    Items = Items.sort(this._orderBycolorWeight);
            var from = desaturationColorMapping.from;
            var to = desaturationColorMapping.to;
            if (from < 0 || from > 1)
                from = 1;
            if (to > 1 || to < 0)
                to = 0;

            var interval = (from - to) / Items.length;
            for (var i = 0; i < Items.length; i++) {
                $(Items[i].Rectangle).css("background-color", desaturationColorMapping.color);
                if (Items[i].colorWeight >= desaturationColorMapping.rangeMinimum && Items[i].colorWeight <= desaturationColorMapping.rangeMaximum) {
                    $(Items[i].Rectangle).css('opacity', from);
                    from = from - interval;
                }
            }
        },

        /**
        * Sets palette color for treemap items
        * @private
        */
        _setPaletteColor: function (Items, PaletteColors) {
            Items = Items.sort(this._orderBycolorWeight);
            for (var i = 0; i < Items.length; i++) {
                if (PaletteColors.colors.length > 0) {
                    Items[i].Rectangle.setAttribute("fill", PaletteColors.Colors[i % (PaletteColors.colors.length)]);
                }
            }
        },

        /**
        * Orders items by color weight
        * @private
        */
        _orderBycolorWeight: function (a, b) {
            if (a.colorWeight == b.colorWeight) {
                return 0;
            } else if (a.colorWeight < b.colorWeight) {
                return 1;
            }
            return -1;
        },
       /**
       * Orders items by area weight
       * @private
       */
        _orderByAreaWight: function (a, b) {
            if (a.AreaByWeight == b.AreaByWeight) {
                return 0;
            } else if (a.AreaByWeight < b.AreaByWeight) {
                return 1;
            }
            return -1;
        },

        /**
       * Calculates slice and dice item size 
       * @private
       */
        _calculateSliceAndDiceItemSize: function (Items, xPos, yPos, width, height, gap, headerHeight, isHorizontal,legendHeight) {

            var Gap = gap;
            var totalWeight = this._getTotalWeight(Items);
            if (isHorizontal == undefined) {
                isHorizontal = width > height ? true : false;
            }
            var AvailableSize = { "Width": width, "Height": height-legendHeight };
            var headerSize = headerHeight < AvailableSize.Height ? headerHeight : 0;
            AvailableSize.Height = AvailableSize.Height - headerSize;
            var AvailableArea = { "X": xPos, "Y": yPos + headerSize, "Width": AvailableSize.Width, "Height": AvailableSize.Height };
            var parentArea = AvailableSize.Height * AvailableSize.Width;
            var itemsCount = Items.length;
            if (isHorizontal) {
                var parentHeight = AvailableSize.Height;
                var allottedWidth = 0;
                for (var i = 0; i < itemsCount; i++) {
                    var childarea = (parentArea / totalWeight) * Items[i].weight;
                    var childWidth = childarea / parentHeight;
                    gap = (childWidth > Gap) ? Gap : 0;

                    if (allottedWidth <= AvailableSize.Width) {
                        Items[i].ItemWidth = (i != itemsCount - 1) ? childWidth - gap : childWidth;
                        Items[i].ItemHeight = parentHeight;
                        Items[i].LeftPosition = allottedWidth + xPos;
                        Items[i].TopPosition = yPos + headerHeight;
                        allottedWidth += childWidth;
                    }
                }
            }
            else {
                var parentWidth = AvailableSize.Width;
                var allottedHeight = 0;
                for (var i = 0; i < itemsCount; i++) {
                    var childarea = (parentArea / totalWeight) * Items[i].weight;
                    var childHeight = childarea / parentWidth;
                    gap = (childHeight > Gap) ? Gap : 0;
                    if (allottedHeight <= AvailableSize.Height) {
                        Items[i].ItemWidth = parentWidth;
                        Items[i].ItemHeight = (i != itemsCount - 1) ? childHeight - gap : childHeight;
                        Items[i].LeftPosition = xPos;
                        Items[i].TopPosition = allottedHeight + yPos + headerSize;
                        allottedHeight += childHeight;
                    }
                }
            }
        },

        /**
       * Calculates squarified item size
       * @private
       */
        _calculateSquarifiedItemSize: function (Items, xPos, yPos, width, height, Gap, headerHeight, legendHeight) {
            var totalweight = this._getTotalWeight(Items);
            var AvailableSize = { "Width": width, "Height": height-legendHeight };
            var headerSize = headerHeight < AvailableSize.Height ? headerHeight : 0;
            AvailableSize.Height = AvailableSize.Height - headerSize;
            for (var i = Items.length - 1; i >= 0; i--) {
                var item = Items[i];
                item["AreaByWeight"] = (AvailableSize.Height * AvailableSize.Width) * item.weight / totalweight;
                item.headerTopPosition = yPos;
            }
            var AvailableArea = { "X": xPos, "Y": yPos + headerSize, "Width": AvailableSize.Width, "Height": AvailableSize.Height };
            var OrderedItems = Items.sort(this._orderByAreaWight);
            var curX = 0, curY = 0;
            var j = 0;
            for (var i = 0; i < Items.length; i = j) {
                var firstTreemapItem = OrderedItems[i];
                weightObject = this._getGroupTotalWeight(firstTreemapItem, OrderedItems, AvailableArea, i);
                var GroupTotalWeight = weightObject.totalWeight;
                j = weightObject.index;
                var currentRect = {};
                var isHorizontal = (AvailableArea.Width > AvailableArea.Height) ? true : false;
                for (var k = i; k < j; k++) {
                    var item = OrderedItems[k];
                    var areaSum = GroupTotalWeight;
                    if (k == i) {
                        currentRect.X = AvailableArea.X;
                        currentRect.Y = AvailableArea.Y;
                        if (isHorizontal) {
                            currentRect.Width = areaSum / AvailableArea.Height;
                            currentRect.Height = AvailableArea.Height;
                            AvailableArea.X += currentRect.Width;
                            AvailableArea.Width = Math.max(0, AvailableArea.Width - currentRect.Width);
                        }
                        else {
                            currentRect.Width = AvailableArea.Width;
                            currentRect.Height = areaSum / AvailableArea.Width;
                            AvailableArea.Y += currentRect.Height;
                            AvailableArea.Height = Math.max(0, AvailableArea.Height - currentRect.Height);
                        }
                        curX = currentRect.X;
                        curY = currentRect.Y;
                    }
                    var rect = {};
                    if (OrderedItems.indexOf(item) != OrderedItems.length - 1) {
                        rect.X = 0;
                        rect.Y = 0;
                        rect.Width = (isHorizontal) ? currentRect.Width - Gap : item.AreaByWeight / currentRect.Height;
                        rect.Height = (isHorizontal) ? item.AreaByWeight / currentRect.Width : currentRect.Height - Gap;
                        if (j - k != 1) {
                            if (isHorizontal) {
                                rect.Height -= Gap;
                            }
                            else {
                                rect.Width -= Gap;
                            }
                        }
                    }
                    else {
                        rect.Width = (isHorizontal) ? currentRect.Width : item.AreaByWeight / currentRect.Height;
                        rect.Height = (isHorizontal) ? item.AreaByWeight / currentRect.Width : currentRect.Height;
                    }
                    item.ItemWidth = rect.Width;
                    item.ItemHeight = rect.Height;
                    item.LeftPosition = curX;
                    item.TopPosition = curY;
                    if (isHorizontal) {
                        curY += (j - k != 1) ? rect.Height + Gap : rect.Height;
                    }
                    else {
                        curX += (j - k != 1) ? rect.Width + Gap : rect.Width;
                    }
                }
            }
        },

        /**
      * Gets group total weight
      * @private
      */
        _getGroupTotalWeight: function (firstTreemapItem, OrderedItems, AvailableArea, j) {
            var GroupTotalWeight = 0;
            var GroupMaxAspectRatio = 0;
            for (; j < OrderedItems.length; j++) {
                var ShorterSideLength = this._getShortersideLength(AvailableArea.Width, AvailableArea.Height);
                var lastTreemapItem = OrderedItems[j];
                GroupTotalWeight += lastTreemapItem.AreaByWeight;
                var GroupWidth = GroupTotalWeight / ShorterSideLength;
                var firstitemheight = firstTreemapItem.AreaByWeight / GroupWidth;
                var lastitemheight = lastTreemapItem.AreaByWeight / GroupWidth;
                if (j == 0)
                    GroupMaxAspectRatio = this._aspectRatio(GroupWidth, ShorterSideLength);
                var TempAspectRatio = Math.max(this._aspectRatio(firstitemheight, GroupWidth), this._aspectRatio(lastitemheight, GroupWidth));
                if (GroupTotalWeight == lastTreemapItem.AreaByWeight || TempAspectRatio < GroupMaxAspectRatio) {
                    GroupMaxAspectRatio = TempAspectRatio;
                }
                else {
                    GroupTotalWeight -= lastTreemapItem.AreaByWeight;
                    GroupWidth = GroupTotalWeight / ShorterSideLength;
                    GroupMaxAspectRatio = Math.max(this._aspectRatio(firstitemheight, GroupWidth), this._aspectRatio(lastitemheight, GroupWidth));
                    break;
                }
            }
            return { totalWeight: GroupTotalWeight, index: j };
        },

        /**
      * Calculates aspect ratio
      * @private
      */
        _aspectRatio: function (x, y) {
            return (x > y) ? (x / y) : (y / x);
        },

        /**
     * Calculates shorter side length
     * @private
     */
        _getShortersideLength: function (width, height) {
            return width > height ? height : width;
        },

        /**
     * Calculates total weight
     * @private
     */

        _getTotalWeight: function (Items) {
            var total = 0;
            for (var i = 0; i < Items.length; i++) {
                total += parseFloat(Items[i].weight);
            }
            return total;

        },

        /**
     * Gets group item
     * @private
     */
        _getGroupitem: function (path, dataSource, headerHeight) {
            var obj = [];            
            if (dataSource != null && this.weightValuePath() != null) {                
                    for (var i in dataSource) {
                        var item = dataSource[i];
                        if (item.hasOwnProperty(path) ? true : this._containsProperty(item,path)) {
                            var itemValue = this._reflection(item, path);
                            if ($.inArray(itemValue, obj) == -1) {
                                obj.push(itemValue);
                            }
                        }
                    }                
                var objCollection = [];
                for (var i = 0; i < obj.length; i++) {
                    var Coll = {};
                    var groupObj = [];
                    var groupId = obj[i];
                    var colorWeight = 0;
                    var weight = 0;
                    var count = 0;
                    for (var j = 0; j < dataSource.length;j++) {
                        var item = dataSource[j];
                        var itemValue = this._reflection(item, path);
                        if (itemValue == groupId) {
                            var weightValue = this._reflection(item, this.weightValuePath());
                            var colorWeightValue = this._reflection(item, this.colorValuePath());
                            groupObj.push(item);
                            count++;
                            if (weightValue != null) {
                                weight += parseFloat(weightValue);
                            }
                            if (colorWeightValue != null) {
                                colorWeight += parseFloat(colorWeightValue);
                            }
                        }
                    }
                    
                    Coll["header"] = groupId;
                    Coll["data"] = groupObj;
                    Coll["weight"] = weight;
                    Coll["colorWeight"] = colorWeight;
                    Coll["headerHeight"] = headerHeight;
                    if (weight > 0) {
                        var newTreemapItem = new treeMapItem(Coll);
                        objCollection.push(newTreemapItem);
                    }
                }
                return objCollection;
            }
            else { return null; }
        },

     /**
     * Gets top group item
     * @private
     */
        _getTopGroupitem: function (path, dataSource) {            
            if (path == null) path = this.model.leafItemSettings.labelPath;
            if (path == null) path = this.weightValuePath();
            var obj = [];
            var Coll = {};
            var weight = 0, colorWeight = 0;
            var groupId="";
            var innerobjCollection = [];
            var itemColl = [];
            for (var j = 0; j < dataSource.length; j++) {
                var item = dataSource[j];
                var itemValue = this._reflection(item, path);
                var innerColl={};
                var weightValue = this._reflection(item, this.weightValuePath());
                var colorWeightValue = this._reflection(item, this.colorValuePath());      
                if (weightValue != null) {
                    weight += parseFloat(weightValue);
                }
                if (colorWeightValue != null) {
                    colorWeight += parseFloat(colorWeightValue);
                }
                itemColl.push(item);
                innerColl["weight"] = weightValue;
                innerColl["data"] = item;                
                innerColl["header"] = itemValue;
                innerColl["colorWeight"] = colorWeightValue;                                       
                var newinnerTreemapItem = new treeMapItem(innerColl);
                innerobjCollection.push(newinnerTreemapItem);
            }        
            Coll["header"] = groupId;
            Coll["data"] = itemColl;
            Coll["weight"] = weight;
            Coll["innerItem"] = innerobjCollection;
            Coll["colorWeight"] = colorWeight;                                   
            var treemapColl = [];
            var treeitem = new treeMapItem(Coll);
            treemapColl.push(treeitem);
            return  treemapColl;
        },         
                     
        /**
        * Checks whether property exists in object
        * @private
        */
        _containsProperty: function (object, propertyName) {
              for (var key in object)
              {
                  if(key == propertyName)
                  {
                      return true;
                  }
              }
              return false;
        },

        /**
        * Gets property from object
        * @private
        */
        _reflection: function (Source, Path) {
            var ShapeValues = Source;
            if (Path != null && Source != null) {
                var parts = Path.split(".");
                var i = 0;
                for (; i < parts.length; i++) {
                    var hasProperty = ShapeValues.hasOwnProperty(parts[i]) ? true : this._containsProperty(ShapeValues,parts[i]);
                    if (hasProperty) {
                        $.each(ShapeValues, function (prop, propval) {
                            if (prop == parts[i]) {
                                ShapeValues = propval;
                                return false;
                            }
                        });
                    }
                }
                return ShapeValues;
            }
            else {
                return null;
            }

        }

    });
    /**
    * Hold the Level settings of TreeMap
    *  @memberof ejTreeMap
    * @member
    * @instance
    */
    var treeMapLevel = function () {

        /** 
        * specifies the group background
			 * @default 'white'
             * @type {string} 
             * @example  
             * // Set the groupBackground during initialization. 			
             * 	$("#container").ejTreeMap({levels: [{ groupBackground:"white" }]})
             * @example 
             * //Get or set the groupBackground after initialization:<br>
             *   //Gets the groupBackground from map.
             *   var property =$("#container").data("ejTreeMap").model.levels[levelIndex].groupBackground;
             *   //Sets the groupBackground to map.
             *   $("#container").data("ejTreeMap").model.levels[levelIndex].groupBackground  = "white";
             * @alias ejTreeMap#treeMapLevel->groupBackground
             */
        this.groupBackground = 'white';

            /** 
            * Specifies the group border color for tree map level.
			 * @default '#58585B'
             * @type {string} 
             * @example  
             * // Set the groupBorderColor during initialization. 			
             * 	$("#container").ejTreeMap({levels: [{ groupBorderColor:"#58585B" }]})
             * @example 
             * //Get or set the groupBorderColor after initialization:<br>
             *   //Gets the groupBorderColor from map.
             *   var property =$("#container").data("ejTreeMap").model.levels[levelIndex].groupBorderColor;
             *   //Sets the groupBorderColor to map.
             *   $("#container").data("ejTreeMap").model.levels[levelIndex].groupBorderColor  = "#58585B";
             * @alias ejTreeMap#treeMapLevel->groupBorderColor
             */
        this.groupBorderColor = '#58585B';

        /** 
        * Specifies the group border thickness for tree map level.
			 * @default 1
             * @type {number} 
             * @example  
             * // Set the groupBorderThickness during initialization. 			
             * 	$("#container").ejTreeMap({levels: [{ groupBorderThickness:1 }]})
             * @example 
             * //Get or set the groupBorderThickness after initialization:<br>
             *   //Gets the groupBorderThickness from map.
             *   var property =$("#container").data("ejTreeMap").model.levels[levelIndex].groupBorderThickness;
             *   //Sets the groupBorderThickness to map.
             *   $("#container").data("ejTreeMap").model.levels[levelIndex].groupBorderThickness  = 1;
             * @alias ejTreeMap#treeMapLevel->groupBorderThickness
             */
        this.groupBorderThickness = 1;

/** 
        * Specifies the group padding for tree map level.
			 * @default 4
             * @type {number} 
             * @example  
             * // Set the groupPadding during initialization. 			
             * 	$("#container").ejTreeMap({levels: [{ groupPadding:4 }]})
             * @example 
             * //Get or set the groupPadding after initialization:<br>
             *   //Gets the groupPadding from map.
             *   var property =$("#container").data("ejTreeMap").model.levels[levelIndex].groupPadding;
             *   //Sets the groupPadding to map.
             *   $("#container").data("ejTreeMap").model.levels[levelIndex].groupPadding  = 4;
             * @alias ejTreeMap#treeMapLevel->groupPadding
             */
        this.groupPadding = 4;

        /** 
        * Specifies the group path for tree map level.
             * @type {string} 
             * @example  
             * // Set the groupPath during initialization. 			
             * 	$("#container").ejTreeMap({levels: [{ groupPath:"pathName" }]})
             * @example 
             * //Get or set the groupPath after initialization:<br>
             *   //Gets the groupPath from map.
             *   var property =$("#container").data("ejTreeMap").model.levels[levelIndex].groupPath;
             *   //Sets the groupPath to map.
             *   $("#container").data("ejTreeMap").model.levels[levelIndex].groupPath  = "pathName";
             * @alias ejTreeMap#treeMapLevel->groupPath
             */
        this.groupPath = null;

        /**
        * Specifies the group gap for tree map level.
			 * @default 1
             * @type {number} 
             * @example  
             * // Set the groupGap during initialization. 			
             * 	$("#container").ejTreeMap({levels: [{ groupGap:1 }]})
             * @example 
             * //Get or set the groupGap after initialization:<br>
             *   //Gets the groupGap from map.
             *   var property =$("#container").data("ejTreeMap").model.levels[levelIndex].groupGap;
             *   //Sets the groupGap to map.
             *   $("#container").data("ejTreeMap").model.levels[levelIndex].groupGap  = 1;
             * @alias ejTreeMap#treeMapLevel->groupGap
             */
        this.groupGap = 1;

        /**
        * Specifies the header height for tree map level.
			 * @default 0
             * @type {number} 
             * @example  
             * // Set the headerHeight during initialization. 			
             * 	$("#container").ejTreeMap({levels: [{ headerHeight:20 }]})
             * @example 
             * //Get or set the headerHeight after initialization:<br>
             *   //Gets the headerHeight from map.
             *   var property =$("#container").data("ejTreeMap").model.levels[levelIndex].headerHeight;
             *   //Sets the headerHeight to map.
             *   $("#container").data("ejTreeMap").model.levels[levelIndex].headerHeight  = 1;
             * @alias ejTreeMap#treeMapLevel->headerHeight
             */
        this.headerHeight = 20;

        /** 
        * Shows or hides the header for tree map level.
            * @default false
            * @type {bool} 
            * @example  
            * // Set the shoeHeader during initialization. 			
            * 	$("#container").ejTreeMap({levels: [{ showHeader:false }]})
            * @example 
            * //Get or set the showHeader after initialization:<br>
            *   //Gets the showHeader from treemap.
            *   var property =$("#container").data("ejTreeMap").model.levels[levelIndex].showHeader;
            *   //Sets the headerHeight to map.
            *   $("#container").data("ejTreeMap").model.levels[levelIndex].showHeader  = true;
            * @alias ejTreeMap#treeMapLevel->showHeader
            */
        this.showHeader = true;

        /**
        * Shows or hides the labels for tree map level.
             * @default false
             * @type {boolean} 
             * @example  
             * // Set the showLabels during initialization. 			
             * 	$("#container").ejTreeMap({levels: [{ showLabels:false }]})
             * @example 
             * //Get or set the showLabels after initialization:<br>
             *   //Gets the showLabels from map.
             *   var property =$("#container").data("ejTreeMap").model.levels[levelIndex].showLabels;
             *   //Sets the showLabels to map.
             *   $("#container").data("ejTreeMap").model.levels[levelIndex].showLabels  = false;
             * @alias ejTreeMap#treeMapLevel->showLabels
             */
        this.showLabels = false;

        /** 
        * Specifies the header template for tree map level.
         * @default null
             * @type {string} 
             * @example  
             * // Set the headerTemplate during initialization. 			
             * 	$("#container").ejTreeMap({levels: [{ headerTemplate:"template" }]})
             * @example 
             * //Get or set the headerTemplate after initialization:<br>
             *   //Gets the headerTemplate from map.
             *   var property =$("#container").data("ejTreeMap").model.levels[levelIndex].headerTemplate;
             *   //Sets the headerTemplate to map.
             *   $("#container").data("ejTreeMap").model.levels[levelIndex].headerTemplate  = "template";
             * @alias ejTreeMap#treeMapLevel->headerTemplate
             */
        this.headerTemplate = null;
        /**
        * Specifies the label template for tree map level.
        * @default null
             * @type {string} 
             * @example  
             * // Set the labelTemplate during initialization. 			
             * 	$("#container").ejTreeMap({levels: [{ labelTemplate:"template" }]})
             * @example 
             * //Get or set the labelTemplate after initialization:<br>
             *   //Gets the labelTemplate from map.
             *   var property =$("#container").data("ejTreeMap").model.levels[levelIndex].labelTemplate;
             *   //Sets the labelTemplate to map.
             *   $("#container").data("ejTreeMap").model.levels[levelIndex].labelTemplate  = "template";
             * @alias ejTreeMap#treeMapLevel->labelTemplate
             */
        this.labelTemplate = null;
        this.treeMapItems = [];
    };

    treeMapLevel.prototype = {

    };

    var treeMapItem = function (params) {
        this.weight = params.weight;
        this.colorWeight = params.colorWeight;
        this.Data = params.data;
        this.headerHeight = params.headerHeight;
        this.header = params.header;
        this.label = params.header;
        this.headerLeftPosition = 0;
        this.headerTopPosition = 0;
        this.innerItems = params.innerItem;
        this.headerWidth = 0;
        this.headerTemplate = null;

    };

    treeMapItem.prototype = {
        /**
        * Generates color mapping for treemap items
        * @private
        */
        _generateColorMappings: function (colorMappings) {
            if (colorMappings.rangeColorMapping.length>0) {
                return this._getRangeBrushColor(colorMappings.rangeColorMapping);
            }
            if (colorMappings.uniColorMapping.color != null) {
                return this._getUniColor(colorMappings.uniColorMapping);
            }
        },

        /**
        * Get unicolor for treemap items
        * @private
        */
        _getUniColor: function (uniColorMapping) {

            return uniColorMapping.color;
        },

        /**
        * Get range brush color for treemap items
        * @private
        */
        _getRangeBrushColor: function (rangeColorMapping) {
            for (var j = 0; j < rangeColorMapping.length; j++) {
                var rangeBrush = rangeColorMapping[j];
                if (this.colorWeight >= rangeBrush.from && this.colorWeight <= rangeBrush.to) {
                    return rangeBrush.color;
                }
            }
        }

    };
	
	/**
    * Enum for TreeMap items layout mode
    * @enum {string}
    * @global 
    */
	ej.datavisualization.TreeMap.ItemsLayoutMode = {
			Squarified: "squarified",
			SliceAndDiceHorizontal: "sliceanddicehorizontal",
			SliceAndDiceVertical: "sliceanddicevertical",
			SliceAndDiceAuto: "sliceanddiceauto"
	};  
	
	ej.datavisualization.TreeMap.SelectionMode={
		Default: "default",
		Multiple: "multiple"
	};
	
	ej.datavisualization.TreeMap.GroupSelectionMode={
		Default: "default",
		Multiple: "multiple"
	};
})(jQuery, Syncfusion);
;
/**
* @fileOverview Plugin to style the Barcode elements
* @copyright Copyright Syncfusion Inc. 2001 - 2014. All rights reserved.
* Use of this code is subject to the terms of our license.
* A copy of the current license can be obtained at any time by e-mailing
* licensing@syncfusion.com. Any infringement will be prosecuted under
* applicable laws.
* @version 12.1
* @author <a href="mailto:licensing@syncfusion.com">Syncfusion Inc</a>
*/

(function ($, ej, undefined) {
    /** ejBarcode is the plugin name.*/
    /** "ej.Barcode" is "namespace.className" will hold functions and properties.*/
    /**
     * @namespace ej
     * @class ejBarcode
     * @requires jQuery
     * @requires ej.common.all
     * @classdesc Custom Design for Barcode control.
     * @example 
     * &lt;barcode id="barcode1"&gt;Barcode&lt;/barcode&gt; <br> 
     * &lt;script&gt;
     * // Create Barcode
     * $('#barcode1').ejBarcode();         
     * &lt;/script&gt;
     */

    ej.widget("ejBarcode", "ej.Barcode", {
        /** Widget element will be automatically set in this*/
        element: {
            target: null
        },

        /** User defined model will be automatically set in this*/
        model: null,
        validTags: ["div", "span"],
        _rootCSS: "e-barcode",

        defaults: {
            /**		
			* Specifies whether to display original text. Applicable only for One dimensional barcode.
			* @default true
			* @type {boolean}
            * @memberof ejBarcode
			* @example 
			* //To set displayText API value during initialization  
			* $("#barcode").ejBarcode({ displayText: true});
			* @example 
			* //Get or set the displayText API, after initialization:<br>
			* //Gets the displayText value  
			* $("#barcode1").ejBarcode("option", "displayText");<br>
			* //Sets the displayText value 
			* $("#barcode1").ejBarcode("option", "displayText", true ); 
			* @instance
			*/
            displayText: true,
            /**		
			* Specifies the text to encode.
			* @default empty
			* @type {string}
            * @memberof ejBarcode
			* @example 
			* //To set text value during initialization  
			* $("#barcode1").ejBarcode({ text:"SYNCFUSION" });
			* @example 
			* //Get or set the text API, after initialization:<br>
			* //Gets the text value  
			* $("#barcode1").ejBarcode("option", "text");<br>
			* //Sets the text value 
			* $("#barcode1").ejBarcode("option", "text", "SYNCFUSION" ); 
			* @instance
			*/
            text: "",
            /**                
             * Specifies the symbology type of the Barcode.        See {@link SymbologyType}
             * @default ej.barcode.SymbologyType.qrbarcode
             * @type {enum}
             * @memberof ejBarcode
             * @example 
             * //To set symbology type API value during initialization  
             * $("#barcode1").ejBarcode({ symbologyType: "code39" });
             * @example 
             * //Get or set the symbology type API, after initialization:<br>
             * //Gets the symbology type value  
             * $("#barcode1").ejBarcode("option", "symbologyType");<br>          
             * //Sets the symbology type value 
             * $("#barcode1").ejBarcode("option", "symbologyType", "code39" ); 
             * @instance
             */
            symbologyType: "qrbarcode",
            /**                
             * Specifies the display text color of the Barcode. Applicable only for One dimensional barcode.
             * @default black
             * @type {object}
             * @memberof ejBarcode
             * @example 
             * //To set text color API value during initialization  
             * $("#barcode1").ejBarcode({ textColor: "black" });
             * @example 
             * //Get or set the text color API, after initialization:<br>
             * //Gets the text color value  
             * $("#barcode1").ejBarcode("option", "textColor");<br>
             * //Sets the text color value 
             * $("#barcode1").ejBarcode("option", "textColor", "black" ); 
             * @instance
             */
            textColor: 'black',
            /**                
             * Specifies the light bar color of the Barcode. Applicable only for One dimensional barcode.
             * @default white
             * @type {object}
             * @memberof ejBarcode
             * @example 
             * //To set light bar color API value during initialization  
             * $("#barcode1").ejBarcode({ lightBarColor: "white" });
             * @example 
             * //Get or set the light bar color API, after initialization:<br>
             * //Gets the light bar color value  
             * $("#barcode1").ejBarcode("option", "lightBarColor");<br>
             * //Sets the light bar color value 
             * $("#barcode1").ejBarcode("option", "lightBarColor", "white" ); 
             * @instance
             */
            lightBarColor: 'white',
            /**                
             * Specifies the dark bar color of the Barcode. Applicable only for One dimensional barcode.
             * @default black
             * @type {object}
             * @memberof ejBarcode
             * @example 
             * //To set dark bar color API value during initialization  
             * $("#barcode1").ejBarcode({ darkBarColor: "black" });
             * @example 
             * //Get or set the dark bar color API, after initialization:<br>
             * //Gets the dark bar color value  
             * $("#barcode1").ejBarcode("option", "darkBarColor");<br>
             * //Sets the dark bar color value 
             * $("#barcode1").ejBarcode("option", "darkBarColor", "black" ); 
             * @instance
             */
            darkBarColor: 'black',
            /**                
             * The blank margin on the side(s) which denotes the reader with the start and stop of the Barcode.
             * @type {object}
             * @memberof ejBarcode
             * @instance
             */
            quietZone: {
                /**                
                 * Specifies the left quiet zone of the Barcode.
                 * @default 1
                 * @type {number}
                 * @example 
                 * //To set the quiet zone API value during initialization  
                 * $("#barcode1").ejBarcode({ quietZone: { left: 5} });
                 * @example 
                 * //Get or set the quietZone API, after initialization:<br>
                 * //Gets the quiet zone value  
                 * $("#barcode1").ejBarcode("option", "quietZone.left");<br>                        
                 * //Sets the quiet zone value 
                 * $("#barcode1").ejBarcode("option", "quietZone", {left: 5} ); 
                 * @alias ejBarcode#quietZone->left
                 * @instance
                 */
                left: 1,
                /**                
                 * Specifies the top quiet zone of the Barcode.
                 * @default 1
                 * @type {number}
                 * @example 
                 * //To set the quiet zone API value during initialization  
                 * $("#barcode1").ejBarcode({ quietZone: { top: 5} });
                 * @example 
                 * //Get or set the quietZone API, after initialization:<br>
                 * //Gets the quiet zone value  
                 * $("#barcode1").ejBarcode("option", "quietZone.top");<br>                   
                 * //Sets the quiet zone value 
                 * $("#barcode1").ejBarcode("option", "quietZone", {top: 5} ); 
                 * @alias ejBarcode#quietZone->top
                 * @instance
                 */
                top: 1,
                /**                
                 * Specifies the right quiet zone of the Barcode.
                 * @default 1
                 * @type {number}
                 * @example 
                 * //To set the quiet zone API value during initialization  
                 * $("#barcode1").ejBarcode({ quietZone: { right: 5} });
                 * @example 
                 * //Get or set the quietZone API, after initialization:<br>
                 * //Gets the quiet zone value  
                 * $("#barcode1").ejBarcode("option", "quietZone.right");<br>                      
                 * //Sets the quiet zone value 
                 * $("#barcode1").ejBarcode("option", "quietZone", {right: 5} ); 
                 * @alias ejBarcode#quietZone->right
                 * @instance
                 */
                right: 1,
                /**                
                 * Specifies the bottom quiet zone of the Barcode.
                 * @default 1
                 * @type {number}
                 * @example 
                 * //To set the quiet zone API value during initialization  
                 * $("#barcode1").ejBarcode({ quietZone: { bottom: 5} });
                 * @example 
                 * //Get or set the quietZone API, after initialization:<br>
                 * //Gets the quiet zone value  
                 * $("#barcode1").ejBarcode("option", "quietZone.bottom");<br>                     
                 * //Sets the quiet zone value 
                 * $("#barcode1").ejBarcode("option", "quietZone", {bottom: 5} ); 
                 * @alias ejBarcode#quietZone->bottom
                 * @instance
                 */
                bottom: 1,
                /**                
                 * Specifies the quiet zone around the Barcode.
                 * @default 1
                 * @type {number}
                 * @example 
                 * //To set the quiet zone API value during initialization  
                 * $("#barcode1").ejBarcode({ quietZone: { all: 5} });
                 * @example 
                 * //Get or set the quietZone API, after initialization:<br>
                 * //Gets the quiet zone value  
                 * $("#barcode1").ejBarcode("option", "quietZone.all");<br>                 
                 * //Sets the quiet zone value 
                 * $("#barcode1").ejBarcode("option", "quietZone", {all: 5} ); 
                 * @alias ejBarcode#quietZone->all
                 * @instance
                 */
                all: 1
            },
            /**                
             * Specifies the width of narrow bar in the Barcode. Applicable only for One dimensional barcode.
             * @default 1
             * @type {number}
             * @memberof ejBarcode
             * @example 
             * //To set narrowBarWidth API value during initialization  
             * $("#barcode1").ejBarcode({ narrowBarWidth: 1 });
             * @example 
             * //Get or set the narrowBarWidth API, after initialization:<br>
             * //Gets the width of the narrow bar value  
             * $("#barcode1").ejBarcode("option", "narrowBarWidth");<br>
             * //Sets the width of the narrow bar value 
             * $("#barcode1").ejBarcode("option", "narrowBarWidth", 1 ); 
             * @instance
             */
            narrowBarWidth: 1,
            /**                
             * Specifies the width of wide bar in the Barcode. Applicable only for One dimensional barcode.
             * @default 3
             * @type {number}
             * @memberof ejBarcode
             * @example 
             * //To set wideBarWidth API value during initialization  
             * $("#barcode1").ejBarcode({ wideBarWidth: 1 });
             * @example 
             * //Get or set the wideBarWidth API, after initialization:<br>
             * //Gets the width of the wide bar value  
             * $("#barcode1").ejBarcode("option", "wideBarWidth");<br>
             * //Sets the width of the wide bar value 
             * $("#barcode1").ejBarcode("option", "wideBarWidth", 3 ); 
             * @instance
             */
            wideBarWidth: 3,
            /**                
             * Specifies the height of bars in the Barcode. Applicable only for One dimensional barcode.
             * @default 150
             * @type {number}
             * @memberof ejBarcode
             * @example 
             * //To set barHeight API value during initialization  
             * $("#barcode1").ejBarcode({ barHeight: 150 });
             * @example 
             * //Get or set the barHeight API, after initialization:<br>
             * //Gets the height of the bars  
             * $("#barcode1").ejBarcode("option", "barHeight");<br>
             * //Sets the height of the bars 
             * $("#barcode1").ejBarcode("option", "barHeight", 150 ); 
             * @instance
             */
            barHeight: 150,
            /**                
             * Specifies the distance between the bar and text in the Barcode. Applicable only for One dimensional barcode.
             * @default 10
             * @type {number}
             * @memberof ejBarcode
             * @example 
             * //To set barcodeToTextGapHeight API value during initialization  
             * $("#barcode1").ejBarcode({ barcodeToTextGapHeight: 10 });
             * @example 
             * //Get or set the barcodeToTextGapHeight API, after initialization:<br>
             * //Gets the gap distance between the bar and the text
             * $("#barcode1").ejBarcode("option", "barcodeToTextGapHeight");<br>
             * //Sets the gap distance between the bar and the text
             * $("#barcode1").ejBarcode("option", "barcodeToTextGapHeight", 10 ); 
             * @instance
             */
            barcodeToTextGapHeight: 10,
            /**                
             * Specifies the size of each block in the Barcode. Applicable only for Two dimensional barcode.
             * @default 4
             * @type {number}
             * @memberof ejBarcode
             * @example 
             * //To set xDimension API value during initialization  
             * $("#barcode1").ejBarcode({ xDimension: 4 });
             * @example 
             * //Get or set the xDimension API, after initialization:<br>
             * //Gets the size of each block
             * $("#barcode1").ejBarcode("option", "xDimension");<br>
             * //Sets the size of each block
             * $("#barcode1").ejBarcode("option", "xDimension", 4 ); 
             * @instance
             */
            xDimension: 4,
            /**                
             * Specifies whether to encode start and stop symbol in the Barcode. Applicable only for One dimensional barcode.
             * @default 10
             * @type {number}
             * @memberof ejBarcode
             * @example 
             * //To set encodeStartStopSymbol API value during initialization  
             * $("#barcode1").ejBarcode({ encodeStartStopSymbol: true });
             * @example 
             * //Get or set the encodeStartStopSymbol API, after initialization:<br>
             * //Gets the encodeStartStopSymbol value
             * $("#barcode1").ejBarcode("option", "encodeStartStopSymbol");<br>
             * //Sets the whether to encode start and stop symbol value
             * $("#barcode1").ejBarcode("option", "encodeStartStopSymbol", true ); 
             * @instance
             */
            encodeStartStopSymbol: true,
            /**     
             * Fires after Barcode control is loaded.
             * @event
             * @memberof ejBarcode
             * @param {Object} argument Event parameters from barcode
             * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
             * @param {object}  argument.model returns the barcode model
             * @param {string}  argument.type returns the name of the event
             * @param {boolean}  argument.status return the barcode state
             * @example 
             * //create event for barcode
                         * $("#barcode1").ejBarcode({
                         *    load: function (args) {}
                         * });
             * @example 
             * //Bind create event using jquery "on"
                         * $("#barcode1").on("ejBarcodeload", function(e) {} );
             * @instance
             */
            load: null,
            /**		
            * Specifies whether the control is enabled.
            * @default true
            * @type {boolean}
            * @memberof ejBarcode
            * @example 
            * //To set enabled API value during initialization  
            * $("#barcode").ejBarcode({ enabled: true});
            * @example 
            * //Get or set the enabled API, after initialization:<br>
            * //Gets the enabled value  
            * $("#barcode1").ejBarcode("option", "enabled");<br>
            * //Sets the enabled value 
            * $("#barcode1").ejBarcode("option", "enabled", true ); 
            * @instance
            */
            enabled: true,
        },

        /**
         * Specify the data types for default properties 
         * @private
         */
        dataTypes: {
            displayText: "boolean",
            text: "string",
            enabled: "boolean",
            symbologyType: "enum",
            narrowBarWidth: "number",
            wideBarWidth: "number",
            barHeight: "number",
            encodeStartStopSymbol: "boolean"
        },

        /**
         * @private
         */
        _defaultScaleValues: function () {
            return {
                enabled: true,
                displayText: true,
                text: "",
                symbologyType: "qrbarcode",
                textColor: 'black',
                lightBarColor: 'white',
                darkBarColor: 'black',
                quietZone: { left: 1, top: 1, right: 1, bottom: 1 },
                narrowBarWidth: 1,
                wideBarWidth: 3,
                barHeight: 150,
                barcodeToTextGapHeight: 10,
                xDimension: 4,
                encodeStartStopSymbol: true
            };
        },

        /**
         * @constructor
         * @private
         */
        _init: function () {
            this._initialize();
            this._renderBarcode();
        },

        /**
         * @private
         */
        _destroy: function () {
            this.element.empty();
        },

        /**
        * To configure the properties at runtime using SetModel		
        * @private
        */
        _setModel: function (options) {
            var option;
            for (option in options) {
                switch (option) {
                    case "enabled": this._disabled(!options[option]); break;
                    case "height": this.model.height = options[option]; break;
                    case "width": this.model.width = options[option]; break;
                    case "lightBarColor": this.model.lightBarColor = options[option]; break;
                    case "darkBarColor": this.model.darkBarColor = options[option]; break;
                    case "textColor": this.model.textColor = options[option]; break;
                    case "displayText": this.model.displayText = options[option]; break;
                    case "quietZone": this.model.quietZone = options[option]; break;
                    case "narrowBarWidth": this.model.narrowBarWidth = options[option]; break;
                    case "wideBarWidth": this.model.wideBarWidth = options[option]; break;
                    case "barHeight": this.model.barHeight = options[option]; break;
                    case "xDimension": this.model.xDimension = options[option]; break;
                    case "encodeStartStopSymbol": this.model.encodeStartStopSymbol = options[option]; break;
                    case "symbologyType":
                        this.model.symbologyType = options[option];
                        this._itemInitialize();
                        break;
                    case "text":
                        this.model.text = options[option];
                        this._itemInitialize();
                        break;
                }
            }
            this._renderBarcode();
        },

        /**
         * Initializes control.
         * @private
         */
        _initialize: function () {
            this.BarcodeEl = this.element;
            this.target = this.element[0];
            this._itemInitialize();
            this.Model = this.model;
        },

        /**
         * Renders barcode.
         * @private
         */
        _renderBarcode: function () {
            this.initialize();
        },

        /**
         * @private
         */
        _itemInitialize: function () {
            var proxy = this;
            if (this.model.scales != null) {
                $.each(this.model.scales, function (index, element) {
                    element = proxy._checkArrayObject(element, index);
                    var obj = proxy._defaultScaleValues();
                    $.extend(obj, element);
                    $.extend(element, obj);
                });
            }
            else {
                this.model.scales = [this._defaultScaleValues()];
            }
        },

        /**
         * @private
         */
        _checkArrayObject: function (element, initialName) {
            var proxy = this;
            $.each(element, function (name, innerElement) {
                if (innerElement instanceof Array) {
                    innerElement = proxy._checkArrayObject(innerElement, name);
                }
                else if (innerElement != null && typeof innerElement == "object") {
                    var allObjects = proxy._defaultScaleValues();
                    innerElement = proxy._LoadIndividualDefaultValues(innerElement, allObjects, (typeof name === "number") ? initialName : name);
                }
            });
            return element;
        },

        /**
         * @private
         */
        _LoadIndividualDefaultValues: function (obj, allObjects, name) {
            var defaultObj = null;
            var proxy = this;
            $.each(allObjects, function (n, element) {
                if (name == n) {
                    defaultObj = element;
                    return;
                }
            });
            if (defaultObj instanceof Array) defaultObj = defaultObj[0];

            $.each(obj, function (objName, Ele) {
                if (Ele instanceof Array) {
                    Ele = proxy._checkArrayObject(Ele, name);
                }
                else if (Ele != null && typeof Ele == "object") {
                    Ele = proxy._LoadIndividualDefaultValues(Ele, defaultObj, (typeof objName === "number") ? name : objName);
                }
            });

            $.extend(defaultObj, obj);
            $.extend(obj, defaultObj);
            return obj;
        },

        /**
         * Initializes the control.
         * @private
         */
        initialize: function () {
            if (!this.model.enabled)
                return;

            this._initObject(this);

            if (this.Model.text != null)
                this._findBarcodeType();
        },

        /**
        * To enable the barcode 		
		* @return jQuery
		* @example 
		* &lt;barcode id="barcode1"&gt;Barcode&lt;/barcode&gt; <br> 
		* &lt;script&gt;
		* // Create Barcode
		* var btnObj = $("#barcode1").data("ejBarcode");
		* btnObj.enable(); // enable the barcode
		* &lt;/script&gt;
		* @example 
		* &lt;barcode id="barcode1"&gt;Barcode&lt;/barcode&gt; <br> 
		* &lt;script&gt;
		* // enable the barcode
		* $("#barcode1").ejBarcode("enable");	
		* &lt;/script&gt;
		* @memberof ejBarcode
		* @instance
         */
        enable: function () {
            if (!this.model.enabled) {
                this.element.removeClass("e-disable");
                this.model.enabled = true;
            }
        },

        /**
        * To disable the barcode  
		* @return jQuery
		* @example 
		* &lt;barcode id="barcode1"&gt;Barcode&lt;/barcode&gt; <br> 
		* &lt;script&gt;
		* // Create Barcode
		* var btnObj = $("#barcode1").data("ejBarcode");
		* btnObj.disable(); // disable the barcode
		* &lt;/script&gt;
		* @example 
		* &lt;barcode id="barcode1"&gt;Barcode&lt;/barcode&gt; <br> 
		* &lt;script&gt;
		* // disable the barcode
		* $("#barcode1").ejBarcode("disable");	
		* &lt;/script&gt;
		* @memberof ejBarcode
		* @instance
        */
        disable: function () {
            if (this.model.enabled) {
                this.element.addClass("e-disable");
                this.model.enabled = false;
            }
        },

        /**
         * @private
         */
        _disabled: function (boolean) {
            if (boolean) this.disable();
            else this.enable();
        },

        /**
         * Initializes rendering canvas.
         * @private
         */
        _initObject: function (element) {
            element.BarcodeEl = $("#" + element.target.id);
            if (element.canvasEl)
                element.canvasEl.remove();
            else
                element.canvasEl = $("<canvas></canvas>");
            element.BarcodeEl.append(element.canvasEl);
            element.BarcodeEl.css("width", element.model.width);
            element.BarcodeEl.css("height", element.model.height);
            element.canvasEl[0].setAttribute("width", element.BarcodeEl.width());
            element.canvasEl[0].setAttribute("height", element.BarcodeEl.height());
            element.centerX = element.canvasEl[0].width / 2;
            element.centerY = element.canvasEl[0].height / 2;
            var elem = element.canvasEl[0];
            if (typeof window.G_vmlCanvasManager != "undefined") {
                elem = window.G_vmlCanvasManager.initElement(elem);
            }
            if (!elem || !elem.getContext) {
                return;
            }
            element.contextEl = element.canvasEl[0].getContext("2d");

            if (element.contextEl == undefined) //IE 8
                return;
        },

        /**
         * Finds the barcode type and encodes text.
         * @private
         */
        _findBarcodeType: function () {
            if (this.contextEl == undefined) //IE 8
                return;

            if (this.Model.text == "")
                return;

            this.textFont = { size: "12px", fontFamily: "Segoe UI", fontStyle: "Regular" };

            if (this.model.quietZone.all > 1)
                this.model.quietZone.left = this.model.quietZone.right = this.model.quietZone.top = this.model.quietZone.bottom = this.model.quietZone.all;

            intercharacterGap = 1;
            extendedText = "";
            continuous = false;
            validatorExp = "";
            encodedText = this.Model.text;
            var startSymbol, stopSymbol, symbolTable, extendedCodes;

            var type = this.Model.symbologyType;
            var oneD = true;

            /** Apply symbology setting.*/
            if (ej.Barcode.SymbologyType.dataMatrix == type || ej.Barcode.SymbologyType.qrBarcode == type) {
                oneD = false;
            }
            else {
                startSymbol = ej.Barcode.symbologySettings[type].startSymbol;
                stopSymbol = ej.Barcode.symbologySettings[type].stopSymbol;
                validatorExp = ej.Barcode.symbologySettings[type].validatorExp;
                symbolTable = ej.Barcode.symbologySettings[type].symbolTable;
                extendedCodes = ej.Barcode.symbologySettings[type].extendedCodes;

                if (ej.Barcode.SymbologyType.code39Extended == type) {
                    var cArray = encodedText.split("");
                    var builder = "";
                    for (var j = 0; j < cArray.length; j++) {
                        var item = cArray[j];
                        for (var k = 0; k < extendedCodes.length; k++) {
                            if (extendedCodes[k][0] == item) {
                                var chArray = extendedCodes[k][1];
                                if (!(chArray == undefined)) {
                                    for (var i = 0; i < chArray.length; i++) {
                                        builder += chArray[i];
                                    }
                                }
                            }
                        }
                    }
                    extendedText = builder;
                }
                else if (ej.Barcode.SymbologyType.code93Extended == type) {
                    var cArray = encodedText.split("");
                    var builder = "";
                    for (var j = 0; j < cArray.length; j++) {
                        var item = cArray[j];
                        for (var k = 0; k < extendedCodes.length; k++) {
                            if (extendedCodes[k][0] == item) {
                                var chArray = extendedCodes[k][1];
                                if (!(chArray == undefined)) {
                                    for (var i = 0; i < chArray.length; i++) {
                                        builder += chArray[i];
                                    }
                                }
                            }
                        }
                    }
                    extendedText = builder;
                }
            }

            /** Process one dimensional barcode.*/
            if (oneD) {
                if (!this._validateText(this.Model.text, validatorExp)) {
                    console.log("Barcode Text contains characters that are not accepted by this barcode specification.");
                    return;
                }

                if (this.model.symbologyType == "code32" && this.model.text.length != 8) {
                    console.log("Barcode Text Length that are not accepted by this barcode specification.");
                    return;
                }

                if (extendedText.length > 0)
                    encodedText = extendedText;

                if (this.Model.encodeStartStopSymbol)
                    encodedText = startSymbol.toString() + encodedText + stopSymbol.toString();

                var w = this._getCharWidth(encodedText, symbolTable);

                if (continuous == false)
                    w -= (intercharacterGap * encodedText.length);
                else {
                    w -= (extendedText.length > 0) ?
                        intercharacterGap * (extendedText.length - this.Model.text.length) :
                        intercharacterGap;
                }

                var textHeight = parseInt(this.Model.displayText ? this.textFont.size.replace('px', '') : 0);
                var h = this.Model.quietZone.top + this.Model.quietZone.bottom + this.Model.barHeight + intercharacterGap + (this.Model.displayText ? (textHeight + this.Model.barcodeToTextGapHeight) : 0);
                w += this.Model.quietZone.left + this.Model.quietZone.right;

                /** Prepare the canvas.*/
                this.Model.width = w;
                this.canvasEl[0].setAttribute("width", this.Model.width);
                this.BarcodeEl.css("width", this.Model.width);

                this.contextEl.beginPath();
                this.contextEl.fillStyle = this.Model.lightBarColor;
                this.contextEl.fillRect(0, 0, w, h);
                this.contextEl.closePath();

                /** Draw one dimensional barcode.*/
                this._draw1DBarcode(encodedText, symbolTable);
            }
                /** Process two dimensional barcode.*/
            else {
                if (ej.Barcode.SymbologyType.dataMatrix == type)
                    this._buildDataMatrix();
                else if (ej.Barcode.SymbologyType.qrBarcode == type)
                    this._buildQRBarcode();
            }

            /** Raise load event*/
            this._raiseEvent("load");
        },

        /**
         * Load event.
         * @private
         * @event
         */
        _raiseEvent: function (event) {
            if (this.model[event])
                this._trigger(event);
        },

        /**
         * Generates QR barcode.
         * @private
         */
        _buildQRBarcode: function () {
            var txt = this.Model.text;
            this.m_EciAssignmentNumber = 3;
            this.mode = "numeric";
            this.isEci = false;
            this.m_Version = 1;
            var dataCapacityOfVersions = new Array(14, 26, 42, 62, 84, 106, 122, 152, 180, 213, 251, 287, 331, 362, 412, 450, 504, 560, 624, 666, 711, 779, 857, 911, 997, 1059, 1125, 1190, 1264, 1370, 1452, 1538, 1628, 1722, 1809, 1911, 1989, 2099, 2213, 2331);
            for (var i = 0; i < txt.length; i++) {
                var cCode = txt.charCodeAt(i);
                if (cCode < 58 && cCode > 47) {
                    /** numeric only*/
                }
                else if ((cCode < 91 && cCode > 64) || txt[i] == '$' || txt[i] == '%' || txt[i] == '*' || txt[i] == '+' || txt[i] == '-' || txt[i] == '.' || txt[i] == '/' || txt[i] == ':' || txt[i] == ' ') {
                    this.mode = "alphanumeric";
                }
                else if ((cCode >= 65377 && cCode <= 65439) || (cCode >= 97 && cCode <= 122)) {
                    this.mode = "binary";
                    break;
                }
                else {
                    this.mode = "binary";
                    this.isEci = true;
                    break;
                }
            }
            if (this.mode == "numeric")
                dataCapacityOfVersions = new Array(34, 63, 101, 149, 202, 255, 293, 365, 432, 513, 604, 691, 796, 871, 991, 1082, 1212, 1346, 1500, 1600, 1708, 1872, 2059, 2188, 2395, 2544, 2701, 2857, 3035, 3289, 3486, 3693, 3909, 4134, 4343, 4588, 4775, 5039, 5313, 5596);
            else if (this.mode == "alphanumeric")
                dataCapacityOfVersions = new Array(20, 38, 61, 90, 122, 154, 178, 221, 262, 311, 366, 419, 483, 528, 600, 656, 734, 816, 909, 970, 1035, 1134, 1248, 1326, 1451, 1542, 1637, 1732, 1839, 1994, 2113, 2238, 2369, 2506, 2632, 2780, 2894, 3054, 3220, 3391);

            if (this.isEci == true) {
                this.m_EciAssignmentNumber = this._findECINumber(txt);
            }
            for (var i = 0; i < dataCapacityOfVersions.length; i++) {
                if (dataCapacityOfVersions[i] > txt.length) {
                    this.m_Version = i + 1;
                    break;
                }
            }

            this.qrbarcodeValues = { Version: this.m_Version, ErrorCorrectionLevel: 1, NumberOfDataCodeWord: this._getNumberOfDataCodeWord(), NumberOfErrorCorrectingCodeWords: this._getNumberOfErrorCorrectingCodeWords(), NumberOfErrorCorrectionBlocks: this._getNumberOfErrorCorrectionBlocks(), End: this._getEnd(), DataCapacity: this._getDataCapacity(), FormatInformation: this._getFormatInformation(), VersionInformation: this._getVersionInformation() };
            this.noOfModules = (this.m_Version - 1) * 4 + 21;

            this.moduleValue = this._create2DArray(this.noOfModules, this.noOfModules);

            for (var i = 0; i < this.noOfModules; i++) {
                for (var j = 0; j < this.noOfModules; j++) {
                    this.moduleValue[i][j] = { IsBlack: false, IsFilled: false, IsPDP: false };
                }
            }

            this._drawPDP(0, 0);
            this._drawPDP(this.noOfModules - 7, 0);
            this._drawPDP(0, this.noOfModules - 7);
            this._drawTimingPattern();

            if (this.m_Version != 1) {
                var allignCoOrdinates = this._getAlignmentPatternCoOrdinates();

                for (var i = 0; i < allignCoOrdinates.length; i++) {
                    for (var j = 0; j < allignCoOrdinates.length; j++) {
                        var x = allignCoOrdinates[i];
                        var y = allignCoOrdinates[j];
                        if (this.moduleValue[x][y].IsPDP != true)
                            this._drawAlignmentPattern(x, y);
                    }
                }
            }

            this._allocateFormatVersionInfo();
            this._encodeData();
            this._drawFormatInformation();
            this._addQuietZone();

            var w = this.moduleValue.length * this.Model.xDimension;
            var h = this.moduleValue[0].length * this.Model.xDimension;

            /** Prepare canvas.*/
            this.Model.width = w;
            this.canvasEl[0].setAttribute("width", this.Model.width);
            this.BarcodeEl.css("width", this.Model.width);

            this.contextEl.beginPath();
            this.contextEl.fillStyle = 'white';
            this.contextEl.fillRect(0, 0, w, h);
            this.contextEl.closePath();

            /** Draw QR barcode in canvas.*/
            this._drawQRBarcode(h);
        },

        /** 
         * Allocate format and version information.
         * @private
         */
        _allocateFormatVersionInfo: function () {
            for (var i = 0; i < 9; i++) {
                this.moduleValue[8][i].IsFilled = true;
                this.moduleValue[i][8].IsFilled = true;
            }
            for (var i = this.noOfModules - 8; i < this.noOfModules; i++) {
                this.moduleValue[8][i].IsFilled = true;
                this.moduleValue[i][8].IsFilled = true;
            }
            if (this.m_Version > 6) {
                var versionInformation = this.qrbarcodeValues.VersionInformation;
                var count = 0;
                for (var i = 0; i < 6; i++) {
                    for (var j = 2; j >= 0; j--) {
                        this.moduleValue[i][this.noOfModules - 9 - j].IsBlack = versionInformation[count] == 1 ? true : false;
                        this.moduleValue[i][this.noOfModules - 9 - j].IsFilled = true;

                        this.moduleValue[this.noOfModules - 9 - j][i].IsBlack = versionInformation[count++] == 1 ? true : false;
                        this.moduleValue[this.noOfModules - 9 - j][i].IsFilled = true;
                    }
                }
            }
        },

        /**
         * @private
         */
        _drawTimingPattern: function () {
            for (var i = 8; i < this.noOfModules - 8; i += 2) {
                this.moduleValue[i][6].IsBlack = true;
                this.moduleValue[i][6].IsFilled = true;

                this.moduleValue[i + 1][6].IsBlack = false;
                this.moduleValue[i + 1][6].IsFilled = true;

                this.moduleValue[6][i].IsBlack = true;
                this.moduleValue[6][i].IsFilled = true;

                this.moduleValue[6][i + 1].IsBlack = false;
                this.moduleValue[6][i + 1].IsFilled = true;
            }
            this.moduleValue[this.noOfModules - 8][8].IsBlack = true;
            this.moduleValue[this.noOfModules - 8][8].IsFilled = true;
        },

        /** 
         * Draws QR barcode.
         * @private
         */
        _drawQRBarcode: function (ht) {
            var quietZone = this.Model.quietZone.all;
            if (quietZone < 2)
                quietZone = 2;

            var whiteBrush = 'white';
            var blackBrush = 'black';

            this.Model.height = ht;
            this.canvasEl[0].setAttribute("height", this.Model.height);
            this.BarcodeEl.css("height", this.Model.height);
            var dim = parseInt(this.Model.xDimension);

            var y = 0;

            var w = this.noOfModules + 2 * quietZone, h = this.noOfModules + 2 * quietZone;

            this.contextEl.beginPath();

            for (var i = 0; i < w; i++) {
                var x = 0;

                for (var j = 0; j < h; j++) {
                    var colorBrush = null;
                    if (this.moduleValue[i][j].IsBlack)
                        colorBrush = 'black';
                    else
                        colorBrush = 'white';

                    if (this.dataAllocationValues[j][i].IsFilled)
                        if (this.dataAllocationValues[j][i].IsBlack)
                            colorBrush = 'black';

                    this.contextEl.fillStyle = colorBrush;
                    this.contextEl.fillRect(x, y, dim, dim)

                    x += dim;
                }

                y += dim;
            }

            this.contextEl.closePath();
            this.contextEl.stroke();
        },

        /** 
         * Adds quiet zone to the barcode.
         * @private
         */
        _addQuietZone: function () {
            var quietZone = this.Model.quietZone.all;
            if (quietZone < 2)
                quietZone = 2;
            var w = this.noOfModules + 2 * quietZone;
            var h = this.noOfModules + 2 * quietZone;
            var tempValue1 = this._create2DArray(w, h);
            var tempValue2 = this._create2DArray(w, h);

            /** Initialize array.*/
            for (var i = 0; i < w; i++) {
                for (var j = 0; j < h; j++) {
                    tempValue1[i][j] = { IsBlack: false, IsFilled: false, IsPDP: false };
                    tempValue2[i][j] = { IsBlack: false, IsFilled: false, IsPDP: false };
                }
            }

            for (var i = quietZone; i < w - quietZone; i++) {
                for (var j = quietZone; j < h - quietZone; j++) {
                    tempValue1[i][j] = this.moduleValue[i - quietZone][j - quietZone];
                    tempValue2[i][j] = this.dataAllocationValues[i - quietZone][j - quietZone];
                }
            }

            this.moduleValue = tempValue1;
            this.dataAllocationValues = tempValue2;
        },

        /** 
         * Adds format information to the barcode.
         * @private
         */
        _drawFormatInformation: function () {
            var formatInformation = this.qrbarcodeValues.FormatInformation;

            var count = 0;
            for (var i = 0; i < 7; i++) {
                /** Draw from 14 to 8.*/
                if (i == 6)
                    this.moduleValue[i + 1][8].IsBlack = formatInformation[count] == 1 ? true : false;
                else
                    this.moduleValue[i][8].IsBlack = formatInformation[count] == 1 ? true : false;

                this.moduleValue[8][this.noOfModules - i - 1].IsBlack = formatInformation[count++] == 1 ? true : false;
            }
            count = 14;
            for (var i = 0; i < 7; i++) {
                /** Draw from 0 to 6.*/
                if (i == 6)
                    this.moduleValue[8][i + 1].IsBlack = formatInformation[count] == 1 ? true : false;
                else
                    this.moduleValue[8][i].IsBlack = formatInformation[count] == 1 ? true : false;
                this.moduleValue[this.noOfModules - i - 1][8].IsBlack = formatInformation[count--] == 1 ? true : false;
            }

            /** Draw 7.*/
            this.moduleValue[8][8].IsBlack = formatInformation[7] == 1 ? true : false;
            this.moduleValue[8][this.noOfModules - 8].IsBlack = formatInformation[7] == 1 ? true : false;
        },

        /** 
         * @private
         */
        _dataAllocationAndMasking: function (Data) {
            var num = this.noOfModules;
            this.dataAllocationValues = this._create2DArray(num, num);
            var point = 0;

            for (var i = 0; i < num; i++) 
                for (var j = 0; j < num; j++) 
                    this.dataAllocationValues[i][j] = { IsBlack: false, IsFilled: false, IsPDP: false };

            for (var i = num - 1; i >= 0; i -= 2) {
                for (var j = num - 1; j >= 0; j--)
                    point = this._allocateValues(Data, i, j, point);

                i -= 2;
                if (i == 6)
                    i--;

                for (var j = 0; j < num; j++)
                    point = this._allocateValues(Data, i, j, point);
            }
            for (var i = 0; i < num; i++)
                for (var j = 0; j < num; j++)
                    if (!this.moduleValue[i][j].IsFilled) {
                        var flag = this.dataAllocationValues[i][j].IsBlack;
                        if (flag)
                            this.dataAllocationValues[i][j].IsBlack = false;
                        else
                            this.dataAllocationValues[i][j].IsBlack = true;
                    }


        },

        /** 
         * @private
         */
        _allocateValues: function (Data, i, j, point) {
            if (!(this.moduleValue[i][j].IsFilled && this.moduleValue[i - 1][j].IsFilled)) {
                if (!this.moduleValue[i][j].IsFilled) {
                    if (point + 1 < Data.length)
                        this.dataAllocationValues[i][j].IsBlack = Data[point++];
                    if ((i + j) % 3 == 0) {
                        if (this.dataAllocationValues[i][j].IsBlack)
                            this.dataAllocationValues[i][j].IsBlack = true;
                        else
                            this.dataAllocationValues[i][j].IsBlack = false;
                    }
                    else {
                        if (this.dataAllocationValues[i][j].IsBlack)
                            this.dataAllocationValues[i][j].IsBlack = false;
                        else
                            this.dataAllocationValues[i][j].IsBlack = true;
                    }

                    this.dataAllocationValues[i][j].IsFilled = true;
                }

                if (!this.moduleValue[i - 1][j].IsFilled) {
                    if (point + 1 < Data.length)
                        this.dataAllocationValues[i - 1][j].IsBlack = Data[point++];
                    if ((i - 1 + j) % 3 == 0) {
                        if (this.dataAllocationValues[i - 1][j].IsBlack)
                            this.dataAllocationValues[i - 1][j].IsBlack = true;
                        else
                            this.dataAllocationValues[i - 1][j].IsBlack = false;
                    }
                    else {
                        if (this.dataAllocationValues[i - 1][j].IsBlack)
                            this.dataAllocationValues[i - 1][j].IsBlack = false;
                        else
                            this.dataAllocationValues[i - 1][j].IsBlack = true;
                    }
                    this.dataAllocationValues[i - 1][j].IsFilled = true;
                }
            }

            return point;
        },

        /** 
         * Encodes data.
         * @private
         */
        _encodeData: function () {
            var eData = new Array();
            var c = -1;

            switch (this.mode) {
                case "numeric":
                    eData[++c] = false;
                    eData[++c] = false;
                    eData[++c] = false;
                    eData[++c] = true;
                    break;

                case "alphanumeric":
                    eData[++c] = false;
                    eData[++c] = false;
                    eData[++c] = true;
                    eData[++c] = false;
                    break;

                case "binary":
                    if (this.isEci) {
                        /** Add ECI Mode Indicator.*/
                        eData[++c] = false;
                        eData[++c] = true;
                        eData[++c] = true;
                        eData[++c] = true;

                        /** Add ECI assignment number.*/
                        var numberInBool = this._stringToBoolArray(this.m_EciAssignmentNumber.toString(), 8);
                        for (var e = 0; e < numberInBool.length; e++)
                            eData[++c] = numberInBool[e];
                    }
                    eData[++c] = false;
                    eData[++c] = true;
                    eData[++c] = false;
                    eData[++c] = false;
                    break;
            }

            var numberOfBitsInCharacterCountIndicator = 0;
            if (this.m_Version < 10) {
                switch (this.mode) {
                    case "numeric":
                        numberOfBitsInCharacterCountIndicator = 10;
                        break;
                    case "alphanumeric":
                        numberOfBitsInCharacterCountIndicator = 9;
                        break;
                    case "binary":
                        numberOfBitsInCharacterCountIndicator = 8;
                        break;
                }
            }
            else if (this.m_Version < 27) {
                switch (this.mode) {
                    case "numeric":
                        numberOfBitsInCharacterCountIndicator = 12;
                        break;
                    case "alphanumeric":
                        numberOfBitsInCharacterCountIndicator = 11;
                        break;
                    case "binary":
                        numberOfBitsInCharacterCountIndicator = 16;
                        break;
                }
            }
            else {
                switch (this.mode) {
                    case "numeric":
                        numberOfBitsInCharacterCountIndicator = 14;
                        break;
                    case "alphanumeric":
                        numberOfBitsInCharacterCountIndicator = 13;
                        break;
                    case "binary":
                        numberOfBitsInCharacterCountIndicator = 16;
                        break;
                }
            }

            var dataStringArray = this.Model.text;
            var numberOfBitsInCharacterCountIndicatorInBool = this._intToBoolArray(dataStringArray.length, numberOfBitsInCharacterCountIndicator);

            for (var i = 0; i < numberOfBitsInCharacterCountIndicator; i++)
                eData[++c] = numberOfBitsInCharacterCountIndicatorInBool[i];

            if (this.mode == "numeric") {
                var number = "";
                for (var i = 0; i < dataStringArray.length; i++) {
                    var numberInBool = new Array();
                    number += dataStringArray[i];

                    if (i % 3 == 2 && i != 0 || i == dataStringArray.length - 1) {
                        if (number.length == 3)
                            numberInBool = this._stringToBoolArray(number, 10);
                        else if (number.length == 2)
                            numberInBool = this._stringToBoolArray(number, 7);
                        else
                            numberInBool = this._stringToBoolArray(number, 4);
                        number = "";
                        for (var j = 0; j < numberInBool.length; j++)
                            eData[++c] = numberInBool[j];
                    }
                }
            }
            else if (this.mode == "alphanumeric") {
                var numberInString = "";
                var number = 0;
                for (var i = 0; i < dataStringArray.length; i++) {
                    var numberInBool = new Array();
                    numberInString += dataStringArray[i];

                    if (i % 2 == 0 && i + 1 != dataStringArray.length)
                        number = 45 * this._getAlphanumericvalues(dataStringArray[i]);

                    if (i % 2 == 1 && i != 0) {
                        number += this._getAlphanumericvalues(dataStringArray[i]);
                        numberInBool = this._intToBoolArray(number, 11);
                        number = 0;
                        for (var n = 0; n < numberInBool.length; n++)
                            eData[++c] = numberInBool[n];
                        numberInString = "";
                    }
                    if (i != 1 && numberInString != null)
                        if (i + 1 == dataStringArray.length && numberInString.length == 1) {
                            number = this._getAlphanumericvalues(dataStringArray[i]);
                            numberInBool = this._intToBoolArray(number, 6);
                            number = 0;
                            for (var j = 0; j < numberInBool.length; j++)
                                eData[++c] = numberInBool[j];
                        }
                }
            }
            else if (this.mode = "binary") {
                {
                    for (var i = 0; i < dataStringArray.length; i++) {
                        var number = 0;
                        if ((dataStringArray.charCodeAt(i) >= 32 && dataStringArray.charCodeAt(i) <= 126) || (dataStringArray.charCodeAt(i) >= 161 && dataStringArray.charCodeAt(i) <= 255))
                            number = dataStringArray.charCodeAt(i);
                        else if (dataStringArray.charCodeAt(i) >= 65377 && dataStringArray.charCodeAt(i) <= 65439)
                            number = dataStringArray.charCodeAt(i) - 65216;
                        else {
                            console.log("Input text contains non-convertable characters");
                            return;
                        }
                        var numberInBool = this._intToBoolArray(number, 8);
                        for (var j = 0; j < numberInBool.length; j++)
                            eData[++c] = numberInBool[j];
                    }
                }
            }

            /** Terminator.*/
            for (var i = 0; i < 4; i++)
                if (eData.length / 8 == this.qrbarcodeValues.NumberOfDataCodeWord)
                    break;
                else
                    eData[++c] = false;

            /** Encode to code words.*/
            for (; ;)              /** Add Padding Bits.*/
            {
                if (eData.length % 8 == 0)
                    break;
                else
                    eData[++c] = false;
            }

            for (; ;) {
                if (eData.length / 8 == this.qrbarcodeValues.NumberOfDataCodeWord)
                    break;
                else {
                    eData[++c] = true;       //11101100
                    eData[++c] = true;
                    eData[++c] = true;
                    eData[++c] = false;
                    eData[++c] = true;
                    eData[++c] = true;
                    eData[++c] = false;
                    eData[++c] = false;
                }
                if (eData.length / 8 == this.qrbarcodeValues.NumberOfDataCodeWord)
                    break;
                else {
                    eData[++c] = false;       //00010001
                    eData[++c] = false;
                    eData[++c] = false;
                    eData[++c] = true;
                    eData[++c] = false;
                    eData[++c] = false;
                    eData[++c] = false;
                    eData[++c] = true;
                }
            }

            var dataBits = this.qrbarcodeValues.NumberOfDataCodeWord;
            blocks = this.qrbarcodeValues.NumberOfErrorCorrectionBlocks;

            var totalBlockSize = blocks[0];
            if (blocks.length == 6)
                totalBlockSize = blocks[0] + blocks[3];

            var ds1 = new Array(totalBlockSize);

            var testEncodeData = eData;
            var n = -1;
            if (blocks.length == 6)     /** If seperated into two seperate blocks.*/
            {
                var dataCodeWordLength = blocks[0] * blocks[2] * 8;
                testEncodeData = new Array(dataCodeWordLength);
                for (var i = 0; i < dataCodeWordLength; i++)
                    testEncodeData[++n] = eData[i];
            }
            var dsOne = this._create2DArray(blocks[0], testEncodeData.length / 8 / blocks[0]);
            dsOne = this._createBlocks(testEncodeData, blocks[0]);

            for (var i = 0; i < blocks[0]; i++) {
                ds1[i] = this._splitCodeWord(dsOne, i, testEncodeData.length / 8 / blocks[0]);
            }

            n = -1;
            if (blocks.length == 6) {
                testEncodeData = new Array();
                for (var i = parseInt(blocks[0] * blocks[2] * 8) ; i < eData.length; i++)
                    testEncodeData[++n] = eData[i];

                var dsTwo = this._create2DArray(blocks[0], testEncodeData.length / 8 / blocks[3]);
                dsTwo = this._createBlocks(testEncodeData, blocks[3]);

                for (var i = blocks[0], count = 0; i < totalBlockSize; i++)
                    ds1[i] = this._splitCodeWord(dsTwo, count++, testEncodeData.length / 8 / blocks[3]);
            }
            eData = new Array();
            c = -1;
            for (var i = 0; i < 125; i++) {
                for (var k = 0; k < totalBlockSize; k++)
                    for (var j = 0; j < 8; j++)
                        if (i < ds1[k].length) {
                            var temp = ds1[k][i][j];
                            eData[++c] = (temp == '1' ? true : false);
                        }
            }

            this.corCodeWords = { DataBits: this.qrbarcodeValues.NumberOfDataCodeWord, ECCW: this.qrbarcodeValues.NumberOfErrorCorrectingCodeWords, DC: 0 };

            var dataBits = this.corCodeWords.DataBits;
            var eccw = this.corCodeWords.ECCW;
            blocks = this.qrbarcodeValues.NumberOfErrorCorrectionBlocks;

            if (blocks.length == 6)
                this.corCodeWords.DataBits = (dataBits - blocks[3] * blocks[5]) / blocks[0];
            else
                this.corCodeWords.DataBits = dataBits / blocks[0];

            this.corCodeWords.ECCW = eccw / totalBlockSize;
            var polynomial = new Array(totalBlockSize);
            var count = 0;

            for (var i = 0; i < blocks[0]; i++) {
                this.corCodeWords.DC = ds1[count];
                polynomial[count++] = this._getERCW();
            }
            if (blocks.length == 6) {
                this.corCodeWords.DataBits = (dataBits - blocks[0] * blocks[2]) / blocks[3];

                for (var i = 0; i < blocks[3]; i++) {
                    this.corCodeWords.DC = ds1[count];
                    polynomial[count++] = this._getERCW();
                }
            }
            if (blocks.length != 6) {
                for (var i = 0; i < polynomial[0].length; i++) {
                    for (var k = 0; k < blocks[0]; k++)
                        for (var j = 0; j < 8; j++)
                            if (i < polynomial[k].length)
                                eData[++c] = (polynomial[k][i][j] == '1' ? true : false);
                }
            }
            else {
                for (var i = 0; i < polynomial[0].length; i++) {
                    for (var k = 0; k < totalBlockSize; k++)
                        for (var j = 0; j < 8; j++)
                            if (i < polynomial[k].length)
                                eData[++c] = (polynomial[k][i][j] == '1' ? true : false);
                }
            }


            this._dataAllocationAndMasking(eData);
        },

        /** 
         * @private
         */
        _getERCW: function () {
            var decimalRepresentation;
            var ecw;
            var decimalValue = new Array(this.corCodeWords.DataBits);
            var gx = new Array();

            switch (this.corCodeWords.ECCW) {
                case 7:
                    gx = new Array(0, 87, 229, 146, 149, 238, 102, 21);
                    gx = this._getElement(gx);
                    break;
                case 10:
                    gx = new Array(0, 251, 67, 46, 61, 118, 70, 64, 94, 32, 45);
                    gx = this._getElement(gx);
                    break;
                case 13:
                    gx = new Array(0, 74, 152, 176, 100, 86, 100, 106, 104, 130, 218, 206, 140, 78);
                    gx = this._getElement(gx);
                    break;
                case 15:
                    gx = new Array(0, 8, 183, 61, 91, 202, 37, 51, 58, 58, 237, 140, 124, 5, 99, 105);
                    gx = this._getElement(gx);
                    break;
                case 16:
                    gx = new Array(0, 120, 104, 107, 109, 102, 161, 76, 3, 91, 191, 147, 169, 182, 194, 225, 120);
                    gx = this._getElement(gx);
                    break;
                case 17:
                    gx = new Array(0, 43, 139, 206, 78, 43, 239, 123, 206, 214, 147, 24, 99, 150, 39, 243, 163, 136);
                    gx = this._getElement(gx);
                    break;
                case 18:
                    gx = new Array(0, 215, 234, 158, 94, 184, 97, 118, 170, 79, 187, 152, 148, 252, 179, 5, 98, 96, 153);
                    gx = this._getElement(gx);
                    break;
                case 20:
                    gx = new Array(0, 17, 60, 79, 50, 61, 163, 26, 187, 202, 180, 221, 225, 83, 239, 156, 164, 212, 212, 188, 190);
                    gx = this._getElement(gx);
                    break;
                case 22:
                    gx = new Array(0, 210, 171, 247, 242, 93, 230, 14, 109, 221, 53, 200, 74, 8, 172, 98, 80, 219, 134, 160, 105, 165, 231);
                    gx = this._getElement(gx);
                    break;
                case 24:
                    gx = new Array(0, 229, 121, 135, 48, 211, 117, 251, 126, 159, 180, 169, 152, 192, 226, 228, 218, 111, 0, 117, 232, 87, 96, 227, 21);
                    gx = this._getElement(gx);
                    break;
                case 26:
                    gx = new Array(0, 173, 125, 158, 2, 103, 182, 118, 17, 145, 201, 111, 28, 165, 53, 161, 21, 245, 142, 13, 102, 48, 227, 153, 145, 218, 70);
                    gx = this._getElement(gx);
                    break;
                case 28:
                    gx = new Array(0, 168, 223, 200, 104, 224, 234, 108, 180, 110, 190, 195, 147, 205, 27, 232, 201, 21, 43, 245, 87, 42, 195, 212, 119, 242, 37, 9, 123);
                    gx = this._getElement(gx);
                    break;
                case 30:
                    gx = new Array(0, 41, 173, 145, 152, 216, 31, 179, 182, 50, 48, 110, 86, 239, 96, 222, 125, 42, 173, 226, 193, 224, 130, 156, 37, 251, 216, 238, 40, 192, 180);
                    gx = this._getElement(gx);
                    break;
                case 32:
                    gx = new Array(0, 10, 6, 106, 190, 249, 167, 4, 67, 209, 138, 138, 32, 242, 123, 89, 27, 120, 185, 80, 156, 38, 69, 171, 60, 28, 222, 80, 52, 254, 185, 220, 241);
                    gx = this._getElement(gx);
                    break;
                case 34:
                    gx = new Array(0, 111, 77, 146, 94, 26, 21, 108, 19, 105, 94, 113, 193, 86, 140, 163, 125, 58, 158, 229, 239, 218, 103, 56, 70, 114, 61, 183, 129, 167, 13, 98, 62, 129, 51);
                    gx = this._getElement(gx);
                    break;
                case 36:
                    gx = new Array(0, 200, 183, 98, 16, 172, 31, 246, 234, 60, 152, 115, 0, 167, 152, 113, 248, 238, 107, 18, 63, 218, 37, 87, 210, 105, 177, 120, 74, 121, 196, 117, 251, 113, 233, 30, 120);
                    gx = this._getElement(gx);
                    break;
                case 40:
                    gx = new Array(0, 59, 116, 79, 161, 252, 98, 128, 205, 128, 161, 247, 57, 163, 56, 235, 106, 53, 26, 187, 174, 226, 104, 170, 7, 175, 35, 181, 114, 88, 41, 47, 163, 125, 134, 72, 20, 232, 53, 35, 15);
                    gx = this._getElement(gx);
                    break;
                case 42:
                    gx = new Array(0, 250, 103, 221, 230, 25, 18, 137, 231, 0, 3, 58, 242, 221, 191, 110, 84, 230, 8, 188, 106, 96, 147, 15, 131, 139, 34, 101, 223, 39, 101, 213, 199, 237, 254, 201, 123, 171, 162, 194, 117, 50, 96);
                    gx = this._getElement(gx);
                    break;
                case 44:
                    gx = new Array(0, 190, 7, 61, 121, 71, 246, 69, 55, 168, 188, 89, 243, 191, 25, 72, 123, 9, 145, 14, 247, 1, 238, 44, 78, 143, 62, 224, 126, 118, 114, 68, 163, 52, 194, 217, 147, 204, 169, 37, 130, 113, 102, 73, 181);
                    gx = this._getElement(gx);
                    break;
                case 46:
                    gx = new Array(0, 112, 94, 88, 112, 253, 224, 202, 115, 187, 99, 89, 5, 54, 113, 129, 44, 58, 16, 135, 216, 169, 211, 36, 1, 4, 96, 60, 241, 73, 104, 234, 8, 249, 245, 119, 174, 52, 25, 157, 224, 43, 202, 223, 19, 82, 15);
                    gx = this._getElement(gx);
                    break;
                case 48:
                    gx = new Array(0, 228, 25, 196, 130, 211, 146, 60, 24, 251, 90, 39, 102, 240, 61, 178, 63, 46, 123, 115, 18, 221, 111, 135, 160, 182, 205, 107, 206, 95, 150, 120, 184, 91, 21, 247, 156, 140, 238, 191, 11, 94, 227, 84, 50, 163, 39, 34, 108);
                    gx = this._getElement(gx);
                    break;
                case 50:
                    gx = new Array(0, 232, 125, 157, 161, 164, 9, 118, 46, 209, 99, 203, 193, 35, 3, 209, 111, 195, 242, 203, 225, 46, 13, 32, 160, 126, 209, 130, 160, 242, 215, 242, 75, 77, 42, 189, 32, 113, 65, 124, 69, 228, 114, 235, 175, 124, 170, 215, 232, 133, 205);
                    gx = this._getElement(gx);
                    break;
                case 52:
                    gx = new Array(0, 116, 50, 86, 186, 50, 220, 251, 89, 192, 46, 86, 127, 124, 19, 184, 233, 151, 215, 22, 14, 59, 145, 37, 242, 203, 134, 254, 89, 190, 94, 59, 65, 124, 113, 100, 233, 235, 121, 22, 76, 86, 97, 39, 242, 200, 220, 101, 33, 239, 254, 116, 51);
                    gx = this._getElement(gx);
                    break;
                case 54:
                    gx = new Array(0, 183, 26, 201, 87, 210, 221, 113, 21, 46, 65, 45, 50, 238, 184, 249, 225, 102, 58, 209, 218, 109, 165, 26, 95, 184, 192, 52, 245, 35, 254, 238, 175, 172, 79, 123, 25, 122, 43, 120, 108, 215, 80, 128, 201, 235, 8, 153, 59, 101, 31, 198, 76, 31, 156);
                    gx = this._getElement(gx);
                    break;
                case 56:
                    gx = new Array(0, 106, 120, 107, 157, 164, 216, 112, 116, 2, 91, 248, 163, 36, 201, 202, 229, 6, 144, 254, 155, 135, 208, 170, 209, 12, 139, 127, 142, 182, 249, 177, 174, 190, 28, 10, 85, 239, 184, 101, 124, 152, 206, 96, 23, 163, 61, 27, 196, 247, 151, 154, 202, 207, 20, 61, 10);
                    gx = this._getElement(gx);
                    break;
                case 58:
                    gx = new Array(0, 82, 116, 26, 247, 66, 27, 62, 107, 252, 182, 200, 185, 235, 55, 251, 242, 210, 144, 154, 237, 176, 141, 192, 248, 152, 249, 206, 85, 253, 142, 65, 165, 125, 23, 24, 30, 122, 240, 214, 6, 129, 218, 29, 145, 127, 134, 206, 245, 117, 29, 41, 63, 159, 142, 233, 125, 148, 123);
                    gx = this._getElement(gx);
                    break;
                case 60:
                    gx = new Array(0, 107, 140, 26, 12, 9, 141, 243, 197, 226, 197, 219, 45, 211, 101, 219, 120, 28, 181, 127, 6, 100, 247, 2, 205, 198, 57, 115, 219, 101, 109, 160, 82, 37, 38, 238, 49, 160, 209, 121, 86, 11, 124, 30, 181, 84, 25, 194, 87, 65, 102, 190, 220, 70, 27, 209, 16, 89, 7, 33, 240);
                    gx = this._getElement(gx);
                    break;
                case 62:
                    gx = new Array(0, 65, 202, 113, 98, 71, 223, 248, 118, 214, 94, 0, 122, 37, 23, 2, 228, 58, 121, 7, 105, 135, 78, 243, 118, 70, 76, 223, 89, 72, 50, 70, 111, 194, 17, 212, 126, 181, 35, 221, 117, 235, 11, 229, 149, 147, 123, 213, 40, 115, 6, 200, 100, 26, 246, 182, 218, 127, 215, 36, 186, 110, 106);
                    gx = this._getElement(gx);
                    break;
                case 64:
                    gx = new Array(0, 45, 51, 175, 9, 7, 158, 159, 49, 68, 119, 92, 123, 177, 204, 187, 254, 200, 78, 141, 149, 119, 26, 127, 53, 160, 93, 199, 212, 29, 24, 145, 156, 208, 150, 218, 209, 4, 216, 91, 47, 184, 146, 47, 140, 195, 195, 125, 242, 238, 63, 99, 108, 140, 230, 242, 31, 204, 11, 178, 243, 217, 156, 213, 231);
                    gx = this._getElement(gx);
                    break;
                case 66:
                    gx = new Array(0, 5, 118, 222, 180, 136, 136, 162, 51, 46, 117, 13, 215, 81, 17, 139, 247, 197, 171, 95, 173, 65, 137, 178, 68, 111, 95, 101, 41, 72, 214, 169, 197, 95, 7, 44, 154, 77, 111, 236, 40, 121, 143, 63, 87, 80, 253, 240, 126, 217, 77, 34, 232, 106, 50, 168, 82, 76, 146, 67, 106, 171, 25, 132, 93, 45, 105);
                    gx = this._getElement(gx);
                    break;
                case 68:
                    gx = new Array(0, 247, 159, 223, 33, 224, 93, 77, 70, 90, 160, 32, 254, 43, 150, 84, 101, 190, 205, 133, 52, 60, 202, 165, 220, 203, 151, 93, 84, 15, 84, 253, 173, 160, 89, 227, 52, 199, 97, 95, 231, 52, 177, 41, 125, 137, 241, 166, 225, 118, 2, 54, 32, 82, 215, 175, 198, 43, 238, 235, 27, 101, 184, 127, 3, 5, 8, 163, 238);
                    gx = this._getElement(gx);
                    break;
            }

            decimalValue = this._toDecimal(this.corCodeWords.DC, decimalValue);
            decimalRepresentation = this._divide(gx, decimalValue);
            ecw = this._toBinary(decimalRepresentation);
            return ecw;
        },

        /** 
         * Convert to binary.
         * @private
         */
        _toBinary: function (decimalRepresentation) {
            var toBinary = new Array(this.corCodeWords.ECCW);
            for (var i = 0; i < decimalRepresentation.length; i++) {
                var temp = decimalRepresentation[i].toString(2);
                var str = new String();
                for (var j = 0; j < (8 - temp.length) ; j++)
                    str += "0";
                toBinary[i] = str + temp;
            }
            return toBinary;
        },

        /** 
         * @private
         */
        _divide: function (gx, decimalValue) {
            var alpha = new Array(1, 2, 4, 8, 16, 32, 64, 128, 29, 58, 116, 232, 205, 135, 19, 38, 76, 152, 45, 90, 180, 117, 234, 201, 143, 3, 6, 12, 24, 48, 96, 192, 157, 39, 78, 156, 37, 74, 148, 53, 106, 212, 181, 119, 238, 193, 159, 35, 70, 140, 5, 10, 20, 40, 80, 160, 93, 186, 105, 210, 185, 111, 222, 161, 95, 190, 97, 194, 153, 47, 94, 188, 101, 202, 137, 15, 30, 60, 120, 240, 253, 231, 211, 187, 107, 214, 177, 127, 254, 225, 223, 163, 91, 182, 113, 226, 217, 175, 67, 134, 17, 34, 68, 136, 13, 26, 52, 104, 208, 189, 103, 206, 129, 31, 62, 124, 248, 237, 199, 147, 59, 118, 236, 197, 151, 51, 102, 204, 133, 23, 46, 92, 184, 109, 218, 169, 79, 158, 33, 66, 132, 21, 42, 84, 168, 77, 154, 41, 82, 164, 85, 170, 73, 146, 57, 114, 228, 213, 183, 115, 230, 209, 191, 99, 198, 145, 63, 126, 252, 229, 215, 179, 123, 246, 241, 255, 227, 219, 171, 75, 150, 49, 98, 196, 149, 55, 110, 220, 165, 87, 174, 65, 130, 25, 50, 100, 200, 141, 7, 14, 28, 56, 112, 224, 221, 167, 83, 166, 81, 162, 89, 178, 121, 242, 249, 239, 195, 155, 43, 86, 172, 69, 138, 9, 18, 36, 72, 144, 61, 122, 244, 245, 247, 243, 251, 235, 203, 139, 11, 22, 44, 88, 176, 125, 250, 233, 207, 131, 27, 54, 108, 216, 173, 71, 142);
            var m_length = this.corCodeWords.DataBits + this.corCodeWords.ECCW;
            var result = new Array(m_length);
            var fx = new Array(this.corCodeWords.ECCW);
            var a1 = new Array(gx.length);
            var a2 = new Array(m_length);

            for (var i = 0; i < gx.length; i++)
                a1[i] = this._findElement(gx[i], alpha);

            for (var i = 0; i < decimalValue.length; i++)
                result[i] = decimalValue[i];

            for (var i = 0; i < this.corCodeWords.DC.length; i++) {
                a2[i] = this._findElement(result[i], alpha);
                var g = i, z = i;
                for (var j = 0; j < gx.length; j++) {
                    var e = a1[j] + a2[i];
                    if (e >= 255)
                        e = e - 255;
                    result[z] = alpha[e] ^ result[g];
                    z++;
                    g++;
                }
            }
            var k = result.length - 1;
            for (var i = this.corCodeWords.ECCW - 1; i >= 0; i--) {
                fx[i] = result[k];
                k--;
            }
            return fx;
        },

        /** 
         * @private
         */
        _findElement: function (element, alpha) {
            var j = 0;
            for (; j < alpha.length; j++) {
                if (element == alpha[j])
                    break;
            }
            return j;
        },

        /** 
         * @private
         */
        _getElement: function (element) {
            var gx = new Array(element.length);
            var alpha = new Array(1, 2, 4, 8, 16, 32, 64, 128, 29, 58, 116, 232, 205, 135, 19, 38, 76, 152, 45, 90, 180, 117, 234, 201, 143, 3, 6, 12, 24, 48, 96, 192, 157, 39, 78, 156, 37, 74, 148, 53, 106, 212, 181, 119, 238, 193, 159, 35, 70, 140, 5, 10, 20, 40, 80, 160, 93, 186, 105, 210, 185, 111, 222, 161, 95, 190, 97, 194, 153, 47, 94, 188, 101, 202, 137, 15, 30, 60, 120, 240, 253, 231, 211, 187, 107, 214, 177, 127, 254, 225, 223, 163, 91, 182, 113, 226, 217, 175, 67, 134, 17, 34, 68, 136, 13, 26, 52, 104, 208, 189, 103, 206, 129, 31, 62, 124, 248, 237, 199, 147, 59, 118, 236, 197, 151, 51, 102, 204, 133, 23, 46, 92, 184, 109, 218, 169, 79, 158, 33, 66, 132, 21, 42, 84, 168, 77, 154, 41, 82, 164, 85, 170, 73, 146, 57, 114, 228, 213, 183, 115, 230, 209, 191, 99, 198, 145, 63, 126, 252, 229, 215, 179, 123, 246, 241, 255, 227, 219, 171, 75, 150, 49, 98, 196, 149, 55, 110, 220, 165, 87, 174, 65, 130, 25, 50, 100, 200, 141, 7, 14, 28, 56, 112, 224, 221, 167, 83, 166, 81, 162, 89, 178, 121, 242, 249, 239, 195, 155, 43, 86, 172, 69, 138, 9, 18, 36, 72, 144, 61, 122, 244, 245, 247, 243, 251, 235, 203, 139, 11, 22, 44, 88, 176, 125, 250, 233, 207, 131, 27, 54, 108, 216, 173, 71, 142);
            for (var i = 0; i < element.length; i++) {
                if (element[i] > 255)
                    element[i] = element[i] - 255;
                gx[i] = alpha[element[i]];
            }
            return gx;
        },

        /** 
         * @private
         */
        _toDecimal: function (inString) {
            var decimalValue = new Array(inString.length);
            for (var i = 0; i < inString.length; i++)
                decimalValue[i] = parseInt(inString[i], 2);

            return decimalValue;
        },

        /** 
         * @private
         */
        _splitCodeWord: function (ds, blk, count) {
            count = parseInt(count);
            var ds1 = new Array(count);
            for (var i = 0; i < count; i++)
                ds1[i] = ds[blk][i];
            return ds1;
        },

        /** 
         * Create blocks for datamatrix barcode.
         * @private
         */
        _createBlocks: function (data, noOfBlocks) {
            var ret = this._create2DArray(noOfBlocks, data.length / 8 / noOfBlocks);
            var stringValue = "";
            var j = 0, i = 0, blockNumber = 0;

            for (; j < data.length; j++) {
                if (j % 8 == 0 && j != 0) {
                    ret[blockNumber][i] = stringValue;
                    stringValue = "";
                    i++;

                    if (i == (data.length / noOfBlocks / 8)) {
                        blockNumber++;
                        i = 0;
                    }
                }

                stringValue += data[j] ? 1 : 0;
            }
            ret[blockNumber][i] = stringValue;
            return ret;
        },

        /** 
         * Returns alphanumeric values for the character.
         * @private
         */
        _getAlphanumericvalues: function (charValue) {
            var valueInInt = -1;
            var alphaNumArray = new Array('0', '1', '2', '3', '4', '5', '6', '7', '8', '9', 'A', 'B', 'C', 'D', 'E', 'F', 'G', 'H', 'I', 'J', 'K', 'L', 'M', 'N', 'O', 'P', 'Q', 'R', 'S', 'T', 'U', 'V', 'W', 'X', 'Y', 'Z', ' ', '$', '%', '*', '+', '-', '.', '/', ':');
            valueInInt = alphaNumArray.indexOf(charValue);
            return valueInInt;
        },

        /** 
         * @private
         */
        _stringToBoolArray: function (numberInString, noOfBits) {
            var numberInBool = new Array(noOfBits);

            var number = 0;
            for (var i = 0; i < numberInString.length; i++)
                number = number * 10 + numberInString.charCodeAt(i) - 48;

            for (var i = 0; i < noOfBits; i++)
                numberInBool[noOfBits - i - 1] = ((number >> i) & 1) == 1;

            return numberInBool;
        },

        /** 
         * @private
         */
        _intToBoolArray: function (number, noOfBits) {
            var numberInBool = new Array(noOfBits);
            for (var i = 0; i < noOfBits; i++)
                numberInBool[noOfBits - i - 1] = ((number >> i) & 1) == 1;
            return numberInBool;
        },

        /** 
         * @private
         */
        _drawPDP: function (x, y) {
            var i, j;
            for (i = x, j = y; i < x + 7; i++, j++) {
                this.moduleValue[i][y].IsBlack = true;
                this.moduleValue[i][y].IsFilled = true;
                this.moduleValue[i][y].IsPDP = true;

                this.moduleValue[i][y + 6].IsBlack = true;
                this.moduleValue[i][y + 6].IsFilled = true;
                this.moduleValue[i][y + 6].IsPDP = true;

                if (y + 7 < this.noOfModules) {
                    this.moduleValue[i][y + 7].IsBlack = false;
                    this.moduleValue[i][y + 7].IsFilled = true;
                    this.moduleValue[i][y + 7].IsPDP = true;
                }
                else if (y - 1 >= 0) {
                    this.moduleValue[i][y - 1].IsBlack = false;
                    this.moduleValue[i][y - 1].IsFilled = true;
                    this.moduleValue[i][y - 1].IsPDP = true;
                }

                this.moduleValue[x][j].IsBlack = true;
                this.moduleValue[x][j].IsFilled = true;
                this.moduleValue[x][j].IsPDP = true;

                this.moduleValue[x + 6][j].IsBlack = true;
                this.moduleValue[x + 6][j].IsFilled = true;
                this.moduleValue[x + 6][j].IsPDP = true;

                if (x + 7 < this.noOfModules) {
                    this.moduleValue[x + 7][j].IsBlack = false;
                    this.moduleValue[x + 7][j].IsFilled = true;
                    this.moduleValue[x + 7][j].IsPDP = true;
                }
                else if (x - 1 >= 0) {
                    this.moduleValue[x - 1][j].IsBlack = false;
                    this.moduleValue[x - 1][j].IsFilled = true;
                    this.moduleValue[x - 1][j].IsPDP = true;
                }
            }

            if (x + 7 < this.noOfModules && y + 7 < this.noOfModules) {
                this.moduleValue[x + 7][y + 7].IsBlack = false;
                this.moduleValue[x + 7][y + 7].IsFilled = true;
                this.moduleValue[x + 7][y + 7].IsPDP = true;
            }
            else if (x + 7 < this.noOfModules && y + 7 >= this.noOfModules) {
                this.moduleValue[x + 7][y - 1].IsBlack = false;
                this.moduleValue[x + 7][y - 1].IsFilled = true;
                this.moduleValue[x + 7][y - 1].IsPDP = true;
            }
            else if (x + 7 >= this.noOfModules && y + 7 < this.noOfModules) {
                this.moduleValue[x - 1][y + 7].IsBlack = false;
                this.moduleValue[x - 1][y + 7].IsFilled = true;
                this.moduleValue[x - 1][y + 7].IsPDP = true;
            }

            x++;
            y++;
            for (i = x, j = y; i < x + 5; i++, j++) {
                this.moduleValue[i][y].IsBlack = false;
                this.moduleValue[i][y].IsFilled = true;
                this.moduleValue[i][y].IsPDP = true;

                this.moduleValue[i][y + 4].IsBlack = false;
                this.moduleValue[i][y + 4].IsFilled = true;
                this.moduleValue[i][y + 4].IsPDP = true;

                this.moduleValue[x][j].IsBlack = false;
                this.moduleValue[x][j].IsFilled = true;
                this.moduleValue[x][j].IsPDP = true;

                this.moduleValue[x + 4][j].IsBlack = false;
                this.moduleValue[x + 4][j].IsFilled = true;
                this.moduleValue[x + 4][j].IsPDP = true;
            }

            x++;
            y++;
            for (i = x, j = y; i < x + 3; i++, j++) {
                this.moduleValue[i][y].IsBlack = true;
                this.moduleValue[i][y].IsFilled = true;
                this.moduleValue[i][y].IsPDP = true;

                this.moduleValue[i][y + 2].IsBlack = true;
                this.moduleValue[i][y + 2].IsFilled = true;
                this.moduleValue[i][y + 2].IsPDP = true;

                this.moduleValue[x][j].IsBlack = true;
                this.moduleValue[x][j].IsFilled = true;
                this.moduleValue[x][j].IsPDP = true;

                this.moduleValue[x + 2][j].IsBlack = true;
                this.moduleValue[x + 2][j].IsFilled = true;
                this.moduleValue[x + 2][j].IsPDP = true;
            }
            this.moduleValue[x + 1][y + 1].IsBlack = true;
            this.moduleValue[x + 1][y + 1].IsFilled = true;
            this.moduleValue[x + 1][y + 1].IsPDP = true;
        },

        /** 
         * @private
         */
        _drawAlignmentPattern: function (x, y) {
            var i, j;
            for (i = x - 2, j = y - 2; i < x + 3; i++, j++) {
                this.moduleValue[i][y - 2].IsBlack = true;
                this.moduleValue[i][y - 2].IsFilled = true;

                this.moduleValue[i][y + 2].IsBlack = true;
                this.moduleValue[i][y + 2].IsFilled = true;

                this.moduleValue[x - 2][j].IsBlack = true;
                this.moduleValue[x - 2][j].IsFilled = true;

                this.moduleValue[x + 2][j].IsBlack = true;
                this.moduleValue[x + 2][j].IsFilled = true;
            }

            for (i = x - 1, j = y - 1; i < x + 2; i++, j++) {
                this.moduleValue[i][y - 1].IsBlack = false;
                this.moduleValue[i][y - 1].IsFilled = true;

                this.moduleValue[i][y + 1].IsBlack = false;
                this.moduleValue[i][y + 1].IsFilled = true;

                this.moduleValue[x - 1][j].IsBlack = false;
                this.moduleValue[x - 1][j].IsFilled = true;

                this.moduleValue[x + 1][j].IsBlack = false;
                this.moduleValue[x + 1][j].IsFilled = true;
            }
            this.moduleValue[x][y].IsBlack = true;
            this.moduleValue[x][y].IsFilled = true;
        },

        /** 
         * @private
         */
        _getAlignmentPatternCoOrdinates: function () {
            var allign = new Array();
            switch (this.m_Version) {
                case 02:
                    allign = new Array(6, 18);
                    break;
                case 03:
                    allign = new Array(6, 22);
                    break;
                case 04:
                    allign = new Array(6, 26);
                    break;
                case 05:
                    allign = new Array(6, 30);
                    break;
                case 06:
                    allign = new Array(6, 34);
                    break;
                case 07:
                    allign = new Array(6, 22, 38);
                    break;
                case 08:
                    allign = new Array(6, 24, 42);
                    break;
                case 09:
                    allign = new Array(6, 26, 46);
                    break;
                case 10:
                    allign = new Array(6, 28, 50);
                    break;
                case 11:
                    allign = new Array(6, 30, 54);
                    break;
                case 12:
                    allign = new Array(6, 32, 58);
                    break;
                case 13:
                    allign = new Array(6, 34, 62);
                    break;
                case 14:
                    allign = new Array(6, 26, 46, 66);
                    break;
                case 15:
                    allign = new Array(6, 26, 48, 70);
                    break;
                case 16:
                    allign = new Array(6, 26, 50, 74);
                    break;
                case 17:
                    allign = new Array(6, 30, 54, 78);
                    break;
                case 18:
                    allign = new Array(6, 30, 56, 82);
                    break;
                case 19:
                    allign = new Array(6, 30, 58, 86);
                    break;
                case 20:
                    allign = new Array(6, 34, 62, 90);
                    break;
                case 21:
                    allign = new Array(6, 28, 50, 72, 94);
                    break;
                case 22:
                    allign = new Array(6, 26, 50, 74, 98);
                    break;
                case 23:
                    allign = new Array(6, 30, 54, 78, 102);
                    break;
                case 24:
                    allign = new Array(6, 28, 54, 80, 106);
                    break;
                case 25:
                    allign = new Array(6, 32, 58, 84, 110);
                    break;
                case 26:
                    allign = new Array(6, 30, 58, 86, 114);
                    break;
                case 27:
                    allign = new Array(6, 34, 62, 90, 118);
                    break;
                case 28:
                    allign = new Array(6, 26, 50, 74, 98, 122);
                    break;
                case 29:
                    allign = new Array(6, 30, 54, 78, 102, 126);
                    break;
                case 30:
                    allign = new Array(6, 26, 52, 78, 104, 130);
                    break;
                case 31:
                    allign = new Array(6, 30, 56, 82, 108, 134);
                    break;
                case 32:
                    allign = new Array(6, 34, 60, 86, 112, 138);
                    break;
                case 33:
                    allign = new Array(6, 30, 58, 86, 114, 142);
                    break;
                case 34:
                    allign = new Array(6, 34, 62, 90, 118, 146);
                    break;
                case 35:
                    allign = new Array(6, 30, 54, 78, 102, 126, 150);
                    break;
                case 36:
                    allign = new Array(6, 24, 50, 76, 102, 128, 154);
                    break;
                case 37:
                    allign = new Array(6, 28, 54, 80, 106, 132, 158);
                    break;
                case 38:
                    allign = new Array(6, 32, 58, 84, 110, 136, 162);
                    break;
                case 39:
                    allign = new Array(6, 26, 54, 82, 110, 138, 166);
                    break;
                case 40:
                    allign = new Array(6, 30, 58, 86, 114, 142, 170);
                    break;
            }
            return allign;
        },

        /** 
         * @private
         */
        _findECINumber: function (text) {
            var num = 3;
            for (var i = 0; i < text.length; i++) {
                var cCode = text.charCodeAt(i);
                if (cCode >= 32 && cCode <= 255) {
                    continue;
                }

                var check = cCode.toString(16);
                var CP437CharSet = new Array("2591", "2592", "2593", "2502", "2524", "2561", "2562", "2556", "2555", "2563", "2551", "2557", "255D", "255C", "255B", "2510", "2514", "2534", "252C", "251C", "2500", "253C", "255E", "255F", "255A", "2554", "2569", "2566", "2560", "2550", "256C", "2567", "2568", "2564", "2565", "2559", "2558", "2552", "2553", "256B", "256A", "2518", "250C", "2588", "2584", "258C", "2590", "2580", "25A0");
                var latin2CharSet = new Array("104", "2D8", "141", "13D", "15A", "160", "15E", "164", "179", "17D", "17B", "105", "2DB", "142", "13E", "15B", "2C7", "161", "15F", "165", "17A", "2DD", "17E", "17C", "154", "102", "139", "106", "10C", "118", "11A", "10E", "110", "143", "147", "150", "158", "16E", "170", "162", "155", "103", "13A", "107", "10D", "119", "11B", "10F", "111", "144", "148", "151", "159", "16F", "171", "163", "2D9");
                var latin3CharSet = new Array("126", "124", "130", "15E", "11E", "134", "17B", "127", "125", "131", "15F", "11F", "135", "17C", "10A", "108", "120", "11C", "16C", "15C", "10B", "109", "121", "11D", "16D", "15D");
                var latin4CharSet = new Array("104", "138", "156", "128", "13B", "160", "112", "122", "166", "17D", "105", "2DB", "157", "129", "13C", "2C7", "161", "113", "123", "167", "14A", "17E", "14B", "100", "12E", "10C", "118", "116", "12A", "110", "145", "14C", "136", "172", "168", "16A", "101", "12F", "10D", "119", "117", "12B", "111", "146", "14D", "137", "173", "169", "16B");
                var windows1250CharSet = new Array("141", "104", "15E", "17B", "142", "105", "15F", "13D", "13E", "17C");
                var windows1251CharSet = new Array("402", "403", "453", "409", "40A", "40C", "40B", "40F", "452", "459", "45A", "45C", "45B", "45F", "40E", "45E", "408", "490", "401", "404", "407", "406", "456", "491", "451", "454", "458", "405", "455", "457");
                var windows1252CharSet = new Array("20AC", "201A", "192", "201E", "2026", "2020", "2021", "2C6", "2030", "160", "2039", "152", "17D", "2018", "2019", "201C", "201D", "2022", "2013", "2014", "2DC", "2122", "161", "203A", "153", "17E", "178");
                var windows1256CharSet = new Array("67E", "679", "152", "686", "698", "688", "6AF", "6A9", "691", "153", "6BA", "6BE", "6C1");

                /** Check for CP437*/
                if (CP437CharSet.indexOf(check) != -1) {
                    num = 2;
                    break;
                }
                    /** Check for ISO/IEC 8859-2*/
                else if (latin2CharSet.indexOf(check) != -1) {
                    num = 4;
                    break;
                }
                    /** Check for ISO/IEC 8859-3*/
                else if (latin3CharSet.indexOf(check) != -1) {
                    num = 5;
                    break;
                }
                    /** Check for ISO/IEC 8859-4*/
                else if (latin4CharSet.indexOf(check) != -1) {
                    num = 6;
                    break;
                }
                    /** Check for ISO/IEC 8859-5*/
                else if (check >= 1025 && check <= 1119 && check != 1037 && check != 1104 && check != 1117) {
                    num = 7;
                    break;
                }
                    /** Check for ISO/IEC 8859-6*/
                else if ((check >= 1569 && check <= 1594) || (check >= 1600 && check <= 1618) || check == 1567 || check == 1563 || check == 1548) {
                    num = 8;
                    break;
                }
                    /** Check for ISO/IEC 8859-7*/
                else if ((check >= 900 && check <= 974) || check == 890) {
                    num = 9;
                    break;
                }
                    /** Check for ISO/IEC 8859-8*/
                else if (check >= 1488 && check <= 1514) {
                    num = 10;
                    break;
                }
                    /** Check for ISO/IEC 8859-8*/
                else if (check >= 3585 && check <= 3675) {
                    num = 13;
                    break;
                }
                    /** Check for Windows1250*/
                else if ((windows1250CharSet.indexOf(check) != 1) || (check >= 1569 && check <= 1610)) {
                    num = 21;
                    break;
                }
                    /** Check for Windows1251*/
                else if ((windows1251CharSet.indexOf(check) != 1) || (check >= 1040 && check <= 1103)) {
                    num = 22;
                    break;
                }
                    /** Check for Windows1252*/
                else if (windows1252CharSet.indexOf(check) != -1) {
                    num = 23;
                    break;
                }
                    /** Check for Windows1256*/
                else if (windows1256CharSet.indexOf(check) != -1) {
                    num = 24;
                    break;
                }
            }
            return num;
        },

        /** 
         * @private
         */
        _getNumberOfDataCodeWord: function () {
            var countOfDataCodeWord = 0;

            switch (this.m_Version) {
                case 01:
                    countOfDataCodeWord = 16;
                    break;
                case 02:
                    countOfDataCodeWord = 28;
                    break;
                case 03:
                    countOfDataCodeWord = 44;
                    break;
                case 04:
                    countOfDataCodeWord = 64;
                    break;
                case 05:
                    countOfDataCodeWord = 86;
                    break;
                case 06:
                    countOfDataCodeWord = 108;
                    break;
                case 07:
                    countOfDataCodeWord = 124;
                    break;
                case 08:
                    countOfDataCodeWord = 154;
                    break;
                case 09:
                    countOfDataCodeWord = 182;
                    break;
                case 10:
                    countOfDataCodeWord = 216;
                    break;
                case 11:
                    countOfDataCodeWord = 254;
                    break;
                case 12:
                    countOfDataCodeWord = 290;
                    break;
                case 13:
                    countOfDataCodeWord = 334;
                    break;
                case 14:
                    countOfDataCodeWord = 365;
                    break;
                case 15:
                    countOfDataCodeWord = 415;
                    break;
                case 16:
                    countOfDataCodeWord = 453;
                    break;
                case 17:
                    countOfDataCodeWord = 507;
                    break;
                case 18:
                    countOfDataCodeWord = 563;
                    break;
                case 19:
                    countOfDataCodeWord = 627;
                    break;
                case 20:
                    countOfDataCodeWord = 669;
                    break;
                case 21:
                    countOfDataCodeWord = 714;
                    break;
                case 22:
                    countOfDataCodeWord = 782;
                    break;
                case 23:
                    countOfDataCodeWord = 860;
                    break;
                case 24:
                    countOfDataCodeWord = 914;
                    break;
                case 25:
                    countOfDataCodeWord = 1000;
                    break;
                case 26:
                    countOfDataCodeWord = 1062;
                    break;
                case 27:
                    countOfDataCodeWord = 1128;
                    break;
                case 28:
                    countOfDataCodeWord = 1193;
                    break;
                case 29:
                    countOfDataCodeWord = 1267;
                    break;
                case 30:
                    countOfDataCodeWord = 1373;
                    break;
                case 31:
                    countOfDataCodeWord = 1455;
                    break;
                case 32:
                    countOfDataCodeWord = 1541;
                    break;
                case 33:
                    countOfDataCodeWord = 1631;
                    break;
                case 34:
                    countOfDataCodeWord = 1725;
                    break;
                case 35:
                    countOfDataCodeWord = 1812;
                    break;
                case 36:
                    countOfDataCodeWord = 1914;
                    break;
                case 37:
                    countOfDataCodeWord = 1992;
                    break;
                case 38:
                    countOfDataCodeWord = 2102;
                    break;
                case 39:
                    countOfDataCodeWord = 2216;
                    break;
                case 40:
                    countOfDataCodeWord = 2334;
                    break;
            }

            return countOfDataCodeWord;
        },

        /** 
         * Returns the number of error correcting codewords based on QR version.
         * @private
         */
        _getNumberOfErrorCorrectingCodeWords: function () {
            var index = (this.m_Version - 1) * 4;
            index += 1;
            var numberOfErrorCorrectingCodeWords = new Array(7, 10, 13, 17, 10, 16, 22, 28, 15, 26, 36, 44, 20, 36, 52, 64, 26, 48, 72, 88, 36, 64, 96, 112, 40, 72, 108, 130, 48, 88, 132, 156, 60, 110, 160, 192, 72, 130, 192, 224, 80, 150, 224, 264, 96, 176, 260, 308, 104, 198, 288, 352, 120, 216, 320, 384, 132, 240, 360, 432, 144, 280, 408, 480, 168, 308, 448, 532, 180, 338, 504, 588, 196, 364, 546, 650, 224, 416, 600, 700, 224, 442, 644, 750, 252, 476, 690, 816, 270, 504, 750, 900, 300, 560, 810, 960, 312, 588, 870, 1050, 336, 644, 952, 1110, 360, 700, 1020, 1200, 390, 728, 1050, 1260, 420, 784, 1140, 1350, 450, 812, 1200, 1440, 480, 868, 1290, 1530, 510, 924, 1350, 1620, 540, 980, 1440, 1710, 570, 1036, 1530, 1800, 570, 1064, 1590, 1890, 600, 1120, 1680, 1980, 630, 1204, 1770, 2100, 660, 1260, 1860, 2220, 720, 1316, 1950, 2310, 750, 1372, 2040, 2430);
            return numberOfErrorCorrectingCodeWords[index];
        },

        /** 
         * Returns the number of error correction blocks based on QR version.
         * @private
         */
        _getNumberOfErrorCorrectionBlocks: function () {
            var numberOfErrorCorrectionBlocks = new Array();
            switch (this.m_Version) {
                case 01:
                case 02:
                case 03:
                    numberOfErrorCorrectionBlocks[0] = 1;
                    break;
                case 04:
                case 05:
                    numberOfErrorCorrectionBlocks[0] = 2;
                    break;
                case 06:
                case 07:
                    numberOfErrorCorrectionBlocks[0] = 4;
                    break;
                case 08:
                    numberOfErrorCorrectionBlocks = new Array(2, 60, 38, 2, 61, 39);
                    break;
                case 09:
                    numberOfErrorCorrectionBlocks = new Array(3, 58, 36, 2, 59, 37);
                    break;
                case 10:
                    numberOfErrorCorrectionBlocks = new Array(4, 69, 43, 1, 70, 44);
                    break;
                case 11:
                    numberOfErrorCorrectionBlocks = new Array(1, 80, 50, 4, 81, 51);
                    break;
                case 12:
                    numberOfErrorCorrectionBlocks = new Array(6, 58, 36, 2, 59, 37);
                    break;
                case 13:
                    numberOfErrorCorrectionBlocks = new Array(8, 59, 37, 1, 60, 38);
                    break;
                case 14:
                    numberOfErrorCorrectionBlocks = new Array(4, 64, 40, 5, 65, 41);
                    break;
                case 15:
                    numberOfErrorCorrectionBlocks = new Array(5, 65, 41, 5, 66, 42);
                    break;
                case 16:
                    numberOfErrorCorrectionBlocks = new Array(7, 73, 45, 3, 74, 46);
                    break;
                case 17:
                    numberOfErrorCorrectionBlocks = new Array(10, 74, 46, 1, 75, 47);
                    break;
                case 18:
                    numberOfErrorCorrectionBlocks = new Array(9, 69, 43, 4, 70, 44);
                    break;
                case 19:
                    numberOfErrorCorrectionBlocks = new Array(3, 70, 44, 11, 71, 45);
                    break;
                case 20:
                    numberOfErrorCorrectionBlocks = new Array(3, 67, 41, 13, 68, 42);
                    break;
                case 21:
                case 22:
                    numberOfErrorCorrectionBlocks[0] = 17;
                    break;
                case 23:
                    numberOfErrorCorrectionBlocks = new Array(4, 75, 47, 14, 76, 48);
                    break;
                case 24:
                    numberOfErrorCorrectionBlocks = new Array(6, 73, 45, 14, 74, 46);
                    break;
                case 25:
                    numberOfErrorCorrectionBlocks = new Array(8, 75, 47, 13, 76, 48);
                    break;
                case 26:
                    numberOfErrorCorrectionBlocks = new Array(19, 74, 46, 4, 75, 47);
                    break;
                case 27:
                    numberOfErrorCorrectionBlocks = new Array(22, 73, 45, 3, 74, 46);
                    break;
                case 28:
                    numberOfErrorCorrectionBlocks = new Array(3, 73, 45, 23, 74, 46);
                    break;
                case 29:
                    numberOfErrorCorrectionBlocks = new Array(21, 73, 45, 7, 74, 46);
                    break;
                case 30:
                    numberOfErrorCorrectionBlocks = new Array(19, 75, 47, 10, 76, 48);
                    break;
                case 31:
                    numberOfErrorCorrectionBlocks = new Array(2, 74, 46, 29, 75, 47);
                    break;
                case 32:
                    numberOfErrorCorrectionBlocks = new Array(10, 74, 46, 23, 75, 47);
                    break;
                case 33:
                    numberOfErrorCorrectionBlocks = new Array(14, 74, 46, 21, 75, 47);
                    break;
                case 34:
                    numberOfErrorCorrectionBlocks = new Array(14, 74, 46, 23, 75, 47);
                    break;
                case 35:
                    numberOfErrorCorrectionBlocks = new Array(12, 75, 47, 26, 76, 48);
                    break;
                case 36:
                    numberOfErrorCorrectionBlocks = new Array(6, 75, 47, 34, 76, 48);
                    break;
                case 37:
                    numberOfErrorCorrectionBlocks = new Array(29, 74, 46, 14, 75, 47);
                    break;
                case 38:
                    numberOfErrorCorrectionBlocks = new Array(13, 74, 46, 32, 75, 47);
                    break;
                case 39:
                    numberOfErrorCorrectionBlocks = new Array(40, 75, 47, 7, 76, 48);
                    break;
                case 40:
                    numberOfErrorCorrectionBlocks = new Array(18, 75, 47, 31, 76, 48);
                    break;
            }

            return numberOfErrorCorrectionBlocks;
        },

        /** 
         * Returns the end values based on qr version
         * @private
         */
        _getEnd: function () {
            var endValues = new Array(208, 359, 567, 807, 1079, 1383, 1568, 1936, 2336, 2768, 3232, 3728, 4256, 4651, 5243, 5867, 6523, 7211, 7931, 8683, 9252, 10068, 10916, 11796, 12708, 13652, 14628, 15371, 16411, 17483, 18587, 19723, 20891, 22091, 23008, 24272, 25568, 26896, 28256, 29648);
            return endValues[this.m_Version - 1];
        },

        /** 
         * Returns the data capacity based on QR version
         * @private
         */
        _getDataCapacity: function () {
            var dataCapacityValues = new Array(26, 44, 70, 100, 134, 172, 196, 242, 292, 346, 404, 466, 532, 581, 655, 733, 815, 901, 991, 1085, 1156, 1258, 1364, 1474, 1588, 1706, 1828, 1921, 2051, 2185, 2323, 2465, 2611, 2761, 2876, 3034, 3196, 3362, 3532, 3706);
            return dataCapacityValues[this.m_Version - 1];
        },

        /** 
         * @private
         */
        _getFormatInformation: function () {
            var formatInformation = new Array(1, 1, 0, 1, 0, 0, 1, 0, 1, 1, 0, 1, 1, 0, 1);
            return formatInformation;
        },

        /** 
         * Returns the QR version
         * @private
         */
        _getVersionInformation: function () {
            var versionInformation = new Array();
            switch (this.m_Version) {
                case 07:
                    versionInformation = new Array(0, 0, 1, 0, 1, 0, 0, 1, 0, 0, 1, 1, 1, 1, 1, 0, 0, 0);
                    break;
                case 08:
                    versionInformation = new Array(0, 0, 1, 1, 1, 1, 0, 1, 1, 0, 1, 0, 0, 0, 0, 1, 0, 0);
                    break;
                case 09:
                    versionInformation = new Array(1, 0, 0, 1, 1, 0, 0, 1, 0, 1, 0, 1, 1, 0, 0, 1, 0, 0);
                    break;
                case 10:
                    versionInformation = new Array(1, 1, 0, 0, 1, 0, 1, 1, 0, 0, 1, 0, 0, 1, 0, 1, 0, 0);
                    break;
                case 11:
                    versionInformation = new Array(0, 1, 1, 0, 1, 1, 1, 1, 1, 1, 0, 1, 1, 1, 0, 1, 0, 0);
                    break;
                case 12:
                    versionInformation = new Array(0, 1, 0, 0, 0, 1, 1, 0, 1, 1, 1, 0, 0, 0, 1, 1, 0, 0);
                    break;
                case 13:
                    versionInformation = new Array(1, 1, 1, 0, 0, 0, 1, 0, 0, 0, 0, 1, 1, 0, 1, 1, 0, 0);
                    break;
                case 14:
                    versionInformation = new Array(1, 0, 1, 1, 0, 0, 0, 0, 0, 1, 1, 0, 0, 1, 1, 1, 0, 0);
                    break;
                case 15:
                    versionInformation = new Array(0, 0, 0, 1, 0, 1, 0, 0, 1, 0, 0, 1, 1, 1, 1, 1, 0, 0);
                    break;
                case 16:
                    versionInformation = new Array(0, 0, 0, 1, 1, 1, 1, 0, 1, 1, 0, 1, 0, 0, 0, 0, 1, 0);
                    break;
                case 17:
                    versionInformation = new Array(1, 0, 1, 1, 1, 0, 1, 0, 0, 0, 1, 0, 1, 0, 0, 0, 1, 0);
                    break;
                case 18:
                    versionInformation = new Array(1, 1, 1, 0, 1, 0, 0, 0, 0, 1, 0, 1, 0, 1, 0, 0, 1, 0);
                    break;
                case 19:
                    versionInformation = new Array(0, 1, 0, 0, 1, 1, 0, 0, 1, 0, 1, 0, 1, 1, 0, 0, 1, 0);
                    break;
                case 20:
                    versionInformation = new Array(0, 1, 1, 0, 0, 1, 0, 1, 1, 0, 0, 1, 0, 0, 1, 0, 1, 0);
                    break;
                case 21:
                    versionInformation = new Array(1, 1, 0, 0, 0, 0, 0, 1, 0, 1, 1, 0, 1, 0, 1, 0, 1, 0);
                    break;
                case 22:
                    versionInformation = new Array(1, 0, 0, 1, 0, 0, 1, 1, 0, 0, 0, 1, 0, 1, 1, 0, 1, 0);
                    break;
                case 23:
                    versionInformation = new Array(0, 0, 1, 1, 0, 1, 1, 1, 1, 1, 1, 0, 1, 1, 1, 0, 1, 0);
                    break;
                case 24:
                    versionInformation = new Array(0, 0, 1, 0, 0, 0, 1, 1, 0, 1, 1, 1, 0, 0, 0, 1, 1, 0);
                    break;
                case 25:
                    versionInformation = new Array(1, 0, 0, 0, 0, 1, 1, 1, 1, 0, 0, 0, 1, 0, 0, 1, 1, 0);
                    break;
                case 26:
                    versionInformation = new Array(1, 1, 0, 1, 0, 1, 0, 1, 1, 1, 1, 1, 0, 1, 0, 1, 1, 0);
                    break;
                case 27:
                    versionInformation = new Array(0, 1, 1, 1, 0, 0, 0, 1, 0, 0, 0, 0, 1, 1, 0, 1, 1, 0);
                    break;
                case 28:
                    versionInformation = new Array(0, 1, 0, 1, 1, 0, 0, 0, 0, 0, 1, 1, 0, 0, 1, 1, 1, 0);
                    break;
                case 29:
                    versionInformation = new Array(1, 1, 1, 1, 1, 1, 0, 0, 1, 1, 0, 0, 1, 0, 1, 1, 1, 0);
                    break;
                case 30:
                    versionInformation = new Array(1, 0, 1, 0, 1, 1, 1, 0, 1, 0, 1, 1, 0, 1, 1, 1, 1, 0);
                    break;
                case 31:
                    versionInformation = new Array(0, 0, 0, 0, 1, 0, 1, 0, 0, 1, 0, 0, 1, 1, 1, 1, 1, 0);
                    break;
                case 32:
                    versionInformation = new Array(1, 0, 1, 0, 1, 0, 1, 1, 1, 0, 0, 1, 0, 0, 0, 0, 0, 1);
                    break;
                case 33:
                    versionInformation = new Array(0, 0, 0, 0, 1, 1, 1, 1, 0, 1, 1, 0, 1, 0, 0, 0, 0, 1);
                    break;
                case 34:
                    versionInformation = new Array(0, 1, 0, 1, 1, 1, 0, 1, 0, 0, 0, 1, 0, 1, 0, 0, 0, 1);
                    break;
                case 35:
                    versionInformation = new Array(1, 1, 1, 1, 1, 0, 0, 1, 1, 1, 1, 0, 1, 1, 0, 0, 0, 1);
                    break;
                case 36:
                    versionInformation = new Array(1, 1, 0, 1, 0, 0, 0, 0, 1, 1, 0, 1, 0, 0, 1, 0, 0, 1);
                    break;
                case 37:
                    versionInformation = new Array(0, 1, 1, 1, 0, 1, 0, 0, 0, 0, 1, 0, 1, 0, 1, 0, 0, 1);
                    break;
                case 38:
                    versionInformation = new Array(0, 0, 1, 0, 0, 1, 1, 0, 0, 1, 0, 1, 0, 1, 1, 0, 0, 1);
                    break;
                case 39:
                    versionInformation = new Array(1, 0, 0, 0, 0, 0, 1, 0, 1, 0, 1, 0, 1, 1, 1, 0, 0, 1);
                    break;
                case 40:
                    versionInformation = new Array(1, 0, 0, 1, 0, 1, 1, 0, 0, 0, 1, 1, 0, 0, 0, 1, 0, 1);
                    break;
            }

            return versionInformation;
        },

        /** 
         * Returns the code points of the text
         * @private
         */
        _getStringCodePoints: (function () {
            function surrogatePairToCodePoint(charCode1, charCode2) {
                return ((charCode1 & 0x3FF) << 10) + (charCode2 & 0x3FF) + 0x10000;
            }

            /** Read string in character by character and create an array of code points.*/
            return function (str) {
                var codePoints = [], i = 0, charCode;
                while (i < str.length) {
                    charCode = str.charCodeAt(i);
                    if ((charCode & 0xF800) == 0xD800)
                        codePoints.push(surrogatePairToCodePoint(charCode, str.charCodeAt(++i)));
                    else
                        codePoints.push(charCode);
                    ++i;
                }
                return codePoints;
            }
        })(),

        /** 
         * Creates 2 dimensional array.
         * @private
         */
        _create2DArray: function (rows, columns) {
            rows = parseInt(rows);
            columns = parseInt(columns);
            var twoDArray = new Array(rows);

            for (var i = 0; i < rows; i++)
                twoDArray[i] = new Array(columns);

            return twoDArray;
        },

        /** 
         * Builds datamatrix barcode.
         * @private
         */
        _buildDataMatrix: function () {
            var symbolAttributes = [[10, 10, 1, 1, 3, 5, 1, 3], [12, 12, 1, 1, 5, 7, 1, 5], [14, 14, 1, 1, 8, 10, 1, 8], [16, 16, 1, 1, 12, 12, 1, 12], [18, 18, 1, 1, 18, 14, 1, 18], [20, 20, 1, 1, 22, 18, 1, 22], [22, 22, 1, 1, 30, 20, 1, 30], [24, 24, 1, 1, 36, 24, 1, 36], [26, 26, 1, 1, 44, 28, 1, 44], [32, 32, 2, 2, 62, 36, 1, 62], [36, 36, 2, 2, 86, 42, 1, 86], [40, 40, 2, 2, 114, 48, 1, 114], [44, 44, 2, 2, 144, 56, 1, 144], [48, 48, 2, 2, 174, 68, 1, 174], [52, 52, 2, 2, 204, 84, 2, 102], [64, 64, 4, 4, 280, 112, 2, 140], [72, 72, 4, 4, 368, 144, 4, 92], [80, 80, 4, 4, 456, 192, 4, 114], [88, 88, 4, 4, 576, 224, 4, 144], [96, 96, 4, 4, 696, 272, 4, 174], [104, 104, 4, 4, 816, 336, 6, 136], [120, 120, 6, 6, 1050, 408, 6, 175], [132, 132, 6, 6, 1304, 496, 8, 163], [144, 144, 6, 6, 1558, 620, 10, 156], [8, 18, 1, 1, 5, 7, 1, 5], [8, 32, 2, 1, 10, 11, 1, 10], [12, 26, 1, 1, 16, 14, 1, 16], [12, 36, 2, 1, 22, 18, 1, 22], [16, 36, 2, 1, 32, 24, 1, 32], [16, 48, 2, 1, 49, 28, 1, 49]];
            var codeWords = this._getStringCodePoints(this.Model.text);
            var correctionCodewords;
            var symAttr;

            /** ASCII Encoder.*/
            var encodedPoints = new Array(); var n = 0;
            for (var e = 0; e < codeWords.length; e++) {
                if (codeWords[e] < 127)
                    encodedPoints[n] = codeWords[e] + 1;
                else {
                    encodedPoints[n++] = 235;
                    encodedPoints[n] = codeWords[e] - 127;
                }
                n++;
            }

            /** Find symbol attribute.*/
            for (var e = 0; e < symbolAttributes.length; e++) {
                if (symbolAttributes[e][4] >= codeWords.length) {
                    symAttr = symbolAttributes[e];
                    break;
                }
            }

            if (symAttr[4] > encodedPoints.length) { //pad codewords
                var l = encodedPoints.length;
                var temp = new Array(symAttr[4]);
                var t = 0;

                for (var i = 0; i < l; i++)
                    temp[i] = encodedPoints[i];

                if (l < (symAttr[4] + 1))
                    temp[l] = 129; // denotes padding.

                l++;
                while (l < symAttr[4]) {	// more padding
                    var v = 129 + (((l + 1) * 149) % 253) + 1;
                    if (v > 254)
                        v -= 254;
                    temp[l++] = v;
                }

                encodedPoints = temp;
            }

            correctionCodewords = symAttr[5];
            errCorrArray = this._errorCorrection(correctionCodewords, encodedPoints);

            var finalCodeWord = encodedPoints.concat(errCorrArray);
            var dataMatrixArray = this._createMatrix(finalCodeWord, symAttr);

            var w = dataMatrixArray.length * this.Model.xDimension;
            var h = dataMatrixArray[0].length * this.Model.xDimension;

            /** Prepare the canvas.*/
            this.Model.width = w;
            this.canvasEl[0].setAttribute("width", this.Model.width);
            this.BarcodeEl.css("width", this.Model.width);

            this.contextEl.beginPath();
            this.contextEl.fillStyle = 'white';
            this.contextEl.fillRect(0, 0, w, h);
            this.contextEl.closePath();

            /** Draw DataMatrix.*/
            this._drawDataMatrix(dataMatrixArray, w, h);
        },

        /** 
         * Draws datamatrix barcode.
         * @private
         */
        _drawDataMatrix: function (dmArray, w, h) {
            this.Model.height = h;
            this.canvasEl[0].setAttribute("height", this.Model.height);
            this.BarcodeEl.css("height", this.Model.height);
            var dim = parseInt(this.Model.xDimension);
            var y = 0;

            this.contextEl.beginPath();

            for (var i = 0; i < dmArray.length; i++) {
                var x = 0;
                for (var j = 0; j < dmArray[i].length; j++) {
                    var colorBrush = 'white';
                    if (dmArray[i][j] == 1)
                        colorBrush = 'black';
                    this.contextEl.fillStyle = colorBrush;
                    this.contextEl.fillRect(x, y, dim, dim)
                    x += dim;
                }
                y += dim;
            }

            this.contextEl.closePath();
            this.contextEl.stroke();
        },

        /** 
         * Creates matrix.
         * @private
         */
        _createMatrix: function (codeWord, symAttr) {
            var x, y, NC, NR;
            var places = new Array();

            var W = symAttr[1], H = symAttr[0];
            var FW = W / symAttr[2], FH = H / symAttr[3];

            NC = W - 2 * (W / FW);
            NR = H - 2 * (H / FH);

            var places = new Array(NC * NR);
            this._ecc200placement(places, NR, NC);
            var matrix = new Array(W * H);

            for (var i = 0; i < matrix.length; i++)
                matrix[i] = 0;

            for (y = 0; y < H; y += FH) {
                for (x = 0; x < W; x++)
                    matrix[parseInt(y * W) + x] = 1;
                for (x = 0; x < W; x += 2)
                    matrix[parseInt((y + FH - 1) * W) + x] = 1;
            }
            for (x = 0; x < W; x += FW) {
                for (y = 0; y < H; y++)
                    matrix[parseInt(y * W) + x] = 1;
                for (y = 0; y < H; y += 2)
                    matrix[parseInt(y * W) + x + FW - 1] = 1;
            }
            for (y = 0; y < NR; y++) {
                for (x = 0; x < NC; x++) {
                    var v = places[((NR - y - 1) * NC) + x];
                    if (v == 1 || v > 7 && (codeWord[(v >> 3) - 1] & (1 << (v & 7))) != 0) {
                        var inde = parseInt((parseInt(1 + y + (2 * parseInt(y / (FH - 2)))) * W) + 1 + x + +(2 * parseInt(x / (FW - 2))));
                        matrix[inde] = 1;
                    }
                }
            }

            var _w = symAttr[1], _h = symAttr[0];

            var tempArray = this._create2DArray(_w, _h);

            for (var x1 = 0; x1 < _w; x1++) {
                for (var y1 = 0; y1 < _h; y1++) {
                    tempArray[x1][y1] = matrix[(_w * y1) + x1];
                }
            }

            /** Rotate the array left.*/
            var tempArray2 = this._create2DArray(_h, _w);

            for (var i = 0; i < _h; i++) {
                for (var j = 0; j < _w; j++) {
                    tempArray2[_h - 1 - i][j] = tempArray[j][i];
                }
            }

            var w = symAttr[0] + (this.Model.quietZone.top + this.Model.quietZone.bottom);
            var h = symAttr[1] + (this.Model.quietZone.left + this.Model.quietZone.right);
            var quietZone = this.Model.quietZone.all;

            var dataMatrixArray = this._create2DArray(w, h);

            for (var i = 0; i < w; i++)
                for (var j = 0; j < h; j++)
                    dataMatrixArray[i][j] = 0;

            for (var i = quietZone; i < w - quietZone; i++) {
                for (var j = quietZone; j < h - quietZone; j++) {
                    dataMatrixArray[i][j] = tempArray2[i - quietZone][j - quietZone];
                }
            }

            return dataMatrixArray;
        },

        /** 
         * @private
         */
        _ecc200placement: function (array, NR, NC) {
            var r, c, p;
            // invalidate
            for (r = 0; r < NR; r++)
                for (c = 0; c < NC; c++)
                    array[r * NC + c] = 0;
            // start
            p = 1;
            r = 4;
            c = 0;
            do {
                // check corner
                if (r == NR && !(c != 0))
                    this._ecc200placementcornerA(array, NR, NC, p++);
                if ((r == NR - 2) && !(c != 0) && ((NC % 4) != 0))
                    this._ecc200placementcornerB(array, NR, NC, p++);
                if (r == NR - 2 && !(c != 0) && (NC % 8) == 4)
                    this._ecc200placementcornerC(array, NR, NC, p++);
                if (r == NR + 4 && c == 2 && !((NC % 8) != 0))
                    this._ecc200placementcornerD(array, NR, NC, p++);
                // up/right
                do {
                    if (r < NR && c >= 0 && !(array[r * NC + c] != 0)) {
                        this._ecc200placementblock(array, NR, NC, r, c, p++);
                    }
                    r -= 2;
                    c += 2;
                }
                while (r >= 0 && c < NC);
                r++;
                c += 3;
                // down/left
                do {
                    if (r > -1) {
                        if (c < NC && !(array[r * NC + c] != 0)) {
                            this._ecc200placementblock(array, NR, NC, r, c, p++);
                        }
                    }
                    r += 2;
                    c -= 2;
                }
                while (r < NR && c >= 0);
                r += 3;
                c++;
            }
            while (r < NR || c < NC);
            // unfilled corner
            if (!(array[NR * NC - 1] != 0))
                array[NR * NC - 1] = array[NR * NC - NC - 2] = 1;
        },

        /** 
         * @private
         */
        _ecc200placementcornerA: function (array, NR, NC, p) {
            this._ecc200placementbit(array, NR, NC, NR - 1, 0, p, 7);
            this._ecc200placementbit(array, NR, NC, NR - 1, 1, p, 6);
            this._ecc200placementbit(array, NR, NC, NR - 1, 2, p, 5);
            this._ecc200placementbit(array, NR, NC, 0, NC - 2, p, 4);
            this._ecc200placementbit(array, NR, NC, 0, NC - 1, p, 3);
            this._ecc200placementbit(array, NR, NC, 1, NC - 1, p, 2);
            this._ecc200placementbit(array, NR, NC, 2, NC - 1, p, 1);
            this._ecc200placementbit(array, NR, NC, 3, NC - 1, p, 0);
        },

        /** 
         * @private
         */
        _ecc200placementcornerB: function (array, NR, NC, p) {
            this._ecc200placementbit(array, NR, NC, NR - 3, 0, p, 7);
            this._ecc200placementbit(array, NR, NC, NR - 2, 0, p, 6);
            this._ecc200placementbit(array, NR, NC, NR - 1, 0, p, 5);
            this._ecc200placementbit(array, NR, NC, 0, NC - 4, p, 4);
            this._ecc200placementbit(array, NR, NC, 0, NC - 3, p, 3);
            this._ecc200placementbit(array, NR, NC, 0, NC - 2, p, 2);
            this._ecc200placementbit(array, NR, NC, 0, NC - 1, p, 1);
            this._ecc200placementbit(array, NR, NC, 1, NC - 1, p, 0);
        },

        /** 
         * @private
         */
        _ecc200placementcornerC: function (array, NR, NC, p) {
            this._ecc200placementbit(array, NR, NC, NR - 3, 0, p, 7);
            this._ecc200placementbit(array, NR, NC, NR - 2, 0, p, 6);
            this._ecc200placementbit(array, NR, NC, NR - 1, 0, p, 5);
            this._ecc200placementbit(array, NR, NC, 0, NC - 2, p, 4);
            this._ecc200placementbit(array, NR, NC, 0, NC - 1, p, 3);
            this._ecc200placementbit(array, NR, NC, 1, NC - 1, p, 2);
            this._ecc200placementbit(array, NR, NC, 2, NC - 1, p, 1);
            this._ecc200placementbit(array, NR, NC, 3, NC - 1, p, 0);
        },

        /** 
         * @private
         */
        _ecc200placementcornerD: function (array, NR, NC, p) {
            this._ecc200placementbit(array, NR, NC, NR - 1, 0, p, 7);
            this._ecc200placementbit(array, NR, NC, NR - 1, NC - 1, p, 6);
            this._ecc200placementbit(array, NR, NC, 0, NC - 3, p, 5);
            this._ecc200placementbit(array, NR, NC, 0, NC - 2, p, 4);
            this._ecc200placementbit(array, NR, NC, 0, NC - 1, p, 3);
            this._ecc200placementbit(array, NR, NC, 1, NC - 3, p, 2);
            this._ecc200placementbit(array, NR, NC, 1, NC - 2, p, 1);
            this._ecc200placementbit(array, NR, NC, 1, NC - 1, p, 0);
        },

        /** 
         * @private
         */
        _ecc200placementblock: function (array, NR, NC, r, c, p) {
            this._ecc200placementbit(array, NR, NC, r - 2, c - 2, p, 7);
            this._ecc200placementbit(array, NR, NC, r - 2, c - 1, p, 6);
            this._ecc200placementbit(array, NR, NC, r - 1, c - 2, p, 5);
            this._ecc200placementbit(array, NR, NC, r - 1, c - 1, p, 4);
            this._ecc200placementbit(array, NR, NC, r - 1, c - 0, p, 3);
            this._ecc200placementbit(array, NR, NC, r - 0, c - 2, p, 2);
            this._ecc200placementbit(array, NR, NC, r - 0, c - 1, p, 1);
            this._ecc200placementbit(array, NR, NC, r - 0, c - 0, p, 0);
        },

        /** 
         * @private
         */
        _ecc200placementbit: function (array, NR, NC, r, c, p, b) {
            if (r < 0) {
                r += NR;
                c += 4 - ((NR + 4) % 8);
            }
            if (c < 0) {
                c += NC;
                r += 4 - ((NC + 4) % 8);
            }
            array[r * NC + c] = (p << 3) + b;
        },

        /** 
         * Adds error correction codeword.
         * @private
         */
        _errorCorrection: function (num, codeWords) {
            var i, k, index = 1;
            var rsPolynomial;
            var temp = new Array(num + 1);

            /** Create log array.*/
            {
                var m_log = new Array(256);
                var m_aLog = new Array(256);

                m_log[0] = -255;
                m_aLog[0] = 1;

                for (var i = 1; i <= 255; i++) {
                    m_aLog[i] = m_aLog[i - 1] * 2;

                    if (m_aLog[i] >= 256)
                        m_aLog[i] = m_aLog[i] ^ 301;

                    m_log[m_aLog[i]] = i;
                }
            }

            /** CreateRSPolynomial.*/
            temp[0] = 1;
            for (i = 1; i <= num; i++) {
                temp[i] = 1;
                for (k = i - 1; k > 0; k--) {
                    if (temp[k] != 0)
                        temp[k] = m_aLog[(m_log[temp[k]] + index) % 255];
                    temp[k] ^= temp[k - 1];
                }
                temp[0] = m_aLog[(m_log[temp[0]] + index) % 255];
                index++;
            }

            rsPolynomial = new Array(num);
            for (var e = 0; e < rsPolynomial.length; e++)
                rsPolynomial[e] = temp[e];

            var t = 0;

            /** Create error correction array.*/
            var errCorrArray = new Array(num);

            for (var i = 0; i < num; i++)
                errCorrArray[i] = 0;

            for (var i = 0; i < codeWords.length; i++) {
                t = errCorrArray[num - 1] ^ codeWords[i];
                for (var j = num - 1; j > 0; j--) {
                    if (t != 0 && rsPolynomial[j] != 0)
                        errCorrArray[j] = (errCorrArray[j - 1] ^ m_aLog[(m_log[t] + m_log[rsPolynomial[j]]) % 255]);
                    else
                        errCorrArray[j] = errCorrArray[j - 1];
                }
                if (t != 0 && rsPolynomial[0] != 0)
                    errCorrArray[0] = (m_aLog[(m_log[t] + m_log[rsPolynomial[0]]) % 255]);
                else
                    errCorrArray[0] = 0;
            }

            errCorrArray = errCorrArray.reverse();
            return errCorrArray;
        },

        /** 
         * Validates the text against the barcode type.
         * @private
         */
        _validateText: function (str, exp) {
            var patt = new RegExp(exp);
            return patt.test(str);
        },

        /** 
         * @private
         */
        _getCharWidth: function (encodedText, symbolTable) {
            var charW = 0;
            var charArray = encodedText.split("");
            for (var e = 0; e < charArray.length; e++)
                charW = charW + intercharacterGap + this._getWidth(symbolTable, charArray[e]);

            return charW;
        },

        /** 
         * @private
         */
        _getWidth: function (symbolTable, c) {
            var check = false;
            var wid = 0;
            for (var e = 0; e < symbolTable.length; e++) {
                if (symbolTable[e][0] == c) {
                    var bars = symbolTable[e][2];
                    for (var i = 0; i < bars.length; i++) {
                        if (check == false) {
                            if (bars.length % 2 != 0)
                                continuous = true;
                        }

                        wid = wid + (bars[i] * this.Model.narrowBarWidth);
                        check = true;
                    }
                    return wid;
                }
            }
            return wid;
        },

        /** 
         * Draws 1 dimensional barcode.
         * @private
         */
        _draw1DBarcode: function (encodedText, symbolTable) {
            if (symbolTable.length > 0) {
                var charArray = encodedText.split("");
                var left = this.Model.quietZone.left;
                var top = this.Model.quietZone.top;
                this.Model.height = top + this.Model.barHeight + this.Model.quietZone.bottom;

                var textSize = parseInt(this.textFont.size.replace('px', ''));
                if (this.Model.displayText) {
                    this.Model.height += (textSize + this.Model.barcodeToTextGapHeight);
                }
                this.canvasEl[0].setAttribute("height", this.Model.height);
                this.BarcodeEl.css("height", this.Model.height);
                var wid = this.Model.width;
                var hei = this.Model.height;
                this.contextEl.beginPath();

                this.contextEl.fillStyle = 'white';
                this.contextEl.fillRect(0, 0, wid, hei);

                for (var e = 0; e < charArray.length; e++) {
                    for (var h = 0; h < symbolTable.length; h++) {
                        if (symbolTable[h][0] == charArray[e]) {
                            var bars = symbolTable[h][2];
                            for (var i = 0; i < bars.length; i++) {
                                var barWidth = bars[i] * this.Model.narrowBarWidth;
                                if (i % 2 == 0) {
                                    this.contextEl.fillStyle = this.Model.darkBarColor;
                                    this.contextEl.fillRect(left, top, barWidth, this.Model.barHeight)
                                }
                                left += barWidth;
                            }
                            if (bars.length % 2 != 0)
                                left += intercharacterGap;
                        }
                    }
                }

                this.contextEl.closePath();
                this.contextEl.stroke();

                this.Model.width = left += (this.Model.quietZone.right);

                /** Draws text for 1D barcodes.*/
                if (this.Model.displayText) {
                    var txtF = (this.textFont.fontStyle.toLowerCase() == "regular") ? "" : +this.textFont.fontStyle.toLowerCase() + " ";
                    txtF += this.textFont.size + " " + this.textFont.fontFamily.toLowerCase() + ",sans-serif";
                    this.contextEl.font = txtF;

                    var textWidth = 0;
                    textWidth = this.contextEl.measureText(this.Model.text).width;
                    if (this.Model.width < textWidth) {
                        this.Model.width = textWidth;
                        this.canvasEl[0].setAttribute("width", this.Model.width);
                        this.BarcodeEl.css("width", this.Model.width);
                    }

                    this.contextEl.beginPath();
                    this.contextEl.fillStyle = this.Model.textColor;
                    this.contextEl.textAlign = "center";
                    hei = hei - textSize + textSize / 2;
                    this.contextEl.fillText(this.Model.text, (this.Model.width / 2), hei);
                    this.contextEl.closePath();
                    this.contextEl.stroke();
                }
            }
        }
    });

    /**
     * Enum for Symbol type.
	 * @enum {string}
     * @global
     */
    ej.Barcode.SymbologyType = {
        /** Specifies Code 39 Barcode. */
        code39: "code39",
        /** Specifies Code 39 Extended Barcode. */
        code39Extended: "code39extended",
        /** Specifies Code 11 Barcode. */
        code11: "code11",
        /** Specifies Codabar Barcode. */
        codabar: "codabar",
        /** Specifies Code 32 Barcode. */
        code32: "code32",
        /** Specifies Code 93 Barcode. */
        code93: "code93",
        /** Specifies Code 93 Extended Barcode. */
        code93Extended: "code93extended",
        /** Specifies Code 128 A Barcode. */
        code128A: "code128a",
        /** Specifies Code 128 B Barcode. */
        code128B: "code128b",
        /** Specifies Code 128 C Barcode. */
        code128C: "code128c",
        /** Specifies DataMatrix Barcode. */
        dataMatrix: "datamatrix",
        /** Specifies QR Barcode. */
        qrBarcode: "qrbarcode"
    };

    /**
     * Enum for Symbol type.
     * @readonly
     * @private
     */
    ej.Barcode.symbologySettings = {
        /** Specifies symbology settinngs for the Code 39 Barcode. */
        code39: {
            startSymbol: '*',
            stopSymbol: '*',
            validatorExp: "^[\x41-\x5A\x30-\x39\x20\-\.\$\/\+\%\ ]+$",
            symbolTable: [["0", 0, [1, 1, 1, 3, 3, 1, 3, 1, 1]], ["1", 1, [3, 1, 1, 3, 1, 1, 1, 1, 3]], ["2", 2, [1, 1, 3, 3, 1, 1, 1, 1, 3]], ["3", 3, [3, 1, 3, 3, 1, 1, 1, 1, 1]], ["4", 4, [1, 1, 1, 3, 3, 1, 1, 1, 3]], ["5", 5, [3, 1, 1, 3, 3, 1, 1, 1, 1]], ["6", 6, [1, 1, 3, 3, 3, 1, 1, 1, 1]], ["7", 7, [1, 1, 1, 3, 1, 1, 3, 1, 3]], ["8", 8, [3, 1, 1, 3, 1, 1, 3, 1, 1]], ["9", 9, [1, 1, 3, 3, 1, 1, 3, 1, 1]], ["A", 10, [3, 1, 1, 1, 1, 3, 1, 1, 3]], ["B", 11, [1, 1, 3, 1, 1, 3, 1, 1, 3]], ["C", 12, [3, 1, 3, 1, 1, 3, 1, 1, 1]], ["D", 13, [1, 1, 1, 1, 3, 3, 1, 1, 3]], ["E", 14, [3, 1, 1, 1, 3, 3, 1, 1, 1]], ["F", 15, [1, 1, 3, 1, 3, 3, 1, 1, 1]], ["G", 16, [1, 1, 1, 1, 1, 3, 3, 1, 3]], ["H", 17, [3, 1, 1, 1, 1, 3, 3, 1, 1]], ["I", 18, [1, 1, 3, 1, 1, 3, 3, 1, 1]], ["J", 19, [1, 1, 1, 1, 3, 3, 3, 1, 1]], ["K", 20, [3, 1, 1, 1, 1, 1, 1, 3, 3]], ["L", 21, [1, 1, 3, 1, 1, 1, 1, 3, 3]], ["M", 22, [3, 1, 3, 1, 1, 1, 1, 3, 1]], ["N", 23, [1, 1, 1, 1, 3, 1, 1, 3, 3]], ["O", 24, [3, 1, 1, 1, 3, 1, 1, 3, 1]], ["P", 25, [1, 1, 3, 1, 3, 1, 1, 3, 1]], ["Q", 26, [1, 1, 1, 1, 1, 1, 3, 3, 3]], ["R", 27, [3, 1, 1, 1, 1, 1, 3, 3, 1]], ["S", 28, [1, 1, 3, 1, 1, 1, 3, 3, 1]], ["T", 29, [1, 1, 1, 1, 3, 1, 3, 3, 1]], ["U", 30, [3, 3, 1, 1, 1, 1, 1, 1, 3]], ["V", 31, [1, 3, 3, 1, 1, 1, 1, 1, 3]], ["W", 32, [3, 3, 3, 1, 1, 1, 1, 1, 1]], ["X", 33, [1, 3, 1, 1, 3, 1, 1, 1, 3]], ["Y", 34, [3, 3, 1, 1, 3, 1, 1, 1, 1]], ["Z", 35, [1, 3, 3, 1, 3, 1, 1, 1, 1]], ["-", 36, [1, 3, 1, 1, 1, 1, 3, 1, 3]], [".", 37, [3, 3, 1, 1, 1, 1, 3, 1, 1]], [" ", 38, [1, 3, 3, 1, 1, 1, 3, 1, 1]], ["$", 39, [1, 3, 1, 3, 1, 3, 1, 1, 1]], ["/", 40, [1, 3, 1, 3, 1, 1, 1, 3, 1]], ["+", 41, [1, 3, 1, 1, 1, 3, 1, 3, 1]], ["%", 42, [1, 1, 1, 3, 1, 3, 1, 3, 1]], ["*", 0, [1, 3, 1, 1, 3, 1, 3, 1, 1]]]
        },
        /** Specifies symbology settings for the Code 39 Extended Barcode. */
        code39extended: {
            startSymbol: '*',
            stopSymbol: '*',
            validatorExp: "^[\x00-\x7F]+$",
            symbolTable: [["0", 0, [1, 1, 1, 3, 3, 1, 3, 1, 1]], ["1", 1, [3, 1, 1, 3, 1, 1, 1, 1, 3]], ["2", 2, [1, 1, 3, 3, 1, 1, 1, 1, 3]], ["3", 3, [3, 1, 3, 3, 1, 1, 1, 1, 1]], ["4", 4, [1, 1, 1, 3, 3, 1, 1, 1, 3]], ["5", 5, [3, 1, 1, 3, 3, 1, 1, 1, 1]], ["6", 6, [1, 1, 3, 3, 3, 1, 1, 1, 1]], ["7", 7, [1, 1, 1, 3, 1, 1, 3, 1, 3]], ["8", 8, [3, 1, 1, 3, 1, 1, 3, 1, 1]], ["9", 9, [1, 1, 3, 3, 1, 1, 3, 1, 1]], ["A", 10, [3, 1, 1, 1, 1, 3, 1, 1, 3]], ["B", 11, [1, 1, 3, 1, 1, 3, 1, 1, 3]], ["C", 12, [3, 1, 3, 1, 1, 3, 1, 1, 1]], ["D", 13, [1, 1, 1, 1, 3, 3, 1, 1, 3]], ["E", 14, [3, 1, 1, 1, 3, 3, 1, 1, 1]], ["F", 15, [1, 1, 3, 1, 3, 3, 1, 1, 1]], ["G", 16, [1, 1, 1, 1, 1, 3, 3, 1, 3]], ["H", 17, [3, 1, 1, 1, 1, 3, 3, 1, 1]], ["I", 18, [1, 1, 3, 1, 1, 3, 3, 1, 1]], ["J", 19, [1, 1, 1, 1, 3, 3, 3, 1, 1]], ["K", 20, [3, 1, 1, 1, 1, 1, 1, 3, 3]], ["L", 21, [1, 1, 3, 1, 1, 1, 1, 3, 3]], ["M", 22, [3, 1, 3, 1, 1, 1, 1, 3, 1]], ["N", 23, [1, 1, 1, 1, 3, 1, 1, 3, 3]], ["O", 24, [3, 1, 1, 1, 3, 1, 1, 3, 1]], ["P", 25, [1, 1, 3, 1, 3, 1, 1, 3, 1]], ["Q", 26, [1, 1, 1, 1, 1, 1, 3, 3, 3]], ["R", 27, [3, 1, 1, 1, 1, 1, 3, 3, 1]], ["S", 28, [1, 1, 3, 1, 1, 1, 3, 3, 1]], ["T", 29, [1, 1, 1, 1, 3, 1, 3, 3, 1]], ["U", 30, [3, 3, 1, 1, 1, 1, 1, 1, 3]], ["V", 31, [1, 3, 3, 1, 1, 1, 1, 1, 3]], ["W", 32, [3, 3, 3, 1, 1, 1, 1, 1, 1]], ["X", 33, [1, 3, 1, 1, 3, 1, 1, 1, 3]], ["Y", 34, [3, 3, 1, 1, 3, 1, 1, 1, 1]], ["Z", 35, [1, 3, 3, 1, 3, 1, 1, 1, 1]], ["-", 36, [1, 3, 1, 1, 1, 1, 3, 1, 3]], [".", 37, [3, 3, 1, 1, 1, 1, 3, 1, 1]], [" ", 38, [1, 3, 3, 1, 1, 1, 3, 1, 1]], ["$", 39, [1, 3, 1, 3, 1, 3, 1, 1, 1]], ["/", 40, [1, 3, 1, 3, 1, 1, 1, 3, 1]], ["+", 41, [1, 3, 1, 1, 1, 3, 1, 3, 1]], ["%", 42, [1, 1, 1, 3, 1, 3, 1, 3, 1]], ["*", 0, [1, 3, 1, 1, 3, 1, 3, 1, 1]]],
            extendedCodes: [['\0', ['%', 'U']], [String.fromCharCode(parseInt('0001', 16)), ['$', 'A']], [String.fromCharCode(parseInt('0002', 16)), ['$', 'B']], [String.fromCharCode(parseInt('0003', 16)), ['$', 'C']], [String.fromCharCode(parseInt('0004', 16)), ['$', 'D']], [String.fromCharCode(parseInt('0005', 16)), ['$', 'E']], [String.fromCharCode(parseInt('0006', 16)), ['$', 'F']], ['\a', ['$', 'G']], ['\b', ['$', 'H']], ['\t', ['$', 'I']], ['\n', ['$', 'J']], ['\v', ['$', 'K']], ['\f', ['$', 'L']], ['\r', ['$', 'M']], [String.fromCharCode(parseInt('000e', 16)), ['$', 'N']], [String.fromCharCode(parseInt('000f', 16)), ['$', 'O']], [String.fromCharCode(parseInt('0010', 16)), ['$', 'P']], [String.fromCharCode(parseInt('0011', 16)), ['$', 'Q']], [String.fromCharCode(parseInt('0012', 16)), ['$', 'R']], [String.fromCharCode(parseInt('0013', 16)), ['$', 'S']], [String.fromCharCode(parseInt('0014', 16)), ['$', 'T']], [String.fromCharCode(parseInt('0015', 16)), ['$', 'U']], [String.fromCharCode(parseInt('0016', 16)), ['$', 'V']], [String.fromCharCode(parseInt('0017', 16)), ['$', 'W']], [String.fromCharCode(parseInt('0018', 16)), ['$', 'X']], [String.fromCharCode(parseInt('0019', 16)), ['$', 'Y']], [String.fromCharCode(parseInt('001a', 16)), ['$', 'Z']], [String.fromCharCode(parseInt('001b', 16)), ['%', 'A']], [String.fromCharCode(parseInt('001c', 16)), ['%', 'B']], [String.fromCharCode(parseInt('001d', 16)), ['%', 'C']], [String.fromCharCode(parseInt('001e', 16)), ['%', 'D']], [String.fromCharCode(parseInt('001f', 16)), ['%', 'E']], [' ', [' ']], ['!', ['/', 'A']], ['"', ['/', 'B']], ['#', ['/', 'C']], ['$', ['/', 'D']], ['%', ['/', 'E']], ['&', ['/', 'F']], ['\'', ['/', 'G']], ['(', ['/', 'H']], [')', ['/', 'I']], ['*', ['/', 'J']], ['+', ['/', 'K']], [',', ['/', 'L']], ['-', ['/', 'M']], ['.', ['/', 'N']], ['/', ['/', 'O']], ['0', ['0']], ['1', ['1']], ['2', ['2']], ['3', ['3']], ['4', ['4']], ['5', ['5']], ['6', ['6']], ['7', ['7']], ['8', ['8']], ['9', ['9']], [':', ['/', 'Z']], [';', ['%', 'F']], ['<', ['%', 'G']], ['=', ['%', 'H']], ['>', ['%', 'I']], ['?', ['%', 'J']], ['@', ['%', 'V']], ['A', ['A']], ['B', ['B']], ['C', ['C']], ['D', ['D']], ['E', ['E']], ['F', ['F']], ['G', ['G']], ['H', ['H']], ['I', ['I']], ['J', ['J']], ['K', ['K']], ['L', ['L']], ['M', ['M']], ['N', ['N']], ['O', ['O']], ['P', ['P']], ['Q', ['Q']], ['R', ['R']], ['S', ['S']], ['T', ['T']], ['U', ['U']], ['V', ['V']], ['W', ['W']], ['X', ['X']], ['Y', ['Y']], ['Z', ['Z']], ['[', ['%', 'K']], ['\\', ['%', 'L']], [']', ['%', 'M']], ['^', ['%', 'N']], ['_', ['%', 'O']], ['`', ['%', 'W']], ['a', ['+', 'A']], ['b', ['+', 'B']], ['c', ['+', 'C']], ['d', ['+', 'D']], ['e', ['+', 'E']], ['f', ['+', 'F']], ['g', ['+', 'G']], ['h', ['+', 'H']], ['i', ['+', 'I']], ['j', ['+', 'J']], ['k', ['+', 'K']], ['l', ['+', 'L']], ['m', ['+', 'M']], ['n', ['+', 'N']], ['o', ['+', 'O']], ['p', ['+', 'P']], ['q', ['+', 'Q']], ['r', ['+', 'R']], ['s', ['+', 'S']], ['t', ['+', 'T']], ['u', ['+', 'U']], ['v', ['+', 'V']], ['w', ['+', 'W']], ['x', ['+', 'X']], ['y', ['+', 'Y']], ['z', ['+', 'Z']], ['{', ['%', 'P']], ['|', ['%', 'Q']], ['}', ['%', 'R']], ['~', ['%', 'S']], [String.fromCharCode(parseInt('007f', 16)), ['%', 'T']]]
        },
        /** Specifies symbology settings for the Code 11 Barcode. */
        code11: {
            startSymbol: '*',
            stopSymbol: '*',
            validatorExp: "^[0-9\-]*$",
            symbolTable: [["0", 0, [1, 1, 1, 1, 2]], ["1", 1, [2, 1, 1, 1, 2]], ["2", 2, [1, 2, 1, 1, 2]], ["3", 3, [2, 2, 1, 1, 1]], ["4", 4, [1, 1, 2, 1, 2]], ["5", 5, [2, 1, 2, 1, 1]], ["6", 6, [1, 2, 2, 1, 1]], ["7", 7, [1, 1, 1, 2, 2]], ["8", 8, [2, 1, 1, 2, 1]], ["9", 9, [2, 1, 1, 1, 1]], ["-", 10, [1, 1, 2, 1, 1]], ["*", 0, [1, 1, 2, 2, 1]]]
        },
        /** Specifies symbology settings for the Codabar Barcode. */
        codabar: {
            startSymbol: 'A',
            stopSymbol: 'B',
            validatorExp: "^[0-9\-\.\$\/\:\+]*$",
            symbolTable: [["0", 0, [1, 1, 1, 1, 1, 2, 2]], ["1", 0, [1, 1, 1, 1, 2, 2, 1]], ["2", 0, [1, 1, 1, 2, 1, 1, 2]], ["3", 0, [2, 2, 1, 1, 1, 1, 1]], ["4", 0, [1, 1, 2, 1, 1, 2, 1]], ["5", 0, [2, 1, 1, 1, 1, 2, 1]], ["6", 0, [1, 2, 1, 1, 1, 1, 2]], ["7", 0, [1, 2, 1, 1, 2, 1, 1]], ["8", 0, [1, 2, 2, 1, 1, 1, 1]], ["9", 0, [2, 1, 1, 2, 1, 1, 1]], ["-", 0, [1, 1, 1, 2, 2, 1, 1]], ["$", 0, [1, 1, 2, 2, 1, 1, 1]], [":", 0, [2, 1, 1, 1, 2, 1, 2]], ["/", 0, [2, 1, 2, 1, 1, 1, 2]], [".", 0, [2, 1, 2, 1, 2, 1, 1]], ["+", 0, [1, 1, 2, 1, 2, 1, 2]], ["A", 0, [1, 1, 2, 2, 1, 2, 1]], ["B", 0, [1, 1, 1, 2, 1, 2, 2]]]
        },
        /** Specifies symbology settings for the Code 32 Barcode. */
        code32: {
            startSymbol: '*',
            stopSymbol: '*',
            validatorExp: "^[\x41-\x5A\x30-\x39\x20\-\*\.\$\/\+\%]+$",
            symbolTable: [["0", 0, [1, 1, 1, 3, 3, 1, 3, 1, 1]], ["1", 1, [3, 1, 1, 3, 1, 1, 1, 1, 3]], ["2", 2, [1, 1, 3, 3, 1, 1, 1, 1, 3]], ["3", 3, [3, 1, 3, 3, 1, 1, 1, 1, 1]], ["4", 4, [1, 1, 1, 3, 3, 1, 1, 1, 3]], ["5", 5, [3, 1, 1, 3, 3, 1, 1, 1, 1]], ["6", 6, [1, 1, 3, 3, 3, 1, 1, 1, 1]], ["7", 7, [1, 1, 1, 3, 1, 1, 3, 1, 3]], ["8", 8, [3, 1, 1, 3, 1, 1, 3, 1, 1]], ["9", 9, [1, 1, 3, 3, 1, 1, 3, 1, 1]], ["B", 10, [1, 1, 3, 1, 1, 3, 1, 1, 3]], ["C", 11, [3, 1, 3, 1, 1, 3, 1, 1, 1]], ["D", 12, [1, 1, 1, 1, 3, 3, 1, 1, 3]], ["F", 13, [1, 1, 3, 1, 3, 3, 1, 1, 1]], ["G", 14, [1, 1, 1, 1, 1, 3, 3, 1, 3]], ["H", 15, [3, 1, 1, 1, 1, 3, 3, 1, 1]], ["J", 0x10, [1, 1, 1, 1, 3, 3, 3, 1, 1]], ["K", 0x11, [3, 1, 1, 1, 1, 1, 1, 3, 3]], ["L", 0x12, [1, 1, 3, 1, 1, 1, 1, 3, 3]], ["M", 0x13, [3, 1, 3, 1, 1, 1, 1, 3, 1]], ["N", 20, [1, 1, 1, 1, 3, 1, 1, 3, 3]], ["P", 0x15, [1, 1, 3, 1, 3, 1, 1, 3, 1]], ["Q", 0x16, [1, 1, 1, 1, 1, 1, 3, 3, 3]], ["R", 0x17, [3, 1, 1, 1, 1, 1, 3, 3, 1]], ["S", 0x18, [1, 1, 3, 1, 1, 1, 3, 3, 1]], ["T", 0x19, [1, 1, 1, 1, 3, 1, 3, 3, 1]], ["U", 0x1a, [3, 3, 1, 1, 1, 1, 1, 1, 3]], ["V", 0x1b, [1, 3, 3, 1, 1, 1, 1, 1, 3]], ["W", 0x1c, [3, 3, 3, 1, 1, 1, 1, 1, 1]], ["X", 0x1d, [1, 3, 1, 1, 3, 1, 1, 1, 3]], ["Y", 30, [3, 3, 1, 1, 3, 1, 1, 1, 1]], ["Z", 0x1f, [1, 3, 3, 1, 3, 1, 1, 1, 1]], ["*", 0, [1, 3, 1, 1, 3, 1, 3, 1, 1]]]
        },
        /** Specifies symbology settings for the Code 93 Barcode. */
        code93: {
            startSymbol: '*',
            stopSymbol: String.fromCharCode(parseInt('00ff', 16)),
            validatorExp: "^[\x41-\x5A\x30-\x39\x20\-\.\$\/\+\%\ ]+$",
            symbolTable: [["0", 0, [1, 3, 1, 1, 1, 2]], ["1", 1, [1, 1, 1, 2, 1, 3]], ["2", 2, [1, 1, 1, 3, 1, 2]], ["3", 3, [1, 1, 1, 4, 1, 1]], ["4", 4, [1, 2, 1, 1, 1, 2]], ["5", 5, [1, 2, 1, 2, 1, 2]], ["6", 6, [1, 2, 1, 3, 1, 1]], ["7", 7, [1, 1, 1, 1, 1, 4]], ["8", 8, [1, 3, 1, 2, 1, 1]], ["9", 9, [1, 4, 1, 1, 1, 1]], ["A", 10, [2, 1, 1, 1, 1, 3]], ["B", 11, [2, 1, 1, 2, 1, 2]], ["C", 12, [2, 1, 1, 3, 1, 1]], ["D", 13, [2, 2, 1, 1, 1, 2]], ["E", 14, [2, 2, 1, 2, 1, 1]], ["F", 15, [2, 3, 1, 1, 1, 1]], ["G", 0x10, [1, 1, 2, 1, 1, 3]], ["H", 0x11, [1, 1, 2, 2, 1, 2]], ["I", 0x12, [1, 1, 2, 3, 1, 1]], ["J", 0x13, [1, 2, 2, 1, 1, 2]], ["K", 20, [1, 3, 2, 1, 1, 1]], ["L", 0x15, [1, 1, 1, 1, 2, 3]], ["M", 0x16, [1, 1, 1, 2, 2, 2]], ["N", 0x17, [1, 1, 1, 3, 2, 1]], ["O", 0x18, [1, 2, 1, 1, 2, 2]], ["P", 0x19, [1, 3, 1, 1, 2, 1]], ["Q", 0x1a, [2, 1, 2, 1, 1, 2]], ["R", 0x1b, [2, 1, 2, 2, 1, 1]], ["S", 0x1c, [2, 1, 1, 1, 2, 2]], ["T", 0x1d, [2, 1, 1, 2, 2, 1]], ["U", 30, [2, 2, 1, 1, 2, 1]], ["V", 0x1f, [2, 2, 2, 1, 1, 1]], ["W", 0x20, [1, 1, 2, 1, 2, 2]], ["X", 0x21, [1, 1, 2, 2, 2, 1]], ["Y", 0x22, [1, 2, 2, 1, 2, 1]], ["Z", 0x23, [1, 2, 3, 1, 1, 1]], ["-", 0x24, [1, 2, 1, 1, 3, 1]], [".", 0x25, [3, 1, 1, 1, 1, 2]], [" ", 0x26, [3, 1, 1, 2, 1, 1]], ["$", 0x27, [3, 2, 1, 1, 1, 1]], ["/", 40, [1, 1, 2, 1, 3, 1]], ["+", 0x29, [1, 1, 3, 1, 2, 1]], ["%", 0x2a, [2, 1, 1, 1, 3, 1]], ["*", 0, [1, 1, 1, 1, 4, 1]], [String.fromCharCode(parseInt('00ff', 16)), 0, [1, 1, 1, 1, 4, 1, 1]], [String.fromCharCode(parseInt('00fb', 16)), 0x2b, [1, 2, 1, 2, 2, 0]], [String.fromCharCode(parseInt('00fc', 16)), 0x2c, [3, 1, 2, 1, 1, 1]], [String.fromCharCode(parseInt('00fd', 16)), 0x2d, [3, 1, 1, 1, 2, 1]], [String.fromCharCode(parseInt('00fe', 16)), 0x2e, [1, 2, 2, 2, 1, 1]]]
        },
        /** Specifies symbology settings for the Code 93 Extended Barcode. */
        code93extended: {
            startSymbol: '*',
            stopSymbol: String.fromCharCode(parseInt('00ff', 16)),
            validatorExp: "^[\x00-\x7F\x00fb\x00fd\x00fe\'�'\'�'\'�']+$",
            symbolTable: [["0", 0, [1, 3, 1, 1, 1, 2]], ["1", 1, [1, 1, 1, 2, 1, 3]], ["2", 2, [1, 1, 1, 3, 1, 2]], ["3", 3, [1, 1, 1, 4, 1, 1]], ["4", 4, [1, 2, 1, 1, 1, 2]], ["5", 5, [1, 2, 1, 2, 1, 2]], ["6", 6, [1, 2, 1, 3, 1, 1]], ["7", 7, [1, 1, 1, 1, 1, 4]], ["8", 8, [1, 3, 1, 2, 1, 1]], ["9", 9, [1, 4, 1, 1, 1, 1]], ["A", 10, [2, 1, 1, 1, 1, 3]], ["B", 11, [2, 1, 1, 2, 1, 2]], ["C", 12, [2, 1, 1, 3, 1, 1]], ["D", 13, [2, 2, 1, 1, 1, 2]], ["E", 14, [2, 2, 1, 2, 1, 1]], ["F", 15, [2, 3, 1, 1, 1, 1]], ["G", 0x10, [1, 1, 2, 1, 1, 3]], ["H", 0x11, [1, 1, 2, 2, 1, 2]], ["I", 0x12, [1, 1, 2, 3, 1, 1]], ["J", 0x13, [1, 2, 2, 1, 1, 2]], ["K", 20, [1, 3, 2, 1, 1, 1]], ["L", 0x15, [1, 1, 1, 1, 2, 3]], ["M", 0x16, [1, 1, 1, 2, 2, 2]], ["N", 0x17, [1, 1, 1, 3, 2, 1]], ["O", 0x18, [1, 2, 1, 1, 2, 2]], ["P", 0x19, [1, 3, 1, 1, 2, 1]], ["Q", 0x1a, [2, 1, 2, 1, 1, 2]], ["R", 0x1b, [2, 1, 2, 2, 1, 1]], ["S", 0x1c, [2, 1, 1, 1, 2, 2]], ["T", 0x1d, [2, 1, 1, 2, 2, 1]], ["U", 30, [2, 2, 1, 1, 2, 1]], ["V", 0x1f, [2, 2, 2, 1, 1, 1]], ["W", 0x20, [1, 1, 2, 1, 2, 2]], ["X", 0x21, [1, 1, 2, 2, 2, 1]], ["Y", 0x22, [1, 2, 2, 1, 2, 1]], ["Z", 0x23, [1, 2, 3, 1, 1, 1]], ["-", 0x24, [1, 2, 1, 1, 3, 1]], [".", 0x25, [3, 1, 1, 1, 1, 2]], [" ", 0x26, [3, 1, 1, 2, 1, 1]], ["$", 0x27, [3, 2, 1, 1, 1, 1]], ["/", 40, [1, 1, 2, 1, 3, 1]], ["+", 0x29, [1, 1, 3, 1, 2, 1]], ["%", 0x2a, [2, 1, 1, 1, 3, 1]], ["*", 0, [1, 1, 1, 1, 4, 1]], [String.fromCharCode(parseInt('00ff', 16)), 0, [1, 1, 1, 1, 4, 1, 1]], [String.fromCharCode(parseInt('00fb', 16)), 0x2b, [1, 2, 1, 2, 2, 0]], [String.fromCharCode(parseInt('00fc', 16)), 0x2c, [3, 1, 2, 1, 1, 1]], [String.fromCharCode(parseInt('00fd', 16)), 0x2d, [3, 1, 1, 1, 2, 1]], [String.fromCharCode(parseInt('00fe', 16)), 0x2e, [1, 2, 2, 2, 1, 1]]],
            extendedCodes: [['\0', [String.fromCharCode(parseInt('00fc', 16)), 'U']], [String.fromCharCode(parseInt('0001', 16)), [String.fromCharCode(parseInt('00fb', 16)), 'A']], [String.fromCharCode(parseInt('0002', 16)), [String.fromCharCode(parseInt('00fb', 16)), 'B']], [String.fromCharCode(parseInt('0003', 16)), [String.fromCharCode(parseInt('00fb', 16)), 'C']], [String.fromCharCode(parseInt('0004', 16)), [String.fromCharCode(parseInt('00fb', 16)), 'D']], [String.fromCharCode(parseInt('0005', 16)), [String.fromCharCode(parseInt('00fb', 16)), 'E']], [String.fromCharCode(parseInt('0006', 16)), [String.fromCharCode(parseInt('00fb', 16)), 'F']], ['\a', [String.fromCharCode(parseInt('00fb', 16)), 'G']], ['\b', [String.fromCharCode(parseInt('00fb', 16)), 'H']], ['\t', [String.fromCharCode(parseInt('00fb', 16)), 'I']], ['\n', [String.fromCharCode(parseInt('00fb', 16)), 'J']], ['\v', [String.fromCharCode(parseInt('00fb', 16)), 'K']], ['\f', [String.fromCharCode(parseInt('00fb', 16)), 'L']], ['\r', [String.fromCharCode(parseInt('00fb', 16)), 'M']], [String.fromCharCode(parseInt('000e', 16)), [String.fromCharCode(parseInt('00fb', 16)), 'N']], [String.fromCharCode(parseInt('000f', 16)), [String.fromCharCode(parseInt('00fb', 16)), 'O']], [String.fromCharCode(parseInt('0010', 16)), [String.fromCharCode(parseInt('00fb', 16)), 'P']], [String.fromCharCode(parseInt('0011', 16)), [String.fromCharCode(parseInt('00fb', 16)), 'Q']], [String.fromCharCode(parseInt('0012', 16)), [String.fromCharCode(parseInt('00fb', 16)), 'R']], [String.fromCharCode(parseInt('0013', 16)), [String.fromCharCode(parseInt('00fb', 16)), 'S']], [String.fromCharCode(parseInt('0014', 16)), [String.fromCharCode(parseInt('00fb', 16)), 'T']], [String.fromCharCode(parseInt('0015', 16)), [String.fromCharCode(parseInt('00fb', 16)), 'U']], [String.fromCharCode(parseInt('0016', 16)), [String.fromCharCode(parseInt('00fb', 16)), 'V']], [String.fromCharCode(parseInt('0017', 16)), [String.fromCharCode(parseInt('00fb', 16)), 'W']], [String.fromCharCode(parseInt('0018', 16)), [String.fromCharCode(parseInt('00fb', 16)), 'X']], [String.fromCharCode(parseInt('0019', 16)), [String.fromCharCode(parseInt('00fb', 16)), 'Y']], [String.fromCharCode(parseInt('001a', 16)), [String.fromCharCode(parseInt('00fb', 16)), 'Z']], [String.fromCharCode(parseInt('001b', 16)), [String.fromCharCode(parseInt('00fc', 16)), 'A']], [String.fromCharCode(parseInt('001c', 16)), [String.fromCharCode(parseInt('00fc', 16)), 'B']], [String.fromCharCode(parseInt('001d', 16)), [String.fromCharCode(parseInt('00fc', 16)), 'C']], [String.fromCharCode(parseInt('001e', 16)), [String.fromCharCode(parseInt('00fc', 16)), 'D']], [String.fromCharCode(parseInt('001f', 16)), [String.fromCharCode(parseInt('00fc', 16)), 'E']], [' ', [' ']], ['!', [String.fromCharCode(parseInt('00fd', 16)), 'A']], ['"', [String.fromCharCode(parseInt('00fd', 16)), 'B']], ['#', [String.fromCharCode(parseInt('00fd', 16)), 'C']], ['$', [String.fromCharCode(parseInt('00fd', 16)), 'D']], ['%', [String.fromCharCode(parseInt('00fd', 16)), 'E']], ['&', [String.fromCharCode(parseInt('00fd', 16)), 'F']], ['\'', [String.fromCharCode(parseInt('00fd', 16)), 'G']], ['(', [String.fromCharCode(parseInt('00fd', 16)), 'H']], [')', [String.fromCharCode(parseInt('00fd', 16)), 'I']], ['*', [String.fromCharCode(parseInt('00fd', 16)), 'J']], ['+', [String.fromCharCode(parseInt('00fd', 16)), 'K']], [',', [String.fromCharCode(parseInt('00fd', 16)), 'L']], ['-', [String.fromCharCode(parseInt('00fd', 16)), 'M']], ['.', [String.fromCharCode(parseInt('00fd', 16)), 'N']], ['/', [String.fromCharCode(parseInt('00fd', 16)), 'O']], ['0', ['0']], ['1', ['1']], ['2', ['2']], ['3', ['3']], ['4', ['4']], ['5', ['5']], ['6', ['6']], ['7', ['7']], ['8', ['8']], ['9', ['9']], [':', [String.fromCharCode(parseInt('00fd', 16)), 'Z']], [';', [String.fromCharCode(parseInt('00fc', 16)), 'F']], ['<', [String.fromCharCode(parseInt('00fc', 16)), 'G']], ['=', [String.fromCharCode(parseInt('00fc', 16)), 'H']], ['>', [String.fromCharCode(parseInt('00fc', 16)), 'I']], ['?', [String.fromCharCode(parseInt('00fc', 16)), 'J']], ['@', [String.fromCharCode(parseInt('00fc', 16)), 'V']], ['A', ['A']], ['B', ['B']], ['C', ['C']], ['D', ['D']], ['E', ['E']], ['F', ['F']], ['G', ['G']], ['H', ['H']], ['I', ['I']], ['J', ['J']], ['K', ['K']], ['L', ['L']], ['M', ['M']], ['N', ['N']], ['O', ['O']], ['P', ['P']], ['Q', ['Q']], ['R', ['R']], ['S', ['S']], ['T', ['T']], ['U', ['U']], ['V', ['V']], ['W', ['W']], ['X', ['X']], ['Y', ['Y']], ['Z', ['Z']], ['[', [String.fromCharCode(parseInt('00fc', 16)), 'K']], ['\\', [String.fromCharCode(parseInt('00fc', 16)), 'L']], [']', [String.fromCharCode(parseInt('00fc', 16)), 'M']], ['^', [String.fromCharCode(parseInt('00fc', 16)), 'N']], ['_', [String.fromCharCode(parseInt('00fc', 16)), 'O']], ['`', [String.fromCharCode(parseInt('00fc', 16)), 'W']], ['a', [String.fromCharCode(parseInt('00fe', 16)), 'A']], ['b', [String.fromCharCode(parseInt('00fe', 16)), 'B']], ['c', [String.fromCharCode(parseInt('00fe', 16)), 'C']], ['d', [String.fromCharCode(parseInt('00fe', 16)), 'D']], ['e', [String.fromCharCode(parseInt('00fe', 16)), 'E']], ['f', [String.fromCharCode(parseInt('00fe', 16)), 'F']], ['g', [String.fromCharCode(parseInt('00fe', 16)), 'G']], ['h', [String.fromCharCode(parseInt('00fe', 16)), 'H']], ['i', [String.fromCharCode(parseInt('00fe', 16)), 'I']], ['j', [String.fromCharCode(parseInt('00fe', 16)), 'J']], ['k', [String.fromCharCode(parseInt('00fe', 16)), 'K']], ['l', [String.fromCharCode(parseInt('00fe', 16)), 'L']], ['m', [String.fromCharCode(parseInt('00fe', 16)), 'M']], ['n', [String.fromCharCode(parseInt('00fe', 16)), 'N']], ['o', [String.fromCharCode(parseInt('00fe', 16)), 'O']], ['p', [String.fromCharCode(parseInt('00fe', 16)), 'P']], ['q', [String.fromCharCode(parseInt('00fe', 16)), 'Q']], ['r', [String.fromCharCode(parseInt('00fe', 16)), 'R']], ['s', [String.fromCharCode(parseInt('00fe', 16)), 'S']], ['t', [String.fromCharCode(parseInt('00fe', 16)), 'T']], ['u', [String.fromCharCode(parseInt('00fe', 16)), 'U']], ['v', [String.fromCharCode(parseInt('00fe', 16)), 'V']], ['w', [String.fromCharCode(parseInt('00fe', 16)), 'W']], ['x', [String.fromCharCode(parseInt('00fe', 16)), 'X']], ['y', [String.fromCharCode(parseInt('00fe', 16)), 'Y']], ['z', [String.fromCharCode(parseInt('00fe', 16)), 'Z']], ['{', [String.fromCharCode(parseInt('00fc', 16)), 'P']], ['|', [String.fromCharCode(parseInt('00fc', 16)), 'Q']], ['}', [String.fromCharCode(parseInt('00fc', 16)), 'R']], ['~', [String.fromCharCode(parseInt('00fc', 16)), 'S']], [String.fromCharCode(parseInt('007f', 16)), [String.fromCharCode(parseInt('00fc', 16)), 'T']]],
        },
        /** Specifies symbology settings for the Code 128 A Barcode. */
        code128a: {
            startSymbol: String.fromCharCode(parseInt('00f9', 16)),
            stopSymbol: String.fromCharCode(parseInt('00ff', 16)),
            validatorExp: "[\0\x0001\x0002\x0003\x0004\x0005\x0006\a\b\t\n\v\f\r\x000e\x000f\x0010\x0011\x0012\x0013\x0014\x0015\x0016\x0017\x0018\x0019\x001a\x001b\x001c\x001d\x001e\x001f !\"#$%&'()*+,-./0123456789:;<=>?@ABCDEFGHIJKLMNOPQRSTUVWXYZ[\\]^_\x00f0\x00f1\x00f2\x00f3\x00f4]",
            symbolTable: [["\0", 0x40, [1, 1, 1, 4, 2, 2]], [String.fromCharCode(parseInt('0001', 16)), 0x41, [1, 2, 1, 1, 2, 4]], [String.fromCharCode(parseInt('0002', 16)), 0x42, [1, 2, 1, 4, 2, 1]], [String.fromCharCode(parseInt('0003', 16)), 0x43, [1, 4, 1, 1, 2, 2]], [String.fromCharCode(parseInt('0004', 16)), 0x44, [1, 4, 1, 2, 2, 1]], [String.fromCharCode(parseInt('0005', 16)), 0x45, [1, 1, 2, 2, 1, 4]], [String.fromCharCode(parseInt('0006', 16)), 70, [1, 1, 2, 4, 1, 2]], ["\a", 0x47, [1, 2, 2, 1, 1, 4]], ["\b", 0x48, [1, 2, 2, 4, 1, 1]], ["\t", 0x49, [1, 4, 2, 1, 1, 2]], ["\n", 0x4a, [1, 4, 2, 2, 1, 1]], ["\v", 0x4b, [2, 4, 1, 2, 1, 1]], ["\f", 0x4c, [2, 2, 1, 1, 1, 4]], ["\r", 0x4d, [4, 1, 3, 1, 1, 1]], [String.fromCharCode(parseInt('000e', 16)), 0x4e, [2, 4, 1, 1, 1, 2]], [String.fromCharCode(parseInt('000f', 16)), 0x4f, [1, 3, 4, 1, 1, 1]], [String.fromCharCode(parseInt('0010', 16)), 80, [1, 1, 1, 2, 4, 2]], [String.fromCharCode(parseInt('0011', 16)), 0x51, [1, 2, 1, 1, 4, 2]], [String.fromCharCode(parseInt('0012', 16)), 0x52, [1, 2, 1, 2, 4, 1]], [String.fromCharCode(parseInt('0013', 16)), 0x53, [1, 1, 4, 2, 1, 2]], [String.fromCharCode(parseInt('0014', 16)), 0x54, [1, 2, 4, 1, 1, 2]], [String.fromCharCode(parseInt('0015', 16)), 0x55, [1, 2, 4, 2, 1, 1]], [String.fromCharCode(parseInt('0016', 16)), 0x56, [4, 1, 1, 2, 1, 2]], [String.fromCharCode(parseInt('0017', 16)), 0x57, [4, 2, 1, 1, 1, 2]], [String.fromCharCode(parseInt('0018', 16)), 0x58, [4, 2, 1, 2, 1, 1]], [String.fromCharCode(parseInt('0019', 16)), 0x59, [2, 1, 2, 1, 4, 1]], [String.fromCharCode(parseInt('001a', 16)), 90, [2, 1, 4, 1, 2, 1]], [String.fromCharCode(parseInt('001b', 16)), 0x5b, [4, 1, 2, 1, 2, 1]], [String.fromCharCode(parseInt('001c', 16)), 0x5c, [1, 1, 1, 1, 4, 3]], [String.fromCharCode(parseInt('001d', 16)), 0x5d, [1, 1, 1, 3, 4, 1]], [String.fromCharCode(parseInt('001e', 16)), 0x5e, [1, 3, 1, 1, 4, 1]], [String.fromCharCode(parseInt('001f', 16)), 0x5f, [1, 1, 4, 1, 1, 3]], [" ", 0, [2, 1, 2, 2, 2, 2]], ["!", 1, [2, 2, 2, 1, 2, 2]], ["\"", 2, [2, 2, 2, 2, 2, 1]], ["#", 3, [1, 2, 1, 2, 2, 3]], ["$", 4, [1, 2, 1, 3, 2, 2]], ["%", 5, [1, 3, 1, 2, 2, 2]], ["&", 6, [1, 2, 2, 2, 1, 3]], ["\'", 7, [1, 2, 2, 3, 1, 2]], ["(", 8, [1, 3, 2, 2, 1, 2]], [")", 9, [2, 2, 1, 2, 1, 3]], ["*", 10, [2, 2, 1, 3, 1, 2]], ["+", 11, [2, 3, 1, 2, 1, 2]], [",", 12, [1, 1, 2, 2, 3, 2]], ["-", 13, [1, 2, 2, 1, 3, 2]], [".", 14, [1, 2, 2, 2, 3, 1]], ["/", 15, [1, 1, 3, 2, 2, 2]], ["0", 0x10, [1, 2, 3, 1, 2, 2]], ["1", 0x11, [1, 2, 3, 2, 2, 1]], ["2", 0x12, [2, 2, 3, 2, 1, 1]], ["3", 0x13, [2, 2, 1, 1, 3, 2]], ["4", 20, [2, 2, 1, 2, 3, 1]], ["5", 0x15, [2, 1, 3, 2, 1, 2]], ["6", 0x16, [2, 2, 3, 1, 1, 2]], ["7", 0x17, [3, 1, 2, 1, 3, 1]], ["8", 0x18, [3, 1, 1, 2, 2, 2]], ["9", 0x19, [3, 2, 1, 1, 2, 2]], [":", 0x1a, [3, 2, 1, 2, 2, 1]], [";", 0x1b, [3, 1, 2, 2, 1, 2]], ["<", 0x1c, [3, 2, 2, 1, 1, 2]], ["=", 0x1d, [3, 2, 2, 2, 1, 1]], [">", 30, [2, 1, 2, 1, 2, 3]], ["?", 0x1f, [2, 1, 2, 3, 2, 1]], ["@", 0x20, [2, 3, 2, 1, 2, 1]], ["A", 0x21, [1, 1, 1, 3, 2, 3]], ["B", 0x22, [1, 3, 1, 1, 2, 3]], ["C", 0x23, [1, 3, 1, 3, 2, 1]], ["D", 0x24, [1, 1, 2, 3, 1, 3]], ["E", 0x25, [1, 3, 2, 1, 1, 3]], ["F", 0x26, [1, 3, 2, 3, 1, 1]], ["G", 0x27, [2, 1, 1, 3, 1, 3]], ["H", 40, [2, 3, 1, 1, 1, 3]], ["I", 0x29, [2, 3, 1, 3, 1, 1]], ["J", 0x2a, [1, 1, 2, 1, 3, 3]], ["K", 0x2b, [1, 1, 2, 3, 3, 1]], ["L", 0x2c, [1, 3, 2, 1, 3, 1]], ["M", 0x2d, [1, 1, 3, 1, 2, 3]], ["N", 0x2e, [1, 1, 3, 3, 2, 1]], ["O", 0x2f, [1, 3, 3, 1, 2, 1]], ["P", 0x30, [3, 1, 3, 1, 2, 1]], ["Q", 0x31, [2, 1, 1, 3, 3, 1]], ["R", 50, [2, 3, 1, 1, 3, 1]], ["S", 0x33, [2, 1, 3, 1, 1, 3]], ["T", 0x34, [2, 1, 3, 3, 1, 1]], ["U", 0x35, [2, 1, 3, 1, 3, 1]], ["V", 0x36, [3, 1, 1, 1, 2, 3]], ["W", 0x37, [3, 1, 1, 3, 2, 1]], ["X", 0x38, [3, 3, 1, 1, 2, 1]], ["Y", 0x39, [3, 1, 2, 1, 1, 3]], ["Z", 0x3a, [3, 1, 2, 3, 1, 1]], ["[", 0x3b, [3, 3, 2, 1, 1, 1]], ["\\", 60, [3, 1, 4, 1, 1, 1]], ["]", 0x3d, [2, 2, 1, 4, 1, 1]], ["^", 0x3e, [4, 3, 1, 1, 1, 1]], ["_", 0x3f, [1, 1, 1, 2, 2, 4]], [String.fromCharCode(parseInt('00f0', 16)), 0x66, [4, 1, 1, 1, 3, 1]], [String.fromCharCode(parseInt('00f1', 16)), 0x61, [4, 1, 1, 1, 1, 3]], [String.fromCharCode(parseInt('00f2', 16)), 0x60, [1, 1, 4, 3, 1, 1]], [String.fromCharCode(parseInt('00f3', 16)), 0x65, [3, 1, 1, 1, 4, 1]], [String.fromCharCode(parseInt('00f4', 16)), 0x62, [4, 1, 1, 3, 1, 1]], [String.fromCharCode(parseInt('00fc', 16)), 0x63, [1, 1, 3, 1, 4, 1]], [String.fromCharCode(parseInt('00fb', 16)), 100, [1, 1, 4, 1, 3, 1]], [String.fromCharCode(parseInt('00f9', 16)), 0x67, [2, 1, 1, 4, 1, 2]], [String.fromCharCode(parseInt('00ff', 16)), -1, [2, 3, 3, 1, 1, 1, 2]]],
        },
        /** Specifies symbology settings for the Code 128 B Barcode. */
        code128b: {
            startSymbol: String.fromCharCode(parseInt('00fd', 16)),
            stopSymbol: String.fromCharCode(parseInt('00ff', 16)),
            validatorExp: "^[\x00-\x7F]",
            symbolTable: [[" ", 0, [2, 1, 2, 2, 2, 2]], ["!", 1, [2, 2, 2, 1, 2, 2]], ["\"", 2, [2, 2, 2, 2, 2, 1]], ["#", 3, [1, 2, 1, 2, 2, 3]], ["$", 4, [1, 2, 1, 3, 2, 2]], ["%", 5, [1, 3, 1, 2, 2, 2]], ["&", 6, [1, 2, 2, 2, 1, 3]], ["\'", 7, [1, 2, 2, 3, 1, 2]], ["(", 8, [1, 3, 2, 2, 1, 2]], [")", 9, [2, 2, 1, 2, 1, 3]], ["*", 10, [2, 2, 1, 3, 1, 2]], ["+", 11, [2, 3, 1, 2, 1, 2]], [",", 12, [1, 1, 2, 2, 3, 2]], ["-", 13, [1, 2, 2, 1, 3, 2]], [".", 14, [1, 2, 2, 2, 3, 1]], ["/", 15, [1, 1, 3, 2, 2, 2]], ["0", 0x10, [1, 2, 3, 1, 2, 2]], ["1", 0x11, [1, 2, 3, 2, 2, 1]], ["2", 0x12, [2, 2, 3, 2, 1, 1]], ["3", 0x13, [2, 2, 1, 1, 3, 2]], ["4", 20, [2, 2, 1, 2, 3, 1]], ["5", 0x15, [2, 1, 3, 2, 1, 2]], ["6", 0x16, [2, 2, 3, 1, 1, 2]], ["7", 0x17, [3, 1, 2, 1, 3, 1]], ["8", 0x18, [3, 1, 1, 2, 2, 2]], ["9", 0x19, [3, 2, 1, 1, 2, 2]], [":", 0x1a, [3, 2, 1, 2, 2, 1]], [";", 0x1b, [3, 1, 2, 2, 1, 2]], ["<", 0x1c, [3, 2, 2, 1, 1, 2]], ["=", 0x1d, [3, 2, 2, 2, 1, 1]], [">", 30, [2, 1, 2, 1, 2, 3]], ["?", 0x1f, [2, 1, 2, 3, 2, 1]], ["@", 0x20, [2, 3, 2, 1, 2, 1]], ["A", 0x21, [1, 1, 1, 3, 2, 3]], ["B", 0x22, [1, 3, 1, 1, 2, 3]], ["C", 0x23, [1, 3, 1, 3, 2, 1]], ["D", 0x24, [1, 1, 2, 3, 1, 3]], ["E", 0x25, [1, 3, 2, 1, 1, 3]], ["F", 0x26, [1, 3, 2, 3, 1, 1]], ["G", 0x27, [2, 1, 1, 3, 1, 3]], ["H", 40, [2, 3, 1, 1, 1, 3]], ["I", 0x29, [2, 3, 1, 3, 1, 1]], ["J", 0x2a, [1, 1, 2, 1, 3, 3]], ["K", 0x2b, [1, 1, 2, 3, 3, 1]], ["L", 0x2c, [1, 3, 2, 1, 3, 1]], ["M", 0x2d, [1, 1, 3, 1, 2, 3]], ["N", 0x2e, [1, 1, 3, 3, 2, 1]], ["O", 0x2f, [1, 3, 3, 1, 2, 1]], ["P", 0x30, [3, 1, 3, 1, 2, 1]], ["Q", 0x31, [2, 1, 1, 3, 3, 1]], ["R", 50, [2, 3, 1, 1, 3, 1]], ["S", 0x33, [2, 1, 3, 1, 1, 3]], ["T", 0x34, [2, 1, 3, 3, 1, 1]], ["U", 0x35, [2, 1, 3, 1, 3, 1]], ["V", 0x36, [3, 1, 1, 1, 2, 3]], ["W", 0x37, [3, 1, 1, 3, 2, 1]], ["X", 0x38, [3, 3, 1, 1, 2, 1]], ["Y", 0x39, [3, 1, 2, 1, 1, 3]], ["Z", 0x3a, [3, 1, 2, 3, 1, 1]], ["[", 0x3b, [3, 3, 2, 1, 1, 1]], ["\\", 60, [3, 1, 4, 1, 1, 1]], ["]", 0x3d, [2, 2, 1, 4, 1, 1]], ["^", 0x3e, [4, 3, 1, 1, 1, 1]], ["_", 0x3f, [1, 1, 1, 2, 2, 4]], ["`", 0x40, [1, 1, 1, 4, 2, 2]], ["a", 0x41, [1, 2, 1, 1, 2, 4]], ["b", 0x42, [1, 2, 1, 4, 2, 1]], ["c", 0x43, [1, 4, 1, 1, 2, 2]], ["d", 0x44, [1, 4, 1, 2, 2, 1]], ["e", 0x45, [1, 1, 2, 2, 1, 4]], ["f", 70, [1, 1, 2, 4, 1, 2]], ["g", 0x47, [1, 2, 2, 1, 1, 4]], ["h", 0x48, [1, 2, 2, 4, 1, 1]], ["i", 0x49, [1, 4, 2, 1, 1, 2]], ["j", 0x4a, [1, 4, 2, 2, 1, 1]], ["k", 0x4b, [2, 4, 1, 2, 1, 1]], ["l", 0x4c, [2, 2, 1, 1, 1, 4]], ["m", 0x4d, [4, 1, 3, 1, 1, 1]], ["n", 0x4e, [2, 4, 1, 1, 1, 2]], ["o", 0x4f, [1, 3, 4, 1, 1, 1]], ["p", 80, [1, 1, 1, 2, 4, 2]], ["q", 0x51, [1, 2, 1, 1, 4, 2]], ["r", 0x52, [1, 2, 1, 2, 4, 1]], ["s", 0x53, [1, 1, 4, 2, 1, 2]], ["t", 0x54, [1, 2, 4, 1, 1, 2]], ["u", 0x55, [1, 2, 4, 2, 1, 1]], ["v", 0x56, [4, 1, 1, 2, 1, 2]], ["w", 0x57, [4, 2, 1, 1, 1, 2]], ["x", 0x58, [4, 2, 1, 2, 1, 1]], ["y", 0x59, [2, 1, 2, 1, 4, 1]], ["z", 90, [2, 1, 4, 1, 2, 1]], ["{", 0x5b, [4, 1, 2, 1, 2, 1]], ["|", 0x5c, [1, 1, 1, 1, 4, 3]], ["}", 0x5d, [1, 1, 1, 3, 4, 1]], ["~", 0x5e, [1, 3, 1, 1, 4, 1]], [String.fromCharCode(parseInt('007f', 16)), 0x5f, [1, 1, 4, 1, 1, 3]], [String.fromCharCode(parseInt('00f0', 16)), 0x66, [4, 1, 1, 1, 3, 1]], [String.fromCharCode(parseInt('00f1', 16)), 0x61, [4, 1, 1, 1, 1, 3]], [String.fromCharCode(parseInt('00f2', 16)), 0x60, [1, 1, 4, 3, 1, 1]], [String.fromCharCode(parseInt('00f3', 16)), 100, [1, 1, 4, 1, 3, 1]], [String.fromCharCode(parseInt('00f4', 16)), 0x62, [4, 1, 1, 3, 1, 1]], [String.fromCharCode(parseInt('00fc', 16)), 0x63, [1, 1, 3, 1, 4, 1]], [String.fromCharCode(parseInt('00fa', 16)), 0x65, [3, 1, 1, 1, 4, 1]], [String.fromCharCode(parseInt('00fd', 16)), 0x68, [2, 1, 1, 2, 1, 4]], [String.fromCharCode(parseInt('00ff', 16)), -1, [2, 3, 3, 1, 1, 1, 2]]],
        },
        /** Specifies symbology settings for the Code 128 C Barcode. */
        code128c: {
            startSymbol: String.fromCharCode(parseInt('00fe', 16)),
            stopSymbol: String.fromCharCode(parseInt('00ff', 16)),
            validatorExp: "[0-9]",
            symbolTable: [["\0", 0, [2, 1, 2, 2, 2, 2]], [String.fromCharCode(parseInt('0001', 16)), 1, [2, 2, 2, 1, 2, 2]], [String.fromCharCode(parseInt('0002', 16)), 2, [2, 2, 2, 2, 2, 1]], [String.fromCharCode(parseInt('0003', 16)), 3, [1, 2, 1, 2, 2, 3]], [String.fromCharCode(parseInt('0004', 16)), 4, [1, 2, 1, 3, 2, 2]], [String.fromCharCode(parseInt('0005', 16)), 5, [1, 3, 1, 2, 2, 2]], [String.fromCharCode(parseInt('0006', 16)), 6, [1, 2, 2, 2, 1, 3]], ["\a", 7, [1, 2, 2, 3, 1, 2]], ["\b", 8, [1, 3, 2, 2, 1, 2]], ["\t", 9, [2, 2, 1, 2, 1, 3]], ["\n", 10, [2, 2, 1, 3, 1, 2]], ["\v", 11, [2, 3, 1, 2, 1, 2]], ["\f", 12, [1, 1, 2, 2, 3, 2]], ["\r", 13, [1, 2, 2, 1, 3, 2]], [String.fromCharCode(parseInt('000e', 16)), 14, [1, 2, 2, 2, 3, 1]], [String.fromCharCode(parseInt('000f', 16)), 15, [1, 1, 3, 2, 2, 2]], [String.fromCharCode(parseInt('0010', 16)), 0x10, [1, 2, 3, 1, 2, 2]], [String.fromCharCode(parseInt('0011', 16)), 0x11, [1, 2, 3, 2, 2, 1]], [String.fromCharCode(parseInt('0012', 16)), 0x12, [2, 2, 3, 2, 1, 1]], [String.fromCharCode(parseInt('0013', 16)), 0x13, [2, 2, 1, 1, 3, 2]], [String.fromCharCode(parseInt('0014', 16)), 20, [2, 2, 1, 2, 3, 1]], [String.fromCharCode(parseInt('0015', 16)), 0x15, [2, 1, 3, 2, 1, 2]], [String.fromCharCode(parseInt('0016', 16)), 0x16, [2, 2, 3, 1, 1, 2]], [String.fromCharCode(parseInt('0017', 16)), 0x17, [3, 1, 2, 1, 3, 1]], [String.fromCharCode(parseInt('0018', 16)), 0x18, [3, 1, 1, 2, 2, 2]], [String.fromCharCode(parseInt('0019', 16)), 0x19, [3, 2, 1, 1, 2, 2]], [String.fromCharCode(parseInt('001a', 16)), 0x1a, [3, 2, 1, 2, 2, 1]], [String.fromCharCode(parseInt('001b', 16)), 0x1b, [3, 1, 2, 2, 1, 2]], [String.fromCharCode(parseInt('001c', 16)), 0x1c, [3, 2, 2, 1, 1, 2]], [String.fromCharCode(parseInt('001d', 16)), 0x1d, [3, 2, 2, 2, 1, 1]], [String.fromCharCode(parseInt('001e', 16)), 30, [2, 1, 2, 1, 2, 3]], [String.fromCharCode(parseInt('001f', 16)), 0x1f, [2, 1, 2, 3, 2, 1]], [" ", 0x20, [2, 3, 2, 1, 2, 1]], ["!", 0x21, [1, 1, 1, 3, 2, 3]], ["\"", 0x22, [1, 3, 1, 1, 2, 3]], ["#", 0x23, [1, 3, 1, 3, 2, 1]], ["$", 0x24, [1, 1, 2, 3, 1, 3]], ["%", 0x25, [1, 3, 2, 1, 1, 3]], ["&", 0x26, [1, 3, 2, 3, 1, 1]], ["\'", 0x27, [2, 1, 1, 3, 1, 3]], ["[", 40, [2, 3, 1, 1, 1, 3]], [")", 0x29, [2, 3, 1, 3, 1, 1]], ["*", 0x2a, [1, 1, 2, 1, 3, 3]], ["+", 0x2b, [1, 1, 2, 3, 3, 1]], [",", 0x2c, [1, 3, 2, 1, 3, 1]], ["-", 0x2d, [1, 1, 3, 1, 2, 3]], [".", 0x2e, [1, 1, 3, 3, 2, 1]], ["/", 0x2f, [1, 3, 3, 1, 2, 1]], ["0", 0x30, [3, 1, 3, 1, 2, 1]], ["1", 0x31, [2, 1, 1, 3, 3, 1]], ["2", 50, [2, 3, 1, 1, 3, 1]], ["3", 0x33, [2, 1, 3, 1, 1, 3]], ["4", 0x34, [2, 1, 3, 3, 1, 1]], ["5", 0x35, [2, 1, 3, 1, 3, 1]], ["6", 0x36, [3, 1, 1, 1, 2, 3]], ["7", 0x37, [3, 1, 1, 3, 2, 1]], ["8", 0x38, [3, 3, 1, 1, 2, 1]], ["9", 0x39, [3, 1, 2, 1, 1, 3]], [":", 0x3a, [3, 1, 2, 3, 1, 1]], [";", 0x3b, [3, 3, 2, 1, 1, 1]], ["<", 60, [3, 1, 4, 1, 1, 1]], ["=", 0x3d, [2, 2, 1, 4, 1, 1]], [">", 0x3e, [4, 3, 1, 1, 1, 1]], ["?", 0x3f, [1, 1, 1, 2, 2, 4]], ["@", 0x40, [1, 1, 1, 4, 2, 2]], ["A", 0x41, [1, 2, 1, 1, 2, 4]], ["B", 0x42, [1, 2, 1, 4, 2, 1]], ["C", 0x43, [1, 4, 1, 1, 2, 2]], ["D", 0x44, [1, 4, 1, 2, 2, 1]], ["E", 0x45, [1, 1, 2, 2, 1, 4]], ["F", 70, [1, 1, 2, 4, 1, 2]], ["G", 0x47, [1, 2, 2, 1, 1, 4]], ["H", 0x48, [1, 2, 2, 4, 1, 1]], ["I", 0x49, [1, 4, 2, 1, 1, 2]], ["J", 0x4a, [1, 4, 2, 2, 1, 1]], ["K", 0x4b, [2, 4, 1, 2, 1, 1]], ["L", 0x4c, [2, 2, 1, 1, 1, 4]], ["M", 0x4d, [4, 1, 3, 1, 1, 1]], ["N", 0x4e, [2, 4, 1, 1, 1, 2]], ["O", 0x4f, [1, 3, 4, 1, 1, 1]], ["P", 80, [1, 1, 1, 2, 4, 2]], ["Q", 0x51, [1, 2, 1, 1, 4, 2]], ["R", 0x52, [1, 2, 1, 2, 4, 1]], ["S", 0x53, [1, 1, 4, 2, 1, 2]], ["T", 0x54, [1, 2, 4, 1, 1, 2]], ["U", 0x55, [1, 2, 4, 2, 1, 1]], ["V", 0x56, [4, 1, 1, 2, 1, 2]], ["W", 0x57, [4, 2, 1, 1, 1, 2]], ["X", 0x58, [4, 2, 1, 2, 1, 1]], ["Y", 0x59, [2, 1, 2, 1, 4, 1]], ["Z", 90, [2, 1, 4, 1, 2, 1]], ["[", 0x5b, [4, 1, 2, 1, 2, 1]], ["\\", 0x5c, [1, 1, 1, 1, 4, 3]], ["]", 0x5d, [1, 1, 1, 3, 4, 1]], ["^", 0x5e, [1, 3, 1, 1, 4, 1]], ["_", 0x5f, [1, 1, 4, 1, 1, 3]], ["`", 0x60, [1, 1, 4, 3, 1, 1]], ["a", 0x61, [4, 1, 1, 1, 1, 3]], ["b", 0x62, [4, 1, 1, 3, 1, 1]], ["c", 0x63, [1, 1, 3, 1, 4, 1]], [String.fromCharCode(parseInt('00f0', 16)), 0x66, [4, 1, 1, 1, 3, 1]], [String.fromCharCode(parseInt('00fa', 16)), 0x65, [3, 1, 1, 1, 4, 1]], [String.fromCharCode(parseInt('00fb', 16)), 100, [1, 1, 4, 1, 3, 1]], [String.fromCharCode(parseInt('00fe', 16)), 0x69, [2, 1, 1, 2, 3, 2]], [String.fromCharCode(parseInt('00ff', 16)), -1, [2, 3, 3, 1, 1, 1, 2]]],
        },
    };

})(jQuery, Syncfusion);;
/**
* @fileOverview Plugin to style the Html Button elements
* @copyright Copyright Syncfusion Inc. 2001 - 2013. All rights reserved.
* Use of this code is subject to the terms of our license.
* A copy of the current license can be obtained at any time by e-mailing
* licensing@syncfusion.com. Any infringement will be prosecuted under
* applicable laws. 
* @version 12.1 
* @author <a href="mailto:licensing@syncfusion.com">Syncfusion Inc</a>
*/
(function ($, ej, undefined) {
    /**
    * @namespace ej
    * @class ejDiagram
    * @requires jquery.js
    * @requires jquery.easing.min.js
    * @requires jsrender.min.js
    * @requires ej.core.js
    * @requires ej.draggable.js
    * @requires ej.scroller.js
    * @requires ej.touch.js
    * @requires ej.diagram.js
    * @requires ej.diagramcommon.js
    * @requires ej.diagraminteraction.js
    * @requires ej.diagramsvg.js
    * @requires ej.diagramtools.js
    * @requires ej.diagramlayout.js
    * @requires ej.matrix.js
    * @classdesc The diagram control provides 2D surface to visualize the data as shapes, lines, text and images. It can be configured to DOM element such as DIV.
    * @example 
    * &lt;div id="diagram"&gt;&lt;/div&gt;<br>
    * &lt;script&gt;
    * //Create Diagram
    $("#diagram").ejDiagram();
    * &lt;/script&gt
    */

    "use strict";
    //#region ej.Diagram widget
    ej.widget("ejDiagram", "ej.datavisualization.Diagram", {
        // widget element will be automatically set in this
        element: null,

        // user defined model will be automatically set in this
        model: null,
        validTags: ["div"],

        //#region defaults
        defaults: {
            /**Specifies the width of the diagram. 
            * @default null
            * @type {string}
            * @example
            * &lt;div id="diagram"&gt;&lt;/div&gt;<br>
            * &lt;script&gt;
            * $("#diagram").ejDiagram({ width:"100%" });
            * &lt;/script&gt
              * @memberof ejDiagram
              * @instance
            */
            width: null,
            /**Specifies the height of the diagram.
            * @default null
            * @type {string}
            * @example
            * &lt;div id="diagram"&gt;&lt;/div&gt;<br>
            * &lt;script&gt;
            * $("#diagram").ejDiagram({ height:"100%" });
            * &lt;/script&gt
              * @memberof ejDiagram
              * @instance
            */
            height: null,

            deleteDependent: true,

            adjustDependent: true,

            /**Array of node objects where each object has definition/properties of node.
            * @default []
            * @type {array}
            * @example
            * &lt;div id="diagram"&gt;&lt;/div&gt;<br>
            * &lt;script&gt;
            * var nodes = [{ name: "node", width: 175, height: 60}];
            * $("#diagram").ejDiagram({ nodes:nodes });
            * &lt;/script&gt
              * @memberof ejDiagram
              * @instance
            */
            nodes: [],
            /**Array of connector objects where each object has definition/properties of connector.
            * @default []
            * @type {array}
            * @example
            * &lt;div id="diagram"&gt;&lt;/div&gt;<br>
            * &lt;script&gt;
            * var connectors = [{ name: "connector" sourceNode: "node1", targetNode: "node2"}];
            * $("#diagram").ejDiagram({ connectors:connectors });
            * &lt;/script&gt
              * @memberof ejDiagram
              * @instance
            */
            connectors: [],

                /** To set the default properties for nodes and connectors.
            * @default {}
            * @type {object}
            * @example
            * &lt;div id="diagram"&gt;&lt;/div&gt;<br>
            * &lt;script&gt;
            * $("#diagram").ejDiagram({ defaultSettings: { node:  {width: 110, height: 40, fillColor:"skyblue"}}});
            * &lt;/script&gt
            * @memberof ejDiagram
            * @instance
           */
            defaultSettings: {
                /** To set the default properties of connector.
                * @default null
                * @type {object}
                * @example
                * &lt;div id="diagram"&gt;&lt;/div&gt;<br>
                * &lt;script&gt;
                * $("#diagram").ejDiagram({ defaultSettings:{ connector:{ lineColor: "gray", lineWidth: 2 }}});
                * &lt;/script&gt
                 * @alias  ejDiagram#defaultSettings->connector
                  * @memberof ejDiagram
                  * @instance
                */
                connector: null,
                /**To set the default properties of node.
                * @default null
                * @type {object}
                * @example
                * &lt;div id="diagram"&gt;&lt;/div&gt;<br>
                * &lt;script&gt;
                * $("#diagram").ejDiagram({ defaultSettings:{node: { fillColor: "#83A93F", borderColor: "#000000" }}});
                * &lt;/script&gt
                * @alias  ejDiagram#defaultSettings->node
                  * @memberof ejDiagram
                  * @instance
                */
                node: null
            },


            /**To customize the node properties before rendering the node.
               * @default null
               * @type {object}
               * @example
               * &lt;div id="diagram"&gt;&lt;/div&gt;<br>
               * &lt;script&gt;
               *function nodeTemplate(diagram, node, data) {node.labels[0].text = data.Name};
               * $("#diagram").ejDiagram({ nodeTemplate:nodeTemplate});
               * &lt;/script&gt
                 * @memberof ejDiagram
                 * @instance
               */

            nodeTemplate: null,
            /** To customize the connector properties before rendering the connector.
               * @default null
               * @type {object}
               * @example
               * &lt;div id="diagram"&gt;&lt;/div&gt;<br>
               * &lt;script&gt;
               *function connectorTemplate(diagram, connector, data) {connector.labels[0].text = data.Name};
               * $("#diagram").ejDiagram({ connectorTemplate:connectorTemplate});
               * &lt;/script&gt
                 * @memberof ejDiagram
                 * @instance
               */
            connectorTemplate: null,
            /** Configures the datasource for diagram.
      * @default {id: "Id", parent: "ReportingPerson",dataSource: data  }
      * @type {object}
      * @example
      * &lt;div id="diagram"&gt;&lt;/div&gt;<br>
      * &lt;script&gt;
      * $("#diagram").ejDiagram({ dataSourceSettings: {id: "Id", parent: "ReportingPerson",dataSource: data  }});
      * &lt;/script&gt
         * @memberof ejDiagram
        * @instance
      */
            dataSourceSettings: {
            /** Describes the datasource for diagram. 
        * @default null
        * @type {object}
        * @example
        * &lt;div id="diagram"&gt;&lt;/div&gt;<br>
        * &lt;script&gt;
        * $("#diagram").ejDiagram({ dataSourceSettings: {dataSource: localData}});
        * &lt;/script&gt
        * @alias ejDiagram#dataSourceSettings->dataSource
        */

                dataSource: null,
                /**Describes query to retrieve data from the datasource.
        * @default null
        * @type {object}
        * @example
        * &lt;div id="diagram"&gt;&lt;/div&gt;<br>
        * &lt;script&gt;
        *query: ej.Query().from("Categories").select("CategoryID,CategoryName").take(3);
        * &lt;/script&gt
        * @alias ejDiagram#dataSourceSettings->query
        */
                query: null,
                /**Describes table name to execute query on the corresponding table.
     * @default null
     * @type {String}
     * @example
     * &lt;div id="diagram"&gt;&lt;/div&gt;<br>
     * &lt;script&gt;
     * $("#diagram").ejDiagram({ dataSourceSettings:{tableName: "Categories"}});
     * &lt;/script&gt
     * @alias ejDiagram#dataSourceSettings->tablename
     */
                tableName: null,
                /** Describes the unique id of datasource items.
        * @default null
        * @type {String}
        * @example
        * &lt;div id="diagram"&gt;&lt;/div&gt;<br>
        * &lt;script&gt;
        * $("#diagram").ejDiagram({ dataSourceSettings: {id: "CategoryID"}});
        * &lt;/script&gt
        * @alias ejDiagram#dataSourceSettings->id
        */
                id: "",

                /**Describes the parent id of datasource items.
        * @default null
        * @type {String}
        * @example
        * &lt;div id="diagram"&gt;&lt;/div&gt;<br>
        * &lt;script&gt;
        * $("#diagram").ejDiagram({ dataSourceSettings: { parent: "reportingPerson"}});
        * &lt;/script&gt
        * @alias ejDiagram#dataSourceSettings->parent
        */
                parent: "",
  
                nodes: null,
     
                connectors: null,
                //nodeBinding: null,
                //connectorBinding: null,
            },
            /**Specifies the snap settings of the diagram.
            * @type {object}
               * @memberof ejDiagram
              * @instance
            */
            snapSettings: /** @lends ejDiagram# */ {
                /**Specifies the settings of horizontal grid lines.
                * @type {object}
                * @alias ejDiagram#snapSettings->horizontalGridLines
                */
                horizontalGridLines: {
                    /**Specifies the space between horizontal grid lines
                    * @default [1.25, 18.75, 0.25, 19.75, 0.25, 19.75, 0.25, 19.75, 0.25, 19.75]
                    * @type {array}
                    * @example
                    * &lt;div id="diagram"&gt;&lt;/div&gt;<br>
                    * &lt;script&gt;
                    * $("#diagram").ejDiagram({ snapSettings: { 
                    *     horizontalGridLines: { linesInterval: [1, 14, 0.25, 15, 0.25, 15, 0.25, 15, 0.25, 15] }
                    * }}); 
                    * &lt;/script&gt
                    * @alias ejDiagram#snapSettings->horizontalGridLines->linesInterval
                    */
                    linesInterval: [1.25, 18.75, 0.25, 19.75, 0.25, 19.75, 0.25, 19.75, 0.25, 19.75],
                    /**Specifies the snap intervals for horizontal grid lines.
                    * @default [20]
                    * @type {array}
                    * @example 
                    * &lt;div id="diagram"&gt;&lt;/div&gt;<br>
                    * &lt;script&gt;
                    * $("#diagram").ejDiagram({ snapSettings: { horizontalGridLines: { snapInterval: [10] }}}); 
                    * &lt;/script&gt
                   * @alias ejDiagram#snapSettings->horizontalGridLines->snapInterval
                    */
                    snapInterval: [20],
                    /**Specifies the pattern of dashes and gaps used to stroke horizontal grid lines
                    * @default "0"
                    * @type {string}
                    * @example 
                    * &lt;div id="diagram"&gt;&lt;/div&gt;<br>
                    * &lt;/script&gt
                    * $("#diagram").ejDiagram({ snapSettings: { horizontalGridLines: { lineDashArray: "2 2"}}}); 
                    * &lt;/script&gt
                     * @alias ejDiagram#snapSettings->horizontalGridLines->lineDashArray
                    */
                    lineDashArray: "0",
                    /**		
                    * Line color for horizontal gridlines
                    * @default "lightgray"
                    * @type {string}
                    * @example 
                    * &lt;div id="diagramcontent"&gt;&lt;/div&gt;<br>
                    * &lt;/script&gt
                    * var gridline = { "snapConstraints":"lineColor": "blue" };
                    * //Linecolor of the horizontal gridlines
                    * $("#diagramContent").ejDiagram({snapSettings: { horizontalGridLines: gridline} }); 
                    * &lt;/script&gt
                   * @alias ejDiagram#snapSettings->horizontalGridLines->lineColor
                    */
                    lineColor: "lightgray"
                },
                /**Specifies the settings of vertical grid lines.
                * @type {object}
                * @alias ejDiagram#snapSettings->verticalGridLines
                */
                verticalGridLines: {
                    /**Specifies the space between vertical grid lines
                    * @default [1.25, 18.75, 0.25, 19.75, 0.25, 19.75, 0.25, 19.75, 0.25, 19.75]
                    * @type {array}
                    * @example
                    * &lt;div id="diagram"&gt;&lt;/div&gt;<br>
                    * &lt;script&gt;
                    * $("#diagram").ejDiagram({ snapSettings: { 
                    *     verticalGridLines: { linesInterval: [1, 14, 0.25, 15, 0.25, 15, 0.25, 15, 0.25, 15] }
                    * }}); 
                    * &lt;/script&gt
                    * @alias ejDiagram#snapSettings->verticalGridLines->linesInterval
                    */
                    linesInterval: [1.25, 18.75, 0.25, 19.75, 0.25, 19.75, 0.25, 19.75, 0.25, 19.75],
                    /**Specifies the snap intervals for vertical grid lines.
                    * @default [20]
                    * @type {array}
                    * @example 
                    * &lt;div id="diagram"&gt;&lt;/div&gt;<br>
                    * &lt;script&gt;
                    * $("#diagram").ejDiagram({ snapSettings: { verticalGridLines: { snapInterval: [10] }}}); 
                    * &lt;/script&gt
                  * @alias ejDiagram#snapSettings->verticalGridLines->snapInterval
                    */
                    snapInterval: [20],
                    /**Specifies the pattern of dashes and gaps used to stroke vertical grid lines
                    * @default "0"
                    * @type {string}
                    * @example 
                    * &lt;div id="diagram"&gt;&lt;/div&gt;<br>
                    * &lt;/script&gt
                    * $("#diagram").ejDiagram({ snapSettings: { verticalGridLines: { lineDashArray: "2 2"}}}); 
                    * &lt;/script&gt
                   * @alias ejDiagram#snapSettings->verticalGridLines->lineDashArray
                    */
                    lineDashArray: "0",
                    /**		
               * Line Color of vertical grid lines
               * @default "lightgray"
               * @type {string}
               * @example 
               * &lt;div id="diagramcontent"&gt;&lt;/div&gt;<br>
               * &lt;/script&gt
               * var gridline = { "snapConstraints":"lineColor": "blue" };
               * //Linecolor of the vertical gridlines
               * $("#diagramContent").ejDiagram({snapSettings: { verticalGridLines: gridline} });
               * &lt;/script&gt
              * @alias ejDiagram#snapSettings->verticalGridLines->lineColor
               */
                    lineColor: "lightgray"
                },
                /**		
           * Enables or disables the snapconstraints of gridlines see{@link snapConstraints}
           * @name SnapConstraints
           * @default SnapConstraints.ShowLines 
           * @type {enum}
           * @example 
           * &lt;div id="diagramcontent"&gt;&lt;/div&gt;<br>
           * &lt;/script&gt
           * var gridline = { "snapConstraints": ej.datavisualization.Diagram.SnapConstraints.ShowLines};
           * //snap constraints of gridlines
           * $("#diagramContent").ejDiagram({snapSettings: gridline} });
           * &lt;/script&gt
           * @alias ejDiagram#snapSettings->snapConstraints
           */
                snapConstraints: 1 | 2 | 4 | 8,
                /**		
            * Enables or disables the snap object 
            * @default true
            * @type {Boolean}
            * @example 
            *var snap = {"enableSnapToObject":true};
            * //enableSnapToObject: 
            * $("#diagramContent").ejDiagram({snapSettings: snap}); 
            * @alias ejDiagram#snapSettings->enableSnapToObject
            */
                enableSnapToObject: true,
                /**		
         * The Angle to which the object to be snapped
         * @default 5
         * @type {Number}
         * @example 
         *var snap = {"snapAngle":5};
         * //enableSnapToObject: 
         *$("#diagramContent").ejDiagram({snapSettings: snap}); 
         * @alias ejDiagram#snapSettings->snapAngle
         */
                snapAngle: 5,
                /**		
            * Relative distance between the nearer object and selected object
            * @default 5
            * @type {Number}
            * @example 
            *var snap = {"snapObjectDistance":5};
            * //snapObjectDistance
            *$("#diagramContent").ejDiagram({snapSettings: snap}); 
            * @alias ejDiagram#snapSettings->snapObjectDistance
            */
                snapObjectDistance: 5
            },
            /**		
            * The pageSetting of the diagram
               * @memberof ejDiagram
              * @instance*/
            pageSettings: /** @lends ejDiagram# */ {
                /**		
         * This is used to set the width of the page 
         * @default null
         * @type {Number}
         * @example 
         * //pageWidth
         *$("#diagramContent").ejDiagram({pageSettings: {pageWidth: 4500}}); 
         * @alias ejDiagram#pageSettings->pageWidth
         */
                pageWidth: null,
                /**		
         * This is used to set the height of the page 
         * @default null
         * @type {Number}
         * @example 
         * //pageHeight of the diagram
         *$("#diagramContent").ejDiagram({pageSettings: {pageHeight: 4500}}); 
         * @alias ejDiagram#pageSettings->pageHeight
         */
                pageHeight: null,
                /**		
         * Enables or disables the multiple page
         * @default false
         * @type {Boolean}
         * @example 
         * //multiplePage of the diagram
         *$("#diagramContent").ejDiagram({pageSettings: {multiplePage: false}}); 
         * @alias ejDiagram#pageSettings->multiplePage
         */
                multiplePage: false,
                /**		
         *This is used to set the width of the page border 
         * @default 0
         * @type {Number}
         * @example 
         * //pageBorderWidth of the diagram
         *$("#diagramContent").ejDiagram({pageSettings: {pageBorderWidth: 3}}); 
         * @alias ejDiagram#pageSettings->pageBorderWidth
         */

                pageBorderWidth: 0,
                /**		
          * Used set the background color of the page 
          * @default "#ffffff"
          * @type {String}
          * @example 
          * //pageBackgroundColor of the diagram
          * $("#diagramContent").ejDiagram({pageSettings: {pageBackgroundColor: "#ffffff"}}); 
          * @alias ejDiagram#pageSettings->pageBackgroundColor
          */
                pageBackgroundColor: "#ffffff",
                /**		
         * Used to set the border color of the page
         * @default "#565656"
         * @type {String}
         * @example 
         * //pageBorderColor of the diagram
         *$("#diagramContent").ejDiagram({pageSettings: {pageBorderColor: "#565656"}}); 
         * @alias ejDiagram#pageSettings->pageBorderColor
         */
                pageBorderColor: "#565656",
                /**		
         * Margin of the page
         * @default 24
         * @type {Number}
         * @example 
         * //pageMargin of the diagram
         *$("#diagramContent").ejDiagram({pageSettings: {pageMargin: 24}}); 
         * @alias ejDiagram#pageSettings->pageMargin
         */
                pageMargin: 24,
                /**		
         * Enables or disables the page breaks in diagram
         * @default false
         * @type {Boolean}
         * @example 
         * //showPageBreak of the diagram
         *$("#diagramContent").ejDiagram({pageSettings: {showPageBreak: false}}); 
         * @alias ejDiagram#pageSettings->showPageBreak
         */
                showPageBreak: false,
                /**		
          * Orientation allows placement of element on a page that set through page set up see{@link PageOrientations}
          * @name pageOrientation
          * @default "portrait"
          * @type {enum}
          * @example 
          * //pageOrientation of the diagram
          *$("#diagramContent").ejDiagram({pageSettings: {pageOrientation: "portrait"}}); 
          * @alias ejDiagram#pageSettings->pageOrientation
          */
                pageOrientation: "portrait",

                /**		
      * scrollLimit used to set the scroll limit on the diagram that set through page set up see{@link ScrollLimit}
      * @name scrollLimit
      * @default "infinity"
      * @type {enum}
      * @example 
      * //scrollLimit of the diagram
      *$("#diagramContent").ejDiagram({pageSettings: {scrollLimit: "infinity"}}); 
      * @alias ejDiagram#pageSettings->scrollLimit
      */
                scrollLimit: "infinity",

                /**		
     * scrollableArea used to set the scroll area on the diagram.
     * @name scrollableArea
     * @default {scrollableArea: { x: Infinity, y: Infinity, width: Infinity, height: Infinity }}
     * @type {object}
     * @example 
     * //scrollableArea of the diagram
     *$("#diagramContent").ejDiagram({pageSettings: {scrollableArea: { x: 0, y: 0, width: 0, height: 0 }}}); 
     * @alias ejDiagram#pageSettings->scrollableArea
     */
                scrollableArea: { x: Infinity, y: Infinity, width: Infinity, height: Infinity },

                /**
     * Specifics the auto scroll starting point of diagram control.
     * @name autoScrollBorder
     * @default { autoScrollBorder: { left: 15, top: 15, right: 15, bottom: 15 }}
     * @type {object}
     * @example
     * //scrollableArea of the diagram
     *$("#diagramContent").ejDiagram({pageSettings: {autoScrollBorder: { left: 15, top: 15, right: 15, bottom: 15 }}}); 
     * @alias ejDiagram#pageSettings->autoScrollBorder
     */
                autoScrollBorder: { left: 15, top: 15, right: 15, bottom: 15 },
            },

            locale: "en-US",
            /**		
           *The contextmenu of the diagram
              * @memberof ejDiagram
              * @instance*/
            contextMenu: {
                /**		
      * Collection of items of Contextmenu
      * @default []
      * @type {Object}                          
      * @example 
	  * &lt;div id="diagramcontent"&gt;&lt;/div&gt;<br>
      * &lt;script&gt;
      * //Collection of  items
      *var menucollection = [{ "name": "hyperLink", "text": "Hyperlink", "image": "", "style": "" }];
	  *var contextMenu = { items: menucollection};
      *$("#diagramContent").ejDiagram({contextMenu: contextMenu}); 
      * &lt;/script&gt
      * @alias ejDiagram#contextMenu->items
       * @memberof ejDiagram
         * @instance
        */
                items: [],
                /**
               *Events to trigger action, when click option in diagram
               * @event
               * @name ejDiagram#contextMenu#click
               * @param {Object} argument args parameter from diagram 
               * @param {Object} argument.element parameter returns node or connector 
               * @example
                * &lt;div id="diagramcontent"&gt;&lt;/div&gt;<br>
                                         * &lt;script&gt;
               * //  click event for Contextmenu item
               *var contextMenu = { items: menucollection, onClick: contextMenuClick};
               *$("#diagramContent").ejDiagram({contextMenu: contextMenu}); 
                * &lt;/script&gt
               });
                 * @alias ejDiagram#contextMenu->click
               */
                click: null,
                /**
*Events to trigger before, open context menu
* @event
* @name ejDiagram#contextMenu#beforeOpen
* @param {Object} argument args parameter from diagram 
* @param {Object} argument.element parameter returns node or connector 
* @example
* //  beforeOpen event for Contextmenu item
 * &lt;div id="diagramcontent"&gt;&lt;/div&gt;<br>
                          * &lt;script&gt;
* //  click event for Contextmenu item
*var contextMenu = { items: menucollection, onBeforeOpen: contextMenuBeforeOpen};
*$("#diagramContent").ejDiagram({contextMenu: contextMenu}); 
 * &lt;/script&gt
});
*/

                beforeOpen: null,
                /**To display or hide items of contextmenu
                          * @default true
                          * @type {Boolean}
                          * @example
                          * &lt;div id="diagramcontent"&gt;&lt;/div&gt;<br>
                          * &lt;script&gt;
                          * $("#diagramcontent").ejDiagram({showCustomMenuItemsOnly: true});
                          * &lt;/script&gt
						    * @alias ejDiagram#contextMenu->showCustomMenuItemsOnly
                             * @memberof ejDiagram
                             * @instance
                          */

                showCustomMenuItemsOnly: false
            },
            enableContextMenu: true,
            /** Enables or disables Auto scroll in diagram
           * @default true
           * @type {Boolean}
           * @example
           * &lt;div id="diagramcontent"&gt;&lt;/div&gt;<br>
           * &lt;script&gt;
           * $("#diagramcontent").ejDiagram({enableAutoScroll: true});
           * &lt;/script&gt
              * @memberof ejDiagram
              * @instance
           */
            enableAutoScroll: true,
            /** Enables or disables tooltip of diagram
         * @default true
         * @type {Boolean}
         * @example
         * &lt;div id="diagramcontent"&gt;&lt;/div&gt;<br>
         * &lt;script&gt;
         * $("#diagramcontent").ejDiagram({showTooltip: true});
         * &lt;/script&gt
            * @memberof ejDiagram
              * @instance
         */
            showTooltip: true,
            /** To set custom style for tool tip of diagram
         * @default ""
         * @type {String}
         * @example
         * &lt;script type="text/x-jsrender" id="toolTipId"&gt;
         * &lt;div id="diagramcontent"&gt;&lt;/div&gt;<br>
         *&lt;span &gt;{{:offsetX}}{{:x}}{{:offsetY}}{{:y}}
         * &lt;/script&gt
         *&lt;/span&gt;
         * &lt;script&gt;
         * $("#diagramcontent").ejDiagram({ tooltipTemplateId: "toolTipId"});
         * &lt;/script&gt
            * @memberof ejDiagram
              * @instance
         */
            tooltipTemplateId: "",
            /** Distance between the node and the viewport from where autoscroll should start
         * @default 20
         * @type {Number}
         * @example
         * &lt;div id="diagramcontent"&gt;&lt;/div&gt;<br>
         * &lt;script&gt;
         * $("#diagramcontent").ejDiagram({autoScrollMargin: 20});
         * &lt;/script&gt
            * @memberof ejDiagram
              * @instance
         */
            autoScrollMargin: 20,
            /** To arrange elements on the page in diagram
               * @memberof ejDiagram
              * @instance*/
            layout: /** @lends ejDiagram# */{
                /**		
          * Type of the layout on the page
          * @default "none"
          * @type {String}
          * @example 
          * //type of the layout
          * $("#diagramContent").ejDiagram({layout: {type: "none"}}); 
          * @alias ejDiagram#layout->type
          */
                type: "none",
                /**		
          * Alignment of elements based on some specific direction see {@link LayoutOrientations}
          * @default "topToBottom"
          * @type {String}
          * @example 
          * //orientation of the layout
          *$("#diagramContent").ejDiagram({layout: {orientation: "topToBottom"}}); 
          * @alias ejDiagram#layout->orientation
          */
                orientation: "toptobottom",
                /**		
          * Horizontal Spacing between the nodes in layout
          * @default 30
          * @type {Number}
          * @example 
          * //horizontalSpacing of the layout
          *$("#diagramContent").ejDiagram({layout: {horizontalSpacing: 30}}); 
          * @alias ejDiagram#layout->horizontalSpacing
          */
                horizontalSpacing: 30,
                /**		
          * Vertical Spacing between the nodes in layout
          * @default 30
          * @type {Number}
          * @example 
          * //verticalSpacing of the layout
          *$("#diagramContent").ejDiagram({layout: {verticalSpacing: 30}}); 
          * @alias ejDiagram#layout->verticalSpacing
          */
                verticalSpacing: 30,
                /**		
         * Margin value along X -Axis 
         * @default 0
         * @type {Number}
         * @example 
         * //marginX of the layout
         *$("#diagramContent").ejDiagram({layout: {marginX: 0}}); 
         * @alias ejDiagram#layout->marginX
         */
                marginX: 0,
                /**		
          * Margin value along y -Axis
          * @default 0
          * @type {Number}
          * @example 
          * // marginY of the layout
          *$("#diagramContent").ejDiagram({layout: { marginY: 0}}); 
          * @alias ejDiagram#layout-> marginY
          */
                marginY: 0,
                /**	 Nodes position is set as fixed and draw all node based on the fixed node
             * @default ""
             * @type {String}
             * @example 
             * //fixedNode of the layout
             *$("#diagramContent").ejDiagram({layout: { fixedNode: "nodename"}}); 
             * @alias ejDiagram#layout-> fixedNode
             */
                fixedNode: "",
            },
            /** To add frequently using commands around selector
         * @default ""
         * @type {string}
         * @example
         * &lt;div id="diagramcontent"&gt;&lt;/div&gt;<br>
         * &lt;script&gt;
         * var userHandle= [];
         * function createUserHandles() {
         * var cloneHandle = ej.datavisualization.Diagram.UserHandle();
         * userhandle.push(cloneHandles);
         * }
         * $("#diagramcontent").ejDiagram({userHandles:userHandle});
         * &lt;/script&gt
            * @memberof ejDiagram
              * @instance
         */
            userHandles: [],
            /** Enables to choose which interactive feature to be performed
         * @default ""
         * @type {Object}
         * @example
         * &lt;div id="diagramcontent"&gt;&lt;/div&gt;<br>
         * &lt;script&gt;
         * $("#diagramcontent").ejDiagram({ drawingTools: {textTool: ej.datavisualization.Diagram.TextTool()}});
         * &lt;/script&gt
            * @memberof ejDiagram
              * @instance
         */
            drawingTools: {},
            /** Image to be used as an background of the elements
         * @default null
         * @type {String}
         * @example
         * &lt;div id="diagramcontent"&gt;&lt;/div&gt;<br>
         * &lt;script&gt;
         * $("#diagramcontent").ejDiagram({ backgroundImage: "Syncfusion.png"});
         * &lt;/script&gt
            * @memberof ejDiagram
              * @instance
         */
            backgroundImage: null,
            /** color to be used as an background of the elements
     * @default null
     * @type {String}
     * @example
     * &lt;div id="diagramcontent"&gt;&lt;/div&gt;<br>
     * &lt;script&gt;
     * $("#diagramcontent").ejDiagram({ backgroundColor: "red"});
     * &lt;/script&gt
        * @memberof ejDiagram
          * @instance
     */
            backgroundColor: "transparent",
            /**		
              * Enables or disables the selector constraints see{@link SelectorConstraints}
              * @default SelectorConstraints.All 
              * @type {enum}
              * @example 
              * &lt;div id="diagramcontent"&gt;&lt;/div&gt;<br>
              * &lt;script&gt;
              *$("#diagramContent").ejDiagram({selectorConstraints: ej.datavisualization.Diagram.SelectorConstraints.None} }); 
              * &lt;/script&gt
                 * @memberof ejDiagram
              * @instance
              */
            selectorConstraints: 1 << 1 | 1 << 2 | 1 << 3,

           	
          /** Sets the default behaviour of the diagram see{@link DiagramConstraints}
         * @default constraints.All 
         * @type {enum}
         * @example 
         * &lt;div id="diagramcontent"&gt;&lt;/div&gt;<br>
         * &lt;script&gt;
         *$("#diagramContent").ejDiagram({constraints: ej.datavisualization.Diagram.DiagramConstraints.Default} }); 
         * &lt;/script&gt
            * @memberof ejDiagram
         * @instance
         */

            constraints: 1 << 1 | 1 << 3 | 1 << 4 | 1 << 5 | 1 << 6,

            /** Sets the default behaviour of the diagram see{@link Tool}
       * @default Tool.All 
       * @type {enum}
       * @example 
       * &lt;div id="diagramcontent"&gt;&lt;/div&gt;<br>
       * &lt;script&gt;
       *$("#diagramContent").ejDiagram({tool: ej.datavisualization.Diagram.Tool.Default} }); 
       * &lt;/script&gt
          * @memberof ejDiagram
       * @instance
       */


            tool: 1 << 1 | 1 << 2,
            /**		
   * Type of the Connector to be drawn through drawing tool see {@link ConnectorType}
   * @default 1
   * @type {String}
   * @example 
   * &lt;div id="diagramcontent"&gt;&lt;/div&gt;<br>
   * &lt;script&gt;
    *$("#diagramContent").ejDiagram({connectorType: ej.datavisualization.Diagram.ConnectorType.straightLine} }); 
   * &lt;/script&gt
   * @memberof ejDiagram
         * @instance
   */


            connectorType: "orthogonalLine",

            /**		
        * Allow to set the factor by which we can zoom in or zoom out
        * @default 1
        * @type {Number}
        * @example 
        * &lt;div id="diagramcontent"&gt;&lt;/div&gt;<br>
        * &lt;script&gt;
        *$("#diagramContent").ejDiagram({zoomFactor: 1}); 
        * &lt;/script&gt
        * @memberof ejDiagram
              * @instance
        */
            zoomFactor: 0.2,
            /**
        *Triggers When node collection get change
        * @event
        * @name ejDiagram#nodeCollectionChange
        * @param {Object} argument args parameter from diagram              
        * @param {String} argument.changetype return should be insert or remove
        * @param {Object} argument.element parameter returns node which to be added or deleted
        * @example
        * // nodeCollectionChange event for diagram
        * $("#diagramcontent").ejDiagram({
        * nodeCollectionChange:function (args)  {}
        *  });
        */
            nodeCollectionChange: null,
            /**
       *Triggers When auto scroll get change
       * @event
       * @name ejDiagram#autoScrollChange             
       * @param {String} argument.delay return should be insert or remove
       * @param {Object} argument.distanceFromDiagram parameter returns node which to be added or deleted
       * @example
       * // nodeCollectionChange event for diagram
       * $("#diagramcontent").ejDiagram({
       * autoScrollChange:function (args)  {}
       *  });
       */
            autoScrollChange: null,
            /**
        *Triggers the selected object changed in the diagram model
        * @event
        * @name ejDiagram#itemClick
        * @param {object} argument args.cancel  if the event should be cancelled ,otherwise fasle             
        * @param {String} argument.type return event name
        * @param {Object} argument.actualObject node was clicked
        * @param {Object} argument.selectedObject actual selected object
        * @param {Object} argument.model contains holds the reference of diagram
        
        * @example
        * // connectorCollectionChange event for diagram
        * $("#diagramcontent").ejDiagram({
        * connectorCollectionChange:function (args) {}
        });
        */
            itemClick: null,
            /**
        *Triggers When connector collection get change
        * @event
        * @name ejDiagram#connectorCollectionChange
        * @param {Object} argument args parameter from diagram              
        * @param {String} argument.changetype return should be insert or remove
        * @param {Object} argument.element parameter returns connector which to be added or deleted
        * @example
        * // connectorCollectionChange event for diagram
        * $("#diagramcontent").ejDiagram({
        * connectorCollectionChange:function (args) {}
        });
        */
            connectorCollectionChange: null,
            /**
        *Triggers When selection get change
        * @event
        * @name ejDiagram#selectionChanged
        * @param {Object} argument args parameter from diagram 
        * @param {String} argument.changetype return should be inserted or removed
        * @param {Object} argument.element parameter returns node or connector which to be added or deleted
        * @example
        * //  selectionChange event for diagram
        * $("#diagramcontent").ejDiagram({
        * selectionChange:function (args)  {}
        });
        */
            selectionChange: null,
            /**
        *Triggers When mouse leaves node/connector
        * @event
        * @name ejDiagram#mouseLeave
        * @param {Object} argument args parameter from diagram 
        * @param {Object} argument.element parameter returns node or connector 
            * @param {Object} argument.source the object dragged from this container 
            * @param {Object} argument.target the object is going to be dropped.  
            * @example
            * //  mouseLeave event for diagram
            * $("#diagramcontent").ejDiagram({
            * mouseLeave:function (args)  {}
               });
            */
            mouseLeave: null,
            /**
        *Triggers When mouse enter node/connector
        * @event
        * @name ejDiagram#mouseEnter
        * @param {Object} argument args parameter from diagram 
        * @param {Object} argument.element parameter returns node or connector 
        * @param {Object} argument.source the object dragged from this container 
        * @param {Object} argument.target the object is going to be dropped.  
        * @example
        * //  mouseEnter event for diagram
        * $("#diagramcontent").ejDiagram({
        * mouseEnter:function (args)  {}
        });
        */
            mouseEnter: null,
            /**
        *Triggers When mouse Hover on node/connector
        * @event
        * @name ejDiagram#mouseOver
        * @param {Object} argument args parameter from diagram 
        * @param {Object} argument.diagram parameter returns node or connector 
        * @param {Object} argument.source the object dragged from this container 
        * @param {Object} argument.target the object is going to be dropped.  
        * @example
        * //  mouseOver event for diagram
        * $("#diagramcontent").ejDiagram({
        * mouseOver:function (args)  {}
        });   
        */
            mouseOver: null,
            /**
        *Triggers When click node/connector
        * @event
        * @name ejDiagram#click
        * @param {Object} argument args parameter from diagram 
        * @param {Object} argument.element parameter returns diagram
        * @example
        * //  click event for diagram
        * $("#diagramcontent").ejDiagram({
        * click:function (args)  {}
        });
        */
            click: null,
            /**
        *Triggers When doubleClick node/connector
        * @event
        * @name ejDiagram#doubleClick
        * @param {Object} argument args parameter from diagram 
        * @param {Boolean} argument.cancel if the event should be cancelled ,otherwise fasle
        * @param {Object} argument.element parameter returns selected object in list
        * @example
        * //  doubleClick event for diagram
        * $("#diagramcontent").ejDiagram({
        * doubleClick:function (args)  {}
        });
        */
            doubleClick: null,
            /**
        *Triggers When drag and drop symbols from palette to drawing area 
        * @event
        * @name ejDiagram#drop
        * @param {Object} argument args parameter from diagram 
        * @param {Object} argument.element parameter returns node or connector
        * @example
        * //  drop event for diagram
        * $("#diagramcontent").ejDiagram({
        * drop:function (args)  {}
        });
        */
            drop: null,
            /**
  *Triggers When drag and drop symbols from palette to drawing area 
  * @event
  * @name ejDiagram#drag
  * @param {Object} argument args parameter from diagram 
  * @param {Object} argument.element parameter returns node or connector
  * @example
  * //  deag event for diagram
  * $("#diagramcontent").ejDiagram({
  * drag:function (args)  {}
  });
  */
            drag: null,
            /**
        *Triggers When label edit gets ended 
        * @event
        * @name ejDiagram#textchanged
        * @param {Object} argument args parameter from diagram 
        * @param {Object} argument.element parameter returns shape
        * @param {String} argument.value parameter returns value in label edit box
        * @example
        * // textChanged event for diagram
        * $("#diagramcontent").ejDiagram({
        * textChanged:function (args)  {}
        });
        */
            textChange: null,
            /**
        *Triggers When node size has been change
        * @event
        * @name ejDiagram#nodeSizeChange
        * @param {Object} argument args parameter from diagram 
        * @param {Object} argument.element parameter returns node that was resized
        * @param {Number} argument.offset parameter returns size of the node
        * @param {Boolean} argument.cancel if the event should be cancelled ,otherwise fasle
        * @example
        * // sizeChange event for diagram
        * $("#diagramcontent").ejDiagram({
        * sizeChange:function (args)  {}
         });
        */
            sizeChange: null,
            /**
        *Triggers When connection changes has been done
        * @event
        * @name ejDiagram#connectionChange
        * @param {Object} argument args parameter from diagram 
        * @param {Object} argument.element parameter returns connector
        * @param {String} argument.connection parameter returns source or target node that was connected
        * @param {port} argument.port parameter returns source or target port that was connected
        * @example
        * // connectionChange event for diagram
        * $("#diagramcontent").ejDiagram({
        * connectionChange:function (args)  {}
        }); 
        */
            connectionChange: null,
            /**
        *Triggers When rotate node
        * @event
        * @name ejDiagram#rotationChange
        * @param {Object} argument args parameter from diagram 
        * @param {Object} argument.element parameter returns node
        * @param {Boolean} argument.cancel if the event should be cancelled ,otherwise fasle
        * @example
        * // rotationChange event for diagram
        * $("#diagramcontent").ejDiagram({
        * rotationChange:function (args)  {}
        });
        */
            rotationChange: null
        },

        observables: [
            "enableContextMenu",
            "enableAutoScroll",
            "showTooltip",
            "zoomFactor",
            "pageSettings.pageHeight",
            "pageSettings.pageWidth",
            "pageSettings.multiplePage",
            "pageSettings.pageBorderWidth",
            "pageSettings.pageBackgroundColor",
            "pageSettings.pageBorderColor",
            "pageSettings.pageMargin",
            "pageSettings.showPageBreak",
            "pageSettings.pageOrientation",
			"pageSettings.scrollLimit",
            "snapSettings.enableSnapToObject",
            "snapSettings.snapAngle",
            "snapSettings.snapObjectDistance",
        ],

        enableContextMenu: ej.util.valueFunction("enableContextMenu"),
        enableAutoScroll: ej.util.valueFunction("enableAutoScroll"),
        showTooltip: ej.util.valueFunction("showTooltip"),
        zoomFactor: ej.util.valueFunction("zoomFactor"),
        _pageHeight: ej.util.valueFunction("pageSettings.pageHeight"),
        _pageWidth: ej.util.valueFunction("pageSettings.pageWidth"),
        _multiplePage: ej.util.valueFunction("pageSettings.multiplePage"),
        _pageBorderWidth: ej.util.valueFunction("pageSettings.pageBorderWidth"),
        _pageBackgroundColor: ej.util.valueFunction("pageSettings.pageBackgroundColor"),
        _pageBorderColor: ej.util.valueFunction("pageSettings.pageBorderColor"),
        _pageMargin: ej.util.valueFunction("pageSettings.pageMargin"),
        _showPageBreak: ej.util.valueFunction("pageSettings.showPageBreak"),
        _pageOrientation: ej.util.valueFunction("pageSettings.pageOrientation"),
		_scrollLimit:ej.util.valueFunction("pageSettings.scrollLimit"),
        _enableSnapToObject: ej.util.valueFunction("snapSettings.enableSnapToObject"),
        _snapAngle: ej.util.valueFunction("snapSettings.snapAngle"),
        _snapObjectDistance: ej.util.valueFunction("snapSettings.snapObjectDistance"),
        dataTypes: {
            nodes: "array",
            connectors: "array",
            enableAutoScroll: "boolean",
            autoScrollMargin: "number",
            userHandles: "array",
            contextMenu: {
                items: "array",
                showCustomMenuItemsOnly: "boolean"
            },
            layout: {
                type: "enum",
                orientation: "enum",
                horizontalSpacing: "number",
                verticalSpacing: "number",
                marginX: "number",
                marginY: "number"
            },
        },

        //#endregion

        //#region Local members
        _canvas: null,
        _svg: null,
        _toolToActivate: null,
        _inAction: false,
        _isPinching: false,
        _isEditing: false,
        _nodeToHit: null,
        _currentCursor: null,
        _pasteIndex: null,
        _page: null,
        _view: null,
        _currentLabel: null,
        _zOrder: 0,
        _clipboardData: null,
        _currZoom: 1,
        _historyList: {
            currentEntry: {
                next: null,
                previous: null
            },
            canUndo: false,
            canRedo: false
        },

        _defaultContextMenuItems: [
            { name: "cut", text: "Cut" },
            { name: "copy", text: "Copy" },
            { name: "paste", text: "Paste" },
            { name: "undo", text: "Undo" },
            { name: "redo", text: "Redo" },
            { name: "selectAll", text: "Select All" },
            {
                name: "grouping",
                text: "Grouping",
                subItems: [
                            { name: "group", text: "Group" },
                            { name: "ungroup", text: "Ungroup" }
                ]
            },
            {
                name: "order",
                text: "Order",
                subItems: [
                            { name: "bringToFront", text: "Bring To Front" },
                            { name: "moveForward", text: "Move Forward" },
                            { name: "sendToBack", text: "Send To Back" },
                            { name: "sendBackward", text: "Send Backward" }
                ]
            }
        ],

        _hScrollOffset: 0,
        _vScrollOffset: 0,
        _hScrollbar: null,
        _vScrollbar: null,
        _scrollPixel: 30,
        //#endregion

        //#region public members
        activeTool: null,
        selectionList: [],
        tools: null,
        nameTable: {},
        //#endregion

        //#region Initialization
        _updateScrollOffset: function (hScrollOffset, vScrollOffset, canScale) {
            this._hScrollOffset = hScrollOffset;
            this._vScrollOffset = vScrollOffset;
            ej.datavisualization.Diagram.ScrollUtil._transform(this, hScrollOffset, vScrollOffset, canScale);
            var diagram = this;
            this._views.forEach(function (viewid) {
                var view = diagram._views[viewid];
                if (view.type == "overview") {
                    var ovw = $("#" + viewid).ejOverview("instance");
                    if (ovw)
                        ovw._scrollOverviewRect(hScrollOffset, vScrollOffset, diagram._currZoom);
                }
            });
        },

        _init: function () {
            this._spatialSearch = ej.datavisualization.Diagram.SpatialSearch();
            this._initDefaults();
            this._initViews();
            this._initData();
            this._doLayout();
            this._initCanvas();
            this._initVisualGuide();
            this._wireEvents();
            this._initHandles();
            this._initContextMenu();
            this._drawingTool = false;
            this._isDropped = false;
        },

        _setModel: function (options) {
            for (var option in options) {
                switch (option) {
                    case "nodes":
                        this.model.nodes = options[option];
                        this._initNodeCollection(true);
                        this.element.remove(".ej-d-node");
                        this.element.remove(".ej-d-group");
                        var diagram = this;
                        this._views.forEach(function (viewid) {
                            var view = diagram._views[viewid];
                            diagram._renderNodes(view, true);
                        });
                        break;
                    case "connectors":
                        this.model.connectors = options[option];
                        this._initConnectorCollection();
                        this.element.remove(".ej-d-connector");
                        var diagram = this;
                        this._views.forEach(function (viewid) {
                            var view = diagram._views[viewid];
                            diagram._renderConnectors(view);
                        });
                        break;
                    case "enableContextMenu":
                    case "contextMenu":
                        this.model.contextMenu = options[option];
                        this._initContextMenu();
                        break;
                    case "snapSettings":
                        var object = options[option];
                        if (object && Object.keys(object).length > 0) {
                            this._updateSnapSettings(options[option]);
                            ej.datavisualization.Diagram.SvgContext._renderGrid(this._canvas, this._svg, this._page, this);
                        }
                        break;
                    case "selectorConstraints":
                        this._setSelectorConstraints(options[option]);
                        break;
                    case "constraints":
                        if (this._hasSelection())
                            this._clearSelection();
                        break;
                    case "userHandles":
                        this.model.userHandles = options[option];
                        this._initHandles();
                        break;
                    case "drawingTools":
                        this.model.drawingTools = options[option];
                        this._registerDrawingTools();
                        break;
                    case "backgroundImage":
                        this.model.backgroundImage = options[option];
                        ej.datavisualization.Diagram.SvgContext._renderBackground(this._canvas, this._svg, this._page, this.model);
                        break;
                    case "enableAutoScroll":
                        this.enableAutoScroll(options[option]);
                        break;
                    case "autoScrollMargin":
                        this.model.autoScrollMargin = options[option];
                        break;
                    case "layout":
                        this.model.layout = $.extend(true, {}, this.model.layout, options[option]);
                        this._setLayout(options[option]);
                        break;
                    case "pageSettings":
                        this.updatePageSettings(options[option]);
                        break;
                    case "locale":
                        this.model.locale = options[option];
                        var contextmenu = document.getElementById(this.element[0].id + "_contextMenu");
                        contextmenu.parentNode.removeChild(contextmenu);
                        this._renderContextMenu();
                        break;
                    case "nodeTemplate":
                        this.clear();
                        this._initData();
                        var diagram = this;
                        this._views.forEach(function (viewid) {
                            var view = diagram._views[viewid];
                            diagram._renderNodes(view);
                            diagram._renderConnectors(view);
                        });
                        this.layout();
                        break;
                    case "connectorTemplate":
                        this._updateConnectors();
                        break;
                    case "dataSourceSettings":
                        var object = options[option];
                        if (object && Object.keys(object).length > 0) {
                            this.clear();
                            $.extend(true, this.model.dataSourceSettings, {}, options[option]);
                            this._initData();
                            var diagram = this;
                            this._views.forEach(function (viewid) {
                                var view = diagram._views[viewid];
                                diagram._renderNodes(view);
                                diagram._renderConnectors(view);
                            });
                            this.layout();
                        }
                        break;
                }
            }
        },

        _initData: function () {
            if (this.model.dataSourceSettings)
                var dataSource = this.model.dataSourceSettings.dataSource || this.model.dataSourceSettings.nodes || this.model.dataSourceSettings.connectors;
            if (dataSource) {
                var nodes = [];
                var connectors = [];
                var dataSourceApplied;
                if (this.model.dataSourceSettings.nodes || this.model.dataSourceSettings.connectors) {
                    if (this.model.dataSourceSettings.nodes) {
                        dataSource = this.model.dataSourceSettings.nodes.dataSource || this.model.dataSourceSettings.dataSource;
                        if (dataSource) {
                            if (dataSource instanceof ej.DataManager) {
                                $.ajaxSetup({
                                    async: false
                                });
                                var query = this._findQuery(this.model.dataSourceSettings.nodes, this.model.dataSourceSettings.nodeBinding);
                                var queryPromise = dataSource.executeQuery(eval(query));
                                var diagram = this;
                                queryPromise.done(function (e) {
                                    diagram.retriveData = e.result;
                                    diagram._applyTemplate(diagram.model.dataSourceSettings, e.result, nodes, "nodes");
                                });
                            } else
                                this._applyTemplate(this.model.dataSourceSettings, dataSource, nodes, "nodes");
                        }
                        if (this.model.dataSourceSettings.connectors) {
                            dataSource = this.model.dataSourceSettings.connectors.dataSource || this.model.dataSourceSettings.dataSource;
                            if (dataSource) {
                                if (dataSource instanceof ej.DataManager) {
                                    $.ajaxSetup({
                                        async: false
                                    });
                                    query = this._findQuery(this.model.dataSourceSettings.connectors, this.model.dataSourceSettings.connectorBinding);
                                    queryPromise = dataSource.executeQuery(eval(query));
                                    queryPromise.done(function (e) {
                                        diagram.retriveData = e.result;
                                        diagram._applyTemplate(diagram.model.dataSourceSettings, e.result, connectors, "connectors");
                                    });
                                } else {
                                    this._applyTemplate(this.model.dataSourceSettings, dataSource, connectors, "connectors");
                                }
                            }
                        }
                    }
                } else if (dataSource instanceof ej.DataManager)
                    this._initDataSource(dataSource, nodes, connectors);
                else if (dataSource.length) {
                    this._applyDataSource(this.model.dataSourceSettings, dataSource, nodes, connectors);
                    dataSourceApplied = true;
                }
                this.model.nodes = nodes;
                this.model.connectors = connectors;
            }
            var collapsedNodes = this._initNodeCollection(false, dataSourceApplied);
            this._initConnectorCollection(dataSourceApplied);
            if (collapsedNodes.length > 0) {
                this._collapseNodes(collapsedNodes);
            }
        },

        _initDataSource: function (dataSource, nodes, connectors) {
            var mapper = this.model.dataSourceSettings;
            if (dataSource instanceof ej.DataManager) {
                $.ajaxSetup({
                    async: false
                });
                var query = this._findQuery(mapper, this.model.dataSourceSettings.nodeBinding);
                var queryPromise = this.model.dataSourceSettings["dataSource"].executeQuery(eval(query));
                var diagram = this;
                queryPromise.done(function (e) {
                    diagram.retriveData = e.result;
                    diagram._applyDataSource(diagram.model.dataSourceSettings, e.result, nodes, connectors);
                });
            }
        },

        _findQuery: function (mapper, binding) {
            var column = [], queryManager = ej.Query();
            var query = mapper.query || this.model.dataSourceSettings.query;

            if (ej.isNullOrUndefined(query)) {
                var dataSource = mapper.dataSource || this.model.dataSourceSettings.dataSource;
                var tableName = mapper.tableName || this.model.dataSourceSettings.tableName;;
                for (var col in mapper) {
                    if (col !== "tableName" && col !== "dataSource" && col !== "designTemplate" && !col.match("Binding"))
                        if (mapper[col]) column.push(mapper[col]);
                }
                for (var tempcol in binding) {
                    if (tempcol === "labels") {
                        var labels = binding[tempcol];
                        for (var i = 0; i < labels.length; i++) {
                            if (labels[i].text) column.push(labels[i].text);
                        }
                    } else if (tempcol == "shape" || tempcol == "line") {
                        if (binding[tempcol][type]) col.push(binding[tempcol][type]);
                    } else
                        if (binding[tempcol]) column.push(binding[tempcol]);
                }

                if (column.length > 0)
                    queryManager.select(column);
                if (!dataSource["dataSource"].url.match(tableName + "$"))
                    !ej.isNullOrUndefined(tableName) && queryManager.from(tableName);

            } else
                queryManager = query;
            return queryManager;
        },

        _applyDataSource: function (mapper, data, nodes, connectors, ancestor) {
            var rootNodes = ej.DataManager(data).executeLocal(ej.Query().group(mapper.parent));
            if (mapper.parent) {
                var firstLvel = [];
                for (var i = 0; i < rootNodes.length; i++) {
                    if (rootNodes[i].key == undefined || rootNodes[i].key == "\"\"") {
                        firstLvel.push(rootNodes[i]);
                    }
                }
            }
            else firstLvel = rootNodes;
            var nodeBinding = this.model.dataSourceSettings.nodeBinding || {};
            var nodeDefaults = this.model.defaultSettings.node || {};
            var connectorDefaults = this.model.defaultSettings.connector || {};
            var connectorBinding = this.model.dataSourceSettings.connectorBinding || {};
            for (var i = 0; i < firstLvel.length; i++) {
                for (var j = 0; j < firstLvel[i].items.length; j++) {
                    var item = firstLvel[i].items[j];
                    var node = this._applyNodeTemplate(mapper, item, nodeBinding, nodeDefaults);
                    nodes.push(node);
                    var nextLevel = ej.DataManager(rootNodes).executeLocal(ej.Query().where("key", ej.FilterOperators.equal, item[mapper.id]));
                    if (nextLevel[0] && nextLevel[0].items)
                        this._renderChildNodes(mapper, nextLevel[0], node.name, nodes, connectors, rootNodes);
                }
            }
        },

        _renderChildNodes: function (mapper, parent, ancestor, nodes, connectors, rootNodes) {
            var nodeBinding = this.model.dataSourceSettings.nodeBinding || {};
            var nodeDefaults = this.model.defaultSettings.node || {};
            var connectorDefaults = this.model.defaultSettings.connector || {};
            var connectorBinding = this.model.dataSourceSettings.connectorBinding || {};
            for (var j = 0; j < parent.items.length; j++) {
                var child = parent.items[j];
                var node = this._applyNodeTemplate(mapper, child, nodeBinding, nodeDefaults);
                nodes.push(node);
                connectors.push(this._applyConnectorTemplate(mapper, null, connectorBinding, connectorDefaults, ancestor, node.name));
                var nextLevel = ej.DataManager(rootNodes).executeLocal(ej.Query().where("key", ej.FilterOperators.equal, child[mapper.id]));
                if (nextLevel[0] && nextLevel[0].items)
                    this._renderChildNodes(mapper, nextLevel[0], node.name, nodes, connectors, rootNodes);
            }
        },

        _applyTemplate: function (mapper, data, collection, collname) {
            if (collname == "nodes") {
                var template = this.model.dataSourceSettings.nodeBinding || {};
                var defaults = this.model.defaultSettings.node || {};
                for (var i = 0; i < data.length; i++) {
                    var node = this._applyNodeTemplate(mapper, data[i], template, defaults);
                    collection.push(node);
                }
            }
            if (collname == "connectors") {
                var template = this.model.dataSourceSettings.connectorBinding || {};
                var defaults = this.model.defaultSettings.connector || {};
                for (var i = 0; i < data.length; i++) {
                    var conn = this._applyConnectorTemplate(mapper, data[i], template, defaults);
                    collection.push(conn);
                }
            }
        },

        _applyNodeTemplate: function (mapper, item, nodeBinding, nodeDefaults) {
            var root = jQuery.extend(true, {}, nodeDefaults);
            root.name = ej.datavisualization.Diagram.Util.randomId();
            if (nodeBinding) {
                for (var prop in nodeBinding) {
                    if (prop != "labels")
                        root[prop] = item[nodeBinding[prop]];
                }
                if (nodeBinding.labels) {
                    if (!root.labels) {
                        root.labels = [];
                    }
                    for (var i = 0; i < nodeBinding.labels.length; i++) {
                        var label = nodeDefaults && nodeDefaults.labels && nodeDefaults.labels.length ? nodeDefaults.labels[i % nodeDefaults.labels.length] : {};
                        label.name = ej.datavisualization.Diagram.Util.randomId();
                        label.text = item[nodeBinding.labels[i].text];
                        root.labels[i] = label;
                    }
                }
                if (nodeBinding.shape) {
                    var shape = nodeDefaults && nodeDefaults.shape ? nodeDefaults.shape : {};
                    shape.type = item[nodeBinding.shape.type];
                    root.shape = shape;
                }
            }
            root = ej.datavisualization.Diagram.Node(root);
            if (this.model.nodeTemplate) {
                this.model.nodeTemplate(this, root, item);
            }
            return root;
        },

        _applyConnectorTemplate: function (mapper, item, connectorBinding, connectorDefaults, sourceNode, targetNode) {
            var connector = jQuery.extend(true, {}, connectorDefaults);
            connector.name = ej.datavisualization.Diagram.Util.randomId();
            connector.sourceNode = sourceNode;
            connector.targetNode = targetNode;
            if (connectorBinding) {
                for (var prop in connectorBinding) {
                    if (prop != "labels")
                        connector[prop] = item[connectorBinding[prop]];
                }
                if (connectorBinding.labels) {
                    if (!connector.labels) {
                        connector.labels = [];
                    }
                    for (var i = 0; i < connectorBinding.labels.length; i++) {
                        var label = connectorDefaults && connectorDefaults.labels && connectorDefaults.labels.length ? connectorDefaults.labels[i % connectorDefaults.labels.length] : {};
                        label.name = ej.datavisualization.Diagram.Util.randomId();
                        label.text = item[connectorBinding.labels[i].text];
                        connector.labels[i] = label;
                    }
                }
                //if (connectorBinding.segments) {
                //    var line = {};
                //    line.type = item[connectorBinding.line.type];
                //    connector.line = line;
                //}
            }
            var connector = ej.datavisualization.Diagram.Connector(connector);
            if (this.model.connectorTemplate) {
                this.model.connectorTemplate(this, connector, item);
            }
            return connector;
        },

        _collapseNodes: function (nodes) {
            for (var i = 0; i < nodes.length; i++) {
                var node = nodes[i];
                this._collapseChildren(node);
            }
        },

        _initViews: function () {
            this._views = [];
            this._views.push(this._id);
            this._views[this._id] = {
                "type": "mainview",
                "context": ej.datavisualization.Diagram.SvgContext,
                "style": "position:relative; height:" + this.model.height + "; width:" + this.model.width + ";"
            };
        },
        _setOverview: function (overview, id) {
            if (overview) {
                this._overview = overview;
                if (!this._views[overview._id]) {
                    this._views.push(overview._id);
                    this._views[overview._id] = { "context": ej.datavisualization.Diagram.SvgContext, "type": "overview" };
                    overview._renderDocument(this._views[overview._id]);
                    this._renderNodes(this._views[overview._id], null, true);
                    this._renderConnectors(this._views[overview._id]);
                    overview._updateOverview(this._views[overview._id]);
                }
            } else {
                var overview = $("#" + id).ejOverview("instance");
                overview._removeDocument(this._views[id]);
                this._views[id] = undefined;
                var index = this._views.indexOf(id);
                this._views.splice(index, 1);
                //this._overview = null;
            }

        },

        _initNodeCollection: function (isLoad, dataSourceApplied) {
            var collapsedNodes = [];
            var nodes = this.model.nodes;
            for (var i = 0; i < nodes.length; i++) {
                if (nodes[i].type == "swimlane" || nodes[i].isSwimlane) {
                    if (isLoad)
                        nodes[i] = nodes[i];
                    else
                        nodes[i] = ej.datavisualization.Diagram.SwimLaneHelper._initSwimLane(nodes[i], this);
                    this.nameTable[nodes[i].name] = nodes[i];
                }
                else if (nodes[i].isGroup) {
                    nodes[i] = ej.datavisualization.Diagram.Group(nodes[i]);
                    this.nameTable[nodes[i].name] = nodes[i];
                }
                else {
                    if (!dataSourceApplied)
                        nodes[i] = this._getNewNode(nodes[i]); //ej.datavisualization.Diagram.Node(nodes[i]);
                    this.nameTable[nodes[i].name] = nodes[i];
                    ej.datavisualization.Diagram.SpatialUtil._updateQuad(this, this._spatialSearch, nodes[i]);
                }
                this.nameTable[nodes[i].name] = nodes[i];
                if (!nodes[i].isExpanded) {
                    collapsedNodes.push(nodes[i]);
                }
            }
            return collapsedNodes;
        },

        _getNewNode: function (options) {
            return ej.datavisualization.Diagram.Node($.extend(true, {}, this.model.defaultSettings.node, options));
        },

        _getNewConnector: function (options) {
            var obj = $.extend(true, {}, this.model.defaultSettings.connector, options);
            if (this.model.defaultSettings.connector && (this.model.defaultSettings.connector.segments && this.model.defaultSettings.connector.segments[0])) {
                obj.defaultType = this.model.defaultSettings.connector.segments[0].type;
            }
            return ej.datavisualization.Diagram.Connector(obj);
        },
        _getTightChildrenBounds: function (node) {
            var children = node.children, rect, bounds = null, child;
            for (var i = 0, len = children.length; i < len; i++) {
                child = this.nameTable[children[i]];
                if (child) {
                    var rect1 = ej.datavisualization.Diagram.Util.bounds(child);
                    rect = ej.datavisualization.Diagram.Geometry.rect([rect1.topLeft, rect1.topRight, rect1.bottomRight, rect1.bottomLeft]);
                    if (child.verticalAlign == "stretch") {
                        rect.height = 0;
                    } else if (child.horizontalAlign == "stretch") {
                        rect.width = 0;
                    }
                }
                if (bounds)
                    bounds = this._union(rect, bounds);
                else
                    bounds = rect;
            }
            return bounds;
        },
        _getBoundsWithoutStretch: function (node, vertical, horizontal) {
            var children = node.children, rect, rect1, bounds = null, child;
            for (var i = 0, len = children.length; i < len; i++) {
                child = this.nameTable[children[i]];
                rect1 = ej.datavisualization.Diagram.Util.bounds(child);
                rect = ej.datavisualization.Diagram.Geometry.rect([rect1.topLeft, rect1.topRight, rect1.bottomRight, rect1.bottomLeft]);
                if (vertical) {
                    if (child.verticalAlign == "stretch" && !(child.verticalAlign == "stretch" && child.horizontalAlign == "stretch")) {
                        rect.height = 0;
                    }
                } else if (horizontal) {
                    if (child.horizontalAlign == "stretch" && !(child.verticalAlign == "stretch" && child.horizontalAlign == "stretch")) {
                        rect.width = 0;
                    }
                }

                if (bounds)
                    bounds = this._union(rect, bounds);
                else
                    bounds = rect;
            }

            bounds.width += node.paddingLeft + node.paddingRight;
            bounds.height += node.paddingTop + node.paddingBottom;
            return bounds;
        },
        _getMinMaxSize: function (node, width, height, stretch) {
            var nw = null, nh = null;
            if (width >= node.minWidth && width <= node.maxWidth) {
                nw = width;
            } else {
                if (node.horizontalAlign == "stretch" && node.parent && !stretch) {
                    var group = this.nameTable[node.parent];
                    if (group) {
                        var rSize = ej.datavisualization.Diagram.Util.bounds(group);// ej.datavisualization.Diagram.SvgContext._measureCanvasSize(group, this);
                        if (rSize.width == 0 && rSize.height == 0) {
                            rSize.width = width;
                            rSize.height = height;
                        }
                        var newSize = this._getMinMaxSize(group, rSize.width, rSize.height);
                        newSize.width -= (group.paddingLeft + group.paddingRight + group.marginLeft + group.marginRight);
                        newSize.height -= (group.paddingTop + group.paddingBottom + group.marginTop + group.marginBottom);
                        nw = newSize.width ? newSize.width : width;
                    }
                } else if (width < node.minWidth && node.minWidth != 0)
                    nw = node.minWidth;
                else if (width > node.maxWidth && node.maxWidth != 0)
                    nw = node.maxWidth;
                else
                    nw = width;
            }

            if (height >= node.minHeight && height <= node.maxHeight) {
                nh = height;
            } else {
                if (node.verticalAlign == "stretch" && node.parent && !stretch) {
                    var group = this.nameTable[node.parent];
                    var rSize = ej.datavisualization.Diagram.Util.bounds(group);
                    if (rSize.width == 0 && rSize.height == 0) {
                        //rSize.width = width;
                        //rSize.height = height;
                        var status = null;
                        var state = false;
                        rSize.width = node.minWidth;
                        rSize.height = node.minHeight;
                        if (node.minWidth == 0 && node.minHeight == 0) {
                            rSize.width = width;
                            rSize.height = height;
                            status = false;
                            state = true;
                        }
                        if (!state)
                            status = true;
                    }
                    var newSize = this._getMinMaxSize(group, rSize.width, rSize.height);
                    newSize.width -= (group.paddingLeft + group.paddingRight + group.marginLeft + group.marginRight);
                    newSize.height -= (group.paddingTop + group.paddingBottom + group.marginTop + group.marginBottom);
                    if (status) {
                        newSize.height -= node.marginTop;

                    }
                    nh = newSize.height ? newSize.height : height;
                } else if (height < node.minHeight && node.minHeight != 0)
                    nh = node.minHeight;
                else if (height > node.maxHeight && node.maxHeight != 0)
                    nh = node.maxHeight;
                else
                    nh = height;
            }
            return { width: nw, height: nh };
        },
        _initConnectorCollection: function (dataSourceApplied) {
            var connectors = this.model.connectors;
            var connector;
            for (var i = 0, len = connectors.length; i < len; i++) {
                connector = dataSourceApplied ? connectors[i] : this._getNewConnector(connectors[i]);
                this.nameTable[connector.name] = connector;
                ej.datavisualization.Diagram.SpatialUtil._updateQuad(this, this._spatialSearch, connector);
                var segment = connectors[i].segments || (this.model.defaultSettings.connector && this.model.defaultSettings.connector.segments);
                if (segment)
                    segment = $.extend(true, {}, { value: segment });
                var update = false;
                var port, bounds;
                if (connector.sourceNode && connector.sourcePort) {
                    bounds = ej.datavisualization.Diagram.Util.bounds(this.nameTable[connector.sourceNode], true);
                    port = ej.datavisualization.Diagram.Util.findPortByName(this.nameTable[connector.sourceNode], connector.sourcePort);
                    connector.sourcePoint = ej.datavisualization.Diagram.Util._getPortPosition(port, bounds);
                    update = true;
                }
                if (connector.targetNode && connector.targetPort) {
                    bounds = ej.datavisualization.Diagram.Util.bounds(this.nameTable[connector.targetNode], true);
                    port = ej.datavisualization.Diagram.Util.findPortByName(this.nameTable[connector.targetNode], connector.targetPort);
                    connector.targetPoint = ej.datavisualization.Diagram.Util._getPortPosition(port, bounds);
                    update = true;
                }
                if (update)
                    ej.datavisualization.Diagram.Util._updateConnectorSegments(connector, null, null, this);
                this._dock(connector, this.nameTable);
                if (segment && connector.segments.length > 1 && !connector.sourcePort) {
                    if (this.model.defaultSettings.connector && (this.model.defaultSettings.connector.segments && this.model.defaultSettings.connector.segments[0])) {
                        var defaultType = this.model.defaultSettings.connector.segments[0].type;
                    }
                    if (segment)
                        connector.segments = segment.value;
                    ej.datavisualization.Diagram.Util._initializeSegments(connector, defaultType);
                    ej.datavisualization.Diagram.Util.updateBridging(connector, this);
                }
                this._updateEdges(connector);
                connectors[i] = connector;
            }
        },

        _initDefaults: function () {
            this._initTools();
        },

        _initCanvas: function () {
            this._renderCanvas();
        },

        _updateConnectorBridging: function (baseConnector) {
            var childBounds = ej.datavisualization.Diagram.Util.bounds(baseConnector);
            var quads = ej.datavisualization.Diagram.SpatialUtil.findQuads(this._spatialSearch, childBounds);
            for (var item = 0; item < quads.length; item++) {
                var quad = quads[item];
                if (quad.objects.length > 0) {
                    for (var q = 0; q < quad.objects.length; q++) {
                        var connector = quad.objects[q];
                        if (connector.segments) {
                            if (connector != baseConnector) {
                                ej.datavisualization.Diagram.Util.updateBridging(connector, this);
                                ej.datavisualization.Diagram.DiagramContext.update(connector, this);
                            }
                        }
                    }
                }
            }
        },

        _initVisualGuide: function () {
            if (this.showTooltip() && !this.model.tooltipTemplateId) {
                var script = document.createElement("script");
                var attr = {
                    "id": this.element[0].id + "_defaultTooltip",
                    "type": "text/x-jsrender"
                };
                ej.datavisualization.Diagram.Util.attr(script, attr);
                script.innerHTML = "<span class=\"e-diagram-tooltipLabel-default\"> {{:type}}{{:x}}{{:type1}}{{:y}}</span>";
                this._canvas.appendChild(script);
            } else {
                this.element.remove("#" + this.element[0].id + "_defaultTooltip");
            }
        },

        _initTools: function () {
            this.tools = {
                "select": new ej.datavisualization.Diagram.SelectTool(this),
                "move": new ej.datavisualization.Diagram.MoveTool(this),
                "resize": new ej.datavisualization.Diagram.ResizeTool(this),
                "rotate": new ej.datavisualization.Diagram.RotateTool(this),
                //"pivot": new ej.datavisualization.Diagram.PivotTool(this),
                "endPoint": new ej.datavisualization.Diagram.ConnectionEditTool(this),
                "panTool": new ej.datavisualization.Diagram.PanTool(this),
                "straightLine": new ej.datavisualization.Diagram.StraightLineTool(this),
                "orthogonalLine": new ej.datavisualization.Diagram.OrthogonalLineTool(this),
                "phase": new ej.datavisualization.Diagram.PhaseTool(this)
            };
            this._toolToActivate = "select";
            this.activeTool = this.tools[this._toolToActivate];
            this._registerDrawingTools();
        },

        _initHandles: function () {
            var userHandles = this.model.userHandles;
            if (userHandles) {
                for (var i = 0; i < userHandles.length; i++) {
                    userHandles[i] = ej.datavisualization.Diagram.UserHandle(userHandles[i]);
                    if (userHandles[i].tool) {
                        userHandles[i].tool.diagram = this;
                        this.tools[userHandles[i].name] = userHandles[i].tool;
                    }
                }
            }
        },

        _destroy: function () {
            this.element.empty().removeClass("e-datavisualization-diagram");
        },
        //#endregion       

        updatePageSettings: function (option) {
            if (option) {
                if (option.pageHeight) {
                    this._pageHeight(Number(typeof option.pageHeight === 'function' ? option.pageHeight() : option.pageHeight));
                }
                if (option.pageWidth) {
                    this._pageWidth(Number(typeof option.pageWidth === 'function' ? option.pageWidth() : option.pageWidth));
                }
                if (option.pageBorderWidth) {
                    this._pageBorderWidth(Number(typeof option.pageBorderWidth === 'function' ? option.pageBorderWidth() : option.pageBorderWidth));
                }
                if (option.pageMargin) {
                    this._pageMargin(Number(typeof option.pageMargin === 'function' ? option.pageMargin() : option.pageMargin));
                }
                if (option.pageBackgroundColor) {
                    this._pageBackgroundColor(typeof option.pageBackgroundColor === 'function' ? option.pageBackgroundColor() : option.pageBackgroundColor);
                }
                if (option.scrollLimit) {
                    this._scrollLimit(typeof option.scrollLimit === 'function' ? option.scrollLimit() : option.scrollLimit);
                }
                if (option.scrollableArea) {
                    this.model.pageSettings.scrollableArea = option.scrollableArea;
                }

                ej.datavisualization.Diagram.PageUtil._updatePageSize(this);
                ej.datavisualization.Diagram.SvgContext._updateBackground(this._hScrollOffset, this._vScrollOffset, this._currZoom, this);
                ej.datavisualization.Diagram.SvgContext._updateGrid(this._hScrollOffset, this._vScrollOffset, this._currZoom, this);
            }
        },

        _updateSnapSettings: function (option) {
            if (option) {
                if (option.snapAngle) {
                    this._snapAngle(Number(typeof option.snapAngle === 'function' ? option.snapAngle() : option.snapAngle));
                }
                if (option.snapObjectDistance) {
                    this._snapObjectDistance(Number(typeof option.snapObjectDistance === 'function' ? option.snapObjectDistance() : option.snapObjectDistance));
                }
            }
        },

        /**
     *update Node with their specific properties
     *@return jQuery
     *@param {String} name  to identify the Node from the model
     *@param {String} options to specify which property of the Node to be update
     *@example
     *&lt;div id="diagramcontent"&gt;&lt;/div&gt;
     *&lt;script&gt;
     *var diagram=$("#diagramcontent").ejDiagram("instance");
     *var value = ej.datavisualization.Diagram.NodeConstraints.Default & ~ej.datavisualization.Diagram.NodeConstraints.Connect;
     *diagram.updateNode(node.name, { constraints: value });
     *&lt;/script&gt;
     *@memberof ejDiagram
     *@instance
     */
        //#region Public methods
        updateNode: function (name, options) {
            if (name) {
                var node = this._findNode(name);
                if (node) {
                    if (options.offsetX) {
                        this._translate(node, options.offsetX - node.offsetX, 0, this.nameTable);
                    }
                    if (options.offsetY) {
                        this._translate(node, 0, options.offsetY - node.offsetY, this.nameTable);
                    }
                    if (options.rotateAngle) {
                        this._rotate(node, options.rotateAngle - node.rotateAngle, this.nameTable);
                    }
                    if (options.width) {
                        node.width = node.width ? node.width : 1;
                        this.scale(node, options.width / node.width, 1, new ej.datavisualization.Diagram.Point(node.offsetX, node.offsetY), this.nameTable);
                    }
                    if (options.height) {
                        node.height = node.height ? node.height : 1;
                        this.scale(node, 1, options.height / node.height, new ej.datavisualization.Diagram.Point(node.offsetX, node.offsetY), this.nameTable);
                    }
                    if (options.connectionPadding != undefined) {
                        node.connectionPadding = options.connectionPadding;
                        this.scale(node, 1, 1, new ej.datavisualization.Diagram.Point(node.offsetX, node.offsetY), this.nameTable);
                    }
                    if (options.constraints) {
                        if (!node.isGroup) {
                            if (!(node.constraints & ej.datavisualization.Diagram.NodeConstraints.Shadow)) {
                                node.constraints = options.constraints;
                                ej.datavisualization.Diagram.DiagramContext.renderShadow(node, this);
                            }
                            else {
                                node.constraints = options.constraints;
                                ej.datavisualization.Diagram.DiagramContext.removeShadow(node, this);
                            }
                        }
                        node.constraints = options.constraints;
                        if (this.selectionList[0] && this.selectionList[0].name === node.name) {
                            ej.datavisualization.Diagram.SvgContext.clearSelector(this._svg, this._adornerLayer);
                            ej.datavisualization.Diagram.SvgContext.renderSelector(this.selectionList[0], this._svg, this._adornerLayer, this._currZoom, this.model.selectorConstraints);
                            if (this.model.selectorConstraints & ej.datavisualization.Diagram.SelectorConstraints.UserHandles)
                                ej.datavisualization.Diagram.SvgContext.renderUserHandles(this.model.userHandles, this.selectionList[0], this._svg, this.selectionList[0].isPseudoGroup,
                                    this._currZoom, this._adornerLayer);
                        }
                    }
                    if (!node.isGroup) {
                        if (options.fillColor) {
                            node.fillColor = options.fillColor;
                        }
                        if (options.opacity) {
                            node.opacity = options.opacity;
                        }
                        if (options.borderColor) {
                            node.borderColor = options.borderColor;
                        }
                        if (options.borderWidth) {
                            node.borderWidth = options.borderWidth;
                        }
                        if (options.borderDashArray) {
                            node.borderDashArray = options.borderDashArray;
                        }
                        if (options.gradient !== undefined)
                            node.gradient = options.gradient;
                    }
                    if (options.shape) {
                        if (ej.browserInfo().name === "msie") {
                            if (node.shape && node.shape.html) {
                                var htmlelement = document.getElementById(node.name + "_parentdiv");
                                htmlelement.parentNode.removeChild(htmlelement);
                            }
                        }
                        node.shape = ej.datavisualization.Diagram.Shape(options.shape);
                        ej.datavisualization.Diagram.DiagramContext.setNodeShape(node, this);
                    }
                    ej.datavisualization.Diagram.DiagramContext.update(node, this);
                    ej.datavisualization.Diagram.DiagramContext.updateNodeStyle(node, this);
                    if (this.selectionList[0] && this.selectionList[0].name === node.name)
                        ej.datavisualization.Diagram.SvgContext.updateSelector(node, this._svg, this._currZoom, this, this.model.selectorConstraints);

                    if (options.isExpanded !== undefined) {
                        //if (this.model.layout.type == "hierarchicaltree")
                        {
                            if (options.isExpanded) {
                                node.isExpanded = true;
                                this._expandChildren(node);
                            }
                            else {
                                node.isExpanded = false;
                                this._collapseChildren(node);
                            }
                            this.layout();
                        }
                    }
                }
            }
        },

        /**
       *update Connector with their specific properties
       *@return jQuery
       *@param {String} name  to identify the connector from the model
       *@param {object} options to specify which property of the Connector to be update
       *@example
       *&lt;div id="diagramcontent"&gt;&lt;/div&gt;
       *&lt;script&gt;
       *var diagram=$("#diagramcontent").ejDiagram("instance");
       *var value = ej.datavisualization.Diagram.ConnectorConstraints.Default;
       *diagram.updateConnector(connector.name, { constraints: value });      
       *&lt;/script&gt;
       *@memberof ejDiagram
       *@instance
       */
        updateConnector: function (name, options) {
            if (name) {
                var connector = this._findConnector(name);
                if (connector) {
                    if (options.constraints) {
                        connector.constraints = options.constraints;
                    }
                    if (options.lineColor) {
                        connector.lineColor = options.lineColor;
                    }
                    if (options.lineWidth) {
                        connector.lineWidth = options.lineWidth;
                    }
                    if (options.lineDashArray) {
                        connector.lineDashArray = options.lineDashArray;
                    }
                    if (options.sourceIndent != undefined) {
                        connector.sourceIndent = options.sourceIndent;
                        this._dock(connector, this.nameTable);
                    }
                    if (options.targetIndent != undefined) {
                        connector.targetIndent = options.targetIndent;
                        this._dock(connector, this.nameTable);
                    }
                    if (options.segments) {
                        connector.segments = options.segments;
                        if (this.model.defaultSettings.connector && this.model.defaultSettings.connector.segments[0]) {
                            var defaultType = this.model.defaultSettings.connector.segments[0].type;
                        }
                        ej.datavisualization.Diagram.Util._initializeSegments(connector, defaultType);
                        this._dock(connector, this.nameTable);
                        ej.datavisualization.Diagram.Util.updateBridging(connector, this);
                        this._updateQuad(this, connector);
                        if (connector == this.selectionList[0]) {
                            this._clearSelection();
                            this._addSelection(connector);
                        }
                    }

                    var decorator;
                    if (options.targetDecorator) {
                        if (connector.targetDecorator) {
                            decorator = $.extend(true, {}, connector.targetDecorator, options.targetDecorator);
                        } else {
                            decorator = ej.datavisualization.Diagram.Decorator(options.targetDecorator);
                        }
                        connector.targetDecorator = decorator;
                    }
                    if (options.sourceDecorator) {
                        if (connector.sourceDecorator) {
                            decorator = $.extend(true, {}, connector.sourceDecorator, options.sourceDecorator);
                        } else {
                            decorator = ej.datavisualization.Diagram.Decorator(options.sourceDecorator);
                        }
                        connector.sourceDecorator = decorator;
                    }
                    if (options.sourceDecorator || options.targetDecorator) {
                        ej.datavisualization.Diagram.DiagramContext.clearDecorators(connector, this);
                        ej.datavisualization.Diagram.DiagramContext.renderDecorators(connector, this);
                    }
                    if (this._svg) {
                        ej.datavisualization.Diagram.DiagramContext.update(connector, this);
                        ej.datavisualization.Diagram.DiagramContext.updateConnectorStyle(connector, this);
                    }
                    if (this.selectionList[0] && this.selectionList[0].name === connector.name)
                        ej.datavisualization.Diagram.SvgContext.updateSelector(connector, this._svg, this._currZoom, this, this.model.selectorConstraints);
                }
            }
        },

        /**
      *update Diagram with its specific properties
      *@return jQuery
      *@param {String} name  to identify the diagram from the model
      *@param {object} options to specify which properties of the diagram to be updated
      *@example
      *&lt;div id="diagramcontent"&gt;&lt;/div&gt;
      *&lt;script&gt;
      *var diagram=$("#diagramcontent").ejDiagram("instance");
      *var value = ej.datavisualization.Diagram.DiagramConstraints.Default;
      *diagram.updateDiagram(diagram.name, { constraints: value });      
      *&lt;/script&gt;
      *@memberof ejDiagram
      *@instance
      */

        update: function (option) {
            if (option.tool) {
                this.activeTool._showAllPorts(true);
                var tool = option.tool;
                this.model.tool = tool;
                if (tool & ej.datavisualization.Diagram.Tool.ContinuesDraw) {
                    this.activateTool(this.model.connectorType, false);
                }
                else if (tool & ej.datavisualization.Diagram.Tool.DrawOnce) {
                    this.activateTool(this.model.connectorType, true);
                }
                else if (tool & ej.datavisualization.Diagram.Tool.ZoomPan) {
                    if (ej.datavisualization.Diagram.Util.canPanning(this)) {
                        this._toolToActivate = "panTool";
                        this.activeTool = this.tools[this._toolToActivate];
                        this.activeTool._isMouseDown = false;
                        this._currentCursor = "pointer";
                    }
                }
                else if (tool & ej.datavisualization.Diagram.Tool.MultipleSelect) {
                    this.activateTool("select");
                }
            }
            if (option.bridgeDirection)
            {
                this.model.bridgeDirection = option.bridgeDirection;
                var connectors = this.model.connectors;
                for (var i = 0, len = connectors.length; i < len; i++) {
                    var connector = connectors[i];
                    ej.datavisualization.Diagram.Util.updateBridging(connector, this);
                    ej.datavisualization.Diagram.DiagramContext.update(connector, this);
                }
            }
        },
        /**
       *To bring the specified portion of the diagram content to center of the diagram viewport
       *@return jQuery
       *@param {Object} rect pass bounds value (x,y,height,width)
       *@example
       
       *&lt;div id="diagramcontent"&gt;&lt;/div&gt;
       *&lt;script&gt;
       *var diagram=$("#diagramcontent").ejDiagram("instance");
       *diagram.bringToCenter(ej.datavisualization.Diagram.Rectangle(700, 500, 80, 80));
       *&lt;/script&gt;
       *@memberof ejDiagram
        *@instance
       */
        bringToCenter: function (rect) {
            var scale = this._currZoom;
            var viewPort = ej.datavisualization.Diagram.ScrollUtil._viewPort(this);
            var hoffset = rect.x + (rect.width) / 2 - viewPort.width / 2;
            var voffset = rect.y + (rect.height) / 2 - viewPort.height / 2;
            hoffset = Math.max(0, hoffset) * scale;
            voffset = Math.max(0, voffset) * scale;
            ej.datavisualization.Diagram.ZoomUtil._applyTransform(this, scale, ej.datavisualization.Diagram.Point(-hoffset, -voffset), null);
        },
        /**
        *To bring the specified portion of the diagram content to the diagram viewport
        *@return jQuery
        *@param {Object} bounds pass bounds value (x,y,height,width)
        *@example
        
        *&lt;div id="diagramcontent"&gt;&lt;/div&gt;
        *&lt;script&gt;
        *var diagram=$("#diagramcontent").ejDiagram("instance");
        *diagram.bringIntoView(ej.datavisualization.Diagram.Rectangle(700, 500, 80, 80));
        *&lt;/script&gt;
        *@memberof ejDiagram
        *@instance
        */
        bringIntoView: function (bounds) {
            var x = 0, y = 0;
            bounds.right = bounds.right ? bounds.right : bounds.x + bounds.width;
            bounds.bottom = bounds.bottom ? bounds.bottom : bounds.y + bounds.height;
            var hoffset = this._hScrollOffset;
            var voffset = this._vScrollOffset;
            var viewPort = ej.datavisualization.Diagram.ScrollUtil._viewPort(this);
            if (bounds.right > (-hoffset + viewPort.width)) {
                x = bounds.right - viewPort.width;
            }
            if (bounds.x < -hoffset) {
                x = bounds.x;
            }
            if (bounds.bottom > (-voffset + viewPort.height)) {
                y = bounds.bottom - viewPort.height;
            }
            if (bounds.y < -voffset) {
                y = bounds.y;
            }
            ej.datavisualization.Diagram.ZoomUtil._applyTransform(this, this._currZoom, ej.datavisualization.Diagram.Point(-x, -y), null);

        },
        getBridgeSegment: function (startPt, endPt, angle, bridgeSpace, sweep) {
            var path = "A " + bridgeSpace / 2 + " " + bridgeSpace / 2 + "  " + angle + " , 1 " + sweep + " " + endPt.x + "," + endPt.y;
            return path;
        },


        /**
      
          *To fit the diagram content into diagram viewport       
          *@return jQuery
          *@param {Object} type to specify diagramcontent either by width or height or both of the viewport    
          *@param {Object} region to set fit whether to diagramcontent and pagesettings
          *@param {Object} margin to set the required margin viewport
          *@example         
          *&lt;div id="diagramcontent"&gt;&lt;/div&gt;
          *&lt;script&gt;
          *var diagram=$("#diagramcontent").ejDiagram("instance");
          *diagram.fitToPage(type,region,margin);
          *&lt;/script&gt;
          *@memberof ejDiagram
          *@instance
          */

        fitToPage: function (type, region, margin) {
            var zoomFactor, factor, scale = {}, bounds;
            if (this.model.nodes.length > 0 || this.model.connectors.length > 0) {
                var viewPort = ej.datavisualization.Diagram.ScrollUtil._viewPort(this);
                if (!margin)
                    margin = ej.datavisualization.Diagram.Margin();
                if (region != "custom") {
                    if (region == "content") {
                        bounds = new ej.datavisualization.Diagram.Rectangle();
                        bounds.x = this._spatialSearch.pageLeft;
                        bounds.y = this._spatialSearch.pageTop;
                        bounds.width = this._spatialSearch.pageRight;
                        bounds.height = this._spatialSearch.pageBottom;
                    }
                    else
                        bounds = this._getDigramBounds();
                }
                var width = region != "custom" ? bounds.width - bounds.x : bounds.width;
                var height = region != "custom" ? bounds.height - bounds.y : bounds.height;
                scale.x = (viewPort.width - (margin.left + margin.right)) / width;
                scale.y = (viewPort.height - (margin.top + margin.bottom)) / height;
                var deltaX = this._hScrollOffset;
                var deltaY = this._vScrollOffset;

                switch (type) {
                    case "width":
                        zoomFactor = scale.x;
                        factor = zoomFactor / this._getCurrentZoom();
                        deltaX -= bounds.x * zoomFactor;
                        deltaY -= this._vScrollOffset * factor;
                        deltaY = region != "custom" ? deltaY : deltaY + this._vScrollOffset * factor;
                        break;
                    case "height":
                        zoomFactor = scale.y;
                        factor = zoomFactor / this._getCurrentZoom();
                        //var centerX = (viewPort.width - (bounds.x > 0 ? bounds.width : width) * zoomFactor - bounds.x * zoomFactor) / 2;
                        //deltaX += centerX - (bounds.x * zoomFactor) / 4 + margin.left;
                        //deltaY -= bounds.y * zoomFactor - margin.top;
                        var centerX = (viewPort.width - (region != "content" ? bounds.width : width) * zoomFactor) / 2 - bounds.x * zoomFactor;
                        var centerY = (viewPort.height - (region != "content" ? bounds.height : height) * zoomFactor) / 2 - bounds.y * zoomFactor;
                        deltaX += centerX + (margin.left - margin.right) / 2 * zoomFactor;
                        deltaY += centerY + (margin.top - margin.bottom) / 2 * zoomFactor;
                        break;
                    case "page":
                        zoomFactor = Math.min(scale.x, scale.y);
                        factor = zoomFactor / this._getCurrentZoom();
                        if (Math.min(scale.x, scale.y) == scale.x) {
                            deltaX -= bounds.x * zoomFactor;
                            deltaY -= this._vScrollOffset * factor;
                            deltaY = region != "custom" ? deltaY : deltaY + this._vScrollOffset * factor;
                        }
                        else {
                            var centerX = (viewPort.width - (region != "content" ? bounds.width : width) * zoomFactor) / 2 - bounds.x * zoomFactor;
                            var centerY = (viewPort.height - (region != "content" ? bounds.height : height) * zoomFactor) / 2 - bounds.y * zoomFactor;
                            deltaX += centerX + (margin.left - margin.right)/2 * zoomFactor;
                            deltaY += centerY + (margin.top - margin.bottom)/2 * zoomFactor;
                        }
                    break;
            }
                ej.datavisualization.Diagram.ZoomUtil.zoomPan(this, factor, deltaX, deltaY, new ej.datavisualization.Diagram.Point(0, 0), true);
            }
            else {
                factor = 1 / this._getCurrentZoom();
                ej.datavisualization.Diagram.ZoomUtil.zoomPan(this, factor, this._hScrollOffset, this._vScrollOffset, new ej.datavisualization.Diagram.Point(0, 0), true);
            }
        },



        /**
    *To export the diagram as image .
    *@return jQuery
    *@example
 
    *&lt;div id="diagramcontent"&gt;&lt;/div&gt;
    *&lt;script&gt;
    *var diagram=$("#diagramcontent").ejDiagram("instance");
    *diagram.exportAsImage();
    *&lt;/script&gt;
    *@memberof ejDiagram
    *@instance
 */
        exportAsImage: function () {
            var bounds = this._getDigramBounds();
            var bounds = {
                x: this._spatialSearch.pageLeft || bounds.x,
                y: this._spatialSearch.pageTop || bounds.y,
                width: this._spatialSearch.pageRight - this._spatialSearch.pageLeft || bounds.width,
                height: this._spatialSearch.pageBottom - this._spatialSearch.pageTop || bounds.height
            };
            var canvas = new ej.datavisualization.Diagram.Canvas({ "id": "mycanvas", "width": bounds.width + 50, "height": bounds.height + 50 });
            var context = canvas.document.getContext("2d");
            context.translate(-bounds.x + 25, -bounds.y + 25);
            context.fillStyle = this.model.pageSettings.pageBackgroundColor;
            context.fillRect(bounds.x - 25, bounds.y - 25, bounds.width + 50, bounds.height + 50);
            for (var i = 0; i < this.model.nodes.length; i++) {
                if (this.model.nodes[i].isGroup && !this.model.nodes[i].parent)
                    ej.datavisualization.Diagram.CanvasContext.renderGroup(this.model.nodes[i], canvas, this);
                else if (!this.model.nodes[i].parent)
                    ej.datavisualization.Diagram.CanvasContext.renderNode(this.model.nodes[i], canvas);
            }
            for (var i = 0; i < this.model.connectors.length; i++) {
                ej.datavisualization.Diagram.CanvasContext.renderConnector(this.model.connectors[i], canvas);
            }
            var url = canvas.document.toDataURL('image/png');
            return url;
        },
       /**
    *To print the diagram as image .
    *@return jQuery
    *@example
 
    *&lt;div id="diagramcontent"&gt;&lt;/div&gt;
    *&lt;script&gt;
    *var diagram=$("#diagramcontent").ejDiagram("instance");
    *diagram.print();
    *&lt;/script&gt;
    *@memberof ejDiagram
    *@instance
 */
        print: function () {
            var url = this.exportAsImage();
            var img = document.createElement('img');
            var attr = {
                "id": this.element[0].id + "_printImage",
                "src": url,
            };
            ej.datavisualization.Diagram.Util.attr(img, attr);
            var div = document.createElement("div");
            div.appendChild(img);

            // specify window parameters
            var PrintWind = window.open('');
            if (PrintWind != null) {
                PrintWind.document.write('<html><head><title></title></head>');
                PrintWind.document.write('<BODY onload="window.print()">');
                PrintWind.document.write("<center>" + div.innerHTML + "</center>");
                PrintWind.document.close();
            }
        },

        /**
*To save the diagram .
*@return jQuery
*@param {Object} 
*@example

*&lt;div id="diagramcontent"&gt;&lt;/div&gt;
*&lt;script&gt;
*var diagram=$("#diagramcontent").ejDiagram("instance");
*diagram.save();
*&lt;/script&gt;
*@memberof ejDiagram
*@instance
*/
        save: function () {
            var drawingTools = this.model.drawingTools;
            var userHandles = this.model.userHandles;
            this.model.drawingTools = null;
            this.model.userHandles = null;
            if (this.nameTable["multipleSelection"]) {
                ej.datavisualization.Diagram.Util.removeItem(this.model.nodes, this.nameTable["multipleSelection"]);
            }
            this._updateTableNodes();
            var model = jQuery.extend(true, {}, this.model);
            this.model.drawingTools = drawingTools;
            this.model.userHandles = userHandles;
            return model;
        },

        _updateTableNodes: function () {
            var i = 0, node;
            for (i; i < this.model.nodes.length; i++) {
                node = this.nameTable[this.model.nodes[i].name];
                if (node) {
                    this.model.nodes[i] = node;
                }
            }
        },


        /**
*To load the diagram .
*@return jQuery 
*@param {Object} 
*@example

*&lt;div id="diagramcontent"&gt;&lt;/div&gt;
*&lt;script&gt;
*var diagram=$("#diagramcontent").ejDiagram("instance");
*diagram.load(data);
*&lt;/script&gt;
*@memberof ejDiagram
*@instance
*/
        load: function (data) {
            this.clear();
            var contextmenu = document.getElementById(this.element[0].id + "_contextMenu");
            if (contextmenu)
                contextmenu.parentNode.removeChild(contextmenu);
            var load = {
                nodes: data.nodes ? data.nodes : [],
                connectors: data.connectors ? data.connectors : [],
                contextMenu: data.contextMenu ? data.contextMenu : {},
                snapSettings: data.snapSettings ? data.snapSettings : {},
                backgroundImage: data.backgroundImage ? data.backgroundImage : null,
                enableAutoScroll: data.enableAutoScroll ? data.enableAutoScroll : true,
                autoScrollMargin: data.autoScrollMargin ? data.autoScrollMargin : 20,
                layout: data.layout ? data.layout : {},
                pageSettings: data.pageSettings ? data.pageSettings : {},
                click: data.click ? data.click : null,
                connectionChange: data.connectionChange ? data.connectionChange : null,
                defaultSettings: data.defaultSettings ? data.defaultSettings : null,
                dataSource: data.dataSource ? data.dataSource : null,
                dataSourceSchema: data.dataSourceSettings ? data.dataSourceSettings : null,
                doubleClick: data.doubleClick ? data.doubleClick : null,
                enableContextMenu: data.enableContextMenu ? data.enableContextMenu : true,
                showTooltip: data.showTooltip ? data.showTooltip : true,
                handles: data.handles ? data.handles : [],
                height: data.height ? data.height : null,
                mouseEnter: data.mouseEnter ? data.mouseEnter : null,
                mouseOver: data.mouseOver ? data.mouseOver : null,
                mouseLeave: data.mouseLeave ? data.mouseLeave : null,
                nodeTemplate: data.nodeTemplate ? data.nodeTemplate : null,
                rotationChange: data.rotationChange ? data.rotationChange : null,
                selectorConstraints: data.selectorConstraints ? data.selectorConstraints : ej.datavisualization.Diagram.SelectorConstraints.All,
                tooltipTemplateId: data.tooltipTemplateId ? data.tooltipTemplateId : null,
                width: data.width ? data.width : null,
                constraints: data.constraints,
                zoomFactor: data.zoomFactor ? data.zoomFactor : 1,

            };
            $("#" + this._id).ejDiagram(load);
        },

        /**
*To arrange the elements on the page in the diagram .
*@return jQuery
*@param {String} 
*@example

*&lt;div id="diagramcontent"&gt;&lt;/div&gt;
*&lt;script&gt;
*var diagram=$("#diagramcontent").ejDiagram("instance");
*diagram.layout();
*&lt;/script&gt;
*@memberof ejDiagram
*@instance
*/

        layout: function () {
            this._setLayout();
        },

        _setLayout: function () {
            //this.model.layout = new ej.datavisualization.Diagram.HierarchicalLayout(this.model.layout);
            this._doLayout();
            ej.datavisualization.Diagram.PageUtil._updatePageSize(this);
            this._clearSelection();
            //this._updateNodes();
            //this._updateConnectors();
        },
        /**
               *Select the target node
               *@return jQuery
               *@param {Object} node pass target node to be updated
               *@example
               *&lt;div id="diagramcontent"&gt;&lt;/div&gt;
               *&lt;script&gt;
               *var diagram=$("#diagramcontent").ejDiagram("instance");
               *var node=diagram.model.nodes[0];
               *diagram.addSelection(node);
               *&lt;/script&gt;
               *@memberof ejDiagram
*@instance
               */
        addSelection: function (node) {
            this._addSelection(node);
        },

        /**
     *update specified node as selected object.
     *@return jQuery
     *@param {String} name  to identify the object from the model 
     *@example
     *&lt;div id="diagramcontent"&gt;&lt;/div&gt;
     *&lt;script&gt;
     *var diagram=$("#diagramcontent").ejDiagram("instance"); 
     *diagram.updateSelectedObject(name);    
     *&lt;/script&gt;
     *@memberof ejDiagram
     *@instance
     */
        updateSelectedObject: function (nodeName) {
            var node = this.nameTable[nodeName];
            if (node) {
                this.activeTool.selectedObject = node;

            }
        },
        /**
        *Update the selection of target node/connector
        *@return jQuery
        *@param {Boolean} isDragging pass true/false whether to drag or not
        *@example
        *&lt;div id="diagramcontent"&gt;&lt;/div&gt;
        *&lt;script&gt;
        *var diagram=$("#diagramcontent").ejDiagram("instance");
        *diagram.updateSelection(true);
        *&lt;/script&gt;
        *@memberof ejDiagram
        *@instance
        */

        updateSelection: function (isDragging) {
            if (ej.datavisualization.Diagram.Util.isPageEditable(this)) {
                this._updateSelectionHandle(isDragging);
            }
        },
        /**
        *To activate registered tool 
        *@return jQuery
        *@param {String} toolName name of the tool to be activated
        *@param {Boolean} singleAction default value set to true tool action to be performed only once
        *@example
        *&lt;div id="diagramcontent"&gt;&lt;/div&gt;
        *&lt;script&gt;
        *var diagram=$("#diagramcontent").ejDiagram("instance");
        *diagram.activateTool("panTool");
                 *&lt;/script&gt;

          *@example
            *&lt;div id="diagramcontent"&gt;&lt;/div&gt;
                 *&lt;script&gt;
                 *var diagram=$("#diagramcontent").ejDiagram("instance");                 
                 *diagram.activateTool("orthogonalTool",false);
                 *&lt;/script&gt;
                 *@memberof ejDiagram
                 *@instance
        */
        activateTool: function (toolName, singleAction) {

            if (this.tools[toolName]) {
                if (this.activeTool instanceof ej.datavisualization.Diagram.LineTool) {
                    this.activeTool._showAllPorts(true);
                }
                this._toolToActivate = toolName;
                this.activeTool = this.tools[this._toolToActivate];
                if (singleAction != undefined)
                    this.activeTool.singleAction = singleAction;
            }
            if (this.activeTool instanceof ej.datavisualization.Diagram.ImageTool && this.activeTool.autoOpenUpload) {
                this.activeTool._showPopup();
            }
            if (this.activeTool instanceof ej.datavisualization.Diagram.PanTool) {
                this._currentCursor = "pointer";
            }
            if (this.activeTool instanceof ej.datavisualization.Diagram.LineTool) {
                if (this.activeTool.singleAction)
                    this.model.tool = this.model.tool | ej.datavisualization.Diagram.Tool.DrawOnce;
                else
                    this.model.tool = this.model.tool | ej.datavisualization.Diagram.Tool.ContinuesDraw;
                this.activeTool._showAllPorts();
            }
            this._drawingTool = true;
        },
        /**
       *Deactivate the current activated tool
       *@return jQuery
       *@example
       *&lt;div id="diagramcontent"&gt;&lt;/div&gt;
       *&lt;script&gt;
       *var diagram=$("#diagramcontent").ejDiagram("instance");
       *diagram.deactivateTool();
       *&lt;/script&gt;
       *@memberof ejDiagram
       *@instance
       */
        deactivateTool: function () {
            if (this.activeTool instanceof ej.datavisualization.Diagram.LineTool) {
                this.activeTool._showAllPorts(true);
            }
            if (this.model.tool & ej.datavisualization.Diagram.Tool.DrawOnce)
                this.model.tool = this.model.tool ^ ej.datavisualization.Diagram.Tool.DrawOnce;
            if (this.model.tool & ej.datavisualization.Diagram.Tool.ContinuesDraw)
                this.model.tool = this.model.tool ^ ej.datavisualization.Diagram.Tool.ContinuesDraw;
            if (this.model.tool & ej.datavisualization.Diagram.Tool.ZoomPan) {
                this._toolToActivate = "panTool";
                this.activeTool = this.tools[this._toolToActivate];
                this.activeTool._mousedown = false;
                this._currentCursor = "pointer";
            }
            else {
                this._toolToActivate = "select";
                this.activeTool = this.tools[this._toolToActivate];
            }
        },

        /**
                    *Update the given label style
                    *@return jQuery
                    *@param {String} nodeName pass the name of node/connector
                    *@param {Object} label pass the label to be modified
                    *@param {Object} obj pass the new porperties of the label
                    *@example
                    
                    *&lt;div id="diagramcontent"&gt;&lt;/div&gt;
                    *&lt;script&gt;
                    *var diagram=$("#diagramcontent").ejDiagram("instance");
                    *var node=diagram.selectionList[0];
                    *var label = [];
                    *label =[{"name":"node1" "text": "node", "bold": true, "italic": true}]
                    *diagram.updateLabel(node.name,node.labels[0],label);
                    *&lt;/script&gt;
                    *@memberof ejDiagram
*@instance
                    */
        updateLabel: function (nodeName, label, obj) {
            if (nodeName && ej.datavisualization.Diagram.Util.isPageEditable(this)) {
                var mNode = this._findNode(nodeName);
                if (!mNode)
                    mNode = this._findConnector(nodeName);
                if (!obj.name || label.name == obj.name) {
                    if (mNode) {
                        for (var prop in obj)
                            if (label.hasOwnProperty(prop)) {
                                label[prop] = obj[prop];
                            }
                        if (mNode.segments || (mNode.shape && mNode.shape.type !== "text")) {
                            ej.datavisualization.Diagram.DiagramContext.updateLabelStyle(mNode, label, this);
                            ej.datavisualization.Diagram.DiagramContext.updateLabel(mNode, label, this);
                        } else if (mNode.shape && mNode.shape.type === "text") {
                            ej.datavisualization.Diagram.DiagramContext.updateTextBlock(mNode, label, this);
                        }
                    }
                }
            }
        },
        /**
                    *Starts the editing of the specified label in the given node
                    *@return jQuery
                    *@param {Object} node pass the node/connector to which label to be edited
                      *@param {Object} label pass the label to be edited
                    *@example
                    
                    *&lt;div id="diagramcontent"&gt;&lt;/div&gt;
                    *&lt;script&gt;
                    *var diagram=$("#diagramcontent").ejDiagram("instance");
                    *var node=diagram.selectionList[0];
                    *diagram.startLabelEdit(node,node.labels[0]);
                    *&lt;/script&gt;
                    *@memberof ejDiagram
                    *@instance
                    */
        startLabelEdit: function (node, label) {
            if (node) {
                //Disables the edit mode of labels in the selectionList
                for (var i = 0; i < this.selectionList.length; i++) {
                    var mNode = this.selectionList[i];
                    if (mNode !== node && typeof mNode.labels.length != "undefined") {
                        for (var j = 0; j < mNode.labels.length; j++)
                            mNode.labels[j].mode = ej.datavisualization.Diagram.LabelEditMode.View;
                    }
                }
                if (!label && node.shape.type === "text")
                    label = node.shape.textBlock;
                if (this._setLabelEditing(label))
                    this._startEdit(node);
                else {
                    this._endEdit();
                }
            }
        },

        _setSelectorConstraints: function (constraints) {
            this.model.selectorConstraints = constraints;
            if (this.selectionList[0]) {
                ej.datavisualization.Diagram.SvgContext.clearSelector(this._svg, this._adornerLayer);
                ej.datavisualization.Diagram.SvgContext.renderSelector(this.selectionList[0], this._svg, this._adornerLayer, this._currZoom, constraints);
                if (this.model.selectorConstraints & ej.datavisualization.Diagram.SelectorConstraints.UserHandles)
                    ej.datavisualization.Diagram.SvgContext.renderUserHandles(this.model.userHandles, this.selectionList[0], this._svg, this.selectionList[0].isPseudoGroup,
                        this._currZoom, this._adornerLayer);
            }
        },
        /**
        *Group the selected node and connector
        *@return jQuery
        *@example
       
        *&lt;div id="diagramcontent"&gt;&lt;/div&gt;
        *&lt;script&gt;
        *var diagram=$("#diagramcontent").ejDiagram("instance");
        *diagram.group(); 
        *&lt;/script&gt;
        *@memberof ejDiagram
        *@instance
        */
        group: function () {
            if (this.selectionList[0] && this.selectionList[0].isPseudoGroup) {
                var selectionList = [];
                var i;
                for (i = 0; i < this.selectionList[0].children.length; i++)
                    selectionList.push(this.nameTable[this.selectionList[0].children[i]]);
                this._sortByZIndex(selectionList, true);
                var element;
                var groupName = ej.datavisualization.Diagram.Util.randomId();
                var group = new ej.datavisualization.Diagram.Group({ name: groupName, });
                this._clearSelection();
                for (i = 0; i < selectionList.length; i++) {
                    this._isUndo = true;
                    group.children.push(selectionList[i].name);
                    selectionList[i].parent = groupName;
                    ej.datavisualization.Diagram.Util.removeItem(this.selectionList, selectionList[i].name);
                    if (ej.browserInfo().name === "msie") {
                        if (selectionList[i].shape && selectionList[i].shape.type == "html") {
                            element = document.getElementById(selectionList[i].name + "_parentdiv");
                            element.parentNode.removeChild(element);
                        }
                        else {
                            if (selectionList[i].isGroup)
                                this._checkForHtmlNode(selectionList[i]);
                        }
                    }
                    // $("#" + selectionList[i].name)[0].parentNode.removeChild($("#" + selectionList[i].name)[0]);
                    element = this._svg.document.getElementById(selectionList[i].name);
                    element.parentNode.removeChild(element);
                }
                ej.datavisualization.Diagram.Util.clear(this.selectionList);

                this.add(group);
                this._addSelection(group);
                var entry = { type: "groupchanged", object: group, actionType: "group" };
                this.addHistoryEntry(entry);
                //var entry = new ej.datavisualization.Diagram.HistoryEntry(new ej.datavisualization.Diagram.GroupHandleCmd(group, "group"));
                //this.historyManager.addHistoryEntry(entry);
                this._isUndo = false;
            }
        },

        _checkForHtmlNode: function (group) {
            for (var i = 0; i < group.children.length; i++) {
                var child = this.nameTable[group.children[i]];
                if (child) {
                    if (child.isGroup) {
                        this._checkForHtmlNode(child);
                    } else {
                        if (child.shape && child.shape.type == "html") {
                            var element = document.getElementById(child.name + "_parentdiv");
                            element.parentNode.removeChild(element);
                        }
                    }
                }
            }
        },
        _removeChildren: function (group, args) {
            for (var i = 0; i < group.children.length; i++) {
                var child = this.nameTable[group.children[i]];
                if (child) {
                    this._removeConnector(child, args);
                    ej.datavisualization.Diagram.Util.removeItem(this.model.nodes, child);
                    if (child.isGroup) {
                        this._removeChildren(child, args);
                    } else {
                        ej.datavisualization.Diagram.SpatialUtil._removeFromaQuad(this._spatialSearch, this._spatialSearch.quadTable[child.name], child);
                        ej.datavisualization.Diagram.SpatialUtil._updateBounds(this, this._spatialSearch, child);
                        delete this.nameTable[child.name];
                    }
                }
            }
        },
        /**
*Ungroup the selected group
*@return jQuery
*@example

*&lt;div id="diagramcontent"&gt;&lt;/div&gt;
*&lt;script&gt;
*var diagram=$("#diagramcontent").ejDiagram("instance");
*diagram.ungroup(); 
*&lt;/script&gt;
*@memberof ejDiagram
*@instance
*/
        ungroup: function () {
            if (this.selectionList[0] && this.selectionList[0].isGroup) {
                this._isUndo = true;
                var selectionList = this.selectionList[0].children;
                var i, node;
                node = this.selectionList[0];
                $("#" + this.selectionList[0].name)[0].parentNode.removeChild($("#" + this.selectionList[0].name)[0]);
                if (ej.browserInfo().name === "msie")
                    this._checkForHtmlNode(this.selectionList[0]);
                this._clearSelection();
                var object;
                ej.datavisualization.Diagram.Util.clear(this.selectionList);
                for (i = 0; i < selectionList.length; i++) {
                    var item = this.nameTable[selectionList[i]];
                    if (item) {
                        if (item.parent)
                            item.parent = null;
                        if (item.segments) {
                            object = item;
                            object.parent = "";
                            this._updateEdges(object);
                            this._dock(object, this.nameTable);
                            ej.datavisualization.Diagram.DiagramContext.renderConnector(object, this);
                        } else {
                            object = item;
                            object.parent = "";
                            if (object.isGroup) {
                                this._updateChildrenEdges(object);
                                ej.datavisualization.Diagram.DiagramContext.renderGroup(object, this);
                            }
                            else
                                ej.datavisualization.Diagram.DiagramContext.renderNode(object, this);
                        }
                    }

                }
                //this._clearSelection();
                ej.datavisualization.Diagram.Util.removeItem(this.model.nodes, this._findNode(node.name));

                var entry = { type: "groupchanged", object: node, actionType: "ungroup" };
                this.addHistoryEntry(entry);

                //var entry = new ej.datavisualization.Diagram.HistoryEntry(new ej.datavisualization.Diagram.GroupHandleCmd(node, "ungroup"));
                //this.historyManager.addHistoryEntry(entry);
                this._isUndo = false;

            }
        },


        /**
       *Executes nudge command tp move selected node/connector one pixel 
       *@return jQuery
       *@param {String} direction  specify the driection to move the selected objects ("left","right",top","bottom")
       *@example
       *&lt;div id="diagramcontent"&gt;&lt;/div&gt;
       *&lt;script&gt;
       *var diagram=$("#diagramcontent").ejDiagram("instance");
       *diagram.nudge("direction"); 
       *&lt;/script&gt;
       *@memberof ejDiagram
       *@instance
       */

        nudge: function (direction, nudgeDelta) {
            nudgeDelta = !nudgeDelta ? 1 : nudgeDelta;
            var dir = { up: { x: 0, y: -nudgeDelta }, down: { x: 0, y: nudgeDelta }, left: { x: -nudgeDelta, y: 0 }, right: { x: nudgeDelta, y: 0 } };
            var nudge = dir[direction];
            if (nudge && this.selectionList[0]) {
                this._nudge(this.selectionList[0], nudge.x, nudge.y);
            }
        },

        /**
              *Select all nodes and connector in diagram
              *@return jQuery
              *@example
             
              *&lt;div id="diagramcontent"&gt;&lt;/div&gt;
              *&lt;script&gt;
              *var diagram=$("#diagramcontent").ejDiagram("instance");
              *diagram.selectAll(); 
              *&lt;/script&gt;
              *@memberof ejDiagram
              *@instance
              */
        selectAll: function () {
            var collection, nodes, i, len, item = null;
            this._clearSelection();
            collection = ej.datavisualization.Diagram.Group({ isPseudoGroup: true, name: "multipleSelection" });
            nodes = this.model.nodes;
            for (i = 0, len = nodes.length; i < len; i++) {
                if (ej.datavisualization.Diagram.Util.canSelect(nodes[i])) {
                    item = this.nameTable[nodes[i].name];
                    if (item.parent == "") {
                        collection.children.push(item.name);
                    }
                }
            }
            var connectors = this.model.connectors;
            for (i = 0, len = connectors.length; i < len; i++) {
                if (ej.datavisualization.Diagram.Util.canSelect(connectors[i])) {
                    if (connectors[i].parent == "") {
                        collection.children.push(connectors[i].name);
                    }
                }
            }
            ej.datavisualization.Diagram.Util._updateGroupBounds(collection, this.nameTable);
            //ej.datavisualization.Diagram.Util.clear(this.selectionList);
            this.model.nodes.push(collection);
            this.nameTable[collection.name] = collection;
            if (collection.children.length > 0)
                this._addSelection(collection);
        },
        /**
        *Align selected object based on reference object and direction
        *@return jQuery
        *@param {String} direction To specify the driection for alignment ("left","right",top","bottom")
        *@example
        *&lt;div id="diagramcontent"&gt;&lt;/div&gt;
        *&lt;script&gt;
        *var diagram=$("#diagramcontent").ejDiagram("instance");
        *diagram.align("left"); 
        *&lt;/script&gt;
        *@memberof ejDiagram
        *@instance
        */
        align: function (direction) {
            if (this.selectionList[0] && this.selectionList[0].isPseudoGroup && this.selectionList[0].children && this.selectionList[0].children.length > 0) {
                var list = this.nameTable[this.selectionList[0].name].children;
                var cNode;
                if (direction === "left")
                    cNode = this._findCornerNodes().left;
                else if (direction === "right")
                    cNode = this._findCornerNodes().right;
                else if (direction === "top")
                    cNode = this._findCornerNodes().top;
                else if (direction === "bottom")
                    cNode = this._findCornerNodes().bottom;
                var cNodeBounds;
                if (cNode)
                    cNodeBounds = ej.datavisualization.Diagram.Util.bounds(cNode);
                else if (direction === "center" || direction === "middle") {
                    cNodeBounds = ej.datavisualization.Diagram.Util.bounds(this.selectionList[0]);
                }
                if (cNodeBounds) {
                    var i, bounds, node, x = 0, y = 0;
                    for (i = 0; i < list.length; i++) {
                        node = this.nameTable[list[i]];
                        bounds = ej.datavisualization.Diagram.Util.bounds(node);
                        if (direction === "left")
                            x = -(bounds.x - cNodeBounds.x);
                        else if (direction === "right")
                            x = -((bounds.x + bounds.width) - (cNodeBounds.x + cNodeBounds.width));
                        else if (direction === "top")
                            y = -(bounds.y - cNodeBounds.y);
                        else if (direction === "bottom")
                            y = -((bounds.y + bounds.height) - (cNodeBounds.y + cNodeBounds.height));
                        else if (direction === "center")
                            x = -((bounds.x + bounds.width / 2) - (cNodeBounds.x + cNodeBounds.width / 2));
                        else if (direction === "middle")
                            y = -((bounds.y + bounds.height / 2) - (cNodeBounds.y + cNodeBounds.height / 2));
                        this._moveNode(x, y, node);
                    }
                    ej.datavisualization.Diagram.Util._updateGroupBounds(this.selectionList[0], this.nameTable);
                    ej.datavisualization.Diagram.SvgContext.updateSelector(this.selectionList[0], this._svg, this._currZoom, this, this.model.selectorConstraints);
                }
            }
        },
        /**
        *Update size of the object in the selection list to the size of first object in selction list 
        *@return jQuery
        *@example
        
        *&lt;div id="diagramcontent"&gt;&lt;/div&gt;
        *&lt;script&gt;
        *var diagram=$("#diagramcontent").ejDiagram("instance");
        *diagram.sameSize(); 
        *&lt;/script&gt;
        *@memberof ejDiagram
        *@instance
        */
        sameSize: function () {
            if (this.selectionList[0] && this.selectionList[0].isPseudoGroup && this.selectionList[0].children && this.selectionList[0].children.length > 0) {
                var list = this.nameTable[this.selectionList[0].name].children;
                var fNode = this.nameTable[this.selectionList[0].children[0]];
                var fNodeBounds = ej.datavisualization.Diagram.Util.bounds(fNode);
                for (var i = 0; i < list.length; i++) {
                    var element = this.nameTable[list[i]];
                    if (!(element.segments)) {
                        element.height = fNodeBounds.height;
                        element.width = fNodeBounds.width;
                        this._updateQuad(this, element);
                        ej.datavisualization.Diagram.DiagramContext.update(element, this);
                    }
                }
                ej.datavisualization.Diagram.Util._updateGroupBounds(this.selectionList[0], this.nameTable);
                ej.datavisualization.Diagram.SvgContext.updateSelector(this.selectionList[0], this._svg, this._currZoom, this, this.model.selectorConstraints);
            }
        },
        /**
        *Update height of the object in the selection list to the height of first object in selection list 
        *@return jQuery
        *@example
        
        *&lt;div id="diagramcontent"&gt;&lt;/div&gt;
        *&lt;script&gt;
        *var diagram=$("#diagramcontent").ejDiagram("instance");
        *diagram.sameHeight(); 
        *&lt;/script&gt;
        *@memberof ejDiagram
*@instance
        */
        sameHeight: function () {
            if (this.selectionList[0] && this.selectionList[0].isPseudoGroup && this.selectionList[0].children && this.selectionList[0].children.length > 0) {
                var list = this.nameTable[this.selectionList[0].name].children;
                var fNode = this.nameTable[this.selectionList[0].children[0]];
                var fNodeBounds = ej.datavisualization.Diagram.Util.bounds(fNode);
                for (var i = 0; i < list.length; i++) {
                    var element = this.nameTable[list[i]];
                    if (!(element.segments)) {
                        element.height = fNodeBounds.height;
                        this._updateQuad(this, element);
                        ej.datavisualization.Diagram.DiagramContext.update(element, this);
                    }
                }
                ej.datavisualization.Diagram.Util._updateGroupBounds(this.selectionList[0], this.nameTable);
                ej.datavisualization.Diagram.SvgContext.updateSelector(this.selectionList[0], this._svg, this._currZoom, this, this.model.selectorConstraints);
            }
        },
        /**
        *Update width of the object in the selection list to the width of first object in selection list 
        *@return jQuery
        *@example
        
        *&lt;div id="diagramcontent"&gt;&lt;/div&gt;
        *&lt;script&gt;
        *var diagram=$("#diagramcontent").ejDiagram("instance");
        *diagram.sameWidth(); 
        *&lt;/script&gt;
        *@memberof ejDiagram
*@instance
        */
        sameWidth: function () {
            if (this.selectionList[0] && this.selectionList[0].isPseudoGroup && this.selectionList[0].children && this.selectionList[0].children.length > 0) {
                var list = this.nameTable[this.selectionList[0].name].children;
                var fNode = this.nameTable[this.selectionList[0].children[0]];
                var fNodeBounds = ej.datavisualization.Diagram.Util.bounds(fNode);
                for (var i = 0; i < list.length; i++) {
                    var element = this.nameTable[list[i]];
                    if (!(element.segments)) {
                        element.width = fNodeBounds.width;
                        this._updateQuad(this, element);
                        ej.datavisualization.Diagram.DiagramContext.update(element, this);
                    }
                }
                ej.datavisualization.Diagram.Util._updateGroupBounds(this.selectionList[0], this.nameTable);
                ej.datavisualization.Diagram.SvgContext.updateSelector(this.selectionList[0], this._svg, this._currZoom, this, this.model.selectorConstraints);
            }
        },
        /**
        *Update vertical space between the object is set with selection boundary
        *@return jQuery
        *@example
        
        *&lt;div id="diagramcontent"&gt;&lt;/div&gt;
        *&lt;script&gt;
        *var diagram=$("#diagramcontent").ejDiagram("instance");
        *diagram.spaceDown(); 
        *&lt;/script&gt;
        *@memberof ejDiagram
*@instance
        */
        spaceDown: function () {
            if (this.selectionList[0] && this.selectionList[0].isPseudoGroup) {
                var lstSelections = this.selectionList[0].children;
                this._sortByYvalue(this.selectionList[0].children);
                var selectionCount = lstSelections.length, fSpacing;
                if (lstSelections && selectionCount > 2) {
                    var cornerNodes = this._findCornerNodes();
                    var firstNode = cornerNodes.top;
                    var lastNode = cornerNodes.bottom;
                    if (firstNode && lastNode) {
                        var bFirst = ej.datavisualization.Diagram.Util.bounds(firstNode);
                        var bLast = ej.datavisualization.Diagram.Util.bounds(lastNode);
                        fSpacing = (bLast.y - bFirst.y) / (selectionCount - 1);
                        var y = 1;
                        for (var i = 0; i < lstSelections.length; i++) {
                            var inode = this.nameTable[lstSelections[i]];
                            if (inode !== firstNode && inode !== lastNode) {
                                if (!inode.segments) {
                                    inode.offsetY = bFirst.y + (y * fSpacing) + (inode.height / 2);
                                }
                                else {
                                    var dify;
                                    var inodes = ej.datavisualization.Diagram.Util.bounds(inode);
                                    dify = (bFirst.y + (y * fSpacing)) - (inodes.y);
                                    this._translate(inode, 0, dify, this.nameTable);
                                }
                                this._updateQuad(this, inode);
                                y += 1;
                                ej.datavisualization.Diagram.DiagramContext.update(inode, this);
                            }
                        }
                    }
                }
            }
        },
        /**
        *Update horizontal space between the object is set with selection boundary
        *@return jQuery
        *@example
        
        *&lt;div id="diagramcontent"&gt;&lt;/div&gt;
        *&lt;script&gt;
        *var diagram=$("#diagramcontent").ejDiagram("instance");
        *diagram.spaceAcross(); 
        *&lt;/script&gt;
        *@memberof ejDiagram
*@instance
        */
        spaceAcross: function () {
            if (this.selectionList[0] && this.selectionList[0].isPseudoGroup) {
                var lstSelections = this.selectionList[0].children;
                this._sortByXvalue(this.selectionList[0].children);
                var selectionCount = lstSelections.length, fSpacing;
                if (lstSelections && selectionCount > 2) {
                    var cornerNodes = this._findCornerNodes();
                    var firstNode = cornerNodes.left;
                    var lastNode = cornerNodes.right;
                    if (firstNode && lastNode) {
                        var bFirst = ej.datavisualization.Diagram.Util.bounds(firstNode);
                        var bLast = ej.datavisualization.Diagram.Util.bounds(lastNode);
                        fSpacing = (bLast.x - bFirst.x) / (selectionCount - 1);
                        var x = 1;
                        for (var i = 0; i < lstSelections.length; i++) {
                            var inode = this.nameTable[lstSelections[i]];
                            if (inode !== firstNode && inode !== lastNode) {
                                if (!inode.segments) {
                                    inode.offsetX = bFirst.x + (x * fSpacing) + (inode.width / 2);
                                }
                                else {
                                    var difx;
                                    var inodes = ej.datavisualization.Diagram.Util.bounds(inode);
                                    difx = (bFirst.x + (x * fSpacing)) - (inodes.x);
                                    this._translate(inode, difx, 0, this.nameTable);
                                }
                                this._updateQuad(this, inode);
                                x += 1;
                                ej.datavisualization.Diagram.DiagramContext.update(inode, this);
                            }
                        }
                    }
                }
            }
        },
        /**
        *Decrease the Z-index value of selected object by 1
        *@return jQuery
        *@example
        
        *&lt;div id="diagramcontent"&gt;&lt;/div&gt;
        *&lt;script&gt;
        *var diagram=$("#diagramcontent").ejDiagram("instance");
        *diagram.sendBackward(); 
        *&lt;/script&gt;
        *@memberof ejDiagram
*@instance
        */
        sendBackward: function () {
            var ovarLapNodes, i;
            var prevNode;
            if (this.selectionList[0] && !this.selectionList[0].isPseudoGroup) {
                var node = this.selectionList[0];
                if (node) {
                    ovarLapNodes = this._findOverlapNode(node);
                    this._sortByZIndex(ovarLapNodes);
                    for (var k = 0; k < ovarLapNodes.length; k++)
                        if (node.zOrder > ovarLapNodes[k].zOrder) {
                            prevNode = ovarLapNodes[k];
                            break;
                        }
                    var start = node.zOrder;
                    if (prevNode) {
                        var end = prevNode.zOrder;
                        for (i = end; i < start; i++) {
                            if (node.parent != "")
                                this._findObjectByIndex(i, true).zOrder += 1;
                            else
                                this._findObjectByIndex(i).zOrder += 1;
                        }
                        if (node.parent != "") {
                            if (node.parent) {
                                var parent = this.nameTable[node.parent];
                                this._findChildren(parent, node.name).zOrder = end;
                            }
                        } else
                            this._findObjectByName(node.name).zOrder = end;
                        this._svg.getElementById(prevNode.name).parentNode.insertBefore(this._svg.getElementById(node.name), this._svg.getElementById(prevNode.name));
                        ej.datavisualization.Diagram.SvgContext.renderSelector(node, this._svg, this._adornerLayer, this._currZoom,
                        this.model.selectorConstraints);
                    }
                }
            }
        },
        /**
               *Increase the Z-index value of selected object by 1
               *@return jQuery
               *@example
              
               *&lt;div id="diagramcontent"&gt;&lt;/div&gt;
               *&lt;script&gt;
               *var diagram=$("#diagramcontent").ejDiagram("instance");
               *diagram.moveForward(); 
               *&lt;/script&gt;
               *@memberof ejDiagram
*@instance
               */
        moveForward: function () {
            var ovarLapNodes, i;
            var prevNode;
            if (this.selectionList[0] && !this.selectionList[0].isPseudoGroup) {
                var node = this.selectionList[0];
                if (node) {
                    ovarLapNodes = this._findOverlapNode(node);
                    this._sortByZIndex(ovarLapNodes);
                    for (var k = ovarLapNodes.length - 1; k >= 0; k--)
                        if (node.zOrder < ovarLapNodes[k].zOrder) {
                            prevNode = ovarLapNodes[k];
                            break;
                        }

                    var start = node.zOrder;
                    if (prevNode) {
                        var end = prevNode.zOrder;
                        for (i = end; i > start; i--) {
                            if (node.parent != "")
                                this._findObjectByIndex(i, true).zOrder -= 1;
                            else
                                this._findObjectByIndex(i).zOrder -= 1;
                        }
                        if (node.parent != "") {
                            if (node.parent) {
                                var parent = this.nameTable[node.parent];
                                this._findChildren(parent, node.name).zOrder = end;
                            }
                        }
                        else
                            this._findObjectByName(node.name).zOrder = end;
                        this._svg.getElementById(prevNode.name).parentNode.insertBefore(this._svg.getElementById(node.name), this._svg.getElementById(prevNode.name).nextSibling);
                        ej.datavisualization.Diagram.SvgContext.renderSelector(node, this._svg, this._adornerLayer, this._currZoom,
                        this.model.selectorConstraints);

                    }
                }
            }
        },
        /**
        *Move the selected object behind other object
        *@return jQuery
        *@example
        
        *&lt;div id="diagramcontent"&gt;&lt;/div&gt;
        *&lt;script&gt;
        *var diagram=$("#diagramcontent").ejDiagram("instance");
        *diagram.sendToBack(); 
        *&lt;/script&gt;
        *@memberof ejDiagram
*@instance
        */
        sendToBack: function () {
            var node = this.selectionList[0];
            var prevNode;
            if (node && !node.isPseudoGroup) {
                var ovarLapNodes = this._findOverlapNode(node);
                var nodes = this._sortByZIndex(ovarLapNodes);
                var start = node.zOrder;
                var end = node.zOrder;
                for (var i = 0; i < nodes.length; i++) {
                    if (nodes[i].zOrder < end) {
                        end = nodes[i].zOrder;
                        prevNode = nodes[i];
                    }
                }
                if (prevNode) {
                    for (i = end; i < start; i++) {
                        if (node.parent != "")
                            this._findObjectByIndex(i, true).zOrder += 1;
                        else
                            this._findObjectByIndex(i).zOrder += 1;
                    }
                    if (node.parent != "") {
                        if (node.parent) {
                            var parent = this.nameTable[node.parent];
                            this._findChildren(parent, node.name).zOrder = end;
                        }
                    } else
                        this._findObjectByName(node.name).zOrder = end;
                    this._svg.getElementById(prevNode.name).parentNode.insertBefore(this._svg.getElementById(node.name), this._svg.getElementById(prevNode.name));
                    ej.datavisualization.Diagram.SvgContext.renderSelector(node, this._svg, this._adornerLayer, this._currZoom,
                        this.model.selectorConstraints);
                }
            }
        },
        /**
        *Move the selected object over other object
        *@return jQuery
        *@example
        
        *&lt;div id="diagramcontent"&gt;&lt;/div&gt;
        *&lt;script&gt;
        *var diagram=$("#diagramcontent").ejDiagram("instance");
        *diagram.bringToFront(); 
        *&lt;/script&gt;
        *@memberof ejDiagram
*@instance
        */

        bringToFront: function () {
            var node = this.selectionList[0];
            var prevNode;
            if (node && !node.isPseudoGroup) {
                var ovarLapNodes = this._findOverlapNode(node);
                var nodes = this._sortByZIndex(ovarLapNodes);
                var start = node.zOrder;
                var end = node.zOrder;
                for (var i = 0; i < nodes.length; i++) {
                    if (nodes[i].zOrder > end) {
                        end = nodes[i].zOrder;
                        prevNode = nodes[i];
                    }
                }
                if (prevNode) {
                    for (i = 0; i < nodes.length; i++) {
                        if (nodes[i].zOrder > start) {
                            nodes[i].zOrder -= 1;
                        }
                    }
                    if (node.parent) {
                        var parent = this.nameTable[node.parent];
                        this._findChildren(parent, node.name).zOrder = end;
                    } else
                        this._findObjectByName(node.name).zOrder = end;

                    this._svg.getElementById(prevNode.name).parentNode.insertBefore(this._svg.getElementById(node.name), this._svg.getElementById(prevNode.name).nextSibling);
                    ej.datavisualization.Diagram.SvgContext.renderSelector(node, this._svg, this._adornerLayer, this._currZoom,
                        this.model.selectorConstraints);
                }
            }
        },
        /**
        *Reverse the last action thta was performed
        *@return jQuery
        *@example
        
        *&lt;div id="diagramcontent"&gt;&lt;/div&gt;
        *&lt;script&gt;
        *var diagram=$("#diagramcontent").ejDiagram("instance");
        *diagram.undo(); 
        *&lt;/script&gt;
        *@memberof ejDiagram
*@instance
        */
        undo: function () {
            if (ej.datavisualization.Diagram.Util.isPageEditable(this) && ej.datavisualization.Diagram.Util.canUndo(this)) {
                var args;
                this._isUndo = true;
                //this.historyManager.undo();
                var entryItem = this.getUndoEntry();
                switch (entryItem && entryItem.type) {
                    case "positionchanged":
                        args = { object: entryItem.undoObject, isMultipleNode: entryItem.isMultipleNode };
                        this._recordPinPointChanged(args);
                        break;
                    case "rotationchanged":
                        args = { object: entryItem.undoObject, isMultipleNode: entryItem.isMultipleNode };
                        this._recordRotationChanged(args);
                        break;
                    case "sizechanged":
                        args = { object: entryItem.undoObject, isMultipleNode: entryItem.isMultipleNode };
                        this._recordSizeChanged(args);
                        break;
                    case "endpointchanged":
                        args = { undoObject: entryItem.undoObject, redoObject: entryItem.redoObject, isMultipleNode: entryItem.isMultipleNode, _isUndo: true };
                        this._recordEndPointChanged(args);
                        break;
                    case "collectionchanged":
                        args = { changeType: entryItem.changeType, childTable: entryItem.childTable, edgeTable: entryItem.edgeTable, isUndo: true, sourcePorts: entryItem.sourcePorts, targetPorts: entryItem.targetPorts };
                        args.object = jQuery.extend(true, {}, entryItem.object);
                        this._recordCollectionChanged(args);
                        break;
                    case "labelchanged":
                        args = { object: entryItem.shape, label: entryItem.previousLabelValue };
                        this._recordLabelChanged(args);
                        break;
                    case "groupchanged":
                        args = { object: entryItem.object, isUndo: true, actionType: entryItem.actionType };
                        this._recordGroupChanged(args);
                        break;
                }

                this._isUndo = false;
            }
        },
        /**
        *Restore the last action thta was performed
        *@return jQuery
        *@example
        
        *&lt;div id="diagramcontent"&gt;&lt;/div&gt;
        *&lt;script&gt;
        *var diagram=$("#diagramcontent").ejDiagram("instance");
        *diagram.redo(); 
        *&lt;/script&gt;
        *@memberof ejDiagram
*@instance
        */
        redo: function () {
            this._isUndo = true;
            //this.historyManager.redo();
            var entryItem = this.getRedoEntry();
            switch (entryItem && entryItem.type) {
                case "positionchanged":
                    var args = { object: entryItem.redoObject, isMultipleNode: entryItem.isMultipleNode };
                    this._recordPinPointChanged(args);
                    break;
                case "rotationchanged":
                    args = { object: entryItem.redoObject, isMultipleNode: entryItem.isMultipleNode };
                    this._recordRotationChanged(args);
                    break;
                case "sizechanged":
                    args = { object: entryItem.redoObject, isMultipleNode: entryItem.isMultipleNode };
                    this._recordSizeChanged(args);
                    break;
                case "endpointchanged":
                    args = { undoObject: entryItem.undoObject, redoObject: entryItem.redoObject, isMultipleNode: entryItem.isMultipleNode, _isUndo: false };
                    this._recordEndPointChanged(args);
                case "collectionchanged":
                    args = { changeType: entryItem.changeType, childTable: entryItem.childTable, isUndo: false };
                    args.object = jQuery.extend(true, {}, entryItem.object);
                    this._recordCollectionChanged(args);
                    break;
                case "labelchanged":
                    args = { object: entryItem.shape, label: entryItem.updatedLabelValue };
                    this._recordLabelChanged(args);
                    break;
                case "groupchanged":
                    args = { object: entryItem.object, actionType: entryItem.actionType, isUndo: false };
                    this._recordGroupChanged(args);
                    break;
            }
            this._isUndo = false;
        },

        addHistoryEntry: function (entry) {
            if (this._historyList.currentEntry != null) {
                var cEntry = this._historyList.currentEntry;
                if (cEntry.next) {
                    if (cEntry.previous) {
                        var entry1 = cEntry.next;
                        entry1.previous = null;
                        cEntry.next = entry;
                        entry.previous = cEntry;
                    }
                } else {
                    cEntry.next = entry;
                    entry.previous = cEntry;
                }
            }
            this._historyList.currentEntry = entry;
            this._historyList.canUndo = true;
            this._historyList.canRedo = false;
        },

        clearHistory: function () {
            var hList = this._historyList;
            hList.currentEntry = null;
            hList.canUndo = false;
            hList.canRedo = false;
            this._hScrollbar._scrollData = null;
            this._vScrollbar._scrollData = null;
        },

        getUndoEntry: function () {
            var undoEntry = null;
            var currentEntry;
            var hList = this._historyList;
            if (hList.canUndo) {
                undoEntry = hList.currentEntry;
                currentEntry = hList.currentEntry.previous;
                if (currentEntry) {
                    hList.currentEntry = currentEntry;
                    if (!hList.canRedo)
                        hList.canRedo = true;
                } else {
                    hList.canRedo = true;
                    hList.canUndo = false;
                }
            }
            return undoEntry;
        },

        getRedoEntry: function () {
            var redoEntry = null;
            var entryCurrent;
            var hList = this._historyList;
            if (hList.canRedo) {
                if (!hList.currentEntry.previous && !hList.canUndo) {
                    entryCurrent = hList.currentEntry;
                } else
                    entryCurrent = hList.currentEntry.next;
                if (entryCurrent) {
                    hList.currentEntry = entryCurrent;
                    if (!hList.canUndo)
                        hList.canUndo = true;
                    if (!entryCurrent.next) {
                        hList.canRedo = false;
                        hList.canUndo = true;
                    }
                }
                redoEntry = hList.currentEntry;
            }
            return redoEntry;
        },
        /**
        *cut the selected object from diagram to diagram internal clipboard
        *@return jQuery
        *@example
        
        *&lt;div id="diagramcontent"&gt;&lt;/div&gt;
        *&lt;script&gt;
        *var diagram=$("#diagramcontent").ejDiagram("instance");
        *diagram.cut(); 
        *&lt;/script&gt;
        *@memberof ejDiagram
*@instance
        */
        cut: function () {
            this._pasteIndex = 0;
            this._setClipboard();
            this._delete();
        },


        /**
 *Add ports to the node
 *@return jQuery
 *@param {String} name  to identify the object from the model 
 *@example
 
 *&lt;div id="diagramcontent"&gt;&lt;/div&gt;
 *&lt;script&gt;
 *var diagram = $("#diagramcontent").ejDiagram("instance");
 *var port = [{ offset: { x: 0, y: 0.5 }, name: "aport", fillColor: "yellow"}, { offset: { x: 0.5, y: 0.5 }, name: "bport", fillColor: "yellow",  }];
 *diagram.addPorts("Rect1", port);
 *&lt;/script&gt;
 *@memberof ejDiagram
 *@instance
 */
        addPorts: function (name, ports) {
            if (ej.datavisualization.Diagram.Util.isPageEditable(this)) {
                var node = this.nameTable[name];
                if (node && ports) {
                    var port;
                    for (var i = 0; i < ports.length; i++) {
                        port = ej.datavisualization.Diagram.Port(ports[i]);
                        node.ports.push(port);
                        ej.datavisualization.Diagram.DiagramContext.renderPort(node, port, this);
                    }
                }
            }
        },


        /**
        *copy the selected object from internal clipboard
        *@return jQuery
        *@example
        
        *&lt;div id="diagramcontent"&gt;&lt;/div&gt;
        *&lt;script&gt;
        *var diagram=$("#diagramcontent").ejDiagram("instance");
        *diagram.copy(); 
        *&lt;/script&gt;
        *@memberof ejDiagram
*@instance
        */

        copy: function () {
            this._pasteIndex = 1;
            if (this.selectionList && this.selectionList.length > 0) {
                this._setClipboard();
            }
        },
        /**
        *paste the selected object from internal clipboard to diagram
        *@return jQuery
        *@example
        
        *&lt;div id="diagramcontent"&gt;&lt;/div&gt;
        *&lt;script&gt;
        *var diagram=$("#diagramcontent").ejDiagram("instance");
        *diagram.paste(); 
        *&lt;/script&gt;
        *@memberof ejDiagram
*@instance
        */
        paste: function () {
            //this._isUndo = true; 
            this._paste();
            //this._isUndo = false;
        },

        _setClipboard: function () {
            if (this.selectionList.length > 0 && !this.selectionList[0].isSwimlane) {
                var data, node = this.selectionList[0];
                var childTable = {};
                if (node.isGroup || node.isPseudoGroup) {
                    var temp, i, j;
                    for (i = 0; i < node.children.length; i++) {
                        for (j = i + 1; j < node.children.length; j++) {
                            var nodei = this.nameTable[node.children[i]];
                            var nodej = this.nameTable[node.children[j]];
                            if (nodej && nodei) {
                                if (nodej.zOrder < nodei.zOrder) {
                                    temp = node.children[i];
                                    node.children[i] = node.children[j];
                                    node.children[j] = temp;
                                }
                            }
                        }
                    }
                    childTable = this._getChildTable(node, childTable);
                }
                data = $.extend(true, {}, { "childTable": childTable, "node": node });
                if (data.node.segments) {
                    data.node.targetNode = null;
                    data.node.targetPort = null;
                    data.node.sourceNode = null;
                    data.node.sourcePort = null;
                }
                this._preserveHtmlContent(node, data.node);
                if (data.node.children && data.node.children.length > 0) {
                    for (var child in data.childTable) {
                        this._preserveHtmlContent(childTable[child], data.childTable[child]);
                    }
                }
                this._clipboardData = data;
            }
        },

        _preserveHtmlContent: function (node, cloneNode) {
            if (node.shape) {
                if (cloneNode.shape.type === "html") {
                    if (!(typeof (node.shape.html) === 'string'))
                        cloneNode.shape.html = node.shape.html.cloneNode(true);
                }
                else if (cloneNode.shape.type === "native") {
                    cloneNode.shape.content = node.shape.content.cloneNode(true);
                }
            }
        },

        _getChildTable: function (group, childTable) {
            var child, names, name;
            names = group.children;
            if (names) {
                for (var i = 0; i < names.length; i++) {
                    name = names[i];
                    child = this.nameTable[name];
                    if (child && (child.parent === group.name || group.isPseudoGroup)) {
                        var node = childTable[child.name] = $.extend(true, {}, child);
                        if (node.isGroup || node.isPseudoGroup)
                            childTable = this._getChildTable(node, childTable);
                    }
                }
            }
            return childTable;
        },

        _paste: function () {
            if (this._clipboardData) {
                var data = this._clipboardData;
                var dx, child;
                data = $.extend(true, {}, { "childTable": data.childTable, "node": data.node });
                this._preserveHtmlContent(this._clipboardData.node, data.node);
                if (data.node.children && data.node.children.length > 0) {
                    for (child in data.childTable) {
                        this._preserveHtmlContent(this._clipboardData.childTable[child], data.childTable[child]);
                    }
                }
                var node = data.node;
                if (!node.container) {
                    if (node.isPseudoGroup) {
                        var names = node.children;
                        var pseudoGroup = ej.datavisualization.Diagram.Group({ isPseudoGroup: true, "name": "multipleSelection" });
                        this._isUndo = true;
                        for (var i = 0; i < names.length; i++) {
                            var name = names[i];
                            child = data.childTable[name];
                            if (child) {
                                name = name + ej.datavisualization.Diagram.Util.randomId();
                                delete (data.childTable[child.name]);
                                child.name = name;
                                child.parent = "";
                                data.childTable[child.name] = child;
                                this._preserveConnection(data.childTable, child);
                                pseudoGroup.children.push(name);
                                if (child.isGroup) {
                                    this._pasteChildren(data.childTable, child);
                                }
                                dx = this._pasteIndex * 10;
                                this._translate(child, dx, dx, this.nameTable);
                                this.add(child);
                            }
                        }
                        this._clearSelection();
                        this.model.nodes.push(pseudoGroup);
                        this.nameTable[pseudoGroup.name] = pseudoGroup;
                        ej.datavisualization.Diagram.Util._updateGroupBounds(pseudoGroup, this.nameTable);
                        this._addSelection(pseudoGroup);
                        var childTable = {};
                        childTable = this._getChildTable(this.selectionList[0], childTable);
                        var entry = { type: "collectionchanged", object: jQuery.extend(true, {}, this.selectionList[0]), childTable: jQuery.extend(true, {}, childTable), changeType: "insert" };
                        this.addHistoryEntry(entry);
                        this._isUndo = false;
                    } else {
                        node.name = node.name + ej.datavisualization.Diagram.Util.randomId();
                        node.parent = "";
                        if (node.isGroup) {
                            this._pasteChildren(data.childTable, node);
                        } else if (node.segments) {
                            //this._disConnect(node);
                        } else if (!node.segments) {
                            //@Fix
                            //this._disConnect(node);
                            node.inEdges = [];
                            node.outEdges = [];
                        }

                        dx = this._pasteIndex * 10;
                        if (!node.segments)
                            this._translate(node, dx, dx, this.nameTable);
                        else
                            ej.datavisualization.Diagram.Util._translateLine(node, dx, dx);
                        this.add(node);
                        // ej.datavisualization.Diagram.DiagramContext.update(node, this);
                        //this._clearSelection();
                        //this._addSelection(node);
                    }
                }
                else {
                    var child;
                    var name = ej.datavisualization.Diagram.Util.randomId();
                    node.name += name;
                    this._updateChildName(node, name);
                    for (var i in data.childTable) {
                        child = data.childTable[i];
                        child.name = child.name + name;
                        this.nameTable[child.name] = child;
                    }
                    dx = this._pasteIndex * 10;
                    this._translate(node, dx, dx, this.nameTable);

                    if (node.isSwimlane) {
                        node.width = 0;
                        node.height = 0;
                        this.add(node);
                    }
                    else if (node.isLane) {
                        var swimlane = ej.datavisualization.Diagram.SwimLaneHelper._createSwimlane(node, this, data);
                        if (swimlane) {
                            swimlane.width = 0;
                            swimlane.height = 0;
                            this.add(swimlane);
                        }
                    }
                }
                this._pasteIndex++;
            }
        },

        _updateChildName: function (node, name) {
            if (node && name) {
                for (var i = 0; i < node.children.length; i++) {
                    node.children[i] = node.children[i] + name;
                }
            }
        },

        _pasteChildren: function (childTable, group) {
            var child, names, name;
            var childNames = [];
            names = group.children;
            for (var i = 0; i < names.length; i++) {
                name = names[i];
                child = childTable[name];
                if (child) {
                    name = name + ej.datavisualization.Diagram.Util.randomId();
                    delete (childTable[child.name]);
                    child.name = name;
                    childTable[child.name] = child;
                    this._preserveConnection(childTable, child);
                    child.parent = group.name;
                    childNames.push(name);
                    if (child.isGroup) {
                        this._pasteChildren(childTable, child);
                    }
                    this.add(child);
                }
            }
            group.children = childNames;
            this._preserveConnection(childTable, group);
        },

        _preserveConnections: function (childTable, group) {
            var child, names;
            names = group.children;
            for (var i = 0; i < names.length; i++) {
                child = childTable[names[i]];
                if (child) {
                    this._preserveConnection(childTable, child);
                    if (child.isGroup)
                        this._preserveConnections(childTable, group);
                }
            }
        },

        _preserveConnection: function (childTable, child) {
            var i, connection, len;
            if (!child.segments) {
                len = child.inEdges.length;
                if (len > 0) {
                    for (i = 0; i < len; i++) {
                        connection = childTable[child.inEdges[i]];
                        if (connection) {
                            connection.targetNode = child.name;
                        }
                    }
                    child.inEdges = [];
                }
                len = child.outEdges.length;
                if (len > 0) {
                    for (i = 0; i < child.outEdges.length; i++) {
                        connection = childTable[child.outEdges[i]];
                        if (connection) {
                            connection.sourceNode = child.name;
                        }
                    }
                    child.outEdges = [];
                }
            } else {
                var node;
                if (child.targetNode) {
                    node = childTable[child.targetNode];
                    if (node)
                        node.inEdges.push(child.name);
                    //else
                    //    child.targetNode = null;
                }
                if (child.sourceNode) {
                    node = childTable[child.sourceNode];
                    if (node)
                        node.outEdges.push(child.name);
                    //else
                    //    child.sourceNode = null;
                }
            }
        },

        _expandChildren: function (node) {
            for (var i = 0; i < node.outEdges.length; i++) {
                var conn = this.nameTable[node.outEdges[i]];
                conn.visible = true;
                var target = this.nameTable[conn.targetNode];
                if (target.isExpanded)
                    this._expandChildren(target);
                target.visible = true;
                target.excludeFromLayout = false;
            }
        },

        _collapseChildren: function (node) {
            for (var i = 0; i < node.outEdges.length; i++) {
                var conn = this.nameTable[node.outEdges[i]];
                conn.visible = false;
                var target = this.nameTable[conn.targetNode];
                if (target.isExpanded)
                    this._collapseChildren(target);
                target.visible = false;
                target.excludeFromLayout = true;
                target.offsetX = 0;
                target.offsetY = 0;
            }
        },
        /**
        *Used to zoomIn/zoomOut diagram
        *@return jQuery
        *@param {Object} zoom pass zoom commands of the diagram
        *@example
        
        *&lt;div id="diagramcontent"&gt;&lt;/div&gt;
        *&lt;script&gt;
        *var diagram=$("#diagramcontent").ejDiagram("instance");
        *var zoom =  ej.datavisualization.Diagram.Zoom();
        *zoom.zoomFactor = .1;
        *zoom.zoomCommand = ej.datavisualization.Diagram.ZoomCommand.ZoomIn;
        *diagram.zoomTo(zoom);
        *&lt;/script&gt;
        *@memberof ejDiagram
*@instance
        */
        zoomTo: function (zoom) {
            if (!zoom) {
                zoom = new ej.datavisualization.Diagram.Zoom();
            }
            if (zoom.zoomCommand & ej.datavisualization.Diagram.ZoomCommand.ZoomIn) {
                ej.datavisualization.Diagram.ZoomUtil.zoomPan(this, (1 + zoom.zoomFactor), 0, 0, zoom.focusPoint, true);
            } else {
                ej.datavisualization.Diagram.ZoomUtil.zoomPan(this, (1 / (1 + zoom.zoomFactor)), 0, 0, zoom.focusPoint, true);
            }
        },
        /**
        *Removes either given node/connector or the selected element from diagram
        *@return jQuery
        *@param {Object} node pass node/connector to be removed
        *@example
        
        *&lt;div id="diagramcontent"&gt;&lt;/div&gt;
        *&lt;script&gt;
        *var diagram=$("#diagramcontent").ejDiagram("instance");
        *diagram.remove(); 
        *&lt;/script&gt;
        *@memberof ejDiagram
*@instance
        */
        remove: function (node) {
            if (node && this.nameTable[node.name] && ej.datavisualization.Diagram.Util.isPageEditable(this)) {
                this._remove(node);
            }
            else
                this._delete();
        },
        /**
               *Remove all the elements from diagram
               *@return jQuery
               *@example
               
               *&lt;div id="diagramcontent"&gt;&lt;/div&gt;
               *&lt;script&gt;
               *var diagram=$("#diagramcontent").ejDiagram("instance");
               *diagram.clear(); 
               *&lt;/script&gt;
               *@memberof ejDiagram
               *@instance
               */
        clear: function () {
            this._clearSelection();
            var i;
            for (i = this.model.connectors.length - 1; i >= 0; i--) {
                this._remove(this.model.connectors[i]);
            }
            for (i = this.model.nodes.length - 1; i >= 0; i--) {
                this._remove(this.model.nodes[i]);
            }
            this.clearHistory();
            $("#" + this._canvas.id + "_hScrollbar").ejScrollbar({ width: 0 });
            $("#" + this._canvas.id + "_vScrollbar").ejScrollbar({ height: 0 });
        },

        _add: function (node, render, arg) {
            var success = false;
            if (node.length > 0) {
                for (var i = 0; i < node.length; i++) {
                    this.add(node[i]);
                }
            } else {
                if (node.segments) var type = "connector"; else type = "node";
                var args = { changeType: "insert", element: node, cancel: false };
                this._raiseEvent(type + "CollectionChange", args);
                if (!args.cancel) {
                    this._setZorder(node);
                    if (!node.segments) {
                        if (node.isGroup)
                            node = ej.datavisualization.Diagram.Group(node);
                        else {
                            if (!(this.activeTool && this.activeTool.name === "textTool"))
                                node = this._getNewNode(node);
                        }
                        success = this.model.nodes.push(node);
                        this.nameTable[node.name] = node;
                        if (success) {
                            if (node.isGroup) {
                                this._updateParent(node);
                                if (node.offsetX == 0 && node.offsetY == 0)
                                    ej.datavisualization.Diagram.Util._updateGroupBounds(node, this.nameTable);
                                this._updateChildrenEdges(node);
                                if (render)
                                    ej.datavisualization.Diagram.DiagramContext.renderGroup(node, this);
                            } else {
                                ej.datavisualization.Diagram.SpatialUtil._updateQuad(this, this._spatialSearch, node);
                                if (render && !node.parent)
                                    ej.datavisualization.Diagram.DiagramContext.renderNode(node, this);
                            }
                            this.nameTable[node.name] = node;
                            if (render) {
                                this._clearSelection();
                                this._addSelection(node);
                            }
                        }
                    } else {
                        node = this._getNewConnector(node);
                        success = this.model.connectors.push(node);
                        if (success) {
                            ej.datavisualization.Diagram.SpatialUtil._updateQuad(this, this._spatialSearch, node);
                            this._updateEdges(node);
                            this._dock(node, this.nameTable);
                            ej.datavisualization.Diagram.Util.updateBridging(node, this);
                            if (!node.parent) {
                                if (render)
                                    ej.datavisualization.Diagram.DiagramContext.renderConnector(node, this);
                            }
                            this.nameTable[node.name] = node;
                            if (render) {
                                this._clearSelection();
                                this._addSelection(node);
                            }
                        }
                    }
                }
            }
            if (!this._isUndo && !this._selectedSymbol) {
                if (node.parent == "") {
                    var childTable = {};
                    if (this.selectionList && this.selectionList.length > 0)
                        childTable = this._getChildTable(this.selectionList[0], childTable);
                    var entry = { type: "collectionchanged", object: jQuery.extend(true, {}, node), childTable: jQuery.extend(true, {}, childTable), changeType: "insert" };
                    this.addHistoryEntry(entry);
                }
            }
            return success;
        },
        /**
              *Add node/connector
              *@return jQuery
              *@param {Object} node pass node/connector to be added
              *@example
              
              *&lt;div id="diagramcontent"&gt;&lt;/div&gt;
              *&lt;script&gt;
              *var diagram=$("#diagramcontent").ejDiagram("instance");
              *var node = [];
              *node=[{ name: "node", width: 175, height: 60, offsetX: 400, offsetY: 60, shape: { type: "Rectangle"}];
              *diagram.add(node);
              *&lt;/script&gt;
              *@memberof ejDiagram
              *@instance
              */
        add: function (node, args) {
            if (ej.datavisualization.Diagram.Util.isPageEditable(this)) {
                return this._add(node, true, args);
            }
        },
        /**
           *Update userhandles of given node
           *@return jQuery
            *@param {Object} node pass node/connector 
           *@example
          
           *&lt;div id="diagramcontent"&gt;&lt;/div&gt;
           *&lt;script&gt;
           *var diagram=$("#diagramcontent").ejDiagram("instance");
           * var node = diagram.selectionList[0];
           * diagram.updateUSerHandles(node); 
           *&lt;/script&gt;
           *@memberof ejDiagram
*@instance
           */
        updateUserHandles: function (node) {
            ej.datavisualization.Diagram.SvgContext.updateUserHandles(this.model.userHandles, node, this._svg, false, false, this._currZoom);
        },
        /**
           *Update offsetX of given node
           *@return jQuery
            *@param {Object} node pass node/connector to be added
           *@example
          
           *&lt;div id="diagramcontent"&gt;&lt;/div&gt;
           *&lt;script&gt;
           *var diagram=$("#diagramcontent").ejDiagram("instance");
           * var node = diagram.selectionList[0];
           * diagram.scrollToNode(node); 
           *&lt;/script&gt;
           *@memberof ejDiagram
*@instance
           */
        scrollToNode: function (node) {
            var nodeX, nodeY;
            var curZoomfactor = this._currZoom;
            var elementBounds = this.element[0].getBoundingClientRect();
            var nodeBounds = ej.datavisualization.Diagram.Util.bounds(node);
            nodeX = (nodeBounds.x + nodeBounds.width / 2) * curZoomfactor;
            nodeY = (nodeBounds.y + nodeBounds.height / 2) * curZoomfactor;
            var hScrollOffset = this._hScrollOffset, vScrollOffset = this._vScrollOffset;
            if (elementBounds.width + this._hScrollOffset < nodeX + nodeBounds.width / 2) {
                hScrollOffset = (nodeX + (nodeBounds.width * curZoomfactor) - elementBounds.width);
            }
            if (elementBounds.height + this._vScrollOffset < nodeY + nodeBounds.height / 2) {
                vScrollOffset = (nodeY + (nodeBounds.height * curZoomfactor) - elementBounds.height);
            }
            this._updateScrollOffset(hScrollOffset, vScrollOffset);
        },
        //#endregion

        //#region support methods

        _getElementType: function (element) {
            if (element) {
                if (element.isSwimlane)
                    return "swimlane";
                else if (element.isGroup)
                    return "group";
                else if (element.segments)
                    return "connector";
                else if (element.shape)
                    return "node";
                else return null;
            }
            return null;
        },

        _registerDrawingTools: function () {
            for (var toolname in this.model.drawingTools) {
                var tool = this.model.drawingTools[toolname];
                tool.diagram = this;
                this.tools[tool.name] = tool;
            }
        },

        _sortByXvalue: function (nodes) {
            var temp, i, j;
            var fNode, lNode;
            for (i = 0; i < nodes.length; i++) {
                for (j = i + 1; j < nodes.length; j++) {
                    var nodei = this.nameTable[nodes[i]];
                    var nodej = this.nameTable[nodes[j]];
                    fNode = ej.datavisualization.Diagram.Util.bounds(nodei);
                    lNode = ej.datavisualization.Diagram.Util.bounds(nodej);
                    if (fNode.x > lNode.x) {
                        temp = nodes[i];
                        nodes[i] = nodes[j];
                        nodes[j] = temp;
                    }
                }
            }
            return nodes;
        },

        _sortByYvalue: function (nodes) {
            var temp, i, j;
            var fNode, lNode;
            for (i = 0; i < nodes.length; i++) {
                for (j = i + 1; j < nodes.length; j++) {
                    var nodei = this.nameTable[nodes[i]];
                    var nodej = this.nameTable[nodes[j]];
                    fNode = ej.datavisualization.Diagram.Util.bounds(nodei);
                    lNode = ej.datavisualization.Diagram.Util.bounds(nodej);
                    if (fNode.y > lNode.y) {
                        temp = nodes[i];
                        nodes[i] = nodes[j];
                        nodes[j] = temp;
                    }
                }
            }
            return nodes;
        },

        _sortByZIndex: function (nodes, asc) {
            var temp, i, j;
            if (!asc) {
                for (i = 0; i < nodes.length; i++) {
                    for (j = i + 1; j < nodes.length; j++) {
                        if (nodes[i].zOrder < nodes[j].zOrder) {
                            temp = nodes[i];
                            nodes[i] = nodes[j];
                            nodes[j] = temp;
                        }
                    }

                }
            }
            else {
                for (i = 0; i < nodes.length; i++) {
                    for (j = i + 1; j < nodes.length; j++) {
                        if (nodes[i].zOrder > nodes[j].zOrder) {
                            temp = nodes[i];
                            nodes[i] = nodes[j];
                            nodes[j] = temp;
                        }
                    }

                }
            }
            return nodes;
        },

        _findCornerNodes: function () {
            var lNode, rNode, tNode, bNode, i, nBounds;
            var list = this.selectionList[0].children;
            var fnode = this.nameTable[this.selectionList[0].children[0]];
            lNode = fnode, rNode = fnode, tNode = fnode, bNode = fnode;

            if (list.length > 1) {
                for (i = 0; i < list.length; i++) {
                    var node = this.nameTable[list[i]];
                    nBounds = ej.datavisualization.Diagram.Util.bounds(node);

                    var rNodeBounds = ej.datavisualization.Diagram.Util.bounds(rNode);    //Right Node
                    if ((nBounds.x + nBounds.width) > (rNodeBounds.x + rNodeBounds.width))
                        rNode = node;

                    var bNodeBounds = ej.datavisualization.Diagram.Util.bounds(bNode);    //Bottom Node
                    if ((nBounds.y + nBounds.height) > (bNodeBounds.y + bNodeBounds.height))
                        bNode = node;

                    var lNodeBounds = ej.datavisualization.Diagram.Util.bounds(lNode);    //Left Node
                    if (nBounds.x < lNodeBounds.x)
                        lNode = node;

                    var tNodeBounds = ej.datavisualization.Diagram.Util.bounds(tNode);    //top Node
                    if (nBounds.y < tNodeBounds.y)
                        tNode = node;

                }
            }
            return { left: lNode, right: rNode, top: tNode, bottom: bNode };
        },

        _moveNode: function (diffX, diffY, node) {
            this._translate(node, diffX, diffY, this.nameTable);
            ej.datavisualization.Diagram.DiagramContext.update(node, this);
        },

        _nudge: function (node, x, y) {
            this.activateTool("move");
            this.activeTool.inAction = true;
            this._translate(node, x, y, this.nameTable);
            ej.datavisualization.Diagram.DiagramContext.update(node, this);
            this._updateSelectionHandle(true);
            //ej.datavisualization.Diagram.SvgContext.updateSelector(node, this._svg, this._currZoom, this, this.model.selectorConstraints);
        },

        _findObjectByName: function (name) {
            var node = null;
            if (this._findNode(name))
                node = this._findNode(name);
            if (this._findConnector(name))
                node = this._findConnector(name);
            return node;
        },

        _findObjectByIndex: function (index, group) {
            var nodes, findNode = null, node, m, k;
            if (group) {
                var parent = this.nameTable[this.selectionList[0].parent];
                if (parent) {
                    nodes = parent.children;
                    for (k = 0; k < nodes.length; k++) {
                        node = this.nameTable[nodes[k]];
                        if (node.zOrder === index)
                            findNode = nodes[k];
                    }
                }
            } else {
                for (m = 0; m < this.model.nodes.length; m++) {
                    node = this.model.nodes[m];
                    if (node.zOrder === index)
                        findNode = node;
                }
                for (m = 0; m < this.model.connectors.length; m++) {
                    node = this.model.connectors[m];
                    if (node.zOrder === index)
                        findNode = node;
                }
            }

            return findNode;
        },

        _findOverlapNode: function (node) {
            var oNodes = [], m, i, rNodes = [];
            if (!node.parent) {
                for (m = 0; m < this.model.nodes.length; m++) {
                    oNodes.push(this.model.nodes[m]);
                }
                for (m = 0; m < this.model.connectors.length; m++) {
                    oNodes.push(this.model.connectors[m]);
                }
                var nBound = ej.datavisualization.Diagram.Util.bounds(node);
                if (node) {
                    for (i = 0; i < oNodes.length; i++) {
                        var chBound = ej.datavisualization.Diagram.Util.bounds(oNodes[i]);
                        if (node.name !== oNodes[i].name) {
                            if (this._intersect(nBound, chBound))
                                rNodes.push(oNodes[i]);
                        }
                    }
                }
            } else {
                var parent = this.nameTable[node.parent], child;
                if (parent) {
                    for (m = 0; m < parent.children.length; m++) {
                        if (parent.children[m] !== node.name) {
                            child = this.nameTable[parent.children[m]];
                            if (child)
                                rNodes.push(child);
                        }
                    }
                }
            }
            return rNodes;
        },

        _intersect: function (a, b) {
            return (a.x <= (b.x + b.width) &&
                b.x <= (a.x + a.width) &&
                a.y <= (b.y + b.height) &&
                b.y <= (a.y + a.height));
        },
        //#endregion 

        //#region Rendering
        _createTooltipDiv: function () {
            var div = document.createElement("div");
            var attr = {
                "id": this.element[0].id + "_tooltipDiv",
                "class": "e-diagram-tooltip-default", "style": "padding-top:3px; height: 24px;pointer-events:none;",
            };
            ej.datavisualization.Diagram.Util.attr(div, attr);
            this._canvas.appendChild(div);
            return div;
        },

        _renderTooltip: function (object) {
            if (!(object.parent && this.nameTable[object.parent] && this.nameTable[object.parent].container)) {
                if (this.showTooltip() && $.templates) {
                    var tooltipDiv = $("#" + this.element[0].id + "_tooltipDiv")[0];
                    if (!tooltipDiv) {
                        tooltipDiv = this._createTooltipDiv();
                    }
                    var template;
                    var objectValues;
                    var html;

                    var curZoomfactor = this._currZoom;
                    if (!this.model.tooltipTemplateId) {
                        template = $.templates("#" + this.element[0].id + "_defaultTooltip");
                        if (this.activeTool instanceof ej.datavisualization.Diagram.MoveTool) {
                            tooltipDiv.style.width = "150px";
                            if (!object.segments)
                                objectValues = [{ type: "X : ", type1: "Y : ", x: Math.round((object.offsetX - object.width / 2)) + "px  ", y: Math.round((object.offsetY - object.height / 2)) + "px" }];
                        } else if (this.activeTool instanceof ej.datavisualization.Diagram.ResizeTool) {
                            tooltipDiv.style.width = "150px";
                            objectValues = [{ type: "W : ", type1: "H : ", x: Math.round(object.width) + "px  ", y: Math.round(object.height) + "px" }];
                        } else if (this.activeTool instanceof ej.datavisualization.Diagram.RotateTool) {
                            tooltipDiv.style.width = "50px";
                            objectValues = [{ type: "", type1: "", x: Math.round(object.rotateAngle) + "&deg; ", y: "" }];
                        }
                        if (objectValues) {
                            tooltipDiv.style.top = -this._vScrollOffset + ((object.offsetY * curZoomfactor - (object.height / 2) * curZoomfactor) + (object.height * curZoomfactor) + 15) + "px";
                            tooltipDiv.style.left = -this._hScrollOffset + ((object.offsetX * curZoomfactor - (object.width / 2) * curZoomfactor)) + (object.width / 2 * curZoomfactor - $(tooltipDiv).width() / 2) + "px";
                            html = template.render(objectValues);
                            $(tooltipDiv).html(html);
                        } else {
                            this._removeTooltip();
                        }
                    } else {
                        if (!object.segments) {
                            tooltipDiv.style.top = -this._vScrollOffset + ((object.offsetY * curZoomfactor - (object.height / 2) * curZoomfactor) + (object.height * curZoomfactor) + 15) + "px";
                            tooltipDiv.style.left = -this._hScrollOffset + ((object.offsetX * curZoomfactor - (object.width / 2) * curZoomfactor)) + (object.width / 2 * curZoomfactor - $(tooltipDiv).width() / 2) + "px";
                            template = $.templates("#" + this.model.tooltipTemplateId);
                            objectValues = [{ name: this.activeTool.selectedObject.name, x: (object.offsetX - object.width / 2) + "px", y: (object.offsetY - object.height / 2) + "px", width: object.width + "px", height: object.height + "px", rotationAngle: object.rotateAngle }];
                            html = template.render(objectValues);
                            $(tooltipDiv).html(html);
                        }
                    }
                }
            }
        },

        _removeTooltip: function () {
            var tooltipDiv = document.getElementById(this.element[0].id + "_tooltipDiv");
            if (tooltipDiv) {
                tooltipDiv.parentNode.removeChild(tooltipDiv);
            }
        },

        _renderCanvas: function () {
            this._canvas = document.createElement("div");
            this._canvas.setAttribute("id", this.element[0].id + "_canvas");
            this._canvas.setAttribute("class", "drawing");
            this._canvas.setAttribute("style", "position:relative; height:" + this.model.height + "; width:" + this.model.width + ";");
            this.element.css({
                overflow: "hidden", outline: "none", display: "block", height: this.model.height, width: this.model.width
            });
            this._views[this._id].canvas = this._canvas;
            this.element[0].setAttribute("tabindex", 0);
            this.element.append(this._canvas);
            this._renderDiagram();
        },

        _getDigramBounds: function () {
            var left, top, right, bottom;
            left = Math.min(0, this._spatialSearch.pageLeft);
            top = Math.min(0, this._spatialSearch.pageTop);
            if (!this._pageWidth()) {
                right = this._spatialSearch.pageRight - Math.min(0, this._spatialSearch.pageLeft);
            }
            if (!this._pageHeight()) {
                bottom = this._spatialSearch.pageBottom - Math.min(0, this._spatialSearch.pageTop);
            }
            if (this.model.pageSettings && this._multiplePage()) {
                left = Math.min(0, this._spatialSearch.pageLeft);
                top = Math.min(0, this._spatialSearch.pageTop);
                if (this._pageWidth()) {
                    left = Math.floor(left / this._pageWidth()) * this._pageWidth();
                    right = Math.ceil(this._spatialSearch.pageRight / this._pageWidth()) * this._pageWidth() - left;
                }
                if (this._pageHeight()) {
                    top = Math.floor(top / this._pageHeight()) * this._pageHeight();
                    bottom = Math.ceil(this._spatialSearch.pageBottom / this._pageHeight()) * this._pageHeight() - top;
                }
                if (this._spatialSearch.pageRight === null) {
                    if (this._pageWidth() !== 0) {
                        right = this._pageWidth();
                    }
                }
                if (this._spatialSearch.pageBottom === null) {
                    if (this._pageHeight() !== 0) {
                        bottom = this._pageHeight();
                    }
                }
            }
            else {
                if (this._pageWidth()) {
                    left = Math.min(0, this._spatialSearch.pageLeft);
                    right = this._pageWidth();
                }
                if (this._pageHeight()) {
                    top = Math.min(0, this._spatialSearch.pageTop);

                    bottom = this._pageHeight();
                }
            }
            var svgBounds = new ej.datavisualization.Diagram.Rectangle();
            svgBounds.x = left;
            svgBounds.y = top;
            svgBounds.width = right;
            svgBounds.height = bottom;
            return svgBounds;
        },

        _initContextMenu: function () {
            if (!this.model.contextMenu.showCustomMenuItemsOnly) {
                this.model.contextMenu.items = $.extend([], this._defaultContextMenuItems, this.model.contextMenu.items);
            }
            if (this.enableContextMenu()) {
                this._renderContextMenu();
            }
            else {
                $("#" + this.element[0].id + "_contextMenu").remove();
            }
        },

        _renderContextMenuItem: function (item) {
            var li = document.createElement("li");
            var a = document.createElement("a");
            var menu = ej.datavisualization.Diagram.Locale[this.model.locale];
            $.each(menu, function (otext, ntext) {
                if (item.name === otext) {
                    a.innerHTML = ntext;
                    item.text = ntext;
                }
            });
            li.setAttribute("id", this.element[0].id + "_contextMenu_" + item.name);

            var div = document.createElement("span");
            div.setAttribute("id", item.text + "_image");
            div.setAttribute("style", "margin-top:-5px;");
            div.setAttribute("class", "sf-d-menuitem " + item.name);
            a.appendChild(div);
            if (item.subItems && item.subItems.length > 0) {
                var subUl = document.createElement("ul");
                var i = 0, subli;
                for (i; i < item.subItems.length; i++) {
                    subli = this._renderContextMenuItem(item.subItems[i], subUl);
                    subUl.appendChild(subli);
                }
                li.appendChild(subUl);
            }
            li.appendChild(a);
            return li;
        },

        _renderContextMenu: function () {
            var menuCollection = this.model.contextMenu.items;
            var i = 0, item, mItem;
            var ul = document.createElement("ul");
            ul.setAttribute("id", this.element[0].id + "_contextMenu");
            for (i; i < menuCollection.length; i++) {
                item = menuCollection[i];
                mItem = this._renderContextMenuItem(item, ul);
                ul.appendChild(mItem);
            }
            this._canvas.appendChild(ul);
            $("#" + this.element[0].id + "_contextMenu").ejMenu({
                menuType: ej.MenuType.ContextMenu,
                contextMenuTarget: "#" + this.element[0].id + "_canvas_svg",
                click: $.proxy(this._onMenuItemClick, this),
                beforeOpen: $.proxy(this._onContextMenuOpen, this),
                mouseover: $.proxy(this._onMenuItemMouseOver, this),
                mouseout: $.proxy(this._onMenuItemMouseOut, this),
                close: $.proxy(this._onContextMenuClose, this),
            });
        },

        _onContextMenuClose: function () {
            this.element[0].focus();
        },

        _onContextMenuOpen: function (args) {
            var diagramId = this.element[0].id;
            if (this.enableContextMenu()) {
                document.getElementById(diagramId + "_contextMenu_cut").style.display = "block";
                document.getElementById(diagramId + "_contextMenu_copy").style.display = "block";
                document.getElementById(diagramId + "_contextMenu_paste").style.display = "block";
                document.getElementById(diagramId + "_contextMenu_undo").style.display = "block";
                document.getElementById(diagramId + "_contextMenu_redo").style.display = "block";
                document.getElementById(diagramId + "_contextMenu_selectAll").style.display = "block";
                document.getElementById(diagramId + "_contextMenu_cut").style.display = "block";
                document.getElementById(diagramId + "_contextMenu_grouping").style.display = "block";
                document.getElementById(diagramId + "_contextMenu_order").style.display = "block";
                if (this.selectionList.length == 0) {
                    document.getElementById(diagramId + "_contextMenu_cut").style.display = "none";
                    document.getElementById(diagramId + "_contextMenu_copy").style.display = "none";
                    document.getElementById(diagramId + "_contextMenu_cut").style.display = "none";
                    document.getElementById(diagramId + "_contextMenu_grouping").style.display = "none";
                    document.getElementById(diagramId + "_contextMenu_order").style.display = "none";
                }
            }
            var obj = ({ diagram: this, contextmenu: args });
            this._raiseEvent(this.model.contextMenu.beforeOpen, obj);
        },

        _onMenuItemClick: function (args) {
            var menuId = args.events.ID.replace(this._id + "_" + "contextMenu_", "");
            switch (menuId) {
                case "cut":
                    this.cut();
                    break;
                case "copy":
                    this.copy();
                    break;
                case "paste":
                    this.paste();
                    break;
                case "undo":
                    this.undo();
                    break;
                case "redo":
                    this.redo();
                    break;
                case "selectAll":
                    this.selectAll();
                    break;
                case "group":
                    this.group();
                    break;
                case "ungroup":
                    this.ungroup();
                    break;
                case "bringToFront":
                    this.bringToFront();
                    break;
                case "sendToBack":
                    this.sendToBack();
                    break;
                case "moveForward":
                    this.moveForward();
                    break;
                case "sendBackward":
                    this.sendBackward();
                    break;
                default:
                    break;
            }
            this._raiseEvent(this.model.contextMenu.click, args);
        },

        _onMenuItemMouseOver: function (args) {
            var t = args.events.element.lastChild.childNodes[1];
            $(t).addClass("hover");
        },

        _onMenuItemMouseOut: function (args) {
            var t = args.events.element.lastChild.childNodes[1];
            $(t).removeClass("hover");
        },

        _renderDiagram: function () {
            var diagram = this;
            this._views.forEach(function (viewid) {
                var view = diagram._views[viewid];
                view.context._renderDocument(view, diagram); if (view.type == "mainview" && view.context == ej.datavisualization.Diagram.SvgContext) {
                    diagram._createScrollbar(diagram._canvas);
                    ej.datavisualization.Diagram.ScrollUtil._initScrollbar(diagram);
                    ej.datavisualization.Diagram.PageUtil._updatePageSize(diagram);
                }
                //if (diagram.model.layout && diagram.model.layout.type !== ej.datavisualization.Diagram.LayoutTypes.None) {
                //    diagram._doLayout();
                //};
                diagram._renderNodes(view);
                diagram._renderConnectors(view);
                diagram._renderphases(view);
            });
            $(this._svg.document).pinchDiagram(this._view, this);
        },

        _createScrollbar: function (canvas) {
            var hScrollbar = document.createElement("div");
            hScrollbar.setAttribute("id", canvas.id + "_hScrollbar");
            canvas.appendChild(hScrollbar);
            var vScrollbar = document.createElement("div");
            vScrollbar.setAttribute("id", canvas.id + "_vScrollbar");
            canvas.appendChild(vScrollbar);
        },

        _doLayout: function () {
            if (this.model.layout)// && this.model.layout.type !== ej.datavisualization.Diagram.LayoutTypes.None)
            {
                //this.model.layout = new ej.datavisualization.Diagram.HierarchicalLayout(this.model.layout);
                //this.model.layout._setModel(this);
                this._updateEdgeCollection();
                ej.datavisualization.Diagram.Layout.doLayout(this);
                //this.model.layout.doLayout(this);
            }
        },

        _updateNodes: function () {
            var nodes = this.model.nodes;
            for (var i = 0; i < nodes.length; i++) {
                ej.datavisualization.Diagram.DiagramContext.update(nodes[i], this);
            }
        },

        _updateConnectors: function () {
            var connections = this.model.connectors;
            for (var i = 0; i < connections.length; i++) {
                ej.datavisualization.Diagram.DiagramContext.update(connections[i], this);
            }
        },

        _updateEdgeCollection: function () {
            for (var i = 0; i < this.model.connectors.length; i++) {
                this._updateEdges(this.model.connectors[i]);
            }
        },
        /**
        *To update viewport when window resize
        *@return jQuery
        *@example
        
        *&lt;div id="diagramcontent"&gt;&lt;/div&gt;
        *&lt;script&gt;
        *var diagram=$("#diagramcontent").ejDiagram("instance");
        *diagram.updateViewPort();
        *&lt;/script&gt;
        *@memberof ejDiagram
*@instance
        */

        updateViewPort: function () {
            ej.datavisualization.Diagram.DiagramContext.updateViewPort(this);
        },

        _setZorder: function (node) {
            if (node) {
                node.zOrder = this._zOrder;
                this._zOrder++;
                if (node.isGroup) {
                    var i = 0;
                    if (node.children > 0) {
                        for (i; i < node.children.length; i++) {
                            this.nameTable[node.children[i]].zOrder = i;
                        }
                    }
                }
            }
        },

        _union: function (rect1, rect2) {
            var x = Math.min(rect1.x, rect2.x);
            var y = Math.min(rect1.y, rect2.y);
            var overWidth = Math.max(rect1.x + rect1.width, rect2.x + rect2.width);
            var overheight = Math.max(rect1.y + rect1.height, rect2.y + rect2.height);
            return new ej.datavisualization.Diagram.Rectangle(x, y, overWidth - x, overheight - y);
        },
        _getSeperetor: function (id) {
            var phases = this.model.phases;
            if (phases) {
                for (var i = 0, len = phases.length; i < len; i++) {
                    if (phases[i].name == id) {
                        return phases[i];
                    }
                }
            }
        },
        _renderphases: function () {
            var phases = this.model.phases;
            if (phases) {
                for (var i = 0, len = phases.length; i < len; i++) {
                    if (phases[i].type == "phase") {
                        ej.datavisualization.Diagram.SvgContext.renderphase(phases[i], this._svg, this._diagramLayer, this);
                    }
                }
            }
        },
        _renderNodes: function (view, isLoad, isoverView) {
            if (this.model.nodes) {
                var nodes = this.model.nodes;
                for (var i = 0, len = nodes.length; i < len; ++i) {
                    var node = nodes[i];
                    this._setZorder(node);
                    var panel = view.svg || view._canvas;
                    if (node.isGroup && !node.parent) {
                        if (!node.container) {
                            //this._updateChildBounds(node, this.nameTable);
                            ej.datavisualization.Diagram.Util._updateGroupBounds(node, this.nameTable);
                        }
                        view.context.renderGroup(node, panel, view.diagramLayer, this.nameTable, this, isLoad, isoverView);
                    }
                    else {

                        if (!node.parent)
                            view.context.renderNode(node, panel, view.diagramLayer, this);
                    }
                }
            }
        },

        _renderConnectors: function (view) {
            if (this.model.connectors) {
                var connectors = this.model.connectors;
                var panel = view.svg || view._canvas;
                var connector;
                for (var i = 0, len = connectors.length; i < len; ++i) {
                    connector = connectors[i];
                    this._setZorder(connector);
                }
                for (var i = 0, len = connectors.length; i < len; ++i) {
                    connector = connectors[i];
                    ej.datavisualization.Diagram.Util.updateBridging(connector, this);
                    view.context.renderConnector(connector, panel, view.diagramLayer, this);
                }
            }
        },

        //#endregion

        //#region Events
        _wireEvents: function () {
            //svg document events
            var svgDocument = $(this._canvas);
            this._on(svgDocument, ej.eventType.mouseDown, this._mousedown);
            this._on(svgDocument, ej.eventType.mouseMove, this._mousemove);
            this._on(svgDocument, ej.eventType.mouseUp, this._mouseup);
            this._on(svgDocument, ej.eventType.mouseLeave, this._documentmouseup);
            this._on(svgDocument, "dblclick", this._doubleclick);
            this._on(svgDocument, "doubletap", this._doubleclick);

            this._on(this.element, "keydown", this._keydown);
            this._on(this.element, "keyup", this._keyup);
            //this._on(this.element, "contextmenu", this._contextmenu);
            this._on(svgDocument, "pinchin", this._pinchin);
            this._on(svgDocument, "pinchout", this._pinchout);
            this._on(this.element, "touchstart", this._handleTouchStart);
            if (ej.browserInfo().name === "mozilla") {
                this._on(this.element, "DOMMouseScroll", this._handleMouseWheel);
            }
            else
                this._on(this.element, "mousewheel", this._handleMouseWheel);
            //window document event
            this._on($(document), "mouseup", this._documentmouseup);

            if (window) {
                this._on($(window), "resize", this.updateViewPort);
            }
            var obj = this;
            $(svgDocument).ejDroppable({
                drop: function (event, ui) {
                    obj._svgdrop(event, ui);
                    obj._isDropped = true;
                }
            });
        },

        _mousePosition: function (evt, exOffset) {
            var e = this._isTouchEvent(evt);
            var scrollLeft = this._canvas.scrollLeft;
            var scrollTop = this._canvas.scrollTop;
            var controlBBox = this._canvas.getBoundingClientRect();
            var layerx = (e.clientX + scrollLeft) - controlBBox.left;
            var layery = (scrollTop + e.clientY) - controlBBox.top;
            if (!exOffset) {
                layerx = (layerx + this._hScrollOffset) / this._currZoom;
                layery = (layery + this._vScrollOffset) / this._currZoom;
            }
            return new ej.datavisualization.Diagram.Point(Math.round(layerx * 100) / 100, Math.round(layery * 100) / 100);
        },

        _contextmenu: function (evt) {
            evt.preventDefault();
            evt.stopPropagation();
        },

        _pinchin: function (evt) {
            evt.preventDefault();
        },

        _pinchout: function (evt) {
            evt.preventDefault();
        },

        _handleTouchStart: function (evt) {
            evt.preventDefault();
        },

        _mousedown: function (evt) {
            this._isPinching = false;
            if (this._invoke(evt)) {
                if (!this._isForeignObject(evt.originalEvent.target))
                    evt.preventDefault();
                if (this.model.contextMenu && $("#" + this.element[0].id + "_contextMenu").css("display") !== "none") {
                    evt.cancel = true;
                }
                if (this._isEditing)
                    this._endEdit();
                if (evt.which === 1 || (evt.which === 0 && evt.originalEvent.changedTouches)) {
                    if (evt.which === 0 && evt.originalEvent.changedTouches) {
                        var node = null;
                        var point = this._mousePosition(evt);
                        var isUserHandle = this._isUserHandle(evt);
                        if ((this._isInternalTool(this.activeTool) && !this.activeTool.inAction)) {
                            if (!isUserHandle) {
                                node = this._findNodeUnderMouse(evt);
                                this._raiseMouseEvents(node);
                            }
                            if (!node && this.selectionList[0]) {
                                node = this.selectionList[0];
                            }
                            this._checkToolToActivate(evt, node, point);
                        }
                    }
                    if (evt.ctrlKey && evt.shiftKey) {
                        node = this._findNodeUnderMouse(evt);
                        if (!node && this.selectionList[0]) {
                            node = this.selectionList[0];
                        }
                        this._checkToolToActivate(evt, node, point);
                    }
                    if (this._toolToActivate === "panTool") {
                        this.activeTool = this.tools[this._toolToActivate];
                    }
                    else {
                        this.activateTool(this._toolToActivate);
                    }
                    this.activeTool.mousedown(evt);
                }
            }
        },

        _invoke: function (evt) {
            var className = evt.originalEvent.target.className;
            if (evt.originalEvent.target && (className == "edit" || className == "editBox" || className == "e-hhandle" || className == "e-hhandlespace" || className == "e-vhandle" || className == "e-vhandlespace")) {
                return false;
            }

            return true;
        },

        _handleMouseWheel: function (evt) {
            if (evt.originalEvent.preventDefault)
                evt.originalEvent.preventDefault();
            var position = new ej.datavisualization.Diagram.Point(0, 0);
            //this.deactivateTool();
            if (evt.ctrlKey) {
                position.x = (evt.originalEvent.layerX + this._hScrollOffset) / this._currZoom;
                position.y = (evt.originalEvent.layerY + this._vScrollOffset) / this._currZoom;
                if (evt.originalEvent.wheelDelta > 0 || -40 * evt.originalEvent.detail > 0) {
                    ej.datavisualization.Diagram.ZoomUtil.zoomPan(this, (1 + Number(this.zoomFactor())), 0, 0, position, true);
                }
                else {
                    ej.datavisualization.Diagram.ZoomUtil.zoomPan(this, (1 / (1 + Number(this.zoomFactor()))), 0, 0, position, true);
                }
            }
            else {
                var offsetY = 0;
                if (evt.originalEvent.wheelDelta > 0 || -40 * evt.originalEvent.detail > 0) {
                    var top = this._vScrollbar.model.minimum;
                    if ((this._vScrollOffset - this._scrollPixel) >= top || this._scrollLimit() === ej.datavisualization.Diagram.ScrollLimit.Infinity) {
                        this._updateScrollOffset(this._hScrollOffset, (this._vScrollOffset - this._scrollPixel));
                        offsetY = -this._scrollPixel;
                    }
                    else {
                        this._updateScrollOffset(this._hScrollOffset, top);
                        offsetY = 0;
                    }
                }
                else {

                    var cHeight = this._vScrollbar.model.maximum;
                    var viewPort = ej.datavisualization.Diagram.ScrollUtil._viewPort(this);
                    if ((cHeight) >= (this._vScrollOffset + this._scrollPixel) || this._scrollLimit() === ej.datavisualization.Diagram.ScrollLimit.Infinity) {
                        this._updateScrollOffset(this._hScrollOffset, (this._vScrollOffset + this._scrollPixel));
                        offsetY = this._scrollPixel;
                    }
                    else {
                        this._updateScrollOffset(this._hScrollOffset, cHeight);
                        offsetY = 0;
                    }

                }
                this._updateEditBox(0, offsetY);
            }
        },

        _mousemove: function (evt) {
            if (evt.type == "touchmove" && evt.originalEvent.touches.length > 1) {
                this._isPinching = true;
            }
            else {
                this._isPinching = false;
                var node = null;
                var point = this._mousePosition(evt);
                var isUserHandle = this._isUserHandle(evt);
                if ((this.activeTool.inAction === false) && (this._selectedSymbol)) {
                    var scale = this._currZoom;
                    var offset = $("#" + this.element[0].id).offset();
                    var coor = this._isTouchEvent(evt);
                    var pageX;
                    var pageY;
                    if (coor) {
                        pageX = coor.pageX;
                        pageY = coor.pageY;
                    }
                    else {
                        pageX = evt.pageX;
                        pageY = evt.pageY;
                    }
                    var x = pageX + this._hScrollOffset - offset.left;
                    var y = pageY + this._vScrollOffset - offset.top;

                    {
                        var paletteItem = $.extend(true, {}, this._selectedSymbol);
                        if (this._selectedSymbol) {

                            var paletteItem = $.extend(true, {}, this._selectedSymbol);
                            paletteItem.name += ej.datavisualization.Diagram.Util.randomId();
                            if (paletteItem.segments) {
                                ej.datavisualization.Diagram.Util._translateLine(paletteItem, x/scale, y/scale);
                                args = { element: paletteItem };
                                if (!args.cancel) {
                                    this.add(paletteItem);
                                }
                            }
                            else {
                                var palNameTable = this._palNameTable;
                                var args = { element: paletteItem };
                                if (!args.cancel) {
                                    var randomId = ej.datavisualization.Diagram.Util.randomId();
                                    this._cloneGroupNode(paletteItem, randomId);
                                    //paletteItem.name = randomId;
                                    if (paletteItem.isGroup) {
                                        var cBound = ej.datavisualization.Diagram.Util._getChildrenBounds(paletteItem, this.nameTable);
                                        this._translate(paletteItem, (x / scale - cBound.x - cBound.height / 2), (y / scale - cBound.y - cBound.width / 2), this.nameTable);
                                    } else {
                                        paletteItem.offsetX = x / scale;
                                        paletteItem.offsetY = y / scale;
                                    }
                                    if (paletteItem.isLane) {
                                        var obj = ej.datavisualization.Diagram.SwimLaneHelper._createDiagramLane(paletteItem, this.nameTable);
                                        obj.isLane = paletteItem.isLane;
                                        obj.orientation = paletteItem.orientation;
                                        this.add(obj);
                                        ej.datavisualization.Diagram.SvgContext._hideNode(obj, this._svg);

                                    }
                                    else
                                        this.add(paletteItem);

                                }
                            }
                            this.scrollToNode(paletteItem);
                            this.element[0].focus();
                            this._toolToActivate = "move";
                            this.activateTool("move");
                            var displays = document.getElementsByClassName("dragClone")[0];
                            if (displays) {
                                displays.style.display = "none";
                            }
                            this.activateTool("move", true);
                            this.activeTool.mousedown(evt);
                            this.activeTool.mousemove(evt);

                        }
                    }
                }
                else {
                    if ((this._isInternalTool(this.activeTool) && !this.activeTool.inAction)) {
                        if (!isUserHandle) {
                            var node = this._findNodeUnderMouse(evt);
                            if (node) {
                                if (this._nodeToHit && this._nodeToHit != node && !this._nodeToHit.segments) {
                                    this.activeTool._showPort(this._nodeToHit, true);
                                }
                                if (!node.segments) {
                                    this.activeTool._showPort(node);
                                }
                            }
                            this._raiseMouseEvents(node);
                        }
                        if (!node && this.selectionList[0]) {
                            node = this.selectionList[0];
                        }
                        if (ej.datavisualization.Diagram.Util.isPageEditable(this)) {
                            this._checkToolToActivate(evt, node, point);
                        }
                    } else {
                        if (!this.activeTool.inAction) {
                            this._toolToActivate = this.activeTool.name;
                            this._currentCursor = this.activeTool.cursor;
                        }

                        node = this._findNodeUnderMouse(evt);
                        if (node) {
                            this._raiseMouseEvents(node);
                        }

                    }
                    this._updateCursor();
                    this.activeTool.mousemove(evt);
                    if (this.activeTool.inAction && this.enableAutoScroll()) {
                        var viewPort = ej.datavisualization.Diagram.ScrollUtil._viewPort(this);
                        var pt = this._mousePosition(evt, true);
                        var autoScrollBorder = this.model.pageSettings.autoScrollBorder;
                        if (pt.x + autoScrollBorder.right >= viewPort.width - 18) {
                            this._beginAutoScroll("right", evt);
                            this._canAutoScroll = true;
                        }
                        else if (pt.x <= autoScrollBorder.left) {
                            this._beginAutoScroll("left", evt);
                            this._canAutoScroll = true;
                        }
                        else if (pt.y + autoScrollBorder.bottom >= viewPort.height - 18) {
                            this._beginAutoScroll("bottom", evt);
                            this._canAutoScroll = true;
                        }
                        else if (pt.y <= autoScrollBorder.top) {
                            this._beginAutoScroll("top", evt);
                            this._canAutoScroll = true;
                        }
                        else {
                            this._canAutoScroll = false;
                        }
                    }
                }
            }
        },

        _cloneGroupNode: function (node, id, nameTable) {
            var child = null, clnObj;
            if (node.isGroup) {
                for (var i = 0; i < node.children.length; i++) {
                    if (nameTable)
                        child = nameTable[node.children[i]];
                    else
                        child = this._paletteTable[node.children[i]];
                    if (child) {
                        child = $.extend(true, {}, child);
                        if (child.isGroup)
                            this._cloneGroupNode(child, id);
                        clnObj = $.extend(true, {}, child);
                        clnObj.name += id;
                        node.children[i] = clnObj.name;
                        this.nameTable[clnObj.name] = clnObj;
                    }
                }
            }
        },
        _setActiveTool: function (evt) {
            var point = this._mousePosition(evt);
            if (!this.activeTool.inAction && (this._isUserHandleActive(this.activeTool.name) || !this.model.userHandles || !this.model.userHandles.length)) {
                if (this._isInternalTool(this.activeTool) || this.activeTool.singleAction) {
                    var node;
                    var success = false;
                    //var handle = this._isHandle(evt);
                    if (this.selectionList[0]) {
                        node = this.selectionList[0];
                        success = this._checkToolToActivate(evt, node, point);
                    }
                    if (!success) {
                        node = this._findNodeUnderMouse(evt);
                        if (this._isUserHandle(evt)) {
                            var handleName = $(evt.target).parents("g").first().context.id.split("_")[0];
                            this._checkToolToActivate(evt, node, point, handleName);
                            this.activeTool = this.tools[this._toolToActivate];
                        }
                        else {
                            this._checkToolToActivate(evt, node, point);
                            this._raiseMouseEvents(node);
                            if (this._isUserHandleActive(this.activateTool.name) && !this._drawingTool)
                                this.activeTool = this.tools[this._toolToActivate];
                        }
                    }
                }
            }
        },

        _beginAutoScroll: function (option, evt) {
            var delay = 100;
            var callBack = this;
            var left = 0, top = 0;
            var viewPort = ej.datavisualization.Diagram.ScrollUtil._viewPort(this);
            var cWidth = this._hScrollbar.model.maximum + viewPort.width;
            var cHeight = this._vScrollbar.model.maximum + viewPort.height;
            if (this._canAutoScroll) {
                switch (option) {
                    case "right":
                        if ((this._hScrollOffset + viewPort.width + 10) < cWidth || this._scrollLimit() === ej.datavisualization.Diagram.ScrollLimit.Infinity) {
                            left = 10;
                        }
                        break;
                    case "left":
                        if (this._hScrollOffset - 10 > this._hScrollbar.model.minimum || this._scrollLimit() === ej.datavisualization.Diagram.ScrollLimit.Infinity) {
                            left = -10;
                        }
                        break;
                    case "bottom":
                        if (this._vScrollOffset + viewPort.height + 10 < cHeight || this._scrollLimit() === ej.datavisualization.Diagram.ScrollLimit.Infinity) {
                            top = 10;
                        }
                        break;
                    case "top":
                        if (this._vScrollOffset - 10 > this._vScrollbar.model.minimum || this._scrollLimit() === ej.datavisualization.Diagram.ScrollLimit.Infinity) {
                            top = -10;
                        }
                        break;
                }
                setTimeout(function () {
                    callBack._raiseEvent("autoScrollChange", { delay: delay })
                    callBack._updateScrollBar(left, top, option, evt);
                }, delay);
            }
        },

        _updateScrollBar: function (x, y, option, evt) {
            if (x === 0 && y === 0) {
                this._canAutoScroll = false;
            }
            else if (this.activeTool.inAction) {
                var hScrollOffset = this._hScrollOffset + (x * this._currZoom);
                var vScrollOffset = this._vScrollOffset + (y * this._currZoom);
                this._updateSelectionOnScroll(x, y);
                this._updateScrollOffset(hScrollOffset, vScrollOffset);
                this._beginAutoScroll(option, evt);
            }
        },

        _updateSelectionOnScroll: function (x, y) {
            var tool = this.activeTool;
            if (tool.name === "select") {
                tool.currentPoint = new ej.datavisualization.Diagram.Point(tool.currentPoint.x + x, tool.currentPoint.y + y);
                tool._updateHelper();
            }
            else if (tool.name === "orthogonalLine" || tool.name === "straightLine") {
                tool.currentPoint = new ej.datavisualization.Diagram.Point(tool.currentPoint.x + x, tool.currentPoint.y + y);
                tool._targetPossibleConnection = tool._currentPossibleConnection;
                tool._targetPort = tool._possibleConnectionPort;
                tool._updateHelper();
            }
            else if (this.selectionList.length > 0) {
                var node = this.selectionList[0];
                tool.currentPoint = ej.datavisualization.Diagram.Point(tool.currentPoint.x + x, tool.currentPoint.y + y);
                if (tool.name === "move") {
                    this._translate(node, x, y, this.nameTable);
                }
                else if (tool.name === "resize") {
                    ej.datavisualization.Diagram.SnapUtil._removeGuidelines(diagram);
                    tool._updateSize(node, tool.previousPoint, tool.currentPoint, x, y);
                }
                else if (tool.name === "endPoint") {
                    if (tool.selectedSegment && tool._endPoint == "segmentEnd") {
                        var difx = tool.currentPoint.x - tool.previousPoint.x;
                        var dify = tool.currentPoint.y - tool.previousPoint.y;
                        tool._updateAdjacentSegments(difx, dify);

                    }
                    else {
                        var updateSelection = tool._updatePoints(node);
                        tool._disconnect(node);
                        tool._updateConnection(node, updateSelection);
                    }
                }
                tool.previousPoint = ej.datavisualization.Diagram.Point(tool.currentPoint.x, tool.currentPoint.y);
                ej.datavisualization.Diagram.DiagramContext.update(node, this);
                this._updateSelectionHandle();
                this._renderTooltip(node);
            }
        },

        _isUserHandle: function (evt) {
            if (evt.target.parentNode.className.animVal === "userHandle") {
                return true;
            }
            else {
                return false;
            }
        },

        _isUserHandleActive: function (name) {
            var userHandleActive = false;
            if (this.model.userHandles && this.model.userHandles.length > 0) {
                var i = 0;
                var length = this.model.userHandles.length;
                for (i; i < length; i++) {
                    if (this.model.userHandles[i].name === name) {
                        userHandleActive = true;
                        break;
                    }
                    else
                        userHandleActive = true;
                }
            }
            return userHandleActive;
        },

        _raiseMouseEvents: function (node) {
            if (!node && this._nodeToHit) {
                this._raiseEvent("mouseLeave", { element: this._nodeToHit });

                if (this._nodeToHit != null && (this._nodeToHit.shape) && !(this.activeTool instanceof ej.datavisualization.Diagram.LineTool)) {
                    this.activeTool._showPort(this._nodeToHit, true);

                }
            }
            if (node) {
                if (this._nodeToHit !== node) {
                    this._nodeToHit = node;
                    this._raiseEvent("mouseEnter", { element: node });
                }
            }
            else {
                this._nodeToHit = null;
            }
            if (node) {
                this._raiseEvent("mouseOver", { element: node });
            }
        },

        _isHandle: function (evt) {
            var handle = $(evt.target).parent("g.handle");
            if (handle.length > 0) {
                return true;
            }
            return false;
        },

        _findNodeUnderMouse: function (evt) {
            var node = null;
            var type;
            var parent = $(evt.target).parents(".ej-d-node,.ej-d-connector,.ej-d-group");
            if (parent) {
                type = parent.attr("class");
                if (type === "ej-d-node" || type === "ej-d-group") {
                    node = this._findNode(parent.attr("id"));
                }
                else if (type === "ej-d-connector") {
                    node = this._findConnector(parent.attr("id"));
                }
            }
            return node;
        },

        _isForeignObject: function (target) {
            var foreignObj = target.parentNode;
            if (foreignObj) {
                while (foreignObj.parentNode != null) {
                    if (foreignObj.className === "foreignObject") {
                        return foreignObj;
                    }
                    else {
                        foreignObj = foreignObj.parentNode;
                    }
                }
            }
            return null;
        },

        _mouseup: function (evt) {
            if (!this._isPinching) {
                if (this._invoke(evt)) {
                    var foreignObject = this._isForeignObject(evt.originalEvent.target);
                    if (!foreignObject)
                        evt.preventDefault();
                    if (evt.target.id === this._id + "_canvas_svg")
                        this._raiseEvent("click", { element: this });
                    this.activeTool.mouseup(evt);
                    if (this.activeTool instanceof ej.datavisualization.Diagram.PanTool) {
                        document.onmousemove = null;
                        document.onmouseup = null;
                    }

                    if (this._selectedSymbol) {
                        var args;
                        args = { element: this.selectionList[0], cancel: false };

                        if (args.cancel === true) {
                            this._remove(args.element);
                        }
                        var node = this.selectionList[0];
                        if (node) {
                            if ((node.labels.length > 0 && node.labels[0].mode === "edit")) {
                                this._isEditing = true;
                                this.startLabelEdit(node, node.labels[0]);
                            }

                            else {
                                this.element[0].focus();
                            }

                            this._raiseDropEvent(args);
                            this._raiseEvent("selectionChange", args);

                        }
                        var childTable = {};
                        if (this.selectionList && this.selectionList.length > 0)
                            childTable = this._getChildTable(this.selectionList[0], childTable);
                        var entry = { type: "collectionchanged", object: jQuery.extend(true, {}, node), childTable: jQuery.extend(true, {}, childTable), changeType: "insert" };
                        this.addHistoryEntry(entry);
                        this._selectedSymbol = null;
                    }
                    if ((!(this.activeTool instanceof ej.datavisualization.Diagram.PolygonTool) && !(this.activeTool instanceof ej.datavisualization.Diagram.TextTool)) && document.activeElement && document.activeElement.id !== this.element[0].id)
                        if (!this._isEditing && !foreignObject)
                            this.element[0].focus();
                    this._currentCursor = this.activeTool.cursor;
                    this._updateCursor();
                }
            }
        },

        _doubleclick: function (evt) {
            var args;
            args = { element: this._nodeToHit, cancel: false, actualObject: this.activeTool.actualObject }
            if (!this._isEditing)
                this._raiseEvent("doubleClick", args);
            if (this._nodeToHit)
                this.activeTool._doubleClick(this._nodeToHit);
            if (!this.activeTool.inAction && ((this.selectionList.length > 0) || this.activeTool.selectedObject) && !args.cancel) {

                if (this.activeTool.selectedObject)
                    var obj = this.activeTool.selectedObject;
                else
                    var obj = this.selectionList[0];
                if (!this._isEditing && !(obj.isPseudoGroup) && !args.cancel) {
                    if (!this._isEditing && obj.shape && obj.shape.type === "text" && this._setLabelEditing(obj.shape.textBlock)) {
                        this.scrollToNode(obj);
                        this._startEdit(obj);
                    }
                    else if (!this._isEditing && ((obj.labels.length === 0 || typeof obj.labels.length === "undefined") || (obj.labels.length > 0 && this._setLabelEditing(obj.labels[0])))) {
                        this.scrollToNode(obj);
                        this._startEdit(obj);
                    }
                }
            }
            else if (this.activeTool instanceof ej.datavisualization.Diagram.PolygonTool) {
                this.activeTool.doubleclick(evt);
            }
        },

        _keydown: function (evt) {
            var keycode = evt.keyCode ? evt.keyCode : evt.which;
            if (keycode === 113) {
                if (this.selectionList.length > 0) {
                    var shape = this.selectionList[0];
                    if (!(shape.isPseudoGroup)) {
                        if (shape.shape.type === "text" && this._setLabelEditing(shape.shape.textBlock))
                            this._startEdit(shape);
                        else if (shape.labels.length > 0 && this._setLabelEditing(shape.labels[0]))
                            this._startEdit(shape);
                    }
                }
            }
            else if (this._isEditing && keycode === 27) {
                this._endEdit();
            }
            else if (!this._isEditing && keycode === 46) {
                this._delete();
            }
            else if (!this._isEditing && evt.ctrlKey && keycode === 88) {
                this.cut();
            }
            else if (!this._isEditing && evt.ctrlKey && keycode === 67) {
                this.copy();
            }
            else if (!this._isEditing && evt.ctrlKey && keycode === 86) {
                this.paste();
            }
            else if (evt.ctrlKey && keycode === 65) {
                if (this._isEditing)
                    this._endEdit();
                evt.preventDefault();
                this.selectAll();
            }
            else if (!this._isEditing && evt.ctrlKey && keycode === 89) {
                this.redo();
            }
            else if (!this._isEditing && evt.ctrlKey && keycode === 90) {
                this.undo();
            }
            if (!this._isEditing && (keycode === 37 || keycode === 38 || keycode === 39 || keycode === 40)) {
                evt.preventDefault();
                if (keycode === 37) { //left
                    !evt.shiftKey ? this.nudge("left") : this.nudge("left", 5);
                } else if (keycode === 38) { //up
                    !evt.shiftKey ? this.nudge("up") : this.nudge("up", 5);
                } else if (keycode === 39) { //right
                    !evt.shiftKey ? this.nudge("right") : this.nudge("right", 5);
                } else if (keycode === 40) { //down
                    !evt.shiftKey ? this.nudge("down") : this.nudge("down", 5);
                }
            }
            this.activeTool.keydown(evt);
        },

        _keyup: function (evt) {
            var keycode = evt.keyCode ? evt.keyCode : evt.which;
            if (!this._isEditing && (keycode === 37 || keycode === 38 || keycode === 39 || keycode === 40)) {
                evt.preventDefault();
                this.activeTool.inAction = false;
                this._updateSelectionHandle();
            }
        },

        _registerHistoryEvents: function () {
            this.historyManager.addEventHandler("collectionchanged", $.proxy(this._recordCollectionChanged, this));
            this.historyManager.addEventHandler("positionchanged", $.proxy(this._recordPinPointChanged, this));
            this.historyManager.addEventHandler("sizechanged", $.proxy(this._recordResizeChanged, this));
            this.historyManager.addEventHandler("rotationchanged", $.proxy(this._recordRotationChanged, this));
            this.historyManager.addEventHandler("endpointchanged", $.proxy(this._recordEndPointChanged, this));
            this.historyManager.addEventHandler("labelchanged", $.proxy(this._recordLabelChanged, this));
            this.historyManager.addEventHandler("groupingchanged", $.proxy(this._recordGroupChanged, this));
        },

        _updatePoints: function (helper, startPoint, endPoint, isTarget) {
            var offset = new ej.datavisualization.Diagram.Point(endPoint.x - startPoint.x, endPoint.y - startPoint.y);
            if (!ej.datavisualization.Diagram.Geometry.isEmptyPoint(offset)) {
                helper._updateEndPoint(offset.x, offset.y, isTarget);
            }
        },

        _recordCollectionChanged: function (args) {
            this._clearSelection();
            this._isUndo = true;
            var i;
            var changeType = args.changeType;
            if (args.isUndo) {
                if (args.changeType == "insert")
                    changeType = "remove";
                else
                    changeType = "insert";
            }
            if (changeType == "insert") {
                if (!args.object.container) {
                    if (!args.object.isPseudoGroup) {
                        if (!args.object.segments) {
                            var port = null;
                            if (args.object.inEdges.length > 0) {
                                for (i = 0; i < args.object.inEdges.length; i++) {
                                    if (this.nameTable[args.object.inEdges[i]])
                                        this.nameTable[args.object.inEdges[i]].targetNode = args.object.name;
                                    port = args.targetPorts[args.object.inEdges[i]];
                                    if (port)
                                        if (this.nameTable[args.object.inEdges[i]])
                                            this.nameTable[args.object.inEdges[i]].targetPort = port;
                                }
                            }
                            if (args.object.outEdges.length > 0) {
                                for (i = 0; i < args.object.outEdges.length; i++) {
                                    if (this.nameTable[args.object.outEdges[i]])
                                        this.nameTable[args.object.outEdges[i]].sourceNode = args.object.name;
                                    port = args.sourcePorts[args.object.outEdges[i]];
                                    if (port)
                                        if (this.nameTable[args.object.outEdges[i]])
                                            this.nameTable[args.object.outEdges[i]].sourcePort = port;
                                }
                            }
                        }
                        if (args.object.isGroup)
                            this._addChildren(args.object, args.childTable);
                        this.add(args.object, args);
                        if (args.object.isGroup) {
                            this._addGroupConnector(args.object, args)
                        }
                        else {
                            this._addConnector(args.object, args);
                        }
                        this._clearSelection();
                        this.addSelection(args.object);
                    } else {
                        delete this.nameTable["multipleSelection"];
                        var node, j;
                        var pseudoGroup = ej.datavisualization.Diagram.Group({ "name": "multipleSelection", isPseudoGroup: true });
                        for (i = 0; i < args.object.children.length; i++) {
                            pseudoGroup.children.push(args.object.children[i]);
                            node = args.childTable[args.object.children[i]];
                            node.parent = "";
                            if (node.isGroup)
                                this._addChildren(node, args.childTable);
                            this.add(args.childTable[args.object.children[i]], args);
                        }
                        this._addGroupConnector(args.object, args);
                        for (i = 0; i < pseudoGroup.children.length; i++) {
                            node = this.nameTable[pseudoGroup.children[i]];
                            if (!node.segments) {
                                if (node.inEdges.length > 0) {
                                    for (j = 0; j < node.inEdges.length; j++)
                                        this.nameTable[node.inEdges[j]].targetNode = node.name;
                                }
                                if (node.outEdges.length > 0) {
                                    for (j = 0; j < node.outEdges.length; j++)
                                        this.nameTable[node.outEdges[j]].sourceNode = node.name;
                                }
                            }
                        }
                        this._clearSelection();
                        this.model.nodes.push(pseudoGroup);
                        pseudoGroup.offsetX = args.object.offsetX;
                        pseudoGroup.offsetY = args.object.offsetY;
                        pseudoGroup.height = args.object.height;
                        pseudoGroup.width = args.object.width;
                        pseudoGroup.rotateAngle = args.object.rotateAngle;
                        this.addSelection(pseudoGroup);
                        this.nameTable[pseudoGroup.name] = pseudoGroup;
                    }
                } else {
                    if (!args.object.container) {
                        var child;
                        for (var i in args.childTable) {
                            child = args.childTable[i];
                            this.nameTable[i] = child;
                        }
                        node = args.object;
                        if (node.isSwimlane) {
                            node.width = 0;
                            node.height = 0;
                            this.add(node);
                        } else if (node.isLane) {
                            var swimlane = ej.datavisualization.Diagram.SwimLaneHelper._createSwimlane(node, this);
                            if (swimlane) {
                                swimlane.width = 0;
                                swimlane.height = 0;
                                this.add(swimlane);
                            }
                        }
                    }
                }

            }
            else if (changeType == "remove") {
                if (!args.object.isPseudoGroup) {
                    this._remove(args.object);
                }
                else {
                    for (i = 0; i < args.object.children.length; i++) {
                        this._remove(this.nameTable[args.object.children[i]]);
                    }
                }
            }
            this._isUndo = false;
        },

        _addGroupConnector: function (group, arg) {
            var edges;
            var connector;
            var k, j, len, l;
            if (!group.segments) {
                for (k = 0; k < group.children.length; k++) {
                    var child = this.nameTable[group.children[k]];
                    if (child.inEdges && child.inEdges.length > 0) {
                        edges = child.inEdges;
                        for (j = 0, len = edges.length; j < len; j++) {
                            if (arg.edgeTable)
                                connector = arg.edgeTable[edges[j]];
                            else
                                connector = arg.childTable[edges[j]];
                            if (connector)
                                this.add(connector);
                        }
                    }
                    if (child.outEdges && child.outEdges.length > 0) {
                        edges = child.outEdges;
                        for (j = 0, len = edges.length; j < len; j++) {
                            if (arg.edgeTable)
                                connector = arg.edgeTable[edges[j]];
                            else
                                connector = arg.childTable[edges[j]];
                            if (connector)
                                this.add(connector);
                        }
                    }
                }
            }
        },

        _addConnector: function (node, arg) {
            var edges;
            var connector;
            var k, j, len, l;
            if (!node.segments) {
                if (node.inEdges && node.inEdges.length > 0) {
                    edges = node.inEdges;
                    for (j = 0, len = edges.length; j < len; j++) {
                        connector = arg.edgeTable[edges[j]];
                        this.add(connector);
                    }
                }
                if (node.outEdges && node.outEdges.length > 0) {
                    edges = node.outEdges;
                    for (j = 0, len = edges.length; j < len; j++) {
                        connector = arg.edgeTable[edges[j]];
                        this.add(connector);
                    }
                }
            }
        },

        _addChildren: function (group, childTable) {
            for (var i = 0; i < group.children.length; i++) {
                var node = childTable[group.children[i]];
                if (node.isGroup)
                    this._addChildren(node, childTable);
                else
                    this.nameTable[group.children[i]] = node;
            }
        },

        _updateRecordPinPoint: function (node, childTable, isMultipleSelection) {
            var object, i, item, objNode;
            // this._clearSelection();
            if (node.isPseudoGroup) {
                var undoObj = node;
                for (i = 0; i < node.children.length; i++) {
                    item = this.nameTable[node.children[i]];
                    objNode = childTable[node.children[i]];
                    if (objNode) {
                        if (!item.segments) {
                            this._translate(item, objNode.offsetX - item.offsetX, objNode.offsetY - item.offsetY, this.nameTable);
                        } else {
                            var objLine = childTable[item.name];
                            item.sourcePoint = objLine.sourcePoint;
                            item.targetPoint = objLine.targetPoint;
                            item.segments = objLine.segments;
                        }
                        ej.datavisualization.Diagram.DiagramContext.update(item, this);
                    }
                }
                if (this.selectionList[0]) {
                    if (this.selectionList[0].name === undoObj.name) {
                        this.selectionList[0].offsetX = undoObj.offsetX;
                        this.selectionList[0].offsetY = undoObj.offsetY;
                    }
                }
                ej.datavisualization.Diagram.SvgContext.updateSelector(undoObj, this._svg, this._currZoom, this, this.model.selectorConstraints);
            } else {
                if (node.segments) {
                    if (isMultipleSelection) {
                        object = childTable[node.name];
                        node.sourcePoint = object.sourcePoint;
                        node.targetPoint = object.targetPoint;
                        node.segments = object.segments;
                        ej.datavisualization.Diagram.DiagramContext.update(node, this);
                    } else {
                        this._clearSelection();
                        object = this._findConnector(node.name);
                        var obj;

                        if (node.targetNode) {
                            obj = this.nameTable[node.targetNode];
                            ej.datavisualization.Diagram.Util.removeItem(obj.inEdges, node);
                        }
                        if (node.sourceNode) {
                            obj = this.nameTable[node.sourceNode];
                            ej.datavisualization.Diagram.Util.removeItem(obj.outEdges, node);
                        }
                        if (node.targetNode) {
                            obj = this.nameTable[node.targetNode];
                            obj.inEdges.push(node.name);
                        }
                        if (node.sourceNode) {
                            obj = this.nameTable[node.sourceNode];
                            obj.outEdges.push(node.name);
                        }
                        object.targetNode = node.targetNode;
                        object.sourceNode = node.sourceNode;
                        object.targetPort = node.targetPort;
                        object.sourcePort = node.sourcePort;

                        object.sourcePoint = node.sourcePoint;
                        object.targetPoint = node.targetPoint;
                        object.segments = node.segments;
                        this._updateLabels(object, node.labels);
                        ej.datavisualization.Diagram.DiagramContext.update(object, this);
                        ej.datavisualization.Diagram.SvgContext.updateSelector(node, this._svg, this._currZoom, this, this.model.selectorConstraints);
                        this.addSelection(node);
                    }
                } else {
                    if (isMultipleSelection) {
                        object = childTable[node.name];
                        this._translate(node, object.offsetX - node.offsetX, object.offsetY - node.offsetY, childTable);
                        ej.datavisualization.Diagram.DiagramContext.update(object, this);
                    } else {
                        this._clearSelection();
                        object = this._findNode(node.name);
                        this._translate(object, node.offsetX - object.offsetX, node.offsetY - object.offsetY, this.nameTable);
                        this._updateLabels(object, node.labels);
                        if (node.parent) {
                            ej.datavisualization.Diagram.Util._updateGroupBounds(this.nameTable[object.parent], this.nameTable);
                        }
                        if (object.parent) {
                            var parent = this.nameTable[object.parent];
                            if (parent && parent.container) {
                                if (parent.container.type == "canvas") {
                                    ej.datavisualization.Diagram.SvgContext._alignOnCanvas1(parent, this);
                                }
                                else if (parent.container.type == "stack") {

                                    ej.datavisualization.Diagram.SvgContext._alignOnStack(parent, this);
                                }

                            }
                        }
                        ej.datavisualization.Diagram.DiagramContext.update(object, this);
                        this.addSelection(object);
                    }
                }
            }
        },
        _updateLabels: function (node, labels) {
            node.labels = labels;
            for (var i = 0; i < node.labels.length; i++)
                ej.datavisualization.Diagram.DiagramContext.updateLabel(node, node.labels[i], this);
        },
        _recordPinPointChanged: function (args) {
            var node = args.object.node;
            var childTable = args.object.childTable;
            if (!node.container)
                this._updateRecordPinPoint(node, childTable);
        },
        _updateRecordRotation: function (object, childTable) {
            var node, i, item, objNodeChild;
            if (object.isPseudoGroup) {
                node = this.nameTable[object.name];

                for (i = 0; i < object.children.length; i++) {
                    item = this.nameTable[object.children[i]];
                    objNodeChild = childTable[object.children[i]];
                    if (item) {
                        if (!item.segments) {
                            if (objNodeChild) {
                                this._translate(item, objNodeChild.offsetX - item.offsetX, objNodeChild.offsetY - item.offsetY, this.nameTable);
                                this._rotate(item, objNodeChild.rotateAngle - item.rotateAngle, this.nameTable);
                            }
                        } else {
                            var objLine = childTable[item.name];
                            item.sourcePoint = objLine.sourcePoint;
                            item.targetPoint = objLine.targetPoint;
                            item.segments = objLine.segments;
                        }
                        ej.datavisualization.Diagram.DiagramContext.update(item, this);
                    }
                }
                if (node) {
                    var newangle = object.rotateAngle - node.rotateAngle;
                    node.rotateAngle += newangle;
                    if (this.selectionList[0])
                        this.selectionList[0].rotateAngle = node.rotateAngle;
                }
                ej.datavisualization.Diagram.SvgContext.updateSelector(object, this._svg, this._currZoom, this, this.model.selectorConstraints);
            }
            else {
                var parent = null;
                if (object.parent) {
                    parent = this.nameTable[object.parent];
                }
                if (parent && parent.container) {
                    if (parent.container.type == "canvas") {
                        ej.datavisualization.Diagram.SvgContext._alignOnCanvas1(parent, this);
                    }
                    else if (parent.container.type == "stack") {

                        ej.datavisualization.Diagram.SvgContext._alignOnStack(parent, this);
                    }

                }
                else if (object) {
                    this._clearSelection();
                    node = this._findNode(object.name);
                    this._rotate(node, object.rotateAngle - node.rotateAngle, this.nameTable);
                    ej.datavisualization.Diagram.DiagramContext.update(node, this);
                    this.addSelection(node);
                }
            }
        },
        _recordSizeChanged: function (args) {
            var node = args.object.node;
            var childTable = args.object.childTable;
            if (!node.container)
                this._updateRecordSize(node, childTable);
        },
        _updateRecordSize: function (object, childTable) {
            var node, i, childObj;
            if (object.isGroup || object.isPseudoGroup) {
                if (object.isPseudoGroup)
                    this.nameTable[object.name] = object;
                node = this._findNode(object.name);
                for (i = 0; i < node.children.length; i++) {
                    var item = this.nameTable[object.children[i]];
                    childObj = childTable[object.children[i]];
                    if (childObj) {
                        if (!item.segments) {
                            this._translate(item, childObj.offsetX - item.offsetX, childObj.offsetY - item.offsetY, this.nameTable);
                            this.scale(item, (childObj.width / item.width), (childObj.height / item.height), ej.datavisualization.Diagram.Point(item.offsetX, item.offsetY), this.nameTable);
                            ej.datavisualization.Diagram.DiagramContext.update(item, this);
                        } else {
                            var kk = childTable[item.name];
                            item.sourcePoint = kk.sourcePoint;
                            item.targetPoint = kk.targetPoint;
                            item.segments = kk.segments;
                            ej.datavisualization.Diagram.DiagramContext.update(item, this);
                        }
                    }
                }
                //if (node.name === object.name) {
                node.offsetX = object.offsetX;
                node.offsetY = object.offsetY;
                node.width = object.width;
                node.height = object.height;
                //}
                ej.datavisualization.Diagram.SvgContext.updateSelector(node, this._svg, this._currZoom, this, this.model.selectorConstraints);
            }
            else {
                var parent = null;
                if (object.parent) {
                    parent = this.nameTable[object.parent];
                }
                if (parent && parent.container) {
                    if (parent.container.type == "canvas") {
                        ej.datavisualization.Diagram.SvgContext._alignOnCanvas1(parent, this);
                    }
                    else if (parent.container.type == "stack") {

                        ej.datavisualization.Diagram.SvgContext._alignOnStack(parent, this);
                    }

                }
                else if (!object.segments) {
                    node = this._findNode(object.name);
                    this._translate(node, object.offsetX - node.offsetX, object.offsetY - node.offsetY, this.nameTable);
                    this.scale(node, object.width / node.width, object.height / node.height, ej.datavisualization.Diagram.Point(node.offsetX, node.offsetY), this.nameTable);
                    ej.datavisualization.Diagram.DiagramContext.update(node, this);
                    ej.datavisualization.Diagram.SvgContext.updateSelector(node, this._svg, this._currZoom, this, this.model.selectorConstraints);
                }
                else {

                }
            }

        },
        _recordRotationChanged: function (args) {
            var node = args.object.node;
            var childTable = args.object.childTable;
            if (!node.container)
                this._updateRecordRotation(node, childTable);
        },
        _recordEndPointChanged: function (args) {

            var connector, currentObject, previousObject, obj;
            if (args._isUndo) {
                connector = currentObject = args.undoObject;
                previousObject = args.redoObject;
            }
            else {
                connector = currentObject = args.redoObject;
                previousObject = args.undoObject;
            }
            if (connector.parent) {

                connector = this._findChildren(connector.parent, connector.name);
            }
            else
                connector = this._findConnector(connector.name);

            if (previousObject.targetNode) {
                obj = this.nameTable[previousObject.targetNode];
                ej.datavisualization.Diagram.Util.removeItem(obj.inEdges, previousObject);
            }
            if (previousObject.sourceNode) {
                obj = this.nameTable[previousObject.sourceNode];
                ej.datavisualization.Diagram.Util.removeItem(obj.outEdges, previousObject);
            }
            if (currentObject.targetNode) {
                obj = this.nameTable[currentObject.targetNode];
                obj.inEdges.push(currentObject.name);
            }
            if (currentObject.sourceNode) {
                obj = this.nameTable[currentObject.sourceNode];
                obj.outEdges.push(currentObject.name);
            }
            connector.targetNode = currentObject.targetNode;
            connector.sourceNode = currentObject.sourceNode;
            connector.targetPort = currentObject.targetPort;
            connector.sourcePort = currentObject.sourcePort;
            connector.sourcePoint = currentObject.sourcePoint;
            connector.targetPoint = currentObject.targetPoint;
            connector.segments = currentObject.segments;
            this._updateEdges(connector);
            this._dock(connector, this.nameTable);
            ej.datavisualization.Diagram.Util._updateConnectorSegments(connector, connector.sourcePort, connector.targetPort, this);
            ej.datavisualization.Diagram.DiagramContext._refreshSegments(connector, this);
            ej.datavisualization.Diagram.Util.updateBridging(connector, this);
            ej.datavisualization.Diagram.DiagramContext.update(connector, this);
            this._updateConnectorBridging(connector);
            this._updateSelectionHandle();
            if (this.selectionList[0])
                ej.datavisualization.Diagram.SvgContext._refreshEndPointHandles(this.selectionList[0], this._svg, this.model.zoomFactor);
            ej.datavisualization.Diagram.SvgContext.updateSelector(connector, this._svg, this._currZoom, this, this.model.selectorConstraints);
        },

        _recordLabelChanged: function (args) {
            var node = this.nameTable[args.object.name];
            if (args.object.shape && args.object.shape.type === "text") {
                node.shape.textBlock.text = args.label;
                ej.datavisualization.Diagram.DiagramContext.updateTextBlock(node, node.shape.textBlock, this);
            } else {
                node.labels[0].text = args.label;
                ej.datavisualization.Diagram.DiagramContext.updateLabel(node, node.labels[0], this);
            }
        },

        _recordGroupChanged: function (args) {
            this._clearSelection();
            var actionType = args.actionType;
            if (args.isUndo) {
                if (args.actionType === "group")
                    actionType = "ungroup";
                if (args.actionType === "ungroup")
                    actionType = "group";
            }
            var selectionList = args.object.children;
            if (actionType === "group") {
                var group = new ej.datavisualization.Diagram.Group({ name: args.object.name });
                for (i = 0; i < selectionList.length; i++) {
                    group.children.push(selectionList[i]);
                    this.nameTable[selectionList[i]].parent = group.name;
                    var groupElement = this._svg.document.getElementById(selectionList[i]);
                    if (groupElement)
                        groupElement.parentNode.removeChild(groupElement);
                }
                //group.name = args.object.name;
                this.add(group);
                this._addSelection(group);
            }
            else {
                var i, node;
                node = args.object;
                var groupElement = this._svg.document.getElementById(node.name);
                if (groupElement)
                    groupElement.parentNode.removeChild(groupElement);
                for (i = 0; i < node.children.length; i++) {
                    var childNode = this.nameTable[node.children[i]];
                    childNode.parent = "";
                    this._isUndo = true;
                    if (childNode.segments) {
                        ej.datavisualization.Diagram.DiagramContext.renderConnector(childNode, this);
                    }
                    else if (!childNode.isGroup) {
                        ej.datavisualization.Diagram.DiagramContext.renderNode(childNode, this);
                    } else {
                        ej.datavisualization.Diagram.DiagramContext.renderGroup(childNode, this);
                    }
                    this._isUndo = false;
                }
                ej.datavisualization.Diagram.Util.removeItem(this.model.nodes, this._findNode(node.name));
            }
        },

        _svgdrop: function (evt, ui) {
            var args = {};
            var e = evt.originalEvent;
            if (e.preventDefault)
                e.preventDefault();
            else
                e.returnValue = false;
            var scale = this._currZoom;
            var offset = $("#" + this.element[0].id).offset();
            var coor = this._isTouchEvent(evt);
            var pageX;
            var pageY;
            if (coor) {
                pageX = coor.pageX;
                pageY = coor.pageY;
            }
            else {
                pageX = evt.pageX;
                pageY = evt.pageY;
            }
            var x = pageX + this._hScrollOffset - offset.left;
            var y = pageY + this._vScrollOffset - offset.top;
            var symbolPaletteId = ui.helper[0].getAttribute("paletteId");
            var symbolPalette = $("#" + symbolPaletteId).ejSymbolPalette("instance");
            if (symbolPaletteId) {
                var paletteItem = $.extend(true, {}, this._selectedSymbol);
                this._selectedSymbol = null;
                paletteItem.name += ej.datavisualization.Diagram.Util.randomId();
                if (paletteItem.segments) {
                    if (paletteItem.segments) {
                        ej.datavisualization.Diagram.Util._translateLine(paletteItem, x, y);
                    }
                    args = { element: paletteItem };
                    this._raiseDropEvent(args);
                    if (!args.cancel) {
                        this.add(paletteItem);
                    }
                    paletteItem = this.selectionList[0];
                }
                else {

                    var palNameTable = $.extend(true, {}, symbolPalette.nameTable);
                    paletteItem.offsetX = x / scale;
                    paletteItem.offsetY = y / scale;
                    if (paletteItem.isGroup) {
                        for (var j = 0; j < paletteItem.children.length; j++) {
                            var newObj = $.extend(true, {}, palNameTable[paletteItem.children[j]]);
                            newObj.parent = paletteItem.name;
                            newObj.name = paletteItem.name + newObj.name;
                            paletteItem.children[j] = newObj.name;
                            this.nameTable[newObj.name] = newObj;
                            this.model.nodes.push(newObj);
                            ej.datavisualization.Diagram.SpatialUtil._updateQuad(this, this._spatialSearch, newObj);
                        }
                        paletteItem.height = paletteItem.width = 0;
                        this._updateChildBounds(paletteItem, this.nameTable);
                    }
                    args = { element: paletteItem };
                    this._raiseDropEvent(args);
                    if (!args.cancel) {
                        this.add(paletteItem);
                    }
                    paletteItem = this.selectionList[0];
                }
                this.scrollToNode(paletteItem);
                if (paletteItem.labels.length > 0 && paletteItem.labels[0].mode === "edit") {
                    this._isEditing = true;
                    this.startLabelEdit(paletteItem, paletteItem.labels[0]);
                }
                else
                    this.element[0].focus();

            }
            else {
                var node = null;
                node = this._findNodeUnderMouse(evt);
                args = { element: ui.helper, e: evt, draggable: ui.draggable, targetNode: node };
                this._raiseDropEvent(args);
            }
        },

        _isTouchEvent: function (evt) {
            if (evt.type == "touchmove" || evt.type == "touchstart" || evt.type == "touchend") {
                return evt.originalEvent ? evt.originalEvent.changedTouches[0] : evt.changedTouches[0];
            }
            return evt;
        },

        _raiseDropEvent: function (args) {
            this._raiseEvent("drop", args);
        },

        _documentmouseup: function (evt) {
            if (this.activeTool && this.activeTool.inAction && !(this.activeTool instanceof ej.datavisualization.Diagram.PolygonTool) && !(this.activeTool instanceof ej.datavisualization.Diagram.TextTool)) {
                this.activeTool.mouseup(evt, true);
            }

            if (this._selectedSymbol) {
                var displays = document.getElementsByClassName("dragClone")[0];
                if (displays != null) {
                    displays.style.display = "block";
                }
                this._remove(this.selectionList[0]);

            }

        },
        //#endregion

        //#region Helper methods
        _updateEdges: function (connector) {
            var node;
            if (ej.datavisualization.Diagram.Util.isTargetConnected(connector)) {
                node = this.nameTable[connector.targetNode];
                if (node && node.inEdges.indexOf(connector.name) == -1)
                    node.inEdges.push(connector.name);
            }
            if (ej.datavisualization.Diagram.Util.isSourceConnected(connector)) {
                node = this.nameTable[connector.sourceNode];
                if (node && node.outEdges.indexOf(connector.name) == -1)
                    node.outEdges.push(connector.name);
            }
        },

        _updateChildrenEdges: function (object) {
            for (var i = 0; i < object.children.length; i++) {
                var child = object.children[i];
                if (child.segments) {
                    this._updateEdges(child);
                    this._dock(child, this.nameTable);
                }
                else if (child.isGroup) {
                    this._updateChildrenEdges(child);
                }
            }
        },
        _updateParent: function (object) {
            for (var i = 0; i < object.children.length; i++) {
                var child = this._getNewNode(this.nameTable[object.children[i]]);
                if (child && child.isGroup) {
                    this._updateParent(child);
                }
                else if (child) {
                    child.parent = object.name;
                    this.nameTable[child.name] = child;
                }
            }
        },
        _findPortAtPoint: function (point, node) {
            var ports;
            var port;
            var location;
            var size, pt, x, y;
            var bounds;
            if (node) {
                bounds = ej.datavisualization.Diagram.Util.bounds(node, true);
                ports = node.ports;
                var matrix;
                for (var i = 0, len = ports.length; i < len; ++i) {
                    port = ports[i];
                    size = ej.datavisualization.Diagram.Size(port.size, port.size);
                    pt = ej.datavisualization.Diagram.Util._getPortPosition(port, bounds);
                    matrix = ej.Matrix.identity();
                    ej.Matrix.rotate(matrix, node.rotateAngle, node.offsetX, node.offsetY);
                    pt = ej.Matrix.transform(matrix, pt);
                    x = pt.x - port.size / 2;
                    y = pt.y - port.size / 2;
                    location = ej.datavisualization.Diagram.Rectangle(x, y, size.width, size.height);
                    if (ej.datavisualization.Diagram.Geometry.containsPoint(location, point)) {
                        return port;
                    }
                }

            }
            return null;
        },

        _findLabelAtPoint: function (point, node) {
            var labels, label = null;
            var location;
            var size, pt, x, y, width, height;
            var bounds, textBounds, matrix;
            if (node) {
                bounds = ej.datavisualization.Diagram.Util.bounds(node);
                labels = node.labels;
                if (labels.length > 0) {
                    label = labels[0];
                    for (var i = 0, len = labels.length; i < len; i++) {
                        label = labels[i];
                        textBounds = this._svg.getElementById(node.name + "_" + label.name).getBBox();
                        width = textBounds.width == 0 ? bounds.width : textBounds.width;
                        height = textBounds.height == 0 ? label.fontSize : textBounds.height;
                        size = ej.datavisualization.Diagram.Size(width, height);
                        pt = ej.datavisualization.Diagram.Util._getLabelPosition(label, bounds);
                        pt.x = bounds.x + pt.x;
                        pt.y = bounds.y + pt.y;
                        matrix = ej.Matrix.identity();
                        if (!node.segments)
                            ej.Matrix.rotate(matrix, node.rotateAngle, node.offsetX, node.offsetY);
                        else
                            ej.Matrix.rotate(matrix, 0, bounds.x + bounds.width / 2, bounds.y + bounds.height / 2);
                        pt = ej.Matrix.transform(matrix, pt);
                        x = pt.x - size.width / 2;
                        y = pt.y - size.height / 2;
                        location = ej.datavisualization.Diagram.Rectangle(x, y, size.width, size.height);
                        if (ej.datavisualization.Diagram.Geometry.containsPoint(location, point)) {
                            break;
                        }
                    }
                }
            }
            return label;
        },

        _getCurrentZoom: function () {
            return this._currZoom;
        },

        _findNode: function (id) {
            return this.nameTable[id];
        },

        _findPort: function (node, portId) {
            var ports = node.ports;
            for (var i = 0, len = ports.length; i < len; i++) {
                if (ports[i].name === portId) {
                    return ports[i];
                }
            }
            return null;
        },

        _findConnector: function (id) {
            return this.nameTable[id];
        },

        _findChildren: function (group, name) {
            var child, innerchild;
            for (var i = 0; i < group.children.length; i++) {
                child = this.nameTable[group.children[i]];
                if (child) {
                    if (name == child.name) {
                        return child;
                    }
                    if (child.isGroup) {
                        innerchild = this._findChildren(child, name);
                        if (innerchild) return innerchild;
                    }
                }
            }
            return null;
        },

        _delete: function () {
            if (this.selectionList[0]) {
                var childTable = {};
                var edgeTable = {};
                childTable = this._getChildTable(this.selectionList[0], childTable);
                edgeTable = this._getEdgeTable(this.selectionList[0], edgeTable);
                if (!this._isUndo) {
                    var sourcePorts = {}, targetPorts = {}, connector, count, i;
                    if (!this.selectionList[0].segments) {
                        if (this.selectionList[0].outEdges.length > 0) {
                            count = this.selectionList[0].outEdges.length;
                            for (i = 0; i < count; i++) {
                                connector = this.nameTable[this.selectionList[0].outEdges[i]];
                                if (connector.sourcePort)
                                    sourcePorts[connector.name] = connector.sourcePort;
                            }

                        }
                        if (this.selectionList[0].inEdges.length > 0) {
                            count = this.selectionList[0].inEdges.length;
                            for (i = 0; i < count; i++) {
                                connector = this.nameTable[this.selectionList[0].inEdges[i]];
                                if (connector.targetPort)
                                    targetPorts[connector.name] = connector.targetPort;
                            }

                        }
                    }
                    var entry = { type: "collectionchanged", object: jQuery.extend(true, {}, this.selectionList[0]), childTable: jQuery.extend(true, {}, childTable), edgeTable: jQuery.extend(true, {}, edgeTable), sourcePorts: sourcePorts, targetPorts: targetPorts, changeType: "remove" };
                    this.addHistoryEntry(entry);
                }
                if (this.selectionList.length > 0) {
                    var obj = this.selectionList[0];
                    if (obj.isPseudoGroup) {
                        var children = obj.children;
                        for (var i = children.length - 1; i >= 0; i--) {
                            this._remove(this.nameTable[children[i]]);
                            this._isUndo = true;
                        }
                        this._isUndo = false;
                    }
                    else {
                        if (this._nodeToHit === obj)
                            this._nodeToHit = null;
                        this._remove(obj);
                        if (obj.container && obj.parent) {
                            var parent = this.nameTable[obj.parent];
                            if (parent && parent.container) {
                                if (parent.container.type == "stack") {
                                    if (parent.children[0]) {
                                        this.tools.move._updateContainer(this.nameTable[parent.children[0]]);
                                        ej.datavisualization.Diagram.DiagramContext.update(parent, this);
                                    } else {
                                        var prePar = this.nameTable[parent.parent];
                                        if (prePar) {
                                            this._remove(prePar);
                                        }
                                    }
                                }

                            }
                        }
                    }
                }
                this._clearSelection();
            }
        },
        _getEdgeTable: function (group, edgeTable) {
            var edges, len, i, j, depend;
            if (!group.segments) {
                if (group.isGroup) {
                    for (i = 0; i < group.children.length; i++) {
                        var node = this.nameTable[group.children[i]];
                        if (node.inEdges && node.inEdges.length > 0) {
                            edges = node.inEdges;
                            for (j = 0, len = edges.length; j < len; j++) {
                                depend = this.nameTable[edges[j]];
                                var table = edgeTable[depend.name] = $.extend(true, {}, depend);
                            }
                        }
                        if (node.outEdges && node.outEdges.length > 0) {
                            edges = node.outEdges;
                            for (j = 0, len = edges.length; j < len; j++) {
                                depend = this.nameTable[edges[j]];
                                var table = edgeTable[depend.name] = $.extend(true, {}, depend);
                            }
                        }
                    }
                }
                else {
                    if (group.inEdges && group.inEdges.length > 0) {
                        edges = group.inEdges;
                        for (j = 0, len = edges.length; j < len; j++) {
                            depend = this.nameTable[edges[j]];
                            var table = edgeTable[depend.name] = $.extend(true, {}, depend);
                        }
                    }
                    if (group.outEdges && group.outEdges.length > 0) {
                        edges = group.outEdges;
                        for (j = 0, len = edges.length; j < len; j++) {
                            depend = this.nameTable[edges[j]];
                            var table = edgeTable[depend.name] = $.extend(true, {}, depend);
                        }
                    }
                }
            }
            return edgeTable;
        },

        _remove: function (obj, dependent) {
            if (obj) {
                if (obj.segments) var type = "connector";
                else type = "node";
                var dependent = dependent ? ej.datavisualization.Diagram.dependentconnector.dependent : ej.datavisualization.Diagram.dependentconnector.independent;
                var args = { changeType: "remove", element: obj, cancel: false, dependent: dependent };
                this._raiseEvent(type + "CollectionChange", args);
                if (!args.cancel) {
                    if (ej.datavisualization.Diagram.Util.canDelete(obj)) {
                        var name = obj.name;
                        if (!obj.segments) {
                            this._disConnect(obj);
                            if (!obj.isGroup)
                                this._removeConnector(obj, args);
                            ej.datavisualization.Diagram.Util.removeItem(this.model.nodes, this.nameTable[obj.name]);
                        } else {
                            this._removeEdges(obj);
                            obj.sourceNode = obj.sourcePort = obj.targetNode = obj.targetPort = null;
                            ej.datavisualization.Diagram.Util.removeItem(this.model.connectors, this.nameTable[obj.name]);
                        }
                        if (obj.shape || obj.segments) {
                            ej.datavisualization.Diagram.SpatialUtil._removeFromaQuad(this._spatialSearch, this._spatialSearch.quadTable[obj.name], obj);
                            ej.datavisualization.Diagram.SpatialUtil._updateBounds(this, this._spatialSearch, obj);
                            if (obj.segments)
                                this._updateConnectorBridging(obj);
                        }
                        if (obj.parent) {
                            var parent = this.nameTable[obj.parent];
                            if (parent && parent.isGroup) {
                                ej.datavisualization.Diagram.Util.removeItem(parent.children, obj.name);
                                ej.datavisualization.Diagram.Util._updateGroupBounds(parent, this.nameTable);
                            }
                        }
                        delete this.nameTable[name];
                        var element = this._svg.getElementById(obj.name);
                        if (ej.browserInfo().name === "msie") {
                            if (obj.shape && obj.shape.html && element) {
                                var htmlelement = document.getElementById(obj.name + "_parentdiv");
                                htmlelement.parentNode.removeChild(htmlelement);
                            } else if (obj.isGroup)
                                this._checkForHtmlNode(obj);
                        }
                        if (obj.isGroup && !obj.isPseudoGroup) {
                            this._removeChildren(obj, args);
                        }
                        this._removeElement(obj);
                        if (obj.isGroup && obj.container) {
                            ej.datavisualization.Diagram.SvgContext._removephases(obj, this);
                        }
                        if (element && element.parentNode)
                            element.parentNode.removeChild(element);
                        if (this.selectionList[0]) {
                            this._clearSelection();
                        }
                    }
                }
            }
        },

        _removeConnector: function (node, args) {
            var edges;
            var connector;
            var i, len;
            if (!node.segments) {
                //If adjustDependent is false all connector associated with Nodes get deleted.
                if (this.model.deleteDependent && !(this.model.adjustDependent)) {
                    if (node.inEdges && node.inEdges.length > 0) {
                        edges = node.inEdges;
                        for (i = edges.length - 1 ; i >= 0; i--) {
                            //Remove the Connector
                            this._updateConnection(this.nameTable[edges[i]], true);
                        }
                    }
                    if (node.outEdges && node.outEdges.length > 0) {
                        edges = node.outEdges;
                        for (i = 0, len = edges.length; i < len; i++) {
                            //Remove the Connector
                            this._updateConnection(this.nameTable[edges[i]], true);
                        }
                    }
                }
                //If both deleteDependent & adjustDependent are true changes to Visio behaviour
                if (this.model.deleteDependent && this.model.adjustDependent) {
                    if (node.inEdges && node.inEdges.length == 0 && node.outEdges.length > 0) {
                        edges = node.outEdges;
                        for (i = edges.length - 1 ; i >= 0; i--) {
                            //Remove the Connector
                            this._updateConnection(this.nameTable[edges[i]], true);
                        }
                    }
                    else if (node.outEdges && node.outEdges.length == 0 && node.inEdges.length > 0) {
                        edges = node.inEdges;
                        for (i = edges.length - 1 ; i >= 0; i--) {
                            //Remove the Connector
                            if (this.nameTable[edges[i]])
                                this._updateConnection(this.nameTable[edges[i]], true);
                        }
                    }
                    else if ((node.inEdges && node.inEdges.length == 1 || node.inEdges.length > 1) && (node.outEdges.length == 1)) {
                        edges = node.outEdges;
                        for (i = 0, len = edges.length; i < len; i++) {
                            //Remove the Connector
                            this._updateConnection(this.nameTable[edges[i]], true);
                        }
                    }
                    else if (!(node.inEdges && node.inEdges.length > 1 && node.outEdges.length > 1)) {
                        edges = node.inEdges;
                        for (i = 0, len = edges.length; i < len; i++) {
                            //Remove the Connector
                            this._updateConnection(this.nameTable[edges[i]], true);
                        }
                    }
                }
            }
        },

        _updateConnection: function (connector, args) {
            this._remove(connector, args);
            this.updateConnector(connector.name);
        },
        _removeElement: function (element) {
            for (var i = 1; i < this._views.length; i++) {
                var view = this._views[i];
                this._views[view].context.removeChild(element, this._views[view]);
            }

        },
        _disConnect: function (node) {
            var edgesIn, edgesOut, connector, parentNode, targetNode, data;
            var i, j, leni, lenj;

            if (this.model.deleteDependent && this.model.adjustDependent) {
                if (node.inEdges && node.inEdges.length > 0 && node.outEdges.length > 0) {
                    edgesIn = node.inEdges;
                    edgesOut = node.outEdges;
                    if ((node.inEdges.length > 1 || node.inEdges.length == 1) && node.outEdges.length == 1) {
                        for (i = 0, leni = edgesIn.length; i < leni; i++) {
                            if ((this.nameTable[edgesIn[i]].targetNode == this.nameTable[edgesOut[0]].sourceNode)) {
                                targetNode = this.nameTable[this.nameTable[edgesOut[0]].targetNode];
                                data = $.extend(true, {}, targetNode);
                                this.nameTable[edgesIn[i]].targetNode = this.nameTable[edgesOut[0]].targetNode;
                                this.nameTable[edgesIn[i]].targetPort = this.nameTable[edgesOut[0]].targetPort;
                                targetNode.inEdges[i] = this.nameTable[edgesIn[i]].name;
                                //Updating the Connector Dock and Connector
                                ej.datavisualization.Diagram.Util.dock(this.nameTable[edgesIn[i]], this.nameTable);
                                ej.datavisualization.Diagram.DiagramContext.update(this.nameTable[edgesIn[i]], this);
                            }
                        }
                        this.nameTable[edgesOut[0]].sourceNode = null;
                        this.nameTable[edgesOut[0]].sourcePort = null;
                    }
                    else if (!(node.inEdges.length > 1 && node.outEdges.length > 1)) {
                        // Need to code when many in and only one out 
                        for (i = 0, leni = edgesIn.length; i < leni; i++) {
                            if (node.outEdges.length > 0) {
                                parentNode = this.nameTable[this.nameTable[edgesIn[i]].sourceNode];
                                data = $.extend(true, {}, parentNode);
                                for (j = 0, lenj = edgesOut.length; j < lenj; j++) {
                                    if ((this.nameTable[edgesIn[i]].targetNode == this.nameTable[edgesOut[j]].sourceNode)) {
                                        this.nameTable[edgesOut[j]].sourceNode = this.nameTable[edgesIn[i]].sourceNode;
                                        this.nameTable[edgesOut[j]].sourcePort = this.nameTable[edgesIn[i]].sourcePort;
                                        parentNode.outEdges[data.outEdges.length + j] = this.nameTable[edgesOut[j]].name;
                                        //Updating the Connector Dock and Connector
                                        ej.datavisualization.Diagram.Util.dock(this.nameTable[edgesOut[j]], this.nameTable);
                                        ej.datavisualization.Diagram.DiagramContext.update(this.nameTable[edgesOut[j]], this);
                                    }
                                }
                                this.nameTable[edgesIn[i]].targetNode = null;
                                this.nameTable[edgesIn[i]].targetPort = null;
                            }
                        }
                    }
                }
            }
            else {
                if (node.inEdges && node.inEdges.length > 0) {
                    edgesIn = node.inEdges;
                    for (i = 0, leni = edgesIn.length; i < leni; i++) {
                        connector = this.nameTable[edgesIn[i]];
                        connector.targetNode = null;
                        connector.targetPort = null;
                    }
                }
                if (node.outEdges && node.outEdges.length > 0) {
                    edgesOut = node.outEdges;
                    for (i = 0, leni = edgesOut.length; i < leni; i++) {
                        connector = this.nameTable[edgesOut[i]];
                        connector.sourceNode = null;
                        connector.sourcePort = null;
                    }
                }
            }
        },

        _removeEdges: function (connector) {
            if (connector.targetNode) {
                if (this.nameTable[connector.targetNode] && this.nameTable[connector.targetNode].inEdges)
                    ej.datavisualization.Diagram.Util.removeItem(this.nameTable[connector.targetNode].inEdges, connector.name);
            }
            if (connector.sourceNode) {
                if (this.nameTable[connector.sourceNode] && this.nameTable[connector.sourceNode].outEdges)
                    ej.datavisualization.Diagram.Util.removeItem(this.nameTable[connector.sourceNode].outEdges, connector.name);
            }
        },

        _selectionContains: function (node) {
            if (this._hasSelection() && node)
                for (var i = 0; i < this.selectionList.length; i++) {
                    if (node.name === this.selectionList[i].name)
                        return true;
                }
            return false;
        },

        _addSelection: function (selectedShape) {
            if (selectedShape && ej.datavisualization.Diagram.Util.canSelect(selectedShape)) {
                if (!this._selectionContains(selectedShape)) {
                    this.selectionList.push(selectedShape);
                    var args = { changeType: "insert", element: selectedShape, cancel: false };
                    if (!this._selectedSymbol)
                        this._raiseEvent("selectionChange", args);

                    if (!args.cancel) {
                        ej.datavisualization.Diagram.SvgContext.renderSelector(selectedShape, this._svg, this._adornerLayer, this._currZoom,
                            this.model.selectorConstraints);
                        if ((this.model.userHandles !== null && this.model.userHandles.length > 0)) {
                            var isMultipleSelection = false;
                            if (selectedShape.isPseudoGroup)
                                isMultipleSelection = true;
                            if (this.model.selectorConstraints & ej.datavisualization.Diagram.SelectorConstraints.UserHandles)
                                ej.datavisualization.Diagram.SvgContext.renderUserHandles(this.model.userHandles, selectedShape, this._svg, isMultipleSelection, this._currZoom, this._adornerLayer);
                        }
                    }
                    else {
                        ej.datavisualization.Diagram.Util.removeItem(this.selectionList, selectedShape);
                    }
                }
            }
        },
        _raiseEvent: function (type, args) {
            args.elementType = this._getElementType(args.element);
            return this._trigger(type, args);

        },
        _clearSelection: function () {
            if (this._isEditing && this.selectionList.length > 0) {
                this._endEdit();
            }
            var args = { changeType: "remove", element: this.selectionList, cancel: false };
            if (!this._selectedSymbol)
                this._raiseEvent("selectionChange", args);
            if (!args.cancel) {
                if (this.selectionList[0] && this.selectionList[0].isPseudoGroup) {
                    ej.datavisualization.Diagram.Util.removeItem(this.model.nodes, this.selectionList[0]);
                    delete this.nameTable[this.selectionList[0].name];
                }
                ej.datavisualization.Diagram.Util.clear(this.selectionList);
                ej.datavisualization.Diagram.SvgContext.clearSelector(this._svg, this._adornerLayer);

            }
        },

        _updateSelectionHandle: function (isDragging) {
            if (this.selectionList.length > 0) {
                var shape = this.nameTable[this.selectionList[0].name];
                var constraints = this.model.selectorConstraints;
                if (isDragging && (this.model.selectorConstraints & ej.datavisualization.Diagram.SelectorConstraints.UserHandles))
                    constraints = constraints &~ ej.datavisualization.Diagram.SelectorConstraints.UserHandles;
                ej.datavisualization.Diagram.SvgContext.updateSelector(shape, this._svg, this._currZoom, this, constraints);
                if ((this.model.userHandles !== null && this.model.userHandles.length > 0)) {
                    var isMultipleSelection = false;
                    if (shape.isPseudoGroup)
                        isMultipleSelection = true;
                    if (this.model.selectorConstraints & ej.datavisualization.Diagram.SelectorConstraints.UserHandles)
                        ej.datavisualization.Diagram.SvgContext.updateUserHandles(this.model.userHandles, shape, this._svg, isMultipleSelection, isDragging, this._currZoom);
                }
            }
        },

        _hasSelection: function () {
            return this.selectionList.length > 0;
        },

        _checkToolToActivate: function (evt, node, point) {
            var success = false;
            if (node) {
                if (node.segments && (evt.ctrlKey && evt.shiftKey) || (evt.ctrlKey && evt.altKey)) {
                    this._toolToActivate = "endPoint";
                    this._currentCursor = "move";
                    success = true;
                }
                if (!success) {
                    if (this._selectionContains(node)) {
                        if (node.segments) {
                            success = this._canActivateEndPointTool(evt, node, point);
                        }
                        else {
                            success = this._canActivateResizeTool(evt, node, point);
                            if (!success) {
                                success = this._canActivateRotateTool(evt, node, point);
                            }
                        }
                        if (!success) {
                            success = this._canActivateUserHandle(evt, node, point);
                        }
                        if (!success) {
                            success = this._canActivateMoveTool(evt, node, point);
                        }
                        if (!success) {
                            success = this._canActivatephaseTool(evt, point);
                        }
                    }
                    else {
                        success = this._canActivateMoveTool(evt, node, point);
                    }
                }
            }
            else {
                success = this._canActivatephaseTool(evt, point);
            }
            if (!success) {
                if (this.model.tool & ej.datavisualization.Diagram.Tool.ZoomPan) {
                    this._toolToActivate = "panTool";
                    this._currentCursor = "pointer";
                }
                else {
                    this._toolToActivate = "select";
                    this._currentCursor = "default";
                }
            }
            if (this._toolToActivate != "rotate") {
                this._svg.document.removeAttribute("class");
            }
            return success;
        },

        _canActivatephaseTool: function (evt, point) {
            if ($(evt.target).parents("g").first()[0]) {
                var id = $(evt.target).parents("g").first()[0].id;
                var phase = this._getSeperetor(id);
                if (phase) {
                    if (phase.orientation == "horizontal")
                        this._currentCursor = "e-resize";
                    else
                        this._currentCursor = "n-resize";
                }
                else
                    this._currentCursor = "default";
                this._toolToActivate = "phase";

                return true;
            }
        },

        _canActivateUserHandle: function (evt) {
            if (this._isUserHandle(evt)) {
                var handleName = $(evt.target).parents("g").first().context.id.split("_")[0];
                this._toolToActivate = handleName;
                this._currentCursor = this.tools[this._toolToActivate].cursor;
                return true;
            }
            return false;
        },

        _canActivateEndPointTool: function (evt, connector) {
            if (connector) {
                var endPointHandle = evt.target.getAttribute("class");
                if (endPointHandle && ((endPointHandle === "targetEndPoint" && ej.datavisualization.Diagram.Util.canDragTargetEnd(connector)) ||
                    (endPointHandle === "sourceEndPoint" && ej.datavisualization.Diagram.Util.canDragSourceEnd(connector)) ||
                (endPointHandle.match("bezierpoint") && ej.datavisualization.Diagram.Util.canDragSegmentThumbs(connector)) ||
                    endPointHandle == "segmentEnd")) {
                    this._toolToActivate = "endPoint";
                    this._currentCursor = "move";
                    return true;
                }
            }
            return false;
        },

        _canActivateResizeTool: function (evt, node) {
            if (ej.datavisualization.Diagram.Util.canResize(node) && (this.model.selectorConstraints & ej.datavisualization.Diagram.SelectorConstraints.Resizer)) {
                var dir = evt.target.getAttribute("class");
                if (dir === "nw-resize" || dir === "n-resize" || dir === "ne-resize" || dir === "w-resize" || dir === "e-resize" || dir === "sw-resize" || dir === "s-resize" || dir === "se-resize") {
                    this._toolToActivate = "resize";
                    if ((node.rotateAngle >= 0 && node.rotateAngle < 25) || (node.rotateAngle >= 160 && node.rotateAngle <= 200) || (node.rotateAngle >= 340 && node.rotateAngle <= 360)) {
                        this._currentCursor = dir;
                    }
                    else if ((node.rotateAngle >= 25 && node.rotateAngle <= 65) || (node.rotateAngle >= 205 && node.rotateAngle <= 240)) {

                        if (dir === "n-resize" || dir === "s-resize") {
                            this._currentCursor = "ne-resize";
                        }
                        else if (dir === "nw-resize" || dir === "se-resize") {
                            this._currentCursor = "n-resize";
                        }
                        else if (dir === "e-resize" || dir === "w-resize") {
                            this._currentCursor = "nw-resize";
                        }
                        else {
                            this._currentCursor = "e-resize";
                        }
                    }
                    else if ((node.rotateAngle >= 70 && node.rotateAngle <= 110) || (node.rotateAngle >= 250 && node.rotateAngle <= 290)) {
                        if (dir === "n-resize" || dir === "s-resize") {
                            this._currentCursor = "e-resize";
                        }
                        else if (dir === "nw-resize" || dir === "se-resize") {
                            this._currentCursor = "ne-resize";
                        }
                        else if (dir === "e-resize" || dir === "w-resize") {
                            this._currentCursor = "n-resize";
                        }
                        else {
                            this._currentCursor = "nw-resize";
                        }
                    }
                    else if ((node.rotateAngle >= 115 && node.rotateAngle <= 155) || (node.rotateAngle >= 295 && node.rotateAngle <= 335)) {
                        if (dir === "n-resize" || dir === "s-resize") {
                            this._currentCursor = "nw-resize";
                        }
                        else if (dir === "nw-resize" || dir === "se-resize") {
                            this._currentCursor = "e-resize";
                        }
                        else if (dir === "e-resize" || dir === "w-resize") {
                            this._currentCursor = "ne-resize";
                        }
                        else {
                            this._currentCursor = "n-resize";
                        }
                    }
                    return true;
                }
            }
            return false;
        },

        _canActivateMoveTool: function (evt, node) {
            var findNode = this._findNodeUnderMouse(evt);
            if (findNode && findNode.name === node.name) {
                var canMove = true;
                if (node.parent) {
                    if (this.selectionList.length > 0 && this._selectionContains(node))
                        canMove = true;
                    else
                        canMove = false;
                }
                if (canMove && ej.datavisualization.Diagram.Util.canMoveLabel(node)) {
                    var id = evt.target.id.split('_');
                    if (id[id.length - 1] == "lblbg") {
                        this._toolToActivate = "move";
                        this._currentCursor = "pointer";
                        return true;
                    }
                }

                if (ej.datavisualization.Diagram.Util.canMove(node)) {
                    this._toolToActivate = "move";
                    if (this.selectionList.length > 0 && (this._selectionContains(node) || (this.selectionList[0].children && this._findChildren(this.selectionList[0], node.name)))) {
                        this._currentCursor = "move";
                    }
                    else {
                        this._currentCursor = "default";
                    }
                    return true;
                }
            }
            return false;
        },

        _canActivateRotateTool: function (evt, node) {
            if (ej.datavisualization.Diagram.Util.canRotate(node) && (this.model.selectorConstraints & ej.datavisualization.Diagram.SelectorConstraints.Rotator)) {
                var rotateHandle = evt.target.getAttribute("class");
                if (rotateHandle === "rotate") {
                    this._toolToActivate = "rotate";
                    this._svg.document.setAttribute("class", "svg-rotate");
                    this._currentCursor = "";
                    return true;
                }
            }
            return false;
        },

        _canActivatePivotTool: function (evt) {
            var pivotHandle = evt.target.getAttribute("class");
            if (pivotHandle === "pivot") {
                this._toolToActivate = "pivot";
                this._currentCursor = "default";
                return true;
            }

            return false;
        },

        _updateCursor: function () {
            this._svg.document.style.cursor = this._currentCursor;
        },

        _getResizeCursor: function (resizeDirection) {
            var cursor = "default";
            switch (resizeDirection) {
                case "topleft": cursor = "nw-resize";
                    break;
                case "topcenter": cursor = "n-resize";
                    break;
                case "topright": cursor = "ne-resize";
                    break;
                case "middleleft": cursor = "w-resize";
                    break;
                case "middleright": cursor = "e-resize";
                    break;
                case "bottomleft": cursor = "sw-resize";
                    break;
                case "bottomcenter": cursor = "s-resize";
                    break;
                case "bottomright": cursor = "se-resize";
                    break;
            }
            return cursor;
        },

        _startEdit: function (shape) {
            var label;
            var centerX, centerY;
            var bbox;
            var nodeBounds;
            var bounds;
            var curZoomfactor;
            var matrix = ej.Matrix.identity();
            ej.Matrix.rotate(matrix, shape.rotateAngle, shape.width / 2, shape.height / 2);
            if (shape.shape && shape.shape.type === "text") {
                if (this._svg.getElementById(shape.name + "_shape") !== null)
                    bbox = this._svg.getElementById(shape.name + "_shape").getBBox();
                else
                    bbox = { x: shape.offsetX - shape.width / 2, y: shape.offsetY - shape.height / 2, width: shape.width, height: shape.height };
                label = shape.shape.textBlock;
                if (label) {
                    nodeBounds = ej.datavisualization.Diagram.Util.bounds(shape);
                    bounds = new ej.datavisualization.Diagram.Rectangle(bbox.x - label.fontSize, bbox.y - label.fontSize, bbox.width + label.fontSize / 2, bbox.height + label.fontSize / 2);
                    curZoomfactor = this._currZoom;
                    var diff = ej.Matrix.transform(matrix, ej.datavisualization.Diagram.Util._getLabelPosition(label, nodeBounds));
                    centerX = shape.offsetX - shape.width / 2 + diff.x;
                    centerY = shape.offsetY - shape.height / 2 + diff.y;
                    if (label.horizontalAlignment == "left")
                        centerX += bbox.width / 2;
                    else if (label.horizontalAlignment == "right")
                        centerX -= bbox.width / 2;
                    if (label.verticalAlignment == "top")
                        centerY += bbox.height / 2;
                    else if (label.verticalAlignment == "bottom")
                        centerY -= bbox.height / 2;
                    if (bbox.width === 0 || bbox.height === 0) {
                        bbox = ej.datavisualization.Diagram.Util.bounds(shape);
                        bounds.width = bbox.width == 0 ? 50 : nodeBounds.width * curZoomfactor;
                        bounds.x = (centerX * curZoomfactor - bounds.width / 2 * curZoomfactor) - this._hScrollOffset;

                        bounds.height = label.fontSize;
                        bounds.y = (centerY * curZoomfactor - bounds.height / 2 * curZoomfactor) - this._vScrollOffset;
                    }
                    else {
                        bounds.width = 50;
                        bounds.height = label.fontSize;
                        bounds.x = (centerX * curZoomfactor - (bounds.width / 2 * curZoomfactor)) - this._hScrollOffset;
                        bounds.y = (centerY * curZoomfactor - (bounds.height / 2 * curZoomfactor)) - this._vScrollOffset;
                    }
                    this._boundingBox = bounds;
                    this._createEditBox(label, bounds.x, bounds.y, bounds.width, bounds.height, shape);
                    this._isEditing = true;
                }
            }
            else {
                if (shape.labels.length > 0) {
                    if (this.activeTool.name === "textTool") {
                        label = this.activeTool.getLabelUnderMouse();
                        if (label)
                            label = shape.labels[0];
                    }
                    else
                        label = shape.labels[0];
                }
                else {
                    var mNode = this._findNode(shape.name);
                    label = new ej.datavisualization.Diagram.Label({ "name": "label" + ej.datavisualization.Diagram.Util.randomId() });
                    shape.labels.push(label);
                    var parent = (mNode !== shape) ? mNode : null;
                    ej.datavisualization.Diagram.DiagramContext.addNodeLabel(shape, label, parent, this);
                    if (this.activeTool.name === "textTool")
                        this.activateTool.activeLabel = label;
                }
                if (label && !label.readOnly) {
                    bbox = this._svg.getElementById(shape.name + "_" + label.name).getBBox();
                    nodeBounds = ej.datavisualization.Diagram.Util.bounds(shape);
                    bounds = new ej.datavisualization.Diagram.Rectangle(bbox.x, bbox.y, bbox.width, bbox.height);
                    curZoomfactor = this._currZoom;
                    var position = ej.datavisualization.Diagram.Util._getLabelPosition(label, nodeBounds);
                    if (shape.segments) {
                        centerX = nodeBounds.x + position.x;
                        centerY = nodeBounds.y + position.y;
                    }
                    else {
                        var diff = ej.Matrix.transform(matrix, position);
                        centerX = shape.offsetX - shape.width / 2 + diff.x;
                        centerY = shape.offsetY - shape.height / 2 + diff.y;
                        if (label.horizontalAlignment == "left")
                            centerX += bbox.width / 2;
                        else if (label.horizontalAlignment == "right")
                            centerX -= bbox.width / 2;
                        if (label.verticalAlignment == "top")
                            centerY += bbox.height / 2;
                        else if (label.verticalAlignment == "bottom")
                            centerY -= bbox.height / 2;
                    }
                    if (bbox.width === 0 || bbox.height === 0) {
                        bbox = ej.datavisualization.Diagram.Util.bounds(shape);
                        if (!shape.shape) {
                            bounds.width = bbox.width < 50 ? 50 : nodeBounds.width * curZoomfactor;
                            bounds.x = (centerX * curZoomfactor - bounds.width / 2 * curZoomfactor) - this._hScrollOffset;
                        }
                        else {
                            bounds.width = bbox.width < 50 ? 50 : (nodeBounds.width * curZoomfactor);
                            bounds.x = (centerX * curZoomfactor - bounds.width / 2) - this._hScrollOffset;
                        }
                        bounds.height = label.fontSize;
                        bounds.y = (centerY * curZoomfactor - bounds.height / 2 * curZoomfactor) - this._vScrollOffset;
                    }
                    else {
                        bounds.x = (centerX * curZoomfactor - (bounds.width / 2 * curZoomfactor)) - this._hScrollOffset;
                        bounds.y = (centerY * curZoomfactor - (bounds.height / 2 * curZoomfactor)) - this._vScrollOffset;
                    }
                    this._boundingBox = bounds;
                    this._createEditBox(label, bounds.x, bounds.y, bounds.width, bounds.height, shape);
                    this._isEditing = true;
                }
            }
        },

        _createEditBox: function (label, x, y, width, height, shape) {
            var nodeBounds = ej.datavisualization.Diagram.Util.bounds(shape);
            var div = document.createElement("div");

            var attr = {
                "id": this.element[0].id + "_editBoxDiv",
                "class": "edit",
                "style": "top:" + y + "px;left:" + x + "px;width:" + width + "px;height:" + height + "px;" +
                    "font-family:" + label.fontFamily + ";font-size:" + label.fontSize + "px;min-width: 50px;min-height:" + label.fontSize + "px;",
            };
            ej.datavisualization.Diagram.Util.attr(div, attr);
            var textBox = document.createElement("textarea");
            attr = {
                "id": this.element[0].id + "_editBox",
                "class": "editBox",
                "style": "font-size:" + label.fontSize + "px;font-family:" + label.fontFamily + ";min-height:" + label.fontSize + "px;line-height:" + label.fontSize + "px;",
                "nodeWidth": nodeBounds.width, "nodeHeight": nodeBounds.height, "fontSize": label.fontSize, "nodeOffsetX": shape.offsetX, "nodeOffsetY": shape.offsetY, "containerName": shape.name
            };
            ej.datavisualization.Diagram.Util.attr(textBox, attr);
            textBox.value = label.text;
            div.appendChild(textBox);
            $(this._canvas).append(div);

            if (label.text !== "") {
                this._updateEditor(null, div, textBox, nodeBounds.width, label.fontSize);
            }
            this._on($(textBox), "focusout", this._editboxfocusout);
            this._on($(textBox), "keyup", this._editBoxKeyUp);
            this._on($(textBox), "input", this._editBoxTextChange);
            textBox.focus();
            textBox.select();
        },

        _editBoxKeyUp: function (evt) {
            this._updateLabelPosition(evt);
        },

        _editBoxTextChange: function (evt) {
            this._updateLabelPosition(evt);
        },

        _updateEditBox: function (x, y) {
            var editBox = document.getElementById(this.element[0].id + "_editBoxDiv");
            if (editBox) {
                editBox.style.left = editBox.offsetLeft - x + "px";
                editBox.style.top = editBox.offsetTop - y + "px";
            }
        },

        _editboxfocusout: function () {
            this._endEdit();
        },

        _updateLabelPosition: function (evt) {
            var minWidth = 50;
            var curZoomfactor = this._currZoom;
            var parentBoundary = this._boundingBox;
            var fontSize = Number(evt.target.getAttribute("fontSize"));
            var nodeWidth = Number(evt.target.getAttribute("nodeWidth"));
            var conCenterX = parentBoundary.x + (parentBoundary.width / 2);
            var conCenterY = parentBoundary.y + (parentBoundary.height / 2);
            var bounds = this._textLength(evt);

            var width;
            if (bounds.width === 0) {
                width = parentBoundary.width < minWidth ? minWidth : parentBoundary.width;
                evt.target.parentNode.style.width = width + "px";
                evt.target.parentNode.style.left = conCenterX - width / 2 + "px";
            }
            else {
                if (bounds.width <= (nodeWidth * curZoomfactor))
                    width = bounds.width < minWidth ? minWidth : bounds.width;
                else
                    width = nodeWidth < minWidth ? minWidth : (nodeWidth * curZoomfactor);
                evt.target.parentNode.style.width = width + "px";
                evt.target.parentNode.style.left = conCenterX - (width / 2) + "px";
            }
            if (bounds.height === 0) {
                evt.target.style.height = fontSize + "px";
                evt.target.parentNode.style.top = conCenterY - fontSize / 2 + 'px';
            }
            else {
                this._updateEditor(evt, evt.target.parentNode, evt.target, nodeWidth, fontSize);
            }
        },

        _updateEditor: function (evt, div, textBox, nodeWidth, fontSize) {
            var offsetX, offsetY;
            var elHeight;
            var bounds = this._textLength(evt);
            var curZoomfactor = this._currZoom;
            var minWidth = 50;
            var x = this._boundingBox.x;
            var y = this._boundingBox.y;
            var width = this._boundingBox.width;
            var height = this._boundingBox.height;

            var str = textBox.value;
            var line = str.split('\n');

            var temp = (nodeWidth * curZoomfactor) < bounds.width ? (nodeWidth * curZoomfactor) : bounds.width;
            var elWidth = temp < minWidth ? minWidth : temp;
            div.style.width = elWidth + "px";
            if (evt)
                offsetX = (elWidth - width) / 2;
            else
                offsetX = (elWidth - width * curZoomfactor) / 2;
            div.style.left = x - offsetX + "px";

            if (line.length > 1 || bounds.width >= nodeWidth) {
                if (ej.browserInfo().name === "mozilla")
                    elHeight = textBox.scrollHeight;
                else
                    elHeight = textBox.scrollHeight - 4;
                textBox.style.height = 'auto';
            }
            else
                elHeight = fontSize;

            textBox.style.height = elHeight + "px";
            if (evt)
                offsetY = (elHeight - height) / 2;
            else
                offsetY = (elHeight - height * curZoomfactor) / 2;
            this._boundingBox = new ej.datavisualization.Diagram.Rectangle(x - offsetX, y - offsetY, elWidth, elHeight);
            div.style.top = y - offsetY + "px";
        },

        _textLength: function (evt) {
            var temp = document.createElement("div");
            var textElement = document.getElementById(this.element[0].id + "_editBox");
            var attr = {
                "id": this.element[0].id + "_editBoxHiddenDiv",
                "style": "position:absolute; width:" + textElement.width + "px;height:" + textElement.height + "px;" +
                         "font-size:" + textElement.style.fontSize + "px;font-family:" + textElement.style.fontFamily + ";padding: 4px;word-wrap: break-word;white-space:pre-wrap;",
            };
            ej.datavisualization.Diagram.Util.attr(temp, attr);
            if (evt) {
                temp.innerHTML = evt.target.value;
            } else {
                temp.innerHTML = textElement.value;
            }
            document.body.appendChild(temp);
            var bounds = temp.getBoundingClientRect();
            document.body.removeChild(temp);
            return bounds;
        },

        _endEdit: function () {
            var args, entry;
            var editBox = document.getElementById(this.element[0].id + "_editBox");
            if (editBox) {
                var shape = this._findNode(editBox.getAttribute("containerName"));;
                if (this.activeTool.name === "textTool")
                    shape = this.activeTool.selectedObject;
                else if (!shape) {
                    for (var i = 0; i < this.selectionList.length; i++) {
                        var mNode = this.selectionList[i];
                        if (typeof mNode.labels.length != "undefined" && mNode.labels.length > 0 && mNode.labels[0].mode === ej.datavisualization.Diagram.LabelEditMode.Edit)
                            shape = this.selectionList[i];
                    }
                }

                if (shape) {
                    this._isEditing = false;
                    args = this._raiseEvent("textChange", { element: shape, value: editBox.value });

                    if (!args) {
                        if (shape && shape.shape !== undefined && shape.shape.type === "text") {
                            var oldText = shape.shape.textBlock.text;
                            shape.shape.textBlock.text = editBox.value;
                            shape.shape.textBlock.mode = ej.datavisualization.Diagram.LabelEditMode.View;
                            if (!this._findNode(shape.name)) {
                                if (editBox.value !== "") {
                                    var height = Number(editBox.style.height.substring(0, editBox.style.height.length - 2));
                                    if (shape.height < height) {
                                        shape.offsetY = (shape.offsetY - (shape.height / 2)) + (height / 2);
                                        shape.height = height;
                                    }
                                    if (this.selectionList.length > 0)
                                        this._clearSelection();
                                    this.add(shape);
                                }
                                this.activeTool.selectedObject = null;
                            } else {
                                ej.datavisualization.Diagram.DiagramContext.update(shape, this);

                                entry = { type: "labelchanged", shape: shape, previousLabelValue: oldText, updatedLabelValue: editBox.value };
                                this.addHistoryEntry(entry);
                                //entry = new ej.datavisualization.Diagram.HistoryEntry(new ej.datavisualization.Diagram.TextChangeCmd(shape, shape.shape.textBlock.text, editBox.value));
                                //this.historyManager.addHistoryEntry(entry);
                            }
                        } else if (shape != undefined && !(shape.isPseudoGroup)) {
                            var label;
                            if (this.activeTool.name === "textTool")
                                label = this.activeTool.getLabelUnderMouse();
                            else
                                label = shape.labels[0];
                            if (label != undefined) {
                                if (label.text != editBox.value) {

                                    entry = { type: "labelchanged", shape: shape, previousLabelValue: label.text, updatedLabelValue: editBox.value };
                                    this.addHistoryEntry(entry);
                                    //entry = new ej.datavisualization.Diagram.HistoryEntry(new ej.datavisualization.Diagram.LabelChangeCmd(shape, label.text, editBox.value));
                                    //this.historyManager.addHistoryEntry(entry);
                                }
                                label.text = editBox.value;
                                label.mode = ej.datavisualization.Diagram.LabelEditMode.View;
                                ej.datavisualization.Diagram.DiagramContext.updateLabel(shape, label, this);
                            }
                            else if (shape.labels.length == 0) {
                                label = ej.datavisualization.Diagram.Label();
                                label.text = editBox.value;
                                label.mode = ej.datavisualization.Diagram.LabelEditMode.View;
                                shape.labels.push(label);
                                ej.datavisualization.Diagram.DiagramContext.addNodeLabel(shape, label, null, this);
                            }
                        }
                    }
                }

                this._off($(editBox), "focusout", this._editboxfocusout);
                this._off($(editBox), "keyup", this._editBoxKeyUp);
                this._off($(editBox), "input", this._editBoxTextChange);
                var element = $("#" + this.element[0].id + "_editBoxDiv")[0];
                if (element)
                    element.parentNode.removeChild(element);
            }
        },

        _setLabelEditing: function (label) {
            var success;
            if (label) {
                if (label.readOnly) {
                    label.mode = ej.datavisualization.Diagram.LabelEditMode.View;
                    success = false;
                }
                else {
                    if (this._currentLabel)
                        this._currentLabel.mode = ej.datavisualization.Diagram.LabelEditMode.View;
                    label.mode = ej.datavisualization.Diagram.LabelEditMode.Edit;
                    this._currentLabel = label;
                    success = true;
                }
                return success;
            }
        },

        _isInternalTool: function (tool) {
            if (tool instanceof ej.datavisualization.Diagram.SelectTool || tool instanceof ej.datavisualization.Diagram.ConnectionEditTool || tool instanceof ej.datavisualization.Diagram.MoveTool || tool instanceof ej.datavisualization.Diagram.ResizeTool
                  || tool instanceof ej.datavisualization.Diagram.PhaseTool || tool instanceof ej.datavisualization.Diagram.RotateTool || (tool instanceof ej.datavisualization.Diagram.PanTool && (this.model.tool & ej.datavisualization.Diagram.Tool.SingleSelect) && (this.model.tool & ej.datavisualization.Diagram.Tool.ZoomPan)))
                return true;
            else
                return false;
        },

        _translate: function (node, dx, dy, nameTable, isContainer) {
            if (ej.datavisualization.Diagram.Util.canMove(node)) {
                if (!node.segments) {
                    node.offsetX += dx;
                    node.offsetY += dy;
                    if (node.isGroup && !isContainer) {
                        var nodes = node.children;
                        var child;
                        for (var i = 0; i < nodes.length; i++) {
                            child = nameTable[nodes[i]];
                            if (child) {
                                this._translate(child, dx, dy, nameTable);
                                if (child.parent && (child.parent != node.name && !node.isPseudoGroup))
                                    ej.datavisualization.Diagram.Util._updateGroupBounds(nameTable[child.parent], nameTable);
                            }
                        }
                        if (!node.isGroup && node.container)
                            ej.datavisualization.Diagram.Util._updateGroupBounds(node, nameTable);
                    }
                    this._updateAssociatedConnectorEnds(node, nameTable);
                } else {
                    ej.datavisualization.Diagram.Util._translateLine(node, dx, dy, node);
                    this._dock(node, nameTable);
                    ej.datavisualization.Diagram.Util.updateBridging(node, this);
                    this._updateConnectorBridging(node);
                }
                this._updateQuad(node);
            }
        },

        _rotate: function (node, angle, nameTable) {
            if (ej.datavisualization.Diagram.Util.canRotate(node)) {
                if (!node.segments) {
                    node.rotateAngle += angle;
                    node.rotateAngle %= 360;
                    if (node.rotateAngle < 0) node.rotateAngle += 360;
                    if (node.isGroup) {
                        var matrix = ej.Matrix.identity();
                        var child, pinx, piny;
                        ej.Matrix.rotate(matrix, angle, node.offsetX, node.offsetY);
                        var nodes = node.children;
                        for (var i = 0; i < nodes.length; i++) {
                            child = nameTable[nodes[i]];
                            if (child) {
                                if (!(child.segments)) {
                                    this._rotate(child, angle, nameTable);
                                    pinx = child.offsetX;
                                    piny = child.offsetY;
                                    var actualpt = ej.Matrix.transform(matrix, ej.datavisualization.Diagram.Point(pinx, piny));
                                    this._translate(child, actualpt.x - pinx, actualpt.y - piny, nameTable);
                                    this._updateAssociatedConnectorEnds(child, nameTable);
                                } else {
                                    if (!child.sourceNode)
                                        this._setEndPoint(child, ej.Matrix.transform(matrix, child.sourcePoint), false);
                                    if (!child.targetNode)
                                        this._setEndPoint(child, ej.Matrix.transform(matrix, child.targetPoint), true);
                                }
                                if (child.parent && (child.parent != node.name && !node.isPseudoGroup))
                                    ej.datavisualization.Diagram.Util._updateGroupBounds(nameTable[child.parent], nameTable);
                            }
                        }

                        ej.datavisualization.Diagram.Util._updateGroupBounds(node, nameTable);
                    }
                    this._updateQuad(node);
                    this._updateAssociatedConnectorEnds(node, nameTable);
                }
            }
        },

        scale: function (node, sw, sh, pivot, nameTable, skipScalOnChild, updateMinMax, isHelper) {
            if (!node.container) {
                //if (ej.datavisualization.Diagram.Util.canResize(node)) {
                var matrix = ej.Matrix.identity();
                if (!node.segments) {
                    if (node.isGroup) {
                        var nodes = node.children;
                        var child;
                        for (var i = 0; i < nodes.length; i++) {
                            child = nameTable[nodes[i]];
                            if (child) {
                                this.scale(child, sw, sh, pivot, nameTable);
                                if (child.parent && (child.parent != node.name && !node.isPseudoGroup))
                                    ej.datavisualization.Diagram.Util._updateGroupBounds(nameTable[child.parent], nameTable);
                            }
                        }
                        ej.datavisualization.Diagram.Util._updateGroupBounds(node, nameTable);
                    } else {
                        ej.Matrix.rotate(matrix, -node.rotateAngle, pivot.x, pivot.y);
                        ej.Matrix.scale(matrix, sw, sh, pivot.x, pivot.y);
                        ej.Matrix.rotate(matrix, node.rotateAngle, pivot.x, pivot.y);
                        var newPosition = ej.Matrix.transform(matrix, ej.datavisualization.Diagram.Point(node.offsetX, node.offsetY));
                        var width = node.width * sw;
                        var height = node.height * sh;
                        if (width > 1) {
                            node.width = width;
                            node.offsetX = newPosition.x;
                        }
                        if (height > 1) {
                            node.height = height;
                            node.offsetY = newPosition.y;
                        }

                    }
                    this._updateAssociatedConnectorEnds(node, nameTable);
                } else {
                    ej.Matrix.scale(matrix, sw, sh, pivot.x, pivot.y);
                    for (var i = 0; i < node.segments.length; i++) {
                        var segment = node.segments[i];
                        for (var j = 0; j < segment.points.length; j++) {
                            segment.points[j] = ej.Matrix.transform(matrix, segment.points[j]);
                            if (i == 0 && j == 0 && !node.sourceNode) {
                                segment._startPoint = segment.points[0];
                                segment._endPoint = segment.points[segment.points.length - 1];
                                ej.datavisualization.Diagram.Util._setLineEndPoint(node, ej.Matrix.transform(matrix, node.sourcePoint), false);
                            }
                            if (i == node.segments.length - 1 && j == segment.points.length - 1 && !node.targetNode) {
                                segment._startPoint = segment.points[0];
                                segment._endPoint = segment.points[segment.points.length - 1];
                                node.targetPoint = segment._endPoint;
                                //ej.datavisualization.Diagram.Util._setLineEndPoint(node, ej.Matrix.transform(matrix, node.targetPoint), true);
                            }
                        }
                        segment._startPoint = segment.points[0];
                        segment._endPoint = segment.points[segment.points.length - 1];
                        if (segment.type == "orthogonal") {
                            if (segment.length || segment.length === 0)
                                segment.length = segment._length = ej.datavisualization.Diagram.Geometry.distance(segment._startPoint, segment._endPoint);
                            else
                                ej.datavisualization.Diagram.Util._addOrthogonalPoints(segment, node.segments[i - 1], node.segments[i + 1], node.sourcePoint, node.targetPoint);
                        } else {
                            segment.point = segment._endPoint;
                        }

                    }
                    if (node.sourceNode || node.targetNode)
                        this._dock(node, nameTable);
                }
                this._updateQuad(node);
                //}
            }
            else {
                switch (node.container.type) {
                    case "canvas":
                        var newSize = {};
                        var matrix = ej.Matrix.identity();
                        ej.Matrix.rotate(matrix, -node.rotateAngle, pivot.x, pivot.y);
                        ej.Matrix.scale(matrix, sw, sh, pivot.x, pivot.y);
                        ej.Matrix.rotate(matrix, node.rotateAngle, pivot.x, pivot.y);
                        newPosition = ej.Matrix.transform(matrix, ej.datavisualization.Diagram.Point(node.offsetX, node.offsetY));
                        width = node.width * sw;
                        height = node.height * sh;

                        if (updateMinMax) {
                            node.minHeight = height;
                            node.minWidth = width;
                        }
                        if (isHelper != true)
                            newSize = this._getMinMaxSize(node, width, height);
                        else {
                            newSize.width = width;
                            newSize.height = height;

                        }
                        if (newSize && newSize.width > 1) {
                            var x = node.offsetX - node.width / 2;
                            node.width = newSize.width;
                            node.offsetX = x + node.width / 2;
                        }
                        if (newSize && newSize.height > 1) {
                            var y = node.offsetY - node.height / 2;
                            node.height = newSize.height;
                            node.offsetY = y + node.height / 2;
                        }
                        if (!skipScalOnChild)
                            ej.datavisualization.Diagram.SvgContext._alignOnCanvas1(node, this, true);
                        break;
                    case "stack":
                        var matrix = ej.Matrix.identity();
                        ej.Matrix.rotate(matrix, -node.rotateAngle, pivot.x, pivot.y);
                        ej.Matrix.scale(matrix, sw, sh, pivot.x, pivot.y);
                        ej.Matrix.rotate(matrix, node.rotateAngle, pivot.x, pivot.y);
                        newPosition = ej.Matrix.transform(matrix, ej.datavisualization.Diagram.Point(node.offsetX, node.offsetY));
                        width = (node.width * sw) - (node.marginLeft + node.marginRight);
                        height = (node.height * sh) - (node.marginTop + node.marginBottom);


                        newSize = this._getMinMaxSize(node, width, height);
                        if (newSize.width > 1) {
                            x = node.offsetX - node.width / 2;
                            node.width = newSize.width - node.marginLeft;
                            node.offsetX = x + node.width / 2 + (node.marginLeft);
                        }
                        if (newSize.height > 1) {
                            y = node.offsetY - node.height / 2;
                            node.height = newSize.height - node.marginTop;
                            node.offsetY = y + node.height / 2 + (node.marginTop);
                        }
                        ej.datavisualization.Diagram.SvgContext._alignOnStack(node, this);
                        break;
                }
            }
        },

        _dock: function (connector, nameTable, updateSelection) {
            var length = connector.segments.length;
            var srcPointslength = connector.segments[0].points.length;
            var targetPointslength = connector.segments[connector.segments.length - 1].points.length;
            ej.datavisualization.Diagram.Util.dock(connector, nameTable);
            if (length != connector.segments.length) {
                ej.datavisualization.Diagram.Util._updateConnectorSegments(connector, connector.sourcePort, connector.targetPort, this);
                ej.datavisualization.Diagram.DiagramContext._refreshSegments(connector, this);
            }
            if (this.selectionList[0] == connector) {
                if (updateSelection || srcPointslength != connector.segments[0].points.length || targetPointslength != connector.segments[connector.segments.length - 1].points.length) {
                    ej.datavisualization.Diagram.SvgContext._refreshEndPointHandles(this.selectionList[0], this._svg, this._currZoom);
                }
            }
            this._updateQuad(connector);
        },

        _setEndPoint: function (connector, point, isTarget) {
            ej.datavisualization.Diagram.Util._setLineEndPoint(connector, point, isTarget);
            this._updateQuad(connector);
        },

        _updateAssociatedConnectorEnds: function (node, nameTable) {
            var i, len;
            var connector;
            for (i = 0, len = node.inEdges.length; i < len; i++) {
                connector = nameTable[node.inEdges[i]];
                if (connector)
                    this._dock(connector, nameTable);
            }
            for (i = 0, len = node.outEdges.length; i < len; i++) {
                connector = nameTable[node.outEdges[i]];
                if (connector)
                    this._dock(connector, nameTable);
            }
        },
        _updateChildBounds: function (node, nameTable) {
            var offX, offY;
            if (node.width == 0 && node.height == 0) {
                offX = node.offsetX;
                offY = node.offsetY;
                ej.datavisualization.Diagram.Util._updateGroupBounds(node, nameTable);
                this._translate(node, offX - node.offsetX, offY - node.offsetY, nameTable);
            }
            else {
                offX = node.offsetX;
                offY = node.offsetY;
                var childBounds = ej.datavisualization.Diagram.Util._getChildrenBounds(node, nameTable);
                node.offsetX = (childBounds.x + childBounds.width / 2);
                node.offsetY = (childBounds.y + childBounds.height / 2);
                var dx = offX - node.offsetX;
                var dy = offX - node.offsetY;
                this._translate(node, dx, dy, nameTable);
                var deltaWidth = node.width / childBounds.width;
                var deltaHeight = node.height / childBounds.height;
                this._scale(node, deltaWidth, deltaHeight, node.pivot, nameTable);;
                this._translate(node, offX - node.offsetX, offY - node.offsetY, nameTable);
            }
        },
        _updateQuad: function (object) {
            if (this._spatialSearch && this._spatialSearch.quadTable[object.name])
                if (object.segments || object.shape)
                    ej.datavisualization.Diagram.SpatialUtil._updateQuad(this, this._spatialSearch, object);
        }
        //#endregion
    });
    ej.datavisualization.Diagram.Locale = {};
    ej.datavisualization.Diagram.Locale["en-US"] = {
        cut: "Cut",
        copy: "Copy",
        paste: "Paste",
        undo: "Undo",
        redo: "Redo",
        selectAll: "SelectAll",
        grouping: "Grouping",
        group: "Group",
        ungroup: "Ungroup",
        order: "Order",
        bringToFront: "BringToFront",
        moveForward: "MoveForward",
        sendBackward: "SendBackward",
        sendToBack: "SendToBack"
    }

    $.fn.pinchDiagram = function (rootgroup, map) {

        var startTouches = [],

        handleTouchStart = function (evt) {
            var touches = evt.touches;
            if (touches.length >= 2) {
                for (var i = 0; i < touches.length; i++) {
                    var touch = touches[i]
                      , found = false;
                    for (var j = 0; j < startTouches.length; j++) {
                        var startTouch = startTouches[j];
                        if (touch.identifier === startTouch.identifier) {
                            found = true;
                            break;
                        }
                    }
                    if (!found) {
                        var touchCopy = $.extend({}, touch);
                        startTouches.push(touchCopy);
                    }
                }
            }

            evt.preventDefault();
        },
        getDistance = function (touch1, touch2) {
            var x = touch2.pageX - touch1.pageX,
                y = touch2.pageY - touch1.pageY;
            return Math.sqrt((x * x) + (y * y));
        },

        handleTouchEnd = function (evt) {
            var changedTouches = evt.changedTouches;
            for (var i = 0; i < changedTouches.length; i++) {
                var changedTouch = changedTouches[i];
                for (var j = 0; j < startTouches.length; j++) {
                    var startTouch = startTouches[j];
                    if (startTouch.identifier === changedTouch.identifier) {
                        var idx = startTouches.indexOf(startTouch);
                        startTouches.splice(idx, 1);
                    }
                }
            }
            evt.preventDefault();
        },

        updateTouch = function (dstTouch, srcTouch) {
            dstTouch.pageX = srcTouch.pageX;
            dstTouch.pageY = srcTouch.pageY;
        },

        handleTouchMove = function (evt) {
            evt.preventDefault();
            var touches = evt.touches, z;
            if (touches.length >= 2) {
                var touch0 = touches[0]
                  , touch1 = touches[1]
                  , startTouch0 = startTouches[0]
                  , startTouch1 = startTouches[1];
                z = getDistance(touch0, touch1) / getDistance(startTouch0, startTouch1);
                var focusPoint = map._mousePosition(evt, false);
                ej.datavisualization.Diagram.ZoomUtil.zoomPan(map, z, 0, 0, focusPoint, true, true);
                updateTouch(startTouch0, touch0);
                updateTouch(startTouch1, touch1);
            }
        };
        this[0].addEventListener('touchstart', handleTouchStart, false);
        this[0].addEventListener('touchend', handleTouchEnd, false);
        this[0].addEventListener('touchmove', handleTouchMove, false);


    };
    //#endregion

})(jQuery, Syncfusion);;
(function ($, ej) {
    "use strict";
    ej.MatrixTypes = { Identity: 0, Translation: 1, Scaling: 2, Unknown: 4 };
    ej.MatrixDefaults = { m11: 1, m12: 0, m21: 0, m22: 1, offsetX: 0, offsetY: 0, type: ej.MatrixTypes.Identity };
    ej.Matrix = {
        identity: function () {
            return $.extend(true, {}, ej.MatrixDefaults);
        },
        multiply: function (matrix1, matrix2) {
            var type = matrix1.type;
            var type2 = matrix2.type;
            if (type2 == ej.MatrixTypes.Identity) {
                return;
            }
            if (type == ej.MatrixTypes.Identity) {
                matrix1.m11 = matrix2.m11;
                matrix1.m12 = matrix2.m12;
                matrix1.m21 = matrix2.m21;
                matrix1.m22 = matrix2.m22;
                matrix1.offsetX = matrix2.offsetX;
                matrix1.offsetY = matrix2.offsetY;
                matrix1.type = matrix2.type;
                return;
            }
            if (type2 == ej.MatrixTypes.Translation) {
                matrix1.offsetX += matrix2.offsetX;
                matrix1.offsetY += matrix2.offsetY;
                if (type != ej.MatrixTypes.Unknown) {
                    matrix1.type |= ej.MatrixTypes.Translation;
                }
                return;
            }
            if (type != ej.MatrixTypes.Translation) {
                var num = type << 4 | type2;
                switch (num) {
                    case 34:
                        matrix1.m11 *= matrix2.m11;
                        matrix1.m22 *= matrix2.m22;
                        return;
                    case 35:
                        matrix1.m11 *= matrix2.m11;
                        matrix1.m22 *= matrix2.m22;
                        matrix1.offsetX = matrix2.offsetX;
                        matrix1.offsetY = matrix2.offsetY;
                        matrix1.type = (ej.MatrixTypes.Translation | ej.MatrixTypes.Scaling);
                        return;
                    case 36: break;
                    default:
                        {
                            switch (num) {
                                case 50:
                                    matrix1.m11 *= matrix2.m11;
                                    matrix1.m22 *= matrix2.m22;
                                    matrix1.offsetX *= matrix2.m11;
                                    matrix1.offsetY *= matrix2.m22;
                                    return;
                                case 51:
                                    matrix1.m11 *= matrix2.m11;
                                    matrix1.m22 *= matrix2.m22;
                                    matrix1.offsetX = matrix2.m11 * matrix1.offsetX + matrix2.offsetX;
                                    matrix1.offsetY = matrix2.m22 * matrix1.offsetY + matrix2.offsetY;
                                    return;
                                case 52: break;
                                default:
                                    switch (num) {
                                        case 66:
                                        case 67:
                                        case 68: break;
                                        default: return;
                                    }
                                    break;
                            }
                            break;
                        }
                }
                var result = this.identity();
                this._setMatrix(result, matrix1.m11 * matrix2.m11 + matrix1.m12 * matrix2.m21,
                    matrix1.m11 * matrix2.m12 + matrix1.m12 * matrix2.m22, matrix1.m21 * matrix2.m11 + matrix1.m22 * matrix2.m21,
                    matrix1.m21 * matrix2.m12 + matrix1.m22 * matrix2.m22,
                    matrix1.offsetX * matrix2.m11 + matrix1.offsetY * matrix2.m21 + matrix2.offsetX, matrix1.offsetX * matrix2.m12 + matrix1.offsetY * matrix2.m22 + matrix2.offsetY);
                if (result.m21 || result.m12) {
                    result.type = ej.MatrixTypes.Unknown;
                }
                else {
                    if (result.m11 && result.m11 != 1.0 || result.m22 && result.m22 != 1.0) {
                        result.type = ej.MatrixTypes.Scaling;
                    }
                    if (result.offsetX || result.offsetY) {
                        result.type |= ej.MatrixTypes.Translation;
                    }
                    if ((result.type & (ej.MatrixTypes.Translation | ej.MatrixTypes.Scaling)) == ej.MatrixTypes.Identity) {
                        result.type = ej.MatrixTypes.Identity;
                    }
                }
                matrix1.m11 = result.m11;
                matrix1.m12 = result.m12;
                matrix1.m21 = result.m21;
                matrix1.m22 = result.m22;
                matrix1.offsetX = result.offsetX;
                matrix1.offsetY = result.offsetY;
                matrix1.type = result.type;
                return;
            }
            var offsetX = matrix1.offsetX;
            var offsetY = matrix1.offsetY;
            matrix1.m11 = matrix2.m11;
            matrix1.m12 = matrix2.m12;
            matrix1.m21 = matrix2.m21;
            matrix1.m22 = matrix2.m22;
            matrix1.offsetX = matrix2.offsetX;
            matrix1.offsetY = matrix2.offsetY;
            matrix1.type = matrix2.type;
            matrix1.offsetX = offsetX * matrix2.m11 + offsetY * matrix2.m21 + matrix2.offsetX;
            matrix1.offsetY = offsetX * matrix2.m12 + offsetY * matrix2.m22 + matrix2.offsetY;
            if (type2 == ej.MatrixTypes.Unknown) {
                matrix1.type = ej.MatrixTypes.Unknown;
                return;
            }
            matrix1.type = (ej.MatrixTypes.Translation | ej.MatrixTypes.Scaling);
        },
        transform: function (matrix, point) {
            var pt = this._multiplyPoint(matrix, point.x, point.y);
            return { x: Math.round(pt.x * 100) / 100, y: Math.round(pt.y * 100) / 100 };
        },
        rotate: function (matrix, angle, centerX, centerY) {
            angle %= 360.0;
            this.multiply(matrix, this._createRotationRadians(angle * 0.017453292519943295, centerX ? centerX : 0, centerY ? centerY : 0));
        },
        scale: function (matrix, scaleX, scaleY, centerX, centerY) {
            this.multiply(matrix, this._createScaling(scaleX, scaleY, centerX ? centerX : 0, centerY ? centerY : 0));
        },
        translate: function (matrix, offsetX, offsetY) {
            if (matrix.type & ej.MatrixTypes.Identity) {
                this._setMatrix(matrix, 1.0, 0.0, 0.0, 1.0, offsetX, offsetY, ej.MatrixTypes.Translation);
                return;
            }
            if (matrix.type & ej.MatrixTypes.Unknown) {
                matrix.offsetX += offsetX;
                matrix.offsetY += offsetY;
                return;
            }
            matrix.offsetX += offsetX;
            matrix.offsetY += offsetY;
            matrix.type |= ej.MatrixTypes.Translation;
        },
        _createScaling: function (scaleX, scaleY, centerX, centerY) {
            var result = this.identity();
            this._setMatrix(result, scaleX, 0.0, 0.0, scaleY, centerX - scaleX * centerX, centerY - scaleY * centerY,
                !(centerX || centerY) ? ej.MatrixTypes.Scaling : ej.MatrixTypes.Scaling | ej.MatrixTypes.Translation);
            return result;
        },
        _createRotationRadians: function (angle, centerX, centerY) {
            var result = this.identity();
            var num = Math.sin(angle);
            var num2 = Math.cos(angle);
            var offsetX = centerX * (1.0 - num2) + centerY * num;
            var offsetY = centerY * (1.0 - num2) - centerX * num;
            this._setMatrix(result, num2, num, -num, num2, offsetX, offsetY, ej.MatrixTypes.Unknown);
            return result;
        },
        _multiplyPoint: function (matrix, x, y) {
            switch (matrix.type) {
                case ej.MatrixTypes.Identity: break;
                case ej.MatrixTypes.Translation:
                    x += matrix.offsetX;
                    y += matrix.offsetY;
                    break;
                case ej.MatrixTypes.Scaling:
                    x *= matrix.m11;
                    y *= matrix.m22;
                    break;
                case ej.MatrixTypes.Translation | ej.MatrixTypes.Scaling:
                    x *= matrix.m11;
                    x += matrix.offsetX;
                    y *= matrix.m22;
                    y += matrix.offsetY;
                    break;
                default:
                    var num = y * matrix.m21 + matrix.offsetX;
                    var num2 = x * matrix.m12 + matrix.offsetY;
                    x *= matrix.m11;
                    x += num;
                    y *= matrix.m22;
                    y += num2;
                    break;
            }
            return { x: x, y: y };
        },
        _setMatrix: function (matrix, m11, m12, m21, m22, offsetX, offsetY, type) {
            matrix.m11 = m11;
            matrix.m12 = m12;
            matrix.m21 = m21;
            matrix.m22 = m22;
            matrix.offsetX = offsetX;
            matrix.offsetY = offsetY;
            matrix.type = type || ej.MatrixTypes.Identity;
        }
    };
})(jQuery, Syncfusion);;
/**
* @fileOverview Plugin to style the Html Button elements
* @copyright Copyright Syncfusion Inc. 2001 - 2013. All rights reserved.
*  Use of this code is subject to the terms of our license.
*  A copy of the current license can be obtained at any time by e-mailing
*  licensing@syncfusion.com. Any infringement will be prosecuted under
*  applicable laws. 
* @version 12.1 
* @author <a href="mailto:licensing@syncfusion.com">Syncfusion Inc</a>
*/
(function ($, ej) {

    "use strict";

    //#region HitTesting
    ej.datavisualization.Diagram.HitTesting = {
        _getResizeHandleCenter: function (shape, position, scale) {
            if (!scale) scale = 1;
            var corners = this.bounds(shape);
            var center = ej.datavisualization.Diagram.Point(0, 0);
            switch (position) {
                case "topleft":
                    center = corners.topLeft;
                    break;
                case "topcenter":
                    center = corners.top;
                    break;
                case "topright":
                    center = corners.topRight;
                    break;
                case "middleleft":
                    center = corners.left;
                    break;
                case "middleright":
                    center = corners.right;
                    break;
                case "bottomleft":
                    center = corners.bottomLeft;
                    break;
                case "bottomcenter":
                    center = corners.bottom;
                    break;
                case "bottomright":
                    center = corners.bottomRight;
                    break;
                case "pivot":
                    center = ej.datavisualization.Diagram.Point(shape.offsetX, shape.offsetY);
                    break;
                case "rotate":
                    var thumbHeight = 20;
                    var dist = 20 / scale; //distance between element top and rotator thumb
                    var pt = ej.datavisualization.Diagram.Point(shape.offsetX, (shape.offsetY - shape.height * shape.pivot.y) - thumbHeight / 2 - dist);
                    var matrix = ej.Matrix.identity();
                    ej.Matrix.rotate(matrix, shape.rotateAngle, shape.offsetX, shape.offsetY);
                    center = ej.Matrix.transform(matrix, pt);
                    break;
            }
            return center;
        },
        findResizeHandleAtPoint: function (node, point) {
            var resizeDirection = null;
            if (node) {
                var center;
                var success;
                var directions = ["topleft", "topcenter", "topright", "middleleft", "middleright", "bottomleft", "bottomcenter", "bottomright"];
                for (var i = 0, len = directions.length; i < len; i++) {
                    center = this._getResizeHandleCenter(node, directions[i]);
                    if (center !== ej.datavisualization.Diagram.Point(0, 0)) {
                        success = ej.datavisualization.Diagram.Geometry.checkPointOnCircle(point, ej.datavisualization.Diagram.Point(center.x, center.y), 7); // right now 7 is default handle radius size, need to give as constant variable
                        if (success) {
                            resizeDirection = directions[i];
                            break;
                        }
                    }
                }
            }
            return resizeDirection;
        },
        findRotationHandleAtPoint: function (node, point, scale) {
            var center = this.xxx_getResizeHandleCenter(node, "rotate", scale);
            var success = ej.datavisualization.Diagram.Geometry.checkPointOnCircle(point, ej.datavisualization.Diagram.Point(center.x, center.y), 10); // right now 7 is default handle radius size, need to give as constant variable
            if (success) {
                return true;

            }
            return false;
        },
        findPivotHandleAtPoint: function (node, point) {
            var center = this._getResizeHandleCenter(node, "pivot");
            var success = ej.datavisualization.Diagram.Geometry.checkPointOnCircle(point, ej.datavisualization.Diagram.Point(center.x, center.y), 7); // right now 7 is default handle radius size, need to give as constant variable
            if (success) {
                return true;

            }
            return false;
        },
        findEndPointHandleAtPoint: function (connector, point) {
            var endPointHandle = null;
            if (connector) {
                var success;
                var points = connector.line.points;
                var pt = points[points.length - 1];
                success = ej.datavisualization.Diagram.Geometry.checkPointOnCircle(point, ej.datavisualization.Diagram.Point(pt.x, pt.y), 7); // right now 7 is default handle radius size, need to give as constant variable
                if (success) {
                    endPointHandle = "targetEndPoint";
                }
                if (!success) {
                    pt = points[0];
                    success = ej.datavisualization.Diagram.Geometry.checkPointOnCircle(point, ej.datavisualization.Diagram.Point(pt.x, pt.y), 7); // right now 7 is default handle radius size, need to give as constant variable
                    if (success) {
                        endPointHandle = "sourceEndPoint";
                    }
                }
                if (!success && connector.line.type === "bezier") {
                    pt = connector.line._point1;
                    success = ej.datavisualization.Diagram.Geometry.checkPointOnCircle(point, ej.datavisualization.Diagram.Point(pt.x, pt.y), 7); // right now 7 is default handle radius size, need to give as constant variable
                    if (success) {
                        endPointHandle = "bezierpoint1";
                    }
                    if (!success) {
                        var pt1 = connector.line._point2;
                        success = ej.datavisualization.Diagram.Geometry.checkPointOnCircle(point, ej.datavisualization.Diagram.Point(pt1.x, pt1.y), 7); // right now 7 is default handle radius size, need to give as constant variable
                        if (success) {
                            endPointHandle = "bezierpoint2";
                        }
                    }
                }
            }
            return endPointHandle;
        }
    };
    //#endregion

    //#region Util
    ej.datavisualization.Diagram.Util = {
        findPortByName: function (node, portName) {
            var port;
            if (node.ports)
                for (var i = 0, len = node.ports.length; i < len; i++) {
                    port = node.ports[i];
                    if (port.name === portName) {
                        return port;
                    }
                }
            return null;
        },
        attr: function (element, attributes) {
            for (var atr in attributes) {
                element.setAttribute(atr.toString(), attributes[atr]);
            }
        },
        removeItem: function (array, item) {
            var index = array.indexOf(item);
            if (index >= 0)
                array.splice(index, 1);
        },
        clear: function (array) {
            while (array.length > 0) {
                array.pop();
            }
        },
        randomId: function () {
            var chars = "0123456789ABCDEFGHIJKLMNOPQRSTUVWXTZabcdefghiklmnopqrstuvwxyz";
            var id = "";
            //get 4 random chars
            for (var i = 0; i < 4; i++) {
                var num = Math.floor(Math.random() * chars.length);
                id += chars.substring(num, num + 1);
            }
            return id;
        },
        htmlEncode: function (html) {
            var str = html.replace(/&amp;/g, '&').replace(/&lt;/g, '<').replace(/&gt;/g, '>').replace(/&quot;/g, '"').replace(/&apos;/g, '\'');
            return str;
        },
        canMove: function (node) {
            if (node.segments) {
                return node.constraints & ej.datavisualization.Diagram.ConnectorConstraints.Drag;
            } else
                return node.constraints & ej.datavisualization.Diagram.NodeConstraints.Drag;
        },
        canMoveLabel: function (node) {
            if (node.segments) {
                return node.constraints & ej.datavisualization.Diagram.ConnectorConstraints.DragLabel;
            } else
                return node.constraints & ej.datavisualization.Diagram.NodeConstraints.DragLabel;
        },
        canRotate: function (node) {
            if (node.segments) {
                return node.constraints & ej.datavisualization.Diagram.ConnectorConstraints.Rotate;

            } else
                return node.constraints & ej.datavisualization.Diagram.NodeConstraints.Rotate;
        },
        canSelect: function (node) {
            if (node.segments) {
                return node.constraints & ej.datavisualization.Diagram.ConnectorConstraints.Select;
            } else
                return node.constraints & ej.datavisualization.Diagram.NodeConstraints.Select;
        },
        canBridge: function (connector, diagram) {
            if (connector.constraints & ej.datavisualization.Diagram.ConnectorConstraints.Bridging) {
                return connector.constraints & ej.datavisualization.Diagram.ConnectorConstraints.Bridging;
            }
            else if (connector.constraints & ej.datavisualization.Diagram.ConnectorConstraints.InheritBridging) {
                return diagram.model.constraints & ej.datavisualization.Diagram.DiagramConstraints.Bridging;
            }
        },
        canAllowPan: function (node) {
            return node.constraints & ej.datavisualization.Diagram.NodeConstraints.AllowPan;
        },
        canDelete: function (node) {
            if (node.segments) {
                return node.constraints & ej.datavisualization.Diagram.ConnectorConstraints.Delete;
            } else
                return node.constraints & ej.datavisualization.Diagram.NodeConstraints.Delete;
        },
        isPageEditable: function (diagram) {
            if (diagram) {
                return diagram.model.constraints & ej.datavisualization.Diagram.DiagramConstraints.PageEditable;
            }

        },
        canDoMultipleSelection: function (diagram) {
            if (diagram) {
                return diagram.model.tool & ej.datavisualization.Diagram.Tool.MultipleSelect;
            }
        },
        canDoSingleSelection: function (diagram) {
            if (diagram) {
                return diagram.model.tool & ej.datavisualization.Diagram.Tool.SingleSelect;
            }
        },
        canPanning: function (diagram) {
            if (diagram) {
                return (diagram.model.constraints & ej.datavisualization.Diagram.DiagramConstraints.PannableX) || (diagram.model.constraints & ej.datavisualization.Diagram.DiagramConstraints.PannableY);
            }
        },
        canZooming: function (diagram) {
            if (diagram) {
                return (diagram.model.constraints & ej.datavisualization.Diagram.DiagramConstraints.Zoomable);
            }
        },
        canUndo: function (diagram) {
            if (diagram) {
                return (diagram.model.constraints & ej.datavisualization.Diagram.DiagramConstraints.Undoable);
            }
        },
        canDragSourceEnd: function (connector) {
            return connector.constraints & ej.datavisualization.Diagram.ConnectorConstraints.DragSourceEnd;
        },
        canDragTargetEnd: function (connector) {
            return connector.constraints & ej.datavisualization.Diagram.ConnectorConstraints.DragTargetEnd;
        },
        canDragSegmentThumbs: function (connector) {
            return connector.constraints & ej.datavisualization.Diagram.ConnectorConstraints.DragSegmentThumb;
        },
        isTargetConnected: function (connector) {
            return connector.targetNode ? true : false;
        },
        isSourceConnected: function (connector) {
            return connector.sourceNode ? true : false;
        },
        bounds: function (object, excludeRotation) {
            var rect;
            if (object.segments) {
                for (var i = 0; i < object.segments.length; i++) {
                    var segment = object.segments[i];
                    if (i == 0)
                        rect = ej.datavisualization.Diagram.Geometry.rect(segment.points);
                    else
                        rect = ej.datavisualization.Diagram.Geometry.union(rect, ej.datavisualization.Diagram.Geometry.rect(segment.points));
                    rect.width = rect.width == 0 ? object.lineWidth : rect.width;
                    rect.height = rect.height == 0 ? object.lineWidth : rect.height;
                }
            }
            else if (object.shape || object.isGroup) {
                rect = ej.datavisualization.Diagram.Rectangle(object.offsetX - object.width * object.pivot.x, object.offsetY - object.height * object.pivot.y, object.width, object.height);
            }
            else
                rect = object;
            if (rect) {
                var bounds = {};
                rect.x = Math.round(rect.x * 100) / 100;
                rect.y = Math.round(rect.y * 100) / 100;
                rect.width = Math.round(rect.width * 100) / 100;
                rect.height = Math.round(rect.height * 100) / 100;
                bounds["width"] = rect.width;
                bounds["height"] = rect.height;
                bounds["x"] = bounds["left"] = rect.x;
                bounds["right"] = rect.x + rect.width;
                bounds["y"] = bounds["top"] = rect.y;
                bounds["bottom"] = rect.y + rect.height;
                bounds["center"] = ej.datavisualization.Diagram.Point(rect.x + rect.width / 2, rect.y + rect.height / 2);
                bounds["topLeft"] = ej.datavisualization.Diagram.Point(rect.x, rect.y);
                bounds["topCenter"] = ej.datavisualization.Diagram.Point(rect.x + rect.width / 2, rect.y);
                bounds["topRight"] = ej.datavisualization.Diagram.Point(rect.x + rect.width, rect.y);
                bounds["middleLeft"] = ej.datavisualization.Diagram.Point(rect.x, rect.y + rect.height / 2);
                bounds["middleRight"] = ej.datavisualization.Diagram.Point(rect.x + rect.width, rect.y + rect.height / 2);
                bounds["bottomLeft"] = ej.datavisualization.Diagram.Point(rect.x, rect.y + rect.height);
                bounds["bottomCenter"] = ej.datavisualization.Diagram.Point(rect.x + rect.width / 2, rect.y + rect.height);
                bounds["bottomRight"] = ej.datavisualization.Diagram.Point(rect.x + rect.width, rect.y + rect.height);
                if (object.rotateAngle && !excludeRotation) {
                    var matrix = ej.Matrix.identity();
                    ej.Matrix.rotate(matrix, object.rotateAngle, object.offsetX, object.offsetY);
                    bounds.topLeft = ej.Matrix.transform(matrix, bounds.topLeft);
                    bounds.topCenter = ej.Matrix.transform(matrix, bounds.topCenter);
                    bounds.topRight = ej.Matrix.transform(matrix, bounds.topRight);
                    bounds.middleLeft = ej.Matrix.transform(matrix, bounds.middleLeft);
                    bounds.middleRight = ej.Matrix.transform(matrix, bounds.middleRight);
                    bounds.bottomLeft = ej.Matrix.transform(matrix, bounds.bottomLeft);
                    bounds.bottomCenter = ej.Matrix.transform(matrix, bounds.bottomCenter);
                    bounds.bottomRight = ej.Matrix.transform(matrix, bounds.bottomRight);
                    var rbounds = ej.datavisualization.Diagram.Geometry.rect([bounds.topLeft, bounds.topRight, bounds.bottomRight, bounds.bottomLeft]);
                    bounds.x = bounds.left = rbounds.x;
                    bounds.y = bounds.top = rbounds.y;
                    bounds.right = rbounds.x + rbounds.width;
                    bounds.bottom = rbounds.y + rbounds.height;
                    bounds.center = ej.datavisualization.Diagram.Point(rbounds.x + rbounds.width / 2, rbounds.y + rbounds.height / 2);
                }
                if (object.rotateAngle < 45) { }
                else if (object.rotateAngle <= 135) {
                    var matrix = ej.Matrix.identity();
                    ej.Matrix.rotate(matrix, object.rotateAngle, object.offsetX, object.offsetY);
                    var temp;
                    temp = bounds["topCenter"];
                    bounds["topCenter"] = bounds["middleLeft"];
                    bounds["middleLeft"] = bounds["bottomCenter"];
                    bounds["bottomCenter"] = bounds["middleRight"];
                    bounds["middleRight"] = temp;
                    temp = bounds["topLeft"];
                    bounds["topLeft"] = bounds["bottomLeft"];
                    bounds["bottomLeft"] = bounds["bottomRight"];
                    bounds["bottomRight"] = bounds["topRight"];
                    bounds["topRight"] = temp;
                }

                else if (object.rotateAngle <= 225) {
                    var matrix = ej.Matrix.identity();
                    ej.Matrix.rotate(matrix, object.rotateAngle, object.offsetX, object.offsetY);
                    var temp;
                    temp = bounds["topCenter"];
                    bounds["topCenter"] = bounds["bottomCenter"];
                    bounds["bottomCenter"] = temp;
                    temp = bounds["middleLeft"];
                    bounds["middleLeft"] = bounds["middleRight"];
                    bounds["middleRight"] = temp;
                    temp = bounds["bottomLeft"];
                    bounds["bottomLeft"] = bounds["topLeft"];
                    bounds["topLeft"] = temp;
                    temp = bounds["bottomRight"];
                    bounds["bottomRight"] = bounds["topRight"];
                    bounds["topRight"] = temp;
                }

                else if (object.rotateAngle <= 315) {
                    var matrix = ej.Matrix.identity();
                    ej.Matrix.rotate(matrix, object.rotateAngle, object.offsetX, object.offsetY);
                    var temp;
                    temp = bounds["topCenter"];
                    bounds["topCenter"] = bounds["middleRight"];
                    bounds["middleRight"] = bounds["bottomCenter"];
                    bounds["bottomCenter"] = bounds["middleLeft"];
                    bounds["middleLeft"] = temp;
                    temp = bounds["bottomRight"];
                    bounds["bottomRight"] = bounds["topRight"];
                    bounds["topRight"] = bounds["bottomLeft"];
                    bounds["bottomLeft"] = bounds["topLeft"];
                    bounds["topLeft"] = temp;
                }
                bounds["points"] = [bounds.topLeft, bounds.topRight, bounds.bottomRight, bounds.bottomLeft];
                return bounds;
            }
            return rect;
        },
        canResize: function (node) {
            return node.constraints & ej.datavisualization.Diagram.NodeConstraints.Resize;
        },
        canConnect: function (node, isPort) {
            var connect = null;
            if (isPort) {
                connect = node.constraints & ej.datavisualization.Diagram.PortConstraints.Connect;
            }
            else if (!node.segments)
                connect = node.constraints & ej.datavisualization.Diagram.NodeConstraints.Connect;
            return connect;
        },
        _initializeSegments: function (connector, defaultType) {
            if (connector.segments && connector.segments.length <= 1) {
                if (!connector.segments.length) {
                    connector.segments.push(ej.datavisualization.Diagram.Segment({ type: defaultType || "straight" }));
                }
                var first = connector.segments[0];
                switch (first.type) {
                    case "straight":
                        if (!first.point) { first._point = connector.targetPoint; }
                        else if (first.point.x != connector.targetPoint.x || first.point.y != connector.targetPoint.y) {
                            //Need to add a method
                            connector.segments.push({ type: defaultType || "straight" });
                        }
                        break;
                    case "orthogonal":
                        if (first.length && first.direction) {
                            ej.datavisualization.Diagram.Util._addOrthogonalPoints(first, null, null, connector.sourcePoint, connector.targetPoint);
                            if (first._endPoint.x != connector.targetPoint.x || first._endPoint.y != connector.targetPoint.y) {
                                connector.segments.push({ type: defaultType || "straight" });
                            }
                        }
                        else {
                            first._length = first.length || 20;
                            first._direction = first.direction || "auto";
                        }
                        break;
                    case "bezier":
                        if (!first.point) first._point = connector.targetPoint;
                        else if (first.point.x != connector.targetPoint.x || first.point.y != connector.targetPoint.y) {
                            //Need to add a method
                            connector.segments.push({ type: defaultType || "straight" });
                        }
                        break;
                }
            }
            var startPoint = connector.sourcePoint;
            var prev = null;
            var next = null;
            var runAngle;
            for (var i = 0; i < connector.segments.length; i++) {
                var segment = connector.segments[i];
                segment._startPoint = startPoint;
                next = connector.segments[i + 1];
                segment = connector.segments[i] = ej.datavisualization.Diagram.Segment(connector.segments[i]);
                if (segment.type == "orthogonal") {
                    segment._direction = segment.direction || "auto";
                    segment._length = segment.length || 20;
                    runAngle = ej.datavisualization.Diagram.Util._addOrthogonalPoints(connector.segments[i], prev, next, connector.sourcePoint, connector.targetPoint, runAngle);
                    startPoint = connector.segments[i]._endPoint;
                    if (!segment.length || !segment.direction) {
                        break;
                    }
                }
                else if (segment.type == "straight") {
                    segment.points.push(startPoint);
                    segment.points.push(connector.segments[i].point || connector.targetPoint);
                    segment._point = segment._endPoint = connector.segments[i].point || connector.targetPoint;
                    runAngle = ej.datavisualization.Diagram.Geometry.findAngle(startPoint, segment._endPoint);
                    startPoint = connector.segments[i]._endPoint;
                    if (!segment.point) {
                        break;
                    }
                }
                else if (segment.type == "bezier") {
                    segment._point = segment._endPoint = connector.segments[i].point || connector.targetPoint;
                    segment.points.push(startPoint);
                    segment.points.push(connector.segments[i].point || connector.targetPoint);
                    runAngle = ej.datavisualization.Diagram.Geometry.findAngle(startPoint, segment._endPoint);
                    ej.datavisualization.Diagram.Util._updateBezierPoints(segment);
                    startPoint = connector.segments[i]._endPoint;
                    if (!segment.point) {
                        break;
                    }
                }
                else {
                    var segment = ej.datavisualization.Diagram.Segment({ type: "orthogonal" });
                    segment._direction = "auto";
                    segment._length = 20;
                    segment._startPoint = startPoint;
                    segment._endPoint = connector.targetPoint;
                    connector.segments.push(segment);
                    ej.datavisualization.Diagram.Util._addOrthogonalPoints(segment, prev, null, connector.sourcePoint, connector.targetPoint, runAngle);
                    break;
                }
                prev = connector.segments[i];
            }
            if (startPoint.x != connector.targetPoint.x || startPoint.y != connector.targetPoint.y) {
                var segment = ej.datavisualization.Diagram.Segment({ type: defaultType || "straight", });
                segment._startPoint = startPoint;
                segment._endPoint = connector.targetPoint;
                if (segment.type == "orthogonal") {
                    ej.datavisualization.Diagram.Util._addOrthogonalPoints(segment, prev, null, connector.sourcePoint, connector.targetPoint, runAngle);
                }
                else {
                    segment._point = connector._endPoint;
                    segment.points.push(startPoint);
                    segment.points.push(connector.targetPoint);
                    if (segment.type == "bezier") { this._updateBezierPoints(segment); }
                }
                connector.segments.push(segment);
            }
        },
        dock: function (connector, nameTable) {
            var targetNode, targetPort = null, sourceNode, sourcePort = null;
            targetNode = nameTable[connector.targetNode];
            if (targetNode)
                targetPort = this.findPortByName(targetNode, connector.targetPort);
            sourceNode = nameTable[connector.sourceNode];
            if (sourceNode)
                sourcePort = this.findPortByName(sourceNode, connector.sourcePort);
            var sourceConnected = this.isSourceConnected(connector);
            var targetConnected = this.isTargetConnected(connector);
            if (targetConnected && sourceConnected && targetPort && sourcePort) {
                this._dockPortToPort(connector, targetNode, targetPort, sourceNode, sourcePort);
            }
            else if (targetConnected && targetPort) {
                if (sourceConnected) {
                    if (connector.segments.length == 1) {
                        this._dockNode(connector, sourceNode, null, targetConnected, sourceConnected);
                        this._dockPortToNode(connector, targetNode, targetPort, sourceNode, sourcePort, true);
                    }
                    else {
                        this._dockNode(connector, sourceNode, null, targetConnected, sourceConnected);
                        this._updateIntermediateSegments(connector);
                        this._dockPortToPoint(connector, targetNode, targetPort, true);
                        this._updatePreviousSegment(connector);
                    }
                }
                else {
                    this._dockPortToPoint(connector, targetNode, targetPort, true);
                    this._updatePreviousSegment(connector);
                }
            }
            else if (sourceConnected && sourcePort) {
                if (targetConnected) {
                    if (connector.segments.length == 1) {
                        this._dockNode(connector, targetNode, null, targetConnected, sourceConnected);
                        this._dockPortToNode(connector, sourceNode, sourcePort, targetNode, targetPort, false);
                    }
                    else {
                        this._dockPortToPoint(connector, sourceNode, sourcePort, false);
                        this._updateIntermediateSegments(connector);
                        this._dockNode(connector, targetNode, null, targetConnected, sourceConnected);
                        this._updatePreviousSegment(connector);
                    }
                }
                else {
                    this._dockPortToPoint(connector, sourceNode, sourcePort, false);
                    this._updateIntermediateSegments(connector);
                }
            }
            else if (targetConnected && sourceConnected) {
                if (targetNode && sourceNode)
                    this._dockNode(connector, targetNode, sourceNode, targetConnected, sourceConnected);
            }
            else if (targetConnected) {
                this._dockNode(connector, targetNode, null, targetConnected, sourceConnected);
            }
            else if (sourceConnected) {
                this._dockNode(connector, sourceNode, null, targetConnected, sourceConnected);
            }
        },
        _dockPortToPort: function (connector, targetNode, targetPort, sourceNode, sourcePort) {
            if (connector.segments.length == 1) {
                var portLocation;
                if (targetNode)
                    var targetBounds = this.bounds(targetNode, true);
                if (sourceNode)
                    var sourceBounds = this.bounds(sourceNode, true);
                var matrix;
                var first = connector.segments[0];
                var last = connector.segments[connector.segments.length - 1];
                var segment = connector.segments[0];
                if (segment.type !== "orthogonal") {
                    if (targetPort) {
                        portLocation = this._getPortPosition(targetPort, targetBounds);
                        matrix = ej.Matrix.identity();
                        ej.Matrix.rotate(matrix, targetNode.rotateAngle, targetNode.offsetX, targetNode.offsetY);
                        portLocation = ej.Matrix.transform(matrix, portLocation);
                        this._setLineEndPoint(connector, portLocation, true);
                    }
                    if (sourcePort) {
                        portLocation = this._getPortPosition(sourcePort, sourceBounds);
                        matrix = ej.Matrix.identity();
                        ej.Matrix.rotate(matrix, sourceNode.rotateAngle, sourceNode.offsetX, sourceNode.offsetY);
                        portLocation = ej.Matrix.transform(matrix, portLocation);
                        this._setLineEndPoint(connector, portLocation, false);
                    }
                    if (segment.type === "bezier")
                        this._updateBezierPoints(segment, targetNode, targetPort, targetBounds, sourceNode, sourcePort, sourceBounds);
                }
                else {
                    var targetPortLocation = this._getPortPosition(targetPort, targetBounds);
                    var sourcePortLocation = this._getPortPosition(sourcePort, sourceBounds);
                    var targetDirection = this._swapDirection(targetNode.rotateAngle, this._getDirection(targetBounds, targetPortLocation));
                    var sourceDirection = this._swapDirection(sourceNode.rotateAngle, this._getDirection(sourceBounds, sourcePortLocation));
                    matrix = ej.Matrix.identity();
                    ej.Matrix.rotate(matrix, sourceNode.rotateAngle, sourceNode.offsetX, sourceNode.offsetY);
                    sourcePortLocation = ej.Matrix.transform(matrix, sourcePortLocation);
                    matrix = ej.Matrix.identity();
                    ej.Matrix.rotate(matrix, targetNode.rotateAngle, targetNode.offsetX, targetNode.offsetY);
                    targetPortLocation = ej.Matrix.transform(matrix, targetPortLocation);
                    targetBounds = this.bounds(targetNode);
                    sourceBounds = this.bounds(sourceNode);
                    this._constructSegements(connector, first, targetBounds, targetPortLocation, targetDirection, sourceBounds, sourcePortLocation, sourceDirection);
                    this._setLineEndPoint(connector, first.points[0], false);
                    this._setLineEndPoint(connector, first.points[first.points.length - 1], true);
                }
            }
            else {
                this._dockPortToPoint(connector, sourceNode, sourcePort, false);
                this._updateIntermediateSegments(connector);
                this._dockPortToPoint(connector, targetNode, targetPort, true);
                this._updatePreviousSegment(connector);
            }

        },
        _dockPortToNode: function (connector, targetNode, targetPort, sourceNode, sourcePort, isTarget) {
            var first = connector.segments[0];
            var last = connector.segments[connector.segments.length - 1];
            if (targetNode)
                var targetBounds = this.bounds(targetNode, true);
            if (sourceNode)
                var sourceBounds = this.bounds(sourceNode, true);
            var matrix;
            var segment = isTarget ? last : first;
            if (segment.type !== "orthogonal") {
                var portLocation = this._getPortPosition(targetPort, targetBounds);
                matrix = ej.Matrix.identity();
                ej.Matrix.rotate(matrix, targetNode.rotateAngle, targetNode.offsetX, targetNode.offsetY);
                portLocation = ej.Matrix.transform(matrix, portLocation);
                //segment._endPoint = portLocation;
                this._setLineEndPoint(connector, portLocation, isTarget);
            }
            else {
                var targetPortLocation = this._getPortPosition(targetPort, targetBounds);
                var targetDirection = this._swapDirection(targetNode.rotateAngle, this._getDirection(targetBounds, targetPortLocation));
                matrix = ej.Matrix.identity();
                ej.Matrix.rotate(matrix, targetNode.rotateAngle, targetNode.offsetX, targetNode.offsetY);
                targetPortLocation = ej.Matrix.transform(matrix, targetPortLocation);
                var sourcePortLocation;
                if (!isTarget) {
                    sourcePortLocation = connector.targetPoint;
                }
                else {
                    sourcePortLocation = connector.sourcePoint;
                }
                var sourceDirection;
                if (sourceNode) {
                    sourceDirection = this._getDirection(sourceBounds, sourcePortLocation);
                }
                else sourceDirection = "top";
                if (sourceDirection) {
                    targetBounds = this.bounds(targetNode);
                    if (sourceNode)
                        sourceBounds = this.bounds(sourceNode);
                    else sourceBounds = { left: sourcePortLocation.x, right: sourcePortLocation.x, top: sourcePortLocation.y, bottom: sourcePortLocation.y };
                    if (isTarget) {
                        this._setLineEndPoint(connector, targetPortLocation, true);
                        this._constructSegements(connector, last, targetBounds, targetPortLocation, targetDirection, sourceBounds, sourcePortLocation, sourceDirection);
                    }
                    else {
                        this._setLineEndPoint(connector, targetPortLocation, false);
                        this._constructSegements(connector, first, sourceBounds, sourcePortLocation, sourceDirection, targetBounds, targetPortLocation, targetDirection);
                    }
                }
            }
        },
        _dockPortToPoint: function (connector, targetNode, targetPort, isTarget) {
            var first = connector.segments[0];
            var last = connector.segments[connector.segments.length - 1];
            var segment = isTarget ? last : first;
            if (segment.type !== "orthogonal") {
                this._dockPortToNode(connector, targetNode, targetPort, null, null, isTarget);
            }
            else {
                var targetBounds = this.bounds(targetNode, true);
                var targetPortLocation = this._getPortPosition(targetPort, targetBounds);
                var targetDirection = this._swapDirection(targetNode.rotateAngle, this._getDirection(targetBounds, targetPortLocation));
                var matrix = ej.Matrix.identity();
                ej.Matrix.rotate(matrix, targetNode.rotateAngle, targetNode.offsetX, targetNode.offsetY);
                targetPortLocation = ej.Matrix.transform(matrix, targetPortLocation);
                var sourcePortLocation;
                var sourcePortLocation;
                if (!isTarget) {
                    sourcePortLocation = first._endPoint;
                }
                else {
                    sourcePortLocation = last._startPoint;
                }
                var sourceBounds = this.bounds(ej.datavisualization.Diagram.Rectangle(sourcePortLocation.x, sourcePortLocation.y, 0, 0));
                targetBounds = this.bounds(targetNode);
                var sourceDirection = "top";
                if (isTarget) {
                    this._setLineEndPoint(connector, targetPortLocation, true);
                    this._pointsFromNodeToPoint(connector, last, targetDirection, targetBounds, targetNode.rotateAngle, targetPortLocation, sourcePortLocation, true); //{(connector, last, targetBounds, targetPortLocation, targetDirection, sourceBounds, sourcePortLocation, sourceDirection);

                }
                else {
                    this._setLineEndPoint(connector, targetPortLocation, false);
                    this._pointsFromNodeToPoint(connector, first, targetDirection, targetBounds, targetNode.rotateAngle, targetPortLocation, sourcePortLocation, false);
                }
            }
        },
        _dockNode: function (connector, targetNode, sourceNode, targetConnected, sourceConnected) {
            var startsegment, endsegment;
            if (connector.segments.length == 1) {
                startsegment = endsegment = connector.segments[0];
            }
            else {
                startsegment = connector.segments[0];
                endsegment = connector.segments[connector.segments.length - 1];
            }
            if (targetConnected && sourceConnected && !connector.targetPort && !connector.sourcePort) {
                if (connector.segments.length == 1) {
                    if (connector.segments[0].type == "orthogonal") {
                        var sourceBounds = this.bounds(sourceNode);
                        var targetBounds = this.bounds(targetNode);
                        var srcDirection, tarDirection;
                        var srcPoint, tarPoint;
                        if (sourceBounds.y > targetBounds.bottom) {
                            srcDirection = "top";
                            tarDirection = "bottom";
                            srcPoint = sourceBounds.topCenter;
                            tarPoint = targetBounds.bottomCenter;
                        }
                        else if (sourceBounds.bottom < targetBounds.y) {
                            srcDirection = "bottom";
                            tarDirection = "top";
                            srcPoint = sourceBounds.bottomCenter;
                            tarPoint = targetBounds.topCenter;
                        }
                        else if (sourceBounds.x > targetBounds.right) {
                            srcDirection = "left";
                            tarDirection = "right";
                            srcPoint = sourceBounds.middleLeft;
                            tarPoint = targetBounds.middleRight;
                        }
                        else {
                            srcDirection = "right";
                            tarDirection = "left";
                            srcPoint = sourceBounds.middleRight;
                            tarPoint = targetBounds.middleLeft;
                        }
                        this._setLineEndPoint(connector, srcPoint, false);
                        this._setLineEndPoint(connector, tarPoint, true);
                        if (connector.segments[0].type == "orthogonal") {
                            this._constructSegements(connector, connector.segments[0], targetBounds, connector.targetPoint, tarDirection, sourceBounds, connector.sourcePoint, srcDirection);
                            connector.segments[0]._direction = srcDirection;
                        }
                        var indentSpace = connector.sourceIndent + connector.targetIndent+ sourceNode.connectionPadding;
                        var srcPt = this._adjustPoint(connector.segments[0]._startPoint, connector.segments[0].points[1], true, indentSpace);
                        var tarPt = this._adjustPoint(connector.segments[0].points[connector.segments[0].points.length - 2], connector.segments[0]._endPoint, false, indentSpace);
                        this._setLineEndPoint(connector, srcPt, false);
                        this._setLineEndPoint(connector, tarPt, true);
                    }
                    else {
                        if (targetNode) {
                            targetBounds = this.bounds(targetNode);
                            if (sourceNode) {
                                sourceBounds = this.bounds(sourceNode);
                                this._setLineEndPoint(connector, sourceBounds.center, false);
                                this._setLineEndPoint(connector, targetBounds.center, true);
                                this._dockBounds(connector, sourceNode, sourceBounds, false);
                            }
                            this._setLineEndPoint(connector, targetBounds.center, true);
                            this._dockBounds(connector, targetNode, targetBounds, true);
                        }
                        if (connector.segments[0].type == "bezier") {
                            this._updateBezierPoints(connector.segments[0]);
                        }
                    }
                }
                else {
                    this._dock(connector, startsegment, sourceNode, null, false);
                    this._updateIntermediateSegments(connector);
                    if (connector.segments.length == 1) {
                        this._dock(connector, startsegment, sourceNode, targetNode, false);
                        this._dock(connector, endsegment, targetNode, sourceNode, true);
                    }
                    else {
                        this._dock(connector, endsegment, targetNode, null, true);
                        this._updatePreviousSegment(connector);
                    }
                }
            }
            else if (targetConnected && !connector.targetPort) {
                this._dock(connector, endsegment, targetNode, sourceNode, true);
                this._updatePreviousSegment(connector);
            }
            else if (sourceConnected && !connector.sourcePort) {
                this._dock(connector, startsegment, targetNode, sourceNode, false);
                this._updateIntermediateSegments(connector);
            }
        },
        _dock: function (connector, line, targetNode, sourceNode, isTarget) {
            var targetBounds;
            var sourceBounds;
            var points;
            if (targetNode) {
                var node = targetNode;
                var point = isTarget ? line._startPoint : line._endPoint;
                var sourceBounds = this.bounds(targetNode);
                if (!isTarget) {
                    if (line.type == "orthogonal" && connector.segments[1] && connector.segments[1].type == "orthogonal") {
                        if (connector.segments[0]._isInternal && ej.datavisualization.Diagram.Geometry.containsPoint(sourceBounds, line._endPoint)) {
                            this._insertSegmentAtSourceEnd(connector, line, sourceBounds, isTarget);
                        }
                        else if (ej.datavisualization.Diagram.Geometry.containsPoint(sourceBounds, line._endPoint)) {
                            this._removeFirstSegment(connector, line, sourceBounds, isTarget);
                        }
                        else {
                            var dockPoint, dockDirection;;
                            switch (line.direction) {
                                case "left":
                                    dockDirection = "left";
                                    dockPoint = sourceBounds.middleLeft;
                                    break;
                                case "right": dockDirection = "right";
                                    dockPoint = sourceBounds.middleRight;
                                    break;
                                case "top": dockDirection = "top";
                                    dockPoint = sourceBounds.topCenter;
                                    break;
                                case "bottom": dockDirection = "bottom";
                                    dockPoint = sourceBounds.bottomCenter;
                                    break;
                            }
                            line.points = [];
                            line.points.push(dockPoint);
                            line.points.push(line._endPoint);
                            this._setLineEndPoint(connector, dockPoint, isTarget);
                        }
                    }
                    else {
                        this._dockNodeToPoint(connector, line._endPoint, targetNode, isTarget);
                    }
                }
                else {
                    if (line.type == "orthogonal") {
                        var second = connector.segments[connector.segments.length - 2];
                        if (second && second.type == "orthogonal") {
                            if (line.points.length > 2 &&
                                ej.datavisualization.Diagram.Geometry.containsPoint(sourceBounds, line.points[line.points.length - 2])) {
                                var removePreSegment = false;
                                if (second.direction == "top" || second.direction == "bottom") {
                                    if (second._endPoint.x >= sourceBounds.left && second._endPoint.x <= sourceBounds.right) {
                                        this._removeLastSegment(connector, line, second, targetNode, sourceNode, isTarget); return;
                                    }
                                    else {
                                        second._length = second.length = Math.abs(second._startPoint.y - sourceBounds.middleLeft.y);;
                                        if (second.length == 0) {
                                            removePreSegment = true;
                                        }
                                        else {
                                            if (second.length < 0) {
                                                second._length = second.length *= -1;
                                                second._direction = second.direction = (second.direction == "top") ? "bottom" : "top";
                                            }
                                            connector.targetPoint = second._endPoint.x > sourceBounds.right ? sourceBounds.middleRight : sourceBounds.middleLeft;
                                        }
                                    }
                                }
                                else {
                                    if (second._endPoint.y >= sourceBounds.top && second._endPoint.y <= sourceBounds.bottom) {
                                        this._removeLastSegment(connector, line, second, targetNode, sourceNode, isTarget); return;
                                    }
                                    else {
                                        second._length = second.length = Math.abs(second._startPoint.x - sourceBounds.topCenter.x);;
                                        if (second.length == 0) removePreSegment = true;
                                        else {
                                            if (second.length < 0) {
                                                second._length = second.length *= -1;
                                                second._direction = second.direction = (second.direction == "left") ? "right" : "left";
                                            }
                                            connector.targetPoint = second._endPoint.y > sourceBounds.bottom ? sourceBounds.bottomCenter : sourceBounds.topCenter;
                                        }
                                    }
                                }
                                if (removePreSegment) {
                                    connector.segments.splice(connector.segments.length - 2, 1);
                                    this._dock(connector, line, targetNode, sourceNode, isTarget);
                                    return;
                                }
                                this._updateConnectorSegments(connector, connector.sourcePort, connector.targetPort);
                                line._direction = this._getOrthoDirection(line._startPoint, line._endPoint, second);
                                this._setLineEndPoint(connector, connector.targetPoint, isTarget);

                            }
                            else if (ej.datavisualization.Diagram.Geometry.containsPoint(sourceBounds, line._startPoint)) {
                                var diflength = second.direction == "left" || second.direction == "right" ? node.width / 2 : node.height / 2;
                                second._length = second.length -= diflength + 30;
                                if (second.length == 0) {
                                    connector.segments.splice(connector.segments.length - 2, 1);
                                    this._dock(connector, line, targetNode, sourceNode, isTarget);
                                    return;
                                }
                                if (second.direction == "left" || second.direction == "right")
                                    connector.targetPoint = second.direction == "right" ? sourceBounds.middleLeft : sourceBounds.middleRight;
                                else
                                    connector.targetPoint = second.direction == "bottom" ? sourceBounds.topCenter : sourceBounds.bottomCenter;
                                this._updateConnectorSegments(connector, connector.sourcePort, connector.targetPort);
                                line._direction = this._getOrthoDirection(line._startPoint, line._endPoint, second);
                                this._setLineEndPoint(connector, connector.targetPoint, isTarget);

                            }
                            else {
                                this._dockNodeToPoint(connector, point, targetNode, isTarget);
                            }
                        } else {
                            this._dockNodeToPoint(connector, point, targetNode, isTarget);
                        }
                    }
                    else { this._dockNodeToPoint(connector, point, targetNode, isTarget); }

                }
            }
        },
        _removeLastSegment: function (connector, line, second, targetNode, sourceNode, isTarget) {
            connector.segments.splice(connector.segments.length - 1, 1);
            second.length = second._length = null;
            second.direction = null;
            second._direction = line._direction;
            this._dock(connector, second, targetNode, sourceNode, isTarget);
        },
        _insertSegmentAtSourceEnd: function (connector, line, sourceBounds, isTarget) {
            var second = connector.segments[1];
            var third = connector.segments[2];
            if (connector.segments[0].direction == "left" || connector.segments[0].direction == "right") {
                var orientation = "horizontal";
                connector.sourcePoint = second.direction == "top" ? sourceBounds.topCenter : sourceBounds.bottomCenter;
                if (second && (second.length || second.length === 0))
                    second._length = second.length = Math.max(Math.abs(connector.sourcePoint.y - second._endPoint.y), 25);
                if (third && (third.length || third.length === 0)) {
                    third.length = third._length = Math.abs(sourceBounds.topCenter.x - third._endPoint.x);
                }
            }
            else {
                var orientation = "vertical";
                connector.sourcePoint = second.direction == "left" ? sourceBounds.middleLeft : sourceBounds.middleRight;
                if (second && (second.length || second.length === 0))
                    second._length = second.length = Math.max(Math.abs(connector.sourcePoint.x - second._endPoint.x), 25);
                if (third && (third.length || third.length === 0)) {
                    third.length = third._length = Math.abs(sourceBounds.middleRight.y - third._endPoint.y);
                }
            }
            //Updating adjacent segments
            if (connector.segments[3] && (connector.segments[3].length || connector.segments[3].length == 0)) {
                var value = second.direction == "bottom" || second.direction == "right" ? second.length : -second.length;
                connector.segments[3].length = connector.segments[3]._length = orientation == "vertical" ?
                    Math.abs(connector.sourcePoint.x + value - connector.segments[3]._endPoint.x) :
                    Math.abs(connector.sourcePoint.y + value - connector.segments[3]._endPoint.y);
                if (connector.segments[3].length < 0) {
                    var value = second.direction == "bottom" || second.direction == "right" ? second.length : -second.length;
                    connector.segments[3].length = connector.segments[3]._length = orientation == "vertical" ?
                        Math.abs(connector.sourcePoint.x + value - connector.segments[3]._endPoint.x) :
                    Math.abs(connector.sourcePoint.y + value - connector.segments[3]._endPoint.y);
                }
            }
            connector.segments.splice(0, 1);
            this._updateConnectorSegments(connector, connector.sourcePort, connector.targetPort);
            this._setLineEndPoint(connector, connector.sourcePoint, isTarget);
        },
        _removeFirstSegment: function (connector, line, sourceBounds, isTarget) {
            var newLength, firstSegLength;
            var direction = connector.segments[1].direction || this._getBezierDirection(connector.segments[1].points[0], connector.segments[1].points[1]);
            if (direction == "left" || direction == "right") {
                var length = line._endPoint.y - sourceBounds.middleRight.y;
                connector.segments[0].direction = connector.segments[0]._direction = length >= 0 ? "bottom" : "top";
                connector.segments[0].length = connector.segments[0]._length = Math.abs(length);
                var actualdist = direction == "right" ? connector.segments[1]._endPoint.x - sourceBounds.right :
                    sourceBounds.left - connector.segments[1]._endPoint.x;
                connector.sourcePoint = direction == "left" ? sourceBounds.middleLeft : sourceBounds.middleRight;
                newLength = Math.abs(connector.sourcePoint.x - connector.segments[0]._startPoint.x);
                firstSegLength = Math.abs(connector.sourcePoint.y - connector.segments[0]._endPoint.y);
            }
            else {
                if (connector.segments[0].length || connector.segments[0].length == 0) {
                    var length = line._endPoint.x - sourceBounds.bottomCenter.x;
                    connector.segments[0].direction = connector.segments[0]._direction = length >= 0 ? "right" : "left";
                    connector.segments[0].length = connector.segments[0]._length = Math.abs(length);
                }
                var actualdist = direction == "bottom" ? connector.segments[1]._endPoint.y - sourceBounds.bottom :
                    sourceBounds.top - connector.segments[1]._endPoint.y;
                connector.sourcePoint = direction == "top" ? sourceBounds.topCenter : sourceBounds.bottomCenter;
                newLength = Math.abs(connector.sourcePoint.y - connector.segments[0]._startPoint.y);
                firstSegLength = Math.abs(connector.sourcePoint.x - connector.segments[0]._endPoint.x);
            }

            if (connector.segments[1] && (connector.segments[1].length || connector.segments[1].length === 0)) {
                connector.segments[1]._length = connector.segments[1].length = actualdist - 25;
                if (connector.segments[1]._length < 0) {
                    connector.segments.splice(0, 2);
                    if (connector.segments[0].length || connector.segments[0].length === 0)
                        connector.segments[0].length = connector.segments[0]._length = newLength;
                    var newSegment = ej.datavisualization.Diagram.Segment({ type: "orthogonal", length: firstSegLength, direction: direction });
                }
            }
            if (!newSegment) {
                newSegment = ej.datavisualization.Diagram.Segment({ type: "orthogonal", length: 25, direction: direction });
            }
            var segments = [newSegment];
            newSegment._isInternal = true;
            connector.segments = segments.concat(connector.segments);
            this._updateConnectorSegments(connector, connector.sourcePort, connector.targetPort);
            this._setLineEndPoint(connector, connector.sourcePoint, isTarget);
        },
        _dockNodeToPoint: function (connector, fixedPoint, node, isTarget) {
            var line, sourceBounds;
            line = isTarget ? connector.segments[connector.segments.length - 1] : connector.segments[0];
            if (line.type != "orthogonal") {
                sourceBounds = this.bounds(node);
                this._setLineEndPoint(connector, sourceBounds.center, isTarget);
                this._dockBounds(connector, node, sourceBounds, isTarget);
            }
            else {
                var sourceBounds = this.bounds(node);
                var dockPoint, dockDirection, targetDirection;
                var point = fixedPoint;
                var prev = connector.segments[connector.segments.length - 2];
                var left = Math.abs(point.x - sourceBounds.left);
                var right = Math.abs(point.x - sourceBounds.right);
                var top = Math.abs(point.y - sourceBounds.top);
                var bottom = Math.abs(point.y - sourceBounds.bottom);
                if (prev && isTarget) {
                    var horizontal = prev.direction == "left" || prev.direction == "right";
                    var vertical = prev.direction == "top" || prev.direction == "bottom";
                }
                line = !isTarget ? connector.segments[0] : connector.segments[connector.segments.length - 1];
                if ((line.points.length == 2 || !prev) && line.type == "orthogonal") {
                    horizontal = vertical = false;
                }
                if (sourceBounds.bottom < point.y && !horizontal) {
                    dockDirection = "bottom";
                    targetDirection = "top";
                    dockPoint = sourceBounds.bottomCenter;
                }
                else if (sourceBounds.y > point.y && !horizontal) {
                    dockDirection = "top";
                    targetDirection = "bottom";
                    dockPoint = sourceBounds.topCenter;
                }
                else if (sourceBounds.right < point.x && !vertical) {
                    dockDirection = "right";
                    targetDirection = "left";
                    dockPoint = sourceBounds.middleRight;
                }
                else if (sourceBounds.x > point.x && !vertical) {
                    dockDirection = "left";
                    targetDirection = "right";
                    dockPoint = sourceBounds.middleLeft;
                }
                else {
                    var min = left;
                    dockPoint = sourceBounds.middleLeft;
                    dockDirection = "left";
                    targetDirection = "right";
                    if (min >= right && right != 0) {
                        dockPoint = sourceBounds.middleRight;
                        dockDirection = "right";
                        targetDirection = "left";
                        min = right;
                    }
                    if (min > top && top != 0) {
                        dockPoint = sourceBounds.topCenter;
                        dockDirection = "top";
                        targetDirection = "bottom";
                        min = top;
                    }
                    if (min > bottom && bottom != 0) {
                        dockPoint = sourceBounds.bottomCenter;
                        dockDirection = "bottom";
                        targetDirection = "top";
                        min = bottom;
                    }
                }
                if (dockPoint) {
                    if (isTarget || connector.segments.length == 1) {
                        var segment = connector.segments[connector.segments.length - 1];
                        if (segment.type == "orthogonal") {
                            segment._direction = isTarget ? targetDirection : dockDirection;
                        }
                    }
                    else { var segment = connector.segments[0]; }
                    isTarget ? connector.targetPoint = dockPoint : connector.sourcePoint = dockPoint;
                    if (connector.segments.length != 1)
                        this._addOrthogonalPoints(segment, !isTarget ? null : connector.segments[connector.segments.length - 2], !isTarget ? connector.segments[1] : null, connector.sourcePoint, connector.targetPoint);
                    this._setLineEndPoint(connector, dockPoint, isTarget);
                }
            }
        },
        _getDockSegment: function (connector, line, rotateAngle, bounds, isTarget, direction) {
            var segment; var segments = this._getEdges(rotateAngle, bounds);
            {
                var direction = "", segment, intersecSeg;
                //var line = connector.line;

                var length = ej.datavisualization.Diagram.Geometry.distance(line._startPoint, line._endPoint);
                if (isTarget)
                    intersecSeg = ej.datavisualization.Diagram.LineSegment(line.points[line.points.length - 2], line.points[line.points.length - 1]);
                else {
                    if (length > 5) {
                        intersecSeg = ej.datavisualization.Diagram.LineSegment(line._startPoint, line._endPoint);
                    }
                    else {
                        intersecSeg = ej.datavisualization.Diagram.LineSegment(line._startPoint, connector.segments[1] ? connector.segments[1]._endPoint : connector.targetPoint);
                    }
                }
                if (ej.datavisualization.Diagram.Geometry.intersectSegment(intersecSeg, segments.left)) {
                    segment = segments.left;
                    direction = "left";
                }
                else if (ej.datavisualization.Diagram.Geometry.intersectSegment(intersecSeg, segments.top)) {
                    segment = segments.top;
                    direction = "top";
                }
                else if (ej.datavisualization.Diagram.Geometry.intersectSegment(intersecSeg, segments.right)) {
                    segment = segments.right;
                    direction = "right";
                }
                else if (ej.datavisualization.Diagram.Geometry.intersectSegment(intersecSeg, segments.bottom)) {
                    segment = segments.bottom;
                    direction = "bottom";
                }
            }
            direction = this._swapDirection(rotateAngle, direction);
            if (segment) {
                return { "dockPoint": ej.datavisualization.Diagram.Geometry.midPoint(segment), "direction": direction };
            }
            return null;
        },
        _dockBounds: function (connector, node, bounds, isTarget) {
            var line;
            line = isTarget ? connector.segments[connector.segments.length - 1] : connector.segments[0];
            var segmentPoints, length;
            if (node.shape.type === "polygon") {
                length = node.shape.points.length;
                segmentPoints = $.extend(false, {}, node.shape.points);
                ej.datavisualization.Diagram.Geometry.updatePolygonPoints(node, segmentPoints);
                for (var i = 0; i < length; i++) {
                    var pt1 = ej.datavisualization.Diagram.Point(segmentPoints[i].x + (node.offsetX - node.width / 2), segmentPoints[i].y + (node.offsetY - node.height / 2));
                    if (node.rotateAngle != 0) {
                        var matrix = ej.Matrix.identity();
                        ej.Matrix.rotate(matrix, node.rotateAngle, node.offsetX, node.offsetY);
                        pt1 = ej.Matrix.transform(matrix, pt1);
                    }
                    if (node.connectionPadding) {
                        var x = node.offsetX == pt1.x ? 0 : (node.offsetX < pt1.x ? 1 : -1);
                        var y = node.offsetY == pt1.y ? 0 : (node.offsetY < pt1.y ? 1 : -1);
                        pt1 = ej.datavisualization.Diagram.Geometry.translate(pt1, x * node.connectionPadding, y * node.connectionPadding);
                    }
                    segmentPoints[i] = pt1;
                }
            }
            else if (node.shape.type === "path") {
                var svg = document.createElementNS('http://www.w3.org/2000/svg', 'svg');
                svg.setAttribute('xlink', 'http://www.w3.org/1999/xlink');
                document.body.appendChild(svg);

                var pathEl = document.createElementNS('http://www.w3.org/2000/svg', "path");
                pathEl.setAttributeNS(null, "d", node.shape.pathData);
                svg.appendChild(pathEl);

                var pathBounds = pathEl.getBBox();
                var pathData = ej.datavisualization.Diagram.Geometry.updatePath(-node.connectionPadding, -node.connectionPadding, node.width + 2 * node.connectionPadding, node.height + 2 * node.connectionPadding, node.shape.pathData, svg, pathBounds);
                pathEl.setAttributeNS(null, "d", pathData);
                var length = pathEl.getTotalLength(), pts = [];
                for (var i = 0; i < length - 1; i++) {
                    var pathSeg = pathEl.getPointAtLength(i);
                    var pt1 = ej.datavisualization.Diagram.Point(pathSeg.x + (node.offsetX - node.width / 2), pathSeg.y + (node.offsetY - node.height / 2));
                    if (node.rotateAngle != 0) {
                        var matrix = ej.Matrix.identity();
                        ej.Matrix.rotate(matrix, node.rotateAngle, node.offsetX, node.offsetY);
                        pt1 = ej.Matrix.transform(matrix, pt1);
                    }
                    pts[i] = pt1;
                }
                length = pts.length;
                segmentPoints = pts;
                document.body.removeChild(svg);
            }
            else {
                if (node.connectionPadding != 0) {
                    var rect = ej.datavisualization.Diagram.Rectangle(bounds.x - node.connectionPadding, bounds.y - node.connectionPadding, node.width + 2 * node.connectionPadding, node.height + 2 * node.connectionPadding);
                    bounds = this.bounds(rect);
                }
                length = bounds.points.length;
                segmentPoints = bounds.points;
            }
            var thisSegment = ej.datavisualization.Diagram.LineSegment(line._startPoint, line._endPoint || line.point);
            var i;
            var segment = ej.datavisualization.Diagram.LineSegment(segmentPoints[length - 1], segmentPoints[0]);
            var intersection = ej.datavisualization.Diagram.Geometry.intersectSegment(thisSegment, segment);
            if (!intersection) {
                for (i = 0; i < length - 1; i++) {
                    segment = ej.datavisualization.Diagram.LineSegment(segmentPoints[i], segmentPoints[i + 1]);
                    intersection = ej.datavisualization.Diagram.Geometry.intersectSegment(thisSegment, segment);
                    if (intersection)
                        break;
                }
            }
            if (intersection) {
                var pt;
                if (isTarget)
                    pt = this._adjustPoint(intersection, connector.sourcePoint, isTarget, connector.targetIndent);
                else
                    pt = this._adjustPoint(connector.targetPoint, intersection, isTarget, connector.sourceIndent);
                this._setLineEndPoint(connector, pt, isTarget);
                if (line.type === "bezier") {
                    this._updateBezierPoints(line);
                }
            }
        },
        _getEdges: function (rotateAngle, bounds) {
            var points = bounds.points;
            var segments = {};
            var angle = rotateAngle % 360;
            var temp;
            segments.left = ej.datavisualization.Diagram.LineSegment(points[3], points[0]);
            segments.top = ej.datavisualization.Diagram.LineSegment(points[0], points[1]);
            segments.right = ej.datavisualization.Diagram.LineSegment(points[1], points[2]);
            segments.bottom = ej.datavisualization.Diagram.LineSegment(points[2], points[3]);
            if (angle < 0) {
                angle += 360;
            }
            if (angle > 315 || angle < 45) {

            }
            else if (angle < 135) {
                temp = segments.top;
                segments.top = segments.left;
                segments.left = segments.bottom;
                segments.bottom = segments.right;
                segments.right = temp;
            }
            else if (angle < 225) {
                temp = segments.top;
                segments.top = segments.bottom;
                segments.bottom = temp;
                temp = segments.left;
                segments.left = segments.right;
                segments.right = temp;
            }
            else {
                temp = segments.bottom;
                segments.bottom = segments.left;
                segments.left = segments.top;
                segments.top = segments.right;
                segments.right = temp;
            }
            return segments;
        },
        _pointsFromNodeToPoint: function (connector, segment, direction, bounds, angle, point, endPoint, isTarget) {
            var dir, num;
            var points = [];
            points.push(point);
            var straight;
            straight = (point.y == endPoint.y && (direction == "left" && endPoint.x < point.x || direction == "right" && endPoint.x > point.x)) ||
               (point.x == endPoint.x && (direction == "top" && endPoint.y < point.y || direction == "bottom" && endPoint.y > point.y));
            if (angle == 0)
                if (!straight && !isTarget) straight = point.x > bounds.left && point.x < bounds.right && point.y > bounds.top && point.y < bounds.bottom;
            if (!straight) {
                if (direction == "top" || direction == "bottom") {
                    if (direction == "top" && endPoint.y < point.y || direction == "bottom" && endPoint.y > point.y) {
                        //twosegments
                        points.push({ x: point.x, y: endPoint.y });
                    }
                    else if ((endPoint.x < bounds.left || endPoint.x > bounds.right) && endPoint.y != point.y) {
                        //three segments
                        var y = direction == "top" ? bounds.top - 15 : bounds.bottom + 15;
                        points.push({ x: point.x, y: y });
                        points.push({ x: endPoint.x, y: y });
                    }
                    else {
                        //four segments
                        var y = direction == "top" ? bounds.top - 15 : bounds.bottom + 15;
                        var x = (endPoint.x < point.x) ? bounds.left - 15 : bounds.right + 15;
                        points.push({ x: point.x, y: y });
                        points.push({ x: x, y: y });
                        points.push({ x: x, y: endPoint.y });
                    }
                }
                else {
                    if (direction == "left" && endPoint.x < point.x || direction == "right" && endPoint.x > point.x) {
                        points.push({ x: endPoint.x, y: point.y });
                        //twosegments
                    }
                    else if ((endPoint.y < bounds.top || endPoint.y > bounds.bottom) && endPoint.x != point.x) {
                        //three segments
                        var x = direction == "left" ? bounds.left - 15 : bounds.right + 15;
                        points.push({ x: x, y: point.y });
                        points.push({ x: x, y: endPoint.y });
                    }
                    else {
                        //four segments
                        var y = (endPoint.y < point.y) ? bounds.top - 15 : bounds.bottom + 15;
                        var x = direction == "left" ? bounds.left - 15 : bounds.right + 15;
                        points.push({ x: x, y: point.y });
                        points.push({ x: x, y: y });
                        points.push({ x: endPoint.x, y: y });
                    }
                }
            }
            points.push(endPoint);
            if (isTarget) points.reverse();
            segment._startPoint = points[0];
            segment._endPoint = points[points.length - 1];
            segment.points = points;
        },
        _constructSegements: function (connector, segment, targetBounds, targetPortLocation, targetDirection, sourceBounds, sourcePortLocation, sourceDirection) {
            var points, reverse;
            if (sourceDirection === "right") {
                if (targetDirection === "left") {
                    points = this._rightToLeft(targetBounds, targetPortLocation, sourceBounds, sourcePortLocation);
                }
                else if (targetDirection === "right") {
                    points = this._rightToRight(targetBounds, targetPortLocation, sourceBounds, sourcePortLocation);
                }
                else if (targetDirection === "top") {
                    points = this._rightToTop(targetBounds, targetPortLocation, sourceBounds, sourcePortLocation);
                }
                else if (targetDirection === "bottom") {
                    points = this._rightToBottom(targetBounds, targetPortLocation, sourceBounds, sourcePortLocation);
                }
            }
            if (sourceDirection === "left") {
                if (targetDirection === "right") {
                    points = this._rightToLeft(sourceBounds, sourcePortLocation, targetBounds, targetPortLocation);
                    reverse = true;
                }
                if (targetDirection === "left") {
                    points = this._leftToLeft(targetBounds, targetPortLocation, sourceBounds, sourcePortLocation);
                }
                if (targetDirection === "top") {
                    points = this._leftToTop(targetBounds, targetPortLocation, sourceBounds, sourcePortLocation);
                }
                if (targetDirection === "bottom") {
                    points = this._leftToBottom(targetBounds, targetPortLocation, sourceBounds, sourcePortLocation);
                }
            }
            if (sourceDirection === "top") {
                if (targetDirection === "left") {
                    points = this._leftToTop(sourceBounds, sourcePortLocation, targetBounds, targetPortLocation);
                    reverse = true;
                }
                if (targetDirection === "right") {
                    points = this._rightToTop(sourceBounds, sourcePortLocation, targetBounds, targetPortLocation);
                    reverse = true;
                }
                if (targetDirection === "top") {
                    points = this._topToTop(targetBounds, targetPortLocation, sourceBounds, sourcePortLocation);
                }
                if (targetDirection === "bottom") {
                    points = this._topToBottom(targetBounds, targetPortLocation, sourceBounds, sourcePortLocation);
                }
            }
            if (sourceDirection === "bottom") {
                if (targetDirection === "left") {
                    points = this._leftToBottom(sourceBounds, sourcePortLocation, targetBounds, targetPortLocation);
                    reverse = true;
                }
                if (targetDirection === "right") {
                    points = this._rightToBottom(sourceBounds, sourcePortLocation, targetBounds, targetPortLocation);
                    reverse = true;
                }
                if (targetDirection === "top") {
                    points = this._topToBottom(sourceBounds, sourcePortLocation, targetBounds, targetPortLocation);
                    reverse = true;
                }
                if (targetDirection === "bottom") {
                    points = this._bottomToBottom(targetBounds, targetPortLocation, sourceBounds, sourcePortLocation);
                }
            }
            if (reverse) {
                points.reverse();
            }
            var point1 = points[points.length - 1];
            var point2 = points[points.length - 2];
            while (points.length > 2 && point1.x == point2.x && point1.y == point2.y) {
                points.splice(points.length - 1, 1);
                point1 = points[points.length - 1];
                point2 = points[points.length - 2];
            }
            point1 = points[0];
            point2 = points[1];
            while (points.length > 2 && point1.x == point2.x && point1.y == point2.y) {
                points.splice(0, 1);
                point1 = points[0];
                point2 = points[1];
            }
            segment._startPoint = points[0];
            segment._endPoint = points[points.length - 1];
            segment.points = points;
            //this._setPoints(connector.line, points);
        },
        _updateIntermediateSegments: function (connector) {
            var startPoint = connector.segments[0]._endPoint;
            var prev = connector.segments[0];
            var length = prev.points.length;
            var next = null;
            var runAngle;
            if (connector.segments[1]) {
                var segment = connector.segments[1];
                segment._startPoint = startPoint;
                if (segment.type == "orthogonal" && (segment.length || segment.length === 0)) {
                    if (segment.direction == "left" || segment.direction == "right") {
                        if (connector.segments[2]) {
                            if (connector.segments[2].length || connector.segments[2].length === 0) {
                                if (connector.segments[2].direction == "top")
                                    connector.segments[2].length += segment._startPoint.y - segment._endPoint.y;
                                else
                                    connector.segments[2].length -= segment._startPoint.y - segment._endPoint.y;
                                if (connector.segments[2].length < 0) {
                                    connector.segments[2].length *= -1;
                                    connector.segments[2].direction = connector.segments[2]._direction = segment.direction == "right" ? "left" : "right";
                                }
                                connector.segments[2]._length = connector.segments[2].length;

                            }
                        }
                        segment._endPoint.y = segment._startPoint.y;
                    }
                    else {
                        if (connector.segments[2]) {
                            if (connector.segments[2].length || connector.segments[2].length === 0) {
                                if (connector.segments[2].direction == "left")
                                    connector.segments[2].length += segment._startPoint.x - segment._endPoint.x;
                                else
                                    connector.segments[2].length -= segment._startPoint.x - segment._endPoint.x;
                                if (connector.segments[2].length < 0) {
                                    connector.segments[2].length *= -1;
                                    connector.segments[2].direction = connector.segments[2]._direction = segment.direction == "bottom" ? "top" : "bottom";
                                }
                                connector.segments[2]._length = connector.segments[2].length;
                            }
                        }
                        segment._endPoint.x = segment._startPoint.x;
                    }
                    if (connector.sourcePort && prev.points.length > 2) {
                        if (segment.direction == "left" || segment.direction == "right") {
                            if (prev.points[prev.points.length - 1].y == prev.points[prev.points.length - 2].y) {
                                segment._startPoint = segment.points[0] = prev.points[prev.points.length - 2];
                                prev.points.splice(prev.points.length - 1, 1);
                                prev._endPoint = prev.points[prev.points.length - 1];
                                if (segment.points.length == 2) {
                                    segment._endPoint.y = segment._startPoint.y;
                                }
                            }
                        }
                        else {
                            if (prev.points[prev.points.length - 1].x == prev.points[prev.points.length - 2].x) {
                                segment._startPoint = segment.points[0] = prev.points[prev.points.length - 2];
                                prev.points.splice(prev.points.length - 1, 1);
                                prev._endPoint = prev.points[prev.points.length - 1];
                                if (segment.points.length == 2) {
                                    segment._endPoint.x = segment._startPoint.x;
                                }
                            }
                        }
                    }
                    if (segment.length || segment.length === 0)
                        segment.length = segment._length = ej.datavisualization.Diagram.Geometry.distance(segment._startPoint, segment.points[1]);
                    if (segment.length > 0)
                        segment.direction = segment._direction = this._getBezierDirection(segment._startPoint, segment._endPoint) || segment._direction;
                    this._addOrthogonalPoints(segment, prev, connector.segments[2], connector.sourcePoint, connector.targetPoint);
                    if (connector.segments[2])
                        this._addOrthogonalPoints(connector.segments[2], connector.segments[1], connector.segments[3], connector.sourcePoint, connector.targetPoint);
                }
                else if (segment.type == "orthogonal" && segment.length === null) {
                    if (connector.sourcePort && prev.points.length > 2) {
                        if (segment._direction == "left" || segment._direction == "right") {
                            if (prev.points[prev.points.length - 1].y == prev.points[prev.points.length - 2].y) {
                                segment._startPoint = segment.points[0] = prev.points[prev.points.length - 2];
                                prev.points.splice(prev.points.length - 1, 1);
                                prev._endPoint = prev.points[prev.points.length - 1];
                            }
                        }
                        else {
                            if (prev.points[prev.points.length - 1].x == prev.points[prev.points.length - 2].x) {
                                segment._startPoint = segment.points[0] = prev.points[prev.points.length - 2];
                                prev.points.splice(prev.points.length - 1, 1);
                                prev._endPoint = prev.points[prev.points.length - 1];
                            }
                        }
                    }
                    segment._startPoint = segment.points[0] = connector.segments[0]._endPoint;
                    this._addOrthogonalPoints(connector.segments[1], connector.segments[0], null, connector.sourcePoint, connector.targetPoint);
                }
                else {
                    segment._startPoint = segment.points[0] = connector.segments[0]._endPoint;
                    if (segment.type == "bezier") {
                        this._updateBezierPoints(segment);
                    }
                }
            }
        },
        _updatePreviousSegment: function (connector) {
            var previousSegment = connector.segments[connector.segments.length - 2];
            var current = connector.segments[connector.segments.length - 1];
            if (previousSegment) {
                if (previousSegment.type == "orthogonal" && current.points.length > 2) {
                    var update = false;
                    var direction = previousSegment.points.length == 2 ? previousSegment.direction :
                        ej.datavisualization.Diagram.Util._getBezierDirection(previousSegment.points[previousSegment.points.length - 2],
                        previousSegment.points[previousSegment.points.length - 1]);
                    if (direction == "left" || direction == "right") {
                        if (current.points[0].y == current.points[1].y) {
                            update = true;
                            var dif = current.points[1].x - current.points[0].x;
                            current.points.splice(0, 1);
                            current._startPoint = current.points[0];
                            previousSegment._endPoint = current.points[0];
                        }
                    }
                    else {
                        if (current.points[0].x == current.points[1].x) {
                            update = true;
                            current.points.splice(0, 1);
                            current._startPoint = current.points[0];
                            previousSegment._endPoint = current.points[0];
                        }
                    }
                }
                previousSegment._endPoint = current._startPoint;
                previousSegment.points[previousSegment.points.length - 1] = current._startPoint;
                if (previousSegment.type == "orthogonal" && update) {
                    previousSegment._length = previousSegment.length = ej.datavisualization.Diagram.Geometry.distance(previousSegment._startPoint, previousSegment._endPoint);
                    if (previousSegment._length > 0)
                        previousSegment._direction = previousSegment.direction = this._getBezierDirection(previousSegment._startPoint, previousSegment._endPoint);
                    if (connector.segments.length > 2 || !connector.sourcePort)
                        this._addOrthogonalPoints(previousSegment, connector.segments[connector.segments.length - 3], current, connector.sourcePoint, connector.targetPoint);
                }
                else {
                    previousSegment.point = current._startPoint;
                }
            }
        },
        _swapDirection: function (rotateAngle, direction) {
            var dir;
            var angle = rotateAngle % 360;
            if (angle < 0) {
                angle += 360;
            }
            var dirs;
            if (angle > 315 || angle < 45) {
                dir = direction;
            }
            else if (angle < 135) {
                dirs = { left: "top", top: "right", right: "bottom", bottom: "left" };
                dir = dirs[direction];
            }
            else if (angle < 225) {
                dirs = { left: "right", top: "bottom", right: "left", bottom: "top" };
                dir = dirs[direction];
            }
            else {
                dirs = { left: "bottom", top: "left", right: "top", bottom: "right" };
                dir = dirs[direction];
            }
            return dir;
        },
        _getIntersectionPoint: function (node, connectorSeg, segmentPoints) {
            var intersection, len = segmentPoints.length, i;
            var segment = ej.datavisualization.Diagram.Segment(segmentPoints[len - 1], segmentPoints[0]);
            if (node.shape.type != 'ellipse') {
                intersection = ej.datavisualization.Diagram.Geometry.intersectSegment(connectorSeg, segment);
            }
            if (!intersection) {
                for (i = 0; i < len - 1; i++) {
                    segment = ej.datavisualization.Diagram.Segment(segmentPoints[i], segmentPoints[i + 1]);
                    intersection = ej.datavisualization.Diagram.Geometry.intersectSegment(connectorSeg, segment);
                    if (intersection)
                        return intersection;
                }
            }
            else
                return intersection;
        },

        _getEllipseIntersectCoords: function (node, connectorSeg, connector) {
            var segmentPoints = [], intersection;
            var rx = node.width / 2;
            var ry = node.height / 2;
            var cx = node.offsetX - node.width / 2;
            var cy = node.offsetY - node.height / 2;

            //Convert Top Round Side to Path
            var topArchPath = "M " + (node.offsetX - node.width / 2) + ", " + (node.offsetY) + " A " + rx + " " + ry + " 0 0 " + 1 + " " + (node.offsetX + node.width / 2) + ", " + (node.offsetY);
            var path = document.createElementNS('http://www.w3.org/2000/svg', "path");
            path.setAttributeNS(null, "d", topArchPath);
            var len = path.getTotalLength(), pts = [], pt1, pt2, coor;
            for (var i = 0; i < len - 1; i++) {
                var pathSeg = path.getPointAtLength(i);
                pts[i] = pathSeg;
            }
            segmentPoints = pts;
            intersection = this._getIntersectionPoint(node, connectorSeg, segmentPoints);
            if (intersection)
                return intersection;

            segmentPoints = [];
            segmentPoints[0] = path.getPointAtLength(0);
            //Convert bottom Round Side to Path
            var bottomArchPath = "M " + (node.offsetX - node.width / 2) + ", " + (node.offsetY) + " A " + rx + " " + ry + " 0 0 " + 0 + " " + (node.offsetX + node.width / 2) + ", " + (node.offsetY);
            path.setAttributeNS(null, "d", bottomArchPath);
            len = path.getTotalLength();
            segmentPoints[1] = path.getPointAtLength(0);
            intersection = this._getIntersectionPoint(node, connectorSeg, segmentPoints);
            if (intersection)
                return intersection;
            segmentPoints = [];
            for (i = 0; i < len - 1; i++) {
                segmentPoints[i] = path.getPointAtLength(i);
            }
            intersection = this._getIntersectionPoint(node, connectorSeg, segmentPoints);
            if (intersection)
                return intersection;

            segmentPoints = [];
            segmentPoints[0] = path.getPointAtLength(len);
            path.setAttributeNS(null, "d", topArchPath);
            len = path.getTotalLength();
            segmentPoints[1] = path.getPointAtLength((len - 1));
            intersection = this._getIntersectionPoint(node, connectorSeg, segmentPoints);
            if (intersection)
                return intersection;
        },


        _rightToLeft: function (targetBounds, targetPortLocation, sourceBounds, sourcePortLocation) {
            var points = [];
            var x, y;
            points.push(sourcePortLocation);
            var srcExtension = 15;
            var tarExtension = 15;
            if (sourceBounds.width == 0) srcExtension = 0;
            if (targetBounds.width == 0) tarExtension = 0;
            if (targetPortLocation.x > sourcePortLocation.x) {
                if (sourcePortLocation.y != targetPortLocation.y) {
                    if ((sourcePortLocation.x + srcExtension) >= targetBounds.left) {
                        x = sourceBounds.right + srcExtension;
                        y = sourcePortLocation.y;
                        points.push(ej.datavisualization.Diagram.Point(x, y));
                        if (targetPortLocation.y > sourceBounds.top) {
                            y = targetPortLocation.y;
                        }
                        else {
                            y = targetBounds.bottom + tarExtension;
                        }
                        points.push(ej.datavisualization.Diagram.Point(x, y));
                        if (targetPortLocation.y <= sourceBounds.top) {
                            x = targetBounds.right + tarExtension;
                            points.push(ej.datavisualization.Diagram.Point(x, y));
                            y = targetBounds.top - tarExtension;
                            points.push(ej.datavisualization.Diagram.Point(x, y));
                            x = targetPortLocation.x;
                            points.push(ej.datavisualization.Diagram.Point(x, y));
                        }
                    }
                    else {
                        x = sourceBounds.right + (targetBounds.left - sourceBounds.right) / 2;
                        y = sourcePortLocation.y;
                        points.push(ej.datavisualization.Diagram.Point(x, y));
                        y = targetPortLocation.y;
                        points.push(ej.datavisualization.Diagram.Point(x, y));
                    }
                }
            }
            else {
                //5 segment
                if (Math.abs(targetBounds.top - sourceBounds.bottom) < 30 || Math.abs(targetBounds.bottom - sourceBounds.top) < 30) {
                    if ((sourcePortLocation.x + srcExtension) > targetBounds.right) {
                        x = sourcePortLocation.x + srcExtension;
                        y = sourcePortLocation.y;
                    }
                    else {
                        x = targetBounds.right + tarExtension;
                        y = sourcePortLocation.y;
                    }
                }
                else {
                    x = sourceBounds.right + srcExtension;
                    y = sourcePortLocation.y;
                }
                points.push(ej.datavisualization.Diagram.Point(x, y));
                if ((targetBounds.top - sourceBounds.bottom) >= 30) {
                    y = targetBounds.top - tarExtension;
                    points.push(ej.datavisualization.Diagram.Point(x, y));
                }
                else if (sourcePortLocation.y > targetPortLocation.y) {
                    if ((sourceBounds.top - targetBounds.bottom) < 30) {
                        y = targetBounds.top - tarExtension;
                    }
                    else {
                        y = targetBounds.bottom + tarExtension;
                    }
                    points.push(ej.datavisualization.Diagram.Point(x, y));
                }
                else {
                    y = targetBounds.bottom + tarExtension;
                    points.push(ej.datavisualization.Diagram.Point(x, y));
                }
                x = targetBounds.left - tarExtension;
                points.push(ej.datavisualization.Diagram.Point(x, y));
                y = targetPortLocation.y;
                points.push(ej.datavisualization.Diagram.Point(x, y));
            }
            points.push(targetPortLocation);
            return points;
        },
        _rightToRight: function (targetBounds, targetPortLocation, sourceBounds, sourcePortLocation) {
            var points = [];
            var x, y;
            points.push(sourcePortLocation);
            y = sourcePortLocation.y;
            var srcExtension = 15;
            var tarExtension = 15;
            if (sourceBounds.width == 0) srcExtension = 0;
            if (targetBounds.width == 0) tarExtension = 0;
            if (targetBounds.right >= sourceBounds.left) {
                if (sourcePortLocation.y >= targetBounds.top && sourcePortLocation.y <= targetBounds.bottom) {
                    x = sourcePortLocation.x + srcExtension;
                    points.push(ej.datavisualization.Diagram.Point(x, y));
                    if (sourcePortLocation.y >= targetBounds.center.y) {
                        if (sourcePortLocation.x > targetBounds.left) {
                            if (sourcePortLocation.y > targetPortLocation.y) {
                                y = targetBounds.bottom + tarExtension;
                            }
                            else {
                                y = targetBounds.top - tarExtension;
                            }
                        }
                        else {
                            y = targetBounds.top - tarExtension;
                        }
                        points.push(ej.datavisualization.Diagram.Point(x, y));
                        x = targetBounds.right + tarExtension;
                        points.push(ej.datavisualization.Diagram.Point(x, y));
                    }
                    else {
                        y = targetBounds.bottom + tarExtension;
                        points.push(ej.datavisualization.Diagram.Point(x, y));
                        x = targetBounds.right + tarExtension;
                        points.push(ej.datavisualization.Diagram.Point(x, y));
                    }
                }
                else if (targetBounds.right < sourceBounds.right) {
                    x = sourceBounds.right + srcExtension;
                    points.push(ej.datavisualization.Diagram.Point(x, y));
                    //points.push(ej.datavisualization.Diagram.Point(x, targetPortLocation.y));
                } else {
                    x = targetPortLocation.x + tarExtension;
                    points.push(ej.datavisualization.Diagram.Point(x, y));
                    //points.push(ej.datavisualization.Diagram.Point(x, targetPortLocation.y));
                }
            }
            else {
                x = sourcePortLocation.x + srcExtension;
                points.push(ej.datavisualization.Diagram.Point(x, y));
                if (targetPortLocation.y >= sourceBounds.top && targetPortLocation.y <= sourceBounds.bottom) {
                    if (targetPortLocation.y <= sourceBounds.center.y) {
                        y = sourceBounds.top - srcExtension;
                        points.push(ej.datavisualization.Diagram.Point(x, y));
                        x = targetBounds.right + tarExtension;
                        points.push(ej.datavisualization.Diagram.Point(x, y));
                    }
                    else {
                        y = sourceBounds.bottom + srcExtension;
                        points.push(ej.datavisualization.Diagram.Point(x, y));
                        x = targetBounds.right + tarExtension;
                        points.push(ej.datavisualization.Diagram.Point(x, y));
                    }
                }
            }
            points.push(ej.datavisualization.Diagram.Point(x, targetPortLocation.y));
            points.push(targetPortLocation);
            return points;
        },
        _rightToTop: function (targetBounds, targetPortLocation, sourceBounds, sourcePortLocation) {
            var points = [];
            var x, y;
            points.push(sourcePortLocation);
            y = sourcePortLocation.y;
            var srcExtension = 15;
            var tarExtension = 15;
            if (sourceBounds.width == 0) srcExtension = 0;
            if (targetBounds.width == 0) tarExtension = 0;
            if (targetPortLocation.x >= sourcePortLocation.x) {
                if ((sourcePortLocation.y + srcExtension) >= targetPortLocation.y && targetBounds.height !== 0) {
                    if (targetBounds.left <= sourceBounds.right) {
                        x = targetBounds.right + tarExtension;
                    }
                    else if ((targetBounds.left - sourceBounds.right) > 30) {
                        x = sourceBounds.right + srcExtension;
                    }
                    else {
                        x = sourceBounds.right + (targetBounds.left - sourceBounds.right) / 2;
                    }
                    points.push(ej.datavisualization.Diagram.Point(x, y));
                    y = targetBounds.top - tarExtension;
                    points.push(ej.datavisualization.Diagram.Point(x, y));
                }
            }
            else {
                if (sourceBounds.bottom >= targetBounds.top) {
                    if ((sourcePortLocation.x + srcExtension) <= targetBounds.right) {
                        x = targetBounds.right + tarExtension;
                    }
                    else {
                        x = sourceBounds.right + srcExtension;
                    }
                    points.push(ej.datavisualization.Diagram.Point(x, y));
                    if (sourceBounds.top >= targetBounds.top) {
                        if (targetBounds.height !== 0) {
                            y = targetBounds.top - tarExtension;
                        }
                        else {
                            y = targetPortLocation.y;
                        }
                    }
                    else {
                        y = sourceBounds.top - srcExtension;
                    }
                    points.push(ej.datavisualization.Diagram.Point(x, y));
                }
                else {
                    x = sourcePortLocation.x + srcExtension;
                    points.push(ej.datavisualization.Diagram.Point(x, y));
                    if ((targetBounds.top - sourceBounds.bottom) < 30 && targetBounds.height !== 0) {
                        if ((sourceBounds.left - targetBounds.right) >= 30) {
                            y = sourceBounds.bottom + srcExtension;
                            points.push(ej.datavisualization.Diagram.Point(x, y));
                            x = sourceBounds.left - srcExtension;
                            points.push(ej.datavisualization.Diagram.Point(x, y));
                            y = targetBounds.top - tarExtension;
                            points.push(ej.datavisualization.Diagram.Point(x, y));
                        }
                        else {
                            y = targetBounds.top - (targetBounds.top - sourceBounds.bottom) / 2;
                            points.push(ej.datavisualization.Diagram.Point(x, y));
                        }
                    }
                    else {
                        if (targetBounds.height !== 0) {
                            y = targetBounds.top - tarExtension;
                        }
                        else {
                            y = targetPortLocation.y;
                        }
                        points.push(ej.datavisualization.Diagram.Point(x, y));
                    }
                }
            }
            points.push(ej.datavisualization.Diagram.Point(targetPortLocation.x, y));
            points.push(targetPortLocation);
            return points;
        },
        _rightToBottom: function (targetBounds, targetPortLocation, sourceBounds, sourcePortLocation) {
            var points = [];
            var x, y;
            points.push(sourcePortLocation);
            y = sourcePortLocation.y;
            var srcExtension = 15;
            var tarExtension = 15;
            if (sourceBounds.width == 0) srcExtension = 0;
            if (targetBounds.width == 0) tarExtension = 0;
            if (targetPortLocation.x >= sourcePortLocation.x) {
                if (sourcePortLocation.y <= targetBounds.bottom) {
                    if (targetBounds.left > (sourceBounds.right + srcExtension)) {
                        x = sourcePortLocation.x + srcExtension;
                        points.push(ej.datavisualization.Diagram.Point(x, y));
                        y = targetBounds.bottom + tarExtension;
                        points.push(ej.datavisualization.Diagram.Point(x, y));
                    }
                    else {
                        x = targetBounds.right + tarExtension;
                        points.push(ej.datavisualization.Diagram.Point(x, y));
                        y = targetBounds.bottom + tarExtension;
                        points.push(ej.datavisualization.Diagram.Point(x, y));
                    }
                }
                else {
                    if ((sourceBounds.right + srcExtension) > targetPortLocation.x) {
                        x = sourcePortLocation.x + srcExtension;
                        points.push(ej.datavisualization.Diagram.Point(x, y));
                        if ((sourceBounds.top - targetBounds.bottom) > 30) {
                            y = targetBounds.bottom + tarExtension;
                        }
                        else {
                            y = targetBounds.bottom + (sourceBounds.top - targetBounds.bottom) / 2;
                        }
                        points.push(ej.datavisualization.Diagram.Point(x, y));
                    }
                    else {
                        y = sourcePortLocation.y;
                    }
                }
            }
            else {
                if (sourcePortLocation.y <= targetBounds.bottom) {
                    if ((sourceBounds.right + srcExtension) > targetBounds.right) {
                        x = sourceBounds.right + srcExtension;
                    }
                    else {
                        x = targetBounds.right + tarExtension;
                    }
                    points.push(ej.datavisualization.Diagram.Point(x, y));
                    y = targetBounds.bottom + tarExtension;
                    if (y <= (sourceBounds.bottom + srcExtension)) {
                        y = sourceBounds.bottom + srcExtension;
                    }
                    points.push(ej.datavisualization.Diagram.Point(x, y));
                }
                else {
                    if ((targetBounds.bottom) < sourceBounds.top) {
                        x = sourceBounds.right + srcExtension;
                        points.push(ej.datavisualization.Diagram.Point(x, y));
                        if ((sourceBounds.top - targetBounds.bottom) < 30) {
                            y = targetBounds.bottom + (sourceBounds.top - targetBounds.bottom) / 2;
                        }
                        else {
                            y = targetBounds.bottom + tarExtension;
                        }
                        points.push(ej.datavisualization.Diagram.Point(x, y));
                    }
                    else {
                        x = sourceBounds.right + srcExtension;
                        points.push(ej.datavisualization.Diagram.Point(x, y));
                        y = sourceBounds.bottom + srcExtension;
                        points.push(ej.datavisualization.Diagram.Point(x, y));
                    }
                }
            }
            points.push(ej.datavisualization.Diagram.Point(targetPortLocation.x, y));
            points.push(targetPortLocation);
            return points;
        },
        _leftToLeft: function (targetBounds, targetPortLocation, sourceBounds, sourcePortLocation) {
            var points = [];
            var x, y;
            points.push(sourcePortLocation);
            y = sourcePortLocation.y;
            var srcExtension = 15;
            var tarExtension = 15;
            if (sourceBounds.width == 0) srcExtension = 0;
            if (targetBounds.width == 0) tarExtension = 0;
            if (targetPortLocation.x >= sourcePortLocation.x) {
                x = sourceBounds.left - srcExtension;
                points.push(ej.datavisualization.Diagram.Point(x, y));
                if (sourcePortLocation.y > targetPortLocation.y) {
                    if ((targetPortLocation.y + tarExtension) >= sourceBounds.top) {
                        y = sourceBounds.top - srcExtension;
                        points.push(ej.datavisualization.Diagram.Point(x, y));
                        if ((targetBounds.left - sourceBounds.right) > 30) {
                            x = targetBounds.left - tarExtension;
                        }
                        else {
                            x = sourceBounds.right + (targetBounds.left - sourceBounds.right) / 2;
                        }
                        points.push(ej.datavisualization.Diagram.Point(x, y));
                    }
                    else {
                        y = targetPortLocation.y;
                        points.push(ej.datavisualization.Diagram.Point(x, y));
                        x = targetPortLocation.x;
                    }
                }
                else {
                    if ((sourceBounds.bottom + srcExtension) < targetPortLocation.y) {
                        y = targetPortLocation.y;
                        points.push(ej.datavisualization.Diagram.Point(x, y));
                        x = targetPortLocation.x;
                    }
                    else {
                        y = sourceBounds.bottom + srcExtension;
                        points.push(ej.datavisualization.Diagram.Point(x, y));
                        if ((targetBounds.left - sourceBounds.right) > 30) {
                            x = targetBounds.left - tarExtension;
                        }
                        else {
                            x = sourceBounds.right + (targetBounds.left - sourceBounds.right) / 2;
                        }
                        points.push(ej.datavisualization.Diagram.Point(x, y));
                    }
                }
            }
            else {
                if (targetPortLocation.y > sourcePortLocation.y) {
                    if (sourcePortLocation.y > (targetBounds.top - tarExtension)) {
                        if ((sourceBounds.left - targetBounds.right) < 30) {
                            x = targetBounds.right + (sourceBounds.left - targetBounds.right) / 2;
                        }
                        else {
                            x = sourceBounds.left - srcExtension;
                        }
                        points.push(ej.datavisualization.Diagram.Point(x, y));
                        y = targetBounds.top - tarExtension;
                        points.push(ej.datavisualization.Diagram.Point(x, y));
                    }
                }
                else {
                    if ((targetBounds.bottom + tarExtension) >= sourcePortLocation.y) {
                        if ((sourceBounds.left - targetBounds.right) < 30) {
                            x = targetBounds.right + (sourceBounds.left - targetBounds.right) / 2;
                        }
                        else {
                            x = sourceBounds.left - srcExtension;
                        }
                        points.push(ej.datavisualization.Diagram.Point(x, y));
                        y = targetBounds.bottom + tarExtension;
                        points.push(ej.datavisualization.Diagram.Point(x, y));
                        x = targetBounds.left - tarExtension;
                        points.push(ej.datavisualization.Diagram.Point(x, y));
                    }
                }
                x = targetBounds.left - tarExtension;
                points.push(ej.datavisualization.Diagram.Point(x, y));
            }
            points.push(ej.datavisualization.Diagram.Point(x, targetPortLocation.y));
            points.push(targetPortLocation);
            return points;
        },
        _leftToTop: function (targetBounds, targetPortLocation, sourceBounds, sourcePortLocation) {
            var points = [];
            var x, y;
            points.push(sourcePortLocation);
            y = sourcePortLocation.y;
            var srcExtension = 15;
            var tarExtension = 15;
            if (sourceBounds.width == 0) srcExtension = 0;
            if (targetBounds.width == 0) tarExtension = 0;
            if (sourcePortLocation.x <= targetPortLocation.x) {
                if (sourcePortLocation.y > targetPortLocation.y && targetBounds.left <= sourceBounds.left) {
                    x = targetBounds.left - tarExtension;
                }
                else {
                    x = sourceBounds.left - srcExtension;
                }
                points.push(ej.datavisualization.Diagram.Point(x, y));
                if (sourceBounds.bottom <= targetBounds.top) {
                    if ((sourceBounds.bottom + srcExtension) < targetBounds.top) {
                        if (targetBounds.height !== 0) {
                            y = targetBounds.top - tarExtension;
                        }
                        else {
                            y = targetPortLocation.y;
                        }
                    }
                    else if (targetPortLocation.x < sourceBounds.right) {
                        y = sourceBounds.bottom + (targetBounds.top - sourceBounds.bottom) / 2;
                    }
                    else {
                        y = sourceBounds.top - srcExtension;
                    }
                }
                else {
                    if (sourceBounds.top >= targetBounds.top) {
                        if (targetBounds.height !== 0) {
                            y = targetBounds.top - tarExtension;
                        }
                        else {
                            y = targetPortLocation.y;
                        }
                    }
                    else {
                        y = sourceBounds.top - srcExtension;
                    }
                }
                points.push(ej.datavisualization.Diagram.Point(x, y));
            }
            else {
                if (targetPortLocation.y <= (sourcePortLocation.y + srcExtension)) {
                    if ((sourceBounds.left - targetBounds.right) > 30) {
                        x = sourceBounds.left - srcExtension;
                    }
                    else {
                        if (targetBounds.right >= sourceBounds.left) {
                            x = targetBounds.left - tarExtension;
                        }
                        else {
                            x = targetBounds.right + (sourceBounds.left - targetBounds.right) / 2;
                        }
                    }
                    points.push(ej.datavisualization.Diagram.Point(x, y));
                    y = targetBounds.top - tarExtension;
                    points.push(ej.datavisualization.Diagram.Point(x, y));
                }
            }
            points.push(ej.datavisualization.Diagram.Point(targetPortLocation.x, y));
            points.push(targetPortLocation);
            return points;
        },
        _leftToBottom: function (targetBounds, targetPortLocation, sourceBounds, sourcePortLocation) {
            var points = [];
            var x, y;
            points.push(sourcePortLocation);
            y = sourcePortLocation.y;
            var srcExtension = 15;
            var tarExtension = 15;
            if (sourceBounds.width == 0) srcExtension = 0;
            if (targetBounds.width == 0) tarExtension = 0;
            if (sourcePortLocation.x <= targetPortLocation.x) {
                if (targetBounds.left <= sourcePortLocation.x && sourcePortLocation.y <= targetBounds.top) {
                    x = targetBounds.left - tarExtension;
                }
                else {
                    x = sourceBounds.left - srcExtension;
                }
                points.push(ej.datavisualization.Diagram.Point(x, y));
                if (targetBounds.bottom < sourceBounds.top) {
                    if ((sourceBounds.top - targetBounds.bottom) < 30) {
                        y = targetBounds.bottom + (sourceBounds.top - targetBounds.bottom) / 2;
                    }
                    else {
                        y = targetBounds.bottom + tarExtension;
                    }
                    points.push(ej.datavisualization.Diagram.Point(x, y));
                }
                else {
                    if (targetBounds.bottom < sourceBounds.bottom) {
                        y = sourceBounds.bottom + srcExtension;
                    }
                    else {
                        y = targetBounds.bottom + tarExtension;
                    }
                    points.push(ej.datavisualization.Diagram.Point(x, y));
                }
            }
            else {
                if (!(sourcePortLocation.y > targetPortLocation.y)) {
                    if (targetBounds.right >= sourceBounds.left) {
                        x = targetBounds.left - tarExtension;
                    }
                    else if ((sourceBounds.left - targetBounds.right) < 30) {
                        x = sourceBounds.left - (sourceBounds.left - targetBounds.right) / 2;
                    }
                    else {
                        x = sourceBounds.left - srcExtension;
                    }
                    points.push(ej.datavisualization.Diagram.Point(x, y));
                    y = targetBounds.bottom + tarExtension;
                    points.push(ej.datavisualization.Diagram.Point(x, y));
                }
            }
            points.push(ej.datavisualization.Diagram.Point(targetPortLocation.x, y));
            points.push(targetPortLocation);
            return points;
        },
        _topToTop: function (targetBounds, targetPortLocation, sourceBounds, sourcePortLocation) {
            var points = [];
            var x, y;
            points.push(sourcePortLocation);
            x = sourcePortLocation.x;
            var srcExtension = 15;
            var tarExtension = 15;
            if (sourceBounds.width == 0) srcExtension = 0;
            if (targetBounds.width == 0) tarExtension = 0;
            if (targetPortLocation.x >= sourcePortLocation.x) {
                if (targetPortLocation.y >= sourcePortLocation.y) {
                    y = sourceBounds.top - srcExtension;
                    points.push(ej.datavisualization.Diagram.Point(x, y));
                    if (targetPortLocation.x <= sourceBounds.right) {
                        x = sourceBounds.right + srcExtension;
                        points.push(ej.datavisualization.Diagram.Point(x, y));
                        if (targetBounds.height !== 0) {
                            if ((targetBounds.top - sourceBounds.bottom) > 30) {
                                y = targetBounds.top - tarExtension;
                            }
                            else {
                                y = sourceBounds.bottom + (targetBounds.top - sourceBounds.bottom) / 2;
                            }
                        }
                        else {
                            y = targetPortLocation.y;
                        }
                        points.push(ej.datavisualization.Diagram.Point(x, y));
                    }
                }
                else {
                    if (targetBounds.left <= sourcePortLocation.x) {
                        if ((sourceBounds.top - targetBounds.bottom) > 30) {
                            y = sourceBounds.top - srcExtension;
                        }
                        else {
                            y = targetBounds.bottom + (sourceBounds.top - targetBounds.bottom) / 2;
                        }
                        points.push(ej.datavisualization.Diagram.Point(x, y));
                        x = targetBounds.left - tarExtension;
                        points.push(ej.datavisualization.Diagram.Point(x, y));
                        y = targetBounds.top - tarExtension;
                        points.push(ej.datavisualization.Diagram.Point(x, y));
                    }
                    else {
                        if (targetBounds.height !== 0) {
                            y = targetBounds.top - tarExtension;
                        }
                        else {
                            y = targetPortLocation.y;
                        }
                        points.push(ej.datavisualization.Diagram.Point(x, y));
                    }
                }
            }
            else {
                if (targetPortLocation.y >= sourcePortLocation.y) {
                    y = sourceBounds.top - srcExtension;
                    points.push(ej.datavisualization.Diagram.Point(x, y));
                    if (targetPortLocation.x >= sourceBounds.left) {
                        x = sourceBounds.left - srcExtension;
                        points.push(ej.datavisualization.Diagram.Point(x, y));
                        if (targetBounds.height !== 0) {
                            if ((targetBounds.top - sourceBounds.bottom) > 30) {
                                y = targetBounds.top - tarExtension;
                            }
                            else {
                                y = sourceBounds.bottom + (targetBounds.top - sourceBounds.bottom) / 2;
                            }
                        }
                        else {
                            y = targetPortLocation.y;
                        }
                        points.push(ej.datavisualization.Diagram.Point(x, y));
                    }
                }
                else {
                    if (targetBounds.right >= sourcePortLocation.x) {
                        if ((sourceBounds.top - targetBounds.bottom) > 30) {
                            y = sourceBounds.top - srcExtension;
                        }
                        else {
                            y = targetBounds.bottom + (sourceBounds.top - targetBounds.bottom) / 2;
                        }
                        points.push(ej.datavisualization.Diagram.Point(x, y));
                        x = targetBounds.right + tarExtension;
                        points.push(ej.datavisualization.Diagram.Point(x, y));
                        y = targetBounds.top - tarExtension;
                        points.push(ej.datavisualization.Diagram.Point(x, y));
                    }
                    else {
                        if (targetBounds.height !== 0) {
                            y = targetBounds.top - tarExtension;
                        }
                        else {
                            y = targetPortLocation.y;
                        }
                        points.push(ej.datavisualization.Diagram.Point(x, y));
                    }
                }
            }
            x = targetPortLocation.x;
            points.push(ej.datavisualization.Diagram.Point(x, y));
            points.push(targetPortLocation);
            return points;
        },
        _topToBottom: function (targetBounds, targetPortLocation, sourceBounds, sourcePortLocation) {
            var points = [];
            var x, y;
            points.push(sourcePortLocation);
            x = sourcePortLocation.x;
            var srcExtension = 15;
            var tarExtension = 15;
            if (sourceBounds.width == 0) srcExtension = 0;
            if (targetBounds.width == 0) tarExtension = 0;
            var min = 30;
            if (targetPortLocation.x > sourcePortLocation.x) {
                if (sourceBounds.top >= targetBounds.bottom) {
                    if ((sourceBounds.top - targetBounds.bottom) >= 30) {
                        y = targetBounds.bottom + tarExtension;
                        points.push(ej.datavisualization.Diagram.Point(x, y));
                    }
                    else {
                        y = targetBounds.bottom + (sourceBounds.top - targetBounds.bottom) / 2;
                        points.push(ej.datavisualization.Diagram.Point(x, y));
                        if ((targetBounds.left - sourceBounds.right) >= 30) {
                            x = sourceBounds.right + srcExtension;
                        }
                        else {
                            x = sourceBounds.right + (targetBounds.left - sourceBounds.right) / 2;
                        }
                        points.push(ej.datavisualization.Diagram.Point(x, y));
                        y = targetBounds.bottom + tarExtension;
                        points.push(ej.datavisualization.Diagram.Point(x, y));
                    }
                }
                else {
                    if (targetBounds.top < sourceBounds.top) {
                        if (targetBounds.left >= (sourceBounds.right + 15)) {
                            y = sourceBounds.top - srcExtension;
                        }
                        else {
                            y = targetBounds.top - tarExtension;
                        }
                    }
                    else {
                        y = sourceBounds.top - srcExtension;
                    }
                    points.push(ej.datavisualization.Diagram.Point(x, y));
                    if (targetBounds.left >= (sourceBounds.right + 15)) {
                        x = sourceBounds.right + srcExtension;
                    }
                    else {
                        x = targetBounds.right + tarExtension;
                    }
                    points.push(ej.datavisualization.Diagram.Point(x, y));
                    y = targetBounds.bottom + tarExtension;
                    points.push(ej.datavisualization.Diagram.Point(x, y));
                }
                points.push(ej.datavisualization.Diagram.Point(targetPortLocation.x, y));
            }
            else if (targetPortLocation.x < sourcePortLocation.x) {
                if (sourceBounds.top >= targetBounds.bottom) {
                    if ((sourceBounds.top - targetBounds.bottom) > 30) {
                        y = targetBounds.bottom + tarExtension;
                        points.push(ej.datavisualization.Diagram.Point(x, y));
                    }
                    else {
                        y = targetBounds.bottom + (sourceBounds.top - targetBounds.bottom) / 2;
                        points.push(ej.datavisualization.Diagram.Point(x, y));
                        if ((sourceBounds.left - targetBounds.right) >= 30) {
                            x = sourceBounds.left - srcExtension;
                        }
                        else {
                            x = targetBounds.right + (sourceBounds.left - targetBounds.right) / 2;
                        }
                        points.push(ej.datavisualization.Diagram.Point(x, y));
                        y = targetBounds.bottom + tarExtension;
                        points.push(ej.datavisualization.Diagram.Point(x, y));
                    }
                }
                else {
                    if (sourceBounds.left >= (targetBounds.right + 15)) {
                        y = sourceBounds.top - srcExtension;
                    }
                    else {
                        y = sourceBounds.top - srcExtension;
                    }
                    points.push(ej.datavisualization.Diagram.Point(x, y));
                    if (sourceBounds.left >= (targetBounds.right + 15)) {
                        x = targetBounds.right + tarExtension;
                    }
                    else {
                        x = sourceBounds.right + srcExtension;
                    }
                    points.push(ej.datavisualization.Diagram.Point(x, y));
                    if (sourceBounds.left >= (targetBounds.right + 15)) {
                        y = targetBounds.bottom + tarExtension;
                    }
                    else {
                        if (targetBounds.bottom <= sourceBounds.bottom) {
                            y = sourceBounds.bottom + srcExtension;
                        }
                        else {
                            y = targetBounds.bottom + tarExtension;
                        }
                    }
                    points.push(ej.datavisualization.Diagram.Point(x, y));
                }
                points.push(ej.datavisualization.Diagram.Point(targetPortLocation.x, y));
            }
            points.push(targetPortLocation);
            return points;
        },
        _bottomToBottom: function (targetBounds, targetPortLocation, sourceBounds, sourcePortLocation) {
            var points = [];
            var x, y;
            points.push(sourcePortLocation);
            x = sourcePortLocation.x;
            var srcExtension = 15;
            var tarExtension = 15;
            if (sourceBounds.width == 0) srcExtension = 0;
            if (targetBounds.width == 0) tarExtension = 0;
            if (targetPortLocation.x >= sourcePortLocation.x) {
                if (sourceBounds.bottom >= targetBounds.bottom) {
                    y = sourceBounds.bottom + srcExtension;
                    points.push(ej.datavisualization.Diagram.Point(x, y));
                    if ((targetPortLocation.x) < sourceBounds.right) {
                        x = sourceBounds.right + srcExtension;
                        points.push(ej.datavisualization.Diagram.Point(x, y));
                        if (sourceBounds.top >= targetBounds.bottom) {
                            if ((sourceBounds.top - targetBounds.bottom) >= 30) {
                                y = targetBounds.bottom + tarExtension;
                            }
                            else {
                                y = targetBounds.bottom + (sourceBounds.top - targetBounds.bottom) / 2;
                            }
                        }
                        else {
                            y = targetBounds.bottom + tarExtension;
                        }
                        points.push(ej.datavisualization.Diagram.Point(x, y));
                    }
                }
                else {
                    if (sourcePortLocation.x > targetBounds.left) {
                        if ((targetBounds.bottom - sourceBounds.bottom) >= 30) {
                            y = sourceBounds.bottom + srcExtension;
                        }
                        else {
                            y = sourceBounds.bottom + (targetBounds.bottom - sourceBounds.bottom) / 2;
                        }
                        points.push(ej.datavisualization.Diagram.Point(x, y));
                        x = targetBounds.left - tarExtension;
                        points.push(ej.datavisualization.Diagram.Point(x, y));
                        y = targetBounds.bottom + tarExtension;
                        points.push(ej.datavisualization.Diagram.Point(x, y));
                    }
                    else {
                        y = targetBounds.bottom + tarExtension;
                        points.push(ej.datavisualization.Diagram.Point(x, y));
                    }
                }
            }
            else {
                if (sourceBounds.bottom >= targetBounds.bottom) {
                    y = sourceBounds.bottom + srcExtension;
                    points.push(ej.datavisualization.Diagram.Point(x, y));
                    if (targetPortLocation.x > sourceBounds.left) {
                        x = sourceBounds.left - srcExtension;
                        points.push(ej.datavisualization.Diagram.Point(x, y));
                        if (sourceBounds.top >= targetBounds.bottom) {
                            if ((sourceBounds.top - targetBounds.bottom) >= 30) {
                                y = targetBounds.bottom + tarExtension;
                            }
                            else {
                                y = targetBounds.bottom + (sourceBounds.top - targetBounds.bottom) / 2;
                            }
                        }
                        else {
                            y = targetBounds.bottom + tarExtension;
                        }
                        points.push(ej.datavisualization.Diagram.Point(x, y));
                    }
                }
                else {
                    if (sourcePortLocation.x < targetBounds.right) {
                        if ((targetBounds.bottom - sourceBounds.bottom) >= 30) {
                            y = sourceBounds.bottom + srcExtension;
                        }
                        else {
                            y = sourceBounds.bottom + (targetBounds.bottom - sourceBounds.bottom) / 2;
                        }
                        points.push(ej.datavisualization.Diagram.Point(x, y));
                        x = targetBounds.right + tarExtension;
                        points.push(ej.datavisualization.Diagram.Point(x, y));
                        y = targetBounds.bottom + tarExtension;
                        points.push(ej.datavisualization.Diagram.Point(x, y));
                    }
                    else {
                        y = targetBounds.bottom + tarExtension;
                        points.push(ej.datavisualization.Diagram.Point(x, y));
                    }
                }
            }
            points.push(ej.datavisualization.Diagram.Point(targetPortLocation.x, y));
            points.push(targetPortLocation);
            return points;
        },
        _addOrthogonalPoints: function (line, prev, next, sourcePt, endPt, runAngle) {
            var points = [];
            var strt = prev ? prev._endPoint : sourcePt;
            points.push(strt);
            if (!line._direction) line._direction = this._getOrthoDirection(strt, endPt, prev);
            var angle = (!runAngle && runAngle != 0) ? this._directionToAngle(line._direction || this._getBezierDirection(strt, endPt)) : runAngle;
            var direction = line._direction;
            var angles = { "right": 0, "left": 180, "top": 270, "bottom": 90, "auto": 90 };
            if (direction == "left" || direction == "right" || direction == "top" || direction == "bottom") {
                angle = angles[direction];
            }
            else {
                switch (direction) {
                    case "straight":
                        angle = 1 - 1;
                        break;
                    case "clockWise90":
                        angle += 91 - 1;
                        break;
                    case "opposite":
                        angle += 181 - 1;
                        break;
                    case "antiClockWise90":
                        angle += 271 - 1;
                        break;
                }
            }
            if ((line.length || line.length == 0) && line.direction) {
                var ctrlPoint1 = ej.datavisualization.Diagram.Geometry.transform(strt, angle || 0, line._length);
                ctrlPoint1.x = Math.round(ctrlPoint1.x * 100) / 100;
                ctrlPoint1.y = Math.round(ctrlPoint1.y * 100) / 100;
                points.push(ctrlPoint1);
                line._endPoint = ctrlPoint1;
                if (next) {
                    next._startPoint = line._endPoint;
                    if (next.points) {
                        next.points[0] = line._endPoint;
                        if (next.type == "bezier") {
                            this._updateBezierPoints(next);
                        }
                    }
                }
            }
            else {
                if (strt.x != endPt.x && strt.y != endPt.y) {
                    var extra = 20;
                    if (!prev) {
                        var ctrlPoint1 = ej.datavisualization.Diagram.Geometry.transform(strt, angle, extra);
                    }
                    var tempx = endPt.x - (ctrlPoint1 ? ctrlPoint1.x : strt.x);
                    var tempy = endPt.y - (ctrlPoint1 ? ctrlPoint1.y : strt.y);
                    tempx = Math.abs(tempx) <= 0.0001 ? 0 : tempx;
                    tempy = Math.abs(tempy) <= 0.0001 ? 0 : tempy;
                    var direction = line._direction == "auto" ? prev ? prev._direction == "left" || prev.direction == "right" ? "left" : "top" : "bottom" : line._direction;
                    if ((direction == "left" || direction == "right") && tempx != 0 || tempy == 0) {
                        tempy = tempy > 0 ? tempy : -tempy;
                        if (tempy > 0) {
                            var ctrlPoint2 = ej.datavisualization.Diagram.Geometry.transform(ctrlPoint1 || strt, 90, endPt.y - (ctrlPoint1 ? ctrlPoint1.y : strt.y));
                        }
                    }
                    else {
                        tempx = tempx > 0 ? tempx : -tempx;
                        if (tempx > 0) {
                            var ctrlPoint2 = ej.datavisualization.Diagram.Geometry.transform(ctrlPoint1 || strt, 0, endPt.x - (ctrlPoint1 ? ctrlPoint1.x : strt.x));
                        }

                    }
                    if (ctrlPoint1) {
                        ctrlPoint1.x = Math.round(ctrlPoint1.x * 100) / 100;
                        ctrlPoint1.y = Math.round(ctrlPoint1.y * 100) / 100;
                        points.push(ctrlPoint1);
                    }
                    if (ctrlPoint2) {
                        ctrlPoint2.x = Math.round(ctrlPoint2.x * 100) / 100;
                        ctrlPoint2.y = Math.round(ctrlPoint2.y * 100) / 100;
                        points.push(ctrlPoint2);
                    }
                }
                line._direction = this._getBezierDirection(points[points.length - 1], endPt);
                points.push(endPt);
                line._endPoint = endPt;
                if (next) {
                    next._startPoint = line._endPoint;
                    if (!next.points) next.points = [];
                    next.points[0] = line._endPoint;
                    if (next.type == "bezier") {
                        this._updateBezierPoints(next);
                    }
                }
            }
            line.points = points;
            return angle;
        },
        _getOrthoDirection: function (src, point, prev) {
            if (!prev) {
                if (src.y < point.y) {
                    return "bottom";
                }
                else if (src.y > point.y) {
                    return "top";
                }
                else if (src.x < point.x) {
                    return "right";
                }
                else if (src.x > point.x) {
                    return "left";
                }
            }
            else {
                var tar = point;
                if (Math.abs(tar.x - src.x) > Math.abs(tar.y - src.y) && (prev && (prev._direction == "left" || prev._direction == "right"))) {
                    return src.x < tar.x ? "right" : "left";
                }
                else {
                    return src.y < tar.y ? "bottom" : "top";
                }
            }
        },
        _directionToAngle: function (direction) {
            var angles = { "right": 0, "left": 180, "top": 270, "bottom": 90, "auto": 90 };
            return angles[direction];
        },
        _resetOrthogonalPoints: function (line) {
            var points = [];
            points.push(line.sourcePoint);
            points.push(line.targetPoint);
            line.points = [];
            line.sourcePoint = points[0];
            line.points.push(points[0]);
            var ctrlPoints = this._constructOrthogonalPoints(points[0], points[1]);
            line.points.push(ctrlPoints[0]);
            line.points.push(ctrlPoints[1]);
            line.targetPoint = points[1];
            line.points.push(points[1]);
        },
        _updateBezierPoints: function (line, targetNode, targetPort, targetBounds, sourceNode, sourcePort, sourceBounds) {
            var bounds;
            var angles = { "right": 0, "left": 180, "top": 270, "bottom": 90 };
            if (line.vector1 || line.vector2) {
                if (line.vector1) {
                    line._point1 = ej.datavisualization.Diagram.Geometry.transform(line._startPoint, line.vector1.angle, line.vector1.distance);
                }
                if (line.vector2) {
                    line._point2 = ej.datavisualization.Diagram.Geometry.transform(line._endPoint, line.vector2.angle, line.vector2.distance);
                }
            }
            if ((line.point1 == null && !line.vector1) || (line.point2 == null && !line.vector2)) {
                var target = line._endPoint;
                var source = line._startPoint;
                var dir;
                var matrix;
                if (!sourcePort)
                    dir = this._getBezierDirection(source, target);
                else {
                    bounds = sourceBounds;
                    source = this._getPortPosition(sourcePort, bounds);
                    if (sourceNode.rotateAngle !== 0) {
                        matrix = ej.Matrix.identity();
                        ej.Matrix.rotate(matrix, sourceNode.rotateAngle, sourceNode.offsetX, sourceNode.offsetY);
                        source = ej.Matrix.transform(matrix, source);
                    }
                    dir = this._getDirection(bounds, source);
                }
                if (!line.vector1)
                    line._point1 = line.point1 || this._getBezierAdjPoint(angles[dir], line._startPoint, line._endPoint);
                if (!targetPort) {
                    dir = this._getBezierDirection(target, source);
                }
                else {
                    bounds = targetBounds;
                    target = this._getPortPosition(targetPort, bounds);
                    if (targetNode.rotateAngle !== 0) {
                        matrix = ej.Matrix.identity();
                        ej.Matrix.rotate(matrix, targetNode.rotateAngle, targetNode.offsetX, targetNode.offsetY);
                        target = ej.Matrix.transform(matrix, target);
                    }
                    dir = this._getDirection(bounds, target);
                }
                if (!line.vector2)
                    line._point2 = line.point2 || this._getBezierAdjPoint(angles[dir], line._endPoint, line._startPoint);
            }
        },
        _getBezierDirection: function (src, tar) {
            if (Math.abs(tar.x - src.x) > Math.abs(tar.y - src.y)) {
                return src.x < tar.x ? "right" : "left";
            }
            else {
                return src.y < tar.y ? "bottom" : "top";
            }
        },
        _getDirection: function (bounds, point) {
            var points = bounds.points;
            var center = bounds.center;
            var part = 360 / (8 * (bounds.width / bounds.height));
            var fourty5 = part;
            var one35 = (180 - part);
            var two25 = one35 + (2 * part);
            var three15 = 360 - part;
            var dir;
            var angle = ej.datavisualization.Diagram.Geometry.findAngle(point, center);
            if (angle > fourty5 && angle < one35) {
                dir = "top";
            }
            else if (angle > one35 && angle < two25) {
                dir = "right";
            }
            else if (angle > two25 && angle < three15) {
                dir = "bottom";
            }
            else {
                dir = "left";
            }
            return dir;
        },
        _getBezierAdjPoint: function (angle, srcEnd, tarEnd) {
            var distance = 60;
            var endAdj = ej.datavisualization.Diagram.Point(0, 0);
            var dir;
            if (angle > 45 && angle < 135) {
                dir = "Bottom";
            }
            else if (angle > 135 && angle < 225) {
                dir = "Left";
            }
            else if (angle > 225 && angle < 315) {
                dir = "Top";
            }
            else {
                dir = "Right";
            }
            switch (dir) {
                case "Right":
                    distance = Math.min(Math.abs(srcEnd.x - tarEnd.x) * 0.45, distance);
                    endAdj = ej.datavisualization.Diagram.Point(srcEnd.x + distance, srcEnd.y);
                    break;
                case "Bottom":
                    distance = Math.min(Math.abs(srcEnd.y - tarEnd.y) * 0.45, distance);
                    endAdj = ej.datavisualization.Diagram.Point(srcEnd.x, srcEnd.y + distance);
                    break;
                case "Left":
                    distance = Math.min(Math.abs(srcEnd.x - tarEnd.x) * 0.45, distance);
                    endAdj = ej.datavisualization.Diagram.Point(srcEnd.x - distance, srcEnd.y);
                    break;
                case "Top":
                    distance = Math.min(Math.abs(srcEnd.y - tarEnd.y) * 0.45, distance);
                    endAdj = ej.datavisualization.Diagram.Point(srcEnd.x, srcEnd.y - distance);
                    break;
            }
            return endAdj;
        },
        _setLineEndPoint: function (connector, point, isTarget, dock) {
            var line = connector;
            point.x = Math.round(point.x * 100) / 100;
            point.y = Math.round(point.y * 100) / 100;
            if (isTarget) {
                var lastsegment = connector.segments[connector.segments.length - 1];
                line.targetPoint = point;
                if (lastsegment.type != "orthogonal") {
                    lastsegment.point = lastsegment._point = point;
                }
                lastsegment.points[lastsegment.points.length - 1] = point;
                lastsegment._endPoint = point;
            }
            else {
                line.sourcePoint = point;
                var first = connector.segments[0];
                first.points[0] = point;
                first._startPoint = point;
                if (first.type == "orthogonal" && !connector.sourcePort) {
                    if (first.points.length == 2) {
                        if (first._direction == "left" || first._direction == "right") {
                            first._endPoint.y = first._startPoint.y;
                            var length = Math.abs(first._startPoint.x - first._endPoint.x);
                        }
                        else {
                            first._endPoint.x = first._startPoint.x;
                            var length = Math.abs(first._startPoint.y - first._endPoint.y);
                        }


                        connector.segments[0]._length = length;
                        if (connector.segments[0].length || connector.segments[0].length === 0) {
                            connector.segments[0].length = connector.segments[0]._length;
                            connector.segments[0].direction = connector.segments[0]._direction;
                        }
                    }
                }
            }
            var segment = isTarget ? connector.segments[connector.segments.length - 1] : connector.segments[0];
            if (segment.type === "orthogonal" && ((!isTarget && !connector.sourcePort) || isTarget && !connector.targetPort) && (!(connector.sourceNode && connector.targetNode) || connector.segments.length > 1)) {
                segment.points = [];
                this._addOrthogonalPoints(segment, isTarget ? connector.segments[connector.segments.length - 2] : null,
                    isTarget ? null : connector.segments[1], connector.sourcePoint, connector.targetPoint);

            }
            else if (segment.type === "bezier") {
                this._updateBezierPoints(segment);
            }

        },
        _setBezierPoint: function (segment, point, whichpoint) {
            if (whichpoint === "bezierpoint1") {
                if (segment.vector1) {
                    segment.vector1 = {
                        distance: ej.datavisualization.Diagram.Geometry.distance(segment._startPoint, point),
                        angle: ej.datavisualization.Diagram.Geometry.findAngle(segment._startPoint, point),
                    };
                    segment._point1 = point;
                }
                else
                    segment.point1 = segment._point1 = point;
            }
            else {
                if (segment.vector2) {
                    segment.vector2 = {
                        distance: ej.datavisualization.Diagram.Geometry.distance(segment._endPoint, point),
                        angle: ej.datavisualization.Diagram.Geometry.findAngle(segment._endPoint, point),
                    };
                    segment._point2 = point;
                }
                else
                    segment.point2 = segment._point2 = point;
            }
        },
        _setPoints: function (line, points) {
            line.sourcePoint = line.points[0] = points[0];
            line.targetPoint = line.points[line.points.length - 1] = points[points.length - 1];
            line.points = points;
            if (line.type === "bezier") {
                line.point1 = points[2] || line.point1;
                line.point2 = points[3] || line.point2;
                if (!line.point1)
                    line._point1 = line.point1;
                if (!line.point2)
                    line._point2 = line.point2;
                this._updateBezierPoints(line);
            }
        },
        _updateConnectorSegments: function (connector, ignoreSource, ignoreTarget, diagram) {
            var runAngle;
            var startPoint = connector.sourcePoint;
            for (var i = 0; i < connector.segments.length; i++) {
                var segment = connector.segments[i];
                var prev = connector.segments[i - 1];
                var next = connector.segments[i + 1];
                segment._startPoint = prev ? prev._endPoint : startPoint;
                if (!(i == 0 && ignoreSource) && !(i == connector.segments.length - 1 && ignoreTarget)) {
                    if (segment.type == "orthogonal")
                        runAngle = ej.datavisualization.Diagram.Util._addOrthogonalPoints(segment, prev, next, connector.sourcePoint, connector.targetPoint, runAngle);
                    else {
                        runAngle = ej.datavisualization.Diagram.Geometry.findAngle(startPoint, segment._endPoint);
                        if (segment.type == "bezier") {
                            this._updateBezierPoints(segment);
                        }
                    }
                }
                startPoint = segment._endPoint;
            }
            if (diagram) {
                diagram._dock(connector, diagram.nameTable);
            }
        },
        _translateLine: function (connector, dx, dy, parent) {
            for (var i = 0; i < connector.segments.length; i++) {
                var segment = connector.segments[i];
                var next = connector.segments[i + 1];
                for (var j = 0; j < segment.points.length; j++) {
                    if (j == 0) {
                        segment._startPoint = segment.points[j] = ej.datavisualization.Diagram.Geometry.translate(segment.points[j], dx, dy);
                        if (i == 0) {
                            connector.sourcePoint = segment._startPoint;
                        }
                    }
                    else if (j == segment.points.length - 1) {
                        segment._endPoint = segment.points[j] = ej.datavisualization.Diagram.Geometry.translate(segment.points[j], dx, dy)
                        if (i == connector.segments.length - 1)
                            connector.targetPoint = segment._endPoint;
                    }
                    else segment.points[j] = ej.datavisualization.Diagram.Geometry.translate(segment.points[j], dx, dy);
                }
                if (segment.type === ej.datavisualization.Diagram.Segments.Bezier) {
                    segment._point1 = ej.datavisualization.Diagram.Geometry.translate(segment._point1, dx, dy);
                    segment._point2 = ej.datavisualization.Diagram.Geometry.translate(segment._point2, dx, dy);
                    if (segment.point1) { segment.point1 = segment._point1; }
                    if (segment.point2) { segment.point2 = segment._point2; }
                    segment.point = segment._endPoint;
                }
                else if (segment.type == "straight") {
                    segment.point = segment._point = segment._endPoint;
                }
            }
            //if (parent) {
            //    if(connector.segments[0].type == "orthogonal"){
            //        if (!this.isSourceConnected(parent)) {
            //            this._addOrthogonalPoints(connector.segments[0],null,connector.segments[1],connector.sourcePoint,connector.targetPoint);
            //        }
            //    }
            //    if(connector.segments[connector.segments.length-1].type == "orthogonal"){
            //        var segment=connector.segments[connector.segments.length-1];
            //        if( !this.isTargetConnected(parent)){
            //            this._addOrthogonalPoints(segment,connector.segments[connector.segments.length-2],null,connector.sourcePoint,connector.targetPoint);
            //        }
            //    }
            //}
        },
        _adjustPoint: function (source, target, isStart, length) {
            var pt = isStart ? { x: source.x, y: source.y } : { x: target.x, y: target.y };

            if (source.x == target.x) {
                if (source.y < target.y && isStart || source.y > target.y && !isStart)
                    pt.y += length;
                else pt.y -= length;
            }
            else if (source.y == target.y) {
                if (source.x < target.x && isStart || source.x > target.x && !isStart)
                    pt.x += length;
                else pt.x -= length;
            }
            else {
                var angle;
                if (isStart) {
                    angle = ej.datavisualization.Diagram.Geometry.findAngle(source, target);
                    pt = ej.datavisualization.Diagram.Geometry.transform(source, angle, length);
                }
                else {
                    angle = ej.datavisualization.Diagram.Geometry.findAngle(target, source);
                    pt = ej.datavisualization.Diagram.Geometry.transform(target, angle, length);
                }
            }
            return pt;
        },
        _getLabelPosition: function (label, bounds) {
            var x = (bounds.width * label.offset.x);
            var y = (bounds.height * label.offset.y);
            if (label.horizontalAlignment != "right")
                x += label.margin.left;
            else
                x -= label.margin.right;
            if (label.verticalAlignment != "bottom")
                y += label.margin.top;
            else
                y -= label.margin.bottom;
            return ej.datavisualization.Diagram.Point(x, y);
        },
        _getPortPosition: function (port, bounds, bExcludeBoundsLocation) {
            if (bExcludeBoundsLocation) {
                return ej.datavisualization.Diagram.Point(bounds.width * port.offset.x, bounds.height * port.offset.y);
            }
            else {
                var x = bounds.x + (bounds.width * port.offset.x);
                var y = bounds.y + (bounds.height * port.offset.y);
                return ej.datavisualization.Diagram.Point(x, y);
            }
        },

        _translate: function (node, dx, dy, nameTable, isContainer) {
            if (ej.datavisualization.Diagram.Util.canMove(node)) {
                if (!node.segments) {
                    node.offsetX += dx;
                    node.offsetY += dy;
                    if (node.isGroup && !isContainer) {
                        var nodes = node.children;
                        var child;
                        for (var i = 0; i < nodes.length; i++) {
                            child = nameTable[nodes[i]];
                            this._translate(child, dx, dy, nameTable);
                            if (child.parent && (child.parent != node.name && !node.isPseudoGroup))
                                ej.datavisualization.Diagram.Util._updateGroupBounds(nameTable[child.parent], nameTable);
                        }
                        if (!node.isGroup && node.container)
                            ej.datavisualization.Diagram.Util._updateGroupBounds(node, nameTable);
                    }
                } else {
                    ej.datavisualization.Diagram.Util._translateLine(node, dx, dy, node);
                }
            }
        },

        scale: function (node, sw, sh, pivot, nameTable, skipScalOnChild, updateMinMax, isHelper) {
            if (!node.container) {
                var matrix = ej.Matrix.identity();
                if (!node.segments) {
                    if (node.isGroup) {
                        var nodes = node.children;
                        var child;
                        for (var i = 0; i < nodes.length; i++) {
                            child = nameTable[nodes[i]];
                            if (child) {
                                this.scale(child, sw, sh, pivot, nameTable);
                                if (child.parent && (child.parent != node.name && !node.isPseudoGroup))
                                    ej.datavisualization.Diagram.Util._updateGroupBounds(nameTable[child.parent], nameTable);
                            }
                        }
                        ej.datavisualization.Diagram.Util._updateGroupBounds(node, nameTable);
                    } else {
                        ej.Matrix.rotate(matrix, -node.rotateAngle, pivot.x, pivot.y);
                        ej.Matrix.scale(matrix, sw, sh, pivot.x, pivot.y);
                        ej.Matrix.rotate(matrix, node.rotateAngle, pivot.x, pivot.y);
                        var newPosition = ej.Matrix.transform(matrix, ej.datavisualization.Diagram.Point(node.offsetX, node.offsetY));
                        var width = node.width * sw;
                        var height = node.height * sh;
                        if (width > 1) {
                            node.width = width;
                            node.offsetX = newPosition.x;
                        }
                        if (height > 1) {
                            node.height = height;
                            node.offsetY = newPosition.y;
                        }
                    }
                } else {
                    ej.Matrix.scale(matrix, sw, sh, pivot.x, pivot.y);
                    for (var i = 0; i < node.segments.length; i++) {
                        var segment = node.segments[i];
                        for (var j = 0; j < segment.points.length; j++) {
                            segment.points[j] = ej.Matrix.transform(matrix, segment.points[j]);
                            if (i == 0 && j == 0 && !node.sourceNode) {
                                segment._startPoint = segment.points[0];
                                segment._endPoint = segment.points[segment.points.length - 1];
                                ej.datavisualization.Diagram.Util._setLineEndPoint(node, ej.Matrix.transform(matrix, node.sourcePoint), false);
                            }
                            if (i == node.segments.length - 1 && j == segment.points.length - 1 && !node.targetNode) {
                                segment._startPoint = segment.points[0];
                                segment._endPoint = segment.points[segment.points.length - 1];
                                node.targetPoint = segment._endPoint;
                            }
                        }
                        segment._startPoint = segment.points[0];
                        segment._endPoint = segment.points[segment.points.length - 1];
                        if (segment.type == "orthogonal") {
                            if (segment.length || segment.length === 0)
                                segment.length = segment._length = ej.datavisualization.Diagram.Geometry.distance(segment._startPoint, segment._endPoint);
                            else
                                ej.datavisualization.Diagram.Util._addOrthogonalPoints(segment, node.segments[i - 1], node.segments[i + 1], node.sourcePoint, node.targetPoint);
                        } else {
                            segment.point = segment._endPoint;
                        }
                    }
                }
            }
            else {
                switch (node.container.type) {
                    case "canvas":
                        var newSize = {};
                        var matrix = ej.Matrix.identity();
                        ej.Matrix.rotate(matrix, -node.rotateAngle, pivot.x, pivot.y);
                        ej.Matrix.scale(matrix, sw, sh, pivot.x, pivot.y);
                        ej.Matrix.rotate(matrix, node.rotateAngle, pivot.x, pivot.y);
                        newPosition = ej.Matrix.transform(matrix, ej.datavisualization.Diagram.Point(node.offsetX, node.offsetY));
                        width = node.width * sw;
                        height = node.height * sh;
                        if (updateMinMax) {
                            node.minHeight = height;
                            node.minWidth = width;
                        }
                        if (isHelper != true)
                            newSize = this._getMinMaxSize(node, width, height);
                        else {
                            newSize.width = width;
                            newSize.height = height;
                        }
                        if (newSize && newSize.width > 1) {
                            var x = node.offsetX - node.width / 2;
                            node.width = newSize.width;
                            node.offsetX = x + node.width / 2;
                        }
                        if (newSize && newSize.height > 1) {
                            var y = node.offsetY - node.height / 2;
                            node.height = newSize.height;
                            node.offsetY = y + node.height / 2;
                        }
                        if (!skipScalOnChild)
                            ej.datavisualization.Diagram.SvgContext._alignOnCanvas1(node, this, true);
                        break;
                    case "stack":
                        var matrix = ej.Matrix.identity();
                        ej.Matrix.rotate(matrix, -node.rotateAngle, pivot.x, pivot.y);
                        ej.Matrix.scale(matrix, sw, sh, pivot.x, pivot.y);
                        ej.Matrix.rotate(matrix, node.rotateAngle, pivot.x, pivot.y);
                        newPosition = ej.Matrix.transform(matrix, ej.datavisualization.Diagram.Point(node.offsetX, node.offsetY));
                        width = (node.width * sw) - (node.marginLeft + node.marginRight);
                        height = (node.height * sh) - (node.marginTop + node.marginBottom);
                        newSize = this._getMinMaxSize(node, width, height);
                        if (newSize.width > 1) {
                            x = node.offsetX - node.width / 2;
                            node.width = newSize.width - node.marginLeft;
                            node.offsetX = x + node.width / 2 + (node.marginLeft);
                        }
                        if (newSize.height > 1) {
                            y = node.offsetY - node.height / 2;
                            node.height = newSize.height - node.marginTop;
                            node.offsetY = y + node.height / 2 + (node.marginTop);
                        }
                        ej.datavisualization.Diagram.SvgContext._alignOnStack(node, this);
                        break;
                }
            }
        },
        _updateChildBounds: function (node, nameTable) {
            var offX, offY, dx, dy;
            dx = node.width;
            dy = node.height;
            offX = node.offsetX;
            offY = node.offsetY;
            var childBounds = ej.datavisualization.Diagram.Util._getChildrenBounds(node, nameTable);
            node.offsetX = childBounds.x + childBounds.width / 2;
            node.offsetY = childBounds.y + childBounds.height / 2;
            node.height = childBounds.height;
            node.width = childBounds.width;
            var deltaWidth = dx / node.width;
            var deltaHeight = dy / node.height;
            this.scale(node, deltaWidth, deltaHeight, node.pivot, nameTable);
            this._translate(node, offX - node.offsetX, offY - node.offsetY, nameTable);
        },

        _updateGroupBounds: function (node, nameTable, diagram) {
            if (node && !node.container) {
                if (node && node.isGroup && node.children.length > 0) {
                    this._updateRotateAngle(node, nameTable);
                    var bounds = this._getChildrenBounds(node, nameTable);
                    var x = bounds.x + bounds.width * node.pivot.x;
                    var y = bounds.y + bounds.height * node.pivot.y;
                    var matrix = ej.Matrix.identity();
                    ej.Matrix.rotate(matrix, node.rotateAngle);
                    var newposition = ej.Matrix.transform(matrix, { x: x, y: y });
                    if (bounds) {
                        node.offsetX = newposition.x;
                        node.offsetY = newposition.y;
                        node.width = bounds.width;
                        node.height = bounds.height;
                    }
                }
            }
        },
        _getChildrenBounds: function (group, nameTable, diagram) {
            var children = group.children, rect, bounds = ej.datavisualization.Diagram.Rectangle(), child;
            if (children.length > 0) {
                child = nameTable[children[0]];
                if (child)
                    bounds = this._rotateChildBounds(child, group);
            }
            for (var i = 1, len = children.length; i < len; i++) {
                child = nameTable[children[i]];
                if (child) {
                    if (child.isGroup) {
                        this._updateGroupBounds(child, nameTable, diagram);
                        rect = this._rotateChildBounds(child, group);
                    } else
                        rect = this._rotateChildBounds(child, group);
                }
                if (rect)
                    bounds = ej.datavisualization.Diagram.Geometry.union(bounds, rect);
            }
            return bounds;
        },

        _updateRotateAngle: function (node, nameTable) {
            if (node.offsetX == 0 && node.offsetY == 0 && node.width == 0 && node.height == 0) {
                for (var i = 0; i < node.children.length; i++) {
                    var child = nameTable[node.children[i]];
                    if (child && !child.segments && !node.rotateAngle) {
                        node.rotateAngle = child.rotateAngle;
                        return;
                    }
                }
            }
        },

        _rotateChildBounds: function (child, group) {
            var matrix = ej.Matrix.identity();
            ej.Matrix.rotate(matrix, -group.rotateAngle);
            var bounds = this.bounds(child);
            var topleft = ej.Matrix.transform(matrix, bounds.topLeft);
            var topright = ej.Matrix.transform(matrix, bounds.topRight);
            var bottomLeft = ej.Matrix.transform(matrix, bounds.bottomLeft);
            var bottomRight = ej.Matrix.transform(matrix, bounds.bottomRight);
            var x = Math.min(topleft.x, topright.x, bottomLeft.x, bottomRight.x);
            var y = Math.min(topleft.y, topright.y, bottomLeft.y, bottomRight.y);
            return {
                x: x,
                width: Math.max(topleft.x, topright.x, bottomLeft.x, bottomRight.x) - x,
                y: y,
                height: Math.max(topleft.y, topright.y, bottomLeft.y, bottomRight.y) - y
            };
        },

        updateBridging: function (connector, diagram) {
            var points1 = this.getPoints(connector);
            var bridgeSpacing = connector.bridgeSpace;
            var lstBridge = [];
            var count = -1;
                if (connector.segments.length > 0) {
                    for (var m = 0; m < connector.segments.length; m++) {
                        connector.segments[m]._bridges = [];
                    }
                }
            if (ej.datavisualization.Diagram.Util.canBridge(connector, diagram)) {
                var childBounds = ej.datavisualization.Diagram.Util.bounds(connector);
                var quads = ej.datavisualization.Diagram.SpatialUtil.findQuads(diagram._spatialSearch, childBounds);
                for (var item = 0; item < quads.length; item++) {
                    var quad = quads[item];
                    if (quad.objects.length > 0) {
                        for (var q = 0; q < quad.objects.length; q++) {
                            var connector1 = quad.objects[q];
                            if (connector1.segments) {
                                    var bounds = ej.datavisualization.Diagram.Util.bounds(connector);
                                    var bounds1 = ej.datavisualization.Diagram.Util.bounds(connector1);
                                    if (ej.datavisualization.Diagram.Geometry.intersectsRect(bounds, bounds1)) {
                                        var points2 = this.getPoints(connector1);
                                        var intersectPts = this.interSect(points1, points2, false, diagram, connector.zOrder > connector1.zOrder ? true : false);
                                        /* Start For Loop */
                                        if (intersectPts.length > 0) {
                                            for (var i = 0; i < intersectPts.length; i++) {
                                                var fullLength = 0;
                                                var segmentIndex = 0;
                                                var length = 0;
                                                var pointIndex = 0;
                                                var obj = this.getLengthAtFractionPoint(connector, intersectPts[i]);
                                                length = obj.lengthAtFractionPt, fullLength = obj.fullLength, segmentIndex = obj.segmentIndex, pointIndex = obj.pointIndex;
                                                if (segmentIndex < 0) {
                                                    continue;
                                                }
                                                if (connector.segments[0]) {
                                                    var startBridge, endBridge;
                                                    var fractLength = (length - (bridgeSpacing / 2)) / fullLength;
                                                    startBridge = this.getPointAtLength((length - (bridgeSpacing / 2)), points1);
                                                    fractLength = (length + (bridgeSpacing / 2)) / fullLength;
                                                    endBridge = this.getPointAtLength((length + (bridgeSpacing / 2)), points1);
                                                    if (ej.datavisualization.Diagram.Geometry.isEmptyPoint(endBridge))
                                                        endBridge = startBridge;
                                                    var start, end;
                                                    if (segmentIndex === 0) {
                                                        start = connector.sourcePoint;
                                                    }
                                                    else {
                                                        start = connector.segments[segmentIndex - 1]._endPoint;
                                                    }
                                                    if (connector.segments[segmentIndex].type === "straight")
                                                        end = connector.segments[segmentIndex]._point;
                                                    else
                                                        end = connector.segments[segmentIndex].points[pointIndex];
                                                    var angle = this._findAngle(start, end);
                                                    if (lstBridge.length > segmentIndex && lstBridge[segmentIndex] != null) {
                                                        var fixedPoint;
                                                        if (segmentIndex === 0) {
                                                            fixedPoint = connector.sourcePoint;
                                                        }
                                                        else {
                                                            fixedPoint = connector.segments[segmentIndex - 1]._endPoint;
                                                        }
                                                        var fix = Math.abs(this.findLength(fixedPoint, endBridge));
                                                        var var1 = 0;
                                                        var insertAt = -1;
                                                        count = -1;
                                                        for (var k = 0; k < lstBridge[segmentIndex].bridges.length; k++) {
                                                            count++;
                                                            var arcSeg = lstBridge[segmentIndex].bridges[k];
                                                            var1 = Math.abs(this.findLength(fixedPoint, arcSeg.endPoint));
                                                            if (fix < var1) {
                                                                insertAt = count;
                                                                break;
                                                            }
                                                        }
                                                        if (insertAt >= 0) {
                                                            var paths;
                                                            paths = this.createSegment(startBridge, endBridge, angle, pointIndex, connector, diagram);
                                                            lstBridge[segmentIndex].bridges.splice(insertAt, 0, paths);
                                                            lstBridge[segmentIndex].bridges.join();
                                                            lstBridge[segmentIndex].bridgeStartPoint.splice(insertAt, 0, startBridge);
                                                            lstBridge[segmentIndex].bridgeStartPoint.join();
                                                            lstBridge[segmentIndex].segmentIndex = segmentIndex;
                                                        }
                                                        else {
                                                            var paths = this.createSegment(startBridge, endBridge, angle, pointIndex, connector, diagram);
                                                            lstBridge[segmentIndex].bridges.push(paths);
                                                            lstBridge[segmentIndex].bridgeStartPoint.push(startBridge);
                                                            lstBridge[segmentIndex].segmentIndex = segmentIndex;
                                                        }
                                                    }
                                                    else {
                                                        if (!isNaN(startBridge.x) && !isNaN(startBridge.y) && !ej.datavisualization.Diagram.Geometry.isEmptyPoint(endBridge)) {
                                                            var arcs = this.createSegment(startBridge, endBridge, angle, pointIndex, connector, diagram);
                                                            var stPoints = [];
                                                            var edPoints = [];
                                                            stPoints.push(startBridge);
                                                            edPoints.push(endBridge);
                                                            if (lstBridge.length < segmentIndex + 1) {

                                                            }
                                                            lstBridge[segmentIndex] = $.extend(true, {}, ej.datavisualization.Diagram.BridgeSegments, {});
                                                            lstBridge[segmentIndex].bridges.push(arcs);
                                                            lstBridge[segmentIndex].bridgeStartPoint = stPoints;
                                                            lstBridge[segmentIndex].segmentIndex = segmentIndex;

                                                        }
                                                    }
                                                }
                                            }
                                        }
                                        /* End For Loop */
                                    }
                                
                            }
                        }
                    }
                }

                if (lstBridge.length != 0) {
                    for (var i = 0; i < lstBridge.length; i++) {
                        var bridge = lstBridge[i];
                        if (!bridge)
                            continue;
                        for (var k = 1; k < bridge.bridges.length; k++) {
                            if (this.findLength(bridge.bridges[k].endPoint,
                                bridge.bridges[k - 1].endPoint) < bridgeSpacing) {
                                bridge.bridges[k - 1].endPoint = bridge.bridges[k].endPoint;
                                var subBridge = bridge.bridges[k - 1];
                                var arc = diagram.getBridgeSegment(subBridge.startPoint, subBridge.endPoint, subBridge.angle, bridgeSpacing, subBridge.sweep);
                                bridge.bridges[k - 1].path = arc;
                                bridge.bridges.splice(k, 1);
                                bridge.bridgeStartPoint.splice(k, 1);
                                k--;
                            }
                        }

                        var pre = connector.sourcePoint;
                        for (var j = 0; j < bridge.bridges.length; j++) {
                            var subBridge = bridge.bridges[j];
                            var end;
                            if (connector.segments[bridge.segmentIndex].type === "straight")
                                end = connector.segments[bridge.segmentIndex]._endPoint;
                            else if (connector.segments[bridge.segmentIndex].type === "orthogonal")
                                end = connector.segments[bridge.segmentIndex].points[subBridge.segmentPointIndex];
                            if (segmentIndex != 0)
                                pre = connector.segments[bridge.segmentIndex]._endPoint;
                            var rectPoints = [];
                            rectPoints.push(ej.datavisualization.Diagram.Point(end.x - bridgeSpacing, end.y - bridgeSpacing));
                            rectPoints.push(ej.datavisualization.Diagram.Point(end.x + bridgeSpacing, end.y + bridgeSpacing));
                            var rect = ej.datavisualization.Diagram.Geometry.rect(rectPoints);
                            rectPoints = [];
                            rectPoints.push(ej.datavisualization.Diagram.Point(pre.x - bridgeSpacing, pre.y - bridgeSpacing));
                            rectPoints.push(ej.datavisualization.Diagram.Point(pre.x + bridgeSpacing, pre.y + bridgeSpacing));
                            var rect1 = ej.datavisualization.Diagram.Geometry.rect(rectPoints);
                            if (!ej.datavisualization.Diagram.Geometry.containsPoint(rect, bridge.bridgeStartPoint[j]) &&
                                !ej.datavisualization.Diagram.Geometry.containsPoint(rect1, subBridge.endPoint)) {
                                connector.segments[bridge.segmentIndex]._bridges.push(subBridge);
                                pre = subBridge.endPoint;
                            }
                        }

                    }
                }
            }
        },

        createSegment: function (start, end, angle, segmentPointIndex, connector, diagram) {
            var pathSegment = $.extend(true, {}, ej.datavisualization.Diagram.PathSegment, {});
            var arc, sweep;
            sweep = this.sweepDirection(angle, connector,diagram);
            arc = diagram.getBridgeSegment(start, end, angle, connector.bridgeSpace, sweep);
            pathSegment.path = arc;
            pathSegment.startPoint = start;
            pathSegment.endPoint = end;
            pathSegment.angle = angle;
            pathSegment.segmentPointIndex = segmentPointIndex;
            pathSegment.sweep = sweep;
            return pathSegment;
        },

        sweepDirection: function (angle, connector, diagram) {
            var angle1 = Math.abs(angle);
            if (diagram.model.bridgeDirection === "top" || diagram.model.bridgeDirection === "bottom") {
                var sweep = 1;
                if (angle1 >= 0 && angle1 <= 90) {
                    sweep = 0;
                }
            }
            else if (diagram.model.bridgeDirection === "right" || diagram.model.bridgeDirection === "left") {
                var sweep = 1;
                if (angle < 0 && angle >= -180) {
                    sweep = 0;
                }
            }
            if (diagram.model.bridgeDirection === "right" || diagram.model.bridgeDirection === "bottom") {
                if (sweep === 0)
                    sweep = 1;
                else
                    sweep = 0;
            }
            return sweep;
        },

        getPointAtLength: function (length, pts) {
            var run = 0;
            var pre = null;
            var found = ej.datavisualization.Diagram.Point(0, 0);
            for (var i = 0; i < pts.length; i++) {
                var pt = pts[i];
                if (!pre) {
                    pre = pt;
                    continue;
                }
                else {
                    var l = this.findLength(pre, pt);
                    if (run + l > length) {
                        var r = length - run;
                        var deg = this.findAngle(pre, pt);
                        var x = r * Math.cos(deg * Math.PI / 180);
                        var y = r * Math.sin(deg * Math.PI / 180);
                        found = ej.datavisualization.Diagram.Point(pre.x + x, pre.y + y);
                        break;
                    }
                    else {
                        run += l;
                    }
                }
                pre = pt;
            }
            return found;
        },

        getLengthAtFractionPoint: function (connector, pointAt) {
            var confirm = 100, fullLength = 0, segmentIndex = -1, count = 0, lengthAtFractionPt = 0, pointIndex = -1;
            if (connector.segments === null)
                return 0;
            var segments = connector.segments;
            var pt1 = connector.sourcePoint;
            var previouspt2 = pt1;
            for (var i = 0; i < connector.segments.length; i++) {
                var points = [];
                var segment1 = connector.segments[i];
                for (var j = 0; j < segment1.points.length; j++) {
                    var point2 = segment1.points[j];
                    points.push(point2);
                }

                for (var j = 0; j < points.length; j++) {
                    var pt2 = points[j];
                    var suspect = this.getSlope(pt2, pt1, pointAt, connector);
                    if (suspect < confirm) {
                        confirm = suspect;
                        lengthAtFractionPt = fullLength + this.findLength(pointAt, previouspt2);
                        segmentIndex = count;
                        pointIndex = j;
                    }
                    fullLength += this.findLength(pt2, pt1);
                    pt1 = pt2;
                    previouspt2 = pt2;
                }
                count++;
            }
            return { "lengthAtFractionPt": lengthAtFractionPt, "fullLength": fullLength, "segmentIndex": segmentIndex, "pointIndex": pointIndex };
        },

        getSlope: function (startPt, endPt, point, connector) {
            var three = 3.0;
            var delX = Math.abs(startPt.x - endPt.x);
            var delY = Math.abs(startPt.y - endPt.y);
            var lhs = ((point.y - startPt.y) / (endPt.y - startPt.y));
            var rhs = ((point.x - startPt.x) / (endPt.x - startPt.x));
            if (!isFinite(lhs) || !isFinite(rhs) || isNaN(lhs) || isNaN(rhs)) {
                if (startPt.x === endPt.x) {
                    if (startPt.y === endPt.y) {
                        return 10000;
                    }
                    else if (((startPt.y > point.y) && (point.y > endPt.y)) || ((startPt.y < point.y) && (point.y < endPt.y))) {
                        return Math.abs(startPt.x - point.x);
                    }
                    else {
                        return 10000;
                    }
                }
                else if (startPt.y == endPt.y) {
                    if (((startPt.x > point.x) && (point.x > endPt.x)) || ((startPt.x < point.x) && (point.x < endPt.x))) {
                        return Math.abs(startPt.y - point.y);
                    }
                    else {
                        return 10000;
                    }
                }
                else {
                    return 10000;
                }
            }
            else if (connector.segments.length > 0) {
                if ((startPt.x >= point.x && point.x >= endPt.x) || (startPt.x <= point.x && point.x <= endPt.x) || delX < three) {
                    if ((startPt.y >= point.y && point.y >= endPt.y) || (startPt.y <= point.y && point.y <= endPt.y) || delY < three) {
                        return Math.abs(lhs - rhs);
                    }
                    else {
                        return 10000;
                    }
                }
                else {
                    return 10000;
                }
            }
            else {
                return 10000;
            }
        },

        getPoints: function (connector) {
            var points = [];
            // points.push(connector.sourcePoint);
            if (connector.segments.length > 0) {
                for (var i = 0; i < connector.segments.length; i++) {
                    var segment = connector.segments[i];
                    if (segment.type === "straight" || segment.type === "bezier") {
                        if (i === 0)
                            points.push(connector.sourcePoint);
                        points.push(segment._point);
                    }
                    else if (segment.type === "orthogonal") {
                        for (var j = 0; j < connector.segments[i].points.length; j++) {
                            points.push(connector.segments[i].points[j]);
                        }
                    }
                }
            }
            return points;
        },

        interSect: function (points1, points2, self, diagram, zOrder) {
            if (self && points2.length >= 2) {
                points2.splice(0, 1);
                points2.splice(0, 1);
            }
            var points = [];
            for (var i = 0; i < points1.length - 1; i++) {
                var pt = this.interSect1(points1[i], points1[i + 1], points2, diagram, zOrder);
                if (pt.length > 0) {
                    for (var k = 0; k < pt.length; k++) {
                        points.push(pt[k]);
                    }
                }
                if (self && points2.length >= 1) {
                    points2.splice(0, 1);
                }
            }
            return points;
        },

        checkforHorizontalLine: function (angle) {
            var temp = 0;
            var ang = Math.abs(angle);
            if (ang > 90) {
                temp = 180 - ang;
            }
            else {
                temp = ang;
            }
            return temp;
        },

        interSect1: function (startPoint, endPoint, points, diagram, zOrder) {
            var points1 = [];
            for (var i = 0; i < points.length - 1; i++) {
                var point = this.interSect2(startPoint, endPoint, points[i], points[i + 1]);
                if (!ej.datavisualization.Diagram.Geometry.isEmptyPoint(point)) {
                    var angle = this._findAngle(startPoint, endPoint);
                    var angle1 = this._findAngle(points[i], points[i + 1]);
                    angle = this.checkforHorizontalLine(angle);
                    angle1 = this.checkforHorizontalLine(angle1);
                    if (diagram.model.bridgeDirection === "left" || diagram.model.bridgeDirection === "right") {
                        if (angle > angle1) {
                            points1.push(point);
                        }
                    }
                    else if (diagram.model.bridgeDirection === "top" || diagram.model.bridgeDirection === "bottom") {
                        if (angle < angle1) {
                            points1.push(point);
                        }
                    }
                    if (angle === angle1 && zOrder) {
                        points1.push(point);
                    }
                }
            }
            return points1;
        },

        interSect2: function (start1, end1, start2, end2) {
            var lineUtil1 = ej.datavisualization.Diagram.LineUtil(start1.x, start1.y, end1.x, end1.y);
            var lineUtil2 = ej.datavisualization.Diagram.LineUtil(start2.x, start2.y, end2.x, end2.y);
            var line3 = this.interSect3(lineUtil1, lineUtil2);
            if (line3.enabled) {
                return line3.intersectPt;
            }
            else {
                return ej.datavisualization.Diagram.Point(0, 0);
            }
        },

        interSect3: function (lineUtil1, lineUtil2) {
            var point = ej.datavisualization.Diagram.Point(0, 0);
            var l1 = lineUtil1;
            var l2 = lineUtil2;
            var d = (l2.y2 - l2.y1) * (l1.x2 - l1.x1) - (l2.x2 - l2.x1) * (l1.y2 - l1.y1);
            var n_a = (l2.x2 - l2.x1) * (l1.y1 - l2.y1) - (l2.y2 - l2.y1) * (l1.x1 - l2.x1);
            var n_b = (l1.x2 - l1.x1) * (l1.y1 - l2.y1) - (l1.y2 - l1.y1) * (l1.x1 - l2.x1);

            if (d == 0)
                return { enabled: false, intersectPt: point };

            var ua = n_a / d;
            var ub = n_b / d;

            if (ua >= 0 && ua <= 1 && ub >= 0 && ub <= 1) {
                point.x = l1.x1 + (ua * (l1.x2 - l1.x1));
                point.y = l1.y1 + (ua * (l1.y2 - l1.y1));
                return { enabled: true, intersectPt: point };
            }
            return { enabled: false, intersectPt: point };
        },

        findLength: function (startPt, endPt) {
            try {
                var length = Math.sqrt(Math.pow((startPt.x - endPt.x), 2) + Math.pow((startPt.y - endPt.y), 2));
                return length;
            }
            catch (e) {
                throw e;
            }
        },

        _findAngle: function (startPt, endPt) {
            var xDiff = startPt.x - endPt.x; var yDiff = startPt.y - endPt.y;
            return Math.atan2(yDiff, xDiff) * (180 / Math.PI);
        },

        findAngle: function (startPt, endPt) {
            if (ej.datavisualization.Diagram.Geometry.isEqualPoint(startPt, endPt))
                return 0;
            var r = ej.datavisualization.Diagram.Point(endPt.x, startPt.y);
            var sr = this.findLength(startPt, r);
            var re = this.findLength(r, endPt);
            var es = this.findLength(endPt, startPt);
            var ang = Math.asin(re / es);
            ang = ang * 180 / Math.PI;
            if (startPt.x < endPt.x) {
                if (startPt.y < endPt.y) {

                }
                else {
                    ang = 360 - ang;
                }
            }
            else {
                if (startPt.y < endPt.y) {
                    ang = 180 - ang;
                }
                else {
                    ang = 180 + ang;
                }
            }
            return ang;
        }
    };

    ej.datavisualization.Diagram.BridgeSegments = {
        bridges: [],
        bridgeStartPoint: [],
        segmentIndex: -1
    };

    ej.datavisualization.Diagram.PathSegment = {
        path: "",
        startPoint: { "x": 0, "y": 0 },
        endPoint: { "x": 0, "y": 0 },
        angle: 0,
        segmentPointIndex: -1,
        sweep: 1
    };

    ej.datavisualization.Diagram.LineUtil = function (x1, y1, x2, y2) {
        return { "x1": Number(x1) || 0, "y1": Number(y1) || 0, "x2": Number(x2) || 0, "y2": Number(y2) || 0 };
    };
    //#endregion

    //#region Geometry
    ej.datavisualization.Diagram.Geometry = {
        checkPointOnCircle: function (point, center, radius) {
            var r2 = Math.pow(radius, 2);
            var x2 = Math.pow((point.x - center.x), 2);
            var y2 = Math.pow((point.y - center.y), 2);
            if ((x2 + y2) <= r2) {
                return true;
            }
            return false;
        },
        rect: function (points) {
            var rectangle = ej.datavisualization.Diagram.Rectangle();
            if (arguments.length === 1) {
                var left = Number.MAX_VALUE;
                var top = Number.MAX_VALUE;
                var right = Number.MIN_VALUE;
                var bottom = Number.MIN_VALUE;
                var len = points.length;
                var x, y;
                for (var i = 0; i < len; i++) {
                    x = points[i].x;
                    y = points[i].y;
                    if (x < left) {
                        left = x;
                    }
                    if (x > right) {
                        right = x;
                    }
                    if (y < top) {
                        top = y;
                    }
                    if (y > bottom) {
                        bottom = y;
                    }
                }
                rectangle = ej.datavisualization.Diagram.Rectangle(left, top, right - left, bottom - top);
            }
            else if (arguments.length === 2) {
                var ptStart = ej.datavisualization.Diagram.Point(arguments[0]);
                var ptEnd = ej.datavisualization.Diagram.Point(arguments[1]);
                var swap;
                if (ptStart.x > ptEnd.x) {
                    swap = ptStart.x;
                    ptStart.x = ptEnd.x;
                    ptEnd.x = swap;
                }
                if (ptStart.y > ptEnd.y) {
                    swap = ptStart.y;
                    ptStart.y = ptEnd.y;
                    ptEnd.y = swap;
                }
                rectangle = ej.datavisualization.Diagram.Rectangle(ptStart.x, ptStart.y, ptEnd.x - ptStart.x, ptEnd.y - ptStart.y);
            }
            return rectangle;
        },
        union: function (rect1, rect2) {
            var x = Math.min(rect1.x, rect2.x);
            var y = Math.min(rect1.y, rect2.y);
            var width = Math.max(rect1.x + rect1.width, rect2.x + rect2.width);
            var height = Math.max(rect1.y + rect1.height, rect2.y + rect2.height);
            return ej.datavisualization.Diagram.Rectangle(x, y, width - x, height - y);
        },
        distance: function (pt1, pt2) {
            return Math.sqrt(Math.pow(pt2.x - pt1.x, 2) + Math.pow(pt2.y - pt1.y, 2));
        },
        translate: function (pt1, dx, dy) {
            return ej.datavisualization.Diagram.Point(pt1.x + dx, pt1.y + dy);
        },
        intersectsRect: function (rect1, rect2) {
            return ((((rect2.x < (rect1.x + rect1.width)) && (rect1.x < (rect2.x + rect2.width)))
                && (rect2.y < (rect1.y + rect1.height))) && (rect1.y < (rect2.y + rect2.height)));
        },
        findAngle: function (point1, point2) {
            var angle = Math.atan2(point2.y - point1.y, point2.x - point1.x);
            angle = (180 * angle / Math.PI);
            angle %= 360;
            if (angle < 0) {
                angle += 360;
            }
            return angle;
        },
        intersectSegment: function (segment1, segment2) {
            var x1 = segment1.point1.x;
            var y1 = segment1.point1.y;
            var x2 = segment1.point2.x;
            var y2 = segment1.point2.y;
            var x3 = segment2.point1.x;
            var y3 = segment2.point1.y;
            var x4 = segment2.point2.x;
            var y4 = segment2.point2.y;
            var a1, a2, b1, b2, c1, c2, x, y;
            var r1, r2, r3, r4;
            var denom, offset, num;
            a1 = y2 - y1;
            b1 = x1 - x2;
            c1 = (x2 * y1) - (x1 * y2);
            r3 = ((a1 * x3) + (b1 * y3) + c1);
            r4 = ((a1 * x4) + (b1 * y4) + c1);
            if ((r3 !== 0) && (r4 !== 0) && this.sameSign(r3, r4)) {
                return null;
            }
            a2 = y4 - y3;
            b2 = x3 - x4;
            c2 = (x4 * y3) - (x3 * y4);
            r1 = (a2 * x1) + (b2 * y1) + c2;
            r2 = (a2 * x2) + (b2 * y2) + c2;
            if ((r1 !== 0) && (r2 !== 0) && (this.sameSign(r1, r2))) {
                return null;
            }
            denom = (a1 * b2) - (a2 * b1);

            if (denom === 0) {
                return null;
            }

            if (denom < 0) {
                offset = -denom / 2;
            }
            else {
                offset = denom / 2;
            }
            offset = 0;
            num = (b1 * c2) - (b2 * c1);
            if (num < 0) {
                x = (num - offset) / denom;
            }
            else {
                x = (num + offset) / denom;
            }

            num = (a2 * c1) - (a1 * c2);
            if (num < 0) {
                y = (num - offset) / denom;
            }
            else {
                y = (num + offset) / denom;
            }
            return ej.datavisualization.Diagram.Point(x, y);
        },

        updatePath: function (x, y, width, height, pathData, svg, bounds) {
            var bBox, isInit;
            if (svg && svg.pathBounds) {
                isInit = true;
                bBox = svg.pathBounds(pathData);
            }
            else {
                isInit = false;
                bBox = bounds;
            }

            var newX = 0, newY = 0;
            var isResize = false;
            if (x.toString() !== bBox.x || y.toString() !== bBox.y) {
                newX = x - Number(bBox.x);
                newY = y - Number(bBox.y);
            }
            if (width !== bBox.width || height !== bBox.height) {
                newX = width / Number(bBox.width);
                newY = height / Number(bBox.height);
                isResize = true;
            }
            return ej.datavisualization.Diagram.Geometry.processPathData(pathData, newX, newY, isResize, bBox.x, bBox.y, x, y, isInit, svg);
        },

        processPathData: function (path, newX, newY, isResizing, oldX, oldY, offsetX, offsetY, isInit, svg) {
            var shape = document.createElementNS('http://www.w3.org/2000/svg', "path");
            shape.setAttribute("d", path);
            if (isInit && svg) {
                path = svg.absolutePath(shape).getAttribute('d');
                shape.setAttribute("d", path);
            }
            var x0, y0, x1, y1, x2, y2, segs = shape.pathSegList;
            for (var x = 0, y = 0, i = 0, length = segs.numberOfItems; i < length; ++i) {
                var seg = segs.getItem(i), char = seg.pathSegTypeAsLetter;

                if ('x1' in seg) x1 = seg.x1;
                if ('x2' in seg) x2 = seg.x2;
                if ('y1' in seg) y1 = seg.y1;
                if ('y2' in seg) y2 = seg.y2;
                if ('x' in seg) x = seg.x;
                if ('y' in seg) y = seg.y;


                if (isResizing) {
                    if (x != undefined) {
                        x = ej.datavisualization.Diagram.Geometry.scalePathData(x, newX, oldX, offsetX);
                    }
                    if (y != undefined) {
                        y = ej.datavisualization.Diagram.Geometry.scalePathData(y, newY, oldY, offsetY);
                    }
                    if (x1 != undefined) {
                        x1 = ej.datavisualization.Diagram.Geometry.scalePathData(x1, newX, oldX, offsetX);
                    }
                    if (y1 != undefined) {
                        y1 = ej.datavisualization.Diagram.Geometry.scalePathData(y1, newY, oldY, offsetY);
                    }
                    if (x2 != undefined) {
                        x2 = ej.datavisualization.Diagram.Geometry.scalePathData(x2, newX, oldX, offsetX);
                    }
                    if (y2 != undefined) {
                        y2 = ej.datavisualization.Diagram.Geometry.scalePathData(y2, newY, oldY, offsetY);
                    }
                }

                else {
                    if (x != undefined) {
                        x = (x + newX);
                    }
                    if (y != undefined) {
                        y = (y + newY);
                    }
                    if (x1 != undefined) {
                        x1 = (x1 + newX);
                    }
                    if (y1 != undefined) {
                        y1 = (y1 + newY);
                    }
                    if (x2 != undefined) {
                        x2 = (x2 + newX);
                    }
                    if (y2 != undefined) {
                        y2 = (y2 + newY);
                    }
                }

                switch (char) {
                    case 'M':
                        segs.replaceItem(shape.createSVGPathSegMovetoAbs(x, y), i);
                        break;
                    case 'L':
                        segs.replaceItem(shape.createSVGPathSegLinetoAbs(x, y), i);
                        break;
                    case 'H':
                        segs.replaceItem(shape.createSVGPathSegLinetoHorizontalAbs(x), i);
                        break;
                    case 'V':
                        segs.replaceItem(shape.createSVGPathSegLinetoVerticalAbs(y), i);
                        break;
                    case 'C':
                        segs.replaceItem(shape.createSVGPathSegCurvetoCubicAbs(x, y, x1, y1, x2, y2), i);
                        break;
                    case 'S':
                        segs.replaceItem(shape.createSVGPathSegCurvetoCubicSmoothAbs(x, y, x2, y2), i);
                        break;
                    case 'Q':
                        segs.replaceItem(shape.createSVGPathSegCurvetoQuadraticAbs(x, y, x1, y1), i);
                        break;
                    case 'T':
                        segs.replaceItem(shape.createSVGPathSegCurvetoQuadraticSmoothAbs(x, y), i);
                        break;
                    case 'A':
                        var r1 = seg.r1, r2 = seg.r2;
                        if (isResizing) {
                            r1 = (r1 * newX);
                            r2 = (r2 * newY);
                        }
                        segs.replaceItem(shape.createSVGPathSegArcAbs(x, y, r1, r2, seg.angle, seg.largeArcFlag, seg.sweepFlag), i);
                        break;
                    case 'Z':
                        x = x0; y = y0;
                        break;
                }
                // Record the start of a subpath
                if (char === 'M' || char === 'm') x0 = x, y0 = y;
            }
            var pathData = shape.getAttribute("d");
            return pathData;
        },

        scalePathData: function (val, scaleFactor, oldOffset, newOffset) {
            if (val !== oldOffset) {
                if (newOffset !== oldOffset) {
                    val = (((val * scaleFactor) - (Number(oldOffset) * scaleFactor - Number(oldOffset)))
                        + (newOffset - Number(oldOffset)));
                }
                else
                    val = ((Number(val) * scaleFactor) - (Number(oldOffset) * scaleFactor - Number(oldOffset)));
            }
            else {
                if (newOffset !== oldOffset) {
                    val = newOffset;
                }
            }
            return val;
        },

        updatePolygonPoints: function (node, points) {
            if (node.shape.points) {
                if (!points)
                    points = node.shape.points;
                ej.datavisualization.Diagram.Geometry.scalePoints(points, node);
                var bounds = ej.datavisualization.Diagram.Geometry.rect(node.shape.points);
                for (var i = 0, len = node.shape.points.length; i < len; ++i) {
                    var point = ej.datavisualization.Diagram.Geometry.translate(points[i], -bounds.x, -bounds.y);
                    point = ej.datavisualization.Diagram.Geometry.translate(point, 0, 0);
                    points[i] = point;
                }
            }
        },

        scalePoints: function (points, node) {
            var bounds = ej.datavisualization.Diagram.Geometry.rect(node.shape.points);
            if (bounds.width > 0 && bounds.height > 0) {
                var scaleX = node.width / bounds.width;
                var scaleY = node.height / bounds.height;
                for (var i = 0, len = node.shape.points.length; i < len; ++i) {
                    points[i].x = points[i].x * scaleX;
                    points[i].y = points[i].y * scaleY;
                }
            }
        },

        sameSign: function (a, b) {
            return ((a * b) >= 0);
        },
        containsPoint: function (rect, point) {
            return ((((rect.x <= point.x) && (point.x <= (rect.x + rect.width)))
            && (rect.y <= point.y)) && (point.y <= (rect.y + rect.height)));
        },
        containsRect: function (rect, value) {
            return ((((rect.x <= value.x) && ((value.x + value.width) <= (rect.x + rect.width)))
               && (rect.y <= value.y)) && ((value.y + value.height) <= (rect.y + rect.height)));
        },
        inflate: function (rect, x, y) {
            rect.x -= x;
            rect.y -= y;
            rect.width += 2 * x;
            rect.height += 2 * y;
        },
        isEmptyRect: function (rect) {
            return rect.x === 0 && rect.y === 0 && rect.width === 0 && rect.height === 0;
        },
        isEmptyPoint: function (point) {
            return point.x === 0 && point.y === 0;
        },
        isEqualPoint: function (point1, point2) {
            return point1.x === point2.x && point1.y === point2.y;
        },
        isEmptySize: function (size) {
            return size.width === 0 && size.height === 0;
        },
        length: function (segment) {
            return ej.datavisualization.Diagram.Geometry.distance(segment.point1, segment.point2);
        },
        midPoint: function (segment) {
            return ej.datavisualization.Diagram.Point((segment.point1.x + segment.point2.x) / 2, (segment.point1.y + segment.point2.y) / 2);
        },
        transform: function (point, angle, length) {
            var pt = {};
            pt.x = Math.round((point.x + length * Math.cos(angle * Math.PI / 180)) * 100) / 100;
            pt.y = Math.round((point.y + length * Math.sin(angle * Math.PI / 180)) * 100) / 100;
            return pt;
        }
    };
    ej.datavisualization.Diagram.LineSegment = function (point1, point2) {
        return { "point1": point1, "point2": point2 };
    };
    ej.datavisualization.Diagram.Point = function (x, y) {
        return { "x": Number(x) || 0, "y": Number(y) || 0 };
    };
    ej.datavisualization.Diagram.Size = function (width, height) {
        return { "width": Number(width) || 0, "height": Number(height) || 0 };
    };
    ej.datavisualization.Diagram.Rectangle = function (x, y, width, height) {
        return { "x": Number(x) || 0, "y": Number(y) || 0, "width": Number(width) || 0, "height": Number(height) || 0 };
    };
    //#endregion

    //#region Enums
    /**
    * Enum for various decorator shapes in diagram	 
    * @enum {string}
    * @global 
    */
    ej.datavisualization.Diagram.DecoratorShapes = {
        /**Used to set decorator shape as none */
        None: "none",
        /**Used to set decorator shape as Arrow*/
        Arrow: "arrow",
        /**Used to set decorator shape as Open Arrow */
        OpenArrow: "openarrow",
        /**Used to set decorator shape as Circle */
        Circle: "circle",
        /**Used to set decorator shape as Diamond */
        Diamond: "diamond",
        /**Used to set decorator shape as path */
        Path: "path"
    };

    ej.datavisualization.Diagram.dependentconnector = {
        dependent: 1,
        independent: 2
    }
    /**
    * Enum for various containerType in diagram	 
    * @enum {string}
    * @global 
    */
    ej.datavisualization.Diagram.ContainerType = {
        /**Set the container type as Canvas*/
        Canvas: "canvas",
        /**Set the container type as Stack*/
        Stack: "stack",
    };
    /**
    * Enum for various port shapes in diagram	 
    * @enum {string}
    * @global 
    */
    ej.datavisualization.Diagram.PortShapes = {
        /**Used to set port shape as X*/
        X: "x",
        /**Used to set port shape as Circle*/
        Circle: "circle",
        /**Used to set port shape as Square*/
        Square: "square",
        /**Used to set port shape as Path*/
        Path: "path"
    };
    /**
    * Enum for port visibility in diagram	 
    * @enum {PortVisibility}
    * @global 
    */
    ej.datavisualization.Diagram.PortVisibility = {
        /**Set the port visibility as Visible*/
        Visible: 1 << 0,
        /**Set the port visibility as Hidden*/
        Hidden: 1 << 1,
        /**Port get visible when hover connector on node*/
        Hover: 1 << 2,
        /** Port gets visibile when connect connector to node*/
        Connect: 1 << 3,
        /**Specifies the port visibility as default*/
        Default: 1 << 3
    };
    /**
    * Enum for the Text Alignment in diagram	 
    * @enum {string}
    * @global 
    */
    ej.datavisualization.Diagram.TextAlign = {
        /** Used to align text on left side of node/connector*/
        Left: "left",
        /** Used to align text on center of node/connector*/
        Center: "center",
        /** Used to align text on Right side of node/connector*/
        Right: "right"
    };
    /**
    * Enum for Horizontal Alignment of text in diagram	 
    * @enum {string}
    * @global 
    */
    ej.datavisualization.Diagram.HorizontalAlignment = {
        /** Used to align text horizontally on left side of node/connector*/
        Left: "left",
        /** Used to align text horizontally on center of node/connector*/
        Center: "center",
        /** Used to align text horizontally on right side of node/connector*/
        Right: "right"
    };
    /**
    * Enum for LabelEditMode of text in diagram	 
    * @enum {string}
    * @global 
    */
    ej.datavisualization.Diagram.LabelEditMode = {
        /** Used to set label edit mode as edit*/
        Edit: "edit",
        /** Used to set label edit mode as view*/
        View: "view"
    };
    /**
    * Enum for Vertical Alignment of text in diagram	 
    * @enum {string}
    * @global 
    */
    ej.datavisualization.Diagram.VerticalAlignment = {
        /** Used to align text Vertically on left side of node/connector*/
        Top: "top",
        /** Used to align text Vertically on center of node/connector*/
        Center: "center",
        /** Used to align text Vertically on bottom of node/connector*/
        Bottom: "bottom"
    };
    /**
    * Enum for TextDecorations in diagram	 
    * @enum {string}
    * @global 
    */
    ej.datavisualization.Diagram.TextDecorations = {
        /** Used to set text decoration of the label as Underline*/
        Underline: "underline",
        /** Used to set text decoration of the label as Overline*/
        Overline: "overline",
        /** Used to set text decoration of the label as LineThrough*/
        LineThrough: "line-through",
        /** Used to set text decoration of the label as None*/
        None: "none"
    };
    /**
    * Enum for the NodeConstraints in diagram	 
    * @enum {NodeConstraints}
    * @global 
    */
    ej.datavisualization.Diagram.NodeConstraints = {
        /**Disable all node Constraints */
        None: 1 << 0,
        /**Enables node to be selected*/
        Select: 1 << 1,
        /**Enables node to be Deleted*/
        Delete: 1 << 2,
        /**Enables node to be Resized */
        Resize: 1 << 3,
        /**Enables node to be Dragged*/
        Drag: 1 << 4,
        /**Enables node to be Rotated*/
        Rotate: 1 << 5,
        /**Enables node to be connected*/
        Connect: 1 << 6,
        /**Enables node to be resize north east*/
        ResizeNorthEast: 1 << 7,
        /**Enables node to be resize east*/
        ResizeEast: 1 << 8,
        /**Enables node to be resize south east*/
        ResizeSouthEast: 1 << 9,
        /**Enables node to be resize south*/
        ResizeSouth: 1 << 10,
        /**Enables node to be resize south west*/
        ResizeSouthWest: 1 << 11,
        /**Enables node to be resize west*/
        ResizeWest: 1 << 12,
        /**Enables node to be resize north west*/
        ResizeNorthWest: 1 << 13,
        /**Enables node to be resize north */
        ResizeNorth: 1 << 14,
        /**Enables shadow*/
        Shadow: 1 << 15,
        /**Enables label of node to be Dragged*/
        DragLabel: 1 << 16,
        /**Enables panning should be done while node dragging */
        AllowPan: 1 << 17,
        /**Enables Proportional resize for node */
        AspectRatio: 1 << 18,
        /**Enables all node constraints*/
        Default: 1 << 1 | 1 << 2 | 1 << 3 | 1 << 4 | 1 << 5 | 1 << 6 | 1 << 7 | 1 << 8 | 1 << 9 | 1 << 10 | 1 << 11 | 1 << 12 | 1 << 13 | 1 << 14
    };
    /**
    * Enum for ConnectorConstraints in diagram	 
    * @enum {ConnectorConstraints}
    * @global 
    */
    ej.datavisualization.Diagram.ConnectorConstraints = {
        /**Disable all connector Constraints */
        None: 1 << 0,
        /**Enables connector to be selected*/
        Select: 1 << 1,
        /**Enables connector to be Deleted*/
        Delete: 1 << 2,
        /**Enables connector to be Dragged*/
        Drag: 1 << 3,
        /**Enables connectors source end to be selected*/
        DragSourceEnd: 1 << 4,
        /**Enables connectors target end to be selected*/
        DragTargetEnd: 1 << 5,
        /**Enables control point and end point of every segment in a connector for editing*/
        DragSegmentThumb: 1 << 6,
        /** Enables bridging to the connector */
        Bridging: 1 << 7,
        /**Enables label of node to be Dragged*/
        DragLabel: 1 << 8,
        /** Enables bridging to the connector */
        InheritBridging: 1 << 9,
        /**Enables all constraints*/
        Default: 1 << 1 | 1 << 2 | 1 << 3 | 1 << 4 | 1 << 5 | 1 << 6 | 1 << 9
    };

    /**
    * Enum for PortConstraints in diagram	 
    * @enum {PortConstraints}
    * @global 
    */
    ej.datavisualization.Diagram.PortConstraints = {
        /**Disable all constraints*/
        None: 1 << 0,
        /**Enables connections with connector*/
        Connect: 1 << 1
    };
    /**
    * Enum for specify the SelectorConstraints in diagram	 
    * @enum {SelectorConstraints}
    * @global 
    */
    ej.datavisualization.Diagram.SelectorConstraints = {
        /** Disables all constraints*/
        None: 1 << 0,
        /** Enables the rotation handle when select node*/
        Rotator: 1 << 1,
        /**Enables the resize when select node*/
        Resizer: 1 << 2,
        /** Enables the user handles when select node*/
        UserHandles: 1 << 3,
        /** Enables all the constraints*/
        All: 1 << 1 | 1 << 2 | 1 << 3
    };
    /**
    * Enum for Position in diagram	 
    * @enum {String}
    * @global 
    */
    ej.datavisualization.Diagram.UserHandlePositions = {
        /**Used to set position as TopLeft*/
        TopLeft: "topleft",
        /**Used to set position as TopCenter*/
        TopCenter: "topcenter",
        /**Used to set position as TopRight*/
        TopRight: "topright",
        /**Used to set position as MiddleLeft*/
        MiddleLeft: "middleleft",
        /**Used to set position as  MiddleRight*/
        MiddleRight: "middleright",
        /**Used to set position as BottomLeft*/
        BottomLeft: "bottomleft",
        /**Used to set position as BottomCenter*/
        BottomCenter: "bottomcenter",
        /**Used to set position as BottomRight*/
        BottomRight: "bottomright"
    };
    /**
    * Enum for SnapConstraints in diagram	 
    * @enum {SnapConstraints}
    * @global 
    */
    ej.datavisualization.Diagram.SnapConstraints = {
        /**Enables node to be snapped to horizontal gridlines*/
        None: 0,
        /**Enables node to be snapped to vertical gridlines*/
        SnapToHorizontalLines: 1,
        /**Enables node to be snapped to horizontal gridlines*/
        SnapToVerticalLines: 2,
        /**Enables node to be snapped to gridlines*/
        SnapToLines: 1 | 2,
        /**Enable horizontal lines*/
        ShowHorizontalLines: 4,
        /**Enable vertical lines*/
        ShowVerticalLines: 8,
        /**Enable both horizontal and vertical lines*/
        ShowLines: 4 | 8,
        /**Enable all the constraints*/
        All: 1 | 2 | 4 | 8
    };
    /**
    * Enum for PageOrientations in diagram	 
    * @enum {String}
    * @global 
    */
    ej.datavisualization.Diagram.PageOrientations = {
        /** Used to set orientation as Landscape*/
        Landscape: "landscape",
        /** Used to set orientation as portrait*/
        Portrait: "portrait"
    };

    /**
    * Enum for ScrollLimit in diagram	 
    * @enum {String}
    * @global 
    */
    ej.datavisualization.Diagram.ScrollLimit = {
        /** Used to set scrollLimit as Infinity*/
        Infinity: "infinity",
        /** Used to set scrollLimit as Diagram*/
        Diagram: "diagram",
        /** Used to set scrollLimit as Limited*/
        Limited: "limited"
    };

    /**
    * Enum for LayoutOrientations in diagram	
    * @enum {String}
    * @global 
    */
    ej.datavisualization.Diagram.LayoutOrientations = {
        /** Used to set LayoutOrientation from top to bottom */
        TopToBottom: "toptobottom",
        /** Used to set LayoutOrientation from bottom to top */
        BottomToTop: "bottomtotop",
        /** Used to set LayoutOrientation from left to right */
        LeftToRight: "lefttoright",
        /** Used to set LayoutOrientation from right to left */
        RightToLeft: "righttoleft"
    };
    //#endregion

    //#region Palette
    ej.datavisualization.Diagram.PaletteDefaults = {
        /** specifies the name of the palette
     * @default ""
     * @type {String}
     * @example
     * &lt;div id="symbolpalette"&gt;&lt;/div&gt;<br>
     * &lt;script&gt;
      var palette = [{name: "Basic Shapes"}];
     *  $("#symbolpalette").ejSymbolPalette({palettes:palette});
     * &lt;/script&gt
     * @memberof ejSymbolpalette
     * @instance
     */
        name: "",
        /** Enable or disable the expand property of palette
      * @default false
      * @type {Boolean}
      * @example
      * &lt;div id="symbolpalette"&gt;&lt;/div&gt;<br>
      * &lt;script&gt;
       var palette = [{expanded: true}];
      *  $("#symbolpalette").ejSymbolPalette({palettes:palette});
      * &lt;/script&gt
      * @memberof ejSymbolpalette
      * @instance
      */
        expanded: false,
        subControlId: null,
        /** Collection of items in plaette
    * @default []
    * @type {Array}
    * @example
    * &lt;div id="symbolpalette"&gt;&lt;/div&gt;<br>
    * &lt;script&gt;
     var palette = [{ items:[{ name: "Triangle"},{ name: "Ellipse"}]}];
    *  $("#symbolpalette").ejSymbolPalette({palettes:palette});
    * &lt;/script&gt
    * @memberof ejSymbolpalette
    * @instance
    */
        items: []
    };
    ej.datavisualization.Diagram.Palette = function (options, palette) {
        if (options.items)
            for (var i = 0; i < options.items.length; i++) {
                if (options.items[i].isLane) {
                    options.items[i] = ej.datavisualization.Diagram.SwimLaneHelper._createPaletteLane(options.items[i], palette.nameTable);
                }
                else if (options.items[i].segments)
                    options.items[i] = ej.datavisualization.Diagram.Connector(options.items[i]);
                else if (options.items[i].isGroup || options.items[i].type == "group")
                    options.items[i] = ej.datavisualization.Diagram.Group(options.items[i]);
                else
                    options.items[i] = ej.datavisualization.Diagram.Node(options.items[i]);
            }
        return $.extend(true, {}, ej.datavisualization.Diagram.PaletteDefaults, options);
    };
    //#endregion

    //#region Shapes
    /**
    * Enum for various shapes of node in diagram	 
    * @enum {string}
    * @global 
    */
    ej.datavisualization.Diagram.Shapes = {
        /**Used to specify node Shape as Rectangle*/
        Rectangle: "rectangle",
        /**Used to specify node Shape as Ellipse*/
        Ellipse: "ellipse",
        /**Used to specify node Shape as Image*/
        Image: "image",
        /**Used to specify node Shape as Text*/
        Text: "text",
        /**Used to specify node Shape as Path*/
        Path: "path",
        /**Used to specify node Shape as Polygon*/
        Polygon: "polygon",
        /**Used to specify node Shape as Html*/
        Html: "html",
        /**Used to specify node Shape as Native*/
        Native: "native"
    };


    ej.datavisualization.Diagram.ShapeDefaults = {
        /** The type of node See {@link Shapes}
         
         * @default ej.datavisualization.Diagram.Shapes.Rectangle
         * @type {enum}
         * @example
         * &lt;div id="diagramcontent"&gt;&lt;/div&gt;<br>
         * &lt;script&gt;
         * var node=[];
         * node={[shape:{type:"rectangle"}]};
         * $("#diagramcontent").ejDiagram({nodes:node});
         * &lt;/script&gt
        * @alias ejDiagram#nodes->shape->type
         * @memberof ejDiagram
         * @instance
         */
        type: ej.datavisualization.Diagram.Shapes.Rectangle,
        /** Defines the shape of the corner
         * @default 0
         * @type {Integer}
         * @example
         * &lt;div id="diagramcontent"&gt;&lt;/div&gt;<br>
         * &lt;script&gt;
         * var node=[];
         * node={[shape:{type:"rectangle",cornerRadius:2}]};
         * $("#diagramcontent").ejDiagram({nodes:node});
         * &lt;/script&gt
       * @alias ejDiagram#nodes->shape->cornerRadius
         * @memberof ejDiagram
         * @instance
         */
        cornerRadius: 0,
        /** The src location for the image node
      * @default ""
      * @type {String}
      * @example
      * &lt;div id="diagramcontent"&gt;&lt;/div&gt;<br>
      * &lt;script&gt;
      * var node=[];
      * node={[shape:{type:"image",src:"Syncfusion.png"}]};
      * $("#diagramcontent").ejDiagram({nodes:node});
      * &lt;/script&gt
     * @alias ejDiagram#nodes->shape->src
      * @memberof ejDiagram
      * @instance
      */
        src: "",
        /** Set the geometry that specifies the shapes to be drawn
       * @default ""
       * @type {String}
       * @example
       * &lt;div id="diagramcontent"&gt;&lt;/div&gt;<br>
       * &lt;script&gt;
       * var node=[];
       * node={[shape:{type:"path",pathData:"M 269.711,29.3333C 269.711,44.061 257.772,56 243.044,56L 158.058,56C 143.33z"}]};
       * $("#diagramcontent").ejDiagram({nodes:node});
       * &lt;/script&gt
     * @alias ejDiagram#nodes->shape->pathData
       * @memberof ejDiagram
       * @instance
       */
        pathData: "",
        /**This is used to display text
      * @default ""
      * @type {Integer}
      * @example
      * &lt;div id="diagramcontent"&gt;&lt;/div&gt;<br>
      * &lt;script&gt;
      * var node=[];
      * var textBlock = ej.datavisualization.Diagram.TextBlockDefaults;
      * textBlock.text = "TextNode";
      * node={[shape: { type: "text", textBlock: textBlock }]};
      * $("#diagramcontent").ejDiagram({nodes:node});
      * &lt;/script&gt
     * @alias ejDiagram#nodes->shape->textBlock
      * @memberof ejDiagram
      * @instance
      */
        textBlock: null,
        /** Points to define X and Y coordinates for the polygon
       * @default []
       * @type {Array}
       * @example
       * &lt;div id="diagramcontent"&gt;&lt;/div&gt;<br>
       * &lt;script&gt;
       * var node=[];
       * node={[shape: { type: "polygon", points:[{ x: 0, y: 12.5 }, { x: 0, y: 50 }, { x: 50, y: 50 }, { x: 50, y: 0 }, { x: 12.5, y: 0 }, { x: 0, y: 12.5 }]}]};
       * $("#diagramcontent").ejDiagram({nodes:node});
       * &lt;/script&gt
    * @alias ejDiagram#nodes->shape->points
       * @memberof ejDiagram
       * @instance
       */
        points: [],
        /** Content of the html element is to be updated
    * @default ""
    * @type {String}
    * @example
    * &lt;div id="diagramcontent"&gt;&lt;/div&gt;<br>
    * &lt;script&gt;
    * var node=[];
    * node={[shape: { type: "html", html:"Html Node"}]};
    * $("#diagramcontent").ejDiagram({nodes:node});
    * &lt;/script&gt
    * @alias ejDiagram#nodes->shape->html
    * @memberof ejDiagram
    * @instance
    */
        html: "",
        /** Content for the native node to be updated
     * @default null
     * @type {Object}
     * @example
     * &lt;div id="diagramcontent"&gt;&lt;/div&gt;<br>
     * &lt;script&gt;
     * var node=[];
     * var g = document.getElementById("CompensationInterruptingGateWay");
     * node={[shape: { type: "native", content:g}]};
     * $("#diagramcontent").ejDiagram({nodes:node});
     * &lt;/script&gt
     * @alias ejDiagram#nodes->shape->content
     * @memberof ejDiagram
     * @instance
     */
        content: null
    };
    ej.datavisualization.Diagram.Shape = function (options) {
        if (options) {
            if (options.type === "html") {
                if (typeof (options.html) === "string")
                    options.html = ej.datavisualization.Diagram.Util.htmlEncode(options.html);
            }
            if (options.type === "text" && options.textBlock)
                options.textBlock = ej.datavisualization.Diagram.TextBlock(options.textBlock);
        }
        return $.extend(true, {}, ej.datavisualization.Diagram.ShapeDefaults, options);
    };
    //#endregion

    //#region Lines
    /**
    * Enum for various segments of connector in diagram	 
    * @enum {string}
    * @global 
    */
    ej.datavisualization.Diagram.Segments = {
        /** Used to specify the lines as Straight*/
        Straight: "straight",
        /** Used to specify the lines as Orthogonal*/
        Orthogonal: "orthogonal",
        /** Used to specify the lines as Bezier*/
        Bezier: "bezier"
    };


    ej.datavisualization.Diagram.SegmentDefaults = {
        /**Type of the segments See {@link Segments}         
                 * @default ej.datavisualization.Diagram.Segments.Straight
                 * @type {enum}
                 * @example
                 * &lt;div id="diagramcontent"&gt;&lt;/div&gt;<br>
                 * &lt;script&gt;
                 * var link=[];
                 * link=[{segments:[{type:"straight"}]}];
                 * $("#diagramcontent").ejDiagram({connectors:link});
                 * &lt;/script&gt
                 * @alias  ejDiagram#connectors->segments->type
                 * @memberof ejDiagram
                 * @instance
                 */
        type: ej.datavisualization.Diagram.Segments.Straight,
     
        /** Describes the end point of segment
      * @default Diagram.Point()
      * @type {Point}
      * @example
      * &lt;div id="diagramcontent"&gt;&lt;/div&gt;<br>
      * &lt;script&gt;
      * var segments = [{ type: "straight",point:{x:10,y:30}}];
      * connectors = [{ name: "connector1", segments: segments, sourcePoint: { x: 450, y: 150 }, targetPoint: { x: 210, y: 40 }}];
      * $("#diagramcontent").ejDiagram({segments:segments});
      * &lt;/script&gt
      * @alias  ejDiagram#connectors->segments->point
      * @memberof ejDiagram
      * @instance
      */
        point: null,
        /** Describes the first control point of bezier segment
       * @default null
       * @type {Point}
       * @example
       * &lt;div id="diagramcontent"&gt;&lt;/div&gt;<br>
       * &lt;script&gt;
       * var connectors = [];
       * var point1={x:10,y:10}];
       * connectors=[{segments: [{type:"bezier", point1:point1}]}];
       * $("#diagramcontent").ejDiagram({connectors:connectors});
       * &lt;/script&gt
       * @alias ejDiagram#connectors->segments->point1
       * @memberof ejDiagram
       * @instance
       */
        point1: null,
        /** Describes the second control point of bezier segment
           * @default null
           * @type {Point}
           * @example
           * &lt;div id="diagramcontent"&gt;&lt;/div&gt;<br>
           * &lt;script&gt;
           * var connectors = [];
           * var point2= {x:10,y:10};
           * connectors=[{segments: [{type:"bezier",point2:point2}]}];
           * $("#diagramcontent").ejDiagram({connectors:connectors});
           * &lt;/script&gt
           * @alias ejDiagram#connectors->segments->point2
           * @memberof ejDiagram
           * @instance
           */
        point2: null,
        /** Describes the length and angle between the first control point and start point of segment
         * @default null
         * @type {Point}
         * @example
         * &lt;div id="diagramcontent"&gt;&lt;/div&gt;<br>
         * &lt;script&gt;
         * var connectors = [];
         * var vector1= {x:20,y:20};
         * connectors=[{segments: [{type:"bezier",vector1:vector1}]}];
         * $("#diagramcontent").ejDiagram({connectors:connectors});
         * &lt;/script&gt
         * @alias ejDiagram#connectors->segments->vector1
         * @memberof ejDiagram
         * @instance
            */
        vector1: null,
        /** Describes the length and angle between the second control point and end point of segment
        * @default null
        * @type {Point}
        * @example
        * &lt;div id="diagramcontent"&gt;&lt;/div&gt;<br>
        * &lt;script&gt;
        * var connectors = [];
        * var vector2= {x:20,y:20};
        * connectors=[{segments: [{type:"bezier",vector2:vector2}]}];
        * $("#diagramcontent").ejDiagram({connectors:connectors});
        * &lt;/script&gt
        * @alias ejDiagram#connectors->segments->vector2
        * @memberof ejDiagram
        * @instance
           */
        vector2: null,
        _point1: ej.datavisualization.Diagram.Point(),
        _point2: ej.datavisualization.Diagram.Point(),
            /** Describes the length of orthogonal segment
    * @default null
    * @type {Number}
    * @example
    * &lt;div id="diagramcontent"&gt;&lt;/div&gt;<br>
    * &lt;script&gt;
    * var connectors = [];
    * connectors=[{segments: [{type:"orthogonal",length:20}]}];
    * $("#diagramcontent").ejDiagram({connectors:connectors});
    * &lt;/script&gt
    * @alias ejDiagram#connectors->segments->length
    * @memberof ejDiagram
    * @instance
    */

        length: null, _length: null,
        _bridges: [],
        /** Describes the  direction of orthogonal segment
   * @default null
   * @type {String}
   * @example
   * &lt;div id="diagramcontent"&gt;&lt;/div&gt;<br>
   * &lt;script&gt;
   * var connectors = [];
   * connectors=[{segments: [{type:"orthogonal",direction:"left"}]}];
   * $("#diagramcontent").ejDiagram({connectors:connectors});
   * &lt;/script&gt
   * @alias ejDiagram#connectors->segments->direction
   * @memberof ejDiagram
   * @instance
    */
        direction: null, _direction: null,
    };
    ej.datavisualization.Diagram.Segment = function (options) {
        var line = $.extend(true, {}, ej.datavisualization.Diagram.SegmentDefaults, options);
        var points = [];
        line["points"] = points;
        if (line.type == "orthogonal") {
            line._length = line.length;
            line._direction = line.direction;
        }
        else {
            if (line.type == "bezier") {
                line._point1 = line.point1;
                line._point2 = line.point2;
            }
            line._point = line.point;
        }
        return line;
    };
    //#endregion


    //#region Margin
    /** This clears area around the label in diagram 
    * @alias ejDiagram.Label#margin
    */

    ej.datavisualization.Diagram.MarginDefaults = {
        /**		
                  * This allows to set the top margin for the label
                  * @default 0
                  * @type {Number}
                  * @example 
                  * &lt;div id="diagramcontent"&gt;&lt;/div&gt;<br>
                  * &lt;script&gt;
                  * var node = [];
                  *node={[labels: [{"margin": new ej.datavisualization.Diagram.Margin({ "top": 30})}]};
                  *$("#diagramContent").ejDiagram({nodes:node}); 
                  * &lt;/script&gt
                  * @alias  ejDiagram.Label#margin->top
                   * @memberof ejDiagram.Label
                  * @instance
                  */
        top: 0,
        /**		
                 * This allows to set the left margin for the label
                 * @default 0
                 * @type {Number}
                 * @example 
                 * &lt;div id="diagramcontent"&gt;&lt;/div&gt;<br>
                 * &lt;script&gt;
                 * var node = [];
                 *node={[labels: [{"margin": new ej.datavisualization.Diagram.Margin({ "left": 30})}]};
                 *$("#diagramContent").ejDiagram({nodes:node}); 
                 * &lt;/script&gt
                 * @alias  ejDiagram.Label#margin->left
                  * @memberof ejDiagram.Label
                 * @instance
                 */
        left: 0,
        /**		
                 * This allows to set the right margin for the label
                 * @default 0
                 * @type {Number}
                 * @example 
                 * &lt;div id="diagramcontent"&gt;&lt;/div&gt;<br>
                 * &lt;script&gt;
                 * var node = [];
                 *node={[labels: [{"margin": new ej.datavisualization.Diagram.Margin({ "right": 30})}]};
                 *$("#diagramContent").ejDiagram({nodes:node}); 
                 * &lt;/script&gt
                 * @alias  ejDiagram.Label#margin->right
                  * @memberof ejDiagram.Label
                 * @instance
                 */
        right: 0,
        /**		
                * This allows to set the bottom margin for the label 
                * @default 0
                * @type {Number}
                * @example 
                * &lt;div id="diagramcontent"&gt;&lt;/div&gt;<br>
                * &lt;script&gt;
                * var node = [];
                *node={[labels: [{"margin": new ej.datavisualization.Diagram.Margin({ "bottom": 30})}]};
                *$("#diagramContent").ejDiagram({nodes:node}); 
                * &lt;/script&gt
                * @alias  ejDiagram.Label#margin->bottom
                 * @memberof ejDiagram.Label
                * @instance
                */
        bottom: 0
    };
    ej.datavisualization.Diagram.Margin = function (options) {
        return $.extend(true, {}, ej.datavisualization.Diagram.MarginDefaults, options);
    };
    //#endregion

    //#region Path Util
    ej.datavisualization.Diagram.Path = function () {
        this._path = "";
    };
    ej.datavisualization.Diagram.Path.prototype = {
        moveTo: function (x, y) {
            this._path += "M" + x + "," + y; return this;
        },
        lineTo: function (x, y) {
            this._path += "L" + x + "," + y; return this;
        },
        cubicBezierTo: function (points, isSmooth) {
            if (isSmooth)
                this._path += "S";
            else
                this._path += "C";
            this._addPoints(points);
            return this;
        },
        quadraticBezierTo: function (points, isSmooth) {
            if (isSmooth)
                this._path += "T";
            else
                this._path += "Q";
            this._addPoints(points);
            return this;
        },
        ellipticalArcTo: function (rx, ry, xAngle, arcFlag, sweepFlag, x, y) {
            this._path += "A" + rx + "," + ry + " " + xAngle + " " + arcFlag + "," + sweepFlag + " " + x + "," + y;
            return this;
        },
        close: function () {
            this._path += "Z"; return this;
        },
        toString: function () {
            return this._path;
        },
        _addPoints: function (points) {
            var point;
            for (var i = 0; i < points.length; i++) {
                point = points[i];
                this._path += point.x + ", " + point.y;
                if (i !== (points.length - 1))
                    this._path += " ";
            }
        }
    };
    //#endregion

    ej.datavisualization.Diagram.DecoratorDefaults = {
        /** The shape of decorator in connector See {@link DecoratorShapes}
     
        * @default ej.datavisualization.Diagram.DecoratorShapes.Arrow
        * @type {enum}
        * @example
        * &lt;div id="diagramcontent"&gt;&lt;/div&gt;<br>
        * &lt;script&gt;
        * var link=[];
        * link={[targetDecorator:{shape:"arrow"}]};
        * $("#diagramcontent").ejDiagram({connectors:link});
        * &lt;/script&gt
        * @alias ejDiagram#connectors->sourceDecorator->shape
    
        * @memberof ejDiagram
        * @instance
        */

        shape: ej.datavisualization.Diagram.DecoratorShapes.Arrow,
        /** This allows to set the width of the decorator in connector
    * @default 8
    * @type {Number}
    * @example
    * &lt;div id="diagramcontent"&gt;&lt;/div&gt;<br>
    * &lt;script&gt;
    * var link=[];
    * link={[targetDecorator:{width:10}]};
    * $("#diagramcontent").ejDiagram({connectors:link});
    * &lt;/script&gt
     * @alias ejDiagram#connectors->sourceDecorator->width
     
    * @memberof ejDiagram
    * @instance
    */
        width: 8,
        /** This allows to set the height of the decorator in connector
         * @default 8
         * @type {Number}
         * @example
         * &lt;div id="diagramcontent"&gt;&lt;/div&gt;<br>
         * &lt;script&gt;
         * var link=[];
         * link={[targetDecorator:{height:10}]};
         * $("#diagramcontent").ejDiagram({connectors:link});
         * &lt;/script&gt
           * @alias ejDiagram#connectors->sourceDecorator->height
      
      * @memberof ejDiagram
         * @instance
         */
        height: 8,
        /** Sets the border color of the decorator
    * @default "black"
    * @type {String}
    * @example
    * &lt;div id="diagramcontent"&gt;&lt;/div&gt;<br>
    * &lt;script&gt;
    * var link=[];
    * link={[targetDecorator:{borderColor: "green"}]};
    * $("#diagramcontent").ejDiagram({connectors:link});
    * &lt;/script&gt
      * @alias ejDiagram#connectors->sourceDecorator->borderColor
      
     * @memberof ejDiagram
    * @instance
    */
        borderColor: "black",
        /** The color which sets inside the decorator 
    * @default "black"
    * @type {String}
    * @example
    * &lt;div id="diagramcontent"&gt;&lt;/div&gt;<br>
    * &lt;script&gt;
    * var link=[];
    * link={[targetDecorator:{fillColor: "green"}]};
    * $("#diagramcontent").ejDiagram({connectors:link});
    * &lt;/script&gt
     * @alias ejDiagram#connectors->sourceDecorator->fillColor
     
    * @memberof ejDiagram
    * @instance
    */
        fillColor: "black",
        /** Path data to draw custom shape for decorator
    * @default ""
    * @type {String}
    * @example
    * &lt;div id="diagramcontent"&gt;&lt;/div&gt;<br>
    * &lt;script&gt;
    * var link=[];
    * link={[targetDecorator:{pathData: "M 269.711,29.3333C 269.711,44.061 257.772,56 243.044,56z"}]};
    * $("#diagramcontent").ejDiagram({connectors:link});
    * &lt;/script&gt
    * @alias ejDiagram#connectors->sourceDecorator->pathData
    
    * @memberof ejDiagram
    * @instance
    */
        pathData: ""
    };
    ej.datavisualization.Diagram.Decorator = function (options) {
        return $.extend(true, {}, ej.datavisualization.Diagram.DecoratorDefaults, options);
    };
    //#endregion


    ej.datavisualization.Diagram.TextBlockDefaults = {
        /** Allow the label to be read only
     * @default false
     * @type {Boolean}
     * @example
     * &lt;div id="diagramcontent"&gt;&lt;/div&gt;<br>
     * &lt;script&gt;
     * var node=[];
     * node=[{labels:{"readOnly": false}}];
     * $("#diagramcontent").ejDiagram({nodes:node});
     * &lt;/script&gt
      * @alias ejDiagram#nodes->labels->readOnly
      
    * @memberof ejDiagram
     * @instance
     */
        readOnly: false,
        /** To displays the bold font style for label
      * @default false
      * @type {Boolean}
      * @example
      * &lt;div id="diagramcontent"&gt;&lt;/div&gt;<br>
      * &lt;script&gt;
      * var node=[];
      * node=[{labels:{"bold": false}}];
      * $("#diagramcontent").ejDiagram({nodes:node});
      * &lt;/script&gt
       * @alias ejDiagram#nodes->labels->bold
      
    * @memberof ejDiagram
      * @instance
      */
        bold: false,
        /** To displays the italic font style for label
    * @default false
    * @type {Boolean}
    * @example
    * &lt;div id="diagramcontent"&gt;&lt;/div&gt;<br>
    * &lt;script&gt;
    * var node=[];
    * node=[{labels:{"italic": false}}];
    * $("#diagramcontent").ejDiagram({nodes:node});
    * &lt;/script&gt
     * @alias ejDiagram#nodes->labels->italic
     
    * @memberof ejDiagram
    * @instance
    */
        italic: false,
        /** Set text for the label
    * @default ""
    * @type {String}
    * @example
    * &lt;div id="diagramcontent"&gt;&lt;/div&gt;<br>
    * &lt;script&gt;
    * var node=[];
    * node=[{labels:{"text": "Label"}}];
    * $("#diagramcontent").ejDiagram({nodes:node});
    * &lt;/script&gt
    * @alias ejDiagram#nodes->labels->text
     
    * @memberof ejDiagram
    * @instance
    */
        text: "",
        /**Thi allows to specifies the decoration addeed to text see{@link TextDecorations}
    
      * @default TextDecorations.None
      * @type {Enum}
      * @example
      * &lt;div id="diagramcontent"&gt;&lt;/div&gt;<br>
      * &lt;script&gt;
      * var node=[];
      * node=[{labels:{textDecoration: ej.datavisualization.Diagram.TextDecorations.Underline}}];
      * $("#diagramcontent").ejDiagram({nodes:node});
      * &lt;/script&gt
       * @alias ejDiagram#nodes->labels->textDecoration
      
    * @memberof ejDiagram
      * @instance
      */
        textDecoration: ej.datavisualization.Diagram.TextDecorations.None,
        /** To sets the size of the font of the label
    * @default 12
    * @type {Integer}
    * @example
    * &lt;div id="diagramcontent"&gt;&lt;/div&gt;<br>
    * &lt;script&gt;
    * var node=[];
    * node=[{labels:{"fontSize": 12}}];
    * $("#diagramcontent").ejDiagram({nodes:node});
    * &lt;/script&gt
    * @alias ejDiagram#nodes->labels->fontSize
    
    
    * @memberof ejDiagram
    * @instance
    */
        fontSize: 12,
        /** This allows to set font for label
    * @default "arial"
    * @type {String}
    * @example
    * &lt;div id="diagramcontent"&gt;&lt;/div&gt;<br>
    * &lt;script&gt;
    * var node=[];
    * node=[{labels:{"fontFamily": "Arial"}}];
    * $("#diagramcontent").ejDiagram({nodes:node});
    * &lt;/script&gt
    * @alias ejDiagram#nodes->labels->fontFamily
    
    * @memberof ejDiagram
    * @instance
    */
        fontFamily: "Arial",
        /** Used to set the text color
    * @default "black"
    * @type {String}
    * @example
    * &lt;div id="diagramcontent"&gt;&lt;/div&gt;<br>
    * &lt;script&gt;
    * var node=[];
    * node=[{labels:{"fontColor": "black"}}];
    * $("#diagramcontent").ejDiagram({nodes:node});
    * &lt;/script&gt
    * @alias ejDiagram#nodes->labels->fontColor
    
    * @memberof ejDiagram
    * @instance
    */
        fontColor: "black",
        /** Set offset to align label poistion
    * @default Point(0.5, 0.5)
    * @type {Point}
    * @example
    * &lt;div id="diagramcontent"&gt;&lt;/div&gt;<br>
    * &lt;script&gt;
    * var node=[];
    * node=[{labels:{offset: ej.datavisualization.Diagram.Point(0.5, 0.5)}}];
    * $("#diagramcontent").ejDiagram({nodes:node});
    * &lt;/script&gt
    * @alias ejDiagram#nodes->labels->offset
    
    * @memberof ejDiagram
    * @instance
    */
        offset: ej.datavisualization.Diagram.Point(0.5, 0.5),
        /** Alignment of text in an element see{@link TextAlign}
    
      * @default TextAlign.Center
      * @type {Enum}
      * @example
      * &lt;div id="diagramcontent"&gt;&lt;/div&gt;<br>
      * &lt;script&gt;
      * var node=[];
      * node=[{labels:{textAlign: ej.datavisualization.Diagram.TextAlign.Center}}];
      * $("#diagramcontent").ejDiagram({nodes:node});
      * &lt;/script&gt
       * @alias ejDiagram#nodes->labels->textAlign
       
     * @memberof ejDiagram
      * @instance
      */
        textAlign: ej.datavisualization.Diagram.TextAlign.Center,
        /** Horizontal alignment of text in an element see{@link HorizontalAlignment}
    
      * @default HorizontalAlignment.Center
      * @type {Enum}
      * @example
      * &lt;div id="diagramcontent"&gt;&lt;/div&gt;<br>
      * &lt;script&gt;
      * var node=[];
      * node=[{labels:{horizontalAlignment: ej.datavisualization.Diagram.HorizontalAlignment.Center}}];
      * $("#diagramcontent").ejDiagram({nodes:node});
      * &lt;/script&gt
       * @alias ejDiagram#nodes->labels->horizontalAlignment
      
    * @memberof ejDiagram
      * @instance
      */
        horizontalAlignment: ej.datavisualization.Diagram.HorizontalAlignment.Center,
        /** Vertical alignment of text in an element see{@link VerticalAlignment}
      
      * @default VerticalAlignment.Center
      * @type {Enum}
      * @example
      * &lt;div id="diagramcontent"&gt;&lt;/div&gt;<br>
      * &lt;script&gt;
      * var node=[];
      * node=[{labels:{verticalAlignment: ej.datavisualization.Diagram.VerticalAlignment.Center}}];
      * $("#diagramcontent").ejDiagram({nodes:node});
      * &lt;/script&gt
       * @alias ejDiagram#nodes->labels->verticalAlignment
    
    * @memberof ejDiagram
      * @instance
      */
        verticalAlignment: ej.datavisualization.Diagram.VerticalAlignment.Center,
        /** This enables or disables the line breaking rules for text
    * @default true
    * @type {Boolean}
    * @example
    * &lt;div id="diagramcontent"&gt;&lt;/div&gt;<br>
    * &lt;script&gt;
    * var node=[];
    * node=[{labels:{wrapText: true}}];
    * $("#diagramcontent").ejDiagram({nodes:node});
    * &lt;/script&gt
     * @alias ejDiagram#nodes->labels->wrapText
    
    * @memberof ejDiagram
    * @instance
    */
        wrapText: true,
        /** margin of the label 
      * @default ej.datavisualization.Diagram.Margin()
      * @type {Object}
      * @example
      * &lt;div id="diagramcontent"&gt;&lt;/div&gt;<br>
      * &lt;script&gt;
      * var node=[];
      * node=[{labels:{"margin": ej.datavisualization.Diagram.Margin()}}];
      * $("#diagramcontent").ejDiagram({nodes:node});
      * &lt;/script&gt
       * @alias ejDiagram#nodes->labels->margin
       
     * @memberof ejDiagram
      * @instance
      */
        margin: ej.datavisualization.Diagram.Margin(),
        /** To set the label edit mode see{@link LabelEditMode}       
      * @default LabelEditMode.Edit
      * @type {Enum}
      * @example
      * &lt;div id="diagramcontent"&gt;&lt;/div&gt;<br>
      * &lt;script&gt;
      * var node=[];
      * node=[{labels:{mode: ej.datavisualization.Diagram.LabelEditMode.Edit}}];
      * $("#diagramcontent").ejDiagram({nodes:node});
      * &lt;/script&gt
       * @alias ejDiagram#nodes->labels->mode
       
     * @memberof ejDiagram
      * @instance
      */
        mode: ej.datavisualization.Diagram.LabelEditMode.Edit,
        /** To set the width of the label
     * @default 50
     * @type {Interger}
     * @example
     * &lt;div id="diagramcontent"&gt;&lt;/div&gt;<br>
     * &lt;script&gt;
     * var node=[];
     * node=[{labels:{width: 50}}];
     * $("#diagramcontent").ejDiagram({nodes:node});
     * &lt;/script&gt
      * @alias ejDiagram#nodes->labels->width
     
     * @memberof ejDiagram
     * @instance
     */
        width: 50,
        /** To set the angle of the label
     * @default 0
     * @type {Number}
     * @example
     * &lt;div id="diagramcontent"&gt;&lt;/div&gt;<br>
     * &lt;script&gt;
     * var node=[];
     * node=[{labels:{rotateAngle: 90}}];
     * $("#diagramcontent").ejDiagram({nodes:node});
     * &lt;/script&gt
     * @memberof ejDiagram
     * @instance
     */
        rotateAngle: 0
    };
    ej.datavisualization.Diagram.TextBlock = function (options) {
        return $.extend(true, {}, ej.datavisualization.Diagram.TextBlockDefaults, options);
    };
    ///**The label properties for node/Connector
    //* @alias ejDiagram#node->labels*/
    ej.datavisualization.Diagram.LabelDefaults = $.extend(true, {}, ej.datavisualization.Diagram.TextBlockDefaults, {
        /** Name of the label 
      * @default ""
      * @type {String}
      * @example
      * &lt;div id="diagramcontent"&gt;&lt;/div&gt;<br>
      * &lt;script&gt;
      * var node=[];
      * node=[{labels:{name:"Label"}}];
      * $("#diagramcontent").ejDiagram({nodes:node});
      * &lt;/script&gt
       * @alias ejDiagram#nodes->labels->name
        
    * @memberof ejDiagram
      * @instance
      */
        name: "",
        /**Enables or disables the visibility of the label 
      * @default true
      * @type {Boolean}
      * @example
      * &lt;div id="diagramcontent"&gt;&lt;/div&gt;<br>
      * &lt;script&gt;
      * var node=[];
      * node=[{labels:{visible: true}}];
      * $("#diagramcontent").ejDiagram({nodes:node});
      * &lt;/script&gt
       * @alias ejDiagram#nodes->labels->visible
     
    * @memberof ejDiagram
      * @instance
      */
        visible: true,
        /**The color of the border for the text
      * @default "transparent"
      * @type {String}
      * @example
      * &lt;div id="diagramcontent"&gt;&lt;/div&gt;<br>
      * &lt;script&gt;
      * var node=[];
      * node=[{labels:{borderColor: "transparent"}}];
      * $("#diagramcontent").ejDiagram({nodes:node});
      * &lt;/script&gt
       * @alias ejDiagram#nodes->labels->borderColor
        
      * @memberof ejDiagram
      * @instance
      */
        borderColor: "transparent",
        /**Sets the border width of the label 
      * @default 0
      * @type {Number}
      * @example
      * &lt;div id="diagramcontent"&gt;&lt;/div&gt;<br>
      * &lt;script&gt;
      * var node=[];
      * node=[{labels:{borderWidth: 4}}];
      * $("#diagramcontent").ejDiagram({nodes:node});
      * &lt;/script&gt
       * @alias ejDiagram#nodes->labels->borderWidth
         
      * @memberof ejDiagram
      * @instance
      */
        borderWidth: 0,
        /**The color which is filled inside the text 
      * @default "transparent"
      * @type {String}
      * @example
      * &lt;div id="diagramcontent"&gt;&lt;/div&gt;<br>
      * &lt;script&gt;
      * var node=[];
      * node=[{labels:{fillColor: "green"}}];
      * $("#diagramcontent").ejDiagram({nodes:node});
      * &lt;/script&gt
       * @alias ejDiagram#nodes->labels->fillColor
       
      * @memberof ejDiagram
      * @instance
      */
        fillColor: "transparent"
    });
    ej.datavisualization.Diagram.Label = function (options) {
        var label = $.extend(true, {}, ej.datavisualization.Diagram.LabelDefaults, options);
        if (label.horizontalAlignment == "center")
            label.margin.left = 0;
        if (label.verticalAlignment == "center")
            label.margin.top = 0;
        return label;
    };
    //#endregion

    //#region Port


    //    /**
    //* @namespace ej
    //* @class ejDiagram.Port 
    //* @classdesc Port is a specific point on node to make a static connection with node
    //* @example 
    //* &lt;div id="diagram"&gt;&lt;/div&gt;<br>
    //* &lt;script&gt;
    //* //Create Diagram
    //$("#diagram").ejDiagram();
    //* &lt;/script&gt
    //*/
    ej.datavisualization.Diagram.PortDefaults = {
        /** The name of port to be specified
           * @default ""
           * @type {String}
           * @example
           * &lt;div id="diagramcontent"&gt;&lt;/div&gt;<br>
           * &lt;script&gt;
           * var node=[];
           * node={ports:[{ name: "port"}]};
           * $("#diagramcontent").ejDiagram({nodes:node});
           * &lt;/script&gt
    
            * @alias ejDiagram#nodes->ports->name;
           * @memberof ejDiagram 
           * @instance
           */
        name: "",
        /** Enables or disables the visibility of port See {@link PortVisibility}
      
      * @default PortVisibility.Default
      * @type {enum}
      * @example
      * &lt;div id="diagramcontent"&gt;&lt;/div&gt;<br>
      * &lt;script&gt;
      * var node=[];
      * node={ports:[{visibility: ej.datavisualization.Diagram.PortVisibility.Default}]};
      * $("#diagramcontent").ejDiagram({nodes:port});
      * &lt;/script&gt 
        * @alias ejDiagram#nodes->ports->visibility
           * @memberof ejDiagram 
      * @instance
      */
        visibility: ej.datavisualization.Diagram.PortVisibility.Default,
        /** Enables or disables the behavior of port See {@link PortConstraints}
      
      * @default PortConstraints.Connect
      * @type {enum}
      * @example
      * &lt;div id="diagramcontent"&gt;&lt;/div&gt;<br>
      * &lt;script&gt;
      * var node=[];
      * node={ports:[{constraints: ej.datavisualization.Diagram.PortConstraints.Connect}]};
      * $("#diagramcontent").ejDiagram({nodes:node});
      * &lt;/script&gt
        * @alias ejDiagram#nodes->ports->constraints
          * @memberof ejDiagram 
      * @instance
      */
        constraints: ej.datavisualization.Diagram.PortConstraints.Connect,
        /**Size of the port
    * @default 8
    * @type {Number}
    * @example
    * &lt;div id="diagramcontent"&gt;&lt;/div&gt;<br>
    * &lt;script&gt;
    * var node=[];
    * node={ports:[{size: 10}]};
    * $("#diagramcontent").ejDiagram({nodes:node});
    * &lt;/script&gt
     * @alias ejDiagram#nodes->ports->size
          * @memberof ejDiagram 
    * @instance
    */
        size: 8,
        /** Offset value to align the label position
    * @default Point(0, 0)
    * @type {Point}
    * @example
    * &lt;div id="diagramcontent"&gt;&lt;/div&gt;<br>
    * &lt;script&gt;
    * var node=[];
    * node={ports:[{{ x: 0, y: 0.5 }}]};
    * $("#diagramcontent").ejDiagram({nodes:node});
    * &lt;/script&gt
      * @alias ejDiagram#nodes->ports->offset
           * @memberof ejDiagram 
    * @instance
    */
        offset: ej.datavisualization.Diagram.Point(0, 0),
        /** Sets the border color of the port
    * @default "#1a1a1a"
    * @type {String}
    * @example
    * &lt;div id="diagramcontent"&gt;&lt;/div&gt;<br>
    * &lt;script&gt;
    * var node=[];
    * node={ports:[{borderColor:"green"}]};
    * $("#diagramcontent").ejDiagram({nodes:node});
    * &lt;/script&gt
       * @alias ejDiagram#nodes->ports->borderColor
          * @memberof ejDiagram 
    * @instance
    */
        borderColor: "#1a1a1a",
        /** Sets the border width of the port
    * @default 1
    * @type {Number}
    * @example
    * &lt;div id="diagramcontent"&gt;&lt;/div&gt;<br>
    * &lt;script&gt;
    * var node=[];
    * node={ports:[{borderWidth: 1}]};
    * $("#diagramcontent").ejDiagram({nodes:node});
    * &lt;/script&gt
      * @alias ejDiagram#nodes->ports->borderWidth
            * @memberof ejDiagram 
    * @instance
    */
        borderWidth: 1,
        /** The color to be filled inside the port 
     * @default "white"
     * @type {String}
     * @example
     * &lt;div id="diagramcontent"&gt;&lt;/div&gt;<br>
     * &lt;script&gt;
     * var node=[];
     * node={ports:[{fillColor:"green"}]};
     * $("#diagramcontent").ejDiagram({nodes:node});
     * &lt;/script&gt
      * @alias ejDiagram#nodes->ports->fillColor
           * @memberof ejDiagram 
     * @instance
     */
        fillColor: "white",
        /** Shape of the port to be updated See {@link PortShapes}
    
    * @default PortShapes.Square
    * @type {enum}
    * @example
    * &lt;div id="diagramcontent"&gt;&lt;/div&gt;<br>
    * &lt;script&gt;
    * var node=[];
    * node={ports:[{shape: ej.datavisualization.Diagram.PortShapes.Square}]};
    * $("#diagramcontent").ejDiagram({nodes:node});
    * &lt;/script&gt
     * @alias ejDiagram#nodes->ports->shape
            * @memberof ejDiagram
    * @instance
    */
        shape: ej.datavisualization.Diagram.PortShapes.Square,
        /**Path data to draw the custom port 
    * @default ""
    * @type {String}
    * @example
    * &lt;div id="diagramcontent"&gt;&lt;/div&gt;<br>
    * &lt;script&gt;
    * var node=[];
    * node={ports:[{pathData:""}]};
    * $("#diagramcontent").ejDiagram({nodes:node});
    * &lt;/script&gt
    * @alias ejDiagram#nodes->ports->pathData
          * @memberof ejDiagram 
    * @instance
    */
        pathData: ""
    };
    ej.datavisualization.Diagram.Port = function (options) {
        return $.extend(true, {}, ej.datavisualization.Diagram.PortDefaults, options);
    };
    //#endregion
    // /**
    //* @namespace ej
    //* @class Gradient 
    //* @classdesc The diagram control provides 2D surface to visualize the data as shapes, lines, text and images. It can be configured to DOM element such as DIV.
    //* @example 
    //* &lt;div id="diagram"&gt;&lt;/div&gt;<br>
    //* &lt;script&gt;
    //* //Create Diagram
    //$("#diagram").ejDiagram();
    //* &lt;/script&gt

    // */
    /**Specifies the gradient of the Node.
           * @type {object} 
              * @memberof ejDiagram
             * @alias ejDiagram#nodes->gradient->Stop
             * @instance
           */

    //#region Gradient
    ej.datavisualization.Diagram.StopDefaults = {
        /**The color for applied gradient region 
          * @default ""
          * @type {String}
          * @example
          * &lt;div id="diagramcontent"&gt;&lt;/div&gt;<br>
          * &lt;script&gt;
          * var gradient = ej.datavisualization.Diagram.LinearGradientDefaults();
          * gradient.type = "linear;
          * gradient.x1 = 0;
          * gradient.x2 = 50;
          * gradient.y1 = 0;
          * gradient.y2 = 50;
          * var stop = ej.datavisualization.Diagram.Stop();
          * stop.color = "white";
          * stop.offset = 0;
          * gradient.stops.push(stop);
          * stop = ej.datavisualization.Diagram.Stop();
          * stop.color = "darkCyan";
          * stop.offset = 100;
          * gradient.stops.push(stop);
          * gradient.stops.push(stop);
          * var nodes = [{name:"Node1",width: 100,height: 100,gradient : gradient}];
          * $("#diagramcontent").ejDiagram({nodes:nodes});
          * &lt;/script&gt
        * @alias ejDiagram#nodes->gradient->Stop->color
          * @memberof ejDiagram
          * @instance
          */
        color: "",
        /**To set desired offset to apply color to the node region
          * @default 0
          * @type {Int}
          * @example
          * &lt;div id="diagramcontent"&gt;&lt;/div&gt;<br>
          * &lt;script&gt;
          * var gradient = ej.datavisualization.Diagram.LinearGradientDefaults();
          * gradient.type = "linear;
          * gradient.x1 = 0;
          * gradient.x2 = 50;
          * gradient.y1 = 0;
          * gradient.y2 = 50;
          * var stop = ej.datavisualization.Diagram.Stop();
          * stop.color = "white";
          * stop.offset = 0;
          * gradient.stops.push(stop);
          * stop = ej.datavisualization.Diagram.Stop();
          * stop.color = "darkCyan";
          * stop.offset = 100;
          * gradient.stops.push(stop);
          * gradient.stops.push(stop);
          * var nodes = [{name:"Node1",width: 100,height: 100,gradient : gradient}];
          * $("#diagramcontent").ejDiagram({nodes:nodes});
          * &lt;/script&gt
          * @alias ejDiagram#nodes->gradient->Stop->offset
          * @memberof ejDiagram
          * @instance
          */
        offset: 0,
        /**Decribes the transparency level for the region
          * @default 1
          * @type {Int}
          * @example
          * &lt;div id="diagramcontent"&gt;&lt;/div&gt;<br>
          * &lt;script&gt;
          * var gradient = ej.datavisualization.Diagram.LinearGradientDefaults();
          * gradient.type = "linear;
          * gradient.x1 = 0;
          * gradient.x2 = 50;
          * gradient.y1 = 0;
          * gradient.y2 = 50;
          * var stop = ej.datavisualization.Diagram.Stop();
          * stop.color = "white";
          * stop.opacity = 0.8;
          * stop.offset = 0;
          * gradient.stops.push(stop);
          * stop = ej.datavisualization.Diagram.Stop();
          * stop.color = "darkCyan";
          * stop.offset = 100;
          * gradient.stops.push(stop);
          * gradient.stops.push(stop);
          * var nodes = [{name:"Node1",width: 100,height: 100,gradient : gradient}];
          * $("#diagramcontent").ejDiagram({nodes:nodes});
          * &lt;/script&gt
        * @alias ejDiagram#nodes->gradient->Stop->opacity
          * @memberof ejDiagram
          * @instance
          */
        opacity: 1
    };
    ej.datavisualization.Diagram.Stop = function (options) {
        return $.extend(true, {}, ej.datavisualization.Diagram.StopDefaults, options);
    };

    /**Paints an area with a linear gradient. 
           * @type {object}
            
              * @alias ejDiagram#nodes->gradient->LinearGradient
             
           */
    ej.datavisualization.Diagram.LinearGradientDefaults = {
        type: "linear",
        /**The starting X-Axis for the region
        * @default 0
        * @type {Int}
        * @example
        * &lt;div id="diagramcontent"&gt;&lt;/div&gt;<br>
        * &lt;script&gt;
        * var gradient = ej.datavisualization.Diagram.LinearGradientDefaults();
        * gradient.type = "linear;
        * gradient.x1 = 10;
        * gradient.x2 = 50;
        * gradient.y1 = 0;
        * gradient.y2 = 50;
        * var nodes = [{name:"Node1",width: 100,height: 100,gradient : gradient}];
        * $("#diagramcontent").ejDiagram({nodes:nodes});
        * &lt;/script&gt
          * @alias ejDiagram#nodes->gradient->LinearGradient->x1
        * @memberof ejDiagram
        * @instance
        */
        x1: 0,
        /**The ending X-Axis for the region
        * @default 0
        * @type {Int}
        * @example
        * &lt;div id="diagramcontent"&gt;&lt;/div&gt;<br>
        * &lt;script&gt;
        * var gradient = ej.datavisualization.Diagram.LinearGradientDefaults();
        * gradient.type = "linear;
        * gradient.x1 = 10;
        * gradient.x2 = 50;
        * gradient.y1 = 20;
        * gradient.y2 = 50;
        * var nodes = [{name:"Node1",width: 100,height: 100,gradient : gradient}];
        * $("#diagramcontent").ejDiagram({nodes:nodes});
        * &lt;/script&gt
          * @alias ejDiagram#nodes->gradient->LinearGradient->x2
        * @memberof ejDiagram
        * @instance
        */
        x2: 0,
        /**The starting Y-Axis for the region
        * @default 0
        * @type {Int}
        * @example
        * &lt;div id="diagramcontent"&gt;&lt;/div&gt;<br>
        * &lt;script&gt;
        * var gradient = ej.datavisualization.Diagram.LinearGradientDefaults();
        * gradient.type = "linear;
        * gradient.x1 = 10;
        * gradient.x2 = 50;
        * gradient.y1 = 0;
        * gradient.y2 = 50;
        * var nodes = [{name:"Node1",width: 100,height: 100,gradient : gradient}];
        * $("#diagramcontent").ejDiagram({nodes:nodes});
        * &lt;/script&gt
           * @alias ejDiagram#nodes->gradient->LinearGradient->y1
        * @memberof ejDiagram
        * @instance
        */
        y1: 0,
        /**The ending Y-Axis for the region
        * @default 0
        * @type {Int}
        * @example
        * &lt;div id="diagramcontent"&gt;&lt;/div&gt;<br>
        * &lt;script&gt;
        * var gradient = ej.datavisualization.Diagram.LinearGradientDefaults();
        * gradient.type = "linear;
        * gradient.x1 = 10;
        * gradient.x2 = 50;
        * gradient.y1 = 0;
        * gradient.y2 = 50;
        * var nodes = [{name:"Node1",width: 100,height: 100,gradient : gradient}];
        * $("#diagramcontent").ejDiagram({nodes:nodes});
        * &lt;/script&gt
          * @alias ejDiagram#nodes->gradient->LinearGradient->y2
        * @memberof ejDiagram
        * @instance
        */
        y2: 0,
        /**The stop region for the gradient
        * @default ""
          * @type {Array}
          * @example
          * &lt;div id="diagramcontent"&gt;&lt;/div&gt;<br>
          * &lt;script&gt;
          * var gradient = ej.datavisualization.Diagram.LinearGradientDefaults();
          * gradient.type = "linear;
          * gradient.x1 = 10;
          * gradient.x2 = 50;
          * gradient.y1 = 20;
          * gradient.y2 = 50;
          * var stop = ej.datavisualization.Diagram.Stop();
          * stop.color = "white";
          * stop.opacity = 0.8;
          * stop.offset = 0;
          * gradient.stops.push(stop);
          * stop = ej.datavisualization.Diagram.Stop();
          * stop.color = "darkCyan";
          * stop.offset = 100;
          * gradient.stops.push(stop);
          * gradient.stops.push(stop);
          * var nodes = [{name:"Node1",width: 100,height: 100,gradient : gradient}];
          * $("#diagramcontent").ejDiagram({nodes:nodes});
          * &lt;/script&gt
           * @alias ejDiagram#nodes->gradient->LinearGradient->stops
          * @memberof ejDiagram
          * @instance
        */
        stops: []
    };
    ej.datavisualization.Diagram.LinearGradient = function (options) {
        for (var i = 0; i < options.stops.length; i++) {
            options.stops[i] = ej.datavisualization.Diagram.Stop(options.stops[i]);
        }
        return $.extend(true, {}, ej.datavisualization.Diagram.LinearGradientDefaults, options);
    };
    /**Paints an area with a radial gradient. A focal point defines the beginning of the gradient, and a circle defines the end point of the gradient. 
           * @type {object}
             
                 * @alias ejDiagram#nodes->gradient->RadialGradient
          
           */
    ej.datavisualization.Diagram.RadialGradientDefaults = {
        type: "radial",
        /**The starting X-Axis for the region
        * @default 0
        * @type {Int}
        * @example
        * &lt;div id="diagramcontent"&gt;&lt;/div&gt;<br>
        * &lt;script&gt;
        * var gradient = ej.datavisualization.Diagram.RadialGradientDefaults();
        * gradient.type = "linear;
        * gradient.cx = 20;
        * gradient.cy = 50;
        * gradient.fx = 20;
        * gradient.fy = 50;
        * var nodes = [{name:"Node1",width: 100,height: 100,gradient : gradient}];
        * $("#diagramcontent").ejDiagram({nodes:nodes});
        * &lt;/script&gt
        * @alias ejDiagram#nodes->gradient->RadialGradient->cx
        * @memberof ejDiagram
        * @instance
        */
        cx: 0,
        /**The ending X-Axis for the region
       * @default 0
       * @type {Int}
       * @example
       * &lt;div id="diagramcontent"&gt;&lt;/div&gt;<br>
       * &lt;script&gt;
       * var gradient = ej.datavisualization.Diagram.RadialGradientDefaults();
       * gradient.type = "linear;
       * gradient.cx = 20;
       * gradient.cy = 50;
       * gradient.fx = 20;
       * gradient.fy = 50;
       * var nodes = [{name:"Node1",width: 100,height: 100,gradient : gradient}];
       * $("#diagramcontent").ejDiagram({nodes:nodes});
       * &lt;/script&gt
    * @alias ejDiagram#nodes->gradient->RadialGradient->cy
       * @memberof ejDiagram
       * @instance
       */
        cy: 0,
        /**The starting X-Axis for the region
        * @default 0
        * @type {Int}
        * @example
        * &lt;div id="diagramcontent"&gt;&lt;/div&gt;<br>
        * &lt;script&gt;
        * var gradient = ej.datavisualization.Diagram.RadialGradientDefaults();
        * gradient.type = "linear;
        * gradient.cx = 20;
        * gradient.cy = 50;
        * gradient.fx = 20;
        * gradient.fy = 50;
        * var nodes = [{name:"Node1",width: 100,height: 100,gradient : gradient}];
        * $("#diagramcontent").ejDiagram({nodes:nodes});
        * &lt;/script&gt
      * @alias ejDiagram#nodes->gradient->RadialGradient->fx
        * @memberof ejDiagram
        * @instance
        */
        fx: 0,
        /**The ending Y-Axis for the region
       * @default 0
       * @type {Int}
       * @example
       * &lt;div id="diagramcontent"&gt;&lt;/div&gt;<br>
       * &lt;script&gt;
       * var gradient = ej.datavisualization.Diagram.RadialGradientDefaults();
       * gradient.type = "linear;
       * gradient.cx = 20;
       * gradient.cy = 50;
       * gradient.fx = 20;
       * gradient.fy = 50;
       * var nodes = [{name:"Node1",width: 100,height: 100,gradient : gradient}];
       * $("#diagramcontent").ejDiagram({nodes:nodes});
       * &lt;/script&gt
    * @alias ejDiagram#nodes->gradient->RadialGradient->fy
       * @memberof ejDiagram
       * @instance
       */
        fy: 0,
        r: 50,
        /**The stop region for the gradient
        * @default ""
          * @type {Array}
          * @example
          * &lt;div id="diagramcontent"&gt;&lt;/div&gt;<br>
          * &lt;script&gt;
          * var gradient = ej.datavisualization.Diagram.RadialGradientDefaults();
          * gradient.type = "linear;
          * gradient.x1 = 10;
          * gradient.x2 = 50;
          * gradient.y1 = 20;
          * gradient.y2 = 50;
          * var stop = ej.datavisualization.Diagram.Stop();
          * stop.color = "white";
          * stop.opacity = 0.8;
          * stop.offset = 0;
          * gradient.stops.push(stop);
          * stop = ej.datavisualization.Diagram.Stop();
          * stop.color = "darkCyan";
          * stop.offset = 100;
          * gradient.stops.push(stop);
          * gradient.stops.push(stop);
          * var nodes = [{name:"Node1",width: 100,height: 100,gradient : gradient}];
          * $("#diagramcontent").ejDiagram({nodes:nodes});
          * &lt;/script&gt
          * @alias ejDiagram#nodes->gradient->RadialGradient->stops
          * @memberof ejDiagram
          * @instance
        */
        stops: []
    };
    ej.datavisualization.Diagram.RadialGradient = function (options) {
        for (var i = 0; i < options.stops.length; i++) {
            options.stops[i] = ej.datavisualization.Diagram.Stop(options.stops[i]);
        }
        return $.extend(true, {}, ej.datavisualization.Diagram.RadialGradientDefaults, options);
    };


    //#region Node
    ej.datavisualization.Diagram.NodeBaseDefaults = {
        /** To set the name of node
          * @default ""
          * @type {String}
          * @example
          * &lt;div id="diagramcontent"&gt;&lt;/div&gt;<br>
          * &lt;script&gt;
          * var nodes = [];
          * nodes=[{name:"Node1"}];
          * $("#diagramcontent").ejDiagram({nodes:nodes});
          * &lt;/script&gt
          * @alias ejDiagram#nodes->name
          * @memberof ejDiagram
          * @instance
          */
        name: "",
        /** To sets the width of the node
          * @default 0
          * @type {Int}
          * @example
          * &lt;div id="diagramcontent"&gt;&lt;/div&gt;<br>
          * &lt;script&gt;
          * var nodes = [];
          * nodes=[{width: 100}];
          * $("#diagramcontent").ejDiagram({nodes:nodes});
          * &lt;/script&gt
            * @alias ejDiagram#nodes->width
          * @memberof ejDiagram
          * @instance
          */
        width: 0,
        /** To sets the height of the node
          * @default 0
          * @type {Int}
          * @example
          * &lt;div id="diagramcontent"&gt;&lt;/div&gt;<br>
          * &lt;script&gt;
          * var nodes = [];
          * nodes=[{height: 100}];
          * $("#diagramcontent").ejDiagram({nodes:nodes});
          * &lt;/script&gt
          * @alias ejDiagram#nodes->height
          * @memberof ejDiagram
          * @instance
          */
        height: 0,
        /** Align nodes position along X-axis
          * @default 0
          * @type {Int}
          * @example
          * &lt;div id="diagramcontent"&gt;&lt;/div&gt;<br>
          * &lt;script&gt;
          * var nodes = [];
          * nodes=[{offsetX: 100}];
          * $("#diagramcontent").ejDiagram({nodes:nodes});
          * &lt;/script&gt
           * @alias ejDiagram#nodes->offsetX
          * @memberof ejDiagram
          * @instance
          */
        offsetX: 0,
        /** Align nodes position along y axis
          * @default 0
          * @type {Int}
          * @example
          * &lt;div id="diagramcontent"&gt;&lt;/div&gt;<br>
          * &lt;script&gt;
          * var nodes = [];
          * nodes=[{offsetY: 100}];
          * $("#diagramcontent").ejDiagram({nodes:nodes});
          * &lt;/script&gt
          * @alias ejDiagram#nodes->offsetY
          * @memberof ejDiagram
          * @instance
          */
        offsetY: 0,
        /** To set the visibility of the node
          * @default true
          * @type {Boolean}
          * @example
          * &lt;div id="diagramcontent"&gt;&lt;/div&gt;<br>
          * &lt;script&gt;
          * var nodes = [];
          * nodes=[{visible: true}];
          * $("#diagramcontent").ejDiagram({nodes:nodes});
          * &lt;/script&gt
           * @alias ejDiagram#nodes->visible
          * @memberof ejDiagram
          * @instance
          */
        visible: true,
        /** Zorder of the node
          * @default 0
          * @type {Int}
          * @example
          * &lt;div id="diagramcontent"&gt;&lt;/div&gt;<br>
          * &lt;script&gt;
          * var nodes = [];
          * nodes=[{zOrder: 1}];
          * $("#diagramcontent").ejDiagram({nodes:nodes});
          * &lt;/script&gt
          * @alias ejDiagram#nodes->zOrder
          * @memberof ejDiagram
          * @instance
          */
        zOrder: 0,
        /**The arrangements of node in layout
          * @default false
          * @type {Boolean}
          * @example
          * &lt;div id="diagramcontent"&gt;&lt;/div&gt;<br>
          * &lt;script&gt;
          * var nodes = [];
          * nodes=[{visible: true}];
          * $("#diagramcontent").ejDiagram({nodes:nodes});
          * &lt;/script&gt
          * @alias ejDiagram#nodes->excludeFromLayout
          * @memberof ejDiagram
          * @instance
          */
        excludeFromLayout: false,
        /** To enables or disables the default behaviour of the node see{@link NodeConstraints}
      
          * @default NodeConstraints.Default
          * @type {Constraints}
          * @example
          * &lt;div id="diagramcontent"&gt;&lt;/div&gt;<br>
          * &lt;script&gt;
          * var nodes = [];
          * nodes=[{Constraints: ej.datavisualization.Diagram.NodeConstraints.Select}];
          * $("#diagramcontent").ejDiagram({nodes:nodes});
          * &lt;/script&gt
          * @alias ejDiagram#nodes->constraints
          * @memberof ejDiagram
          * @instance
          */
        constraints: ej.datavisualization.Diagram.NodeConstraints.Default,
        /**parent name of the node
           * @default ""
           * @type {String}
           * @example
           * &lt;div id="diagramcontent"&gt;&lt;/div&gt;<br>
           * &lt;script&gt;
           * var node = [ {name: "rect",parent:"group1"}];
           * &lt;/script&gt
           * @alias ejDiagram#nodes->parent
           * @memberof ejDiagram
           * @instance
           */
        parent: "",
        /** Collection labels of node
          * @default ""
          * @type {Array}
          * @example
          * &lt;div id="diagramcontent"&gt;&lt;/div&gt;<br>
          * &lt;script&gt;
          * var label = [];
          * label = { "text": "Node1", "fontColor": "Red"};
          * nodes=[{labels:label}];
          * $("#diagramcontent").ejDiagram({nodes:nodes});
          * &lt;/script&gt
          * @alias ejDiagram#nodes->labels
          * @memberof ejDiagram
          * @instance
          */
        labels: [],
        /**To add port to the node
          * @default ""
          * @type {Array}
          * @example
          * &lt;div id="diagramcontent"&gt;&lt;/div&gt;<br>
          * &lt;script&gt;
          * var port: [{ offset: { x: 0, y: 0.5 }, name: "aport" }]
          * nodes=[{ports:port}];
          * $("#diagramcontent").ejDiagram({nodes:nodes});
          * &lt;/script&gt
          * @alias ejDiagram#nodes->ports
          * @memberof ejDiagram
          * @instance
          */
        ports: [],
        /**The connectors which comes to the Node.
          * @default ""
          * @type {Array}
          * @example
          * &lt;div id="diagramcontent"&gt;&lt;/div&gt;<br>
          * &lt;script&gt;
          * var connectors = [{ name: "connector1", sourceNode: "Node1", targetNode: "Node2"}];
          * nodes=[{inEdges:connectors}];
          * $("#diagramcontent").ejDiagram({nodes:nodes});
          * &lt;/script&gt
          * @alias ejDiagram#nodes->inEdges
          * @memberof ejDiagram
          * @instance
          */
        inEdges: [],
        /** The connectors which leave from the Node.
          * @default ""
          * @type {Array}
          * @example
          * &lt;div id="diagramcontent"&gt;&lt;/div&gt;<br>
          * &lt;script&gt;
          * var connectors = [{ name: "connector1", sourceNode: "Node1", targetNode: "Node2"}];
          * nodes=[{outEdges:connectors}];
          * $("#diagramcontent").ejDiagram({nodes:nodes});
          * &lt;/script&gt
          * @alias ejDiagram#nodes->outEdges
          * @memberof ejDiagram
          * @instance
          */
        outEdges: [],
        /** To set the rotation angle of the node
          * @default 0
          * @type {Number}
          * @example
          * &lt;div id="diagramcontent"&gt;&lt;/div&gt;<br>
          * &lt;script&gt;
          * var nodes = [];
          * nodes=[{rotateAngle: 120}];
          * $("#diagramcontent").ejDiagram({nodes:nodes});
          * &lt;/script&gt
          * @alias ejDiagram#nodes->rotateAngle
          * @memberof ejDiagram
          * @instance
          */
        rotateAngle: 0,
        /** To set the pivot point of the node
          * @default Points(0.5,0.5)
          * @type {Point}
          * @example
          * &lt;div id="diagramcontent"&gt;&lt;/div&gt;<br>
          * &lt;script&gt;
          * var nodes = [];
          * nodes=[{pivot: ej.datavisualization.Diagram.Point(0.8, 0.8)}];
          * $("#diagramcontent").ejDiagram({nodes:nodes});
          * &lt;/script&gt
          * @alias ejDiagram#nodes->pivot
          * @memberof ejDiagram
          * @instance
          */
        pivot: ej.datavisualization.Diagram.Point(0.5, 0.5),
        /** To provide/save extra information about Node
          * @default null
          * @type {Object}
          * @example
          * &lt;div id="diagramcontent"&gt;&lt;/div&gt;<br>
          * &lt;script&gt;
          * var nodes = [];
          * var addInfo = {Name1:"Node1",Name2:"Name2"}
          * nodes=[{addInfo: addInfo}];
          * $("#diagramcontent").ejDiagram({nodes:nodes});
          * &lt;/script&gt
          * @alias ejDiagram#nodes->addInfo
          * @memberof ejDiagram
          * @instance
          */
        addInfo: {},
        /** To set marginleft for the node. Applicable if the type is group.
         * @default 0
         * @type {Int}
          * @example
          * &lt;div id="diagramcontent"&gt;&lt;/div&gt;<br>
          * &lt;script&gt;
          * var nodes = [];
          * nodes=[{marginLeft: 1}];
          * $("#diagramcontent").ejDiagram({nodes:nodes});
          * &lt;/script&gt
          * @alias ejDiagram#nodes->marginLeft
          * @memberof ejDiagram
          * @instance
         */
        marginLeft: 0,
        /** To set marginTop for the node. Applicable if the type is group.
      * @default 0
      * @type {Int}
       * @example
       * &lt;div id="diagramcontent"&gt;&lt;/div&gt;<br>
       * &lt;script&gt;
       * var nodes = [];
       * node={name:"node1",marginTop: 1};
       * $("#diagramcontent").ejDiagram({nodes:[node]});
       * &lt;/script&gt
       * @alias ejDiagram#nodes->marginTop
       * @memberof ejDiagram
       * @instance
      */
        marginTop: 0,
        /** To set marginRight for the node. Applicable if the type is group.
        * @default 0
        * @type {Int}
         * @example
         * &lt;div id="diagramcontent"&gt;&lt;/div&gt;<br>
         * &lt;script&gt;
         * var nodes = [];
         * node={name:"node1",marginRight: 1};
        * $("#diagramcontent").ejDiagram({nodes:[node]});
         * &lt;/script&gt
         * @alias ejDiagram#nodes->marginRight
         * @memberof ejDiagram
         * @instance
        */
        marginRight: 0,
        /** To set marginBottom for the node. Applicable if the type is group.
         * @default 0
         * @type {Int}
          * @example
          * &lt;div id="diagramcontent"&gt;&lt;/div&gt;<br>
          * &lt;script&gt;
          * var nodes = [];
          * node={name:"node1",marginBottom: 1};
          * $("#diagramcontent").ejDiagram({nodes:[node]});
          * &lt;/script&gt
          * @alias ejDiagram#nodes->marginBottom
          * @memberof ejDiagram
          * @instance
         */
        marginBottom: 0,
        /** To set the horizontal alignment for node. Applicable if the type is group.
         * @default HorizontalAlignment.Left
         * @type {HorizontalAlignment}
         * @example
         * &lt;div id="diagramcontent"&gt;&lt;/div&gt;<br>
         * &lt;script&gt; 
          * node={name:"node1", ej.datavisualization.Diagram.HorizontalAlignment.Left};
         * $("#diagramcontent").ejDiagram({nodes:[node]});
         * &lt;/script&gt
         * @alias ejDiagram#nodes->horizontalAlign
         * @memberof ejDiagram
         * @instance
         */
        horizontalAlign: ej.datavisualization.Diagram.HorizontalAlignment.Left,
        /** To set the Vertical  alignment for node. Applicable if the type is group.
        * @default VerticalAlignment.Top
        * @type {VerticalAlignment}
        * @example
        * &lt;div id="diagramcontent"&gt;&lt;/div&gt;<br>
        * &lt;script&gt;
        * var nodes = [];
        * node={name:"node1",VerticalAlignment: ej.datavisualization.Diagram.VerticalAlignment.Top};
        * $("#diagramcontent").ejDiagram({nodes:[node]});
        * &lt;/script&gt
        * @alias ejDiagram#nodes->vertical Align
        * @memberof ejDiagram
        * @instance
        */
        verticalAlign: ej.datavisualization.Diagram.VerticalAlignment.Top,
        /** To set minimum width for the node. Applicable if the type is group.
         * @default 0
         * @type {Int}
          * @example
          * &lt;div id="diagramcontent"&gt;&lt;/div&gt;<br>
          * &lt;script&gt;
          * var nodes = [];
          * node={name:"node1",minWidth: 1};
          * $("#diagramcontent").ejDiagram({nodes:[node]});
          * &lt;/script&gt
          * @alias ejDiagram#nodes->minWidth
          * @memberof ejDiagram
          * @instance
         */
        minWidth: 0,
        /** To set maximum width for the node. Applicable if the type is group.
         * @default 0
         * @type {Int}
          * @example
          * &lt;div id="diagramcontent"&gt;&lt;/div&gt;<br>
          * &lt;script&gt;
          * var nodes = [];
          * node={name:"node1",maxWidth: 1};
          * $("#diagramcontent").ejDiagram({nodes:[node]});
          * &lt;/script&gt
          * @alias ejDiagram#nodes->maxWidth
          * @memberof ejDiagram
          * @instance
         */
        maxWidth: 0,
        /** To set minimum height for the node. Applicable if the type is group.
         * @default 0
         * @type {Int}
          * @example
          * &lt;div id="diagramcontent"&gt;&lt;/div&gt;<br>
          * &lt;script&gt;
          * var nodes = [];
          * node={name:"node1",minHeight: 1};
          * $("#diagramcontent").ejDiagram({nodes:[node]});
          * &lt;/script&gt
          * @alias ejDiagram#nodes->minHeight
          * @memberof ejDiagram
          * @instance
         */
        minHeight: 0,
        /** To set maximum height for the node. Applicable if the type is group.
         * @default 0
         * @type {Int}
          * @example
          * &lt;div id="diagramcontent"&gt;&lt;/div&gt;<br>
          * &lt;script&gt;
          * var nodes = [];
          * node={name:"node1",maxHeight: 1};
          * $("#diagramcontent").ejDiagram({nodes:[node]});
          * &lt;/script&gt
          * @alias ejDiagram#nodes->maxHeight
          * @memberof ejDiagram
          * @instance
         */
        maxHeight: 0,
        /** To set distance to left of between node and connector.
         * @default 0
         * @type {Int}
          * @example
          * &lt;div id="diagramcontent"&gt;&lt;/div&gt;<br>
          * &lt;script&gt;
          * var nodes = [];
          * node={name:"node1",connectionPadding: 2};
          * $("#diagramcontent").ejDiagram({nodes:[node]});
          * &lt;/script&gt
          * @alias ejDiagram#nodes->maxHeight
          * @memberof ejDiagram
          * @instance
         */
        connectionPadding:0
    };
    ej.datavisualization.Diagram.NodeBase = function (options) {
        var i;
        if (options.labels)
            for (i = 0; i < options.labels.length; i++) {
                options.labels[i] = ej.datavisualization.Diagram.Label(options.labels[i]);
            }
        if (options.ports)
            for (i = 0; i < options.ports.length; i++) {
                options.ports[i] = ej.datavisualization.Diagram.Port(options.ports[i]);
            }
        return $.extend(true, {}, ej.datavisualization.Diagram.NodeBaseDefaults, options);
    };

    ej.datavisualization.Diagram.NodeDefaults = {
        /** The color to be filled inside the node
          * @default "White"
          * @type {String}
          * @example
          * &lt;div id="diagramcontent"&gt;&lt;/div&gt;<br>
          * &lt;script&gt;
          * var nodes = [];
          * nodes=[{fillColor: "white"}];
          * $("#diagramcontent").ejDiagram({nodes:nodes});
          * &lt;/script&gt
           * @alias ejDiagram#nodes->fillColor
          * @memberof ejDiagram
          * @instance
          */
        fillColor: "white",
        /** Sets the border color of the node
      * @default "black"
      * @type {String}
      * @example
      * &lt;div id="diagramcontent"&gt;&lt;/div&gt;<br>
      * &lt;script&gt;
      * var nodes = [];
      * nodes=[{borderColor: "black"}];
      * $("#diagramcontent").ejDiagram({nodes:nodes});
      * &lt;/script&gt
       * @alias ejDiagram#nodes->borderColor
      * @memberof ejDiagram
      * @instance
      */
        borderColor: "black",
        /** Sets the border width of the node
     * @default 1
     * @type {Number}
     * @example
     * &lt;div id="diagramcontent"&gt;&lt;/div&gt;<br>
     * &lt;script&gt;
     * var nodes = [];
     * nodes=[{borderWidth: 3}];
     * $("#diagramcontent").ejDiagram({nodes:nodes});
     * &lt;/script&gt
      * @alias ejDiagram#nodes->borderWidth
     * @memberof ejDiagram
     * @instance
     */
        borderWidth: 1,
        /** The dotted line for the node
    * @default ""
    * @type {String}
    * @example
    * &lt;div id="diagramcontent"&gt;&lt;/div&gt;<br>
    * &lt;script&gt;
    * var nodes = [];
    * nodes=[{borderDashArray: "2 2"}];
    * $("#diagramcontent").ejDiagram({nodes:nodes});
    * &lt;/script&gt
     * @alias ejDiagram#nodes->borderDashArray
    * @memberof ejDiagram
    * @instance
    */
        borderDashArray: "",
        /**Defines the transparency of the node
    * @default 1
    * @type {Number}
    * @example
    * &lt;div id="diagramcontent"&gt;&lt;/div&gt;<br>
    * &lt;script&gt;
    * var nodes = [];
    * nodes=[{opacity: 2}];
    * $("#diagramcontent").ejDiagram({nodes:nodes});
    * &lt;/script&gt
    * @alias ejDiagram#nodes->opacity
    * @memberof ejDiagram
    * @instance
    */
        opacity: 1,
        /** Smooth transition from one color to another color in node   
         * @alias ejDiagram#nodes->gradient
    * @memberof ejDiagram
    * @instance
    */
        gradient: null,
        /** The shape of the node see{@link Shapes}
    
    * @default Diagram.Shape()
    * @type {Enum}
    * @example
    * &lt;div id="diagramcontent"&gt;&lt;/div&gt;<br>
    * &lt;script&gt;
    * var nodes = [];
    * nodes=[shape:{type: "rectangle"}];
    * $("#diagramcontent").ejDiagram({nodes:nodes});
    * &lt;/script&gt
     * @alias ejDiagram#nodes->shape
    * @memberof ejDiagram
    * @instance
    */
        shape: ej.datavisualization.Diagram.Shape(),
        /** To expand the diagram.
      * @default true
      * @type {Boolean}
      * @example
      * &lt;div id="diagramcontent"&gt;&lt;/div&gt;<br>
      * &lt;script&gt;
      * var nodes = [];
      * nodes=[{isExpanded: true}];
      * $("#diagramcontent").ejDiagram({nodes:nodes});
      * &lt;/script&gt
          * @alias ejDiagram#nodes->expanded
      * @memberof ejDiagram
      * @instance
      */
        isExpanded: true
    };
    ej.datavisualization.Diagram.Node = function (options) {
        if (options.gradient) {
            if (options.gradient.type === "linear") {
                options.gradient = ej.datavisualization.Diagram.LinearGradient(options.gradient);
            }
            else if (options.gradient.type === "radial") {
                options.gradient = ej.datavisualization.Diagram.RadialGradient(options.gradient);
            }
        }
        if (options.shape) {
            options.shape = ej.datavisualization.Diagram.Shape(options.shape);
        }
        options.minWidth = options.minWidth ? options.minWidth : 0;
        options.minHeight = options.minHeight ? options.minHeight : 0;
        options.maxWidth = options.maxWidth ? options.maxWidth : 0;
        options.maxHeight = options.maxHeight ? options.maxHeight : 0;

        return $.extend(true, {}, ej.datavisualization.Diagram.NodeDefaults, ej.datavisualization.Diagram.NodeBase(options));
    };
    //#endregion

    //#region Connector
    ej.datavisualization.Diagram.ConnectorDefaults = {
        /**Name of the connector
          * @default ""
          * @type {String}
          * @example
          * &lt;div id="diagramcontent"&gt;&lt;/div&gt;<br>
          * &lt;script&gt;
          * var connectors = [];
          * connectors=[{name:"Connector1"}];
          * $("#diagramcontent").ejDiagram({connectors:connectors});
          * &lt;/script&gt
           * @alias ejDiagram#connectors->name
          * @memberof ejDiagram
          * @instance
          */
        name: "",
        /** Enables or disables the visibility of connector
          * @default true
          * @type {Boolean}
          * @example
          * &lt;div id="diagramcontent"&gt;&lt;/div&gt;<br>
          * &lt;script&gt;
          * var connectors = [];
          * connectors=[{visible: true}];
          * $("#diagramcontent").ejDiagram({connectors:connectors});
          * &lt;/script&gt
              * @alias ejDiagram#connectors->visible
          * @memberof ejDiagram
          * @instance
          */
        visible: true,
        /** Dotted lines for connector
          * @default ""
          * @type {String}
          * @example
          * &lt;div id="diagramcontent"&gt;&lt;/div&gt;<br>
          * &lt;script&gt;
          * var connectors = [];
          * connectors=[{lineDashArray:"4 4"}];
          * $("#diagramcontent").ejDiagram({connectors:connectors});
          * &lt;/script&gt
              * @alias ejDiagram#connectors->lineDashArray
          * @memberof ejDiagram
          * @instance
          */
        lineDashArray: "",
        /** To set targetDecorator for connector
          * @default null
          * @type {String}
          * @example
          * &lt;div id="diagramcontent"&gt;&lt;/div&gt;<br>
          * &lt;script&gt;
          * var connectors = [];
          * var targetDecorator = { shape: "arrow", borderColor: "#606060", width: "10", height: "10" };
          * connectors=[{targetDecorator:targetDecorator}];
          * $("#diagramcontent").ejDiagram({connectors:connectors});
          * &lt;/script&gt
              * @alias ejDiagram#connectors->targetDecorator
          * @memberof ejDiagram
          * @instance
          */
        targetDecorator:  /** @lends ejdiagram# */
            {
                /** The shape of decorator in connector See {@link DecoratorShapes}
     
        * @default ej.datavisualization.Diagram.DecoratorShapes.Arrow
        * @type {enum}
        * @example
        * &lt;div id="diagramcontent"&gt;&lt;/div&gt;<br>
        * &lt;script&gt;
        * var link=[];
        * link={[targetDecorator:{shape:"arrow"}]};
        * $("#diagramcontent").ejDiagram({connectors:link});
        * &lt;/script&gt
        * @alias ejDiagram#connectors->targetDecorator->shape
    
        * @memberof ejDiagram
        * @instance
        */

                shape: ej.datavisualization.Diagram.DecoratorShapes.Arrow,
                /** This allows to set the width of the decorator in connector
            * @default 8
            * @type {Number}
            * @example
            * &lt;div id="diagramcontent"&gt;&lt;/div&gt;<br>
            * &lt;script&gt;
            * var link=[];
            * link={[targetDecorator:{width:10}]};
            * $("#diagramcontent").ejDiagram({connectors:link});
            * &lt;/script&gt
             * @alias ejDiagram#connectors->targetDecorator->width
             
            * @memberof ejDiagram
            * @instance
            */
                width: 8,
                /** This allows to set the height of the decorator in connector
                 * @default 8
                 * @type {Number}
                 * @example
                 * &lt;div id="diagramcontent"&gt;&lt;/div&gt;<br>
                 * &lt;script&gt;
                 * var link=[];
                 * link={[targetDecorator:{height:10}]};
                 * $("#diagramcontent").ejDiagram({connectors:link});
                 * &lt;/script&gt
                   * @alias ejDiagram#connectors->targetDecorator->height
              
              * @memberof ejDiagram
                 * @instance
                 */
                height: 8,
                /** Sets the border color of the decorator
            * @default "black"
            * @type {String}
            * @example
            * &lt;div id="diagramcontent"&gt;&lt;/div&gt;<br>
            * &lt;script&gt;
            * var link=[];
            * link={[targetDecorator:{borderColor: "green"}]};
            * $("#diagramcontent").ejDiagram({connectors:link});
            * &lt;/script&gt
              * @alias ejDiagram#connectors->targetDecorator->borderColor
              
             * @memberof ejDiagram
            * @instance
            */
                borderColor: "black",
                /** The color which sets inside the decorator 
            * @default "black"
            * @type {String}
            * @example
            * &lt;div id="diagramcontent"&gt;&lt;/div&gt;<br>
            * &lt;script&gt;
            * var link=[];
            * link={[targetDecorator:{fillColor: "green"}]};
            * $("#diagramcontent").ejDiagram({connectors:link});
            * &lt;/script&gt
             * @alias ejDiagram#connectors->targetDecorator->fillColor
             
            * @memberof ejDiagram
            * @instance
            */
                fillColor: "black",
                /** Path data to draw custom shape for decorator
            * @default ""
            * @type {String}
            * @example
            * &lt;div id="diagramcontent"&gt;&lt;/div&gt;<br>
            * &lt;script&gt;
            * var link=[];
            * link={[targetDecorator:{pathData: "M 269.711,29.3333C 269.711,44.061 257.772,56 243.044,56z"}]};
            * $("#diagramcontent").ejDiagram({connectors:link});
            * &lt;/script&gt
            * @alias ejDiagram#connectors->targetDecorator->pathData
            
            * @memberof ejDiagram
            * @instance
            */
                pathData: ""
            },
        /** To set sourceDecorator for connector
         * @default null
         * @type {String}
         * @example
         * &lt;div id="diagramcontent"&gt;&lt;/div&gt;<br>
         * &lt;script&gt;
         * var connectors = [];
         * var targetDecorator = { shape: "arrow", borderColor: "#606060", width: "10", height: "10" };
         * connectors=[{targetDecorator:targetDecorator}];
         * $("#diagramcontent").ejDiagram({connectors:connectors});
         * &lt;/script&gt
             * @alias ejDiagram#connectors->sourceDecorator
         * @memberof ejDiagram
         * @instance
         */

        sourceDecorator:
            /** @lends ejdiagram# */
            {
                /** The shape of decorator in connector See {@link DecoratorShapes}
     
        * @default ej.datavisualization.Diagram.DecoratorShapes.Arrow
        * @type {enum}
        * @example
        * &lt;div id="diagramcontent"&gt;&lt;/div&gt;<br>
        * &lt;script&gt;
        * var link=[];
        * link={[targetDecorator:{shape:"arrow"}]};
        * $("#diagramcontent").ejDiagram({connectors:link});
        * &lt;/script&gt
        * @alias ejDiagram#connectors->sourceDecorator->shape
    
        * @memberof ejDiagram
        * @instance
        */

                shape: ej.datavisualization.Diagram.DecoratorShapes.Arrow,
                /** This allows to set the width of the decorator in connector
            * @default 8
            * @type {Number}
            * @example
            * &lt;div id="diagramcontent"&gt;&lt;/div&gt;<br>
            * &lt;script&gt;
            * var link=[];
            * link={[targetDecorator:{width:10}]};
            * $("#diagramcontent").ejDiagram({connectors:link});
            * &lt;/script&gt
             * @alias ejDiagram#connectors->sourceDecorator->width
             
            * @memberof ejDiagram
            * @instance
            */
                width: 8,
                /** This allows to set the height of the decorator in connector
                 * @default 8
                 * @type {Number}
                 * @example
                 * &lt;div id="diagramcontent"&gt;&lt;/div&gt;<br>
                 * &lt;script&gt;
                 * var link=[];
                 * link={[targetDecorator:{height:10}]};
                 * $("#diagramcontent").ejDiagram({connectors:link});
                 * &lt;/script&gt
                   * @alias ejDiagram#connectors->sourceDecorator->height
              
              * @memberof ejDiagram
                 * @instance
                 */
                height: 8,
                /** Sets the border color of the decorator
            * @default "black"
            * @type {String}
            * @example
            * &lt;div id="diagramcontent"&gt;&lt;/div&gt;<br>
            * &lt;script&gt;
            * var link=[];
            * link={[targetDecorator:{borderColor: "green"}]};
            * $("#diagramcontent").ejDiagram({connectors:link});
            * &lt;/script&gt
              * @alias ejDiagram#connectors->sourceDecorator->borderColor
              
             * @memberof ejDiagram
            * @instance
            */
                borderColor: "black",
                /** The color which sets inside the decorator 
            * @default "black"
            * @type {String}
            * @example
            * &lt;div id="diagramcontent"&gt;&lt;/div&gt;<br>
            * &lt;script&gt;
            * var link=[];
            * link={[targetDecorator:{fillColor: "green"}]};
            * $("#diagramcontent").ejDiagram({connectors:link});
            * &lt;/script&gt
             * @alias ejDiagram#connectors->sourceDecorator->fillColor
             
            * @memberof ejDiagram
            * @instance
            */
                fillColor: "black",
                /** Path data to draw custom shape for decorator
            * @default ""
            * @type {String}
            * @example
            * &lt;div id="diagramcontent"&gt;&lt;/div&gt;<br>
            * &lt;script&gt;
            * var link=[];
            * link={[targetDecorator:{pathData: "M 269.711,29.3333C 269.711,44.061 257.772,56 243.044,56z"}]};
            * $("#diagramcontent").ejDiagram({connectors:link});
            * &lt;/script&gt
            * @alias ejDiagram#connectors->sourceDecorator->pathData
            
            * @memberof ejDiagram
            * @instance
            */
                pathData: ""
            },
        /** Describes the collection of segments
             * @default [ Segment ]
             * @type {Array}
             * @example
             * &lt;div id="diagramcontent"&gt;&lt;/div&gt;<br>
             * &lt;script&gt;
             * var connectors = [];
             * var point= {x:20,y:20};
             * connectors=[{segments: [{type:"straight",point:point}]}];
             * $("#diagramcontent").ejDiagram({connectors:connectors});
             * &lt;/script&gt
             * @alias ejDiagram#connectors->segments
             * @memberof ejDiagram
             * @instance
             */
        segments: [ej.datavisualization.Diagram.Segment({ type: "straight" })],
        
                /** Describes the sourcePoint of connector
         * @default Point
         * @type {Point}
         * @example
         * &lt;div id="diagramcontent"&gt;&lt;/div&gt;<br>
         * &lt;script&gt;
         * var connectors = [];
         * var sourcePoint= {x:20,y:20};
         * connectors=[{ name: "connector1",sourcePoint: { x: 450, y: 150 }}];
        * $("#diagramcontent").ejDiagram({connectors:connectors});
         * &lt;/script&gt
         * @alias ejDiagram#connectors->sourcePoint
         * @memberof ejDiagram
         * @instance
         */
        sourcePoint: ej.datavisualization.Diagram.Point(),
                  /** Describes the targetPoint of connector
       * @default Point
       * @type {Point}
       * @example
       * &lt;div id="diagramcontent"&gt;&lt;/div&gt;<br>
       * &lt;script&gt;
       * var connectors = [];
       * var targetPoint= {x:20,y:20};
       * connectors=[{ name: "connector1",targetPoint: { x: 450, y: 150 }}];
       * $("#diagramcontent").ejDiagram({connectors:connectors});
       * &lt;/script&gt
       * @alias ejDiagram#connectors->targetPoint
       * @memberof ejDiagram
       * @instances
       */
        targetPoint: ej.datavisualization.Diagram.Point(),
        /**Sets the color of the connector
          * @default black
          * @type {String}
          * @example
          * &lt;div id="diagramcontent"&gt;&lt;/div&gt;<br>
          * &lt;script&gt;
          * var connectors = [];
          * connectors=[{lineColor:"blue"}];
          * $("#diagramcontent").ejDiagram({connectors:connectors});
          * &lt;/script&gt
              * @alias ejDiagram#connectors->lineColor
          * @memberof ejDiagram
          * @instance
          */
        lineColor: "black",
        /**Sets the width of the line
          * @default 1
          * @type {Int}
          * @example
          * &lt;div id="diagramcontent"&gt;&lt;/div&gt;<br>
          * &lt;script&gt;
          * var connectors = [];
          * connectors=[{lineWidth:4}];
          * $("#diagramcontent").ejDiagram({connectors:connectors});
          * &lt;/script&gt
              * @alias ejDiagram#connectors->lineWidth
          * @memberof ejDiagram
          * @instance
          */
        lineWidth: 1,
        /** Enables or disables the behaviour of connector see{@link ConnectorConstraints}
          * @default ConnectorConstraints.Default
          * @type {Constraints}
          * @example
          * &lt;div id="diagramcontent"&gt;&lt;/div&gt;<br>
          * &lt;script&gt;
          * var connectors = [];
          * connectors=[{constraints: ej.datavisualization.Diagram.ConnectorConstraints.Select}];
          * $("#diagramcontent").ejDiagram({connectors:connectors});
          * &lt;/script&gt
              * @alias ejDiagram#connectors->constraints
          * @memberof ejDiagram
          * @instance
          */
        constraints: ej.datavisualization.Diagram.ConnectorConstraints.Default,
        /**The transparency of the connector
          * @default 1
          * @type {Int}
          * @example
          * &lt;div id="diagramcontent"&gt;&lt;/div&gt;<br>
          * &lt;script&gt;
          * var connectors = [];
          * connectors=[{opacity:0.6}];
          * $("#diagramcontent").ejDiagram({connectors:connectors});
          * &lt;/script&gt
              * @alias ejDiagram#connectors->opacity
          * @memberof ejDiagram
          * @instance
          */
        opacity: 1,
          /**parent name of the connector
                * @default ""
                * @type {String}
                * @example
                * &lt;div id="diagramcontent"&gt;&lt;/div&gt;<br>
                * &lt;script&gt;
                * var connector = [ {name: "rect",parent:"group1"}];
                * &lt;/script&gt
                * @alias ejDiagram#connectors->parent
                * @memberof ejDiagram
                * @instance
                */
        parent: "",

        /**The labels property of connectors
                * @default ""
                * @type {Array}
                * @example
                * Refer nodes label for label properties details <br>
                
                * &lt;div id="diagramcontent"&gt;&lt;/div&gt;<br>
                * &lt;script&gt;
                * var label = [];
                * label = [{ "text": "Connector1", "fontColor": "Red"}];
                * $("#diagramcontent").ejDiagram({label:label});
                * &lt;/script&gt
                * @alias ejDiagram#connectors->labels
                * @memberof ejDiagram
                * @instance
                */
        labels: [],
        /** Zorder of the connector
          * @default 0
          * @type {Int}
          * @example
          * &lt;div id="diagramcontent"&gt;&lt;/div&gt;<br>
          * &lt;script&gt;
          * var connectors= [];
          * connectors=[{connectors: 1}];
          * $("#diagramcontent").ejDiagram({connectors:connectors});
          * &lt;/script&gt
              * @alias ejDiagram#connectors->zOrder
          * @memberof ejDiagram
          * @instance
          */
        zOrder: 0,
        /** The padding added to the line to detect line during interaction
          * @default 10
          * @type {Int}
          * @example
          * &lt;div id="diagramcontent"&gt;&lt;/div&gt;<br>
          * &lt;script&gt;
          * var connectors= [];
          * connectors=[{lineHitPadding: 30}];
          * $("#diagramcontent").ejDiagram({connectors:connectors});
          * &lt;/script&gt
              * @alias ejDiagram#connectors->lineHitPadding
          * @memberof ejDiagram
          * @instance
          */
        lineHitPadding: 10,
        /**To provide/save extra information about Node
          * @default null
          * @type {Object}
          * @example
          * &lt;div id="diagramcontent"&gt;&lt;/div&gt;<br>
          * &lt;script&gt;
          * var connectors = [];
          * var addInfo = {Name1:"Connector1",Connector2:"Connector3"}
          * connectors=[{addInfo: addInfo}];
          * $("#diagramcontent").ejDiagram({connectors:connectors});
          * &lt;/script&gt
              * @alias ejDiagram#connectors->addInfo
          * @memberof ejDiagram
          * @instance
          */
        addInfo: {},
        /** The target node of the connector
          * @default null
          * @type {Object}
          * @example
          * &lt;div id="diagramcontent"&gt;&lt;/div&gt;<br>
          * &lt;script&gt;
          * var nodes =[];
          * var connectors = [];
          * nodes = {name: "openBrowses", width: 175, height: 60};
          * connectors=[{targetNode: "openBrowses"}];
          * $("#diagramcontent").ejDiagram({connectors:connectors});
          * &lt;/script&gt
              * @alias ejDiagram#connectors->targetNode
          * @memberof ejDiagram
          * @instance
          */
        targetNode: null,
        /** The targetPort of the connector
          * @default null
          * @type {Object}
          * @example
          * &lt;div id="diagramcontent"&gt;&lt;/div&gt;<br>
          * &lt;script&gt;
          * var nodes = [];
          * var ports = [];
          * var connectors = [];
          * nodes = {name: "openBrowses", width: 175, height: 60};
          * ports = { offset: { x: 0, y: 0.5 }, name: "aport" };
          * connectors=[{targetPort: "aport"}];
          * $("#diagramcontent").ejDiagram({connectors:connectors});
          * &lt;/script&gt
              * @alias ejDiagram#connectors->targetPort
          * @memberof ejDiagram
          * @instance
          */
        targetPort: null,
        /** The sourceNode of the connector
          * @default null
          * @type {Object}
          * @example
          * &lt;div id="diagramcontent"&gt;&lt;/div&gt;<br>
          * &lt;script&gt;
          * var nodes = [];
          * var connectors = [];
          * nodes = {name: "openBrowses", width: 175, height: 60};
          * connectors=[{sourceNode: "openBrowses"}];
          * $("#diagramcontent").ejDiagram({connectors:connectors});
          * &lt;/script&gt
              * @alias ejDiagram#connectors->sourceNode
          * @memberof ejDiagram
          * @instance
          */
        sourceNode: null,
        /** The sourcePort of the connector
          * @default null
          * @type {Object}
          * @example
          * &lt;div id="diagramcontent"&gt;&lt;/div&gt;<br>
          * &lt;script&gt;
          * var nodes = [];
          * var ports = [];
          * var connectors = [];
          * nodes = {name: "openBrowses", width: 175, height: 60};
          * ports = { offset: { x: 0, y: 0.5 }, name: "bport" };
          * connectors=[{sourcePort: "bport"}];
          * $("#diagramcontent").ejDiagram({connectors:connectors});
          * &lt;/script&gt
              * @alias ejDiagram#connectors->sourcePort
          * @memberof ejDiagram
          * @instance
          */
        sourcePort: null,
        /** To set margin left for the connector.Applicable if the parent is group.
         * @default ""
         * @type {Int}
          * @example
          * &lt;div id="diagramcontent"&gt;&lt;/div&gt;<br>
          * &lt;script&gt;
          * var connectors = [];
          * connectors=[{name:"connector",marginLeft: 1}];
          * $("#diagramcontent").ejDiagram({connectors:connectors});
          * &lt;/script&gt
          * @alias ejDiagram#connectors->marginLeft
          * @memberof ejDiagram
          * @instance
         */
        marginLeft: 0,
        /** To set top margin for the connector.Applicable if the parent is group.
        * @default 0
        * @type {Int}
         * @example
         * &lt;div id="diagramcontent"&gt;&lt;/div&gt;<br>
         * &lt;script&gt;
         * var connectors = [];
         * connectors=[{marginTop: 1}];
         * $("#diagramcontent").ejDiagram({connectors:connectors});
         * &lt;/script&gt
         * @alias ejDiagram#connectors->marginTop
         * @memberof ejDiagram
         * @instance
        */
        marginTop: 0,
        /** To set right margin for the connector.Applicable if the parent is group.
        * @default 0
        * @type {Int}
         * @example
         * &lt;div id="diagramcontent"&gt;&lt;/div&gt;<br>
         * &lt;script&gt;
         * var connectors = [];
         * connectors=[{marginRight: 1}];
         * $("#diagramcontent").ejDiagram({connectors:connectors});
         * &lt;/script&gt
         * @alias ejDiagram#connectors->marginRight
         * @memberof ejDiagram
         * @instance
        */
        marginRight: 0,
        /** To set margin bottom for the connector.Applicable if the parent is group.
     * @default 0
     * @type {Int}
      * @example
      * &lt;div id="diagramcontent"&gt;&lt;/div&gt;<br>
      * &lt;script&gt;
      * var connectors = [];
      * connectors=[{marginBottom: 1}];
      * $("#diagramcontent").ejDiagram({connectors:connectors});
      * &lt;/script&gt
      * @alias ejDiagram#connectors->marginBottom
      * @memberof ejDiagram
      * @instance
     */
        marginBottom: 0,
        /** To set the horizontal alignment for connector.Applicable if the parent is group.
         * @default HorizontalAlignment.Left
         * @type {HorizontalAlignment}
         * @example
         * &lt;div id="diagramcontent"&gt;&lt;/div&gt;<br>
         * &lt;script&gt;
         * var connectors = [];
         * connectors=[{HorizontalAlignment: ej.datavisualization.Diagram.HorizontalAlignment.Left}];
         * $("#diagramcontent").ejDiagram({connectors:connectors});
         * &lt;/script&gt
         * @alias ejDiagram#nodes->horizontalAlign
         * @memberof ejDiagram
         * @instance
         */
        horizontalAlign: ej.datavisualization.Diagram.HorizontalAlignment.Left,
        /** To set the vertical alignment for connector.Applicable if the parent is group.
        * @default VerticalAlignment.Top
        * @type {VerticalAlignment}
        * @example
        * &lt;div id="diagramcontent"&gt;&lt;/div&gt;<br>
        * &lt;script&gt;
        * var connectors = [];
        * connectors=[{VerticalAlignment: ej.datavisualization.Diagram.VerticalAlignment.Top}];
        * $("#diagramcontent").ejDiagram({connectors:connectors});
        * &lt;/script&gt
        * @alias ejDiagram#nodes->verticalAlign
        * @memberof ejDiagram
        * @instance
        */
        verticalAlign: ej.datavisualization.Diagram.VerticalAlignment.Top,
        /** Defines the radius of the rounded corner
        * @default 0
        * @type {Integer}
        * @example
        * &lt;div id="diagramcontent"&gt;&lt;/div&gt;<br>
        * &lt;script&gt;
        * var connectors=[];
        * connectors=[{cornerRadius:2}];
        * $("#diagramcontent").ejDiagram({connectors:connectors});
        * &lt;/script&gt
      * @alias ejDiagram#connectors->cornerRadius
        * @memberof ejDiagram
        * @instance
        */
        cornerRadius: 0,
        /** Defines the bridge spacing of the connector
       * @default 0
       * @type {Integer}
       * @example
       * &lt;div id="diagramcontent"&gt;&lt;/div&gt;<br>
       * &lt;script&gt;
       * var connectors=[];
       * connectors=[{bridgeSpace:15}];
       * $("#diagramcontent").ejDiagram({connectors:connectors});
       * &lt;/script&gt
     * @alias ejDiagram#connectors->bridgeSpace
       * @memberof ejDiagram
       * @instance
       */
        bridgeSpace: 10,

        /** Defines the space between node and connector's source point 
       * @default 0
       * @type {Integer}
       * @example
       * &lt;div id="diagramcontent"&gt;&lt;/div&gt;<br>
       * &lt;script&gt;
       * var connectors=[];
       * connectors=[{sourceIndent:5}];
       * $("#diagramcontent").ejDiagram({connectors:connectors});
       * &lt;/script&gt
       * @alias ejDiagram#connectors->sourceIndent
       * @memberof ejDiagram
       * @instance
       */
        sourceIndent: 0,
        /** Defines the space between node and connector's target point 
       * @default 0
       * @type {Integer}
       * @example
       * &lt;div id="diagramcontent"&gt;&lt;/div&gt;<br>
       * &lt;script&gt;
       * var connectors=[];
       * connectors=[{targetIndent:5}];
       * $("#diagramcontent").ejDiagram({connectors:connectors});
       * &lt;/script&gt
       * @alias ejDiagram#connectors->targetIndent
       * @memberof ejDiagram
       * @instance
       */
        targetIndent: 0
    };
    ej.datavisualization.Diagram.Connector = function (options) {
        options.targetDecorator = ej.datavisualization.Diagram.Decorator(options.targetDecorator || {});
        options.sourceDecorator = ej.datavisualization.Diagram.Decorator(options.sourceDecorator || { shape: ej.datavisualization.Diagram.DecoratorShapes.None });
        options.segments = options.segments || [ej.datavisualization.Diagram.Segment({ type: "straight" })];
        if (options.segments.length) {
            options.sourcePoint = options.sourcePoint || ej.datavisualization.Diagram.Point();
            options.targetPoint = options.targetPoint || ej.datavisualization.Diagram.Point();
            ej.datavisualization.Diagram.Util._initializeSegments(options, options.defaultType);
        }
        if (options.labels)
            for (var i = 0; i < options.labels.length; i++) {
                options.labels[i] = ej.datavisualization.Diagram.Label(options.labels[i]);
            }
        return $.extend(true, {}, ej.datavisualization.Diagram.ConnectorDefaults, options);
    };
    //#endregion

    // /**
    //* @namespace ej
    //* @class ejDiagram.Container    
    //* @classdesc Container is a container of Nodes and Connectors as a single entity
    //* @example 
    //* &lt;div id="diagram"&gt;&lt;/div&gt;<br>
    //* &lt;script&gt;
    //* //Create Diagram
    //$("#diagram").ejDiagram();
    //* &lt;/script&gt
    //*/
    //#region Container
    ej.datavisualization.Diagram.ContainerDefaults = {
        /**To indicate the container type.Applicable if the group as container.
        * @default "Canvas"
          * @type {string}
          * @example
          * &lt;div id="diagramcontent"&gt;&lt;/div&gt;<br>
          * &lt;script&gt;
          * var container = ej.datavisualization.Diagram.ContainerType.Canvas
          * container.type = "linear; 
          * var nodes = [{name:"Node1",container : container}];
          * $("#diagramcontent").ejDiagram({nodes:nodes});
          * &lt;/script&gt
          * @alias ejDiagram#nodes->container->type
          * @memberof ejDiagram
          * @instance
        */
        type: ej.datavisualization.Diagram.ContainerType.Canvas,
        /**To indicate the container orientation.Applicable if the group as container.
        * @default "vertical"
         * @type {string}
         * @example
         * &lt;div id="diagramcontent"&gt;&lt;/div&gt;<br>
         * &lt;script&gt;
         * var container = ej.datavisualization.Diagram.ContainerType.Canvas
         * container.orientation = "vertical; 
         * var nodes = [{name:"Node1",container : container}];
         * $("#diagramcontent").ejDiagram({nodes:nodes});
         * &lt;/script&gt
         * @alias ejDiagram#nodes->container->orientation
         * @memberof ejDiagram
         * @instance
       */
        orientation: "vertical",
    };
    ej.datavisualization.Diagram.Container = function (options) {
        return $.extend(true, {}, ej.datavisualization.Diagram.ContainerDefaults, options);
    };
    // #endregion
    // /**
    //* @namespace ej
    //* @class ejDiagram.Group    
    //* @classdesc Group is a container of Nodes and Connectors as a single entity
    //* @example 
    //* &lt;div id="diagram"&gt;&lt;/div&gt;<br>
    //* &lt;script&gt;
    //* //Create Diagram
    //$("#diagram").ejDiagram();
    //* &lt;/script&gt
    //*/
    //#region Group
    ej.datavisualization.Diagram.GroupDefaults = {
        /** To set node as group
          * @default true
          * @type {Boolean}
          * @example
          * &lt;div id="diagramcontent"&gt;&lt;/div&gt;<br>
          * &lt;script&gt;
          * var nodes = [];
          * nodes=[{name:"GroupNode",isGroup:true}];
          * $("#diagramcontent").ejDiagram({nodes:nodes});
          * &lt;/script&gt
           * @alias ejDiagram#nodes->isGroup
          * @memberof ejDiagram
          * @instance
          */
        isGroup: true,
        /** Collection of children in group node
          * @default ""
          * @type {Array}
          * @example
          * &lt;div id="diagramcontent"&gt;&lt;/div&gt;<br>
          * &lt;script&gt;
          * var nodes = [];
          * var children = ["Node1","Node2"];
          * nodes=[{name:"GroupNode",isGroup:true,children:children}];
          * $("#diagramcontent").ejDiagram({nodes:nodes});
          * &lt;/script&gt
           * @alias ejDiagram#nodes->children
          * @memberof ejDiagram
          * @instance
          */
        children: [],
        /**This is used to set whether we can ungroup the group or not
          * @default true
          * @type {Boolean}
          * @example
          * &lt;div id="diagramcontent"&gt;&lt;/div&gt;<br>
          * &lt;script&gt;
          * var nodes = [];
          * var children = ["Node1","Node2"];
          * nodes=[{name:"GroupNode",isGroup:true,children:children,canUngroup:false}];
          * $("#diagramcontent").ejDiagram({nodes:nodes});
          * &lt;/script&gt
           * @alias ejDiagram#nodes->canUngroup
          * @memberof ejDiagram
          * @instance
          */
        canUngroup: true,
        /** To indicating whether this element can be used as the target of a drop operation. Applicable if the type is group.
           * @default true
           * @type {Boolean}
           * @example
           * &lt;div id="diagramcontent"&gt;&lt;/div&gt;<br>
           * &lt;script&gt;
           * var nodes = [];
           * nodes=[{name:"GroupNode",allowDrop:true}];
           * $("#diagramcontent").ejDiagram({nodes:nodes});
           * &lt;/script&gt
            * @alias ejDiagram#nodes->allowDrop
           * @memberof ejDiagram
           * @instance
           */
        allowDrop: true,
        /** To indicating whether this element can be used as container.Applicable only if the object as Group.
          * @default null
          * @type {obj}
          * @example
          * &lt;div id="diagramcontent"&gt;&lt;/div&gt;<br>
          * &lt;script&gt;
          * var nodes = [];
          * nodes=[{name:"GroupNode",container:{}}];
          * $("#diagramcontent").ejDiagram({nodes:nodes});
          * &lt;/script&gt
           * @alias ejDiagram#nodes->container
          * @memberof ejDiagram
          * @instance
          */
        container: null,
        /** To set padding left value for the group.
         * @default 0
         * @type {Int}
          * @example
          * &lt;div id="diagramcontent"&gt;&lt;/div&gt;<br>
          * &lt;script&gt;
          * var nodes = [];
          * nodes=[{paddingLeft: 1}];
          * $("#diagramcontent").ejDiagram({nodes:nodes});
          * &lt;/script&gt
          * @alias ejDiagram#nodes->paddingLeft
          * @memberof ejDiagram
          * @instance
         */
        paddingLeft: 0,
        /** To set Top padding value for the group. Applicable if the type is group.
       * @default 0
       * @type {Int}
        * @example
        * &lt;div id="diagramcontent"&gt;&lt;/div&gt;<br>
        * &lt;script&gt;
        * var nodes = [];
        * nodes=[{paddingTop: 1}];
        * $("#diagramcontent").ejDiagram({nodes:nodes});
        * &lt;/script&gt
        * @alias ejDiagram#nodes->paddingTop
        * @memberof ejDiagram
        * @instance
       */
        paddingTop: 0,
        /** To set Right padding value for the group. Applicable if the type is group.
     * @default 0
     * @type {Int}
      * @example
      * &lt;div id="diagramcontent"&gt;&lt;/div&gt;<br>
      * &lt;script&gt;
      * var nodes = [];
      * nodes=[{paddingRight: 1}];
      * $("#diagramcontent").ejDiagram({nodes:nodes});
      * &lt;/script&gt
      * @alias ejDiagram#nodes->paddingRight
      * @memberof ejDiagram
      * @instance
     */
        paddingRight: 0,
        /** To set bottom padding value for the group. Applicable if the type is group.
   * @default 0
   * @type {Int}
    * @example
    * &lt;div id="diagramcontent"&gt;&lt;/div&gt;<br>
    * &lt;script&gt;
    * var nodes = [];
    * nodes=[{paddingBottom: 1}];
    * $("#diagramcontent").ejDiagram({nodes:nodes});
    * &lt;/script&gt
    * @alias ejDiagram#nodes->paddingBottom
    * @memberof ejDiagram
    * @instance
   */
        paddingBottom: 0,
        /** To set minimum width for the node. Applicable if the type is group.
         * @default 0
         * @type {Int}
          * @example
          * &lt;div id="diagramcontent"&gt;&lt;/div&gt;<br>
          * &lt;script&gt;
          * var nodes = [];
          * nodes=[{minWidth: 1}];
          * $("#diagramcontent").ejDiagram({nodes:nodes});
          * &lt;/script&gt
          * @alias ejDiagram#nodes->minWidth
          * @memberof ejDiagram
          * @instance
         */
        minWidth: 0,
        /** To set maximum width for the group. Applicable if the type is group.
         * @default 0
         * @type {Int}
          * @example
          * &lt;div id="diagramcontent"&gt;&lt;/div&gt;<br>
          * &lt;script&gt;
          * var nodes = [];
          * nodes=[{maxWidth: 1}];
          * $("#diagramcontent").ejDiagram({nodes:nodes});
          * &lt;/script&gt
          * @alias ejDiagram#nodes->maxWidth
          * @memberof ejDiagram
          * @instance
         */
        maxWidth: 0,
        /** To set minimum height for the group. Applicable if the type is group.
         * @default 0
         * @type {Int}
          * @example
          * &lt;div id="diagramcontent"&gt;&lt;/div&gt;<br>
          * &lt;script&gt;
          * var nodes = [];
          * nodes=[{minHeight: 1}];
          * $("#diagramcontent").ejDiagram({nodes:nodes});
          * &lt;/script&gt
          * @alias ejDiagram#nodes->minHeight
          * @memberof ejDiagram
          * @instance
         */
        minHeight: 0,
        /** To set maximum height for the group. Applicable if the type is group.
         * @default 0
         * @type {Int}
          * @example
          * &lt;div id="diagramcontent"&gt;&lt;/div&gt;<br>
          * &lt;script&gt;
          * var nodes = [];
          * nodes=[{maxHeight: 1}];
          * $("#diagramcontent").ejDiagram({nodes:nodes});
          * &lt;/script&gt
          * @alias ejDiagram#nodes->maxHeight
          * @memberof ejDiagram
          * @instance
         */
        maxHeight: 0,

    };
    ej.datavisualization.Diagram.Group = function (options) {
        options = options ? options : null;
        options.minWidth = options.minWidth ? options.minWidth : 0;
        options.minHeight = options.minHeight ? options.minHeight : 0;
        options.maxWidth = options.maxWidth ? options.maxWidth : 0;
        options.maxHeight = options.maxHeight ? options.maxHeight : 0;
        options.container = options.container ? ej.datavisualization.Diagram.Container(options.container) : null;
        options.fillColor = options.fillColor ? options.fillColor : "transparent";
        options.borderColor = options.borderColor ? options.borderColor : "transparent";
        return $.extend(true, {}, ej.datavisualization.Diagram.GroupDefaults, ej.datavisualization.Diagram.NodeDefaults, ej.datavisualization.Diagram.NodeBase(options));
    };
    //#endregion

    //#region ZoomCommand
    ej.datavisualization.Diagram.ZoomCommand = {
        ///** ZoomIn operation assigns the value to ,which operation is to be performed
        //  * @default 1
        //  * @type {Int}
        //  * @example
        //  * &lt;div id="diagramcontent"&gt;&lt;/div&gt;<br>
        //  * &lt;script&gt;
        //  * var zoom = ZoomCommand.ZoomIn;
        //  * &lt;/script&gt
        //  * @memberof ejDiagram
        //  * @instance
        //  */
        ZoomIn: 1,
        ///** ZoomOut operation assigns the value to ,which operation is to be performed
        // * @default 2
        // * @type {Int}
        // * @example
        // * &lt;div id="diagramcontent"&gt;&lt;/div&gt;<br>
        // * &lt;script&gt;
        // * var zoom = ZoomCommand.ZoomOut;
        // * &lt;/script&gt
        // * @memberof ejDiagram
        // * @instance
        // */
        ZoomOut: 2
    };
    ej.datavisualization.Diagram.Zoom = function (options) {
        return $.extend(true, {}, { zoomFactor: 0.2, focusPoint: null, zoomCommand: ej.datavisualization.Diagram.ZoomCommand.ZoomIn }, options);
    };
    //#endregion

       /**
   * Enum for the DiagramConstraints in diagram	 
   * @enum {DiagramConstraints}
   * @global 
   */
    ej.datavisualization.Diagram.DiagramConstraints = {
        /**Disables all  DiagramConstraints */
        None: 1 << 0,
        /**Enables/Disables PageEditing */
        PageEditable: 1 << 1,
        /**Enables/Disables Bridging */
        Bridging: 1 << 2,
        /**Enables/Disables Zooming*/
        Zoomable: 1 << 3,
        /**Enables/Disables panning on horizontal axis */
        PannableX: 1 << 4,
        /**Enables/Disables panning on vertical axis */
        PannableY: 1 << 5,
        /**Enables/Disables Panning */
        Pannable: 1 << 4 | 1 << 5,
        /**Enables/Disables undo actions */
        Undoable: 1 << 6,
        /**Enables all Constraints */
        Default: 1 << 1 | 1 << 3 | 1 << 4 | 1 << 5 | 1 << 6
    };

       //#endregion
       /**
   * Enum for the Tool in diagram	 
   * @enum {Tool}
   * @global 
   */
    ej.datavisualization.Diagram.Tool = {
        /**Disables all Tools */
        None: 1 << 0,
        /**Enables/Disables SingleSelect tool */
        SingleSelect: 1 << 1,
       /**Enables/Disables MultiSelect tool */
        MultipleSelect: 1 << 2,
        /**Enables/Disables ZoomPan tool*/
        ZoomPan: 1 << 3,
        /**Enables/Disables DrawOnce tool*/
        DrawOnce: 1 << 4,
        /**Enables/Disables ContinuousDraw tool*/
        ContinuesDraw: 1 << 5
    };
          //#endregion
       /**
   * Enum to specify the ConnectorType in diagram	 
   * @enum {ConnectorType}
   * @global 
   */
    ej.datavisualization.Diagram.ConnectorType = {
      /** Enables/Disables straight line drawing tool*/
        StraightLine: "straightLine",
        /**Enables/Disables orthogonal line drawing tool*/
        OrthogonalLine: "orthogonalLine"
    };

    //  /**
    //* @namespace ej
    //* @class  ejDiagram.UserHandle    
    //* @classdesc Some frequently used commands can be added around the selector though user handles.
    //* @example 
    //* &lt;div id="diagram"&gt;&lt;/div&gt;<br>
    //* &lt;script&gt;
    //* //Create Diagram
    //$("#diagram").ejDiagram();
    //* &lt;/script&gt
    //*/
    //#region UserHandle
    ej.datavisualization.Diagram.UserHandleDefaults = {
        /** The name of the user handles
          * @default ""
          * @type {String}
          * @example
          * &lt;div id="diagramcontent"&gt;&lt;/div&gt;<br>
          * &lt;script&gt;
          * var userHandles = ej.datavisualization.Diagram.UserHandle();
          * userHandles.name = "Clone";
          * $("#diagramcontent").ejDiagram({userHandles:userHandles});
          * &lt;/script&gt
          * @memberof ejDiagram.UserHandle 
          * @instance
          */
        name: "",
        /** To customize the userhandles shape using path data
          * @default ""
          * @type {String}
          * @example
          * &lt;div id="diagramcontent"&gt;&lt;/div&gt;<br>
          * &lt;script&gt;
          * var userHandles = ej.datavisualization.Diagram.UserHandle();
          * userHandles.pathData = "M4.6350084,4.8909971 L4.6350084,9.3649971 9.5480137,9.3649971 9.5480137,4.8909971 z M3.0000062,2.8189973 L11.184016,2.8189973 11.184016,10.999997 3.0000062,10.999997 z M0,0 L7.3649998,0 7.3649998,1.4020001 1.4029988,1.4020001 1.4029988,8.0660002 0,8.0660002 0,1.4020001 0,0.70300276 z";
          * $("#diagramcontent").ejDiagram({userHandles:userHandles});
          * &lt;/script&gt
          * @memberof ejDiagram.UserHandle 
          * @instance
          */
        pathData: "",
        /** The border color of user handles
          * @default ""
          * @type {String}
          * @example
          * &lt;div id="diagramcontent"&gt;&lt;/div&gt;<br>
          * &lt;script&gt;
          * var userHandles = ej.datavisualization.Diagram.UserHandle();
          * userHandles.borderColor = "red";
          * $("#diagramcontent").ejDiagram({userHandles:userHandles});
          * &lt;/script&gt
          * @memberof ejDiagram.UserHandle 
          * @instance
          */
        borderColor: "",
        /**The background color of user handle
          * @default #2382c3
          * @type {String}
          * @example
          * &lt;div id="diagramcontent"&gt;&lt;/div&gt;<br>
          * &lt;script&gt;
          * var userHandles = ej.datavisualization.Diagram.UserHandle();
          * userHandles.backgroundColor = "red";
          * $("#diagramcontent").ejDiagram({userHandles:userHandles});
          * &lt;/script&gt
          * @memberof ejDiagram.UserHandle 
          * @instance
          */
        backgroundColor: "#2382c3",
        /** The Position of user handles to be updated See {@link Position}
          * @name Position
          * @default ej.datavisualization.Diagram.UserHandlePositions.BottomCenter
          * @type {enum}
          * @example
          * &lt;div id="diagramcontent"&gt;&lt;/div&gt;<br>
          * &lt;script&gt;
          * var userHandles = ej.datavisualization.Diagram.UserHandle();
          * userHandles.position = ej.datavisualization.Diagram.UserHandlePositions.BottomLeft;
          * $("#diagramcontent").ejDiagram({userHandles:userHandles});
          * &lt;/script&gt
          * @memberof ejDiagram.UserHandle 
          * @instance
          */
        position: ej.datavisualization.Diagram.UserHandlePositions.BottomCenter,
        /** Customize the color of user handles
          * @default white
          * @type {String}
          * @example
          * &lt;div id="diagramcontent"&gt;&lt;/div&gt;<br>
          * &lt;script&gt;
          * var userHandles = ej.datavisualization.Diagram.UserHandle();
          * userHandles.pathColor = "red";
          * $("#diagramcontent").ejDiagram({userHandles:userHandles});
          * &lt;/script&gt
          * @memberof ejDiagram.UserHandle 
          * @instance
          */
        pathColor: "white",
        /**The tool to be used as a user handle
          * @default ""
          * @type {String}
          * @example
          * &lt;div id="diagramcontent"&gt;&lt;/div&gt;<br>
          * &lt;script&gt;
          * var userHandles = ej.datavisualization.Diagram.UserHandle();
          * userHandles.tool = new CloneTool(cloneHandle.name);
          * function CloneTool(name) { base.call(this, name);this.singleAction = true;} //Properties can be modified
          * $("#diagramcontent").ejDiagram({userHandles:userHandles});
          * &lt;/script&gt
          * @memberof ejDiagram.UserHandle 
          * @instance
          */
        tool: "",
        /**The size of the tool
          * @default 20
          * @type {Int}
          * @example
          * &lt;div id="diagramcontent"&gt;&lt;/div&gt;<br>
          * &lt;script&gt;
          * var userHandles = ej.datavisualization.Diagram.UserHandle();
          * userHandles.size = 30;
          * $("#diagramcontent").ejDiagram({userHandles:userHandles});
          * &lt;/script&gt
          * @memberof ejDiagram.UserHandle 
          * @instance
          */
        size: 20,
        /**Enables or disables the visibility of the user handle
          * @default true
          * @type {Boolean}
          * @example
          * &lt;div id="diagramcontent"&gt;&lt;/div&gt;<br>
          * &lt;script&gt;
          * var userHandles = ej.datavisualization.Diagram.UserHandle();
          * userHandles.visibility = false;
          * $("#diagramcontent").ejDiagram({userHandles:userHandles});
          * &lt;/script&gt
          * @memberof ejDiagram.UserHandle 
          * @instance
          */
        visible: true,
        /** The functionality to enables or disables the multiple selection
          * @default false
          * @type {Boolean}
          * @example
          * &lt;div id="diagramcontent"&gt;&lt;/div&gt;<br>
          * &lt;script&gt;
          * var userHandles = ej.datavisualization.Diagram.UserHandle();
          * userHandles.enableMultiSelection = false;
          * $("#diagramcontent").ejDiagram({userHandles:userHandles});
          * &lt;/script&gt
          * @memberof ejDiagram.UserHandle 
          * @instance
          */
        enableMultiSelection: false
    };
    ej.datavisualization.Diagram.UserHandle = function (options) {
        return $.extend(true, {}, ej.datavisualization.Diagram.UserHandleDefaults, options);
    };
    //#endregion
    //#region Swimlane
    ej.datavisualization.Diagram.SwimLaneDefaults = {
        /** The header of the swimlane . Applicable if the type is group.
          * @default "Null"
          * @type {obj}
          * @example
          * &lt;div id="diagramcontent"&gt;&lt;/div&gt;<br>
          * &lt;script&gt;
          * $("#diagramcontent").ejDiagram({nodes:{type: "swimlane",name: "swimlane",  header: {text: "Header", width:50}}});
          * &lt;/script&gt
           * @alias ejDiagram#nodes->header
          * @memberof ejDiagram
          * @instance
          */
        header: {
            /** Used to specify the width of the swimlane header*/
            width: 30,
            /** Used to specify the text of the swimlane header*/
            text: "",
        },
        /** The color to be filled inside the swimlane. Applicable if type is swimlane.
          * @default "White"
          * @type {String}
          * @example
          * &lt;div id="diagramcontent"&gt;&lt;/div&gt;<br>
          * &lt;script&gt;
           * $("#diagramcontent").ejDiagram({nodes:{type: "swimlane",name: "swimlane",  fillColor: "red"}});
          * &lt;/script&gt
           * @alias ejDiagram#nodes->fillColor
          * @memberof ejDiagram
          * @instance
          */
        fillColor: "transparent",
        /** The orientation type of the swimlane. Applicable if type is swimlane.
          * @default "vertical"
          * @type {String}
          * @example
          * &lt;div id="diagramcontent"&gt;&lt;/div&gt;<br>
          * &lt;script&gt;
           * $("#diagramcontent").ejDiagram({nodes:{type: "swimlane",name: "swimlane",  orientation: "vertical"}});
          * &lt;/script&gt
           * @alias ejDiagram#nodes->orientation
          * @memberof ejDiagram
          * @instance
          */
        orientation: "vertical", isGroup: true,
        /** Indicates class as swimlane. Applicable if type is swimlane.
          * @default true
          * @type {Boolean}
          * @example
          * &lt;div id="diagramcontent"&gt;&lt;/div&gt;<br>
          * &lt;script&gt; 
          * $("#diagramcontent").ejDiagram({nodes:{type: "swimlane",name: "swimlane",  isSwimlane: true}});
          * &lt;/script&gt
           * @alias ejDiagram#nodes->isSwimlane
          * @memberof ejDiagram
          * @instance
          */
        isSwimlane: true,
        /** Align swimlane position along X-axis. Applicable if type is swimlane.
          * @default 0
          * @type {Int}
          * @example
          * &lt;div id="diagramcontent"&gt;&lt;/div&gt;<br>
          * &lt;script&gt; 
          * $("#diagramcontent").ejDiagram({nodes:{type: "swimlane",name: "swimlane",offsetX:100}});
          * &lt;/script&gt
           * @alias ejDiagram#nodes->offsetX
          * @memberof ejDiagram
          * @instance
          */
        offsetX: 100,
        /** Align swimlane position along Y-axis. Applicable if type is swimlane.
         * @default 0
         * @type {Int}
         * @example
         * &lt;div id="diagramcontent"&gt;&lt;/div&gt;<br>
         * &lt;script&gt; 
         * $("#diagramcontent").ejDiagram({nodes:{type: "swimlane",name: "swimlane",offsetY:100}});
         * &lt;/script&gt
          * @alias ejDiagram#nodes->offsetY
         * @memberof ejDiagram
         * @instance
         */
        offsetY: 100,
        /** To set minimum height for the swimlane. Applicable if type is swimlane.
         * @default 0
         * @type {Int}
          * @example
          * &lt;div id="diagramcontent"&gt;&lt;/div&gt;<br>
          * &lt;script&gt;
         * $("#diagramcontent").ejDiagram({nodes:{type: "swimlane",name: "swimlane",minHeight:100}});
          * &lt;/script&gt
          * @alias ejDiagram#nodes->minHeight
          * @memberof ejDiagram
          * @instance
         */
        minHeight: 100,
        /** To set maximum height for the swimlane. Applicable if type is swimlane.
         * @default 0
         * @type {Int}
          * @example
          * &lt;div id="diagramcontent"&gt;&lt;/div&gt;<br>
          * &lt;script&gt;
          * $("#diagramcontent").ejDiagram({nodes:{type: "swimlane",name: "swimlane",maxHeight:100}});
          * &lt;/script&gt
          * @alias ejDiagram#nodes->maxHeight
          * @memberof ejDiagram
          * @instance
         */
        maxHeight: 100,
        /** To set minimum width for the swimlane. Applicable if type is swimlane.
         * @default 0
         * @type {Int}
        * @example
        * &lt;div id="diagramcontent"&gt;&lt;/div&gt;<br>
        * &lt;script&gt;
        * $("#diagramcontent").ejDiagram({nodes:{type: "swimlane",name: "swimlane",minWidth:100}});
        * &lt;/script&gt
        * @alias ejDiagram#nodes->minWidth
        * @memberof ejDiagram
        * @instance
         */
        minWidth: 100,
        /** To set maximum width for the swimlane. Applicable if type is swimlane.
         * @default 0
         * @type {Int}
          * @example
          * &lt;div id="diagramcontent"&gt;&lt;/div&gt;<br>
          * &lt;script&gt; 
          * $("#diagramcontent").ejDiagram({nodes:{type: "swimlane",name: "swimlane",maxWidth:100}});
          * &lt;/script&gt
          * @alias ejDiagram#nodes->maxWidth
          * @memberof ejDiagram
          * @instance
         */
        maxWidth: 100,
        /** Collection of Lanes in swimlane. Applicable if type is swimlane.
           * @default []
          * @type {Array}
          * @example
          * &lt;div id="diagramcontent"&gt;&lt;/div&gt;<br>
          * &lt;script&gt;
          * var lanes = []; 
          * lanes=[{["Lane1","Lane1"]}];
          * $("#diagramcontent").ejDiagram( );
          * var nodes = [ {type: "swimlane",name: "swimlane",lanes: []}
          * &lt;/script&gt
           * @alias ejDiagram#nodes->lanes
          * @memberof ejDiagram
          * @instance
          */
        lanes: [],
        /** Collection of phases in swimlane node. Applicable if type is swimlane.
          * @default []          
          * @type {Array}
          * @example
          * &lt;div id="diagramcontent"&gt;&lt;/div&gt;<br>
          * &lt;script&gt;
          * var phases = []; 
          * phases=[{["phase1","phase2"]}];
          * $("#diagramcontent").ejDiagram( );
          * var nodes = [ {type: "swimlane",name: "swimlane",phases: []}
          * &lt;/script&gt
           * @alias ejDiagram#nodes->phases
          * @memberof ejDiagram
          * @instance
          */
        phases: [],
    };
    ej.datavisualization.Diagram.SwimLane = function (options) {
        return $.extend(true, {}, ej.datavisualization.Diagram.SwimLaneDefaults, options);
    };
    //#endregion  
    //#region Lane
    ej.datavisualization.Diagram.LaneDefaults = {
        header: { width: 30, text: "", }, name: "", children: [], isLane: true, orientation: "horizontal",
    };
    ej.datavisualization.Diagram.Lane = function (options) {
        return $.extend(true, {}, ej.datavisualization.Diagram.LaneDefaults, options);
    };
    //#endregion  
    //#region phase
    ej.datavisualization.Diagram.phaseDefaults = {
        name: "", type: "phase", offset: 0, lineWidth: 1, lineDashArray: "", lineColor: "black", parent: "", orientation: "",
    };
    ej.datavisualization.Diagram.phase = function (options) {
        return $.extend(true, {}, ej.datavisualization.Diagram.phaseDefaults, options);
    };
    //#endregion 
    ej.datavisualization.Diagram.SwimLaneHelper = {
        //#region diagram
        _initSwimLane: function (obj, diagram, headAdded) {
            var canvas = null;
            var lanes = null;
            var phases = this._initphases(obj, diagram);
            lanes = this._initLanes(obj, diagram, headAdded);
            var childern = [];
            var header = null;
            if (!obj.header) {
                obj.header = { text: "", height: 0, fillColor: "white" };
            }
            if (obj.orientation == "vertical") {
                var height = obj.header.height;
            } else {
                var width = obj.header.width;

            }
            if (obj.header) {
                header = ej.datavisualization.Diagram.Node({
                    type: "node",
                    name: obj.name + "_header_swimlane",
                    labels: [
                        {
                            text: obj.header.text ? obj.header.text : "",
                            offset: { x: 0.5, y: .5 },
                            fontColor: obj.header.fontColor ? obj.header.fontColor : "black",
                            fontSize: obj.header.fontSize ? obj.header.fontSize : 12,
                            fontFamily: obj.header.fontFamily ? obj.header.fontFamily : "Arial",
                            bold: obj.header.bold ? obj.header.bold : false,
                            italic: obj.header.italic ? obj.header.italic : false,
                            textDecoration: obj.header.textDecoration ? obj.header.textDecoration : "none",
                        },
                    ],
                    height: height ? height : 50,
                    width: width ? width : 50,
                    fillColor: obj.header.fillColor ? obj.header.fillColor : "white",
                    rotateAngle: 0,
                    constraints: ej.datavisualization.Diagram.NodeConstraints.Default ^ ej.datavisualization.Diagram.NodeConstraints.Select ^ ej.datavisualization.Diagram.NodeConstraints.Connect,
                    horizontalAlign: "stretch",
                    parent: obj.name,
                });
            }

            if (header) {
                diagram.model.nodes.push(header);
                diagram.nameTable[header.name] = header;
                childern.push(header.name);
            }

            var hAlign = null, vAlign = null;
            if (obj.orientation == "vertical")
                vAlign = "stretch";
            else
                hAlign = "stretch";

            var orientation;
            if (obj.orientation == "horizontal")
                orientation = "vertical";
            else
                orientation = "horizontal";


            var stack = ej.datavisualization.Diagram.Group({
                name: obj.name + "_stack",
                children: lanes,
                parent: obj.name,
                container: { type: ej.datavisualization.Diagram.ContainerType.Stack, orientation: orientation },
                horizontalAlign: "stretch",
                verticalAlign: "stretch",
                marginTop: obj.header ? obj.header.height : 50,
                width: 400,
                height: 400,

            });

            //if (obj.orientation == "horizontal") {
            //    stack.minHeight = 250;

            //}
            diagram.model.nodes.push(stack);
            diagram.nameTable[stack.name] = stack;
            childern.push(stack.name);
            canvas = ej.datavisualization.Diagram.Group({
                name: obj.name ? obj.name : "",
                isGroup: true,
                parent: "",
                offsetX: obj.offsetX ? obj.offsetX : 100,
                offsetY: obj.offsetY ? obj.offsetY : 100,
                children: childern,
                fillColor: "transparent",
                container: { type: ej.datavisualization.Diagram.ContainerType.Canvas, },
                minHeight: obj.height ? obj.height : 100,
                maxHeight: obj.maxHeight ? obj.maxHeight : 1000,
                minWidth: obj.width ? obj.width : 100,
                maxWidth: obj.maxWidth ? obj.maxWidth : 1000,
                constraints: ej.datavisualization.Diagram.NodeConstraints.Default ^ ej.datavisualization.Diagram.NodeConstraints.ResizeNorth ^
                      ej.datavisualization.Diagram.NodeConstraints.ResizeWest ^ ej.datavisualization.Diagram.NodeConstraints.ResizeNorthWest ^
                  ej.datavisualization.Diagram.NodeConstraints.ResizeNorthEast ^ ej.datavisualization.Diagram.NodeConstraints.ResizeSouthWest,
            });
            canvas.isSwimlane = true;
            //diagram.model.nodes.push(canvas);
            return canvas;
        },
        _initLanes: function (obj, diagram, headAdded) {
            var lanes = [], lane = null;
            if (obj.lanes) {
                for (var i = 0; i < obj.lanes.length; i++) {
                    lane = this._initLane(obj.lanes[i], obj, diagram, headAdded);
                    lanes.push(lane.name);
                    diagram.model.nodes.push(lane);
                    diagram.nameTable[lane.name] = lane;
                }
            }
            return lanes;
        },
        _initLane: function (laneobj, obj, diagram, headAdded) {
            var hAlign = null, vAlign = null;
            if (obj.orientation == "horizontal")
                hAlign = "stretch";
            else vAlign = "stretch";

            var lane = ej.datavisualization.Diagram.Group({
                name: laneobj.name,
                isGroup: true,
                isLane: true,
                parent: obj.name + "_stack",
                offsetX: 0,
                offsetY: 0,
                borderColor: laneobj.borderColor ? laneobj.borderColor : "black",
                orientation: obj.orientation ? obj.orientation : "horizontal",
                fillColor: laneobj.fillColor,
                container: { type: ej.datavisualization.Diagram.ContainerType.Canvas, },
                horizontalAlign: hAlign ? hAlign : "left",
                verticalAlign: vAlign ? vAlign : "top",
                minHeight: laneobj.height ? laneobj.height : 100,
                maxHeight: laneobj.maxHeight ? laneobj.maxHeight : 1000,
                minWidth: laneobj.width ? laneobj.width : 100,
                maxWidth: laneobj.maxWidth ? laneobj.maxWidth : 1000,
                paddingTop: 20,
                paddingRight: 20,
                paddingBottom: 20,
                paddingLeft: 20,
                constraints: ej.datavisualization.Diagram.NodeConstraints.Default ^ ej.datavisualization.Diagram.NodeConstraints.ResizeNorth ^
                        ej.datavisualization.Diagram.NodeConstraints.ResizeWest ^ ej.datavisualization.Diagram.NodeConstraints.ResizeNorthWest ^
                    ej.datavisualization.Diagram.NodeConstraints.ResizeNorthEast ^ ej.datavisualization.Diagram.NodeConstraints.ResizeSouthWest ^
                     ej.datavisualization.Diagram.NodeConstraints.Connect,
            });
            if (!headAdded) {
                var header = this._iniLaneHeader(laneobj, lane.name, obj, diagram);
                if (header) {
                    diagram.model.nodes.push(header);
                    diagram.nameTable[header.name] = header;
                    lane.children.push(header.name);
                }
            }
            var laneChildren;
            laneChildren = this._initChildren(laneobj, lane.name, diagram);
            for (var i = 0; i < laneChildren.length; i++) {
                lane.children.push(laneChildren[i]);
            }

            return lane;
        },
        _iniLaneHeader: function (headObj, parentName, obj, diagram) {
            var node = null;
            if (headObj.header) {
                node = ej.datavisualization.Diagram.Node({
                    name: parentName + headObj.header.text + "_Headerr_",
                    type: "node",
                    labels: [
                    {
                        text: headObj.header.text ? headObj.header.text : "", offset: { x: 0.5, y: .5 }, width: 100, rotateAngle: 270, marginLeft: -10,
                        fontColor: headObj.header.fontColor ? headObj.header.fontColor : "black",
                        fontSize: headObj.header.fontSize ? headObj.header.fontSize : 12,
                        fontFamily: headObj.header.fontFamily ? headObj.header.fontFamily : "Arial",
                        bold: headObj.header.bold ? headObj.header.bold : false,
                        italic: headObj.header.italic ? headObj.header.italic : false,
                        textDecoration: headObj.header.textDecoration ? headObj.header.textDecoration : "none",
                    }],
                    height: headObj.header.height ? headObj.header.height : 50,
                    width: headObj.header.width ? headObj.header.width : 50,
                    fillColor: headObj.header.fillColor ? headObj.header.fillColor : "white",
                    rotateAngle: 0,
                    constraints: ej.datavisualization.Diagram.NodeConstraints.Default ^ ej.datavisualization.Diagram.NodeConstraints.Select ^ ej.datavisualization.Diagram.NodeConstraints.Connect,
                    marginLeft: -20,
                    marginTop: -20,
                    marginRight: -20,
                    marginBottom: -20,
                    parent: parentName,
                });
                if (obj.orientation && obj.orientation == "horizontal") {
                    node.verticalAlign = "stretch";
                } else if (obj.orientation && obj.orientation == "vertical") {
                    node.horizontalAlign = "stretch";
                    node.labels[0].rotateAngle = 0;
                }
            }
            return node;
        },
        _initChildren: function (laneObj, parentName, diagram) {
            var children = [], child = null;
            if (laneObj.children) {
                for (var i = 0; i < laneObj.children.length; i++) {
                    child = this._initChild(laneObj.children[i], parentName, diagram);
                    if (child.shape)
                        diagram.model.nodes.push(child);
                    else if (!child.isGroup)
                        diagram.model.connectors.push(child);
                    children.push(child.name);
                    diagram.nameTable[child.name] = child;
                }
            }
            return children;
        },
        _initChild: function (child, parentName, diagram) {
            var node = null;
            node = diagram._getNewNode(child);
            node.parent = parentName;
            return node;
        },
        _initphases: function (obj, diagram) {
            var phases = [];
            var phase = null;
            if (!diagram.model.phases)
                diagram.model.phases = [];
            if (obj.phases && obj.phases.length > 0) {
                for (var i = 0; i < obj.phases.length; i++) {
                    phase = ej.datavisualization.Diagram.phase(obj.phases[i]);
                    phase.parent = obj.name;
                    phase.orientation = obj.orientation;
                    diagram.model.phases.push(phase);
                    diagram.nameTable[phase.name] = phase;
                    phases.push(phase.name);
                }
            }
            return phases;
        },
        //#endregion

        //#region palette
        _createSwimlane: function (obj, diagram, data) {
            var children = [], child = null, headAdded = false;
            for (var i = 0; i < obj.children.length; i++) {
                child = diagram.nameTable[obj.children[i]];
                if (child)
                    children.push(child);
            }
            var nameTable = diagram.nameTable;
            var height = 500, width = 500;
            var headNode = null;
            if (obj.children[0]) {
                var node = nameTable[obj.children[0]];
                headNode = { text: "HEADER", width: 50, fillColor: node.fillColor, height: 50 };

            } else headNode = { text: "HEADER", width: 50, fillColor: "white", height: 50 };

            if (!obj.orientation) {
                if (obj.container)
                    obj.orientation = obj.container.orientation;
            }

            if (obj.orientation == "horizontal") {
                height = obj.height ? obj.height : 100;
                width = obj.width ? obj.width : 650;
            } else {
                height = obj.height ? obj.height : 650;
                width = obj.width ? obj.width : 100;
            }

            var nSwimLane = {
                name: obj.name,
                header: headNode,
                fillColor: "white",
                minWidth: width,
                minHeight: height,
                height: height + 1,
                width: width + 1,
                maxHeight: height + 1000,
                maxWidth: width + 1000,
                offsetX: obj.offsetX,
                offsetY: obj.offsetY,
                orientation: obj.orientation ? obj.orientation : "horizontal",
                lanes: [
                    {
                        name: obj.name + "_canvas",
                        header: { text: "HEADER", width: 50, fillColor: node.fillColor },
                        fillColor: obj.fillColor ? obj.fillColor : "white",
                        height: 120,
                        children: children ? children : [],
                    },
                ]
            };
            var item = this._initSwimLane(ej.datavisualization.Diagram.SwimLane(nSwimLane), diagram, headAdded);
            return item;
        },
        _createDiagramLane: function (obj, nameTable) {
            var height = 0, width = 0, hAlign, vAlign, headerWidth = 0;
            if (obj.orientation == "horizontal") {
                height = 120;
                width = 450;
                hAlign = "stretch";
                vAlign = "top";
                headerWidth = 50;
            }
            else if (obj.orientation == "vertical") {
                height = 450;
                width = 120;
                hAlign = "left";
                vAlign = "stretch";
                headerWidth = 50;
            }
            var content = nameTable[obj.children[1]];

            var canvas = ej.datavisualization.Diagram.Group({
                name: obj.name,
                isGroup: true,
                isLane: true,
                borderColor: "black",
                offsetX: obj.offsetX,
                offsetY: obj.offsetY,
                fillColor: content.fillColor,
                container: { type: ej.datavisualization.Diagram.ContainerType.Canvas, },
                horizontalAlign: hAlign ? hAlign : "left",
                verticalAlign: vAlign ? vAlign : "top",
                minHeight: height ? height : 100,
                maxHeight: 1000,
                minWidth: width ? width : 100,
                orientation: obj.orientation ? obj.orientation : "horizontal",
                maxWidth: 1000,
                paddingTop: 20,
                paddingRight: 20,
                paddingBottom: 20,
                paddingLeft: 20,
                constraints: ej.datavisualization.Diagram.NodeConstraints.Default ^ ej.datavisualization.Diagram.NodeConstraints.ResizeNorth ^
                        ej.datavisualization.Diagram.NodeConstraints.ResizeWest ^ ej.datavisualization.Diagram.NodeConstraints.ResizeNorthWest ^
                    ej.datavisualization.Diagram.NodeConstraints.ResizeNorthEast ^ ej.datavisualization.Diagram.NodeConstraints.ResizeSouthWest ^
                ej.datavisualization.Diagram.NodeConstraints.Connect,
            });

            var head = nameTable[obj.children[0]];
            var headObj = { header: { text: "HEADER", width: headerWidth, fillColor: head.fillColor } };

            var header = this._iniLaneHeader(headObj, canvas.name, { orientation: obj.orientation }, null);
            header.name = canvas.name + "_Headerr_";
            nameTable[header.name] = header;
            canvas.children.push(header.name);
            return canvas;
        },
        _createPaletteLane: function (obj, nameTable) {
            var height = 0, width = 0, offsetX = 0, offsetY = 0, rotateAngle = 0, mleft = 0, mTop = 0, align = "left";
            if (obj.orientation == "horizontal") {
                height = obj.height ? obj.height : 50;
                width = 25;
                offsetX = 15;
                offsetY = height / 2;
                rotateAngle = 270;
                align = "left";
            }
            else if (obj.orientation == "vertical") {
                height = 25;
                width = obj.width ? obj.width : 50;
                offsetX = width / 2;
                offsetY = 15;
                align = "left";
            }
            var name = obj.header.text ? obj.header.text : "";
            var header = ej.datavisualization.Diagram.Node({
                name: obj.name + "_" + name,
                type: "node",
                labels: [
                    { text: obj.header.text ? obj.header.text : "", fontSize: 10, offset: { x: 0.5, y: .5 }, width: 100, rotateAngle: rotateAngle, }
                ],
                height: height,
                width: width,
                fillColor: obj.header.fillColor ? obj.header.fillColor : "white",
                //constraints: ej.datavisualization.Diagram.NodeConstraints.Default ^ ej.datavisualization.Diagram.NodeConstraints.Select,
                parent: obj.name,
                offsetX: offsetX,
                offsetY: offsetY,
                constraints: ej.datavisualization.Diagram.NodeConstraints.Default ^ ej.datavisualization.Diagram.NodeConstraints.Connect,
            });
            nameTable[header.name] = header;
            if (obj.orientation == "horizontal") {
                height = obj.height;
                width = obj.width - 25;
                offsetX = 25 + width / 2;
                offsetY = height / 2;
            }
            else if (obj.orientation == "vertical") {
                height = obj.height - 25;
                width = obj.width;
                offsetX = width / 2;
                offsetY = 25 + height / 2;
            }

            var content = ej.datavisualization.Diagram.Node({
                name: obj.name + "content",
                type: "node",
                height: height,
                width: width,
                fillColor: obj.fillColor ? obj.fillColor : "white",
                //constraints: ej.datavisualization.Diagram.NodeConstraints.Default ^ ej.datavisualization.Diagram.NodeConstraints.Select,
                parent: obj.name,
                offsetX: offsetX,
                offsetY: offsetY,
            });
            nameTable[content.name] = content;


            var group = ej.datavisualization.Diagram.Group({
                name: obj.name + "_group",
                height: obj.height,
                width: obj.width,
                offsetX: obj.offsetX,
                offsetY: obj.offsetY,
                children: [header.name, content.name, ]
            });
            group.isLane = true;
            group.orientation = obj.orientation;
            return group;
            //var lane = ej.datavisualization.Diagram.Group({
            //    name: obj.name,
            //    isGroup: true,
            //    parent: "",
            //    offsetX: 0,
            //    offsetY: 0,
            //    fillColor: obj.fillColor ? obj.fillColor : "white",
            //    container: { type: ej.datavisualization.Diagram.ContainerType.Canvas, },
            //    horizontalAlign: "left",
            //    verticalAlign: "top",
            //    minHeight: obj.height ? obj.height : 100,
            //    maxHeight: obj.maxHeight ? obj.maxHeight : 1000,
            //    minWidth: obj.width ? obj.width : 100,
            //    maxWidth: obj.maxWidth ? obj.maxWidth : 1000,
            //    paddingTop: 20,
            //    paddingRight: 20,
            //    paddingBottom: 20,
            //    paddingLeft: 20,
            //});

            //var tObj = {};
            //tObj.orientation = "horizontal";
            //var header = this._iniLaneHeader(obj, lane.name, tObj, diagram);
            //nameTable[header.name] = header;
            //lane.children.push(header.name);
            //return lane;

        },
        //#endregion
    };

})(jQuery, Syncfusion);

;

(function ($, ej) {
    "use strict";

    //#region SVG
    var Svg = (function () {
        function Svg(attr) {
            this.xmlns = "http://www.w3.org/2000/svg";
            this.document = document.createElementNS(this.xmlns, "svg");
            this.id = attr.id;
            if (attr) {
                ej.datavisualization.Diagram.Util.attr(this.document, attr);
            }
            return this;
        }
        Svg.prototype.element = function (attr, tag) {
            var element;
            if (attr && attr.id) {
                element = this.document.getElementById(attr.id);
            }
            if (!element) {
                element = document.createElementNS(this.xmlns, tag);
            }
            if (attr)
                ej.datavisualization.Diagram.Util.attr(element, attr);
            return element;
        };
        Svg.prototype.g = function (attr) {
            return this.element(attr, "g");
        };
        Svg.prototype.defs = function (attr) {
            return this.element(attr, "defs");
        };
        Svg.prototype.image = function (attr) {
            return this.element(attr, "image");
        };
        Svg.prototype.path = function (attr) {
            return this.element(attr, "path");
        };
        Svg.prototype.text = function (attr) {
            return this.element(attr, "text");
        };
        Svg.prototype.textPath = function (attr) {
            return this.element(attr, "textPath");
        };
        Svg.prototype.tspan = function (attr) {
            return this.element(attr, "tspan");
        };
        Svg.prototype.rect = function (attr) {
            return this.element(attr, "rect");
        };
        Svg.prototype.ellipse = function (attr) {
            return this.element(attr, "ellipse");
        };
        Svg.prototype.circle = function (attr) {
            return this.element(attr, "circle");
        };
        Svg.prototype.line = function (attr) {
            return this.element(attr, "line");
        };
        Svg.prototype.polyline = function (attr) {
            return this.element(attr, "polyline");
        };
        Svg.prototype.polygon = function (attr) {
            return this.element(attr, "polygon");
        };
        Svg.prototype.foreignObject = function (attr) {
            return this.element(attr, "foreignObject");
        };
        Svg.prototype.linearGradient = function (attr) {
            return this.element(attr, "linearGradient");
        };
        Svg.prototype.radialGradient = function (attr) {
            return this.element(attr, "radialGradient");
        };
        Svg.prototype.stop = function (attr) {
            return this.element(attr, "stop");
        };
        Svg.prototype.filter = function (attr) {
            return this.element(attr, "filter");
        };
        Svg.prototype.feGaussianBlur = function (attr) {
            return this.element(attr, "feGaussianBlur");
        };
        Svg.prototype.feBlend = function (attr) {
            return this.element(attr, "feBlend");
        };
        Svg.prototype.feOffset = function (attr) {
            return this.element(attr, "feOffset");
        };
        Svg.prototype.feColorMatrix = function (attr) {
            return this.element(attr, "feColorMatrix");
        };
        Svg.prototype.pattern = function (attr) {
            return this.element(attr, "pattern");
        };
        Svg.prototype.appendChild = function (element) {
            this.document.appendChild(element);
        };
        Svg.prototype.removeChild = function (element, parent) {
            if (parent) {
                parent.removeChild(element);
            } else {
                this.document.removeChild(element);
            }
        };
        Svg.prototype.getElementById = function (id) {
            return this.document.getElementById(id);
        };
        Svg.prototype.getElementsByClassName = function (name) {
            return this.document.getElementsByClassName(name);
        };
        Svg.prototype.getElementsByTagName = function (name) {
            return this.document.getElementsByTagName(name);
        };
        Svg.prototype.pathBounds = function (data) {
            var attributes = { d: data };
            var path = this.path(attributes);
            this.document.appendChild(path);
            var bounds = path.getBBox();
            this.document.removeChild(path);
            return bounds;
        };
        Svg.prototype.textBounds = function (text, attr, textElement) {
            var bounds = null;
            if (text && attr) {
                var svgText = this.text(attr);
                svgText.appendChild(document.createTextNode(text));
                this.document.appendChild(svgText);
                bounds = svgText.getBBox();
                this.document.removeChild(svgText);
            }
            else if (textElement) {
                this.document.appendChild(textElement);
                bounds = textElement.getBBox();
                this.document.removeChild(textElement);
            }
            return bounds;
        };
        Svg.prototype.absolutePath = function (path) {
            var x0, y0, x1, y1, x2, y2, segs = path.pathSegList;
            for (var x = 0, y = 0, i = 0, length = segs.numberOfItems; i < length; ++i) {
                var seg = segs.getItem(i), char = seg.pathSegTypeAsLetter;
                if (/[MLHVCSQTA]/.test(char)) {
                    if ('x' in seg) x = seg.x;
                    if ('y' in seg) y = seg.y;
                } else {
                    if ('x1' in seg) x1 = x + seg.x1;
                    if ('x2' in seg) x2 = x + seg.x2;
                    if ('y1' in seg) y1 = y + seg.y1;
                    if ('y2' in seg) y2 = y + seg.y2;
                    if ('x' in seg) x += seg.x;
                    if ('y' in seg) y += seg.y;
                    switch (char) {
                        case 'm': segs.replaceItem(path.createSVGPathSegMovetoAbs(x, y), i); break;
                        case 'l': segs.replaceItem(path.createSVGPathSegLinetoAbs(x, y), i); break;
                        case 'h': segs.replaceItem(path.createSVGPathSegLinetoHorizontalAbs(x), i); break;
                        case 'v': segs.replaceItem(path.createSVGPathSegLinetoVerticalAbs(y), i); break;
                        case 'c': segs.replaceItem(path.createSVGPathSegCurvetoCubicAbs(x, y, x1, y1, x2, y2), i); break;
                        case 's': segs.replaceItem(path.createSVGPathSegCurvetoCubicSmoothAbs(x, y, x2, y2), i); break;
                        case 'q': segs.replaceItem(path.createSVGPathSegCurvetoQuadraticAbs(x, y, x1, y1), i); break;
                        case 't': segs.replaceItem(path.createSVGPathSegCurvetoQuadraticSmoothAbs(x, y), i); break;
                        case 'a': segs.replaceItem(path.createSVGPathSegArcAbs(x, y, seg.r1, seg.r2, seg.angle, seg.largeArcFlag, seg.sweepFlag), i); break;
                        case 'z': case 'Z': x = x0; y = y0; break;
                    }
                }
                if (char === 'M' || char === 'm') x0 = x, y0 = y;
            }
            return path;
        };
        return Svg;
    })();

    ej.datavisualization.Diagram.Svg = Svg;
    //#endregion

    //#region SVG Renderer
    ej.datavisualization.Diagram.SvgContext = {
        _renderDocument: function (view, diagram) {
            if (window.SVGSVGElement) {
                var attr = {
                    id: view.canvas.id + "_svg",
                    version: "1.1",
                    xlink: "http://www.w3.org/1999/xlink",
                    "style": "position:absolute",
                };
                var svg = new ej.datavisualization.Diagram.Svg(attr);
                view.svg = svg;
                if (diagram) {
                    diagram._svg = svg;
                    view.canvas.appendChild(svg.document);
                    diagram._view = this._renderView(view.canvas, view.svg);
                    diagram._page = this._renderPage(view.canvas, view.svg, diagram._view);
                    diagram._htmlLayer = this._renderHtmlLayer(view.canvas);
                    this._renderBackground(view.canvas, view.svg, diagram._page, diagram.model);
                    diagram._pageBackgroundLayer = this._renderBackgroundLayer(view.canvas, view.svg, diagram._page);
                    this._renderGrid(view.canvas, view.svg, diagram._page, diagram);
                    diagram._diagramLayer = view.diagramLayer = this._renderDiagramLayer(view.canvas, view.svg, diagram._page);
                    diagram._adornerLayer = this._renderAdornerLayer(view.canvas, view.svg, diagram._page);
                }
                else {


                }
            }
        },

        _renderView: function (canvas, svg) {
            var attr = { id: canvas.id + "_view" };
            var view = svg.g(attr);
            svg.appendChild(view);
            return view;
        },

        _renderHtmlLayer: function (canvas) {
            var div = document.createElement("div");
            var attr = { "id": canvas.id + "_htmlLayer", "class": "htmlLayer" };
            ej.datavisualization.Diagram.Util.attr(div, attr);
            var htmlLayer = div;
            div.style.pointerEvents = "none";
            canvas.appendChild(htmlLayer);
            return htmlLayer;
        },

        _renderPage: function (canvas, svg, parent) {
            var attr = { "id": canvas.id + "_page", "class": "page" };
            var page = svg.g(attr);
            parent.appendChild(page);
            return page;
        },

        _renderDiagramLayer: function (canvas, svg, parent) {
            var attr = { "id": canvas.id + "_diagramLayer", "class": "DiagramLayer" };
            var diagramLayer = svg.g(attr);
            parent.appendChild(diagramLayer);
            return diagramLayer;
        },

        _renderAdornerLayer: function (canvas, svg, parent) {
            var attr = { "id": canvas.id + "_adornerLayer", "class": "AdornerLayer" };
            var adornerLayer = svg.g(attr);
            parent.appendChild(adornerLayer);
            return adornerLayer;
        },

        _renderBackgroundLayer: function (canvas, svg, parent) {
            var attr = {
                "id": canvas.id + "_pageBackground",
                "class": "PageBackgroundLayer",
                "style": "pointer-events:none;",
            };
            var pageBackgroundLayer = svg.g(attr);
            parent.appendChild(pageBackgroundLayer);
            return pageBackgroundLayer;
        },

        _renderBackground: function (canvas, svg, parent, model) {
            svg.document.style.msTouchAction = "none";
            svg.document.style.display = "block";
            svg.document.style.backgroundColor = "white";
            var attr = { id: canvas.id + "_backgroundLayer" };
            var g = svg.g(attr);
            attr = { "id": canvas.id + "_background", "x": "0", "y": "0", "width": "100%", "height": "100%", "preserveAspectRatio": "none", "pointer-events": "none" };
            var image = svg.image(attr);
            if (model.backgroundImage) {
                image.setAttributeNS("http://www.w3.org/1999/xlink", "xlink:href", model.backgroundImage);
                if (!svg.getElementById(canvas.id + "_background")) {
                    g.appendChild(image);
                }
            }
            else {
                if (svg.getElementById(image.id)) {
                    g.removeChild(svg.getElementById(image.id));
                }
            }
            var rect = svg.rect({
                "id": canvas.id + "_backgroundColor", "x": "0", "y": "0", "width": "100%", "height": "100%", "fill": model.backgroundColor, "preserveAspectRatio": "none", "pointer-events": "none"
            });
            if (model.backgroundColor) {
                if (!svg.getElementById(canvas.id + "_backgroundColor")) {
                    g.appendChild(rect);
                }
            }
            else {
                if (svg.getElementById(rect.id)) {
                    g.removeChild(svg.getElementById(rect.id));
                }
            }

            if (!svg.getElementById(g.id))
                parent.appendChild(g);

        },

        _renderGrid: function (canvas, svg, parent, diagram) {
            var defs = svg.defs({ "id": canvas.id + "patterndefinition" });
            var g;
            svg.appendChild(defs);
            if (diagram.model.snapSettings.snapConstraints & ej.datavisualization.Diagram.SnapConstraints.ShowHorizontalLines ||
                diagram.model.snapSettings.snapConstraints & ej.datavisualization.Diagram.SnapConstraints.ShowVerticalLines) {
                var grid = document.getElementById(canvas.id + "_grid");
                var rect = document.getElementById(canvas.id + "pattern_gridline");
                var pattern = document.getElementById(canvas.id + "pattern_grid");
                if (pattern)
                    defs.removeChild(pattern);
                var attr = { id: canvas.id + "_grid", "pointer-events": "none" };
                g = svg.g(attr);
                var height = 0;
                if (diagram.model.snapSettings.horizontalGridLines.linesInterval) {
                    diagram.model.snapSettings.horizontalGridLines.linesInterval.forEach(function (entry) {
                        height += entry;
                    });
                }
                var width = 0;
                if (diagram.model.snapSettings.verticalGridLines.linesInterval) {
                    diagram.model.snapSettings.verticalGridLines.linesInterval.forEach(function (entry) {
                        width += entry;
                    });
                }
                var scale = diagram._currZoom;

                attr = { id: canvas.id + "pattern_grid", x: 0, y: 0, width: width * scale, height: height * scale, patternUnits: "userSpaceOnUse" };
                var pattern = svg.pattern(attr);
                //pattern.setAttribute("patternUnits", "userSpaceOnUse");
                scale = this._scaleSnapInterval(diagram.model, scale);
                this._renderHorizontalGridlines(scale, pattern, width, diagram.model, svg);
                this._renderVerticalGridlines(scale, pattern, height, diagram.model, svg);
                defs.appendChild(pattern);
                var rect = svg.rect({ id: canvas.id + "pattern_gridline", "x": diagram._hScrollOffset, "y": diagram._vScrollOffset, "width": "100%", "height": "100%", "fill": "url(#" + canvas.id + "pattern_grid)" });
                g.appendChild(rect);
                if (!grid)
                    parent.appendChild(g);
            }
            else {
                g = document.getElementById(canvas.id + "_grid");
                if (g) {
                    var grid = document.getElementById(canvas.id + "pattern_gridline");
                    if (grid)
                        g.removeChild(grid);
                }
            }
        },

        _updateBackground: function (hoffset, voffset, scale, diagram) {
            var attr = {
                x: hoffset,
                y: voffset
            };
            var bgClr = document.getElementById(diagram._id + "_canvas_backgroundColor");
            if (bgClr)
                ej.datavisualization.Diagram.Util.attr(bgClr, attr);
            var bgImg = document.getElementById(diagram._id + "_canvas_background");
            if (bgImg)
                ej.datavisualization.Diagram.Util.attr(bgImg, { "transform": "scale(" + scale + ")" });
        },

        _updateGrid: function (hoffset, voffset, scale, diagram) {
            var grid = document.getElementById(diagram._canvas.id + "pattern_gridline");
            var i;
            if (grid) {
                var pattern = document.getElementById(diagram._canvas.id + "pattern_grid");
                var height = 0;
                if (diagram.model.snapSettings.horizontalGridLines.linesInterval) {
                    for (i = 0; i < diagram.model.snapSettings.horizontalGridLines.linesInterval.length; i++) {
                        height += diagram.model.snapSettings.horizontalGridLines.linesInterval[i];
                    }
                }
                var width = 0;
                if (diagram.model.snapSettings.verticalGridLines.linesInterval) {
                    for (i = 0; i < diagram.model.snapSettings.verticalGridLines.linesInterval.length; i++) {
                        width += diagram.model.snapSettings.verticalGridLines.linesInterval[i];
                    }
                }

                var attr = {
                    x: hoffset,
                    y: voffset
                };
                ej.datavisualization.Diagram.Util.attr(grid, attr);

                scale = this._scaleSnapInterval(diagram.model, scale);
                var svg = diagram._svg;
                var defs = svg.defs({ "id": diagram._canvas.id + "patterndefinition" });
                if (pattern)
                    defs.removeChild(pattern);
                var attr = { id: diagram._canvas.id + "pattern_grid", x: 0, y: 0, width: width * scale, height: height * scale, patternUnits: "userSpaceOnUse" };
                pattern = diagram._svg.pattern(attr);
                this._renderHorizontalGridlines(scale, pattern, width, diagram.model, diagram._svg);
                this._renderVerticalGridlines(scale, pattern, height, diagram.model, diagram._svg);
                defs.appendChild(pattern);
            }
        },
        _scaleSnapInterval: function (model, scale) {
            if (scale >= 2) {
                while (scale >= 2) {
                    scale /= 2;
                }
            }
            else if (scale <= 0.5) {
                while (scale <= 0.5) {
                    scale *= 2;
                }
            }
            var i;
            if (scale !== 1) {
                model.snapSettings.horizontalGridLines._snapinterval = [];
                for (i = 0; i < model.snapSettings.horizontalGridLines.snapInterval.length; i++) {
                    model.snapSettings.horizontalGridLines._snapinterval[i] =
                        model.snapSettings.horizontalGridLines.snapInterval[i] * scale;
                }
                model.snapSettings.verticalGridLines._snapinterval = [];
                for (i = 0; i < model.snapSettings.verticalGridLines.snapInterval.length; i++) {
                    model.snapSettings.verticalGridLines._snapinterval[i] =
                        model.snapSettings.verticalGridLines.snapInterval[i] * scale;
                }
            }
            return scale;
        },

        _renderHorizontalGridlines: function (scale, pattern, width, model, svg) {
            var snapSettings = model.snapSettings;
            if (snapSettings.snapConstraints & ej.datavisualization.Diagram.SnapConstraints.ShowHorizontalLines) {
                var space = 0;
                var cnt = 0;
                for (var i = 0; i < snapSettings.horizontalGridLines.linesInterval.length; i = i + 2) {
                    var thickness = snapSettings.horizontalGridLines.linesInterval[i];
                    var line = svg.line({
                        "x1": 0, "y1": space * scale, "x2": width * scale, "y2": space * scale, "stroke": snapSettings.horizontalGridLines.lineColor,
                        "stroke-width": thickness
                    });
                    //svg.appendChild(line);
                    space += snapSettings.horizontalGridLines.linesInterval[i + 1] + thickness;
                    pattern.appendChild(line);
                    cnt++;
                }
            }
        },

        _renderVerticalGridlines: function (scale, pattern, height, model, svg) {
            var snapSettings = model.snapSettings;
            if (snapSettings.snapConstraints & ej.datavisualization.Diagram.SnapConstraints.ShowVerticalLines) {
                var space = 0;
                var cnt = 0;
                for (var i = 0; i < snapSettings.verticalGridLines.linesInterval.length; i = i + 2) {
                    var thickness = model.snapSettings.verticalGridLines.linesInterval[i];
                    var line = svg.line({
                        "x1": space * scale, "y1": 0, "x2": space * scale, "y2": height * scale, "stroke": snapSettings.verticalGridLines.lineColor,
                        "stroke-width": thickness, "stroke-dasharray": snapSettings.verticalGridLines.lineDashArray
                    });
                    space += snapSettings.verticalGridLines.linesInterval[i + 1] + thickness;
                    pattern.appendChild(line);
                    cnt++;
                }
            }
        },
        //#region Transformation
        transformView: function (diagram, hOffset, vOffset) {
            var view = diagram._view;
            var attr = {
                "transform": "translate(" + hOffset + "," + vOffset + ")"
            };
            ej.datavisualization.Diagram.Util.attr(view, attr);
            if (ej.browserInfo().name === "msie") {
                if (diagram._htmlLayer && diagram._htmlLayer.children.length > 0) {
                    diagram._htmlLayer.style.left = hOffset + "px";
                    diagram._htmlLayer.style.top = vOffset + "px";
                    diagram._htmlLayer.style.position = "absolute";
                }
            }
        },
        scaleContent: function (diagram, scale) {
            var diagramLayer = diagram._diagramLayer;
            ej.datavisualization.Diagram.Util.attr(diagramLayer, { "transform": "scale(" + scale + ")" });
            if (ej.browserInfo().name === "msie") {
                if (diagram._htmlLayer && diagram._htmlLayer.children.length > 0) {
                    diagram._htmlLayer.style.transform = "scale(" + scale + ")";
                }
            }
            if (diagram.selectionList.length > 0)
                this.updateSelector(diagram.selectionList[0], diagram._svg, scale, diagram, diagram.model.selectorConstraints);
            if (diagram.selectionList[0]) {
                if ((diagram.model.userHandles && diagram.model.userHandles.length > 0)) {
                    var isMultipleSelection = false;
                    if (diagram.selectionList[0].isPseudoGroup)
                        isMultipleSelection = true;
                    if (diagram.model.selectorConstraints & ej.datavisualization.Diagram.SelectorConstraints.UserHandles)
                        ej.datavisualization.Diagram.SvgContext.updateUserHandles(diagram.model.userHandles, diagram.selectionList[0], diagram._svg, isMultipleSelection, false, scale);
                }
            }
        },
        setSize: function (svg, width, height) {
            ej.datavisualization.Diagram.Util.attr(svg.document, { width: width + "px", height: height + "px" });
        },
        //#endregion

        _findIndex: function (diagram, node) {
            var lastNode, newIndex = 1, index, elements;
            if (node.parent)
                elements = $("#" + node.parent.name).find(">.node,>.connector,>.group");
            else
                elements = $("#DiagramContent_canvas_svg").find(">.node,>.connector,>.group");
            if (node.name != "helper") {
                lastNode = elements[elements.length - 1];
                if (lastNode && lastNode.id == "helper")
                    lastNode = elements[elements.length - 2];
                if (lastNode) {
                    index = 0;
                    if (lastNode.className.animVal == "ej-d-node" || lastNode.className.animVal == "ej-d-group") {
                        newIndex = index + 1;
                    }
                    else if (lastNode.className.animVal == "ej-d-connector") {
                        newIndex = index + 1;
                    }
                }
            }
            return newIndex;
        },
        //#region Render Node

        //#region render
        renderNode: function (node, svg, parent) {
            var g;
            // if (node.visible) 
            {
                var name = node.name;
                var visible = node.visible ? "visible" : "hidden";
                var x = node.offsetX - node.width * node.pivot.x;
                var y = node.offsetY - node.height * node.pivot.y;
                g = svg.g({
                    "id": name, "class": "ej-d-node", "transform": "rotate(" + node.rotateAngle + "," + node.offsetX + "," + node.offsetY +
                        "),translate(" + x + "," + y + ")", "visibility": visible
                });
                if (parent) {
                    parent.appendChild(g);
                }
                else {
                    svg.appendChild(g);
                }
                this._renderNode(node, svg, g);
            }
            return g;
        },

        _renderNode: function (node, svg, g) {
            this._renderShape(node, svg, g);
            this._renderLabels(node, svg, g);
            this._renderPorts(node, svg, g);
        },

        _renderShadow: function (node, svg) {
            var filter = this._renderFilter(node, svg);
            if (node.type == "image" || node.type == "html" || node.type == "text") {
                var element = svg.document.getElementById(node.name + "_backrect");
            }
            else element = svg.document.getElementById(node.name + "_shape");
            element.setAttribute("filter", filter);
        },

        _removeShadow: function (node, svg) {
            if (node.type == "image" || node.type == "html" || node.type == "text") {
                var element = svg.document.getElementById(node.name + "_backrect");
            }
            else element = svg.document.getElementById(node.name + "_shape");
            element.removeAttribute("filter");
        },
        renderphase: function (node, svg, parent, diagram) {
            var g;
            g = svg.g({ "id": node.name, "class": "ej-d-sepetator" });
            if (node.parent) {
                //parent = svg.getElementById(node.parent);
                //if (parent)
                parent.appendChild(g);
            } else {
                svg.appendChild(g);
            }
            var visibility = "visible";
            var tx = 0, ty = 0;
            var parNode = diagram.nameTable[node.parent];
            if (parNode) {
                if (parNode.children.length > 1)
                    var header = diagram.nameTable[parNode.children[0]];
                var bounds = ej.datavisualization.Diagram.Util.bounds(parNode);
                var points = [], nPoint;
                var top = bounds.top + 50;
                var left = bounds.left + 50;
                if (node.orientation == "vertical") {
                    points.push({ x: 0, y: 0 });
                    points.push({ x: bounds.width, y: 0 });
                    nPoint = this._convertToSVGPoints(points);
                    tx = bounds.x;
                    ty = bounds.y + node.offset;
                    if (header)
                        top += header.height;
                    if (!(ty < bounds.bottom && ty > top)) {
                        visibility = "hidden";
                    }
                } else {
                    points.push({ x: 0, y: header.height ? header.height : 0 });
                    points.push({ x: 0, y: bounds.height });
                    nPoint = this._convertToSVGPoints(points);
                    tx = bounds.x + node.offset;
                    ty = bounds.y;

                    if (!(tx < bounds.right && tx > left)) {
                        visibility = "hidden";
                    }
                }


                var attr = {
                    "id": node.name + "_phase",
                    "stroke": node.lineColor,
                    "stroke-width": node.lineWidth,
                    "stroke-dasharray": node.lineDashArray,
                    "points": nPoint,
                    //"transform": "translate(" + tx + "," + ty + ")",
                };
                var line = svg.polyline(attr);
                g.appendChild(line);

                attr = {
                    "id": node.name + "_phase_hitTest",
                    "stroke-width": 10,
                    "points": nPoint,
                    //"transform": "translate(" + tx + "," + ty + ")",
                    "stroke": "transparent",

                };
                line = svg.polyline(attr);
                g.appendChild(line);
                g.setAttribute("transform", "translate(" + tx + "," + ty + ")");
            }

        },

        _renderGroupBackground: function (group, g, svg) {
            if (group.fillColor != "none") {
                var x = group.offsetX - group.width / 2;
                var y = group.offsetY - group.height / 2;

                var angle = group.rotateAngle;
                var pt = new ej.datavisualization.Diagram.Point(x + group.width / 2, y + group.height / 2);
                var attr = {
                    "id": group.name + "_shape",
                    "rx": group.shape.cornerRadius,
                    "ry": group.shape.cornerRadius,
                    "width": Math.round(group.width),
                    "height": Math.round(group.height),
                    "fill": group.fillColor,
                    "stroke": group.borderColor,
                    "stroke-width": group.borderWidth,
                    "opacity": group.opacity,
                    "stroke-dasharray": group.borderDashArray,
                    "transform": "rotate(" + angle + "," + pt.x + "," + pt.y + "),translate(" + x + "," + y + ")",
                };
                g.appendChild(svg.rect(attr));
            }
        },

        renderGroup: function (group, svg, parent, nameTable, diagram, isLoad, overView) {
            var g;
            var visible = group.visible ? "visible" : "hidden";
            //if (group.visible) 
            {
                g = svg.g({ "id": group.name, "class": "ej-d-group", "visibility": visible });
                if (parent) {
                    parent.appendChild(g);
                } else {
                    svg.appendChild(g);
                }
                if (group.parent == "" && group.container) {
                    if (isLoad || overView) {
                        group.height = 0;
                        group.width = 0;
                    }
                    this._alignOnContainer(group, diagram);
                }
                this._renderGroupBackground(group, g, svg);

                var children = group.children;
                for (var i = 0, len = children.length; i < len; i++) {
                    var child = nameTable[children[i]];
                    if (child) {
                        if (child.isGroup)
                            this.renderGroup(child, svg, g, nameTable, diagram);
                        else if (child.segments)
                            this.renderConnector(child, svg, g, diagram);
                        else
                            this.renderNode(child, svg, g);
                    }
                }
                this._renderLabels(group, svg, g);
                this._renderPorts(group, svg, g);
            }
            return g;
        },
        _alignOnContainer: function (group, diagram, stretch) {
            switch (group.container && group.container.type) {
                case "canvas":
                    var angle = group.rotateAngle;
                    //group.rotateAngle = 0;
                    if (!(group.rotateAngle == 0 && angle == 0)) {
                        if (group.parent == "")
                            group.rotateAngle = 0;
                        else
                            diagram._rotate(group, -angle, diagram.nameTable);
                    }

                    this._alignOnCanvasContainer1(group, diagram, stretch);
                    if (!((group.rotateAngle == 0) && (angle == 0)))
                        diagram._rotate(group, angle, diagram.nameTable);
                    break;
                case "stack":
                    this._alignOnStackContainer(group, diagram);
                    break;
            }
        },

        _removeFromContainer: function (node, svg, parent) {
            var element = svg.getElementById(node.name);
            if (element)
                element.parentNode.removeChild(element);
        },

        _childrenBounds: function (group, diagram) {
            var children = group.children;
            var bounds = new ej.datavisualization.Diagram.Rectangle();
            bounds = ej.datavisualization.Diagram.Util.bounds(diagram.nameTable[children[0]]);
            for (var i = 1, len = children.length; i < len; i++) {
                var rect = ej.datavisualization.Diagram.Util.bounds(diagram.nameTable[children[i]], diagram);
                if (rect)
                    bounds = diagram._union(bounds, rect);
            }
            return bounds;
        },

        _alignOnCanvasContainer1: function (group, diagram, stretch) {
            var rSize = this._measureCanvasSize1(group, diagram);
            var newSize = diagram._getMinMaxSize(group, rSize.width, rSize.height, stretch);
            group.width = newSize.width;
            group.height = newSize.height;
            this._alignOnCanvas1(group, diagram);
        },
        _alignOnCanvas1: function (group, diagram, resize) {
            var child = null;
            for (var i = 0; i < group.children.length; i++) {
                child = diagram.nameTable[group.children[i]];
                if (child) {
                    var pos = this._getCanvasPosition1(group, child, diagram, resize);
                    diagram._translate(child, pos.x, pos.y, diagram.nameTable);
                    ej.datavisualization.Diagram.SpatialUtil._updateQuad(diagram, diagram._spatialSearch, child);
                }
            }
        },
        _getCanvasPosition1: function (group, node, diagram, resize) {
            var dx = null, dy = null;
            var groupBounds = ej.datavisualization.Diagram.Util.bounds(group);
            var bounds = ej.datavisualization.Diagram.Util.bounds(node);
            switch (node.horizontalAlign) {
                case "left":
                    dx = groupBounds.x - bounds.x + node.marginLeft + group.paddingLeft;
                    break;
                case "center":
                    dx = (groupBounds.x + groupBounds.width / 2) - (bounds.x + bounds.width / 2);
                    if (node.marginLeft)
                        dy = groupBounds.x - bounds.x + node.marginLeft + group.paddingLeft;
                    break;
                case "right":
                    dx = (groupBounds.x + groupBounds.width) - (bounds.x + bounds.width + group.paddingRight);
                    break;
                case "stretch":
                    var angle = node.rotateAngle;
                    if (!(node.rotateAngle == 0) && (angle == 0))
                        diagram._rotate(node, -node.rotateAngle, diagram.nameTable);

                    var deltaWidth = (group.width - group.paddingLeft - group.paddingRight - node.marginLeft - node.marginRight) / node.width;
                    diagram.scale(node, deltaWidth, 0, group.pivot, diagram.nameTable);
                    groupBounds = ej.datavisualization.Diagram.Util.bounds(group);
                    bounds = ej.datavisualization.Diagram.Util.bounds(node, diagram);
                    dx = (groupBounds.x + group.paddingLeft) - (bounds.x) + node.marginLeft;
                    if (!(node.rotateAngle == 0) && (angle == 0))
                        diagram._rotate(node, angle, diagram.nameTable);

                    if (node.container && node.container.type == "stack") {
                        var childBounds = ej.datavisualization.Diagram.Util._getChildrenBounds(node, diagram.nameTable, diagram);
                        var stackWidth = diagram.nameTable[node.name].width;
                        if (childBounds.width < stackWidth) {
                            var diffX = stackWidth - childBounds.width - group.paddingLeft - group.paddingRight;
                            var lastChild = diagram.nameTable[node.children[node.children.length - 1]];
                            if (lastChild) {
                                lastChild.width += diffX;
                                lastChild.offsetX += diffX / 2;
                                lastChild.minWidth = lastChild.width;
                                if (resize && lastChild.container && lastChild.container.type == "canvas") {
                                    this._alignOnCanvas1(lastChild, diagram);
                                }
                            }
                        }
                    }
                    break;
            }
            switch (node.verticalAlign) {
                case "top":
                    dy = groupBounds.y - bounds.y + node.marginTop + group.paddingTop;
                    break;
                case "center":
                    dy = (groupBounds.y + groupBounds.height / 2) - (bounds.y + bounds.height / 2) + node.marginTop;
                    if (node.marginTop)
                        dy = groupBounds.y - bounds.y + node.marginTop + group.paddingTop;
                    break;
                case "bottom":
                    dy = (groupBounds.y + groupBounds.height) - (bounds.y + bounds.height + group.paddingBottom);
                    break;
                case "stretch":
                    var angle = node.rotateAngle; var height
                    var nodeBounds123 = ej.datavisualization.Diagram.Util.bounds(node);
                    if (!(node.rotateAngle == 0) && (angle == 0))
                        diagram._rotate(node, -node.rotateAngle, diagram.nameTable);
                    if (node.height == 0)
                        height = 1;
                    else height = node.height;
                    var deltaHeight = (group.height - group.paddingTop - group.paddingBottom - node.marginTop - node.marginBottom) / height;
                    diagram.scale(node, 0, deltaHeight, group.pivot, diagram.nameTable);
                    groupBounds = ej.datavisualization.Diagram.Util.bounds(group);
                    bounds = ej.datavisualization.Diagram.Util.bounds(node, diagram);
                    dy = (groupBounds.y + group.paddingTop) - (bounds.y) + node.marginTop;
                    if (!(node.rotateAngle == 0) && (angle == 0))
                        diagram._rotate(node, angle, diagram.nameTable);
                    if (node.container && node.container.type == "stack") {
                        childBounds = ej.datavisualization.Diagram.Util._getChildrenBounds(node, diagram.nameTable, diagram);
                        var stackHeight = diagram.nameTable[node.name].height;
                        var diffY = stackHeight - childBounds.height - -group.paddingTop - group.paddingBottom;
                        lastChild = diagram.nameTable[node.children[node.children.length - 1]];
                        if (lastChild) {
                            lastChild.height += diffY;
                            lastChild.offsetY += diffY / 2;
                            lastChild.minHeight = lastChild.height;

                            if (resize && lastChild.container && lastChild.container.type == "canvas") {
                                this._alignOnCanvas1(lastChild, diagram);
                            }
                        }
                    }
                    break;
            }
            return { x: dx, y: dy };
        },
        _measureCanvasSize1: function (group, diagram) {
            var child = null;
            var bounds = ej.datavisualization.Diagram.Rectangle(0, 0, 0, 0), transX = 0, transY = 0;
            var rect = ej.datavisualization.Diagram.Rectangle(0, 0, 0, 0);
            for (var i = 0; i < group.children.length; i++) {
                child = diagram.nameTable[group.children[i]];
                if (child) {
                    if (!child.isGroup) {
                        var bounds1 = ej.datavisualization.Diagram.Util.bounds(child);
                        bounds = ej.datavisualization.Diagram.Geometry.rect([bounds1.topLeft, bounds1.topRight, bounds1.bottomRight, bounds1.bottomLeft]);
                        bounds.y = 0;
                        bounds.x = 0;
                    } else {
                        this._alignOnContainer(child, diagram);
                        bounds1 = ej.datavisualization.Diagram.Util.bounds(child);
                        bounds = ej.datavisualization.Diagram.Geometry.rect([bounds1.topLeft, bounds1.topRight, bounds1.bottomRight, bounds1.bottomLeft]);
                        bounds.y = 0;
                        bounds.x = 0;
                        if (child.rotateAngle == 0) {
                            bounds.width = bounds1.width;
                            bounds.height = bounds1.height;
                        }
                    }
                }
                if (child) {
                    if (child.container && child.container.type == "stack") {
                        bounds.width += child.marginLeft;
                        bounds.height += child.marginTop;
                    } else {
                        bounds.width += child.marginLeft + child.marginRight;
                        bounds.height += child.marginTop + child.marginBottom;
                    }
                }
                if (bounds) {
                    rect = diagram._union(bounds, rect);
                }
            }
            rect.width += group.paddingLeft + group.paddingRight;
            rect.height += group.paddingTop + group.paddingBottom;
            return rect;
        },

        _updateMargin: function (node, group) {
            if (group && group.container && group.container.type == "canvas") {
                var groupBounds = ej.datavisualization.Diagram.Util.bounds(group); //ej.datavisualization.Diagram.SvgContext.getCanvasBoundingBox(group, this.diagram, node, true);
                var bounds = ej.datavisualization.Diagram.Util.bounds(node);
                node.marginLeft = bounds.x - (groupBounds.x + group.paddingLeft);
                node.marginRight = (groupBounds.x + group.width) - (bounds.x + bounds.width) - group.paddingRight;
                node.marginTop = bounds.y - (groupBounds.y + group.paddingTop);
                node.marginBottom = (groupBounds.y + group.height) - (bounds.y + bounds.height) - group.paddingBottom;
            }
        },
        _alignOnStackContainer: function (group, diagram) {

            var angle = group.rotateAngle;
            if (group.parent == "")
                group.rotateAngle = 0;
            else
                diagram._rotate(group, -angle, diagram.nameTable);
            var rSize = this._measureStackSize(group, diagram);
            var newSize = diagram._getMinMaxSize(group, rSize.width, rSize.height);
            group.width = newSize.width;
            group.height = newSize.height;// - group.marginTop;
            this._alignOnStack(group, diagram);
            diagram._rotate(group, angle, diagram.nameTable);

        },

        _alignOnStack: function (group, diagram) {
            var child = null, bounds, transX = 0, transY = 0;
            for (var i = 0; i < group.children.length; i++) {
                child = diagram.nameTable[group.children[i]];
                if (child) {
                    var grAngle = child.rotateAngle;
                    if (!((child.rotateAngle == 0) && (grAngle == 0)))
                        diagram._rotate(child, -child.rotateAngle, diagram.nameTable);
                    var pos = this._getStackPosition(group, child, transX, transY, diagram);

                    diagram._translate(child, pos.x, pos.y, diagram.nameTable);
                    if (group.container.orientation == "vertical")
                        transY += child.height + child.marginTop + child.marginBottom;
                    else
                        transX += child.width + child.marginLeft + child.marginRight;
                    if (!((child.rotateAngle == 0) && (grAngle == 0)))
                        diagram._rotate(child, grAngle, diagram.nameTable);
                    ej.datavisualization.Diagram.SpatialUtil._updateQuad(diagram, diagram._spatialSearch, child);
                }
            }
        },
        _getStackPosition: function (group, node, transX, transY, diagram) {
            var groupBounds = ej.datavisualization.Diagram.Util.bounds(group);
            var bounds = ej.datavisualization.Diagram.Util.bounds(node);
            var dx = null, dy = null;
            if (group.container.orientation == "vertical") {
                dy = groupBounds.y - bounds.y + transY + node.marginTop + group.paddingTop;
                switch (node.horizontalAlign) {
                    case "left":
                        dx = groupBounds.x - bounds.x + node.marginLeft + group.paddingLeft;
                        break;
                    case "center":
                        dx = groupBounds.center.x - bounds.center.x;
                        break;
                    case "right":
                        dx = (groupBounds.x + groupBounds.width) - (bounds.x + bounds.width + group.paddingRight);
                        break;
                    case "stretch":
                        var deltaWidth = (group.width - group.paddingLeft - group.paddingRight) / node.width;
                        diagram.scale(node, deltaWidth, 1, group.pivot, diagram.nameTable);
                        groupBounds = ej.datavisualization.Diagram.Util.bounds(group);
                        bounds = ej.datavisualization.Diagram.Util.bounds(node, diagram);
                        dx = (groupBounds.x + group.paddingLeft) - (bounds.x);
                        break;
                }
            }
            else {
                dx = groupBounds.x - bounds.x + transX + node.marginLeft + group.paddingLeft;
                switch (node.verticalAlign) {
                    case "top":
                        dy = groupBounds.y - bounds.y + node.marginTop + group.paddingTop;
                        break;
                    case "center":
                        dy = groupBounds.center.y - bounds.center.y;
                        break;
                    case "bottom":
                        dy = (groupBounds.y + groupBounds.height) - (bounds.y + bounds.height + group.paddingBottom);
                        break;
                    case "stretch":
                        var deltaHeight = (group.height - group.paddingTop - group.paddingBottom) / node.height;
                        diagram.scale(node, 1, deltaHeight, group.pivot, diagram.nameTable);
                        groupBounds = ej.datavisualization.Diagram.Util.bounds(group);
                        bounds = ej.datavisualization.Diagram.Util.bounds(node, diagram);
                        dy = (groupBounds.y + group.paddingTop) - (bounds.y);
                        break;
                }
            }
            return { x: dx, y: dy }
        },
        _measureStackSize: function (group, diagram, stretch) {
            var child = null;
            var bounds = new ej.datavisualization.Diagram.Rectangle(), transX = 0, transY = 0;
            var rect = ej.datavisualization.Diagram.Rectangle(0, 0, 0, 0);
            for (var i = 0; i < group.children.length; i++) {
                child = diagram.nameTable[group.children[i]];
                if (child) {
                    if (!child.isGroup) {
                        bounds = ej.datavisualization.Diagram.Util.bounds(child);
                        bounds.y = 0;
                        bounds.x = 0;
                    } else {
                        bounds = this._measureContainerSize(child, diagram, stretch);
                        bounds.y = 0;
                        bounds.x = 0;
                    }

                    bounds.width += transX + child.marginLeft + child.marginRight;
                    bounds.height += transY + child.marginTop + child.marginBottom;
                }
                if (bounds) {
                    rect = diagram._union(bounds, rect);
                }
                if (group.container.orientation == "horizontal") {
                    transX = bounds.width;
                }
                else {
                    transY = bounds.height;
                }
            }
            rect.width += group.paddingLeft + group.paddingRight;
            rect.height += group.paddingTop + group.paddingBottom;
            return rect;
        },

        _measureContainerSize: function (node, diagram, stretch) {
            var bounds = null;
            if (node.isGroup && node.container) {
                switch (node.container.type) {
                    case "canvas":
                        this._alignOnContainer(node, diagram, stretch);
                        return ej.datavisualization.Diagram.Util.bounds(node);
                        break;
                    case "stack":
                        break;
                }
            }
        },

        _renderFilter: function (node, svg) {
            if (node.constraints & ej.datavisualization.Diagram.NodeConstraints.Shadow) {
                var defs = svg.getElementsByTagName("defs")[0];
                var width = node.width + 10; var height = node.height + 10;
                if (ej.browserInfo().name === "chrome") { width = node.width / 3; height = node.height / 3; }
                var filter = svg.filter({ id: node.name + "_filter", width: width, height: height });
                var offset = svg.feOffset({ result: "offOut", "in": "SourceAlpha", dx: "5", dy: "5" });
                filter.appendChild(offset);
                var matrix = svg.feColorMatrix({ result: "matrixOut", "in": "offOut", type: "matrix", values: "0.2 0 0 0 0 0 0.2 0 0 0 0 0 0.2 0 0 0 0 0 0.3 0" });
                filter.appendChild(matrix);
                var blur = svg.feGaussianBlur({ result: "blurOut", "in": "matrixOut", stdDeviation: "5" });
                filter.appendChild(blur);
                var blend = svg.feBlend({ "in": "SourceGraphic", in2: "blurOut", mode: "normal" });
                filter.appendChild(blend);
                defs.appendChild(filter);
                return "url(#" + node.name + "_filter)";
            }
            return null;
        },
        _renderGradient: function (name, gradient, svg) {
            var fill;
            var id;
            if (gradient.type === "linear") {
                id = this._renderLinearGradient(name, gradient, svg);
                fill = "url(#" + id + ")";
            }
            else if (gradient.type === "radial") {
                id = this._renderRadialGradient(name, gradient, svg);
                fill = "url(#" + id + ")";
            }
            return fill;
        },

        _renderLinearGradient: function (name, gradient, svg) {
            var defs = svg.getElementsByTagName("defs")[0];
            if (defs) {
                var attr = { "id": name + "_gradient", "x1": gradient.x1 + "%", "y1": gradient.y1 + "%", "x2": gradient.x2 + "%", "y2": gradient.y2 + "%" };
                var linear = svg.linearGradient(attr);
                this._renderStops(gradient, svg, linear);
                defs.appendChild(linear);
                return linear.id;
            }
            return null;
        },

        _renderRadialGradient: function (name, gradient, svg) {
            var defs = svg.getElementsByTagName("defs")[0];
            if (defs) {
                var attr = {
                    "id": name + "_gradient", "cx": gradient.cx + "%", "cy": gradient.cy + "%",
                    "fx": gradient.fx + "%", "fy": gradient.fy + "%", "r": gradient.r + "%"
                };
                var radial = svg.radialGradient(attr);
                this._renderStops(gradient, svg, radial);
                defs.appendChild(radial);
                return radial.id;
            }
            return null;
        },

        _renderStops: function (gradient, svg, element) {
            if (svg.getElementById(element.getAttribute("id")) === element) {
                while (element.firstChild) {
                    element.removeChild(element.firstChild);
                }
            }
            var stops = gradient.stops;
            var stop;
            for (var i = 0, len = stops.length; i < len; i++) {
                stop = stops[i];
                element.appendChild(svg.stop({ "offset": stop.offset + "%", "stop-color": stop.color, "stop-opacity": stop.opacity }));
            }
        },

        _renderShape: function (node, svg, g) {
            switch (node.shape.type) {
                case "rectangle":
                    this._renderRect(node, svg, g);
                    break;
                case "ellipse":
                    this._renderEllipse(node, svg, g);
                    break;
                case "image":
                    this._renderImage(node, svg, g);
                    break;
                case "path":
                    this._renderPath(node, svg, g);
                    break;
                case "polygon":
                    this._renderPolygon(node, svg, g);
                    break;
                case "text":
                    this._renderTextElement(node, svg, g);
                    break;
                case "html":
                    this._renderHtmlElement(node, svg, g);
                    break;
                case "native":
                    this._rendercontent(node, svg, g);
                    break;
            }
        },

        _rendercontent: function (node, svg, g) {
            //if (node.constraints & ej.datavisualization.Diagram.NodeConstraints.Shadow) {
            //    var fill = node.fillColor;
            //    var shadow = {
            //        "id": node.name + "_shadow", "width": node.width, "height": node.height, "stroke": "lightgrey", "fill": fill != "none" || fill != "transparent" ? "lightgrey" : "none",
            //        "transform": "translate(" + 5 + "," + 5 + ")", "opacity": 0.7
            //    };
            //    g.appendChild(svg.rect(shadow));
            //}
            if (node.shape.content) {
                if (typeof (node.shape.content) == 'string') {
                    var content = document.getElementById(node.shape.content);
                    node.shape.content = content.cloneNode(true);
                }
                if (node.shape.content instanceof SVGElement) {
                    node.shape.content = node.shape.content.cloneNode(true);
                }
                node.shape.content.setAttribute("preserveAspectRatio", "none");
                g.appendChild(node.shape.content);
                var bounds = node.shape.content.getBBox();
                var delwidth = bounds.width ? node.width / bounds.width : 1;
                var delheight = bounds.height ? node.height / bounds.height : 1;
                node.shape.content.setAttribute("transform", "scale(" + delwidth + "," + delheight + ")");
                node.shape.content.setAttribute("id", node.name + "_shape");
                node.shape.content.setAttribute("filter", this._renderFilter(node, svg));
            }
        },

        _fill: function (node, svg) {
            var fill;
            if (node.gradient) {
                fill = this._renderGradient(node.name, node.gradient, svg);
            }
            if (!fill) {
                fill = node.fillColor;
            }
            return fill;
        },

        _renderRect: function (node, svg, g) {
            var fill = this._fill(node, svg);
            var attr = {
                "id": node.name + "_shape", "rx": node.shape.cornerRadius, "ry": node.shape.cornerRadius,
                "width": node.width, "height": node.height, "fill": fill, "stroke": node.borderColor,
                "stroke-width": node.borderWidth, "opacity": node.opacity, "stroke-dasharray": node.borderDashArray, "filter": this._renderFilter(node, svg)
            };
            //if (node.constraints & ej.datavisualization.Diagram.NodeConstraints.Shadow) {
            //    var shadow = {
            //        "id": node.name + "_shadow", "rx": node.shape.cornerRadius, "ry": node.shape.cornerRadius,
            //        "width": node.width, "height": node.height, "stroke": "lightgrey", "fill": fill != "none" || fill != "transparent" ? "lightgrey" : "none",
            //        "transform": "translate(" + 5 + "," + 5 + ")", "opacity": 0.7
            //    };
            //    g.appendChild(svg.rect(shadow));
            //}
            g.appendChild(svg.rect(attr));
        },

        _renderEllipse: function (node, svg, g) {
            var fill = this._fill(node, svg);
            var attr = {
                "id": node.name + "_shape", "rx": node.width / 2, "ry": node.height / 2,
                "cx": node.width / 2, "cy": node.height / 2,
                "fill": fill, "stroke": node.borderColor, "stroke-width": node.borderWidth,
                "opacity": node.opacity, "stroke-dasharray": node.borderDashArray, "filter": this._renderFilter(node, svg)

            };
            //if (node.constraints & ej.datavisualization.Diagram.NodeConstraints.Shadow) {
            //    var shadow = {
            //        "id": node.name + "_shadow", "rx": node.width / 2, "ry": node.height / 2,
            //        "cx": node.width / 2, "cy": node.height / 2, "stroke": "lightgrey", "fill": fill != "none" || fill != "transparent" ? "lightgrey" : "none",
            //        "transform": "translate(" + 5 + "," + 5 + ")", "opacity": 0.7
            //    };
            //    g.appendChild(svg.ellipse(shadow));
            //}
            g.appendChild(svg.ellipse(attr));
        },

        _renderImage: function (node, svg, g) {
            var fill = node.fillColor;
            var backRect = this._renderBackgroundRect(node, svg);
            g.appendChild(backRect);
            var attr = {
                "id": node.name + "_shape", "width": node.width, "height": node.height,
                "opacity": node.opacity, "preserveAspectRatio": "none",
            };
            var shape = svg.image(attr);
            shape.setAttributeNS("http://www.w3.org/1999/xlink", "xlink:href", node.shape.src);
            g.appendChild(shape);
        },

        _renderPolygon: function (node, svg, g) {
            var fill = this._fill(node, svg);
            ej.datavisualization.Diagram.Geometry.updatePolygonPoints(node);
            var points = this._convertToSVGPoints(node.shape.points);
            var attr = {
                "id": node.name + "_shape", "fill": fill, "stroke": node.borderColor,
                "stroke-width": node.borderWidth, "opacity": node.opacity,
                "stroke-dasharray": node.borderDashArray, "points": points, "filter": this._renderFilter(node, svg)
            };
            g.appendChild(svg.polygon(attr));
        },

        _renderPath: function (node, svg, g) {
            var fill = this._fill(node, svg);
            var d = ej.datavisualization.Diagram.Geometry.updatePath(0, 0, node.width, node.height, node.shape.pathData, svg);
            node._absolutePath = d;
            var attr = {
                "id": node.name + "_shape", "width": node.width,
                "height": node.height, "d": d, "opacity": node.opacity,
                "stroke-dasharray": node.borderDashArray, "stroke": node.borderColor,
                "stroke-width": node.borderWidth, "fill": fill, "filter": this._renderFilter(node, svg)

            };
            g.appendChild(svg.path(attr));
        },

        _renderHtmlElement: function (node, svg, g) {
            var backRect = this._renderBackgroundRect(node, svg);
            g.appendChild(backRect);
            if (node.shape.html) {
                var x = node.offsetX - node.width * node.pivot.x;
                var y = node.offsetY - node.height * node.pivot.y;
                var div = document.createElement("div");
                var style = "width:" + node.width + "px;height:" + node.height + "px;padding:1px; opacity:" + node.opacity + ";left:" + x + "px; top:" + y + "px; ";
                if (ej.browserInfo().name === "msie")
                    style += "display: block; position: absolute; border:0px;";
                var attr = { "id": node.name + "_html", "class": "foreignObject", "style": style };
                ej.datavisualization.Diagram.Util.attr(div, attr);
                if (node.shape.html instanceof HTMLElement) {
                    div.appendChild(node.shape.html);
                }
                else
                    div.innerHTML = node.shape.html.toString();
                if (ej.browserInfo().name === "msie") {
                    var parentdiv = document.createElement("div");
                    var attr1 = { "id": node.name + "_parentdiv", "class": "ej-d-node" };
                    ej.datavisualization.Diagram.Util.attr(parentdiv, attr1);
                    parentdiv.appendChild(div);
                    var htmlLayer = svg.document.parentNode.getElementsByClassName("htmlLayer")[0];
                    if (htmlLayer)
                        htmlLayer.appendChild(parentdiv);
                    else
                        svg.document.parentNode.appendChild(div);
                }
                else {
                    attr = { "id": node.name + "_shape", "width": node.width, "height": node.height, "style": "pointer-events:none" };
                    var shape = svg.foreignObject(attr);
                    shape.appendChild(div);
                    g.appendChild(shape);
                }
            }
        },

        _initializeGuidelines: function (svg, attr, parent) {
            var g = svg.g(attr);
            parent.appendChild(g);
            return g;
        },

        _renderSideAlignmentLines: function (start, end, g, svg, scale) {
            var line = svg.line({
                "x1": start.x * scale, "y1": start.y * scale,
                "x2": end.x * scale, "y2": end.y * scale, "stroke": "#07EDE1", "stroke-width": 1
            });
            g.appendChild(line);
        },

        _renderCenterAlignmentLines: function (start, end, g, svg, scale) {
            var line = svg.line({
                "x1": start.x * scale, "y1": start.y * scale,
                "x2": end.x * scale, "y2": end.y * scale, "stroke": "#07EDE1", "stroke-width": 1,
            });
            g.appendChild(line);
        },

        _renderSpacingLines: function (start, end, g, svg, scale) {
            var d, d1;
            var div1 = svg.g({});
            if (start.x == end.x) {
                d = "M" + (start.x * scale - 5) + " " + (start.y * scale + 5) + " L" + start.x * scale + " " + start.y * scale + " L" + (start.x * scale + 5) + " " + (start.y * scale + 5) + "z";
                d1 = "M" + (end.x * scale - 5) + " " + (end.y * scale - 5) + " L" + end.x * scale + " " + end.y * scale + " L" + (end.x * scale + 5) + " " + (end.y * scale - 5) + "z";
                div1.appendChild(svg.line({ "x1": start.x * scale - 8, "x2": start.x * scale + 8, "y1": start.y * scale - 1, "y2": start.y * scale - 1, "stroke": "#07EDE1", "stroke-width": 1 }));
                div1.appendChild(svg.line({ "x1": end.x * scale - 8, "x2": end.x * scale + 8, "y1": end.y * scale + 1, "y2": end.y * scale + 1, "stroke": "#07EDE1", "stroke-width": 1 }));
            }
            else {
                d = "M" + (start.x * scale + 5) + " " + (start.y * scale + 5) + " L" + start.x * scale + " " + start.y * scale + " L" + (start.x * scale + 5) + " " + (start.y * scale - 5) + "z";
                d1 = "M" + (end.x * scale - 5) + " " + (end.y * scale - 5) + " L" + end.x * scale + " " + end.y * scale + " L" + (end.x * scale - 5) + " " + (end.y * scale + 5) + "z";
                div1.appendChild(svg.line({ "x1": start.x * scale - 1, "x2": start.x * scale - 1, "y1": start.y * scale - 8, "y2": start.y * scale + 8, "stroke": "#07EDE1", "stroke-width": 1 }));
                div1.appendChild(svg.line({ "x1": end.x * scale + 1, "x2": end.x * scale + 1, "y1": end.y * scale - 8, "y2": end.y * scale + 8, "stroke": "#07EDE1", "stroke-width": 1 }));
            }
            div1.appendChild(svg.path({ "d": d, "fill": "#07EDE1" }));
            var line = svg.line({
                "x1": start.x * scale, "y1": start.y * scale,
                "x2": end.x * scale, "y2": end.y * scale, "stroke": "#07EDE1", "stroke-width": 0.6, "fill": "#07EDE1"
            });
            div1.appendChild(line);
            div1.appendChild(svg.path({ "d": d1, "fill": "#07EDE1" }));
            g.appendChild(div1);
        },

        _removeGuidelines: function (parent, g) {
            if (g != null) {
                parent.removeChild(g);
            }
        },

        //#endregion

        //#region update
        update: function (data, diagram) {
            var svg = diagram._svg;
            if (data.isGroup)
                this.updateGroup(data, svg, diagram);
            else if (data.segments)
                this.updateConnector(data, svg, diagram);
            else
                this.updateNode(data, svg, diagram);
        },

        _updateGoupBackground: function (group, svg) {
            var x = group.offsetX - group.width / 2;
            var y = group.offsetY - group.height / 2;
            var angle = group.rotateAngle;
            var pt = new ej.datavisualization.Diagram.Point(x + group.width / 2, y + group.height / 2);
            var attr = {
                "id": group.name + "_shape",
                "rx": group.shape.cornerRadius,
                "ry": group.shape.cornerRadius,
                "width": group.width,
                "height": group.height,
                "fill": group.fillColor,
                "stroke": group.borderColor,
                "stroke-width": group.borderWidth,
                "opacity": group.opacity,
                "stroke-dasharray": group.borderDashArray,
                "transform": "rotate(" + angle + "," + pt.x + "," + pt.y + "),translate(" + x + "," + y + ")",
            };
            if (svg.getElementById((group.name + "_shape")))
                ej.datavisualization.Diagram.Util.attr(svg.getElementById((group.name + "_shape")), attr);
        },

        updateGroup: function (group, svg, diagram) {
            var children = group.children;
            var visible = group.visible ? "visible" : "hidden";
            svg.g({ "id": group.name, "visibility": visible });
            for (var i = 0, len = children.length; i < len; i++) {
                var child = diagram.nameTable[children[i]];
                if (child) {
                    if (child.isGroup)
                        this.updateGroup(child, svg, diagram);
                    else if (child.segments)
                        this.updateConnector(child, svg, diagram);
                    else
                        this.updateNode(child, svg, diagram);
                }
            }
            this._updateGoupBackground(group, svg);

            this._updateAssociatedConnector(group, svg, diagram);
            this._updateLabels(group, svg);
            this._updatePorts(group, svg);
            if (group.isSwimlane && diagram.model.phases && diagram.model.phases.length > 0) {
                var des = null;
                for (var j = 0; j < diagram.model.phases.length; j++) {
                    des = diagram.model.phases[j];
                    if (des.parent == group.name)
                        this._updatephase(des, diagram);
                }
            }
        },

        _removephases: function (group, diagram) {
            if (group.isSwimlane && diagram.model.phases && diagram.model.phases.length > 0) {
                var des = null;
                for (var j = diagram.model.phases.length - 1; j >= 0; j--) {
                    des = diagram.model.phases[j];
                    if (des) {
                        var element = diagram._svg.getElementById(des.name + "_phase");
                        element.parentNode.removeChild(element);
                        ej.datavisualization.Diagram.Util.removeItem(diagram.model.phases, des);
                    }
                }
            }
        },

        _updatephase: function (node, diagram) {
            var parNode = diagram.nameTable[node.parent];
            if (parNode) {
                var tx = 0, ty = 0, transform = null;
                var points = [], nPoint;
                var bounds = ej.datavisualization.Diagram.Util.bounds(parNode);
                if (parNode.children.length > 1)
                    var header = diagram.nameTable[parNode.children[0]];
                var top = bounds.top + 50;
                var left = bounds.left + 50;
                var visibility = "visible";
                if (node.orientation == "vertical") {
                    tx = bounds.x;
                    ty = bounds.y + node.offset;
                    if (header)
                        top += header.height;
                    if (ty < bounds.bottom && ty > top) {
                        transform = "translate(" + tx + "," + ty + ")";
                    } else {
                        visibility = "hidden";
                    }
                    points.push({ x: 0, y: 0 });
                    points.push({ x: bounds.width, y: 0 });
                    nPoint = this._convertToSVGPoints(points);
                }
                else {
                    tx = bounds.x + node.offset;
                    ty = bounds.y;
                    if (tx < bounds.right && tx > left) {
                        transform = "translate(" + tx + "," + ty + ")";
                    }
                    else {
                        visibility = "hidden";
                    }
                    points.push({ x: 0, y: header.height });
                    points.push({ x: 0, y: bounds.height });
                    nPoint = this._convertToSVGPoints(points);
                }

                var attr = {
                    "points": nPoint,
                    "id": node.name + "_phase",
                };
                diagram._svg.polyline(attr);
                attr = {
                    "points": nPoint,
                    "id": node.name + "_phase_hitTest",
                };
                diagram._svg.polyline(attr);
                attr = {
                    "id": node.name,
                    "transform": transform,
                    "visibility": visibility
                }
                diagram._svg.g(attr);
                if (diagram._svg.getElementById(attr.id))
                    $('#' + attr.id).insertAfter($('#' + attr.id)[0].parentNode.lastChild);
            }
        },
        updateLabelStyle: function (node, label, svg) {
            var attr = {};
            if (node && svg && label) {
                if (label.bold)
                    attr["font-weight"] = "bold";
                else
                    attr["font-weight"] = "";

                if (label.italic)
                    attr["font-style"] = "italic";
                else
                    attr["font-style"] = "";

                if (label.textDecoration)
                    attr["text-decoration"] = label.textDecoration;
                attr["font-family"] = label.fontFamily;
                attr["font-size"] = label.fontSize;
                attr["fill"] = label.fontColor;
            }
            ej.datavisualization.Diagram.Util.attr(svg.getElementById(node.name + "_" + label.name), attr);
        },

        _updateNodeStyle: function (node, svg) {
            if (node && svg) {
                var attr = {};
                attr["opacity"] = node.opacity;
                attr["fill"] = node.fillColor;
                attr["stroke"] = node.borderColor;
                attr["stroke-width"] = node.borderWidth;
                attr["stroke-dasharray"] = node.borderDashArray;
                if (node.gradient)
                    attr["fill"] = this._renderGradient(node.name, node.gradient, svg);
                var type = node.shape.type == "text" ? "_backRect" : "_shape";
                if (ej.browserInfo().name === "msie" && node.shape && node.shape.type == "html") {
                    //ej.datavisualization.Diagram.Util.attr(svg.getElementById(node.name + "_html"), attr);
                } else {
                    if (svg.getElementById(node.name + type))
                        ej.datavisualization.Diagram.Util.attr(svg.getElementById(node.name + type), attr);
                }
            }
        },

        _updateConnectorStyle: function (connector, svg) {
            if (connector && svg) {
                var attr = {};
                attr["stroke"] = connector.lineColor;
                attr["stroke-width"] = connector.lineWidth;
                attr["stroke-dasharray"] = connector.lineDashArray;
                ej.datavisualization.Diagram.Util.attr(svg.getElementById(connector.name + "_segments"), attr);
            }
        },

        updateTargetDecoratorStyle: function (connector, svg) {
            if (svg) {
                var attr = {};
                attr["stroke"] = connector.targetDecorator.borderColor;
                attr["fill"] = connector.targetDecorator.fillColor;
                ej.datavisualization.Diagram.Util.attr(svg.getElementById(connector.name + "_targetDecorator"), attr);
            }
        },

        updateSourceDecoratorStyle: function (connector, svg) {
            if (svg) {
                var attr = {};
                attr["stroke"] = connector.sourceDecorator.borderColor;
                attr["fill"] = connector.sourceDecorator.fillColor;
                ej.datavisualization.Diagram.Util.attr(svg.getElementById(connector.name + "_sourceDecorator"), attr);
            }
        },

        updateNode: function (node, svg, diagram) {
            var g, bounds;
            if (node.shape.type === "html" && ej.browserInfo().name === "msie") {
                g = document.getElementById(node.name + "_html").getBoundingClientRect();
                bounds = { x: g.left, y: g.top, width: g.width, height: g.height };
            }
            else 
                g = svg.getElementById(node.name + "_shape");
            this._updateNode(node, svg);
            if (diagram._layoutInAction) return;
            this._updateAssociatedConnector(node, svg, diagram);
            if (!diagram.activeTool.inAction || diagram.activeTool.name !== "move" && diagram.activeTool.name !== "rotate") {
                this._updateLabels(node, svg);
                if (g && !bounds)
                    bounds = g.getBBox();
                if (bounds && (bounds && bounds.width != node.width || bounds.height != node.height)) {
                    this._updatePorts(node, svg);
                }
            }
        },

        updateTextBlock: function (node, label, svg) {
            var attr = {};
            var label = node.shape.textBlock;
            node.shape.textBlock = label;
            if (node && svg && label) {
                if (label.bold)
                    attr["font-weight"] = "bold";
                else
                    attr["font-weight"] = "";

                if (label.italic)
                    attr["font-style"] = "italic";
                else
                    attr["font-style"] = "";

                if (label.textDecoration)
                    attr["text-decoration"] = label.textDecoration;
                attr["font-family"] = label.fontFamily;
                attr["font-size"] = label.fontSize;
                attr["fill"] = label.fontColor;
            }
            ej.datavisualization.Diagram.Util.attr(svg.getElementById(node.name + "_shape"), attr);
            this._updateTextElement(node, svg);
        },

        _updateAssociatedConnector: function (node, svg, diagram) {
            var i, len;
            for (i = 0, len = node.inEdges.length; i < len; i++) {
                if (diagram.nameTable[node.inEdges[i]]) {
                    ej.datavisualization.Diagram.Util.updateBridging(diagram.nameTable[node.inEdges[i]], diagram);
                    ej.datavisualization.Diagram.DiagramContext.update(diagram.nameTable[node.inEdges[i]], diagram);
                    diagram._updateConnectorBridging(diagram.nameTable[node.inEdges[i]]);
                }
            }
            for (i = 0, len = node.outEdges.length; i < len; i++) {
                if (diagram.nameTable[node.outEdges[i]]) {
                    ej.datavisualization.Diagram.Util.updateBridging(diagram.nameTable[node.outEdges[i]], diagram);
                    ej.datavisualization.Diagram.DiagramContext.update(diagram.nameTable[node.outEdges[i]], diagram);
                    diagram._updateConnectorBridging(diagram.nameTable[node.outEdges[i]]);
                }
            }
        },

        _updateNode: function (node, svg) {
            var x = node.offsetX - node.width * node.pivot.x;
            var y = node.offsetY - node.height * node.pivot.y;
            var angle = node.rotateAngle;
            var pt = ej.datavisualization.Diagram.Point(node.offsetX, node.offsetY);
            var visible = node.visible ? "visible" : "hidden";
            var attr = ({
                "id": node.name, "transform": "rotate(" + angle + "," + pt.x + "," + pt.y + "),translate(" + x + "," + y + ")",
                "visibility": visible
            });
            svg.g(attr);
            switch (node.shape.type) {
                case "rectangle":
                    svg.rect({
                        "id": node.name + "_shape", "width": node.width, "height": node.height,
                    });
                    break;
                case "ellipse":
                    svg.ellipse({
                        "id": node.name + "_shape", "rx": node.width / 2, "ry": node.height / 2,
                        "cx": node.width / 2, "cy": node.height / 2
                    });
                    break;
                case "image":
                    this._updateBackgroundRect(node, svg);
                    svg.image({
                        "id": node.name + "_shape", "width": node.width, "height": node.height,
                    });
                    break;
                case "path":
                    var d = ej.datavisualization.Diagram.Geometry.updatePath(0, 0, node.width, node.height, node.shape.pathData, svg);
                    node._absolutePath = d;
                    svg.path({
                        "id": node.name + "_shape", "d": d,
                    });
                    break;
                case "polygon":
                    this._updatePolygon(node, svg);
                    break;
                case "text":
                    this._updateTextElement(node, svg);
                    break;
                case "html":
                    this._updateHtmlElement(node, svg);
                    break;
                case "native":
                    var bounds = node.shape.content.getBBox();
                    var delwidth = bounds.width ? node.width / bounds.width : 1;
                    var delheight = bounds.height ? node.height / bounds.height : 1;
                    svg.g({
                        "id": node.name + "_shape", "width": node.width, "height": node.height,
                        "transform": "scale(" + delwidth + "," + delheight + ")"
                    });
                    break;
            }
        },

        _updateHtmlElement: function (node, svg) {
            this._updateBackgroundRect(node, svg);
            var div;
            if (ej.browserInfo().name === "msie") {
                var x = node.offsetX - node.width * node.pivot.x;
                var y = node.offsetY - node.height * node.pivot.y;
                div = document.getElementById(node.name + "_html");
                div.style.left = x + "px";
                div.style.top = y + "px";
                div.style.transform = "rotate(" + node.rotateAngle + "deg)";
            }
            else {
                div = svg.getElementById(node.name + "_shape");
                svg.foreignObject({
                    "id": node.name, "width": node.width, "height": node.height
                });
            }
            div.style.width = node.width.toString() + "px";
            div.style.height = node.height.toString() + "px";
        },

        _updatePolygon: function (node, svg) {
            ej.datavisualization.Diagram.Geometry.updatePolygonPoints(node);
            svg.polygon({
                "id": node.name + "_shape", "points": this._convertToSVGPoints(node.shape.points)
            });
        },

        _convertToSVGPoints: function (points) {
            var pts = "";
            for (var i = 0, len = points.length; i < len; i++) {
                pts += points[i].x + "," + points[i].y + " ";
            }
            return pts.trim();
        },

        _renderTextElement: function (node, svg, g) {
            var textElement = node.shape.textBlock;
            if (textElement) {
                var backRect = this._renderBackgroundRect(node, svg);
                g.appendChild(backRect);
                var attr = {
                    "id": node.name + "_shape", "class": "ej-d-label", "font-family": textElement.fontFamily,
                    "font-size": textElement.fontSize, "fill": textElement.fontColor,
                    "text-decoration": textElement.textDecoration, "opacity": node.opacity,
                    "alignment-baseline": "middle"
                };
                if (textElement.bold)
                    attr["font-weight"] = "bold";
                if (textElement.italic)
                    attr["font-style"] = "italic";
                var text = svg.text(attr);
                text.appendChild(document.createTextNode(textElement.text));
                g.appendChild(text);
                var bounds = ej.datavisualization.Diagram.Util.bounds(node);
                this._wrapText(node, bounds, text, node.shape.textBlock, svg);
                this._alignTextOnLabel(node, bounds, text, node.shape.textBlock, svg);
                this._renderShadow(node, svg, g);
            }
        },

        _updateTextElement: function (node, svg) {
            this._updateBackgroundRect(node, svg);
            var text = svg.getElementById(node.name + "_shape");
            var bounds = ej.datavisualization.Diagram.Util.bounds(node);
            this._wrapText(node, bounds, text, node.shape.textBlock, svg);
            this._alignTextOnLabel(node, bounds, text, node.shape.textBlock, svg);
        },

        _renderBackgroundRect: function (node, svg) {
            var fill = this._fill(node, svg);
            var attr = {
                "id": node.name + "_backRect", "class": "backrect", "x": 0, "y": 0,
                "width": node.width, "height": node.height, "fill": fill,
                "stroke": node.borderColor, "stroke-width": node.borderWidth, "opacity": node.opacity,
                "stroke-dasharray": node.borderDashArray, "filter": this._renderFilter(node, svg)
            };
            return svg.rect(attr);
        },

        _updateBackgroundRect: function (node, svg) {
            var attr = {
                "id": node.name + "_backRect", "x": 0, "y": 0,
                "width": node.width, "height": node.height,
            };
            svg.rect(attr);
        },

        _hideNode: function (node, svg) {
            var attr = { "id": node.name, "style": "pointer-events:none;" };
            svg.rect(attr);
        },

        _showNode: function (node, svg) {
            var attr = { "id": node.name, "style": "pointer-events:auto;" };
            svg.rect(attr);
        },

        _drawContainerHelper: function (diagram) {
            var shape = diagram.activeTool.helper;
            var svg = diagram._svg;
            var scale = diagram._currZoom;
            if (shape.segments)
                var bounds = ej.datavisualization.Diagram.Util.bounds(shape);
            var height = shape.height ? shape.height : bounds.height;
            var width = shape.width ? shape.width : bounds.width;
            var offsetX = shape.offsetX ? (shape.offsetX) : (bounds.x + bounds.width / 2);
            var offsetY = shape.offsetY ? (shape.offsetY) : (bounds.y + bounds.height / 2);
            var rotateAngle = shape.rotateAngle ? shape.rotateAngle : 0;
            var rect = svg.rect({
                "id": "helper", "class": "helper",
                "stroke-width": 1, "stroke-dasharray": "3,3", "pointer-events": "none",
                "width": width * scale, "height": height * scale, "fill": "transparent", "stroke": "red",
                "transform": "translate(" + (offsetX - width / 2) * scale + "," + (offsetY - height / 2) * scale + "),rotate(" + rotateAngle + "," + (width / 2) * scale + "," + (height / 2) * scale + ")"
            });
            diagram._adornerLayer.appendChild(rect);
        },

        _updateContainerHelper: function (diagram) {

            var attr = null;
            var shape = diagram.activeTool.helper;
            var scale = diagram._currZoom;
            if (shape.segments)
                var bounds = ej.datavisualization.Diagram.Util.bounds(shape);
            var height = shape.height ? shape.height : bounds.height;
            var width = shape.width ? shape.width : bounds.width;
            var offsetX = shape.offsetX ? (shape.offsetX) : (bounds.x + bounds.width / 2);
            var offsetY = shape.offsetY ? (shape.offsetY) : (bounds.y + bounds.height / 2);
            var rotateAngle = shape.rotateAngle ? shape.rotateAngle : 0;
            attr = {
                "id": "helper", "class": "helper",
                "width": width * scale, "height": height * scale, "fill": "transparent", "stroke": "red",
                "transform": "translate(" + (offsetX - width / 2) * scale + "," + (offsetY - height / 2) * scale + "),rotate(" + rotateAngle + "," + (width / 2) * scale + "," + (height / 2) * scale + ")"
            };
            diagram._svg.rect(attr);
        },

        _removeContainerHelper: function (node, svg, parent) {
            if (svg.getElementById("helper"))
                svg.removeChild(svg.getElementById("helper"), parent);
        },
        //#endregion

        setNodeShape: function (node, svg, parent) {
            var g;
            if (parent) {
                var parentg = svg.getElementById(parent.name);
                g = $(parentg).find("#" + node.name)[0];
            }
            else {
                g = svg.getElementById(node.name);
            }
            $(g).empty();
            this._renderNode(node, svg, g);
        },

        setLine: function (connector, svg, parent, diagram) {
            var g;
            if (parent) {
                var parentg = svg.getElementById(parent.name);
                g = $(parentg).find("#" + connector.name)[0];
            }
            else {
                g = svg.getElementById(connector.name);
            }
            $(g).empty();

            this.renderConnector(connector, svg, g, diagram);
        },

        addNodeLabel: function (node, label, svg, parent) {
            var g;
            if (parent) {
                var parentg = svg.getElementById(parent.name);
                g = $(parentg).find("#" + node.name)[0];
            }
            else {
                g = svg.getElementById(node.name);
            }
            this._renderLabel(node, label, svg, g);
        },
        //#endregion

        //#region Render Connector
        renderConnector: function (connector, svg, parent, diagram) {
            var g;
            var line;
            var visible = connector.visible ? "visible" : "hidden";
            //if (connector.visible) 
            {
                g = svg.g({ "id": connector.name, "class": "ej-d-connector", "visibility": visible });
                if (parent) {
                    if (parent.id !== g.id) {
                        parent.appendChild(g);
                    }
                }
                else {
                    svg.appendChild(g);
                }
                var seg = svg.g({ "id": connector.name + "segments" });
                this._renderSegments(connector, svg, seg, diagram);

                g.appendChild(seg);
                this._renderLabels(connector, svg, g);
                this._renderDecorators(connector, svg, g, diagram);
            }
            return g;
        },
        _renderSegments: function (connector, svg, g, diagram) {
            var visibility = (ej.browserInfo().name === "msie") ? "collapse" : "hidden";

            var path = this._findPath(connector, diagram);
            var attr = {
                "id": connector.name + "_segments",
                "d": path, "fill": "none", "stroke": connector.lineColor,
                "stroke-width": connector.lineWidth, "stroke-dasharray": connector.lineDashArray, "opacity": connector.opacity
            };
            var line = svg.path(attr);
            g.appendChild(line);
            attr = { "id": connector.name + "_hitTest", "stroke-width": connector.lineHitPadding, "d": path, "pointer-events": "stroke", "stroke": "transparent", "visibility": visibility };
            line = svg.path(attr);
            g.appendChild(line);
        },

        _findPath: function (connector, diagram) {
            var st, end, path;
            for (var i = 0; i < connector.segments.length; i++) {
                var seg = connector.segments[i];
                if (seg._bridges.length > 0) {
                    for (var n = 0; n < seg._bridges.length; n++) {
                        var bridge = seg._bridges[n];
                        bridge._rendered = false;
                    }
                }
                var points = seg.points;
                if (i == 0) {
                    if (diagram && connector.sourceNode) var node = diagram._findNode(connector.sourceNode);
                    points = this._clipDecorators(connector, seg, true, node);
                    if (seg.type == "bezier" && connector.sourceDecorator && connector.sourceDecorator.shape !== ej.datavisualization.Diagram.DecoratorShapes.None) {
                        points[0] = ej.datavisualization.Diagram.Util._adjustPoint(points[0], seg._point1, true, connector.lineWidth);
                    }
                    path = "M" + points[0].x + " " + points[0].y;
                }
                if (i == connector.segments.length - 1) {
                    if (diagram && connector.targetNode) var node = diagram._findNode(connector.targetNode);
                    points = this._clipDecorators(connector, seg, false, node);
                }
                if (seg.type != "bezier") {
                    if (connector.cornerRadius > 0) {
                        for (var j = 0; j < points.length - 1; j++) {
                            var segLength = ej.datavisualization.Diagram.Geometry.distance(points[j], points[j + 1]);
                            if (segLength > 0) {
                                if (i < connector.segments.length - 1 || j < points.length - 2) {
                                    if (segLength < connector.cornerRadius * 2) {
                                        end = ej.datavisualization.Diagram.Util._adjustPoint(points[j], points[j + 1], false, segLength / 2);
                                    }
                                    else end = ej.datavisualization.Diagram.Util._adjustPoint(points[j], points[j + 1], false, connector.cornerRadius);
                                }
                                else end = points[j + 1];

                                if (i > 0 || j > 0) {
                                    if (segLength < connector.cornerRadius * 2) {
                                        st = ej.datavisualization.Diagram.Util._adjustPoint(points[j], points[j + 1], true, segLength / 2);
                                        if (i < connector.segments.length - 1 || j < points.length - 2)
                                            end = null;
                                    }
                                    else st = ej.datavisualization.Diagram.Util._adjustPoint(points[j], points[j + 1], true, connector.cornerRadius);
                                }

                                if (st) {
                                    path += " Q" + points[j].x + " " + points[j].y + " " + st.x + " " + st.y;
                                }
                                if (end) {
                                    if (seg._bridges.length > 0) {
                                        path = this._updateBridging(seg, path, j);
                                        if (seg.type === "orthogonal")
                                            path = this._updateBridging(seg, path, j + 1);
                                    }
                                    path += " L" + end.x + " " + end.y;
                                }
                            }
                        }
                    }
                    else {
                        for (var j = 0; j < points.length; j++) {
                            if (j > 0) {
                                path = this._updateBridging(seg, path, j);
                                path += " L" + points[j].x + " " + points[j].y;
                            }
                        }
                    }
                }
                else {
                    var endPoint = seg._endPoint;
                    if (connector.targetDecorator && connector.targetDecorator.shape !== ej.datavisualization.Diagram.DecoratorShapes.None) {
                        endPoint = ej.datavisualization.Diagram.Util._adjustPoint(seg._endPoint, seg._point2, true, connector.lineWidth);
                    }
                    if (connector.targetNode && diagram) {
                        var targetNode = diagram._findNode(connector.targetNode);
                        if (node.borderColor != "none") 
                            endPoint = ej.datavisualization.Diagram.Util._adjustPoint(endPoint, seg._point2, true, targetNode.borderWidth / 2);
                    }
                    path += " C" + seg._point1.x + " " + seg._point1.y + " " + seg._point2.x + " " + seg._point2.y + " " + endPoint.x + " " + endPoint.y;
                }
            }
            return path;
        },

        _updateBridging: function (seg, path, pointIndex) {
            var pathData = path;
            if (seg._bridges.length > 0) {
                if (seg.type === "straight") {
                    for (var n = 0; n < seg._bridges.length; n++) {
                        var bridge = seg._bridges[n];
                        if (!bridge._rendered) {
                            pathData += " L" + bridge.startPoint.x + " " + bridge.startPoint.y;
                            pathData += bridge.path;
                        }
                    }
                }
                else if (seg.type === "orthogonal") {
                    for (var n = 0; n < seg._bridges.length; n++) {
                        var bridge = seg._bridges[n];
                        if (bridge.segmentPointIndex === pointIndex && !bridge._rendered) {
                            pathData += " L" + bridge.startPoint.x + " " + bridge.startPoint.y;
                            pathData += bridge.path;
                            bridge._rendered = true;
                        }
                    }
                }
            }
            return pathData;
        },

        _refreshSegments: function (connector, svg, diagram) {
            this._updateConnector(connector, svg, diagram);
            this._updateDecorators(connector, svg, diagram);
        },

        _clipDecorators: function (connector, segment, source, node) {
            var points = $.extend(true, [], segment.points);
            if (!source && connector.targetDecorator && connector.targetDecorator.shape !== ej.datavisualization.Diagram.DecoratorShapes.None) {
                var start, end;
                start = points[points.length - 1];
                end = points[points.length - 2];
                var len = ej.datavisualization.Diagram.Geometry.distance(start, end);
                len = (len == 0) ? 1 : len;
                var width = 2 * connector.lineWidth;
                var newPoint = ej.datavisualization.Diagram.Point();
                newPoint.x = start.x + width * (end.x - start.x) / len;
                newPoint.y = start.y + width * (end.y - start.y) / len;
                if (node && node.borderColor != "none")
                    newPoint = ej.datavisualization.Diagram.Util._adjustPoint(newPoint, end, true, node.borderWidth / 2);
                points[points.length - 1] = newPoint;
            }
            else if (source && connector.sourceDecorator && connector.sourceDecorator.shape !== ej.datavisualization.Diagram.DecoratorShapes.None) {
                start = points[0];
                end = points[1];
                var len = ej.datavisualization.Diagram.Geometry.distance(start, end);
                len = (len == 0) ? 1 : len;
                var width = 2 * connector.lineWidth;
                var newPoint = ej.datavisualization.Diagram.Point();
                newPoint.x = start.x + width * (end.x - start.x) / len;
                newPoint.y = start.y + width * (end.y - start.y) / len;
                if (segment.type == "bezier") {
                    newPoint.x = start.x + width * (segment._point1.x - start.x) / len;
                    newPoint.y = start.y + width * (segment._point1.y - start.y) / len;
                }
                if (node && node.borderColor != "none")
                    newPoint = ej.datavisualization.Diagram.Util._adjustPoint(newPoint, segment.type != "bezier" ? end : segment._point1, true, node.borderWidth / 2);
                points[0] = newPoint;
            }
            else if (source) {
                if (node && node.borderColor != "none")
                    points[0] = ej.datavisualization.Diagram.Util._adjustPoint(points[0], points[1], true, node.borderWidth / 2);
            }
            else {
                if (node && node.borderColor != "none")
                    points[points.length - 1] = ej.datavisualization.Diagram.Util._adjustPoint(points[points.length - 1], points[points.length - 2], true, node.borderWidth / 2);
            }
            return points;
        },

        _updateConnector: function (connector, svg, diagram) {
            var path = this._findPath(connector, diagram);
            svg.path({ "id": connector.name + "_segments", "d": path });
            svg.path({ "id": connector.name + "_hitTest", "d": path });
        },

        updateConnector: function (connector, svg, diagram) {
            //var svg = diagram._svg;
            var visible = connector.visible ? "visible" : "hidden";
            svg.g({ "id": connector.name, "visibility": visible });
            this._updateConnector(connector, svg, diagram);
            this._updateLabels(connector, svg);
            this._updateDecorators(connector, svg, diagram);
        },

        renderDecorators: function (connector, svg, diagram) {
            var g = svg.getElementById(connector.name);
            this._renderDecorators(connector, svg, g, diagram);
        },
        clearDecorators: function (connector, svg, diagram) {
            this._updateConnector(connector, svg, diagram);
            var g = svg.getElementById(connector.name);
            var dec = svg.getElementById(connector.name + "_targetDecorator");
            if (dec)
                svg.removeChild(dec, g);
            dec = svg.getElementById(connector.name + "_sourceDecorator");
            if (dec)
                svg.removeChild(dec, g);
        },
        //#endregion

        //#region Render Decorator
        _renderDecorators: function (connector, svg, g, diagram) {
            var startPoint;
            var endPoint;
            if (connector.targetDecorator && connector.targetDecorator.shape) {
                var segment = connector.segments[connector.segments.length - 1];
                startPoint = segment.points[segment.points.length - 2];
                endPoint = connector.targetPoint;
                if (segment.type == "bezier") {
                    startPoint = segment._point2;
                }
                if (connector.targetNode && diagram) {
                    var targetNode = diagram._findNode(connector.targetNode);
                    if (targetNode.borderColor != "none")
                        endPoint = ej.datavisualization.Diagram.Util._adjustPoint(endPoint, startPoint, true, targetNode.borderWidth / 2);
                }
                this._renderDecorator(connector.name + "_targetDecorator", endPoint,
                    startPoint, connector, connector.targetDecorator, svg, g);
            }
            if (connector.sourceDecorator && connector.sourceDecorator.shape) {
                var segment = connector.segments[0];
                startPoint = connector.sourcePoint;
                endPoint = segment.points[1];
                if (segment.type == "bezier") {
                    endPoint = segment._point1;
                }
                if (connector.sourceNode && diagram) {
                    var sourceNode = diagram._findNode(connector.sourceNode);
                    if (sourceNode.borderColor != "none")
                        startPoint = ej.datavisualization.Diagram.Util._adjustPoint(startPoint, endPoint, true, sourceNode.borderWidth / 2);
                }
                this._renderDecorator(connector.name + "_sourceDecorator", startPoint,
                    endPoint, connector, connector.sourceDecorator, svg, g);
            }
        },

        _renderDecorator: function (name, point1, point2, connector, decorator, svg, g) {
            var shape;
            var attr;
            var d, x;
            var size = ej.datavisualization.Diagram.Size(decorator.width, decorator.height);
            switch (decorator.shape) {
                case "arrow":
                    d = this._constructArrow(point1, size);
                    attr = { "id": name, "class": "decorator", "d": d, "fill": decorator.fillColor, "stroke": decorator.borderColor, "stroke-width": connector.lineWidth };
                    shape = svg.path(attr);
                    x = 1.1 * connector.lineWidth;
                    break;
                case "openarrow":
                    d = this._constructArrow(point1, size, true);
                    attr = { "id": name, "class": "decorator", "d": d, "fill": "transparent", "stroke": decorator.borderColor, "stroke-width": connector.lineWidth };
                    shape = svg.path(attr);
                    x = 1.1 * connector.lineWidth;
                    break;
                case "circle":
                    var rx = size.width / 2;
                    var ry = size.height / 2;
                    attr = {
                        "id": name, "class": "decorator", "rx": rx, "ry": ry, "cx": (point1.x + size.width / 2), "cy": (point1.y),
                        "fill": decorator.fillColor, "stroke": decorator.borderColor, "stroke-width": connector.lineWidth
                    };
                    shape = svg.ellipse(attr);
                    x = 0.5 * connector.lineWidth;
                    break;
                case "diamond":
                    d = this._constructDiamond(point1, size, svg);
                    attr = {
                        "id": name, "class": "decorator", "x": point1.x, "y": point1.y, "width": size.width,
                        "height": size.height, "d": d, "fill": decorator.fillColor, "stroke": decorator.borderColor, "stroke-width": connector.lineWidth
                    };
                    shape = svg.path(attr);
                    x = 0.7 * connector.lineWidth;
                    break;
                case "path":
                    d = ej.datavisualization.Diagram.Geometry.updatePath(point1.x, point1.y - size.height / 2, size.width, size.height, decorator.pathData, svg);
                    attr = {
                        "id": name, "class": "decorator", "x": point1.x, "y": point1.y, "width": size.width,
                        "height": size.height, "d": d, "fill": decorator.fillColor, "stroke": decorator.borderColor, "stroke-width": connector.lineWidth
                    };
                    shape = svg.path(attr);
                    x = 0.5 * connector.lineWidth;
                    break;
            }
            //apply translate matrix and rotate matrix here
            if (shape) {
                if (decorator.borderColor != "none") {
                    var angle = ej.datavisualization.Diagram.Geometry.findAngle(point1, point2);
                    var matrix = ej.Matrix.identity();
                    ej.Matrix.rotate(matrix, angle);
                    var extra = ej.Matrix.transform(matrix, { x: x, y: 0 });
                    shape.setAttribute("transform", "translate(" + extra.x + "," + extra.y + "),rotate(" + angle + " " + point1.x + " " + point1.y + ")");
                }
                else
                    shape.setAttribute("transform", "rotate(" + ej.datavisualization.Diagram.Geometry.findAngle(point1, point2) + " " + point1.x + " " + point1.y + ")");
                g.appendChild(shape);
            }
        },

        _updateDecorators: function (connector, svg, diagram) {
            var startPoint;
            var endPoint;
            if (connector.targetDecorator && connector.targetDecorator.shape !== ej.datavisualization.Diagram.DecoratorShapes.None) {
                var segment = connector.segments[connector.segments.length - 1];
                var points = segment.points;
                startPoint = points[points.length - 2];
                endPoint = connector.targetPoint;
                if (segment.type == "bezier") {
                    startPoint = segment._point2;
                }
                if (connector.targetNode && diagram) {
                    var targetNode = diagram._findNode(connector.targetNode);
                    if (targetNode.borderColor != "none")
                        endPoint = ej.datavisualization.Diagram.Util._adjustPoint(connector.targetPoint, startPoint, true, targetNode.borderWidth / 2);
                }
                this._updateDecorator(connector.name + "_targetDecorator", endPoint, startPoint, connector, connector.targetDecorator, svg);
            }
            if (connector.sourceDecorator && connector.sourceDecorator.shape !== ej.datavisualization.Diagram.DecoratorShapes.None) {
                var segment = connector.segments[0];
                var points = segment.points;
                startPoint = connector.sourcePoint;
                endPoint = points[1];
                if (segment.type == "bezier") {
                    endPoint = segment._point1;
                }
                if (connector.sourceNode && diagram) {
                    var sourceNode = diagram._findNode(connector.sourceNode);
                    if (sourceNode.borderColor != "none")
                        startPoint = ej.datavisualization.Diagram.Util._adjustPoint(connector.sourcePoint, endPoint, true, sourceNode.borderWidth / 2);
                }
                this._updateDecorator(connector.name + "_sourceDecorator", startPoint, endPoint, connector, connector.sourceDecorator, svg);
            }
        },

        _updateDecorator: function (name, point1, point2, connector, decorator, svg) {
            var shape;
            var attr;
            var d, x;
            var size = ej.datavisualization.Diagram.Size(decorator.width, decorator.height);
            switch (decorator.shape) {
                case "arrow":
                    d = this._constructArrow(point1, size);
                    attr = { "id": name, "d": d };
                    shape = svg.path(attr);
                    x = 1.1 * connector.lineWidth;
                    break;
                case "openarrow":
                    d = this._constructArrow(point1, size, true);
                    attr = { "id": name, "d": d };
                    shape = svg.path(attr);
                    x = 1.1 * connector.lineWidth;
                    break;
                case "circle":
                    var rx = size.width / 2;
                    var ry = size.height / 2;
                    attr = { "id": name, "rx": rx, "ry": ry, "cx": (point1.x + size.width / 2), "cy": (point1.y) };
                    shape = svg.ellipse(attr);
                    x = 0.5 * connector.lineWidth;
                    break;
                case "diamond":
                    d = this._constructDiamond(point1, size, svg);
                    attr = { "id": name, "x": point1.x, "y": point1.y, "width": size.width, "height": size.height, "d": d };
                    shape = svg.path(attr);
                    x = 0.7 * connector.lineWidth;
                    break;
                case "path":
                    d = ej.datavisualization.Diagram.Geometry.updatePath(point1.x, point1.y - size.height / 2, size.width, size.height, decorator.pathData, svg);
                    attr = {
                        "id": name, "x": point1.x, "y": point1.y, "width": size.width, "height": size.height, "d": d
                    };
                    shape = svg.path(attr);
                    x = 0.5 * connector.lineWidth;
                    break;
            }
            //apply translate matrix and rotate matrix here
            if (shape) {
                if (decorator.borderColor != "none") {
                    var angle = ej.datavisualization.Diagram.Geometry.findAngle(point1, point2);
                    var matrix = ej.Matrix.identity();
                    ej.Matrix.rotate(matrix, angle);
                    var extra = ej.Matrix.transform(matrix, { x: x, y: 0 });
                    shape.setAttribute("transform", "translate(" + extra.x + "," + extra.y + "),rotate(" + angle + " " + point1.x + " " + point1.y + ")");
                }
                else
                 shape.setAttribute("transform", "rotate(" + ej.datavisualization.Diagram.Geometry.findAngle(point1, point2) + " " + point1.x + " " + point1.y + ")");
            }
        },

        _constructArrow: function (point, size, isOpen) {
            var path = new ej.datavisualization.Diagram.Path();
            path.moveTo(point.x + size.width, point.y + size.height / 2);
            path.lineTo(point.x, point.y);
            path.lineTo(point.x + size.width, point.y - size.height / 2);
            if (!isOpen) {
                path.close();
            }
            return path.toString();
        },

        _constructDiamond: function (point, size, svg) {
            var path = new ej.datavisualization.Diagram.Path();
            path.moveTo(point.x + size.width, point.y);
            path.lineTo(point.x + size.width / 2, point.y + size.height / 2);
            path.lineTo(point.x, point.y);
            path.lineTo(point.x + size.width / 2, point.y - size.height / 2);
            path.lineTo(point.x + size.width, point.y);
            var d = path.toString();
            return d;
        },
        //#endregion

        //#region Render Label
        _renderLabels: function (node, svg, g) {
            var labels = node.labels;
            for (var i = 0, len = labels.length; i < len; ++i) {
                this._renderLabel(node, labels[i], svg, g);
            }
        },

        _renderLabelBackground: function (label, node, text, svg) {
            var bounds;
            if (ej.browserInfo().name === "mozilla")
                bounds = text.getBoundingClientRect();
            else
                bounds = text.getBBox();
            var attr = {
                "id": node.name + "_" + label.name + "_lblbg",
                "width": bounds.width, "height": bounds.height, "fill": label.fillColor,
                "stroke": label.borderColor, "stroke-width": label.borderWidth, "transform": "translate(" + 0 + "," + 0 + ")"
            };
            return svg.rect(attr);
        },

        _renderLabel: function (node, label, svg, g) {
            var attr = {
                "id": node.name + "_" + label.name, "class": "ej-d-label", "font-family": label.fontFamily,
                "font-size": label.fontSize, "fill": label.fontColor, "text-decoration": label.textDecoration,
                "dominant-baseline": "middle", "pointer-events": "none"
            };
            if (label.bold)
                attr["font-weight"] = "bold";
            if (label.italic)
                attr["font-style"] = "italic";
            var text = svg.text(attr);
            text.textContent = label.text;
            g.appendChild(text);
            var bg = this._renderLabelBackground(label, node, text, svg);
            g.insertBefore(bg, text);
            var bounds = ej.datavisualization.Diagram.Util.bounds(node);
            this._wrapText(node, bounds, text, label, svg);
            this._alignTextOnLabel(node, bounds, text, label, svg);
        },

        _updateLabels: function (node, svg) {
            var labels = node.labels;
            for (var i = 0, len = labels.length; i < len; ++i) {
                var label = labels[i];
                var text = svg.getElementById(node.name + "_" + label.name);
                if (text && text.textContent != "") {
                    text.setAttribute("fill", label.fontColor);
                    var bounds = ej.datavisualization.Diagram.Util.bounds(node);
                    this._wrapText(node, bounds, text, label, svg);
                    this._alignTextOnLabel(node, bounds, text, label, svg);
                }
            }
        },

        updateLabel: function (node, label, svg) {
            var text = svg.getElementById(node.name + "_" + label.name);
            text.setAttribute("fill", label.fontColor);
            text.textContent = label.text;
            var bounds = ej.datavisualization.Diagram.Util.bounds(node);
            this._wrapText(node, bounds, text, label, svg);
            this._alignTextOnLabel(node, bounds, text, label, svg);
        },

        _alignTextOnLabel: function (node, nodeBounds, text, label, svg) {
            var bounds = text.getBBox();
            var offset = ej.datavisualization.Diagram.Util._getLabelPosition(label, nodeBounds);
            var point = { x: 0, y: 0 };
            var y = 0;

            if (label.verticalAlignment == ej.datavisualization.Diagram.VerticalAlignment.Top)
                y = offset.y;
            else if (label.verticalAlignment == ej.datavisualization.Diagram.VerticalAlignment.Center)
                y = offset.y - bounds.height / 2;
            else
                y = offset.y - bounds.height;
            point.y = y;

            switch (label.horizontalAlignment) {
                case ej.datavisualization.Diagram.HorizontalAlignment.Left:
                    switch (label.textAlign) {
                        case ej.datavisualization.Diagram.TextAlign.Left:
                            point.x = offset.x;
                            break;
                        case ej.datavisualization.Diagram.TextAlign.Center:
                            point.x = offset.x + bounds.width / 2;
                            break;
                        case ej.datavisualization.Diagram.TextAlign.Right:
                            point.x = offset.x + bounds.width;
                            break;
                    }
                    break;
                case ej.datavisualization.Diagram.HorizontalAlignment.Center:
                    switch (label.textAlign) {
                        case ej.datavisualization.Diagram.TextAlign.Left:
                            point.x = offset.x - bounds.width / 2;
                            break;
                        case ej.datavisualization.Diagram.TextAlign.Center:
                            point.x = offset.x;
                            break;
                        case ej.datavisualization.Diagram.TextAlign.Right:
                            point.x = offset.x + bounds.width / 2;
                            break;
                    }
                    break;
                case ej.datavisualization.Diagram.HorizontalAlignment.Right:
                    switch (label.textAlign) {
                        case ej.datavisualization.Diagram.TextAlign.Left:
                            point.x = offset.x - bounds.width;
                            break;
                        case ej.datavisualization.Diagram.TextAlign.Center:
                            point.x = offset.x - bounds.width / 2;
                            break;
                        case ej.datavisualization.Diagram.TextAlign.Right:
                            point.x = offset.x;
                            break;
                    }
                    break;
            }
            if (node.segments || node.isGroup) {
                point.x = point.x + nodeBounds.x;
                point.y = point.y + nodeBounds.y;
                offset.x += nodeBounds.x;
                offset.y += nodeBounds.y;
            }
            if (ej.browserInfo().name === "mozilla")
                point.y = point.y - label.fontSize / 2;
            text.setAttribute("transform", "translate(" + point.x + "," + point.y + "), rotate(" + label.rotateAngle + "," + (offset.x - point.x) + "," + (offset.y - point.y) + ")");
            //Updates the label background
            var background = svg.getElementById(node.name + "_" + label.name + "_lblbg");
            if (background) {
                var x = point.x + bounds.x;
                y = point.y;
                if (ej.browserInfo().name === "mozilla")
                    y = point.y + label.fontSize / 2;
                var attr = {
                    "width": bounds.width, "height": bounds.height, "transform": "translate(" + x + "," + y + "), rotate(" + label.rotateAngle + "," + (offset.x - x) + "," + (offset.y - y) + ")", "fill": label.fillColor,
                    "stroke": label.borderColor, "stroke-width": label.borderWidth
                };
                ej.datavisualization.Diagram.Util.attr(background, attr);
            }
        },

        _wrapText: function (node, textBBox, text, label, svg) {
            var str = label.text; var attr = null;
            while (text.hasChildNodes()) {
                text.removeChild(text.lastChild);
            }
            var bbWidth, bbHeight;
            if (node.segments) {
                bbWidth = textBBox.width;
                bbHeight = textBBox.height;
            }
            else {
                bbWidth = textBBox.width - label.margin.left - label.margin.right;
                bbHeight = textBBox.height - label.margin.top - label.margin.bottom;
            }
            bbWidth = bbWidth < label.width ? label.width : bbWidth;
            var eachLine = str.split('\n');
            var x, y, tspan, j, string = "", posX, posY, childNodes, bounds, i;
            if (svg.getElementById(text.id))
                childNodes = svg.getElementById(text.id).childNodes;
            if (childNodes) {
                for (j = 0; j < eachLine.length; j++) {
                    tspan = svg.tspan();
                    tspan.style.fontSize = label.fontSize;
                    text.appendChild(tspan);
                    for (i = 0; i < eachLine[j].length; i++) {
                        bounds = childNodes[childNodes.length - 1].getComputedTextLength();
                        if (label.wrapText) {
                            if (bounds >= (bbWidth - label.fontSize)) {
                                tspan.textContent += eachLine[j][i];
                                posX = -bounds;
                                if (!(i == eachLine[j].length - 1)) {
                                    tspan = svg.tspan();
                                    text.appendChild(tspan);
                                }
                            }
                            else {
                                tspan.textContent += eachLine[j][i];
                            }
                        }
                        else {
                            tspan.textContent += eachLine[j][i];
                        }
                    }
                }
                this._wrapTextAlign(text, childNodes, label.fontSize, label.textAlign);
            }
        },

        _wrapTextAlign: function (text, childNodes, height, textAlign) {
            for (var i = 0; i < childNodes.length; i++) {
                var x = childNodes[i].getComputedTextLength();
                switch (textAlign) {
                    case "left":
                        x = 0;
                        break;
                    case "center":
                        x = -x / 2;
                        break;
                    case "right":
                        x = -x;
                        break;
                }
                var attr = { "x": Number(x), "dy": height };
                var tspan = childNodes[i];
                ej.datavisualization.Diagram.Util.attr(tspan, attr);
            }
        },
        //#endregion

        //#region Render Port
        _renderPorts: function (node, svg, g) {
            var ports = node.ports;
            var port;
            for (var i = 0, len = ports.length; i < len; ++i) {
                port = ports[i];
                var shape = this._renderPort(node, port, svg);
                g.appendChild(shape);
                if (port.visibility & ej.datavisualization.Diagram.PortVisibility.Hidden || port.visibility & ej.datavisualization.Diagram.PortVisibility.Hover || port.visibility & ej.datavisualization.Diagram.PortVisibility.Connect) {
                    shape.setAttribute("visibility", "hidden");
                }
            }
        },

        _renderPort: function (node, port, svg) {
            var shape;
            var attr;
            var size = ej.datavisualization.Diagram.Size(port.size, port.size);
            var point;
            if (node.isGroup)
                point = ej.datavisualization.Diagram.Util._getPortPosition(port, ej.datavisualization.Diagram.Util.bounds(node, true), false);
            else
                point = ej.datavisualization.Diagram.Util._getPortPosition(port, ej.datavisualization.Diagram.Util.bounds(node, true), true);
            switch (port.shape) {
                case "x":
                    var d = this._constructX(point, port.size);
                    attr = {
                        "id": node.name + "_" + port.name, "class": "ej-d-port", "fill": port.fillColor,
                        "stroke": port.borderColor, "stroke-width": port.borderWidth, "d": d
                    };
                    shape = svg.path(attr);
                    break;
                case "circle":
                    var rx = size.width / 2;
                    var ry = size.height / 2;
                    attr = {
                        "id": node.name + "_" + port.name, "class": "ej-d-port", "rx": rx, "ry": ry,
                        "cx": point.x, "cy": point.y, "fill": port.fillColor, "stroke": port.borderColor,
                        "stroke-width": port.borderWidth
                    };
                    shape = svg.ellipse(attr);
                    break;
                case "square":
                    attr = {
                        "id": node.name + "_" + port.name, "class": "ej-d-port", "x": point.x - port.size / 2,
                        "y": point.y - port.size / 2, "width": size.width, "height": size.height,
                        "fill": port.fillColor, "stroke": port.borderColor, "stroke-width": port.borderWidth
                    };
                    shape = svg.rect(attr);
                    break;
            }
            return shape;
        },
        _insertPort: function (node, port, svg) {
            var g;
            g = document.getElementById(node.name);
            var portshape = ej.datavisualization.Diagram.SvgContext._renderPort(node, port, svg);
            g.appendChild(portshape);
            if (port.visibility & ej.datavisualization.Diagram.PortVisibility.Hidden || port.visibility & ej.datavisualization.Diagram.PortVisibility.Hover || port.visibility & ej.datavisualization.Diagram.PortVisibility.Connect) {
                portshape.setAttribute("visibility", "hidden");
            }

        },
        _updatePort: function (node, port, svg) {
            var size = ej.datavisualization.Diagram.Size(port.size, port.size);
            var point = null;
            if (node.isGroup)
                point = ej.datavisualization.Diagram.Util._getPortPosition(port, ej.datavisualization.Diagram.Util.bounds(node, true), false);
            else
                point = ej.datavisualization.Diagram.Util._getPortPosition(port, ej.datavisualization.Diagram.Util.bounds(node, true), true);
            var id = node.name + "_" + port.name;
            switch (port.shape) {
                case "x":
                    var d = this._constructX(point, port.size);
                    svg.path({ "id": id, "d": d });
                    break;
                case "circle":
                    var rx = size.width / 2;
                    var ry = size.height / 2;
                    svg.ellipse({ "id": id, "rx": rx, "ry": ry, "cx": point.x, "cy": point.y });
                    break;
                case "square":
                    svg.rect({ "id": id, "x": point.x - port.size / 2, "y": point.y - port.size / 2 });
                    break;
            }
        },

        _updatePorts: function (node, svg) {
            var ports = node.ports;
            for (var i = 0, len = ports.length; i < len; ++i) {
                var port = ports[i];
                this._updatePort(node, port, svg);
            }
        },

        _constructX: function (point, size) {
            var path = new ej.datavisualization.Diagram.Path();
            path.moveTo((point.x - size / 2), (point.y - size / 2));
            path.lineTo((point.x + size / 2), (point.y + size / 2));
            path.moveTo((point.x + size / 2), (point.y - size / 2));
            path.lineTo((point.x - size / 2), (point.y + size / 2));
            return path.toString();
        },
        _enableSelectedNode: function (node, svg, nameTable) {

            if (node.isPseudoGroup) {
                var children = node.children;
                for (var i = 0, len = children.length; i < len; i++) {
                    var child = nameTable[children[i]];
                    if (child)
                        this._enableSelectedNode(child, svg);
                }
            } else {
                var attr = {
                    "id": node.name,
                    "pointer-events": "auto",
                    "style": "pointer-events:block",
                }
                svg.g(attr);
            }
        },
        _disableSelectedNode: function (node, svg, nameTable) {

            if (node.isPseudoGroup) {
                var children = node.children;
                for (var i = 0, len = children.length; i < len; i++) {
                    var child = nameTable[children[i]];
                    if (child)
                        this._disableSelectedNode(child, svg);
                }
            } else {
                var attr = {
                    "id": node.name,
                    "style": "pointer-events:none",
                    "pointer-events": "none",
                }
                svg.g(attr);
            }
        },
        _drawNodeHighlighter: function (node, svg, parent, scale) {
            var shape;
            var x = node.offsetX - node.width * node.pivot.x;
            var y = node.offsetY - node.height * node.pivot.y;
            var center = ej.datavisualization.Diagram.Point(node.offsetX - node.width * node.pivot.x + node.width / 2, node.offsetY - node.height * node.pivot.y + node.height / 2);
            var transform = "rotate(" + node.rotateAngle + "," + center.x * scale + "," + center.y * scale + ")";
            var attr = {
                "id": "nodeHighlighter",
                "class": "ej-d-node",
                "x": x * scale,
                "y": y * scale,
                "width": node.width * scale,
                "height": node.height * scale,
                "fill": "transparent",
                "stroke": "red",
                "stroke-width": 3,
                "pointer-events": "none",
                "transform": transform,
                "style": "pointer-events: none",
            };
            shape = svg.rect(attr);
            parent.appendChild(shape);
            return shape;
        },

        _removeNodeHighlighter: function (svg, parent) {
            var highlighter = svg.getElementById("nodeHighlighter");
            if (highlighter) {
                svg.removeChild(highlighter, parent);
            }
        },
        _drawPortHighlighter: function (port, node, svg, parent, scale) {
            var shape, size, point, x, y, transform;
            var bounds;
            var fill = "#8CC63F";
            var border = "#1A1A1A";
            var borderWidth = 0.5;
            if (!port) {
                bounds = ej.datavisualization.Diagram.Util.bounds(node, true);
                size = ej.datavisualization.Diagram.Size(bounds.width, bounds.height);
                point = ej.datavisualization.Diagram.Point(bounds.x, bounds.y);
                x = point.x * scale;
                y = point.y * scale;
                border = fill;
                fill = "transparent";
                borderWidth = 2;
                var center = ej.datavisualization.Diagram.Point(node.offsetX - node.width * node.pivot.x + node.width / 2, node.offsetY - node.height * node.pivot.y + node.height / 2);
                transform = "rotate(" + node.rotateAngle + "," + center.x * scale + "," + center.y * scale + ")";
            }
            else {
                size = ej.datavisualization.Diagram.Size(port.size * 2, port.size * 2);
                point = ej.datavisualization.Diagram.Util._getPortPosition(port, ej.datavisualization.Diagram.Util.bounds(node, true));
                var matrix = ej.Matrix.identity();
                ej.Matrix.rotate(matrix, node.rotateAngle, node.offsetX, node.offsetY);
                point = ej.Matrix.transform(matrix, point);
                x = (point.x - port.size) * scale;
                y = (point.y - port.size) * scale;
                transform = "rotate(" + 0 + "," + (x + size.width / 2) + "," + (y + size.height / 2) + ")";
            }
            var attr = {
                "id": "portHighlighter", "class": "ej-d-port", "x": x,
                "y": y, "width": size.width * scale, "height": size.height * scale,
                "fill": fill, "stroke": border, "stroke-width": borderWidth, "pointer-events": "none", "transform": transform
            };
            shape = svg.rect(attr);
            parent.appendChild(shape);
            return shape;
        },

        _removePortHighlighter: function (svg, parent) {
            var highlighter = svg.getElementById("portHighlighter");
            if (highlighter) {
                svg.removeChild(highlighter, parent);
            }
        },
        //#endregion

        //#region Render Selector

        renderSelector: function (shape, svg, parent, scale, constraints) {
            if (shape.segments) {
                var g = svg.g({
                    "id": svg.document.id + "handle_g", "class": "handle"
                });
                parent.appendChild(g);
                this._renderEndPointHandle(shape, svg, scale);
            }
            else {
                var g = svg.g({
                    "id": svg.document.id + "handle_g", "class": "handle",
                    "transform": "translate(" + (shape.offsetX - shape.width / 2) * scale + "," + (shape.offsetY - shape.height / 2) * scale + "),rotate(" + shape.rotateAngle + "," + (shape.width / 2) * scale + "," + (shape.height / 2) * scale + ")"
                });
                parent.appendChild(g);
                this._renderResizeHandle(shape, svg, scale, constraints);
            }
        },

        _renderResizeHandle: function (node, svg, scale, constraints) {

            var rotate = ej.datavisualization.Diagram.Point(node.width / 2, -20 / scale);
            if (constraints & ej.datavisualization.Diagram.SelectorConstraints.Rotator) {
                this._renderPivotLine(node, svg, scale);
                this._renderRotateThumb("rotate", node, rotate.x, rotate.y, scale, svg);
            }
            if (constraints & ej.datavisualization.Diagram.SelectorConstraints.Resizer) {
                this._renderResizeBorder("resizeBorder", node, svg, scale);

                this._renderResizeCorner("nw-resize", node, 0, 0, svg, !(node.constraints & ej.datavisualization.Diagram.NodeConstraints.ResizeNorthWest));
                this._renderResizeCorner("n-resize", node, (node.width / 2) * scale, 0, svg, !(node.constraints & ej.datavisualization.Diagram.NodeConstraints.ResizeNorth));
                this._renderResizeCorner("ne-resize", node, node.width * scale, 0, svg, !(node.constraints & ej.datavisualization.Diagram.NodeConstraints.ResizeNorthEast));
                this._renderResizeCorner("w-resize", node, 0, (node.height / 2) * scale, svg, !(node.constraints & ej.datavisualization.Diagram.NodeConstraints.ResizeWest));
                this._renderResizeCorner("e-resize", node, node.width * scale, (node.height / 2) * scale, svg, !(node.constraints & ej.datavisualization.Diagram.NodeConstraints.ResizeEast));
                this._renderResizeCorner("sw-resize", node, 0, node.height * scale, svg, !(node.constraints & ej.datavisualization.Diagram.NodeConstraints.ResizeSouthWest));
                this._renderResizeCorner("s-resize", node, (node.width / 2) * scale, node.height * scale, svg, !(node.constraints & ej.datavisualization.Diagram.NodeConstraints.ResizeSouth));
                this._renderResizeCorner("se-resize", node, node.width * scale, node.height * scale, svg, !(node.constraints & ej.datavisualization.Diagram.NodeConstraints.ResizeSouthEast));
            }
        },

        renderUserHandles: function (handles, shape, svg, isMultipleSelection, scale, parent) {
            for (var handleIndex = 0; handleIndex < handles.length; handleIndex++) {
                if ((isMultipleSelection && handles[handleIndex].enableMultiSelection) || !isMultipleSelection) {
                    if (handles[handleIndex].visible)
                        this._renderUserHandle(handles[handleIndex], shape, svg, scale, parent);
                }
            }
        },

        _renderUserHandle: function (handle, node, svg, scale, parent) {
            var g = svg.g({ "id": svg.document.id + "userHandle_g", "class": "userHandle" });
            parent.appendChild(g);
            var position = this._getHandlePosition(handle, node, scale);
            var d = ej.datavisualization.Diagram.Geometry.updatePath(position.x * scale - handle.size / 4, position.y * scale - handle.size / 4, (handle.size / 2), (handle.size / 2), handle.pathData, svg);
            var shape = this._getHandleShape(handle, position.x * scale, position.y * scale);
            var icon = this._getHandleIcon(handle, d, position.x, position.y);
            g.appendChild(svg.circle(shape));
            g.appendChild(svg.path(icon));
        },

        _getHandleIcon: function (handle, d) {
            var icon = {
                "id": handle.name + "_icon", "class": "userHandle-icon", "d": d, "fill": handle.pathColor
            };
            return icon;
        },

        _getHandlePosition: function (handle, node, scale) {
            var matrix = ej.Matrix.identity();
            if (node.rotateAngle)
                ej.Matrix.rotate(matrix, 0);
            var positionPoints = ej.datavisualization.Diagram.Point();
            var bounds = ej.datavisualization.Diagram.Util.bounds(node);
            var position = handle.position;
            var distance = 10;           
            var point = new ej.datavisualization.Diagram.Point();
            switch (position) {
                case "topleft":
                    point = ej.Matrix.transform(matrix, ej.datavisualization.Diagram.Point(distance + (handle.size / 2), distance + (handle.size / 2)));
                    positionPoints.x = bounds.topLeft.x - (point.x / scale);
                    positionPoints.y = bounds.topLeft.y - (point.y / scale);
                    break;
                case "topcenter":
                    point = ej.Matrix.transform(matrix, ej.datavisualization.Diagram.Point(0, distance + (handle.size / 2)));
                    positionPoints.x = bounds.topCenter.x - (point.x / scale);
                    positionPoints.y = bounds.topCenter.y - point.y / scale;
                    break;
                case "topright":
                    point = ej.Matrix.transform(matrix, ej.datavisualization.Diagram.Point(distance + (handle.size / 2), (distance / 2) - (handle.size)));
                    positionPoints.x = bounds.topRight.x + (point.x / scale);
                    positionPoints.y = bounds.topRight.y + (point.y / scale);
                    break;
                case "middleleft":
                    point = ej.Matrix.transform(matrix, ej.datavisualization.Diagram.Point(-(distance + (handle.size / 2)), 0));
                    positionPoints.x = bounds.middleLeft.x + (point.x / scale);
                    positionPoints.y = bounds.middleLeft.y + (point.y / scale);
                    break;
                case "middleright":
                    point = ej.Matrix.transform(matrix, ej.datavisualization.Diagram.Point(distance + (handle.size / 2), 0));
                    positionPoints.x = bounds.middleRight.x + (point.x / scale);
                    positionPoints.y = bounds.middleRight.y + (point.y / scale);
                    break;
                case "bottomleft":
                    point = ej.Matrix.transform(matrix, ej.datavisualization.Diagram.Point(-(distance + (handle.size / 2)), distance + (handle.size / 2)));
                    positionPoints.x = bounds.bottomLeft.x + (point.x / scale);
                    positionPoints.y = bounds.bottomLeft.y + (point.y / scale);
                    break;
                case "bottomcenter":
                    point = ej.Matrix.transform(matrix, ej.datavisualization.Diagram.Point(0, ((distance / 2) - (handle.size))));
                    positionPoints.x = bounds.bottomCenter.x - (point.x / scale);
                    positionPoints.y = bounds.bottomCenter.y - (point.y / scale);
                    break;
                case "bottomright":
                    point = ej.Matrix.transform(matrix, ej.datavisualization.Diagram.Point(distance + (handle.size / 2), distance + (handle.size / 2)));
                    positionPoints.x = bounds.bottomRight.x + (point.x / scale);
                    positionPoints.y = bounds.bottomRight.y + (point.y / scale);
                    break;
            }
            return positionPoints;
        },

        _getHandleShape: function (handle, cx, cy) {
            var shape = {
                "id": handle.name + "_shape", "class": "userHandle", "cx": cx, "cy": cy, "r": handle.size / 2, "fill": handle.backgroundColor,
                "stroke": handle.borderColor, "stroke-width": handle.borderWidth, "title": handle.name
            };
            return shape;
        },

        _updateHandleShape: function (handle, cx, cy, svg) {
            var attr = { "id": handle.name + "_shape", "cx": cx, "cy": cy };
            svg.circle(attr);
        },

        _updateHanleIcon: function (handle, d, svg) {
            var attr = { "id": handle.name + "_icon", "d": d };
            svg.path(attr);
        },

        _getHanleIcon: function (handle, d, cx, cy) {
            var icon = {
                "id": handle.name, "d": d, "stroke": handle.borderColor,
                "stroke-width": handle.borderWidth, "fill": handle.fillColor
            };
            return icon;
        },

        renderPivotPoint: function (node, scale, svg) {
            if (!svg.getElementById("pivot")) {
                ej.datavisualization.Diagram.SvgContext._renderResizeCorner("pivot", node,
                      node.offsetX * scale, node.offsetY * scale, svg);
                this._updatePivotLine(node, svg, scale);
            }
        },

        _renderPivotLine: function (node, svg, scale) {
            if (!node.container) {
                var rotateThumb = ej.datavisualization.Diagram.Point(node.width / 2, -20 / scale);
                var pivot = ej.datavisualization.Diagram.Point(node.width / 2, 0);
                var handle = svg.getElementById(svg.document.id + "handle_g");
                var line = svg.line({
                    "id": "pivotLine", "x1": rotateThumb.x * scale, "y1": rotateThumb.y * scale, "x2": pivot.x * scale, "y2": pivot.y * scale,
                    "stroke": "black", "stroke-width": 1, "stroke-dasharray": "2,3", "fill": "none"
                });
                handle.appendChild(line);
            }
        },

        _renderResizeBorder: function (id, node, svg, scale) {
            var handle = svg.getElementById(svg.document.id + "handle_g");
            var rect = svg.rect({
                "id": id, "width": node.width * scale, "height": node.height * scale, "stroke": "#097F7F", "stroke-width": 0.6, "stroke-dasharray": "6,3", "fill": "none"
            });
            handle.appendChild(rect);
        },

        _renderResizeCorner: function (corner, node, cx, cy, svg, state) {
            var handle = svg.getElementById(svg.document.id + "handle_g");
            var pEvnts = "default";
            var fill = "white";
            if (!(ej.datavisualization.Diagram.Util.canResize(node))) {
                fill = "darkgray";
            }
            if (state) {
                fill = "darkgray";
                pEvnts = "none";
            }
            var attr = { "id": corner, "fill": fill, "stroke": "black", "stroke-width": 1, "cx": cx, "cy": cy, "r": 7, "pointer-events": pEvnts };
            handle.appendChild(svg.circle(attr));
            attr = { "id": corner + "_transparent", "class": corner, "fill": "transparent", "cx": cx, "cy": cy, "r": 10, "pointer-events": pEvnts };
            handle.appendChild(svg.circle(attr));
        },

        _renderRotateThumb: function (corner, node, cx, cy, scale, svg) {
            if (!node.container) {
                var handle = svg.getElementById(svg.document.id + "handle_g");
                var d = "M 16.856 10.239 L 18 3.438 l -2.189 0.817 c -0.974 -1.694 -2.482 -2.995 -4.326 -3.696 C 9.375 -0.245 7.071" +
                    "-0.18 5.01 0.744 C 2.942 1.668 1.365 3.342 0.558 5.453 c -0.803 2.115 -0.738 4.414 0.185 6.478 c 0.925 2.064 " +
                "2.6 3.645 4.714 4.45 c 0.969 0.371 1.993 0.554 3.013 0.554 c 1.345 0 2.685 -0.317 3.897 -0.948 l -1.016 -1.962 c -1.584 0.822" +
                "-3.445 0.929 -5.114 0.293 c -1.56 -0.596 -2.793 -1.762 -3.479 -3.289 C 2.078 9.502 2.031 7.803 2.622 6.244 C 3.216 4.68 4.387" +
                " 3.443 5.914 2.761 C 7.437 2.079 9.137 2.03 10.7 2.626 c 1.246 0.475 2.271 1.328 2.986 2.424 L 11.54 5.851 L 16.856 10.239 Z";
                var fill = "#231f20";
                if (!(ej.datavisualization.Diagram.Util.canRotate(node)))
                    fill = "darkgray";
                var width = 20;
                var height = 20;
                var dist = 20;
                var attr = {
                    "id": "rotatehandle", "class": corner, "fill": fill, "stroke": "black", "stroke-width": 0.5, "width": width, "height": height, "d": d,
                    "transform": "translate(" + (cx * scale - width / 2) + "," + (cy * scale - height) + ")"
                };
                var path = svg.path(attr);
                var circle = svg.circle({
                    "id": "rotate", "class": corner, "fill": "transparent", "stroke": "none", "cx": width / 2, "cy": height / 2, "r": 13,
                    "transform": "translate(" + (cx * scale - width / 2) + "," + (cy * scale - height) + ")"
                });
                handle.appendChild(path);
                handle.appendChild(circle);
            }
        },

        _refreshEndPointHandles: function (shape, svg, scale) {
            if (shape.segments) {
                var handle = svg.getElementById(svg.document.id + "handle_g");
                $(handle).empty();
                this._renderEndPointHandle(shape, svg, scale);
            }
        },
        _renderEndPointHandle: function (connector, svg, scale) {
            for (var i = 0; i < connector.segments.length; i++) {
                var segment = connector.segments[i];
                if (segment.type == "bezier") {
                    var enabled = ej.datavisualization.Diagram.Util.canDragSegmentThumbs(connector);
                    this._renderBezierLine("bezierline1", segment._startPoint, segment._point1, scale, svg);
                    this._renderBezierLine("bezierline2", segment._endPoint, segment._point2, scale, svg);
                    this._renderEndPointCorner("bezierpoint1_" + i, segment._point1.x * scale, segment._point1.y * scale, false, svg, enabled);
                    this._renderEndPointCorner("bezierpoint2_" + i, segment._point2.x * scale, segment._point2.y * scale, false, svg, enabled);
                }
                var points = segment.points;
                if (i == 0) {
                    this._renderEndPointCorner("sourceEndPoint", segment._startPoint.x * scale, segment._startPoint.y * scale, ej.datavisualization.Diagram.Util.isSourceConnected(connector), svg,
                          ej.datavisualization.Diagram.Util.canDragSourceEnd(connector));
                }
                if (i == connector.segments.length - 1) {
                    this._renderEndPointCorner("targetEndPoint", segment._endPoint.x * scale, segment._endPoint.y * scale, ej.datavisualization.Diagram.Util.isTargetConnected(connector), svg,
                        ej.datavisualization.Diagram.Util.canDragTargetEnd(connector));
                    if (segment.type == "orthogonal") {
                        this._renderTerminalOrthoThumbs(segment, svg, scale, i);
                    }
                }
                else {
                    if (segment.type != 'orthogonal') {
                        this._renderSegmentEndThumb("segmentEnd_" + i, segment._endPoint.x * scale, segment._endPoint.y * scale, ej.datavisualization.Diagram.Util.isSourceConnected(connector), svg,
                               ej.datavisualization.Diagram.Util.canDragSourceEnd(connector));
                    } else {
                        this._renderTerminalOrthoThumbs(segment, svg, scale, i);
                    }

                }
            }
        },

        _renderTerminalOrthoThumbs: function (segment, svg, scale, index, orientation) {
            if (segment.points.length > 2)
                for (var i = 0; i < segment.points.length - 1; i++) {
                    var length = ej.datavisualization.Diagram.Geometry.distance(segment.points[i], segment.points[i + 1]);
                    if (segment.points[i].y == segment.points[i + 1].y) {
                        orientation = "horizontal";
                    }
                    else orientation = "vertical";
                    var visible;
                    if (length >= 50) visible = "visible"; else visible = "hidden";
                    this._renderOrthogonalThumb(("OrthoThumb_" + i) + "_" + index, ((segment.points[i].x + segment.points[i + 1].x) / 2) * scale, ((segment.points[i].y + segment.points[i + 1].y) / 2) * scale, svg, visible, orientation);
                }
            else {
                var length = segment.length || ej.datavisualization.Diagram.Geometry.distance(segment._startPoint, segment._endPoint);
                if (length >= 50) visible = "visible"; else visible = "hidden";
                if (segment._startPoint.y == segment._endPoint.y) {
                    orientation = "horizontal";
                }
                else orientation = "vertical";
                this._renderOrthogonalThumb("OrthoThumb_" + index, ((segment._endPoint.x + segment._startPoint.x) / 2) * scale, ((segment._endPoint.y + segment._startPoint.y) / 2) * scale, svg, visible, orientation);
            }
        },

        _renderEndPointCorner: function (corner, x, y, isConnected, svg, isenabled) {
            var handle = svg.getElementById(svg.document.id + "handle_g");
            var fill = isConnected ? "#8CC63F" : "white";
            if (!isenabled) fill = "darkgray";
            var attr = { "id": corner, "class": corner, opacity: 0.75, "fill": fill, "stroke": "black", "stroke-width": 2, "cx": x, "cy": y, "r": 7 };
            handle.appendChild(svg.circle(attr));
        },

        _renderSegmentEndThumb: function (corner, x, y, isConnected, svg, isenabled) {
            var handle = svg.getElementById(svg.document.id + "handle_g");
            var fill = "#e2e2e2";
            var attr = { "id": corner, "class": "segmentEnd", "fill": fill, "stroke": "black", "stroke-width": 2, "cx": x, "cy": y, "r": 5 };
            handle.appendChild(svg.circle(attr));
        },
        _renderOrthogonalThumb: function (corner, x, y, svg, visibility, orientation) {
            var path, h, v; if (orientation == "horizontal") {
                path = "M0,7 L15,0 L30,7 L15,14 z"; h = -15; v = -7;
            } else { path = "M7,0 L0,15 L7,30 L14,15 z"; h = -7; v = -15; };
            var handle = svg.getElementById(svg.document.id + "handle_g");
            var fill = "#e2e2e2";
            var attr = {
                "id": corner, "class": "segmentEnd", "fill": fill, "stroke": "black", "stroke-width": 1, d: path, transform: "translate(" + (x + h) + "," + (y + v) + ")",
                "visibility": visibility
            };
            handle.appendChild(svg.path(attr));
        },
        _renderBezierLine: function (id, start, end, scale, svg) {
            var line = svg.line({
                "id": id, "x1": start.x * scale, "y1": start.y * scale, "x2": end.x * scale, "y2": end.y * scale,
                "stroke": "black", "stroke-width": 1, "stroke-dasharray": "3,3", "fill": "none"
            });
            var handle = svg.getElementById(svg.document.id + "handle_g");
            handle.appendChild(line);
        },
        _renderControlPointCorner: function (corner, point, index, svg) {
            var g = svg.g({ "class": corner });
            svg.getElementById(svg.document.id + "handle_g").appendChild(g);
            var attr = {
                "id": corner + "_" + index, "fill": "white", "stroke": "black", "stroke-width": 2,
                "cx": point.x, "cy": point.y, "r": 5
            };
            g.appendChild(svg.circle(attr));
        },
        _removePivotPoint: function (node, svg, scale) {
            var parent = svg.getElementById(svg.document.id + "handle_g");
            if (parent != null && svg.getElementById("pivot")) {
                parent.removeChild(svg.getElementById("pivot"));
                this._updatePivotLine(node, svg, scale);
            }
        },
        updateSelector: function (shape, svg, scale, diagram, constraints) {
            if (shape.segments) {
                this._updateEndPointHandle(shape, svg, scale);
                if (constraints & ej.datavisualization.Diagram.SelectorConstraints.UserHandles)
                    this.updateUserHandles(diagram.model.userHandles, shape, svg, false, false, scale);
            } else {
                var g = svg.g({
                    "id": svg.document.id + "handle_g",
                    "transform": "translate(" + (shape.offsetX - shape.width / 2) * scale + "," + (shape.offsetY - shape.height / 2) * scale + "),rotate(" + shape.rotateAngle + "," + (shape.width / 2) * scale + "," + (shape.height / 2) * scale + ")"
                });
                //if ((diagram.activeTool.name != "move" && diagram.activeTool.name != "rotate") || shape.line || shape.isGroup || shape.isPseudoGroup)
                this._updateResizeHandle(shape, svg, scale, constraints);
                if (diagram.model.userHandles && diagram.model.userHandles.length > 0) {
                    var isMultipleSelection = false;
                    if (diagram.selectionList[0] && diagram.selectionList[0].name == "multipleSelection")
                        isMultipleSelection = true;
                    if (constraints & ej.datavisualization.Diagram.SelectorConstraints.UserHandles)
                        this.updateUserHandles(diagram.model.userHandles, shape, svg, isMultipleSelection, false, scale);
                }
            }
        },
        _updateResizeHandle: function (node, svg, scale, constraints) {
            var corner = node._corners;
            var rotate = ej.datavisualization.Diagram.Point(node.width / 2, -20 / scale);
            if (constraints & ej.datavisualization.Diagram.SelectorConstraints.Rotator) {
                this._updateRotateHandle("rotate", rotate.x, rotate.y, scale, svg, node);
                this._updatePivotLine(node, svg, scale);
            }
            //this._updateResizeCorner("pivot", node.offsetX * scale, node.offsetY * scale, svg);
            if (constraints & ej.datavisualization.Diagram.SelectorConstraints.Resizer) {
                this._updateResizeCorner("nw-resize", 0, 0, svg);
                this._updateResizeCorner("n-resize", (node.width / 2) * scale, 0, svg);
                this._updateResizeCorner("ne-resize", node.width * scale, 0, svg);
                this._updateResizeCorner("w-resize", 0, (node.height / 2) * scale, svg);
                this._updateResizeCorner("e-resize", node.width * scale, (node.height / 2) * scale, svg);
                this._updateResizeCorner("sw-resize", 0, node.height * scale, svg);
                this._updateResizeCorner("s-resize", (node.width / 2) * scale, node.height * scale, svg);
                this._updateResizeCorner("se-resize", node.width * scale, node.height * scale, svg);
                this._updateResizeBorder("resizeBorder", node, svg, scale);
            }
        },
        _updateRotateHandle: function (thumb, cx, cy, scale, svg, node) {
            var width = 20;
            var height = 20;
            var dist = 20;
            var attr = {
                "id": "rotatehandle",
                "transform": "translate(" + (cx * scale - width / 2) + "," + (cy * scale - height) + ")"
            };
            svg.path(attr);
            svg.circle({
                "id": "rotate",
                "transform": "translate(" + (cx * scale - width / 2) + "," + (cy * scale - height) + ")"
            });
        },
        _updatePivotLine: function (node, svg, scale) {
            var rotateThumb = ej.datavisualization.Diagram.Point(node.width / 2, -20 / scale);
            var pivot = ej.datavisualization.Diagram.Point(node.width / 2, 0);
            var line = svg.line({
                "id": "pivotLine", "x1": rotateThumb.x * scale, "y1": rotateThumb.y * scale, "x2": pivot.x * scale, "y2": pivot.y * scale
            });
        },
        _updateResizeBorder: function (id, node, svg, scale) {
            svg.rect({
                "id": id, "width": node.width * scale, "height": node.height * scale,
            });
        },

        _updateResizeCorner: function (corner, cx, cy, svg) {
            var attr = { "id": corner, "cx": cx, "cy": cy };
            svg.circle(attr);
            var attr = { "id": corner + "_transparent", "cx": cx, "cy": cy };
            svg.circle(attr);
        },

        _updateEndPointHandle: function (connector, svg, scale) {
            for (var i = 0; i < connector.segments.length; i++) {
                var segment = connector.segments[i];
                var points = segment.points;
                if (i == connector.segments.length - 1) {
                    this._updateEndPointCorner("targetEndPoint", points[points.length - 1].x * scale, points[points.length - 1].y * scale, ej.datavisualization.Diagram.Util.isTargetConnected(connector), svg,
                ej.datavisualization.Diagram.Util.canDragTargetEnd(connector));
                    if (segment.type == "orthogonal") {
                        this._updateTerminalOrthoThumbs(segment, svg, scale, i);
                    }
                }
                else {
                    if (segment.type != "orthogonal") {
                        this._updateSegmentEndThumb("segmentEnd_" + i, segment._endPoint.x * scale, segment._endPoint.y * scale, ej.datavisualization.Diagram.Util.isSourceConnected(connector), svg,
                            ej.datavisualization.Diagram.Util.canDragSourceEnd(connector));
                    }
                    else {
                        this._updateTerminalOrthoThumbs(segment, svg, scale, i);
                    }
                }
                if (i == 0)
                    this._updateEndPointCorner("sourceEndPoint", points[0].x * scale, points[0].y * scale, ej.datavisualization.Diagram.Util.isSourceConnected(connector), svg,
                        ej.datavisualization.Diagram.Util.canDragSourceEnd(connector));
                if (segment.type == "bezier") {
                    var enabled = ej.datavisualization.Diagram.Util.canDragSegmentThumbs(connector);
                    this._updateEndPointCorner("bezierpoint1_" + i, segment._point1.x * scale, segment._point1.y * scale, false, svg, enabled);
                    this._updateEndPointCorner("bezierpoint2_" + i, segment._point2.x * scale, segment._point2.y * scale, false, svg, enabled);
                    this._updateBezierLine("bezierline1", segment._startPoint, segment._point1, scale, svg);
                    this._updateBezierLine("bezierline2", segment._endPoint, segment._point2, scale, svg);
                }

            }
            //if (points.length > 2) {
            //    for (var i = 1, len = points.length; i < len - 1; ++i) {
            //        this._updateControlPointCorner("controlPoint", points[i], i, svg);
            //    }
            //}
        },

        _updateEndPointCorner: function (corner, x, y, isConnected, svg, isenabled) {
            var fill = isConnected ? "#8CC63F" : "white";
            if (!isenabled) fill = "darkgray";
            var attr = { "id": corner, "fill": fill, "cx": x, "cy": y };
            svg.circle(attr);
        },
        _updateSegmentEndThumb: function (corner, x, y, isConnected, svg, isenabled) {
            var attr = { "id": corner, "cx": x, "cy": y };
            svg.circle(attr);
        },
        _updateOrthoThumb: function (corner, x, y, svg, visibility, orientation) {
            var path, h, v; if (orientation == "horizontal") {
                path = "M0,7 L15,0 L30,7 L15,14 z"; h = -15; v = -7;
            } else { path = "M7,0 L0,15 L7,30 L14,15 z"; h = -7; v = -15; };
            var attr = { "id": corner, visibility: visibility, transform: "translate(" + (x + h) + "," + (y + v) + ")", d: path };
            svg.path(attr);
        },
        _updateTerminalOrthoThumbs: function (segment, svg, scale, index) {
            var orientation;
            if (segment.points.length > 2)
                for (var i = 0; i < segment.points.length - 1; i++) {
                    var length = ej.datavisualization.Diagram.Geometry.distance(segment.points[i], segment.points[i + 1]);
                    if (segment.points[i].y == segment.points[i + 1].y) {
                        orientation = "horizontal";
                    }
                    else orientation = "vertical";
                    var visible;
                    if (length >= 50) visible = "visible"; else visible = "hidden";
                    this._updateOrthoThumb(("OrthoThumb_" + i) + "_" + index, ((segment.points[i].x + segment.points[i + 1].x) / 2) * scale, ((segment.points[i].y + segment.points[i + 1].y) / 2) * scale, svg, visible, orientation);

                }
            else {
                var length = segment.length || ej.datavisualization.Diagram.Geometry.distance(segment._startPoint, segment._endPoint);
                if (segment._startPoint.y == segment._endPoint.y) {
                    orientation = "horizontal";
                }
                else orientation = "vertical";
                if (length >= 50) visible = "visible"; else visible = "hidden";
                this._updateOrthoThumb("OrthoThumb_" + index, ((segment._endPoint.x + segment._startPoint.x) / 2) * scale, ((segment._endPoint.y + segment._startPoint.y) / 2) * scale, svg, visible, orientation);
            }
        },
        _updateBezierLine: function (id, start, end, scale, svg) {
            var line = svg.line({
                "id": id, "x1": start.x * scale, "y1": start.y * scale, "x2": end.x * scale, "y2": end.y * scale
            });
        },
        _updateControlPointCorner: function (corner, point, index, svg) {
            svg.circle({ "id": corner + "_" + index, "cx": point.x, "cy": point.y });
        },

        updateUserHandles: function (handles, node, svg, isMultipleSelection, isDragging, scale) {
            if (handles && handles.length > 0) {
                for (var handleIndex = 0; handleIndex < handles.length; handleIndex++) {
                    if ((isMultipleSelection && handles[handleIndex].enableMultiSelection) || !isMultipleSelection)
                        this._updateHandle(handles[handleIndex], node, svg, isDragging, scale);
                }
            }
        },

        _updateHandle: function (handle, node, svg, isDragging, scale) {
            if (!isDragging && handle.visible) {
                var position = this._getHandlePosition(handle, node, scale);
                var d = ej.datavisualization.Diagram.Geometry.updatePath((position.x * scale) - (handle.size) / 4, (position.y * scale) - (handle.size) / 4, (handle.size / 2), (handle.size / 2), handle.pathData, svg);
                this._updateHandleShape(handle, position.x * scale, position.y * scale, svg);
                this._updateHanleIcon(handle, d, svg);
                if (svg.getElementById(handle.name + "_shape") && handle.visible) {
                    svg.getElementById(handle.name + "_shape").style.display = "block";
                    svg.getElementById(handle.name + "_icon").style.display = "block";
                }
            }
            else {
                if (svg.getElementById(handle.name + "_shape")) {
                    svg.getElementById(handle.name + "_shape").style.display = "none";
                    svg.getElementById(handle.name + "_icon").style.display = "none";
                }
            }
        },

        clearSelector: function (svg, parent) {
            var handle = svg.getElementById(svg.document.id + "handle_g");
            if (handle)
                parent.removeChild(handle);
            handle = svg.getElementById(svg.document.id + "userHandle_g");
            if (handle)
                parent.removeChild(handle);
        },

        _initializePageBreaks: function (svg, attr, g) {
            var g1 = svg.g(attr);
            g.appendChild(g1);
            return g1;
        },

        _renderVPageBreakLine: function (start, end, svg, g) {
            var line = svg.line({
                "x1": start.x, "y1": start.y,
                "x2": end.x, "y2": end.y,
                "style": "stroke:#aaaaaa;stroke-width:1;stroke-dasharray:10,10 "
            });
            g.appendChild(line);
        },

        _renderPageSettingsRect: function (start, end, svg, g, _stroke, _fill, _strokewidth) {
            var rect = svg.rect({
                "id": "pageback",
                "x": start.x, "y": start.y, "width": end.x, "height": end.y,
                "style": "fill:" + _fill + ";stroke:" + _stroke + ";stroke-width:" + _strokewidth + ""
            });
            g.appendChild(rect);
        },

        _updatePageBakground: function (start, end, svg, g, _stroke, _fill, _strokewidth) {
            var rect = document.getElementById("pageback");
            var attr = {
                "x": start.x, "y": start.y, "width": end.x, "height": end.y,
                "style": "fill:" + _fill + ";stroke:" + _stroke + ";stroke-width:" + _strokewidth + ""
            };
            if (rect)
                ej.datavisualization.Diagram.Util.attr(rect, attr);
        },

        _renderHPageBreakLine: function (start, end, svg, g) {
            var line = svg.line({
                "x1": start.x, "y1": start.y,
                "x2": end.x, "y2": end.y,
                "style": "stroke:#aaaaaa;stroke-width:1;stroke-dasharray:10,10 "
            });
            g.appendChild(line);
        },

        _removePageBreaks: function (svg, g, layer) {
            if (g != null) {
                if (layer != null) {
                    layer.removeChild(g);
                }
            }
        },
        removeChild: function (element, view) {
            var elementTar = view.svg.document.getElementById(element.name);
            if (elementTar)
                elementTar.parentNode.removeChild(elementTar)
        }
        //#endregion
    };
    //#endregion

    //#region Common Renderer
    ej.datavisualization.Diagram.DiagramContext = {
        updateViewPort: function (diagram) {
            var view, panel;
            diagram._views.forEach(function (viewid) {
                view = diagram._views[viewid];
                if (view.context == ej.datavisualization.Diagram.SvgContext && view.type == "mainview") {
                    ej.datavisualization.Diagram.PageUtil._updatePageSize(diagram);
                }
            });
        },
        renderNode: function (node, diagram, parent) {
            var view, panel;
            diagram._views.forEach(function (viewid) {
                view = diagram._views[viewid];
                panel = view.svg || view._canvas;
                view.context.renderNode(node, panel, parent || view.diagramLayer);
            });
        },

        renderPort: function (node, port, diagram) {
            var view, panel;
            diagram._views.forEach(function (viewid) {
                view = diagram._views[viewid];
                panel = view.svg || view._canvas;
                view.context._insertPort(node, port, panel);
            });
        },
        renderConnector: function (connector, diagram, parent) {
            var view, panel;
            diagram._views.forEach(function (viewid) {
                view = diagram._views[viewid];
                panel = view.svg || view._canvas;
                view.context.renderConnector(connector, panel, parent || view.diagramLayer, diagram);
            });
        },

        renderGroup: function (group, diagram, parent) {
            var view, panel;

            diagram._views.forEach(function (viewid) {
                var isOverView
                view = diagram._views[viewid];
                panel = view.svg || view._canvas;
                parent = view.svg.getElementById(group.parent);
                if (view.type == "overview") {
                    isOverView = true;
                }
                view.context.renderGroup(group, panel, parent || view.diagramLayer, diagram.nameTable, diagram, null, isOverView);
            });
        },
        update: function (data, diagram) {
            var view, panel;
            diagram._views.forEach(function (viewid) {
                view = diagram._views[viewid];
                panel = view.svg || view._canvas;
                if (data.isGroup)
                    view.context.updateGroup(data, panel, diagram);
                else if (data.segments)
                    view.context.updateConnector(data, panel, diagram);
                else
                    view.context.updateNode(data, panel, diagram);
            });
        },
        _refreshSegments: function (connector, diagram) {
            var view, panel;
            diagram._views.forEach(function (viewid) {
                view = diagram._views[viewid];
                panel = view.svg || view.canvas;
                if (connector.segments)
                    view.context._refreshSegments(connector, panel, diagram);
            });
        },
        renderShadow: function (node, diagram) {
            var view, panel;
            diagram._views.forEach(function (viewid) {
                view = diagram._views[viewid];
                panel = view.svg || view._canvas;
                view.context._renderShadow(node, panel);
            });
        },
        removeShadow: function (node, diagram) {
            var view, panel;

            diagram._views.forEach(function (viewid) {
                view = diagram._views[viewid];
                panel = view.svg || view._canvas;
                view.context._removeShadow(node, panel);
            });
        },
        addNodeLabel: function (shape, label, parent, diagram) {
            var view, panel;
            if (diagram._views) {
                diagram._views.forEach(function (viewid) {
                    view = diagram._views[viewid];
                    panel = view.svg || view.canvas;
                    view.context.addNodeLabel(shape, label, panel, parent);
                });
            }
        },

        updateLabel: function (node, label, diagram) {
            var view, panel;
            diagram._views.forEach(function (viewid) {
                view = diagram._views[viewid];
                panel = view.svg || view.canvas;
                view.context.updateLabel(node, label, panel);
            });
        },

        setNodeShape: function (node, diagram, parent) {
            var view, panel;
            diagram._views.forEach(function (viewid) {
                view = diagram._views[viewid];
                panel = view.svg || view.canvas;
                view.context.setNodeShape(node, panel, parent);
            });
        },

        setLine: function (connector, diagram, parent) {
            var view, panel;
            diagram._views.forEach(function (viewid) {
                view = diagram._views[viewid];
                panel = view.svg || view.canvas;
                view.context.setLine(connector, panel, parent, diagram);
            });
        },

        renderDecorators: function (connector, diagram) {
            var view, panel;
            diagram._views.forEach(function (viewid) {
                view = diagram._views[viewid];
                panel = view.svg || view.canvas;
                view.context.renderDecorators(connector, panel, diagram);
            });
        },

        clearDecorators: function (connector, diagram) {
            var view, panel;
            diagram._views.forEach(function (viewid) {
                view = diagram._views[viewid];
                panel = view.svg || view.canvas;
                view.context.clearDecorators(connector, panel, diagram);
            });
        },

        updateTargetDecoratorStyle: function (node, diagram) {
            var view, panel;
            diagram._views.forEach(function (viewid) {
                view = diagram._views[viewid];
                panel = view.svg || view.canvas;
                view.context.updateTargetDecoratorStyle(node, panel);
            });
        },

        updateSourceDecoratorStyle: function (node, diagram) {
            var view, panel;
            diagram._views.forEach(function (viewid) {
                view = diagram._views[viewid];
                panel = view.svg || view.canvas;
                view.context.updateSourceDecoratorStyle(node, panel);
            });
        },

        updateNodeStyle: function (node, diagram) {
            var view, panel;
            diagram._views.forEach(function (viewid) {
                view = diagram._views[viewid];
                panel = view.svg || view.canvas;
                view.context._updateNodeStyle(node, panel);
            });
        },

        updateLabelStyle: function (node, label, diagram) {
            diagram._views.forEach(function (viewid) {
                var view = diagram._views[viewid];
                var panel = view.svg || view.canvas;
                view.context.updateLabelStyle(node, label, panel);
            });
        },

        updateConnectorStyle: function (connector, diagram) {
            var view, panel;
            diagram._views.forEach(function (viewid) {
                view = diagram._views[viewid];
                panel = view.svg || view.canvas;
                view.context._updateConnectorStyle(connector, panel);
            });
        },

        updateTextBlock: function (node, label, diagram) {
            var view, panel;
            diagram._views.forEach(function (viewid) {
                view = diagram._views[viewid];
                panel = view.svg || view.canvas;
                view.context.updateTextBlock(node, label, panel);
            });

        }
    };
    //#endregion
})(jQuery, Syncfusion);;
/// <reference path="ej.diagramcommon.js" />
/// <reference path="ej.diagramsvg.js" />
/**
* @fileOverview Plugin to style the Html Button elements
* @copyright Copyright Syncfusion Inc. 2001 - 2013. All rights reserved.
*  Use of this code is subject to the terms of our license.
*  A copy of the current license can be obtained at any time by e-mailing
*  licensing@syncfusion.com. Any infringement will be prosecuted under
*  applicable laws. 
* @version 12.1 
* @author <a href="mailto:licensing@syncfusion.com">Syncfusion Inc</a>
*/
(function ($, ej, undefined) {
    /**
* @namespace ej
* @class ejSymbolPalette
* @requires jQuery.js
* @requires ej.common.all
* @requires ej.widgets.all
* @requires jquery.easing.js
* @requires jquery.globalize.js
* @requires jsrender.js
* @requires jquery.validate.js
* @requires jquery.validate.unobtrusive.js
* @classdesc defined values to render symbolpalette
* @example 
* &lt;div id="symbolpalette"&gt;&lt;/div&gt;<br>
* &lt;script&gt;
* //Create symbolpalette
$("#symbolpalette").ejSymbolPalette();
* &lt;/script&gt
*/
    "use strict";
    //#region SymbolPalette widget
    ej.widget("ejSymbolPalette", "ej.datavisualization.SymbolPalette", {
        // widget element will be automatically set in this
        element: null,

        // user defined model will be automatically set in this
        model: null,
        nameTable: {},

        //#region Initialization
        defaults: {
            /**The width of the palette 
          * @default 250
          * @type {Number}
          * @example
          * &lt;div id="symbolpalette"&gt;&lt;/div&gt;<br>
          * &lt;script&gt;
          * $("#symbolpalette").ejSymbolPalette({width:300});
          * &lt;/script&gt
          * @memberof ejSymbolPalette
          * @instance
          */
            width: 250,
            /**The height of the palette 
        * @default 400
        * @type {Number}
        * @example
        * &lt;div id="symbolpalette"&gt;&lt;/div&gt;<br>
        * &lt;script&gt;
        * $("#symbolpalette").ejSymbolPalette({height:500});
        * &lt;/script&gt
        * @memberof ejSymbolPalette
        * @instance
        */
            height: 400,
            /**Width of the palette item
        * @default 50
        * @type {Number}
        * @example
        * &lt;div id="symbolpalette"&gt;&lt;/div&gt;<br>
        * &lt;script&gt;
        * $("#symbolpalette").ejSymbolPalette({paletteItemWidth: 50});
        * &lt;/script&gt
        * @memberof ejSymbolPalette
        * @instance
        */
            paletteItemWidth: 50,
            /**Height of the items in palette
        * @default 50
        * @type {Number}
        * @example
        * &lt;div id="symbolpalette"&gt;&lt;/div&gt;<br>
        * &lt;script&gt;
        * $("#symbolpalette").ejSymbolPalette({paletteItemHeight: 50});
        * &lt;/script&gt
        * @memberof ejSymbolPalette
        * @instance
        */
            paletteItemHeight: 50,
            /**Preview width of the palette items
        * @default 100
        * @type {Number}
        * @example
        * &lt;div id="symbolpalette"&gt;&lt;/div&gt;<br>
        * &lt;script&gt;
        * $("#symbolpalette").ejSymbolPalette({previewWidth: 100});
        * &lt;/script&gt
        * @memberof ejSymbolPalette
        * @instance
        */
            previewWidth: 110,
            /**Preview height of the palette items
       * @default 100
       * @type {Number}
       * @example
       * &lt;div id="symbolpalette"&gt;&lt;/div&gt;<br>
       * &lt;script&gt;
       * $("#symbolpalette").ejSymbolPalette({previewHeight: 100});
       * &lt;/script&gt
       * @memberof ejSymbolPalette
       * @instance
       */
            previewHeight: 110,
            /**The preview offset for the palette items
     * @default (102, 102)
     * @type {Object}
     * @example
     * &lt;div id="symbolpalette"&gt;&lt;/div&gt;<br>
     * &lt;script&gt;
     * $("#symbolpalette").ejSymbolPalette({previewOffset: {x: 102, y: 102 }});
     * &lt;/script&gt
     * @memberof ejSymbolPalette
     * @instance
     */
            previewOffset: { x: 112, y: 112 },
            /**The diagramId of the palette 
      * @default null
      * @type {}
      * @example
      * &lt;div id="symbolpalette"&gt;&lt;/div&gt;<br>
      * &lt;script&gt;
      * $("#symbolpalette").ejSymbolPalette({diagramId: "diagram"});
      * &lt;/script&gt
      * @memberof ejSymbolPalette
      * @instance
      */
            diagramId: null,
            /**Height of the palette header
      * @default 30
      * @type {Number}
      * @example
      * &lt;div id="symbolpalette"&gt;&lt;/div&gt;<br>
      * &lt;script&gt;
      * $("#symbolpalette").ejSymbolPalette({headerHeight: 30});
      * &lt;/script&gt
      * @memberof ejSymbolPalette
      * @instance
      */
            headerHeight: 30,
            /** The selectedPaletteName of the symbol palette
      * @default 0
      * @type {Number}
      * @example
      * &lt;div id="symbolpalette"&gt;&lt;/div&gt;<br>
      * &lt;script&gt;
      * $("#symbolpalette").ejSymbolPalette({selectedPaletteName: "paletteName"});
      * &lt;/script&gt
      * @memberof ejSymbolPalette
      * @instance
      */
            selectedPaletteName: 0,
            /** Used to change the style of the node 
         * @default "e-symbolpalette"
         * @type {String}
         * @example
         * &lt;div id="symbolpalette"&gt;&lt;/div&gt;<br>
         * &lt;script&gt;
         * $("#symbolpalette").ejSymbolPalette({cssClass: "e-symbolpalette"});
         * &lt;/script&gt
         * @memberof ejSymbolPalette
         * @instance
         */
            cssClass: "e-datavisualization-symbolpalette",
            /** Enable or disable the palette item text 
       * @default true
       * @type {Boolean}
       * @example
       * &lt;div id="symbolpalette"&gt;&lt;/div&gt;<br>
       * &lt;script&gt;
       * $("#symbolpalette").ejSymbolPalette({showPaletteItemText: true});
       * &lt;/script&gt
       * @memberof ejSymbolPalette
       * @instance
       */
            showPaletteItemText: true,
            /** Enable or disable the Drag function of items in the palette 
       * @default true
       * @type {Boolean}
       * @example
       * &lt;div id="symbolpalette"&gt;&lt;/div&gt;<br>
       * &lt;script&gt;
       * $("#symbolpalette").ejSymbolPalette({allowDrag: true});
       * &lt;/script&gt
       * @memberof ejSymbolPalette
       * @instance
       */
            allowDrag: true,
            /** Collection of palette items 
        * @default null
        * @type {Array}
        * @example
        * &lt;div id="symbolpalette"&gt;&lt;/div&gt;<br>
        * &lt;script&gt;
        * $("#symbolpalette").ejSymbolPalette({palettes: palette});
        * &lt;/script&gt
        * @memberof ejSymbolPalette
        * @instance
        */
            palettes: []
        },

        dataTypes: {
            paletteItemWidth: "number",
            paletteItemHeight: "number",
            previewWidth: "number",
            previewHeight: "number",
            headerHeight: "number",
            showPaletteItemText: "boolean",
            allowDrag: "boolean",
            palettes: "array"
        },

        _canvas: null,

        _init: function () {
            this.selectedItem = null;
            this.activePalette = null;
            this._selectedElement = null;
            this._renderSymbolPalette();
            this._wireEvents();
            this.updateScrollerViewport();
        },

        _setModel: function (options) {
            for (var option in options) {
                switch (option) {
                    case "width":
                        this._setWidth(options[option]);
                        break;
                    case "height":
                        this._setHeight(options[option]);
                        break;
                    case "paletteItemWidth":
                        this.model.paletteItemWidth = options[option];
                        this._redrawContent();
                        break;
                    case "paletteItemHeight":
                        this.model.paletteItemHeight = options[option];
                        this._redrawContent();
                        break;
                    case "headerHeight":
                        this._setHeaderHeight(options[option]);
                        break;
                    case "cssClass":
                        this._setCssClass(options[option]);
                        break;
                    case "showPaletteItemText":
                        this._showItemText(options[option]);
                        break;
                }
            }
        },

        _destroy: function () {
            this.element.empty().removeClass(this.model.cssClass);
        },
        //#endregion

        //#region Rendering
        _renderSymbolPalette: function () {
            this._canvas = document.createElement("div");
            this._canvas.setAttribute("class", "e-scanvas");
            this._canvas.setAttribute("id", this.element[0].id + "_canvas");

            this.element[0].setAttribute("style", "height:" + this.model.height + "; width:" + this.model.width + ";");
            this.element.append(this._canvas);
            this.element.ejScroller({ width: this._setViewPortWidth(), height: this._setViewPortHeight() });
            var palettes = this.model.palettes;
            if (palettes) {
                for (var i = 0, len = palettes.length; i < len; i++) {
                    palettes[i] = ej.datavisualization.Diagram.Palette(palettes[i], this);
                    this._renderPalette(palettes[i]);
                }
            }
        },

        _setViewPortWidth: function () {
            this.element[0].setAttribute("style", "width:" + this.model.width + ";");
            var w = this.element.width();
            // var left = this.element[0].offsetLeft;
            return Math.abs(w);
        },

        _setViewPortHeight: function () {
            this.element[0].setAttribute("style", "height:" + this.model.height + ";");
            var h = this.element.height();
            //var top = this.element[0].offsetTop;
            return Math.abs(h);
        },
        _renderPalette: function (palette) {
            this._renderHeader(palette);
            this._renderContent(palette);
            this._collapse(palette);
        },

        _renderHeader: function (palette) {
            var header = document.createElement("div");
            var attr = { "id": palette.name + "_header", "class": "e-header", "style": "height:" + this.model.headerHeight + "px;" };
            ej.datavisualization.Diagram.Util.attr(header, attr);
            this._renderHeaderArrow(header, palette.expanded, palette.name);
            this._renderHeaderText(header, palette.name);
            this._addExpandedClass(header, palette.expanded);
            if (this.model.selectedPaletteName === palette.name) {
                $(header).addClass("selected");
                this.activePalette = palette;
            }
            this._canvas.appendChild(header);
        },

        _renderHeaderText: function (header, text) {
            var span = document.createElement("span");
            span.setAttribute("class", "e-header-text");
            span.innerHTML = text;
            header.appendChild(span);
        },

        _renderHeaderArrow: function (header, expanded, index) {
            var arrow = document.createElement("span");
            arrow.setAttribute("class", "e-header-arrow e-icon");
            arrow.setAttribute("style", "top:" + (this.model.headerHeight - 16) / 2 + "px;");
            this._addExpandedClass(arrow, expanded);
            if (this.model.selectedPaletteName === name)
                $(arrow).addClass("selected");
            header.appendChild(arrow);
        },

        _renderContent: function (palette) {
            var content = document.createElement("div");
            content.setAttribute("id", palette.name + "_content");
            content.setAttribute("class", "e-scontent");
            if ((this.model.palettes.length - 1) === this.model.palettes.indexOf(palette))
                content.setAttribute("style", "border-bottom-width: 0px;");
            this._canvas.appendChild(content);
            var paletteItems = palette.items;
            if (paletteItems && paletteItems.length > 0) {
                for (var i = 0, len = paletteItems.length; i < len; i++) {
                    if (paletteItems[i].parent)
                        this.nameTable[paletteItems[i].name] = paletteItems[i];
                    else {
                        this._renderItem(paletteItems[i], content);
                        this.nameTable[paletteItems[i].name] = paletteItems[i];
                    }
                }
            }
        },

        _renderItem: function (paletteItem, content) {
            var svg = this._renderItemContainer(paletteItem.name, content);
            if (paletteItem.segments)
                this._renderConnector(paletteItem, svg);
            else if (!paletteItem.isGroup)
                this._renderNode(paletteItem, svg);
            else
                this._renderGroup(paletteItem, svg);
        },

        _renderItemContainer: function (id, content, isDrag) {
            var container = document.createElement("div");
            var attr = { "id": id + "_paletteItem", "draggable": "true", "height": this.model.paletteItemHeight + "px", "width": this.model.paletteItemWidth + "px", "class": "e-paletteItem", "style": "-ms-touch-action: none;touch-action: none;" };
            ej.datavisualization.Diagram.Util.attr(container, attr);
            content.appendChild(container);
            var paletteId = this.element[0].id;
            var context = this;
            if (!isDrag) {
                $(container).ejDraggable({
                    clone: true,
                    cursorAt: { top: context.model.previewOffset.y, left: context.model.previewOffset.x },
                    helper: function (event) {
                        var diagram = $("#" + context.model.diagramId).ejDiagram("instance");
                        var paletteItemWidth, paletteItemHeight;
                        paletteItemWidth = context.model.paletteItemWidth;
                        paletteItemHeight = context.model.paletteItemHeight;
                        context.model.paletteItemWidth = context.model.previewWidth;
                        context.model.paletteItemHeight = context.model.previewHeight;
                        var svg = context._renderItemContainer(context.selectedItem.name + "_clone", document.body, true);
                        svg.document.setAttribute("drag", "true");
                        var item = $.extend(true, {}, context.selectedItem);
                        diagram._selectedSymbol = item;
                        diagram._paletteTable = context.nameTable;
                        diagram._palNameTable = $.extend(true, {}, context.nameTable);
                        if (item.segments) {
                            var bounds = ej.datavisualization.Diagram.Util.bounds(item);
                            var delWidth = (context.model.previewWidth - 10) / bounds.width;
                            var delHeight = (context.model.previewHeight - 10) / bounds.height;
                            diagram.scale(item, delWidth, delHeight, bounds.topLeft, diagram.nameTable);
                            context._renderConnector(item, svg);
                        }
                        else {
                            item.width = context.model.previewWidth - 10;
                            item.height = context.model.previewHeight - 10;
                            item.offsetX = item.width / 2 + 5;
                            item.offsetY = item.height / 2 + 5;
                            if (!item.isGroup)
                                context._renderNode(item, svg);
                            else {
                                context._renderGroup(item, svg);

                            }
                        }
                        var helper = document.getElementById(context.selectedItem.name + "_clone_paletteItem");
                        $(helper).addClass("dragClone");
                        context.model.paletteItemWidth = paletteItemWidth;
                        context.model.paletteItemHeight = paletteItemHeight;
                        return $(helper);
                    },
                    drag: function () {
                        var proxy = this;
                        var element = $(".dragClone")[0];
                        if (element) {
                            element.setAttribute("paletteId", paletteId);
                        }
                    },
                    dragStop: function (event, ui) {
                        var diagram = $("#" + context.model.diagramId).ejDiagram("instance");
                        diagram._selectedSymbol = null;
                        $(".dragClone").remove();
                    }
                });
            }
            return this._renderAnchor(container, id);
        },

        _renderAnchor: function (container, id) {
            var anchor = document.createElement("div");
            var attr = { "id": id + "_anchor", "height": this.model.paletteItemHeight + "px", "width": this.model.paletteItemWidth + "px", "class": "e-anchor" };
            ej.datavisualization.Diagram.Util.attr(anchor, attr);
            container.appendChild(anchor);
            var svg = this._renderSvg(anchor, id);
            this._renderText(anchor, id);
            return svg;
        },

        _renderSvg: function (anchor, id) {
            var wrapper = document.createElement("div");
            var attr = {
                "id": id + "_svgWrapper", "width": this.model.paletteItemWidth + "px;",
                "height": +this.model.paletteItemHeight + "px;", "class": "e-svg-container"
            };
            ej.datavisualization.Diagram.Util.attr(wrapper, attr);
            attr = {
                "id": id + "_svg", "width": this.model.paletteItemWidth, "height": this.model.paletteItemHeight,
                "version": "1.1", "xlink": "http://www.w3.org/1999/xlink"
            };
            var svg = new ej.datavisualization.Diagram.Svg(attr);
            wrapper.appendChild(svg.document);
            anchor.appendChild(wrapper);
            return svg;
        },

        _renderText: function (anchor, text) {
            var div = document.createElement("div");
            div.setAttribute("class", "e-text-container");
            div.setAttribute("style", "width:" + this.model.paletteItemWidth + "px;");
            var span = document.createElement("span");
            span.setAttribute("class", "e-paletteItemText");
            span.innerHTML = text;
            div.appendChild(span);
            if (!this.model.showPaletteItemText)
                div.style.display = "none";
            anchor.appendChild(div);
        },

        _renderNode: function (node, svg) {
            var group = ej.datavisualization.Diagram.SvgContext.renderNode(node, svg);
            var isDrag = false;
            if (svg.document.getAttribute("drag") != null) {
                isDrag = true;
            }
            this._transformItem(group, node.width, node.height, node, isDrag);
        },

        _updateChildBounds: function (node, nameTable) {
            var child = null;
            if (node.isGroup) {
                for (var i = 0; i < node.children.length; i++) {
                    child = nameTable[node.children[i]];
                    if (child.isGroup)
                        this._updateChildBounds(child, nameTable);
                }
                ej.datavisualization.Diagram.Util._updateChildBounds(node, nameTable);
            }
        },
        _renderGroup: function (node, svg) {
            this._updateChildBounds(node, this.nameTable);
            var group = ej.datavisualization.Diagram.SvgContext.renderGroup(node, svg, null, this.nameTable);
            var isDrag = false;
            if (svg.document.getAttribute("drag") != null) {
                isDrag = true;
            }
            this._transformItem(group, node.width, node.height, node, isDrag);

        },
        _renderConnector: function (connector, svg) {
            var group = ej.datavisualization.Diagram.SvgContext.renderConnector(connector, svg);
            var bounds = ej.datavisualization.Diagram.Util.bounds(connector);
            var isDrag = false;
            if (svg.document.getAttribute("drag") != null) {
                isDrag = true;
            }
            this._transformItem(group, bounds.width, bounds.height, null, isDrag);
        },

        //#endregion

        //#region Events
        _wireEvents: function () {
            this._on(this.element, ej.eventType.click, ".e-header", this._headerClick);
            this._on(this.element, ej.eventType.mouseDown, ".e-header", this._headerMouseDown);
            this._on(this.element, "mouseover", ".e-header", this._headerMouseOver);
            this._on(this.element, "mouseout", ".e-header", this._headerMouseOut);
            this._on(this.element, ej.eventType.mouseDown, ".e-paletteItem", this._itemMouseDown);
            this._on(this.element, "mouseover", ".e-paletteItem", this._itemMouseOver);
            this._on(this.element, "mouseout", ".e-paletteItem", this._itemMouseOut);
            this._on(this.element, ej.eventType.click, ".e-anchor", this._itemClick);
        },

        _headerClick: function (evt) {
            var proxy = this;
            $(evt.currentTarget).next(".e-scontent").slideToggle(200, function () {
                proxy._refresh();
            });
            this._updateHeaderState(evt.currentTarget);
            var palette = this._getActivePalette(evt.currentTarget.id);
            if (palette && palette !== this.activePalette)
                this.activePalette = palette;


        },

        _headerMouseDown: function (evt) {
            evt.preventDefault();
            evt.stopPropagation();
        },

        _headerMouseOver: function (evt) {
            $(evt.currentTarget).addClass("hover");
            $(evt.currentTarget).find(".e-header-arrow").addClass("hover");
        },

        _headerMouseOut: function (evt) {
            $(evt.currentTarget).removeClass("hover");
            $(evt.currentTarget).find(".e-header-arrow").removeClass("hover");
        },

        _itemMouseDown: function (evt) {
            if (this.model.diagramId) {
                var diagram = $("#" + this.model.diagramId).ejDiagram("instance");
                if (diagram.activeTool instanceof ej.datavisualization.Diagram.LineTool) {
                    diagram.activeTool._showAllPorts(true);
                    diagram.deactivateTool();
                }
            }
            if (!this.model.allowDrag) {
                evt.preventDefault();
                evt.stopPropagation();
            }
            if (this._selectedElement) {
                if (this._selectedElement !== evt.currentTarget) {
                    $(this._selectedElement).removeClass("selected");
                    this._selectedElement = evt.currentTarget;
                }
            }
            else
                this._selectedElement = evt.currentTarget;
            if (!($(evt.currentTarget).hasClass("selected")))
                $(evt.currentTarget).addClass("selected");

            var palette = this._getActivePalette(evt.currentTarget.parentElement.id);
            if (palette && palette !== this.activePalette) {
                this._clearHeaderSelection();
                this._updateHeaderSelection($(evt.currentTarget.parentElement).prev());
                this.activePalette = palette;
            }

            if (this.activePalette && this._selectedElement) {
                var name = this._selectedElement.id.slice(0, this._selectedElement.id.lastIndexOf('_'));
                var items = this.activePalette.items;
                for (var i = 0, len = items.length; i < len; i++) {
                    var item = items[i];
                    if (item.name === name)
                        this.selectedItem = item;
                }
            }
        },

        _itemMouseOver: function (evt) {
            $(evt.currentTarget).addClass("hover");
        },

        _itemMouseOut: function (evt) {
            $(evt.currentTarget).removeClass("hover");
        },

        _itemDraStart: function (evt) {
            var dataTransfer = evt.originalEvent.dataTransfer;
            dataTransfer.setData("Text", this.element[0].id);
            dataTransfer.effectAllowed = "copy";
        },

        _itemClick: function (evt) {
            if (this.model.diagramId) {
                if (evt.type === "click" || (evt.type === "touchend" && !$(".dragClone")[0])) {
                    var diagram = $("#" + this.model.diagramId).ejDiagram("instance");
                    var item = $.extend(true, {}, this.selectedItem);
                    item.name += ej.datavisualization.Diagram.Util.randomId();
                    if (item.isGroup) {
                        for (var j = 0; j < item.children.length; j++) {
                            var newObj = $.extend(true, {}, this.nameTable[item.children[j]]);
                            newObj.parent = item.name;
                            newObj.name = item.name + newObj.name;
                            item.children[j] = newObj.name;
                            diagram.nameTable[newObj.name] = newObj;
                            diagram.model.nodes.push(newObj);
                            ej.datavisualization.Diagram.SpatialUtil._updateQuad(diagram, diagram._spatialSearch, newObj);
                        }
                        item.height = item.width = 0;
                        diagram._updateChildBounds(item, diagram.nameTable);
                    }
                    if (item && !item.isLane)
                        diagram.add(item);
                }
            }
        },

        _collectionChanged: function (args) {
            if (args.changeType == "insert") {
                this._renderPalette(args.element);
            }
            else if (args.changeType == "remove") {
                var palette = args.element;
                if (palette != null) {
                    var paletteHeader = document.getElementById(palette.name + "_header");
                    if (paletteHeader)
                        paletteHeader.parentNode.removeChild(paletteHeader);
                    var paletteContent = document.getElementById(palette.name + "_content");
                    if (paletteContent)
                        paletteContent.parentNode.removeChild(paletteContent);
                }
            }
        },
        //#endregion

        //#region Public methods

        addPaletteItem: function (paletteName, item) {
            var content, palette;
            for (var i = 0; i < this.model.palettes.length; i++) {
                palette = this.model.palettes[i];
                if (palette.name === paletteName) {
                    palette.items.push(item);
                    content = document.getElementById(paletteName + "_content");
                    this._renderItem(item, content);
                }
            }
        },

        removePaletteItem: function (paletteName, item) {
            var content, palette;
            for (var i = 0; i < this.model.palettes.length; i++) {
                palette = this.model.palettes[i];
                if (palette.name === paletteName) {
                    ej.datavisualization.Diagram.Util.removeItem(palette.items, item);
                    content = document.getElementById(paletteName + "_content");
                    content.removeChild(content.getElementsByClassName("e-paletteItem e-draggable e-js").item(item.name + "_paletteItem"));
                    this.selectedItem = null;
                }
            }
        },

        addPalette: function (palette) {
            this.model.palettes.push(palette);
            this._renderPalette(palette);
        },

        removePalette: function (paletteName) {
            var paletteHeader, paletteContent, palette;
            for (var i = 0; i < this.model.palettes.length; i++) {
                palette = this.model.palettes[i];
                if (palette.name === paletteName) {
                    ej.datavisualization.Diagram.Util.removeItem(this.model.palettes, palette);
                    paletteHeader = document.getElementById(palette.name + "_header");
                    if (paletteHeader)
                        paletteHeader.parentNode.removeChild(paletteHeader);
                    paletteContent = document.getElementById(palette.name + "_content");
                    if (paletteContent)
                        paletteContent.parentNode.removeChild(paletteContent);
                }
            }
        },
        setWidth: function (value) {
            this.model.width = value;
            this._setWidth(value);
        },

        setHeight: function (value) {
            this.model.height = value;
            this._setHeight(value);
        },

        setPaletteItemWidth: function (value) {
            this.model.paletteItemWidth = value;
            this._redrawContent();
        },

        setPaletteItemHeight: function (value) {
            this.model.paletteItemHeight = value;
            this._redrawContent();
        },

        setHeaderHeight: function (value) {
            this.model.headerHeight = value;
            this._setHeaderHeight(value);
        },

        setCssClass: function (value) {
            this._setCssClass(value);
            this.model.cssClass = value;
        },

        showPaletteItemText: function (value) {
            this._showItemText(value);
            this.model.showPaletteItemText = value;
        },

        searchPalette: function (palette, key) {
            var paletteCollection = new ej.datavisualization.Diagram.Collection();
            var paletteItems = null;
            if (key != "") {
                palette = this._getSearchPaletteItems(key);
                paletteCollection.add(palette);
            }
            return paletteCollection;
        },
        //#endregion

        //#region Helper methods
        _getSearchPaletteItems: function (key) {
            var palette = new ej.datavisualization.Diagram.Palette();
            var palettes = this.model.palettes, i, j;
            for (i = 0; i < palettes.length; i++) {
                if ((((palettes[i].name).toLocaleLowerCase()).search(key.toLocaleLowerCase())) >= 0) {
                    for (j = 0; j < palettes[i].items.length; j++) {
                        this.element.find("#" + palettes[i].items[j].name + "_paletteItem")[0].style.display = "block";
                    }
                }
                else {
                    for (j = 0; j < palettes[i].items.length; j++) {
                        if ((((palettes[i].items[j].name).toLocaleLowerCase()).search(key.toLocaleLowerCase())) >= 0) {
                            this.element.find("#" + palettes[i].items[j].name + "_paletteItem")[0].style.display = "block";
                        }
                        else
                            this.element.find("#" + palettes[i].items[j].name + "_paletteItem")[0].style.display = "none";
                    }
                }
            }
            return palette;
        },

        _addExpandedClass: function (element, expanded) {
            if (expanded)
                $(element).addClass("expanded");
            else
                $(element).addClass("collapsed");
        },

        _collapse: function (palette) {
            if (!palette.expanded)
                $(document.getElementById(palette.name + "_header")).next(".e-scontent").slideToggle(10);

        },

        _transformItem: function (group, width, height, node, isDrag) {
            if (!isDrag) {
                var scale = this._getScaleFactor(width, height);
                var translate = this._getTranslateFactor(width, height);
                var transform = "";
                //translate
                transform += "translate(" + translate + "," + translate + ")";
                //scale
                transform += "scale(" + scale + "," + scale + ")";
                group.setAttribute("transform", transform);
            }
            if (node && ej.browserInfo().name === "msie") {
                if (!node.isGroup && node.shape.type === "html") {
                    var htmlDiv = document.getElementById(node.name + "_html");
                    htmlDiv.setAttribute("style", "zoom:" + scale + ";");
                }
            }
        },

        _getScaleFactor: function (width, height) {
            var fx = (30 * this.model.paletteItemWidth) / 100;
            var fy = (30 * this.model.paletteItemHeight) / 100;
            var sx = (this.model.paletteItemWidth - fx) / width;
            var sy = (this.model.paletteItemHeight - fy) / height;
            sx = sx > 1 ? 1 : sx;
            sy = sy > 1 ? 1 : sy;
            return Math.min(sx, sy);
        },

        _getTranslateFactor: function (width, height) {
            var x = (15 / 100) * this.model.paletteItemWidth;
            var y = (15 / 100) * this.model.paletteItemHeight;
            return Math.min(x, y);
        },

        _setWidth: function (val) {
            this.element.css("width", val);
        },

        _setHeight: function (val) {
            this.element.css("height", val);
        },

        _setHeaderHeight: function (val) {
            this.element.find(".e-header").css("height", val);
            this.element.find(".e-header-arrow").css("top", (val - 16) / 2);
        },

        _setCssClass: function (val) {
            this.element.removeClass(this.model.cssClass).addClass(val);
        },

        _showItemText: function (val) {
            if (val !== this.model.showPaletteItemText)
                if (val) {
                    this.element.find(".e-text-container").show();
                }
                else
                    this.element.find(".e-text-container").hide();
        },

        _redrawContent: function () {
            var contents = this.element.find(".e-scontent");
            for (var i = 0, len = contents.length; i < len; ++i) {
                var content = contents[i];
                $(content).empty();
                var palette = this.model.palettes[i];
                var paletteItems = palette.items;
                if (paletteItems && paletteItems.length > 0) {
                    for (var count = 0, l = paletteItems.length; count < l; count++) {
                        this._renderItem(paletteItems[count], content);
                    }
                }
            }
        },

        _getActivePalette: function (elementId) {
            var name = elementId.slice(0, elementId.lastIndexOf('_'));
            var palettes = this.model.palettes;
            for (var i = 0, len = palettes.length; i < len; i++) {
                var palette = palettes[i];
                if (palette.name === name && palette !== this.activePalette)
                    return palette;
            }
            return null;
        },

        _updateHeaderState: function (target) {
            this._clearHeaderSelection();
            this._updateExpandState(target);
            this._updateHeaderSelection(target);
        },

        _updateExpandState: function (target) {
            if (($(target).hasClass("expanded"))) {
                $(target).removeClass("expanded").addClass("collapsed");
                $(target).find(".e-header-arrow").removeClass("expanded").addClass("collapsed");
            }
            else {
                $(target).removeClass("collapsed").addClass("expanded");
                $(target).find(".e-header-arrow").removeClass("collapsed").addClass("expanded");
            }
        },

        _updateHeaderSelection: function (target) {
            if (!($(target).hasClass("selected"))) {
                $(target).addClass("selected");
                $(target).find(".e-header-arrow").addClass("selected");
            }
        },

        _clearHeaderSelection: function () {
            this.element.find(".e-header").removeClass("selected");
            this.element.find(".e-header-arrow").removeClass("selected");
        },

        _addItem: function (palette, item) {
            var content = document.getElementById(palette.name + "_content");
            this._renderItem(item, content);
        },

        _removeItem: function (palette, item) {
            var content = document.getElementById(palette.name + "_content");
            content.removeChild(document.getElementById(item.name + "_paletteItem"));
            if (item === this._selectedElement)
                this._selectedElement = null;
        },

        updateScrollerViewport: function () {
            var scroller = this.element.ejScroller("instance");
            scroller.model.touchScroll = false;
            scroller.model.width = this._setViewPortWidth();
            scroller.model.height = this._setViewPortHeight();
            this._refresh();
        },

        _refresh: function () {
            this.element.ejScroller("refresh");
        },

        //#endregion
    });
    //#endregion

})(jQuery, Syncfusion);;


(function ($, ej) {
    "use strict";
    //#region Extends
    ej.datavisualization.Diagram.extend = function (d, b) {
        for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
        function __() { this.constructor = d; }
        __.prototype = b.prototype;
        d.prototype = new __();
    }
    //#endregion

    //#region Tool
    var ToolBase = (function () {
        function ToolBase(name, diagram) {
            this.name = name;
            this.diagram = diagram;
            this.helper = null;
            this.inAction = false;
            this.selectedObject = null;
            this.startPoint = ej.datavisualization.Diagram.Point(0, 0);
            this.currentPoint = ej.datavisualization.Diagram.Point(0, 0);
            this.previousPoint = ej.datavisualization.Diagram.Point(0, 0);
            this.cursor = "default";
            this._currentPossibleConnection = null;
            this._previousPossibleConnection = null;
            this._possibleConnectionPort = null;
            this.singleAction = false;
            this.svgHelper = null;
            this.single = null;
            this._centralPoint = ej.datavisualization.Diagram.Point(0, 0);
            this._enableAutoNode = null;
            this._prevTool = null;
        }
        ToolBase.prototype.abort = function (evt) {
            this._endAction();
            ej.datavisualization.Diagram.SvgContext.updateSelector(this.diagram.selectionList[0], this.diagram._svg, this.diagram._currZoom, this.diagram, this.diagram.model.selectorConstraints);
            this.diagram._removeTooltip();
        };
        ToolBase.prototype.keydown = function (evt) {
            var keycode = evt.keyCode ? evt.keyCode : evt.which;
            if (keycode === 27 && this.inAction) {
                this.abort();
            }
            else if (!this.inAction && keycode == 27) {
                this.diagram._clearSelection();
            }
        };

        ToolBase.prototype._containChil = function (node, list) {
            for (var i = 0; i < list.length; i++) {
                if (list[i] == node.name)
                    return true;
            }
        };
        ToolBase.prototype._decideSelectedItem = function (evt, node, skip) {
            var obj = null;
            if (node) {
                if (node.parent) {
                    if (this.diagram.selectionList && this.diagram.selectionList[0] && node.parent == this.diagram.selectionList[0].name && !skip)
                        obj = node;
                    else {
                        if (skip && this.diagram.selectionList[0] && this.diagram.selectionList[0].name == node.name) {
                            obj = node;
                        }
                        else {
                            var item = this.diagram.nameTable[node.parent];
                            obj = this._decideSelectedItem(evt, item, skip);
                            if (this.diagram.selectionList && this.diagram.selectionList[0]) {
                                if (obj && this.diagram.selectionList[0].name == obj.name && !skip)
                                    obj = node;
                            }
                        }
                    }
                } else {
                    obj = node;
                }
            }
            else {
                return null;
            }
            return obj;
        };


        ToolBase.prototype._removeHighLighter = function () {
            ej.datavisualization.Diagram.SvgContext._removeNodeHighlighter(this.diagram._svg, this.diagram._adornerLayer);
        };
        ToolBase.prototype._nodeHighLighter = function (overNode) {
            var source = null, childBounds = null, target = null;
            if (overNode && this.selectedObject.parent) //&& this.selectedObject.parent != overNode.name)
            {
                var parent = this.diagram.nameTable[this.selectedObject.parent];
                if (parent.container && parent.container.type == "stack" && this.selectedObject.name != overNode.name) {
                    ej.datavisualization.Diagram.SvgContext._drawNodeHighlighter(overNode, this.diagram._svg, this.diagram._adornerLayer, this.diagram._currZoom);
                }
                else if (parent.container && parent.container.type == "canvas" && this.selectedObject.parent != overNode.name) {
                    ej.datavisualization.Diagram.SvgContext._drawNodeHighlighter(overNode, this.diagram._svg, this.diagram._adornerLayer, this.diagram._currZoom);
                }
                else this._removeHighLighter();
            }
            else if (overNode.container) {
                ej.datavisualization.Diagram.SvgContext._drawNodeHighlighter(overNode, this.diagram._svg, this.diagram._adornerLayer, this.diagram._currZoom);
            }

            else
                this._removeHighLighter();
            if (this._nodeToHit && overNode && overNode.name != this._nodeToHit.name) {
                childBounds = ej.datavisualization.Diagram.Util.bounds(this._nodeToHit);
                target = this._getTargetNode(this._nodeToHit, childBounds);
                this._raiseEvent("mouseLeave", { element: this.selectedObject, source: this._nodeToHit, target: target });
            }
            if (overNode) {
                if (this._nodeToHit && this._nodeToHit.name !== overNode.name || !this._nodeToHit) {
                    this._nodeToHit = overNode;
                    if (this.selectedObject.parent)
                        source = this.diagram.nameTable[this.selectedObject.parent];
                    if (this.selectedObject.isPseudoGroup) {
                        var fChild = this.diagram.nameTable[this.selectedObject.children[0]];
                        if (fChild && fChild.parent)
                            source = this.diagram.nameTable[fChild.parent];
                    }
                    childBounds = ej.datavisualization.Diagram.Util.bounds(overNode);
                    target = this._getTargetNode(overNode, childBounds);
                    this._raiseEvent("mouseEnter", { element: this.selectedObject, source: source, target: target });
                }
            }
            else {
                this._nodeToHit = null;
            }
            if (overNode) {
                if (this.selectedObject.parent)
                    source = this.diagram.nameTable[this.selectedObject.parent];
                childBounds = ej.datavisualization.Diagram.Util.bounds(overNode);
                target = this._getTargetNode(overNode, childBounds);
                this._raiseEvent("mouseOver", { element: this.selectedObject, source: source, target: target });
            }
        };

        ToolBase.prototype._doubleClick = function (element) {
            if (element && element.parent) {
                var parent = this.diagram.nameTable[element.parent]
                if (parent && parent.container) {
                    var name = element.name;
                    this.diagram.updateSelectedObject(name);
                    this.diagram.addSelection(element);
                }
            }
        };

        ToolBase.prototype._itemClick = function (element, actualObject, selectedObject) {
            if (actualObject && actualObject.parent) {
                var parent = this.diagram.nameTable[actualObject.parent]
                if (parent && parent.container) {
                    if (actualObject && !this.diagram._selectedSymbol) {
                        var name = actualObject.name;
                        if (name.indexOf("_header_swimlane") != -1) {
                            this.diagram.updateSelectedObject(name.split("_header_swimlane")[0]);
                        }
                        else if (name.indexOf("_Headerr_") != -1)
                            this.diagram.updateSelectedObject(actualObject.parent);
                        else
                            this.diagram.updateSelectedObject(name);
                    }
                }
            }
            else if (actualObject) {
                this.diagram.updateSelectedObject(actualObject.name);
            }

        };
        ToolBase.prototype._getTargetNode = function (node, childBounds) {
            var bounds = null, nodes = [];
            var quads = ej.datavisualization.Diagram.SpatialUtil.findQuads(this.diagram._spatialSearch, childBounds);
            for (var i = 0; i < quads.length; i++) {
                var quad = quads[i];
                if (quad.objects.length > 0) {
                    for (var j = 0; j < quad.objects.length; j++) {
                        var nd = quad.objects[j];
                        if (!nd.segments) {
                            bounds = ej.datavisualization.Diagram.Util.bounds(nd);
                            if (nodes.indexOf(nd) == -1 && ej.datavisualization.Diagram.Geometry.intersectsRect(childBounds, bounds))
                                nodes.push(nd);

                        }
                    }
                }
            }
            var collection = [];
            collection.push(node);
            for (i = 0; i < nodes.length; i++) {
                if (nodes[i].name != node.name)
                    collection.push(nodes[i]);
            }
            return collection;
        };
        ToolBase.prototype._updateMargin = function (node, group) {
            if (node.isPseudoGroup) {
                var child = null;
                for (var i = 0; i < node.children.length; i++) {
                    child = this.diagram.nameTable[node.children[i]];
                    this._updateMargin(child, group);
                }
            }
            else if (group && group.container && group.container.type == "canvas") {
                var groupBounds = ej.datavisualization.Diagram.Util.bounds(group); //ej.datavisualization.Diagram.SvgContext.getCanvasBoundingBox(group, this.diagram, node, true);
                var bounds1 = ej.datavisualization.Diagram.Util.bounds(node);
                var bounds = ej.datavisualization.Diagram.Geometry.rect([bounds1.topLeft, bounds1.topRight, bounds1.bottomRight, bounds1.bottomLeft]);
                node.marginLeft = bounds.x - (groupBounds.x + group.paddingLeft);
                node.marginRight = (groupBounds.x + group.width) - (bounds.x + bounds.width) - group.paddingRight;
                node.marginTop = bounds.y - (groupBounds.y + group.paddingTop);
                node.marginBottom = groupBounds.bottom - (bounds.y + bounds.height) - group.paddingBottom;
            }
        };
        ToolBase.prototype._getNodeUnderMouse = function (evt) {
            var type;
            var node = null;
            var obj = null;
            var parent = $(evt.target).parents(".ej-d-node,.ej-d-connector,.ej-d-group");
            if (parent) {
                type = parent.attr("class");
                if (type === "ej-d-node" || type === "ej-d-group") {
                    node = this.diagram._findNode(parent.attr("id"));
                } else if (type === "ej-d-connector") {
                    node = this.diagram._findConnector(parent.attr("id"));
                }
            }
            if (node) {
                if (node.isGroup) {
                    obj = node;
                }
                else {
                    if (node.parent) {
                        var parent1 = this.diagram.nameTable[node.parent];
                        if (parent1)
                            obj = parent1;
                    }
                }
            }
            return obj;
        };

        ToolBase.prototype._isChange = function (obj1, obj2) {
            var status = null;
            if (obj1.isGroup || obj1.isPseudoGroup || obj2.isGroup || obj2.isPseudoGroup) {
                if ((obj1.isGroup || obj1.isPseudoGroup) && !(obj2.isGroup || obj2.isPseudoGroup)) {
                    status = !this._containsChild(obj1, obj2.name);
                }
                if (!(obj1.isGroup || obj1.isPseudoGroup) && (obj2.isGroup || obj2.isPseudoGroup)) {
                    status = !this._containsChild(obj2, obj1.name);
                }
                else if ((obj1.isGroup || obj1.isPseudoGroup) && (obj2.isGroup || obj2.isPseudoGroup)) {
                    if (obj1.name == obj2.name)
                        status = true;
                }
            }
            else {
                if (obj1.name == obj2.name)
                    status = true;
            }
            return status;
        };
        ToolBase.prototype._containsChild = function (node, childName) {
            var status = true;
            var child = null;
            if (node.children) {
                for (var i = 0; i < node.children.length; i++) {
                    child = this.diagram.nameTable[node.children[i]];
                    if (child.isGroup)
                        status = this._containsChild(child, childName);
                    else {
                        if (child.name == childName)
                            status = false;
                    }
                }
            }
            return status;
        };

        ToolBase.prototype._findNodeUnderMouse = function (evt, skip) {
            var obj = null;
            var node = null;
            var type;
            var parent = $(evt.target).parents(".ej-d-node,.ej-d-connector,.ej-d-group");
            if (parent) {
                type = parent.attr("class");
                if (type === "ej-d-node" || type === "ej-d-group") {
                    node = this.diagram._findNode(parent.attr("id"));
                } else if (type === "ej-d-connector") {
                    node = this.diagram._findConnector(parent.attr("id"));
                }
            }
            if (node && !node.parent)
                obj = node;
            else
                obj = this._decideSelectedItem(evt, node, skip);
            this.prevSelectObject = obj;

            if (skip && this.diagram.selectionList[0]) {
                if (obj && obj.isGroup && !this._containsChild(obj, this.diagram.selectionList[0].name))
                    obj = obj;
                else if (obj && this.diagram.selectionList[0].name != obj.name && this._isChange(this.diagram.selectionList[0], obj))
                    obj = this.diagram.selectionList[0];
                else if (node && node.name == this.diagram.selectionList[0].name && !this._isChange(this.diagram.selectionList[0], obj)) {
                    obj = this.diagram.selectionList[0];

                }
            }
            this.actualObject = node;
            return obj;

        };

        ToolBase.prototype._findLabelUnderMouse = function (evt, skip) {
            var obj;
            var rect = $(evt.target);
            var rectid = rect.context.id.split('_');
            if (rectid[rectid.length - 1] == "lblbg") {
                var parent = $(evt.target).parents(".ej-d-node,.ej-d-connector,.ej-d-group");
                if (parent) {
                    var id = parent[0].getAttribute("id");
                    var type = parent[0].getAttribute("class");
                    if (type) {
                        if (type === "ej-d-node" || type === "ej-d-group") {
                            obj = this.diagram._findNode(id);
                            var curlabel = this.diagram._findLabelAtPoint(this.currentPoint, obj);
                            this.activeLabel = curlabel;
                        }
                        else if (type === "ej-d-connector") {
                            obj = this.diagram._findConnector(id);
                            var curlabel = this.diagram._findLabelAtPoint(this.currentPoint, obj);
                            this.activeLabel = curlabel;
                        }
                    }
                }
            }
            return obj;
        };
        ToolBase.prototype._selectionContainsChild = function (name) {
            if (this.diagram.selectionList[0].children) {
                var children = this.diagram.selectionList[0].children;
                var child = null;
                for (var i = 0; i < children.length; i++) {
                    child = this.diagram.nameTable[children[i]];
                    if (child.name == name)
                        return true;
                }
            }
            else if (this.diagram.selectionList[0].isGroup && !this.diagram.selectionList[0].isPseudoGroup) {
                if (this.diagram.selectionList[0].name == name)
                    return true;
            }
            else if (this.diagram.nameTable[name].isGroup) {
                children = this.diagram.nameTable[name].children;
                for (var j = 0; j < children.length; j++) {
                    child = this.diagram.nameTable[children[j]];
                    var status = this._selectionContainsChild(child.name);
                    if (status)
                        return status;
                }
            }
            else if (this.diagram.selectionList[0].name == name)
                return true;

            return false;
        };

        ToolBase.prototype._isSelected = function (name) {
            var child = null;
            var node = this.diagram.nameTable[name];
            if (this._selectionContainsChild(node.name))
                return true;
            else if (node.isGroup) {
                for (var i = 0; i < node.children.length; i++) {
                    child = node.children[i];
                    if (this._selectionContainsChild(child))
                        return true;
                }
            }
            return false;
        };
        ToolBase.prototype._isColleagueSelected = function (name) {
            var node = this.diagram.nameTable[name];
            var child = null;
            if (node.parent) {
                var parent = this.diagram.nameTable[node.parent];
                if (parent) {
                    for (var i = 0; i < parent.children.length; i++) {
                        child = parent.children[i];
                        if (this._isSelected(child))
                            return true;
                    }
                }
            }
            return false;
        };
        ToolBase.prototype._isInSelection = function (name) {
            if (this.diagram.selectionList[0].isPseudoGroup && this.diagram.selectionList[0].children) {
                var children = this.diagram.selectionList[0].children;
                var child = null;
                for (var i = 0; i < children.length; i++) {
                    child = this.diagram.nameTable[children[i]];
                    if (child.name == name)
                        return true;
                }
            }
        };
        ToolBase.prototype._getCloneNode = function (node) {
            var obj = null;
            obj = jQuery.extend(true, {}, node);
            obj.children = [];
            obj.minHeight = 0;
            obj.minWidth = 0;
            obj.maxHeight = 0;
            obj.maxWidth = 0;
            return obj;
        };
        ToolBase.prototype._getProcessedObject = function (name, obj) {
            var node = this.diagram.nameTable[name];
            if (node) {
                if (this._isInSelection(name)) {
                    return null;
                }
                else if (this._selectionContainsChild(node.name))
                    return { add: node, remove: null };
                else {
                    if (node.parent) {
                        if (this._selectionContainsChild(node.parent)) {
                            if (this._isInSelection(node.parent))
                                return { add: node, remove: this.diagram.nameTable[node.parent] };
                            else {
                                var parent = this.diagram.nameTable[node.parent];
                                if (parent)
                                    return { add: parent, remove: this.diagram.nameTable[parent.parent] };
                            }
                        } else {
                            if (this._isColleagueSelected(node.parent) && !this._selectionContainsChild(node.parent)) {
                                if (this._isSelected(node.parent)) {
                                    return { add: node, remove: null };
                                }
                                else
                                    return { add: this.diagram.nameTable[node.parent], remove: null };
                            }
                            else if (this.diagram.selectionList[0] && this._isInSelection(node.parent)) {
                                return { add: this.diagram.nameTable[node.name], remove: this.diagram.nameTable[node.parent] };

                            }
                            else
                                return this._getProcessedObject(node.parent);
                        }
                    }
                    else {
                        return { add: node, remove: null };
                    }
                }
            }
        };

        ToolBase.prototype._measureStackBounds = function (group) {
            var child = null;
            var bounds;
            var rect = null;
            for (var i = 0; i < group.children.length; i++) {
                child = this.diagram.nameTable[group.children[i]];
                bounds = ej.datavisualization.Diagram.Util.bounds(child);

                if (bounds) {
                    if (!rect)
                        rect = bounds;
                    else
                        rect = this.diagram._union(bounds, rect);
                }
            }
            rect.width += group.paddingLeft + group.paddingRight;
            rect.height += group.paddingTop + group.paddingBottom;
            return rect;
        };
        ToolBase.prototype.hasSameParent = function () {
            if (this.selectedObject.isPseudoGroup) {
                var list = this.selectedObject;
                var child = null;
                var parent = this.diagram.nameTable[list.children[0]].parent;
                for (var i = 0; i < list.children.length; i++) {
                    child = this.diagram.nameTable[list.children[i]];
                    if (child.parent != parent || child.parent == "")
                        return false;
                }
                return true;
            }
            return false;
        },
            ToolBase.prototype._fromDiagram = function (list) {
                var state = true;
                var child = null;
                for (var i = 0; i < list.length; i++) {
                    child = this.diagram.nameTable[list[i]];
                    if (child && child.parent) {
                        state = false;
                        break;
                    }
                }
                return state;
            },
            ToolBase.prototype._fromContainer = function (list) {
                var child = null, parent = null;
                for (var i = 0; i < list.length; i++) {
                    child = this.diagram.nameTable[list[i]];
                    if (child.parent) {
                        parent = this.diagram.nameTable[child.parent];
                        if (parent) {
                            if (parent.container)
                                return true;
                        }
                    }
                }
            };

        ToolBase.prototype._fromSameContainer = function (list) {
            var child = null;
            var parent = null;
            for (var i = 0; i < list.length; i++) {
                child = this.diagram.nameTable[list[i]];
                if (!parent && child.parent)
                    parent = child.parent;
                if (child.parent) {
                    if (child.parent != parent)
                        return false;
                }
            }
            return true;
        },

        ToolBase.prototype._processCtrlKey = function (evt) {
            var obj;
            if (this.diagram.selectionList.length > 0) {
                var selectionList = this.diagram.selectionList;
                var isMultipleSelection;
                if (selectionList[0].isPseudoGroup) {
                    isMultipleSelection = true;
                    obj = this._findNodeUnderMouse(evt);
                }
                else {

                    obj = this._findNodeUnderMouse(evt);
                }
                var obj1 = this._findNodeUnderMouse(evt, true);
                var tempVal = this.selectedObject;
                var args = this._raiseEvent("itemClick", { element: this.selectedObject, actualObject: this.actualObject, selectedObject: obj1, cancel: false });
                this._itemClick(this.selectedObject, this.actualObject, obj1);
                if (args.cancel)
                    this.selectedObject = tempVal;
                if (this.actualObject) {
                    if (this.actualObject.parent) {
                        if (isMultipleSelection) {

                            var mulSelection = selectionList[0];
                            var temp = this._getProcessedObject(this.actualObject.name);
                            if (temp) {
                                if (this.selectedObject)
                                    ej.datavisualization.Diagram.Util.removeItem(mulSelection.children, this.selectedObject.name);
                                else if (temp.remove)
                                    ej.datavisualization.Diagram.Util.removeItem(mulSelection.children, temp.remove.name);


                                if (this.selectedObject) {
                                    if (!this._hasMultipleSelection(this.selectedObject)) {
                                        mulSelection.children.push(this.selectedObject.name);
                                    }
                                } else if (temp.add) {
                                    if (!this._hasMultipleSelection(temp.add)) {
                                        mulSelection.children.push(temp.add.name);
                                    }
                                }
                            } else {

                                nObj = this.actualObject;
                                if (this.selectedObject) {
                                    if (!this._hasMultipleSelection(this.selectedObject))
                                        mulSelection.children.push(this.selectedObject.name);
                                    else
                                        ej.datavisualization.Diagram.Util.removeItem(mulSelection.children, this.selectedObject.name);
                                } else {
                                    if (!this._hasMultipleSelection(nObj))
                                        mulSelection.children.push(nObj.name);
                                    else
                                        ej.datavisualization.Diagram.Util.removeItem(mulSelection.children, nObj.name);
                                }
                            }
                            ej.datavisualization.Diagram.Util._updateGroupBounds(mulSelection, this.diagram.nameTable);
                            if (mulSelection.children.length <= 1) {
                                var child = this.diagram.nameTable[mulSelection.children[0]];
                                if (this.diagram._hasSelection()) {
                                    this.diagram._clearSelection();
                                }
                                this.selectedObject = child;
                                this.diagram._addSelection(this.selectedObject);
                            } else {
                                this.selectedObject = selectionList[0];
                                ej.datavisualization.Diagram.SvgContext.updateSelector(this.selectedObject, this.diagram._svg, this.diagram._currZoom, this.diagram, this.diagram.model.selectorConstraints);
                            }

                        } else {
                            if (this.diagram.selectionList[0].isGroup) {
                                temp = this._getProcessedObject(this.actualObject.name);
                                if (temp) {
                                    if (this.diagram._hasSelection()) {
                                        this.diagram._clearSelection();
                                    }
                                    this.selectedObject = temp.add;
                                    this.diagram._addSelection(this.selectedObject);
                                }

                            } else {
                                temp = null;
                                if (obj.isGroup) {
                                    temp = this._getProcessedObject(this.actualObject.name);
                                }
                                if (temp)
                                    nObj = temp.add;
                                else
                                    nObj = obj;
                                selectionList = this.diagram.selectionList[0];
                                if (this.diagram._hasSelection()) {
                                    this.diagram._clearSelection();
                                }
                                pseudoGroup = ej.datavisualization.Diagram.Group({ isPseudoGroup: true, "name": "multipleSelection" });
                                if (selectionList.name != nObj.name)
                                    pseudoGroup.children.push(selectionList.name);
                                if (this.selectedObject)
                                    pseudoGroup.children.push(this.selectedObject.name);
                                else
                                    if (this._containsChild(pseudoGroup, nObj.name))
                                        pseudoGroup.children.push(nObj.name);

                                this._checkRepeatedChild(pseudoGroup);

                                this.diagram.model.nodes.push(pseudoGroup);
                                this.diagram.nameTable[pseudoGroup.name] = pseudoGroup;
                                ej.datavisualization.Diagram.Util._updateGroupBounds(pseudoGroup, this.diagram.nameTable);
                                this.selectedObject = pseudoGroup;
                                this.diagram._addSelection(this.selectedObject);
                            }
                        }
                    }
                    else {
                        if (isMultipleSelection) {
                            mulSelection = selectionList[0];
                            nObj = this.actualObject;
                            if (this.selectedObject) {
                                if (!this._hasMultipleSelection(this.selectedObject))
                                    mulSelection.children.push(this.selectedObject.name);
                                else
                                    ej.datavisualization.Diagram.Util.removeItem(mulSelection.children, this.selectedObject.name);
                            } else {
                                if (!this._hasMultipleSelection(nObj))
                                    mulSelection.children.push(nObj.name);
                                else
                                    ej.datavisualization.Diagram.Util.removeItem(mulSelection.children, nObj.name);
                            }
                            ej.datavisualization.Diagram.Util._updateGroupBounds(mulSelection, this.diagram.nameTable);
                            this.selectedObject = selectionList[0];
                            ej.datavisualization.Diagram.SvgContext.updateSelector(this.selectedObject, this.diagram._svg, this.diagram._currZoom, this.diagram, this.diagram.model.selectorConstraints);
                        }
                        else {
                            var nObj = this.actualObject;
                            selectionList = this.diagram.selectionList[0];
                            if (this.diagram._hasSelection()) {
                                this.diagram._clearSelection();
                            }
                            var pseudoGroup = ej.datavisualization.Diagram.Group({ isPseudoGroup: true, "name": "multipleSelection" });
                            pseudoGroup.children.push(selectionList.name);
                            if (this.selectedObject)
                                pseudoGroup.children.push(this.selectedObject.name);
                            else
                                pseudoGroup.children.push(nObj.name);
                            this.diagram.model.nodes.push(pseudoGroup);
                            this.diagram.nameTable[pseudoGroup.name] = pseudoGroup;
                            ej.datavisualization.Diagram.Util._updateGroupBounds(pseudoGroup, this.diagram.nameTable);
                            this.selectedObject = pseudoGroup;
                            this.diagram._addSelection(this.selectedObject);
                        }
                    }
                }

                this.selectedObject = this.diagram.selectionList[0];
            }
            else {
                obj = this._findNodeUnderMouse(evt);
                this.selectedObject = obj;
            }
            return obj;
        };

        ToolBase.prototype._checkRepeatedChild = function (list) {
            var child = null;
            var cloneList = $.extend(true, {}, list);
            for (var i = 0; i < cloneList.children.length; i++) {
                child = this.diagram.nameTable[cloneList.children[i]];
                if (child.isGroup) {
                    for (var j = 0; j < cloneList.children.length; j++) {
                        if (!this._containsChild(child, cloneList.children[j])) {
                            ej.datavisualization.Diagram.Util.removeItem(list.children, cloneList.children[j]);
                        }
                    }
                }
            }
        };
        ToolBase.prototype._hasMultipleSelection = function (node) {
            var children = this.diagram.selectionList[0].children;
            for (var i = 0; i < children.length; i++) {
                if (node.name == children[i])
                    return true;
            }
            return false;
        };

        ToolBase.prototype._getNextParent = function (node) {
            if (node.parent) {
                var parent = this.diagram.nameTable[node.parent];
                if (this._hasMultipleSelection(parent))
                    return node;
                else
                    return this._getNextParent(parent);
            }
            else
                return node;
        };
        ToolBase.prototype._hasMultipleSelection = function (node) {
            var children = this.diagram.selectionList[0].children;
            for (var i = 0; i < children.length; i++) {
                if (node.name == children[i])
                    return true;
            }
            return false;
        };

        ToolBase.prototype._getNextParent = function (node) {
            if (node.parent) {
                var parent = this.diagram.nameTable[node.parent];
                if (this._hasMultipleSelection(parent))
                    return node;
                else
                    return this._getNextParent(parent);
            }
            else
                return node;
        };
        ToolBase.prototype._findObj = function (element, group) {
            var obj;
            var id = element.getAttribute("id");
            if (group) {
                obj = this.diagram._findChildren(group, id);
            }
            else {
                var type = element.getAttribute("class");
                if (type) {
                    if (type === "ej-d-node" || type === "ej-d-group") {
                        obj = this.diagram._findNode(id);
                    }
                    else if (type === "ej-d-connector") {
                        obj = this.diagram._findConnector(id);
                    }
                }
            }
            return obj;
        };
        ToolBase.prototype.mousedown = function (evt) {
            this.startPoint = this.mousePosition(evt);
            this.currentPoint = this.mousePosition(evt);
            this.previousPoint = this.startPoint;
        };

        ToolBase.prototype.mousemove = function (evt) {
            this.currentPoint = this.mousePosition(evt);
        };
        ToolBase.prototype.mouseup = function (evt) {
            this._endAction();
            if (this.singleAction) {
                this.diagram.activateTool("select");
            }
        };
        ToolBase.prototype.mousePosition = function (evt) {
            return this.diagram._mousePosition(evt);
        };
        ToolBase.prototype.updateCursor = function (cursor) {
            this.diagram._currentCursor = cursor;
            this.diagram._updateCursor();
        };
        ToolBase.prototype.nearestGridPoint = function (point) {
            var snapPt = ej.datavisualization.Diagram.Point();
            if (this.diagram.model.snap & ej.datavisualization.Diagram.Snap.Grid) {
                var magnification = this.diagram.model.magnification / 100;
                var hSpacing = this.diagram.model.grid.horizontalSpacing * magnification;
                var vSpacing = this.diagram.model.grid.verticalSpacing * magnification;
                var width = point.x % hSpacing;
                var height = point.y % vSpacing;
                if (width >= (hSpacing / 2)) {
                    snapPt.x = point.x + hSpacing - width;
                }
                else {
                    snapPt.x = point.x - width;
                }
                if (height >= (vSpacing / 2)) {
                    snapPt.y = point.y + vSpacing - height;
                }
                else {
                    snapPt.y = point.y - height;
                }
            }
            return snapPt;
        };
        ToolBase.prototype.snap = function (point) {
            var snapSettings = this.diagram.model.snapSettings;

            var zoomFactor = this.diagram._currZoom;
            if (snapSettings.snapConstraints & ej.datavisualization.Diagram.SnapConstraints.SnapToVerticalLines)
                point.x = ej.datavisualization.Diagram.SnapUtil._round(point.x, snapSettings.verticalGridLines.snapInterval, zoomFactor);
            if (snapSettings.snapConstraints & ej.datavisualization.Diagram.SnapConstraints.SnapToHorizontalLines)
                point.y = ej.datavisualization.Diagram.SnapUtil._round(point.y, snapSettings.horizontalGridLines.snapInterval, zoomFactor);
            return point;

        };
        ToolBase.prototype._endAction = function () {
            this.inAction = false;

            if (this.svgHelper) {
                var helper = this.helper;
                var search = this.diagram._spatialSearch;
                if (!helper.isGroup) {
                    ej.datavisualization.Diagram.SpatialUtil._removeFromaQuad(search, search.quadTable[helper.name], helper);
                }
                else {
                    var length = this.helper.children.length - 1;
                    for (var i = length; i >= 0; i--) {
                        ej.datavisualization.Diagram.SpatialUtil._removeFromaQuad(search, search.quadTable[helper.children[i].name], helper.children[i]);
                    }
                }
            }
            if (this.helper) {
                if (this.svgHelper)
                    this.diagram._svg.removeChild(this.svgHelper, this.diagram._diagramLayer);
                if (this.helper.gradient) {
                    var defs = this.diagram._svg.getElementsByTagName("defs")[0];
                    defs.removeChild(this.diagram._svg.getElementById(this.helper.name + "_gradient"));
                }
            }
            if (this._previousPossibleConnection) {
                this._previousPossibleConnection = null;
                ej.datavisualization.Diagram.SvgContext._removePortHighlighter(this.diagram._svg, this.diagram._adornerLayer);
            }
            this._currentPossibleConnection = null;
            this._possibleConnectionPort = null;
            this.helper = null;
            this.svgHelper = null;
            this.selectedObject = null;
            this.startPoint = ej.datavisualization.Diagram.Point(0, 0);
            this.currentPoint = ej.datavisualization.Diagram.Point(0, 0);
            this.previousPoint = ej.datavisualization.Diagram.Point(0, 0);
        };
        ToolBase.prototype._isResizeCorner = function (type) {
            var isResizeCorner = false;
            switch (type) {
                case "n-resize":
                case "e-resize":
                case "w-resize":
                case "s-resize":
                case "ne-resize":
                case "nw-resize":
                case "se-resize":
                case "sw-resize":
                case "targetEndPoint":
                case "sourceEndPoint":
                    isResizeCorner = true;
                    break;
            }
            return isResizeCorner;
        };
        ToolBase.prototype._initHelper = function () {
            if (this.selectedObject) {
                this.helper = $.extend(true, {}, this.selectedObject);
                this._updateHelperName(this.helper);
            }
        };
        ToolBase.prototype._updateHelperName = function (helper, child) {
            if (child) {
                helper.name += "helper";
            }
            else
                helper.name = "helper";
            helper.opacity = 0.5;
            var labels = helper.labels;
            for (var i = 0, len = labels.length; i < len; ++i) {
                labels[i].name += "helper";
            }
            if (helper.ports) {
                var ports = helper.ports;
                for (i = 0, len = ports.length; i < len; ++i) {
                    ports[i].name += "helper";
                }
            }
            if (helper.isGroup) {
                var children = helper.children;
                for (i = 0, len = children.length; i < len; i++) {
                    this._updateHelperName(children[i], true);
                }
            }
        };
        ToolBase.prototype._raiseEvent = function (type, args) {
            args.elementType = this.diagram._getElementType(args.element);
            this.diagram._trigger(type, args);
            return args;
        };
        ToolBase.prototype._findPort = function (pt, node) {
            var port = this.diagram._findPortAtPoint(pt, node);
            if (!port && node.isGroup) {
                var childPort;
                for (var j = 0; j < node.children.length; j++) {
                    var child = this.diagram.nameTable[node.children[j]];
                    if (child && !child.segments) {
                        childPort = this._findPort(pt, child);
                        if (childPort && !child.isGroup) {
                            this._currentPossibleConnection = child;
                            port = childPort;
                        }
                        else if (childPort) {
                            return childPort;
                        }
                    }
                }
            }
            return port;
        };
        ToolBase.prototype._checkConnectionPossible = function (evt) {
            var port = null;
            var pt = ej.datavisualization.Diagram.Point(this.currentPoint.x, this.currentPoint.y);
            var node = this.diagram._findNodeUnderMouse(evt);
            if (node && !(node.segments)) {
                if (!(node.isPseudoGroup)) {
                    this._showPorts(node);
                    this._currentPossibleConnection = node;
                    if (this._previousPossibleConnection == null || this._previousPossibleConnection.name !== this._currentPossibleConnection.name) {
                        this._previousPossibleConnection = this._currentPossibleConnection;
                    }
                }
            }
            else if (this.diagram.activeTool instanceof ej.datavisualization.Diagram.LineTool) {
                this._showPorts();
                this._currentPossibleConnection = null;
                this._possibleConnectionPort = null;
            }
            else {

                var epoint = ej.datavisualization.Diagram.Point(0, 0);
                if (this._endPoint == "targetEndPoint") {
                    epoint.x = this.selectedObject.targetPoint.x - 25;
                    epoint.y = this.selectedObject.targetPoint.y - 25;
                }
                else if (this._endPoint == "sourceEndPoint") {

                    epoint.x = this.selectedObject.sourcePoint.x - 25;
                    epoint.y = this.selectedObject.sourcePoint.y - 25;
                }
                var bounds = ej.datavisualization.Diagram.Rectangle(epoint.x, epoint.y, 50, 50);
                var quads = ej.datavisualization.Diagram.SpatialUtil.findQuads(this.diagram._spatialSearch, bounds);
                for (var i = 0; i < quads.length; i++) {
                    var quad = quads[i];
                    if (quad.objects.length > 0) {
                        for (var j = 0; j < quad.objects.length; j++) {
                            var nd = quad.objects[j];
                            var nodebounds = ej.datavisualization.Diagram.Util.bounds(nd);
                            if (ej.datavisualization.Diagram.Geometry.intersectsRect(bounds, nodebounds)) {
                                if (!nd.segments) {
                                    this._showPort(nd);
                                }
                            }
                            else {
                                if (!nd.segments) {
                                    this._showPort(nd, true);
                                }
                            }
                        }
                    }
                }

                this._currentPossibleConnection = null;
                this._possibleConnectionPort = null;
            }
            if (this._currentPossibleConnection) {
                port = this._findPort(pt, this._currentPossibleConnection);
                if ((port && ej.datavisualization.Diagram.Util.canConnect(port, true)) || ej.datavisualization.Diagram.Util.canConnect(this._currentPossibleConnection)) {
                    ej.datavisualization.Diagram.SvgContext._drawPortHighlighter(port, this._currentPossibleConnection, this.diagram._svg, this.diagram._adornerLayer,
                        this.diagram._currZoom);
                }
                else {
                    ej.datavisualization.Diagram.SvgContext._removePortHighlighter(this.diagram._svg, this.diagram._adornerLayer);
                }
                this._possibleConnectionPort = port;
                if (port && ej.datavisualization.Diagram.Util.canConnect(port, true)) {
                    var connectionNode = this._currentPossibleConnection;
                    var point = ej.datavisualization.Diagram.Util._getPortPosition(this._possibleConnectionPort, ej.datavisualization.Diagram.Util.bounds(connectionNode, true));
                    var matrix = ej.Matrix.identity();
                    ej.Matrix.rotate(matrix, connectionNode.rotateAngle, connectionNode.offsetX, connectionNode.offsetY);
                    point = ej.Matrix.transform(matrix, point);
                    this.currentPoint = point;
                }
            }
            else {
                if (this._previousPossibleConnection) {
                    this._previousPossibleConnection = null;
                    ej.datavisualization.Diagram.SvgContext._removePortHighlighter(this.diagram._svg, this.diagram._adornerLayer);
                    this._possibleConnectionPort = null;
                }
            }

        };
        ToolBase.prototype._showPorts = function (node) {
            if (node) {
                if (this._currentPossibleConnection && this._currentPossibleConnection.name !== node.name) {
                    this._showPort(this._currentPossibleConnection, true);
                }
                this._showPort(node);
            }
            else {
                if (this._currentPossibleConnection != null) {
                    this._showPort(this._currentPossibleConnection, true);
                }
            }
        };
        ToolBase.prototype._showPort = function (node, hide) {
            var ports;
            var port;
            var portShape;
            var i;
            var len;
            if (hide) {
                ports = node.ports;
                for (i = 0, len = ports.length; i < len; ++i) {
                    port = ports[i];
                    if (port.visibility & ej.datavisualization.Diagram.PortVisibility.Hover ||
                        (port.visibility & ej.datavisualization.Diagram.PortVisibility.Connect &&
                        (this.name == "endPoint" || this instanceof ej.datavisualization.Diagram.LineTool))) {
                        portShape = this.diagram._svg.getElementById(node.name + "_" + port.name);
                        if (portShape)
                            portShape.setAttribute("visibility", "hidden");
                    }
                }
            }
            else {
                ports = node.ports;
                for (i = 0, len = ports.length; i < len; ++i) {
                    port = ports[i];
                    if (port.visibility & ej.datavisualization.Diagram.PortVisibility.Hover ||
                        (port.visibility & ej.datavisualization.Diagram.PortVisibility.Connect &&
                        (this.name == "endPoint" && this.inAction || this instanceof ej.datavisualization.Diagram.LineTool))) {
                        portShape = this.diagram._svg.getElementById(node.name + "_" + port.name);
                        if (portShape)
                            portShape.setAttribute("visibility", "visible");
                    }
                }
            }
        };
        ToolBase.prototype._showAllPorts = function (hide) {
            var i;
            var len;
            var nodes = this.diagram.model.nodes;
            if (hide) {
                for (i = 0, len = nodes.length; i < len; i++) {
                    this._showPort(nodes[i], hide);
                }
            }
            else {
                for (i = 0, len = nodes.length; i < len; ++i) {
                    this._showPort(nodes[i]);
                }
            }
        };
        ToolBase.prototype._disconnect = function (connector) {
            var args;
            var node = this.diagram.nameTable[connector.targetNode];
            var port = null;
            if (node) {
                port = ej.datavisualization.Diagram.Util.findPortByName(node, connector.targetPort);
                args = this._raiseEvent("connectionChange", { element: connector, connection: node, port: port, cancel: false });
                if (!args.cancel) {
                    ej.datavisualization.Diagram.Util.removeItem(node.inEdges, connector.name);
                    connector.targetNode = null;
                    connector.targetPort = null;
                }
            }
            node = this.diagram.nameTable[connector.sourceNode];
            if (node) {
                port = ej.datavisualization.Diagram.Util.findPortByName(node, connector.sourcePort);
                args = this._raiseEvent("connectionChange", { element: connector, connection: node, port: port, cancel: false });
                if (!args.cancel) {
                    ej.datavisualization.Diagram.Util.removeItem(node.outEdges, connector.name);
                    connector.sourceNode = null;
                    connector.sourcePort = null;
                }
            }
        };
        return ToolBase;
    })();

    ej.datavisualization.Diagram.ToolBase = ToolBase;
    //#endregion

    //#region SelectTool
    var SelectTool = (function (base) {
        ej.datavisualization.Diagram.extend(SelectTool, base);
        function SelectTool(diagram) {
            base.call(this, "select", diagram);
            this.cursor = "default";
            this._svgHelper = null;
        }
        SelectTool.prototype.mousedown = function (evt) {
            base.prototype.mousedown.call(this, evt);
            var node = this._findNodeUnderMouse(evt);
            if (node) {
                this.selectedObject = node;
            }
        };
        SelectTool.prototype.mousemove = function (evt) {
            base.prototype.mousemove.call(this, evt);
            if (!ej.datavisualization.Diagram.Geometry.isEmptyPoint(this.startPoint)) {
                if (ej.datavisualization.Diagram.Util.canDoMultipleSelection(this.diagram)) {
                    if (!this.inAction && (this.startPoint.x != this.currentPoint.x || this.startPoint.y != this.currentPoint.y)) {
                        this.inAction = true;
                        this._initHelper();
                    }
                    else
                        this._updateHelper();
                }
            }
        };
        SelectTool.prototype.mouseup = function (evt) {
            if (this.inAction) {
                this.currentPoint = this.mousePosition(evt);
                var rect = ej.datavisualization.Diagram.Geometry.rect([{ x: this.startPoint.x, y: this.startPoint.y }, { x: this.currentPoint.x, y: this.currentPoint.y }]);
                if (rect.width !== 0 || rect.height !== 0) {
                    var i;
                    var len;
                    var bounds;
                    var collection = [];
                    var nodes = this.diagram.model.nodes;
                    for (i = 0, len = nodes.length; i < len; i++) {
                        var node = this.diagram.nameTable[nodes[i].name];
                        if (node) {
                            bounds = ej.datavisualization.Diagram.Util.bounds(node);
                            bounds = ej.datavisualization.Diagram.Geometry.rect([bounds.topLeft, bounds.topRight, bounds.bottomRight, bounds.bottomLeft]);
                            if (ej.datavisualization.Diagram.Util.canSelect(node)) {
                                if (ej.datavisualization.Diagram.Geometry.containsRect(rect, bounds)) {
                                    if (node.parent === "")
                                        collection.push(node);
                                }
                                else if (node.isGroup) {
                                    this._checkGroupChildren(collection, node, rect);
                                }
                            }
                        }
                    }
                    var connectors = this.diagram.model.connectors;
                    for (i = 0, len = connectors.length; i < len; i++) {
                        var connector = this.diagram.nameTable[connectors[i].name];
                        if (connector) {
                            bounds = ej.datavisualization.Diagram.Util.bounds(connector);
                            bounds = ej.datavisualization.Diagram.Geometry.rect([bounds.topLeft, bounds.topRight, bounds.bottomRight, bounds.bottomLeft]);
                            if (ej.datavisualization.Diagram.Util.canSelect(connector) && ej.datavisualization.Diagram.Geometry.containsRect(rect, bounds)) {
                                if (connector.parent === "")
                                    collection.push(connector);
                            }
                        }
                    }
                    if (this.diagram._hasSelection()) {
                        this.diagram._clearSelection();
                    }
                    if (collection.length > 0) {
                        var children = collection;
                        var selection;
                        if (children.length > 1) {
                            var pseudoGroup = ej.datavisualization.Diagram.Group({ isPseudoGroup: true, "name": "multipleSelection" });
                            for (i = 0, len = children.length; i < len; i++) {
                                //children[i].parent = "multipleSelection";
                                pseudoGroup.children.push(children[i].name);
                            }
                            this.diagram.model.nodes.push(pseudoGroup);
                            this.diagram.nameTable[pseudoGroup.name] = pseudoGroup;
                            selection = pseudoGroup;
                        }
                        else {
                            if (ej.datavisualization.Diagram.Util.canDoSingleSelection(this.diagram))
                                selection = collection[0];
                        }
                        if (selection) {
                            if (selection.isGroup)
                                ej.datavisualization.Diagram.Util._updateGroupBounds(selection, this.diagram.nameTable);
                            this.diagram._addSelection(selection);
                        }
                    }
                }
                else {
                    if (this.diagram._hasSelection() && !this.diagram._isDropped) {
                        this.diagram._clearSelection();
                    }
                }
            }
            else {

                if (this.selectedObject || this.diagram._hasSelection()) {

                    if (evt.ctrlKey || evt.shiftKey) {
                        if (!ej.datavisualization.Diagram.Util.canDoMultipleSelection(this.diagram)) {

                            this._processCtrlKey(evt);
                        }
                    }
                    else if (this.diagram._hasSelection())
                        this.diagram._clearSelection();
                    this.diagram._addSelection(this.selectedObject);
                }
                else if (this.diagram._hasSelection() && !this.diagram._isDropped) {
                    this.diagram._clearSelection();
                }
            }
            this.diagram._isDropped = false;
            if (this.selectedObject)
                this._raiseEvent("click", { element: this.selectedObject });
            base.prototype.mouseup.call(this, evt);

        };
        SelectTool.prototype._endAction = function () {
            if (this._svgHelper) {
                this.diagram._adornerLayer.removeChild(this._svgHelper);
                this.selectedObject = null;
                this._svgHelper = null;
            }
            base.prototype._endAction.apply(this);
        };
        SelectTool.prototype._initHelper = function () {
            if (!this._svgHelper) {
                var g = this.diagram._svg.g();
                this.diagram._adornerLayer.appendChild(g);

                var scale = this.diagram._currZoom;
                var selectionRect = this.diagram._svg.rect({
                    "id": "helper", "x": this.startPoint.x * scale, "y": this.startPoint.y * scale,
                    "fill": "transparent", "stroke": "gray", "stroke-dasharray": "2 2", "shape-rendering": "crispEdges"
                });
                g.appendChild(selectionRect);
                g.setAttribute("pointer-events", "none");
                this._svgHelper = g;
            }
        };
        SelectTool.prototype._updateHelper = function () {
            var helper = this.diagram._svg.getElementById("helper");
            var width = Math.abs(this.startPoint.x - this.currentPoint.x);
            var height = Math.abs(this.startPoint.y - this.currentPoint.y);
            var scale = this.diagram._currZoom;
            var x = ((this.startPoint.x > this.currentPoint.x) ? this.currentPoint.x : this.startPoint.x) * scale;
            var y = ((this.startPoint.y > this.currentPoint.y) ? this.currentPoint.y : this.startPoint.y) * scale;

            if (helper) {
                this.diagram._svg.rect({
                    "id": helper.id, "x": x,
                    "y": y, "width": width * scale, "height": height * scale
                });
            }
        };
        SelectTool.prototype._checkGroupChildren = function (collection, group, bounds) {
            var child, childBounds;
            for (var i = 0; i < group.children.length; i++) {
                child = this.diagram.nameTable[group.children[i]];
                if (child) {
                    childBounds = ej.datavisualization.Diagram.Util.bounds(child);
                    if (ej.datavisualization.Diagram.Geometry.containsRect(bounds, childBounds)) {
                        collection.push(child);
                    }
                    else if (child.isGroup) {
                        this._checkGroupChildren(collection, child, bounds);
                    }
                }
            }
        }
        return SelectTool;
    })(ToolBase);

    ej.datavisualization.Diagram.SelectTool = SelectTool;
    //#endregion

    //#region MoveTool
    var MoveTool = (function (base) {
        ej.datavisualization.Diagram.extend(MoveTool, base);
        function MoveTool(diagram) {
            base.call(this, "move", diagram);
            this.cursor = "move";
            this._isMouseDown = false;
            this.diffx = 0;
            this.diffy = 0;
            this.helper = null;
            this.hoverNode = null;
            this.stackOverNode = null;
            this.undoObject = [];
            this.activeLabel = null;
        }
        MoveTool.prototype.mousedown = function (evt) {
            ej.datavisualization.Diagram.SnapUtil._removeGuidelines(this.diagram);
            this.diffx = 0;
            this.diffy = 0;
            this._allowPan(this.selectedObject, "mousedown", evt);
            base.prototype.mousedown.call(this, evt);
            this._isMouseDown = true;
            if (this.diagram._currentCursor != "pointer")
                this.updateCursor(this.cursor);
            this._findLabelUnderMouse(evt);
        };

        MoveTool.prototype._containsChild = function (node, childName) {
            var status = true;
            var child = null;
            for (var i = 0; i < node.children.length; i++) {
                child = this.diagram.nameTable[node.children[i]];
                if (child) {
                    if (child.isGroup)
                        status = this._containsChild(child, childName);
                    else {
                        if (child.name == childName)
                            status = false;
                    }
                }
            }
            return status;
        };
        MoveTool.prototype._nodeCanHighlight = function (hoverNode) {
            var status = false;
            if ((hoverNode.children).indexOf(this.selectedObject.name) == -1) {
                if (!this.selectedObject.isGroup) {
                    status = true;
                }
                else {
                    if (hoverNode.isGroup) {
                        if (!this._containsChild(this.selectedObject, hoverNode.name))
                            status = true;
                    }
                }
            }
            return status;
        },

        MoveTool.prototype._containerMouseMove = function (evt) {
            //#region for Container
            if (this.selectedObject.parent && this.diagram.nameTable[this.selectedObject.parent] && this.diagram.nameTable[this.selectedObject.parent].container) {
                var container = this.diagram.nameTable[this.selectedObject.parent].container;
                //ej.datavisualization.Diagram.SvgContext._hideNode(this.selectedObject, this.diagram._svg);
                if (!this.helper) {
                    ej.datavisualization.Diagram.SvgContext._disableSelectedNode(this.selectedObject, this.diagram._svg, this.diagram.nameTable);
                    this.helper = this._getCloneNode(this.selectedObject);
                    this.helper.name = "helper";
                    this.startPoint = this.currentPoint;
                    var bounds = ej.datavisualization.Diagram.Util.bounds(this.helper);
                    ej.datavisualization.Diagram.SvgContext._drawContainerHelper(this.diagram);
                } else {
                    this._updateHelperXY(this.helper, this.previousPoint, this.currentPoint);
                    ej.datavisualization.Diagram.SvgContext._updateContainerHelper(this.diagram);
                }
                if (container.type == "canvas") {
                    var overNode = this._getNodeUnderMouse(evt);
                    if (overNode && !overNode.segments && overNode.isGroup) {
                        if (overNode.container && overNode.container.type != "stack") {
                            this._nodeHighLighter(overNode);
                        }
                        this.hoverNode = overNode;
                    } else {
                        this._removeHighLighter();
                        this.hoverNode = null;
                    }
                } else if (container.type == "stack") {
                    var stackOverNode = this._findStackOverNode(this.helper, evt);
                    if (stackOverNode) {
                        this._nodeHighLighter(stackOverNode);
                        this.stackOverNode = stackOverNode;
                    } else {
                        this._removeHighLighter();
                        this.stackOverNode = null;
                    }
                }
            } else {
                if (!this.selectedObject.isSwimlane) {
                    if (this.selectedObject.isPseudoGroup && this.hasSameParent()) {
                        if (!this.helper) {
                            this.helper = this._getCloneNode(this.selectedObject);
                            this.helper.name = "helper";
                            this.helper.children = [];
                            this.startPoint = this.currentPoint;
                            ej.datavisualization.Diagram.SvgContext._drawContainerHelper(this.diagram);
                        } else {
                            bounds = ej.datavisualization.Diagram.Util.bounds(this.helper);
                            this._updateHelperXY(this.helper, this.previousPoint, this.currentPoint);

                            ej.datavisualization.Diagram.SvgContext._updateContainerHelper(this.diagram);
                        }
                        ej.datavisualization.Diagram.SvgContext._disableSelectedNode(this.selectedObject, this.diagram._svg, this.diagram.nameTable);
                        overNode = this._getNodeUnderMouse(evt);

                        var gHighLight = true;
                        if (overNode && !overNode.segments) {
                            if (overNode.container && overNode.container.type != "stack") {
                                this._nodeHighLighter(overNode);
                            }
                            this.hoverNode = overNode;
                        } else {
                            this._removeHighLighter();
                            this.hoverNode = null;
                        }
                        this._nodeHighLighter(overNode);
                    } else if (!this.isDiagram(evt)) {
                        overNode = this._getNodeUnderMouse(evt);
                        ej.datavisualization.Diagram.SvgContext._disableSelectedNode(this.selectedObject, this.diagram._svg, this.diagram.nameTable);
                        if (overNode && overNode.container && (this.selectedObject.isSwimlane || this.selectedObject.isLane)) {
                            if (this._getSwimLaneNode(overNode, this.selectedObject))
                                this._nodeHighLighter(overNode);
                        }
                        else if (overNode && overNode.container && overNode.container.type != "stack") {
                            if (this.selectedObject.name != overNode.name)
                                this._nodeHighLighter(overNode);
                        }
                        this.hoverNode = overNode;
                    } else
                        this._removeHighLighter();
                }
                if (!gHighLight) {
                    if (this.selectedObject.isPseudoGroup) {
                        if (this._fromDiagram(this.selectedObject.children))
                            this._updateXY(this.selectedObject, this.previousPoint, this.currentPoint);
                        else if (this._fromSameContainer(this.selectedObject.children) && this.hasSameParent()) {
                            this._updateXY(this.selectedObject, this.previousPoint, this.currentPoint);
                        }
                    }
                    else this._updateXY(this.selectedObject, this.previousPoint, this.currentPoint);
                }
            }
            //#endregion  
        },

        MoveTool.prototype.mousemove = function (evt) {
            this._allowPan(this.selectedObject, "mousemove", evt);
            base.prototype.mousemove.call(this, evt);
            ej.datavisualization.Diagram.SnapUtil._removeGuidelines(this.diagram);
            if (this._isMouseDown && !this.inAction) {//&& (this.startPoint.x != this.currentPoint.x || this.startPoint.y != this.currentPoint.y)) {
                if (!this.selectedObject) {
                    var selectedObject;
                    selectedObject = this._findNodeUnderMouse(evt, true);

                    if (this.diagram._selectedSymbol) {
                        selectedObject = this.diagram.selectionList[0];
                    }

                    if (selectedObject) {
                        if (!ej.datavisualization.Diagram.Util.canMoveLabel(selectedObject))
                            this.activeLabel = null;
                        if (this.diagram.selectionList[0] && this.diagram.selectionList[0].isPseudoGroup) {
                            if (this.activeLabel || this.diagram.selectionList[0] != selectedObject){
                                this.selectedObject = selectedObject;
                                this.diagram._clearSelection();
                            }
                            else
                                this.selectedObject = this.diagram.selectionList[0];
                        } else {
                            if (this.diagram.selectionList[0] != selectedObject) {
                                this.diagram._clearSelection();
                                this.selectedObject = this.diagram.nameTable[selectedObject.name];
                            } else {
                                this.selectedObject = this.diagram.nameTable[this.diagram.selectionList[0].name];
                            }
                            var tempVal = this.selectedObject;
                            var args = this._raiseEvent("itemClick", { actualObject: this.actualObject, selectedObject: this.selectedObject, model: this.diagram.model, cancel: false });
                            this._itemClick(this.selectedObject, this.actualObject, this.diagram.model);
                            if (args.cancel)
                                this.selectedObject = tempVal;
                            if (this.diagram.selectionList[0] != this.selectedObject)
                                this.diagram._clearSelection();
                        }
                        var data;
                        var childTable = {};
                        if (this.selectedObject.isGroup || this.selectedObject.isPseudoGroup)
                            childTable = this.diagram._getChildTable(this.selectedObject, childTable);
                        var temp = { "childTable": childTable, "node": this.selectedObject };
                        data = $.extend(true, {}, temp);
                        this.undoObject = jQuery.extend(true, {}, data);
                    }
                }
            }

            if (this.selectedObject && ej.datavisualization.Diagram.Util.canSelect(this.selectedObject)) {
                if (!this.inAction) {
                    //ej.datavisualization.Diagram.SvgContext._removePivotPoint(this.selectedObject, this.diagram._svg, this.diagram.model.zoomFactor);
                    this.inAction = true;
                    if (!this.activeLabel)
                        this.updateCursor("move");
                    this._updateSelection();
                }
                if (this.activeLabel)
                    this._updateLabelXY(this.activeLabel, this.previousPoint, this.currentPoint);
                else {
                    if (this.diagram && !this.diagram._isEditing) {
                        this._containerMouseMove(evt);
                    }

                    if (this.selectedObject.segments) {
                        var connector = this.selectedObject;
                        if (ej.datavisualization.Diagram.Util.isTargetConnected(connector) || ej.datavisualization.Diagram.Util.isSourceConnected(connector)) {
                            this._disconnect(connector);
                        }
                    }
                    if (this.selectedObject.isPseudoGroup) {
                        var child;
                        for (var i = 0, len = this.selectedObject.children.length; i < len; i++) {
                            child = this.diagram.nameTable[this.selectedObject.children[i]];
                            ej.datavisualization.Diagram.DiagramContext.update(child, this.diagram);
                        }
                    }
                    else {
                        if (this.selectedObject.parent) {
                            ej.datavisualization.Diagram.Util._updateGroupBounds(this.diagram.nameTable[this.selectedObject.parent], this.diagram.nameTable, this.diagram);
                        }
                        ej.datavisualization.Diagram.DiagramContext.update(this.selectedObject, this.diagram);
                    }

                    this.diagram._renderTooltip(this.selectedObject);
                    this._updateSelection();
                    this.diagram._updateSelectionHandle(true);
                }
            }
            this.previousPoint = this.currentPoint;
        };
        MoveTool.prototype._findStackOverNode = function (node, evt) {
            var parent = this.diagram.nameTable[node.parent];
            var child = null, pos, childBounds;
            if (parent) {
                var length = parent.children.length;
                for (var i = 0; i < length; i++) {
                    child = this.diagram.nameTable[parent.children[i]];
                    pos = this.mousePosition(evt);
                    childBounds = ej.datavisualization.Diagram.Util.bounds(child);
                    if (parent.container.orientation == "vertical") {
                        var lastChild = this.diagram.nameTable[parent.children[length - 1]];
                        var firstChild = this.diagram.nameTable[parent.children[0]];
                        if (pos.y >= childBounds.y && pos.y <= childBounds.bottom) {
                            return child;
                        }
                        else if (pos.y <= firstChild.offsetY - firstChild.height / 2) {
                            return firstChild;
                        }
                        else if (pos.y >= lastChild.offsetY + lastChild.height / 2) {
                            return lastChild;
                        }

                    } else {
                        lastChild = this.diagram.nameTable[parent.children[length - 1]];
                        firstChild = this.diagram.nameTable[parent.children[0]];
                        if (pos.x >= childBounds.x && pos.x <= childBounds.right) {
                            return child;
                        }
                        else if (pos.x <= firstChild.offsetX - firstChild.width / 2) {
                            return firstChild;
                        }
                        else if (pos.x >= lastChild.offsetX + lastChild.width / 2) {
                            return lastChild;
                        }
                    }
                }

            }
        };

        MoveTool.prototype._removeFromContainer = function (node) {

            if (node.isPseudoGroup) {
                var child = null;
                for (var i = 0; i < node.children.length; i++) {
                    child = this.diagram.nameTable[node.children[i]];
                    if (child) {
                        ej.datavisualization.Diagram.SvgContext._removeFromContainer(child, this.diagram._svg, this.diagram._adornerLayer);
                        //delete this.diagram.nameTable[child.name];
                    }
                }
            } else {
                //delete this.diagram.nameTable[node.name];
                ej.datavisualization.Diagram.SvgContext._removeFromContainer(node, this.diagram._svg, this.diagram._adornerLayer);
            }
        };

        MoveTool.prototype._updateContainerProperties = function (node) {
            if (node && node.isPseudoGroup) {
                var child = null;
                var diffX = this.helper.offsetX - this.selectedObject.offsetX;
                var diffY = this.helper.offsetY - this.selectedObject.offsetY;
                for (var i = 0; i < node.children.length; i++) {
                    child = this.diagram.nameTable[node.children[i]];
                    if (child) {
                        oldParent = this.diagram.nameTable[this.selectedObject.parent];
                        if (oldParent)
                            ej.datavisualization.Diagram.Util.removeItem(oldParent.children, this.selectedObject.name);

                        newObj = this.diagram.nameTable[child.name];
                        if (newObj) {
                            var oldPar = this.diagram.nameTable[newObj.parent];
                            ej.datavisualization.Diagram.Util.removeItem(oldPar.children, newObj.name);
                            newObj.parent = this.hoverNode.name;
                            var newpar = this.diagram.nameTable[newObj.parent];
                            if (this._containsChild(newpar, newObj.name))
                                newpar.children.push(newObj.name);
                            this.diagram.updateNode(newObj.name, { offsetX: newObj.offsetX + diffX });
                            this.diagram.updateNode(newObj.name, { offsetY: newObj.offsetY + diffY });
                            this.diagram.nameTable[newObj.name] = newObj;
                        }
                    }
                }
                node.offsetX += diffX;
                node.offsetY += diffY;
                return node;
            }
            else {
                var oldParent = this.diagram.nameTable[this.selectedObject.parent];
                if (oldParent && oldParent.children)
                    ej.datavisualization.Diagram.Util.removeItem(oldParent.children, this.selectedObject.name);
                var newObj = this.selectedObject;//$.extend(true, {}, this.selectedObject);
                newObj.parent = this.hoverNode.name;
                this.diagram.nameTable[newObj.name] = newObj;
                return newObj;
            }
        };

        MoveTool.prototype._updatePreviousContainer = function (node) {
            if (node.isGroup) {
                if (node.container.type == "canvas") {
                    var groupBounds = ej.datavisualization.Diagram.Util.bounds(node);
                    var newSize = this.diagram._getMinMaxSize(node, groupBounds.width, groupBounds.height);
                    node.width = newSize.width;
                    node.height = newSize.height;

                } else if (node.container.type == "stack") {

                }
            }
        };

        MoveTool.prototype._updateContainer = function (newObj, prevParent, isDrop) {
            var bounds, groupBounds, dx = 0, dy = 0, newSize;
            if (newObj) {
                var parObj = this.diagram.nameTable[newObj.parent];
                if (!parObj && newObj.isPseudoGroup) {
                    var fNode = this.diagram.nameTable[this.selectedObject.children[0]];
                    parObj = this.diagram.nameTable[fNode.parent];
                }
                if (parObj) {
                    if (parObj.container) {
                        if (parObj.container.type == "canvas") {
                            if (isDrop) {
                                var group = this.diagram.nameTable[newObj.parent];
                                if (!group && newObj.isPseudoGroup) {
                                    group = parObj;
                                }
                                var groupBounds123 = ej.datavisualization.Diagram.Util.bounds(group);

                                var grAngle = group.rotateAngle;
                                this.diagram._rotate(group, -group.rotateAngle, this.diagram.nameTable);
                                bounds = ej.datavisualization.Diagram.Util.bounds(newObj);
                                var bounds1 = ej.datavisualization.Diagram.Geometry.rect([bounds.topLeft, bounds.topRight, bounds.bottomRight, bounds.bottomLeft]);
                                groupBounds = ej.datavisualization.Diagram.Util.bounds(group);
                                var tightchildBounds = this.diagram._getTightChildrenBounds(group);
                                var right = true;
                                var bottom = true;
                                var bRight = bounds1.x + bounds1.width;
                                var bBottom = bounds1.y + bounds1.height;
                                if (bRight >= groupBounds.right) {
                                    ///if exeeds right 
                                    dx = bRight - groupBounds.right + group.paddingRight;
                                    newSize = this.diagram._getMinMaxSize(group, (group.width + dx), group.height, true);
                                    widDiff = newSize.width - group.width;
                                    group.width = newSize.width;
                                    group.offsetX += (widDiff / 2);
                                    if (bRight < groupBounds.right) {
                                        right = false;
                                    }
                                }
                                if (bBottom >= groupBounds.bottom) {
                                    ///if exeeds bottom 
                                    dy = bBottom - groupBounds.bottom + group.paddingBottom;
                                    newSize = this.diagram._getMinMaxSize(group, (group.width), (group.height + dy), true);
                                    heiDiff = newSize.height - group.height;
                                    group.height = newSize.height;
                                    group.offsetY += (heiDiff / 2);
                                    //group.offsetY += (dy / 2);
                                    //group.height += dy;
                                    bottom = true;
                                    if (bBottom < groupBounds.bottom) {
                                        bottom = false;
                                    }
                                }
                                if (bRight < groupBounds.right && right) { //if decreces from right
                                    dx = (tightchildBounds.x + tightchildBounds.width) + group.paddingRight - groupBounds.right;
                                    newSize = this.diagram._getMinMaxSize(group, (group.width + dx), group.height, true);
                                    var widDiff = newSize.width - group.width;
                                    group.width = newSize.width;
                                    group.offsetX += (widDiff / 2);
                                }
                                if (bBottom < groupBounds.bottom && bottom) { //if decreces from bottom
                                    dy = (tightchildBounds.y + tightchildBounds.height) + group.paddingBottom - groupBounds.bottom;
                                    newSize = this.diagram._getMinMaxSize(group, (group.width), (group.height + dy), true);
                                    var heiDiff = newSize.height - group.height;
                                    group.height = newSize.height;
                                    group.offsetY += (heiDiff / 2);
                                }

                                this._updateMargin(newObj, group);
                                ej.datavisualization.Diagram.SvgContext._alignOnCanvas1(group, this.diagram);
                                this.diagram._rotate(group, grAngle, this.diagram.nameTable);
                                var groupBounds1231 = ej.datavisualization.Diagram.Util.bounds(group);
                                this.diagram._translate(group, groupBounds123.topLeft.x - groupBounds1231.topLeft.x, groupBounds123.topLeft.y - groupBounds1231.topLeft.y, this.diagram.nameTable);
                                if (group.parent && group.container && group.container.type == "canvas")
                                    this._updateMargin(group, this.diagram.nameTable[group.parent]);

                                ej.datavisualization.Diagram.DiagramContext.update(group, this.diagram);

                            }
                            else {
                                var vertical = false;
                                var horizontal = false;
                                group = this.diagram.nameTable[newObj.parent];
                                groupBounds123 = ej.datavisualization.Diagram.Util.bounds(group);
                                grAngle = group.rotateAngle;
                                if (!((group.rotateAngle == 0) && (grAngle == 0)))
                                    this.diagram._rotate(group, -group.rotateAngle, this.diagram.nameTable);
                                if (group.container && group.container.type == "canvas") {
                                    this._updateMargin(newObj, group);
                                }
                                if (newObj.container.type == "stack") {
                                    if (newObj.container.orientation == "vertical")
                                        vertical = true;
                                    else
                                        horizontal = true;
                                }
                                bounds = this.diagram._getBoundsWithoutStretch(group, vertical, horizontal);// ej.datavisualization.Diagram.SvgContext._measureStackSize(group, this.diagram, true);
                                newSize = this.diagram._getMinMaxSize(group, (bounds.width), (bounds.height), true);
                                widDif = newSize.width - group.width;
                                heiDif = newSize.height - group.height;
                                group.offsetX += (widDif / 2);
                                group.width = newSize.width;
                                group.offsetY += (heiDif / 2);
                                group.height = newSize.height;
                                ej.datavisualization.Diagram.SvgContext._alignOnCanvas1(group, this.diagram);
                                if (!((group.rotateAngle == 0) && (grAngle == 0)))
                                    this.diagram._rotate(group, grAngle, this.diagram.nameTable);
                                groupBounds1231 = ej.datavisualization.Diagram.Util.bounds(group);
                                this.diagram._translate(group, groupBounds123.topLeft.x - groupBounds1231.topLeft.x, groupBounds123.topLeft.y - groupBounds1231.topLeft.y, this.diagram.nameTable);
                            }
                            if (newObj.parent) {
                                this._updateContainer(this.diagram.nameTable[newObj.parent]);
                                if (!this.diagram.nameTable[newObj.parent].parent)
                                    ej.datavisualization.Diagram.DiagramContext.update(this.diagram.nameTable[newObj.parent], this.diagram);
                            } else if (newObj.isPseudoGroup) {
                                fNode = this.diagram.nameTable[newObj.children[0]];
                                var parObj2 = this.diagram.nameTable[fNode.parent];
                                if (parObj2.parent) {
                                    this._updateContainer(this.diagram.nameTable[parObj2.name]);
                                    if (!this.diagram.nameTable[parObj2.name])
                                        ej.datavisualization.Diagram.DiagramContext.update(this.diagram.nameTable[parObj2.name], this.diagram);
                                }
                            }

                        }//end canvas

                        else if (parObj.container.type == "stack") {
                            group = this.diagram.nameTable[newObj.parent];
                            bounds = ej.datavisualization.Diagram.SvgContext._measureStackSize(group, this.diagram, true);
                            groupBounds123 = ej.datavisualization.Diagram.Util.bounds(group);
                            grAngle = group.rotateAngle;
                            if (!((group.rotateAngle == 0) && (grAngle == 0)))
                                this.diagram._rotate(group, -group.rotateAngle, this.diagram.nameTable);
                            newSize = this.diagram._getMinMaxSize(group, (bounds.width), (bounds.height), true);
                            var widDif = newSize.width - group.width;
                            var heiDif = newSize.height - group.height;

                            group.offsetX += (widDif / 2);
                            group.width = newSize.width;
                            group.offsetY += (heiDif / 2);
                            group.height = newSize.height;


                            ej.datavisualization.Diagram.SvgContext._alignOnStack(group, this.diagram);
                            if (!((group.rotateAngle == 0) && (grAngle == 0)))
                                this.diagram._rotate(group, grAngle, this.diagram.nameTable);
                            groupBounds1231 = ej.datavisualization.Diagram.Util.bounds(group);
                            this.diagram._translate(group, groupBounds123.topLeft.x - groupBounds1231.topLeft.x, groupBounds123.topLeft.y - groupBounds1231.topLeft.y, this.diagram.nameTable);
                            if (group.parent)
                                this._updateContainer(group);
                            ej.datavisualization.Diagram.DiagramContext.update(group, this.diagram);
                        }
                    }
                }
            }
        };

        MoveTool.prototype._addToContainer = function (node) {
            if (node.isPseudoGroup) {
                var child = null;
                for (var i = 0; i < node.children.length; i++) {
                    child = this.diagram.nameTable[node.children[i]];
                    if (child) {
                        parent = this.diagram._svg.getElementById(child.parent);
                        if (!parent)
                            parent = this.diagram._diagramLayer;
                        if (child.isGroup) {
                            ej.datavisualization.Diagram.SvgContext.renderGroup(child, this.diagram._svg, parent, this.diagram.nameTable, this.diagram);
                        } else if (child.segments) {
                            ej.datavisualization.Diagram.SvgContext.renderConnector(child, this.diagram._svg, parent);
                        } else {
                            ej.datavisualization.Diagram.SvgContext.renderNode(child, this.diagram._svg, parent);
                        }
                    }
                }
            } else {
                this.diagram._clearSelection();
                var parent;
                if (this.hoverNode && this.hoverNode.children) {
                    this.hoverNode.children.push(node.name);
                    //this.diagram.nameTable[node.name] = node;
                }
                if (node) {
                    parent = this.diagram._svg.getElementById(node.parent);
                    if (!parent)
                        parent = this.diagram._diagramLayer;
                    if (node.isGroup) {
                        ej.datavisualization.Diagram.SvgContext.renderGroup(node, this.diagram._svg, parent, this.diagram.nameTable, this.diagram);
                    } else if (node.segments) {
                        ej.datavisualization.Diagram.SvgContext.renderConnector(node, this.diagram._svg, parent);
                    } else {
                        ej.datavisualization.Diagram.SvgContext.renderNode(node, this.diagram._svg, parent);
                    }
                }
            }
            this.selectedObject = node;
        };
        MoveTool.prototype._addToDiagram = function () {
            this.seletedObject = this.diagram.nameTable[this.selectedObject.name];
            this._removeFromContainer(this.selectedObject);
            var oldParent = this.diagram.nameTable[this.selectedObject.parent];
            ej.datavisualization.Diagram.Util.removeItem(oldParent.children, this.selectedObject.name);
            this.selectedObject.parent = "";
            this.diagram.nameTable[this.selectedObject.name] = this.selectedObject;
            this._updateXY(this.diagram.nameTable[this.selectedObject.name], this.startPoint, this.currentPoint);
            this._addToContainer(this.selectedObject);
        };
        MoveTool.prototype.isDiagram = function (evt) {
            if (evt.target.id == this.diagram._id + "_canvas_svg")
                return true;
            else
                return false;
        };
        MoveTool.prototype._allowPan = function (node, moveType, evt) {
            if (moveType === "mousedown")
                this.diagram.tools["panTool"].mousedown(evt);
            if (node && (node.isGroup || node.shape) && (node.constraints & ej.datavisualization.Diagram.NodeConstraints.AllowPan)) {
                if (moveType === "mousemove")
                    this.diagram.tools["panTool"].mousemove(evt);
                else if (moveType === "mouseup")
                    this.diagram.tools["panTool"].mouseup(evt);
            }
        };
        MoveTool.prototype._checkForDropEvent = function (hoverNode) {
            var source = null, target = null, childBounds = null;
            if (this.selectedObject && this.selectedObject.parent)
                source = this.diagram.nameTable[this.selectedObject.parent];
            if (hoverNode) {
                childBounds = ej.datavisualization.Diagram.Util.bounds(hoverNode);
                target = this._getTargetNode(hoverNode, childBounds);
            }
            var args = this._raiseEvent("drop", { cancel: false, source: source, target: target });
            return args.cancel;
        };

        MoveTool.prototype._getSwimLaneNode = function (node, selNode) {
            if (node.isSwimlane) {
                if (this.diagram.nameTable[node.children[1]].container.orientation != selNode.orientation)
                    return node;
            }
            if (node.parent) {
                var parent = this.diagram.nameTable[node.parent];
                if (parent) {
                    return this._getSwimLaneNode(parent, selNode);
                }
            }
            return null;
        };
        MoveTool.prototype._cloneGroupNode = function (node, id) {
            var child = null, child1, clnObj, node1;
            node1 = $.extend(true, {}, node);
            node1.name += id;
            if (node.isGroup) {
                node1.children = [];
                for (var i = 0; i < node.children.length; i++) {
                    child = this.diagram.nameTable[node.children[i]];
                    child1 = $.extend(true, {}, child);
                    child1.name += id;
                    if (child1.parent)
                        child1.parent += id;
                    if (child1.isGroup)
                        this._cloneGroupNode(child1, id);
                    this.diagram.nameTable[child1.name] = child1;
                    this.diagram.model.nodes.push(child1);
                    node1.children.push(child1.name);
                }
            }
            return node1;
        };
        MoveTool.prototype._getSwimLaneStackIndex = function (node) {
            if (node.container.type == "canvas") {
                if (node.parent)
                    var parent = this.diagram.nameTable[node.parent];
                if (parent) {
                    return parent.children.indexOf(node.name);
                }
            } else {
                return null;
            }
        };
        MoveTool.prototype._changeNodeState = function (evt, doc) {
            var newObj, pNode, mergNode = null;
            //#region drop fropm palette
            if (this.diagram._selectedSymbol && this.diagram._selectedSymbol.isLane && !doc) {
                ej.datavisualization.Diagram.SvgContext._enableSelectedNode(this.selectedObject, this.diagram._svg);
                var btmNode = this._findNodeUnderMouse(evt);
                if (btmNode)
                    mergNode = this._getSwimLaneNode(btmNode, this.selectedObject);
                if (btmNode && mergNode) {
                    ej.datavisualization.Diagram.SvgContext._removeContainerHelper(this.selectedObject, this.diagram._svg, this.diagram._adornerLayer);
                    this._removeHighLighter();
                    var stack = this.diagram.nameTable[mergNode.children[1]];
                    if (stack) {
                        var id = ej.datavisualization.Diagram.Util.randomId();
                        var selctObject = this._cloneGroupNode(this.selectedObject, id);
                        this.diagram.remove(this.selectedObject);
                        if (this.hoverNode)
                            var index = this._getSwimLaneStackIndex(this.hoverNode);
                        if (index || index == 0) {
                            stack.children.splice(index, 0, selctObject.name);
                        }
                        else
                            stack.children.push(selctObject.name);
                        selctObject.parent = stack.name;
                        this.diagram.add(selctObject);
                        var stackParent = this.diagram.nameTable[stack.parent];
                        if (stackParent) {
                            this._updateContainer(selctObject);
                        }
                        ej.datavisualization.Diagram.DiagramContext.update(stackParent, this.diagram);
                        this.selectedObject = stackParent;
                    }
                }
                else {
                    //add new stack
                    var swimlane = ej.datavisualization.Diagram.SwimLaneHelper._createSwimlane(this.selectedObject, this.diagram);
                    if (swimlane) {
                        this.diagram.remove(this.selectedObject);
                        this.diagram.add(swimlane);
                        this.selectedObject = swimlane;
                    }
                }

            }//#endregion
                //#region for Container 
            else if (this.selectedObject && this.selectedObject.parent && this.diagram.nameTable[this.selectedObject.parent].container) {
                ej.datavisualization.Diagram.SvgContext._enableSelectedNode(this.selectedObject, this.diagram._svg, this.diagram.nameTable);
                var container = this.diagram.nameTable[this.selectedObject.parent].container;
                if (container.type == "canvas") {
                    ej.datavisualization.Diagram.SvgContext._showNode(this.selectedObject, this.diagram._svg);
                    ej.datavisualization.Diagram.SvgContext._removeContainerHelper(this.selectedObject, this.diagram._svg, this.diagram._adornerLayer);
                    this._removeHighLighter(); //ej.datavisualization.Diagram.SvgContext._removeNodeHighlighter(this.diagram._svg, this.diagram._adornerLayer);
                    var checkDrop = !this._checkForDropEvent(this.hoverNode);
                    if (checkDrop) {
                        if (this.hoverNode && this.hoverNode.allowDrop || (this.hoverNode && this.hoverNode.name == this.selectedObject.parent)) {
                            if (this.hoverNode.container && this.hoverNode.container.type == "canvas") {
                                //var event = this._checkForDropEvent(evt);
                                //if (!event) {
                                this.seletedObject = this.diagram.nameTable[this.selectedObject.name];
                                pNode = this.selectedObject.parent;
                                this._removeFromContainer(this.selectedObject);
                                newObj = this._updateContainerProperties(this.diagram.nameTable[this.selectedObject.name]);
                                this._updateXY(this.diagram.nameTable[this.selectedObject.name], this.startPoint, this.currentPoint);
                                this._addToContainer(newObj);
                                //}
                            }
                        } else { //add to diagram
                            if (this.isDiagram(evt)) {
                                this._addToDiagram();
                            }
                            this.diffx = 0;
                            this.diffy = 0;
                        }
                    }
                } else if (container.type == "stack") {
                    ej.datavisualization.Diagram.SvgContext._removeContainerHelper(this.selectedObject, this.diagram._svg, this.diagram._adornerLayer);
                    this._removeHighLighter();
                }
            }
            else {///added from diagram to container
                ej.datavisualization.Diagram.SvgContext._enableSelectedNode(this.selectedObject, this.diagram._svg, this.diagram.nameTable);
                if (!this.selectedObject.isSwimlane) {
                    if (this.selectedObject.isPseudoGroup && this.hasSameParent()) {
                        this.currentPoint = this.previousPoint;
                        this.diffx = 0;
                        this.diffy = 0;
                        this._removeHighLighter();
                        ej.datavisualization.Diagram.SvgContext._removeContainerHelper(this.selectedObject, this.diagram._svg, this.diagram._adornerLayer);
                        if (this.hoverNode && this.hoverNode.allowDrop && !this._checkForDropEvent(this.hoverNode) || (this.hoverNode && this.hoverNode.name == this.selectedObject.parent)) {
                            this.seletedObject = this.diagram.nameTable[this.selectedObject.name];
                            pNode = this.selectedObject.parent;
                            newObj = this._updateContainerProperties(this.diagram.nameTable[this.selectedObject.name]);
                            this._removeFromContainer(this.selectedObject);
                            this._addToContainer(newObj);
                        } else {
                            this.diffx = 0;
                            this.diffy = 0;
                        }
                    } else if (!this.isDiagram(evt) && !this.selectedObject.isPseudoGroup) {
                        var overNode = this.hoverNode;
                        if (overNode && overNode.container && this.hoverNode.allowDrop && !this._checkForDropEvent(this.hoverNode) || (this.hoverNode && this.hoverNode.container && this.hoverNode.name == this.selectedObject.parent)) {
                            ej.datavisualization.Diagram.SvgContext._enableSelectedNode(this.selectedObject, this.diagram._svg, this.diagram.nameTable);
                            ej.datavisualization.Diagram.SvgContext._removeContainerHelper(this.selectedObject, this.diagram._svg, this.diagram._adornerLayer);
                            this._removeHighLighter();
                            this.seletedObject = this.diagram.nameTable[this.selectedObject.name];
                            this._removeFromContainer(this.selectedObject);
                            newObj = this._updateContainerProperties(this.diagram.nameTable[this.selectedObject.name]);
                            //this._updateXY(this.diagram.nameTable[this.selectedObject.name], this.startPoint, this.currentPoint);
                            var parent = this.diagram.nameTable[newObj.parent];
                            if (parent)
                                this._updateMargin(newObj, parent);
                            this._addToContainer(newObj);
                            ej.datavisualization.Diagram.SvgContext._alignOnCanvas1(parent, this.diagram);
                        }
                    } else {

                        //    overNode = this.hoverNode;
                        //    if (overNode && overNode.container && this.hoverNode.allowDrop || (this.hoverNode && this.hoverNode.name == this.selectedObject.parent)) {
                        //        ej.datavisualization.Diagram.SvgContext._enableSelectedNode(this.selectedObject, this.diagram._svg, this.diagram.nameTable);
                        //        ej.datavisualization.Diagram.SvgContext._removeContainerHelper(this.selectedObject, this.diagram._svg, this.diagram._adornerLayer);
                        //        ej.datavisualization.Diagram.SvgContext._removeNodeHighlighter(this.diagram._svg, this.diagram._adornerLayer);
                        //        this.seletedObject = this.diagram.nameTable[this.selectedObject.name];
                        //        this._removeFromContainer(this.selectedObject);
                        //        newObj = this._updateContainerProperties(this.diagram.nameTable[this.selectedObject.name]);
                        //        //this._updateXY(this.diagram.nameTable[this.selectedObject.name], this.startPoint, this.currentPoint);
                        //        parent = this.diagram.nameTable[newObj.parent];
                        //        if (parent)
                        //            this._updateMargin(newObj, parent);
                        //        this._addToContainer(newObj);
                        //        ej.datavisualization.Diagram.SvgContext._alignOnCanvas1(parent, this.diagram);
                        //    }
                    }
                }
            }
            return newObj;
            //#endregion   
        },

         MoveTool.prototype._updateNodeState = function (evt, newObj) {
             if (this.diagram.nameTable[this.selectedObject.parent])
                 var container = this.diagram.nameTable[this.selectedObject.parent].container;
             if (this.selectedObject.parent && this.diagram.nameTable[this.selectedObject.parent].container) {
                 container = this.diagram.nameTable[this.selectedObject.parent].container;
                 if (container.type == "canvas") {
                     if (this.hoverNode && newObj) {
                         var pNode = this.selectedObject.parent;
                         this._updateContainer(newObj, pNode, true);
                         ej.datavisualization.Diagram.DiagramContext.update(newObj, this.diagram);
                     }
                 }
                 else if (container.type == "stack") {

                     var evtArgs = this._checkForDropEvent(this.stackOverNode);
                     if (!evtArgs) {
                         var stackOverNode = this.stackOverNode;
                         if (stackOverNode) {
                             var parent = this.diagram.nameTable[stackOverNode.parent];

                             var groupBounds123 = ej.datavisualization.Diagram.Util.bounds(parent);
                             var grAngle = parent.rotateAngle;
                             if (!(parent.rotateAngle == 0) && (grAngle == 0))
                                 this.diagram._rotate(parent, -parent.rotateAngle, this.diagram.nameTable);

                             this._swapChildren(parent.children, this.selectedObject.name, stackOverNode.name);
                             this._updateContainer(this.selectedObject);

                             if (!(parent.rotateAngle == 0) && (grAngle == 0))
                                 this.diagram._rotate(parent, grAngle, this.diagram.nameTable);
                             var groupBounds1231 = ej.datavisualization.Diagram.Util.bounds(parent);
                             this.diagram._translate(parent, groupBounds123.topLeft.x - groupBounds1231.topLeft.x, groupBounds123.topLeft.y - groupBounds1231.topLeft.y, this.diagram.nameTable);
                             ej.datavisualization.Diagram.DiagramContext.update(parent, this.diagram);

                         } else {
                             this.diffx = 0;
                             this.diffy = 0;
                         }
                     }
                 }
             }
             else if (this.selectedObject.isPseudoGroup) {
                 var fNode = this.diagram.nameTable[this.selectedObject.children[0]];
                 if (fNode.parent)
                     container = this.diagram.nameTable[fNode.parent].container;
                 if (container && container.type == "canvas") {
                     if (this.hoverNode && newObj) {
                         pNode = this.selectedObject.parent;
                         this._updateContainer(newObj, pNode, true);
                         ej.datavisualization.Diagram.DiagramContext.update(newObj, this.diagram);
                     }
                 }

             }
         },
        MoveTool.prototype.mouseup = function (evt, doc) {
            var skip = this.inAction;
            this.diffx = 0;
            this.diffy = 0;
            this._isMouseDown = false;
            ej.datavisualization.Diagram.SnapUtil._removeGuidelines(this.diagram);
            var clickedObject;
            if (this.inAction) {
                var newObj = this._changeNodeState(evt, doc);
                this.inAction = false;
                if (this.activeLabel)
                    this._updateLabelXY(this.activeLabel, this.previousPoint, this.currentPoint);
                else
                    this._updateXY(this.selectedObject, this.previousPoint, this.currentPoint);
                //#region for Container
                this._updateNodeState(evt, newObj);
                //#endregion 
                this._updateObject();

                var data;
                var childTable = {};
                if (this.selectedObject.isGroup || this.selectedObject.isPseudoGroup)
                    childTable = this.diagram._getChildTable(this.selectedObject, childTable);
                var temp = { "childTable": childTable, "node": this.selectedObject };
                data = $.extend(true, {}, temp);

                if (!this.diagram._selectedSymbol) {
                    var entry = { type: "positionchanged", undoObject: this.undoObject, redoObject: jQuery.extend(true, {}, data), isMultipleNode: this.selectedObject.isGroup ? true : false };
                    this.diagram.addHistoryEntry(entry);
                }
                //var entry = new ej.datavisualization.Diagram.HistoryEntry(new ej.datavisualization.Diagram.MoveHandleCmd(this.undoObject, ej.datavisualization.Diagram.Util.clone(this.selectedObject), this.selectedObject.isPseudoGroup ? true : false));
                //this.diagram.historyManager.addHistoryEntry(entry);
                this.diagram._updateSelectionHandle();
                clickedObject = this.selectedObject;
            }
            else {
                if (!this.selectedObject) {
                    if (evt.which == 3) {
                        if (this.diagram.selectionList && !(this.diagram.selectionList[0].isPseudoGroup)) {
                            if (this.diagram.selectionList.length > 0) {
                                this.selectedObject = this._findNodeUnderMouse(evt, true);
                                clickedObject = this.selectedObject;
                            } else {
                                this.selectedObject = this._findNodeUnderMouse(evt);
                                clickedObject = this.selectedObject;
                            }
                            if (this.selectedObject && !this.diagram._selectionContains(this.selectedObject))
                                this.diagram._clearSelection();
                        }
                    }
                    else {
                        if (evt.ctrlKey || evt.shiftKey) {
                            clickedObject = this._processCtrlKey(evt);
                        }
                        else {
                            if (ej.datavisualization.Diagram.Util.canDoSingleSelection(this.diagram))
                                this.selectedObject = this._findNodeUnderMouse(evt);
                            clickedObject = this.selectedObject;
                        }
                        if (this.selectedObject && !this.diagram._selectionContains(this.selectedObject))
                            this.diagram._clearSelection();
                    }
                }
                else {
                    clickedObject = this._findNodeUnderMouse(evt);
                }
            }

            if (this.selectedObject && !this.selectedObject.isPseudoGroup) {
                var obj1 = this._findNodeUnderMouse(evt, true);
                if (!skip) {
                    var tempVal = this.selectedObject;
                    var args = this._raiseEvent("itemClick", { element: this.selectedObject, actualObject: this.actualObject, selectedObject: obj1, model: this.diagram.model, cancel: false });
                    this._itemClick(this.selectedObject, this.actualObject, obj1);
                    if (args.cancel)
                        this.selectedObject = tempVal;
                }
            }
            this._allowPan(this.selectedObject, "mouseup", evt);
            this._updateSelection();
            this.activeLabel = null;
            this.diagram._removeTooltip();
            if (!this.diagram._isEditing)
                this.diagram._endEdit();
            this.hoverNode = null;
            //raise click event
            if (clickedObject) {
                this._raiseEvent("click", { element: clickedObject });
            }
            base.prototype.mouseup.call(this, evt);
        };

        MoveTool.prototype._swapChildren = function (list, name1, name2) {
            var moveIndex = list.indexOf(name2);
            list.splice(list.indexOf(name1), 1);
            list.splice(moveIndex, 0, name1);
        };
        MoveTool.prototype._updateObject = function () {
            if (this.selectedObject.isPseudoGroup) {
                var child;
                for (var i = 0, len = this.selectedObject.children.length; i < len; i++) {
                    child = this.diagram.nameTable[this.selectedObject.children[i]];
                    ej.datavisualization.Diagram.DiagramContext.update(child, this.diagram);
                }
            }
            else
                ej.datavisualization.Diagram.DiagramContext.update(this.selectedObject, this.diagram);
        };
        MoveTool.prototype._endAction = function () {
            base.prototype._endAction.apply(this);
            this._isMouseDown = false;
            this._mouseOffset = ej.datavisualization.Diagram.Size(0, 0);
        };

        MoveTool.prototype._renderHelper = function () {
            if (!this.svgHelper) {
                if (this.helper.segments)
                    ej.datavisualization.Diagram.SvgContext.renderConnector(this.helper, this.diagram._svg, this.diagram._diagramLayer);
                else if (this.helper.isGroup)
                    ej.datavisualization.Diagram.SvgContext.renderGroup(this.helper, this.diagram._svg, this.diagram._diagramLayer);
                else
                    ej.datavisualization.Diagram.SvgContext.renderNode(this.helper, this.diagram._svg, this.diagram._diagramLayer);
                this.svgHelper = document.getElementById(this.helper.name);
                this.svgHelper.setAttribute("pointer-events", "none");
            }
        };
        MoveTool.prototype._updateHelper = function () {
            this._updateXY(this.helper, this.previousPoint, this.currentPoint);
            ej.datavisualization.Diagram.SvgContext.update(this.helper, this.diagram);
        };
        MoveTool.prototype._updateHelperXY = function (shape, startPoint, endPoint) {
            var towardsLeft = endPoint.x < startPoint.x;
            var towardsTop = endPoint.y < startPoint.y;
            var difx = this.diffx + (endPoint.x - startPoint.x);
            var dify = this.diffy + (endPoint.y - startPoint.y);
            var offset;
            if (((this.diagram.model.snapSettings.snapConstraints & ej.datavisualization.Diagram.SnapConstraints.SnapToHorizontalLines) ||
                (this.diagram.model.snapSettings.snapConstraints & ej.datavisualization.Diagram.SnapConstraints.SnapToVerticalLines)) && this.diagram._enableSnapToObject())
                offset = ej.datavisualization.Diagram.SnapUtil._snapPoint(this.diagram, this.helper, towardsLeft, towardsTop, ej.datavisualization.Diagram.Point(difx, dify),
                    endPoint, startPoint);

            if (!offset)
                offset = ej.datavisualization.Diagram.Point(difx, dify);

            this.diffx = difx - offset.x;
            this.diffy = dify - offset.y;
            if (!ej.datavisualization.Diagram.Geometry.isEmptyPoint(offset)) {
                var args = this._raiseEvent("drag", { element: this.helper, offset: offset, cancel: false });
                if (!args.cancel) {
                    this.diagram._translate(shape, offset.x, offset.y, this.diagram.nameTable);
                    this._updateParent(shape);
                    if (shape.isGroup) {
                        ej.datavisualization.Diagram.Util._updateGroupBounds(shape, this.diagram.nameTable, this.diagram);
                    }
                }
            }
        };

        MoveTool.prototype._updateXY = function (shape, startPoint, endPoint) {
            var towardsLeft = endPoint.x < startPoint.x;
            var towardsTop = endPoint.y < startPoint.y;
            var difx = this.diffx + (endPoint.x - startPoint.x);
            var dify = this.diffy + (endPoint.y - startPoint.y);
            var offset;
            if (((this.diagram.model.snapSettings.snapConstraints & ej.datavisualization.Diagram.SnapConstraints.SnapToHorizontalLines) ||
                (this.diagram.model.snapSettings.snapConstraints & ej.datavisualization.Diagram.SnapConstraints.SnapToVerticalLines)) || this.diagram._enableSnapToObject())
                offset = ej.datavisualization.Diagram.SnapUtil._snapPoint(this.diagram, this.selectedObject, towardsLeft, towardsTop, ej.datavisualization.Diagram.Point(difx, dify),
                    endPoint, startPoint);

            if (!offset)
                offset = ej.datavisualization.Diagram.Point(difx, dify);

            this.diffx = difx - offset.x;
            this.diffy = dify - offset.y;
            if (!ej.datavisualization.Diagram.Geometry.isEmptyPoint(offset)) {
                var args;
                if (!this.diagram._selectedSymbol)
                    args = this._raiseEvent("drag", { element: this.selectedObject, offset: offset, cancel: false });
                if ((args && !args.cancel) || this.diagram._selectedSymbol) {
                    var canPan = false;
                    if (shape.shape) {
                        if (ej.datavisualization.Diagram.Util.canAllowPan(shape)) {
                            if (this.diagram._hScrollOffset === 0) {
                                offset.x = 0;
                            }
                            if (this.diagram._vScrollOffset === 0) {
                                offset.y = 0;
                            }
                        }
                    }
                    this.diagram._translate(shape, offset.x, offset.y, this.diagram.nameTable);
                    this._updateParent(shape);
                    if (shape.isGroup) {
                        ej.datavisualization.Diagram.Util._updateGroupBounds(shape, this.diagram.nameTable, this.diagram);
                    }
                }
            }
        };
        MoveTool.prototype._updateLabelXY = function (label, startPoint, endPoint) {
            var bounds = ej.datavisualization.Diagram.Util.bounds(this.selectedObject);
            var difx = endPoint.x - startPoint.x;
            var dify = endPoint.y - startPoint.y;
            if (!this.selectedObject.segments) {
                var matrix = ej.Matrix.identity();
                ej.Matrix.rotate(matrix, -this.selectedObject.rotateAngle);
                var diff = ej.Matrix.transform(matrix, ej.datavisualization.Diagram.Point(difx, dify)); difx = diff.x; dify = diff.y;
            }
            label.margin["left"] += difx;
            label.margin["top"] += dify;
            label.margin["right"] -= difx;
            label.margin["bottom"] -= dify;
            var args = this._raiseEvent("drag", { element: label, offset: label.offset, cancel: false });
            if (!args.cancel) {
                ej.datavisualization.Diagram.DiagramContext.updateLabel(this.selectedObject, label, this.diagram);
            }
        };
        MoveTool.prototype._updateParent = function (shape) {
            var child;
            if (shape.isPseudoGroup) {
                for (var i = 0; i < shape.children.length; i++) {
                    child = this.diagram.nameTable[shape.children[i]];
                    if (child && child.parent) {
                        ej.datavisualization.Diagram.Util._updateGroupBounds(this.diagram.nameTable[child.parent], this.diagram.nameTable);
                        ej.datavisualization.Diagram.DiagramContext.update(this.diagram.nameTable[child.parent], this.diagram);
                        this.diagram._updateAssociatedConnectorEnds(this.diagram.nameTable[child.parent], this.diagram.nameTable);
                    }
                }
            } else {
                if (shape && shape.parent) {
                    ej.datavisualization.Diagram.Util._updateGroupBounds(this.diagram.nameTable[shape.parent], this.diagram.nameTable);
                    ej.datavisualization.Diagram.DiagramContext.update(this.diagram.nameTable[shape.parent], this.diagram);
                    this.diagram._updateAssociatedConnectorEnds(this.diagram.nameTable[shape.parent], this.diagram.nameTable);
                }
            }
        };
        MoveTool.prototype._updateSelection = function () {
            if (!this.diagram._selectionContains(this.selectedObject)) {
                this.diagram._addSelection(this.selectedObject);
            }
        };
        return MoveTool;
    })(ToolBase);

    ej.datavisualization.Diagram.MoveTool = MoveTool;
    //#endregion

    //#region ResizeTool
    var ResizeTool = (function (base) {
        ej.datavisualization.Diagram.extend(ResizeTool, base);
        function ResizeTool(diagram) {
            base.call(this, "resize", diagram);
            this.cursor = "default";//update resize cursor during action i.e in action
            this._resizeDirection = null;
            this._mouseOffset = ej.datavisualization.Diagram.Size(0, 0);
            this.diffx = 0;
            this.diffy = 0;
            this._startPoint = ej.datavisualization.Diagram.Size(0, 0);
            this.undoObject = null;
        }

        ResizeTool.prototype.mousedown = function (evt) {
            ej.datavisualization.Diagram.SnapUtil._removeGuidelines(this.diagram);
            base.prototype.mousedown.call(this, evt);
            this.selectedObject = this.diagram.nameTable[this.diagram.selectionList[0].name];
            var data;
            var childTable = {};
            if (this.selectedObject.isGroup || this.selectedObject.isPseudoGroup)
                childTable = this.diagram._getChildTable(this.selectedObject, childTable);
            var temp = { "childTable": childTable, "node": this.selectedObject };
            data = $.extend(true, {}, temp);
            this.undoObject = jQuery.extend(true, {}, data);
            //this.undoObject = ej.datavisualization.Diagram.Util.clone(this.selectedObject);
            this._resizeDirection = evt.target.getAttribute("class");
            this.initialBounds = { width: this.selectedObject.width, height: this.selectedObject.height, y: this.selectedObject.offsetY, x: this.selectedObject.offsetX };
        };
        ResizeTool.prototype._createUpdateHelper = function () {
            if (!this.helper) {
                this.helper = this._getCloneNode(this.selectedObject);
                this.helper.isGroup = null;
                this.helper.isPseudoGroup = null;
                this.helper.name = "helper";
                ej.datavisualization.Diagram.SvgContext._drawContainerHelper(this.diagram);
                this._startPoint = this.currentPoint;
                this._updateSize(this.helper, this.previousPoint, this.currentPoint, null, null, null, true);
            } else {
                var groupBounds123 = ej.datavisualization.Diagram.Util.bounds(this.helper);
                this._updateSize(this.helper, this.previousPoint, this.currentPoint, null, null, null, true);
                var groupBounds1231 = ej.datavisualization.Diagram.Util.bounds(this.helper);
                if (this.selectedObject.container)
                    this.diagram._translate(this.helper, groupBounds123.topLeft.x - groupBounds1231.topLeft.x, groupBounds123.topLeft.y - groupBounds1231.topLeft.y, this.diagram.nameTable);
                ej.datavisualization.Diagram.SvgContext._updateContainerHelper(this.diagram);
            }
        };
        ResizeTool.prototype.mousemove = function (evt) {
            base.prototype.mousemove.call(this, evt);
            ej.datavisualization.Diagram.SnapUtil._removeGuidelines(this.diagram);
            if (this.selectedObject) {
                if (!this.inAction) {
                    ej.datavisualization.Diagram.SvgContext._removePivotPoint(this.selectedObject, this.diagram._svg, this.diagram._currZoom);
                    this.inAction = true;
                }
                var canvasChild = false;
                if (this.selectedObject && this.selectedObject.parent) {
                    var parNode = this.diagram.nameTable[this.selectedObject.parent];
                    if (parNode.container && parNode.container.type == "canvas")
                        canvasChild = true;
                }
                else if (this.selectedObject.isPseudoGroup && this.hasSameParent() && this._fromContainer(this.selectedObject.children)) {
                    canvasChild = true;
                }
                if (this.selectedObject && canvasChild) {
                    this._createUpdateHelper();
                }
                else if (this.selectedObject && this.diagram.nameTable[this.selectedObject.name].container) {
                    this._createUpdateHelper();
                }
                else if (this.selectedObject.parent && this.diagram.nameTable[this.selectedObject.parent] &&
                    this.diagram.nameTable[this.selectedObject.parent].container && this.diagram.nameTable[this.selectedObject.parent].container.type == "stack") {
                    if (!this.helper) {
                        this.helper = this._getCloneNode();
                    }
                    ej.datavisualization.Diagram.SvgContext._drawContainerHelper(this.diagram);
                    this._startPoint = this.currentPoint;
                    this._updateSize(this.helper, this.previousPoint, this.currentPoint);
                    ej.datavisualization.Diagram.DiagramContext.update(this.helper, this.diagram);
                }
                else {
                    if (!this.selectedObject.isPseudoGroup)
                        this._updateSize(this.selectedObject, this.previousPoint, this.currentPoint);
                    else if (this.selectedObject.isPseudoGroup && !this._fromContainer(this.selectedObject.children) && this._fromSameContainer(this.selectedObject.children))
                        this._updateSize(this.selectedObject, this.previousPoint, this.currentPoint);
                }
                ej.datavisualization.Diagram.DiagramContext.update(this.selectedObject, this.diagram);
                this.diagram._renderTooltip(this.selectedObject);
                this.diagram._updateSelectionHandle(true);
            }

            this.previousPoint = this.currentPoint;
        };
        ResizeTool.prototype._updateResizeObject = function () {
            var object = this.diagram.nameTable[this.selectedObject.name];
            var groupBounds123 = ej.datavisualization.Diagram.Util.bounds(object);
            var grAngle = object.rotateAngle;
            this.diagram._rotate(object, -object.rotateAngle, this.diagram.nameTable);
            var dX = object.width - this.helper.width;
            var dY = object.height - this.helper.height;
            if ((!object.container && object.rotateAngle == 0)) {
                this._updateSize(object, this.startPoint, this.currentPoint);
            }
            else {
                this.diagram.updateNode(object.name, { width: this.helper.width });
                this.diagram.updateNode(object.name, { height: this.helper.height });
            }

            object.minWidth = object.width;
            object.minHeight = object.height;
            this.diagram._rotate(object, grAngle, this.diagram.nameTable);
            var groupBounds1231 = ej.datavisualization.Diagram.Util.bounds(object);
            if (!object.container && object.rotateAngle == 0) { //this.diagram._translate(object, dX / 2, dY / 2, this.diagram.nameTable);
            }
            else
                this.diagram._translate(object, groupBounds123.topLeft.x - groupBounds1231.topLeft.x, groupBounds123.topLeft.y - groupBounds1231.topLeft.y, this.diagram.nameTable);
            return object;
        };
        ResizeTool.prototype.mouseup = function (evt) {
            if (this.inAction) {
                ej.datavisualization.Diagram.SnapUtil._removeGuidelines(this.diagram);
                this.inAction = false;
                var canvasChild = false;
                if (this.selectedObject && this.selectedObject.parent) {
                    var parNode = this.diagram.nameTable[this.selectedObject.parent];
                    if (parNode.container && parNode.container.type == "canvas")
                        canvasChild = true;
                }
                else if (this.selectedObject.isPseudoGroup && this.hasSameParent() && this._fromContainer(this.selectedObject.children)) {
                    canvasChild = true;
                }
                if (this.selectedObject && canvasChild) {
                    ej.datavisualization.Diagram.SvgContext._removeContainerHelper(this.selectedObject, this.diagram._svg, this.diagram._adornerLayer);
                    var object = this._updateResizeObject();
                    this._updateContainer(object, null, true);
                }
                else if (this.selectedObject && this.diagram.nameTable[this.selectedObject.name].container) {
                    ej.datavisualization.Diagram.SvgContext._removeContainerHelper(this.selectedObject, this.diagram._svg, this.diagram._adornerLayer);
                    object = this.diagram.nameTable[this.selectedObject.name];
                    if (this.selectedObject.container) {
                        object.minWidth = this.helper.width.width;
                        object.minHeight = this.helper.width.height;
                    }
                    this._updateSize(this.selectedObject, this._startPoint, this.currentPoint, null, null, true);
                    if (this.selectedObject.parent) {//canvas panel
                        var parent = this.diagram.nameTable[this.selectedObject.parent];
                        if (parent.isGroup && parent.container && parent.container.type == "stack") {
                            ej.datavisualization.Diagram.SvgContext._removeContainerHelper(this.selectedObject, this.diagram._svg, this.diagram._adornerLayer);
                            object = this.diagram.nameTable[this.selectedObject.name];
                            var newsize = this.diagram._getMinMaxSize(this.selectedObject, this.helper.width, this.helper.height);
                            object.height = newsize.height;
                            object.width = newsize.width;
                            if (this.selectedObject.container) {
                                object.minWidth = object.width;
                                object.minHeight = object.height;
                            }
                            this._updateContainer(this.diagram.nameTable[this.selectedObject.name]);
                        }
                    }
                    else if (this.selectedObject) {//stack panel
                        this.selectedObject.minWidth = this.selectedObject.width;
                        this.selectedObject.minHeight = this.selectedObject.height;
                    }
                }
                else if (this.selectedObject.parent && this.diagram.nameTable[this.selectedObject.parent] &&
                this.diagram.nameTable[this.selectedObject.parent].container && this.diagram.nameTable[this.selectedObject.parent].container.type == "stack") {
                    ej.datavisualization.Diagram.SvgContext._removeContainerHelper(this.selectedObject, this.diagram._svg, this.diagram._adornerLayer);
                    this.selectedObject.height = this.helper.height;
                    this.selectedObject.width = this.helper.width;
                    this._updateContainer(this.selectedObject);
                }
                else {
                    this._updateSize(this.selectedObject, this.previousPoint, this.currentPoint);
                }
                if (this.selectedObject.parent) {
                    ej.datavisualization.Diagram.Util._updateGroupBounds(this.diagram.nameTable[this.selectedObject.parent], this.diagram.nameTable);
                }
                ej.datavisualization.Diagram.DiagramContext.update(this.selectedObject, this.diagram);
                this.diagram._updateSelectionHandle();

                var data;
                var childTable = {};
                if (this.selectedObject.isGroup || this.selectedObject.isPseudoGroup)
                    childTable = this.diagram._getChildTable(this.selectedObject, childTable);
                var temp = { "childTable": childTable, "node": this.selectedObject };
                data = $.extend(true, {}, temp);

                var entry = { type: "sizechanged", undoObject: this.undoObject, redoObject: jQuery.extend(true, {}, data), isMultipleNode: this.selectedObject.isGroup ? true : false };
                this.diagram.addHistoryEntry(entry);

                //var entry = new ej.datavisualization.Diagram.HistoryEntry(new ej.datavisualization.Diagram.ResizeHandleCmd(this.undoObject, ej.datavisualization.Diagram.Util.clone(this.selectedObject), this.selectedObject.isPseudoGroup ? true: false));
                //this.diagram.historyManager.addHistoryEntry(entry);
                this.diagram._removeTooltip();
            }
            base.prototype.mouseup.call(this, evt);
        };

        ResizeTool.prototype._updateContainer = function (newObj, prevParent, isDrop) {
            var bounds, groupBounds, dx = 0, dy = 0, newSize;
            if (newObj) {

                var parObj = this.diagram.nameTable[newObj.parent];
                if (!parObj && newObj.isPseudoGroup) {
                    var fNode = this.diagram.nameTable[this.selectedObject.children[0]];
                    parObj = this.diagram.nameTable[fNode.parent];
                }
                if (parObj) {
                    if (parObj.container.type == "canvas") {
                        if (isDrop) {
                            isDrop = false;
                            var group = this.diagram.nameTable[newObj.parent];

                            if (!group && newObj.isPseudoGroup) {
                                group = parObj;
                            }
                            var groupBounds123 = ej.datavisualization.Diagram.Util.bounds(group);
                            var grAngle = group.rotateAngle;
                            this.diagram._rotate(group, -group.rotateAngle, this.diagram.nameTable);
                            bounds = ej.datavisualization.Diagram.Util.bounds(newObj);
                            var bounds1 = ej.datavisualization.Diagram.Geometry.rect([bounds.topLeft, bounds.topRight, bounds.bottomRight, bounds.bottomLeft]);
                            groupBounds = ej.datavisualization.Diagram.Util.bounds(group);
                            var tightchildBounds = this.diagram._getTightChildrenBounds(group);
                            var right = true;
                            var bottom = true;
                            var bRight = bounds1.x + bounds1.width;
                            var bBottom = bounds1.y + bounds1.height;

                            if (bRight >= groupBounds.right) {
                                ///if exeeds right 
                                dx = bRight - groupBounds.right + group.paddingRight;
                                newSize = this.diagram._getMinMaxSize(group, (group.width + dx), group.height, true);
                                widDiff = newSize.width - group.width;
                                group.width = newSize.width;
                                group.offsetX += (widDiff / 2);
                                if (bRight < groupBounds.right) {
                                    right = false;
                                }
                            }
                            if (bBottom >= groupBounds.bottom) {
                                ///if exeeds bottom 
                                dy = bBottom - groupBounds.bottom + group.paddingBottom;
                                newSize = this.diagram._getMinMaxSize(group, (group.width), (group.height + dy), true);
                                heiDiff = newSize.height - group.height;
                                group.height = newSize.height;
                                group.offsetY += (heiDiff / 2);
                                bottom = true;
                                if (bBottom < groupBounds.bottom) {
                                    bottom = false;
                                }
                            }
                            if (bRight < groupBounds.right && right) { //if decreces from right
                                dx = (tightchildBounds.x + tightchildBounds.width) + group.paddingRight - groupBounds.right;
                                newSize = this.diagram._getMinMaxSize(group, (group.width + dx), group.height, true);
                                var widDiff = newSize.width - group.width;
                                group.width = newSize.width;
                                group.offsetX += (widDiff / 2);
                            }
                            if (bBottom < groupBounds.bottom && bottom) { //if decreces from bottom
                                dy = (tightchildBounds.y + tightchildBounds.height) + group.paddingBottom - groupBounds.bottom;
                                newSize = this.diagram._getMinMaxSize(group, (group.width), (group.height + dy), true);
                                var heiDiff = newSize.height - group.height;
                                group.height = newSize.height;
                                group.offsetY += (heiDiff / 2);
                            }
                            this._updateMargin(newObj, group);
                            ej.datavisualization.Diagram.SvgContext._alignOnCanvas1(group, this.diagram);
                            this.diagram._rotate(group, grAngle, this.diagram.nameTable);
                            var groupBounds1231 = ej.datavisualization.Diagram.Util.bounds(group);
                            this.diagram._translate(group, groupBounds123.topLeft.x - groupBounds1231.topLeft.x, groupBounds123.topLeft.y - groupBounds1231.topLeft.y, this.diagram.nameTable);
                            if (group.parent && group.container && group.container.type == "canvas")
                                this._updateMargin(group, this.diagram.nameTable[group.parent]);
                            ej.datavisualization.Diagram.DiagramContext.update(group, this.diagram);
                        } else {
                            var vertical = false;
                            var horizontal = false;
                            group = this.diagram.nameTable[newObj.parent];
                            if (newObj.container && newObj.container.type == "stack") {
                                if (newObj.container.orientation == "vertical")
                                    vertical = true;
                                else
                                    horizontal = true;
                            }
                            bounds = this.diagram._getBoundsWithoutStretch(group, vertical, horizontal);
                            bounds.width += group.paddingLeft + group.paddingRight;
                            bounds.height += group.paddingTop + group.paddingBottom;
                            newSize = this.diagram._getMinMaxSize(group, (bounds.width), (bounds.height), true);
                            widDif = newSize.width - group.width;
                            heiDif = newSize.height - group.height;
                            group.offsetX += (widDif / 2);
                            group.width = newSize.width;
                            group.offsetY += (heiDif / 2);
                            group.height = newSize.height;
                            ej.datavisualization.Diagram.SvgContext._alignOnCanvas1(group, this.diagram);
                            ej.datavisualization.Diagram.DiagramContext.update(group, this.diagram);
                        }
                        if (group.parent) {
                            if (this.diagram.nameTable[group.parent])
                                isDrop = true;
                            this._updateContainer(group, null, isDrop);
                        }
                    }//end canvas

                    else if (parObj.container.type == "stack") {
                        group = this.diagram.nameTable[newObj.parent];
                        bounds = ej.datavisualization.Diagram.SvgContext._measureStackSize(group, this.diagram, true);
                        newSize = this.diagram._getMinMaxSize(group, (bounds.width), (bounds.height), true);
                        var widDif = newSize.width - group.width;
                        var heiDif = newSize.height - group.height;
                        group.offsetX += (widDif / 2);
                        group.width = newSize.width;
                        group.offsetY += (heiDif / 2);
                        group.height = newSize.height;
                        ej.datavisualization.Diagram.SvgContext._alignOnStack(group, this.diagram);
                        ej.datavisualization.Diagram.DiagramContext.update(group, this.diagram);
                        if (group.parent)
                            this._updateContainer(group);

                    }
                }
            }
        };
        ResizeTool.prototype._updateSize = function (shape, startPoint, endPoint, x1, y1, updateMinMax, isHelper) {
            var deltaWidth, deltaHeight, diff;
            var matrix = ej.Matrix.identity();
            ej.Matrix.rotate(matrix, -shape.rotateAngle);
            var horizontalsnap = { snapped: false, offset: 0, left: false, right: false };
            var verticalsnap = { snapped: false, offset: 0, top: false, bottom: false };
            var difx = x1 ? x1 : (this.currentPoint.x - this.startPoint.x);
            var dify = y1 ? y1 : (this.currentPoint.y - this.startPoint.y);
            var pivot = ej.datavisualization.Diagram.Point(0, 0);
            var trans = ej.Matrix.identity();
            var x = shape.offsetX;
            var y = shape.offsetY;
            ej.Matrix.rotate(trans, shape.rotateAngle, x, y);
            var w = shape.width;
            var h = shape.height;
            x = x - w * shape.pivot.x;
            y = y - h * shape.pivot.y;
            switch (this._resizeDirection) {
                case "n-resize":
                    pivot = ej.Matrix.transform(trans, ej.datavisualization.Diagram.Point(x + w / 2, y + h));
                    var deltaWidth = 1;
                    diff = ej.Matrix.transform(matrix, ej.datavisualization.Diagram.Point(difx, dify));
                    difx = diff.x; dify = diff.y;
                    dify = this._snapTop(horizontalsnap, verticalsnap, difx, dify, shape, endPoint == startPoint);
                    deltaHeight = (this.initialBounds.height - dify) / shape.height;
                    break;
                case "e-resize":
                    pivot = ej.Matrix.transform(trans, ej.datavisualization.Diagram.Point(x, y + h / 2));
                    diff = ej.Matrix.transform(matrix, ej.datavisualization.Diagram.Point(difx, dify)); difx = diff.x; dify = diff.y;
                    difx = this._snapRight(horizontalsnap, verticalsnap, difx, dify, shape, endPoint == startPoint);
                    dify = 0;
                    deltaWidth = (this.initialBounds.width + difx) / shape.width;
                    deltaHeight = 1;
                    break;
                case "w-resize":
                    pivot = ej.Matrix.transform(trans, ej.datavisualization.Diagram.Point(x + w, y + h / 2));
                    diff = ej.Matrix.transform(matrix, ej.datavisualization.Diagram.Point(difx, dify)); difx = diff.x; dify = diff.y;
                    difx = this._snapLeft(horizontalsnap, verticalsnap, difx, dify, shape, endPoint == startPoint);
                    dify = 0;
                    deltaWidth = (this.initialBounds.width - difx) / shape.width;
                    deltaHeight = 1;
                    break;
                case "s-resize":
                    pivot = ej.Matrix.transform(trans, ej.datavisualization.Diagram.Point(x + w / 2, y));
                    deltaWidth = 1;
                    diff = ej.Matrix.transform(matrix, ej.datavisualization.Diagram.Point(difx, dify));
                    difx = diff.x; dify = diff.y;
                    dify = this._snapBottom(horizontalsnap, verticalsnap, diff.x, diff.y, shape, endPoint == startPoint);
                    deltaHeight = (this.initialBounds.height + dify) / shape.height;
                    break;
                case "ne-resize":
                    pivot = ej.Matrix.transform(trans, ej.datavisualization.Diagram.Point(x, y + h));
                    diff = ej.Matrix.transform(matrix, ej.datavisualization.Diagram.Point(difx, dify)); difx = diff.x; dify = diff.y;
                    difx = this._snapRight(horizontalsnap, verticalsnap, difx, dify, shape, endPoint == startPoint);
                    dify = this._snapTop(horizontalsnap, verticalsnap, difx, dify, shape, endPoint == startPoint);
                    deltaWidth = (this.initialBounds.width + difx) / shape.width;
                    deltaHeight = (this.initialBounds.height - dify) / shape.height;
                    break;
                case "nw-resize":
                    pivot = ej.Matrix.transform(trans, ej.datavisualization.Diagram.Point(x + w, y + h));
                    diff = ej.Matrix.transform(matrix, ej.datavisualization.Diagram.Point(difx, dify)); difx = diff.x; dify = diff.y;
                    dify = this._snapTop(horizontalsnap, verticalsnap, difx, dify, shape, endPoint == startPoint);
                    difx = this._snapLeft(horizontalsnap, verticalsnap, difx, dify, shape, endPoint == startPoint);
                    deltaWidth = (this.initialBounds.width - difx) / shape.width;
                    deltaHeight = (this.initialBounds.height - dify) / shape.height;
                    break;
                case "se-resize":
                    pivot = ej.Matrix.transform(trans, ej.datavisualization.Diagram.Point(x, y));
                    diff = ej.Matrix.transform(matrix, ej.datavisualization.Diagram.Point(difx, dify)); difx = diff.x; dify = diff.y;
                    dify = this._snapBottom(horizontalsnap, verticalsnap, difx, dify, shape, endPoint == startPoint);
                    difx = this._snapRight(horizontalsnap, verticalsnap, difx, dify, shape, endPoint == startPoint);
                    deltaWidth = (this.initialBounds.width + difx) / shape.width;
                    deltaHeight = (this.initialBounds.height + dify) / shape.height;
                    break;
                case "sw-resize":
                    pivot = ej.Matrix.transform(trans, ej.datavisualization.Diagram.Point(x + w, y));
                    diff = ej.Matrix.transform(matrix, ej.datavisualization.Diagram.Point(difx, dify)); difx = diff.x; dify = diff.y;
                    dify = this._snapBottom(horizontalsnap, verticalsnap, difx, dify, shape, endPoint == startPoint);
                    difx = this._snapLeft(horizontalsnap, verticalsnap, difx, dify, shape, endPoint == startPoint);
                    deltaWidth = (this.initialBounds.width - difx) / shape.width;
                    deltaHeight = (this.initialBounds.height + dify) / shape.height;
                    break;
            }
            var propResize;
            if (shape)
                if (shape.isPseudoGroup === true || shape.isGroup === true) {
                    var list = this.diagram.nameTable[this.diagram.selectionList[0].name].children;
                    for (var i = 0; i < list.length; i++) {
                        var element = this.diagram.nameTable[list[i]];
                        if (element && !(element.segments) && (element.constraints & ej.datavisualization.Diagram.NodeConstraints.AspectRatio)) {
                            propResize = true;
                        }
                    }
                }
            if (propResize || (shape.constraints & ej.datavisualization.Diagram.NodeConstraints.AspectRatio)) {
                if (this._resizeDirection == "n-resize" || this._resizeDirection == "s-resize" || this._resizeDirection == "w-resize" || this._resizeDirection == "e-resize") {
                    if (Math.abs(this.previousPoint.x - this.currentPoint.x) > Math.abs(this.previousPoint.y - this.currentPoint.y))
                        deltaHeight = deltaWidth;
                    else
                        deltaWidth = deltaHeight;
                }
                else if (this.previousPoint != this.currentPoint)
                        deltaHeight = deltaWidth = Math.max(deltaHeight, deltaWidth);
                else
                    deltaHeight = deltaWidth = 0;
            }
            var size = ej.datavisualization.Diagram.Size();
            size.width = deltaWidth;
            size.height = deltaHeight;
            if (!ej.datavisualization.Diagram.Geometry.isEmptySize(size)) {
                this.diagram.scale(shape, deltaWidth, deltaHeight, pivot, this.diagram.nameTable, null, updateMinMax, isHelper);
                var args = this._raiseEvent("sizeChange", { element: this.selectedObject, offset: size, cancel: false });
                if (args.cancel && !isHelper) {
                    this.diagram.scale(shape, 1 / deltaWidth, 1 / deltaHeight, pivot, this.diagram.nameTable);
                }
                if (shape.isGroup) {
                    ej.datavisualization.Diagram.Util._updateGroupBounds(shape, this.diagram.nameTable, this.diagram);
                }
            }
        };

        ResizeTool.prototype._snapTop = function (horizontalsnap, verticalsnap, delx, dely, shape, ended) {
            var dify = dely;
            verticalsnap.top = true;
            var snapSettings = this.diagram.model.snapSettings;
            var zoomFactor = this.diagram._currZoom;
            if (this.diagram._enableSnapToObject() && !shape.rotateAngle) {
                var y = this.initialBounds.y - this.initialBounds.height / 2 + dely - (shape.offsetY - shape.height / 2);
                ej.datavisualization.Diagram.SnapUtil._snapSize(this.diagram, horizontalsnap, verticalsnap, delx, y, this.selectedObject, ended);
            }
            if (!verticalsnap.snapped) {
                if (snapSettings.snapConstraints & ej.datavisualization.Diagram.SnapConstraints.SnapToHorizontalLines) {
                    var top = this.initialBounds.y - this.initialBounds.height / 2;
                    var actualTop = top + dely;
                    var roundedTop = ej.datavisualization.Diagram.SnapUtil._round(actualTop, snapSettings.horizontalGridLines.snapInterval, zoomFactor);
                    dify = roundedTop - top;
                }
            }
            else {
                dify = (dely - y) + verticalsnap.offset;
            }
            return dify;
        };
        ResizeTool.prototype._snapBottom = function (horizontalsnap, verticalsnap, delx, dely, shape, ended, matrix) {
            var dify = dely;
            verticalsnap.bottom = true;
            var snapSettings = this.diagram.model.snapSettings;
            var zoomFactor = this.diagram._currZoom;
            if (this.diagram._enableSnapToObject() && !shape.rotateAngle) {
                var y = this.initialBounds.y + this.initialBounds.height / 2 + dely - (shape.offsetY + shape.height / 2);
                ej.datavisualization.Diagram.SnapUtil._snapSize(this.diagram, horizontalsnap, verticalsnap, delx, y, this.selectedObject, ended);;
            }
            var bounds = ej.datavisualization.Diagram.Util.bounds(shape);
            if (!verticalsnap.snapped) {
                if (snapSettings.snapConstraints & ej.datavisualization.Diagram.SnapConstraints.SnapToHorizontalLines) {
                    var bottom = this.initialBounds.y + this.initialBounds.height / 2;
                    var actualBottom = bottom + dely;
                    var roundedBottom = ej.datavisualization.Diagram.SnapUtil._round(actualBottom, snapSettings.horizontalGridLines.snapInterval, zoomFactor);
                    dify = roundedBottom - bottom;
                }
            }
            else {
                dify = (dely - y) + verticalsnap.offset;
            }
            return dify;
        };
        ResizeTool.prototype._snapLeft = function (horizontalsnap, verticalsnap, delx, dely, shape, ended) {
            var difx = delx;
            horizontalsnap.left = true;
            var snapSettings = this.diagram.model.snapSettings;
            var zoomFactor = this.diagram._currZoom;
            if (this.diagram._enableSnapToObject() && !shape.rotateAngle) {
                var x = this.initialBounds.x - this.initialBounds.width / 2 + delx - (shape.offsetX - shape.width / 2);
                ej.datavisualization.Diagram.SnapUtil._snapSize(this.diagram, horizontalsnap, verticalsnap, x, dely, this.selectedObject, ended);;
            }
            var bounds = ej.datavisualization.Diagram.Util.bounds(shape);
            if (!horizontalsnap.snapped) {
                if (snapSettings.snapConstraints & ej.datavisualization.Diagram.SnapConstraints.SnapToVerticalLines) {
                    var left = this.initialBounds.x - this.initialBounds.width / 2;
                    var actualLeft = left + delx;
                    var roundedLeft = ej.datavisualization.Diagram.SnapUtil._round(actualLeft, snapSettings.horizontalGridLines.snapInterval, zoomFactor);
                    difx = roundedLeft - left;
                }
            }
            else {
                difx = (delx - x) + horizontalsnap.offset;
            }
            return difx;
        };
        ResizeTool.prototype._snapRight = function (horizontalsnap, verticalsnap, delx, dely, shape, ended) {
            var difx = delx;
            horizontalsnap.right = true;
            var snapSettings = this.diagram.model.snapSettings;
            var zoomFactor = this.diagram._currZoom;
            if (this.diagram._enableSnapToObject() && !shape.rotateAngle) {
                var x = this.initialBounds.x + this.initialBounds.width / 2 + delx - (shape.offsetX + shape.width / 2);
                ej.datavisualization.Diagram.SnapUtil._snapSize(this.diagram, horizontalsnap, verticalsnap, x, dely, this.selectedObject, ended);
            }
            if (!horizontalsnap.snapped) {
                if (snapSettings.snapConstraints & ej.datavisualization.Diagram.SnapConstraints.SnapToVerticalLines) {
                    var right = this.initialBounds.x + this.initialBounds.width / 2;
                    var actualRight = right + delx;
                    var roundedRight = ej.datavisualization.Diagram.SnapUtil._round(actualRight, snapSettings.verticalGridLines.snapInterval, zoomFactor);
                    difx = roundedRight - right;
                }
            }
            else {
                difx = (delx - x) + horizontalsnap.offset;
            }
            return difx;
        };
        return ResizeTool;
    })(ToolBase);

    ej.datavisualization.Diagram.ResizeTool = ResizeTool;
    //#endregion

    //#region EndPointTool
    var ConnectionEditTool = (function (base) {
        ej.datavisualization.Diagram.extend(ConnectionEditTool, base);
        function ConnectionEditTool(diagram) {
            base.call(this, "endPoint", diagram);
            this.cursor = "move";
            this._endPoint = null;
            this._undoObject = null;
        }
        ConnectionEditTool.prototype.mousedown = function (evt) {
            base.prototype.mousedown.call(this, evt);
            if (evt.ctrlKey && evt.shiftKey) {
                var parent = $(evt.target).parents(".ej-d-connector");
                if (parent.length > 0) {
                    this.selectedObject = this.diagram.nameTable[parent[0].id];
                    if (this.diagram._hasSelection())
                        this.diagram._clearSelection();
                    this.diagram._addSelection(this.selectedObject);
                }
            }
            if (this.diagram.selectionList[0] && !this.selectedObject) this.selectedObject = this.diagram.nameTable[this.diagram.selectionList[0].name];
            this.undoObject = this._getClonedObject(this.selectedObject);
            this._endPoint = evt.target.getAttribute("class");
            this.targetid = evt.target.id;
            this._selectedSegment = "";
        };
        ConnectionEditTool.prototype.mousemove = function (evt) {
            base.prototype.mousemove.call(this, evt);
            if (this.selectedObject && !(evt.ctrlKey && evt.shiftKey)) {
                var isTwo = false;
                if (this.selectedObject.segments.length == 2) isTwo = true;
                var difx = this.currentPoint.x - this.previousPoint.x;
                var dify = this.currentPoint.y - this.previousPoint.y;
                if (!this.inAction && (this.startPoint.x != this.currentPoint.x || this.startPoint.y != this.currentPoint.y)) {
                    this.inAction = true;
                    if (this.selectedObject.segments.length > 0) {
                        for (var m = 0; m < this.selectedObject.segments.length; m++) {
                            this.selectedObject.segments[m]._bridges = [];
                        }
                    }
                    (this.diagram._svg.getElementById(this.selectedObject.name + "_hitTest")).style["pointer-events"] = "none";
                    (this.diagram._svg.getElementById(this.selectedObject.name)).style["pointer-events"] = "none";
                    (this.diagram._svg.getElementById(this.diagram._svg.document.id + "handle_g")).style["pointer-events"] = "none";
                    var coll = this.targetid.split("_");
                    coll.reverse();
                    var index = Number(coll[0]);
                    this.selectedSegment = this.selectedObject.segments[index];
                    var segment = this.selectedSegment;
                    if (this.selectedSegment && this.selectedSegment.type == "orthogonal") {
                        var refresh = false;
                        if (index === 0) {
                            var editfirstsegment = true;
                        }
                        if (!this.selectedSegment.length) {
                            this._addTerminalSegment(difx, dify, coll);
                            refresh = true;
                        }
                        else if (index == 0) {
                            this.insertFirstSegment(difx, dify, coll);
                            refresh = true;
                        }
                        else {
                            this.nextSegment = this.selectedObject.segments[index + 1];
                        }
                    }
                    else
                        this.nextSegment = this.selectedObject.segments[index + 1];
                    if (refresh) {
                        ej.datavisualization.Diagram.Util._updateConnectorSegments(this.selectedObject, !(isTwo || editfirstsegment || !this.selectedObject.sourcePort), !(isTwo || !(segment.length || segment.length === 0) || !this.selectedObject.targetPort), this.diagram);
                        ej.datavisualization.Diagram.DiagramContext._refreshSegments(this.selectedObject, this.diagram);
                        ej.datavisualization.Diagram.SvgContext._refreshEndPointHandles(this.diagram.selectionList[0], this.diagram._svg, this.diagram._currZoom);
                    }
                    if (!this.selectedSegment && this._endPoint == "targetEndPoint") this.selectedSegment = this.selectedObject.segments[this.selectedObject.segments.length - 1];
                    if (!this.selectedSegment && this._endPoint == "sourceEndPoint") {
                        this.selectedSegment = this.selectedObject.segments[0];
                        this.nextSegment = this.selectedObject.segments[1];
                    }
                    this.undoObject = this._getClonedObject(this.selectedObject);
                }
                else if (this.inAction) {
                    if (this.selectedSegment && this._endPoint == "segmentEnd") {
                        this._updateAdjacentSegments(difx, dify);
                    }
                    else if (this._endPoint != "segmentEnd") {
                        if (this._endPoint == "targetEndPoint" || this._endPoint == "sourceEndPoint")
                            this._checkConnectionPossible(evt);
                        var updateSelection = this._updatePoints(this.selectedObject);
                        this._disconnect(this.selectedObject);
                        this._updateConnection(this.selectedObject, updateSelection);

                    }
                    ej.datavisualization.Diagram.DiagramContext.update(this.selectedObject, this.diagram);
                    this.diagram._updateSelectionHandle(true);
                }
            }
            this.previousPoint = this.currentPoint;
        };
        ConnectionEditTool.prototype._updateAdjacentSegments = function (difx, dify) {
            var current = this.selectedSegment;
            var pt = { x: difx + current._endPoint.x, y: dify + current._endPoint.y };
            if (current.type == "orthogonal") {
                if (current._direction == "left" || current._direction == "right") {
                    current._startPoint.y += dify;
                    current._endPoint.y = current._startPoint.y;
                }
                else {
                    current._startPoint.x += difx;
                    current._endPoint.x = current._startPoint.x;
                }
                current.points[0] = current._startPoint;
                current.points[current.points.length - 1] = current._endPoint;
                var index = this.selectedObject.segments.indexOf(this.selectedSegment);
                var prev = this.selectedObject.segments[index - 1];
                if (prev) this.updatePreviousSegment(current, prev, difx, dify);
            }
            else if (current.type == "straight") {
                current.points[current.points.length - 1] = current._endPoint = pt;
                current._point = current.point = pt;
            }
            else {
                current.points[current.points.length - 1] = current._endPoint = pt;
                current._point = current.point = pt;
                ej.datavisualization.Diagram.Util._updateBezierPoints(current);
            }
            if (this.nextSegment)
                this.updateNextSegment(current, this.nextSegment, difx, dify);
        }
        ConnectionEditTool.prototype._addTerminalSegment = function (difx, dify, coll) {
            var segment = this.selectedSegment;
            this.selectedObject.segments.pop();
            var first = this.selectedObject.segments[this.selectedObject.segments.length - 1] && this.selectedObject.segments[this.selectedObject.segments.length - 1].type == "orthogonal" ? this.selectedObject.segments[this.selectedObject.segments.length - 1] : null;
            for (var i = 0; i < segment.points.length - 2; i++) {
                var seg = ej.datavisualization.Diagram.Segment({
                    type: "orthogonal",
                    length: ej.datavisualization.Diagram.Geometry.distance(segment.points[i], segment.points[i + 1]),
                    direction: ej.datavisualization.Diagram.Util._getBezierDirection(segment.points[i], segment.points[i + 1])
                });
                if (seg.length == 0) {
                    if (first.direction == "top" || first.direction == "bottom") {
                        seg._direction = seg.direction = difx > 0 ? "right" : "left";
                    }
                    else {
                        seg._direction = seg.direction = dify > 0 ? "bottom" : "top";
                    }
                }
                first = seg;
                this.selectedObject.segments.push(seg);
            }
            var sec = Number(coll[1]);
            if (segment.points.length == 2 || sec == segment.points.length - 2) {
                if (first) {
                    first.length += 5;
                    first._length = first.length;
                }
                if (sec != 0) {
                    var length = ej.datavisualization.Diagram.Geometry.distance(segment.points[segment.points.length - 2], segment._endPoint);
                    var direction = ej.datavisualization.Diagram.Util._getBezierDirection(segment.points[segment.points.length - 2], segment._endPoint);
                    var newsegment = ej.datavisualization.Diagram.Segment({ type: "orthogonal", length: 2 * length / 3, direction: direction });
                    this.selectedObject.segments.push(newsegment);

                }
            }
            var lastseg = ej.datavisualization.Diagram.Segment({ type: "orthogonal" });
            this.selectedObject.segments.push(lastseg);
            var index = Number(coll[0]) + (Number(coll[1]) || 0);
            this.selectedSegment = this.selectedObject.segments[index];
            this.nextSegment = this.selectedObject.segments[index + 1];
            if (index == this.selectedObject.segments.length - 2) {
                if (Number(coll[0]) == 0) {
                    this.nextSegment._direction = ej.datavisualization.Diagram.Util._getBezierDirection(segment.points[segment.points.length - 2], segment._endPoint);;
                }
                else
                    this.nextSegment._direction = segment._direction;
            }
            else {
                lastseg._direction = ej.datavisualization.Diagram.Util._getBezierDirection(segment.points[segment.points.length - 2], segment._endPoint);;
            }
            if (index == 0) this.insertFirstSegment(difx, dify, coll);

        },
        ConnectionEditTool.prototype.insertFirstSegment = function (difx, dify, coll) {
            var segment = this.selectedSegment;
            if (this.selectedObject.sourcePort && segment.length && this.selectedObject.segments[0].points.length > 2) {
                this.selectedObject.segments.splice(0, 1);
                var segments = [];
                var prev;
                for (var i = 0; i < segment.points.length - 1; i++) {
                    var first = ej.datavisualization.Diagram.Segment({
                        type: "orthogonal",
                        length: ej.datavisualization.Diagram.Geometry.distance(segment.points[i], segment.points[i + 1]),
                        direction: ej.datavisualization.Diagram.Util._getBezierDirection(segment.points[i], segment.points[i + 1])
                    });
                    if (first.length == 0) {
                        if (prev.direction == "top" || prev.direction == "bottom") {
                            first._direction = first.direction = difx > 0 ? "right" : "left";
                        }
                        else {
                            first._direction = first.direction = dify > 0 ? "bottom" : "top";
                        }
                    }
                    prev = first;
                    segments.push(first);
                }
                this.selectedObject.segments = segments.concat(this.selectedObject.segments);
                this.selectedSegment = this.selectedObject.segments[Number(coll[1])];
                this.nextSegment = this.selectedObject.segments[Number(coll[1]) + 1];
            }
            else {
                var segment = this.selectedObject.segments[0];
                var segments = [];
                segments.push(ej.datavisualization.Diagram.Segment({ type: "orthogonal", "direction": segment.direction, length: segment.length / 3 }));
                if (segment.direction == "bottom" || segment.direction == "top") {
                    var length = Math.max(5, Math.abs(difx));
                    var direction = difx > 0 ? "right" : "left";
                }
                else {
                    var length = Math.max(5, Math.abs(dify));
                    var direction = dify > 0 ? "bottom" : "top";
                }
                segments.push(ej.datavisualization.Diagram.Segment({ type: "orthogonal", "direction": direction, length: length }));
                var nextseg = this.selectedObject.segments[1];
                if (nextseg && nextseg.length) {
                    if (direction != nextseg.direction) {
                        length *= -1;
                    }
                    nextseg.length -= length;
                    nextseg._length = nextseg.length;
                }
                segment.length = segment._length = 2 * segment._length / 3;
                this.selectedObject.segments = segments.concat(this.selectedObject.segments);
                var index = Number(coll[0]) + Number(coll[1]);
                this.selectedSegment = this.selectedObject.segments[2];
                this.nextSegment = this.selectedObject.segments[3];
            }
        },
        ConnectionEditTool.prototype.updatePreviousSegment = function (current, prev, difx, dify) {
            prev._endPoint = prev.points[prev.points.length - 1] = current._startPoint;
            var direction = current.type == "orthogonal" && !current.direction ? ej.datavisualization.Diagram.Util._getBezierDirection(current._startPoint, current._endPoint) : current.direction;
            if (prev.type == "orthogonal") {
                if (direction == "left" || direction == "right") {
                    if (prev.direction == "bottom") {
                        prev.length += dify;
                        if (prev.length < 0) {
                            prev._direction = prev.direction = "top";
                            prev.length *= -1;
                        }
                    }
                    else {
                        prev.length -= dify;
                        if (prev.length < 0) {
                            prev._direction = prev.direction = "bottom";
                            prev.length *= -1;
                        }
                    }
                    prev._length = prev.length;
                }
                else {
                    if (prev) {
                        if (prev.direction == "right") {
                            prev.length += difx
                            if (prev.length < 0) {
                                prev._direction = prev.direction = "left";
                                prev.length *= -1;
                            }
                        }
                        else {
                            prev.length -= difx;
                            if (prev.length < 0) {
                                prev._direction = prev.direction = "right";
                                prev.length *= -1;
                            }
                        }
                        prev._length = prev.length;
                    }
                }
            }
            else {
                if (prev.type == "bezier") {
                    ej.datavisualization.Diagram.Util._updateBezierPoints(prev);
                }
            }

        };
        ConnectionEditTool.prototype.updateNextSegment = function (current, next, difx, dify) {
            var pt = current._endPoint;
            next._startPoint = next.points[0] = this.selectedSegment._endPoint;
            if (next) {
                if (next.type == "orthogonal") {
                    if (next.length || next.length === 0) {
                        if (next._direction == "left" || next._direction == "right") {
                            if (difx != 0) {
                                if (next._direction == "right") {
                                    next._length -= difx;
                                }
                                else {
                                    next._length += difx;
                                }

                                if (next.length || next.length === 0) {
                                    next.length = next._length;
                                    if (next.length < 0) {
                                        if (next.direction == "left") next.direction = "right";
                                        else if (next.direction == "right") next.direction = "left";
                                        next.length *= -1;
                                        next._direction = next.direction;
                                        next._length = next.length;
                                    }
                                }

                            }
                            if (dify != 0) {
                                if (current.type != "orthogonal") {
                                    next._startPoint.y = next.points[0].y = next._endPoint.y = next.points[next.points.length - 1].y = pt.y;
                                    var index = this.selectedObject.segments.indexOf(next);
                                    var segment = this.selectedObject.segments[index + 1];
                                    segment._startPoint.y = segment.points[0].y = next._endPoint.y;
                                    if (segment.length) {
                                        if (segment.direction == "bottom") {
                                            segment.length -= dify;
                                            if (segment.length < 0) {
                                                segment.direction = "top";
                                                segment.length *= -1;
                                            }
                                        }
                                        else {
                                            segment.length += dify;
                                            if (segment.length < 0) {
                                                segment.direction = "bottom";
                                                segment.length *= -1;
                                            }
                                        }
                                    }
                                }
                            }

                        }
                        else {
                            if (dify != 0) {
                                if (next._direction == "bottom") {
                                    next._length -= dify;
                                }
                                else {
                                    next._length += dify;
                                }

                                if (next.length || next.length === 0) {
                                    next.length = next._length;
                                    if (next.length < 0) {
                                        if (next.direction == "top") next.direction = "bottom";
                                        else if (next.direction == "bottom") next.direction = "top";
                                        next.length *= -1;
                                        next._length = next.length;
                                        next._direction = next.direction;
                                    }
                                }

                            }
                            if (difx != 0) {
                                if (current.type != "orthogonal") {
                                    next._startPoint.x = next.points[0].x = next._endPoint.x = next.points[next.points.length - 1].x = pt.x;
                                    var index = this.selectedObject.segments.indexOf(next);
                                    var segment = this.selectedObject.segments[index + 1];
                                    if (segment.length) {
                                        if (segment.direction == "right") {
                                            segment.length += difx;
                                            if (segment.length < 0) {
                                                segment.direction = "left";
                                                segment.length *= -1;
                                            }
                                        }
                                        else {
                                            segment.length -= difx;
                                            if (segment.length < 0) {
                                                segment.direction = "right";
                                                segment.length *= -1;
                                            }
                                        }
                                    }
                                }
                            }
                        }

                    }
                }
                else {

                    if (next.type == "bezier")
                        ej.datavisualization.Diagram.Util._updateBezierPoints(next);
                }
            }
        };
        ConnectionEditTool.prototype.mouseup = function (evt) {
            if (evt.shiftKey && evt.ctrlKey) {
                this._addOrRemoveStraightSegments(evt);
            }
            else
                if (this.inAction) {
                    this.inAction = false;
                    if (this._endPoint == "segmentEnd" && this.selectedSegment.type == "orthogonal") {
                        var index = this.selectedObject.segments.indexOf(this.selectedSegment);
                        var prev = this.selectedObject.segments[index - 1];
                        var next = this.selectedObject.segments[index + 1];
                        var refresh = false;
                        if (prev && prev.type == "orthogonal" && Math.abs(prev.length) < 5) {
                            if (index != 1) {
                                refresh = this._removePrevSegment(index);
                            }
                        }
                        else if (next && next.type == "orthogonal") {
                            var length = (next.length || next.length === 0) ? next.length : ej.datavisualization.Diagram.Geometry.distance(next.points[0], next.points[1]);
                            if (Math.abs(length <= 5)) {
                                refresh = this._removeNextSegment(index);
                            }
                        }
                        if (refresh) {
                            ej.datavisualization.Diagram.Util._updateConnectorSegments(this.selectedObject, this.selectedObject.sourcePort, this.selectedObject.targetPort, this.diagram);
                            ej.datavisualization.Diagram.DiagramContext._refreshSegments(this.selectedObject, this.diagram);
                            ej.datavisualization.Diagram.SvgContext._refreshEndPointHandles(this.diagram.selectionList[0], this.diagram._svg, this.diagram._currZoom);
                        }
                    }
                    else if (this._endPoint != "segmentEnd") {
                        this._updatePoints(this.selectedObject);
                        this._disconnect(this.selectedObject);
                        var args;
                        if (this._currentPossibleConnection && !this.inAction) {
                            this._updateConnection(this.selectedObject);
                        }
                        else {
                            if (this.selectedObject.sourceNode || this.selectedObject.targetNode) {
                                this._updateConnection(this.selectedObject);
                            }
                        }
                        ej.datavisualization.Diagram.DiagramContext.update(this.selectedObject, this.diagram);
                        if (this.selectedObject.parent)
                            ej.datavisualization.Diagram.Util._updateGroupBounds(this.diagram.nameTable[this.selectedObject.parent], this.diagram.nameTable);
                    }

                    ej.datavisualization.Diagram.Util.updateBridging(this.selectedObject, this.diagram);
                    ej.datavisualization.Diagram.SvgContext.update(this.selectedObject, this.diagram);
                    this.diagram._updateConnectorBridging(this.selectedObject);
                    this.diagram._updateSelectionHandle();

                    if (this.selectedObject) {
                        (this.diagram._svg.getElementById(this.selectedObject.name + "_hitTest")).style["pointer-events"] = "";
                        (this.diagram._svg.getElementById(this.selectedObject.name)).style["pointer-events"] = "";
                    }
                    (this.diagram._svg.getElementById(this.diagram._svg.document.id + "handle_g")).style["pointer-events"] = "";
                }
            if (this.selectedObject) {
                var entry = { type: "endpointchanged", undoObject: this.undoObject, redoObject: this._getClonedObject(this.selectedObject), isMultipleNode: this.selectedObject.isGroup ? true : false };
                this.diagram.addHistoryEntry(entry);
            }
            base.prototype.mouseup.call(this, evt);
        };
        ConnectionEditTool.prototype._addOrRemoveStraightSegments = function (evt) {
            if (this._endPoint == "segmentEnd") {
                var targetid = evt.target.id;
                var coll = targetid.split("_");
                coll.reverse();
                var index = Number(coll[0]);
                var segment = this.selectedObject.segments[index];
                if (segment && segment.type == "straight") {
                    var prevseg = this.selectedObject.segments[index + 1];
                    if (prevseg) {
                        this.selectedObject.segments.splice(index, 1);
                        prevseg._startPoint = prevseg.points[0] = segment._startPoint;
                        update = true;
                    }
                }
            }
            else {
                var className = evt.target.getAttribute("class");
                var targetid = evt.target.id;
                var update = false;
                var index = this._findIndex();
                if (this.selectedObject.segments && this.selectedObject.segments[index] && this.selectedObject.segments[index].type == "straight") {
                    var segment = this.selectedObject.segments[index];
                    var newseg = ej.datavisualization.Diagram.Segment({ type: "straight", point: segment._endPoint });
                    this.selectedObject.segments.splice(index + 1, 0, newseg);
                    segment._point = segment.point = segment.points[1] = segment._endPoint = this.currentPoint;
                    newseg.points[0] = newseg._startPoint = this.currentPoint;
                    newseg.points[1] = newseg._endPoint = newseg.point;
                    update = true;
                }
            }
            if (update) {
                ej.datavisualization.Diagram.Util._updateConnectorSegments(this.selectedObject, this.selectedObject.sourcePort, this.selectedObject.targetPort, this.diagram);
                ej.datavisualization.Diagram.Util.updateBridging(this.selectedObject, this.diagram);
                ej.datavisualization.Diagram.DiagramContext._refreshSegments(this.selectedObject, this.diagram);
                //ej.datavisualization.Diagram.SvgContext.clearSelector(this.diagram._svg, this.diagram._adornerLayer);
                ej.datavisualization.Diagram.SvgContext._refreshEndPointHandles(this.diagram.selectionList[0], this.diagram._svg, this.diagram._currZoom);
            }
        };
        ConnectionEditTool.prototype._findIndex = function () {
            var intersectingSegs = [];
            for (var i = 0; i < this.selectedObject.segments.length; i++) {
                var segment = this.selectedObject.segments[i];
                var rect;
                rect = {
                    x: Math.min(segment._startPoint.x, segment._endPoint.x), y: Math.min(segment._startPoint.y, segment._endPoint.y),
                    width: Math.abs(segment._startPoint.x - segment._endPoint.x), height: Math.abs(segment._startPoint.y - segment._endPoint.y)
                };
                ej.datavisualization.Diagram.Geometry.inflate(rect, this.selectedObject.lineHitPadding / 2, this.selectedObject.lineHitPadding / 2);
                if (ej.datavisualization.Diagram.Geometry.containsPoint(rect, this.currentPoint)) {
                    intersectingSegs.push(segment);
                }
            }
            if (intersectingSegs.length == 1) {
                return this.selectedObject.segments.indexOf(intersectingSegs[0]);
            }
            else {
                var ratio, min, index, seg, v, h;
                for (var i = 0; i < intersectingSegs.length; i++) {
                    seg = intersectingSegs[i];
                    var point = this.currentPoint;
                    v = (point.y - seg._startPoint.y) / (seg._endPoint.y - point.y);
                    h = (point.x - seg._startPoint.x) / (seg._endPoint.x - point.x);
                    ratio = Math.abs(v - h);
                    if (i == 0) { min = ratio; index = 0; }
                    if (ratio < min) { min = ratio; index = i; }
                }
                return this.selectedObject.segments.indexOf(intersectingSegs[index]);
            }
        }
        ConnectionEditTool.prototype._removeNextSegment = function (index) {
            var first = this.selectedObject.segments[index - 1];
            var last = this.selectedObject.segments[index + 2];
            var next = this.selectedObject.segments[index + 1];
            if (next.length || next.length === 0) {
                this.selectedObject.segments.splice(index, 2);
                if (this.selectedSegment.direction == "top" || this.selectedSegment.direction == "bottom") {
                    last._startPoint.y = this.selectedSegment._startPoint.y;
                    first._endPoint.x = last._startPoint.x;
                }
                else {
                    last._startPoint.x = this.selectedSegment._startPoint.x;
                    first._endPoint.y = last._startPoint.y;
                }
            }
            else {
                this.selectedObject.segments.splice(index + 1, 1);
                if (this.selectedSegment.direction == "top" || this.selectedSegment.direction == "bottom") {
                    first._endPoint.x = next._endPoint.x;
                }
                else {
                    first._endPoint.y = next._endPoint.y;
                }
                first._length = first.length = ej.datavisualization.Diagram.Geometry.distance(first._startPoint, first._endPoint);
                if (first.length)
                    first._direction = first.direction = ej.datavisualization.Diagram.Util._getBezierDirection(first._startPoint, first._endPoint);
                this.selectedSegment.length = this.selectedSegment._length = this.selectedSegment.direction = null;
            }

            if (first && last) {
                first._length = first.length = ej.datavisualization.Diagram.Geometry.distance(first._startPoint, last._startPoint);
                if (first.length)
                    first._direction = first.direction = ej.datavisualization.Diagram.Util._getBezierDirection(first._startPoint, last._startPoint);
                if (last.length || last.length === 0) {
                    last._length = last.length = ej.datavisualization.Diagram.Geometry.distance(first._endPoint, last._endPoint);
                    if (last.length)
                        last._direction = last.direction = ej.datavisualization.Diagram.Util._getBezierDirection(first._endPoint, last._endPoint);
                }
            }
            return true;
        },
        ConnectionEditTool.prototype._removePrevSegment = function (index) {
            var first = this.selectedObject.segments[index - 2];
            var next = this.selectedObject.segments[index + 1];
            var length = (next.length || next.length === 0) ? next.length : ej.datavisualization.Diagram.Geometry.distance(next.points[0], next.points[1]);
            if (length <= 5) {
                var end = this.selectedObject.segments[index + 2];
                if (next.length === length)
                    this.selectedObject.segments.splice(index - 1, 4);
                else
                    this.selectedObject.segments.splice(index - 1, 3);
                if (end) {
                    if (first.direction == "top" || first.direction == "bottom") {
                        first._endPoint.y = end._endPoint.y;
                        end._startPoint.x = first._endPoint.x;
                    }
                    else {
                        first._endPoint.x = end._endPoint.x;
                        end._startPoint.y = first._endPoint.y;
                    }
                    if (first.length || first.length === 0) {
                        first._length = first.length = ej.datavisualization.Diagram.Geometry.distance(first._startPoint, first._endPoint);
                        if (first.length)
                            first._direction = first.direction = ej.datavisualization.Diagram.Util._getBezierDirection(first._startPoint, first._endPoint);
                    }
                    if (end.length || end.length === 0) {
                        end._length = end.length = ej.datavisualization.Diagram.Geometry.distance(end._startPoint, end._endPoint);
                        if (end._length)
                            end._direction = end.direction = ej.datavisualization.Diagram.Util._getBezierDirection(end._startPoint, end._endPoint);
                    }
                }
                else {
                    first._endPoint = this.selectedObject.endPoint;
                    first.direction = first.length = first._length = null;
                    first._direction = next._direction;
                }
            }
            else {

                var last = this.selectedObject.segments[index + 1];
                this.selectedObject.segments.splice(index - 1, 2);
                var segment = this.selectedSegment;
                if (segment.direction == "left" || segment.direction == "right") {
                    first._endPoint.x = last._startPoint.x;
                    last._startPoint.y = first._endPoint.y;
                }
                else {
                    first._endPoint.y = last._startPoint.y;
                    last._startPoint.x = first._endPoint.x;
                }
                if (this.selectedSegment.length || this.selectedSegment.length === 0) {
                    first._length = first.length = ej.datavisualization.Diagram.Geometry.distance(first._startPoint, first._endPoint);
                    if (first._length)
                        first._direction = first.direction = ej.datavisualization.Diagram.Util._getBezierDirection(first._startPoint, first._endPoint);
                }
                if (last.length || last.length === 0) {
                    last._length = last.length = ej.datavisualization.Diagram.Geometry.distance(last._startPoint, last._endPoint);
                    if (last.length)
                        last._direction = last.direction = ej.datavisualization.Diagram.Util._getBezierDirection(last._startPoint, last._endPoint);
                }
            }

            return true;
        },
        ConnectionEditTool.prototype._getClonedObject = function (connector) {
            if (connector) {
                var clonedObject = $.extend(true, {}, connector);
                clonedObject.targetNode = connector.targetNode;
                clonedObject.sourceNode = connector.sourceNode;
                clonedObject.targetPort = connector.targetPort;
                clonedObject.sourcePort = connector.sourcePort;
                return clonedObject;
            }
        };
        ConnectionEditTool.prototype._updateConnection = function (connector, updateSelection) {
            var connectionChange;
            if (this._endPoint == "targetEndPoint") {
                if (this._currentPossibleConnection) {
                    if (this._possibleConnectionPort && ej.datavisualization.Diagram.Util.canConnect(this._possibleConnectionPort, true)) {
                        if (connector.targetNode != this._currentPossibleConnection.name || connector.targetPort != this._possibleConnectionPort.name) {
                            connectionChange = this._raiseConnectionChangeEvent(connector, true);
                        }
                    }
                    else if (ej.datavisualization.Diagram.Util.canConnect(this._currentPossibleConnection)) {
                        if (connector.targetNode != this._currentPossibleConnection.name || connector.targetPort) {
                            connectionChange = this._raiseConnectionChangeEvent(connector, true);
                        }
                    }
                }
                else {
                    if (this.selectedObject.targetNode) {
                        connectionChange = this._raiseConnectionChangeEvent(connector, true);
                    }
                }
                if (connectionChange) {
                    ej.datavisualization.Diagram.Util._updateConnectorSegments(this.selectedObject, this.selectedObject.sourcePort, this.selectedObject.targetPort, this.diagram);
                    ej.datavisualization.Diagram.DiagramContext._refreshSegments(this.selectedObject, this.diagram);
                    if (connector.segments.length > 2 || connector.segments[0].points.length == 2)
                        ej.datavisualization.Diagram.Util._updatePreviousSegment(connector);
                    ej.datavisualization.Diagram.SvgContext._refreshEndPointHandles(this.diagram.selectionList[0], this.diagram._svg, this.diagram.model.zoomFactor);
                }
            }
            else if (this._endPoint == "sourceEndPoint") {
                if (this._currentPossibleConnection) {
                    if (this._possibleConnectionPort && ej.datavisualization.Diagram.Util.canConnect(this._possibleConnectionPort, true)) {
                        if (connector.sourceNode != this._currentPossibleConnection.name || connector.sourcePort != this._possibleConnectionPort.name) {
                            connectionChange = this._raiseConnectionChangeEvent(connector);
                        }
                    }
                    else if (ej.datavisualization.Diagram.Util.canConnect(this._currentPossibleConnection)) {
                        if (connector.sourceNode != this._currentPossibleConnection.name || connector.sourcePort) {
                            connectionChange = this._raiseConnectionChangeEvent(connector);
                        }
                    }

                }
                else {
                    if (this.selectedObject.sourceNode) {
                        connectionChange = this._raiseConnectionChangeEvent(connector);
                        //if (connectionChange)
                        {
                            ej.datavisualization.Diagram.Util._updateConnectorSegments(this.selectedObject, this.selectedObject.sourcePort, this.selectedObject.targetPort, this.diagram);
                            ej.datavisualization.Diagram.DiagramContext._refreshSegments(this.selectedObject, this.diagram);

                        }
                    }
                    if (connectionChange) {
                        ej.datavisualization.Diagram.SvgContext._refreshEndPointHandles(this.diagram.selectionList[0], this.diagram._svg, this.diagram.model.zoomFactor);
                    }
                }

            }

            var endPoint = this.selectedSegment._endPoint;
            this.diagram._dock(connector, this.diagram.nameTable, updateSelection);
            if (connectionChange && this._endPoint == "sourceEndPoint" && this.selectedObject.segments.length > 1) {
                if (this.selectedSegment.points.length == 2 && !this.selectedObject.sourcePort && this.selectedSegment.length) {
                    if (this.selectedSegment.direction == "right" || this.selectedSegment.direction == "left") {
                        if (this.selectedSegment._startPoint.x < endPoint.x) {
                            this.selectedSegment.direction = this.selectedSegment._direction = "right";
                        }
                        else this.selectedSegment.direction = this.selectedSegment._direction = "left";
                        this.selectedSegment.length = this.selectedSegment._length = Math.abs(this.selectedSegment._startPoint.x - endPoint.x);
                    }
                    else {
                        if (this.selectedSegment._startPoint.y < endPoint.y) {
                            this.selectedSegment.direction = this.selectedSegment._direction = "bottom";
                        }
                        else this.selectedSegment.direction = this.selectedSegment._direction = "top";
                        this.selectedSegment.length = this.selectedSegment._length = Math.abs(this.selectedSegment._startPoint.y - endPoint.y);

                    }
                    ej.datavisualization.Diagram.Util._updateConnectorSegments(this.selectedObject, this.selectedObject.sourcePort, this.selectedObject.targetPort, this.diagram);
                    ej.datavisualization.Diagram.DiagramContext._refreshSegments(this.selectedObject, this.diagram);
                    ej.datavisualization.Diagram.SvgContext._refreshEndPointHandles(this.diagram.selectionList[0], this.diagram._svg, this.diagram.model.zoomFactor);
                }
                else {
                    if (connectionChange) {
                        ej.datavisualization.Diagram.Util._updateConnectorSegments(this.selectedObject, this.selectedObject.sourcePort, this.selectedObject.targetPort, this.diagram);
                        ej.datavisualization.Diagram.DiagramContext._refreshSegments(this.selectedObject, this.diagram);
                        ej.datavisualization.Diagram.SvgContext._refreshEndPointHandles(this.diagram.selectionList[0], this.diagram._svg, this.diagram.model.zoomFactor);
                    }
                }
            }
        };
        ConnectionEditTool.prototype._raiseConnectionChangeEvent = function (connector, isTarget) {
            var args = this._raiseEvent("connectionChange", { element: this.selectedObject, endPoint: this._endPoint, connection: this._currentPossibleConnection, port: this._possibleConnectionPort, cancel: false });
            if (!args.cancel) {
                if (isTarget) {
                    if (!this._currentPossibleConnection || connector.targetNode != this._currentPossibleConnection.name) {
                        if (this._currentPossibleConnection)
                            this._currentPossibleConnection.inEdges.push(connector.name);
                        if (connector.targetNode) {
                            var node = this.diagram.nameTable[this.selectedObject.targetNode];
                            ej.datavisualization.Diagram.Util.removeItem(node.inEdges, connector.name);
                        }
                    }
                    connector.targetNode = this._currentPossibleConnection ? this._currentPossibleConnection.name : null;
                    connector.targetPort = this._possibleConnectionPort ? this._possibleConnectionPort.name : null;
                }
                else {
                    if (!this._currentPossibleConnection || connector.sourceNode != this._currentPossibleConnection.name) {
                        if (this._currentPossibleConnection)
                            this._currentPossibleConnection.outEdges.push(connector.name);
                        if (connector.sourceNode) {
                            var node = this.diagram.nameTable[this.selectedObject.sourceNode];
                            ej.datavisualization.Diagram.Util.removeItem(node.outEdges, connector.name);
                        }
                    }
                    connector.sourceNode = this._currentPossibleConnection ? this._currentPossibleConnection.name : null;
                    connector.sourcePort = this._possibleConnectionPort ? this._possibleConnectionPort.name : null;
                }

                return true;
            }
        }
        ConnectionEditTool.prototype._disconnect = function (connector) {
            var args;
            //var node = this.diagram.nameTable[connector.targetNode];
            var port = null;
            //if (this._endPoint == "targetEndPoint" && node) {
            //    port = ej.datavisualization.Diagram.Util.findPortByName(node, connector.targetPort);
            //    args = this._raiseEvent("connectionChange", { element: connector, endPoint: this._endPoint, connection: node, port: port, cancel: false });
            //    if (!args.cancel) {
            //        ej.datavisualization.Diagram.Util.removeItem(node.inEdges, connector.name);
            //        connector.targetNode = null;
            //        connector.targetPort = null;
            //    }

            //}
            //var node = this.diagram.nameTable[connector.sourceNode];
            //if ((this._endPoint == "sourceEndPoint") && node) {
            //    port = ej.datavisualization.Diagram.Util.findPortByName(node, connector.sourcePort);
            //    args = this._raiseEvent("connectionChange", { element: connector, endPoint: this._endPoint, connection: node, port: port, cancel: false });
            //    if (!args.cancel) {
            //        ej.datavisualization.Diagram.Util.removeItem(node.outEdges, connector.name);
            //        connector.sourceNode = null;
            //        connector.sourcePort = null;
            //    }
            //}
        };
        ConnectionEditTool.prototype._endAction = function () {
            this._showPorts();
            var possibleConnection;
            if (this._currentPossibleConnection && !this._possibleConnectionPort) {
                possibleConnection = this._currentPossibleConnection;
            }
            base.prototype._endAction.apply(this);
            if (this._endPoint) {
                this._endPoint = null;
                if (possibleConnection) {
                    this.diagram._checkToolToActivate(possibleConnection, this.currentPoint);
                    this.diagram._updateCursor();
                }
            }
        };
        ConnectionEditTool.prototype._updatePoints = function (connector) {
            if (this._endPoint != "segmentEnd") {
                var isTarget = (this._endPoint == "targetEndPoint") ? true : false;
                var isSource = (this._endPoint == "sourceEndPoint") ? true : false;

                var endPoint;
                if (this._endPoint) {
                    var end = this._endPoint.match("bezierpoint1") ? "bezierpoint1" : this._endPoint.match("bezierpoint2") ? "bezierpoint2" : this._endPoint;;

                    switch (end) {
                        case "targetEndPoint":
                            endPoint = this.selectedSegment._endPoint;
                            break;
                        case "sourceEndPoint":
                            endPoint = this.selectedSegment._startPoint;
                            break;
                        case "bezierpoint1":
                            endPoint = this.selectedSegment._point1;
                            break;
                        case "bezierpoint2":
                            endPoint = this.selectedSegment._point2;
                            break;
                    }

                    var ptCur = this.currentPoint;
                    if (this._possibleConnectionPort == null) {
                        ptCur = this.snap(ptCur);
                    }
                    var offset = ej.datavisualization.Diagram.Point(endPoint.x - ptCur.x, endPoint.y - ptCur.y);
                    if (!ej.datavisualization.Diagram.Geometry.isEmptyPoint(offset)) {
                        var cancel = false;
                        var args;
                        var type = null;
                        if (!this.inAction) {
                            if (isTarget) {
                                type = "target";
                            }
                            else if (isSource) {
                                type = "source";
                            }
                            args = this._raiseEvent("drag", { end: type, element: this.selectedObject, cancel: false });
                            cancel = args.cancel;
                        }
                        if (!cancel) {
                            if (isTarget || isSource) {
                                var connected = this._currentPossibleConnection ? ej.datavisualization.Diagram.Util.canConnect(this._currentPossibleConnection, false) : false;
                                if (this.selectedObject.segments.length > 1) {
                                    if (!isTarget && !connected && this.selectedSegment.type == "orthogonal" && (this.selectedSegment.length || this.selectedSegment.length === 0)) {
                                        this.selectedObject.sourcePoint = ptCur;
                                        var difx = ptCur.x - this.selectedSegment._startPoint.x;
                                        var dify = ptCur.y - this.selectedSegment._startPoint.y;
                                        if (this.selectedSegment.direction == "left" || this.selectedSegment.direction == "right") {
                                            this.selectedSegment._endPoint.y = this.selectedSegment.points[this.selectedSegment.points.length - 1].y = ptCur.y;
                                        }
                                        else {
                                            this.selectedSegment._endPoint.x = this.selectedSegment.points[this.selectedSegment.points.length - 1].x = ptCur.x;
                                        }
                                        this.selectedSegment._startPoint = this.selectedSegment.points[0] = ptCur;
                                        this.selectedSegment._length = this.selectedSegment.length = ej.datavisualization.Diagram.Geometry.distance(ptCur, this.selectedSegment._endPoint);
                                        if (this.selectedSegment._length)
                                            this.selectedSegment._direction = this.selectedSegment.direction = ej.datavisualization.Diagram.Util._getBezierDirection(ptCur, this.selectedSegment._endPoint);

                                        this.updateNextSegment(this.selectedObject.segments[0], this.selectedObject.segments[1], difx,
                                            dify);
                                        var next = this.selectedObject.segments[1];
                                        var updated = false;
                                        if (next && (next.length)) {
                                            next.length = next._length = ej.datavisualization.Diagram.Geometry.distance(this.selectedSegment._endPoint, next._endPoint);
                                            if (next.length)
                                                next.direction = next._direction = ej.datavisualization.Diagram.Util._getBezierDirection(this.selectedSegment._endPoint, next._endPoint);
                                        }
                                        if (this.selectedSegment.length == 0 || next.length === 0) {
                                            if (next.length == 0 && this.selectedObject.segments.length > 2) {
                                                this.selectedObject.segments.splice(0, 2);
                                            }
                                            else if (this.selectedObject.segments.length > 1) {
                                                this.selectedObject.segments.splice(0, 1);
                                            }
                                            this.selectedSegment = this.selectedObject.segments[0];
                                            this.selectedSegment._startPoint = this.selectedSegment.points[0] = ptCur;
                                            if (this.selectedSegment._direction == "left" || this.selectedSegment._direction == "right") {
                                                this.selectedSegment._endPoint.y = this.selectedSegment.points[this.selectedSegment.points.length - 1].y = ptCur.y;
                                            }
                                            else {
                                                this.selectedSegment._endPoint.x = this.selectedSegment.points[this.selectedSegment.points.length - 1].x = ptCur.x;
                                            }

                                            var next = this.selectedObject.segments[1];
                                            if (this.selectedObject.segments.length > 1) {
                                                this.selectedSegment._length = this.selectedSegment.length = ej.datavisualization.Diagram.Geometry.distance(ptCur, next._startPoint);
                                                ej.datavisualization.Diagram.Util._addOrthogonalPoints(this.selectedSegment, null, connector.segments[1], connector.sourcePoint, connector.targetPoint);
                                                ej.datavisualization.Diagram.Util._addOrthogonalPoints(connector.segments[1], connector.segments[0], connector.segments[2], connector.sourcePoint, connector.targetPoint);
                                                next._startPoint = next.points[0] = connector.segments[0]._endPoint;
                                            }
                                            else ej.datavisualization.Diagram.Util._addOrthogonalPoints(this.selectedSegment, null, connector.segments[1], connector.sourcePoint, connector.targetPoint);

                                            updated = true;
                                        }
                                        else ej.datavisualization.Diagram.Util._addOrthogonalPoints(this.selectedSegment, null, connector.segments[1], connector.sourcePoint, connector.targetPoint);
                                        if (next && (next.length)) {
                                            next.length = next._length = ej.datavisualization.Diagram.Geometry.distance(this.selectedSegment._endPoint, next._endPoint);
                                            if (next.length)
                                                next.direction = next._direction = ej.datavisualization.Diagram.Util._getBezierDirection(this.selectedSegment._endPoint, next._endPoint);
                                        }
                                    }
                                    else if (isTarget && !connected && this.selectedSegment.type == "orthogonal" && this.selectedSegment.points.length == 2) {
                                        var difx = ptCur.x - this.selectedSegment._endPoint.x;
                                        var dify = ptCur.y - this.selectedSegment._endPoint.y;

                                        if (this.selectedSegment._direction == "left" || this.selectedSegment._direction == "right") {
                                            this.selectedSegment._startPoint.y = (this.selectedSegment.points[0].y += dify);
                                            this.selectedSegment._endPoint.y = this.selectedSegment.points[1].y = this.selectedSegment._startPoint.y;
                                        }
                                        else {
                                            this.selectedSegment._startPoint.x = (this.selectedSegment.points[0].x += difx);
                                            this.selectedSegment._endPoint.x = this.selectedSegment.points[1].x = this.selectedSegment._startPoint.x;
                                        }
                                        var index = this.selectedObject.segments.length - 1;
                                        if (this.selectedObject.segments[index - 1])
                                            this.updatePreviousSegment(this.selectedObject.segments[index], this.selectedObject.segments[index - 1], difx,
                                                dify);
                                    }
                                }
                                if ((isSource || isTarget) && !connected) {
                                    var srclength = this.selectedSegment.points.length;
                                    this.diagram._setEndPoint(connector, ej.datavisualization.Diagram.Point(ptCur.x, ptCur.y), isTarget);
                                    if (srclength != this.selectedSegment.points.length || updated) {
                                        return true;
                                    }
                                }
                            }
                            else
                                ej.datavisualization.Diagram.Util._setBezierPoint(this.selectedSegment, ej.datavisualization.Diagram.Point(ptCur.x, ptCur.y), end);
                        }
                    }
                }
            }
        };
        return ConnectionEditTool;
    })(ToolBase);

    ej.datavisualization.Diagram.ConnectionEditTool = ConnectionEditTool;
    //#endregion

    //#region LineTool
    var LineTool = (function (base) {
        ej.datavisualization.Diagram.extend(LineTool, base);
        function LineTool(name, diagram) {
            base.call(this, name, diagram);
            this.cursor = "crosshair";
            this._isMouseDown = false;
            this._targetPossibleConnection = null;
            this._sourcePossibleConnection = null;
            this._targetPort = null;
            this._sourcePort = null;
        }
        LineTool.prototype.mousedown = function (evt) {
            base.prototype.mousedown.call(this, evt);
            this._isMouseDown = true;
            if (this.diagram._hasSelection()) {
                this.diagram._clearSelection();
            }
            if (this._currentPossibleConnection) {
                this._sourcePossibleConnection = this._currentPossibleConnection;
                this._sourcePort = this._possibleConnectionPort;
            }
        };
        LineTool.prototype.mousemove = function (evt) {
            base.prototype.mousemove.call(this, evt);
            this._checkConnectionPossible(evt);
            if (this._isMouseDown) {
                if (!this.inAction) {
                    this.inAction = true;
                    this._initHelper();
                    this._renderHelper();
                }
                else {
                    this._targetPossibleConnection = this._currentPossibleConnection;
                    this._targetPort = this._possibleConnectionPort;
                    this._updateHelper();
                }
            }
            this.previousPoint = this.currentPoint;
        };
        LineTool.prototype.mouseup = function (evt) {
            this._isMouseDown = false;
            if (this.inAction) {
                this.inAction = false;
                var ptStart = this._getStartPoint();
                var ptEnd = this._getCurrentPoint();
                var distance = ej.datavisualization.Diagram.Geometry.distance(ptStart, ptEnd);
                if (distance > 0) {
                    var connector = this.createConnector(ptStart, ptEnd);
                    connector.name = "connector" + ej.datavisualization.Diagram.Util.randomId();
                    var args;
                    if (this._sourcePossibleConnection) {
                        if (this._sourcePort && ej.datavisualization.Diagram.Util.canConnect(this._sourcePort, true)) {
                            args = this._raiseEvent("connectionChange", { element: connector, connection: this._sourcePossibleConnection, port: this._sourcePort, cancel: false });
                            if (!args.cancel) {
                                connector.sourceNode = this._sourcePossibleConnection.name;
                                connector.sourcePort = this._sourcePort.name;
                                ej.datavisualization.Diagram.Util.removeItem(this._sourcePossibleConnection.outEdges, this.helper.name);
                            }
                        }
                        else if (ej.datavisualization.Diagram.Util.canConnect(this._sourcePossibleConnection)) {
                            args = this._raiseEvent("connectionChange", { element: connector, connection: this._sourcePossibleConnection, port: this._sourcePort, cancel: false });
                            if (!args.cancel) {
                                connector.sourceNode = this._sourcePossibleConnection.name;
                                ej.datavisualization.Diagram.Util.removeItem(this._sourcePossibleConnection.outEdges, this.helper.name);
                            }
                        }
                    }
                    if (this._targetPossibleConnection) {
                        if (this._targetPort && ej.datavisualization.Diagram.Util.canConnect(this._targetPort, true)) {
                            args = this._raiseEvent("connectionChange", { element: connector, connection: this._targetPossibleConnection, port: this._targetPort, cancel: false });
                            if (!args.cancel) {
                                connector.targetNode = this._targetPossibleConnection.name;
                                connector.targetPort = this._targetPort.name;
                                ej.datavisualization.Diagram.Util.removeItem(this._targetPossibleConnection.inEdges, this.helper.name);
                            }
                        }
                        else if (ej.datavisualization.Diagram.Util.canConnect(this._targetPossibleConnection)) {
                            args = this._raiseEvent("connectionChange", { element: connector, connection: this._targetPossibleConnection, port: this._targetPort, cancel: false });
                            if (!args.cancel) {
                                connector.targetNode = this._targetPossibleConnection.name;
                                ej.datavisualization.Diagram.Util.removeItem(this._targetPossibleConnection.inEdges, this.helper.name);
                            }
                        }

                    }
                    this.diagram.add(connector);
                    ej.datavisualization.Diagram.Util.updateBridging(connector, this.diagram);
                    ej.datavisualization.Diagram.SvgContext.update(connector, this.diagram);
                    this.diagram._updateConnectorBridging(connector);
                }
            }
            if (this.singleAction)
                this.diagram.deactivateTool();
            if (this._prevTool)
                this._prevTool.inAction = false;
            this._showPorts();
            base.prototype.mouseup.call(this, evt);
        };

        LineTool.prototype._getStartPoint = function () {
            var ptStart;
            if (this._sourcePossibleConnection && this._sourcePort) {
                ptStart = ej.datavisualization.Diagram.Util._getPortPosition(this._sourcePort, ej.datavisualization.Diagram.Util.bounds(this._sourcePossibleConnection));
            }
            else {
                ptStart = this.snap(this.startPoint);
            }
            return ptStart;
        };
        LineTool.prototype._getCurrentPoint = function () {
            var ptCur;
            if (this._targetPossibleConnection && this._targetPort) {
                ptCur = ej.datavisualization.Diagram.Util._getPortPosition(this._targetPort, ej.datavisualization.Diagram.Util.bounds(this._targetPossibleConnection));
            }
            else {
                ptCur = this.snap(this.currentPoint);
            }
            return ptCur;
        };
        LineTool.prototype._initHelper = function () {
            this.helper = this.createConnector(ej.datavisualization.Diagram.Point(0, 0), ej.datavisualization.Diagram.Point(0, 0));
            this.diagram._setZorder(this.helper);
            this.helper.name = "helper";
        };
        LineTool.prototype.createConnector = function (startPoint, endPoint) {
            return null;
        };
        LineTool.prototype._renderHelper = function () {
            if (!this.svgHelper) {
                this.svgHelper = ej.datavisualization.Diagram.SvgContext.renderConnector(this.helper, this.diagram._svg, this.diagram._diagramLayer);
                this.svgHelper = document.getElementById(this.helper.name);
                this.svgHelper.setAttribute("pointer-events", "none");
                $(this.diagram._svg.getElementById(this.svgHelper.id)).attr("pointer-events", "none");
                $(this.diagram._svg.getElementById(this.svgHelper.id + "_hitTest")).attr("pointer-events", "none");
            }
        };
        LineTool.prototype._updateHelper = function () {
            var points = [];
            var ptStart = this._getStartPoint();
            var ptEnd = this._getCurrentPoint();
            points.push(ptStart);
            points.push(ptEnd);
            this.diagram._setEndPoint(this.helper, ptStart, false);
            this.diagram._setEndPoint(this.helper, ptEnd, true);
            this._disconnect(this.helper);
            if (this._sourcePossibleConnection) {
                if (this._sourcePort && ej.datavisualization.Diagram.Util.canConnect(this._sourcePort, true)) {
                    this.helper.sourceNode = this._sourcePossibleConnection.name;
                    this.helper.sourcePort = this._sourcePort.name;
                    this._sourcePossibleConnection.outEdges.push(this.helper.name);
                }
                else if (ej.datavisualization.Diagram.Util.canConnect(this._sourcePossibleConnection)) {
                    this.helper.sourceNode = this._sourcePossibleConnection.name;
                    this._sourcePossibleConnection.outEdges.push(this.helper.name);
                }
            }
            if (this._targetPossibleConnection) {
                if (this._targetPort && ej.datavisualization.Diagram.Util.canConnect(this._targetPort, true)) {
                    this.helper.targetNode = this._targetPossibleConnection.name;
                    this.helper.targetPort = this._targetPort.name;
                    this._targetPossibleConnection.inEdges.push(this.helper.name);
                }
                else if (ej.datavisualization.Diagram.Util.canConnect(this._targetPossibleConnection)) {
                    this.helper.targetNode = this._targetPossibleConnection.name;
                    this._targetPossibleConnection.inEdges.push(this.helper.name);
                }
            }
            this.diagram._dock(this.helper, this.diagram.nameTable);
            
            ej.datavisualization.Diagram.SvgContext.update(this.helper, this.diagram);
        };
        LineTool.prototype._endAction = function () {
            base.prototype._endAction.apply(this);
            this._targetPossibleConnection = null;
            this._sourcePossibleConnection = null;
            this._targetPort = null;
            this._sourcePort = null;
            this._isMouseDown = false;
        };
        LineTool.prototype._showPorts = function (node) {
        };
        LineTool.prototype._showAllPorts = function (hide) {
            base.prototype._showAllPorts.call(this, hide);
        };
        return LineTool;
    })(ToolBase);

    ej.datavisualization.Diagram.LineTool = LineTool;
    //#endregion

    //#region StraightLineTool
    var StraightLineTool = (function (base) {
        ej.datavisualization.Diagram.extend(StraightLineTool, base);
        function StraightLineTool(diagram) {
            base.call(this, "straightLine", diagram);
        }
        StraightLineTool.prototype.createConnector = function (startPoint, endPoint) {
            var connector = this.diagram._getNewConnector({ "segments": [{ "type": "straight" }], "sourcePoint": startPoint, "targetPoint": endPoint });
            return connector;
        };
        return StraightLineTool;
    })(LineTool);

    ej.datavisualization.Diagram.StraightLineTool = StraightLineTool;
    //#endregion

    //#region OrthogonalLineTool
    var OrthogonalLineTool = (function (base) {
        ej.datavisualization.Diagram.extend(OrthogonalLineTool, base);
        function OrthogonalLineTool(diagram) {
            base.call(this, "orthogonalLine", diagram);
        }
        OrthogonalLineTool.prototype.createConnector = function (startPoint, endPoint) {
            var connector = this.diagram._getNewConnector({ "segments": [{ "type": "orthogonal" }], "sourcePoint": startPoint, "targetPoint": endPoint });
            return connector;
        };
        return OrthogonalLineTool;
    })(LineTool);

    ej.datavisualization.Diagram.OrthogonalLineTool = OrthogonalLineTool;
    //#endregion

    //#region RotateTool
    var RotateTool = (function (base) {
        ej.datavisualization.Diagram.extend(RotateTool, base);
        function RotateTool(diagram) {
            base.call(this, "rotatetool", diagram);
            this._mouseOffset = ej.datavisualization.Diagram.Size(0, 0);
            this.undoObject = null;
        }
        RotateTool.prototype.mousedown = function (evt) {
            base.prototype.mousedown.call(this, evt);
            this.selectedObject = this.diagram.selectionList[0];
            //this.undoObject = ej.datavisualization.Diagram.Util.clone(this.selectedObject);

            var data;
            var childTable = {};
            if (this.selectedObject.isGroup || this.selectedObject.isPseudoGroup)
                childTable = this.diagram._getChildTable(this.selectedObject, childTable);
            var temp = { "childTable": childTable, "node": this.selectedObject };
            data = $.extend(true, {}, temp);
            this.undoObject = jQuery.extend(true, {}, data);

            var scale = this.diagram._currZoom;
            //ej.datavisualization.Diagram.SvgContext.renderPivotPoint(this.selectedObject, this.diagram._currZoom, this.diagram._svg);

        };
        RotateTool.prototype.mousemove = function (evt) {
            base.prototype.mousemove.call(this, evt);

            if (this.selectedObject) {
                if (!this.inAction) {
                    this.inAction = true;
                }
                else {
                    var isContainer = false;
                    if (this.selectedObject.parent) {
                        var parent = this.diagram.nameTable[this.selectedObject.parent];
                        if (parent.container) {
                            isContainer = true;
                        }
                    }
                    else if (this.selectedObject.isPseudoGroup && this.hasSameParent() && this._fromContainer(this.selectedObject.children)) {
                        isContainer = true;
                    }
                    if (isContainer) {
                        if (!this.helper) {
                            this.helper = this._getCloneNode(this.selectedObject);
                            this.helper.name = "helper";
                        }
                        this._rotate(this.helper);
                        ej.datavisualization.Diagram.SvgContext._drawContainerHelper(this.diagram);
                    }

                    else {
                        if (this.selectedObject.isPseudoGroup && this._fromContainer(this.selectedObject.children)) {// && this._fromContainer(this.selectedObject.children) && this.hasSameParent()) {
                            if (this._fromContainer(this.selectedObject.children) && this.hasSameParent()) {
                                this._rotate(this.selectedObject);
                                if (this.selectedObject.parent) {
                                    ej.datavisualization.Diagram.Util._updateGroupBounds(this.diagram.nameTable[this.selectedObject.parent], this.diagram.nameTable);
                                }
                                ej.datavisualization.Diagram.DiagramContext.update(this.selectedObject, this.diagram);
                                this.diagram._updateSelectionHandle(true);
                                this.diagram._renderTooltip(this.selectedObject);
                            }
                        }
                        else {
                            this._rotate(this.selectedObject);
                            if (this.selectedObject.parent) {
                                ej.datavisualization.Diagram.Util._updateGroupBounds(this.diagram.nameTable[this.selectedObject.parent], this.diagram.nameTable);
                            }
                            ej.datavisualization.Diagram.DiagramContext.update(this.selectedObject, this.diagram);
                            this.diagram._updateSelectionHandle(true);
                            this.diagram._renderTooltip(this.selectedObject);
                        }
                    }
                }
            }
            this.previousPoint = this.currentPoint;
        };
        RotateTool.prototype._rotate = function (node) {
            var cancel = false;
            var args;
            if (!this.inAction) {
                args = this._raiseEvent("rotationChange", { element: node, cancel: false });
                cancel = args.cancel;
            }
            if (!cancel) {
                var pt = ej.datavisualization.Diagram.Point(node.offsetX, node.offsetY);
                var crntpoint = this.currentPoint;
                var angle = ej.datavisualization.Diagram.Geometry.findAngle(ej.datavisualization.Diagram.Point(pt.x, pt.y), crntpoint) + 90;
                angle = ej.datavisualization.Diagram.SnapUtil._snapAngle(this.diagram, angle);
                this.diagram._rotate(node, angle - node.rotateAngle, this.diagram.nameTable);
            }
        };

        RotateTool.prototype._updateRotateObject = function () {
            var object = this.diagram.nameTable[this.selectedObject.name];
            ej.datavisualization.Diagram.DiagramContext.update(this.selectedObject, this.diagram);
            if (this.helper) {
                var difAngle = this.helper.rotateAngle - this.diagram.nameTable[this.selectedObject.name].rotateAngle;
                //object.rotateAngle += difAngle;
                this.diagram._rotate(this.selectedObject, difAngle, this.diagram.nameTable);
                ej.datavisualization.Diagram.DiagramContext.update(this.selectedObject, this.diagram);
            }
            return object;
        };
        //RotateTool.prototype._updateMargin = function (node, group) {
        //    if (node.isPseudoGroup) {
        //        var child = null;
        //        for (var i = 0; i < node.children.length; i++) {
        //            child = this.diagram.nameTable[node.children[i]];
        //            this._updateMargin(child, group);
        //        }
        //    }
        //    else if (group && group.container && group.container.type == "canvas") {
        //        var groupBounds = ej.datavisualization.Diagram.Util.bounds(group); //ej.datavisualization.Diagram.SvgContext.getCanvasBoundingBox(group, this.diagram, node, true);
        //        var bounds1 = ej.datavisualization.Diagram.Util.bounds(node);
        //        var bounds = ej.datavisualization.Diagram.Geometry.rect([bounds1.topLeft, bounds1.topRight, bounds1.bottomRight, bounds1.bottomLeft]);
        //        node.marginLeft = bounds.x - (groupBounds.x + group.paddingLeft);
        //        node.marginRight = (groupBounds.x + group.width) - (bounds.x + bounds.width) - group.paddingRight;
        //        node.marginTop = bounds.y - (groupBounds.y + group.paddingTop);
        //        node.marginBottom = groupBounds.bottom - (bounds.y + bounds.height) - group.paddingBottom;
        //    }
        //};
        RotateTool.prototype._updateContainer = function (newObj, prevParent, isDrop) {
            var bounds, groupBounds, dx = 0, dy = 0, newSize;
            if (newObj) {
                var parObj = this.diagram.nameTable[newObj.parent];
                if (!parObj && newObj.isPseudoGroup) {
                    var fNode = this.diagram.nameTable[this.selectedObject.children[0]];
                    parObj = this.diagram.nameTable[fNode.parent];
                }
                if (parObj) {
                    if (parObj.container.type == "canvas") {
                        if (isDrop) {
                            isDrop = false;
                            var group = this.diagram.nameTable[newObj.parent];
                            if (!group && newObj.isPseudoGroup) {
                                group = parObj;
                            }
                            var groupBounds123 = ej.datavisualization.Diagram.Util.bounds(group);
                            var grAngle = group.rotateAngle;
                            this.diagram._rotate(group, -group.rotateAngle, this.diagram.nameTable);
                            ej.datavisualization.Diagram.DiagramContext.update(group, this.diagram);
                            bounds = ej.datavisualization.Diagram.Util.bounds(newObj);
                            var bounds1 = ej.datavisualization.Diagram.Geometry.rect([bounds.topLeft, bounds.topRight, bounds.bottomRight, bounds.bottomLeft]);
                            groupBounds = ej.datavisualization.Diagram.Util.bounds(group);
                            var tightchildBounds = this.diagram._getTightChildrenBounds(group);
                            var right = true;
                            var bottom = true;
                            var bRight = bounds1.x + bounds1.width;
                            var bBottom = bounds1.y + bounds1.height;
                            if (bRight >= groupBounds.right) {
                                ///if exeeds right 
                                dx = bRight - groupBounds.right + group.paddingRight;
                                newSize = this.diagram._getMinMaxSize(group, (group.width + dx), group.height, true);
                                widDiff = newSize.width - group.width;
                                group.width = newSize.width;
                                group.offsetX += (widDiff / 2);
                                if (bRight < groupBounds.right) {
                                    right = false;
                                }
                            }
                            if (bBottom >= groupBounds.bottom) {
                                ///if exeeds bottom 
                                dy = bBottom - groupBounds.bottom + group.paddingBottom;
                                newSize = this.diagram._getMinMaxSize(group, (group.width), (group.height + dy), true);
                                heiDiff = newSize.height - group.height;
                                group.height = newSize.height;
                                group.offsetY += (heiDiff / 2);
                                bottom = true;
                                if (bBottom < groupBounds.bottom) {
                                    bottom = false;
                                }
                            }
                            if (bRight < groupBounds.right && right) { //if decreces from right
                                dx = (tightchildBounds.x + tightchildBounds.width) + group.paddingRight - groupBounds.right;
                                newSize = this.diagram._getMinMaxSize(group, (group.width + dx), group.height, true);
                                var widDiff = newSize.width - group.width;
                                group.width = newSize.width;
                                group.offsetX += (widDiff / 2);
                            }
                            if (bBottom < groupBounds.bottom && bottom) { //if decreces from bottom
                                dy = (tightchildBounds.y + tightchildBounds.height) + group.paddingBottom - groupBounds.bottom;
                                newSize = this.diagram._getMinMaxSize(group, (group.width), (group.height + dy), true);
                                var heiDiff = newSize.height - group.height;
                                group.height = newSize.height;
                                group.offsetY += (heiDiff / 2);
                            }
                            ej.datavisualization.Diagram.DiagramContext.update(group, this.diagram);
                            this._updateMargin(newObj, group);
                            ej.datavisualization.Diagram.SvgContext._alignOnCanvas1(group, this.diagram);
                            this.diagram._rotate(group, grAngle, this.diagram.nameTable);
                            var groupBounds1231 = ej.datavisualization.Diagram.Util.bounds(group);
                            this.diagram._translate(group, groupBounds123.topLeft.x - groupBounds1231.topLeft.x, groupBounds123.topLeft.y - groupBounds1231.topLeft.y, this.diagram.nameTable);
                            if (group.parent && group.container && group.container.type == "canvas")
                                this._updateMargin(group, this.diagram.nameTable[group.parent]);
                            ej.datavisualization.Diagram.DiagramContext.update(group, this.diagram);
                        } else {
                            var vertical = false;
                            var horizontal = false;
                            group = this.diagram.nameTable[newObj.parent];
                            if (newObj.container && newObj.container.type == "stack") {
                                if (newObj.container.orientation == "vertical")
                                    vertical = true;
                                else
                                    horizontal = true;
                            }
                            bounds = this.diagram._getBoundsWithoutStretch(group, vertical, horizontal); // ej.datavisualization.Diagram.SvgContext._measureStackSize(group, this.diagram, true);
                            bounds.width = group.paddingLeft + group.paddingRight;
                            bounds.height = group.paddingTop + group.paddingBottom;
                            newSize = this.diagram._getMinMaxSize(group, (bounds.width), (bounds.height), true);
                            widDif = newSize.width - group.width;
                            heiDif = newSize.height - group.height;
                            group.offsetX += (widDif / 2);
                            group.width = newSize.width;
                            group.offsetY += (heiDif / 2);
                            group.height = newSize.height;
                            ej.datavisualization.Diagram.SvgContext._alignOnCanvas1(group, this.diagram);
                            ej.datavisualization.Diagram.DiagramContext.update(group, this.diagram);
                        }
                        if (group.parent) {
                            if (this.diagram.nameTable[group.parent])
                                isDrop = true;
                            this._updateContainer(group, null, isDrop);
                        }
                    }
                    else if (parObj.container.type == "stack") {
                        group = this.diagram.nameTable[newObj.parent];
                        bounds = ej.datavisualization.Diagram.SvgContext._measureStackSize(group, this.diagram, true);
                        newSize = this.diagram._getMinMaxSize(group, (bounds.width), (bounds.height), true);
                        var widDif = newSize.width - group.width;
                        var heiDif = newSize.height - group.height;
                        group.offsetX += (widDif / 2);
                        group.width = newSize.width;
                        group.offsetY += (heiDif / 2);
                        group.height = newSize.height;
                        ej.datavisualization.Diagram.SvgContext._alignOnStack(group, this.diagram);
                        ej.datavisualization.Diagram.DiagramContext.update(group, this.diagram);
                        if (group.parent)
                            this._updateContainer(group);
                    }
                }
            }
        };
        RotateTool.prototype.mouseup = function (evt) {

            if (this.inAction) {
                this.inAction = false;
                var isContainer = false;
                var canvasChild = false;
                if (this.selectedObject.parent) {
                    var parent = this.diagram.nameTable[this.selectedObject.parent];
                    if (parent.container) {
                        isContainer = true;
                    }
                }
                else if (this.selectedObject.isPseudoGroup && this.hasSameParent() && this._fromContainer(this.selectedObject.children)) {
                    var fNode = this.diagram.nameTable[this.selectedObject.children[0]];
                    parent = this.diagram.nameTable[fNode.parent];
                    if (parent.container) {
                        isContainer = true;
                    }
                }
                if (isContainer) {
                    ej.datavisualization.Diagram.SvgContext._removeContainerHelper(this.selectedObject, this.diagram._svg, this.diagram._adornerLayer);
                    this._removeHighLighter(); //ej.datavisualization.Diagram.SvgContext._removeNodeHighlighter(this.diagram._svg, this.diagram._adornerLayer);
                    if (parent.container.type == "canvas")
                        canvasChild = true;
                    if (canvasChild) {
                        var object = this._updateRotateObject();
                        this._updateContainer(object, null, true);
                    }
                }
                if (!this.selectedObject.isPseudoGroup)
                    this._rotate(this.selectedObject);
                else {
                    if (!this._fromContainer(this.selectedObject.children))
                        this._rotate(this.selectedObject);
                }
                ej.datavisualization.Diagram.DiagramContext.update(this.selectedObject, this.diagram);

                var data;
                var childTable = {};
                if (this.selectedObject.isGroup || this.selectedObject.isPseudoGroup)
                    childTable = this.diagram._getChildTable(this.selectedObject, childTable);
                var temp = { "childTable": childTable, "node": this.selectedObject };
                data = $.extend(true, {}, temp);
                var entry = { type: "rotationchanged", undoObject: this.undoObject, redoObject: jQuery.extend(true, {}, data), isMultipleNode: this.selectedObject.isGroup ? true : false };
                this.diagram.addHistoryEntry(entry);

                //var entry = new ej.datavisualization.Diagram.HistoryEntry(new ej.datavisualization.Diagram.RotationHandleCmd(this.undoObject, ej.datavisualization.Diagram.Util.clone(this.selectedObject), this.selectedObject.isPseudoGroup ? true : false));
                //this.diagram.historyManager.addHistoryEntry(entry);
                this.diagram._updateSelectionHandle();
                this.diagram._removeTooltip();
            }
            //this.diagram._removeTooltip();
            base.prototype.mouseup.call(this, evt);
        };
        return RotateTool;
    })(ToolBase);

    ej.datavisualization.Diagram.RotateTool = RotateTool;
    //#endregion 

    //#region PivotTool
    var PivotTool = (function (base) {
        ej.datavisualization.Diagram.extend(PivotTool, base);
        function PivotTool(diagram) {
            base.call(this, "PivotTool", diagram);
            this._mouseOffset = ej.datavisualization.Diagram.Size(0, 0);
        }
        PivotTool.prototype.mousedown = function (evt) {
            base.prototype.mousedown.call(this, evt);
            this.selectedObject = this.diagram.selectionList[0];
        };
        PivotTool.prototype.mousemove = function (evt) {
            base.prototype.mousemove.call(this, evt);

            if (this.selectedObject) {
                if (!this.inAction) {
                    this.inAction = true;
                }
                else {
                    var delx = this.currentPoint.x - this.previousPoint.x;
                    var dely = this.currentPoint.y - this.previousPoint.y;
                    var matrix = ej.Matrix.identity();
                    ej.Matrix.rotate(matrix, -this.selectedObject.rotateAngle);
                    var delta = ej.Matrix.transform(matrix, ej.datavisualization.Diagram.Point(delx, dely));
                    var difx = (this.selectedObject.width * this.selectedObject.pivot.x + delta.x) / this.selectedObject.width;
                    var dify = (this.selectedObject.height * this.selectedObject.pivot.y + delta.y) / this.selectedObject.height;
                    this.selectedObject.pivot.x = difx;
                    this.selectedObject.pivot.y = dify;
                    this.selectedObject.offsetX += delx;
                    this.selectedObject.offsetY += dely;
                    this.diagram._updateSelectionHandle();
                }
            }
            this.previousPoint = this.currentPoint;
        };
        PivotTool.prototype._rotate = function (node) {
            var pt = ej.datavisualization.Diagram.Point(node.offsetX, node.offsetY);
            var crntpoint = this.currentPoint;
            var angle = this.FindAngle(pt, crntpoint);
            this.diagram._rotate(node, angle - node.rotateAngle, pt, this.diagram.nameTable);
        };
        PivotTool.prototype.mouseup = function (evt) {
            if (this.inAction) {
                this.inAction = false;
                this.diagram._updateSelectionHandle();
            }
            base.prototype.mouseup.call(this, evt);
        };
        return PivotTool;
    })(ToolBase);
    ej.datavisualization.Diagram.PivotTool = PivotTool;
    //#endregion

    //#region PanTool
    var PanTool = (function (base) {
        ej.datavisualization.Diagram.extend(PanTool, base);
        function PanTool(diagram) {
            base.call(this, "panTool", diagram);
            this.diagram = diagram;
            this._isMouseDown = false;
            this.cursor = "pointer";
        }
        PanTool.prototype.mousedown = function (evt) {
            base.prototype.mousedown.call(this, evt);
            var svgDocument = this.diagram._svg.document;
            var pt = this.diagram._svg.document.createSVGPoint();
            pt.x = evt.clientX; pt.y = evt.clientY;
            var mousept = pt.matrixTransform(this.diagram._svg.document.getScreenCTM().inverse());
            svgDocument.lastMouseX = mousept.x;
            svgDocument.lastMouseY = mousept.y;
            this._isMouseDown = true;
            return false;
        };

        PanTool.prototype.mousemove = function (evt) {
            base.prototype.mousemove.call(this, evt);
            if (this._isMouseDown) {
                this.inAction = true;
                var svgDocument = this.diagram._svg.document;
                var pt = this.diagram._svg.document.createSVGPoint();
                pt.x = evt.clientX; pt.y = evt.clientY;
                var mousept = pt.matrixTransform(this.diagram._svg.document.getScreenCTM().inverse());
                var ex = mousept.x;
                var ey = mousept.y;
                var nx, ny;

                nx = ((ex - svgDocument.lastMouseX));
                ny = ((ey - svgDocument.lastMouseY));
                if (!(this.diagram.model.constraints & ej.datavisualization.Diagram.DiagramConstraints.PannableX))
                    nx = 0;
                if (!(this.diagram.model.constraints & ej.datavisualization.Diagram.DiagramConstraints.PannableY))
                    ny = 0;
                ej.datavisualization.Diagram.ZoomUtil.zoomPan(this.diagram, 1, nx, ny, ej.datavisualization.Diagram.Point(ex, ey));
                svgDocument.lastMouseX = ex;
                svgDocument.lastMouseY = ey;
                return false;
            }
        };
        PanTool.prototype.mouseup = function (evt) {
            this.inAction = false;
            document.onmousemove = null;
            document.onmouseup = null;
            this._isMouseDown = false;
            base.prototype.mouseup.call(this, evt);
        };
        return PanTool;
    })(ToolBase);
    ej.datavisualization.Diagram.PanTool = PanTool;
    //#endregion

    //#region RectangleToolBase
    var RectangleToolBase = (function (base) {
        ej.datavisualization.Diagram.extend(RectangleToolBase, base);
        function RectangleToolBase(name, diagram) {
            base.call(this, name, diagram);
            this.cursor = "crosshair";
        }
        RectangleToolBase.prototype.mousedown = function (evt) {
            base.prototype.mousedown.call(this, evt);
            this.selectedObject = this.createNode();
        };
        RectangleToolBase.prototype.createNode = function () {
        };
        RectangleToolBase.prototype.mousemove = function (evt) {
            base.prototype.mousemove.call(this, evt);
            if (this.selectedObject) {
                if (!this.inAction) {
                    this.inAction = true;
                    this._initHelper();
                    this._renderHelper();
                    this._calcOffset();
                }
                else {
                    this._updateHelper();
                }
            }
        };
        RectangleToolBase.prototype.mouseup = function (evt) {
            if (this.inAction) {
                this.inAction = false;
                if (this.diagram.model.snapSettings.snapConstraints.snapToLines) {
                    this._calcOffset();
                }
                this._updateSize(this.selectedObject, this.startPoint, this.currentPoint);
                this.diagram.add(this.selectedObject);
                this.diagram._clearSelection();
                this.diagram._addSelection(this.selectedObject);
            }
            this.diagram._toolToActivate = "selectTool";
            this.diagram._drawingTool = false;
            base.prototype.mouseup.call(this, evt);
        };
        RectangleToolBase.prototype._calcOffset = function () {
            var scale = this.diagram.model.magnification / 100;
            var bounds = ej.datavisualization.Diagram.Util.bounds(this.helper);
            var location = bounds.topLeft;
            location = ej.datavisualization.Diagram.Point(bounds.x + bounds.width, bounds.y + bounds.height);
            var x = location.x * scale;
            var y = location.y * scale;
            var xy = ej.datavisualization.Diagram.Point(x, y);
            var snapxy = this.snap(ej.datavisualization.Diagram.Point(x, y));
            this._mouseOffset = ej.datavisualization.Diagram.Size(xy.x - snapxy.x, xy.y - snapxy.y);
        };
        RectangleToolBase.prototype.getBoundingRect = function (startPoint, currentPoint) {
            var width = Math.abs(this.startPoint.x - this.currentPoint.x);
            var height = Math.abs(this.startPoint.y - this.currentPoint.y);
            var diff = ej.datavisualization.Diagram.Size(width, height);
            if (this.diagram.model.snapSettings.snapConstraints.snapToVerticalLines) {
                if (this._mouseOffset.width != 0) {
                    diff.width -= this._mouseOffset.width;
                    this._mouseOffset.width = 0;
                }
            }
            if (this.diagram.model.snapSettings.snapConstraints.snapToHorizontalLines) {
                if (this._mouseOffset.height != 0) {
                    diff.height -= this._mouseOffset.height;
                    this._mouseOffset.height = 0;
                }
            }
            var x = (this.startPoint.x > this.currentPoint.x) ? this.currentPoint.x : this.startPoint.x;
            var y = (this.startPoint.y > this.currentPoint.y) ? this.currentPoint.y : this.startPoint.y;
            return { size: diff, offset: ej.datavisualization.Diagram.Point(x, y) };
        };
        RectangleToolBase.prototype._renderHelper = function () {
            if (!this.svgHelper) {
                this.svgHelper = ej.datavisualization.Diagram.SvgContext.renderNode(this.helper, this.diagram._svg, this.diagram._diagramLayer);
                this.svgHelper = document.getElementById(this.helper.name);
                this.svgHelper.setAttribute("pointer-events", "none");
            }
        };
        RectangleToolBase.prototype._updateHelper = function () {
            this._updateSize(this.helper, this.startPoint, this.currentPoint);
            ej.datavisualization.Diagram.SvgContext.update(this.helper, this.diagram);
        };
        RectangleToolBase.prototype._updateSize = function (shape, startPoint, currentPoint) {
            var rect = this.getBoundingRect(startPoint, currentPoint);
            var size = rect.size;
            var offset = rect.offset;
            if (!ej.datavisualization.Diagram.Geometry.isEmptySize(size)) {
                shape.offsetX = offset.x + size.width / 2;
                shape.offsetY = offset.y + size.height / 2;
                shape.width = size.width;
                shape.height = size.height;
            }
        };
        return RectangleToolBase;
    })(ej.datavisualization.Diagram.ToolBase);
    ej.datavisualization.Diagram.RectangleToolBase = RectangleToolBase;
    //#endregion

    //#region PolygonToolBase
    var PolygonToolBase = (function (base) {
        ej.datavisualization.Diagram.extend(PolygonToolBase, base);
        function PolygonToolBase(name, diagram) {
            base.call(this, name, diagram);
            this.cursor = "crosshair";
        }
        PolygonToolBase.prototype.createNode = function () {

        };
        PolygonToolBase.prototype.mousedown = function (evt) {
            base.prototype.mousedown.call(this, evt);
        };
        PolygonToolBase.prototype.mousemove = function (evt) {

            base.prototype.mousemove.call(this, evt);
            if (this.selectedObject) {
                if (!this.inAction) {
                    this.inAction = true;
                    this._initHelper();
                    this._renderHelper();
                    this._calcOffset();
                }
                else {
                    this._updateHelper(this.snap(this.currentPoint));
                }
            }
        };
        PolygonToolBase.prototype.mouseup = function (evt) {
            if (!this.selectedObject) {
                if (this.startPoint) {
                    this.selectedObject = this.createNode();
                    var sp = this.snap(this.startPoint);
                    var ep = this.snap(this.currentPoint);
                    this.selectedObject.shape.points.push(ej.datavisualization.Diagram.Point(sp.x, sp.y));
                    this.selectedObject.shape.points.push(ej.datavisualization.Diagram.Point(ep.x, ep.y));
                }
            }
            else if (this.inAction) {
                this.helper.shape.points.push(ej.datavisualization.Diagram.Point(this.currentPoint.x, this.currentPoint.y));
            }
        };
        PolygonToolBase.prototype._calcOffset = function () {
            var scale = this.diagram.model.magnification / 100;
            var bounds = ej.datavisualization.Diagram.Util.bounds(this.helper);
            var location = bounds.topLeft;
            location = ej.datavisualization.Diagram.Point(bounds.x + bounds.width, bounds.y + bounds.height);
            var x = location.x * scale;
            var y = location.y * scale;
            var xy = ej.datavisualization.Diagram.Point(x, y);
            var snapxy = this.snap(ej.datavisualization.Diagram.Point(x, y));
            this._mouseOffset = ej.datavisualization.Diagram.Size(xy.x - snapxy.x, xy.y - snapxy.y);
        };
        PolygonToolBase.prototype._renderHelper = function () {
            if (!this.svgHelper) {

                var rect = ej.datavisualization.Diagram.Geometry.rect(this.helper.shape.points);
                this.helper.offsetX = rect.x + rect.width / 2;
                this.helper.offsetY = rect.y + rect.height / 2;
                this.helper.width = rect.width;
                this.helper.height = rect.height;
                this.svgHelper = ej.datavisualization.Diagram.SvgContext.renderNode(this.helper, this.diagram._svg, this.diagram._diagramLayer);
                this.svgHelper = document.getElementById(this.helper.name);
                this.svgHelper.setAttribute("pointer-events", "none");
            }
        };
        PolygonToolBase.prototype.doubleclick = function (currentPoint) {
            if (this.selectedObject) {

                ej.datavisualization.Diagram.Util.removeItem(this.helper.shape.points, this.helper.shape.points[this.helper.shape.points.length - 2]);
                ej.datavisualization.Diagram.Util.removeItem(this.helper.shape.points, this.helper.shape.points[this.helper.shape.points.length - 1]);
                this.selectedObject.shape.points = this.helper.shape.points;
                var rect = ej.datavisualization.Diagram.Geometry.rect(this.helper.shape.points);
                this.selectedObject.offsetX = rect.x + rect.width / 2;
                this.selectedObject.offsetY = rect.y + rect.height / 2;
                this.selectedObject.width = rect.width;
                this.selectedObject.height = rect.height;
                this.diagram.model.nodes.push(this.selectedObject);
                this.diagram.nameTable[this.selectedObject.name] = this.selectedObject;
                ej.datavisualization.Diagram.DiagramContext.renderNode(this.selectedObject, this.diagram);

                this.diagram._clearSelection();
                this.diagram._addSelection(this.selectedObject);
            }
            if (this._svgHelper) {
                this.diagram._svg.removeChild(this._svgHelper);
                this._svgHelper = null;
            }
            this._endAction();
            this.diagram._drawingTool = false;
            this.diagram._toolToActivate = "selectTool";
            this._mouseOffset = ej.datavisualization.Diagram.Size(0, 0);
        };
        PolygonToolBase.prototype._updateHelper = function (currentPoint) {
            this.helper.shape.points[this.helper.shape.points.length - 1] = ej.datavisualization.Diagram.Point(currentPoint.x, currentPoint.y);
            var rect = ej.datavisualization.Diagram.Geometry.rect(this.helper.shape.points);
            this.helper.offsetX = rect.x + rect.width / 2;
            this.helper.offsetY = rect.y + rect.height / 2;
            this.helper.width = rect.width;
            this.helper.height = rect.height;
            ej.datavisualization.Diagram.SvgContext.update(this.helper, this.diagram);
        };
        return PolygonToolBase;
    })(ej.datavisualization.Diagram.ToolBase);
    ej.datavisualization.Diagram.PolygonToolBase = PolygonToolBase;
    //#endregion

    //#region TextToolBase
    var TextToolBase = (function (base) {
        ej.datavisualization.Diagram.extend(TextToolBase, base);
        function TextToolBase(name, diagram) {
            base.call(this, name, diagram);
            this.boundingRect = ej.datavisualization.Diagram.Rectangle();
            this.activeLabel = null;
            this.cursor = "crosshair";
        }
        TextToolBase.prototype.mousedown = function (evt) {
            base.prototype.mousedown.call(this, evt);
            if (this.diagram._isEditing) {
                this.diagram._endEdit();
            }
        };
        TextToolBase.prototype.createNode = function (rect) {

        };
        TextToolBase.prototype.mousemove = function (evt) {
            base.prototype.mousemove.call(this, evt);
            if (ej.datavisualization.Diagram.Util.isPageEditable(this.diagram)) {
                if (!this.selectedObject && !ej.datavisualization.Diagram.Geometry.isEmptyPoint(this.startPoint) && !this.diagram._isEditing) {
                    if (!this.inAction) {
                        this.inAction = true;
                        this._initHelper();
                    }
                    else {
                        this._updateHelper();
                    }
                }
            }
        };
        TextToolBase.prototype.mouseup = function (evt) {
            this.activeLabel = null;
            var minHeight = 12;
            if (this.inAction) {
                var rect = this.getBoundingRect(this.startPoint, this.currentPoint);
                this._endAction();
                if (ej.browserInfo().name === "chrome" && (rect.width === 0 || rect.height === 0)) {
                    rect.width = 90;
                    rect.height = minHeight;
                }
                this.selectedObject = this.createNode(rect.x + (rect.width / 2), (rect.y + (rect.height / 2)), rect.width, rect.height);
                this.diagram._startEdit(this.selectedObject);
            }
            else {
                var node = this._findNodeUnderMouse(evt);
                if (node != null && ej.datavisualization.Diagram.Util.isPageEditable(this.diagram)) {
                    this.selectedObject = node;
                    this.diagram._startEdit(this.selectedObject);
                }
                else {
                    var rect = ej.datavisualization.Diagram.Rectangle(this.currentPoint.x - (90 / 2), this.currentPoint.y - (minHeight / 2), 90, minHeight);
                    this._endAction();
                    this.selectedObject = this.createNode(rect.x + (rect.width / 2), rect.y + (rect.height / 2), rect.width, minHeight);
                    this.diagram._startEdit(this.selectedObject);
                }
            }
        };
        TextToolBase.prototype.getBoundingRect = function (startPoint, currentPoint) {
            var width = Math.abs(this.startPoint.x - this.currentPoint.x);
            var height = Math.abs(this.startPoint.y - this.currentPoint.y);
            var x = (this.startPoint.x > this.currentPoint.x) ? this.currentPoint.x : this.startPoint.x;
            var y = (this.startPoint.y > this.currentPoint.y) ? this.currentPoint.y : this.startPoint.y;
            return { x: x, y: y, width: width, height: height };
        };
        TextToolBase.prototype._initHelper = function () {
            if (!this._svgHelper) {
                var g = this.diagram._svg.g();
                this.diagram._adornerLayer.appendChild(g);
                var scale = this.diagram._currZoom;
                var selectionRect = this.diagram._svg.rect({ "id": "helper", "x": this.startPoint.x * scale, "y": this.startPoint.y * scale, "fill": "transparent", "stroke": "gray", "stroke-dasharray": "2 2", "shape-rendering": "crispEdges" });
                g.appendChild(selectionRect);
                g.setAttribute("pointer-events", "none");
                this._svgHelper = g;
            }
        };
        TextToolBase.prototype._updateHelper = function () {
            var helper = this.diagram._svg.getElementById("helper");
            var width = Math.abs(this.startPoint.x - this.currentPoint.x);
            var height = Math.abs(this.startPoint.y - this.currentPoint.y);
            var scale = this.diagram._currZoom;
            var x = ((this.startPoint.x > this.currentPoint.x) ? this.currentPoint.x : this.startPoint.x) * scale;
            var y = ((this.startPoint.y > this.currentPoint.y) ? this.currentPoint.y : this.startPoint.y) * scale;
            if (helper) {
                this.diagram._svg.rect({ "id": helper.id, "x": x, "y": y, "width": width * scale, "height": height * scale });
            }
        };
        TextToolBase.prototype._endAction = function () {
            if (this._svgHelper) {
                this.diagram._adornerLayer.removeChild(this._svgHelper);
                this._svgHelper = null;
            }
            base.prototype._endAction.apply(this);
        };
        TextToolBase.prototype._findNodeUnderMouse = function (evt, skip) {
            var obj;
            var textElement = $(evt.target).parents("text")[0];
            if (textElement != null) {
                if (textElement.getAttribute("class") == "ej-d-label") {
                    var parentNode = $(textElement).parents("g")[0];
                    obj = this._findObj(parentNode);
                    if (obj.shape.type != "text") {
                        var label = textElement.id.split('_');
                        this.activeLabel = obj.labels.find(label[1]);
                    }
                }
            }
            else {
                var parents = $(evt.target).parents("g");
                if (parents.length > 3) {
                    var element = parents[parents.length - 4];
                    var id = element.getAttribute("id");
                    var type = element.getAttribute("class");
                    if (type) {
                        if (type === "ej-d-node" || type === "ej-d-group") {
                            obj = this.diagram._findNode(id);
                            var curlabel = this.diagram._findLabelAtPoint(this.currentPoint, obj);
                            this.activeLabel = curlabel;
                        }
                        else if (type === "ej-d-connector") {
                            obj = this.diagram._findConnector(id);
                            var curlabel = this.diagram._findLabelAtPoint(this.currentPoint, obj);
                            this.activeLabel = curlabel;
                        }
                    }
                }
            }
            return obj;
        };
        TextToolBase.prototype.getLabelUnderMouse = function () {
            return this.activeLabel;
        };
        return TextToolBase;
    })(ej.datavisualization.Diagram.ToolBase);
    ej.datavisualization.Diagram.TextToolBase = TextToolBase;
    //#endregion

    //#region RectangleTool
    var RectangleTool = (function (base) {
        ej.datavisualization.Diagram.extend(RectangleTool, base);
        function RectangleTool(diagram) {
            base.call(this, "rectangleTool", diagram);
        }
        RectangleTool.prototype.createNode = function () {
            var node = this.diagram._getNewNode({ name: "rectangle" + ej.datavisualization.Diagram.Util.randomId() });
            node.offsetX = 0;
            node.offsetY = 0;
            node.width = 0;
            node.height = 0;
            return node;
        };
        return RectangleTool;
    })(ej.datavisualization.Diagram.RectangleToolBase);
    ej.datavisualization.Diagram.RectangleTool = function (diagram) {
        return new RectangleTool(diagram);
    };
    //#endregion

    //#region EllipseTool
    var EllipseTool = (function (base) {
        ej.datavisualization.Diagram.extend(EllipseTool, base);
        function EllipseTool(diagram) {
            base.call(this, "ellipseTool", diagram);
        }
        EllipseTool.prototype.createNode = function () {
            var node = this.diagram._getNewNode({ name: "ellipse" + ej.datavisualization.Diagram.Util.randomId() });
            node.shape.type = ej.datavisualization.Diagram.Shapes.Ellipse;
            node.fillColor = "white";
            node.offsetX = 0;
            node.offsetY = 0;
            node.width = 0;
            node.height = 0;

            return node;
        };
        return EllipseTool;
    })(ej.datavisualization.Diagram.RectangleToolBase);
    ej.datavisualization.Diagram.EllipseTool = function (diagram) {
        return new EllipseTool(diagram);
    };
    //#endregion

    //#region RoundRectTool
    var RoundRectTool = (function (base) {
        ej.datavisualization.Diagram.extend(RoundRectTool, base);
        function RoundRectTool(diagram) {
            base.call(this, "roundRectTool", diagram);
        }
        RoundRectTool.prototype.createNode = function () {
            var node = this.diagram._getNewNode({ name: "roundrect" + ej.datavisualization.Diagram.Util.randomId() });
            node.shape.type = ej.datavisualization.Diagram.Shapes.Rectangle;
            node.shape.cornerRadius = 15;
            node.offsetX = 0;
            node.offsetY = 0;
            node.width = 0;
            node.height = 0;

            return node;
        };
        return RoundRectTool;
    })(ej.datavisualization.Diagram.RectangleToolBase);
    ej.datavisualization.Diagram.RoundRectTool = function (diagram) {
        return new RoundRectTool(diagram);
    };
    //#endregion

    //#region TextTool
    var TextTool = (function (base) {
        ej.datavisualization.Diagram.extend(TextTool, base);
        function TextTool(diagram) {
            base.call(this, "textTool", diagram);
        }
        TextTool.prototype.createNode = function (x, y, width, height) {
            var node = ej.datavisualization.Diagram.Node({ name: "text" + ej.datavisualization.Diagram.Util.randomId(), offsetX: x, offsetY: y, width: width, height: height, borderColor: "transparent", fillColor: "transparent", labels: [], ports: [], shape: { type: "text", textBlock: ej.datavisualization.Diagram.TextBlockDefaults } });
            return node;
        };
        return TextTool;
    })(ej.datavisualization.Diagram.TextToolBase);
    ej.datavisualization.Diagram.TextTool = function (diagram) {
        return new TextTool(diagram);
    };
    //#endregion

    //#region PolygonTool
    var PolygonTool = (function (base) {
        ej.datavisualization.Diagram.extend(PolygonTool, base);
        function PolygonTool(diagram) {
            base.call(this, "polygonTool", diagram);
        }
        PolygonTool.prototype.createNode = function () {
            var node = this.diagram._getNewNode({ name: "polygon" + ej.datavisualization.Diagram.Util.randomId() });
            node.shape.type = ej.datavisualization.Diagram.Shapes.Polygon;
            node.offsetX = 0;
            node.offsetY = 0;
            node.width = 0;
            node.height = 0;
            return node;
        };
        return PolygonTool;
    })(ej.datavisualization.Diagram.PolygonToolBase);
    ej.datavisualization.Diagram.PolygonTool = function (diagram) {
        return new PolygonTool(diagram);
    };;
    //#endregion

    //#region ImageTool
    var ImageTool = (function (base) {
        ej.datavisualization.Diagram.extend(ImageTool, base);
        function ImageTool(saveUrl, removeUrl, uploadfileUrl, autoOpenUpload, diagram) {
            base.call(this, "imageTool", diagram);
            this.saveUrl = saveUrl;
            this.removeUrl = removeUrl;
            this.uploadfileUrl = uploadfileUrl;
            this.autoOpenUpload = autoOpenUpload;
            this.bounds = ej.datavisualization.Diagram.Rectangle(0, 0, 0, 0);
            this.cursor = "crosshair";
        }
        ImageTool.prototype.createNode = function (imageUrl) {
            var node = this.diagram._getNewNode();
            node.name = "image" + ej.datavisualization.Diagram.Util.randomId();
            node.shape = ej.datavisualization.Diagram.Shapes.Image(imageUrl);
            node.borderDashArray = "5,5";
            node.offsetX = 0;
            node.offsetY = 0;
            node.width = 0;
            node.height = 0;

            return node;
        };
        ImageTool.prototype.mouseup = function (evt) {
            if (this.inAction)
                this.bounds = this.getBoundingRect(this.snap(this.startPoint), this.snap(this.currentPoint));
            this._renderDialogControl();
            this.diagram._toolToActivate = "selectTool";
            this.diagram._drawingTool = false;
            this._endAction();
        };
        ImageTool.prototype._showPopup = function () {
            this._renderDialogControl();
            this.diagram._toolToActivate = "selectTool";
            this.diagram._drawingTool = false;
            this._endAction();
        };
        ImageTool.prototype._renderDialogControl = function () {
            var dialogDiv = document.createElement("div");
            var attr = {
                "id": "basicDialog",
                "title": "Insert Image",
                "style": "display:none"
            };
            ej.datavisualization.Diagram.Util.attr(dialogDiv, attr);
            document.body.appendChild(dialogDiv);
            var uploadDiv = document.createElement("div");
            attr = {
                "id": "UploadDefault",
                "style": "padding-Left:15px; float:left"
            };
            ej.datavisualization.Diagram.Util.attr(uploadDiv, attr);
            var table = document.createElement("table");
            var tr = document.createElement("tr");
            var td = document.createElement("td");
            attr = {
                "align": "left",
                "style": "padding-Top:10px"
            }
            ej.datavisualization.Diagram.Util.attr(td, attr);
            var childDiv = document.createElement("div");
            childDiv.style.cssFloat = "left";
            var rdoComputer = document.createElement("input");
            attr = {
                "type": "radio",
                "name": "uploadOption",
                "id": "rdoComputer",
                "checked": "true"
            };
            ej.datavisualization.Diagram.Util.attr(rdoComputer, attr);
            rdoComputer.onchange = $.proxy(this._rdoChanged, this);
            childDiv.appendChild(rdoComputer);
            var label = document.createElement("label");
            label.innerHTML = "My Computer";
            label.className = "dialogLabel";
            attr = {
                "style": "font-weight:normal; padding-left:5px; vertical-align: top;"
            }
            ej.datavisualization.Diagram.Util.attr(label, attr);
            label.onclick = $.proxy(this._labelClick, this);
            childDiv.appendChild(label);
            td.style.paddingBottom = "5px";
            td.appendChild(childDiv);
            td.appendChild(uploadDiv);
            var label = document.createElement("label");
            attr = {
                "id": "lblComputerUrl",
                "style": "font-weight:normal;padding-left:10px; vertical-align: top;color:red;"
            };
            ej.datavisualization.Diagram.Util.attr(label, attr);
            td.appendChild(label);
            tr.appendChild(td);
            table.appendChild(tr);
            tr = document.createElement("tr");
            td = document.createElement("td");
            var rdoWebUrl = document.createElement("input");
            attr = {
                "id": "rdoWebUrl",
                "type": "radio",
                "name": "uploadOption"
            };
            ej.datavisualization.Diagram.Util.attr(rdoWebUrl, attr);
            rdoWebUrl.onchange = $.proxy(this._rdoChanged, this);
            td.appendChild(rdoWebUrl);
            var label = document.createElement("label");
            label.innerHTML = "Web URL";
            label.className = "dialogLabel";
            attr = {
                "style": "font-weight:normal; padding-left:5px; vertical-align: top;"
            };
            ej.datavisualization.Diagram.Util.attr(label, attr);
            label.onclick = $.proxy(this._labelClick, this);
            td.appendChild(label);
            var label = document.createElement("label");
            attr = {
                "id": "lblWebUrl",
                "style": "font-weight:normal;padding-left:25px; vertical-align: top;color:red;"
            };
            ej.datavisualization.Diagram.Util.attr(label, attr);
            td.appendChild(label);
            tr.appendChild(td);
            table.appendChild(tr);
            tr = document.createElement("tr");
            td = document.createElement("td");
            attr = {
                "style": "padding-Top:5px;padding-Bottom:15px"
            };
            ej.datavisualization.Diagram.Util.attr(td, attr);
            var txtImageUrl = document.createElement("input");
            attr = {
                "id": "txtImageUrl",
                "type": "text",
                "placeholder": "Image URL",
                "style": "width:460px;height:20px"
            };
            ej.datavisualization.Diagram.Util.attr(txtImageUrl, attr);
            txtImageUrl.onfocus = $.proxy(this._textFocus, this);
            txtImageUrl.onfocusout = $.proxy(this._textFocusOut, this);
            td.appendChild(txtImageUrl);
            tr.appendChild(td);
            table.appendChild(tr);
            tr = document.createElement("tr");
            td = document.createElement("td");
            var btnOk = document.createElement("input");
            attr = {
                "value": "Ok",
                "type": "button",
                "id": "btnOk",
                "style": "width : 100px; height: 30px; margin-Right: 5px; border: 1px solid #F7952E; color : white; font-Family:Segoe UI;font-weight:bold"
            };
            ej.datavisualization.Diagram.Util.attr(btnOk, attr);
            btnOk.onclick = $.proxy(this._uploadClick, this);
            td.appendChild(btnOk);
            var btnCancel = document.createElement("input");
            attr = {
                "value": "Cancel",
                "type": "button",
                "id": "btnCancel",
                "style": "width : 100px; height: 30px; border: 1px solid #DFDFDE; font-Family:Segoe UI;font-weight:bold"
            };
            ej.datavisualization.Diagram.Util.attr(btnCancel, attr);
            btnCancel.onclick = $.proxy(this._closeDialog, this);
            td.appendChild(btnCancel);
            attr = {
                "align": "right"
            };
            ej.datavisualization.Diagram.Util.attr(td, attr);
            tr.appendChild(td);
            table.appendChild(tr);
            dialogDiv.appendChild(table);
            $("#UploadDefault").ejUploadbox({
                saveUrl: this.saveUrl,
                removeUrl: this.removeUrl,
                complete: $.proxy(this.fileuploadComplete, this)
            });
            $("#basicDialog").ejDialog({ height: 230, width: 500, modal: true });
            $("#basicDialog").ejDialog("open");
        };
        ImageTool.prototype.fileuploadComplete = function (e, ui) {
            var fileName = this.uploadfileUrl + "/" + e.files.name;
            this._renderImageNode(fileName);
            this._closeDialog();
        };
        ImageTool.prototype._uploadClick = function () {
            var imageUrl = document.getElementById("txtImageUrl").value;
            if (/^(http:\/\/www\.|https:\/\/www\.)[a-z0-9]+([\-\.]{1}[a-z0-9]+)*\.[a-z]{2,5}(:[0-9]{1,5})?(\/.*)?$/.test(imageUrl)) {
                this._renderImageNode(imageUrl);
                this._closeDialog();
            }
            else {
                if (document.getElementById("rdoComputer").checked === true) {
                    var lblComputerUrl = document.getElementById("lblComputerUrl");
                    var lblWebUrl = document.getElementById("lblWebUrl");
                    lblComputerUrl.innerHTML = "Browse any image to insert";
                    lblWebUrl.innerHTML = "";
                }
                else {
                    var lblComputerUrl = document.getElementById("lblComputerUrl");
                    var lblWebUrl = document.getElementById("lblWebUrl");
                    lblComputerUrl.innerHTML = "";
                    lblWebUrl.innerHTML = "Invalid Url";
                    document.getElementById("txtImageUrl").focus();
                }
            }
        };
        ImageTool.prototype._rdoChanged = function () {
            var lblComputerUrl = document.getElementById("lblComputerUrl");
            var lblWebUrl = document.getElementById("lblWebUrl");
            lblComputerUrl.innerHTML = "";
            lblWebUrl.innerHTML = "";
        };
        ImageTool.prototype._closeDialog = function () {
            $("#basicDialog").ejDialog("close");
            var dialog = document.getElementById("basicDialog");
            dialog.parentNode.removeChild(dialog);
        };
        ImageTool.prototype._labelClick = function (evt) {
            if (evt.target.innerHTML == "My Computer") {
                var radio = document.getElementById("rdoComputer");
                radio.checked = true;
            }
            else {
                var radio = document.getElementById("rdoWebUrl");
                radio.checked = true;
            }
            this._rdoChanged();
        };
        ImageTool.prototype._textFocus = function (evt) {
            var radio = document.getElementById("rdoWebUrl");
            radio.checked = true;
        };
        ImageTool.prototype._renderImageNode = function (fileName) {
            var node = this.createNode(fileName);
            if (this.bounds.size.width != undefined && !ej.datavisualization.Diagram.Geometry.isEmptySize(this.bounds.size)) {
                node.offsetX = this.bounds.offset.x + this.bounds.size.width / 2;
                node.offsetY = this.bounds.offset.y + this.bounds.size.height / 2;
                node.width = this.bounds.size.width;
                node.height = this.bounds.size.height;
            }
            else {
                node.offsetX = 150;
                node.offsetY = 150;
                node.width = 100;
                node.height = 100;
            }
            node.borderDashArray = "";

            this.diagram.model.nodes.push(node);
            ej.datavisualization.Diagram.DiagramContext.renderNode(node, this.diagram);

            this.diagram._clearSelection();
            this.diagram._addSelection(node);
        };
        return ImageTool;
    })(ej.datavisualization.Diagram.RectangleToolBase);
    ej.datavisualization.Diagram.ImageTool = function (saveUrl, removeUrl, uploadfileUrl, autoOpenUpload, diagram) {
        return new ImageTool(saveUrl, removeUrl, uploadfileUrl, autoOpenUpload, diagram);
    };;
    //#endregion


    var PhaseTool = (function (base) {
        ej.datavisualization.Diagram.extend(PhaseTool, base);
        function PhaseTool(diagram) {
            base.call(this, "phase", diagram);
            this.selectedSeperetor = null;
            this._svgHelper = null;
        }

        PhaseTool.prototype.mousedown = function (evt) {
            base.prototype.mousedown.call(this, evt);
            this._inAction = true;
        };
        PhaseTool.prototype.mousemove = function (evt) {
            base.prototype.mousemove.call(this, evt);
            if (this._inAction) {
                if ($(evt.target).parents("g").first()[0]) {
                    var id = $(evt.target).parents("g").first()[0].id;
                    var phase = this._getphase(id);
                    if (phase && !this.selectedSeperetor) {
                        this.selectedSeperetor = phase;
                    }
                }
            }
            if (this.selectedSeperetor) {
                var dx = this.previousPoint.x - this.currentPoint.x;
                var dy = this.previousPoint.y - this.currentPoint.y;

                if (this.selectedSeperetor.orientation == "horizontal")
                    this.selectedSeperetor.offset -= dx;
                else
                    this.selectedSeperetor.offset -= dy;
                ej.datavisualization.Diagram.SvgContext._updatephase(this.selectedSeperetor, this.diagram);
            }
            this.previousPoint = this.currentPoint;
        };
        PhaseTool.prototype._getphase = function (id) {
            if (this.diagram.model.phases) {
                for (var i = 0; i < this.diagram.model.phases.length; i++) {
                    if (this.diagram.model.phases[i].name == id)
                        return this.diagram.model.phases[i];
                }
            }
        };
        PhaseTool.prototype.mouseup = function (evt) {
            base.prototype.mouseup.call(this, evt);
            this._inAction = false;
            this.selectedSeperetor = null;
        };
        return PhaseTool;
    })(ToolBase);
    ej.datavisualization.Diagram.PhaseTool = PhaseTool;
})(jQuery, Syncfusion);;
(function ($, ej) {
    "use strict";

    ej.datavisualization.Diagram.LayoutTypes = { None: "none", HierarchicalTree: "hierarchicaltree", OrganizationalChart: "organizationalchart" };

    ej.datavisualization.Diagram.Layout = {
        doLayout: function (diagram) {
            if (diagram.model.layout.type === "hierarchicaltree") {
                ej.datavisualization.Diagram.HierarchicalLayout.doLayout(diagram);
            }
            else if (diagram.model.layout.type === "organizationalchart") {
                ej.datavisualization.Diagram.HierarchicalLayout.doLayout(diagram, true);
            }
        },
    };

    //region Hierarchical layout
    ej.datavisualization.Diagram.HierarchicalLayout = {
        // #region Methods
        doLayout: function (diagram, findLevel) {
            var layout = {
                _mDepths: [], _mMaxDepth: 0, fixedNode: null, diagram: diagram, rootNode: null, _mAnchorX: 0, _mAnchorY: 0,
                _mEqualities: {}, _firstLevelNodes: [], level: null
            };
            var model = diagram.model;
            layout = $.extend(true, {}, layout, model.layout);
            if (model.nodes.length > 0) {
                layout._mEqualities = {};
                layout._firstLevelNodes = [];
                for (var i = 0; i < model.nodes.length; i++) {
                    var node = model.nodes[i];
                    var layoutInfo = layout._mEqualities[node.name] = this._setUpLayoutInfo(node);
                    layoutInfo.split = true;
                    layoutInfo.waterfall = true;
                    if (!node.inEdges || !node.inEdges.length) { layout._firstLevelNodes.push(node); }
                }
                if (findLevel) {
                    layout.level = Math.max(this._findLevel(layout, layout._firstLevelNodes[0], 1), 2);
                    layout.orientation = "toptobottom";
                }
                for (var i = 0; i < layout._firstLevelNodes.length; i++) {
                    var node = layout._firstLevelNodes[i];
                    this._updateEdges(layout, node, 1);
                }
                if (layout._firstLevelNodes.length > 0) {
                    layout._rootNode = layout._firstLevelNodes[0];
                    this._doLayout(layout);
                }
            }
        },

        _findLevel: function (layout, node, depth, min) {
            var layoutInfo = layout._mEqualities[node.name];

            var hasChild = true;
            if (node.outEdges && node.outEdges.length && node.isExpanded) {
                for (var j = 0; j < node.outEdges.length; j++) {
                    var edge = layout.diagram.nameTable[layout.diagram.nameTable[node.outEdges[j]].targetNode];
                    if (edge.outEdges && edge.outEdges.length && edge.isExpanded) {
                        var level = this._findLevel(layout, edge, depth + 1, min);
                        if (!min) min = level;
                        min = Math.min(min, level);
                        hasChild = false;
                    }
                    else return depth;
                }
                if (hasChild) return depth;
            }
            return min;
        },

        _updateEdges: function (layout, node, depth) {
            var layoutInfo = layout._mEqualities[node.name];
            if (node.outEdges && node.outEdges.length) {
                for (var j = 0; j < node.outEdges.length; j++) {
                    var edge = layout.diagram.nameTable[layout.diagram.nameTable[node.outEdges[j]].targetNode];
                    if (!edge.excludeFromLayout) {
                        if (depth < layout.level || layout.level === null) {
                            layoutInfo.children.push(edge.name);
                        }
                        else layoutInfo.waterfallChildren.push(edge.name);
                    }
                    if (node.outEdges.length > 1) {
                        if (edge.outEdges && edge.outEdges.length && edge.isExpanded) {
                            layoutInfo.split = false;
                            layoutInfo.waterfall = false;
                        }
                    }
                    this._updateEdges(layout, edge, depth + 1);
                }

            }
        },

        _getNextSibling: function (layout, node, edge, waterfall, i) {
            i = i ? i : 1;
            var node = layout._mEqualities[node.name];
            if (!waterfall) {
                if (node.children && node.children.length) {
                    var index = node.children.indexOf(edge);
                    if (index < node.children.length - i)
                        return layout.diagram.nameTable[node.children[index + i]];
                }
            }
            else {
                if (node.waterfallChildren && node.waterfallChildren.length) {
                    var index = node.waterfallChildren.indexOf(edge);
                    if (index < node.waterfallChildren.length - i)
                        return layout.diagram.nameTable[node.waterfallChildren[index + i]];
                }
            }
        },

        _getPrevSibling: function (layout, node, edge, waterfall) {
            var node = layout._mEqualities[node.name];
            if (!waterfall) {
                if (node.children && node.children.length) {
                    var index = node.children.indexOf(edge);
                    if (index > 0)
                        return layout.diagram.nameTable[node.children[index - 1]];
                }
            }
            else {
                if (node.waterfallChildren && node.waterfallChildren.length) {
                    var index = node.waterfallChildren.indexOf(edge);
                    if (index > 0)
                        return layout.diagram.nameTable[node.waterfallChildren[index - 1]];
                }
            }
        },

        _getLastChild: function (layout, node, waterfall) {
            var node = layout._mEqualities[node.name];
            if (!waterfall) {
                if (node.children && node.children.length)
                    return layout.diagram.nameTable[node.children[node.children.length - 1]];
            }
            else {
                if (node.waterfallChildren && node.waterfallChildren.length)
                    return layout.diagram.nameTable[node.waterfallChildren[node.waterfallChildren.length - 1]];
            }
        },

        _getFirstChild: function (layout, node, waterfall) {
            var node = layout._mEqualities[node.name];
            if (!waterfall) {
                if (node.children && node.children.length)
                    return layout.diagram.nameTable[node.children[0]];
            }
            else {
                if (node.waterfallChildren && node.waterfallChildren.length)
                    return layout.diagram.nameTable[node.waterfallChildren[0]];
            }
        },
        _getParentNode: function (layout, node) {
            if (node.inEdges && node.inEdges.length) {
                return layout.diagram.nameTable[layout.diagram.nameTable[node.inEdges[0]].sourceNode];
            }
        },

        _hasChild: function (layout, shape) {
            var shape = layout._mEqualities[shape.name];
            return shape.children && shape.children.length || shape.waterfallChildren && shape.waterfallChildren.length;
        },

        _getSpace: function (layout, l, r, siblings) {
            var hspace = 0;
            var stree = 0;
            var w = (layout.orientation == "toptobottom" || layout.orientation == "bottomtotop");
            hspace = layout.horizontalSpacing;
            stree = layout.horizontalSpacing;
            var info = layout._mEqualities[l.name];
            if (info.bounds) {
                return (siblings ? hspace : stree) + info.bounds.right;
            }
            return (siblings ? hspace : stree) + 0.5 * (w ? l.width + r.width : l.height + r.height);
        },

        _updateDepths: function (layout, depth, item) {
            var v = (layout.orientation == "toptobottom" || layout.orientation == "bottomtotop");
            var d = (v ? item.height : item.width);
            if (layout.orientation == "bottomtotop" || layout.orientation == "righttoleft") {
                d = -d;
                layout._mDepths[depth] = Math.min(layout._mDepths[depth] || 0, d);
            }
            else {
                layout._mDepths[depth] = Math.max(layout._mDepths[depth] || 0, d);
            }
            layout._mMaxDepth = Math.max(layout._mMaxDepth, depth);
        },

        _checkDepths: function (layout) {
            for (var i = 1; i <= layout._mMaxDepth; ++i) {
                var vspace = 0;
                vspace = layout.verticalSpacing;
                if (layout.orientation == "bottomtotop" || layout.orientation == "righttoleft")
                    vspace = -vspace;
                layout._mDepths[i] += (layout._mDepths[i - 1] || 0) + vspace;
            }
        },

        _doLayout: function (layout) {
            layout._mDepths = [];
            layout._mDepths[0] = 0;
            layout._mAnchorX = 0;
            layout._mAnchorY = 0;
            layout._mMaxDepth = 0;
            layout.diagram._layoutInAction = true;
            var fixedNode, fixedX, fixedY;

            if (layout._firstLevelNodes.length > 1) {
                this._doFirstWalk(layout, layout._firstLevelNodes[0], 0, 1, null, null, layout._firstLevelNodes[1]);
            }
            else {
                this._doFirstWalk(layout, layout._firstLevelNodes[0], 0, 1, null, null, null);
            }

            this._checkDepths(layout);
            var viewPort = ej.datavisualization.Diagram.ScrollUtil._viewPort(layout.diagram);
            if (layout.orientation == "toptobottom") {
                layout._mAnchorX = viewPort.width / 2 - layout._rootNode.width / 2 + layout.marginX;
                layout._mAnchorY = layout.marginY;
            }
            else if (layout.orientation == "bottomtotop") {
                layout._mAnchorX = viewPort.width / 2 - layout._rootNode.width / 2 + layout.marginX;
                layout._mAnchorY = viewPort.height - layout.marginY;
            }
            else if (layout.orientation == "lefttoright") {
                layout._mAnchorX = layout.marginX;
                layout._mAnchorY = viewPort.height / 2  + layout.marginY;
            }
            else if (layout.orientation == "righttoleft") {
                layout._mAnchorX = viewPort.width - layout.marginX;
                layout._mAnchorY = viewPort.height / 2  + layout.marginY;
            }
          
            if (layout.fixedNode) {
                fixedNode = layout.diagram.nameTable[layout.fixedNode];
                fixedX = fixedNode.offsetX;
                fixedY = fixedNode.offsetY;
            }

            var rp = layout._mEqualities[layout._firstLevelNodes[0].name];
            this._doSecondWalk(layout, layout._firstLevelNodes[0], null, -rp.prelim, 0);
            this._updateNodes(layout, fixedX || 0, fixedY || 0);
            this._updateConnectors(layout, layout._rootNode, 1);
            layout.diagram._layoutInAction = false;
        },
        _updateNodes: function (layout, fixedx, fixedy) {
            if (layout.fixedNode) {
                var fixedNode = layout.diagram.nameTable[layout.fixedNode];
                var delx = fixedx - fixedNode.offsetX;
                var dely = fixedy - fixedNode.offsetY;
            }
            else { delx = 0; dely = 0; }
            var nodes = layout.diagram.model.nodes;
            for (var i = 0; i < nodes.length; i++) {
                if (!nodes[i].excludeFromLayout) {
                    nodes[i].offsetX += delx;
                    nodes[i].offsetY += dely;
                }
                ej.datavisualization.Diagram.SpatialUtil._updateQuad(layout.diagram, layout.diagram._spatialSearch, nodes[i]);
                if (layout.diagram._svg)
                    ej.datavisualization.Diagram.DiagramContext.update(nodes[i], layout.diagram);
            }
        },
        _updateConnectors: function (layout, node, level) {
            for (var i = 0; i < node.outEdges.length; i++) {
                var conn = layout.diagram.nameTable[node.outEdges[i]];
                var target = layout.diagram.nameTable[conn.targetNode];
                if (layout.type == "organizationalchart") {
                    conn.sourcePoint = { x: node.offsetX, y: node.offsetY + node.height / 2 };
                    conn.targetPoint = { x: target.offsetX + target.width / 2, y: node.offsetY };
                    var info = layout._mEqualities[node.name];
                    if (!layout.level || level < layout.level) {
                        layout.diagram.updateConnector(conn.name, { segments: [{ type: "orthogonal" }] });
                    }
                    else {
                        if (target.offsetX == node.offsetX) {
                            conn.sourcePoint = { x: node.offsetX, y: node.offsetY + node.height / 2 };
                            conn.targetPoint = { x: target.offsetX, y: target.offsetY - target.height / 2 };
                            layout.diagram.updateConnector(conn.name, { segments: [{ type: "orthogonal" }] });
                        }
                        else {
                            var segments = [{ type: "orthogonal", length: target.offsetY - (node.offsetY + node.height / 2), direction: "bottom" },
                            { type: "orthogonal" }];
                            layout.diagram.updateConnector(conn.name, { segments: segments });
                        }
                    }
                    //layout.diagram._dock(conn, layout.diagram.nameTable);
                }
                layout.diagram._dock(conn, layout.diagram.nameTable);
                if (layout.diagram._svg)
                    ej.datavisualization.Diagram.DiagramContext.update(conn, layout.diagram);
                if (target.isExpanded || this._hasChild(layout, target))
                    this._updateConnectors(layout, target, level + 1);
            }
        },

        _doFirstWalk: function (layout, shape, number, depth, parent, prev, next) {
            var layoutInfo = layout._mEqualities[shape.name];
            layoutInfo.number = number;
            this._updateDepths(layout, depth, shape);
            var isExpanded = shape.isExpanded;
            if (!this._hasChild(layout, shape) || !isExpanded || depth == layout.level) {
                if (depth == layout.level) {
                    if (prev) {
                        var prevInfo = layout._mEqualities[prev.name];
                    }
                    var bounds = this._doWaterFallFirstWalk(layout, shape, null, null, 0, 0, prevInfo ? prevInfo.bounds : {});
                    layoutInfo.bounds = bounds;
                    layoutInfo.waterfallMod = layoutInfo.dimensions.x;
                    layoutInfo.prelim = 0;
                    if (prevInfo) {
                        layoutInfo.prelim += (prevInfo.prelim - prevInfo.mod) + prevInfo.bounds.right + layout.horizontalSpacing;
                    }
                }
                else {
                    var l = prev;
                    if (l == null) {
                        layoutInfo.prelim = 0;
                    }
                    else {
                        layoutInfo.prelim = layout._mEqualities[l.name].prelim + this._getSpace(layout, l, shape, true);
                    }
                }
            }
            else if (isExpanded) {
                var leftMostShape = this._getFirstChild(layout, shape);
                var rightMostShape = this._getLastChild(layout, shape);
                var defaultAncestor = leftMostShape;
                var c = leftMostShape;
                for (var i = 0; c != null; ++i, c = this._getNextSibling(layout, shape, c.name)) {
                    var p = this._getPrevSibling(layout, shape, c.name);
                    var n = this._getNextSibling(layout, shape, c.name);
                    this._doFirstWalk(layout, c, i, depth + 1, shape, p, n);
                    defaultAncestor = this._allocateSpace(layout, c, defaultAncestor, shape, p, n, depth);
                }

                this._translateShapePosition(layout, shape);

                var midpoint = 0.5 *
                    (layout._mEqualities[leftMostShape.name].prelim + layout._mEqualities[leftMostShape.name].waterfallMod +
                    layout._mEqualities[rightMostShape.name].prelim + layout._mEqualities[rightMostShape.name].waterfallMod);

                var left = null;
                if (layout._firstLevelNodes.indexOf(shape) != -1) {
                    if (layout._firstLevelNodes.indexOf(shape) > 0) {
                        left = layout._firstLevelNodes[layout._firstLevelNodes.indexOf(shape) - 1];
                    }
                }
                else {
                    if (this._getPrevSibling(layout, parent, shape.name))
                        left = this._getPrevSibling(layout, parent, shape.name);
                }
                if (left) {
                    layoutInfo.prelim = layout._mEqualities[left.name].prelim + this._getSpace(layout, left, shape, true);
                    layoutInfo.mod = layoutInfo.prelim - midpoint;
                }
                else {
                    layoutInfo.prelim = (midpoint);
                }
            }
            if (layout._firstLevelNodes.indexOf(shape) != -1 && layout._firstLevelNodes.length > 1)
                this._allocateSpace(layout, shape, layout._firstLevelNodes[0], null, prev, next);

            //finding the next root level node

            if (layout._firstLevelNodes.indexOf(shape) != -1 && layout._firstLevelNodes.length > 1) {
                var node = null;

                if (layout._firstLevelNodes.indexOf(shape) < layout._firstLevelNodes.length - 1) {
                    node = layout._firstLevelNodes[layout._firstLevelNodes.indexOf(shape) + 1];
                }
                if (node != null) {
                    var prevRoot, nextRoot;

                    if (layout._firstLevelNodes.indexOf(node) == 0) {
                        prevRoot = null;
                    }
                    else {
                        prevRoot = layout._firstLevelNodes[layout._firstLevelNodes.indexOf(node) - 1];
                    }

                    if (layout._firstLevelNodes.indexOf(node) == layout._firstLevelNodes.length - 1) {
                        nextRoot = null;
                    }
                    else {
                        nextRoot = layout._firstLevelNodes[layout._firstLevelNodes.indexOf(shape) + 1];
                    }

                    this._doFirstWalk(layout, node, 0, 1, null, prevRoot, nextRoot);
                }
            }
        },


        _setUpLayoutInfo: function (item) {
            var info = {};
            info.ancestor = item;
            info.number = -1;
            info.ancestor = info.thread = null;
            info.shift = 0;
            info.change = 0;
            info.mod = 0;
            info.children = [];
            info.waterfallChildren = [];
            info.waterfallMod = 0;
            return info;
        },

        _allocateSpace: function (layout, v, a, parent, prev, next, depth) {
            var w = prev;
            if (w != null) {
                var vip, vim, vop, vom;
                var sip, sim, sop, som;

                vip = vop = v;
                vim = w;
                vom = null;
                if (parent)
                    vom = this._getFirstChild(layout, parent);
                else {
                    vom = layout._firstLevelNodes[0];
                }
                sip = layout._mEqualities[vip.name].mod;
                sop = layout._mEqualities[vop.name].mod;
                sim = layout._mEqualities[vim.name].mod;
                som = layout._mEqualities[vom.name].mod;
                var nr = this._adjacentRight(layout, vim);
                var nl = this._adjacentLeft(layout, vip);
                while (nr != null && nl != null) {
                    vim = nr;
                    vip = nl;
                    vom = this._adjacentLeft(layout, vom) || vom;
                    vop = this._adjacentRight(layout, vop) || vop;
                    var parms = layout._mEqualities[vop.name];
                    parms.ancestor = v;
                    var shift = (layout._mEqualities[vim.name].prelim + sim) -
                        (layout._mEqualities[vip.name].prelim + sip) + this._getSpace(layout, vim, vip, false);
                    if (shift > 0) {
                        this._shiftSubTree(layout, this._ancestorShape(layout, vim, v, a), v, shift);
                        sip += shift;
                        sop += shift;
                    }
                    if (!vom || !vop)
                        break;
                    sim += layout._mEqualities[vim.name].mod;
                    sip += layout._mEqualities[vip.name].mod;
                    som += layout._mEqualities[vom.name].mod;
                    sop += layout._mEqualities[vop.name].mod;

                    nr = this._adjacentRight(layout, vim);
                    nl = this._adjacentLeft(layout, vip);
                    if (this._adjacentRight(layout, vop) == null || this._adjacentLeft(layout, vom) == null)
                        break;
                }
                if (nr != null && this._adjacentRight(layout, vop) == null) {
                    var vopp = layout._mEqualities[vop.name];
                    vopp.thread = nr;
                    vopp.mod += sim - sop;
                }
                if (nl != null && this._adjacentLeft(layout, vom) == null) {
                    var vomp = layout._mEqualities[vom.name];
                    vomp.thread = nl;
                    vomp.mod += sip - som;
                    a = v;
                }
            }
            return a;
        },

        _adjacentLeft: function (layout, shape) {
            var tempshape = null;
            if (shape.isExpanded) tempshape = this._getFirstChild(layout, shape);
            return (tempshape || layout._mEqualities[shape.name].thread);
        },

        _adjacentRight: function (layout, shape) {
            var tempShape = null;
            if (shape.isExpanded) tempShape = this._getLastChild(layout, shape);
            return (tempShape || layout._mEqualities[shape.name].thread);
        },

        _shiftSubTree: function (layout, shape, shape2, shift) {
            var wmp = layout._mEqualities[shape.name];
            var wpp = layout._mEqualities[shape2.name];
            var distance = wpp.number - wmp.number;
            wpp.change -= shift / distance;
            wmp.change += shift / distance;
            wpp.shift += shift;
            wpp.prelim += shift;
            wpp.mod += shift;
        },

        _translateShapePosition: function (layout, shape) {
            var shift = 0, change = 0;
            var c;
            var index;
            for (c = this._getLastChild(layout, shape), index = shape.outEdges.length - 1; c != null; c = this._getPrevSibling(layout, shape, c.name), index--) {
                var cp = layout._mEqualities[c.name];
                cp.prelim += shift;
                cp.mod += shift;
                change += cp.change;
                shift += cp.shift + change;
            }
        },

        _ancestorShape: function (layout, shape, shape1, adjucentShape) {
            var parentshape = this._getParentNode(layout, shape1);
            var shapeLayoutInfo = layout._mEqualities[shape.name];
            if (shapeLayoutInfo.ancestor != null && this._getParentNode(layout, shapeLayoutInfo.ancestor) == parentshape) {
                return shapeLayoutInfo.ancestor;
            }
            else {
                return adjucentShape;
            }
        },

        _doSecondWalk: function (layout, node, previousShape, space, depth) {
            var np = layout._mEqualities[node.name];
            if (layout.orientation == "lefttoright" || layout.orientation == "righttoleft") {
                this._setBreadthSpace(layout, node, layout._mDepths[depth], depth);
                this._setDepthSpace(layout, node, np.prelim + space);
            }
            else {
                this._setBreadthSpace(layout, node, np.prelim + space + (np.waterfallMod || 0) + node.width / 2, depth);
                this._setDepthSpace(layout, node, layout._mDepths[depth]);
            }
            if (node.isExpanded && this._hasChild(layout, node)) {
                depth += 1;
                var i = 0;
                if (depth < layout.level || layout.level === null) {
                    for (var c = this._getFirstChild(layout, node) ; c != null; c = this._getNextSibling(layout, node, c.name), i++) {
                        this._doSecondWalk(layout, c, node, space + np.mod, depth);
                    }
                }
                else {
                    for (var c = this._getFirstChild(layout, node, true) ; c != null; c = this._getNextSibling(layout, node, c.name, true), i++) {
                        this._waterFallSecondWalk(layout, c, node.offsetX - node.width / 2 - np.waterfallMod || 0, depth);
                    }
                }
            }
            if (layout._firstLevelNodes.indexOf(node) != -1) {
                var nextnode = null;

                if (layout._firstLevelNodes.indexOf(node) < layout._firstLevelNodes.length - 1) {
                    nextnode = layout._firstLevelNodes[layout._firstLevelNodes.indexOf(node) + 1];
                }
                if (nextnode != null) {
                    this._doSecondWalk(layout, nextnode, null, space, 0);
                }
            }

            //np.ClearInfo();
        },
        _waterFallSecondWalk: function (layout, shape, space, depth) {
            var layoutInfo = layout._mEqualities[shape.name];
            shape.offsetX = layoutInfo.dimensions.x + space + shape.width / 2 + layoutInfo.mod;
            shape.offsetY = layoutInfo.dimensions.y + shape.height / 2 + layout._mDepths[depth - 1];
            if (shape.isExpanded)
                for (var c = this._getFirstChild(layout, shape, true) ; c != null; c = this._getNextSibling(layout, shape, c.name, true)) {
                    this._waterFallSecondWalk(layout, c, space + layoutInfo.mod, depth);
                }
        },
        _setBreadthSpace: function (layout, shapeNext, space) {
            if (layout.orientation == "righttoleft") {
                this._setX(shapeNext, layout._mAnchorX + space - shapeNext.width / 2);
            }
            else
                if (layout.orientation == "lefttoright") {
                    this._setX(shapeNext, layout._mAnchorX + space + shapeNext.width / 2);
                }
                else {
                    this._setX(shapeNext, layout._mAnchorX + space);
                }
        },

        _setDepthSpace: function (layout, shapeNext, space) {
            if (layout.orientation == "bottomtotop") {
                this._setY(shapeNext, layout._mAnchorY + space - shapeNext.height / 2);
            }
            else if (layout.orientation == "toptobottom") {
                this._setY(shapeNext, layout._mAnchorY + space + shapeNext.height / 2);
            }
            else
                this._setY(shapeNext, layout._mAnchorY + space);
        },
        _setX: function (item, x) {
            item.offsetX = x;
        },
        _setY: function (item, y) {
            item.offsetY = y;
        },
        _doWaterFallFirstWalk: function (layout, shape, parent, bounds, x, y, limit) {
            if (!limit) limit = { x: 0, y: 0 };
            if (!layout._mEqualities[shape.name]) {
                var layoutInfo = {};
                this._setUpLayoutInfo(shape, layoutInfo);
                layout._mEqualities[shape.name] = layoutInfo;
            }
            else {
                var layoutInfo = layout._mEqualities[shape.name];
            }
            if (!parent) {
                var bounds = { x: 0, y: 0, right: 0, bottom: 0 };
                var initx = 0, inity = 0;
                layoutInfo.dimensions = { x: x, y: y };
            }
            else {
                var parentInfo = layout._mEqualities[parent.name];
                layoutInfo.dimensions = { x: x, y: y };
            }
            var leftMostShape = this._getFirstChild(layout, shape, true);
            var v = y + shape.height + layout.verticalSpacing / 2;
            var h;
            var c = leftMostShape;
            var ver = v;
            var leftChilds = [];
            if (shape.isExpanded && this._hasChild(layout, shape)) {
                if (layoutInfo.waterfallChildren.length > 1 && layoutInfo.waterfallChildren.length <= 2 || !layoutInfo.split) {
                    h = x + shape.width / 2 + 20;
                    for (var i = 0; c != null; i++, c = this._getNextSibling(layout, shape, c.name, true)) {
                        var bnds = this._doWaterFallFirstWalk(layout, c, shape, bounds, h, v, { x: 0, y: 0 });
                        v = bnds.bottom + layout.verticalSpacing / 2;
                        limit.x = Math.max(bnds.right, limit.x);
                        limit.y = Math.max(bnds.bottom, limit.y);
                        bounds.right = Math.max(bounds.right, bnds.right);
                        bounds.bottom = Math.max(bounds.bottom, bnds.bottom);
                    }
                }
                else {
                    for (var i = 0; c != null && (i < shape.outEdges.length) ; i = i + 2, c = this._getNextSibling(layout, shape, c.name, true, 2)) {
                        h = x;
                        var bnds = this._doWaterFallFirstWalk(layout, c, shape, bounds, h, v, { x: 0, y: 0 });
                        layout._mEqualities[c.name].bounds = bnds;
                        v = bnds.bottom + layout.verticalSpacing / 2;
                        leftChilds.push({ name: c.name, right: bnds.right });
                        limit.x = Math.max(bnds.right, limit.x);
                        limit.y = Math.max(bnds.bottom, limit.y);
                        bounds.right = Math.max(bounds.right, bnds.right);
                        bounds.bottom = Math.max(bounds.bottom, bnds.bottom);
                    }
                    if (leftChilds.length > 1) {
                        for (var i = 0; i < leftChilds.length; i++) {
                            layout._mEqualities[leftChilds[i].name].mod = limit.x - leftChilds[i].right;
                        }
                    }
                    layoutInfo.dimensions.x = shape.outEdges.length > 1 ? limit.x + 20 - shape.width / 2 : layout._mEqualities[leftMostShape.name].dimensions.x;
                    h = limit.x + 40;
                    v = ver;
                    c = this._getNextSibling(layout, shape, leftMostShape.name, true);
                    for (var i = 1; c != null && i < shape.outEdges.length; i = i + 2, c = this._getNextSibling(layout, shape, c.name, true, 2)) {
                        var bnds = this._doWaterFallFirstWalk(layout, c, shape, bounds, h, v, { x: 0, y: 0 });
                        v = bnds.bottom + layout.verticalSpacing / 2;
                        bounds.right = Math.max(bounds.right, bnds.right);
                        bounds.bottom = Math.max(bounds.bottom, bnds.bottom);
                        limit.x = Math.max(bnds.right, limit.x);
                        limit.y = Math.max(bnds.bottom, limit.y);
                    }
                }
                return !parent ? bounds : { x: 0, y: 0, right: limit.x, bottom: limit.y };
            }
            else return { x: x || 0, y: y || 0, right: (x || 0) + shape.width, bottom: (y || 0) + shape.height };
        }
        // #endregion

    };
    //end region
})(jQuery, Syncfusion);;

(function ($, ej) {
    "use strict";

    //#region ZoomAndPan
    ej.datavisualization.Diagram.ZoomUtil = {
        zoomPan: function (diagram, currentZoom, deltaX, deltaY, focusPoint, isZoom) {
            if (ej.datavisualization.Diagram.Util.canZooming(diagram)) {
                var matrix = ej.Matrix.identity();
                ej.Matrix.scale(matrix, diagram._currZoom, diagram._currZoom);
                ej.Matrix.translate(matrix, -diagram._hScrollOffset, -diagram._vScrollOffset);
                var newScale = diagram._currZoom * currentZoom;
                if (newScale > 30) {
                    currentZoom = 30 / diagram._currZoom;
                }
                else if (newScale < 0.25) {
                    currentZoom = 0.25 / diagram._currZoom;
                }
                var dMatrix = ej.Matrix.identity();
                var pivot;
                if (focusPoint) {
                    pivot = ej.Matrix.transform(matrix, focusPoint);
                }
                else {
                    pivot = ej.datavisualization.Diagram.Point($(diagram.element).width() / 2, $(diagram.element).height() / 2);
                }
                ej.Matrix.scale(dMatrix, currentZoom, currentZoom, pivot.x, pivot.y);
                newScale = diagram._currZoom * currentZoom;
                ej.Matrix.translate(dMatrix, deltaX, deltaY);
                ej.Matrix.multiply(matrix, dMatrix);
                var point = ej.Matrix.transform(matrix, ej.datavisualization.Diagram.Point(0, 0));
                this._applyTransform(diagram, newScale, point, isZoom);
            }
        },
        _applyTransform: function (diagram, currentZoom, point, isZoom) {
            diagram._currZoom = currentZoom;
            var zoomX = point.x * -1;
            var zoomY = point.y * -1;
            if (!isZoom) {
                if (diagram._scrollLimit() !== ej.datavisualization.Diagram.ScrollLimit.Infinity) {
                    if (zoomX < diagram._hScrollbar.model.minimum)
                        zoomX = diagram._hScrollbar.model.minimum;
                    else if (zoomX > diagram._hScrollbar.model.maximum) {
                        zoomX = diagram._hScrollbar.model.maximum;
                    }
                    if (zoomY < diagram._vScrollbar.model.minimum)
                        zoomY = diagram._vScrollbar.model.minimum;
                    else if (zoomY > diagram._vScrollbar.model.maximum) {
                        zoomY = diagram._vScrollbar.model.maximum;
                    }
                }
            }
            diagram._updateScrollOffset(zoomX, zoomY, true, isZoom);
        }
    };
    //#endregion 

    //#region PageSettings
    ej.datavisualization.Diagram.PageUtil = {
        _createPageBreaks: function (diagram) {
            this._removePageBreaks(diagram);
            var attr = { id: diagram._canvas.id + "pagebreaks", "pointer-events": "none" };
            var g = ej.datavisualization.Diagram.SvgContext._initializePageBreaks(diagram._svg, attr, diagram._pageBackgroundLayer);
            this._updatePageBreaks(g, diagram);
        },
        _removePageBreaks: function (diagram) {
            var g = diagram._svg.getElementById(diagram._canvas.id + "pagebreaks");
            if (g) { ej.datavisualization.Diagram.SvgContext._removePageBreaks(diagram._svg, g, diagram._pageBackgroundLayer); }
        },
        _updatePageBreaks: function (g, diagram) {
            var bounds = diagram._getDigramBounds();
            var pageSettings = diagram.model.pageSettings;
            var left = bounds.x;
            var right = bounds.width +bounds.x;
            var top = bounds.y;
            var bottom = bounds.height + bounds.y;
            var zoom = diagram._currZoom;
            var pHeight = diagram._pageHeight();
            var pWidth = diagram._pageWidth();
            var pMargin = diagram._pageMargin();
            var columncount = parseInt(Math.ceil((right - left) / pWidth));
            var rowcount = parseInt(Math.ceil((bottom - top) / pHeight));
            left = -diagram._canvas.clientLeft + (pMargin + left) * zoom;
            top = -diagram._canvas.clientTop + (pMargin + top) * zoom;
            var width = (pWidth * columncount - pMargin * 2) * zoom;
            var height = (pHeight * rowcount - pMargin * 2) * zoom;
            ej.datavisualization.Diagram.SvgContext._renderVPageBreakLine(ej.datavisualization.Diagram.Point(left, top), ej.datavisualization.Diagram.Point(left + width, top), diagram._svg, g);
            ej.datavisualization.Diagram.SvgContext._renderHPageBreakLine(ej.datavisualization.Diagram.Point(left, top), ej.datavisualization.Diagram.Point(left, top + height), diagram._svg, g);
            for (var column = 1; column < columncount; column++) {
                ej.datavisualization.Diagram.SvgContext._renderVPageBreakLine(ej.datavisualization.Diagram.Point(left + (pWidth * column - pMargin) * zoom, top),
                    ej.datavisualization.Diagram.Point(left + (pWidth * column - pMargin) * zoom, top + height), diagram._svg, g);
            }
            for (var row = 1; row < rowcount; row++) {
                ej.datavisualization.Diagram.SvgContext._renderHPageBreakLine(ej.datavisualization.Diagram.Point(left, top + (pHeight * row - pMargin) * zoom),
                    ej.datavisualization.Diagram.Point(left + width, top + (pHeight * row - pMargin) * zoom), diagram._svg, g);
            }
            ej.datavisualization.Diagram.SvgContext._renderVPageBreakLine(ej.datavisualization.Diagram.Point(left + width, top), ej.datavisualization.Diagram.Point(left + width, top + height), diagram._svg, g);
            ej.datavisualization.Diagram.SvgContext._renderHPageBreakLine(ej.datavisualization.Diagram.Point(left, top + height), ej.datavisualization.Diagram.Point(left + width, top + height), diagram._svg, g);
        },
        _updatePageSize: function (diagram) {
            var swap = false;
            if (diagram._pageOrientation() == ej.datavisualization.Diagram.PageOrientations.Landscape) {
                if (diagram._pageHeight() > diagram._pageWidth()) {
                    swap = true;
                }
            }
            else {
                if (diagram._pageWidth() > diagram._pageHeight()) {
                    swap = true;
                }
            }
            if (swap) {
                var temp = diagram._pageWidth();
                diagram._pageWidth(diagram._pageHeight());
                diagram._pageHeight(temp);
            }
            var bounds = diagram._getDigramBounds();
            var zoom = diagram._currZoom;
            var left = bounds.x * zoom;
            var top = bounds.y * zoom;
            var width = bounds.width * zoom;
            var height = bounds.height * zoom;
            var pageSettings = diagram.model.pageSettings;
            if (diagram._pageBackgroundLayer && diagram._pageBackgroundLayer.firstChild) {
                ej.datavisualization.Diagram.SvgContext._updatePageBakground(ej.datavisualization.Diagram.Point(left, top), ej.datavisualization.Diagram.Point(width, height), diagram._svg, diagram._pageBackgroundLayer, diagram._pageBorderColor(), diagram._pageBackgroundColor(), diagram._pageBorderWidth());
            }
            else {
                ej.datavisualization.Diagram.SvgContext._renderPageSettingsRect(ej.datavisualization.Diagram.Point(left, top), ej.datavisualization.Diagram.Point(width, height), diagram._svg, diagram._pageBackgroundLayer, diagram._pageBorderColor(), diagram._pageBackgroundColor(), diagram._pageBorderWidth());
            }
            diagram._svg.document.style.left = 0;
            diagram._svg.document.style.top = 0;
            var check = false;
            if (!diagram._pageWidth()) {
                if (!diagram._pageHeight()) {
                    check = true;
                }
            }
            if (pageSettings.pageWidth && diagram._pageHeight()) {
                if (diagram._showPageBreak()) {
                    this._createPageBreaks(diagram);
                }
                else {
                    this._removePageBreaks(diagram);
                }
            }
            ej.datavisualization.Diagram.ScrollUtil._setScrollContentSize(diagram);
        }
    };
    //#endregion

    //#region Scroller
    ej.datavisualization.Diagram.ScrollUtil = {
        _transform: function (diagram, hOffset, vOffset, canScale) {
            ej.datavisualization.Diagram.SvgContext.transformView(diagram, -hOffset, -vOffset);
            if (canScale) {
                ej.datavisualization.Diagram.SvgContext.scaleContent(diagram, diagram._currZoom);
            }
            ej.datavisualization.Diagram.PageUtil._updatePageSize(diagram);
            ej.datavisualization.Diagram.SvgContext._updateBackground(hOffset, vOffset, diagram._currZoom, diagram);
            ej.datavisualization.Diagram.SvgContext._updateGrid(hOffset, vOffset, diagram._currZoom, diagram);
        },
        _viewPort: function (diagram) {
            var element = diagram.element[0];
            var eWidth = $(element).width();
            var eHeight = $(element).height();
            var bRect = diagram.element[0].getBoundingClientRect();
            var screenX = (window.screenX < 0) ? window.screenX * -1 : window.screenX;
            if (eWidth === 0) {
                eWidth = Math.floor(((window.innerWidth - screenX) - Math.floor(bRect.left)));
            }
            var screenY = (window.screenY < 0) ? window.screenY * -1 : window.screenY;
            if (eHeight === 0) {
                eHeight = Math.floor(((window.innerHeight - screenY) - Math.floor(bRect.top)));
            }
            return ej.datavisualization.Diagram.Size(eWidth, eHeight);
        },

        _union: function (rect, rect1) {
            if (rect.width <= 0) {
                rect = rect1;
                return rect;
            }
            if (rect.width > 0) {
                var num = Math.min(rect.x, rect1.x);
                var num2 = Math.min(rect.y, rect1.y);
                if (rect1.width === Infinity || rect.width === Infinity) {
                    rect.width = Infinity;
                }
                else {
                    var num3 = Math.max(rect.x + rect.width, rect1.x + rect1.width);
                    rect.width = Math.max(num3 - num, 0);
                }
                if (rect1.height === Infinity || rect.height === Infinity) {
                    rect.height = Infinity;
                }
                else {
                    var num4 = Math.max(rect.y + rect.height, rect1.y + rect1.height);
                    rect.height = Math.max(num4 - num2, 0);
                }
                rect.x = num;
                rect.y = num2;
                return rect;
            }
        },

        _setScrollContentSize: function (diagram) {
            var scale = diagram._currZoom;
            var viewPort = this._viewPort(diagram);
            viewPort = ej.datavisualization.Diagram.Rectangle(diagram._hScrollOffset, diagram._vScrollOffset, viewPort.width, viewPort.height);
            var left = diagram._spatialSearch.pageLeft;
            var right = diagram._spatialSearch.pageRight;
            var top = diagram._spatialSearch.pageTop;
            var bottom = diagram._spatialSearch.pageBottom;
            if (left > right) {
                left = right = 0;
            }
            if (top > bottom) {
                top = bottom = 0;
            }

            if (diagram.model.pageSettings) {
                var pageWidth = diagram._pageWidth();
                var pageHeight = diagram._pageHeight();

                if (pageWidth > 0 && pageHeight > 0) {
                    if (diagram._multiplePage()) {
                        left = Math.floor(left / pageWidth) * pageWidth;
                        top = Math.floor(top / pageHeight) * pageHeight;
                        right = Math.ceil(right / pageWidth) * pageWidth;
                        bottom = Math.ceil(bottom / pageHeight) * pageHeight;
                    }
                    else {
                        left = 0;
                        top = 0;
                        right = pageWidth;
                        bottom = pageHeight;
                    }
                }
                else {
                    left = left < 0 ? left : 0;
                    top = top < 0 ? top : 0;
                    right = right > viewPort.width ? right : viewPort.width;
                    bottom = bottom > viewPort.height ? bottom : viewPort.height;
                }
            }
            left = left * scale;
            top = top * scale;
            right = right * scale;
            bottom = bottom * scale;
            
            var diagramArea = ej.datavisualization.Diagram.Rectangle(0, 0, 0, 0);
            diagramArea = this._union(diagramArea, ej.datavisualization.Diagram.Rectangle(0, 0, viewPort.width, viewPort.height));
            diagramArea = this._union(diagramArea, ej.datavisualization.Diagram.Geometry.rect([{ x: left, y: top }, { x: right, y: bottom }]));

            diagramArea.width -= viewPort.width;
            diagramArea.height -= viewPort.height;

            if (diagramArea.x > diagram._hScrollOffset) {
                var diff = diagramArea.x - diagram._hScrollOffset;
                diagramArea.x -= diff;
                diagramArea.width += diff;
            }
            if (diagram._hScrollOffset > diagramArea.x + diagramArea.width) {
                diagramArea.width = diagram._hScrollOffset - diagramArea.x;
            }

            var minimumX = diagramArea.x;
            var maximumX = Math.max(0, diagramArea.x + diagramArea.width);

            if (diagramArea.y > diagram._vScrollOffset) {
                var diff = diagramArea.y - diagram._vScrollOffset;
                diagramArea.y -= diff;
                diagramArea.height += diff;
            }
            if (diagram._vScrollOffset > diagramArea.y + diagramArea.height) {
                diagramArea.height = diagram._vScrollOffset - diagramArea.y;
            }
            var minimumY = diagramArea.y;
            var maximumY = Math.max(0, diagramArea.y + diagramArea.height);

            diagramArea.width = diagramArea.width < viewPort.width ? viewPort.width : diagramArea.width;
            diagramArea.height = diagramArea.height < viewPort.height ? viewPort.height : diagramArea.height;

            if (diagram._scrollLimit() === ej.datavisualization.Diagram.ScrollLimit.Limited) {
                var range = this._setMinMaxValues(diagram, minimumX, minimumY, maximumX, maximumY);
                minimumX = range.minX;
                minimumY = range.minY;
                maximumX = range.maxX;
                maximumY = range.maxY;
            }
            
            if (left >= diagram._hScrollOffset && right <= viewPort.width + diagram._hScrollOffset) {
                $("#" + diagram._canvas.id + "_hScrollbar").ejScrollbar("remove");
				diagram._hScrollbar.model.width = 0;
            }
            else {
                $("#" + diagram._canvas.id + "_hScrollbar").ejScrollbar({ width: viewPort.width - 18, viewportSize: viewPort.width - 18, maximum: maximumX, minimum: minimumX });
                $("#" + diagram._canvas.id + "_hScrollbar").ejScrollbar("scrollX", diagram._hScrollOffset);
                var hScroll = document.getElementById(diagram._canvas.id + "_hScrollbar");
                hScroll.style.marginTop = viewPort.height - 18 + "px";
            }

            if (top >= diagram._vScrollOffset && bottom <= viewPort.height + diagram._vScrollOffset) {
                $("#" + diagram._canvas.id + "_vScrollbar").ejScrollbar("remove");
				diagram._vScrollbar.model.height = 0;
            }
            else {
                $("#" + diagram._canvas.id + "_vScrollbar").ejScrollbar({ height: viewPort.height - 18, viewportSize: viewPort.height - 18, maximum: maximumY, minimum: minimumY });
                $("#" + diagram._canvas.id + "_vScrollbar").ejScrollbar("scrollY", diagram._vScrollOffset);
            }
            ej.datavisualization.Diagram.SvgContext.setSize(diagram._svg, diagramArea.width, diagramArea.height);
        },

        _setMinMaxValues: function (diagram, minX, minY, maxX, maxY) {
            var scrollableArea = diagram.model.pageSettings.scrollableArea;
            if (minX < scrollableArea.left)
                minX = scrollableArea.left;
            if (minY < scrollableArea.top)
                minY = scrollableArea.top;
            if (maxX > scrollableArea.right)
                maxX = scrollableArea.right;
            if (maxY > scrollableArea.bottom)
                maxY = scrollableArea.bottom;
            return { minX: minX, minY: minY, maxX: maxX, maxY: maxY }
        },
        
        _initScrollbar: function (diagram) {
            var canvas = diagram._canvas;
            var hScrollbar = document.getElementById(canvas.id + "_hScrollbar");
            var vScrollbar = document.getElementById(canvas.id + "_vScrollbar");
            var viewPort = this._viewPort(diagram);
            $(hScrollbar).ejScrollbar({
                orientation: ej.Scrollbar.Orientation.Horizontal,
                width: viewPort.width - 18,
                viewportSize: viewPort.width - 18,
                scroll: function (e) {
                    if (e.source != null) {
                        if (e.source == "thumb" || e.source == "button" || e.source == "key") {
                            if (e.scrollData != null) {
                                diagram._updateScrollOffset(e.scrollLeft, diagram._vScrollOffset);
                            }
                        }
                    }
                }
            });
            diagram._hScrollbar = $(hScrollbar).ejScrollbar("instance");

            $(vScrollbar).ejScrollbar({
                orientation: ej.Scrollbar.Orientation.Vertical,
                height: viewPort.height - 18,
                viewportSize: viewPort.height - 18,
                scroll: function (e) {
                    if (e.source != null) {
                        if (e.source == "thumb" || e.source == "button" || e.source == "key") {
                            if (e.scrollData != null) {
                                diagram._updateScrollOffset(diagram._hScrollOffset, e.scrollTop);
                            }
                        }
                    }
                }
            });
            diagram._vScrollbar = $(vScrollbar).ejScrollbar("instance");

        },
    };
    //#endregion

    //#region Snapping
    ej.datavisualization.Diagram.SnapUtil = {
        _canConsider: function (nameTable, selectedObject, target) {
            var consider = false;
            if (selectedObject.isGroup) {
                consider = !this._contains(nameTable, selectedObject, target);
            }
            else if (!selectedObject.segments) {
                consider = !(selectedObject == target);
            }
            return consider;
        },
        _contains: function (nameTable, group, child) {
            var name = child.name;
            var node = null;
            if (group.children.indexOf(name) >= 0)
                node = nameTable[name];
            if (node && !node.segments) {
                if (node.isGroup)
                    return this._contains(nameTable, node, child);
                else if (node == child)
                    return true;
            }
            return false;
        },
        _round: function (value, snapintervals, scale) {
            var cutoff = 0, i;
            for (i = 0; i < snapintervals.length; i++) {
                cutoff += snapintervals[i];
            }
            cutoff /= scale;
            var quotient = Math.floor(Math.abs(value) / cutoff);
            var bal = value % cutoff;
            var prev = quotient * cutoff;
            if (value >= 0) {
                for (i = 0; i < snapintervals.length ; i++) {
                    if (bal <= snapintervals[i] / scale) {
                        return prev + (bal < (snapintervals[i] / (2 * scale)) ? 0 : snapintervals[i] / scale);
                    }
                    else {
                        prev += snapintervals[i] / scale;
                        bal -= snapintervals[i] / scale;
                    }
                }
            }
            else {
                prev = prev * -1;
                for (i = snapintervals.length - 1; i >= 0; i--) {
                    if (Math.abs(bal) <= snapintervals[i] / scale) {
                        return prev - (Math.abs(bal) < (snapintervals[i] / (2 * scale)) ? 0 : snapintervals[i] / scale);
                    }
                    else {
                        prev -= snapintervals[i] / scale;
                        bal += snapintervals[i] / scale;
                    }
                }
            }
            return value;
        },
        _findNodes: function (spatialSearch, node, child, viewPort, nodesInView) {
            var nodes = [], quad, nd, bounds;
            var quads = ej.datavisualization.Diagram.SpatialUtil.findQuads(spatialSearch, viewPort);
            var childBounds = ej.datavisualization.Diagram.Util.bounds(child);
            for (var i = 0; i < quads.length; i++) {
                quad = quads[i];
                if (quad.objects.length > 0) {
                    for (var j = 0; j < quad.objects.length; j++) {
                        nd = quad.objects[j];
                        if (!nd.segments) {
                            bounds = ej.datavisualization.Diagram.Util.bounds(nd);
                            if (nodes.indexOf(nd) == -1 && ej.datavisualization.Diagram.Geometry.intersectsRect(childBounds, bounds))
                                nodes.push(nd);
                            if (nodesInView && nodesInView.indexOf(nd) && ej.datavisualization.Diagram.Geometry.intersectsRect(viewPort, bounds)) {
                                nodesInView.push(nd);
                            }
                        }
                    }
                }
            }
            return nodes;
        },
        _snapAngle: function (diagram, angle) {
            var snapAngle = diagram._snapAngle();
            var width = angle % snapAngle;
            if (width >= (snapAngle / 2)) {
                return angle + snapAngle - width;
            }
            else {
                return angle - width;
            }
        },
        _snapPoint: function (diagram, selectedObject, towardsLeft, towardsTop, del, endPoint, startPoint) {
            var snapSettings = diagram.model.snapSettings;
            var zoomFactor = diagram._currZoom;
            var offset = ej.datavisualization.Diagram.Point();
            var bounds = ej.datavisualization.Diagram.Util.bounds(selectedObject);
            var horizontallysnapped = { snapped: false, offset: 0 };
            var verticallysnapped = { snapped: false, offset: 0 };
            if (diagram._enableSnapToObject())
                this._snapObject(diagram, selectedObject, horizontallysnapped,
                    verticallysnapped, del, startPoint == endPoint);
            //original position
            var left = bounds.x + del.x;
            var top = bounds.y + del.y;
            var right = bounds.x + bounds.width + del.x;
            var bottom = bounds.y + bounds.height + del.y;
            //snapped positions
            var roundedRight = this._round(right, snapSettings.verticalGridLines.snapInterval, zoomFactor);
            var roundedLeft = this._round(left, snapSettings.verticalGridLines.snapInterval, zoomFactor);
            var roundedTop = this._round(top, snapSettings.horizontalGridLines.snapInterval, zoomFactor);
            var roundedBottom = this._round(bottom, snapSettings.horizontalGridLines.snapInterval, zoomFactor);
            //currentposition
            var currentright = bounds.x + bounds.width;
            var currentbottom = bounds.y + bounds.height;
            if (!horizontallysnapped.snapped) {
                if (snapSettings.snapConstraints & ej.datavisualization.Diagram.SnapConstraints.SnapToVerticalLines) {
                    if (Math.abs(del.x) >= 1) {
                        if (towardsLeft) {
                            if (Math.abs(roundedRight - currentright) > Math.abs(roundedLeft - bounds.x)) {
                                offset.x += roundedLeft - bounds.x;
                            } else
                                offset.x += roundedRight - currentright;
                        } else {
                            if (Math.abs(roundedRight - currentright) < Math.abs(roundedLeft - bounds.x)) {
                                offset.x += roundedRight - currentright;
                            } else
                                offset.x += roundedLeft - bounds.x;
                        }
                    }
                } else {
                    offset.x = endPoint.x - startPoint.x;
                }
            } else {
                if (diagram._enableSnapToObject())
                    offset.x = horizontallysnapped.offset;
                else
                    offset.x = endPoint.x - startPoint.x;
            }
            if (!verticallysnapped.snapped) {
                if (snapSettings.snapConstraints & ej.datavisualization.Diagram.SnapConstraints.SnapToHorizontalLines) {
                    if (Math.abs(del.y) >= 1) {
                        if (towardsTop) {
                            if (Math.abs(roundedBottom - currentbottom) > Math.abs(roundedTop - bounds.y)) {
                                offset.y += roundedTop - bounds.y;
                            }
                            else
                                offset.y += roundedBottom - currentbottom;
                        }
                        else {
                            if (Math.abs(roundedBottom - currentbottom) < Math.abs(roundedTop - bounds.y)) {
                                offset.y += roundedBottom - currentbottom;
                            }
                            else
                                offset.y += roundedTop - bounds.y;
                        }
                    }
                }
                else
                    offset.y = endPoint.y - startPoint.y;
            }
            else
                offset.y = verticallysnapped.offset;

            return offset;
        },
        _snapObject: function (diagram, selectedObject, hSnap, vSnap, del, ended) {
            var lengthX = null, lengthY = null;
            var hTarget, vTarget;
            var objectsAtLeft = [], objectsAtRight = [], objectsAtTop = [], objectsAtBottom = [];
            var bounds = ej.datavisualization.Diagram.Util.bounds(selectedObject);
            var scale = diagram._currZoom;
            var hoffset = diagram._hScrollOffset;
            var voffset = diagram._vScrollOffset;
            var snapObjDistance = diagram._snapObjectDistance();
            var viewPort = ej.datavisualization.Diagram.ScrollUtil._viewPort(diagram);
            var hIntersectRect = ej.datavisualization.Diagram.Rectangle(hoffset / scale, (bounds.y - 5), viewPort.width / scale,
                (bounds.height + 10));
            var vIntersectRect = ej.datavisualization.Diagram.Rectangle((bounds.x - 5), voffset / scale,
                (bounds.width + 10), viewPort.height / scale);
            viewPort = ej.datavisualization.Diagram.Rectangle(hoffset / scale, voffset / scale, viewPort.width / scale,
                viewPort.height / scale);
            var nodes = this._findNodes(diagram._spatialSearch, selectedObject, vIntersectRect, viewPort);
            var i, target, targetBounds;
            var nameTable = diagram.nameTable;
            for (i = 0; i < nodes.length; i++) {
                target = nodes[i];
                if (!target.segments && this._canConsider(nameTable, selectedObject, target)) {
                    targetBounds = ej.datavisualization.Diagram.Util.bounds(target);
                    if (targetBounds.y + targetBounds.height < bounds.y + del.y) {
                        objectsAtTop.push({ object: target, distance: Math.abs(bounds.y + del.y - targetBounds.y - targetBounds.height) });
                    }
                    else if (targetBounds.y > bounds.y + del.y + bounds.height) {
                        objectsAtBottom.push({ object: target, distance: Math.abs(bounds.y + del.y + bounds.height - targetBounds.y) });
                    }
                    if (lengthX == null || lengthX > Math.abs(targetBounds.y - bounds.y - del.y)) {
                        if (Math.abs(targetBounds.x + targetBounds.width / 2 - (bounds.x + bounds.width / 2 + del.x)) <= snapObjDistance) {
                            hTarget = this._createSnapObject(targetBounds, bounds, "centerX");
                            lengthX = Math.abs(targetBounds.y - bounds.y);
                        } else if (Math.abs(targetBounds.x - (bounds.x + del.x)) <= snapObjDistance) {
                            hTarget = this._createSnapObject(targetBounds, bounds, "left");
                            lengthX = Math.abs(targetBounds.y - bounds.y);
                        } else if (Math.abs(targetBounds.x + targetBounds.width - (bounds.x + bounds.width + del.x)) <= snapObjDistance) {
                            hTarget = this._createSnapObject(targetBounds, bounds, "right");
                            lengthX = Math.abs(targetBounds.y - bounds.y);
                        } else if (Math.abs(targetBounds.x + targetBounds.width - (bounds.x + del.x)) <= snapObjDistance) {
                            hTarget = this._createSnapObject(targetBounds, bounds, "leftRight");
                            lengthX = Math.abs(targetBounds.y - bounds.y);
                        } else if (Math.abs(targetBounds.x - (bounds.x + bounds.width + del.x)) <= snapObjDistance) {
                            hTarget = this._createSnapObject(targetBounds, bounds, "rightLeft");
                            lengthX = Math.abs(targetBounds.y - bounds.y);
                        }
                    }
                }
            }
            nodes = this._findNodes(diagram._spatialSearch, selectedObject, hIntersectRect, viewPort);
            for (i = 0; i < nodes.length; i++) {
                target = nodes[i];
                if (!target.segments && this._canConsider(nameTable, selectedObject, target)) {
                    targetBounds = ej.datavisualization.Diagram.Util.bounds(target);
                    if (targetBounds.x + targetBounds.width < bounds.x + del.x) {
                        objectsAtLeft[objectsAtLeft.length] = { object: target, distance: Math.abs((bounds.x + del.x) - targetBounds.x - targetBounds.width) };
                    }
                    if (targetBounds.x > bounds.x + del.x + bounds.width) {
                        objectsAtRight[objectsAtRight.length] = { object: target, distance: Math.abs(bounds.x + del.x + bounds.width - targetBounds.x) };
                    }
                    if (lengthY == null || lengthY > Math.abs(targetBounds.x - bounds.x - del.x)) {
                        if (Math.abs(targetBounds.y + targetBounds.height / 2 - (bounds.y + bounds.height / 2 + del.y)) <= snapObjDistance) {
                            vTarget = this._createSnapObject(targetBounds, bounds, "centerY");
                            lengthY = Math.abs(targetBounds.x - bounds.x);
                        } else if (Math.abs(targetBounds.y - bounds.y - del.y) <= snapObjDistance) {
                            vTarget = this._createSnapObject(targetBounds, bounds, "top");
                            lengthY = Math.abs(targetBounds.x - bounds.x);
                        } else if (Math.abs(targetBounds.y + targetBounds.height - (bounds.y + bounds.height + del.y)) <= snapObjDistance) {
                            vTarget = this._createSnapObject(targetBounds, bounds, "bottom");
                            lengthY = Math.abs(targetBounds.x - bounds.x);
                        } else if (Math.abs(targetBounds.y + targetBounds.height - bounds.y - del.y) <= snapObjDistance) {
                            vTarget = this._createSnapObject(targetBounds, bounds, "topBottom");
                            lengthY = Math.abs(targetBounds.x - bounds.x);
                        } else if (Math.abs(targetBounds.y - (bounds.y + bounds.height + del.y)) <= snapObjDistance) {
                            vTarget = this._createSnapObject(targetBounds, bounds, "bottomTop");
                            lengthY = Math.abs(targetBounds.x - bounds.x);
                        }
                    }
                }
            }
            var g = this._createGuidelines(diagram, hTarget, vTarget, hSnap, vSnap, ended);
            if (!hSnap.snapped) {
                this._createHSpacingLines(diagram, selectedObject, objectsAtLeft, objectsAtRight, hSnap, vSnap, ended, g, del, snapObjDistance);
            }
            if (!vSnap.snapped) {
                this._createVSpacingLines(diagram, selectedObject, objectsAtTop, objectsAtBottom, hSnap, vSnap, ended, g, del, snapObjDistance);
            }
        },
        _snapSize: function (diagram, hSnap, vSnap, delx, dely, selectedObject, ended) {
            var lengthX = null, lengthY = null;
            var hTarget;
            var vTarget;
            var bounds = ej.datavisualization.Diagram.Util.bounds(selectedObject);
            var nameTable = diagram.nameTable;
            var sameWidth = [];
            var sameHeight = [];
            var scale = diagram._currZoom;
            var hoffset = diagram._hScrollOffset;
            var voffset = diagram._vScrollOffset;
            var snapObjDistance = diagram._snapObjectDistance();
            var viewPort = ej.datavisualization.Diagram.ScrollUtil._viewPort(diagram);
            var hintersectedrect = ej.datavisualization.Diagram.Rectangle(hoffset / scale, (bounds.y - 5) / scale,
                viewPort.width / scale, (bounds.height + 10) / scale);
            var vintersectedrect = ej.datavisualization.Diagram.Rectangle((bounds.x - 5) / scale, voffset / scale,
                (bounds.width + 10) / scale, viewPort.height / scale);
            viewPort = ej.datavisualization.Diagram.Rectangle(hoffset / scale, voffset / scale, viewPort.width / scale,
               viewPort.height / scale);
            var nodesInView = [];
            var nodes = this._findNodes(diagram._spatialSearch, selectedObject, vintersectedrect, viewPort, nodesInView);
            var i, target, targetBounds;
            for (i = 0; i < nodes.length; i++) {
                target = nodes[i];
                if (this._canConsider(nameTable, selectedObject, target) && !target.segments) {
                    targetBounds = ej.datavisualization.Diagram.Util.bounds(target);
                    if (lengthX == null || lengthX > Math.abs(targetBounds.y - bounds.y)) {
                        if (hSnap.left) {
                            if (Math.abs(bounds.x + delx - targetBounds.x) <= snapObjDistance) {
                                hTarget = this._createSnapObject(targetBounds, bounds, "left");
                                lengthX = Math.abs(targetBounds.y - bounds.y);
                            }
                            else if (Math.abs(bounds.x + delx - targetBounds.x - targetBounds.width) <= snapObjDistance) {
                                hTarget = this._createSnapObject(targetBounds, bounds, "leftRight");
                                lengthX = Math.abs(targetBounds.y - bounds.y);
                            }
                        }
                        else if (hSnap.right) {
                            if (Math.abs(bounds.x + delx + bounds.width - targetBounds.x - targetBounds.width) <= snapObjDistance) {
                                hTarget = this._createSnapObject(targetBounds, bounds, "right");
                                lengthX = Math.abs(targetBounds.y - bounds.y);
                            }
                            else if (Math.abs(bounds.x + delx + bounds.width - targetBounds.x) <= snapObjDistance) {
                                hTarget = this._createSnapObject(targetBounds, bounds, "rightLeft");
                                lengthX = Math.abs(targetBounds.y - bounds.y);
                            }
                        }
                    }
                }
            }
            nodes = this._findNodes(diagram._spatialSearch, selectedObject, hintersectedrect, viewPort);
            for (i = 0; i < nodes.length; i++) {
                target = nodes[i];
                if (this._canConsider(nameTable, selectedObject, target) && !target.segments) {
                    targetBounds = ej.datavisualization.Diagram.Util.bounds(target);
                    if (lengthY == null || lengthY > Math.abs(targetBounds.x - bounds.x)) {
                        if (vSnap.top) {
                            if (Math.abs(bounds.y + dely - targetBounds.y) <= snapObjDistance) {
                                vTarget = this._createSnapObject(targetBounds, bounds, "top");
                                lengthY = Math.abs(targetBounds.x - bounds.x);
                            }
                            else if (Math.abs(bounds.y + dely - targetBounds.y - targetBounds.height) <= snapObjDistance) {
                                vTarget = this._createSnapObject(targetBounds, bounds, "topBottom");
                                lengthY = Math.abs(targetBounds.x - bounds.x);
                            }
                        }
                        else if (vSnap.bottom) {
                            if (Math.abs(bounds.y + bounds.height + dely - targetBounds.y - targetBounds.height) <= snapObjDistance) {
                                vTarget = this._createSnapObject(targetBounds, bounds, "bottom");
                                lengthY = Math.abs(targetBounds.x - bounds.x);
                            }
                            else if (Math.abs(bounds.y + bounds.height + dely - targetBounds.y) <= snapObjDistance) {
                                vTarget = this._createSnapObject(targetBounds, bounds, "bottomTop");
                                lengthY = Math.abs(targetBounds.x - bounds.x);
                            }
                        }
                    }
                }
            }
            for (i = 0; i < nodesInView.length; i++) {
                target = nodesInView[i];
                if (this._canConsider(nameTable, selectedObject, target)) {
                    var targetbounds = ej.datavisualization.Diagram.Util.bounds(target);
                    var del = hSnap.left ? -delx : delx;
                    var diff = Math.abs(bounds.width + del - targetbounds.width);
                    var actualDiff;
                    if (diff <= snapObjDistance) {
                        actualDiff = hSnap.left ? -targetbounds.width + bounds.width : targetbounds.width - bounds.width;
                        sameWidth[sameWidth.length] = { source: target, diff: diff, offset: actualDiff };
                    }
                    del = vSnap.top ? -dely : dely;
                    var dify = Math.abs(bounds.height + del - targetbounds.height);
                    if (dify <= snapObjDistance) {
                        actualDiff = vSnap.top ? -targetbounds.height + bounds.height : targetbounds.height - bounds.height;
                        sameHeight[sameHeight.length] = { source: target, diff: dify, offset: actualDiff };
                    }
                }
            }
            var g = this._createGuidelines(diagram, hTarget, vTarget, hSnap, vSnap, ended);
            if (!hSnap.snapped && sameWidth.length > 0 && (hSnap.left || hSnap.right)) {
                this._addSameWidthLines(diagram, sameWidth, hSnap, ended, g, selectedObject);
            }
            if (!vSnap.snapped && sameHeight.length > 0 && (vSnap.top || vSnap.bottom)) {
                this._addSameHeightLines(diagram, sameHeight, vSnap, ended, g, selectedObject);
            }
        },
        _createSnapObject: function (targetbounds, bounds, snap) {
            var snapObject;
            switch (snap) {
                case "left":
                    snapObject = {
                        start: ej.datavisualization.Diagram.Point(targetbounds.x, Math.min(targetbounds.y, bounds.y)),
                        end: ej.datavisualization.Diagram.Point(targetbounds.x, Math.max(targetbounds.y + targetbounds.height, bounds.y + bounds.height)),
                        offsetX: targetbounds.x - bounds.x, offsetY: 0, type: "sideAlign"
                    };
                    break;
                case "right":
                    snapObject = {
                        start: ej.datavisualization.Diagram.Point(targetbounds.x + targetbounds.width, Math.min(targetbounds.y, bounds.y)),
                        end: ej.datavisualization.Diagram.Point(targetbounds.x + targetbounds.width, Math.max(targetbounds.y + targetbounds.height, bounds.y + bounds.height)),
                        offsetX: targetbounds.x + targetbounds.width - bounds.x - bounds.width, offsetY: 0, type: "sideAlign"
                    };
                    break;
                case "top":
                    snapObject = {
                        start: ej.datavisualization.Diagram.Point(Math.min(targetbounds.x, bounds.x), targetbounds.y),
                        end: ej.datavisualization.Diagram.Point(Math.max(targetbounds.x + targetbounds.width, bounds.x + bounds.width), targetbounds.y),
                        offsetY: targetbounds.y - bounds.y, offsetX: 0, type: "sideAlign"
                    };
                    break;
                case "bottom":
                    snapObject = {
                        start: ej.datavisualization.Diagram.Point(Math.min(targetbounds.x, bounds.x), targetbounds.y + targetbounds.height),
                        end: ej.datavisualization.Diagram.Point(Math.max(targetbounds.x + targetbounds.width, bounds.x + bounds.width), targetbounds.y + targetbounds.height),
                        offsetY: targetbounds.y + targetbounds.height - bounds.y - bounds.height, offsetX: 0, type: "sideAlign"
                    };
                    break;
                case "topBottom":
                    snapObject = {
                        start: ej.datavisualization.Diagram.Point(Math.min(targetbounds.x, bounds.x), targetbounds.y + targetbounds.height),
                        end: ej.datavisualization.Diagram.Point(Math.max(targetbounds.x + targetbounds.width, bounds.x + bounds.width), targetbounds.y + targetbounds.height),
                        offsetY: targetbounds.y + targetbounds.height - bounds.y, offsetX: 0, type: "sideAlign"
                    };
                    break;
                case "bottomTop":
                    snapObject = {
                        start: ej.datavisualization.Diagram.Point(Math.min(targetbounds.x, bounds.x), targetbounds.y),
                        end: ej.datavisualization.Diagram.Point(Math.max(targetbounds.x + targetbounds.width, bounds.x + bounds.width), targetbounds.y),
                        offsetY: targetbounds.y - bounds.y - bounds.height, offsetX: 0, type: "sideAlign"
                    };
                    break;
                case "leftRight":
                    snapObject = {
                        start: ej.datavisualization.Diagram.Point(targetbounds.x + targetbounds.width, Math.min(targetbounds.y, bounds.y)),
                        end: ej.datavisualization.Diagram.Point(targetbounds.x + targetbounds.width, Math.max(targetbounds.y + targetbounds.height, bounds.y + bounds.height)),
                        offsetX: targetbounds.x + targetbounds.width - bounds.x, offsetY: 0, type: "sideAlign"
                    };
                    break;
                case "rightLeft":
                    snapObject = {
                        start: ej.datavisualization.Diagram.Point(targetbounds.x, Math.min(targetbounds.y, bounds.y)),
                        end: ej.datavisualization.Diagram.Point(targetbounds.x, Math.max(targetbounds.y + targetbounds.height, bounds.y + bounds.height)),
                        offsetX: targetbounds.x - bounds.x - bounds.width, offsetY: 0, type: "sideAlign"
                    };
                    break;
                case "centerX":
                    snapObject = {
                        start: ej.datavisualization.Diagram.Point(targetbounds.x + targetbounds.width / 2, Math.min(targetbounds.y, bounds.y)),
                        end: ej.datavisualization.Diagram.Point(targetbounds.x + targetbounds.width / 2, Math.max(targetbounds.y + targetbounds.height, bounds.y + bounds.height)),
                        offsetX: targetbounds.x + targetbounds.width / 2 - (bounds.x + bounds.width / 2), offsetY: 0, type: "centerAlign"
                    };
                    break;
                case "centerY":
                    snapObject = {
                        start: ej.datavisualization.Diagram.Point(Math.min(targetbounds.x, bounds.x), targetbounds.y + targetbounds.height / 2),
                        end: ej.datavisualization.Diagram.Point(Math.max(targetbounds.x + targetbounds.width, bounds.x + bounds.width), targetbounds.y + targetbounds.height / 2),
                        offsetY: targetbounds.y + targetbounds.height / 2 - (bounds.y + bounds.height / 2), offsetX: 0, type: "centerAlign"
                    };
                    break;
            }
            return snapObject;
        },
        _createGuidelines: function (diagram, hTarget, vTarget, hsnap, vsnap, ended) {
            var attr = { id: diagram._canvas.id + "guideline", "pointer-events": "none", "style": "pointer-events:none;" };
            var g = ej.datavisualization.Diagram.SvgContext._initializeGuidelines(diagram._svg, attr, diagram._adornerLayer);
            var scale = diagram._currZoom;
            var hOffset = diagram._hScrollOffset;
            var vOffset = diagram._vScrollOffset;
            if (hTarget) {
                hsnap.offset = hTarget.offsetX;
                hsnap.snapped = true;
                if (!ended) {
                    if (hTarget.type == "sideAlign") {
                        ej.datavisualization.Diagram.SvgContext._renderSideAlignmentLines(hTarget.start, hTarget.end, g, diagram._svg, scale, hOffset, vOffset);
                    }
                    else if (hTarget.type == "centerAlign") {
                        ej.datavisualization.Diagram.SvgContext._renderCenterAlignmentLines(hTarget.start, hTarget.end, g, diagram._svg, scale, hOffset, vOffset);
                    }
                }
            }
            if (vTarget) {
                vsnap.offset = vTarget.offsetY;
                vsnap.snapped = true;
                if (!ended) {
                    if (vTarget.type == "sideAlign") {
                        ej.datavisualization.Diagram.SvgContext._renderSideAlignmentLines(vTarget.start, vTarget.end, g, diagram._svg, scale, hOffset, vOffset);
                    }
                    else if (vTarget.type == "centerAlign") {
                        ej.datavisualization.Diagram.SvgContext._renderCenterAlignmentLines(vTarget.start, vTarget.end, g, diagram._svg, scale, hOffset, vOffset);
                    }
                }
            }
            return g;
        },
        _removeGuidelines: function (diagram) {
            var g = diagram._svg.getElementById(diagram._canvas.id + "guideline");
            if (g != null) { ej.datavisualization.Diagram.SvgContext._removeGuidelines(diagram._adornerLayer, g); }
        },
        _createHSpacingLines: function (diagram, shape, objectsAtLeft, objectsAtRight, hSnap, vSnap, ended, g, del, snapObjDistance) {
            var top = null;
            objectsAtLeft.sort(function (a, b) { return a.distance - b.distance; });
            objectsAtRight.sort(function (a, b) { return a.distance - b.distance; });
            var equallySpaced = [];
            var bounds = ej.datavisualization.Diagram.Util.bounds(shape);
            var nearestleft, nearestright;
            var targetBounds;
            if (objectsAtLeft.length > 0) {
                equallySpaced[equallySpaced.length] = objectsAtLeft[0];
                nearestleft = ej.datavisualization.Diagram.Util.bounds(objectsAtLeft[0].object);
                top = nearestleft.y;
                if (objectsAtLeft.length > 1) {
                    targetBounds = ej.datavisualization.Diagram.Util.bounds(objectsAtLeft[1].object);
                    var equaldistance = nearestleft.x - targetBounds.x - targetBounds.width;
                    if (Math.abs(equaldistance - objectsAtLeft[0].distance) <= snapObjDistance) {
                        top = this._findEquallySpacedNodesAtLeft(objectsAtLeft, equaldistance, top, equallySpaced);

                    } else equaldistance = objectsAtLeft[0].distance;
                } else equaldistance = objectsAtLeft[0].distance;
            }
            equallySpaced.sort(function (a, b) { return b.distance - a.distance; });
            equallySpaced[equallySpaced.length] = { object: shape, distance: 0 };
            top = bounds.y < top || !top ? bounds.y : top;
            if (objectsAtRight.length > 0) {
                var dist;
                nearestright = ej.datavisualization.Diagram.Util.bounds(objectsAtRight[0].object);
                top = nearestright.y < top ? nearestright.y : top;
                if (objectsAtRight.length > 1) {
                    targetBounds = ej.datavisualization.Diagram.Util.bounds(objectsAtRight[1].object);
                    dist = targetBounds.x - nearestright.x - nearestright.width;
                }
                if (objectsAtLeft.length > 0) {
                    if (Math.abs(objectsAtRight[0].distance - objectsAtLeft[0].distance) <= snapObjDistance) {
                        var adjustablevalue = Math.abs(objectsAtRight[0].distance - objectsAtLeft[0].distance) / 2;
                        (objectsAtRight[0].distance < objectsAtLeft[0].distance) ?
                        equaldistance -= adjustablevalue : equaldistance += adjustablevalue;
                        equallySpaced[equallySpaced.length] = objectsAtRight[0];
                    }
                    else if (objectsAtLeft.length == 1) {
                        nearestleft = undefined;
                        equallySpaced.splice(0, 1);
                        equallySpaced[equallySpaced.length] = objectsAtRight[0];
                        equaldistance = dist;
                    }
                }
                else {
                    equaldistance = dist;
                    equallySpaced[equallySpaced.length] = objectsAtRight[0];
                }
                if (objectsAtRight.length > 1 && nearestright.x + nearestright.width < targetBounds.x) {
                    top = this._findEquallySpacedNodesAtRight(objectsAtRight, dist, top, equallySpaced, snapObjDistance);
                }
            }
            if (equallySpaced.length > 2) {
                this._addHSpacingLines(diagram, equallySpaced, ended, g, top);
                var delta = 0;
                if (ended)
                    delta = del.x;
                if (nearestleft)
                    hSnap.offset = equaldistance - Math.abs(bounds.x + delta - nearestleft.x - nearestleft.width) + delta;
                else if (nearestright)
                    hSnap.offset = Math.abs(bounds.x + bounds.width + delta - nearestright.x) - equaldistance + delta;
                hSnap.snapped = true;
            }
        },
        _createVSpacingLines: function (diagram, shape, objectsAtTop, objectsAtBottom, hSnap, vSnap, ended, g, del, snapObjDistance) {
            var right = null;
            objectsAtTop.sort(function (a, b) { return a.distance - b.distance; });
            objectsAtBottom.sort(function (a, b) { return a.distance - b.distance; });
            var equallySpaced = [];
            var bounds = ej.datavisualization.Diagram.Util.bounds(shape);
            var nearesttop, nearestbottom;
            var targetBounds;
            if (objectsAtTop.length > 0) {
                equallySpaced[equallySpaced.length] = objectsAtTop[0];
                nearesttop = ej.datavisualization.Diagram.Util.bounds(objectsAtTop[0].object);
                right = nearesttop.x + nearesttop.width;
                if (objectsAtTop.length > 1) {
                    targetBounds = ej.datavisualization.Diagram.Util.bounds(objectsAtTop[1].object);
                    var equaldistance = nearesttop.y - targetBounds.y - targetBounds.height;
                    if (Math.abs(equaldistance - objectsAtTop[0].distance) <= snapObjDistance) {
                        right = this._findEquallySpacedNodesAtTop(objectsAtTop, equaldistance, right, equallySpaced);
                    } else equaldistance = objectsAtTop[0].distance;
                } else equaldistance = objectsAtTop[0].distance;
            }

            equallySpaced.sort(function (a, b) { return b.distance - a.distance; });
            equallySpaced[equallySpaced.length] = { object: shape, distance: 0 };
            right = bounds.x + bounds.width > right || !right ? bounds.x + bounds.width : right;
            var dist;
            if (objectsAtBottom.length > 0) {
                nearestbottom = ej.datavisualization.Diagram.Util.bounds(objectsAtBottom[0].object);
                right = nearestbottom.x + nearestbottom.width > right ? nearestbottom.x + nearestbottom.width : right;
                if (objectsAtBottom.length > 1) {
                    targetBounds = ej.datavisualization.Diagram.Util.bounds(objectsAtBottom[1].object);
                    dist = targetBounds.y - nearestbottom.y - nearestbottom.height;
                }

                if (objectsAtTop.length > 0) {
                    if (Math.abs(objectsAtBottom[0].distance - objectsAtTop[0].distance) <= snapObjDistance) {
                        var adjustablevalue = Math.abs(objectsAtBottom[0].distance - objectsAtTop[0].distance) / 2;
                        (objectsAtBottom[0].distance < objectsAtTop[0].distance) ?
                        equaldistance -= adjustablevalue : equaldistance += adjustablevalue;
                        equallySpaced[equallySpaced.length] = objectsAtBottom[0];
                    }
                    else if (objectsAtTop.length == 1) {
                        nearesttop = undefined;
                        equallySpaced.splice(0, 1);
                        equallySpaced[equallySpaced.length] = objectsAtBottom[0];
                        equaldistance = dist;
                    }
                }
                else {
                    equaldistance = dist;
                    equallySpaced[equallySpaced.length] = objectsAtBottom[0];
                }
                if (objectsAtBottom.length > 1 && targetBounds.y > nearestbottom.y + nearestbottom.height) {
                    right = this._findEquallySpacedNodesAtBottom(objectsAtBottom, dist, right, equallySpaced, snapObjDistance);
                }

            }
            if (equallySpaced.length > 2) {
                this._addVSpacingLine(diagram, equallySpaced, ended, g, right);
                var delta = 0;
                if (ended)
                    delta = del.y;
                if (nearesttop)
                    vSnap.offset = equaldistance - Math.abs(bounds.y + delta - nearesttop.y - nearesttop.height) + delta;
                else if (nearestbottom)
                    vSnap.offset = Math.abs(bounds.y + bounds.height + delta - nearestbottom.y) - equaldistance + delta;
                vSnap.snapped = true;
            }
        },
        _addHSpacingLines: function (diagram, equallySpaced, ended, g, top) {
            var scale = diagram._currZoom;
            var hoffset = diagram._hScrollOffset;
            var voffset = diagram._vScrollOffset;
            if (equallySpaced.length > 2) {
                if (!ended) {
                    for (var i = 0; i < equallySpaced.length - 1; i++) {
                        var crnt = ej.datavisualization.Diagram.Util.bounds(equallySpaced[i].object);
                        var next = ej.datavisualization.Diagram.Util.bounds(equallySpaced[i + 1].object);
                        ej.datavisualization.Diagram.SvgContext._renderSpacingLines(ej.datavisualization.Diagram.Point(crnt.x + crnt.width, top - 15),
                       ej.datavisualization.Diagram.Point(next.x, top - 15), g, diagram._svg, scale, hoffset, voffset);
                    }
                }

            }
        },
        _addVSpacingLine: function (diagram, equallySpaced, ended, g, right) {
            var scale = diagram._currZoom;
            var hoffset = diagram._hScrollOffset;
            var voffset = diagram._vScrollOffset;
            if (equallySpaced.length > 2) {
                if (!ended) {
                    var crnt, next;
                    for (var i = 0; i < equallySpaced.length - 1; i++) {
                        crnt = ej.datavisualization.Diagram.Util.bounds(equallySpaced[i].object);
                        next = ej.datavisualization.Diagram.Util.bounds(equallySpaced[i + 1].object);
                        ej.datavisualization.Diagram.SvgContext._renderSpacingLines(ej.datavisualization.Diagram.Point(right + 15, crnt.y + crnt.height),
                      ej.datavisualization.Diagram.Point(right + 15, next.y), g, diagram._svg, scale, hoffset, voffset);
                    }
                }

            }
        },
        _addSameWidthLines: function (diagram, sameWidths, hSnap, ended, g, shape) {
            var scale = diagram._currZoom;
            var hOffset = diagram._hScrollOffset;
            var vOffset = diagram._vScrollOffset;
            sameWidths.sort(function (a, b) { return a.offset - b.offset; });
            var bounds = ej.datavisualization.Diagram.Util.bounds(shape);
            var target = sameWidths[0];
            var targetBounds = ej.datavisualization.Diagram.Util.bounds(target.source);
            var sameSizes = [];
            sameSizes.push(sameWidths[0]);
            var i, crntbounds;
            for (i = 1; i < sameWidths.length; i++) {
                crntbounds = ej.datavisualization.Diagram.Util.bounds(sameWidths[i].source);
                if (crntbounds.width == targetBounds.width)
                    sameSizes.push(sameWidths[i]);
            }
            if (sameSizes.length > 0) {
                if (!ended) {
                    ej.datavisualization.Diagram.SvgContext._renderSpacingLines(ej.datavisualization.Diagram.Point(bounds.x + target.offset, bounds.y - 15),
                        ej.datavisualization.Diagram.Point(bounds.x + bounds.width + target.offset, bounds.y - 15), g, diagram._svg, scale, hOffset, vOffset);
                    for (i = 0; i < sameSizes.length; i++) {
                        bounds = ej.datavisualization.Diagram.Util.bounds(sameSizes[i].source);
                        ej.datavisualization.Diagram.SvgContext._renderSpacingLines(ej.datavisualization.Diagram.Point(bounds.x, bounds.y - 15),
                        ej.datavisualization.Diagram.Point(bounds.x + bounds.width, bounds.y - 15), g, diagram._svg, scale, hOffset, vOffset);
                    }
                }
            }
            hSnap.offset = target.offset;
            hSnap.snapped = true;
        },
        _addSameHeightLines: function (diagram, sameHeights, vSnap, ended, g, shape) {
            var scale = diagram._currZoom;
            var hOffset = diagram._hScrollOffset;
            var vOffset = diagram._vScrollOffset;
            sameHeights.sort(function (a, b) { return a.offset - b.offset; });
            var bounds = ej.datavisualization.Diagram.Util.bounds(shape);
            var target = sameHeights[0];
            var targetBounds = ej.datavisualization.Diagram.Util.bounds(target.source);
            var sameSizes = [];
            sameSizes.push(sameHeights[0]);
            var i, crntBounds;
            for (i = 0; i < sameHeights.length; i++) {
                crntBounds = ej.datavisualization.Diagram.Util.bounds(sameHeights[i].source);
                if (crntBounds.height == targetBounds.height)
                    sameSizes.push(sameHeights[i]);
            }
            if (sameSizes.length > 0) {
                if (!ended) {
                    ej.datavisualization.Diagram.SvgContext._renderSpacingLines(ej.datavisualization.Diagram.Point(bounds.x + bounds.width + 15, bounds.y + target.offset),
                        ej.datavisualization.Diagram.Point(bounds.x + bounds.width + 15, bounds.y + bounds.height + target.offset), g, diagram._svg, scale, hOffset, vOffset);
                    for (i = 0; i < sameSizes.length; i++) {
                        bounds = ej.datavisualization.Diagram.Util.bounds(sameSizes[i].source);
                        ej.datavisualization.Diagram.SvgContext._renderSpacingLines(ej.datavisualization.Diagram.Point(bounds.x + bounds.width + 15, bounds.y),
                        ej.datavisualization.Diagram.Point(bounds.x + bounds.width + 15, bounds.y + bounds.height), g, diagram._svg, scale, hOffset, vOffset);

                    }
                }
                vSnap.offset = target.offset;
                vSnap.snapped = true;
            }
        },
        _findEquallySpacedNodesAtLeft: function (objectsAtLeft, equalDistance, top, equallySpaced) {
            var prevBounds, targetBounds, dist;
            for (var i = 1; i < objectsAtLeft.length; i++) {
                prevBounds = ej.datavisualization.Diagram.Util.bounds(objectsAtLeft[i - 1].object);
                targetBounds = ej.datavisualization.Diagram.Util.bounds(objectsAtLeft[i].object);
                dist = prevBounds.x - targetBounds.x - targetBounds.width;
                if (Math.abs(dist - equalDistance) <= 1) {
                    equallySpaced[equallySpaced.length] = objectsAtLeft[i];
                    if (targetBounds.y < top)
                        top = targetBounds.y;
                }
                else
                    break;
            }
            return top;
        },
        _findEquallySpacedNodesAtRight: function (objectsAtRight, equalDistance, top, equallySpaced, snapObjDistance) {
            var actualDistance = objectsAtRight[0].distance;
            var target, targetBounds, prevBounds, dist;
            if (Math.abs(equalDistance - actualDistance) <= snapObjDistance) {
                for (var i = 0; i < objectsAtRight.length - 1; i++) {
                    target = objectsAtRight[i].object;
                    targetBounds = ej.datavisualization.Diagram.Util.bounds(objectsAtRight[i + 1].object);
                    prevBounds = ej.datavisualization.Diagram.Util.bounds(target);
                    dist = targetBounds.x - prevBounds.x - prevBounds.width;
                    if (Math.abs(dist - equalDistance) <= 1) {
                        equallySpaced[equallySpaced.length] = objectsAtRight[i + 1];
                        if (prevBounds.y < top) {
                            top = prevBounds.y;
                        }
                    }
                    else
                        break;
                }
            }
            return top;
        },
        _findEquallySpacedNodesAtTop: function (objectsAtTop, equalDistance, right, equallySpaced) {
            var prevBounds, targetBounds, dist;
            for (var i = 1; i < objectsAtTop.length; i++) {
                prevBounds = ej.datavisualization.Diagram.Util.bounds(objectsAtTop[i - 1].object);
                targetBounds = ej.datavisualization.Diagram.Util.bounds(objectsAtTop[i].object);
                dist = prevBounds.y - targetBounds.y - targetBounds.height;
                if (Math.abs(dist - equalDistance) <= 1) {
                    equallySpaced[equallySpaced.length] = objectsAtTop[i];
                    if (targetBounds.x + targetBounds.width > right)
                        right = targetBounds.x + targetBounds.width;
                }
                else
                    break;
            }
            return right;
        },
        _findEquallySpacedNodesAtBottom: function (objectsAtBottom, equalDistance, right, equallySpaced, snapObjDistance) {
            var actualDistance = objectsAtBottom[0].distance;
            var target, targetBounds, prevBounds, dist;
            if (Math.abs(equalDistance - actualDistance) <= snapObjDistance) {
                for (var i = 0; i < objectsAtBottom.length - 1; i++) {
                    target = objectsAtBottom[i].object;
                    targetBounds = ej.datavisualization.Diagram.Util.bounds(objectsAtBottom[i + 1].object);
                    prevBounds = ej.datavisualization.Diagram.Util.bounds(target);
                    dist = targetBounds.y - prevBounds.y - prevBounds.height;
                    if (Math.abs(dist - equalDistance) <= 1) {
                        equallySpaced[equallySpaced.length] = objectsAtBottom[i + 1];
                        if (prevBounds.x + prevBounds.width > right)
                            right = prevBounds.x + prevBounds.width;
                    }
                    else
                        break;
                }
            }
            return right;
        },

    };
    //#endregion

    //#region SpatialSearch
    ej.datavisualization.Diagram.SpatialUtil = {
        findQuads: function (spatialSearch, viewPort) {
            spatialSearch.quads = [];
            var quad = spatialSearch.parentQuad;
            this._findQuads(spatialSearch, quad, viewPort);
            return spatialSearch.quads;
        },
        _addIntoAQuad: function (spatialSearch) {
            var isAdded = false;
            while (!isAdded) {
                isAdded = this._add(spatialSearch, spatialSearch.parentQuad);
            }
        },
        _setCurrentNode: function (spatialSearch, node) {
            spatialSearch.childnode = node;
            var bounds = ej.datavisualization.Diagram.Util.bounds(node);
            spatialSearch.childLeft = bounds.left;
            spatialSearch.childTop = bounds.top;
            spatialSearch.childRight = bounds.right;
            spatialSearch.childBottom = bounds.bottom;
        },
        _add: function (spatialSearch, quad) {
            if (this._isContained(spatialSearch, quad)) {
                this._selectQuad(spatialSearch, quad);
                return true;
            } else {
                var newParent;
                var isEmpty = !quad.objects.length && quad.first == null && quad.second == null && quad.third == null && quad.fourth == null;
                if (spatialSearch.childLeft < quad.left) {
                    if (spatialSearch.childTop < quad.top) {
                        newParent = ej.datavisualization.Diagram.Quad(quad.left - quad.width, quad.top - quad.height, quad.width * 2, quad.height * 2);
                        if (!isEmpty)
                            newParent.fourth = quad;
                    }
                    else {
                        newParent = ej.datavisualization.Diagram.Quad(quad.left - quad.width, quad.top, quad.width * 2, quad.height * 2);
                        if (!isEmpty)
                            newParent.second = quad;
                    }
                }
                else if (spatialSearch.childTop < quad.top) {
                    newParent = ej.datavisualization.Diagram.Quad(quad.left, quad.top - quad.height, quad.width * 2, quad.height * 2);
                    if (!isEmpty)
                        newParent.third = quad;
                }
                else {
                    newParent = ej.datavisualization.Diagram.Quad(quad.left, quad.top, quad.width * 2, quad.height * 2);
                    if (!isEmpty)
                        newParent.first = quad;
                }
                quad.parent = newParent;
                spatialSearch.parentQuad = newParent;
                return false;
            }
        },
        _isContained: function (spatialSearch, quad) {
            if (spatialSearch.childLeft >= quad.left && spatialSearch.childRight <= quad.left + quad.width &&
                spatialSearch.childTop >= quad.top && spatialSearch.childBottom <= quad.top + quad.height) {
                return true;
            }
            return false;
        },
        _selectQuad: function (spatialSearch, quad) {
            var current = quad;
            while (current !== null) {
                current = this._getQuad(spatialSearch, current);
            }
        },
        _getQuad: function (spatialSearch, quad) {
            var halfWidth = quad.width / 2;
            var halfHeight = quad.height / 2;
            if (halfWidth >= 100 && halfHeight >= 100) {
                var xCenter = quad.left + halfWidth;
                var yCenter = quad.top + halfHeight;
                if (spatialSearch.childRight <= xCenter) {
                    if (spatialSearch.childBottom <= yCenter) {
                        return quad.first ? quad.first :
                               quad.first = ej.datavisualization.Diagram.Quad(quad.left, quad.top, halfWidth, halfHeight, quad);
                    }
                    else if (spatialSearch.childTop >= yCenter) {
                        return quad.third ? quad.third :
                                quad.third = ej.datavisualization.Diagram.Quad(quad.left, yCenter, halfWidth, halfHeight, quad);
                    }
                }
                else if (spatialSearch.childLeft >= xCenter) {
                    if (spatialSearch.childBottom <= yCenter) {
                        return quad.second ? quad.second :
                               quad.second = ej.datavisualization.Diagram.Quad(xCenter, quad.top, halfWidth, halfHeight, quad);
                    }
                    else if (spatialSearch.childTop >= yCenter) {
                        return quad.fourth ? quad.fourth :
                                quad.fourth = ej.datavisualization.Diagram.Quad(xCenter, yCenter, halfWidth, halfHeight, quad);
                    }
                }
            }
            quad.objects.push(spatialSearch.childnode);
            spatialSearch.quadTable[spatialSearch.childnode.name] = quad;
            return null;
        },
        _findQuads: function (spatialSearch, quad, viewPort) {
            if (quad.first && this._isIntersect(quad.first, viewPort)) {
                this._findQuads(spatialSearch, quad.first, viewPort);
            }
            if (quad.second && this._isIntersect(quad.second, viewPort)) {
                this._findQuads(spatialSearch, quad.second, viewPort);
            }
            if (quad.third && this._isIntersect(quad.third, viewPort)) {
                this._findQuads(spatialSearch, quad.third, viewPort);
            }
            if (quad.fourth && this._isIntersect(quad.fourth, viewPort)) {
                this._findQuads(spatialSearch, quad.fourth, viewPort);
            }
            if (quad.objects.length > 0)
                spatialSearch.quads.push(quad);
        },
        _isIntersect: function (quad, viewPort) {
            if (quad.left + quad.width < viewPort.left || quad.top + quad.height < viewPort.top || quad.left > viewPort.right || quad.top > viewPort.bottom) {
                return false;
            }
            return true;
        },
        _update: function (quad) {
            if (quad.parent != null && quad.objects.length == 0 && quad.first == null &&
                quad.second == null && quad.third == null && quad.fourth == null) {
                var parent = quad.parent;
                if (parent.first == quad) {
                    parent.first = null;
                }
                else if (parent.second == quad) {
                    parent.second = null;
                }
                else if (parent.third == quad) {
                    parent.third = null;
                }
                else if (parent.fourth == quad) {
                    parent.fourth = null;
                }
                this._update(quad.parent);
            }
            else
                return;
        },
        _isWithinPageBounds: function (spatialSearch, bounds) {
            if (bounds.left >= spatialSearch.pageLeft && bounds.right <= spatialSearch.pageRight &&
                bounds.top >= spatialSearch.pageTop && bounds.bottom <= spatialSearch.pageBottom)
                return true;
            else
                return false;
        },
        _updateBounds: function (diagram, spatialSearch, node) {
            var modified = false;
            if (spatialSearch.topElement && node.name == spatialSearch.topElement.name) {
                modified = true;
                spatialSearch.pageTop = null;
                this._findQuadElement(spatialSearch, spatialSearch.parentQuad, "top");
            }
            if (spatialSearch.leftElement && node.name == spatialSearch.leftElement.name) {
                modified = true;
                spatialSearch.pageLeft = null;
                this._findQuadElement(spatialSearch, spatialSearch.parentQuad, "left");
            }
            if (spatialSearch.rightElement && node.name == spatialSearch.rightElement.name) {
                modified = true;
                spatialSearch.pageRight = null;
                this._findQuadElement(spatialSearch, spatialSearch.parentQuad, "right");
            }
            if (spatialSearch.bottomElement && node.name == spatialSearch.bottomElement.name) {
                modified = true;
                spatialSearch.pageBottom = null;
                this._findQuadElement(spatialSearch, spatialSearch.parentQuad, "bottom");
            }
            if (modified) {
                if (diagram._pageBackgroundLayer) {
                    ej.datavisualization.Diagram.PageUtil._updatePageSize(diagram);
                }
            }
        },
        _findQuadElement: function (spatialSearch, quad, direction) {
            switch (direction) {
                case "bottom":
                    if (quad.third != null || quad.fourth != null) {
                        if (quad.third != null) {
                            this._findQuadElement(spatialSearch, quad.third, direction);
                        }
                        if (quad.fourth != null) {
                            this._findQuadElement(spatialSearch, quad.fourth, direction);
                        }
                    }
                    else {
                        if (quad.first != null) {
                            this._findQuadElement(spatialSearch, quad.first, direction);
                        }
                        if (quad.second != null) {
                            this._findQuadElement(spatialSearch, quad.second, direction);
                        }
                    }
                    break;
                case "top":
                    if (quad.first != null || quad.second != null) {
                        if (quad.first != null) {
                            this._findQuadElement(spatialSearch, quad.first, direction);
                        }
                        if (quad.second != null) {
                            this._findQuadElement(spatialSearch, quad.second, direction);
                        }
                    }
                    else {
                        if (quad.third != null) {
                            this._findQuadElement(spatialSearch, quad.third, direction);
                        }
                        if (quad.fourth != null) {
                            this._findQuadElement(spatialSearch, quad.fourth, direction);
                        }
                    }
                    break;
                case "left":
                    if (quad.first != null || quad.third != null) {
                        if (quad.first != null) {
                            this._findQuadElement(spatialSearch, quad.first, direction);
                        }
                        if (quad.third != null) {
                            this._findQuadElement(spatialSearch, quad.third, direction);
                        }
                    }
                    else {
                        if (quad.second != null) {
                            this._findQuadElement(spatialSearch, quad.second, direction);
                        }
                        if (quad.fourth != null) {
                            this._findQuadElement(spatialSearch, quad.fourth, direction);
                        }
                    }
                    break;
                case "right":
                    if (quad.second != null || quad.fourth != null) {
                        if (quad.second != null) {
                            this._findQuadElement(spatialSearch, quad.second, direction);
                        }
                        if (quad.fourth != null) {
                            this._findQuadElement(spatialSearch, quad.fourth, direction);
                        }
                    }
                    else {
                        if (quad.third != null) {
                            this._findQuadElement(spatialSearch, quad.third, direction);
                        }
                        if (quad.first != null) {
                            this._findQuadElement(spatialSearch, quad.first, direction);
                        }
                    }

                    break;
            }

            var bounds;
            for (var i = 0; i < quad.objects.length; i++) {
                bounds = ej.datavisualization.Diagram.Util.bounds(quad.objects[i]);
                if (direction === "bottom" && (spatialSearch.pageBottom <= bounds.bottom || spatialSearch.pageBottom == null)) {
                    spatialSearch.pageBottom = bounds.bottom;
                    spatialSearch.bottomElement = quad.objects[i];
                }
                else if (direction === "top" && (spatialSearch.pageTop >= bounds.top || spatialSearch.pageTop == null)) {
                    spatialSearch.pageTop = bounds.top;
                    spatialSearch.topElement = quad.objects[i];
                }
                else if (direction === "left" && (spatialSearch.pageLeft >= bounds.left || spatialSearch.pageLeft == null)) {
                    spatialSearch.pageLeft = bounds.left;
                    spatialSearch.leftElement = quad.objects[i];
                }
                else if (direction === "right" && (spatialSearch.pageRight <= bounds.right || spatialSearch.pageRight == null)) {
                    spatialSearch.pageRight = bounds.right;
                    spatialSearch.rightElement = quad.objects[i];
                }
            }
        },
        _initializeNodes: function (diagram, spatialSearch) {
            var nodes = diagram.model.nodes;
            for (var i = 0; i < nodes.length; i++) {
                var node = nodes[i];
                if (!node.isGroup) {
                    this._updateQuad(diagram, spatialSearch, node);
                }
            }
        },

        _initializeConnectors: function (diagram, spatialSearch) {
            var connectors = diagram.model.connectors;
            for (var i = 0; i < connectors.length; i++) {
                this._updateQuad(diagram, spatialSearch, connectors[i]);
            }
        },

        _updateQuad: function (diagram, spatialSearch, node, layoutinAction) {
            this._setCurrentNode(spatialSearch, node);
            var bounds = ej.datavisualization.Diagram.Util.bounds(node);
            var quad = spatialSearch.quadTable[node.name];
            if (quad) {
                if (node.segments) {
                    this._removeFromaQuad(spatialSearch, quad, node);
                    this._addIntoAQuad(spatialSearch);
                }
                else if (!this._isContained(spatialSearch, quad)) {
                    this._removeFromaQuad(spatialSearch, quad, node);
                    this._addIntoAQuad(spatialSearch);
                }
            }
            else {
                if (!spatialSearch.parentQuad)
                    spatialSearch.parentQuad = ej.datavisualization.Diagram.Quad(0, 0, 200, 200);
                this._addIntoAQuad(spatialSearch);
            }
            if (!(this._isWithinPageBounds(spatialSearch, bounds) && spatialSearch.leftElement != node &&
                spatialSearch.topElement != node && spatialSearch.rightElement != node && spatialSearch.bottomElement != node)) {
                var modified = false;
                if (spatialSearch.pageLeft >= spatialSearch.childLeft || spatialSearch.pageLeft == null) {
                    spatialSearch.pageLeft = spatialSearch.childLeft;
                    spatialSearch.leftElement = node;
                    modified = true;
                }
                else if (node == spatialSearch.leftElement) {
                    spatialSearch.pageLeft = null;
                    this._findQuadElement(spatialSearch, spatialSearch.parentQuad, "left");
                    modified = true;
                }

                if (spatialSearch.pageTop >= spatialSearch.childTop || spatialSearch.pageTop == null) {
                    spatialSearch.pageTop = spatialSearch.childTop;
                    spatialSearch.topElement = node;
                    modified = true;
                }
                else if (node == spatialSearch.topElement) {
                    spatialSearch.pageTop = null;
                    this._findQuadElement(spatialSearch, spatialSearch.parentQuad, "top");
                    modified = true;
                }

                if (spatialSearch.pageBottom <= spatialSearch.childBottom || spatialSearch.pageBottom == null) {
                    modified = true;
                    spatialSearch.pageBottom = spatialSearch.childBottom;
                    spatialSearch.bottomElement = node;
                }
                else if (node == spatialSearch.bottomElement) {
                    spatialSearch.pageBottom = null;
                    this._findQuadElement(spatialSearch, spatialSearch.parentQuad, "bottom");
                    modified = true;
                }

                if (spatialSearch.pageRight <= spatialSearch.childRight || spatialSearch.pageRight == null) {
                    spatialSearch.pageRight = spatialSearch.childRight;
                    spatialSearch.rightElement = node;
                    modified = true;
                }
                else if (node == spatialSearch.rightElement) {
                    spatialSearch.pageRight = null;
                    this._findQuadElement(spatialSearch, spatialSearch.parentQuad, "right");
                    modified = true;
                }
            }
            if (modified) {
                diagram._views.forEach(function (viewid) {
                    var view = diagram._views[viewid];
                    if (view.type == "overview") {
                        var ovw = $("#" + viewid).ejOverview("instance");
                        if (ovw)
                            ovw._updateOverview(view);
                    }
                });
                if (!diagram._layoutInAction && diagram._pageBackgroundLayer) {
                    ej.datavisualization.Diagram.PageUtil._updatePageSize(diagram);
                }
            }
        },
        _removeFromaQuad: function (spatialSearch, quad, node) {
            if (quad) {
                var index = quad.objects.indexOf(node);
                quad.objects.splice(index, 1);
                this._update(quad);
                delete spatialSearch.quadTable[node.name];
            }
        }

    };

    ej.datavisualization.Diagram.Quad = function (left, top, width, height, parent) {
        return {
            left: left, top: top, width: width, height: height, parent: parent,
            first: null, second: null, third: null, fourth: null, objects: []
        };
    }
    ej.datavisualization.Diagram.SpatialSearch = function () {
        return {
            pageLeft: null, pageRight: null, pageTop: null, pageBottom: null,
            topElement: null, bottomElement: null, rightElement: null, leftElement: null,
            childLeft: null, childTop: null, childRight: null, childBottom: null, childNode: null,
            quads: null, parentQuad: ej.datavisualization.Diagram.Quad(0, 0, 200, 200), quadTable: {}
        };
    };
    //#endregion

})(jQuery, Syncfusion);;
(function ($, ej, undefined) {
    "use strict";

    //#region SymbolPalette widget
    ej.widget("ejOverview", "ej.datavisualization.Overview", {
        // widget element will be automatically set in this
        element: null,

        validTags: ["div"],

        // user defined model will be automatically set in this
        model: null,

        //#region Initialization
        defaults: {
            /**The sourceID of the overview 
   * @default ""
   * @type string
   * @example
   * &lt;div id="overview"&gt;&lt;/div&gt;<br>
   * &lt;script&gt;
   * $("#overview").ejOverview({sourceID: "diagram"});
   * &lt;/script&gt
   * @memberof ejOverview
   * @instance
   */
            sourceID: "",
            width: null,
            height: null
        },

        _canvas: null,

        _actionName: "",
        _startPoint: null,
        _currentPoint: null,
        _prevPoint: null,
        _helper: null,
        _viewPortRatio: 1,
        _resizeDirection: "",
        _inAction: false,
        _init: function () {
            this._wireEvents();
            this._renderCanvas();
            this._setParent(this.model.sourceID);
        },

        _renderCanvas: function () {
            var canvas = document.createElement("div");
            canvas.setAttribute("id", this.element[0].id + "_canvas");
            canvas.setAttribute("class", "drawing");
            canvas.setAttribute("style", "position:relative; height:" + this.model.height + "; width:" + this.model.width + ";style:-ms-touch-action: none;touch-action: none;");
            this.element[0].setAttribute("tabindex", 0);
            this.element.css({
                overflow: "hidden", outline: "none", display: "block", height: this.model.height, width: this.model.width
            });
            this.element.append(canvas);
            this._canvas = canvas;
        },

        _renderDocument: function (view) {
            view.canvas = this._canvas;
            if (window.SVGSVGElement) {
                var attr = {
                    id: this._canvas.id + "_svg",
                    version: "1.1",
                    xlink: "http://www.w3.org/1999/xlink",
                    "style": view.style,
                };

                var svg = new ej.datavisualization.Diagram.Svg(attr);
                this._svg = svg;
                view.svg = svg;
                var ovw = document.getElementById(this._id);
                var element = ovw;
                var eWidth = $(element).width();
                var eHeight = $(element).height();
                var bRect = element.getBoundingClientRect();
                var screenX = (window.screenX < 0) ? window.screenX * -1 : window.screenX;
                if (eWidth === 0) {
                    eWidth = Math.floor(((window.innerWidth - screenX) - Math.floor(bRect.left)));
                }
                var screenY = (window.screenY < 0) ? window.screenY * -1 : window.screenY;
                if (eHeight === 0) {
                    eHeight = Math.floor(((window.innerHeight - screenY) - Math.floor(bRect.top)));
                }
                svg.document.setAttribute("width", eWidth);
                svg.document.setAttribute("height", eHeight);
                this.width = eWidth;
                this.height = eHeight;
                var defs = svg.defs({ "id": view.canvas.id + "definition" });
                svg.appendChild(defs);
                view.diagramLayer = ej.datavisualization.Diagram.SvgContext._renderDiagramLayer(view.canvas, view.svg, view.svg);
                view.canvas.appendChild(svg.document);
                this._addOverviewRectPanel();
            }
        },
        _removeDocument: function (view) {
            var svg = document.getElementById(this._canvas.id + "_svg");
            this._canvas.removeChild(svg);
            //var rect = document.getElementById("overview_canvasrect");
            //this._canvas.removeChild(rect);
        },

        _setModel: function (options) {
            for (var option in options) {
                switch (option) {
                    case "sourceID":
                        this._setParent(options[option]);
                        break;
                }
            }
        },

        _setParent: function (diagram) {
            if (this._parent) {
                var oldparent = $("#" + this._parent._id).ejDiagram("instance");
                oldparent._setOverview(null, this._id);
            }
            if (diagram) {
                this.model.sourceID = diagram;
                this._parent = $("#" + diagram).ejDiagram("instance");
                if (this._parent)
                    this._parent._setOverview(this);
            }
        },

        _destroy: function () {
            this.element.empty().removeClass(this.model.cssClass);
        },
        //#endregion

        //#region Events
        _wireEvents: function () {
            var canvas = $(this._canvas);
            this._on(canvas, ej.eventType.mouseDown, this._mousedown);
            this._on(canvas, ej.eventType.mouseMove, this._mousemove);
            this._on(canvas, ej.eventType.mouseUp, this._mouseup);
            this._on(canvas, ej.eventType.mouseLeave, this._documentmouseup);
            this._on(canvas, "touchstart", this.handleTouchStart);
            this._on(canvas, "touchmove", this.handleTouchMove);
            this._on(canvas, "touchend", this.handleTouchEnd);

            if (window) {
                this._on($(window), "resize", this.updateScrollerViewport);
            }
        },
        updateScrollerViewport: function () {
            //var div = document.getElementById("overview_canvasrect");
            //overview.setAttribute("style", "position:absolute;left:" + this._canvas.offsetLeft + "px;top:" + this._canvas.offsetTop + "px;-ms-touch-action: none;");
        },

        _mouseup: function (evt) {
            this._currentPoint = this._mouseposition(evt);
            var offwidth = this.width;
            var offheight = this.height;
            if (this._actionName)
                if ((this._startPoint.x != this._currentPoint.x || this._startPoint.y != this._currentPoint.y)) {
                    if (this._actionName == "pan") { }
                    else {
                        if (this._helper) {
                            var bounds = this._helper.getBBox();
                            this._resizeDirection = this._resizeDirection || "";
                            var x = bounds.x;
                            var y = bounds.y;
                            var width = bounds.width;
                            var height = bounds.height;
                            var adjust = this._resizeDirection == "topleft" || this._resizeDirection == "topright" || this._resizeDirection == "bottomleft" ||
                                this._resizeDirection == "bottomright";
                            this._renderOverviewRect(x, y, width, height);
                        }
                    }
                }
                else
                    if (evt.target.id == this._canvas.id + "overviewbackrect" || evt.target.id == "helper")
                        if (this._startPoint.x == this._currentPoint.x && this._startPoint.y == this._currentPoint.y) {
                            var g = document.getElementById(this._canvas.id + "overviewrect");
                            var bounds = g.getBBox();
                            var width = bounds.width || 100;
                            var height = bounds.height || 100 / this._viewPortRatio;
                            this.inAction = true;
                            this._actionName = "pan";
                            this._renderOverviewRect(this._currentPoint.x - width / 2, this._currentPoint.y - height / 2, width, height);
                            this.inAction = false;
                        }
            if (this._helper) {
                var g = document.getElementById(this._canvas.id + "overviewhandle");
                g.removeChild(this._helper);
                this._helper = null;
            }
            this._actionName = "";
            this._startPoint = null;
            this._currentPoint = null;
            this._prevPoint = null;
            this._helper = null;
            this._viewPortRatio = 1;
            this._resizeDirection = "";
            this._inAction = false
        },

        _mousedown: function (evt) {
            if (evt.target.className.animVal == "overviewbackrect")
                this._actionName = "draw";
            if (evt.target.id == this._canvas.id + "overviewrect") this._actionName = "pan";
            if (evt.target.className.animVal == "overviewresizer") {
                this._actionName = "scale";
                switch (evt.target.id) {
                    case this._canvas.id + "left":
                        this._resizeDirection = "left";
                        break;
                    case this._canvas.id + "right":
                        this._resizeDirection = "right"; break;
                    case this._canvas.id + "top":
                        this._resizeDirection = "top"; break;
                    case this._canvas.id + "bottom":
                        this._resizeDirection = "bottom"; break;
                    case this._canvas.id + "topleft":
                        this._resizeDirection = "topleft"; break;
                    case this._canvas.id + "topright":
                        this._resizeDirection = "topright"; break;
                    case this._canvas.id + "bottomleft":
                        this._resizeDirection = "bottomleft"; break;
                    case this._canvas.id + "bottomright":
                        this._resizeDirection = "bottomright"; break;
                }
            }
            this._startPoint = this._prevPoint = this._mouseposition(evt);
            var viewPort = ej.datavisualization.Diagram.ScrollUtil._viewPort(this._parent);
            this._viewPortRatio = viewPort.width / viewPort.height;
        },

        _mouseposition: function (evt) {
            var e = this._parent._isTouchEvent(evt);
            if (!e) {
                e = evt;
            }
            var scrollLeft = this._canvas.scrollLeft;
            var scrollTop = this._canvas.scrollTop;
            var controlBBox = this._canvas.getBoundingClientRect();
            var layerx = (e.clientX + scrollLeft) - controlBBox.left;
            var layery = (scrollTop + e.clientY) - controlBBox.top;
            return new ej.datavisualization.Diagram.Point(layerx, layery);
        },

        _updateCursor: function (evt) {
            if (evt.target.className.animVal == "overviewresizer") {
                switch (evt.target.id) {
                    case this._canvas.id + "left":
                        this._canvas.style.cursor = "w-resize";
                        break;
                    case this._canvas.id + "right":
                        this._canvas.style.cursor = "e-resize"; break;
                    case this._canvas.id + "top":
                        this._canvas.style.cursor = "n-resize"; break;
                    case this._canvas.id + "bottom":
                        this._canvas.style.cursor = "s-resize"; break;
                    case this._canvas.id + "topleft":
                        this._canvas.style.cursor = "nw-resize"; break;
                    case this._canvas.id + "topright":
                        this._canvas.style.cursor = "ne-resize"; break;
                    case this._canvas.id + "bottomleft":
                        this._canvas.style.cursor = "sw-resize"; break;
                    case this._canvas.id + "bottomright":
                        this._canvas.style.cursor = "se-resize"; break;
                }
            }
            else this._canvas.style.cursor = "";

        },

        _mousemove: function (evt) {
            this._updateCursor(evt);
            this._currentPoint = this._mouseposition(evt);
            if (this._actionName)
                switch (this._actionName) {
                    case "draw":
                        if (!this._inAction) {
                            this._initHelper();
                            this._inAction = true;
                        }
                        this._updateHelper();
                        break;
                    case "scale":
                        if (!this._inAction) {
                            this._initHelper();
                            this._inAction = true;
                        }
                        this._updateOverviewRectangle();
                        break;
                    case "pan":
                        this._inAction = true;
                        this._translateOverviewRectangle();
                        break;
                }
            this._prevPoint = this._currentPoint;
        },
        _documentmouseup: function (evt) {
            this._inAction = false;
            this._actionName = "";
            if (this._helper) {
                var g = document.getElementById(this._canvas.id + "overviewhandle");
                g.removeChild(this._helper);
                this._helper = null;
            }
        },

        _addOverviewRectPanel: function (view) {
            var svg = this._svg;
            var ovw = svg.g({ "id": this._id + "_overviewlayer" });
            this._canvas.firstChild.appendChild(ovw);
            var rect = svg.rect({ "fill": "transparent", "width": "100%", "height": "100%", "class": "overviewbackrect", "id": this._canvas.id + "overviewbackrect" });
            ovw.appendChild(rect);
            var svgDocument = $(ovw);
            this._on(svgDocument, ej.eventType.mouseDown, this._mousedown);
            this._on(svgDocument, ej.eventType.mouseMove, this._mousemove);
            this._on(svgDocument, ej.eventType.mouseUp, this._mouseup);
            this._on(svgDocument, ej.eventType.mouseLeave, this._documentmouseup);
            this._on(svgDocument, "touchstart", this.handleTouchStart);
            this._on(svgDocument, "touchmove", this.handleTouchMove);
            this._on(svgDocument, "touchend", this.handleTouchEnd);
            var g = svg.g({ "id": this._canvas.id + "overviewhandle" });
            ovw.appendChild(g);
            var innerrect = svg.rect({ "id": this._canvas.id + "overviewrect", "fill": "transparent" });
            g.appendChild(innerrect);
            this._renderOverviewCorner("left", g);
            this._renderOverviewCorner("right", g);
            this._renderOverviewCorner("top", g);
            this._renderOverviewCorner("bottom", g);
            this._renderOverviewCorner("topleft", g);
            this._renderOverviewCorner("topright", g);
            this._renderOverviewCorner("bottomleft", g);
            this._renderOverviewCorner("bottomright", g);

        },

        _updateOverview: function (view) {
            view = view || this._parent._views[this._id];
            var width, height;
            var bounds = this._parent._getDigramBounds();
            width = bounds.width;
            height = bounds.height;
            var viewPort = ej.datavisualization.Diagram.ScrollUtil._viewPort(this._parent);
            var offwidth = this.width;
            var offheight = this.height;
            if (view.context == ej.datavisualization.Diagram.SvgContext) {
                var w = Math.max(width, viewPort.width);
                var h = Math.max(height, viewPort.height);
                this.contentWidth = w = Math.max(w, (offwidth / offheight) * h);
                this.contentHeight = h = Math.max(h, (offheight / offwidth) * w);
                var g = document.getElementById(this._canvas.id + "_diagramLayer");
                var scale = Math.min(offwidth / w, offheight / h);
                this.scale = scale;
                g.setAttribute("transform", "scale(" + scale + "," + scale + "),translate(" + -bounds.x + "," + (-bounds.y) + ")");
                // if (bounds.x < 0 || bounds.y < 0) 
                {
                    var ovw = document.getElementById(this._id + "_overviewlayer");
                    ovw.setAttribute("transform", "translate(" + (-bounds.x * scale) + "," + (-bounds.y * scale) + ")");
                }
            }
            else {
                var context = view._canvas.document.getContext("2d");
                if (view.scale) {
                    context.scale(1 / view.scale.x, 1 / view.scale.y);
                }
                view.scale = { x: scale, y: scale };
                context.scale(scale, scale);
                ej.datavisualization.Diagram.CanvasContext.refreshCanvas(this._parent.model, view);
            }
            this._scrollOverviewRect(this._parent._hScrollOffset, this._parent._vScrollOffset, this._parent._currZoom);
        },

        _updateOverviewRectangle: function () {
            var difx = this._currentPoint.x - this._prevPoint.x;
            var dify = this._currentPoint.y - this._prevPoint.y;
            if (this._actionName == "scale") {
                var size = { "width": 0, "height": 0 };
                var x = 0, y = 0; var w, h;
                switch (this._resizeDirection) {
                    case "left":
                        size.width -= difx;
                        size.height -= difx / this._viewPortRatio;
                        x = difx;
                        y = difx / this._viewPortRatio;
                        y /= 2;
                        break;
                    case "right":
                        size.width += difx;
                        size.height += difx / this._viewPortRatio;
                        y = difx / this._viewPortRatio;
                        y /= -2;
                        break;
                    case "top":
                        size.height -= dify;
                        size.width -= dify * this._viewPortRatio;
                        y = dify;
                        x = dify * this._viewPortRatio;
                        x /= 2;
                        break;
                    case "bottom":
                        size.height += dify;
                        size.width += dify * this._viewPortRatio;
                        x = dify * this._viewPortRatio;
                        x /= -2;
                        break;
                    case "topleft":
                        if (Math.abs(dify) > Math.abs(difx)) { difx = dify * this._viewPortRatio; }
                        else
                            dify = difx / this._viewPortRatio;
                        size.width -= difx;
                        size.height -= dify;
                        x = difx;
                        y = dify;
                        break;
                    case "topright":
                        if (Math.abs(dify) > Math.abs(difx)) { difx = -dify * this._viewPortRatio; }
                        else
                            dify = -(difx / this._viewPortRatio);
                        y = dify;
                        size.width += difx;
                        size.height -= dify;
                        break;
                    case "bottomleft":
                        if (Math.abs(dify) > Math.abs(difx)) { difx = -dify * this._viewPortRatio; }
                        else
                            dify = -difx / this._viewPortRatio;
                        x = difx;
                        size.width -= difx;
                        size.height += dify;
                        break;
                    case "bottomright":
                        if (Math.abs(dify) > Math.abs(difx)) { difx = dify * this._viewPortRatio; }
                        else
                            dify = difx / this._viewPortRatio;
                        size.width += difx;
                        size.height += dify;
                        break;
                }
                this._updateHelper(x, y, size, w, h);
            }

        },

        _initHelper: function () {
            if (!this._helper) {
                var svg = this._svg;
                var g = document.getElementById(this._canvas.id + "overviewhandle");
                var scale = this._parent._currZoom;
                var x = this._startPoint.x;
                var y = this._startPoint.y; var width = 1; var height = 1;
                if (this._actionName == "scale") {
                    var rect = document.getElementById(this._canvas.id + "overviewrect");
                    var bounds = rect.getBBox();
                    x = bounds.x; y = bounds.y; width = bounds.width; height = bounds.height;
                }

                var selectionRect = svg.rect({
                    "id": "helper", x: x, y: y, width: width, height: height,
                    "fill": "transparent", "stroke": "gray", "stroke-dasharray": "2 2", "shape-rendering": "crispEdges"
                });
                g.appendChild(selectionRect);
                this._helper = selectionRect;
            }
        },

        _updateHelper: function (difx, dify, size) {
            if (size) {
                var bounds = this._helper.getBBox();
                var x = bounds.x + difx;
                var y = bounds.y + dify;
                var width = bounds.width + size.width;
                var height = bounds.height + size.height;
            }
            else {
                var difx;
                if (this._currentPoint.x > this._startPoint.x)
                    difx = this._currentPoint.x - this._prevPoint.x;
                else
                    difx = this._prevPoint.x - this._currentPoint.x;
                var dify;
                if (this._currentPoint.y > this._startPoint.y)
                    dify = this._currentPoint.y - this._prevPoint.y;
                else dify = this._prevPoint.y - this._currentPoint.y;
                var w, h;
                if (Math.abs(dify) > Math.abs(difx)) {
                    difx = this._viewPortRatio * dify; h = true; w = false;
                }
                else { dify = difx / this._viewPortRatio; w = true; h = false; }
                var bounds = this._helper.getBBox();
                var x = ((this._startPoint.x > this._currentPoint.x) ?
                    bounds.x - difx : bounds.x);
                var y = ((this._startPoint.y > this._currentPoint.y) ? bounds.y - dify : bounds.y);
                var width = bounds.width + difx;
                var height = bounds.height + dify;
            }
            if (this._helper) {
                ej.datavisualization.Diagram.Util.attr(this._helper, {
                    "id": this._helper.id, "x": x, "y": y,
                    "width": width, "height": height
                });
            }

        },

        _renderOverviewCorner: function (name, parent) {
            var svg = this._svg;
            if (name == "top" || name == "bottom" || name == "right" || name == "left") {
                var innerrect = svg.rect({ "id": this._canvas.id + "visible" + name });
                parent.appendChild(innerrect);
                var transrect = svg.rect({ "id": this._canvas.id + name, "class": "overviewresizer", "fill": "transparent" });
                parent.appendChild(transrect);
            }
            else {
                var innerrect = svg.circle({ "id": this._canvas.id + "visible" + name });
                parent.appendChild(innerrect);
                var transrect = svg.circle({ "id": this._canvas.id + name, "class": "overviewresizer", "fill": "transparent" });
                parent.appendChild(transrect);
            }
        },
        
        _updateOverviewrect: function (x, y, width, height) {
            if (width && height) {
                var rect = document.getElementById(this._canvas.id + "overviewrect");
                var attr = { x: x, y: y, width: Math.max(1, width), height: Math.max(1, height) };
                ej.datavisualization.Diagram.Util.attr(rect, attr);
                this._updateOverviewCorner("top", x + 8, y - 2, Math.max(0, width - 16), 2);
                this._updateOverviewCorner("bottom", x + 8, y + height, Math.max(0, width - 16), 2);
                this._updateOverviewCorner("left", x - 2, y + 8, 2, Math.max(0, height - 16));
                this._updateOverviewCorner("right", x + width, y + 8, 2, Math.max(0, height - 16));
                this._updateOverviewCorner("topleft", x, y, 5, 5);
                this._updateOverviewCorner("topright", x + width, y, 5, 5);
                this._updateOverviewCorner("bottomleft", x, y + height, 5, 5);
                this._updateOverviewCorner("bottomright", x + width, y + height, 5, 5);
            }
        },

        _scrollOverviewRect: function (hoffset, voffset, currentzoom) {
            if (!(this._actionName)) {
                var offwidth = this.width;
                var offheight = this.height;
                var scale = Math.min(this.contentWidth / offwidth, this.contentHeight / offheight);
                var bounds = {};
                var x = bounds.x = (hoffset / currentzoom) / scale;
                var y = bounds.y = (voffset / currentzoom) / scale;
                var viewPort = ej.datavisualization.Diagram.ScrollUtil._viewPort(this._parent);
                var width = bounds.width = (viewPort.width / currentzoom) / scale;
                var height = bounds.height = (viewPort.height / currentzoom) / scale;
                var ratio = viewPort.width / viewPort.height;
                if (x + width > offwidth) {
                    var dif = (x + width) - offwidth;
                    x -= dif;
                }
                if (y + height > offheight) {
                    var dif = (y + height) - offheight;
                    y -= dif;
                }
                this._updateOverviewrect(x, y, width, height);
            }
        },

        _updateOverviewCorner: function (name, x, y, width, height) {
            var rectname = "visible" + name;
            var rect = document.getElementById(this._canvas.id + rectname);
            if (name == "top" || name == "bottom" || name == "right" || name == "left") {
                var attr = { x: x, y: y, width: width, height: height, fill: "#ED1C24" };
                var transattr = { x: x - 2, y: y - 2, width: width == 2 ? 4 : width, height: height == 2 ? 4 : height };
            }
            else {
                var attr = { cx: x, cy: y, "r": 4, fill: "#ED1C24" };
                var transattr = { cx: x, cy: y, "r": 6, fill: "transparent" };
            }
            ej.datavisualization.Diagram.Util.attr(rect, attr);
            var transrect = document.getElementById(this._canvas.id + name);
            ej.datavisualization.Diagram.Util.attr(transrect, transattr);
        },

        _translateOverviewRectangle: function () {
            var offwidth = this.width;
            var offheight = this.height;
            var difx = this._currentPoint.x - this._prevPoint.x;
            var dify = this._currentPoint.y - this._prevPoint.y;
            var viewPort = ej.datavisualization.Diagram.ScrollUtil._viewPort(this._parent);
            var zoom = Math.min(viewPort.width / offwidth, viewPort.height / offheight);
            var panel = document.getElementById(this._canvas.id + "overviewrect");
            var bounds = panel.getBBox();
            var x = bounds.x + difx;
            var y = bounds.y + dify;
            var width = bounds.width;
            var height = bounds.height;
            this._updateOverviewrect(x, y, width, height);
            this._updateView(this._parent._currZoom, x, y, width, height, null);
        },

        _updateView: function (zoom, x, y, width, height, focuspoint) {
            var offwidth = this.width;
            var offheight = this.height;
            var scalex = this.contentWidth / offwidth;
            var scaley = this.contentHeight / offheight;
            var hoffset = x * scalex * zoom;
            var voffset = y * scaley * zoom;
            var bounds = this._parent._getDigramBounds();
            if (zoom != 1 || this._actionName == "pan") {
                var scrollModel = $("#" + this._parent._canvas.id + "_hScrollbar").ejScrollbar("instance").model;
                var scrollwidth = -hoffset + (scrollModel.maximum);
                var delx = -hoffset + this._parent._hScrollOffset;;
                var dely = -voffset + this._parent._vScrollOffset;
                if (bounds.width * zoom < scrollwidth) {
                    var difwidth = bounds.width * zoom - scrollwidth;
                    if (Math.abs(difwidth) < Math.abs(delx)) {
                        delx = delx - difwidth;
                    }
                    else delx = 0;
                }
                scrollModel = $("#" + this._parent._canvas.id + "_vScrollbar").ejScrollbar("instance").model;
                var scrollheight = -voffset + (scrollModel.maximum);
                if (bounds.height * zoom < scrollheight) {
                    var difwidth = bounds.height * zoom - scrollheight;
                    if (Math.abs(difwidth) < Math.abs(dely)) {
                        dely = dely - difwidth;
                    }
                    else dely = 0;
                }
            }
            ej.datavisualization.Diagram.ZoomUtil.zoomPan(this._parent, zoom / this._parent._currZoom, delx, dely, focuspoint, false);
        },

        _renderOverviewRect: function (x, y, width, height) {
            var offwidth = this.width;
            var offheight = this.height;
            var viewPort = ej.datavisualization.Diagram.ScrollUtil._viewPort(this._parent);
            var viewwidth = (width / offwidth) * this.contentWidth;
            var viewheight = (height / offheight) * this.contentHeight;
            var zoom = Math.max(viewPort.width / viewwidth, viewPort.height / viewheight);
            if (zoom >= 0.25 && zoom <= 30) {
                this._updateView(zoom, x, y, width, height, new ej.datavisualization.Diagram.Point(0, 0));
                this._updateOverviewrect(x, y, width, height);
            }
        },

        handleTouchStart: function (evt) {
            evt.preventDefault();
        },

        handleTouchMove: function (evt) {
            evt.preventDefault();
        },

        handleTouchEnd: function (evt) {
            evt.preventDefault();
        },

        handleTouchLeave: function (evt) {
            evt.preventDefault();
        },

        //#endregion

    });
    //#endregion

})(jQuery, Syncfusion);;

(function ($, ej) {
    "use strict";

    //#region SVG
    var Canvas = (function () {
        function Canvas(attr) {
            this.document = document.createElement("canvas");
            this.id = attr.id;
            if (attr) {
                ej.datavisualization.Diagram.Util.attr(this.document, attr);
            }
            return this;
        }

        //rect
        Canvas.prototype.rect = function (x, y, width, height, rotation, style) {
            var ctx = this.document.getContext("2d");
            ctx.beginPath();
            if (rotation && rotation.angle) {
                this.rotateContext(rotation, ctx);
            }
            if (style) {
                this.setStyle(style, ctx);
                if (style.dashArray)
                    this.dashedRectangle(x, y, width, height, style.dashArray);
                else
                    ctx.strokeRect(x, y, width, height);
            }
            else ctx.strokeRect(x, y, width, height);
            ctx.fillRect(x, y, width, height);
            ctx.closePath();
            if (rotation && rotation.angle)
                ctx.restore();
        }

        //ellipse
        Canvas.prototype.ellipse = function (cx, cy, rx, ry, rotation, style) {
            var ctx = this.document.getContext("2d");
            if (style) {
                this.setStyle(style, ctx);
            }
            var r = 4 * ((Math.sqrt(2) - 1) / 3);
            ctx.beginPath();
            if (rotation && rotation.angle) {
                this.rotateContext(rotation, ctx);
            }
            ctx.moveTo(cx, cy - ry);
            ctx.bezierCurveTo(cx + (r * rx), cy - ry, cx + rx, cy - (r * ry), cx + rx, cy);
            ctx.bezierCurveTo(cx + rx, cy + (r * ry), cx + (r * rx), cy + ry, cx, cy + ry);
            ctx.bezierCurveTo(cx - (r * rx), cy + ry, cx - rx, cy + (r * ry), cx - rx, cy);
            ctx.bezierCurveTo(cx - rx, cy - (r * ry), cx - (r * rx), cy - ry, cx, cy - ry);
            ctx.fill();
            if (style && style.dashArray) {
                this.bezierCurve(ej.datavisualization.Diagram.Point(cx, cy - ry),
                    new ej.datavisualization.Diagram.Point(cx + (r * rx), cy - ry),
                    new ej.datavisualization.Diagram.Point(cx + rx, cy - (r * ry)),
                    new ej.datavisualization.Diagram.Point(cx + rx, cy), style);
                this.bezierCurve(ej.datavisualization.Diagram.Point(cx + rx, cy),
                    new ej.datavisualization.Diagram.Point(cx + rx, cy + (r * ry)),
                    new ej.datavisualization.Diagram.Point(cx + (r * rx), cy + ry),
                    new ej.datavisualization.Diagram.Point(cx, cy + ry), style);
                this.bezierCurve(ej.datavisualization.Diagram.Point(cx, cy + ry),
                    new ej.datavisualization.Diagram.Point(cx - (r * rx), cy + ry),
                    new ej.datavisualization.Diagram.Point(cx - rx, cy + (r * ry)),
                    new ej.datavisualization.Diagram.Point(cx - rx, cy), style);
                this.bezierCurve(ej.datavisualization.Diagram.Point(cx - rx, cy),
                    new ej.datavisualization.Diagram.Point(cx - rx, cy - (r * ry)),
                    new ej.datavisualization.Diagram.Point(cx - (r * rx), cy - ry),
                    new ej.datavisualization.Diagram.Point(cx, cy - ry), style);
            }
            ctx.stroke();
            ctx.closePath();
            if (rotation && rotation.angle)
                ctx.restore();
        }

        //circle
        Canvas.prototype.circle = function (centerx, centery, radius, rotation, style) {
            var ctx = this.document.getContext("2d");
            if (style) {
                this.setStyle(style, ctx);
            }
            if (rotation && rotation.angle) {
                this.rotateContext(rotation, ctx);
            }
            if (style && style.dashArray) {
                ctx.beginPath();
                ctx.arc(centerx, centery, radius, 0, 2 * Math.PI);
                ctx.fill();
                ctx.stroke();
                ctx.closePath();
            }
            else { this.ellipse(centerx, centery, radius * 2, radius * 2, rotation, style); }
            if (rotation && rotation.angle)
                ctx.restore();
        }
        Canvas.prototype.arc = function (centerx, centery, radius, angle, style) {
            var rotation = { "angle": angle, x: centerx, y: centery };
            var ctx = this.document.getContext("2d");
            if (style) {
                this.setStyle(style, ctx);
            }
            if (rotation && rotation.angle) {
                this.rotateContext(rotation, ctx);
            }
            ctx.beginPath();
            if (style && style.dashArray) {
                ctx.arc(centerx, centery, radius, 0, Math.PI);
            }
            else { ctx.arc(centerx, centery, radius, 0, Math.PI); }
            //ctx.fill();
            ctx.stroke();
            ctx.closePath();
            if (rotation && rotation.angle)
                ctx.restore();
        }

        //path
        Canvas.prototype.path = function (path, x, y, width, height, rotation, style) {
            var ctx = this.document.getContext("2d");
            if (style) {
                this.setStyle(style, ctx);
            }
            ctx.beginPath();
            ctx.translate(x, y);
            if (rotation && rotation.angle) {
                this.rotateContext(rotation, ctx);
            }
            var shape = document.createElementNS('http://www.w3.org/2000/svg', "path");
            shape.setAttribute("d", path);
            this.document.appendChild(shape);

            //path = this.absolutePath(shape);
            var segments = this.getSegments(shape);
            //var bounds = this.pathBounds(segments);
            //var oldX = bounds.x;
            //var oldY = bounds.y;
            //var newX = width / bounds.width;
            //var newY = height / bounds.height;
            var offsetX = x;
            var offsetY = y;
            var pathstr = "";
            var x0, y0, x1, y1, x2, y2, segs = segments;
            for (var x = 0, y = 0, i = 0, length = segs.length; i < length; ++i) {
                var seg = segs[i], char = seg.command;
                if ('x1' in seg) x1 = seg.x1;
                if ('x2' in seg) x2 = seg.x2;
                if ('y1' in seg) y1 = seg.y1;
                if ('y2' in seg) y2 = seg.y2;
                if ('x' in seg) x = seg.x;
                if ('y' in seg) y = seg.y;
                //if (bounds.width && bounds.height && (width != bounds.width || height != bounds.height)) {
                //    if (x != undefined) {
                //        x = this._scalePathData(x, newX, oldX, offsetX);
                //    }
                //    if (y != undefined) {
                //        y = this._scalePathData(y, newY, oldY, offsetY);
                //    }
                //    if (x1 != undefined) {
                //        x1 = this._scalePathData(x1, newX, oldX, offsetX);
                //    }
                //    if (y1 != undefined) {
                //        y1 = this._scalePathData(y1, newY, oldY, offsetY);
                //    }
                //    if (x2 != undefined) {
                //        x2 = this._scalePathData(x2, newX, oldX, offsetX);
                //    }
                //    if (y2 != undefined) {
                //        y2 = this._scalePathData(y2, newY, oldY, offsetY);
                //    }
                //}
                //else if (bounds.width && bounds.height) {
                //    if (x != undefined) {
                //        x = (x + newX);
                //    }
                //    if (y != undefined) {
                //        y = (y + newY);
                //    }
                //    if (x1 != undefined) {
                //        x1 = (x1 + newX);
                //    }
                //    if (y1 != undefined) {
                //        y1 = (y1 + newY);
                //    }
                //    if (x2 != undefined) {
                //        x2 = (x2 + newX);
                //    }
                //    if (y2 != undefined) {
                //        y2 = (y2 + newY);
                //    }
                //}

                switch (char) {
                    case 'M':
                        ctx.moveTo(x, y); seg.x = x; seg.y = y;
                        break;
                    case 'L':
                        ctx.lineTo(x, y);
                        seg.x = x; seg.y = y;
                        break;
                    case 'C':
                        ctx.bezierCurveTo(x1, y1, x2, y2, x, y);
                        seg.x = x; seg.y = y; seg.x1 = x1; seg.y1 = y1; seg.x2 = x2; seg.y2 = y2;
                        break;
                    case 'Q':
                        ctx.quadraticCurveTo(x1, y1, x, y);
                        seg.x = x; seg.y = y; seg.x1 = x1; seg.y1 = y1;
                        break;
                    case 'A'://need to update for "A"
                        var curr = new ej.datavisualization.Diagram.Point(x0, y0);
                        var rx = seg.r1, ry = seg.r2;
                        //if (isResizing) 
                        //{
                        //    rx = (rx * newX);
                        //    ry = (ry * newY);
                        //}
                        var xAxisRotation = seg.angle * (Math.PI / 180.0);
                        var largeArcFlag = seg.largeArcFlag;
                        var sweepFlag = seg.sweepFlag;
                        var cp = new ej.datavisualization.Diagram.Point(x, y);

                        // Conversion from endpoint to center parameterization
                        // http://www.w3.org/TR/SVG11/implnote.html#ArcImplementationNotes
                        // x1', y1'
                        var currp = new ej.datavisualization.Diagram.Point(
                            Math.cos(xAxisRotation) * (curr.x - cp.x) / 2.0 + Math.sin(xAxisRotation) * (curr.y - cp.y) / 2.0,
                            -Math.sin(xAxisRotation) * (curr.x - cp.x) / 2.0 + Math.cos(xAxisRotation) * (curr.y - cp.y) / 2.0
                        );
                        // adjust radii
                        var l = Math.pow(currp.x, 2) / Math.pow(rx, 2) + Math.pow(currp.y, 2) / Math.pow(ry, 2);
                        if (l > 1) {
                            rx *= Math.sqrt(l);
                            ry *= Math.sqrt(l);
                        }
                        // cx', cy'
                        var s = (largeArcFlag == sweepFlag ? -1 : 1) * Math.sqrt(
                            ((Math.pow(rx, 2) * Math.pow(ry, 2)) - (Math.pow(rx, 2) * Math.pow(currp.y, 2)) - (Math.pow(ry, 2) * Math.pow(currp.x, 2))) /
                            (Math.pow(rx, 2) * Math.pow(currp.y, 2) + Math.pow(ry, 2) * Math.pow(currp.x, 2))
                        );
                        if (isNaN(s)) s = 0;
                        var cpp = new ej.datavisualization.Diagram.Point(s * rx * currp.y / ry, s * -ry * currp.x / rx);
                        // cx, cy
                        var centp = new ej.datavisualization.Diagram.Point(
                            (curr.x + cp.x) / 2.0 + Math.cos(xAxisRotation) * cpp.x - Math.sin(xAxisRotation) * cpp.y,
                            (curr.y + cp.y) / 2.0 + Math.sin(xAxisRotation) * cpp.x + Math.cos(xAxisRotation) * cpp.y
                        );
                        // vector magnitude
                        var m = function (v) { return Math.sqrt(Math.pow(v[0], 2) + Math.pow(v[1], 2)); }
                        // ratio between two vectors
                        var r = function (u, v) { return (u[0] * v[0] + u[1] * v[1]) / (m(u) * m(v)) }
                        // angle between two vectors
                        var a = function (u, v) { return (u[0] * v[1] < u[1] * v[0] ? -1 : 1) * Math.acos(r(u, v)); }
                        // initial angle
                        var a1 = a([1, 0], [(currp.x - cpp.x) / rx, (currp.y - cpp.y) / ry]);
                        // angle delta
                        var u = [(currp.x - cpp.x) / rx, (currp.y - cpp.y) / ry];
                        var v = [(-currp.x - cpp.x) / rx, (-currp.y - cpp.y) / ry];
                        var ad = a(u, v);
                        if (r(u, v) <= -1) ad = Math.PI;
                        if (r(u, v) >= 1) ad = 0;

                        // for markers
                        var dir = 1 - sweepFlag ? 1.0 : -1.0;
                        var ah = a1 + dir * (ad / 2.0);
                        var halfWay = new ej.datavisualization.Diagram.Point(
                            centp.x + rx * Math.cos(ah),
                            centp.y + ry * Math.sin(ah)
                        );
                        seg.centp = centp;
                        seg.xAxisRotation = xAxisRotation;
                        seg.rx = rx;
                        seg.ry = ry;
                        seg.a1 = a1; seg.ad = ad; seg.sweepFlag = sweepFlag;
                        if (ctx != null) {
                            var r = rx > ry ? rx : ry;
                            var sx = rx > ry ? 1 : rx / ry;
                            var sy = rx > ry ? ry / rx : 1;
                            ctx.save();
                            ctx.translate(centp.x, centp.y);
                            ctx.rotate(xAxisRotation);
                            ctx.scale(sx, sy);
                            ctx.arc(0, 0, r, a1, a1 + ad, 1 - sweepFlag);
                            ctx.scale(1 / sx, 1 / sy);
                            ctx.rotate(-xAxisRotation);
                            ctx.translate(-centp.x, -centp.y);
                            ctx.restore();
                        }
                        break;
                    case 'Z':
                        ctx.closePath();
                        x = x0; y = y0;
                        break;
                }
                // Record the start of a subpath
                x0 = x, y0 = y;
            }
            ctx.fill();
            if (style && style.dashArray)
                this.drawDashedPath(segs, style);
            else
                ctx.stroke();
            ctx.translate(-x, -y);
            if (rotation && rotation.angle)
                ctx.restore();
        }

        Canvas.prototype.getSegments = function (shape) {
            var x0, y0, x1, y1, x2, y2, segs = shape.pathSegList;
            var segments = [];
            for (var x = 0, y = 0, i = 0, length = segs.numberOfItems; i < length; ++i) {
                var seg = segs.getItem(i), char = seg.pathSegTypeAsLetter;
                if ('x1' in seg) x1 = seg.x1;
                if ('x2' in seg) x2 = seg.x2;
                if ('y1' in seg) y1 = seg.y1;
                if ('y2' in seg) y2 = seg.y2;
                if ('x' in seg) x = seg.x;
                if ('y' in seg) y = seg.y;
                switch (char) {
                    case 'M':
                        segments.push({ command: "M", x: x, y: y });
                        break;
                    case 'L':
                        segments.push({ command: "L", x0: x0, y0: y0, x: x, y: y });
                        break;
                    case 'H':
                        segments.push({ command: "L", x0: x0, y0: y0, x: x, y: y0 });
                        break;
                    case 'V':
                        segments.push({ command: "L", x0: x0, y0: y0, x: x0, y: y });
                        break;
                    case 'C':
                        segments.push({ command: "C", x0: x0, y0: y0, x1: x1, y1: y1, x2: x2, y2: y2, x: x, y: y });
                        break;
                    case 'S':
                        if (prev) {
                            if (prev.command == "C" || prev.command == "S") {
                                var ctrl = { x: prev.x2, y: prev.y2 };
                            }
                            else
                                var ctrl = { x: prev.x0, y: prev.y0 };
                            var cpt2 = new ej.datavisualization.Diagram.Point(2 * x0 - ctrl.x, 2 * y0 - ctrl.y);

                            segments.push({ command: "C", x0: x0, y0: y0, x1: cpt2.x, y1: cpt2.y, x2: x2, y2: y2, x: x, y: y });
                        }
                        break;
                    case 'Q':
                        //ctx.quadraticCurveTo(x1, y1, x, y);
                        segments.push({ command: "Q", x0: x0, y0: y0, x1: x1, y1: y1, x: x, y: y });
                        break;
                    case 'T':
                        if (prev) {
                            if (prev.command == "Q") {
                                var ctrl = { x: prev.x1, y: prev.y1 };
                            }
                            else
                                var ctrl = { x: prev.x0, y: prev.y0 };
                            var cpt2 = new ej.datavisualization.Diagram.Point(2 * x0 - ctrl.x, 2 * y0 - ctrl.y);

                            segments.push({ command: "Q", x0: x0, y0: y0, x1: cpt2.x, y1: cpt2.y, x: x, y: y });
                        }
                        break;
                    case 'A'://need to update for "A"
                        var newSeg = $.extend(true, {}, seg);
                        newSeg.command = "A";
                        segments.push(newSeg);
                        break;
                    case 'Z':
                        segments.push({ command: "L", x0: x, y0: y, x: x0, y: y0 });
                        x = x0; y = y0;
                        break;
                }
                var prev = segments[segments.length - 1];

                // Record the start of a subpath
                if (char === 'M' || char === 'm') x0 = x, y0 = y;
            }
            return segments;
        }

        Canvas.prototype.drawDashedPath = function (segs, style) {
            var x1, y1, x2, y2, x0, y0;
            var initial;
            for (var x = 0, y = 0, i = 0, length = segs.length; i < length; ++i) {
                var seg = segs[i], char = seg.command;
                if ('x1' in seg) x1 = seg.x1;
                if ('x2' in seg) x2 = seg.x2;
                if ('y1' in seg) y1 = seg.y1;
                if ('y2' in seg) y2 = seg.y2;
                if ('x' in seg) x = seg.x;
                if ('y' in seg) y = seg.y;
                switch (char) {
                    case 'M':
                        initial = { x: x, y: y };
                        break;
                    case 'L':
                        this.dashedLine(x0, y0, x, y, style.dashArray);
                        break;
                    case 'H':
                        this.dashedLine(x0, y0, x, y0, style.dashArray);
                        break;
                    case 'V':
                        this.dashedLine(x0, y0, x0, y, style.dashArray);
                        break;
                    case 'C':
                        this.bezierCurve(ej.datavisualization.Diagram.Point(x0, y0), ej.datavisualization.Diagram.Point(x1, y1), ej.datavisualization.Diagram.Point(x2, y2), ej.datavisualization.Diagram.Point(x, y), style);
                        break;
                    case 'Q':
                        this.quadraticCurve(ej.datavisualization.Diagram.Point(x0, y0), ej.datavisualization.Diagram.Point(x1, y1), ej.datavisualization.Diagram.Point(x, y), style);
                        break;
                    case 'A':
                        var ctx = this.document.getContext("2d");
                        if (ctx != null) {
                            var r = seg.rx > seg.ry ? seg.rx : seg.ry;
                            var sx = seg.rx > seg.ry ? 1 : seg.rx / seg.ry;
                            var sy = seg.rx > seg.ry ? seg.ry / seg.rx : 1;
                            ctx.save();
                            ctx.translate(seg.centp.x, seg.centp.y);
                            ctx.rotate(seg.xAxisRotation);
                            ctx.scale(sx, sy);
                            this.dashedArc(r, 0, 0, seg.a1, seg.a1 + seg.ad, 1 - seg.sweepFlag, style.dashArray);
                            ctx.scale(1 / sx, 1 / sy);
                            ctx.rotate(-seg.xAxisRotation);
                            ctx.translate(-seg.centp.x, -seg.centp.y);
                            ctx.restore();
                        }
                        break;
                    case 'Z':
                        //ctx.closePath();
                        this.dashedLine(x, y, initial.x, initial.y, style.dashArray);
                        x = x0; y = y0;
                        break;
                }
                // Record the start of a subpath
                //if (char === 'M' || char === 'm') x0 = x, y0 = y;
                x0 = x, y0 = y;
            }
            //ctx.fill();
        }
        Canvas.prototype.calculateCirclePoints = function (radius, cx, cy, stAngle, endAngle, couterClockwise, dashSize) {
            stAngle = stAngle * 180 / Math.PI;
            endAngle = endAngle * 180 / Math.PI;
            var n = (2 * Math.PI * radius) * (Math.abs(stAngle - endAngle) / 360) * 0.7;
            var alpha = Math.abs(stAngle - endAngle) / n;
            alpha = couterClockwise ? alpha * -1 : alpha;
            var i = 0;
            var points = [];
            while (i < n) {
                points.push(ej.datavisualization.Diagram.Geometry.transform({ x: cx, y: cy }, stAngle + alpha * i, radius));
                i++;
            }
            return points;
        }

        //polygon
        Canvas.prototype.polygon = function (points, rotation, style) {
            var ctx = this.document.getContext("2d");
            ctx.beginPath();
            if (rotation && rotation.angle) {
                this.rotateContext(rotation, ctx);
            }
            if (style) {
                this.setStyle(style, ctx);
            }
            ctx.moveTo(points[0].x, points[0].y);
            for (var i = 1; i < points.length; i++) {
                ctx.lineTo(points[i].x, points[i].y);
            }
            ctx.lineTo(points[0].x, points[0].y);
            ctx.fill();
            if (style && style.dashArray) {
                for (var i = 1; i < points.length; i++) {
                    this.line(points[i - 1], points[i], style);
                }
                this.line(points[i - 1], points[0], style);
            }
            ctx.stroke();
            ctx.closePath();
            if (rotation && rotation.angle)
                ctx.restore();
        }

        //image
        Canvas.prototype.image = function (image, x, y, width, height, rotation, style) {
            var ctx = this.document.getContext("2d");
            ctx.beginPath();
            if (rotation && rotation.angle) {
                this.rotateContext(rotation, ctx);
            }
            if (style) {
                this.setStyle(style, ctx);
            }
            if (width && height)
                ctx.drawImage(image, x, y, width, height);
            else
                ctx.drawImage(image, x, y);
            ctx.closePath();
            if (rotation && rotation.angle)
                ctx.restore();
        }

        //text
        Canvas.prototype.text = function (text, x, y, rotation, style) {
            var ctx = this.document.getContext("2d");
            ctx.beginPath();
            if (rotation && rotation.angle) {
                this.rotateContext(rotation, ctx);
            }
            if (style)
                this.setStyle(style, ctx);
            ctx.fillText(text.text, x, y);
            ctx.closePath();
            if (rotation && rotation.angle)
                ctx.restore();
        }

        //line
        Canvas.prototype.line = function (start, end, style) {
            var ctx = this.document.getContext("2d");
            ctx.beginPath();
            if (style) {
                this.setStyle(style, ctx);
            }
            if (style && style.dashArray) {
                this.dashedLine(start.x, start.y, end.x, end.y, style.dashArray);
            }
            else {
                ctx.moveTo(start.x, start.y);
                ctx.lineTo(end.x, end.y);
                ctx.stroke();
            }
            ctx.closePath();
        }

        //polyline
        Canvas.prototype.polyline = function (points, style) {
            var ctx = this.document.getContext("2d");
            ctx.beginPath();
            if (style) {
                this.setStyle(style, ctx);
            }
            for (var i = 1; i < points.length; i++) {
                this.line(points[i - 1], points[i], style);
            }
            ctx.stroke();
        }

        //bezier curve
        Canvas.prototype.bezierCurve = function (startPoint, cpt1, cpt2, cpt3, style) {
            var ctx = this.document.getContext("2d");
            ctx.beginPath();
            ctx.moveTo(startPoint.x, startPoint.y);
            if (style) {
                this.setStyle(style, ctx);
            }
            if (style && style.dashArray) {
                var points = [];
                this.flattenCubicBezier(points, startPoint, cpt1, cpt2, cpt3);
                var dashLength = 0;
                for (var i = 0; i < style.dashArray.length; i++) {
                    dashLength += Number(style.dashArray[i]);
                }
                var dashcount = points.length / dashLength;
                var draw = true;
                var drawn = 0;
                for (var i = 0; i < dashcount; i++) {
                    for (var j = 0; j < style.dashArray.length; j++) {
                        drawn += Number(style.dashArray[j]);
                        if (drawn < points.length) {
                            if (draw) {
                                ctx.lineTo(points[drawn].x, points[drawn].y);
                            }
                            else ctx.moveTo(points[drawn].x, points[drawn].y);
                        }
                        draw = !draw;
                    }
                }


            }
            else {
                ctx.moveTo(startPoint.x, startPoint.y);
                ctx.bezierCurveTo(cpt1.x, cpt1.y, cpt2.x, cpt2.y, cpt3.x, cpt3.y);
            }
            ctx.stroke();
        }

        Canvas.prototype.quadraticCurve = function (startPoint, cpt1, cpt2, style) {
            var ctx = this.document.getContext("2d");
            ctx.beginPath();
            ctx.moveTo(startPoint.x, startPoint.y);
            if (style) {
                this.setStyle(style, ctx);
            }
            if (style && style.dashArray) {
                var points = [];
                this.flattenQuadraticBezier(points, startPoint, cpt1, cpt2);
                var dashLength = 0;
                for (var i = 0; i < style.dashArray.length; i++) {
                    dashLength += Number(style.dashArray[i]);
                }
                var dashcount = points.length / dashLength;
                var draw = true;
                var drawn = 0;
                for (var i = 0; i < dashcount; i++) {
                    for (var j = 0; j < style.dashArray.length; j++) {
                        drawn += Number(style.dashArray[j]);
                        if (drawn < points.length) {
                            if (draw) {
                                ctx.lineTo(points[drawn].x, points[drawn].y);
                            }
                            else ctx.moveTo(points[drawn].x, points[drawn].y);
                        }
                        draw = !draw;
                    }
                }

            }
            else {
                ctx.moveTo(startPoint.x, startPoint.y);
                ctx.quadraticCurveTo(cpt1.x, cpt1.y, cpt2.x, cpt2.y);
            }
            ctx.stroke();
        }

        //arrow
        Canvas.prototype.arrow = function (point, rotation, size, isOpen, style) {
            var ctx = this.document.getContext("2d");
            ctx.beginPath();
            if (style) {
                this.setStyle(style, ctx);
            }
            if (rotation && rotation.angle) {
                this.rotateContext(rotation, ctx);
            }
            ctx.moveTo(point.x + size.width, point.y + size.height / 2);
            ctx.lineTo(point.x, point.y);
            ctx.lineTo(point.x + size.width, point.y - size.height / 2);
            if (!isOpen) {
                ctx.closePath();
                ctx.fill();
            }
            ctx.stroke();
            if (rotation && rotation.angle)
                ctx.restore();
        }

        //diamond
        Canvas.prototype.diamond = function (point, rotation, size, style) {
            var ctx = this.document.getContext("2d");
            ctx.beginPath();
            if (style) {
                this.setStyle(style, ctx);
            }
            if (rotation && rotation.angle) {
                this.rotateContext(rotation, ctx);
            }
            ctx.moveTo(point.x + size.width, point.y);
            ctx.lineTo(point.x + size.width / 2, point.y + size.height / 2);
            ctx.lineTo(point.x, point.y);
            ctx.lineTo(point.x + size.width / 2, point.y - size.height / 2);
            ctx.stroke();
            ctx.fill();
            ctx.closePath();
            if (rotation && rotation.angle)
                ctx.restore();
        }

        Canvas.prototype.dashedLine = function (x1, y1, x2, y2, dashArray) {
            var ctx = this.document.getContext("2d");
            ctx.save();
            var dx = (x2 - x1), dy = (y2 - y1);
            var length = Math.sqrt(dx * dx + dy * dy);
            var angle = Math.atan2(dy, dx);
            ctx.beginPath();
            ctx.translate(x1, y1);
            ctx.moveTo(0, 0);
            ctx.rotate(angle);
            var dcount = dashArray.length;
            var dindex = 0, draw = true;
            var x = 0;
            while (length > x) {
                x += Number(dashArray[dindex++ % dcount]);
                if (x > length) x = length;
                draw ? ctx.lineTo(x, 0) : ctx.moveTo(x, 0);
                draw = !draw;
                ctx.stroke();
            }
            ctx.restore();
        }

        Canvas.prototype.dashedRectangle = function (x, y, width, height, dashArray) {
            this.dashedLine(x, y, x + width, y, dashArray);
            this.dashedLine(x + width, y, x + width, y + height, dashArray);
            this.dashedLine(x + width, y + height, x, y + height, dashArray);
            this.dashedLine(x, y + height, x, y, dashArray);
        }

        Canvas.prototype.dashedArc = function (radius, cx, cy, stAngle, endAngle, couterClockwise, dashArray) {
            var dashLength = 0;
            for (var i = 0; i < dashArray.length; i++) {
                dashLength += Number(dashArray[i]);
            }
            var points = this.calculateCirclePoints(radius, cx, cy, stAngle, endAngle, couterClockwise, dashLength);
            var ctx = this.document.getContext("2d");
            ctx.beginPath();

            var dashcount = points.length / dashLength;
            var draw = true;
            var drawn = 0;
            for (var i = 0; i < dashcount; i++) {
                for (var j = 0; j < dashArray.length; j++) {
                    drawn += Number(dashArray[j]);
                    if (drawn < points.length) {
                        if (draw) {
                            ctx.lineTo(points[drawn].x, points[drawn].y);
                            ctx.stroke();
                        }
                        else ctx.moveTo(points[drawn].x, points[drawn].y);
                    }
                    draw = !draw;
                }
            }
            ctx.closePath();
        }
        Canvas.prototype.absolutePath = function (path) {
            var minx, miny, maxx, maxy;
            var x0, y0, x1, y1, x2, y2, segs = path.pathSegList;
            for (var x = 0, y = 0, i = 0, length = segs.numberOfItems; i < length; ++i) {
                var seg = segs.getItem(i), char = seg.pathSegTypeAsLetter;
                if (/[MLHVCSQTA]/.test(char)) {
                    if ('x' in seg) x = seg.x;
                    if ('y' in seg) y = seg.y;
                } else {
                    if ('x1' in seg) x1 = x + seg.x1;
                    if ('x2' in seg) x2 = x + seg.x2;
                    if ('y1' in seg) y1 = y + seg.y1;
                    if ('y2' in seg) y2 = y + seg.y2;
                    if ('x' in seg) x += seg.x;
                    if ('y' in seg) y += seg.y;
                    switch (char) {
                        case 'm': segs.replaceItem(path.createSVGPathSegMovetoAbs(x, y), i); break;
                        case 'l': segs.replaceItem(path.createSVGPathSegLinetoAbs(x, y), i); break;
                        case 'h': segs.replaceItem(path.createSVGPathSegLinetoHorizontalAbs(x), i); break;
                        case 'v': segs.replaceItem(path.createSVGPathSegLinetoVerticalAbs(y), i); break;
                        case 'c': segs.replaceItem(path.createSVGPathSegCurvetoCubicAbs(x, y, x1, y1, x2, y2), i); break;
                        case 's': segs.replaceItem(path.createSVGPathSegCurvetoCubicSmoothAbs(x, y, x2, y2), i); break;
                        case 'q': segs.replaceItem(path.createSVGPathSegCurvetoQuadraticAbs(x, y, x1, y1), i); break;
                        case 't': segs.replaceItem(path.createSVGPathSegCurvetoQuadraticSmoothAbs(x, y), i); break;
                        case 'a': segs.replaceItem(path.createSVGPathSegArcAbs(x, y, seg.r1, seg.r2, seg.angle, seg.largeArcFlag, seg.sweepFlag), i); break;
                        case 'z': case 'Z': x = x0; y = y0; break;
                    }
                }
                if (char === 'M' || char === 'm') x0 = x, y0 = y;
            }

            Canvas.prototype._scalePathData = function (val, scaleFactor, oldOffset, newOffset) {
                if (val !== oldOffset) {
                    if (newOffset !== oldOffset) {
                        val = (((val * scaleFactor) - (Number(oldOffset) * scaleFactor - Number(oldOffset)))
                            + (newOffset - Number(oldOffset)));
                    }
                    else
                        val = ((Number(val) * scaleFactor) - (Number(oldOffset) * scaleFactor - Number(oldOffset)));
                }
                else {
                    if (newOffset !== oldOffset) {
                        val = newOffset;
                    }
                }
                return val;
            }
            return path;
        };

        Canvas.prototype.setStyle = function (style, ctx) {
            if (style.fill)
                ctx.fillStyle = style.fill;
            if (style.stroke)
                ctx.strokeStyle = style.stroke;
            if (style.font)
                ctx.font = style.font;
        }

        Canvas.prototype.rotateContext = function (rotation, ctx) {
            ctx.save();
            ctx.translate(rotation.x, rotation.y);
            ctx.rotate(rotation.angle * Math.PI / 180);
            ctx.translate(-rotation.x, -rotation.y);
        }

        Canvas.prototype.pathBounds = function (segments) {
            //var segs = shape.pathSegList;
            var x1, y1, x2, y2, x0, y0;
            var minx, miny, maxx, maxy;
            for (var x = 0, y = 0, i = 0, length = segments.length; i < length; ++i) {
                var seg = segments[i], char = seg.command;
                if (i == 0 && seg.x && seg.y) {
                    minx = maxx = seg.x;
                    miny = maxy = seg.y;
                }
                //var seg = segs.getItem(i), char = seg.pathSegTypeAsLetter;
                if ('x1' in seg) x1 = seg.x1;
                if ('x2' in seg) x2 = seg.x2;
                if ('y1' in seg) y1 = seg.y1;
                if ('y2' in seg) y2 = seg.y2;
                if ('x' in seg) x = seg.x;
                if ('y' in seg) y = seg.y;

                switch (char) {
                    case 'M':
                        x0 = x; y0 = y;
                    case 'L':
                    case 'H':
                    case 'V':
                        if (x) {
                            minx = Math.min(x, minx);
                            maxx = Math.max(x, maxx);
                        }
                        if (y) {
                            miny = Math.min(y, miny);
                            maxy = Math.max(y, maxy);
                        }
                        break;
                    case 'C':

                    case 'S':
                        var bounds = this.getBezierBounds(ej.datavisualization.Diagram.Point(x0, y0), ej.datavisualization.Diagram.Point(x1, y1), ej.datavisualization.Diagram.Point(x2, y2), ej.datavisualization.Diagram.Point(x, y));
                        minx = Math.min(bounds.x, minx);
                        maxx = Math.max(bounds.x + bounds.width, maxx);
                        miny = Math.min(bounds.y, miny);
                        maxy = Math.max(bounds.y + bounds.height, maxy);
                        break;
                    case 'Q':
                    case "q":
                    case 'T':
                        var bounds = this.getQuadraticBounds(ej.datavisualization.Diagram.Point(x0, y0), ej.datavisualization.Diagram.Point(x1, y1), ej.datavisualization.Diagram.Point(x, y));
                        minx = Math.min(bounds.x, minx);
                        maxx = Math.max(bounds.x + bounds.width, maxx);
                        miny = Math.min(bounds.y, miny);
                        maxy = Math.max(bounds.y + bounds.height, maxy);
                        break;
                    case 'A':
                        break;
                    case 'Z':
                        //ctx.closePath();
                        x = x0; y = y0;
                        break;
                }
            }
            return { x: minx, y: miny, width: maxx - minx, height: maxy - miny };
        };

        Canvas.prototype.flattenCubicBezier = function (points, ptStart, ptCtrl1, ptCtrl2, ptEnd, tolerance) {
            var tolerance = 1.5;

            var max = Number((ej.datavisualization.Diagram.Geometry.distance(ptCtrl1, ptStart) +
                              ej.datavisualization.Diagram.Geometry.distance(ptCtrl2, ptCtrl1) +
                              ej.datavisualization.Diagram.Geometry.distance(ptEnd, ptCtrl2)) / tolerance);

            for (var i = 0; i <= max; i++) {
                var t = i / max;

                var x = (1 - t) * (1 - t) * (1 - t) * ptStart.x +
                           3 * t * (1 - t) * (1 - t) * ptCtrl1.x +
                           3 * t * t * (1 - t) * ptCtrl2.x +
                           t * t * t * ptEnd.x;

                var y = (1 - t) * (1 - t) * (1 - t) * ptStart.y +
                           3 * t * (1 - t) * (1 - t) * ptCtrl1.y +
                           3 * t * t * (1 - t) * ptCtrl2.y +
                           t * t * t * ptEnd.y;

                points.push(new ej.datavisualization.Diagram.Point(x, y));
            }
        }

        Canvas.prototype.flattenQuadraticBezier = function (points, ptStart, ptCtrl1, ptEnd, tolerance) {
            var tolerance = 1.5;

            var max = Number((ej.datavisualization.Diagram.Geometry.distance(ptCtrl1, ptStart) +
                             ej.datavisualization.Diagram.Geometry.distance(ptEnd, ptCtrl1)) / tolerance);

            for (var i = 0; i <= max; i++) {
                var t = i / max;

                var x = (1 - t) * (1 - t) * ptStart.x +
                                          2 * t * (1 - t) * ptCtrl1.x +
                                          t * t * ptEnd.x;

                var y = (1 - t) * (1 - t) * ptStart.y +
                           2 * t * (1 - t) * ptCtrl1.y +
                           t * t * ptEnd.y;

                points.push(new ej.datavisualization.Diagram.Point(x, y));
            }
        }

        Canvas.prototype.getBezierBounds = function (ptStart, ptCtrl1, ptCtrl2, ptEnd, tolerance) {
            var tolerance = 3;
            var minx, miny, maxx, maxy;
            var max = Number((ej.datavisualization.Diagram.Geometry.distance(ptCtrl1, ptStart) +
                              ej.datavisualization.Diagram.Geometry.distance(ptCtrl2, ptCtrl1) +
                              ej.datavisualization.Diagram.Geometry.distance(ptEnd, ptCtrl2)) / tolerance);

            for (var i = 0; i <= max; i++) {

                var t = i / max;

                var x = (1 - t) * (1 - t) * (1 - t) * ptStart.x +
                           3 * t * (1 - t) * (1 - t) * ptCtrl1.x +
                           3 * t * t * (1 - t) * ptCtrl2.x +
                           t * t * t * ptEnd.x;

                var y = (1 - t) * (1 - t) * (1 - t) * ptStart.y +
                           3 * t * (1 - t) * (1 - t) * ptCtrl1.y +
                           3 * t * t * (1 - t) * ptCtrl2.y +
                           t * t * t * ptEnd.y;
                if (i == 0) {
                    minx = maxx = x;
                    miny = maxy = y;
                }
                else {
                    minx = Math.min(x, minx);
                    miny = Math.min(y, miny);
                    maxx = Math.max(x, maxx);
                    maxy = Math.max(y, maxy);
                }
            }
            return { x: minx, y: miny, width: maxx - minx, height: maxy - miny };
        }

        Canvas.prototype.getQuadraticBounds = function (ptStart, ptCtrl1, ptEnd, tolerance) {
            var tolerance = 3;
            var minx, miny, maxx, maxy;
            var max = Number((ej.datavisualization.Diagram.Geometry.distance(ptCtrl1, ptStart) +
                              ej.datavisualization.Diagram.Geometry.distance(ptEnd, ptCtrl1)) / tolerance);

            for (var i = 0; i <= max; i++) {
                var t = i / max;


                var x = (1 - t) * (1 - t) * ptStart.x +
                           2 * t * (1 - t) * ptCtrl1.x +
                           t * t * ptEnd.x;

                var y = (1 - t) * (1 - t) * ptStart.y +
                           2 * t * (1 - t) * ptCtrl1.y +
                           t * t * ptEnd.y;

                if (i == 0) {
                    minx = maxx = x;
                    miny = maxy = y;
                }
                else {
                    minx = Math.min(x, minx);
                    miny = Math.min(y, miny);
                    maxx = Math.max(x, maxx);
                    maxy = Math.max(y, maxy);
                }
            }
            return { x: minx, y: miny, width: maxx - minx, height: maxy - miny };
        }

        return Canvas;
    })();

    ej.datavisualization.Diagram.Canvas = Canvas;
    //#endregion

    //#region Renderer
    ej.datavisualization.Diagram.CanvasContext = {

        _renderDocument: function (view, diagram) {
            var attr = {
                id: view.canvas.id + "_canvas",
                width: diagram._canvas.clientWidth,
                height: diagram._canvas.clientHeight,
                version: "1.1",
                xlink: "http://www.w3.org/1999/xlink",
                "style": view.style,
                "pointer-events": "none"
            };
            view._canvas = new ej.datavisualization.Diagram.Canvas(attr);
            diagram._view = view.canvas;
            view.canvas.appendChild(view._canvas.document);
            if (view.type == "mainview") {
                this._canvas = view.canvas;
                diagram._svg = this._svg = view._canvas;
            }
        },

        _renderGroupBackground: function (group, canvas, diagram) {
            if (!group.pivot) {
                group.pivot = {};
                group.pivot.x = .5;
                group.pivot.y = .5;
            }

            var ctx = canvas.document.getContext("2d");
            if (group.visible) {
                var rotation = { "angle": group.rotateAngle, x: group.offsetX, y: group.offsetY };
                ctx.save();
                ctx.translate(rotation.x, rotation.y);
                ctx.rotate(rotation.angle * Math.PI / 180);
                ctx.translate(-rotation.x, -rotation.y);
                ctx.save();
                ctx.fillStyle = group.fillColor;
                ctx.strokeStyle = group.borderColor;
                ctx.lineWidth = group.borderWidth;
                if (group.borderDashArray) {
                    if (group.borderDashArray.indexOf("[") != -1 || group.borderDashArray instanceof Array) {
                        dashArray = group.borderDashArray;
                    } else if (group.borderDashArray.indexOf(",") != -1) {
                        var dashArray = group.borderDashArray.split(",");
                    } else if (group.borderDashArray.indexOf(" ") != -1) {
                        var dashArray = group.borderDashArray.split(" ");
                    }
                    if (ctx.setLineDash) {
                        ctx.setLineDash(dashArray);
                    } else if (ctx.mozDash) {
                        ctx.mozDash = dashArray;
                    } else {
                        var style = { dashArray: dashArray };
                    }
                }
            }

            var x = group.offsetX - group.width * group.pivot.x;
            var y = group.offsetY - group.height * group.pivot.y;
            canvas.rect(x, y, group.width, group.height, null, style);
        },

        renderGroup: function (group, canvas, diagram) {
            if (group.visible) {
                this._renderGroupBackground(group, canvas, diagram);
                for (var i = 0; i < group.children.length; i++) {
                    var child = diagram.nameTable[group.children[i]];
                    if (child.isGroup) {
                        this.renderGroup(child, canvas, diagram);
                    }
                    else if (child.shape) {
                        this.renderNode(child, canvas);
                    }
                    else if (child.line) {
                        this.renderConnector(child, canvas);
                    }
                }
            }
        },

        renderNode: function (node, canvas) {

            var ctx = canvas.document.getContext("2d");
            if (node.visible) {
                var rotation = { "angle": node.rotateAngle, x: node.offsetX, y: node.offsetY };
                ctx.save();
                ctx.translate(rotation.x, rotation.y);
                ctx.rotate(rotation.angle * Math.PI / 180);
                ctx.translate(-rotation.x, -rotation.y);
                ctx.save();
                ctx.fillStyle = node.fillColor;
                ctx.strokeStyle = node.borderColor;
                ctx.lineWidth = node.borderWidth;
                if (node.borderDashArray) {
                    if (node.borderDashArray.indexOf("[") != -1 || node.borderDashArray instanceof Array) {
                        dashArray = node.borderDashArray;
                    }
                    else
                        if (node.borderDashArray.indexOf(",") != -1) {
                            var dashArray = node.borderDashArray.split(",");
                        }
                        else if (node.borderDashArray.indexOf(" ") != -1) { var dashArray = node.borderDashArray.split(" "); }
                    if (ctx.setLineDash) {
                        ctx.setLineDash(dashArray);
                    }
                    else if (ctx.mozDash) { ctx.mozDash = dashArray; }
                    else
                    {
                        var style = { dashArray: dashArray };
                    }
                }
                var x = node.offsetX - node.width * node.pivot.x;
                var y = node.offsetY - node.height * node.pivot.y;
                if (node.shape.type == "path")
                    this._renderGradient(node, ctx, 0, 0);
                else
                    this._renderGradient(node, ctx, x, y);
                var fill = node.fillColor;
                if (node.constraints & ej.datavisualization.Diagram.NodeConstraints.Shadow) {
                    ctx.save();
                    ctx.strokeStyle = "lightgrey";
                    ctx.fillStyle = fill != "none" || fill != "transparent" ? "lightgrey" : "none";
                    switch (node.shape.type) {
                        case "image":
                        case "rectangle":
                            canvas.rect(x + 5, y + 5, node.width, node.height, null);
                            break;
                        case "ellipse":
                            canvas.ellipse(node.offsetX + 5, node.offsetY + 5, node.width, node.height, null);
                            break;
                        case "path":
                            canvas.path(node._absolutePath, x + 5, y + 5, node.width, node.height, null);
                            break;
                        case "polygon":
                            this._updatePolygonPoints(node);
                            ctx.translate(5, 5);
                            canvas.polygon(node.shape.points, null, style);
                            break;
                    }
                    ctx.restore();
                }

                switch (node.shape.type) {
                    case "rectangle":
                        canvas.rect(x, y, node.width, node.height, null, style);
                        break;
                    case "ellipse":
                        canvas.ellipse(node.offsetX, node.offsetY, node.width / 2, node.height / 2, null, style);
                        break;
                    case "path":
                        canvas.path(node._absolutePath, x, y, node.width, node.height, null, style);
                        break;
                    case "image":
                        canvas.rect(x, y, node.width, node.height, null, style);
                        var image = new Image();
                        image.src = node.shape.src;
                        var rotation = { angle: node.rotateAngle, x: node.offsetX, y: node.offsetY };
                        canvas.image(image, x, y, node.width, node.height, rotation);
                        
                        break;
                    case "text":
                        canvas.rect(x, y, node.width, node.height, null, style);
                        if (node.shape.textBlock) {
                            var font = "";
                            if (node.shape.textBlock.italic)
                                font += "italic ";
                            if (node.shape.textBlock.bold)
                                font += "bold ";
                            font += node.shape.textBlock.fontSize + " ";
                            font += node.shape.textBlock.fontFamily;
                            ctx.font = font;
                            ctx.fillStyle = node.shape.textBlock.fontColor;
                            this._renderTextElement(node, node.shape.textBlock, canvas);
                        }
                        break;
                    case "polygon":
                        this._updatePolygonPoints(node);
                        canvas.polygon(node.shape.points, null, style);
                        break;
                }
                ctx.restore();
                this.renderLabels(node, canvas);
                this.renderPorts(node, canvas, x, y);
                ctx.restore();
            }
        },

        renderConnector: function (connector, canvas) {
            if (connector.visible) {
                var ctx = canvas.document.getContext("2d");
                ctx.save();
                ctx.strokeStyle = connector.lineColor;
                ctx.lineWidth = connector.lineWidth;
                if (connector.lineDashArray) {
                    if (connector.lineDashArray.indexOf("[") != -1 || connector.lineDashArray instanceof Array) {
                        dashArray = connector.lineDashArray;
                    }
                    else {
                        if (connector.lineDashArray.indexOf(",") != -1) {
                            var dashArray = connector.lineDashArray.split(",");
                        }
                        else if (connector.lineDashArray.indexOf(" ") != -1) { var dashArray = connector.lineDashArray.split(" "); }
                    }

                    if (ctx.setLineDash) { ctx.setLineDash(dashArray); }
                    else if (ctx.mozDash) { ctx.mozDash = dashArray; }
                    else
                        var style = { "dashArray": dashArray }
                }
                if (connector.segments) {
                    this._renderSegments(connector, canvas, style);
                }
                ctx.restore();
                this.renderLabels(connector, canvas);
                this._renderDecorators(connector, canvas);
            }
        },
        _renderSegments: function (connector, canvas, style) {
            for (var i = 0; i < connector.segments.length; i++) {
                var segment = connector.segments[i];
                if (segment._bridges.length > 0) {
                    for (var n = 0; n < segment._bridges.length; n++) {
                        var bridge = segment._bridges[n];
                        bridge._rendered = false;
                    }
                }
                if (segment.type == "bezier") {
                    canvas.bezierCurve(segment.sourcePoint, segment._point1, segment._point2, segment.point3, style);
                }
                else {
                    if (connector.cornerRadius > 0) {
                        var points = segment.points;
                        var st, end;
                        for (var j = 0; j < points.length - 1; j++) {
                            if (segment._bridges.length > 0 && j > 0) {
                                this._updateBridging(segment, canvas, j, st ? st : points[j], end ? end : points[j + 1], connector.bridgeSpace, style, true);
                            }
                            var segLength = ej.datavisualization.Diagram.Geometry.distance(points[j], points[j + 1]);
                            if (segLength > 0) {
                                var endpoint = end;
                                if (i < connector.segments.length - 1 || j < points.length - 2) {
                                    if (segLength < connector.cornerRadius * 2) {
                                        end = this._adjustPoint(points[j], points[j + 1], false, segLength / 2);
                                    }
                                    else end = this._adjustPoint(points[j], points[j + 1], false, connector.cornerRadius);
                                }
                                else end = points[j + 1];

                                if (i > 0 || j > 0) {
                                    if (segLength < connector.cornerRadius * 2) {
                                        st = this._adjustPoint(points[j], points[j + 1], true, segLength / 2);
                                        if (i < connector.segments.length - 1 || j < points.length - 2)
                                            end = null;
                                    }
                                    else st = this._adjustPoint(points[j], points[j + 1], true, connector.cornerRadius);
                                }
                                if (st) {
                                    canvas.quadraticCurve(endpoint, points[j], st, style);
                                }
                                else st = points[j];
                                if (end) {
                                    if (j + 1 === points.length - 1 && segment._bridges.length > 0) {
                                        this._updateBridging(segment, canvas, j + 1, st ? st : points[j], end ? end : points[j + 1], connector.bridgeSpace, style, true);
                                        var renderLastPt = false;
                                        for (var n = 0; n < segment._bridges.length; n++) {
                                            var bridge = segment._bridges[n];
                                            if (bridge.segmentPointIndex === j + 1) {
                                                renderLastPt = true;
                                            }
                                        }
                                        if (!renderLastPt)
                                            canvas.line(st, end, style);
                                    }
                                    else if (!(segment._bridges.length > 0)) {
                                        canvas.line(st, end, style);
                                    }
                                }
                            }
                        }
                    }
                    else {
                        for (var k = 1; k < segment.points.length; k++) {
                            if (segment._bridges.length > 0) {
                                this._updateBridging(segment, canvas, k, segment.points[k - 1], segment.points[k], connector.bridgeSpace, style);
                            }
                            else
                                canvas.line(segment.points[k - 1], segment.points[k], style);
                        }
                    };
                }
            }
        },

        _updateBridging: function (seg, canvas, pointIndex, startPt, endPt, bridgeSpace, style, end) {
            var bridgeSP;
            var angle = this._findAngle(startPt, endPt);
            if (seg._bridges.length > 0) {
                if (seg.type === "straight") {
                    for (var n = 0; n < seg._bridges.length; n++) {
                        var bridge = seg._bridges[n];
                        if (!bridge._rendered) {
                            bridgeSP = seg._bridges[n - 1] != null ? seg._bridges[n - 1].endPoint : startPt;
                            canvas.line(bridgeSP, bridge.startPoint, style);
                            var centerX = bridge.startPoint.x + ((bridge.endPoint.x - bridge.startPoint.x) / 2);
                            var centerY = bridge.startPoint.y + ((bridge.endPoint.y - bridge.startPoint.y) / 2);
                            canvas.arc(centerX, centerY, bridgeSpace / 2, angle, style);
                            if (seg._bridges[n + 1] != null)
                                canvas.line(bridge.endPoint, seg._bridges[n + 1].startPoint, style);
                            else
                                canvas.line(bridge.endPoint, endPt, style);
                            bridge._rendered = true;
                        }

                    }
                }
                else if (seg.type === "orthogonal") {
                    for (var n = 0; n < seg._bridges.length; n++) {
                        var bridge = seg._bridges[n];
                        if (bridge.segmentPointIndex === pointIndex && !bridge._rendered) {
                            bridgeSP = seg._bridges[n - 1] != null ? seg._bridges[n - 1].endPoint : startPt;
                            canvas.line(bridgeSP, bridge.startPoint, style);
                            var centerX = bridge.startPoint.x + ((bridge.endPoint.x - bridge.startPoint.x) / 2);
                            var centerY = bridge.startPoint.y + ((bridge.endPoint.y - bridge.startPoint.y) / 2);
                            canvas.arc(centerX, centerY, bridgeSpace / 2, angle, style);
                            if (seg._bridges[n + 1] != null)
                                canvas.line(bridge.endPoint, seg._bridges[n + 1].startPoint, style);
                            else
                                canvas.line(bridge.endPoint, endPt, style);
                            bridge._rendered = true;
                        }
                        else {
                            if (!end)
                                canvas.line(startPt, endPt, style);
                        }
                    }
                }
            }
        },

        _findAngle: function (startPt, endPt) {
            var xDiff = startPt.x - endPt.x; var yDiff = startPt.y - endPt.y;
            return Math.atan2(yDiff, xDiff) * (180 / Math.PI);
        },

        _adjustPoint: function (source, target, isStart, length) {
            var pt = isStart ? { x: source.x, y: source.y } : { x: target.x, y: target.y };

            if (source.x == target.x) {
                if (source.y < target.y && isStart || source.y > target.y && !isStart)
                    pt.y += length;
                else pt.y -= length;
            }
            else if (source.y == target.y) {
                if (source.x < target.x && isStart || source.x > target.x && !isStart)
                    pt.x += length;
                else pt.x -= length;
            }
            else {
                var angle;
                if (isStart) {
                    angle = ej.datavisualization.Diagram.Geometry.findAngle(source, target);
                    pt = ej.datavisualization.Diagram.Geometry.transform(source, angle, length);
                }
                else {
                    angle = ej.datavisualization.Diagram.Geometry.findAngle(target, source);
                    pt = ej.datavisualization.Diagram.Geometry.transform(target, angle, length);
                }
            }
            return pt;
        },
        renderLabels: function (node, canvas) {
            for (var i = 0; i < node.labels.length; i++) {
                var label = node.labels[i];
                var bounds = ej.datavisualization.Diagram.Util.bounds(node);
                var position = ej.datavisualization.Diagram.Util._getLabelPosition(label, bounds);
                var ctx = canvas.document.getContext("2d");
                ctx.lineWidth = label.borderWidth;
                ctx.strokeStyle = label.borderColor;
                this._renderTextElement(node, label, canvas);
            }
        },

        renderPorts: function (node, canvas, x, y) {
            var ctx = canvas.document.getContext("2d");
            for (var i = 0; i < node.ports.length; i++) {
                var port = node.ports[i];
                if (!(port.visibility & ej.datavisualization.Diagram.PortVisibility.Hidden || port.visibility & ej.datavisualization.Diagram.PortVisibility.Hover || port.visibility & ej.datavisualization.Diagram.PortVisibility.Connect)) {
                    var point = ej.datavisualization.Diagram.Util._getPortPosition(port, node.bounds());
                    ctx.fillStyle = port.fillColor;
                    ctx.strokeStyle = port.borderColor;
                    ctx.lineWidth = port.borderWidth;
                    switch (port.shape) {
                        case "x":
                            var size = new ej.datavisualization.Diagram.Size(port.size, port.size);
                            ctx.beginPath();
                            ctx.moveTo((point.x - size / 2), (point.y - size / 2));
                            ctx.lineTo((point.x + size / 2), (point.y + size / 2));
                            ctx.moveTo((point.x + size / 2), (point.y - size / 2));
                            ctx.lineTo((point.x - size / 2), (point.y + size / 2));
                            ctx.stroke();
                            ctx.fill();
                            ctx.closePath();
                            break;
                        case "circle":
                            var rx = size.width / 2;
                            var ry = size.height / 2;
                            canvas.ellipse(x + point.x, y + point.y, rx, ry);
                            break;
                        case "square":
                            canvas.rect(point.x - port.size / 2, point.y - port.size / 2, port.size, port.size);
                            break;
                    }
                }
            }
        },

        _renderTextElement: function (node, textElement, canvas) {
            if (textElement) {
                var text = { childNodes: [] };
                this._wrapText(node, text, textElement, canvas);
                this._alignTextOnLabel(textElement, node, text, canvas);
            }
        },

        _alignTextOnLabel: function (label, node, text, canvas) {
            var bounds = { "width": label.width, "height": text.childNodes.length * label.fontSize };
            var nodeBounds = ej.datavisualization.Diagram.Util.bounds(node);
            var offset = ej.datavisualization.Diagram.Util._getLabelPosition(label, nodeBounds);
            var point = { x: 0, y: 0 };
            var y = 0;

            if (label.verticalAlignment == ej.datavisualization.Diagram.VerticalAlignment.Top)
                y = offset.y;
            else if (label.verticalAlignment == ej.datavisualization.Diagram.VerticalAlignment.Center)
                y = offset.y - bounds.height / 2;
            else
                y = offset.y - bounds.height;
            point.y = y;

            switch (label.horizontalAlignment) {
                case ej.datavisualization.Diagram.HorizontalAlignment.Left:
                    switch (label.textAlign) {
                        case ej.datavisualization.Diagram.TextAlign.Left:
                            point.x = offset.x;
                            break;
                        case ej.datavisualization.Diagram.TextAlign.Center:
                            point.x = offset.x + bounds.width / 2;
                            break;
                        case ej.datavisualization.Diagram.TextAlign.Right:
                            point.x = offset.x + bounds.width;
                            break;
                    }
                    break;
                case ej.datavisualization.Diagram.HorizontalAlignment.Center:
                    switch (label.textAlign) {
                        case ej.datavisualization.Diagram.TextAlign.Left:
                            point.x = offset.x - bounds.width / 2;
                            break;
                        case ej.datavisualization.Diagram.TextAlign.Center:
                            point.x = offset.x;
                            break;
                        case ej.datavisualization.Diagram.TextAlign.Right:
                            point.x = offset.x + bounds.width / 2;
                            break;
                    }
                    break;
                case ej.datavisualization.Diagram.HorizontalAlignment.Right:
                    switch (label.textAlign) {
                        case ej.datavisualization.Diagram.TextAlign.Left:
                            point.x = offset.x - bounds.width;
                            break;
                        case ej.datavisualization.Diagram.TextAlign.Center:
                            point.x = offset.x - bounds.width / 2;
                            break;
                        case ej.datavisualization.Diagram.TextAlign.Right:
                            point.x = offset.x;
                            break;
                    }
                    break;
            }
            var ctx = canvas.document.getContext("2d");
            var font = "";
            if (label.italic)
                font += "italic ";
            if (label.bold)
                font += "bold ";
            font += label.fontSize + "px ";
            font += label.fontFamily;
            for (var i = 0; i < text.childNodes.length; i++) {
                var child = text.childNodes[i];
                var text_x = nodeBounds.x + point.x + child.x;
                var text_y = nodeBounds.y + point.y + 3 * label.fontSize / 4 + child.dy * i;
                var width = ctx.measureText(child.text).width;
                canvas.rect(text_x, text_y - 3 * label.fontSize / 4, width, label.fontSize, null, { fill: label.fillColor });
                ctx.textAlign = label.textAlign;
                canvas.text({ text: child.text}, text_x - child.x, text_y, null, { fill: label.fontColor, font: font });
            }
        },

        _wrapText: function (node, text, label, canvas) {
            var str = label.text; var attr = null;
            var textBBox = ej.datavisualization.Diagram.Util.bounds(node);
            var bbWidth, bbHeight;
            if (node.line) {
                bbWidth = textBBox.width;
                bbHeight = textBBox.height;
            }
            else {
                bbWidth = textBBox.width - label.margin.left - label.margin.right;
                bbHeight = textBBox.height - label.margin.top - label.margin.bottom;
            }
            bbWidth = bbWidth < label.width ? label.width : bbWidth;
            var eachLine = str.split('\n');
            var x, y, tspan, j, string = "", posX, posY, childNodes, bounds, i;
            var ctx = canvas.document.getContext("2d");
            for (j = 0; j < eachLine.length; j++) {
                var txt = "";
                for (i = 0; i < eachLine[j].length; i++) {
                    bounds = ctx.measureText(txt).width;
                    if (label.wrapText) {
                        if (bounds >= (bbWidth - label.fontSize)) {
                            txt += eachLine[j][i];
                            posX = -bounds;
                            if (!(i == eachLine[j].length - 1)) {
                                text.childNodes[text.childNodes.length] = { text: txt };
                                txt = "";
                            }
                        }
                        else {
                            txt += eachLine[j][i];
                            if (i == eachLine[j].length - 1) {
                                text.childNodes[text.childNodes.length] = { text: txt };
                                txt = "";
                            }
                        }
                    }
                    else {
                        txt += eachLine[j][i];
                    }
                }
            }
            this._wrapTextAlign(text, text.childNodes, label.fontSize, label.textAlign, ctx);
        },

        _wrapTextAlign: function (text, childNodes, height, textAlign, ctx) {
            for (var i = 0; i < childNodes.length; i++) {
                var x = ctx.measureText(childNodes[i].text).width;
                switch (textAlign) {
                    case "left":
                        x = 0;
                        break;
                    case "center":
                        x = -x / 2;
                        break;
                    case "right":
                        x = -x;
                        break;
                }
                childNodes[i].x = Number(x);
                childNodes[i].dy = height;
            }
        },

        _renderGradient: function (node, ctx, x, y) {
            if (node.gradient) {
                var max, min;
                for (var i = 0; i < node.gradient.stops.length; i++) {
                    max = !max ? node.gradient.stops[i].offset : Math.max(max, node.gradient.stops[i].offset);
                    min = !min ? node.gradient.stops[i].offset : Math.min(min, node.gradient.stops[i].offset);
                }
                if (node.gradient.type == "linear") {
                    var grd = ctx.createLinearGradient(x + node.gradient.x1, y + node.gradient.y1, x + node.gradient.x2, y + node.gradient.y2);
                    for (var i = 0; i < node.gradient.stops.length; i++) {
                        var stop = node.gradient.stops[i];
                        var offset = min < 0 ? (max + stop.offset) / (2 * max) : stop.offset / max;
                        grd.addColorStop(offset, stop.color);
                    }
                }
                else if (node.gradient.type == "radial") {
                    var grd = ctx.createRadialGradient(x + node.gradient.cx, y + node.gradient.cy, 0, x + node.gradient.fx, y + node.gradient.fy, node.gradient.r);
                    for (var i = 0; i < node.gradient.stops.length; i++) {
                        var stop = node.gradient.stops[i];
                        var offset = min < 0 ? (max + stop.offset) / (2 * max) : stop.offset / max;
                        grd.addColorStop(offset, stop.color);
                    }
                }
                ctx.fillStyle = grd;
            }
        },

        _renderDecorators: function (connector, canvas) {
            var startPoint;
            var endPoint;
            if (connector.targetDecorator && connector.targetDecorator.shape) {
                var segment = connector.segments[connector.segments.length - 1];
                startPoint = segment.points[segment.points.length - 2];
                endPoint = connector.targetPoint;
                if (segment.type == "bezier") {
                    startPoint = segment._point2;
                }
                this._renderDecorator(connector.name + "_targetDecorator", endPoint,
                    startPoint, connector.targetDecorator, canvas);
            }
            if (connector.sourceDecorator && connector.sourceDecorator.shape) {
                var segment = connector.segments[0];
                startPoint = connector.sourcePoint;
                endPoint = segment.points[1];
                if (segment.type == "bezier") {
                    endPoint = segment._point1;
                }
                this._renderDecorator(connector.name + "_sourceDecorator", startPoint,
                    endPoint, connector.sourceDecorator, canvas);
            }
        },

        _renderDecorator: function (name, point1, point2, decorator, canvas) {
            var shape;
            var attr;
            var d;
            var size = new ej.datavisualization.Diagram.Size(decorator.width, decorator.height);
            var ctx = canvas.document.getContext("2d");
            ctx.fillStyle = decorator.fillColor;
            ctx.strokeStyle = decorator.borderColor;
            var angle = ej.datavisualization.Diagram.Geometry.findAngle(point1, point2);
            var rotation = { angle: angle, x: point1.x, y: point1.y };
            switch (decorator.shape) {
                case "arrow":
                    canvas.arrow(point1, rotation, size);
                    break;
                case "openarrow":
                    canvas.arrow(point1, rotation, size, true);
                    break;
                case "circle":
                    var rx = size.width / 2;
                    var ry = size.height / 2;
                    canvas.ellipse(point1.x + size.width / 2, point1.y, rx, ry, rotation);
                    break;
                case "diamond":
                    canvas.diamond(point1, rotation, size);
                    break;
                case "path":
                    canvas.path(decorator.pathData, point1.x, point1.y, size.width, size.height, rotation);
                    break;
            }
            //apply translate matrix and rotate matrix here
            //shape.setAttribute("transform", "rotate(" + ej.datavisualization.Diagram.Geometry.findAngle(point1, point2) + " " + point1.x + " " + point1.y + ")");
            return shape;
        },

        _updatePolygonPoints: function (node) {
            if (node.shape.points) {
                var points = node.shape.points;
                if (points.length) {
                    this._scalePoints(points, node);
                    var bounds = ej.datavisualization.Diagram.Geometry.rect(points);
                    var x = node.offsetX - node.width / 2 - bounds.x;
                    var y = node.offsetY - node.height / 2 - bounds.y;
                    for (var i = 0, len = points.length; i < len; ++i) {
                        var point = ej.datavisualization.Diagram.Geometry.translate(points[i], x, y);
                        points[i] = point;
                    }
                }
            }
        },

        _scalePoints: function (points, node) {
            var bounds = ej.datavisualization.Diagram.Geometry.rect(points);
            if (bounds.width > 0 && bounds.height > 0) {
                var scaleX = node.width / bounds.width;
                var scaleY = node.height / bounds.height;
                for (var i = 0, len = points.length; i < len; ++i) {
                    points[i].x = points[i].x * scaleX;
                    points[i].y = points[i].y * scaleY;
                }
            }
        },

        updateNode: function (model, canvas) {
            this.refreshCanvas(model, canvas);
        },

        updateConnector: function (model, canvas) { this.refreshCanvas(model, canvas); },

        updateGroup: function (model, canvas) { this.refreshCanvas(model, canvas); },

        refreshCanvas: function (model, canvas) {

            var ctx = canvas.document.getContext("2d");
            ctx.save();
            if (view.scale) {
                ctx.scale(1 / view.scale.x, 1 / view.scale.y);
            }
            var bounds = { x: canvas.document.offsetLeft, y: canvas.document.offsetTop, width: canvas.document.offsetWidth, height: canvas.document.offsetHeight };
            ctx.clearRect(bounds.x, bounds.y, bounds.width, bounds.height);
            if (view.scale) {
                ctx.scale(view.scale.x, view.scale.y);
            }
            ctx.restore();
            for (var i = 0; i < model.nodes.length; i++) {
                if (model.nodes[i].shape)
                    this.renderNode(model.nodes[i], canvas);
                else if (model.nodes[i].isGroup)
                    this.renderGroup(model.nodes[i], canvas);
            }
            for (var i = 0; i < model.connectors.length; i++) {
                this.renderConnector(model.connectors[i], canvas);
            }
        },

        updateViewport: function (diagram) {
            //diagram._scroller._updateViewPort();
            //diagram.model.pageSettings._updatePageSize(diagram, true);
        },

        addNodeLabel: function (model, canvas) { this.refreshCanvas(model, canvas); },

        updateLabel: function (model, canvas) { this.refreshCanvas(model, canvas); },

        setNodeShape: function (model, canvas) { this.refreshCanvas(model, canvas); },

        setLine: function (model, canvas) { this.refreshCanvas(model, canvas); },

        renderDecorators: function (model, canvas) { this.refreshCanvas(model, canvas); },

        clearDecorators: function (model, canvas) { this.refreshCanvas(model, canvas); },

        updateTargetDecoratorStyle: function (model, canvas) { this.refreshCanvas(model, canvas); },

        updateSourceDecoratorStyle: function (model, canvas) { this.refreshCanvas(model, canvas); },

        _updateNodeStyle: function (model, canvas) { this.refreshCanvas(model, canvas); },

        updateLabelStyle: function (model, canvas) { this.refreshCanvas(model, canvas); },

        updateConnectorStyle: function (model, canvas) { this.refreshCanvas(model, canvas); },

        updateTextBlock: function (model, label, canvas) { this.refreshCanvas(model, canvas); },

        removeChild: function (element, view, diagram) {
        }
    };
    //#endregion

})(jQuery, Syncfusion);;
/**
* @fileOverview Pivot Grid control to visualize the data
* @copyright Copyright Syncfusion Inc. 2001 - 2013. All rights reserved.
*  Use of this code is subject to the terms of our license.
*  A copy of the current license can be obtained at any time by e-mailing
*  licensing@syncfusion.com. Any infringement will be prosecuted under
*  applicable laws. 
* @version 12.1 
* @author <a href="mailto:licensing@syncfusion.com">Syncfusion Inc</a>
*/

(function ($, ej, undefined) {

	/**
    * @namespace ej
	* @class ejPivotGrid
    * @requires jquery-1.10.2.min.js
    * @requires jquery.easing.1.3.min.js
    * @requires ej.core.js
    * @requires ej.data.js
    * @requires ej.touch.js
    * @requires ej.waitingpopup.js
    * @requires ej.pivotgrid.js
    * @requires ej.olappager.js
	* @classdesc Custom Design for Pivot Grid control.
	* @example 
	* &lt;div id="PivotGrid"&gt; &lt;/div&gt; <br> 
	* &lt;script&gt;
	* // Create Pivot Grid
    * $("#PivotGrid").ejPivotGrid(...); 	
	* &lt;/script&gt;
	*/
    ej.widget("ejPivotGrid", "ej.PivotGrid", {

        // Root Css Class
        _rootCSS: "e-pivotgrid",

        // Widget element will be automatically set in this
        element: null,

        // User-defined model will be automatically set in this
        model: null,

        validTags: ["div", "span"],

        defaults: {
            //properties
            /**		
            * Connects the service using the specified URL for any server updates.	See {@link url}
            * @default ""
            * @type {String}
            * @example 
            * //To set service url during initialization  
            * $("#PivotGrid").ejPivotGrid({url: "/wcf/PivotGridService.svc"});
            * @example 
            * //Get or set the size API, after initialization:<br>
            * //Gets the url value  
            * $("#PivotGrid").ejPivotGrid("option","url");<br>   
            * //Sets the url value 
            * $("#PivotGrid").ejPivotGrid("option","url", "/wcf/PivotGridService.svc" ); 
            * @memberof ejPivotGrid
            * @instance
            */
            url: "",

            /**		
            * Sets the CSS name for custom operation.
            * @default ""
            * @type {String}
            * @example 
            * //To set the CSS name during initialization  
            * $("#PivotGrid").ejPivotGrid({cssClass: "gradient-lime"});	
            * @example 
            * //Get or set css class for initialization:<br>
            *	// Gets the css name.		
            *	$("#PivotGrid").ejPivotGrid("option", "cssClass");			
            *	// Sets the rounded corner to button
            *	$("#PivotGrid").ejPivotGrid("option", "cssClass",  "gradient-lime" ); 		
            * @memberof ejPivotGrid
            * @instance
            */
            cssClass: "",

            /**		
            * Contains the serialized OlapReport at the instant.
            * @default ""
            * @type {String}	
            * @memberof ejPivotGrid
            * @instance
            */
            currentReport: "",

            /**		
            * Allows the user access the different layouts for Pivot Grid.
            * @default ej.PivotGrid.Layout.Normal
            * @type {enum}
            * @example 
            * //To set the layout during initialization  
            * $("#PivotGrid").ejPivotGrid({layout: ej.PivotGrid.Layout.NoSummaries});
            * @example 
            * //Get or set the grid layout, after initialization:<br>
            *	//Gets the layout value
            *	$("#PivotGrid").ejPivotGrid("option", "layout");<br>			
            *	//Sets the grid layout
            *	$("#PivotGrid").ejPivotGrid("option", "layout",ej.PivotGrid.Layout.Normal);			* 		
            * @memberof ejPivotGrid
            * @instance
            */
            layout: "Normal",
            analysisMode: "olap",
            enablePivotFieldList: true,

            /**		
            * Allows the user to load large amount of data into pages.
            * @default false
            * @type {Boolean}
            * @example 
            * //To set the virtual scrolling during initialization  
            * $("#PivotGrid").ejPivotGrid({enableVirtualScrolling: true});
            * @example 
            * //Get or set the virtual scrolling, after initialization:<br>
            *	//Gets the virtual calling state
            *	$("#PivotGrid").ejPivotGrid("option", "enableVirtualScrolling");<br>		
            *	//Sets the virtual scrolling 
            *	$("#PivotGrid").ejPivotGrid("option", "enableVirtualScrolling","true");			* 		
            * @memberof ejPivotGrid
            * @instance
            */
            enableVirtualScrolling: false,

            /**		
            * Allows the user to access/operate each cell through a custom design on right click.
            * @default false
            * @type {Boolean}
            * @example 
            * //To set the cell context option during initialization  
            * $("#PivotGrid").ejPivotGrid({enableCellContext: true});
            * @example 
            * //Get or set the cell context, after initialization:<br>
            *	//Gets the cell context state
            *	$("#PivotGrid").ejPivotGrid("option", "enableCellContext");<br>			
            *	//Sets the cell context value
            *	$("#PivotGrid").ejPivotGrid("option", "enableCellContext","true");			* 		
            * @memberof ejPivotGrid
            * @instance
            */
            enableCellContext: false,

            /**		
            * Allows the user to enable RTL support.
            * @default false
            * @type {Boolean}
            * @example 
            * //To set the enableRTL option during initialization  
            * $("#PivotGrid").ejPivotGrid({enableRTL: true});
            * @example 
            * //Get or set the enableRTL, after initialization:<br>
            *	//Gets the enableRTL values state
            *	$("#PivotGrid").ejPivotGrid("option", "enableRTL");<br>			
            *	//Sets the cell context value
            *	$("#PivotGrid").ejPivotGrid("option", "enableRTL","true");			* 		
            * @memberof ejPivotGrid
            * @instance
            */
            enableRTL: false,

            /**		
            * Allows the user to enable ToolTip support.
            * @default false
            * @type {Boolean}
            * @example 
            * //To set the enableToolTip option during initialization  
            * $("#PivotGrid").ejPivotGrid({enableToolTip: true});
            * @example 
            * //Get or set the enableToolTip, after initialization:<br>
            *	//Gets the enableToolTip values state
            *	$("#PivotGrid").ejPivotGrid("option", "enableToolTip");<br>			
            *	//Sets the cell context value
            *	$("#PivotGrid").ejPivotGrid("option", "enableToolTip","true");			* 		
            * @memberof ejPivotGrid
            * @instance
            */
            enableToolTip: true,
            /**	
            * Allows the user to configure hyperlink settings to Pivot Grid control.
            * @default false
            * @type {Object}
            * @example 
            * //To configure the hyperlink settings to Pivot Grid during initialization  
            * $("#PivotGrid").ejPivotGrid({hyperlinkSettings:{enableValueCellHyperlink: true, enableRowHeaderHyperlink: true}});
            * @example 
            * //Get or set the hyperlink settings, after initialization:<br>
            *	//Gets the hyperlink settings
            *	$("#PivotGrid").ejPivotGrid("option", "hyperlinkSettings");<br>			
            *	//Sets the hyperlink settings to Pivot Grid
            *	$("#PivotGrid").ejPivotGrid("option", "hyperlinkSettings", {enableValueCellHyperlink: true, enableRowHeaderHyperlink: true});			* 		
            * @memberof ejPivotGrid
            * @instance
            */
            hyperlinkSettings: {
                /**		
                * Allows the user to enable/diable hyperlink for value cells.
                * @default false
                * @type {Boolean}
                * @example 
                * //To set the Hyperlink option for value cell during initialization  
                * $("#PivotGrid").ejPivotGrid({hyperlinkSettings:{enableValueCellHyperlink: true}});
                * @example 
                * //Get or set the hyper link for cell, after initialization:<br>
                *	//Gets the hyper link state
                *	$("#PivotGrid").ejPivotGrid("option", "hyperlinkSettings");<br>			
                *	//Sets the cell hyperlink value
                *	$("#PivotGrid").ejPivotGrid("option", "hyperlinkSettings.enableValueCellHyperlink","true");			* 		
                * @alias ejPivotGrid#hyperlinkSettings->enableValueCellHyperlink
                * @instance
                */
                enableValueCellHyperlink: false,

                /**		
                * Allows the user to enable/diable hyperlink for row header.
                * @default false
                * @type {Boolean}
                * @example 
                * //To set the Hyperlink option for row header during initialization  
                * $("#PivotGrid").ejPivotGrid({hyperlinkSettings:{enableRowHeaderHyperlink: true}});
                * @example 
                * //Get or set the hyper link for row header, after initialization:<br>
                *	//Gets the row header hyperlink state
                *	$("#PivotGrid").ejPivotGrid("option", "hyperlinkSettings");<br>		
                *	//Sets the row header hyperlink
                *	$("#PivotGrid").ejPivotGrid("option", "hyperlinkSettings.enableRowHeaderHyperlink","true");			* 		
                * @alias ejPivotGrid#hyperlinkSettings->enableRowHeaderHyperlink
                * @instance
                */
                enableRowHeaderHyperlink: false,

                /**		
                * Allows the user to enable/diable hyperlink for column header.
                * @default false
                * @type {Boolean}
                * @example 
                * //To set the Hyperlink option for column header during initialization  
                * $("#PivotGrid").ejPivotGrid({hyperlinkSettings:{enableColumnHeaderHyperlink: true}});
                * @example 
                * //Get or set the hyper link for column header, after initialization:<br>
                *	//Gets the hyper link state
                *	$("#PivotGrid").ejPivotGrid("option", "hyperlinkSettings");<br>   				
                *	//Sets the column header hyper link state
                *	$("#PivotGrid").ejPivotGrid("option", "hyperlinkSettings.enableColumnHeaderHyperlink","true");			* 		
                * @alias ejPivotGrid#hyperlinkSettings->enableColumnHeaderHyperlink
                * @instance
                */
                enableColumnHeaderHyperlink: false,

                /**		
                * Allows the user to enable/diable hyperlink for summary cells.
                * @default false
                * @type {Boolean}
                * @example 
                * //To set the Hyperlink option for summary cell during initialization  
                * $("#PivotGrid").ejPivotGrid({hyperlinkSettings:{enableSummaryCellHyperlink: true}});
                * @example 
                * //Get or set the hyper link for summary cell, after initialization:<br>
                *	//Gets the summary cell hyperlink state
                *   $("#PivotGrid").ejPivotGrid("option", "hyperlinkSettings");<br>   		
                *	//Sets the summary cell hyperlink 
                *   $("#PivotGrid").ejPivotGrid("option", "hyperlinkSettings.enableSummaryCellHyperlink","true");			* 		
                * @alias ejPivotGrid#hyperlinkSettings->enableSummaryCellHyperlink
                * @instance
                */
                enableSummaryCellHyperlink: false
            },

            /**		
            * Sets the progress indicator type which arises on any UI operation.
            * @default ej.PivotGrid.ProgressMode.Infinite
            * @type {enum}
            * @example 
            * //To set the progress mode during initialization  
            * $("#PivotGrid").ejPivotGrid({progressMode: ej.PivotGrid.ProgressMode.Infinite});
            * @example 
            * //Get or set the progress mode, after initialization:<br>
            *	//Gets the hyper link state
            * $("#PivotGrid").ejPivotGrid("option", "progressMode");<br>   			
            *	//Sets the progress mode value
            * $("#PivotGrid").ejPivotGrid("option", "progressMode", ej.PivotGrid.ProgressMode.Progress ); 
            * 		
            * @memberof ejPivotGrid
            * @instance
            */
            progressMode: "infinite",

            /**  
            * Allows the user to set custom name for the methods at service-end invoked on AJAX post.
            * @default {}
            * @type {Object}
            * @example 
            * //To set serviceMethodSettings API value, to invoke the appropriate service method on UI operation.
            * $("#PivotGrid").ejPivotGrid({ servieMethods.initialize: "InitializeGrid" });
            * @example 
            * //Gets or sets the serviceMethodSettings API, to invoke the appropriate service method on UI operation:<br>
            * //Gets the serviceMethodSettings value   
            * $("#PivotGrid").ejPivotGrid("option", "serviceMethodSettings");<br>   						
            * //Sets the serviceMethodSettings value   			
            * $("#PivotGrid").ejPivotGrid("option", "serviceMethodSettings",  {initialize: "InitializeGrid"} ); 
            * @memberof ejPivotGrid
            * @instance
            */
            serviceMethodSettings: {
                /**  
                * Allows the user to set the custom name for the service method that�s responsible for initializing Pivot Grid. 			
                * @default "InitializeGrid"
                * @type {string}
                * @example 
                * //To set initialize API value, to invoke the corresponding service method for Pivot Grid initialization.
                * $("#PivotGrid").ejPivotGrid({ servieMethods",  {initialize: "InitializeGrid"} ); 
                * @example 
                * //Gets or sets the initialize API, to invoke the corresponding service method for Pivot Grid initialization:<br>
                * //Gets the initialize value   
                * $("#PivotGrid").ejPivotGrid("option", "serviceMethodSettings");<br>   						
                * //Sets the initialize value   			
                * $("#PivotGrid").ejPivotGrid("option", "serviceMethodSettings.initialize", "InitializeGrid" ); 
                * @alias ejPivotGrid#serviceMethodSettings->initialize
                * @instance
                */
                initialize: "InitializeGrid",

                /**  
                * Allows the user to set the custom name for the service method that�s responsible for drilling up/down operation in Pivot Grid. 			
                * @default "DrillGrid"
                * @type {string}
                * @example 
                * //To set drillDown API value, to invoke the corresponding service method for performing server-side operation while drilling up/down in Pivot Grid.   
                * $("#PivotGrid").ejPivotGrid({ serviceMethodSettings: { drillDown: "DrillGridMyMethod" } });
                * @example 
                * //Get or set the drillDown API, to invoke the corresponding service method for performing server-side operation while drilling up/down in Pivot Grid:<br>
                * //Gets the drillDown value   
                * $("#PivotGrid").ejPivotGrid("option", "serviceMethodSettings");<br>   						
                * //Sets the drillDown value   			
                * $("#PivotGrid").ejPivotGrid("option", "serviceMethodSettings.drillDown", "DrillGridMyMethod"} ); 
                * @alias ejPivotGrid#serviceMethodSettings->drillDown
                * @instance
                */
                drillDown: "DrillGrid",

                /**  
                * Allows the user to set the custom name for the service method that�s responsible for performing exporting operation in Pivot Grid.		
                * @default "ExportOptions"
                * @type {string}
                * @example 
                * //To set exportOptions API value, to invoke the corresponding service method for performing server-side operation while exporting.  
                * $("#PivotGrid").ejPivotGrid({ serviceMethodSettings: { exportOptions: "ExportOptionsMyMethod" } });
                * @example 
                * //Gets or sets the exportOptions API, to invoke the corresponding service method for performing server-side operation while exporting.:<br>
                * //Gets the exportOptions value   
                * $("#PivotGrid").ejPivotGrid("option", "serviceMethodSettings");<br>   						
                * //Sets the exportOptions value   			
                * $("#PivotGrid").ejPivotGrid("option", "serviceMethodSettings.exportOptions", "ExportOptionsMyMethod"); 
                * @alias ejPivotGrid#serviceMethodSettings->exportOptions
                * @instance
                */
                exportOptions: "ExportOptions",

                /**  
                * Allows the user to set the custom name for the service method that�s responsible for performing paging operation in Pivot Grid.					
                * @default "Paging" 
                * @type {string}
                * @example 
                * //To set paging API value, to invoke the corresponding service method for performing server-side operation during paging in Pivot Grid.  
                * $("#PivotGrid").ejPivotGrid({ serviceMethodSettings: { paging: "PagingMyMethod" } });
                * @example 
                * //Gets or sets the paging API, to invoke the corresponding service method for performing server-side operation during paging in Pivot Grid:<br>
                * //Gets the paging value   
                * $("#PivotGrid").ejPivotGrid("option", "serviceMethodSettings");<br>   						
                * //Sets the paging value   			
                * $("#PivotGrid").ejPivotGrid("option", "serviceMethodSettings.paging",  "PagingMyMethod"); 
                * @alias ejPivotGrid#serviceMethodSettings->paging
                * @instance
                */
                paging: "Paging"
            },

            /**		
            * Custom object to pass additional information between client-end and service-end.
            * @default null
            * @type {Object}
            * @example 
            * // To pass additional information between client-end and service-end   
            *  $("#PivotGrid").ejPivotGrid({customObject: { Language: "en-US" }}); 
            * @example 
            * //Get or set the custom object API, after initialization:<br>
            *	//Gets the custom object value  
            *	$("#PivotGrid").ejPivotGrid("option","customObject");<br>			
            *	//Sets the custom object value 
            *	$("#PivotGrid").ejPivotGrid("option","customObject", {Language: "en-US"} );  		
            * @memberof ejPivotGrid
            * @instance
            */
            customObject: {},

            /**		
            * Sets the localized language for the control.
            * @default "en-US"
            * @type {String}
            * @example 
            * //Sets the localized language    
            *  $("#PivotGrid").ejPivotGrid({locale: "en-US"}}); 
            * @example 
            * //Get or set the size API, after initialization:<br>
            *	//Gets the localization value  
            *	$("#PivotGrid").ejPivotGrid("option","locale");<br>			
            *	//Sets the localization value 
            *	$("#PivotGrid").ejPivotGrid("option","locale", "en-US" );  		
            * @memberof ejPivotGrid
            * @instance
            */
            locale: "en-US",

            //events

            /**     
            * Fires when the value cell is clicked once if Pivot Grid enabled with hyperlink.
            * @event
            * @name ejPivotGrid#valueCellHyperlinkClick 	
            * @param {Object} argument Event parameters from Pivot Grid.
            * @param {object}  argument.args returns the original event args.
            * @param {string}  argument.cellPosition returns the cell position (row index and column index) in table.
            * @param {string}  argument.cellType returns the type of the cell.
            * @param {string}  argument.rowData returns the seriealized data of the header cells.
            * @param {string}  argument.uniqueName returns the unique name of levels/members.   
            * @example 
            * //valueCellHyperlinkClick event for Pivot Grid
            * $("#PivotGrid").ejPivotGrid({
            *    valueCellHyperlinkClick: function (args) {}
            * });      
            * @memberof ejPivotGrid
            * @instance
            */
            valueCellHyperlinkClick: "",

            /**     
            * Fires when the row header cell is clicked once if Pivot Grid enabled with hyperlink.
            * @event
            * @name ejPivotGrid#rowHeaderHyperlinkClick 	
            * @param {Object} argument Event parameters from Pivot Grid.
            * @param {object}  argument.args returns the original event args.
            * @param {string}  argument.cellPosition returns the cell position (row index and column index) in table.
            * @param {string}  argument.cellType returns the type of the cell.
            * @param {string}  argument.rowData returns the seriealized data of the header cells.
            * @param {string}  argument.uniqueName returns the unique name of levels/members.   
            * @example 
            * //rowHeaderHyperlinkClick event for Pivot Grid
            * $("#PivotGrid").ejPivotGrid({
            *    rowHeaderHyperlinkClick: function (args) {}
            * });      
            * @memberof ejPivotGrid
            * @instance
            */
            rowHeaderHyperlinkClick: "",

            /**     
            * Fires when the column header cell is clicked once if Pivot Grid enabled with hyperlink.
            * @event
            * @name ejPivotGrid#columnHeaderHyperlinkClick 	
            * @param {Object} argument Event parameters from Pivot Grid.
            * @param {object}  argument.args returns the original event args.
            * @param {string}  argument.cellPosition returns the cell position (row index and column index) in table.
            * @param {string}  argument.cellType returns the type of the cell.
            * @param {string}  argument.rowData returns the seriealized data of the header cells.
            * @param {string}  argument.uniqueName returns the unique name of levels/members.     
            * @example 
            * //columnHeaderHyperlinkClick event for Pivot Grid
            * $("#PivotGrid").ejPivotGrid({
            *    columnHeaderHyperlinkClick: function (args) {}
            * });      
            * @memberof ejPivotGrid
            * @instance
            */
            columnHeaderHyperlinkClick: "",

            /**     
            * Fires when the summary cell is clicked.
            * @event
            * @name ejPivotGrid#summaryCellHyperlinkClick 	
            * @param {Object} argument Event parameters from Pivot Grid.
            * @param {object}  argument.args returns the original event args.
            * @param {string}  argument.cellPosition returns the cell position (row index and column index) in table.
            * @param {string}  argument.cellType returns the type of the cell.
            * @param {string}  argument.rowData returns the seriealized data of the header cells.
            * @param {string}  argument.uniqueName returns the unique name of levels/members.
            * @example 
            * //summaryCellHyperlinkClick event for Pivot Grid
            * $("#PivotGrid").ejPivotGrid({
            *    summaryCellHyperlinkClick: function (args) {}
            * });      
            * @memberof ejPivotGrid
            * @instance
            */
            summaryCellHyperlinkClick: "",

            /**     
            * Fires before service invoked.
            * @event
            * @name ejPivotGrid#beforeServiceInvoke 	
            * @param {Object} argument Event parameters from Pivot Grid  
            * @param {string}  argument.action return the current action of Pivot Grid control.		 
            * @param {object}  argument.customObject return the custom object bounds with Pivot Grid control.
            * @param {string}  argument.element return the outer HTML of Pivot Grid control.
            * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
            * @param {object}  argument.model returns the Pivot Grid model
            * @param {string}  argument.type returns the name of the event
            * @example 
            * //beforeServiceInvoke event for Pivot Grid
            * $("#PivotGrid").ejPivotGrid({
            *    beforeServiceInvoke: function (args) {}
            * });      
            * @memberof ejPivotGrid
            * @instance
            */
            beforeServiceInvoke: null,

            /**     
            * Fires after the service is invoked.
            * @event
            * @name ejPivotGrid#afterServiceInvoke 	
            * @param {Object} argument Event parameters from Pivot Grid 
            * @param {string}  argument.action return the current action of Pivot Grid control.		 
            * @param {object}  argument.customObject return the custom object bounds with Pivot Grid control.
            * @param {string}  argument.element return the outer HTML of Pivot Grid control.
            * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
            * @param {object}  argument.model returns the Pivot Grid model
            * @param {string}  argument.type returns the name of the event			 
            * @example 
            * //afterServiceInvoke event for Pivot Grid
            * $("#PivotGrid").ejPivotGrid({
            *    afterServiceInvoke: function (args) {}
            * });      
            * @memberof ejPivotGrid
            * @instance
            */
            afterServiceInvoke: null,

            /**     
            * Fires after drill down of Pivot Grid.
            * @event
            * @name ejPivotGrid#drillSuccess 	
            * @param {Object} argument Event parameters from Pivot Grid    
            * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
            * @param {object}  argument.model returns the Pivot Grid model
            * @param {string}  argument.type returns the name of the event				 
            * @example 
            * //drillSuccess event for Pivot Grid
            * $("#PivotGrid").ejPivotGrid({
            *    drillSuccess: function (args) {}
            * });      
            * @memberof ejPivotGrid
            * @instance
            */
            drillSuccess: null,

            /**     
            * Fires when the cell is right clicked.
            * @event
            * @name ejPivotGrid#cellContext 	
            * @param {Object} argument Event parameters from Pivot Grid.
            * @param {object}  argument.args returns the original event args.
            * @param {string}  argument.cellPosition returns the cell position (row index and column index) in table.
            * @param {string}  argument.cellType returns the type of the cell.
            * @param {string}  argument.rowData returns the seriealized data of the header cells.
            * @param {string}  argument.uniqueName returns the unique name of levels/members.
            * @example 
            * //cellContext event for Pivot Grid
            * $("#PivotGrid").ejPivotGrid({
            *    cellContext: function (args) {}
            * });      
            * @memberof ejPivotGrid
            * @instance
            */
            cellContext: "",

            /**     
            * Fires when Pivot Grid Start loading.
            * @event
            * @name ejPivotGrid#load 	
            * @param {Object} argument Event parameters from Pivot Grid.
            * @param {object}  argument.args returns the original event args.
            * @param {string}  argument.action returns the current action of Pivot Grid control.
            * @param {Object}  argument.customObject returns the custom object bounded with the control.
            * @param {string}  argument.element returns the HTML of Pivot Grid control.
            * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
            * @param {object}  argument.model returns the Pivot Grid model.
            * @param {string}  argument.type returns the name of the event.
            * @example 
            * //load event for Pivot Grid
            * $("#PivotGrid").ejPivotGrid({
            *    load: function (args) {}
            * });      
            * @memberof ejPivotGrid
            * @instance
            */
            load: null,

            /**     
            * Fires when Pivot Grid completely finished its rendering.
            * @event
            * @name ejPivotGrid#renderComplete 	
            * @param {Object} argument Event parameters from Pivot Grid.
            * @param {object}  argument.args returns the original event args.
            * @param {string}  argument.action returns the current action of Pivot Grid control.
            * @param {Object}  argument.customObject returns the custom object bounded with the control.
            * @param {string}  argument.element returns the HTML of Pivot Grid control.
            * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
            * @param {object}  argument.model returns the Pivot Grid model.
            * @param {string}  argument.type returns the name of the event.
            * @example 
            * //renderComplete event for Pivot Grid
            * $("#PivotGrid").ejPivotGrid({
            *    renderComplete: function (args) {}
            * });      
            * @memberof ejPivotGrid
            * @instance
            */
            renderComplete: null,

            /**     
            * Fires while any discrepancies occurs during the rendering time.
            * @event
            * @name ejPivotGrid#renderFailure 	
            * @param {Object} argument Event parameters from Pivot Grid.
            * @param {object}  argument.args returns the original event args.
            * @param {string}  argument.action returns the current action of Pivot Grid control.
            * @param {Object}  argument.customObject returns the custom object bounded with the control.
            * @param {string}  argument.element returns the HTML of Pivot Grid control.
            * @param {Object}  argument.message returns the error message with error code.
            * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
            * @param {object}  argument.model returns the Pivot Grid model.
            * @param {string}  argument.type returns the name of the event.
            * @example 
            * //renderFailure event for Pivot Grid
            * $("#PivotGrid").ejPivotGrid({
            *    renderFailure: function (args) {}
            * });      
            * @memberof ejPivotGrid
            * @instance
            */
            renderFailure: null,

            /**     
            * Fires when Pivot Grid successfully finished its rendering.
            * @event
            * @name ejPivotGrid#renderSuccess 	
            * @param {Object} argument Event parameters from Pivot Grid.
            * @param {object}  argument.args returns the original event args.
            * @param {string}  argument.action returns the current action of Pivot Grid control.
            * @param {Object}  argument.customObject returns the custom object bounded with the control.
            * @param {string}  argument.element returns the HTML of Pivot Grid control.
            * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
            * @param {object}  argument.model returns the Pivot Grid model.
            * @param {string}  argument.type returns the name of the event.
            * @example 
            * //renderSuccess event for Pivot Grid
            * $("#PivotGrid").ejPivotGrid({
            *    renderSuccess: function (args) {}
            * });      
            * @memberof ejPivotGrid
            * @instance
            */
            renderSuccess: null
        },

        /**
        * Provides information about the properties of type Array and Object.
        *
        * @private
        */
        dataTypes: {
            serviceMethodSettings: "data",
            customObject: "data"
        },

        observables: ["layout", "enableCellContext", "hyperlinkSettings.enableValueCellHyperlink", "hyperlinkSettings.enableRowHeaderHyperlink", "hyperlinkSettings.enableColumnHeaderHyperlink", "hyperlinkSettings.enableSummaryCellHyperlink"],
        layout: ej.util.valueFunction("layout"),
        enableCellContext: ej.util.valueFunction("enableCellContext"),
        enableValueCellHyperlink: ej.util.valueFunction("hyperlinkSettings.enableValueCellHyperlink"),
        enableRowHeaderHyperlink: ej.util.valueFunction("hyperlinkSettings.enableRowHeaderHyperlink"),
        enableColumnHeaderHyperlink: ej.util.valueFunction("hyperlinkSettings.enableColumnHeaderHyperlink"),
        enableSummaryCellHyperlink: ej.util.valueFunction("hyperlinkSettings.enableSummaryCellHyperlink"),
        locale: ej.util.valueFunction("locale"),

        /**
        * Gets the serialized current Pivot Report of the control.
        *
        * @private 
        */
        getOlapReport: function () {
            return this._olapReport;
        },

        /**
        * Sets the serialized current Pivot Report of the control.
        *
        * @private 
        */
        setOlapReport: function (value) {
            this._olapReport = value;
        },

        /**
        * Gets the JSON formatted records for the control.
        *
        * @private 
        */
        getJSONRecords: function () {
            return this._JSONRecords;
        },

        /**
        * Sets the JSON formatted records for the control.
        *
        * @private 
        */
        setJSONRecords: function (value) {
            this._JSONRecords = $.parseJSON(value);
        },

        /**
        * Initializes the variables and loads the widget. 
        *
        * @private
        */
        _init: function () {
            this._initPrivateProperties();
            this._load();
        },

        /**
        * Destroys the widget while it's no longer required. 
        *
        * @private
        */
        _destroy: function () {
            this.element.empty().removeClass("e-pivotgrid" + this.model.cssClass);
            if (this.ogridWaitingPopup != undefined)
                this.ogridWaitingPopup._destroy();
        },

        /**
        * Initializes the local variables. 
        *
        * @private
        */
        _initPrivateProperties: function () {
            this._id = this.element.attr("id");
            this._olapReport = "";
            this._JSONRecords = null;
            this._drillCaption = "";
            this._drillAction = "";
            this._startDrilldown = false;
            this._pagerObj = null;
            this._seriesPageCount = null;
            this._categPageCount = null;
            this._seriesCurrentPage = 1;
            this._categCurrentPage = 1;
            this._drillHeaders = new Array();
            this._collapsedRowHeaders = new Array();
            ogridWaitingPopup = null;
            _proxy = this;
            ogridProgressBar = null;
            ogridprogressText = null;
            oGridTimer = null;
            this._maxViewLoading = null;
        },

        /**
        * This function is the point were loading/rendering of the Pivot Grid widget is initiated.
        *
        * @private  
        */
        _load: function () {
            var eventArgs = { action: ej.PivotGrid.GridAction.Initialize, element: this.element, customObject: this.model.customObject };
            this._trigger("load", eventArgs);
            this.element.addClass(this.model.cssClass);
            if (typeof oclientWaitingPopup != 'undefined' && oclientWaitingPopup != null)
                if ($("#" + oclientProxy._id + "_maxView")[0]) {
                    $("#" + oclientProxy._id + "_maxView").ejWaitingPopup({ showOnInit: true });
                    this._maxViewLoading = ("#" + oclientProxy._id + "_maxView").data("ejWaitingPopup");
                }
                else
                    oclientWaitingPopup.show();
            else {
                if (_proxy.model.progressMode == "infinite" && typeof oclientWaitingPopup == 'undefined') {
                    $("#" + this._id).ejWaitingPopup({ showOnInit: true });
                    ogridWaitingPopup = $("#" + this._id).data("ejWaitingPopup");
                }
                else {
                    $("#" + _proxy._id + "progressContainer").remove();
                    this._createProgressBar();
                    this._progressStatus(0, 70, 140);
                }
            }

            if (this.model.beforeServiceInvoke != null)
                this._trigger("beforeServiceInvoke", { action: ej.PivotGrid.GridAction.Initialize, element: this.element, customObject: this.model.customObject });
            if (this.model.currentReport != "")
                this.model.customObject = { currentReport: this.model.currentReport };
            var serializedCustomObject = JSON.stringify(this.model.customObject);
            if (_proxy.model.progressMode == "progress")
                $("#" + _proxy._id + "progressText")[0].innerHTML = _proxy._getLocalizedLabels("PreparingAndExecutingMDXquery") + "...";
            if (this.element[0] != null && this.element.parents().find(".controlPanel").length > 0 && this.layout() != "" && this.layout() != ej.PivotGrid.Layout.Normal)
                this.doAjaxPost("POST", this.model.url + "/" + this.model.serviceMethodSettings.initialize, JSON.stringify({ "action": "initializeGrid", "currentReport": this.model.olapReport, "gridLayout": this.layout(), "customObject": serializedCustomObject }), this._renderControlSuccess);
            else if (this.element[0] != null && this.element.parents().find(".controlPanel").length > 0)
                this.doAjaxPost("POST", this.model.url + "/" + this.model.serviceMethodSettings.initialize, JSON.stringify({ "action": "initializeGrid", "currentReport": this.model.olapReport, "customObject": serializedCustomObject }), this._renderControlSuccess);
            else if (this.layout() != "" && this.layout() != ej.PivotGrid.Layout.Normal)
                this.doAjaxPost("POST", this.model.url + "/" + this.model.serviceMethodSettings.initialize, JSON.stringify({ "action": "initializeGrid", "gridLayout": this.layout(), "enablePivotFieldList": this.model.enablePivotFieldList, "customObject": serializedCustomObject }), this._renderControlSuccess);
            else
                this.doAjaxPost("POST", this.model.url + "/" + this.model.serviceMethodSettings.initialize, JSON.stringify({ "action": "initializeGrid", "enablePivotFieldList": this.model.enablePivotFieldList, "customObject": serializedCustomObject }), this._renderControlSuccess);
        },
        _setFirst: true,

        /**
        * To configure the properties at runtime using SetModel		
        * @private
        */
        _setModel: function (options) {
            for (var key in options) {
                switch (key) {
                    case "OlapReport": this.setOlapReport(options[key]); break;
                    case "JsonData": this.setOlapReport(options[key]); break;
                    case "RefreshOlapGrid": this.element.renderControlFromJSON(options[key]); break;
                    case "layout": this._load(); break;
                    case "customObject": this.model.customObject = options[key]; break;
                    case "enableCellContext": this._load(); break;
                    case "enableValueCellHyperlink": this._load(); break;
                    case "enableRowHeaderHyperlink": this._load(); break;
                    case "enableColumnHeaderHyperlink": this._load(); break;
                    case "enableSummaryCellHyperlink": this._load(); break;
                    case "locale": this._load(); break;
                }
            }
        },

        /**
        * This function is invoked to wire the events for UI interaction.
        *
        * @private  
        */
        _wireEvents: function () {
            if (this.enableCellContext() == true) {
                if (document.addEventListener) {
                    document.removeEventListener('contextmenu', _proxy._cellContext, false);
                    document.addEventListener('contextmenu', _proxy._cellContext, false);
                }
                else {
                    document.detachEvent('oncontextmenu', _proxy._cellContext);
                    document.attachEvent('oncontextmenu', _proxy._cellContext);
                }
            }
            this.drillDownHandler = $.proxy(this._drillDown, this);
            this.addHyperlinkHandler = $.proxy(this._addHyperlink, this);
            this.removeHyperlinkHandler = $.proxy(this._removeHyperlink, this);
            this._on(this.element, "mouseover", ".value, .rowheader, .colheader", this.addHyperlinkHandler);
            this._on(this.element, "mouseleave", ".value, .rowheader, .colheader", this.removeHyperlinkHandler);
            this._on(this.element, "click", ".value, .colheader, .rowheader", function (e) {
                if (e.target.className.indexOf("hyperlink") != -1) {
                    var cellPos = $(e.target).attr("p");
                    var rawdata = this.getJSONRecords()[parseInt((parseInt(cellPos.split(",")[0]) * this.rowCount) + parseInt(cellPos.split(",")[1]))].Info;
                    var cellInfo = {
                        cellValue: e.target.innerHTML,
                        cellPosition: cellPos,
                        cellType: e.target.className.split(' ')[0],
                        uniqueName: rawdata.split('::')[0],
                        args: e,
                        rawdata: rawdata
                    };
                    if (this.enableSummaryCellHyperlink() && e.target.className.indexOf("summary value") >= 0)
                        this._trigger("summaryCellHyperlinkClick", { element: this.element, customObject: this.model.customObject });
                    else if (this.enableValueCellHyperlink() && e.target.className.indexOf("value") >= 0)
                        this._trigger("valueCellHyperlinkClick", { args: cellPos });
                    else if (this.enableRowHeaderHyperlink() && e.target.className.indexOf("rowheader") >= 0)
                        this._trigger("rowHeaderHyperlinkClick", { element: this.element, customObject: this.model.customObject });
                    else if (this.enableColumnHeaderHyperlink() && e.target.className.indexOf("colheader") >= 0)
                        this._trigger("columnHeaderHyperlinkClick", { element: this.element, customObject: this.model.customObject });
                }
            });
            this._on(this.element, "click", ".expand, .collapse", ej.proxy(this._drillDown, this));
            this._on(this.element, "mouseover", ".colheader, .rowheader, .expand, .collapse", function (evt) {
                if (evt.target.className.indexOf("expand") > -1 && !$(evt.target).hasClass("header-hover-expand"))
                    $(evt.target).addClass("header-hover-expand");
                else if (evt.target.className.indexOf("collapse") > -1 && !$(evt.target).hasClass("header-hover-collapse"))
                    $(evt.target).addClass("header-hover-collapse");
                else if ((evt.target.className.indexOf("expand") < 0 || evt.target.className.indexOf("collapse") < 0) && $(evt.target).children("span").length > 0) {
                    var child = $(evt.target).children("span")[0];
                    if (!$(child).hasClass("header-hover-expand") && child.className.indexOf("expand") > -1)
                        $(child).addClass("header-hover-expand");
                    else if (!$(child).hasClass("header-hover-collapse") && child.className.indexOf("collapse") > -1)
                        $(child).addClass("header-hover-collapse");
                }
            });
            this._on(this.element, "mouseleave", ".colheader, .rowheader, .expand, .collapse", function (evt) {
                this.element.find(".expand, .collapse").removeClass("header-hover-expand header-hover-collapse");
            });
            if (this.model.enableToolTip) {
                this._on(this.element, "mouseover", ".value", this._applyToolTip);
                this._on(this.element, "mouseleave", ".value", function (e) {
                    this.element.find('#gridTooltip').hide("slow");
                });
            }
        },

        /**
        * This function is invoked to unwire the events registered for UI interaction.
        *
        * @private  
        */
        _unWireEvents: function () {
            this._off(this.element, "click", ".expand, .collapse");
            this._off(this.element, "click", ".value, .colheader, .rowheader");
            this._off(this.element, "mouseover", ".colheader, .rowheader, .expand, .collapse");
            this._off(this.element, "mouseleave", ".colheader, .rowheader, .expand, .collapse");
            this._off(this.element, "mouseover", ".value, .rowheader, .colheader", this.addHyperlinkHandler);
            this._off(this.element, "mouseleave", ".value, .rowheader, .colheader", this.removeHyperlinkHandler);
            this._off(this.element, "mouseover", ".value");
            this._off(this.element, "mouseleave", ".value");
        },

        _applyToolTip: function (e) {
            {
                var toolTipInfo; var rowValue; var cellPos; var columnValue = ""; valueCell = "";
                if ($(e.target).hasClass("summary")) {
                    if ($(e.target).siblings("th").hasClass("rowheader"))
                        cellPos = $(e.target).parents("tr").find(".rowheader").last().attr("p");
                    else
                        cellPos = $(e.target).parents("tr").find(".summary").attr("p");
                }
                else
                    cellPos = $(e.target).parents("tr").find(".rowheader").last().attr("p");
                if (cellPos != null) {
                    if ((cellPos[0] == 0) && ((parseInt(cellPos.split(",")[0])) != null)) {
                        if (cellPos == _proxy.getJSONRecords()[parseInt((parseInt(cellPos.split(",")[0]) * _proxy.rowCount) + parseInt(cellPos.split(",")[1]))].Index) {
                            rowValue = _proxy.getJSONRecords()[parseInt((parseInt(cellPos.split(",")[0]) * _proxy.rowCount) + parseInt(cellPos.split(",")[1]))].Value;
                        }
                    }
                    else
                        while (cellPos[0] >= 0) {
                            if (cellPos == _proxy.getJSONRecords()[parseInt((parseInt(cellPos.split(",")[0]) * _proxy.rowCount) + parseInt(cellPos.split(",")[1]))].Index) {
                                toolTipInfo = _proxy.getJSONRecords()[parseInt((parseInt(cellPos.split(",")[0]) * _proxy.rowCount) + parseInt(cellPos.split(",")[1]))].Value;
                                cellPos = parseInt(cellPos.split(",")[0]) - 1 + "," + parseInt(cellPos.split(",")[1]);
                            }
                            if (rowValue)
                                rowValue = toolTipInfo + "-" + rowValue;
                            else
                                rowValue = toolTipInfo;
                        }
                }
                var columnHeaderCount = $(e.target).closest('tbody').prev().children('tr:last').children('th').attr("p")[2];
                var currentCellPos = $(e.target).attr("p");
                if ((currentCellPos != undefined) && (currentCellPos == _proxy.getJSONRecords()[parseInt((parseInt(currentCellPos.split(",")[0]) * _proxy.rowCount) + parseInt(currentCellPos.split(",")[1]))].Index)) {
                    var colPos = parseInt(currentCellPos.split(",")[0]);
                }
                for (var i = 0; i <= columnHeaderCount; i++) {
                    var columnHeaderPos = colPos + "," + i;
                    if (colPos != null)
                        if (columnHeaderPos == _proxy.getJSONRecords()[parseInt((parseInt(columnHeaderPos.split(",")[0]) * _proxy.rowCount) + parseInt(columnHeaderPos.split(",")[1]))].Index) {
                            toolTipInfo = _proxy.getJSONRecords()[parseInt((parseInt(columnHeaderPos.split(",")[0]) * _proxy.rowCount) + parseInt(columnHeaderPos.split(",")[1]))].Value;
                            if (columnValue == "")
                                columnValue = toolTipInfo;
                            else if (toolTipInfo != "")
                                columnValue = columnValue + "-" + toolTipInfo;
                        }
                }
                this.element.find('#gridTooltip').remove();
                var tdPos = $(e.target).offset();
                $(e.target).append("<div id=\"gridTooltip\" class=\"tooltip\"></div>");
                this.element.find('#gridTooltip').css("left", (tdPos.left + 30) + 'px', "top", (tdPos.left + 30) + 'px');
                this.element.find('#gridTooltip').append("<p id=\"value\" class=\"tooltipText\">Value : " + $(e.target).text() + "</p>");
                this.element.find('#gridTooltip').append("<p id=\"row\" class=\"tooltipText\">Row : " + rowValue + "</p>");
                this.element.find('#gridTooltip').append("<p id=\"column\" class=\"tooltipText\">Column : " + columnValue + "</p>");
                this.element.find('#gridTooltip').show("fast");
            }
        },
        /**
        * This function is used to create and handle virtual scrollbar.
        *
        * @private  
        */
        _applyVScrolling: function () {
            var _proxy = this;
            var oGridDiv = $(this.element).find('.virtualScrollGrid');
            if (this._seriesPageCount > 1) {
                var getMinVerticalPos = function () { return (_proxy.element.find(".vScrollPanel").position().top + 1); };
                var vScrollBar = $(this.element).find(".vScrollPanel");
                vScrollBar.height($(oGridDiv).height());
                vScrollBar.html(ej.buildTag("div.vScrollThumb")[0].outerHTML);
                var vScrollHandle = vScrollBar.find(".vScrollThumb");
                vScrollHandle.height(Math.max(Math.ceil($(oGridDiv).height() / _proxy._seriesPageCount), 15));
                var vDragRange = vScrollBar.height() - vScrollHandle.height() - 2;
                vScrollHandle.offset({ top: getMinVerticalPos() + (_proxy._seriesCurrentPage - 1) * (vDragRange / (_proxy._seriesPageCount - 1)) });

                $(vScrollHandle).bind('mousedown', function (e) {
                    $(this).addClass("dragging");
                    var initCursorPos = e.pageY - getMinVerticalPos();
                    var initThumbPos = vScrollHandle.position().top - getMinVerticalPos();
                    _proxy.element.find(".seriesPageIndicator").removeClass("inActive").css({ top: vScrollHandle.position().top, left: vScrollHandle.position().left + 20 });
                    var currentPage = _proxy._seriesCurrentPage;
                    $(document).bind('mousemove', function (e) {
                        if (window.navigator.userAgent.indexOf('MSIE ') > -1)
                            $(document).on("selectstart", function (e) { e.preventDefault(); });
                        else
                            window.getSelection().removeAllRanges();
                        var movedDistance = (e.pageY - getMinVerticalPos()) - initCursorPos;
                        if (movedDistance > 0 && initThumbPos + movedDistance <= vDragRange)
                            vScrollHandle.offset({ top: initThumbPos + movedDistance + getMinVerticalPos() });
                        else if (movedDistance < 0 && initThumbPos + movedDistance >= 0)
                            vScrollHandle.offset({ top: initThumbPos + movedDistance + getMinVerticalPos() });
                        else if (initThumbPos + movedDistance > vDragRange)
                            vScrollHandle.offset({ top: getMinVerticalPos() + vDragRange });
                        else if (initThumbPos + movedDistance < 0)
                            vScrollHandle.offset({ top: getMinVerticalPos() });
                        currentPage = Math.ceil((vScrollHandle.position().top - getMinVerticalPos()) / (vDragRange / _proxy._seriesPageCount)) == 0 ? 1 : Math.ceil((vScrollHandle.position().top - getMinVerticalPos()) / (vDragRange / _proxy._seriesPageCount));
                        _proxy.element.find(".seriesPageIndicator").css({ top: vScrollHandle.position().top });
                        _proxy.element.find(".series_CurrentPage").html(currentPage);
                    });
                    $(document).bind('mouseup', function () {
                        $(document).off("selectstart");
                        $(this).unbind('mousemove').unbind('mouseup');
                        $(vScrollHandle).removeClass("dragging");
                        if (currentPage != _proxy._seriesCurrentPage) {
                            _proxy._seriesCurrentPage = currentPage;
                            _proxy.refreshPagedOlapGrid("series", _proxy._seriesCurrentPage);
                        }
                        else
                            _proxy.element.find(".seriesPageIndicator").addClass("inActive");
                    });
                });
            }

            if (this._categPageCount > 1) {
                var getMinHorizontalPos = function () { return (_proxy.element.find(".hScrollPanel").position().left + 1); };
                var hScrollBar = $(this.element).find(".hScrollPanel");
                hScrollBar.width($(oGridDiv).width());
                hScrollBar.html(ej.buildTag("div.hScrollThumb")[0].outerHTML);
                var hScrollHandle = hScrollBar.find(".hScrollThumb");
                hScrollHandle.width(Math.max(Math.ceil($(oGridDiv).width() / _proxy._categPageCount), 15));
                var hDragRange = hScrollBar.width() - hScrollHandle.width() - 2;
                hScrollHandle.offset({ left: getMinHorizontalPos() + (_proxy._categCurrentPage - 1) * (hDragRange / (_proxy._categPageCount - 1)) });

                $(hScrollHandle).bind('mousedown', function (e) {
                    $(this).addClass("dragging");
                    var initCursorPos = e.pageX - getMinHorizontalPos();
                    var initThumbPos = hScrollHandle.position().left - getMinHorizontalPos();
                    _proxy.element.find(".categPageIndicator").removeClass("inActive").css({ left: hScrollHandle.position().left, top: hScrollHandle.position().top + 20 });
                    var currentPage = _proxy._categCurrentPage;
                    $(document).bind('mousemove', function (e) {
                        if (window.navigator.userAgent.indexOf('MSIE ') > -1)
                            $(document).on("selectstart", function (e) { e.preventDefault(); });
                        else
                            window.getSelection().removeAllRanges();
                        var movedDistance = (e.pageX - getMinHorizontalPos()) - initCursorPos;
                        if (movedDistance > 0 && initThumbPos + movedDistance <= hDragRange)
                            hScrollHandle.offset({ left: initThumbPos + movedDistance + getMinHorizontalPos() });
                        else if (movedDistance < 0 && initThumbPos + movedDistance >= 0)
                            hScrollHandle.offset({ left: initThumbPos + movedDistance + getMinHorizontalPos() });
                        else if (initThumbPos + movedDistance > hDragRange)
                            hScrollHandle.offset({ left: getMinHorizontalPos() + hDragRange });
                        else if (initThumbPos + movedDistance < 0)
                            hScrollHandle.offset({ left: getMinHorizontalPos() });
                        currentPage = Math.ceil((hScrollHandle.position().left - getMinHorizontalPos()) / (hDragRange / _proxy._categPageCount)) == 0 ? 1 : Math.ceil((hScrollHandle.position().left - getMinHorizontalPos()) / (hDragRange / _proxy._categPageCount));
                        _proxy.element.find(".categPageIndicator").css({ left: hScrollHandle.position().left });
                        _proxy.element.find(".categ_CurrentPage").html(currentPage);
                    });
                    $(document).bind('mouseup', function () {
                        $(document).off("selectstart");
                        $(this).unbind('mousemove').unbind('mouseup');
                        $(hScrollHandle).removeClass("dragging");
                        if (currentPage != _proxy._categCurrentPage) {
                            _proxy._categCurrentPage = currentPage;
                            _proxy.refreshPagedOlapGrid("categ", _proxy._categCurrentPage);
                        }
                        else
                            _proxy.element.find(".categPageIndicator").addClass("inActive");
                    });
                });
            }
        },

        /**
        * To get the localized labels.
        * @private
        */
        //get Localized text
        _getLocalizedLabels: function (property) {
			return ej.PivotGrid.locale[this.locale()][property] === undefined ? ej.PivotGrid.locale["en-US"][property] : ej.PivotGrid.locale[this.locale()][property];
        },

        /**
        * This function would be raised while clicking expand/collapse icons to perform drill up/down operations in Pivot Grid.
        *
        * @private  
        */
        _drillDown: function (e) {
            this._off(this.element, "click", ".expand, .collapse");
            var targetClass = e.target.className;
            this._startDrilldown = true;
            targetClass = targetClass.split(" ");
            if (targetClass[0] == "expand")
                this._drillAction = "drilldown";
            else if (targetClass[0] == "collapse")
                this._drillAction = "drillup";

            if (this.model.beforeServiceInvoke != null)
                this._trigger("beforeServiceInvoke", { action: ej.PivotGrid.GridAction.DrillDown, element: this.element, customObject: this.model.customObject });
            var serializedCustomObject = JSON.stringify(this.model.customObject);
            var cellPos = $(e.target).parent().attr("p");
            this._drillCaption = $.trim($(e.currentTarget).parent().text());

            if (typeof oclientWaitingPopup != 'undefined' && oclientWaitingPopup != null) {
                if ($("#" + oclientProxy._id + "_maxView")[0]) {
                    $("#" + oclientProxy._id + "_maxView").ejWaitingPopup({ showOnInit: true });
                    this._maxViewLoading = $("#" + oclientProxy._id + "_maxView").data("ejWaitingPopup");
                }
                else
                    oclientWaitingPopup.show();
            }
            else {
                if (_proxy.model.progressMode == "infinite")
                    ogridWaitingPopup.show();
                else {
                    this._progressStatus(0, 70, 140);
                    if (_proxy.model.progressMode == "progress")
                        $("#" + _proxy._id + "progressText")[0].innerHTML = _proxy._getLocalizedLabels("PreparingAndExecutingMDXquery") + "...";
                }
            }
            var headerInfo = this.model.analysisMode == "relational" ? _proxy._drillHeaders.join("||") : this.getJSONRecords()[parseInt((parseInt(cellPos.split(",")[0]) * this.rowCount) + parseInt(cellPos.split(",")[1]))].Info;
            var report;
            try {
                report = JSON.parse(this.getOlapReport()).Report;
            }
            catch (err) {
                report = this.getOlapReport();
            }
            if (this.layout() != "" || this.layout() != "normal") {
                if (typeof (oclientProxy) != "undefined")
                    this.doAjaxPost("POST", this.model.url + "/" + this.model.serviceMethodSettings.drillDown, JSON.stringify({ "action": "drillDownGrid", "cellPosition": cellPos, "currentReport": report, "clientReports": reports, "headerInfo": headerInfo, "layout": this.layout(), "customObject": serializedCustomObject }), this._drillDownSuccess);
                else
                    this.doAjaxPost("POST", this.model.url + "/" + this.model.serviceMethodSettings.drillDown, JSON.stringify({ "action": "drillDownGrid", "cellPosition": cellPos, "currentReport": report, "headerInfo": headerInfo, "layout": this.layout(), "customObject": serializedCustomObject }), this._drillDownSuccess);
            }
            else {
                if (typeof (oclientProxy) != "undefined")
                    this.doAjaxPost("POST", this.model.url + "/" + this.model.serviceMethodSettings.drillDown, JSON.stringify({ "action": "drillDownGrid", "cellPosition": cellPos, "currentReport": report, "clientReports": reports, "headerInfo": headerInfo, "customObject": serializedCustomObject }), this._drillDownSuccess);
                else
                    this.doAjaxPost("POST", this.model.url + "/" + this.model.serviceMethodSettings.drillDown, JSON.stringify({ "action": "drillDownGrid", "cellPosition": cellPos, "currentReport": report, "headerInfo": headerInfo, "customObject": serializedCustomObject }), this._drillDownSuccess);
            }
        },

        /**
        * This function would apply hyperlink to the appropriate cells.
        *
        * @private  
        */
        _addHyperlink: function (e) {
            if ($(".hyperlinkValueCell")[0])
                $($(".hyperlinkValueCell")[0]).removeClass("hyperlinkValueCell");
            if ($(".hyperlinkHeaderCell")[0])
                $($(".hyperlinkHeaderCell")[0]).removeClass("hyperlinkHeaderCell");
            if (e.target.className == "value" && this.enableValueCellHyperlink())
                $(e.target).addClass("hyperlinkValueCell");
            else if (e.target.className == "summary value" && this.enableSummaryCellHyperlink())
                $(e.target).addClass("hyperlinkValueCell");
            else if (e.target.className == "rowheader" && this.enableRowHeaderHyperlink())
                $(e.target).addClass("hyperlinkHeaderCell");
            else if (e.target.className == "colheader" && this.enableColumnHeaderHyperlink())
                $(e.target).addClass("hyperlinkHeaderCell");
        },

        /**
        * This function would remove/disable hyperlink in Pivot Grid.
        *
        * @private  
        */
        _removeHyperlink: function (e) {
            $(e.target).removeClass("hyperlinkValueCell").removeClass("hyperlinkHeaderCell");
        },

        /**
        * This function would be raised while right clicking over any cell in Pivot Grid.
        *
        * @private  
        */
        _cellContext: function (e) {
            var currentCell = e.target || e.srcElement;
            if (_proxy.enableCellContext() == true && $(currentCell).parents("#" + _proxy._id)[0]) {
                var cellPos = $(currentCell).attr("p");
                if (cellPos) {
                    var cellPos = $(currentCell).attr("p");
                    var rawdata = _proxy.getJSONRecords()[parseInt((parseInt(cellPos.split(",")[0]) * _proxy.rowCount) + parseInt(cellPos.split(",")[1]))].Info;
                    var cellInfo = {
                        cellValue: currentCell.innerHTML,
                        cellPosition: cellPos,
                        cellType: currentCell.className.split(' ')[0],
                        uniqueName: rawdata.split('::')[0],
                        args: e,
                        rawdata: rawdata
                    };
                    if (cellInfo.cellType != "" && cellInfo.cellValue != "")
                        _proxy._trigger("cellContext", { args: cellInfo });
                    e.target ? e.preventDefault() : (window.event.returnValue = false);
                }
            }
        },

        /**
        * This function would be raised while exporting Pivot Grid to Excel.
        *
        * @private  
        */
        exportToExcel: function () {
            var args = {
                currentReport: _proxy.getOlapReport(),
                layout: _proxy.layout(),
                summaryCellColor: $(".summary").css("background-color")
            };
            this.doPostBack(_proxy.model.url + "/" + _proxy.model.serviceMethodSettings.exportOptions, args);
        },

        /**
        * This function is used to create progress bar containing status text and percentage.
        *
        * @private  
        */
        _createProgressBar: function () {
            var progressContainer = ej.buildTag("div#" + this._id + "progressContainer.progressContainer", "", { top: typeof (oclientProxy) != "undefined" ? ((oclientProxy.model.displaySettings.controlPlacement == "tab" || oclientProxy.model.displaySettings.mode != "chartAndGrid") ? oclientProxy.progressPos.top + 300 : (oclientProxy.model.displaySettings.defaultView == "chart" ? oclientProxy.progressPos.top + 500 : oclientProxy.progressPos.top + 200)) : ($("#" + this._id).position().top + ($("#" + this._id).height() / 2) - 30), left: typeof (oclientProxy) != "undefined" ? oclientProxy.progressPos.left + 550 : ($("#" + this._id).position().left + ($("#" + this._id).width() / 3) - 100) })[0].outerHTML;
            var progressBar = ej.buildTag("div#" + this._id + "progressBar.progressBar", "")[0].outerHTML;
            var blockControl = ej.buildTag("div#" + this._id + "blockDiv.blockDiv", "", { width: typeof (oclientProxy) != "undefined" ? ($("#" + oclientProxy._id).width()) : $("#" + this._id).width(), display: "block", opacity: "0.3", backgroundColor: "#E9E9E9", height: typeof (oclientProxy) != "undefined" ? ($("#" + oclientProxy._id).height()) : $("#" + this._id).height(), position: 'fixed', top: typeof (oclientProxy) != "undefined" ? oclientProxy.progressPos.top : ($("#" + this._id).position().top), left: typeof (oclientProxy) != "undefined" ? oclientProxy.progressPos.left : ($("#" + this._id).position().left) })[0].outerHTML;
            $("#" + _proxy._id).after(progressContainer);
            if (_proxy.model.progressMode == "normal") {
                $("#" + this._id + "progressContainer").css({ width: 0, height: 0 });
                $("#" + _proxy._id + "progressContainer").append(progressBar).append(blockControl);
            }
            else {
                var progressText = ej.buildTag("span#" + this._id + "progressText.progressText", "PivotGrid initial loading...")[0].outerHTML;
                $("#" + _proxy._id + "progressContainer").append(progressBar).append(progressText).append(blockControl);
            }
            $("#" + _proxy._id + "progressBar").ejProgressBar({ value: 0, text: 0 + " %", complete: "progressbarCompleted" });
            ogridProgressBar = $("#" + _proxy._id + "progressBar").data("ejProgressBar");
            ogridProgressText = $("#" + _proxy._id + "progressText");
        },

        /**
        * This function is used update the progress status.
        *
        * @private  
        */
        _progressStatus: function (current_Status, completion_Status, progress_Speed) {
            if (jQuery.type(completion_Status) === "string") {
                var progressText = completion_Status.split(':');
                if (progressText != undefined) completion_Status = parseInt(progressText[0]);
            }
            ogridProgressBar.setModel({ value: current_Status, text: current_Status + " %" });
            $("#" + _proxy._id + "progressContainer").show();
            $("#" + _proxy._id + "blockDiv").position = $("#" + _proxy._id).position();
            oGridTimer = window.setInterval(function () {
                ogridProgressBar.setModel({ value: current_Status, text: current_Status + " %" });
                current_Status++;
                if (current_Status >= completion_Status) {
                    window.clearInterval(oGridTimer);
                    if (progressText != undefined)
                        ogridProgressBar.setModel({ value: 100, text: progressText[1] });

                };
            }, progress_Speed);
        },

        /**
        * 	This function receives the controls update from service-end which would be utilized for rendering the widget.
        *
        * @private  
        */
        _renderControlSuccess: function (msg) {
            _proxy = this;
            if (_proxy.model.progressMode != "infinite") {
                if (_proxy.model.progressMode == "progress")
                    $("#" + _proxy._id + "progressText")[0].innerHTML = _proxy._getLocalizedLabels("MDXqueryExecutedSuccessfully") + "...";
                window.clearInterval(oGridTimer);
            }
            try {
                if (_proxy.model.progressMode != "infinite") {
                    if (_proxy.model.progressMode == "progress")
                        $("#" + _proxy._id + "progressText")[0].innerHTML = _proxy._getLocalizedLabels("RenderingStarted") + "...";
                    _proxy._progressStatus(70, 85, 50);
                    window.setTimeout(function () { progressStatus() }, 300);
                }
                else
                    window.setTimeout(function () { progressStatus() }, 1);
                function progressStatus() {
                    if (msg[0] != undefined && msg.length > 0) {
                        _proxy.setJSONRecords(msg[0].Value); _proxy.setOlapReport(msg[1].Value);
                        if (msg[2] != null && msg[2] != undefined && msg[2].Key == "PageSettings") {
                            if (_proxy.model.enableVirtualScrolling) {
                                _proxy._categPageCount = Math.ceil(JSON.parse(msg[3].Value).Column / JSON.parse(msg[2].Value).CategorialPageSize);
                                _proxy._seriesPageCount = Math.ceil(JSON.parse(msg[3].Value).Row / JSON.parse(msg[2].Value).SeriesPageSize);
                                _proxy._categCurrentPage = JSON.parse(msg[2].Value).CategorialCurrentPage;
                                _proxy._seriesCurrentPage = JSON.parse(msg[2].Value).SeriesPageSize;
                            }
                            else if ($(".e-olappager")[0] != null && $(".e-olappager")[0] != undefined)
                                _proxy._pagerObj.initPagerProperties(JSON.parse(msg[3].Value), JSON.parse(msg[2].Value));
                            if (msg[4] != null && msg[4] != undefined)
                                _proxy.model.customObject = msg[4].Value;
                        }
                        if (msg[2] != null && msg[2] != undefined && msg[2].Key != "PageSettings")
                            _proxy.model.customObject = msg[2].Value;
                    }
                    else if (msg.d != undefined && msg.d.length > 0) {
                        _proxy.setJSONRecords(msg.d[0].Value); _proxy.setOlapReport(msg.d[1].Value);
                        if (msg.d[2] != null && msg.d[2] != undefined && msg.d[2].Key == "PageSettings") {
                            if (_proxy.model.enableVirtualScrolling) {
                                _proxy._categPageCount = Math.ceil(JSON.parse(msg.d[3].Value).Column / JSON.parse(msg.d[2].Value).CategorialPageSize);
                                _proxy._seriesPageCount = Math.ceil(JSON.parse(msg.d[3].Value).Row / JSON.parse(msg.d[2].Value).SeriesPageSize);
                                _proxy._categCurrentPage = JSON.parse(msg.d[2].Value).CategorialCurrentPage;
                                _proxy._seriesCurrentPage = JSON.parse(msg.d[2].Value).SeriesCurrentPage;
                            }
                            else if ($(".e-olappager")[0] != null && $(".e-olappager")[0] != undefined)
                                _proxy._pagerObj.initPagerProperties(JSON.parse(msg.d[3].Value), JSON.parse(msg.d[2].Value));
                            if (msg.d[4] != null && msg.d[4] != undefined)
                                _proxy.model.customObject = msg.d[4].Value;
                        }
                        if (msg.d[2] != null && msg.d[2] != undefined && msg.d[2].Value == "PageSettings")
                            _proxy.model.customObject = msg.d[2].Value;
                    }
                    else if (!ej.isNullOrUndefined(msg) && ej.isNullOrUndefined(msg.d)) {
                        if (msg.PivotRecords != undefined) {
                            _proxy.setJSONRecords(msg.PivotRecords); _proxy.setOlapReport(msg.OlapReport);
                            if (msg.HeaderCounts != undefined && _proxy.model.enableVirtualScrolling) {
                                _proxy._categPageCount = Math.ceil(JSON.parse(msg.HeaderCounts).Column / JSON.parse(msg.PageSettings).CategorialPageSize);
                                _proxy._seriesPageCount = Math.ceil(JSON.parse(msg.HeaderCounts).Row / JSON.parse(msg.PageSettings).SeriesPageSize);
                                _proxy._categCurrentPage = JSON.parse(msg.PageSettings).CategorialCurrentPage;
                                _proxy._seriesCurrentPage = JSON.parse(msg.PageSettings).SeriesCurrentPage;
                            }
                            else if (msg.HeaderCounts != undefined && $(".e-olappager")[0] != null && $(".e-olappager")[0] != undefined)
                                _proxy._pagerObj.initPagerProperties(JSON.parse(msg.HeaderCounts), JSON.parse(msg.PageSettings));
                        }
                        if (msg.customObject != null && msg.customObject != undefined)
                            _proxy.model.customObject = msg.customObject;
                    }
                    if (_proxy.model.afterServiceInvoke != null)
                        _proxy._trigger("afterServiceInvoke", { action: ej.PivotGrid.GridAction.Initialize, element: _proxy.element, customObject: _proxy.model.customObject });
                    if (_proxy.getJSONRecords() != null && _proxy.getJSONRecords().length > 0)
                        _proxy.renderControlFromJSON(_proxy.getJSONRecords());
                    if (_proxy.model.enableVirtualScrolling)
                        _proxy._applyVScrolling();
                    if (_proxy.model.progressMode != "infinite") {
                        if (_proxy._JSONRecords == null)
                            $("#" + _proxy._id + "progressContainer").hide();
                        else {
                            if (_proxy.model.progressMode == "progress")
                                $("#" + _proxy._id + "progressText")[0].innerHTML = _proxy._getLocalizedLabels("RenderingSucceeded") + "...";
                        }
                    }
                    else if (typeof oclientWaitingPopup != 'undefined' && oclientWaitingPopup != null) {
                        if ($("#" + oclientProxy._id + "_maxView")[0] && this._maxViewLoading) {
                            this._maxViewLoading.hide();
                        }
                        else
                            oclientWaitingPopup.hide();
                    }
                    else
                        ogridWaitingPopup.hide();
                }
            }
            catch (err) {
                if (_proxy.model.progressMode != "infinite") {
                    if (_proxy.model.progressMode == "progress")
                        $("#" + _proxy._id + "progressText")[0].innerHTML = _proxy._getLocalizedLabels("RenderingFailed") + "!!!";
                    window.clearInterval(oChartTimer);
                    ogridProgressBar.setModel({ value: 70, text: " Failed" });
                }
            }
            var eventArgs = { action: "initialize", customObject: this.model.customObject, element: this.element };
            this._trigger("renderSuccess", eventArgs);
        },

        /**
        * 	This function receives the controls update from service-end after drilldown which would be utilized for rendering the widget.
        *
        * @private  
        */
        _drillDownSuccess: function (msg) {
            _proxy = this;
            if (_proxy.model.progressMode == "progress")
                $("#" + _proxy._id + "progressText")[0].innerHTML = _proxy._getLocalizedLabels("MDXqueryExecutedSuccessfully") + "...";
            if (msg[0] != undefined) {
                this.setJSONRecords(msg[0].Value);
                this.setOlapReport(msg[1].Value);
                if (typeof (oclientProxy) != "undefined") {
                    oclientProxy.currentReport = msg[1].Value;
                    oclientProxy.reports = msg[2].Value;
                }
                if (msg[3] != null && msg[3] != undefined && msg[3].Key == "PageSettings") {
                    if (_proxy.model.enableVirtualScrolling) {
                        _proxy._categPageCount = Math.ceil(JSON.parse(msg[4].Value).Column / JSON.parse(msg[3].Value).CategorialPageSize);
                        _proxy._seriesPageCount = Math.ceil(JSON.parse(msg[4].Value).Row / JSON.parse(msg[3].Value).SeriesPageSize);
                        _proxy._categCurrentPage = JSON.parse(msg[3].Value).CategorialCurrentPage;
                        _proxy._seriesCurrentPage = JSON.parse(msg[3].Value).SeriesPageSize;
                    }
                    else if ($(".e-olappager")[0] != null && $(".e-olappager")[0] != undefined)
                        _proxy._pagerObj.initPagerProperties(JSON.parse(msg[4].Value), JSON.parse(msg[3].Value));
                    if (msg[5] != null && msg[5] != undefined)
                        _proxy.model.customObject = msg[5].Value;
                }
                if (msg[3] != null && msg[3] != undefined && msg[3].Key != "PageSettings")
                    _proxy.model.customObject = msg[3].Value;
            }
            else if (msg.d != undefined) {
                this.setJSONRecords(msg.d[0].Value);
                this.setOlapReport(msg.d[1].Value);
                if (typeof (oclientProxy) != "undefined") {
                    oclientProxy.currentReport = msg.d[1].Value;
                    oclientProxy.reports = msg.d[2].Value;
                }
                if (msg.d[3] != null && msg.d[3] != undefined && msg.d[3].Key == "PageSettings") {
                    if (_proxy.model.enableVirtualScrolling) {
                        _proxy._categPageCount = Math.ceil(JSON.parse(msg.d[4].Value).Column / JSON.parse(msg.d[3].Value).CategorialPageSize);
                        _proxy._seriesPageCount = Math.ceil(JSON.parse(msg.d[4].Value).Row / JSON.parse(msg.d[3].Value).SeriesPageSize);
                        _proxy._categCurrentPage = JSON.parse(msg.d[3].Value).CategorialCurrentPage;
                        _proxy._seriesCurrentPage = JSON.parse(msg.d[3].Value).SeriesCurrentPage;
                    }
                    else if ($(".e-olappager")[0] != null && $(".e-olappager")[0] != undefined)
                        _proxy._pagerObj.initPagerProperties(JSON.parse(msg.d[4].Value), JSON.parse(msg.d[3].Value));
                    if (msg.d[5] != null && msg.d[5] != undefined)
                        _proxy.model.customObject = msg.d[5].Value;
                }
                if (msg.d[3] != null && msg.d[3] != undefined && msg.d[3].Key != "PageSettings")
                    _proxy.model.customObject = msg.d[3].Value;
            }
            else {
                this.setJSONRecords(msg.PivotRecords);
                this.setOlapReport(msg.OlapReport);
                if (typeof (oclientProxy) != "undefined") {
                    oclientProxy.currentReport = msg.OlapReport;
                    oclientProxy.reports = msg.reports;
                }
                if (msg.length > 3) {
                    if (_proxy.model.enableVirtualScrolling) {
                        _proxy._categPageCount = Math.ceil(JSON.parse(msg.HeaderCounts).Column / JSON.parse(msg.PageSettings).CategorialPageSize);
                        _proxy._seriesPageCount = Math.ceil(JSON.parse(msg.HeaderCounts).Row / JSON.parse(msg.PageSettings).SeriesPageSize);
                        _proxy._categCurrentPage = JSON.parse(msg.PageSettings).CategorialCurrentPage;
                        _proxy._seriesCurrentPage = JSON.parse(msg.PageSettings).SeriesCurrentPage;
                    }
                    else if ($(".e-olappager")[0] != null && $(".e-olappager")[0] != undefined)
                        _proxy._pagerObj.initPagerProperties(JSON.parse(msg.HeaderCounts), JSON.parse(msg.PageSettings));
                }
                if (msg.customObject != null && msg.customObject != undefined)
                    _proxy.model.customObject = msg.customObject;
            }
            if (this.model.afterServiceInvoke != null)
                this._trigger("afterServiceInvoke", { action: ej.PivotGrid.GridAction.DrillDown, element: this.element, customObject: this.model.customObject });
            if (_proxy.model.progressMode == "progress")
                $("#" + _proxy._id + "progressText")[0].innerHTML = _proxy._getLocalizedLabels("RenderingStarted") + "...";
            this.renderControlFromJSON(this.getJSONRecords());
            this.model.currentReport = this.getOlapReport();
            this._trigger("drillSuccess", this.element);
            if (_proxy.model.enableVirtualScrolling)
                _proxy._applyVScrolling();
            if (typeof oclientWaitingPopup != 'undefined' && oclientWaitingPopup != null) {
                if ($("#" + oclientProxy._id + "_maxView")[0] && this._maxViewLoading) {
                    this._maxViewLoading.hide();
                    oclientWaitingPopup.hide();
                }
                else if (oclientProxy.model.displaySettings.mode == "gridOnly")
                    oclientWaitingPopup.hide();
            }
            else if (typeof oclientProxy != 'undefined') {
                if (oclientProxy && !oclientProxy.ogridObj._startDrilldown && !oclientProxy.ochartObj._startDrilldown || oclientProxy.model.displaySettings.mode == "gridOnly")
                    oclientWaitingPopup.hide();
            }
            else if (_proxy.model.progressMode == "infinite" && typeof oclientWaitingPopup == 'undefined') {
                ogridWaitingPopup = $("#" + this._id).data("ejWaitingPopup");
                ogridWaitingPopup.hide();
            }
            if (typeof oclientProxy != 'undefined')
                oclientProxy.ogridObj._startDrilldown = false;
            else
                ogridWaitingPopup.hide();
            var eventArgs = { action: this._drillAction, customObject: this.model.customObject, element: this.element };
            this._trigger("renderSuccess", eventArgs);
        },

        /**
        * Perform an asynchronous HTTP (Ajax) request.
        * @return jQuery
        * @example 
        * &lt;div id="PivotGrid"&gt;&lt;/div&gt; <br> 
        * &lt;script&gt;
        * // Create Pivot Grid
        * $('#PivotGrid').ejPivotGrid({
        *       url: "PivotGridService.svc",
        *   });
        * var gridObj = $("#PivotGrid").data("ejPivotGrid");
        * gridObj.doAjaxPost("POST", "/PivotGridService.svc/Initialize", {"key", "Hello World"}, "_renderControlSuccess", null);
        * // Initiate an Ajax request.
        * &lt;/script&gt;
        * @memberof ejPivotGrid
        * @instance
        */
        doAjaxPost: function (type, url, data, onSuccess, onComplete) {
            $.ajax({
                type: type,
                url: url,
                contentType: 'application/json; charset=utf-8',
                dataType: 'json',
                data: data,
                success: $.proxy(onSuccess, this),
                complete: ej.proxy(function (onComplete) {
                    $.proxy(onComplete, this);
                    var eventArgs = { "action": this._currentAction, "customObject": this.model.customObject, element: this.element };
                    this._trigger("renderComplete", eventArgs);
                }, this),
                error: ej.proxy(function (msg, textStatus, errorThrown) {
                    if (_proxy.model.progressMode != "infinite") {
                        window.clearInterval(oGridTimer);
                        if (_proxy.model.progressMode == "progress")
                            $("#" + _proxy._id + "progressText")[0].innerHTML = _proxy._getLocalizedLabels("MDXqueryExecutionFailed") + "!!!";
                        $("#" + ogridProgressBar._id).show();
                        ogridProgressBar.setModel({ value: 0, text: errorThrown });
                    }
                    else
                        ogridWaitingPopup.hide();
                    var eventArgs = { "action": this._drillAction != "" ? this._drillAction : "initialize", "customObject": this.model.customObject, "element": this.element, "Message": msg };
                    this._trigger("renderFailure", eventArgs);
                }, this)
            });
        },

        /**
        * Perform an asynchronous HTTP (FullPost) submit.
        * @return jQuery
        * @example 
        * &lt;div id="PivotGrid"&gt;&lt;/div&gt; <br> 
        * &lt;script&gt;
        * // Submitting required informatin to the service.
        * $('#PivotGrid').ejPivotGrid({
        *       url: "PivotGridService.svc",
        *   });
        * var gridObj = $("#PivotGrid").data("ejPivotGrid");
        * gridObj.doPostBack("/PivotGridService.svc/Initialize", {"key", "Hello World"});
        * // Initiate an submit.
        * &lt;/script&gt;
        * @memberof ejPivotGrid
        * @instance
        */
        doPostBack: function (url, params) {
            var form = $('<form>').attr({ 'action': url, 'method': 'POST', 'name': 'export' });
            var addParam = function (paramName, paramValue) {
                var input = $('<input type="hidden" title="params">').attr({
                    'id': paramName,
                    'name': paramName,
                    'value': paramValue
                }).appendTo(form);
            };
            for (var item in params)
                addParam(item, params[item]);
            form.appendTo(document.body).submit().remove();
        },

        /**
        * 	This function re-renders the Pivot Grid on clicking the navigating buttons on Pivot Pager.
        * @return jQuery
        * @example 
        * &lt;div id="PivotGrid"&gt;&lt;/div&gt; <br> 
        * &lt;script&gt;
        * // Refreshing Pivot Grid while perform paging.
        * $('#PivotGrid').ejPivotGrid({
        *       url: "PivotGridService.svc",
        *   });
        * var gridObj = $("#PivotGrid").data("ejPivotGrid");
        * gridObj.refreshPagedOlapGrid("series", 2);
        * // Initiate an Ajax request.
        * &lt;/script&gt;
        * @memberof ejPivotGrid
        * @instance
        */
        refreshPagedOlapGrid: function (axis, pageNo) {
            if (_proxy.model.progressMode == "infinite" && typeof oclientWaitingPopup == 'undefined') {
                ogridWaitingPopup = $("#" + this._id).data("ejWaitingPopup");
                ogridWaitingPopup.show();
            }
            else {
                this._progressStatus(0, 70, 140);
                if (_proxy.model.progressMode == "progress")
                    $("#" + _proxy._id + "progressText")[0].innerHTML = "Preparing and executing MDX query...";
            }
            axis = axis.indexOf('categ') != -1 ? "categorical" : "series";
            var report;
            try {
                report = JSON.parse(this.getOlapReport()).Report;
            }
            catch (err) {
                report = this.getOlapReport();
            }
            this.doAjaxPost("POST", this.model.url + "/" + this.model.serviceMethodSettings.paging, JSON.stringify({ "action": "Paging", "pagingInfo": axis + ":" + pageNo, "currentReport": report, "layout": this.layout(), "customObject": null }), this._renderControlSuccess);
        },

        /**
        * 	This function receives the JSON formatted datasource to render the Pivot Grid control.
        * @return jQuery
        * @example 
        * &lt;div id="PivotGrid"&gt;&lt;/div&gt; <br> 
        * &lt;script&gt;
        * // Rendering Pivot Grid from given JSON formatted data.
        * $('#PivotGrid').ejPivotGrid({
        *       url: "PivotGridService.svc",
        *   });
        * var gridObj = $("#PivotGrid").data("ejPivotGrid");
        * gridObj.renderControlFromJSON({this.getJSONRecords()});
        * // Rendering Pivot Grid.
        * &lt;/script&gt;
        * @memberof ejPivotGrid
        * @instance
        */
        renderControlFromJSON: function (jsonObj) {
            _proxy = this;
            if (_proxy.model.progressMode != "infinite")
                window.clearInterval(oGridTimer);
            var tableOlapGrid = $("<table cellspacing=\"0\" cellpadding=\"0\" role='grid'  aria-readonly='true'></table>");
            this.rowCount = 0;

            if (jsonObj != null) {
                jQuery.each(jsonObj, function (index, value) {
                    if (this.Index[0] == 0)
                        _proxy.rowCount++;
                });

                var theadOlapGrid = $("<thead></thead>");
                var tbodyOlapGrid = $("<tbody></tbody>");
                var istheadOlapGrid = true; var columnHeaderHeight = 0;
                if (jsonObj[0].CSS == "none")
                    columnHeaderHeight = jsonObj[0].RowSpan - 1;
                else {
                    jQuery.each(jsonObj, function (index, value) {
                        columnHeaderHeight++;
                        return (jsonObj[index].CSS == "colheader");
                    });
                    columnHeaderHeight = columnHeaderHeight - 2;
                }

                jQuery.each(jsonObj, function (index, value) {
                    if (index < _proxy.rowCount) {
                        if (istheadOlapGrid) {
                            istheadOlapGrid = (index >= columnHeaderHeight) ? false : true;
                            var tRow = "<tr role='row'>";
                            for (var column = index; column < jsonObj.length; column = column + _proxy.rowCount) {
                                if (jsonObj[column].Span != "Block") {
                                    tRow = tRow + "<th" + (jsonObj[column].CSS == "none" ? "" : " class=" + jsonObj[column].CSS) + " p=" + jsonObj[column].Index + " colspan=" + jsonObj[column].ColSpan + " rowspan=" + jsonObj[column].RowSpan + " role='columnheader'>" + (jsonObj[column].State == 0 ? "<span style=\"margin-left: 10px\"></span>" : jsonObj[column].State == 2 ? "<span class=\"expand\ e-icon\">&nbsp;</span>" : (jsonObj[column].State == 1 ? "<span class=\"collapse\ e-icon\" title=\"Collapsed\">&nbsp;</span>" : "")) + jsonObj[column].Value + "</th>";
                                    if (jsonObj[column].RowSpan > 1 && jsonObj[column].ColSpan <= 1) {
                                        for (var i = column + 1; i < column + jsonObj[column].RowSpan; i++)
                                            jsonObj[i].Span = "Block";
                                    }
                                    else if (jsonObj[column].ColSpan > 1 && jsonObj[column].RowSpan <= 1) {
                                        for (var i = column + _proxy.rowCount; i < column + (jsonObj[column].ColSpan * _proxy.rowCount); i = i + _proxy.rowCount)
                                            jsonObj[i].Span = "Block";
                                    }
                                    else if (jsonObj[column].ColSpan > 1 && jsonObj[column].RowSpan > 1) {
                                        for (var i = column; i < column + jsonObj[column].RowSpan; i++) {
                                            for (var j = i + _proxy.rowCount; j < i + (jsonObj[column].ColSpan * _proxy.rowCount); j = j + _proxy.rowCount)
                                                jsonObj[j].Span = "Block";
                                            jsonObj[i].Span = "Block";
                                        }
                                    }
                                }
                            }
                            tRow += "</tr>";
                            $(theadOlapGrid).append(tRow);
                        }
                        else {
                            var tRow = "<tr role='row'>";
                            for (var column = index; column < jsonObj.length; column = column + _proxy.rowCount) {
                                if (jsonObj[column].Span != "Block") {
                                    if (jsonObj[column].CSS == "rowheader" || jsonObj[column].CSS == "summary" || jsonObj[column].CSS == "summary") {
                                        if (_proxy.layout().toLowerCase() == "excellikelayout") {
                                            tRow = tRow + "<th" + (jsonObj[column].CSS == "none" ? "" : " class=\"" + jsonObj[column].CSS) + "\" p=" + jsonObj[column].Index + " colspan=" + jsonObj[column].ColSpan + " rowspan=" + jsonObj[column].RowSpan + " role='rowheader'>" + (jsonObj[column].State == 2 ? "<span style=\"margin-left:" + (jsonObj[column].Level * 10) + "px\" class=\"expand\ e-icon\">&nbsp;</span>" : (jsonObj[column].State == 1 ? "<span style=\"margin-left:" + (jsonObj[column].Level * 10) + "px\" class=\"collapse\ e-icon\">&nbsp;</span>" : "<span style=\"margin-left:" + (jsonObj[column].Level * 10) + "px\">&nbsp;</span>")) + jsonObj[column].Value + "</th>";
                                        }
                                        else {
                                            tRow = tRow + "<th" + (jsonObj[column].CSS == "none" ? "" : " class=\"" + jsonObj[column].CSS) + "\" p=" + jsonObj[column].Index + " colspan=" + jsonObj[column].ColSpan + " rowspan=" + jsonObj[column].RowSpan + " role='rowheader'>" + (jsonObj[column].State == 0 ? "<span style=\"margin-left: 10px\"></span>" : jsonObj[column].State == 2 ? "<span class=\"expand\ e-icon\">&nbsp;</span>" : (jsonObj[column].State == 1 ? "<span class=\"collapse\ e-icon\">&nbsp;</span>" : "")) + jsonObj[column].Value + "</th>";
                                        }
                                    }
                                    else
                                        tRow = tRow + "<td" + (jsonObj[column].CSS == "none" ? "" : " class=\"" + jsonObj[column].CSS) + "\" p=" + jsonObj[column].Index + " colspan=" + jsonObj[column].ColSpan + " rowspan=" + jsonObj[column].RowSpan + " role='gridcell'>" + (jsonObj[column].State == 2 ? "<span class=\"expand\ e-icon\">&nbsp;</span>" : (jsonObj[column].State == 1 ? "<span class=\"collapse\ e-icon\">&nbsp;</span>" : "")) + jsonObj[column].Value + "</td>";

                                    if (jsonObj[column].RowSpan > 1 && jsonObj[column].ColSpan <= 1) {
                                        for (var i = column + 1; i < column + jsonObj[column].RowSpan; i++)
                                            jsonObj[i].Span = "Block";
                                    }
                                    else if (jsonObj[column].ColSpan > 1 && jsonObj[column].RowSpan <= 1) {
                                        for (var i = column + _proxy.rowCount; i < column + (jsonObj[column].ColSpan * _proxy.rowCount); i = i + _proxy.rowCount)
                                            jsonObj[i].Span = "Block";
                                    }
                                    else if (jsonObj[column].ColSpan > 1 && jsonObj[column].RowSpan > 1) {
                                        for (var i = column; i < column + jsonObj[column].RowSpan; i++) {
                                            for (var j = i + _proxy.rowCount; j < i + (jsonObj[column].ColSpan * _proxy.rowCount); j = j + _proxy.rowCount)
                                                jsonObj[j].Span = "Block";
                                            jsonObj[i].Span = "Block";
                                        }
                                    }
                                }
                            }
                            tRow += "</tr>";
                            $(tbodyOlapGrid).append(tRow);
                        }

                    }
                });
                if (_proxy.model.progressMode != "infinite") {
                    window.clearInterval(oGridTimer);
                    this._progressStatus(85, 90, 80);
                    window.setTimeout(function () { progressStatus() }, 400);
                    function progressStatus() {
                        $(tableOlapGrid).append(theadOlapGrid);
                        $(tableOlapGrid).append(tbodyOlapGrid);
                    }
                }
                else {
                    $(tableOlapGrid).append(theadOlapGrid);
                    $(tableOlapGrid).append(tbodyOlapGrid);
                }
            }
            if (_proxy.model.progressMode != "infinite") {
                window.clearInterval(oGridTimer);
                this._progressStatus(90, 101 + ":100%", 80);
                window.setTimeout(function () { timeOut() }, 900);
                function timeOut() {
                    _proxy.element.html("");
                    if (_proxy.model.enableVirtualScrolling) {
                        _proxy._createVirtualOlapGrid(tableOlapGrid);
                    }
                    else
                        _proxy.element.append(tableOlapGrid);
                    _proxy._unWireEvents();
                    _proxy._wireEvents();
                    $("#" + _proxy._id + "progressContainer").hide();
                }
            }
            else {
                this.element.html("");
                if (_proxy.model.enableVirtualScrolling) {
                    _proxy._createVirtualOlapGrid(tableOlapGrid);
                }
                else
                    this.element.append(tableOlapGrid);
                if (_proxy.model.enableRTL) {
                    this.element.find('table').addClass("e-rtl");
                    this.element.find('table .rowheader,.colheader,.summary').css("text-align", "right");
                }
                else {
                    this.element.find('table').removeClass("e-rtl");
                    this.element.find('table .rowheader,.colheader,.summary').css("text-align", "left");
                }
                this._unWireEvents();
                this._wireEvents();
            }
        },

        /**
        * 	This function appends virtual scroll bar to the Pivot Grid.
        *
        * @private  
        */
        _createVirtualOlapGrid: function (tableOlapGrid) {
            var verticalScroll = _proxy._seriesPageCount > 1 ? ej.buildTag("div.vScrollPanel")[0].outerHTML : "";
            var horizontalScroll = _proxy._categPageCount > 1 ? ej.buildTag("div.hScrollPanel")[0].outerHTML : "";
            var oGridDiv = ej.buildTag("td.virtualScrollGrid", tableOlapGrid[0].outerHTML)[0];
            var vertiScrollTd = ej.buildTag("td.virtualScrollElement", verticalScroll);
            var horiScrollTd = ej.buildTag("td.virtualScrollElement", horizontalScroll);
            var trow = ej.buildTag("tr.virtualScrollElement", oGridDiv.outerHTML + vertiScrollTd[0].outerHTML);
            var trow2 = ej.buildTag("tr.virtualScrollElement", horiScrollTd[0].outerHTML + ej.buildTag("td.virtualScrollElement")[0].outerHTML);
            var virtualOlapGrid = ej.buildTag("tbody.oGridOuterDiv.virtualScrollElement", trow[0].outerHTML + trow2[0].outerHTML);
            this.element.html(ej.buildTag("table.virtualScrollElement", virtualOlapGrid[0].outerHTML));
            var seriesPageIndicator = ej.buildTag("div.seriesPageIndicator inActive", ej.buildTag("span.axislabel", this._getLocalizedLabels("SeriesPage"))[0].outerHTML + ej.buildTag("span.series_CurrentPage", _proxy._seriesCurrentPage)[0].outerHTML + " / " + ej.buildTag("span.series_pageCount", _proxy._seriesPageCount)[0].outerHTML)[0].outerHTML;
            if (_proxy._seriesPageCount > 1) _proxy.element.append(seriesPageIndicator);
            var categPageIndicator = ej.buildTag("div.categPageIndicator inActive", ej.buildTag("span.axislabel", this._getLocalizedLabels("CategoricalPage"))[0].outerHTML + ej.buildTag("span.categ_CurrentPage", _proxy._categCurrentPage)[0].outerHTML + " / " + ej.buildTag("span.categ_pageCount", _proxy._categPageCount)[0].outerHTML)[0].outerHTML;
            if (_proxy._categPageCount > 1) _proxy.element.append(categPageIndicator);
        }
    });

    /**
    * Enum for Layout Types. Allows the user access the different layouts for Pivot Grid. 
    *
    * @enum {String}
    * @global
    */
    ej.PivotGrid.Layout = {
        Normal: "normal",
        NormalTopSummary: "normaltopsummary",
        NoSummaries: "nosummaries",
        ExcelLikeLayout: "excellikelayout"
    };

    //Pivot Grid Localization
   ej.PivotGrid.locale = {};
    //Default english localized values
    ej.PivotGrid.locale["en-US"] = {
        SeriesPage: "Series Page",
        CategoricalPage: "Categorical Page",
        MDXqueryExecutionFailed: "MDX query execution failed",
        PreparingAndExecutingMDXquery: "Preparing and executing MDX query",
        MDXqueryExecutedSuccessfully: "MDX query executed successfully",
        RenderingStarted: "Rendering Started",
        RenderingSucceeded: "Rendering Succeeded",
        RenderingFailed: "Rendering failed"
    };

    /**
    * Enum for Grid Action. Contains the information about current action of the Pivot Grid. 
    *
    * @enum {String}
    * @global
    */
    ej.PivotGrid.GridAction = {
        Initialize: "initialize",
        DrillDown: "drillDown"
    };

    /**
    * Enum for Progress Mode. Allows the user to select a appropriate progress indicator. 
    *
    * @enum {String}
    * @global
    */
    ej.PivotGrid.ProgressMode = {
        Infinite: "infinite",
        Normal: "normal",
        Progress: "progress"
    };

    ej.PivotGrid.AnalysisMode = {
        Olap: "olap",
        Relational: "relational"
    };

})(jQuery, Syncfusion); ;
/**
* @fileOverview OLAP Grid control to visualize the data
* @copyright Copyright Syncfusion Inc. 2001 - 2013. All rights reserved.
*  Use of this code is subject to the terms of our license.
*  A copy of the current license can be obtained at any time by e-mailing
*  licensing@syncfusion.com. Any infringement will be prosecuted under
*  applicable laws. 
* @version 12.1 
* @author <a href="mailto:licensing@syncfusion.com">Syncfusion Inc</a>
*/

(function ($, ej, undefined) {

	/**
    * @namespace ej
	* @class ejOlapGrid
    * @requires jquery-1.10.2.min.js
    * @requires jquery.easing.1.3.min.js
    * @requires ej.core.js
    * @requires ej.data.js
    * @requires ej.touch.js
    * @requires ej.waitingpopup.js
    * @requires ej.olapgrid.js
    * @requires ej.olappager.js
	* @classdesc Custom Design for OLAP Grid control.
	* @example 
	* &lt;div id="OlapGrid"&gt; &lt;/div&gt; <br> 
	* &lt;script&gt;
	* // Create OLAP Grid
    * $("#OlapGrid").ejOlapGrid(...); 	
	* &lt;/script&gt;
	*/
    ej.widget("ejOlapGrid", "ej.olap.OlapGrid", {

        // Root Css Class
        _rootCSS: "e-olapgrid",

        // Widget element will be automatically set in this
        element: null,

        // User-defined model will be automatically set in this
        model: null,

        validTags: ["div", "span"],

        defaults: {
            //properties
			/**		
			* Connects the service using the specified URL for any server updates.	See {@link url}
			* @default ""
			* @type {String}
			* @example 
			* //To set service url during initialization  
			* $("#OlapGrid").ejOlapGrid({url: "/wcf/OlapGridService.svc"});
			* @example 
			* //Get or set the size API, after initialization:<br>
            * //Gets the url value  
            * $("#OlapGrid").ejOlapGrid("option","url");<br>   
            * //Sets the url value 
            * $("#OlapGrid").ejOlapGrid("option","url", "/wcf/OlapGridService.svc" ); 
            * @memberof ejOlapGrid
            * @instance
            */
            url: "",

            /**		
            * Sets the CSS name for custom operation.
            * @default ""
            * @type {String}
            * @example 
            * //To set the CSS name during initialization  
            * $("#OlapGrid").ejOlapGrid({cssClass: "gradient-lime"});	
            * @example 
			* //Get or set css class for initialization:<br>
			*	// Gets the css name.		
			*	$("#OlapGrid").ejOlapGrid("option", "cssClass");			
			*	// Sets the rounded corner to button
			*	$("#OlapGrid").ejOlapGrid("option", "cssClass",  "gradient-lime" ); 		
			* @memberof ejOlapGrid
			* @instance
			*/
            cssClass: "",

            /**		
            * Contains the serialized OlapReport at the instant.
            * @default ""
            * @type {String}	
            * @memberof ejOlapGrid
            * @instance
            */
            currentReport: "",

            /**		
            * Allows the user access the different layouts for OLAP Grid.
            * @default ej.olap.OlapGrid.Layout.Normal
			* @type {enum}
            * @example 
            * //To set the layout during initialization  
            * $("#OlapGrid").ejOlapGrid({layout: ej.olap.OlapGrid.Layout.NoSummaries});
            * @example 
            * //Get or set the grid layout, after initialization:<br>
            *	//Gets the layout value
            *	$("#OlapGrid").ejOlapGrid("option", "layout");<br>			
            *	//Sets the grid layout
            *	$("#OlapGrid").ejOlapGrid("option", "layout",ej.olap.OlapGrid.Layout.Normal);			* 		
            * @memberof ejOlapGrid
            * @instance
            */
            layout: "Normal",

            /**		
            * Allows the user to load large amount of data into pages.
            * @default false
            * @type {Boolean}
            * @example 
            * //To set the virtual scrolling during initialization  
            * $("#OlapGrid").ejOlapGrid({enableVirtualScrolling: true});
            * @example 
            * //Get or set the virtual scrolling, after initialization:<br>
            *	//Gets the virtual calling state
			*	$("#OlapGrid").ejOlapGrid("option", "enableVirtualScrolling");<br>		
			*	//Sets the virtual scrolling 
			*	$("#OlapGrid").ejOlapGrid("option", "enableVirtualScrolling","true");			* 		
			* @memberof ejOlapGrid
			* @instance
			*/
            enableVirtualScrolling: false,

			/**		
			* Allows the user to access/operate each cell through a custom design on right click.
			* @default false
			* @type {Boolean}
			* @example 
			* //To set the cell context option during initialization  
			* $("#OlapGrid").ejOlapGrid({enableCellContext: true});
            * @example 
			* //Get or set the cell context, after initialization:<br>
			*	//Gets the cell context state
			*	$("#OlapGrid").ejOlapGrid("option", "enableCellContext");<br>			
			*	//Sets the cell context value
			*	$("#OlapGrid").ejOlapGrid("option", "enableCellContext","true");			* 		
			* @memberof ejOlapGrid
			* @instance
			*/
            enableCellContext: false,

            /**		
            * Allows the user to enable RTL support.
            * @default false
            * @type {Boolean}
            * @example 
            * //To set the enableRTL option during initialization  
            * $("#OlapGrid").ejOlapGrid({enableRTL: true});
            * @example 
            * //Get or set the enableRTL, after initialization:<br>
            *	//Gets the enableRTL values state
            *	$("#OlapGrid").ejOlapGrid("option", "enableRTL");<br>			
            *	//Sets the cell context value
            *	$("#OlapGrid").ejOlapGrid("option", "enableRTL","true");			* 		
            * @memberof ejOlapGrid
            * @instance
            */
            enableRTL: false,

            /**		
            * Allows the user to enable ToolTip support.
            * @default false
            * @type {Boolean}
            * @example 
            * //To set the enableToolTip option during initialization  
            * $("#OlapGrid").ejOlapGrid({enableToolTip: true});
            * @example 
            * //Get or set the enableToolTip, after initialization:<br>
            *	//Gets the enableToolTip values state
            *	$("#OlapGrid").ejOlapGrid("option", "enableToolTip");<br>			
            *	//Sets the cell context value
            *	$("#OlapGrid").ejOlapGrid("option", "enableToolTip","true");			* 		
            * @memberof ejOlapGrid
            * @instance
            */
            enableToolTip: true,
			/**	
            * Allows the user to configure hyperlink settings to OLAP Grid control.
            * @default false
            * @type {Object}
            * @example 
            * //To configure the hyperlink settings to OLAP Grid during initialization  
            * $("#OlapGrid").ejOlapGrid({hyperlinkSettings:{enableValueCellHyperlink: true, enableRowHeaderHyperlink: true}});
            * @example 
            * //Get or set the hyperlink settings, after initialization:<br>
            *	//Gets the hyperlink settings
            *	$("#OlapGrid").ejOlapGrid("option", "hyperlinkSettings");<br>			
            *	//Sets the hyperlink settings to OLAP Grid
            *	$("#OlapGrid").ejOlapGrid("option", "hyperlinkSettings", {enableValueCellHyperlink: true, enableRowHeaderHyperlink: true});			* 		
            * @memberof ejOlapGrid
            * @instance
            */
            hyperlinkSettings: {
                /**		
			* Allows the user to enable/diable hyperlink for value cells.
			* @default false
			* @type {Boolean}
			* @example 
                * //To set the Hyperlink option for value cell during initialization  
                * $("#OlapGrid").ejOlapGrid({hyperlinkSettings:{enableValueCellHyperlink: true}});
                * @example 
                * //Get or set the hyper link for cell, after initialization:<br>
                *	//Gets the hyper link state
                *	$("#OlapGrid").ejOlapGrid("option", "hyperlinkSettings");<br>			
                *	//Sets the cell hyperlink value
                *	$("#OlapGrid").ejOlapGrid("option", "hyperlinkSettings.enableValueCellHyperlink","true");			* 		
                * @alias ejOlapGrid#hyperlinkSettings->enableValueCellHyperlink
			* @instance
			*/
            enableValueCellHyperlink: false,

			/**		
			* Allows the user to enable/diable hyperlink for row header.
			* @default false
			* @type {Boolean}
			* @example 
			* //To set the Hyperlink option for row header during initialization  
                * $("#OlapGrid").ejOlapGrid({hyperlinkSettings:{enableRowHeaderHyperlink: true}});
            * @example 
			* //Get or set the hyper link for row header, after initialization:<br>
			*	//Gets the row header hyperlink state
                *	$("#OlapGrid").ejOlapGrid("option", "hyperlinkSettings");<br>		
			*	//Sets the row header hyperlink
                *	$("#OlapGrid").ejOlapGrid("option", "hyperlinkSettings.enableRowHeaderHyperlink","true");			* 		
                * @alias ejOlapGrid#hyperlinkSettings->enableRowHeaderHyperlink
			* @instance
			*/
                enableRowHeaderHyperlink: false,

                /**		
                * Allows the user to enable/diable hyperlink for column header.
                * @default false
                * @type {Boolean}
                * @example 
                * //To set the Hyperlink option for column header during initialization  
                * $("#OlapGrid").ejOlapGrid({hyperlinkSettings:{enableColumnHeaderHyperlink: true}});
                * @example 
                * //Get or set the hyper link for column header, after initialization:<br>
                *	//Gets the hyper link state
                *	$("#OlapGrid").ejOlapGrid("option", "hyperlinkSettings");<br>   				
                *	//Sets the column header hyper link state
                *	$("#OlapGrid").ejOlapGrid("option", "hyperlinkSettings.enableColumnHeaderHyperlink","true");			* 		
                * @alias ejOlapGrid#hyperlinkSettings->enableColumnHeaderHyperlink
                * @instance
                */
            enableColumnHeaderHyperlink: false,

			/**		
                * Allows the user to enable/diable hyperlink for summary cells.
                * @default false
                * @type {Boolean}
                * @example 
                * //To set the Hyperlink option for summary cell during initialization  
                * $("#OlapGrid").ejOlapGrid({hyperlinkSettings:{enableSummaryCellHyperlink: true}});
                * @example 
                * //Get or set the hyper link for summary cell, after initialization:<br>
                *	//Gets the summary cell hyperlink state
                *   $("#OlapGrid").ejOlapGrid("option", "hyperlinkSettings");<br>   		
                *	//Sets the summary cell hyperlink 
                *   $("#OlapGrid").ejOlapGrid("option", "hyperlinkSettings.enableSummaryCellHyperlink","true");			* 		
                * @alias ejOlapGrid#hyperlinkSettings->enableSummaryCellHyperlink
                * @instance
                */
                enableSummaryCellHyperlink: false
            },

            /**		
            * Sets the progress indicator type which arises on any UI operation.
            * @default ej.olap.OlapGrid.ProgressMode.Infinite
            * @type {enum}
            * @example 
            * //To set the progress mode during initialization  
            * $("#OlapGrid").ejOlapGrid({progressMode: ej.olap.OlapGrid.ProgressMode.Infinite});
            * @example 
			* //Get or set the progress mode, after initialization:<br>
			*	//Gets the hyper link state
			* $("#OlapGrid").ejOlapGrid("option", "progressMode");<br>   			
			*	//Sets the progress mode value
            * $("#OlapGrid").ejOlapGrid("option", "progressMode", ej.olap.OlapGrid.ProgressMode.Progress ); 
			* 		
            * @memberof ejOlapGrid
            * @instance
            */
            progressMode: "infinite",

            /**  
            * Allows the user to set custom name for the methods at service-end invoked on AJAX post.
            * @default {}
            * @type {Object}
            * @example 
            * //To set serviceMethodSettings API value, to invoke the appropriate service method on UI operation.
            * $("#OlapGrid").ejOlapGrid({ servieMethods.initialize: "InitializeGrid" });
            * @example 
            * //Gets or sets the serviceMethodSettings API, to invoke the appropriate service method on UI operation:<br>
            * //Gets the serviceMethodSettings value   
            * $("#OlapGrid").ejOlapGrid("option", "serviceMethodSettings");<br>   						
            * //Sets the serviceMethodSettings value   			
            * $("#OlapGrid").ejOlapGrid("option", "serviceMethodSettings",  {initialize: "InitializeGrid"} ); 
            * @memberof ejOlapGrid
            * @instance
            */
            serviceMethodSettings: {
                /**  
                * Allows the user to set the custom name for the service method that’s responsible for initializing OLAP Grid. 			
                * @default "InitializeGrid"
                * @type {string}
                * @example 
                * //To set initialize API value, to invoke the corresponding service method for OLAP Grid initialization.
                * $("#OlapGrid").ejOlapGrid({ servieMethods",  {initialize: "InitializeGrid"} ); 
                * @example 
                * //Gets or sets the initialize API, to invoke the corresponding service method for OLAP Grid initialization:<br>
                * //Gets the initialize value   
                * $("#OlapGrid").ejOlapGrid("option", "serviceMethodSettings");<br>   						
                * //Sets the initialize value   			
                * $("#OlapGrid").ejOlapGrid("option", "serviceMethodSettings.initialize", "InitializeGrid" ); 
                * @alias ejOlapGrid#serviceMethodSettings->initialize
                * @instance
                */
                initialize: "InitializeGrid",

                /**  
                * Allows the user to set the custom name for the service method that’s responsible for drilling up/down operation in OLAP Grid. 			
                * @default "DrillGrid"
                * @type {string}
                * @example 
                * //To set drillDown API value, to invoke the corresponding service method for performing server-side operation while drilling up/down in OLAP Grid.   
                * $("#OlapGrid").ejOlapGrid({ serviceMethodSettings: { drillDown: "DrillGridMyMethod" } });
                * @example 
                * //Get or set the drillDown API, to invoke the corresponding service method for performing server-side operation while drilling up/down in OLAP Grid:<br>
                * //Gets the drillDown value   
                * $("#OlapGrid").ejOlapGrid("option", "serviceMethodSettings");<br>   						
                * //Sets the drillDown value   			
                * $("#OlapGrid").ejOlapGrid("option", "serviceMethodSettings.drillDown", "DrillGridMyMethod"} ); 
                * @alias ejOlapGrid#serviceMethodSettings->drillDown
                * @instance
                */
                drillDown: "DrillGrid",

                /**  
                * Allows the user to set the custom name for the service method that’s responsible for performing exporting operation in OLAP Grid.		
                * @default "ExportOptions"
                * @type {string}
                * @example 
                * //To set exportOptions API value, to invoke the corresponding service method for performing server-side operation while exporting.  
                * $("#OlapGrid").ejOlapGrid({ serviceMethodSettings: { exportOptions: "ExportOptionsMyMethod" } });
                * @example 
                * //Gets or sets the exportOptions API, to invoke the corresponding service method for performing server-side operation while exporting.:<br>
                * //Gets the exportOptions value   
                * $("#OlapGrid").ejOlapGrid("option", "serviceMethodSettings");<br>   						
                * //Sets the exportOptions value   			
                * $("#OlapGrid").ejOlapGrid("option", "serviceMethodSettings.exportOptions", "ExportOptionsMyMethod"); 
                * @alias ejOlapGrid#serviceMethodSettings->exportOptions
                * @instance
                */
                exportOptions: "ExportOptions",

                /**  
                * Allows the user to set the custom name for the service method that’s responsible for performing paging operation in OLAP Grid.					
                * @default "Paging" 
                * @type {string}
                * @example 
                * //To set paging API value, to invoke the corresponding service method for performing server-side operation during paging in OLAP Grid.  
                * $("#OlapGrid").ejOlapGrid({ serviceMethodSettings: { paging: "PagingMyMethod" } });
                * @example 
                * //Gets or sets the paging API, to invoke the corresponding service method for performing server-side operation during paging in OLAP Grid:<br>
                * //Gets the paging value   
                * $("#OlapGrid").ejOlapGrid("option", "serviceMethodSettings");<br>   						
                * //Sets the paging value   			
                * $("#OlapGrid").ejOlapGrid("option", "serviceMethodSettings.paging",  "PagingMyMethod"); 
                * @alias ejOlapGrid#serviceMethodSettings->paging
                * @instance
                */
                paging: "Paging"
            },

            /**		
            * Custom object to pass additional information between client-end and service-end.
            * @default null
            * @type {Object}
            * @example 
            * // To pass additional information between client-end and service-end   
            *  $("#OlapGrid").ejOlapGrid({customObject: { Language: "en-US" }}); 
            * @example 
            * //Get or set the custom object API, after initialization:<br>
            *	//Gets the custom object value  
            *	$("#OlapGrid").ejOlapGrid("option","customObject");<br>			
            *	//Sets the custom object value 
            *	$("#OlapGrid").ejOlapGrid("option","customObject", {Language: "en-US"} );  		
            * @memberof ejOlapGrid
            * @instance
            */
            customObject: {},

            /**		
            * Sets the localized language for the control.
            * @default "en-US"
            * @type {String}
            * @example 
            * //Sets the localized language    
            *  $("#OlapGrid").ejOlapGrid({locale: "en-US"}}); 
            * @example 
            * //Get or set the size API, after initialization:<br>
            *	//Gets the localization value  
            *	$("#OlapGrid").ejOlapGrid("option","locale");<br>			
            *	//Sets the localization value 
            *	$("#OlapGrid").ejOlapGrid("option","locale", "en-US" );  		
            * @memberof ejOlapGrid
            * @instance
            */
            locale: "en-US",

            //events

            /**     
			 * Fires when the value cell is clicked once if OLAP Grid enabled with hyperlink.
			 * @event
			 * @name ejOlapGrid#valueCellHyperlinkClick 	
			 * @param {Object} argument Event parameters from OLAP Grid.
			 * @param {object}  argument.args returns the original event args.
			 * @param {string}  argument.cellPosition returns the cell position (row index and column index) in table.
			 * @param {string}  argument.cellType returns the type of the cell.
			 * @param {string}  argument.rowData returns the seriealized data of the header cells.
			 * @param {string}  argument.uniqueName returns the unique name of levels/members.   
			 * @example 
			 * //valueCellHyperlinkClick event for OLAP Grid
             * $("#OlapGrid").ejOlapGrid({
             *    valueCellHyperlinkClick: function (args) {}
             * });      
			 * @memberof ejOlapGrid
			 * @instance
            */
            valueCellHyperlinkClick: "",

            /**     
            * Fires when the row header cell is clicked once if OLAP Grid enabled with hyperlink.
            * @event
			 * @name ejOlapGrid#rowHeaderHyperlinkClick 	
			 * @param {Object} argument Event parameters from OLAP Grid.
			 * @param {object}  argument.args returns the original event args.
			 * @param {string}  argument.cellPosition returns the cell position (row index and column index) in table.
			 * @param {string}  argument.cellType returns the type of the cell.
			 * @param {string}  argument.rowData returns the seriealized data of the header cells.
			 * @param {string}  argument.uniqueName returns the unique name of levels/members.   
			 * @example 
			 * //rowHeaderHyperlinkClick event for OLAP Grid
             * $("#OlapGrid").ejOlapGrid({
             *    rowHeaderHyperlinkClick: function (args) {}
             * });      
			 * @memberof ejOlapGrid
			 * @instance
			 */
            rowHeaderHyperlinkClick: "",

			 /**     
			 * Fires when the column header cell is clicked once if OLAP Grid enabled with hyperlink.
			 * @event
			 * @name ejOlapGrid#columnHeaderHyperlinkClick 	
			 * @param {Object} argument Event parameters from OLAP Grid.
			 * @param {object}  argument.args returns the original event args.
			 * @param {string}  argument.cellPosition returns the cell position (row index and column index) in table.
			 * @param {string}  argument.cellType returns the type of the cell.
			 * @param {string}  argument.rowData returns the seriealized data of the header cells.
			 * @param {string}  argument.uniqueName returns the unique name of levels/members.     
			 * @example 
			 * //columnHeaderHyperlinkClick event for OLAP Grid
             * $("#OlapGrid").ejOlapGrid({
             *    columnHeaderHyperlinkClick: function (args) {}
             * });      
			 * @memberof ejOlapGrid
			 * @instance
			 */
            columnHeaderHyperlinkClick: "",

			 /**     
			 * Fires when the summary cell is clicked.
			 * @event
			 * @name ejOlapGrid#summaryCellHyperlinkClick 	
			 * @param {Object} argument Event parameters from OLAP Grid.
			 * @param {object}  argument.args returns the original event args.
			 * @param {string}  argument.cellPosition returns the cell position (row index and column index) in table.
			 * @param {string}  argument.cellType returns the type of the cell.
			 * @param {string}  argument.rowData returns the seriealized data of the header cells.
			 * @param {string}  argument.uniqueName returns the unique name of levels/members.
			 * @example 
			 * //summaryCellHyperlinkClick event for OLAP Grid
             * $("#OlapGrid").ejOlapGrid({
             *    summaryCellHyperlinkClick: function (args) {}
             * });      
			 * @memberof ejOlapGrid
			 * @instance
			 */
            summaryCellHyperlinkClick: "",

			 /**     
			 * Fires before service invoked.
			 * @event
            * @name ejOlapGrid#beforeServiceInvoke 	
            * @param {Object} argument Event parameters from OLAP Grid  
            * @param {string}  argument.action return the current action of OLAP Grid control.		 
            * @param {object}  argument.customObject return the custom object bounds with OLAP Grid control.
            * @param {string}  argument.element return the outer HTML of OLAP Grid control.
            * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
            * @param {object}  argument.model returns the OLAP Grid model
            * @param {string}  argument.type returns the name of the event
			 * @example 
			 * //beforeServiceInvoke event for OLAP Grid
             * $("#OlapGrid").ejOlapGrid({
             *    beforeServiceInvoke: function (args) {}
             * });      
			 * @memberof ejOlapGrid
            * @instance
            */
            beforeServiceInvoke: null,

            /**     
            * Fires after the service is invoked.
            * @event
            * @name ejOlapGrid#afterServiceInvoke 	
            * @param {Object} argument Event parameters from OLAP Grid 
            * @param {string}  argument.action return the current action of OLAP Grid control.		 
            * @param {object}  argument.customObject return the custom object bounds with OLAP Grid control.
            * @param {string}  argument.element return the outer HTML of OLAP Grid control.
            * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
            * @param {object}  argument.model returns the OLAP Grid model
			 * @param {string}  argument.type returns the name of the event			 
			 * @example 
			 * //afterServiceInvoke event for OLAP Grid
             * $("#OlapGrid").ejOlapGrid({
             *    afterServiceInvoke: function (args) {}
             * });      
			 * @memberof ejOlapGrid
			 * @instance
			 */
            afterServiceInvoke: null,

			 /**     
			 * Fires after drill down of OLAP Grid.
			 * @event
			 * @name ejOlapGrid#drillSuccess 	
			 * @param {Object} argument Event parameters from OLAP Grid    
			 * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
			 * @param {object}  argument.model returns the OLAP Grid model
			 * @param {string}  argument.type returns the name of the event				 
			 * @example 
			 * //drillSuccess event for OLAP Grid
             * $("#OlapGrid").ejOlapGrid({
             *    drillSuccess: function (args) {}
             * });      
			 * @memberof ejOlapGrid
			 * @instance
			 */
            drillSuccess: null,

            /**     
            * Fires when the cell is right clicked.
            * @event
            * @name ejOlapGrid#cellContext 	
            * @param {Object} argument Event parameters from OLAP Grid.
            * @param {object}  argument.args returns the original event args.
            * @param {string}  argument.cellPosition returns the cell position (row index and column index) in table.
            * @param {string}  argument.cellType returns the type of the cell.
            * @param {string}  argument.rowData returns the seriealized data of the header cells.
            * @param {string}  argument.uniqueName returns the unique name of levels/members.
            * @example 
            * //cellContext event for OLAP Grid
            * $("#OlapGrid").ejOlapGrid({
            *    cellContext: function (args) {}
            * });      
			 * @memberof ejOlapGrid
			 * @instance
			 */
            cellContext: "",

            /**     
            * Fires when OLAP Grid Start loading.
            * @event
            * @name ejOlapGrid#load 	
            * @param {Object} argument Event parameters from OLAP Grid.
            * @param {object}  argument.args returns the original event args.
            * @param {string}  argument.action returns the current action of OLAP Grid control.
            * @param {Object}  argument.customObject returns the custom object bounded with the control.
            * @param {string}  argument.element returns the HTML of OLAP Grid control.
            * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
            * @param {object}  argument.model returns the OLAP Grid model.
            * @param {string}  argument.type returns the name of the event.
            * @example 
            * //load event for OLAP Grid
            * $("#OlapGrid").ejOlapGrid({
            *    load: function (args) {}
            * });      
            * @memberof ejOlapGrid
            * @instance
            */
            load: null,

            /**     
            * Fires when OLAP Grid completely finished its rendering.
            * @event
            * @name ejOlapGrid#renderComplete 	
            * @param {Object} argument Event parameters from OLAP Grid.
            * @param {object}  argument.args returns the original event args.
            * @param {string}  argument.action returns the current action of OLAP Grid control.
            * @param {Object}  argument.customObject returns the custom object bounded with the control.
            * @param {string}  argument.element returns the HTML of OLAP Grid control.
            * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
            * @param {object}  argument.model returns the OLAP Grid model.
            * @param {string}  argument.type returns the name of the event.
            * @example 
            * //renderComplete event for OLAP Grid
            * $("#OlapGrid").ejOlapGrid({
            *    renderComplete: function (args) {}
            * });      
            * @memberof ejOlapGrid
            * @instance
            */
            renderComplete: null,

            /**     
            * Fires while any discrepancies occurs during the rendering time.
            * @event
            * @name ejOlapGrid#renderFailure 	
            * @param {Object} argument Event parameters from OLAP Grid.
            * @param {object}  argument.args returns the original event args.
            * @param {string}  argument.action returns the current action of OLAP Grid control.
            * @param {Object}  argument.customObject returns the custom object bounded with the control.
            * @param {string}  argument.element returns the HTML of OLAP Grid control.
            * @param {Object}  argument.message returns the error message with error code.
            * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
            * @param {object}  argument.model returns the OLAP Grid model.
            * @param {string}  argument.type returns the name of the event.
            * @example 
            * //renderFailure event for OLAP Grid
            * $("#OlapGrid").ejOlapGrid({
            *    renderFailure: function (args) {}
            * });      
            * @memberof ejOlapGrid
            * @instance
            */
            renderFailure: null,

            /**     
            * Fires when OLAP Grid successfully finished its rendering.
            * @event
            * @name ejOlapGrid#renderSuccess 	
            * @param {Object} argument Event parameters from OLAP Grid.
            * @param {object}  argument.args returns the original event args.
            * @param {string}  argument.action returns the current action of OLAP Grid control.
            * @param {Object}  argument.customObject returns the custom object bounded with the control.
            * @param {string}  argument.element returns the HTML of OLAP Grid control.
            * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
            * @param {object}  argument.model returns the OLAP Grid model.
            * @param {string}  argument.type returns the name of the event.
            * @example 
            * //renderSuccess event for OLAP Grid
            * $("#OlapGrid").ejOlapGrid({
            *    renderSuccess: function (args) {}
            * });      
            * @memberof ejOlapGrid
            * @instance
            */
            renderSuccess: null
        },

        /**
         * Provides information about the properties of type Array and Object.
         *
         * @private
         */
        dataTypes: {
            serviceMethodSettings: "data",
            customObject: "data"
        },

        observables: ["layout", "enableCellContext", "hyperlinkSettings.enableValueCellHyperlink", "hyperlinkSettings.enableRowHeaderHyperlink", "hyperlinkSettings.enableColumnHeaderHyperlink", "hyperlinkSettings.enableSummaryCellHyperlink"],
        layout: ej.util.valueFunction("layout"),
        enableCellContext: ej.util.valueFunction("enableCellContext"),
        enableValueCellHyperlink: ej.util.valueFunction("hyperlinkSettings.enableValueCellHyperlink"),
        enableRowHeaderHyperlink: ej.util.valueFunction("hyperlinkSettings.enableRowHeaderHyperlink"),
        enableColumnHeaderHyperlink: ej.util.valueFunction("hyperlinkSettings.enableColumnHeaderHyperlink"),
        enableSummaryCellHyperlink: ej.util.valueFunction("hyperlinkSettings.enableSummaryCellHyperlink"),
        locale: ej.util.valueFunction("locale"),

        /**
        * Gets the serialized current OLAP Report of the control.
        *
         * @private 
         */
        getOlapReport: function () {
            return this._olapReport;
        },

        /**
         * Sets the serialized current OLAP Report of the control.
         *
         * @private 
         */
        setOlapReport: function (value) {
            this._olapReport = value;
        },

        /**
        * Gets the JSON formatted records for the control.
        *
        * @private 
        */
        getJSONRecords: function () {
            return this._JSONRecords;
        },

        /**
        * Sets the JSON formatted records for the control.
        *
        * @private 
        */
        setJSONRecords: function (value) {
            this._JSONRecords = $.parseJSON(value);
        },

        /**
         * Initializes the variables and loads the widget. 
         *
         * @private
         */
        _init: function () {
            this._initPrivateProperties();
            this._load();
        },

        /**
         * Destroys the widget while it's no longer required. 
         *
         * @private
         */
        _destroy: function () {
            this.element.empty().removeClass("e-olapgrid" + this.model.cssClass);
            if (this.ogridWaitingPopup != undefined)
                this.ogridWaitingPopup._destroy();

        },

        /**
         * Initializes the local variables. 
         *
         * @private
         */
        _initPrivateProperties: function () {
            this._id = this.element.attr("id");
            this._olapReport = "";
            this._JSONRecords = null;
            this._drillCaption = "";
            this._drillAction = "";
            this._startDrilldown = false;
            this._pagerObj = null;
            this._seriesPageCount = null;
            this._categPageCount = null;
            this._seriesCurrentPage = 1;
            this._categCurrentPage = 1;
            ogridWaitingPopup = null;
            _proxy = this;
            ogridProgressBar = null;
            ogridprogressText = null;
            oGridTimer = null;
            this._maxViewLoading = null;
        },

        /**
         * This function is the point were loading/rendering of the OLAP Grid widget is initiated.
         *
         * @private  
         */
        _load: function () {
            var eventArgs = { action: ej.olap.OlapGrid.GridAction.Initialize, element: this.element, customObject: this.model.customObject };
            this._trigger("load", eventArgs);
            this.element.addClass(this.model.cssClass);
            if (typeof oclientWaitingPopup != 'undefined' && oclientWaitingPopup != null)
                if ($("#" + oclientProxy._id + "_maxView")[0]) {
                    $("#" + oclientProxy._id + "_maxView").ejWaitingPopup({ showOnInit: true });
                    this._maxViewLoading = ("#" + oclientProxy._id + "_maxView").data("ejWaitingPopup");
                }
                else
                    oclientWaitingPopup.show();
            else {
                if (_proxy.model.progressMode == "infinite" && typeof oclientWaitingPopup == 'undefined') {
                    $("#" + this._id).ejWaitingPopup({ showOnInit: true });
                    ogridWaitingPopup = $("#" + this._id).data("ejWaitingPopup");
                }
                else {
                    $("#" + _proxy._id + "progressContainer").remove();
                    this._createProgressBar();
                    this._progressStatus(0, 70, 140);
                }
            }

            if (this.model.beforeServiceInvoke != null)
                this._trigger("beforeServiceInvoke", { action: ej.olap.OlapGrid.GridAction.Initialize, element: this.element, customObject: this.model.customObject });
            if (this.model.currentReport != "")
                this.model.customObject = { currentReport: this.model.currentReport };
            var serializedCustomObject = JSON.stringify(this.model.customObject);
            if (_proxy.model.progressMode == "progress")
                $("#" + _proxy._id + "progressText")[0].innerHTML = _proxy._getLocalizedLabels("PreparingAndExecutingMDXquery") + "...";
            if (this.element[0] != null && this.element.parents().find(".controlPanel").length > 0 && this.layout() != "" && this.layout() != ej.olap.OlapGrid.Layout.Normal)
                this.doAjaxPost("POST", this.model.url + "/" + this.model.serviceMethodSettings.initialize, JSON.stringify({ "action": "initializeGrid", "olapReport": this.model.olapReport, "layout": this.layout(), "customObject": serializedCustomObject }), this._renderControlSuccess);
            else if (this.element[0] != null && this.element.parents().find(".controlPanel").length > 0)
                this.doAjaxPost("POST", this.model.url + "/" + this.model.serviceMethodSettings.initialize, JSON.stringify({ "action": "initializeGrid", "olapReport": this.model.olapReport, "customObject": serializedCustomObject }), this._renderControlSuccess);
            else if (this.layout() != "" && this.layout() != ej.olap.OlapGrid.Layout.Normal)
                this.doAjaxPost("POST", this.model.url + "/" + this.model.serviceMethodSettings.initialize, JSON.stringify({ "action": "initializeGrid", "layout": this.layout(), "customObject": serializedCustomObject }), this._renderControlSuccess);
            else
                this.doAjaxPost("POST", this.model.url + "/" + this.model.serviceMethodSettings.initialize, JSON.stringify({ "action": "initializeGrid", "customObject": serializedCustomObject }), this._renderControlSuccess);
        },
        _setFirst: true,

        /**
        * To configure the properties at runtime using SetModel		
        * @private
        */
        _setModel: function (options) {
            for (var key in options) {
                switch (key) {
                    case "OlapReport": this.setOlapReport(options[key]); break;
                    case "JsonData": this.setOlapReport(options[key]); break;
                    case "RefreshOlapGrid": this.element.renderControlFromJSON(options[key]); break;
                    case "layout": this._load(); break;
                    case "customObject": this.model.customObject = options[key]; break;
                    case "enableCellContext": this._load(); break;
                    case "enableValueCellHyperlink": this._load(); break;
                    case "enableRowHeaderHyperlink": this._load(); break;
                    case "enableColumnHeaderHyperlink": this._load(); break;
                    case "enableSummaryCellHyperlink": this._load(); break;
                    case "locale": this._load(); break;
                }
            }
        },

        /**
        * This function is invoked to wire the events for UI interaction.
        *
        * @private  
        */
        _wireEvents: function () {
            if (this.enableCellContext() == true) {
                if (document.addEventListener) {
                    document.removeEventListener('contextmenu', _proxy._cellContext, false);
                    document.addEventListener('contextmenu', _proxy._cellContext, false);
                }
                else {
                    document.detachEvent('oncontextmenu', _proxy._cellContext);
                    document.attachEvent('oncontextmenu', _proxy._cellContext);
                }
            }
            this.drillDownHandler = $.proxy(this._drillDown, this);
            this.addHyperlinkHandler = $.proxy(this._addHyperlink, this);
            this.removeHyperlinkHandler = $.proxy(this._removeHyperlink, this);
            this._on(this.element, "mouseover", ".value, .rowheader, .colheader", this.addHyperlinkHandler);
            this._on(this.element, "mouseleave", ".value, .rowheader, .colheader", this.removeHyperlinkHandler);
            this._on(this.element, "click", ".value, .colheader, .rowheader", function (e) {
                if (e.target.className.indexOf("hyperlink") != -1) {
                    var cellPos = $(e.target).attr("p");
                    var rawdata = this.getJSONRecords()[parseInt((parseInt(cellPos.split(",")[0]) * this.rowCount) + parseInt(cellPos.split(",")[1]))].Info;
                    var cellInfo = {
                        cellValue: e.target.innerHTML,
                        cellPosition: cellPos,
                        cellType: e.target.className.split(' ')[0],
                        uniqueName: rawdata.split('::')[0],
                        args: e,
                        rawdata: rawdata
                    };
                    if (this.enableSummaryCellHyperlink() && e.target.className.indexOf("summary value") >= 0)
                        this._trigger("summaryCellHyperlinkClick", { element: this.element, customObject: this.model.customObject });
                    else if (this.enableValueCellHyperlink() && e.target.className.indexOf("value") >= 0)
                        this._trigger("valueCellHyperlinkClick", { args: cellPos });
                    else if (this.enableRowHeaderHyperlink() && e.target.className.indexOf("rowheader") >= 0)
                        this._trigger("rowHeaderHyperlinkClick", { element: this.element, customObject: this.model.customObject });
                    else if (this.enableColumnHeaderHyperlink() && e.target.className.indexOf("colheader") >= 0)
                        this._trigger("columnHeaderHyperlinkClick", { element: this.element, customObject: this.model.customObject });
                }
            });
            this._on(this.element, "click", ".expand, .collapse", ej.proxy(this._drillDown, this));
            this._on(this.element, "mouseover", ".colheader, .rowheader, .expand, .collapse", function (evt) {
                if (evt.target.className.indexOf("expand") > -1 && !$(evt.target).hasClass("header-hover-expand"))
                    $(evt.target).addClass("header-hover-expand");
                else if (evt.target.className.indexOf("collapse") > -1 && !$(evt.target).hasClass("header-hover-collapse"))
                    $(evt.target).addClass("header-hover-collapse");
                else if ((evt.target.className.indexOf("expand") < 0 || evt.target.className.indexOf("collapse") < 0) && $(evt.target).children("span").length > 0) {
                    var child = $(evt.target).children("span")[0];
                    if (!$(child).hasClass("header-hover-expand") && child.className.indexOf("expand") > -1)
                        $(child).addClass("header-hover-expand");
                    else if (!$(child).hasClass("header-hover-collapse") && child.className.indexOf("collapse") > -1)
                        $(child).addClass("header-hover-collapse");
                }
            });
            this._on(this.element, "mouseleave", ".colheader, .rowheader, .expand, .collapse", function (evt) {
                this.element.find(".expand, .collapse").removeClass("header-hover-expand header-hover-collapse");
            });
            if (this.model.enableToolTip) {
                this._on(this.element, "mouseover", ".value", this._applyToolTip);
                this._on(this.element, "mouseleave", ".value", function (e) {
                    this.element.find('#gridTooltip').hide("slow");
                });
            }
        },

        /**
         * This function is invoked to unwire the events registered for UI interaction.
         *
         * @private  
         */
        _unWireEvents: function () {
            this._off(this.element, "click", ".expand, .collapse");
            this._off(this.element, "click", ".value, .colheader, .rowheader");
            this._off(this.element, "mouseover", ".colheader, .rowheader, .expand, .collapse");
            this._off(this.element, "mouseleave", ".colheader, .rowheader, .expand, .collapse");
            this._off(this.element, "mouseover", ".value, .rowheader, .colheader", this.addHyperlinkHandler);
            this._off(this.element, "mouseleave", ".value, .rowheader, .colheader", this.removeHyperlinkHandler);
            this._off(this.element, "mouseover", ".value");
            this._off(this.element, "mouseleave", ".value");
        },

        _applyToolTip: function (e) {
            {
                var toolTipInfo; var rowValue; var cellPos; var columnValue = ""; valueCell = "";
                if ($(e.target).hasClass("summary")) {
                    if ($(e.target).siblings("th").hasClass("rowheader"))
                        cellPos = $(e.target).parents("tr").find(".rowheader").last().attr("p");
                    else
                        cellPos = $(e.target).parents("tr").find(".summary").attr("p");
                }
                else
                    cellPos = $(e.target).parents("tr").find(".rowheader").last().attr("p");
                if (cellPos != null) {
                    if ((cellPos[0] == 0) && ((parseInt(cellPos.split(",")[0])) != null)) {
                        if (cellPos == _proxy.getJSONRecords()[parseInt((parseInt(cellPos.split(",")[0]) * _proxy.rowCount) + parseInt(cellPos.split(",")[1]))].Index) {
                            rowValue = _proxy.getJSONRecords()[parseInt((parseInt(cellPos.split(",")[0]) * _proxy.rowCount) + parseInt(cellPos.split(",")[1]))].Value;
                        }
                    }
                    else
                        while (cellPos[0] >= 0) {
                            if (cellPos == _proxy.getJSONRecords()[parseInt((parseInt(cellPos.split(",")[0]) * _proxy.rowCount) + parseInt(cellPos.split(",")[1]))].Index) {
                                toolTipInfo = _proxy.getJSONRecords()[parseInt((parseInt(cellPos.split(",")[0]) * _proxy.rowCount) + parseInt(cellPos.split(",")[1]))].Value;
                                cellPos = parseInt(cellPos.split(",")[0]) - 1 + "," + parseInt(cellPos.split(",")[1]);
                            }
                            if (rowValue)
                                rowValue = toolTipInfo + "-" + rowValue;
                            else
                                rowValue = toolTipInfo;
                        }
                }
                var columnHeaderCount = $(e.target).closest('tbody').prev().children('tr:last').children('th').attr("p")[2];
                var currentCellPos = $(e.target).attr("p");
                if ((currentCellPos != undefined) && (currentCellPos == _proxy.getJSONRecords()[parseInt((parseInt(currentCellPos.split(",")[0]) * _proxy.rowCount) + parseInt(currentCellPos.split(",")[1]))].Index)) {
                    var colPos = parseInt(currentCellPos.split(",")[0]);
                }
                for (var i = 0; i <= columnHeaderCount; i++) {
                    var columnHeaderPos = colPos + "," + i;
                    if (colPos != null)
                        if (columnHeaderPos == _proxy.getJSONRecords()[parseInt((parseInt(columnHeaderPos.split(",")[0]) * _proxy.rowCount) + parseInt(columnHeaderPos.split(",")[1]))].Index) {
                            toolTipInfo = _proxy.getJSONRecords()[parseInt((parseInt(columnHeaderPos.split(",")[0]) * _proxy.rowCount) + parseInt(columnHeaderPos.split(",")[1]))].Value;
                            if (columnValue == "")
                                columnValue = toolTipInfo;
                            else if (toolTipInfo != "")
                                columnValue = columnValue + "-" + toolTipInfo;
                        }
                }
                this.element.find('#gridTooltip').remove();
                var tdPos = $(e.target).offset();
                $(e.target).append("<div id=\"gridTooltip\" class=\"tooltip\"></div>");
                this.element.find('#gridTooltip').css("left", (tdPos.left + 30) + 'px', "top", (tdPos.left + 30) + 'px');
                this.element.find('#gridTooltip').append("<p id=\"value\" class=\"tooltipText\">Value : " + $(e.target).text() + "</p>");
                this.element.find('#gridTooltip').append("<p id=\"row\" class=\"tooltipText\">Row : " + rowValue + "</p>");
                this.element.find('#gridTooltip').append("<p id=\"column\" class=\"tooltipText\">Column : " + columnValue + "</p>");
                this.element.find('#gridTooltip').show("fast");
            }
        },
        /**
         * This function is used to create and handle virtual scrollbar.
         *
         * @private  
         */
        _applyVScrolling: function () {
            var _proxy = this;
            var oGridDiv = $(this.element).find('.virtualScrollGrid');
            if (this._seriesPageCount > 1) {
                var getMinVerticalPos = function () { return (_proxy.element.find(".vScrollPanel").position().top + 1); };
                var vScrollBar = $(this.element).find(".vScrollPanel");
                vScrollBar.height($(oGridDiv).height());
                vScrollBar.html(ej.buildTag("div.vScrollThumb")[0].outerHTML);
                var vScrollHandle = vScrollBar.find(".vScrollThumb");
                vScrollHandle.height(Math.max(Math.ceil($(oGridDiv).height() / _proxy._seriesPageCount), 15));
                var vDragRange = vScrollBar.height() - vScrollHandle.height() - 2;
                vScrollHandle.offset({ top: getMinVerticalPos() + (_proxy._seriesCurrentPage - 1) * (vDragRange / (_proxy._seriesPageCount - 1)) });

                $(vScrollHandle).bind('mousedown', function (e) {
                    $(this).addClass("dragging");
                    var initCursorPos = e.pageY - getMinVerticalPos();
                    var initThumbPos = vScrollHandle.position().top - getMinVerticalPos();
                    _proxy.element.find(".seriesPageIndicator").removeClass("inActive").css({ top: vScrollHandle.position().top, left: vScrollHandle.position().left + 20 });
                    var currentPage = _proxy._seriesCurrentPage;
                    $(document).bind('mousemove', function (e) {
                        if (window.navigator.userAgent.indexOf('MSIE ') > -1)
                            $(document).on("selectstart", function (e) { e.preventDefault(); });
                        else
                            window.getSelection().removeAllRanges();
                        var movedDistance = (e.pageY - getMinVerticalPos()) - initCursorPos;
                        if (movedDistance > 0 && initThumbPos + movedDistance <= vDragRange)
                            vScrollHandle.offset({ top: initThumbPos + movedDistance + getMinVerticalPos() });
                        else if (movedDistance < 0 && initThumbPos + movedDistance >= 0)
                            vScrollHandle.offset({ top: initThumbPos + movedDistance + getMinVerticalPos() });
                        else if (initThumbPos + movedDistance > vDragRange)
                            vScrollHandle.offset({ top: getMinVerticalPos() + vDragRange });
                        else if (initThumbPos + movedDistance < 0)
                            vScrollHandle.offset({ top: getMinVerticalPos() });
                        currentPage = Math.ceil((vScrollHandle.position().top - getMinVerticalPos()) / (vDragRange / _proxy._seriesPageCount)) == 0 ? 1 : Math.ceil((vScrollHandle.position().top - getMinVerticalPos()) / (vDragRange / _proxy._seriesPageCount));
                        _proxy.element.find(".seriesPageIndicator").css({ top: vScrollHandle.position().top });
                        _proxy.element.find(".series_CurrentPage").html(currentPage);
                    });
                    $(document).bind('mouseup', function () {
                        $(document).off("selectstart");
                        $(this).unbind('mousemove').unbind('mouseup');
                        $(vScrollHandle).removeClass("dragging");
                        if (currentPage != _proxy._seriesCurrentPage) {
                            _proxy._seriesCurrentPage = currentPage;
                            _proxy.refreshPagedOlapGrid("series", _proxy._seriesCurrentPage);
                        }
                        else
                            _proxy.element.find(".seriesPageIndicator").addClass("inActive");
                    });
                });
            }

            if (this._categPageCount > 1) {
                var getMinHorizontalPos = function () { return (_proxy.element.find(".hScrollPanel").position().left + 1); };
                var hScrollBar = $(this.element).find(".hScrollPanel");
                hScrollBar.width($(oGridDiv).width());
                hScrollBar.html(ej.buildTag("div.hScrollThumb")[0].outerHTML);
                var hScrollHandle = hScrollBar.find(".hScrollThumb");
                hScrollHandle.width(Math.max(Math.ceil($(oGridDiv).width() / _proxy._categPageCount), 15));
                var hDragRange = hScrollBar.width() - hScrollHandle.width() - 2;
                hScrollHandle.offset({ left: getMinHorizontalPos() + (_proxy._categCurrentPage - 1) * (hDragRange / (_proxy._categPageCount - 1)) });

                $(hScrollHandle).bind('mousedown', function (e) {
                    $(this).addClass("dragging");
                    var initCursorPos = e.pageX - getMinHorizontalPos();
                    var initThumbPos = hScrollHandle.position().left - getMinHorizontalPos();
                    _proxy.element.find(".categPageIndicator").removeClass("inActive").css({ left: hScrollHandle.position().left, top: hScrollHandle.position().top + 20 });
                    var currentPage = _proxy._categCurrentPage;
                    $(document).bind('mousemove', function (e) {
                        if (window.navigator.userAgent.indexOf('MSIE ') > -1)
                            $(document).on("selectstart", function (e) { e.preventDefault(); });
                        else
                            window.getSelection().removeAllRanges();
                        var movedDistance = (e.pageX - getMinHorizontalPos()) - initCursorPos;
                        if (movedDistance > 0 && initThumbPos + movedDistance <= hDragRange)
                            hScrollHandle.offset({ left: initThumbPos + movedDistance + getMinHorizontalPos() });
                        else if (movedDistance < 0 && initThumbPos + movedDistance >= 0)
                            hScrollHandle.offset({ left: initThumbPos + movedDistance + getMinHorizontalPos() });
                        else if (initThumbPos + movedDistance > hDragRange)
                            hScrollHandle.offset({ left: getMinHorizontalPos() + hDragRange });
                        else if (initThumbPos + movedDistance < 0)
                            hScrollHandle.offset({ left: getMinHorizontalPos() });
                        currentPage = Math.ceil((hScrollHandle.position().left - getMinHorizontalPos()) / (hDragRange / _proxy._categPageCount)) == 0 ? 1 : Math.ceil((hScrollHandle.position().left - getMinHorizontalPos()) / (hDragRange / _proxy._categPageCount));
                        _proxy.element.find(".categPageIndicator").css({ left: hScrollHandle.position().left });
                        _proxy.element.find(".categ_CurrentPage").html(currentPage);
                    });
                    $(document).bind('mouseup', function () {
                        $(document).off("selectstart");
                        $(this).unbind('mousemove').unbind('mouseup');
                        $(hScrollHandle).removeClass("dragging");
                        if (currentPage != _proxy._categCurrentPage) {
                            _proxy._categCurrentPage = currentPage;
                            _proxy.refreshPagedOlapGrid("categ", _proxy._categCurrentPage);
                        }
                        else
                            _proxy.element.find(".categPageIndicator").addClass("inActive");
                    });
                });
            }
        },

        /**
        * To get the localized labels.
        * @private
        */
        //get Localized text
        _getLocalizedLabels: function (property) {

            return ej.olap.OlapGrid.locale[this.locale()][property] === undefined ? ej.olap.OlapGrid.locale["en-US"][property] : ej.olap.OlapGrid.locale[this.locale()][property];
        },

        /**
         * This function would be raised while clicking expand/collapse icons to perform drill up/down operations in OLAP Grid.
         *
         * @private  
         */
        _drillDown: function (e) {
            this._off(this.element, "click", ".expand, .collapse");
            var targetClass = e.target.className;
            this._startDrilldown = true;
            targetClass = targetClass.split(" ");
            if (targetClass[0] == "expand")
                this._drillAction = "drilldown";
            else if (targetClass[0] == "collapse")
                this._drillAction = "drillup";

            if (this.model.beforeServiceInvoke != null)
                this._trigger("beforeServiceInvoke", { action: ej.olap.OlapGrid.GridAction.DrillDown, element: this.element, customObject: this.model.customObject });
            var serializedCustomObject = JSON.stringify(this.model.customObject);
            var cellPos = $(e.target).parent().attr("p");
            this._drillCaption = $.trim($(e.currentTarget).parent().text());

            if (typeof oclientWaitingPopup != 'undefined' && oclientWaitingPopup != null) {
                if ($("#" + oclientProxy._id + "_maxView")[0]) {
                    $("#" + oclientProxy._id + "_maxView").ejWaitingPopup({ showOnInit: true });
                    this._maxViewLoading = $("#" + oclientProxy._id + "_maxView").data("ejWaitingPopup");
                }
                else
                    oclientWaitingPopup.show();
            }
            else {
                if (_proxy.model.progressMode == "infinite")
                    ogridWaitingPopup.show();
                else {
                    this._progressStatus(0, 70, 140);
                    if (_proxy.model.progressMode == "progress")
                        $("#" + _proxy._id + "progressText")[0].innerHTML = _proxy._getLocalizedLabels("PreparingAndExecutingMDXquery") + "...";
                }
            }
            var report;
            try {
                report = JSON.parse(this.getOlapReport()).Report;
            }
            catch (err) {
                report = this.getOlapReport();
            }
            if (this.layout() != "" || this.layout() != "normal") {
                if (typeof (oclientProxy) != "undefined")
                    this.doAjaxPost("POST", this.model.url + "/" + this.model.serviceMethodSettings.drillDown, JSON.stringify({ "action": "drillDownGrid", "cellPosition": cellPos, "currentReport": report, "clientReports": oclientProxy.reports, "headerInfo": this.getJSONRecords()[parseInt((parseInt(cellPos.split(",")[0]) * this.rowCount) + parseInt(cellPos.split(",")[1]))].Info, "layout": this.layout(), "customObject": serializedCustomObject }), this._drillDownSuccess);
                else
                    this.doAjaxPost("POST", this.model.url + "/" + this.model.serviceMethodSettings.drillDown, JSON.stringify({ "action": "drillDownGrid", "cellPosition": cellPos, "currentReport": report, "headerInfo": this.getJSONRecords()[parseInt((parseInt(cellPos.split(",")[0]) * this.rowCount) + parseInt(cellPos.split(",")[1]))].Info, "layout": this.layout(), "customObject": serializedCustomObject }), this._drillDownSuccess);
            }
            else {
                if (typeof (oclientProxy) != "undefined")
                    this.doAjaxPost("POST", this.model.url + "/" + this.model.serviceMethodSettings.drillDown, JSON.stringify({ "action": "drillDownGrid", "cellPosition": cellPos, "currentReport": report, "clientReports": oclientProxy.reports, "headerInfo": this.getJSONRecords()[parseInt((parseInt(cellPos.split(",")[0]) * this.rowCount) + parseInt(cellPos.split(",")[1]))].Info, "customObject": serializedCustomObject }), this._drillDownSuccess);
                else
                    this.doAjaxPost("POST", this.model.url + "/" + this.model.serviceMethodSettings.drillDown, JSON.stringify({ "action": "drillDownGrid", "cellPosition": cellPos, "currentReport": report, "headerInfo": this.getJSONRecords()[parseInt((parseInt(cellPos.split(",")[0]) * this.rowCount) + parseInt(cellPos.split(",")[1]))].Info, "customObject": serializedCustomObject }), this._drillDownSuccess);
            }
        },

        /**
         * This function would apply hyperlink to the appropriate cells.
         *
         * @private  
         */
        _addHyperlink: function (e) {
            if ($(".hyperlinkValueCell")[0])
                $($(".hyperlinkValueCell")[0]).removeClass("hyperlinkValueCell");
            if ($(".hyperlinkHeaderCell")[0])
                $($(".hyperlinkHeaderCell")[0]).removeClass("hyperlinkHeaderCell");
            if (e.target.className == "value" && this.enableValueCellHyperlink())
                $(e.target).addClass("hyperlinkValueCell");
            else if (e.target.className == "summary value" && this.enableSummaryCellHyperlink())
                $(e.target).addClass("hyperlinkValueCell");
            else if (e.target.className == "rowheader" && this.enableRowHeaderHyperlink())
                $(e.target).addClass("hyperlinkHeaderCell");
            else if (e.target.className == "colheader" && this.enableColumnHeaderHyperlink())
                $(e.target).addClass("hyperlinkHeaderCell");
        },

        /**
         * This function would remove/disable hyperlink in OLAP Grid.
         *
         * @private  
         */
        _removeHyperlink: function (e) {
            $(e.target).removeClass("hyperlinkValueCell").removeClass("hyperlinkHeaderCell");
        },

        /**
         * This function would be raised while right clicking over any cell in OLAP Grid.
         *
         * @private  
         */
        _cellContext: function (e) {
            var currentCell = e.target || e.srcElement;
            if (_proxy.enableCellContext() == true && $(currentCell).parents("#" + _proxy._id)[0]) {
                var cellPos = $(currentCell).attr("p");
                if (cellPos) {
                    var cellPos = $(currentCell).attr("p");
                    var rawdata = _proxy.getJSONRecords()[parseInt((parseInt(cellPos.split(",")[0]) * _proxy.rowCount) + parseInt(cellPos.split(",")[1]))].Info;
                    var cellInfo = {
                        cellValue: currentCell.innerHTML,
                        cellPosition: cellPos,
                        cellType: currentCell.className.split(' ')[0],
                        uniqueName: rawdata.split('::')[0],
                        args: e,
                        rawdata: rawdata
                    };
                    if (cellInfo.cellType != "" && cellInfo.cellValue != "")
                        _proxy._trigger("cellContext", { args: cellInfo });
                    e.target ? e.preventDefault() : (window.event.returnValue = false);
                }
            }
        },

        /**
         * This function would be raised while exporting OLAP Grid to Excel.
         *
         * @private  
         */
        exportToExcel: function () {
            var report;
            try {
                report = JSON.parse(this.getOlapReport()).Report;
            }
            catch (err) {
                report = this.getOlapReport();
            }
            var args = {
                currentReport: report,
                layout: _proxy.layout(),
                summaryCellColor: $(".summary").css("background-color")
            };
            this.doPostBack(_proxy.model.url + "/" + _proxy.model.serviceMethodSettings.exportOptions, args);
        },

        /**
         * This function is used to create progress bar containing status text and percentage.
         *
         * @private  
         */
        _createProgressBar: function () {
            var progressContainer = ej.buildTag("div#" + this._id + "progressContainer.progressContainer", "", { top: typeof (oclientProxy) != "undefined" ? ((oclientProxy.model.displaySettings.controlPlacement == "tab" || oclientProxy.model.displaySettings.mode != "chartAndGrid") ? oclientProxy.progressPos.top + 300 : (oclientProxy.model.displaySettings.defaultView == "chart" ? oclientProxy.progressPos.top + 500 : oclientProxy.progressPos.top + 200)) : ($("#" + this._id).position().top + ($("#" + this._id).height() / 2) - 30), left: typeof (oclientProxy) != "undefined" ? oclientProxy.progressPos.left + 550 : ($("#" + this._id).position().left + ($("#" + this._id).width() / 3) - 100) })[0].outerHTML;
            var progressBar = ej.buildTag("div#" + this._id + "progressBar.progressBar", "")[0].outerHTML;
            var blockControl = ej.buildTag("div#" + this._id + "blockDiv.blockDiv", "", { width: typeof (oclientProxy) != "undefined" ? ($("#" + oclientProxy._id).width()) : $("#" + this._id).width(), display: "block", opacity: "0.3", backgroundColor: "#E9E9E9", height: typeof (oclientProxy) != "undefined" ? ($("#" + oclientProxy._id).height()) : $("#" + this._id).height(), position: 'fixed', top: typeof (oclientProxy) != "undefined" ? oclientProxy.progressPos.top : ($("#" + this._id).position().top), left: typeof (oclientProxy) != "undefined" ? oclientProxy.progressPos.left : ($("#" + this._id).position().left) })[0].outerHTML;
            $("#" + _proxy._id).after(progressContainer);
            if (_proxy.model.progressMode == "normal") {
                $("#" + this._id + "progressContainer").css({ width: 0, height: 0 });
                $("#" + _proxy._id + "progressContainer").append(progressBar).append(blockControl);
            }
            else {
                var progressText = ej.buildTag("span#" + this._id + "progressText.progressText", "OlapGrid initial loading...")[0].outerHTML;
                $("#" + _proxy._id + "progressContainer").append(progressBar).append(progressText).append(blockControl);
            }
            $("#" + _proxy._id + "progressBar").ejProgressBar({ value: 0, text: 0 + " %", complete: "progressbarCompleted" });
            ogridProgressBar = $("#" + _proxy._id + "progressBar").data("ejProgressBar");
            ogridProgressText = $("#" + _proxy._id + "progressText");
        },

        /**
         * This function is used update the progress status.
         *
         * @private  
         */
        _progressStatus: function (current_Status, completion_Status, progress_Speed) {
            if (jQuery.type(completion_Status) === "string") {
                var progressText = completion_Status.split(':');
                if (progressText != undefined) completion_Status = parseInt(progressText[0]);
            }
            ogridProgressBar.setModel({ value: current_Status, text: current_Status + " %" });
            $("#" + _proxy._id + "progressContainer").show();
            $("#" + _proxy._id + "blockDiv").position = $("#" + _proxy._id).position();
            oGridTimer = window.setInterval(function () {
                ogridProgressBar.setModel({ value: current_Status, text: current_Status + " %" });
                current_Status++;
                if (current_Status >= completion_Status) {
                    window.clearInterval(oGridTimer);
                    if (progressText != undefined)
                        ogridProgressBar.setModel({ value: 100, text: progressText[1] });

                };
            }, progress_Speed);
        },

        /**
         * 	This function receives the controls update from service-end which would be utilized for rendering the widget.
         *
         * @private  
         */
        _renderControlSuccess: function (msg) {
            _proxy = this;
            if (_proxy.model.progressMode != "infinite") {
                if (_proxy.model.progressMode == "progress")
                    $("#" + _proxy._id + "progressText")[0].innerHTML = _proxy._getLocalizedLabels("MDXqueryExecutedSuccessfully") + "...";
                window.clearInterval(oGridTimer);
            }
            try {
                if (_proxy.model.progressMode != "infinite") {
                    if (_proxy.model.progressMode == "progress")
                        $("#" + _proxy._id + "progressText")[0].innerHTML = _proxy._getLocalizedLabels("RenderingStarted") + "...";
                    _proxy._progressStatus(70, 85, 50);
                    window.setTimeout(function () { progressStatus() }, 300);
                }
                else
                    window.setTimeout(function () { progressStatus() }, 1);
                function progressStatus() {
                    if (msg[0] != undefined && msg.length > 0) {
                        _proxy.setJSONRecords(msg[0].Value); _proxy.setOlapReport(msg[1].Value);
                        if (msg[2] != null && msg[2] != undefined && msg[2].Key == "PageSettings") {
                            if (_proxy.model.enableVirtualScrolling) {
                                _proxy._categPageCount = Math.ceil(JSON.parse(msg[3].Value).Column / JSON.parse(msg[2].Value).CategorialPageSize);
                                _proxy._seriesPageCount = Math.ceil(JSON.parse(msg[3].Value).Row / JSON.parse(msg[2].Value).SeriesPageSize);
                                _proxy._categCurrentPage = JSON.parse(msg[2].Value).CategorialCurrentPage;
                                _proxy._seriesCurrentPage = JSON.parse(msg[2].Value).SeriesPageSize;
                            }
                            else if ($(".e-olappager")[0] != null && $(".e-olappager")[0] != undefined)
                                _proxy._pagerObj.initPagerProperties(JSON.parse(msg[3].Value), JSON.parse(msg[2].Value));
                            if (msg[4] != null && msg[4] != undefined)
                                _proxy.model.customObject = msg[4].Value;
                        }
                        if (msg[2] != null && msg[2] != undefined && msg[2].Key != "PageSettings")
                            _proxy.model.customObject = msg[2].Value;
                    }
                    else if (msg.d != undefined && msg.d.length > 0) {
                        _proxy.setJSONRecords(msg.d[0].Value); _proxy.setOlapReport(msg.d[1].Value);
                        if (msg.d[2] != null && msg.d[2] != undefined && msg.d[2].Key == "PageSettings") {
                            if (_proxy.model.enableVirtualScrolling) {
                                _proxy._categPageCount = Math.ceil(JSON.parse(msg.d[3].Value).Column / JSON.parse(msg.d[2].Value).CategorialPageSize);
                                _proxy._seriesPageCount = Math.ceil(JSON.parse(msg.d[3].Value).Row / JSON.parse(msg.d[2].Value).SeriesPageSize);
                                _proxy._categCurrentPage = JSON.parse(msg.d[2].Value).CategorialCurrentPage;
                                _proxy._seriesCurrentPage = JSON.parse(msg.d[2].Value).SeriesCurrentPage;
                            }
                            else if ($(".e-olappager")[0] != null && $(".e-olappager")[0] != undefined)
                                _proxy._pagerObj.initPagerProperties(JSON.parse(msg.d[3].Value), JSON.parse(msg.d[2].Value));
                            if (msg.d[4] != null && msg.d[4] != undefined)
                                _proxy.model.customObject = msg.d[4].Value;
                        }
                        if (msg.d[2] != null && msg.d[2] != undefined && msg.d[2].Value == "PageSettings")
                            _proxy.model.customObject = msg.d[2].Value;
                    }
                    else if (!ej.isNullOrUndefined(msg) && ej.isNullOrUndefined(msg.d)) {
                        if (msg.PivotRecords != undefined) {
                            _proxy.setJSONRecords(msg.PivotRecords); _proxy.setOlapReport(msg.OlapReport);
                            if (msg.HeaderCounts != undefined && _proxy.model.enableVirtualScrolling) {
                                _proxy._categPageCount = Math.ceil(JSON.parse(msg.HeaderCounts).Column / JSON.parse(msg.PageSettings).CategorialPageSize);
                                _proxy._seriesPageCount = Math.ceil(JSON.parse(msg.HeaderCounts).Row / JSON.parse(msg.PageSettings).SeriesPageSize);
                                _proxy._categCurrentPage = JSON.parse(msg.PageSettings).CategorialCurrentPage;
                                _proxy._seriesCurrentPage = JSON.parse(msg.PageSettings).SeriesCurrentPage;
                            }
                            else if (msg.HeaderCounts != undefined && $(".e-olappager")[0] != null && $(".e-olappager")[0] != undefined)
                                _proxy._pagerObj.initPagerProperties(JSON.parse(msg.HeaderCounts), JSON.parse(msg.PageSettings));
                        }
                        if (msg.customObject != null && msg.customObject != undefined)
                            _proxy.model.customObject = msg.customObject;
                    }
                    if (_proxy.model.afterServiceInvoke != null)
                        _proxy._trigger("afterServiceInvoke", { action: ej.olap.OlapGrid.GridAction.DrillDown, element: _proxy.element, customObject: _proxy.model.customObject });
                    if (_proxy.getJSONRecords() != null && _proxy.getJSONRecords().length > 0)
                        _proxy.renderControlFromJSON(_proxy.getJSONRecords());
                    if (_proxy.model.enableVirtualScrolling)
                        _proxy._applyVScrolling();
                    if (_proxy.model.progressMode != "infinite") {
                        if (_proxy._JSONRecords == null)
                            $("#" + _proxy._id + "progressContainer").hide();
                        else {
                            if (_proxy.model.progressMode == "progress")
                                $("#" + _proxy._id + "progressText")[0].innerHTML = _proxy._getLocalizedLabels("RenderingSucceeded") + "...";
                        }
                    }
                    else if (typeof oclientWaitingPopup != 'undefined' && oclientWaitingPopup != null) {
                        if ($("#" + oclientProxy._id + "_maxView")[0] && this._maxViewLoading) {
                            this._maxViewLoading.hide();
                        }
                        else
                            oclientWaitingPopup.hide();
                    }
                    else
                        ogridWaitingPopup.hide();
                }
            }
            catch (err) {
                if (_proxy.model.progressMode != "infinite") {
                    if (_proxy.model.progressMode == "progress")
                        $("#" + _proxy._id + "progressText")[0].innerHTML = _proxy._getLocalizedLabels("RenderingFailed") + "!!!";
                    window.clearInterval(oChartTimer);
                    ogridProgressBar.setModel({ value: 70, text: " Failed" });
                }
            }
            var eventArgs = { action: "initialize", customObject: this.model.customObject, element: this.element };
            this._trigger("renderSuccess", eventArgs);
        },

        /**
         * 	This function receives the controls update from service-end after drilldown which would be utilized for rendering the widget.
         *
         * @private  
         */
        _drillDownSuccess: function (msg) {
            _proxy = this;
            if (_proxy.model.progressMode == "progress")
                $("#" + _proxy._id + "progressText")[0].innerHTML = _proxy._getLocalizedLabels("MDXqueryExecutedSuccessfully") + "...";
            if (msg[0] != undefined) {
                this.setJSONRecords(msg[0].Value);
                this.setOlapReport(msg[1].Value);
                if (typeof (oclientProxy) != "undefined") {
                    oclientProxy.currentReport = msg[1].Value;
                    oclientProxy.reports = msg[2].Value;
                }
                if (msg[3] != null && msg[3] != undefined && msg[3].Key == "PageSettings") {
                    if (_proxy.model.enableVirtualScrolling) {
                        _proxy._categPageCount = Math.ceil(JSON.parse(msg[4].Value).Column / JSON.parse(msg[3].Value).CategorialPageSize);
                        _proxy._seriesPageCount = Math.ceil(JSON.parse(msg[4].Value).Row / JSON.parse(msg[3].Value).SeriesPageSize);
                        _proxy._categCurrentPage = JSON.parse(msg[3].Value).CategorialCurrentPage;
                        _proxy._seriesCurrentPage = JSON.parse(msg[3].Value).SeriesPageSize;
                    }
                    else if ($(".e-olappager")[0] != null && $(".e-olappager")[0] != undefined)
                        _proxy._pagerObj.initPagerProperties(JSON.parse(msg[4].Value), JSON.parse(msg[3].Value));
                    if (msg[5] != null && msg[5] != undefined)
                        _proxy.model.customObject = msg[5].Value;
                }
                if (msg[3] != null && msg[3] != undefined && msg[3].Key != "PageSettings")
                    _proxy.model.customObject = msg[3].Value;
            }
            else if (msg.d != undefined) {
                this.setJSONRecords(msg.d[0].Value);
                this.setOlapReport(msg.d[1].Value);
                if (typeof (oclientProxy) != "undefined") {
                    oclientProxy.currentReport = msg.d[1].Value;
                    oclientProxy.reports = msg.d[2].Value;
                }
                if (msg.d[3] != null && msg.d[3] != undefined && msg.d[3].Key == "PageSettings") {
                    if (_proxy.model.enableVirtualScrolling) {
                        _proxy._categPageCount = Math.ceil(JSON.parse(msg.d[4].Value).Column / JSON.parse(msg.d[3].Value).CategorialPageSize);
                        _proxy._seriesPageCount = Math.ceil(JSON.parse(msg.d[4].Value).Row / JSON.parse(msg.d[3].Value).SeriesPageSize);
                        _proxy._categCurrentPage = JSON.parse(msg.d[3].Value).CategorialCurrentPage;
                        _proxy._seriesCurrentPage = JSON.parse(msg.d[3].Value).SeriesCurrentPage;
                    }
                    else if ($(".e-olappager")[0] != null && $(".e-olappager")[0] != undefined)
                        _proxy._pagerObj.initPagerProperties(JSON.parse(msg.d[4].Value), JSON.parse(msg.d[3].Value));
                    if (msg.d[5] != null && msg.d[5] != undefined)
                        _proxy.model.customObject = msg.d[5].Value;
                }
                if (msg.d[3] != null && msg.d[3] != undefined && msg.d[3].Key != "PageSettings")
                    _proxy.model.customObject = msg.d[3].Value;
            }
            else {
                this.setJSONRecords(msg.PivotRecords);
                this.setOlapReport(msg.OlapReport);
                if (typeof (oclientProxy) != "undefined") {
                    oclientProxy.currentReport = msg.OlapReport;
                    oclientProxy.reports = msg.reports;
                }
                if (msg.length > 3) {
                    if (_proxy.model.enableVirtualScrolling) {
                        _proxy._categPageCount = Math.ceil(JSON.parse(msg.HeaderCounts).Column / JSON.parse(msg.PageSettings).CategorialPageSize);
                        _proxy._seriesPageCount = Math.ceil(JSON.parse(msg.HeaderCounts).Row / JSON.parse(msg.PageSettings).SeriesPageSize);
                        _proxy._categCurrentPage = JSON.parse(msg.PageSettings).CategorialCurrentPage;
                        _proxy._seriesCurrentPage = JSON.parse(msg.PageSettings).SeriesCurrentPage;
                    }
                    else if ($(".e-olappager")[0] != null && $(".e-olappager")[0] != undefined)
                        _proxy._pagerObj.initPagerProperties(JSON.parse(msg.HeaderCounts), JSON.parse(msg.PageSettings));
                }
                if (msg.customObject != null && msg.customObject != undefined)
                    _proxy.model.customObject = msg.customObject;
            }
            if (this.model.afterServiceInvoke != null)
                this._trigger("afterServiceInvoke", { action: ej.olap.OlapGrid.GridAction.DrillDown, element: this.element, customObject: this.model.customObject });
            if (_proxy.model.progressMode == "progress")
                $("#" + _proxy._id + "progressText")[0].innerHTML = _proxy._getLocalizedLabels("RenderingStarted") + "...";
            this.renderControlFromJSON(this.getJSONRecords());
            this.model.currentReport = this.getOlapReport();
            this._trigger("drillSuccess", this.element);
            if (_proxy.model.enableVirtualScrolling)
                _proxy._applyVScrolling();
            if (typeof oclientWaitingPopup != 'undefined' && oclientWaitingPopup != null) {
                if ($("#" + oclientProxy._id + "_maxView")[0] && this._maxViewLoading) {
                    this._maxViewLoading.hide();
                    oclientWaitingPopup.hide();
                }
                else if (oclientProxy.model.displaySettings.mode == "gridOnly")
                    oclientWaitingPopup.hide();
            }
            else if (typeof oclientProxy != 'undefined') {
                if (oclientProxy && !oclientProxy.ogridObj._startDrilldown && !oclientProxy.ochartObj._startDrilldown || oclientProxy.model.displaySettings.mode == "gridOnly")
                    oclientWaitingPopup.hide();
            }
            else if (_proxy.model.progressMode == "infinite" && typeof oclientWaitingPopup == 'undefined') {
                ogridWaitingPopup = $("#" + this._id).data("ejWaitingPopup");
                ogridWaitingPopup.hide();
            }
            if (typeof oclientProxy != 'undefined')
                oclientProxy.ogridObj._startDrilldown = false;
            else
                ogridWaitingPopup.hide();
            var eventArgs = { action: this._drillAction, customObject: this.model.customObject, element: this.element };
            this._trigger("renderSuccess", eventArgs);
        },

        /**
        * Perform an asynchronous HTTP (Ajax) request.
        * @return jQuery
        * @example 
        * &lt;div id="OlapGrid"&gt;&lt;/div&gt; <br> 
        * &lt;script&gt;
        * // Create OLAP Grid
        * $('#OlapGrid').ejOlapGrid({
        *       url: "OlapGridService.svc",
        *   });
        * var gridObj = $("#OlapGrid").data("ejOlapGrid");
        * gridObj.doAjaxPost("POST", "/OlapGridService.svc/Initialize", {"key", "Hello World"}, "_renderControlSuccess", null);
        * // Initiate an Ajax request.
        * &lt;/script&gt;
        * @memberof ejOlapGrid
        * @instance
        */
        doAjaxPost: function (type, url, data, onSuccess, onComplete) {
            $.ajax({
                type: type,
                url: url,
                contentType: 'application/json; charset=utf-8',
                dataType: 'json',
                data: data,
                success: $.proxy(onSuccess, this),
                complete: ej.proxy(function (onComplete) {
                    $.proxy(onComplete, this);
                    var eventArgs = { "action": this._currentAction, "customObject": this.model.customObject, element: this.element };
                    this._trigger("renderComplete", eventArgs);
                }, this),
                error: ej.proxy(function (msg, textStatus, errorThrown) {
                    if (_proxy.model.progressMode != "infinite") {
                        window.clearInterval(oGridTimer);
                        if (_proxy.model.progressMode == "progress")
                            $("#" + _proxy._id + "progressText")[0].innerHTML = _proxy._getLocalizedLabels("MDXqueryExecutionFailed") + "!!!";
                        $("#" + ogridProgressBar._id).show();
                        ogridProgressBar.setModel({ value: 0, text: errorThrown });
                    }
                    else
                        ogridWaitingPopup.hide();
                    var eventArgs = { "action": this._drillAction != "" ? this._drillAction : "initialize", "customObject": this.model.customObject, "element": this.element, "Message": msg };
                    this._trigger("renderFailure", eventArgs);
                }, this)
            });
        },

        /**
        * Perform an asynchronous HTTP (FullPost) submit.
        * @return jQuery
        * @example 
        * &lt;div id="OlapGrid"&gt;&lt;/div&gt; <br> 
        * &lt;script&gt;
        * // Submitting required informatin to the service.
        * $('#OlapGrid').ejOlapGrid({
        *       url: "OlapGridService.svc",
        *   });
        * var gridObj = $("#OlapGrid").data("ejOlapGrid");
        * gridObj.doPostBack("/OlapGridService.svc/Initialize", {"key", "Hello World"});
        * // Initiate an submit.
        * &lt;/script&gt;
        * @memberof ejOlapGrid
        * @instance
        */
        doPostBack: function (url, params) {
            var form = $('<form>').attr({ 'action': url, 'method': 'POST', 'name': 'export' });
            var addParam = function (paramName, paramValue) {
                var input = $('<input type="hidden" title="params">').attr({
                    'id': paramName,
                    'name': paramName,
                    'value': paramValue
                }).appendTo(form);
            };
            for (var item in params)
                addParam(item, params[item]);
            form.appendTo(document.body).submit().remove();
        },

        /**
        * 	This function re-renders the OLAP Grid on clicking the navigating buttons on OLAP Pager.
        * @return jQuery
        * @example 
        * &lt;div id="OlapGrid"&gt;&lt;/div&gt; <br> 
        * &lt;script&gt;
        * // Refreshing OLAP Grid while perform paging.
        * $('#OlapGrid').ejOlapGrid({
        *       url: "OlapGridService.svc",
        *   });
        * var gridObj = $("#OlapGrid").data("ejOlapGrid");
        * gridObj.refreshPagedOlapGrid("series", 2);
        * // Initiate an Ajax request.
        * &lt;/script&gt;
        * @memberof ejOlapGrid
        * @instance
        */
        refreshPagedOlapGrid: function (axis, pageNo) {
            if (_proxy.model.progressMode == "infinite" && typeof oclientWaitingPopup == 'undefined') {
                ogridWaitingPopup = $("#" + this._id).data("ejWaitingPopup");
                ogridWaitingPopup.show();
            }
            else {
                this._progressStatus(0, 70, 140);
                if (_proxy.model.progressMode == "progress")
                    $("#" + _proxy._id + "progressText")[0].innerHTML = "Preparing and executing MDX query...";
            }
            axis = axis.indexOf('categ') != -1 ? "categorical" : "series";
            var report;
            try {
                report = JSON.parse(this.getOlapReport()).Report;
            }
            catch (err) {
                report = this.getOlapReport();
            }
            this.doAjaxPost("POST", this.model.url + "/" + this.model.serviceMethodSettings.paging, JSON.stringify({ "action": "Paging", "pagingInfo": axis + ":" + pageNo, "currentReport": report, "layout": this.layout(), "customObject": null }), this._renderControlSuccess);
        },

        /**
        * 	This function receives the JSON formatted datasource to render the OLAP Grid control.
        * @return jQuery
        * @example 
        * &lt;div id="OlapGrid"&gt;&lt;/div&gt; <br> 
        * &lt;script&gt;
        * // Rendering OLAP Grid from given JSON formatted data.
        * $('#OlapGrid').ejOlapGrid({
        *       url: "OlapGridService.svc",
        *   });
        * var gridObj = $("#OlapGrid").data("ejOlapGrid");
        * gridObj.renderControlFromJSON({this.getJSONRecords()});
        * // Rendering OLAP Grid.
        * &lt;/script&gt;
        * @memberof ejOlapGrid
        * @instance
        */
        renderControlFromJSON: function (jsonObj) {
            _proxy = this;
            if (_proxy.model.progressMode != "infinite")
                window.clearInterval(oGridTimer);
            var tableOlapGrid = $("<table cellspacing=\"0\" cellpadding=\"0\" role='grid'  aria-readonly='true'></table>");
            this.rowCount = 0;

            if (jsonObj != null) {
                jQuery.each(jsonObj, function (index, value) {
                    if (this.Index[0] == 0)
                        _proxy.rowCount++;
                });

                var theadOlapGrid = $("<thead></thead>");
                var tbodyOlapGrid = $("<tbody></tbody>");
                var istheadOlapGrid = true; var columnHeaderHeight = 0;
                if (jsonObj[0].CSS == "none")
                    columnHeaderHeight = jsonObj[0].RowSpan - 1;
                else {
                    jQuery.each(jsonObj, function (index, value) {
                        columnHeaderHeight++;
                        return (jsonObj[index].CSS == "colheader");
                    });
                    columnHeaderHeight = columnHeaderHeight - 2;
                }

                jQuery.each(jsonObj, function (index, value) {
                    if (index < _proxy.rowCount) {
                        if (istheadOlapGrid) {
                            istheadOlapGrid = (index >= columnHeaderHeight) ? false : true;
                            var tRow = "<tr role='row'>";
                            for (var column = index; column < jsonObj.length; column = column + _proxy.rowCount) {
                                if (jsonObj[column].Span != "Block") {
                                    tRow = tRow + "<th" + (jsonObj[column].CSS == "none" ? "" : " class=" + jsonObj[column].CSS) + " p=" + jsonObj[column].Index + " colspan=" + jsonObj[column].ColSpan + " rowspan=" + jsonObj[column].RowSpan + " role='columnheader'>" + (jsonObj[column].State == 2 ? "<span class=\"expand\ e-icon\">&nbsp;</span>" : (jsonObj[column].State == 1 ? "<span class=\"collapse\ e-icon\" title=\"Collapsed\">&nbsp;</span>" : "")) + jsonObj[column].Value + "</th>";
                                    if (jsonObj[column].RowSpan > 1 && jsonObj[column].ColSpan <= 1) {
                                        for (var i = column + 1; i < column + jsonObj[column].RowSpan; i++)
                                            jsonObj[i].Span = "Block";
                                    }
                                    else if (jsonObj[column].ColSpan > 1 && jsonObj[column].RowSpan <= 1) {
                                        for (var i = column + _proxy.rowCount; i < column + (jsonObj[column].ColSpan * _proxy.rowCount); i = i + _proxy.rowCount)
                                            jsonObj[i].Span = "Block";
                                    }
                                    else if (jsonObj[column].ColSpan > 1 && jsonObj[column].RowSpan > 1) {
                                        for (var i = column; i < column + jsonObj[column].RowSpan; i++) {
                                            for (var j = i + _proxy.rowCount; j < i + (jsonObj[column].ColSpan * _proxy.rowCount); j = j + _proxy.rowCount)
                                                jsonObj[j].Span = "Block";
                                            jsonObj[i].Span = "Block";
                                        }
                                    }
                                }
                            }
                            tRow += "</tr>";
                            $(theadOlapGrid).append(tRow);
                        }
                        else {
                            var tRow = "<tr role='row'>";
                            for (var column = index; column < jsonObj.length; column = column + _proxy.rowCount) {
                                if (jsonObj[column].Span != "Block") {
                                    if (jsonObj[column].CSS == "rowheader" || jsonObj[column].CSS == "summary" || jsonObj[column].CSS == "summary") {
                                        if (_proxy.layout().toLowerCase() == "excellikelayout") {
                                            tRow = tRow + "<th" + (jsonObj[column].CSS == "none" ? "" : " class=\"" + jsonObj[column].CSS) + "\" p=" + jsonObj[column].Index + " colspan=" + jsonObj[column].ColSpan + " rowspan=" + jsonObj[column].RowSpan + " role='rowheader'>" + (jsonObj[column].State == 2 ? "<span style=\"margin-left:" + (jsonObj[column].Level * 10) + "px\" class=\"expand\ e-icon\">&nbsp;</span>" : (jsonObj[column].State == 1 ? "<span style=\"margin-left:" + (jsonObj[column].Level * 10) + "px\" class=\"collapse\ e-icon\">&nbsp;</span>" : "<span style=\"margin-left:" + (jsonObj[column].Level * 10) + "px\">&nbsp;</span>")) + jsonObj[column].Value + "</th>";
                                        }
                                        else {
                                            tRow = tRow + "<th" + (jsonObj[column].CSS == "none" ? "" : " class=\"" + jsonObj[column].CSS) + "\" p=" + jsonObj[column].Index + " colspan=" + jsonObj[column].ColSpan + " rowspan=" + jsonObj[column].RowSpan + " role='rowheader'>" + (jsonObj[column].State == 2 ? "<span class=\"expand\ e-icon\">&nbsp;</span>" : (jsonObj[column].State == 1 ? "<span class=\"collapse\ e-icon\">&nbsp;</span>" : "")) + jsonObj[column].Value + "</th>";
                                        }
                                    }
                                    else
                                        tRow = tRow + "<td" + (jsonObj[column].CSS == "none" ? "" : " class=\"" + jsonObj[column].CSS) + "\" p=" + jsonObj[column].Index + " colspan=" + jsonObj[column].ColSpan + " rowspan=" + jsonObj[column].RowSpan + " role='gridcell'>" + (jsonObj[column].State == 2 ? "<span class=\"expand\ e-icon\">&nbsp;</span>" : (jsonObj[column].State == 1 ? "<span class=\"collapse\ e-icon\">&nbsp;</span>" : "")) + jsonObj[column].Value + "</td>";

                                    if (jsonObj[column].RowSpan > 1 && jsonObj[column].ColSpan <= 1) {
                                        for (var i = column + 1; i < column + jsonObj[column].RowSpan; i++)
                                            jsonObj[i].Span = "Block";
                                    }
                                    else if (jsonObj[column].ColSpan > 1 && jsonObj[column].RowSpan <= 1) {
                                        for (var i = column + _proxy.rowCount; i < column + (jsonObj[column].ColSpan * _proxy.rowCount); i = i + _proxy.rowCount)
                                            jsonObj[i].Span = "Block";
                                    }
                                    else if (jsonObj[column].ColSpan > 1 && jsonObj[column].RowSpan > 1) {
                                        for (var i = column; i < column + jsonObj[column].RowSpan; i++) {
                                            for (var j = i + _proxy.rowCount; j < i + (jsonObj[column].ColSpan * _proxy.rowCount); j = j + _proxy.rowCount)
                                                jsonObj[j].Span = "Block";
                                            jsonObj[i].Span = "Block";
                                        }
                                    }
                                }
                            }
                            tRow += "</tr>";
                            $(tbodyOlapGrid).append(tRow);
                        }

                    }
                });
                if (_proxy.model.progressMode != "infinite") {
                    window.clearInterval(oGridTimer);
                    this._progressStatus(85, 90, 80);
                    window.setTimeout(function () { progressStatus() }, 400);
                    function progressStatus() {
                        $(tableOlapGrid).append(theadOlapGrid);
                        $(tableOlapGrid).append(tbodyOlapGrid);
                    }
                }
                else {
                    $(tableOlapGrid).append(theadOlapGrid);
                    $(tableOlapGrid).append(tbodyOlapGrid);
                }
            }
            if (_proxy.model.progressMode != "infinite") {
                window.clearInterval(oGridTimer);
                this._progressStatus(90, 101 + ":100%", 80);
                window.setTimeout(function () { timeOut() }, 900);
                function timeOut() {
                    _proxy.element.html("");
                    if (_proxy.model.enableVirtualScrolling) {
                        _proxy._createVirtualOlapGrid(tableOlapGrid);
                    }
                    else
                        _proxy.element.append(tableOlapGrid);
                    _proxy._unWireEvents();
                    _proxy._wireEvents();
                    $("#" + _proxy._id + "progressContainer").hide();
                }
            }
            else {
                this.element.html("");
                if (_proxy.model.enableVirtualScrolling) {
                    _proxy._createVirtualOlapGrid(tableOlapGrid);
                }
                else
                    this.element.append(tableOlapGrid);
                if (_proxy.model.enableRTL) {
                    this.element.find('table').addClass("e-rtl");
                    this.element.find('table .rowheader,.colheader,.summary').css("text-align", "right");
                }
                else {
                    this.element.find('table').removeClass("e-rtl");
                    this.element.find('table .rowheader,.colheader,.summary').css("text-align", "left");
                }
                this._unWireEvents();
                this._wireEvents();
            }
        },

        /**
        * 	This function appends virtual scroll bar to the OLAP Grid.
        *
        * @private  
        */
        _createVirtualOlapGrid: function (tableOlapGrid) {
            var verticalScroll = _proxy._seriesPageCount > 1 ? ej.buildTag("div.vScrollPanel")[0].outerHTML : "";
            var horizontalScroll = _proxy._categPageCount > 1 ? ej.buildTag("div.hScrollPanel")[0].outerHTML : "";
            var oGridDiv = ej.buildTag("td.virtualScrollGrid", tableOlapGrid[0].outerHTML)[0];
            var vertiScrollTd = ej.buildTag("td.virtualScrollElement", verticalScroll);
            var horiScrollTd = ej.buildTag("td.virtualScrollElement", horizontalScroll);
            var trow = ej.buildTag("tr.virtualScrollElement", oGridDiv.outerHTML + vertiScrollTd[0].outerHTML);
            var trow2 = ej.buildTag("tr.virtualScrollElement", horiScrollTd[0].outerHTML + ej.buildTag("td.virtualScrollElement")[0].outerHTML);
            var virtualOlapGrid = ej.buildTag("tbody.oGridOuterDiv.virtualScrollElement", trow[0].outerHTML + trow2[0].outerHTML);
            this.element.html(ej.buildTag("table.virtualScrollElement", virtualOlapGrid[0].outerHTML));
            var seriesPageIndicator = ej.buildTag("div.seriesPageIndicator inActive", ej.buildTag("span.axislabel", this._getLocalizedLabels("SeriesPage"))[0].outerHTML + ej.buildTag("span.series_CurrentPage", _proxy._seriesCurrentPage)[0].outerHTML + " / " + ej.buildTag("span.series_pageCount", _proxy._seriesPageCount)[0].outerHTML)[0].outerHTML;
            if (_proxy._seriesPageCount > 1) _proxy.element.append(seriesPageIndicator);
            var categPageIndicator = ej.buildTag("div.categPageIndicator inActive", ej.buildTag("span.axislabel", this._getLocalizedLabels("CategoricalPage"))[0].outerHTML + ej.buildTag("span.categ_CurrentPage", _proxy._categCurrentPage)[0].outerHTML + " / " + ej.buildTag("span.categ_pageCount", _proxy._categPageCount)[0].outerHTML)[0].outerHTML;
            if (_proxy._categPageCount > 1) _proxy.element.append(categPageIndicator);
        }
    });

    /**
    * Enum for Layout Types. Allows the user access the different layouts for OLAP Grid. 
    *
    * @enum {String}
    * @global
    */
    ej.olap.OlapGrid.Layout = {
        Normal: "normal",
        NormalTopSummary: "normaltopsummary",
        NoSummaries: "nosummaries",
        ExcelLikeLayout: "excellikelayout"
    };

    //OLAP Grid Localization
    ej.olap.OlapGrid.locale = {};
    //Default english localized values
    ej.olap.OlapGrid.locale["en-US"] = {
        SeriesPage: "Series Page",
        CategoricalPage: "Categorical Page",
        MDXqueryExecutionFailed: "MDX query execution failed",
        PreparingAndExecutingMDXquery: "Preparing and executing MDX query",
        MDXqueryExecutedSuccessfully: "MDX query executed successfully",
        RenderingStarted: "Rendering Started",
        RenderingSucceeded: "Rendering Succeeded",
        RenderingFailed: "Rendering failed"
    };

    /**
    * Enum for Grid Action. Contains the information about current action of the OLAP Grid. 
    *
    * @enum {String}
    * @global
    */
    ej.olap.OlapGrid.GridAction = {
        Initialize: "initialize",
        DrillDown: "drillDown"
    };

    /**
    * Enum for Progress Mode. Allows the user to select a appropriate progress indicator. 
    *
    * @enum {String}
    * @global
    */
    ej.olap.OlapGrid.ProgressMode = {
        Infinite: "infinite",
        Normal: "normal",
        Progress: "progress"
    };

})(jQuery, Syncfusion); ;
/**
* @fileOverview Pivot Schema Designer control to list out and analyze the data
* @copyright Copyright Syncfusion Inc. 2001 - 2013. All rights reserved.
*  Use of this code is subject to the terms of our license.
*  A copy of the current license can be obtained at any time by e-mailing
*  licensing@syncfusion.com. Any infringement will be prosecuted under
*  applicable laws. 
* @version 12.1 
* @author <a href="mailto:licensing@syncfusion.com">Syncfusion Inc</a>
*/

(function ($, ej, undefined) {

    /**
    * @namespace ej
    * @class ejPivotSchemaDesigner
    * @requires jQuery
    * @requires ej.common.all
    * @requires ej.tools.js
    * @classdesc Custom Design for Pivot Schema Designer control.
    * @example 
    * &lt;div id="PivotSchemaDesigner"&gt; &lt;/div&gt; <br> 
    * &lt;script&gt;
    * // Create Pivot Schema Designer
    * $("#PivotSchemaDesigner").ejPivotSchemaDesigner(...); 	
    * &lt;/script&gt;
    */
    ej.widget("ejPivotSchemaDesigner", "ej.PivotSchemaDesigner", {

        // Root Css Class
        _rootCSS: "e-pivotschemadesigner",

        // Widget element will be automatically set in this
        element: null,

        // User-defined model will be automatically set in this
        model: null,

        validTags: ["div", "span"],

        defaults: {
            //properties
            /**		
            * Connects the service using the specified URL for any server updates.	See {@link url}
            * @default ""
            * @type {String}
            * @example 
            * //To set service url during initialization  
            * $("#PivotSchemaDesigner").ejPivotSchemaDesigner({url: "/wcf/PivotService.svc"});
            * @example 
            * //Get or set the size API, after initialization:<br>
            * //Gets the url value  
            * $("#PivotSchemaDesigner").ejPivotSchemaDesigner("option","url");<br>   
            * //Sets the url value 
            * $("#PivotSchemaDesigner").ejPivotSchemaDesigner("option","url", "/wcf/PivotService.svc" ); 
            * @memberof ejPivotSchemaDesigner
            * @instance
            */
            url: "",

            /**		
            * Sets the CSS name for custom operation.
            * @default ""
            * @type {String}
            * @example 
            * //To set the CSS name during initialization  
            * $("#PivotSchemaDesigner").ejPivotSchemaDesigner({cssClass: "gradient-lime"});	
            * @example 
            * //Get or set css class for initialization:<br>
            *	// Gets the css name.		
            *	$("#PivotSchemaDesigner").ejPivotSchemaDesigner("option", "cssClass");			
            *	// Sets the css class to PivotSchemaDesigner
            *	$("#PivotSchemaDesigner").ejPivotSchemaDesigner("option", "cssClass",  "gradient-lime" ); 		
            * @memberof ejPivotSchemaDesigner
            * @instance
            */
            cssClass: "",

            height: "630px",
            width: "415px",
            layout: "normal",

            /**		
            * Sets the pivot control bounds with the Pivot Schema Designer.
            * @default null
            * @type {object}
            * @example 
            * //To set the pivot control during initialization  
            * var control = $("#OlapGrid").data("ejOlapGrid");
            * $("#PivotSchemaDesigner").ejPivotSchemaDesigner({pivotControl: control});
            * @example 
            * //Get or set the pivot control, after initialization:<br>
            *	//Gets the pivot control value
            *	$("#PivotSchemaDesigner").ejPivotSchemaDesigner("option", "pivotControl");<br>			
            *	//Sets the pivot control
            *	$("#PivotSchemaDesigner").ejPivotSchemaDesigner("option", "pivotControl", control);			* 		
            * @memberof ejPivotSchemaDesigner
            * @instance
            */
            pivotControl: null,

            /**		
            * Allows the user to Pivot Table Field List to Pivot Schema Designer.
            * @default new Array()
            * @type {Array}
            * @example 
            * //To set the Pivot Table Field List during initialization  
            * $("#PivotSchemaDesigner").ejPivotSchemaDesigner({pivotTableFieldList: });
            * @example 
            * //Get or set the Pivot Table Field List, after initialization:<br>
            *	//Gets the virtual calling state
            *	$("#PivotSchemaDesigner").ejPivotSchemaDesigner("option", "pivotTableFieldList");<br>		
            *	//Sets the Pivot Table Field List 
            *	$("#PivotSchemaDesigner").ejPivotSchemaDesigner("option", "pivotTableFieldList","true");			* 		
            * @memberof ejPivotSchemaDesigner
            * @instance
            */
            pivotTableFields: [],

            /**		
            * Allows the user to set Pivot Calculation List.
            * @default new Array()
            * @type {Array}
            * @example 
            * //To set the pivot calculation cist during initialization  
            * $("#PivotSchemaDesigner").ejPivotSchemaDesigner({pivotTableFieldList: });
            * @example 
            * //Get or set the pivot calculation list, after initialization:<br>
            *	//Gets the cell context state
            *	$("#PivotSchemaDesigner").ejPivotSchemaDesigner("option", "pivotTableFieldList");<br>			
            *	//Sets the pivot calculation list value
            *	$("#PivotSchemaDesigner").ejPivotSchemaDesigner("option", "pivotTableFieldList","true");			* 		
            * @memberof ejPivotSchemaDesigner
            * @instance
            */
            pivotCalculations: [],

            /**		
            * Allows the user to set the pivot column list.
            * @default new Array()
            * @type {Array}
            * @example 
            * //To set the pivot column list during initialization  
            * $("#PivotSchemaDesigner").ejPivotSchemaDesigner({pivotTableFieldList: true});
            * @example 
            * //Get or set the pivot column list, after initialization:<br>
            *	//Gets the pivot column list
            *	$("#PivotSchemaDesigner").ejPivotSchemaDesigner("option", "pivotTableFieldList");<br>			
            *	//Sets the pivot column list value
            *	$("#PivotSchemaDesigner").ejPivotSchemaDesigner("option", "pivotTableFieldList","true");			* 		
            * @memberof ejOlapGrid
            * @instance
            */
            pivotColumns: [],

            /**		
            * Allows the user to set the pivot row list.
            * @default new Array()
            * @type {Array}
            * @example 
            * //To set the pivot row list during initialization  
            * $("#PivotSchemaDesigner").ejPivotSchemaDesigner({pivotRowList: true});
            * @example 
            * //Get or set the pivot row list, after initialization:<br>
            *	//Gets the pivot row list
            *	$("#PivotSchemaDesigner").ejPivotSchemaDesigner("option", "pivotRowList");<br>		
            *	//Sets the pivot row list
            *	$("#PivotSchemaDesigner").ejPivotSchemaDesigner("option", "pivotRowList","true");			* 		
            * @memberof ejPivotSchemaDesigner
            * @instance
            */
            pivotRows: [],

            /**		
            * Allows the user to set the filter list.
            * @default new Array()
            * @type {Array}
            * @example 
            * //To set the filter list during initialization  
            * $("#PivotSchemaDesigner").ejPivotSchemaDesigner({filterList: true});
            * @example 
            * //Get or set the filter list, after initialization:<br>
            *	//Gets the filter list
            *	$("#PivotSchemaDesigner").ejPivotSchemaDesigner("option", "filterList");<br>   				
            *	//Sets the filter list
            *	$("#PivotSchemaDesigner").ejPivotSchemaDesigner("option", "filterList","true");			* 		
            * @memberof ejPivotSchemaDesigner
            * @instance
            */
            filters: [],

            serviceMethods: {
                fetchMembers: "FetchMembers", nodeStateModified: "NodeStateModified",
                nodeDropped: "NodeDropped", removeButton: "RemoveButton", memberExpand: "MemberExpanded", filtering: "filtering"
            },
        },

        /**
        * Provides information about the properties of type Array and Object.
        *
        * @private
        */
        dataTypes: {
            serviceMethods: "data",
            customObject: "data",
            pivotControl: "data",
            pivotTableFieldList: "array",
            pivotCalculationList: "array",
            pivotColumnList: "array",
            pivotRowList: "array",
            filterList: "array"
        },

        /**
        * Initializes the variables and loads the widget. 
        *
        * @private
        */
        _init: function () {
            this._initPrivateProperties();
            this._load();
        },

        /**
        * Destroys the widget while it's no longer required. 
        *
        * @private
        */
        _destroy: function () {
            this.element.empty().removeClass("e-pivotschemadesigner" + this.model.cssClass);
        },

        /**
        * Initializes the local variables. 
        *
        * @private
        */
        _initPrivateProperties: function () {
            this._dialogTitle = "";
            this._currentMembers = new Array();
            this._memberTreeObj = null;
            this._tableTreeObj = null;
            this._dialogOKBtnObj = null;
            this._curFilteredText = "";
            this._curFilteredAxis = "";
            this._tempFilterData = null;
            this._droppedClass = "";
            this._selectedTreeNode = null;
            this._selectedMember = "";
            this._isDragging = false;
            this._dataModel = "";
            this._droppedPosition = "";
            this._currentCubeName = "";
            _pivotProxy = this;
            this._nodeDropedParams = "";
        },

        /**
        * This function is the point were loading/rendering of the OLAP Grid widget is initiated.
        *
        * @private  
        */
        _load: function () {
            var itemProps = "", report;
            if (!jQuery.isEmptyObject(this.model.pivotControl.getOlapReport())) {
                itemProps = JSON.parse(JSON.parse(this.model.pivotControl.getOlapReport()).ItemsProperties);
                report = JSON.parse(this.model.pivotControl.getOlapReport());
                this._setTableFields(itemProps);
                this._setPivotRows(report.PivotRows);
                this._setPivotColumns(report.PivotColumns);
                this._setPivotCalculations(report.PivotCalculations);
                this._setFilters(report.Filters);
                this._dataModel = report.DataModel;
                this._currentCubeName = report.CurrentCube;
            }
            var fieldTable = ej.buildTag("table.headerTable", ej.buildTag("tr", ej.buildTag("td", ej.buildTag("div.listHeader", ej.buildTag("span.headerText", "PivotTable Field List", {})[0].outerHTML, {})[0].outerHTML + ej.buildTag("div.listSubhead", ej.buildTag("span.subheadText", "Choose fields to add to report:", {})[0].outerHTML, {})[0].outerHTML, {}).attr("valign", "top")[0].outerHTML, {})[0].outerHTML, {}).attr("width", "100%").attr("height", "10%")[0].outerHTML + ej.buildTag("table.fieldTable", ej.buildTag("tbody", ej.buildTag("tr", ej.buildTag("td", ej.buildTag("div.schemaFieldList", ej.buildTag("div.schemaFieldTree#schemaFieldTree")[0].outerHTML)[0].outerHTML).attr("valign", "top")[0].outerHTML, {})[0].outerHTML, {})[0].outerHTML, {}).attr("width", "95%").attr("height", "30%")[0].outerHTML,
            axisTable = ej.buildTag("table.axisTable", ej.buildTag("tbody", ej.buildTag("tr", ej.buildTag("td", ej.buildTag("div.pivotHeader", ej.buildTag("span.headerText", "Report Filter", {})[0].outerHTML, {})[0].outerHTML + ej.buildTag("div.schemaFilter", this._createPivotButtons(this.model.filters, "filter"))[0].outerHTML, {}).attr("width", "50%").attr("height", "50%").attr("valign", "top")[0].outerHTML + ej.buildTag("td", ej.buildTag("div.rPivotHeader", ej.buildTag("span.headerText", "Column Label", {})[0].outerHTML, {})[0].outerHTML + ej.buildTag("div.schemaColumn", this._createPivotButtons(this.model.pivotColumns, "column"), {})[0].outerHTML, {}).attr("width", "50%").attr("height", "50%").attr("valign", "top")[0].outerHTML, {})[0].outerHTML + ej.buildTag("tr", ej.buildTag("td", ej.buildTag("div.pivotHeader", ej.buildTag("span.headerText", "Row Label", {})[0].outerHTML, {})[0].outerHTML + ej.buildTag("div.schemaRow", this._createPivotButtons(this.model.pivotRows, "row"), {})[0].outerHTML, {}).attr("width", "50%").attr("height", "50%").attr("valign", "top")[0].outerHTML + ej.buildTag("td", ej.buildTag("div.rPivotHeader", ej.buildTag("span.headerText", "Values", {})[0].outerHTML, {})[0].outerHTML + ej.buildTag("div.schemaValue", this._createPivotButtons(this.model.pivotCalculations, "values"), {})[0].outerHTML, {}).attr("width", "50%").attr("height", "50%").attr("valign", "top")[0].outerHTML, {})[0].outerHTML, {})[0].outerHTML, {}).attr("width", "100%").attr("height", "46%")[0].outerHTML,
            htmlTag = fieldTable + ej.buildTag("div.centerDiv")[0].outerHTML + ej.buildTag("div.centerHead", "Drag fields between areas below:")[0].outerHTML + axisTable;
            $(htmlTag).appendTo("#" + this._id);
            this.element.width(this.model.width).height(this.model.height);
            this.element.find(".pivotButton .pvtBtn").ejButton({ size: "normal"}).ejDraggable({
                handle: 'button', clone: true,
                cursorAt: { left: -5, top: -5 },
                dragStart: ej.proxy(function (args) {
                    this._isDragging = true;
                }, this),
                dragStop: ej.proxy(this._onPvtBtnDropped, this),
                helper: ej.proxy(function (event, ui) {
                    if (event.sender.target.className.indexOf("e-btn") > -1)
                        return $(event.sender.target).clone().addClass("dragClone").appendTo(this.element);
                    else
                        return false;
                }, this)
            });

            this.element.find(".pivotButton .filterBtn").ejButton({
                size: "normal",
                contentType: "imageonly",
                prefixIcon: "filter"
            });
            this.element.find(".pivotButton input").ejToggleButton({
                size: "normal",
                contentType: "imageonly",
                defaultPrefixIcon: "ascending",
                activePrefixIcon: "descending",
                click: ej.proxy(this._onSortBtnClick, this)
            });
            this.element.find(".schemaFieldTree").ejTreeView({
                showCheckbox: true,
                fields: { id: "id", parentId: "pid", text: "name", isChecked: "isSelected", spriteCssClass: "spriteCssClass", dataSource: this.model.pivotTableFields },
                allowDragAndDrop: true,
                allowDropChild: false,
                allowDropSibling: false,
                dragAndDropAcrossControl: true,
                nodeDropped: ej.proxy(this._onNodeDropped, this),
                nodeDragStart: ej.proxy(this._onNodeDrag, this)
            });
            this._tableTreeObj = this.element.find(".schemaFieldTree").data("ejTreeView");
            this._tableTreeObj.element.find(".measureGroupCDB").siblings(".e-chkbox-wrap").remove();
            this._tableTreeObj.element.find(".folderCDB").siblings(".e-chkbox-wrap").remove();
            this._tableTreeObj.element.find(".dimensionCDB").siblings(".e-chkbox-wrap").remove();
            this._tableTreeObj.element.find(".measureGroupCDB").parents("li").append(ej.buildTag("span.elementSeparator")[0].outerHTML);
            this._tableTreeObj.element.find(".dimensionCDB").parents("li").append(ej.buildTag("span.elementSeparator")[0].outerHTML);
            var treeViewElements = this._tableTreeObj.element.find("li");
            for (var i = 0; i < treeViewElements.length; i++) {
                treeViewElements[i].setAttribute("tag", this.model.pivotTableFields[i].tag);
                var checkedBoxNode = $(treeViewElements[i]).find(".e-chkbox-wrap");
                if (checkedBoxNode[0].attributes[4].value == "true") {
                    var dropSpan = ej.buildTag("span.e-icon", { "display": "none" }).addClass("treeDrop")[0].outerHTML;
                    $($(treeViewElements[i]).find(".e-text")[0]).after(dropSpan);
                }
            }
            this._tableTreeObj.model.nodeCheck = ej.proxy(this._onCheckedStateModified, this);
            this._tableTreeObj.model.nodeUncheck = ej.proxy(this._onCheckedStateModified, this);
            this._tableTreeObj.element.find(".attributeCDB").parent().siblings("ul").remove();
            this._tableTreeObj.element.find(".attributeCDB").siblings(".e-plus").remove();
            this._tableTreeObj.element.find(".hierarchyCDB").parent().siblings("ul").find(".e-chkbox-wrap").remove();
			if(this._tableTreeObj.element.find(".e-plus").length == 0){
             this._tableTreeObj.element.find(".e-item").css("padding","0px");
             } 
            this.element.find(".schemaFilter, .schemaColumn, .schemaRow, .schemaValue").ejDroppable({
            });
            var contextTag = ej.buildTag("ul.pivotTreeContext#pivotTreeContext", ej.buildTag("li", ej.buildTag("a", "Add to Report Filter")[0].outerHTML)[0].outerHTML + ej.buildTag("li", ej.buildTag("a", "Add to Row Labels")[0].outerHTML)[0].outerHTML + ej.buildTag("li", ej.buildTag("a", "Add to Column Labels")[0].outerHTML)[0].outerHTML + ej.buildTag("li", ej.buildTag("a", "Add to Values")[0].outerHTML)[0].outerHTML)[0].outerHTML;
            $(this.element).append(contextTag);
            $("#pivotTreeContext").ejMenu({
                menuType: ej.MenuType.ContextMenu,
                openOnClick: false,
                contextMenuTarget: this._tableTreeObj.element[0],
                click: ej.proxy(this._onTreeContextClick, this),
                beforeOpen: ej.proxy(this._onContextOpen, this)
            });
            this._unWireEvents();
            this._wireEvents();
        },

        _setFirst: true,

        /**
        * To configure the properties at runtime using SetModel		
        * @private
        */
        _setModel: function (options) {
            for (var key in options) {
                switch (key) {
                    case "OlapReport": this.setOlapReport(options[key]); break;
                }
            }
        },

        /**
        * This function is invoked to wire the events for UI interaction.
        *
        * @private  
        */
        _wireEvents: function () {
            if (this.model.layout != "excel") {
                this._on(this.element, "mouseover", ".pivotButton .pvtBtn", ej.proxy(function (evt) {
                    $(evt.target.parentNode).find("span.removePivotBtn").css("display", "inline-block");
                }, this));
                this._on(this.element, "mouseleave", ".pivotButton", ej.proxy(function (evt) {
                    this.element.find(".pivotButton > span.removePivotBtn").css("display", "none");
                }, this));
            }
            else {
                this._on(this.element, "mouseover", ".pivotButton .pvtBtn", ej.proxy(function (evt) {
                    if (this._isDragging) {
                        this.element.find(".dropIndicator").removeClass("dropIndicatorHover");
                        $(evt.target).siblings(".dropIndicator").addClass("dropIndicatorHover");
                    }
                }, this));
                this._on(this.element, "mouseleave", ".pivotButton", ej.proxy(function (evt) {
                    if (this._isDragging)
                        $(evt.target).siblings(".dropIndicator").removeClass("dropIndicatorHover");
                }, this));
            }
            this._on(this.element, "mouseover", ".schemaFieldTree .e-text", ej.proxy(function (evt) {
                if ($(evt.target).parent().find(".measureGroupCDB").length == 0 && $(evt.target).parent().find(".chartCDB").length == 0 && $(evt.target).parent().find(".dimensionCDB").length == 0 && $(evt.target).parent().find(".folderCDB").length == 0)
                    $(evt.target).siblings("span.e-icon.treeDrop").css({ display: "inline-block", color: "#5c5c5c","margin-left": "-20px", "margin-top": "3px", "position": "absolute" });
                else if ($(evt.target).parent().find(".measureGroupCDB").length > 0 || $(evt.target).parent().find(".dimensionCDB").length > 0 || $(evt.target).parent().find(".folderCDB").length > 0)
                    $(evt.target).css({ "cursor": "default" });
            }, this));
            this._on(this.element, "mouseout", ".schemaFieldTree li", ej.proxy(function (evt) {
                if (evt.target.className.indexOf("treeDrop") < 0 && evt.target.className.indexOf("e-text") < 0)
                    $(this._tableTreeObj.element).find("span.e-icon.treeDrop").css({ display: "none" });
            }, this));
            this._on(this.element, "click", ".filterBtn", ej.proxy(this._onFilterBtnClick, this));
            this._on(this.element, "click", ".removePivotBtn", ej.proxy(this._onRemoveBtnClick, this));
            this._on(this._tableTreeObj.element, "click", ".treeDrop", ej.proxy(this._onFilterBtnClick, this));
        },

        /**
        * This function is invoked to unwire the events registered for UI interaction.
        *
        * @private  
        */
        _unWireEvents: function () {
            this._off(this.element, "click", ".filterBtn");
            this._off(this.element, "mouseover", ".pivotButton");
            this._off(this.element, "mouseover", ".pivotButton .pvtBtn");
            this._off(this.element, "mouseleave", ".pivotButton");
            this._off(this.element, "mouseover", ".schemaFieldTree li");
            this._off(this.element, "mouseout", ".schemaFieldTree li");
            this._off(this._tableTreeObj.element, "click", ".treeDrop");
        },

        /**
        * To get the localized labels.
        * @private
        */
        //get Localization text
        _getLocalizedLabels: function (property) {
            return ej.OlapGrid.localization[this.localization()][property] === undefined ? ej.OlapGrid.localization["en-US"][property] : ej.OlapGrid.localization[this.localization()][property];
        },

        _setTableFields: function (items) {
            if (!ej.isNullOrUndefined(items))
                this.model.pivotTableFields = items;
        },

        _setPivotRows: function (items) {
            if (!ej.isNullOrUndefined(items))
                this.model.pivotRows = items;
        },

        _setPivotColumns: function (items) {
            if (!ej.isNullOrUndefined(items))
                this.model.pivotColumns = items;
        },

        _setPivotCalculations: function (items) {
            if (!ej.isNullOrUndefined(items))
                this.model.pivotCalculations = items;
        },

        _setFilters: function (items) {
            if (!ej.isNullOrUndefined(items))
                this.model.filters = items;
        },

        _getSortedHeaders: function () {
            var labels = this.element.find(".descending").parents("label"), headers = "";
            for (var i = 0; i < labels.length; i++)
                headers += $(labels[i]).attr("for").replace("toggleBtn", "") + "##";
            return headers;
        },

        _onContextOpen: function (args) {
            if ($(args.target.parentElement).find(".measureGroupCDB").length > 0 || $(args.target.parentElement).find(".folderCDB").length > 0 || $(args.target.parentElement).find(".dimensionCDB").length > 0)
                return false;
            else if (this._dataModel == "Olap")
                this._selectedMember = $(args.target).parents("li:eq(0)").attr("tag");
            else if (this._dataModel == "Pivot")
                this._selectedMember = args.target.text;
        },

        _onNodeDrag: function (args) {
            this._isDragging = true;
            if ($(args.target.parentElement).find(".measureGroupCDB").length > 0 || $(args.target.parentElement).find(".folderCDB").length > 0 || $(args.target.parentElement).find(".dimensionCDB").length > 0)
                return false;
        },

        _onTreeContextClick: function (args) {
            var headerTag, filterTag, filterItems = "",
            selectedPvtBtn = this.element.find(".pivotButton:contains(" + this._selectedMember + ")"),
            isFiltered = $(selectedPvtBtn).find(".filterIndicator").length > 0 ? true : false,
            isSorted = $(selectedPvtBtn).find("~ span:eq(0) span.descending").length > 0 ? true : false, report, axisName,
            params, eventArgs;
            selectedTreeNode = _pivotProxy._getNodeByUniqueName(this._selectedMember);
            this._droppedClass = args.events.text == "Add to Report Filter" ? "schemaFilter" : args.events.text == "Add to Row Labels" ? "schemaRow" :
            args.events.text == "Add to Column Labels" ? "schemaColumn" : args.events.text == "Add to Values" ? "schemaValue" : "";
            for (var i = 0; i < this.model.pivotTableFields.length; i++) {
                if (this.model.pivotTableFields[i].name == this._selectedMember) {
                    this.model.pivotTableFields[i].isSelected = true;
                    headerTag = this.model.pivotTableFields[i];
                }
            }
            if (!ej.isNullOrUndefined(this._tempFilterData)) {
                for (var i = 0; i < this._tempFilterData.length; i++) {
                    if (!ej.isNullOrUndefined(this._tempFilterData[i][this._selectedMember])) {
                        for (var j = 0; j < this._tempFilterData[i][this._selectedMember].length; j++)
                            filterItems += "##" + this._tempFilterData[i][this._selectedMember][j];
                    }
                }
            }
            if (selectedPvtBtn.length > 1)
                for (var i = 0; i < selectedElement.length; i++) {
                    if ($(selectedPvtBtn[i]).text() == this._selectedMember)
                        $(selectedPvtBtn[i]).remove();
                }
            else
                $(selectedPvtBtn).remove();
            try {
                report = JSON.parse(this.model.pivotControl.getOlapReport()).Report;
            }
            catch (err) {
                report = this.model.pivotControl.getOlapReport();
            }
            if (this._dataModel == "Olap") {
                _pivotProxy._nodeDropedParams = this._droppedClass == "schemaFilter" ? "Slicer" : this._droppedClass == "schemaColumn" ? "Categorical" : this._droppedClass == "schemaRow" ? "Series" : "";
                if (this._droppedClass == "schemaValue") {
                    var tempAxis = this.element.find(".pivotButton:contains('Measures')").parents(".e-droppable").className.split(" ")[0];
                    _pivotProxy._nodeDropedParams = tempAxis == "schemaColumn" ? "Categorical" : tempAxis == "schemaRow" ? "Series" : "";
                }
                this._tableTreeObj.checkNode(selectedTreeNode);
                _pivotProxy._nodeDropedParams = "";
            }
            else {
                if (selectedTreeNode.find(".e-chk-inact").length > 0)
                    this._tableTreeObj.checkNode(selectedTreeNode);
                else {
                    this._createPivotButton(this._selectedMember, this._droppedClass == "schemaRow" ? "row" : this._droppedClass == "schemaColumn" ? "column" : this._droppedClass == "schemaFilter" ? "filter" : this._droppedClass == "schemaValue" ? "value" : "", isFiltered, isSorted, "");
                    eventArgs = JSON.stringify({ "action": "nodeDropped", "dropAxis": this._droppedClass + "::", "headerTag": JSON.stringify(headerTag), "sortedHeaders": this._getSortedHeaders(), "currentReport": report });
                    if (!ej.isNullOrUndefined(ogridWaitingPopup))
                        ogridWaitingPopup.show();
                    this.model.pivotControl.doAjaxPost("POST", this.model.pivotControl.model.url + "/" + this.model.serviceMethods.nodeDropped, eventArgs, this.model.pivotControl._renderControlSuccess);
                }
            }
        },

        _onSortBtnClick: function (args) {
            var serializedCustomObject = JSON.stringify(this.model.customObject),
            headers = this._getSortedHeaders();
            var eventArgs = this.model.customObject != {} && headers != "" ? JSON.stringify({ "action": "sorting", "sortedHeaders": headers, "currentReport": this.model.pivotControl.getOlapReport(), "customObject": serializedCustomObject }) :
            this.model.customObject != {} ? JSON.stringify({ "action": "sorting", "currentReport": this.model.pivotControl.getOlapReport(), "customObject": serializedCustomObject }) :
            headers != "" ? JSON.stringify({ "action": "sorting", "sortedHeaders": headers, "currentReport": this.model.pivotControl.getOlapReport() }) : JSON.stringify({ "action": "sorting" });
            if (!ej.isNullOrUndefined(ogridWaitingPopup))
                ogridWaitingPopup.show();
            this.model.pivotControl.doAjaxPost("POST", this.model.pivotControl.model.url + "/" + this.model.pivotControl.model.serviceMethods.sorting, eventArgs, this.model.pivotControl._renderControlSuccess);
        },

        _onFilterBtnClick: function (args) {
            $(this._tableTreeObj.element).find("span.e-icon.treeDrop").css({ display: "none" });
            this._selectedTreeNode = $($(args.target).parent()).parent();
            if (args.target.tagName.toLowerCase() == "span") {
                this.model.layout != "excel" ? this._curFilteredText = $($(args.target).parents(".pivotButton")).find(".pvtBtn").text() : this._curFilteredText = $(args.target).parent().text();
                if ($(args.target).parents("li:eq(1)").children().children("span").hasClass("hierarchyCDB")) {
                    this._curFilteredText = $($(args.target).parents("li:eq(1)")).children().find(".e-text")[0].innerHTML;
                }
                if (this._dataModel == "Pivot")
                this._curFilteredAxis = !ej.isNullOrUndefined(this.element.find(".e-btn:contains('" + this._curFilteredText + "')").parents(".e-droppable")[0]) ? this.element.find(".e-btn:contains('" + this._curFilteredText + "')").parents(".e-droppable")[0].className.split(" ")[0] : "";
                this._dialogTitle = this._dataModel == "Olap" ? $(this.element.find(".pivotButton:contains('" + $(args.target).parent().text() + "')")[0]).attr("tag") : $(args.target).parent().text() || args.target.id.replace("filterBtn", "") || $(args.target).parents(".filterBtn")[0].id.replace("filterBtn", "");
                if (this._dataModel == "Olap")
                for (var btnCount = 0 ; btnCount < this.element.find(".e-btn").length; btnCount++) {
                    if (!ej.isNullOrUndefined($($($(this.element.find(".e-btn"))[btnCount]).parent()[0]).attr("tag")) && $($($(this.element.find(".e-btn"))[btnCount]).parent()[0]).attr("tag").indexOf(this._curFilteredText) > -1) {
                        this._curFilteredAxis = !ej.isNullOrUndefined($($(this.element.find(".e-btn"))[0]).parents(".e-droppable")[0]) ? $($(this.element.find(".e-btn"))[0]).parents(".e-droppable")[0].className.split(" ")[0] : "";
                        this._dialogTitle = $($($(this.element.find(".e-btn"))[btnCount]).parent()[0]).attr("tag");
                    }
                }
                var report;
                try {
                    report = JSON.parse(this.model.pivotControl.getOlapReport()).Report;
                }
                catch (err) {
                    report = this.model.pivotControl.getOlapReport();
                }
                sortedHeaders = this._getSortedHeaders();
                eventArgs = JSON.stringify({ "action": "fetchMembers", "headerTag": this._dialogTitle || "UniqueName##" + $($(args.target).parents("li")[0]).attr("tag"), "sortedHeaders": sortedHeaders, "currentReport": report });
                if (!ej.isNullOrUndefined(ogridWaitingPopup))
                    ogridWaitingPopup.show();
                this.doAjaxPost("POST", this.model.pivotControl.model.url + "/" + this.model.serviceMethods.fetchMembers, eventArgs, this._fetchMemberSuccess);
            }
        },

        _onRemoveBtnClick: function (args) {
            var headerText = $(args.target.parentElement).text(),
            selectedTreeNode = $($(this._tableTreeObj.element).find("div:contains(" + headerText + ")>span")[1]).find(".e-chk-act"),
            headerTag;
            $(selectedTreeNode).removeClass("e-chk-act").addClass("e-chk-inact");
            $(selectedTreeNode).children("e-chk-activeicon").removeClass("e-chk-activeicon");
            for (var i = 0; i < this.model.pivotTableFields.length; i++) {
                if (this.model.pivotTableFields[i].FieldHeader == headerText) {
                    this.model.pivotTableFields[i].IsSelected = false;
                    headerTag = this.model.pivotTableFields[i];
                }
            }
            $(args.target.parentElement).remove();
            var eventArgs = JSON.stringify({ "action": "nodeStateModified", "headerTag": JSON.stringify(headerTag), "sortedHeaders": this._getSortedHeaders(), "currentReport": this.model.pivotControl.getOlapReport() });
            if (!ej.isNullOrUndefined(ogridWaitingPopup))
                ogridWaitingPopup.show();
            this.model.pivotControl.doAjaxPost("POST", this.model.pivotControl.model.url + "/" + this.model.serviceMethods.nodeStateModified, eventArgs, this.model.pivotControl._renderControlSuccess);
        },

        _onNodeCheckChanges: function (args) {
            if (args.value == "(All)" && args.type == "nodeUncheck") {
                this._memberTreeObj.model.nodeCheck = "";
                this._memberTreeObj.model.nodeUncheck = "";
                this.element.find(".editorTreeView").ejTreeView("unCheckAll");
                this._memberTreeObj.model.nodeCheck = ej.proxy(this._onNodeCheckChanges, this);
                this._memberTreeObj.model.nodeUncheck = ej.proxy(this._onNodeCheckChanges, this);
                this._dialogOKBtnObj.disable();
            }
            else if (args.value == "(All)" && args.type == "nodeCheck") {
                this._memberTreeObj.model.nodeCheck = "";
                this._memberTreeObj.model.nodeUncheck = "";
                this.element.find(".editorTreeView").ejTreeView("checkAll");
                $(this._memberTreeObj.element.find("li:first")).find("div .e-chkbox-small > span > span:eq(0)").removeClass("e-chk-indetericon");
                this._memberTreeObj.model.nodeCheck = ej.proxy(this._onNodeCheckChanges, this);
                this._memberTreeObj.model.nodeUncheck = ej.proxy(this._onNodeCheckChanges, this);
                this._dialogOKBtnObj.enable();
            }
            else if (args.value != "(All)") {
                var uncheckedNodes = this._memberTreeObj.element.find(":input:gt(0).nodecheckbox:not(:checked)"),
                firstNode = $(this._memberTreeObj.element.find("li:first")).find("div .e-chkbox-small > span > span:eq(0)");
                if (uncheckedNodes.length == 0 || (uncheckedNodes.length == 1 && uncheckedNodes[0].id[uncheckedNodes[0].id.length - 1] == 0)) {
                    $(firstNode).parent().removeClass("e-chk-inact").removeClass("e-chk-ind").addClass("e-chk-act");
                    firstNode.removeClass("e-chk-indetericon").addClass("e-chk-activeicon");
                }
                else if (args.type == "check" && uncheckedNodes.length == 1 && uncheckedNodes[0].id[uncheckedNodes[0].id.length - 1] == 0)
                    firstNode.removeClass("e-chk-indetericon").addClass("e-chk-activeicon");
                else if (uncheckedNodes.length > 0)
                    firstNode.removeClass("e-chk-activeicon").addClass("e-chk-indetericon");
                this._dialogOKBtnObj.enable();
                if (args.type == "uncheck" && uncheckedNodes.length + 1 == this._memberTreeObj.element.find("li").length) {
                    firstNode.removeClass("e-chk-activeicon").removeClass("e-chk-indetericon");
                    this._dialogOKBtnObj.disable();
                }
            }
        },

        _onCheckedStateModified: function (args) {
            var headerText = $(args.currentElement).find("a")[0].textContent, filterTag = "", filterItems = "", selectedElement = "", headerTag = "", uniqueName = "";
            if (this._dataModel == "Olap") {
                if (!args.currentElement[0].id.indexOf("[Measures]") > -1)
                for (var i = 0; i < _pivotProxy.element.find(".pivotButton").length; i++) {
                    headerTag = $(_pivotProxy.element.find(".pivotButton")[i]).attr("tag"); uniqueName = "";
                    uniqueName = ((headerTag.indexOf("[Measures]") > -1) || (headerTag.indexOf("[") > -1)) ? headerTag.split(":")[1] : this._getNodeUniqueName(headerTag);
                    if ($(args.currentElement).attr("tag") == uniqueName) {
                        selectedElement = $(_pivotProxy.element.find(".pivotButton")[i]);
                        break;
                    }
                }
                else
                    selectedElement = _pivotProxy.element.find(".pivotButton:contains(" + headerText + ")"), headerTag, report;
            }
            else
                selectedElement = _pivotProxy.element.find(".pivotButton:contains(" + headerText + ")"), headerTag, report;
            var isFiltered = selectedElement.length > 0 && $(selectedElement).siblings(".filterIndicator").length > 0 ? true : false,
            isSorted = args.currentElement.find("~ span:eq(0) span.descending").length > 0 ? true : false;
            for (var i = 0; i < _pivotProxy.model.pivotTableFields.length; i++) {
                if (_pivotProxy.model.pivotTableFields[i].name == headerText && args.type == "nodeCheck") {
                    _pivotProxy.model.pivotTableFields[i].isSelected = true;
                    headerTag = _pivotProxy.model.pivotTableFields[i];
                }
                else if (this.model.pivotTableFields[i].name == headerText && args.type == "nodeUncheck") {
                    _pivotProxy.model.pivotTableFields[i].isSelected = false;
                    headerTag = _pivotProxy.model.pivotTableFields[i];
                }
            }
            if (ej.isNullOrUndefined(headerTag))
                headerTag = $(args.currentElement).attr("tag");

            if (args.type == "nodeUncheck") {
                if (selectedElement.length > 1)
                    for (var i = 0; i < selectedElement.length; i++) {
                        if ($(selectedElement[i]).text() == headerText)
                            $(selectedElement[i]).remove();
                    }
                else
                    $(selectedElement).remove();
            }
            else {
                if (this._droppedClass != "")
                    this._createPivotButton(headerText, this._droppedClass == "schemaColumn" ? "column" : this._droppedClass == "schemaRow" ? "row" : this._droppedClass == "schemaValue" ? "value" : this._droppedClass == "schemaFilter" ? "filter" : "", isFiltered, isSorted, "");
                else
                    this._createPivotButton(headerText, ((this._dataModel == "Olap" && $(args.currentElement).attr("tag").indexOf("[Measures]") < 0) || (this._dataModel == "Pivot" && headerTag.pivotType == "PivotItem")) ? "row" : "value", isFiltered, isSorted, "");
            }
            if (!ej.isNullOrUndefined(this._tempFilterData)) {
                for (var i = 0; i < this._tempFilterData.length; i++) {
                    if (!ej.isNullOrUndefined(this._tempFilterData[i][headerText])) {
                        for (var j = 0; j < this._tempFilterData[i][headerText].length; j++)
                            filterItems += "##" + this._tempFilterData[i][headerText][j];
                    }
                }
            }
            try {
                report = JSON.parse(this.model.pivotControl.getOlapReport()).Report;
            }
            catch (err) {
                report = this.model.pivotControl.getOlapReport();
            }
            filterTag = "schemaRow::" + headerText + "::FILTERED" + filterItems;
            var dropAxis = this._droppedClass != "" ? this._droppedClass : headerTag.pivotType == "PivotItem" ? "schemaRow" : "schemaValue", eventArgs, axisName, params;
            this._droppedClass = "";
            if (!ej.isNullOrUndefined(ogridWaitingPopup))
                ogridWaitingPopup.show();
            if (!(args.type == "nodeCheck") && !(args.type == "nodeUncheck") && dropAxis != "") {
                if (_pivotProxy._dataModel == "Pivote") {
                    eventArgs = JSON.stringify({ "action": "nodeStateModified", "headerTag": JSON.stringify(headerTag), "dropAxis": dropAxis + "::" + this._droppedPosition, "sortedHeaders": this._getSortedHeaders(), "filterParams": filterTag, "currentReport": report });
                    _pivotProxy._droppedPosition = "";
                    _pivotProxy.model.pivotControl.doAjaxPost("POST", this.model.pivotControl.model.url + "/" + this.model.serviceMethods.nodeStateModified, eventArgs, this.model.pivotControl._renderControlSuccess);
                }
                else {
                    var dropSpan = ej.buildTag("span.e-icon", { "display": "none" }).addClass("treeDrop")[0].outerHTML;
                    $(args.currentElement.find(".e-text")[0]).after(dropSpan);
                    headerTag = $(args.currentElement).attr("tag");
                    axisName = this._nodeDropedParams == "" ? ($(args.currentElement).attr("tag").indexOf("[Measures]") >= 0 ? "Categorical" : "Series") : this._nodeDropedParams;
                    params = this._currentCubeName + "--" + headerTag + "--" + axisName + "--", report;
                    eventArgs = JSON.stringify({ "action": "nodeDropped", "dropType": "TreeNode", "nodeInfo": params, "currentReport": report });
                    this.doAjaxPost("POST", this.model.pivotControl.model.url + "/" + this.model.serviceMethods.nodeDropped, eventArgs, this._onDroppedSuccess);
                }
            }
            else if (args.type == "nodeCheck") {
                if (this._dataModel == "Olap") {
                    var dropSpan = ej.buildTag("span.e-icon", { "display": "none" }).addClass("treeDrop")[0].outerHTML;
                    $(args.currentElement.find(".e-text")[0]).after(dropSpan);
                    headerTag = $(args.currentElement).attr("tag");
                    axisName = this._nodeDropedParams == "" ? ($(args.currentElement).attr("tag").indexOf("[Measures]") >= 0 ? "Categorical" : "Series") : this._nodeDropedParams;
                    params = this._currentCubeName + "--" + headerTag + "--" + axisName + "--", report;
                    eventArgs = JSON.stringify({ "action": "nodeDropped", "dropType": "TreeNode", "nodeInfo": params, "currentReport": report });
                    this.doAjaxPost("POST", this.model.pivotControl.model.url + "/" + this.model.serviceMethods.nodeDropped, eventArgs, this._onDroppedSuccess);
                }
                else {
                    eventArgs = JSON.stringify({ "action": "nodeStateModified", "headerTag": JSON.stringify(headerTag), "dropAxis": dropAxis + "::", "sortedHeaders": this._getSortedHeaders(), "filterParams": filterTag, "currentReport": report });
                    this.model.pivotControl.doAjaxPost("POST", this.model.pivotControl.model.url + "/" + this.model.serviceMethods.nodeStateModified, eventArgs, this.model.pivotControl._renderControlSuccess);
                }
            }
            else if (args.type == "nodeUncheck") {
                if (this._dataModel == "Olap") {
                    $(args.currentElement.find(".treeDrop")).remove();
                    $(args.currentElement.find(".e-icon")).remove();
                    $(args.currentElement).removeClass("filter");
                    var headerTag = $(selectedElement).attr("tag"), report, eventArgs;
                    eventArgs = JSON.stringify({ "action": "removeButton", "headerInfo": headerTag, "currentReport": report });
                    if (ej.isNullOrUndefined(headerTag))
                        return false;
                    if ( headerTag.indexOf("[Measures]") >= 0 && _pivotProxy.element.find(".schemaValue .pivotButton").length == 0) {
                        var tmp = headerTag.split(":"),
                        tempAxis = tmp[0] == "Rows" ? ".schemaRow" : tmp[0] == "Columns" ? ".schemaColumn" : "";
                        _pivotProxy.element.find(tempAxis + " .pivotButton:contains('Measures')").remove();
                    }
                    else if (headerTag.indexOf("[Measures]") < 0 && headerTag.indexOf("Measures") >= 0)
                        _pivotProxy.element.find(".schemaValue .pivotButton").remove();
                    this.model.pivotControl.doAjaxPost("POST", _pivotProxy.model.pivotControl.model.url + "/" + _pivotProxy.model.serviceMethods.removeButton, eventArgs, _pivotProxy.model.pivotControl._renderControlSuccess)
                }
                else {
                    eventArgs = JSON.stringify({ "action": "nodeStateModified", "headerTag": JSON.stringify(headerTag), "dropAxis": dropAxis + "::", "sortedHeaders": _pivotProxy._getSortedHeaders(), "filterParams": filterTag, "currentReport": report });
                    _pivotProxy.model.pivotControl.doAjaxPost("POST", _pivotProxy.model.pivotControl.model.url + "/" + _pivotProxy.model.serviceMethods.nodeStateModified, eventArgs, _pivotProxy.model.pivotControl._renderControlSuccess);
                }
            }
        },

        _onNodeDropped: function (args) {
            if (args.target.context.className.indexOf("e-droppable") >= 0) {

                //---------------------------------------------------------------------

                var headerText = args.dropedElement.text(), headerTag = args.dropedElement.parent().parent().attr("tag"), report, eventArgs;
                try {
                    report = JSON.parse(this.model.pivotControl.getOlapReport()).Report;
                }
                catch (err) {
                    report = this.model.pivotControl.getOlapReport();
                }
                if (this._dataModel == "Olap") {
                    selectedTreeNode = _pivotProxy._getNodeByUniqueName(headerTag);
                    var axis = args.target[0].className == "pivotButton" ? $(args.target).parents()[0] : args.target[0].tagName.toLowerCase() == "td" ? args.target.children(":last")[0] : args.target;
                    this._droppedClass = axis.className.split(" ")[0];
                    var axisName = axis.className.split(" ")[0] == "schemaColumn" ? "Categorical" : axis.className.split(" ")[0] == "schemaRow" ? "Series" : axis.className.split(" ")[0] == "schemaFilter" ? "Slicer" : "";
                    if (axisName == "" && args.dropedElement.parents("li:eq(0)").attr("tag").indexOf("[Measures]") > -1)
                        axisName = this.element.find(".schemaRow .pivotButton:contains('Measures')").length > 0 ? "Series" : "Categorical";
                    _pivotProxy._nodeDropedParams = axisName;
                    if ($(selectedTreeNode.find(".e-chkbox-wrap")).attr("aria-checked") == "true")
                        this._tableTreeObj.unCheckNode(selectedTreeNode);
                    else
                        this._tableTreeObj.checkNode(selectedTreeNode);
                    _pivotProxy._nodeDropedParams = "";
                }
                else {
                    this._droppedClass = args.event.target.className.split(" ")[0];
                    selectedTreeNode = this._tableTreeObj.element.find("li:contains('" + headerText + "')");
                    this._tableTreeObj.checkNode(selectedTreeNode);
                    this._droppedClass = "";
                }


                //----------------------------------------------------------------------

                this._isDragging = false;
                var temp = this.element.find(".dropIndicator").removeClass("dropIndicatorHover");
                var headerTag = $(args.dropedElement).parents("li:eq(0)").attr("tag"), headerText = args.dropedElement.text(), filterTag, filterItems = "", report,
            selectedPvtBtn = this.element.find(".pivotButton:contains(" + headerText + ")"),
            selectedTreeNode = this._tableTreeObj.element.find("li:contains('" + headerText + "'):last"),
            isFiltered = $(selectedPvtBtn).find(".filterIndicator").length > 0 ? true : false,
            isSorted = $(selectedPvtBtn).find("~ span:eq(0) span.descending").length > 0 ? true : false;
                this._droppedClass = args.event.target.className.split(" ")[0];
                if (ej.isNullOrUndefined(headerTag))
                    for (var i = 0; i < this.model.pivotTableFields.length; i++) {
                        if (this.model.pivotTableFields[i].name == headerText) {
                            this.model.pivotTableFields[i].isSelected = true;
                            headerTag = this.model.pivotTableFields[i];
                        }
                    }
                if (!ej.isNullOrUndefined(this._tempFilterData))
                    for (var i = 0; i < this._tempFilterData.length; i++) {
                        if (!ej.isNullOrUndefined(this._tempFilterData[i][headerText])) {
                            for (var j = 0; j < this._tempFilterData[i][headerText].length; j++)
                                filterItems += "##" + this._tempFilterData[i][headerText][j];
                        }
                    }
                if (args.dropedElement[0].tagName.toLowerCase() == "button") {
                    if (selectedPvtBtn.length > 1)
                        for (var i = 0; i < selectedPvtBtn.length; i++) {
                            if ($(selectedPvtBtn[i]).text() == headerText)
                                $(selectedPvtBtn[i]).remove();
                        }
                    else
                        $(selectedPvtBtn).remove();
                }
                try {
                    report = JSON.parse(this.model.pivotControl.getOlapReport()).Report;
                }
                catch (err) {
                    report = this.model.pivotControl.getOlapReport();
                }
               
			}
        },

        _onPvtBtnDropped: function (args) {
            this._isDragging = false;
            this.element.find(".dropIndicator").removeClass("dropIndicatorHover");
            var axis = null, axisName = "", headerTag = "", headerText, droppedClass, isFiltered = false, isSorted = false;
            if (this._dataModel == "Olap") {
                axis = args.target.className.indexOf("pvtBtn") > -1 ? $(args.target).parents(".e-droppable")[0] : args.target.className.indexOf("e-droppable") ? args.target : args.target[0].tagName.toLowerCase() == "td" ? args.target.children(":last")[0] : args.target;
                axisName = axis.className.split(" ")[0] == "schemaColumn" ? "Categorical" : axis.className.split(" ")[0] == "schemaRow" ? "Series" : axis.className.split(" ")[0] == "schemaFilter" ? "Slicer" : "";
                if (axisName == "" && args.element.parent().attr("tag").indexOf("[Measures]") > -1)
                    axisName = this.element.find(".schemaRow .pivotButton:contains('Measures')").length > 0 ? "Series" : "Categorical";
                headerTag = $(args.element.parent()[0]).attr("tag");
                droppedClass = axis.className.split(" ")[0];
            }
            else {
                headerText = args.element.text(), droppedClass = args.target.className.indexOf("e-droppable") >= 0 ? args.target.className.split(" ")[0] : "",
                    headerTag, isFiltered = args.element.find("~ .filterIndicator").length > 0 ? true : false,
                    isSorted = args.element.find("~ span:eq(0) span.descending").length > 0 ? true : false;
                for (var i = 0; i < this.model.pivotTableFields.length; i++) {
                    if (this.model.pivotTableFields[i].name == headerText) {
                        this.model.pivotTableFields[i].isSelected = true;
                        headerTag = this.model.pivotTableFields[i];
                    }
                }
            }
            this.element.find(".dragClone").remove();
            if (droppedClass == "schemaValue" || droppedClass == "schemaRow" || droppedClass == "schemaColumn" || droppedClass == "schemaFilter") {
                var droppedPosition = this._setSplitBtnTargetPos(args.event), params, report, eventArgs;
                try {
                    report = JSON.parse(this.model.pivotControl.getOlapReport()).Report;
                }
                catch (err) {
                    report = this.model.pivotControl.getOlapReport();
                }
                $(args.element.parent()).remove();
                if (!ej.isNullOrUndefined(ogridWaitingPopup))
                    ogridWaitingPopup.show();
                if (this._dataModel == "Olap") {
                    params = this._currentCubeName + "--" + headerTag + "--" + axisName + "--" + droppedPosition;
                    eventArgs = JSON.stringify({ "action": "nodeDropped", "dropType": "SplitButton", "nodeInfo": params, "currentReport": report });
                    this.doAjaxPost("POST", this.model.pivotControl.model.url + "/" + this.model.serviceMethods.nodeDropped, eventArgs, this._onDroppedSuccess);
                }
                else {
                    this._createPivotButton(headerText, droppedClass == "schemaRow" ? "row" : droppedClass == "schemaColumn" ? "column" : droppedClass == "schemaFilter" ? "filter" : droppedClass == "schemaValue" ? "value" : "", isFiltered, isSorted, droppedPosition);
                    eventArgs = JSON.stringify({ "action": "nodeDropped", "dropAxis": droppedClass + "::" + droppedPosition, "headerTag": JSON.stringify(headerTag), "sortedHeaders": this._getSortedHeaders(), "currentReport": report });
                    successMethod = this.model.pivotControl._renderControlSuccess;
                    this.model.pivotControl.doAjaxPost("POST", this.model.pivotControl.model.url + "/" + this.model.serviceMethods.nodeDropped, eventArgs, this.model.pivotControl._renderControlSuccess);
                }
            }
            else
                this._removePvtBtn(args);
        },

        _dialogBtnClick: function (args) {
            this.element.find(".e-dialog, .clientDialog").hide();
            var unselectedNodes = "", enableFilterIndigator;
            if (args.model.text.toLowerCase() != "cancel") {
                var uncheckedNodes = this._dataModel == "Olap" ? unselectedNodes = this._getUnSelectedNodes() + "FILTERED" + this._getSelectedNodes(this._curFilteredAxis == "Filter" ? true : false) : this._memberTreeObj.element.find(":input:gt(0).nodecheckbox:not(:checked)"), isFiltered = false,
                filterParams = this._dataModel == "Olap" ? uncheckedNodes : this._curFilteredAxis + "::" + this._curFilteredText + "::FILTERED";
                var text = this._curFilteredText, textArr = new Array(), obj = {};
                enableFilterIndigator = this._getUnSelectedNodes() != "" ? true : false;
                if (this._dataModel == "Pivot") {
                    for (var i = 0; i < uncheckedNodes.length; i++)
                        textArr.push($(uncheckedNodes[i].parentElement).siblings("a").text());
                    obj[text] = textArr;
                    if (ej.isNullOrUndefined(this._tempFilterData))
                        this._tempFilterData = new Array();
                    for (var i = 0; i < this._tempFilterData.length; i++) {
                        if (!ej.isNullOrUndefined(this._tempFilterData[i][text])) {
                            this._tempFilterData[i][text] = textArr;
                            isFiltered = true;
                        }
                    }
                    if (!isFiltered)
                        this._tempFilterData.push(obj);
                    if (ej.isNullOrUndefined(this._curFilteredAxis) || this._curFilteredAxis != "") {
                        for (var i = 0; i < uncheckedNodes.length; i++)
                            filterParams += "##" + $(uncheckedNodes[i].parentElement).siblings("a").text();
                    }
                }
                else {
                    obj[this._curFilteredText] = filterParams;
                    if (ej.isNullOrUndefined(this._tempFilterData))
                        this._tempFilterData = new Array();
                    for (var i = 0; i < this._tempFilterData.length; i++) {
                        if (!ej.isNullOrUndefined(this._tempFilterData[i][this._curFilteredText])) {
                            this._tempFilterData[i][this._curFilteredText] = textArr;
                            isFiltered = true;
                        }
                    }
                    if (!isFiltered)
                        this._tempFilterData.push(obj);
                }
                var sortedHeaders = this._getSortedHeaders(),
                filteredBtn = this.model.layout == "excel" ? this.element.find(".schemaFieldTree li:contains('" + this._curFilteredText + "')") : this.element.find("." + this._curFilteredAxis + " .pivotButton:contains(" + this._curFilteredText + ") .filterBtn");
                if (this.model.layout == "excel") {
                    if ($(_pivotProxy._selectedTreeNode).parents("li:eq(0)").children().children("span").hasClass("hierarchyCDB")) {
                        _pivotProxy._selectedTreeNode = $($(_pivotProxy._selectedTreeNode).parents("li:eq(0)"));
                    }
                    if (enableFilterIndigator)
                        $(_pivotProxy._selectedTreeNode).addClass("e-icon filter");
                    else if (!enableFilterIndigator)
                        $(_pivotProxy._selectedTreeNode).removeClass("filter");
                    if (this._curFilteredAxis == "")
                        return false;
                }
                else {
                    if (uncheckedNodes.length > 0 && !ej.isNullOrUndefined(filteredBtn))
                        $(filteredBtn).addClass("filterIndicator");
                    else if (uncheckedNodes.length == 0 && !ej.isNullOrUndefined(filteredBtn))
                        $(filteredBtn).removeClass("filterIndicator");
                }
                var report;
                try {
                    report = JSON.parse(this.model.pivotControl.getOlapReport()).Report;
                }
                catch (err) {
                    report = this.model.pivotControl.getOlapReport();
                }
                var serializedCustomObject = JSON.stringify(this.model.pivotControl.model.customObject),
                eventArgs = this.model.pivotControl.model.customObject != {} && filterParams != "" ? JSON.stringify({ "action": "filtering", "filterParams": filterParams, "sortedHeaders": sortedHeaders, "currentReport": report, "customObject": serializedCustomObject }) :
            this.model.pivotControl.model.customObject != {} ? JSON.stringify({ "action": "filtering", "currentReport": report, "customObject": serializedCustomObject }) :
            filterParams != "" ? JSON.stringify({ "action": "filtering", "filterParams": filterParams, "sortedHeaders": sortedHeaders, "currentReport": report }) : JSON.stringify({ "action": "filtering" });
                if (!ej.isNullOrUndefined(ogridWaitingPopup))
                    ogridWaitingPopup.show();
                this.model.pivotControl.doAjaxPost("POST", this.model.pivotControl.model.url + "/" + this.model.serviceMethods.filtering, eventArgs, this.model.pivotControl._renderControlSuccess);
            }
            _pivotProxy._selectedTreeNode = null;
        },

        _onNodeExpand: function (args) {
            var selectedItem = (args.event.originalEvent != undefined && args.event.originalEvent.target != undefined) ? args.event.originalEvent.target : args.event.target; var tagInfo, dimenName ="";
            var childCount = $(selectedItem).parents("li").first().children().find("li").length;
            if (selectedItem.className.indexOf("e-plus") > -1 && childCount == 0) {
                var checkedState = $(selectedItem).parent().find("input[type=checkbox]").prop("checked"), report;
                var tagInfo = $(selectedItem).parents("li").first().attr("tag");
                this.pNode = selectedItem;
                try {
                    report = JSON.parse(this.model.pivotControl.getOlapReport()).Report;
                }
                catch (err) {
                    report = this.model.pivotControl.getOlapReport();
                }
                var pivotButtons = this.element.find(".pivotButton"), parentTag;
                for (var bt = 0; bt < pivotButtons.length > 0; bt++) {
                    if ($(pivotButtons[bt]).attr("tag").indexOf(this._curFilteredText)>-1)
                        dimenName = $(pivotButtons[bt]).attr("tag");
                }
                //eventArgs = JSON.stringify({ "action": "fetchMembers", "headerTag": this._dialogTitle || "UniqueName##" + $($(args.target).parents("li")[0]).attr("tag"), "sortedHeaders": sortedHeaders, "currentReport": report });
                this.doAjaxPost("POST", this.model.pivotControl.model.url + "/" + this.model.serviceMethods.memberExpand, JSON.stringify({
                    "action": "memberExpanded", "checkedStatus": checkedState, "parentNode": $(selectedItem).parents("li")[0].id, "tag": tagInfo, "cubeName": this._currentCubeName + "##" + dimenName, "currentReport": report
                }), this._fetchChildNodeSuccess);
            }
        },

        _fetchChildNodeSuccess: function (data) {
            var newDataSource; var parentNode;
            if (data.length > 1 && data[0] != undefined)
                newDataSource = JSON.parse(data[0].Value);
            else if (data.d != undefined)
                newDataSource = JSON.parse(data.d[0].Value);
            else
                newDataSource = JSON.parse(data.ChildNodes);

            var mapper = { id: "id", parentId: "pid", hasChild: "hasChildren", text: "name", isChecked: "checkedStatus" };
            if ($(this.pNode).parents("li").length > 1)
                parentNode = $(this.pNode).parents("li").first();
            else
                parentNode = $(this.pNode).parents("li");
            $($(parentNode).find('input.nodecheckbox')[0]).ejCheckBox({ checked: false })
            this._memberTreeObj._createSubNodesWhenLoadOnDemand(newDataSource, this.pNode, mapper);
            $.each($(parentNode).children().find("li"), function (index, value) {
                value.setAttribute("tag", newDataSource[index].tag);
            });
           // this._memberTreeObj._expandCollapseAction(this.pNode);
        },

        _fetchMemberSuccess: function (msg) {
            if (msg[0] != undefined && msg.length > 0) {
                this._currentMembers = msg[0].Value;
            }
            else if (msg.d != undefined && msg.d.length > 0) {
                this._currentMembers = msg.d[0].Value;
            }
            else if (msg != undefined && msg.length > 0) {
                this._currentMembers = msg.MemberList;
            }
            this._createDialog(this._dialogTitle, this._currentMembers);
        },

        _onDroppedSuccess: function (args) {
            var report = null;
            if (!ej.isNullOrUndefined(args[0]) && args.length > 0)
                report = JSON.parse(args[1].Value);
            else if (!ej.isNullOrUndefined(args.d) && args.d.length > 0)
                report = JSON.parse(args.d[1].Value);
            else if (!ej.isNullOrUndefined(args) && args.length > 0)
                report = JSON.parse(args.OlapReport);
            this.element.find(".axisTable .pivotButton").remove();
            this._setPivotRows(report.PivotRows);
            this._setPivotColumns(report.PivotColumns);
            this._setPivotCalculations(report.PivotCalculations);
            this._setFilters(report.Filters);
            this.element.find(".axisTable .schemaFilter").append(this._createPivotButtons(this.model.filters, "filter"));
            this.element.find(".axisTable .schemaRow").append(this._createPivotButtons(this.model.pivotRows, "row"));
            this.element.find(".axisTable .schemaColumn").append(this._createPivotButtons(this.model.pivotColumns, "column"));
            this.element.find(".axisTable .schemaValue").append(this._createPivotButtons(this.model.pivotCalculations, "values"));
            this.element.find(".pivotButton .pvtBtn").ejButton({ size: "normal" }).ejDraggable({
                handle: 'button', clone: true,
                cursorAt: { left: -5, top: -5 },
                dragStart: ej.proxy(function (args) {
                    this._isDragging = true;
                }, this),
                dragStop: ej.proxy(this._onPvtBtnDropped, this),
                helper: ej.proxy(function (event, ui) {
                    if (event.sender.target.className.indexOf("e-btn") > -1)
                        return $(event.sender.target).clone().addClass("dragClone").appendTo(this.element);
                    else
                        return false;
                }, this)
            });
            this._unWireEvents();
            this._wireEvents();
            this.model.pivotControl._renderControlSuccess(args);
        },

        /**
        * 	This function is used to create the toolbar, member editor and measure editor dialogs
        *
        * @private
        */
        _removePvtBtn: function (args) {
            var headerText = args.element.text(), headerTag = args.element.parent().attr("tag"), report, eventArgs;
            try {
                report = JSON.parse(this.model.pivotControl.getOlapReport()).Report;
            }
            catch (err) {
                report = this.model.pivotControl.getOlapReport();
            }
            if (this._dataModel == "Olap") {
                var uniqueName = ((headerTag.indexOf("[Measures]") > -1) || (headerTag.indexOf("[") > -1)) ? headerTag.split(":")[1] : this._getNodeUniqueName(headerTag);
                selectedTreeNode = this._getNodeByUniqueName(uniqueName);               
                this._tableTreeObj.unCheckNode(selectedTreeNode);               
            }
            else {
                selectedTreeNode = this._tableTreeObj.element.find("li:contains('" + headerText + "')");
                this._tableTreeObj.unCheckNode(selectedTreeNode);
            }
        },

        _createDialog: function (title, treeViewData) {
            this.element.find(".e-dialog, .clientDialog").remove();
            var dialogContent = ej.buildTag("div#EditorDiv.editorDiv", ej.buildTag("div", ej.buildTag("div.memberEditorDiv", ej.buildTag("div#editorTreeView.editorTreeView")[0].outerHTML)[0].outerHTML)[0].outerHTML)[0].outerHTML + "</br>",
            dialogFooter = ej.buildTag("div", ej.buildTag("button#OKBtn.dialogOKBtn", "OK")[0].outerHTML + ej.buildTag("button#CancelBtn.dialogCancelBtn", "Cancel")[0].outerHTML, { "float": "right", "margin": "-5px 0 6px" })[0].outerHTML,
            ejDialog = ej.buildTag("div#clientDialog.clientDialog", dialogContent + dialogFooter, { "opacity": "1" }).attr("title", title)[0].outerHTML,
            treeViewData = JSON.parse(treeViewData);
            var selectedData;
            if (!ej.isNullOrUndefined(this._tempFilterData)) {
                for (var i = 0; i < this._tempFilterData.length; i++) {
                    if (!ej.isNullOrUndefined(this._tempFilterData[i][title]))
                        selectedData = this._tempFilterData[i][title];
                }
                if (!ej.isNullOrUndefined(selectedData)) {
                    for (var i = 0; i < selectedData.length; i++) {
                        for (var j = 0; j < treeViewData.length; j++) {
                            if (selectedData[i] == treeViewData[j].name)
                                treeViewData[j].checkedStatus = false;
                        }
                    }
                }
            }
            $(ejDialog).appendTo("#" + this._id);
            this.element.find(".editorTreeView").ejTreeView({
                showCheckbox: true,
                loadOnDemand: true,
                height: $(".memberEditorDiv").height(),
                fields: { id: "id", text: "name", isChecked: "checkedStatus", hasChild: "hasChildren", dataSource: treeViewData },
                nodeClick: ej.proxy(this._onNodeExpand, this)
            });
            var treeViewElements = this.element.find(".editorTreeView").find("li");
            for (var i = 0; i < treeViewElements.length; i++) {
                treeViewElements[i].setAttribute("tag", treeViewData[i].tag);
            }
            this.element.find(".dialogOKBtn, .dialogCancelBtn").ejButton({ click: ej.proxy(this._dialogBtnClick, this) });
            this._dialogOKBtnObj = this.element.find(".dialogOKBtn").data("ejButton");
            this._memberTreeObj = this.element.find(".editorTreeView").data("ejTreeView");
            var treeViewLi = this.element.find(".editorTreeView li:gt(0)"),
            uncheckedNodes = $(treeViewLi).find(":input.nodecheckbox:not(:checked)");
            if (uncheckedNodes.length > 0) {
                var firstNode = $(this._memberTreeObj.element.find("li:first")).find("div .e-chkbox-small > span > span:eq(0)");
                $(firstNode).removeClass("e-chk-activeicon").addClass("e-chk-indetericon");
            }
            this.element.find(".clientDialog").ejDialog({ width: 265, content: "#" + this._id, enableResize: false });
            this._memberTreeObj.model.nodeCheck = ej.proxy(this._onNodeCheckChanges, this);
            this._memberTreeObj.model.nodeUncheck = ej.proxy(this._onNodeCheckChanges, this);
			if(this._memberTreeObj.element.find(".e-plus").length == 0){
             this._memberTreeObj.element.find(".e-item").css("padding","0px");
            } 
            this._unWireEvents();
            this._wireEvents();
            if (!ej.isNullOrUndefined(ogridWaitingPopup))
                ogridWaitingPopup.hide();
        },
        _createPivotButton: function (text, axis, isFiltered, isSorted, dropPostion) {
            var pvtAxis = axis == "row" ? ".schemaRow" : axis == "column" ? ".schemaColumn" : axis == "value" ? ".schemaValue" : ".schemaFilter";
            var sortBtn = (axis == "column" || axis == "row") && this.model.layout != "excel" ? ej.buildTag("input#toggleBtn" + text, {}).attr("type", "checkbox")[0].outerHTML : "",
                    filterBtn = (axis == "column" || axis == "row" || axis == "filter") && this.model.layout != "excel" ? ej.buildTag("button.filterBtn#filterBtn" + text, {}).addClass(isFiltered ? "filterIndicator" : "")[0].outerHTML : "",

            rowBtn = ej.buildTag("div.pivotButton", ej.buildTag("div.dropIndicator")[0].outerHTML + ej.buildTag("button.pvtBtn#pivotButton" + text, text)[0].outerHTML +
                    sortBtn + filterBtn + ej.buildTag("span.removePivotBtn", "", { "display": "none" }).addClass("e-icon")[0].outerHTML)[0].outerHTML;
            if (typeof (dropPostion) == "number" && $(this.element.find(pvtAxis + " .pivotButton")[dropPostion]).length > 0)
                axis == "row" ? $(this.element.find(".schemaRow .pivotButton")[dropPostion]).before(rowBtn) : axis == "column" ? $(this.element.find(".schemaColumn .pivotButton")[dropPostion]).before(rowBtn) :
            axis == "value" ? $(this.element.find(".schemaValue .pivotButton")[dropPostion]).before(rowBtn) : axis == "filter" ? $(this.element.find(".schemaFilter .pivotButton")[dropPostion]).before(rowBtn) : "";
            else
                axis == "row" ? this.element.find(".schemaRow").append(rowBtn) : axis == "column" ? this.element.find(".schemaColumn").append(rowBtn) :
            axis == "value" ? this.element.find(".schemaValue").append(rowBtn) : axis == "filter" ? this.element.find(".schemaFilter").append(rowBtn) : "";
            var pvtExp = (typeof (dropPostion) != "number" && dropPostion == "") ? " .pivotButton .pvtBtn:last" : " .pivotButton .pvtBtn:eq(" + dropPostion + ")";
            this.element.find(pvtAxis + pvtExp).ejButton({ size: "normal" }).ejDraggable({
                handle: 'button', clone: true,
                cursorAt: { left: -5, top: -5 },
                dragStart: ej.proxy(function (args) {
                    this._isDragging = true;
                }, this),
                dragStop: ej.proxy(this._onPvtBtnDropped, this),
                helper: ej.proxy(function (event, ui) {
                    if (event.sender.target.className.indexOf("e-btn") > -1)
                        return $(event.sender.target).clone().addClass("dragClone").appendTo(this.element);
                    else
                        return false;
                }, this)
            });
            this.element.find(pvtAxis + " .pivotButton .filterBtn:last").ejButton({
                size: "normal",
                contentType: "imageonly",
                prefixIcon: "filter"
            });
            this.element.find(pvtAxis + " .pivotButton input:last").ejToggleButton({
                size: "normal",
                contentType: "imageonly",
                defaultPrefixIcon: "ascending",
                activePrefixIcon: "descending",
                click: ej.proxy(this._onSortBtnClick, this)
            });
            if (isSorted) {
                this.element.find(pvtAxis + " .pivotButton:contains(" + text + ") span:eq(0) span").removeClass("ascending").addClass("descending");
                this.element.find(pvtAxis + " .pivotButton:contains(" + text + ") span:eq(0) button").addClass("e-active");
            }
            this.element.find(".dropIndicator").removeClass("dropIndicatorHover");
        },

        _createPivotButtons: function (rows, axis) {
            if (!ej.isNullOrUndefined(rows)) {
                var rowBtns = "", tagAxis = axis == "column" ? "Columns" : axis == "row" ? "Rows" : axis == "filter" ? "Slicers" : "";
                if (tagAxis == "" && axis == "values") {
                    var tempAxis = this.element.find(".pivotButton:contains('Measures')").length > 0 ? this.element.find(".pivotButton:contains('Measures')").parent()[0].className.split(" ")[0] : "schemaColumn";
                    tagAxis = tempAxis == "schemaColumn" ? "Columns" : "Rows";
                }
                for (var i = 0; i < rows.length; i++) {
                    var sortBtn = (axis == "column" || axis == "row") && this.model.layout != "excel" ? ej.buildTag("input#toggleBtn" + rows[i].FieldHeader || rows[i].FieldName, {}).attr("type", "checkbox")[0].outerHTML : "",
                    filterBtn = (axis == "column" || axis == "row" || axis == "filter") && this.model.layout != "excel" ? ej.buildTag("button.filterBtn#filterBtn" + (rows[i].FieldHeader || rows[i].FieldName || rows[i].Name || rows[i].DimensionName), {})[0].outerHTML : "";
                    rowBtns += ej.buildTag("div.pivotButton", ej.buildTag("div.dropIndicator")[0].outerHTML + ej.buildTag("button.pvtBtn#pivotButton" + rows[i].FieldHeader || rows[i].FieldName || rows[i].Name || rows[i].DimensionName, rows[i].FieldHeader || rows[i].FieldName || rows[i].Name || rows[i].DimensionName)[0].outerHTML +
                    sortBtn + filterBtn + ej.buildTag("span.removePivotBtn", "", { "display": "none" }).addClass("e-icon")[0].outerHTML).attr("tag", tagAxis + ":" + rows[i].Tag)[0].outerHTML;
                }
                return rowBtns;
            }
        },

        _setSplitBtnTargetPos: function (event) {
            var targetPosition = ""; var AEBdiv; var targetSplitBtn; var className;
            if (event.event != undefined && event.event.type == "touchend") {
                targetSplitBtn = event.event.originalEvent.target != null ? $(event.event.originalEvent.target).parents(".pivotButton") : $(event.event.originalEvent.srcElement).parents(".pivotButton");
                className = event.event.originalEvent.target != null ? event.event.originalEvent.target.className : event.event.originalEvent.srcElement.className;
            }
            else {
                targetSplitBtn = event.originalEvent.target != null ? $(event.originalEvent.target).parents(".pivotButton") : $(event.originalEvent.srcElement).parents(".pivotButton");
                className = event.originalEvent.target != null ? event.originalEvent.target.className : event.originalEvent.srcElement.className;
            }
            this._droppedClass = targetSplitBtn.length > 0 ? $(targetSplitBtn[0]).parent()[0].className.split(" ")[0] : className.split(" ")[0];
            if (targetSplitBtn[0] || (className != undefined && className != null && jQuery.type(className) == "string" && className.indexOf("pivotButton") > -1)) {
                targetSplitBtn = targetSplitBtn[0] ? targetSplitBtn[0] : event.originalEvent.target != null ? event.originalEvent.target : event.originalEvent.srcElement;
                if (event.event != undefined && event.event.type == "touchend")
                    AEBdiv = event.target;
                else if (this._droppedClass != null && this._droppedClass != undefined && this._droppedClass != "")
                    AEBdiv = this.element.find("." + this._droppedClass)[0];
                else
                    AEBdiv = targetSplitBtn.parentNode;
                for (var i = 0; i < AEBdiv.children.length; i++) {
                    if ($(AEBdiv.childNodes[i]).attr("tag") == $(targetSplitBtn).attr("tag"))
                        targetPosition = i;
                }
            }
            return targetPosition;
        },

        _getUnSelectedNodes: function () {
            var treeElement = this.element.find(".editorTreeView")[0];
            var unselectedNodes = "";
            var data = $(treeElement).find(":input.nodecheckbox:not(:checked)");
            for (var i = 0; i < data.length; i++) {
                if (!($(data[i].parentElement).find('span:nth-child(1)').attr('class').indexOf("e-chk-act") > -1) && $(data[i].parentElement).attr('aria-checked') != 'mixed') {
                    var parentNode = $(data[i]).parents('li:eq(0)');
                    unselectedNodes += "::" + parentNode[0].id + "||" + $(parentNode).attr('tag');
                }
            }
            return unselectedNodes;
        },

        _getSelectedNodes: function (isSlicer) {
            if (isSlicer) {
                var treeElement = this.element.find(".editorTreeView")[0].childNodes[0];
                var selectedNodes = new Array();
                var data = $(treeElement).children();
                for (var i = 0; i < data.length; i++) {
                    var parentNode = data[i];
                    var nodeInfo = { caption: $(parentNode.firstChild).find("a").text(), parentId: parentNode.parentElement.parentElement.className.indexOf("editorTreeView") > -1 ? "None" : $(parentNode).parents()[1].id, id: parentNode.id, checked: $(parentNode).find(':input.nodecheckbox')[0].checked, expanded: $(parentNode.firstChild).find(".e-minus").length > 0 ? true : false, childNodes: new Array(), tag: $(parentNode).attr('tag') };
                    if ($(parentNode).find("ul:first").children().length > 0) {
                        var liElements = $(parentNode).find("ul:first").children();
                        for (var j = 0; j < liElements.length; j++) {
                            nodeInfo.childNodes.push(this._getNodeInfo(liElements[j]));
                        }
                    }
                    selectedNodes.push(nodeInfo);
                }
                return JSON.stringify(selectedNodes);
            }
            else {
                var treeElement = $(".editorTreeView");
                var selectedNodes = "";
                var data = $(treeElement).find(':input.nodecheckbox');
                for (var i = 0; i < data.length; i++) {
                    if (data[i].checked || $(data[i].parentElement).attr('aria-checked') == 'mixed') {
                        var parentNode = $(data[i]).parents('li:eq(0)');
                        selectedNodes += "::" + parentNode[0].id + "||" + $(parentNode).attr('tag');
                    }
                }
                return selectedNodes;
            }
        },
        _getNodeUniqueName: function (headerTag) {
            var uniqueName = "", selectedTreeNode = null;
            for (var i = 0; i < (headerTag.split(':')[1]).split('.').length; i++) {
                if (uniqueName != "")
                    uniqueName += ".";
                uniqueName += "[" + ((headerTag.split(':')[1]).split('.')[i]) + "]";
            }
            return uniqueName;
        },
        _getNodeByUniqueName: function (nodeUniqueName) {
            for (var i = 0; i < $(_pivotProxy._tableTreeObj.element.find("li")).length; i++) {
                if ($(_pivotProxy._tableTreeObj.element.find("li")[i]).attr("tag") == nodeUniqueName)
                    selectedTreeNode = $((_pivotProxy._tableTreeObj.element.find("li"))[i]);
            }
           
            return selectedTreeNode;
        },

        /**
        * Perform an asynchronous HTTP (Ajax) request.
        * @return jQuery
        * @example 
        * &lt;div id="OlapGrid"&gt;&lt;/div&gt; <br> 
        * &lt;script&gt;
        * // Create OLAP Grid
        * $('#OlapGrid').ejOlapGrid({
        *       url: "OlapGridService.svc",
        *   });
        * var gridObj = $("#OlapGrid").data("ejOlapGrid");
        * gridObj.doAjaxPost("POST", "/OlapGridService.svc/Initialize", {"key", "Hello World"}, "_renderControlSuccess", null);
        * // Initiate an Ajax request.
        * &lt;/script&gt;
        * @memberof ejOlapGrid
        * @instance
        */
        doAjaxPost: function (type, url, data, onSuccess, onComplete) {
            $.ajax({
                type: type,
                url: url,
                contentType: 'application/json; charset=utf-8',
                dataType: 'json',
                data: data,
                success: $.proxy(onSuccess, this),
                complete: $.proxy(onComplete, this),
                error: function (msg, textStatus, errorThrown) {
                    if (_proxy.model.progressMode != "infinite") {
                        window.clearInterval(oGridTimer);
                        if (_proxy.model.progressMode == "progress")
                            $("#" + _proxy._id + "progressText")[0].innerHTML = _proxy._getLocalizedLabels("MDXqueryExecutionFailed") + "!!!";
                        $("#" + ogridProgressBar._id).show();
                        ogridProgressBar.setModel({ value: 0, text: errorThrown });
                    }
                    else
                        ogridWaitingPopup.hide();
                }
            });
        }
    });

    ej.PivotSchemaDesigner.localization = {};
    ej.PivotSchemaDesigner.localization["en-US"] = {
        SeriesPage: "Series Page",
        CategoricalPage: "Categorical Page",
        MDXqueryExecutionFailed: "MDX query execution failed",
        PreparingAndExecutingMDXquery: "Preparing and executing MDX query",
        MDXqueryExecutedSuccessfully: "MDX query executed successfully",
        RenderingStarted: "Rendering Started",
        RenderingSucceeded: "Rendering Succeeded",
        RenderingFailed: "Rendering failed"
    };

    /**
    * Enum for Progress Mode. Allows the user to select a appropriate progress indicator. 
    *
    * @enum {String}
    * @global
    */
    ej.PivotSchemaDesigner.Layouts = {
        Excel: "excel",
        Normal: "normal"
    };

})(jQuery, Syncfusion); ;
/**
* @fileOverview Plugin to experience Html OLAP Client component elements
* @copyright Copyright Syncfusion Inc. 2001 - 2013. All rights reserved.
*  Use of this code is subject to the terms of our license.
*  A copy of the current license can be obtained at any time by e-mailing
*  licensing@syncfusion.com. Any infringement will be prosecuted under
*  applicable laws. 
* @version 12.1 
* @author <a href="mailto:licensing@syncfusion.com">Syncfusion Inc</a>
*/

(function ($, ej, undefined) {
    /**
    * @namespace ej
	* @class ejOlapClient
	* @requires jquery-1.10.2.min.js
	* @requires jquery.easing.1.3.min.js
    * @requires jquery.globalize.min.js
    * @requires ej.core.js
    * @requires ej.data.js
    * @requires ej.touch.js
    * @requires ej.button.js
    * @requires ej.togglebutton.js
    * @requires ej.checkbox.js
    * @requires ej.radiobutton.js
    * @requires ej.dropdownlist.js
    * @requires ej.dialog.js
    * @requires ej.draggable.js
    * @requires ej.toolbar.js
    * @requires ej.scroller.js
    * @requires ej.maskedit.js
    * @requires ej.waitingpopup.js
    * @requires ej.treeview.js
    * @requires ej.tab.js
    * @requires ej.chart.js
    * @requires ej.olapchart.js
    * @requires ej.olapgrid.js
    * @requires ej.olapclient.js
	* @classdesc Custom Design for Html OLAP Client control.
	* @example 
    * &lt;div id="OlapClient"&gt;&lt;/div&gt; <br> 
	* &lt;script&gt;
	* // Create OLAP Client
    * $("#OlapClient").ejOlapClient(...); 	
	* &lt;/script&gt;
	*/
    ej.widget("ejOlapClient", "ej.olap.OlapClient", {

        //Root Css Class
        _rootCSS: "e-olapclient",

		//Widget element will be automatically set in this
        element: null,

        //User-defined model will be automatically set in this
        model: null,

        validTags: ["div", "span"],

        //Default model values
        defaults: {
            /**		
			* Connects the service using the specified URL for any server updates.
			* @default null
			* @type {string}
			* @example 
			* //To set url value during initialization
            * $("#OlapClient").ejOlapClient({ url: "/wcf/OlapClientService.svc" });
			* @example 
			* //Get or set the url, after initialization:<br>
			*	//Gets the url value  
			*	$("#OlapClient").ejOlapClient("option","url");<br>			
			*	//Sets the url value 
			*	$("#OlapClient").ejOlapClient("option","url", "/wcf/OlapClientService.svc" ); 
			 * @memberof ejOlapClient
			* @instance
			*/
            url: "",

            /**		
			* Sets the CSS name for custom operation.
			* @default null
			* @type {string}
			* @example 
			* //To set cssClass API value during initialization
            * $("#OlapClient").ejOlapClient({ cssClass: "Olive" });
			* @example 
			* //Get or set the cssClass API, after initialization:<br>
			*	//Gets the cssClass value  
			*	$("#OlapClient").ejOlapClient("option","cssClass");<br>			
			*	//Sets the cssClass value 
			*	$("#OlapClient").ejOlapClient("option","cssClass", "Olive" ); 
			* @memberof ejOlapClient
			* @instance
			*/
            cssClass: "",

            /**		
			* Sets the title for OLAP Client.
			* @default null
			* @type {string}
			* @example 
			* //To set title API value during initialization
            * $("#OlapClient").ejOlapClient({ title: "Olap Browser" });
			* @example 
			* //Get or set the title API, after initialization:<br>
			*	//Gets the title value  
			*	$("#OlapClient").ejOlapClient("option","title");<br>			
			*	//Sets the title value 
			*	$("#OlapClient").ejOlapClient("option","title", "Olap Browser" ); 
			* @memberof ejOlapClient
			* @instance
			*/
            title: "",

            /**		
			* Sets the layout for OLAP Grid component inside OLAP Client.
			* @default ej.olap.OlapGrid.Layout.Normal
			* @type {enum}
			* @example 
			* //To set gridLayout API value during initialization
            * $("#OlapClient").ejOlapClient({ gridLayout: ej.olap.OlapGrid.Layout.NoSummaries });
			* @example 
			* //Get or set the gridLayout API, after initialization:<br>
			*	//Gets the gridLayout value  
			*	$("#OlapClient").ejOlapClient("option","gridLayout");<br>			
			*	//Sets the gridLayout value 
			*	$("#OlapClient").ejOlapClient("option","gridLayout", ej.olap.OlapGrid.Layout.NormalTopSummary); 
			* @memberof ejOlapClient
			* @instance
			*/
            gridLayout: "Normal",

            /**		
			* Sets the type for OLAP Chart component inside OLAP Client.
			* @default ej.olap.OlapChart.ChartTypes.Column
			* @type {enum}
			* @example 
			* //To set chartType API value during initialization
            * $("#OlapClient").ejOlapClient({ chartType: ej.olap.OlapChart.ChartTypes.Spline });
			* @example 
			* //Get or set the chartType API, after initialization:<br>
			*	//Gets the chartType value  
			*	$("#OlapClient").ejOlapClient("option","chartType");<br>			
			*	//Sets the chartType value 
			*	$("#OlapClient").ejOlapClient("option","chartType", ej.olap.OlapChart.ChartTypes.Area); 
			* @memberof ejOlapClient
			* @instance
			*/
            chartType: "column",
          
			
			/**		
			* Allows the user to customize the control layout and appearance.
            * @default null
            * @type {object}
            * @example 
            * //To set displaySettings API value, in-order to customize the control layout and appearance.
            *  $("#OlapClient").ejOlapClient({  displaySettings: {mode: "chartandgrid"}});
            * @example 
            * //Gets or sets the displaySettings API, to customize the control layout and appearance:<br>
            * //Gets the displaySettings value  
            * $("#OlapClient").ejOlapClient("option", "displaySettings");<br>   
            * //Sets the displaySettings value 
            * $("#OlapClient").ejOlapClient("option", "displaySettings", {mode: "chartandgrid"} ); 
            * @memberof ejOlapClient
            * @instance
			*/
            displaySettings: {
                /**		
			    * Sets the display mode (Only Chart/Only Grid/Both Chart And Grid) of OLAP Client.
			    * @default ej.olap.OlapClient.DisplayMode.ChartAndGrid
			    * @type {enum}
			    * @example 
			    * //To set mode API value during initialization
                * $("#OlapClient").ejOlapClient({ displaySettings: { mode: ej.olap.OlapClient.DisplayMode.ChartOnly }});
			    * @example 
			    * //Get or set the mode API, after initialization:<br>
			    *	//Gets the mode value  
			    *	$("#OlapClient").ejOlapClient("option","displaySettings.mode");<br>			
			    *	//Sets the mode value 
			    *	$("#OlapClient").ejOlapClient("option","displaySettings.mode", ej.olap.OlapClient.DisplayMode.ChartOnly ); 
			    * @alias ejOlapClient#displaySettings->mode
			    * @instance
			    */
                mode: "chartandgrid",

                /**		
			    * Lets the user to set Chart or Grid tab as default.
			    * @default ej.olap.OlapClient.DefaultView.Grid
			    * @type {enum}
			    * @example 
			    * //To set defaultView API value during initialization
                * $("#OlapClient").ejOlapClient({ displaySettings: {defaultView: ej.olap.OlapClient.DefaultView.Grid} });
			    * @example 
			    * //Get or set the defaultView API, after initialization:<br>
			    *	//Gets the defaultView value  
			    *	$("#OlapClient").ejOlapClient("option","displaySettings.defaultView");<br>			
			    *	//Sets the defaultView value 
			    *	$("#OlapClient").ejOlapClient("option","displaySettings.defaultView", ej.olap.OlapClient.DefaultView.Chart ); 
			    * @alias ejOlapClient#displaySettings->defaultView
			    * @instance
			    */
                defaultView: "grid",

                /**		
			    * Lets the user to customize the display of OLAP Chart and OLAP Grid controls inside the OLAP Client component.
			    * @default ej.olap.OlapClient.ControlPlacement.Tab
			    * @type {enum}
			    * @example 
			    * //To set controlPlacement API value during initialization
                * $("#OlapClient").ejOlapClient({ displaySettings: {controlPlacement: ej.olap.OlapClient.ControlPlacement.Tab} });
			    * @example 
			    * //Get or set the controlPlacement API, after initialization:<br>
			    *	//Gets the controlPlacement value  
			    *	$("#OlapClient").ejOlapClient("option","displaySettings.controlPlacement");<br>			
			    *	//Sets the controlPlacement value 
			    *	$("#OlapClient").ejOlapClient("option","displaySettings.controlPlacement", ej.olap.OlapClient.ControlPlacement.Tile ); 
			    * @alias ejOlapClient#displaySettings->controlPlacement
			    * @instance
			    */
                controlPlacement: "tab",

                /**		
			    * Sets the Toggle Panel visibility mode.
			    * @default false
			    * @type {boolean}
			    * @example 
			    * //To set enableTogglePanel API value during initialization
                * $("#OlapClient").ejOlapClient({ displaySettings.enableTogglePanel: true });
			    * @example 
			    * //Get or set the enableTogglePanel API, after initialization:<br>
			    *	//Gets the enableTogglePanel value  
			    *	$("#OlapClient").ejOlapClient("option","displaySettings.enableTogglePanel");<br>			
			    *	//Sets the enableTogglePanel value 
			    *	$("#OlapClient").ejOlapClient("option","displaySettings.enableTogglePanel", true ); 
			    * @alias ejOlapClient#displaySettings->enableTogglePanel
			    * @instance
			    */
                enableTogglePanel: false,
                
                /**		
			    * Enables/ disables the full screen view of the OLAP components(OLAP Chart, OLAP Grid) in OLAP Client.
			    * @default false
			    * @type {boolean}
			    * @example 
			    * //To set enableFullScreen API value during initialization
                * $("#OlapClient").ejOlapClient({ displaySettings.enableFullScreen: true });
			    * @example 
			    * //Get or set the enableFullScreen API, after initialization:<br>
			    *	//Gets the enableFullScreen value  
			    *	$("#OlapClient").ejOlapClient("option","displaySettings.enableFullScreen");<br>			
			    *	//Sets the enableFullScreen value 
			    *	$("#OlapClient").ejOlapClient("option","displaySettings.enableFullScreen", true ); 
			    * @alias ejOlapClient#displaySettings->enableFullScreen
			    * @instance
			    */
                enableFullScreen: false
            },         

            /**		
			* Allows the user to set custom name for the methods at service-end invoked on AJAX post.
            * @default null
            * @type {object}
            * @example 
            * //To set serviceMethodSettings API value, to invoke the appropriate service method on UI operation.
            *  $("#OlapClient").ejOlapClient({  serviceMethodSettings: {initialize: "MyMethod"}});
            * @example 
            * //Gets or sets the serviceMethodSettings API, to invoke the appropriate service method on UI operation:<br>
            * //Gets the serviceMethodSettings value  
            * $("#OlapClient").ejOlapClient("option", "serviceMethodSettings");<br>   
            * //Sets the serviceMethodSettings value 
            * $("#OlapClient").ejOlapClient("option", "serviceMethodSettings",  {initialize: "MyMethod"} ); 
            * @memberof ejOlapClient
            * @instance
			*/
            serviceMethodSettings: {
			/**		
			* Allows the user to set the custom name for the service method that’s responsible for initializing OLAP Client.
            * @default "InitializeClient"
            * @type {string}
            * @example 
            * //To set initialize API value, to invoke the corresponding service method for OLAP Client initialization.
            *  $("#OlapClient").ejOlapClient({  serviceMethodSettings: {initialize: "InitializeClientMyMethod"});
            * @example 
            * //Gets or sets the initialize API, to invoke the corresponding service method for OLAP Client initialization:<br>
            * //Gets the initialize value  
            * $("#OlapClient").ejOlapClient("option", "serviceMethodSettings");<br>   
            * //Sets the initialize value 
            * $("#OlapClient").ejOlapClient("option", "serviceMethodSettings.initialize", "InitializeClientMyMethod" ); 
            * @alias ejOlapClient#serviceMethodSettings->initialize
            * @instance
			*/
				initialize: "InitializeClient",	
			
			/**		
			* Allows the user to set the custom name for the service method that’s responsible for updating report while removing SplitButton from AxisElementBuilder.
            * @default "RemoveSplitButton"
            * @type {string}
            * @example 
            * //To set removeSplitButton API value, to invoke the corresponding service method for performing server-side operation while removing SplitButton from the AxisElementBuilder.
            *  $("#OlapClient").ejOlapClient({  serviceMethodSettings: {removeSplitButton: "RemoveSplitButtonMyMethod"}});
            * @example 
            * //Gets or sets the removeSplitButton API, to invoke the corresponding service method for performing server-side operation while removing SplitButton from the AxisElementBuilder:<br>
            * //Gets the removeSplitButton value  
            * $("#OlapClient").ejOlapClient("option", "serviceMethodSettings");<br>   
            * //Sets the removeSplitButton value 
            * $("#OlapClient").ejOlapClient("option", "serviceMethodSettings.removeSplitButton", "RemoveSplitButtonMyMethod" ); 
            * @alias ejOlapClient#serviceMethodSettings->removeSplitButton
            * @instance
			*/
				removeSplitButton: "RemoveSplitButton",
           
  		    /**		
			* Allows the user to set the custom name for the service method that’s responsible for updating reports while filtering members.			
            * @default "FilterElement"
            * @type {string}
            * @example 
            * //To set filterElement API value, to invoke the corresponding service method for performing server-side operation while filtering members.
            *  $("#OlapClient").ejOlapClient({  serviceMethodSettings: {filterElement: "filterElementMyMethod"}});
            * @example 
            * //Get or set the filterElement API, to invoke the corresponding service method for performing server-side operation while filtering members:<br>
            * //Gets the filterElement value  
            * $("#OlapClient").ejOlapClient("option", "serviceMethodSettings");<br>   
            * //Sets the filterElement value 
            * $("#OlapClient").ejOlapClient("option", "serviceMethodSettings.filterElement", "FilterElementMyMethod" ); 
            * @alias ejOlapClient#serviceMethodSettings->filterElement
            * @instance
			*/				
				filterElement: "FilterElement", 											
			
		    /**  
            * Allows the user to set the custom name for the service method that’s responsible for updating report while dropping a Node/SplitButton inside AxisElementBuilder.
            * @default "NodeDropped"
            * @type {string}
            * @example 
            * //To set nodeDropped API value, to invoke the corresponding service method for performing server-side operation while dropping a Node/SplitButton inside AxisElementBuilder.
            *  $("#OlapClient").ejOlapClient({  serviceMethodSettings: {nodeDropped: "NodeDroppedMyMethod"}});
            * @example 
            * //Gets or sets the nodeDropped API, to invoke the corresponding service method for performing server-side operation while dropping a Node/SplitButton inside AxisElementBuilder:<br>
            * //Gets the nodeDropped value  
            * $("#OlapClient").ejOlapClient("option", "serviceMethodSettings");<br>   
            * //Sets the nodeDropped value 
            * $("#OlapClient").ejOlapClient("option", "serviceMethodSettings.nodeDropped","NodeDroppedMyMethod"); 
            * @alias ejOlapClient#serviceMethodSettings->nodeDropped
            * @instance
    		 */
				nodeDropped: "NodeDropped",

			/**		
			* Allows the user to set the custom name for the service method that’s responsible to get members for the tree-view inside member-editor dialog.
            * @default "FetchMemberTreeNodes"
            * @type {string}
            * @example 
            * //To set fetchMemberTreeNodes API value, to invoke the corresponding service method to get members for the tree-view inside member-editor dialog.
            *  $("#OlapClient").ejOlapClient({  serviceMethodSettings: {fetchMemberTreeNodes: "FetchMemberTreeNodesMyMethod"}});
            * @example 
            * //Gets or sets the fetchMemberTreeNodes API, to invoke the corresponding service method to get members for the tree-view inside member-editor dialog:<br>
            * //Gets the fetchMemberTreeNodes value  
            * $("#OlapClient").ejOlapClient("option", "serviceMethodSettings");<br>   
            * //Sets the fetchMemberTreeNodes value 
            * $("#OlapClient").ejOlapClient("option", "serviceMethodSettings.fetchMemberTreeNodes", "FetchMemberTreeNodesMyMethod" ); 
            * @alias ejOlapClient#serviceMethodSettings->fetchMemberTreeNodes
            * @instance
			 */
				fetchMemberTreeNodes: "FetchMemberTreeNodes",
				
			/**  
 		    * Allows the user to set the custom name for the service method that’s responsible for updating entire report and control while changing the Cube.
            * @default "CubeChanged"
            * @type {string}
            * @example 
            * //To set cubeChanged API value, to invoke the corresponding service method for updating entire report and control while changing the Cube.
            *  $("#OlapClient").ejOlapClient({  serviceMethodSettings: {cubeChanged: "CubeChangedMyMethod"}});
            * @example 
            * //Gets or sets the cubeChanged API, to invoke the corresponding service method for updating entire report and control while changing the Cube:<br>
            * //Gets the cubeChanged value  
            * $("#OlapClient").ejOlapClient("option", "serviceMethodSettings");<br>   
            * //Sets the cubeChanged value 
            * $("#OlapClient").ejOlapClient("option", "serviceMethodSettings.cubeChanged","CubeChangedMyMethod" ); 
            * @alias ejOlapClient#serviceMethodSettings->cubeChanged
            * @instance
			*/			
				cubeChanged: "CubeChanged",

            /**  
 		    * Allows the user to set the custom name for the service method that’s responsible for updating cube browser tree while changing the measure group.
            * @default "MeasureGroupChanged"
            * @type {string}
            * @example 
            * //To set measureGroupChanged API value, to invoke the corresponding service method for updating the cube browser tree while changing the measure group.
            *  $("#OlapClient").ejOlapClient({  serviceMethodSettings: {measureGroupChanged: "MeasureGroupChangedMyMethod"}});
            * @example 
            * //Gets or sets the measureGroupChanged API, to invoke the corresponding service method for updating the cube browser tree while changing the Measure group:<br>
            * //Gets the measureGroupChanged value  
            * $("#OlapClient").ejOlapClient("option", "serviceMethodSettings");<br>   
            * //Sets the measureGroupChanged value 
            * $("#OlapClient").ejOlapClient("option", "serviceMethodSettings.measureGroupChanged","MeasureGroupChangedMyMethod" ); 
            * @alias ejOlapClient#serviceMethodSettings->measureGroupChanged
            * @instance
			*/
                measureGroupChanged: "MeasureGroupChanged",

			/**  
		   * Allows the user to set the custom name for the service method that’s responsible for any toolbar operation.
           * @default "ToolbarOperations"
           * @type {string}
           * @example 
           * //To set the toolbarServices API value, to invoke the corresponding service method responsible for any toolbar operation.
           *  $("#OlapClient").ejOlapClient({  serviceMethodSettings: {toolbarServices: "ToolbarOperationsMyMethod"}});
           * @example 
           * //Gets or sets the toolbarServices API value, to invoke the corresponding service method responsible for any toolbar operation:<br>
           * //Gets the toolbarServices value  
           * $("#OlapClient").ejOlapClient("option", "serviceMethodSettings");<br>   
           * //Sets the toolbarServices value 
           * $("#OlapClient").ejOlapClient("option", "serviceMethodSettings.toolbarServices", "ToolbarOperationsMyMethod"} ); 
           * @alias ejOlapClient#serviceMethodSettings->toolbarServices
           * @instance
		   */	
				toolbarServices: "ToolbarOperations",			
				
			/**  
            * Allows the user to set the custom name for the service method that’s responsible to get the child members on node expand.
            * @default "MemberExpanded"
            * @type {string}
            * @example 
            * //To set memberExpand API value, to invoke the corresponding service method which would get the child members on node expand.
            *  $("#OlapClient").ejOlapClient({  serviceMethodSettings: {memberExpand: "MemberExpandedMyMethod"}});
            * @example 
            * //Gets or sets the memberExpand API, to invoke the corresponding service method which would get the child members on node expand:<br>
            * //Gets the memberExpand value  
            * $("#OlapClient").ejOlapClient("option", "serviceMethodSettings");<br>   
            * //Sets the memberExpand value 
            * $("#OlapClient").ejOlapClient("option", "serviceMethodSettings.memberExpand", "MemberExpandedMyMethod" ); 
            * @alias ejOlapClient#serviceMethodSettings->memberExpand
            * @instance
   		    */
                memberExpand: "MemberExpanded", 
				
			/**  
			 * Allows the user to set the custom name for the service method that’s responsible for saving the report collection to database.
            * @default "SaveReportToDB"
            * @type {string}
            * @example 
            * //To set saveReport API value, to invoke the corresponding service method which would save the report collection to database.
            *  $("#OlapClient").ejOlapClient({  serviceMethodSettings: {saveReport: "SaveReportToDBMyMethod"}});
            * @example 
            * //Gets or sets the saveReport API, to invoke the corresponding service method which would save the report collection to database:<br>
            * //Gets the saveReport value  
            * $("#OlapClient").ejOlapClient("option", "serviceMethodSettings");<br>   
            * //Sets the saveReport value 
            * $("#OlapClient").ejOlapClient("option", "serviceMethodSettings.saveReport", "SaveReportToDBMyMethod" ); 
            * @alias ejOlapClient#serviceMethodSettings->saveReport
            * @instance
  			 */
				saveReport: "SaveReportToDB",
			
			/**		
			* Allows the user to set the custom name for the service method that’s responsible for fetching the report names from the database.
            * @default "FetchReportListFromDB"
            * @type {string}
            * @example 			
            * //To set fetchReportList API value, to invoke the corresponding service method that’s responsible for fetching the report names from the database.
            *  $("#OlapClient").ejOlapClient({  serviceMethodSettings: {fetchReportList: "FetchReportListFromDBMyMethod"}});
            * @example 
            * //Gets or sets the fetchReportList API, to invoke the corresponding service method that’s responsible for fetching the report names from the database:<br>
            * //Gets the fetchReportList value  
            * $("#OlapClient").ejOlapClient("option", "serviceMethodSettings");<br>   
            * //Sets the fetchReportList value 
            * $("#OlapClient").ejOlapClient("option", "serviceMethodSettings.fetchReportList", "FetchReportListFromDBMyMethod" ); 
            * @alias ejOlapClient#serviceMethodSettings->fetchReportList
            * @instance
			*/
				fetchReportList: "FetchReportListFromDB",
				
			/**		
			*Allows the user to set the custom name for the service method that’s responsible for loading the report collection from database.
            * @default "LoadReportFromDB"
            * @type {string}
            * @example 
            * //To set loadReport API value, to invoke the corresponding service method that’s responsible for loading the report collection from database.
            *  $("#OlapClient").ejOlapClient({  serviceMethodSettings: {loadReport: "LoadReportFromDBMyMethod"}});
            * @example 
            * //Get or set the loadReport API, to invoke the corresponding service method that’s responsible for loading the report from the database:<br>
            * //Gets the loadReport value  
            * $("#OlapClient").ejOlapClient("option", "serviceMethodSettings");<br>   
            * //Sets the loadReport value 
            * $("#OlapClient").ejOlapClient("option", "serviceMethodSettings.loadReport", "LoadReportFromDBMyMethod"} ); 
            * @alias ejOlapClient#serviceMethodSettings->loadReport
            * @instance
			*/
				loadReport: "LoadReportFromDB",

            /**		
			*Allows the user to set the custom name for the service method that’s responsible for updating report collection.
            * @default "UpdateReport"
            * @type {string}
            * @example 
            * //To set updateReport API value, to invoke the corresponding service method that’s responsible for updating the report collection and current report.
            *  $("#OlapClient").ejOlapClient({  serviceMethodSettings: {updateReport: "UpdateReportFromMyMethod"}});
            * @example 
            * //Get or set the updateReport API, to invoke the corresponding service method that’s responsible for updating the current report and report collection:<br>
            * //Gets the updateReport value  
            * $("#OlapClient").ejOlapClient("option", "serviceMethodSettings");<br>   
            * //Sets the updateReport value 
            * $("#OlapClient").ejOlapClient("option", "serviceMethodSettings.updateReport", "UpdateReportFromMyMethod"} ); 
            * @alias ejOlapClient#serviceMethodSettings->updateReport
            * @instance
			*/
				updateReport: "UpdateReport"
            },

            /**		
			* Custom object to pass additional information between client-end and service-end.
			* @default {}
			* @type {Object}
			* @example 
			* //To set customObject API value during initialization
            * $("#OlapClient").ejOlapClient({ customObject: {"MyObject": "Hi Syncfusion!!"} });
			* @example 
			* //Get or set the customObject API, after initialization:<br>
			*	//Gets the customObject value  
			*	$("#OlapClient").ejOlapClient("option","customObject");<br>			
			*	//Sets the customObject value 
			*	$("#OlapClient").ejOlapClient("option","customObject", {"MyObject": "Hello World!!"} ); 
			* @memberof ejOlapClient
			* @instance
			*/
            customObject: {},

            /**		
			* Enables/ disables the visibility of measure group selector drop down in Cube browser in OLAP Client.
			* @default false
			* @type {boolean}
			* @example 
			* //To set enableMeasureGroups API value during initialization
            * $("#OlapClient").ejOlapClient({ enableMeasureGroups : true });
			* @example 
			* //Get or set the enableMeasureGroups API, after initialization:<br>
			*	//Gets the enableMeasureGroups value  
			*	$("#OlapClient").ejOlapClient("option","enableMeasureGroups");<br>			
			*	//Sets the enableMeasureGroups value 
			*	$("#OlapClient").ejOlapClient("option","enableMeasureGroups", true ); 
			* @alias ejOlapClient#enableMeasureGroups
			* @instance
			*/
            enableMeasureGroups: false,

            /**		
			* Sets the progress indicator type which arises on any UI operation.
			* @default ej.olap.OlapChart.progressMode.Infinite
			* @type {enum}
			* @example 
			* //To set progressMode API value during initialization
            * $("#OlapClient").ejOlapClient({ progressMode: ej.olap.OlapChart.progressMode.Normal });
			* @example 
			* //Get or set the progressMode API, after initialization:<br>
			*	//Gets the progressMode value  
			*	$("#OlapClient").ejOlapClient("option","progressMode");<br>			
			*	//Sets the progressMode value 
			*	$("#OlapClient").ejOlapClient("option","progressMode", ej.olap.OlapChart.progressMode.Progress ); 
			* @memberof ejOlapClient
			* @instance
			*/
            progressMode: "infinite",

            /**		
			* Sets the localized language for the control.
			* @default "en-US"
			* @type {string}
			* @example 
			* //To set locale API value during initialization
            * $("#OlapClient").ejOlapClient({ locale: "en-US" });
			* @example 
			* //Get or set the locale API, after initialization:<br>
			*	//Gets the locale value  
			*	$("#OlapClient").ejOlapClient("option","locale");<br>			
			*	//Sets the locale value 
			*	$("#OlapClient").ejOlapClient("option","locale", "fr-FR" ); 
			* @memberof ejOlapClient
			* @instance
			*/
            locale: "en-US",

            //events

            /**     
			 * Fires when the control is rendered success
			 * @event
			 * @name ejOlapClient#renderSuccess	
			 * @param {Object} argument Event parameters from OLAP Client component.
             * @param {Object}  argument.customObject returns the custom object bounded with the control.
             * @param {string}  argument.element returns the outer HTML of OLAP Client control.
			 * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
			 * @param {object}  argument.model returns the OLAP Client model.
			 * @param {string}  argument.type returns the name of the event.
			 * @example 
			 * //success event for rendering the control
             * $("#OlapClient").ejOlapClient({
             *    renderSuccess: function (args) {}
             * });      
			 * @memberof ejOlapClient
			 * @instance
			 */
            renderSuccess: null,

            /**     
			 * Fires when error occured in rendering the control
			 * @event
			 * @name ejOlapClient#renderFailure 	
			 * @param {Object} argument Event parameters from OLAP Client component
             * @param {Object}  argument.customObject returns the custom object bounded with the control.
             * @param {string}  argument.element returns the outer HTML of OLAP Client control.
			 * @param {Object}  argument.message returns the error message with error code.
			 * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
			 * @param {object}  argument.model returns the OLAP Client model.
			 * @param {string}  argument.type returns the name of the event.
			 * @example 
			 * //event triggered when error occurs in rendering the control
             * $("#OlapClient").ejOlapClient({
             *    renderFailure: function (args) {}
             * });      
			 * @memberof ejOlapClient
			 * @instance
			 */
            renderFailure: null,

            /**     
			 * Fires on completion of rendering control
			 * @event
			 * @name ejOlapClient#renderComplete 	
			 * @param {Object} argument Event parameters from OLAP Client component
			 * @param {Object}  argument.customObject returns the custom object bounded with the control.
             * @param {string}  argument.element returns the outer HTML of OLAP Client control.
			 * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
			 * @param {object}  argument.model returns the OLAP Client model.
			 * @param {string}  argument.type returns the name of the event.
			 * @example 
			 * //event triggered after completing rendering the control
             * $("#OlapClient").ejOlapClient({
             *    renderComplete: function (args) {}
             * });      
			 * @memberof ejOlapClient
			 * @instance
			 */
            renderComplete: null,

            /**     
			 * Fires on loading the control
			 * @event
			 * @name ejOlapClient#load 	
			 * @param {Object} argument Event parameters from OLAP Client component.
			 * @param {string}  argument.element returns the outer HTML of OLAP Client component.
			 * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
			 * @param {object}  argument.model returns the OLAP Client model.
			 * @param {string}  argument.type returns the name of the event.
			 * @example 
			 * //event triggered on loading the control
             * $("#OlapClient").ejOlapClient({
             *    load: function (args) {}
             * });      
			 * @memberof ejOlapClient
			 * @instance
			 */
            load: null,

            /**     
			 * Fires before rendering the chart
			 * @event
			 * @name ejOlapClient#chartLoad 	
			 * @param {Object} argument Event parameters from OLAP Chart component.
			 * @param {string}  argument.action return the current action of OLAP Chart control.		 
			 * @param {object}  argument.customObject return the custom object bounds with OLAP Chart control.
			 * @param {string}  argument.element return the outer HTML of OLAP Chart control.
			 * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
			 * @param {object}  argument.model returns the OLAP Chart model.
			 * @param {string}  argument.type returns the name of the event.	
			 * @example 
			 * //event triggered when OLAP Chart begins rendering.
             * $("#OlapClient").ejOlapClient({
             *    chartLoad: function (args) {}
             * });      
			 * @memberof ejOlapClient
			 * @instance
			 */
            chartLoad: "",

            /**    
            * Fires when the summary cell is clicked.
            * @event
            * @name ejOlapClient#beforeServiceInvoke 
			* @param {Object} argument Event parameters from OLAP Client component.
			* @param {string}  argument.action return the current action of OLAP Client control.		 
			* @param {object}  argument.customObject return the custom object bounds with OLAP Client control.
			* @param {string}  argument.element return the outer HTML of OLAP Client control.
			* @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
			* @param {object}  argument.model returns the OLAP Client model.
			* @param {string}  argument.type returns the name of the event.
            * @example 
            * //beforeServiceInvoke event
            * $("#OlapClient").ejOlapClient({
			* 		beforeServiceInvoke: function (args) {}
			* });
            * @memberof ejOlapClient
            * @instance
            */
            beforeServiceInvoke: null,

            /**    
            * Fires after the service is invoked.
            * @event
            * @name ejOlapClient#afterServiceInvoke 
			* @param {Object} argument Event parameters from OLAP Client component.
			* @param {string}  argument.action return the current action of OLAP Client control.		 
			* @param {object}  argument.customObject return the custom object bounds with OLAP Client control.
			* @param {string}  argument.element return the outer HTML of OLAP Client control.
			* @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
			* @param {object}  argument.model returns the OLAP Client model.
			* @param {string}  argument.type returns the name of the event.
            * @example 
            * //afterServiceInvoke event
            * $("#OlapClient").ejOlapClient({
			*		afterServiceInvoke: function (args) {}
			* });
            * @memberof ejOlapClient
            * @instance
            */
            afterServiceInvoke: null
        },

        /**
         * Specify the data types for default properties 
		 * @private
         */
        dataTypes: {
            displaySettings: "data",
            serviceMethodSettings: "data",
            customObject:  "data"
        },
		
		observables: ["title", "gridLayout", "displaySettings.mode", "displaySettings.defaultView", "displaySettings.controlPlacement", "displaySettings.enableTogglePanel", "locale"],
		title: ej.util.valueFunction("title"),
		gridLayout: ej.util.valueFunction("gridLayout"),
		displayMode: ej.util.valueFunction("displaySettings.mode"),
		defaultView: ej.util.valueFunction("displaySettings.defaultView"),
		controlPlacement: ej.util.valueFunction("displaySettings.controlPlacement"),
		enableTogglePanel: ej.util.valueFunction("displaySettings.enableTogglePanel"),
        locale: ej.util.valueFunction("locale"),
		
        /**
         * Initializes the variables and loads the widget. 
         *
         * @private
         */
        _init: function () {
            this._initPrivateProperties();
            this._load();
        },

        /**
         * Initializes the local variables. 
         *
         * @private
         */
        _initPrivateProperties: function () {
            this._id = this.element.attr("id");
            oclientProxy = this;
            currentReport = "";
            currentCubeName = "";
            reports = "";
            reportsCount = 0;
            ogridObj = null;
            ochartObj = null;
            chartObj = null;
            this._keypress = false;
            memberTreeObj = null;
            this._chartHeight = 0;
            this._chartWidth = 0;
            this._gridHeight = 0;
            this._gridWidth = 0;
            this._initStyles = new Array();
            this._toggleStyles = new Array();
            this._initToggle = true;
            this._dimensionName = "";
            this._dllSortMeasure = null;
            this._axis = "";
            this._isSorted = false;
            this._isFiltered = false;
            this._sortOrFilterTab = '';
            this._currentAxis = "";
            this.pNode = "";
            this.progressPos = null;
            draggedSplitBtn = null;
            isDragging = false;
            oclientWaitingPopup = null;
            currentTab = this.defaultView();          
        },

        /**
         * Destroys the widget while it's no longer required. 
         *
         * @private
         */
        _destroy: function () {
            this.element.empty().removeClass("e-olapclient" + this.model.cssClass);
            if (this.oclientWaitingPopup != undefined)
                this.oclientWaitingPopup._destroy();
        },

        /**
         * This function is the point where loading/rendering of the OLAP Client widget is initiated.
         *
         * @private  
         */
        _load: function () {
             
            this.element.addClass(this.model.cssClass);
            if (this.model.progressMode != "infinite") {
                $("#" + oclientProxy._id + "progressContainer").remove();
                this._createProgressBar();
                if (oclientProxy.model.progressMode == "progress")
                    $("#" + oclientProxy._id + "progressText")[0].innerHTML = oclientProxy._getLocalizedLabels("RenderingLayoutStarted");
                this._progressStatus(5, 70, 140);
            }
            else {
                $("#" + this._id).ejWaitingPopup({ showOnInit: true });
                oclientWaitingPopup = $("#" + this._id).data("ejWaitingPopup");
            }

            if (this.model.beforeServiceInvoke != null)
                this._trigger("beforeServiceInvoke", { action: "initializeClient", element: this.element, customObject: this.model.customObject });
            var serializedCustomObject = JSON.stringify(this.model.customObject);
            this.doAjaxPost("POST", this.model.url + "/" + this.model.serviceMethodSettings.initialize, JSON.stringify({
                "action": "initializeClient", "customObject": serializedCustomObject, "clientParams": this.model.enableMeasureGroups + "-" + this.model.chartType
            }), this._renderControlSuccess);
        },

        _setFirst: true,

        /**
         * This function is invoked on any change in the widgets property in-order to assign the altered value.
         *
         * @private 
         */
        _setModel: function (options) {
            for (var key in options) {
                switch (key) {
                    case "title": this.element.find(".titleText").text(this.title()); break;
                    case "currentCubeName": this.currentCubeName = options[key]; break;
                    case "gridLayout": this._renderOlapGrid(); break;
                    case "olapReport": this.currentReport = options[key]; break;
                    case "clientReports": this.reports = options[key]; break;
                    case "customObject": this.model.customObject = options[key]; break;
                    case "locale": { this.locale(); if(this.ogridObj || this.ochartObj) this._load(); break;}
                    case "displaySettings":{ if(this.ogridObj || this.ochartObj) this._load(); break;}
                }
            }
        },

        /**
         * This function is used to get the list of measure elements added in the current report.
         *
         * @private  
         */
        _getMeasuresList: function () {
            var measureName = "";
            this.element.find(".memberEditorDiv").find("div").each(function () { measureName += $(this)[0].id + ","; });
            return measureName;
        },

        /**
         * This function is used to get the unchecked nodes from the tree view inside member editor.
         *
         * @private  
         */
        _getUnSelectedNodes: function () {
            var treeElement = this.element.find(".editorTreeView")[0];
            var unselectedNodes = "";
            var data = $(treeElement).find(":input.nodecheckbox:not(:checked)");
            for (var i = 0; i < data.length; i++) {
                if ($(data[i].parentElement).attr('aria-checked') != 'mixed') {
                    var parentNode = $(data[i]).parents('li:eq(0)');
                    unselectedNodes += "::" + parentNode[0].id + "||" + $(parentNode).attr('tag');
                }
            }
            return unselectedNodes;
        },

        /**
         * This function is used to get the checked nodes from the tree view inside member editor.
         *
         * @private  
         */
        _getSelectedNodes: function (isSlicer) {
             
            if (isSlicer) {
                var treeElement = this.element.find(".editorTreeView")[0].childNodes[0];
                var selectedNodes = new Array();
                var data = $(treeElement).children();
                for (var i = 0; i < data.length; i++) {
                    var parentNode = data[i];
                    var nodeInfo = { caption: $(parentNode.firstChild).find("a").text(), parentId: parentNode.parentElement.parentElement.className.indexOf("editorTreeView") > -1 ? "None" : $(parentNode).parents()[1].id, id: parentNode.id, checked: ($(parentNode).find(':input.nodecheckbox')[0].checked || $(parentNode).find('span:nth-child(1)').attr('class').indexOf("e-chk-indeter") > -1), expanded: $(parentNode.firstChild).find(".e-minus").length > 0 ? true : false, childNodes: new Array(), tag: $(parentNode).attr('tag') };
                    if ($(parentNode).find("ul:first").children().length > 0) {
                        var liElements = $(parentNode).find("ul:first").children();
                        for (var j = 0; j < liElements.length; j++) {
                            nodeInfo.childNodes.push(this._getNodeInfo(liElements[j]));
                        }
                    }
                    selectedNodes.push(nodeInfo);
                }
                return JSON.stringify(selectedNodes);
            }
            else {
                var treeElement = $(".editorTreeView");
                var selectedNodes = "";
                var data = $(treeElement).find(':input.nodecheckbox');
                for (var i = 0; i < data.length; i++) {
                    if (data[i].checked || $(data[i].parentElement).attr('aria-checked') == 'mixed') {
                        var parentNode = $(data[i]).parents('li:eq(0)');
                        selectedNodes += "::" + parentNode[0].id + "||" + $(parentNode).attr('tag');
                    }
                }
                return selectedNodes;
            }
        },

        _getNodeInfo: function (node) {
            var childNode = { caption: $(node.firstChild).find("a").text(), parentId: node.parentElement.parentElement.className.indexOf("editorTreeView") > -1 ? "None" : $(node).parents()[1].id, id: node.id, checked: ($(node).find(':input.nodecheckbox')[0].checked || $(node).find('span:nth-child(1)').attr('class').indexOf("e-chk-indeter") > -1), expanded: $(node.firstChild).find(".e-minus").length > 0 ? true : false, childNodes: new Array(), tag: $(node).attr('tag') };
            if ($(node).find("ul:first").children().length > 0) {
                var liElements = $(node).find("ul:first").children();
                for (var i = 0; i < liElements.length; i++) {
                    childNode.childNodes.push(this._getNodeInfo(liElements[i]));
                }
            }
            return childNode;
        },

        /**
         * This function is used to remove a Split Button.
         *
         * @private  
         */
        _removeSplitBtn: function () {
             
            var dragBtn = $(document).find(".dragClone"); var btnTag;
            var clientBtns = this.element.find(".splitBtn");
            jQuery.each(clientBtns, function (index, value) {
                if ($($(value).children()[0]).text() == $(dragBtn).text()) {
                    btnTag = $(value).attr("tag");
                    $(value).remove();
                }
            });
            $(".dragClone").remove();
            if (this.model.progressMode != "infinite") {
                oclientProxy._progressStatus(0, 80, 50);
                if (oclientProxy.model.progressMode == "progress")
                    $("#" + oclientProxy._id + "progressText")[0].innerHTML = oclientProxy._getLocalizedLabels("PreparingAndExecutingMDXquery");
            }
            else
                oclientWaitingPopup.show();
            if (oclientProxy.model.beforeServiceInvoke != null)
                this._trigger("beforeServiceInvoke", { action: "removeSplitButton", element: this.element, customObject: oclientProxy.model.customObject });
            var serializedCustomObject = JSON.stringify(oclientProxy.model.customObject);
            oclientProxy.doAjaxPost("POST", oclientProxy.model.url + "/" + oclientProxy.model.serviceMethodSettings.removeSplitButton, JSON.stringify({ "action": "removeSplitButton", "clientParams": btnTag, "olapReport": oclientProxy.currentReport, "clientReports": oclientProxy.reports, "customObject": serializedCustomObject }), oclientProxy._removeSplitButtonSuccess)
        },

        /**
         * This function is invoked to wire the events for UI interaction.
         *
         * @private  
         */
        _wireEvents: function () {
             
            oclientProxy._wireDialogEvent();
            oclientProxy._wireEditorRemoveEvent();
            oclientProxy._wireMeasureRemoveEvent();
            this._on($(document), 'keydown', this._KeyPress);
            this.element.find(".categoricalAxis, .rowAxis, .slicerAxis").find("button").ejDraggable({
                handle: 'button',
                clone: true,
                cursorAt: { top: -10, left: -10 },
                dragStart: function (event, ui) {
                    event.event.preventDefault();
                    $(this.element.find(".e-txt")).unbind('touchstart');
                    $(this.element.find(".e-txt")).unbind(ej.eventType.click);
                    isDragging = true;
                    this.element.find(".e-dialog").hide();
                    draggedSplitBtn = event.event.target;
                },
                dragStop: function (event, ui) {
                    this.element.find(".targetAxis").removeClass("targetAxis");
                    isDragging = false;
                    var cssName = null; var droppedPosition;
                    if (event.event.type == "touchend")
                        droppedPosition = oclientProxy.setSplitBtnTargetPos(event);
                    else
                        droppedPosition = oclientProxy.setSplitBtnTargetPos(event.event);
                    if (oclientProxy._dropAxisClassName != undefined && oclientProxy._dropAxisClassName != "") {
                        if (oclientProxy._dropAxisClassName == "outOfAxis")
                            oclientProxy._removeSplitBtn();
                        else
                            cssName = oclientProxy._dropAxisClassName;
                        oclientProxy._dropAxisClassName = "";
                    }
                    else if (event.target != undefined) {
                        if ($(event.target).hasClass("e-btn") || $(event.target).hasClass("removeSplitBtn"))
                            cssName = $(event.target).parents("div:eq(1)").attr("class");
                        else if (jQuery.type(event.target.className) != "string")
                            oclientProxy._removeSplitBtn();
                        else {
                            if (event.target.className.indexOf("splitBtn") > -1)
                                cssName = $(event.target).parents("div:eq(0)").attr("class");
                            else
                                cssName = event.target.className;
                        }
                    }
                    else {
                        if ($(event.event.originalEvent.srcElement).hasClass("e-btn") || $(event.event.originalEvent.srcElement).hasClass("removeSplitBtn"))
                            cssName = $(event.event.originalEvent.srcElement).parents("div:eq(1)").attr("class");
                        else if (event.event.originalEvent.srcElement.className.indexOf("splitBtn") > -1)
                            cssName = $(event.event.originalEvent.srcElement).parents("div:eq(0)").attr("class");
                        else
                            cssName = event.event.originalEvent.srcElement.className;
                    }
                    if (cssName != undefined && cssName != null) {
                        var axisName = (cssName.indexOf("categoricalAxis") > cssName.indexOf("rowAxis")) ? ((cssName.indexOf("categoricalAxis") > cssName.indexOf("slicerAxis")) ? "Categorical" : "Slicer") : ((cssName.indexOf("rowAxis") > cssName.indexOf("slicerAxis")) ? "Series" : (cssName.indexOf("slicerAxis") >= 0) ? "Slicer" : null);
                        if (axisName != null)
                            oclientProxy._splitButtonDropped(axisName, droppedPosition);
                        else
                            oclientProxy._removeSplitBtn();
                    }
                },
                helper: function (event, ui) {
                    if (event.sender.target.className.indexOf("e-btn") > -1)
                        return $(event.sender.target).clone().addClass("dragClone").appendTo(oclientProxy.element);
                    else
                        return false;
                }
            });

            this.element.find(".categoricalAxis, .rowAxis, .slicerAxis").mouseover(function () {
                if (isDragging)
                    $(this).addClass("targetAxis");
            });
            this.element.find(".categoricalAxis, .rowAxis, .slicerAxis").mouseleave(function () {
                $(this).removeClass("targetAxis");
            });

            this.element.find(".e-btn").mouseover(function (evt) {
                $(evt.target.parentNode).find("span").css("display", "inline");
            });

            this.element.find(".splitBtn").mouseover(function (evt) {
                if (isDragging)
                    $(this).addClass("dropIndicator");
            });
            this.element.find(".splitBtn").mouseleave(function (evt) {
                $(this).find("span").css("display", "none");
                $(this).removeClass("dropIndicator");
            });
            this.element.find(".sortDisable, .sortEnable, .filterDisable, .filterEnable ").bind(ej.eventType.click, function (evt) {
                if (evt.target.className == "sortDisable") {
                    $('.measuresList_wrapper,.radioBtnAsc, .radioBtnDesc, .preserveHrchy').attr("disabled", "disabled");
                    oclientProxy._dllSortMeasure.disable();
                }
                else if (evt.target.className == "sortEnable") {
                    $('.measuresList_wrapper,.radioBtnAsc, .radioBtnDesc, .preserveHrchy').removeAttr("disabled");
                    oclientProxy._dllSortMeasure.enable();
                }
                else if (evt.target.className == "filterDisable") {
                    $('.filterFrom, .filterTo').attr("disabled", "disabled");
                    oclientProxy._dllFilterCondition.disable();
                    oclientProxy._dllfMeasuresList.disable();
                }
                else if (evt.target.className == "filterEnable") {
                    $('.filterFrom').removeAttr("disabled");
                    oclientProxy._dllFilterCondition.enable();
                    oclientProxy._dllfMeasuresList.enable();
                }
            });
            this.element.find(".filterFrom , .filterTo").keypress(function (event) {
                if (event.which == 8 || event.which == 0) {
                    return true;
                }

                if (event.which < 46 || event.which > 58) {
                    return false;
                }
                if ((event.which == 46 && $(this).val().indexOf('.') != -1) || event.which == 47) {
                    return false;
                }
            });
            this.element.find(".toggleExpandButton").click(function (evt) {
                if (oclientProxy._initToggle) {
                    oclientProxy._initToggle = false; var items = {};
                    var tempWidth = oclientProxy.element.find(".cubeTable").width() + oclientProxy.element.find(".controlPanel").width() - 17
                    items["controlPanelWidth"] = tempWidth;
                    items["chartOuterWidth"] = items["chartModelWidth"] = items["gridOuterWidth"] = tempWidth - 17;
                    oclientProxy._toggleStyles.push(items);
                }
                oclientProxy._performToggleAction(oclientProxy._toggleStyles);
                oclientProxy.element.find(".csHeader, .cubeTable").hide();
                oclientProxy.element.find(".toggleCollapseButton").show();
                oclientProxy.element.find(".toggleText").show();

            });
            this.element.find(".toggleCollapseButton").click(function (evt) {
                oclientProxy._performToggleAction(oclientProxy._initStyles);
                oclientProxy.element.find(".csHeader, .cubeTable").show();
                oclientProxy.element.find(".toggleCollapseButton").hide();
                oclientProxy.element.find(".toggleText").hide();
            });
            this.element.find(".maximizedView").click(function (evt) {
                oclientProxy._maxViewBtnClick();
            });
            $(document).on('click', '.winCloseBtn', function () {
                oclientProxy._maxViewClsBtnClick();
            });
        },

        /**
         * This function is used to perform UI interactions on pressing the keyboard keys.
         *
         * @private
         */
        _KeyPress: function (e) {
            
            if (e.keyCode == 9) {
                if ($(".e-dialog").length > 0) {
                    $(".clientDialog").focus();
                    $(".reportName").addClass("e-focus");
                }
            }
            if ($(".e-dialog").length > 0 && e.keyCode == 13) {
                this.element.find(".dialogOKBtn").trigger("click");
                oclientProxy._off(this.element, "click", ".dialogOKBtn");
            }
            else if ($(".e-dialog").length > 0 && e.keyCode == 27) {
                this.element.find(".dialogCancelBtn").trigger("click");
                oclientProxy._off(this.element, "click", ".dialogCancelBtn");
            }
            else if ($(".e-dialog").length == 0 && e.keyCode == 27 && $(".fullScreenView").length != 0) {
                this._maxViewClsBtnClick();
            }
            else if (e.keyCode == 9 && $(".borderFilterSortDlg").length > 0) {

            }
            else if (e.keyCode == 13) {
                if (document.activeElement.className.indexOf("newReportImg") > -1) {
                    this.element.find(".newReportImg").trigger("click").off("click");
                }
                else if (document.activeElement.className.indexOf("addReportImg") > -1) {
                    this.element.find(".addReportImg ").trigger("click").off("click");
                        
                }
                else if (document.activeElement.className.indexOf("removeReportImg") > -1) {
                    this.element.find(".removeReportImg ").trigger("click").off("click");
                }
                else if (document.activeElement.className.indexOf("renameReportImg") > -1) {
                    this.element.find(".renameReportImg").trigger("click").off("click");
                }
                else if (document.activeElement.className.indexOf("saveReportImg") > -1) {
                    this.element.find(".saveReportImg").trigger("click").off("click");
                }
                else if (document.activeElement.className.indexOf("loadReportImg") > -1) {
                    this.element.find(".loadReportImg").trigger("click").off("click");
                }
                else if (document.activeElement.className.indexOf("excelExportImg") > -1) {
                    this.element.find(".excelExportImg").trigger("click").off("click");
                }
                else if (document.activeElement.className.indexOf("maximizedView") > -1) {
                    this.element.find(".maximizedView").trigger("click").off("click");
                }
                else if (document.activeElement.className.indexOf("colSortFilterImg") > -1) {
                    this.element.find(".colSortFilterImg ").trigger("click").off("click");
                }
                else if (document.activeElement.className.indexOf("rowSortFilterImg") > -1) {
                    this.element.find(".rowSortFilterImg ").trigger("click").off("click");
                }
            }
        },

        /**
         * This function is invoked to wire the events for removing a measure.
         *
         * @private  
         */
        _wireMeasureRemoveEvent: function () {
             
            this.element.find(".removeMeasure").click(function (evt) {
                $($(evt.target).parent()).remove();
            });
        },

        /**
         * This function is invoked to wire the events for removing a Split Button.
         *
         * @private  
         */
        _wireEditorRemoveEvent: function () {
             
            this.element.find(".removeSplitBtn").click(function (evt) {
                if (oclientProxy.model.progressMode != "infinite")
                    oclientProxy._progressStatus(0, 80, 50);
                else
                    oclientWaitingPopup.show();
                $($(evt.target).parent()).remove();
                oclientProxy._off(this.element, "click", ".removeSplitBtn");
                if (oclientProxy.model.progressMode == "progress")
                    $("#" + oclientProxy._id + "progressText")[0].innerHTML = oclientProxy._getLocalizedLabels("PreparingAndExecutingMDXquery");
                if (oclientProxy.model.beforeServiceInvoke != null)
                    oclientProxy._trigger("beforeServiceInvoke", { action: "removeSplitButton", element: this.element, customObject: oclientProxy.model.customObject });
                var serializedCustomObject = JSON.stringify(oclientProxy.model.customObject);
                oclientProxy.doAjaxPost("POST", oclientProxy.model.url + "/" + oclientProxy.model.serviceMethodSettings.removeSplitButton, JSON.stringify({ "action": "removeSplitButton", "clientParams": $(evt.target).parent().attr("tag"), "olapReport": oclientProxy.currentReport, "clientReports": oclientProxy.reports, "customObject": serializedCustomObject }), oclientProxy._removeSplitButtonSuccess);
            });
        },

        /**
         * This function is invoked to wire the toolbar dialog events.
         *
         * @private
         */
        _wireDialogEvent: function () {
               
            oclientProxy.element.find(".newReportImg, .addReportImg, .removeReportImg, .renameReportImg, .saveReportImg, .loadReportImg, .e-txt,.colSortFilterImg, .rowSortFilterImg").bind(ej.eventType.click, function (evt) {
                if (evt.currentTarget.parentElement.className.indexOf("splitBtn") > -1) {
                    if ($($(evt.currentTarget).parents(".splitBtn")[0]).attr('tag').indexOf("NAMEDSET") > -1) return false;
                    oclientProxy._currentAxis = $($(evt.currentTarget).parents(".splitBtn")[0]).attr('tag').split(':')[0];
                }
                if (!isDragging) {
                    evt.preventDefault();
                    if (evt.target.innerHTML != oclientProxy._getLocalizedLabels("Cancel") && evt.target.innerHTML != oclientProxy._getLocalizedLabels("OK")) {
                        oclientProxy._off(oclientProxy.element, "click", ".newReportImg, .addReportImg, .removeReportImg, .renameReportImg, .saveReportImg, .loadReportImg, .e-txt,.colSortFilterImg, .rowSortFilterImg, .chartTypesImg");
                        if (evt.target.className.indexOf("loadReportImg") >= 0) {
                            if (oclientProxy.model.progressMode != "infinite") {
                                if (oclientProxy.model.progressMode == "progress")
                                    $("#" + oclientProxy._id + "progressText")[0].innerHTML = oclientProxy._getLocalizedLabels("OpeningDialog") + "...";
                                oclientProxy._progressStatus(0, 80, 50);
                            }
                            else
                                oclientWaitingPopup.show();
                            if (oclientProxy.model.beforeServiceInvoke != null)
                                oclientProxy._trigger("beforeServiceInvoke", { action: "FetchingReportList", element: oclientProxy.element, customObject: oclientProxy.model.customObject });
                            var serializedCustomObject = JSON.stringify(oclientProxy.model.customObject);
                            oclientProxy.doAjaxPost("POST", oclientProxy.model.url + "/" + oclientProxy.model.serviceMethodSettings.fetchReportList, JSON.stringify({ "customObject": serializedCustomObject }), oclientProxy._fetchReportListSuccess);
                        }
                        else if (evt.target.className.indexOf("colSortFilterImg") >= 0 || evt.target.className.indexOf("rowSortFilterImg") >= 0) {
                            if (evt.target.className.indexOf("SortImg") >= 0)
                                oclientProxy._sortOrFilterTab = "sort";
                            else
                                oclientProxy._sortOrFilterTab = "filter";
                            if (oclientProxy.model.progressMode != "infinite") {
                                if (oclientProxy.model.progressMode == "progress")
                                    $("#" + oclientProxy._id + "progressText")[0].innerHTML = oclientProxy._getLocalizedLabels("OpeningDialog")+"...";
                                oclientProxy._progressStatus(0, 80, 50);
                            }
                            else
                                oclientWaitingPopup.show();
                            oclientProxy._isSorted = false;
                            oclientProxy._isFiltered = false;
                            oclientProxy.doAjaxPost("POST", oclientProxy.model.url + "/" + oclientProxy.model.serviceMethodSettings.toolbarServices, JSON.stringify({ "action": "FetchSortState", "toolbarOperation": null, "clientInfo": (oclientProxy._axis = (evt.target.className.indexOf("colSortFilterImg") >= 0) ? "Column" : "Row"), "olapReport": oclientProxy.currentReport, "clientReports": "" }), oclientProxy._fetchSortState);
                        }
                        else
                            oclientProxy._createDialogRequest(evt);
                        oclientProxy._dimensionName = $(evt.currentTarget).parent().attr("tag");
                    }
                }
            });

            this.element.find(".excelExportImg").click(function (evt) {
                if (oclientProxy.ogridObj)
                    oclientProxy.ogridObj.exportToExcel();
            });

            this.element.find(".chartTypesImg").click(function (evt) {
                var chartTypesDlg = ej.buildTag("div.chartTypesDialog#chartTypesDialog", ej.buildTag("table", ej.buildTag("tbody", ej.buildTag("tr",
                     ej.buildTag("td", ej.buildTag("div.line chartTypesIcon").attr({ "title": oclientProxy._getLocalizedLabels("Line"), tabindex: 0 })[0].outerHTML)[0].outerHTML +
                    ej.buildTag("td", ej.buildTag("div.spline chartTypesIcon").attr({ "title": oclientProxy._getLocalizedLabels("Spline"), tabindex: 0 })[0].outerHTML)[0].outerHTML +
                    ej.buildTag("td", ej.buildTag("div.column chartTypesIcon").attr({ "title": oclientProxy._getLocalizedLabels("Column"), tabindex: 0 })[0].outerHTML)[0].outerHTML +
                    ej.buildTag("td", ej.buildTag("div.area chartTypesIcon").attr({ "title": oclientProxy._getLocalizedLabels("Area"), tabindex: 0 })[0].outerHTML)[0].outerHTML +
                    ej.buildTag("td", ej.buildTag("div.splinearea chartTypesIcon").attr({ "title": oclientProxy._getLocalizedLabels("SplineArea"), tabindex: 0 })[0].outerHTML)[0].outerHTML)[0].outerHTML +
                    ej.buildTag("tr", ej.buildTag("td", ej.buildTag("div.stepline chartTypesIcon").attr({ "title": oclientProxy._getLocalizedLabels("StepLine"), tabindex: 0 })[0].outerHTML)[0].outerHTML +
                    ej.buildTag("td", ej.buildTag("div.steparea chartTypesIcon").attr({ "title": oclientProxy._getLocalizedLabels("StepArea"), tabindex: 0 })[0].outerHTML)[0].outerHTML +
                    ej.buildTag("td", ej.buildTag("div.pie chartTypesIcon").attr({ "title": oclientProxy._getLocalizedLabels("Pie"), tabindex: 0 })[0].outerHTML)[0].outerHTML +
                    ej.buildTag("td", ej.buildTag("div.bar chartTypesIcon").attr({ "title": oclientProxy._getLocalizedLabels("Bar"), tabindex: 0 })[0].outerHTML)[0].outerHTML +
                    ej.buildTag("td", ej.buildTag("div.stackingarea chartTypesIcon").attr({ "title": oclientProxy._getLocalizedLabels("StackingArea"), tabindex: 0 })[0].outerHTML)[0].outerHTML)[0].outerHTML +
                    ej.buildTag("tr", ej.buildTag("td", ej.buildTag("div.stackingcolumn chartTypesIcon").attr({ "title": oclientProxy._getLocalizedLabels("StackingColumn"), tabindex: 0 })[0].outerHTML)[0].outerHTML +
                    ej.buildTag("td", ej.buildTag("div.stackingbar chartTypesIcon").attr({ "title": oclientProxy._getLocalizedLabels("StackingBar"), tabindex: 0 })[0].outerHTML)[0].outerHTML +
                      ej.buildTag("td", ej.buildTag("div.funnel chartTypesIcon").attr({ "title": oclientProxy._getLocalizedLabels("Funnel"), tabindex: 0 })[0].outerHTML)[0].outerHTML +
                    ej.buildTag("td", ej.buildTag("div.pyramid chartTypesIcon").attr({ "title": oclientProxy._getLocalizedLabels("Pyramid"), tabindex: 0 })[0].outerHTML)[0].outerHTML)[0].outerHTML)[0].outerHTML)[0].outerHTML);
               
                var ele = oclientProxy.element.find('div.chartTypesDialog');
                if (ele.length == 0) {
                    $(chartTypesDlg).appendTo(oclientProxy.element);
                    $(chartTypesDlg).css("left", evt.originalEvent.clientX + 10 + "px").css("top", evt.originalEvent.clientY + 10 + "px");

                }
                $(".chartTypesIcon").click(function (e) {
               
                    $(chartTypesDlg).remove();
                    var selectedtype = e.target.className.split(" ")[0];
                    oclientProxy.model.chartType = selectedtype;
                    oclientProxy.ochartObj.model.type = selectedtype;
                    oclientProxy.ochartObj.model.commonSeriesOptions.type = selectedtype;
                    oclientProxy.ochartObj.renderControlSuccess({ "JsonRecords": JSON.stringify(oclientProxy.ochartObj.getJsonRecords()), "OlapReport": oclientProxy.ochartObj.getOlapReport() });
                    var serializedCustomObject = JSON.stringify(oclientProxy.model.customObject);
                    oclientProxy.doAjaxPost("POST", oclientProxy.model.url + "/" + oclientProxy.model.serviceMethodSettings.updateReport, JSON.stringify({
                        "action": "chartTypeChanged", "clientParams": selectedtype, "olapReport": oclientProxy.currentReport, "clientReports": oclientProxy.reports, "customObject": serializedCustomObject
                    }), oclientProxy._chartTypeChangedSuccess);
                });
            });

            $(document).click(function (e) {
                if (e.target.className.indexOf('chartTypesImg') == -1)
                    $(".chartTypesDialog").remove();
            })

            this.element.find(".dialogCancelBtn").bind(ej.eventType.click, function (e) {
                e.preventDefault();
                oclientProxy.element.find(".e-dialog").hide();
                oclientProxy.element.find(".e-dialog, .clientDialog").remove();
            });

            this.element.find(".dialogOKBtn").bind(ej.eventType.click, function (e) {
            e.preventDefault();
            var loadDlgTitle = oclientProxy.element.find(".e-titlebar")[0].textContent == undefined ? oclientProxy.element.find(".e-titlebar")[0].innerText : oclientProxy.element.find(".e-titlebar")[0].textContent;
            if (loadDlgTitle == oclientProxy._getLocalizedLabels("LoadReport")) {
                    var selectedReport = oclientProxy.element.find(".reportNameList")[0].value;
                    if (selectedReport != "") {
                        if (oclientProxy.model.progressMode != "infinite") {
                            if (oclientProxy.model.progressMode == "progress")
                                $("#" + oclientProxy._id + "progressText")[0].innerHTML = oclientProxy._getLocalizedLabels("PreparingAndExecutingMDXquery");
                            oclientProxy._progressStatus(0, 80, 50);
                        }
                        else
                            oclientWaitingPopup.show();
                        if (oclientProxy.model.beforeServiceInvoke != null)
                            oclientProxy._trigger("beforeServiceInvoke", { action: "loadReport", element: this.element, customObject: oclientProxy.model.customObject });
                        var serializedCustomObject = JSON.stringify(oclientProxy.model.customObject);
                        oclientProxy.doAjaxPost("POST", oclientProxy.model.url + "/" + oclientProxy.model.serviceMethodSettings.loadReport, JSON.stringify({
                            "reportName": selectedReport, "olapReport": currentReport, "clientReports": oclientProxy.reports, "customObject": serializedCustomObject
                        }), oclientProxy._toolbarOperationSuccess);
                    }
                    else {
                        alert("Please select a valid report");
                        return false;
                    }
                }

                else if (oclientProxy.element.find(".sortingDlg").length > 0) {
                    if (oclientProxy.element.find(".sortEnable")[0].checked == true && oclientProxy.element.find(".measuresList")[0].value == "") {
                        alert("Please select a valid measure.");
                        return false;
                    }
                    else if (oclientProxy.element.find(".filterEnable")[0].checked == true) {
                        if (oclientProxy._axis == "Row" && $(".rowAxis ")[0].childElementCount == 0) {
                            alert("Element not found in row axis. Please add an element in row axis for filtering."); return false;
                        }
                        if (oclientProxy._axis == "Column" && $(".categoricalAxis")[0].childElementCount == 0) {
                            alert("Element not found in column axis. Please add an element in column axis for filtering."); return false;
                        }
                        if (oclientProxy.element.find(".fMeasuresList")[0].value == "") {
                            alert("Please select a valid measure."); return false;
                        }
                        else if (oclientProxy.element.find(".filterCondition")[0].value == "") {
                            alert("Please set a valid condition."); return false;
                        }
                        else if (oclientProxy.element.find(".filterFrom")[0].value == "") {
                            alert("Please set a start value."); return false;
                        }
                        else if ((oclientProxy.element.find(".filterCondition")[0].value == "Between" || oclientProxy.element.find(".filterCondition")[0].value == "NotBetween") && oclientProxy.element.find(".filterTo")[0].value == "") {
                            alert("Please set a end value."); return false;
                        }

                    }
                    var sortingDetails = null, filteringDetails = null;
                    if ((oclientProxy._isSorted == false || oclientProxy._isSorted == true) && oclientProxy.element.find(".sortEnable")[0].checked == true)
                        sortingDetails = $("li[tag*='" + oclientProxy.element.find(".measuresList")[0].value + "']").attr("tag") + "::" + ((oclientProxy.element.find(".radioBtnAsc")[0].checked == true) ? "ASC" : "DESC") + "::" + oclientProxy._axis + "::" + ((oclientProxy.element.find(".preserveHrchy")[0].checked == true) ? "PHT" : "PHF");
                    else if (oclientProxy._isSorted == true && oclientProxy.element.find(".sortDisable")[0].checked == true)
                        sortingDetails = "Disable Sorting" + "::" + " " + "::" + oclientProxy._axis + "::" + " ";
                    else
                        sortingDetails = " " + "::" + " " + "::" + oclientProxy._axis + "::" + " ";
                    if ((oclientProxy._isFiltered == false || oclientProxy._isFiltered == true) && oclientProxy.element.find(".filterEnable")[0].checked == true)
                        filteringDetails = $("li[tag*='" + oclientProxy.element.find(".fMeasuresList")[0].value + "']").attr("tag") + "::" + oclientProxy.element.find(".filterCondition")[0].value + "::" + oclientProxy.element.find(".filterFrom")[0].value + "::" + oclientProxy.element.find(".filterTo")[0].value;
                    else if (oclientProxy._isFiltered == true && oclientProxy.element.find(".filterDisable")[0].checked == true)
                        filteringDetails = "Disable Filtering";
                    else
                        filteringDetails = "";
                    oclientProxy._SortFilterDetails = sortingDetails + "||" + filteringDetails;

                    if (oclientProxy.model.progressMode != "infinite") {
                        if (oclientProxy.model.progressMode == "progress")
                            $("#" + oclientProxy._id + "progressText")[0].innerHTML = oclientProxy._getLocalizedLabels("PreparingAndExecutingMDXquery");
                        oclientProxy._progressStatus(0, 80, 50);
                    }
                    else
                        oclientWaitingPopup.show();
                    if (oclientProxy.element.find(".sortEnable")[0].checked == true || oclientProxy.element.find(".filterEnable")[0].checked == true) {
                        oclientProxy.doAjaxPost("POST", oclientProxy.model.url + "/" + oclientProxy.model.serviceMethodSettings.toolbarServices, JSON.stringify({ "action": "toolbarOperation", "toolbarOperation": "SortOrFilter", "clientInfo": oclientProxy._SortFilterDetails, "olapReport": oclientProxy.currentReport, "clientReports": oclientProxy.reports }), oclientProxy._toolbarOperationSuccess);
                    }
                    else if (oclientProxy.element.find(".sortDisable")[0].checked == true || oclientProxy.element.find(".filterDisable")[0].checked == true) {
                        if (oclientProxy._isSorted == true || oclientProxy._isFiltered == true) {
                            oclientProxy.doAjaxPost("POST", oclientProxy.model.url + "/" + oclientProxy.model.serviceMethodSettings.toolbarServices, JSON.stringify({ "action": "toolbarOperation", "toolbarOperation": "SortOrFilter", "clientInfo": oclientProxy._SortFilterDetails, "olapReport": oclientProxy.currentReport, "clientReports": oclientProxy.reports }), oclientProxy._toolbarOperationSuccess);
                            oclientProxy._isSorted = false;
                        }
                        else {
                            alert("Invalid operation !");
                            if (oclientProxy.model.progressMode == "progress")
                                oclientProxy._progressHide();
                            else
                                oclientWaitingPopup.hide();
                            return false;
                        }
                    }

                }
                else {
                    if (oclientProxy._getSelectedNodes() == 0 && args_innerHTML != oclientProxy._getLocalizedLabels("Measures") && args_innerHTML != "ToolbarButtons")
                        return false;
                    var unselectedNodes = null;
                    if (args_innerHTML != "ToolbarButtons") {
                        if (oclientProxy.model.progressMode != "infinite")
                            oclientProxy._progressStatus(0, 80, 50);
                        else
                            oclientWaitingPopup.show();
                        if (args_innerHTML == oclientProxy._getLocalizedLabels("Measures"))
                            unselectedNodes = "Measures:" + oclientProxy._getMeasuresList();
                        else if (args_innerHTML != "ToolbarButtons" && args_innerHTML != undefined)
                            unselectedNodes = oclientProxy._getUnSelectedNodes() + "CHECKED" + oclientProxy._getSelectedNodes(oclientProxy._currentAxis == "Slicers" ? true : false);
                        if (args_innerHTML == oclientProxy._getLocalizedLabels("Measures") && unselectedNodes != null && unselectedNodes.split(":")[1] == "") {
                            $(oclientProxy.element).find(".splitBtn").each(function (index, value) {
                                if (value.firstChild.innerHTML == oclientProxy._getLocalizedLabels("Measures"))
                                    $(value).remove();
                            });
                        }
                        $(oclientProxy).find(".e-dialog").hide();
                        if (oclientProxy.model.progressMode == "progress")
                            $("#" + oclientProxy._id + "progressText")[0].innerHTML = oclientProxy._getLocalizedLabels("PreparingAndExecutingMDXquery");
                        if (oclientProxy.model.beforeServiceInvoke != null)
                            oclientProxy._trigger("beforeServiceInvoke", { "action": "filtering", element: this.element, customObject: oclientProxy.model.customObject });
                        var serializedCustomObject = JSON.stringify(oclientProxy.model.customObject);
                        oclientProxy.doAjaxPost("POST", oclientProxy.model.url + "/" + oclientProxy.model.serviceMethodSettings.filterElement, JSON.stringify({ "action": "filtering", "clientParams": unselectedNodes, "olapReport": oclientProxy.currentReport, "clientReports": oclientProxy.reports, "customObject": serializedCustomObject }), oclientProxy._filterElementSuccess);
                    }
                    else {
                        if (oclientProxy.element.find(".e-titlebar").first().text() == oclientProxy._getLocalizedLabels("RemoveReport") && reportsCount < 2)
                            oclientProxy.element.find(".e-dialog").hide();
                        else if (oclientProxy.element.find(".reportName").val() == "") {
                            return false;
                        }
                        else {
                            var reportName = oclientProxy.element.find(".reportName").val() || reportDropTarget.selectedTextValue;                         
                            var operation = oclientProxy.element.find(".e-titlebar").first().attr("tag");
                            var olapReport = (operation == "New Report" ? "" : oclientProxy.currentReport);
                            var clientReports = (operation == "New Report" ? "" : oclientProxy.reports);
                            oclientProxy.element.find(".e-dialog").hide();
                            if (oclientProxy.model.progressMode == "progress")
                                $("#" + oclientProxy._id + "progressText")[0].innerHTML = oclientProxy._getLocalizedLabels("OpeningDialog") + "...";
                            if (operation == "Save Report") {
                                if (oclientProxy.model.beforeServiceInvoke != null)
                                    oclientProxy._trigger("beforeServiceInvoke", { action: "saveReport", element: this.element, customObject: oclientProxy.model.customObject });
                                var serializedCustomObject = JSON.stringify(oclientProxy.model.customObject);
                                oclientProxy.doAjaxPost("POST", oclientProxy.model.url + "/" + oclientProxy.model.serviceMethodSettings.saveReport, JSON.stringify({
                                    "reportName": reportName, "olapReport": olapReport, "clientReports": clientReports, "customObject": serializedCustomObject
                                }), oclientProxy._toolbarOperationSuccess);

                            } else
                                if (oclientProxy.model.beforeServiceInvoke != null)
                                    oclientProxy._trigger("beforeServiceInvoke", { action: "toolbarOperation", element: this.element, customObject: oclientProxy.model.customObject });
                            var serializedCustomObject = JSON.stringify(oclientProxy.model.customObject);
                            oclientProxy.doAjaxPost("POST", oclientProxy.model.url + "/" + oclientProxy.model.serviceMethodSettings.toolbarServices, JSON.stringify({
                                "action": "toolbarOperation", "toolbarOperation": operation, "clientInfo": reportName, "olapReport": olapReport, "clientReports": clientReports, "customObject": serializedCustomObject
                            }), oclientProxy._toolbarOperationSuccess);
                        }
                    }
                }
                oclientProxy.element.find(".e-dialog, .clientDialog").remove();
            });
            this.element.find(".measureEditor").mouseover(function (evt) {
                $(evt.target).find("span").css("display", "inline");
            });
            this.element.find(".measureEditor").mouseleave(function (evt) {
                $(this).find("span").css("display", "none");
            });
            this.element.find(".unCheckAll,.checkAll").click(function (evt) {
                if (evt.target.className.indexOf("checkAll")>-1)
                    oclientProxy.element.find(".editorTreeView").ejTreeView("checkAll");
                else
                    oclientProxy.element.find(".editorTreeView").ejTreeView("unCheckAll");
            });
        },

        /**
         * This function is used to extract the appropriate localized text.
         *
         * @private  
         */
        _getLocalizedLabels: function (property) {
            return ej.olap.OlapClient.locale[this.locale()][property] === undefined ? ej.olap.OlapClient.locale["en-US"][property] : ej.olap.OlapClient.locale[this.locale()][property];
        },

        /**
         * This function is invoked to unwire the events registered for UI interaction.
         *
         * @private  
         */
        _unWireEvents: function () {
            $(this.element.find(".dialogCancelBtn, .dialogOKBtn, .newReportImg, .addReportImg, .removeReportImg, .renameReportImg, .e-txt, .removeSplitBtn, .unCheckAll, .checkAll, .removeMeasure, .toggleCollapseButton, .toggleExpandButton, .saveReportImg, .loadReportImg,.maximizedView,.colSortFilterImg, .rowSortFilterImg, .chartTypesImg")).unbind(ej.eventType.click);
            $(this.element.find(".dialogCancelBtn, .dialogOKBtn, .newReportImg, .addReportImg, .removeReportImg, .renameReportImg, .e-txt, .removeSplitBtn, .unCheckAll, .checkAll, .removeMeasure, .toggleCollapseButton, .toggleExpandButton, .saveReportImg, .loadReportImg,.maximizedView,.colSortFilterImg, .rowSortFilterImg, .chartTypesImg")).unbind('click');
            $(document).find(".winCloseBtn").unbind(ej.eventType.click);
            $(document).find(".winCloseBtn").unbind('click');
            this._off($(document), 'keydown', this._KeyPress);
        },

        /**
         * This function is used to create progress bar containing status text and percentage.
         *
         * @private  
         */
        _createProgressBar: function () {
             
            var progressContainer = ej.buildTag("div#" + oclientProxy._id + "progressContainer.clientProgressContainer", "", { top: ($("#" + this._id).position().top + ($("#" + this._id).height() / 2) - 30), left: ($("#" + this._id).position().left + ($("#" + this._id).width() / 3) - 50) })[0].outerHTML;
            var blockControl = ej.buildTag("div#" + oclientProxy._id + "blockDiv.blockDiv", "", { width: $("#" + this._id).width(), display: "block", opacity: "0.3", backgroundColor: "#E9E9E9", height: $("#" + this._id).height(), position: 'fixed', top: ($("#" + this._id).position().top), left: ($("#" + this._id).position().left) })[0].outerHTML;
            $("#" + oclientProxy._id).after(progressContainer);
            var progressBar = ej.buildTag("div#" + oclientProxy._id + "progressBar.clientProgressBar", "", {})[0].outerHTML;
            if (oclientProxy.model.progressMode == "normal") {
                $("#" + oclientProxy._id + "progressContainer").css({ width: 0, height: 0 });
                $("#" + oclientProxy._id + "progressContainer").append(progressBar).append(blockControl);
            }
            else {
                var progressText = ej.buildTag("span#" + oclientProxy._id + "progressText.clientProgressText", "Initializing...", {})[0].outerHTML;
                $("#" + oclientProxy._id + "progressContainer").append(progressBar).append(progressText).append(blockControl);
            }
            $("#" + oclientProxy._id + "progressBar").ejProgressBar({ value: 10, text: 10 + " %", complete: "progressbarCompleted" });
            oclientProgressBar = $("#" + oclientProxy._id + "progressBar").data("ejProgressBar");
        },

        /**
         * This function is used to update the progress status.
         *
         * @private  
         */
        _progressStatus: function (current_Status, completion_Status, progress_Speed) {
            if (jQuery.type(completion_Status) === "string") {
                var progressText = completion_Status.split(':');
                if (progressText != undefined) completion_Status = parseInt(progressText[0]);
            }
            oclientProgressBar.setModel({ value: current_Status, text: current_Status + " %" });
            $("#" + oclientProxy._id + "progressContainer").show();
            oClientTimer = window.setInterval(function () {
                oclientProgressBar.setModel({ value: current_Status, text: current_Status + " %" });
                current_Status++;
                if (current_Status >= completion_Status) {
                    window.clearInterval(oClientTimer);
                    if (progressText != undefined) {
                        oclientProgressBar.setModel({ value: 100, text: progressText[1] });
                    }

                };
            }, progress_Speed);

        },

        /**
         * This function is used to hide the progress bar.
         *
         * @private  
         */
        _progressHide: function () {
            if (oclientProxy.model.progressMode != "infinite") {
                window.clearInterval(oClientTimer);
                $("#" + oclientProxy._id + "progressContainer").hide();
            }
            else
                oclientWaitingPopup.hide();
        },

        /**
         * This function is used to perform appropriate action (visibility) while toggling panel.
         *
         * @private  
         */
        _performToggleAction: function (styles) {
            this.element.find(".controlPanel").width(styles[0].controlPanelWidth);
            if ((this.controlPlacement() != "horizontal") || (this.controlPlacement() == "horizontal" && this.displayMode() == "chartOnly") ||
                this.controlPlacement() == "horizontal" && this.displayMode() == "gridOnly") {
                this.element.find(".e-olapchart").width(styles[0].chartOuterWidth);
                this.element.find(".e-olapgrid").width(styles[0].gridOuterWidth);
                if (this.ochartObj != null || this.ochartObj != undefined) {
                    this.element.find("#" + this.ochartObj._id).ejOlapChart("option", "width", styles[0].chartOuterWidth+"px");
                    this.chartObj = this.element.find("#" + this.ochartObj._id + "Container").data("ejChart");
                    if (this.chartObj) {
                        this.element.find("#" + this.chartObj._id).ejChart("option", { "model": { size: { width: styles[0].chartOuterWidth+"px" } } });
                        this.element.find("#" + this.chartObj._id + "_svg").width(styles[0].chartOuterWidth);
                        this.chartObj.redraw();
                    }
                }
            }
        },

        /**
         * This function is used to update data in OLAP controls while dragging and dropping the Split Button.
         *
         * @private  
         */
        _splitButtonDropped: function (axis, droppedPosition) {
            if (draggedSplitBtn != null) {
                var params = this.element.find(".cubeName").html() + "--" + $(draggedSplitBtn).parent("div:eq(0)").attr("Tag") + "--" + axis + "--" + droppedPosition;
                draggedSplitBtn = null;
                if (oclientProxy.model.progressMode != "infinite") {
                    if (oclientProxy.model.progressMode == "progress")
                        $("#" + oclientProxy._id + "progressText")[0].innerHTML = oclientProxy._getLocalizedLabels("PreparingAndExecutingMDXquery");
                    this._progressStatus(0, 80, 100);
                }
                else
                    oclientWaitingPopup.show();
                if (oclientProxy.model.beforeServiceInvoke != null)
                    this._trigger("beforeServiceInvoke", { action: "nodeDropped", element: this.element, customObject: this.model.customObject });
                var serializedCustomObject = JSON.stringify(oclientProxy.model.customObject);
                oclientProxy.doAjaxPost("POST", oclientProxy.model.url + "/" + oclientProxy.model.serviceMethodSettings.nodeDropped, JSON.stringify({ "action": "nodeDropped", "dropType": "SplitButton", "nodeInfo": params, "olapReport": oclientProxy.currentReport, "clientReports": oclientProxy.reports, "customObject": serializedCustomObject }), oclientProxy._nodeDroppedSuccess);
            }
        },

        /**
         * This function receives the controls' update from service-end which would be utilized after the filtering action.
         *
         * @private  
         */
        _filterElementSuccess: function (report) {
             
            if (oclientProxy.model.progressMode != "infinite") {
                if (oclientProxy.model.progressMode == "progress")
                    $("#" + oclientProxy._id + "progressText")[0].innerHTML = oclientProxy._getLocalizedLabels("MDXqueryExecutedSuccessfully")+"...";
                window.clearInterval(oClientTimer);
            }
            if (report[0] != undefined) {
                oclientProxy.currentReport = report[0].Value; oclientProxy.reports = report[1].Value;
                if (report[2] != null && report[2] != undefined)
                    oclientProxy.model.customObject = report[2].Value;
            }
            else if (report.d != undefined) {
                oclientProxy.currentReport = report.d[0].Value; oclientProxy.reports = report.d[1].Value;
                if (report.d[2] != null && report.d[2] != undefined)
                    oclientProxy.model.customObject = report.d[2].Value;
            }
            else {
                oclientProxy.currentReport = report.UpdatedReport; oclientProxy.reports = report.ClientReports;
                if (report.customObject != null && report.customObject != undefined)
                    oclientProxy.model.customObject = report.customObject;
            }
            if (this.model.afterServiceInvoke != null)
                this._trigger("afterServiceInvoke", { action: "filtering", element: this.element, customObject: this.model.customObject });
            if (this.model.progressMode != "infinite") {
                this._progressStatus(90, 101 + ":100%", 50);
                window.setTimeout(function () { progressStatus() }, 300);
            }
            else
                window.setTimeout(function () { progressStatus() }, 1);
            function progressStatus() {
                if (oclientProxy.model.progressMode != "infinite") {
                    window.clearInterval(oClientTimer);
                    if (oclientProxy.model.progressMode == "progress")
                        $("#" + oclientProxy._id + "progressText")[0].innerHTML = oclientProxy._getLocalizedLabels("RenderingSucceeded");
                    oclientProxy._progressHide();
                }
                oclientProxy._renderControls();
                oclientProxy._unWireEvents();
                oclientProxy._wireEvents();
            }
            oclientProxy._successAction = "Filter";
            oclientProxy._trigger("renderSuccess", oclientProxy);
        },

        /**
         * This function is used to switch the control(s) to maximized view.
         *
         * @private  
         */
        _maxViewBtnClick: function () {
            var winWidth = $(window).width() - 150;
            var winHeight = $(window).height() - 100;
            var docDivTag = ej.buildTag("div.fullScreenView", "", { width: $(document).width(), height: $(document).height() });
            var winDivTag = ej.buildTag("div#" + oclientProxy._id + "_maxView.maximumView", "", { width: winWidth, height: winHeight });
            var maxViewCls = ej.buildTag("div#Winclose.winCloseBtn e-icon", "");
            if (oclientProxy.displayMode() == "chartonly" && ($("#" + oclientProxy.ochartObj._id + "Container_svg")[0])) {
                oclientProxy._fullScreenView(100, 160);
                winDivTag.append(oclientProxy.element.find("#OlapChart"));
            }
            if (oclientProxy.displayMode() == "gridOnly")
                winDivTag.append(oclientProxy.element.find("#OlapGrid").css({ "max-height": winHeight, "width": winWidth }));
            if (oclientProxy.controlPlacement() == ej.olap.OlapClient.ControlPlacement.Tab && oclientProxy.displayMode() == "chartandgrid") {
                if (currentTab == "grid")
                    winDivTag.append(oclientProxy.element.find("#OlapGrid").css({ "max-height": winHeight, "width": winWidth }));
                else if (currentTab == "chart" && ($("#" + oclientProxy.ochartObj._id + "Container_svg")[0])) {
                    oclientProxy._fullScreenView(100, 160);
                    winDivTag.append(oclientProxy.element.find("#OlapChart"));
                }
            }
            else if (oclientProxy.controlPlacement() == ej.olap.OlapClient.ControlPlacement.Tile && oclientProxy.displayMode() == "chartandgrid") {
                var hght = ($(window).height() / 2) - 50;
                oclientProxy.element.find("#OlapGrid").css({ "max-height": hght, "margin-left": "0px" });
                oclientProxy._fullScreenView(50, 160);
                if(($("#" + oclientProxy.ochartObj._id + "Container_svg")[0]))
                    if (oclientProxy.defaultView() == "chart")
                    winDivTag.append(oclientProxy.element.find("#OlapChart")).append(oclientProxy.element.find("#OlapGrid").width(($(window).width() - 160)));
                else
                    winDivTag.append(oclientProxy.element.find("#OlapGrid").width($(window).width() - 160)).append(oclientProxy.element.find("#OlapChart"));
                winDivTag.append(oclientProxy.element.find("#OlapGrid").width($(window).width() - 160));
            }
            winDivTag.append(maxViewCls);
            docDivTag.append(winDivTag);
            $("body").append(docDivTag);
        },

        /**
         * This function is used to restore the control(s) from maximized view.
         *
         * @private  
         */
        _maxViewClsBtnClick: function () {
            var fullScreen = $(".fullScreenView");
            fullScreen.find("#OlapGrid").css("margin-left", "7px");
            fullScreen.find("#OlapChart").appendTo(oclientProxy.element.find("#chartPanel"));
            if (oclientProxy.displayMode() == "chartonly" && ($("#" + oclientProxy.ochartObj._id + "Container_svg")[0]))
                oclientProxy._fullScreenViewCls(560, 790);
            if (oclientProxy.displayMode() == "gridOnly")
                fullScreen.find("#OlapGrid").css({ "width": "inherit", "max-height": '' });
            if (oclientProxy.controlPlacement() == ej.olap.OlapClient.ControlPlacement.Tab && oclientProxy.displayMode() == "chartandgrid") {
                if (currentTab == "chart" && $("#" + oclientProxy.ochartObj._id + "Container_svg")[0])
                    oclientProxy._fullScreenViewCls(547, 557);
                else
                    fullScreen.find("#OlapGrid").css({ "max-height": "530px", "width": "555px" });
            }
            if (oclientProxy.controlPlacement() == ej.olap.OlapClient.ControlPlacement.Tile && oclientProxy.displayMode() == "chartandgrid")
                if ($("#" + oclientProxy.ochartObj._id + "Container_svg")[0]) {
                    oclientProxy._fullScreenViewCls(275, 555);
                    fullScreen.find("#OlapGrid").css({ "max-height": "300px", "width": "555px" });
                }else
                    fullScreen.find("#OlapGrid").css({ "max-height": "275px", "width": "555px" });
            if (oclientProxy.enableTogglePanel() && (oclientProxy.element.find(".cubeTable").is(':visible')) == false)
                fullScreen.find("#OlapGrid").css("width", "950px");
            fullScreen.find("#OlapGrid").appendTo(oclientProxy.element.find("#gridPanel"));
            $(".maximumView").remove();
            $(".fullScreenView").remove();
        },

        /**
         * This function changes the default view to full screen view. It displays only the active control in full screen mode.
         *
         * @private  
         */
        _fullScreenView: function (height, width) {
            if (oclientProxy.ochartObj != null && oclientProxy.ochartObj != undefined && ($("#" + oclientProxy.ochartObj._id + "Container_svg")[0])) {
                oclientProxy.chartObj = oclientProxy.element.find("#" + oclientProxy.ochartObj._id + "Container").data("ejChart");
                var chartWidth = oclientProxy.chartObj.model.size.width = oclientProxy.ochartObj.model.size.width = $(window).width() - width+"px";
                if (oclientProxy.controlPlacement() == ej.olap.OlapClient.ControlPlacement.Tile && oclientProxy.displayMode() == "chartandgrid")
                    var chartHeight = oclientProxy.ochartObj.model.size.height = oclientProxy.chartObj.model.size.height = ($(window).height() / 2) - height+"px";
                else
                    var chartHeight = oclientProxy.ochartObj.model.size.height = oclientProxy.chartObj.model.size.height = $(window).height() - height+"px";
                oclientProxy.element.find("#OlapChart").css({ "min-height": chartHeight, "width": chartWidth });
                oclientProxy.chartObj.redraw();
            }
        },

        /**
         * This function resets the control to default view from maximized view.
         *
         * @private  
         */
        _fullScreenViewCls: function (height, width) {
            if (oclientProxy.ochartObj != null && oclientProxy.ochartObj != undefined) {
                oclientProxy.chartObj = oclientProxy.element.find("#" + oclientProxy.ochartObj._id + "Container").data("ejChart");
                var chartHeight = oclientProxy.chartObj.model.size.height = oclientProxy.ochartObj.model.size.height =height+"px";
                if (oclientProxy.enableTogglePanel() && (oclientProxy.element.find(".cubeTable").is(':visible')) == false)
                    var chartWidth = oclientProxy.chartObj.model.size.width = oclientProxy.ochartObj.model.size.width = "950px";                 
                else
                    var chartWidth = oclientProxy.chartObj.model.size.width = oclientProxy.ochartObj.model.size.width = width+"px";
                oclientProxy.element.find("#OlapChart").css({ "min-height": chartHeight, "width": chartWidth });
                oclientProxy.chartObj.redraw();
            }
        },

        /**
         * This function receives the control's update from service-end which would be utilized after dropping a treenode to Axis Element Builder.
         *
         * @private  
         */
        _nodeDroppedSuccess: function (data) {
            if (oclientProxy.model.progressMode != "infinite") {
                if (oclientProxy.model.progressMode == "progress")
                    $("#" + oclientProxy._id + "progressText")[0].innerHTML = oclientProxy._getLocalizedLabels("MDXqueryExecutedSuccessfully")+"...";
                window.clearInterval(oClientTimer);
                oclientProgressBar.setModel({ value: 85, text: 85 + " %" });
            }

            var columnElements, rowElements, slicerElements;
            if (data[0] != undefined) {
                columnElements = data[0].Value; rowElements = data[1].Value; slicerElements = data[2].Value;
                oclientProxy.currentReport = data[3].Value; oclientProxy.reports = data[4].Value;
                if (data[5] != null && data[5] != undefined)
                    oclientProxy.model.customObject = data[5].Value;
            }
            else if (data.d != undefined) {
                columnElements = data.d[0].Value; rowElements = data.d[1].Value; slicerElements = data.d[2].Value;
                oclientProxy.currentReport = data.d[3].Value; oclientProxy.reports = data.d[4].Value;
                if (data.d[5] != null && data.d[5] != undefined)
                    oclientProxy.model.customObject = data.d[5].Value;
            }
            else {
                columnElements = data.Columns; rowElements = data.Rows; slicerElements = data.Slicers;
                oclientProxy.currentReport = data.UpdatedReport; oclientProxy.reports = data.ClientReports;
                if (data.customObject != null && data.customObject != undefined)
                    oclientProxy.model.customObject = data.customObject;
            }
            if (this.model.afterServiceInvoke != null)
                this._trigger("afterServiceInvoke", { action: "nodeDropped", element: this.element, customObject: this.model.customObject });
            if (oclientProxy.model.progressMode != "infinite")
                oclientProgressBar.setModel({ value: 90, text: 90 + " %" });
            this.element.find(".splitBtn, .e-btn").remove();
            $(oclientProxy._createSplitButtons(columnElements, "Columns")).appendTo(".categoricalAxis");
            $(oclientProxy._createSplitButtons(rowElements, "Rows")).appendTo(".rowAxis");
            $(oclientProxy._createSplitButtons(slicerElements, "Slicers")).appendTo(".slicerAxis");
            this.element.find(".categoricalAxis, .rowAxis, .slicerAxis").find("button").ejButton({ height: "20px" });
            if (oclientProxy.model.progressMode != "infinite") {
                this._progressStatus(95, 101 + "::100%", 50);
                if (oclientProxy.model.progressMode == "progress")
                    $("#" + oclientProxy._id + "progressText")[0].innerHTML = oclientProxy._getLocalizedLabels("RenderingSucceeded");
                window.setTimeout(function () { progressStatus() }, 300);
            }
            else
                window.setTimeout(function () { progressStatus() }, 1);
            function progressStatus() {
                oclientProxy._renderControls();
                oclientProxy._unWireEvents();
                oclientProxy._wireEvents();
                if (oclientProxy.model.progressMode != "infinite") {
                    window.clearInterval(oClientTimer);
                    oclientProxy._progressHide();
                }
            }
            oclientProxy._successAction = "NodeDrop";
            oclientProxy._trigger("renderSuccess", oclientProxy);
        },

        /**
         * This function receives the control's update from service-end after getting the members to populate the tree inside member editor.
         *
         * @private  
         */
        _editorTreeInfoSuccess: function (editorTree) {
              
            if (editorTree[0] != undefined) {
                if (editorTree[2] != null && editorTree[2] != undefined)
                    oclientProxy.model.customObject = editorTree[2].Value;
            }
            else if (editorTree.d != undefined) {
                if (editorTree.d[2] != null && editorTree.d[2] != undefined)
                    oclientProxy.model.customObject = editorTree.d[2].Value;
            }
            else {
                if (editorTree.customObject != null && editorTree.customObject != undefined)
                    oclientProxy.model.customObject = editorTree.customObject;
            }
            if (this.model.afterServiceInvoke != null)
                this._trigger("afterServiceInvoke", { action: "fetchMemberTreeNodes", element: this.element, customObject: this.model.customObject });
            if (oclientProxy.model.progressMode != "infinite") {
                window.clearInterval(oClientTimer);
                if (oclientProxy.model.progressMode == "progress")
                    $("#" + oclientProxy._id + "progressText")[0].innerHTML = oclientProxy._getLocalizedLabels("OpeningDialogSucceeded") + "...";
                this._progressStatus(95, 101 + "::100%", 50);
                window.setTimeout(function () { progressStatus() }, 300);
            }
            else
                window.setTimeout(function () { progressStatus() }, 1);
            function progressStatus() {
                oclientProxy._createDialog(args_className, args_innerHTML, editorTree);
                if (oclientProxy.model.progressMode != "infinite") {
                    window.clearInterval(oClientTimer);
                    oclientProxy._progressHide();
                }
                else
                    oclientWaitingPopup.hide();
            }
            oclientProxy._successAction = "EditorTreeInfo";
            oclientProxy._trigger("renderSuccess", oclientProxy);
        },

        /**
         * 	This function receives the control's update from service-end which would be utilized for rendering the widget.
         *
         * @private  
         */
        _renderControlSuccess: function (msg) {
             
            eventArgs = { element: this.element, customObject: this.model.customObject };
            oclientProxy._trigger("load", eventArgs);
            if (oclientProxy.model.progressMode != "infinite") {
                if (oclientProxy.model.progressMode == "progress")
                    $("#" + oclientProxy._id + "progressText")[0].innerHTML =  oclientProxy._getLocalizedLabels("RenderingLayoutCompleted")+"...";
                window.clearInterval(oClientTimer);
                if (oclientProxy.model.progressMode == "progress")
                    $("#" + oclientProxy._id + "progressText")[0].innerHTML = oclientProxy._getLocalizedLabels("RenderingStarted")+"...";
                oclientProgressBar.setModel({ value: 80, text: "80%" });
            }
            var cubes, columnElements, rowElements, slicerElements, cubeTreeInfo, measureGroupInfo;
            if (msg[0] != undefined) {
                cubes = msg[0].Value; columnElements = msg[1].Value; rowElements = msg[2].Value;
                slicerElements = msg[3].Value; cubeTreeInfo = msg[4].Value; oclientProxy.currentReport = msg[5].Value;
                oclientProxy.reports = msg[6].Value; reportsCount = msg[7].Value; reportList = $.parseJSON(msg[8].Value);
                if (oclientProxy.model.enableMeasureGroups) measureGroupInfo = $.parseJSON(msg[9].Value);
                if (msg[10] != null && msg[10] != undefined)
                    oclientProxy.model.customObject = msg[10].Value;
            }
            else if (msg.d != undefined) {
                cubes = msg.d[0].Value; columnElements = msg.d[1].Value; rowElements = msg.d[2].Value;
                slicerElements = msg.d[3].Value; cubeTreeInfo = msg.d[4].Value; oclientProxy.currentReport = msg.d[5].Value;
                oclientProxy.reports = msg.d[6].Value; reportsCount = msg.d[7].Value; reportList = $.parseJSON(msg.d[8].Value);
                if (oclientProxy.model.enableMeasureGroups) measureGroupInfo = $.parseJSON(msg.d[9].Value);
                if (msg.d[10] != null && msg.d[10] != undefined)
                    oclientProxy.model.customObject = msg.d[10].Value;
            }
            else {
                cubes = msg.Cubes; columnElements = msg.Columns; rowElements = msg.Rows;
                slicerElements = msg.Slicers; cubeTreeInfo = msg.CubeTreeInfo; oclientProxy.currentReport = msg.CurrentReport;
                oclientProxy.reports = msg.ClientReports; reportList = $.parseJSON(msg.ReportList); reportsCount = msg.ClientReportsCount;
                if (oclientProxy.model.enableMeasureGroups) measureGroupInfo = $.parseJSON(msg.MeasureGroups);
                if (msg.customObject != null && msg.customObject != undefined)
                    oclientProxy.model.customObject = msg.customObject;
            }
            if (this.model.afterServiceInvoke != null)
                this._trigger("afterServiceInvoke", { action: "initialize", element: this.element, customObject: this.model.customObject });
            
            if (oclientProxy.model.progressMode != "infinite") {
                this._progressStatus(89, 101 + ":100%", 10);
                window.setTimeout(function () { progressStatus() }, 300);
            }
            else
                window.setTimeout(function () { progressStatus() }, 1);
            function progressStatus() {
                if (oclientProxy.model.progressMode == "progress")
                    $("#" + oclientProxy._id + "progressText")[0].innerHTML = oclientProxy._getLocalizedLabels("RenderingLayoutStarted") + "...";
                var treeViewData = $.parseJSON(cubeTreeInfo);
                oclientProxy.currentCubeName = $.parseJSON(cubes)[0].name;
                var reportBar = ej.buildTag("div#reportToolbar.reportToolbar", oclientProxy._reportToolbar(), { width: "410px", height: "29px" })[0].outerHTML;
                var browserPanel = oclientProxy._createCubeSelector() + "<table class=cubeTable><tr><td valign=\"bottom\">" + oclientProxy._createCubeBrowser(cubeTreeInfo) + "</td><td valign=\"bottom\" style='padding-left:5px;'>" + oclientProxy._createAxisElementBuilder(columnElements, rowElements, slicerElements) + "</td></tr></table>";
                var htmlTag = ej.buildTag("span.outerPanel", "", { width: "1000px", height: "600px" }).append("<div class=\"titleText\"><span style='padding-left:10px'>" + oclientProxy.title() + "</span></div>" + reportBar + "<table class=\"outerTable\"><tr><td>" + browserPanel + "</td><td class=\"controlPanelTD\">" + oclientProxy._controlPanel() + "</td></tr></table>");
                oclientProxy.element.html(htmlTag).width($(".outerPanel").width());
                oclientProxy.element.find(".reportToolbar").ejToolbar();
                if ((oclientProxy.controlPlacement() == ej.olap.OlapClient.ControlPlacement.Tab && oclientProxy.defaultView() == ej.olap.OlapClient.DefaultView.Grid) || (oclientProxy.displayMode() == ej.olap.OlapClient.DisplayMode.GridOnly))
                    oclientProxy.element.find(".chartTypesImg").addClass("chartTypesOnGridView");
                var reportDropDown = "<div class ='reportList' ><input type='text' id='reportList' class='reportlist' /></div>";
                if (oclientProxy.model.enableMeasureGroups) {
                    var measureGroupDropdown = "<div class ='measureGroupselector' style='margin:5px 5px 0px 5px'><input type='text' id='measureGroupSelector' class='measureGroupSelector' /></div>";
                    $(measureGroupDropdown).prependTo(".cubeBrowser");
                    oclientProxy.element.find(".measureGroupSelector").ejDropDownList({
                        dataSource: measureGroupInfo,
                        fields: { text: "name", value: "name" },
                        height: "25px",
                        width: "100%"
                    });
                    oclientProxy.element.find(".measureGroupSelector").attr("tabindex", 0);
                }
                $(reportDropDown).appendTo(".reportToolbar");
                oclientProxy.element.find(".reportlist").ejDropDownList({
                    dataSource: reportList,
                    fields: { text: "name", value: "name" },
                    height: "20px"
                });
                oclientProxy.element.find(".reportlist").attr("tabindex", 0);
                var tempddlwidth = oclientProxy.element.find(".csHeader").width() - oclientProxy.element.find(".cubeText").width()- oclientProxy.element.find(".toggleExpandButton").width()-20;
                var cubeddlWidth = oclientProxy.enableTogglePanel() ? tempddlwidth - 25 : tempddlwidth;
                oclientProxy.element.find(".cubeSelector").ejDropDownList({
                    dataSource: $.parseJSON(cubes),
                    fields: { text: "name", value: "name" },
                    width: ""+cubeddlWidth+"px"
                });
                ddlTarget = oclientProxy.element.find('.cubeSelector').data("ejDropDownList"); 
                reportDropTarget = oclientProxy.element.find('#reportList').data("ejDropDownList");
                if (oclientProxy.model.enableMeasureGroups)
                    measureDropTarget = oclientProxy.element.find('#measureGroupSelector').data("ejDropDownList");
                ddlTarget.setSelectedText(ddlTarget.model.dataSource[0].name); 
                reportDropTarget.setSelectedText(reportDropTarget.model.dataSource[0].name);
                if (oclientProxy.model.enableMeasureGroups)
                    measureDropTarget.setSelectedText(measureDropTarget.model.dataSource[0].name);
                oclientProxy.element.find(".cubeSelector").ejDropDownList("option", "change", ej.proxy(oclientProxy.cubeChanged, oclientProxy));
                oclientProxy.element.find(".cubeSelector").attr("tabindex", 0);
                oclientProxy.element.find(".reportlist").ejDropDownList("option", "change", ej.proxy(oclientProxy.reportChanged, oclientProxy));
                if (oclientProxy.model.enableMeasureGroups)
                    oclientProxy.element.find(".measureGroupSelector").ejDropDownList("option", "change", ej.proxy(oclientProxy.measureGroupChanged, oclientProxy));
                oclientProxy.element.find(".cubeTreeView").ejTreeView({
                    fields: { id: "id", parentId: "pid", text: "name", spriteCssClass: "spriteCssClass", dataSource: treeViewData },
                    allowDragAndDrop: true,
                    allowDropChild: false,
                    allowDropSibling: false,
                    dragAndDropAcrossControl: true,
                    nodeDragStart: function () {
                        isDragging = true;
                    },
                    nodeDropped: ej.proxy(oclientProxy.nodeDropped, oclientProxy),
                    height: "466px"
                });
                oclientProxy.element.find(".cubeTreeView").attr("tabindex", 0);
                var treeViewElements = oclientProxy.element.find(".cubeTreeView").find("li");
                for (var i = 0; i < treeViewElements.length; i++) {
                    treeViewElements[i].setAttribute("tag", treeViewData[i].tag);
                }
                var cubeName = ej.buildTag("div.cubeName", $.parseJSON(cubes)[0].name)[0].outerHTML;
                $(cubeName).prependTo(".cubeBrowser");                
                oclientProxy.element.find(".categoricalAxis, .rowAxis, .slicerAxis").ejDroppable({ drop: ej.proxy(oclientProxy.onDropped, oclientProxy) });
                oclientProxy.element.find(".categoricalAxis, .rowAxis, .slicerAxis").find("button").ejButton({ height: "20px" });
                if (oclientProxy.enableTogglePanel()) {
                    $(ej.buildTag("div.toggleExpandButton e-icon")[0].outerHTML).appendTo(oclientProxy.element.find(".csHeader"));
                    $(oclientProxy.element.find(".outerTable").find("td")[0]).append(ej.buildTag("div.toggleCollapseButton e-icon")[0].outerHTML);
                    $((ej.buildTag("div.toggleText")[0].outerHTML)).insertAfter(oclientProxy.element.find(".toggleCollapseButton"));
                    oclientProxy.element.find(".toggleCollapseButton").hide();
                    oclientProxy.element.find(".toggleText").hide();
                }
                oclientProxy.element.find("#clientTab").ejTab({ itemActive: ej.proxy(oclientProxy.onTabClick, oclientProxy) });
                if (oclientProxy.defaultView() == ej.olap.OlapClient.DefaultView.Chart) {
                    oclientProxy.element.find("#OlapChart").ejOlapChart({ url: oclientProxy.model.url, customObject: oclientProxy.currentReport, locale: oclientProxy.locale(), showTooltip: true, size: { height: oclientProxy._chartHeight, width: oclientProxy._chartWidth }, commonSeriesOptions: { type: oclientProxy.model.chartType, tooltip: { visible: true } }, beforeServiceInvoke: oclientProxy.model.chartLoad, drillSuccess: ej.proxy(oclientProxy.chartDrillSuccess, oclientProxy) });
                    if (oclientProxy.gridLayout() != ej.olap.OlapGrid.Layout.Normal)
                        oclientProxy.element.find("#OlapGrid").ejOlapGrid({ url: oclientProxy.model.url, olapReport: oclientProxy.currentReport, layout: oclientProxy.gridLayout(), drillSuccess: ej.proxy(oclientProxy.gridDrillSuccess, oclientProxy) });
                    else
                        oclientProxy.element.find("#OlapGrid").ejOlapGrid({ url: oclientProxy.model.url, olapReport: oclientProxy.currentReport, drillSuccess: ej.proxy(oclientProxy.gridDrillSuccess, oclientProxy) });
                }
                else {
                    if (oclientProxy.gridLayout() != ej.olap.OlapGrid.Layout.Normal)
                        oclientProxy.element.find("#OlapGrid").ejOlapGrid({ url: oclientProxy.model.url, olapReport: oclientProxy.currentReport, layout: oclientProxy.gridLayout(), drillSuccess: ej.proxy(oclientProxy.gridDrillSuccess, oclientProxy) });
                    else
                        oclientProxy.element.find("#OlapGrid").ejOlapGrid({ url: oclientProxy.model.url, olapReport: oclientProxy.currentReport, drillSuccess: ej.proxy(oclientProxy.gridDrillSuccess, oclientProxy) });
                    oclientProxy.element.find("#OlapChart").ejOlapChart({ url: oclientProxy.model.url, customObject: oclientProxy.currentReport, locale: oclientProxy.locale(), showTooltip: true, size: { height: oclientProxy._chartHeight, width: oclientProxy._chartWidth }, commonSeriesOptions: { type: oclientProxy.model.chartType, tooltip: { visible: true } }, drillSuccess: ej.proxy(oclientProxy.chartDrillSuccess, oclientProxy), beforeServiceInvoke: oclientProxy.model.chartLoad });
                }
                if (oclientProxy.displayMode() != "chartonly")
                    oclientProxy.ogridObj = oclientProxy.element.find('#OlapGrid').data("ejOlapGrid");
                if (oclientProxy.displayMode() != "gridOnly")
                    oclientProxy.ochartObj = oclientProxy.element.find('#OlapChart').data("ejOlapChart");
                var items = {};
                if (oclientProxy.ochartObj != null) {
                    oclientProxy.element.find('#OlapChart').width(oclientProxy.ochartObj.model.size.width);
                    items["chartModelWidth"] = oclientProxy.ochartObj.model.size.width;
                }
                items["controlPanelWidth"] = oclientProxy.element.find(".controlPanel").width();
                items["chartOuterWidth"] = oclientProxy._chartWidth;
                items["gridOuterWidth"] = oclientProxy._gridWidth;
                oclientProxy._initStyles.push(items);
                oclientProxy._wireEvents();
                if (oclientProxy.model.progressMode != "infinite") {
                    oclientProxy._progressStatus(100, 101, 10);
                    oclientProxy._progressHide();
                    $("#" + oclientProxy._id + "progressContainer").css({ top: ($("#" + oclientProxy._id).height() / 2) - 30, left: (($("#" + oclientProxy._id).width() / 2)), position: 'absolute' });
                    $("#" + oclientProxy._id + "blockDiv").css({ height: $("#" + oclientProxy._id).height(), width: $("#" + oclientProxy._id).width() });
                }
            }
            if (oclientProxy.model.progressMode == "progress")
                $("#" + oclientProxy._id + "progressText")[0].innerHTML = oclientProxy._getLocalizedLabels("RenderingSucceeded") + "...";
            oclientProxy._successAction = "ClientRender";
            oclientProxy._trigger("renderSuccess", oclientProxy);
            oclientProxy.progressPos = $("#" + oclientProxy._id).position();
        },

        _measureGroupChangedSuccess: function (cubeTree) {
            var cubeTreeInfo;
            if (cubeTree[0] != undefined) {
                cubeTreeInfo = cubeTree[0].Value;
            }
            if (cubeTree.d != undefined) {
                cubeTreeInfo = cubeTree.d[0].Value;
            }
            else
                cubeTreeInfo = cubeTree.CubeTreeInfo;
            this.element.find(".e-treeview, .cubeTreeView").remove();
            var cubeTree = ej.buildTag("div#cubeTreeView.cubeTreeView")[0].outerHTML;
            this.element.find(".cubeBrowser").append(cubeTree);
            var treeViewData = $.parseJSON(cubeTreeInfo);
            this.element.find(".cubeTreeView").ejTreeView({
                fields: { id: "id", parentId: "pid", text: "name", spriteCssClass: "spriteCssClass", dataSource: treeViewData },
                allowDragAndDrop: true,
                allowDropChild: false,
                allowDropSibling: false,
                dragAndDropAcrossControl: true,
                nodeDropped: ej.proxy(this.nodeDropped, this),
                height: "466px"
            });
            var treeViewElements = oclientProxy.element.find(".cubeTreeView").find("li");
            for (var i = 0; i < treeViewElements.length; i++) {
                treeViewElements[i].setAttribute("tag", treeViewData[i].tag);
            }
            if (oclientProxy.model.progressMode != "infinite") {
                if (oclientProxy.model.progressMode == "progress")
                    $("#" + oclientProxy._id + "progressText")[0].innerHTML = oclientProxy._getLocalizedLabels("RenderingSucceeded") + "...";
                this._progressStatus(90, 101 + ":100%", 20);
                window.setTimeout(function () { progressStatus() }, 300);
            }
            else
                window.setTimeout(function () { progressStatus() }, 1);
            function progressStatus() {
                oclientProxy._unWireEvents();
                oclientProxy._wireEvents();
                if (oclientProxy.model.progressMode != "infinite")
                    oclientProxy._progressHide();
                else
                    oclientWaitingPopup.hide();
            }
            oclientProxy._successAction = "MeasureGroupChange";
        },

        /**
         * 	This function receives the control's update from service-end after changing the cube.
         *
         * @private  
         */
        _cubeChangedSuccess: function (report) {
             
                 
            if (oclientProxy.model.progressMode != "infinite") {
                if (oclientProxy.model.progressMode == "progress")
                    $("#" + oclientProxy._id + "progressText")[0].innerHTML = oclientProxy._getLocalizedLabels("RenderingLayoutCompleted")+"...";
                window.clearInterval(oClientTimer);
                oclientProgressBar.setModel({ value: 83, text: 83 + " %" });
            }
            var cubeTreeInfo, measureGroupInfo, chartSettings;
            if (oclientProxy.model.progressMode == "progress")
                $("#" + oclientProxy._id + "progressText")[0].innerHTML = oclientProxy._getLocalizedLabels("RenderingStarted")+"...";
            if (report[0] != undefined) {
                oclientProxy.currentReport = report[0].Value; cubeTreeInfo = report[1].Value; oclientProxy.reports = report[2].Value;
                reportsCount = report[3].Value; reportList = $.parseJSON(report[4].Value);
                if (oclientProxy.model.enableMeasureGroups) measureGroupInfo = $.parseJSON(report[5].Value);
                chartSettings = $.parseJSON(report[6].Value);
                if (report[7] != null && report[7] != undefined)
                    oclientProxy.model.customObject = report[7].Value;
            }
            else if (report.d != undefined) {
                oclientProxy.currentReport = report.d[0].Value; cubeTreeInfo = report.d[1].Value; oclientProxy.reports = report.d[2].Value;
                reportsCount = report.d[3].Value; reportList = $.parseJSON(report.d[4].Value);
                if (oclientProxy.model.enableMeasureGroups) measureGroupInfo = $.parseJSON(report.d[5].Value);
                chartSettings = $.parseJSON(report.d[6].Value); 
                if (report.d[7] != null && report.d[7] != undefined)
                    oclientProxy.model.customObject = report.d[7].Value;
            }
            else {
                 
                oclientProxy.currentReport = report.NewReport; cubeTreeInfo = report.CubeTreeInfo; oclientProxy.reports = report.ClientReports;
                reportsCount = report.ReportsCount; reportList = $.parseJSON(report.ReportList);
                if (oclientProxy.model.enableMeasureGroups) measureGroupInfo = $.parseJSON(report.MeasureGroups);
                chartSettings = $.parseJSON(report.ControlSettings);
                if (report.customObject != null && report.customObject != undefined)
                    oclientProxy.model.customObject = report.customObject;
            }
            oclientProxy.model.chartType = chartSettings.ChartType.toLowerCase();
            if (this.model.afterServiceInvoke != null)
                this._trigger("afterServiceInvoke", { action: "cubeChanged", element: this.element, customObject: this.model.customObject });
            this.element.find(".splitBtn, .e-olapgrid, .e-olapchart, .e-treeview, .cubeTreeView, .cubeName, .measureGroupselector").remove();
            var cubeTree = ej.buildTag("div#cubeTreeView.cubeTreeView")[0].outerHTML;
            var cubeName = ej.buildTag("div.cubeName", this.currentCubeName)[0].outerHTML;
            if (this.model.enableMeasureGroups) {
                var measureGroupDropdown = "<div class ='measureGroupselector' style='margin:5px 5px 0px 5px'><input type='text' id='measureGroupSelector' class='measureGroupSelector' /></div>";
                $(measureGroupDropdown).prependTo(".cubeBrowser");
                this.element.find(".measureGroupSelector").ejDropDownList({
                    dataSource: measureGroupInfo,
                    fields: { text: "name", value: "name" },
                    height: "25px",
                    width: "100%"
                });
                this.element.find(".measureGroupSelector").attr("tabindex", 0);
                measureDropTarget = oclientProxy.element.find('#measureGroupSelector').data("ejDropDownList");
                measureDropTarget.setSelectedText(measureDropTarget.model.dataSource[0].name);
                this.element.find(".measureGroupSelector").ejDropDownList("option", "change", ej.proxy(oclientProxy.measureGroupChanged, oclientProxy));
            }            
            var treeViewData = $.parseJSON(cubeTreeInfo);
            $(cubeTree).appendTo(".cubeBrowser");
            this.element.find(".cubeTreeView").ejTreeView({
                fields: { id: "id", parentId: "pid", text: "name", spriteCssClass: "spriteCssClass", dataSource: treeViewData },
                allowDragAndDrop: true,
                allowDropChild: false,
                allowDropSibling: false,
                dragAndDropAcrossControl: true,
                nodeDropped: ej.proxy(this.nodeDropped, this),
                height: "466px"
            });
            var treeViewElements = oclientProxy.element.find(".cubeTreeView").find("li");
            for (var i = 0; i < treeViewElements.length; i++) {
                treeViewElements[i].setAttribute("tag", treeViewData[i].tag);
            }
            this.element.find(".reportlist").ejDropDownList("option", "change", "");
            this.element.find(".reportlist").ejDropDownList("option", "dataSource", reportList);
            reportDropTarget.setSelectedText(reportDropTarget.model.dataSource[reportList.length - 1].name);
            if (oclientProxy.model.progressMode != "infinite") {
                if (oclientProxy.model.progressMode == "progress")
                    $("#" + oclientProxy._id + "progressText")[0].innerHTML = oclientProxy._getLocalizedLabels("RenderingSucceeded") + "...";
                this._progressStatus(90, 101 + ":100%", 20);
                window.setTimeout(function () { progressStatus() }, 300);
            }
            else
                window.setTimeout(function () { progressStatus() }, 1);
            function progressStatus() {
                $(cubeName).prependTo(".cubeBrowser");
                oclientProxy._unWireEvents();
                oclientProxy._wireEvents();
                if (oclientProxy.model.progressMode != "infinite")
                    oclientProxy._progressHide();
                else
                    oclientWaitingPopup.hide();
            }
            oclientProxy._successAction = "CubeChange";
            oclientProxy._trigger("renderSuccess", oclientProxy);
        },

        /**
         * 	This function receives the control's update from service-end after removing Split Button.
         *
         * @private  
         */
        _removeSplitButtonSuccess: function (report) {
             
            if (oclientProxy.model.progressMode != "infinite") {
                if (oclientProxy.model.progressMode == "progress")
                    $("#" + oclientProxy._id + "progressText")[0].innerHTML = oclientProxy._getLocalizedLabels("MDXqueryExecutedSuccessfully")+"...";
                window.clearInterval(oClientTimer);
                oclientProgressBar.setModel({ value: 89, text: 89 + " %" });
            }
            if (report[0] != undefined) {
                oclientProxy.currentReport = report[0].Value; oclientProxy.reports = report[1].Value;
                if (report[2] != null && report[2] != undefined)
                    oclientProxy.model.customObject = report[2].Value;
            }
            else if (report.d != undefined) {
                oclientProxy.currentReport = report.d[0].Value; oclientProxy.reports = report.d[1].Value;
                if (report.d[2] != null && report.d[2] != undefined)
                    oclientProxy.model.customObject = report.d[2].Value;
            }
            else {
                oclientProxy.currentReport = report.UpdatedReport; oclientProxy.reports = report.ClientReports;
                if (report.customObject != null && report.customObject != undefined)
                    oclientProxy.model.customObject = report.customObject;
            }
            if (this.model.afterServiceInvoke != null)
                this._trigger("afterServiceInvoke", { action: "removeSplitButton", element: this.element, customObject: this.model.customObject });
            if (oclientProxy.model.progressMode != "infinite") {
                if (oclientProxy.model.progressMode == "progress")
                    $("#" + oclientProxy._id + "progressText")[0].innerHTML = oclientProxy._getLocalizedLabels("RenderingSucceeded");
                this._progressStatus(95, 101 + ":100%", 50);
                window.setTimeout(function () { progressStatus() }, 300);
            }
            else
                window.setTimeout(function () { progressStatus() }, 1);
            function progressStatus() {
                oclientProxy._renderControls();
                oclientProxy._unWireEvents();
                oclientProxy._wireEvents();
                if (oclientProxy.model.progressMode != "infinite")
                    oclientProxy._progressHide();
            }
            oclientProxy._successAction = "ButtonRemove";
            oclientProxy._trigger("renderSuccess", oclientProxy);
        },

        _chartTypeChangedSuccess: function (report) {
            if (report[0] != undefined) {
                this.currentReport = report[0].Value;
                this.reports = report[1].Value;
            }
            else if (report.d != undefined) {
                this.currentReport = report.d[0].Value;
                this.reports = report.d[1].Value;
            }
            else {
                this.currentReport = report.CurrentReport;
                this.reports = report.ClientReports;
            }
          
        },

        /**
         * 	This function receives the control's update from service-end which would be utilized after performing any toolbar operation.
         *
         * @private  
         */
        _toolbarOperationSuccess: function (report) {
            if (oclientProxy.model.progressMode != "infinite") {
                if (oclientProxy.model.progressMode == "progress")
                    $("#" + oclientProxy._id + "progressText")[0].innerHTML = oclientProxy._getLocalizedLabels("OpeningDialogSucceeded") + "...";
                window.clearInterval(oClientTimer);
            }
            if (oclientProxy.model.progressMode == "progress")
                $("#" + oclientProxy._id + "progressText")[0].innerHTML = "Rendering started";
            if (report != null)
                if((report.d != undefined && report.d || report[0] != undefined && report[0])||report.CurrentReport) {
                var columnElements, rowElements, slicerElements, action, renamedReport, chartSettings;
                if (report.length > 1 && report[0] != undefined) {
                    oclientProxy.currentReport = report[0].Value; oclientProxy.reports = report[1].Value; reportsCount = report[2].Value; columnElements = report[3].Value;
                    rowElements = report[4].Value; slicerElements = report[5].Value; action = report[6].Value; reportList = $.parseJSON(report[7].Value);
                    renamedReport = report[8].Value; chartSettings = $.parseJSON(report[9].Value); 
                    if (report[10] != null && report[10] != undefined)
                        oclientProxy.model.customObject = report[10].Value;
                }
                else if (report.d != undefined) {
                    oclientProxy.currentReport = report.d[0].Value; oclientProxy.reports = report.d[1].Value; reportsCount = report.d[2].Value; columnElements = report.d[3].Value;
                    rowElements = report.d[4].Value; slicerElements = report.d[5].Value; action = report.d[6].Value; reportList = $.parseJSON(report.d[7].Value);
                    renamedReport = report.d[8].Value; chartSettings = $.parseJSON(report.d[9].Value); 
                    if (report.d[10] != null && report.d[10] != undefined)
                        oclientProxy.model.customObject = report.d[10].Value;
                }
                else {
                     
                    oclientProxy.currentReport = report.CurrentReport; reports = report.Reports; reportsCount = report.ReportsCount; columnElements = report.Columns;
                    rowElements = report.Rows; slicerElements = report.Slicers; action = report.CurrentAction; reportList = $.parseJSON(report.ReportList);
                    renamedReport = report.RenamedReport; chartSettings = $.parseJSON(report.ControlSettings);
                    if (report.customObject != null && report.customObject != undefined)
                        oclientProxy.model.customObject = report.customObject;
                }
                oclientProxy.model.chartType = chartSettings.ChartType.toLowerCase();
             
                if (this.model.afterServiceInvoke != null)
                    this._trigger("afterServiceInvoke", { action: "toolbarOperation", element: this.element, customObject: this.model.customObject });
                if (action != "Report Change") {
                    this.element.find(".reportlist").ejDropDownList("option", "change", "");
                    this.element.find(".reportlist").ejDropDownList("option", "dataSource", reportList);
                    if (action == "Load Report")
                        reportDropTarget.setSelectedText(reportDropTarget.model.dataSource[0].name);
                    else if (action != "Rename Report")
                        reportDropTarget.setSelectedText(reportDropTarget.model.dataSource[reportList.length - 1].name);
                    else
                        reportDropTarget.setSelectedText(renamedReport);
                    this.element.find(".reportlist").ejDropDownList("option", "change", ej.proxy(this.reportChanged, this));
                }
                if (action != "Rename Report")
                    this.element.find(".splitBtn, .e-olapgrid, .e-olapchart").remove();
                if (action == "Remove Report" || action == "Report Change" || action == "Load Report" || action == "SortOrFilter" || action == "Save Report") {
                    $(oclientProxy._createSplitButtons(columnElements, "Columns")).appendTo(".categoricalAxis");
                    $(oclientProxy._createSplitButtons(rowElements, "Rows")).appendTo(".rowAxis");
                    $(oclientProxy._createSplitButtons(slicerElements, "Slicers")).appendTo(".slicerAxis");
                    this.element.find(".categoricalAxis, .rowAxis, .slicerAxis").find("button").ejButton({ height: "20px" });
                    if (oclientProxy.model.progressMode == "progress")
                        $("#" + oclientProxy._id + "progressText")[0].innerHTML = oclientProxy._getLocalizedLabels("RenderingSucceeded");
                    oclientProxy._renderControls();
                    if (oclientProxy.model.progressMode != "infinite")
                        oclientProxy._progressHide();
                }
            }
            oclientProxy._unWireEvents();
            oclientProxy._wireEvents();
            if (oclientProxy.model.progressMode != "infinite")
                oclientProxy._progressHide();
            oclientProxy._successAction = "ToolbarOperation";
            oclientProxy._trigger("renderSuccess", oclientProxy);
        },

        /**
         * 	This function receives the control's update from service-end after fetching the child nodes.
         *
         * @private  
         */
        _fetchChildNodeSuccess: function (data) {
             
            var newDataSource; var parentNode;
            if (data.length > 1 && data[0] != undefined)
                newDataSource = JSON.parse(data[0].Value);
            else if (data.d != undefined)
                newDataSource = JSON.parse(data.d[0].Value);
            else
                newDataSource = JSON.parse(data.ChildNodes);
            if (data[0] != undefined) {
                if (data[2] != null && data[2] != undefined)
                    oclientProxy.model.customObject = data[2].Value;
            }
            else if (data.d != undefined) {
                if (data.d[2] != null && data.d[2] != undefined)
                    oclientProxy.model.customObject = data.d[2].Value;
            }
            else {
                if (data.customObject != null && data.customObject != undefined)
                    oclientProxy.model.customObject = data.customObject;
            }
            if (this.model.afterServiceInvoke != null)
                this._trigger("afterServiceInvoke", { action: "memberExpanded", element: this.element, customObject: this.model.customObject });
            var mapper = { id: "id", parentId: "pid", hasChild: "hasChildren", text: "name", isChecked: "checkedStatus" };
            if ($(this.pNode).parents("li").length > 1)
                parentNode = $(this.pNode).parents("li").first();
            else
                parentNode = $(this.pNode).parents("li");
            $($(parentNode).find('input.nodecheckbox')[0]).ejCheckBox({ checked: false })
            this.memberTreeObj._createSubNodesWhenLoadOnDemand(newDataSource, this.pNode, mapper);
            $.each($(parentNode).children().find("li"), function (index, value) {
                value.setAttribute("tag", newDataSource[index].tag);
            });
           // this.memberTreeObj._expandCollapseAction(this.pNode);
            oclientProxy._successAction = "FetchChildNode";
            oclientProxy._trigger("renderSuccess", oclientProxy);
        },

        /**
         * 	This function is used to build the toolbar icons.
         *
         * @private
         */
        _reportToolbar: function () {
            return ej.buildTag("ul", ej.buildTag("li.newReportImg e-icon", "", {}).attr({ "title": this._getLocalizedLabels("NewReport"), tabindex: 0 })[0].outerHTML +
            ej.buildTag("li.addReportImg e-icon", "", {}).attr({ "title": this._getLocalizedLabels("AddReport"), tabindex: 0 })[0].outerHTML +
            ej.buildTag("li.removeReportImg e-icon", "", {}).attr({ "title": this._getLocalizedLabels("RemoveReport"), tabindex: 0 })[0].outerHTML +
            ej.buildTag("li.renameReportImg e-icon", "", {}).attr({ "title": this._getLocalizedLabels("RenameReport"), tabindex: 0 })[0].outerHTML +
            ej.buildTag("li.saveReportImg e-icon", "", {}).attr({ "title": this._getLocalizedLabels("SaveReport"), tabindex: 0 })[0].outerHTML +
            ej.buildTag("li.loadReportImg e-icon", "", {}).attr({ "title": this._getLocalizedLabels("LoadReport"), tabindex: 0 })[0].outerHTML +
            ej.buildTag("li.excelExportImg e-icon", "", {}).attr({ "title": this._getLocalizedLabels("ExportToExcel"), tabindex: 0 })[0].outerHTML +
            (oclientProxy.model.displaySettings.enableFullScreen ? (ej.buildTag("li.maximizedView e-icon", "", {}).attr({ "title": this._getLocalizedLabels("FullScreen"), tabindex: 0 })[0].outerHTML) : "") +
			ej.buildTag("li.colSortFilterImg e-icon", "", {}).attr({ "title": this._getLocalizedLabels("SortOrFilterColumn"), tabindex: 0 })[0].outerHTML +
            ej.buildTag("li.rowSortFilterImg e-icon", "", {}).attr({ "title": this._getLocalizedLabels("SortOrFilterRow"), tabindex: 0 })[0].outerHTML +
            ej.buildTag("li.chartTypesImg e-icon", "", {}).attr({ "title": this._getLocalizedLabels("ChartTypes"), tabindex: 0 })[0].outerHTML);
        },

        /**
         * 	This function is used to create Cube Selector.
         *
         * @private  
         */
        _createCubeSelector: function () {
            return ej.buildTag("div.csHeader", ej.buildTag("span.cubeText", this._getLocalizedLabels("CubeSelector"), { "padding-left": "6px", "float": "left" })[0].outerHTML + ej.buildTag("div", "<input type='text' id='cubeSelector' class ='cubeSelector' />", { display: "inline", "float": "left", "padding-left": "5px", "position": "relative", "top": "-3px" })[0].outerHTML, { width: "415px", height: "20px" })[0].outerHTML;
        },

        /**
         * 	This function is used to create Cube Dimension Browser.
         *
         * @private  
         */
        _createCubeBrowser: function (cubeTreeInfo) {
            return ej.buildTag("div.cdbHeader", ej.buildTag("span", this._getLocalizedLabels("CubeDimensionBrowser"), { "padding-left": "5px" })[0].outerHTML, { width: "200px", height: "30px" })[0].outerHTML + ej.buildTag("div.cubeBrowser", ej.buildTag("div.cubeTreeView.visibleHide")[0].outerHTML, { width: "200px", height: "523px" })[0].outerHTML;
        },

        /**
         * 	This function is used to create Axis Element Builder.
         *
         * @private  
         */
        _createAxisElementBuilder: function (columnDimensions, rowDimensions, slicerDimensions) {
            var AEBPanel = ej.buildTag("div.categoricalAxis", this._createSplitButtons(columnDimensions, "Columns"), { width: "200px", height: "150px" })[0].outerHTML + ej.buildTag("div.axisHeader", ej.buildTag("span", this._getLocalizedLabels("Column"), { "padding-left": "5px" })[0].outerHTML, { height: "30px" })[0].outerHTML +
            ej.buildTag("div.rowAxis", this._createSplitButtons(rowDimensions, "Rows"), { width: "200px", height: "150px" })[0].outerHTML + ej.buildTag("div.axisHeader", ej.buildTag("span", this._getLocalizedLabels("Row"), { "padding-left": "5px" })[0].outerHTML, { height: "30px" })[0].outerHTML +
            ej.buildTag("div.slicerAxis", this._createSplitButtons(slicerDimensions, "Slicers"), { width: "200px", height: "150px" })[0].outerHTML + ej.buildTag("div.axisHeader", ej.buildTag("span", this._getLocalizedLabels("Slicer"), { "padding-left": "5px" })[0].outerHTML, { height: "30px" })[0].outerHTML;
            return AEBPanel;
        },

        /**
         * 	This function is used to create the Split Button(s).
         *
         * @private  
         */
        _createSplitButtons: function (dimensionElements, axis) {
            var splitButtons = '';
            var captionName = '';
            if (dimensionElements.indexOf("~") > -1)
                for (var i = 0; i < dimensionElements.split('~')[0].split('#').length - 1; i++) {
                    captionName = dimensionElements.split('~')[0].split('#')[i + 1].split('.')[0];
                    if (captionName == "Measures") {
                        captionName = oclientProxy._getLocalizedLabels("Measures");
                    }
                    else if (!(dimensionElements.startsWith("#Measure")))
                        captionName = dimensionElements.split('~')[i + 2];
                    else if ((dimensionElements.indexOf("#Measure")) > -1)
                        captionName = dimensionElements.split('~')[i + 1];
                    if (captionName == undefined)
                        captionName = dimensionElements.split('~')[i + 1];
                       splitButtons += ej.buildTag("div.splitBtn", ej.buildTag("button", captionName).attr({ "title": dimensionElements.split('~')[0].split('#')[i + 1].replace('.', ' - ') })[0].outerHTML + ej.buildTag("span.removeSplitBtn e-icon")[0].outerHTML, "", { "tag": axis + ':' + dimensionElements.split('~')[0].split('#')[i + 1] })[0].outerHTML;
                }
            else
                for (var i = 0; i < dimensionElements.split('#').length - 1; i++) {
                    var caption = dimensionElements.split('#')[i + 1].split('.')[0] == "Measures" ? oclientProxy._getLocalizedLabels("Measures") : dimensionElements.split('#')[i + 1].split('.')[0];
                    if (caption.split("::")[1] == "NAMEDSET") {
                        caption = caption.split("::")[0];
                    }
                    splitButtons += ej.buildTag("div.splitBtn", ej.buildTag("button", caption).attr({ "title": dimensionElements.split('#')[i + 1].replace('.', ' - ') })[0].outerHTML + ej.buildTag("span.removeSplitBtn e-icon")[0].outerHTML, "", { "tag": axis + ':' + dimensionElements.split('#')[i + 1] })[0].outerHTML;
                }
            return splitButtons;
        },

        /**
         * This function renders and returns the outer panel of OLAP Client control.
         *
         * @private
         */
        _controlPanel: function () {
            var controlTable; var gridDiv
            if (this.controlPlacement() == ej.olap.OlapClient.ControlPlacement.Tab && (this.displayMode() == ej.olap.OlapClient.DisplayMode.ChartAndGrid || this.displayMode() == "")) {
                this._chartHeight = "547px";
                this._chartWidth = "557px";
                this._gridHeight = "530px";
                this._gridWidth = 555;
                this._panelOverflow = "auto";
               
            }
            else if (this.controlPlacement() == ej.olap.OlapClient.ControlPlacement.Tile && (this.displayMode() == ej.olap.OlapClient.DisplayMode.ChartAndGrid || this.displayMode() == "")) {
                this._chartHeight = "275px";
                this._chartWidth = "555px";
                this._gridHeight = "530px";
                this._gridWidth = 555;
                this._panelOverflow = "hidden";
            }
            else {
                this._chartHeight = "560px";
                this._chartWidth = "790px";
                this._gridHeight = "550px";
                this._gridWidth = 555;
                this._panelOverflow = "auto";
            }
            if (this.displayMode() != ej.olap.OlapClient.DisplayMode.ChartOnly) {
                if (this.controlPlacement() == ej.olap.OlapClient.ControlPlacement.Tile && this.displayMode() == ej.olap.OlapClient.DisplayMode.ChartAndGrid)
                    gridDiv = ej.buildTag("div#gridPanel.gridPanel", ej.buildTag("div#OlapGrid", "", { "margin": "5px 7px 7px", "max-height": "300px", "width": "563px", "overflow": "auto" })[0].outerHTML, { width: "auto", height: "auto" })[0].outerHTML;
                else
                    gridDiv = ej.buildTag("div#gridPanel.gridPanel", ej.buildTag("div#OlapGrid", "", { "margin": "5px 7px 7px", "min-height": "300px", "overflow": "auto", "max-height": this._gridHeight })[0].outerHTML, { width: "auto", height: "auto" })[0].outerHTML;
            }
            if (this.displayMode() != ej.olap.OlapClient.DisplayMode.GridOnly)
                var chartDiv = ej.buildTag("div#chartPanel.chartPanel", ej.buildTag("div#OlapChart", "", { "min-height": "275px" })[0].outerHTML, { width: "auto", height: "auto" })[0].outerHTML;
            if (this.controlPlacement() == ej.olap.OlapClient.ControlPlacement.Tab && this.displayMode() == ej.olap.OlapClient.DisplayMode.ChartAndGrid) {
                var tabUl; var tabDiv;
                var gridAnchor = "<a style='font: bold 12px Segoe UI' href='#gridPanel' tabindex='0'>" + this._getLocalizedLabels("Grid") + "</a>";
                var chartAnchor = "<a style='font: bold 12px Segoe UI' href='#chartPanel' tabindex='0'>" + this._getLocalizedLabels("Chart") + "</a>";
                if (this.defaultView() == ej.olap.OlapClient.DefaultView.Chart) {
                    tabUl = ej.buildTag("ul.clientTab", ej.buildTag("li", chartAnchor)[0].outerHTML + ej.buildTag("li", gridAnchor)[0].outerHTML)[0].outerHTML;
                    tabDiv = ej.buildTag("div#clientTab", tabUl + chartDiv + gridDiv)[0].outerHTML;
                }
                else {
                    tabUl = ej.buildTag("ul.clientTab", ej.buildTag("li", gridAnchor)[0].outerHTML + ej.buildTag("li", chartAnchor)[0].outerHTML)[0].outerHTML;
                    tabDiv = ej.buildTag("div#clientTab", tabUl + gridDiv + chartDiv)[0].outerHTML;
                }
                controlTable = tabDiv;
            }
            else {
                controlTable = this._createControlContainer(this.controlPlacement(), chartDiv, gridDiv);
            }

            return ej.buildTag("div.controlPanel", controlTable, { width: "572px", height: "600px", "overflow": this._panelOverflow })[0].outerHTML
        },

        /**
         * This function creates the HTML tag containing the control and returns it.
         *
         * @private  
         */
        _createControlContainer: function (controlPlacement, chartDiv, gridDiv) {
            var controlTable;
            if (this.displayMode() == ej.olap.OlapClient.DisplayMode.ChartOnly)
                controlTable = "<table><tr><td>" + chartDiv + "</td></tr></table>";
            else if (this.displayMode() == ej.olap.OlapClient.DisplayMode.GridOnly)
                controlTable = "<table><tr><td>" + gridDiv + "</td></tr></table>";
            else {
                if (this.controlPlacement() == ej.olap.OlapClient.ControlPlacement.Tile) {
                    if (this.defaultView() == ej.olap.OlapClient.DefaultView.Chart)
                        controlTable = "<table><tr><td>" + chartDiv + "</td></tr><tr><td>" + gridDiv + "</td></tr></table>";
                    else
                        controlTable = "<table><tr><td>" + gridDiv + "</td></tr><tr><td>" + chartDiv + "</td></tr></table>";
                }
                else {
                    if (this.defaultView() == ej.olap.OlapClient.DefaultView.Chart)
                        controlTable = "<table><tr><td valign='top'>" + chartDiv + "</td><td valign='top'>" + gridDiv + "</td></tr></table>";
                    else
                        controlTable = "<table><tr><td valign='top'>" + gridDiv + "</td><td valign='top'>" + chartDiv + "</td></tr></table>";
                }
            }
            return controlTable;
        },

        /**
         * 	This function is used to render the OLAP Grid component inside of OLAP Client component.
         *
         * @private  
         */
        _renderOlapGrid: function (tagInfo) {
                
            this.element.find(".e-dialog").hide();
            if (this.displayMode() != ej.olap.OlapClient.DisplayMode.ChartOnly) {
                var gridDiv;
                if (this.ochartObj != undefined && this.ochartObj._startDrilldown) {
                    var report;
                    try {
                        report = JSON.parse(this.ogridObj.getOlapReport()).Report;
                    }
                    catch (err) {
                        report = this.ogridObj.getOlapReport();
                    }
                    this.ogridObj.doAjaxPost("POST", this.model.url + "/" + this.ogridObj.model.serviceMethodSettings.drillDown, JSON.stringify({ "action": "drillDownGrid", "cellPosition": "", "currentReport": report,"clientReports": this.reports, "headerInfo": tagInfo }), this.ogridObj._drillDownSuccess);
                }
                else {
                    this.element.find(".e-olapgrid, #OlapGrid").remove();
                    if (this.controlPlacement() == ej.olap.OlapClient.ControlPlacement.Tile && this.displayMode() == ej.olap.OlapClient.DisplayMode.ChartAndGrid)
                        gridDiv = ej.buildTag("div#OlapGrid", "", { "margin": "5px 7px 7px", "max-height": "300px", "width": "563px", "overflow": "auto" })[0].outerHTML;
                    else
                        gridDiv = ej.buildTag("div#OlapGrid", "", { "margin": "5px 7px 7px", "min-height": "300px", "overflow": "auto", "max-height": this._gridHeight })[0].outerHTML;
                    $(gridDiv).appendTo(this.element.find(".gridPanel"));
                    if (this.gridLayout() != ej.olap.OlapGrid.Layout.Normal)
                        this.element.find("#OlapGrid").ejOlapGrid({ url: this.model.url, olapReport: this.currentReport, layout: this.gridLayout(), drillSuccess: ej.proxy(this.gridDrillSuccess, this) });
                    else
                        this.element.find("#OlapGrid").ejOlapGrid({ url: this.model.url, olapReport: this.currentReport, drillSuccess: ej.proxy(this.gridDrillSuccess, this) });
                    this.ogridObj = this.element.find('#OlapGrid').data("ejOlapGrid");
                }
            }
        },

        /**
         * 	This function is used to render the OLAP Chart component inside of OLAP Client component.
         *
         * @private  
         */
        _renderOlapChart: function (drillTag, drillAction) {
              
            
            this.element.find(".e-dialog").hide();
            if (this.displayMode() != ej.olap.OlapClient.DisplayMode.GridOnly) {
                if (this.ogridObj != undefined && this.ogridObj._startDrilldown) {
                    this.ochartObj.doAjaxPost("POST", this.model.url + "/" + this.ochartObj.model.serviceMethodSettings.drillDown, JSON.stringify({ "action": drillAction, "drilledSeries": drillTag, "olapReport": this.ochartObj.getOlapReport(),"clientReports": this.reports  }), this.ochartObj.renderControlSuccess);
                }
                else {
                    this.element.find(".e-olapchart").remove();
                    var chartDiv = ej.buildTag("div#OlapChart", "", { "min-height": "275px" })[0].outerHTML;
                    $(chartDiv).appendTo(this.element.find(".chartPanel"));
                    this.element.find("#OlapChart").ejOlapChart({ url: this.model.url, customObject: this.currentReport, locale: oclientProxy.locale(), showTooltip: true, size: { height: oclientProxy._chartHeight, width: oclientProxy._chartWidth }, commonSeriesOptions: { type: oclientProxy.model.chartType, tooltip: { visible: true } }, beforeServiceInvoke: oclientProxy.model.chartLoad, drillSuccess: ej.proxy(oclientProxy.chartDrillSuccess, oclientProxy) });
                    this.ochartObj = this.element.find('#OlapChart').data("ejOlapChart");
                    this.element.find('#OlapChart').width(this.ochartObj.model.size.width);
                     }
            }
        },

        /**
         * 	This function is used to render the OLAP Chart and OLAP Grid widgets.
         *
         * @private  
         */
        _renderControls: function () {
             
            if (this.defaultView() == ej.olap.OlapClient.DefaultView.Chart) {
                oclientProxy._renderOlapChart();
                oclientProxy._renderOlapGrid();
            }
            else {
                oclientProxy._renderOlapGrid();
                oclientProxy._renderOlapChart();
            }
        },

        /**
         * 	This function is used to initiate the dialog creation.
         *
         * @private  
         */
        _createDialogRequest: function (evt) {
             
            if (oclientProxy.model.progressMode != "infinite")
                this._progressStatus(0, 80, 50);
            args_className = evt.target.className;
            args_innerHTML = (evt.target.innerHTML == "" ? "ToolbarButtons" : evt.target.innerHTML);
            hierarchyCaption = (evt.currentTarget.attributes == "" ? "" : evt.currentTarget.attributes.getNamedItem("title").value);
            this.element.find(".e-dialog, .clientDialog").remove();
            if (args_innerHTML != "ToolbarButtons" && args_innerHTML != undefined) {
                if (oclientProxy.model.progressMode == "progress")
                    $("#" + oclientProxy._id + "progressText")[0].innerHTML = oclientProxy._getLocalizedLabels("OpeningDialog") + "...";
                else if (oclientProxy.model.progressMode == "infinite")
                    oclientWaitingPopup.show();
                if (this.model.beforeServiceInvoke != null)
                    this._trigger("beforeServiceInvoke", { action: "fetchMemberTreeNodes", element: this.element, customObject: this.model.customObject });
                var serializedCustomObject = JSON.stringify(this.model.customObject);
                this.doAjaxPost("POST", oclientProxy.model.url + "/" + oclientProxy.model.serviceMethodSettings.fetchMemberTreeNodes, JSON.stringify({ "action": "fetchMemberTreeNodes", "dimensionName": $(evt.target).parent().attr("Tag"), "olapReport": oclientProxy.currentReport, "customObject": serializedCustomObject }), oclientProxy._editorTreeInfoSuccess);
            }
            else {
                if (oclientProxy.model.progressMode != "infinite")
                    oclientProxy._progressHide();
                else
                    oclientProxy._createDialog(args_className, args_innerHTML, "");
            }
        },

        /**
         * 	This function receives the list of report names from the database.
         *
         * @private  
         */
        _fetchReportListSuccess: function (reportNameList) {
             
            if (reportNameList[0] != undefined) {
                if (reportNameList[2] != null && reportNameList[2] != undefined)
                    oclientProxy.model.customObject = reportNameList[2].Value;
            }
            else if (reportNameList.d != undefined) {
                if (reportNameList.d[2] != null && reportNameList.d[2] != undefined)
                    oclientProxy.model.customObject = reportNameList.d[2].Value;
            }
            else {
                if (reportNameList.customObject != null && reportNameList.customObject != undefined)
                    oclientProxy.model.customObject = reportNameList.customObject;
            }
            if (this.model.afterServiceInvoke != null)
                this._trigger("afterServiceInvoke", { action: "fetchReportList", element: this.element, customObject: this.model.customObject });
            if (oclientProxy.model.progressMode != "infinite") {
                if (oclientProxy.model.progressMode == "progress")
                    $("#" + oclientProxy._id + "progressText")[0].innerHTML = oclientProxy._getLocalizedLabels("OpeningDialogSucceeded") + "...";
                window.clearInterval(oClientTimer);
                this._progressStatus(90, 101 + ":100%", 50);
                window.setTimeout(function () { progressStatus() }, 300);
            }
            else {
                oclientWaitingPopup.hide();
                window.setTimeout(function () { progressStatus() }, 1);
            }
            if (this.model.afterServiceInvoke != null)
                this._trigger("afterServiceInvoke", { action: "fetchReportList", element: this.element, customObject: this.model.customObject });
            function progressStatus() {
                oclientProxy.element.find(".e-dialog, .clientDialog").remove();
                if (reportNameList.d != undefined)
                    reportNameList = (reportNameList.d[0].Value || reportNameList[0].Value);
                else
                    reportNameList = reportNameList.ReportNameList;
                oclientProxy._createDialog("LoadReport", reportNameList, "");
            }
        },

        /**
         * 	This function is used to maintain the sorting state inside the sort dialog.
         *
         * @private 
         */
        _fetchSortState: function (value) {
             
            if (oclientProxy.model.progressMode != "infinite") {
                if (oclientProxy.model.progressMode == "progress")
                    $("#" + oclientProxy._id + "progressText")[0].innerHTML = oclientProxy._getLocalizedLabels("OpeningDialogSucceeded");
                window.clearInterval(oClientTimer);
                this._progressStatus(90, 101 + ":100%", 50);
                window.setTimeout(function () { progressStatus() }, 300);
            }
            else {
                oclientWaitingPopup.hide();
                window.setTimeout(function () { progressStatus() }, 1);
            }
            function progressStatus() {
                if (value[0] != undefined) {
                    oclientProxy._createDialog("SortFilterDlg", value[0].Value, "");
                }
                else if (value.d != undefined) {
                    oclientProxy._createDialog("SortFilterDlg", value.d[0].Value, "");
                }
                else {
                    oclientProxy._createDialog("SortFilterDlg",value.FetchSortState, "");
                }

            }
        },

        /**
         * 	This function is used to create the toolbar, member editor and measure editor dialogs
         *
         * @private
         */
        _createDialog: function (args_className, args_innerHTML, editorTree) {
              
            var sortDlgField = "";
            var sortState = "", filterState = "";
            if (this.model.progressMode != "infinite")
                window.clearInterval(oClientTimer);
            var editorTreeInfo;
            if (editorTree[0] != undefined)
                editorTreeInfo = editorTree[0].Value;
            else if (editorTree.d != undefined)
                editorTreeInfo = editorTree.d[0].Value;
            else
                editorTreeInfo = editorTree.EditorTreeInfo;
            var ejDialog = ej.buildTag("div#clientDialog.clientDialog", { "opacity": "1" })[0].outerHTML;
            var dialogContent; var dialogTitle; var currentTag; var searchText;

            if (args_className.split(" ")[0] == "newReportImg") {
                dialogTitle = this._getLocalizedLabels("NewReport");
                currentTag = "New Report";
                var newReportLabel = "<p class='dialogPara'>" + this._getLocalizedLabels("ReportName") + "</p>";
                var newReport = "<input type=text id=reportName class='reportName'/></br>";
                dialogContent = newReportLabel + newReport;
            }
            else if (args_className.split(" ")[0] == "addReportImg") {
                dialogTitle = this._getLocalizedLabels("AddReport");
                currentTag = "Add Report";
                var addReportLabel = "<p class='dialogPara'>" + this._getLocalizedLabels("ReportName") + "</p>";
                var addReport = "<input type=text id=reportName class='reportName'/></br>";
                dialogContent = addReportLabel + addReport;
            }
            else if (args_className.split(" ")[0] == "removeReportImg") {
                dialogTitle = this._getLocalizedLabels("RemoveReport");
                currentTag = "Remove Report";
                if (reportsCount > 1)
                    dialogContent = "<p>" + this._getLocalizedLabels("AreYouSureToDeleteTheReport") + "?</p>";
                else
                    dialogContent = "<p>" + this._getLocalizedLabels("CannotRemoveSingleReport") + "!</p>";
            }
            else if (args_className.split(" ")[0] == "SortFilterDlg") {

                sortDlgField = args_innerHTML.split("||");
                if (sortDlgField[1] != "")
                    sortState = sortDlgField[1].split("<<");
                if (sortDlgField[2] != "")
                    filterState = sortDlgField[2].split("<<");
                oclientProxy.element.find(".e-dialog, .clientDialog").remove();
                dialogTitle = this._getLocalizedLabels("SortingAndFiltering");

                var sortingLbl = ej.buildTag("label#sortingLbl.sortingLbl", this._getLocalizedLabels("Sorting"))[0].outerHTML;
                var measureListLbl = ej.buildTag("label#measureListLbl.measureListLbl", this._getLocalizedLabels("Measure"))[0].outerHTML;
                var orderLbl = ej.buildTag("label#orderLbl.orderLbl", this._getLocalizedLabels("Order"))[0].outerHTML;
                var measureListDropDown = ej.buildTag("input#measuresList.measuresList","",{},{type:'text', accesskey:'M'})[0].outerHTML;

                var sortDisableRadio = ej.buildTag("input#sortDisable.sortDisable", "", {}, { name: "sort", type: "radio", checked: 'checked', role: 'radio', tabindex: 0, accesskey: 'i' })[0].outerHTML + ej.buildTag("label.sortDisableLbl",  this._getLocalizedLabels("Disable"))[0].outerHTML;
                var sortEnableRadio = ej.buildTag("input#sortEnable.sortEnable", "", {}, { name: 'sort', type: 'radio', role: 'radio', tabindex: 1, accesskey: 'n' })[0].outerHTML + ej.buildTag("label.sortEnableLbl",  this._getLocalizedLabels("Enable"))[0].outerHTML + "<br/>";
                var newascRadio = ej.buildTag("input#radioBtnAsc.radioBtnAsc", "", {}, { name: 'order', type: 'radio', checked: 'checked', role: 'radio', tabindex: 0, accesskey: 'A' })[0].outerHTML + ej.buildTag("label.radioBtnAscLbl",  this._getLocalizedLabels("Ascending"))[0].outerHTML + "<br/>";
                var newdescRadio = ej.buildTag("input#radioBtnDesc.radioBtnDesc", "", {}, { name: 'order', type: 'radio', role: 'radio', tabindex: 2, accesskey: 'e' })[0].outerHTML + ej.buildTag("label.radioBtnDescLbl",  this._getLocalizedLabels("Descending"))[0].outerHTML;
                var preservHierarchy = ej.buildTag("input#preserveHrchy.preserveHrchy", "", {}, { type: 'checkbox', checked: 'checked', tabindex: 0, role: 'radio', accesskey: 'r' })[0].outerHTML + ej.buildTag("label.preserveHrchyLbl", this._getLocalizedLabels("PreserveHierarchy"))[0].outerHTML;

                var filter = ej.buildTag("label#filterLbl.filterLbl", this._getLocalizedLabels("Filtering"))[0].outerHTML;
                var filterDisableRadio = ej.buildTag("input#filterDisable.filterDisable", "", {}, { name: 'filter', type: 'radio', checked: 'checked', role: 'radio', tabindex: 0, accesskey: 'i' })[0].outerHTML + ej.buildTag("label.filterDisableLbl",  this._getLocalizedLabels("Disable"))[0].outerHTML;
                var filterEnableRadio = ej.buildTag("input#filterEnable.filterEnable", "", {}, { name: 'filter', type: 'radio', role: 'radio', tabindex: 0, accesskey: 'n' })[0].outerHTML + ej.buildTag("label.filterDisableLbl",  this._getLocalizedLabels("Enable"))[0].outerHTML + "<br/>";
                var conditionLbl =ej.buildTag("label#conditionLbl.conditionLbl", this._getLocalizedLabels("Condition"))[0].outerHTML;
                var conditionDropDown = ej.buildTag("input#filterCondition.filterCondition", "", {}, { type: 'text', tabindex: 0, accesskey: 'o' })[0].outerHTML;

                var filterFrom = ej.buildTag("div#filterfrom.filterFrmDiv", ej.buildTag("input#filterFrom.filterFrom", "", {}, { name: 'inputVal', type: 'text', tabindex: 0 })[0].outerHTML)[0].outerHTML;
                var filterbtw = ej.buildTag("div#filterbtw.filterBtw", "<p>"+ this._getLocalizedLabels("and") +"</p>")[0].outerHTML;
                var filterTo = ej.buildTag("div#filterto.filterToDiv", ej.buildTag("input#filterTo.filterTo", "", {}, { name: 'inputVal', type: 'text', tabindex: 0, accesskey: 'd' })[0].outerHTML)[0].outerHTML;
                var fMeasureListLbl =ej.buildTag("label#filterMeasureListLbl.filterMeasureListLbl",this._getLocalizedLabels("Measure"))[0].outerHTML;
                var fMeasureList = ej.buildTag("input#fMeasuresList.fMeasuresList", "", {}, { type: 'text', tabindex: 0, accesskey:'M' })[0].outerHTML;
                var fValueLbl = ej.buildTag("label#filterValueLbl.filterValueLbl", this._getLocalizedLabels("Value"), {},{ tabindex : 0, accesskey:'a'})[0].outerHTML;

                var sortDiv = ej.buildTag("div#sortingDlg.sortingDlg", "<table class='sortReportTbl'><tr><td>" + sortingLbl + "</td><td>" + sortEnableRadio + sortDisableRadio + "</td></tr><tr><td>" + measureListLbl + "</td><td>" + measureListDropDown + "</td></tr> <tr><td>" + orderLbl + "</td><td>" + newascRadio + newdescRadio + "</td></tr><tr><td></td><td>" + preservHierarchy + "</td></tr></table>")[0].outerHTML;
                var filterDiv = ej.buildTag("div#filteringDlg.filteringDlg", "<table class='sortReportTbl'><tr><td>" + filter + "</td><td>" + filterEnableRadio + filterDisableRadio + "</td></tr><tr><td>" + fMeasureListLbl + "</td><td>" + fMeasureList + "</td></tr><tr><td>" + conditionLbl + "</td><td>" + conditionDropDown + "</td></tr><tr><td>" + fValueLbl + "</td><td>" + filterFrom + filterbtw + filterTo + "</td></tr></table>")[0].outerHTML;

                var sortTab = "<li><a style='font: bold 12px Segoe UI' href='#sortingDlg'>"+this._getLocalizedLabels("Sorting")+"</a></li>";
                var filterTab = "<li><a style='font: bold 12px Segoe UI' href='#filteringDlg'>" + this._getLocalizedLabels("Filtering") + "</a></li>";
                var sortfilterTab = ej.buildTag("div#sortfilterTab.sortfilterTab", "<ul class ='sortfiltTab'>" + (sortTab + filterTab + "</ul>" +"<div class='borderFilterSortDlg'>"+ sortDiv + filterDiv+"</div>"))[0].outerHTML;

                dialogContent = sortfilterTab;
            }
            else if (args_className.split(" ")[0] == "renameReportImg") {
                dialogTitle = this._getLocalizedLabels("RenameReport");
                currentTag = "Rename Report";
                var renameReportLabel = "<p class='dialogPara'>" + oclientProxy._getLocalizedLabels("ReportName") + "</p>";
                renameReport = "<input type=text id=reportName class='reportName'/></br>";
                dialogContent = renameReportLabel + renameReport;
            }
            else if (args_className.split(" ")[0] == "saveReportImg") {
                dialogTitle = this._getLocalizedLabels("SaveReport");
                currentTag = "Save Report";
                var reportNameLabel = "<p class='dialogPara'>" + oclientProxy._getLocalizedLabels("ReportName") + "</p>";
                reportName = "<input type=text id=reportName class='reportName'/></br>";
                dialogContent = reportNameLabel + reportName;
            }
            else if (args_className.split(" ")[0] == "LoadReport") {
                dialogTitle = this._getLocalizedLabels("LoadReport");
                currentTag = "Load Report";
                var reportNameLabel = "<table class='loadReportTbl'><tr><td class='loadReportTd'>" + oclientProxy._getLocalizedLabels("ReportName") + "</td>";
                var reportNamesDropdown = "<td><input type='text' id='reportNameList' class='reportNameList'/></td></tr></table>";
                dialogContent = reportNameLabel + reportNamesDropdown;
            }
            else if (args_className.split(" ")[0] == "chartTypesImg") {
                dialogTitle = this._getLocalizedLabels("ChartTypes");
                currentTag = "Chart Types";
                var reportNameLabel = "<p class='dialogPara'>" + oclientProxy._getLocalizedLabels("ChartTypes") + "</p>";
                reportName = "<input type=text id=reportName class='reportName'/></br>";
                dialogContent = reportNameLabel + reportName;
            }
            else if (args_className.indexOf("e-txt") > -1) {
              
                if (args_innerHTML == oclientProxy._getLocalizedLabels("Measures")) {
                    dialogTitle = this._getLocalizedLabels("MeasureEditor");
                    dialogContent = "<p class='editorPara'>" + this._getLocalizedLabels("Measures") + "</p>"
                }
                else {
                    dialogTitle = this._getLocalizedLabels("MemberEditor");
                    dialogContent = "<p class='editorPara'>" + hierarchyCaption + "</p>"
                   
                }
                var memberEditor; var checkOption = "";
                $(innerDiv).appendTo(EditorDiv);
                 if (args_innerHTML != oclientProxy._getLocalizedLabels("Measures")) {
                    var checkAll = ej.buildTag("div.checkAll e-icon")[0].outerHTML;
                    var unCheckAll = ej.buildTag("div.unCheckAll e-icon")[0].outerHTML;
                    checkOption = ej.buildTag("div", checkAll + unCheckAll, { "height": "19px" })[0].outerHTML;
                }
                if (args_innerHTML == oclientProxy._getLocalizedLabels("Measures")) {
                    memberEditor = oclientProxy._createMeasureEditor(editorTree);
                }
                else
                memberEditor = ej.buildTag("div#editorTreeView.editorTreeView")[0].outerHTML;
                var memberEditorDiv = ej.buildTag("div.memberEditorDiv", memberEditor)[0].outerHTML;
                var innerDiv = ej.buildTag("div", checkOption + memberEditorDiv)[0].outerHTML;
                var EditorDiv = ej.buildTag("div#EditorDiv.editorDiv", innerDiv)[0].outerHTML;
                dialogContent += EditorDiv + "</br>";
            }
            var dialogFooter;
            var OKBtn = "<button id=OKBtn class='dialogOKBtn'>" + this._getLocalizedLabels("OK") + "</button>";
            var CancelBtn = "<button id=CancelBtn class='dialogCancelBtn'>" + this._getLocalizedLabels("Cancel") + "</button>";
            dialogFooter = ej.buildTag("div", OKBtn + CancelBtn, { "float": "right", "margin": "-5px 0 6px" })[0].outerHTML;
            $(ejDialog).appendTo("#" + oclientProxy._id);
            $(dialogContent + dialogFooter).appendTo(this.element.find(".clientDialog"));
            this.element.find(".clientDialog").ejDialog({ width: args_className.split(" ")[0] == "SortFilterDlg" ? 320 : 265, content: "#" + oclientProxy._id, enableResize: false });
            this.element.find(".e-titlebar").prepend(ej.buildTag("div", dialogTitle, { "display": "inline" })[0].outerHTML)[0].setAttribute("tag", currentTag)
            this.element.find(".reportName").ejMaskEdit({ width: "155px" });
            this.element.find(".e-mask").addClass("dialogInput");
            this.element.find(".dialogOKBtn, .dialogCancelBtn").ejButton();
            $(".sortfilterTab").ejTab();
            $(".reportNameList").ejDropDownList({
                dataSource: args_innerHTML.split("__"),
                width: "150px",
                height: "20px",
            });
            $(".filterFrom,.filterTo").ejMaskEdit({
                name: "mask",
                inputMode: ej.InputMode.Text,
                width: "80px",
                height: "20px",
            });
            $(".measuresList").ejDropDownList({
                dataSource: args_innerHTML.split("||")[0].split("__"),
                width: "200px",
                height: "20px",
            });
            $(".fMeasuresList").ejDropDownList({
                dataSource: args_innerHTML.split("||")[0].split("__"),
                width: "200px",
                height: "20px",
            });
            $(".filterCondition").ejDropDownList({
                change: "_onActiveConditionChange",
                dataSource: ['EqualTo', 'NotEquals', 'GreaterThan', 'GreaterThanOrEqualTo', 'LessThan', 'LessThanOrEqualTo', 'Between', 'NotBetween'],
                width: "200px",
                height: "20px",
            });
            oclientProxy.element.find(".filterCondition").ejDropDownList("option", "change", ej.proxy(oclientProxy._onActiveConditionChange, oclientProxy));
            oclientProxy._dllSortMeasure = $(".measuresList").data("ejDropDownList");
            oclientProxy._dllFilterCondition = $(".filterCondition").data("ejDropDownList");
            oclientProxy._dllfMeasuresList = $(".fMeasuresList").data("ejDropDownList");
            if (sortState != "") {
                this.element.find(".radioBtnAsc")[0].checked = sortState[0] == "ASC" ? true : false;
                this.element.find(".radioBtnDesc")[0].checked = sortState[0] == "DESC" ? true : false;
                this.element.find(".sortDisable")[0].checked = !(this.element.find(".sortEnable")[0].checked = true);
                this.element.find(".preserveHrchy")[0].checked = sortState[1] == "PH" ? true : false;
                oclientProxy._isSorted = true;
                oclientProxy._dllSortMeasure.setModel({ value: (sortState[2]) });
            }
            if (filterState != "") {
                this.element.find(".filterEnable")[0].checked = true;
                oclientProxy._dllfMeasuresList.setModel({ value: (filterState[0]) });
                oclientProxy._dllFilterCondition.setModel({ value: (filterState[1]) });
                oclientProxy.element.find(".filterFrom").val(filterState[2]);
                if (filterState[3] != undefined && filterState[3] != "" && (filterState[1] == "Between" || filterState[1] == "NotBetween"))
                    oclientProxy.element.find(".filterTo").val(filterState[3]);
                else
                    $(".filterTo").attr("disabled", "disabled");
                oclientProxy._isFiltered = true;
            }
            if (this.element.find(".sortDisable")[0] != undefined && this.element.find(".sortDisable")[0].checked == true) {
                $('.measuresList_wrapper,.radioBtnAsc, .radioBtnDesc, .preserveHrchy').attr("disabled", "disabled");
                oclientProxy._dllSortMeasure.disable();
            }
            if (this.element.find(".filterDisable")[0] != undefined && this.element.find(".filterDisable")[0].checked == true) {
                $('.filterFrom, .filterTo').attr("disabled", "disabled");
                oclientProxy._dllFilterCondition.disable();
                oclientProxy._dllfMeasuresList.disable();
            }
            this.element.find(".e-widget-content").height("auto");
            if (editorTree != "" && editorTree != undefined && args_innerHTML != oclientProxy._getLocalizedLabels("Measures")) {
                var treeViewData = $.parseJSON(editorTreeInfo);
                this.element.find(".editorTreeView").ejTreeView({
                    showCheckbox: true,
                    loadOnDemand: true,
                    height: $(".memberEditorDiv").height(),
                    fields: { id: "id", hasChild: "hasChildren", text: "name", isChecked: "checkedStatus", dataSource: treeViewData },
                    nodeClick: ej.proxy(this._onNodeExpand, this),
                });
                var treeViewElements = this.element.find(".editorTreeView").find("li");
                for (var i = 0; i < treeViewElements.length; i++) {
                    treeViewElements[i].setAttribute("tag", treeViewData[i].tag);
                }
            }
            else {
                if (this.model.progressMode != "infinite")
                    this._progressHide();
            }
            this.memberTreeObj = this.element.find('.editorTreeView').data("ejTreeView");
            oclientProxy._unWireEvents();
            oclientProxy._wireEvents();
        },

        /**
         * 	This function is used to create the measure editor dialog.
         *
         * @private  
         * @param {String} editorTree - Contains the information of the measures included in current report.
         * @return {String} String containing the html content of the measure editor.
         */
        _createMeasureEditor: function (editorTree) {
            var editorTreeInfo;
            if (editorTree[0] != undefined)
                editorTreeInfo = $.parseJSON(editorTree[0].Value);
            else if (editorTree.d != undefined)
                editorTreeInfo = $.parseJSON(editorTree.d[0].Value);
            else
                editorTreeInfo = $.parseJSON(editorTree.EditorTreeInfo);
            var measureEditor = "";
            for (var i = 0; i < editorTreeInfo.length; i++) {
                measureEditor += "<div id=\"" + editorTreeInfo[i].uniqueName + "\"class=measureEditor>" + editorTreeInfo[i].name + "<span class='removeMeasure e-icon'></span></div>";
            }
            return measureEditor;
        },

        /**
         * This function is raised to fetch the child nodes on node expansion.
         *
         * @private  
         */
        _onNodeExpand: function (args) {
             
            var selectedItem = (args.event.originalEvent != undefined && args.event.originalEvent.target != undefined) ? args.event.originalEvent.target : args.event.target; var tagInfo;
            var childCount = $(selectedItem).parents("li").first().children().find("li").length;
            if (selectedItem.className.indexOf("e-plus") > -1 && childCount == 0) {
                var checkedState = $(selectedItem).parent().find("input[type=checkbox]").prop("checked");
                var tagInfo = $(selectedItem).parents("li").first().attr("tag");
                this.pNode = selectedItem;
                if (this.model.beforeServiceInvoke != null)
                    this._trigger("beforeServiceInvoke", { action: "memberExpanded", element: this.element, customObject: this.model.customObject });
                var serializedCustomObject = JSON.stringify(this.model.customObject);
                this.doAjaxPost("POST", this.model.url + "/" + this.model.serviceMethodSettings.memberExpand, JSON.stringify({
                    "action": "memberExpanded", "checkedStatus": checkedState, "parentNode": $(selectedItem).parents("li")[0].id, "tag": tagInfo, "dimensionName": this._dimensionName, "cubeName": this.currentCubeName, "olapReport": this.currentReport, "clientReports": this.reports, "customObject": serializedCustomObject
                }), this._fetchChildNodeSuccess);
            }
        },

        /**
         * 	The function is used to perform action while changing the condition in filter dialog.
         *
         * @private  
         */
        _onActiveConditionChange: function (args) {
            if (args.selectedText == "Between" || args.selectedText == "NotBetween")
                $(".filterTo").removeAttr("disabled");
            else {
                $(".filterTo").attr("disabled", "disabled");
                oclientProxy.element.find(".filterTo").val("");
            }
        },

		/**
        * This function is used to drill down the OLAP Grid widget once OLAP Chart drill down.
		* @return jQuery
        * @example 
        * &lt;div id="OlapClient"&gt;&lt;/div&gt; <br> 
        * &lt;script&gt;
        * // Create OLAP Client
        * $('#OlapClient').ejOlapClient({
        *       url: "OlapClientService.svc"
        *   });
        * var clientObj = $("#OlapClient").data("ejOlapClient");
        * clientObj.gridDrillSuccess(ej.proxy(function(){}, this));
        * // raised after OLAP Chart drill down
        * &lt;/script&gt;
        * @memberof ejOlapClient
        * @instance
        */
        chartDrillSuccess: function (sender) {
                 
            if (oclientProxy.currentReport != undefined)
                currentReport = oclientProxy.currentReport;
            if (oclientProxy.reports != undefined)
                reports = oclientProxy.reports;
            if (this.ochartObj._startDrilldown && this.ogridObj != null && this.ogridObj != undefined)
                this._renderOlapGrid(this.ochartObj._selectedTagInfo);
            else if (oclientProxy.ogridObj != undefined && oclientProxy.ogridObj.element.find("table") == "block")
                oclientWaitingPopup.hide();
        },

		/**
        * This function is used to drill down the OLAP Chart widget once OLAP Grid component drill down.
		* @return jQuery
        * @example 
        * &lt;div id="OlapClient"&gt;&lt;/div&gt; <br> 
        * &lt;script&gt;
        * // Create OLAP Client
        * $('#OlapClient').ejOlapClient({
        *       url: "OlapClientService.svc"
        *   });
        * var clientObj = $("#OlapClient").data("ejOlapClient");
        * clientObj.gridDrillSuccess(ej.proxy(function(){}, this));
        * // raised after OLAP Grid drill down
        * &lt;/script&gt;
        * @memberof ejOlapClient
        * @instance
        */
        gridDrillSuccess: function (sender) {
          
            if (oclientProxy.currentReport != undefined)
                currentReport = oclientProxy.currentReport;
            if (oclientProxy.reports != undefined)
                reports = oclientProxy.reports;
            if (this.ogridObj._startDrilldown && this.ochartObj != null && this.ochartObj != undefined) {
                var masktag; var levelCaption; var levelTag;
                levelCaption = this.ogridObj._drillCaption;
                jQuery.each(this.ochartObj._labelCurrentTags, function (index, value) {
                    if (value.name == levelCaption) {
                        levelTag = value.tag;
                        masktag = value;
                    }
                });
                oclientProxy.ochartObj._drillAction = this.ogridObj._drillAction;
                oclientProxy.ochartObj._selectedItem = levelCaption;
                if (this.ogridObj._drillAction == "drilldown")
                    oclientProxy.ochartObj._tagCollection.push(masktag)
                else if(!ej.isNullOrUndefined(masktag)){
                    jQuery.each(this.element.find(".rowAxis").find("button"), function (index, value) {
                        if (value.innerHTML == masktag.tag.split("[")[1].split("]")[0])
                            oclientProxy.ochartObj._dimensionIndex = index;
                    });
                    jQuery.each(oclientProxy.ochartObj._tagCollection, function (index, value) {
                        if (value.name == levelCaption) {
                            oclientProxy.ochartObj._selectedIndex = index + oclientProxy.ochartObj._dimensionIndex;
                            oclientProxy.ochartObj._tagCollection.splice(index, oclientProxy.ochartObj._tagCollection.length);
                            return false;
                        }
                    });
                }
                if (levelTag != null && levelTag != undefined)
                    oclientProxy._renderOlapChart(levelTag, this.ogridObj._drillAction);
                else
                    oclientWaitingPopup.hide();
            }
            else if($("#"+oclientWaitingPopup._id+"_WaitingPopup")[0].style.display == "block")
                oclientWaitingPopup.hide();
        },

		/**
        * This function is used to perform required action after dropping a tree node in Axis Element builder.
		* @return jQuery
        * @example 
        * &lt;div id="OlapClient"&gt;&lt;/div&gt; <br> 
        * &lt;script&gt;
        * // Create OLAP Client
        * $('#OlapClient').ejOlapClient({
        *       url: "OlapClientService.svc"
        *   });
        * var clientObj = $("#OlapClient").data("ejOlapClient");
        * clientObj.nodeDropped(ej.proxy(function(){}, this));
        * // raised while dropping tree nodes
        * &lt;/script&gt;
        * @memberof ejOlapClient
        * @instance
        */
        nodeDropped: function (sender) {
             
            if (sender.target.prevObject != undefined)
                sender.target.prevObject.removeClass("targetAxis");
            else
                $(sender.target).removeClass("targetAxis");
            isDragging = false;
            var droppedPosition = this.setSplitBtnTargetPos(sender.event);
            sender.target.prevObject != undefined ? sender.target.prevObject.parents().find(".e-dialog").hide() : $(sender.target).parents().find(".e-dialog").hide();
            var cssName = null;
            if ($(sender.target).hasClass("splitBtn"))
                cssName = $(sender.target).parents("div:eq(0)").attr("class");
            else if (sender.event.type == "touchend")
                cssName = sender.target.context.className;
            else if (sender.event.target != undefined) {
                if (sender.event.target.className == undefined || sender.event.target.className == null)
                    return false;
                if (sender.event.target.className.indexOf("splitBtn") > -1)
                    cssName = sender.target.className;
                else
                    cssName = sender.event.target.className;
            }
            else {
                if (sender.event.originalEvent.srcElement.className.indexOf("splitBtn") > -1)
                    cssName = sender.target.className;
                else
                    cssName = sender.event.originalEvent.srcElement.className;
            }

            var axisName = (cssName.indexOf("categoricalAxis") > cssName.indexOf("rowAxis")) ? ((cssName.indexOf("categoricalAxis") > cssName.indexOf("slicerAxis")) ? "Categorical" : "Slicer") : ((cssName.indexOf("rowAxis") > cssName.indexOf("slicerAxis")) ? "Series" : (cssName.indexOf("slicerAxis") >= 0) ? "Slicer" : null);
            var tagVal = $(sender.dropedElement.parents("li:eq(0)")[0]).attr("tag");
            if (axisName != null && tagVal != "Measures" && tagVal.lastIndexOf(".DF") != tagVal.length - ".DF".length) {
                if (oclientProxy.model.progressMode != "infinite") {
                    if (oclientProxy.model.progressMode == "progress")
                        $("#" + oclientProxy._id + "progressText")[0].innerHTML = oclientProxy._getLocalizedLabels("PreparingAndExecutingMDXquery");
                    this._progressStatus(10, 80, 120);
                }
                else
                    oclientWaitingPopup.show();
                if (this.model.beforeServiceInvoke != null)
                    this._trigger("beforeServiceInvoke", { action: "nodeDropped", element: this.element, customObject: this.model.customObject });
                var serializedCustomObject = JSON.stringify(this.model.customObject);
                var params = oclientProxy.currentCubeName + "--" + $(sender.dropedElement.parents("li:eq(0)")[0]).attr("tag") + "--" + axisName + "--" + droppedPosition;
                oclientProxy.doAjaxPost("POST", oclientProxy.model.url + "/" + oclientProxy.model.serviceMethodSettings.nodeDropped, JSON.stringify({ "action": "nodeDropped", "dropType": "TreeNode", "nodeInfo": params, "olapReport": oclientProxy.currentReport, "clientReports": oclientProxy.reports, "customObject": serializedCustomObject }), oclientProxy._nodeDroppedSuccess);
            }
        },

		/**
        * This function is used to set the position to currently dropped split button in respective axes.
		* @return jQuery
        * @example 
        * &lt;div id="OlapClient"&gt;&lt;/div&gt; <br> 
        * &lt;script&gt;
        * // Create OLAP Client
        * $('#OlapClient').ejOlapClient({
        *       url: "OlapClientService.svc"
        *   });
        * var clientObj = $("#OlapClient").data("ejOlapClient");
        * clientObj.setSplitBtnTargetPos(eve);
        * // used to set the position to the dropped split button
        * &lt;/script&gt;
        * @memberof ejOlapClient
        * @instance
        */
        setSplitBtnTargetPos: function (event) {
             
            var targetPosition = ""; var AEBdiv; var targetSplitBtn; var className;
            if (event.event != undefined && event.event.type == "touchend") {
                targetSplitBtn = event.event.originalEvent.target != null ? $(event.event.originalEvent.target).parents(".splitBtn") : $(event.event.originalEvent.srcElement).parents(".splitBtn");
                className = event.event.originalEvent.target != null ? event.event.originalEvent.target.className : event.event.originalEvent.srcElement.className;
            }
            else {
                targetSplitBtn = event.originalEvent.target != null ? $(event.originalEvent.target).parents(".splitBtn") : $(event.originalEvent.srcElement).parents(".splitBtn");
                className = event.originalEvent.target != null ? event.originalEvent.target.className : event.originalEvent.srcElement.className;
            }
            if (targetSplitBtn[0] || (className != undefined && className != null && jQuery.type(className) == "string" && className.indexOf("splitBtn") > -1)) {
                targetSplitBtn = targetSplitBtn[0] ? targetSplitBtn[0] : event.originalEvent.target != null ? event.originalEvent.target : event.originalEvent.srcElement;
                if (event.event != undefined && event.event.type == "touchend")
                    AEBdiv = event.target;
                else if (this._dropAxisClassName != null && this._dropAxisClassName != undefined && this._dropAxisClassName != "")
                    AEBdiv = this.element.find("." + this._dropAxisClassName)[0];
                else
                    AEBdiv = targetSplitBtn.parentNode;
                for (var i = 0; i < AEBdiv.children.length; i++) {
                    if (AEBdiv.childNodes[i] == targetSplitBtn)
                        targetPosition = i;
                }
            }
            return targetPosition;
        },

		/**
        * This function is raised while changing the cube.
		* @return jQuery
        * @example 
        * &lt;div id="OlapClient"&gt;&lt;/div&gt; <br> 
        * &lt;script&gt;
        * // Create OLAP Client
        * $('#OlapClient').ejOlapClient({
        *       url: "OlapClientService.svc"
        *   });
        * var clientObj = $("#OlapClient").data("ejOlapClient");
        * clientObj.cubeChanged(ej.proxy(function(){}, this));
        * // raised while changing cubes
        * &lt;/script&gt;
        * @memberof ejOlapClient
        * @instance
        */
        cubeChanged: function (sender) {
            this.currentCubeName = sender.text;
            if (oclientProxy.model.progressMode != "infinite") {
                if (oclientProxy.model.progressMode == "progress")
                    $("#" + oclientProxy._id + "progressText")[0].innerHTML = oclientProxy._getLocalizedLabels("RenderingLayoutStarted");
                this._progressStatus(0, 80, 120);
            }
            else
                oclientWaitingPopup.show();
            if (this.model.beforeServiceInvoke != null)
                this._trigger("beforeServiceInvoke", { action: "cubeChanged", element: this.element, customObject: this.model.customObject });
            var serializedCustomObject = JSON.stringify(this.model.customObject);
            this.doAjaxPost("POST", this.model.url + "/" + this.model.serviceMethodSettings.cubeChanged, JSON.stringify({ "action": "cubeChanged", "cubeName": this.currentCubeName, "customObject": serializedCustomObject, "clientParams": this.model.enableMeasureGroups }), this._cubeChangedSuccess);
        },

        measureGroupChanged: function (sender) {
            if (oclientProxy.model.progressMode != "infinite") {
                if (oclientProxy.model.progressMode == "progress")
                    $("#" + oclientProxy._id + "progressText")[0].innerHTML = oclientProxy._getLocalizedLabels("RenderingLayoutStarted");
                this._progressStatus(0, 80, 120);
            }
            else
                oclientWaitingPopup.show();
            if (this.model.beforeServiceInvoke != null)
                this._trigger("beforeServiceInvoke", { action: "cubeChanged", element: this.element, customObject: this.model.customObject });
            var serializedCustomObject = JSON.stringify(this.model.customObject);
            this.doAjaxPost("POST", this.model.url + "/" + this.model.serviceMethodSettings.measureGroupChanged, JSON.stringify({ "action": "measureGroupChanged", "measureGroupName": sender.text, "customObject": serializedCustomObject }), this._measureGroupChangedSuccess);
        },

		/**
        * This function is raised while changing the report.
		* @return jQuery
        * @example 
        * &lt;div id="OlapClient"&gt;&lt;/div&gt; <br> 
        * &lt;script&gt;
        * // Create OLAP Client
        * $('#OlapClient').ejOlapClient({
        *       url: "OlapClientService.svc"
        *   });
        * var clientObj = $("#OlapClient").data("ejOlapClient");
        * clientObj.reportChanged(ej.proxy(function(){}, this));
        * // raised while changing reports
        * &lt;/script&gt;
        * @memberof ejOlapClient
        * @instance
        */
        reportChanged: function (sender) {
             
            if (oclientProxy.model.progressMode != "infinite") {
                if (oclientProxy.model.progressMode == "progress")
                    $("#" + oclientProxy._id + "progressText")[0].innerHTML = oclientProxy._getLocalizedLabels("PreparingAndExecutingMDXquery");
                this._progressStatus(5, 80, 100);
            }
            else
                oclientWaitingPopup.show();
            if (oclientProxy.model.beforeServiceInvoke != null)
                this._trigger("beforeServiceInvoke", { action: "toolbarOperation", element: this.element, customObject: this.model.customObject });
            var serializedCustomObject = JSON.stringify(oclientProxy.model.customObject);
            this.doAjaxPost("POST", this.model.url + "/" + this.model.serviceMethodSettings.toolbarServices, JSON.stringify({ "action": "toolbarOperation", "toolbarOperation": "Report Change", "clientInfo": sender.text, "olapReport": null, "clientReports": this.reports, "customObject": serializedCustomObject }), this._toolbarOperationSuccess);
        },

		/**
        * This function is raised after dropping the element in Axis Element Builder.
		* @return jQuery
        * @example 
        * &lt;div id="OlapClient"&gt;&lt;/div&gt; <br> 
        * &lt;script&gt;
        * // Create OLAP Client
        * $('#OlapClient').ejOlapClient({
        *       url: "OlapClientService.svc"
        *   });
        * var clientObj = $("#OlapClient").data("ejOlapClient");
        * clientObj.onDropped(ej.proxy(function(){}, this));
        * // raised while dropping tree nodes
        * &lt;/script&gt;
        * @memberof ejOlapClient
        * @instance
        */
        onDropped: function (sender) {
            if (sender.target.className.indexOf("e-button") > -1) {
                var columnPosInfo = this.getAxisPosition(this.element.find(".categoricalAxis"));
                var rowPosInfo = this.getAxisPosition(this.element.find(".rowAxis"));
                var slicerPosInfo = this.getAxisPosition(this.element.find(".slicerAxis"));
                this._dropAxisClassName;
                if (sender.pageX != undefined && sender.pageY != undefined) {
                    if (sender.pageX > columnPosInfo.left && sender.pageX < columnPosInfo.right && sender.pageY > columnPosInfo.top && sender.pageY < columnPosInfo.bottom)
                        this._dropAxisClassName = "categoricalAxis";
                    else if (sender.pageX > rowPosInfo.left && sender.pageX < rowPosInfo.right && sender.pageY > rowPosInfo.top && sender.pageY < rowPosInfo.bottom)
                        this._dropAxisClassName = "rowAxis";
                    else if (sender.pageX > slicerPosInfo.left && sender.pageX < slicerPosInfo.right && sender.pageY > slicerPosInfo.top && sender.pageY < slicerPosInfo.bottom)
                        this._dropAxisClassName = "slicerAxis";
                    else
                        this._dropAxisClassName = "outOfAxis";
                }
            }
        },

       /**
        * 	This function is used to get the position of the axis element builders.
		* @return jQuery
        * @example 
        * &lt;div id="OlapClient"&gt;&lt;/div&gt; <br> 
        * &lt;script&gt;
        * // Create OLAP Client
        * $('#OlapClient').ejOlapClient({
        *       url: "OlapClientService.svc"
        *   });
        * var clientObj = $("#OlapClient").data("ejOlapClient");
        * clientObj.getAxisPosition(eve);
        * // used to get the position of the axis element builders
        * &lt;/script&gt;
        * @memberof ejOlapClient
        * @instance
        */
        getAxisPosition: function (targetElement) {
            var tempPos = targetElement.position();
            var item = {};
            item["top"] = tempPos.top;
            item["right"] = tempPos.left + $(targetElement).width();
            item["bottom"] = tempPos.top + $(targetElement).height();
            item["left"] = tempPos.left;
            return item;
        },

       /**
        * This function is raised while tab changes.
		* @return jQuery
        * @example 
        * &lt;div id="OlapClient"&gt;&lt;/div&gt; <br> 
        * &lt;script&gt;
        * // Create OLAP Client
        * $('#OlapClient').ejOlapClient({
        *       url: "OlapClientService.svc"
        *   });
        * var clientObj = $("#OlapClient").data("ejOlapClient");
        * clientObj.onTabClick(ej.proxy(function(){}, this));
        * // raised while changing tabs
        * &lt;/script&gt;
        * @memberof ejOlapClient
        * @instance
        */
        onTabClick: function (args) {
            
            if (oclientProxy.ochartObj != null && oclientProxy.ochartObj != undefined)
                oclientProxy.chartObj = oclientProxy.element.find("#" + oclientProxy.ochartObj._id + "Container").data("ejChart");
            currentTab = "chart";
            if ($(args.activeHeader).find("a").text() == "Grid") {
                currentTab = "grid";
                oclientProxy.element.find(".chartTypesImg").addClass("chartTypesOnGridView");
                oclientProxy.element.find(".gridPanel").css("display", "inline");
            }
            else if (oclientProxy.chartObj != null && oclientProxy.chartObj != undefined) {
                currentTab = "chart";
                oclientProxy.element.find(".chartTypesImg").removeClass("chartTypesOnGridView");
                if (oclientProxy.model.progressMode == "infinite")
                    oclientWaitingPopup.show();
                oclientProxy.chartObj.redraw();
                if (oclientProxy.model.progressMode == "infinite")
                    oclientWaitingPopup.hide();
            }
        },

        /**
        * Perform an asynchronous HTTP (Ajax) request.
        * @return jQuery
        * @example 
        * &lt;div id="OlapClient"&gt;&lt;/div&gt; <br> 
        * &lt;script&gt;
        * // Create OLAP Client
        * $('#OlapClient').ejOlapClient({
        *       url: "OlapClientService.svc"
        *   });
        * var clientObj = $("#OlapClient").data("ejOlapClient");
        * clientObj.doAjaxPost("POST", "/OlapClientService.svc/Initialize", {"key", "Hello World"}, "renderControlSuccess", null);
        * // initiate an Ajax request
        * &lt;/script&gt;
        * @memberof ejOlapClient
        * @instance
        */
        doAjaxPost: function (type, url, data, onSuccess, onComplete) {
             
            $.ajax({
                type: type,
                url: url,
                contentType: 'application/json; charset=utf-8',
                dataType: 'json',
                data: data,
                success: $.proxy(onSuccess, this),
                complete: ej.proxy(function () {
                    var eventArgs = { "customObject": this.model.customObject, "element": this.element};
                    this._trigger("renderComplete", eventArgs);
                },this),
                error: ej.proxy(function (msg, textStatus, errorThrown) {
                    if (this.model.progressMode != "infinite") {
                        window.clearInterval(oClientTimer);
                        if (this.model.progressMode == "progress")
                            $("#" + this._id + "progressText")[0].innerHTML =this._getLocalizedLabels("MDXqueryExecutionFailed")+"!!!";
                        $("#" + this._id).show();
                        oclientProgressBar.setModel({ value: 0, text: "Failed!!!" });
                    }
                    else {
                        oclientWaitingPopup = $("#" + this._id).data("ejWaitingPopup");
                        oclientWaitingPopup.hide();
                    }
                    var eventArgs = { "customObject": this.model.customObject, "element": this.element, "Message": msg };
                    this._trigger("renderFailure",eventArgs);
                },this)
            });
        }
    });

    //OLAP Client Localization
    ej.olap.OlapClient.locale = {};
	//Default english localized values
    ej.olap.OlapClient.locale["en-US"] = {
        Column: "Column",
        Row: "Row",
        Slicer: "Slicer",
        CubeSelector: "Cube Selector",
        ReportName: "Report Name",
        NewReport: "New Report",
        CubeDimensionBrowser: "Cube Dimension Browser",
        AddReport: "Add Report",
        RemoveReport: "Remove Report",
        CannotRemoveSingleReport:"Can not remove Single Report",
        AreYouSureToDeleteTheReport: "Are you sure to delete the Report",
        RenameReport: "Rename Report",
        SaveReport: "Save Report",
        ChartTypes: "Chart Types",
        LoadReport: "Load Report",
        ExportToExcel: "Export To Excel",
        FullScreen: "Full Screen",
        Grid: "Grid",
        Chart: "Chart",
        OK: "OK",
        Cancel: "Cancel",
        MeasureEditor: "Measure Editor",
        MemberEditor: "Member Editor",
        Measures: "Measures",
        OpeningDialog: "Opening dialog",
        OpeningDialogSucceeded: "Opening dialog succeeded",
        PreparingAndExecutingMDXquery: "Preparing and executing MDX query",
        MDXqueryExecutionFailed: "MDX query execution failed",
        MDXqueryExecutedSuccessfully:"MDX query executed successfully",
        RenderingLayoutStarted: "Rendering Layout Started",
        RenderingLayoutCompleted: "Rendering Layout Completed",
        RenderingStarted:"Rendering Started",
        RenderingSucceeded: "Rendering Succeeded",
        SortOrFilterColumn: "Sort/Filter (Column)",
        SortOrFilterRow: "Sort/Filter (Row)",
        SortingAndFiltering: "Sorting And Filtering",
        Sorting: "Sorting",
        Measure: "<u>M</u>easure",
        Order: "Order",
        Filtering: "Filtering",
        Condition: "C<u>o</u>ndition",
        Value: "Val<u>u</u>e",
        PreserveHierarchy: "P<u>r</u>eserve  Hierarchy",
        Ascending: "<u>A</u>scending",
        Descending: "D<u>e</u>scending",
        Enable: "E<u>n</u>able",
        Disable: "D<u>i</u>sable",
        and: "<u>a</u>nd",
        Line: "Line",
    Spline:"Spline",
    Column:"Column",
    Area:"Area",
    SplineArea:"Spline Area",
    StepLine:"Step Line",
    StepArea:"Step Area",
    Pie:"Pie",
    Bar: "Bar",
    StackingArea:"Stacking Area",
    StackingColumn:"Stacking Column",
    StackingBar: "Stacking Bar",
    Pyramid: "Pyramid",
        Funnel: "Funnel"
    }

    /**
     * Enum for control placement. Allows the user to select a appropriate view for positioning the controls. 
     * @enum {String}
	 * @global
     */
    ej.olap.OlapClient.ControlPlacement = {
        Tab: "tab",
        Tile: "tile"
    };

    /**
     * Enum for display mode. Allows the user to select a appropriate display mode. 
     * @enum {String}
	 * @global
     */
    ej.olap.OlapClient.DisplayMode = {
        ChartOnly: "chartonly",
        GridOnly: "gridonly",
        ChartAndGrid: "chartandgrid"
    };

    /**
     * Enum for default view. Allows the user to set the default view of the control. 
     * @enum {String}
	 * @global
     */
    ej.olap.OlapClient.DefaultView = {
        Chart: "chart",
        Grid: "grid"
    };

    /**
     * Enum for progress mode. Allows the user to select an appropriate progress indicator. 
     * @enum {String}
	 * @global
     */
    ej.olap.OlapClient.ProgressMode = {
        Infinite: "infinite",
        Normal: "normal",
        Progress: "progress"
    };

})(jQuery, Syncfusion);;
/**
* @fileOverview Plugin to style the Html OLAP Chart elements
* @copyright Copyright Syncfusion Inc. 2001 - 2013. All rights reserved.
*  Use of this code is subject to the terms of our license.
*  A copy of the current license can be obtained at any time by e-mailing
*  licensing@syncfusion.com. Any infringement will be prosecuted under
*  applicable laws. 
* @version 12.1 
* @author <a href="mailto:licensing@syncfusion.com">Syncfusion Inc</a>
*/

(function ($, ej, undefined) {
    /**
    * @namespace ej
    * @class ejOlapChart
    * @requires jquery-1.10.2.min.js
    * @requires jquery.easing.1.3.min.js
    * @requires jquery.globalize.min.js
    * @requires ej.core.js
    * @requires ej.data.js
    * @requires ej.touch.js
    * @requires ej.dialog.js
    * @requires ej.draggable.js
    * @requires ej.waitingpopup.js
    * @requires ej.chart.js
    * @requires ej.olapchart.js
    * @classdesc Custom Design for Html OLAP Chart control.
    * @example 
    * &lt;div id="OlapChart"&gt;&lt;/div&gt; <br> 
    * &lt;script&gt;
    * // Create OLAP Chart
    * $("#OlapChart").ejOlapChart(...); 	
    * &lt;/script&gt;
    */
    ej.widget("ejOlapChart", "ej.olap.OlapChart", {

        //Root Css Class
        _rootCSS: "e-olapchart",

        //Widget element will be automatically set in this
        element: null,

        //User-defined model will be automatically set in this
        model: null,

        validTags: ["div", "span"],

        //Default model values
        defaults: $.extend(ej.datavisualization.Chart.prototype.defaults, {
            //Properties
            /**		
            * Connects the service using the specified URL for any server updates.
            * @default ""
            * @type {string}
            * @example 
            * //To set url API value during initialization  
            * 	$("#OlapChart").ejOlapChart({  url: "/OlapChartService.svc"});					* 
            * @example 
            * //Get or set the url API, after initialization:<br>
            *	//Gets the url value  
            *	$("#OlapChart").ejOlapChart("option", "url");<br>			
            *	//Sets the url value 
            *	$("#OlapChart").ejOlapChart("option", "url",  "/OlapChartService.svc" ); 
            * @memberof ejOlapChart
            * @instance
            */
            url: "",
            /**		
            * Specify the CSS class to OLAP Chart to achieve custom theme.
            * @default ""
            * @type {string}
            * @example 
            * // Set the CSS class during initialization. 			
            * 	$("#OlapChart").ejOlapChart({  cssClass : "gradient-lime" });			* 
            * @example 
            * //Get or set the CSS class after initialization:<br>
            *	// Gets the CSS class value.		
            *	$("#OlapChart").ejOlapChart("option", "cssClass");			
            *	// Sets the CSS class to OLAP Chart
            *	$("#OlapChart").ejOlapChart("option", "cssClass",  "gradient-lime" );
            * @memberof ejOlapChart
            * @instance
            */
            cssClass: "",

            /**		
            * Contains the serialized OlapReport at the instant.
            * @default ""
            * @type {String}	
            * @memberof ejOlapChart
            * @instance
            */
            currentReport: "",

            /**		
            * Custom object to pass additional information between client-end and service-end.
            * @default null
            * @type {object}
            * @example 
            * //To set customObject API value during initialization  
            * 	$("#OlapChart").ejOlapChart({  customObject: {"key":"Hello World"}});					* 
            * @example 
            * //Get or set the customObject API, after initialization:<br>
            *	//Gets the customObject value  
            *	$("#OlapChart").ejOlapChart("option", "customObject");<br>			
            *	//Sets the customObject value 
            *	$("#OlapChart").ejOlapChart("option", "customObject",  {"key":"Hi Syncfusion!"} ); 
            * @memberof ejOlapChart
            * @instance
            */
            customObject: {},
            /**		
            * Allows the user to set custom name for the methods at service-end invoked on AJAX post.
            * @default null
            * @type {object}
            * @example 
            * //To set serviceMethodSettings API value, to invoke the appropriate service method on UI operation.
            * 	$("#OlapChart").ejOlapChart({  serviceMethodSettings: {initialize: "MyMethod", drillDown: "DrillChart"});
            * @example 
            * //Gets or sets the serviceMethodSettings API, to invoke the appropriate service method on UI operation:<br>
            * //Gets the serviceMethodSettings value  
            * $("#OlapChart").ejOlapChart("option", "serviceMethodSettings");<br>			
            * //Sets the serviceMethodSettings value 
            * $("#OlapChart").ejOlapChart("option", "serviceMethodSettings",  {initialize: "MyMethod", drillDown: "DrillChart"} ); 
            * @memberof ejOlapChart
            * @instance
            */
            serviceMethodSettings: {
                /**		
                * Allows the user to set the custom name for the service method that’s responsible for initializing OLAP Chart.
                * @default "InitializeChart"
                * @type {string}
                * @example 
                * //To set initialize API value, to invoke the corresponding service method for OLAP Chart initialization. 
                * $("#OlapChart").ejOlapChart({  serviceMethodSettings: {initialize: "IninlizeChartMyMethod"}); 
                * @example 
                * //Gets or sets the initialize API, to invoke the corresponding service method for OLAP Chart initialization:<br>
                * //Gets the initialize value  
                * $("#OlapChart").ejOlapChart("option", "serviceMethodSettings");<br>			
                * //Sets the initialize value 
                * $("#OlapChart").ejOlapChart("option", "serviceMethodSettings.initialize", "IninlizeChartMyMethod"} ); 
                * @alias ejOlapChart#serviceMethodSettings->initialize
                * @instance
                */
                initialize: "InitializeChart",

                /**		
                * Allows the user to set the custom name for the service method that’s responsible for drilling up/down operation in OLAP Chart. 
                * @default "DrillChart"
                * @type {string}
                * @example 
                * //To set drillDown API value, to invoke the corresponding service method for performing server-side operation while drilling up/down in OLAP Chart. 
                * $("#OlapChart").ejOlapChart({  serviceMethodSettings: {drillDown: "DrillChartMyMethod"});					
                * @example 
                * //Get or set the drillDown API, to invoke the corresponding service method for performing server-side operation while drilling up/down in OLAP Chart:<br>
                * //Gets the drillDown value  
                * $("#OlapChart").ejOlapChart("option", "serviceMethodSettings");<br>			
                * //Sets the drillDown value 
                * $("#OlapChart").ejOlapChart("option", "serviceMethodSettings.drillDown", "DrillChartMyMethod"} ); 
                * @alias ejOlapChart#serviceMethodSettings->drillDown
                * @instance
                */
                drillDown: "DrillChart"
            },

            /**		
            * Sets the progress indicator type which arises on any UI operation.	See {@link progressMode}
            * @default ej.olap.OlapChart.ProgressMode.Infinite
            * @type {enum}
            * @example 
            * //To set progressMode API value during initialization  
            * 	$("#OlapChart").ejOlapChart({  progressMode: ej.olap.OlapChart.ProgressMode.Normal});					* 
            * @example 
            * //Get or set the progressMode API, after initialization:<br>
            *	//Gets the progressMode value  
            *	$("#OlapChart").ejOlapChart("option", "progressMode");<br>			
            *	//Sets the progressMode value 
            *	$("#OlapChart").ejOlapChart("option", "progressMode",  ej.olap.OlapChart.ProgressMode.Progress ); 
            * @memberof ejOlapChart
            * @instance
            */
            progressMode: "infinite",
            /**		
            * Sets the localized language for the control.
            * @default "en-US"
            * @type {string}
            * @example 
            * //To set localization API value during initialization  
            * 	$("#OlapChart").ejOlapChart({  locale: "fr-FR"});					* 
            * @example 
            * //Get or set the localization API, after initialization:<br>
            *	//Gets the localization value  
            *	$("#OlapChart").ejOlapChart("option", "locale");<br>			
            *	//Sets the localization value 
            *	$("#OlapChart").ejOlapChart("option", "locale",  "fr-FR" ); 
            * @memberof ejOlapChart
            * @instance
            */
            locale: "en-US",
            //Events
            /**     
            * Fires when drilldown happens in OLAP Chart control.
            * @event
            * @name ejOlapChart#drillSuccess 	
            * @param {Object} argument Event parameters from OLAP Chart     
            * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
            * @param {object}  argument.model returns the OLAP Chart model.
            * @param {string}  argument.type returns the name of the event.
            * @example 
            * //drillSuccess event for OLAP Chart
            * $("#OlapChart").ejOlapChart({
            *    drillSuccess: function (args) {}
            * });      
            * @memberof ejOlapChart
            * @instance
            */
            drillSuccess: null,
            /**     
            * Fires before any Ajax request passed from OLAP Chart to service methods.
            * @event
            * @name ejOlapChart#beforeServiceInvoke 	
            * @param {Object} argument Event parameters from OLAP Chart    
            * @param {string}  argument.action return the current action of OLAP Chart control.		 
            * @param {object}  argument.customObject return the custom object bounds with OLAP Chart control.
            * @param {string}  argument.element return the outer HTML of OLAP Chart control.
            * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
            * @param {object}  argument.model returns the OLAP Chart model.
            * @param {string}  argument.type returns the name of the event.			 
            * @example 
            * //beforeServiceInvoke event for OLAP Chart
            * $("#OlapChart").ejOlapChart({
            *    beforeServiceInvoke: function (args) {}
            * });      
            * @memberof ejOlapChart
            * @instance
            */
            beforeServiceInvoke: null,
            /**     
            * Fires when it reaches client script after any Ajax request.
            * @event
            * @name ejOlapChart#afterServiceInvoke 	
            * @param {Object} argument Event parameters from OLAP Chart  
            * @param {string}  argument.action return the current action of OLAP Chart control.		 
            * @param {object}  argument.customObject return the custom object bounds with OLAP Chart control.
            * @param {string}  argument.element return the outer HTML of OLAP Chart control.
            * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
            * @param {object}  argument.model returns the OLAP Chart model.
            * @param {string}  argument.type returns the name of the event.			 
            * @example 
            * //afterServiceInvoke event for OLAP Chart
            * $("#OlapChart").ejOlapChart({
            *    afterServiceInvoke: function (args) {}
            * });      
            * @memberof ejOlapChart
            * @instance
            */
            afterServiceInvoke: null,
            /**     
            * Fires when OLAP Chart completely finishes its rendering.
            * @event
            * @name ejOlapChart#renderComplete 	
            * @param {Object} argument Event parameters from OLAP Chart  
            * @param {string}  argument.action return the current action of OLAP Chart control.		 
            * @param {object}  argument.customObject return the custom object bounds with OLAP Chart control.
            * @param {string}  argument.element return the outer HTML of OLAP Chart control.
            * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
            * @param {object}  argument.model returns the OLAP Chart model.
            * @param {string}  argument.type returns the name of the event.			 
            * @example 
            * //renderComplete event for OLAP Chart
            * $("#OlapChart").ejOlapChart({
            *    renderComplete: function (args) {}
            * });      
            * @memberof ejOlapChart
            * @instance
            */
            renderComplete: null,
            /**     
            * Fires while any discrepancies occurs during the rendering time.
            * @event
            * @name ejOlapChart#renderFailure 	
            * @param {Object} argument Event parameters from OLAP Chart  
            * @param {string}  argument.action return the current action of OLAP Chart control.		 
            * @param {object}  argument.customObject return the custom object bounds with OLAP Chart control.
            * @param {object}  argument.message return the error stacke tace of the original exception.
            * @param {string}  argument.element return the outer HTML of OLAP Chart control.
            * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
            * @param {object}  argument.model returns the OLAP Chart model.
            * @param {string}  argument.type returns the name of the event.			 
            * @example 
            * //renderFailure event for OLAP Chart
            * $("#OlapChart").ejOlapChart({
            *    renderFailure: function (args) {}
            * });      
            * @memberof ejOlapChart
            * @instance
            */
            renderFailure: null,
            /**     
            * Fires when OLAP Chart successfully finished its rendering.
            * @event
            * @name ejOlapChart#renderSuccess 	
            * @param {Object} argument Event parameters from OLAP Chart  
            * @param {string}  argument.action return the current action of OLAP Chart control.		 
            * @param {object}  argument.customObject return the custom object bounds with OLAP Chart control.
            * @param {string}  argument.element return the outer HTML of OLAP Chart control.
            * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
            * @param {object}  argument.model returns the OLAP Chart model.
            * @param {string}  argument.type returns the name of the event.			 
            * @example 
            * //renderSuccess event for OLAP Chart
            * $("#OlapChart").ejOlapChart({
            *    renderSuccess: function (args) {}
            * });      
            * @memberof ejOlapChart
            * @instance
            */
            renderSuccess: null
        }),

        /**
        * Specify the data types for default properties 
        * @private
        */
        dataTypes: {
            marker: "data",
            crossHair: "data",
            size: "data",
            serviceMethodSettings: "data",
            zooming: "data",
            customObject: "data"
        },

        observables: ["title.text", "commonSeriesOptions.type", "locale"],
        titleText: ej.util.valueFunction("title.text"),
        seriesType: ej.util.valueFunction("commonSeriesOptions.type"),
        locale: ej.util.valueFunction("locale"),

        /**
        * Gets the serialized current OLAP Report of the control.
        *
        * @private 
        */
        //get the current OLAP Report
        getOlapReport: function () {
            return this._olapReport;
        },

        /**
        * Sets the serialized current OLAP Report of the control.
        *
        * @private 
        */
        //set the current OLAP Report
        setOlapReport: function (value) {
            this._olapReport = value;
        },

        /**
        * Gets the JSON formatted records for the control.
        *
        * @private 
        */
        //get the JSON formatted records
        getJsonRecords: function () {
            return this._JSONRecords;
        },

        /**
        * Sets the JSON formatted records for the control.
        *
        * @private 
        */
        //set the JSON formatted records
        setJsonRecords: function (value) {
            this._JSONRecords = $.parseJSON(value);
        },

        // constructor function
        /**
        * Initializes the variables and loads the widget. 
        *
        * @private
        */
        _init: function () {
            this._initPrivateProperties();
            this._load();
        },

        /**
        * Destroy the OLAP Chart widget
        * all events bound using this._on will be unbind automatically and bring the control to pre-init state.
        * @alias destroy
        * @return jQuery
        * @example 
        * &lt;div id="OlapChart"&gt;&lt;/div&gt; <br> 
        * &lt;script&gt;
        * // Create OLAP Chart
        * var chartObj = $("#OlapChart").data("ejOlapChart");
        * chartObj.destroy(); // destroy the OLAP Chart
        * &lt;/script&gt;
        * @example 
        * &lt;div id="OlapChart"&gt;&lt;/div&gt; <br> 
        * &lt;script&gt;
        * // enable the OLAP Chart
        * $("#OlapChart").ejOlapChart("destroy");	
        * &lt;/script&gt;
        * @memberof ejOlapChart
        * @instance
        */
        _destroy: function () {
            
            this.element.empty().removeClass("e-olapchart" + this.model.cssClass);
            if (this.ochartWaitingPopup != undefined)
                this.ochartWaitingPopup._destroy();
        },

        /**
        * To configure the private properties of OLAP Chart		
        * @private
        */
        _initPrivateProperties: function () {
            this._id = this.element.attr("id");
            ochartProxy = this;
            this._olapReport = "";
            this._JSONRecords = null;
            this._currentAction = "initialize";
            this._selectedItem = "";
            this._selectedIndex = -1;
            this._selectedTagInfo = null;
            this._tagCollection = new Array();
            this._labelCurrentTags = new Array();
            this._startDrilldown = false;
            this._drillAction = "";
            this._initZooming = false;
            this._dimensionIndex = -1;
            this._isDrilled = false;
            this._selectedMenuItem = "";
            ochartWaitingPopup = null;
            ochartProgressBar = null;
            oChartTimer = null;
        },

        /**
        * This function is the point were loading/rendering of the OLAP Chart widget is initiated.
        *
        * @private  
        */
        _load: function () {
            this.element.addClass(this.model.cssClass);
            if (this.model.progressMode != "infinite") {
                $("#" + ochartProxy._id + "progressContainer").remove();
                this._createProgressBar();
                this._progressStatus(5, 69, 140);
            }
            else if (typeof oclientWaitingPopup != 'undefined' && oclientWaitingPopup != null)
                if ($("#" + oclientProxy._id + "_maxView")[0]) {
                    $("#" + oclientProxy._id + "_maxView").ejWaitingPopup({ showOnInit: true });
                }
                else
                    oclientWaitingPopup.show();
            else {
                this.element.ejWaitingPopup({ showOnInit: true });
                ochartWaitingPopup = this.element.data("ejWaitingPopup");
            }
            if (this.model.zooming != "")
                this._initZooming = true;
            if (this.model.beforeServiceInvoke != null)
                this._trigger("beforeServiceInvoke", { action: this._currentAction, element: this.element, customObject: this.model.customObject });
            if (ochartProxy.model.progressMode == "progress")
                $("#" + ochartProxy._id + "progressText")[0].innerHTML = ochartProxy._getLocalizedLabels("PreparingAndExecutingMDXquery") + "...";
            if (this.model.currentReport != "")
                this.model.customObject = { currentReport: this.model.currentReport };
            var serializedCustomObject = JSON.stringify(this.model.customObject);
            if (this.model.customObject != "" && this.model.customObject != null && this.model.customObject != undefined)
                this.doAjaxPost("POST", this.model.url + "/" + this.model.serviceMethodSettings.initialize, JSON.stringify({ "action": "initialize", "customObject": serializedCustomObject }), this.renderControlSuccess);
            else
                this.doAjaxPost("POST", this.model.url + "/" + this.model.serviceMethodSettings.initialize, JSON.stringify({ "action": "initialize" }), this.renderControlSuccess);
        },
        _setFirst: true,

        /**
        * To configure the properties at runtime using SetModel		
        * @private
        */
        _setModel: function (options) {
            var chartObj = this.element.find("#" + this._id + "Container").data("ejChart");
            for (var key in options) {
                switch (key) {
                    case "olapReport": this.setOlapReport(options[key]); break;
                    case "jsonData": this.setJsonRecords(options[key]); break;
                    case "refreshOlapChart": this.element.renderChartFromJSON(options[key]); break;
                    case "customObject": this.model.customObject = options[key]; break;
                    case "height": this.model.size.height = options[key]; break;
                    case "width": this.model.size.width = options[key]; break;
                    case "commonSeriesOptions":
                        {
                            if (chartObj) {
                                chartObj.model.commonSeriesOptions = options[key];
                                chartObj.model.type = chartObj.model.commonSeriesOptions.type = this.seriesType();
                                for (var i = 0; i < chartObj.model.series.length; i++)
                                    chartObj.model.series[i].type = chartObj.model.type;
                            } break;
                        }
                    case "title":
                        {
                            if (chartObj)
                                chartObj.model.title.text = this.titleText();
                            break;
                        }
                    case "animation": this.model.animation = options[key]; break;
                    case "crossHair": this.model.crossHair = options[key]; break;
                    case "marker": this.model.marker = options[key]; break;
                    case "zooming": this.model.zooming = options[key]; break;
                    case "legend": this.model.legend = options[key]; break;
                    case "primaryXAxis": this.model.primaryXAxis = options[key]; break;
                    case "primaryYAxis": this.model.primaryYAxis = options[key]; break;
                    case "locale": this.locale(); break;
                    default:
                        { $.extend(true, this.model, {}, options[key]); }
                }
            }
            if (chartObj)
                this.renderControlSuccess({ "JsonRecords": JSON.stringify(this.getJsonRecords()), "OlapReport": this.getOlapReport() });
        },

        /**
        * This function is invoked to wire the events for UI interaction.
        *
        * @private  
        */
        _wireEvents: function () {
            window.clearInterval(oChartTimer);
            this._on(this.element, "click", ".menuList", function (evt) {
                ochartWaitingPopup = this.element.data("ejWaitingPopup");
                if (evt.target.innerHTML == ochartProxy._getLocalizedLabels("Exit")) {
                    this.element.find("#" + ochartProxy._id + "ExpandMenu, .expandMenu, .e-dialog").remove();
                }
                else if (evt.target.innerHTML.indexOf(ochartProxy._getLocalizedLabels("Expand")) > -1) {
                    if (typeof oclientWaitingPopup != 'undefined' && oclientWaitingPopup != null) {
                        if ($("#" + oclientProxy._id + "_maxView")[0])
                            $("#" + oclientProxy._id + "_maxView").ejWaitingPopup({ showOnInit: true });
                        else
                            oclientWaitingPopup.show();
                    }
                    else {
                        if (this.model.progressMode != "infinite")
                            ochartProxy._progressStatus(0, 70, 120);
                        else
                            ochartWaitingPopup.show();
                    }
                    if (!this._isDrilled) {
                        this._isDrilled = true;
                        jQuery.each(evt.target.parentElement.children, function (index, value) {
                            if (value.innerHTML == evt.target.innerHTML)
                                ochartProxy._dimensionIndex = index;
                        });
                    }
                    this._drillAction = "drilldown";
                    this._selectedItem = $.trim(evt.target.innerHTML.split("-")[1]);
                    jQuery.each(this._labelCurrentTags, function (index, value) {
                        if (value.name == ochartProxy._selectedItem) {
                            ochartProxy._tagCollection.push(value);
                            ochartProxy._selectedTagInfo = value.tag;
                            ochartProxy._selectedMenuItem = value.name;
                            return false;
                        }
                    });
                    this._startDrilldown = true;
                    if (this.model.beforeServiceInvoke != null)
                        this._trigger("beforeServiceInvoke", { action: this._drillAction, element: this.element, customObject: this.model.customObject });
                    if (ochartProxy.model.progressMode == "progress")
                        $("#" + ochartProxy._id + "progressText")[0].innerHTML = ochartProxy._getLocalizedLabels("PreparingAndExecutingMDXquery") + "...";
                    var serializedCustomObject = JSON.stringify(this.model.customObject);
                    if (this.model.customObject != "" && this.model.customObject != null && this.model.customObject != undefined) {
                        if (typeof (oclientProxy) != "undefined")
                            this.doAjaxPost("POST", this.model.url + "/" + this.model.serviceMethodSettings.drillDown, JSON.stringify({ "action": "drilldown", "drilledSeries": this._selectedTagInfo, "olapReport": this.getOlapReport(), "clientReports": oclientProxy.reports, "customObject": serializedCustomObject }), this.renderControlSuccess);
                        else
                            this.doAjaxPost("POST", this.model.url + "/" + this.model.serviceMethodSettings.drillDown, JSON.stringify({ "action": "drilldown", "drilledSeries": this._selectedTagInfo, "olapReport": this.getOlapReport(), "customObject": serializedCustomObject }), this.renderControlSuccess);
                    }
                    else {
                        if (!typeof (oclientProxy) != "undefined")
                            this.doAjaxPost("POST", this.model.url + "/" + this.model.serviceMethodSettings.drillDown, JSON.stringify({ "action": "drilldown", "drilledSeries": this._selectedTagInfo, "olapReport": this.getOlapReport() }), this.renderControlSuccess);
                        else
                            this.doAjaxPost("POST", this.model.url + "/" + this.model.serviceMethodSettings.drillDown, JSON.stringify({ "action": "drilldown", "drilledSeries": this._selectedTagInfo, "olapReport": this.getOlapReport(), "clientReports": oclientProxy.reports }), this.renderControlSuccess);
                    }

                }
                else {
                    if (typeof oclientWaitingPopup != 'undefined' && oclientWaitingPopup != null) {
                        if ($("#" + oclientProxy._id + "_maxView")[0])
                            $("#" + oclientProxy._id + "_maxView").ejWaitingPopup({ showOnInit: true });
                        else
                            oclientWaitingPopup.show();
                    }
                    else {
                        if (this.model.progressMode != "infinite")
                            ochartProxy._progressStatus(0, 70, 120);
                        else
                            ochartWaitingPopup.show();
                    }
                    this._drillAction = "drillup";
                    this._selectedItem = $.trim(evt.target.innerHTML.split("-")[1]);
                    jQuery.each(this._tagCollection, function (index, value) {
                        if (value.name == ochartProxy._selectedItem) {
                            ochartProxy._selectedIndex = index + ochartProxy._dimensionIndex;
                            ochartProxy._selectedTagInfo = value.tag;
                            ochartProxy._tagCollection.splice(index, ochartProxy._tagCollection.length);
                            return false;
                        }
                    });
                    if (this._tagCollection.length == 0)
                        this._isDrilled = false;
                    this._startDrilldown = true;
                    if (this.model.beforeServiceInvoke != null)
                        this._trigger("beforeServiceInvoke", { action: this._drillAction, element: this.element, customObject: this.model.customObject });
                    if (ochartProxy.model.progressMode == "progress")
                        $("#" + ochartProxy._id + "progressText")[0].innerHTML = ochartProxy._getLocalizedLabels("PreparingAndExecutingMDXquery") + "...";
                    var serializedCustomObject = JSON.stringify(this.model.customObject);
                    if (this.model.customObject != "" && this.model.customObject != null && this.model.customObject != undefined) {
                        if (typeof (oclientProxy) != "undefined")
                            this.doAjaxPost("POST", this.model.url + "/" + this.model.serviceMethodSettings.drillDown, JSON.stringify({ "action": "drillup", "drilledSeries": this._selectedTagInfo, "olapReport": this.getOlapReport(), "clientReports": reports, "customObject": serializedCustomObject }), this.renderControlSuccess);
                        else
                            this.doAjaxPost("POST", this.model.url + "/" + this.model.serviceMethodSettings.drillDown, JSON.stringify({ "action": "drillup", "drilledSeries": this._selectedTagInfo, "olapReport": this.getOlapReport(), "customObject": serializedCustomObject }), this.renderControlSuccess);
                    }
                    else {
                        if (!typeof (oclientProxy) != "undefined")
                            this.doAjaxPost("POST", this.model.url + "/" + this.model.serviceMethodSettings.drillDown, JSON.stringify({ "action": "drillup", "drilledSeries": this._selectedTagInfo, "olapReport": this.getOlapReport() }), this.renderControlSuccess);
                        else
                            this.doAjaxPost("POST", this.model.url + "/" + this.model.serviceMethodSettings.drillDown, JSON.stringify({ "action": "drillup", "drilledSeries": this._selectedTagInfo, "olapReport": this.getOlapReport(), "clientReports": reports }), this.renderControlSuccess);
                    }
                }
            });
        },

        /**
        * This function is invoked to unwire the events registered for UI interaction.
        *
        * @private  
        */
        _unWireEvents: function () {
            $(document.body).unbind("click");
            this._off(this.element, "click", ".menuList");
        },

        /**
        * This function receives the controls update from service-end which would be utilized for rendering the widget.
        * @return jQuery
        * @example 
        * &lt;div id="OlapChart"&gt;&lt;/div&gt; <br> 
        * &lt;script&gt;
        * // Create OLAP Chart
        * $('#OlapChart').ejOlapChart({
        *       url: "OlapChartService.svc",
        *		animation: true, type: ej.olap.OlapChart.ChartTypes.Column, 
        *                        commonSeriesOptions: { type: ej.olap.OlapChart.ChartTypes.Column, tooltip: { visible: true} },
        *                        size: { height: 460, width: 950 }, primaryXAxis: { title: { text: "Fiscal Year" }, labelRotation: 0 },
        *                        primaryYAxis: { title: { text: "Customer Count"} }, legend: { visible: true, rowCount: 2 },
        *                        load: "loadTheme"
        *   });
        * var chartObj = $("#OlapChart").data("ejOlapChart");
        * chartObj.renderControlSuccess({"OlapReport": this.getOlapReport(), "JsonRecords": this.getJSONRecords()});
        * // creating OLAP Chart after Ajax request
        * &lt;/script&gt;
        * @memberof ejOlapChart
        * @instance
        */
        renderControlSuccess: function (msg) {
            ochartProxy = this;
            if (ochartProxy.model.progressMode == "progress")
                $("#" + ochartProxy._id + "progressText")[0].innerHTML = ochartProxy._getLocalizedLabels("MDXqueryExecutedSuccessfully") + "...";
            else if (typeof oclientWaitingPopup != 'undefined' && oclientWaitingPopup != null)
                if ($("#" + oclientProxy._id + "_maxView")[0])
                    $("#" + oclientProxy._id + "_maxView").ejWaitingPopup({ showOnInit: true });
                else
                    oclientWaitingPopup.show();
            try {
                if (ochartProxy.model.progressMode != "infinite") {
                    window.clearInterval(oChartTimer);
                    if (ochartProxy.model.progressMode == "progress")
                        $("#" + ochartProxy._id + "progressText")[0].innerHTML = ochartProxy._getLocalizedLabels("RenderingStarted") + "...";
                    ochartProxy._progressStatus(75, 98, 50);
                    window.setTimeout(function () { progressStatus() }, 400);
                }
                else window.setTimeout(function () { progressStatus() }, 1);

                function progressStatus() {
                    if (msg[0] != undefined) {
                        ochartProxy.setJsonRecords(msg[0].Value); ochartProxy.setOlapReport(msg[1].Value);
                        if (typeof (oclientProxy) != "undefined") {
                            oclientProxy.currentReport = msg[1].Value;
                            if (msg[2] != null && msg[2] != undefined && msg[2].Key == "ClientReports")
                                oclientProxy.reports = msg[2].Value;
                        }
                        if (msg[2] != null && msg[2] != undefined && !msg[2].Key == "ClientReports")
                            ochartProxy.model.customObject = msg[2].Value;
                    }
                    else if (msg.d != undefined) {
                        ochartProxy.setJsonRecords(msg.d[0].Value); ochartProxy.setOlapReport(msg.d[1].Value);
                        if (typeof (oclientProxy) != "undefined") {
                            oclientProxy.currentReport = msg.d[1].Value;
                            if (msg.d[2] != null && msg.d[2] != undefined && msg.d[2].Key == "ClientReports")
                                oclientProxy.reports = msg.d[2].Value;
                        }
                        if (msg.d[2] != null && msg.d[2] != undefined && !msg.d[2].Key == "ClientReports")
                            ochartProxy.model.customObject = msg.d[2].Value;
                    }
                    else {
                        ochartProxy.setJsonRecords(msg.JsonRecords); ochartProxy.setOlapReport(msg.OlapReport);
                        if (msg.customObject != null && msg.customObject != null)
                            ochartProxy.model.customObject = msg.customObject;
                        if (typeof (oclientProxy) != "undefined") {
                            if (typeof (oclientProxy.currentReport) != "undefined")
                                oclientProxy.currentReport = msg.OlapReport;
                            if (typeof (oclientProxy.reports) != "undefined")
                                oclientProxy.reports = msg.reports;
                        }
                    }
                    if (ochartProxy.model.afterServiceInvoke != null) {
                        var eventArgs;
                        if (ochartProxy._drillAction != "")
                            eventArgs = { action: ochartProxy._drillAction, element: ochartProxy.element, customObject: ochartProxy.model.customObject };
                        else
                            eventArgs = { action: ochartProxy._currentAction, element: ochartProxy.element, customObject: ochartProxy.model.customObject };
                        ochartProxy._trigger("afterServiceInvoke", eventArgs);
                    }
                    var htmlTag = ej.buildTag("div#" + ochartProxy._id + "Container")[0].outerHTML;
                    ochartProxy.element.html(htmlTag);
                    ochartProxy.renderChartFromJSON(ochartProxy.getJsonRecords());
                    if (ochartProxy.model.progressMode != "infinite") {
                        ochartProxy._progressStatus(100, 101, 50);
                        if (ochartProxy.model.progressMode == "progress")
                            $("#" + ochartProxy._id + "progressText")[0].innerHTML = ochartProxy._getLocalizedLabels("RenderingSucceeded") + "...";
                    }
                    ochartProxy._unWireEvents();
                    ochartProxy._wireEvents();
                    if (ochartProxy._drillAction != "") {
                        ochartProxy.model.currentReport = ochartProxy.getOlapReport();
                        ochartProxy._trigger("drillSuccess", ochartProxy.element);
                    }
                    if (ochartProxy.model.progressMode != "infinite" && (ochartProxy._JSONRecords == null || ochartProxy._JSONRecords.chartLables.length == 0 || ochartProxy._JSONRecords.points_Y.length == 0)) {
                        $("#" + ochartProxy._id + "progressContainer").hide();
                    }
                    else if (typeof oclientWaitingPopup != 'undefined' && oclientWaitingPopup != null) {
                        if ($("#" + oclientProxy._id + "_maxView")[0]) {
                            $("#" + oclientProxy._id + "_maxView").ejWaitingPopup({ showOnInit: false });
                            oclientWaitingPopup.hide();
                        }
                        else if (typeof oclientProxy.ogridObj != "undefined") {
                            if (oclientProxy && (oclientProxy.ogridObj._drillAction && !oclientProxy.ogridObj._startDrilldown) || oclientProxy.ochartObj._drillAction && !oclientProxy.ochartObj._startDrilldown)
                                oclientWaitingPopup.hide();
                            else if (oclientProxy && oclientProxy.ogridObj._drillAction == "" && oclientProxy.ochartObj._drillAction == "" && !oclientProxy.ogridObj._startDrilldown && !oclientProxy.ochartObj._startDrilldown)
                                oclientWaitingPopup.hide();
                            else if (oclientProxy.ochartObj._startDrilldown && !oclientProxy.ogridObj._startDrilldown && !$("#" + oclientProxy._id + "_maxView")[0])
                                oclientWaitingPopup.show();
                            else if (!oclientProxy.ochartObj._startDrilldown && !oclientProxy.ogridObj._startDrilldown && oclientProxy.ochartObj._drillAction == "" && oclientProxy.ogridObj._drillAction == "")
                                oclientWaitingPopup.hide();
                        }
                        if (oclientProxy.model.displaySettings.mode == "chartonly")
                            oclientWaitingPopup.hide();
                    }
                    else
                        ochartWaitingPopup.hide();
                    if (typeof oclientProxy != 'undefined')
                        oclientProxy.ochartObj._startDrilldown = false;
                }
            }
            catch (err) {
                if (ochartProxy.model.progressMode != "infinite") {
                    if (ochartProxy.model.progressMode == "progress")
                        $("#" + ochartProxy._id + "progressText")[0].innerHTML = ochartProxy._getLocalizedLabels("RenderingFailed") + "!!!";
                    window.clearInterval(oChartTimer);
                    ochartProgressBar.setModel({ value: 70, text: " Failed" });
                }
            }
            var eventArgs = { "action": this._currentAction, "customObject": this.model.customObject, "element": this.element };
            this._trigger("renderSuccess", eventArgs);
        },

        /**
        * This function receives the JSON formatted datasource to render the OLAP Chart control.
        * @return jQuery
        * @example 
        * &lt;div id="OlapChart"&gt;&lt;/div&gt; <br> 
        * &lt;script&gt;
        * // Create OLAP Chart
        * $('#OlapChart').ejOlapChart({
        *       url: "OlapChartService.svc",
        *		animation: true, type: ej.olap.OlapChart.ChartTypes.Column, 
        *                        commonSeriesOptions: { type: ej.olap.OlapChart.ChartTypes.Column, tooltip: { visible: true} },
        *                        size: { height: 460, width: 950 }, primaryXAxis: { title: { text: "Fiscal Year" }, labelRotation: 0 },
        *                        primaryYAxis: { title: { text: "Customer Count"} }, legend: { visible: true, rowCount: 2 },
        *                        load: "loadTheme"
        *   });
        * var chartObj = $("#OlapChart").data("ejOlapChart");
        * chartObj.renderControlFromJSON(this.getJSONRecords());
        * // render the OLAP Chart from JSON formatted data.
        * &lt;/script&gt;
        * @memberof ejOlapChart
        * @instance
        */
        renderChartFromJSON: function (jsonData) {
            window.clearInterval(oChartTimer);
            if (jsonData != null && jsonData.chartLables.length > 0 && jsonData.points_Y.length > 0) {
                var chartSeries = new Array(); var seriesPoints = new Array(); var XValues = jsonData.chartLables; var tempArray = new Array();
                var YValues = new Array(); var pointsCount = 0; var count = 0; var seriesName = new Array(); var points;

                jQuery.each(jsonData.chartLables, function (index, value) {
                    var YPoints = new Array();
                    for (var i = 0; i < jsonData.points_Y[index].length; i++) {
                        points = { "xValues": jsonData.chartLables[index], "yValues": (jsonData.points_Y[index])[i] };
                        YPoints.push(points);
                    }
                    seriesPoints.push(YPoints);
                });

                jQuery.each(jsonData.seriesNames, function (index, value) {
                    tempArray.push(new Array());
                });
                jQuery.each(jsonData.chartLables, function (index, value) {
                    for (var i = 0; i < seriesPoints[index].length; i++) {
                        tempArray[i].push((seriesPoints[index])[i]);
                    }
                });
                jQuery.each(jsonData.seriesNames, function (index, value) {
                    chartSeries[pointsCount] = { dataSource: tempArray[index], xName: "xValues", yName: ["yValues"], name: jsonData.seriesNames[pointsCount] };
                    pointsCount++;
                });
                if (jsonData.chartLables.length > 10 && this.model.zooming == "")
                    this.model.zooming = { enable: true, type: 'x,y', enableMouseWheel: true };
                else if (jsonData.chartLables.length < 10 && this._initZooming == "")
                    this.model.zooming = this._initZooming;
                if (jQuery.inArray(this.seriesType(), ["line", "spline", "area", "splinearea", "stepline", "steparea", "stackingarea"]) > -1 && !this.model.commonSeriesOptions.marker.visible)
                    this.model.commonSeriesOptions.marker = {
                        shape: ej.olap.OlapChart.SymbolShapes.Circle,
                        size: { height: 12, width: 12 },
                        visible: true,
                        connectorLine: { height: 30, type: "line" },
                        border: { width: 3, color: 'white' }
                    };
                else if ((this.seriesType() == "pie" || this.seriesType() == "pyramid")) {
                    this.model.commonSeriesOptions.marker = {
                        dataLabel: {
                            visible: true,
                            font: { color: 'white', size: '15px', fontWeight: 'lighter' }
                        }
                    };
                }
                else if (jQuery.inArray(this.seriesType(), ["column", "stackingcolumn", "bar", "stackingcolumn"]) > -1 && this.model.commonSeriesOptions.marker) {
                    if (this.model.commonSeriesOptions.marker.dataLabel)
                        this.model.commonSeriesOptions.marker.dataLabel.visible = false;
                    this.model.commonSeriesOptions.marker.visible = false;
                }
                if (this.model.crosshair.visible) {
                    this.model.commonSeriesOptions.tooltip.format = "#point.x# : #point.y#";
                }
                else
                    this.model.commonSeriesOptions.tooltip.format = (this._getLocalizedLabels("Measure")) + " : " + jsonData.measureNames + " <br/>" + (this._getLocalizedLabels("Row")) + " : #series.name# <br/>" + (this._getLocalizedLabels("Column")) + " : #point.x# <br/>" + (this._getLocalizedLabels("Value")) + " : #point.y#";
                if (this.model.progressMode != "infinite") {
                    ochartProxy._progressStatus(99, 100 + ":100%", 70);
                    window.setTimeout(function () { progressStatus() }, 300);
                }
                else
                    window.setTimeout(function () { progressStatus() }, 1);
                function progressStatus() {
                    $("#" + ochartProxy._id + "Container").ejChart(
                            {
                                border: ochartProxy.model.border,
                                chartArea: ochartProxy.model.chartArea,
                                primaryXAxis: ochartProxy.model.primaryXAxis,
                                primaryYAxis: ochartProxy.model.primaryYAxis,
                                secondaryX: ochartProxy.model.secondaryX,
                                secondaryY: ochartProxy.model.secondaryY,
                                striplineDefault: ochartProxy.model.striplineDefault,
                                title: { text: ochartProxy.titleText(),
                                    textAlignment: ochartProxy.model.title.textAlignment,
                                    font: {
                                        color: ochartProxy.model.title.color,
                                        fontFamily: ochartProxy.model.title.fontFamily,
                                        fontWeight: ochartProxy.model.title.fontWeight,
                                        opacity: ochartProxy.model.title.opacity,
                                        size: ochartProxy.model.title.size,
                                        fontStyle: ochartProxy.model.title.fontStyle
                                    }
                                },
                                lineCap: ochartProxy.model.lineCap,
                                lineJoin: ochartProxy.model.lineJoin,
                                legendAlignment: ochartProxy.model.legendAlignment,
                                legendPosition: ochartProxy.model.legendPosition,
                                legend: ochartProxy.model.legend,
                                animation: ochartProxy.model.animation,
                                crosshair: ochartProxy.model.crosshair,
                                commonSeriesOptions: {
                                    doughnutCoefficient: ochartProxy.model.commonSeriesOptions.doughnutCoefficient,
                                    explodeOffset: ochartProxy.model.commonSeriesOptions.explodeOffset,
                                    pyramidMode: ochartProxy.model.commonSeriesOptions.pyramidMode,
                                    gapRatio: ochartProxy.model.commonSeriesOptions.gapRatio,
                                    pieCoefficient: ochartProxy.model.commonSeriesOptions.pieCoefficient,
                                    doughnutSize: ochartProxy.model.commonSeriesOptions.doughnutSize,
                                    startAngle: ochartProxy.model.commonSeriesOptions.startAngle,
                                    xAxisName: ochartProxy.model.commonSeriesOptions.xAxisName,
                                    yAxisName: ochartProxy.model.commonSeriesOptions.yAxisName,
                                    explodeAll: ochartProxy.model.commonSeriesOptions.explodeAll,
                                    explodeIndex: ochartProxy.model.commonSeriesOptions.explodeIndex,
                                    tooltipOptions: ochartProxy.model.commonSeriesOptions.tooltipOptions,
                                    marker: ochartProxy.model.commonSeriesOptions.marker || ochartProxy.model.marker,
                                    font: ochartProxy.model.commonSeriesOptions.font,
                                    type: ochartProxy.seriesType(),
                                    enableAnimation:ochartProxy.model.commonSeriesOptions.enableAnimation,
                                    style: ochartProxy.model.commonSeriesOptions.style,
                                    explode: ochartProxy.model.commonSeriesOptions.explode,
                                    labelPosition: ochartProxy.model.commonSeriesOptions.labelPosition,
                                    tooltip: ochartProxy.model.commonSeriesOptions.tooltip
                                },
                                seriesStyle: ochartProxy.model.seriesStyle,
                                pointStyle: ochartProxy.model.pointStyle,
                                textStyle: ochartProxy.model.textStyle,
                                initSeriesRender: ochartProxy.model.initSeriesRender,
                                theme: ochartProxy.model.theme,
                                canResize: ochartProxy.model.canResize,
                                zooming: ochartProxy.model.zooming,
                                margin: ochartProxy.model.margin,
                                elementSpacing: ochartProxy.model.elementSpacing,
                                seriesColors: ochartProxy.model.seriesColors,
                                seriesBorderColors: ochartProxy.model.seriesBorderColors,
                                pointColors: ochartProxy.model.pointColors,
                                pointBorderColors: ochartProxy.model.pointBorderColors,
                                series: chartSeries,
                                size: ochartProxy.model.size,
                                //Events
                                load: ochartProxy.model.load,
                                axesRangeCalculate: ochartProxy.model.axesRangeCalculate,
                                axesTitleRendering: ochartProxy.model.axesTitleRendering,
                                chartAreaBoundsCalculate: ochartProxy.model.chartAreaBoundsCalculate,
                                legendItemRendering: ochartProxy.model.legendItemRendering,
                                lengendBoundsCalculate: ochartProxy.model.lengendBoundsCalculate,
                                preRender: ochartProxy.model.preRender,
                                seriesRendering: ochartProxy.model.seriesRendering,
                                symbolRendering: ochartProxy.model.symbolRendering,
                                titleRendering: ochartProxy.model.titleRendering,
                                axesLabelsInitialize: ochartProxy.model.axesLabelsInitialize,
                                pointRegionMouseMove: ochartProxy.model.pointRegionMouseMove,
                                legendItemClick: ochartProxy.model.legendItemClick,
                                legendItemMouseMove: ochartProxy.model.legendItemMouseMove,
                                displayTextRendering: ochartProxy.model.displayTextRendering,
                                toolTipInitialize: ochartProxy.model.toolTipInitialize,
                                trackAxisToolTip: ochartProxy.model.trackAxisToolTip,
                                trackToolTip: ochartProxy.model.trackToolTip,
                                animationComplete: ochartProxy.model.animationComplete,
                                destroy: ochartProxy.model.destroy,
                                create: ochartProxy.model.create,
                                axesLabelRendering: ej.proxy(ochartProxy._onLabelRenders, ochartProxy),
                                pointRegionClick: ej.proxy(ochartProxy._onSeriesClick, ochartProxy)
                            });
                    $("#" + ochartProxy._id + "progressContainer").hide();
                }
                ochartProxy._labelCurrentTags.splice(0, ochartProxy._labelCurrentTags.length);
                jQuery.each(jsonData.labelTags, function (index, value) {
                    var currentTag = new Object();
                    var splitData = value.split("::");
                    currentTag = { "name": splitData[2], "state": splitData[splitData.length - 1], "tag": value };
                    ochartProxy._labelCurrentTags.push(currentTag);
                });
              

                    //jQuery.each(ochartProxy._labelCurrentTags, function (index, value) {
                    //       if (value.name == ochartProxy._selectedItem) {
                    //        ochartProxy._selectedTagInfo = ochartProxy._labelCurrentTags[index].tag;
                    //        return false;
                    //    }
                    //});
             
            }
            if (ochartWaitingPopup != null && ochartProxy.model.progressMode == "infinite")
                this.element.ejWaitingPopup({ showOnInit: false });
            else
                $("#" + ochartProxy._id + "progressContainer").hide();
        },

        /**
        * This function would fetch the label text which could be seen beneath each series.
        *
        * @private  
        */
        _onLabelRenders: function (args) {
            ochartProxy._trigger("axesLabelRendering", args);
            if ((args.data.axis.orientation.toLowerCase() == "horizontal" && args.data.label.Text != undefined) || ((ochartProxy.seriesType() == "bar" || ochartProxy.seriesType() == "stackingbar") && args.data.axis.orientation.toLowerCase() == "vertical" && args.data.label.Text != undefined)) {
                var splitName = args.data.label.Text.toString().split("~~");
                if (ochartProxy._tagCollection.length == 0)
                    args.data.label.Text = splitName[splitName.length - 1];
                else if (ochartProxy._drillAction == "drilldown" && splitName.indexOf(ochartProxy._selectedItem) + 1 < splitName.length)
                    args.data.label.Text = splitName[splitName.indexOf(ochartProxy._selectedItem) + 1];
                else if (ochartProxy._drillAction == "drilldown" && splitName.indexOf(ochartProxy._selectedItem) + 1 >= splitName.length)
                    args.data.label.Text = splitName[splitName.indexOf(ochartProxy._selectedItem)];
                else if (splitName.length == 1)
                    args.data.label.Text = splitName[0];
                else if (ochartProxy._drillAction == "drillup")
                    args.data.label.Text = splitName[ochartProxy._selectedIndex];
                else
                    args.data.label.Text = splitName[splitName.length - 1];
            }
        },

        /**
        * To get the localized labels.
        * @private
        */
        //get localized text
        _getLocalizedLabels: function (property) {

            return ej.olap.OlapChart.locale[this.locale()][property] === undefined ? ej.olap.OlapChart.locale["en-US"][property] : ej.olap.OlapChart.locale[this.locale()][property];
        },

        /**
        * This function is used to create progress bar containing status text and percentage.
        *
        * @private  
        */
        _createProgressBar: function () {
            var progressContainer = ej.buildTag("div#" + this._id + "progressContainer.progressContainer", "", { top: typeof (oclientProxy) != "undefined" ? ((oclientProxy.model.displaySettings.controlPlacement == "tab" || oclientProxy.model.displaySettings.mode != "chartAndGrid") ? oclientProxy.progressPos.top + 300 : (oclientProxy.model.displaySettings.defaultView == "chart" ? oclientProxy.progressPos.top + 200 : oclientProxy.progressPos.top + 500)) : ($("#" + this._id).position().top + ($("#" + this._id).height() / 2) - 35), left: typeof (oclientProxy) != "undefined" ? oclientProxy.progressPos.left + 550 : ($("#" + this._id).position().left + ($("#" + this._id).width() / 3) - 50), zIndex: 999 })[0].outerHTML;
            var progressBar = ej.buildTag("div#" + this._id + "progressBar.progressBar", "")[0].outerHTML;
            var blockControl = ej.buildTag("div#" + this._id + "blockDiv.blockDiv", "", { width: typeof (oclientProxy) != "undefined" ? ($("#" + oclientProxy._id).width()) : $("#" + this._id).width(), display: "block", opacity: "0.3", backgroundColor: "#E9E9E9", height: typeof (oclientProxy) != "undefined" ? ($("#" + oclientProxy._id).height()) : $("#" + this._id).height(), position: 'fixed', top: typeof (oclientProxy) != "undefined" ? oclientProxy.progressPos.top : ($("#" + this._id).position().top), left: typeof (oclientProxy) != "undefined" ? oclientProxy.progressPos.left : ($("#" + this._id).position().left) })[0].outerHTML;
            $("#" + ochartProxy._id).after(progressContainer);
            if (ochartProxy.model.progressMode == "normal") {
                $("#" + this._id + "progressContainer").css({ height: 0, width: 0 });
                $("#" + this._id + "progressContainer").removeClass(".progressContainer");
                $("#" + ochartProxy._id + "progressContainer").append(progressBar).append(blockControl);
            }
            else {
                var progressText = ej.buildTag("span#" + this._id + "progressText.progressText", "OlapChart initial loading...")[0].outerHTML;
                $("#" + ochartProxy._id + "progressContainer").append(progressBar).append(progressText).append(blockControl);
            }
            $("#" + ochartProxy._id + "progressBar").ejProgressBar({ value: 0, text: 0 + " %", complete: "progressbarCompleted" });
            ochartProgressBar = $("#" + ochartProxy._id + "progressBar").data("ejProgressBar");
        },

        /**
        * This function is used to update the progress status.
        *
        * @private  
        */
        _progressStatus: function (current_Status, completion_Status, progress_Speed) {
            window.clearInterval(oChartTimer);
            if (jQuery.type(completion_Status) === "string") {
                var progressText = completion_Status.split(':');
                if (progressText != undefined) completion_Status = parseInt(progressText[0]);
            }
            ochartProgressBar.setModel({ value: current_Status, text: current_Status + " %" });
            $("#" + ochartProxy._id + "progressContainer").show();
            oChartTimer = window.setInterval(function () {
                ochartProgressBar.setModel({ value: current_Status, text: current_Status + " %" });
                current_Status++;
                if (current_Status >= completion_Status) {
                    window.clearInterval(oChartTimer);
                    if (progressText != undefined)
                        ochartProgressBar.setModel({ value: 100, text: progressText[1] });
                };
            }, progress_Speed);
        },

        /**
        * This function would be raised while clicking the series.
        *
        * @private  
        */
        _onSeriesClick: function (sender) {
            ochartProxy = this; var tempLabel; var selectedLabel; var pointIndex;
            var selectedTags = new Array(); var menuList = new Array();
            var chartOffset = this.element.position();
            if (sender.data.region != null && sender.data.region != undefined) {
                selectedLabel = sender.model.primaryXAxis.labels[sender.data.region.Region.PointIndex];
                pointIndex = sender.data.region.Region.PointIndex;
            }
            else if (sender.data.point.x != null && sender.data.point.x != undefined && sender.data.point.x != "") {
                selectedLabel = sender.data.series.xAxis.labels[sender.data.point.X];
                pointIndex = sender.data.point.X;
            }

            if ($($("#" + ochartProxy._id + "Container_vml_XAxisLabels_0")[0]).children()[pointIndex] != undefined)
                tempLabel = $($("#" + ochartProxy._id + "Container_vml_XAxisLabels_0")[0]).children()[pointIndex].innerHTML;
            else if ($("#" + ochartProxy._id + "Container_svg_XAxisLabels_0").children()[pointIndex])
                tempLabel = $("#" + ochartProxy._id + "Container_svg_XAxisLabels_0").children()[pointIndex].textContent;
            else if (this.seriesType() == "bar" || this.seriesType() == "stackingbar") {
                if ($($("#" + ochartProxy._id + "Container_vml_YAxisLabels_0")[0]).children()[pointIndex] != undefined)
                    tempLabel = $($("#" + ochartProxy._id + "Container_vml_YAxisLabels_0")[0]).children()[pointIndex].innerHTML;
                else if ($("#" + ochartProxy._id + "Container_svg_YAxisLabels_0").children()[pointIndex])
                    tempLabel = $("#" + ochartProxy._id + "Container_svg_YAxisLabels_0").children()[pointIndex].textContent;
                else {
                    var temp = sender.model.primaryXAxis.labels[pointIndex].split("~~");
                    tempLabel = temp[temp.indexOf(ochartProxy._selectedMenuItem) + 1];
                }
            }
            else {
                var temp = sender.model.primaryXAxis.labels[pointIndex].split("~~");
                tempLabel = temp[temp.indexOf(ochartProxy._selectedMenuItem) + 1];
            }

            if (tempLabel) {
                $("#" + ochartProxy._id).find("#" + ochartProxy._id + "ExpandMenu, .expandMenu, .e-dialog").remove();
                var splitLabel = selectedLabel.split("~~");

                jQuery.each(splitLabel, function (index, value) {
                    jQuery.each(ochartProxy._labelCurrentTags, function (key, val) {
                        if ($.trim(value) == val.name) {
                            selectedTags.push(val);
                            return false;
                        }
                    });
                });

                jQuery.each(selectedTags, function (index, value) {
                    if (value.state > 1 && ochartProxy._tagCollection.length == 0)
                        menuList.push($(ej.buildTag("li.menuList", (ochartProxy._getLocalizedLabels("Expand")) + " - " + value.name)[0]).attr("role", "presentation")[0].outerHTML);
                    else if (value.state > 1 && ochartProxy._tagCollection.length > 0 && value.name == tempLabel) {
                        menuList.push($(ej.buildTag("li.menuList", (ochartProxy._getLocalizedLabels("Expand")) + " - " + value.name)[0]).attr("role", "presentation")[0].outerHTML);
                    }
                });
                if (ochartProxy._tagCollection.length > 0) {
                    jQuery.each(ochartProxy._tagCollection, function (index, value) {
                        for (var i = 0; i < menuList.length; i++) {
                            if ($.trim(menuList[i].split('-')[1].replace("</li>", "")).replace("</LI>", "") == value.name)
                                menuList.splice(i, 1);
                        }
                        menuList.push($(ej.buildTag("li.menuList", (ochartProxy._getLocalizedLabels("Collapse")) + " - " + value.name)[0]).attr("role", "presentation")[0].outerHTML);
                    });
                }
                else if (!ochartProxy._JSONRecords.addInfo.namedsetEnabled) {
                    jQuery.each(selectedTags, function (index, value) {
                        if (value.state == 1) {
                            menuList.push($(ej.buildTag("li.menuList", ochartProxy._getLocalizedLabels("Collapse") + " - " + value.name)[0]).attr("role", "presentation")[0].outerHTML);
                            ochartProxy._tagCollection.push(value);
                        }
                    });
                }
                menuList.push($(ej.buildTag("li.menuList", ochartProxy._getLocalizedLabels("Exit"))[0]).attr("role", "presentation")[0].outerHTML);
                var expandMenu = $(ej.buildTag("div#" + ochartProxy._id + "ExpandMenu.expandMenu", menuList)[0]).attr("role", "presentation")[0].outerHTML;
                $(expandMenu).ejDialog({ width: "auto", content: "#" + ochartProxy._id, enableResize: false });
                $("#" + ochartProxy._id + "ExpandMenu_wrapper").appendTo(this.element).css({ "left": sender.data.location.x + 8 + chartOffset.left, "top": sender.data.location.y + 8 + chartOffset.top });
                this.element.find(".e-titlebar, .e-header").remove();
            }
            ochartProxy._trigger("pointRegionClick", sender);
        },

        /**
        * Perform an asynchronous HTTP (Ajax) request.
        * @return jQuery
        * @example 
        * &lt;div id="OlapChart"&gt;&lt;/div&gt; <br> 
        * &lt;script&gt;
        * // Create OLAP Chart
        * $('#OlapChart').ejOlapChart({
        *       url: "OlapChartService.svc",
        *		animation: true, type: ej.olap.OlapChart.ChartTypes.Column, 
        *                        commonSeriesOptions: { type: ej.olap.OlapChart.ChartTypes.Column, tooltip: { visible: true} },
        *                        size: { height: 460, width: 950 }, primaryXAxis: { title: { text: "Fiscal Year" }, labelRotation: 0 },
        *                        primaryYAxis: { title: { text: "Customer Count"} }, legend: { visible: true, rowCount: 2 },
        *                        load: "loadTheme"
        *   });
        * var chartObj = $("#OlapChart").data("ejOlapChart");
        * chartObj.doAjaxPost("POST", "/OlapChartService.svc/Initialize", {"key", "Hello World"}, "renderControlSuccess", null);
        * // initiate an Ajax request
        * &lt;/script&gt;
        * @memberof ejOlapChart
        * @instance
        */
        doAjaxPost: function (type, url, data, onSuccess) {
            $.ajax({
                type: type,
                url: url,
                data: data,
                contentType: 'application/json; charset=utf-8',
                dataType: 'json',
                success: $.proxy(onSuccess, this),
                complete: ej.proxy(function () {
                    var eventArgs = { "action": this._currentAction, "customObject": this.model.customObject, "element": this.element };
                    this._trigger("renderComplete", eventArgs);
                }, this),
                error: ej.proxy(function (msg, textStatus, errorThrown) {
                    if (this.model.progressMode != "infinite") {
                        if (this.model.progressMode == "progress")
                            $("#" + this._id + "progressText")[0].innerHTML = this._getLocalizedLabels("MDXqueryExecutionFailed") + "!!!";
                        window.clearInterval(oChartTimer);
                        ochartProgressBar.setModel({ value: 10, text: " Failed" });
                    }
                    else
                        ochartWaitingPopup.hide();
                    var eventArgs = { "action": this._currentAction, "customObject": this.model.customObject, "element": this.element, "Message": msg };
                    this._trigger("renderFailure", eventArgs);
                }, this)
            });
        }
    });

    //OLAP Chart Localization
    ej.olap.OlapChart.locale = {};
    //Default english localized values
    ej.olap.OlapChart.locale["en-US"] = {
        Measure: "Measure",
        Row: "Row",
        Column: "Column",
        Value: "Value",
        Expand: "Expand",
        Collapse: "Collapse",
        Exit: "Exit",
        MDXqueryExecutionFailed: "MDX query execution failed",
        PreparingAndExecutingMDXquery: "Preparing and executing MDX query",
        MDXqueryExecutedSuccessfully: "MDX query executed successfully",
        RenderingSucceeded: "Rendering Succeeded",
        RenderingStarted: "Rendering Started",
        RenderingFailed: "Rendering failed"
    };

    /**
    * Enum for Chart Types. Allows the user to select a appropriate chart type. 
    * @enum {String}
    * @global
    */
    ej.olap.OlapChart.ChartTypes = {
        Line: 'line', Spline: 'spline', Column: 'column', Area: 'area', SplineArea: 'splinearea', StepLine: 'stepline', StepArea: 'steparea', Pie: 'pie', Bar: 'bar', StackingArea: 'stackingarea', StackingColumn: 'stackingcolumn', StackingBar: 'stackingbar', Pyramid: 'pyramid', Funnel : 'funnel'
    };

    /**
    * Enum for Progress Mode. Allows the user to select a appropriate progress indicator. 
    * @enum {String}
    * @global
    */
    ej.olap.OlapChart.ProgressMode = {
        Infinite: "infinite",
        Normal: "normal",
        Progress: "progress"
    };

    /**
    * Enum for Shapes. Allows the user to select a appropriate chart symbol shape. 
    * @enum {String}
    * @global
    */
    ej.olap.OlapChart.SymbolShapes = {
        None: "none",
        LeftArrow: "leftarrow",
        RightArrow: "rightarrow",
        Circle: "circle",
        Cross: "cross",
        HorizLine: "horizline",
        VertLine: "vertline",
        Diamond: "diamond",
        Rectangle: "rectangle",
        Triangle: "triangle",
        InvertedTriangle: "invertedtriangle",
        Hexagon: "hexagon",
        Pentagon: "pentagon",
        Star: "star",
        Ellipse: "ellipse",
        Wedge: "wedge",
        Trapezoid: "trapezoid",
        UpArrow: "uparrow",
        DownArrow: "downarrow",
        Image: "image"
    };

})(jQuery, Syncfusion);;
/**
* @fileOverview Plugin to style the Html OLAP Gauge elements
* @copyright Copyright Syncfusion Inc. 2001 - 2013. All rights reserved.
*  Use of this code is subject to the terms of our license.
*  A copy of the current license can be obtained at any time by e-mailing
*  licensing@syncfusion.com. Any infringement will be prosecuted under
*  applicable laws. 
* @version 12.1 
* @author <a href="mailto:licensing@syncfusion.com">Syncfusion Inc</a>
*/

(function ($, ej, undefined) {
    /**
    * @namespace ej
    * @class ejOlapGauge
    * @requires jquery-1.10.2.min.js
    * @requires jquery.easing.1.3.min.js
    * @requires jquery.globalize.min.js
    * @requires ej.core.js
    * @requires ej.data.js
    * @requires ej.waitingpopup.js
    * @requires ej.circulargauge.js
    * @requires ej.olapgauge.js
    * @classdesc Custom Design for Html OLAP Gauge control.
    * @example 
    * &lt;div id="OlapGauge"&gt;&lt;/div&gt; <br> 
    * &lt;script&gt;
    * // Create OLAP Gauge
    * $("#OlapGauge").ejOlapGauge(...); 	
    * &lt;/script&gt;
    */
    ej.widget("ejOlapGauge", "ej.olap.OlapGauge", {

        //Root Css Class
        _rootCSS: "e-olapgauge",

        // Widget element will be automatically set in this
        element: null,

        // User-defined model will be automatically set in this
        model: null,

        validTags: ["div", "span"],

        defaults: $.extend({}, ej.datavisualization.CircularGauge.prototype.defaults, {
            /**		
            * Connects the service using the specified URL for any server updates.
            * @default ""
            * @type {string}
            * @example 
            * //To set url API value during initialization  
            * 	$("#OlapGauge").ejOlapGauge({  url: "/OlapGaugeService.svc"});					* 
            * @example 
            * //Get or set the url API, after initialization:<br>
            *	//Gets the url value  
            *	$("#OlapGauge").ejOlapGauge("option", "url");<br>			
            *	//Sets the url value 
            *	$("#OlapGauge").ejOlapGauge("option", "url",  "/OlapGaugeService" ); 
            * @memberof ejOlapGauge
            * @instance
            */
            url: "",
            /**		
            * Specify the CSS class to OLAP Gauge to achieve custom theme.
            * @default ""
            * @type {string}
            * @example 
            * // Set the CSS class during initialization. 			
            * 	$("#OlapGauge").ejOlapGauge({  cssClass : "gradient-lime" });			* 
            * @example 
            * //Get or set the CSS class after initialization:<br>
            *	// Gets the CSS class value.		
            *	$("#OlapGauge").ejOlapGauge("option", "cssClass");			
            *	// Sets the CSS class to OLAP Gauge
            *	$("#OlapGauge").ejOlapGauge("option", "cssClass",  "gradient-lime" );
            * @memberof ejOlapGauge
            * @instance
            */
            cssClass: "",
            /**		
            * Sets the number of rows in OLAP Gauges for its arrangement.
            * @default 0
            * @type {number}
            * @example 
            * //To set rowsCount API value during initialization  
            * 	$("#OlapGauge").ejOlapGauge({  rowsCount: 1});					* 
            * @example 
            * //Get or set the rowsCOunt API, after initialization:<br>
            *	//Gets the rowsCount value  
            *	$("#OlapGauge").ejOlapGauge("option", "rowsCount");<br>			
            *	//Sets the rowsCount value 
            *	$("#OlapGauge").ejOlapGauge("option", "rowsCount", 2); 
            * @memberof ejOlapGauge
            * @instance
            */
            rowsCount: 0,
            /**		
            * Sets the number of colunsCount in OLAP Gauges for its arrangement.
            * @default 0
            * @type {number}
            * @example 
            * //To set columnsCount API value during initialization  
            * 	$("#OlapGauge").ejOlapGauge({  columnsCount: 1});					* 
            * @example 
            * //Get or set the columnsCount API, after initialization:<br>
            *	//Gets the columnsCount value  
            *	$("#OlapGauge").ejOlapGauge("option", "columnsCount");<br>			
            *	//Sets the columnsCount value 
            *	$("#OlapGauge").ejOlapGauge("option", "columnsCount", 2 ); 
            * @memberof ejOlapGauge
            * @instance
            */
            columnsCount: 0,
            /**		
            * Enables/disables tooltip in OLAP Gauge.
            * @default false
            * @type {boolean}
            * @example 
            * //To set enableTooltip API value during initialization  
            * 	$("#OlapGauge").ejOlapGauge({  enableTooltip: true});					* 
            * @example 
            * //Get or set the enableTooltip API, after initialization:<br>
            *	//Gets the enableTooltip value  
            *	$("#OlapGauge").ejOlapGauge("option", "enableTooltip");<br>			
            *	//Sets the enableTooltip value 
            *	$("#OlapGauge").ejOlapGauge("option", "enableTooltip", false ); 
            * @memberof ejOlapGauge
            * @instance
            */
            enableTooltip: false,
            /**		
            * Enables/disables the header labels in OLAP Gauge.
            * @default true
            * @type {boolean}
            * @example 
            * //To set showHeaderLabel API value during initialization  
            * 	$("#OlapGauge").ejOlapGauge({  showHeaderLabel: false});					* 
            * @example 
            * //Get or set the showHeaderLabel API, after initialization:<br>
            *	//Gets the showHeaderLabel value  
            *	$("#OlapGauge").ejOlapGauge("option", "showHeaderLabel");<br>			
            *	//Sets the showHeaderLabel value 
            *	$("#OlapGauge").ejOlapGauge("option", "showHeaderLabel", true); 
            * @memberof ejOlapGauge
            * @instance
            */
            showHeaderLabel: true,
            /**		
            * Sets the scale values such as pointers, indicators, etc... for OLAP Gauge.
            * @default null
            * @type {object}
            * @example 
            * //To set scales API value during initialization  
            * 	$("#OlapGauge").ejOlapGauge({  scales: {showRanges: true, showIndicators: true}});					* 
            * @example 
            * //Get or set the scales API, after initialization:<br>
            *	//Gets the scales value  
            *	$("#OlapGauge").ejOlapGauge("option", "scales");<br>			
            *	//Sets the scales value 
            *	$("#OlapGauge").ejOlapGauge("option", "scales", {showRanges: true, showIndicators: true}); 
            * @memberof ejOlapGauge
            * @instance
            */
            scales: ej.datavisualization.CircularGauge.prototype._defaultScaleValues(),
            /**		
            * Custom object to pass additional information between client-end and service-end.
            * @default null
            * @type {object}
            * @example 
            * //To set customObject API value during initialization  
            * 	$("#OlapGauge").ejOlapGauge({  customObject: {"key":"Hello World"}});					* 
            * @example 
            * //Get or set the customObject API, after initialization:<br>
            *	//Gets the customObject value  
            *	$("#OlapGauge").ejOlapGauge("option", "customObject");<br>			
            *	//Sets the customObject value 
            *	$("#OlapGauge").ejOlapGauge("option", "customObject",  {"key":"Hi Syncfusion!"} ); 
            * @memberof ejOlapGauge
            * @instance
            */
            customObject: {},
            /**		
            * Sets the progress indicator type which arises on any UI operation.	See {@link progressMode}
            * @default ej.olap.OlapGauge.ProgressMode.Infinite
            * @type {enum}
            * @example 
            * //To set progressMode API value during initialization  
            * 	$("#OlapGauge").ejOlapGauge({  progressMode: ej.olap.OlapGauge.ProgressMode.Normal});					* 
            * @example 
            * //Get or set the progressMode API, after initialization:<br>
            *	//Gets the progressMode value  
            *	$("#OlapGauge").ejOlapGauge("option", "progressMode");<br>			
            *	//Sets the progressMode value 
            *	$("#OlapGauge").ejOlapGauge("option", "progressMode",  ej.olap.OlapGauge.ProgressMode.Progress ); 
            * @memberof ejOlapGauge
            * @instance
            */
            progressMode: "infinite",
            /**		
            * Sets the localized language for the control.
            * @default "en-US"
            * @type {string}
            * @example 
            * //To set localization API value during initialization  
            * 	$("#OlapGauge").ejOlapGauge({  locale: "fr-FR"});					* 
            * @example 
            * //Get or set the localization API, after initialization:<br>
            *	//Gets the localization value  
            *	$("#OlapGauge").ejOlapGauge("option", "locale");<br>			
            *	//Sets the localization value 
            *	$("#OlapGauge").ejOlapGauge("option", "locale",  "fr-FR" ); 
            * @memberof ejOlapGauge
            * @instance
            */
            locale: "en-US",
            /**		
            * Allows the user to set custom name for the methods at service-end invoked on AJAX post.
            * @default null
            * @type {object}
            * @example 
            * /To set serviceMethodSettings API value, to invoke the appropriate service method on UI operation.  
            * $("#OlapGauge").ejOlapGauge({  serviceMethodSettings: {initialize: "MyMethod"});
            * @example 
            * //Gets or sets the serviceMethodSettings API, to invoke the appropriate service method on UI operation:<br>
            * //Gets the serviceMethodSettings value  
            * $("#OlapGauge").ejOlapGauge("option", "serviceMethodSettings");<br>			
            * //Sets the serviceMethodSettings value 
            * $("#OlapGauge").ejOlapGauge("option", "serviceMethodSettings",  {initialize: "MyMethod"} ); 
            * @memberof ejOlapGauge
            * @instance
            */
            serviceMethodSettings: {
                /**		
                * Allows the user to set the custom name for the service method that’s responsible for initializing OLAP Guage.
                * @default "InitializeGuage"
                * @type {string}
                * @example 
                * //To set initialize API value, to invoke the corresponding service method for OLAP Gauge initialization.
                *  $("#OlapGauge").ejOlapGauge({  serviceMethodSettings: {initialize: "InitializeGuageMyMethod"});
                * @example 
                * //Gets or sets the initialize API, to invoke the corresponding service method for OLAP Gauge initialization:<br>
                * //Gets the initialize value  
                * $("#OlapGauge").ejOlapGauge("option", "serviceMethodSettings");<br>   
                * //Sets the initialize value 
                * $("#OlapGauge").ejOlapGauge("option", "serviceMethodSettings.initialize", "InitializeGuageMyMethod" ); 
                * @alias ejOlapGauge#serviceMethodSettings->initialize
                * @instance
                */
                initialize: "InitializeGauge"
            },
            /**     
            * Fires when OLAP Gauge control successfully reaches client script after any Ajax request.
            * @event
            * @name ejOlapGauge#renderSuccess 	
            * @param {Object} argument Event parameters from OLAP Gauge.   
            * @param {string}  argument.element returns the outer HTML of OLAP Gauge control.
            * @param {Object} argument.customObject returns the custom object bounded with the control.
            * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
            * @param {object}  argument.model returns the OLAP Gauge model.
            * @param {string}  argument.type returns the name of the event.
            * @example 
            * //renderSuccess event for OLAP Gauge.
            * $("#OlapGauge").ejOlapGauge({
            *    renderSuccess: function (args) {}
            * });      
            * @memberof ejOlapGauge
            * @instance
            */
            renderSuccess: null,
            /**     
            * Fires when OLAP Gauge control completes all operations at client script after any Ajax request.
            * @event
            * @name ejOlapGauge#renderComplete	
            * @param {Object} argument Event parameters from OLAP Gauge.
            * @param {string}  argument.element returns the outer HTML of OLAP Gauge control.
            * @param {Object} argument.customObject returns the custom object bounded with the control.
            * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
            * @param {object}  argument.model returns the OLAP Gauge model.
            * @param {string}  argument.type returns the name of the event.
            * @example 
            * //renderCompleteevent for OLAP Gauge
            * $("#OlapGauge").ejOlapGauge({
            *    renderComplete: function (args) {}
            * });      
            * @memberof ejOlapGauge
            * @instance
            */
            renderComplete: null,
            /**     
            * Fires when error occurs during Ajax request.
            * @event
            * @name ejOlapGauge#renderFailure 	
            * @param {Object} argument Event parameters from OLAP Gauge.
            * @param {string}  argument.element returns the outer HTML of OLAP Gauge control.
            * @param {Object} argument.customObject returns the custom object bounded with the control.
            * @param {Object} argument.message returns the error message with error code.
            * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
            * @param {object}  argument.model returns the OLAP Gauge model.
            * @param {string}  argument.type returns the name of the event.
            * @param {object}  argument.responseJSON returns the JSON formatted response while error occurs.
            * @example 
            * //renderFailure event for OLAP Gauge
            * $("#OlapGauge").ejOlapGauge({
            *    renderFailure: function (args) {}
            * });      
            * @memberof ejOlapGauge
            * @instance
            */
            renderFailure: null,
            /**     
            * Fires before any Ajax request passed from OLAP Gauge to service methods.
            * @event
            * @name ejOlapGauge#beforeServiceInvoke 	
            * @param {Object} argument Event parameters from OLAP Gauge.
            * @param {string}  argument.action return the current action of OLAP Gauge control.			 
            * @param {object}  argument.customObject return the custom object bounds with OLAP Gauge control.
            * @param {string}  argument.element return the outer HTML of OLAP Gauge control.
            * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
            * @param {object}  argument.model returns the OLAP Gauge model.
            * @param {string}  argument.type returns the name of the event.
            * @example 
            * //beforeServiceInvoke event for OLAP Gauge
            * $("#OlapGauge").ejOlapGauge({
            *    beforeServiceInvoke: function (args) {}
            * });      
            * @memberof ejOlapGauge
            * @instance
            */
            beforeServiceInvoke: null,
            /**     
            * Fires when it reaches client script after any Ajax request.
            * @event
            * @name ejOlapGauge#afterServiceInvoke 	
            * @param {Object} argument Event parameters from OLAP Gauge.
            * @param {string}  argument.action return the current action of OLAP Gauge control.			 
            * @param {object}  argument.customObject return the custom object bounds with OLAP Gauge control.
            * @param {string}  argument.element return the outer HTML of OLAP Gauge control.
            * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
            * @param {object}  argument.model returns the OLAP Gauge model.
            * @param {string}  argument.type returns the name of the event.
            * @example 
            * //afterServiceInvoke event for OLAP Gauge
            * $("#OlapGauge").ejOlapGauge({
            *    afterServiceInvoke: function (args) {}
            * });      
            * @memberof ejOlapGauge
            * @instance
            */
            afterServiceInvoke: null,
            /**     
            * Fires when gauge started loading at client-side.
            * @event
            * @name ejOlapGauge#load 	
            * @param {Object} argument Event parameters from OLAP Gauge.  
            * @param {boolean}  argument.cancel if the event should be canceled; otherwise, false.
            * @param {object}  argument.model returns the OLAP Gauge model.
            * @param {string}  argument.type returns the name of the event.			 
            * @example 
            * //load event for OLAP Gauge
            * $("#OlapGauge").ejOlapGauge({
            *    load: function (args) {}
            * });      
            * @memberof ejOlapGauge
            * @instance
            */
            load: null
        }),

        /**
        * Specify the data types for default properties 
        * @private
        */
        dataTypes: {
            serviceMethodSettings: "data",
            customObject: "data",
            scales: "data",
            progressMode: "enum"
        },

        observables: ["rowsCount", "columnsCount", "showHeaderLabel", "locale", "radius", "frameType"],
        rowsCount: ej.util.valueFunction("rowsCount"),
        columnsCount: ej.util.valueFunction("columnsCount"),
        showHeaderLabel: ej.util.valueFunction("showHeaderLabel"),
        locale: ej.util.valueFunction("locale"),
        radius: ej.util.valueFunction("radius"),
        frameType: ej.util.valueFunction("frameType"),

        /**
        * Gets the serialized current OLAP Report of the control.
        *
        * @private 
        */
        //get the current OLAP Report
        getOlapReport: function () {
            return this._olapReport;
        },

        /**
        * Sets the serialized current OLAP Report of the control.
        *
        * @private 
        */
        //set the current OLAP Report
        setOlapReport: function (value) {
            this._olapReport = value;
        },

        /**
        * Gets the JSON formatted records for the control.
        *
        * @private 
        */
        //get the JSON formatted records
        getJSONRecords: function () {
            return this._JSONRecords;
        },

        /**
        * Sets the JSON formatted records for the control.
        *
        * @private 
        */
        //set the JSON formatted records
        setJSONRecords: function (value) {
            this._JSONRecords = $.parseJSON(value);
        },

        // constructor function
        /**
        * Initializes the variables and loads the widget. 
        *
        * @private
        */
        _init: function () {
            this.model = $.extend(ej.datavisualization.CircularGauge.prototype.defaults, this.model);
            this._scalesInitialize();
            this._initPrivateProperties();
            this._load();
        },

        _scalesInitialize: function () {
            if (this.model.scales != null) {
                $.each(this.model.scales, ej.proxy(function (index, element) {
                var obj = ej.datavisualization.CircularGauge.prototype._defaultScaleValues();
                    $.extend(obj, element);
                    $.extend(element, obj);
                }, this));
            }
            else {
                this.model.scales = [ej.datavisualization.CircularGauge.prototype._defaultScaleValues()];
            }
        },

        /**
        * Destroy the OLAP Gauge widget
        * all events bound using this._on will be unbind automatically and bring the control to pre-init state.
        * @alias destroy
        * @return jQuery
        * @example 
        * &lt;div id="OlapGauge1"&gt;&lt;/div&gt; <br> 
        * &lt;script&gt;
        * // Create OLAP Gauge
        * var gaugeObj = $("#OlapGauge").data("ejOlapGauge");
        * gaugeObj.destroy(); // destroy the OLAP Gauge
        * &lt;/script&gt;
        * @example 
        * &lt;div id="OlapGauge"&gt;&lt;/div&gt; <br> 
        * &lt;script&gt;
        * // enable the OLAP Gauge
        * $("#OlapGauge").ejOlapGauge("destroy");	
        * &lt;/script&gt;
        * @memberof ejOlapGauge
        * @instance
        */
        _destroy: function () {
            this._unWireEvents();
            this.element.empty().removeClass("e-olapgauge" + this.model.cssClass);
            if (this._ogaugeWaitingPopup != undefined)
                this._ogaugeWaitingPopup._destroy();
        },

        /**
        * To configure the private properties of OLAP Gauge		
        * @private
        */
        _initPrivateProperties: function () {
            this._id = this.element.attr("id");
            this._olapReport = "";
            this._JSONRecords = null;
            this._maximum = new Array();
            this._maxValue = 0;
            this._pointerValue_0 = 0;
            this._pointerValue_1 = 0;
            this._currentLayout = "";
            this._defaultArea = 0;
            this._defaultWidth = 0;
            this._gaugeObj = {};
            this._ogaugeWaitingPopup = null;
            this._ogaugeProgressBar = null;
            this._ogaugeProgressText = null;
            this._ogaugeTimer = null;
        },

        /**
        * This function is the point were loading/rendering of the OLAP Gauge widget is initiated.
        *
        * @private  
        */
        _load: function () {
            this.element.addClass(this.model.cssClass);
            if (this.model.progressMode == "infinite") {
                $("#" + this._id).ejWaitingPopup({ showOnInit: true });
                this._ogaugeWaitingPopup = $("#" + this._id).data("ejWaitingPopup");
            }
            else {
                $("#" + this._id + "progressContainer").remove();
                this._createProgressBar();
                this._progressStatus(0, 70, 140);
            }

            if (this.model.beforeServiceInvoke != null)
                this._trigger("beforeServiceInvoke", { element: this.element, customObject: this.model.customObject });
            var serializedCustomObject = JSON.stringify(this.model.customObject);
            if (this.model.progressMode == "progress")
                $("#" + this._id + "progressText")[0].innerHTML = this._getLocalizedLabels("PreparingAndExecutingMDXquery");
            if (this.model.customObject != null && this.model.customObject != undefined && this.model.customObject != {})
                this.doAjaxPost("POST", this.model.url + "/" + this.model.serviceMethodSettings.initialize, JSON.stringify({ "action": "initializeGauge", "customObject": serializedCustomObject }), this._renderControlSuccess);
            else
                this.doAjaxPost("POST", this.model.url + "/" + this.model.serviceMethodSettings.initialize, JSON.stringify({ "action": "initializeGauge" }), this._renderControlSuccess);
        },

        _setFirst: true,

        /**
        * To configure the properties at runtime using SetModel		
        * @private
        */
        _setModel: function (options) {
            for (var key in options) {
                switch (key) {
                    case "OlapReport": this.setOlapReport(options[key]); break;
                    case "JsonData": this.setOlapReport(options[key]); break;
                    case "RefreshOlapGauge": this.element.renderControlFromJSON(options[key]); break;
                    case "customObject": this.model.customObject = options[key]; break;
                    case "locale": this.locale(); break;
                    case "rowsCount": this.rowsCount(); break;
                    case "columnsCount": this.columnsCount(); break;
                    case "showHeaderLabel": this.showHeaderLabel(); break;
                    case "locale": this.locale(); break;
                    case "radius": this.radius(); break;
                    case "frameType": this.frameType(); break;
                    case "scales": break;
                }
            }
            if (this._gaugeObj["obj0"]) {
                this._renderControlSuccess({ "PivotRecords": JSON.stringify(this.getJSONRecords()), "OlapReport": this.getOlapReport() });
            }
        },

        /**
        * This function is invoked to wire the events for UI interaction.
        *
        * @private  
        */
        _wireEvents: function () {
            if (this.model.enableTooltip) {
                this.element.find("canvas").mouseenter(ej.proxy(function (evt) {
                    this._showTooltip(evt);
                }, this)).mouseout(ej.proxy(function () {
                    $(this._tooltip).fadeOut("slow");
                }, this)).mousemove(ej.proxy(function (evt) {
                    this._showTooltip(evt);
                    this._hideTooltip();
                }, this));
            }
        },

        /**
        * To get the localized labels.
        * @private
        */
        //get Localized text
        _getLocalizedLabels: function (property) {
            return ej.olap.OlapGauge.locale[this.locale()][property] === undefined ? ej.olap.OlapGauge.locale["en-US"][property] : ej.olap.OlapGauge.locale[this.locale()][property];
        },

        /**
        * This function is invoked to unwire the events registered for UI interaction.
        *
        * @private  
        */
        _unWireEvents: function () {

        },

        /**
        * This function is used to initialize and manage the tooltip in OLAP Gauges.
        *
        * @private  
        */
        _showTooltip: function (event) {
            $(this._tooltip).remove();
            var gaugeIndexValues = event.target.parentNode.id.split("_") == "" ? event.target.parentNode.parentNode.id.split("_") : event.target.parentNode.id.split("_");
            var gaugeIndex = gaugeIndexValues[gaugeIndexValues.length - 1];
            var goal = this._JSONRecords[gaugeIndex].GoalValue || 0;
            var value = this._JSONRecords[gaugeIndex].MeasureValue || this._JSONRecords[gaugeIndex].Value;
            var member = this._JSONRecords[gaugeIndex].MemberName || this._JSONRecords[gaugeIndex].Measure;
            var tooltipText = member + "<br/>" + this._getLocalizedLabels("RevenueGoal") + ": " + goal + "<br/>" + this._getLocalizedLabels("RevenueValue") + ": " + value;
            this._tooltip = ej.buildTag("div.e-olapgauge-tooltip", tooltipText);
            $(this._tooltip).appendTo('body');
            $(this._tooltip).addClass(".e-olapgauge-active");
            this._changeTooltipPos(event);
        },

        /**
        * This function is used to update the tooltip while mouse moves between OLAP Gauges.
        *
        * @private  
        */
        _changeTooltipPos: function (event) {
            var tooltipX = event.pageX - 8;
            var tooltipY = event.pageY + 8;
            $(this._tooltip).css({ top: tooltipY, left: tooltipX });
        },

        /**
        * 	This function is used to hide the tooltip while mouse leaves the OLAP Gauge.
        *
        * @private  
        */
        _hideTooltip: function () {
            window.setTimeout(ej.proxy(function () { $(this._tooltip).fadeOut(1500, "linear"); }, this), 3000);
        },

        /**
        * This function is used to create progress bar containing status text and percentage.
        *
        * @private  
        */
        _createProgressBar: function () {
            var progressContainer = ej.buildTag("div#" + this._id + "progressContainer.progressContainer", "", { top: ($("#" + this._id).position().top + ($("#" + this._id).height() / 2) - 30), left: ($("#" + this._id).position().left + ($("#" + this._id).width() / 3) - 50) })[0].outerHTML;
            var progressBar = ej.buildTag("div#" + this._id + "progressBar.progressBar", "")[0].outerHTML;
            var blockControl = ej.buildTag("div#" + this._id + "blockDiv.blockDiv", "", { width: $("#" + this._id).width(), display: "block", opacity: "0.3", backgroundColor: "#E9E9E9", height: $("#" + this._id).height(), position: 'fixed', top: ($("#" + this._id).position().top), left: ($("#" + this._id).position().left) })[0].outerHTML;
            $("#" + this._id).after(progressContainer);
            if (this.model.progressMode == "normal") {
                $("#" + this._id + "progressContainer").css({ width: 0, height: 0 });
                $("#" + this._id + "progressContainer").append(progressBar).append(blockControl);
            }
            else {
                var progressText = ej.buildTag("span#" + this._id + "progressText.progressText", "OlapGauge initial loading...")[0].outerHTML;
                $("#" + this._id + "progressContainer").append(progressBar).append(progressText).append(blockControl);
            }
            $("#" + this._id + "progressBar").ejProgressBar({ value: 0, text: 0 + " %", complete: "progressbarCompleted" });
            this._ogaugeProgressBar = $("#" + this._id + "progressBar").data("ejProgressBar");
            this._ogaugeProgressText = $("#" + this._id + "progressText");
        },

        /**
        * This function is used to update the progress status.
        *
        * @private  
        */
        _progressStatus: function (current_Status, completion_Status, progress_Speed) {
            if (jQuery.type(completion_Status) === "string") {
                var progressText = completion_Status.split(':');
                if (progressText != undefined) completion_Status = parseInt(progressText[0]);
            }
            this._ogaugeProgressBar.setModel({ value: current_Status, text: current_Status + " %" });
            $("#" + this._id + "progressContainer").show();
            $("#" + this._id + "blockDiv").position = $("#" + this._id).position();
            this._ogaugeTimer = window.setInterval(ej.proxy(function () {
                this._ogaugeProgressBar.setModel({ value: current_Status, text: current_Status + " %" });
                current_Status++;
                if (current_Status >= completion_Status) {
                    window.clearInterval(this._ogaugeTimer);
                    if (progressText != undefined)
                        this._ogaugeProgressBar.setModel({ value: 100, text: progressText[1] });

                };
            }, this), progress_Speed);
        },

        /**
        * This function receives the control's update from service-end which would be utilized for rendering the widget.
        *
        * @private  
        */
        _renderControlSuccess: function (msg) {
            if (this.model.progressMode != "infinite") {
                if (this.model.progressMode == "progress")
                    $("#" + this._id + "progressText")[0].innerHTML = this._getLocalizedLabels("MDXqueryExecutedSuccessfully") + "...";
                window.clearInterval(this._ogaugeTimer);
            }
            try {
                if (this.model.progressMode != "infinite") {
                    if (this.model.progressMode == "progress")
                        $("#" + this._id + "progressText")[0].innerHTML = this._getLocalizedLabels("RenderingStarted") + "...";
                    this._progressStatus(70, 100, 50);
                    window.setTimeout(ej.proxy(this.progressStatus(msg), this), 300);
                }
                else
                    window.setTimeout(ej.proxy(this.progressStatus(msg), this), 1);
            }
            catch (err) {
                if (this.model.progressMode != "infinite") {
                    if (this.model.progressMode == "progress")
                        $("#" + this._id + "progressText")[0].innerHTML = this._getLocalizedLabels("RenderingFailed") + "!!!";
                    window.clearInterval(this._ogaugeTimer);
                    this._ogaugeProgressBar.setModel({ value: 70, text: " Failed" });
                }
            }
        },

        /**
        * Perform an asynchronous HTTP (Ajax) request.
        * @return jQuery
        * @example 
        * &lt;div id="OlapGauge"&gt;&lt;/div&gt; <br> 
        * &lt;script&gt;
        * // Create OLAP Gauge
        * $('#OlapGauge').ejOlapGauge({
        *       url: "OlapGaugeService.svc",
        *		enableTooltip: true,
        *		scales: [{
        *		pointers: [{
        *                            showBackNeedle: true,
        *                            backNeedleLength: 20,
        *                            length: 120,
        *                            width: 7
        *                        },
        *                {
        *                    type: "marker",
        *                    markerType: "diamond",
        *                    distanceFromScale: 5,
        *                    placement: "center",
        *                    backgroundColor: "#29A4D9",
        *                    length: 25,
        *                    width: 15
        *                }]
        *				}]
        *   });
        * var gaugeObj = $("#OlapGauge").data("ejOlapGauge");
        * gaugeObj.doAjaxPost("POST", "/OlapGaugeService.svc/Initialize", {"key", "Hello World"}, "renderControlSuccess", null);
        * // initiate an Ajax request
        * &lt;/script&gt;
        * @memberof ejOlapGauge
        * @instance
        */
        doAjaxPost: function (type, url, data, onSuccess, onComplete) {
            $.ajax({
                type: type,
                url: url,
                contentType: 'application/json; charset=utf-8',
                dataType: 'json',
                data: data,
                success: $.proxy(onSuccess, this),
                complete: ej.proxy(function () {
                    var eventArgs = { element: this.element, customObject: this.model.customObject };
                    this._trigger("renderComplete", eventArgs);
                }, this),
                error: ej.proxy(function (msg, textStatus, errorThrown) {
                    if (this.model.progressMode != "infinite") {
                        window.clearInterval(this._ogaugeTimer);
                        if (this.model.progressMode == "progress")
                            $("#" + this._id + "progressText")[0].innerHTML = this._getLocalizedLabels("MDXqueryExecutionFailed") + "!!!";
                        $("#" + this._ogaugeProgressBar._id).show();
                        this._ogaugeProgressBar.setModel({ value: 0, text: errorThrown });
                    }
                    else
                        this._ogaugeWaitingPopup.hide();
                    var eventArgs = { element: this.element, customObject: this.model.customObject, Message: msg };
                    this._trigger("renderFailure", eventArgs);
                }, this)
            });
        },

        /**
        * This function receives JSON data and prepares for rendering the widget after service request.
        * @return jQuery
        * @example 
        * &lt;div id="OlapGauge"&gt;&lt;/div&gt; <br> 
        * &lt;script&gt;
        * // Create OLAP Gauge
        * $('#OlapGauge').ejOlapGauge({
        *       url: "OlapGaugeService.svc",
        *		enableTooltip: true,
        *		scales: [{
        *		pointers: [{
        *                            showBackNeedle: true,
        *                            backNeedleLength: 20,
        *                            length: 120,
        *                            width: 7
        *                        },
        *                {
        *                    type: "marker",
        *                    markerType: "diamond",
        *                    distanceFromScale: 5,
        *                    placement: "center",
        *                    backgroundColor: "#29A4D9",
        *                    length: 25,
        *                    width: 15
        *                }]
        *				}]
        *   });
        * var gaugeObj = $("#OlapGauge").data("ejOlapGauge");
        * gaugeObj.progressStatus({"OlapReport": this.getOlapReport(), "JsonRecords": this.getJSONRecords()});
        * // creating OLAP Gauge after Ajax request
        * &lt;/script&gt;
        * @memberof ejOlapGauge
        * @instance
        */
        progressStatus: function (msg) {
            var htmlTag = "", stausImage = "", trendImage = "", liTag = "", gaugeId = 0;
            if (msg[0] != undefined && msg.length > 0) {
                this.setJSONRecords(msg[0].Value);
                this.setOlapReport(msg[1].Value);
                if (msg[2] != null && msg[2] != undefined)
                    this.model.customObject = msg[2].Value;
            }
            else if (msg.d != undefined && msg.d.length > 0) {
                this.setJSONRecords(msg.d[0].Value);
                this.setOlapReport(msg.d[1].Value);
                if (msg.d[2] != null && msg.d[2] != undefined)
                    this.model.customObject = msg.d[2].Value;
            }
            else if (msg.PivotRecords && msg.OlapReport) {
                this.setJSONRecords(msg.PivotRecords);
                this.setOlapReport(msg.OlapReport);
                if (msg.customObject != null && msg.customObject != undefined)
                    this.model.customObject = msg.customObject;
            }

            if (this.model.afterServiceInvoke != null)
                this._trigger("afterServiceInvoke", { element: this.element, customObject: this.model.customObject });
            var ogaugeInfo = this.getJSONRecords();

            if (ogaugeInfo != null && ogaugeInfo.length > 0) {
                if (this.rowsCount() < 1 && this.columnsCount() < 1) {
                    this._currentLayout = "WrapPanel";
                    for (var k = 0; k < ogaugeInfo.length; k++) {
                        stausImage = ej.buildTag("img#stausIndicator_" + k + ".kpiiconvalue")[0].outerHTML;
                        trendImage = ej.buildTag("img#trendIndicator_" + k + ".kpiiconvalue")[0].outerHTML;
                        liTag += ej.buildTag("li", ej.buildTag("div#" + this._id + "_" + k, stausImage + trendImage)[0].outerHTML)[0].outerHTML;
                    }
                    htmlTag = ej.buildTag("ul.wrapLayout", liTag)[0].outerHTML;
                }
                else {
                    this._currentLayout = "Table";
                    if (this.columnsCount() == undefined || this.columnsCount() < 1)
                        this.columnsCount((ogaugeInfo.length / this.rowsCount()).toFixed());
                    if (this.rowsCount() == undefined || this.rowsCount() < 1)
                        this.rowsCount((ogaugeInfo.length / this.columnsCount()).toFixed());
                    htmlTag = "<table><tbody>";
                    for (var i = 0; i < this.rowsCount(); i++) {
                        htmlTag += "<tr>";
                        for (var j = 0; j < this.columnsCount(); j++) {
                            stausImage = ej.buildTag("img#stausIndicator_" + gaugeId + ".kpiiconvalue")[0].outerHTML;
                            trendImage = ej.buildTag("img#trendIndicator_" + gaugeId + ".kpiiconvalue")[0].outerHTML;
                            htmlTag += "<td>" + ej.buildTag("div#" + this._id + "_" + gaugeId, stausImage + trendImage)[0].outerHTML + "</td>";
                            gaugeId++;
                        }
                        htmlTag += "</tr>";
                    }
                    htmlTag += "</tbody></table>";
                }
                this.element.html(htmlTag);
                this.renderControlFromJSON(ogaugeInfo);
                this._defaultWidth = this.element.find("canvas").width();
                this._defaultArea = this.element.find("table").height() * this.element.find("table").width();
            }
            if (this.model.progressMode != "infinite") {
                if (this.model.progressMode == "normal")
                    $("#" + this._id + "progressContainer").delay(1500).fadeOut("slow");
                else if (this.model.progressMode == "progress") {
                    $("#" + this._id + "progressText")[0].innerHTML = this._getLocalizedLabels("RenderingSucceeded") + "...";
                    $("#" + this._id + "progressContainer").delay(2000).fadeOut("slow");
                }
            }
            else
                this._ogaugeWaitingPopup.hide();
            var eventArgs = { element: this.element, customObject: this.model.customObject };
            this._trigger("renderSuccess", eventArgs);
        },

        /**
        * This function receives the processed KPI value and factor to calculate the upper limit.
        *
        * @private  
        */
        _getCalculatedValue: function (value) {
            var stringValue = value.toString();
            var factor = 1; var len; var dividend;
            if (stringValue.indexOf('.') > -1)
                len = stringValue.split('.')[0].length;
            else
                len = stringValue.length;
            dividend = parseInt(stringValue);
            for (var i = 0; i < len; i++) {
                if (dividend > this._maxValue) {
                    dividend = dividend / 10;
                    factor = factor * 10;
                }
            }
            return factor;
        },

        /**
        * This function receives the processed KPI value and factor to calculate the upper limit.
        *
        * @private  
        */
        _findNearestUpperLimit: function (number, factor) {
            var convertedNumber = number.toString();
            var j = parseInt(convertedNumber);
            var k = j % factor;
            return j + (factor - k);
        },

        /**
        * This function receives the KPI goal and KPI values to calculate the factor.
        *
        * @private  
        */
        _setMaxScaleValue: function (goal, value, i) {
            var factor;
            if (value > goal) {
                factor = this._getCalculatedValue(value);
                this._maximum[i] = this._findNearestUpperLimit((value / factor), 10);
            }
            else {
                factor = this._getCalculatedValue(goal);
                this._maximum[i] = this._findNearestUpperLimit((((goal / factor) * 4) / 3), 10);
            }
            return factor;
        },

        /**
        * This function removes the KPI image tags in OLAP Gauge control.
        * @return jQuery
        * @example 
        * &lt;div id="OlapGauge"&gt;&lt;/div&gt; <br> 
        * &lt;script&gt;
        * // Create OLAP Gauge
        * $('#OlapGauge').ejOlapGauge({
        *       url: "OlapGaugeService.svc",
        *		enableTooltip: true,
        *		scales: [{
        *		pointers: [{
        *                            showBackNeedle: true,
        *                            backNeedleLength: 20,
        *                            length: 120,
        *                            width: 7
        *                        },
        *                {
        *                    type: "marker",
        *                    markerType: "diamond",
        *                    distanceFromScale: 5,
        *                    placement: "center",
        *                    backgroundColor: "#29A4D9",
        *                    length: 25,
        *                    width: 15
        *                }]
        *				}]
        *   });
        * var gaugeObj = $("#OlapGauge").data("ejOlapGauge");
        * gaugeObj.removeImg();
        * // removes HTML image tags inside of OLAP Gauge.
        * &lt;/script&gt;
        * @memberof ejOlapGauge
        * @instance
        */
        removeImg: function () {
            this.element.find("img").remove();
        },

        /**
        * This function receives the JSON formatted datasource to render the OLAP Gauge control.
        * @return jQuery
        * @example 
        * &lt;div id="OlapGauge"&gt;&lt;/div&gt; <br> 
        * &lt;script&gt;
        * // Create OLAP Gauge
        * $('#OlapGauge').ejOlapGauge({
        *       url: "OlapGaugeService.svc",
        *		enableTooltip: true,
        *		scales: [{
        *		pointers: [{
        *                            showBackNeedle: true,
        *                            backNeedleLength: 20,
        *                            length: 120,
        *                            width: 7
        *                        },
        *                {
        *                    type: "marker",
        *                    markerType: "diamond",
        *                    distanceFromScale: 5,
        *                    placement: "center",
        *                    backgroundColor: "#29A4D9",
        *                    length: 25,
        *                    width: 15
        *                }]
        *				}]
        *   });
        * var gaugeObj = $("#OlapGauge").data("ejOlapGauge");
        * gaugeObj.renderControlFromJSON(this.getJSONRecords());
        * // render the OLAP Gauge from JSON formatted data.
        * &lt;/script&gt;
        * @memberof ejOlapGauge
        * @instance
        */
        renderControlFromJSON: function (jsonObj) {
            //Calculating pointer value, danger and safer regions
            this._maxValue = 99, maxGaugeCount = 0;
            var pointerCapRadius = (this.radius() / 22);
            if (this._currentLayout == "Table" && (this.rowsCount() * this.columnsCount()) <= jsonObj.length)
                maxGaugeCount = this.rowsCount() * this.columnsCount();
            else if (this._currentLayout == "WrapPanel" || (this.rowsCount() * this.columnsCount()) > jsonObj.length)
                maxGaugeCount = jsonObj.length;
            for (var i = 0; i < maxGaugeCount; i++) {
                var goal = 0.0, value = 0.0, startValue = 0.0, endValue = 0.0, dValue = 0.0, factor = 0, mark = 0.0,
                percentageValue = 0.0, customLabel = "", imgDiv, extendRanges = new Array(), extendPointers = new Array(),
                extendLabels = new Array(), extendCustomLabels = new Array(), extendTicks = new Array(), extendIndicators = new Array(),
                extendScales = new Array(), statusClass = "", trendClass = "";

                if (jsonObj[i].MeasureValue != null && jsonObj[i].MeasureValue != undefined) {
                    value = parseFloat(jsonObj[i].MeasureValue) || 0;
                    goal = parseFloat(jsonObj[i].GoalValue) || 0;
                    customLabel =jsonObj[i].MeasureCaption+" - " + jsonObj[i].MemberName;
                }
                else {
                    value = parseFloat(jsonObj[i].Value);
                    goal = 0.0;
                    customLabel = jsonObj[i].Measure;
                    this.model.scales[0].showIndicators = false;
                }
                factor = this._setMaxScaleValue(goal, value, i);
                dValue = value;
                mark = goal / factor;
                percentageValue = (dValue / factor);
                this._pointerValue_0 = endValue + percentageValue;
                this._pointerValue_1 = mark + endValue;
                startValue = endValue;
                endValue = endValue + percentageValue;

                //Edting Model values
                if (jsonObj[i].StatusGraphic == "Traffic Signals") {
                    if (jsonObj[i].StatusValue == -1)
                        statusClass = "kpiredroad";
                    else if (jsonObj[i].StatusValue == 0)
                        statusClass = "kpiallcolor";
                    else if (jsonObj[i].StatusValue == 1)
                        statusClass = "kpigreenroad";
                }
                else {
                    if (jsonObj[i].StatusValue == -1)
                        statusClass = "kpidiamond";
                    else if (jsonObj[i].StatusValue == 0)
                        statusClass = "kpitriangle";
                    else if (jsonObj[i].StatusValue == 1)
                        statusClass = "kpicircle";
                }
                imgDiv = this.element.find("#statusIndicator_" + [i]).addClass(statusClass);
                if (imgDiv != null && imgDiv != undefined) {
                    var statusImage = $(imgDiv).css('background-image');
                    var statusIndicator = {
                        height: 15,
                        width: 15,
                        type: "image",
                        imageUrl: (!ej.isNullOrUndefined(statusImage) && statusImage != "none") ? statusImage.split("(")[1].split(")")[0].replace('"', "").replace('"', "") : "",
                        position: { x: this.radius() - 20, y: 260 }
                    };
                }

                if (jsonObj[i].TrendGraphic != null && jsonObj[i].TrendGraphic != undefined && jsonObj[i].TrendGraphic != "" && jsonObj[i].TrendGraphic.toLowerCase() == "standard arrow") {
                    if (jsonObj[i].TrendValue == -1)
                        trendClass = "kpidownarrow";
                    else if (jsonObj[i].TrendValue == 0)
                        trendClass = "kpirightarrow";
                    else if (jsonObj[i].TrendValue == 1)
                        trendClass = "kpiuparrow";
                    imgDiv = this.element.find("#trendIndicator_" + [i]).toggleClass(trendClass);
                }
                if (!ej.isNullOrUndefined(imgDiv)) {
                    var trendImage = $(imgDiv).css('background-image');
                    var trendIndicator = {
                        height: 15,
                        width: 15,
                        type: "image",
                        imageUrl: (!ej.isNullOrUndefined(trendImage) && trendImage != "none") ? trendImage.split("(")[1].split(")")[0].replace('"', "").replace('"', "") : "",
                        position: { x: this.radius(), y: 260 }
                    };
                }
                if (!ej.isNullOrUndefined(this.model.scales[0])) {
                    if (this.frameType() == "fullcircle") {
                        this.model.scales[0].startAngle = 122;
                        this.model.scales[0].sweepAngle = 296;
                    }
                    else if (this.frameType() == "halfcircle") {
                        this.model.scales[0].startAngle = 180;
                        this.model.scales[0].sweepAngle = 180;
                    }
                    this.model.scales[0].indicators = this.model.scales[0].showIndicators ? [statusIndicator, trendIndicator] : this.model.scales[0].indicators;
                    if (!ej.isNullOrUndefined(this.model.scales[0].pointers[0])) {
                        var newPointerCap = ej.datavisualization.CircularGauge.prototype._defaultScaleValues().pointerCap;
                        //this.model.scales[0].pointers[0] = $.extend({}, newPointer, this.model.scales[0].pointers[0]);
                        this.model.scales[0].pointers[0].value = this._pointerValue_0;
                        this.model.scales[0].pointers[0].pointerCap = newPointerCap;
                        this.model.scales[0].pointers[0].pointerCap.radius = pointerCapRadius;
                    }
                    if (!ej.isNullOrUndefined(this.model.scales[0].pointers[1])) {
                        this.model.scales[0].pointers[1].value = this._pointerValue_1;
                        this.model.scales[0].pointers[1].type = this.model.scales[0].pointers[1].type || "marker";
                        this.model.scales[0].pointers[1].markerType = this.model.scales[0].pointers[1].markerType || "diamond";
                        this.model.scales[0].pointers[1].placement = this.model.scales[0].pointers[1].placement || "center";
                    }
                    if (!ej.isNullOrUndefined(this.model.scales[0].ranges[0])) {
                        this.model.scales[0].ranges[0].startValue = startValue;
                        this.model.scales[0].ranges[0].endValue = mark;
                    }
                    if (!ej.isNullOrUndefined(this.model.scales[0].ranges[1])) {
                        this.model.scales[0].ranges[1].startValue = mark;
                        this.model.scales[0].ranges[1].endValue = this._maximum[i];
                    }
                    if (!ej.isNullOrUndefined(this.model.scales[0].customLabels[0]))
                        this.model.scales[0].customLabels[0].value = this.showHeaderLabel() ? "X " + factor : "";
                    if (!ej.isNullOrUndefined(this.model.scales[0].customLabels[1]))
                        this.model.scales[0].customLabels[1].value = this.showHeaderLabel() ? customLabel : "";
                    if (!ej.isNullOrUndefined(this.model.scales[0].customLabels[2]))
                        this.model.scales[0].customLabels[2].value = this.showHeaderLabel() ? customLabel : "";

                    for (var j = 0; j < this.model.scales[0].ranges.length; j++)
                        extendRanges.push($.extend({}, ej.datavisualization.CircularGauge.prototype._defaultScaleValues().ranges[0], this.model.scales[0].ranges[j]));
                    for (var j = 0; j < this.model.scales[0].pointers.length; j++)
                        extendPointers.push($.extend({}, ej.datavisualization.CircularGauge.prototype._defaultScaleValues().pointers[0], this.model.scales[0].pointers[j]));
                    for (var j = 0; j < this.model.scales[0].labels.length; j++)
                        extendLabels.push($.extend({}, ej.datavisualization.CircularGauge.prototype._defaultScaleValues().labels[0], this.model.scales[0].labels[j]));
                    for (var j = 0; j < this.model.scales[0].labels.length; j++)
                        extendLabels.push($.extend({}, ej.datavisualization.CircularGauge.prototype._defaultScaleValues().labels[0], this.model.scales[0].labels[j]));
                    for (var j = 0; j < this.model.scales[0].customLabels.length; j++)
                        extendCustomLabels.push($.extend({}, ej.datavisualization.CircularGauge.prototype._defaultScaleValues().customLabels[0], this.model.scales[0].customLabels[j]));
                    for (var j = 0; j < this.model.scales[0].ticks.length; j++)
                        extendTicks.push($.extend({}, ej.datavisualization.CircularGauge.prototype._defaultScaleValues().ticks[0], this.model.scales[0].ticks[j]));
                    if (this.model.scales[0].indicators) {
                        for (var j = 0; j < this.model.scales[0].indicators.length; j++)
                            extendIndicators.push($.extend({}, ej.datavisualization.CircularGauge.prototype._defaultScaleValues().indicators[0], this.model.scales[0].indicators[j]));
                    }
                    newScale = {
                        pointerCap: {
                            radius: pointerCapRadius || this.model.scales[0].pointerCap.radius,
                            borderWidth: this.model.scales[0].pointerCap.borderWidth,
                            interiorGradient: this.model.scales[0].pointerCap.interiorGradient,
                            borderColor: this.model.scales[0].pointerCap.borderColor,
                            backgroundColor: this.model.scales[0].pointerCap.backgroundColor
                        },
                        border: {
                            color: this.model.scales[0].border.color,
                            width: this.model.scales[0].border.width
                        },
                        majorIntervalValue: this.model.scales[0].majorIntervalValue,
                        minorIntervalValue: this.model.scales[0].minorIntervalValue,
                        minimum: this.model.scales[0].minimum,
                        backgroundColor: this.model.scales[0].backgroundColor,
                        direction: this.model.scales[0].direction,
                        showPointers: this.model.scales[0].showPointers,
                        showTicks: this.model.scales[0].showTicks,
                        showLabels: this.model.scales[0].showLabels,
                        showIndicators: this.model.scales[0].showIndicators,
                        showRanges: this.model.scales[0].showRanges,
                        showScaleBar: this.model.scales[0].showScaleBar,
                        startAngle: this.model.scales[0].startAngle,
                        sweepAngle: this.model.scales[0].sweepAngle,
                        radius: this.model.scales[0].radius,
                        size: this.model.scales[0].size,
                        maximum: this.model.scales[0].maximum || this._maximum[i],
                        pointers: extendPointers,
                        ticks: extendTicks,
                        labels: extendLabels,
                        ranges: extendRanges,
                        customLabels: extendCustomLabels,
                        indicators: extendIndicators
                    };
                    for (var j = 0; j < this.model.scales.length; j++) {
                        if (j == 0)
                            extendScales.push($.extend({}, ej.datavisualization.CircularGauge.prototype._defaultScaleValues(), newScale));
                        else
                            extendScales.push($.extend({}, ej.datavisualization.CircularGauge.prototype._defaultScaleValues(), this.model.scales[j]));
                    }
                }
                //Creating Gauges
                $("#" + this._id + "_" + i).ejCircularGauge({
                    backgroundColor: this.model.backgroundColor,
                    frame: {
                        frameType: this.model.frame.frameType,
                        halfCircleFrameStartAngle: this.model.halfCircleFrameStartAngle,
                        halfCircleFrameEndAngle: this.model.halfCircleFrameEndAngle
                    },
                    
                    radius: this.radius(),
                    width: this.model.width, // Width Should be 2*radious 
                    height: this.model.height,
                    interiorGradient: this.model.interiorGradient,
                    readOnly: this.model.readOnly,
                    enableResize: this.model.enableResize,
                   
                    enableAnimation: true,
                    animationSpeed: this.model.animationSpeed,
                    theme: this.model.theme,
                    isRadialGradient: this.model.isRadialGradient,
                    //Events
                    load: this.model.load,
                    drawTicks: this.model.drawTicks,
                    drawLabels: this.model.drawLabels,
                    drawPointers: this.model.drawPointers,
                    drawRange: this.model.drawRange,
                    drawCustomLabel: this.model.drawCustomLabel,
                    drawIndicators: this.model.drawIndicators,
                    drawPointerCap: this.model.drawPointerCap,
                    renderComplete: this.model.renderComplete,
                    mouseClick: this.model.mouseClick,
                    mouseClickMove: this.model.mouseClickMove,
                    mouseClickUp: this.model.mouseClickUp,
                    scales: extendScales
                });
                this._gaugeObj["obj" + i] = $("#" + this._id + "_" + i).data("ejCircularGauge");
            }
            this.removeImg();
            this._wireEvents();
        },

        /**
        * This function is used to refresh the OLAP Gauge at client side.
        * @return jQuery
        * @example 
        * &lt;div id="OlapGauge"&gt;&lt;/div&gt; <br> 
        * &lt;script&gt;
        * // Create OLAP Gauge
        * $('#OlapGauge').ejOlapGauge({
        *       url: "OlapGaugeService.svc",
        *		enableTooltip: true,
        *		scales: [{
        *		pointers: [{
        *                            showBackNeedle: true,
        *                            backNeedleLength: 20,
        *                            length: 120,
        *                            width: 7
        *                        },
        *                {
        *                    pointerType: "marker",
        *                    markerType: "diamond",
        *                    distanceFromScale: 5,
        *                    placement: "center",
        *                    backgroundColor: "#29A4D9",
        *                    length: 25,
        *                    width: 15
        *                }]
        *				}]
        *   });
        * var gaugeObj = $("#OlapGauge").data("ejOlapGauge");
        * gaugeObj.refresh();
        * // refresh the OLAP Gauge in client-side.
        * &lt;/script&gt;
        * @memberof ejOlapGauge
        * @instance
        */
        refresh: function () {
            if (this._gaugeObj == null || this._gaugeObj == undefined) {
                for (var i = 0; i < this._JSONRecords.length; i++) {
                    this._gaugeObj["obj" + i] = $("#" + this._id + "_" + i).data("ejCircularGauge");
                    this._gaugeObj["obj" + i].refresh();
                }
            }
            else {
                for (key in this._gaugeObj) {
                    this._gaugeObj[key].refresh();
                }
            }
        }
    });

    //OLAP Gauge Localization
    ej.olap.OlapGauge.locale = {};
    //Default english localized values
    ej.olap.OlapGauge.locale["en-US"] = {
        RevenueGoal: "Revenue Goals",
        RevenueValue: "Revenue Values",
        MDXqueryExecutionFailed: "MDX query execution failed",
        PreparingAndExecutingMDXquery: "Preparing and executing MDX query",
        MDXqueryExecutedSuccessfully: "MDX query executed successfully",
        RenderingStarted: "Rendering Started",
        RenderingSucceeded: "Rendering Succeeded",
        RenderingFailed: "Rendering failed"
    };

    /**
    * Enum for Progres Mode. Allows the user to select a appropriate progress indicator. 
    * @enum {String}
    * @global
    */
    ej.olap.OlapGauge.ProgressMode = {
        Infinite: "infinite",
        Normal: "normal",
        Progress: "progress"
    };

})(jQuery, Syncfusion); ;
/**
* @fileOverview Plugin to style the Html OLAP Pager elements
* @copyright Copyright Syncfusion Inc. 2001 - 2013. All rights reserved.
*  Use of this code is subject to the terms of our license.
*  A copy of the current license can be obtained at any time by e-mailing
*  licensing@syncfusion.com. Any infringement will be prosecuted under
*  applicable laws. 
* @version 12.1 
* @author <a href="mailto:licensing@syncfusion.com">Syncfusion Inc</a>
*/

(function ($, ej, undefined) {
    /**
    * @namespace ej
    * @class ejOlapPager
    * @requires jquery-1.10.2.min.js
    * @requires jquery.easing.1.3.min.js
    * @requires ej.core.js
    * @requires ej.data.js
    * @requires ej.touch.js
    * @requires ej.waitingpopup.js
    * @requires ej.olappager.js
    * @requires ej.pivotgrid
    * @classdesc Custom Design for Html OLAP Pager control.
    * @example 
    * &lt;div id="OlapPager"&gt;&lt;/div&gt; <br> 
    * &lt;script&gt;
    * // Create OLAP Pager
    * $("#OlapPager").ejOlapPager(...); 	
    * &lt;/script&gt;
    */
    ej.widget("ejOlapPager", "ej.olap.OlapPager", {

        //Root Css Class
        _rootCSS: "e-olappager",

        // Widget element will be automatically set in this
        element: null,

        // User-defined model will be automatically set in this
        model: null,

        /**
        * Returns the model of OLAP Pager widget.
        *
        * @private
        */
        _getModel: function () {
            return this.model;
        },

        /**
        * Sets the model of OLAP Pager widget.
        *
        * @private
        */
        _setModel: function (value) {
            this.model = value;
        },

        defaults: {
            /**		
            * Contains the id of the target element for which paging is enabled.
            * @default ""
            * @type {string}
            * @example 
            * //To set targetControlID API value during initialization  
            * 	$("#OlapPager").ejOlapPager({  targetControlID: "PivotGrid"});					* 
            * @example 
            * //Get or set the targetControlID API, after initialization:<br>
            *	//Gets the targetControlID value  
            *	$("#OlapPager").ejOlapPager("option", "targetControlID");<br>			
            *	//Sets the targetControlID value 
            *	$("#OlapPager").ejOlapPager("option", "targetControlID",  "PivotGrid1" ); 
            * @memberof ejOlapPager
            * @instance
            */
            targetControlID: "",
            /**		
            * Contains the current page number in categorical axis.
            * @default 1
            * @type {number}
            * @example 
            * //To set categoricalCurrentPage API value during initialization  
            * 	$("#OlapPager").ejOlapPager({  categoricalCurrentPage: 1});					* 
            * @example 
            * //Get or set the categoricalCurrentPage API, after initialization:<br>
            *	//Gets the categoricalCurrentPage value  
            *	$("#OlapPager").ejOlapPager("option", "categoricalCurrentPage");<br>			
            *	//Sets the categoricalCurrentPage value 
            *	$("#OlapPager").ejOlapPager("option", "categoricalCurrentPage", 2 ); 
            * @memberof ejOlapPager
            * @instance
            */
            categoricalCurrentPage: 1,
            /**		
            * Contains the current page number in series axis.
            * @default 1
            * @type {number}
            * @example 
            * //To set seriesCurrentPage API value during initialization  
            * 	$("#OlapPager").ejOlapPager({  seriesCurrentPage: 1});					* 
            * @example 
            * //Get or set the seriesCurrentPage API, after initialization:<br>
            *	//Gets the seriesCurrentPage value  
            *	$("#OlapPager").ejOlapPager("option", "seriesCurrentPage");<br>			
            *	//Sets the seriesCurrentPage value 
            *	$("#OlapPager").ejOlapPager("option", "seriesCurrentPage", 2 ); 
            * @memberof ejOlapPager
            * @instance
            */
            seriesCurrentPage: 1,
            /**		
            * Contains the total page count in series axis.
            * @default 1
            * @type {number}
            * @example 
            * //To set seriesPageCount API value during initialization  
            * 	$("#OlapPager").ejOlapPager({  seriesPageCount: 1});					* 
            * @example 
            * //Get or set the seriesPageCount API, after initialization:<br>
            *	//Gets the seriesPageCount value  
            *	$("#OlapPager").ejOlapPager("option", "seriesPageCount");<br>			
            *	//Sets the seriesPageCount value 
            *	$("#OlapPager").ejOlapPager("option", "seriesPageCount", 2 ); 
            * @memberof ejOlapPager
            * @instance
            */
            seriesPageCount: 0,
            /**		
            * Contains the total page count in categorical axis.
            * @default 1
            * @type {number}
            * @example 
            * //To set categPageCount API value during initialization  
            * 	$("#OlapPager").ejOlapPager({  categPageCount: 1});					* 
            * @example 
            * //Get or set the categPageCount API, after initialization:<br>
            *	//Gets the categPageCount value  
            *	$("#OlapPager").ejOlapPager("option", "categPageCount");<br>			
            *	//Sets the categPageCount value 
            *	$("#OlapPager").ejOlapPager("option", "categPageCount", 2 ); 
            * @memberof ejOlapPager
            * @instance
            */
            categoricalPageCount: 0,
            /**		
            * Sets the localized language for the control.
            * @default 1
            * @type {string}
            * @example 
            * //To set localization API value during initialization  
            * 	$("#OlapPager").ejOlapPager({  locale: "en-US"});					* 
            * @example 
            * //Get or set the localization API, after initialization:<br>
            *	//Gets the localization value  
            *	$("#OlapPager").ejOlapPager("option", "locale");<br>			
            *	//Sets the localization value 
            *	$("#OlapPager").ejOlapPager("option", "locale", "fr-FR" ); 
            * @memberof ejOlapPager
            * @instance
            */
            locale: "en-US",
            /**		
            * Sets the pager mode for the Olap pager.
            * @default ej.olap.OlapPager.Mode.Both
            * @type {enum}
            * @example 
            * //To set pagerMode API value during initialization  
            * 	$("#OlapPager").ejOlapPager({  mode: ej.olap.OlapPager.Mode.Series});					* 
            * @example 
            * //Get or set the pagerMode API, after initialization:<br>
            *	//Gets the pagerMode value  
            *	$("#OlapPager").ejOlapPager("option", "mode");<br>			
            *	//Sets the pagerMode value 
            *	$("#OlapPager").ejOlapPager("option", "mode", ej.olap.OlapPager.Mode.Categorical ); 
            * @memberof ejOlapPager
            * @instance
            */
            mode: "both"
        },

        /**
        * Initializes the variables and loads the widget. 
        *
        * @private
        */
        _init: function () {
            if ($("#" + this.model.targetControlID).hasClass('e-pivotgrid')) {
                this.targetControlName = "PivotGrid";
                this.targetControl = $("#" + this.model.targetControlID).data("ejPivotGrid");
                this.targetControl._pagerObj = this;
            }
			else if ($("#" + this.model.targetControlID).hasClass('e-olapgrid')) {
                this.targetControlName = "OlapGrid";
                this.targetControl = $("#" + this.model.targetControlID).data("ejOlapGrid");
                this.targetControl._pagerObj = this;
            }
            this._createPager();
            this.unwireEvents();
            this.wireEvents();
        },

        /**
        * Destroys the widget while it's no longer required. 
        *
        * @private
        */
        _destroy: function () {
            this.element.empty().removeClass("e-olappager");
        },

        /**
        * 	This function is invoked to wire the events for UI interaction.
        * @return jQuery
        * @example 
        * &lt;div id="OlapPager"&gt;&lt;/div&gt; <br> 
        * &lt;script&gt;
        * // Wiring events to OLAP Pager control.
        * $('#OlapPager').ejOlapPager({
        *       url: "Pager.svc",
        *   });
        * var pagerObj = $("#OlapPager").data("ejOlapPager");
        * pagerObj.wireEvents();
        * // Wiring events with OLAP Pager.
        * &lt;/script&gt;
        * @memberof ejOlapPager
        * @instance
        */
        wireEvents: function () {
            this._on(this.element, "keydown", ".pagerTextBox", this._pagerTextBoxClick);
            this._on(this.element, "click", ".moveNext", this._moveNextPage);
            this._on(this.element, "click", ".moveLast", this._moveLastPage);
            this._on(this.element, "click", ".moveFirst", this._moveFirstPage);
            this._on(this.element, "blur", ".pagerTextBox", this._restorePageNo);
            this._on(this.element, "click", ".movePrevious", this._movePreviousPage);
        },

        /**
        * 	This function is invoked to unwire the events registered for UI interaction.
        * @return jQuery
        * @example 
        * &lt;div id="OlapPager"&gt;&lt;/div&gt; <br> 
        * &lt;script&gt;
        * // Unwiring events from OLAP Pager control.
        * $('#OlapPager').ejOlapPager({
        *       url: "Pager.svc",
        *   });
        * var pagerObj = $("#OlapPager").data("ejOlapPager");
        * pagerObj.unwireEvents();
        * // Unwiring events from OLAP Pager.
        * &lt;/script&gt;
        * @memberof ejOlapPager
        * @instance
        */
        unwireEvents: function () {
            this._off(this.element, "click", ".moveNext", this._moveNextPage);
            this._off(this.element, "blur", ".pagerTextBox", this._restorePageNo);
            this._off(this.element, "click", ".moveLast", this._moveLastPage);
            this._off(this.element, "click", ".moveFirst", this._moveFirstPage);
            this._off(this.element, "click", ".movePrevious", this._movePreviousPage);
            this._off(this.element, "keydown", ".pagerTextBox", this._pagerTextBoxClick);
        },

        /**
        * This function is used to extract the appropriate localized text.
        *
        * @private  
        */
        _getLocalizedLabels: function (property) {
            return ej.olap.OlapPager.locale[this.model.locale][property] === undefined ? ej.olap.OlapPager.locale["en-US"][property] : ej.olap.OlapPager.locale[this.model.locale][property];
        },

        /**
        * This function creates the OlapPager element.
        *
        * @private
        */
        _createPager: function () {
            $(this.element).addClass("olapPager").attr("targetControlID", this.model.targetControlID);
            var pagerContent = "";
            if (this.model.mode != "series") {
                var categoricalPager = ej.buildTag("td.categPagerTd", "");
                if (this.model.mode == "both") $(categoricalPager[0]).css("border-right", "1px solid #808080");
                $(categoricalPager).html(ej.buildTag("div.pagerDiv", ej.buildTag("span.moveFirst e-icon")[0].outerHTML + ej.buildTag("span.movePrevious e-icon")[0].outerHTML + ej.buildTag("span.pagerLabel", this._getLocalizedLabels("CategoricalPage"))[0].outerHTML + ej.buildTag("input#" + this._id + "_CategCurrentPage.pagerTextBox")[0].outerHTML + ej.buildTag("span.categPageCount")[0].outerHTML + ej.buildTag("span.moveNext e-icon")[0].outerHTML + ej.buildTag("span.moveLast e-icon")[0].outerHTML));
                pagerContent = categoricalPager[0].outerHTML;
            }
            if (this.model.mode != "categorical") {
                var seriesPager = ej.buildTag("td.seriesPagerTd", "");
                $(seriesPager).html(ej.buildTag("div.pagerDiv", ej.buildTag("span.moveFirst e-icon")[0].outerHTML + ej.buildTag("span.movePrevious e-icon")[0].outerHTML + ej.buildTag("span.pagerLabel", this._getLocalizedLabels("SeriesPage"))[0].outerHTML + ej.buildTag("input#" + this._id + "_SeriesCurrentPage.pagerTextBox")[0].outerHTML + ej.buildTag("span.seriesPageCount")[0].outerHTML + ej.buildTag("span.moveNext e-icon")[0].outerHTML + ej.buildTag("span.moveLast e-icon")[0].outerHTML));
                pagerContent += seriesPager[0].outerHTML;
            }
            var pagerTable = ej.buildTag("table", ej.buildTag("tbody", ej.buildTag("tr", pagerContent)))[0];
            $(this.element).html(pagerTable);
        },

        /**
        * 	This function initializes the page counts and page numbers of the OlapPager.
        * @return jQuery
        * @example 
        * &lt;div id="OlapPager"&gt;&lt;/div&gt; <br> 
        * &lt;script&gt;
        * // Initializing page count and numbers.
        * $('#OlapPager').ejOlapPager({
        *       url: "Pager.svc",
        *   });
        * var pagerObj = $("#OlapPager").data("ejOlapPager");
        * pagerObj.initPagerProperties(10, {CategorialPageSize: 1, SeriesPageSize: 1, CategorialCurrentPage: 3, SeriesCurrentPage: 4});
        * // Initializing page count and numbers.
        * &lt;/script&gt;
        * @memberof ejOlapPager
        * @instance
        */
        initPagerProperties: function (headerCounts, pageSettings) {
            this.model.categoricalPageCount = Math.ceil(headerCounts.Column / pageSettings.CategorialPageSize);
            this.model.seriesPageCount = Math.ceil(headerCounts.Row / pageSettings.SeriesPageSize);
            this.model.categoricalCurrentPage = pageSettings.CategorialCurrentPage;
            this.model.seriesCurrentPage = pageSettings.SeriesCurrentPage;
            if (this.model.mode != "series") {
                $(this.element.find(".categPageCount")[0]).html("/ " + this.model.categoricalPageCount);
                $("#" + this._id + "_CategCurrentPage")[0].value = pageSettings.CategorialCurrentPage;
                this._setNavigators(this.model.categoricalCurrentPage, this.model.categoricalPageCount, this.element.find(".categPagerTd")[0]);
            }
            if (this.model.mode != "categorical") {
                $(this.element.find(".seriesPageCount")[0]).html("/ " + this.model.seriesPageCount);
                $("#" + this._id + "_SeriesCurrentPage")[0].value = pageSettings.SeriesCurrentPage;
                this._setNavigators(this.model.seriesCurrentPage, this.model.seriesPageCount, this.element.find(".seriesPagerTd")[0]);
            }
        },

        /**
        * This function is used to move the target control to next page.
        *
        * @private  
        */
        _moveNextPage: function (e) {
            if (!$(e.target).hasClass("disabled")) {
                var pagerInput = $($(e.target).parents('td')[0]).find(".pagerTextBox")[0];
                $($($(e.target).parents('td')[0]).find(".movePrevious")[0]).removeClass("disabled");
                $($($(e.target).parents('td')[0]).find(".moveFirst")[0]).removeClass("disabled");
                pagerInput.value = parseInt(pagerInput.value) + 1;
                if (pagerInput.id.indexOf("Categ") != -1) {
                    this.model.categoricalCurrentPage = pagerInput.value;
                    if (this.targetControlName == "PivotGrid" || this.targetControlName == "OlapGrid") this.targetControl.refreshPagedOlapGrid("categorical", pagerInput.value);
                    if (pagerInput.value == this.model.categoricalPageCount) {
                        $(e.target).addClass("disabled");
                        $($($(e.target).parents('td')[0]).find(".moveLast")[0]).addClass("disabled");
                    }
                }
                else if (pagerInput.id.indexOf("Series") != -1) {
                    this.model.seriesCurrentPage = pagerInput.value;
                    if (this.targetControlName == "PivotGrid" || this.targetControlName == "OlapGrid") this.targetControl.refreshPagedOlapGrid("series", pagerInput.value);
                    if (pagerInput.value == this.model.seriesPageCount) {
                        $(e.target).addClass("disabled");
                        $($($(e.target).parents('td')[0]).find(".moveLast")[0]).addClass("disabled");
                    }
                }
            }
        },

        /**
        * This function is used to move the target control to previous page.
        *
        * @private  
        */
        _movePreviousPage: function (e) {
            if (!$(e.target).hasClass("disabled")) {
                var pagerInput = $($(e.target).parents('td')[0]).find(".pagerTextBox")[0];
                $($($(e.target).parents('td')[0]).find(".moveNext")[0]).removeClass("disabled");
                $($($(e.target).parents('td')[0]).find(".moveLast")[0]).removeClass("disabled");
                pagerInput.value = parseInt(pagerInput.value) - 1;
                if (pagerInput.id.indexOf("Categ") != -1) {
                    this.model.categoricalCurrentPage = pagerInput.value;
                    if (this.targetControlName == "PivotGrid" || this.targetControlName == "OlapGrid") this.targetControl.refreshPagedOlapGrid("categorical", pagerInput.value);
                    if (pagerInput.value == "1") {
                        $(e.target).addClass("disabled");
                        $($($(e.target).parents('td')[0]).find(".moveFirst")[0]).addClass("disabled");
                    }
                }
                else if (pagerInput.id.indexOf("Series") != -1) {
                    this.model.seriesCurrentPage = pagerInput.value;
                    if (this.targetControlName == "PivotGrid" || this.targetControlName == "OlapGrid") this.targetControl.refreshPagedOlapGrid("series", pagerInput.value);
                    if (pagerInput.value == "1") {
                        $(e.target).addClass("disabled");
                        $($($(e.target).parents('td')[0]).find(".moveFirst")[0]).addClass("disabled");
                    }
                }
            }
        },

        /**
        * This function is used to move the target control to last page.
        *
        * @private  
        */
        _moveLastPage: function (e) {
            if (!$(e.target).hasClass("disabled")) {
                var pagerInput = $($(e.target).parents('td')[0]).find(".pagerTextBox")[0];
                $($($(e.target).parents('td')[0]).find(".movePrevious")[0]).removeClass("disabled");
                $($($(e.target).parents('td')[0]).find(".moveFirst")[0]).removeClass("disabled");
                $(e.target).addClass("disabled");
                $($($(e.target).parents('td')[0]).find(".moveNext")[0]).addClass("disabled");
                if (pagerInput.id.indexOf("Categ") != -1) {
                    this.model.categoricalCurrentPage = pagerInput.value = this.model.categoricalPageCount;
                    if (this.targetControlName == "PivotGrid" || this.targetControlName == "OlapGrid") this.targetControl.refreshPagedOlapGrid("categorical", pagerInput.value);
                }
                else if (pagerInput.id.indexOf("Series") != -1) {
                    this.model.seriesCurrentPage = pagerInput.value = this.model.seriesPageCount;
                    if (this.targetControlName == "PivotGrid" || this.targetControlName == "OlapGrid") this.targetControl.refreshPagedOlapGrid("series", pagerInput.value);
                }
            }
        },

        /**
        * This function is used to move the target control to first page.
        *
        * @private  
        */
        _moveFirstPage: function (e) {
            if (!$(e.target).hasClass("disabled")) {
                var pagerInput = $($(e.target).parents('td')[0]).find(".pagerTextBox")[0];
                $($($(e.target).parents('td')[0]).find(".moveNext")[0]).removeClass("disabled");
                $($($(e.target).parents('td')[0]).find(".moveLast")[0]).removeClass("disabled");
                $(e.target).addClass("disabled");
                $($($(e.target).parents('td')[0]).find(".movePrevious")[0]).addClass("disabled");
                if (pagerInput.id.indexOf("Categ") != -1) {
                    this.model.categoricalCurrentPage = pagerInput.value = 1;
                    if (this.targetControlName == "PivotGrid" || this.targetControlName == "OlapGrid") this.targetControl.refreshPagedOlapGrid("categorical", pagerInput.value);
                }
                else if (pagerInput.id.indexOf("Series") != -1) {
                    this.model.seriesCurrentPage = pagerInput.value = 1;
                    if (this.targetControlName == "PivotGrid" || this.targetControlName == "OlapGrid") this.targetControl.refreshPagedOlapGrid("series", pagerInput.value);
                }
            }
        },

        /**
        * This function is used to move the target control to entered page.
        *
        * @private  
        */
        _pagerTextBoxClick: function (e) {
            if (e.which == 13) {
                e.preventDefault();
                var pagerTd = $(e.target).parents('td')[0];
                var pageCount = pagerTd.className.indexOf('categ') != -1 ? this.model.categoricalPageCount : this.model.seriesPageCount;
                if (parseInt(e.target.value) > pageCount || parseInt(e.target.value) < 1 || e.target.value == "") {
                    this._restorePageNo(e);
                    alert("Enter valid page number");
                }
                else {
                    if (pagerTd.className.indexOf('categ') != -1 && this.model.categoricalCurrentPage != parseInt(e.target.value)) {
                        this.model.categoricalCurrentPage = e.target.value;
                        if (this.targetControlName == "PivotGrid" || this.targetControlName == "OlapGrid") this.targetControl.refreshPagedOlapGrid(pagerTd.className, e.target.value);
                    }
                    else if (pagerTd.className.indexOf('series') != -1 && this.model.seriesCurrentPage != parseInt(e.target.value)) {
                        this.model.seriesCurrentPage = e.target.value;
                        if (this.targetControlName == "PivotGrid" || this.targetControlName == "OlapGrid") this.targetControl.refreshPagedOlapGrid(pagerTd.className, e.target.value);
                    }
                    this._setNavigators(e.target.value, pageCount, pagerTd);
                    e.target.blur();
                }
            }
            else {
                if ($.inArray(e.keyCode, [46, 8, 9, 27, 13, 190]) !== -1 || (e.keyCode == 65 && e.ctrlKey === true) || (e.keyCode >= 35 && e.keyCode <= 39)) return;
                if ((e.shiftKey || (e.keyCode < 48 || e.keyCode > 57)) && (e.keyCode < 96 || e.keyCode > 105)) e.preventDefault();
            }
        },

        /**
        * This function is used to restore the current page numbers in Olap Pager.
        *
        * @private  
        */
        _restorePageNo: function (e) { e.target.value = e.target.id.indexOf("Categ") != -1 ? this.model.categoricalCurrentPage : this.model.seriesCurrentPage; },

        /**
        * This function is used to enable/disable the navigators according to the current page on initial loading.
        *
        * @private  
        */
        _setNavigators: function (value, pageCount, pagerTd) {
            if (value == pageCount) {
                $($(pagerTd).find(".moveNext")[0]).addClass("disabled");
                $($(pagerTd).find(".moveLast")[0]).addClass("disabled");
            }
            if (value == "1") {
                $($(pagerTd).find(".movePrevious")[0]).addClass("disabled");
                $($(pagerTd).find(".moveFirst")[0]).addClass("disabled");
            }
            if (value > 1) {
                $($(pagerTd).find(".movePrevious")[0]).removeClass("disabled");
                $($(pagerTd).find(".moveFirst")[0]).removeClass("disabled");
            }
            if (value < pageCount) {
                $($(pagerTd).find(".moveNext")[0]).removeClass("disabled");
                $($(pagerTd).find(".moveLast")[0]).removeClass("disabled");
            }
        }
    })

    //OLAP Pager Localization
    ej.olap.OlapPager.locale = {};
    //Default english localized values
    ej.olap.OlapPager.locale["en-US"] = {
        SeriesPage: "Series Page",
        CategoricalPage: "Categorical Page"
    };

    /**
    * Enum for PagerModes. Allows the user to select a pager mode to enable the pager for both or any one of the axes. 
    *
    * @enum {String}
    * @global
    */
    ej.olap.OlapPager.Mode = {
        Both: "both",
        Categorical: "categorical",
        Series: "series"
    };

})(jQuery, Syncfusion); ;

        ;